#include "dofwidget.h"
#include "ui_dofwidget.h"
#include "crystaldata.h"
#include "nofocusdelegate.h"
#include "checkboxdelegate.h"
#include "mylistdialog.h"
#include "docdialog.h"
#include "doublespinboxdelegate.h"
#include "floatdelegate.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "mainwindow.h"

#include <QDebug>

extern "C" int fragment_number(int id);
extern "C" int rotation_number(int id);
extern "C" void get_fragment_atom_number(int id, int natfrag[]);
extern "C" int crystal_atom_number(int id);
extern "C" void get_crystal_atoms(int id, crydata::c_atom_type atm[]);
extern "C" void put_crystal_atoms(int id, crydata::c_atom_type atm[], int coord);
extern "C" void get_fragment_atoms(int id, int idfrag, int atomvet[], int rcod[6], float tlow[3], float thigh[3], float rbound[2], int *rat);
extern "C" void put_fragment_atoms(int id, int idfrag, int rcod[6], float tlow[3], float thigh[3], float rbound[2], int rat);
extern "C" void get_rotation_atoms(int id, int idrot, int atomvet[4], int *rcod, float bounds[2], float *torsion);
extern "C" void put_rotation_atoms(int id, int idrot, int rcod, float bounds[2], float torsion);
extern "C" void get_info_po(int idph, int hkl[3], float *gpar, int *code);
extern "C" void put_info_po(int idph, int hkl[3], float gpar, int code);

DOFWidget::DOFWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::DOFWidget),
    phaseId(-1),
    selectedFragment(0),
    hklPO{0,0,1},
    codePO(0),
    parPO(1.0),
    parTableChangeEnabled(true),
    dofTableChangeEnabled(true)
{
    ui->setupUi(this);
    createFragmentsModel();
    createParametersModel();
    setFragmentTableView();
    setParametersTableView();
    createIntDOFModel();
    setIntDOFTableView();

    connect(ui->poWidget, &PrefOrientationWidget::poCheckBoxClicked, this, &DOFWidget::parameterChecked);
}

DOFWidget::~DOFWidget()
{
    delete ui;
}

void DOFWidget::setWidget(int id)
{
    phaseId = id;    
    makeFragmentsAndRotations();

    updateFragmentModel();
    ui->fragmentTableView->horizontalHeader()->setSectionResizeMode(1,QHeaderView::Stretch);

    if(fragments.size() > 0) {
        updateParametersModel(0);
        ui->parametersTableView->horizontalHeader()->setSectionResizeMode(0,QHeaderView::Stretch);
    }

    updateIntDOFModel();
    ui->intDOFTableView->horizontalHeader()->setSectionResizeMode(0,QHeaderView::Stretch);    
    intDofCbHeader->setChecked(dofTableCol::DOFREF_COL, rowCheckBoxClickedCount(intDOFModel, dofTableCol::DOFREF_COL) == intDOFModel->rowCount());

    getPatternInfo();
    ui->poWidget->setWidget(hklPO, parPO, codePO);
}

void DOFWidget::makeFragmentsAndRotations()
{
    fragments.clear();
    rotations.clear();

    int nfrag = fragment_number(phaseId);
    if (nfrag == 0) return;

    //Atoms
    int nat = crystal_atom_number(phaseId);
    if (nat == 0) return;
    atoms.resize(nat);

    auto atm = new crydata::c_atom_type[nat];
    get_crystal_atoms(phaseId, atm);
    for (int i = 0; i < nat; i++) {        
        atoms[i] = {i, QString::fromUtf8(atm[i].lab),atm[i].biso,
                    atm[i].occ,atm[i].rcod[3],atm[i].doc};
    }

    //Make fragments
    auto natfrag = new int[nfrag];
    get_fragment_atom_number(phaseId, natfrag);

    fragments.resize(nfrag);
    for (int i = 0; i < nfrag; i++) {
        fragments[i].nat = natfrag[i];
        auto atfrag = new int[fragments[i].nat];
        get_fragment_atoms(phaseId, i+1, atfrag, fragments[i].rcod,
                           fragments[i].tlow, fragments[i].thigh,
                           fragments[i].rbound, &fragments[i].rat);
        fragments[i].atomIndex.resize(nat);
        for (int j = 0; j < fragments[i].nat; j++) {
            fragments[i].atomIndex[j] = atfrag[j];
        }
        delete [] atfrag;
    }

    delete [] atm;
    delete [] natfrag;

    //Make rotations
    int nrot = rotation_number(phaseId);
    if (nrot == 0) return;

    int atomIndex[4];
    rotations.resize(nrot);
    for (int i = 0; i < nrot; i++) {
        get_rotation_atoms(phaseId, i+1, atomIndex, &rotations[i].rcod, rotations[i].bound, &rotations[i].torsion);
        for (int j = 0; j < 4; j++) {                        
            rotations[i].atomIndex[j] = atomIndex[j]-1;
        }        
    }
}

void DOFWidget::getPatternInfo()
{
    get_info_po(phaseId, hklPO, &parPO, &codePO);
}

QString DOFWidget::getFragmentLabel(int index)
{
    QString label = atoms[fragments[index].atomIndex[0]].label;
    for (int i = 1; i < fragments[index].nat; i++) {
        label.append(" " + atoms[fragments[index].atomIndex[i]].label);
    }
    return label;
}

QString DOFWidget::getRotationLabel(int index)
{
    return QString("%1:%2-%3:%4").
            arg(atoms[rotations[index].atomIndex[0]].label,
            atoms[rotations[index].atomIndex[1]].label,
            atoms[rotations[index].atomIndex[2]].label,
            atoms[rotations[index].atomIndex[3]].label);
}

void DOFWidget::createFragmentsModel()
{
    fragmentsModel = new QStandardItemModel(this);
    fragmentsProxyModel = new QSortFilterProxyModel(this);
    fragmentsProxyModel->setSourceModel(fragmentsModel);
    fragmentsModel->setColumnCount(2);
    const QStringList titles = {"n.", "Atoms"};
    fragmentsModel->setHorizontalHeaderLabels(titles);
}

void DOFWidget::updateFragmentModel()
{    
    int nfrag = fragments.size();
    fragmentsModel->setRowCount(nfrag);
    for (int r = 0; r < nfrag; r++) {
        auto item = new QStandardItem();
        item->setData(r+1, Qt::DisplayRole);
        fragmentsModel->setItem(r,0,item);

        item = new QStandardItem(getFragmentLabel(r));
        fragmentsModel->setItem(r,1,item);
    }    
}

void DOFWidget::setFragmentTableView()
{
    ui->fragmentTableView->setModel(fragmentsProxyModel);
    ui->fragmentTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->fragmentTableView->setSortingEnabled(true);
    ui->fragmentTableView->sortByColumn(0,Qt::AscendingOrder);
    ui->fragmentTableView->verticalHeader()->hide();
    ui->fragmentTableView->setShowGrid(false);
    ui->fragmentTableView->setAlternatingRowColors(true);
    ui->fragmentTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->fragmentTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->fragmentTableView->horizontalHeader()->stretchLastSection();
    ui->fragmentTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->fragmentTableView->setItemDelegate(new NoFocusDelegate()); //to remove focus rectangle

    connect(ui->fragmentTableView->selectionModel(), &QItemSelectionModel::currentRowChanged, this,
            [this](const QModelIndex &current) {        
        QModelIndex index = fragmentsProxyModel->mapToSource(current);
        updateParametersModel(index.row());
    });
}

void DOFWidget::createParametersModel()
{
    parametersModel = new QStandardItemModel(this);
    parametersModel->setColumnCount(4);
    const QStringList titles = {"Parameter", "Refine", "Lower", "Upper"};
    parametersModel->setHorizontalHeaderLabels(titles);
    connect(parametersModel, &QStandardItemModel::itemChanged, this, &DOFWidget::fragmentChanged);
}

void DOFWidget::setParametersTableView()
{
    ui->parametersTableView->setModel(parametersModel);
    ui->parametersTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->parametersTableView->setSortingEnabled(false);
    ui->parametersTableView->verticalHeader()->hide();
    ui->parametersTableView->setShowGrid(false);
    ui->parametersTableView->setAlternatingRowColors(true);
    ui->parametersTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->parametersTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->parametersTableView->horizontalHeader()->stretchLastSection();
    ui->parametersTableView->setItemDelegate(new NoFocusDelegate()); //to remove focus rectangle

    auto checkDelegate = new CheckBoxDelegate(ui->parametersTableView);
    checkDelegate->setColumn(parTableCol::REF_COL);
    ui->parametersTableView->setItemDelegateForColumn(parTableCol::REF_COL,checkDelegate);
    connect(checkDelegate, &CheckBoxDelegate::rowChecked, this, &DOFWidget::extDofRowCheckBoxClicked);

    extDofCbHeader = new CheckBoxHeader(Qt::Horizontal, {parTableCol::REF_COL}, true, ui->parametersTableView);
    ui->parametersTableView->setHorizontalHeader(extDofCbHeader);
    extDofCbHeader->setChecked(parTableCol::REF_COL, false);
    connect(extDofCbHeader, &CheckBoxHeader::checkBoxClicked, this, &DOFWidget::extDofHeaderCheckBoxClicked);

    FloatDelegate *floatDelegate = new FloatDelegate(ui->parametersTableView);
    floatDelegate->setDecimals(3);
    ui->parametersTableView->setItemDelegateForColumn(parTableCol::MIN_COL, floatDelegate);
    ui->parametersTableView->setItemDelegateForColumn(parTableCol::MAX_COL, floatDelegate);
}

void DOFWidget::updateParametersModel(int index)
{
    parTableChangeEnabled = false;
    selectedFragment = index;
    ui->parametersGroupBox->setTitle(QString("Parameters for Fragment n. %1").arg(index+1));

    if (fragments[index].nat > 1) {
        parametersModel->setRowCount(4);
    } else {
        parametersModel->setRowCount(3);
    }

    const QStringList par = {"X Translation", "Y Translation", "Z Translation", "Rotation"};
    for (int r = 0; r < parametersModel->rowCount(); r++) {
        auto item = new QStandardItem(par.at(r));
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        parametersModel->setItem(r,parTableCol::ATOM_COL,item);

        item = new QStandardItem();
        item->setData(fragments[index].rcod[r],Qt::DisplayRole);
        parametersModel->setItem(r,parTableCol::REF_COL,item);

        item = new QStandardItem();
        if (r == 3)
            item->setData(fragments[index].rbound[0], Qt::DisplayRole);
        else
            item->setData(fragments[index].tlow[r], Qt::DisplayRole);
        parametersModel->setItem(r,parTableCol::MIN_COL,item);

        item = new QStandardItem();
        if (r == 3)
            item->setData(fragments[index].rbound[1], Qt::DisplayRole);
        else
            item->setData(fragments[index].thigh[r], Qt::DisplayRole);
        parametersModel->setItem(r,parTableCol::MAX_COL,item);
    }

    if (fragments[index].nat > 1) {
        ui->centreGroupBox->setEnabled(true);
        int center = fragments[index].rat;
        if (center >= 0)
            ui->comRadioButton->setChecked(true);
        else {
            ui->comRadioButton->setChecked(true);
            center = 0;
        }
        ui->atomsPushButton->setText(atoms[center].label);
    } else {
        ui->centreGroupBox->setEnabled(false);
    }

    extDofCbHeader->setChecked(parTableCol::REF_COL, rowCheckBoxClickedCount(parametersModel, parTableCol::REF_COL) == parametersModel->rowCount());
    parTableChangeEnabled = true;
}

void DOFWidget::createIntDOFModel()
{
    intDOFModel = new QStandardItemModel(this);
    intDOFProxyModel = new QSortFilterProxyModel(this);
    intDOFProxyModel->setSourceModel(intDOFModel);
    intDOFModel->setColumnCount(5);
    const QStringList titles = {"Torsion", "Refine", "Value", "Lower", "Upper"};
    intDOFModel->setHorizontalHeaderLabels(titles);
    connect(intDOFModel, &QStandardItemModel::itemChanged, this, &DOFWidget::intDofChanged);
}

void DOFWidget::setIntDOFTableView()
{
    ui->intDOFTableView->setModel(intDOFProxyModel);
    ui->intDOFTableView->verticalHeader()->setDefaultSectionSize(22);    
    ui->intDOFTableView->verticalHeader()->hide();
    ui->intDOFTableView->setShowGrid(false);
    ui->intDOFTableView->setAlternatingRowColors(true);
    ui->intDOFTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->intDOFTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->intDOFTableView->horizontalHeader()->stretchLastSection();    
    ui->intDOFTableView->setItemDelegate(new NoFocusDelegate()); //to remove focus rectangle

    auto checkDelegate = new CheckBoxDelegate(ui->intDOFTableView);
    checkDelegate->setColumn(dofTableCol::DOFREF_COL);
    ui->intDOFTableView->setItemDelegateForColumn(dofTableCol::DOFREF_COL,checkDelegate);
    connect(checkDelegate, &CheckBoxDelegate::rowChecked, this, &DOFWidget::intDofRowCheckBoxClicked);

    intDofCbHeader = new CheckBoxHeader(Qt::Horizontal, {dofTableCol::DOFREF_COL}, true, ui->intDOFTableView);
    ui->intDOFTableView->setHorizontalHeader(intDofCbHeader);
    intDofCbHeader->setChecked(dofTableCol::DOFREF_COL, false);
    connect(intDofCbHeader, &CheckBoxHeader::checkBoxClicked, this, &DOFWidget::intDofHeaderCheckBoxClicked);

    auto spinBoxDelegate = new DoubleSpinBoxDelegate(this);
    spinBoxDelegate->setDecimals(2);
    spinBoxDelegate->setRange(-180.00,180.00);
    ui->intDOFTableView->setItemDelegateForColumn(dofTableCol::VALUE_COL,spinBoxDelegate);
    ui->intDOFTableView->setItemDelegateForColumn(dofTableCol::DOFMIN_COL,spinBoxDelegate);
    ui->intDOFTableView->setItemDelegateForColumn(dofTableCol::DOFMAX_COL,spinBoxDelegate);

    ui->intDOFTableView->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder); //avoid that setSortingEnabled force a sort
    ui->intDOFTableView->setSortingEnabled(true);

    connect(ui->intDOFTableView->selectionModel(), &QItemSelectionModel::currentRowChanged, this, &DOFWidget::intDofSelected);
}

void DOFWidget::updateIntDOFModel()
{
    dofTableChangeEnabled = false;
    intDOFModel->setRowCount(rotations.size());

    for (int r = 0; r < intDOFModel->rowCount(); r++) {
        auto item = new QStandardItem(getRotationLabel(r));
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        intDOFModel->setItem(r,dofTableCol::TORSION_COL,item);

        item = new QStandardItem();
        item->setData(rotations[r].rcod,Qt::EditRole);
        intDOFModel->setItem(r,dofTableCol::DOFREF_COL,item);

        item = new QStandardItem();
        item->setData(rotations[r].torsion,Qt::EditRole);
        intDOFModel->setItem(r,dofTableCol::VALUE_COL,item);

        item = new QStandardItem();
        item->setData(rotations[r].bound[0],Qt::EditRole);
        intDOFModel->setItem(r,dofTableCol::DOFMIN_COL,item);

        item = new QStandardItem();
        item->setData(rotations[r].bound[1],Qt::EditRole);
        intDOFModel->setItem(r,dofTableCol::DOFMAX_COL,item);
    }
    dofTableChangeEnabled = true;
}

int DOFWidget::rowCheckBoxClickedCount(const QStandardItemModel *model, int col) const
{
    int count = 0;
    for (int r = 0; r < model->rowCount(); r++) {
        QModelIndex index = model->index(r,col,QModelIndex());        
        if (index.data().toBool()) count++;
    }
    return count;
}

void DOFWidget::updateComputingVariables()
{
    for (int i = 0; i < fragments.size(); i++) {
        put_fragment_atoms(phaseId, i+1, fragments[i].rcod, fragments[i].tlow, fragments[i].thigh, fragments[i].rbound, fragments[i].rat);
    }
    for (int i = 0; i < rotations.size(); i++) {
        put_rotation_atoms(phaseId, i+1, rotations[i].rcod, rotations[i].bound, rotations[i].torsion);
    }
    ui->poWidget->getPO(hklPO, parPO, codePO);
    put_info_po(phaseId, hklPO, parPO, codePO);
}

void DOFWidget::on_atomsPushButton_clicked()
{
    QStringList atomList;
    foreach(auto at, atoms) {
        atomList.append(at.label);
    }
    int selection = fragments[selectedFragment].rat < 0? 0 : fragments[selectedFragment].rat;
    auto atomListDialog = new MyListDialog(atomList,"Set Centre of Rotation","Atoms", selection, this);
    int ret = atomListDialog->exec();
    if (ret == QDialog::Accepted) {
        int index = atomListDialog->currentIndex();
        fragments[selectedFragment].rat = index;
        ui->atomsPushButton->setText(atoms.at(index).label);
    }    
}

void DOFWidget::on_docPushButton_clicked()
{
    auto docDialog = new DocDialog(atoms, this);

    int ret = docDialog->exec();
    if (ret == QDialog::Accepted) {
        docDialog->updateAtoms(atoms);

        auto atm = new crydata::c_atom_type[atoms.size()];
        for (int i = 0; i < atoms.size(); i++) {
            atm[i].biso = atoms.at(i).biso;
            atm[i].occ = atoms.at(i).occ;
            atm[i].rcod[3] = atoms.at(i).bisoCode;
            atm[i].doc = atoms.at(i).doc;
        }
        put_crystal_atoms(phaseId, atm, 0);
        delete [] atm;

        emit parameterChecked(1);
    }
}

void DOFWidget::intDofSelected(const QModelIndex &index)
{
    int row = intDOFProxyModel->mapToSource(index).row();

    MainWindow *mw = getMainWindow();
    if (!mw) return;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = mw->getJav();
#else
    JavWindow *jav = mw->getJav();
#endif
    if (jav->isVisible()) {
        QVector<int> sel (4);
        for(int i=0; i<4; i++)
            sel[i] = rotations.at(row).atomIndex[i];
        jav->selectAtomsAndLegs(sel);
#if !defined(JAV_WIDGET)
        //Raise only if JAV doesn't overlap with SA dialog
        if (!this->window()->geometry().intersects(jav->geometry()))
            jav->raise();
#endif
    }
}

void DOFWidget::extDofHeaderCheckBoxClicked(bool state)
{
    for (int row = 0; row < parametersModel->rowCount(); row++) {
        QModelIndex index = parametersModel->index(row, parTableCol::REF_COL, QModelIndex());
        parametersModel->setData(index, QVariant(state?1:0));
        fragments[selectedFragment].rcod[row] = state?1:0;
    }
    if (parametersModel->rowCount() == 4) {
        fragments[selectedFragment].rcod[4] = fragments[selectedFragment].rcod[3];
        fragments[selectedFragment].rcod[5] = fragments[selectedFragment].rcod[3];
    }

    emit parameterChecked(state ? parametersModel->rowCount()+2 : -parametersModel->rowCount()-2);
}

void DOFWidget::extDofRowCheckBoxClicked(const QModelIndex &index, bool state)
{
    Q_UNUSED(index);
    //If state is false uncheck the checkBoxHeader
    if (!state && extDofCbHeader->isChecked(parTableCol::REF_COL)) {
        extDofCbHeader->setChecked(parTableCol::REF_COL, false);
        return;
    }

    //If state is true check the checkBoxheader if all checkboxes have been clicked
    if (state && !extDofCbHeader->isChecked(parTableCol::REF_COL)) {
        if (rowCheckBoxClickedCount(parametersModel, parTableCol::REF_COL) == parametersModel->rowCount())
            extDofCbHeader->setChecked(parTableCol::REF_COL, true);
    }
}

void DOFWidget::intDofHeaderCheckBoxClicked(bool state)
{
    for (int row = 0; row < intDOFModel->rowCount(); row++) {
        QModelIndex index = intDOFModel->index(row, dofTableCol::DOFREF_COL, QModelIndex());
        intDOFModel->setData(index, QVariant(state?1:0));
        rotations[row].rcod = state?1:0;
    }

    emit parameterChecked(state ? intDOFModel->rowCount() : -intDOFModel->rowCount());
}

void DOFWidget::intDofRowCheckBoxClicked(const QModelIndex &index, bool state)
{    
    //If state is false uncheck the checkBoxHeader
    if (!state && intDofCbHeader->isChecked(dofTableCol::DOFREF_COL)) {
        intDofCbHeader->setChecked(dofTableCol::DOFREF_COL, false);
        return;
    }

    //If state is true check the checkBoxheader if all checkboxes have been clicked
    if (state && !intDofCbHeader->isChecked(dofTableCol::DOFREF_COL)) {
        if (rowCheckBoxClickedCount(intDOFModel, dofTableCol::DOFREF_COL) == intDOFModel->rowCount())
            intDofCbHeader->setChecked(dofTableCol::DOFREF_COL, true);
    }

    QModelIndex sourceIndex = intDOFProxyModel->mapToSource(index);
    rotations[sourceIndex.row()].rcod = state;

    emit parameterChecked(state?1:-1);
}

void DOFWidget::fragmentChanged(QStandardItem *item)
{
    if (parTableChangeEnabled && (item->column() == parTableCol::MIN_COL || item->column() == parTableCol::MAX_COL)) {
        qInfo() << "Frag changed: " << item->row() << item->column() << item->data(Qt::DisplayRole);
        if (item->row()  < 3) {
            if (item->column() == parTableCol::MIN_COL) {
                fragments[selectedFragment].tlow[item->row()] = item->data(Qt::DisplayRole).toFloat();
            } else if (item->column() == parTableCol::MAX_COL) {
                fragments[selectedFragment].thigh[item->row()] = item->data(Qt::DisplayRole).toFloat();
            }
        } else {
            if (item->column() == parTableCol::MIN_COL) {
                fragments[selectedFragment].rbound[0] = item->data(Qt::DisplayRole).toFloat();
            } else if (item->column() == parTableCol::MAX_COL) {
                fragments[selectedFragment].rbound[1] = item->data(Qt::DisplayRole).toFloat();
            }
        }
    }
}

void DOFWidget::intDofChanged(QStandardItem *item)
{
    if (item->column() == dofTableCol::TORSION_COL || item->column() == dofTableCol::DOFREF_COL) return;

    if (dofTableChangeEnabled) {
        if (item->column() == dofTableCol::VALUE_COL)
            rotations[item->row()].torsion = item->data(Qt::DisplayRole).toFloat();
        else if (item->column() == dofTableCol::DOFMIN_COL)
            rotations[item->row()].bound[0] = item->data(Qt::DisplayRole).toFloat();
        else if (item->column() == dofTableCol::DOFMAX_COL)
            rotations[item->row()].bound[1] = item->data(Qt::DisplayRole).toFloat();
    }
}

int DOFWidget::getNumberParam() const
{
    int npar = rowCheckBoxClickedCount(intDOFModel, dofTableCol::DOFREF_COL);
    foreach(auto frag, fragments) {
        for (int i = 0; i < 6; i++) if (frag.rcod[i]) npar++;
    }
    npar += ui->poWidget->isChecked() ? 1 : 0;
    foreach(auto at, atoms) {
        npar += at.bisoCode;
    }
    return npar;
}

void DOFWidget::on_comRadioButton_clicked()
{
    ui->atomsPushButton->setEnabled(false);
    fragments[selectedFragment].rat = -(fragments[selectedFragment].rat+1);
}

void DOFWidget::on_atomRadioButton_clicked()
{
    ui->atomsPushButton->setEnabled(true);
    fragments[selectedFragment].rat = -fragments[selectedFragment].rat-1;
}
