#include <QStringList>
#include <QGraphicsTextItem>
#include <QGraphicsSceneMouseEvent>
#include <QScrollBar>
#include "infodialog.h"
#include "ui_infodialog.h"
#include "properties.h"
#include "libcomune.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "qt_utils.h"
#include "texteditor.h"
#include "sequencedialog.h"
#include "aligndelegate.h"

extern "C" void getangle_(int *whichModel);
extern "C" void gettorsion_(int *whichModel);
extern "C" void getstructureinfo_(structure_info_type *StructInfo, int *ifCalc,
                 int *wm);

float getRFvalue(int wm)
{
  structure_info_type StructInfo;
  int ifCalc = 1;
  getstructureinfo_(&StructInfo, &ifCalc, &wm);
  return StructInfo.r_factor;
}

InfoDialog::InfoDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::InfoDialog),
    h0(-1)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);
    ui->exportButton->hide();
    ui->atomsView->verticalHeader()->setDefaultSectionSize(22);
    ui->bondsView->verticalHeader()->setDefaultSectionSize(22);
    ui->anglesView->verticalHeader()->setDefaultSectionSize(22);
    ui->torsionsView->verticalHeader()->setDefaultSectionSize(22);
    ui->contactsView->verticalHeader()->setDefaultSectionSize(22);
#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0))
    connect(ui->ADPcombo, QOverload<int>::of(&QComboBox::currentIndexChanged),
       [=](int index){ updateInfoAtoms(index); });
#else
    connect(ui->ADPcombo, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
      [=](int index){ updateInfoAtoms(index); });
#endif
}

InfoDialog::~InfoDialog()
{
    delete ui;
}

void InfoDialog::showEvent(QShowEvent *event)
{
    FillPages();
    QDialog::showEvent(event);
}

int InfoDialog::CurrentPage()
{
    return ui->tabWidget->currentIndex();
}

void InfoDialog::on_button_text_clicked()
{
    TextEditor *m_text = new TextEditor(this);
    m_text->LoadFile(md->fileName);
    m_text->show();
}

void InfoDialog::on_button_sequence_clicked()
{
    SequenceDialog *sd = new SequenceDialog(this, md);
    QString str = "Sequence for ";
    str.append(md->modname.c_str());
    sd->setWindowTitle(str);
    sd->show();
    sd->FillSequence(false);
}

void InfoDialog::on_button_secondary_clicked()
{
    QString os;
    PrintSecondary(md, os);
    TextEditor *m_text = new TextEditor(this);
    m_text->LoadFromString(os);
    QString str = "Secondary structure for ";
    str.append(md->modname.c_str());
    m_text->setWindowTitle(str);
    m_text->show();
}

void InfoDialog::on_exportButton_clicked()
{

}

void InfoDialog::on_closeButton_clicked()
{
    this->hide();
}

void InfoDialog::reject()
{
    QDialog::reject();
}

Model *InfoDialog::GetModel(int k)
{
    return jav->GetModel(k+1);
}

// Info ->Structure

static void AddInfoChain(QString &os, CHAIN ch)
{
   int nother=0, namino=0, nnucle=0;
   for(int rr=0; rr<ch.nResidues; rr++)
   {
       int which = ch.Residues[rr].code;
       if(which == 0)
          nother++;
       else if(which < N_AMINO)
          namino++;
       else
          nnucle++;
   }
   if(namino)
   {
      os.append(ch.id.c_str());
      os.append(" ");
      os.append("(");
      os.append(QString::number(namino));
      os.append(" residues)");
   }
   else
      os.append(ch.id.c_str());
}

QString &InfoDialog::InfoStructure(bool im)
{
    static QString os;
    QString sm("");
    QString em("");
    QStringList numbStrings = {"Number of Coils      ", "Number of Helices    ",
                               "Number of Sheets     ", "Number of Turns      "};
    os.clear();
    os.append("<pre>");
    if(im)
    {
       sm = QString("<span> <font color='#0000FF'>");
       em = QString("</span>");
    }

    float volume = (float) CellVolume(Properties::Cell);
    os.append("Name                 ");
    os.append(sm);
    os.append(QString(md->modname.c_str()));
    os.append(em);
    os.append("<br/>");
    os.append("Cell Lengths          a     ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[0], 'f', 3));
    os.append(em);
    os.append("  b    ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[1], 'f', 3));
    os.append(em);
    os.append("  c     ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[2], 'f', 3));
    os.append(em);
    os.append("<br/>");
    os.append("Cell Angles           alpha ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[3], 'f', 3));
    os.append(em);
    os.append("  beta ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[4], 'f', 3));
    os.append(em);
    os.append("  gamma ");
    os.append(sm);
    os.append(QString::number(Properties::Cell[5], 'f', 3));
    os.append(em);
    os.append("<br/>");
    QString spg(Properties::sy.spg);
    if(!spg.isEmpty())
    {
       os.append("Space Group          ");
       os.append(sm);
       os.append(spg);
       if(Properties::sy.icent == 1)
          os.append(" (centrosymmetric) ");
       else
          os.append(" (non-centrosymmetric) ");
       os.append(em);
       os.append("<br/>");
    }
    else
    {
       os.append("Space Group symbol   ");
       os.append(sm);
       os.append("n.a. ");
       os.append(em);
       os.append("<br/>");
    }
    os.append("Symmetry operations<br/>");
    for(int i=1; i<=Properties::sy.nsymop; i++)
    {
        os.append("                     ");
        os.append(sm);
	os.append(QString::number(i));
	os.append(") ");
	os.append(Properties::getStringSymm(&i));
        os.append(em);
        os.append("<br/>");
    }
    os.append("Cell Volume          ");
    os.append(sm);
    os.append(QString::number(volume, 'f', 3));
    os.append(em);
    os.append("<br/>");
    os.append("Cell Content         ");
    for(int i=0; i<md->numSpec; i++)
    {
        os.append(sm);
	os.append(md->LabSpec[i]);
	os.append(QString::number(md->Content[i]));
        os.append(em);
        os.append("<br/>");
        if(i != (md->numSpec -1))
           os.append("                     ") ;
    }
    os.append("Volume per atom      ");
    os.append(sm);
    os.append(QString::number(md->VolumePerAtom(volume, Properties::sy.nsymop), 'f', 3));
    os.append(em);
    os.append("<br/>");
    if(abs(md->RF) > 1e-8)
    {
       os.append("R-Factor (All refl.) ");
       os.append(sm);
       os.append(QString::number(md->RF, 'f', 3));
       os.append(" %");
       os.append(em);
       os.append("<br/>");
    }
    os.append("Atoms in model       ");
    os.append(sm);
    os.append(QString::number(md->nats));
    os.append(em);
    if(md->ifSymmetry)
      os.append(" (symmetry equivalent included)");
    os.append("<br/>");
    
    if(md->nResAmino > 0)
    {
       os.append("Number of chains     ");
       os.append(sm);
       os.append(QString::number(md->nChains));
       os.append(em);
       os.append("<br/>");
       os.append("Chains               ");
       os.append(sm);
       for(int kk=0; kk<md->nChains; kk++)
       {
           if(kk)
              os.append(", ");
           AddInfoChain(os, md->Chains[kk]);
       }
       os.append(em);
       os.append("<br/>");
       int kfin = (int)(sizeof(md->numbStructs)/sizeof(md->numbStructs[0]));
       for(int ks=0; ks<kfin; ks++)
       {
           if(md->numbStructs[ks] <= 0)
	      continue;
           os.append(numbStrings[ks]);
	   os.append(sm);
	   os.append(QString::number(md->numbStructs[ks]));
	   os.append(em);
           os.append("<br/>");
       }
    }
    
    os.append("</pre>");
    return os;
}

void InfoDialog::FillInfoStructure()
{
     QString *os = &InfoStructure();
     //ui->text_Structure->setTextFormat(Qt::RichText);
     ui->text_Structure->setHtml(*os);
}

// Info ->Atoms

void InfoDialog::updateInfoAtoms(int index)
{
    int nrows = modelProxyAtoms->rowCount();
    modelProxyAtoms->removeRows(0, nrows);
    if(index < 0)
        index = ui->ADPcombo->currentIndex();
    populateAtomsModel(false, (index <= 0));
    setStdProperties(ui->atomsView);
}

void InfoDialog::atomsSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     if(pgSel)
        return;
     for(int i=0; i<md->nats; i++)
         md->atm[i].selected = false;
     md->atoms_sel.clear();
	
     QModelIndexList selection = ui->atomsView->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
        if(index.column() == 0)  {
            int row = modelProxyAtoms->data(index, Qt::DisplayRole).toInt() - 1;
            md->atm[row].selected = true;
            md->atoms_sel.push_back(row+1);
        }
     }
     jav->Draw();
     jav->update_items_modify(true, false);
}

void InfoDialog::FillInfoAtoms()
{
    QStringList headers = { "Number", "Label", "Type", "Intensity",
                            "Xfrac", "Yfrac","Zfrac",
                            "  B  "," s.o.f.", "Symm.Op", "z"};
    const char *tipi = "iasffffffau";
    int n = headers.count();

    int lastcol = headers.size() - 1;
    if(modelAtoms)
    {
       modelAtoms->clear();
       delete modelAtoms;
    }
    if(modelProxyAtoms)
    {
       modelProxyAtoms->invalidate();
       delete modelProxyAtoms;
    }
    modelAtoms = CreaLista(headers);
    modelProxyAtoms = new MyProxyModel(this);
    modelProxyAtoms->setColumnForSort(10);
    modelProxyAtoms->setColumnTypes(n, tipi);
    modelProxyAtoms->setSourceModel(modelAtoms);
    bool ifB = (ui->ADPcombo->currentIndex() <= 0);
    populateAtomsModel(false, ifB);
    AlignFloatDelegate *delegate0 = new AlignFloatDelegate(ui->atomsView);
    delegate0->setDecimals(0);
    ui->atomsView->setItemDelegateForColumn(0, delegate0);
    ui->atomsView->setItemDelegateForColumn(3, delegate0);
    AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->atomsView);
    delegate->setDecimals(5);
    ui->atomsView->setItemDelegateForColumn(4, delegate);
    ui->atomsView->setItemDelegateForColumn(5, delegate);
    ui->atomsView->setItemDelegateForColumn(6, delegate);
    ui->atomsView->setItemDelegateForColumn(7, delegate);
    ui->atomsView->setItemDelegateForColumn(8, delegate);
    ui->atomsView->show();
    ui->chainsView->hide();
//    ui->atomsView->setModel(modelAtoms);
//    modelProxyAtoms->setSourceModel(modelAtoms);
    ui->atomsView->setModel(modelProxyAtoms);
    setStdProperties(ui->atomsView);
    ui->atomsView->setColumnHidden(lastcol, true);
    QItemSelectionModel *selectionModel = ui->atomsView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::atomsSelectionChanged);
    UpdateAtomsSelection(selectionModel);
}

void InfoDialog::UpdateAtomsSelection(QItemSelectionModel *selectionModel)
{
     if(md->IsProtein())
        return;
      pgSel = true;
      std::vector<int> iord (md->nats, 0);
      numRow(modelProxyAtoms, 0, iord);
      selectionModel->clearSelection();
      int nsel = (int) md->atoms_sel.size();
      ui->atomsView->setSelectionMode(QAbstractItemView::MultiSelection);

      QItemSelection selectedItems = selectionModel->selection();
      for(int i=0; i<nsel; i++)
      {
         int kk = (int) md->atoms_sel[i] - 1;
         ui->atomsView->selectRow(iord.at(kk));
         selectedItems.merge(selectionModel->selection(), QItemSelectionModel::Select);
      }
      selectionModel->select(selectedItems, QItemSelectionModel::Select);
      pgSel = false;
}

void InfoDialog::chainsSelectionChanged(const QItemSelection &, const QItemSelection &)
{
    if(pgSel)
       return;

    QModelIndex index = ui->chainsView->selectionModel()->currentIndex();
    QString selectedText = index.data(Qt::DisplayRole).toString();
    int hierarchyLevel = 1;
    QModelIndex seekRoot = index;
    while (seekRoot.parent() != QModelIndex()) {
        seekRoot = seekRoot.parent();
	selectedChain = seekRoot.row();
        hierarchyLevel++;
    }
    if(hierarchyLevel > 1)
       selectedResidue = index.row() ;	
     FillInfoAtoms_PDB();
}

void InfoDialog::populateAtomsModel(bool ifEdit, bool ifB)
{
    QList<QStandardItem *>  *atom_info;
    int nats = 0;
    if(md->IsProtein())
       atom_info = md->FillListAtomsPDB(ifB, selectedChain,
		       selectedResidue, &nats);
    else
       atom_info = md->FillListAtoms(ifEdit, ifB, &nats);
    for (int i=0; i<nats; i++)
         modelAtoms->appendRow(atom_info[i]);
}

void InfoDialog::populateChainsModel()
{
    QList<QStandardItem *>  chains_info = md->FillListChains();
    QList<QStandardItem *>  *resids_info = md->FillListResidues();

    QStandardItem *item = modelChains->invisibleRootItem();
    for (int i=0; i<md->nChains; i++)
    {
         item->appendRow(chains_info.at(i));
	 for(int k=0; k<md->Chains[i].nResidues; k++)
             chains_info.at(i)->appendRow(resids_info[i].at(k));
    }
}

void InfoDialog::FillInfoChains()
{
    QStringList headers1 = { "Chains"};
    if(modelChains)
    {
       modelChains->clear();
       delete modelChains;
    }
    modelChains = new QStandardItemModel(this);
    populateChainsModel();
    modelChains->setHorizontalHeaderLabels(headers1);
    ui->chainsView->setModel(modelChains);
    ui->chainsView->show();
    QItemSelectionModel *selectionModel1 = ui->chainsView->selectionModel();
    connect(selectionModel1, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::chainsSelectionChanged);
    QModelIndex index = modelChains->index(0, 0);
    ui->chainsView->expand(index);
    ui->chainsView->setCurrentIndex(modelChains->index(0,0, index));
    ui->chainsView->resizeColumnToContents(0);
}

void InfoDialog::FillInfoAtoms_PDB()
{
    QStringList headers = { "Number", "Label", 
                            "Xfrac", "Yfrac","Zfrac",
                            "occ, ","B"};
    if(modelAtoms)
    {
       modelAtoms->clear();
       delete modelAtoms;
    }
    modelAtoms = CreaLista(headers);
    bool ifB = (ui->ADPcombo->currentIndex() <= 0);
    populateAtomsModel(false, ifB);
    ui->atomsView->setModel(modelAtoms);
    ui->atomsView->setColumnHidden(0, true);
    setStdProperties(ui->atomsView, false);
    AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->atomsView);
    delegate->setDecimals(5);
    for(int i=2;i<7;i++)
        ui->atomsView->setItemDelegateForColumn(i, delegate);
    ui->atomsView->updateGeometry();
    ui->chainsView->updateGeometry();
    QItemSelectionModel *selectionModel = ui->atomsView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::atomsSelectionChanged);
    UpdateAtomsSelection(selectionModel);
}

// Info ->Bonds

void InfoDialog::bondsSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     if(pgSel)
        return;
     for(int i=0; i<md->nconn_prot; i++)
         md->bonds[i].selected = false;
     md->bonds_sel.clear();

     QModelIndexList selection = ui->bondsView->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
        if(index.column() == 0)  {
            //int row = index.row();
            int row = modelBonds->data(index, Qt::DisplayRole).toInt() - 1;
            md->bonds[row].selected = true;
            md->bonds_sel.push_back(row+1);
        }
     }
     jav->Draw();
     jav->update_items_modify(true, false);
}

void InfoDialog::UpdateBondsSelection(QItemSelectionModel *selectionModel)
{
     if(md->isPDB)
        return;
      pgSel = true;
      std::vector<int> iord (md->nconn_prot, 0);
      numRow(modelBonds, 0, iord);
      selectionModel->clearSelection();
      int nsel = (int) md->bonds_sel.size();
      for(int i=0; i<nsel; i++)
      {
         int kk = (int) md->bonds_sel[i] - 1;
         if(kk < md->nconn_prot)
            ui->bondsView->selectRow(iord.at(kk));
      }
      pgSel = false;
}

void InfoDialog::FillInfoBonds()
{
    QStringList headers = {"Number", "Type", "Atom 1", "Atom 2", "Distance",
                           "Symm.Op1", "Symm.Op2"};
    if(modelBonds)
    {
       modelBonds->clear();
       delete modelBonds;
    }
    modelBonds = CreaLista(headers);
    populateBondsModel(false);
    setStdProperties(ui->bondsView);
    ui->bondsView->setModel(modelBonds);
    AlignFloatDelegate *delegate0 = new AlignFloatDelegate(ui->bondsView);
    delegate0->setDecimals(0);
    ui->bondsView->setItemDelegateForColumn(0, delegate0);
    AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->bondsView);
    delegate->setDecimals(5);
    ui->bondsView->setItemDelegateForColumn(4, delegate);

    QItemSelectionModel *selectionModel = ui->bondsView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::bondsSelectionChanged);
    UpdateBondsSelection(selectionModel);
}

void InfoDialog::UpdateInfoSelections()
{
    QItemSelectionModel *selectionModel = ui->atomsView->selectionModel();
    UpdateAtomsSelection(selectionModel);
    selectionModel = ui->bondsView->selectionModel();
    UpdateBondsSelection(selectionModel);
}

void InfoDialog::populateBondsModel(bool ifEdit)
{
    QList<QStandardItem *>  *bond_info = md->FillListBonds(ifEdit);
    for (int i=0; i<md->nconn_prot; i++)
         modelBonds->appendRow(bond_info[i]);
}

// Info ->Angles

void InfoDialog::FillInfoAngles()
{
    QStringList headers = {"Number", "Type", "Start Atom", "Vertex", "End Atom",
                           "Angle"};
    if(modelAngles)
    {
       modelAngles->clear();
       delete modelAngles;
    }
    modelAngles = CreaLista(headers);
    populateAnglesModel();
    setStdProperties(ui->anglesView);
    ui->anglesView->setModel(modelAngles);
    AlignFloatDelegate *delegate0 = new AlignFloatDelegate(ui->anglesView);
    delegate0->setDecimals(0);
    ui->anglesView->setItemDelegateForColumn(0, delegate0);
    AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->anglesView);
    delegate->setDecimals(5);
    ui->anglesView->setItemDelegateForColumn(5, delegate);
    QItemSelectionModel *selectionModel = ui->anglesView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::anglesSelectionChanged);
}

void InfoDialog::anglesSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     if(pgSel)
        return;
     for(int i=0; i<md->nats; i++)
         md->atm[i].selected = false;
     md->atoms_sel.clear();
     for(int i=0; i<md->nconn_prot; i++)
         md->bonds[i].selected = false;
     md->bonds_sel.clear();

     QModelIndexList selection = ui->anglesView->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
        if(index.column() == 0)  {
            int row = modelAngles->data(index, Qt::DisplayRole).toInt() - 1;
            md->SelezionaAngolo(row);
        }
     }
     jav->Draw();
     jav->update_items_modify(true, false);
}

void InfoDialog::populateAnglesModel()
{
    QList<QStandardItem *>  *angles_info = md->FillListAngles();
    for (int i=0; i<md->nangbonds; i++)
         modelAngles->appendRow(angles_info[i]);
}

// Info ->Torsions

void InfoDialog::FillInfoTorsions()
{
    QStringList headers = {"Number", "Type", "Atom 1", "Atom 2", "Atom 3",
                           "Atom 4", "Angle"};
    if(modelTorsions)
    {
       modelTorsions->clear();
       delete modelTorsions;
    }
    modelTorsions = CreaLista(headers);
    populateTorsionsModel();
    setStdProperties(ui->torsionsView);
    ui->torsionsView->setModel(modelTorsions);
    AlignFloatDelegate *delegate0 = new AlignFloatDelegate(ui->torsionsView);
    delegate0->setDecimals(0);
    ui->torsionsView->setItemDelegateForColumn(0, delegate0);
    AlignFloatDelegate *delegate = new AlignFloatDelegate(ui->torsionsView);
    delegate->setDecimals(5);
    ui->torsionsView->setItemDelegateForColumn(6, delegate);
    QItemSelectionModel *selectionModel = ui->torsionsView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::torsionsSelectionChanged);
}

void InfoDialog::torsionsSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     if(pgSel)
        return;
     for(int i=0; i<md->nats; i++)
         md->atm[i].selected = false;
     md->atoms_sel.clear();
     for(int i=0; i<md->nconn_prot; i++)
         md->bonds[i].selected = false;
     md->bonds_sel.clear();

     QModelIndexList selection = ui->torsionsView->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
        if(index.column() == 0)  {
            int row = modelTorsions->data(index, Qt::DisplayRole).toInt() - 1;
            md->SelezionaTorsion(row);
        }
     }
     jav->Draw();
     jav->update_items_modify(true, false);
}

void InfoDialog::populateTorsionsModel()
{
    QList<QStandardItem *>  *torsions_info = md->FillListTorsions();
    for (int i=0; i<md->ntorsions; i++)
         modelTorsions->appendRow(torsions_info[i]);
}

// Info ->BarChart

void InfoDialog::FillInfoBarChart()
{
    QVector<bar_legend> barLegend;
    QVector<bar_info> bars = md->FillBarChart(barLegend);
    m_Scene.clear();
    ui->barChartView->setMouseTracking(true);
    ui->barChartView->setScene(&m_Scene);
    m_Scene.populateBarChart(barLegend, bars);
    ui->barChartView->show();
    ui->barChartView->horizontalScrollBar()->setSliderPosition(0);
    int maxi = ui->barChartView->verticalScrollBar()->maximum();
    ui->barChartView->verticalScrollBar()->setSliderPosition(maxi);
}

void BC::populateBarChart(QVector<bar_legend> barLegend, QVector<bar_info> bars)
{
    QVector<qreal> values, nno;
    int delta = 20;
    int margx = 20;
    int nBars = bars.count();
    int nSpec = barLegend.count();
    for(int i=0; i<nSpec; i++)
        nno.append(barLegend.at(i).val);
     float maxNo = *std::max_element(nno.begin(), nno.end());
    
    for (int i=0; i<nBars; i++)
	 values.append(bars.at(i).value);
    float maxInte = *std::max_element(values.begin(), values.end());
    int wid = delta*nBars + 2*margx + 100;
    QRectF rect(0, 0, wid+50, (int) maxNo * 18 + 100);
	
    setSceneRect(rect);
    setBackgroundBrush(QBrush("#FFFFFF"));    
    int xini = margx, xfin = width() - margx;

    int h0 = 10, h1 = rect.height() - 5;
    for(int i=0; i<nSpec; i++)
    {
      QString s = barLegend.at(i).label;
      int hpix = (int) (nno[i] / maxNo * (float)(h1 - h0) + 0.5f);
      for(int ix=xini+30; ix<xfin; ix+=6)
          addLine(ix, h1 - hpix, ix+2, h1 - hpix, QPen("#7F7F7F"));
      addRect(xini, h1-hpix-4, 8, 8, QPen("#7F7F7F"),
                      QBrush(barLegend.at(i).color));
      QGraphicsTextItem *text = addText(s);
      text->setDefaultTextColor("#000000");
      text->setPos(xini+10, h1-hpix-14);
    }
    int x1 = 120;
    int y1 = h1;
    int LargBar = 15;

    Bars.clear();
    Tips.clear();
    for(int i=0; i<nBars; i++)
    {
        float inten =  bars.at(i).value;
        int hpix = (int) ((abs(inten/maxInte)) * (float)(h1 - h0) + 0.5f);
	QGraphicsItem *bar = 
        addRect(x1, y1-hpix, LargBar, hpix,
			QPen(bars.at(i).color.darker(120)),
			QBrush(bars.at(i).color));
	Bars.append(bar);
	Tips.append(bars.at(i).tip);
        x1 += delta;
    }
}

BC::BC(QObject *parent)
    :QGraphicsScene(parent)
{ 
}

void BC::mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent)
{
     QGraphicsItem *item = itemAt(mouseEvent->scenePos(), QTransform());
     QPoint pos((int) mouseEvent->screenPos().x(), (int) mouseEvent->screenPos().y());
     if(item != nullptr)
     {
	int nBars = Bars.count();
	for(int i=0; i<nBars; i++)
        {
	    if(item == Bars.at(i))
               QToolTip::showText(pos, Tips.at(i), nullptr, QRect(), 10000);
	}
     }
     else
        QToolTip::hideText();
}

// Info ->Contacts

void InfoDialog::FillInfoContacts()
{
    QStringList headers = {"Number", "Atom 1", "Atom 2", "Distance",
                           "Symm.Op1", "Symm.Op2"};
    if(modelContacts)
    {
       modelContacts->clear();
       delete modelContacts;
    }
    modelContacts = CreaLista(headers);
    AlignIconDelegate *delegate = new AlignIconDelegate(ui->contactsView);
    ui->contactsView->setItemDelegateForColumn(0, delegate);
    AlignFloatDelegate *delegate1 = new AlignFloatDelegate(ui->contactsView);
    delegate1->setDecimals(5);
    ui->contactsView->setItemDelegateForColumn(3, delegate1);
    populateContactsModel();
    setStdProperties(ui->contactsView);
    ui->contactsView->setModel(modelContacts);
    QItemSelectionModel *selectionModel = ui->contactsView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &InfoDialog::contactsSelectionChanged);
}

void InfoDialog::contactsSelectionChanged(const QItemSelection &, const QItemSelection &)
{
     if(pgSel)
        return;
     for(int i=0; i<md->nlegsH; i++)
         md->legsH[i].selected = false;
     for(int i=0; i<md->nlegsW; i++)
         md->legsW[i].selected = false;

     QModelIndexList selection = ui->contactsView->selectionModel()->selectedIndexes();
     for (const QModelIndex &index : selection) {
        if(index.column() == 0)  {
            int row = modelContacts->data(index, Qt::DisplayRole).toInt() - 1;
	    if(row < md->nlegsH)
	       md->legsH[row].selected = true;
	    else
	       md->legsW[row-md->nlegsH].selected = true;
        }
     }
     jav->Draw();
     jav->update_items_modify(true, false);
}

void InfoDialog::on_comboModels_currentIndexChanged(int index)
{
    m_kmodel = index+1;
    Properties::SetActiveModel(index+1);
    FillInfo(CurrentPage());
}

void InfoDialog::populateContactsModel()
{
    int ncontacts = md->nlegsH + md->nlegsW;
    QList<QStandardItem *>  *contacts_info = md->FillListContacts(&ncontacts);
    for (int i=0; i<ncontacts; i++)
         modelContacts->appendRow(contacts_info[i]);
}

void InfoDialog::FillPages()
{
    int nModels = Properties::nModels;
    ui->labelModels->setVisible(nModels > 1);
    ui->comboModels->setVisible(nModels > 1);
    m_kmodel = Properties::GetActiveModel();
    if(m_kmodel <= 0)
    {
        this->hide();
        return;
    }
    ui->comboModels->blockSignals(true);
    ui->comboModels->clear();

    if(nModels > 1)
    {
       for (int i=1; i<=nModels; i++)
           ui->comboModels->addItem(QString(jav->ModName(i)));
       ui->comboModels->setCurrentIndex(m_kmodel-1);
    }
    ui->comboModels->blockSignals(false);
    md = GetModel(m_kmodel-1);
    md->RF = getRFvalue(m_kmodel);
    getangle_(&m_kmodel);
    gettorsion_(&m_kmodel);
    ui->button_text->setVisible(md->IsProtein());
    ui->button_sequence->setVisible(md->IsProtein());
    ui->button_secondary->setVisible(md->IsProtein());
    FillInfoStructure();
    if(md->IsProtein())
    {
       FillInfoChains();
       FillInfoAtoms_PDB();
       ui->splitter->setSizes(QList<int>({width()/3, 2*width()/3}));
       int ind = ui->splitter->indexOf(ui->chainsView);
       ui->splitter->setStretchFactor(ind, 0);
       ui->splitter->setStretchFactor(1-ind, 1);
    }
    else
       FillInfoAtoms();
    FillInfoBonds();
    FillInfoAngles();
    FillInfoTorsions();
    FillInfoBarChart();
    FillInfoContacts();

    int larg = width();
    int largp = 0;
    for(int i=0; i<modelAtoms->columnCount(); i++)
        largp += ui->atomsView->columnWidth(i);
    larg = max(larg, largp);
    largp = 0;
    for(int i=0; i<modelBonds->columnCount(); i++)
        largp += ui->bondsView->columnWidth(i);
    larg = max(larg, largp);
    largp = 0;
    for(int i=0; i<modelAngles->columnCount(); i++)
        largp += ui->anglesView->columnWidth(i);
    larg = max(larg, largp);
    largp = 0;
    for(int i=0; i<modelTorsions->columnCount(); i++)
        largp += ui->torsionsView->columnWidth(i);
    larg = max(larg, largp);
    int h = height();
    if(h0 == -1)
    {
       h0 = h;
       larg += 20;
    }
    int he = max((larg*9) / 16, h0);
    this->resize(larg, he);
}

void InfoDialog::FillInfo(int page)
{
    if(page < 0)
       page = 0;
    ui->tabWidget->setCurrentIndex(page);
}

QStandardItemModel *InfoDialog::CreaLista(QStringList headers)
{
    QStandardItemModel *model = new QStandardItemModel(this);
    model->setHorizontalHeaderLabels(headers);
    return model;
}

void InfoDialog::setStdProperties(QTableView *table, bool sorting)
{
    table->setEditTriggers(QAbstractItemView::NoEditTriggers);
    table->setSelectionBehavior(QAbstractItemView::SelectRows);
    table->setSelectionMode(QAbstractItemView::MultiSelection);
    table->verticalHeader()->hide();
    table->horizontalHeader()->setStretchLastSection(false);
    table->horizontalHeader()->setStretchLastSection(true);
    if(sorting)
    {
       table->setSortingEnabled(true);
       table->sortByColumn(0, Qt::AscendingOrder);
    }
    table->resizeColumnsToContents();
}

void InfoDialog::numRow(QAbstractItemModel *model, int col, std::vector<int> &iord)
{
     for(int row = 0; row < model->rowCount(); ++row) {
        QModelIndex index = model->index(row, col);
    int val = model->data(index, Qt::DisplayRole).toInt();
    iord.at(val-1) = row;
     }
}

