#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#if !defined(JAV_WIDGET)
#include "javwindow.h"
#endif
#include "myfiledialog.h"
#include "graphitem.h"
#include "plotstyledialog.h"
#include "highlighter.h"
#include "customplotzoom.h"
#include "rangedialog.h"
#include "indexingdialog.h"
#include "peaksearchdialog.h"
#include "backgrounddialog.h"
#include "extrabackgrounddialog.h"
#include "smoothingdialog.h"
#include "spacegdialog.h"
#include "celllistdialog.h"
#include "spacegrouplistdialog.h"
#include "globaloptdialog.h"
#include "exploretrialsdialog.h"
#include "directmethodsdialog.h"
#include "fourierdialog.h"
#include "fourierpeaklistdialog.h"
#include "hkllistdialog.h"
#include "plotsettings.h"
#include "texteditor.h"
#include "rietvelddialog.h"
#include "infotablewidget.h"
#include "expoaboutdialog.h"
#include "intervalsdialog.h"
#include "crystalsceneview.h"
#include "crystalviewmenu.h"

#include <QMainWindow>

typedef struct {
    QString fileName;
    QString fileType;
} RecentFileInfo;

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    enum MouseAction {
        NoZoom,
        HorizontalZoom,
        RectangleZoom,        
        Pan,
        AddBackgroundPoint,
        DeleteBackgroundPoint,
        AddPeak,
        DeletePeak,
        NoAction
    };
    Q_ENUM(MouseAction)

    enum RecentFileType {Input, Data, Structure, Project};
    Q_ENUM(RecentFileType)

    enum EnabledActions {
        InitAction,
        PatternAction,
        StructureAction,
        PatternAndStructureAction,
        DialogOpenAction,
        RunAction,
        RunSkipAction,
        NextAction,
        PhaseAction,
        SaveAction,
        RestoreAction,
        IntervalsAction,
        ExtraAction,
        ExtraBackgroundAction,
        ProfileAction,
        CycleAction,
        PeaksAction
    };
    Q_ENUM(EnabledActions)

    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();
    QProgressBar *getStatusProgressBar() const;
    double getLastYOffset() const;
#if defined(JAV_WIDGET)
    CrystalViewWidget *getJav() const;
#else
    JavWindow *getJav() const;
#endif
    InfoTableWidget *getInfoTable() const;
    CustomPlotZoom *getPlot() const;
    CustomPlotZoom *getPlot2() const;
    void clearPlot2();    

    void setGraphicArea();
    void addPlot(float x[], float y[], int num, int type, int visible, float wave, const QString name);
    void addPlot(float x[], int num, int type, int visible, float wave);
    void draw();
    void redrawPlot(bool computeLimits = true);    
    void applyOffset(double yOffset = 0);    
    void setAction(const MouseAction &action, bool writeConfig = true);
    void setCurrentFile(const QString &fullFileName, const QString &fileType);
    void setOutputFileName(const QString &value);
    void setStatusMessage(const QString &message);
    void clearStatusMessage();
    void openSpgDialog(double cell[6], int &sgIndex, QString &spg, QString &content, int &findspace, int &ret);
    void openCellDialog(cell_info_type cell[], int ncell, int &selectedCell);
    void openSpaceGroupListDialog(spgext_info_type spgList[], int listSize, int &selectedSpace);
    void openExploreTrialsDialog(int kproc, int saf, int nset, int selset, trialInfo tinfo[]);
    void openGlobalOptDialog(bool ifButton);
    void openFourierPeakDialog(FourierPeakInfo fPeaks[], int nPeaks, int nOldPeaks);
    void openRietveldDialog(const refine_condition_type &rcond);
    void openHklListDialog(c_hkl_info hklInfo[], int nHkl, int vis[]);
    void exportCellList();
    void setSelectedRef(const QVector<int> &selected);
    void drawPeaks();
    void drawSelectedPeaks(const QVector<int> &selected);
    void deleteSelectedPeaks();
    void clearSelectedPeaks();
    void drawSelectedRef();
    void clearSelectedRef();
    void setSystematicAbsences(const QVector<int> &refIndex);
    void drawSystematicAbsences();
    void clearSystematicAbsences();
    void updatePeakListTable();
    bool rescalePlotEnabled() const;
    void enableRescalePlot(bool value);
    void checkAction(MouseAction action);
    void saveAction();
    void restoreAction();
    void setZoomAction();
    QString getCurrentFile() const;
    void enableActions(MainWindow::EnabledActions action, bool state=false);
    void setNextActionVisible(bool visible);
    void enableNextAction();
    void saveEnabledActions();
    void restoreEnabledActions();
    void openHelp(const QString &page = QString());
    void setCrystalDockTitle(const QString &title);
    void showCrystalDock();
    void enableMenu(QMenu *menu, bool enab);
    bool openImportStructure(const QString &title = "");
    void loadExpFile(const QString &fileName, const QString outName);

    static QString getPathDataFiles();
    static void setPathDataFiles(const QString &newPathDataFiles);
    CrystalViewMenu *getCrystalViewMenu() const;
    void crystalViewMenuEnabled(bool enabled);
    void resizeMyDocks();
    
    QString loadExamplesFolder();
    
signals:
    void mainWindowClosed();
    void actionStopTriggered();

private slots:
    void closeEvent(QCloseEvent *event);
    void on_editSelected(QString filename, MyFileDialog *dlg);
    void on_actionAutoscale_triggered();
    void horzScrollBarChanged(int value);
    void xAxisChanged(QCPRange range);
    void myMoveEvent(QMouseEvent *event);
    void zoomGroupTriggered(QAction *action);
    void plotStyleClosed(QDialogButtonBox::StandardButton button);
    void selectionChanged();
    void addDeletePoint(QPoint mousePos);

    //File
    void on_actionLoad_Go_triggered();
    void openRecentFile();
    void on_actionLoad_Examples_triggered();
    void on_actionImport_Structure_triggered();
//    void onActionExportStructureTriggered();
    void onActionImagePowderPatternTriggered();
    void on_actionPowder_Pattern_triggered();
    void on_actionReflections_triggered();
    void on_actionImport_Diffraction_Pattern_triggered();
    void on_actionLoad_Project_triggered();
    void on_actionSave_Project_triggered();
    void on_actionSave_Project_As_triggered();
    //void on_actionExit_triggered();

    //Pattern
    void on_actionRange_triggered();
    void actionIntervalsTriggered();
    void on_actionBackground_triggered();
    void actionExtraBackgroundTriggered();
    void on_actionSmoothing_triggered();
    void on_actionPeak_Search_triggered();
    void on_actionPeak_Search_Conditions_triggered();
    void on_actionLoad_Peaks_triggered();    
    void on_actionSave_Peaks_triggered();

    void on_actionN_Treor_triggered();
    void on_actionDICVOL_triggered();
    void on_actionMcMaille_triggered();
    void on_actionOutputN_TREOR09_triggered();
    void on_actionOutputDICVOL14_triggered();
    void on_actionOutputMcMaille_triggered();
    void on_actionCell_List_triggered();

    void on_actionSpace_Group_Determination_triggered();
    void on_actionSpace_Group_List_triggered();

    //View
    void on_actionPlot_Style_triggered();
//    void on_actionJav_Molecular_Viewer_triggered();
    void on_actionReset_Zoom_triggered();

    //Solve
    void on_actionExplore_Trials_triggered();
    void on_actionRAMM_Procedure_triggered();
    void on_actionSimulated_Annealing_triggered();
    void on_actionRecycle_in_Extra_triggered();
    void on_actionDirect_Methods_triggered();

    //Refine
    void on_actionRBM_Direct_Space_triggered();
    void on_actionRBM_Direct_and_Reciprocal_Space_triggered();
    void on_actionRBM_COVMAP_triggered();
    void on_actionFourier_Recycling_triggered();
    void on_actionFourier_triggered();
    void on_actionRietveld_triggered();

    //Info
    void on_actionHKL_List_triggered();
    void on_actionSummary_triggered();
    void on_actionView_Output_File_triggered();
    void actionProfileTriggered(bool checked);

    //Window
    void on_actionDefault_Layout_triggered();

    //Help
    void actionDocumentationHtmlTriggered();
    void actionDocumentationPdfTriggered();
    void actionCheckForUpdatesTriggered();
    void actionAboutTriggered();

    //Run toolbar
    void on_actionStop_triggered();
    void on_actionSkip_triggered();

    //Next toolbar
    void actionCycleTriggered();

    //CrystalView
    void onActionLabelsOnOfftriggered();

private:
    void createDialogs();
    void actionsSetup();
    void writeSettings();
    void readSettings();
    void readAction();
//    void createActionConnect();
    void makePlot(const QVector<double> &x, const QVector<double> &y, graphItem &item);
    void plotStyleConfig();
    void getYLimits(double &yMin, double &yMax);
    void drawReflections(const QVector<double>& x, double y, double length, QPen pen, int &itemStart, int &itemEnd);
    //void drawIntervals(const QVector<double>& x, const QVector<double>& y);
    void drawIntervals();
    void reDrawReflections(graphItem &ref, double y, double length);
    void clearGraphSelection(graphItem &item);
    void drawGraphicItem(graphItem &item);    
//    void createOpenGLDock();
    void createCrystalViewMenu();
    void createCrystalViewConnect();
    void exportFile(const QString &fileName, int tipo);

    Ui::MainWindow *ui;
#if !defined(JAV_WIDGET)
    JavWindow *jav;
#endif
    CrystalSceneView *m_crystalScene;

    QLabel *statusLabel1;
    QProgressBar *statusProgressBar;
    bool single_it;
    MouseAction mAction;
    MouseAction savedAction;
    MouseAction savedZoomAction;
    QMap<QAction *, bool> stateActions;
    void enumerateEnabledActionsMenu(QMenu *menu);

    //Dialog Windows
    MyFileDialog *mFileDialog;
    RangeDialog *rangeDialog;
    IntervalsDialog *intervalsDialog;
    IndexingDialog *indexingDialog;
    PeakSearchDialog *peakSearchDialog;
    PlotStyleDialog *plotStyleDialog;
    BackgroundDialog *backgroundDialog;
    ExtraBackgroundDialog *extraBackgroundDialog;
    SmoothingDialog *smoothingDialog;
    SpacegDialog *spgDialog;
    CellListDialog *cellListDialog;
    SpaceGroupListDialog *spaceListDialog;
    ExploreTrialsDialog *exploreTrialsDialog;
    GlobalOptDialog *globOptDialog;
    DirectMethodsDialog *dmDialog;
    FourierDialog *fourDialog;
    FourierPeakListDialog *fourPeakDialog;
    HklListDialog *hklDialog;
    TextEditor *textViewer;
    RietveldDialog *rietDialog;
    ExpoAboutDialog *aboutDialog;

    QVector<graphItem> obs;
    QVector<graphItem> refl;
    QVector<graphItem> profCurves;
    graphItem calc;
    graphItem back;
    graphItem bpoints;
    graphItem diff;
    graphItem cdiff;
    graphItem peaks;
    graphItem smooth;
    graphItem undpeaks;
    graphItem sysAbs;
    graphItem selectedRef;
    graphItem selectedPeak;
    graphItem intervalLimit;
    int nObserved;
    int nReflections;
    int nProfCurves;
    QVector<double> plotWave;    

    double yLowerRange, yUpperRange;
    double yLowerRangeNoRef;

    Highlighter *m_highlighter;
    const int scrollBarLen = 1000000000; //Length of the scrollbar. It can be any large value.
    PlotSettings plotSettings;

    //Files
    QString currentFile;
    QString outputFileName;
    QString currentProjectFile;
    bool projectFileSaved;
    void clearProjectFile();
    static QString pathDataFiles;
    enum { maxFileNr = 10 };
    QList<QAction*> recentFileActionList;
    void saveFile(const QString &fileName);
    void createRecentActions();
    void updateRecentFileActions();
    void setRecentFiles(const QString &fullFileName, const QString &fileType);
    QString getLastFile() const;    

    double mLastYOffset;
    bool rescalePlot;  

    QByteArray defaultState;    

    void createActionGroup();
    void applyRefine(int action);

    enum {STOP_EVENT=1, SKIP_EVENT, CYCLE_EVENT};

    //CrystalView Menu
    CrystalViewMenu *crystalViewMenu;
    QMenu *structureMenu;
};

MainWindow* getMainWindow();

#endif // MAINWINDOW_H
