#include "qtdownload.h"
#include <QFileInfo>
#include <QProgressBar>
#include <QDir>

QtDownload::QtDownload(QObject *parent, QProgressBar *progBar) :
    QObject(parent),
    m_progBar(progBar)
{
    m_progBar = progBar;
    m_file = nullptr;
    m_reply = nullptr;
    m_manager = new QNetworkAccessManager(parent);
}

QString QtDownload::getFileName() const
{
    return m_file->fileName();
}

QtDownload::~QtDownload() {
    delete m_manager;
}

void QtDownload::startRequest(QUrl url, QString fileName)
{
    if (fileName.isEmpty()) {
        // create file name from url
        QFileInfo fileInfo(url.path());
        fileName = fileInfo.fileName();
        if (fileName.isEmpty()) fileName = "downlodedfile";
    }

    // create the file and open it for reading
    m_file = new QFile(fileName);
    if (!m_file->open(QIODevice::WriteOnly)) {
        delete m_file;
        m_file = nullptr;
        return;
    }

    // create network request to download stuff
    m_reply = m_manager->get(QNetworkRequest(url));

    // connect the signals to monitor the download
    connect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadReadyRead()));
    if (m_progBar) connect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(updateDownloadProgress(qint64,qint64)));
    connect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
}

void QtDownload::downloadReadyRead()
{
    if (m_file) m_file->write(m_reply->readAll());
}

void QtDownload::updateDownloadProgress(qint64 bytesRead, qint64 totalBytes)
{
    if (totalBytes < 1)
    { //unable to determine file size, so display a busy indicator
        m_progBar->setMinimum(0);
        m_progBar->setMaximum(0);
        return;
    }
    //set the progress bar value
    m_progBar->setMaximum(100);
    m_progBar->setValue(static_cast<int>(100.0f*bytesRead/totalBytes));
    //qDebug() << "Total: " << totalBytes << " Bytes: " << bytesRead << " Progress: " << (100.0 * bytesRead) / totalBytes << endl;
}

void QtDownload::downloadFinished()
{
    QString errMsg = "";

    // download fineshed normally
    if (m_progBar) {
        m_progBar->setValue(0);
        m_progBar->setMaximum(100);
    }
    m_file->flush();
    m_file->close();

    // get redirection url
    QVariant redirectionTarget = m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
    if (!redirectionTarget.isNull())
    {
        QUrl newUrl = redirectionTarget.toUrl();
        m_reply->deleteLater();
        m_file->open(QIODevice::WriteOnly);
        m_file->resize(0);

        //create a network request to download stuff
        m_manager = new QNetworkAccessManager();
        m_reply = m_manager->get(QNetworkRequest(newUrl));
        connect(m_reply, SIGNAL(readyRead()), this, SLOT(httpReadyRead()));
        if (m_progBar) connect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(updateDownloadProgress(qint64,qint64)));
        connect(m_reply, SIGNAL(finished()), this, SLOT(httpDownloadFinished()));
        return;
    }

    bool success = true;
    if (m_reply->error()) {
        success = false;
        m_file->remove();
        errMsg = m_reply->errorString();
    }       
    m_reply->deleteLater();
    m_reply = nullptr;
    errMsg = m_file->fileName();
    delete m_file;
    m_file = nullptr;

    emit downloadComplete(success, errMsg);
}
