#include "restraintswidget.h"
#include "floatdelegate.h"
#include "checkboxdelegate.h"
#include "ui_restraintswidget.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "mainwindow.h"

#include <QTextStream>

RestraintsWidget::RestraintsWidget(bool esdCol, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::RestraintsWidget),
    isEsd(esdCol),
    activeCol(ACTIVATE_COL)
{
    ui->setupUi(this);

    createRestraintModel();
    setRestraintTableView();
    auto doubleValidator = new QDoubleValidator(0.0, 999999.0, 3, this);
    ui->weiLineEdit->setValidator(doubleValidator);
}

RestraintsWidget::~RestraintsWidget()
{
    delete ui;
}

void RestraintsWidget::setRestraints(const QVector<crydata::RestraintProperties> &restr, const QStringList &labelList, double wei, const QString &restraintString)
{
    restrList = restr;
    labels = labelList;
    updateRestraintModel(restr);
    int nChecks = rowCheckBoxClickedCount(activeCol);
    cbHeader->setChecked(activeCol, (nChecks == resModel->rowCount() && nChecks));
    ui->weightLabel->setText("Weight on "+restraintString+" restraints: ");
    ui->weiLineEdit->setText(QString::number(wei));
}

QVector<crydata::RestraintProperties> RestraintsWidget::getRestraints()
{
    for (int r = 0; r < resModel->rowCount(); r++) {
        QModelIndex index = resModel->index(r,TARGET_COL,QModelIndex());
        restrList[r].targ = index.data().toFloat();

        if (isEsd) {
            index = resModel->index(r,ESD_COL,QModelIndex());
            restrList[r].sigma = index.data().toFloat();
        }

        index = resModel->index(r,ACTIVATE_COL,QModelIndex());
        restrList[r].active = index.data().toBool();
    }

    return restrList;
}

void RestraintsWidget::getSettings(double &wei) const
{
    wei = ui->weiLineEdit->text().toDouble();
}

void RestraintsWidget::printRestraints(QTextStream &stream, const QString &weiString)
{
    stream.setRealNumberPrecision(6);
    for (int r=0; r < resModel->rowCount(); r++) {
        stream << qSetFieldWidth(12) << restraintString(restrList.at(r), labels, " ") << qSetFieldWidth(0) << " ";
        QModelIndex index = resModel->index(r,TARGET_COL,QModelIndex());
        stream << qSetFieldWidth(10) << index.data().toFloat();
        if (isEsd) {
            index = resModel->index(r,ESD_COL,QModelIndex());
            stream << qSetFieldWidth(0) << " " << qSetFieldWidth(10) << index.data().toFloat();
        }
        stream << qSetFieldWidth(0) << Qt::endl;
    }
    stream << weiString << " " << ui->weiLineEdit->text().toFloat() << Qt::endl;
}

void RestraintsWidget::createRestraintModel()
{
    resModel = new QStandardItemModel(this);
    resProxyModel = new QSortFilterProxyModel(this);

    resProxyModel->setSourceModel(resModel);

    QStringList titles = {"#","Type", "Atoms", "Current", "Target", "ESD", "Active"};
    if (!isEsd) {
        titles.removeAt(ESD_COL);
        activeCol--;
    }

    resModel->setColumnCount(titles.size());
    resModel->setHorizontalHeaderLabels(titles);
}

void RestraintsWidget::setRestraintTableView()
{
    ui->resTableView->setModel(resProxyModel);
    ui->resTableView->verticalHeader()->hide();
    ui->resTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->resTableView->setShowGrid(false);
    ui->resTableView->setAlternatingRowColors(true);
    ui->resTableView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    //ui->resTableView->setSelectionMode(QAbstractItemView::MultiSelection);
    ui->resTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    //ui->resTableView->setSelectionBehavior(QAbstractItemView::SelectItems);
    ui->resTableView->horizontalHeader()->stretchLastSection();

    auto floatDelegate = new FloatDelegate(ui->resTableView);
    floatDelegate->setDecimals(5);
    ui->resTableView->setItemDelegateForColumn(CURRENT_COL, floatDelegate);
    ui->resTableView->setItemDelegateForColumn(TARGET_COL, floatDelegate);
    ui->resTableView->setItemDelegateForColumn(ESD_COL, floatDelegate);

    auto restrCheckDelegate = new CheckBoxDelegate(ui->resTableView);
    restrCheckDelegate->setColumn(activeCol);
    ui->resTableView->setItemDelegateForColumn(activeCol, restrCheckDelegate);
    connect(restrCheckDelegate, &CheckBoxDelegate::rowChecked, this,
            &RestraintsWidget::checkBoxClicked);

    cbHeader = new CheckBoxHeader(Qt::Horizontal, {activeCol}, true, ui->resTableView);
    ui->resTableView->setHorizontalHeader(cbHeader);
    connect(cbHeader, &CheckBoxHeader::checkBoxClicked, this, &RestraintsWidget::headerCheckBoxClicked);

    ui->resTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    ui->resTableView->setSortingEnabled(true);
    ui->resTableView->sortByColumn(ID_COL,Qt::AscendingOrder);

    connect(ui->resTableView->selectionModel(), &QItemSelectionModel::selectionChanged,
            this, &RestraintsWidget::markSelectedRestraints);
}

QString RestraintsWidget::restraintString(const crydata::RestraintProperties &restr, const QStringList &labels, const QString &sep)
{
    if (restr.atm.size() == 0) return QString();

    QString rStr;
    for (int i = 0; i < restr.atm.size() - 1; i++) {
        rStr.append(labels.at(restr.atm.at(i))+sep);
    }
    rStr.append(labels.at(restr.atm.at(restr.atm.size()-1)));

    return rStr;
}

int RestraintsWidget::rowCheckBoxClickedCount(int col) const
{
    int count = 0;
    for (int r = 0; r < resModel->rowCount(); r++) {
        QModelIndex index = resModel->index(r,col,QModelIndex());
        if (index.data().toBool()) count++;
    }
    return count;
}

void RestraintsWidget::updateRestraintModel(const QVector<crydata::RestraintProperties> &restr)
{
    resModel->setRowCount(0);

    for (int r = 0; r < restr.size(); r ++) {
        QList<QStandardItem *> rowData;

        auto item = new QStandardItem();
        item->setData(r+1, Qt::DisplayRole);
        item->setTextAlignment(Qt::AlignRight);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        rowData.append(item);

        item = new QStandardItem(restr.at(r).type.join('-'));
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        rowData.append(item);

        item = new QStandardItem(restraintString(restr.at(r), labels));
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        rowData.append(item);

        item = new QStandardItem();
        item->setData(restr.at(r).val, Qt::DisplayRole);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        rowData.append(item);

        item = new QStandardItem();
        item->setData(restr.at(r).targ, Qt::DisplayRole);
        rowData.append(item);

        if (isEsd) {
            item = new QStandardItem();
            item->setData(restr.at(r).sigma, Qt::DisplayRole);
            rowData.append(item);
        }

        item = new QStandardItem();
        item->setData(restr.at(r).active, Qt::DisplayRole);
        rowData.append(item);

        resModel->appendRow(rowData);
    }
}

void RestraintsWidget::headerCheckBoxClicked(bool state, int column)
{
    for (int row = 0; row < resModel->rowCount(); row++) {
        QModelIndex index = resModel->index(row, column, QModelIndex());
        resModel->setData(index, QVariant(state?1:0));
    }

    emit restraintsNumberChanged(rowCheckBoxClickedCount(column));
}

void RestraintsWidget::checkBoxClicked(const QModelIndex &index, bool state)
{
    int nChecks = rowCheckBoxClickedCount(index.column());

    //If state is false uncheck the checkBoxHeader
    if (!state && cbHeader->isChecked(index.column())) {
        cbHeader->setChecked(index.column(), false);
        return;
    } else {
        //If state is true check the checkBoxheader if all checkboxes have been clicked
        if (state && !cbHeader->isChecked(index.column())) {
            if (nChecks == resModel->rowCount())
                cbHeader->setChecked(index.column(), true);
        }
    }

    emit restraintsNumberChanged(nChecks);
}

void RestraintsWidget::markSelectedRestraints()
{
    QModelIndexList selection = ui->resTableView->selectionModel()->selectedRows();
    QVector<int> selAtoms;
    for(int i=0; i < selection.count(); i++)
    {
        int idRes = resProxyModel->mapToSource(selection.at(i)).row();
        selAtoms.append(restrList.at(idRes).atm);
    }

    MainWindow *mw = getMainWindow();
    if (!mw) return;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = mw->getJav();
#else
    JavWindow *jav = mw->getJav();
#endif
    if (jav->isVisible()) {
        jav->selectAtomsAndLegs(selAtoms);
#if !defined(JAV_WIDGET)
        //Raise only if JAV doesn't overlap with SA dialog
        if (!this->window()->geometry().intersects(jav->geometry()))
            jav->raise();
#endif
    }
}
