module nist_mod

   implicit none
   
   private 

   public :: get_nist_parameters
   public :: print_nist_parameters
   public :: nist_set_folder
   public :: generate_load_nist_table
   public :: angstroms_to_kev, kev_to_angstroms

   character(len=:), allocatable   :: nist_folder
   integer, parameter              :: MAXZVAL = 92
   real, allocatable, dimension(:) :: nist_tab
   real                            :: frel

!  To generate the subroutine load_nist_table: 
!
!  1) Download nist tables for all 92 elementes using the following bash script here reported:
!
!         #!/bin/bash
!         htm="https://physics.nist.gov/cgi-bin/ffast/ffast.pl?Z=zz&Formula=&gtype=4&range=S&lower=0.0&upper=433&density=&frames=no"
!         for i in `seq 1 92`;
!         do
!         result=$(echo "$htm" | sed -r "s/zz/"$i"/g")
!         echo $result
!         wget -O $i.html $result
!         done
!
!  2) Use the following two lines of program to create the subroutine, assuming the files are in the folder elements
!
!         ......
!         call nist_set_folder("../elements/")
!         call generate_load_nist_parameters(err)
!         ......
!     The subroutine will be printed on the standard output
!
contains

   subroutine get_nist_parameters(zval,wave,f1val,f2val,muval,err,kpr)
!
!  Look up f', f'', mu  for element Z in the NIST table
!
   use errormod
   use fileutil
   use iso_fortran_env, only: stdout => output_unit
   use lib_array
   integer, intent(in)             :: zval          ! Z 
   real, intent(in)                :: wave          ! wavelength in angstrom
   real, intent(out)               :: f1val, f2val  ! f' and f''
   real, intent(out)               :: muval         ! mass attenuation coefficient (cm^2/g)        
   type(error_type), intent(out)   :: err           ! error reading table
   integer, intent(in), optional   :: kpr           ! if >= 0 table for element Z is printed on unit kpr
   real, dimension(:), allocatable :: kev,f1,f2,mu
   integer                         :: i,j,nrow
   integer, parameter              :: NCOL = 4
   real                            :: xvalue
   real                            :: f1calc,f2calc,mucalc
!
   f1val = 0.0
   f2val = 0.0
   muval = 0.0
   call load_nist_table(zval)
!
   if (.not.allocated(nist_tab)) return
!
   nrow = size(nist_tab) / NCOL
   allocate(kev(nrow),f1(nrow),f2(nrow),mu(nrow))

   do i=1,nrow
      j = (i-1)*NCOL + 1
      kev(i) = nist_tab(j)
      f1(i) = nist_tab(j+1)
      f2(i) = nist_tab(j+2)
      mu(i) = nist_tab(j+3)
   enddo
!
!  linear interpolation in log-log space
   xvalue = angstroms_to_kev(wave)
   mucalc = interp1d_loglog(kev,mu,xvalue)
   f1calc = interp1d(kev,f1,xvalue) - zval + frel
   f2calc = interp1d_loglog(kev,f2,xvalue) 
   if (libarr_error) then
       call err%set(libarr_error_string)
       return
   else
       muval = mucalc
       f1val = f1calc
       f2val = f2calc
   endif
!
   if (present(kpr)) then
       if (kpr >= 0) then
           write(stdout,'(a)')"             E(keV)      wave(ang)     f1(e atom-1)  f2(e atom-1)  mu/p(cm2g-1)"
           do i=1,nrow
              write(stdout,'(i6,5(f15.6))')i,kev(i),kev_to_angstroms(kev(i)),f1(i),f2(i),mu(i)
           enddo
       endif
   endif
!
   end subroutine get_nist_parameters

!  --------------------------------------------------------------------------------------------

   subroutine print_nist_parameters(prn,wave,f1,f2,mu)
   integer, intent(in) :: prn
   real, intent(in)    :: wave,f1,f2,mu
!
   if (prn < 0) return
   write(prn,'(a,f12.6,a,f12.6,a)')"X-ray dispersion coefficients for lambda = ",wave, &
                                   " ang => ", angstroms_to_kev(wave)," kev"
   write(prn,'(a)')      "        f'         f''       mu/r (cm^2/g)" 
   write(prn,'(3(f12.5))')f1,f2,mu
!
   end subroutine print_nist_parameters

!  --------------------------------------------------------------------------------------------

   real function angstroms_to_kev(angstroms)
   implicit none
   real, intent(in) :: angstroms
   real, parameter :: h = 6.62607015e-34  ! Planck's constant
   real, parameter :: c = 2.998e8         ! speed of light
   real, parameter :: e = 1.602176634e-19 ! elementary charge
   real, parameter :: conversion_const = (1e7)*h*c/e
   
   angstroms_to_kev = conversion_const/angstroms
   
   end function angstroms_to_kev

!  --------------------------------------------------------------------------------------------

   real function kev_to_angstroms(kev)
   implicit none
   real, intent(in) :: kev
   real, parameter :: h = 6.62607015e-34  ! Planck's constant
   real, parameter :: c = 2.998e8         ! speed of light
   real, parameter :: e = 1.602176634e-19 ! elementary charge
   real, parameter :: conversion_const = (1e7)*h*c/e
   
   kev_to_angstroms = conversion_const/kev
   
   end function kev_to_angstroms

!  --------------------------------------------------------------------------------------------

   subroutine nist_set_folder(foldername)
   character(len=*), intent(in) :: foldername
!
   nist_folder = foldername
!
   end subroutine nist_set_folder

!  --------------------------------------------------------------------------------------------

   subroutine generate_load_nist_table(err)
   use errormod
   use iso_fortran_env, only: stdout => output_unit
   type(error_type), intent(out)  :: err
   integer                        :: zval
!
   write(stdout,'(3x,a)')  'subroutine load_nist_table(zval)'
   write(stdout,'(3x,a/a)')'integer, intent(in) :: zval','!'
   write(stdout,'(3x,a)')  'if (allocated(nist_tab)) deallocate(nist_tab)'
   write(stdout,'(3x,a)')  'select case(zval)'
!
   do zval=1,MAXZVAL
      call parse_nist_html(zval,err)
      if (err%signal) return
   enddo
!
   write(stdout,'(3x,a)')   'end select'
   write(stdout,'(a/,3x,a)')'!','end subroutine load_nist_table'
   
!
   end subroutine generate_load_nist_table

!  --------------------------------------------------------------------------------------------

   subroutine parse_nist_html(zval,err)
   use strutil
   use fileutil
   use errormod
   use iso_fortran_env, only: stdout => output_unit
   integer, intent(in)                         :: zval
   type(error_type), intent(out)               :: err
   type(file_handle)                           :: filen
   character(len=:), allocatable               :: sfile
   integer                                     :: ierror,ier,nline
   character(len=:), allocatable               :: line
   character(len=:), allocatable, dimension(:) :: wordv
   integer                                     :: nword, nrow
   character(len=:), allocatable               :: frelstring
!
   if (zval <= 0 .or. zval > MAXZVAL) then
       call err%set("Can't read NIST file for Z="//i_to_s(zval))
       return
   endif
!
   sfile = nist_folder//i_to_s(zval)//".html"
!
   call filen%fopen(sfile)
   if (filen%fail()) then
       call err%set(filen%err_msg())
       return
   endif
!
!  Count array size
   call find_key_file_a(filen%handle(),'Photoelectric Coh+inc      Total',nline,line,ier)
   read(filen%handle(),*,iostat=ierror,err=10)
   nrow = 0
   do while(get_line(filen%handle(),line,trimmed=.true.,filter=.true.))
      if (index(line,'BODY') > 0) exit
      nrow = nrow + 1
   enddo
   call filen%rew()
!
!  Find relativistic correction
   call find_key_file_a(filen%handle(),'Relativistic correction estimate',nline,line,ier)
   call get_words1(line,wordv,nword)
   frelstring = trim(wordv(8))
!
!  Print array
   call find_key_file_a(filen%handle(),'Photoelectric Coh+inc      Total',nline,line,ier)
   read(filen%handle(),*,iostat=ierror,err=10)

   write(stdout,'(3x,a)')'case('//i_to_s(zval)//')'
   write(stdout,'(6x,a)')'frel = '//frelstring
   write(stdout,'(6x,a)')'allocate(nist_tab('//i_to_s(nrow*4)//'))'
   write(stdout,'(6x,a,i0,a)')'nist_tab(:) = [  &'
   write(stdout,'(a)')'!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)'
   
   nrow = 0
   do while(get_line(filen%handle(),line,trimmed=.true.,filter=.true.))
      if (index(line,'BODY') > 0) exit
      nrow = nrow + 1
      if (nrow > 1) then
          write(stdout,'(a)',advance='yes')', &'
      endif
      call get_words1(line,wordv,nword)
      write(stdout,'(6x,a,3(a,", "),a,a)',advance='no') '', &
            trim(wordv(1)),trim(wordv(2)),trim(wordv(3)),trim(wordv(6))
   enddo

   write(stdout,'(a)',advance='yes')']'

   return

10 call err%set("Error reading file: "//sfile)

   end subroutine parse_nist_html

!  --------------------------------------------------------------------------------------------

   subroutine load_nist_table(zval)
   integer, intent(in) :: zval
!
   if (allocated(nist_tab)) deallocate(nist_tab)
   select case(zval)
   case(1)
      frel = 0.0000E+00
      allocate(nist_tab(660))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 4.54682E-01, 0.0000E+00, 2.2129E-01, &
      1.142761E-02, 3.97170E-01, 0.0000E+00, 2.4489E-01, &
      1.221612E-02, 2.99485E-01, 0.0000E+00, 2.7018E-01, &
      1.305903E-02, 6.85829E-02, 0.0000E+00, 2.9718E-01, &
      1.332800E-02, -1.17232E-01, 0.0000E+00, 3.0577E-01, &
      1.353200E-02, -5.16414E-01, 0.0000E+00, 3.1227E-01, &
      1.358640E-02, -9.98811E-01, 0.0000E+00, 3.1401E-01, &
      1.366800E-02, -4.92163E-01, 9.5197E-01, 2.9076E+06, &
      1.387200E-02, -4.62805E-02, 9.3030E-01, 2.7996E+06, &
      1.396010E-02, 4.68513E-02, 9.2117E-01, 2.7547E+06, &
      1.492335E-02, 4.91043E-01, 8.2970E-01, 2.3210E+06, &
      1.595306E-02, 6.84358E-01, 7.4640E-01, 1.9532E+06, &
      1.705382E-02, 8.03468E-01, 6.7066E-01, 1.6417E+06, &
      1.823053E-02, 8.84640E-01, 6.0188E-01, 1.3783E+06, &
      1.948844E-02, 9.42415E-01, 5.3953E-01, 1.1557E+06, &
      2.083314E-02, 9.84335E-01, 4.8306E-01, 9.6798E+05, &
      2.227063E-02, 1.01491E+00, 4.3201E-01, 8.0981E+05, &
      2.380730E-02, 1.03710E+00, 3.8592E-01, 6.7671E+05, &
      2.545001E-02, 1.05295E+00, 3.4435E-01, 5.6484E+05, &
      2.720606E-02, 1.06395E+00, 3.0692E-01, 4.7095E+05, &
      2.908327E-02, 1.07121E+00, 2.7325E-01, 3.9222E+05, &
      3.109002E-02, 1.07558E+00, 2.4301E-01, 3.2631E+05, &
      3.323523E-02, 1.07772E+00, 2.1589E-01, 2.7117E+05, &
      3.552846E-02, 1.07815E+00, 1.9159E-01, 2.2512E+05, &
      3.797993E-02, 1.07727E+00, 1.6984E-01, 1.8669E+05, &
      4.060054E-02, 1.07541E+00, 1.5041E-01, 1.5465E+05, &
      4.340198E-02, 1.07283E+00, 1.3306E-01, 1.2799E+05, &
      4.639671E-02, 1.06974E+00, 1.1760E-01, 1.0581E+05, &
      4.959809E-02, 1.06630E+00, 1.0383E-01, 8.7390E+04, &
      5.302035E-02, 1.06265E+00, 9.1576E-02, 7.2105E+04, &
      5.667876E-02, 1.05889E+00, 8.0693E-02, 5.9435E+04, &
      6.058959E-02, 1.05510E+00, 7.1034E-02, 4.8944E+04, &
      6.477028E-02, 1.05134E+00, 6.2473E-02, 4.0266E+04, &
      6.923942E-02, 1.04767E+00, 5.4891E-02, 3.3096E+04, &
      7.401695E-02, 1.04411E+00, 4.8185E-02, 2.7178E+04, &
      7.912411E-02, 1.04070E+00, 4.2260E-02, 2.2297E+04, &
      8.458368E-02, 1.03745E+00, 3.7030E-02, 1.8277E+04, &
      9.041995E-02, 1.03437E+00, 3.2418E-02, 1.4968E+04, &
      9.665893E-02, 1.03147E+00, 2.8356E-02, 1.2248E+04, &
      1.033284E-01, 1.02876E+00, 2.4781E-02, 1.0013E+04, &
      1.104581E-01, 1.02622E+00, 2.1639E-02, 8.1797E+03, &
      1.180797E-01, 1.02387E+00, 1.8879E-02, 6.6761E+03, &
      1.262272E-01, 1.02169E+00, 1.6458E-02, 5.4445E+03, &
      1.349368E-01, 1.01968E+00, 1.4336E-02, 4.4365E+03, &
      1.442475E-01, 1.01782E+00, 1.2477E-02, 3.6123E+03, &
      1.542005E-01, 1.01612E+00, 1.0851E-02, 2.9390E+03, &
      1.648404E-01, 1.01456E+00, 9.4291E-03, 2.3894E+03, &
      1.762144E-01, 1.01313E+00, 8.1876E-03, 1.9411E+03, &
      1.883732E-01, 1.01183E+00, 7.1041E-03, 1.5758E+03, &
      2.013709E-01, 1.01065E+00, 6.1595E-03, 1.2783E+03, &
      2.152655E-01, 1.00957E+00, 5.3367E-03, 1.0362E+03, &
      2.301188E-01, 1.00859E+00, 4.6205E-03, 8.3953E+02, &
      2.459970E-01, 1.00771E+00, 3.9976E-03, 6.7970E+02, &
      2.629708E-01, 1.00691E+00, 3.4563E-03, 5.4995E+02, &
      2.811158E-01, 1.00618E+00, 2.9863E-03, 4.4471E+02, &
      3.005128E-01, 1.00553E+00, 2.5785E-03, 3.5941E+02, &
      3.212482E-01, 1.00494E+00, 2.2249E-03, 2.9031E+02, &
      3.434143E-01, 1.00441E+00, 1.9186E-03, 2.3438E+02, &
      3.671099E-01, 1.00394E+00, 1.6535E-03, 1.8914E+02, &
      3.924405E-01, 1.00351E+00, 1.4241E-03, 1.5256E+02, &
      4.195189E-01, 1.00313E+00, 1.2258E-03, 1.2302E+02, &
      4.484657E-01, 1.00279E+00, 1.0545E-03, 9.9164E+01, &
      4.794098E-01, 1.00248E+00, 9.0664E-04, 7.9915E+01, &
      5.124891E-01, 1.00220E+00, 7.7906E-04, 6.4390E+01, &
      5.478508E-01, 1.00196E+00, 6.6907E-04, 5.1876E+01, &
      5.856525E-01, 1.00174E+00, 5.7430E-04, 4.1793E+01, &
      6.260625E-01, 1.00154E+00, 4.9270E-04, 3.3675E+01, &
      6.692609E-01, 1.00137E+00, 4.2248E-04, 2.7139E+01, &
      7.154399E-01, 1.00121E+00, 3.6208E-04, 2.1880E+01, &
      7.648052E-01, 1.00108E+00, 3.1017E-04, 1.7650E+01, &
      8.175768E-01, 1.00095E+00, 2.6558E-04, 1.4248E+01, &
      8.739896E-01, 1.00084E+00, 2.2730E-04, 1.1514E+01, &
      9.342948E-01, 1.00075E+00, 1.9444E-04, 9.3158E+00, &
      9.987612E-01, 1.00066E+00, 1.6627E-04, 7.5495E+00, &
      1.067676E+00, 1.00058E+00, 1.4211E-04, 6.1302E+00, &
      1.141345E+00, 1.00052E+00, 1.2142E-04, 4.9898E+00, &
      1.220098E+00, 1.00046E+00, 1.0370E-04, 4.0736E+00, &
      1.304285E+00, 1.00040E+00, 8.8528E-05, 3.3374E+00, &
      1.394281E+00, 1.00036E+00, 7.5550E-05, 2.7459E+00, &
      1.490486E+00, 1.00031E+00, 6.4451E-05, 2.2707E+00, &
      1.593329E+00, 1.00028E+00, 5.4965E-05, 1.8888E+00, &
      1.703269E+00, 1.00024E+00, 4.6859E-05, 1.5819E+00, &
      1.820795E+00, 1.00022E+00, 3.9935E-05, 1.3353E+00, &
      1.946430E+00, 1.00019E+00, 3.4025E-05, 1.1373E+00, &
      2.080733E+00, 1.00017E+00, 2.8980E-05, 9.7820E-01, &
      2.224304E+00, 1.00015E+00, 2.4677E-05, 8.5057E-01, &
      2.377781E+00, 1.00013E+00, 2.1007E-05, 7.4825E-01, &
      2.541848E+00, 1.00011E+00, 1.7878E-05, 6.6634E-01, &
      2.717235E+00, 1.00010E+00, 1.5211E-05, 6.0091E-01, &
      2.904724E+00, 1.00009E+00, 1.2940E-05, 5.4880E-01, &
      3.105150E+00, 1.00008E+00, 1.1005E-05, 5.0745E-01, &
      3.319406E+00, 1.00007E+00, 9.3572E-06, 4.7482E-01, &
      3.548445E+00, 1.00006E+00, 7.9549E-06, 4.4926E-01, &
      3.793288E+00, 1.00005E+00, 6.7615E-06, 4.2941E-01, &
      4.055024E+00, 1.00005E+00, 5.7461E-06, 4.1422E-01, &
      4.334821E+00, 1.00004E+00, 4.8825E-06, 4.0277E-01, &
      4.633924E+00, 1.00004E+00, 4.1481E-06, 3.9437E-01, &
      4.953664E+00, 1.00003E+00, 3.5237E-06, 3.8842E-01, &
      5.295467E+00, 1.00003E+00, 2.9930E-06, 3.8444E-01, &
      5.660855E+00, 1.00002E+00, 2.5419E-06, 3.8202E-01, &
      6.051453E+00, 1.00002E+00, 2.1586E-06, 3.8085E-01, &
      6.469004E+00, 1.00002E+00, 1.8330E-06, 3.8065E-01, &
      6.915365E+00, 1.00002E+00, 1.5564E-06, 3.8120E-01, &
      7.392525E+00, 1.00001E+00, 1.3214E-06, 3.8228E-01, &
      7.902609E+00, 1.00001E+00, 1.1219E-06, 3.8376E-01, &
      8.447890E+00, 1.00001E+00, 9.5248E-07, 3.8548E-01, &
      9.030794E+00, 1.00001E+00, 8.0863E-07, 3.8734E-01, &
      9.653919E+00, 1.00001E+00, 6.8651E-07, 3.8924E-01, &
      1.032004E+01, 1.00001E+00, 5.8283E-07, 3.9110E-01, &
      1.103212E+01, 1.00001E+00, 4.9482E-07, 3.9284E-01, &
      1.179334E+01, 1.00001E+00, 4.2012E-07, 3.9442E-01, &
      1.260708E+01, 1.00001E+00, 3.5671E-07, 3.9577E-01, &
      1.347697E+01, 1.00000E+00, 3.0289E-07, 3.9687E-01, &
      1.440688E+01, 1.00000E+00, 2.5721E-07, 3.9768E-01, &
      1.540095E+01, 1.00000E+00, 2.1844E-07, 3.9816E-01, &
      1.646362E+01, 1.00000E+00, 1.8553E-07, 3.9831E-01, &
      1.759961E+01, 1.00000E+00, 1.5760E-07, 3.9810E-01, &
      1.881398E+01, 1.00000E+00, 1.3389E-07, 3.9755E-01, &
      2.011215E+01, 1.00000E+00, 1.1376E-07, 3.9662E-01, &
      2.149988E+01, 1.00000E+00, 9.6669E-08, 3.9532E-01, &
      2.298338E+01, 1.00000E+00, 8.2161E-08, 3.9365E-01, &
      2.456923E+01, 1.00000E+00, 6.9843E-08, 3.9162E-01, &
      2.626450E+01, 1.00000E+00, 5.9383E-08, 3.8923E-01, &
      2.807676E+01, 1.00000E+00, 5.0499E-08, 3.8651E-01, &
      3.001405E+01, 1.00000E+00, 4.2954E-08, 3.8344E-01, &
      3.208502E+01, 1.00000E+00, 3.6544E-08, 3.8006E-01, &
      3.429889E+01, 1.00000E+00, 3.1098E-08, 3.7637E-01, &
      3.666551E+01, 1.00000E+00, 2.6471E-08, 3.7239E-01, &
      3.919543E+01, 1.00000E+00, 2.2537E-08, 3.6814E-01, &
      4.189992E+01, 1.00000E+00, 1.9194E-08, 3.6364E-01, &
      4.479101E+01, 1.00000E+00, 1.6351E-08, 3.5891E-01, &
      4.788159E+01, 1.00000E+00, 1.3934E-08, 3.5396E-01, &
      5.118542E+01, 1.00000E+00, 1.1877E-08, 3.4881E-01, &
      5.471721E+01, 1.00000E+00, 1.0128E-08, 3.4350E-01, &
      5.849270E+01, 1.00000E+00, 8.6390E-09, 3.3802E-01, &
      6.252870E+01, 1.00000E+00, 7.3715E-09, 3.3240E-01, &
      6.684318E+01, 1.00000E+00, 6.2923E-09, 3.2668E-01, &
      7.145536E+01, 1.00000E+00, 5.3732E-09, 3.2085E-01, &
      7.638578E+01, 1.00000E+00, 4.5901E-09, 3.1494E-01, &
      8.165640E+01, 1.00000E+00, 3.9227E-09, 3.0898E-01, &
      8.729069E+01, 1.00000E+00, 3.3537E-09, 3.0296E-01, &
      9.331374E+01, 1.00000E+00, 2.8685E-09, 2.9691E-01, &
      9.975239E+01, 1.00000E+00, 2.4545E-09, 2.9086E-01, &
      1.066353E+02, 1.00000E+00, 2.1013E-09, 2.8480E-01, &
      1.139931E+02, 1.00000E+00, 1.7997E-09, 2.7875E-01, &
      1.218587E+02, 1.00000E+00, 1.5422E-09, 2.7274E-01, &
      1.302669E+02, 1.00000E+00, 1.3221E-09, 2.6676E-01, &
      1.392553E+02, 1.00000E+00, 1.1341E-09, 2.6083E-01, &
      1.488640E+02, 1.00000E+00, 9.7323E-10, 2.5497E-01, &
      1.591356E+02, 1.00000E+00, 8.3566E-10, 2.4918E-01, &
      1.701159E+02, 1.00000E+00, 7.1792E-10, 2.4346E-01, &
      1.818539E+02, 1.00000E+00, 6.1711E-10, 2.3784E-01, &
      1.944018E+02, 1.00000E+00, 5.3076E-10, 2.3231E-01, &
      2.078156E+02, 1.00000E+00, 4.5676E-10, 2.2689E-01, &
      2.221548E+02, 1.00000E+00, 3.9331E-10, 2.2157E-01, &
      2.374835E+02, 1.00000E+00, 3.3887E-10, 2.1637E-01, &
      2.538699E+02, 1.00000E+00, 2.9216E-10, 2.1129E-01, &
      2.713869E+02, 1.00000E+00, 2.5204E-10, 2.0633E-01, &
      2.901126E+02, 1.00000E+00, 2.1757E-10, 2.0150E-01, &
      3.101304E+02, 1.00000E+00, 1.8793E-10, 1.9680E-01, &
      3.315294E+02, 1.00000E+00, 1.6245E-10, 1.9223E-01, &
      3.544049E+02, 1.00000E+00, 1.4051E-10, 1.8779E-01, &
      3.788588E+02, 1.00000E+00, 1.2162E-10, 1.8349E-01, &
      4.050001E+02, 1.00000E+00, 1.0534E-10, 1.7933E-01, &
      4.329451E+02, 1.00000E+00, 9.1313E-11, 1.7530E-01]
   case(2)
      frel = 0.0000E+00
      allocate(nist_tab(660))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 8.09152E-01, 0.0000E+00, 1.2498E-05, &
      1.142761E-02, 7.94174E-01, 0.0000E+00, 1.7671E-05, &
      1.221612E-02, 7.76288E-01, 0.0000E+00, 2.4815E-05, &
      1.305903E-02, 7.54752E-01, 0.0000E+00, 3.4613E-05, &
      1.396010E-02, 7.28554E-01, 0.0000E+00, 4.7958E-05, &
      1.492335E-02, 6.96270E-01, 0.0000E+00, 6.6005E-05, &
      1.595306E-02, 6.55815E-01, 0.0000E+00, 9.0244E-05, &
      1.705382E-02, 6.03976E-01, 0.0000E+00, 1.2258E-04, &
      1.823053E-02, 5.35466E-01, 0.0000E+00, 1.6541E-04, &
      1.948844E-02, 4.40723E-01, 0.0000E+00, 2.2177E-04, &
      2.083314E-02, 2.99815E-01, 0.0000E+00, 2.9543E-04, &
      2.227063E-02, 5.95946E-02, 0.0000E+00, 3.9106E-04, &
      2.380730E-02, -5.37052E-01, 0.0000E+00, 5.1437E-04, &
      2.410800E-02, -8.18127E-01, 0.0000E+00, 5.4121E-04, &
      2.447700E-02, -1.66675E+00, 0.0000E+00, 5.7539E-04, &
      2.457540E-02, -2.67916E+00, 0.0000E+00, 5.8474E-04, &
      2.472300E-02, -1.63405E+00, 1.9911E+00, 8.4663E+05, &
      2.509200E-02, -7.18902E-01, 1.9648E+00, 8.2316E+05, &
      2.545001E-02, -3.46603E-01, 1.9399E+00, 8.0128E+05, &
      2.720606E-02, 4.44157E-01, 1.8248E+00, 7.0509E+05, &
      2.908327E-02, 8.35659E-01, 1.7138E+00, 6.1945E+05, &
      3.109002E-02, 1.09847E+00, 1.6069E+00, 5.4334E+05, &
      3.323523E-02, 1.29391E+00, 1.5043E+00, 4.7581E+05, &
      3.552846E-02, 1.44669E+00, 1.4060E+00, 4.1601E+05, &
      3.797993E-02, 1.56954E+00, 1.3120E+00, 3.6314E+05, &
      4.060054E-02, 1.66999E+00, 1.2223E+00, 3.1648E+05, &
      4.340198E-02, 1.75296E+00, 1.1369E+00, 2.7537E+05, &
      4.639671E-02, 1.82186E+00, 1.0558E+00, 2.3922E+05, &
      4.959809E-02, 1.87923E+00, 9.7893E-01, 2.0748E+05, &
      5.302035E-02, 1.92697E+00, 9.0618E-01, 1.7967E+05, &
      5.667876E-02, 1.96660E+00, 8.3748E-01, 1.5533E+05, &
      6.058959E-02, 1.99934E+00, 7.7275E-01, 1.3407E+05, &
      6.477028E-02, 2.02621E+00, 7.1188E-01, 1.1554E+05, &
      6.923942E-02, 2.04803E+00, 6.5476E-01, 9.9407E+04, &
      7.401695E-02, 2.06555E+00, 6.0124E-01, 8.5391E+04, &
      7.912411E-02, 2.07934E+00, 5.5122E-01, 7.3234E+04, &
      8.458368E-02, 2.08994E+00, 5.0455E-01, 6.2706E+04, &
      9.041995E-02, 2.09782E+00, 4.6109E-01, 5.3606E+04, &
      9.665893E-02, 2.10336E+00, 4.2069E-01, 4.5753E+04, &
      1.033284E-01, 2.10693E+00, 3.8322E-01, 3.8987E+04, &
      1.104581E-01, 2.10882E+00, 3.4853E-01, 3.3169E+04, &
      1.180797E-01, 2.10930E+00, 3.1646E-01, 2.8174E+04, &
      1.262272E-01, 2.10861E+00, 2.8689E-01, 2.3892E+04, &
      1.349368E-01, 2.10695E+00, 2.5966E-01, 2.0229E+04, &
      1.442475E-01, 2.10451E+00, 2.3464E-01, 1.7100E+04, &
      1.542005E-01, 2.10143E+00, 2.1169E-01, 1.4431E+04, &
      1.648404E-01, 2.09787E+00, 1.9068E-01, 1.2160E+04, &
      1.762144E-01, 2.09393E+00, 1.7148E-01, 1.0230E+04, &
      1.883732E-01, 2.08972E+00, 1.5396E-01, 8.5919E+03, &
      2.013709E-01, 2.08533E+00, 1.3801E-01, 7.2049E+03, &
      2.152655E-01, 2.08083E+00, 1.2352E-01, 6.0320E+03, &
      2.301188E-01, 2.07628E+00, 1.1037E-01, 5.0420E+03, &
      2.459970E-01, 2.07175E+00, 9.8459E-02, 4.2077E+03, &
      2.629708E-01, 2.06727E+00, 8.7695E-02, 3.5058E+03, &
      2.811158E-01, 2.06289E+00, 7.7982E-02, 2.9163E+03, &
      3.005128E-01, 2.05862E+00, 6.9233E-02, 2.4221E+03, &
      3.212482E-01, 2.05450E+00, 6.1368E-02, 2.0084E+03, &
      3.434143E-01, 2.05053E+00, 5.4308E-02, 1.6627E+03, &
      3.671099E-01, 2.04675E+00, 4.7984E-02, 1.3743E+03, &
      3.924405E-01, 2.04315E+00, 4.2328E-02, 1.1341E+03, &
      4.195189E-01, 2.03973E+00, 3.7279E-02, 9.3446E+02, &
      4.484657E-01, 2.03651E+00, 3.2779E-02, 7.6870E+02, &
      4.794098E-01, 2.03349E+00, 2.8776E-02, 6.3136E+02, &
      5.124891E-01, 2.03065E+00, 2.5222E-02, 5.1773E+02, &
      5.478508E-01, 2.02800E+00, 2.2071E-02, 4.2389E+02, &
      5.856525E-01, 2.02553E+00, 1.9283E-02, 3.4651E+02, &
      6.260625E-01, 2.02324E+00, 1.6819E-02, 2.8282E+02, &
      6.692609E-01, 2.02112E+00, 1.4647E-02, 2.3049E+02, &
      7.154399E-01, 2.01917E+00, 1.2735E-02, 1.8756E+02, &
      7.648052E-01, 2.01736E+00, 1.1055E-02, 1.5238E+02, &
      8.175768E-01, 2.01570E+00, 9.5814E-03, 1.2363E+02, &
      8.739896E-01, 2.01418E+00, 8.2906E-03, 1.0016E+02, &
      9.342948E-01, 2.01279E+00, 7.1622E-03, 8.1027E+01, &
      9.987612E-01, 2.01152E+00, 6.1775E-03, 6.5461E+01, &
      1.067676E+00, 2.01036E+00, 5.3000E-03, 5.2622E+01, &
      1.141345E+00, 2.00929E+00, 4.5425E-03, 4.2274E+01, &
      1.220098E+00, 2.00833E+00, 3.8898E-03, 3.3946E+01, &
      1.304285E+00, 2.00745E+00, 3.3280E-03, 2.7250E+01, &
      1.394281E+00, 2.00665E+00, 2.8448E-03, 2.1870E+01, &
      1.490486E+00, 2.00594E+00, 2.4298E-03, 1.7551E+01, &
      1.593329E+00, 2.00529E+00, 2.0737E-03, 1.4088E+01, &
      1.703269E+00, 2.00471E+00, 1.7684E-03, 1.1311E+01, &
      1.820795E+00, 2.00419E+00, 1.5069E-03, 9.0882E+00, &
      1.946430E+00, 2.00373E+00, 1.2832E-03, 7.3086E+00, &
      2.080733E+00, 2.00331E+00, 1.0919E-03, 5.8850E+00, &
      2.224304E+00, 2.00294E+00, 9.2851E-04, 4.7466E+00, &
      2.377781E+00, 2.00260E+00, 7.8905E-04, 3.8366E+00, &
      2.541848E+00, 2.00231E+00, 6.7012E-04, 3.1095E+00, &
      2.717235E+00, 2.00204E+00, 5.6878E-04, 2.5285E+00, &
      2.904724E+00, 2.00181E+00, 4.8249E-04, 2.0643E+00, &
      3.105150E+00, 2.00160E+00, 4.0907E-04, 1.6935E+00, &
      3.319406E+00, 2.00141E+00, 3.4664E-04, 1.3972E+00, &
      3.548445E+00, 2.00125E+00, 2.9360E-04, 1.1603E+00, &
      3.793288E+00, 2.00110E+00, 2.4855E-04, 9.7101E-01, &
      4.055024E+00, 2.00097E+00, 2.1033E-04, 8.1960E-01, &
      4.334821E+00, 2.00086E+00, 1.7791E-04, 6.9847E-01, &
      4.633924E+00, 2.00076E+00, 1.5043E-04, 6.0148E-01, &
      4.953664E+00, 2.00067E+00, 1.2714E-04, 5.2382E-01, &
      5.295467E+00, 2.00059E+00, 1.0743E-04, 4.6158E-01, &
      5.660855E+00, 2.00052E+00, 9.0740E-05, 4.1168E-01, &
      6.051453E+00, 2.00046E+00, 7.6624E-05, 3.7168E-01, &
      6.469004E+00, 2.00040E+00, 6.4687E-05, 3.3958E-01, &
      6.915365E+00, 2.00035E+00, 5.4596E-05, 3.1382E-01, &
      7.392525E+00, 2.00031E+00, 4.6070E-05, 2.9315E-01, &
      7.902609E+00, 2.00027E+00, 3.8868E-05, 2.7657E-01, &
      8.447890E+00, 2.00024E+00, 3.2787E-05, 2.6327E-01, &
      9.030794E+00, 2.00021E+00, 2.7654E-05, 2.5259E-01, &
      9.653919E+00, 2.00019E+00, 2.3322E-05, 2.4402E-01, &
      1.032004E+01, 2.00016E+00, 1.9667E-05, 2.3714E-01, &
      1.103212E+01, 2.00014E+00, 1.6584E-05, 2.3160E-01, &
      1.179334E+01, 2.00013E+00, 1.3984E-05, 2.2712E-01, &
      1.260708E+01, 2.00011E+00, 1.1791E-05, 2.2348E-01, &
      1.347697E+01, 2.00010E+00, 9.9427E-06, 2.2049E-01, &
      1.440688E+01, 2.00009E+00, 8.3843E-06, 2.1799E-01, &
      1.540095E+01, 2.00007E+00, 7.0708E-06, 2.1587E-01, &
      1.646362E+01, 2.00007E+00, 5.9636E-06, 2.1401E-01, &
      1.759961E+01, 2.00006E+00, 5.0304E-06, 2.1233E-01, &
      1.881398E+01, 2.00005E+00, 4.2439E-06, 2.1078E-01, &
      2.011215E+01, 2.00004E+00, 3.5810E-06, 2.0927E-01, &
      2.149988E+01, 2.00004E+00, 3.0223E-06, 2.0779E-01, &
      2.298338E+01, 2.00003E+00, 2.5513E-06, 2.0627E-01, &
      2.456923E+01, 2.00003E+00, 2.1543E-06, 2.0469E-01, &
      2.626450E+01, 2.00003E+00, 1.8195E-06, 2.0304E-01, &
      2.807676E+01, 2.00002E+00, 1.5373E-06, 2.0129E-01, &
      3.001405E+01, 2.00002E+00, 1.2992E-06, 1.9942E-01, &
      3.208502E+01, 2.00002E+00, 1.0984E-06, 1.9743E-01, &
      3.429889E+01, 2.00002E+00, 9.2904E-07, 1.9532E-01, &
      3.666551E+01, 2.00001E+00, 7.8609E-07, 1.9310E-01, &
      3.919543E+01, 2.00001E+00, 6.6542E-07, 1.9073E-01, &
      4.189992E+01, 2.00001E+00, 5.6354E-07, 1.8825E-01, &
      4.479101E+01, 2.00001E+00, 4.7748E-07, 1.8565E-01, &
      4.788159E+01, 2.00001E+00, 4.0478E-07, 1.8293E-01, &
      5.118542E+01, 2.00001E+00, 3.4333E-07, 1.8011E-01, &
      5.471721E+01, 2.00001E+00, 2.9137E-07, 1.7720E-01, &
      5.849270E+01, 2.00001E+00, 2.4742E-07, 1.7419E-01, &
      6.252870E+01, 2.00000E+00, 2.1023E-07, 1.7112E-01, &
      6.684318E+01, 2.00000E+00, 1.7875E-07, 1.6797E-01, &
      7.145536E+01, 2.00000E+00, 1.5208E-07, 1.6476E-01, &
      7.638578E+01, 2.00000E+00, 1.2948E-07, 1.6151E-01, &
      8.165640E+01, 2.00000E+00, 1.1032E-07, 1.5822E-01, &
      8.729069E+01, 2.00000E+00, 9.4059E-08, 1.5491E-01, &
      9.331374E+01, 2.00000E+00, 8.0260E-08, 1.5159E-01, &
      9.975239E+01, 2.00000E+00, 6.8540E-08, 1.4826E-01, &
      1.066353E+02, 2.00000E+00, 5.8579E-08, 1.4493E-01, &
      1.139931E+02, 2.00000E+00, 5.0108E-08, 1.4160E-01, &
      1.218587E+02, 2.00000E+00, 4.2900E-08, 1.3830E-01, &
      1.302669E+02, 2.00000E+00, 3.6762E-08, 1.3504E-01, &
      1.392553E+02, 2.00000E+00, 3.1531E-08, 1.3181E-01, &
      1.488640E+02, 2.00000E+00, 2.7070E-08, 1.2862E-01, &
      1.591356E+02, 2.00000E+00, 2.3263E-08, 1.2548E-01, &
      1.701159E+02, 2.00000E+00, 2.0011E-08, 1.2240E-01, &
      1.818539E+02, 2.00000E+00, 1.7232E-08, 1.1937E-01, &
      1.944018E+02, 2.00000E+00, 1.4854E-08, 1.1641E-01, &
      2.078156E+02, 2.00000E+00, 1.2818E-08, 1.1353E-01, &
      2.221548E+02, 2.00000E+00, 1.1073E-08, 1.1071E-01, &
      2.374835E+02, 2.00000E+00, 9.5771E-09, 1.0797E-01, &
      2.538699E+02, 2.00000E+00, 8.2925E-09, 1.0531E-01, &
      2.713869E+02, 2.00000E+00, 7.1887E-09, 1.0273E-01, &
      2.901126E+02, 2.00000E+00, 6.2394E-09, 1.0023E-01, &
      3.101304E+02, 2.00000E+00, 5.4221E-09, 9.7822E-02, &
      3.315294E+02, 2.00000E+00, 4.7179E-09, 9.5498E-02, &
      3.544049E+02, 2.00000E+00, 4.1103E-09, 9.3262E-02, &
      3.788588E+02, 2.00000E+00, 3.5857E-09, 9.1115E-02, &
      4.050001E+02, 2.00000E+00, 3.1323E-09, 8.9056E-02, &
      4.329451E+02, 2.00000E+00, 2.7399E-09, 8.7088E-02]
   case(3)
      frel = -6.0000E-04
      allocate(nist_tab(680))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.233200E-03, 9.08733E-01, 0.0000E+00, 2.3914E-07, &
      5.313300E-03, 8.59283E-01, 0.0000E+00, 2.5404E-07, &
      5.334660E-03, 8.03599E-01, 0.0000E+00, 2.5813E-07, &
      5.366700E-03, 8.56971E-01, 1.0769E-01, 1.2165E+05, &
      5.446800E-03, 9.01968E-01, 1.0987E-01, 1.2230E+05, &
      1.069000E-02, 1.07660E+00, 1.7745E-01, 1.0064E+05, &
      1.142761E-02, 1.08752E+00, 1.7721E-01, 9.4014E+04, &
      1.221612E-02, 1.09737E+00, 1.7581E-01, 8.7249E+04, &
      1.305903E-02, 1.10595E+00, 1.7335E-01, 8.0477E+04, &
      1.396010E-02, 1.11307E+00, 1.6997E-01, 7.3813E+04, &
      1.492335E-02, 1.11854E+00, 1.6578E-01, 6.7348E+04, &
      1.595306E-02, 1.12215E+00, 1.6092E-01, 6.1155E+04, &
      1.705382E-02, 1.12376E+00, 1.5551E-01, 5.5284E+04, &
      1.823053E-02, 1.12305E+00, 1.4967E-01, 4.9772E+04, &
      1.948844E-02, 1.11980E+00, 1.4350E-01, 4.4641E+04, &
      2.083314E-02, 1.11370E+00, 1.3711E-01, 3.9900E+04, &
      2.227063E-02, 1.10441E+00, 1.3057E-01, 3.5543E+04, &
      2.380730E-02, 1.09140E+00, 1.2391E-01, 3.1553E+04, &
      2.545001E-02, 1.07404E+00, 1.1721E-01, 2.7922E+04, &
      2.720606E-02, 1.05151E+00, 1.1056E-01, 2.4637E+04, &
      2.908327E-02, 1.02273E+00, 1.0401E-01, 2.1683E+04, &
      3.109002E-02, 9.86239E-01, 9.7625E-02, 1.9037E+04, &
      3.323523E-02, 9.39944E-01, 9.1431E-02, 1.6678E+04, &
      3.552846E-02, 8.80803E-01, 8.5462E-02, 1.4583E+04, &
      3.797993E-02, 8.04138E-01, 7.9741E-02, 1.2729E+04, &
      4.060054E-02, 7.02294E-01, 7.4280E-02, 1.1092E+04, &
      4.340198E-02, 5.61559E-01, 6.9091E-02, 9.6509E+03, &
      4.639671E-02, 3.53730E-01, 6.4175E-02, 8.3856E+03, &
      4.959809E-02, 4.88984E-03, 5.9534E-02, 7.2770E+03, &
      5.302035E-02, -8.35477E-01, 5.5164E-02, 6.3076E+03, &
      5.365500E-02, -1.19807E+00, 5.4412E-02, 6.1482E+03, &
      5.447625E-02, -2.31019E+00, 5.3466E-02, 5.9501E+03, &
      5.469525E-02, -3.60723E+00, 5.3218E-02, 5.8988E+03, &
      5.502375E-02, -2.31071E+00, 2.5879E+00, 2.8513E+05, &
      5.584500E-02, -1.18751E+00, 2.5985E+00, 2.8210E+05, &
      5.667876E-02, -7.21050E-01, 2.6092E+00, 2.7909E+05, &
      6.058959E-02, 2.50867E-01, 2.6581E+00, 2.6597E+05, &
      6.477028E-02, 8.17156E-01, 2.6783E+00, 2.5069E+05, &
      6.923942E-02, 1.26129E+00, 2.6515E+00, 2.3217E+05, &
      7.401695E-02, 1.63438E+00, 2.5848E+00, 2.1171E+05, &
      7.912411E-02, 1.95333E+00, 2.4858E+00, 1.9047E+05, &
      8.458368E-02, 2.22569E+00, 2.3627E+00, 1.6935E+05, &
      9.041995E-02, 2.45620E+00, 2.2228E+00, 1.4904E+05, &
      9.665893E-02, 2.64887E+00, 2.0728E+00, 1.3001E+05, &
      1.033284E-01, 2.80757E+00, 1.9181E+00, 1.1254E+05, &
      1.104581E-01, 2.93608E+00, 1.7633E+00, 9.6781E+04, &
      1.180797E-01, 3.03823E+00, 1.6120E+00, 8.2763E+04, &
      1.262272E-01, 3.11772E+00, 1.4666E+00, 7.0439E+04, &
      1.349368E-01, 3.17802E+00, 1.3290E+00, 5.9708E+04, &
      1.442475E-01, 3.22227E+00, 1.2001E+00, 5.0439E+04, &
      1.542005E-01, 3.25329E+00, 1.0806E+00, 4.2485E+04, &
      1.648404E-01, 3.27358E+00, 9.7059E-01, 3.5697E+04, &
      1.762144E-01, 3.28517E+00, 8.6995E-01, 2.9930E+04, &
      1.883732E-01, 3.28990E+00, 7.7835E-01, 2.5050E+04, &
      2.013709E-01, 3.28929E+00, 6.9532E-01, 2.0934E+04, &
      2.152655E-01, 3.28462E+00, 6.2031E-01, 1.7470E+04, &
      2.301188E-01, 3.27691E+00, 5.5269E-01, 1.4561E+04, &
      2.459970E-01, 3.26695E+00, 4.9180E-01, 1.2120E+04, &
      2.629708E-01, 3.25537E+00, 4.3709E-01, 1.0077E+04, &
      2.811158E-01, 3.24269E+00, 3.8805E-01, 8.3688E+03, &
      3.005128E-01, 3.22935E+00, 3.4415E-01, 6.9431E+03, &
      3.212482E-01, 3.21568E+00, 3.0493E-01, 5.7548E+03, &
      3.434143E-01, 3.20197E+00, 2.6993E-01, 4.7655E+03, &
      3.671099E-01, 3.18842E+00, 2.3873E-01, 3.9426E+03, &
      3.924405E-01, 3.17518E+00, 2.1095E-01, 3.2590E+03, &
      4.195189E-01, 3.16239E+00, 1.8625E-01, 2.6917E+03, &
      4.484657E-01, 3.15011E+00, 1.6430E-01, 2.2212E+03, &
      4.794098E-01, 3.13842E+00, 1.4482E-01, 1.8315E+03, &
      5.124891E-01, 3.12734E+00, 1.2754E-01, 1.5091E+03, &
      5.478508E-01, 3.11690E+00, 1.1224E-01, 1.2424E+03, &
      5.856525E-01, 3.10711E+00, 9.8699E-02, 1.0220E+03, &
      6.260625E-01, 3.09796E+00, 8.6728E-02, 8.4013E+02, &
      6.692609E-01, 3.08945E+00, 7.6155E-02, 6.9017E+02, &
      7.154399E-01, 3.08155E+00, 6.6826E-02, 5.6660E+02, &
      7.648052E-01, 3.07424E+00, 5.8603E-02, 4.6488E+02, &
      8.175768E-01, 3.06751E+00, 5.1362E-02, 3.8120E+02, &
      8.739896E-01, 3.06132E+00, 4.4990E-02, 3.1243E+02, &
      9.342948E-01, 3.05566E+00, 3.9390E-02, 2.5596E+02, &
      9.987612E-01, 3.05049E+00, 3.4471E-02, 2.0960E+02, &
      1.067676E+00, 3.04576E+00, 2.9934E-02, 1.7034E+02, &
      1.141345E+00, 3.04136E+00, 2.5916E-02, 1.3803E+02, &
      1.220098E+00, 3.03729E+00, 2.2438E-02, 1.1186E+02, &
      1.304285E+00, 3.03355E+00, 1.9426E-02, 9.0673E+01, &
      1.394281E+00, 3.03015E+00, 1.6819E-02, 7.3508E+01, &
      1.490486E+00, 3.02705E+00, 1.4561E-02, 5.9606E+01, &
      1.593329E+00, 3.02425E+00, 1.2607E-02, 4.8344E+01, &
      1.703269E+00, 3.02172E+00, 1.0915E-02, 3.9224E+01, &
      1.820795E+00, 3.01943E+00, 9.4497E-03, 3.1835E+01, &
      1.946430E+00, 3.01738E+00, 8.1814E-03, 2.5850E+01, &
      2.080733E+00, 3.01553E+00, 7.0833E-03, 2.1000E+01, &
      2.224304E+00, 3.01387E+00, 6.1326E-03, 1.7072E+01, &
      2.377781E+00, 3.01238E+00, 5.3094E-03, 1.3888E+01, &
      2.541848E+00, 3.01105E+00, 4.5965E-03, 1.1307E+01, &
      2.717235E+00, 3.00986E+00, 3.9792E-03, 9.2148E+00, &
      2.904724E+00, 3.00881E+00, 3.4448E-03, 7.5187E+00, &
      3.105150E+00, 3.00786E+00, 2.9684E-03, 6.1166E+00, &
      3.319406E+00, 3.00699E+00, 2.5468E-03, 4.9646E+00, &
      3.548445E+00, 3.00621E+00, 2.1851E-03, 4.0382E+00, &
      3.793288E+00, 3.00552E+00, 1.8748E-03, 3.2930E+00, &
      4.055024E+00, 3.00490E+00, 1.6085E-03, 2.6932E+00, &
      4.334821E+00, 3.00435E+00, 1.3801E-03, 2.2104E+00, &
      4.633924E+00, 3.00385E+00, 1.1841E-03, 1.8214E+00, &
      4.953664E+00, 3.00341E+00, 1.0159E-03, 1.5078E+00, &
      5.295467E+00, 3.00302E+00, 8.7162E-04, 1.2547E+00, &
      5.660855E+00, 3.00268E+00, 7.4783E-04, 1.0504E+00, &
      6.051453E+00, 3.00237E+00, 6.4162E-04, 8.8529E-01, &
      6.469004E+00, 3.00210E+00, 5.5050E-04, 7.5169E-01, &
      6.915365E+00, 3.00186E+00, 4.7232E-04, 6.4348E-01, &
      7.392525E+00, 3.00164E+00, 4.0524E-04, 5.5572E-01, &
      7.902609E+00, 3.00145E+00, 3.4768E-04, 4.8445E-01, &
      8.447890E+00, 3.00128E+00, 2.9831E-04, 4.2649E-01, &
      9.030794E+00, 3.00113E+00, 2.5574E-04, 3.7913E-01, &
      9.653919E+00, 3.00100E+00, 2.1863E-04, 3.4014E-01, &
      1.032004E+01, 3.00088E+00, 1.8691E-04, 3.0837E-01, &
      1.103212E+01, 3.00078E+00, 1.5979E-04, 2.8242E-01, &
      1.179334E+01, 3.00069E+00, 1.3661E-04, 2.6120E-01, &
      1.260708E+01, 3.00061E+00, 1.1679E-04, 2.4377E-01, &
      1.347697E+01, 3.00053E+00, 9.9841E-05, 2.2944E-01, &
      1.440688E+01, 3.00047E+00, 8.5355E-05, 2.1761E-01, &
      1.540095E+01, 3.00042E+00, 7.2971E-05, 2.0780E-01, &
      1.646362E+01, 3.00037E+00, 6.2383E-05, 1.9965E-01, &
      1.759961E+01, 3.00032E+00, 5.3332E-05, 1.9282E-01, &
      1.881398E+01, 3.00028E+00, 4.5594E-05, 1.8709E-01, &
      2.011215E+01, 3.00025E+00, 3.8979E-05, 1.8223E-01, &
      2.149988E+01, 3.00022E+00, 3.3323E-05, 1.7808E-01, &
      2.298338E+01, 3.00019E+00, 2.8488E-05, 1.7449E-01, &
      2.456923E+01, 3.00017E+00, 2.4355E-05, 1.7138E-01, &
      2.626450E+01, 3.00015E+00, 2.0857E-05, 1.6863E-01, &
      2.807676E+01, 3.00013E+00, 1.7837E-05, 1.6617E-01, &
      3.001405E+01, 3.00012E+00, 1.5245E-05, 1.6394E-01, &
      3.208502E+01, 3.00010E+00, 1.3030E-05, 1.6187E-01, &
      3.429889E+01, 3.00009E+00, 1.1137E-05, 1.5994E-01, &
      3.666551E+01, 3.00008E+00, 9.5189E-06, 1.5810E-01, &
      3.919543E+01, 3.00007E+00, 8.1358E-06, 1.5633E-01, &
      4.189992E+01, 3.00006E+00, 6.9538E-06, 1.5461E-01, &
      4.479101E+01, 3.00005E+00, 5.9434E-06, 1.5289E-01, &
      4.788159E+01, 3.00005E+00, 5.0799E-06, 1.5119E-01, &
      5.118542E+01, 3.00004E+00, 4.3418E-06, 1.4948E-01, &
      5.471721E+01, 3.00004E+00, 3.7110E-06, 1.4775E-01, &
      5.849270E+01, 3.00003E+00, 3.1718E-06, 1.4599E-01, &
      6.252870E+01, 3.00003E+00, 2.7110E-06, 1.4420E-01, &
      6.684318E+01, 3.00002E+00, 2.3171E-06, 1.4237E-01, &
      7.145536E+01, 3.00002E+00, 1.9805E-06, 1.4050E-01, &
      7.638578E+01, 3.00002E+00, 1.6927E-06, 1.3858E-01, &
      8.165640E+01, 3.00002E+00, 1.4468E-06, 1.3663E-01, &
      8.729069E+01, 3.00001E+00, 1.2366E-06, 1.3463E-01, &
      9.331374E+01, 3.00001E+00, 1.0569E-06, 1.3258E-01, &
      9.975239E+01, 3.00001E+00, 9.0339E-07, 1.3048E-01, &
      1.066353E+02, 3.00001E+00, 7.7214E-07, 1.2836E-01, &
      1.139931E+02, 3.00001E+00, 6.5996E-07, 1.2620E-01, &
      1.218587E+02, 3.00001E+00, 5.6408E-07, 1.2399E-01, &
      1.302669E+02, 3.00001E+00, 4.8213E-07, 1.2175E-01, &
      1.392553E+02, 3.00001E+00, 4.1209E-07, 1.1949E-01, &
      1.488640E+02, 3.00001E+00, 3.5222E-07, 1.1719E-01, &
      1.591356E+02, 3.00000E+00, 3.0105E-07, 1.1488E-01, &
      1.701159E+02, 3.00000E+00, 2.5732E-07, 1.1255E-01, &
      1.818539E+02, 3.00000E+00, 2.1993E-07, 1.1020E-01, &
      1.944018E+02, 3.00000E+00, 1.8798E-07, 1.0784E-01, &
      2.078156E+02, 3.00000E+00, 1.6067E-07, 1.0547E-01, &
      2.221548E+02, 3.00000E+00, 1.3733E-07, 1.0310E-01, &
      2.374835E+02, 3.00000E+00, 1.1738E-07, 1.0073E-01, &
      2.538699E+02, 3.00000E+00, 1.0033E-07, 9.8360E-02, &
      2.713869E+02, 3.00000E+00, 8.5755E-08, 9.5998E-02, &
      2.901126E+02, 3.00000E+00, 7.3297E-08, 9.3646E-02, &
      3.101304E+02, 3.00000E+00, 6.2649E-08, 9.1305E-02, &
      3.315294E+02, 3.00000E+00, 5.3548E-08, 8.8980E-02, &
      3.544049E+02, 3.00000E+00, 4.5770E-08, 8.6674E-02, &
      3.788588E+02, 3.00000E+00, 3.9121E-08, 8.4390E-02, &
      4.050001E+02, 3.00000E+00, 3.3438E-08, 8.2128E-02, &
      4.329451E+02, 3.00000E+00, 2.8581E-08, 7.9892E-02]
   case(4)
      frel = -6.0000E-04
      allocate(nist_tab(676))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      8.251600E-03, 1.55864E+00, 0.0000E+00, 3.7164E-05, &
      8.377900E-03, 1.50947E+00, 0.0000E+00, 3.9696E-05, &
      8.411580E-03, 1.45794E+00, 0.0000E+00, 4.0392E-05, &
      8.462100E-03, 1.50266E+00, 9.7318E-02, 5.3699E+04, &
      8.588400E-03, 1.53756E+00, 1.0286E-01, 5.5921E+04, &
      1.069000E-02, 1.58480E+00, 2.1247E-01, 9.2803E+04, &
      1.142761E-02, 1.60258E+00, 2.4694E-01, 1.0090E+05, &
      1.221612E-02, 1.62546E+00, 2.7870E-01, 1.0653E+05, &
      1.305903E-02, 1.65269E+00, 3.0660E-01, 1.0962E+05, &
      1.396010E-02, 1.68318E+00, 3.2988E-01, 1.1034E+05, &
      1.492335E-02, 1.71575E+00, 3.4820E-01, 1.0895E+05, &
      1.595306E-02, 1.74924E+00, 3.6151E-01, 1.0581E+05, &
      1.705382E-02, 1.78263E+00, 3.7006E-01, 1.0132E+05, &
      1.823053E-02, 1.81506E+00, 3.7426E-01, 9.5856E+04, &
      1.948844E-02, 1.84585E+00, 3.7463E-01, 8.9757E+04, &
      2.083314E-02, 1.87451E+00, 3.7173E-01, 8.3316E+04, &
      2.227063E-02, 1.90068E+00, 3.6615E-01, 7.6767E+04, &
      2.380730E-02, 1.92412E+00, 3.5841E-01, 7.0294E+04, &
      2.545001E-02, 1.94468E+00, 3.4900E-01, 6.4030E+04, &
      2.720606E-02, 1.96230E+00, 3.3833E-01, 5.8066E+04, &
      2.908327E-02, 1.97692E+00, 3.2677E-01, 5.2462E+04, &
      3.109002E-02, 1.98850E+00, 3.1461E-01, 4.7249E+04, &
      3.323523E-02, 1.99700E+00, 3.0208E-01, 4.2439E+04, &
      3.552846E-02, 2.00234E+00, 2.8933E-01, 3.8024E+04, &
      3.797993E-02, 2.00432E+00, 2.7642E-01, 3.3983E+04, &
      4.060054E-02, 2.00267E+00, 2.6351E-01, 3.0305E+04, &
      4.340198E-02, 1.99706E+00, 2.5070E-01, 2.6970E+04, &
      4.639671E-02, 1.98704E+00, 2.3807E-01, 2.3959E+04, &
      4.959809E-02, 1.97202E+00, 2.2569E-01, 2.1247E+04, &
      5.302035E-02, 1.95120E+00, 2.1360E-01, 1.8811E+04, &
      5.667876E-02, 1.92346E+00, 2.0184E-01, 1.6628E+04, &
      6.058959E-02, 1.88729E+00, 1.9044E-01, 1.4676E+04, &
      6.477028E-02, 1.84054E+00, 1.7941E-01, 1.2933E+04, &
      6.923942E-02, 1.78009E+00, 1.6876E-01, 1.1381E+04, &
      7.401695E-02, 1.70124E+00, 1.5851E-01, 9.9991E+03, &
      7.912411E-02, 1.59647E+00, 1.4865E-01, 8.7723E+03, &
      8.458368E-02, 1.45290E+00, 1.3921E-01, 7.6848E+03, &
      9.041995E-02, 1.24579E+00, 1.3017E-01, 6.7221E+03, &
      9.665893E-02, 9.18719E-01, 1.2155E-01, 5.8716E+03, &
      1.033284E-01, 2.91953E-01, 1.1333E-01, 5.1215E+03, &
      1.087800E-01, -1.10059E+00, 1.0728E-01, 4.6052E+03, &
      1.104581E-01, -2.81138E+00, 1.0553E-01, 4.4611E+03, &
      1.108890E-01, -4.79732E+00, 1.0509E-01, 4.4251E+03, &
      1.115550E-01, -2.70425E+00, 4.0732E+00, 1.7049E+05, &
      1.132200E-01, -8.67705E-01, 4.0070E+00, 1.6525E+05, &
      1.180797E-01, 7.43908E-01, 3.8249E+00, 1.5125E+05, &
      1.262272E-01, 1.85031E+00, 3.5508E+00, 1.3135E+05, &
      1.349368E-01, 2.50334E+00, 3.2850E+00, 1.1367E+05, &
      1.442475E-01, 2.95829E+00, 3.0279E+00, 9.8012E+04, &
      1.542005E-01, 3.29619E+00, 2.7813E+00, 8.4219E+04, &
      1.648404E-01, 3.55465E+00, 2.5461E+00, 7.2122E+04, &
      1.762144E-01, 3.75464E+00, 2.3237E+00, 6.1572E+04, &
      1.883732E-01, 3.90990E+00, 2.1149E+00, 5.2422E+04, &
      2.013709E-01, 4.03003E+00, 1.9199E+00, 4.4519E+04, &
      2.152655E-01, 4.12212E+00, 1.7389E+00, 3.7718E+04, &
      2.301188E-01, 4.19162E+00, 1.5715E+00, 3.1887E+04, &
      2.459970E-01, 4.24285E+00, 1.4173E+00, 2.6902E+04, &
      2.629708E-01, 4.27928E+00, 1.2758E+00, 2.2654E+04, &
      2.811158E-01, 4.30376E+00, 1.1464E+00, 1.9042E+04, &
      3.005128E-01, 4.31861E+00, 1.0285E+00, 1.5980E+04, &
      3.212482E-01, 4.32576E+00, 9.2117E-01, 1.3389E+04, &
      3.434143E-01, 4.32684E+00, 8.2386E-01, 1.1202E+04, &
      3.671099E-01, 4.32314E+00, 7.3579E-01, 9.3589E+03, &
      3.924405E-01, 4.31578E+00, 6.5626E-01, 7.8086E+03, &
      4.195189E-01, 4.30567E+00, 5.8459E-01, 6.5069E+03, &
      4.484657E-01, 4.29356E+00, 5.2011E-01, 5.4156E+03, &
      4.794098E-01, 4.28005E+00, 4.6212E-01, 4.5012E+03, &
      5.124891E-01, 4.26557E+00, 4.1004E-01, 3.7364E+03, &
      5.478508E-01, 4.25050E+00, 3.6339E-01, 3.0976E+03, &
      5.856525E-01, 4.23517E+00, 3.2168E-01, 2.5652E+03, &
      6.260625E-01, 4.21985E+00, 2.8447E-01, 2.1221E+03, &
      6.692609E-01, 4.20476E+00, 2.5134E-01, 1.7540E+03, &
      7.154399E-01, 4.19008E+00, 2.2187E-01, 1.4486E+03, &
      7.648052E-01, 4.17592E+00, 1.9572E-01, 1.1954E+03, &
      8.175768E-01, 4.16240E+00, 1.7254E-01, 9.8589E+02, &
      8.739896E-01, 4.14957E+00, 1.5201E-01, 8.1263E+02, &
      9.342948E-01, 4.13749E+00, 1.3385E-01, 6.6949E+02, &
      9.987612E-01, 4.12618E+00, 1.1782E-01, 5.5134E+02, &
      1.067676E+00, 4.11586E+00, 1.0297E-01, 4.5084E+02, &
      1.141345E+00, 4.10588E+00, 8.9977E-02, 3.6863E+02, &
      1.220098E+00, 4.09645E+00, 7.8625E-02, 3.0143E+02, &
      1.304285E+00, 4.08767E+00, 6.8705E-02, 2.4648E+02, &
      1.394281E+00, 4.07955E+00, 6.0037E-02, 2.0158E+02, &
      1.490486E+00, 4.07207E+00, 5.2463E-02, 1.6486E+02, &
      1.593329E+00, 4.06521E+00, 4.5844E-02, 1.3485E+02, &
      1.703269E+00, 4.05894E+00, 4.0060E-02, 1.1032E+02, &
      1.820795E+00, 4.05324E+00, 3.5006E-02, 9.0256E+01, &
      1.946430E+00, 4.04805E+00, 3.0589E-02, 7.3857E+01, &
      2.080733E+00, 4.04336E+00, 2.6730E-02, 6.0449E+01, &
      2.224304E+00, 4.03909E+00, 2.3204E-02, 4.9165E+01, &
      2.377781E+00, 4.03518E+00, 2.0139E-02, 3.9991E+01, &
      2.541848E+00, 4.03162E+00, 1.7480E-02, 3.2541E+01, &
      2.717235E+00, 4.02839E+00, 1.5171E-02, 2.6489E+01, &
      2.904724E+00, 4.02551E+00, 1.3168E-02, 2.1574E+01, &
      3.105150E+00, 4.02306E+00, 1.1388E-02, 1.7518E+01, &
      3.319406E+00, 4.02063E+00, 9.8151E-03, 1.4188E+01, &
      3.548445E+00, 4.01844E+00, 8.4596E-03, 1.1502E+01, &
      3.793288E+00, 4.01646E+00, 7.2914E-03, 9.3328E+00, &
      4.055024E+00, 4.01468E+00, 6.2835E-03, 7.5809E+00, &
      4.334821E+00, 4.01308E+00, 5.4148E-03, 6.1665E+00, &
      4.633924E+00, 4.01165E+00, 4.6662E-03, 5.0244E+00, &
      4.953664E+00, 4.01037E+00, 4.0211E-03, 4.1019E+00, &
      5.295467E+00, 4.00923E+00, 3.4651E-03, 3.3565E+00, &
      5.660855E+00, 4.00820E+00, 2.9861E-03, 2.7540E+00, &
      6.051453E+00, 4.00729E+00, 2.5732E-03, 2.2668E+00, &
      6.469004E+00, 4.00647E+00, 2.2175E-03, 1.8727E+00, &
      6.915365E+00, 4.00574E+00, 1.9109E-03, 1.5537E+00, &
      7.392525E+00, 4.00510E+00, 1.6467E-03, 1.2952E+00, &
      7.902609E+00, 4.00452E+00, 1.4191E-03, 1.0858E+00, &
      8.447890E+00, 4.00401E+00, 1.2229E-03, 9.1596E-01, &
      9.030794E+00, 4.00356E+00, 1.0525E-03, 7.7740E-01, &
      9.653919E+00, 4.00316E+00, 9.0177E-04, 6.6303E-01, &
      1.032004E+01, 4.00280E+00, 7.7264E-04, 5.7052E-01, &
      1.103212E+01, 4.00248E+00, 6.6200E-04, 4.9562E-01, &
      1.179334E+01, 4.00219E+00, 5.6721E-04, 4.3489E-01, &
      1.260708E+01, 4.00194E+00, 4.8599E-04, 3.8559E-01, &
      1.347697E+01, 4.00171E+00, 4.1640E-04, 3.4549E-01, &
      1.440688E+01, 4.00151E+00, 3.5677E-04, 3.1280E-01, &
      1.540095E+01, 4.00134E+00, 3.0569E-04, 2.8611E-01, &
      1.646362E+01, 4.00118E+00, 2.6192E-04, 2.6425E-01, &
      1.759961E+01, 4.00104E+00, 2.2441E-04, 2.4631E-01, &
      1.881398E+01, 4.00092E+00, 1.9228E-04, 2.3151E-01, &
      2.011215E+01, 4.00081E+00, 1.6475E-04, 2.1927E-01, &
      2.149988E+01, 4.00072E+00, 1.4116E-04, 2.0910E-01, &
      2.298338E+01, 4.00063E+00, 1.2094E-04, 2.0059E-01, &
      2.456923E+01, 4.00056E+00, 1.0363E-04, 1.9342E-01, &
      2.626450E+01, 4.00049E+00, 8.8788E-05, 1.8736E-01, &
      2.807676E+01, 4.00043E+00, 7.5940E-05, 1.8214E-01, &
      3.001405E+01, 4.00038E+00, 6.4915E-05, 1.7762E-01, &
      3.208502E+01, 4.00034E+00, 5.5490E-05, 1.7369E-01, &
      3.429889E+01, 4.00030E+00, 4.7433E-05, 1.7020E-01, &
      3.666551E+01, 4.00026E+00, 4.0546E-05, 1.6708E-01, &
      3.919543E+01, 4.00023E+00, 3.4660E-05, 1.6424E-01, &
      4.189992E+01, 4.00020E+00, 2.9628E-05, 1.6163E-01, &
      4.479101E+01, 4.00018E+00, 2.5326E-05, 1.5918E-01, &
      4.788159E+01, 4.00016E+00, 2.1649E-05, 1.5686E-01, &
      5.118542E+01, 4.00014E+00, 1.8512E-05, 1.5464E-01, &
      5.471721E+01, 4.00012E+00, 1.5845E-05, 1.5248E-01, &
      5.849270E+01, 4.00011E+00, 1.3563E-05, 1.5036E-01, &
      6.252870E+01, 4.00009E+00, 1.1610E-05, 1.4828E-01, &
      6.684318E+01, 4.00008E+00, 9.9374E-06, 1.4619E-01, &
      7.145536E+01, 4.00007E+00, 8.5061E-06, 1.4413E-01, &
      7.638578E+01, 4.00006E+00, 7.2810E-06, 1.4204E-01, &
      8.165640E+01, 4.00006E+00, 6.2323E-06, 1.3994E-01, &
      8.729069E+01, 4.00005E+00, 5.3347E-06, 1.3782E-01, &
      9.331374E+01, 4.00004E+00, 4.5664E-06, 1.3567E-01, &
      9.975239E+01, 4.00004E+00, 3.9087E-06, 1.3350E-01, &
      1.066353E+02, 4.00003E+00, 3.3457E-06, 1.3131E-01, &
      1.139931E+02, 4.00003E+00, 2.8639E-06, 1.2908E-01, &
      1.218587E+02, 4.00003E+00, 2.4514E-06, 1.2683E-01, &
      1.302669E+02, 4.00002E+00, 2.0984E-06, 1.2456E-01, &
      1.392553E+02, 4.00002E+00, 1.7962E-06, 1.2226E-01, &
      1.488640E+02, 4.00002E+00, 1.5375E-06, 1.1994E-01, &
      1.591356E+02, 4.00002E+00, 1.3161E-06, 1.1760E-01, &
      1.701159E+02, 4.00001E+00, 1.1265E-06, 1.1525E-01, &
      1.818539E+02, 4.00001E+00, 9.6428E-07, 1.1288E-01, &
      1.944018E+02, 4.00001E+00, 8.2541E-07, 1.1050E-01, &
      2.078156E+02, 4.00001E+00, 7.0654E-07, 1.0812E-01, &
      2.221548E+02, 4.00001E+00, 6.0479E-07, 1.0572E-01, &
      2.374835E+02, 4.00001E+00, 5.1770E-07, 1.0333E-01, &
      2.538699E+02, 4.00001E+00, 4.4314E-07, 1.0094E-01, &
      2.713869E+02, 4.00001E+00, 3.7933E-07, 9.8550E-02, &
      2.901126E+02, 4.00000E+00, 3.2470E-07, 9.6169E-02, &
      3.101304E+02, 4.00000E+00, 2.7794E-07, 9.3799E-02, &
      3.315294E+02, 4.00000E+00, 2.3792E-07, 9.1442E-02, &
      3.544049E+02, 4.00000E+00, 2.0366E-07, 8.9101E-02, &
      3.788588E+02, 4.00000E+00, 1.7433E-07, 8.6778E-02, &
      4.050001E+02, 4.00000E+00, 1.4923E-07, 8.4476E-02, &
      4.329451E+02, 4.00000E+00, 1.2774E-07, 8.2196E-02]
   case(5)
      frel = -1.2000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.606000E-03, 8.03768E-01, 0.0000E+00, 6.1552E-05, &
      4.676500E-03, 3.03582E-01, 0.0000E+00, 6.5531E-05, &
      4.695300E-03, -2.92582E-01, 0.0000E+00, 6.6621E-05, &
      4.723500E-03, 3.23173E-01, 1.1721E+00, 9.6591E+05, &
      4.794000E-03, 8.61328E-01, 1.1548E+00, 9.3769E+05, &
      1.069000E-02, 2.36087E+00, 6.3908E-01, 2.3272E+05, &
      1.142761E-02, 2.37363E+00, 6.0622E-01, 2.0650E+05, &
      1.221612E-02, 2.37133E+00, 5.7327E-01, 1.8267E+05, &
      1.305903E-02, 2.33137E+00, 5.4038E-01, 1.6108E+05, &
      1.320060E-02, 2.31282E+00, 5.3508E-01, 1.5779E+05, &
      1.340265E-02, 2.25109E+00, 5.2763E-01, 1.5325E+05, &
      1.345653E-02, 2.18102E+00, 5.2567E-01, 1.5207E+05, &
      1.353735E-02, 2.24636E+00, 6.5844E-01, 1.8934E+05, &
      1.373940E-02, 2.30171E+00, 6.5819E-01, 1.8648E+05, &
      1.396010E-02, 2.32499E+00, 6.5834E-01, 1.8358E+05, &
      1.492335E-02, 2.36971E+00, 6.6435E-01, 1.7329E+05, &
      1.595306E-02, 2.40102E+00, 6.7606E-01, 1.6497E+05, &
      1.705382E-02, 2.43266E+00, 6.8985E-01, 1.5746E+05, &
      1.823053E-02, 2.46722E+00, 7.0420E-01, 1.5037E+05, &
      1.948844E-02, 2.50542E+00, 7.1767E-01, 1.4335E+05, &
      2.083314E-02, 2.54708E+00, 7.2892E-01, 1.3620E+05, &
      2.227063E-02, 2.59154E+00, 7.3700E-01, 1.2882E+05, &
      2.380730E-02, 2.63793E+00, 7.4132E-01, 1.2121E+05, &
      2.545001E-02, 2.68534E+00, 7.4153E-01, 1.1342E+05, &
      2.720606E-02, 2.73282E+00, 7.3754E-01, 1.0553E+05, &
      2.908327E-02, 2.77952E+00, 7.2948E-01, 9.7639E+04, &
      3.109002E-02, 2.82470E+00, 7.1764E-01, 8.9854E+04, &
      3.323523E-02, 2.86772E+00, 7.0241E-01, 8.2270E+04, &
      3.552846E-02, 2.90809E+00, 6.8424E-01, 7.4970E+04, &
      3.797993E-02, 2.94544E+00, 6.6364E-01, 6.8019E+04, &
      4.060054E-02, 2.97932E+00, 6.4107E-01, 6.1464E+04, &
      4.340198E-02, 3.00989E+00, 6.1699E-01, 5.5338E+04, &
      4.639671E-02, 3.03686E+00, 5.9184E-01, 4.9655E+04, &
      4.959809E-02, 3.06015E+00, 5.6597E-01, 4.4421E+04, &
      5.302035E-02, 3.07973E+00, 5.3974E-01, 3.9627E+04, &
      5.667876E-02, 3.09557E+00, 5.1332E-01, 3.5255E+04, &
      6.058959E-02, 3.10748E+00, 4.8680E-01, 3.1276E+04, &
      6.477028E-02, 3.11523E+00, 4.6045E-01, 2.7673E+04, &
      6.923942E-02, 3.11858E+00, 4.3449E-01, 2.4428E+04, &
      7.401695E-02, 3.11726E+00, 4.0910E-01, 2.1515E+04, &
      7.912411E-02, 3.11091E+00, 3.8440E-01, 1.8912E+04, &
      8.458368E-02, 3.09899E+00, 3.6052E-01, 1.6592E+04, &
      9.041995E-02, 3.08080E+00, 3.3727E-01, 1.4520E+04, &
      9.665893E-02, 3.05508E+00, 3.1461E-01, 1.2670E+04, &
      1.033284E-01, 3.02038E+00, 2.9304E-01, 1.1040E+04, &
      1.104581E-01, 2.97471E+00, 2.7258E-01, 9.6063E+03, &
      1.180797E-01, 2.91504E+00, 2.5321E-01, 8.3478E+03, &
      1.262272E-01, 2.83673E+00, 2.3492E-01, 7.2451E+03, &
      1.349368E-01, 2.73225E+00, 2.1770E-01, 6.2807E+03, &
      1.442475E-01, 2.58847E+00, 2.0152E-01, 5.4387E+03, &
      1.542005E-01, 2.37982E+00, 1.8634E-01, 4.7045E+03, &
      1.648404E-01, 2.04643E+00, 1.7213E-01, 4.0653E+03, &
      1.762144E-01, 1.38697E+00, 1.5886E-01, 3.5097E+03, &
      1.842400E-01, 1.47973E-01, 1.5049E-01, 3.1801E+03, &
      1.870600E-01, -1.46533E+00, 1.4773E-01, 3.0748E+03, &
      1.878120E-01, -3.39747E+00, 1.4701E-01, 3.0475E+03, &
      1.883732E-01, -2.53721E+00, 3.9624E+00, 8.1883E+04, &
      1.889400E-01, -1.39355E+00, 3.9498E+00, 8.1378E+04, &
      1.917600E-01, 3.64335E-01, 3.8885E+00, 7.8936E+04, &
      2.013709E-01, 2.05708E+00, 3.6926E+00, 7.1382E+04, &
      2.152655E-01, 3.05311E+00, 3.4372E+00, 6.2156E+04, &
      2.301188E-01, 3.66377E+00, 3.1815E+00, 5.3820E+04, &
      2.459970E-01, 4.09461E+00, 2.9293E+00, 4.6355E+04, &
      2.629708E-01, 4.41524E+00, 2.6840E+00, 3.9732E+04, &
      2.811158E-01, 4.65896E+00, 2.4491E+00, 3.3915E+04, &
      3.005128E-01, 4.84566E+00, 2.2270E+00, 2.8849E+04, &
      3.212482E-01, 4.98856E+00, 2.0189E+00, 2.4465E+04, &
      3.434143E-01, 5.09713E+00, 1.8254E+00, 2.0693E+04, &
      3.671099E-01, 5.17848E+00, 1.6466E+00, 1.7461E+04, &
      3.924405E-01, 5.23814E+00, 1.4821E+00, 1.4703E+04, &
      4.195189E-01, 5.28048E+00, 1.3316E+00, 1.2357E+04, &
      4.484657E-01, 5.30900E+00, 1.1942E+00, 1.0367E+04, &
      4.794098E-01, 5.32654E+00, 1.0694E+00, 8.6840E+03, &
      5.124891E-01, 5.33541E+00, 9.5617E-01, 7.2636E+03, &
      5.478508E-01, 5.33746E+00, 8.5376E-01, 6.0671E+03, &
      5.856525E-01, 5.33457E+00, 7.6132E-01, 5.0612E+03, &
      6.260625E-01, 5.32733E+00, 6.7805E-01, 4.2168E+03, &
      6.692609E-01, 5.31709E+00, 6.0317E-01, 3.5091E+03, &
      7.154399E-01, 5.30466E+00, 5.3594E-01, 2.9169E+03, &
      7.648052E-01, 5.29073E+00, 4.7567E-01, 2.4219E+03, &
      8.175768E-01, 5.27579E+00, 4.2165E-01, 2.0084E+03, &
      8.739896E-01, 5.26025E+00, 3.7337E-01, 1.6638E+03, &
      9.342948E-01, 5.24449E+00, 3.3033E-01, 1.3771E+03, &
      9.987612E-01, 5.22885E+00, 2.9207E-01, 1.1392E+03, &
      1.067676E+00, 5.21494E+00, 2.5600E-01, 9.3416E+02, &
      1.141345E+00, 5.19961E+00, 2.2436E-01, 7.6597E+02, &
      1.220098E+00, 5.18432E+00, 1.9662E-01, 6.2808E+02, &
      1.304285E+00, 5.16952E+00, 1.7231E-01, 5.1502E+02, &
      1.394281E+00, 5.15544E+00, 1.5101E-01, 4.2235E+02, &
      1.490486E+00, 5.14217E+00, 1.3235E-01, 3.4636E+02, &
      1.593329E+00, 5.12975E+00, 1.1599E-01, 2.8407E+02, &
      1.703269E+00, 5.11821E+00, 1.0165E-01, 2.3299E+02, &
      1.820795E+00, 5.10752E+00, 8.9083E-02, 1.9111E+02, &
      1.946430E+00, 5.09766E+00, 7.8072E-02, 1.5678E+02, &
      2.080733E+00, 5.08860E+00, 6.8421E-02, 1.2863E+02, &
      2.224304E+00, 5.08029E+00, 5.9964E-02, 1.0554E+02, &
      2.377781E+00, 5.07270E+00, 5.2552E-02, 8.6615E+01, &
      2.541848E+00, 5.06577E+00, 4.6056E-02, 7.1093E+01, &
      2.717235E+00, 5.05947E+00, 4.0363E-02, 5.8365E+01, &
      2.904724E+00, 5.05375E+00, 3.5374E-02, 4.7927E+01, &
      3.105150E+00, 5.04875E+00, 3.0872E-02, 3.9204E+01, &
      3.319406E+00, 5.04408E+00, 2.6837E-02, 3.1956E+01, &
      3.548445E+00, 5.03978E+00, 2.3320E-02, 2.6047E+01, &
      3.793288E+00, 5.03582E+00, 2.0164E-02, 2.1139E+01, &
      4.055024E+00, 5.03219E+00, 1.7435E-02, 1.7166E+01, &
      4.334821E+00, 5.02888E+00, 1.5075E-02, 1.3950E+01, &
      4.633924E+00, 5.02588E+00, 1.3035E-02, 1.1346E+01, &
      4.953664E+00, 5.02316E+00, 1.1271E-02, 9.2367E+00, &
      5.295467E+00, 5.02071E+00, 9.7453E-03, 7.5288E+00, &
      5.660855E+00, 5.01850E+00, 8.4264E-03, 6.1454E+00, &
      6.051453E+00, 5.01652E+00, 7.2860E-03, 5.0242E+00, &
      6.469004E+00, 5.01474E+00, 6.2986E-03, 4.1149E+00, &
      6.915365E+00, 5.01314E+00, 5.4448E-03, 3.3776E+00, &
      7.392525E+00, 5.01172E+00, 4.7067E-03, 2.7800E+00, &
      7.902609E+00, 5.01044E+00, 4.0687E-03, 2.2951E+00, &
      8.447890E+00, 5.00930E+00, 3.5172E-03, 1.9017E+00, &
      9.030794E+00, 5.00828E+00, 3.0360E-03, 1.5804E+00, &
      9.653919E+00, 5.00737E+00, 2.6070E-03, 1.3143E+00, &
      1.032004E+01, 5.00654E+00, 2.2386E-03, 1.0996E+00, &
      1.103212E+01, 5.00581E+00, 1.9223E-03, 9.2600E-01, &
      1.179334E+01, 5.00516E+00, 1.6507E-03, 7.8566E-01, &
      1.260708E+01, 5.00457E+00, 1.4174E-03, 6.7208E-01, &
      1.347697E+01, 5.00405E+00, 1.2171E-03, 5.8006E-01, &
      1.440688E+01, 5.00359E+00, 1.0451E-03, 5.0542E-01, &
      1.540095E+01, 5.00318E+00, 8.9746E-04, 4.4478E-01, &
      1.646362E+01, 5.00282E+00, 7.7065E-04, 3.9543E-01, &
      1.759961E+01, 5.00249E+00, 6.6175E-04, 3.5521E-01, &
      1.881398E+01, 5.00220E+00, 5.6825E-04, 3.2233E-01, &
      2.011215E+01, 5.00195E+00, 4.8795E-04, 2.9539E-01, &
      2.149988E+01, 5.00172E+00, 4.1900E-04, 2.7324E-01, &
      2.298338E+01, 5.00152E+00, 3.5980E-04, 2.5497E-01, &
      2.456923E+01, 5.00135E+00, 3.0896E-04, 2.3981E-01, &
      2.626450E+01, 5.00119E+00, 2.6530E-04, 2.2718E-01, &
      2.807676E+01, 5.00105E+00, 2.2730E-04, 2.1650E-01, &
      3.001405E+01, 5.00093E+00, 1.9460E-04, 2.0748E-01, &
      3.208502E+01, 5.00082E+00, 1.6660E-04, 1.9980E-01, &
      3.429889E+01, 5.00072E+00, 1.4263E-04, 1.9321E-01, &
      3.666551E+01, 5.00064E+00, 1.2211E-04, 1.8748E-01, &
      3.919543E+01, 5.00056E+00, 1.0455E-04, 1.8244E-01, &
      4.189992E+01, 5.00049E+00, 8.9505E-05, 1.7796E-01, &
      4.479101E+01, 5.00044E+00, 7.6628E-05, 1.7390E-01, &
      4.788159E+01, 5.00038E+00, 6.5604E-05, 1.7020E-01, &
      5.118542E+01, 5.00034E+00, 5.6166E-05, 1.6677E-01, &
      5.471721E+01, 5.00030E+00, 4.8086E-05, 1.6355E-01, &
      5.849270E+01, 5.00026E+00, 4.1169E-05, 1.6050E-01, &
      6.252870E+01, 5.00023E+00, 3.5246E-05, 1.5757E-01, &
      6.684318E+01, 5.00020E+00, 3.0176E-05, 1.5474E-01, &
      7.145536E+01, 5.00018E+00, 2.5835E-05, 1.5198E-01, &
      7.638578E+01, 5.00016E+00, 2.2118E-05, 1.4928E-01, &
      8.165640E+01, 5.00014E+00, 1.8973E-05, 1.4661E-01, &
      8.729069E+01, 5.00012E+00, 1.6348E-05, 1.4398E-01, &
      9.331374E+01, 5.00011E+00, 1.4085E-05, 1.4136E-01, &
      9.975239E+01, 5.00009E+00, 1.2136E-05, 1.3875E-01, &
      1.066353E+02, 5.00008E+00, 1.0456E-05, 1.3615E-01, &
      1.139931E+02, 5.00007E+00, 9.0092E-06, 1.3356E-01, &
      1.218587E+02, 5.00006E+00, 7.7624E-06, 1.3097E-01, &
      1.302669E+02, 5.00006E+00, 6.6881E-06, 1.2838E-01, &
      1.392553E+02, 5.00005E+00, 5.7625E-06, 1.2580E-01, &
      1.488640E+02, 5.00004E+00, 4.9650E-06, 1.2322E-01, &
      1.591356E+02, 5.00004E+00, 4.2779E-06, 1.2064E-01, &
      1.701159E+02, 5.00003E+00, 3.6858E-06, 1.1808E-01, &
      1.818539E+02, 5.00003E+00, 3.1757E-06, 1.1553E-01, &
      1.944018E+02, 5.00003E+00, 2.7362E-06, 1.1299E-01, &
      2.078156E+02, 5.00002E+00, 2.3575E-06, 1.1046E-01, &
      2.221548E+02, 5.00002E+00, 2.0313E-06, 1.0796E-01, &
      2.374835E+02, 5.00002E+00, 1.7502E-06, 1.0547E-01, &
      2.538699E+02, 5.00002E+00, 1.5079E-06, 1.0300E-01, &
      2.713869E+02, 5.00001E+00, 1.2993E-06, 1.0057E-01, &
      2.901126E+02, 5.00001E+00, 1.1194E-06, 9.8152E-02, &
      3.101304E+02, 5.00001E+00, 9.6452E-07, 9.5768E-02, &
      3.315294E+02, 5.00001E+00, 8.3104E-07, 9.3416E-02, &
      3.544049E+02, 5.00001E+00, 7.1603E-07, 9.1097E-02, &
      3.788588E+02, 5.00001E+00, 6.1693E-07, 8.8814E-02, &
      4.050001E+02, 5.00001E+00, 5.3155E-07, 8.6568E-02, &
      4.329451E+02, 5.00001E+00, 4.5799E-07, 8.4360E-02]
   case(6)
      frel = -1.8000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.272000E-03, -3.20134E-01, 0.0000E+00, 6.6098E-04, &
      6.368000E-03, -1.17579E+00, 0.0000E+00, 6.9744E-04, &
      6.393600E-03, -2.18154E+00, 0.0000E+00, 7.0737E-04, &
      6.432000E-03, -1.16186E+00, 1.9697E+00, 1.0729E+06, &
      6.528000E-03, -2.76671E-01, 1.9598E+00, 1.0518E+06, &
      1.069000E-02, 2.10173E+00, 1.7392E+00, 5.6999E+05, &
      1.142761E-02, 2.22488E+00, 1.7134E+00, 5.2530E+05, &
      1.221612E-02, 2.33994E+00, 1.6847E+00, 4.8316E+05, &
      1.305903E-02, 2.44742E+00, 1.6524E+00, 4.4330E+05, &
      1.396010E-02, 2.54710E+00, 1.6161E+00, 4.0558E+05, &
      1.492335E-02, 2.63805E+00, 1.5757E+00, 3.6992E+05, &
      1.595306E-02, 2.71844E+00, 1.5311E+00, 3.3626E+05, &
      1.705382E-02, 2.78446E+00, 1.4826E+00, 3.0459E+05, &
      1.823053E-02, 2.82559E+00, 1.4304E+00, 2.7489E+05, &
      1.911980E-02, 2.80931E+00, 1.3911E+00, 2.5491E+05, &
      1.941245E-02, 2.75154E+00, 1.3783E+00, 2.4874E+05, &
      1.948844E-02, 2.67983E+00, 1.3749E+00, 2.4718E+05, &
      1.949049E-02, 2.67501E+00, 1.3748E+00, 2.4713E+05, &
      1.960755E-02, 2.75433E+00, 1.5242E+00, 2.7234E+05, &
      1.990020E-02, 2.82926E+00, 1.5191E+00, 2.6745E+05, &
      2.083314E-02, 2.91938E+00, 1.5052E+00, 2.5312E+05, &
      2.227063E-02, 3.00738E+00, 1.4898E+00, 2.3437E+05, &
      2.380730E-02, 3.08738E+00, 1.4761E+00, 2.1723E+05, &
      2.545001E-02, 3.16576E+00, 1.4626E+00, 2.0134E+05, &
      2.720606E-02, 3.24448E+00, 1.4477E+00, 1.8643E+05, &
      2.908327E-02, 3.32364E+00, 1.4301E+00, 1.7227E+05, &
      3.109002E-02, 3.40280E+00, 1.4091E+00, 1.5879E+05, &
      3.323523E-02, 3.48136E+00, 1.3844E+00, 1.4594E+05, &
      3.552846E-02, 3.55864E+00, 1.3557E+00, 1.3369E+05, &
      3.797993E-02, 3.63385E+00, 1.3230E+00, 1.2204E+05, &
      4.060054E-02, 3.70629E+00, 1.2864E+00, 1.1100E+05, &
      4.340198E-02, 3.77526E+00, 1.2462E+00, 1.0059E+05, &
      4.639671E-02, 3.84022E+00, 1.2028E+00, 9.0829E+04, &
      4.959809E-02, 3.90069E+00, 1.1569E+00, 8.1721E+04, &
      5.302035E-02, 3.95635E+00, 1.1089E+00, 7.3276E+04, &
      5.667876E-02, 4.00697E+00, 1.0595E+00, 6.5488E+04, &
      6.058959E-02, 4.05263E+00, 1.0091E+00, 5.8347E+04, &
      6.477028E-02, 4.09290E+00, 9.5827E-01, 5.1834E+04, &
      6.923942E-02, 4.12801E+00, 9.0756E-01, 4.5922E+04, &
      7.401695E-02, 4.15805E+00, 8.5736E-01, 4.0582E+04, &
      7.912411E-02, 4.18316E+00, 8.0800E-01, 3.5777E+04, &
      8.458368E-02, 4.20340E+00, 7.5955E-01, 3.1461E+04, &
      9.041995E-02, 4.21865E+00, 7.1222E-01, 2.7596E+04, &
      9.665893E-02, 4.22883E+00, 6.6630E-01, 2.4151E+04, &
      1.033284E-01, 4.23389E+00, 6.2204E-01, 2.1091E+04, &
      1.104581E-01, 4.23373E+00, 5.7959E-01, 1.8383E+04, &
      1.180797E-01, 4.22817E+00, 5.3908E-01, 1.5996E+04, &
      1.262272E-01, 4.21665E+00, 4.9886E-01, 1.3847E+04, &
      1.349368E-01, 4.19786E+00, 4.6044E-01, 1.1956E+04, &
      1.442475E-01, 4.17119E+00, 4.2445E-01, 1.0310E+04, &
      1.542005E-01, 4.13557E+00, 3.9083E-01, 8.8805E+03, &
      1.648404E-01, 4.08928E+00, 3.5950E-01, 7.6415E+03, &
      1.762144E-01, 4.02965E+00, 3.3037E-01, 6.5691E+03, &
      1.883732E-01, 3.95250E+00, 3.0333E-01, 5.6424E+03, &
      2.013709E-01, 3.85109E+00, 2.7829E-01, 4.8425E+03, &
      2.152655E-01, 3.71374E+00, 2.5513E-01, 4.1531E+03, &
      2.301188E-01, 3.51819E+00, 2.3374E-01, 3.5596E+03, &
      2.459970E-01, 3.21423E+00, 2.1403E-01, 3.0490E+03, &
      2.629708E-01, 2.64704E+00, 1.9587E-01, 2.6104E+03, &
      2.781240E-01, 1.26093E+00, 1.8175E-01, 2.2904E+03, &
      2.811158E-01, 4.07656E-01, 1.7916E-01, 2.2339E+03, &
      2.823810E-01, -3.35665E-01, 1.7809E-01, 2.2105E+03, &
      2.835162E-01, -2.24717E+00, 1.7713E-01, 2.1899E+03, &
      2.852190E-01, -2.65349E-01, 3.9387E+00, 4.8382E+04, &
      2.894760E-01, 1.47293E+00, 3.8777E+00, 4.6932E+04, &
      3.005128E-01, 2.89123E+00, 3.7279E+00, 4.3462E+04, &
      3.212482E-01, 4.00159E+00, 3.4725E+00, 3.7872E+04, &
      3.434143E-01, 4.65259E+00, 3.2143E+00, 3.2793E+04, &
      3.671099E-01, 5.10424E+00, 2.9562E+00, 2.8213E+04, &
      3.924405E-01, 5.43616E+00, 2.7033E+00, 2.4135E+04, &
      4.195189E-01, 5.68561E+00, 2.4612E+00, 2.0555E+04, &
      4.484657E-01, 5.87450E+00, 2.2326E+00, 1.7442E+04, &
      4.794098E-01, 6.01732E+00, 2.0190E+00, 1.4756E+04, &
      5.124891E-01, 6.12442E+00, 1.8212E+00, 1.2451E+04, &
      5.478508E-01, 6.20354E+00, 1.6391E+00, 1.0483E+04, &
      5.856525E-01, 6.26062E+00, 1.4724E+00, 8.8093E+03, &
      6.260625E-01, 6.30033E+00, 1.3204E+00, 7.3902E+03, &
      6.692609E-01, 6.32635E+00, 1.1823E+00, 6.1901E+03, &
      7.154399E-01, 6.34162E+00, 1.0571E+00, 5.1777E+03, &
      7.648052E-01, 6.34849E+00, 9.4391E-01, 4.3251E+03, &
      8.175768E-01, 6.34886E+00, 8.4179E-01, 3.6084E+03, &
      8.739896E-01, 6.34469E+00, 7.4981E-01, 3.0069E+03, &
      9.342948E-01, 6.33647E+00, 6.6709E-01, 2.5027E+03, &
      9.987612E-01, 6.32562E+00, 5.9282E-01, 2.0806E+03, &
      1.067676E+00, 6.31267E+00, 5.2585E-01, 1.7266E+03, &
      1.141345E+00, 6.29863E+00, 4.6573E-01, 1.4307E+03, &
      1.220098E+00, 6.28281E+00, 4.0956E-01, 1.1771E+03, &
      1.304285E+00, 6.26551E+00, 3.6016E-01, 9.6850E+02, &
      1.394281E+00, 6.24774E+00, 3.1672E-01, 7.9688E+02, &
      1.490486E+00, 6.23009E+00, 2.7853E-01, 6.5569E+02, &
      1.593329E+00, 6.21289E+00, 2.4494E-01, 5.3955E+02, &
      1.703269E+00, 6.19636E+00, 2.1540E-01, 4.4400E+02, &
      1.820795E+00, 6.18064E+00, 1.8942E-01, 3.6539E+02, &
      1.946430E+00, 6.16581E+00, 1.6658E-01, 3.0072E+02, &
      2.080733E+00, 6.15192E+00, 1.4650E-01, 2.4752E+02, &
      2.224304E+00, 6.13897E+00, 1.2883E-01, 2.0374E+02, &
      2.377781E+00, 6.12696E+00, 1.1330E-01, 1.6773E+02, &
      2.541848E+00, 6.11588E+00, 9.9638E-02, 1.3809E+02, &
      2.717235E+00, 6.10571E+00, 8.7625E-02, 1.1371E+02, &
      2.904724E+00, 6.09643E+00, 7.7061E-02, 9.3646E+01, &
      3.105150E+00, 6.08790E+00, 6.7420E-02, 7.6739E+01, &
      3.319406E+00, 6.07984E+00, 5.8701E-02, 6.2598E+01, &
      3.548445E+00, 6.07234E+00, 5.1109E-02, 5.1075E+01, &
      3.793288E+00, 6.06543E+00, 4.4499E-02, 4.1686E+01, &
      4.055024E+00, 6.05909E+00, 3.8744E-02, 3.4035E+01, &
      4.334821E+00, 6.05329E+00, 3.3733E-02, 2.7799E+01, &
      4.633924E+00, 6.04802E+00, 2.9370E-02, 2.2717E+01, &
      4.953664E+00, 6.04323E+00, 2.5572E-02, 1.8574E+01, &
      5.295467E+00, 6.03890E+00, 2.2265E-02, 1.5197E+01, &
      5.660855E+00, 6.03498E+00, 1.9289E-02, 1.2384E+01, &
      6.051453E+00, 6.03140E+00, 1.6703E-02, 1.0096E+01, &
      6.469004E+00, 6.02815E+00, 1.4463E-02, 8.2403E+00, &
      6.915365E+00, 6.02520E+00, 1.2524E-02, 6.7348E+00, &
      7.392525E+00, 6.02255E+00, 1.0844E-02, 5.5130E+00, &
      7.902609E+00, 6.02016E+00, 9.3904E-03, 4.5212E+00, &
      8.447890E+00, 6.01801E+00, 8.1313E-03, 3.7161E+00, &
      9.030794E+00, 6.01610E+00, 7.0314E-03, 3.0584E+00, &
      9.653919E+00, 6.01439E+00, 6.0514E-03, 2.5142E+00, &
      1.032004E+01, 6.01284E+00, 5.2079E-03, 2.0746E+00, &
      1.103212E+01, 6.01144E+00, 4.4820E-03, 1.7194E+00, &
      1.179334E+01, 6.01018E+00, 3.8573E-03, 1.4320E+00, &
      1.260708E+01, 6.00906E+00, 3.3197E-03, 1.1995E+00, &
      1.347697E+01, 6.00805E+00, 2.8570E-03, 1.0112E+00, &
      1.440688E+01, 6.00715E+00, 2.4588E-03, 8.5861E-01, &
      1.540095E+01, 6.00635E+00, 2.1161E-03, 7.3485E-01, &
      1.646362E+01, 6.00563E+00, 1.8212E-03, 6.3435E-01, &
      1.759961E+01, 6.00499E+00, 1.5673E-03, 5.5263E-01, &
      1.881398E+01, 6.00443E+00, 1.3489E-03, 4.8610E-01, &
      2.011215E+01, 6.00392E+00, 1.1609E-03, 4.3181E-01, &
      2.149988E+01, 6.00348E+00, 9.9909E-04, 3.8745E-01, &
      2.298338E+01, 6.00308E+00, 8.5985E-04, 3.5109E-01, &
      2.456923E+01, 6.00272E+00, 7.4001E-04, 3.2120E-01, &
      2.626450E+01, 6.00241E+00, 6.3687E-04, 2.9654E-01, &
      2.807676E+01, 6.00213E+00, 5.4642E-04, 2.7590E-01, &
      3.001405E+01, 6.00189E+00, 4.6835E-04, 2.5871E-01, &
      3.208502E+01, 6.00167E+00, 4.0143E-04, 2.4435E-01, &
      3.429889E+01, 6.00148E+00, 3.4407E-04, 2.3229E-01, &
      3.666551E+01, 6.00130E+00, 2.9491E-04, 2.2206E-01, &
      3.919543E+01, 6.00115E+00, 2.5277E-04, 2.1330E-01, &
      4.189992E+01, 6.00102E+00, 2.1666E-04, 2.0573E-01, &
      4.479101E+01, 6.00090E+00, 1.8570E-04, 1.9910E-01, &
      4.788159E+01, 6.00079E+00, 1.5917E-04, 1.9325E-01, &
      5.118542E+01, 6.00070E+00, 1.3643E-04, 1.8799E-01, &
      5.471721E+01, 6.00062E+00, 1.1694E-04, 1.8322E-01, &
      5.849270E+01, 6.00054E+00, 1.0023E-04, 1.7883E-01, &
      6.252870E+01, 6.00048E+00, 8.5909E-05, 1.7474E-01, &
      6.684318E+01, 6.00042E+00, 7.3635E-05, 1.7091E-01, &
      7.145536E+01, 6.00037E+00, 6.3115E-05, 1.6726E-01, &
      7.638578E+01, 6.00033E+00, 5.4098E-05, 1.6376E-01, &
      8.165640E+01, 6.00029E+00, 4.6381E-05, 1.6039E-01, &
      8.729069E+01, 6.00025E+00, 3.9791E-05, 1.5711E-01, &
      9.331374E+01, 6.00022E+00, 3.4136E-05, 1.5390E-01, &
      9.975239E+01, 6.00020E+00, 2.9285E-05, 1.5076E-01, &
      1.066353E+02, 6.00017E+00, 2.5124E-05, 1.4766E-01, &
      1.139931E+02, 6.00015E+00, 2.1554E-05, 1.4460E-01, &
      1.218587E+02, 6.00013E+00, 1.8491E-05, 1.4158E-01, &
      1.302669E+02, 6.00012E+00, 1.5863E-05, 1.3859E-01, &
      1.392553E+02, 6.00010E+00, 1.3609E-05, 1.3562E-01, &
      1.488640E+02, 6.00009E+00, 1.1675E-05, 1.3268E-01, &
      1.591356E+02, 6.00008E+00, 1.0016E-05, 1.2978E-01, &
      1.701159E+02, 6.00007E+00, 8.5931E-06, 1.2690E-01, &
      1.818539E+02, 6.00006E+00, 7.3720E-06, 1.2404E-01, &
      1.944018E+02, 6.00005E+00, 6.3245E-06, 1.2122E-01, &
      2.078156E+02, 6.00005E+00, 5.4258E-06, 1.1843E-01, &
      2.221548E+02, 6.00004E+00, 4.6549E-06, 1.1567E-01, &
      2.374835E+02, 6.00004E+00, 3.9934E-06, 1.1296E-01, &
      2.538699E+02, 6.00003E+00, 3.4260E-06, 1.1027E-01, &
      2.713869E+02, 6.00003E+00, 2.9392E-06, 1.0763E-01, &
      2.901126E+02, 6.00003E+00, 2.5216E-06, 1.0503E-01, &
      3.101304E+02, 6.00002E+00, 2.1633E-06, 1.0247E-01, &
      3.315294E+02, 6.00002E+00, 1.8559E-06, 9.9959E-02, &
      3.544049E+02, 6.00002E+00, 1.5922E-06, 9.7495E-02, &
      3.788588E+02, 6.00001E+00, 1.3660E-06, 9.5081E-02, &
      4.050001E+02, 6.00001E+00, 1.1719E-06, 9.2716E-02, &
      4.329451E+02, 6.00001E+00, 1.0054E-06, 9.0403E-02]
   case(7)
      frel = -3.0000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      9.016000E-03, -9.91623E-01, 0.0000E+00, 1.3099E-03, &
      9.154000E-03, -1.99965E+00, 0.0000E+00, 1.3815E-03, &
      9.190800E-03, -3.16894E+00, 0.0000E+00, 1.4010E-03, &
      9.246000E-03, -2.00383E+00, 2.2815E+00, 7.4131E+05, &
      9.384000E-03, -1.00020E+00, 2.2891E+00, 7.3286E+05, &
      1.069000E-02, 5.47159E-01, 2.3575E+00, 6.6255E+05, &
      1.142761E-02, 8.70870E-01, 2.3922E+00, 6.2891E+05, &
      1.221612E-02, 1.13142E+00, 2.4252E+00, 5.9643E+05, &
      1.305903E-02, 1.35881E+00, 2.4553E+00, 5.6486E+05, &
      1.396010E-02, 1.56675E+00, 2.4814E+00, 5.3402E+05, &
      1.492335E-02, 1.76235E+00, 2.5024E+00, 5.0377E+05, &
      1.595306E-02, 1.94939E+00, 2.5171E+00, 4.7403E+05, &
      1.705382E-02, 2.12978E+00, 2.5248E+00, 4.4479E+05, &
      1.823053E-02, 2.30416E+00, 2.5246E+00, 4.1603E+05, &
      1.948844E-02, 2.47213E+00, 2.5157E+00, 3.8782E+05, &
      2.083314E-02, 2.63228E+00, 2.4977E+00, 3.6019E+05, &
      2.227063E-02, 2.78168E+00, 2.4704E+00, 3.3325E+05, &
      2.380730E-02, 2.91376E+00, 2.4334E+00, 3.0708E+05, &
      2.545001E-02, 3.00357E+00, 2.3870E+00, 2.8178E+05, &
      2.578380E-02, 3.00473E+00, 2.3769E+00, 2.7695E+05, &
      2.617845E-02, 2.96440E+00, 2.3646E+00, 2.7137E+05, &
      2.628369E-02, 2.89288E+00, 2.3613E+00, 2.6990E+05, &
      2.644155E-02, 2.97828E+00, 2.5098E+00, 2.8517E+05, &
      2.683620E-02, 3.06936E+00, 2.5052E+00, 2.8046E+05, &
      2.720606E-02, 3.11898E+00, 2.5010E+00, 2.7618E+05, &
      2.908327E-02, 3.29678E+00, 2.4810E+00, 2.5629E+05, &
      3.109002E-02, 3.45322E+00, 2.4598E+00, 2.3770E+05, &
      3.323523E-02, 3.60471E+00, 2.4340E+00, 2.2002E+05, &
      3.552846E-02, 3.75399E+00, 2.4027E+00, 2.0317E+05, &
      3.797993E-02, 3.90190E+00, 2.3648E+00, 1.8706E+05, &
      4.060054E-02, 4.04756E+00, 2.3181E+00, 1.7153E+05, &
      4.340198E-02, 4.18906E+00, 2.2628E+00, 1.5663E+05, &
      4.639671E-02, 4.32493E+00, 2.1993E+00, 1.4241E+05, &
      4.959809E-02, 4.45392E+00, 2.1284E+00, 1.2892E+05, &
      5.302035E-02, 4.57503E+00, 2.0510E+00, 1.1622E+05, &
      5.667876E-02, 4.68748E+00, 1.9684E+00, 1.0434E+05, &
      6.058959E-02, 4.79073E+00, 1.8816E+00, 9.3299E+04, &
      6.477028E-02, 4.88450E+00, 1.7919E+00, 8.3116E+04, &
      6.923942E-02, 4.96870E+00, 1.7005E+00, 7.3783E+04, &
      7.401695E-02, 5.04344E+00, 1.6083E+00, 6.5282E+04, &
      7.912411E-02, 5.10950E+00, 1.5166E+00, 5.7584E+04, &
      8.458368E-02, 5.16622E+00, 1.4260E+00, 5.0650E+04, &
      9.041995E-02, 5.21458E+00, 1.3374E+00, 4.4437E+04, &
      9.665893E-02, 5.25510E+00, 1.2514E+00, 3.8894E+04, &
      1.033284E-01, 5.28833E+00, 1.1683E+00, 3.3969E+04, &
      1.104581E-01, 5.31484E+00, 1.0886E+00, 2.9607E+04, &
      1.180797E-01, 5.33491E+00, 1.0120E+00, 2.5750E+04, &
      1.262272E-01, 5.34874E+00, 9.3899E-01, 2.2350E+04, &
      1.349368E-01, 5.35661E+00, 8.6967E-01, 1.9364E+04, &
      1.442475E-01, 5.35870E+00, 8.0415E-01, 1.6749E+04, &
      1.542005E-01, 5.35518E+00, 7.4248E-01, 1.4467E+04, &
      1.648404E-01, 5.34609E+00, 6.8462E-01, 1.2478E+04, &
      1.762144E-01, 5.33130E+00, 6.2932E-01, 1.0730E+04, &
      1.883732E-01, 5.30915E+00, 5.7532E-01, 9.1764E+03, &
      2.013709E-01, 5.27886E+00, 5.2549E-01, 7.8407E+03, &
      2.152655E-01, 5.23972E+00, 4.7959E-01, 6.6942E+03, &
      2.301188E-01, 5.19030E+00, 4.3741E-01, 5.7115E+03, &
      2.459970E-01, 5.12826E+00, 3.9869E-01, 4.8701E+03, &
      2.629708E-01, 5.04985E+00, 3.6322E-01, 4.1506E+03, &
      2.811158E-01, 4.94898E+00, 3.3075E-01, 3.5358E+03, &
      3.005128E-01, 4.81524E+00, 3.0107E-01, 3.0110E+03, &
      3.212482E-01, 4.62915E+00, 2.7396E-01, 2.5633E+03, &
      3.434143E-01, 4.34850E+00, 2.4923E-01, 2.1816E+03, &
      3.671099E-01, 3.85352E+00, 2.2669E-01, 1.8564E+03, &
      3.924405E-01, 2.43899E+00, 2.0615E-01, 1.5795E+03, &
      3.935680E-01, 2.29017E+00, 2.0531E-01, 1.5686E+03, &
      3.995920E-01, 6.66889E-01, 2.0092E-01, 1.5119E+03, &
      4.011984E-01, -1.27959E+00, 1.9977E-01, 1.4973E+03, &
      4.036080E-01, 7.43479E-01, 4.0314E+00, 3.0009E+04, &
      4.096320E-01, 2.51867E+00, 3.9626E+00, 2.9063E+04, &
      4.195189E-01, 3.58763E+00, 3.8543E+00, 2.7603E+04, &
      4.484657E-01, 4.91711E+00, 3.5663E+00, 2.3892E+04, &
      4.794098E-01, 5.62761E+00, 3.2860E+00, 2.0593E+04, &
      5.124891E-01, 6.10433E+00, 3.0109E+00, 1.7652E+04, &
      5.478508E-01, 6.44813E+00, 2.7458E+00, 1.5059E+04, &
      5.856525E-01, 6.70350E+00, 2.4943E+00, 1.2797E+04, &
      6.260625E-01, 6.89525E+00, 2.2583E+00, 1.0838E+04, &
      6.692609E-01, 7.03922E+00, 2.0388E+00, 9.1536E+03, &
      7.154399E-01, 7.14645E+00, 1.8360E+00, 7.7114E+03, &
      7.648052E-01, 7.22503E+00, 1.6498E+00, 6.4823E+03, &
      8.175768E-01, 7.28114E+00, 1.4796E+00, 5.4386E+03, &
      8.739896E-01, 7.31959E+00, 1.3248E+00, 4.5552E+03, &
      9.342948E-01, 7.34418E+00, 1.1842E+00, 3.8094E+03, &
      9.987612E-01, 7.35796E+00, 1.0571E+00, 3.1812E+03, &
      1.067676E+00, 7.36340E+00, 9.4164E-01, 2.6510E+03, &
      1.141345E+00, 7.36196E+00, 8.3758E-01, 2.2061E+03, &
      1.220098E+00, 7.35540E+00, 7.4413E-01, 1.8336E+03, &
      1.304285E+00, 7.34489E+00, 6.6043E-01, 1.5225E+03, &
      1.394281E+00, 7.33166E+00, 5.8564E-01, 1.2632E+03, &
      1.490486E+00, 7.31668E+00, 5.1897E-01, 1.0473E+03, &
      1.593329E+00, 7.30081E+00, 4.5964E-01, 8.6789E+02, &
      1.703269E+00, 7.28407E+00, 4.0501E-01, 7.1555E+02, &
      1.820795E+00, 7.26624E+00, 3.5664E-01, 5.8960E+02, &
      1.946430E+00, 7.24821E+00, 3.1405E-01, 4.8583E+02, &
      2.080733E+00, 7.23046E+00, 2.7654E-01, 4.0035E+02, &
      2.224304E+00, 7.21328E+00, 2.4352E-01, 3.2994E+02, &
      2.377781E+00, 7.19688E+00, 2.1445E-01, 2.7193E+02, &
      2.541848E+00, 7.18139E+00, 1.8884E-01, 2.2414E+02, &
      2.717235E+00, 7.16696E+00, 1.6630E-01, 1.8477E+02, &
      2.904724E+00, 7.15398E+00, 1.4645E-01, 1.5234E+02, &
      3.105150E+00, 7.14238E+00, 1.2829E-01, 1.2496E+02, &
      3.319406E+00, 7.13003E+00, 1.1184E-01, 1.0202E+02, &
      3.548445E+00, 7.11843E+00, 9.7505E-02, 8.3307E+01, &
      3.793288E+00, 7.10763E+00, 8.5005E-02, 6.8043E+01, &
      4.055024E+00, 7.09763E+00, 7.4107E-02, 5.5590E+01, &
      4.334821E+00, 7.08842E+00, 6.4606E-02, 4.5428E+01, &
      4.633924E+00, 7.07996E+00, 5.6324E-02, 3.7136E+01, &
      4.953664E+00, 7.07222E+00, 4.9103E-02, 3.0370E+01, &
      5.295467E+00, 7.06516E+00, 4.2809E-02, 2.4848E+01, &
      5.660855E+00, 7.05874E+00, 3.7321E-02, 2.0340E+01, &
      6.051453E+00, 7.05290E+00, 3.2537E-02, 1.6660E+01, &
      6.469004E+00, 7.04762E+00, 2.8366E-02, 1.3656E+01, &
      6.915365E+00, 7.04284E+00, 2.4730E-02, 1.1203E+01, &
      7.392525E+00, 7.03853E+00, 2.1560E-02, 9.1994E+00, &
      7.902609E+00, 7.03465E+00, 1.8724E-02, 7.5352E+00, &
      8.447890E+00, 7.03113E+00, 1.6235E-02, 6.1714E+00, &
      9.030794E+00, 7.02795E+00, 1.4060E-02, 5.0575E+00, &
      9.653919E+00, 7.02503E+00, 1.2128E-02, 4.1376E+00, &
      1.032004E+01, 7.02238E+00, 1.0461E-02, 3.3933E+00, &
      1.103212E+01, 7.02000E+00, 9.0226E-03, 2.7910E+00, &
      1.179334E+01, 7.01785E+00, 7.7825E-03, 2.3032E+00, &
      1.260708E+01, 7.01591E+00, 6.7122E-03, 1.9080E+00, &
      1.347697E+01, 7.01418E+00, 5.7889E-03, 1.5878E+00, &
      1.440688E+01, 7.01263E+00, 4.9927E-03, 1.3280E+00, &
      1.540095E+01, 7.01123E+00, 4.3060E-03, 1.1174E+00, &
      1.646362E+01, 7.00999E+00, 3.7138E-03, 9.4626E-01, &
      1.759961E+01, 7.00888E+00, 3.2030E-03, 8.0721E-01, &
      1.881398E+01, 7.00789E+00, 2.7624E-03, 6.9407E-01, &
      2.011215E+01, 7.00701E+00, 2.3825E-03, 6.0190E-01, &
      2.149988E+01, 7.00622E+00, 2.0548E-03, 5.2671E-01, &
      2.298338E+01, 7.00552E+00, 1.7722E-03, 4.6527E-01, &
      2.456923E+01, 7.00489E+00, 1.5285E-03, 4.1494E-01, &
      2.626450E+01, 7.00435E+00, 1.3183E-03, 3.7363E-01, &
      2.807676E+01, 7.00387E+00, 1.1327E-03, 3.3915E-01, &
      3.001405E+01, 7.00343E+00, 9.7197E-04, 3.1063E-01, &
      3.208502E+01, 7.00304E+00, 8.3408E-04, 2.8707E-01, &
      3.429889E+01, 7.00269E+00, 7.1576E-04, 2.6750E-01, &
      3.666551E+01, 7.00238E+00, 6.1422E-04, 2.5116E-01, &
      3.919543E+01, 7.00210E+00, 5.2709E-04, 2.3740E-01, &
      4.189992E+01, 7.00186E+00, 4.5232E-04, 2.2573E-01, &
      4.479101E+01, 7.00164E+00, 3.8815E-04, 2.1574E-01, &
      4.788159E+01, 7.00145E+00, 3.3309E-04, 2.0710E-01, &
      5.118542E+01, 7.00128E+00, 2.8584E-04, 1.9955E-01, &
      5.471721E+01, 7.00113E+00, 2.4529E-04, 1.9287E-01, &
      5.849270E+01, 7.00100E+00, 2.1050E-04, 1.8689E-01, &
      6.252870E+01, 7.00088E+00, 1.8064E-04, 1.8148E-01, &
      6.684318E+01, 7.00078E+00, 1.5501E-04, 1.7652E-01, &
      7.145536E+01, 7.00068E+00, 1.3303E-04, 1.7192E-01, &
      7.638578E+01, 7.00060E+00, 1.1416E-04, 1.6762E-01, &
      8.165640E+01, 7.00053E+00, 9.7951E-05, 1.6355E-01, &
      8.729069E+01, 7.00047E+00, 8.4023E-05, 1.5967E-01, &
      9.331374E+01, 7.00041E+00, 7.2075E-05, 1.5595E-01, &
      9.975239E+01, 7.00036E+00, 6.1826E-05, 1.5235E-01, &
      1.066353E+02, 7.00032E+00, 5.3035E-05, 1.4886E-01, &
      1.139931E+02, 7.00028E+00, 4.5494E-05, 1.4547E-01, &
      1.218587E+02, 7.00025E+00, 3.9025E-05, 1.4215E-01, &
      1.302669E+02, 7.00022E+00, 3.3476E-05, 1.3890E-01, &
      1.392553E+02, 7.00019E+00, 2.8716E-05, 1.3571E-01, &
      1.488640E+02, 7.00017E+00, 2.4633E-05, 1.3258E-01, &
      1.591356E+02, 7.00015E+00, 2.1131E-05, 1.2949E-01, &
      1.701159E+02, 7.00013E+00, 1.8126E-05, 1.2646E-01, &
      1.818539E+02, 7.00012E+00, 1.5549E-05, 1.2349E-01, &
      1.944018E+02, 7.00010E+00, 1.3338E-05, 1.2056E-01, &
      2.078156E+02, 7.00009E+00, 1.1442E-05, 1.1768E-01, &
      2.221548E+02, 7.00008E+00, 9.8150E-06, 1.1485E-01, &
      2.374835E+02, 7.00007E+00, 8.4195E-06, 1.1208E-01, &
      2.538699E+02, 7.00006E+00, 7.2224E-06, 1.0936E-01, &
      2.713869E+02, 7.00005E+00, 6.1956E-06, 1.0669E-01, &
      2.901126E+02, 7.00005E+00, 5.3147E-06, 1.0408E-01, &
      3.101304E+02, 7.00004E+00, 4.5591E-06, 1.0152E-01, &
      3.315294E+02, 7.00004E+00, 3.9109E-06, 9.9036E-02, &
      3.544049E+02, 7.00003E+00, 3.3549E-06, 9.6605E-02, &
      3.788588E+02, 7.00003E+00, 2.8779E-06, 9.4236E-02, &
      4.050001E+02, 7.00002E+00, 2.4688E-06, 9.1929E-02, &
      4.329451E+02, 7.00002E+00, 2.1178E-06, 8.9686E-02]
   case(8)
      frel = -4.2000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.958000E-03, -2.20730E+00, 0.0000E+00, 2.2483E-04, &
      7.064500E-03, -3.19554E+00, 0.0000E+00, 2.3940E-04, &
      7.092900E-03, -4.32288E+00, 0.0000E+00, 2.4339E-04, &
      7.135500E-03, -3.22458E+00, 2.1888E+00, 8.0680E+05, &
      7.242000E-03, -2.28890E+00, 2.2193E+00, 8.0601E+05, &
      1.069000E-02, 2.11490E-01, 2.8635E+00, 7.0453E+05, &
      1.142761E-02, 4.32528E-01, 2.9247E+00, 6.7313E+05, &
      1.221612E-02, 6.41679E-01, 2.9771E+00, 6.4097E+05, &
      1.305903E-02, 8.41330E-01, 3.0227E+00, 6.0878E+05, &
      1.396010E-02, 1.03339E+00, 3.0629E+00, 5.7706E+05, &
      1.492335E-02, 1.21943E+00, 3.0988E+00, 5.4613E+05, &
      1.595306E-02, 1.40070E+00, 3.1309E+00, 5.1617E+05, &
      1.705382E-02, 1.57805E+00, 3.1595E+00, 4.8727E+05, &
      1.823053E-02, 1.75182E+00, 3.1845E+00, 4.5943E+05, &
      1.948844E-02, 1.92140E+00, 3.2056E+00, 4.3262E+05, &
      2.083314E-02, 2.08430E+00, 3.2222E+00, 4.0679E+05, &
      2.227063E-02, 2.23222E+00, 3.2335E+00, 3.8187E+05, &
      2.322600E-02, 2.29401E+00, 3.2375E+00, 3.6662E+05, &
      2.358150E-02, 2.27485E+00, 3.2384E+00, 3.6119E+05, &
      2.367630E-02, 2.21999E+00, 3.2385E+00, 3.5976E+05, &
      2.380730E-02, 2.29056E+00, 3.3643E+00, 3.7167E+05, &
      2.381850E-02, 2.29549E+00, 3.3645E+00, 3.7152E+05, &
      2.417400E-02, 2.38419E+00, 3.3716E+00, 3.6683E+05, &
      2.545001E-02, 2.56373E+00, 3.3962E+00, 3.5098E+05, &
      2.720606E-02, 2.77017E+00, 3.4281E+00, 3.3141E+05, &
      2.908327E-02, 2.97896E+00, 3.4550E+00, 3.1245E+05, &
      3.109002E-02, 3.19372E+00, 3.4718E+00, 2.9370E+05, &
      3.323523E-02, 3.41313E+00, 3.4762E+00, 2.7509E+05, &
      3.552846E-02, 3.63537E+00, 3.4671E+00, 2.5667E+05, &
      3.797993E-02, 3.85841E+00, 3.4438E+00, 2.3849E+05, &
      4.060054E-02, 4.08006E+00, 3.4059E+00, 2.2064E+05, &
      4.340198E-02, 4.29812E+00, 3.3536E+00, 2.0322E+05, &
      4.639671E-02, 4.51045E+00, 3.2873E+00, 1.8635E+05, &
      4.959809E-02, 4.71509E+00, 3.2080E+00, 1.7012E+05, &
      5.302035E-02, 4.91032E+00, 3.1170E+00, 1.5462E+05, &
      5.667876E-02, 5.09473E+00, 3.0157E+00, 1.3994E+05, &
      6.058959E-02, 5.26727E+00, 2.9059E+00, 1.2614E+05, &
      6.477028E-02, 5.42719E+00, 2.7892E+00, 1.1326E+05, &
      6.923942E-02, 5.57410E+00, 2.6674E+00, 1.0132E+05, &
      7.401695E-02, 5.70887E+00, 2.5423E+00, 9.0338E+04, &
      7.912411E-02, 5.82981E+00, 2.4154E+00, 8.0288E+04, &
      8.458368E-02, 5.93830E+00, 2.2881E+00, 7.1150E+04, &
      9.041995E-02, 6.03498E+00, 2.1619E+00, 6.2884E+04, &
      9.665893E-02, 6.12073E+00, 2.0376E+00, 5.5445E+04, &
      1.033284E-01, 6.19637E+00, 1.9159E+00, 4.8768E+04, &
      1.104581E-01, 6.26258E+00, 1.7974E+00, 4.2799E+04, &
      1.180797E-01, 6.32029E+00, 1.6829E+00, 3.7485E+04, &
      1.262272E-01, 6.37066E+00, 1.5728E+00, 3.2773E+04, &
      1.349368E-01, 6.41516E+00, 1.4636E+00, 2.8528E+04, &
      1.442475E-01, 6.44826E+00, 1.3430E+00, 2.4488E+04, &
      1.542005E-01, 6.46854E+00, 1.2306E+00, 2.0990E+04, &
      1.648404E-01, 6.47860E+00, 1.1262E+00, 1.7971E+04, &
      1.762144E-01, 6.48016E+00, 1.0297E+00, 1.5370E+04, &
      1.883732E-01, 6.47437E+00, 9.4068E-01, 1.3135E+04, &
      2.013709E-01, 6.46203E+00, 8.5876E-01, 1.1217E+04, &
      2.152655E-01, 6.44358E+00, 7.8355E-01, 9.5744E+03, &
      2.301188E-01, 6.41919E+00, 7.1464E-01, 8.1689E+03, &
      2.459970E-01, 6.38871E+00, 6.5159E-01, 6.9677E+03, &
      2.629708E-01, 6.35166E+00, 5.9399E-01, 5.9419E+03, &
      2.811158E-01, 6.30713E+00, 5.4143E-01, 5.0668E+03, &
      3.005128E-01, 6.25366E+00, 4.9351E-01, 4.3204E+03, &
      3.212482E-01, 6.18896E+00, 4.4986E-01, 3.6844E+03, &
      3.434143E-01, 6.10952E+00, 4.1012E-01, 3.1423E+03, &
      3.671099E-01, 6.00975E+00, 3.7397E-01, 2.6807E+03, &
      3.924405E-01, 5.88028E+00, 3.4110E-01, 2.2874E+03, &
      4.195189E-01, 5.70405E+00, 3.1121E-01, 1.9526E+03, &
      4.484657E-01, 5.44549E+00, 2.8376E-01, 1.6656E+03, &
      4.794098E-01, 5.01012E+00, 2.5661E-01, 1.4093E+03, &
      5.124891E-01, 3.97074E+00, 2.3191E-01, 1.1918E+03, &
      5.213600E-01, 3.28506E+00, 2.2593E-01, 1.1413E+03, &
      5.293400E-01, 1.63837E+00, 2.2075E-01, 1.0985E+03, &
      5.314680E-01, -3.36417E-01, 2.1941E-01, 1.0874E+03, &
      5.346600E-01, 1.71635E+00, 4.1067E+00, 2.0204E+04, &
      5.426400E-01, 3.51780E+00, 4.0360E+00, 1.9564E+04, &
      5.478508E-01, 4.05204E+00, 3.9911E+00, 1.9162E+04, &
      5.856525E-01, 5.74654E+00, 3.6911E+00, 1.6579E+04, &
      6.260625E-01, 6.55266E+00, 3.3971E+00, 1.4273E+04, &
      6.692609E-01, 7.07480E+00, 3.1024E+00, 1.2194E+04, &
      7.154399E-01, 7.44087E+00, 2.8186E+00, 1.0364E+04, &
      7.648052E-01, 7.70602E+00, 2.5523E+00, 8.7790E+03, &
      8.175768E-01, 7.90150E+00, 2.3068E+00, 7.4226E+03, &
      8.739896E-01, 8.04830E+00, 2.0831E+00, 6.2705E+03, &
      9.342948E-01, 8.16115E+00, 1.8753E+00, 5.2808E+03, &
      9.987612E-01, 8.24254E+00, 1.6834E+00, 4.4347E+03, &
      1.067676E+00, 8.30267E+00, 1.5016E+00, 3.7006E+03, &
      1.141345E+00, 8.33861E+00, 1.3385E+00, 3.0861E+03, &
      1.220098E+00, 8.35935E+00, 1.1926E+00, 2.5724E+03, &
      1.304285E+00, 8.36902E+00, 1.0621E+00, 2.1433E+03, &
      1.394281E+00, 8.37050E+00, 9.4552E-01, 1.7851E+03, &
      1.490486E+00, 8.36596E+00, 8.4147E-01, 1.4864E+03, &
      1.593329E+00, 8.35705E+00, 7.4865E-01, 1.2372E+03, &
      1.703269E+00, 8.34592E+00, 6.6590E-01, 1.0297E+03, &
      1.820795E+00, 8.33198E+00, 5.9217E-01, 8.5675E+02, &
      1.946430E+00, 8.31689E+00, 5.2651E-01, 7.1277E+02, &
      2.080733E+00, 8.30145E+00, 4.6808E-01, 5.9295E+02, &
      2.224304E+00, 8.28581E+00, 4.1395E-01, 4.9071E+02, &
      2.377781E+00, 8.26885E+00, 3.6513E-01, 4.0507E+02, &
      2.541848E+00, 8.25152E+00, 3.2207E-01, 3.3439E+02, &
      2.717235E+00, 8.23442E+00, 2.8409E-01, 2.7607E+02, &
      2.904724E+00, 8.21791E+00, 2.5059E-01, 2.2795E+02, &
      3.105150E+00, 8.20256E+00, 2.1994E-01, 1.8730E+02, &
      3.319406E+00, 8.18728E+00, 1.9213E-01, 1.5320E+02, &
      3.548445E+00, 8.17222E+00, 1.6784E-01, 1.2531E+02, &
      3.793288E+00, 8.15779E+00, 1.4662E-01, 1.0253E+02, &
      4.055024E+00, 8.14414E+00, 1.2808E-01, 8.3902E+01, &
      4.334821E+00, 8.13136E+00, 1.1189E-01, 6.8674E+01, &
      4.633924E+00, 8.11947E+00, 9.7749E-02, 5.6224E+01, &
      4.953664E+00, 8.10847E+00, 8.5394E-02, 4.6045E+01, &
      5.295467E+00, 8.09834E+00, 7.4601E-02, 3.7721E+01, &
      5.660855E+00, 8.08903E+00, 6.5172E-02, 3.0914E+01, &
      6.051453E+00, 8.08051E+00, 5.6936E-02, 2.5347E+01, &
      6.469004E+00, 8.07273E+00, 4.9740E-02, 2.0793E+01, &
      6.915365E+00, 8.06564E+00, 4.3455E-02, 1.7067E+01, &
      7.392525E+00, 8.05920E+00, 3.7964E-02, 1.4019E+01, &
      7.902609E+00, 8.05336E+00, 3.3166E-02, 1.1524E+01, &
      8.447890E+00, 8.04808E+00, 2.8976E-02, 9.4830E+00, &
      9.030794E+00, 8.04333E+00, 2.5277E-02, 7.8007E+00, &
      9.653919E+00, 8.03905E+00, 2.1935E-02, 6.3937E+00, &
      1.032004E+01, 8.03513E+00, 1.8995E-02, 5.2389E+00, &
      1.103212E+01, 8.03154E+00, 1.6411E-02, 4.2922E+00, &
      1.179334E+01, 8.02827E+00, 1.4177E-02, 3.5245E+00, &
      1.260708E+01, 8.02530E+00, 1.2248E-02, 2.9021E+00, &
      1.347697E+01, 8.02262E+00, 1.0581E-02, 2.3975E+00, &
      1.440688E+01, 8.02021E+00, 9.1406E-03, 1.9880E+00, &
      1.540095E+01, 8.01804E+00, 7.8965E-03, 1.6555E+00, &
      1.646362E+01, 8.01609E+00, 6.8218E-03, 1.3855E+00, &
      1.759961E+01, 8.01434E+00, 5.8934E-03, 1.1660E+00, &
      1.881398E+01, 8.01277E+00, 5.0913E-03, 9.8748E-01, &
      2.011215E+01, 8.01137E+00, 4.3984E-03, 8.4207E-01, &
      2.149988E+01, 8.01012E+00, 3.7998E-03, 7.2355E-01, &
      2.298338E+01, 8.00900E+00, 3.2827E-03, 6.2680E-01, &
      2.456923E+01, 8.00801E+00, 2.8360E-03, 5.4770E-01, &
      2.626450E+01, 8.00713E+00, 2.4500E-03, 4.8291E-01, &
      2.807676E+01, 8.00633E+00, 2.1079E-03, 4.2890E-01, &
      3.001405E+01, 8.00562E+00, 1.8110E-03, 3.8440E-01, &
      3.208502E+01, 8.00498E+00, 1.5560E-03, 3.4783E-01, &
      3.429889E+01, 8.00442E+00, 1.3368E-03, 3.1767E-01, &
      3.666551E+01, 8.00392E+00, 1.1486E-03, 2.9267E-01, &
      3.919543E+01, 8.00347E+00, 9.8682E-04, 2.7185E-01, &
      4.189992E+01, 8.00307E+00, 8.4786E-04, 2.5437E-01, &
      4.479101E+01, 8.00272E+00, 7.2846E-04, 2.3962E-01, &
      4.788159E+01, 8.00240E+00, 6.2588E-04, 2.2704E-01, &
      5.118542E+01, 8.00213E+00, 5.3774E-04, 2.1622E-01, &
      5.471721E+01, 8.00188E+00, 4.6201E-04, 2.0683E-01, &
      5.849270E+01, 8.00166E+00, 3.9695E-04, 1.9859E-01, &
      6.252870E+01, 8.00147E+00, 3.4106E-04, 1.9128E-01, &
      6.684318E+01, 8.00130E+00, 2.9303E-04, 1.8471E-01, &
      7.145536E+01, 8.00114E+00, 2.5177E-04, 1.7875E-01, &
      7.638578E+01, 8.00101E+00, 2.1632E-04, 1.7328E-01, &
      8.165640E+01, 8.00089E+00, 1.8580E-04, 1.6821E-01, &
      8.729069E+01, 8.00079E+00, 1.5948E-04, 1.6348E-01, &
      9.331374E+01, 8.00069E+00, 1.3689E-04, 1.5900E-01, &
      9.975239E+01, 8.00061E+00, 1.1750E-04, 1.5476E-01, &
      1.066353E+02, 8.00054E+00, 1.0086E-04, 1.5070E-01, &
      1.139931E+02, 8.00048E+00, 8.6571E-05, 1.4681E-01, &
      1.218587E+02, 8.00042E+00, 7.4309E-05, 1.4304E-01, &
      1.302669E+02, 8.00037E+00, 6.3783E-05, 1.3941E-01, &
      1.392553E+02, 8.00033E+00, 5.4749E-05, 1.3588E-01, &
      1.488640E+02, 8.00029E+00, 4.6994E-05, 1.3245E-01, &
      1.591356E+02, 8.00025E+00, 4.0338E-05, 1.2912E-01, &
      1.701159E+02, 8.00022E+00, 3.4625E-05, 1.2586E-01, &
      1.818539E+02, 8.00020E+00, 2.9721E-05, 1.2269E-01, &
      1.944018E+02, 8.00017E+00, 2.5511E-05, 1.1960E-01, &
      2.078156E+02, 8.00015E+00, 2.1898E-05, 1.1659E-01, &
      2.221548E+02, 8.00013E+00, 1.8797E-05, 1.1365E-01, &
      2.374835E+02, 8.00012E+00, 1.6135E-05, 1.1080E-01, &
      2.538699E+02, 8.00010E+00, 1.3849E-05, 1.0801E-01, &
      2.713869E+02, 8.00009E+00, 1.1888E-05, 1.0532E-01, &
      2.901126E+02, 8.00008E+00, 1.0204E-05, 1.0269E-01, &
      3.101304E+02, 8.00007E+00, 8.7592E-06, 1.0014E-01, &
      3.315294E+02, 8.00006E+00, 7.5187E-06, 9.7683E-02, &
      3.544049E+02, 8.00005E+00, 6.4539E-06, 9.5297E-02, &
      3.788588E+02, 8.00005E+00, 5.5399E-06, 9.2992E-02, &
      4.050001E+02, 8.00004E+00, 4.7554E-06, 9.0768E-02, &
      4.329451E+02, 8.00004E+00, 4.0819E-06, 8.8625E-02]
   case(9)
      frel = -5.4000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      8.428000E-03, -2.53843E+00, 0.0000E+00, 1.5103E-04, &
      8.557000E-03, -3.50839E+00, 0.0000E+00, 1.6145E-04, &
      8.591400E-03, -4.60765E+00, 0.0000E+00, 1.6431E-04, &
      8.643000E-03, -3.54459E+00, 2.1300E+00, 5.4585E+05, &
      8.772000E-03, -2.64402E+00, 2.1646E+00, 5.4656E+05, &
      1.069000E-02, -1.03438E+00, 2.6472E+00, 5.4849E+05, &
      1.142761E-02, -7.85561E-01, 2.7969E+00, 5.4210E+05, &
      1.221612E-02, -5.55727E-01, 2.9380E+00, 5.3270E+05, &
      1.305903E-02, -3.35163E-01, 3.0710E+00, 5.2087E+05, &
      1.396010E-02, -1.18924E-01, 3.1962E+00, 5.0711E+05, &
      1.492335E-02, 9.58117E-02, 3.3139E+00, 4.9186E+05, &
      1.595306E-02, 3.10798E-01, 3.4245E+00, 4.7546E+05, &
      1.705382E-02, 5.27186E-01, 3.5280E+00, 4.5822E+05, &
      1.823053E-02, 7.45735E-01, 3.6244E+00, 4.4035E+05, &
      1.948844E-02, 9.66891E-01, 3.7134E+00, 4.2205E+05, &
      2.083314E-02, 1.19079E+00, 3.7948E+00, 4.0345E+05, &
      2.227063E-02, 1.41722E+00, 3.8678E+00, 3.8468E+05, &
      2.380730E-02, 1.64545E+00, 3.9320E+00, 3.6582E+05, &
      2.545001E-02, 1.87395E+00, 3.9866E+00, 3.4696E+05, &
      2.720606E-02, 2.10000E+00, 4.0307E+00, 3.2816E+05, &
      2.908327E-02, 2.31315E+00, 4.0636E+00, 3.0948E+05, &
      3.038000E-02, 2.42332E+00, 4.0786E+00, 2.9736E+05, &
      3.084500E-02, 2.42239E+00, 4.0826E+00, 2.9316E+05, &
      3.096900E-02, 2.37534E+00, 4.0835E+00, 2.9206E+05, &
      3.109002E-02, 2.42622E+00, 4.2017E+00, 2.9934E+05, &
      3.115500E-02, 2.45285E+00, 4.2031E+00, 2.9881E+05, &
      3.162000E-02, 2.55277E+00, 4.2127E+00, 2.9510E+05, &
      3.323523E-02, 2.77327E+00, 4.2434E+00, 2.8280E+05, &
      3.552846E-02, 3.04456E+00, 4.2797E+00, 2.6681E+05, &
      3.797993E-02, 3.31742E+00, 4.3029E+00, 2.5094E+05, &
      4.060054E-02, 3.59298E+00, 4.3112E+00, 2.3519E+05, &
      4.340198E-02, 3.87021E+00, 4.3036E+00, 2.1963E+05, &
      4.639671E-02, 4.14747E+00, 4.2796E+00, 2.0431E+05, &
      4.959809E-02, 4.42281E+00, 4.2389E+00, 1.8930E+05, &
      5.302035E-02, 4.69410E+00, 4.1815E+00, 1.7468E+05, &
      5.667876E-02, 4.95919E+00, 4.1078E+00, 1.6053E+05, &
      6.058959E-02, 5.21598E+00, 4.0186E+00, 1.4690E+05, &
      6.477028E-02, 5.46254E+00, 3.9150E+00, 1.3388E+05, &
      6.923942E-02, 5.69718E+00, 3.7985E+00, 1.2151E+05, &
      7.401695E-02, 5.91853E+00, 3.6708E+00, 1.0985E+05, &
      7.912411E-02, 6.12552E+00, 3.5338E+00, 9.8922E+04, &
      8.458368E-02, 6.31749E+00, 3.3894E+00, 8.8755E+04, &
      9.041995E-02, 6.49407E+00, 3.2395E+00, 7.9355E+04, &
      9.665893E-02, 6.65510E+00, 3.0860E+00, 7.0717E+04, &
      1.033284E-01, 6.80118E+00, 2.9308E+00, 6.2825E+04, &
      1.104581E-01, 6.93272E+00, 2.7755E+00, 5.5655E+04, &
      1.180797E-01, 7.05053E+00, 2.6214E+00, 4.9173E+04, &
      1.262272E-01, 7.15570E+00, 2.4699E+00, 4.3341E+04, &
      1.349368E-01, 7.24938E+00, 2.3217E+00, 3.8109E+04, &
      1.442475E-01, 7.33266E+00, 2.1774E+00, 3.3434E+04, &
      1.542005E-01, 7.40712E+00, 2.0380E+00, 2.9275E+04, &
      1.648404E-01, 7.47445E+00, 1.8939E+00, 2.5449E+04, &
      1.762144E-01, 7.52489E+00, 1.7353E+00, 2.1813E+04, &
      1.883732E-01, 7.55803E+00, 1.5870E+00, 1.8661E+04, &
      2.013709E-01, 7.57744E+00, 1.4490E+00, 1.5939E+04, &
      2.152655E-01, 7.58557E+00, 1.3211E+00, 1.3595E+04, &
      2.301188E-01, 7.58420E+00, 1.2031E+00, 1.1581E+04, &
      2.459970E-01, 7.57462E+00, 1.0944E+00, 9.8550E+03, &
      2.629708E-01, 7.55776E+00, 9.9472E-01, 8.3793E+03, &
      2.811158E-01, 7.53420E+00, 9.0347E-01, 7.1196E+03, &
      3.005128E-01, 7.50422E+00, 8.2014E-01, 6.0460E+03, &
      3.212482E-01, 7.46776E+00, 7.4419E-01, 5.1323E+03, &
      3.434143E-01, 7.42436E+00, 6.7509E-01, 4.3553E+03, &
      3.671099E-01, 7.37312E+00, 6.1229E-01, 3.6955E+03, &
      3.924405E-01, 7.31248E+00, 5.5531E-01, 3.1356E+03, &
      4.195189E-01, 7.24000E+00, 5.0364E-01, 2.6605E+03, &
      4.484657E-01, 7.15184E+00, 4.5684E-01, 2.2578E+03, &
      4.794098E-01, 7.04174E+00, 4.1448E-01, 1.9164E+03, &
      5.124891E-01, 6.89910E+00, 3.7614E-01, 1.6273E+03, &
      5.478508E-01, 6.70400E+00, 3.4148E-01, 1.3822E+03, &
      5.856525E-01, 6.41337E+00, 3.0993E-01, 1.1737E+03, &
      6.260625E-01, 5.90422E+00, 2.7941E-01, 9.9018E+02, &
      6.692609E-01, 4.46336E+00, 2.5176E-01, 8.3491E+02, &
      6.716920E-01, 4.27069E+00, 2.5034E-01, 8.2720E+02, &
      6.819730E-01, 2.57294E+00, 2.4446E-01, 7.9565E+02, &
      6.847146E-01, 5.29790E-01, 2.4292E-01, 7.8752E+02, &
      6.888270E-01, 2.66519E+00, 4.2681E+00, 1.3726E+04, &
      6.991080E-01, 4.54067E+00, 4.1803E+00, 1.3246E+04, &
      7.154399E-01, 5.64407E+00, 4.0471E+00, 1.2531E+04, &
      7.648052E-01, 7.04400E+00, 3.6853E+00, 1.0675E+04, &
      8.175768E-01, 7.76036E+00, 3.3518E+00, 9.0822E+03, &
      8.739896E-01, 8.22518E+00, 3.0432E+00, 7.7141E+03, &
      9.342948E-01, 8.55284E+00, 2.7592E+00, 6.5430E+03, &
      9.987612E-01, 8.79468E+00, 2.4992E+00, 5.5441E+03, &
      1.067676E+00, 8.97744E+00, 2.2538E+00, 4.6774E+03, &
      1.141345E+00, 9.11455E+00, 2.0306E+00, 3.9424E+03, &
      1.220098E+00, 9.21735E+00, 1.8172E+00, 3.3007E+03, &
      1.304285E+00, 9.28588E+00, 1.6221E+00, 2.7564E+03, &
      1.394281E+00, 9.33097E+00, 1.4469E+00, 2.3002E+03, &
      1.490486E+00, 9.35888E+00, 1.2898E+00, 1.9183E+03, &
      1.593329E+00, 9.37393E+00, 1.1491E+00, 1.5990E+03, &
      1.703269E+00, 9.37936E+00, 1.0233E+00, 1.3322E+03, &
      1.820795E+00, 9.37762E+00, 9.1083E-01, 1.1095E+03, &
      1.946430E+00, 9.37061E+00, 8.1050E-01, 9.2379E+02, &
      2.080733E+00, 9.36013E+00, 7.2104E-01, 7.6898E+02, &
      2.224304E+00, 9.34689E+00, 6.4132E-01, 6.4000E+02, &
      2.377781E+00, 9.33207E+00, 5.7033E-01, 5.3261E+02, &
      2.541848E+00, 9.31652E+00, 5.0714E-01, 4.4321E+02, &
      2.717235E+00, 9.30109E+00, 4.5093E-01, 3.6882E+02, &
      2.904724E+00, 9.28588E+00, 3.9862E-01, 3.0515E+02, &
      3.105150E+00, 9.26977E+00, 3.5055E-01, 2.5119E+02, &
      3.319406E+00, 9.25188E+00, 3.0702E-01, 2.0596E+02, &
      3.548445E+00, 9.23378E+00, 2.6891E-01, 1.6889E+02, &
      3.793288E+00, 9.21598E+00, 2.3553E-01, 1.3852E+02, &
      4.055024E+00, 9.19881E+00, 2.0629E-01, 1.1362E+02, &
      4.334821E+00, 9.18245E+00, 1.8068E-01, 9.3212E+01, &
      4.633924E+00, 9.16700E+00, 1.5825E-01, 7.6488E+01, &
      4.953664E+00, 9.15253E+00, 1.3861E-01, 6.2779E+01, &
      5.295467E+00, 9.13905E+00, 1.2141E-01, 5.1541E+01, &
      5.660855E+00, 9.12654E+00, 1.0634E-01, 4.2327E+01, &
      6.051453E+00, 9.11500E+00, 9.3147E-02, 3.4772E+01, &
      6.469004E+00, 9.10438E+00, 8.1590E-02, 2.8578E+01, &
      6.915365E+00, 9.09464E+00, 7.1467E-02, 2.3497E+01, &
      7.392525E+00, 9.08575E+00, 6.2601E-02, 1.9331E+01, &
      7.902609E+00, 9.07766E+00, 5.4836E-02, 1.5911E+01, &
      8.447890E+00, 9.07041E+00, 4.8034E-02, 1.3107E+01, &
      9.030794E+00, 9.06412E+00, 4.1998E-02, 1.0786E+01, &
      9.653919E+00, 9.05787E+00, 3.6478E-02, 8.8287E+00, &
      1.032004E+01, 9.05214E+00, 3.1684E-02, 7.2354E+00, &
      1.103212E+01, 9.04693E+00, 2.7520E-02, 5.9381E+00, &
      1.179334E+01, 9.04219E+00, 2.3903E-02, 4.8815E+00, &
      1.260708E+01, 9.03790E+00, 2.0762E-02, 4.0207E+00, &
      1.347697E+01, 9.03403E+00, 1.7980E-02, 3.3102E+00, &
      1.440688E+01, 9.03050E+00, 1.5549E-02, 2.7294E+00, &
      1.540095E+01, 9.02731E+00, 1.3445E-02, 2.2575E+00, &
      1.646362E+01, 9.02442E+00, 1.1626E-02, 1.8740E+00, &
      1.759961E+01, 9.02183E+00, 1.0053E-02, 1.5623E+00, &
      1.881398E+01, 9.01949E+00, 8.6933E-03, 1.3087E+00, &
      2.011215E+01, 9.01739E+00, 7.5172E-03, 1.1024E+00, &
      2.149988E+01, 9.01551E+00, 6.5003E-03, 9.3423E-01, &
      2.298338E+01, 9.01382E+00, 5.6210E-03, 7.9708E-01, &
      2.456923E+01, 9.01231E+00, 4.8606E-03, 6.8509E-01, &
      2.626450E+01, 9.01096E+00, 4.2031E-03, 5.9351E-01, &
      2.807676E+01, 9.00976E+00, 3.6202E-03, 5.1737E-01, &
      3.001405E+01, 9.00868E+00, 3.1141E-03, 4.5481E-01, &
      3.208502E+01, 9.00772E+00, 2.6787E-03, 4.0360E-01, &
      3.429889E+01, 9.00685E+00, 2.3042E-03, 3.6155E-01, &
      3.666551E+01, 9.00608E+00, 1.9821E-03, 3.2692E-01, &
      3.919543E+01, 9.00539E+00, 1.7050E-03, 2.9826E-01, &
      4.189992E+01, 9.00478E+00, 1.4667E-03, 2.7443E-01, &
      4.479101E+01, 9.00424E+00, 1.2616E-03, 2.5452E-01, &
      4.788159E+01, 9.00375E+00, 1.0853E-03, 2.3776E-01, &
      5.118542E+01, 9.00332E+00, 9.3356E-04, 2.2357E-01, &
      5.471721E+01, 9.00294E+00, 8.0306E-04, 2.1144E-01, &
      5.849270E+01, 9.00260E+00, 6.9081E-04, 2.0098E-01, &
      6.252870E+01, 9.00230E+00, 5.9424E-04, 1.9187E-01, &
      6.684318E+01, 9.00204E+00, 5.1118E-04, 1.8387E-01, &
      7.145536E+01, 9.00180E+00, 4.3973E-04, 1.7675E-01, &
      7.638578E+01, 9.00159E+00, 3.7826E-04, 1.7036E-01, &
      8.165640E+01, 9.00140E+00, 3.2542E-04, 1.6456E-01, &
      8.729069E+01, 9.00124E+00, 2.8001E-04, 1.5924E-01, &
      9.331374E+01, 9.00110E+00, 2.4094E-04, 1.5432E-01, &
      9.975239E+01, 9.00097E+00, 2.0732E-04, 1.4972E-01, &
      1.066353E+02, 9.00085E+00, 1.7839E-04, 1.4541E-01, &
      1.139931E+02, 9.00075E+00, 1.5350E-04, 1.4131E-01, &
      1.218587E+02, 9.00066E+00, 1.3208E-04, 1.3741E-01, &
      1.302669E+02, 9.00059E+00, 1.1365E-04, 1.3367E-01, &
      1.392553E+02, 9.00052E+00, 9.7791E-05, 1.3009E-01, &
      1.488640E+02, 9.00046E+00, 8.4146E-05, 1.2663E-01, &
      1.591356E+02, 9.00040E+00, 7.2405E-05, 1.2329E-01, &
      1.701159E+02, 9.00035E+00, 6.2302E-05, 1.2006E-01, &
      1.818539E+02, 9.00031E+00, 5.3609E-05, 1.1692E-01, &
      1.944018E+02, 9.00027E+00, 4.6129E-05, 1.1388E-01, &
      2.078156E+02, 9.00024E+00, 3.9693E-05, 1.1093E-01, &
      2.221548E+02, 9.00021E+00, 3.4155E-05, 1.0806E-01, &
      2.374835E+02, 9.00019E+00, 2.9389E-05, 1.0528E-01, &
      2.538699E+02, 9.00017E+00, 2.5289E-05, 1.0259E-01, &
      2.713869E+02, 9.00015E+00, 2.1760E-05, 9.9972E-02, &
      2.901126E+02, 9.00013E+00, 1.8724E-05, 9.7438E-02, &
      3.101304E+02, 9.00011E+00, 1.6112E-05, 9.4986E-02, &
      3.315294E+02, 9.00010E+00, 1.3864E-05, 9.2617E-02, &
      3.544049E+02, 9.00009E+00, 1.1930E-05, 9.0327E-02, &
      3.788588E+02, 9.00008E+00, 1.0265E-05, 8.8119E-02, &
      4.050001E+02, 9.00007E+00, 8.8331E-06, 8.5992E-02, &
      4.329451E+02, 9.00006E+00, 7.6007E-06, 8.3946E-02]
   case(10)
      frel = -6.6000E-03
      allocate(nist_tab(700))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.14761E-01, 0.0000E+00, 1.5256E-04, &
      1.142761E-02, 4.33365E-01, 0.0000E+00, 2.0627E-04, &
      1.221612E-02, 3.33357E-01, 0.0000E+00, 2.7730E-04, &
      1.305903E-02, 2.08132E-01, 0.0000E+00, 3.7071E-04, &
      1.396010E-02, 4.70096E-02, 0.0000E+00, 4.9283E-04, &
      1.492335E-02, -1.69204E-01, 0.0000E+00, 6.5155E-04, &
      1.595306E-02, -4.81386E-01, 0.0000E+00, 8.5665E-04, &
      1.705382E-02, -1.01047E+00, 0.0000E+00, 1.1202E-03, &
      1.793400E-02, -1.97188E+00, 0.0000E+00, 1.3664E-03, &
      1.820850E-02, -2.99087E+00, 0.0000E+00, 1.4500E-03, &
      1.823053E-02, -3.18853E+00, 0.0000E+00, 1.4568E-03, &
      1.828170E-02, -4.13413E+00, 0.0000E+00, 1.4728E-03, &
      1.839150E-02, -3.04138E+00, 2.2083E+00, 2.5039E+05, &
      1.866600E-02, -2.12394E+00, 2.2523E+00, 2.5163E+05, &
      1.948844E-02, -1.40016E+00, 2.3856E+00, 2.5527E+05, &
      2.083314E-02, -9.64316E-01, 2.6068E+00, 2.6094E+05, &
      2.227063E-02, -6.95045E-01, 2.8426E+00, 2.6617E+05, &
      2.380730E-02, -4.70803E-01, 3.0893E+00, 2.7060E+05, &
      2.545001E-02, -2.54421E-01, 3.3431E+00, 2.7393E+05, &
      2.720606E-02, -2.93826E-02, 3.5995E+00, 2.7590E+05, &
      2.908327E-02, 2.12777E-01, 3.8537E+00, 2.7632E+05, &
      3.109002E-02, 4.76291E-01, 4.1006E+00, 2.7505E+05, &
      3.323523E-02, 7.62562E-01, 4.3349E+00, 2.7200E+05, &
      3.552846E-02, 1.07065E+00, 4.5515E+00, 2.6715E+05, &
      3.797993E-02, 1.39717E+00, 4.7454E+00, 2.6055E+05, &
      4.060054E-02, 1.73490E+00, 4.9122E+00, 2.5231E+05, &
      4.340198E-02, 2.06269E+00, 5.0483E+00, 2.4256E+05, &
      4.410000E-02, 2.12873E+00, 5.0760E+00, 2.4003E+05, &
      4.477500E-02, 2.15951E+00, 5.1005E+00, 2.3755E+05, &
      4.495500E-02, 2.12160E+00, 5.1067E+00, 2.3689E+05, &
      4.522500E-02, 2.21023E+00, 5.2321E+00, 2.4126E+05, &
      4.590000E-02, 2.33986E+00, 5.2606E+00, 2.3900E+05, &
      4.639671E-02, 2.41484E+00, 5.2805E+00, 2.3734E+05, &
      4.959809E-02, 2.83688E+00, 5.3892E+00, 2.2659E+05, &
      5.302035E-02, 3.25250E+00, 5.4705E+00, 2.1516E+05, &
      5.667876E-02, 3.67274E+00, 5.5205E+00, 2.0311E+05, &
      6.058959E-02, 4.09378E+00, 5.5384E+00, 1.9062E+05, &
      6.477028E-02, 4.51362E+00, 5.5237E+00, 1.7784E+05, &
      6.923942E-02, 4.92887E+00, 5.4767E+00, 1.6495E+05, &
      7.401695E-02, 5.33657E+00, 5.3983E+00, 1.5209E+05, &
      7.912411E-02, 5.73279E+00, 5.2860E+00, 1.3932E+05, &
      8.458368E-02, 6.11057E+00, 5.1415E+00, 1.2676E+05, &
      9.041995E-02, 6.46494E+00, 4.9692E+00, 1.1460E+05, &
      9.665893E-02, 6.79261E+00, 4.7741E+00, 1.0300E+05, &
      1.033284E-01, 7.09156E+00, 4.5611E+00, 9.2050E+04, &
      1.104581E-01, 7.36082E+00, 4.3350E+00, 8.1840E+04, &
      1.180797E-01, 7.60028E+00, 4.1003E+00, 7.2414E+04, &
      1.262272E-01, 7.81058E+00, 3.8613E+00, 6.3791E+04, &
      1.349368E-01, 7.99292E+00, 3.6215E+00, 5.5968E+04, &
      1.442475E-01, 8.14807E+00, 3.3842E+00, 4.8925E+04, &
      1.542005E-01, 8.27968E+00, 3.1520E+00, 4.2627E+04, &
      1.648404E-01, 8.38901E+00, 2.9270E+00, 3.7030E+04, &
      1.762144E-01, 8.47827E+00, 2.7109E+00, 3.2082E+04, &
      1.883732E-01, 8.54971E+00, 2.5047E+00, 2.7729E+04, &
      2.013709E-01, 8.60523E+00, 2.3089E+00, 2.3911E+04, &
      2.152655E-01, 8.64653E+00, 2.1241E+00, 2.0578E+04, &
      2.301188E-01, 8.67533E+00, 1.9506E+00, 1.7678E+04, &
      2.459970E-01, 8.69321E+00, 1.7886E+00, 1.5163E+04, &
      2.629708E-01, 8.70161E+00, 1.6378E+00, 1.2989E+04, &
      2.811158E-01, 8.70179E+00, 1.4980E+00, 1.1113E+04, &
      3.005128E-01, 8.69485E+00, 1.3688E+00, 9.4995E+03, &
      3.212482E-01, 8.68175E+00, 1.2497E+00, 8.1134E+03, &
      3.434143E-01, 8.66338E+00, 1.1402E+00, 6.9247E+03, &
      3.671099E-01, 8.63782E+00, 1.0288E+00, 5.8453E+03, &
      3.924405E-01, 8.60277E+00, 9.2751E-01, 4.9300E+03, &
      4.195189E-01, 8.55918E+00, 8.3571E-01, 4.1555E+03, &
      4.484657E-01, 8.50711E+00, 7.5265E-01, 3.5013E+03, &
      4.794098E-01, 8.44583E+00, 6.7764E-01, 2.9492E+03, &
      5.124891E-01, 8.37371E+00, 6.0999E-01, 2.4837E+03, &
      5.478508E-01, 8.28790E+00, 5.4905E-01, 2.0916E+03, &
      5.856525E-01, 8.18372E+00, 4.9421E-01, 1.7616E+03, &
      6.260625E-01, 8.05325E+00, 4.4490E-01, 1.4837E+03, &
      6.692609E-01, 7.88235E+00, 4.0059E-01, 1.2500E+03, &
      7.154399E-01, 7.64279E+00, 3.6079E-01, 1.0535E+03, &
      7.648052E-01, 7.26655E+00, 3.2506E-01, 8.8824E+02, &
      8.175768E-01, 6.51081E+00, 2.9299E-01, 7.4926E+02, &
      8.495620E-01, 5.31790E+00, 2.7594E-01, 6.7928E+02, &
      8.625655E-01, 3.62497E+00, 2.6917E-01, 6.5273E+02, &
      8.660331E-01, 1.59294E+00, 2.6742E-01, 6.4589E+02, &
      8.712345E-01, 3.70829E+00, 4.2677E+00, 1.0217E+04, &
      8.739896E-01, 4.35896E+00, 4.2510E+00, 1.0145E+04, &
      8.842380E-01, 5.56519E+00, 4.1899E+00, 9.8833E+03, &
      9.342948E-01, 7.48184E+00, 3.9130E+00, 8.7359E+03, &
      9.987612E-01, 8.47076E+00, 3.5924E+00, 7.5026E+03, &
      1.067676E+00, 9.07296E+00, 3.2552E+00, 6.3599E+03, &
      1.141345E+00, 9.47400E+00, 2.9372E+00, 5.3686E+03, &
      1.220098E+00, 9.75622E+00, 2.6424E+00, 4.5183E+03, &
      1.304285E+00, 9.95856E+00, 2.3724E+00, 3.7951E+03, &
      1.394281E+00, 1.01050E+01, 2.1276E+00, 3.1841E+03, &
      1.490486E+00, 1.02131E+01, 1.9048E+00, 2.6670E+03, &
      1.593329E+00, 1.02871E+01, 1.6988E+00, 2.2253E+03, &
      1.703269E+00, 1.03356E+01, 1.5147E+00, 1.8564E+03, &
      1.820795E+00, 1.03659E+01, 1.3502E+00, 1.5482E+03, &
      1.946430E+00, 1.03828E+01, 1.2033E+00, 1.2910E+03, &
      2.080733E+00, 1.03896E+01, 1.0721E+00, 1.0762E+03, &
      2.224304E+00, 1.03890E+01, 9.5490E-01, 8.9693E+02, &
      2.377781E+00, 1.03830E+01, 8.5029E-01, 7.4735E+02, &
      2.541848E+00, 1.03731E+01, 7.5695E-01, 6.2258E+02, &
      2.717235E+00, 1.03599E+01, 6.7370E-01, 5.1855E+02, &
      2.904724E+00, 1.03456E+01, 5.9950E-01, 4.3185E+02, &
      3.105150E+00, 1.03349E+01, 5.3135E-01, 3.5825E+02, &
      3.319406E+00, 1.03186E+01, 4.6926E-01, 2.9614E+02, &
      3.548445E+00, 1.03011E+01, 4.1343E-01, 2.4425E+02, &
      3.793288E+00, 1.02822E+01, 3.6257E-01, 2.0054E+02, &
      4.055024E+00, 1.02628E+01, 3.1798E-01, 1.6469E+02, &
      4.334821E+00, 1.02434E+01, 2.7888E-01, 1.3527E+02, &
      4.633924E+00, 1.02247E+01, 2.4459E-01, 1.1112E+02, &
      4.953664E+00, 1.02067E+01, 2.1452E-01, 9.1302E+01, &
      5.295467E+00, 1.01896E+01, 1.8815E-01, 7.5036E+01, &
      5.660855E+00, 1.01736E+01, 1.6502E-01, 6.1683E+01, &
      6.051453E+00, 1.01586E+01, 1.4474E-01, 5.0722E+01, &
      6.469004E+00, 1.01447E+01, 1.2696E-01, 4.1720E+01, &
      6.915365E+00, 1.01319E+01, 1.1136E-01, 3.4330E+01, &
      7.392525E+00, 1.01200E+01, 9.7675E-02, 2.8260E+01, &
      7.902609E+00, 1.01091E+01, 8.5676E-02, 2.3275E+01, &
      8.447890E+00, 1.00992E+01, 7.5151E-02, 1.9179E+01, &
      9.030794E+00, 1.00903E+01, 6.5788E-02, 1.5784E+01, &
      9.653919E+00, 1.00818E+01, 5.7186E-02, 1.2911E+01, &
      1.032004E+01, 1.00740E+01, 4.9709E-02, 1.0571E+01, &
      1.103212E+01, 1.00668E+01, 4.3210E-02, 8.6645E+00, &
      1.179334E+01, 1.00602E+01, 3.7561E-02, 7.1109E+00, &
      1.260708E+01, 1.00542E+01, 3.2650E-02, 5.8444E+00, &
      1.347697E+01, 1.00487E+01, 2.8382E-02, 4.8117E+00, &
      1.440688E+01, 1.00438E+01, 2.4672E-02, 3.9694E+00, &
      1.540095E+01, 1.00393E+01, 2.1447E-02, 3.2820E+00, &
      1.646362E+01, 1.00353E+01, 1.8630E-02, 2.7192E+00, &
      1.759961E+01, 1.00316E+01, 1.6125E-02, 2.2529E+00, &
      1.881398E+01, 1.00283E+01, 1.3956E-02, 1.8734E+00, &
      2.011215E+01, 1.00253E+01, 1.2079E-02, 1.5646E+00, &
      2.149988E+01, 1.00226E+01, 1.0454E-02, 1.3128E+00, &
      2.298338E+01, 1.00202E+01, 9.0465E-03, 1.1075E+00, &
      2.456923E+01, 1.00180E+01, 7.8288E-03, 9.4000E-01, &
      2.626450E+01, 1.00161E+01, 6.7750E-03, 8.0316E-01, &
      2.807676E+01, 1.00144E+01, 5.8415E-03, 6.8965E-01, &
      3.001405E+01, 1.00128E+01, 5.0307E-03, 5.9659E-01, &
      3.208502E+01, 1.00114E+01, 4.3323E-03, 5.2058E-01, &
      3.429889E+01, 1.00101E+01, 3.7310E-03, 4.5836E-01, &
      3.666551E+01, 1.00090E+01, 3.2131E-03, 4.0732E-01, &
      3.919543E+01, 1.00080E+01, 2.7671E-03, 3.6531E-01, &
      4.189992E+01, 1.00071E+01, 2.3830E-03, 3.3062E-01, &
      4.479101E+01, 1.00063E+01, 2.0523E-03, 3.0186E-01, &
      4.788159E+01, 1.00056E+01, 1.7674E-03, 2.7789E-01, &
      5.118542E+01, 1.00050E+01, 1.5221E-03, 2.5781E-01, &
      5.471721E+01, 1.00044E+01, 1.3109E-03, 2.4088E-01, &
      5.849270E+01, 1.00039E+01, 1.1289E-03, 2.2651E-01, &
      6.252870E+01, 1.00034E+01, 9.7225E-04, 2.1420E-01, &
      6.684318E+01, 1.00031E+01, 8.3732E-04, 2.0358E-01, &
      7.145536E+01, 1.00027E+01, 7.2111E-04, 1.9432E-01, &
      7.638578E+01, 1.00024E+01, 6.2104E-04, 1.8616E-01, &
      8.165640E+01, 1.00021E+01, 5.3516E-04, 1.7893E-01, &
      8.729069E+01, 1.00019E+01, 4.6175E-04, 1.7243E-01, &
      9.331374E+01, 1.00016E+01, 3.9840E-04, 1.6653E-01, &
      9.975239E+01, 1.00015E+01, 3.4375E-04, 1.6113E-01, &
      1.066353E+02, 1.00013E+01, 2.9660E-04, 1.5611E-01, &
      1.139931E+02, 1.00011E+01, 2.5591E-04, 1.5143E-01, &
      1.218587E+02, 1.00010E+01, 2.2081E-04, 1.4704E-01, &
      1.302669E+02, 1.00009E+01, 1.9052E-04, 1.4287E-01, &
      1.392553E+02, 1.00008E+01, 1.6439E-04, 1.3891E-01, &
      1.488640E+02, 1.00007E+01, 1.4184E-04, 1.3512E-01, &
      1.591356E+02, 1.00006E+01, 1.2238E-04, 1.3147E-01, &
      1.701159E+02, 1.00005E+01, 1.0560E-04, 1.2796E-01, &
      1.818539E+02, 1.00005E+01, 9.1113E-05, 1.2456E-01, &
      1.944018E+02, 1.00004E+01, 7.8615E-05, 1.2128E-01, &
      2.078156E+02, 1.00004E+01, 6.7832E-05, 1.1810E-01, &
      2.221548E+02, 1.00003E+01, 5.8528E-05, 1.1501E-01, &
      2.374835E+02, 1.00003E+01, 5.0500E-05, 1.1201E-01, &
      2.538699E+02, 1.00003E+01, 4.3573E-05, 1.0910E-01, &
      2.713869E+02, 1.00002E+01, 3.7597E-05, 1.0628E-01, &
      2.901126E+02, 1.00002E+01, 3.2440E-05, 1.0353E-01, &
      3.101304E+02, 1.00002E+01, 2.7990E-05, 1.0087E-01, &
      3.315294E+02, 1.00002E+01, 2.4151E-05, 9.8281E-02, &
      3.544049E+02, 1.00001E+01, 2.0839E-05, 9.5777E-02, &
      3.788588E+02, 1.00001E+01, 1.7980E-05, 9.3352E-02, &
      4.050001E+02, 1.00001E+01, 1.5514E-05, 9.1006E-02, &
      4.329451E+02, 1.00001E+01, 1.3386E-05, 8.8739E-02]
   case(11)
      frel = -8.4000E-03
      allocate(nist_tab(692))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.48693E+00, 0.0000E+00, 6.6576E-05, &
      1.142761E-02, 1.46241E+00, 0.0000E+00, 9.1556E-05, &
      1.221612E-02, 1.43394E+00, 0.0000E+00, 1.2517E-04, &
      1.305903E-02, 1.40080E+00, 0.0000E+00, 1.7014E-04, &
      1.396010E-02, 1.36209E+00, 0.0000E+00, 2.2993E-04, &
      1.492335E-02, 1.31670E+00, 0.0000E+00, 3.0896E-04, &
      1.595306E-02, 1.26325E+00, 0.0000E+00, 4.1278E-04, &
      1.705382E-02, 1.19992E+00, 0.0000E+00, 5.4837E-04, &
      1.823053E-02, 1.12436E+00, 0.0000E+00, 7.2441E-04, &
      1.948844E-02, 1.03337E+00, 0.0000E+00, 9.5163E-04, &
      2.083314E-02, 9.22475E-01, 0.0000E+00, 1.2432E-03, &
      2.227063E-02, 7.85105E-01, 0.0000E+00, 1.6152E-03, &
      2.380730E-02, 6.10932E-01, 0.0000E+00, 2.0870E-03, &
      2.545001E-02, 3.82047E-01, 0.0000E+00, 2.6821E-03, &
      2.720606E-02, 6.19150E-02, 0.0000E+00, 3.4282E-03, &
      2.908327E-02, -4.52629E-01, 0.0000E+00, 4.3585E-03, &
      3.047800E-02, -1.22048E+00, 0.0000E+00, 5.1423E-03, &
      3.094450E-02, -1.96928E+00, 0.0000E+00, 5.4225E-03, &
      3.109002E-02, -3.28343E+00, 0.0000E+00, 5.5118E-03, &
      3.125550E-02, -2.07435E+00, 1.4752E+00, 8.6393E+04, &
      3.172200E-02, -1.54870E+00, 1.5580E+00, 8.9898E+04, &
      3.323523E-02, -1.24532E+00, 1.8498E+00, 1.0188E+05, &
      3.552846E-02, -1.15725E+00, 2.3528E+00, 1.2122E+05, &
      3.797993E-02, -1.06542E+00, 2.9049E+00, 1.4000E+05, &
      4.060054E-02, -9.09350E-01, 3.4793E+00, 1.5686E+05, &
      4.340198E-02, -6.70261E-01, 4.0538E+00, 1.7096E+05, &
      4.639671E-02, -3.44944E-01, 4.6061E+00, 1.8172E+05, &
      4.959809E-02, 6.01422E-02, 5.1157E+00, 1.8879E+05, &
      5.302035E-02, 5.30816E-01, 5.5651E+00, 1.9212E+05, &
      5.667876E-02, 1.04425E+00, 5.9411E+00, 1.9186E+05, &
      6.058959E-02, 1.55378E+00, 6.2352E+00, 1.8836E+05, &
      6.203400E-02, 1.69884E+00, 6.3186E+00, 1.8644E+05, &
      6.298350E-02, 1.71644E+00, 6.3666E+00, 1.8502E+05, &
      6.323670E-02, 1.61823E+00, 6.3785E+00, 1.8463E+05, &
      6.361650E-02, 1.79758E+00, 6.6559E+00, 1.9151E+05, &
      6.456600E-02, 2.03826E+00, 6.7090E+00, 1.9019E+05, &
      6.477028E-02, 2.07828E+00, 6.7198E+00, 1.8990E+05, &
      6.923942E-02, 2.80061E+00, 6.9123E+00, 1.8273E+05, &
      7.401695E-02, 3.48119E+00, 7.0330E+00, 1.7392E+05, &
      7.912411E-02, 4.14972E+00, 7.0772E+00, 1.6372E+05, &
      8.458368E-02, 4.79962E+00, 7.0494E+00, 1.5255E+05, &
      9.041995E-02, 5.42266E+00, 6.9560E+00, 1.4081E+05, &
      9.665893E-02, 6.01127E+00, 6.8045E+00, 1.2885E+05, &
      1.033284E-01, 6.56006E+00, 6.6036E+00, 1.1698E+05, &
      1.104581E-01, 7.06488E+00, 6.3621E+00, 1.0543E+05, &
      1.180797E-01, 7.52371E+00, 6.0888E+00, 9.4384E+04, &
      1.262272E-01, 7.93649E+00, 5.7918E+00, 8.3986E+04, &
      1.349368E-01, 8.30337E+00, 5.4758E+00, 7.4278E+04, &
      1.442475E-01, 8.62338E+00, 5.1475E+00, 6.5318E+04, &
      1.542005E-01, 8.89786E+00, 4.8145E+00, 5.7149E+04, &
      1.648404E-01, 9.12956E+00, 4.4830E+00, 4.9779E+04, &
      1.762144E-01, 9.32200E+00, 4.1579E+00, 4.3189E+04, &
      1.883732E-01, 9.47902E+00, 3.8431E+00, 3.7342E+04, &
      2.013709E-01, 9.60562E+00, 3.5414E+00, 3.2191E+04, &
      2.152655E-01, 9.70360E+00, 3.2547E+00, 2.7676E+04, &
      2.301188E-01, 9.77764E+00, 2.9844E+00, 2.3740E+04, &
      2.459970E-01, 9.83116E+00, 2.7312E+00, 2.0323E+04, &
      2.629708E-01, 9.86727E+00, 2.4950E+00, 1.7367E+04, &
      2.811158E-01, 9.88851E+00, 2.2755E+00, 1.4817E+04, &
      3.005128E-01, 9.89698E+00, 2.0722E+00, 1.2622E+04, &
      3.212482E-01, 9.89457E+00, 1.8846E+00, 1.0739E+04, &
      3.434143E-01, 9.88287E+00, 1.7121E+00, 9.1265E+03, &
      3.671099E-01, 9.86321E+00, 1.5539E+00, 7.7485E+03, &
      3.924405E-01, 9.83660E+00, 1.4090E+00, 6.5730E+03, &
      4.195189E-01, 9.80379E+00, 1.2767E+00, 5.5716E+03, &
      4.484657E-01, 9.76525E+00, 1.1561E+00, 4.7199E+03, &
      4.794098E-01, 9.72113E+00, 1.0463E+00, 3.9962E+03, &
      5.124891E-01, 9.67130E+00, 9.4647E-01, 3.3818E+03, &
      5.478508E-01, 9.61523E+00, 8.5583E-01, 2.8609E+03, &
      5.856525E-01, 9.55196E+00, 7.7342E-01, 2.4188E+03, &
      6.260625E-01, 9.47867E+00, 6.9457E-01, 2.0323E+03, &
      6.692609E-01, 9.39178E+00, 6.2355E-01, 1.7071E+03, &
      7.154399E-01, 9.28740E+00, 5.5968E-01, 1.4336E+03, &
      7.648052E-01, 9.15862E+00, 5.0234E-01, 1.2040E+03, &
      8.175768E-01, 8.99314E+00, 4.5092E-01, 1.0113E+03, &
      8.739896E-01, 8.76706E+00, 4.0486E-01, 8.4972E+02, &
      9.342948E-01, 8.42597E+00, 3.6364E-01, 7.1425E+02, &
      9.987612E-01, 7.79970E+00, 3.2676E-01, 6.0071E+02, &
      1.050658E+00, 6.52412E+00, 2.9882E-01, 5.2246E+02, &
      1.067676E+00, 4.91181E+00, 2.9049E-01, 4.9990E+02, &
      1.071028E+00, 3.52095E+00, 2.8890E-01, 4.9562E+02, &
      1.077461E+00, 4.89062E+00, 3.3468E+00, 5.6874E+03, &
      1.093542E+00, 6.12496E+00, 3.6044E+00, 6.0350E+03, &
      1.141345E+00, 7.70749E+00, 4.2220E+00, 6.7728E+03, &
      1.220098E+00, 9.32686E+00, 4.0897E+00, 6.1373E+03, &
      1.304285E+00, 1.02655E+01, 3.5285E+00, 4.9536E+03, &
      1.394281E+00, 1.06769E+01, 2.9168E+00, 3.8310E+03, &
      1.490486E+00, 1.08500E+01, 2.5451E+00, 3.1274E+03, &
      1.593329E+00, 1.09680E+01, 2.2568E+00, 2.5945E+03, &
      1.703269E+00, 1.10540E+01, 2.0296E+00, 2.1828E+03, &
      1.820795E+00, 1.11263E+01, 1.8413E+00, 1.8528E+03, &
      1.946430E+00, 1.11914E+01, 1.6757E+00, 1.5776E+03, &
      2.080733E+00, 1.12505E+01, 1.5228E+00, 1.3413E+03, &
      2.224304E+00, 1.13511E+01, 1.3668E+00, 1.1265E+03, &
      2.377781E+00, 1.13777E+01, 1.2171E+00, 9.3855E+02, &
      2.541848E+00, 1.13894E+01, 1.0838E+00, 7.8203E+02, &
      2.717235E+00, 1.13918E+01, 9.6513E-01, 6.5167E+02, &
      2.904724E+00, 1.13880E+01, 8.5948E-01, 5.4309E+02, &
      3.105150E+00, 1.13852E+01, 7.6236E-01, 4.5083E+02, &
      3.319406E+00, 1.13721E+01, 6.7370E-01, 3.7287E+02, &
      3.548445E+00, 1.13556E+01, 5.9538E-01, 3.0843E+02, &
      3.793288E+00, 1.13373E+01, 5.2617E-01, 2.5517E+02, &
      4.055024E+00, 1.13182E+01, 4.6503E-01, 2.1112E+02, &
      4.334821E+00, 1.12991E+01, 4.1100E-01, 1.7470E+02, &
      4.633924E+00, 1.12804E+01, 3.6326E-01, 1.4459E+02, &
      4.953664E+00, 1.12619E+01, 3.1883E-01, 1.1885E+02, &
      5.295467E+00, 1.12430E+01, 2.7975E-01, 9.7689E+01, &
      5.660855E+00, 1.12245E+01, 2.4548E-01, 8.0311E+01, &
      6.051453E+00, 1.12068E+01, 2.1540E-01, 6.6041E+01, &
      6.469004E+00, 1.11899E+01, 1.8902E-01, 5.4321E+01, &
      6.915365E+00, 1.11740E+01, 1.6587E-01, 4.4694E+01, &
      7.392525E+00, 1.11591E+01, 1.4555E-01, 3.6786E+01, &
      7.902609E+00, 1.11453E+01, 1.2773E-01, 3.0289E+01, &
      8.447890E+00, 1.11325E+01, 1.1209E-01, 2.4950E+01, &
      9.030794E+00, 1.11217E+01, 9.8173E-02, 2.0522E+01, &
      9.653919E+00, 1.11112E+01, 8.5365E-02, 1.6773E+01, &
      1.032004E+01, 1.11009E+01, 7.4228E-02, 1.3718E+01, &
      1.103212E+01, 1.10913E+01, 6.4546E-02, 1.1230E+01, &
      1.179334E+01, 1.10825E+01, 5.6127E-02, 9.2018E+00, &
      1.260708E+01, 1.10744E+01, 4.8806E-02, 7.5485E+00, &
      1.347697E+01, 1.10671E+01, 4.2441E-02, 6.2003E+00, &
      1.440688E+01, 1.10604E+01, 3.6907E-02, 5.1006E+00, &
      1.540095E+01, 1.10543E+01, 3.2094E-02, 4.2033E+00, &
      1.646362E+01, 1.10487E+01, 2.7909E-02, 3.4711E+00, &
      1.759961E+01, 1.10438E+01, 2.4270E-02, 2.8730E+00, &
      1.881398E+01, 1.10393E+01, 2.1106E-02, 2.3845E+00, &
      2.011215E+01, 1.10352E+01, 1.8354E-02, 1.9853E+00, &
      2.149988E+01, 1.10315E+01, 1.5961E-02, 1.6589E+00, &
      2.298338E+01, 1.10283E+01, 1.3875E-02, 1.3913E+00, &
      2.456923E+01, 1.10253E+01, 1.2014E-02, 1.1688E+00, &
      2.626450E+01, 1.10226E+01, 1.0402E-02, 9.8731E-01, &
      2.807676E+01, 1.10202E+01, 8.9800E-03, 8.3737E-01, &
      3.001405E+01, 1.10181E+01, 7.7433E-03, 7.1459E-01, &
      3.208502E+01, 1.10161E+01, 6.6761E-03, 6.1447E-01, &
      3.429889E+01, 1.10143E+01, 5.7560E-03, 5.3276E-01, &
      3.666551E+01, 1.10128E+01, 4.9627E-03, 4.6598E-01, &
      3.919543E+01, 1.10114E+01, 4.2788E-03, 4.1129E-01, &
      4.189992E+01, 1.10101E+01, 3.6891E-03, 3.6639E-01, &
      4.479101E+01, 1.10090E+01, 3.1807E-03, 3.2944E-01, &
      4.788159E+01, 1.10080E+01, 2.7424E-03, 2.9894E-01, &
      5.118542E+01, 1.10071E+01, 2.3645E-03, 2.7367E-01, &
      5.471721E+01, 1.10063E+01, 2.0387E-03, 2.5265E-01, &
      5.849270E+01, 1.10056E+01, 1.7577E-03, 2.3506E-01, &
      6.252870E+01, 1.10049E+01, 1.5155E-03, 2.2027E-01, &
      6.684318E+01, 1.10044E+01, 1.3067E-03, 2.0774E-01, &
      7.145536E+01, 1.10039E+01, 1.1267E-03, 1.9706E-01, &
      7.638578E+01, 1.10034E+01, 9.7142E-04, 1.8786E-01, &
      8.165640E+01, 1.10030E+01, 8.3757E-04, 1.7988E-01, &
      8.729069E+01, 1.10027E+01, 7.2216E-04, 1.7286E-01, &
      9.331374E+01, 1.10024E+01, 6.2266E-04, 1.6664E-01, &
      9.975239E+01, 1.10021E+01, 5.3687E-04, 1.6106E-01, &
      1.066353E+02, 1.10019E+01, 4.6290E-04, 1.5601E-01, &
      1.139931E+02, 1.10016E+01, 3.9913E-04, 1.5137E-01, &
      1.218587E+02, 1.10015E+01, 3.4414E-04, 1.4708E-01, &
      1.302669E+02, 1.10013E+01, 2.9672E-04, 1.4306E-01, &
      1.392553E+02, 1.10011E+01, 2.5584E-04, 1.3926E-01, &
      1.488640E+02, 1.10010E+01, 2.2060E-04, 1.3565E-01, &
      1.591356E+02, 1.10009E+01, 1.9020E-04, 1.3219E-01, &
      1.701159E+02, 1.10008E+01, 1.6400E-04, 1.2884E-01, &
      1.818539E+02, 1.10007E+01, 1.4141E-04, 1.2560E-01, &
      1.944018E+02, 1.10006E+01, 1.2193E-04, 1.2245E-01, &
      2.078156E+02, 1.10005E+01, 1.0513E-04, 1.1937E-01, &
      2.221548E+02, 1.10005E+01, 9.0646E-05, 1.1636E-01, &
      2.374835E+02, 1.10004E+01, 7.8159E-05, 1.1339E-01, &
      2.538699E+02, 1.10004E+01, 6.7391E-05, 1.1049E-01, &
      2.713869E+02, 1.10003E+01, 5.8108E-05, 1.0762E-01, &
      2.901126E+02, 1.10003E+01, 5.0103E-05, 1.0480E-01, &
      3.101304E+02, 1.10002E+01, 4.3201E-05, 1.0202E-01, &
      3.315294E+02, 1.10002E+01, 3.7250E-05, 9.9293E-02, &
      3.544049E+02, 1.10002E+01, 3.2118E-05, 9.6603E-02, &
      3.788588E+02, 1.10002E+01, 2.7694E-05, 9.3956E-02, &
      4.050001E+02, 1.10001E+01, 2.3879E-05, 9.1352E-02, &
      4.329451E+02, 1.10001E+01, 2.0590E-05, 8.8791E-02]
   case(12)
      frel = -1.0800E-02
      allocate(nist_tab(696))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.46798E+00, 0.0000E+00, 9.8483E-05, &
      1.142761E-02, 2.45703E+00, 0.0000E+00, 1.3394E-04, &
      1.221612E-02, 2.44446E+00, 0.0000E+00, 1.8114E-04, &
      1.305903E-02, 2.42999E+00, 0.0000E+00, 2.4362E-04, &
      1.396010E-02, 2.41334E+00, 0.0000E+00, 3.2583E-04, &
      1.492335E-02, 2.39414E+00, 0.0000E+00, 4.3340E-04, &
      1.595306E-02, 2.37197E+00, 0.0000E+00, 5.7333E-04, &
      1.705382E-02, 2.34635E+00, 0.0000E+00, 7.5433E-04, &
      1.823053E-02, 2.31667E+00, 0.0000E+00, 9.8712E-04, &
      1.948844E-02, 2.28222E+00, 0.0000E+00, 1.2848E-03, &
      2.083314E-02, 2.24214E+00, 0.0000E+00, 1.6635E-03, &
      2.227063E-02, 2.19535E+00, 0.0000E+00, 2.1423E-03, &
      2.380730E-02, 2.14053E+00, 0.0000E+00, 2.7446E-03, &
      2.545001E-02, 2.07604E+00, 0.0000E+00, 3.4978E-03, &
      2.720606E-02, 1.99976E+00, 0.0000E+00, 4.4347E-03, &
      2.908327E-02, 1.90893E+00, 0.0000E+00, 5.5937E-03, &
      3.109002E-02, 1.79989E+00, 0.0000E+00, 7.0197E-03, &
      3.323523E-02, 1.66760E+00, 0.0000E+00, 8.7645E-03, &
      3.552846E-02, 1.50483E+00, 0.0000E+00, 1.0888E-02, &
      3.797993E-02, 1.30071E+00, 0.0000E+00, 1.3458E-02, &
      4.060054E-02, 1.03750E+00, 0.0000E+00, 1.6553E-02, &
      4.340198E-02, 6.82720E-01, 0.0000E+00, 2.0260E-02, &
      4.639671E-02, 1.63175E-01, 0.0000E+00, 2.4675E-02, &
      4.959809E-02, -7.94660E-01, 0.0000E+00, 2.9907E-02, &
      5.037200E-02, -1.23235E+00, 0.0000E+00, 3.1250E-02, &
      5.114300E-02, -2.14633E+00, 0.0000E+00, 3.2618E-02, &
      5.134860E-02, -3.06571E+00, 0.0000E+00, 3.2988E-02, &
      5.165700E-02, -2.31811E+00, 1.7107E+00, 5.7335E+04, &
      5.242800E-02, -1.77102E+00, 1.8464E+00, 6.0975E+04, &
      5.302035E-02, -1.65181E+00, 1.9565E+00, 6.3888E+04, &
      5.667876E-02, -1.58097E+00, 2.7595E+00, 8.4293E+04, &
      6.058959E-02, -1.51530E+00, 3.7482E+00, 1.0711E+05, &
      6.477028E-02, -1.24345E+00, 4.7712E+00, 1.2754E+05, &
      6.923942E-02, -7.50020E-01, 5.7446E+00, 1.4365E+05, &
      7.401695E-02, -6.57205E-02, 6.5956E+00, 1.5428E+05, &
      7.912411E-02, 7.45736E-01, 7.2727E+00, 1.5914E+05, &
      8.458368E-02, 1.57760E+00, 7.7495E+00, 1.5862E+05, &
      8.761200E-02, 1.91414E+00, 7.9183E+00, 1.5648E+05, &
      8.895300E-02, 1.87327E+00, 7.9741E+00, 1.5520E+05, &
      8.931060E-02, 1.62388E+00, 7.9871E+00, 1.5483E+05, &
      8.984700E-02, 2.01523E+00, 8.6080E+00, 1.6588E+05, &
      9.041995E-02, 2.26495E+00, 8.6311E+00, 1.6527E+05, &
      9.118800E-02, 2.49637E+00, 8.6592E+00, 1.6441E+05, &
      9.665893E-02, 3.62633E+00, 8.7766E+00, 1.5720E+05, &
      1.033284E-01, 4.71331E+00, 8.7615E+00, 1.4681E+05, &
      1.104581E-01, 5.69293E+00, 8.6076E+00, 1.3492E+05, &
      1.180797E-01, 6.56897E+00, 8.3455E+00, 1.2237E+05, &
      1.262272E-01, 7.34102E+00, 8.0043E+00, 1.0979E+05, &
      1.349368E-01, 8.01184E+00, 7.6096E+00, 9.7637E+04, &
      1.442475E-01, 8.58744E+00, 7.1826E+00, 8.6210E+04, &
      1.542005E-01, 9.07657E+00, 6.7401E+00, 7.5677E+04, &
      1.648404E-01, 9.48729E+00, 6.2947E+00, 6.6114E+04, &
      1.762144E-01, 9.82965E+00, 5.8557E+00, 5.7534E+04, &
      1.883732E-01, 1.01128E+01, 5.4298E+00, 4.9906E+04, &
      2.013709E-01, 1.03452E+01, 5.0212E+00, 4.3172E+04, &
      2.152655E-01, 1.05347E+01, 4.6320E+00, 3.7255E+04, &
      2.301188E-01, 1.06868E+01, 4.2624E+00, 3.2070E+04, &
      2.459970E-01, 1.08064E+01, 3.9136E+00, 2.7545E+04, &
      2.629708E-01, 1.08979E+01, 3.5864E+00, 2.3613E+04, &
      2.811158E-01, 1.09658E+01, 3.2807E+00, 2.0206E+04, &
      3.005128E-01, 1.10131E+01, 2.9961E+00, 1.7263E+04, &
      3.212482E-01, 1.10431E+01, 2.7321E+00, 1.4726E+04, &
      3.434143E-01, 1.10586E+01, 2.4878E+00, 1.2544E+04, &
      3.671099E-01, 1.10617E+01, 2.2623E+00, 1.0671E+04, &
      3.924405E-01, 1.10544E+01, 2.0544E+00, 9.0646E+03, &
      4.195189E-01, 1.10378E+01, 1.8629E+00, 7.6895E+03, &
      4.484657E-01, 1.10132E+01, 1.6872E+00, 6.5149E+03, &
      4.794098E-01, 1.09815E+01, 1.5262E+00, 5.5133E+03, &
      5.124891E-01, 1.09433E+01, 1.3791E+00, 4.6607E+03, &
      5.478508E-01, 1.08992E+01, 1.2450E+00, 3.9362E+03, &
      5.856525E-01, 1.08494E+01, 1.1230E+00, 3.3217E+03, &
      6.260625E-01, 1.07938E+01, 1.0123E+00, 2.8010E+03, &
      6.692609E-01, 1.07317E+01, 9.1188E-01, 2.3608E+03, &
      7.154399E-01, 1.06624E+01, 8.2108E-01, 1.9888E+03, &
      7.648052E-01, 1.05839E+01, 7.3909E-01, 1.6750E+03, &
      8.175768E-01, 1.04938E+01, 6.6515E-01, 1.4105E+03, &
      8.739896E-01, 1.03877E+01, 5.9857E-01, 1.1878E+03, &
      9.342948E-01, 1.02587E+01, 5.3870E-01, 1.0003E+03, &
      9.987612E-01, 1.00942E+01, 4.8399E-01, 8.4101E+02, &
      1.067676E+00, 9.86553E+00, 4.2897E-01, 6.9765E+02, &
      1.141345E+00, 9.51607E+00, 3.8061E-01, 5.7942E+02, &
      1.220098E+00, 8.85649E+00, 3.3816E-01, 4.8190E+02, &
      1.278900E+00, 7.57149E+00, 3.1133E-01, 4.2352E+02, &
      1.298475E+00, 5.98525E+00, 3.0318E-01, 4.0630E+02, &
      1.304285E+00, 3.41984E+00, 3.0083E-01, 4.0137E+02, &
      1.311525E+00, 5.92695E+00, 3.9592E+00, 5.2285E+03, &
      1.331100E+00, 7.57432E+00, 3.9913E+00, 5.1935E+03, &
      1.394281E+00, 9.30219E+00, 3.9993E+00, 4.9681E+03, &
      1.490486E+00, 1.04949E+01, 3.6712E+00, 4.2665E+03, &
      1.593329E+00, 1.11690E+01, 3.2546E+00, 3.5385E+03, &
      1.703269E+00, 1.15495E+01, 2.8511E+00, 2.9001E+03, &
      1.820795E+00, 1.17834E+01, 2.5397E+00, 2.4169E+03, &
      1.946430E+00, 1.19494E+01, 2.2732E+00, 2.0239E+03, &
      2.080733E+00, 1.20708E+01, 2.0422E+00, 1.7012E+03, &
      2.224304E+00, 1.21624E+01, 1.8378E+00, 1.4323E+03, &
      2.377781E+00, 1.22323E+01, 1.6539E+00, 1.2060E+03, &
      2.541848E+00, 1.22852E+01, 1.4866E+00, 1.0143E+03, &
      2.717235E+00, 1.23707E+01, 1.3291E+00, 8.4855E+02, &
      2.904724E+00, 1.23918E+01, 1.1850E+00, 7.0792E+02, &
      3.105150E+00, 1.24049E+01, 1.0522E+00, 5.8827E+02, &
      3.319406E+00, 1.24053E+01, 9.3089E-01, 4.8705E+02, &
      3.548445E+00, 1.23973E+01, 8.2358E-01, 4.0329E+02, &
      3.793288E+00, 1.23843E+01, 7.2867E-01, 3.3397E+02, &
      4.055024E+00, 1.23682E+01, 6.4473E-01, 2.7660E+02, &
      4.334821E+00, 1.23501E+01, 5.7048E-01, 2.2912E+02, &
      4.633924E+00, 1.23310E+01, 5.0479E-01, 1.8981E+02, &
      4.953664E+00, 1.23116E+01, 4.4669E-01, 1.5727E+02, &
      5.295467E+00, 1.22924E+01, 3.9528E-01, 1.3034E+02, &
      5.660855E+00, 1.22740E+01, 3.4977E-01, 1.0801E+02, &
      6.051453E+00, 1.22558E+01, 3.0719E-01, 8.8872E+01, &
      6.469004E+00, 1.22374E+01, 2.6981E-01, 7.3140E+01, &
      6.915365E+00, 1.22194E+01, 2.3698E-01, 6.0208E+01, &
      7.392525E+00, 1.22022E+01, 2.0815E-01, 4.9578E+01, &
      7.902609E+00, 1.21860E+01, 1.8283E-01, 4.0836E+01, &
      8.447890E+00, 1.21710E+01, 1.6060E-01, 3.3649E+01, &
      9.030794E+00, 1.21570E+01, 1.4079E-01, 2.7683E+01, &
      9.653919E+00, 1.21436E+01, 1.2254E-01, 2.2628E+01, &
      1.032004E+01, 1.21308E+01, 1.0666E-01, 1.8507E+01, &
      1.103212E+01, 1.21189E+01, 9.2844E-02, 1.5147E+01, &
      1.179334E+01, 1.21078E+01, 8.0816E-02, 1.2406E+01, &
      1.260708E+01, 1.20976E+01, 7.0348E-02, 1.0171E+01, &
      1.347697E+01, 1.20882E+01, 6.1236E-02, 8.3468E+00, &
      1.440688E+01, 1.20796E+01, 5.3306E-02, 6.8581E+00, &
      1.540095E+01, 1.20718E+01, 4.6403E-02, 5.6427E+00, &
      1.646362E+01, 1.20646E+01, 4.0394E-02, 4.6500E+00, &
      1.759961E+01, 1.20581E+01, 3.5164E-02, 3.8391E+00, &
      1.881398E+01, 1.20522E+01, 3.0611E-02, 3.1763E+00, &
      2.011215E+01, 1.20469E+01, 2.6648E-02, 2.6344E+00, &
      2.149988E+01, 1.20421E+01, 2.3199E-02, 2.1912E+00, &
      2.298338E+01, 1.20378E+01, 2.0196E-02, 1.8284E+00, &
      2.456923E+01, 1.20339E+01, 1.7582E-02, 1.5313E+00, &
      2.626450E+01, 1.20304E+01, 1.5306E-02, 1.2879E+00, &
      2.807676E+01, 1.20272E+01, 1.3269E-02, 1.0848E+00, &
      3.001405E+01, 1.20244E+01, 1.1454E-02, 9.1596E-01, &
      3.208502E+01, 1.20218E+01, 9.8871E-03, 7.7843E-01, &
      3.429889E+01, 1.20195E+01, 8.5347E-03, 6.6626E-01, &
      3.666551E+01, 1.20174E+01, 7.3674E-03, 5.7464E-01, &
      3.919543E+01, 1.20155E+01, 6.3597E-03, 4.9970E-01, &
      4.189992E+01, 1.20138E+01, 5.4892E-03, 4.3827E-01, &
      4.479101E+01, 1.20123E+01, 4.7372E-03, 3.8781E-01, &
      4.788159E+01, 1.20109E+01, 4.0882E-03, 3.4628E-01, &
      5.118542E+01, 1.20097E+01, 3.5282E-03, 3.1201E-01, &
      5.471721E+01, 1.20086E+01, 3.0449E-03, 2.8363E-01, &
      5.849270E+01, 1.20077E+01, 2.6278E-03, 2.6004E-01, &
      6.252870E+01, 1.20068E+01, 2.2678E-03, 2.4033E-01, &
      6.684318E+01, 1.20060E+01, 1.9572E-03, 2.2379E-01, &
      7.145536E+01, 1.20053E+01, 1.6891E-03, 2.0982E-01, &
      7.638578E+01, 1.20047E+01, 1.4577E-03, 1.9792E-01, &
      8.165640E+01, 1.20042E+01, 1.2580E-03, 1.8772E-01, &
      8.729069E+01, 1.20037E+01, 1.0857E-03, 1.7890E-01, &
      9.331374E+01, 1.20033E+01, 9.3701E-04, 1.7120E-01, &
      9.975239E+01, 1.20029E+01, 8.0867E-04, 1.6440E-01, &
      1.066353E+02, 1.20026E+01, 6.9791E-04, 1.5834E-01, &
      1.139931E+02, 1.20023E+01, 6.0232E-04, 1.5287E-01, &
      1.218587E+02, 1.20020E+01, 5.1983E-04, 1.4789E-01, &
      1.302669E+02, 1.20018E+01, 4.4863E-04, 1.4329E-01, &
      1.392553E+02, 1.20016E+01, 3.8719E-04, 1.3901E-01, &
      1.488640E+02, 1.20014E+01, 3.3416E-04, 1.3500E-01, &
      1.591356E+02, 1.20012E+01, 2.8839E-04, 1.3119E-01, &
      1.701159E+02, 1.20011E+01, 2.4890E-04, 1.2755E-01, &
      1.818539E+02, 1.20010E+01, 2.1481E-04, 1.2406E-01, &
      1.944018E+02, 1.20008E+01, 1.8539E-04, 1.2068E-01, &
      2.078156E+02, 1.20007E+01, 1.6000E-04, 1.1739E-01, &
      2.221548E+02, 1.20007E+01, 1.3809E-04, 1.1419E-01, &
      2.374835E+02, 1.20006E+01, 1.1918E-04, 1.1105E-01, &
      2.538699E+02, 1.20005E+01, 1.0286E-04, 1.0798E-01, &
      2.713869E+02, 1.20005E+01, 8.8772E-05, 1.0496E-01, &
      2.901126E+02, 1.20004E+01, 7.6615E-05, 1.0198E-01, &
      3.101304E+02, 1.20004E+01, 6.6123E-05, 9.9053E-02, &
      3.315294E+02, 1.20003E+01, 5.7068E-05, 9.6167E-02, &
      3.544049E+02, 1.20003E+01, 4.9253E-05, 9.3324E-02, &
      3.788588E+02, 1.20002E+01, 4.2508E-05, 9.0520E-02, &
      4.050001E+02, 1.20002E+01, 3.6687E-05, 8.7759E-02, &
      4.329451E+02, 1.20002E+01, 3.1663E-05, 8.5040E-02]
   case(13)
      frel = -1.2600E-02
      allocate(nist_tab(720))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      8.208152E-03, 3.11903E+00, 0.0000E+00, 3.6085E-05, &
      8.333787E-03, 3.07927E+00, 0.0000E+00, 3.8846E-05, &
      8.367290E-03, 3.02769E+00, 0.0000E+00, 3.9608E-05, &
      8.417543E-03, 3.08902E+00, 1.0319E-01, 1.9119E+04, &
      8.543178E-03, 3.14160E+00, 9.0457E-02, 1.6513E+04, &
      1.069000E-02, 3.16731E+00, 1.8351E-02, 2.6773E+03, &
      1.142761E-02, 3.15650E+00, 1.5050E-02, 2.0540E+03, &
      1.221612E-02, 3.14495E+00, 1.3667E-02, 1.7448E+03, &
      1.305903E-02, 3.13240E+00, 1.3480E-02, 1.6099E+03, &
      1.396010E-02, 3.11858E+00, 1.4200E-02, 1.5864E+03, &
      1.492335E-02, 3.10318E+00, 1.5741E-02, 1.6451E+03, &
      1.595306E-02, 3.08594E+00, 1.8134E-02, 1.7728E+03, &
      1.705382E-02, 3.06659E+00, 2.1472E-02, 1.9636E+03, &
      1.823053E-02, 3.04487E+00, 2.5892E-02, 2.2150E+03, &
      1.948844E-02, 3.02055E+00, 3.1548E-02, 2.5247E+03, &
      2.083314E-02, 2.99341E+00, 3.8589E-02, 2.8888E+03, &
      2.227063E-02, 2.96319E+00, 4.7130E-02, 3.3005E+03, &
      2.380730E-02, 2.92964E+00, 5.7225E-02, 3.7488E+03, &
      2.545001E-02, 2.89315E+00, 6.8841E-02, 4.2186E+03, &
      2.720606E-02, 2.85179E+00, 8.1830E-02, 4.6910E+03, &
      2.908327E-02, 2.80561E+00, 9.5929E-02, 5.1442E+03, &
      3.109002E-02, 2.75366E+00, 1.1075E-01, 5.5558E+03, &
      3.323523E-02, 2.69458E+00, 1.2583E-01, 5.9046E+03, &
      3.552846E-02, 2.62647E+00, 1.4038E-01, 6.1625E+03, &
      3.797993E-02, 2.54633E+00, 1.5354E-01, 6.3051E+03, &
      4.060054E-02, 2.45013E+00, 1.6490E-01, 6.3342E+03, &
      4.340198E-02, 2.33262E+00, 1.7418E-01, 6.2591E+03, &
      4.639671E-02, 2.18672E+00, 1.8132E-01, 6.0951E+03, &
      4.959809E-02, 2.00256E+00, 1.8615E-01, 5.8535E+03, &
      5.302035E-02, 1.76493E+00, 1.8830E-01, 5.5387E+03, &
      5.667876E-02, 1.44912E+00, 1.8825E-01, 5.1801E+03, &
      6.058959E-02, 1.01038E+00, 1.8630E-01, 4.7956E+03, &
      6.477028E-02, 3.49959E-01, 1.8275E-01, 4.4006E+03, &
      6.923942E-02, -8.55994E-01, 1.7794E-01, 4.0082E+03, &
      7.163800E-02, -2.25520E+00, 1.7511E-01, 3.8123E+03, &
      7.273450E-02, -4.06506E+00, 1.7377E-01, 3.7262E+03, &
      7.302690E-02, -6.02938E+00, 1.7342E-01, 3.7037E+03, &
      7.346550E-02, -4.25024E+00, 3.9275E+00, 8.3378E+04, &
      7.401695E-02, -3.26725E+00, 4.0145E+00, 8.4589E+04, &
      7.456200E-02, -2.79918E+00, 4.1018E+00, 8.5797E+04, &
      7.912411E-02, -1.53427E+00, 4.8866E+00, 9.6319E+04, &
      8.458368E-02, -8.05339E-01, 5.8791E+00, 1.0840E+05, &
      9.041995E-02, -1.06801E-02, 6.7792E+00, 1.1693E+05, &
      9.665893E-02, 8.82403E-01, 7.5222E+00, 1.2137E+05, &
      1.033284E-01, 1.83035E+00, 8.0732E+00, 1.2185E+05, &
      1.104581E-01, 2.73433E+00, 8.4204E+00, 1.1889E+05, &
      1.153460E-01, 3.13128E+00, 8.5391E+00, 1.1546E+05, &
      1.171115E-01, 2.99611E+00, 8.5619E+00, 1.1402E+05, &
      1.175823E-01, 2.62299E+00, 8.5664E+00, 1.1362E+05, &
      1.180797E-01, 3.00312E+00, 9.4208E+00, 1.2443E+05, &
      1.182885E-01, 3.14862E+00, 9.4235E+00, 1.2425E+05, &
      1.200540E-01, 3.75272E+00, 9.4411E+00, 1.2265E+05, &
      1.262272E-01, 4.87197E+00, 9.4410E+00, 1.1665E+05, &
      1.349368E-01, 6.04241E+00, 9.3134E+00, 1.0764E+05, &
      1.442475E-01, 7.06078E+00, 9.0606E+00, 9.7963E+04, &
      1.542005E-01, 7.95326E+00, 8.7125E+00, 8.8119E+04, &
      1.648404E-01, 8.72829E+00, 8.2962E+00, 7.8493E+04, &
      1.762144E-01, 9.39304E+00, 7.8365E+00, 6.9358E+04, &
      1.883732E-01, 9.95657E+00, 7.3536E+00, 6.0884E+04, &
      2.013709E-01, 1.04290E+01, 6.8631E+00, 5.3155E+04, &
      2.152655E-01, 1.08209E+01, 6.3766E+00, 4.6199E+04, &
      2.301188E-01, 1.11424E+01, 5.9026E+00, 4.0005E+04, &
      2.459970E-01, 1.14037E+01, 5.4470E+00, 3.4535E+04, &
      2.629708E-01, 1.16133E+01, 5.0136E+00, 2.9735E+04, &
      2.811158E-01, 1.17794E+01, 4.6044E+00, 2.5546E+04, &
      3.005128E-01, 1.19090E+01, 4.2203E+00, 2.1904E+04, &
      3.212482E-01, 1.20075E+01, 3.8603E+00, 1.8742E+04, &
      3.434143E-01, 1.20791E+01, 3.5244E+00, 1.6007E+04, &
      3.671099E-01, 1.21278E+01, 3.2123E+00, 1.3648E+04, &
      3.924405E-01, 1.21573E+01, 2.9236E+00, 1.1620E+04, &
      4.195189E-01, 1.21706E+01, 2.6572E+00, 9.8797E+03, &
      4.484657E-01, 1.21704E+01, 2.4121E+00, 8.3898E+03, &
      4.794098E-01, 1.21590E+01, 2.1872E+00, 7.1167E+03, &
      5.124891E-01, 1.21381E+01, 1.9809E+00, 6.0298E+03, &
      5.478508E-01, 1.21089E+01, 1.7921E+00, 5.1033E+03, &
      5.856525E-01, 1.20725E+01, 1.6196E+00, 4.3148E+03, &
      6.260625E-01, 1.20297E+01, 1.4625E+00, 3.6452E+03, &
      6.692609E-01, 1.19809E+01, 1.3197E+00, 3.0771E+03, &
      7.154399E-01, 1.19265E+01, 1.1901E+00, 2.5962E+03, &
      7.648052E-01, 1.18662E+01, 1.0727E+00, 2.1894E+03, &
      8.175768E-01, 1.17996E+01, 9.6659E-01, 1.8459E+03, &
      8.739896E-01, 1.17255E+01, 8.7081E-01, 1.5559E+03, &
      9.342948E-01, 1.16420E+01, 7.8450E-01, 1.3115E+03, &
      9.987612E-01, 1.15461E+01, 7.0682E-01, 1.1058E+03, &
      1.067676E+00, 1.14444E+01, 6.2898E-01, 9.2086E+02, &
      1.141345E+00, 1.13023E+01, 5.6009E-01, 7.6742E+02, &
      1.220098E+00, 1.11171E+01, 4.9919E-01, 6.4019E+02, &
      1.304285E+00, 1.08582E+01, 4.4531E-01, 5.3458E+02, &
      1.394281E+00, 1.04460E+01, 3.9707E-01, 4.4624E+02, &
      1.490486E+00, 9.55217E+00, 3.5292E-01, 3.7136E+02, &
      1.528408E+00, 8.67724E+00, 3.3771E-01, 3.4667E+02, &
      1.551802E+00, 7.08006E+00, 3.2887E-01, 3.3258E+02, &
      1.558040E+00, 5.18127E+00, 3.2657E-01, 3.2896E+02, &
      1.567398E+00, 7.05820E+00, 4.0561E+00, 4.0380E+03, &
      1.590792E+00, 8.76616E+00, 4.0340E+00, 3.9570E+03, &
      1.593329E+00, 8.86568E+00, 4.0316E+00, 3.9484E+03, &
      1.703269E+00, 1.09116E+01, 3.8047E+00, 3.4858E+03, &
      1.820795E+00, 1.18026E+01, 3.4335E+00, 2.9430E+03, &
      1.946430E+00, 1.23246E+01, 3.0522E+00, 2.4476E+03, &
      2.080733E+00, 1.26397E+01, 2.7132E+00, 2.0355E+03, &
      2.224304E+00, 1.28559E+01, 2.4305E+00, 1.7060E+03, &
      2.377781E+00, 1.30119E+01, 2.1807E+00, 1.4321E+03, &
      2.541848E+00, 1.31269E+01, 1.9587E+00, 1.2036E+03, &
      2.717235E+00, 1.32127E+01, 1.7592E+00, 1.0114E+03, &
      2.904724E+00, 1.32769E+01, 1.5787E+00, 8.4933E+02, &
      3.105150E+00, 1.33285E+01, 1.4117E+00, 7.1068E+02, &
      3.319406E+00, 1.34011E+01, 1.2507E+00, 5.8918E+02, &
      3.548445E+00, 1.34116E+01, 1.1075E+00, 4.8826E+02, &
      3.793288E+00, 1.34114E+01, 9.8077E-01, 4.0468E+02, &
      4.055024E+00, 1.34039E+01, 8.6856E-01, 3.3543E+02, &
      4.334821E+00, 1.33916E+01, 7.6922E-01, 2.7807E+02, &
      4.633924E+00, 1.33759E+01, 6.8127E-01, 2.3055E+02, &
      4.953664E+00, 1.33581E+01, 6.0341E-01, 1.9118E+02, &
      5.295467E+00, 1.33392E+01, 5.3446E-01, 1.5855E+02, &
      5.660855E+00, 1.33198E+01, 4.7341E-01, 1.3151E+02, &
      6.051453E+00, 1.33004E+01, 4.1935E-01, 1.0910E+02, &
      6.469004E+00, 1.32816E+01, 3.7148E-01, 9.0526E+01, &
      6.915365E+00, 1.32637E+01, 3.2822E-01, 7.4937E+01, &
      7.392525E+00, 1.32456E+01, 2.8857E-01, 6.1744E+01, &
      7.902609E+00, 1.32277E+01, 2.5373E-01, 5.0887E+01, &
      8.447890E+00, 1.32105E+01, 2.2309E-01, 4.1953E+01, &
      9.030794E+00, 1.31945E+01, 1.9580E-01, 3.4536E+01, &
      9.653919E+00, 1.31795E+01, 1.7070E-01, 2.8257E+01, &
      1.032004E+01, 1.31646E+01, 1.4882E-01, 2.3131E+01, &
      1.103212E+01, 1.31503E+01, 1.2975E-01, 1.8945E+01, &
      1.179334E+01, 1.31369E+01, 1.1313E-01, 1.5527E+01, &
      1.260708E+01, 1.31243E+01, 9.8639E-02, 1.2734E+01, &
      1.347697E+01, 1.31128E+01, 8.6005E-02, 1.0454E+01, &
      1.440688E+01, 1.31021E+01, 7.4990E-02, 8.5895E+00, &
      1.540095E+01, 1.30923E+01, 6.5387E-02, 7.0658E+00, &
      1.646362E+01, 1.30833E+01, 5.7015E-02, 5.8199E+00, &
      1.759961E+01, 1.30752E+01, 4.9716E-02, 4.8010E+00, &
      1.881398E+01, 1.30677E+01, 4.3351E-02, 3.9673E+00, &
      2.011215E+01, 1.30610E+01, 3.7802E-02, 3.2851E+00, &
      2.149988E+01, 1.30548E+01, 3.2964E-02, 2.7264E+00, &
      2.298338E+01, 1.30493E+01, 2.8745E-02, 2.2689E+00, &
      2.456923E+01, 1.30443E+01, 2.5066E-02, 1.8939E+00, &
      2.626450E+01, 1.30397E+01, 2.1859E-02, 1.5865E+00, &
      2.807676E+01, 1.30358E+01, 1.8978E-02, 1.3296E+00, &
      3.001405E+01, 1.30321E+01, 1.6453E-02, 1.1184E+00, &
      3.208502E+01, 1.30288E+01, 1.4264E-02, 9.4583E-01, &
      3.429889E+01, 1.30258E+01, 1.2344E-02, 8.0374E-01, &
      3.666551E+01, 1.30231E+01, 1.0660E-02, 6.8672E-01, &
      3.919543E+01, 1.30206E+01, 9.2057E-03, 5.9119E-01, &
      4.189992E+01, 1.30184E+01, 7.9499E-03, 5.1309E-01, &
      4.479101E+01, 1.30164E+01, 6.8656E-03, 4.4915E-01, &
      4.788159E+01, 1.30146E+01, 5.9291E-03, 3.9671E-01, &
      5.118542E+01, 1.30130E+01, 5.1204E-03, 3.5359E-01, &
      5.471721E+01, 1.30116E+01, 4.4218E-03, 3.1803E-01, &
      5.849270E+01, 1.30103E+01, 3.8180E-03, 2.8863E-01, &
      6.252870E+01, 1.30091E+01, 3.2967E-03, 2.6423E-01, &
      6.684318E+01, 1.30081E+01, 2.8466E-03, 2.4389E-01, &
      7.145536E+01, 1.30072E+01, 2.4579E-03, 2.2686E-01, &
      7.638578E+01, 1.30064E+01, 2.1224E-03, 2.1251E-01, &
      8.165640E+01, 1.30057E+01, 1.8326E-03, 2.0034E-01, &
      8.729069E+01, 1.30050E+01, 1.5824E-03, 1.8995E-01, &
      9.331374E+01, 1.30045E+01, 1.3664E-03, 1.8100E-01, &
      9.975239E+01, 1.30040E+01, 1.1798E-03, 1.7321E-01, &
      1.066353E+02, 1.30035E+01, 1.0188E-03, 1.6636E-01, &
      1.139931E+02, 1.30031E+01, 8.7971E-04, 1.6028E-01, &
      1.218587E+02, 1.30027E+01, 7.5962E-04, 1.5482E-01, &
      1.302669E+02, 1.30024E+01, 6.5593E-04, 1.4987E-01, &
      1.392553E+02, 1.30022E+01, 5.6639E-04, 1.4532E-01, &
      1.488640E+02, 1.30019E+01, 4.8908E-04, 1.4110E-01, &
      1.591356E+02, 1.30017E+01, 4.2232E-04, 1.3716E-01, &
      1.701159E+02, 1.30015E+01, 3.6468E-04, 1.3342E-01, &
      1.818539E+02, 1.30013E+01, 3.1490E-04, 1.2986E-01, &
      1.944018E+02, 1.30012E+01, 2.7192E-04, 1.2644E-01, &
      2.078156E+02, 1.30010E+01, 2.3481E-04, 1.2314E-01, &
      2.221548E+02, 1.30009E+01, 2.0276E-04, 1.1993E-01, &
      2.374835E+02, 1.30008E+01, 1.7508E-04, 1.1680E-01, &
      2.538699E+02, 1.30007E+01, 1.5119E-04, 1.1374E-01, &
      2.713869E+02, 1.30006E+01, 1.3055E-04, 1.1073E-01, &
      2.901126E+02, 1.30005E+01, 1.1274E-04, 1.0778E-01, &
      3.101304E+02, 1.30005E+01, 9.7350E-05, 1.0486E-01, &
      3.315294E+02, 1.30004E+01, 8.4064E-05, 1.0199E-01, &
      3.544049E+02, 1.30004E+01, 7.2591E-05, 9.9153E-02, &
      3.788588E+02, 1.30003E+01, 6.2685E-05, 9.6355E-02, &
      4.050001E+02, 1.30003E+01, 5.4130E-05, 9.3592E-02, &
      4.329451E+02, 1.30003E+01, 4.6743E-05, 9.0866E-02]
   case(14)
      frel = -1.5600E-02
      allocate(nist_tab(740))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.981393E-03, 5.50431E-01, 0.0000E+00, 2.1060E-06, &
      5.057639E-03, -3.20849E-01, 0.0000E+00, 2.2934E-06, &
      5.077971E-03, -1.28029E+00, 0.0000E+00, 2.3455E-06, &
      5.108469E-03, -4.07656E-01, 1.8433E+00, 5.4064E+05, &
      5.184715E-03, 3.19503E-01, 1.9476E+00, 5.6283E+05, &
      1.069000E-02, 4.49043E+00, 4.9472E-01, 6.9340E+04, &
      1.113003E-02, 4.41607E+00, 4.0981E-01, 5.5167E+04, &
      1.130038E-02, 4.38482E+00, 3.8216E-01, 5.0670E+04, &
      1.134581E-02, 4.37198E+00, 3.7522E-01, 4.9550E+04, &
      1.141396E-02, 4.36769E+00, 3.7687E-01, 4.9471E+04, &
      1.142761E-02, 4.36646E+00, 3.7492E-01, 4.9156E+04, &
      1.158431E-02, 4.34801E+00, 3.5360E-01, 4.5733E+04, &
      1.221612E-02, 4.26861E+00, 2.8398E-01, 3.4829E+04, &
      1.305903E-02, 4.17557E+00, 2.2096E-01, 2.5351E+04, &
      1.396010E-02, 4.09293E+00, 1.7788E-01, 1.9091E+04, &
      1.492335E-02, 4.02013E+00, 1.4900E-01, 1.4960E+04, &
      1.595306E-02, 3.95663E+00, 1.3038E-01, 1.2246E+04, &
      1.705382E-02, 3.89925E+00, 1.1936E-01, 1.0487E+04, &
      1.823053E-02, 3.84744E+00, 1.1415E-01, 9.3811E+03, &
      1.948844E-02, 3.79998E+00, 1.1354E-01, 8.7290E+03, &
      2.083314E-02, 3.75582E+00, 1.1673E-01, 8.3951E+03, &
      2.227063E-02, 3.71414E+00, 1.2324E-01, 8.2913E+03, &
      2.380730E-02, 3.67429E+00, 1.3259E-01, 8.3445E+03, &
      2.545001E-02, 3.63561E+00, 1.4434E-01, 8.4979E+03, &
      2.720606E-02, 3.59745E+00, 1.5811E-01, 8.7075E+03, &
      2.908327E-02, 3.55913E+00, 1.7351E-01, 8.9389E+03, &
      3.109002E-02, 3.51987E+00, 1.9016E-01, 9.1644E+03, &
      3.323523E-02, 3.47876E+00, 2.0768E-01, 9.3624E+03, &
      3.552846E-02, 3.43487E+00, 2.2566E-01, 9.5165E+03, &
      3.797993E-02, 3.38651E+00, 2.4374E-01, 9.6153E+03, &
      4.060054E-02, 3.33221E+00, 2.6154E-01, 9.6518E+03, &
      4.340198E-02, 3.26991E+00, 2.7876E-01, 9.6232E+03, &
      4.639671E-02, 3.19712E+00, 2.9508E-01, 9.5291E+03, &
      4.959809E-02, 3.11054E+00, 3.1003E-01, 9.3656E+03, &
      5.302035E-02, 3.00579E+00, 3.2339E-01, 9.1385E+03, &
      5.667876E-02, 2.87721E+00, 3.3506E-01, 8.8572E+03, &
      6.058959E-02, 2.71713E+00, 3.4502E-01, 8.5318E+03, &
      6.477028E-02, 2.51474E+00, 3.5326E-01, 8.1718E+03, &
      6.923942E-02, 2.25390E+00, 3.5981E-01, 7.7861E+03, &
      7.401695E-02, 1.90880E+00, 3.6472E-01, 7.3830E+03, &
      7.912411E-02, 1.43406E+00, 3.6805E-01, 6.9695E+03, &
      8.458368E-02, 7.37405E-01, 3.6987E-01, 6.5519E+03, &
      9.041995E-02, -4.23197E-01, 3.7026E-01, 6.1354E+03, &
      9.665893E-02, -3.28171E+00, 3.6804E-01, 5.7051E+03, &
      9.721600E-02, -3.84572E+00, 3.6732E-01, 5.6613E+03, &
      9.870400E-02, -6.95115E+00, 3.6535E-01, 5.5460E+03, &
      9.910080E-02, -1.04791E+01, 3.6481E-01, 5.5157E+03, &
      9.969600E-02, -7.07666E+00, 7.2061E+00, 1.0830E+05, &
      1.011840E-01, -4.18129E+00, 7.3507E+00, 1.0885E+05, &
      1.033284E-01, -2.67470E+00, 7.5606E+00, 1.0963E+05, &
      1.104581E-01, -4.73104E-01, 8.2681E+00, 1.1215E+05, &
      1.180797E-01, 1.02847E+00, 8.8713E+00, 1.1257E+05, &
      1.262272E-01, 2.34527E+00, 9.2705E+00, 1.1004E+05, &
      1.349368E-01, 3.51620E+00, 9.4680E+00, 1.0513E+05, &
      1.442475E-01, 4.39577E+00, 9.4800E+00, 9.8468E+04, &
      1.457260E-01, 4.44809E+00, 9.4670E+00, 9.7336E+04, &
      1.479565E-01, 4.27331E+00, 9.4409E+00, 9.5604E+04, &
      1.485513E-01, 3.83506E+00, 9.4327E+00, 9.5138E+04, &
      1.494435E-01, 4.43854E+00, 1.0408E+01, 1.0435E+05, &
      1.516740E-01, 5.11882E+00, 1.0379E+01, 1.0252E+05, &
      1.542005E-01, 5.58201E+00, 1.0338E+01, 1.0045E+05, &
      1.648404E-01, 6.95675E+00, 1.0098E+01, 9.1788E+04, &
      1.762144E-01, 8.06341E+00, 9.7527E+00, 8.2925E+04, &
      1.883732E-01, 9.00930E+00, 9.3252E+00, 7.4172E+04, &
      2.013709E-01, 9.82004E+00, 8.8416E+00, 6.5786E+04, &
      2.152655E-01, 1.05105E+01, 8.3235E+00, 5.7934E+04, &
      2.301188E-01, 1.10929E+01, 7.7878E+00, 5.0707E+04, &
      2.459970E-01, 1.15791E+01, 7.2498E+00, 4.4157E+04, &
      2.629708E-01, 1.19807E+01, 6.7202E+00, 3.8290E+04, &
      2.811158E-01, 1.23087E+01, 6.2068E+00, 3.3082E+04, &
      3.005128E-01, 1.25735E+01, 5.7150E+00, 2.8495E+04, &
      3.212482E-01, 1.27839E+01, 5.2483E+00, 2.4479E+04, &
      3.434143E-01, 1.29483E+01, 4.8086E+00, 2.0981E+04, &
      3.671099E-01, 1.30739E+01, 4.3969E+00, 1.7946E+04, &
      3.924405E-01, 1.31670E+01, 4.0133E+00, 1.5323E+04, &
      4.195189E-01, 1.32331E+01, 3.6568E+00, 1.3061E+04, &
      4.484657E-01, 1.32758E+01, 3.3258E+00, 1.1113E+04, &
      4.794098E-01, 1.32987E+01, 3.0200E+00, 9.4401E+03, &
      5.124891E-01, 1.33048E+01, 2.7385E+00, 8.0080E+03, &
      5.478508E-01, 1.32972E+01, 2.4802E+00, 6.7848E+03, &
      5.856525E-01, 1.32781E+01, 2.2438E+00, 5.7424E+03, &
      6.260625E-01, 1.32495E+01, 2.0280E+00, 4.8552E+03, &
      6.692609E-01, 1.32130E+01, 1.8311E+00, 4.1013E+03, &
      7.154399E-01, 1.31695E+01, 1.6519E+00, 3.4615E+03, &
      7.648052E-01, 1.31201E+01, 1.4894E+00, 2.9199E+03, &
      8.175768E-01, 1.30654E+01, 1.3422E+00, 2.4618E+03, &
      8.739896E-01, 1.30058E+01, 1.2092E+00, 2.0752E+03, &
      9.342948E-01, 1.29418E+01, 1.0893E+00, 1.7491E+03, &
      9.987612E-01, 1.28743E+01, 9.8137E-01, 1.4744E+03, &
      1.067676E+00, 1.27932E+01, 8.7137E-01, 1.2251E+03, &
      1.141345E+00, 1.26978E+01, 7.7420E-01, 1.0186E+03, &
      1.220098E+00, 1.25861E+01, 6.8851E-01, 8.4779E+02, &
      1.304285E+00, 1.24527E+01, 6.1287E-01, 7.0633E+02, &
      1.394281E+00, 1.22868E+01, 5.4605E-01, 5.8907E+02, &
      1.490486E+00, 1.20669E+01, 4.8696E-01, 4.9179E+02, &
      1.593329E+00, 1.17442E+01, 4.3467E-01, 4.1100E+02, &
      1.703269E+00, 1.11665E+01, 3.8835E-01, 3.4385E+02, &
      1.802122E+00, 9.77316E+00, 3.5330E-01, 2.9595E+02, &
      1.820795E+00, 8.96867E+00, 3.4728E-01, 2.8798E+02, &
      1.829705E+00, 8.18207E+00, 3.4446E-01, 2.8427E+02, &
      1.837061E+00, 6.28036E+00, 3.4216E-01, 2.8127E+02, &
      1.848094E+00, 8.18045E+00, 4.0819E+00, 3.3115E+03, &
      1.875678E+00, 9.90765E+00, 4.0304E+00, 3.2217E+03, &
      1.946430E+00, 1.13530E+01, 3.8896E+00, 2.9963E+03, &
      2.080733E+00, 1.25086E+01, 3.5283E+00, 2.5428E+03, &
      2.224304E+00, 1.31320E+01, 3.1614E+00, 2.1316E+03, &
      2.377781E+00, 1.35165E+01, 2.8224E+00, 1.7805E+03, &
      2.541848E+00, 1.37720E+01, 2.5325E+00, 1.4948E+03, &
      2.717235E+00, 1.39567E+01, 2.2762E+00, 1.2570E+03, &
      2.904724E+00, 1.40941E+01, 2.0459E+00, 1.0572E+03, &
      3.105150E+00, 1.41991E+01, 1.8323E+00, 8.8593E+02, &
      3.319406E+00, 1.42699E+01, 1.6363E+00, 7.4032E+02, &
      3.548445E+00, 1.43185E+01, 1.4614E+00, 6.1872E+02, &
      3.793288E+00, 1.43972E+01, 1.2999E+00, 5.1505E+02, &
      4.055024E+00, 1.44122E+01, 1.1509E+00, 4.2677E+02, &
      4.334821E+00, 1.44148E+01, 1.0190E+00, 3.5366E+02, &
      4.633924E+00, 1.44092E+01, 9.0223E-01, 2.9311E+02, &
      4.953664E+00, 1.43980E+01, 7.9890E-01, 2.4297E+02, &
      5.295467E+00, 1.43830E+01, 7.0743E-01, 2.0142E+02, &
      5.660855E+00, 1.43657E+01, 6.2644E-01, 1.6700E+02, &
      6.051453E+00, 1.43470E+01, 5.5475E-01, 1.3848E+02, &
      6.469004E+00, 1.43275E+01, 4.9128E-01, 1.1486E+02, &
      6.915365E+00, 1.43080E+01, 4.3509E-01, 9.5280E+01, &
      7.392525E+00, 1.42890E+01, 3.8534E-01, 7.9055E+01, &
      7.902609E+00, 1.42709E+01, 3.4129E-01, 6.5607E+01, &
      8.447890E+00, 1.42537E+01, 3.0059E-01, 5.4160E+01, &
      9.030794E+00, 1.42373E+01, 2.6409E-01, 4.4613E+01, &
      9.653919E+00, 1.42193E+01, 2.3060E-01, 3.6539E+01, &
      1.032004E+01, 1.42018E+01, 2.0136E-01, 2.9938E+01, &
      1.103212E+01, 1.41850E+01, 1.7583E-01, 2.4541E+01, &
      1.179334E+01, 1.41691E+01, 1.5354E-01, 2.0129E+01, &
      1.260708E+01, 1.41542E+01, 1.3408E-01, 1.6519E+01, &
      1.347697E+01, 1.41404E+01, 1.1709E-01, 1.3566E+01, &
      1.440688E+01, 1.41275E+01, 1.0225E-01, 1.1150E+01, &
      1.540095E+01, 1.41156E+01, 8.9301E-02, 9.1726E+00, &
      1.646362E+01, 1.41047E+01, 7.7990E-02, 7.5540E+00, &
      1.759961E+01, 1.40947E+01, 6.8113E-02, 6.2286E+00, &
      1.881398E+01, 1.40856E+01, 5.9487E-02, 5.1429E+00, &
      2.011215E+01, 1.40772E+01, 5.1955E-02, 4.2534E+00, &
      2.149988E+01, 1.40696E+01, 4.5377E-02, 3.5245E+00, &
      2.298338E+01, 1.40628E+01, 3.9633E-02, 2.9268E+00, &
      2.456923E+01, 1.40566E+01, 3.4616E-02, 2.4365E+00, &
      2.626450E+01, 1.40511E+01, 3.0235E-02, 2.0342E+00, &
      2.807676E+01, 1.40462E+01, 2.6276E-02, 1.6969E+00, &
      3.001405E+01, 1.40415E+01, 2.2797E-02, 1.4192E+00, &
      3.208502E+01, 1.40372E+01, 1.9779E-02, 1.1925E+00, &
      3.429889E+01, 1.40333E+01, 1.7161E-02, 1.0073E+00, &
      3.666551E+01, 1.40298E+01, 1.4890E-02, 8.5575E-01, &
      3.919543E+01, 1.40267E+01, 1.2919E-02, 7.3170E-01, &
      4.189992E+01, 1.40239E+01, 1.1160E-02, 6.2826E-01, &
      4.479101E+01, 1.40213E+01, 9.6400E-03, 5.4372E-01, &
      4.788159E+01, 1.40191E+01, 8.3272E-03, 4.7454E-01, &
      5.118542E+01, 1.40170E+01, 7.1932E-03, 4.1781E-01, &
      5.471721E+01, 1.40151E+01, 6.2137E-03, 3.7119E-01, &
      5.849270E+01, 1.40135E+01, 5.3676E-03, 3.3278E-01, &
      6.252870E+01, 1.40120E+01, 4.6367E-03, 3.0103E-01, &
      6.684318E+01, 1.40107E+01, 4.0054E-03, 2.7471E-01, &
      7.145536E+01, 1.40095E+01, 3.4605E-03, 2.5280E-01, &
      7.638578E+01, 1.40084E+01, 2.9899E-03, 2.3447E-01, &
      8.165640E+01, 1.40075E+01, 2.5832E-03, 2.1903E-01, &
      8.729069E+01, 1.40066E+01, 2.2319E-03, 2.0595E-01, &
      9.331374E+01, 1.40059E+01, 1.9284E-03, 1.9478E-01, &
      9.975239E+01, 1.40052E+01, 1.6662E-03, 1.8518E-01, &
      1.066353E+02, 1.40046E+01, 1.4396E-03, 1.7684E-01, &
      1.139931E+02, 1.40041E+01, 1.2438E-03, 1.6952E-01, &
      1.218587E+02, 1.40036E+01, 1.0747E-03, 1.6304E-01, &
      1.302669E+02, 1.40032E+01, 9.2859E-04, 1.5723E-01, &
      1.392553E+02, 1.40029E+01, 8.0234E-04, 1.5198E-01, &
      1.488640E+02, 1.40025E+01, 6.9326E-04, 1.4717E-01, &
      1.591356E+02, 1.40022E+01, 5.9902E-04, 1.4272E-01, &
      1.701159E+02, 1.40020E+01, 5.1758E-04, 1.3856E-01, &
      1.818539E+02, 1.40017E+01, 4.4723E-04, 1.3464E-01, &
      1.944018E+02, 1.40015E+01, 3.8643E-04, 1.3093E-01, &
      2.078156E+02, 1.40014E+01, 3.3391E-04, 1.2737E-01, &
      2.221548E+02, 1.40012E+01, 2.8852E-04, 1.2394E-01, &
      2.374835E+02, 1.40011E+01, 2.4931E-04, 1.2062E-01, &
      2.538699E+02, 1.40009E+01, 2.1542E-04, 1.1739E-01, &
      2.713869E+02, 1.40008E+01, 1.8614E-04, 1.1425E-01, &
      2.901126E+02, 1.40007E+01, 1.6085E-04, 1.1117E-01, &
      3.101304E+02, 1.40006E+01, 1.3899E-04, 1.0814E-01, &
      3.315294E+02, 1.40006E+01, 1.2010E-04, 1.0518E-01, &
      3.544049E+02, 1.40005E+01, 1.0378E-04, 1.0226E-01, &
      3.788588E+02, 1.40004E+01, 8.9678E-05, 9.9388E-02, &
      4.050001E+02, 1.40004E+01, 7.7492E-05, 9.6560E-02, &
      4.329451E+02, 1.40003E+01, 6.6963E-05, 9.3776E-02]
   case(15)
      frel = -1.8000E-02
      allocate(nist_tab(752))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.209957E-03, -1.26929E+00, 0.0000E+00, 6.7573E-06, &
      6.257232E-03, -1.66855E+00, 0.0000E+00, 7.0373E-06, &
      6.305007E-03, -2.31139E+00, 0.0000E+00, 7.3292E-06, &
      6.330354E-03, -3.07745E+00, 0.0000E+00, 7.4878E-06, &
      6.353005E-03, -3.18301E+00, 9.4416E-01, 2.0191E+05, &
      6.368374E-03, -3.42247E+00, 9.5617E-01, 2.0398E+05, &
      6.378545E-03, -3.94072E+00, 9.6419E-01, 2.0536E+05, &
      6.416855E-03, -2.90422E+00, 2.9193E+00, 6.1808E+05, &
      6.463425E-03, -2.33032E+00, 3.0314E+00, 6.3719E+05, &
      6.512629E-03, -2.02658E+00, 3.1536E+00, 6.5787E+05, &
      1.069000E-02, 6.73661E+00, 3.0730E+00, 3.9054E+05, &
      1.142761E-02, 6.70372E+00, 2.2393E+00, 2.6622E+05, &
      1.221612E-02, 6.52061E+00, 1.6032E+00, 1.7830E+05, &
      1.305903E-02, 6.27590E+00, 1.1415E+00, 1.1875E+05, &
      1.396010E-02, 6.02132E+00, 8.1618E-01, 7.9430E+04, &
      1.417226E-02, 5.96539E+00, 7.5768E-01, 7.2633E+04, &
      1.438918E-02, 5.91002E+00, 7.0351E-01, 6.6423E+04, &
      1.444703E-02, 5.89549E+00, 6.8995E-01, 6.4882E+04, &
      1.453380E-02, 5.87424E+00, 6.7052E-01, 6.2678E+04, &
      1.475072E-02, 5.82233E+00, 6.2463E-01, 5.7530E+04, &
      1.492335E-02, 5.78240E+00, 5.9118E-01, 5.3820E+04, &
      1.595306E-02, 5.56893E+00, 4.3681E-01, 3.7199E+04, &
      1.705382E-02, 5.38200E+00, 3.3202E-01, 2.6450E+04, &
      1.823053E-02, 5.21926E+00, 2.6246E-01, 1.9559E+04, &
      1.948844E-02, 5.07806E+00, 2.1892E-01, 1.5262E+04, &
      2.083314E-02, 4.95436E+00, 1.9555E-01, 1.2752E+04, &
      2.227063E-02, 4.84692E+00, 1.8825E-01, 1.1484E+04, &
      2.380730E-02, 4.75458E+00, 1.9364E-01, 1.1050E+04, &
      2.545001E-02, 4.67639E+00, 2.0852E-01, 1.1132E+04, &
      2.720606E-02, 4.61122E+00, 2.2988E-01, 1.1479E+04, &
      2.908327E-02, 4.55762E+00, 2.5483E-01, 1.1904E+04, &
      3.109002E-02, 4.51357E+00, 2.8079E-01, 1.2270E+04, &
      3.323523E-02, 4.47659E+00, 3.0586E-01, 1.2503E+04, &
      3.552846E-02, 4.44405E+00, 3.2885E-01, 1.2575E+04, &
      3.797993E-02, 4.41334E+00, 3.4935E-01, 1.2496E+04, &
      4.060054E-02, 4.38215E+00, 3.6723E-01, 1.2288E+04, &
      4.340198E-02, 4.34843E+00, 3.8272E-01, 1.1980E+04, &
      4.639671E-02, 4.30977E+00, 3.9627E-01, 1.1603E+04, &
      4.959809E-02, 4.26427E+00, 4.0847E-01, 1.1189E+04, &
      5.302035E-02, 4.20979E+00, 4.2008E-01, 1.0764E+04, &
      5.667876E-02, 4.14493E+00, 4.3148E-01, 1.0342E+04, &
      6.058959E-02, 4.06700E+00, 4.4289E-01, 9.9308E+03, &
      6.477028E-02, 3.97288E+00, 4.5446E-01, 9.5326E+03, &
      6.923942E-02, 3.85846E+00, 4.6625E-01, 9.1487E+03, &
      7.401695E-02, 3.71823E+00, 4.7820E-01, 8.7774E+03, &
      7.912411E-02, 3.54439E+00, 4.9011E-01, 8.4154E+03, &
      8.458368E-02, 3.32567E+00, 5.0174E-01, 8.0590E+03, &
      9.041995E-02, 3.04495E+00, 5.1277E-01, 7.7045E+03, &
      9.665893E-02, 2.67477E+00, 5.2288E-01, 7.3495E+03, &
      1.033284E-01, 2.16732E+00, 5.3177E-01, 6.9919E+03, &
      1.104581E-01, 1.42815E+00, 5.3914E-01, 6.6314E+03, &
      1.180797E-01, 2.26515E-01, 5.4479E-01, 6.2685E+03, &
      1.262272E-01, -2.33819E+00, 5.3712E-01, 5.7813E+03, &
      1.295560E-01, -4.86950E+00, 5.3236E-01, 5.5828E+03, &
      1.315390E-01, -9.24896E+00, 5.2939E-01, 5.4681E+03, &
      1.320678E-01, -1.43590E+01, 5.2859E-01, 5.4379E+03, &
      1.328610E-01, -9.24590E+00, 1.0516E+01, 1.0753E+05, &
      1.349368E-01, -4.70611E+00, 1.0551E+01, 1.0623E+05, &
      1.442475E-01, 2.39838E-01, 1.0704E+01, 1.0082E+05, &
      1.542005E-01, 2.55586E+00, 1.0790E+01, 9.5065E+04, &
      1.648404E-01, 4.24825E+00, 1.0709E+01, 8.8257E+04, &
      1.762144E-01, 5.53947E+00, 1.0481E+01, 8.0810E+04, &
      1.855140E-01, 6.11355E+00, 1.0223E+01, 7.4868E+04, &
      1.883732E-01, 5.89518E+00, 1.0134E+01, 7.3091E+04, &
      1.891107E-01, 5.40553E+00, 1.0111E+01, 7.2638E+04, &
      1.902465E-01, 6.07332E+00, 1.1179E+01, 7.9835E+04, &
      1.930860E-01, 6.81272E+00, 1.1093E+01, 7.8055E+04, &
      2.013709E-01, 7.90149E+00, 1.0828E+01, 7.3051E+04, &
      2.152655E-01, 9.14404E+00, 1.0352E+01, 6.5335E+04, &
      2.301188E-01, 1.01493E+01, 9.8221E+00, 5.7989E+04, &
      2.459970E-01, 1.09897E+01, 9.2561E+00, 5.1120E+04, &
      2.629708E-01, 1.16948E+01, 8.6714E+00, 4.4800E+04, &
      2.811158E-01, 1.22841E+01, 8.0814E+00, 3.9057E+04, &
      3.005128E-01, 1.27723E+01, 7.4974E+00, 3.3896E+04, &
      3.212482E-01, 1.31729E+01, 6.9286E+00, 2.9303E+04, &
      3.434143E-01, 1.34977E+01, 6.3813E+00, 2.5246E+04, &
      3.671099E-01, 1.37575E+01, 5.8599E+00, 2.1687E+04, &
      3.924405E-01, 1.39617E+01, 5.3671E+00, 1.8581E+04, &
      4.195189E-01, 1.41172E+01, 4.9045E+00, 1.5885E+04, &
      4.484657E-01, 1.42345E+01, 4.4727E+00, 1.3551E+04, &
      4.794098E-01, 1.43183E+01, 4.0715E+00, 1.1540E+04, &
      5.124891E-01, 1.43744E+01, 3.7002E+00, 9.8109E+03, &
      5.478508E-01, 1.44075E+01, 3.3577E+00, 8.3282E+03, &
      5.856525E-01, 1.44230E+01, 3.0418E+00, 7.0581E+03, &
      6.260625E-01, 1.44202E+01, 2.7517E+00, 5.9732E+03, &
      6.692609E-01, 1.44036E+01, 2.4862E+00, 5.0490E+03, &
      7.154399E-01, 1.43758E+01, 2.2441E+00, 4.2635E+03, &
      7.648052E-01, 1.43390E+01, 2.0239E+00, 3.5974E+03, &
      8.175768E-01, 1.42948E+01, 1.8239E+00, 3.0331E+03, &
      8.739896E-01, 1.42446E+01, 1.6428E+00, 2.5559E+03, &
      9.342948E-01, 1.41907E+01, 1.4792E+00, 2.1532E+03, &
      9.987612E-01, 1.41474E+01, 1.3319E+00, 1.8140E+03, &
      1.067676E+00, 1.40846E+01, 1.1806E+00, 1.5047E+03, &
      1.141345E+00, 1.40043E+01, 1.0473E+00, 1.2491E+03, &
      1.220098E+00, 1.39140E+01, 9.3000E-01, 1.0380E+03, &
      1.304285E+00, 1.38136E+01, 8.2665E-01, 8.6345E+02, &
      1.394281E+00, 1.37007E+01, 7.3553E-01, 7.1908E+02, &
      1.490486E+00, 1.35703E+01, 6.5511E-01, 5.9952E+02, &
      1.593329E+00, 1.34131E+01, 5.8408E-01, 5.0039E+02, &
      1.703269E+00, 1.32121E+01, 5.2128E-01, 4.1812E+02, &
      1.820795E+00, 1.29304E+01, 4.6570E-01, 3.4978E+02, &
      1.946430E+00, 1.24678E+01, 4.1646E-01, 2.9294E+02, &
      2.080733E+00, 1.13134E+01, 3.7279E-01, 2.4563E+02, &
      2.102590E+00, 1.08633E+01, 3.6641E-01, 2.3897E+02, &
      2.134772E+00, 9.28431E+00, 3.5735E-01, 2.2962E+02, &
      2.143354E+00, 7.39192E+00, 3.5500E-01, 2.2722E+02, &
      2.156227E+00, 9.29308E+00, 4.0785E+00, 2.5720E+03, &
      2.188410E+00, 1.10202E+01, 4.0118E+00, 2.4928E+03, &
      2.224304E+00, 1.18112E+01, 3.9399E+00, 2.4087E+03, &
      2.377781E+00, 1.32998E+01, 3.5859E+00, 2.0510E+03, &
      2.541848E+00, 1.40078E+01, 3.2218E+00, 1.7241E+03, &
      2.717235E+00, 1.44377E+01, 2.8870E+00, 1.4455E+03, &
      2.904724E+00, 1.47250E+01, 2.5936E+00, 1.2150E+03, &
      3.105150E+00, 1.49354E+01, 2.3262E+00, 1.0197E+03, &
      3.319406E+00, 1.50821E+01, 2.0806E+00, 8.5336E+02, &
      3.548445E+00, 1.51854E+01, 1.8611E+00, 7.1431E+02, &
      3.793288E+00, 1.52583E+01, 1.6640E+00, 5.9764E+02, &
      4.055024E+00, 1.53093E+01, 1.4879E+00, 5.0008E+02, &
      4.334821E+00, 1.53912E+01, 1.3284E+00, 4.1786E+02, &
      4.633924E+00, 1.54105E+01, 1.1761E+00, 3.4626E+02, &
      4.953664E+00, 1.54155E+01, 1.0413E+00, 2.8696E+02, &
      5.295467E+00, 1.54114E+01, 9.2199E-01, 2.3785E+02, &
      5.660855E+00, 1.54013E+01, 8.1640E-01, 1.9718E+02, &
      6.051453E+00, 1.53871E+01, 7.2293E-01, 1.6348E+02, &
      6.469004E+00, 1.53703E+01, 6.4020E-01, 1.3556E+02, &
      6.915365E+00, 1.53519E+01, 5.6693E-01, 1.1243E+02, &
      7.392525E+00, 1.53326E+01, 5.0207E-01, 9.3260E+01, &
      7.902609E+00, 1.53133E+01, 4.4464E-01, 7.7375E+01, &
      8.447890E+00, 1.52943E+01, 3.9380E-01, 6.4209E+01, &
      9.030794E+00, 1.52766E+01, 3.4826E-01, 5.3218E+01, &
      9.653919E+00, 1.52590E+01, 3.0528E-01, 4.3738E+01, &
      1.032004E+01, 1.52404E+01, 2.6689E-01, 3.5864E+01, &
      1.103212E+01, 1.52220E+01, 2.3334E-01, 2.9419E+01, &
      1.179334E+01, 1.52041E+01, 2.0401E-01, 2.4144E+01, &
      1.260708E+01, 1.51870E+01, 1.7838E-01, 1.9824E+01, &
      1.347697E+01, 1.51710E+01, 1.5596E-01, 1.6287E+01, &
      1.440688E+01, 1.51559E+01, 1.3637E-01, 1.3390E+01, &
      1.540095E+01, 1.51419E+01, 1.1924E-01, 1.1017E+01, &
      1.646362E+01, 1.51290E+01, 1.0427E-01, 9.0721E+00, &
      1.759961E+01, 1.51170E+01, 9.1177E-02, 7.4782E+00, &
      1.881398E+01, 1.51061E+01, 7.9731E-02, 6.1716E+00, &
      2.011215E+01, 1.50960E+01, 6.9723E-02, 5.1002E+00, &
      2.149988E+01, 1.50869E+01, 6.0972E-02, 4.2213E+00, &
      2.298338E+01, 1.50785E+01, 5.3321E-02, 3.5002E+00, &
      2.456923E+01, 1.50710E+01, 4.6630E-02, 2.9083E+00, &
      2.626450E+01, 1.50642E+01, 4.0780E-02, 2.4222E+00, &
      2.807676E+01, 1.50579E+01, 3.5454E-02, 2.0128E+00, &
      3.001405E+01, 1.50521E+01, 3.0764E-02, 1.6755E+00, &
      3.208502E+01, 1.50469E+01, 2.6694E-02, 1.4003E+00, &
      3.429889E+01, 1.50421E+01, 2.3164E-02, 1.1756E+00, &
      3.666551E+01, 1.50377E+01, 2.0100E-02, 9.9193E-01, &
      3.919543E+01, 1.50338E+01, 1.7442E-02, 8.4173E-01, &
      4.189992E+01, 1.50303E+01, 1.5135E-02, 7.1877E-01, &
      4.479101E+01, 1.50271E+01, 1.3134E-02, 6.1800E-01, &
      4.788159E+01, 1.50242E+01, 1.1368E-02, 5.3446E-01, &
      5.118542E+01, 1.50216E+01, 9.8273E-03, 4.6566E-01, &
      5.471721E+01, 1.50193E+01, 8.4957E-03, 4.0921E-01, &
      5.849270E+01, 1.50172E+01, 7.3447E-03, 3.6279E-01, &
      6.252870E+01, 1.50154E+01, 6.3496E-03, 3.2452E-01, &
      6.684318E+01, 1.50137E+01, 5.4893E-03, 2.9287E-01, &
      7.145536E+01, 1.50122E+01, 4.7457E-03, 2.6661E-01, &
      7.638578E+01, 1.50108E+01, 4.1028E-03, 2.4472E-01, &
      8.165640E+01, 1.50096E+01, 3.5473E-03, 2.2640E-01, &
      8.729069E+01, 1.50085E+01, 3.0675E-03, 2.1097E-01, &
      9.331374E+01, 1.50076E+01, 2.6527E-03, 1.9790E-01, &
      9.975239E+01, 1.50067E+01, 2.2940E-03, 1.8673E-01, &
      1.066353E+02, 1.50060E+01, 1.9838E-03, 1.7712E-01, &
      1.139931E+02, 1.50053E+01, 1.7155E-03, 1.6879E-01, &
      1.218587E+02, 1.50047E+01, 1.4836E-03, 1.6147E-01, &
      1.302669E+02, 1.50042E+01, 1.2830E-03, 1.5500E-01, &
      1.392553E+02, 1.50037E+01, 1.1096E-03, 1.4921E-01, &
      1.488640E+02, 1.50033E+01, 9.5957E-04, 1.4398E-01, &
      1.591356E+02, 1.50029E+01, 8.2986E-04, 1.3920E-01, &
      1.701159E+02, 1.50026E+01, 7.1769E-04, 1.3480E-01, &
      1.818539E+02, 1.50023E+01, 6.2069E-04, 1.3071E-01, &
      1.944018E+02, 1.50020E+01, 5.3680E-04, 1.2685E-01, &
      2.078156E+02, 1.50018E+01, 4.6426E-04, 1.2322E-01, &
      2.221548E+02, 1.50016E+01, 4.0152E-04, 1.1975E-01, &
      2.374835E+02, 1.50014E+01, 3.4726E-04, 1.1642E-01, &
      2.538699E+02, 1.50012E+01, 3.0034E-04, 1.1322E-01, &
      2.713869E+02, 1.50011E+01, 2.5976E-04, 1.1012E-01, &
      2.901126E+02, 1.50010E+01, 2.2466E-04, 1.0711E-01, &
      3.101304E+02, 1.50008E+01, 1.9431E-04, 1.0418E-01, &
      3.315294E+02, 1.50007E+01, 1.6806E-04, 1.0132E-01, &
      3.544049E+02, 1.50007E+01, 1.4536E-04, 9.8522E-02, &
      3.788588E+02, 1.50006E+01, 1.2572E-04, 9.5784E-02, &
      4.050001E+02, 1.50005E+01, 1.0874E-04, 9.3102E-02, &
      4.329451E+02, 1.50005E+01, 9.4055E-05, 9.0472E-02]
   case(16)
      frel = -2.1000E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      7.580180E-03, -3.48343E+00, 0.0000E+00, 2.2563E-05, &
      7.657359E-03, -4.26512E+00, 0.0000E+00, 2.3751E-05, &
      7.696204E-03, -4.91639E+00, 0.0000E+00, 2.4366E-05, &
      7.727143E-03, -6.07961E+00, 0.0000E+00, 2.4865E-05, &
      7.774563E-03, -5.80548E+00, 1.7889E+00, 3.0201E+05, &
      7.805818E-03, -6.49112E+00, 1.8360E+00, 3.0873E+05, &
      7.852700E-03, -5.51019E+00, 3.6918E+00, 6.1706E+05, &
      7.889576E-03, -5.10778E+00, 3.8061E+00, 6.3320E+05, &
      7.969904E-03, -4.68850E+00, 4.0655E+00, 6.6954E+05, &
      1.069000E-02, 4.85407E+00, 1.0046E+01, 1.2334E+06, &
      1.142761E-02, 7.02455E+00, 8.6428E+00, 9.9270E+05, &
      1.221612E-02, 8.29260E+00, 6.8704E+00, 7.3818E+05, &
      1.305903E-02, 8.79689E+00, 5.1618E+00, 5.1881E+05, &
      1.396010E-02, 8.78922E+00, 3.7368E+00, 3.5134E+05, &
      1.492335E-02, 8.50205E+00, 2.6492E+00, 2.3300E+05, &
      1.595306E-02, 8.09906E+00, 1.8642E+00, 1.5338E+05, &
      1.705382E-02, 7.67610E+00, 1.3165E+00, 1.0132E+05, &
      1.733447E-02, 7.57568E+00, 1.2110E+00, 9.1697E+04, &
      1.759979E-02, 7.48403E+00, 1.1213E+00, 8.3627E+04, &
      1.767054E-02, 7.46005E+00, 1.0989E+00, 8.1627E+04, &
      1.777667E-02, 7.42486E+00, 1.0667E+00, 7.8762E+04, &
      1.804199E-02, 7.33908E+00, 9.9108E-01, 7.2101E+04, &
      1.823053E-02, 7.28016E+00, 9.4176E-01, 6.7804E+04, &
      1.948844E-02, 6.92902E+00, 6.8765E-01, 4.6314E+04, &
      2.083314E-02, 6.62473E+00, 5.1689E-01, 3.2566E+04, &
      2.227063E-02, 6.36305E+00, 4.0433E-01, 2.3830E+04, &
      2.380730E-02, 6.13733E+00, 3.3392E-01, 1.8410E+04, &
      2.545001E-02, 5.94417E+00, 2.9544E-01, 1.5237E+04, &
      2.720606E-02, 5.77933E+00, 2.8174E-01, 1.3592E+04, &
      2.908327E-02, 5.64047E+00, 2.8698E-01, 1.2951E+04, &
      3.109002E-02, 5.52557E+00, 3.0570E-01, 1.2906E+04, &
      3.323523E-02, 5.43223E+00, 3.3304E-01, 1.3153E+04, &
      3.552846E-02, 5.35763E+00, 3.6459E-01, 1.3469E+04, &
      3.797993E-02, 5.29818E+00, 3.9668E-01, 1.3709E+04, &
      4.060054E-02, 5.24989E+00, 4.2687E-01, 1.3800E+04, &
      4.340198E-02, 5.20867E+00, 4.5409E-01, 1.3733E+04, &
      4.639671E-02, 5.17122E+00, 4.7795E-01, 1.3521E+04, &
      4.959809E-02, 5.13443E+00, 4.9862E-01, 1.3195E+04, &
      5.302035E-02, 5.09561E+00, 5.1680E-01, 1.2794E+04, &
      5.667876E-02, 5.05383E+00, 5.3333E-01, 1.2351E+04, &
      6.058959E-02, 5.00445E+00, 5.4903E-01, 1.1894E+04, &
      6.477028E-02, 4.94551E+00, 5.6524E-01, 1.1454E+04, &
      6.923942E-02, 4.87744E+00, 5.8227E-01, 1.1038E+04, &
      7.401695E-02, 4.79720E+00, 6.0028E-01, 1.0645E+04, &
      7.912411E-02, 4.70176E+00, 6.1932E-01, 1.0274E+04, &
      8.458368E-02, 4.58711E+00, 6.3925E-01, 9.9199E+03, &
      9.041995E-02, 4.44776E+00, 6.5975E-01, 9.5772E+03, &
      9.665893E-02, 4.27591E+00, 6.8034E-01, 9.2386E+03, &
      1.033284E-01, 4.06018E+00, 7.0045E-01, 8.8979E+03, &
      1.104581E-01, 3.78344E+00, 7.1950E-01, 8.5500E+03, &
      1.180797E-01, 3.41854E+00, 7.3688E-01, 8.1913E+03, &
      1.262272E-01, 2.91916E+00, 7.5205E-01, 7.8203E+03, &
      1.349368E-01, 2.19699E+00, 7.6456E-01, 7.4374E+03, &
      1.442475E-01, 1.05025E+00, 7.7408E-01, 7.0440E+03, &
      1.542005E-01, -1.18249E+00, 7.6165E-01, 6.4836E+03, &
      1.615040E-01, -5.49062E+00, 7.4766E-01, 6.0768E+03, &
      1.639760E-01, -1.08763E+01, 7.4264E-01, 5.9450E+03, &
      1.646352E-01, -1.72556E+01, 7.4128E-01, 5.9103E+03, &
      1.648404E-01, -2.28517E+01, 1.3291E+01, 1.0583E+05, &
      1.656240E-01, -1.07387E+01, 1.3261E+01, 1.0509E+05, &
      1.680960E-01, -5.05078E+00, 1.3165E+01, 1.0280E+05, &
      1.762144E-01, 2.68651E-01, 1.2863E+01, 9.5815E+04, &
      1.883732E-01, 3.57484E+00, 1.2429E+01, 8.6607E+04, &
      2.013709E-01, 5.64507E+00, 1.1923E+01, 7.7718E+04, &
      2.152655E-01, 7.05788E+00, 1.1353E+01, 6.9226E+04, &
      2.246160E-01, 7.51754E+00, 1.0965E+01, 6.4073E+04, &
      2.280540E-01, 7.27360E+00, 1.0822E+01, 6.2286E+04, &
      2.289708E-01, 6.72984E+00, 1.0784E+01, 6.1819E+04, &
      2.301188E-01, 7.35124E+00, 1.1942E+01, 6.8113E+04, &
      2.303460E-01, 7.45350E+00, 1.1933E+01, 6.7995E+04, &
      2.337840E-01, 8.24499E+00, 1.1797E+01, 6.6234E+04, &
      2.459970E-01, 9.56827E+00, 1.1322E+01, 6.0412E+04, &
      2.629708E-01, 1.07717E+01, 1.0685E+01, 5.3331E+04, &
      2.811158E-01, 1.17286E+01, 1.0032E+01, 4.6841E+04, &
      3.005128E-01, 1.25150E+01, 9.3747E+00, 4.0947E+04, &
      3.212482E-01, 1.31657E+01, 8.7232E+00, 3.5642E+04, &
      3.434143E-01, 1.37032E+01, 8.0847E+00, 3.0901E+04, &
      3.671099E-01, 1.41439E+01, 7.4660E+00, 2.6696E+04, &
      3.924405E-01, 1.45017E+01, 6.8727E+00, 2.2988E+04, &
      4.195189E-01, 1.47886E+01, 6.3084E+00, 1.9739E+04, &
      4.484657E-01, 1.50149E+01, 5.7755E+00, 1.6906E+04, &
      4.794098E-01, 1.51896E+01, 5.2754E+00, 1.4445E+04, &
      5.124891E-01, 1.53221E+01, 4.8084E+00, 1.2317E+04, &
      5.478508E-01, 1.54165E+01, 4.3745E+00, 1.0482E+04, &
      5.856525E-01, 1.54802E+01, 3.9729E+00, 8.9062E+03, &
      6.260625E-01, 1.55185E+01, 3.6027E+00, 7.5553E+03, &
      6.692609E-01, 1.55359E+01, 3.2623E+00, 6.4003E+03, &
      7.154399E-01, 1.55359E+01, 2.9496E+00, 5.4137E+03, &
      7.648052E-01, 1.55214E+01, 2.6633E+00, 4.5731E+03, &
      8.175768E-01, 1.54951E+01, 2.4023E+00, 3.8592E+03, &
      8.739896E-01, 1.54600E+01, 2.1653E+00, 3.2543E+03, &
      9.342948E-01, 1.54191E+01, 1.9505E+00, 2.7428E+03, &
      9.987612E-01, 1.53770E+01, 1.7565E+00, 2.3109E+03, &
      1.067676E+00, 1.53185E+01, 1.5563E+00, 1.9158E+03, &
      1.141345E+00, 1.52465E+01, 1.3799E+00, 1.5895E+03, &
      1.220098E+00, 1.51657E+01, 1.2248E+00, 1.3203E+03, &
      1.304285E+00, 1.50779E+01, 1.0883E+00, 1.0979E+03, &
      1.394281E+00, 1.49834E+01, 9.6802E-01, 9.1392E+02, &
      1.490486E+00, 1.48807E+01, 8.6197E-01, 7.6170E+02, &
      1.593329E+00, 1.47671E+01, 7.6836E-01, 6.3557E+02, &
      1.703269E+00, 1.46378E+01, 6.8566E-01, 5.3097E+02, &
      1.820795E+00, 1.44842E+01, 6.1253E-01, 4.4410E+02, &
      1.946430E+00, 1.42907E+01, 5.4778E-01, 3.7190E+02, &
      2.080733E+00, 1.40245E+01, 4.9041E-01, 3.1181E+02, &
      2.224304E+00, 1.35997E+01, 4.3951E-01, 2.6176E+02, &
      2.377781E+00, 1.26322E+01, 3.9431E-01, 2.2001E+02, &
      2.422560E+00, 1.19404E+01, 3.8259E-01, 2.0962E+02, &
      2.459640E+00, 1.03757E+01, 3.7333E-01, 2.0154E+02, &
      2.469528E+00, 8.49659E+00, 3.7093E-01, 1.9946E+02, &
      2.484360E+00, 1.03922E+01, 4.0700E+00, 2.1526E+03, &
      2.521440E+00, 1.21135E+01, 3.9926E+00, 2.0807E+03, &
      2.541848E+00, 1.25569E+01, 3.9512E+00, 2.0426E+03, &
      2.717235E+00, 1.42376E+01, 3.5845E+00, 1.7337E+03, &
      2.904724E+00, 1.49741E+01, 3.2216E+00, 1.4578E+03, &
      3.105150E+00, 1.54198E+01, 2.8955E+00, 1.2260E+03, &
      3.319406E+00, 1.57231E+01, 2.5973E+00, 1.0290E+03, &
      3.548445E+00, 1.59311E+01, 2.3237E+00, 8.6145E+02, &
      3.793288E+00, 1.60769E+01, 2.0792E+00, 7.2127E+02, &
      4.055024E+00, 1.61802E+01, 1.8606E+00, 6.0400E+02, &
      4.334821E+00, 1.62533E+01, 1.6650E+00, 5.0583E+02, &
      4.633924E+00, 1.63047E+01, 1.4893E+00, 4.2343E+02, &
      4.953664E+00, 1.63850E+01, 1.3318E+00, 3.5442E+02, &
      5.295467E+00, 1.64057E+01, 1.1801E+00, 2.9395E+02, &
      5.660855E+00, 1.64117E+01, 1.0457E+00, 2.4384E+02, &
      6.051453E+00, 1.64086E+01, 9.2670E-01, 2.0230E+02, &
      6.469004E+00, 1.63993E+01, 8.2127E-01, 1.6786E+02, &
      6.915365E+00, 1.63859E+01, 7.2786E-01, 1.3931E+02, &
      7.392525E+00, 1.63698E+01, 6.4511E-01, 1.1563E+02, &
      7.902609E+00, 1.63522E+01, 5.7180E-01, 9.5998E+01, &
      8.447890E+00, 1.63342E+01, 5.0681E-01, 7.9710E+01, &
      9.030794E+00, 1.63180E+01, 4.4855E-01, 6.6101E+01, &
      9.653919E+00, 1.62986E+01, 3.9488E-01, 5.4539E+01, &
      1.032004E+01, 1.62792E+01, 3.4764E-01, 4.5014E+01, &
      1.103212E+01, 1.62603E+01, 3.0521E-01, 3.7062E+01, &
      1.179334E+01, 1.62413E+01, 2.6702E-01, 3.0421E+01, &
      1.260708E+01, 1.62226E+01, 2.3361E-01, 2.4981E+01, &
      1.347697E+01, 1.62045E+01, 2.0439E-01, 2.0523E+01, &
      1.440688E+01, 1.61874E+01, 1.7883E-01, 1.6872E+01, &
      1.540095E+01, 1.61713E+01, 1.5647E-01, 1.3878E+01, &
      1.646362E+01, 1.61562E+01, 1.3691E-01, 1.1424E+01, &
      1.759961E+01, 1.61422E+01, 1.1980E-01, 9.4130E+00, &
      1.881398E+01, 1.61293E+01, 1.0483E-01, 7.7634E+00, &
      2.011215E+01, 1.61174E+01, 9.1735E-02, 6.4100E+00, &
      2.149988E+01, 1.61064E+01, 8.0275E-02, 5.2996E+00, &
      2.298338E+01, 1.60964E+01, 7.0249E-02, 4.3883E+00, &
      2.456923E+01, 1.60873E+01, 6.1476E-02, 3.6400E+00, &
      2.626450E+01, 1.60790E+01, 5.3800E-02, 3.0255E+00, &
      2.807676E+01, 1.60715E+01, 4.6793E-02, 2.5070E+00, &
      3.001405E+01, 1.60645E+01, 4.0617E-02, 2.0798E+00, &
      3.208502E+01, 1.60581E+01, 3.5256E-02, 1.7314E+00, &
      3.429889E+01, 1.60523E+01, 3.0603E-02, 1.4468E+00, &
      3.666551E+01, 1.60470E+01, 2.6565E-02, 1.2145E+00, &
      3.919543E+01, 1.60421E+01, 2.3060E-02, 1.0245E+00, &
      4.189992E+01, 1.60378E+01, 2.0017E-02, 8.6911E-01, &
      4.479101E+01, 1.60338E+01, 1.7377E-02, 7.4183E-01, &
      4.788159E+01, 1.60303E+01, 1.5084E-02, 6.3747E-01, &
      5.118542E+01, 1.60271E+01, 1.3095E-02, 5.5177E-01, &
      5.471721E+01, 1.60242E+01, 1.1346E-02, 4.8081E-01, &
      5.849270E+01, 1.60216E+01, 9.8178E-03, 4.2211E-01, &
      6.252870E+01, 1.60193E+01, 8.4952E-03, 3.7379E-01, &
      6.684318E+01, 1.60172E+01, 7.3509E-03, 3.3390E-01, &
      7.145536E+01, 1.60153E+01, 6.3607E-03, 3.0088E-01, &
      7.638578E+01, 1.60137E+01, 5.5040E-03, 2.7343E-01, &
      8.165640E+01, 1.60122E+01, 4.7619E-03, 2.5050E-01, &
      8.729069E+01, 1.60108E+01, 4.1184E-03, 2.3127E-01, &
      9.331374E+01, 1.60096E+01, 3.5618E-03, 2.1504E-01, &
      9.975239E+01, 1.60086E+01, 3.0804E-03, 2.0127E-01, &
      1.066353E+02, 1.60076E+01, 2.6642E-03, 1.8951E-01, &
      1.139931E+02, 1.60067E+01, 2.3041E-03, 1.7938E-01, &
      1.218587E+02, 1.60060E+01, 1.9928E-03, 1.7058E-01, &
      1.302669E+02, 1.60053E+01, 1.7235E-03, 1.6287E-01, &
      1.392553E+02, 1.60047E+01, 1.4906E-03, 1.5604E-01, &
      1.488640E+02, 1.60042E+01, 1.2892E-03, 1.4995E-01, &
      1.591356E+02, 1.60037E+01, 1.1150E-03, 1.4444E-01, &
      1.701159E+02, 1.60033E+01, 9.6437E-04, 1.3943E-01, &
      1.818539E+02, 1.60029E+01, 8.3407E-04, 1.3481E-01, &
      1.944018E+02, 1.60026E+01, 7.2139E-04, 1.3054E-01, &
      2.078156E+02, 1.60023E+01, 6.2393E-04, 1.2653E-01, &
      2.221548E+02, 1.60020E+01, 5.3963E-04, 1.2277E-01, &
      2.374835E+02, 1.60018E+01, 4.6673E-04, 1.1919E-01, &
      2.538699E+02, 1.60016E+01, 4.0368E-04, 1.1579E-01, &
      2.713869E+02, 1.60014E+01, 3.4915E-04, 1.1252E-01, &
      2.901126E+02, 1.60012E+01, 3.0198E-04, 1.0938E-01, &
      3.101304E+02, 1.60011E+01, 2.6119E-04, 1.0635E-01, &
      3.315294E+02, 1.60010E+01, 2.2591E-04, 1.0340E-01, &
      3.544049E+02, 1.60009E+01, 1.9539E-04, 1.0056E-01, &
      3.788588E+02, 1.60008E+01, 1.6900E-04, 9.7786E-02, &
      4.050001E+02, 1.60007E+01, 1.4617E-04, 9.5092E-02, &
      4.329451E+02, 1.60006E+01, 1.2643E-04, 9.2467E-02]
   case(17)
      frel = -2.4600E-02
      allocate(nist_tab(760))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.664000E-03, -6.48754E+00, 0.0000E+00, 1.6791E-05, &
      6.766000E-03, -8.71036E+00, 0.0000E+00, 1.8145E-05, &
      6.793200E-03, -1.09679E+01, 0.0000E+00, 1.8519E-05, &
      6.834000E-03, -9.15794E+00, 4.2153E+00, 7.3211E+05, &
      6.936000E-03, -7.81269E+00, 4.6476E+00, 7.9532E+05, &
      1.069000E-02, 8.13522E+00, 1.0632E+01, 1.1805E+06, &
      1.142761E-02, 9.37473E+00, 8.5751E+00, 8.9066E+05, &
      1.221612E-02, 9.87659E+00, 6.6572E+00, 6.4682E+05, &
      1.305903E-02, 9.88286E+00, 5.0456E+00, 4.5859E+05, &
      1.396010E-02, 9.60491E+00, 3.7778E+00, 3.2120E+05, &
      1.492335E-02, 9.19403E+00, 2.8220E+00, 2.2445E+05, &
      1.595306E-02, 8.74430E+00, 2.1201E+00, 1.5774E+05, &
      1.705382E-02, 8.30670E+00, 1.6121E+00, 1.1220E+05, &
      1.715000E-02, 8.27119E+00, 1.5764E+00, 1.0910E+05, &
      1.741250E-02, 8.17619E+00, 1.4849E+00, 1.0122E+05, &
      1.748250E-02, 8.15072E+00, 1.4619E+00, 9.9249E+04, &
      1.758750E-02, 8.11518E+00, 1.4302E+00, 9.6518E+04, &
      1.785000E-02, 8.02747E+00, 1.3517E+00, 8.9881E+04, &
      1.823053E-02, 7.90572E+00, 1.2497E+00, 8.1361E+04, &
      1.948844E-02, 7.54978E+00, 9.9072E-01, 6.0339E+04, &
      2.083314E-02, 7.24039E+00, 8.0826E-01, 4.6049E+04, &
      2.227063E-02, 6.97002E+00, 6.8213E-01, 3.6355E+04, &
      2.380730E-02, 6.73643E+00, 5.9833E-01, 2.9830E+04, &
      2.545001E-02, 6.53547E+00, 5.4685E-01, 2.5504E+04, &
      2.720606E-02, 6.36342E+00, 5.2018E-01, 2.2694E+04, &
      2.908327E-02, 6.21710E+00, 5.1307E-01, 2.0939E+04, &
      3.109002E-02, 6.09417E+00, 5.2094E-01, 1.9888E+04, &
      3.323523E-02, 5.99205E+00, 5.3952E-01, 1.9268E+04, &
      3.552846E-02, 5.90798E+00, 5.6545E-01, 1.8890E+04, &
      3.797993E-02, 5.83917E+00, 5.9621E-01, 1.8633E+04, &
      4.060054E-02, 5.78294E+00, 6.3002E-01, 1.8418E+04, &
      4.340198E-02, 5.73680E+00, 6.6568E-01, 1.8205E+04, &
      4.639671E-02, 5.69856E+00, 7.0249E-01, 1.7971E+04, &
      4.959809E-02, 5.66627E+00, 7.4008E-01, 1.7711E+04, &
      5.302035E-02, 5.64012E+00, 7.7828E-01, 1.7423E+04, &
      5.667876E-02, 5.61503E+00, 8.1703E-01, 1.7110E+04, &
      6.058959E-02, 5.59156E+00, 8.5625E-01, 1.6774E+04, &
      6.477028E-02, 5.56850E+00, 8.9580E-01, 1.6416E+04, &
      6.923942E-02, 5.54462E+00, 9.3544E-01, 1.6036E+04, &
      7.401695E-02, 5.51848E+00, 9.7470E-01, 1.5630E+04, &
      7.912411E-02, 5.48825E+00, 1.0130E+00, 1.5196E+04, &
      8.458368E-02, 5.45162E+00, 1.0498E+00, 1.4731E+04, &
      9.041995E-02, 5.40568E+00, 1.0844E+00, 1.4235E+04, &
      9.665893E-02, 5.34667E+00, 1.1163E+00, 1.3707E+04, &
      1.033284E-01, 5.26968E+00, 1.1447E+00, 1.3149E+04, &
      1.104581E-01, 5.16820E+00, 1.1690E+00, 1.2562E+04, &
      1.180797E-01, 5.03347E+00, 1.1889E+00, 1.1951E+04, &
      1.262272E-01, 4.85335E+00, 1.2040E+00, 1.1321E+04, &
      1.349368E-01, 4.60508E+00, 1.1885E+00, 1.0455E+04, &
      1.442475E-01, 4.24737E+00, 1.1615E+00, 9.5577E+03, &
      1.542005E-01, 3.72741E+00, 1.1286E+00, 8.6877E+03, &
      1.648404E-01, 2.94256E+00, 1.0906E+00, 7.8536E+03, &
      1.762144E-01, 1.65231E+00, 1.0484E+00, 7.0622E+03, &
      1.883732E-01, -9.63484E-01, 1.0027E+00, 6.3189E+03, &
      1.960000E-01, -5.11596E+00, 9.7436E-01, 5.9013E+03, &
      1.975680E-01, -7.04464E+00, 9.6858E-01, 5.8197E+03, &
      1.990000E-01, -1.03536E+01, 9.6331E-01, 5.7465E+03, &
      1.998000E-01, -1.58375E+01, 9.6038E-01, 5.7060E+03, &
      2.005920E-01, -1.32492E+01, 1.0701E+01, 6.3322E+04, &
      2.010000E-01, -1.23713E+01, 1.0680E+01, 6.3066E+04, &
      2.013709E-01, -1.28311E+01, 1.0660E+01, 6.2835E+04, &
      2.013984E-01, -1.29630E+01, 1.0659E+01, 6.2818E+04, &
      2.026080E-01, -8.42225E+00, 1.5407E+01, 9.0258E+04, &
      2.040000E-01, -5.64408E+00, 1.5302E+01, 8.9035E+04, &
      2.056320E-01, -3.66881E+00, 1.5182E+01, 8.7633E+04, &
      2.152655E-01, 1.76525E+00, 1.4507E+01, 7.9990E+04, &
      2.301188E-01, 5.36699E+00, 1.3570E+01, 6.9996E+04, &
      2.459970E-01, 7.46894E+00, 1.2673E+01, 6.1146E+04, &
      2.629708E-01, 8.63568E+00, 1.1807E+01, 5.3291E+04, &
      2.647960E-01, 8.66322E+00, 1.1719E+01, 5.2529E+04, &
      2.688490E-01, 8.40100E+00, 1.1526E+01, 5.0888E+04, &
      2.699298E-01, 7.81491E+00, 1.1476E+01, 5.0462E+04, &
      2.715510E-01, 8.59454E+00, 1.2699E+01, 5.5508E+04, &
      2.756040E-01, 9.44336E+00, 1.2518E+01, 5.3912E+04, &
      2.811158E-01, 1.00833E+01, 1.2278E+01, 5.1843E+04, &
      3.005128E-01, 1.15368E+01, 1.1490E+01, 4.5381E+04, &
      3.212482E-01, 1.26019E+01, 1.0723E+01, 3.9621E+04, &
      3.434143E-01, 1.34520E+01, 9.9762E+00, 3.4483E+04, &
      3.671099E-01, 1.41442E+01, 9.2522E+00, 2.9916E+04, &
      3.924405E-01, 1.47107E+01, 8.5551E+00, 2.5877E+04, &
      4.195189E-01, 1.51732E+01, 7.8877E+00, 2.2318E+04, &
      4.484657E-01, 1.55481E+01, 7.2524E+00, 1.9197E+04, &
      4.794098E-01, 1.58486E+01, 6.6510E+00, 1.6469E+04, &
      5.124891E-01, 1.60858E+01, 6.0847E+00, 1.4094E+04, &
      5.478508E-01, 1.62691E+01, 5.5541E+00, 1.2035E+04, &
      5.856525E-01, 1.64066E+01, 5.0594E+00, 1.0256E+04, &
      6.260625E-01, 1.65034E+01, 4.5999E+00, 8.7232E+03, &
      6.692609E-01, 1.65699E+01, 4.1750E+00, 7.4067E+03, &
      7.154399E-01, 1.66094E+01, 3.7833E+00, 6.2791E+03, &
      7.648052E-01, 1.66268E+01, 3.4235E+00, 5.3156E+03, &
      8.175768E-01, 1.66272E+01, 3.0938E+00, 4.4941E+03, &
      8.739896E-01, 1.66118E+01, 2.7921E+00, 3.7945E+03, &
      9.342948E-01, 1.65843E+01, 2.5173E+00, 3.2007E+03, &
      9.987612E-01, 1.65482E+01, 2.2683E+00, 2.6983E+03, &
      1.067676E+00, 1.65280E+01, 2.0210E+00, 2.2494E+03, &
      1.141345E+00, 1.64667E+01, 1.7948E+00, 1.8693E+03, &
      1.220098E+00, 1.63923E+01, 1.5940E+00, 1.5535E+03, &
      1.304285E+00, 1.63103E+01, 1.4171E+00, 1.2924E+03, &
      1.394281E+00, 1.62230E+01, 1.2611E+00, 1.0764E+03, &
      1.490486E+00, 1.61307E+01, 1.1235E+00, 8.9740E+02, &
      1.593329E+00, 1.60330E+01, 1.0019E+00, 7.4903E+02, &
      1.703269E+00, 1.59283E+01, 8.9432E-01, 6.2589E+02, &
      1.820795E+00, 1.58134E+01, 7.9917E-01, 5.2360E+02, &
      1.946430E+00, 1.56835E+01, 7.1488E-01, 4.3853E+02, &
      2.080733E+00, 1.55299E+01, 6.4014E-01, 3.6770E+02, &
      2.224304E+00, 1.53369E+01, 5.7382E-01, 3.0869E+02, &
      2.377781E+00, 1.50720E+01, 5.1489E-01, 2.5945E+02, &
      2.541848E+00, 1.46490E+01, 4.6250E-01, 2.1833E+02, &
      2.717235E+00, 1.36771E+01, 4.1585E-01, 1.8394E+02, &
      2.765952E+00, 1.30165E+01, 4.0432E-01, 1.7577E+02, &
      2.808288E+00, 1.14668E+01, 3.9473E-01, 1.6908E+02, &
      2.819578E+00, 9.60284E+00, 3.9224E-01, 1.6737E+02, &
      2.836512E+00, 1.14882E+01, 4.0625E+00, 1.7022E+03, &
      2.878848E+00, 1.32000E+01, 3.9785E+00, 1.6425E+03, &
      2.904724E+00, 1.36816E+01, 3.9285E+00, 1.6075E+03, &
      3.105150E+00, 1.53069E+01, 3.5410E+00, 1.3556E+03, &
      3.319406E+00, 1.60128E+01, 3.1795E+00, 1.1390E+03, &
      3.548445E+00, 1.64430E+01, 2.8622E+00, 9.5936E+02, &
      3.793288E+00, 1.67430E+01, 2.5699E+00, 8.0603E+02, &
      4.055024E+00, 1.69472E+01, 2.2984E+00, 6.7457E+02, &
      4.334821E+00, 1.70895E+01, 2.0558E+00, 5.6464E+02, &
      4.633924E+00, 1.71898E+01, 1.8390E+00, 4.7270E+02, &
      4.953664E+00, 1.72603E+01, 1.6453E+00, 3.9579E+02, &
      5.295467E+00, 1.73095E+01, 1.4714E+00, 3.3130E+02, &
      5.660855E+00, 1.73868E+01, 1.3153E+00, 2.7719E+02, &
      6.051453E+00, 1.74057E+01, 1.1664E+00, 2.3012E+02, &
      6.469004E+00, 1.74110E+01, 1.0345E+00, 1.9106E+02, &
      6.915365E+00, 1.74076E+01, 9.1748E-01, 1.5866E+02, &
      7.392525E+00, 1.73984E+01, 8.1377E-01, 1.3178E+02, &
      7.902609E+00, 1.73854E+01, 7.2183E-01, 1.0946E+02, &
      8.447890E+00, 1.73702E+01, 6.4026E-01, 9.0948E+01, &
      9.030794E+00, 1.73541E+01, 5.6708E-01, 7.5462E+01, &
      9.653919E+00, 1.73362E+01, 4.9958E-01, 6.2296E+01, &
      1.032004E+01, 1.73166E+01, 4.4014E-01, 5.1440E+01, &
      1.103212E+01, 1.72966E+01, 3.8779E-01, 4.2488E+01, &
      1.179334E+01, 1.72769E+01, 3.4169E-01, 3.5107E+01, &
      1.260708E+01, 1.72579E+01, 3.0019E-01, 2.8934E+01, &
      1.347697E+01, 1.72389E+01, 2.6280E-01, 2.3774E+01, &
      1.440688E+01, 1.72202E+01, 2.3008E-01, 1.9546E+01, &
      1.540095E+01, 1.72023E+01, 2.0144E-01, 1.6077E+01, &
      1.646362E+01, 1.71853E+01, 1.7637E-01, 1.3234E+01, &
      1.759961E+01, 1.71694E+01, 1.5443E-01, 1.0901E+01, &
      1.881398E+01, 1.71545E+01, 1.3522E-01, 8.9865E+00, &
      2.011215E+01, 1.71407E+01, 1.1840E-01, 7.4160E+00, &
      2.149988E+01, 1.71279E+01, 1.0368E-01, 6.1268E+00, &
      2.298338E+01, 1.71161E+01, 9.0790E-02, 5.0684E+00, &
      2.456923E+01, 1.71054E+01, 7.9504E-02, 4.1990E+00, &
      2.626450E+01, 1.70955E+01, 6.9623E-02, 3.4850E+00, &
      2.807676E+01, 1.70867E+01, 6.0598E-02, 2.8823E+00, &
      3.001405E+01, 1.70785E+01, 5.2638E-02, 2.3855E+00, &
      3.208502E+01, 1.70709E+01, 4.5724E-02, 1.9802E+00, &
      3.429889E+01, 1.70639E+01, 3.9719E-02, 1.6493E+00, &
      3.666551E+01, 1.70575E+01, 3.4503E-02, 1.3790E+00, &
      3.919543E+01, 1.70517E+01, 2.9972E-02, 1.1581E+00, &
      4.189992E+01, 1.70464E+01, 2.6037E-02, 9.7729E-01, &
      4.479101E+01, 1.70416E+01, 2.2619E-02, 8.2929E-01, &
      4.788159E+01, 1.70373E+01, 1.9650E-02, 7.0796E-01, &
      5.118542E+01, 1.70334E+01, 1.7070E-02, 6.0840E-01, &
      5.471721E+01, 1.70299E+01, 1.4830E-02, 5.2658E-01, &
      5.849270E+01, 1.70267E+01, 1.2883E-02, 4.5923E-01, &
      6.252870E+01, 1.70239E+01, 1.1169E-02, 4.0323E-01, &
      6.684318E+01, 1.70214E+01, 9.6669E-03, 3.5676E-01, &
      7.145536E+01, 1.70191E+01, 8.3672E-03, 3.1838E-01, &
      7.638578E+01, 1.70170E+01, 7.2424E-03, 2.8658E-01, &
      8.165640E+01, 1.70151E+01, 6.2681E-03, 2.6012E-01, &
      8.729069E+01, 1.70135E+01, 5.4237E-03, 2.3801E-01, &
      9.331374E+01, 1.70120E+01, 4.6931E-03, 2.1946E-01, &
      9.975239E+01, 1.70107E+01, 4.0609E-03, 2.0380E-01, &
      1.066353E+02, 1.70095E+01, 3.5139E-03, 1.9050E-01, &
      1.139931E+02, 1.70084E+01, 3.0406E-03, 1.7914E-01, &
      1.218587E+02, 1.70075E+01, 2.6311E-03, 1.6934E-01, &
      1.302669E+02, 1.70067E+01, 2.2767E-03, 1.6082E-01, &
      1.392553E+02, 1.70059E+01, 1.9701E-03, 1.5336E-01, &
      1.488640E+02, 1.70052E+01, 1.7048E-03, 1.4676E-01, &
      1.591356E+02, 1.70047E+01, 1.4752E-03, 1.4086E-01, &
      1.701159E+02, 1.70041E+01, 1.2765E-03, 1.3555E-01, &
      1.818539E+02, 1.70037E+01, 1.1046E-03, 1.3071E-01, &
      1.944018E+02, 1.70032E+01, 9.5585E-04, 1.2626E-01, &
      2.078156E+02, 1.70029E+01, 8.2714E-04, 1.2214E-01, &
      2.221548E+02, 1.70025E+01, 7.1576E-04, 1.1830E-01, &
      2.374835E+02, 1.70022E+01, 6.1938E-04, 1.1470E-01, &
      2.538699E+02, 1.70020E+01, 5.3598E-04, 1.1129E-01, &
      2.713869E+02, 1.70018E+01, 4.6381E-04, 1.0804E-01, &
      2.901126E+02, 1.70016E+01, 4.0136E-04, 1.0494E-01, &
      3.101304E+02, 1.70014E+01, 3.4732E-04, 1.0197E-01, &
      3.315294E+02, 1.70012E+01, 3.0056E-04, 9.9110E-02, &
      3.544049E+02, 1.70011E+01, 2.6009E-04, 9.6350E-02, &
      3.788588E+02, 1.70010E+01, 2.2508E-04, 9.3681E-02, &
      4.050001E+02, 1.70008E+01, 1.9477E-04, 9.1096E-02, &
      4.329451E+02, 1.70007E+01, 1.6855E-04, 8.8588E-02]
   case(18)
      frel = -2.8200E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, -2.55688E+00, 0.0000E+00, 2.0878E-04, &
      1.142761E-02, -4.00996E+00, 0.0000E+00, 2.7977E-04, &
      1.215200E-02, -7.07227E+00, 0.0000E+00, 3.6463E-04, &
      1.221612E-02, -7.64770E+00, 0.0000E+00, 3.7292E-04, &
      1.233800E-02, -9.53105E+00, 0.0000E+00, 3.8904E-04, &
      1.238760E-02, -1.19746E+01, 0.0000E+00, 3.9573E-04, &
      1.246200E-02, -1.00930E+01, 4.5247E+00, 3.8246E+05, &
      1.264800E-02, -8.75530E+00, 5.0555E+00, 4.2104E+05, &
      1.305903E-02, -8.32095E+00, 6.4235E+00, 5.1814E+05, &
      1.396010E-02, -7.63879E+00, 1.0449E+01, 7.8846E+05, &
      1.492335E-02, -4.68957E+00, 1.4367E+01, 1.0141E+06, &
      1.595306E-02, -1.84234E-01, 1.6500E+01, 1.0895E+06, &
      1.705382E-02, 4.52316E+00, 1.6433E+01, 1.0150E+06, &
      1.823053E-02, 8.33043E+00, 1.4663E+01, 8.4723E+05, &
      1.948844E-02, 1.07845E+01, 1.2053E+01, 6.5147E+05, &
      2.083314E-02, 1.19786E+01, 9.3462E+00, 4.7257E+05, &
      2.227063E-02, 1.22589E+01, 6.9752E+00, 3.2992E+05, &
      2.380730E-02, 1.19916E+01, 5.0945E+00, 2.2541E+05, &
      2.479400E-02, 1.16829E+01, 4.1883E+00, 1.7794E+05, &
      2.517350E-02, 1.15494E+01, 3.8917E+00, 1.6285E+05, &
      2.527470E-02, 1.15101E+01, 3.8170E+00, 1.5908E+05, &
      2.545001E-02, 1.14521E+01, 3.6993E+00, 1.5311E+05, &
      2.580600E-02, 1.13281E+01, 3.4612E+00, 1.4128E+05, &
      2.720606E-02, 1.08349E+01, 2.6975E+00, 1.0444E+05, &
      2.908327E-02, 1.02231E+01, 1.9988E+00, 7.2397E+04, &
      3.109002E-02, 9.66544E+00, 1.5200E+00, 5.1499E+04, &
      3.323523E-02, 9.17783E+00, 1.1966E+00, 3.7927E+04, &
      3.552846E-02, 8.76122E+00, 9.8222E-01, 2.9122E+04, &
      3.797993E-02, 8.40482E+00, 8.4376E-01, 2.3402E+04, &
      4.060054E-02, 8.11116E+00, 7.5813E-01, 1.9670E+04, &
      4.340198E-02, 7.86667E+00, 7.0911E-01, 1.7210E+04, &
      4.639671E-02, 7.66343E+00, 6.8529E-01, 1.5559E+04, &
      4.959809E-02, 7.49416E+00, 6.7864E-01, 1.4413E+04, &
      5.302035E-02, 7.35244E+00, 6.8408E-01, 1.3591E+04, &
      5.667876E-02, 7.23317E+00, 6.9799E-01, 1.2972E+04, &
      6.058959E-02, 7.13172E+00, 7.1754E-01, 1.2475E+04, &
      6.477028E-02, 7.04399E+00, 7.4100E-01, 1.2051E+04, &
      6.923942E-02, 6.96644E+00, 7.6735E-01, 1.1674E+04, &
      7.401695E-02, 6.89605E+00, 7.9601E-01, 1.1328E+04, &
      7.912411E-02, 6.83218E+00, 8.2669E-01, 1.1006E+04, &
      8.458368E-02, 6.76853E+00, 8.5920E-01, 1.0700E+04, &
      9.041995E-02, 6.70486E+00, 8.9335E-01, 1.0407E+04, &
      9.665893E-02, 6.63892E+00, 9.2887E-01, 1.0123E+04, &
      1.033284E-01, 6.56824E+00, 9.6534E-01, 9.8415E+03, &
      1.104581E-01, 6.48974E+00, 1.0020E+00, 9.5561E+03, &
      1.180797E-01, 6.39944E+00, 1.0382E+00, 9.2622E+03, &
      1.262272E-01, 6.29218E+00, 1.0731E+00, 8.9555E+03, &
      1.349368E-01, 6.16102E+00, 1.1058E+00, 8.6329E+03, &
      1.442475E-01, 5.99631E+00, 1.1354E+00, 8.2919E+03, &
      1.542005E-01, 5.78428E+00, 1.1610E+00, 7.9318E+03, &
      1.648404E-01, 5.50443E+00, 1.1819E+00, 7.5530E+03, &
      1.762144E-01, 5.12448E+00, 1.1973E+00, 7.1578E+03, &
      1.883732E-01, 4.58949E+00, 1.2069E+00, 6.7496E+03, &
      2.013709E-01, 3.79424E+00, 1.2104E+00, 6.3324E+03, &
      2.152655E-01, 2.49575E+00, 1.2079E+00, 5.9116E+03, &
      2.301188E-01, -1.13261E-01, 1.1996E+00, 5.4922E+03, &
      2.402960E-01, -4.75294E+00, 1.1754E+00, 5.1538E+03, &
      2.423540E-01, -7.01147E+00, 1.1692E+00, 5.0830E+03, &
      2.439740E-01, -1.03826E+01, 1.1643E+00, 5.0280E+03, &
      2.449548E-01, -1.63309E+01, 1.1613E+00, 4.9950E+03, &
      2.459970E-01, -1.31494E+01, 1.1814E+01, 5.0587E+04, &
      2.460635E-01, -1.29562E+01, 1.1809E+01, 5.0556E+04, &
      2.464260E-01, -1.23010E+01, 1.1787E+01, 5.0385E+04, &
      2.470527E-01, -1.29419E+01, 1.1748E+01, 5.0090E+04, &
      2.485365E-01, -8.06178E+00, 1.6906E+01, 7.1652E+04, &
      2.501040E-01, -5.23505E+00, 1.6766E+01, 7.0616E+04, &
      2.522460E-01, -2.88933E+00, 1.6579E+01, 6.9234E+04, &
      2.629708E-01, 2.70864E+00, 1.5693E+01, 6.2861E+04, &
      2.811158E-01, 6.72406E+00, 1.4375E+01, 5.3865E+04, &
      3.005128E-01, 8.87011E+00, 1.3194E+01, 4.6248E+04, &
      3.136000E-01, 9.52780E+00, 1.2497E+01, 4.1979E+04, &
      3.184000E-01, 9.27500E+00, 1.2259E+01, 4.0556E+04, &
      3.196800E-01, 8.65940E+00, 1.2196E+01, 4.0189E+04, &
      3.212482E-01, 9.36235E+00, 1.3497E+01, 4.4258E+04, &
      3.216000E-01, 9.49472E+00, 1.3480E+01, 4.4155E+04, &
      3.264000E-01, 1.04119E+01, 1.3257E+01, 4.2786E+04, &
      3.434143E-01, 1.19186E+01, 1.2521E+01, 3.8407E+04, &
      3.671099E-01, 1.32280E+01, 1.1611E+01, 3.3319E+04, &
      3.924405E-01, 1.42235E+01, 1.0750E+01, 2.8856E+04, &
      4.195189E-01, 1.50147E+01, 9.9311E+00, 2.4938E+04, &
      4.484657E-01, 1.56524E+01, 9.1532E+00, 2.1501E+04, &
      4.794098E-01, 1.61677E+01, 8.4154E+00, 1.8493E+04, &
      5.124891E-01, 1.65820E+01, 7.7167E+00, 1.5863E+04, &
      5.478508E-01, 1.69113E+01, 7.0589E+00, 1.3574E+04, &
      5.856525E-01, 1.71691E+01, 6.4422E+00, 1.1589E+04, &
      6.260625E-01, 1.73665E+01, 5.8666E+00, 9.8731E+03, &
      6.692609E-01, 1.75131E+01, 5.3316E+00, 8.3942E+03, &
      7.154399E-01, 1.76172E+01, 4.8365E+00, 7.1236E+03, &
      7.648052E-01, 1.76890E+01, 4.3801E+00, 6.0354E+03, &
      8.175768E-01, 1.77293E+01, 3.9608E+00, 5.1058E+03, &
      8.739896E-01, 1.77465E+01, 3.5767E+00, 4.3136E+03, &
      9.342948E-01, 1.77457E+01, 3.2260E+00, 3.6399E+03, &
      9.987612E-01, 1.77336E+01, 2.9066E+00, 3.0684E+03, &
      1.067676E+00, 1.77035E+01, 2.5899E+00, 2.5580E+03, &
      1.141345E+00, 1.76519E+01, 2.3089E+00, 2.1337E+03, &
      1.220098E+00, 1.75882E+01, 2.0599E+00, 1.7813E+03, &
      1.304285E+00, 1.75176E+01, 1.8372E+00, 1.4866E+03, &
      1.394281E+00, 1.74390E+01, 1.6349E+00, 1.2380E+03, &
      1.490486E+00, 1.73539E+01, 1.4562E+00, 1.0319E+03, &
      1.593329E+00, 1.72641E+01, 1.2982E+00, 8.6103E+02, &
      1.703269E+00, 1.71699E+01, 1.1585E+00, 7.1919E+02, &
      1.820795E+00, 1.70705E+01, 1.0348E+00, 6.0134E+02, &
      1.946430E+00, 1.69641E+01, 9.2523E-01, 5.0335E+02, &
      2.080733E+00, 1.68476E+01, 8.2805E-01, 4.2177E+02, &
      2.224304E+00, 1.67157E+01, 7.4180E-01, 3.5381E+02, &
      2.377781E+00, 1.65595E+01, 6.6517E-01, 2.9713E+02, &
      2.541848E+00, 1.63623E+01, 5.9704E-01, 2.4980E+02, &
      2.717235E+00, 1.60891E+01, 5.3641E-01, 2.1026E+02, &
      2.904724E+00, 1.56440E+01, 4.8240E-01, 1.7717E+02, &
      3.105150E+00, 1.45379E+01, 4.2981E-01, 1.4797E+02, &
      3.138842E+00, 1.40901E+01, 4.2124E-01, 1.4351E+02, &
      3.186885E+00, 1.25541E+01, 4.0949E-01, 1.3748E+02, &
      3.199697E+00, 1.07060E+01, 4.0644E-01, 1.3593E+02, &
      3.218914E+00, 1.25778E+01, 4.0445E+00, 1.3257E+03, &
      3.266958E+00, 1.42753E+01, 3.9547E+00, 1.2772E+03, &
      3.319406E+00, 1.50333E+01, 3.8605E+00, 1.2272E+03, &
      3.548445E+00, 1.64395E+01, 3.4785E+00, 1.0346E+03, &
      3.793288E+00, 1.71011E+01, 3.1275E+00, 8.7039E+02, &
      4.055024E+00, 1.75109E+01, 2.8093E+00, 7.3160E+02, &
      4.334821E+00, 1.77899E+01, 2.5178E+00, 6.1357E+02, &
      4.633924E+00, 1.79817E+01, 2.2529E+00, 5.1380E+02, &
      4.953664E+00, 1.81162E+01, 2.0148E+00, 4.3003E+02, &
      5.295467E+00, 1.82108E+01, 1.8015E+00, 3.5986E+02, &
      5.660855E+00, 1.82767E+01, 1.6105E+00, 3.0110E+02, &
      6.051453E+00, 1.83220E+01, 1.4394E+00, 2.5189E+02, &
      6.469004E+00, 1.83939E+01, 1.2843E+00, 2.1039E+02, &
      6.915365E+00, 1.84089E+01, 1.1397E+00, 1.7480E+02, &
      7.392525E+00, 1.84120E+01, 1.0114E+00, 1.4525E+02, &
      7.902609E+00, 1.84075E+01, 8.9769E-01, 1.2072E+02, &
      8.447890E+00, 1.83978E+01, 7.9678E-01, 1.0035E+02, &
      9.030794E+00, 1.83906E+01, 7.0619E-01, 8.3304E+01, &
      9.653919E+00, 1.83744E+01, 6.2255E-01, 6.8803E+01, &
      1.032004E+01, 1.83552E+01, 5.4885E-01, 5.6841E+01, &
      1.103212E+01, 1.83348E+01, 4.8390E-01, 4.6971E+01, &
      1.179334E+01, 1.83139E+01, 4.2666E-01, 3.8827E+01, &
      1.260708E+01, 1.82932E+01, 3.7619E-01, 3.2103E+01, &
      1.347697E+01, 1.82732E+01, 3.3171E-01, 2.6555E+01, &
      1.440688E+01, 1.82541E+01, 2.9142E-01, 2.1894E+01, &
      1.540095E+01, 1.82350E+01, 2.5528E-01, 1.8011E+01, &
      1.646362E+01, 1.82165E+01, 2.2364E-01, 1.4823E+01, &
      1.759961E+01, 1.81987E+01, 1.9592E-01, 1.2208E+01, &
      1.881398E+01, 1.81819E+01, 1.7165E-01, 1.0062E+01, &
      2.011215E+01, 1.81662E+01, 1.5039E-01, 8.3002E+00, &
      2.149988E+01, 1.81516E+01, 1.3176E-01, 6.8536E+00, &
      2.298338E+01, 1.81380E+01, 1.1545E-01, 5.6654E+00, &
      2.456923E+01, 1.81255E+01, 1.0116E-01, 4.6893E+00, &
      2.626450E+01, 1.81140E+01, 8.8636E-02, 3.8873E+00, &
      2.807676E+01, 1.81044E+01, 7.7197E-02, 3.2103E+00, &
      3.001405E+01, 1.80947E+01, 6.7103E-02, 2.6521E+00, &
      3.208502E+01, 1.80856E+01, 5.8330E-02, 2.1967E+00, &
      3.429889E+01, 1.80773E+01, 5.0704E-02, 1.8248E+00, &
      3.666551E+01, 1.80696E+01, 4.4077E-02, 1.5209E+00, &
      3.919543E+01, 1.80627E+01, 3.8316E-02, 1.2725E+00, &
      4.189992E+01, 1.80563E+01, 3.3309E-02, 1.0693E+00, &
      4.479101E+01, 1.80506E+01, 2.8957E-02, 9.0295E-01, &
      4.788159E+01, 1.80454E+01, 2.5173E-02, 7.6662E-01, &
      5.118542E+01, 1.80407E+01, 2.1885E-02, 6.5481E-01, &
      5.471721E+01, 1.80365E+01, 1.9026E-02, 5.6296E-01, &
      5.849270E+01, 1.80326E+01, 1.6541E-02, 4.8743E-01, &
      6.252870E+01, 1.80292E+01, 1.4380E-02, 4.2521E-01, &
      6.684318E+01, 1.80261E+01, 1.2502E-02, 3.7385E-01, &
      7.145536E+01, 1.80234E+01, 1.0833E-02, 3.3083E-01, &
      7.638578E+01, 1.80209E+01, 9.3726E-03, 2.9505E-01, &
      8.165640E+01, 1.80186E+01, 8.1074E-03, 2.6540E-01, &
      8.729069E+01, 1.80166E+01, 7.0102E-03, 2.4070E-01, &
      9.331374E+01, 1.80148E+01, 6.0615E-03, 2.2010E-01, &
      9.975239E+01, 1.80132E+01, 5.2412E-03, 2.0283E-01, &
      1.066353E+02, 1.80117E+01, 4.5320E-03, 1.8825E-01, &
      1.139931E+02, 1.80104E+01, 3.9187E-03, 1.7587E-01, &
      1.218587E+02, 1.80093E+01, 3.3885E-03, 1.6529E-01, &
      1.302669E+02, 1.80083E+01, 2.9300E-03, 1.5617E-01, &
      1.392553E+02, 1.80073E+01, 2.5336E-03, 1.4826E-01, &
      1.488640E+02, 1.80065E+01, 2.1909E-03, 1.4131E-01, &
      1.591356E+02, 1.80058E+01, 1.8945E-03, 1.3517E-01, &
      1.701159E+02, 1.80051E+01, 1.6382E-03, 1.2968E-01, &
      1.818539E+02, 1.80045E+01, 1.4166E-03, 1.2474E-01, &
      1.944018E+02, 1.80040E+01, 1.2250E-03, 1.2024E-01, &
      2.078156E+02, 1.80036E+01, 1.0593E-03, 1.1610E-01, &
      2.221548E+02, 1.80032E+01, 9.1600E-04, 1.1227E-01, &
      2.374835E+02, 1.80028E+01, 7.9211E-04, 1.0870E-01, &
      2.538699E+02, 1.80025E+01, 6.8497E-04, 1.0535E-01, &
      2.713869E+02, 1.80022E+01, 5.9233E-04, 1.0219E-01, &
      2.901126E+02, 1.80019E+01, 5.1222E-04, 9.9175E-02, &
      3.101304E+02, 1.80017E+01, 4.4295E-04, 9.6298E-02, &
      3.315294E+02, 1.80015E+01, 3.8305E-04, 9.3542E-02, &
      3.544049E+02, 1.80013E+01, 3.3125E-04, 9.0891E-02, &
      3.788588E+02, 1.80012E+01, 2.8645E-04, 8.8333E-02, &
      4.050001E+02, 1.80010E+01, 2.4772E-04, 8.5862E-02, &
      4.329451E+02, 1.80009E+01, 2.1422E-04, 8.3469E-02]
   case(19)
      frel = -3.1800E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.03631E+00, 0.0000E+00, 1.2556E-04, &
      1.142761E-02, 6.67451E-01, 0.0000E+00, 1.7071E-04, &
      1.221612E-02, 1.82018E-01, 0.0000E+00, 2.3082E-04, &
      1.305903E-02, -4.81008E-01, 0.0000E+00, 3.1040E-04, &
      1.396010E-02, -1.43652E+00, 0.0000E+00, 4.1514E-04, &
      1.492335E-02, -2.93411E+00, 0.0000E+00, 5.5224E-04, &
      1.595306E-02, -5.66247E+00, 0.0000E+00, 7.3068E-04, &
      1.705382E-02, -1.28490E+01, 0.0000E+00, 9.6163E-04, &
      1.744400E-02, -2.01638E+01, 0.0000E+00, 1.0542E-03, &
      1.771100E-02, -3.64057E+01, 0.0000E+00, 1.1209E-03, &
      1.778220E-02, -5.75750E+01, 0.0000E+00, 1.1392E-03, &
      1.788900E-02, -3.27660E+01, 4.2470E+01, 2.5551E+06, &
      1.815600E-02, -1.09873E+01, 3.8222E+01, 2.2658E+06, &
      1.823053E-02, -7.96761E+00, 3.7125E+01, 2.1917E+06, &
      1.948844E-02, 1.05085E+01, 2.3098E+01, 1.2756E+06, &
      2.083314E-02, 1.36677E+01, 1.4677E+01, 7.5824E+05, &
      2.227063E-02, 1.37200E+01, 9.7862E+00, 4.7294E+05, &
      2.380730E-02, 1.30032E+01, 6.8296E+00, 3.0875E+05, &
      2.545001E-02, 1.21580E+01, 4.9747E+00, 2.1038E+05, &
      2.720606E-02, 1.13733E+01, 3.7714E+00, 1.4920E+05, &
      2.908327E-02, 1.06944E+01, 2.9673E+00, 1.0981E+05, &
      3.109002E-02, 1.01175E+01, 2.4161E+00, 8.3640E+04, &
      3.323523E-02, 9.61001E+00, 2.0302E+00, 6.5743E+04, &
      3.373050E-02, 9.48312E+00, 1.9611E+00, 6.2575E+04, &
      3.386610E-02, 9.42106E+00, 1.9433E+00, 6.1757E+04, &
      3.406950E-02, 9.41824E+00, 1.9890E+00, 6.2832E+04, &
      3.457800E-02, 9.35849E+00, 1.9326E+00, 6.0155E+04, &
      3.552846E-02, 9.22330E+00, 1.8414E+00, 5.5782E+04, &
      3.797993E-02, 8.91334E+00, 1.6722E+00, 4.7388E+04, &
      4.060054E-02, 8.65576E+00, 1.5627E+00, 4.1425E+04, &
      4.340198E-02, 8.44571E+00, 1.4956E+00, 3.7087E+04, &
      4.639671E-02, 8.27664E+00, 1.4595E+00, 3.3856E+04, &
      4.959809E-02, 8.14269E+00, 1.4460E+00, 3.1378E+04, &
      5.302035E-02, 8.03876E+00, 1.4489E+00, 2.9412E+04, &
      5.667876E-02, 7.96076E+00, 1.4635E+00, 2.7790E+04, &
      6.058959E-02, 7.90418E+00, 1.4861E+00, 2.6399E+04, &
      6.477028E-02, 7.86604E+00, 1.5142E+00, 2.5161E+04, &
      6.923942E-02, 7.84348E+00, 1.5456E+00, 2.4026E+04, &
      7.401695E-02, 7.83414E+00, 1.5786E+00, 2.2955E+04, &
      7.912411E-02, 7.83561E+00, 1.6112E+00, 2.1916E+04, &
      8.458368E-02, 7.84534E+00, 1.6420E+00, 2.0893E+04, &
      9.041995E-02, 7.86087E+00, 1.6700E+00, 1.9878E+04, &
      9.665893E-02, 7.87971E+00, 1.6941E+00, 1.8864E+04, &
      1.033284E-01, 7.89855E+00, 1.7137E+00, 1.7850E+04, &
      1.104581E-01, 7.91598E+00, 1.7279E+00, 1.6836E+04, &
      1.180797E-01, 7.92812E+00, 1.7363E+00, 1.5826E+04, &
      1.262272E-01, 7.93138E+00, 1.7384E+00, 1.4823E+04, &
      1.349368E-01, 7.92155E+00, 1.7340E+00, 1.3831E+04, &
      1.442475E-01, 7.89362E+00, 1.7229E+00, 1.2855E+04, &
      1.542005E-01, 7.84127E+00, 1.7050E+00, 1.1901E+04, &
      1.648404E-01, 7.75650E+00, 1.6805E+00, 1.0973E+04, &
      1.762144E-01, 7.62874E+00, 1.6500E+00, 1.0079E+04, &
      1.883732E-01, 7.44332E+00, 1.6139E+00, 9.2219E+03, &
      2.013709E-01, 7.17851E+00, 1.5729E+00, 8.4074E+03, &
      2.152655E-01, 6.79980E+00, 1.5275E+00, 7.6382E+03, &
      2.301188E-01, 6.24684E+00, 1.4787E+00, 6.9168E+03, &
      2.459970E-01, 5.39894E+00, 1.4270E+00, 6.2443E+03, &
      2.629708E-01, 3.95518E+00, 1.3732E+00, 5.6214E+03, &
      2.811158E-01, 7.05984E-01, 1.3181E+00, 5.0476E+03, &
      2.877280E-01, -2.13949E+00, 1.2987E+00, 4.8590E+03, &
      2.903740E-01, -4.36755E+00, 1.2910E+00, 4.7863E+03, &
      2.921320E-01, -7.17401E+00, 1.2859E+00, 4.7389E+03, &
      2.933064E-01, -1.24621E+01, 1.2826E+00, 4.7076E+03, &
      2.948185E-01, -9.07883E+00, 1.0759E+01, 3.9276E+04, &
      2.950680E-01, -8.77353E+00, 1.0750E+01, 3.9211E+04, &
      2.960037E-01, -9.35450E+00, 1.0716E+01, 3.8966E+04, &
      2.977815E-01, -5.13667E+00, 1.5353E+01, 5.5491E+04, &
      2.994720E-01, -2.87198E+00, 1.5268E+01, 5.4871E+04, &
      3.005128E-01, -1.90035E+00, 1.5216E+01, 5.4495E+04, &
      3.022260E-01, -6.40194E-01, 1.5131E+01, 5.3883E+04, &
      3.212482E-01, 5.59255E+00, 1.4247E+01, 4.7734E+04, &
      3.434143E-01, 8.59570E+00, 1.3306E+01, 4.1702E+04, &
      3.671099E-01, 1.01429E+01, 1.2353E+01, 3.6218E+04, &
      3.695580E-01, 1.01830E+01, 1.2259E+01, 3.5704E+04, &
      3.752145E-01, 9.89081E+00, 1.2044E+01, 3.4550E+04, &
      3.767229E-01, 9.19868E+00, 1.1988E+01, 3.4250E+04, &
      3.789855E-01, 1.01360E+01, 1.3446E+01, 3.8188E+04, &
      3.846420E-01, 1.11597E+01, 1.3233E+01, 3.7030E+04, &
      3.924405E-01, 1.19388E+01, 1.2947E+01, 3.5508E+04, &
      4.195189E-01, 1.36474E+01, 1.2012E+01, 3.0819E+04, &
      4.484657E-01, 1.48494E+01, 1.1110E+01, 2.6664E+04, &
      4.794098E-01, 1.57726E+01, 1.0243E+01, 2.2998E+04, &
      5.124891E-01, 1.64983E+01, 9.4182E+00, 1.9781E+04, &
      5.478508E-01, 1.70723E+01, 8.6374E+00, 1.6970E+04, &
      5.856525E-01, 1.75257E+01, 7.9026E+00, 1.4525E+04, &
      6.260625E-01, 1.78814E+01, 7.2147E+00, 1.2406E+04, &
      6.692609E-01, 1.81571E+01, 6.5728E+00, 1.0573E+04, &
      7.154399E-01, 1.83667E+01, 5.9766E+00, 8.9936E+03, &
      7.648052E-01, 1.85218E+01, 5.4252E+00, 7.6374E+03, &
      8.175768E-01, 1.86322E+01, 4.9170E+00, 6.4756E+03, &
      8.739896E-01, 1.87063E+01, 4.4499E+00, 5.4827E+03, &
      9.342948E-01, 1.87539E+01, 4.0218E+00, 4.6358E+03, &
      9.987612E-01, 1.87769E+01, 3.6304E+00, 3.9151E+03, &
      1.067676E+00, 1.87770E+01, 3.2670E+00, 3.2963E+03, &
      1.141345E+00, 1.87592E+01, 2.9368E+00, 2.7724E+03, &
      1.220098E+00, 1.87275E+01, 2.6275E+00, 2.3208E+03, &
      1.304285E+00, 1.86758E+01, 2.3429E+00, 1.9363E+03, &
      1.394281E+00, 1.86114E+01, 2.0908E+00, 1.6169E+03, &
      1.490486E+00, 1.85398E+01, 1.8674E+00, 1.3515E+03, &
      1.593329E+00, 1.84623E+01, 1.6641E+00, 1.1271E+03, &
      1.703269E+00, 1.83778E+01, 1.4833E+00, 9.4024E+02, &
      1.820795E+00, 1.82883E+01, 1.3233E+00, 7.8515E+02, &
      1.946430E+00, 1.81940E+01, 1.1817E+00, 6.5633E+02, &
      2.080733E+00, 1.80940E+01, 1.0563E+00, 5.4920E+02, &
      2.224304E+00, 1.79866E+01, 9.4505E-01, 4.6006E+02, &
      2.377781E+00, 1.78682E+01, 8.4636E-01, 3.8581E+02, &
      2.541848E+00, 1.77332E+01, 7.5870E-01, 3.2390E+02, &
      2.717235E+00, 1.75716E+01, 6.8077E-01, 2.7222E+02, &
      2.904724E+00, 1.73645E+01, 6.1143E-01, 2.2904E+02, &
      3.105150E+00, 1.70762E+01, 5.4407E-01, 1.9099E+02, &
      3.319406E+00, 1.65660E+01, 4.7995E-01, 1.5795E+02, &
      3.535252E+00, 1.51657E+01, 4.2673E-01, 1.3215E+02, &
      3.548445E+00, 1.49494E+01, 4.2379E-01, 1.3078E+02, &
      3.589363E+00, 1.36420E+01, 4.1486E-01, 1.2662E+02, &
      3.603793E+00, 1.18067E+01, 4.1178E-01, 1.2520E+02, &
      3.625437E+00, 1.36705E+01, 4.0255E+00, 1.1972E+03, &
      3.679548E+00, 1.53599E+01, 3.9276E+00, 1.1510E+03, &
      3.793288E+00, 1.65531E+01, 3.7342E+00, 1.0616E+03, &
      4.055024E+00, 1.76584E+01, 3.3541E+00, 8.9231E+02, &
      4.334821E+00, 1.82298E+01, 3.0167E+00, 7.5097E+02, &
      4.633924E+00, 1.85966E+01, 2.7144E+00, 6.3232E+02, &
      4.953664E+00, 1.88512E+01, 2.4328E+00, 5.3034E+02, &
      5.295467E+00, 1.90266E+01, 2.1770E+00, 4.4415E+02, &
      5.660855E+00, 1.91497E+01, 1.9467E+00, 3.7173E+02, &
      6.051453E+00, 1.92357E+01, 1.7396E+00, 3.1092E+02, &
      6.469004E+00, 1.92949E+01, 1.5542E+00, 2.6001E+02, &
      6.915365E+00, 1.93348E+01, 1.3883E+00, 2.1742E+02, &
      7.392525E+00, 1.94018E+01, 1.2373E+00, 1.8141E+02, &
      7.902609E+00, 1.94134E+01, 1.0989E+00, 1.5086E+02, &
      8.447890E+00, 1.94155E+01, 9.7605E-01, 1.2548E+02, &
      9.030794E+00, 1.94122E+01, 8.6567E-01, 1.0423E+02, &
      9.653919E+00, 1.94018E+01, 7.6367E-01, 8.6127E+01, &
      1.032004E+01, 1.93862E+01, 6.7372E-01, 7.1189E+01, &
      1.103212E+01, 1.93677E+01, 5.9441E-01, 5.8858E+01, &
      1.179334E+01, 1.93476E+01, 5.2447E-01, 4.8675E+01, &
      1.260708E+01, 1.93268E+01, 4.6278E-01, 4.0266E+01, &
      1.347697E+01, 1.93059E+01, 4.0837E-01, 3.3321E+01, &
      1.440688E+01, 1.92855E+01, 3.6038E-01, 2.7584E+01, &
      1.540095E+01, 1.92659E+01, 3.1804E-01, 2.2844E+01, &
      1.646362E+01, 1.92470E+01, 2.7920E-01, 1.8830E+01, &
      1.759961E+01, 1.92283E+01, 2.4474E-01, 1.5507E+01, &
      1.881398E+01, 1.92102E+01, 2.1455E-01, 1.2779E+01, &
      2.011215E+01, 1.91929E+01, 1.8808E-01, 1.0538E+01, &
      2.149988E+01, 1.91767E+01, 1.6489E-01, 8.6979E+00, &
      2.298338E+01, 1.91616E+01, 1.4456E-01, 7.1858E+00, &
      2.456923E+01, 1.91476E+01, 1.2674E-01, 5.9433E+00, &
      2.626450E+01, 1.91350E+01, 1.1112E-01, 4.9218E+00, &
      2.807676E+01, 1.91233E+01, 9.6820E-02, 4.0585E+00, &
      3.001405E+01, 1.91119E+01, 8.4188E-02, 3.3467E+00, &
      3.208502E+01, 1.91014E+01, 7.3206E-02, 2.7657E+00, &
      3.429889E+01, 1.90917E+01, 6.3658E-02, 2.2915E+00, &
      3.666551E+01, 1.90827E+01, 5.5356E-02, 1.9041E+00, &
      3.919543E+01, 1.90746E+01, 4.8138E-02, 1.5876E+00, &
      4.189992E+01, 1.90671E+01, 4.1862E-02, 1.3288E+00, &
      4.479101E+01, 1.90603E+01, 3.6404E-02, 1.1170E+00, &
      4.788159E+01, 1.90542E+01, 3.1659E-02, 9.4361E-01, &
      5.118542E+01, 1.90486E+01, 2.7533E-02, 8.0152E-01, &
      5.471721E+01, 1.90436E+01, 2.3945E-02, 6.8497E-01, &
      5.849270E+01, 1.90391E+01, 2.0825E-02, 5.8924E-01, &
      6.252870E+01, 1.90350E+01, 1.8111E-02, 5.1054E-01, &
      6.684318E+01, 1.90313E+01, 1.5752E-02, 4.4570E-01, &
      7.145536E+01, 1.90280E+01, 1.3700E-02, 3.9221E-01, &
      7.638578E+01, 1.90251E+01, 1.1915E-02, 3.4797E-01, &
      8.165640E+01, 1.90224E+01, 1.0336E-02, 3.1094E-01, &
      8.729069E+01, 1.90200E+01, 8.9613E-03, 2.8014E-01, &
      9.331374E+01, 1.90178E+01, 7.7691E-03, 2.5452E-01, &
      9.975239E+01, 1.90159E+01, 6.7356E-03, 2.3310E-01, &
      1.066353E+02, 1.90142E+01, 5.8397E-03, 2.1512E-01, &
      1.139931E+02, 1.90126E+01, 5.0629E-03, 1.9993E-01, &
      1.218587E+02, 1.90112E+01, 4.3895E-03, 1.8702E-01, &
      1.302669E+02, 1.90100E+01, 3.8057E-03, 1.7597E-01, &
      1.392553E+02, 1.90089E+01, 3.2996E-03, 1.6645E-01, &
      1.488640E+02, 1.90079E+01, 2.8608E-03, 1.5815E-01, &
      1.591356E+02, 1.90070E+01, 2.4804E-03, 1.5088E-01, &
      1.701159E+02, 1.90062E+01, 2.1505E-03, 1.4443E-01, &
      1.818539E+02, 1.90055E+01, 1.8646E-03, 1.3866E-01, &
      1.944018E+02, 1.90049E+01, 1.6166E-03, 1.3343E-01, &
      2.078156E+02, 1.90044E+01, 1.4017E-03, 1.2867E-01, &
      2.221548E+02, 1.90039E+01, 1.2153E-03, 1.2428E-01, &
      2.374835E+02, 1.90034E+01, 1.0537E-03, 1.2021E-01, &
      2.538699E+02, 1.90030E+01, 9.1366E-04, 1.1638E-01, &
      2.713869E+02, 1.90027E+01, 7.9219E-04, 1.1278E-01, &
      2.901126E+02, 1.90024E+01, 6.8688E-04, 1.0936E-01, &
      3.101304E+02, 1.90021E+01, 5.9557E-04, 1.0609E-01, &
      3.315294E+02, 1.90019E+01, 5.1640E-04, 1.0295E-01, &
      3.544049E+02, 1.90017E+01, 4.4775E-04, 9.9916E-02, &
      3.788588E+02, 1.90015E+01, 3.8823E-04, 9.6989E-02, &
      4.050001E+02, 1.90013E+01, 3.3663E-04, 9.4150E-02, &
      4.329451E+02, 1.90011E+01, 2.9188E-04, 9.1391E-02]
   case(20)
      frel = -3.6000E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 6.74674E+00, 0.0000E+00, 1.1540E-04, &
      1.142761E-02, 6.71135E+00, 0.0000E+00, 1.5728E-04, &
      1.221612E-02, 6.66883E+00, 0.0000E+00, 2.1319E-04, &
      1.305903E-02, 6.61724E+00, 0.0000E+00, 2.8739E-04, &
      1.396010E-02, 6.55391E+00, 0.0000E+00, 3.8531E-04, &
      1.492335E-02, 6.47500E+00, 0.0000E+00, 5.1380E-04, &
      1.595306E-02, 6.37476E+00, 0.0000E+00, 6.8145E-04, &
      1.705382E-02, 6.24417E+00, 0.0000E+00, 8.9899E-04, &
      1.823053E-02, 6.06801E+00, 0.0000E+00, 1.1797E-03, &
      1.948844E-02, 5.81829E+00, 0.0000E+00, 1.5399E-03, &
      2.083314E-02, 5.43633E+00, 0.0000E+00, 1.9995E-03, &
      2.227063E-02, 4.77106E+00, 0.0000E+00, 2.5828E-03, &
      2.380730E-02, 3.24162E+00, 0.0000E+00, 3.3190E-03, &
      2.489200E-02, -3.74657E-01, 0.0000E+00, 3.9129E-03, &
      2.527300E-02, -6.20519E+00, 0.0000E+00, 4.1366E-03, &
      2.537460E-02, -1.41462E+01, 0.0000E+00, 4.1976E-03, &
      2.545001E-02, -9.70039E+00, 1.6608E+01, 6.8516E+05, &
      2.552700E-02, -4.18744E+00, 1.6030E+01, 6.5929E+05, &
      2.590800E-02, 4.30351E+00, 1.3471E+01, 5.4589E+05, &
      2.720606E-02, 1.02441E+01, 7.5878E+00, 2.9282E+05, &
      2.908327E-02, 1.06666E+01, 3.5839E+00, 1.2938E+05, &
      3.109002E-02, 9.98904E+00, 1.9921E+00, 6.7275E+04, &
      3.323523E-02, 9.39459E+00, 1.2880E+00, 4.0688E+04, &
      3.552846E-02, 8.94477E+00, 9.4430E-01, 2.7905E+04, &
      3.797993E-02, 8.59656E+00, 7.6777E-01, 2.1224E+04, &
      4.060054E-02, 8.30210E+00, 6.7898E-01, 1.7558E+04, &
      4.282600E-02, 8.04432E+00, 6.4639E-01, 1.5847E+04, &
      4.340198E-02, 7.92888E+00, 6.4220E-01, 1.5535E+04, &
      4.348150E-02, 7.90165E+00, 6.4174E-01, 1.5495E+04, &
      4.365630E-02, 7.77713E+00, 6.4081E-01, 1.5411E+04, &
      4.391850E-02, 7.86952E+00, 8.5980E-01, 2.0554E+04, &
      4.457400E-02, 7.92308E+00, 8.6461E-01, 2.0365E+04, &
      4.639671E-02, 7.90007E+00, 8.8583E-01, 2.0045E+04, &
      4.959809E-02, 7.81742E+00, 9.4439E-01, 1.9991E+04, &
      5.302035E-02, 7.74364E+00, 1.0243E+00, 2.0284E+04, &
      5.667876E-02, 7.68645E+00, 1.1216E+00, 2.0776E+04, &
      6.058959E-02, 7.64749E+00, 1.2338E+00, 2.1379E+04, &
      6.477028E-02, 7.62780E+00, 1.3587E+00, 2.2024E+04, &
      6.923942E-02, 7.62852E+00, 1.4940E+00, 2.2654E+04, &
      7.401695E-02, 7.65082E+00, 1.6367E+00, 2.3216E+04, &
      7.912411E-02, 7.70078E+00, 1.7833E+00, 2.3663E+04, &
      8.458368E-02, 7.76858E+00, 1.9298E+00, 2.3954E+04, &
      9.041995E-02, 7.85896E+00, 2.0717E+00, 2.4055E+04, &
      9.665893E-02, 7.97064E+00, 2.2042E+00, 2.3942E+04, &
      1.033284E-01, 8.10171E+00, 2.3229E+00, 2.3603E+04, &
      1.104581E-01, 8.24823E+00, 2.4206E+00, 2.3008E+04, &
      1.180797E-01, 8.40291E+00, 2.4932E+00, 2.2168E+04, &
      1.262272E-01, 8.55822E+00, 2.5393E+00, 2.1121E+04, &
      1.349368E-01, 8.70566E+00, 2.5589E+00, 1.9910E+04, &
      1.442475E-01, 8.84003E+00, 2.5530E+00, 1.8582E+04, &
      1.542005E-01, 8.95323E+00, 2.5241E+00, 1.7187E+04, &
      1.648404E-01, 9.03840E+00, 2.4748E+00, 1.5764E+04, &
      1.762144E-01, 9.08864E+00, 2.4086E+00, 1.4352E+04, &
      1.883732E-01, 9.09648E+00, 2.3287E+00, 1.2980E+04, &
      2.013709E-01, 9.05296E+00, 2.2384E+00, 1.1671E+04, &
      2.152655E-01, 8.94650E+00, 2.1408E+00, 1.0442E+04, &
      2.301188E-01, 8.76054E+00, 2.0387E+00, 9.3024E+03, &
      2.459970E-01, 8.46935E+00, 1.9345E+00, 8.2574E+03, &
      2.629708E-01, 8.02921E+00, 1.8301E+00, 7.3079E+03, &
      2.811158E-01, 7.35724E+00, 1.7271E+00, 6.4517E+03, &
      3.005128E-01, 6.26827E+00, 1.6267E+00, 5.6845E+03, &
      3.212482E-01, 4.20000E+00, 1.5296E+00, 5.0006E+03, &
      3.394720E-01, -6.01802E-01, 1.4524E+00, 4.4934E+03, &
      3.430000E-01, -3.23076E+00, 1.4382E+00, 4.4039E+03, &
      3.434143E-01, -3.69940E+00, 1.4366E+00, 4.3935E+03, &
      3.446680E-01, -5.61786E+00, 1.4316E+00, 4.3625E+03, &
      3.460536E-01, -1.09196E+01, 1.4262E+00, 4.3285E+03, &
      3.482500E-01, -6.84427E+00, 1.1000E+01, 3.3165E+04, &
      3.496500E-01, -7.47210E+00, 1.0949E+01, 3.2878E+04, &
      3.517500E-01, -3.36124E+00, 1.5626E+01, 4.6643E+04, &
      3.533280E-01, -1.51093E+00, 1.5545E+01, 4.6193E+04, &
      3.570000E-01, 1.07948E+00, 1.5358E+01, 4.5169E+04, &
      3.671099E-01, 4.79675E+00, 1.4866E+01, 4.2517E+04, &
      3.924405E-01, 8.98735E+00, 1.3749E+01, 3.6784E+04, &
      4.195189E-01, 1.10578E+01, 1.2683E+01, 3.1742E+04, &
      4.290440E-01, 1.13462E+01, 1.2334E+01, 3.0185E+04, &
      4.356110E-01, 1.10638E+01, 1.2102E+01, 2.9169E+04, &
      4.373622E-01, 1.03698E+01, 1.2041E+01, 2.8906E+04, &
      4.399890E-01, 1.13158E+01, 1.3502E+01, 3.2221E+04, &
      4.465560E-01, 1.23521E+01, 1.3275E+01, 3.1213E+04, &
      4.484657E-01, 1.25462E+01, 1.3210E+01, 3.0928E+04, &
      4.794098E-01, 1.45209E+01, 1.2219E+01, 2.6763E+04, &
      5.124891E-01, 1.58108E+01, 1.1265E+01, 2.3080E+04, &
      5.478508E-01, 1.67789E+01, 1.0357E+01, 1.9851E+04, &
      5.856525E-01, 1.75280E+01, 9.5000E+00, 1.7033E+04, &
      6.260625E-01, 1.81133E+01, 8.6947E+00, 1.4584E+04, &
      6.692609E-01, 1.85712E+01, 7.9426E+00, 1.2463E+04, &
      7.154399E-01, 1.89290E+01, 7.2440E+00, 1.0634E+04, &
      7.648052E-01, 1.92085E+01, 6.5931E+00, 9.0538E+03, &
      8.175768E-01, 1.94180E+01, 5.9873E+00, 7.6916E+03, &
      8.739896E-01, 1.95708E+01, 5.4288E+00, 6.5245E+03, &
      9.342948E-01, 1.96784E+01, 4.9166E+00, 5.5280E+03, &
      9.987612E-01, 1.97509E+01, 4.4486E+00, 4.6795E+03, &
      1.067676E+00, 1.98007E+01, 4.0045E+00, 3.9410E+03, &
      1.141345E+00, 1.98129E+01, 3.6007E+00, 3.3154E+03, &
      1.220098E+00, 1.98020E+01, 3.2354E+00, 2.7873E+03, &
      1.304285E+00, 1.97743E+01, 2.9062E+00, 2.3426E+03, &
      1.394281E+00, 1.97367E+01, 2.6095E+00, 1.9681E+03, &
      1.490486E+00, 1.96846E+01, 2.3292E+00, 1.6439E+03, &
      1.593329E+00, 1.96197E+01, 2.0811E+00, 1.3745E+03, &
      1.703269E+00, 1.95480E+01, 1.8618E+00, 1.1507E+03, &
      1.820795E+00, 1.94721E+01, 1.6627E+00, 9.6186E+02, &
      1.946430E+00, 1.93889E+01, 1.4833E+00, 8.0317E+02, &
      2.080733E+00, 1.93001E+01, 1.3245E+00, 6.7135E+02, &
      2.224304E+00, 1.92061E+01, 1.1839E+00, 5.6175E+02, &
      2.377781E+00, 1.91059E+01, 1.0591E+00, 4.7053E+02, &
      2.541848E+00, 1.89976E+01, 9.4843E-01, 3.9456E+02, &
      2.717235E+00, 1.88776E+01, 8.5013E-01, 3.3121E+02, &
      2.904724E+00, 1.87401E+01, 7.6275E-01, 2.7834E+02, &
      3.105150E+00, 1.85764E+01, 6.7803E-01, 2.3182E+02, &
      3.319406E+00, 1.83498E+01, 5.9753E-01, 1.9148E+02, &
      3.548445E+00, 1.80080E+01, 5.2709E-01, 1.5834E+02, &
      3.793288E+00, 1.73592E+01, 4.6540E-01, 1.3110E+02, &
      3.957338E+00, 1.62338E+01, 4.3026E-01, 1.1638E+02, &
      4.017909E+00, 1.47223E+01, 4.1835E-01, 1.1153E+02, &
      4.034062E+00, 1.28987E+01, 4.1526E-01, 1.1028E+02, &
      4.055024E+00, 1.45470E+01, 4.0127E+00, 1.0412E+03, &
      4.058290E+00, 1.47569E+01, 4.0068E+00, 1.0388E+03, &
      4.118862E+00, 1.64400E+01, 3.8997E+00, 9.9622E+02, &
      4.334821E+00, 1.80590E+01, 3.5640E+00, 8.6531E+02, &
      4.633924E+00, 1.88984E+01, 3.1991E+00, 7.2683E+02, &
      4.953664E+00, 1.93754E+01, 2.8820E+00, 6.1274E+02, &
      5.295467E+00, 1.96982E+01, 2.5972E+00, 5.1674E+02, &
      5.660855E+00, 1.99227E+01, 2.3263E+00, 4.3318E+02, &
      6.051453E+00, 2.00785E+01, 2.0813E+00, 3.6273E+02, &
      6.469004E+00, 2.01876E+01, 1.8605E+00, 3.0349E+02, &
      6.915365E+00, 2.02632E+01, 1.6616E+00, 2.5373E+02, &
      7.392525E+00, 2.03145E+01, 1.4837E+00, 2.1209E+02, &
      7.902609E+00, 2.03488E+01, 1.3249E+00, 1.7731E+02, &
      8.447890E+00, 2.04090E+01, 1.1797E+00, 1.4782E+02, &
      9.030794E+00, 2.04237E+01, 1.0471E+00, 1.2286E+02, &
      9.653919E+00, 2.04210E+01, 9.2433E-01, 1.0158E+02, &
      1.032004E+01, 2.04108E+01, 8.1604E-01, 8.4014E+01, &
      1.103212E+01, 2.03960E+01, 7.2049E-01, 6.9498E+01, &
      1.179334E+01, 2.03780E+01, 6.3616E-01, 5.7504E+01, &
      1.260708E+01, 2.03582E+01, 5.6174E-01, 4.7593E+01, &
      1.347697E+01, 2.03376E+01, 4.9605E-01, 3.9402E+01, &
      1.440688E+01, 2.03166E+01, 4.3807E-01, 3.2632E+01, &
      1.540095E+01, 2.02959E+01, 3.8688E-01, 2.7035E+01, &
      1.646362E+01, 2.02759E+01, 3.4169E-01, 2.2407E+01, &
      1.759961E+01, 2.02569E+01, 3.0162E-01, 1.8569E+01, &
      1.881398E+01, 2.02384E+01, 2.6458E-01, 1.5304E+01, &
      2.011215E+01, 2.02201E+01, 2.3209E-01, 1.2620E+01, &
      2.149988E+01, 2.02026E+01, 2.0359E-01, 1.0413E+01, &
      2.298338E+01, 2.01860E+01, 1.7860E-01, 8.6004E+00, &
      2.456923E+01, 2.01706E+01, 1.5668E-01, 7.1099E+00, &
      2.626450E+01, 2.01564E+01, 1.3745E-01, 5.8843E+00, &
      2.807676E+01, 2.01430E+01, 1.1986E-01, 4.8486E+00, &
      3.001405E+01, 2.01302E+01, 1.0431E-01, 3.9944E+00, &
      3.208502E+01, 2.01182E+01, 9.0778E-02, 3.2969E+00, &
      3.429889E+01, 2.01071E+01, 7.9005E-02, 2.7273E+00, &
      3.666551E+01, 2.00969E+01, 6.8760E-02, 2.2618E+00, &
      3.919543E+01, 2.00875E+01, 5.9846E-02, 1.8814E+00, &
      4.189992E+01, 2.00789E+01, 5.2088E-02, 1.5702E+00, &
      4.479101E+01, 2.00711E+01, 4.5337E-02, 1.3156E+00, &
      4.788159E+01, 2.00639E+01, 3.9461E-02, 1.1072E+00, &
      5.118542E+01, 2.00575E+01, 3.4348E-02, 9.3633E-01, &
      5.471721E+01, 2.00516E+01, 2.9897E-02, 7.9623E-01, &
      5.849270E+01, 2.00463E+01, 2.6024E-02, 6.8123E-01, &
      6.252870E+01, 2.00415E+01, 2.2653E-02, 5.8673E-01, &
      6.684318E+01, 2.00372E+01, 1.9719E-02, 5.0896E-01, &
      7.145536E+01, 2.00333E+01, 1.7165E-02, 4.4486E-01, &
      7.638578E+01, 2.00298E+01, 1.4942E-02, 3.9194E-01, &
      8.165640E+01, 2.00267E+01, 1.2994E-02, 3.4798E-01, &
      8.729069E+01, 2.00239E+01, 1.1274E-02, 3.1123E-01, &
      9.331374E+01, 2.00213E+01, 9.7822E-03, 2.8074E-01, &
      9.975239E+01, 2.00190E+01, 8.4877E-03, 2.5535E-01, &
      1.066353E+02, 2.00170E+01, 7.3645E-03, 2.3413E-01, &
      1.139931E+02, 2.00151E+01, 6.3900E-03, 2.1629E-01, &
      1.218587E+02, 2.00135E+01, 5.5445E-03, 2.0122E-01, &
      1.302669E+02, 2.00120E+01, 4.8109E-03, 1.8840E-01, &
      1.392553E+02, 2.00107E+01, 4.1744E-03, 1.7742E-01, &
      1.488640E+02, 2.00095E+01, 3.6222E-03, 1.6794E-01, &
      1.591356E+02, 2.00084E+01, 3.1430E-03, 1.5967E-01, &
      1.701159E+02, 2.00075E+01, 2.7273E-03, 1.5240E-01, &
      1.818539E+02, 2.00067E+01, 2.3665E-03, 1.4594E-01, &
      1.944018E+02, 2.00059E+01, 2.0535E-03, 1.4015E-01, &
      2.078156E+02, 2.00053E+01, 1.7819E-03, 1.3491E-01, &
      2.221548E+02, 2.00047E+01, 1.5463E-03, 1.3011E-01, &
      2.374835E+02, 2.00041E+01, 1.3418E-03, 1.2567E-01, &
      2.538699E+02, 2.00037E+01, 1.1643E-03, 1.2154E-01, &
      2.713869E+02, 2.00033E+01, 1.0104E-03, 1.1765E-01, &
      2.901126E+02, 2.00029E+01, 8.7675E-04, 1.1397E-01, &
      3.101304E+02, 2.00026E+01, 7.6082E-04, 1.1047E-01, &
      3.315294E+02, 2.00023E+01, 6.6022E-04, 1.0711E-01, &
      3.544049E+02, 2.00020E+01, 5.7292E-04, 1.0387E-01, &
      3.788588E+02, 2.00018E+01, 4.9717E-04, 1.0074E-01, &
      4.050001E+02, 2.00016E+01, 4.3144E-04, 9.7706E-02, &
      4.329451E+02, 2.00014E+01, 3.7440E-04, 9.4752E-02]
   case(21)
      frel = -4.0800E-02
      allocate(nist_tab(760))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 8.43172E+00, 0.0000E+00, 1.2810E-04, &
      1.142761E-02, 8.42010E+00, 0.0000E+00, 1.7410E-04, &
      1.221612E-02, 8.40652E+00, 0.0000E+00, 2.3532E-04, &
      1.305903E-02, 8.39057E+00, 0.0000E+00, 3.1634E-04, &
      1.396010E-02, 8.37174E+00, 0.0000E+00, 4.2297E-04, &
      1.492335E-02, 8.34938E+00, 0.0000E+00, 5.6250E-04, &
      1.595306E-02, 8.32265E+00, 0.0000E+00, 7.4409E-04, &
      1.705382E-02, 8.29040E+00, 0.0000E+00, 9.7907E-04, &
      1.823053E-02, 8.25104E+00, 0.0000E+00, 1.2815E-03, &
      1.948844E-02, 8.20232E+00, 0.0000E+00, 1.6686E-03, &
      2.083314E-02, 8.14085E+00, 0.0000E+00, 2.1613E-03, &
      2.227063E-02, 8.06127E+00, 0.0000E+00, 2.7850E-03, &
      2.380730E-02, 7.95448E+00, 0.0000E+00, 3.5704E-03, &
      2.545001E-02, 7.80327E+00, 0.0000E+00, 4.5540E-03, &
      2.720606E-02, 7.56991E+00, 0.0000E+00, 5.7792E-03, &
      2.908327E-02, 7.14726E+00, 0.0000E+00, 7.2971E-03, &
      3.109002E-02, 5.98986E+00, 0.0000E+00, 9.1678E-03, &
      3.165400E-02, 5.06744E+00, 0.0000E+00, 9.7409E-03, &
      3.213850E-02, 2.64257E+00, 0.0000E+00, 1.0250E-02, &
      3.226770E-02, -5.72539E-01, 0.0000E+00, 1.0389E-02, &
      3.246150E-02, 3.30677E+00, 6.4655E+00, 1.8643E+05, &
      3.294600E-02, 6.66886E+00, 5.6490E+00, 1.6050E+05, &
      3.323523E-02, 7.54411E+00, 5.2166E+00, 1.4692E+05, &
      3.552846E-02, 9.63779E+00, 2.8403E+00, 7.4830E+04, &
      3.797993E-02, 9.68356E+00, 1.6748E+00, 4.1275E+04, &
      4.060054E-02, 9.47617E+00, 1.1417E+00, 2.6322E+04, &
      4.340198E-02, 9.26018E+00, 8.7736E-01, 1.8922E+04, &
      4.639671E-02, 9.06182E+00, 7.4331E-01, 1.4996E+04, &
      4.959809E-02, 8.86608E+00, 6.8109E-01, 1.2854E+04, &
      5.272400E-02, 8.62098E+00, 6.6398E-01, 1.1788E+04, &
      5.302035E-02, 8.58033E+00, 6.6389E-01, 1.1720E+04, &
      5.353100E-02, 8.46296E+00, 6.6425E-01, 1.1615E+04, &
      5.374620E-02, 8.30602E+00, 6.6459E-01, 1.1574E+04, &
      5.406900E-02, 8.44091E+00, 9.5669E-01, 1.6562E+04, &
      5.487600E-02, 8.54012E+00, 9.6596E-01, 1.6477E+04, &
      5.667876E-02, 8.57621E+00, 9.9135E-01, 1.6372E+04, &
      6.058959E-02, 8.56318E+00, 1.0641E+00, 1.6439E+04, &
      6.477028E-02, 8.53831E+00, 1.1580E+00, 1.6735E+04, &
      6.923942E-02, 8.52042E+00, 1.2694E+00, 1.7161E+04, &
      7.401695E-02, 8.51531E+00, 1.3961E+00, 1.7655E+04, &
      7.912411E-02, 8.52654E+00, 1.5357E+00, 1.8167E+04, &
      8.458368E-02, 8.55692E+00, 1.6854E+00, 1.8651E+04, &
      9.041995E-02, 8.60871E+00, 1.8415E+00, 1.9063E+04, &
      9.665893E-02, 8.68337E+00, 1.9997E+00, 1.9365E+04, &
      1.033284E-01, 8.77778E+00, 2.1550E+00, 1.9521E+04, &
      1.104581E-01, 8.89838E+00, 2.3021E+00, 1.9508E+04, &
      1.180797E-01, 9.03950E+00, 2.4359E+00, 1.9309E+04, &
      1.262272E-01, 9.19793E+00, 2.5515E+00, 1.8921E+04, &
      1.349368E-01, 9.36968E+00, 2.6440E+00, 1.8341E+04, &
      1.442475E-01, 9.54736E+00, 2.7074E+00, 1.7569E+04, &
      1.542005E-01, 9.72191E+00, 2.7405E+00, 1.6636E+04, &
      1.648404E-01, 9.88537E+00, 2.7441E+00, 1.5583E+04, &
      1.762144E-01, 1.00286E+01, 2.7203E+00, 1.4451E+04, &
      1.883732E-01, 1.01445E+01, 2.6722E+00, 1.3279E+04, &
      2.013709E-01, 1.02257E+01, 2.6035E+00, 1.2103E+04, &
      2.152655E-01, 1.02646E+01, 2.5182E+00, 1.0951E+04, &
      2.301188E-01, 1.02526E+01, 2.4200E+00, 9.8448E+03, &
      2.459970E-01, 1.01788E+01, 2.3126E+00, 8.8009E+03, &
      2.629708E-01, 1.00285E+01, 2.1994E+00, 7.8298E+03, &
      2.811158E-01, 9.77881E+00, 2.0831E+00, 6.9374E+03, &
      3.005128E-01, 9.39216E+00, 1.9662E+00, 6.1257E+03, &
      3.212482E-01, 8.79907E+00, 1.8506E+00, 5.3937E+03, &
      3.434143E-01, 7.85136E+00, 1.7377E+00, 4.7381E+03, &
      3.671099E-01, 6.14027E+00, 1.6287E+00, 4.1543E+03, &
      3.924405E-01, 1.46616E+00, 1.5241E+00, 3.6371E+03, &
      3.941560E-01, 7.48834E-01, 1.5175E+00, 3.6054E+03, &
      3.985660E-01, -2.14125E+00, 1.5006E+00, 3.5260E+03, &
      4.001890E-01, -4.20296E+00, 1.4944E+00, 3.4973E+03, &
      4.017978E-01, -9.45344E+00, 1.4884E+00, 3.4692E+03, &
      4.042110E-01, -5.39644E+00, 1.1021E+01, 2.5524E+04, &
      4.046665E-01, -5.05260E+00, 1.1006E+01, 2.5461E+04, &
      4.062933E-01, -5.85041E+00, 1.0953E+01, 2.5236E+04, &
      4.087335E-01, -1.85144E+00, 1.5603E+01, 3.5735E+04, &
      4.102440E-01, -3.08777E-01, 1.5533E+01, 3.5444E+04, &
      4.148340E-01, 2.49519E+00, 1.5325E+01, 3.4581E+04, &
      4.195189E-01, 4.28326E+00, 1.5117E+01, 3.3730E+04, &
      4.484657E-01, 9.51097E+00, 1.3937E+01, 2.9091E+04, &
      4.794098E-01, 1.18389E+01, 1.2824E+01, 2.5040E+04, &
      4.903920E-01, 1.21697E+01, 1.2458E+01, 2.3781E+04, &
      4.978980E-01, 1.18920E+01, 1.2216E+01, 2.2968E+04, &
      4.998996E-01, 1.11773E+01, 1.2152E+01, 2.2757E+04, &
      5.029020E-01, 1.21617E+01, 1.3665E+01, 2.5435E+04, &
      5.104080E-01, 1.32441E+01, 1.3427E+01, 2.4625E+04, &
      5.124891E-01, 1.34384E+01, 1.3362E+01, 2.4407E+04, &
      5.478508E-01, 1.55039E+01, 1.2329E+01, 2.1066E+04, &
      5.856525E-01, 1.68387E+01, 1.1350E+01, 1.8144E+04, &
      6.260625E-01, 1.78352E+01, 1.0419E+01, 1.5581E+04, &
      6.692609E-01, 1.86004E+01, 9.5373E+00, 1.3342E+04, &
      7.154399E-01, 1.91936E+01, 8.7119E+00, 1.1401E+04, &
      7.648052E-01, 1.96541E+01, 7.9434E+00, 9.7248E+03, &
      8.175768E-01, 2.00108E+01, 7.2315E+00, 8.2822E+03, &
      8.739896E-01, 2.02868E+01, 6.5712E+00, 7.0407E+03, &
      9.342948E-01, 2.04928E+01, 5.9594E+00, 5.9735E+03, &
      9.987612E-01, 2.06451E+01, 5.3984E+00, 5.0624E+03, &
      1.067676E+00, 2.07586E+01, 4.8649E+00, 4.2682E+03, &
      1.141345E+00, 2.08218E+01, 4.3780E+00, 3.5937E+03, &
      1.220098E+00, 2.08494E+01, 3.9360E+00, 3.0228E+03, &
      1.304285E+00, 2.08499E+01, 3.5362E+00, 2.5410E+03, &
      1.394281E+00, 2.08295E+01, 3.1759E+00, 2.1353E+03, &
      1.490486E+00, 2.07942E+01, 2.8520E+00, 1.7943E+03, &
      1.593329E+00, 2.07506E+01, 2.5615E+00, 1.5080E+03, &
      1.703269E+00, 2.06959E+01, 2.2906E+00, 1.2619E+03, &
      1.820795E+00, 2.06294E+01, 2.0483E+00, 1.0561E+03, &
      1.946430E+00, 2.05567E+01, 1.8338E+00, 8.8491E+02, &
      2.080733E+00, 2.04800E+01, 1.6391E+00, 7.4036E+02, &
      2.224304E+00, 2.03965E+01, 1.4635E+00, 6.1882E+02, &
      2.377781E+00, 2.03074E+01, 1.3080E+00, 5.1776E+02, &
      2.541848E+00, 2.02132E+01, 1.1701E+00, 4.3368E+02, &
      2.717235E+00, 2.01132E+01, 1.0477E+00, 3.6363E+02, &
      2.904724E+00, 2.00074E+01, 9.3904E-01, 3.0524E+02, &
      3.105150E+00, 1.98896E+01, 8.3405E-01, 2.5397E+02, &
      3.319406E+00, 1.97362E+01, 7.3458E-01, 2.0960E+02, &
      3.548445E+00, 1.95416E+01, 6.4760E-01, 1.7320E+02, &
      3.793288E+00, 1.92752E+01, 5.7147E-01, 1.4330E+02, &
      4.055024E+00, 1.88520E+01, 5.0478E-01, 1.1870E+02, &
      4.334821E+00, 1.78700E+01, 4.4631E-01, 9.8458E+01, &
      4.402944E+00, 1.72904E+01, 4.3371E-01, 9.4266E+01, &
      4.470336E+00, 1.57929E+01, 4.2180E-01, 9.0360E+01, &
      4.488307E+00, 1.39862E+01, 4.1871E-01, 8.9356E+01, &
      4.515264E+00, 1.58273E+01, 3.9769E+00, 8.2646E+02, &
      4.582656E+00, 1.74949E+01, 3.8694E+00, 7.9237E+02, &
      4.633924E+00, 1.80517E+01, 3.7907E+00, 7.6769E+02, &
      4.953664E+00, 1.94991E+01, 3.3816E+00, 6.4087E+02, &
      5.295467E+00, 2.01389E+01, 3.0409E+00, 5.3932E+02, &
      5.660855E+00, 2.05351E+01, 2.7420E+00, 4.5511E+02, &
      6.051453E+00, 2.08130E+01, 2.4648E+00, 3.8286E+02, &
      6.469004E+00, 2.10038E+01, 2.2063E+00, 3.2077E+02, &
      6.915365E+00, 2.11368E+01, 1.9727E+00, 2.6846E+02, &
      7.392525E+00, 2.12296E+01, 1.7627E+00, 2.2454E+02, &
      7.902609E+00, 2.12935E+01, 1.5741E+00, 1.8772E+02, &
      8.447890E+00, 2.13369E+01, 1.4055E+00, 1.5693E+02, &
      9.030794E+00, 2.14081E+01, 1.2534E+00, 1.3104E+02, &
      9.653919E+00, 2.14223E+01, 1.1075E+00, 1.0844E+02, &
      1.032004E+01, 2.14230E+01, 9.7833E-01, 8.9718E+01, &
      1.103212E+01, 2.14153E+01, 8.6426E-01, 7.4249E+01, &
      1.179334E+01, 2.14022E+01, 7.6355E-01, 6.1461E+01, &
      1.260708E+01, 2.13854E+01, 6.7462E-01, 5.0889E+01, &
      1.347697E+01, 2.13664E+01, 5.9608E-01, 4.2148E+01, &
      1.440688E+01, 2.13461E+01, 5.2672E-01, 3.4919E+01, &
      1.540095E+01, 2.13254E+01, 4.6545E-01, 2.8939E+01, &
      1.646362E+01, 2.13047E+01, 4.1134E-01, 2.3993E+01, &
      1.759961E+01, 2.12844E+01, 3.6353E-01, 1.9900E+01, &
      1.881398E+01, 2.12651E+01, 3.2129E-01, 1.6513E+01, &
      2.011215E+01, 2.12467E+01, 2.8301E-01, 1.3664E+01, &
      2.149988E+01, 2.12285E+01, 2.4840E-01, 1.1276E+01, &
      2.298338E+01, 2.12108E+01, 2.1804E-01, 9.3116E+00, &
      2.456923E+01, 2.11940E+01, 1.9139E-01, 7.6960E+00, &
      2.626450E+01, 2.11783E+01, 1.6800E-01, 6.3666E+00, &
      2.807676E+01, 2.11637E+01, 1.4661E-01, 5.2442E+00, &
      3.001405E+01, 2.11496E+01, 1.2771E-01, 4.3182E+00, &
      3.208502E+01, 2.11362E+01, 1.1124E-01, 3.5617E+00, &
      3.429889E+01, 2.11237E+01, 9.6904E-02, 2.9434E+00, &
      3.666551E+01, 2.11122E+01, 8.4416E-02, 2.4379E+00, &
      3.919543E+01, 2.11015E+01, 7.3539E-02, 2.0244E+00, &
      4.189992E+01, 2.10917E+01, 6.4065E-02, 1.6862E+00, &
      4.479101E+01, 2.10827E+01, 5.5812E-02, 1.4093E+00, &
      4.788159E+01, 2.10746E+01, 4.8624E-02, 1.1824E+00, &
      5.118542E+01, 2.10671E+01, 4.2363E-02, 9.9650E-01, &
      5.471721E+01, 2.10604E+01, 3.6908E-02, 8.4399E-01, &
      5.849270E+01, 2.10543E+01, 3.2157E-02, 7.1880E-01, &
      6.252870E+01, 2.10487E+01, 2.8018E-02, 6.1591E-01, &
      6.684318E+01, 2.10437E+01, 2.4411E-02, 5.3127E-01, &
      7.145536E+01, 2.10392E+01, 2.1270E-02, 4.6154E-01, &
      7.638578E+01, 2.10351E+01, 1.8533E-02, 4.0398E-01, &
      8.165640E+01, 2.10315E+01, 1.6131E-02, 3.5621E-01, &
      8.729069E+01, 2.10282E+01, 1.4007E-02, 3.1630E-01, &
      9.331374E+01, 2.10252E+01, 1.2162E-02, 2.8324E-01, &
      9.975239E+01, 2.10225E+01, 1.0561E-02, 2.5578E-01, &
      1.066353E+02, 2.10201E+01, 9.1701E-03, 2.3288E-01, &
      1.139931E+02, 2.10179E+01, 7.9628E-03, 2.1370E-01, &
      1.218587E+02, 2.10160E+01, 6.9145E-03, 1.9754E-01, &
      1.302669E+02, 2.10143E+01, 6.0043E-03, 1.8386E-01, &
      1.392553E+02, 2.10127E+01, 5.2140E-03, 1.7221E-01, &
      1.488640E+02, 2.10113E+01, 4.5277E-03, 1.6220E-01, &
      1.591356E+02, 2.10101E+01, 3.9318E-03, 1.5354E-01, &
      1.701159E+02, 2.10090E+01, 3.4143E-03, 1.4598E-01, &
      1.818539E+02, 2.10080E+01, 2.9650E-03, 1.3931E-01, &
      1.944018E+02, 2.10071E+01, 2.5748E-03, 1.3338E-01, &
      2.078156E+02, 2.10063E+01, 2.2360E-03, 1.2805E-01, &
      2.221548E+02, 2.10056E+01, 1.9418E-03, 1.2321E-01, &
      2.374835E+02, 2.10050E+01, 1.6863E-03, 1.1878E-01, &
      2.538699E+02, 2.10044E+01, 1.4645E-03, 1.1468E-01, &
      2.713869E+02, 2.10039E+01, 1.2718E-03, 1.1086E-01, &
      2.901126E+02, 2.10035E+01, 1.1045E-03, 1.0726E-01, &
      3.101304E+02, 2.10031E+01, 9.5918E-04, 1.0386E-01, &
      3.315294E+02, 2.10027E+01, 8.3300E-04, 1.0061E-01, &
      3.544049E+02, 2.10024E+01, 7.2343E-04, 9.7507E-02, &
      3.788588E+02, 2.10021E+01, 6.2827E-04, 9.4516E-02, &
      4.050001E+02, 2.10019E+01, 5.4563E-04, 9.1627E-02, &
      4.329451E+02, 2.10017E+01, 4.7387E-04, 8.8828E-02]
   case(22)
      frel = -4.5000E-02
      allocate(nist_tab(760))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 9.42905E+00, 0.0000E+00, 1.4127E-04, &
      1.142761E-02, 9.42053E+00, 0.0000E+00, 1.9172E-04, &
      1.221612E-02, 9.41062E+00, 0.0000E+00, 2.5877E-04, &
      1.305903E-02, 9.39906E+00, 0.0000E+00, 3.4738E-04, &
      1.396010E-02, 9.38551E+00, 0.0000E+00, 4.6384E-04, &
      1.492335E-02, 9.36957E+00, 0.0000E+00, 6.1603E-04, &
      1.595306E-02, 9.35070E+00, 0.0000E+00, 8.1382E-04, &
      1.705382E-02, 9.32824E+00, 0.0000E+00, 1.0694E-03, &
      1.823053E-02, 9.30127E+00, 0.0000E+00, 1.3980E-03, &
      1.948844E-02, 9.26856E+00, 0.0000E+00, 1.8180E-03, &
      2.083314E-02, 9.22834E+00, 0.0000E+00, 2.3520E-03, &
      2.227063E-02, 9.17803E+00, 0.0000E+00, 3.0271E-03, &
      2.380730E-02, 9.11359E+00, 0.0000E+00, 3.8762E-03, &
      2.545001E-02, 9.02823E+00, 0.0000E+00, 4.9383E-03, &
      2.720606E-02, 8.90939E+00, 0.0000E+00, 6.2598E-03, &
      2.908327E-02, 8.73001E+00, 0.0000E+00, 7.8952E-03, &
      3.109002E-02, 8.41542E+00, 0.0000E+00, 9.9085E-03, &
      3.323523E-02, 7.60473E+00, 0.0000E+00, 1.2374E-02, &
      3.390800E-02, 6.89881E+00, 0.0000E+00, 1.3215E-02, &
      3.442700E-02, 5.21284E+00, 0.0000E+00, 1.3886E-02, &
      3.456540E-02, 3.01476E+00, 0.0000E+00, 1.4069E-02, &
      3.477300E-02, 5.60428E+00, 4.4077E+00, 1.1140E+05, &
      3.529200E-02, 7.86368E+00, 3.9360E+00, 9.8018E+04, &
      3.552846E-02, 8.34078E+00, 3.7403E+00, 9.2523E+04, &
      3.797993E-02, 9.99065E+00, 2.2468E+00, 5.1992E+04, &
      4.060054E-02, 1.01319E+01, 1.4413E+00, 3.1199E+04, &
      4.340198E-02, 1.00332E+01, 1.0503E+00, 2.1267E+04, &
      4.639671E-02, 9.89691E+00, 8.5004E-01, 1.6102E+04, &
      4.959809E-02, 9.75735E+00, 7.4921E-01, 1.3276E+04, &
      5.302035E-02, 9.61388E+00, 7.0691E-01, 1.1718E+04, &
      5.667876E-02, 9.44458E+00, 7.0362E-01, 1.0910E+04, &
      5.909400E-02, 9.26972E+00, 7.1680E-01, 1.0661E+04, &
      5.999850E-02, 9.10085E+00, 7.2413E-01, 1.0607E+04, &
      6.023970E-02, 8.92474E+00, 7.2628E-01, 1.0596E+04, &
      6.058959E-02, 9.07958E+00, 1.0619E+00, 1.5404E+04, &
      6.060150E-02, 9.08350E+00, 1.0621E+00, 1.5403E+04, &
      6.150600E-02, 9.20754E+00, 1.0777E+00, 1.5399E+04, &
      6.477028E-02, 9.28310E+00, 1.1418E+00, 1.5493E+04, &
      6.923942E-02, 9.29671E+00, 1.2446E+00, 1.5797E+04, &
      7.401695E-02, 9.30317E+00, 1.3647E+00, 1.6205E+04, &
      7.912411E-02, 9.31776E+00, 1.4997E+00, 1.6658E+04, &
      8.458368E-02, 9.34680E+00, 1.6471E+00, 1.7114E+04, &
      9.041995E-02, 9.39422E+00, 1.8036E+00, 1.7531E+04, &
      9.665893E-02, 9.46274E+00, 1.9656E+00, 1.7872E+04, &
      1.033284E-01, 9.55397E+00, 2.1285E+00, 1.8104E+04, &
      1.104581E-01, 9.66065E+00, 2.2874E+00, 1.8200E+04, &
      1.180797E-01, 9.79749E+00, 2.4371E+00, 1.8139E+04, &
      1.262272E-01, 9.95464E+00, 2.5726E+00, 1.7912E+04, &
      1.349368E-01, 1.01290E+01, 2.6893E+00, 1.7516E+04, &
      1.442475E-01, 1.03167E+01, 2.7827E+00, 1.6956E+04, &
      1.542005E-01, 1.05109E+01, 2.8469E+00, 1.6227E+04, &
      1.648404E-01, 1.07029E+01, 2.8809E+00, 1.5361E+04, &
      1.762144E-01, 1.08847E+01, 2.8854E+00, 1.4392E+04, &
      1.883732E-01, 1.10503E+01, 2.8628E+00, 1.3357E+04, &
      2.013709E-01, 1.11902E+01, 2.8158E+00, 1.2290E+04, &
      2.152655E-01, 1.12992E+01, 2.7480E+00, 1.1220E+04, &
      2.301188E-01, 1.13710E+01, 2.6634E+00, 1.0173E+04, &
      2.459970E-01, 1.13988E+01, 2.5655E+00, 9.1669E+03, &
      2.629708E-01, 1.13743E+01, 2.4579E+00, 8.2158E+03, &
      2.811158E-01, 1.12871E+01, 2.3437E+00, 7.3287E+03, &
      3.005128E-01, 1.11216E+01, 2.2258E+00, 6.5111E+03, &
      3.212482E-01, 1.08540E+01, 2.1067E+00, 5.7650E+03, &
      3.434143E-01, 1.04440E+01, 1.9881E+00, 5.0898E+03, &
      3.671099E-01, 9.81571E+00, 1.8718E+00, 4.4829E+03, &
      3.924405E-01, 8.80332E+00, 1.7587E+00, 3.9405E+03, &
      4.195189E-01, 6.92604E+00, 1.6498E+00, 3.4583E+03, &
      4.463900E-01, 1.92843E+00, 1.5526E+00, 3.0590E+03, &
      4.484657E-01, 9.87301E-01, 1.5455E+00, 3.0310E+03, &
      4.522700E-01, -1.76894E+00, 1.5327E+00, 2.9805E+03, &
      4.532225E-01, -2.96272E+00, 1.5295E+00, 2.9681E+03, &
      4.550445E-01, -8.18637E+00, 1.5234E+00, 2.9445E+03, &
      4.577775E-01, -3.90944E+00, 1.1107E+01, 2.1326E+04, &
      4.591925E-01, -3.08146E+00, 1.1064E+01, 2.1177E+04, &
      4.610385E-01, -4.19430E+00, 1.1008E+01, 2.0986E+04, &
      4.638075E-01, -3.65275E-01, 1.5678E+01, 2.9710E+04, &
      4.646100E-01, 4.04249E-01, 1.5643E+01, 2.9593E+04, &
      4.707300E-01, 3.83984E+00, 1.5387E+01, 2.8729E+04, &
      4.794098E-01, 6.44177E+00, 1.5035E+01, 2.7565E+04, &
      5.124891E-01, 1.10633E+01, 1.3816E+01, 2.3695E+04, &
      5.478508E-01, 1.30866E+01, 1.2665E+01, 2.0320E+04, &
      5.524260E-01, 1.31659E+01, 1.2526E+01, 1.9931E+04, &
      5.608815E-01, 1.28802E+01, 1.2274E+01, 1.9236E+04, &
      5.631363E-01, 1.21445E+01, 1.2208E+01, 1.9056E+04, &
      5.665185E-01, 1.31577E+01, 1.3763E+01, 2.1354E+04, &
      5.749740E-01, 1.42700E+01, 1.3515E+01, 2.0661E+04, &
      5.856525E-01, 1.50556E+01, 1.3211E+01, 1.9828E+04, &
      6.260625E-01, 1.69001E+01, 1.2153E+01, 1.7063E+04, &
      6.692609E-01, 1.81505E+01, 1.1157E+01, 1.4655E+04, &
      7.154399E-01, 1.90837E+01, 1.0224E+01, 1.2562E+04, &
      7.648052E-01, 1.98089E+01, 9.3601E+00, 1.0759E+04, &
      8.175768E-01, 2.03785E+01, 8.5328E+00, 9.1756E+03, &
      8.739896E-01, 2.08094E+01, 7.7655E+00, 7.8120E+03, &
      9.342948E-01, 2.11416E+01, 7.0640E+00, 6.6481E+03, &
      9.987612E-01, 2.14337E+01, 6.4217E+00, 5.6541E+03, &
      1.067676E+00, 2.16411E+01, 5.7774E+00, 4.7590E+03, &
      1.141345E+00, 2.17616E+01, 5.1954E+00, 4.0039E+03, &
      1.220098E+00, 2.18304E+01, 4.6702E+00, 3.3674E+03, &
      1.304285E+00, 2.18617E+01, 4.1983E+00, 2.8322E+03, &
      1.394281E+00, 2.18659E+01, 3.7746E+00, 2.3826E+03, &
      1.490486E+00, 2.18498E+01, 3.3947E+00, 2.0050E+03, &
      1.593329E+00, 2.18189E+01, 3.0542E+00, 1.6880E+03, &
      1.703269E+00, 2.17801E+01, 2.7492E+00, 1.4219E+03, &
      1.820795E+00, 2.17335E+01, 2.4762E+00, 1.1984E+03, &
      1.946430E+00, 2.16786E+01, 2.2158E+00, 1.0037E+03, &
      2.080733E+00, 2.16125E+01, 1.9839E+00, 8.4111E+02, &
      2.224304E+00, 2.15408E+01, 1.7785E+00, 7.0579E+02, &
      2.377781E+00, 2.14645E+01, 1.5901E+00, 5.9072E+02, &
      2.541848E+00, 2.13815E+01, 1.4215E+00, 4.9444E+02, &
      2.717235E+00, 2.12933E+01, 1.2720E+00, 4.1429E+02, &
      2.904724E+00, 2.12005E+01, 1.1394E+00, 3.4749E+02, &
      3.105150E+00, 2.11027E+01, 1.0116E+00, 2.8900E+02, &
      3.319406E+00, 2.09847E+01, 8.9095E-01, 2.3846E+02, &
      3.548445E+00, 2.08432E+01, 7.8546E-01, 1.9702E+02, &
      3.793288E+00, 2.06701E+01, 6.9316E-01, 1.6298E+02, &
      4.055024E+00, 2.04452E+01, 6.1232E-01, 1.3499E+02, &
      4.334821E+00, 2.01195E+01, 5.4145E-01, 1.1196E+02, &
      4.633924E+00, 1.95302E+01, 4.7926E-01, 9.2975E+01, &
      4.867072E+00, 1.83295E+01, 4.3839E-01, 8.1168E+01, &
      4.941568E+00, 1.68446E+01, 4.2651E-01, 7.7838E+01, &
      4.953664E+00, 1.61067E+01, 4.2463E-01, 7.7316E+01, &
      4.961434E+00, 1.50532E+01, 4.2343E-01, 7.6983E+01, &
      4.991232E+00, 1.68795E+01, 3.9516E+00, 6.9778E+02, &
      5.065728E+00, 1.85331E+01, 3.8444E+00, 6.6893E+02, &
      5.295467E+00, 1.99941E+01, 3.5500E+00, 5.9107E+02, &
      5.660855E+00, 2.08781E+01, 3.1852E+00, 4.9629E+02, &
      6.051453E+00, 2.13665E+01, 2.8699E+00, 4.1850E+02, &
      6.469004E+00, 2.16932E+01, 2.5886E+00, 3.5327E+02, &
      6.915365E+00, 2.19215E+01, 2.3196E+00, 2.9629E+02, &
      7.392525E+00, 2.20793E+01, 2.0760E+00, 2.4822E+02, &
      7.902609E+00, 2.21897E+01, 1.8566E+00, 2.0781E+02, &
      8.447890E+00, 2.22671E+01, 1.6599E+00, 1.7394E+02, &
      9.030794E+00, 2.23223E+01, 1.4823E+00, 1.4543E+02, &
      9.653919E+00, 2.23587E+01, 1.3190E+00, 1.2117E+02, &
      1.032004E+01, 2.24196E+01, 1.1692E+00, 1.0059E+02, &
      1.103212E+01, 2.24250E+01, 1.0330E+00, 8.3249E+01, &
      1.179334E+01, 2.24203E+01, 9.1250E-01, 6.8894E+01, &
      1.260708E+01, 2.24091E+01, 8.0612E-01, 5.7028E+01, &
      1.347697E+01, 2.23936E+01, 7.1218E-01, 4.7220E+01, &
      1.440688E+01, 2.23754E+01, 6.2924E-01, 3.9109E+01, &
      1.540095E+01, 2.23555E+01, 5.5599E-01, 3.2402E+01, &
      1.646362E+01, 2.23348E+01, 4.9128E-01, 2.6854E+01, &
      1.759961E+01, 2.23139E+01, 4.3414E-01, 2.2264E+01, &
      1.881398E+01, 2.22934E+01, 3.8365E-01, 1.8468E+01, &
      2.011215E+01, 2.22735E+01, 3.3906E-01, 1.5325E+01, &
      2.149988E+01, 2.22547E+01, 2.9966E-01, 1.2725E+01, &
      2.298338E+01, 2.22365E+01, 2.6320E-01, 1.0508E+01, &
      2.456923E+01, 2.22186E+01, 2.3115E-01, 8.6841E+00, &
      2.626450E+01, 2.22014E+01, 2.0302E-01, 7.1826E+00, &
      2.807676E+01, 2.21864E+01, 1.7732E-01, 5.9158E+00, &
      3.001405E+01, 2.21708E+01, 1.5460E-01, 4.8700E+00, &
      3.208502E+01, 2.21559E+01, 1.3479E-01, 4.0150E+00, &
      3.429889E+01, 2.21420E+01, 1.1752E-01, 3.3158E+00, &
      3.666551E+01, 2.21289E+01, 1.0247E-01, 2.7438E+00, &
      3.919543E+01, 2.21169E+01, 8.9345E-02, 2.2758E+00, &
      4.189992E+01, 2.21058E+01, 7.7906E-02, 1.8925E+00, &
      4.479101E+01, 2.20957E+01, 6.7933E-02, 1.5786E+00, &
      4.788159E+01, 2.20863E+01, 5.9238E-02, 1.3214E+00, &
      5.118542E+01, 2.20779E+01, 5.1657E-02, 1.1105E+00, &
      5.471721E+01, 2.20701E+01, 4.5047E-02, 9.3748E-01, &
      5.849270E+01, 2.20631E+01, 3.9284E-02, 7.9538E-01, &
      6.252870E+01, 2.20568E+01, 3.4259E-02, 6.7859E-01, &
      6.684318E+01, 2.20510E+01, 2.9877E-02, 5.8250E-01, &
      7.145536E+01, 2.20458E+01, 2.6057E-02, 5.0333E-01, &
      7.638578E+01, 2.20411E+01, 2.2725E-02, 4.3803E-01, &
      8.165640E+01, 2.20369E+01, 1.9787E-02, 3.8373E-01, &
      8.729069E+01, 2.20331E+01, 1.7165E-02, 3.3818E-01, &
      9.331374E+01, 2.20297E+01, 1.4891E-02, 3.0058E-01, &
      9.975239E+01, 2.20266E+01, 1.2918E-02, 2.6943E-01, &
      1.066353E+02, 2.20237E+01, 1.1207E-02, 2.4355E-01, &
      1.139931E+02, 2.20212E+01, 9.7228E-03, 2.2195E-01, &
      1.218587E+02, 2.20189E+01, 8.4351E-03, 2.0386E-01, &
      1.302669E+02, 2.20169E+01, 7.3180E-03, 1.8860E-01, &
      1.392553E+02, 2.20150E+01, 6.3489E-03, 1.7567E-01, &
      1.488640E+02, 2.20134E+01, 5.5082E-03, 1.6463E-01, &
      1.591356E+02, 2.20119E+01, 4.7789E-03, 1.5514E-01, &
      1.701159E+02, 2.20106E+01, 4.1462E-03, 1.4691E-01, &
      1.818539E+02, 2.20094E+01, 3.5972E-03, 1.3970E-01, &
      1.944018E+02, 2.20084E+01, 3.1210E-03, 1.3335E-01, &
      2.078156E+02, 2.20075E+01, 2.7079E-03, 1.2768E-01, &
      2.221548E+02, 2.20066E+01, 2.3494E-03, 1.2257E-01, &
      2.374835E+02, 2.20059E+01, 2.0384E-03, 1.1792E-01, &
      2.538699E+02, 2.20052E+01, 1.7686E-03, 1.1366E-01, &
      2.713869E+02, 2.20046E+01, 1.5346E-03, 1.0972E-01, &
      2.901126E+02, 2.20041E+01, 1.3315E-03, 1.0603E-01, &
      3.101304E+02, 2.20037E+01, 1.1553E-03, 1.0256E-01, &
      3.315294E+02, 2.20032E+01, 1.0024E-03, 9.9272E-02, &
      3.544049E+02, 2.20029E+01, 8.6974E-04, 9.6139E-02, &
      3.788588E+02, 2.20025E+01, 7.5465E-04, 9.3138E-02, &
      4.050001E+02, 2.20023E+01, 6.5480E-04, 9.0250E-02, &
      4.329451E+02, 2.20020E+01, 5.6816E-04, 8.7461E-02]
   case(23)
      frel = -5.0400E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 6.03298E+00, 1.3926E+00, 1.0761E+05, &
      1.142761E-02, 6.07191E+00, 1.4821E+00, 1.0713E+05, &
      1.221612E-02, 6.11684E+00, 1.5754E+00, 1.0653E+05, &
      1.305903E-02, 6.16824E+00, 1.6720E+00, 1.0576E+05, &
      1.396010E-02, 6.22652E+00, 1.7714E+00, 1.0482E+05, &
      1.492335E-02, 6.29198E+00, 1.8729E+00, 1.0367E+05, &
      1.595306E-02, 6.36478E+00, 1.9757E+00, 1.0230E+05, &
      1.705382E-02, 6.44492E+00, 2.0791E+00, 1.0071E+05, &
      1.823053E-02, 6.53219E+00, 2.1821E+00, 9.8873E+04, &
      1.948844E-02, 6.62612E+00, 2.2838E+00, 9.6802E+04, &
      2.083314E-02, 6.72589E+00, 2.3833E+00, 9.4499E+04, &
      2.227063E-02, 6.83025E+00, 2.4796E+00, 9.1974E+04, &
      2.380730E-02, 6.93728E+00, 2.5720E+00, 8.9242E+04, &
      2.545001E-02, 7.04404E+00, 2.6595E+00, 8.6323E+04, &
      2.720606E-02, 7.14583E+00, 2.7415E+00, 8.3240E+04, &
      2.908327E-02, 7.23437E+00, 2.8172E+00, 8.0018E+04, &
      3.109002E-02, 7.29311E+00, 2.8862E+00, 7.6686E+04, &
      3.323523E-02, 7.28097E+00, 2.9480E+00, 7.3272E+04, &
      3.552846E-02, 7.04435E+00, 3.0024E+00, 6.9808E+04, &
      3.704400E-02, 6.31229E+00, 3.0326E+00, 6.7625E+04, &
      3.761100E-02, 5.04483E+00, 3.0429E+00, 6.6831E+04, &
      3.776220E-02, 3.35785E+00, 3.0455E+00, 6.6621E+04, &
      3.797993E-02, 5.28829E+00, 6.4585E+00, 1.4047E+05, &
      3.798900E-02, 5.34875E+00, 6.4533E+00, 1.4032E+05, &
      3.855600E-02, 7.12733E+00, 6.1450E+00, 1.3165E+05, &
      4.060054E-02, 8.81634E+00, 5.2816E+00, 1.0746E+05, &
      4.340198E-02, 9.44378E+00, 4.4661E+00, 8.5002E+04, &
      4.639671E-02, 9.66967E+00, 3.9727E+00, 7.0730E+04, &
      4.959809E-02, 9.79279E+00, 3.6535E+00, 6.0848E+04, &
      5.302035E-02, 9.87310E+00, 3.4244E+00, 5.3352E+04, &
      5.667876E-02, 9.92252E+00, 3.2483E+00, 4.7341E+04, &
      6.058959E-02, 9.93366E+00, 3.1077E+00, 4.2369E+04, &
      6.477028E-02, 9.84482E+00, 2.9948E+00, 3.8195E+04, &
      6.517000E-02, 9.81827E+00, 2.9856E+00, 3.7844E+04, &
      6.616750E-02, 9.66616E+00, 2.9639E+00, 3.7002E+04, &
      6.643350E-02, 9.48206E+00, 2.9583E+00, 3.6784E+04, &
      6.683250E-02, 9.66842E+00, 3.3136E+00, 4.0957E+04, &
      6.783000E-02, 9.83442E+00, 3.3006E+00, 4.0195E+04, &
      6.923942E-02, 9.92865E+00, 3.2845E+00, 3.9185E+04, &
      7.401695E-02, 1.00868E+01, 3.2481E+00, 3.6250E+04, &
      7.912411E-02, 1.01956E+01, 3.2329E+00, 3.3751E+04, &
      8.458368E-02, 1.02928E+01, 3.2361E+00, 3.1604E+04, &
      9.041995E-02, 1.03896E+01, 3.2560E+00, 2.9746E+04, &
      9.665893E-02, 1.04924E+01, 3.2903E+00, 2.8119E+04, &
      1.033284E-01, 1.06054E+01, 3.3358E+00, 2.6668E+04, &
      1.104581E-01, 1.07319E+01, 3.3885E+00, 2.5340E+04, &
      1.180797E-01, 1.08633E+01, 3.4437E+00, 2.4091E+04, &
      1.262272E-01, 1.10204E+01, 3.4966E+00, 2.2882E+04, &
      1.349368E-01, 1.11921E+01, 3.5423E+00, 2.1685E+04, &
      1.442475E-01, 1.13764E+01, 3.5763E+00, 2.0481E+04, &
      1.542005E-01, 1.15703E+01, 3.5945E+00, 1.9257E+04, &
      1.648404E-01, 1.17692E+01, 3.5910E+00, 1.7996E+04, &
      1.762144E-01, 1.19653E+01, 3.5633E+00, 1.6705E+04, &
      1.883732E-01, 1.21513E+01, 3.5119E+00, 1.5401E+04, &
      2.013709E-01, 1.23224E+01, 3.4384E+00, 1.4106E+04, &
      2.152655E-01, 1.24690E+01, 3.3451E+00, 1.2837E+04, &
      2.301188E-01, 1.25871E+01, 3.2352E+00, 1.1614E+04, &
      2.459970E-01, 1.26713E+01, 3.1120E+00, 1.0451E+04, &
      2.629708E-01, 1.27160E+01, 2.9789E+00, 9.3587E+03, &
      2.811158E-01, 1.27150E+01, 2.8390E+00, 8.3437E+03, &
      3.005128E-01, 1.26602E+01, 2.6950E+00, 7.4096E+03, &
      3.212482E-01, 1.25404E+01, 2.5496E+00, 6.5578E+03, &
      3.434143E-01, 1.23391E+01, 2.4050E+00, 5.7869E+03, &
      3.671099E-01, 1.20299E+01, 2.2630E+00, 5.0939E+03, &
      3.924405E-01, 1.15670E+01, 2.1248E+00, 4.4745E+03, &
      4.195189E-01, 1.08618E+01, 1.9915E+00, 3.9235E+03, &
      4.484657E-01, 9.71151E+00, 1.8639E+00, 3.4356E+03, &
      4.794098E-01, 7.46757E+00, 1.7424E+00, 3.0047E+03, &
      5.026420E-01, 3.15839E+00, 1.6601E+00, 2.7307E+03, &
      5.100900E-01, -1.39635E+00, 1.6351E+00, 2.6505E+03, &
      5.103355E-01, -1.70484E+00, 1.6343E+00, 2.6479E+03, &
      5.124891E-01, -7.62793E+00, 1.6273E+00, 2.6254E+03, &
      5.154645E-01, -2.50381E+00, 1.1276E+01, 1.8073E+04, &
      5.178975E-01, -1.34166E+00, 1.1206E+01, 1.7877E+04, &
      5.199795E-01, -2.63868E+00, 1.1147E+01, 1.7711E+04, &
      5.231025E-01, 1.09287E+00, 1.5841E+01, 2.5019E+04, &
      5.231580E-01, 1.14400E+00, 1.5839E+01, 2.5013E+04, &
      5.309100E-01, 5.20730E+00, 1.5532E+01, 2.4170E+04, &
      5.478508E-01, 9.04877E+00, 1.4897E+01, 2.2465E+04, &
      5.856525E-01, 1.28372E+01, 1.3629E+01, 1.9226E+04, &
      6.156360E-01, 1.40588E+01, 1.2738E+01, 1.7094E+04, &
      6.250590E-01, 1.37816E+01, 1.2476E+01, 1.6490E+04, &
      6.260625E-01, 1.36222E+01, 1.2448E+01, 1.6427E+04, &
      6.275718E-01, 1.30350E+01, 1.2407E+01, 1.6334E+04, &
      6.313410E-01, 1.40717E+01, 1.3990E+01, 1.8307E+04, &
      6.407640E-01, 1.52143E+01, 1.3733E+01, 1.7707E+04, &
      6.692609E-01, 1.68550E+01, 1.3000E+01, 1.6048E+04, &
      7.154399E-01, 1.84554E+01, 1.1939E+01, 1.3787E+04, &
      7.648052E-01, 1.95960E+01, 1.0937E+01, 1.1815E+04, &
      8.175768E-01, 2.04566E+01, 1.0006E+01, 1.0113E+04, &
      8.739896E-01, 2.11301E+01, 9.1452E+00, 8.6469E+03, &
      9.342948E-01, 2.16581E+01, 8.3284E+00, 7.3668E+03, &
      9.987612E-01, 2.20899E+01, 7.5803E+00, 6.2729E+03, &
      1.067676E+00, 2.24125E+01, 6.8343E+00, 5.2911E+03, &
      1.141345E+00, 2.26235E+01, 6.1482E+00, 4.4533E+03, &
      1.220098E+00, 2.27603E+01, 5.5300E+00, 3.7475E+03, &
      1.304285E+00, 2.28438E+01, 4.9725E+00, 3.1528E+03, &
      1.394281E+00, 2.28869E+01, 4.4710E+00, 2.6524E+03, &
      1.490486E+00, 2.28996E+01, 4.0207E+00, 2.2318E+03, &
      1.593329E+00, 2.28934E+01, 3.6167E+00, 1.8786E+03, &
      1.703269E+00, 2.28669E+01, 3.2545E+00, 1.5818E+03, &
      1.820795E+00, 2.28285E+01, 2.9300E+00, 1.3327E+03, &
      1.946430E+00, 2.27819E+01, 2.6393E+00, 1.1234E+03, &
      2.080733E+00, 2.27317E+01, 2.3764E+00, 9.4670E+02, &
      2.224304E+00, 2.26720E+01, 2.1275E+00, 7.9330E+02, &
      2.377781E+00, 2.26035E+01, 1.9068E+00, 6.6559E+02, &
      2.541848E+00, 2.25308E+01, 1.7099E+00, 5.5873E+02, &
      2.717235E+00, 2.24528E+01, 1.5291E+00, 4.6782E+02, &
      2.904724E+00, 2.23715E+01, 1.3687E+00, 3.9211E+02, &
      3.105150E+00, 2.22923E+01, 1.2150E+00, 3.2602E+02, &
      3.319406E+00, 2.21901E+01, 1.0704E+00, 2.6907E+02, &
      3.548445E+00, 2.20721E+01, 9.4397E-01, 2.2234E+02, &
      3.793288E+00, 2.19364E+01, 8.3331E-01, 1.8396E+02, &
      4.055024E+00, 2.17762E+01, 7.3636E-01, 1.5239E+02, &
      4.334821E+00, 2.15764E+01, 6.5136E-01, 1.2640E+02, &
      4.633924E+00, 2.13044E+01, 5.7675E-01, 1.0498E+02, &
      4.953664E+00, 2.08694E+01, 5.1121E-01, 8.7312E+01, &
      5.295467E+00, 1.98136E+01, 4.5356E-01, 7.2715E+01, &
      5.355798E+00, 1.93670E+01, 4.4449E-01, 7.0501E+01, &
      5.437774E+00, 1.78916E+01, 4.3262E-01, 6.7642E+01, &
      5.459635E+00, 1.61109E+01, 4.2954E-01, 6.6907E+01, &
      5.492425E+00, 1.79281E+01, 3.9371E+00, 5.9404E+02, &
      5.574402E+00, 1.95732E+01, 3.8274E+00, 5.6905E+02, &
      5.660855E+00, 2.02817E+01, 3.7167E+00, 5.4421E+02, &
      6.051453E+00, 2.15912E+01, 3.3154E+00, 4.5433E+02, &
      6.469004E+00, 2.21944E+01, 2.9844E+00, 3.8275E+02, &
      6.915365E+00, 2.25734E+01, 2.6931E+00, 3.2327E+02, &
      7.392525E+00, 2.28396E+01, 2.4213E+00, 2.7203E+02, &
      7.902609E+00, 2.30228E+01, 2.1691E+00, 2.2812E+02, &
      8.447890E+00, 2.31513E+01, 1.9414E+00, 1.9114E+02, &
      9.030794E+00, 2.32454E+01, 1.7355E+00, 1.5997E+02, &
      9.653919E+00, 2.33090E+01, 1.5457E+00, 1.3340E+02, &
      1.032004E+01, 2.33493E+01, 1.3765E+00, 1.1125E+02, &
      1.103212E+01, 2.34148E+01, 1.2246E+00, 9.2693E+01, &
      1.179334E+01, 2.34254E+01, 1.0823E+00, 7.6740E+01, &
      1.260708E+01, 2.34240E+01, 9.5604E-01, 6.3507E+01, &
      1.347697E+01, 2.34149E+01, 8.4453E-01, 5.2570E+01, &
      1.440688E+01, 2.34008E+01, 7.4607E-01, 4.3529E+01, &
      1.540095E+01, 2.33833E+01, 6.5914E-01, 3.6053E+01, &
      1.646362E+01, 2.33639E+01, 5.8237E-01, 2.9871E+01, &
      1.759961E+01, 2.33434E+01, 5.1457E-01, 2.4758E+01, &
      1.881398E+01, 2.33225E+01, 4.5469E-01, 2.0528E+01, &
      2.011215E+01, 2.33018E+01, 4.0180E-01, 1.7029E+01, &
      2.149988E+01, 2.32816E+01, 3.5508E-01, 1.4133E+01, &
      2.298338E+01, 2.32624E+01, 3.1380E-01, 1.1736E+01, &
      2.456923E+01, 2.32443E+01, 2.7646E-01, 9.7222E+00, &
      2.626450E+01, 2.32271E+01, 2.4293E-01, 8.0403E+00, &
      2.807676E+01, 2.32099E+01, 2.1236E-01, 6.6223E+00, &
      3.001405E+01, 2.31928E+01, 1.8533E-01, 5.4517E+00, &
      3.208502E+01, 2.31764E+01, 1.6172E-01, 4.4936E+00, &
      3.429889E+01, 2.31610E+01, 1.4113E-01, 3.7095E+00, &
      3.666551E+01, 2.31466E+01, 1.2316E-01, 3.0677E+00, &
      3.919543E+01, 2.31332E+01, 1.0749E-01, 2.5421E+00, &
      4.189992E+01, 2.31208E+01, 9.3808E-02, 2.1115E+00, &
      4.479101E+01, 2.31094E+01, 8.1873E-02, 1.7587E+00, &
      4.788159E+01, 2.30989E+01, 7.1458E-02, 1.4694E+00, &
      5.118542E+01, 2.30894E+01, 6.2370E-02, 1.2320E+00, &
      5.471721E+01, 2.30806E+01, 5.4438E-02, 1.0372E+00, &
      5.849270E+01, 2.30727E+01, 4.7517E-02, 8.7722E-01, &
      6.252870E+01, 2.30654E+01, 4.1476E-02, 7.4565E-01, &
      6.684318E+01, 2.30589E+01, 3.6204E-02, 6.3741E-01, &
      7.145536E+01, 2.30530E+01, 3.1603E-02, 5.4823E-01, &
      7.638578E+01, 2.30476E+01, 2.7587E-02, 4.7467E-01, &
      8.165640E+01, 2.30431E+01, 2.4042E-02, 4.1350E-01, &
      8.729069E+01, 2.30386E+01, 2.0874E-02, 3.6221E-01, &
      9.331374E+01, 2.30346E+01, 1.8123E-02, 3.1989E-01, &
      9.975239E+01, 2.30310E+01, 1.5735E-02, 2.8489E-01, &
      1.066353E+02, 2.30277E+01, 1.3662E-02, 2.5584E-01, &
      1.139931E+02, 2.30247E+01, 1.1862E-02, 2.3166E-01, &
      1.218587E+02, 2.30221E+01, 1.0300E-02, 2.1145E-01, &
      1.302669E+02, 2.30197E+01, 8.9431E-03, 1.9446E-01, &
      1.392553E+02, 2.30176E+01, 7.7653E-03, 1.8011E-01, &
      1.488640E+02, 2.30157E+01, 6.7426E-03, 1.6792E-01, &
      1.591356E+02, 2.30140E+01, 5.8547E-03, 1.5748E-01, &
      1.701159E+02, 2.30124E+01, 5.0838E-03, 1.4848E-01, &
      1.818539E+02, 2.30111E+01, 4.4144E-03, 1.4064E-01, &
      1.944018E+02, 2.30099E+01, 3.8332E-03, 1.3378E-01, &
      2.078156E+02, 2.30088E+01, 3.3286E-03, 1.2769E-01, &
      2.221548E+02, 2.30078E+01, 2.8904E-03, 1.2226E-01, &
      2.374835E+02, 2.30069E+01, 2.5099E-03, 1.1735E-01, &
      2.538699E+02, 2.30062E+01, 2.1795E-03, 1.1287E-01, &
      2.713869E+02, 2.30055E+01, 1.8926E-03, 1.0876E-01, &
      2.901126E+02, 2.30049E+01, 1.6435E-03, 1.0495E-01, &
      3.101304E+02, 2.30043E+01, 1.4272E-03, 1.0138E-01, &
      3.315294E+02, 2.30038E+01, 1.2394E-03, 9.8023E-02, &
      3.544049E+02, 2.30034E+01, 1.0763E-03, 9.4842E-02, &
      3.788588E+02, 2.30030E+01, 9.3467E-04, 9.1810E-02, &
      4.050001E+02, 2.30027E+01, 8.1168E-04, 8.8906E-02, &
      4.329451E+02, 2.30024E+01, 7.0488E-04, 8.6115E-02]
   case(24)
      frel = -5.5800E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 4.91074E+00, 2.0464E+00, 1.5492E+05, &
      1.142761E-02, 4.96917E+00, 2.1522E+00, 1.5242E+05, &
      1.221612E-02, 5.03413E+00, 2.2639E+00, 1.4998E+05, &
      1.305903E-02, 5.10636E+00, 2.3810E+00, 1.4755E+05, &
      1.396010E-02, 5.18659E+00, 2.5029E+00, 1.4510E+05, &
      1.492335E-02, 5.27545E+00, 2.6290E+00, 1.4257E+05, &
      1.595306E-02, 5.37345E+00, 2.7586E+00, 1.3994E+05, &
      1.705382E-02, 5.48100E+00, 2.8907E+00, 1.3718E+05, &
      1.823053E-02, 5.59831E+00, 3.0245E+00, 1.3427E+05, &
      1.948844E-02, 5.72538E+00, 3.1589E+00, 1.3118E+05, &
      2.083314E-02, 5.86196E+00, 3.2927E+00, 1.2791E+05, &
      2.227063E-02, 6.00743E+00, 3.4249E+00, 1.2446E+05, &
      2.380730E-02, 6.16072E+00, 3.5542E+00, 1.2082E+05, &
      2.545001E-02, 6.32013E+00, 3.6795E+00, 1.1701E+05, &
      2.720606E-02, 6.48301E+00, 3.7997E+00, 1.1303E+05, &
      2.908327E-02, 6.64508E+00, 3.9138E+00, 1.0891E+05, &
      3.109002E-02, 6.79915E+00, 4.0208E+00, 1.0466E+05, &
      3.323523E-02, 6.93175E+00, 4.1198E+00, 1.0032E+05, &
      3.552846E-02, 7.01362E+00, 4.2103E+00, 9.5907E+04, &
      3.797993E-02, 6.96365E+00, 4.2917E+00, 9.1450E+04, &
      4.060054E-02, 6.40477E+00, 4.3636E+00, 8.6981E+04, &
      4.165000E-02, 5.50583E+00, 4.3886E+00, 8.5274E+04, &
      4.228750E-02, 3.43782E+00, 4.4027E+00, 8.4260E+04, &
      4.245750E-02, 6.22173E-01, 4.4064E+00, 8.3992E+04, &
      4.271250E-02, 4.05524E+00, 1.0114E+01, 1.9164E+05, &
      4.335000E-02, 7.09375E+00, 9.4318E+00, 1.7608E+05, &
      4.340198E-02, 7.22680E+00, 9.3778E+00, 1.7486E+05, &
      4.639671E-02, 1.00687E+01, 7.0694E+00, 1.2331E+05, &
      4.959809E-02, 1.05444E+01, 5.8022E+00, 9.4676E+04, &
      5.302035E-02, 1.06760E+01, 5.1298E+00, 7.8301E+04, &
      5.667876E-02, 1.07528E+01, 4.7082E+00, 6.7227E+04, &
      6.058959E-02, 1.08150E+01, 4.4061E+00, 5.8853E+04, &
      6.477028E-02, 1.08599E+01, 4.1693E+00, 5.2095E+04, &
      6.923942E-02, 1.08662E+01, 3.9739E+00, 4.6448E+04, &
      7.261800E-02, 1.07857E+01, 3.8536E+00, 4.2946E+04, &
      7.372950E-02, 1.06519E+01, 3.8182E+00, 4.1910E+04, &
      7.401695E-02, 1.04986E+01, 3.8093E+00, 4.1651E+04, &
      7.402590E-02, 1.04869E+01, 3.8090E+00, 4.1643E+04, &
      7.447050E-02, 1.06552E+01, 4.1232E+00, 4.4808E+04, &
      7.558200E-02, 1.08074E+01, 4.0982E+00, 4.3882E+04, &
      7.912411E-02, 1.09676E+01, 4.0301E+00, 4.1220E+04, &
      8.458368E-02, 1.11006E+01, 3.9538E+00, 3.7830E+04, &
      9.041995E-02, 1.12096E+01, 3.8999E+00, 3.4906E+04, &
      9.665893E-02, 1.13121E+01, 3.8667E+00, 3.2374E+04, &
      1.033284E-01, 1.14160E+01, 3.8526E+00, 3.0175E+04, &
      1.104581E-01, 1.15266E+01, 3.8554E+00, 2.8248E+04, &
      1.180797E-01, 1.16482E+01, 3.8716E+00, 2.6536E+04, &
      1.262272E-01, 1.17840E+01, 3.8970E+00, 2.4985E+04, &
      1.349368E-01, 1.19285E+01, 3.9264E+00, 2.3550E+04, &
      1.442475E-01, 1.20966E+01, 3.9543E+00, 2.2186E+04, &
      1.542005E-01, 1.22801E+01, 3.9752E+00, 2.0864E+04, &
      1.648404E-01, 1.24764E+01, 3.9841E+00, 1.9561E+04, &
      1.762144E-01, 1.26825E+01, 3.9761E+00, 1.8262E+04, &
      1.883732E-01, 1.28920E+01, 3.9445E+00, 1.6948E+04, &
      2.013709E-01, 1.30964E+01, 3.8880E+00, 1.5627E+04, &
      2.152655E-01, 1.32879E+01, 3.8071E+00, 1.4314E+04, &
      2.301188E-01, 1.34608E+01, 3.7041E+00, 1.3028E+04, &
      2.459970E-01, 1.36061E+01, 3.5818E+00, 1.1785E+04, &
      2.629708E-01, 1.37191E+01, 3.4440E+00, 1.0600E+04, &
      2.811158E-01, 1.37945E+01, 3.2943E+00, 9.4855E+03, &
      3.005128E-01, 1.38264E+01, 3.1366E+00, 8.4486E+03, &
      3.212482E-01, 1.38083E+01, 2.9740E+00, 7.4940E+03, &
      3.434143E-01, 1.37312E+01, 2.8096E+00, 6.6231E+03, &
      3.671099E-01, 1.35829E+01, 2.6461E+00, 5.8354E+03, &
      3.924405E-01, 1.33444E+01, 2.4855E+00, 5.1279E+03, &
      4.195189E-01, 1.29845E+01, 2.3296E+00, 4.4964E+03, &
      4.484657E-01, 1.24471E+01, 2.1795E+00, 3.9355E+03, &
      4.794098E-01, 1.16190E+01, 2.0360E+00, 3.4396E+03, &
      5.124891E-01, 1.02203E+01, 1.8997E+00, 3.0028E+03, &
      5.478508E-01, 7.17813E+00, 1.7709E+00, 2.6189E+03, &
      5.630100E-01, 4.03264E+00, 1.7204E+00, 2.4758E+03, &
      5.720260E-01, -1.46698E+00, 1.6915E+00, 2.3962E+03, &
      5.739255E-01, -6.34058E+00, 1.6856E+00, 2.3798E+03, &
      5.773725E-01, -1.66698E+00, 1.1688E+01, 1.6386E+04, &
      5.807815E-01, -2.33901E-01, 1.1587E+01, 1.6149E+04, &
      5.831163E-01, -1.67326E+00, 1.1518E+01, 1.5989E+04, &
      5.856525E-01, 1.18309E+00, 1.6406E+01, 2.2674E+04, &
      5.859900E-01, 1.54437E+00, 1.6392E+01, 2.2641E+04, &
      5.866185E-01, 2.12272E+00, 1.6365E+01, 2.2581E+04, &
      5.953740E-01, 6.31539E+00, 1.6009E+01, 2.1764E+04, &
      6.260625E-01, 1.16720E+01, 1.4855E+01, 1.9206E+04, &
      6.692609E-01, 1.45819E+01, 1.3462E+01, 1.6282E+04, &
      6.807080E-01, 1.48744E+01, 1.3133E+01, 1.5617E+04, &
      6.911270E-01, 1.46210E+01, 1.2846E+01, 1.5046E+04, &
      6.939054E-01, 1.38694E+01, 1.2771E+01, 1.4898E+04, &
      6.980730E-01, 1.49312E+01, 1.4371E+01, 1.6664E+04, &
      7.084920E-01, 1.61104E+01, 1.4093E+01, 1.6102E+04, &
      7.154399E-01, 1.65908E+01, 1.3914E+01, 1.5742E+04, &
      7.648052E-01, 1.87137E+01, 1.2748E+01, 1.3493E+04, &
      8.175768E-01, 2.00955E+01, 1.1672E+01, 1.1558E+04, &
      8.739896E-01, 2.11147E+01, 1.0670E+01, 9.8836E+03, &
      9.342948E-01, 2.19000E+01, 9.7452E+00, 8.4451E+03, &
      9.987612E-01, 2.25466E+01, 8.8912E+00, 7.2083E+03, &
      1.067676E+00, 2.30148E+01, 8.0226E+00, 6.0849E+03, &
      1.141345E+00, 2.33470E+01, 7.2353E+00, 5.1341E+03, &
      1.220098E+00, 2.35870E+01, 6.5161E+00, 4.3259E+03, &
      1.304285E+00, 2.37474E+01, 5.8615E+00, 3.6408E+03, &
      1.394281E+00, 2.38492E+01, 5.2721E+00, 3.0640E+03, &
      1.490486E+00, 2.39066E+01, 4.7405E+00, 2.5778E+03, &
      1.593329E+00, 2.39301E+01, 4.2630E+00, 2.1691E+03, &
      1.703269E+00, 2.39280E+01, 3.8344E+00, 1.8257E+03, &
      1.820795E+00, 2.39099E+01, 3.4502E+00, 1.5372E+03, &
      1.946430E+00, 2.38752E+01, 3.1057E+00, 1.2950E+03, &
      2.080733E+00, 2.38307E+01, 2.7971E+00, 1.0915E+03, &
      2.224304E+00, 2.37813E+01, 2.5207E+00, 9.2068E+02, &
      2.377781E+00, 2.37276E+01, 2.2658E+00, 7.7463E+02, &
      2.541848E+00, 2.36639E+01, 2.0299E+00, 6.4969E+02, &
      2.717235E+00, 2.35939E+01, 1.8208E+00, 5.4558E+02, &
      2.904724E+00, 2.35212E+01, 1.6307E+00, 4.5753E+02, &
      3.105150E+00, 2.34551E+01, 1.4474E+00, 3.8033E+02, &
      3.319406E+00, 2.33672E+01, 1.2756E+00, 3.1398E+02, &
      3.548445E+00, 2.32625E+01, 1.1254E+00, 2.5953E+02, &
      3.793288E+00, 2.31453E+01, 9.9380E-01, 2.1477E+02, &
      4.055024E+00, 2.30137E+01, 8.7851E-01, 1.7796E+02, &
      4.334821E+00, 2.28622E+01, 7.7738E-01, 1.4765E+02, &
      4.633924E+00, 2.26790E+01, 6.8860E-01, 1.2265E+02, &
      4.953664E+00, 2.24400E+01, 6.1057E-01, 1.0203E+02, &
      5.295467E+00, 2.20858E+01, 5.4194E-01, 8.4987E+01, &
      5.660855E+00, 2.13981E+01, 4.8151E-01, 7.0888E+01, &
      5.869416E+00, 2.04015E+01, 4.5180E-01, 6.4284E+01, &
      5.959254E+00, 1.89371E+01, 4.3991E-01, 6.1707E+01, &
      5.983211E+00, 1.71703E+01, 4.3683E-01, 6.1043E+01, &
      6.019146E+00, 1.89719E+01, 3.9165E+00, 5.2853E+02, &
      6.051453E+00, 1.98252E+01, 3.8773E+00, 5.2048E+02, &
      6.108984E+00, 2.06031E+01, 3.8091E+00, 5.0654E+02, &
      6.469004E+00, 2.22661E+01, 3.4413E+00, 4.3235E+02, &
      6.915365E+00, 2.30221E+01, 3.0935E+00, 3.6375E+02, &
      7.392525E+00, 2.34637E+01, 2.7898E+00, 3.0703E+02, &
      7.902609E+00, 2.37651E+01, 2.5128E+00, 2.5885E+02, &
      8.447890E+00, 2.39746E+01, 2.2524E+00, 2.1721E+02, &
      9.030794E+00, 2.41269E+01, 2.0153E+00, 1.8195E+02, &
      9.653919E+00, 2.42270E+01, 1.7961E+00, 1.5182E+02, &
      1.032004E+01, 2.42944E+01, 1.6008E+00, 1.2671E+02, &
      1.103212E+01, 2.43392E+01, 1.4269E+00, 1.0576E+02, &
      1.179334E+01, 2.43684E+01, 1.2717E+00, 8.8283E+01, &
      1.260708E+01, 2.44243E+01, 1.1261E+00, 7.3235E+01, &
      1.347697E+01, 2.44264E+01, 9.9494E-01, 6.0627E+01, &
      1.440688E+01, 2.44195E+01, 8.7882E-01, 5.0186E+01, &
      1.540095E+01, 2.44067E+01, 7.7631E-01, 4.1555E+01, &
      1.646362E+01, 2.43901E+01, 6.8580E-01, 3.4420E+01, &
      1.759961E+01, 2.43712E+01, 6.0588E-01, 2.8519E+01, &
      1.881398E+01, 2.43509E+01, 5.3531E-01, 2.3639E+01, &
      2.011215E+01, 2.43301E+01, 4.7298E-01, 1.9601E+01, &
      2.149988E+01, 2.43093E+01, 4.1793E-01, 1.6262E+01, &
      2.298338E+01, 2.42889E+01, 3.6931E-01, 1.3497E+01, &
      2.456923E+01, 2.42694E+01, 3.2636E-01, 1.1210E+01, &
      2.626450E+01, 2.42512E+01, 2.8815E-01, 9.3084E+00, &
      2.807676E+01, 2.42333E+01, 2.5208E-01, 7.6676E+00, &
      3.001405E+01, 2.42153E+01, 2.2018E-01, 6.3126E+00, &
      3.208502E+01, 2.41978E+01, 1.9232E-01, 5.2032E+00, &
      3.429889E+01, 2.41810E+01, 1.6799E-01, 4.2946E+00, &
      3.666551E+01, 2.41653E+01, 1.4673E-01, 3.5499E+00, &
      3.919543E+01, 2.41506E+01, 1.2816E-01, 2.9397E+00, &
      4.189992E+01, 2.41369E+01, 1.1195E-01, 2.4396E+00, &
      4.479101E+01, 2.41242E+01, 9.7787E-02, 2.0295E+00, &
      4.788159E+01, 2.41125E+01, 8.5421E-02, 1.6930E+00, &
      5.118542E+01, 2.41018E+01, 7.4621E-02, 1.4168E+00, &
      5.471721E+01, 2.40920E+01, 6.5188E-02, 1.1900E+00, &
      5.849270E+01, 2.40831E+01, 5.6949E-02, 1.0037E+00, &
      6.252870E+01, 2.40750E+01, 4.9753E-02, 8.5043E-01, &
      6.684318E+01, 2.40676E+01, 4.3467E-02, 7.2429E-01, &
      7.145536E+01, 2.40609E+01, 3.7976E-02, 6.2037E-01, &
      7.638578E+01, 2.40549E+01, 3.3179E-02, 5.3466E-01, &
      8.165640E+01, 2.40495E+01, 2.8939E-02, 4.6337E-01, &
      8.729069E+01, 2.40444E+01, 2.5144E-02, 4.0359E-01, &
      9.331374E+01, 2.40399E+01, 2.1847E-02, 3.5431E-01, &
      9.975239E+01, 2.40357E+01, 1.8983E-02, 3.1359E-01, &
      1.066353E+02, 2.40320E+01, 1.6494E-02, 2.7984E-01, &
      1.139931E+02, 2.40286E+01, 1.4332E-02, 2.5180E-01, &
      1.218587E+02, 2.40256E+01, 1.2453E-02, 2.2840E-01, &
      1.302669E+02, 2.40229E+01, 1.0821E-02, 2.0879E-01, &
      1.392553E+02, 2.40204E+01, 9.4030E-03, 1.9229E-01, &
      1.488640E+02, 2.40182E+01, 8.1708E-03, 1.7831E-01, &
      1.591356E+02, 2.40162E+01, 7.1002E-03, 1.6640E-01, &
      1.701159E+02, 2.40145E+01, 6.1699E-03, 1.5618E-01, &
      1.818539E+02, 2.40129E+01, 5.3615E-03, 1.4734E-01, &
      1.944018E+02, 2.40115E+01, 4.6591E-03, 1.3964E-01, &
      2.078156E+02, 2.40102E+01, 4.0488E-03, 1.3286E-01, &
      2.221548E+02, 2.40091E+01, 3.5185E-03, 1.2684E-01, &
      2.374835E+02, 2.40081E+01, 3.0576E-03, 1.2145E-01, &
      2.538699E+02, 2.40072E+01, 2.6571E-03, 1.1657E-01, &
      2.713869E+02, 2.40064E+01, 2.3091E-03, 1.1212E-01, &
      2.901126E+02, 2.40057E+01, 2.0067E-03, 1.0802E-01, &
      3.101304E+02, 2.40050E+01, 1.7439E-03, 1.0421E-01, &
      3.315294E+02, 2.40045E+01, 1.5156E-03, 1.0065E-01, &
      3.544049E+02, 2.40040E+01, 1.3171E-03, 9.7300E-02, &
      3.788588E+02, 2.40035E+01, 1.1447E-03, 9.4121E-02, &
      4.050001E+02, 2.40031E+01, 9.9480E-04, 9.1093E-02, &
      4.329451E+02, 2.40028E+01, 8.6456E-04, 8.8194E-02]
   case(25)
      frel = -6.1200E-02
      allocate(nist_tab(796))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      7.000906E-03, 4.87567E+00, 0.0000E+00, 1.8048E-05, &
      7.108063E-03, 4.76264E+00, 0.0000E+00, 1.9528E-05, &
      7.116359E-03, 4.74787E+00, 0.0000E+00, 1.9646E-05, &
      7.136638E-03, 4.69094E+00, 0.0000E+00, 1.9938E-05, &
      7.179501E-03, 4.68054E+00, 8.6582E-02, 9.2372E+03, &
      7.225283E-03, 4.60492E+00, 8.7712E-02, 9.2985E+03, &
      7.254329E-03, 4.43167E+00, 8.8433E-02, 9.3374E+03, &
      7.286658E-03, 4.56450E+00, 4.3229E-01, 4.5441E+04, &
      7.297899E-03, 4.60423E+00, 4.3365E-01, 4.5514E+04, &
      7.406823E-03, 4.74833E+00, 4.4694E-01, 4.6220E+04, &
      1.069000E-02, 4.87379E+00, 8.9599E-01, 6.4199E+04, &
      1.142761E-02, 4.86446E+00, 1.0032E+00, 6.7239E+04, &
      1.221612E-02, 4.85674E+00, 1.1196E+00, 7.0201E+04, &
      1.305903E-02, 4.85198E+00, 1.2460E+00, 7.3081E+04, &
      1.396010E-02, 4.85154E+00, 1.3827E+00, 7.5867E+04, &
      1.492335E-02, 4.85683E+00, 1.5303E+00, 7.8543E+04, &
      1.595306E-02, 4.86935E+00, 1.6889E+00, 8.1089E+04, &
      1.705382E-02, 4.89071E+00, 1.8586E+00, 8.3479E+04, &
      1.823053E-02, 4.92262E+00, 2.0393E+00, 8.5683E+04, &
      1.948844E-02, 4.96681E+00, 2.2306E+00, 8.7669E+04, &
      2.083314E-02, 5.02507E+00, 2.4316E+00, 8.9401E+04, &
      2.227063E-02, 5.09930E+00, 2.6413E+00, 9.0842E+04, &
      2.380730E-02, 5.19091E+00, 2.8582E+00, 9.1957E+04, &
      2.545001E-02, 5.30149E+00, 3.0804E+00, 9.2709E+04, &
      2.720606E-02, 5.43236E+00, 3.3057E+00, 9.3068E+04, &
      2.908327E-02, 5.58491E+00, 3.5314E+00, 9.3005E+04, &
      3.109002E-02, 5.75947E+00, 3.7511E+00, 9.2415E+04, &
      3.323523E-02, 5.95150E+00, 3.9586E+00, 9.1232E+04, &
      3.552846E-02, 6.15427E+00, 4.1504E+00, 8.9479E+04, &
      3.797993E-02, 6.35760E+00, 4.3234E+00, 8.7191E+04, &
      4.060054E-02, 6.54294E+00, 4.4745E+00, 8.4415E+04, &
      4.340198E-02, 6.66665E+00, 4.6014E+00, 8.1205E+04, &
      4.639671E-02, 6.55405E+00, 4.7022E+00, 7.7628E+04, &
      4.762800E-02, 6.20116E+00, 4.7343E+00, 7.6138E+04, &
      4.835700E-02, 5.35353E+00, 4.7510E+00, 7.5254E+04, &
      4.855140E-02, 4.21071E+00, 4.7551E+00, 7.5018E+04, &
      4.884300E-02, 5.54427E+00, 7.0772E+00, 1.1099E+05, &
      4.959809E-02, 6.78893E+00, 6.9247E+00, 1.0694E+05, &
      5.302035E-02, 8.26842E+00, 6.3735E+00, 9.2074E+04, &
      5.667876E-02, 8.82114E+00, 6.0061E+00, 8.1167E+04, &
      6.058959E-02, 9.17629E+00, 5.7945E+00, 7.3253E+04, &
      6.477028E-02, 9.46762E+00, 5.6564E+00, 6.6892E+04, &
      6.923942E-02, 9.72427E+00, 5.5542E+00, 6.1443E+04, &
      7.401695E-02, 9.94746E+00, 5.4705E+00, 5.6611E+04, &
      7.912411E-02, 1.01123E+01, 5.3977E+00, 5.2253E+04, &
      8.222200E-02, 1.01176E+01, 5.3596E+00, 4.9929E+04, &
      8.348050E-02, 9.99109E+00, 5.3453E+00, 4.9044E+04, &
      8.381610E-02, 9.79639E+00, 5.3415E+00, 4.8814E+04, &
      8.431950E-02, 1.00228E+01, 5.7433E+00, 5.2172E+04, &
      8.458368E-02, 1.00962E+01, 5.7418E+00, 5.1996E+04, &
      8.557800E-02, 1.02519E+01, 5.7364E+00, 5.1343E+04, &
      9.041995E-02, 1.06219E+01, 5.7149E+00, 4.8412E+04, &
      9.665893E-02, 1.09524E+01, 5.6970E+00, 4.5145E+04, &
      1.033284E-01, 1.12573E+01, 5.6848E+00, 4.2140E+04, &
      1.104581E-01, 1.15559E+01, 5.6777E+00, 3.9371E+04, &
      1.180797E-01, 1.18579E+01, 5.6747E+00, 3.6811E+04, &
      1.262272E-01, 1.21705E+01, 5.6740E+00, 3.4431E+04, &
      1.349368E-01, 1.25017E+01, 5.6709E+00, 3.2192E+04, &
      1.442475E-01, 1.28362E+01, 5.5608E+00, 2.9529E+04, &
      1.542005E-01, 1.31400E+01, 5.4373E+00, 2.7010E+04, &
      1.648404E-01, 1.34258E+01, 5.3132E+00, 2.4690E+04, &
      1.762144E-01, 1.36967E+01, 5.1855E+00, 2.2541E+04, &
      1.883732E-01, 1.39546E+01, 5.0514E+00, 2.0541E+04, &
      2.013709E-01, 1.41998E+01, 4.9083E+00, 1.8671E+04, &
      2.152655E-01, 1.44295E+01, 4.7517E+00, 1.6909E+04, &
      2.301188E-01, 1.46387E+01, 4.5816E+00, 1.5251E+04, &
      2.459970E-01, 1.48229E+01, 4.3992E+00, 1.3699E+04, &
      2.629708E-01, 1.49795E+01, 4.2068E+00, 1.2254E+04, &
      2.811158E-01, 1.51015E+01, 4.0069E+00, 1.0920E+04, &
      3.005128E-01, 1.51866E+01, 3.8023E+00, 9.6931E+03, &
      3.212482E-01, 1.52308E+01, 3.5956E+00, 8.5750E+03, &
      3.434143E-01, 1.52292E+01, 3.3896E+00, 7.5622E+03, &
      3.671099E-01, 1.51754E+01, 3.1862E+00, 6.6499E+03, &
      3.924405E-01, 1.50600E+01, 2.9874E+00, 5.8329E+03, &
      4.195189E-01, 1.48692E+01, 2.7947E+00, 5.1051E+03, &
      4.484657E-01, 1.45809E+01, 2.6096E+00, 4.4596E+03, &
      4.794098E-01, 1.41575E+01, 2.4328E+00, 3.8896E+03, &
      5.124891E-01, 1.35289E+01, 2.2649E+00, 3.3879E+03, &
      5.478508E-01, 1.25443E+01, 2.1063E+00, 2.9477E+03, &
      5.856525E-01, 1.07856E+01, 1.9569E+00, 2.5624E+03, &
      6.260625E-01, 5.92995E+00, 1.8169E+00, 2.2261E+03, &
      6.274940E-01, 5.54629E+00, 1.8123E+00, 2.2154E+03, &
      6.370985E-01, 7.07354E-01, 1.7817E+00, 2.1453E+03, &
      6.383720E-01, -9.76022E-01, 1.7778E+00, 2.1363E+03, &
      6.396597E-01, -4.53373E+00, 1.7738E+00, 2.1272E+03, &
      6.435015E-01, 7.75254E-02, 1.1535E+01, 1.3733E+04, &
      6.481430E-01, 1.67813E+00, 1.1416E+01, 1.3495E+04, &
      6.507486E-01, 1.72464E-01, 1.1351E+01, 1.3363E+04, &
      6.531060E-01, 2.39202E+00, 1.6147E+01, 1.8940E+04, &
      6.546570E-01, 3.80093E+00, 1.6092E+01, 1.8831E+04, &
      6.644280E-01, 7.80621E+00, 1.5750E+01, 1.8160E+04, &
      6.692609E-01, 8.95254E+00, 1.5586E+01, 1.7841E+04, &
      7.154399E-01, 1.42272E+01, 1.4152E+01, 1.5154E+04, &
      7.536200E-01, 1.57962E+01, 1.3130E+01, 1.3348E+04, &
      7.648052E-01, 1.55761E+01, 1.2855E+01, 1.2877E+04, &
      7.651550E-01, 1.55425E+01, 1.2846E+01, 1.2863E+04, &
      7.682310E-01, 1.47738E+01, 1.2772E+01, 1.2738E+04, &
      7.728450E-01, 1.58637E+01, 1.4414E+01, 1.4290E+04, &
      7.843800E-01, 1.70753E+01, 1.4137E+01, 1.3809E+04, &
      8.175768E-01, 1.87594E+01, 1.3388E+01, 1.2546E+04, &
      8.739896E-01, 2.04727E+01, 1.2261E+01, 1.0750E+04, &
      9.342948E-01, 2.16815E+01, 1.1217E+01, 9.1997E+03, &
      9.987612E-01, 2.25986E+01, 1.0249E+01, 7.8639E+03, &
      1.067676E+00, 2.33123E+01, 9.3203E+00, 6.6903E+03, &
      1.141345E+00, 2.38555E+01, 8.4254E+00, 5.6582E+03, &
      1.220098E+00, 2.42393E+01, 7.5974E+00, 4.7734E+03, &
      1.304285E+00, 2.45176E+01, 6.8499E+00, 4.0266E+03, &
      1.394281E+00, 2.47113E+01, 6.1613E+00, 3.3886E+03, &
      1.490486E+00, 2.48369E+01, 5.5413E+00, 2.8516E+03, &
      1.593329E+00, 2.49125E+01, 4.9834E+00, 2.3996E+03, &
      1.703269E+00, 2.49499E+01, 4.4808E+00, 2.0189E+03, &
      1.820795E+00, 2.49583E+01, 4.0297E+00, 1.6990E+03, &
      1.946430E+00, 2.49447E+01, 3.6251E+00, 1.4304E+03, &
      2.080733E+00, 2.49156E+01, 3.2624E+00, 1.2047E+03, &
      2.224304E+00, 2.48752E+01, 2.9374E+00, 1.0151E+03, &
      2.377781E+00, 2.48294E+01, 2.6463E+00, 8.5600E+02, &
      2.541848E+00, 2.47785E+01, 2.3855E+00, 7.2231E+02, &
      2.717235E+00, 2.47246E+01, 2.1404E+00, 6.0673E+02, &
      2.904724E+00, 2.46691E+01, 1.9200E+00, 5.0956E+02, &
      3.105150E+00, 2.46120E+01, 1.7102E+00, 4.2502E+02, &
      3.319406E+00, 2.45255E+01, 1.5080E+00, 3.5103E+02, &
      3.548445E+00, 2.44274E+01, 1.3310E+00, 2.9025E+02, &
      3.793288E+00, 2.43202E+01, 1.1760E+00, 2.4029E+02, &
      4.055024E+00, 2.42039E+01, 1.0400E+00, 1.9916E+02, &
      4.334821E+00, 2.40759E+01, 9.2070E-01, 1.6528E+02, &
      4.633924E+00, 2.39312E+01, 8.1589E-01, 1.3733E+02, &
      4.953664E+00, 2.37601E+01, 7.2375E-01, 1.1426E+02, &
      5.295467E+00, 2.35437E+01, 6.4266E-01, 9.5190E+01, &
      5.660855E+00, 2.32388E+01, 5.7123E-01, 7.9409E+01, &
      6.051453E+00, 2.27115E+01, 5.0825E-01, 6.6334E+01, &
      6.408220E+00, 2.14354E+01, 4.6011E-01, 5.6903E+01, &
      6.469004E+00, 2.07891E+01, 4.5266E-01, 5.5488E+01, &
      6.506305E+00, 1.99795E+01, 4.4818E-01, 5.4645E+01, &
      6.532461E+00, 1.82218E+01, 4.4509E-01, 5.4064E+01, &
      6.571695E+00, 2.00165E+01, 3.9072E+00, 4.5727E+02, &
      6.669780E+00, 2.16410E+01, 3.7970E+00, 4.3788E+02, &
      6.915365E+00, 2.29017E+01, 3.5470E+00, 3.9466E+02, &
      7.392525E+00, 2.38502E+01, 3.1829E+00, 3.3147E+02, &
      7.902609E+00, 2.43589E+01, 2.8708E+00, 2.7983E+02, &
      8.447890E+00, 2.46957E+01, 2.5884E+00, 2.3617E+02, &
      9.030794E+00, 2.49352E+01, 2.3192E+00, 1.9808E+02, &
      9.653919E+00, 2.50944E+01, 2.0682E+00, 1.6538E+02, &
      1.032004E+01, 2.52022E+01, 1.8446E+00, 1.3811E+02, &
      1.103212E+01, 2.52757E+01, 1.6453E+00, 1.1536E+02, &
      1.179334E+01, 2.53252E+01, 1.4677E+00, 9.6376E+01, &
      1.260708E+01, 2.53579E+01, 1.3092E+00, 8.0521E+01, &
      1.347697E+01, 2.54183E+01, 1.1631E+00, 6.7017E+01, &
      1.440688E+01, 2.54240E+01, 1.0284E+00, 5.5521E+01, &
      1.540095E+01, 2.54196E+01, 9.0870E-01, 4.5981E+01, &
      1.646362E+01, 2.54086E+01, 8.0299E-01, 3.8090E+01, &
      1.759961E+01, 2.53932E+01, 7.0962E-01, 3.1563E+01, &
      1.881398E+01, 2.53751E+01, 6.2716E-01, 2.6164E+01, &
      2.011215E+01, 2.53554E+01, 5.5431E-01, 2.1698E+01, &
      2.149988E+01, 2.53349E+01, 4.8995E-01, 1.8001E+01, &
      2.298338E+01, 2.53143E+01, 4.3309E-01, 1.4941E+01, &
      2.456923E+01, 2.52941E+01, 3.8284E-01, 1.2408E+01, &
      2.626450E+01, 2.52746E+01, 3.3845E-01, 1.0311E+01, &
      2.807676E+01, 2.52565E+01, 2.9781E-01, 8.5359E+00, &
      3.001405E+01, 2.52383E+01, 2.6044E-01, 7.0311E+00, &
      3.208502E+01, 2.52200E+01, 2.2761E-01, 5.7938E+00, &
      3.429889E+01, 2.52021E+01, 1.9892E-01, 4.7800E+00, &
      3.666551E+01, 2.51851E+01, 1.7386E-01, 3.9494E+00, &
      3.919543E+01, 2.51691E+01, 1.5195E-01, 3.2683E+00, &
      4.189992E+01, 2.51541E+01, 1.3281E-01, 2.7097E+00, &
      4.479101E+01, 2.51401E+01, 1.1608E-01, 2.2516E+00, &
      4.788159E+01, 2.51272E+01, 1.0146E-01, 1.8756E+00, &
      5.118542E+01, 2.51153E+01, 8.8683E-02, 1.5670E+00, &
      5.471721E+01, 2.51044E+01, 7.7519E-02, 1.3136E+00, &
      5.849270E+01, 2.50944E+01, 6.7762E-02, 1.1052E+00, &
      6.252870E+01, 2.50853E+01, 5.9235E-02, 9.3385E-01, &
      6.684318E+01, 2.50770E+01, 5.1782E-02, 7.9285E-01, &
      7.145536E+01, 2.50695E+01, 4.5268E-02, 6.7672E-01, &
      7.638578E+01, 2.50626E+01, 3.9574E-02, 5.8096E-01, &
      8.165640E+01, 2.50565E+01, 3.4540E-02, 5.0137E-01, &
      8.729069E+01, 2.50508E+01, 3.0033E-02, 4.3468E-01, &
      9.331374E+01, 2.50456E+01, 2.6114E-02, 3.7974E-01, &
      9.975239E+01, 2.50409E+01, 2.2708E-02, 3.3438E-01, &
      1.066353E+02, 2.50367E+01, 1.9746E-02, 2.9685E-01, &
      1.139931E+02, 2.50329E+01, 1.7170E-02, 2.6570E-01, &
      1.218587E+02, 2.50294E+01, 1.4931E-02, 2.3977E-01, &
      1.302669E+02, 2.50263E+01, 1.2984E-02, 2.1810E-01, &
      1.392553E+02, 2.50235E+01, 1.1291E-02, 1.9990E-01, &
      1.488640E+02, 2.50210E+01, 9.8190E-03, 1.8452E-01, &
      1.591356E+02, 2.50187E+01, 8.5389E-03, 1.7148E-01, &
      1.701159E+02, 2.50167E+01, 7.4258E-03, 1.6032E-01, &
      1.818539E+02, 2.50149E+01, 6.4579E-03, 1.5072E-01, &
      1.944018E+02, 2.50133E+01, 5.6162E-03, 1.4239E-01, &
      2.078156E+02, 2.50118E+01, 4.8843E-03, 1.3509E-01, &
      2.221548E+02, 2.50105E+01, 4.2478E-03, 1.2865E-01, &
      2.374835E+02, 2.50094E+01, 3.6942E-03, 1.2290E-01, &
      2.538699E+02, 2.50083E+01, 3.2129E-03, 1.1771E-01, &
      2.713869E+02, 2.50074E+01, 2.7943E-03, 1.1301E-01, &
      2.901126E+02, 2.50066E+01, 2.4302E-03, 1.0869E-01, &
      3.101304E+02, 2.50059E+01, 2.1136E-03, 1.0469E-01, &
      3.315294E+02, 2.50052E+01, 1.8383E-03, 1.0096E-01, &
      3.544049E+02, 2.50046E+01, 1.5988E-03, 9.7453E-02, &
      3.788588E+02, 2.50041E+01, 1.3905E-03, 9.4138E-02, &
      4.050001E+02, 2.50036E+01, 1.2094E-03, 9.0983E-02, &
      4.329451E+02, 2.50032E+01, 1.0519E-03, 8.7966E-02]
   case(26)
      frel = -6.7800E-02
      allocate(nist_tab(760))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 4.40382E+00, 1.4240E+00, 1.0037E+05, &
      1.142761E-02, 4.40661E+00, 1.5363E+00, 1.0130E+05, &
      1.221612E-02, 4.41481E+00, 1.6592E+00, 1.0234E+05, &
      1.305903E-02, 4.42757E+00, 1.7931E+00, 1.0346E+05, &
      1.396010E-02, 4.44647E+00, 1.9386E+00, 1.0464E+05, &
      1.492335E-02, 4.47360E+00, 2.0960E+00, 1.0583E+05, &
      1.595306E-02, 4.51048E+00, 2.2636E+00, 1.0691E+05, &
      1.705382E-02, 4.55792E+00, 2.4410E+00, 1.0785E+05, &
      1.823053E-02, 4.61691E+00, 2.6278E+00, 1.0861E+05, &
      1.948844E-02, 4.68850E+00, 2.8235E+00, 1.0917E+05, &
      2.083314E-02, 4.77370E+00, 3.0272E+00, 1.0949E+05, &
      2.227063E-02, 4.87349E+00, 3.2383E+00, 1.0956E+05, &
      2.380730E-02, 4.98874E+00, 3.4557E+00, 1.0937E+05, &
      2.545001E-02, 5.12026E+00, 3.6787E+00, 1.0891E+05, &
      2.720606E-02, 5.26878E+00, 3.9061E+00, 1.0818E+05, &
      2.908327E-02, 5.43505E+00, 4.1370E+00, 1.0718E+05, &
      3.109002E-02, 5.61986E+00, 4.3705E+00, 1.0592E+05, &
      3.323523E-02, 5.82820E+00, 4.5990E+00, 1.0427E+05, &
      3.552846E-02, 6.05409E+00, 4.8169E+00, 1.0216E+05, &
      3.797993E-02, 6.29071E+00, 5.0275E+00, 9.9741E+04, &
      4.060054E-02, 6.53388E+00, 5.2288E+00, 9.7039E+04, &
      4.340198E-02, 6.77456E+00, 5.4191E+00, 9.4080E+04, &
      4.639671E-02, 6.99243E+00, 5.5969E+00, 9.0896E+04, &
      4.959809E-02, 7.12657E+00, 5.7609E+00, 8.7520E+04, &
      5.292000E-02, 6.78937E+00, 5.9060E+00, 8.4092E+04, &
      5.302035E-02, 6.74584E+00, 5.9101E+00, 8.3990E+04, &
      5.373000E-02, 6.05634E+00, 5.9379E+00, 8.3272E+04, &
      5.394600E-02, 5.04941E+00, 5.9462E+00, 8.3054E+04, &
      5.427000E-02, 6.23005E+00, 8.0096E+00, 1.1121E+05, &
      5.508000E-02, 7.32766E+00, 7.9125E+00, 1.0824E+05, &
      5.667876E-02, 8.17043E+00, 7.7445E+00, 1.0296E+05, &
      6.058959E-02, 9.12860E+00, 7.4424E+00, 9.2554E+04, &
      6.477028E-02, 9.72729E+00, 7.3003E+00, 8.4927E+04, &
      6.923942E-02, 1.02742E+01, 7.1586E+00, 7.7903E+04, &
      7.401695E-02, 1.07404E+01, 6.8831E+00, 7.0070E+04, &
      7.912411E-02, 1.11204E+01, 6.6327E+00, 6.3163E+04, &
      8.458368E-02, 1.14192E+01, 6.3988E+00, 5.7003E+04, &
      9.041995E-02, 1.15747E+01, 6.1785E+00, 5.1487E+04, &
      9.104200E-02, 1.15657E+01, 6.1566E+00, 5.0954E+04, &
      9.243550E-02, 1.14430E+01, 6.1086E+00, 4.9795E+04, &
      9.280710E-02, 1.12427E+01, 6.0961E+00, 4.9494E+04, &
      9.336450E-02, 1.14812E+01, 6.5009E+00, 5.2465E+04, &
      9.475800E-02, 1.17260E+01, 6.4618E+00, 5.1383E+04, &
      9.665893E-02, 1.18998E+01, 6.4106E+00, 4.9973E+04, &
      1.033284E-01, 1.22954E+01, 6.2493E+00, 4.5571E+04, &
      1.104581E-01, 1.26117E+01, 6.1022E+00, 4.1626E+04, &
      1.180797E-01, 1.28952E+01, 5.9677E+00, 3.8081E+04, &
      1.262272E-01, 1.31595E+01, 5.8454E+00, 3.4893E+04, &
      1.349368E-01, 1.34126E+01, 5.7339E+00, 3.2019E+04, &
      1.442475E-01, 1.36595E+01, 5.6308E+00, 2.9414E+04, &
      1.542005E-01, 1.39037E+01, 5.5330E+00, 2.7038E+04, &
      1.648404E-01, 1.41496E+01, 5.4371E+00, 2.4854E+04, &
      1.762144E-01, 1.43931E+01, 5.3394E+00, 2.2832E+04, &
      1.883732E-01, 1.46359E+01, 5.2365E+00, 2.0947E+04, &
      2.013709E-01, 1.48762E+01, 5.1252E+00, 1.9179E+04, &
      2.152655E-01, 1.51118E+01, 5.0034E+00, 1.7514E+04, &
      2.301188E-01, 1.53398E+01, 4.8675E+00, 1.5939E+04, &
      2.459970E-01, 1.55539E+01, 4.7151E+00, 1.4443E+04, &
      2.629708E-01, 1.57483E+01, 4.5474E+00, 1.3032E+04, &
      2.811158E-01, 1.59181E+01, 4.3664E+00, 1.1706E+04, &
      3.005128E-01, 1.60582E+01, 4.1750E+00, 1.0470E+04, &
      3.212482E-01, 1.61644E+01, 3.9760E+00, 9.3276E+03, &
      3.434143E-01, 1.62326E+01, 3.7724E+00, 8.2791E+03, &
      3.671099E-01, 1.62582E+01, 3.5670E+00, 7.3235E+03, &
      3.924405E-01, 1.62355E+01, 3.3624E+00, 6.4583E+03, &
      4.195189E-01, 1.61566E+01, 3.1607E+00, 5.6793E+03, &
      4.484657E-01, 1.60101E+01, 2.9637E+00, 4.9821E+03, &
      4.794098E-01, 1.57788E+01, 2.7730E+00, 4.3611E+03, &
      5.124891E-01, 1.54340E+01, 2.5897E+00, 3.8105E+03, &
      5.478508E-01, 1.49253E+01, 2.4147E+00, 3.3242E+03, &
      5.856525E-01, 1.41528E+01, 2.2486E+00, 2.8962E+03, &
      6.260625E-01, 1.28786E+01, 2.0915E+00, 2.5206E+03, &
      6.692609E-01, 1.02710E+01, 1.9436E+00, 2.1917E+03, &
      6.939380E-01, 6.60633E+00, 1.8672E+00, 2.0310E+03, &
      7.045595E-01, 1.76776E+00, 1.8359E+00, 1.9670E+03, &
      7.066780E-01, -1.24454E+00, 1.8298E+00, 1.9546E+03, &
      7.073919E-01, -3.49355E+00, 1.8277E+00, 1.9504E+03, &
      7.116405E-01, 1.19707E+00, 1.1657E+01, 1.2347E+04, &
      7.154399E-01, 2.82538E+00, 1.1565E+01, 1.2184E+04, &
      7.174945E-01, 2.96615E+00, 1.1515E+01, 1.2097E+04, &
      7.203789E-01, 1.39388E+00, 1.1447E+01, 1.1977E+04, &
      7.222620E-01, 2.65657E+00, 1.6292E+01, 1.7000E+04, &
      7.247055E-01, 4.99594E+00, 1.6209E+01, 1.6857E+04, &
      7.355220E-01, 8.97159E+00, 1.5852E+01, 1.6244E+04, &
      7.648052E-01, 1.33092E+01, 1.4949E+01, 1.4732E+04, &
      8.175768E-01, 1.64542E+01, 1.3531E+01, 1.2474E+04, &
      8.291780E-01, 1.67052E+01, 1.3253E+01, 1.2047E+04, &
      8.418695E-01, 1.64524E+01, 1.2960E+01, 1.1603E+04, &
      8.452539E-01, 1.56692E+01, 1.2883E+01, 1.1489E+04, &
      8.503305E-01, 1.67842E+01, 1.4559E+01, 1.2905E+04, &
      8.630220E-01, 1.80245E+01, 1.4272E+01, 1.2465E+04, &
      8.739896E-01, 1.86609E+01, 1.4032E+01, 1.2102E+04, &
      9.342948E-01, 2.08209E+01, 1.2835E+01, 1.0355E+04, &
      9.987612E-01, 2.22293E+01, 1.1740E+01, 8.8607E+03, &
      1.067676E+00, 2.32714E+01, 1.0687E+01, 7.5460E+03, &
      1.141345E+00, 2.40488E+01, 9.7229E+00, 6.4230E+03, &
      1.220098E+00, 2.46633E+01, 8.8309E+00, 5.4579E+03, &
      1.304285E+00, 2.51128E+01, 7.9627E+00, 4.6043E+03, &
      1.394281E+00, 2.54331E+01, 7.1798E+00, 3.8843E+03, &
      1.490486E+00, 2.56643E+01, 6.4637E+00, 3.2718E+03, &
      1.593329E+00, 2.58170E+01, 5.8123E+00, 2.7529E+03, &
      1.703269E+00, 2.59124E+01, 5.2261E+00, 2.3161E+03, &
      1.820795E+00, 2.59647E+01, 4.6982E+00, 1.9483E+03, &
      1.946430E+00, 2.59840E+01, 4.2238E+00, 1.6392E+03, &
      2.080733E+00, 2.59787E+01, 3.7984E+00, 1.3795E+03, &
      2.224304E+00, 2.59575E+01, 3.4173E+00, 1.1615E+03, &
      2.377781E+00, 2.59217E+01, 3.0760E+00, 9.7860E+02, &
      2.541848E+00, 2.58778E+01, 2.7704E+00, 8.2500E+02, &
      2.717235E+00, 2.58293E+01, 2.4969E+00, 6.9604E+02, &
      2.904724E+00, 2.57812E+01, 2.2482E+00, 5.8674E+02, &
      3.105150E+00, 2.57281E+01, 2.0033E+00, 4.8954E+02, &
      3.319406E+00, 2.56539E+01, 1.7751E+00, 4.0626E+02, &
      3.548445E+00, 2.55677E+01, 1.5695E+00, 3.3646E+02, &
      3.793288E+00, 2.54704E+01, 1.3870E+00, 2.7857E+02, &
      4.055024E+00, 2.53646E+01, 1.2269E+00, 2.3092E+02, &
      4.334821E+00, 2.52504E+01, 1.0864E+00, 1.9164E+02, &
      4.633924E+00, 2.51258E+01, 9.6283E-01, 1.5924E+02, &
      4.953664E+00, 2.49864E+01, 8.5419E-01, 1.3248E+02, &
      5.295467E+00, 2.48241E+01, 7.5854E-01, 1.1035E+02, &
      5.660855E+00, 2.46232E+01, 6.7426E-01, 9.2046E+01, &
      6.051453E+00, 2.43495E+01, 5.9992E-01, 7.6873E+01, &
      6.469004E+00, 2.39082E+01, 5.3430E-01, 6.4287E+01, &
      6.915365E+00, 2.27890E+01, 4.7631E-01, 5.3834E+01, &
      6.969760E+00, 2.24649E+01, 4.6996E-01, 5.2729E+01, &
      7.076440E+00, 2.10168E+01, 4.5790E-01, 5.0652E+01, &
      7.104888E+00, 1.92682E+01, 4.5477E-01, 5.0118E+01, &
      7.147560E+00, 2.10548E+01, 3.8992E+00, 4.1293E+02, &
      7.254240E+00, 2.26714E+01, 3.7877E+00, 3.9527E+02, &
      7.392525E+00, 2.34808E+01, 3.6502E+00, 3.7387E+02, &
      7.902609E+00, 2.46830E+01, 3.2650E+00, 3.1302E+02, &
      8.447890E+00, 2.52655E+01, 2.9475E+00, 2.6449E+02, &
      9.030794E+00, 2.56451E+01, 2.6546E+00, 2.2299E+02, &
      9.653919E+00, 2.58973E+01, 2.3704E+00, 1.8641E+02, &
      1.032004E+01, 2.60660E+01, 2.1145E+00, 1.5570E+02, &
      1.103212E+01, 2.61815E+01, 1.8865E+00, 1.3007E+02, &
      1.179334E+01, 2.62607E+01, 1.6832E+00, 1.0868E+02, &
      1.260708E+01, 2.63142E+01, 1.5020E+00, 9.0830E+01, &
      1.347697E+01, 2.63498E+01, 1.3404E+00, 7.5925E+01, &
      1.440688E+01, 2.64121E+01, 1.1940E+00, 6.3364E+01, &
      1.540095E+01, 2.64211E+01, 1.0562E+00, 5.2527E+01, &
      1.646362E+01, 2.64188E+01, 9.3409E-01, 4.3541E+01, &
      1.759961E+01, 2.64094E+01, 8.2587E-01, 3.6092E+01, &
      1.881398E+01, 2.63951E+01, 7.3023E-01, 2.9926E+01, &
      2.011215E+01, 2.63778E+01, 6.4572E-01, 2.4823E+01, &
      2.149988E+01, 2.63587E+01, 5.7102E-01, 2.0599E+01, &
      2.298338E+01, 2.63387E+01, 5.0500E-01, 1.7102E+01, &
      2.456923E+01, 2.63183E+01, 4.4663E-01, 1.4205E+01, &
      2.626450E+01, 2.62983E+01, 3.9503E-01, 1.1806E+01, &
      2.807676E+01, 2.62809E+01, 3.4777E-01, 9.7736E+00, &
      3.001405E+01, 2.62618E+01, 3.0571E-01, 8.0858E+00, &
      3.208502E+01, 2.62430E+01, 2.6786E-01, 6.6748E+00, &
      3.429889E+01, 2.62243E+01, 2.3420E-01, 5.5048E+00, &
      3.666551E+01, 2.62062E+01, 2.0478E-01, 4.5458E+00, &
      3.919543E+01, 2.61889E+01, 1.7906E-01, 3.7595E+00, &
      4.189992E+01, 2.61725E+01, 1.5658E-01, 3.1145E+00, &
      4.479101E+01, 2.61573E+01, 1.3691E-01, 2.5851E+00, &
      4.788159E+01, 2.61431E+01, 1.1972E-01, 2.1506E+00, &
      5.118542E+01, 2.61300E+01, 1.0469E-01, 1.7938E+00, &
      5.471721E+01, 2.61179E+01, 9.1550E-02, 1.5008E+00, &
      5.849270E+01, 2.61068E+01, 8.0062E-02, 1.2599E+00, &
      6.252870E+01, 2.60966E+01, 7.0017E-02, 1.0618E+00, &
      6.684318E+01, 2.60873E+01, 6.1234E-02, 8.9883E-01, &
      7.145536E+01, 2.60789E+01, 5.3554E-02, 7.6457E-01, &
      7.638578E+01, 2.60712E+01, 4.6838E-02, 6.5389E-01, &
      8.165640E+01, 2.60643E+01, 4.0894E-02, 5.6188E-01, &
      8.729069E+01, 2.60579E+01, 3.5564E-02, 4.8476E-01, &
      9.331374E+01, 2.60521E+01, 3.0930E-02, 4.2128E-01, &
      9.975239E+01, 2.60468E+01, 2.6900E-02, 3.6894E-01, &
      1.066353E+02, 2.60420E+01, 2.3395E-02, 3.2567E-01, &
      1.139931E+02, 2.60376E+01, 2.0347E-02, 2.8983E-01, &
      1.218587E+02, 2.60337E+01, 1.7697E-02, 2.6003E-01, &
      1.302669E+02, 2.60302E+01, 1.5392E-02, 2.3518E-01, &
      1.392553E+02, 2.60270E+01, 1.3387E-02, 2.1436E-01, &
      1.488640E+02, 2.60241E+01, 1.1644E-02, 1.9685E-01, &
      1.591356E+02, 2.60215E+01, 1.0128E-02, 1.8202E-01, &
      1.701159E+02, 2.60192E+01, 8.8094E-03, 1.6941E-01, &
      1.818539E+02, 2.60171E+01, 7.6625E-03, 1.5860E-01, &
      1.944018E+02, 2.60153E+01, 6.6650E-03, 1.4925E-01, &
      2.078156E+02, 2.60136E+01, 5.7975E-03, 1.4112E-01, &
      2.221548E+02, 2.60121E+01, 5.0429E-03, 1.3397E-01, &
      2.374835E+02, 2.60108E+01, 4.3866E-03, 1.2765E-01, &
      2.538699E+02, 2.60096E+01, 3.8158E-03, 1.2198E-01, &
      2.713869E+02, 2.60086E+01, 3.3192E-03, 1.1687E-01, &
      2.901126E+02, 2.60076E+01, 2.8873E-03, 1.1220E-01, &
      3.101304E+02, 2.60068E+01, 2.5117E-03, 1.0791E-01, &
      3.315294E+02, 2.60060E+01, 2.1849E-03, 1.0394E-01, &
      3.544049E+02, 2.60053E+01, 1.9006E-03, 1.0022E-01, &
      3.788588E+02, 2.60047E+01, 1.6534E-03, 9.6729E-02, &
      4.050001E+02, 2.60042E+01, 1.4383E-03, 9.3420E-02, &
      4.329451E+02, 2.60037E+01, 1.2512E-03, 9.0270E-02]
   case(27)
      frel = -7.3800E-02
      allocate(nist_tab(760))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 4.07651E+00, 1.5562E+00, 1.0395E+05, &
      1.142761E-02, 4.07931E+00, 1.6781E+00, 1.0485E+05, &
      1.221612E-02, 4.08644E+00, 1.8115E+00, 1.0588E+05, &
      1.305903E-02, 4.09903E+00, 1.9562E+00, 1.0696E+05, &
      1.396010E-02, 4.11806E+00, 2.1124E+00, 1.0804E+05, &
      1.492335E-02, 4.14462E+00, 2.2802E+00, 1.0910E+05, &
      1.595306E-02, 4.17992E+00, 2.4597E+00, 1.1009E+05, &
      1.705382E-02, 4.22518E+00, 2.6509E+00, 1.1099E+05, &
      1.823053E-02, 4.28168E+00, 2.8536E+00, 1.1177E+05, &
      1.948844E-02, 4.35067E+00, 3.0675E+00, 1.1239E+05, &
      2.083314E-02, 4.43344E+00, 3.2922E+00, 1.1284E+05, &
      2.227063E-02, 4.53124E+00, 3.5274E+00, 1.1309E+05, &
      2.380730E-02, 4.64533E+00, 3.7723E+00, 1.1314E+05, &
      2.545001E-02, 4.77694E+00, 4.0266E+00, 1.1297E+05, &
      2.720606E-02, 4.92734E+00, 4.2894E+00, 1.1258E+05, &
      2.908327E-02, 5.09779E+00, 4.5603E+00, 1.1196E+05, &
      3.109002E-02, 5.28966E+00, 4.8385E+00, 1.1112E+05, &
      3.323523E-02, 5.50445E+00, 5.1236E+00, 1.1008E+05, &
      3.552846E-02, 5.74388E+00, 5.4151E+00, 1.0883E+05, &
      3.797993E-02, 6.01002E+00, 5.7127E+00, 1.0740E+05, &
      4.060054E-02, 6.30538E+00, 6.0162E+00, 1.0581E+05, &
      4.340198E-02, 6.67459E+00, 6.3044E+00, 1.0372E+05, &
      4.639671E-02, 7.06312E+00, 6.5733E+00, 1.0116E+05, &
      4.959809E-02, 7.46141E+00, 6.8347E+00, 9.8394E+04, &
      5.302035E-02, 7.85964E+00, 7.0873E+00, 9.5446E+04, &
      5.667876E-02, 8.12459E+00, 7.1365E+00, 8.9904E+04, &
      5.831000E-02, 7.95238E+00, 7.0944E+00, 8.6874E+04, &
      5.920250E-02, 7.32491E+00, 7.0696E+00, 8.5265E+04, &
      5.944050E-02, 6.41648E+00, 7.0627E+00, 8.4842E+04, &
      5.979750E-02, 7.50344E+00, 8.9241E+00, 1.0656E+05, &
      6.058959E-02, 8.46511E+00, 8.8069E+00, 1.0379E+05, &
      6.069000E-02, 8.53783E+00, 8.7924E+00, 1.0344E+05, &
      6.477028E-02, 9.99361E+00, 8.2619E+00, 9.1080E+04, &
      6.923942E-02, 1.07000E+01, 7.8176E+00, 8.0619E+04, &
      7.401695E-02, 1.11971E+01, 7.4801E+00, 7.2160E+04, &
      7.912411E-02, 1.16018E+01, 7.1971E+00, 6.4948E+04, &
      8.458368E-02, 1.19426E+01, 6.9439E+00, 5.8619E+04, &
      9.041995E-02, 1.22207E+01, 6.7099E+00, 5.2987E+04, &
      9.665893E-02, 1.23949E+01, 6.4907E+00, 4.7948E+04, &
      9.868600E-02, 1.23849E+01, 6.4253E+00, 4.6490E+04, &
      1.001965E-01, 1.22528E+01, 6.3784E+00, 4.5454E+04, &
      1.005993E-01, 1.20449E+01, 6.3661E+00, 4.5185E+04, &
      1.012035E-01, 1.22888E+01, 6.7844E+00, 4.7867E+04, &
      1.027140E-01, 1.25366E+01, 6.7462E+00, 4.6898E+04, &
      1.033284E-01, 1.25962E+01, 6.7311E+00, 4.6514E+04, &
      1.104581E-01, 1.30306E+01, 6.5708E+00, 4.2475E+04, &
      1.180797E-01, 1.33567E+01, 6.4252E+00, 3.8853E+04, &
      1.262272E-01, 1.36488E+01, 6.2918E+00, 3.5591E+04, &
      1.349368E-01, 1.39231E+01, 6.1699E+00, 3.2650E+04, &
      1.442475E-01, 1.41876E+01, 6.0579E+00, 2.9988E+04, &
      1.542005E-01, 1.44472E+01, 5.9535E+00, 2.7569E+04, &
      1.648404E-01, 1.47051E+01, 5.8536E+00, 2.5357E+04, &
      1.762144E-01, 1.49632E+01, 5.7547E+00, 2.3319E+04, &
      1.883732E-01, 1.52277E+01, 5.6531E+00, 2.1429E+04, &
      2.013709E-01, 1.54860E+01, 5.5455E+00, 1.9665E+04, &
      2.152655E-01, 1.57421E+01, 5.4288E+00, 1.8008E+04, &
      2.301188E-01, 1.59934E+01, 5.3009E+00, 1.6449E+04, &
      2.459970E-01, 1.62372E+01, 5.1597E+00, 1.4978E+04, &
      2.629708E-01, 1.64686E+01, 5.0016E+00, 1.3583E+04, &
      2.811158E-01, 1.66810E+01, 4.8271E+00, 1.2263E+04, &
      3.005128E-01, 1.68691E+01, 4.6386E+00, 1.1024E+04, &
      3.212482E-01, 1.70277E+01, 4.4388E+00, 9.8679E+03, &
      3.434143E-01, 1.71536E+01, 4.2307E+00, 8.7987E+03, &
      3.671099E-01, 1.72425E+01, 4.0175E+00, 7.8163E+03, &
      3.924405E-01, 1.72900E+01, 3.8022E+00, 6.9203E+03, &
      4.195189E-01, 1.72911E+01, 3.5873E+00, 6.1084E+03, &
      4.484657E-01, 1.72394E+01, 3.3752E+00, 5.3766E+03, &
      4.794098E-01, 1.71251E+01, 3.1678E+00, 4.7210E+03, &
      5.124891E-01, 1.69341E+01, 2.9667E+00, 4.1364E+03, &
      5.478508E-01, 1.66433E+01, 2.7733E+00, 3.6177E+03, &
      5.856525E-01, 1.62138E+01, 2.5884E+00, 3.1591E+03, &
      6.260625E-01, 1.55717E+01, 2.4125E+00, 2.7550E+03, &
      6.692609E-01, 1.45571E+01, 2.2461E+00, 2.4000E+03, &
      7.154399E-01, 1.27139E+01, 2.0893E+00, 2.0889E+03, &
      7.630280E-01, 7.65825E+00, 1.9469E+00, 1.8257E+03, &
      7.648052E-01, 7.22164E+00, 1.9419E+00, 1.8168E+03, &
      7.747070E-01, 2.81236E+00, 1.9146E+00, 1.7684E+03, &
      7.777280E-01, -2.10962E+00, 1.9064E+00, 1.7542E+03, &
      7.778214E-01, -2.47515E+00, 1.9061E+00, 1.7537E+03, &
      7.824930E-01, 2.28899E+00, 1.1809E+01, 1.0780E+04, &
      7.896320E-01, 4.20240E+00, 1.1644E+01, 1.0533E+04, &
      7.928064E-01, 2.56916E+00, 1.1572E+01, 1.0426E+04, &
      7.941720E-01, 2.39355E+00, 1.6482E+01, 1.4822E+04, &
      7.975680E-01, 6.16476E+00, 1.6372E+01, 1.4661E+04, &
      8.094720E-01, 1.01298E+01, 1.5995E+01, 1.4114E+04, &
      8.175768E-01, 1.16331E+01, 1.5747E+01, 1.3757E+04, &
      8.739896E-01, 1.65757E+01, 1.4185E+01, 1.1593E+04, &
      9.070880E-01, 1.76437E+01, 1.3401E+01, 1.0553E+04, &
      9.209720E-01, 1.74222E+01, 1.3097E+01, 1.0158E+04, &
      9.246744E-01, 1.66663E+01, 1.3018E+01, 1.0057E+04, &
      9.302280E-01, 1.77529E+01, 1.4641E+01, 1.1242E+04, &
      9.342948E-01, 1.82397E+01, 1.4556E+01, 1.1129E+04, &
      9.441120E-01, 1.89734E+01, 1.4357E+01, 1.0862E+04, &
      9.987612E-01, 2.11535E+01, 1.3340E+01, 9.5409E+03, &
      1.067676E+00, 2.27846E+01, 1.2182E+01, 8.1516E+03, &
      1.141345E+00, 2.39382E+01, 1.1109E+01, 6.9542E+03, &
      1.220098E+00, 2.48005E+01, 1.0116E+01, 5.9242E+03, &
      1.304285E+00, 2.54732E+01, 9.2078E+00, 5.0452E+03, &
      1.394281E+00, 2.59977E+01, 8.3151E+00, 4.2627E+03, &
      1.490486E+00, 2.63628E+01, 7.4892E+00, 3.5922E+03, &
      1.593329E+00, 2.66227E+01, 6.7415E+00, 3.0254E+03, &
      1.703269E+00, 2.67981E+01, 6.0602E+00, 2.5448E+03, &
      1.820795E+00, 2.69098E+01, 5.4480E+00, 2.1408E+03, &
      1.946430E+00, 2.69744E+01, 4.8978E+00, 1.8009E+03, &
      2.080733E+00, 2.70031E+01, 4.4033E+00, 1.5152E+03, &
      2.224304E+00, 2.70049E+01, 3.9603E+00, 1.2754E+03, &
      2.377781E+00, 2.69898E+01, 3.5635E+00, 1.0741E+03, &
      2.541848E+00, 2.69602E+01, 3.2082E+00, 9.0510E+02, &
      2.717235E+00, 2.69203E+01, 2.8901E+00, 7.6326E+02, &
      2.904724E+00, 2.68771E+01, 2.6054E+00, 6.4414E+02, &
      3.105150E+00, 2.68376E+01, 2.3362E+00, 5.4076E+02, &
      3.319406E+00, 2.67746E+01, 2.0714E+00, 4.4901E+02, &
      3.548445E+00, 2.66954E+01, 1.8370E+00, 3.7297E+02, &
      3.793288E+00, 2.66078E+01, 1.6283E+00, 3.0970E+02, &
      4.055024E+00, 2.65107E+01, 1.4403E+00, 2.5667E+02, &
      4.334821E+00, 2.64054E+01, 1.2751E+00, 2.1297E+02, &
      4.633924E+00, 2.62922E+01, 1.1300E+00, 1.7691E+02, &
      4.953664E+00, 2.61697E+01, 1.0023E+00, 1.4714E+02, &
      5.295467E+00, 2.60338E+01, 8.8996E-01, 1.2253E+02, &
      5.660855E+00, 2.58774E+01, 7.9093E-01, 1.0216E+02, &
      6.051453E+00, 2.56869E+01, 7.0360E-01, 8.5288E+01, &
      6.469004E+00, 2.54337E+01, 6.2651E-01, 7.1294E+01, &
      6.915365E+00, 2.50440E+01, 5.5839E-01, 5.9676E+01, &
      7.392525E+00, 2.41983E+01, 4.9816E-01, 5.0016E+01, &
      7.554722E+00, 2.34922E+01, 4.8010E-01, 4.7238E+01, &
      7.670355E+00, 2.20515E+01, 4.6787E-01, 4.5390E+01, &
      7.701191E+00, 2.03118E+01, 4.6470E-01, 4.4915E+01, &
      7.747444E+00, 2.20899E+01, 3.8913E+00, 3.6046E+02, &
      7.863078E+00, 2.36980E+01, 3.7792E+00, 3.4498E+02, &
      7.902609E+00, 2.39681E+01, 3.7420E+00, 3.3989E+02, &
      8.447890E+00, 2.55247E+01, 3.3364E+00, 2.8367E+02, &
      9.030794E+00, 2.61889E+01, 3.0147E+00, 2.3994E+02, &
      9.653919E+00, 2.65956E+01, 2.7028E+00, 2.0136E+02, &
      1.032004E+01, 2.68641E+01, 2.4149E+00, 1.6845E+02, &
      1.103212E+01, 2.70436E+01, 2.1541E+00, 1.4070E+02, &
      1.179334E+01, 2.71663E+01, 1.9216E+00, 1.1754E+02, &
      1.260708E+01, 2.72503E+01, 1.7145E+00, 9.8209E+01, &
      1.347697E+01, 2.73072E+01, 1.5298E+00, 8.2079E+01, &
      1.440688E+01, 2.73449E+01, 1.3652E+00, 6.8614E+01, &
      1.540095E+01, 2.73695E+01, 1.2182E+00, 5.7363E+01, &
      1.646362E+01, 2.74194E+01, 1.0783E+00, 4.7589E+01, &
      1.759961E+01, 2.74188E+01, 9.5438E-01, 3.9482E+01, &
      1.881398E+01, 2.74105E+01, 8.4426E-01, 3.2748E+01, &
      2.011215E+01, 2.73973E+01, 7.4690E-01, 2.7173E+01, &
      2.149988E+01, 2.73808E+01, 6.6081E-01, 2.2555E+01, &
      2.298338E+01, 2.73623E+01, 5.8469E-01, 1.8730E+01, &
      2.456923E+01, 2.73430E+01, 5.1737E-01, 1.5561E+01, &
      2.626450E+01, 2.73241E+01, 4.5783E-01, 1.2935E+01, &
      2.807676E+01, 2.73053E+01, 4.0326E-01, 1.0709E+01, &
      3.001405E+01, 2.72853E+01, 3.5466E-01, 8.8608E+00, &
      3.208502E+01, 2.72654E+01, 3.1194E-01, 7.3372E+00, &
      3.429889E+01, 2.72463E+01, 2.7377E-01, 6.0688E+00, &
      3.666551E+01, 2.72274E+01, 2.3948E-01, 5.0097E+00, &
      3.919543E+01, 2.72091E+01, 2.0950E-01, 4.1410E+00, &
      4.189992E+01, 2.71916E+01, 1.8327E-01, 3.4282E+00, &
      4.479101E+01, 2.71751E+01, 1.6034E-01, 2.8433E+00, &
      4.788159E+01, 2.71597E+01, 1.4027E-01, 2.3629E+00, &
      5.118542E+01, 2.71453E+01, 1.2271E-01, 1.9684E+00, &
      5.471721E+01, 2.71320E+01, 1.0735E-01, 1.6442E+00, &
      5.849270E+01, 2.71198E+01, 9.3920E-02, 1.3778E+00, &
      6.252870E+01, 2.71086E+01, 8.2171E-02, 1.1586E+00, &
      6.684318E+01, 2.70983E+01, 7.1894E-02, 9.7822E-01, &
      7.145536E+01, 2.70889E+01, 6.2904E-02, 8.2966E-01, &
      7.638578E+01, 2.70803E+01, 5.5040E-02, 7.0722E-01, &
      8.165640E+01, 2.70725E+01, 4.8079E-02, 6.0547E-01, &
      8.729069E+01, 2.70655E+01, 4.1842E-02, 5.2026E-01, &
      9.331374E+01, 2.70590E+01, 3.6414E-02, 4.5014E-01, &
      9.975239E+01, 2.70530E+01, 3.1691E-02, 3.9234E-01, &
      1.066353E+02, 2.70476E+01, 2.7581E-02, 3.4458E-01, &
      1.139931E+02, 2.70427E+01, 2.4005E-02, 3.0505E-01, &
      1.218587E+02, 2.70383E+01, 2.0892E-02, 2.7223E-01, &
      1.302669E+02, 2.70343E+01, 1.8184E-02, 2.4489E-01, &
      1.392553E+02, 2.70307E+01, 1.5826E-02, 2.2204E-01, &
      1.488640E+02, 2.70275E+01, 1.3775E-02, 2.0284E-01, &
      1.591356E+02, 2.70246E+01, 1.1990E-02, 1.8666E-01, &
      1.701159E+02, 2.70219E+01, 1.0436E-02, 1.7291E-01, &
      1.818539E+02, 2.70196E+01, 9.0837E-03, 1.6118E-01, &
      1.944018E+02, 2.70175E+01, 7.9067E-03, 1.5108E-01, &
      2.078156E+02, 2.70156E+01, 6.8823E-03, 1.4233E-01, &
      2.221548E+02, 2.70139E+01, 5.9908E-03, 1.3468E-01, &
      2.374835E+02, 2.70124E+01, 5.2147E-03, 1.2795E-01, &
      2.538699E+02, 2.70110E+01, 4.5393E-03, 1.2196E-01, &
      2.713869E+02, 2.70098E+01, 3.9514E-03, 1.1658E-01, &
      2.901126E+02, 2.70087E+01, 3.4396E-03, 1.1170E-01, &
      3.101304E+02, 2.70078E+01, 2.9942E-03, 1.0724E-01, &
      3.315294E+02, 2.70069E+01, 2.6065E-03, 1.0314E-01, &
      3.544049E+02, 2.70061E+01, 2.2690E-03, 9.9325E-02, &
      3.788588E+02, 2.70055E+01, 1.9752E-03, 9.5756E-02, &
      4.050001E+02, 2.70049E+01, 1.7195E-03, 9.2392E-02, &
      4.329451E+02, 2.70043E+01, 1.4969E-03, 8.9206E-02]
   case(28)
      frel = -8.1000E-02
      allocate(nist_tab(756))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 3.57659E+00, 1.4343E+00, 9.6199E+04, &
      1.142761E-02, 3.57599E+00, 1.5448E+00, 9.6924E+04, &
      1.221612E-02, 3.57095E+00, 1.6670E+00, 9.7839E+04, &
      1.305903E-02, 3.56836E+00, 1.8017E+00, 9.8922E+04, &
      1.396010E-02, 3.56968E+00, 1.9500E+00, 1.0015E+05, &
      1.492335E-02, 3.57685E+00, 2.1125E+00, 1.0149E+05, &
      1.595306E-02, 3.59151E+00, 2.2884E+00, 1.0285E+05, &
      1.705382E-02, 3.61476E+00, 2.4778E+00, 1.0418E+05, &
      1.823053E-02, 3.64791E+00, 2.6808E+00, 1.0544E+05, &
      1.948844E-02, 3.69236E+00, 2.8974E+00, 1.0660E+05, &
      2.083314E-02, 3.74958E+00, 3.1272E+00, 1.0762E+05, &
      2.227063E-02, 3.82103E+00, 3.3699E+00, 1.0849E+05, &
      2.380730E-02, 3.90818E+00, 3.6250E+00, 1.0917E+05, &
      2.545001E-02, 4.01245E+00, 3.8920E+00, 1.0965E+05, &
      2.720606E-02, 4.13521E+00, 4.1699E+00, 1.0990E+05, &
      2.908327E-02, 4.27778E+00, 4.4581E+00, 1.0991E+05, &
      3.109002E-02, 4.44144E+00, 4.7557E+00, 1.0967E+05, &
      3.323523E-02, 4.62739E+00, 5.0616E+00, 1.0920E+05, &
      3.552846E-02, 4.83686E+00, 5.3751E+00, 1.0847E+05, &
      3.797993E-02, 5.07105E+00, 5.6954E+00, 1.0752E+05, &
      4.060054E-02, 5.33124E+00, 6.0216E+00, 1.0634E+05, &
      4.340198E-02, 5.61884E+00, 6.3532E+00, 1.0495E+05, &
      4.639671E-02, 5.93534E+00, 6.6897E+00, 1.0338E+05, &
      4.959809E-02, 6.28228E+00, 7.0309E+00, 1.0164E+05, &
      5.302035E-02, 6.72188E+00, 7.3516E+00, 9.9415E+04, &
      5.667876E-02, 7.16643E+00, 7.6429E+00, 9.6684E+04, &
      6.058959E-02, 7.58919E+00, 7.9225E+00, 9.3751E+04, &
      6.477028E-02, 7.89984E+00, 8.1889E+00, 9.0650E+04, &
      6.673800E-02, 7.81083E+00, 8.3039E+00, 8.9212E+04, &
      6.775950E-02, 7.29248E+00, 8.3483E+00, 8.8337E+04, &
      6.803190E-02, 6.49266E+00, 8.3411E+00, 8.7907E+04, &
      6.844050E-02, 7.47679E+00, 1.0005E+01, 1.0481E+05, &
      6.923942E-02, 8.30426E+00, 9.9197E+00, 1.0272E+05, &
      6.946200E-02, 8.44458E+00, 9.8964E+00, 1.0215E+05, &
      7.401695E-02, 9.92898E+00, 9.4522E+00, 9.1562E+04, &
      7.912411E-02, 1.07677E+01, 9.0420E+00, 8.1936E+04, &
      8.458368E-02, 1.13913E+01, 8.7086E+00, 7.3820E+04, &
      9.041995E-02, 1.19094E+01, 8.4128E+00, 6.6710E+04, &
      9.665893E-02, 1.23487E+01, 8.1362E+00, 6.0352E+04, &
      1.033284E-01, 1.27032E+01, 7.8715E+00, 5.4620E+04, &
      1.095640E-01, 1.28584E+01, 7.6469E+00, 5.0042E+04, &
      1.104581E-01, 1.28304E+01, 7.6164E+00, 4.9438E+04, &
      1.112410E-01, 1.27438E+01, 7.5899E+00, 4.8920E+04, &
      1.116882E-01, 1.25364E+01, 7.5749E+00, 4.8628E+04, &
      1.123590E-01, 1.27940E+01, 7.9994E+00, 5.1047E+04, &
      1.140360E-01, 1.30669E+01, 7.9514E+00, 4.9994E+04, &
      1.180797E-01, 1.33934E+01, 7.8409E+00, 4.7611E+04, &
      1.262272E-01, 1.38459E+01, 7.6390E+00, 4.3391E+04, &
      1.349368E-01, 1.42329E+01, 7.4472E+00, 3.9572E+04, &
      1.442475E-01, 1.45869E+01, 7.2647E+00, 3.6111E+04, &
      1.542005E-01, 1.49196E+01, 7.0911E+00, 3.2973E+04, &
      1.648404E-01, 1.52378E+01, 6.9250E+00, 3.0122E+04, &
      1.762144E-01, 1.55459E+01, 6.7644E+00, 2.7524E+04, &
      1.883732E-01, 1.58466E+01, 6.6064E+00, 2.5146E+04, &
      2.013709E-01, 1.61413E+01, 6.4480E+00, 2.2959E+04, &
      2.152655E-01, 1.64384E+01, 6.2862E+00, 2.0939E+04, &
      2.301188E-01, 1.67200E+01, 6.1183E+00, 1.9064E+04, &
      2.459970E-01, 1.69927E+01, 5.9420E+00, 1.7320E+04, &
      2.629708E-01, 1.72537E+01, 5.7560E+00, 1.5696E+04, &
      2.811158E-01, 1.75008E+01, 5.5592E+00, 1.4181E+04, &
      3.005128E-01, 1.77292E+01, 5.3486E+00, 1.2763E+04, &
      3.212482E-01, 1.79330E+01, 5.1254E+00, 1.1441E+04, &
      3.434143E-01, 1.81062E+01, 4.8924E+00, 1.0217E+04, &
      3.671099E-01, 1.82471E+01, 4.6526E+00, 9.0893E+03, &
      3.924405E-01, 1.83508E+01, 4.4092E+00, 8.0582E+03, &
      4.195189E-01, 1.84135E+01, 4.1650E+00, 7.1211E+03, &
      4.484657E-01, 1.84310E+01, 3.9230E+00, 6.2749E+03, &
      4.794098E-01, 1.83977E+01, 3.6854E+00, 5.5148E+03, &
      5.124891E-01, 1.83055E+01, 3.4539E+00, 4.8354E+03, &
      5.478508E-01, 1.81427E+01, 3.2304E+00, 4.2312E+03, &
      5.856525E-01, 1.78905E+01, 3.0161E+00, 3.6960E+03, &
      6.260625E-01, 1.75178E+01, 2.8117E+00, 3.2238E+03, &
      6.692609E-01, 1.69681E+01, 2.6180E+00, 2.8086E+03, &
      7.154399E-01, 1.61269E+01, 2.4352E+00, 2.4444E+03, &
      7.648052E-01, 1.47079E+01, 2.2632E+00, 2.1258E+03, &
      8.175768E-01, 1.15821E+01, 2.1021E+00, 1.8478E+03, &
      8.376060E-01, 8.76961E+00, 2.0462E+00, 1.7559E+03, &
      8.504265E-01, 3.93819E+00, 2.0119E+00, 1.7005E+03, &
      8.544620E-01, -5.44705E+00, 2.0013E+00, 1.6836E+03, &
      8.589735E-01, 3.45285E+00, 1.1918E+01, 9.9522E+03, &
      8.675405E-01, 5.48784E+00, 1.1735E+01, 9.7031E+03, &
      8.710281E-01, 3.82284E+00, 1.1662E+01, 9.6042E+03, &
      8.717940E-01, 6.87490E-01, 1.1646E+01, 9.5826E+03, &
      8.739896E-01, 5.80960E+00, 1.6508E+01, 1.3547E+04, &
      8.762595E-01, 7.36968E+00, 1.6442E+01, 1.3457E+04, &
      8.893380E-01, 1.12789E+01, 1.6066E+01, 1.2957E+04, &
      9.342948E-01, 1.62345E+01, 1.4877E+01, 1.1421E+04, &
      9.879380E-01, 1.84258E+01, 1.3649E+01, 9.9100E+03, &
      9.987612E-01, 1.84133E+01, 1.3424E+01, 9.6410E+03, &
      1.003060E+00, 1.82231E+01, 1.3334E+01, 9.5360E+03, &
      1.007092E+00, 1.74472E+01, 1.3251E+01, 9.4386E+03, &
      1.013141E+00, 1.85689E+01, 1.4933E+01, 1.0573E+04, &
      1.028262E+00, 1.98654E+01, 1.4615E+01, 1.0196E+04, &
      1.067676E+00, 2.15374E+01, 1.3841E+01, 9.2996E+03, &
      1.141345E+00, 2.33736E+01, 1.2599E+01, 7.9192E+03, &
      1.220098E+00, 2.46251E+01, 1.1485E+01, 6.7538E+03, &
      1.304285E+00, 2.55591E+01, 1.0475E+01, 5.7632E+03, &
      1.394281E+00, 2.62818E+01, 9.5264E+00, 4.9036E+03, &
      1.490486E+00, 2.68506E+01, 8.6323E+00, 4.1573E+03, &
      1.593329E+00, 2.72544E+01, 7.7753E+00, 3.5036E+03, &
      1.703269E+00, 2.75358E+01, 7.0012E+00, 2.9519E+03, &
      1.820795E+00, 2.77300E+01, 6.3039E+00, 2.4871E+03, &
      1.946430E+00, 2.78603E+01, 5.6777E+00, 2.0960E+03, &
      2.080733E+00, 2.79855E+01, 5.1095E+00, 1.7652E+03, &
      2.224304E+00, 2.80289E+01, 4.5920E+00, 1.4847E+03, &
      2.377781E+00, 2.80405E+01, 4.1285E+00, 1.2493E+03, &
      2.541848E+00, 2.80299E+01, 3.7138E+00, 1.0519E+03, &
      2.717235E+00, 2.80016E+01, 3.3430E+00, 8.8628E+02, &
      2.904724E+00, 2.79659E+01, 3.0114E+00, 7.4738E+02, &
      3.105150E+00, 2.79386E+01, 2.6981E+00, 6.2692E+02, &
      3.319406E+00, 2.78847E+01, 2.4055E+00, 5.2336E+02, &
      3.548445E+00, 2.78165E+01, 2.1392E+00, 4.3590E+02, &
      3.793288E+00, 2.77340E+01, 1.8970E+00, 3.6208E+02, &
      4.055024E+00, 2.76425E+01, 1.6844E+00, 3.0119E+02, &
      4.334821E+00, 2.75458E+01, 1.4976E+00, 2.5093E+02, &
      4.633924E+00, 2.74435E+01, 1.3269E+00, 2.0839E+02, &
      4.953664E+00, 2.73322E+01, 1.1761E+00, 1.7316E+02, &
      5.295467E+00, 2.72114E+01, 1.0433E+00, 1.4406E+02, &
      5.660855E+00, 2.70779E+01, 9.2645E-01, 1.1998E+02, &
      6.051453E+00, 2.69254E+01, 8.2342E-01, 1.0006E+02, &
      6.469004E+00, 2.67418E+01, 7.3254E-01, 8.3554E+01, &
      6.915365E+00, 2.65023E+01, 6.5229E-01, 6.9859E+01, &
      7.392525E+00, 2.61454E+01, 5.8138E-01, 5.8485E+01, &
      7.902609E+00, 2.54382E+01, 5.1866E-01, 4.9027E+01, &
      8.166144E+00, 2.45187E+01, 4.9051E-01, 4.4977E+01, &
      8.291136E+00, 2.30862E+01, 4.7804E-01, 4.3221E+01, &
      8.324467E+00, 2.13577E+01, 4.7480E-01, 4.2769E+01, &
      8.374464E+00, 2.31229E+01, 3.8783E+00, 3.3391E+02, &
      8.447890E+00, 2.42866E+01, 3.8127E+00, 3.2545E+02, &
      8.499456E+00, 2.47181E+01, 3.7677E+00, 3.1967E+02, &
      9.030794E+00, 2.63786E+01, 3.4017E+00, 2.7180E+02, &
      9.653919E+00, 2.71216E+01, 3.0709E+00, 2.2970E+02, &
      1.032004E+01, 2.75654E+01, 2.7492E+00, 1.9252E+02, &
      1.103212E+01, 2.78453E+01, 2.4485E+00, 1.6054E+02, &
      1.179334E+01, 2.80311E+01, 2.1838E+00, 1.3407E+02, &
      1.260708E+01, 2.81583E+01, 1.9478E+00, 1.1200E+02, &
      1.347697E+01, 2.82454E+01, 1.7376E+00, 9.3573E+01, &
      1.440688E+01, 2.83043E+01, 1.5502E+00, 7.8199E+01, &
      1.540095E+01, 2.83432E+01, 1.3831E+00, 6.5367E+01, &
      1.646362E+01, 2.83682E+01, 1.2341E+00, 5.4649E+01, &
      1.759961E+01, 2.84192E+01, 1.0942E+00, 4.5418E+01, &
      1.881398E+01, 2.84194E+01, 9.6902E-01, 3.7707E+01, &
      2.011215E+01, 2.84120E+01, 8.5820E-01, 3.1316E+01, &
      2.149988E+01, 2.83996E+01, 7.6013E-01, 2.6017E+01, &
      2.298338E+01, 2.83839E+01, 6.7282E-01, 2.1609E+01, &
      2.456923E+01, 2.83663E+01, 5.9559E-01, 1.7956E+01, &
      2.626450E+01, 2.83479E+01, 5.2725E-01, 1.4927E+01, &
      2.807676E+01, 2.83290E+01, 4.6461E-01, 1.2361E+01, &
      3.001405E+01, 2.83086E+01, 4.0881E-01, 1.0228E+01, &
      3.208502E+01, 2.82880E+01, 3.5972E-01, 8.4690E+00, &
      3.429889E+01, 2.82678E+01, 3.1655E-01, 7.0190E+00, &
      3.666551E+01, 2.82484E+01, 2.7821E-01, 5.8159E+00, &
      3.919543E+01, 2.82295E+01, 2.4347E-01, 4.8054E+00, &
      4.189992E+01, 2.82110E+01, 2.1307E-01, 3.9760E+00, &
      4.479101E+01, 2.81934E+01, 1.8647E-01, 3.2952E+00, &
      4.788159E+01, 2.81768E+01, 1.6320E-01, 2.7362E+00, &
      5.118542E+01, 2.81613E+01, 1.4284E-01, 2.2770E+00, &
      5.471721E+01, 2.81468E+01, 1.2502E-01, 1.8995E+00, &
      5.849270E+01, 2.81334E+01, 1.0943E-01, 1.5892E+00, &
      6.252870E+01, 2.81211E+01, 9.5778E-02, 1.3339E+00, &
      6.684318E+01, 2.81097E+01, 8.3836E-02, 1.1238E+00, &
      7.145536E+01, 2.80994E+01, 7.3385E-02, 9.5072E-01, &
      7.638578E+01, 2.80899E+01, 6.4239E-02, 8.0808E-01, &
      8.165640E+01, 2.80813E+01, 5.6155E-02, 6.8970E-01, &
      8.729069E+01, 2.80734E+01, 4.8934E-02, 5.9076E-01, &
      9.331374E+01, 2.80662E+01, 4.2642E-02, 5.0932E-01, &
      9.975239E+01, 2.80596E+01, 3.7161E-02, 4.4215E-01, &
      1.066353E+02, 2.80536E+01, 3.2385E-02, 3.8667E-01, &
      1.139931E+02, 2.80481E+01, 2.8224E-02, 3.4073E-01, &
      1.218587E+02, 2.80432E+01, 2.4598E-02, 3.0261E-01, &
      1.302669E+02, 2.80387E+01, 2.1438E-02, 2.7088E-01, &
      1.392553E+02, 2.80347E+01, 1.8685E-02, 2.4438E-01, &
      1.488640E+02, 2.80311E+01, 1.6286E-02, 2.2215E-01, &
      1.591356E+02, 2.80278E+01, 1.4196E-02, 2.0343E-01, &
      1.701159E+02, 2.80248E+01, 1.2374E-02, 1.8758E-01, &
      1.818539E+02, 2.80222E+01, 1.0786E-02, 1.7408E-01, &
      1.944018E+02, 2.80198E+01, 9.4020E-03, 1.6252E-01, &
      2.078156E+02, 2.80177E+01, 8.1960E-03, 1.5253E-01, &
      2.221548E+02, 2.80158E+01, 7.1448E-03, 1.4383E-01, &
      2.374835E+02, 2.80141E+01, 6.2286E-03, 1.3620E-01, &
      2.538699E+02, 2.80125E+01, 5.4300E-03, 1.2947E-01, &
      2.713869E+02, 2.80112E+01, 4.7339E-03, 1.2344E-01, &
      2.901126E+02, 2.80099E+01, 4.1272E-03, 1.1802E-01, &
      3.101304E+02, 2.80088E+01, 3.5983E-03, 1.1309E-01, &
      3.315294E+02, 2.80079E+01, 3.1373E-03, 1.0857E-01, &
      3.544049E+02, 2.80070E+01, 2.7354E-03, 1.0440E-01, &
      3.788588E+02, 2.80062E+01, 2.3850E-03, 1.0052E-01, &
      4.050001E+02, 2.80055E+01, 2.0796E-03, 9.6876E-02, &
      4.329451E+02, 2.80049E+01, 1.8133E-03, 9.3442E-02]
   case(29)
      frel = -8.7600E-02
      allocate(nist_tab(748))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.11541E+00, 2.1733E+00, 1.3463E+05, &
      1.142761E-02, 5.16418E+00, 2.2889E+00, 1.3264E+05, &
      1.221612E-02, 5.21676E+00, 2.4126E+00, 1.3078E+05, &
      1.305903E-02, 5.27396E+00, 2.5449E+00, 1.2905E+05, &
      1.396010E-02, 5.33667E+00, 2.6864E+00, 1.2743E+05, &
      1.492335E-02, 5.40595E+00, 2.8374E+00, 1.2591E+05, &
      1.595306E-02, 5.48301E+00, 2.9985E+00, 1.2447E+05, &
      1.705382E-02, 5.56923E+00, 3.1701E+00, 1.2310E+05, &
      1.823053E-02, 5.66623E+00, 3.3525E+00, 1.2177E+05, &
      1.948844E-02, 5.77588E+00, 3.5460E+00, 1.2049E+05, &
      2.083314E-02, 5.90040E+00, 3.7507E+00, 1.1922E+05, &
      2.227063E-02, 6.04250E+00, 3.9669E+00, 1.1795E+05, &
      2.380730E-02, 6.20556E+00, 4.1946E+00, 1.1667E+05, &
      2.545001E-02, 6.39395E+00, 4.4336E+00, 1.1536E+05, &
      2.720606E-02, 6.61376E+00, 4.6837E+00, 1.1400E+05, &
      2.908327E-02, 6.87400E+00, 4.9447E+00, 1.1259E+05, &
      3.109002E-02, 7.18087E+00, 5.1429E+00, 1.0954E+05, &
      3.323523E-02, 7.48774E+00, 5.2695E+00, 1.0499E+05, &
      3.552846E-02, 7.79039E+00, 5.3892E+00, 1.0045E+05, &
      3.797993E-02, 8.09250E+00, 5.5005E+00, 9.5904E+04, &
      4.060054E-02, 8.39535E+00, 5.6019E+00, 9.1369E+04, &
      4.340198E-02, 8.69863E+00, 5.6922E+00, 8.6848E+04, &
      4.639671E-02, 9.00071E+00, 5.7698E+00, 8.2351E+04, &
      4.959809E-02, 9.29848E+00, 5.8335E+00, 7.7886E+04, &
      5.302035E-02, 9.58673E+00, 5.8821E+00, 7.3465E+04, &
      5.667876E-02, 9.85667E+00, 5.9145E+00, 6.9102E+04, &
      6.058959E-02, 1.00916E+01, 5.9298E+00, 6.4809E+04, &
      6.477028E-02, 1.02528E+01, 5.9273E+00, 6.0600E+04, &
      6.923942E-02, 1.02020E+01, 5.9066E+00, 5.6490E+04, &
      7.212800E-02, 9.67426E+00, 5.8847E+00, 5.4027E+04, &
      7.323200E-02, 8.64551E+00, 5.8748E+00, 5.3123E+04, &
      7.352640E-02, 7.25829E+00, 5.8721E+00, 5.2886E+04, &
      7.401695E-02, 9.01656E+00, 8.6666E+00, 7.7537E+04, &
      7.507200E-02, 1.03701E+01, 8.4419E+00, 7.4465E+04, &
      7.912411E-02, 1.18948E+01, 7.7340E+00, 6.4727E+04, &
      8.458368E-02, 1.25602E+01, 7.0931E+00, 5.5532E+04, &
      9.041995E-02, 1.29020E+01, 6.7123E+00, 4.9158E+04, &
      9.665893E-02, 1.31577E+01, 6.4597E+00, 4.4255E+04, &
      1.033284E-01, 1.33709E+01, 6.2728E+00, 4.0201E+04, &
      1.104581E-01, 1.35380E+01, 6.1234E+00, 3.6710E+04, &
      1.174040E-01, 1.35700E+01, 6.0085E+00, 3.3890E+04, &
      1.180797E-01, 1.35466E+01, 5.9985E+00, 3.3640E+04, &
      1.192010E-01, 1.34440E+01, 5.9824E+00, 3.3234E+04, &
      1.196802E-01, 1.32559E+01, 5.9757E+00, 3.3064E+04, &
      1.203990E-01, 1.34706E+01, 6.3567E+00, 3.4962E+04, &
      1.221960E-01, 1.36854E+01, 6.3407E+00, 3.4361E+04, &
      1.262272E-01, 1.39079E+01, 6.3091E+00, 3.3098E+04, &
      1.349368E-01, 1.42299E+01, 6.2577E+00, 3.0710E+04, &
      1.442475E-01, 1.45150E+01, 6.2188E+00, 2.8550E+04, &
      1.542005E-01, 1.47908E+01, 6.1892E+00, 2.6580E+04, &
      1.648404E-01, 1.50662E+01, 6.1667E+00, 2.4774E+04, &
      1.762144E-01, 1.53462E+01, 6.1485E+00, 2.3107E+04, &
      1.883732E-01, 1.56336E+01, 6.1310E+00, 2.1554E+04, &
      2.013709E-01, 1.59300E+01, 6.1101E+00, 2.0094E+04, &
      2.152655E-01, 1.62355E+01, 6.0812E+00, 1.8708E+04, &
      2.301188E-01, 1.65503E+01, 6.0398E+00, 1.7382E+04, &
      2.459970E-01, 1.68682E+01, 5.9821E+00, 1.6104E+04, &
      2.629708E-01, 1.71872E+01, 5.9048E+00, 1.4871E+04, &
      2.811158E-01, 1.75025E+01, 5.8057E+00, 1.3678E+04, &
      3.005128E-01, 1.78097E+01, 5.6839E+00, 1.2527E+04, &
      3.212482E-01, 1.81027E+01, 5.5360E+00, 1.1414E+04, &
      3.434143E-01, 1.83733E+01, 5.3635E+00, 1.0344E+04, &
      3.671099E-01, 1.86138E+01, 5.1699E+00, 9.3279E+03, &
      3.924405E-01, 1.88206E+01, 4.9596E+00, 8.3712E+03, &
      4.195189E-01, 1.89882E+01, 4.7369E+00, 7.4798E+03, &
      4.484657E-01, 1.91122E+01, 4.5063E+00, 6.6569E+03, &
      4.794098E-01, 1.91884E+01, 4.2718E+00, 5.9036E+03, &
      5.124891E-01, 1.92116E+01, 4.0367E+00, 5.2191E+03, &
      5.478508E-01, 1.91744E+01, 3.8040E+00, 4.6014E+03, &
      5.856525E-01, 1.90663E+01, 3.5762E+00, 4.0471E+03, &
      6.260625E-01, 1.88708E+01, 3.3552E+00, 3.5526E+03, &
      6.692609E-01, 1.85608E+01, 3.1426E+00, 3.1132E+03, &
      7.154399E-01, 1.80877E+01, 2.9393E+00, 2.7246E+03, &
      7.648052E-01, 1.73572E+01, 2.7122E+00, 2.3524E+03, &
      8.175768E-01, 1.61368E+01, 2.4939E+00, 2.0242E+03, &
      8.739896E-01, 1.36848E+01, 2.2948E+00, 1.7431E+03, &
      9.124780E-01, 9.41949E+00, 2.1756E+00, 1.5833E+03, &
      9.264445E-01, 4.24002E+00, 2.1352E+00, 1.5307E+03, &
      9.301689E-01, -1.51687E+00, 2.1247E+00, 1.5171E+03, &
      9.319800E-01, -1.75592E+00, 1.3119E+01, 9.3260E+03, &
      9.342948E-01, 2.65894E+00, 1.3043E+01, 9.2492E+03, &
      9.357555E-01, 3.88406E+00, 1.2996E+01, 9.2012E+03, &
      9.462450E-01, 6.37000E+00, 1.2662E+01, 8.8659E+03, &
      9.500490E-01, 4.47663E+00, 1.2545E+01, 8.7483E+03, &
      9.557550E-01, 8.42374E+00, 1.7798E+01, 1.2337E+04, &
      9.700200E-01, 1.27452E+01, 1.7165E+01, 1.1723E+04, &
      9.987612E-01, 1.65330E+01, 1.5984E+01, 1.0603E+04, &
      1.067676E+00, 1.98679E+01, 1.3862E+01, 8.6022E+03, &
      1.074178E+00, 1.99670E+01, 1.3705E+01, 8.4537E+03, &
      1.090619E+00, 1.97917E+01, 1.3328E+01, 8.0971E+03, &
      1.095004E+00, 1.91176E+01, 1.3231E+01, 8.0063E+03, &
      1.101580E+00, 2.00415E+01, 1.4651E+01, 8.8120E+03, &
      1.118022E+00, 2.11255E+01, 1.4383E+01, 8.5238E+03, &
      1.141345E+00, 2.20250E+01, 1.4041E+01, 8.1510E+03, &
      1.220098E+00, 2.40494E+01, 1.2862E+01, 6.9854E+03, &
      1.304285E+00, 2.53950E+01, 1.1731E+01, 5.9610E+03, &
      1.394281E+00, 2.63665E+01, 1.0688E+01, 5.0809E+03, &
      1.490486E+00, 2.70949E+01, 9.7449E+00, 4.3343E+03, &
      1.593329E+00, 2.76981E+01, 8.8867E+00, 3.6982E+03, &
      1.703269E+00, 2.81525E+01, 8.0183E+00, 3.1222E+03, &
      1.820795E+00, 2.84670E+01, 7.2303E+00, 2.6343E+03, &
      1.946430E+00, 2.86884E+01, 6.5194E+00, 2.2227E+03, &
      2.080733E+00, 2.88419E+01, 5.8780E+00, 1.8753E+03, &
      2.224304E+00, 2.89864E+01, 5.2957E+00, 1.5811E+03, &
      2.377781E+00, 2.90462E+01, 4.7581E+00, 1.3296E+03, &
      2.541848E+00, 2.90690E+01, 4.2769E+00, 1.1186E+03, &
      2.717235E+00, 2.90677E+01, 3.8470E+00, 9.4178E+02, &
      2.904724E+00, 2.90575E+01, 3.4633E+00, 7.9366E+02, &
      3.105150E+00, 2.90457E+01, 3.1014E+00, 6.6540E+02, &
      3.319406E+00, 2.89922E+01, 2.7637E+00, 5.5521E+02, &
      3.548445E+00, 2.89262E+01, 2.4650E+00, 4.6374E+02, &
      3.793288E+00, 2.88535E+01, 2.1976E+00, 3.8722E+02, &
      4.055024E+00, 2.87704E+01, 1.9496E+00, 3.2183E+02, &
      4.334821E+00, 2.86782E+01, 1.7317E+00, 2.6785E+02, &
      4.633924E+00, 2.85808E+01, 1.5402E+00, 2.2325E+02, &
      4.953664E+00, 2.84790E+01, 1.3670E+00, 1.8575E+02, &
      5.295467E+00, 2.83686E+01, 1.2121E+00, 1.5444E+02, &
      5.660855E+00, 2.82491E+01, 1.0758E+00, 1.2855E+02, &
      6.051453E+00, 2.81175E+01, 9.5563E-01, 1.0714E+02, &
      6.469004E+00, 2.79683E+01, 8.4971E-01, 8.9404E+01, &
      6.915365E+00, 2.77902E+01, 7.5624E-01, 7.4701E+01, &
      7.392525E+00, 2.75610E+01, 6.7369E-01, 6.2497E+01, &
      7.902609E+00, 2.72270E+01, 6.0071E-01, 5.2356E+01, &
      8.447890E+00, 2.65993E+01, 5.3613E-01, 4.3919E+01, &
      8.799322E+00, 2.55649E+01, 5.0037E-01, 3.9475E+01, &
      8.934005E+00, 2.41493E+01, 4.8721E-01, 3.7905E+01, &
      8.969921E+00, 2.24326E+01, 4.8340E-01, 3.7472E+01, &
      9.030794E+00, 2.43489E+01, 3.8582E+00, 2.8468E+02, &
      9.158478E+00, 2.57726E+01, 3.7540E+00, 2.7318E+02, &
      9.653919E+00, 2.73120E+01, 3.4054E+00, 2.3525E+02, &
      1.032004E+01, 2.80727E+01, 3.0504E+00, 1.9728E+02, &
      1.103212E+01, 2.85046E+01, 2.7462E+00, 1.6628E+02, &
      1.179334E+01, 2.87985E+01, 2.4736E+00, 1.4023E+02, &
      1.260708E+01, 2.90017E+01, 2.2129E+00, 1.1748E+02, &
      1.347697E+01, 2.91413E+01, 1.9766E+00, 9.8280E+01, &
      1.440688E+01, 2.92373E+01, 1.7631E+00, 8.2115E+01, &
      1.540095E+01, 2.93017E+01, 1.5712E+00, 6.8553E+01, &
      1.646362E+01, 2.93434E+01, 1.3994E+00, 5.7211E+01, &
      1.759961E+01, 2.93688E+01, 1.2463E+00, 4.7749E+01, &
      1.881398E+01, 2.94189E+01, 1.1059E+00, 3.9716E+01, &
      2.011215E+01, 2.94195E+01, 9.7986E-01, 3.2995E+01, &
      2.149988E+01, 2.94125E+01, 8.6826E-01, 2.7422E+01, &
      2.298338E+01, 2.94006E+01, 7.6945E-01, 2.2797E+01, &
      2.456923E+01, 2.93855E+01, 6.8155E-01, 1.8952E+01, &
      2.626450E+01, 2.93684E+01, 6.0363E-01, 1.5759E+01, &
      2.807676E+01, 2.93516E+01, 5.3216E-01, 1.3052E+01, &
      3.001405E+01, 2.93316E+01, 4.6847E-01, 1.0801E+01, &
      3.208502E+01, 2.93106E+01, 4.1242E-01, 8.9450E+00, &
      3.429889E+01, 2.92896E+01, 3.6310E-01, 7.4136E+00, &
      3.666551E+01, 2.92692E+01, 3.1969E-01, 6.1500E+00, &
      3.919543E+01, 2.92497E+01, 2.8131E-01, 5.1043E+00, &
      4.189992E+01, 2.92308E+01, 2.4628E-01, 4.2217E+00, &
      4.479101E+01, 2.92123E+01, 2.1563E-01, 3.4968E+00, &
      4.788159E+01, 2.91946E+01, 1.8880E-01, 2.9016E+00, &
      5.118542E+01, 2.91779E+01, 1.6531E-01, 2.4124E+00, &
      5.471721E+01, 2.91623E+01, 1.4475E-01, 2.0104E+00, &
      5.849270E+01, 2.91478E+01, 1.2675E-01, 1.6796E+00, &
      6.252870E+01, 2.91344E+01, 1.1098E-01, 1.4076E+00, &
      6.684318E+01, 2.91220E+01, 9.7185E-02, 1.1836E+00, &
      7.145536E+01, 2.91106E+01, 8.5104E-02, 9.9910E-01, &
      7.638578E+01, 2.91001E+01, 7.4527E-02, 8.4703E-01, &
      8.165640E+01, 2.90907E+01, 6.5180E-02, 7.2087E-01, &
      8.729069E+01, 2.90821E+01, 5.6834E-02, 6.1553E-01, &
      9.331374E+01, 2.90741E+01, 4.9559E-02, 5.2879E-01, &
      9.975239E+01, 2.90668E+01, 4.3217E-02, 4.5727E-01, &
      1.066353E+02, 2.90601E+01, 3.7687E-02, 3.9821E-01, &
      1.139931E+02, 2.90541E+01, 3.2866E-02, 3.4934E-01, &
      1.218587E+02, 2.90486E+01, 2.8662E-02, 3.0882E-01, &
      1.302669E+02, 2.90436E+01, 2.4997E-02, 2.7512E-01, &
      1.392553E+02, 2.90391E+01, 2.1801E-02, 2.4700E-01, &
      1.488640E+02, 2.90350E+01, 1.9015E-02, 2.2346E-01, &
      1.591356E+02, 2.90313E+01, 1.6585E-02, 2.0368E-01, &
      1.701159E+02, 2.90280E+01, 1.4466E-02, 1.8697E-01, &
      1.818539E+02, 2.90251E+01, 1.2618E-02, 1.7278E-01, &
      1.944018E+02, 2.90224E+01, 1.1006E-02, 1.6064E-01, &
      2.078156E+02, 2.90200E+01, 9.6007E-03, 1.5021E-01, &
      2.221548E+02, 2.90179E+01, 8.3749E-03, 1.4116E-01, &
      2.374835E+02, 2.90159E+01, 7.3058E-03, 1.3327E-01, &
      2.538699E+02, 2.90142E+01, 6.3734E-03, 1.2632E-01, &
      2.713869E+02, 2.90127E+01, 5.5601E-03, 1.2015E-01, &
      2.901126E+02, 2.90113E+01, 4.8507E-03, 1.1461E-01, &
      3.101304E+02, 2.90101E+01, 4.2319E-03, 1.0961E-01, &
      3.315294E+02, 2.90090E+01, 3.6922E-03, 1.0505E-01, &
      3.544049E+02, 2.90080E+01, 3.2214E-03, 1.0086E-01, &
      3.788588E+02, 2.90071E+01, 2.8106E-03, 9.6977E-02, &
      4.050001E+02, 2.90063E+01, 2.4524E-03, 9.3355E-02, &
      4.329451E+02, 2.90056E+01, 2.1398E-03, 8.9953E-02]
   case(30)
      frel = -9.5400E-02
      allocate(nist_tab(2220))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      7.938000E-03, 1.89392E+00, 0.0000E+00, 2.0403E-05, &
      8.059500E-03, 1.58428E+00, 0.0000E+00, 2.2107E-05, &
      8.091900E-03, 1.24074E+00, 0.0000E+00, 2.2579E-05, &
      8.140500E-03, 1.56676E+00, 6.6117E-01, 5.2275E+04, &
      8.262000E-03, 1.83711E+00, 6.7285E-01, 5.2416E+04, &
      1.069000E-02, 2.18614E+00, 9.1642E-01, 5.5176E+04, &
      1.142761E-02, 2.18313E+00, 9.9609E-01, 5.6102E+04, &
      1.221612E-02, 2.17165E+00, 1.0846E+00, 5.7145E+04, &
      1.305903E-02, 2.15396E+00, 1.1832E+00, 5.8313E+04, &
      1.396010E-02, 2.13164E+00, 1.2930E+00, 5.9612E+04, &
      1.492335E-02, 2.10599E+00, 1.4153E+00, 6.1041E+04, &
      1.595306E-02, 2.07822E+00, 1.5516E+00, 6.2600E+04, &
      1.705382E-02, 2.04961E+00, 1.7033E+00, 6.4283E+04, &
      1.823053E-02, 2.02156E+00, 1.8717E+00, 6.6081E+04, &
      1.948844E-02, 1.99571E+00, 2.0584E+00, 6.7982E+04, &
      2.083314E-02, 1.97391E+00, 2.2648E+00, 6.9969E+04, &
      2.227063E-02, 1.95833E+00, 2.4922E+00, 7.2024E+04, &
      2.380730E-02, 1.95148E+00, 2.7417E+00, 7.4123E+04, &
      2.545001E-02, 1.91980E+00, 3.0145E+00, 7.6237E+04, &
      2.720606E-02, 1.93967E+00, 3.3112E+00, 7.8335E+04, &
      2.908327E-02, 1.97834E+00, 3.6322E+00, 8.0383E+04, &
      3.109002E-02, 2.04040E+00, 3.9774E+00, 8.2341E+04, &
      3.323523E-02, 2.13183E+00, 4.3457E+00, 8.4158E+04, &
      3.552846E-02, 2.25682E+00, 4.7295E+00, 8.5679E+04, &
      3.797993E-02, 2.41567E+00, 5.1247E+00, 8.6845E+04, &
      4.060054E-02, 2.60900E+00, 5.5282E+00, 8.7636E+04, &
      4.340198E-02, 2.83737E+00, 5.9369E+00, 8.8040E+04, &
      4.639671E-02, 3.10102E+00, 6.3475E+00, 8.8054E+04, &
      4.959809E-02, 3.39971E+00, 6.7568E+00, 8.7682E+04, &
      5.302035E-02, 3.73279E+00, 7.1617E+00, 8.6937E+04, &
      5.667876E-02, 4.09943E+00, 7.5593E+00, 8.5841E+04, &
      6.058959E-02, 4.50246E+00, 7.9471E+00, 8.4420E+04, &
      6.477028E-02, 4.96402E+00, 8.3109E+00, 8.2586E+04, &
      6.923942E-02, 5.42968E+00, 8.6353E+00, 8.0271E+04, &
      7.401695E-02, 5.88267E+00, 8.9287E+00, 7.7641E+04, &
      7.912411E-02, 6.28468E+00, 9.1879E+00, 7.4738E+04, &
      8.458368E-02, 6.38470E+00, 9.4104E+00, 7.1607E+04, &
      8.486800E-02, 6.35349E+00, 9.4205E+00, 7.1444E+04, &
      8.616700E-02, 5.90480E+00, 9.4654E+00, 7.0702E+04, &
      8.651340E-02, 5.22864E+00, 9.4769E+00, 7.0505E+04, &
      8.703300E-02, 6.04111E+00, 1.0902E+01, 8.0620E+04, &
      8.833200E-02, 6.84154E+00, 1.0895E+01, 7.9386E+04, &
      9.041995E-02, 7.44473E+00, 1.0884E+01, 7.7473E+04, &
      9.665893E-02, 8.49813E+00, 1.0847E+01, 7.2228E+04, &
      1.033284E-01, 9.27197E+00, 1.0845E+01, 6.7553E+04, &
      1.104581E-01, 9.96770E+00, 1.0871E+01, 6.3343E+04, &
      1.180797E-01, 1.06284E+01, 1.0904E+01, 5.9437E+04, &
      1.262272E-01, 1.12517E+01, 1.0936E+01, 5.5764E+04, &
      1.331820E-01, 1.16453E+01, 1.0958E+01, 5.2958E+04, &
      1.349368E-01, 1.16353E+01, 1.0963E+01, 5.2293E+04, &
      1.352205E-01, 1.16068E+01, 1.0964E+01, 5.2187E+04, &
      1.357641E-01, 1.14140E+01, 1.0965E+01, 5.1985E+04, &
      1.365795E-01, 1.17133E+01, 1.1429E+01, 5.3861E+04, &
      1.386180E-01, 1.20794E+01, 1.1441E+01, 5.3125E+04, &
      1.442475E-01, 1.27012E+01, 1.1472E+01, 5.1190E+04, &
      1.542005E-01, 1.36448E+01, 1.1437E+01, 4.7738E+04, &
      1.648404E-01, 1.44977E+01, 1.1070E+01, 4.3223E+04, &
      1.762144E-01, 1.52357E+01, 1.0696E+01, 3.9069E+04, &
      1.883732E-01, 1.58898E+01, 1.0320E+01, 3.5263E+04, &
      2.013709E-01, 1.64779E+01, 9.9442E+00, 3.1785E+04, &
      2.152655E-01, 1.70117E+01, 9.5693E+00, 2.8612E+04, &
      2.301188E-01, 1.74994E+01, 9.1959E+00, 2.5721E+04, &
      2.459970E-01, 1.79464E+01, 8.8239E+00, 2.3088E+04, &
      2.629708E-01, 1.83590E+01, 8.4529E+00, 2.0691E+04, &
      2.811158E-01, 1.87336E+01, 8.0824E+00, 1.8507E+04, &
      3.005128E-01, 1.90739E+01, 7.7122E+00, 1.6520E+04, &
      3.212482E-01, 1.93800E+01, 7.3424E+00, 1.4713E+04, &
      3.434143E-01, 1.96517E+01, 6.9736E+00, 1.3072E+04, &
      3.671099E-01, 1.98882E+01, 6.6052E+00, 1.1582E+04, &
      3.924405E-01, 2.00860E+01, 6.2364E+00, 1.0231E+04, &
      4.195189E-01, 2.02402E+01, 5.8703E+00, 9.0089E+03, &
      4.484657E-01, 2.03509E+01, 5.5099E+00, 7.9106E+03, &
      4.794098E-01, 2.04139E+01, 5.1581E+00, 6.9281E+03, &
      5.124891E-01, 2.04261E+01, 4.8174E+00, 6.0534E+03, &
      5.478508E-01, 2.03829E+01, 4.4899E+00, 5.2783E+03, &
      5.856525E-01, 2.02777E+01, 4.1770E+00, 4.5941E+03, &
      6.260625E-01, 2.00994E+01, 3.8799E+00, 3.9926E+03, &
      6.692609E-01, 1.98305E+01, 3.5993E+00, 3.4653E+03, &
      7.154399E-01, 1.94417E+01, 3.3356E+00, 3.0049E+03, &
      7.648052E-01, 1.88803E+01, 3.0889E+00, 2.6037E+03, &
      8.175768E-01, 1.80422E+01, 2.8590E+00, 2.2551E+03, &
      8.739896E-01, 1.66852E+01, 2.6453E+00, 1.9525E+03, &
      9.0000000E-01, 1.73931E+01, 2.5564E+00, 1.8327E+03, &
      9.0450000E-01, 1.72630E+01, 2.5415E+00, 1.8131E+03, &
      9.0902250E-01, 1.71262E+01, 2.5268E+00, 1.7936E+03, &
      9.1356761E-01, 1.69821E+01, 2.5121E+00, 1.7744E+03, &
      9.1813545E-01, 1.68299E+01, 2.4976E+00, 1.7554E+03, &
      9.2272613E-01, 1.66690E+01, 2.4831E+00, 1.7366E+03, &
      9.2733976E-01, 1.64984E+01, 2.4687E+00, 1.7180E+03, &
      9.3197646E-01, 1.63171E+01, 2.4544E+00, 1.6996E+03, &
      9.3663634E-01, 1.61239E+01, 2.4401E+00, 1.6814E+03, &
      9.4131952E-01, 1.59173E+01, 2.4260E+00, 1.6634E+03, &
      9.4602612E-01, 1.56957E+01, 2.4119E+00, 1.6455E+03, &
      9.5075625E-01, 1.54569E+01, 2.3979E+00, 1.6279E+03, &
      9.5551003E-01, 1.51986E+01, 2.3840E+00, 1.6105E+03, &
      9.6028758E-01, 1.49176E+01, 2.3702E+00, 1.5932E+03, &
      9.6508902E-01, 1.46101E+01, 2.3565E+00, 1.5762E+03, &
      9.6991446E-01, 1.42713E+01, 2.3428E+00, 1.5593E+03, &
      9.7476404E-01, 1.38946E+01, 2.3292E+00, 1.5426E+03, &
      9.7963786E-01, 1.34716E+01, 2.3157E+00, 1.5261E+03, &
      9.8453605E-01, 1.29904E+01, 2.3023E+00, 1.5098E+03, &
      9.8945873E-01, 1.24339E+01, 2.2890E+00, 1.4936E+03, &
      9.9440602E-01, 1.17757E+01, 2.2757E+00, 1.4776E+03, &
      9.9937805E-01, 1.09729E+01, 2.2625E+00, 1.4618E+03, &
      1.0043749E+00, 9.94316E+00, 2.2458E+00, 1.4439E+03, &
      1.0093968E+00, 8.51478E+00, 2.2287E+00, 1.4258E+03, &
      1.0144438E+00, 6.15159E+00, 2.2118E+00, 1.4080E+03, &
      1.0195160E+00, -5.07956E+00, 2.1950E+00, 1.3904E+03, &
      1.0196290E+00, -8.22555E+00, 2.1946E+00, 1.3901E+03, &
      1.0197711E+00, -8.40332E+00, 1.2572E+01, 7.9394E+03, &
      1.0246136E+00, 5.36770E+00, 1.2469E+01, 7.8376E+03, &
      1.0297367E+00, 7.32086E+00, 1.2362E+01, 7.7317E+03, &
      1.0348853E+00, 7.99378E+00, 1.2256E+01, 7.6273E+03, &
      1.0400598E+00, 7.35582E+00, 1.2151E+01, 7.5244E+03, &
      1.0427279E+00, 1.91437E+00, 1.2098E+01, 7.4720E+03, &
      1.0428720E+00, 1.85341E+00, 1.7202E+01, 1.0621E+04, &
      1.0452601E+00, 7.97659E+00, 1.7134E+01, 1.0555E+04, &
      1.0504864E+00, 1.05445E+01, 1.6987E+01, 1.0413E+04, &
      1.0557388E+00, 1.19953E+01, 1.6842E+01, 1.0272E+04, &
      1.0610175E+00, 1.30750E+01, 1.6697E+01, 1.0134E+04, &
      1.0663226E+00, 1.39517E+01, 1.6554E+01, 9.9968E+03, &
      1.0716542E+00, 1.46953E+01, 1.6412E+01, 9.8619E+03, &
      1.0770125E+00, 1.53428E+01, 1.6272E+01, 9.7289E+03, &
      1.0823975E+00, 1.59165E+01, 1.6132E+01, 9.5976E+03, &
      1.0878095E+00, 1.64313E+01, 1.5994E+01, 9.4682E+03, &
      1.0932486E+00, 1.68974E+01, 1.5857E+01, 9.3405E+03, &
      1.0987148E+00, 1.73222E+01, 1.5722E+01, 9.2145E+03, &
      1.1042084E+00, 1.77116E+01, 1.5587E+01, 9.0903E+03, &
      1.1097294E+00, 1.80696E+01, 1.5454E+01, 8.9677E+03, &
      1.1152781E+00, 1.83997E+01, 1.5321E+01, 8.8468E+03, &
      1.1208545E+00, 1.87044E+01, 1.5190E+01, 8.7275E+03, &
      1.1264587E+00, 1.89856E+01, 1.5060E+01, 8.6099E+03, &
      1.1320910E+00, 1.92447E+01, 1.4932E+01, 8.4939E+03, &
      1.1377515E+00, 1.94825E+01, 1.4804E+01, 8.3794E+03, &
      1.1434402E+00, 1.96995E+01, 1.4677E+01, 8.2665E+03, &
      1.1491574E+00, 1.98953E+01, 1.4552E+01, 8.1551E+03, &
      1.1549032E+00, 2.00688E+01, 1.4427E+01, 8.0452E+03, &
      1.1606777E+00, 2.02177E+01, 1.4304E+01, 7.9369E+03, &
      1.1664811E+00, 2.03376E+01, 1.4182E+01, 7.8300E+03, &
      1.1723135E+00, 2.04199E+01, 1.4061E+01, 7.7245E+03, &
      1.1781751E+00, 2.04469E+01, 1.3941E+01, 7.6205E+03, &
      1.1840660E+00, 2.03734E+01, 1.3822E+01, 7.5179E+03, &
      1.1899863E+00, 2.00079E+01, 1.3703E+01, 7.4167E+03, &
      1.1924542E+00, 1.94199E+01, 1.3655E+01, 7.3750E+03, &
      1.1947459E+00, 1.94670E+01, 1.5440E+01, 8.3226E+03, &
      1.1959362E+00, 1.99205E+01, 1.5416E+01, 8.3013E+03, &
      1.2019159E+00, 2.08465E+01, 1.5294E+01, 8.1951E+03, &
      1.2079255E+00, 2.13413E+01, 1.5174E+01, 8.0903E+03, &
      1.2139651E+00, 2.17172E+01, 1.5055E+01, 7.9869E+03, &
      1.2200350E+00, 2.20337E+01, 1.4937E+01, 7.8849E+03, &
      1.2261351E+00, 2.23130E+01, 1.4820E+01, 7.7842E+03, &
      1.2322658E+00, 2.25663E+01, 1.4704E+01, 7.6848E+03, &
      1.2384271E+00, 2.27996E+01, 1.4589E+01, 7.5868E+03, &
      1.2446193E+00, 2.30170E+01, 1.4474E+01, 7.4900E+03, &
      1.2508424E+00, 2.32211E+01, 1.4361E+01, 7.3945E+03, &
      1.2570966E+00, 2.34138E+01, 1.4249E+01, 7.3005E+03, &
      1.2633821E+00, 2.35968E+01, 1.4139E+01, 7.2079E+03, &
      1.2696990E+00, 2.37712E+01, 1.4029E+01, 7.1165E+03, &
      1.2760475E+00, 2.39379E+01, 1.3921E+01, 7.0264E+03, &
      1.2824277E+00, 2.40975E+01, 1.3813E+01, 6.9374E+03, &
      1.2888399E+00, 2.42507E+01, 1.3706E+01, 6.8496E+03, &
      1.2952840E+00, 2.43978E+01, 1.3601E+01, 6.7630E+03, &
      1.3017605E+00, 2.45392E+01, 1.3496E+01, 6.6776E+03, &
      1.3082693E+00, 2.46754E+01, 1.3392E+01, 6.5933E+03, &
      1.3148106E+00, 2.48065E+01, 1.3289E+01, 6.5101E+03, &
      1.3213847E+00, 2.49329E+01, 1.3187E+01, 6.4280E+03, &
      1.3279916E+00, 2.50546E+01, 1.3086E+01, 6.3474E+03, &
      1.3346316E+00, 2.51727E+01, 1.2989E+01, 6.2687E+03, &
      1.3413047E+00, 2.52874E+01, 1.2892E+01, 6.1911E+03, &
      1.3480112E+00, 2.53986E+01, 1.2796E+01, 6.1145E+03, &
      1.3547513E+00, 2.55066E+01, 1.2701E+01, 6.0389E+03, &
      1.3615250E+00, 2.56115E+01, 1.2607E+01, 5.9648E+03, &
      1.3683327E+00, 2.57138E+01, 1.2515E+01, 5.8917E+03, &
      1.3751743E+00, 2.58134E+01, 1.2423E+01, 5.8195E+03, &
      1.3820502E+00, 2.59104E+01, 1.2333E+01, 5.7484E+03, &
      1.3889605E+00, 2.60050E+01, 1.2243E+01, 5.6781E+03, &
      1.3959053E+00, 2.60973E+01, 1.2154E+01, 5.6088E+03, &
      1.4028848E+00, 2.61872E+01, 1.2065E+01, 5.5403E+03, &
      1.4098992E+00, 2.62750E+01, 1.1978E+01, 5.4728E+03, &
      1.4169487E+00, 2.63606E+01, 1.1891E+01, 5.4061E+03, &
      1.4240335E+00, 2.64442E+01, 1.1805E+01, 5.3403E+03, &
      1.4311536E+00, 2.65257E+01, 1.1719E+01, 5.2754E+03, &
      1.4383094E+00, 2.66054E+01, 1.1634E+01, 5.2112E+03, &
      1.4455009E+00, 2.66833E+01, 1.1550E+01, 5.1479E+03, &
      1.4527284E+00, 2.67594E+01, 1.1467E+01, 5.0855E+03, &
      1.4599921E+00, 2.68337E+01, 1.1385E+01, 5.0238E+03, &
      1.4672920E+00, 2.69065E+01, 1.1303E+01, 4.9629E+03, &
      1.4746285E+00, 2.69776E+01, 1.1222E+01, 4.9028E+03, &
      1.4820016E+00, 2.70472E+01, 1.1141E+01, 4.8434E+03, &
      1.4894117E+00, 2.71153E+01, 1.1061E+01, 4.7849E+03, &
      1.4968587E+00, 2.71820E+01, 1.0982E+01, 4.7270E+03, &
      1.5043430E+00, 2.72473E+01, 1.0903E+01, 4.6699E+03, &
      1.5118647E+00, 2.73113E+01, 1.0825E+01, 4.6135E+03, &
      1.5194240E+00, 2.73741E+01, 1.0748E+01, 4.5578E+03, &
      1.5270212E+00, 2.74356E+01, 1.0671E+01, 4.5028E+03, &
      1.5346563E+00, 2.74960E+01, 1.0595E+01, 4.4485E+03, &
      1.5423295E+00, 2.75554E+01, 1.0520E+01, 4.3949E+03, &
      1.5500412E+00, 2.76139E+01, 1.0445E+01, 4.3420E+03, &
      1.5577914E+00, 2.76711E+01, 1.0369E+01, 4.2891E+03, &
      1.5655804E+00, 2.77270E+01, 1.0294E+01, 4.2368E+03, &
      1.5734083E+00, 2.77815E+01, 1.0219E+01, 4.1851E+03, &
      1.5812753E+00, 2.78347E+01, 1.0144E+01, 4.1340E+03, &
      1.5891817E+00, 2.78868E+01, 1.0070E+01, 4.0835E+03, &
      1.5971276E+00, 2.79377E+01, 9.9972E+00, 4.0337E+03, &
      1.6051132E+00, 2.79874E+01, 9.9245E+00, 3.9845E+03, &
      1.6131388E+00, 2.80361E+01, 9.8523E+00, 3.9359E+03, &
      1.6212045E+00, 2.80838E+01, 9.7807E+00, 3.8879E+03, &
      1.6293105E+00, 2.81304E+01, 9.7092E+00, 3.8404E+03, &
      1.6374571E+00, 2.81761E+01, 9.6375E+00, 3.7931E+03, &
      1.6456443E+00, 2.82207E+01, 9.5663E+00, 3.7464E+03, &
      1.6538726E+00, 2.82644E+01, 9.4957E+00, 3.7003E+03, &
      1.6621419E+00, 2.83071E+01, 9.4256E+00, 3.6548E+03, &
      1.6704526E+00, 2.83489E+01, 9.3561E+00, 3.6098E+03, &
      1.6788049E+00, 2.83898E+01, 9.2870E+00, 3.5654E+03, &
      1.6871989E+00, 2.84298E+01, 9.2185E+00, 3.5216E+03, &
      1.6956349E+00, 2.84690E+01, 9.1505E+00, 3.4783E+03, &
      1.7041131E+00, 2.85073E+01, 9.0830E+00, 3.4355E+03, &
      1.7126337E+00, 2.85449E+01, 9.0160E+00, 3.3932E+03, &
      1.7211968E+00, 2.85816E+01, 8.9495E+00, 3.3515E+03, &
      1.7298028E+00, 2.86175E+01, 8.8835E+00, 3.3103E+03, &
      1.7384518E+00, 2.86527E+01, 8.8180E+00, 3.2696E+03, &
      1.7471441E+00, 2.86872E+01, 8.7530E+00, 3.2294E+03, &
      1.7558798E+00, 2.87209E+01, 8.6884E+00, 3.1897E+03, &
      1.7646592E+00, 2.87539E+01, 8.6244E+00, 3.1505E+03, &
      1.7734825E+00, 2.87862E+01, 8.5608E+00, 3.1118E+03, &
      1.7823499E+00, 2.88179E+01, 8.4977E+00, 3.0735E+03, &
      1.7912617E+00, 2.88489E+01, 8.4350E+00, 3.0357E+03, &
      1.8002180E+00, 2.88793E+01, 8.3728E+00, 2.9984E+03, &
      1.8092191E+00, 2.89090E+01, 8.3111E+00, 2.9616E+03, &
      1.8182652E+00, 2.89381E+01, 8.2499E+00, 2.9252E+03, &
      1.8273565E+00, 2.89666E+01, 8.1891E+00, 2.8892E+03, &
      1.8364933E+00, 2.89946E+01, 8.1287E+00, 2.8537E+03, &
      1.8456757E+00, 2.90220E+01, 8.0688E+00, 2.8186E+03, &
      1.8549041E+00, 2.90488E+01, 8.0093E+00, 2.7840E+03, &
      1.8641786E+00, 2.90750E+01, 7.9503E+00, 2.7498E+03, &
      1.8734995E+00, 2.91008E+01, 7.8917E+00, 2.7160E+03, &
      1.8828670E+00, 2.91260E+01, 7.8335E+00, 2.6826E+03, &
      1.8922814E+00, 2.91507E+01, 7.7758E+00, 2.6497E+03, &
      1.9017428E+00, 2.91750E+01, 7.7185E+00, 2.6171E+03, &
      1.9112515E+00, 2.91987E+01, 7.6616E+00, 2.5850E+03, &
      1.9208077E+00, 2.92220E+01, 7.6052E+00, 2.5532E+03, &
      1.9304118E+00, 2.92448E+01, 7.5491E+00, 2.5218E+03, &
      1.9400638E+00, 2.92672E+01, 7.4935E+00, 2.4909E+03, &
      1.9497642E+00, 2.92892E+01, 7.4383E+00, 2.4603E+03, &
      1.9595130E+00, 2.93107E+01, 7.3835E+00, 2.4300E+03, &
      1.9693105E+00, 2.93318E+01, 7.3291E+00, 2.4002E+03, &
      1.9791571E+00, 2.93526E+01, 7.2751E+00, 2.3707E+03, &
      1.9890529E+00, 2.93730E+01, 7.2216E+00, 2.3416E+03, &
      1.9989981E+00, 2.93930E+01, 7.1684E+00, 2.3128E+03, &
      2.0089931E+00, 2.94126E+01, 7.1156E+00, 2.2845E+03, &
      2.0190381E+00, 2.94320E+01, 7.0632E+00, 2.2564E+03, &
      2.0291333E+00, 2.94510E+01, 7.0112E+00, 2.2287E+03, &
      2.0392790E+00, 2.94697E+01, 6.9596E+00, 2.2013E+03, &
      2.0494754E+00, 2.96021E+01, 6.9060E+00, 2.1736E+03, &
      2.0597227E+00, 2.96199E+01, 6.8528E+00, 2.1462E+03, &
      2.0700213E+00, 2.96373E+01, 6.8000E+00, 2.1191E+03, &
      2.0803714E+00, 2.96541E+01, 6.7476E+00, 2.0923E+03, &
      2.0907733E+00, 2.97278E+01, 6.6950E+00, 2.0658E+03, &
      2.1012272E+00, 2.97436E+01, 6.6423E+00, 2.0394E+03, &
      2.1117333E+00, 2.97589E+01, 6.5900E+00, 2.0133E+03, &
      2.1222920E+00, 2.97737E+01, 6.5380E+00, 1.9875E+03, &
      2.1329034E+00, 2.97879E+01, 6.4865E+00, 1.9621E+03, &
      2.1435680E+00, 2.98017E+01, 6.4354E+00, 1.9370E+03, &
      2.1542858E+00, 2.98149E+01, 6.3847E+00, 1.9123E+03, &
      2.1650572E+00, 2.98278E+01, 6.3345E+00, 1.8878E+03, &
      2.1758825E+00, 2.98401E+01, 6.2846E+00, 1.8637E+03, &
      2.1867619E+00, 2.98521E+01, 6.2351E+00, 1.8399E+03, &
      2.1976957E+00, 2.98637E+01, 6.1860E+00, 1.8164E+03, &
      2.2086842E+00, 2.98748E+01, 6.1373E+00, 1.7932E+03, &
      2.2197276E+00, 2.98856E+01, 6.0890E+00, 1.7702E+03, &
      2.2308263E+00, 2.98960E+01, 6.0411E+00, 1.7476E+03, &
      2.2419804E+00, 2.99061E+01, 5.9935E+00, 1.7253E+03, &
      2.2531903E+00, 2.99158E+01, 5.9464E+00, 1.7033E+03, &
      2.2644562E+00, 2.99252E+01, 5.8996E+00, 1.6815E+03, &
      2.2757785E+00, 2.99343E+01, 5.8532E+00, 1.6601E+03, &
      2.2871574E+00, 2.99430E+01, 5.8072E+00, 1.6389E+03, &
      2.2985932E+00, 2.99515E+01, 5.7616E+00, 1.6179E+03, &
      2.3100862E+00, 2.99596E+01, 5.7163E+00, 1.5973E+03, &
      2.3216366E+00, 2.99675E+01, 5.6714E+00, 1.5769E+03, &
      2.3332448E+00, 2.99751E+01, 5.6269E+00, 1.5568E+03, &
      2.3449110E+00, 2.99824E+01, 5.5827E+00, 1.5370E+03, &
      2.3566356E+00, 2.99895E+01, 5.5389E+00, 1.5174E+03, &
      2.3684187E+00, 2.99964E+01, 5.4955E+00, 1.4980E+03, &
      2.3802608E+00, 3.00030E+01, 5.4524E+00, 1.4789E+03, &
      2.3921621E+00, 3.00496E+01, 5.4093E+00, 1.4600E+03, &
      2.4041230E+00, 3.00559E+01, 5.3661E+00, 1.4412E+03, &
      2.4161436E+00, 3.00619E+01, 5.3232E+00, 1.4226E+03, &
      2.4282243E+00, 3.00676E+01, 5.2806E+00, 1.4043E+03, &
      2.4403654E+00, 3.00730E+01, 5.2385E+00, 1.3862E+03, &
      2.4525672E+00, 3.00782E+01, 5.1966E+00, 1.3683E+03, &
      2.4648301E+00, 3.00831E+01, 5.1551E+00, 1.3507E+03, &
      2.4771542E+00, 3.00877E+01, 5.1140E+00, 1.3333E+03, &
      2.4895400E+00, 3.00921E+01, 5.0732E+00, 1.3161E+03, &
      2.5019877E+00, 3.00962E+01, 5.0327E+00, 1.2992E+03, &
      2.5144976E+00, 3.01001E+01, 4.9925E+00, 1.2824E+03, &
      2.5270701E+00, 3.01038E+01, 4.9527E+00, 1.2659E+03, &
      2.5397055E+00, 3.01073E+01, 4.9133E+00, 1.2497E+03, &
      2.5524040E+00, 3.01105E+01, 4.8741E+00, 1.2336E+03, &
      2.5651660E+00, 3.01136E+01, 4.8351E+00, 1.2177E+03, &
      2.5779919E+00, 3.01165E+01, 4.7965E+00, 1.2020E+03, &
      2.5908818E+00, 3.01191E+01, 4.7582E+00, 1.1865E+03, &
      2.6038362E+00, 3.01216E+01, 4.7202E+00, 1.1712E+03, &
      2.6168554E+00, 3.01239E+01, 4.6825E+00, 1.1561E+03, &
      2.6299397E+00, 3.01260E+01, 4.6451E+00, 1.1412E+03, &
      2.6430894E+00, 3.01280E+01, 4.6080E+00, 1.1266E+03, &
      2.6563048E+00, 3.01297E+01, 4.5713E+00, 1.1121E+03, &
      2.6695863E+00, 3.01313E+01, 4.5348E+00, 1.0977E+03, &
      2.6829343E+00, 3.01328E+01, 4.4987E+00, 1.0836E+03, &
      2.6963489E+00, 3.01341E+01, 4.4629E+00, 1.0697E+03, &
      2.7098307E+00, 3.01353E+01, 4.4273E+00, 1.0559E+03, &
      2.7233798E+00, 3.01363E+01, 4.3921E+00, 1.0424E+03, &
      2.7369967E+00, 3.01372E+01, 4.3571E+00, 1.0290E+03, &
      2.7506817E+00, 3.01379E+01, 4.3225E+00, 1.0158E+03, &
      2.7644351E+00, 3.01386E+01, 4.2881E+00, 1.0027E+03, &
      2.7782573E+00, 3.01391E+01, 4.2541E+00, 9.8987E+02, &
      2.7921486E+00, 3.01395E+01, 4.2203E+00, 9.7717E+02, &
      2.8061093E+00, 3.01397E+01, 4.1868E+00, 9.6464E+02, &
      2.8201399E+00, 3.01399E+01, 4.1536E+00, 9.5227E+02, &
      2.8342406E+00, 3.01400E+01, 4.1206E+00, 9.4006E+02, &
      2.8484118E+00, 3.01400E+01, 4.0880E+00, 9.2802E+02, &
      2.8626539E+00, 3.01399E+01, 4.0556E+00, 9.1613E+02, &
      2.8769671E+00, 3.01397E+01, 4.0235E+00, 9.0440E+02, &
      2.8913520E+00, 3.01394E+01, 3.9916E+00, 8.9283E+02, &
      2.9058087E+00, 3.01391E+01, 3.9601E+00, 8.8141E+02, &
      2.9203378E+00, 3.01387E+01, 3.9288E+00, 8.7013E+02, &
      2.9349394E+00, 3.01382E+01, 3.8977E+00, 8.5901E+02, &
      2.9496141E+00, 3.01377E+01, 3.8669E+00, 8.4803E+02, &
      2.9643622E+00, 3.01372E+01, 3.8364E+00, 8.3720E+02, &
      2.9791840E+00, 3.01366E+01, 3.8062E+00, 8.2651E+02, &
      2.9940799E+00, 3.01360E+01, 3.7762E+00, 8.1596E+02, &
      3.0090503E+00, 3.01369E+01, 3.7445E+00, 8.0514E+02, &
      3.0240956E+00, 3.01381E+01, 3.7119E+00, 7.9420E+02, &
      3.0392161E+00, 3.01387E+01, 3.6796E+00, 7.8342E+02, &
      3.0544122E+00, 3.01388E+01, 3.6475E+00, 7.7278E+02, &
      3.0696842E+00, 3.01383E+01, 3.6158E+00, 7.6230E+02, &
      3.0850326E+00, 3.01375E+01, 3.5844E+00, 7.5196E+02, &
      3.1004578E+00, 3.01364E+01, 3.5532E+00, 7.4176E+02, &
      3.1159601E+00, 3.01349E+01, 3.5224E+00, 7.3171E+02, &
      3.1315399E+00, 3.01331E+01, 3.4918E+00, 7.2180E+02, &
      3.1471976E+00, 3.01312E+01, 3.4615E+00, 7.1202E+02, &
      3.1629336E+00, 3.01289E+01, 3.4315E+00, 7.0238E+02, &
      3.1787482E+00, 3.01265E+01, 3.4018E+00, 6.9288E+02, &
      3.1946420E+00, 3.01239E+01, 3.3723E+00, 6.8351E+02, &
      3.2106152E+00, 3.01211E+01, 3.3432E+00, 6.7427E+02, &
      3.2266683E+00, 3.01181E+01, 3.3142E+00, 6.6516E+02, &
      3.2428016E+00, 3.01149E+01, 3.2856E+00, 6.5617E+02, &
      3.2590156E+00, 3.01116E+01, 3.2572E+00, 6.4731E+02, &
      3.2753107E+00, 3.01082E+01, 3.2291E+00, 6.3857E+02, &
      3.2916873E+00, 3.01046E+01, 3.2012E+00, 6.2996E+02, &
      3.3081457E+00, 3.01009E+01, 3.1736E+00, 6.2146E+02, &
      3.3246864E+00, 3.00970E+01, 3.1462E+00, 6.1308E+02, &
      3.3413099E+00, 3.00930E+01, 3.1191E+00, 6.0482E+02, &
      3.3580164E+00, 3.00890E+01, 3.0923E+00, 5.9667E+02, &
      3.3748065E+00, 3.00848E+01, 3.0657E+00, 5.8864E+02, &
      3.3916805E+00, 3.00805E+01, 3.0393E+00, 5.8072E+02, &
      3.4086389E+00, 3.00761E+01, 3.0132E+00, 5.7290E+02, &
      3.4256821E+00, 3.00716E+01, 2.9873E+00, 5.6520E+02, &
      3.4428105E+00, 3.00670E+01, 2.9617E+00, 5.5760E+02, &
      3.4600246E+00, 3.00624E+01, 2.9362E+00, 5.5011E+02, &
      3.4773247E+00, 3.00576E+01, 2.9111E+00, 5.4272E+02, &
      3.4947113E+00, 3.00528E+01, 2.8861E+00, 5.3543E+02, &
      3.5121849E+00, 3.00479E+01, 2.8614E+00, 5.2825E+02, &
      3.5297458E+00, 3.00429E+01, 2.8369E+00, 5.2116E+02, &
      3.5473945E+00, 3.00379E+01, 2.8126E+00, 5.1417E+02, &
      3.5651315E+00, 3.00327E+01, 2.7886E+00, 5.0728E+02, &
      3.5829572E+00, 3.00276E+01, 2.7647E+00, 5.0049E+02, &
      3.6008719E+00, 3.00223E+01, 2.7411E+00, 4.9378E+02, &
      3.6188763E+00, 3.00170E+01, 2.7177E+00, 4.8717E+02, &
      3.6369707E+00, 3.00116E+01, 2.6946E+00, 4.8066E+02, &
      3.6551555E+00, 3.00062E+01, 2.6716E+00, 4.7423E+02, &
      3.6734313E+00, 3.00007E+01, 2.6488E+00, 4.6789E+02, &
      3.6917985E+00, 2.99952E+01, 2.6263E+00, 4.6163E+02, &
      3.7102575E+00, 2.99896E+01, 2.6039E+00, 4.5546E+02, &
      3.7288088E+00, 2.99840E+01, 2.5817E+00, 4.4938E+02, &
      3.7474528E+00, 2.99783E+01, 2.5598E+00, 4.4338E+02, &
      3.7661901E+00, 2.99726E+01, 2.5380E+00, 4.3746E+02, &
      3.7850210E+00, 2.99668E+01, 2.5164E+00, 4.3163E+02, &
      3.8039461E+00, 2.99610E+01, 2.4951E+00, 4.2587E+02, &
      3.8229659E+00, 2.99551E+01, 2.4739E+00, 4.2019E+02, &
      3.8420807E+00, 2.99492E+01, 2.4529E+00, 4.1460E+02, &
      3.8612911E+00, 2.99433E+01, 2.4321E+00, 4.0907E+02, &
      3.8805975E+00, 2.99374E+01, 2.4115E+00, 4.0363E+02, &
      3.9000005E+00, 2.99314E+01, 2.3911E+00, 3.9826E+02, &
      3.9195005E+00, 2.99253E+01, 2.3709E+00, 3.9296E+02, &
      3.9390980E+00, 2.99193E+01, 2.3508E+00, 3.8774E+02, &
      3.9587935E+00, 2.99132E+01, 2.3310E+00, 3.8259E+02, &
      3.9785875E+00, 2.99071E+01, 2.3113E+00, 3.7751E+02, &
      3.9984804E+00, 2.99009E+01, 2.2918E+00, 3.7249E+02, &
      4.0184728E+00, 2.98948E+01, 2.2724E+00, 3.6755E+02, &
      4.0385652E+00, 2.98886E+01, 2.2533E+00, 3.6268E+02, &
      4.0587580E+00, 2.98824E+01, 2.2343E+00, 3.5787E+02, &
      4.0790518E+00, 2.98762E+01, 2.2155E+00, 3.5313E+02, &
      4.0994471E+00, 2.98699E+01, 2.1969E+00, 3.4845E+02, &
      4.1199443E+00, 2.98636E+01, 2.1784E+00, 3.4384E+02, &
      4.1405440E+00, 2.98574E+01, 2.1601E+00, 3.3929E+02, &
      4.1612467E+00, 2.98511E+01, 2.1419E+00, 3.3480E+02, &
      4.1820530E+00, 2.98447E+01, 2.1240E+00, 3.3037E+02, &
      4.2029632E+00, 2.98384E+01, 2.1062E+00, 3.2601E+02, &
      4.2239781E+00, 2.98321E+01, 2.0885E+00, 3.2170E+02, &
      4.2450980E+00, 2.98257E+01, 2.0710E+00, 3.1746E+02, &
      4.2663234E+00, 2.98194E+01, 2.0537E+00, 3.1327E+02, &
      4.2876551E+00, 2.98130E+01, 2.0365E+00, 3.0913E+02, &
      4.3090933E+00, 2.98066E+01, 2.0195E+00, 3.0506E+02, &
      4.3306388E+00, 2.98003E+01, 2.0026E+00, 3.0104E+02, &
      4.3522920E+00, 2.97939E+01, 1.9859E+00, 2.9707E+02, &
      4.3740535E+00, 2.97875E+01, 1.9693E+00, 2.9316E+02, &
      4.3959237E+00, 2.97811E+01, 1.9529E+00, 2.8931E+02, &
      4.4179033E+00, 2.97748E+01, 1.9366E+00, 2.8550E+02, &
      4.4399929E+00, 2.97684E+01, 1.9198E+00, 2.8165E+02, &
      4.4621928E+00, 2.97620E+01, 1.9032E+00, 2.7785E+02, &
      4.4845038E+00, 2.97555E+01, 1.8867E+00, 2.7411E+02, &
      4.5069263E+00, 2.97490E+01, 1.8703E+00, 2.7042E+02, &
      4.5294609E+00, 2.97424E+01, 1.8540E+00, 2.6676E+02, &
      4.5521082E+00, 2.97358E+01, 1.8376E+00, 2.6312E+02, &
      4.5748688E+00, 2.97291E+01, 1.8214E+00, 2.5953E+02, &
      4.5977431E+00, 2.97224E+01, 1.8054E+00, 2.5600E+02, &
      4.6207318E+00, 2.97156E+01, 1.7895E+00, 2.5252E+02, &
      4.6438355E+00, 2.97087E+01, 1.7737E+00, 2.4908E+02, &
      4.6670547E+00, 2.97018E+01, 1.7581E+00, 2.4570E+02, &
      4.6903900E+00, 2.96949E+01, 1.7427E+00, 2.4236E+02, &
      4.7138419E+00, 2.96879E+01, 1.7274E+00, 2.3907E+02, &
      4.7374111E+00, 2.96809E+01, 1.7123E+00, 2.3583E+02, &
      4.7610982E+00, 2.96738E+01, 1.6973E+00, 2.3263E+02, &
      4.7849037E+00, 2.96667E+01, 1.6824E+00, 2.2948E+02, &
      4.8088282E+00, 2.96595E+01, 1.6677E+00, 2.2638E+02, &
      4.8328723E+00, 2.96523E+01, 1.6531E+00, 2.2331E+02, &
      4.8570367E+00, 2.96451E+01, 1.6387E+00, 2.2029E+02, &
      4.8813219E+00, 2.96379E+01, 1.6244E+00, 2.1732E+02, &
      4.9057285E+00, 2.96306E+01, 1.6102E+00, 2.1438E+02, &
      4.9302571E+00, 2.96233E+01, 1.5962E+00, 2.1149E+02, &
      4.9549084E+00, 2.96160E+01, 1.5823E+00, 2.0864E+02, &
      4.9796829E+00, 2.96086E+01, 1.5686E+00, 2.0583E+02, &
      5.0045814E+00, 2.96012E+01, 1.5550E+00, 2.0306E+02, &
      5.0296043E+00, 2.95938E+01, 1.5415E+00, 2.0032E+02, &
      5.0547523E+00, 2.95864E+01, 1.5281E+00, 1.9763E+02, &
      5.0800260E+00, 2.95789E+01, 1.5149E+00, 1.9497E+02, &
      5.1054262E+00, 2.95714E+01, 1.5018E+00, 1.9235E+02, &
      5.1309533E+00, 2.95639E+01, 1.4888E+00, 1.8977E+02, &
      5.1566081E+00, 2.95564E+01, 1.4759E+00, 1.8722E+02, &
      5.1823911E+00, 2.95489E+01, 1.4630E+00, 1.8469E+02, &
      5.2083031E+00, 2.95414E+01, 1.4498E+00, 1.8214E+02, &
      5.2343446E+00, 2.95337E+01, 1.4367E+00, 1.7963E+02, &
      5.2605163E+00, 2.95260E+01, 1.4238E+00, 1.7716E+02, &
      5.2868189E+00, 2.95183E+01, 1.4109E+00, 1.7472E+02, &
      5.3132530E+00, 2.95105E+01, 1.3982E+00, 1.7231E+02, &
      5.3398192E+00, 2.95026E+01, 1.3857E+00, 1.6994E+02, &
      5.3665183E+00, 2.94947E+01, 1.3732E+00, 1.6761E+02, &
      5.3933509E+00, 2.94867E+01, 1.3609E+00, 1.6530E+02, &
      5.4203177E+00, 2.94787E+01, 1.3487E+00, 1.6303E+02, &
      5.4474193E+00, 2.94706E+01, 1.3366E+00, 1.6080E+02, &
      5.4746564E+00, 2.94625E+01, 1.3246E+00, 1.5859E+02, &
      5.5020297E+00, 2.94543E+01, 1.3127E+00, 1.5642E+02, &
      5.5295398E+00, 2.94461E+01, 1.3010E+00, 1.5427E+02, &
      5.5571875E+00, 2.94378E+01, 1.2893E+00, 1.5216E+02, &
      5.5849734E+00, 2.94294E+01, 1.2778E+00, 1.5007E+02, &
      5.6128983E+00, 2.94210E+01, 1.2664E+00, 1.4802E+02, &
      5.6409628E+00, 2.94126E+01, 1.2550E+00, 1.4600E+02, &
      5.6691676E+00, 2.94041E+01, 1.2438E+00, 1.4400E+02, &
      5.6975135E+00, 2.93955E+01, 1.2327E+00, 1.4203E+02, &
      5.7260010E+00, 2.93870E+01, 1.2217E+00, 1.4009E+02, &
      5.7546310E+00, 2.93783E+01, 1.2109E+00, 1.3818E+02, &
      5.7834042E+00, 2.93696E+01, 1.2001E+00, 1.3629E+02, &
      5.8123212E+00, 2.93608E+01, 1.1894E+00, 1.3444E+02, &
      5.8413828E+00, 2.93520E+01, 1.1788E+00, 1.3260E+02, &
      5.8705897E+00, 2.93432E+01, 1.1683E+00, 1.3080E+02, &
      5.8999427E+00, 2.93342E+01, 1.1579E+00, 1.2902E+02, &
      5.9294424E+00, 2.93253E+01, 1.1477E+00, 1.2726E+02, &
      5.9590896E+00, 2.93162E+01, 1.1375E+00, 1.2553E+02, &
      5.9888850E+00, 2.93071E+01, 1.1274E+00, 1.2382E+02, &
      6.0188295E+00, 2.92980E+01, 1.1174E+00, 1.2214E+02, &
      6.0489236E+00, 2.92887E+01, 1.1075E+00, 1.2048E+02, &
      6.0791682E+00, 2.92795E+01, 1.0977E+00, 1.1884E+02, &
      6.1095641E+00, 2.92701E+01, 1.0880E+00, 1.1723E+02, &
      6.1401119E+00, 2.92607E+01, 1.0784E+00, 1.1564E+02, &
      6.1708125E+00, 2.92512E+01, 1.0688E+00, 1.1407E+02, &
      6.2016665E+00, 2.92416E+01, 1.0594E+00, 1.1253E+02, &
      6.2326749E+00, 2.92320E+01, 1.0500E+00, 1.1100E+02, &
      6.2638382E+00, 2.92223E+01, 1.0408E+00, 1.0950E+02, &
      6.2951574E+00, 2.92125E+01, 1.0316E+00, 1.0802E+02, &
      6.3266332E+00, 2.92027E+01, 1.0225E+00, 1.0656E+02, &
      6.3582664E+00, 2.91928E+01, 1.0135E+00, 1.0512E+02, &
      6.3900577E+00, 2.91827E+01, 1.0046E+00, 1.0370E+02, &
      6.4220080E+00, 2.91726E+01, 9.9576E-01, 1.0230E+02, &
      6.4541180E+00, 2.91624E+01, 9.8700E-01, 1.0092E+02, &
      6.4863886E+00, 2.91522E+01, 9.7833E-01, 9.9558E+01, &
      6.5188206E+00, 2.91418E+01, 9.6975E-01, 9.8216E+01, &
      6.5514147E+00, 2.91313E+01, 9.6124E-01, 9.6892E+01, &
      6.5841717E+00, 2.91207E+01, 9.5281E-01, 9.5588E+01, &
      6.915365E+00, 2.90257E+01, 8.7397E-01, 8.3741E+01, &
      7.392525E+00, 2.88496E+01, 7.7780E-01, 6.9977E+01, &
      7.902609E+00, 2.86263E+01, 6.9289E-01, 5.8554E+01, &
      8.447890E+00, 2.83080E+01, 6.1784E-01, 4.9063E+01, &
      9.030794E+00, 2.77362E+01, 5.4929E-01, 4.1012E+01, &
      9.465428E+00, 2.66177E+01, 5.0093E-01, 3.5840E+01, &
      9.610307E+00, 2.52001E+01, 4.8628E-01, 3.4319E+01, &
      9.653919E+00, 2.27080E+01, 4.8201E-01, 3.3878E+01, &
      9.706893E+00, 2.52381E+01, 3.8531E+00, 2.5721E+02, &
      9.851772E+00, 2.68206E+01, 3.7401E+00, 2.4605E+02, &
      1.032004E+01, 2.82422E+01, 3.4219E+00, 2.1504E+02, &
      1.103212E+01, 2.90456E+01, 3.0610E+00, 1.8010E+02, &
      1.179334E+01, 2.94892E+01, 2.7556E+00, 1.5180E+02, &
      1.260708E+01, 2.97882E+01, 2.4851E+00, 1.2818E+02, &
      1.347697E+01, 2.99968E+01, 2.2238E+00, 1.0742E+02, &
      1.440688E+01, 3.01393E+01, 1.9866E+00, 8.9884E+01, &
      1.540095E+01, 3.02370E+01, 1.7721E+00, 7.5106E+01, &
      1.646362E+01, 3.03025E+01, 1.5790E+00, 6.2701E+01, &
      1.759961E+01, 3.03446E+01, 1.4062E+00, 5.2326E+01, &
      1.881398E+01, 3.03700E+01, 1.2523E+00, 4.3672E+01, &
      2.011215E+01, 3.04195E+01, 1.1123E+00, 3.6365E+01, &
      2.149988E+01, 3.04206E+01, 9.8624E-01, 3.0237E+01, &
      2.298338E+01, 3.04141E+01, 8.7440E-01, 2.5147E+01, &
      2.456923E+01, 3.04028E+01, 7.7527E-01, 2.0920E+01, &
      2.626450E+01, 3.03892E+01, 6.8720E-01, 1.7406E+01, &
      2.807676E+01, 3.03746E+01, 6.0610E-01, 1.4419E+01, &
      3.001405E+01, 3.03546E+01, 5.3381E-01, 1.1934E+01, &
      3.208502E+01, 3.03335E+01, 4.7016E-01, 9.8838E+00, &
      3.429889E+01, 3.03120E+01, 4.1413E-01, 8.1921E+00, &
      3.666551E+01, 3.02907E+01, 3.6481E-01, 6.7958E+00, &
      3.919543E+01, 3.02701E+01, 3.2137E-01, 5.6431E+00, &
      4.189992E+01, 3.02505E+01, 2.8309E-01, 4.6907E+00, &
      4.479101E+01, 3.02315E+01, 2.4795E-01, 3.8834E+00, &
      4.788159E+01, 3.02130E+01, 2.1718E-01, 3.2203E+00, &
      5.118542E+01, 3.01953E+01, 1.9024E-01, 2.6754E+00, &
      5.471721E+01, 3.01786E+01, 1.6665E-01, 2.2274E+00, &
      5.849270E+01, 3.01630E+01, 1.4599E-01, 1.8588E+00, &
      6.252870E+01, 3.01484E+01, 1.2789E-01, 1.5556E+00, &
      6.684318E+01, 3.01350E+01, 1.1203E-01, 1.3058E+00, &
      7.145536E+01, 3.01225E+01, 9.8139E-02, 1.1001E+00, &
      7.638578E+01, 3.01111E+01, 8.5974E-02, 9.3060E-01, &
      8.165640E+01, 3.01011E+01, 7.5223E-02, 7.9002E-01, &
      8.729069E+01, 3.00916E+01, 6.5631E-02, 6.7271E-01, &
      9.331374E+01, 3.00827E+01, 5.7264E-02, 5.7616E-01, &
      9.975239E+01, 3.00746E+01, 4.9966E-02, 4.9657E-01, &
      1.066353E+02, 3.00672E+01, 4.3599E-02, 4.3087E-01, &
      1.139931E+02, 3.00605E+01, 3.8045E-02, 3.7655E-01, &
      1.218587E+02, 3.00544E+01, 3.3199E-02, 3.3154E-01, &
      1.302669E+02, 3.00488E+01, 2.8972E-02, 2.9414E-01, &
      1.392553E+02, 3.00438E+01, 2.5283E-02, 2.6300E-01, &
      1.488640E+02, 3.00393E+01, 2.2065E-02, 2.3695E-01, &
      1.591356E+02, 3.00352E+01, 1.9257E-02, 2.1511E-01, &
      1.701159E+02, 3.00315E+01, 1.6807E-02, 1.9669E-01, &
      1.818539E+02, 3.00282E+01, 1.4669E-02, 1.8110E-01, &
      1.944018E+02, 3.00252E+01, 1.2803E-02, 1.6780E-01, &
      2.078156E+02, 3.00225E+01, 1.1175E-02, 1.5640E-01, &
      2.221548E+02, 3.00201E+01, 9.7545E-03, 1.4655E-01, &
      2.374835E+02, 3.00180E+01, 8.5146E-03, 1.3799E-01, &
      2.538699E+02, 3.00160E+01, 7.4325E-03, 1.3046E-01, &
      2.713869E+02, 3.00143E+01, 6.4882E-03, 1.2381E-01, &
      2.901126E+02, 3.00128E+01, 5.6639E-03, 1.1787E-01, &
      3.101304E+02, 3.00114E+01, 4.9445E-03, 1.1251E-01, &
      3.315294E+02, 3.00101E+01, 4.3166E-03, 1.0764E-01, &
      3.544049E+02, 3.00090E+01, 3.7685E-03, 1.0318E-01, &
      3.788588E+02, 3.00080E+01, 3.2901E-03, 9.9052E-02, &
      4.050001E+02, 3.00071E+01, 2.8725E-03, 9.5210E-02, &
      4.329451E+02, 3.00064E+01, 2.5080E-03, 9.1606E-02]
   case(31)
      frel = -1.0320E-01
      allocate(nist_tab(2236))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 3.15198E+00, 0.0000E+00, 9.6976E-05, &
      1.142761E-02, 3.09183E+00, 0.0000E+00, 1.3376E-04, &
      1.221612E-02, 3.01884E+00, 0.0000E+00, 1.8345E-04, &
      1.305903E-02, 2.92860E+00, 0.0000E+00, 2.5019E-04, &
      1.396010E-02, 2.81368E+00, 0.0000E+00, 3.3933E-04, &
      1.492335E-02, 2.65950E+00, 0.0000E+00, 4.5768E-04, &
      1.595306E-02, 2.42810E+00, 0.0000E+00, 6.1392E-04, &
      1.705382E-02, 1.90191E+00, 0.0000E+00, 8.1901E-04, &
      1.731300E-02, 1.45080E+00, 0.0000E+00, 8.7349E-04, &
      1.738260E-02, 9.45019E-01, 0.0000E+00, 8.8854E-04, &
      1.748700E-02, 1.43339E+00, 9.7516E-01, 3.3658E+04, &
      1.774800E-02, 1.83886E+00, 9.7958E-01, 3.3313E+04, &
      1.823053E-02, 2.06446E+00, 9.8763E-01, 3.2698E+04, &
      1.948844E-02, 2.22367E+00, 1.0092E+00, 3.1256E+04, &
      2.083314E-02, 2.21897E+00, 1.0578E+00, 3.0645E+04, &
      2.227063E-02, 2.15214E+00, 1.1418E+00, 3.0944E+04, &
      2.380730E-02, 2.05080E+00, 1.2615E+00, 3.1981E+04, &
      2.545001E-02, 1.92725E+00, 1.4188E+00, 3.3647E+04, &
      2.720606E-02, 1.78929E+00, 1.6168E+00, 3.5869E+04, &
      2.908327E-02, 1.64356E+00, 1.8592E+00, 3.8585E+04, &
      3.109002E-02, 1.49683E+00, 2.1500E+00, 4.1738E+04, &
      3.323523E-02, 1.35666E+00, 2.4925E+00, 4.5264E+04, &
      3.552846E-02, 1.23162E+00, 2.8896E+00, 4.9089E+04, &
      3.797993E-02, 1.13137E+00, 3.3429E+00, 5.3124E+04, &
      4.060054E-02, 1.06643E+00, 3.8522E+00, 5.7267E+04, &
      4.340198E-02, 1.04790E+00, 4.4153E+00, 6.1401E+04, &
      4.639671E-02, 1.08696E+00, 5.0277E+00, 6.5403E+04, &
      4.959809E-02, 1.19432E+00, 5.6822E+00, 6.9146E+04, &
      5.302035E-02, 1.38877E+00, 6.3693E+00, 7.2505E+04, &
      5.667876E-02, 1.66014E+00, 7.0772E+00, 7.5364E+04, &
      6.058959E-02, 2.02335E+00, 7.7923E+00, 7.7623E+04, &
      6.477028E-02, 2.48203E+00, 8.4996E+00, 7.9203E+04, &
      6.923942E-02, 3.03821E+00, 9.1832E+00, 8.0050E+04, &
      7.401695E-02, 3.69108E+00, 9.8181E+00, 8.0061E+04, &
      7.912411E-02, 4.42213E+00, 1.0377E+01, 7.9152E+04, &
      8.458368E-02, 5.20610E+00, 1.0846E+01, 7.7393E+04, &
      9.041995E-02, 6.01082E+00, 1.1218E+01, 7.4879E+04, &
      9.665893E-02, 6.77799E+00, 1.1487E+01, 7.1726E+04, &
      1.008420E-01, 7.14467E+00, 1.1604E+01, 6.9450E+04, &
      1.023855E-01, 7.11392E+00, 1.1635E+01, 6.8590E+04, &
      1.027971E-01, 6.91461E+00, 1.1643E+01, 6.8359E+04, &
      1.033284E-01, 7.18608E+00, 1.2131E+01, 7.0857E+04, &
      1.034145E-01, 7.22281E+00, 1.2133E+01, 7.0810E+04, &
      1.046640E-01, 7.53327E+00, 1.2159E+01, 7.0117E+04, &
      1.049580E-01, 7.58273E+00, 1.2165E+01, 6.9953E+04, &
      1.062660E-01, 7.72946E+00, 1.2188E+01, 6.9224E+04, &
      1.066932E-01, 7.68165E+00, 1.2195E+01, 6.8985E+04, &
      1.073340E-01, 7.90187E+00, 1.2437E+01, 6.9934E+04, &
      1.089360E-01, 8.24646E+00, 1.2461E+01, 6.9041E+04, &
      1.104581E-01, 8.50418E+00, 1.2480E+01, 6.8192E+04, &
      1.180797E-01, 9.56869E+00, 1.2517E+01, 6.3982E+04, &
      1.262272E-01, 1.05176E+01, 1.2480E+01, 5.9676E+04, &
      1.349368E-01, 1.13901E+01, 1.2380E+01, 5.5376E+04, &
      1.442475E-01, 1.21765E+01, 1.2225E+01, 5.1153E+04, &
      1.542005E-01, 1.27848E+01, 1.2025E+01, 4.7067E+04, &
      1.549380E-01, 1.28065E+01, 1.2009E+01, 4.6781E+04, &
      1.573095E-01, 1.27561E+01, 1.1957E+01, 4.5877E+04, &
      1.579419E-01, 1.25398E+01, 1.1943E+01, 4.5640E+04, &
      1.588905E-01, 1.28669E+01, 1.2434E+01, 4.7232E+04, &
      1.612620E-01, 1.32580E+01, 1.2388E+01, 4.6365E+04, &
      1.648404E-01, 1.36262E+01, 1.2317E+01, 4.5099E+04, &
      1.762144E-01, 1.45147E+01, 1.2084E+01, 4.1392E+04, &
      1.883732E-01, 1.52959E+01, 1.1827E+01, 3.7896E+04, &
      2.013709E-01, 1.60188E+01, 1.1548E+01, 3.4613E+04, &
      2.152655E-01, 1.66967E+01, 1.1251E+01, 3.1546E+04, &
      2.301188E-01, 1.73360E+01, 1.0939E+01, 2.8691E+04, &
      2.459970E-01, 1.79414E+01, 1.0614E+01, 2.6044E+04, &
      2.629708E-01, 1.85163E+01, 1.0279E+01, 2.3595E+04, &
      2.811158E-01, 1.90643E+01, 9.9349E+00, 2.1333E+04, &
      3.005128E-01, 1.95895E+01, 9.5828E+00, 1.9248E+04, &
      3.212482E-01, 2.00952E+01, 9.2241E+00, 1.7332E+04, &
      3.434143E-01, 2.05727E+01, 8.7314E+00, 1.5348E+04, &
      3.671099E-01, 2.09634E+01, 8.2053E+00, 1.3492E+04, &
      3.924405E-01, 2.12796E+01, 7.6944E+00, 1.1837E+04, &
      4.195189E-01, 2.15316E+01, 7.2003E+00, 1.0362E+04, &
      4.484657E-01, 2.17256E+01, 6.7227E+00, 9.0506E+03, &
      4.794098E-01, 2.18639E+01, 6.2622E+00, 7.8869E+03, &
      5.124891E-01, 2.19465E+01, 5.8209E+00, 6.8585E+03, &
      5.478508E-01, 2.19737E+01, 5.4005E+00, 5.9531E+03, &
      5.856525E-01, 2.19438E+01, 5.0023E+00, 5.1588E+03, &
      6.260625E-01, 2.18530E+01, 4.6270E+00, 4.4644E+03, &
      6.692609E-01, 2.16937E+01, 4.2747E+00, 3.8590E+03, &
      7.154399E-01, 2.14528E+01, 3.9455E+00, 3.3325E+03, &
      7.648052E-01, 2.11076E+01, 3.6389E+00, 2.8759E+03, &
      8.175768E-01, 2.06181E+01, 3.3545E+00, 2.4807E+03, &
      8.739896E-01, 1.99078E+01, 3.0916E+00, 2.1394E+03, &
      9.0000000E-01, 2.04665E+01, 2.9826E+00, 2.0046E+03, &
      9.0450000E-01, 2.04043E+01, 2.9644E+00, 1.9826E+03, &
      9.0902250E-01, 2.03401E+01, 2.9464E+00, 1.9608E+03, &
      9.1356761E-01, 2.02736E+01, 2.9284E+00, 1.9392E+03, &
      9.1813545E-01, 2.02050E+01, 2.9106E+00, 1.9179E+03, &
      9.2272613E-01, 2.01339E+01, 2.8929E+00, 1.8968E+03, &
      9.2733976E-01, 2.00604E+01, 2.8753E+00, 1.8759E+03, &
      9.3197646E-01, 1.99842E+01, 2.8578E+00, 1.8553E+03, &
      9.3663634E-01, 1.99053E+01, 2.8404E+00, 1.8349E+03, &
      9.4131952E-01, 1.98233E+01, 2.8232E+00, 1.8147E+03, &
      9.4602612E-01, 1.97383E+01, 2.8060E+00, 1.7948E+03, &
      9.5075625E-01, 1.96499E+01, 2.7890E+00, 1.7750E+03, &
      9.5551003E-01, 1.95580E+01, 2.7720E+00, 1.7555E+03, &
      9.6028758E-01, 1.94622E+01, 2.7552E+00, 1.7362E+03, &
      9.6508902E-01, 1.93625E+01, 2.7384E+00, 1.7172E+03, &
      9.6991446E-01, 1.92584E+01, 2.7218E+00, 1.6983E+03, &
      9.7476404E-01, 1.91497E+01, 2.7053E+00, 1.6797E+03, &
      9.7963786E-01, 1.90360E+01, 2.6889E+00, 1.6612E+03, &
      9.8453605E-01, 1.89169E+01, 2.6725E+00, 1.6430E+03, &
      9.8945873E-01, 1.87920E+01, 2.6563E+00, 1.6250E+03, &
      9.9440602E-01, 1.86608E+01, 2.6402E+00, 1.6071E+03, &
      9.9937805E-01, 1.85227E+01, 2.6242E+00, 1.5895E+03, &
      1.0043749E+00, 1.83744E+01, 2.6042E+00, 1.5696E+03, &
      1.0093968E+00, 1.82171E+01, 2.5837E+00, 1.5496E+03, &
      1.0144438E+00, 1.80503E+01, 2.5634E+00, 1.5298E+03, &
      1.0195160E+00, 1.78730E+01, 2.5434E+00, 1.5103E+03, &
      1.0246136E+00, 1.76841E+01, 2.5234E+00, 1.4911E+03, &
      1.0297367E+00, 1.74821E+01, 2.5037E+00, 1.4722E+03, &
      1.0348853E+00, 1.72655E+01, 2.4842E+00, 1.4535E+03, &
      1.0400598E+00, 1.70322E+01, 2.4648E+00, 1.4350E+03, &
      1.0452601E+00, 1.67800E+01, 2.4456E+00, 1.4168E+03, &
      1.0504864E+00, 1.65058E+01, 2.4266E+00, 1.3989E+03, &
      1.0557388E+00, 1.62061E+01, 2.4077E+00, 1.3812E+03, &
      1.0610175E+00, 1.58760E+01, 2.3890E+00, 1.3637E+03, &
      1.0663226E+00, 1.55096E+01, 2.3705E+00, 1.3465E+03, &
      1.0716542E+00, 1.50984E+01, 2.3521E+00, 1.3294E+03, &
      1.0770125E+00, 1.46313E+01, 2.3339E+00, 1.3127E+03, &
      1.0823975E+00, 1.40917E+01, 2.3159E+00, 1.2961E+03, &
      1.0878095E+00, 1.34544E+01, 2.2981E+00, 1.2798E+03, &
      1.0932486E+00, 1.26780E+01, 2.2803E+00, 1.2637E+03, &
      1.0987148E+00, 1.16858E+01, 2.2628E+00, 1.2478E+03, &
      1.1042084E+00, 1.03097E+01, 2.2454E+00, 1.2321E+03, &
      1.1097294E+00, 8.02547E+00, 2.2282E+00, 1.2166E+03, &
      1.1152781E+00, -4.27780E+00, 2.2111E+00, 1.2013E+03, &
      1.1153150E+00, -5.41991E+00, 2.2110E+00, 1.2012E+03, &
      1.1154850E+00, -5.58976E+00, 1.2157E+01, 6.5824E+03, &
      1.1208545E+00, 7.38309E+00, 1.2061E+01, 6.4992E+03, &
      1.1264587E+00, 9.26436E+00, 1.1962E+01, 6.4139E+03, &
      1.1320910E+00, 9.98691E+00, 1.1864E+01, 6.3297E+03, &
      1.1377515E+00, 9.75120E+00, 1.1766E+01, 6.2466E+03, &
      1.1422130E+00, 4.23877E+00, 1.1690E+01, 6.1822E+03, &
      1.1423871E+00, 4.18116E+00, 1.6591E+01, 8.7704E+03, &
      1.1434402E+00, 8.34121E+00, 1.6566E+01, 8.7491E+03, &
      1.1491574E+00, 1.18240E+01, 1.6430E+01, 8.6344E+03, &
      1.1549032E+00, 1.33476E+01, 1.6296E+01, 8.5213E+03, &
      1.1606777E+00, 1.44324E+01, 1.6163E+01, 8.4096E+03, &
      1.1664811E+00, 1.52974E+01, 1.6031E+01, 8.2994E+03, &
      1.1723135E+00, 1.60239E+01, 1.5900E+01, 8.1907E+03, &
      1.1781751E+00, 1.66524E+01, 1.5770E+01, 8.0834E+03, &
      1.1840660E+00, 1.72067E+01, 1.5641E+01, 7.9775E+03, &
      1.1899863E+00, 1.77023E+01, 1.5513E+01, 7.8730E+03, &
      1.1959362E+00, 1.81496E+01, 1.5386E+01, 7.7699E+03, &
      1.2019159E+00, 1.85562E+01, 1.5261E+01, 7.6681E+03, &
      1.2079255E+00, 1.89278E+01, 1.5136E+01, 7.5677E+03, &
      1.2139651E+00, 1.92684E+01, 1.5012E+01, 7.4686E+03, &
      1.2200350E+00, 1.95812E+01, 1.4890E+01, 7.3709E+03, &
      1.2261351E+00, 1.98687E+01, 1.4768E+01, 7.2744E+03, &
      1.2322658E+00, 2.01326E+01, 1.4648E+01, 7.1792E+03, &
      1.2384271E+00, 2.03739E+01, 1.4528E+01, 7.0852E+03, &
      1.2446193E+00, 2.05931E+01, 1.4409E+01, 6.9925E+03, &
      1.2508424E+00, 2.07900E+01, 1.4292E+01, 6.9010E+03, &
      1.2570966E+00, 2.09633E+01, 1.4175E+01, 6.8107E+03, &
      1.2633821E+00, 2.11106E+01, 1.4060E+01, 6.7216E+03, &
      1.2696990E+00, 2.12267E+01, 1.3945E+01, 6.6337E+03, &
      1.2760475E+00, 2.13018E+01, 1.3831E+01, 6.5469E+03, &
      1.2824277E+00, 2.13144E+01, 1.3718E+01, 6.4613E+03, &
      1.2888399E+00, 2.12054E+01, 1.3607E+01, 6.3768E+03, &
      1.2952840E+00, 2.06512E+01, 1.3496E+01, 6.2934E+03, &
      1.2964841E+00, 2.02896E+01, 1.3475E+01, 6.2781E+03, &
      1.2989160E+00, 2.03323E+01, 1.5253E+01, 7.0926E+03, &
      1.3017605E+00, 2.11138E+01, 1.5205E+01, 7.0547E+03, &
      1.3082693E+00, 2.18519E+01, 1.5095E+01, 6.9689E+03, &
      1.3148106E+00, 2.23083E+01, 1.4986E+01, 6.8842E+03, &
      1.3213847E+00, 2.26677E+01, 1.4878E+01, 6.8006E+03, &
      1.3279916E+00, 2.29755E+01, 1.4771E+01, 6.7181E+03, &
      1.3346316E+00, 2.32501E+01, 1.4665E+01, 6.6367E+03, &
      1.3413047E+00, 2.35011E+01, 1.4559E+01, 6.5562E+03, &
      1.3480112E+00, 2.37338E+01, 1.4455E+01, 6.4768E+03, &
      1.3547513E+00, 2.39522E+01, 1.4351E+01, 6.3984E+03, &
      1.3615250E+00, 2.41587E+01, 1.4248E+01, 6.3210E+03, &
      1.3683327E+00, 2.43549E+01, 1.4144E+01, 6.2437E+03, &
      1.3751743E+00, 2.45413E+01, 1.4041E+01, 6.1674E+03, &
      1.3820502E+00, 2.47192E+01, 1.3938E+01, 6.0919E+03, &
      1.3889605E+00, 2.48895E+01, 1.3836E+01, 6.0173E+03, &
      1.3959053E+00, 2.50529E+01, 1.3735E+01, 5.9436E+03, &
      1.4028848E+00, 2.52099E+01, 1.3635E+01, 5.8709E+03, &
      1.4098992E+00, 2.53612E+01, 1.3535E+01, 5.7990E+03, &
      1.4169487E+00, 2.55071E+01, 1.3436E+01, 5.7279E+03, &
      1.4240335E+00, 2.56479E+01, 1.3337E+01, 5.6578E+03, &
      1.4311536E+00, 2.57841E+01, 1.3240E+01, 5.5884E+03, &
      1.4383094E+00, 2.59158E+01, 1.3143E+01, 5.5199E+03, &
      1.4455009E+00, 2.60433E+01, 1.3046E+01, 5.4523E+03, &
      1.4527284E+00, 2.61669E+01, 1.2951E+01, 5.3855E+03, &
      1.4599921E+00, 2.62868E+01, 1.2856E+01, 5.3196E+03, &
      1.4672920E+00, 2.64031E+01, 1.2762E+01, 5.2545E+03, &
      1.4746285E+00, 2.65161E+01, 1.2669E+01, 5.1901E+03, &
      1.4820016E+00, 2.66260E+01, 1.2576E+01, 5.1266E+03, &
      1.4894117E+00, 2.67327E+01, 1.2484E+01, 5.0638E+03, &
      1.4968587E+00, 2.68366E+01, 1.2393E+01, 5.0018E+03, &
      1.5043430E+00, 2.69376E+01, 1.2302E+01, 4.9406E+03, &
      1.5118647E+00, 2.70359E+01, 1.2212E+01, 4.8801E+03, &
      1.5194240E+00, 2.71317E+01, 1.2123E+01, 4.8203E+03, &
      1.5270212E+00, 2.72249E+01, 1.2034E+01, 4.7613E+03, &
      1.5346563E+00, 2.73158E+01, 1.1946E+01, 4.7030E+03, &
      1.5423295E+00, 2.74043E+01, 1.1858E+01, 4.6454E+03, &
      1.5500412E+00, 2.74906E+01, 1.1772E+01, 4.5885E+03, &
      1.5577914E+00, 2.75748E+01, 1.1685E+01, 4.5324E+03, &
      1.5655804E+00, 2.76569E+01, 1.1600E+01, 4.4769E+03, &
      1.5734083E+00, 2.77370E+01, 1.1515E+01, 4.4221E+03, &
      1.5812753E+00, 2.78151E+01, 1.1431E+01, 4.3679E+03, &
      1.5891817E+00, 2.78914E+01, 1.1347E+01, 4.3145E+03, &
      1.5971276E+00, 2.79659E+01, 1.1264E+01, 4.2616E+03, &
      1.6051132E+00, 2.80387E+01, 1.1182E+01, 4.2095E+03, &
      1.6131388E+00, 2.81097E+01, 1.1100E+01, 4.1580E+03, &
      1.6212045E+00, 2.81791E+01, 1.1019E+01, 4.1071E+03, &
      1.6293105E+00, 2.82470E+01, 1.0938E+01, 4.0569E+03, &
      1.6374571E+00, 2.83133E+01, 1.0858E+01, 4.0072E+03, &
      1.6456443E+00, 2.83781E+01, 1.0779E+01, 3.9582E+03, &
      1.6538726E+00, 2.84415E+01, 1.0700E+01, 3.9098E+03, &
      1.6621419E+00, 2.85036E+01, 1.0622E+01, 3.8620E+03, &
      1.6704526E+00, 2.85643E+01, 1.0545E+01, 3.8148E+03, &
      1.6788049E+00, 2.86238E+01, 1.0468E+01, 3.7682E+03, &
      1.6871989E+00, 2.86822E+01, 1.0391E+01, 3.7221E+03, &
      1.6956349E+00, 2.87392E+01, 1.0315E+01, 3.6763E+03, &
      1.7041131E+00, 2.87948E+01, 1.0238E+01, 3.6311E+03, &
      1.7126337E+00, 2.88491E+01, 1.0163E+01, 3.5864E+03, &
      1.7211968E+00, 2.89022E+01, 1.0088E+01, 3.5423E+03, &
      1.7298028E+00, 2.89540E+01, 1.0013E+01, 3.4987E+03, &
      1.7384518E+00, 2.90047E+01, 9.9394E+00, 3.4557E+03, &
      1.7471441E+00, 2.90542E+01, 9.8661E+00, 3.4132E+03, &
      1.7558798E+00, 2.91026E+01, 9.7933E+00, 3.3712E+03, &
      1.7646592E+00, 2.91499E+01, 9.7210E+00, 3.3297E+03, &
      1.7734825E+00, 2.91962E+01, 9.6493E+00, 3.2888E+03, &
      1.7823499E+00, 2.92415E+01, 9.5782E+00, 3.2483E+03, &
      1.7912617E+00, 2.92858E+01, 9.5075E+00, 3.2084E+03, &
      1.8002180E+00, 2.93291E+01, 9.4374E+00, 3.1690E+03, &
      1.8092191E+00, 2.93715E+01, 9.3679E+00, 3.1300E+03, &
      1.8182652E+00, 2.94130E+01, 9.2988E+00, 3.0915E+03, &
      1.8273565E+00, 2.94536E+01, 9.2303E+00, 3.0535E+03, &
      1.8364933E+00, 2.94933E+01, 9.1622E+00, 3.0160E+03, &
      1.8456757E+00, 2.95322E+01, 9.0947E+00, 2.9789E+03, &
      1.8549041E+00, 2.95702E+01, 9.0277E+00, 2.9423E+03, &
      1.8641786E+00, 2.96075E+01, 8.9612E+00, 2.9062E+03, &
      1.8734995E+00, 2.96439E+01, 8.8952E+00, 2.8705E+03, &
      1.8828670E+00, 2.96796E+01, 8.8297E+00, 2.8352E+03, &
      1.8922814E+00, 2.97146E+01, 8.7646E+00, 2.8004E+03, &
      1.9017428E+00, 2.97488E+01, 8.7001E+00, 2.7660E+03, &
      1.9112515E+00, 2.97823E+01, 8.6360E+00, 2.7320E+03, &
      1.9208077E+00, 2.98151E+01, 8.5724E+00, 2.6985E+03, &
      1.9304118E+00, 2.98473E+01, 8.5093E+00, 2.6653E+03, &
      1.9400638E+00, 2.98787E+01, 8.4462E+00, 2.6324E+03, &
      1.9497642E+00, 2.99096E+01, 8.3834E+00, 2.5999E+03, &
      1.9595130E+00, 2.99397E+01, 8.3210E+00, 2.5678E+03, &
      1.9693105E+00, 2.99692E+01, 8.2591E+00, 2.5361E+03, &
      1.9791571E+00, 2.99981E+01, 8.1977E+00, 2.5048E+03, &
      1.9890529E+00, 3.00264E+01, 8.1368E+00, 2.4738E+03, &
      1.9989981E+00, 3.00541E+01, 8.0763E+00, 2.4433E+03, &
      2.0089931E+00, 3.00812E+01, 8.0162E+00, 2.4131E+03, &
      2.0190381E+00, 3.01077E+01, 7.9563E+00, 2.3832E+03, &
      2.0291333E+00, 3.01337E+01, 7.8969E+00, 2.3537E+03, &
      2.0392790E+00, 3.01591E+01, 7.8379E+00, 2.3245E+03, &
      2.0494754E+00, 3.01839E+01, 7.7793E+00, 2.2957E+03, &
      2.0597227E+00, 3.02083E+01, 7.7212E+00, 2.2673E+03, &
      2.0700213E+00, 3.02321E+01, 7.6636E+00, 2.2392E+03, &
      2.0803714E+00, 3.02554E+01, 7.6064E+00, 2.2115E+03, &
      2.0907733E+00, 3.02782E+01, 7.5496E+00, 2.1841E+03, &
      2.1012272E+00, 3.03006E+01, 7.4932E+00, 2.1571E+03, &
      2.1117333E+00, 3.03225E+01, 7.4373E+00, 2.1304E+03, &
      2.1222920E+00, 3.03439E+01, 7.3818E+00, 2.1040E+03, &
      2.1329034E+00, 3.03649E+01, 7.3268E+00, 2.0780E+03, &
      2.1435680E+00, 3.03855E+01, 7.2721E+00, 2.0523E+03, &
      2.1542858E+00, 3.04057E+01, 7.2179E+00, 2.0269E+03, &
      2.1650572E+00, 3.04255E+01, 7.1641E+00, 2.0018E+03, &
      2.1758825E+00, 3.04448E+01, 7.1107E+00, 1.9771E+03, &
      2.1867619E+00, 3.04639E+01, 7.0577E+00, 1.9526E+03, &
      2.1976957E+00, 3.04825E+01, 7.0051E+00, 1.9285E+03, &
      2.2086842E+00, 3.05008E+01, 6.9529E+00, 1.9047E+03, &
      2.2197276E+00, 3.05188E+01, 6.9011E+00, 1.8811E+03, &
      2.2308263E+00, 3.06455E+01, 6.8497E+00, 1.8579E+03, &
      2.2419804E+00, 3.06628E+01, 6.7967E+00, 1.8344E+03, &
      2.2531903E+00, 3.06796E+01, 6.7441E+00, 1.8112E+03, &
      2.2644562E+00, 3.06959E+01, 6.6920E+00, 1.7883E+03, &
      2.2757785E+00, 3.07117E+01, 6.6403E+00, 1.7657E+03, &
      2.2871574E+00, 3.07822E+01, 6.5887E+00, 1.7433E+03, &
      2.2985932E+00, 3.07971E+01, 6.5368E+00, 1.7210E+03, &
      2.3100862E+00, 3.08114E+01, 6.4853E+00, 1.6990E+03, &
      2.3216366E+00, 3.08253E+01, 6.4343E+00, 1.6773E+03, &
      2.3332448E+00, 3.08387E+01, 6.3836E+00, 1.6559E+03, &
      2.3449110E+00, 3.08516E+01, 6.3333E+00, 1.6347E+03, &
      2.3566356E+00, 3.08641E+01, 6.2835E+00, 1.6139E+03, &
      2.3684187E+00, 3.08762E+01, 6.2340E+00, 1.5932E+03, &
      2.3802608E+00, 3.08879E+01, 6.1849E+00, 1.5729E+03, &
      2.3921621E+00, 3.08991E+01, 6.1363E+00, 1.5528E+03, &
      2.4041230E+00, 3.09100E+01, 6.0880E+00, 1.5330E+03, &
      2.4161436E+00, 3.09206E+01, 6.0401E+00, 1.5134E+03, &
      2.4282243E+00, 3.09307E+01, 5.9926E+00, 1.4941E+03, &
      2.4403654E+00, 3.09406E+01, 5.9455E+00, 1.4750E+03, &
      2.4525672E+00, 3.09501E+01, 5.8987E+00, 1.4562E+03, &
      2.4648301E+00, 3.09593E+01, 5.8523E+00, 1.4376E+03, &
      2.4771542E+00, 3.09682E+01, 5.8064E+00, 1.4192E+03, &
      2.4895400E+00, 3.09768E+01, 5.7607E+00, 1.4011E+03, &
      2.5019877E+00, 3.09850E+01, 5.7155E+00, 1.3833E+03, &
      2.5144976E+00, 3.09931E+01, 5.6706E+00, 1.3656E+03, &
      2.5270701E+00, 3.10008E+01, 5.6261E+00, 1.3482E+03, &
      2.5397055E+00, 3.10083E+01, 5.5820E+00, 1.3310E+03, &
      2.5524040E+00, 3.10155E+01, 5.5382E+00, 1.3141E+03, &
      2.5651660E+00, 3.10225E+01, 5.4947E+00, 1.2973E+03, &
      2.5779919E+00, 3.10293E+01, 5.4517E+00, 1.2808E+03, &
      2.5908818E+00, 3.10358E+01, 5.4090E+00, 1.2645E+03, &
      2.6038362E+00, 3.10839E+01, 5.3661E+00, 1.2483E+03, &
      2.6168554E+00, 3.10901E+01, 5.3233E+00, 1.2322E+03, &
      2.6299397E+00, 3.10961E+01, 5.2809E+00, 1.2164E+03, &
      2.6430894E+00, 3.11018E+01, 5.2388E+00, 1.2007E+03, &
      2.6563048E+00, 3.11072E+01, 5.1970E+00, 1.1853E+03, &
      2.6695863E+00, 3.11124E+01, 5.1556E+00, 1.1700E+03, &
      2.6829343E+00, 3.11173E+01, 5.1146E+00, 1.1550E+03, &
      2.6963489E+00, 3.11220E+01, 5.0739E+00, 1.1401E+03, &
      2.7098307E+00, 3.11265E+01, 5.0335E+00, 1.1255E+03, &
      2.7233798E+00, 3.11308E+01, 4.9935E+00, 1.1110E+03, &
      2.7369967E+00, 3.11350E+01, 4.9538E+00, 1.0968E+03, &
      2.7506817E+00, 3.11389E+01, 4.9145E+00, 1.0827E+03, &
      2.7644351E+00, 3.11426E+01, 4.8754E+00, 1.0688E+03, &
      2.7782573E+00, 3.11462E+01, 4.8367E+00, 1.0551E+03, &
      2.7921486E+00, 3.11496E+01, 4.7983E+00, 1.0415E+03, &
      2.8061093E+00, 3.11529E+01, 4.7603E+00, 1.0282E+03, &
      2.8201399E+00, 3.11561E+01, 4.7225E+00, 1.0150E+03, &
      2.8342406E+00, 3.11591E+01, 4.6851E+00, 1.0020E+03, &
      2.8484118E+00, 3.11621E+01, 4.6480E+00, 9.8916E+02, &
      2.8626539E+00, 3.11649E+01, 4.6112E+00, 9.7650E+02, &
      2.8769671E+00, 3.11677E+01, 4.5747E+00, 9.6400E+02, &
      2.8913520E+00, 3.11705E+01, 4.5385E+00, 9.5166E+02, &
      2.9058087E+00, 3.11732E+01, 4.5026E+00, 9.3948E+02, &
      2.9203378E+00, 3.11759E+01, 4.4671E+00, 9.2747E+02, &
      2.9349394E+00, 3.11787E+01, 4.4318E+00, 9.1561E+02, &
      2.9496141E+00, 3.11817E+01, 4.3968E+00, 9.0391E+02, &
      2.9643622E+00, 3.11848E+01, 4.3621E+00, 8.9237E+02, &
      2.9791840E+00, 3.11882E+01, 4.3277E+00, 8.8095E+02, &
      2.9940799E+00, 3.11921E+01, 4.2934E+00, 8.6968E+02, &
      3.0090503E+00, 3.11967E+01, 4.2573E+00, 8.5811E+02, &
      3.0240956E+00, 3.12005E+01, 4.2200E+00, 8.4642E+02, &
      3.0392161E+00, 3.12033E+01, 4.1831E+00, 8.3490E+02, &
      3.0544122E+00, 3.12054E+01, 4.1466E+00, 8.2353E+02, &
      3.0696842E+00, 3.12068E+01, 4.1104E+00, 8.1232E+02, &
      3.0850326E+00, 3.12078E+01, 4.0745E+00, 8.0127E+02, &
      3.1004578E+00, 3.12083E+01, 4.0389E+00, 7.9038E+02, &
      3.1159601E+00, 3.12084E+01, 4.0037E+00, 7.7964E+02, &
      3.1315399E+00, 3.12082E+01, 3.9688E+00, 7.6904E+02, &
      3.1471976E+00, 3.12077E+01, 3.9343E+00, 7.5860E+02, &
      3.1629336E+00, 3.12068E+01, 3.9000E+00, 7.4830E+02, &
      3.1787482E+00, 3.12057E+01, 3.8661E+00, 7.3815E+02, &
      3.1946420E+00, 3.12044E+01, 3.8325E+00, 7.2813E+02, &
      3.2106152E+00, 3.12028E+01, 3.7992E+00, 7.1826E+02, &
      3.2266683E+00, 3.12010E+01, 3.7662E+00, 7.0852E+02, &
      3.2428016E+00, 3.11990E+01, 3.7335E+00, 6.9893E+02, &
      3.2590156E+00, 3.11968E+01, 3.7011E+00, 6.8946E+02, &
      3.2753107E+00, 3.11944E+01, 3.6690E+00, 6.8013E+02, &
      3.2916873E+00, 3.11919E+01, 3.6372E+00, 6.7092E+02, &
      3.3081457E+00, 3.11891E+01, 3.6057E+00, 6.6185E+02, &
      3.3246864E+00, 3.11862E+01, 3.5745E+00, 6.5290E+02, &
      3.3413099E+00, 3.11832E+01, 3.5436E+00, 6.4407E+02, &
      3.3580164E+00, 3.11800E+01, 3.5129E+00, 6.3537E+02, &
      3.3748065E+00, 3.11767E+01, 3.4826E+00, 6.2679E+02, &
      3.3916805E+00, 3.11733E+01, 3.4525E+00, 6.1833E+02, &
      3.4086389E+00, 3.11697E+01, 3.4227E+00, 6.0999E+02, &
      3.4256821E+00, 3.11660E+01, 3.3932E+00, 6.0176E+02, &
      3.4428105E+00, 3.11622E+01, 3.3639E+00, 5.9365E+02, &
      3.4600246E+00, 3.11582E+01, 3.3349E+00, 5.8565E+02, &
      3.4773247E+00, 3.11542E+01, 3.3062E+00, 5.7776E+02, &
      3.4947113E+00, 3.11500E+01, 3.2778E+00, 5.6998E+02, &
      3.5121849E+00, 3.11458E+01, 3.2496E+00, 5.6230E+02, &
      3.5297458E+00, 3.11414E+01, 3.2216E+00, 5.5474E+02, &
      3.5473945E+00, 3.11370E+01, 3.1940E+00, 5.4728E+02, &
      3.5651315E+00, 3.11325E+01, 3.1665E+00, 5.3992E+02, &
      3.5829572E+00, 3.11279E+01, 3.1394E+00, 5.3267E+02, &
      3.6008719E+00, 3.11232E+01, 3.1124E+00, 5.2551E+02, &
      3.6188763E+00, 3.11184E+01, 3.0858E+00, 5.1846E+02, &
      3.6369707E+00, 3.11135E+01, 3.0593E+00, 5.1150E+02, &
      3.6551555E+00, 3.11086E+01, 3.0331E+00, 5.0464E+02, &
      3.6734313E+00, 3.11036E+01, 3.0072E+00, 4.9787E+02, &
      3.6917985E+00, 3.10985E+01, 2.9815E+00, 4.9120E+02, &
      3.7102575E+00, 3.10933E+01, 2.9560E+00, 4.8462E+02, &
      3.7288088E+00, 3.10881E+01, 2.9307E+00, 4.7813E+02, &
      3.7474528E+00, 3.10828E+01, 2.9057E+00, 4.7173E+02, &
      3.7661901E+00, 3.10775E+01, 2.8809E+00, 4.6541E+02, &
      3.7850210E+00, 3.10721E+01, 2.8564E+00, 4.5919E+02, &
      3.8039461E+00, 3.10666E+01, 2.8320E+00, 4.5305E+02, &
      3.8229659E+00, 3.10611E+01, 2.8079E+00, 4.4700E+02, &
      3.8420807E+00, 3.10556E+01, 2.7840E+00, 4.4103E+02, &
      3.8612911E+00, 3.10499E+01, 2.7603E+00, 4.3514E+02, &
      3.8805975E+00, 3.10443E+01, 2.7369E+00, 4.2933E+02, &
      3.9000005E+00, 3.10386E+01, 2.7136E+00, 4.2360E+02, &
      3.9195005E+00, 3.10328E+01, 2.6906E+00, 4.1796E+02, &
      3.9390980E+00, 3.10270E+01, 2.6678E+00, 4.1239E+02, &
      3.9587935E+00, 3.10211E+01, 2.6451E+00, 4.0689E+02, &
      3.9785875E+00, 3.10153E+01, 2.6227E+00, 4.0147E+02, &
      3.9984804E+00, 3.10093E+01, 2.6005E+00, 3.9613E+02, &
      4.0184728E+00, 3.10034E+01, 2.5785E+00, 3.9086E+02, &
      4.0385652E+00, 3.09974E+01, 2.5567E+00, 3.8566E+02, &
      4.0587580E+00, 3.09913E+01, 2.5351E+00, 3.8054E+02, &
      4.0790518E+00, 3.09853E+01, 2.5136E+00, 3.7548E+02, &
      4.0994471E+00, 3.09791E+01, 2.4924E+00, 3.7049E+02, &
      4.1199443E+00, 3.09730E+01, 2.4714E+00, 3.6557E+02, &
      4.1405440E+00, 3.09668E+01, 2.4505E+00, 3.6072E+02, &
      4.1612467E+00, 3.09606E+01, 2.4299E+00, 3.5594E+02, &
      4.1820530E+00, 3.09544E+01, 2.4094E+00, 3.5122E+02, &
      4.2029632E+00, 3.09482E+01, 2.3891E+00, 3.4657E+02, &
      4.2239781E+00, 3.09419E+01, 2.3690E+00, 3.4198E+02, &
      4.2450980E+00, 3.09356E+01, 2.3491E+00, 3.3745E+02, &
      4.2663234E+00, 3.09293E+01, 2.3294E+00, 3.3298E+02, &
      4.2876551E+00, 3.09229E+01, 2.3098E+00, 3.2858E+02, &
      4.3090933E+00, 3.09166E+01, 2.2904E+00, 3.2424E+02, &
      4.3306388E+00, 3.09102E+01, 2.2712E+00, 3.1995E+02, &
      4.3522920E+00, 3.09038E+01, 2.2522E+00, 3.1573E+02, &
      4.3740535E+00, 3.08974E+01, 2.2333E+00, 3.1156E+02, &
      4.3959237E+00, 3.08909E+01, 2.2146E+00, 3.0745E+02, &
      4.4179033E+00, 3.08845E+01, 2.1961E+00, 3.0339E+02, &
      4.4399929E+00, 3.08780E+01, 2.1777E+00, 2.9939E+02, &
      4.4621928E+00, 3.08715E+01, 2.1596E+00, 2.9545E+02, &
      4.4845038E+00, 3.08650E+01, 2.1415E+00, 2.9155E+02, &
      4.5069263E+00, 3.08585E+01, 2.1236E+00, 2.8772E+02, &
      4.5294609E+00, 3.08520E+01, 2.1059E+00, 2.8393E+02, &
      4.5521082E+00, 3.08455E+01, 2.0884E+00, 2.8020E+02, &
      4.5748688E+00, 3.08390E+01, 2.0710E+00, 2.7651E+02, &
      4.5977431E+00, 3.08324E+01, 2.0538E+00, 2.7288E+02, &
      4.6207318E+00, 3.08259E+01, 2.0367E+00, 2.6929E+02, &
      4.6438355E+00, 3.08193E+01, 2.0197E+00, 2.6576E+02, &
      4.6670547E+00, 3.08128E+01, 2.0030E+00, 2.6227E+02, &
      4.6903900E+00, 3.08062E+01, 1.9863E+00, 2.5883E+02, &
      4.7138419E+00, 3.07997E+01, 1.9699E+00, 2.5544E+02, &
      4.7374111E+00, 3.07931E+01, 1.9535E+00, 2.5209E+02, &
      4.7610982E+00, 3.07866E+01, 1.9373E+00, 2.4879E+02, &
      4.7849037E+00, 3.07800E+01, 1.9213E+00, 2.4553E+02, &
      4.8088282E+00, 3.07735E+01, 1.9054E+00, 2.4232E+02, &
      4.8328723E+00, 3.07669E+01, 1.8896E+00, 2.3915E+02, &
      4.8570367E+00, 3.07604E+01, 1.8734E+00, 2.3594E+02, &
      4.8813219E+00, 3.07538E+01, 1.8572E+00, 2.3278E+02, &
      4.9057285E+00, 3.07472E+01, 1.8413E+00, 2.2966E+02, &
      4.9302571E+00, 3.07405E+01, 1.8255E+00, 2.2659E+02, &
      4.9549084E+00, 3.07338E+01, 1.8097E+00, 2.2355E+02, &
      4.9796829E+00, 3.07271E+01, 1.7939E+00, 2.2052E+02, &
      5.0045814E+00, 3.07203E+01, 1.7782E+00, 2.1753E+02, &
      5.0296043E+00, 3.07134E+01, 1.7626E+00, 2.1458E+02, &
      5.0547523E+00, 3.07065E+01, 1.7472E+00, 2.1168E+02, &
      5.0800260E+00, 3.06995E+01, 1.7320E+00, 2.0882E+02, &
      5.1054262E+00, 3.06925E+01, 1.7168E+00, 2.0600E+02, &
      5.1309533E+00, 3.06855E+01, 1.7019E+00, 2.0322E+02, &
      5.1566081E+00, 3.06784E+01, 1.6871E+00, 2.0048E+02, &
      5.1823911E+00, 3.06713E+01, 1.6724E+00, 1.9777E+02, &
      5.2083031E+00, 3.06641E+01, 1.6579E+00, 1.9511E+02, &
      5.2343446E+00, 3.06569E+01, 1.6435E+00, 1.9248E+02, &
      5.2605163E+00, 3.06496E+01, 1.6292E+00, 1.8989E+02, &
      5.2868189E+00, 3.06423E+01, 1.6151E+00, 1.8734E+02, &
      5.3132530E+00, 3.06350E+01, 1.6011E+00, 1.8482E+02, &
      5.3398192E+00, 3.06277E+01, 1.5872E+00, 1.8234E+02, &
      5.3665183E+00, 3.06203E+01, 1.5735E+00, 1.7989E+02, &
      5.3933509E+00, 3.06130E+01, 1.5599E+00, 1.7747E+02, &
      5.4203177E+00, 3.06055E+01, 1.5464E+00, 1.7510E+02, &
      5.4474193E+00, 3.05981E+01, 1.5331E+00, 1.7275E+02, &
      5.4746564E+00, 3.05906E+01, 1.5199E+00, 1.7044E+02, &
      5.5020297E+00, 3.05832E+01, 1.5068E+00, 1.6816E+02, &
      5.5295398E+00, 3.05756E+01, 1.4938E+00, 1.6591E+02, &
      5.5571875E+00, 3.05681E+01, 1.4810E+00, 1.6369E+02, &
      5.5849734E+00, 3.05606E+01, 1.4683E+00, 1.6151E+02, &
      5.6128983E+00, 3.05530E+01, 1.4557E+00, 1.5935E+02, &
      5.6409628E+00, 3.05454E+01, 1.4429E+00, 1.5719E+02, &
      5.6691676E+00, 3.05378E+01, 1.4299E+00, 1.5503E+02, &
      5.6975135E+00, 3.05301E+01, 1.4171E+00, 1.5291E+02, &
      5.7260010E+00, 3.05224E+01, 1.4044E+00, 1.5081E+02, &
      5.7546310E+00, 3.05146E+01, 1.3919E+00, 1.4874E+02, &
      5.7834042E+00, 3.05067E+01, 1.3794E+00, 1.4671E+02, &
      5.8123212E+00, 3.04988E+01, 1.3671E+00, 1.4470E+02, &
      5.8413828E+00, 3.04908E+01, 1.3549E+00, 1.4272E+02, &
      5.8705897E+00, 3.04828E+01, 1.3428E+00, 1.4077E+02, &
      5.8999427E+00, 3.04747E+01, 1.3308E+00, 1.3885E+02, &
      5.9294424E+00, 3.04665E+01, 1.3189E+00, 1.3695E+02, &
      5.9590896E+00, 3.04583E+01, 1.3072E+00, 1.3508E+02, &
      5.9888850E+00, 3.04501E+01, 1.2956E+00, 1.3324E+02, &
      6.0188295E+00, 3.04418E+01, 1.2840E+00, 1.3142E+02, &
      6.0489236E+00, 3.04334E+01, 1.2726E+00, 1.2963E+02, &
      6.0791682E+00, 3.04250E+01, 1.2613E+00, 1.2786E+02, &
      6.1095641E+00, 3.04165E+01, 1.2501E+00, 1.2612E+02, &
      6.1401119E+00, 3.04080E+01, 1.2390E+00, 1.2441E+02, &
      6.1708125E+00, 3.03995E+01, 1.2280E+00, 1.2271E+02, &
      6.2016665E+00, 3.03908E+01, 1.2171E+00, 1.2104E+02, &
      6.2326749E+00, 3.03822E+01, 1.2063E+00, 1.1940E+02, &
      6.2638382E+00, 3.03734E+01, 1.1956E+00, 1.1778E+02, &
      6.2951574E+00, 3.03647E+01, 1.1851E+00, 1.1618E+02, &
      6.3266332E+00, 3.03558E+01, 1.1746E+00, 1.1460E+02, &
      6.3582664E+00, 3.03469E+01, 1.1642E+00, 1.1305E+02, &
      6.3900577E+00, 3.03380E+01, 1.1539E+00, 1.1152E+02, &
      6.4220080E+00, 3.03289E+01, 1.1437E+00, 1.1001E+02, &
      6.4541180E+00, 3.03199E+01, 1.1336E+00, 1.0852E+02, &
      6.4863886E+00, 3.03107E+01, 1.1236E+00, 1.0705E+02, &
      6.5188206E+00, 3.03015E+01, 1.1137E+00, 1.0560E+02, &
      6.5514147E+00, 3.02923E+01, 1.1039E+00, 1.0417E+02, &
      6.5841717E+00, 3.02829E+01, 1.0942E+00, 1.0276E+02, &
      6.915365E+00, 3.02036E+01, 1.0033E+00, 8.9977E+01, &
      7.392525E+00, 3.00569E+01, 8.9248E-01, 7.5137E+01, &
      7.902609E+00, 2.98851E+01, 7.9466E-01, 6.2828E+01, &
      8.447890E+00, 2.96694E+01, 7.0823E-01, 5.2605E+01, &
      9.030794E+00, 2.93678E+01, 6.2937E-01, 4.3942E+01, &
      9.653919E+00, 2.88162E+01, 5.5215E-01, 3.6278E+01, &
      1.015976E+01, 2.76480E+01, 4.9982E-01, 3.1362E+01, &
      1.032004E+01, 2.61338E+01, 4.8485E-01, 2.9998E+01, &
      1.035673E+01, 2.45278E+01, 4.8152E-01, 2.9698E+01, &
      1.041894E+01, 2.62749E+01, 3.8431E+00, 2.2426E+02, &
      1.057444E+01, 2.78546E+01, 3.7266E+00, 2.1431E+02, &
      1.103212E+01, 2.91935E+01, 3.4259E+00, 1.8896E+02, &
      1.179334E+01, 3.00256E+01, 3.0603E+00, 1.5805E+02, &
      1.260708E+01, 3.04757E+01, 2.7550E+00, 1.3322E+02, &
      1.347697E+01, 3.07777E+01, 2.4874E+00, 1.1263E+02, &
      1.440688E+01, 3.09902E+01, 2.2265E+00, 9.4421E+01, &
      1.540095E+01, 3.11350E+01, 1.9894E+00, 7.9024E+01, &
      1.646362E+01, 3.12343E+01, 1.7746E+00, 6.6039E+01, &
      1.759961E+01, 3.13007E+01, 1.5811E+00, 5.5134E+01, &
      1.881398E+01, 3.13433E+01, 1.4080E+00, 4.6013E+01, &
      2.011215E+01, 3.13689E+01, 1.2538E+00, 3.8407E+01, &
      2.149988E+01, 3.14186E+01, 1.1143E+00, 3.2004E+01, &
      2.298338E+01, 3.14203E+01, 9.8861E-01, 2.6630E+01, &
      2.456923E+01, 3.14148E+01, 8.7706E-01, 2.2164E+01, &
      2.626450E+01, 3.14053E+01, 7.7807E-01, 1.8453E+01, &
      2.807676E+01, 3.13918E+01, 6.8692E-01, 1.5297E+01, &
      3.001405E+01, 3.13739E+01, 6.0523E-01, 1.2663E+01, &
      3.208502E+01, 3.13537E+01, 5.3330E-01, 1.0489E+01, &
      3.429889E+01, 3.13325E+01, 4.6995E-01, 8.6943E+00, &
      3.666551E+01, 3.13111E+01, 4.1415E-01, 7.2125E+00, &
      3.919543E+01, 3.12898E+01, 3.6500E-01, 5.9889E+00, &
      4.189992E+01, 3.12693E+01, 3.2170E-01, 4.9780E+00, &
      4.479101E+01, 3.12498E+01, 2.8355E-01, 4.1427E+00, &
      4.788159E+01, 3.12310E+01, 2.4851E-01, 3.4344E+00, &
      5.118542E+01, 3.12126E+01, 2.1775E-01, 2.8512E+00, &
      5.471721E+01, 3.11950E+01, 1.9081E-01, 2.3718E+00, &
      5.849270E+01, 3.11784E+01, 1.6721E-01, 1.9774E+00, &
      6.252870E+01, 3.11628E+01, 1.4653E-01, 1.6527E+00, &
      6.684318E+01, 3.11484E+01, 1.2842E-01, 1.3856E+00, &
      7.145536E+01, 3.11350E+01, 1.1255E-01, 1.1654E+00, &
      7.638578E+01, 3.11228E+01, 9.8642E-02, 9.8393E-01, &
      8.165640E+01, 3.11117E+01, 8.6354E-02, 8.3347E-01, &
      8.729069E+01, 3.11012E+01, 7.5397E-02, 7.0801E-01, &
      9.331374E+01, 3.10915E+01, 6.5833E-02, 6.0471E-01, &
      9.975239E+01, 3.10826E+01, 5.7485E-02, 5.1958E-01, &
      1.066353E+02, 3.10745E+01, 5.0197E-02, 4.4930E-01, &
      1.139931E+02, 3.10671E+01, 4.3834E-02, 3.9119E-01, &
      1.218587E+02, 3.10604E+01, 3.8280E-02, 3.4307E-01, &
      1.302669E+02, 3.10543E+01, 3.3430E-02, 3.0311E-01, &
      1.392553E+02, 3.10488E+01, 2.9196E-02, 2.6985E-01, &
      1.488640E+02, 3.10438E+01, 2.5499E-02, 2.4208E-01, &
      1.591356E+02, 3.10392E+01, 2.2271E-02, 2.1882E-01, &
      1.701159E+02, 3.10352E+01, 1.9452E-02, 1.9923E-01, &
      1.818539E+02, 3.10315E+01, 1.6990E-02, 1.8269E-01, &
      1.944018E+02, 3.10282E+01, 1.4841E-02, 1.6863E-01, &
      2.078156E+02, 3.10252E+01, 1.2964E-02, 1.5660E-01, &
      2.221548E+02, 3.10225E+01, 1.1324E-02, 1.4625E-01, &
      2.374835E+02, 3.10201E+01, 9.8927E-03, 1.3727E-01, &
      2.538699E+02, 3.10180E+01, 8.6422E-03, 1.2943E-01, &
      2.713869E+02, 3.10161E+01, 7.5501E-03, 1.2252E-01, &
      2.901126E+02, 3.10143E+01, 6.5961E-03, 1.1637E-01, &
      3.101304E+02, 3.10128E+01, 5.7629E-03, 1.1086E-01, &
      3.315294E+02, 3.10114E+01, 5.0350E-03, 1.0588E-01, &
      3.544049E+02, 3.10101E+01, 4.3993E-03, 1.0133E-01, &
      3.788588E+02, 3.10090E+01, 3.8439E-03, 9.7143E-02, &
      4.050001E+02, 3.10080E+01, 3.3587E-03, 9.3260E-02, &
      4.329451E+02, 3.10072E+01, 2.9349E-03, 8.9633E-02]
   case(32)
      frel = -1.1160E-01
      allocate(nist_tab(2244))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 4.82331E+00, 0.0000E+00, 1.1051E-04, &
      1.142761E-02, 4.79856E+00, 0.0000E+00, 1.5183E-04, &
      1.221612E-02, 4.76990E+00, 0.0000E+00, 2.0744E-04, &
      1.305903E-02, 4.73664E+00, 0.0000E+00, 2.8186E-04, &
      1.396010E-02, 4.69793E+00, 0.0000E+00, 3.8087E-04, &
      1.492335E-02, 4.65272E+00, 0.0000E+00, 5.1187E-04, &
      1.595306E-02, 4.59971E+00, 0.0000E+00, 6.8420E-04, &
      1.705382E-02, 4.53719E+00, 0.0000E+00, 9.0963E-04, &
      1.823053E-02, 4.46296E+00, 0.0000E+00, 1.2029E-03, &
      1.948844E-02, 4.37397E+00, 0.0000E+00, 1.5823E-03, &
      2.083314E-02, 4.26589E+00, 0.0000E+00, 2.0703E-03, &
      2.227063E-02, 4.13200E+00, 0.0000E+00, 2.6947E-03, &
      2.380730E-02, 3.96079E+00, 0.0000E+00, 3.4891E-03, &
      2.545001E-02, 3.72818E+00, 0.0000E+00, 4.4944E-03, &
      2.720606E-02, 3.35842E+00, 0.0000E+00, 5.7597E-03, &
      2.812600E-02, 2.99548E+00, 0.0000E+00, 6.5051E-03, &
      2.855650E-02, 2.56016E+00, 0.0000E+00, 6.8742E-03, &
      2.867130E-02, 2.10442E+00, 0.0000E+00, 6.9748E-03, &
      2.884350E-02, 2.50915E+00, 8.6030E-01, 1.7290E+04, &
      2.908327E-02, 2.73980E+00, 8.7594E-01, 1.7459E+04, &
      2.927400E-02, 2.82322E+00, 8.8848E-01, 1.7594E+04, &
      3.109002E-02, 2.99292E+00, 1.0128E+00, 1.8885E+04, &
      3.323523E-02, 2.92695E+00, 1.1767E+00, 2.0524E+04, &
      3.552846E-02, 2.78960E+00, 1.3852E+00, 2.2602E+04, &
      3.797993E-02, 2.61993E+00, 1.6451E+00, 2.5110E+04, &
      4.060054E-02, 2.43542E+00, 1.9627E+00, 2.8023E+04, &
      4.340198E-02, 2.24947E+00, 2.3438E+00, 3.1304E+04, &
      4.639671E-02, 2.07531E+00, 2.7932E+00, 3.4899E+04, &
      4.959809E-02, 1.92706E+00, 3.3138E+00, 3.8731E+04, &
      5.302035E-02, 1.81991E+00, 3.9058E+00, 4.2704E+04, &
      5.667876E-02, 1.76967E+00, 4.5662E+00, 4.6702E+04, &
      6.058959E-02, 1.79203E+00, 5.2882E+00, 5.0596E+04, &
      6.477028E-02, 1.90158E+00, 6.0612E+00, 5.4248E+04, &
      6.923942E-02, 2.11068E+00, 6.8707E+00, 5.7524E+04, &
      7.401695E-02, 2.42839E+00, 7.6989E+00, 6.0297E+04, &
      7.912411E-02, 2.85949E+00, 8.5256E+00, 6.2462E+04, &
      8.458368E-02, 3.40364E+00, 9.3292E+00, 6.3938E+04, &
      9.041995E-02, 4.06865E+00, 1.0088E+01, 6.4674E+04, &
      9.665893E-02, 4.81459E+00, 1.0780E+01, 6.4653E+04, &
      1.033284E-01, 5.63462E+00, 1.1388E+01, 6.3890E+04, &
      1.104581E-01, 6.49340E+00, 1.1896E+01, 6.2430E+04, &
      1.180797E-01, 7.27992E+00, 1.2289E+01, 6.0330E+04, &
      1.183840E-01, 7.30144E+00, 1.2301E+01, 6.0236E+04, &
      1.201960E-01, 7.33901E+00, 1.2370E+01, 5.9660E+04, &
      1.206792E-01, 7.19222E+00, 1.2387E+01, 5.9502E+04, &
      1.214040E-01, 7.46831E+00, 1.2806E+01, 6.1149E+04, &
      1.232160E-01, 7.83726E+00, 1.2880E+01, 6.0595E+04, &
      1.253420E-01, 8.13535E+00, 1.2956E+01, 5.9922E+04, &
      1.262272E-01, 8.23834E+00, 1.2985E+01, 5.9635E+04, &
      1.272605E-01, 8.32672E+00, 1.3017E+01, 5.9297E+04, &
      1.277721E-01, 8.30406E+00, 1.3032E+01, 5.9128E+04, &
      1.285395E-01, 8.49560E+00, 1.3234E+01, 5.9686E+04, &
      1.304580E-01, 8.82407E+00, 1.3292E+01, 5.9065E+04, &
      1.349368E-01, 9.44533E+00, 1.3403E+01, 5.7581E+04, &
      1.442475E-01, 1.05868E+01, 1.3533E+01, 5.4389E+04, &
      1.542005E-01, 1.16674E+01, 1.3550E+01, 5.0941E+04, &
      1.648404E-01, 1.26665E+01, 1.3463E+01, 4.7346E+04, &
      1.762144E-01, 1.34585E+01, 1.3283E+01, 4.3700E+04, &
      1.764000E-01, 1.34654E+01, 1.3280E+01, 4.3642E+04, &
      1.791000E-01, 1.34468E+01, 1.3227E+01, 4.2814E+04, &
      1.798200E-01, 1.32225E+01, 1.3213E+01, 4.2595E+04, &
      1.809000E-01, 1.35903E+01, 1.3744E+01, 4.4042E+04, &
      1.836000E-01, 1.40475E+01, 1.3693E+01, 4.3235E+04, &
      1.883732E-01, 1.45635E+01, 1.3597E+01, 4.1846E+04, &
      2.013709E-01, 1.56526E+01, 1.3306E+01, 3.8306E+04, &
      2.152655E-01, 1.66120E+01, 1.2957E+01, 3.4894E+04, &
      2.301188E-01, 1.74848E+01, 1.2561E+01, 3.1643E+04, &
      2.459970E-01, 1.82815E+01, 1.2127E+01, 2.8578E+04, &
      2.629708E-01, 1.90067E+01, 1.1665E+01, 2.5716E+04, &
      2.811158E-01, 1.96640E+01, 1.1182E+01, 2.3062E+04, &
      3.005128E-01, 2.02567E+01, 1.0687E+01, 2.0618E+04, &
      3.212482E-01, 2.07880E+01, 1.0186E+01, 1.8382E+04, &
      3.434143E-01, 2.12611E+01, 9.6827E+00, 1.6347E+04, &
      3.671099E-01, 2.16795E+01, 9.1826E+00, 1.4502E+04, &
      3.924405E-01, 2.20431E+01, 8.6892E+00, 1.2838E+04, &
      4.195189E-01, 2.23602E+01, 8.2053E+00, 1.1341E+04, &
      4.484657E-01, 2.26310E+01, 7.7335E+00, 9.9994E+03, &
      4.794098E-01, 2.28586E+01, 7.2755E+00, 8.8006E+03, &
      5.124891E-01, 2.30456E+01, 6.8320E+00, 7.7313E+03, &
      5.478508E-01, 2.31952E+01, 6.3846E+00, 6.7591E+03, &
      5.856525E-01, 2.32789E+01, 5.9027E+00, 5.8463E+03, &
      6.260625E-01, 2.32910E+01, 5.4478E+00, 5.0482E+03, &
      6.692609E-01, 2.32338E+01, 5.0209E+00, 4.3529E+03, &
      7.154399E-01, 2.31045E+01, 4.6224E+00, 3.7495E+03, &
      7.648052E-01, 2.28946E+01, 4.2519E+00, 3.2270E+03, &
      8.175768E-01, 2.25869E+01, 3.9088E+00, 2.7759E+03, &
      8.739896E-01, 2.21496E+01, 3.5923E+00, 2.3872E+03, &
      9.0000000E-01, 2.26092E+01, 3.4614E+00, 2.2341E+03, &
      9.0450000E-01, 2.25715E+01, 3.4396E+00, 2.2090E+03, &
      9.0902250E-01, 2.25327E+01, 3.4180E+00, 2.1843E+03, &
      9.1356761E-01, 2.24929E+01, 3.3965E+00, 2.1598E+03, &
      9.1813545E-01, 2.24520E+01, 3.3751E+00, 2.1356E+03, &
      9.2272613E-01, 2.24100E+01, 3.3539E+00, 2.1117E+03, &
      9.2733976E-01, 2.23668E+01, 3.3328E+00, 2.0880E+03, &
      9.3197646E-01, 2.23224E+01, 3.3119E+00, 2.0646E+03, &
      9.3663634E-01, 2.22768E+01, 3.2910E+00, 2.0415E+03, &
      9.4131952E-01, 2.22298E+01, 3.2704E+00, 2.0186E+03, &
      9.4602612E-01, 2.21816E+01, 3.2498E+00, 1.9960E+03, &
      9.5075625E-01, 2.21319E+01, 3.2294E+00, 1.9737E+03, &
      9.5551003E-01, 2.20807E+01, 3.2092E+00, 1.9516E+03, &
      9.6028758E-01, 2.20281E+01, 3.1890E+00, 1.9298E+03, &
      9.6508902E-01, 2.19739E+01, 3.1690E+00, 1.9082E+03, &
      9.6991446E-01, 2.19180E+01, 3.1491E+00, 1.8868E+03, &
      9.7476404E-01, 2.18603E+01, 3.1294E+00, 1.8657E+03, &
      9.7963786E-01, 2.18009E+01, 3.1098E+00, 1.8449E+03, &
      9.8453605E-01, 2.17396E+01, 3.0903E+00, 1.8243E+03, &
      9.8945873E-01, 2.16763E+01, 3.0710E+00, 1.8039E+03, &
      9.9440602E-01, 2.16109E+01, 3.0517E+00, 1.7837E+03, &
      9.9937805E-01, 2.15433E+01, 3.0326E+00, 1.7638E+03, &
      1.0043749E+00, 2.14902E+01, 3.0089E+00, 1.7414E+03, &
      1.0093968E+00, 2.14294E+01, 2.9847E+00, 1.7188E+03, &
      1.0144438E+00, 2.13624E+01, 2.9607E+00, 1.6966E+03, &
      1.0195160E+00, 2.12906E+01, 2.9369E+00, 1.6747E+03, &
      1.0246136E+00, 2.12148E+01, 2.9133E+00, 1.6530E+03, &
      1.0297367E+00, 2.11349E+01, 2.8900E+00, 1.6317E+03, &
      1.0348853E+00, 2.10510E+01, 2.8669E+00, 1.6107E+03, &
      1.0400598E+00, 2.09631E+01, 2.8440E+00, 1.5899E+03, &
      1.0452601E+00, 2.08711E+01, 2.8213E+00, 1.5694E+03, &
      1.0504864E+00, 2.07747E+01, 2.7988E+00, 1.5492E+03, &
      1.0557388E+00, 2.06737E+01, 2.7765E+00, 1.5293E+03, &
      1.0610175E+00, 2.05680E+01, 2.7544E+00, 1.5097E+03, &
      1.0663226E+00, 2.04571E+01, 2.7326E+00, 1.4903E+03, &
      1.0716542E+00, 2.03407E+01, 2.7109E+00, 1.4712E+03, &
      1.0770125E+00, 2.02185E+01, 2.6894E+00, 1.4524E+03, &
      1.0823975E+00, 2.00899E+01, 2.6681E+00, 1.4338E+03, &
      1.0878095E+00, 1.99546E+01, 2.6470E+00, 1.4154E+03, &
      1.0932486E+00, 1.98118E+01, 2.6261E+00, 1.3973E+03, &
      1.0987148E+00, 1.96610E+01, 2.6054E+00, 1.3795E+03, &
      1.1042084E+00, 1.95012E+01, 2.5849E+00, 1.3619E+03, &
      1.1097294E+00, 1.93318E+01, 2.5646E+00, 1.3445E+03, &
      1.1152781E+00, 1.91515E+01, 2.5444E+00, 1.3274E+03, &
      1.1208545E+00, 1.89592E+01, 2.5245E+00, 1.3105E+03, &
      1.1264587E+00, 1.87534E+01, 2.5047E+00, 1.2938E+03, &
      1.1320910E+00, 1.85323E+01, 2.4851E+00, 1.2773E+03, &
      1.1377515E+00, 1.82939E+01, 2.4656E+00, 1.2611E+03, &
      1.1434402E+00, 1.80355E+01, 2.4464E+00, 1.2451E+03, &
      1.1491574E+00, 1.77540E+01, 2.4273E+00, 1.2293E+03, &
      1.1549032E+00, 1.74452E+01, 2.4084E+00, 1.2137E+03, &
      1.1606777E+00, 1.71038E+01, 2.3896E+00, 1.1984E+03, &
      1.1664811E+00, 1.67230E+01, 2.3711E+00, 1.1832E+03, &
      1.1723135E+00, 1.62931E+01, 2.3526E+00, 1.1682E+03, &
      1.1781751E+00, 1.58006E+01, 2.3344E+00, 1.1535E+03, &
      1.1840660E+00, 1.52254E+01, 2.3163E+00, 1.1389E+03, &
      1.1899863E+00, 1.45354E+01, 2.2984E+00, 1.1245E+03, &
      1.1959362E+00, 1.36744E+01, 2.2806E+00, 1.1104E+03, &
      1.2019159E+00, 1.25293E+01, 2.2630E+00, 1.0964E+03, &
      1.2079255E+00, 1.08065E+01, 2.2456E+00, 1.0826E+03, &
      1.2139651E+00, 7.08105E+00, 2.2283E+00, 1.0690E+03, &
      1.2165985E+00, -3.13227E+00, 2.2208E+00, 1.0631E+03, &
      1.2168014E+00, -3.29888E+00, 1.1907E+01, 5.6776E+03, &
      1.2200350E+00, 7.38606E+00, 1.1856E+01, 5.6383E+03, &
      1.2261351E+00, 1.03333E+01, 1.1760E+01, 5.5650E+03, &
      1.2322658E+00, 1.14726E+01, 1.1665E+01, 5.4927E+03, &
      1.2384271E+00, 1.18269E+01, 1.1571E+01, 5.4213E+03, &
      1.2446193E+00, 1.10504E+01, 1.1478E+01, 5.3509E+03, &
      1.2476951E+00, 6.23429E+00, 1.1432E+01, 5.3164E+03, &
      1.2479049E+00, 6.17858E+00, 1.6209E+01, 7.5344E+03, &
      1.2508424E+00, 1.16286E+01, 1.6147E+01, 7.4882E+03, &
      1.2570966E+00, 1.39279E+01, 1.6017E+01, 7.3912E+03, &
      1.2633821E+00, 1.52259E+01, 1.5889E+01, 7.2953E+03, &
      1.2696990E+00, 1.61910E+01, 1.5761E+01, 7.2008E+03, &
      1.2760475E+00, 1.69747E+01, 1.5634E+01, 7.1075E+03, &
      1.2824277E+00, 1.76396E+01, 1.5509E+01, 7.0154E+03, &
      1.2888399E+00, 1.82185E+01, 1.5384E+01, 6.9245E+03, &
      1.2952840E+00, 1.87314E+01, 1.5261E+01, 6.8347E+03, &
      1.3017605E+00, 1.91913E+01, 1.5138E+01, 6.7462E+03, &
      1.3082693E+00, 1.96071E+01, 1.5016E+01, 6.6588E+03, &
      1.3148106E+00, 1.99856E+01, 1.4896E+01, 6.5726E+03, &
      1.3213847E+00, 2.03315E+01, 1.4776E+01, 6.4874E+03, &
      1.3279916E+00, 2.06485E+01, 1.4658E+01, 6.4034E+03, &
      1.3346316E+00, 2.09392E+01, 1.4540E+01, 6.3205E+03, &
      1.3413047E+00, 2.12057E+01, 1.4424E+01, 6.2387E+03, &
      1.3480112E+00, 2.14492E+01, 1.4308E+01, 6.1579E+03, &
      1.3547513E+00, 2.16704E+01, 1.4193E+01, 6.0782E+03, &
      1.3615250E+00, 2.18693E+01, 1.4079E+01, 5.9995E+03, &
      1.3683327E+00, 2.20449E+01, 1.3966E+01, 5.9219E+03, &
      1.3751743E+00, 2.21951E+01, 1.3854E+01, 5.8452E+03, &
      1.3820502E+00, 2.23155E+01, 1.3743E+01, 5.7696E+03, &
      1.3889605E+00, 2.23976E+01, 1.3633E+01, 5.6950E+03, &
      1.3959053E+00, 2.24237E+01, 1.3524E+01, 5.6213E+03, &
      1.4028848E+00, 2.23485E+01, 1.3416E+01, 5.5486E+03, &
      1.4098992E+00, 2.19837E+01, 1.3308E+01, 5.4768E+03, &
      1.4130073E+00, 2.13436E+01, 1.3261E+01, 5.4454E+03, &
      1.4155926E+00, 2.13844E+01, 1.5085E+01, 6.1824E+03, &
      1.4169487E+00, 2.18469E+01, 1.5064E+01, 6.1678E+03, &
      1.4240335E+00, 2.28052E+01, 1.4954E+01, 6.0924E+03, &
      1.4311536E+00, 2.33095E+01, 1.4845E+01, 6.0180E+03, &
      1.4383094E+00, 2.36912E+01, 1.4737E+01, 5.9445E+03, &
      1.4455009E+00, 2.40122E+01, 1.4629E+01, 5.8719E+03, &
      1.4527284E+00, 2.42954E+01, 1.4523E+01, 5.8003E+03, &
      1.4599921E+00, 2.45523E+01, 1.4417E+01, 5.7295E+03, &
      1.4672920E+00, 2.47893E+01, 1.4313E+01, 5.6597E+03, &
      1.4746285E+00, 2.50104E+01, 1.4209E+01, 5.5907E+03, &
      1.4820016E+00, 2.52184E+01, 1.4106E+01, 5.5226E+03, &
      1.4894117E+00, 2.54154E+01, 1.4003E+01, 5.4552E+03, &
      1.4968587E+00, 2.56025E+01, 1.3901E+01, 5.3886E+03, &
      1.5043430E+00, 2.57810E+01, 1.3800E+01, 5.3229E+03, &
      1.5118647E+00, 2.59517E+01, 1.3700E+01, 5.2580E+03, &
      1.5194240E+00, 2.61154E+01, 1.3600E+01, 5.1938E+03, &
      1.5270212E+00, 2.62728E+01, 1.3501E+01, 5.1305E+03, &
      1.5346563E+00, 2.64243E+01, 1.3403E+01, 5.0680E+03, &
      1.5423295E+00, 2.65705E+01, 1.3306E+01, 5.0062E+03, &
      1.5500412E+00, 2.67116E+01, 1.3210E+01, 4.9452E+03, &
      1.5577914E+00, 2.68480E+01, 1.3114E+01, 4.8850E+03, &
      1.5655804E+00, 2.69801E+01, 1.3019E+01, 4.8255E+03, &
      1.5734083E+00, 2.71082E+01, 1.2924E+01, 4.7668E+03, &
      1.5812753E+00, 2.72324E+01, 1.2831E+01, 4.7087E+03, &
      1.5891817E+00, 2.73529E+01, 1.2737E+01, 4.6512E+03, &
      1.5971276E+00, 2.74698E+01, 1.2644E+01, 4.5943E+03, &
      1.6051132E+00, 2.75833E+01, 1.2552E+01, 4.5381E+03, &
      1.6131388E+00, 2.76935E+01, 1.2460E+01, 4.4826E+03, &
      1.6212045E+00, 2.78007E+01, 1.2369E+01, 4.4279E+03, &
      1.6293105E+00, 2.79049E+01, 1.2278E+01, 4.3736E+03, &
      1.6374571E+00, 2.80062E+01, 1.2188E+01, 4.3200E+03, &
      1.6456443E+00, 2.81047E+01, 1.2099E+01, 4.2670E+03, &
      1.6538726E+00, 2.82005E+01, 1.2010E+01, 4.2147E+03, &
      1.6621419E+00, 2.82938E+01, 1.1922E+01, 4.1630E+03, &
      1.6704526E+00, 2.83847E+01, 1.1835E+01, 4.1120E+03, &
      1.6788049E+00, 2.84732E+01, 1.1748E+01, 4.0616E+03, &
      1.6871989E+00, 2.85594E+01, 1.1662E+01, 4.0118E+03, &
      1.6956349E+00, 2.86435E+01, 1.1576E+01, 3.9626E+03, &
      1.7041131E+00, 2.87255E+01, 1.1491E+01, 3.9141E+03, &
      1.7126337E+00, 2.88055E+01, 1.1407E+01, 3.8661E+03, &
      1.7211968E+00, 2.88835E+01, 1.1324E+01, 3.8187E+03, &
      1.7298028E+00, 2.89596E+01, 1.1241E+01, 3.7720E+03, &
      1.7384518E+00, 2.90340E+01, 1.1158E+01, 3.7258E+03, &
      1.7471441E+00, 2.91066E+01, 1.1077E+01, 3.6801E+03, &
      1.7558798E+00, 2.91775E+01, 1.0995E+01, 3.6351E+03, &
      1.7646592E+00, 2.92467E+01, 1.0915E+01, 3.5905E+03, &
      1.7734825E+00, 2.93144E+01, 1.0835E+01, 3.5466E+03, &
      1.7823499E+00, 2.93806E+01, 1.0756E+01, 3.5031E+03, &
      1.7912617E+00, 2.94453E+01, 1.0677E+01, 3.4603E+03, &
      1.8002180E+00, 2.95085E+01, 1.0599E+01, 3.4179E+03, &
      1.8092191E+00, 2.95705E+01, 1.0521E+01, 3.3761E+03, &
      1.8182652E+00, 2.96311E+01, 1.0444E+01, 3.3347E+03, &
      1.8273565E+00, 2.96905E+01, 1.0368E+01, 3.2939E+03, &
      1.8364933E+00, 2.97488E+01, 1.0292E+01, 3.2536E+03, &
      1.8456757E+00, 2.98058E+01, 1.0216E+01, 3.2136E+03, &
      1.8549041E+00, 2.98613E+01, 1.0140E+01, 3.1739E+03, &
      1.8641786E+00, 2.99155E+01, 1.0065E+01, 3.1347E+03, &
      1.8734995E+00, 2.99685E+01, 9.9900E+00, 3.0960E+03, &
      1.8828670E+00, 3.00202E+01, 9.9158E+00, 3.0578E+03, &
      1.8922814E+00, 3.00707E+01, 9.8422E+00, 3.0200E+03, &
      1.9017428E+00, 3.01200E+01, 9.7691E+00, 2.9828E+03, &
      1.9112515E+00, 3.01682E+01, 9.6966E+00, 2.9459E+03, &
      1.9208077E+00, 3.02153E+01, 9.6245E+00, 2.9096E+03, &
      1.9304118E+00, 3.02614E+01, 9.5530E+00, 2.8736E+03, &
      1.9400638E+00, 3.03064E+01, 9.4821E+00, 2.8382E+03, &
      1.9497642E+00, 3.03504E+01, 9.4117E+00, 2.8031E+03, &
      1.9595130E+00, 3.03935E+01, 9.3417E+00, 2.7685E+03, &
      1.9693105E+00, 3.04356E+01, 9.2723E+00, 2.7343E+03, &
      1.9791571E+00, 3.04768E+01, 9.2034E+00, 2.7006E+03, &
      1.9890529E+00, 3.05171E+01, 9.1351E+00, 2.6672E+03, &
      1.9989981E+00, 3.05564E+01, 9.0672E+00, 2.6343E+03, &
      2.0089931E+00, 3.05950E+01, 8.9998E+00, 2.6018E+03, &
      2.0190381E+00, 3.06327E+01, 8.9329E+00, 2.5696E+03, &
      2.0291333E+00, 3.06695E+01, 8.8666E+00, 2.5379E+03, &
      2.0392790E+00, 3.07056E+01, 8.8007E+00, 2.5066E+03, &
      2.0494754E+00, 3.07409E+01, 8.7353E+00, 2.4756E+03, &
      2.0597227E+00, 3.07755E+01, 8.6704E+00, 2.4451E+03, &
      2.0700213E+00, 3.08093E+01, 8.6059E+00, 2.4149E+03, &
      2.0803714E+00, 3.08423E+01, 8.5420E+00, 2.3850E+03, &
      2.0907733E+00, 3.08747E+01, 8.4785E+00, 2.3556E+03, &
      2.1012272E+00, 3.09064E+01, 8.4155E+00, 2.3265E+03, &
      2.1117333E+00, 3.09374E+01, 8.3529E+00, 2.2978E+03, &
      2.1222920E+00, 3.09677E+01, 8.2909E+00, 2.2694E+03, &
      2.1329034E+00, 3.09974E+01, 8.2293E+00, 2.2414E+03, &
      2.1435680E+00, 3.10265E+01, 8.1681E+00, 2.2137E+03, &
      2.1542858E+00, 3.10549E+01, 8.1074E+00, 2.1864E+03, &
      2.1650572E+00, 3.10828E+01, 8.0472E+00, 2.1594E+03, &
      2.1758825E+00, 3.11100E+01, 7.9874E+00, 2.1328E+03, &
      2.1867619E+00, 3.11367E+01, 7.9281E+00, 2.1064E+03, &
      2.1976957E+00, 3.11629E+01, 7.8692E+00, 2.0804E+03, &
      2.2086842E+00, 3.11885E+01, 7.8107E+00, 2.0548E+03, &
      2.2197276E+00, 3.12135E+01, 7.7527E+00, 2.0294E+03, &
      2.2308263E+00, 3.12381E+01, 7.6951E+00, 2.0044E+03, &
      2.2419804E+00, 3.12621E+01, 7.6380E+00, 1.9796E+03, &
      2.2531903E+00, 3.12857E+01, 7.5813E+00, 1.9552E+03, &
      2.2644562E+00, 3.13088E+01, 7.5250E+00, 1.9311E+03, &
      2.2757785E+00, 3.13314E+01, 7.4687E+00, 1.9072E+03, &
      2.2871574E+00, 3.13535E+01, 7.4127E+00, 1.8835E+03, &
      2.2985932E+00, 3.13752E+01, 7.3571E+00, 1.8601E+03, &
      2.3100862E+00, 3.13964E+01, 7.3019E+00, 1.8370E+03, &
      2.3216366E+00, 3.14172E+01, 7.2471E+00, 1.8143E+03, &
      2.3332448E+00, 3.14375E+01, 7.1928E+00, 1.7917E+03, &
      2.3449110E+00, 3.14574E+01, 7.1389E+00, 1.7695E+03, &
      2.3566356E+00, 3.14770E+01, 7.0854E+00, 1.7476E+03, &
      2.3684187E+00, 3.14961E+01, 7.0323E+00, 1.7259E+03, &
      2.3802608E+00, 3.15149E+01, 6.9796E+00, 1.7045E+03, &
      2.3921621E+00, 3.15332E+01, 6.9273E+00, 1.6834E+03, &
      2.4041230E+00, 3.15513E+01, 6.8754E+00, 1.6625E+03, &
      2.4161436E+00, 3.15690E+01, 6.8236E+00, 1.6418E+03, &
      2.4282243E+00, 3.15864E+01, 6.7722E+00, 1.6214E+03, &
      2.4403654E+00, 3.17109E+01, 6.7203E+00, 1.6010E+03, &
      2.4525672E+00, 3.17274E+01, 6.6681E+00, 1.5807E+03, &
      2.4648301E+00, 3.17435E+01, 6.6163E+00, 1.5607E+03, &
      2.4771542E+00, 3.17591E+01, 6.5650E+00, 1.5409E+03, &
      2.4895400E+00, 3.17742E+01, 6.5140E+00, 1.5214E+03, &
      2.5019877E+00, 3.18434E+01, 6.4630E+00, 1.5020E+03, &
      2.5144976E+00, 3.18576E+01, 6.4121E+00, 1.4828E+03, &
      2.5270701E+00, 3.18714E+01, 6.3615E+00, 1.4639E+03, &
      2.5397055E+00, 3.18847E+01, 6.3114E+00, 1.4452E+03, &
      2.5524040E+00, 3.18976E+01, 6.2616E+00, 1.4267E+03, &
      2.5651660E+00, 3.19100E+01, 6.2123E+00, 1.4085E+03, &
      2.5779919E+00, 3.19221E+01, 6.1634E+00, 1.3905E+03, &
      2.5908818E+00, 3.19338E+01, 6.1148E+00, 1.3727E+03, &
      2.6038362E+00, 3.19451E+01, 6.0666E+00, 1.3551E+03, &
      2.6168554E+00, 3.19561E+01, 6.0189E+00, 1.3378E+03, &
      2.6299397E+00, 3.19667E+01, 5.9715E+00, 1.3208E+03, &
      2.6430894E+00, 3.19770E+01, 5.9245E+00, 1.3039E+03, &
      2.6563048E+00, 3.19870E+01, 5.8779E+00, 1.2872E+03, &
      2.6695863E+00, 3.19966E+01, 5.8317E+00, 1.2708E+03, &
      2.6829343E+00, 3.20060E+01, 5.7858E+00, 1.2546E+03, &
      2.6963489E+00, 3.20152E+01, 5.7403E+00, 1.2386E+03, &
      2.7098307E+00, 3.20240E+01, 5.6952E+00, 1.2228E+03, &
      2.7233798E+00, 3.20326E+01, 5.6505E+00, 1.2072E+03, &
      2.7369967E+00, 3.20410E+01, 5.6061E+00, 1.1918E+03, &
      2.7506817E+00, 3.20491E+01, 5.5621E+00, 1.1766E+03, &
      2.7644351E+00, 3.20570E+01, 5.5185E+00, 1.1616E+03, &
      2.7782573E+00, 3.20647E+01, 5.4752E+00, 1.1468E+03, &
      2.7921486E+00, 3.20722E+01, 5.4323E+00, 1.1322E+03, &
      2.8061093E+00, 3.20795E+01, 5.3897E+00, 1.1178E+03, &
      2.8201399E+00, 3.20866E+01, 5.3475E+00, 1.1036E+03, &
      2.8342406E+00, 3.21349E+01, 5.3054E+00, 1.0895E+03, &
      2.8484118E+00, 3.21419E+01, 5.2633E+00, 1.0755E+03, &
      2.8626539E+00, 3.21487E+01, 5.2215E+00, 1.0617E+03, &
      2.8769671E+00, 3.21554E+01, 5.1801E+00, 1.0481E+03, &
      2.8913520E+00, 3.21620E+01, 5.1390E+00, 1.0347E+03, &
      2.9058087E+00, 3.21684E+01, 5.0983E+00, 1.0214E+03, &
      2.9203378E+00, 3.21748E+01, 5.0579E+00, 1.0083E+03, &
      2.9349394E+00, 3.21811E+01, 5.0179E+00, 9.9542E+02, &
      2.9496141E+00, 3.21874E+01, 4.9781E+00, 9.8267E+02, &
      2.9643622E+00, 3.21937E+01, 4.9388E+00, 9.7010E+02, &
      2.9791840E+00, 3.22002E+01, 4.8997E+00, 9.5768E+02, &
      2.9940799E+00, 3.22068E+01, 4.8610E+00, 9.4544E+02, &
      3.0090503E+00, 3.22124E+01, 4.8201E+00, 9.3286E+02, &
      3.0240956E+00, 3.22167E+01, 4.7779E+00, 9.2015E+02, &
      3.0392161E+00, 3.22206E+01, 4.7361E+00, 9.0761E+02, &
      3.0544122E+00, 3.22241E+01, 4.6947E+00, 8.9525E+02, &
      3.0696842E+00, 3.22271E+01, 4.6537E+00, 8.8307E+02, &
      3.0850326E+00, 3.22298E+01, 4.6131E+00, 8.7105E+02, &
      3.1004578E+00, 3.22321E+01, 4.5729E+00, 8.5920E+02, &
      3.1159601E+00, 3.22341E+01, 4.5330E+00, 8.4752E+02, &
      3.1315399E+00, 3.22358E+01, 4.4935E+00, 8.3600E+02, &
      3.1471976E+00, 3.22372E+01, 4.4543E+00, 8.2464E+02, &
      3.1629336E+00, 3.22382E+01, 4.4155E+00, 8.1344E+02, &
      3.1787482E+00, 3.22390E+01, 4.3771E+00, 8.0240E+02, &
      3.1946420E+00, 3.22396E+01, 4.3391E+00, 7.9151E+02, &
      3.2106152E+00, 3.22398E+01, 4.3013E+00, 7.8078E+02, &
      3.2266683E+00, 3.22399E+01, 4.2640E+00, 7.7019E+02, &
      3.2428016E+00, 3.22397E+01, 4.2270E+00, 7.5975E+02, &
      3.2590156E+00, 3.22393E+01, 4.1903E+00, 7.4946E+02, &
      3.2753107E+00, 3.22386E+01, 4.1540E+00, 7.3931E+02, &
      3.2916873E+00, 3.22378E+01, 4.1180E+00, 7.2930E+02, &
      3.3081457E+00, 3.22368E+01, 4.0823E+00, 7.1943E+02, &
      3.3246864E+00, 3.22355E+01, 4.0470E+00, 7.0970E+02, &
      3.3413099E+00, 3.22341E+01, 4.0119E+00, 7.0011E+02, &
      3.3580164E+00, 3.22325E+01, 3.9773E+00, 6.9065E+02, &
      3.3748065E+00, 3.22308E+01, 3.9429E+00, 6.8132E+02, &
      3.3916805E+00, 3.22288E+01, 3.9088E+00, 6.7210E+02, &
      3.4086389E+00, 3.22268E+01, 3.8749E+00, 6.6301E+02, &
      3.4256821E+00, 3.22245E+01, 3.8413E+00, 6.5404E+02, &
      3.4428105E+00, 3.22221E+01, 3.8081E+00, 6.4520E+02, &
      3.4600246E+00, 3.22196E+01, 3.7752E+00, 6.3648E+02, &
      3.4773247E+00, 3.22169E+01, 3.7425E+00, 6.2789E+02, &
      3.4947113E+00, 3.22140E+01, 3.7102E+00, 6.1941E+02, &
      3.5121849E+00, 3.22111E+01, 3.6782E+00, 6.1105E+02, &
      3.5297458E+00, 3.22079E+01, 3.6464E+00, 6.0281E+02, &
      3.5473945E+00, 3.22047E+01, 3.6150E+00, 5.9468E+02, &
      3.5651315E+00, 3.22014E+01, 3.5838E+00, 5.8666E+02, &
      3.5829572E+00, 3.21979E+01, 3.5530E+00, 5.7876E+02, &
      3.6008719E+00, 3.21943E+01, 3.5224E+00, 5.7096E+02, &
      3.6188763E+00, 3.21906E+01, 3.4921E+00, 5.6328E+02, &
      3.6369707E+00, 3.21868E+01, 3.4620E+00, 5.5570E+02, &
      3.6551555E+00, 3.21829E+01, 3.4323E+00, 5.4822E+02, &
      3.6734313E+00, 3.21789E+01, 3.4028E+00, 5.4085E+02, &
      3.6917985E+00, 3.21748E+01, 3.3736E+00, 5.3358E+02, &
      3.7102575E+00, 3.21705E+01, 3.3447E+00, 5.2641E+02, &
      3.7288088E+00, 3.21662E+01, 3.3160E+00, 5.1934E+02, &
      3.7474528E+00, 3.21619E+01, 3.2876E+00, 5.1237E+02, &
      3.7661901E+00, 3.21574E+01, 3.2594E+00, 5.0550E+02, &
      3.7850210E+00, 3.21528E+01, 3.2315E+00, 4.9872E+02, &
      3.8039461E+00, 3.21482E+01, 3.2039E+00, 4.9203E+02, &
      3.8229659E+00, 3.21435E+01, 3.1765E+00, 4.8544E+02, &
      3.8420807E+00, 3.21387E+01, 3.1493E+00, 4.7894E+02, &
      3.8612911E+00, 3.21338E+01, 3.1224E+00, 4.7252E+02, &
      3.8805975E+00, 3.21288E+01, 3.0958E+00, 4.6620E+02, &
      3.9000005E+00, 3.21238E+01, 3.0694E+00, 4.5996E+02, &
      3.9195005E+00, 3.21187E+01, 3.0432E+00, 4.5381E+02, &
      3.9390980E+00, 3.21136E+01, 3.0173E+00, 4.4775E+02, &
      3.9587935E+00, 3.21084E+01, 2.9916E+00, 4.4176E+02, &
      3.9785875E+00, 3.21031E+01, 2.9662E+00, 4.3586E+02, &
      3.9984804E+00, 3.20978E+01, 2.9409E+00, 4.3005E+02, &
      4.0184728E+00, 3.20924E+01, 2.9159E+00, 4.2431E+02, &
      4.0385652E+00, 3.20870E+01, 2.8912E+00, 4.1865E+02, &
      4.0587580E+00, 3.20815E+01, 2.8666E+00, 4.1307E+02, &
      4.0790518E+00, 3.20759E+01, 2.8423E+00, 4.0756E+02, &
      4.0994471E+00, 3.20703E+01, 2.8182E+00, 4.0213E+02, &
      4.1199443E+00, 3.20647E+01, 2.7943E+00, 3.9678E+02, &
      4.1405440E+00, 3.20590E+01, 2.7707E+00, 3.9150E+02, &
      4.1612467E+00, 3.20532E+01, 2.7472E+00, 3.8629E+02, &
      4.1820530E+00, 3.20475E+01, 2.7240E+00, 3.8116E+02, &
      4.2029632E+00, 3.20416E+01, 2.7010E+00, 3.7609E+02, &
      4.2239781E+00, 3.20358E+01, 2.6781E+00, 3.7109E+02, &
      4.2450980E+00, 3.20299E+01, 2.6555E+00, 3.6617E+02, &
      4.2663234E+00, 3.20239E+01, 2.6331E+00, 3.6131E+02, &
      4.2876551E+00, 3.20180E+01, 2.6109E+00, 3.5651E+02, &
      4.3090933E+00, 3.20119E+01, 2.5889E+00, 3.5178E+02, &
      4.3306388E+00, 3.20059E+01, 2.5671E+00, 3.4712E+02, &
      4.3522920E+00, 3.19998E+01, 2.5455E+00, 3.4252E+02, &
      4.3740535E+00, 3.19937E+01, 2.5241E+00, 3.3799E+02, &
      4.3959237E+00, 3.19875E+01, 2.5029E+00, 3.3351E+02, &
      4.4179033E+00, 3.19814E+01, 2.4818E+00, 3.2910E+02, &
      4.4399929E+00, 3.19752E+01, 2.4610E+00, 3.2475E+02, &
      4.4621928E+00, 3.19689E+01, 2.4404E+00, 3.2045E+02, &
      4.4845038E+00, 3.19627E+01, 2.4199E+00, 3.1622E+02, &
      4.5069263E+00, 3.19564E+01, 2.3996E+00, 3.1204E+02, &
      4.5294609E+00, 3.19501E+01, 2.3795E+00, 3.0792E+02, &
      4.5521082E+00, 3.19438E+01, 2.3596E+00, 3.0386E+02, &
      4.5748688E+00, 3.19374E+01, 2.3399E+00, 2.9985E+02, &
      4.5977431E+00, 3.19311E+01, 2.3203E+00, 2.9590E+02, &
      4.6207318E+00, 3.19247E+01, 2.3009E+00, 2.9200E+02, &
      4.6438355E+00, 3.19183E+01, 2.2817E+00, 2.8816E+02, &
      4.6670547E+00, 3.19118E+01, 2.2627E+00, 2.8436E+02, &
      4.6903900E+00, 3.19054E+01, 2.2438E+00, 2.8062E+02, &
      4.7138419E+00, 3.18989E+01, 2.2251E+00, 2.7693E+02, &
      4.7374111E+00, 3.18925E+01, 2.2066E+00, 2.7329E+02, &
      4.7610982E+00, 3.18860E+01, 2.1882E+00, 2.6970E+02, &
      4.7849037E+00, 3.18795E+01, 2.1700E+00, 2.6616E+02, &
      4.8088282E+00, 3.18730E+01, 2.1520E+00, 2.6266E+02, &
      4.8328723E+00, 3.18664E+01, 2.1341E+00, 2.5922E+02, &
      4.8570367E+00, 3.18599E+01, 2.1164E+00, 2.5582E+02, &
      4.8813219E+00, 3.18533E+01, 2.0988E+00, 2.5246E+02, &
      4.9057285E+00, 3.18468E+01, 2.0814E+00, 2.4915E+02, &
      4.9302571E+00, 3.18402E+01, 2.0641E+00, 2.4589E+02, &
      4.9549084E+00, 3.18337E+01, 2.0471E+00, 2.4267E+02, &
      4.9796829E+00, 3.18271E+01, 2.0301E+00, 2.3950E+02, &
      5.0045814E+00, 3.18205E+01, 2.0133E+00, 2.3637E+02, &
      5.0296043E+00, 3.18139E+01, 1.9967E+00, 2.3328E+02, &
      5.0547523E+00, 3.18073E+01, 1.9802E+00, 2.3023E+02, &
      5.0800260E+00, 3.18007E+01, 1.9638E+00, 2.2722E+02, &
      5.1054262E+00, 3.17941E+01, 1.9476E+00, 2.2425E+02, &
      5.1309533E+00, 3.17875E+01, 1.9316E+00, 2.2133E+02, &
      5.1566081E+00, 3.17809E+01, 1.9157E+00, 2.1844E+02, &
      5.1823911E+00, 3.17743E+01, 1.8999E+00, 2.1560E+02, &
      5.2083031E+00, 3.17677E+01, 1.8843E+00, 2.1279E+02, &
      5.2343446E+00, 3.17612E+01, 1.8688E+00, 2.1002E+02, &
      5.2605163E+00, 3.17546E+01, 1.8534E+00, 2.0728E+02, &
      5.2868189E+00, 3.17480E+01, 1.8379E+00, 2.0455E+02, &
      5.3132530E+00, 3.17414E+01, 1.8221E+00, 2.0182E+02, &
      5.3398192E+00, 3.17348E+01, 1.8066E+00, 1.9913E+02, &
      5.3665183E+00, 3.17281E+01, 1.7911E+00, 1.9647E+02, &
      5.3933509E+00, 3.17214E+01, 1.7759E+00, 1.9386E+02, &
      5.4203177E+00, 3.17146E+01, 1.7606E+00, 1.9126E+02, &
      5.4474193E+00, 3.17078E+01, 1.7453E+00, 1.8868E+02, &
      5.4746564E+00, 3.17010E+01, 1.7301E+00, 1.8614E+02, &
      5.5020297E+00, 3.16941E+01, 1.7151E+00, 1.8364E+02, &
      5.5295398E+00, 3.16871E+01, 1.7002E+00, 1.8117E+02, &
      5.5571875E+00, 3.16801E+01, 1.6855E+00, 1.7873E+02, &
      5.5849734E+00, 3.16731E+01, 1.6709E+00, 1.7633E+02, &
      5.6128983E+00, 3.16660E+01, 1.6564E+00, 1.7396E+02, &
      5.6409628E+00, 3.16588E+01, 1.6421E+00, 1.7163E+02, &
      5.6691676E+00, 3.16517E+01, 1.6279E+00, 1.6933E+02, &
      5.6975135E+00, 3.16445E+01, 1.6139E+00, 1.6706E+02, &
      5.7260010E+00, 3.16372E+01, 1.6000E+00, 1.6482E+02, &
      5.7546310E+00, 3.16300E+01, 1.5862E+00, 1.6262E+02, &
      5.7834042E+00, 3.16227E+01, 1.5725E+00, 1.6044E+02, &
      5.8123212E+00, 3.16153E+01, 1.5590E+00, 1.5830E+02, &
      5.8413828E+00, 3.16080E+01, 1.5456E+00, 1.5618E+02, &
      5.8705897E+00, 3.16006E+01, 1.5323E+00, 1.5410E+02, &
      5.8999427E+00, 3.15932E+01, 1.5192E+00, 1.5204E+02, &
      5.9294424E+00, 3.15857E+01, 1.5062E+00, 1.5002E+02, &
      5.9590896E+00, 3.15783E+01, 1.4933E+00, 1.4802E+02, &
      5.9888850E+00, 3.15708E+01, 1.4805E+00, 1.4605E+02, &
      6.0188295E+00, 3.15633E+01, 1.4679E+00, 1.4411E+02, &
      6.0489236E+00, 3.15557E+01, 1.4553E+00, 1.4219E+02, &
      6.0791682E+00, 3.15482E+01, 1.4429E+00, 1.4030E+02, &
      6.1095641E+00, 3.15406E+01, 1.4306E+00, 1.3844E+02, &
      6.1401119E+00, 3.15330E+01, 1.4182E+00, 1.3658E+02, &
      6.1708125E+00, 3.15254E+01, 1.4056E+00, 1.3472E+02, &
      6.2016665E+00, 3.15177E+01, 1.3931E+00, 1.3288E+02, &
      6.2326749E+00, 3.15100E+01, 1.3807E+00, 1.3107E+02, &
      6.2638382E+00, 3.15022E+01, 1.3684E+00, 1.2928E+02, &
      6.2951574E+00, 3.14943E+01, 1.3562E+00, 1.2752E+02, &
      6.3266332E+00, 3.14863E+01, 1.3442E+00, 1.2578E+02, &
      6.3582664E+00, 3.14784E+01, 1.3322E+00, 1.2407E+02, &
      6.3900577E+00, 3.14703E+01, 1.3204E+00, 1.2238E+02, &
      6.4220080E+00, 3.14622E+01, 1.3087E+00, 1.2072E+02, &
      6.4541180E+00, 3.14540E+01, 1.2971E+00, 1.1907E+02, &
      6.4863886E+00, 3.14458E+01, 1.2856E+00, 1.1746E+02, &
      6.5188206E+00, 3.14375E+01, 1.2742E+00, 1.1586E+02, &
      6.5514147E+00, 3.14292E+01, 1.2630E+00, 1.1429E+02, &
      6.5841717E+00, 3.14208E+01, 1.2518E+00, 1.1274E+02, &
      6.915365E+00, 3.13523E+01, 1.1474E+00, 9.8654E+01, &
      7.392525E+00, 3.12238E+01, 1.0201E+00, 8.2323E+01, &
      7.902609E+00, 3.10798E+01, 9.0777E-01, 6.8785E+01, &
      8.447890E+00, 3.09116E+01, 8.0862E-01, 5.7549E+01, &
      9.030794E+00, 3.07014E+01, 7.1829E-01, 4.8041E+01, &
      9.653919E+00, 3.03972E+01, 6.3015E-01, 3.9648E+01, &
      1.032004E+01, 2.98588E+01, 5.5335E-01, 3.2774E+01, &
      1.088104E+01, 2.86740E+01, 4.9947E-01, 2.8210E+01, &
      1.103212E+01, 2.75223E+01, 4.8636E-01, 2.7134E+01, &
      1.104758E+01, 2.72677E+01, 4.8505E-01, 2.7027E+01, &
      1.109200E+01, 2.55686E+01, 4.8132E-01, 2.6724E+01, &
      1.115862E+01, 2.73085E+01, 3.8275E+00, 2.0040E+02, &
      1.132516E+01, 2.88815E+01, 3.7099E+00, 1.9143E+02, &
      1.179334E+01, 3.01773E+01, 3.4186E+00, 1.6951E+02, &
      1.260708E+01, 3.10210E+01, 3.0517E+00, 1.4169E+02, &
      1.347697E+01, 3.14728E+01, 2.7472E+00, 1.1944E+02, &
      1.440688E+01, 3.17750E+01, 2.4811E+00, 1.0101E+02, &
      1.540095E+01, 3.19885E+01, 2.2215E+00, 8.4714E+01, &
      1.646362E+01, 3.21337E+01, 1.9850E+00, 7.0907E+01, &
      1.759961E+01, 3.22331E+01, 1.7706E+00, 5.9261E+01, &
      1.881398E+01, 3.22995E+01, 1.5776E+00, 4.9481E+01, &
      2.011215E+01, 3.23420E+01, 1.4050E+00, 4.1302E+01, &
      2.149988E+01, 3.23676E+01, 1.2513E+00, 3.4484E+01, &
      2.298338E+01, 3.24172E+01, 1.1127E+00, 2.8754E+01, &
      2.456923E+01, 3.24194E+01, 9.8771E-01, 2.3943E+01, &
      2.626450E+01, 3.24148E+01, 8.7684E-01, 1.9944E+01, &
      2.807676E+01, 3.24060E+01, 7.7457E-01, 1.6540E+01, &
      3.001405E+01, 3.23910E+01, 6.8320E-01, 1.3702E+01, &
      3.208502E+01, 3.23726E+01, 6.0227E-01, 1.1352E+01, &
      3.429889E+01, 3.23523E+01, 5.3094E-01, 9.4106E+00, &
      3.666551E+01, 3.23310E+01, 4.6809E-01, 7.8070E+00, &
      3.919543E+01, 3.23095E+01, 4.1271E-01, 6.4824E+00, &
      4.189992E+01, 3.22884E+01, 3.6390E-01, 5.3876E+00, &
      4.479101E+01, 3.22679E+01, 3.2088E-01, 4.4828E+00, &
      4.788159E+01, 3.22485E+01, 2.8296E-01, 3.7348E+00, &
      5.118542E+01, 3.22299E+01, 2.4813E-01, 3.1001E+00, &
      5.471721E+01, 3.22116E+01, 2.1750E-01, 2.5769E+00, &
      5.849270E+01, 3.21942E+01, 1.9066E-01, 2.1465E+00, &
      6.252870E+01, 3.21777E+01, 1.6714E-01, 1.7921E+00, &
      6.684318E+01, 3.21623E+01, 1.4652E-01, 1.5004E+00, &
      7.145536E+01, 3.21479E+01, 1.2846E-01, 1.2600E+00, &
      7.638578E+01, 3.21347E+01, 1.1262E-01, 1.0619E+00, &
      8.165640E+01, 3.21226E+01, 9.8633E-02, 8.9770E-01, &
      8.729069E+01, 3.21112E+01, 8.6170E-02, 7.6086E-01, &
      9.331374E+01, 3.21007E+01, 7.5285E-02, 6.4817E-01, &
      9.975239E+01, 3.20911E+01, 6.5777E-02, 5.5528E-01, &
      1.066353E+02, 3.20822E+01, 5.7472E-02, 4.7861E-01, &
      1.139931E+02, 3.20741E+01, 5.0218E-02, 4.1524E-01, &
      1.218587E+02, 3.20668E+01, 4.3881E-02, 3.6276E-01, &
      1.302669E+02, 3.20601E+01, 3.8345E-02, 3.1922E-01, &
      1.392553E+02, 3.20540E+01, 3.3509E-02, 2.8300E-01, &
      1.488640E+02, 3.20485E+01, 2.9283E-02, 2.5279E-01, &
      1.591356E+02, 3.20436E+01, 2.5591E-02, 2.2752E-01, &
      1.701159E+02, 3.20391E+01, 2.2366E-02, 2.0629E-01, &
      1.818539E+02, 3.20350E+01, 1.9548E-02, 1.8837E-01, &
      1.944018E+02, 3.20314E+01, 1.7085E-02, 1.7319E-01, &
      2.078156E+02, 3.20281E+01, 1.4933E-02, 1.6023E-01, &
      2.221548E+02, 3.20251E+01, 1.3053E-02, 1.4913E-01, &
      2.374835E+02, 3.20225E+01, 1.1409E-02, 1.3953E-01, &
      2.538699E+02, 3.20201E+01, 9.9731E-03, 1.3118E-01, &
      2.713869E+02, 3.20179E+01, 8.7180E-03, 1.2385E-01, &
      2.901126E+02, 3.20160E+01, 7.6211E-03, 1.1738E-01, &
      3.101304E+02, 3.20143E+01, 6.6625E-03, 1.1160E-01, &
      3.315294E+02, 3.20128E+01, 5.8245E-03, 1.0639E-01, &
      3.544049E+02, 3.20114E+01, 5.0922E-03, 1.0167E-01, &
      3.788588E+02, 3.20101E+01, 4.4520E-03, 9.7344E-02, &
      4.050001E+02, 3.20090E+01, 3.8924E-03, 9.3354E-02, &
      4.329451E+02, 3.20080E+01, 3.4033E-03, 8.9643E-02]
   case(33)
      frel = -1.2000E-01
      allocate(nist_tab(2244))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.97388E+00, 0.0000E+00, 1.2935E-04, &
      1.142761E-02, 5.95892E+00, 0.0000E+00, 1.7697E-04, &
      1.221612E-02, 5.94172E+00, 0.0000E+00, 2.4079E-04, &
      1.305903E-02, 5.92193E+00, 0.0000E+00, 3.2586E-04, &
      1.396010E-02, 5.89914E+00, 0.0000E+00, 4.3861E-04, &
      1.492335E-02, 5.87286E+00, 0.0000E+00, 5.8719E-04, &
      1.595306E-02, 5.84252E+00, 0.0000E+00, 7.8193E-04, &
      1.705382E-02, 5.80742E+00, 0.0000E+00, 1.0357E-03, &
      1.823053E-02, 5.76673E+00, 0.0000E+00, 1.3647E-03, &
      1.948844E-02, 5.71947E+00, 0.0000E+00, 1.7888E-03, &
      2.083314E-02, 5.66438E+00, 0.0000E+00, 2.3324E-03, &
      2.227063E-02, 5.59995E+00, 0.0000E+00, 3.0256E-03, &
      2.380730E-02, 5.52424E+00, 0.0000E+00, 3.9047E-03, &
      2.545001E-02, 5.43479E+00, 0.0000E+00, 5.0136E-03, &
      2.720606E-02, 5.32828E+00, 0.0000E+00, 6.4047E-03, &
      2.908327E-02, 5.20019E+00, 0.0000E+00, 8.1408E-03, &
      3.109002E-02, 5.04396E+00, 0.0000E+00, 1.0296E-02, &
      3.323523E-02, 4.84923E+00, 0.0000E+00, 1.2957E-02, &
      3.552846E-02, 4.59747E+00, 0.0000E+00, 1.6225E-02, &
      3.797993E-02, 4.24541E+00, 0.0000E+00, 2.0218E-02, &
      4.037600E-02, 3.66212E+00, 0.0000E+00, 2.4632E-02, &
      4.060054E-02, 3.56057E+00, 0.0000E+00, 2.5072E-02, &
      4.099400E-02, 3.26814E+00, 0.0000E+00, 2.5854E-02, &
      4.115880E-02, 2.89512E+00, 0.0000E+00, 2.6185E-02, &
      4.140600E-02, 3.18075E+00, 6.7799E-01, 9.1967E+03, &
      4.202400E-02, 3.36584E+00, 7.2651E-01, 9.7099E+03, &
      4.340198E-02, 3.36274E+00, 8.4453E-01, 1.0929E+04, &
      4.639671E-02, 3.13444E+00, 1.1515E+00, 1.3940E+04, &
      4.959809E-02, 2.84875E+00, 1.5444E+00, 1.7489E+04, &
      5.302035E-02, 2.55769E+00, 2.0278E+00, 2.1481E+04, &
      5.667876E-02, 2.28864E+00, 2.6072E+00, 2.5836E+04, &
      6.058959E-02, 2.06592E+00, 3.2835E+00, 3.0438E+04, &
      6.477028E-02, 1.91310E+00, 4.0522E+00, 3.5139E+04, &
      6.923942E-02, 1.85256E+00, 4.9031E+00, 3.9773E+04, &
      7.401695E-02, 1.90410E+00, 5.8203E+00, 4.4166E+04, &
      7.912411E-02, 2.08345E+00, 6.7827E+00, 4.8146E+04, &
      8.458368E-02, 2.40093E+00, 7.7653E+00, 5.1564E+04, &
      9.041995E-02, 2.86035E+00, 8.7406E+00, 5.4293E+04, &
      9.665893E-02, 3.45832E+00, 9.6801E+00, 5.6248E+04, &
      1.033284E-01, 4.18397E+00, 1.0556E+01, 5.7381E+04, &
      1.104581E-01, 5.01853E+00, 1.1345E+01, 5.7686E+04, &
      1.180797E-01, 5.93419E+00, 1.2024E+01, 5.7192E+04, &
      1.262272E-01, 6.88034E+00, 1.2577E+01, 5.5963E+04, &
      1.349368E-01, 7.77925E+00, 1.2994E+01, 5.4087E+04, &
      1.376900E-01, 7.98349E+00, 1.3092E+01, 5.3406E+04, &
      1.397975E-01, 7.99944E+00, 1.3158E+01, 5.2863E+04, &
      1.403595E-01, 7.81981E+00, 1.3174E+01, 5.2716E+04, &
      1.412025E-01, 8.12885E+00, 1.3658E+01, 5.4329E+04, &
      1.433100E-01, 8.52176E+00, 1.3733E+01, 5.3824E+04, &
      1.434720E-01, 8.54386E+00, 1.3739E+01, 5.3785E+04, &
      1.442475E-01, 8.64108E+00, 1.3765E+01, 5.3596E+04, &
      1.456680E-01, 8.77001E+00, 1.3809E+01, 5.3246E+04, &
      1.462536E-01, 8.74386E+00, 1.3827E+01, 5.3101E+04, &
      1.471320E-01, 8.95813E+00, 1.4057E+01, 5.3661E+04, &
      1.493280E-01, 9.32082E+00, 1.4127E+01, 5.3135E+04, &
      1.542005E-01, 9.96447E+00, 1.4262E+01, 5.1948E+04, &
      1.648404E-01, 1.12098E+01, 1.4463E+01, 4.9281E+04, &
      1.762144E-01, 1.24105E+01, 1.4537E+01, 4.6335E+04, &
      1.883732E-01, 1.35313E+01, 1.4477E+01, 4.3166E+04, &
      1.994300E-01, 1.42923E+01, 1.4330E+01, 4.0360E+04, &
      2.013709E-01, 1.43350E+01, 1.4297E+01, 3.9879E+04, &
      2.024825E-01, 1.42878E+01, 1.4278E+01, 3.9605E+04, &
      2.032965E-01, 1.40489E+01, 1.4263E+01, 3.9406E+04, &
      2.045175E-01, 1.44540E+01, 1.4839E+01, 4.0754E+04, &
      2.075700E-01, 1.49650E+01, 1.4787E+01, 4.0011E+04, &
      2.152655E-01, 1.57674E+01, 1.4637E+01, 3.8190E+04, &
      2.301188E-01, 1.69797E+01, 1.4296E+01, 3.4893E+04, &
      2.459970E-01, 1.80578E+01, 1.3877E+01, 3.1686E+04, &
      2.629708E-01, 1.90331E+01, 1.3395E+01, 2.8611E+04, &
      2.811158E-01, 1.99130E+01, 1.2864E+01, 2.5704E+04, &
      3.005128E-01, 2.07016E+01, 1.2298E+01, 2.2986E+04, &
      3.212482E-01, 2.14026E+01, 1.1708E+01, 2.0473E+04, &
      3.434143E-01, 2.20202E+01, 1.1107E+01, 1.8167E+04, &
      3.671099E-01, 2.25590E+01, 1.0502E+01, 1.6069E+04, &
      3.924405E-01, 2.30240E+01, 9.9010E+00, 1.4173E+04, &
      4.195189E-01, 2.34203E+01, 9.3104E+00, 1.2468E+04, &
      4.484657E-01, 2.37497E+01, 8.7349E+00, 1.0943E+04, &
      4.794098E-01, 2.40227E+01, 8.1779E+00, 9.5842E+03, &
      5.124891E-01, 2.42405E+01, 7.6422E+00, 8.3788E+03, &
      5.478508E-01, 2.44063E+01, 7.1294E+00, 7.3126E+03, &
      5.856525E-01, 2.45228E+01, 6.6402E+00, 6.3718E+03, &
      6.260625E-01, 2.45907E+01, 6.1736E+00, 5.5424E+03, &
      6.692609E-01, 2.46075E+01, 5.7303E+00, 4.8131E+03, &
      7.154399E-01, 2.45717E+01, 5.3113E+00, 4.1739E+03, &
      7.648052E-01, 2.44792E+01, 4.9173E+00, 3.6155E+03, &
      8.175768E-01, 2.43180E+01, 4.5204E+00, 3.1099E+03, &
      8.739896E-01, 2.40651E+01, 4.1474E+00, 2.6698E+03, &
      9.0000000E-01, 2.44656E+01, 3.9929E+00, 2.4965E+03, &
      9.0450000E-01, 2.44435E+01, 3.9672E+00, 2.4681E+03, &
      9.0902250E-01, 2.44209E+01, 3.9416E+00, 2.4401E+03, &
      9.1356761E-01, 2.43978E+01, 3.9163E+00, 2.4124E+03, &
      9.1813545E-01, 2.43741E+01, 3.8910E+00, 2.3850E+03, &
      9.2272613E-01, 2.43499E+01, 3.8660E+00, 2.3579E+03, &
      9.2733976E-01, 2.43250E+01, 3.8411E+00, 2.3312E+03, &
      9.3197646E-01, 2.42997E+01, 3.8164E+00, 2.3047E+03, &
      9.3663634E-01, 2.42737E+01, 3.7919E+00, 2.2785E+03, &
      9.4131952E-01, 2.42472E+01, 3.7675E+00, 2.2527E+03, &
      9.4602612E-01, 2.42202E+01, 3.7432E+00, 2.2271E+03, &
      9.5075625E-01, 2.41925E+01, 3.7192E+00, 2.2019E+03, &
      9.5551003E-01, 2.41643E+01, 3.6953E+00, 2.1769E+03, &
      9.6028758E-01, 2.41355E+01, 3.6715E+00, 2.1522E+03, &
      9.6508902E-01, 2.41062E+01, 3.6480E+00, 2.1278E+03, &
      9.6991446E-01, 2.40763E+01, 3.6245E+00, 2.1037E+03, &
      9.7476404E-01, 2.40458E+01, 3.6013E+00, 2.0798E+03, &
      9.7963786E-01, 2.40149E+01, 3.5782E+00, 2.0563E+03, &
      9.8453605E-01, 2.39834E+01, 3.5552E+00, 2.0330E+03, &
      9.8945873E-01, 2.39515E+01, 3.5325E+00, 2.0100E+03, &
      9.9440602E-01, 2.39192E+01, 3.5098E+00, 1.9872E+03, &
      9.9937805E-01, 2.38865E+01, 3.4874E+00, 1.9648E+03, &
      1.0043749E+00, 2.38509E+01, 3.4595E+00, 1.9394E+03, &
      1.0093968E+00, 2.38095E+01, 3.4312E+00, 1.9140E+03, &
      1.0144438E+00, 2.37656E+01, 3.4030E+00, 1.8890E+03, &
      1.0195160E+00, 2.37197E+01, 3.3752E+00, 1.8643E+03, &
      1.0246136E+00, 2.36722E+01, 3.3476E+00, 1.8399E+03, &
      1.0297367E+00, 2.36231E+01, 3.3203E+00, 1.8159E+03, &
      1.0348853E+00, 2.35725E+01, 3.2932E+00, 1.7922E+03, &
      1.0400598E+00, 2.35201E+01, 3.2664E+00, 1.7688E+03, &
      1.0452601E+00, 2.34662E+01, 3.2399E+00, 1.7458E+03, &
      1.0504864E+00, 2.34105E+01, 3.2135E+00, 1.7231E+03, &
      1.0557388E+00, 2.33531E+01, 3.1875E+00, 1.7006E+03, &
      1.0610175E+00, 2.32939E+01, 3.1616E+00, 1.6785E+03, &
      1.0663226E+00, 2.32327E+01, 3.1360E+00, 1.6567E+03, &
      1.0716542E+00, 2.31696E+01, 3.1107E+00, 1.6352E+03, &
      1.0770125E+00, 2.31045E+01, 3.0856E+00, 1.6140E+03, &
      1.0823975E+00, 2.30371E+01, 3.0607E+00, 1.5931E+03, &
      1.0878095E+00, 2.29676E+01, 3.0360E+00, 1.5725E+03, &
      1.0932486E+00, 2.28956E+01, 3.0116E+00, 1.5522E+03, &
      1.0987148E+00, 2.28212E+01, 2.9874E+00, 1.5321E+03, &
      1.1042084E+00, 2.27442E+01, 2.9634E+00, 1.5123E+03, &
      1.1097294E+00, 2.26644E+01, 2.9397E+00, 1.4928E+03, &
      1.1152781E+00, 2.25817E+01, 2.9161E+00, 1.4735E+03, &
      1.1208545E+00, 2.24959E+01, 2.8928E+00, 1.4545E+03, &
      1.1264587E+00, 2.24069E+01, 2.8697E+00, 1.4358E+03, &
      1.1320910E+00, 2.23144E+01, 2.8468E+00, 1.4173E+03, &
      1.1377515E+00, 2.22182E+01, 2.8241E+00, 1.3991E+03, &
      1.1434402E+00, 2.21180E+01, 2.8016E+00, 1.3811E+03, &
      1.1491574E+00, 2.20137E+01, 2.7794E+00, 1.3634E+03, &
      1.1549032E+00, 2.19049E+01, 2.7573E+00, 1.3459E+03, &
      1.1606777E+00, 2.17912E+01, 2.7354E+00, 1.3287E+03, &
      1.1664811E+00, 2.16724E+01, 2.7137E+00, 1.3117E+03, &
      1.1723135E+00, 2.15479E+01, 2.6923E+00, 1.2949E+03, &
      1.1781751E+00, 2.14173E+01, 2.6710E+00, 1.2783E+03, &
      1.1840660E+00, 2.12802E+01, 2.6499E+00, 1.2620E+03, &
      1.1899863E+00, 2.11358E+01, 2.6290E+00, 1.2459E+03, &
      1.1959362E+00, 2.09834E+01, 2.6083E+00, 1.2300E+03, &
      1.2019159E+00, 2.08225E+01, 2.5878E+00, 1.2143E+03, &
      1.2079255E+00, 2.06519E+01, 2.5674E+00, 1.1988E+03, &
      1.2139651E+00, 2.04706E+01, 2.5473E+00, 1.1836E+03, &
      1.2200350E+00, 2.02775E+01, 2.5273E+00, 1.1685E+03, &
      1.2261351E+00, 2.00709E+01, 2.5075E+00, 1.1537E+03, &
      1.2322658E+00, 1.98492E+01, 2.4879E+00, 1.1390E+03, &
      1.2384271E+00, 1.96102E+01, 2.4685E+00, 1.1246E+03, &
      1.2446193E+00, 1.93513E+01, 2.4492E+00, 1.1103E+03, &
      1.2508424E+00, 1.90692E+01, 2.4301E+00, 1.0962E+03, &
      1.2570966E+00, 1.87596E+01, 2.4112E+00, 1.0824E+03, &
      1.2633821E+00, 1.84174E+01, 2.3925E+00, 1.0687E+03, &
      1.2696990E+00, 1.80351E+01, 2.3739E+00, 1.0552E+03, &
      1.2760475E+00, 1.76029E+01, 2.3555E+00, 1.0418E+03, &
      1.2824277E+00, 1.71067E+01, 2.3372E+00, 1.0287E+03, &
      1.2888399E+00, 1.65252E+01, 2.3191E+00, 1.0157E+03, &
      1.2952840E+00, 1.58238E+01, 2.3012E+00, 1.0029E+03, &
      1.3017605E+00, 1.49407E+01, 2.2834E+00, 9.9028E+02, &
      1.3082693E+00, 1.37469E+01, 2.2658E+00, 9.7782E+02, &
      1.3148106E+00, 1.18809E+01, 2.2483E+00, 9.6553E+02, &
      1.3213847E+00, 7.00551E+00, 2.2310E+00, 9.5340E+02, &
      1.3229793E+00, -1.02828E+00, 2.2269E+00, 9.5049E+02, &
      1.3232206E+00, -1.19194E+00, 1.1702E+01, 4.9721E+03, &
      1.3279916E+00, 9.86152E+00, 1.1634E+01, 4.9257E+03, &
      1.3346316E+00, 1.21906E+01, 1.1541E+01, 4.8620E+03, &
      1.3413047E+00, 1.31749E+01, 1.1449E+01, 4.7992E+03, &
      1.3480112E+00, 1.34770E+01, 1.1357E+01, 4.7372E+03, &
      1.3547513E+00, 1.27785E+01, 1.1267E+01, 4.6761E+03, &
      1.3584739E+00, 8.07519E+00, 1.1217E+01, 4.6427E+03, &
      1.3587261E+00, 8.02227E+00, 1.5893E+01, 6.5748E+03, &
      1.3615250E+00, 1.29746E+01, 1.5841E+01, 6.5397E+03, &
      1.3683327E+00, 1.53326E+01, 1.5714E+01, 6.4554E+03, &
      1.3751743E+00, 1.66108E+01, 1.5589E+01, 6.3722E+03, &
      1.3820502E+00, 1.75520E+01, 1.5465E+01, 6.2901E+03, &
      1.3889605E+00, 1.83128E+01, 1.5342E+01, 6.2091E+03, &
      1.3959053E+00, 1.89565E+01, 1.5220E+01, 6.1291E+03, &
      1.4028848E+00, 1.95156E+01, 1.5099E+01, 6.0502E+03, &
      1.4098992E+00, 2.00100E+01, 1.4979E+01, 5.9723E+03, &
      1.4169487E+00, 2.04522E+01, 1.4860E+01, 5.8954E+03, &
      1.4240335E+00, 2.08510E+01, 1.4742E+01, 5.8195E+03, &
      1.4311536E+00, 2.12128E+01, 1.4625E+01, 5.7445E+03, &
      1.4383094E+00, 2.15421E+01, 1.4508E+01, 5.6706E+03, &
      1.4455009E+00, 2.18422E+01, 1.4393E+01, 5.5976E+03, &
      1.4527284E+00, 2.21155E+01, 1.4279E+01, 5.5256E+03, &
      1.4599921E+00, 2.23634E+01, 1.4165E+01, 5.4545E+03, &
      1.4672920E+00, 2.25867E+01, 1.4053E+01, 5.3843E+03, &
      1.4746285E+00, 2.27850E+01, 1.3941E+01, 5.3150E+03, &
      1.4820016E+00, 2.29569E+01, 1.3830E+01, 5.2466E+03, &
      1.4894117E+00, 2.30990E+01, 1.3720E+01, 5.1791E+03, &
      1.4968587E+00, 2.32045E+01, 1.3612E+01, 5.1125E+03, &
      1.5043430E+00, 2.32595E+01, 1.3503E+01, 5.0468E+03, &
      1.5118647E+00, 2.32307E+01, 1.3396E+01, 4.9819E+03, &
      1.5194240E+00, 2.30055E+01, 1.3290E+01, 4.9178E+03, &
      1.5251383E+00, 2.21676E+01, 1.3211E+01, 4.8701E+03, &
      1.5270212E+00, 2.16078E+01, 1.5080E+01, 5.5518E+03, &
      1.5278616E+00, 2.22101E+01, 1.5068E+01, 5.5442E+03, &
      1.5346563E+00, 2.34670E+01, 1.4969E+01, 5.4834E+03, &
      1.5423295E+00, 2.40566E+01, 1.4858E+01, 5.4158E+03, &
      1.5500412E+00, 2.44774E+01, 1.4748E+01, 5.3492E+03, &
      1.5577914E+00, 2.48232E+01, 1.4639E+01, 5.2833E+03, &
      1.5655804E+00, 2.51247E+01, 1.4531E+01, 5.2183E+03, &
      1.5734083E+00, 2.53961E+01, 1.4424E+01, 5.1541E+03, &
      1.5812753E+00, 2.56449E+01, 1.4318E+01, 5.0907E+03, &
      1.5891817E+00, 2.58760E+01, 1.4212E+01, 5.0282E+03, &
      1.5971276E+00, 2.60924E+01, 1.4108E+01, 4.9664E+03, &
      1.6051132E+00, 2.62963E+01, 1.4004E+01, 4.9055E+03, &
      1.6131388E+00, 2.64898E+01, 1.3902E+01, 4.8455E+03, &
      1.6212045E+00, 2.66742E+01, 1.3801E+01, 4.7864E+03, &
      1.6293105E+00, 2.68504E+01, 1.3701E+01, 4.7280E+03, &
      1.6374571E+00, 2.70193E+01, 1.3601E+01, 4.6703E+03, &
      1.6456443E+00, 2.71815E+01, 1.3502E+01, 4.6134E+03, &
      1.6538726E+00, 2.73375E+01, 1.3404E+01, 4.5572E+03, &
      1.6621419E+00, 2.74878E+01, 1.3307E+01, 4.5017E+03, &
      1.6704526E+00, 2.76329E+01, 1.3211E+01, 4.4470E+03, &
      1.6788049E+00, 2.77732E+01, 1.3115E+01, 4.3929E+03, &
      1.6871989E+00, 2.79088E+01, 1.3021E+01, 4.3395E+03, &
      1.6956349E+00, 2.80403E+01, 1.2927E+01, 4.2869E+03, &
      1.7041131E+00, 2.81678E+01, 1.2833E+01, 4.2348E+03, &
      1.7126337E+00, 2.82917E+01, 1.2741E+01, 4.1835E+03, &
      1.7211968E+00, 2.84121E+01, 1.2649E+01, 4.1327E+03, &
      1.7298028E+00, 2.85289E+01, 1.2557E+01, 4.0821E+03, &
      1.7384518E+00, 2.86423E+01, 1.2465E+01, 4.0322E+03, &
      1.7471441E+00, 2.87524E+01, 1.2374E+01, 3.9829E+03, &
      1.7558798E+00, 2.88595E+01, 1.2284E+01, 3.9343E+03, &
      1.7646592E+00, 2.89637E+01, 1.2194E+01, 3.8862E+03, &
      1.7734825E+00, 2.90650E+01, 1.2104E+01, 3.8385E+03, &
      1.7823499E+00, 2.91635E+01, 1.2015E+01, 3.7914E+03, &
      1.7912617E+00, 2.92593E+01, 1.1927E+01, 3.7449E+03, &
      1.8002180E+00, 2.93525E+01, 1.1839E+01, 3.6989E+03, &
      1.8092191E+00, 2.94432E+01, 1.1752E+01, 3.6535E+03, &
      1.8182652E+00, 2.95316E+01, 1.1666E+01, 3.6087E+03, &
      1.8273565E+00, 2.96177E+01, 1.1580E+01, 3.5644E+03, &
      1.8364933E+00, 2.97016E+01, 1.1495E+01, 3.5207E+03, &
      1.8456757E+00, 2.97834E+01, 1.1411E+01, 3.4775E+03, &
      1.8549041E+00, 2.98632E+01, 1.1327E+01, 3.4349E+03, &
      1.8641786E+00, 2.99411E+01, 1.1244E+01, 3.3928E+03, &
      1.8734995E+00, 3.00171E+01, 1.1162E+01, 3.3512E+03, &
      1.8828670E+00, 3.00913E+01, 1.1080E+01, 3.3101E+03, &
      1.8922814E+00, 3.01637E+01, 1.0998E+01, 3.2695E+03, &
      1.9017428E+00, 3.02345E+01, 1.0918E+01, 3.2294E+03, &
      1.9112515E+00, 3.03037E+01, 1.0838E+01, 3.1899E+03, &
      1.9208077E+00, 3.03712E+01, 1.0758E+01, 3.1508E+03, &
      1.9304118E+00, 3.04373E+01, 1.0679E+01, 3.1122E+03, &
      1.9400638E+00, 3.05020E+01, 1.0601E+01, 3.0740E+03, &
      1.9497642E+00, 3.05652E+01, 1.0523E+01, 3.0364E+03, &
      1.9595130E+00, 3.06271E+01, 1.0446E+01, 2.9992E+03, &
      1.9693105E+00, 3.06878E+01, 1.0370E+01, 2.9625E+03, &
      1.9791571E+00, 3.07473E+01, 1.0294E+01, 2.9262E+03, &
      1.9890529E+00, 3.08056E+01, 1.0218E+01, 2.8902E+03, &
      1.9989981E+00, 3.08624E+01, 1.0141E+01, 2.8544E+03, &
      2.0089931E+00, 3.09178E+01, 1.0066E+01, 2.8191E+03, &
      2.0190381E+00, 3.09719E+01, 9.9906E+00, 2.7842E+03, &
      2.0291333E+00, 3.10246E+01, 9.9161E+00, 2.7497E+03, &
      2.0392790E+00, 3.10762E+01, 9.8421E+00, 2.7157E+03, &
      2.0494754E+00, 3.11265E+01, 9.7686E+00, 2.6821E+03, &
      2.0597227E+00, 3.11757E+01, 9.6957E+00, 2.6488E+03, &
      2.0700213E+00, 3.12237E+01, 9.6233E+00, 2.6160E+03, &
      2.0803714E+00, 3.12706E+01, 9.5514E+00, 2.5836E+03, &
      2.0907733E+00, 3.13165E+01, 9.4801E+00, 2.5516E+03, &
      2.1012272E+00, 3.13613E+01, 9.4093E+00, 2.5200E+03, &
      2.1117333E+00, 3.14051E+01, 9.3390E+00, 2.4888E+03, &
      2.1222920E+00, 3.14479E+01, 9.2692E+00, 2.4580E+03, &
      2.1329034E+00, 3.14898E+01, 9.1999E+00, 2.4275E+03, &
      2.1435680E+00, 3.15308E+01, 9.1311E+00, 2.3974E+03, &
      2.1542858E+00, 3.15708E+01, 9.0628E+00, 2.3677E+03, &
      2.1650572E+00, 3.16099E+01, 8.9951E+00, 2.3384E+03, &
      2.1758825E+00, 3.16482E+01, 8.9278E+00, 2.3094E+03, &
      2.1867619E+00, 3.16856E+01, 8.8610E+00, 2.2808E+03, &
      2.1976957E+00, 3.17222E+01, 8.7947E+00, 2.2525E+03, &
      2.2086842E+00, 3.17580E+01, 8.7289E+00, 2.2246E+03, &
      2.2197276E+00, 3.17930E+01, 8.6636E+00, 2.1970E+03, &
      2.2308263E+00, 3.18272E+01, 8.5988E+00, 2.1698E+03, &
      2.2419804E+00, 3.18607E+01, 8.5344E+00, 2.1429E+03, &
      2.2531903E+00, 3.18935E+01, 8.4705E+00, 2.1163E+03, &
      2.2644562E+00, 3.19255E+01, 8.4071E+00, 2.0901E+03, &
      2.2757785E+00, 3.19569E+01, 8.3442E+00, 2.0642E+03, &
      2.2871574E+00, 3.19875E+01, 8.2817E+00, 2.0386E+03, &
      2.2985932E+00, 3.20175E+01, 8.2197E+00, 2.0133E+03, &
      2.3100862E+00, 3.20468E+01, 8.1582E+00, 1.9883E+03, &
      2.3216366E+00, 3.20755E+01, 8.0971E+00, 1.9637E+03, &
      2.3332448E+00, 3.21036E+01, 8.0365E+00, 1.9394E+03, &
      2.3449110E+00, 3.21311E+01, 7.9763E+00, 1.9153E+03, &
      2.3566356E+00, 3.21580E+01, 7.9166E+00, 1.8916E+03, &
      2.3684187E+00, 3.21843E+01, 7.8574E+00, 1.8681E+03, &
      2.3802608E+00, 3.22100E+01, 7.7985E+00, 1.8450E+03, &
      2.3921621E+00, 3.22352E+01, 7.7402E+00, 1.8221E+03, &
      2.4041230E+00, 3.22598E+01, 7.6822E+00, 1.7995E+03, &
      2.4161436E+00, 3.22840E+01, 7.6247E+00, 1.7772E+03, &
      2.4282243E+00, 3.23076E+01, 7.5677E+00, 1.7552E+03, &
      2.4403654E+00, 3.23307E+01, 7.5110E+00, 1.7334E+03, &
      2.4525672E+00, 3.23533E+01, 7.4548E+00, 1.7120E+03, &
      2.4648301E+00, 3.23754E+01, 7.3991E+00, 1.6907E+03, &
      2.4771542E+00, 3.23971E+01, 7.3437E+00, 1.6698E+03, &
      2.4895400E+00, 3.24184E+01, 7.2888E+00, 1.6491E+03, &
      2.5019877E+00, 3.24392E+01, 7.2343E+00, 1.6287E+03, &
      2.5144976E+00, 3.24595E+01, 7.1802E+00, 1.6085E+03, &
      2.5270701E+00, 3.24795E+01, 7.1265E+00, 1.5886E+03, &
      2.5397055E+00, 3.24990E+01, 7.0732E+00, 1.5689E+03, &
      2.5524040E+00, 3.25182E+01, 7.0204E+00, 1.5495E+03, &
      2.5651660E+00, 3.25370E+01, 6.9679E+00, 1.5303E+03, &
      2.5779919E+00, 3.25554E+01, 6.9159E+00, 1.5114E+03, &
      2.5908818E+00, 3.25735E+01, 6.8642E+00, 1.4927E+03, &
      2.6038362E+00, 3.25912E+01, 6.8130E+00, 1.4742E+03, &
      2.6168554E+00, 3.26086E+01, 6.7621E+00, 1.4560E+03, &
      2.6299397E+00, 3.26257E+01, 6.7117E+00, 1.4380E+03, &
      2.6430894E+00, 3.26425E+01, 6.6614E+00, 1.4202E+03, &
      2.6563048E+00, 3.27635E+01, 6.6101E+00, 1.4023E+03, &
      2.6695863E+00, 3.27794E+01, 6.5590E+00, 1.3846E+03, &
      2.6829343E+00, 3.27949E+01, 6.5082E+00, 1.3671E+03, &
      2.6963489E+00, 3.28099E+01, 6.4579E+00, 1.3498E+03, &
      2.7098307E+00, 3.28246E+01, 6.4080E+00, 1.3327E+03, &
      2.7233798E+00, 3.28894E+01, 6.3581E+00, 1.3158E+03, &
      2.7369967E+00, 3.29031E+01, 6.3083E+00, 1.2991E+03, &
      2.7506817E+00, 3.29164E+01, 6.2589E+00, 1.2825E+03, &
      2.7644351E+00, 3.29293E+01, 6.2096E+00, 1.2662E+03, &
      2.7782573E+00, 3.29418E+01, 6.1607E+00, 1.2500E+03, &
      2.7921486E+00, 3.29538E+01, 6.1122E+00, 1.2340E+03, &
      2.8061093E+00, 3.29656E+01, 6.0642E+00, 1.2183E+03, &
      2.8201399E+00, 3.29769E+01, 6.0165E+00, 1.2027E+03, &
      2.8342406E+00, 3.29880E+01, 5.9692E+00, 1.1874E+03, &
      2.8484118E+00, 3.29987E+01, 5.9223E+00, 1.1723E+03, &
      2.8626539E+00, 3.30091E+01, 5.8757E+00, 1.1573E+03, &
      2.8769671E+00, 3.30192E+01, 5.8296E+00, 1.1426E+03, &
      2.8913520E+00, 3.30290E+01, 5.7838E+00, 1.1280E+03, &
      2.9058087E+00, 3.30386E+01, 5.7385E+00, 1.1136E+03, &
      2.9203378E+00, 3.30479E+01, 5.6935E+00, 1.0994E+03, &
      2.9349394E+00, 3.30570E+01, 5.6489E+00, 1.0855E+03, &
      2.9496141E+00, 3.30659E+01, 5.6046E+00, 1.0716E+03, &
      2.9643622E+00, 3.30747E+01, 5.5607E+00, 1.0580E+03, &
      2.9791840E+00, 3.30835E+01, 5.5172E+00, 1.0446E+03, &
      2.9940799E+00, 3.30940E+01, 5.4741E+00, 1.0313E+03, &
      3.0090503E+00, 3.31145E+01, 5.4288E+00, 1.0177E+03, &
      3.0240956E+00, 3.31264E+01, 5.3822E+00, 1.0040E+03, &
      3.0392161E+00, 3.31367E+01, 5.3360E+00, 9.9049E+02, &
      3.0544122E+00, 3.31882E+01, 5.2902E+00, 9.7715E+02, &
      3.0696842E+00, 3.31966E+01, 5.2439E+00, 9.6381E+02, &
      3.0850326E+00, 3.32041E+01, 5.1979E+00, 9.5067E+02, &
      3.1004578E+00, 3.32109E+01, 5.1524E+00, 9.3771E+02, &
      3.1159601E+00, 3.32171E+01, 5.1074E+00, 9.2493E+02, &
      3.1315399E+00, 3.32227E+01, 5.0627E+00, 9.1233E+02, &
      3.1471976E+00, 3.32278E+01, 5.0184E+00, 8.9990E+02, &
      3.1629336E+00, 3.32325E+01, 4.9746E+00, 8.8765E+02, &
      3.1787482E+00, 3.32366E+01, 4.9312E+00, 8.7558E+02, &
      3.1946420E+00, 3.32404E+01, 4.8882E+00, 8.6367E+02, &
      3.2106152E+00, 3.32438E+01, 4.8455E+00, 8.5192E+02, &
      3.2266683E+00, 3.32468E+01, 4.8033E+00, 8.4035E+02, &
      3.2428016E+00, 3.32495E+01, 4.7615E+00, 8.2893E+02, &
      3.2590156E+00, 3.32518E+01, 4.7200E+00, 8.1767E+02, &
      3.2753107E+00, 3.32539E+01, 4.6789E+00, 8.0657E+02, &
      3.2916873E+00, 3.32556E+01, 4.6383E+00, 7.9563E+02, &
      3.3081457E+00, 3.32570E+01, 4.5980E+00, 7.8484E+02, &
      3.3246864E+00, 3.32582E+01, 4.5580E+00, 7.7420E+02, &
      3.3413099E+00, 3.32591E+01, 4.5184E+00, 7.6370E+02, &
      3.3580164E+00, 3.32598E+01, 4.4792E+00, 7.5336E+02, &
      3.3748065E+00, 3.32602E+01, 4.4404E+00, 7.4315E+02, &
      3.3916805E+00, 3.32604E+01, 4.4019E+00, 7.3310E+02, &
      3.4086389E+00, 3.32603E+01, 4.3638E+00, 7.2318E+02, &
      3.4256821E+00, 3.32600E+01, 4.3260E+00, 7.1339E+02, &
      3.4428105E+00, 3.32596E+01, 4.2886E+00, 7.0375E+02, &
      3.4600246E+00, 3.32589E+01, 4.2515E+00, 6.9424E+02, &
      3.4773247E+00, 3.32580E+01, 4.2148E+00, 6.8486E+02, &
      3.4947113E+00, 3.32570E+01, 4.1784E+00, 6.7561E+02, &
      3.5121849E+00, 3.32557E+01, 4.1423E+00, 6.6649E+02, &
      3.5297458E+00, 3.32543E+01, 4.1066E+00, 6.5750E+02, &
      3.5473945E+00, 3.32527E+01, 4.0712E+00, 6.4863E+02, &
      3.5651315E+00, 3.32510E+01, 4.0361E+00, 6.3989E+02, &
      3.5829572E+00, 3.32490E+01, 4.0013E+00, 6.3126E+02, &
      3.6008719E+00, 3.32470E+01, 3.9669E+00, 6.2276E+02, &
      3.6188763E+00, 3.32447E+01, 3.9328E+00, 6.1438E+02, &
      3.6369707E+00, 3.32424E+01, 3.8989E+00, 6.0611E+02, &
      3.6551555E+00, 3.32399E+01, 3.8654E+00, 5.9795E+02, &
      3.6734313E+00, 3.32372E+01, 3.8323E+00, 5.8991E+02, &
      3.6917985E+00, 3.32344E+01, 3.7994E+00, 5.8198E+02, &
      3.7102575E+00, 3.32315E+01, 3.7668E+00, 5.7416E+02, &
      3.7288088E+00, 3.32285E+01, 3.7345E+00, 5.6645E+02, &
      3.7474528E+00, 3.32254E+01, 3.7025E+00, 5.5885E+02, &
      3.7661901E+00, 3.32221E+01, 3.6708E+00, 5.5135E+02, &
      3.7850210E+00, 3.32187E+01, 3.6394E+00, 5.4395E+02, &
      3.8039461E+00, 3.32152E+01, 3.6083E+00, 5.3666E+02, &
      3.8229659E+00, 3.32116E+01, 3.5774E+00, 5.2947E+02, &
      3.8420807E+00, 3.32079E+01, 3.5468E+00, 5.2236E+02, &
      3.8612911E+00, 3.32041E+01, 3.5164E+00, 5.1534E+02, &
      3.8805975E+00, 3.32002E+01, 3.4862E+00, 5.0843E+02, &
      3.9000005E+00, 3.31962E+01, 3.4564E+00, 5.0161E+02, &
      3.9195005E+00, 3.31921E+01, 3.4268E+00, 4.9489E+02, &
      3.9390980E+00, 3.31879E+01, 3.3975E+00, 4.8825E+02, &
      3.9587935E+00, 3.31836E+01, 3.3685E+00, 4.8171E+02, &
      3.9785875E+00, 3.31792E+01, 3.3397E+00, 4.7526E+02, &
      3.9984804E+00, 3.31748E+01, 3.3112E+00, 4.6890E+02, &
      4.0184728E+00, 3.31702E+01, 3.2830E+00, 4.6263E+02, &
      4.0385652E+00, 3.31656E+01, 3.2550E+00, 4.5644E+02, &
      4.0587580E+00, 3.31609E+01, 3.2273E+00, 4.5034E+02, &
      4.0790518E+00, 3.31561E+01, 3.1998E+00, 4.4432E+02, &
      4.0994471E+00, 3.31513E+01, 3.1726E+00, 4.3839E+02, &
      4.1199443E+00, 3.31464E+01, 3.1456E+00, 4.3254E+02, &
      4.1405440E+00, 3.31414E+01, 3.1188E+00, 4.2677E+02, &
      4.1612467E+00, 3.31363E+01, 3.0923E+00, 4.2107E+02, &
      4.1820530E+00, 3.31312E+01, 3.0661E+00, 4.1546E+02, &
      4.2029632E+00, 3.31260E+01, 3.0401E+00, 4.0992E+02, &
      4.2239781E+00, 3.31208E+01, 3.0143E+00, 4.0446E+02, &
      4.2450980E+00, 3.31155E+01, 2.9888E+00, 3.9908E+02, &
      4.2663234E+00, 3.31101E+01, 2.9634E+00, 3.9376E+02, &
      4.2876551E+00, 3.31047E+01, 2.9384E+00, 3.8853E+02, &
      4.3090933E+00, 3.30992E+01, 2.9135E+00, 3.8336E+02, &
      4.3306388E+00, 3.30937E+01, 2.8889E+00, 3.7826E+02, &
      4.3522920E+00, 3.30881E+01, 2.8645E+00, 3.7324E+02, &
      4.3740535E+00, 3.30825E+01, 2.8403E+00, 3.6828E+02, &
      4.3959237E+00, 3.30768E+01, 2.8163E+00, 3.6339E+02, &
      4.4179033E+00, 3.30711E+01, 2.7925E+00, 3.5857E+02, &
      4.4399929E+00, 3.30654E+01, 2.7690E+00, 3.5381E+02, &
      4.4621928E+00, 3.30596E+01, 2.7457E+00, 3.4912E+02, &
      4.4845038E+00, 3.30538E+01, 2.7226E+00, 3.4450E+02, &
      4.5069263E+00, 3.30479E+01, 2.6996E+00, 3.3993E+02, &
      4.5294609E+00, 3.30420E+01, 2.6769E+00, 3.3543E+02, &
      4.5521082E+00, 3.30360E+01, 2.6544E+00, 3.3099E+02, &
      4.5748688E+00, 3.30300E+01, 2.6322E+00, 3.2662E+02, &
      4.5977431E+00, 3.30240E+01, 2.6101E+00, 3.2230E+02, &
      4.6207318E+00, 3.30179E+01, 2.5882E+00, 3.1804E+02, &
      4.6438355E+00, 3.30119E+01, 2.5665E+00, 3.1384E+02, &
      4.6670547E+00, 3.30057E+01, 2.5450E+00, 3.0969E+02, &
      4.6903900E+00, 3.29996E+01, 2.5237E+00, 3.0561E+02, &
      4.7138419E+00, 3.29934E+01, 2.5025E+00, 3.0157E+02, &
      4.7374111E+00, 3.29872E+01, 2.4816E+00, 2.9760E+02, &
      4.7610982E+00, 3.29810E+01, 2.4609E+00, 2.9368E+02, &
      4.7849037E+00, 3.29747E+01, 2.4403E+00, 2.8981E+02, &
      4.8088282E+00, 3.29685E+01, 2.4199E+00, 2.8599E+02, &
      4.8328723E+00, 3.29622E+01, 2.3998E+00, 2.8223E+02, &
      4.8570367E+00, 3.29558E+01, 2.3797E+00, 2.7851E+02, &
      4.8813219E+00, 3.29495E+01, 2.3599E+00, 2.7485E+02, &
      4.9057285E+00, 3.29431E+01, 2.3403E+00, 2.7124E+02, &
      4.9302571E+00, 3.29367E+01, 2.3208E+00, 2.6768E+02, &
      4.9549084E+00, 3.29303E+01, 2.3015E+00, 2.6416E+02, &
      4.9796829E+00, 3.29239E+01, 2.2824E+00, 2.6069E+02, &
      5.0045814E+00, 3.29174E+01, 2.2634E+00, 2.5727E+02, &
      5.0296043E+00, 3.29110E+01, 2.2446E+00, 2.5390E+02, &
      5.0547523E+00, 3.29045E+01, 2.2260E+00, 2.5057E+02, &
      5.0800260E+00, 3.28980E+01, 2.2075E+00, 2.4729E+02, &
      5.1054262E+00, 3.28915E+01, 2.1893E+00, 2.4405E+02, &
      5.1309533E+00, 3.28850E+01, 2.1711E+00, 2.4086E+02, &
      5.1566081E+00, 3.28785E+01, 2.1532E+00, 2.3771E+02, &
      5.1823911E+00, 3.28719E+01, 2.1354E+00, 2.3460E+02, &
      5.2083031E+00, 3.28654E+01, 2.1177E+00, 2.3153E+02, &
      5.2343446E+00, 3.28588E+01, 2.1002E+00, 2.2851E+02, &
      5.2605163E+00, 3.28522E+01, 2.0829E+00, 2.2552E+02, &
      5.2868189E+00, 3.28457E+01, 2.0657E+00, 2.2258E+02, &
      5.3132530E+00, 3.28391E+01, 2.0487E+00, 2.1968E+02, &
      5.3398192E+00, 3.28325E+01, 2.0318E+00, 2.1681E+02, &
      5.3665183E+00, 3.28259E+01, 2.0151E+00, 2.1399E+02, &
      5.3933509E+00, 3.28193E+01, 1.9985E+00, 2.1120E+02, &
      5.4203177E+00, 3.28127E+01, 1.9821E+00, 2.0845E+02, &
      5.4474193E+00, 3.28060E+01, 1.9658E+00, 2.0574E+02, &
      5.4746564E+00, 3.27994E+01, 1.9496E+00, 2.0306E+02, &
      5.5020297E+00, 3.27928E+01, 1.9336E+00, 2.0042E+02, &
      5.5295398E+00, 3.27862E+01, 1.9178E+00, 1.9782E+02, &
      5.5571875E+00, 3.27796E+01, 1.9021E+00, 1.9525E+02, &
      5.5849734E+00, 3.27729E+01, 1.8865E+00, 1.9271E+02, &
      5.6128983E+00, 3.27663E+01, 1.8711E+00, 1.9021E+02, &
      5.6409628E+00, 3.27597E+01, 1.8558E+00, 1.8774E+02, &
      5.6691676E+00, 3.27531E+01, 1.8406E+00, 1.8531E+02, &
      5.6975135E+00, 3.27465E+01, 1.8255E+00, 1.8291E+02, &
      5.7260010E+00, 3.27399E+01, 1.8106E+00, 1.8054E+02, &
      5.7546310E+00, 3.27333E+01, 1.7954E+00, 1.7816E+02, &
      5.7834042E+00, 3.27267E+01, 1.7802E+00, 1.7579E+02, &
      5.8123212E+00, 3.27200E+01, 1.7651E+00, 1.7346E+02, &
      5.8413828E+00, 3.27133E+01, 1.7501E+00, 1.7116E+02, &
      5.8705897E+00, 3.27066E+01, 1.7353E+00, 1.6889E+02, &
      5.8999427E+00, 3.26998E+01, 1.7204E+00, 1.6665E+02, &
      5.9294424E+00, 3.26930E+01, 1.7056E+00, 1.6441E+02, &
      5.9590896E+00, 3.26861E+01, 1.6908E+00, 1.6221E+02, &
      5.9888850E+00, 3.26792E+01, 1.6763E+00, 1.6004E+02, &
      6.0188295E+00, 3.26722E+01, 1.6618E+00, 1.5789E+02, &
      6.0489236E+00, 3.26652E+01, 1.6475E+00, 1.5578E+02, &
      6.0791682E+00, 3.26582E+01, 1.6333E+00, 1.5370E+02, &
      6.1095641E+00, 3.26511E+01, 1.6193E+00, 1.5165E+02, &
      6.1401119E+00, 3.26439E+01, 1.6054E+00, 1.4962E+02, &
      6.1708125E+00, 3.26368E+01, 1.5916E+00, 1.4763E+02, &
      6.2016665E+00, 3.26296E+01, 1.5780E+00, 1.4566E+02, &
      6.2326749E+00, 3.26223E+01, 1.5645E+00, 1.4372E+02, &
      6.2638382E+00, 3.26151E+01, 1.5511E+00, 1.4181E+02, &
      6.2951574E+00, 3.26078E+01, 1.5378E+00, 1.3992E+02, &
      6.3266332E+00, 3.26004E+01, 1.5247E+00, 1.3806E+02, &
      6.3582664E+00, 3.25931E+01, 1.5117E+00, 1.3622E+02, &
      6.3900577E+00, 3.25857E+01, 1.4988E+00, 1.3442E+02, &
      6.4220080E+00, 3.25783E+01, 1.4860E+00, 1.3263E+02, &
      6.4541180E+00, 3.25708E+01, 1.4734E+00, 1.3087E+02, &
      6.4863886E+00, 3.25634E+01, 1.4608E+00, 1.2914E+02, &
      6.5188206E+00, 3.25559E+01, 1.4484E+00, 1.2743E+02, &
      6.5514147E+00, 3.25484E+01, 1.4361E+00, 1.2574E+02, &
      6.5841717E+00, 3.25408E+01, 1.4239E+00, 1.2408E+02, &
      6.915365E+00, 3.24812E+01, 1.3052E+00, 1.0857E+02, &
      7.392525E+00, 3.23657E+01, 1.1599E+00, 9.0533E+01, &
      7.902609E+00, 3.22394E+01, 1.0317E+00, 7.5596E+01, &
      8.447890E+00, 3.20983E+01, 9.1860E-01, 6.3205E+01, &
      9.030794E+00, 3.19351E+01, 8.1573E-01, 5.2733E+01, &
      9.653919E+00, 3.17210E+01, 7.1576E-01, 4.3514E+01, &
      1.032004E+01, 3.14141E+01, 6.2865E-01, 3.5964E+01, &
      1.103212E+01, 3.08756E+01, 5.5267E-01, 2.9771E+01, &
      1.162937E+01, 2.96992E+01, 4.9952E-01, 2.5671E+01, &
      1.179334E+01, 2.85199E+01, 4.8634E-01, 2.4685E+01, &
      1.180737E+01, 2.83012E+01, 4.8523E-01, 2.4603E+01, &
      1.185483E+01, 2.66121E+01, 4.8153E-01, 2.4329E+01, &
      1.192603E+01, 2.83415E+01, 3.8077E+00, 1.8084E+02, &
      1.210403E+01, 2.99049E+01, 3.6910E+00, 1.7275E+02, &
      1.260708E+01, 3.11969E+01, 3.4005E+00, 1.5292E+02, &
      1.347697E+01, 3.20334E+01, 3.0358E+00, 1.2784E+02, &
      1.440688E+01, 3.24817E+01, 2.7327E+00, 1.0776E+02, &
      1.540095E+01, 3.27814E+01, 2.4677E+00, 9.1130E+01, &
      1.646362E+01, 3.29930E+01, 2.2092E+00, 7.6420E+01, &
      1.759961E+01, 3.31366E+01, 1.9737E+00, 6.3964E+01, &
      1.881398E+01, 3.32347E+01, 1.7605E+00, 5.3461E+01, &
      2.011215E+01, 3.33002E+01, 1.5688E+00, 4.4646E+01, &
      2.149988E+01, 3.33421E+01, 1.3973E+00, 3.7277E+01, &
      2.298338E+01, 3.33674E+01, 1.2448E+00, 3.1135E+01, &
      2.456923E+01, 3.34162E+01, 1.1074E+00, 2.5977E+01, &
      2.626450E+01, 3.34187E+01, 9.8363E-01, 2.1648E+01, &
      2.807676E+01, 3.34175E+01, 8.6938E-01, 1.7958E+01, &
      3.001405E+01, 3.34061E+01, 7.6708E-01, 1.4879E+01, &
      3.208502E+01, 3.33899E+01, 6.7685E-01, 1.2336E+01, &
      3.429889E+01, 3.33710E+01, 5.9696E-01, 1.0228E+01, &
      3.666551E+01, 3.33504E+01, 5.2646E-01, 8.4847E+00, &
      3.919543E+01, 3.33290E+01, 4.6432E-01, 7.0445E+00, &
      4.189992E+01, 3.33075E+01, 4.0953E-01, 5.8541E+00, &
      4.479101E+01, 3.32864E+01, 3.6124E-01, 4.8699E+00, &
      4.788159E+01, 3.32661E+01, 3.1866E-01, 4.0561E+00, &
      5.118542E+01, 3.32468E+01, 2.8111E-01, 3.3828E+00, &
      5.471721E+01, 3.32282E+01, 2.4662E-01, 2.8115E+00, &
      5.849270E+01, 3.32101E+01, 2.1628E-01, 2.3402E+00, &
      6.252870E+01, 3.31928E+01, 1.8968E-01, 1.9524E+00, &
      6.684318E+01, 3.31765E+01, 1.6636E-01, 1.6328E+00, &
      7.145536E+01, 3.31613E+01, 1.4591E-01, 1.3695E+00, &
      7.638578E+01, 3.31471E+01, 1.2798E-01, 1.1524E+00, &
      8.165640E+01, 3.31340E+01, 1.1213E-01, 9.7237E-01, &
      8.729069E+01, 3.31218E+01, 9.7993E-02, 8.2229E-01, &
      9.331374E+01, 3.31105E+01, 8.5642E-02, 6.9871E-01, &
      9.975239E+01, 3.31000E+01, 7.4852E-02, 5.9687E-01, &
      1.066353E+02, 3.30904E+01, 6.5423E-02, 5.1283E-01, &
      1.139931E+02, 3.30816E+01, 5.7184E-02, 4.4337E-01, &
      1.218587E+02, 3.30736E+01, 4.9985E-02, 3.8590E-01, &
      1.302669E+02, 3.30663E+01, 4.3694E-02, 3.3823E-01, &
      1.392553E+02, 3.30597E+01, 3.8196E-02, 2.9861E-01, &
      1.488640E+02, 3.30537E+01, 3.3391E-02, 2.6560E-01, &
      1.591356E+02, 3.30482E+01, 2.9191E-02, 2.3801E-01, &
      1.701159E+02, 3.30433E+01, 2.5521E-02, 2.1487E-01, &
      1.818539E+02, 3.30388E+01, 2.2313E-02, 1.9538E-01, &
      1.944018E+02, 3.30348E+01, 1.9508E-02, 1.7889E-01, &
      2.078156E+02, 3.30312E+01, 1.7057E-02, 1.6487E-01, &
      2.221548E+02, 3.30279E+01, 1.4915E-02, 1.5288E-01, &
      2.374835E+02, 3.30250E+01, 1.3041E-02, 1.4255E-01, &
      2.538699E+02, 3.30223E+01, 1.1404E-02, 1.3361E-01, &
      2.713869E+02, 3.30200E+01, 9.9724E-03, 1.2579E-01, &
      2.901126E+02, 3.30178E+01, 8.7208E-03, 1.1890E-01, &
      3.101304E+02, 3.30159E+01, 7.6265E-03, 1.1279E-01, &
      3.315294E+02, 3.30142E+01, 6.6698E-03, 1.0731E-01, &
      3.544049E+02, 3.30127E+01, 5.8332E-03, 1.0236E-01, &
      3.788588E+02, 3.30113E+01, 5.1018E-03, 9.7856E-02, &
      4.050001E+02, 3.30101E+01, 4.4622E-03, 9.3718E-02, &
      4.329451E+02, 3.30090E+01, 3.9029E-03, 8.9889E-02]
   case(34)
      frel = -1.2900E-01
      allocate(nist_tab(2232))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 7.12170E+00, 0.0000E+00, 1.4390E-04, &
      1.142761E-02, 7.11201E+00, 0.0000E+00, 1.9625E-04, &
      1.221612E-02, 7.10091E+00, 0.0000E+00, 2.6619E-04, &
      1.305903E-02, 7.08817E+00, 0.0000E+00, 3.5912E-04, &
      1.396010E-02, 7.07355E+00, 0.0000E+00, 4.8192E-04, &
      1.492335E-02, 7.05677E+00, 0.0000E+00, 6.4327E-04, &
      1.595306E-02, 7.03750E+00, 0.0000E+00, 8.5413E-04, &
      1.705382E-02, 7.01534E+00, 0.0000E+00, 1.1282E-03, &
      1.823053E-02, 6.98983E+00, 0.0000E+00, 1.4824E-03, &
      1.948844E-02, 6.96044E+00, 0.0000E+00, 1.9378E-03, &
      2.083314E-02, 6.92655E+00, 0.0000E+00, 2.5200E-03, &
      2.227063E-02, 6.88738E+00, 0.0000E+00, 3.2605E-03, &
      2.380730E-02, 6.84205E+00, 0.0000E+00, 4.1972E-03, &
      2.545001E-02, 6.78948E+00, 0.0000E+00, 5.3758E-03, &
      2.720606E-02, 6.72835E+00, 0.0000E+00, 6.8508E-03, &
      2.908327E-02, 6.65705E+00, 0.0000E+00, 8.6873E-03, &
      3.109002E-02, 6.57359E+00, 0.0000E+00, 1.0962E-02, &
      3.323523E-02, 6.47542E+00, 0.0000E+00, 1.3764E-02, &
      3.552846E-02, 6.35930E+00, 0.0000E+00, 1.7198E-02, &
      3.797993E-02, 6.22090E+00, 0.0000E+00, 2.1385E-02, &
      4.060054E-02, 6.05430E+00, 0.0000E+00, 2.6463E-02, &
      4.340198E-02, 5.85093E+00, 0.0000E+00, 3.2591E-02, &
      4.639671E-02, 5.59743E+00, 0.0000E+00, 3.9948E-02, &
      4.959809E-02, 5.26996E+00, 0.0000E+00, 4.8736E-02, &
      5.302035E-02, 4.81237E+00, 0.0000E+00, 5.9178E-02, &
      5.556600E-02, 4.28445E+00, 0.0000E+00, 6.7641E-02, &
      5.641650E-02, 3.91211E+00, 0.0000E+00, 7.0600E-02, &
      5.667876E-02, 3.42611E+00, 0.0000E+00, 7.1525E-02, &
      5.698350E-02, 3.79296E+00, 5.4255E-01, 5.0742E+03, &
      5.783400E-02, 3.87146E+00, 6.0226E-01, 5.5498E+03, &
      6.058959E-02, 3.62586E+00, 8.3607E-01, 7.3540E+03, &
      6.477028E-02, 3.11653E+00, 1.3253E+00, 1.0905E+04, &
      6.923942E-02, 2.60474E+00, 1.9914E+00, 1.5328E+04, &
      7.401695E-02, 2.15098E+00, 2.8356E+00, 2.0417E+04, &
      7.912411E-02, 1.81009E+00, 3.8477E+00, 2.5916E+04, &
      8.458368E-02, 1.63040E+00, 5.0006E+00, 3.1507E+04, &
      9.041995E-02, 1.64914E+00, 6.2528E+00, 3.6854E+04, &
      9.665893E-02, 1.88864E+00, 7.5533E+00, 4.1645E+04, &
      1.033284E-01, 2.35448E+00, 8.8472E+00, 4.5631E+04, &
      1.104581E-01, 3.03577E+00, 1.0081E+01, 4.8640E+04, &
      1.180797E-01, 3.90686E+00, 1.1208E+01, 5.0587E+04, &
      1.262272E-01, 4.92969E+00, 1.2190E+01, 5.1468E+04, &
      1.349368E-01, 6.05516E+00, 1.3000E+01, 5.1346E+04, &
      1.442475E-01, 7.21906E+00, 1.3623E+01, 5.0332E+04, &
      1.542005E-01, 8.30675E+00, 1.4052E+01, 4.8566E+04, &
      1.586620E-01, 8.64723E+00, 1.4177E+01, 4.7620E+04, &
      1.610905E-01, 8.63184E+00, 1.4229E+01, 4.7076E+04, &
      1.617381E-01, 8.38352E+00, 1.4242E+01, 4.6928E+04, &
      1.627095E-01, 8.77941E+00, 1.4871E+01, 4.8709E+04, &
      1.648404E-01, 9.21656E+00, 1.4928E+01, 4.8262E+04, &
      1.651380E-01, 9.25960E+00, 1.4935E+01, 4.8199E+04, &
      1.673590E-01, 9.49061E+00, 1.4986E+01, 4.7722E+04, &
      1.680318E-01, 9.44902E+00, 1.5000E+01, 4.7576E+04, &
      1.690410E-01, 9.71715E+00, 1.5287E+01, 4.8196E+04, &
      1.715640E-01, 1.01672E+01, 1.5345E+01, 4.7667E+04, &
      1.762144E-01, 1.08049E+01, 1.5433E+01, 4.6676E+04, &
      1.883732E-01, 1.22345E+01, 1.5564E+01, 4.4033E+04, &
      2.013709E-01, 1.35635E+01, 1.5559E+01, 4.1177E+04, &
      2.152655E-01, 1.47764E+01, 1.5427E+01, 3.8193E+04, &
      2.268700E-01, 1.55251E+01, 1.5243E+01, 3.5807E+04, &
      2.301188E-01, 1.55403E+01, 1.5181E+01, 3.5159E+04, &
      2.303425E-01, 1.55218E+01, 1.5177E+01, 3.5114E+04, &
      2.312685E-01, 1.52643E+01, 1.5158E+01, 3.4931E+04, &
      2.326575E-01, 1.57024E+01, 1.5777E+01, 3.6140E+04, &
      2.361300E-01, 1.62554E+01, 1.5710E+01, 3.5457E+04, &
      2.459970E-01, 1.72195E+01, 1.5497E+01, 3.3574E+04, &
      2.629708E-01, 1.85115E+01, 1.5073E+01, 3.0549E+04, &
      2.811158E-01, 1.96545E+01, 1.4566E+01, 2.7616E+04, &
      3.005128E-01, 2.06785E+01, 1.3993E+01, 2.4817E+04, &
      3.212482E-01, 2.15921E+01, 1.3372E+01, 2.2186E+04, &
      3.434143E-01, 2.24007E+01, 1.2719E+01, 1.9740E+04, &
      3.671099E-01, 2.31097E+01, 1.2047E+01, 1.7490E+04, &
      3.924405E-01, 2.37249E+01, 1.1368E+01, 1.5440E+04, &
      4.195189E-01, 2.42525E+01, 1.0691E+01, 1.3584E+04, &
      4.484657E-01, 2.46988E+01, 1.0025E+01, 1.1916E+04, &
      4.794098E-01, 2.50701E+01, 9.3760E+00, 1.0426E+04, &
      5.124891E-01, 2.53770E+01, 8.7484E+00, 9.1007E+03, &
      5.478508E-01, 2.56153E+01, 8.1457E+00, 7.9275E+03, &
      5.856525E-01, 2.57945E+01, 7.5704E+00, 6.8926E+03, &
      6.260625E-01, 2.59182E+01, 7.0238E+00, 5.9829E+03, &
      6.692609E-01, 2.59889E+01, 6.5064E+00, 5.1852E+03, &
      7.154399E-01, 2.60077E+01, 6.0170E+00, 4.4863E+03, &
      7.648052E-01, 2.59709E+01, 5.5560E+00, 3.8759E+03, &
      8.175768E-01, 2.58757E+01, 5.1241E+00, 3.3446E+03, &
      8.739896E-01, 2.57157E+01, 4.7216E+00, 2.8837E+03, &
      9.0000000E-01, 2.60490E+01, 4.5539E+00, 2.7013E+03, &
      9.0450000E-01, 2.60344E+01, 4.5259E+00, 2.6714E+03, &
      9.0902250E-01, 2.60195E+01, 4.4981E+00, 2.6418E+03, &
      9.1356761E-01, 2.60042E+01, 4.4705E+00, 2.6126E+03, &
      9.1813545E-01, 2.59885E+01, 4.4430E+00, 2.5837E+03, &
      9.2272613E-01, 2.59723E+01, 4.4157E+00, 2.5551E+03, &
      9.2733976E-01, 2.59558E+01, 4.3885E+00, 2.5268E+03, &
      9.3197646E-01, 2.59388E+01, 4.3615E+00, 2.4988E+03, &
      9.3663634E-01, 2.59214E+01, 4.3347E+00, 2.4711E+03, &
      9.4131952E-01, 2.59036E+01, 4.3080E+00, 2.4438E+03, &
      9.4602612E-01, 2.58853E+01, 4.2815E+00, 2.4167E+03, &
      9.5075625E-01, 2.58666E+01, 4.2551E+00, 2.3899E+03, &
      9.5551003E-01, 2.58475E+01, 4.2289E+00, 2.3635E+03, &
      9.6028758E-01, 2.58279E+01, 4.2029E+00, 2.3373E+03, &
      9.6508902E-01, 2.58078E+01, 4.1770E+00, 2.3114E+03, &
      9.6991446E-01, 2.57872E+01, 4.1513E+00, 2.2858E+03, &
      9.7476404E-01, 2.57662E+01, 4.1257E+00, 2.2605E+03, &
      9.7963786E-01, 2.57447E+01, 4.1003E+00, 2.2354E+03, &
      9.8453605E-01, 2.57227E+01, 4.0750E+00, 2.2107E+03, &
      9.8945873E-01, 2.57002E+01, 4.0499E+00, 2.1862E+03, &
      9.9440602E-01, 2.56772E+01, 4.0250E+00, 2.1620E+03, &
      9.9937805E-01, 2.56537E+01, 4.0002E+00, 2.1380E+03, &
      1.0043749E+00, 2.56296E+01, 3.9699E+00, 2.1114E+03, &
      1.0093968E+00, 2.56045E+01, 3.9392E+00, 2.0846E+03, &
      1.0144438E+00, 2.55785E+01, 3.9086E+00, 2.0583E+03, &
      1.0195160E+00, 2.55515E+01, 3.8784E+00, 2.0322E+03, &
      1.0246136E+00, 2.55236E+01, 3.8484E+00, 2.0065E+03, &
      1.0297367E+00, 2.54946E+01, 3.8186E+00, 1.9812E+03, &
      1.0348853E+00, 2.54647E+01, 3.7891E+00, 1.9562E+03, &
      1.0400598E+00, 2.54338E+01, 3.7599E+00, 1.9315E+03, &
      1.0452601E+00, 2.54018E+01, 3.7309E+00, 1.9071E+03, &
      1.0504864E+00, 2.53689E+01, 3.7021E+00, 1.8831E+03, &
      1.0557388E+00, 2.53349E+01, 3.6736E+00, 1.8593E+03, &
      1.0610175E+00, 2.52998E+01, 3.6453E+00, 1.8359E+03, &
      1.0663226E+00, 2.52637E+01, 3.6166E+00, 1.8125E+03, &
      1.0716542E+00, 2.52264E+01, 3.5868E+00, 1.7886E+03, &
      1.0770125E+00, 2.51879E+01, 3.5572E+00, 1.7651E+03, &
      1.0823975E+00, 2.51482E+01, 3.5278E+00, 1.7419E+03, &
      1.0878095E+00, 2.51072E+01, 3.4988E+00, 1.7191E+03, &
      1.0932486E+00, 2.50649E+01, 3.4700E+00, 1.6965E+03, &
      1.0987148E+00, 2.50213E+01, 3.4415E+00, 1.6743E+03, &
      1.1042084E+00, 2.49764E+01, 3.4133E+00, 1.6524E+03, &
      1.1097294E+00, 2.49300E+01, 3.3853E+00, 1.6307E+03, &
      1.1152781E+00, 2.48821E+01, 3.3576E+00, 1.6094E+03, &
      1.1208545E+00, 2.48328E+01, 3.3302E+00, 1.5884E+03, &
      1.1264587E+00, 2.47820E+01, 3.3030E+00, 1.5677E+03, &
      1.1320910E+00, 2.47295E+01, 3.2760E+00, 1.5472E+03, &
      1.1377515E+00, 2.46754E+01, 3.2494E+00, 1.5270E+03, &
      1.1434402E+00, 2.46197E+01, 3.2229E+00, 1.5071E+03, &
      1.1491574E+00, 2.45621E+01, 3.1967E+00, 1.4875E+03, &
      1.1549032E+00, 2.45027E+01, 3.1708E+00, 1.4682E+03, &
      1.1606777E+00, 2.44414E+01, 3.1451E+00, 1.4491E+03, &
      1.1664811E+00, 2.43781E+01, 3.1196E+00, 1.4303E+03, &
      1.1723135E+00, 2.43127E+01, 3.0944E+00, 1.4117E+03, &
      1.1781751E+00, 2.42451E+01, 3.0694E+00, 1.3934E+03, &
      1.1840660E+00, 2.41752E+01, 3.0447E+00, 1.3754E+03, &
      1.1899863E+00, 2.41029E+01, 3.0201E+00, 1.3576E+03, &
      1.1959362E+00, 2.40281E+01, 2.9958E+00, 1.3400E+03, &
      1.2019159E+00, 2.39506E+01, 2.9717E+00, 1.3227E+03, &
      1.2079255E+00, 2.38704E+01, 2.9479E+00, 1.3057E+03, &
      1.2139651E+00, 2.37872E+01, 2.9242E+00, 1.2888E+03, &
      1.2200350E+00, 2.37008E+01, 2.9008E+00, 1.2722E+03, &
      1.2261351E+00, 2.36111E+01, 2.8776E+00, 1.2558E+03, &
      1.2322658E+00, 2.35179E+01, 2.8546E+00, 1.2396E+03, &
      1.2384271E+00, 2.34209E+01, 2.8318E+00, 1.2237E+03, &
      1.2446193E+00, 2.33199E+01, 2.8093E+00, 1.2080E+03, &
      1.2508424E+00, 2.32146E+01, 2.7869E+00, 1.1925E+03, &
      1.2570966E+00, 2.31047E+01, 2.7647E+00, 1.1772E+03, &
      1.2633821E+00, 2.29898E+01, 2.7428E+00, 1.1621E+03, &
      1.2696990E+00, 2.28696E+01, 2.7210E+00, 1.1472E+03, &
      1.2760475E+00, 2.27437E+01, 2.6995E+00, 1.1325E+03, &
      1.2824277E+00, 2.26114E+01, 2.6781E+00, 1.1180E+03, &
      1.2888399E+00, 2.24723E+01, 2.6569E+00, 1.1037E+03, &
      1.2952840E+00, 2.23258E+01, 2.6359E+00, 1.0896E+03, &
      1.3017605E+00, 2.21711E+01, 2.6152E+00, 1.0757E+03, &
      1.3082693E+00, 2.20073E+01, 2.5946E+00, 1.0620E+03, &
      1.3148106E+00, 2.18335E+01, 2.5741E+00, 1.0485E+03, &
      1.3213847E+00, 2.16486E+01, 2.5539E+00, 1.0351E+03, &
      1.3279916E+00, 2.14512E+01, 2.5339E+00, 1.0220E+03, &
      1.3346316E+00, 2.12397E+01, 2.5140E+00, 1.0090E+03, &
      1.3413047E+00, 2.10122E+01, 2.4943E+00, 9.9618E+02, &
      1.3480112E+00, 2.07662E+01, 2.4748E+00, 9.8354E+02, &
      1.3547513E+00, 2.04989E+01, 2.4555E+00, 9.7107E+02, &
      1.3615250E+00, 2.02066E+01, 2.4363E+00, 9.5876E+02, &
      1.3683327E+00, 1.98845E+01, 2.4173E+00, 9.4663E+02, &
      1.3751743E+00, 1.95263E+01, 2.3985E+00, 9.3465E+02, &
      1.3820502E+00, 1.91236E+01, 2.3799E+00, 9.2284E+02, &
      1.3889605E+00, 1.86644E+01, 2.3614E+00, 9.1118E+02, &
      1.3959053E+00, 1.81311E+01, 2.3431E+00, 8.9968E+02, &
      1.4028848E+00, 1.74959E+01, 2.3249E+00, 8.8834E+02, &
      1.4098992E+00, 1.67115E+01, 2.3069E+00, 8.7715E+02, &
      1.4169487E+00, 1.56854E+01, 2.2891E+00, 8.6610E+02, &
      1.4240335E+00, 1.41918E+01, 2.2714E+00, 8.5521E+02, &
      1.4311536E+00, 1.13302E+01, 2.2539E+00, 8.4446E+02, &
      1.4356568E+00, 9.59582E-01, 2.2430E+00, 8.3776E+02, &
      1.4359431E+00, 7.98914E-01, 1.1542E+01, 4.2888E+03, &
      1.4383094E+00, 9.23944E+00, 1.1512E+01, 4.2705E+03, &
      1.4455009E+00, 1.30471E+01, 1.1420E+01, 4.2154E+03, &
      1.4527284E+00, 1.43962E+01, 1.1329E+01, 4.1610E+03, &
      1.4599921E+00, 1.49963E+01, 1.1238E+01, 4.1074E+03, &
      1.4672920E+00, 1.49826E+01, 1.1149E+01, 4.0544E+03, &
      1.4746285E+00, 1.31496E+01, 1.1060E+01, 4.0022E+03, &
      1.4760494E+00, 9.84436E+00, 1.1043E+01, 3.9922E+03, &
      1.4763506E+00, 9.79284E+00, 1.5634E+01, 5.6488E+03, &
      1.4820016E+00, 1.55867E+01, 1.5539E+01, 5.5931E+03, &
      1.4894117E+00, 1.72959E+01, 1.5416E+01, 5.5211E+03, &
      1.4968587E+00, 1.83878E+01, 1.5293E+01, 5.4501E+03, &
      1.5043430E+00, 1.92260E+01, 1.5172E+01, 5.3801E+03, &
      1.5118647E+00, 1.99166E+01, 1.5052E+01, 5.3109E+03, &
      1.5194240E+00, 2.05070E+01, 1.4932E+01, 5.2426E+03, &
      1.5270212E+00, 2.10233E+01, 1.4814E+01, 5.1752E+03, &
      1.5346563E+00, 2.14817E+01, 1.4696E+01, 5.1087E+03, &
      1.5423295E+00, 2.18929E+01, 1.4580E+01, 5.0431E+03, &
      1.5500412E+00, 2.22641E+01, 1.4464E+01, 4.9783E+03, &
      1.5577914E+00, 2.26008E+01, 1.4350E+01, 4.9143E+03, &
      1.5655804E+00, 2.29069E+01, 1.4236E+01, 4.8512E+03, &
      1.5734083E+00, 2.31850E+01, 1.4123E+01, 4.7888E+03, &
      1.5812753E+00, 2.34368E+01, 1.4011E+01, 4.7273E+03, &
      1.5891817E+00, 2.36634E+01, 1.3900E+01, 4.6666E+03, &
      1.5971276E+00, 2.38645E+01, 1.3790E+01, 4.6067E+03, &
      1.6051132E+00, 2.40389E+01, 1.3681E+01, 4.5475E+03, &
      1.6131388E+00, 2.41834E+01, 1.3573E+01, 4.4891E+03, &
      1.6212045E+00, 2.42915E+01, 1.3465E+01, 4.4315E+03, &
      1.6293105E+00, 2.43501E+01, 1.3359E+01, 4.3746E+03, &
      1.6374571E+00, 2.43277E+01, 1.3253E+01, 4.3185E+03, &
      1.6456443E+00, 2.41226E+01, 1.3148E+01, 4.2631E+03, &
      1.6524595E+00, 2.32318E+01, 1.3062E+01, 4.2177E+03, &
      1.6538726E+00, 2.08530E+01, 1.3044E+01, 4.2084E+03, &
      1.6553406E+00, 2.32748E+01, 1.4940E+01, 4.8150E+03, &
      1.6621419E+00, 2.45031E+01, 1.4848E+01, 4.7659E+03, &
      1.6704526E+00, 2.51190E+01, 1.4737E+01, 4.7069E+03, &
      1.6788049E+00, 2.55503E+01, 1.4627E+01, 4.6485E+03, &
      1.6871989E+00, 2.59020E+01, 1.4518E+01, 4.5910E+03, &
      1.6956349E+00, 2.62073E+01, 1.4410E+01, 4.5341E+03, &
      1.7041131E+00, 2.64811E+01, 1.4302E+01, 4.4780E+03, &
      1.7126337E+00, 2.67317E+01, 1.4196E+01, 4.4225E+03, &
      1.7211968E+00, 2.69639E+01, 1.4090E+01, 4.3678E+03, &
      1.7298028E+00, 2.71809E+01, 1.3985E+01, 4.3137E+03, &
      1.7384518E+00, 2.73848E+01, 1.3882E+01, 4.2606E+03, &
      1.7471441E+00, 2.75781E+01, 1.3780E+01, 4.2084E+03, &
      1.7558798E+00, 2.77624E+01, 1.3679E+01, 4.1569E+03, &
      1.7646592E+00, 2.79384E+01, 1.3579E+01, 4.1061E+03, &
      1.7734825E+00, 2.81070E+01, 1.3481E+01, 4.0560E+03, &
      1.7823499E+00, 2.82688E+01, 1.3383E+01, 4.0065E+03, &
      1.7912617E+00, 2.84245E+01, 1.3285E+01, 3.9577E+03, &
      1.8002180E+00, 2.85744E+01, 1.3189E+01, 3.9095E+03, &
      1.8092191E+00, 2.87191E+01, 1.3093E+01, 3.8620E+03, &
      1.8182652E+00, 2.88590E+01, 1.2999E+01, 3.8150E+03, &
      1.8273565E+00, 2.89943E+01, 1.2905E+01, 3.7687E+03, &
      1.8364933E+00, 2.91253E+01, 1.2812E+01, 3.7229E+03, &
      1.8456757E+00, 2.92525E+01, 1.2719E+01, 3.6778E+03, &
      1.8549041E+00, 2.93760E+01, 1.2628E+01, 3.6332E+03, &
      1.8641786E+00, 2.94961E+01, 1.2537E+01, 3.5892E+03, &
      1.8734995E+00, 2.96128E+01, 1.2446E+01, 3.5454E+03, &
      1.8828670E+00, 2.97260E+01, 1.2355E+01, 3.5020E+03, &
      1.8922814E+00, 2.98359E+01, 1.2264E+01, 3.4591E+03, &
      1.9017428E+00, 2.99427E+01, 1.2175E+01, 3.4168E+03, &
      1.9112515E+00, 3.00466E+01, 1.2086E+01, 3.3750E+03, &
      1.9208077E+00, 3.01477E+01, 1.1997E+01, 3.3337E+03, &
      1.9304118E+00, 3.02460E+01, 1.1909E+01, 3.2927E+03, &
      1.9400638E+00, 3.03416E+01, 1.1821E+01, 3.2522E+03, &
      1.9497642E+00, 3.04345E+01, 1.1733E+01, 3.2122E+03, &
      1.9595130E+00, 3.05249E+01, 1.1647E+01, 3.1727E+03, &
      1.9693105E+00, 3.06130E+01, 1.1561E+01, 3.1337E+03, &
      1.9791571E+00, 3.06988E+01, 1.1476E+01, 3.0952E+03, &
      1.9890529E+00, 3.07824E+01, 1.1391E+01, 3.0571E+03, &
      1.9989981E+00, 3.08638E+01, 1.1307E+01, 3.0196E+03, &
      2.0089931E+00, 3.09433E+01, 1.1224E+01, 2.9825E+03, &
      2.0190381E+00, 3.10208E+01, 1.1141E+01, 2.9458E+03, &
      2.0291333E+00, 3.10964E+01, 1.1059E+01, 2.9096E+03, &
      2.0392790E+00, 3.11702E+01, 1.0978E+01, 2.8739E+03, &
      2.0494754E+00, 3.12423E+01, 1.0897E+01, 2.8386E+03, &
      2.0597227E+00, 3.13127E+01, 1.0817E+01, 2.8037E+03, &
      2.0700213E+00, 3.13815E+01, 1.0737E+01, 2.7693E+03, &
      2.0803714E+00, 3.14487E+01, 1.0658E+01, 2.7353E+03, &
      2.0907733E+00, 3.15144E+01, 1.0580E+01, 2.7017E+03, &
      2.1012272E+00, 3.15787E+01, 1.0502E+01, 2.6686E+03, &
      2.1117333E+00, 3.16416E+01, 1.0425E+01, 2.6358E+03, &
      2.1222920E+00, 3.17031E+01, 1.0348E+01, 2.6034E+03, &
      2.1329034E+00, 3.17634E+01, 1.0272E+01, 2.5715E+03, &
      2.1435680E+00, 3.18225E+01, 1.0196E+01, 2.5399E+03, &
      2.1542858E+00, 3.18805E+01, 1.0121E+01, 2.5086E+03, &
      2.1650572E+00, 3.19370E+01, 1.0045E+01, 2.4775E+03, &
      2.1758825E+00, 3.19920E+01, 9.9697E+00, 2.4468E+03, &
      2.1867619E+00, 3.20458E+01, 9.8950E+00, 2.4164E+03, &
      2.1976957E+00, 3.20982E+01, 9.8209E+00, 2.3864E+03, &
      2.2086842E+00, 3.21494E+01, 9.7473E+00, 2.3568E+03, &
      2.2197276E+00, 3.21994E+01, 9.6742E+00, 2.3276E+03, &
      2.2308263E+00, 3.22482E+01, 9.6016E+00, 2.2987E+03, &
      2.2419804E+00, 3.22959E+01, 9.5296E+00, 2.2701E+03, &
      2.2531903E+00, 3.23425E+01, 9.4581E+00, 2.2419E+03, &
      2.2644562E+00, 3.23880E+01, 9.3871E+00, 2.2141E+03, &
      2.2757785E+00, 3.24325E+01, 9.3166E+00, 2.1866E+03, &
      2.2871574E+00, 3.24760E+01, 9.2467E+00, 2.1594E+03, &
      2.2985932E+00, 3.25185E+01, 9.1772E+00, 2.1326E+03, &
      2.3100862E+00, 3.25601E+01, 9.1083E+00, 2.1061E+03, &
      2.3216366E+00, 3.26007E+01, 9.0399E+00, 2.0799E+03, &
      2.3332448E+00, 3.26404E+01, 8.9719E+00, 2.0541E+03, &
      2.3449110E+00, 3.26792E+01, 8.9045E+00, 2.0286E+03, &
      2.3566356E+00, 3.27172E+01, 8.8376E+00, 2.0034E+03, &
      2.3684187E+00, 3.27543E+01, 8.7711E+00, 1.9785E+03, &
      2.3802608E+00, 3.27906E+01, 8.7052E+00, 1.9539E+03, &
      2.3921621E+00, 3.28261E+01, 8.6397E+00, 1.9296E+03, &
      2.4041230E+00, 3.28608E+01, 8.5747E+00, 1.9056E+03, &
      2.4161436E+00, 3.28947E+01, 8.5102E+00, 1.8819E+03, &
      2.4282243E+00, 3.29279E+01, 8.4462E+00, 1.8585E+03, &
      2.4403654E+00, 3.29604E+01, 8.3826E+00, 1.8354E+03, &
      2.4525672E+00, 3.29922E+01, 8.3195E+00, 1.8126E+03, &
      2.4648301E+00, 3.30232E+01, 8.2569E+00, 1.7900E+03, &
      2.4771542E+00, 3.30536E+01, 8.1948E+00, 1.7678E+03, &
      2.4895400E+00, 3.30834E+01, 8.1331E+00, 1.7458E+03, &
      2.5019877E+00, 3.31125E+01, 8.0719E+00, 1.7241E+03, &
      2.5144976E+00, 3.31409E+01, 8.0112E+00, 1.7027E+03, &
      2.5270701E+00, 3.31688E+01, 7.9509E+00, 1.6815E+03, &
      2.5397055E+00, 3.31960E+01, 7.8911E+00, 1.6606E+03, &
      2.5524040E+00, 3.32227E+01, 7.8317E+00, 1.6399E+03, &
      2.5651660E+00, 3.32488E+01, 7.7728E+00, 1.6195E+03, &
      2.5779919E+00, 3.32743E+01, 7.7143E+00, 1.5994E+03, &
      2.5908818E+00, 3.32993E+01, 7.6562E+00, 1.5795E+03, &
      2.6038362E+00, 3.33238E+01, 7.5986E+00, 1.5599E+03, &
      2.6168554E+00, 3.33478E+01, 7.5415E+00, 1.5405E+03, &
      2.6299397E+00, 3.33712E+01, 7.4848E+00, 1.5214E+03, &
      2.6430894E+00, 3.33942E+01, 7.4285E+00, 1.5025E+03, &
      2.6563048E+00, 3.34167E+01, 7.3726E+00, 1.4838E+03, &
      2.6695863E+00, 3.34388E+01, 7.3172E+00, 1.4654E+03, &
      2.6829343E+00, 3.34604E+01, 7.2622E+00, 1.4472E+03, &
      2.6963489E+00, 3.34816E+01, 7.2076E+00, 1.4292E+03, &
      2.7098307E+00, 3.35024E+01, 7.1535E+00, 1.4115E+03, &
      2.7233798E+00, 3.35228E+01, 7.0998E+00, 1.3939E+03, &
      2.7369967E+00, 3.35427E+01, 7.0464E+00, 1.3766E+03, &
      2.7506817E+00, 3.35624E+01, 6.9935E+00, 1.3595E+03, &
      2.7644351E+00, 3.35816E+01, 6.9411E+00, 1.3427E+03, &
      2.7782573E+00, 3.36006E+01, 6.8890E+00, 1.3260E+03, &
      2.7921486E+00, 3.36192E+01, 6.8373E+00, 1.3096E+03, &
      2.8061093E+00, 3.36375E+01, 6.7860E+00, 1.2933E+03, &
      2.8201399E+00, 3.36555E+01, 6.7352E+00, 1.2773E+03, &
      2.8342406E+00, 3.36732E+01, 6.6847E+00, 1.2615E+03, &
      2.8484118E+00, 3.36907E+01, 6.6346E+00, 1.2458E+03, &
      2.8626539E+00, 3.37080E+01, 6.5849E+00, 1.2304E+03, &
      2.8769671E+00, 3.38286E+01, 6.5352E+00, 1.2151E+03, &
      2.8913520E+00, 3.38455E+01, 6.4853E+00, 1.1999E+03, &
      2.9058087E+00, 3.38621E+01, 6.4357E+00, 1.1848E+03, &
      2.9203378E+00, 3.38785E+01, 6.3865E+00, 1.1699E+03, &
      2.9349394E+00, 3.38948E+01, 6.3377E+00, 1.1553E+03, &
      2.9496141E+00, 3.39110E+01, 6.2893E+00, 1.1408E+03, &
      2.9643622E+00, 3.39811E+01, 6.2408E+00, 1.1264E+03, &
      2.9791840E+00, 3.39974E+01, 6.1927E+00, 1.1122E+03, &
      2.9940799E+00, 3.40140E+01, 6.1449E+00, 1.0982E+03, &
      3.0090503E+00, 3.40286E+01, 6.0945E+00, 1.0838E+03, &
      3.0240956E+00, 3.40413E+01, 6.0426E+00, 1.0693E+03, &
      3.0392161E+00, 3.40534E+01, 5.9911E+00, 1.0550E+03, &
      3.0544122E+00, 3.40649E+01, 5.9398E+00, 1.0408E+03, &
      3.0696842E+00, 3.40758E+01, 5.8889E+00, 1.0268E+03, &
      3.0850326E+00, 3.40862E+01, 5.8385E+00, 1.0129E+03, &
      3.1004578E+00, 3.40961E+01, 5.7885E+00, 9.9934E+02, &
      3.1159601E+00, 3.41055E+01, 5.7390E+00, 9.8591E+02, &
      3.1315399E+00, 3.41144E+01, 5.6900E+00, 9.7268E+02, &
      3.1471976E+00, 3.41229E+01, 5.6414E+00, 9.5962E+02, &
      3.1629336E+00, 3.41310E+01, 5.5932E+00, 9.4674E+02, &
      3.1787482E+00, 3.41387E+01, 5.5454E+00, 9.3402E+02, &
      3.1946420E+00, 3.41460E+01, 5.4980E+00, 9.2147E+02, &
      3.2106152E+00, 3.41529E+01, 5.4510E+00, 9.0910E+02, &
      3.2266683E+00, 3.41594E+01, 5.4045E+00, 8.9691E+02, &
      3.2428016E+00, 3.41657E+01, 5.3584E+00, 8.8488E+02, &
      3.2590156E+00, 3.41715E+01, 5.3127E+00, 8.7302E+02, &
      3.2753107E+00, 3.41771E+01, 5.2674E+00, 8.6132E+02, &
      3.2916873E+00, 3.41824E+01, 5.2226E+00, 8.4979E+02, &
      3.3081457E+00, 3.42289E+01, 5.1781E+00, 8.3841E+02, &
      3.3246864E+00, 3.42338E+01, 5.1330E+00, 8.2701E+02, &
      3.3413099E+00, 3.42383E+01, 5.0882E+00, 8.1576E+02, &
      3.3580164E+00, 3.42424E+01, 5.0438E+00, 8.0468E+02, &
      3.3748065E+00, 3.42462E+01, 4.9999E+00, 7.9375E+02, &
      3.3916805E+00, 3.42496E+01, 4.9564E+00, 7.8297E+02, &
      3.4086389E+00, 3.42527E+01, 4.9133E+00, 7.7234E+02, &
      3.4256821E+00, 3.42555E+01, 4.8705E+00, 7.6186E+02, &
      3.4428105E+00, 3.42580E+01, 4.8282E+00, 7.5153E+02, &
      3.4600246E+00, 3.42602E+01, 4.7862E+00, 7.4134E+02, &
      3.4773247E+00, 3.42621E+01, 4.7447E+00, 7.3129E+02, &
      3.4947113E+00, 3.42637E+01, 4.7035E+00, 7.2138E+02, &
      3.5121849E+00, 3.42651E+01, 4.6627E+00, 7.1161E+02, &
      3.5297458E+00, 3.42662E+01, 4.6223E+00, 7.0198E+02, &
      3.5473945E+00, 3.42671E+01, 4.5823E+00, 6.9248E+02, &
      3.5651315E+00, 3.42678E+01, 4.5426E+00, 6.8311E+02, &
      3.5829572E+00, 3.42682E+01, 4.5033E+00, 6.7388E+02, &
      3.6008719E+00, 3.42684E+01, 4.4643E+00, 6.6477E+02, &
      3.6188763E+00, 3.42684E+01, 4.4257E+00, 6.5579E+02, &
      3.6369707E+00, 3.42682E+01, 4.3875E+00, 6.4693E+02, &
      3.6551555E+00, 3.42678E+01, 4.3496E+00, 6.3820E+02, &
      3.6734313E+00, 3.42671E+01, 4.3121E+00, 6.2959E+02, &
      3.6917985E+00, 3.42663E+01, 4.2749E+00, 6.2109E+02, &
      3.7102575E+00, 3.42654E+01, 4.2380E+00, 6.1272E+02, &
      3.7288088E+00, 3.42642E+01, 4.2015E+00, 6.0446E+02, &
      3.7474528E+00, 3.42629E+01, 4.1653E+00, 5.9632E+02, &
      3.7661901E+00, 3.42614E+01, 4.1295E+00, 5.8829E+02, &
      3.7850210E+00, 3.42597E+01, 4.0940E+00, 5.8037E+02, &
      3.8039461E+00, 3.42579E+01, 4.0588E+00, 5.7256E+02, &
      3.8229659E+00, 3.42559E+01, 4.0239E+00, 5.6486E+02, &
      3.8420807E+00, 3.42538E+01, 3.9894E+00, 5.5726E+02, &
      3.8612911E+00, 3.42516E+01, 3.9551E+00, 5.4977E+02, &
      3.8805975E+00, 3.42492E+01, 3.9212E+00, 5.4239E+02, &
      3.9000005E+00, 3.42466E+01, 3.8876E+00, 5.3510E+02, &
      3.9195005E+00, 3.42440E+01, 3.8543E+00, 5.2792E+02, &
      3.9390980E+00, 3.42412E+01, 3.8213E+00, 5.2084E+02, &
      3.9587935E+00, 3.42382E+01, 3.7886E+00, 5.1385E+02, &
      3.9785875E+00, 3.42352E+01, 3.7562E+00, 5.0696E+02, &
      3.9984804E+00, 3.42320E+01, 3.7241E+00, 5.0017E+02, &
      4.0184728E+00, 3.42288E+01, 3.6923E+00, 4.9347E+02, &
      4.0385652E+00, 3.42254E+01, 3.6607E+00, 4.8686E+02, &
      4.0587580E+00, 3.42219E+01, 3.6295E+00, 4.8035E+02, &
      4.0790518E+00, 3.42183E+01, 3.5986E+00, 4.7392E+02, &
      4.0994471E+00, 3.42146E+01, 3.5679E+00, 4.6758E+02, &
      4.1199443E+00, 3.42108E+01, 3.5375E+00, 4.6133E+02, &
      4.1405440E+00, 3.42069E+01, 3.5074E+00, 4.5517E+02, &
      4.1612467E+00, 3.42029E+01, 3.4775E+00, 4.4909E+02, &
      4.1820530E+00, 3.41989E+01, 3.4480E+00, 4.4309E+02, &
      4.2029632E+00, 3.41947E+01, 3.4187E+00, 4.3718E+02, &
      4.2239781E+00, 3.41905E+01, 3.3896E+00, 4.3135E+02, &
      4.2450980E+00, 3.41861E+01, 3.3609E+00, 4.2560E+02, &
      4.2663234E+00, 3.41817E+01, 3.3324E+00, 4.1993E+02, &
      4.2876551E+00, 3.41772E+01, 3.3041E+00, 4.1433E+02, &
      4.3090933E+00, 3.41727E+01, 3.2761E+00, 4.0882E+02, &
      4.3306388E+00, 3.41680E+01, 3.2484E+00, 4.0337E+02, &
      4.3522920E+00, 3.41633E+01, 3.2209E+00, 3.9800E+02, &
      4.3740535E+00, 3.41586E+01, 3.1935E+00, 3.9270E+02, &
      4.3959237E+00, 3.41537E+01, 3.1664E+00, 3.8746E+02, &
      4.4179033E+00, 3.41488E+01, 3.1395E+00, 3.8230E+02, &
      4.4399929E+00, 3.41438E+01, 3.1129E+00, 3.7721E+02, &
      4.4621928E+00, 3.41388E+01, 3.0865E+00, 3.7219E+02, &
      4.4845038E+00, 3.41337E+01, 3.0604E+00, 3.6724E+02, &
      4.5069263E+00, 3.41285E+01, 3.0345E+00, 3.6235E+02, &
      4.5294609E+00, 3.41233E+01, 3.0088E+00, 3.5754E+02, &
      4.5521082E+00, 3.41180E+01, 2.9834E+00, 3.5279E+02, &
      4.5748688E+00, 3.41127E+01, 2.9582E+00, 3.4810E+02, &
      4.5977431E+00, 3.41073E+01, 2.9332E+00, 3.4348E+02, &
      4.6207318E+00, 3.41018E+01, 2.9085E+00, 3.3892E+02, &
      4.6438355E+00, 3.40963E+01, 2.8839E+00, 3.3443E+02, &
      4.6670547E+00, 3.40908E+01, 2.8596E+00, 3.2999E+02, &
      4.6903900E+00, 3.40852E+01, 2.8355E+00, 3.2562E+02, &
      4.7138419E+00, 3.40795E+01, 2.8117E+00, 3.2130E+02, &
      4.7374111E+00, 3.40738E+01, 2.7880E+00, 3.1705E+02, &
      4.7610982E+00, 3.40681E+01, 2.7645E+00, 3.1285E+02, &
      4.7849037E+00, 3.40623E+01, 2.7413E+00, 3.0871E+02, &
      4.8088282E+00, 3.40565E+01, 2.7183E+00, 3.0463E+02, &
      4.8328723E+00, 3.40507E+01, 2.6955E+00, 3.0060E+02, &
      4.8570367E+00, 3.40448E+01, 2.6729E+00, 2.9663E+02, &
      4.8813219E+00, 3.40389E+01, 2.6504E+00, 2.9271E+02, &
      4.9057285E+00, 3.40329E+01, 2.6282E+00, 2.8885E+02, &
      4.9302571E+00, 3.40269E+01, 2.6062E+00, 2.8504E+02, &
      4.9549084E+00, 3.40209E+01, 2.5844E+00, 2.8128E+02, &
      4.9796829E+00, 3.40148E+01, 2.5628E+00, 2.7757E+02, &
      5.0045814E+00, 3.40087E+01, 2.5414E+00, 2.7391E+02, &
      5.0296043E+00, 3.40026E+01, 2.5201E+00, 2.7030E+02, &
      5.0547523E+00, 3.39964E+01, 2.4991E+00, 2.6674E+02, &
      5.0800260E+00, 3.39903E+01, 2.4782E+00, 2.6323E+02, &
      5.1054262E+00, 3.39841E+01, 2.4576E+00, 2.5977E+02, &
      5.1309533E+00, 3.39778E+01, 2.4371E+00, 2.5635E+02, &
      5.1566081E+00, 3.39716E+01, 2.4168E+00, 2.5298E+02, &
      5.1823911E+00, 3.39653E+01, 2.3967E+00, 2.4966E+02, &
      5.2083031E+00, 3.39590E+01, 2.3767E+00, 2.4638E+02, &
      5.2343446E+00, 3.39527E+01, 2.3569E+00, 2.4315E+02, &
      5.2605163E+00, 3.39463E+01, 2.3374E+00, 2.3996E+02, &
      5.2868189E+00, 3.39400E+01, 2.3179E+00, 2.3681E+02, &
      5.3132530E+00, 3.39336E+01, 2.2987E+00, 2.3371E+02, &
      5.3398192E+00, 3.39272E+01, 2.2796E+00, 2.3065E+02, &
      5.3665183E+00, 3.39207E+01, 2.2607E+00, 2.2763E+02, &
      5.3933509E+00, 3.39143E+01, 2.2420E+00, 2.2465E+02, &
      5.4203177E+00, 3.39078E+01, 2.2234E+00, 2.2171E+02, &
      5.4474193E+00, 3.39014E+01, 2.2050E+00, 2.1881E+02, &
      5.4746564E+00, 3.38949E+01, 2.1868E+00, 2.1595E+02, &
      5.5020297E+00, 3.38884E+01, 2.1687E+00, 2.1313E+02, &
      5.5295398E+00, 3.38819E+01, 2.1508E+00, 2.1034E+02, &
      5.5571875E+00, 3.38753E+01, 2.1331E+00, 2.0760E+02, &
      5.5849734E+00, 3.38688E+01, 2.1155E+00, 2.0489E+02, &
      5.6128983E+00, 3.38623E+01, 2.0980E+00, 2.0222E+02, &
      5.6409628E+00, 3.38557E+01, 2.0807E+00, 1.9958E+02, &
      5.6691676E+00, 3.38491E+01, 2.0636E+00, 1.9698E+02, &
      5.6975135E+00, 3.38425E+01, 2.0466E+00, 1.9441E+02, &
      5.7260010E+00, 3.38359E+01, 2.0298E+00, 1.9188E+02, &
      5.7546310E+00, 3.38293E+01, 2.0131E+00, 1.8939E+02, &
      5.7834042E+00, 3.38227E+01, 1.9966E+00, 1.8692E+02, &
      5.8123212E+00, 3.38161E+01, 1.9802E+00, 1.8449E+02, &
      5.8413828E+00, 3.38095E+01, 1.9639E+00, 1.8210E+02, &
      5.8705897E+00, 3.38029E+01, 1.9478E+00, 1.7973E+02, &
      5.8999427E+00, 3.37963E+01, 1.9319E+00, 1.7740E+02, &
      5.9294424E+00, 3.37896E+01, 1.9161E+00, 1.7510E+02, &
      5.9590896E+00, 3.37830E+01, 1.9004E+00, 1.7283E+02, &
      5.9888850E+00, 3.37764E+01, 1.8848E+00, 1.7059E+02, &
      6.0188295E+00, 3.37697E+01, 1.8694E+00, 1.6838E+02, &
      6.0489236E+00, 3.37631E+01, 1.8542E+00, 1.6620E+02, &
      6.0791682E+00, 3.37565E+01, 1.8390E+00, 1.6404E+02, &
      6.1095641E+00, 3.37498E+01, 1.8240E+00, 1.6192E+02, &
      6.1401119E+00, 3.37432E+01, 1.8092E+00, 1.5983E+02, &
      6.1708125E+00, 3.37366E+01, 1.7944E+00, 1.5776E+02, &
      6.2016665E+00, 3.37299E+01, 1.7798E+00, 1.5572E+02, &
      6.2326749E+00, 3.37233E+01, 1.7651E+00, 1.5369E+02, &
      6.2638382E+00, 3.37167E+01, 1.7502E+00, 1.5166E+02, &
      6.2951574E+00, 3.37100E+01, 1.7354E+00, 1.4965E+02, &
      6.3266332E+00, 3.37033E+01, 1.7207E+00, 1.4768E+02, &
      6.3582664E+00, 3.36966E+01, 1.7061E+00, 1.4572E+02, &
      6.3900577E+00, 3.36898E+01, 1.6917E+00, 1.4380E+02, &
      6.4220080E+00, 3.36830E+01, 1.6772E+00, 1.4188E+02, &
      6.4541180E+00, 3.36761E+01, 1.6628E+00, 1.3999E+02, &
      6.4863886E+00, 3.36692E+01, 1.6485E+00, 1.3812E+02, &
      6.5188206E+00, 3.36622E+01, 1.6343E+00, 1.3627E+02, &
      6.5514147E+00, 3.36552E+01, 1.6203E+00, 1.3445E+02, &
      6.5841717E+00, 3.36482E+01, 1.6064E+00, 1.3266E+02, &
      6.915365E+00, 3.35926E+01, 1.4764E+00, 1.1636E+02, &
      7.392525E+00, 3.34882E+01, 1.3147E+00, 9.7216E+01, &
      7.902609E+00, 3.33744E+01, 1.1689E+00, 8.1119E+01, &
      8.447890E+00, 3.32495E+01, 1.0402E+00, 6.7775E+01, &
      9.030794E+00, 3.31131E+01, 9.2336E-01, 5.6513E+01, &
      9.653919E+00, 3.29468E+01, 8.1040E-01, 4.6632E+01, &
      1.032004E+01, 3.27281E+01, 7.1195E-01, 3.8538E+01, &
      1.103212E+01, 3.24168E+01, 6.2607E-01, 3.1898E+01, &
      1.179334E+01, 3.18669E+01, 5.5107E-01, 2.6445E+01, &
      1.240464E+01, 3.07234E+01, 5.0063E-01, 2.2970E+01, &
      1.260708E+01, 2.91062E+01, 4.8552E-01, 2.1960E+01, &
      1.264514E+01, 2.76562E+01, 4.8276E-01, 2.1778E+01, &
      1.272109E+01, 2.93734E+01, 3.7868E+00, 1.6006E+02, &
      1.291096E+01, 3.09258E+01, 3.6719E+00, 1.5297E+02, &
      1.347697E+01, 3.22506E+01, 3.3722E+00, 1.3469E+02, &
      1.440688E+01, 3.30616E+01, 3.0123E+00, 1.1267E+02, &
      1.540095E+01, 3.35009E+01, 2.7116E+00, 9.4984E+01, &
      1.646362E+01, 3.37958E+01, 2.4478E+00, 8.0305E+01, &
      1.759961E+01, 3.40028E+01, 2.1904E+00, 6.7316E+01, &
      1.881398E+01, 3.41433E+01, 1.9566E+00, 5.6338E+01, &
      2.011215E+01, 3.42391E+01, 1.7451E+00, 4.7090E+01, &
      2.149988E+01, 3.43029E+01, 1.5552E+00, 3.9334E+01, &
      2.298338E+01, 3.43439E+01, 1.3856E+00, 3.2853E+01, &
      2.456923E+01, 3.43689E+01, 1.2347E+00, 2.7453E+01, &
      2.626450E+01, 3.44192E+01, 1.0988E+00, 2.2916E+01, &
      2.807676E+01, 3.44238E+01, 9.7157E-01, 1.9017E+01, &
      3.001405E+01, 3.44171E+01, 8.5771E-01, 1.5762E+01, &
      3.208502E+01, 3.44042E+01, 7.5718E-01, 1.3070E+01, &
      3.429889E+01, 3.43873E+01, 6.6844E-01, 1.0843E+01, &
      3.666551E+01, 3.43680E+01, 5.8990E-01, 8.9989E+00, &
      3.919543E+01, 3.43473E+01, 5.2047E-01, 7.4717E+00, &
      4.189992E+01, 3.43260E+01, 4.5925E-01, 6.2089E+00, &
      4.479101E+01, 3.43046E+01, 4.0525E-01, 5.1646E+00, &
      4.788159E+01, 3.42837E+01, 3.5762E-01, 4.3008E+00, &
      5.118542E+01, 3.42635E+01, 3.1561E-01, 3.5860E+00, &
      5.471721E+01, 3.42444E+01, 2.7856E-01, 2.9943E+00, &
      5.849270E+01, 3.42261E+01, 2.4442E-01, 2.4912E+00, &
      6.252870E+01, 3.42081E+01, 2.1442E-01, 2.0764E+00, &
      6.684318E+01, 3.41910E+01, 1.8812E-01, 1.7347E+00, &
      7.145536E+01, 3.41749E+01, 1.6505E-01, 1.4532E+00, &
      7.638578E+01, 3.41598E+01, 1.4481E-01, 1.2209E+00, &
      8.165640E+01, 3.41459E+01, 1.2692E-01, 1.0285E+00, &
      8.729069E+01, 3.41328E+01, 1.1097E-01, 8.6797E-01, &
      9.331374E+01, 3.41207E+01, 9.7022E-02, 7.3581E-01, &
      9.975239E+01, 3.41094E+01, 8.4831E-02, 6.2689E-01, &
      1.066353E+02, 3.40990E+01, 7.4175E-02, 5.3700E-01, &
      1.139931E+02, 3.40895E+01, 6.4860E-02, 4.6274E-01, &
      1.218587E+02, 3.40808E+01, 5.6717E-02, 4.0128E-01, &
      1.302669E+02, 3.40729E+01, 4.9598E-02, 3.5035E-01, &
      1.392553E+02, 3.40657E+01, 4.3374E-02, 3.0804E-01, &
      1.488640E+02, 3.40591E+01, 3.7932E-02, 2.7282E-01, &
      1.591356E+02, 3.40531E+01, 3.3174E-02, 2.4341E-01, &
      1.701159E+02, 3.40477E+01, 2.9014E-02, 2.1878E-01, &
      1.818539E+02, 3.40429E+01, 2.5376E-02, 1.9808E-01, &
      1.944018E+02, 3.40385E+01, 2.2195E-02, 1.8060E-01, &
      2.078156E+02, 3.40345E+01, 1.9413E-02, 1.6577E-01, &
      2.221548E+02, 3.40309E+01, 1.6981E-02, 1.5314E-01, &
      2.374835E+02, 3.40277E+01, 1.4853E-02, 1.4230E-01, &
      2.538699E+02, 3.40248E+01, 1.2993E-02, 1.3294E-01, &
      2.713869E+02, 3.40221E+01, 1.1366E-02, 1.2480E-01, &
      2.901126E+02, 3.40198E+01, 9.9429E-03, 1.1768E-01, &
      3.101304E+02, 3.40177E+01, 8.6982E-03, 1.1139E-01, &
      3.315294E+02, 3.40158E+01, 7.6096E-03, 1.0579E-01, &
      3.544049E+02, 3.40141E+01, 6.6574E-03, 1.0076E-01, &
      3.788588E+02, 3.40126E+01, 5.8245E-03, 9.6220E-02, &
      4.050001E+02, 3.40112E+01, 5.0959E-03, 9.2076E-02, &
      4.329451E+02, 3.40100E+01, 4.4586E-03, 8.8267E-02]
   case(35)
      frel = -1.3860E-01
      allocate(nist_tab(2256))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 7.92654E+00, 0.0000E+00, 1.5960E-04, &
      1.142761E-02, 7.91924E+00, 0.0000E+00, 2.1723E-04, &
      1.221612E-02, 7.91088E+00, 0.0000E+00, 2.9410E-04, &
      1.305903E-02, 7.90131E+00, 0.0000E+00, 3.9605E-04, &
      1.396010E-02, 7.89034E+00, 0.0000E+00, 5.3051E-04, &
      1.492335E-02, 7.87777E+00, 0.0000E+00, 7.0689E-04, &
      1.595306E-02, 7.86335E+00, 0.0000E+00, 9.3698E-04, &
      1.705382E-02, 7.84680E+00, 0.0000E+00, 1.2355E-03, &
      1.823053E-02, 7.82781E+00, 0.0000E+00, 1.6208E-03, &
      1.948844E-02, 7.80599E+00, 0.0000E+00, 2.1152E-03, &
      2.083314E-02, 7.78090E+00, 0.0000E+00, 2.7465E-03, &
      2.227063E-02, 7.75202E+00, 0.0000E+00, 3.5481E-03, &
      2.380730E-02, 7.71875E+00, 0.0000E+00, 4.5605E-03, &
      2.545001E-02, 7.68036E+00, 0.0000E+00, 5.8326E-03, &
      2.720606E-02, 7.63602E+00, 0.0000E+00, 7.4224E-03, &
      2.908327E-02, 7.58469E+00, 0.0000E+00, 9.3990E-03, &
      3.109002E-02, 7.52515E+00, 0.0000E+00, 1.1844E-02, &
      3.323523E-02, 7.45592E+00, 0.0000E+00, 1.4851E-02, &
      3.552846E-02, 7.37517E+00, 0.0000E+00, 1.8533E-02, &
      3.797993E-02, 7.28066E+00, 0.0000E+00, 2.3016E-02, &
      4.060054E-02, 7.16951E+00, 0.0000E+00, 2.8447E-02, &
      4.340198E-02, 7.03807E+00, 0.0000E+00, 3.4993E-02, &
      4.639671E-02, 6.88151E+00, 0.0000E+00, 4.2843E-02, &
      4.959809E-02, 6.69323E+00, 0.0000E+00, 5.2208E-02, &
      5.302035E-02, 6.46382E+00, 0.0000E+00, 6.3326E-02, &
      5.667876E-02, 6.17882E+00, 0.0000E+00, 7.6457E-02, &
      6.058959E-02, 5.81322E+00, 0.0000E+00, 9.1889E-02, &
      6.477028E-02, 5.31115E+00, 0.0000E+00, 1.0993E-01, &
      6.762000E-02, 4.81939E+00, 0.0000E+00, 1.2312E-01, &
      6.869800E-02, 4.49210E+00, 0.0000E+00, 1.2829E-01, &
      6.893100E-02, 4.31259E+00, 0.0000E+00, 1.2942E-01, &
      6.923942E-02, 4.34562E+00, 2.8116E-01, 2.1386E+03, &
      6.934500E-02, 4.35127E+00, 2.8479E-01, 2.1629E+03, &
      6.974950E-02, 4.30405E+00, 2.9910E-01, 2.2584E+03, &
      7.002990E-02, 4.18899E+00, 3.0939E-01, 2.3267E+03, &
      7.038000E-02, 4.22950E+00, 5.0909E-01, 3.8095E+03, &
      7.045050E-02, 4.23437E+00, 5.1341E-01, 3.8380E+03, &
      7.150200E-02, 4.18378E+00, 5.8193E-01, 4.2862E+03, &
      7.401695E-02, 3.89155E+00, 7.7949E-01, 5.5463E+03, &
      7.912411E-02, 3.19834E+00, 1.3532E+00, 9.0068E+03, &
      8.458368E-02, 2.50987E+00, 2.1831E+00, 1.3592E+04, &
      9.041995E-02, 1.91726E+00, 3.2703E+00, 1.9048E+04, &
      9.665893E-02, 1.50831E+00, 4.5908E+00, 2.5012E+04, &
      1.033284E-01, 1.35641E+00, 6.0881E+00, 3.1029E+04, &
      1.104581E-01, 1.50912E+00, 7.6828E+00, 3.6629E+04, &
      1.180797E-01, 1.98211E+00, 9.2846E+00, 4.1409E+04, &
      1.262272E-01, 2.75914E+00, 1.0806E+01, 4.5084E+04, &
      1.349368E-01, 3.79675E+00, 1.2171E+01, 4.7503E+04, &
      1.442475E-01, 5.03081E+00, 1.3325E+01, 4.8650E+04, &
      1.542005E-01, 6.38104E+00, 1.4233E+01, 4.8611E+04, &
      1.648404E-01, 7.74433E+00, 1.4882E+01, 4.7545E+04, &
      1.762144E-01, 8.90219E+00, 1.5273E+01, 4.5646E+04, &
      1.778700E-01, 8.99692E+00, 1.5308E+01, 4.5325E+04, &
      1.805925E-01, 8.93602E+00, 1.5355E+01, 4.4779E+04, &
      1.813185E-01, 8.59461E+00, 1.5366E+01, 4.4630E+04, &
      1.824075E-01, 9.11030E+00, 1.6196E+01, 4.6761E+04, &
      1.851300E-01, 9.71976E+00, 1.6253E+01, 4.6236E+04, &
      1.855140E-01, 9.77847E+00, 1.6260E+01, 4.6161E+04, &
      1.883732E-01, 1.00811E+01, 1.6308E+01, 4.5593E+04, &
      1.891107E-01, 1.00116E+01, 1.6318E+01, 4.5444E+04, &
      1.902465E-01, 1.03475E+01, 1.6685E+01, 4.6189E+04, &
      1.930860E-01, 1.08833E+01, 1.6730E+01, 4.5631E+04, &
      2.013709E-01, 1.20378E+01, 1.6812E+01, 4.3970E+04, &
      2.152655E-01, 1.36565E+01, 1.6809E+01, 4.1124E+04, &
      2.301188E-01, 1.51121E+01, 1.6649E+01, 3.8102E+04, &
      2.459970E-01, 1.63544E+01, 1.6352E+01, 3.5009E+04, &
      2.513700E-01, 1.66405E+01, 1.6231E+01, 3.4007E+04, &
      2.552175E-01, 1.66393E+01, 1.6139E+01, 3.3304E+04, &
      2.562435E-01, 1.63641E+01, 1.6114E+01, 3.3119E+04, &
      2.577825E-01, 1.68336E+01, 1.6768E+01, 3.4257E+04, &
      2.616300E-01, 1.74254E+01, 1.6676E+01, 3.3569E+04, &
      2.629708E-01, 1.75748E+01, 1.6643E+01, 3.3333E+04, &
      2.811158E-01, 1.91091E+01, 1.6171E+01, 3.0297E+04, &
      3.005128E-01, 2.03881E+01, 1.5613E+01, 2.7363E+04, &
      3.212482E-01, 2.15199E+01, 1.4981E+01, 2.4561E+04, &
      3.434143E-01, 2.25220E+01, 1.4297E+01, 2.1927E+04, &
      3.671099E-01, 2.34034E+01, 1.3578E+01, 1.9482E+04, &
      3.924405E-01, 2.41723E+01, 1.2841E+01, 1.7236E+04, &
      4.195189E-01, 2.48364E+01, 1.2099E+01, 1.5191E+04, &
      4.484657E-01, 2.54036E+01, 1.1362E+01, 1.3345E+04, &
      4.794098E-01, 2.58819E+01, 1.0638E+01, 1.1689E+04, &
      5.124891E-01, 2.62785E+01, 9.9340E+00, 1.0212E+04, &
      5.478508E-01, 2.66073E+01, 9.2552E+00, 8.9005E+03, &
      5.856525E-01, 2.68654E+01, 8.6047E+00, 7.7415E+03, &
      6.260625E-01, 2.70567E+01, 7.9851E+00, 6.7211E+03, &
      6.692609E-01, 2.71895E+01, 7.3977E+00, 5.8255E+03, &
      7.154399E-01, 2.72671E+01, 6.8432E+00, 5.0417E+03, &
      7.648052E-01, 2.72913E+01, 6.3209E+00, 4.3571E+03, &
      8.175768E-01, 2.72621E+01, 5.8297E+00, 3.7599E+03, &
      8.739896E-01, 2.71732E+01, 5.3703E+00, 3.2409E+03, &
      9.0000000E-01, 2.74506E+01, 5.1786E+00, 3.0352E+03, &
      9.0450000E-01, 2.74412E+01, 5.1467E+00, 3.0015E+03, &
      9.0902250E-01, 2.74315E+01, 5.1149E+00, 2.9682E+03, &
      9.1356761E-01, 2.74215E+01, 5.0832E+00, 2.9352E+03, &
      9.1813545E-01, 2.74111E+01, 5.0518E+00, 2.9026E+03, &
      9.2272613E-01, 2.74004E+01, 5.0206E+00, 2.8704E+03, &
      9.2733976E-01, 2.73893E+01, 4.9895E+00, 2.8385E+03, &
      9.3197646E-01, 2.73778E+01, 4.9586E+00, 2.8070E+03, &
      9.3663634E-01, 2.73661E+01, 4.9279E+00, 2.7758E+03, &
      9.4131952E-01, 2.73539E+01, 4.8974E+00, 2.7449E+03, &
      9.4602612E-01, 2.73414E+01, 4.8670E+00, 2.7144E+03, &
      9.5075625E-01, 2.73285E+01, 4.8368E+00, 2.6842E+03, &
      9.5551003E-01, 2.73153E+01, 4.8068E+00, 2.6543E+03, &
      9.6028758E-01, 2.73017E+01, 4.7770E+00, 2.6248E+03, &
      9.6508902E-01, 2.72877E+01, 4.7474E+00, 2.5956E+03, &
      9.6991446E-01, 2.72733E+01, 4.7180E+00, 2.5668E+03, &
      9.7476404E-01, 2.72586E+01, 4.6887E+00, 2.5382E+03, &
      9.7963786E-01, 2.72434E+01, 4.6596E+00, 2.5100E+03, &
      9.8453605E-01, 2.72279E+01, 4.6307E+00, 2.4820E+03, &
      9.8945873E-01, 2.72119E+01, 4.6019E+00, 2.4544E+03, &
      9.9440602E-01, 2.71956E+01, 4.5734E+00, 2.4271E+03, &
      9.9937805E-01, 2.71788E+01, 4.5450E+00, 2.4001E+03, &
      1.0043749E+00, 2.71723E+01, 4.5108E+00, 2.3703E+03, &
      1.0093968E+00, 2.71649E+01, 4.4761E+00, 2.3404E+03, &
      1.0144438E+00, 2.71553E+01, 4.4416E+00, 2.3109E+03, &
      1.0195160E+00, 2.71437E+01, 4.4075E+00, 2.2818E+03, &
      1.0246136E+00, 2.71305E+01, 4.3736E+00, 2.2531E+03, &
      1.0297367E+00, 2.71157E+01, 4.3396E+00, 2.2245E+03, &
      1.0348853E+00, 2.70996E+01, 4.3058E+00, 2.1963E+03, &
      1.0400598E+00, 2.70821E+01, 4.2722E+00, 2.1684E+03, &
      1.0452601E+00, 2.70634E+01, 4.2390E+00, 2.1409E+03, &
      1.0504864E+00, 2.70435E+01, 4.2060E+00, 2.1137E+03, &
      1.0557388E+00, 2.70225E+01, 4.1733E+00, 2.0869E+03, &
      1.0610175E+00, 2.70005E+01, 4.1409E+00, 2.0605E+03, &
      1.0663226E+00, 2.69775E+01, 4.1088E+00, 2.0344E+03, &
      1.0716542E+00, 2.69534E+01, 4.0769E+00, 2.0087E+03, &
      1.0770125E+00, 2.69284E+01, 4.0454E+00, 1.9833E+03, &
      1.0823975E+00, 2.69024E+01, 4.0141E+00, 1.9582E+03, &
      1.0878095E+00, 2.68754E+01, 3.9830E+00, 1.9335E+03, &
      1.0932486E+00, 2.68475E+01, 3.9523E+00, 1.9091E+03, &
      1.0987148E+00, 2.68187E+01, 3.9217E+00, 1.8850E+03, &
      1.1042084E+00, 2.67889E+01, 3.8915E+00, 1.8612E+03, &
      1.1097294E+00, 2.67582E+01, 3.8615E+00, 1.8377E+03, &
      1.1152781E+00, 2.67266E+01, 3.8318E+00, 1.8146E+03, &
      1.1208545E+00, 2.66940E+01, 3.8023E+00, 1.7917E+03, &
      1.1264587E+00, 2.66604E+01, 3.7731E+00, 1.7692E+03, &
      1.1320910E+00, 2.66258E+01, 3.7441E+00, 1.7469E+03, &
      1.1377515E+00, 2.65903E+01, 3.7153E+00, 1.7250E+03, &
      1.1434402E+00, 2.65538E+01, 3.6869E+00, 1.7033E+03, &
      1.1491574E+00, 2.65162E+01, 3.6586E+00, 1.6819E+03, &
      1.1549032E+00, 2.64775E+01, 3.6306E+00, 1.6608E+03, &
      1.1606777E+00, 2.64378E+01, 3.6028E+00, 1.6400E+03, &
      1.1664811E+00, 2.63970E+01, 3.5753E+00, 1.6194E+03, &
      1.1723135E+00, 2.63551E+01, 3.5480E+00, 1.5991E+03, &
      1.1781751E+00, 2.63120E+01, 3.5209E+00, 1.5791E+03, &
      1.1840660E+00, 2.62677E+01, 3.4941E+00, 1.5593E+03, &
      1.1899863E+00, 2.62222E+01, 3.4675E+00, 1.5398E+03, &
      1.1959362E+00, 2.61754E+01, 3.4411E+00, 1.5206E+03, &
      1.2019159E+00, 2.61273E+01, 3.4149E+00, 1.5016E+03, &
      1.2079255E+00, 2.60778E+01, 3.3890E+00, 1.4828E+03, &
      1.2139651E+00, 2.60269E+01, 3.3633E+00, 1.4643E+03, &
      1.2200350E+00, 2.59746E+01, 3.3377E+00, 1.4461E+03, &
      1.2261351E+00, 2.59207E+01, 3.3124E+00, 1.4280E+03, &
      1.2322658E+00, 2.58653E+01, 3.2874E+00, 1.4102E+03, &
      1.2384271E+00, 2.58082E+01, 3.2625E+00, 1.3927E+03, &
      1.2446193E+00, 2.57494E+01, 3.2378E+00, 1.3753E+03, &
      1.2508424E+00, 2.56889E+01, 3.2134E+00, 1.3582E+03, &
      1.2570966E+00, 2.56264E+01, 3.1891E+00, 1.3414E+03, &
      1.2633821E+00, 2.55620E+01, 3.1651E+00, 1.3247E+03, &
      1.2696990E+00, 2.54956E+01, 3.1412E+00, 1.3082E+03, &
      1.2760475E+00, 2.54270E+01, 3.1176E+00, 1.2920E+03, &
      1.2824277E+00, 2.53562E+01, 3.0941E+00, 1.2760E+03, &
      1.2888399E+00, 2.52830E+01, 3.0709E+00, 1.2601E+03, &
      1.2952840E+00, 2.52072E+01, 3.0478E+00, 1.2445E+03, &
      1.3017605E+00, 2.51289E+01, 3.0240E+00, 1.2287E+03, &
      1.3082693E+00, 2.50477E+01, 3.0004E+00, 1.2131E+03, &
      1.3148106E+00, 2.49634E+01, 2.9769E+00, 1.1977E+03, &
      1.3213847E+00, 2.48761E+01, 2.9532E+00, 1.1823E+03, &
      1.3279916E+00, 2.47853E+01, 2.9295E+00, 1.1671E+03, &
      1.3346316E+00, 2.46908E+01, 2.9061E+00, 1.1521E+03, &
      1.3413047E+00, 2.45925E+01, 2.8828E+00, 1.1372E+03, &
      1.3480112E+00, 2.44901E+01, 2.8598E+00, 1.1226E+03, &
      1.3547513E+00, 2.43832E+01, 2.8370E+00, 1.1082E+03, &
      1.3615250E+00, 2.42716E+01, 2.8144E+00, 1.0940E+03, &
      1.3683327E+00, 2.41549E+01, 2.7920E+00, 1.0800E+03, &
      1.3751743E+00, 2.40326E+01, 2.7699E+00, 1.0661E+03, &
      1.3820502E+00, 2.39044E+01, 2.7479E+00, 1.0525E+03, &
      1.3889605E+00, 2.37696E+01, 2.7261E+00, 1.0390E+03, &
      1.3959053E+00, 2.36277E+01, 2.7046E+00, 1.0257E+03, &
      1.4028848E+00, 2.34781E+01, 2.6832E+00, 1.0126E+03, &
      1.4098992E+00, 2.33198E+01, 2.6620E+00, 9.9972E+02, &
      1.4169487E+00, 2.31521E+01, 2.6411E+00, 9.8698E+02, &
      1.4240335E+00, 2.29739E+01, 2.6203E+00, 9.7441E+02, &
      1.4311536E+00, 2.27839E+01, 2.5997E+00, 9.6202E+02, &
      1.4383094E+00, 2.25806E+01, 2.5793E+00, 9.4979E+02, &
      1.4455009E+00, 2.23623E+01, 2.5591E+00, 9.3772E+02, &
      1.4527284E+00, 2.21268E+01, 2.5390E+00, 9.2582E+02, &
      1.4599921E+00, 2.18715E+01, 2.5192E+00, 9.1408E+02, &
      1.4672920E+00, 2.15929E+01, 2.4995E+00, 9.0250E+02, &
      1.4746285E+00, 2.12869E+01, 2.4800E+00, 8.9108E+02, &
      1.4820016E+00, 2.09478E+01, 2.4607E+00, 8.7981E+02, &
      1.4894117E+00, 2.05681E+01, 2.4415E+00, 8.6869E+02, &
      1.4968587E+00, 2.01376E+01, 2.4226E+00, 8.5772E+02, &
      1.5043430E+00, 1.96410E+01, 2.4038E+00, 8.4690E+02, &
      1.5118647E+00, 1.90553E+01, 2.3852E+00, 8.3623E+02, &
      1.5194240E+00, 1.83419E+01, 2.3667E+00, 8.2570E+02, &
      1.5270212E+00, 1.74295E+01, 2.3484E+00, 8.1531E+02, &
      1.5346563E+00, 1.61588E+01, 2.3303E+00, 8.0506E+02, &
      1.5423295E+00, 1.40202E+01, 2.3123E+00, 7.9495E+02, &
      1.5497311E+00, 2.83862E+00, 2.2952E+00, 7.8537E+02, &
      1.5500412E+00, 2.15632E+00, 1.1454E+01, 3.8970E+03, &
      1.5500690E+00, 2.68037E+00, 1.1454E+01, 3.8968E+03, &
      1.5577914E+00, 1.37486E+01, 1.1363E+01, 3.8467E+03, &
      1.5655804E+00, 1.55172E+01, 1.1272E+01, 3.7971E+03, &
      1.5734083E+00, 1.63532E+01, 1.1182E+01, 3.7481E+03, &
      1.5812753E+00, 1.66565E+01, 1.1093E+01, 3.6998E+03, &
      1.5891817E+00, 1.62799E+01, 1.1004E+01, 3.6521E+03, &
      1.5958196E+00, 1.15761E+01, 1.0931E+01, 3.6127E+03, &
      1.5961803E+00, 1.15274E+01, 1.5450E+01, 5.1028E+03, &
      1.5971276E+00, 1.42378E+01, 1.5435E+01, 5.0950E+03, &
      1.6051132E+00, 1.77760E+01, 1.5313E+01, 5.0294E+03, &
      1.6131388E+00, 1.91375E+01, 1.5191E+01, 4.9648E+03, &
      1.6212045E+00, 2.00886E+01, 1.5071E+01, 4.9009E+03, &
      1.6293105E+00, 2.08413E+01, 1.4951E+01, 4.8379E+03, &
      1.6374571E+00, 2.14709E+01, 1.4832E+01, 4.7757E+03, &
      1.6456443E+00, 2.20139E+01, 1.4715E+01, 4.7143E+03, &
      1.6538726E+00, 2.24914E+01, 1.4598E+01, 4.6538E+03, &
      1.6621419E+00, 2.29166E+01, 1.4482E+01, 4.5940E+03, &
      1.6704526E+00, 2.32986E+01, 1.4367E+01, 4.5349E+03, &
      1.6788049E+00, 2.36436E+01, 1.4254E+01, 4.4767E+03, &
      1.6871989E+00, 2.39561E+01, 1.4141E+01, 4.4192E+03, &
      1.6956349E+00, 2.42392E+01, 1.4029E+01, 4.3624E+03, &
      1.7041131E+00, 2.44950E+01, 1.3917E+01, 4.3064E+03, &
      1.7126337E+00, 2.47245E+01, 1.3807E+01, 4.2511E+03, &
      1.7211968E+00, 2.49278E+01, 1.3698E+01, 4.1965E+03, &
      1.7298028E+00, 2.51036E+01, 1.3589E+01, 4.1426E+03, &
      1.7384518E+00, 2.52488E+01, 1.3482E+01, 4.0895E+03, &
      1.7471441E+00, 2.53568E+01, 1.3375E+01, 4.0370E+03, &
      1.7558798E+00, 2.54137E+01, 1.3269E+01, 3.9852E+03, &
      1.7646592E+00, 2.53870E+01, 1.3164E+01, 3.9340E+03, &
      1.7734825E+00, 2.51675E+01, 1.3060E+01, 3.8836E+03, &
      1.7804852E+00, 2.42756E+01, 1.2979E+01, 3.8442E+03, &
      1.7823499E+00, 2.33907E+01, 1.4883E+01, 4.4029E+03, &
      1.7835146E+00, 2.43169E+01, 1.4869E+01, 4.3957E+03, &
      1.7912617E+00, 2.56026E+01, 1.4771E+01, 4.3481E+03, &
      1.8002180E+00, 2.62089E+01, 1.4660E+01, 4.2940E+03, &
      1.8092191E+00, 2.66377E+01, 1.4550E+01, 4.2406E+03, &
      1.8182652E+00, 2.69884E+01, 1.4441E+01, 4.1879E+03, &
      1.8273565E+00, 2.72932E+01, 1.4332E+01, 4.1358E+03, &
      1.8364933E+00, 2.75667E+01, 1.4225E+01, 4.0844E+03, &
      1.8456757E+00, 2.78169E+01, 1.4118E+01, 4.0336E+03, &
      1.8549041E+00, 2.80486E+01, 1.4012E+01, 3.9835E+03, &
      1.8641786E+00, 2.82649E+01, 1.3907E+01, 3.9340E+03, &
      1.8734995E+00, 2.84681E+01, 1.3803E+01, 3.8853E+03, &
      1.8828670E+00, 2.86607E+01, 1.3702E+01, 3.8377E+03, &
      1.8922814E+00, 2.88442E+01, 1.3601E+01, 3.7907E+03, &
      1.9017428E+00, 2.90195E+01, 1.3502E+01, 3.7443E+03, &
      1.9112515E+00, 2.91874E+01, 1.3404E+01, 3.6986E+03, &
      1.9208077E+00, 2.93485E+01, 1.3306E+01, 3.6534E+03, &
      1.9304118E+00, 2.95034E+01, 1.3209E+01, 3.6089E+03, &
      1.9400638E+00, 2.96527E+01, 1.3113E+01, 3.5649E+03, &
      1.9497642E+00, 2.97967E+01, 1.3018E+01, 3.5215E+03, &
      1.9595130E+00, 2.99357E+01, 1.2924E+01, 3.4787E+03, &
      1.9693105E+00, 3.00703E+01, 1.2830E+01, 3.4364E+03, &
      1.9791571E+00, 3.02006E+01, 1.2738E+01, 3.3947E+03, &
      1.9890529E+00, 3.03270E+01, 1.2646E+01, 3.3535E+03, &
      1.9989981E+00, 3.04497E+01, 1.2555E+01, 3.3129E+03, &
      2.0089931E+00, 3.05691E+01, 1.2465E+01, 3.2728E+03, &
      2.0190381E+00, 3.06852E+01, 1.2374E+01, 3.2329E+03, &
      2.0291333E+00, 3.07978E+01, 1.2284E+01, 3.1934E+03, &
      2.0392790E+00, 3.09070E+01, 1.2194E+01, 3.1543E+03, &
      2.0494754E+00, 3.10132E+01, 1.2105E+01, 3.1157E+03, &
      2.0597227E+00, 3.11164E+01, 1.2016E+01, 3.0776E+03, &
      2.0700213E+00, 3.12169E+01, 1.1929E+01, 3.0400E+03, &
      2.0803714E+00, 3.13147E+01, 1.1841E+01, 3.0027E+03, &
      2.0907733E+00, 3.14098E+01, 1.1754E+01, 2.9658E+03, &
      2.1012272E+00, 3.15023E+01, 1.1667E+01, 2.9293E+03, &
      2.1117333E+00, 3.15922E+01, 1.1581E+01, 2.8933E+03, &
      2.1222920E+00, 3.16798E+01, 1.1496E+01, 2.8577E+03, &
      2.1329034E+00, 3.17650E+01, 1.1411E+01, 2.8226E+03, &
      2.1435680E+00, 3.18481E+01, 1.1327E+01, 2.7880E+03, &
      2.1542858E+00, 3.19291E+01, 1.1243E+01, 2.7537E+03, &
      2.1650572E+00, 3.20081E+01, 1.1160E+01, 2.7199E+03, &
      2.1758825E+00, 3.20851E+01, 1.1078E+01, 2.6865E+03, &
      2.1867619E+00, 3.21603E+01, 1.0997E+01, 2.6535E+03, &
      2.1976957E+00, 3.22336E+01, 1.0916E+01, 2.6209E+03, &
      2.2086842E+00, 3.23052E+01, 1.0835E+01, 2.5887E+03, &
      2.2197276E+00, 3.23752E+01, 1.0756E+01, 2.5569E+03, &
      2.2308263E+00, 3.24435E+01, 1.0676E+01, 2.5255E+03, &
      2.2419804E+00, 3.25103E+01, 1.0598E+01, 2.4945E+03, &
      2.2531903E+00, 3.25755E+01, 1.0520E+01, 2.4639E+03, &
      2.2644562E+00, 3.26394E+01, 1.0442E+01, 2.4337E+03, &
      2.2757785E+00, 3.27018E+01, 1.0366E+01, 2.4038E+03, &
      2.2871574E+00, 3.27629E+01, 1.0289E+01, 2.3743E+03, &
      2.2985932E+00, 3.28227E+01, 1.0214E+01, 2.3451E+03, &
      2.3100862E+00, 3.28814E+01, 1.0138E+01, 2.3163E+03, &
      2.3216366E+00, 3.29389E+01, 1.0063E+01, 2.2878E+03, &
      2.3332448E+00, 3.29949E+01, 9.9879E+00, 2.2594E+03, &
      2.3449110E+00, 3.30495E+01, 9.9131E+00, 2.2314E+03, &
      2.3566356E+00, 3.31028E+01, 9.8389E+00, 2.2038E+03, &
      2.3684187E+00, 3.31548E+01, 9.7652E+00, 2.1764E+03, &
      2.3802608E+00, 3.32056E+01, 9.6921E+00, 2.1494E+03, &
      2.3921621E+00, 3.32552E+01, 9.6195E+00, 2.1228E+03, &
      2.4041230E+00, 3.33037E+01, 9.5474E+00, 2.0964E+03, &
      2.4161436E+00, 3.33510E+01, 9.4758E+00, 2.0704E+03, &
      2.4282243E+00, 3.33972E+01, 9.4048E+00, 2.0447E+03, &
      2.4403654E+00, 3.34424E+01, 9.3342E+00, 2.0194E+03, &
      2.4525672E+00, 3.34866E+01, 9.2642E+00, 1.9943E+03, &
      2.4648301E+00, 3.35297E+01, 9.1947E+00, 1.9695E+03, &
      2.4771542E+00, 3.35719E+01, 9.1257E+00, 1.9451E+03, &
      2.4895400E+00, 3.36131E+01, 9.0572E+00, 1.9209E+03, &
      2.5019877E+00, 3.36534E+01, 8.9892E+00, 1.8971E+03, &
      2.5144976E+00, 3.36928E+01, 8.9217E+00, 1.8735E+03, &
      2.5270701E+00, 3.37314E+01, 8.8547E+00, 1.8503E+03, &
      2.5397055E+00, 3.37690E+01, 8.7882E+00, 1.8273E+03, &
      2.5524040E+00, 3.38059E+01, 8.7222E+00, 1.8046E+03, &
      2.5651660E+00, 3.38419E+01, 8.6567E+00, 1.7822E+03, &
      2.5779919E+00, 3.38771E+01, 8.5917E+00, 1.7600E+03, &
      2.5908818E+00, 3.39116E+01, 8.5271E+00, 1.7382E+03, &
      2.6038362E+00, 3.39453E+01, 8.4631E+00, 1.7166E+03, &
      2.6168554E+00, 3.39783E+01, 8.3995E+00, 1.6953E+03, &
      2.6299397E+00, 3.40105E+01, 8.3364E+00, 1.6742E+03, &
      2.6430894E+00, 3.40421E+01, 8.2737E+00, 1.6534E+03, &
      2.6563048E+00, 3.40730E+01, 8.2116E+00, 1.6329E+03, &
      2.6695863E+00, 3.41032E+01, 8.1499E+00, 1.6126E+03, &
      2.6829343E+00, 3.41328E+01, 8.0887E+00, 1.5926E+03, &
      2.6963489E+00, 3.41617E+01, 8.0279E+00, 1.5728E+03, &
      2.7098307E+00, 3.41900E+01, 7.9676E+00, 1.5533E+03, &
      2.7233798E+00, 3.42177E+01, 7.9078E+00, 1.5340E+03, &
      2.7369967E+00, 3.42449E+01, 7.8484E+00, 1.5150E+03, &
      2.7506817E+00, 3.42715E+01, 7.7895E+00, 1.4962E+03, &
      2.7644351E+00, 3.42975E+01, 7.7310E+00, 1.4776E+03, &
      2.7782573E+00, 3.43230E+01, 7.6730E+00, 1.4593E+03, &
      2.7921486E+00, 3.43480E+01, 7.6155E+00, 1.4412E+03, &
      2.8061093E+00, 3.43725E+01, 7.5583E+00, 1.4233E+03, &
      2.8201399E+00, 3.43964E+01, 7.5017E+00, 1.4056E+03, &
      2.8342406E+00, 3.44200E+01, 7.4454E+00, 1.3882E+03, &
      2.8484118E+00, 3.44431E+01, 7.3896E+00, 1.3710E+03, &
      2.8626539E+00, 3.44657E+01, 7.3342E+00, 1.3540E+03, &
      2.8769671E+00, 3.44879E+01, 7.2793E+00, 1.3372E+03, &
      2.8913520E+00, 3.45098E+01, 7.2248E+00, 1.3206E+03, &
      2.9058087E+00, 3.45312E+01, 7.1707E+00, 1.3043E+03, &
      2.9203378E+00, 3.45523E+01, 7.1171E+00, 1.2881E+03, &
      2.9349394E+00, 3.45731E+01, 7.0638E+00, 1.2722E+03, &
      2.9496141E+00, 3.45937E+01, 7.0110E+00, 1.2564E+03, &
      2.9643622E+00, 3.46139E+01, 6.9586E+00, 1.2409E+03, &
      2.9791840E+00, 3.46341E+01, 6.9066E+00, 1.2256E+03, &
      2.9940799E+00, 3.46541E+01, 6.8551E+00, 1.2104E+03, &
      3.0090503E+00, 3.46779E+01, 6.8016E+00, 1.1950E+03, &
      3.0240956E+00, 3.47003E+01, 6.7472E+00, 1.1796E+03, &
      3.0392161E+00, 3.47205E+01, 6.6931E+00, 1.1644E+03, &
      3.0544122E+00, 3.47395E+01, 6.6395E+00, 1.1494E+03, &
      3.0696842E+00, 3.47576E+01, 6.5864E+00, 1.1346E+03, &
      3.0850326E+00, 3.47751E+01, 6.5337E+00, 1.1199E+03, &
      3.1004578E+00, 3.48925E+01, 6.4813E+00, 1.1055E+03, &
      3.1159601E+00, 3.49089E+01, 6.4271E+00, 1.0908E+03, &
      3.1315399E+00, 3.49247E+01, 6.3735E+00, 1.0764E+03, &
      3.1471976E+00, 3.49397E+01, 6.3203E+00, 1.0621E+03, &
      3.1629336E+00, 3.49542E+01, 6.2676E+00, 1.0481E+03, &
      3.1787482E+00, 3.49681E+01, 6.2154E+00, 1.0342E+03, &
      3.1946420E+00, 3.50325E+01, 6.1634E+00, 1.0205E+03, &
      3.2106152E+00, 3.50453E+01, 6.1109E+00, 1.0069E+03, &
      3.2266683E+00, 3.50576E+01, 6.0589E+00, 9.9339E+02, &
      3.2428016E+00, 3.50692E+01, 6.0074E+00, 9.8009E+02, &
      3.2590156E+00, 3.50803E+01, 5.9564E+00, 9.6698E+02, &
      3.2753107E+00, 3.50908E+01, 5.9058E+00, 9.5404E+02, &
      3.2916873E+00, 3.51008E+01, 5.8557E+00, 9.4129E+02, &
      3.3081457E+00, 3.51104E+01, 5.8060E+00, 9.2872E+02, &
      3.3246864E+00, 3.51195E+01, 5.7568E+00, 9.1631E+02, &
      3.3413099E+00, 3.51282E+01, 5.7081E+00, 9.0409E+02, &
      3.3580164E+00, 3.51365E+01, 5.6598E+00, 8.9202E+02, &
      3.3748065E+00, 3.51443E+01, 5.6120E+00, 8.8013E+02, &
      3.3916805E+00, 3.51518E+01, 5.5645E+00, 8.6840E+02, &
      3.4086389E+00, 3.51590E+01, 5.5176E+00, 8.5683E+02, &
      3.4256821E+00, 3.51658E+01, 5.4707E+00, 8.4537E+02, &
      3.4428105E+00, 3.51722E+01, 5.4242E+00, 8.3407E+02, &
      3.4600246E+00, 3.51783E+01, 5.3782E+00, 8.2293E+02, &
      3.4773247E+00, 3.51841E+01, 5.3326E+00, 8.1194E+02, &
      3.4947113E+00, 3.51896E+01, 5.2874E+00, 8.0110E+02, &
      3.5121849E+00, 3.51948E+01, 5.2426E+00, 7.9041E+02, &
      3.5297458E+00, 3.51998E+01, 5.1983E+00, 7.7987E+02, &
      3.5473945E+00, 3.52045E+01, 5.1543E+00, 7.6948E+02, &
      3.5651315E+00, 3.52495E+01, 5.1106E+00, 7.5920E+02, &
      3.5829572E+00, 3.52539E+01, 5.0661E+00, 7.4888E+02, &
      3.6008719E+00, 3.52579E+01, 5.0219E+00, 7.3871E+02, &
      3.6188763E+00, 3.52616E+01, 4.9782E+00, 7.2868E+02, &
      3.6369707E+00, 3.52649E+01, 4.9348E+00, 7.1879E+02, &
      3.6551555E+00, 3.52678E+01, 4.8919E+00, 7.0903E+02, &
      3.6734313E+00, 3.52705E+01, 4.8493E+00, 6.9941E+02, &
      3.6917985E+00, 3.52728E+01, 4.8072E+00, 6.8993E+02, &
      3.7102575E+00, 3.52749E+01, 4.7654E+00, 6.8058E+02, &
      3.7288088E+00, 3.52767E+01, 4.7240E+00, 6.7136E+02, &
      3.7474528E+00, 3.52782E+01, 4.6830E+00, 6.6227E+02, &
      3.7661901E+00, 3.52794E+01, 4.6424E+00, 6.5330E+02, &
      3.7850210E+00, 3.52804E+01, 4.6022E+00, 6.4446E+02, &
      3.8039461E+00, 3.52812E+01, 4.5623E+00, 6.3574E+02, &
      3.8229659E+00, 3.52817E+01, 4.5228E+00, 6.2714E+02, &
      3.8420807E+00, 3.52820E+01, 4.4836E+00, 6.1867E+02, &
      3.8612911E+00, 3.52821E+01, 4.4448E+00, 6.1031E+02, &
      3.8805975E+00, 3.52819E+01, 4.4064E+00, 6.0207E+02, &
      3.9000005E+00, 3.52816E+01, 4.3683E+00, 5.9394E+02, &
      3.9195005E+00, 3.52810E+01, 4.3306E+00, 5.8592E+02, &
      3.9390980E+00, 3.52803E+01, 4.2932E+00, 5.7802E+02, &
      3.9587935E+00, 3.52794E+01, 4.2562E+00, 5.7022E+02, &
      3.9785875E+00, 3.52783E+01, 4.2195E+00, 5.6254E+02, &
      3.9984804E+00, 3.52770E+01, 4.1831E+00, 5.5496E+02, &
      4.0184728E+00, 3.52755E+01, 4.1471E+00, 5.4748E+02, &
      4.0385652E+00, 3.52739E+01, 4.1114E+00, 5.4011E+02, &
      4.0587580E+00, 3.52722E+01, 4.0760E+00, 5.3284E+02, &
      4.0790518E+00, 3.52702E+01, 4.0410E+00, 5.2568E+02, &
      4.0994471E+00, 3.52681E+01, 4.0063E+00, 5.1861E+02, &
      4.1199443E+00, 3.52659E+01, 3.9719E+00, 5.1164E+02, &
      4.1405440E+00, 3.52635E+01, 3.9378E+00, 5.0476E+02, &
      4.1612467E+00, 3.52610E+01, 3.9040E+00, 4.9798E+02, &
      4.1820530E+00, 3.52584E+01, 3.8705E+00, 4.9130E+02, &
      4.2029632E+00, 3.52556E+01, 3.8374E+00, 4.8470E+02, &
      4.2239781E+00, 3.52527E+01, 3.8045E+00, 4.7820E+02, &
      4.2450980E+00, 3.52497E+01, 3.7719E+00, 4.7179E+02, &
      4.2663234E+00, 3.52466E+01, 3.7397E+00, 4.6547E+02, &
      4.2876551E+00, 3.52433E+01, 3.7077E+00, 4.5923E+02, &
      4.3090933E+00, 3.52399E+01, 3.6760E+00, 4.5308E+02, &
      4.3306388E+00, 3.52364E+01, 3.6446E+00, 4.4702E+02, &
      4.3522920E+00, 3.52329E+01, 3.6135E+00, 4.4104E+02, &
      4.3740535E+00, 3.52292E+01, 3.5827E+00, 4.3514E+02, &
      4.3959237E+00, 3.52254E+01, 3.5521E+00, 4.2932E+02, &
      4.4179033E+00, 3.52215E+01, 3.5219E+00, 4.2358E+02, &
      4.4399929E+00, 3.52175E+01, 3.4919E+00, 4.1793E+02, &
      4.4621928E+00, 3.52134E+01, 3.4622E+00, 4.1235E+02, &
      4.4845038E+00, 3.52093E+01, 3.4327E+00, 4.0684E+02, &
      4.5069263E+00, 3.52050E+01, 3.4035E+00, 4.0141E+02, &
      4.5294609E+00, 3.52007E+01, 3.3746E+00, 3.9606E+02, &
      4.5521082E+00, 3.51963E+01, 3.3460E+00, 3.9078E+02, &
      4.5748688E+00, 3.51918E+01, 3.3176E+00, 3.8558E+02, &
      4.5977431E+00, 3.51872E+01, 3.2894E+00, 3.8044E+02, &
      4.6207318E+00, 3.51826E+01, 3.2616E+00, 3.7538E+02, &
      4.6438355E+00, 3.51779E+01, 3.2339E+00, 3.7038E+02, &
      4.6670547E+00, 3.51731E+01, 3.2066E+00, 3.6545E+02, &
      4.6903900E+00, 3.51682E+01, 3.1794E+00, 3.6060E+02, &
      4.7138419E+00, 3.51633E+01, 3.1525E+00, 3.5580E+02, &
      4.7374111E+00, 3.51583E+01, 3.1259E+00, 3.5108E+02, &
      4.7610982E+00, 3.51533E+01, 3.0995E+00, 3.4642E+02, &
      4.7849037E+00, 3.51482E+01, 3.0733E+00, 3.4182E+02, &
      4.8088282E+00, 3.51430E+01, 3.0474E+00, 3.3728E+02, &
      4.8328723E+00, 3.51378E+01, 3.0216E+00, 3.3280E+02, &
      4.8570367E+00, 3.51325E+01, 2.9961E+00, 3.2838E+02, &
      4.8813219E+00, 3.51272E+01, 2.9707E+00, 3.2402E+02, &
      4.9057285E+00, 3.51218E+01, 2.9456E+00, 3.1972E+02, &
      4.9302571E+00, 3.51164E+01, 2.9207E+00, 3.1547E+02, &
      4.9549084E+00, 3.51109E+01, 2.8961E+00, 3.1129E+02, &
      4.9796829E+00, 3.51053E+01, 2.8716E+00, 3.0716E+02, &
      5.0045814E+00, 3.50997E+01, 2.8474E+00, 3.0309E+02, &
      5.0296043E+00, 3.50941E+01, 2.8234E+00, 2.9907E+02, &
      5.0547523E+00, 3.50884E+01, 2.7996E+00, 2.9511E+02, &
      5.0800260E+00, 3.50826E+01, 2.7761E+00, 2.9120E+02, &
      5.1054262E+00, 3.50769E+01, 2.7527E+00, 2.8735E+02, &
      5.1309533E+00, 3.50711E+01, 2.7296E+00, 2.8355E+02, &
      5.1566081E+00, 3.50652E+01, 2.7066E+00, 2.7980E+02, &
      5.1823911E+00, 3.50593E+01, 2.6839E+00, 2.7610E+02, &
      5.2083031E+00, 3.50534E+01, 2.6614E+00, 2.7245E+02, &
      5.2343446E+00, 3.50474E+01, 2.6390E+00, 2.6886E+02, &
      5.2605163E+00, 3.50414E+01, 2.6169E+00, 2.6531E+02, &
      5.2868189E+00, 3.50353E+01, 2.5950E+00, 2.6181E+02, &
      5.3132530E+00, 3.50293E+01, 2.5732E+00, 2.5835E+02, &
      5.3398192E+00, 3.50232E+01, 2.5517E+00, 2.5495E+02, &
      5.3665183E+00, 3.50170E+01, 2.5304E+00, 2.5159E+02, &
      5.3933509E+00, 3.50109E+01, 2.5092E+00, 2.4828E+02, &
      5.4203177E+00, 3.50047E+01, 2.4882E+00, 2.4501E+02, &
      5.4474193E+00, 3.49984E+01, 2.4675E+00, 2.4178E+02, &
      5.4746564E+00, 3.49922E+01, 2.4469E+00, 2.3860E+02, &
      5.5020297E+00, 3.49859E+01, 2.4265E+00, 2.3547E+02, &
      5.5295398E+00, 3.49796E+01, 2.4062E+00, 2.3237E+02, &
      5.5571875E+00, 3.49733E+01, 2.3862E+00, 2.2932E+02, &
      5.5849734E+00, 3.49669E+01, 2.3663E+00, 2.2631E+02, &
      5.6128983E+00, 3.49606E+01, 2.3466E+00, 2.2334E+02, &
      5.6409628E+00, 3.49542E+01, 2.3271E+00, 2.2041E+02, &
      5.6691676E+00, 3.49478E+01, 2.3078E+00, 2.1752E+02, &
      5.6975135E+00, 3.49413E+01, 2.2886E+00, 2.1467E+02, &
      5.7260010E+00, 3.49349E+01, 2.2696E+00, 2.1186E+02, &
      5.7546310E+00, 3.49284E+01, 2.2508E+00, 2.0908E+02, &
      5.7834042E+00, 3.49219E+01, 2.2321E+00, 2.0635E+02, &
      5.8123212E+00, 3.49154E+01, 2.2136E+00, 2.0365E+02, &
      5.8413828E+00, 3.49089E+01, 2.1953E+00, 2.0099E+02, &
      5.8705897E+00, 3.49024E+01, 2.1771E+00, 1.9836E+02, &
      5.8999427E+00, 3.48958E+01, 2.1591E+00, 1.9577E+02, &
      5.9294424E+00, 3.48892E+01, 2.1413E+00, 1.9321E+02, &
      5.9590896E+00, 3.48827E+01, 2.1236E+00, 1.9069E+02, &
      5.9888850E+00, 3.48761E+01, 2.1061E+00, 1.8820E+02, &
      6.0188295E+00, 3.48695E+01, 2.0887E+00, 1.8575E+02, &
      6.0489236E+00, 3.48629E+01, 2.0715E+00, 1.8333E+02, &
      6.0791682E+00, 3.48562E+01, 2.0544E+00, 1.8094E+02, &
      6.1095641E+00, 3.48496E+01, 2.0375E+00, 1.7858E+02, &
      6.1401119E+00, 3.48430E+01, 2.0207E+00, 1.7626E+02, &
      6.1708125E+00, 3.48363E+01, 2.0041E+00, 1.7397E+02, &
      6.2016665E+00, 3.48297E+01, 1.9876E+00, 1.7170E+02, &
      6.2326749E+00, 3.48230E+01, 1.9713E+00, 1.6947E+02, &
      6.2638382E+00, 3.48163E+01, 1.9551E+00, 1.6727E+02, &
      6.2951574E+00, 3.48096E+01, 1.9390E+00, 1.6510E+02, &
      6.3266332E+00, 3.48029E+01, 1.9231E+00, 1.6296E+02, &
      6.3582664E+00, 3.47963E+01, 1.9074E+00, 1.6084E+02, &
      6.3900577E+00, 3.47896E+01, 1.8918E+00, 1.5876E+02, &
      6.4220080E+00, 3.47829E+01, 1.8763E+00, 1.5670E+02, &
      6.4541180E+00, 3.47762E+01, 1.8609E+00, 1.5467E+02, &
      6.4863886E+00, 3.47695E+01, 1.8457E+00, 1.5267E+02, &
      6.5188206E+00, 3.47628E+01, 1.8306E+00, 1.5069E+02, &
      6.5514147E+00, 3.47561E+01, 1.8157E+00, 1.4874E+02, &
      6.5841717E+00, 3.47494E+01, 1.8009E+00, 1.4682E+02, &
      6.915365E+00, 3.46965E+01, 1.6587E+00, 1.2903E+02, &
      7.392525E+00, 3.45988E+01, 1.4787E+00, 1.0790E+02, &
      7.902609E+00, 3.44956E+01, 1.3179E+00, 9.0238E+01, &
      8.447890E+00, 3.43844E+01, 1.1723E+00, 7.5345E+01, &
      9.030794E+00, 3.42667E+01, 1.0403E+00, 6.2792E+01, &
      9.653919E+00, 3.41242E+01, 9.1328E-01, 5.1814E+01, &
      1.032004E+01, 3.39494E+01, 8.0257E-01, 4.2820E+01, &
      1.103212E+01, 3.37255E+01, 7.0596E-01, 3.5440E+01, &
      1.179334E+01, 3.34058E+01, 6.2158E-01, 2.9380E+01, &
      1.260708E+01, 3.28304E+01, 5.4782E-01, 2.4395E+01, &
      1.320423E+01, 3.17466E+01, 5.0214E-01, 2.1464E+01, &
      1.340633E+01, 3.03664E+01, 4.8804E-01, 2.0586E+01, &
      1.346023E+01, 2.87000E+01, 4.8439E-01, 2.0359E+01, &
      1.347697E+01, 2.71626E+01, 3.8023E+00, 1.4999E+02, &
      1.354107E+01, 3.04042E+01, 3.7655E+00, 1.4785E+02, &
      1.374317E+01, 3.19450E+01, 3.6529E+00, 1.4136E+02, &
      1.440688E+01, 3.33364E+01, 3.3331E+00, 1.2315E+02, &
      1.540095E+01, 3.41052E+01, 2.9803E+00, 1.0312E+02, &
      1.646362E+01, 3.45298E+01, 2.6831E+00, 8.6951E+01, &
      1.759961E+01, 3.48171E+01, 2.4201E+00, 7.3457E+01, &
      1.881398E+01, 3.50169E+01, 2.1650E+00, 6.1564E+01, &
      2.011215E+01, 3.51527E+01, 1.9335E+00, 5.1521E+01, &
      2.149988E+01, 3.52451E+01, 1.7246E+00, 4.3069E+01, &
      2.298338E+01, 3.53066E+01, 1.5372E+00, 3.5986E+01, &
      2.456923E+01, 3.53461E+01, 1.3700E+00, 3.0070E+01, &
      2.626450E+01, 3.53706E+01, 1.2213E+00, 2.5141E+01, &
      2.807676E+01, 3.54243E+01, 1.0819E+00, 2.0897E+01, &
      3.001405E+01, 3.54242E+01, 9.5548E-01, 1.7323E+01, &
      3.208502E+01, 3.54156E+01, 8.4388E-01, 1.4368E+01, &
      3.429889E+01, 3.54016E+01, 7.4531E-01, 1.1923E+01, &
      3.666551E+01, 3.53841E+01, 6.5825E-01, 9.8996E+00, &
      3.919543E+01, 3.53646E+01, 5.8121E-01, 8.2225E+00, &
      4.189992E+01, 3.53438E+01, 5.1301E-01, 6.8324E+00, &
      4.479101E+01, 3.53225E+01, 4.5284E-01, 5.6827E+00, &
      4.788159E+01, 3.53012E+01, 3.9976E-01, 4.7313E+00, &
      5.118542E+01, 3.52804E+01, 3.5293E-01, 3.9438E+00, &
      5.471721E+01, 3.52604E+01, 3.1159E-01, 3.2918E+00, &
      5.849270E+01, 3.52416E+01, 2.7507E-01, 2.7515E+00, &
      6.252870E+01, 3.52234E+01, 2.4139E-01, 2.2917E+00, &
      6.684318E+01, 3.52057E+01, 2.1184E-01, 1.9127E+00, &
      7.145536E+01, 3.51888E+01, 1.8592E-01, 1.6004E+00, &
      7.638578E+01, 3.51728E+01, 1.6317E-01, 1.3430E+00, &
      8.165640E+01, 3.51580E+01, 1.4307E-01, 1.1295E+00, &
      8.729069E+01, 3.51442E+01, 1.2516E-01, 9.5173E-01, &
      9.331374E+01, 3.51312E+01, 1.0949E-01, 8.0528E-01, &
      9.975239E+01, 3.51191E+01, 9.5793E-02, 6.8455E-01, &
      1.066353E+02, 3.51080E+01, 8.3810E-02, 5.8493E-01, &
      1.139931E+02, 3.50977E+01, 7.3328E-02, 5.0262E-01, &
      1.218587E+02, 3.50883E+01, 6.4159E-02, 4.3452E-01, &
      1.302669E+02, 3.50798E+01, 5.6139E-02, 3.7809E-01, &
      1.392553E+02, 3.50719E+01, 4.9123E-02, 3.3124E-01, &
      1.488640E+02, 3.50648E+01, 4.2986E-02, 2.9224E-01, &
      1.591356E+02, 3.50583E+01, 3.7616E-02, 2.5973E-01, &
      1.701159E+02, 3.50524E+01, 3.2918E-02, 2.3252E-01, &
      1.818539E+02, 3.50471E+01, 2.8808E-02, 2.0968E-01, &
      1.944018E+02, 3.50423E+01, 2.5212E-02, 1.9042E-01, &
      2.078156E+02, 3.50380E+01, 2.2066E-02, 1.7413E-01, &
      2.221548E+02, 3.50340E+01, 1.9312E-02, 1.6027E-01, &
      2.374835E+02, 3.50305E+01, 1.6903E-02, 1.4842E-01, &
      2.538699E+02, 3.50273E+01, 1.4795E-02, 1.3822E-01, &
      2.713869E+02, 3.50245E+01, 1.2950E-02, 1.2938E-01, &
      2.901126E+02, 3.50219E+01, 1.1335E-02, 1.2168E-01, &
      3.101304E+02, 3.50196E+01, 9.9222E-03, 1.1490E-01, &
      3.315294E+02, 3.50175E+01, 8.6856E-03, 1.0889E-01, &
      3.544049E+02, 3.50156E+01, 7.6033E-03, 1.0353E-01, &
      3.788588E+02, 3.50139E+01, 6.6560E-03, 9.8706E-02, &
      4.050001E+02, 3.50124E+01, 5.8269E-03, 9.4326E-02, &
      4.329451E+02, 3.50111E+01, 5.1012E-03, 9.0321E-02]
   case(36)
      frel = -1.4820E-01
      allocate(nist_tab(2236))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 9.29037E+00, 0.0000E+00, 1.6413E-04, &
      1.142761E-02, 9.28534E+00, 0.0000E+00, 2.2323E-04, &
      1.221612E-02, 9.27958E+00, 0.0000E+00, 3.0199E-04, &
      1.305903E-02, 9.27298E+00, 0.0000E+00, 4.0636E-04, &
      1.396010E-02, 9.26544E+00, 0.0000E+00, 5.4394E-04, &
      1.492335E-02, 9.25679E+00, 0.0000E+00, 7.2427E-04, &
      1.595306E-02, 9.24690E+00, 0.0000E+00, 9.5936E-04, &
      1.705382E-02, 9.23556E+00, 0.0000E+00, 1.2642E-03, &
      1.823053E-02, 9.22256E+00, 0.0000E+00, 1.6573E-03, &
      1.948844E-02, 9.20767E+00, 0.0000E+00, 2.1615E-03, &
      2.083314E-02, 9.19057E+00, 0.0000E+00, 2.8049E-03, &
      2.227063E-02, 9.17096E+00, 0.0000E+00, 3.6213E-03, &
      2.380730E-02, 9.14843E+00, 0.0000E+00, 4.6520E-03, &
      2.545001E-02, 9.12254E+00, 0.0000E+00, 5.9462E-03, &
      2.720606E-02, 9.09275E+00, 0.0000E+00, 7.5629E-03, &
      2.908327E-02, 9.05845E+00, 0.0000E+00, 9.5718E-03, &
      3.109002E-02, 9.01890E+00, 0.0000E+00, 1.2055E-02, &
      3.323523E-02, 8.97325E+00, 0.0000E+00, 1.5109E-02, &
      3.552846E-02, 8.92046E+00, 0.0000E+00, 1.8845E-02, &
      3.797993E-02, 8.85929E+00, 0.0000E+00, 2.3393E-02, &
      4.060054E-02, 8.78826E+00, 0.0000E+00, 2.8901E-02, &
      4.340198E-02, 8.70556E+00, 0.0000E+00, 3.5536E-02, &
      4.639671E-02, 8.60896E+00, 0.0000E+00, 4.3489E-02, &
      4.959809E-02, 8.49567E+00, 0.0000E+00, 5.2975E-02, &
      5.302035E-02, 8.36216E+00, 0.0000E+00, 6.4231E-02, &
      5.667876E-02, 8.20386E+00, 0.0000E+00, 7.7521E-02, &
      6.058959E-02, 8.01472E+00, 0.0000E+00, 9.3134E-02, &
      6.477028E-02, 7.78638E+00, 0.0000E+00, 1.1139E-01, &
      6.923942E-02, 7.50680E+00, 0.0000E+00, 1.3261E-01, &
      7.401695E-02, 7.15722E+00, 0.0000E+00, 1.5718E-01, &
      7.912411E-02, 6.70431E+00, 0.0000E+00, 1.8548E-01, &
      8.458368E-02, 6.06764E+00, 0.0000E+00, 2.1791E-01, &
      8.712200E-02, 5.64666E+00, 0.0000E+00, 2.3371E-01, &
      8.845550E-02, 5.27889E+00, 0.0000E+00, 2.4218E-01, &
      8.881110E-02, 5.01683E+00, 0.0000E+00, 2.4446E-01, &
      8.934450E-02, 5.11898E+00, 4.1142E-01, 2.3126E+03, &
      9.041995E-02, 5.09657E+00, 4.6449E-01, 2.5799E+03, &
      9.067800E-02, 5.07502E+00, 4.7811E-01, 2.6479E+03, &
      9.665893E-02, 4.30507E+00, 9.1368E-01, 4.7470E+03, &
      1.033284E-01, 3.35762E+00, 1.7260E+00, 8.3882E+03, &
      1.104581E-01, 2.47607E+00, 2.9189E+00, 1.3269E+04, &
      1.180797E-01, 1.80997E+00, 4.4806E+00, 1.9054E+04, &
      1.262272E-01, 1.49791E+00, 6.3314E+00, 2.5188E+04, &
      1.349368E-01, 1.63144E+00, 8.3376E+00, 3.1028E+04, &
      1.442475E-01, 2.23705E+00, 1.0342E+01, 3.6002E+04, &
      1.542005E-01, 3.27653E+00, 1.2194E+01, 3.9710E+04, &
      1.648404E-01, 4.66077E+00, 1.3777E+01, 4.1969E+04, &
      1.762144E-01, 6.26791E+00, 1.5016E+01, 4.2791E+04, &
      1.883732E-01, 7.95392E+00, 1.5880E+01, 4.2334E+04, &
      2.013709E-01, 9.52269E+00, 1.6377E+01, 4.0839E+04, &
      2.095240E-01, 1.01694E+01, 1.6509E+01, 3.9567E+04, &
      2.127310E-01, 1.00543E+01, 1.6530E+01, 3.9020E+04, &
      2.135862E-01, 9.60025E+00, 1.6533E+01, 3.8871E+04, &
      2.148690E-01, 1.02597E+01, 1.7597E+01, 4.1126E+04, &
      2.152655E-01, 1.04030E+01, 1.7602E+01, 4.1060E+04, &
      2.182460E-01, 1.10418E+01, 1.7627E+01, 4.0557E+04, &
      2.215865E-01, 1.14044E+01, 1.7641E+01, 3.9979E+04, &
      2.224773E-01, 1.13064E+01, 1.7643E+01, 3.9823E+04, &
      2.238135E-01, 1.17277E+01, 1.8100E+01, 4.0610E+04, &
      2.271540E-01, 1.23826E+01, 1.8110E+01, 4.0035E+04, &
      2.301188E-01, 1.28377E+01, 1.8110E+01, 3.9519E+04, &
      2.459970E-01, 1.48064E+01, 1.7991E+01, 3.6726E+04, &
      2.629708E-01, 1.64724E+01, 1.7686E+01, 3.3774E+04, &
      2.811158E-01, 1.77817E+01, 1.7230E+01, 3.0779E+04, &
      2.825634E-01, 1.78453E+01, 1.7189E+01, 3.0550E+04, &
      2.868883E-01, 1.78572E+01, 1.7066E+01, 2.9874E+04, &
      2.880417E-01, 1.75673E+01, 1.7033E+01, 2.9696E+04, &
      2.897716E-01, 1.80720E+01, 1.7719E+01, 3.0708E+04, &
      2.940966E-01, 1.87115E+01, 1.7597E+01, 3.0047E+04, &
      3.005128E-01, 1.93348E+01, 1.7410E+01, 2.9093E+04, &
      3.212482E-01, 2.08660E+01, 1.6780E+01, 2.6231E+04, &
      3.434143E-01, 2.21571E+01, 1.6086E+01, 2.3523E+04, &
      3.671099E-01, 2.32881E+01, 1.5344E+01, 2.0991E+04, &
      3.924405E-01, 2.42803E+01, 1.4562E+01, 1.8636E+04, &
      4.195189E-01, 2.51426E+01, 1.3759E+01, 1.6472E+04, &
      4.484657E-01, 2.58850E+01, 1.2949E+01, 1.4502E+04, &
      4.794098E-01, 2.65175E+01, 1.2145E+01, 1.2724E+04, &
      5.124891E-01, 2.70500E+01, 1.1356E+01, 1.1131E+04, &
      5.478508E-01, 2.74917E+01, 1.0589E+01, 9.7098E+03, &
      5.856525E-01, 2.78510E+01, 9.8506E+00, 8.4501E+03, &
      6.260625E-01, 2.81356E+01, 9.1436E+00, 7.3380E+03, &
      6.692609E-01, 2.83463E+01, 8.4710E+00, 6.3602E+03, &
      7.154399E-01, 2.85005E+01, 7.8343E+00, 5.5032E+03, &
      7.648052E-01, 2.85971E+01, 7.2342E+00, 4.7544E+03, &
      8.175768E-01, 2.86402E+01, 6.6708E+00, 4.1020E+03, &
      8.739896E-01, 2.86334E+01, 6.1435E+00, 3.5346E+03, &
      9.0000000E-01, 2.88820E+01, 5.9226E+00, 3.3095E+03, &
      9.0450000E-01, 2.88793E+01, 5.8857E+00, 3.2726E+03, &
      9.0902250E-01, 2.88765E+01, 5.8490E+00, 3.2360E+03, &
      9.1356761E-01, 2.88735E+01, 5.8125E+00, 3.1999E+03, &
      9.1813545E-01, 2.88703E+01, 5.7762E+00, 3.1641E+03, &
      9.2272613E-01, 2.88669E+01, 5.7401E+00, 3.1288E+03, &
      9.2733976E-01, 2.88633E+01, 5.7041E+00, 3.0938E+03, &
      9.3197646E-01, 2.88596E+01, 5.6684E+00, 3.0592E+03, &
      9.3663634E-01, 2.88558E+01, 5.6328E+00, 3.0249E+03, &
      9.4131952E-01, 2.88519E+01, 5.5975E+00, 2.9911E+03, &
      9.4602612E-01, 2.88479E+01, 5.5624E+00, 2.9576E+03, &
      9.5075625E-01, 2.88438E+01, 5.5274E+00, 2.9244E+03, &
      9.5551003E-01, 2.88397E+01, 5.4927E+00, 2.8917E+03, &
      9.6028758E-01, 2.88357E+01, 5.4582E+00, 2.8592E+03, &
      9.6508902E-01, 2.88318E+01, 5.4238E+00, 2.8272E+03, &
      9.6991446E-01, 2.88280E+01, 5.3897E+00, 2.7955E+03, &
      9.7476404E-01, 2.88245E+01, 5.3558E+00, 2.7641E+03, &
      9.7963786E-01, 2.88214E+01, 5.3220E+00, 2.7331E+03, &
      9.8453605E-01, 2.88188E+01, 5.2885E+00, 2.7025E+03, &
      9.8945873E-01, 2.88169E+01, 5.2552E+00, 2.6722E+03, &
      9.9440602E-01, 2.88161E+01, 5.2221E+00, 2.6422E+03, &
      9.9937805E-01, 2.88167E+01, 5.1892E+00, 2.6125E+03, &
      1.0043749E+00, 2.88063E+01, 5.1494E+00, 2.5797E+03, &
      1.0093968E+00, 2.87937E+01, 5.1090E+00, 2.5468E+03, &
      1.0144438E+00, 2.87805E+01, 5.0689E+00, 2.5143E+03, &
      1.0195160E+00, 2.87668E+01, 5.0291E+00, 2.4822E+03, &
      1.0246136E+00, 2.87525E+01, 4.9897E+00, 2.4506E+03, &
      1.0297367E+00, 2.87376E+01, 4.9507E+00, 2.4194E+03, &
      1.0348853E+00, 2.87222E+01, 4.9119E+00, 2.3886E+03, &
      1.0400598E+00, 2.87063E+01, 4.8736E+00, 2.3582E+03, &
      1.0452601E+00, 2.86898E+01, 4.8355E+00, 2.3282E+03, &
      1.0504864E+00, 2.86727E+01, 4.7978E+00, 2.2986E+03, &
      1.0557388E+00, 2.86551E+01, 4.7604E+00, 2.2694E+03, &
      1.0610175E+00, 2.86370E+01, 4.7233E+00, 2.2406E+03, &
      1.0663226E+00, 2.86183E+01, 4.6866E+00, 2.2122E+03, &
      1.0716542E+00, 2.85991E+01, 4.6502E+00, 2.1842E+03, &
      1.0770125E+00, 2.85793E+01, 4.6141E+00, 2.1565E+03, &
      1.0823975E+00, 2.85589E+01, 4.5783E+00, 2.1292E+03, &
      1.0878095E+00, 2.85381E+01, 4.5428E+00, 2.1023E+03, &
      1.0932486E+00, 2.85166E+01, 4.5076E+00, 2.0757E+03, &
      1.0987148E+00, 2.84946E+01, 4.4728E+00, 2.0495E+03, &
      1.1042084E+00, 2.84720E+01, 4.4382E+00, 2.0236E+03, &
      1.1097294E+00, 2.84488E+01, 4.4040E+00, 1.9981E+03, &
      1.1152781E+00, 2.84251E+01, 4.3700E+00, 1.9729E+03, &
      1.1208545E+00, 2.84007E+01, 4.3363E+00, 1.9480E+03, &
      1.1264587E+00, 2.83758E+01, 4.3029E+00, 1.9234E+03, &
      1.1320910E+00, 2.83503E+01, 4.2699E+00, 1.8992E+03, &
      1.1377515E+00, 2.83242E+01, 4.2371E+00, 1.8753E+03, &
      1.1434402E+00, 2.82974E+01, 4.2045E+00, 1.8518E+03, &
      1.1491574E+00, 2.82701E+01, 4.1723E+00, 1.8285E+03, &
      1.1549032E+00, 2.82420E+01, 4.1402E+00, 1.8055E+03, &
      1.1606777E+00, 2.82134E+01, 4.1081E+00, 1.7826E+03, &
      1.1664811E+00, 2.81840E+01, 4.0762E+00, 1.7600E+03, &
      1.1723135E+00, 2.81539E+01, 4.0446E+00, 1.7378E+03, &
      1.1781751E+00, 2.81231E+01, 4.0132E+00, 1.7158E+03, &
      1.1840660E+00, 2.80915E+01, 3.9822E+00, 1.6941E+03, &
      1.1899863E+00, 2.80592E+01, 3.9514E+00, 1.6727E+03, &
      1.1959362E+00, 2.80261E+01, 3.9208E+00, 1.6516E+03, &
      1.2019159E+00, 2.79923E+01, 3.8906E+00, 1.6308E+03, &
      1.2079255E+00, 2.79576E+01, 3.8606E+00, 1.6103E+03, &
      1.2139651E+00, 2.79222E+01, 3.8308E+00, 1.5900E+03, &
      1.2200350E+00, 2.78858E+01, 3.8014E+00, 1.5700E+03, &
      1.2261351E+00, 2.78487E+01, 3.7721E+00, 1.5502E+03, &
      1.2322658E+00, 2.78106E+01, 3.7431E+00, 1.5307E+03, &
      1.2384271E+00, 2.77716E+01, 3.7144E+00, 1.5115E+03, &
      1.2446193E+00, 2.77317E+01, 3.6859E+00, 1.4925E+03, &
      1.2508424E+00, 2.76908E+01, 3.6577E+00, 1.4738E+03, &
      1.2570966E+00, 2.76489E+01, 3.6297E+00, 1.4553E+03, &
      1.2633821E+00, 2.76060E+01, 3.6020E+00, 1.4370E+03, &
      1.2696990E+00, 2.75620E+01, 3.5744E+00, 1.4190E+03, &
      1.2760475E+00, 2.75170E+01, 3.5472E+00, 1.4013E+03, &
      1.2824277E+00, 2.74707E+01, 3.5201E+00, 1.3837E+03, &
      1.2888399E+00, 2.74234E+01, 3.4933E+00, 1.3664E+03, &
      1.2952840E+00, 2.73748E+01, 3.4667E+00, 1.3494E+03, &
      1.3017605E+00, 2.73249E+01, 3.4403E+00, 1.3325E+03, &
      1.3082693E+00, 2.72737E+01, 3.4142E+00, 1.3159E+03, &
      1.3148106E+00, 2.72211E+01, 3.3883E+00, 1.2995E+03, &
      1.3213847E+00, 2.71672E+01, 3.3626E+00, 1.2833E+03, &
      1.3279916E+00, 2.71117E+01, 3.3371E+00, 1.2673E+03, &
      1.3346316E+00, 2.70547E+01, 3.3119E+00, 1.2515E+03, &
      1.3413047E+00, 2.69961E+01, 3.2868E+00, 1.2359E+03, &
      1.3480112E+00, 2.69358E+01, 3.2620E+00, 1.2206E+03, &
      1.3547513E+00, 2.68737E+01, 3.2374E+00, 1.2054E+03, &
      1.3615250E+00, 2.68098E+01, 3.2130E+00, 1.1904E+03, &
      1.3683327E+00, 2.67439E+01, 3.1888E+00, 1.1757E+03, &
      1.3751743E+00, 2.66760E+01, 3.1648E+00, 1.1611E+03, &
      1.3820502E+00, 2.66060E+01, 3.1410E+00, 1.1467E+03, &
      1.3889605E+00, 2.65337E+01, 3.1174E+00, 1.1325E+03, &
      1.3959053E+00, 2.64591E+01, 3.0940E+00, 1.1184E+03, &
      1.4028848E+00, 2.63819E+01, 3.0708E+00, 1.1046E+03, &
      1.4098992E+00, 2.63021E+01, 3.0478E+00, 1.0910E+03, &
      1.4169487E+00, 2.62195E+01, 3.0250E+00, 1.0775E+03, &
      1.4240335E+00, 2.61339E+01, 3.0024E+00, 1.0642E+03, &
      1.4311536E+00, 2.60451E+01, 2.9800E+00, 1.0510E+03, &
      1.4383094E+00, 2.59530E+01, 2.9577E+00, 1.0381E+03, &
      1.4455009E+00, 2.58573E+01, 2.9357E+00, 1.0253E+03, &
      1.4527284E+00, 2.57577E+01, 2.9138E+00, 1.0126E+03, &
      1.4599921E+00, 2.56540E+01, 2.8921E+00, 1.0002E+03, &
      1.4672920E+00, 2.55459E+01, 2.8706E+00, 9.8786E+02, &
      1.4746285E+00, 2.54330E+01, 2.8493E+00, 9.7571E+02, &
      1.4820016E+00, 2.53150E+01, 2.8282E+00, 9.6372E+02, &
      1.4894117E+00, 2.51914E+01, 2.8072E+00, 9.5189E+02, &
      1.4968587E+00, 2.50618E+01, 2.7864E+00, 9.4020E+02, &
      1.5043430E+00, 2.49256E+01, 2.7658E+00, 9.2867E+02, &
      1.5118647E+00, 2.47821E+01, 2.7453E+00, 9.1729E+02, &
      1.5194240E+00, 2.46308E+01, 2.7251E+00, 9.0605E+02, &
      1.5270212E+00, 2.44707E+01, 2.7050E+00, 8.9496E+02, &
      1.5346563E+00, 2.43009E+01, 2.6850E+00, 8.8401E+02, &
      1.5423295E+00, 2.41203E+01, 2.6652E+00, 8.7320E+02, &
      1.5500412E+00, 2.39276E+01, 2.6456E+00, 8.6253E+02, &
      1.5577914E+00, 2.37211E+01, 2.6262E+00, 8.5199E+02, &
      1.5655804E+00, 2.34991E+01, 2.6069E+00, 8.4159E+02, &
      1.5734083E+00, 2.32591E+01, 2.5878E+00, 8.3133E+02, &
      1.5812753E+00, 2.29983E+01, 2.5688E+00, 8.2120E+02, &
      1.5891817E+00, 2.27131E+01, 2.5500E+00, 8.1119E+02, &
      1.5971276E+00, 2.23986E+01, 2.5313E+00, 8.0132E+02, &
      1.6051132E+00, 2.20489E+01, 2.5128E+00, 7.9157E+02, &
      1.6131388E+00, 2.16552E+01, 2.4945E+00, 7.8194E+02, &
      1.6212045E+00, 2.12058E+01, 2.4763E+00, 7.7244E+02, &
      1.6293105E+00, 2.06827E+01, 2.4582E+00, 7.6306E+02, &
      1.6374571E+00, 2.00577E+01, 2.4403E+00, 7.5380E+02, &
      1.6456443E+00, 1.92819E+01, 2.4226E+00, 7.4465E+02, &
      1.6538726E+00, 1.82574E+01, 2.4049E+00, 7.3563E+02, &
      1.6621419E+00, 1.67363E+01, 2.3875E+00, 7.2672E+02, &
      1.6704526E+00, 1.36146E+01, 2.3700E+00, 7.1788E+02, &
      1.6747007E+00, 4.64497E+00, 2.3607E+00, 7.1326E+02, &
      1.6750994E+00, 4.49118E+00, 1.1391E+01, 3.4201E+03, &
      1.6788049E+00, 1.30021E+01, 1.1350E+01, 3.4004E+03, &
      1.6871989E+00, 1.61340E+01, 1.1259E+01, 3.3564E+03, &
      1.6956349E+00, 1.73862E+01, 1.1169E+01, 3.3130E+03, &
      1.7041131E+00, 1.80060E+01, 1.1079E+01, 3.2701E+03, &
      1.7126337E+00, 1.81618E+01, 1.0990E+01, 3.2278E+03, &
      1.7211968E+00, 1.75651E+01, 1.0902E+01, 3.1860E+03, &
      1.7269841E+00, 1.32390E+01, 1.0843E+01, 3.1582E+03, &
      1.7274159E+00, 1.31930E+01, 1.5298E+01, 4.4525E+03, &
      1.7298028E+00, 1.68801E+01, 1.5264E+01, 4.4366E+03, &
      1.7384518E+00, 1.94458E+01, 1.5143E+01, 4.3793E+03, &
      1.7471441E+00, 2.06785E+01, 1.5022E+01, 4.3228E+03, &
      1.7558798E+00, 2.15639E+01, 1.4902E+01, 4.2671E+03, &
      1.7646592E+00, 2.22720E+01, 1.4783E+01, 4.2121E+03, &
      1.7734825E+00, 2.28675E+01, 1.4665E+01, 4.1578E+03, &
      1.7823499E+00, 2.33827E+01, 1.4548E+01, 4.1042E+03, &
      1.7912617E+00, 2.38363E+01, 1.4432E+01, 4.0513E+03, &
      1.8002180E+00, 2.42405E+01, 1.4317E+01, 3.9990E+03, &
      1.8092191E+00, 2.46033E+01, 1.4203E+01, 3.9475E+03, &
      1.8182652E+00, 2.49304E+01, 1.4090E+01, 3.8967E+03, &
      1.8273565E+00, 2.52257E+01, 1.3978E+01, 3.8465E+03, &
      1.8364933E+00, 2.54918E+01, 1.3867E+01, 3.7969E+03, &
      1.8456757E+00, 2.57302E+01, 1.3756E+01, 3.7480E+03, &
      1.8549041E+00, 2.59413E+01, 1.3647E+01, 3.6997E+03, &
      1.8641786E+00, 2.61241E+01, 1.3538E+01, 3.6521E+03, &
      1.8734995E+00, 2.62757E+01, 1.3430E+01, 3.6051E+03, &
      1.8828670E+00, 2.63900E+01, 1.3324E+01, 3.5587E+03, &
      1.8922814E+00, 2.64538E+01, 1.3218E+01, 3.5129E+03, &
      1.9017428E+00, 2.64363E+01, 1.3112E+01, 3.4677E+03, &
      1.9112515E+00, 2.62375E+01, 1.3008E+01, 3.4231E+03, &
      1.9194747E+00, 2.52812E+01, 1.2919E+01, 3.3851E+03, &
      1.9208077E+00, 2.40373E+01, 1.2905E+01, 3.3790E+03, &
      1.9225253E+00, 2.53179E+01, 1.4817E+01, 3.8755E+03, &
      1.9304118E+00, 2.66053E+01, 1.4725E+01, 3.8357E+03, &
      1.9400638E+00, 2.72335E+01, 1.4613E+01, 3.7877E+03, &
      1.9497642E+00, 2.76713E+01, 1.4502E+01, 3.7403E+03, &
      1.9595130E+00, 2.80275E+01, 1.4392E+01, 3.6936E+03, &
      1.9693105E+00, 2.83362E+01, 1.4283E+01, 3.6474E+03, &
      1.9791571E+00, 2.86126E+01, 1.4175E+01, 3.6018E+03, &
      1.9890529E+00, 2.88651E+01, 1.4068E+01, 3.5568E+03, &
      1.9989981E+00, 2.90986E+01, 1.3961E+01, 3.5123E+03, &
      2.0089931E+00, 2.93165E+01, 1.3855E+01, 3.4685E+03, &
      2.0190381E+00, 2.95207E+01, 1.3751E+01, 3.4254E+03, &
      2.0291333E+00, 2.97142E+01, 1.3650E+01, 3.3832E+03, &
      2.0392790E+00, 2.98985E+01, 1.3549E+01, 3.3416E+03, &
      2.0494754E+00, 3.00744E+01, 1.3449E+01, 3.3006E+03, &
      2.0597227E+00, 3.02428E+01, 1.3351E+01, 3.2601E+03, &
      2.0700213E+00, 3.04043E+01, 1.3253E+01, 3.2202E+03, &
      2.0803714E+00, 3.05596E+01, 1.3156E+01, 3.1808E+03, &
      2.0907733E+00, 3.07091E+01, 1.3060E+01, 3.1419E+03, &
      2.1012272E+00, 3.08532E+01, 1.2965E+01, 3.1036E+03, &
      2.1117333E+00, 3.09923E+01, 1.2870E+01, 3.0657E+03, &
      2.1222920E+00, 3.11269E+01, 1.2777E+01, 3.0284E+03, &
      2.1329034E+00, 3.12571E+01, 1.2684E+01, 2.9915E+03, &
      2.1435680E+00, 3.13834E+01, 1.2592E+01, 2.9551E+03, &
      2.1542858E+00, 3.15059E+01, 1.2501E+01, 2.9192E+03, &
      2.1650572E+00, 3.16249E+01, 1.2411E+01, 2.8838E+03, &
      2.1758825E+00, 3.17407E+01, 1.2322E+01, 2.8488E+03, &
      2.1867619E+00, 3.18532E+01, 1.2231E+01, 2.8139E+03, &
      2.1976957E+00, 3.19623E+01, 1.2142E+01, 2.7794E+03, &
      2.2086842E+00, 3.20683E+01, 1.2053E+01, 2.7454E+03, &
      2.2197276E+00, 3.21713E+01, 1.1964E+01, 2.7118E+03, &
      2.2308263E+00, 3.22715E+01, 1.1877E+01, 2.6786E+03, &
      2.2419804E+00, 3.23690E+01, 1.1790E+01, 2.6458E+03, &
      2.2531903E+00, 3.24640E+01, 1.1703E+01, 2.6133E+03, &
      2.2644562E+00, 3.25564E+01, 1.1617E+01, 2.5812E+03, &
      2.2757785E+00, 3.26462E+01, 1.1531E+01, 2.5495E+03, &
      2.2871574E+00, 3.27336E+01, 1.1446E+01, 2.5181E+03, &
      2.2985932E+00, 3.28188E+01, 1.1361E+01, 2.4871E+03, &
      2.3100862E+00, 3.29017E+01, 1.1277E+01, 2.4566E+03, &
      2.3216366E+00, 3.29825E+01, 1.1194E+01, 2.4264E+03, &
      2.3332448E+00, 3.30613E+01, 1.1112E+01, 2.3966E+03, &
      2.3449110E+00, 3.31382E+01, 1.1030E+01, 2.3671E+03, &
      2.3566356E+00, 3.32132E+01, 1.0948E+01, 2.3380E+03, &
      2.3684187E+00, 3.32864E+01, 1.0868E+01, 2.3093E+03, &
      2.3802608E+00, 3.33578E+01, 1.0788E+01, 2.2809E+03, &
      2.3921621E+00, 3.34275E+01, 1.0708E+01, 2.2529E+03, &
      2.4041230E+00, 3.34957E+01, 1.0629E+01, 2.2252E+03, &
      2.4161436E+00, 3.35622E+01, 1.0551E+01, 2.1979E+03, &
      2.4282243E+00, 3.36273E+01, 1.0473E+01, 2.1709E+03, &
      2.4403654E+00, 3.36909E+01, 1.0396E+01, 2.1442E+03, &
      2.4525672E+00, 3.37531E+01, 1.0319E+01, 2.1179E+03, &
      2.4648301E+00, 3.38140E+01, 1.0243E+01, 2.0918E+03, &
      2.4771542E+00, 3.38736E+01, 1.0168E+01, 2.0661E+03, &
      2.4895400E+00, 3.39320E+01, 1.0093E+01, 2.0407E+03, &
      2.5019877E+00, 3.39892E+01, 1.0018E+01, 2.0156E+03, &
      2.5144976E+00, 3.40449E+01, 9.9428E+00, 1.9906E+03, &
      2.5270701E+00, 3.40993E+01, 9.8684E+00, 1.9659E+03, &
      2.5397055E+00, 3.41524E+01, 9.7945E+00, 1.9416E+03, &
      2.5524040E+00, 3.42042E+01, 9.7211E+00, 1.9175E+03, &
      2.5651660E+00, 3.42548E+01, 9.6483E+00, 1.8937E+03, &
      2.5779919E+00, 3.43042E+01, 9.5760E+00, 1.8702E+03, &
      2.5908818E+00, 3.43525E+01, 9.5043E+00, 1.8470E+03, &
      2.6038362E+00, 3.43997E+01, 9.4331E+00, 1.8241E+03, &
      2.6168554E+00, 3.44457E+01, 9.3624E+00, 1.8015E+03, &
      2.6299397E+00, 3.44908E+01, 9.2922E+00, 1.7792E+03, &
      2.6430894E+00, 3.45348E+01, 9.2225E+00, 1.7571E+03, &
      2.6563048E+00, 3.45779E+01, 9.1534E+00, 1.7353E+03, &
      2.6695863E+00, 3.46200E+01, 9.0847E+00, 1.7138E+03, &
      2.6829343E+00, 3.46611E+01, 9.0166E+00, 1.6925E+03, &
      2.6963489E+00, 3.47014E+01, 8.9490E+00, 1.6715E+03, &
      2.7098307E+00, 3.47408E+01, 8.8819E+00, 1.6508E+03, &
      2.7233798E+00, 3.47793E+01, 8.8152E+00, 1.6303E+03, &
      2.7369967E+00, 3.48170E+01, 8.7491E+00, 1.6101E+03, &
      2.7506817E+00, 3.48539E+01, 8.6835E+00, 1.5901E+03, &
      2.7644351E+00, 3.48900E+01, 8.6184E+00, 1.5704E+03, &
      2.7782573E+00, 3.49253E+01, 8.5538E+00, 1.5509E+03, &
      2.7921486E+00, 3.49600E+01, 8.4897E+00, 1.5316E+03, &
      2.8061093E+00, 3.49939E+01, 8.4260E+00, 1.5127E+03, &
      2.8201399E+00, 3.50271E+01, 8.3629E+00, 1.4939E+03, &
      2.8342406E+00, 3.50597E+01, 8.3002E+00, 1.4754E+03, &
      2.8484118E+00, 3.50917E+01, 8.2380E+00, 1.4571E+03, &
      2.8626539E+00, 3.51230E+01, 8.1763E+00, 1.4390E+03, &
      2.8769671E+00, 3.51538E+01, 8.1151E+00, 1.4212E+03, &
      2.8913520E+00, 3.51841E+01, 8.0544E+00, 1.4036E+03, &
      2.9058087E+00, 3.52139E+01, 7.9941E+00, 1.3862E+03, &
      2.9203378E+00, 3.52433E+01, 7.9343E+00, 1.3690E+03, &
      2.9349394E+00, 3.52723E+01, 7.8749E+00, 1.3521E+03, &
      2.9496141E+00, 3.53012E+01, 7.8161E+00, 1.3354E+03, &
      2.9643622E+00, 3.53301E+01, 7.7577E+00, 1.3188E+03, &
      2.9791840E+00, 3.53594E+01, 7.6997E+00, 1.3025E+03, &
      2.9940799E+00, 3.53902E+01, 7.6422E+00, 1.2864E+03, &
      3.0090503E+00, 3.54225E+01, 7.5826E+00, 1.2701E+03, &
      3.0240956E+00, 3.54522E+01, 7.5217E+00, 1.2537E+03, &
      3.0392161E+00, 3.54800E+01, 7.4613E+00, 1.2375E+03, &
      3.0544122E+00, 3.55064E+01, 7.4014E+00, 1.2215E+03, &
      3.0696842E+00, 3.55316E+01, 7.3420E+00, 1.2057E+03, &
      3.0850326E+00, 3.55558E+01, 7.2831E+00, 1.1901E+03, &
      3.1004578E+00, 3.55790E+01, 7.2247E+00, 1.1747E+03, &
      3.1159601E+00, 3.56013E+01, 7.1667E+00, 1.1596E+03, &
      3.1315399E+00, 3.56229E+01, 7.1092E+00, 1.1446E+03, &
      3.1471976E+00, 3.56438E+01, 7.0522E+00, 1.1298E+03, &
      3.1629336E+00, 3.56640E+01, 6.9957E+00, 1.1152E+03, &
      3.1787482E+00, 3.56836E+01, 6.9396E+00, 1.1008E+03, &
      3.1946420E+00, 3.57026E+01, 6.8840E+00, 1.0866E+03, &
      3.2106152E+00, 3.57211E+01, 6.8289E+00, 1.0726E+03, &
      3.2266683E+00, 3.57390E+01, 6.7742E+00, 1.0588E+03, &
      3.2428016E+00, 3.57565E+01, 6.7200E+00, 1.0451E+03, &
      3.2590156E+00, 3.57734E+01, 6.6662E+00, 1.0317E+03, &
      3.2753107E+00, 3.57899E+01, 6.6129E+00, 1.0184E+03, &
      3.2916873E+00, 3.58060E+01, 6.5600E+00, 1.0052E+03, &
      3.3081457E+00, 3.58217E+01, 6.5076E+00, 9.9229E+02, &
      3.3246864E+00, 3.58370E+01, 6.4556E+00, 9.7952E+02, &
      3.3413099E+00, 3.58519E+01, 6.4041E+00, 9.6691E+02, &
      3.3580164E+00, 3.59671E+01, 6.3518E+00, 9.5429E+02, &
      3.3748065E+00, 3.59811E+01, 6.2987E+00, 9.4166E+02, &
      3.3916805E+00, 3.59946E+01, 6.2461E+00, 9.2920E+02, &
      3.4086389E+00, 3.60076E+01, 6.1940E+00, 9.1691E+02, &
      3.4256821E+00, 3.60200E+01, 6.1424E+00, 9.0479E+02, &
      3.4428105E+00, 3.60319E+01, 6.0912E+00, 8.9283E+02, &
      3.4600246E+00, 3.60940E+01, 6.0401E+00, 8.8098E+02, &
      3.4773247E+00, 3.61050E+01, 5.9885E+00, 8.6917E+02, &
      3.4947113E+00, 3.61154E+01, 5.9375E+00, 8.5752E+02, &
      3.5121849E+00, 3.61254E+01, 5.8869E+00, 8.4603E+02, &
      3.5297458E+00, 3.61348E+01, 5.8368E+00, 8.3471E+02, &
      3.5473945E+00, 3.61437E+01, 5.7872E+00, 8.2354E+02, &
      3.5651315E+00, 3.61523E+01, 5.7380E+00, 8.1253E+02, &
      3.5829572E+00, 3.61603E+01, 5.6893E+00, 8.0167E+02, &
      3.6008719E+00, 3.61680E+01, 5.6411E+00, 7.9096E+02, &
      3.6188763E+00, 3.61753E+01, 5.5932E+00, 7.8040E+02, &
      3.6369707E+00, 3.61822E+01, 5.5459E+00, 7.6998E+02, &
      3.6551555E+00, 3.61888E+01, 5.4989E+00, 7.5972E+02, &
      3.6734313E+00, 3.61950E+01, 5.4524E+00, 7.4959E+02, &
      3.6917985E+00, 3.62010E+01, 5.4063E+00, 7.3960E+02, &
      3.7102575E+00, 3.62066E+01, 5.3607E+00, 7.2975E+02, &
      3.7288088E+00, 3.62119E+01, 5.3155E+00, 7.2004E+02, &
      3.7474528E+00, 3.62169E+01, 5.2706E+00, 7.1046E+02, &
      3.7661901E+00, 3.62216E+01, 5.2262E+00, 7.0102E+02, &
      3.7850210E+00, 3.62261E+01, 5.1823E+00, 6.9170E+02, &
      3.8039461E+00, 3.62304E+01, 5.1387E+00, 6.8251E+02, &
      3.8229659E+00, 3.62344E+01, 5.0955E+00, 6.7345E+02, &
      3.8420807E+00, 3.62807E+01, 5.0527E+00, 6.6452E+02, &
      3.8612911E+00, 3.62845E+01, 5.0089E+00, 6.5553E+02, &
      3.8805975E+00, 3.62880E+01, 4.9656E+00, 6.4667E+02, &
      3.9000005E+00, 3.62911E+01, 4.9226E+00, 6.3793E+02, &
      3.9195005E+00, 3.62939E+01, 4.8800E+00, 6.2931E+02, &
      3.9390980E+00, 3.62964E+01, 4.8379E+00, 6.2081E+02, &
      3.9587935E+00, 3.62986E+01, 4.7961E+00, 6.1243E+02, &
      3.9785875E+00, 3.63005E+01, 4.7547E+00, 6.0417E+02, &
      3.9984804E+00, 3.63022E+01, 4.7137E+00, 5.9602E+02, &
      4.0184728E+00, 3.63036E+01, 4.6730E+00, 5.8798E+02, &
      4.0385652E+00, 3.63047E+01, 4.6324E+00, 5.8001E+02, &
      4.0587580E+00, 3.63056E+01, 4.5922E+00, 5.7216E+02, &
      4.0790518E+00, 3.63063E+01, 4.5524E+00, 5.6442E+02, &
      4.0994471E+00, 3.63067E+01, 4.5129E+00, 5.5679E+02, &
      4.1199443E+00, 3.63069E+01, 4.4738E+00, 5.4926E+02, &
      4.1405440E+00, 3.63068E+01, 4.4351E+00, 5.4183E+02, &
      4.1612467E+00, 3.63066E+01, 4.3967E+00, 5.3451E+02, &
      4.1820530E+00, 3.63062E+01, 4.3586E+00, 5.2730E+02, &
      4.2029632E+00, 3.63055E+01, 4.3208E+00, 5.2016E+02, &
      4.2239781E+00, 3.63047E+01, 4.2833E+00, 5.1312E+02, &
      4.2450980E+00, 3.63037E+01, 4.2462E+00, 5.0619E+02, &
      4.2663234E+00, 3.63024E+01, 4.2094E+00, 4.9934E+02, &
      4.2876551E+00, 3.63010E+01, 4.1730E+00, 4.9260E+02, &
      4.3090933E+00, 3.62995E+01, 4.1368E+00, 4.8595E+02, &
      4.3306388E+00, 3.62977E+01, 4.1011E+00, 4.7939E+02, &
      4.3522920E+00, 3.62958E+01, 4.0656E+00, 4.7292E+02, &
      4.3740535E+00, 3.62938E+01, 4.0305E+00, 4.6654E+02, &
      4.3959237E+00, 3.62916E+01, 3.9957E+00, 4.6025E+02, &
      4.4179033E+00, 3.62892E+01, 3.9612E+00, 4.5404E+02, &
      4.4399929E+00, 3.62867E+01, 3.9270E+00, 4.4793E+02, &
      4.4621928E+00, 3.62841E+01, 3.8931E+00, 4.4190E+02, &
      4.4845038E+00, 3.62813E+01, 3.8596E+00, 4.3595E+02, &
      4.5069263E+00, 3.62784E+01, 3.8264E+00, 4.3008E+02, &
      4.5294609E+00, 3.62753E+01, 3.7934E+00, 4.2430E+02, &
      4.5521082E+00, 3.62722E+01, 3.7608E+00, 4.1859E+02, &
      4.5748688E+00, 3.62689E+01, 3.7285E+00, 4.1297E+02, &
      4.5977431E+00, 3.62655E+01, 3.6964E+00, 4.0742E+02, &
      4.6207318E+00, 3.62619E+01, 3.6647E+00, 4.0195E+02, &
      4.6438355E+00, 3.62583E+01, 3.6332E+00, 3.9655E+02, &
      4.6670547E+00, 3.62546E+01, 3.6021E+00, 3.9123E+02, &
      4.6903900E+00, 3.62507E+01, 3.5712E+00, 3.8599E+02, &
      4.7138419E+00, 3.62468E+01, 3.5406E+00, 3.8081E+02, &
      4.7374111E+00, 3.62428E+01, 3.5103E+00, 3.7571E+02, &
      4.7610982E+00, 3.62386E+01, 3.4802E+00, 3.7068E+02, &
      4.7849037E+00, 3.62344E+01, 3.4505E+00, 3.6572E+02, &
      4.8088282E+00, 3.62301E+01, 3.4210E+00, 3.6082E+02, &
      4.8328723E+00, 3.62257E+01, 3.3917E+00, 3.5600E+02, &
      4.8570367E+00, 3.62212E+01, 3.3628E+00, 3.5124E+02, &
      4.8813219E+00, 3.62166E+01, 3.3341E+00, 3.4654E+02, &
      4.9057285E+00, 3.62120E+01, 3.3057E+00, 3.4191E+02, &
      4.9302571E+00, 3.62072E+01, 3.2775E+00, 3.3735E+02, &
      4.9549084E+00, 3.62024E+01, 3.2496E+00, 3.3284E+02, &
      4.9796829E+00, 3.61976E+01, 3.2219E+00, 3.2840E+02, &
      5.0045814E+00, 3.61926E+01, 3.1945E+00, 3.2402E+02, &
      5.0296043E+00, 3.61876E+01, 3.1673E+00, 3.1971E+02, &
      5.0547523E+00, 3.61825E+01, 3.1404E+00, 3.1545E+02, &
      5.0800260E+00, 3.61774E+01, 3.1137E+00, 3.1124E+02, &
      5.1054262E+00, 3.61722E+01, 3.0873E+00, 3.0710E+02, &
      5.1309533E+00, 3.61669E+01, 3.0611E+00, 3.0301E+02, &
      5.1566081E+00, 3.61616E+01, 3.0352E+00, 2.9898E+02, &
      5.1823911E+00, 3.61562E+01, 3.0094E+00, 2.9501E+02, &
      5.2083031E+00, 3.61508E+01, 2.9840E+00, 2.9109E+02, &
      5.2343446E+00, 3.61453E+01, 2.9587E+00, 2.8722E+02, &
      5.2605163E+00, 3.61398E+01, 2.9337E+00, 2.8341E+02, &
      5.2868189E+00, 3.61342E+01, 2.9089E+00, 2.7965E+02, &
      5.3132530E+00, 3.61285E+01, 2.8843E+00, 2.7594E+02, &
      5.3398192E+00, 3.61228E+01, 2.8600E+00, 2.7228E+02, &
      5.3665183E+00, 3.61171E+01, 2.8358E+00, 2.6867E+02, &
      5.3933509E+00, 3.61113E+01, 2.8119E+00, 2.6511E+02, &
      5.4203177E+00, 3.61055E+01, 2.7882E+00, 2.6160E+02, &
      5.4474193E+00, 3.60997E+01, 2.7646E+00, 2.5813E+02, &
      5.4746564E+00, 3.60938E+01, 2.7412E+00, 2.5470E+02, &
      5.5020297E+00, 3.60878E+01, 2.7181E+00, 2.5132E+02, &
      5.5295398E+00, 3.60819E+01, 2.6951E+00, 2.4799E+02, &
      5.5571875E+00, 3.60758E+01, 2.6723E+00, 2.4471E+02, &
      5.5849734E+00, 3.60698E+01, 2.6498E+00, 2.4147E+02, &
      5.6128983E+00, 3.60637E+01, 2.6275E+00, 2.3827E+02, &
      5.6409628E+00, 3.60575E+01, 2.6053E+00, 2.3512E+02, &
      5.6691676E+00, 3.60514E+01, 2.5834E+00, 2.3201E+02, &
      5.6975135E+00, 3.60452E+01, 2.5616E+00, 2.2894E+02, &
      5.7260010E+00, 3.60389E+01, 2.5401E+00, 2.2592E+02, &
      5.7546310E+00, 3.60327E+01, 2.5187E+00, 2.2293E+02, &
      5.7834042E+00, 3.60264E+01, 2.4976E+00, 2.1999E+02, &
      5.8123212E+00, 3.60201E+01, 2.4766E+00, 2.1709E+02, &
      5.8413828E+00, 3.60137E+01, 2.4558E+00, 2.1422E+02, &
      5.8705897E+00, 3.60073E+01, 2.4352E+00, 2.1140E+02, &
      5.8999427E+00, 3.60009E+01, 2.4148E+00, 2.0861E+02, &
      5.9294424E+00, 3.59945E+01, 2.3946E+00, 2.0586E+02, &
      5.9590896E+00, 3.59880E+01, 2.3745E+00, 2.0315E+02, &
      5.9888850E+00, 3.59816E+01, 2.3547E+00, 2.0048E+02, &
      6.0188295E+00, 3.59751E+01, 2.3350E+00, 1.9784E+02, &
      6.0489236E+00, 3.59685E+01, 2.3155E+00, 1.9524E+02, &
      6.0791682E+00, 3.59620E+01, 2.2961E+00, 1.9267E+02, &
      6.1095641E+00, 3.59554E+01, 2.2770E+00, 1.9014E+02, &
      6.1401119E+00, 3.59488E+01, 2.2580E+00, 1.8765E+02, &
      6.1708125E+00, 3.59422E+01, 2.2391E+00, 1.8518E+02, &
      6.2016665E+00, 3.59356E+01, 2.2205E+00, 1.8275E+02, &
      6.2326749E+00, 3.59290E+01, 2.2020E+00, 1.8036E+02, &
      6.2638382E+00, 3.59223E+01, 2.1837E+00, 1.7799E+02, &
      6.2951574E+00, 3.59157E+01, 2.1655E+00, 1.7566E+02, &
      6.3266332E+00, 3.59090E+01, 2.1475E+00, 1.7336E+02, &
      6.3582664E+00, 3.59023E+01, 2.1297E+00, 1.7109E+02, &
      6.3900577E+00, 3.58956E+01, 2.1120E+00, 1.6885E+02, &
      6.4220080E+00, 3.58889E+01, 2.0945E+00, 1.6665E+02, &
      6.4541180E+00, 3.58821E+01, 2.0771E+00, 1.6447E+02, &
      6.4863886E+00, 3.58754E+01, 2.0599E+00, 1.6232E+02, &
      6.5188206E+00, 3.58686E+01, 2.0428E+00, 1.6020E+02, &
      6.5514147E+00, 3.58618E+01, 2.0259E+00, 1.5811E+02, &
      6.5841717E+00, 3.58551E+01, 2.0092E+00, 1.5605E+02, &
      6.915365E+00, 3.57984E+01, 1.8520E+00, 1.3723E+02, &
      7.392525E+00, 3.57052E+01, 1.6573E+00, 1.1517E+02, &
      7.902609E+00, 3.56076E+01, 1.4780E+00, 9.6361E+01, &
      8.447890E+00, 3.55052E+01, 1.3181E+00, 8.0644E+01, &
      9.030794E+00, 3.54026E+01, 1.1691E+00, 6.7165E+01, &
      9.653919E+00, 3.52769E+01, 1.0267E+00, 5.5422E+01, &
      1.032004E+01, 3.51272E+01, 9.0242E-01, 4.5799E+01, &
      1.103212E+01, 3.49475E+01, 7.9399E-01, 3.7904E+01, &
      1.179334E+01, 3.47178E+01, 6.9926E-01, 3.1419E+01, &
      1.260708E+01, 3.43871E+01, 6.1644E-01, 2.6085E+01, &
      1.347697E+01, 3.37744E+01, 5.4394E-01, 2.1692E+01, &
      1.403909E+01, 3.27699E+01, 5.0406E-01, 1.9391E+01, &
      1.425397E+01, 3.13987E+01, 4.9003E-01, 1.8602E+01, &
      1.431127E+01, 2.97438E+01, 4.8640E-01, 1.8400E+01, &
      1.440688E+01, 3.15725E+01, 3.7394E+00, 1.3166E+02, &
      1.461211E+01, 3.29645E+01, 3.6342E+00, 1.2619E+02, &
      1.540095E+01, 3.44353E+01, 3.2888E+00, 1.0846E+02, &
      1.646362E+01, 3.51565E+01, 2.9440E+00, 9.0931E+01, &
      1.759961E+01, 3.55643E+01, 2.6507E+00, 7.6684E+01, &
      1.881398E+01, 3.58418E+01, 2.3881E+00, 6.4714E+01, &
      2.011215E+01, 3.60333E+01, 2.1362E+00, 5.4240E+01, &
      2.149988E+01, 3.61637E+01, 1.9080E+00, 4.5398E+01, &
      2.298338E+01, 3.62526E+01, 1.7024E+00, 3.7966E+01, &
      2.456923E+01, 3.63124E+01, 1.5182E+00, 3.1744E+01, &
      2.626450E+01, 3.63531E+01, 1.3541E+00, 2.6550E+01, &
      2.807676E+01, 3.63793E+01, 1.2040E+00, 2.2143E+01, &
      3.001405E+01, 3.64257E+01, 1.0650E+00, 1.8382E+01, &
      3.208502E+01, 3.64237E+01, 9.4055E-01, 1.5242E+01, &
      3.429889E+01, 3.64141E+01, 8.3069E-01, 1.2646E+01, &
      3.666551E+01, 3.63994E+01, 7.3368E-01, 1.0497E+01, &
      3.919543E+01, 3.63815E+01, 6.4798E-01, 8.7184E+00, &
      4.189992E+01, 3.63617E+01, 5.7220E-01, 7.2450E+00, &
      4.479101E+01, 3.63407E+01, 5.0509E-01, 6.0234E+00, &
      4.788159E+01, 3.63194E+01, 4.4588E-01, 5.0127E+00, &
      5.118542E+01, 3.62981E+01, 3.9364E-01, 4.1762E+00, &
      5.471721E+01, 3.62774E+01, 3.4754E-01, 3.4839E+00, &
      5.849270E+01, 3.62575E+01, 3.0686E-01, 2.9106E+00, &
      6.252870E+01, 3.62388E+01, 2.7079E-01, 2.4343E+00, &
      6.684318E+01, 3.62206E+01, 2.3777E-01, 2.0309E+00, &
      7.145536E+01, 3.62030E+01, 2.0879E-01, 1.6984E+00, &
      7.638578E+01, 3.61863E+01, 1.8335E-01, 1.4240E+00, &
      8.165640E+01, 3.61708E+01, 1.6085E-01, 1.1966E+00, &
      8.729069E+01, 3.61562E+01, 1.4079E-01, 1.0070E+00, &
      9.331374E+01, 3.61424E+01, 1.2323E-01, 8.5095E-01, &
      9.975239E+01, 3.61295E+01, 1.0787E-01, 7.2226E-01, &
      1.066353E+02, 3.61175E+01, 9.4427E-02, 6.1608E-01, &
      1.139931E+02, 3.61065E+01, 8.2662E-02, 5.2838E-01, &
      1.218587E+02, 3.60964E+01, 7.2366E-02, 4.5584E-01, &
      1.302669E+02, 3.60871E+01, 6.3355E-02, 3.9575E-01, &
      1.392553E+02, 3.60786E+01, 5.5468E-02, 3.4590E-01, &
      1.488640E+02, 3.60709E+01, 4.8565E-02, 3.0445E-01, &
      1.591356E+02, 3.60639E+01, 4.2522E-02, 2.6991E-01, &
      1.701159E+02, 3.60575E+01, 3.7233E-02, 2.4105E-01, &
      1.818539E+02, 3.60517E+01, 3.2603E-02, 2.1686E-01, &
      1.944018E+02, 3.60465E+01, 2.8549E-02, 1.9650E-01, &
      2.078156E+02, 3.60417E+01, 2.5000E-02, 1.7931E-01, &
      2.221548E+02, 3.60374E+01, 2.1893E-02, 1.6472E-01, &
      2.374835E+02, 3.60336E+01, 1.9173E-02, 1.5227E-01, &
      2.538699E+02, 3.60301E+01, 1.6792E-02, 1.4159E-01, &
      2.713869E+02, 3.60269E+01, 1.4706E-02, 1.3236E-01, &
      2.901126E+02, 3.60241E+01, 1.2880E-02, 1.2434E-01, &
      3.101304E+02, 3.60216E+01, 1.1281E-02, 1.1732E-01, &
      3.315294E+02, 3.60193E+01, 9.8811E-03, 1.1111E-01, &
      3.544049E+02, 3.60173E+01, 8.6549E-03, 1.0559E-01, &
      3.788588E+02, 3.60154E+01, 7.5812E-03, 1.0064E-01, &
      4.050001E+02, 3.60138E+01, 6.6408E-03, 9.6165E-02, &
      4.329451E+02, 3.60123E+01, 5.8172E-03, 9.2082E-02]
   case(37)
      frel = -1.5840E-01
      allocate(nist_tab(820))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.06052E+01, 0.0000E+00, 1.5427E-04, &
      1.142761E-02, 1.06015E+01, 0.0000E+00, 2.1002E-04, &
      1.221612E-02, 1.05972E+01, 0.0000E+00, 2.8438E-04, &
      1.305903E-02, 1.05923E+01, 0.0000E+00, 3.8304E-04, &
      1.396010E-02, 1.05867E+01, 0.0000E+00, 5.1319E-04, &
      1.492335E-02, 1.05803E+01, 0.0000E+00, 6.8398E-04, &
      1.595306E-02, 1.05730E+01, 0.0000E+00, 9.0686E-04, &
      1.705382E-02, 1.05646E+01, 0.0000E+00, 1.1962E-03, &
      1.823053E-02, 1.05550E+01, 0.0000E+00, 1.5696E-03, &
      1.948844E-02, 1.05440E+01, 0.0000E+00, 2.0492E-03, &
      2.083314E-02, 1.05314E+01, 0.0000E+00, 2.6617E-03, &
      2.227063E-02, 1.05170E+01, 0.0000E+00, 3.4398E-03, &
      2.380730E-02, 1.05004E+01, 0.0000E+00, 4.4231E-03, &
      2.545001E-02, 1.04814E+01, 0.0000E+00, 5.6593E-03, &
      2.720606E-02, 1.04596E+01, 0.0000E+00, 7.2050E-03, &
      2.908327E-02, 1.04345E+01, 0.0000E+00, 9.1280E-03, &
      3.109002E-02, 1.04057E+01, 0.0000E+00, 1.1508E-02, &
      3.323523E-02, 1.03726E+01, 0.0000E+00, 1.4437E-02, &
      3.552846E-02, 1.03345E+01, 0.0000E+00, 1.8026E-02, &
      3.797993E-02, 1.02905E+01, 0.0000E+00, 2.2398E-02, &
      4.060054E-02, 1.02397E+01, 0.0000E+00, 2.7699E-02, &
      4.340198E-02, 1.01809E+01, 0.0000E+00, 3.4093E-02, &
      4.639671E-02, 1.01128E+01, 0.0000E+00, 4.1766E-02, &
      4.959809E-02, 1.00337E+01, 0.0000E+00, 5.0927E-02, &
      5.302035E-02, 9.94164E+00, 0.0000E+00, 6.1811E-02, &
      5.667876E-02, 9.83403E+00, 0.0000E+00, 7.4677E-02, &
      6.058959E-02, 9.70781E+00, 0.0000E+00, 8.9810E-02, &
      6.477028E-02, 9.55905E+00, 0.0000E+00, 1.0752E-01, &
      6.923942E-02, 9.38267E+00, 0.0000E+00, 1.2814E-01, &
      7.401695E-02, 9.17195E+00, 0.0000E+00, 1.5204E-01, &
      7.912411E-02, 8.91774E+00, 0.0000E+00, 1.7960E-01, &
      8.458368E-02, 8.60695E+00, 0.0000E+00, 2.1121E-01, &
      9.041995E-02, 8.21972E+00, 0.0000E+00, 2.4731E-01, &
      9.665893E-02, 7.72246E+00, 0.0000E+00, 2.8832E-01, &
      1.033284E-01, 7.04466E+00, 0.0000E+00, 3.3469E-01, &
      1.080940E-01, 6.36832E+00, 0.0000E+00, 3.6928E-01, &
      1.095640E-01, 6.05004E+00, 0.0000E+00, 3.8018E-01, &
      1.097485E-01, 5.99430E+00, 0.0000E+00, 3.8156E-01, &
      1.101897E-01, 5.78032E+00, 0.0000E+00, 3.8485E-01, &
      1.104581E-01, 5.75275E+00, 2.5914E-01, 1.1555E+03, &
      1.108515E-01, 5.77037E+00, 2.6970E-01, 1.1983E+03, &
      1.112410E-01, 5.72118E+00, 2.8054E-01, 1.2421E+03, &
      1.116882E-01, 5.58945E+00, 2.9347E-01, 1.2941E+03, &
      1.123590E-01, 5.59619E+00, 4.9693E-01, 2.1779E+03, &
      1.125060E-01, 5.59062E+00, 5.0427E-01, 2.2072E+03, &
      1.140360E-01, 5.45079E+00, 5.8678E-01, 2.5338E+03, &
      1.180797E-01, 4.91265E+00, 8.6744E-01, 3.6173E+03, &
      1.262272E-01, 3.69986E+00, 1.7961E+00, 7.0061E+03, &
      1.349368E-01, 2.56355E+00, 3.2757E+00, 1.1953E+04, &
      1.442475E-01, 1.74687E+00, 5.2741E+00, 1.8003E+04, &
      1.542005E-01, 1.46669E+00, 7.6481E+00, 2.4421E+04, &
      1.648404E-01, 1.84424E+00, 1.0163E+01, 3.0357E+04, &
      1.762144E-01, 2.87535E+00, 1.2561E+01, 3.5097E+04, &
      1.883732E-01, 4.44382E+00, 1.4622E+01, 3.8218E+04, &
      2.013709E-01, 6.35797E+00, 1.6204E+01, 3.9621E+04, &
      2.152655E-01, 8.37821E+00, 1.7252E+01, 3.9460E+04, &
      2.301188E-01, 1.01330E+01, 1.7780E+01, 3.8041E+04, &
      2.337300E-01, 1.03741E+01, 1.7834E+01, 3.7568E+04, &
      2.373075E-01, 1.02407E+01, 1.7863E+01, 3.7064E+04, &
      2.382615E-01, 9.69342E+00, 1.7867E+01, 3.6924E+04, &
      2.396925E-01, 1.04916E+01, 1.9152E+01, 3.9343E+04, &
      2.424520E-01, 1.12475E+01, 1.9176E+01, 3.8943E+04, &
      2.432700E-01, 1.13947E+01, 1.9181E+01, 3.8822E+04, &
      2.459970E-01, 1.17264E+01, 1.9190E+01, 3.8410E+04, &
      2.461630E-01, 1.17335E+01, 1.9190E+01, 3.8385E+04, &
      2.471526E-01, 1.16209E+01, 1.9191E+01, 3.8232E+04, &
      2.486370E-01, 1.21407E+01, 1.9742E+01, 3.9096E+04, &
      2.523480E-01, 1.29482E+01, 1.9744E+01, 3.8525E+04, &
      2.629708E-01, 1.45810E+01, 1.9673E+01, 3.6834E+04, &
      2.811158E-01, 1.67594E+01, 1.9343E+01, 3.3880E+04, &
      3.005128E-01, 1.85648E+01, 1.8795E+01, 3.0794E+04, &
      3.156580E-01, 1.95375E+01, 1.8287E+01, 2.8526E+04, &
      3.204895E-01, 1.95433E+01, 1.8117E+01, 2.7835E+04, &
      3.212482E-01, 1.94315E+01, 1.8090E+01, 2.7728E+04, &
      3.217779E-01, 1.92133E+01, 1.8071E+01, 2.7653E+04, &
      3.237105E-01, 1.97819E+01, 1.8835E+01, 2.8650E+04, &
      3.285420E-01, 2.04946E+01, 1.8663E+01, 2.7970E+04, &
      3.434143E-01, 2.17872E+01, 1.8124E+01, 2.5987E+04, &
      3.671099E-01, 2.32924E+01, 1.7264E+01, 2.3157E+04, &
      3.924405E-01, 2.45443E+01, 1.6369E+01, 2.0539E+04, &
      4.195189E-01, 2.56094E+01, 1.5460E+01, 1.8147E+04, &
      4.484657E-01, 2.65213E+01, 1.4553E+01, 1.5980E+04, &
      4.794098E-01, 2.73013E+01, 1.3654E+01, 1.4026E+04, &
      5.124891E-01, 2.79617E+01, 1.2771E+01, 1.2273E+04, &
      5.478508E-01, 2.85146E+01, 1.1912E+01, 1.0709E+04, &
      5.856525E-01, 2.89716E+01, 1.1084E+01, 9.3222E+03, &
      6.260625E-01, 2.93430E+01, 1.0291E+01, 8.0974E+03, &
      6.692609E-01, 2.96376E+01, 9.5364E+00, 7.0199E+03, &
      7.154399E-01, 2.98635E+01, 8.8212E+00, 6.0751E+03, &
      7.648052E-01, 3.00328E+01, 8.1467E+00, 5.2491E+03, &
      8.175768E-01, 3.01412E+01, 7.5128E+00, 4.5291E+03, &
      8.739896E-01, 3.01985E+01, 6.9191E+00, 3.9027E+03, &
      9.342948E-01, 3.02095E+01, 6.3649E+00, 3.3592E+03, &
      9.987612E-01, 3.01813E+01, 5.8487E+00, 2.8884E+03, &
      1.067676E+00, 3.00924E+01, 5.2591E+00, 2.4305E+03, &
      1.141345E+00, 2.98858E+01, 4.7337E+00, 2.0474E+03, &
      1.220098E+00, 2.95799E+01, 4.2670E+00, 1.7273E+03, &
      1.304285E+00, 2.91656E+01, 3.8509E+00, 1.4592E+03, &
      1.394281E+00, 2.86048E+01, 3.4763E+00, 1.2331E+03, &
      1.490486E+00, 2.78159E+01, 3.1425E+00, 1.0436E+03, &
      1.593329E+00, 2.66003E+01, 2.8447E+00, 8.8455E+02, &
      1.703269E+00, 2.42705E+01, 2.5786E+00, 7.5089E+02, &
      1.768312E+00, 2.09998E+01, 2.4416E+00, 6.8533E+02, &
      1.795378E+00, 1.67689E+01, 2.3885E+00, 6.6050E+02, &
      1.802596E+00, 1.20146E+01, 2.3747E+00, 6.5411E+02, &
      1.813422E+00, 1.65024E+01, 1.1451E+01, 3.1146E+03, &
      1.820795E+00, 1.81208E+01, 1.1348E+01, 3.0742E+03, &
      1.826622E+00, 1.88680E+01, 1.1268E+01, 3.0428E+03, &
      1.840488E+00, 1.97523E+01, 1.1081E+01, 2.9697E+03, &
      1.854580E+00, 1.95105E+01, 1.0895E+01, 2.8978E+03, &
      1.862036E+00, 1.76613E+01, 1.0798E+01, 2.8607E+03, &
      1.873219E+00, 2.05312E+01, 1.5142E+01, 3.9853E+03, &
      1.901178E+00, 2.36161E+01, 1.4663E+01, 3.8028E+03, &
      1.946430E+00, 2.58435E+01, 1.4007E+01, 3.5486E+03, &
      2.023798E+00, 2.75287E+01, 1.3090E+01, 3.1900E+03, &
      2.054774E+00, 2.73506E+01, 1.2769E+01, 3.0652E+03, &
      2.063035E+00, 2.65310E+01, 1.2687E+01, 3.0333E+03, &
      2.075425E+00, 2.77221E+01, 1.4480E+01, 3.4404E+03, &
      2.080733E+00, 2.80720E+01, 1.4424E+01, 3.4184E+03, &
      2.106402E+00, 2.90967E+01, 1.4161E+01, 3.3155E+03, &
      2.224304E+00, 3.14076E+01, 1.3092E+01, 2.9032E+03, &
      2.377781E+00, 3.31215E+01, 1.1929E+01, 2.4752E+03, &
      2.541848E+00, 3.43107E+01, 1.0816E+01, 2.1001E+03, &
      2.717235E+00, 3.51644E+01, 9.7911E+00, 1.7791E+03, &
      2.904724E+00, 3.57995E+01, 8.8531E+00, 1.5055E+03, &
      3.105150E+00, 3.62964E+01, 7.9746E+00, 1.2693E+03, &
      3.319406E+00, 3.66150E+01, 7.1601E+00, 1.0666E+03, &
      3.548445E+00, 3.68409E+01, 6.4342E+00, 8.9723E+02, &
      3.793288E+00, 3.71467E+01, 5.7579E+00, 7.5166E+02, &
      4.055024E+00, 3.72307E+01, 5.1392E+00, 6.2815E+02, &
      4.334821E+00, 3.73083E+01, 4.5792E+00, 5.2410E+02, &
      4.633924E+00, 3.73012E+01, 4.0776E+00, 4.3707E+02, &
      4.953664E+00, 3.72647E+01, 3.6308E+00, 3.6453E+02, &
      5.295467E+00, 3.72079E+01, 3.2358E+00, 3.0435E+02, &
      5.660855E+00, 3.71377E+01, 2.8864E+00, 2.5437E+02, &
      6.051453E+00, 3.70586E+01, 2.5770E+00, 2.1283E+02, &
      6.469004E+00, 3.69737E+01, 2.3017E+00, 1.7818E+02, &
      6.915365E+00, 3.68848E+01, 2.0576E+00, 1.4934E+02, &
      7.392525E+00, 3.67938E+01, 1.8411E+00, 1.2530E+02, &
      7.902609E+00, 3.67023E+01, 1.6480E+00, 1.0520E+02, &
      8.447890E+00, 3.66083E+01, 1.4703E+00, 8.8068E+01, &
      9.030794E+00, 3.65147E+01, 1.3082E+00, 7.3552E+01, &
      9.653919E+00, 3.64009E+01, 1.1491E+00, 6.0696E+01, &
      1.032004E+01, 3.62685E+01, 1.0103E+00, 5.0159E+01, &
      1.103212E+01, 3.61154E+01, 8.8918E-01, 4.1513E+01, &
      1.179334E+01, 3.59319E+01, 7.8333E-01, 3.4410E+01, &
      1.260708E+01, 3.56954E+01, 6.9075E-01, 2.8566E+01, &
      1.347697E+01, 3.53489E+01, 6.0970E-01, 2.3753E+01, &
      1.440688E+01, 3.46753E+01, 5.3867E-01, 1.9783E+01, &
      1.489571E+01, 3.37901E+01, 5.0650E-01, 1.8066E+01, &
      1.512370E+01, 3.24280E+01, 4.9254E-01, 1.7337E+01, &
      1.518450E+01, 3.07845E+01, 4.8892E-01, 1.7149E+01, &
      1.527570E+01, 3.24638E+01, 3.7245E+00, 1.2133E+02, &
      1.540095E+01, 3.35245E+01, 3.6643E+00, 1.1842E+02, &
      1.550369E+01, 3.39820E+01, 3.6159E+00, 1.1610E+02, &
      1.646362E+01, 3.55543E+01, 3.2333E+00, 9.7877E+01, &
      1.759961E+01, 3.62184E+01, 2.8978E+00, 8.2162E+01, &
      1.881398E+01, 3.66050E+01, 2.6097E+00, 6.9307E+01, &
      2.011215E+01, 3.68702E+01, 2.3484E+00, 5.8428E+01, &
      2.149988E+01, 3.70520E+01, 2.1007E+00, 4.8973E+01, &
      2.298338E+01, 3.71762E+01, 1.8765E+00, 4.1000E+01, &
      2.456923E+01, 3.72613E+01, 1.6749E+00, 3.4305E+01, &
      2.626450E+01, 3.73198E+01, 1.4947E+00, 2.8705E+01, &
      2.807676E+01, 3.73592E+01, 1.3296E+00, 2.3949E+01, &
      3.001405E+01, 3.73806E+01, 1.1815E+00, 1.9966E+01, &
      3.208502E+01, 3.74248E+01, 1.0446E+00, 1.6570E+01, &
      3.429889E+01, 3.74215E+01, 9.2254E-01, 1.3744E+01, &
      3.666551E+01, 3.74110E+01, 8.1482E-01, 1.1406E+01, &
      3.919543E+01, 3.73958E+01, 7.1970E-01, 9.4711E+00, &
      4.189992E+01, 3.73776E+01, 6.3565E-01, 7.8694E+00, &
      4.479101E+01, 3.73576E+01, 5.6136E-01, 6.5426E+00, &
      4.788159E+01, 3.73367E+01, 4.9552E-01, 5.4419E+00, &
      5.118542E+01, 3.73153E+01, 4.3743E-01, 4.5311E+00, &
      5.471721E+01, 3.72942E+01, 3.8618E-01, 3.7773E+00, &
      5.849270E+01, 3.72736E+01, 3.4095E-01, 3.1532E+00, &
      6.252870E+01, 3.72538E+01, 3.0104E-01, 2.6364E+00, &
      6.684318E+01, 3.72352E+01, 2.6552E-01, 2.2061E+00, &
      7.145536E+01, 3.72171E+01, 2.3329E-01, 1.8434E+00, &
      7.638578E+01, 3.71997E+01, 2.0497E-01, 1.5443E+00, &
      8.165640E+01, 3.71841E+01, 1.7991E-01, 1.2961E+00, &
      8.729069E+01, 3.71686E+01, 1.5755E-01, 1.0892E+00, &
      9.331374E+01, 3.71539E+01, 1.3798E-01, 9.1883E-01, &
      9.975239E+01, 3.71401E+01, 1.2084E-01, 7.7835E-01, &
      1.066353E+02, 3.71273E+01, 1.0583E-01, 6.6242E-01, &
      1.139931E+02, 3.71155E+01, 9.2695E-02, 5.6666E-01, &
      1.218587E+02, 3.71046E+01, 8.1191E-02, 4.8747E-01, &
      1.302669E+02, 3.70947E+01, 7.1118E-02, 4.2189E-01, &
      1.392553E+02, 3.70855E+01, 6.2296E-02, 3.6750E-01, &
      1.488640E+02, 3.70772E+01, 5.4571E-02, 3.2229E-01, &
      1.591356E+02, 3.70696E+01, 4.7806E-02, 2.8463E-01, &
      1.701159E+02, 3.70627E+01, 4.1881E-02, 2.5318E-01, &
      1.818539E+02, 3.70565E+01, 3.6691E-02, 2.2684E-01, &
      1.944018E+02, 3.70508E+01, 3.2146E-02, 2.0469E-01, &
      2.078156E+02, 3.70456E+01, 2.8164E-02, 1.8601E-01, &
      2.221548E+02, 3.70410E+01, 2.4677E-02, 1.7016E-01, &
      2.374835E+02, 3.70368E+01, 2.1622E-02, 1.5666E-01, &
      2.538699E+02, 3.70330E+01, 1.8946E-02, 1.4508E-01, &
      2.713869E+02, 3.70296E+01, 1.6601E-02, 1.3511E-01, &
      2.901126E+02, 3.70265E+01, 1.4547E-02, 1.2644E-01, &
      3.101304E+02, 3.70237E+01, 1.2748E-02, 1.1885E-01, &
      3.315294E+02, 3.70212E+01, 1.1171E-02, 1.1216E-01, &
      3.544049E+02, 3.70190E+01, 9.7899E-03, 1.0621E-01, &
      3.788588E+02, 3.70170E+01, 8.5796E-03, 1.0087E-01, &
      4.050001E+02, 3.70152E+01, 7.5192E-03, 9.6052E-02, &
      4.329451E+02, 3.70135E+01, 6.5900E-03, 9.1657E-02]
   case(38)
      frel = -1.6920E-01
      allocate(nist_tab(856))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 6.01475E+00, 0.0000E+00, 1.2263E-04, &
      1.142761E-02, 5.84877E+00, 0.0000E+00, 1.6787E-04, &
      1.221612E-02, 5.64098E+00, 0.0000E+00, 2.2856E-04, &
      1.305903E-02, 5.37550E+00, 0.0000E+00, 3.0953E-04, &
      1.396010E-02, 5.02694E+00, 0.0000E+00, 4.1694E-04, &
      1.492335E-02, 4.55166E+00, 0.0000E+00, 5.5864E-04, &
      1.595306E-02, 3.86641E+00, 0.0000E+00, 7.4456E-04, &
      1.705382E-02, 2.78500E+00, 0.0000E+00, 9.8715E-04, &
      1.823053E-02, 7.54046E-01, 0.0000E+00, 1.3020E-03, &
      1.948844E-02, -5.82671E+00, 0.0000E+00, 1.7083E-03, &
      1.950200E-02, -6.00105E+00, 0.0000E+00, 1.7131E-03, &
      1.980050E-02, -1.36341E+01, 0.0000E+00, 1.8210E-03, &
      1.988010E-02, -2.30666E+01, 0.0000E+00, 1.8505E-03, &
      1.999950E-02, -1.28619E+01, 1.8720E+01, 4.4953E+05, &
      2.029800E-02, -3.82214E+00, 1.7971E+01, 4.2520E+05, &
      2.083314E-02, 2.01263E+00, 1.6728E+01, 3.8563E+05, &
      2.227063E-02, 8.40831E+00, 1.3904E+01, 2.9984E+05, &
      2.380730E-02, 1.14841E+01, 1.1221E+01, 2.2637E+05, &
      2.545001E-02, 1.29978E+01, 8.7811E+00, 1.6571E+05, &
      2.720606E-02, 1.36030E+01, 6.7384E+00, 1.1895E+05, &
      2.908327E-02, 1.36861E+01, 5.1194E+00, 8.4538E+04, &
      3.109002E-02, 1.34888E+01, 3.8813E+00, 5.9956E+04, &
      3.323523E-02, 1.31574E+01, 2.9556E+00, 4.2709E+04, &
      3.552846E-02, 1.27730E+01, 2.2723E+00, 3.0716E+04, &
      3.694600E-02, 1.25346E+01, 1.9596E+00, 2.5473E+04, &
      3.751150E-02, 1.24291E+01, 1.8527E+00, 2.3721E+04, &
      3.766230E-02, 1.23844E+01, 1.8257E+00, 2.3281E+04, &
      3.788850E-02, 1.23712E+01, 1.8292E+00, 2.3186E+04, &
      3.797993E-02, 1.23628E+01, 1.8141E+00, 2.2940E+04, &
      3.845400E-02, 1.23062E+01, 1.7391E+00, 2.1720E+04, &
      4.060054E-02, 1.20320E+01, 1.4601E+00, 1.7271E+04, &
      4.340198E-02, 1.17120E+01, 1.2070E+00, 1.3356E+04, &
      4.639671E-02, 1.14213E+01, 1.0266E+00, 1.0626E+04, &
      4.959809E-02, 1.11581E+01, 8.9863E-01, 8.7014E+03, &
      5.302035E-02, 1.09182E+01, 8.0874E-01, 7.3256E+03, &
      5.667876E-02, 1.06961E+01, 7.4641E-01, 6.3247E+03, &
      6.058959E-02, 1.04934E+01, 7.0405E-01, 5.5807E+03, &
      6.477028E-02, 1.02891E+01, 6.7615E-01, 5.0136E+03, &
      6.923942E-02, 1.00843E+01, 6.5872E-01, 4.5691E+03, &
      7.401695E-02, 9.87222E+00, 6.4890E-01, 4.2105E+03, &
      7.912411E-02, 9.64530E+00, 6.4464E-01, 3.9130E+03, &
      8.458368E-02, 9.39477E+00, 6.4464E-01, 3.6604E+03, &
      9.041995E-02, 9.11006E+00, 6.4796E-01, 3.4418E+03, &
      9.665893E-02, 8.77712E+00, 6.5373E-01, 3.2484E+03, &
      1.033284E-01, 8.37592E+00, 6.6129E-01, 3.0739E+03, &
      1.104581E-01, 7.87523E+00, 6.7015E-01, 2.9141E+03, &
      1.180797E-01, 7.21893E+00, 6.7994E-01, 2.7659E+03, &
      1.262272E-01, 6.26992E+00, 6.9034E-01, 2.6270E+03, &
      1.304380E-01, 5.54051E+00, 6.9558E-01, 2.5616E+03, &
      1.323000E-01, 5.01232E+00, 6.9786E-01, 2.5338E+03, &
      1.324345E-01, 4.95426E+00, 6.9802E-01, 2.5318E+03, &
      1.329669E-01, 4.56862E+00, 6.9867E-01, 2.5240E+03, &
      1.337655E-01, 4.63112E+00, 1.2472E+00, 4.4782E+03, &
      1.343250E-01, 4.56576E+00, 1.2706E+00, 4.5434E+03, &
      1.349368E-01, 4.25375E+00, 1.2972E+00, 4.6174E+03, &
      1.356750E-01, 4.44240E+00, 1.7037E+00, 6.0313E+03, &
      1.357620E-01, 4.44650E+00, 1.7101E+00, 6.0500E+03, &
      1.377000E-01, 4.35558E+00, 1.8615E+00, 6.4930E+03, &
      1.442475E-01, 3.59201E+00, 2.5308E+00, 8.4265E+03, &
      1.542005E-01, 2.40039E+00, 4.0874E+00, 1.2731E+04, &
      1.648404E-01, 1.56073E+00, 6.2297E+00, 1.8151E+04, &
      1.762144E-01, 1.31547E+00, 8.7792E+00, 2.3928E+04, &
      1.883732E-01, 1.79924E+00, 1.1466E+01, 2.9233E+04, &
      2.013709E-01, 2.99766E+00, 1.3995E+01, 3.3378E+04, &
      2.152655E-01, 4.76573E+00, 1.6122E+01, 3.5970E+04, &
      2.301188E-01, 6.87259E+00, 1.7700E+01, 3.6941E+04, &
      2.459970E-01, 9.02751E+00, 1.8678E+01, 3.6466E+04, &
      2.629708E-01, 1.06739E+01, 1.9087E+01, 3.4860E+04, &
      2.637180E-01, 1.06994E+01, 1.9093E+01, 3.4772E+04, &
      2.677545E-01, 1.04677E+01, 1.9110E+01, 3.4278E+04, &
      2.688309E-01, 9.77365E+00, 1.9110E+01, 3.4142E+04, &
      2.704455E-01, 1.07411E+01, 2.0693E+01, 3.6748E+04, &
      2.742040E-01, 1.17583E+01, 2.0703E+01, 3.6263E+04, &
      2.744820E-01, 1.18076E+01, 2.0703E+01, 3.6226E+04, &
      2.784010E-01, 1.22586E+01, 2.0694E+01, 3.5701E+04, &
      2.795202E-01, 1.20944E+01, 2.0688E+01, 3.5548E+04, &
      2.811158E-01, 1.26846E+01, 2.1367E+01, 3.6505E+04, &
      2.811990E-01, 1.27109E+01, 2.1367E+01, 3.6494E+04, &
      2.853960E-01, 1.36391E+01, 2.1343E+01, 3.5917E+04, &
      3.005128E-01, 1.58617E+01, 2.1135E+01, 3.3778E+04, &
      3.212482E-01, 1.81526E+01, 2.0620E+01, 3.0829E+04, &
      3.434143E-01, 1.99592E+01, 1.9892E+01, 2.7821E+04, &
      3.503500E-01, 2.03353E+01, 1.9644E+01, 2.6930E+04, &
      3.557125E-01, 2.03684E+01, 1.9448E+01, 2.6259E+04, &
      3.571425E-01, 2.00386E+01, 1.9395E+01, 2.6083E+04, &
      3.592875E-01, 2.06284E+01, 2.0165E+01, 2.6957E+04, &
      3.646500E-01, 2.13786E+01, 1.9968E+01, 2.6301E+04, &
      3.671099E-01, 2.16251E+01, 1.9876E+01, 2.6006E+04, &
      3.924405E-01, 2.34986E+01, 1.8928E+01, 2.3166E+04, &
      4.195189E-01, 2.49534E+01, 1.7932E+01, 2.0531E+04, &
      4.484657E-01, 2.61751E+01, 1.6918E+01, 1.8120E+04, &
      4.794098E-01, 2.72060E+01, 1.5907E+01, 1.5938E+04, &
      5.124891E-01, 2.80783E+01, 1.4913E+01, 1.3980E+04, &
      5.478508E-01, 2.88164E+01, 1.3946E+01, 1.2230E+04, &
      5.856525E-01, 2.94368E+01, 1.3007E+01, 1.0671E+04, &
      6.260625E-01, 2.99511E+01, 1.2102E+01, 9.2879E+03, &
      6.692609E-01, 3.03708E+01, 1.1236E+01, 8.0674E+03, &
      7.154399E-01, 3.07066E+01, 1.0413E+01, 6.9942E+03, &
      7.648052E-01, 3.09681E+01, 9.6331E+00, 6.0537E+03, &
      8.175768E-01, 3.11690E+01, 8.8989E+00, 5.2322E+03, &
      8.739896E-01, 3.13124E+01, 8.2102E+00, 4.5164E+03, &
      9.342948E-01, 3.14130E+01, 7.5663E+00, 3.8944E+03, &
      9.987612E-01, 3.15546E+01, 6.9663E+00, 3.3550E+03, &
      1.067676E+00, 3.15036E+01, 6.3301E+00, 2.8527E+03, &
      1.141345E+00, 3.13758E+01, 5.7183E+00, 2.4116E+03, &
      1.220098E+00, 3.11577E+01, 5.1566E+00, 2.0352E+03, &
      1.304285E+00, 3.08513E+01, 4.6569E+00, 1.7202E+03, &
      1.394281E+00, 3.04446E+01, 4.2117E+00, 1.4562E+03, &
      1.490486E+00, 2.99032E+01, 3.8117E+00, 1.2338E+03, &
      1.593329E+00, 2.91497E+01, 3.4528E+00, 1.0463E+03, &
      1.703269E+00, 2.80062E+01, 3.1320E+00, 8.8867E+02, &
      1.820795E+00, 2.58874E+01, 2.8448E+00, 7.5591E+02, &
      1.900808E+00, 2.24478E+01, 2.6757E+00, 6.8157E+02, &
      1.929902E+00, 1.83841E+01, 2.6188E+00, 6.5720E+02, &
      1.937660E+00, 1.38390E+01, 2.6040E+00, 6.5092E+02, &
      1.946430E+00, 1.71473E+01, 1.1304E+01, 2.7945E+03, &
      1.949298E+00, 1.80907E+01, 1.1275E+01, 2.7833E+03, &
      1.966664E+00, 2.06160E+01, 1.1103E+01, 2.7168E+03, &
      1.978392E+00, 2.12315E+01, 1.0989E+01, 2.6732E+03, &
      1.996766E+00, 2.10047E+01, 1.0815E+01, 2.6067E+03, &
      2.004793E+00, 1.92132E+01, 1.0740E+01, 2.5783E+03, &
      2.016834E+00, 2.19144E+01, 1.4918E+01, 3.5578E+03, &
      2.046936E+00, 2.48433E+01, 1.4547E+01, 3.4187E+03, &
      2.080733E+00, 2.64766E+01, 1.4159E+01, 3.2737E+03, &
      2.171974E+00, 2.85735E+01, 1.3212E+01, 2.9267E+03, &
      2.205219E+00, 2.84259E+01, 1.2897E+01, 2.8141E+03, &
      2.214084E+00, 2.76147E+01, 1.2815E+01, 2.7851E+03, &
      2.224304E+00, 2.85604E+01, 1.4635E+01, 3.1653E+03, &
      2.227382E+00, 2.88147E+01, 1.4605E+01, 3.1544E+03, &
      2.260626E+00, 3.02095E+01, 1.4284E+01, 3.0400E+03, &
      2.377781E+00, 3.24386E+01, 1.3260E+01, 2.6834E+03, &
      2.541848E+00, 3.41917E+01, 1.2045E+01, 2.2810E+03, &
      2.717235E+00, 3.53853E+01, 1.0915E+01, 1.9342E+03, &
      2.904724E+00, 3.62474E+01, 9.8856E+00, 1.6395E+03, &
      3.105150E+00, 3.68987E+01, 8.9071E+00, 1.3824E+03, &
      3.319406E+00, 3.73400E+01, 7.9961E+00, 1.1616E+03, &
      3.548445E+00, 3.76486E+01, 7.1843E+00, 9.7691E+02, &
      3.793288E+00, 3.78686E+01, 6.4604E+00, 8.2235E+02, &
      4.055024E+00, 3.81720E+01, 5.7885E+00, 6.8982E+02, &
      4.334821E+00, 3.82552E+01, 5.1674E+00, 5.7659E+02, &
      4.633924E+00, 3.83309E+01, 4.6063E+00, 4.8131E+02, &
      4.953664E+00, 3.83224E+01, 4.1044E+00, 4.0167E+02, &
      5.295467E+00, 3.82855E+01, 3.6580E+00, 3.3533E+02, &
      5.660855E+00, 3.82285E+01, 3.2608E+00, 2.8005E+02, &
      6.051453E+00, 3.81577E+01, 2.9094E+00, 2.3415E+02, &
      6.469004E+00, 3.80780E+01, 2.5983E+00, 1.9598E+02, &
      6.915365E+00, 3.79926E+01, 2.3221E+00, 1.6418E+02, &
      7.392525E+00, 3.79036E+01, 2.0766E+00, 1.3766E+02, &
      7.902609E+00, 3.78125E+01, 1.8587E+00, 1.1556E+02, &
      8.447890E+00, 3.77216E+01, 1.6649E+00, 9.7092E+01, &
      9.030794E+00, 3.76321E+01, 1.4831E+00, 8.1164E+01, &
      9.653919E+00, 3.75290E+01, 1.3088E+00, 6.7259E+01, &
      1.032004E+01, 3.74095E+01, 1.1512E+00, 5.5584E+01, &
      1.103212E+01, 3.72732E+01, 1.0135E+00, 4.6001E+01, &
      1.179334E+01, 3.71160E+01, 8.9312E-01, 3.8125E+01, &
      1.260708E+01, 3.69268E+01, 7.8780E-01, 3.1646E+01, &
      1.347697E+01, 3.66805E+01, 6.9556E-01, 2.6308E+01, &
      1.440688E+01, 3.63115E+01, 6.1469E-01, 2.1904E+01, &
      1.540095E+01, 3.55442E+01, 5.4374E-01, 1.8267E+01, &
      1.578251E+01, 3.48227E+01, 5.1994E-01, 1.7097E+01, &
      1.602408E+01, 3.34696E+01, 5.0573E-01, 1.6410E+01, &
      1.608850E+01, 3.18377E+01, 5.0204E-01, 1.6235E+01, &
      1.618512E+01, 3.35041E+01, 3.7145E+00, 1.1146E+02, &
      1.642669E+01, 3.50109E+01, 3.6075E+00, 1.0669E+02, &
      1.646362E+01, 3.51310E+01, 3.5915E+00, 1.0599E+02, &
      1.759961E+01, 3.66899E+01, 3.1809E+00, 8.7924E+01, &
      1.881398E+01, 3.72957E+01, 2.8540E+00, 7.3894E+01, &
      2.011215E+01, 3.76587E+01, 2.5703E+00, 6.2338E+01, &
      2.149988E+01, 3.79092E+01, 2.3100E+00, 5.2491E+01, &
      2.298338E+01, 3.80799E+01, 2.0662E+00, 4.3997E+01, &
      2.456923E+01, 3.81970E+01, 1.8459E+00, 3.6842E+01, &
      2.626450E+01, 3.82782E+01, 1.6484E+00, 3.0843E+01, &
      2.807676E+01, 3.83342E+01, 1.4669E+00, 2.5739E+01, &
      3.001405E+01, 3.83668E+01, 1.3039E+00, 2.1463E+01, &
      3.208502E+01, 3.83854E+01, 1.1592E+00, 1.7904E+01, &
      3.429889E+01, 3.84267E+01, 1.0241E+00, 1.4852E+01, &
      3.666551E+01, 3.84218E+01, 9.0457E-01, 1.2322E+01, &
      3.919543E+01, 3.84101E+01, 7.9901E-01, 1.0230E+01, &
      4.189992E+01, 3.83941E+01, 7.0582E-01, 8.4978E+00, &
      4.479101E+01, 3.83754E+01, 6.2346E-01, 7.0638E+00, &
      4.788159E+01, 3.83551E+01, 5.5068E-01, 5.8759E+00, &
      5.118542E+01, 3.83340E+01, 4.8613E-01, 4.8900E+00, &
      5.471721E+01, 3.83126E+01, 4.2918E-01, 4.0742E+00, &
      5.849270E+01, 3.82914E+01, 3.7893E-01, 3.3988E+00, &
      6.252870E+01, 3.82709E+01, 3.3458E-01, 2.8396E+00, &
      6.684318E+01, 3.82512E+01, 2.9544E-01, 2.3764E+00, &
      7.145536E+01, 3.82327E+01, 2.6046E-01, 1.9896E+00, &
      7.638578E+01, 3.82149E+01, 2.2893E-01, 1.6651E+00, &
      8.165640E+01, 3.81983E+01, 2.0103E-01, 1.3961E+00, &
      8.729069E+01, 3.81818E+01, 1.7615E-01, 1.1720E+00, &
      9.331374E+01, 3.81662E+01, 1.5436E-01, 9.8731E-01, &
      9.975239E+01, 3.81515E+01, 1.3527E-01, 8.3506E-01, &
      1.066353E+02, 3.81379E+01, 1.1855E-01, 7.0942E-01, &
      1.139931E+02, 3.81252E+01, 1.0389E-01, 6.0566E-01, &
      1.218587E+02, 3.81136E+01, 9.1057E-02, 5.1986E-01, &
      1.302669E+02, 3.81029E+01, 7.9809E-02, 4.4882E-01, &
      1.392553E+02, 3.80931E+01, 6.9953E-02, 3.8992E-01, &
      1.488640E+02, 3.80841E+01, 6.1317E-02, 3.4100E-01, &
      1.591356E+02, 3.80759E+01, 5.3749E-02, 3.0028E-01, &
      1.701159E+02, 3.80684E+01, 4.7116E-02, 2.6630E-01, &
      1.818539E+02, 3.80617E+01, 4.1304E-02, 2.3786E-01, &
      1.944018E+02, 3.80555E+01, 3.6210E-02, 2.1398E-01, &
      2.078156E+02, 3.80499E+01, 3.1745E-02, 1.9387E-01, &
      2.221548E+02, 3.80449E+01, 2.7832E-02, 1.7684E-01, &
      2.374835E+02, 3.80403E+01, 2.4401E-02, 1.6236E-01, &
      2.538699E+02, 3.80362E+01, 2.1395E-02, 1.4996E-01, &
      2.713869E+02, 3.80324E+01, 1.8759E-02, 1.3930E-01, &
      2.901126E+02, 3.80291E+01, 1.6449E-02, 1.3005E-01, &
      3.101304E+02, 3.80261E+01, 1.4423E-02, 1.2198E-01, &
      3.315294E+02, 3.80233E+01, 1.2647E-02, 1.1487E-01, &
      3.544049E+02, 3.80209E+01, 1.1091E-02, 1.0857E-01, &
      3.788588E+02, 3.80187E+01, 9.7258E-03, 1.0293E-01, &
      4.050001E+02, 3.80167E+01, 8.5292E-03, 9.7840E-02, &
      4.329451E+02, 3.80149E+01, 7.4800E-03, 9.3208E-02]
   case(39)
      frel = -1.8000E-01
      allocate(nist_tab(860))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 9.25807E+00, 0.0000E+00, 1.3109E-04, &
      1.142761E-02, 9.21353E+00, 0.0000E+00, 1.7925E-04, &
      1.221612E-02, 9.16034E+00, 0.0000E+00, 2.4378E-04, &
      1.305903E-02, 9.09630E+00, 0.0000E+00, 3.2978E-04, &
      1.396010E-02, 9.01842E+00, 0.0000E+00, 4.4374E-04, &
      1.492335E-02, 8.92250E+00, 0.0000E+00, 5.9393E-04, &
      1.595306E-02, 8.80245E+00, 0.0000E+00, 7.9078E-04, &
      1.705382E-02, 8.64897E+00, 0.0000E+00, 1.0474E-03, &
      1.823053E-02, 8.44706E+00, 0.0000E+00, 1.3800E-03, &
      1.948844E-02, 8.17048E+00, 0.0000E+00, 1.8090E-03, &
      2.083314E-02, 7.76777E+00, 0.0000E+00, 2.3591E-03, &
      2.227063E-02, 7.11823E+00, 0.0000E+00, 3.0608E-03, &
      2.380730E-02, 5.82478E+00, 0.0000E+00, 3.9512E-03, &
      2.508800E-02, 2.66593E+00, 0.0000E+00, 4.8114E-03, &
      2.545001E-02, -8.83560E-01, 0.0000E+00, 5.0749E-03, &
      2.547200E-02, -1.36180E+00, 0.0000E+00, 5.0912E-03, &
      2.557440E-02, -6.35333E+00, 0.0000E+00, 5.1675E-03, &
      2.572800E-02, -9.23289E-01, 9.9117E+00, 1.8234E+05, &
      2.611200E-02, 3.88163E+00, 9.4699E+00, 1.7165E+05, &
      2.720606E-02, 8.00855E+00, 8.3480E+00, 1.4523E+05, &
      2.908327E-02, 1.06268E+01, 6.7893E+00, 1.1049E+05, &
      3.109002E-02, 1.18406E+01, 5.4440E+00, 8.2879E+04, &
      3.323523E-02, 1.24062E+01, 4.3240E+00, 6.1580E+04, &
      3.552846E-02, 1.26179E+01, 3.4253E+00, 4.5632E+04, &
      3.797993E-02, 1.26306E+01, 2.7216E+00, 3.3917E+04, &
      4.060054E-02, 1.25313E+01, 2.1790E+00, 2.5402E+04, &
      4.340198E-02, 1.23632E+01, 1.7643E+00, 1.9240E+04, &
      4.449200E-02, 1.22795E+01, 1.6368E+00, 1.7412E+04, &
      4.517300E-02, 1.22020E+01, 1.5648E+00, 1.6396E+04, &
      4.535460E-02, 1.21481E+01, 1.5466E+00, 1.6140E+04, &
      4.562700E-02, 1.21734E+01, 1.6040E+00, 1.6639E+04, &
      4.630800E-02, 1.21689E+01, 1.5441E+00, 1.5782E+04, &
      4.639671E-02, 1.21661E+01, 1.5367E+00, 1.5676E+04, &
      4.959809E-02, 1.20224E+01, 1.3140E+00, 1.2540E+04, &
      5.302035E-02, 1.18616E+01, 1.1510E+00, 1.0275E+04, &
      5.667876E-02, 1.17016E+01, 1.0318E+00, 8.6163E+03, &
      6.058959E-02, 1.15446E+01, 9.4549E-01, 7.3860E+03, &
      6.477028E-02, 1.13895E+01, 8.8383E-01, 6.4588E+03, &
      6.923942E-02, 1.12337E+01, 8.4065E-01, 5.7467E+03, &
      7.401695E-02, 1.10734E+01, 8.1131E-01, 5.1882E+03, &
      7.912411E-02, 1.09135E+01, 7.9235E-01, 4.7400E+03, &
      8.458368E-02, 1.07301E+01, 7.8119E-01, 4.3716E+03, &
      9.041995E-02, 1.05260E+01, 7.7590E-01, 4.0617E+03, &
      9.665893E-02, 1.02930E+01, 7.7504E-01, 3.7954E+03, &
      1.033284E-01, 1.00210E+01, 7.7753E-01, 3.5619E+03, &
      1.104581E-01, 9.69578E+00, 7.8263E-01, 3.3540E+03, &
      1.180797E-01, 9.29754E+00, 7.8967E-01, 3.1657E+03, &
      1.262272E-01, 8.79516E+00, 7.9802E-01, 2.9928E+03, &
      1.349368E-01, 8.13454E+00, 8.0718E-01, 2.8319E+03, &
      1.442475E-01, 7.20073E+00, 8.1672E-01, 2.6805E+03, &
      1.542005E-01, 5.55290E+00, 8.2624E-01, 2.5368E+03, &
      1.542520E-01, 5.53953E+00, 8.2629E-01, 2.5361E+03, &
      1.564080E-01, 4.79154E+00, 8.2823E-01, 2.5072E+03, &
      1.566130E-01, 4.67786E+00, 8.2842E-01, 2.5044E+03, &
      1.572426E-01, 4.05410E+00, 8.2898E-01, 2.4961E+03, &
      1.581870E-01, 4.23245E+00, 1.7739E+00, 5.3086E+03, &
      1.588020E-01, 4.17477E+00, 1.8063E+00, 5.3845E+03, &
      1.594404E-01, 3.84776E+00, 1.8409E+00, 5.4655E+03, &
      1.603980E-01, 4.09204E+00, 2.5368E+00, 7.4866E+03, &
      1.605480E-01, 4.11345E+00, 2.5506E+00, 7.5201E+03, &
      1.627920E-01, 4.13332E+00, 2.7681E+00, 8.0490E+03, &
      1.648404E-01, 4.00162E+00, 2.9871E+00, 8.5778E+03, &
      1.762144E-01, 2.92375E+00, 4.6314E+00, 1.2441E+04, &
      1.883732E-01, 2.16988E+00, 6.9768E+00, 1.7531E+04, &
      2.013709E-01, 2.08958E+00, 9.7069E+00, 2.2817E+04, &
      2.152655E-01, 2.79942E+00, 1.2508E+01, 2.7502E+04, &
      2.301188E-01, 4.24554E+00, 1.5059E+01, 3.0975E+04, &
      2.459970E-01, 6.23497E+00, 1.7117E+01, 3.2935E+04, &
      2.629708E-01, 8.48830E+00, 1.8549E+01, 3.3389E+04, &
      2.811158E-01, 1.06387E+01, 1.9336E+01, 3.2559E+04, &
      2.942940E-01, 1.15884E+01, 1.9531E+01, 3.1414E+04, &
      2.987985E-01, 1.12613E+01, 1.9540E+01, 3.0954E+04, &
      2.999997E-01, 1.04367E+01, 1.9538E+01, 3.0827E+04, &
      3.005128E-01, 1.02828E+01, 2.1378E+01, 3.3672E+04, &
      3.018015E-01, 1.15479E+01, 2.1381E+01, 3.3534E+04, &
      3.061520E-01, 1.27165E+01, 2.1378E+01, 3.3053E+04, &
      3.063060E-01, 1.27431E+01, 2.1378E+01, 3.3036E+04, &
      3.108380E-01, 1.32377E+01, 2.1353E+01, 3.2516E+04, &
      3.120876E-01, 1.30303E+01, 2.1343E+01, 3.2371E+04, &
      3.139620E-01, 1.37281E+01, 2.2129E+01, 3.3363E+04, &
      3.186480E-01, 1.47549E+01, 2.2086E+01, 3.2808E+04, &
      3.212482E-01, 1.51933E+01, 2.2055E+01, 3.2496E+04, &
      3.434143E-01, 1.80485E+01, 2.1616E+01, 2.9794E+04, &
      3.671099E-01, 2.02626E+01, 2.0907E+01, 2.6958E+04, &
      3.857280E-01, 2.14535E+01, 2.0256E+01, 2.4859E+04, &
      3.916320E-01, 2.14973E+01, 2.0041E+01, 2.4224E+04, &
      3.924405E-01, 2.14060E+01, 2.0011E+01, 2.4138E+04, &
      3.932064E-01, 2.11563E+01, 1.9983E+01, 2.4057E+04, &
      3.955680E-01, 2.17734E+01, 2.0780E+01, 2.4867E+04, &
      4.014720E-01, 2.25608E+01, 2.0562E+01, 2.4245E+04, &
      4.195189E-01, 2.39959E+01, 1.9893E+01, 2.2447E+04, &
      4.484657E-01, 2.56660E+01, 1.8833E+01, 1.9879E+04, &
      4.794098E-01, 2.70341E+01, 1.7749E+01, 1.7526E+04, &
      5.124891E-01, 2.81698E+01, 1.6659E+01, 1.5390E+04, &
      5.478508E-01, 2.91188E+01, 1.5589E+01, 1.3472E+04, &
      5.856525E-01, 2.99121E+01, 1.4551E+01, 1.1764E+04, &
      6.260625E-01, 3.05739E+01, 1.3552E+01, 1.0250E+04, &
      6.692609E-01, 3.11231E+01, 1.2594E+01, 8.9113E+03, &
      7.154399E-01, 3.15714E+01, 1.1678E+01, 7.7303E+03, &
      7.648052E-01, 3.19301E+01, 1.0807E+01, 6.6932E+03, &
      8.175768E-01, 3.22102E+01, 9.9856E+00, 5.7858E+03, &
      8.739896E-01, 3.24224E+01, 9.2134E+00, 4.9947E+03, &
      9.342948E-01, 3.25811E+01, 8.4907E+00, 4.3066E+03, &
      9.987612E-01, 3.26957E+01, 7.8172E+00, 3.7100E+03, &
      1.067676E+00, 3.27199E+01, 7.0930E+00, 3.1499E+03, &
      1.141345E+00, 3.26594E+01, 6.4399E+00, 2.6762E+03, &
      1.220098E+00, 3.25330E+01, 5.8454E+00, 2.2732E+03, &
      1.304285E+00, 3.23292E+01, 5.2774E+00, 1.9208E+03, &
      1.394281E+00, 3.20337E+01, 4.7662E+00, 1.6237E+03, &
      1.490486E+00, 3.16382E+01, 4.3109E+00, 1.3747E+03, &
      1.593329E+00, 3.11118E+01, 3.9039E+00, 1.1655E+03, &
      1.703269E+00, 3.03822E+01, 3.5370E+00, 9.8866E+02, &
      1.820795E+00, 2.92819E+01, 3.2091E+00, 8.3993E+02, &
      1.946430E+00, 2.72756E+01, 2.9155E+00, 7.1465E+02, &
      2.038400E+00, 2.37509E+01, 2.7303E+00, 6.3964E+02, &
      2.069600E+00, 1.97463E+01, 2.6724E+00, 6.1682E+02, &
      2.077920E+00, 1.52496E+01, 2.6573E+00, 6.1094E+02, &
      2.080733E+00, 1.22125E+01, 1.1347E+01, 2.5868E+03, &
      2.090400E+00, 1.94878E+01, 1.1250E+01, 2.5528E+03, &
      2.112390E+00, 2.22543E+01, 1.1032E+01, 2.4775E+03, &
      2.121600E+00, 2.26814E+01, 1.0943E+01, 2.4469E+03, &
      2.144722E+00, 2.24813E+01, 1.0724E+01, 2.3722E+03, &
      2.153344E+00, 2.06853E+01, 1.0644E+01, 2.3452E+03, &
      2.166277E+00, 2.33438E+01, 1.4777E+01, 3.2342E+03, &
      2.198610E+00, 2.62000E+01, 1.4387E+01, 3.1028E+03, &
      2.224304E+00, 2.73921E+01, 1.4100E+01, 3.0061E+03, &
      2.325050E+00, 2.96519E+01, 1.3101E+01, 2.6725E+03, &
      2.360637E+00, 2.95060E+01, 1.2787E+01, 2.5693E+03, &
      2.370127E+00, 2.86977E+01, 1.2706E+01, 2.5428E+03, &
      2.377781E+00, 2.92895E+01, 1.4549E+01, 2.9016E+03, &
      2.384362E+00, 2.98931E+01, 1.4489E+01, 2.8817E+03, &
      2.419950E+00, 3.12811E+01, 1.4173E+01, 2.7776E+03, &
      2.541848E+00, 3.34545E+01, 1.3193E+01, 2.4621E+03, &
      2.717235E+00, 3.52232E+01, 1.1997E+01, 2.0950E+03, &
      2.904724E+00, 3.64368E+01, 1.0880E+01, 1.7780E+03, &
      3.105150E+00, 3.73289E+01, 9.8107E+00, 1.5004E+03, &
      3.319406E+00, 3.79344E+01, 8.8048E+00, 1.2604E+03, &
      3.548445E+00, 3.83570E+01, 7.9085E+00, 1.0596E+03, &
      3.793288E+00, 3.86559E+01, 7.1093E+00, 8.9165E+02, &
      4.055024E+00, 3.88701E+01, 6.3963E+00, 7.5100E+02, &
      4.334821E+00, 3.91683E+01, 5.7359E+00, 6.3053E+02, &
      4.633924E+00, 3.92510E+01, 5.1210E+00, 5.2713E+02, &
      4.953664E+00, 3.93244E+01, 4.5662E+00, 4.4018E+02, &
      5.295467E+00, 3.93154E+01, 4.0696E+00, 3.6746E+02, &
      5.660855E+00, 3.92785E+01, 3.6301E+00, 3.0706E+02, &
      6.051453E+00, 3.92227E+01, 3.2375E+00, 2.5659E+02, &
      6.469004E+00, 3.91530E+01, 2.8894E+00, 2.1460E+02, &
      6.915365E+00, 3.90743E+01, 2.5811E+00, 1.7969E+02, &
      7.392525E+00, 3.89901E+01, 2.3079E+00, 1.5063E+02, &
      7.902609E+00, 3.89024E+01, 2.0645E+00, 1.2635E+02, &
      8.447890E+00, 3.88129E+01, 1.8485E+00, 1.0611E+02, &
      9.030794E+00, 3.87295E+01, 1.6525E+00, 8.8995E+01, &
      9.653919E+00, 3.86362E+01, 1.4600E+00, 7.3814E+01, &
      1.032004E+01, 3.85263E+01, 1.2899E+00, 6.1250E+01, &
      1.103212E+01, 3.84030E+01, 1.1357E+00, 5.0679E+01, &
      1.179334E+01, 3.82634E+01, 1.0008E+00, 4.1991E+01, &
      1.260708E+01, 3.81022E+01, 8.8275E-01, 3.4840E+01, &
      1.347697E+01, 3.79068E+01, 7.7932E-01, 2.8951E+01, &
      1.440688E+01, 3.76486E+01, 6.8866E-01, 2.4094E+01, &
      1.540095E+01, 3.72495E+01, 6.0910E-01, 2.0082E+01, &
      1.646362E+01, 3.63315E+01, 5.3924E-01, 1.6765E+01, &
      1.669763E+01, 3.58476E+01, 5.2558E-01, 1.6141E+01, &
      1.695321E+01, 3.45036E+01, 5.1129E-01, 1.5496E+01, &
      1.702136E+01, 3.28830E+01, 5.0758E-01, 1.5330E+01, &
      1.712359E+01, 3.45372E+01, 3.6974E+00, 1.0341E+02, &
      1.737917E+01, 3.60327E+01, 3.5922E+00, 9.9018E+01, &
      1.759961E+01, 3.65825E+01, 3.5050E+00, 9.5432E+01, &
      1.881398E+01, 3.78229E+01, 3.1169E+00, 7.9496E+01, &
      2.011215E+01, 3.83725E+01, 2.7995E+00, 6.6884E+01, &
      2.149988E+01, 3.87118E+01, 2.5209E+00, 5.6421E+01, &
      2.298338E+01, 3.89467E+01, 2.2626E+00, 4.7450E+01, &
      2.456923E+01, 3.91063E+01, 2.0235E+00, 3.9771E+01, &
      2.626450E+01, 3.92168E+01, 1.8082E+00, 3.3314E+01, &
      2.807676E+01, 3.92939E+01, 1.6098E+00, 2.7811E+01, &
      3.001405E+01, 3.93414E+01, 1.4316E+00, 2.3197E+01, &
      3.208502E+01, 3.93703E+01, 1.2733E+00, 1.9357E+01, &
      3.429889E+01, 3.94221E+01, 1.1318E+00, 1.6149E+01, &
      3.666551E+01, 3.94258E+01, 9.9964E-01, 1.3395E+01, &
      3.919543E+01, 3.94194E+01, 8.8302E-01, 1.1118E+01, &
      4.189992E+01, 3.94068E+01, 7.8007E-01, 9.2337E+00, &
      4.479101E+01, 3.93903E+01, 6.8916E-01, 7.6742E+00, &
      4.788159E+01, 3.93712E+01, 6.0881E-01, 6.3826E+00, &
      5.118542E+01, 3.93507E+01, 5.3780E-01, 5.3125E+00, &
      5.471721E+01, 3.93296E+01, 4.7479E-01, 4.4239E+00, &
      5.849270E+01, 3.93082E+01, 4.1920E-01, 3.6885E+00, &
      6.252870E+01, 3.92872E+01, 3.7014E-01, 3.0796E+00, &
      6.684318E+01, 3.92669E+01, 3.2684E-01, 2.5753E+00, &
      7.145536E+01, 3.92475E+01, 2.8863E-01, 2.1575E+00, &
      7.638578E+01, 3.92293E+01, 2.5432E-01, 1.8077E+00, &
      8.165640E+01, 3.92118E+01, 2.2342E-01, 1.5142E+00, &
      8.729069E+01, 3.91946E+01, 1.9588E-01, 1.2699E+00, &
      9.331374E+01, 3.91782E+01, 1.7174E-01, 1.0685E+00, &
      9.975239E+01, 3.91628E+01, 1.5058E-01, 9.0242E-01, &
      1.066353E+02, 3.91484E+01, 1.3204E-01, 7.6535E-01, &
      1.139931E+02, 3.91350E+01, 1.1579E-01, 6.5212E-01, &
      1.218587E+02, 3.91226E+01, 1.0154E-01, 5.5849E-01, &
      1.302669E+02, 3.91111E+01, 8.9045E-02, 4.8097E-01, &
      1.392553E+02, 3.91007E+01, 7.8093E-02, 4.1670E-01, &
      1.488640E+02, 3.90910E+01, 6.8490E-02, 3.6333E-01, &
      1.591356E+02, 3.90823E+01, 6.0071E-02, 3.1892E-01, &
      1.701159E+02, 3.90742E+01, 5.2689E-02, 2.8188E-01, &
      1.818539E+02, 3.90669E+01, 4.6215E-02, 2.5091E-01, &
      1.944018E+02, 3.90603E+01, 4.0538E-02, 2.2494E-01, &
      2.078156E+02, 3.90543E+01, 3.5560E-02, 2.0307E-01, &
      2.221548E+02, 3.90488E+01, 3.1194E-02, 1.8459E-01, &
      2.374835E+02, 3.90439E+01, 2.7365E-02, 1.6890E-01, &
      2.538699E+02, 3.90394E+01, 2.4007E-02, 1.5551E-01, &
      2.713869E+02, 3.90354E+01, 2.1062E-02, 1.4401E-01, &
      2.901126E+02, 3.90317E+01, 1.8478E-02, 1.3407E-01, &
      3.101304E+02, 3.90285E+01, 1.6212E-02, 1.2542E-01, &
      3.315294E+02, 3.90255E+01, 1.4224E-02, 1.1784E-01, &
      3.544049E+02, 3.90228E+01, 1.2480E-02, 1.1113E-01, &
      3.788588E+02, 3.90204E+01, 1.0951E-02, 1.0516E-01, &
      4.050001E+02, 3.90183E+01, 9.6090E-03, 9.9794E-02, &
      4.329451E+02, 3.90164E+01, 8.4317E-03, 9.4932E-02]
   case(40)
      frel = -1.9140E-01
      allocate(nist_tab(856))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 8.63969E+00, 3.7546E+00, 1.6202E+05, &
      1.142761E-02, 9.20229E+00, 3.8719E+00, 1.5630E+05, &
      1.221612E-02, 9.79034E+00, 3.8448E+00, 1.4519E+05, &
      1.305903E-02, 1.03210E+01, 3.6795E+00, 1.2998E+05, &
      1.396010E-02, 1.07704E+01, 3.3990E+00, 1.1232E+05, &
      1.492335E-02, 1.11106E+01, 3.0377E+00, 9.3901E+04, &
      1.595306E-02, 1.13288E+01, 2.6337E+00, 7.6157E+04, &
      1.705382E-02, 1.14247E+01, 2.2215E+00, 6.0092E+04, &
      1.823053E-02, 1.14062E+01, 1.8295E+00, 4.6295E+04, &
      1.948844E-02, 1.12852E+01, 1.4775E+00, 3.4973E+04, &
      2.083314E-02, 1.10699E+01, 1.1750E+00, 2.6017E+04, &
      2.227063E-02, 1.07559E+01, 9.2388E-01, 1.9137E+04, &
      2.380730E-02, 1.03102E+01, 7.2105E-01, 1.3972E+04, &
      2.545001E-02, 9.62155E+00, 5.6055E-01, 1.0161E+04, &
      2.720606E-02, 8.21966E+00, 4.3549E-01, 7.3841E+03, &
      2.812600E-02, 6.29704E+00, 3.8425E-01, 6.3023E+03, &
      2.855650E-02, 3.21432E+00, 3.6300E-01, 5.8640E+03, &
      2.867130E-02, -5.92300E-01, 3.5760E-01, 5.7536E+03, &
      2.884350E-02, 3.54960E+00, 7.9002E+00, 1.2635E+05, &
      2.908327E-02, 6.11659E+00, 7.6864E+00, 1.2192E+05, &
      2.927400E-02, 7.19898E+00, 7.5218E+00, 1.1853E+05, &
      3.109002E-02, 1.09926E+01, 6.1658E+00, 9.1486E+04, &
      3.323523E-02, 1.23938E+01, 4.9447E+00, 6.8632E+04, &
      3.552846E-02, 1.29928E+01, 3.9611E+00, 5.1432E+04, &
      3.797993E-02, 1.32238E+01, 3.1838E+00, 3.8671E+04, &
      4.060054E-02, 1.32621E+01, 2.5776E+00, 2.9287E+04, &
      4.340198E-02, 1.31953E+01, 2.1087E+00, 2.2413E+04, &
      4.639671E-02, 1.30678E+01, 1.7474E+00, 1.7374E+04, &
      4.959809E-02, 1.28901E+01, 1.4693E+00, 1.3666E+04, &
      5.027400E-02, 1.28410E+01, 1.4213E+00, 1.3041E+04, &
      5.104350E-02, 1.27552E+01, 1.3703E+00, 1.2384E+04, &
      5.124870E-02, 1.26876E+01, 1.3573E+00, 1.2217E+04, &
      5.155650E-02, 1.27297E+01, 1.4517E+00, 1.2989E+04, &
      5.232600E-02, 1.27431E+01, 1.4103E+00, 1.2433E+04, &
      5.302035E-02, 1.27301E+01, 1.3757E+00, 1.1970E+04, &
      5.667876E-02, 1.26150E+01, 1.2296E+00, 1.0007E+04, &
      6.058959E-02, 1.24840E+01, 1.1224E+00, 8.5459E+03, &
      6.477028E-02, 1.23506E+01, 1.0443E+00, 7.4380E+03, &
      6.923942E-02, 1.22161E+01, 9.8836E-01, 6.5850E+03, &
      7.401695E-02, 1.20790E+01, 9.4929E-01, 5.9166E+03, &
      7.912411E-02, 1.19368E+01, 9.2428E-01, 5.3889E+03, &
      8.458368E-02, 1.17864E+01, 9.0958E-01, 4.9609E+03, &
      9.041995E-02, 1.16299E+01, 9.0237E-01, 4.6039E+03, &
      9.665893E-02, 1.14486E+01, 9.0088E-01, 4.2998E+03, &
      1.033284E-01, 1.12422E+01, 9.0376E-01, 4.0351E+03, &
      1.104581E-01, 1.10023E+01, 9.0996E-01, 3.8007E+03, &
      1.180797E-01, 1.07172E+01, 9.1865E-01, 3.5893E+03, &
      1.262272E-01, 1.03714E+01, 9.2912E-01, 3.3960E+03, &
      1.349368E-01, 9.94174E+00, 9.4071E-01, 3.2166E+03, &
      1.442475E-01, 9.39145E+00, 9.5287E-01, 3.0480E+03, &
      1.542005E-01, 8.65667E+00, 9.6510E-01, 2.8880E+03, &
      1.648404E-01, 7.59071E+00, 9.7697E-01, 2.7349E+03, &
      1.762144E-01, 5.62572E+00, 9.8807E-01, 2.5876E+03, &
      1.764000E-01, 5.57321E+00, 9.8824E-01, 2.5854E+03, &
      1.787520E-01, 4.67369E+00, 9.9032E-01, 2.5567E+03, &
      1.791000E-01, 4.45684E+00, 9.9062E-01, 2.5525E+03, &
      1.798200E-01, 3.60635E+00, 9.9124E-01, 2.5439E+03, &
      1.809000E-01, 3.89674E+00, 2.3162E+00, 5.9074E+03, &
      1.814880E-01, 3.85670E+00, 2.3505E+00, 5.9754E+03, &
      1.822176E-01, 3.43839E+00, 2.3941E+00, 6.0618E+03, &
      1.833120E-01, 3.83947E+00, 3.3624E+00, 8.4626E+03, &
      1.836000E-01, 3.89979E+00, 3.3916E+00, 8.5227E+03, &
      1.860480E-01, 4.02739E+00, 3.6524E+00, 9.0571E+03, &
      1.883732E-01, 3.95244E+00, 3.9221E+00, 9.6059E+03, &
      2.013709E-01, 3.06524E+00, 5.8955E+00, 1.3507E+04, &
      2.152655E-01, 2.61432E+00, 8.5578E+00, 1.8340E+04, &
      2.301188E-01, 2.94879E+00, 1.1475E+01, 2.3004E+04, &
      2.459970E-01, 4.10791E+00, 1.4289E+01, 2.6798E+04, &
      2.629708E-01, 5.95134E+00, 1.6688E+01, 2.9276E+04, &
      2.811158E-01, 8.21365E+00, 1.8468E+01, 3.0308E+04, &
      3.005128E-01, 1.05480E+01, 1.9558E+01, 3.0024E+04, &
      3.212482E-01, 1.23356E+01, 1.9988E+01, 2.8704E+04, &
      3.238900E-01, 1.24034E+01, 2.0000E+01, 2.8487E+04, &
      3.288475E-01, 1.19981E+01, 2.0000E+01, 2.8058E+04, &
      3.301695E-01, 1.10640E+01, 1.9996E+01, 2.7939E+04, &
      3.321525E-01, 1.22979E+01, 2.2058E+01, 3.0637E+04, &
      3.373160E-01, 1.36414E+01, 2.2040E+01, 3.0144E+04, &
      3.424790E-01, 1.41694E+01, 2.1998E+01, 2.9634E+04, &
      3.434143E-01, 1.40923E+01, 2.1989E+01, 2.9540E+04, &
      3.438558E-01, 1.39242E+01, 2.1984E+01, 2.9496E+04, &
      3.459210E-01, 1.46887E+01, 2.2860E+01, 3.0488E+04, &
      3.510840E-01, 1.57956E+01, 2.2798E+01, 2.9958E+04, &
      3.671099E-01, 1.80251E+01, 2.2504E+01, 2.8281E+04, &
      3.924405E-01, 2.05907E+01, 2.1810E+01, 2.5640E+04, &
      4.195189E-01, 2.24759E+01, 2.0892E+01, 2.2976E+04, &
      4.216940E-01, 2.25688E+01, 2.0814E+01, 2.2772E+04, &
      4.281485E-01, 2.26184E+01, 2.0580E+01, 2.2177E+04, &
      4.298697E-01, 2.22665E+01, 2.0518E+01, 2.2021E+04, &
      4.324515E-01, 2.29066E+01, 2.1338E+01, 2.2765E+04, &
      4.389060E-01, 2.37239E+01, 2.1102E+01, 2.2182E+04, &
      4.484657E-01, 2.45159E+01, 2.0752E+01, 2.1349E+04, &
      4.794098E-01, 2.63946E+01, 1.9628E+01, 1.8891E+04, &
      5.124891E-01, 2.78803E+01, 1.8475E+01, 1.6633E+04, &
      5.478508E-01, 2.91158E+01, 1.7321E+01, 1.4588E+04, &
      5.856525E-01, 3.01279E+01, 1.6183E+01, 1.2751E+04, &
      6.260625E-01, 3.09657E+01, 1.5083E+01, 1.1118E+04, &
      6.692609E-01, 3.16585E+01, 1.4029E+01, 9.6742E+03, &
      7.154399E-01, 3.22301E+01, 1.3025E+01, 8.4028E+03, &
      7.648052E-01, 3.26986E+01, 1.2066E+01, 7.2826E+03, &
      8.175768E-01, 3.30705E+01, 1.1155E+01, 6.2991E+03, &
      8.739896E-01, 3.33583E+01, 1.0297E+01, 5.4403E+03, &
      9.342948E-01, 3.35740E+01, 9.4945E+00, 4.6933E+03, &
      9.987612E-01, 3.37337E+01, 8.7469E+00, 4.0455E+03, &
      1.067676E+00, 3.38779E+01, 7.9335E+00, 3.4334E+03, &
      1.141345E+00, 3.38850E+01, 7.2000E+00, 2.9158E+03, &
      1.220098E+00, 3.38089E+01, 6.5408E+00, 2.4788E+03, &
      1.304285E+00, 3.36698E+01, 5.9490E+00, 2.1099E+03, &
      1.394281E+00, 3.34666E+01, 5.3904E+00, 1.7894E+03, &
      1.490486E+00, 3.31750E+01, 4.8740E+00, 1.5144E+03, &
      1.593329E+00, 3.27836E+01, 4.4137E+00, 1.2838E+03, &
      1.703269E+00, 3.22630E+01, 4.0029E+00, 1.0900E+03, &
      1.820795E+00, 3.15430E+01, 3.6307E+00, 9.2575E+02, &
      1.946430E+00, 3.04556E+01, 3.2965E+00, 7.8713E+02, &
      2.080733E+00, 2.84688E+01, 2.9969E+00, 6.7022E+02, &
      2.177854E+00, 2.50258E+01, 2.8100E+00, 6.0095E+02, &
      2.211189E+00, 2.10778E+01, 2.7507E+00, 5.7959E+02, &
      2.220078E+00, 1.66305E+01, 2.7353E+00, 5.7408E+02, &
      2.224304E+00, 1.62010E+01, 1.1335E+01, 2.3565E+03, &
      2.233412E+00, 2.08476E+01, 1.1245E+01, 2.3283E+03, &
      2.260566E+00, 2.38230E+01, 1.0983E+01, 2.2470E+03, &
      2.266746E+00, 2.40772E+01, 1.0925E+01, 2.2291E+03, &
      2.295166E+00, 2.39079E+01, 1.0663E+01, 2.1489E+03, &
      2.304393E+00, 2.21098E+01, 1.0581E+01, 2.1238E+03, &
      2.318233E+00, 2.47235E+01, 1.4670E+01, 2.9248E+03, &
      2.352834E+00, 2.75108E+01, 1.4271E+01, 2.8037E+03, &
      2.377781E+00, 2.85758E+01, 1.4007E+01, 2.7230E+03, &
      2.480968E+00, 3.07315E+01, 1.3041E+01, 2.4304E+03, &
      2.518942E+00, 3.05892E+01, 1.2728E+01, 2.3365E+03, &
      2.529068E+00, 2.97840E+01, 1.2648E+01, 2.3125E+03, &
      2.541848E+00, 3.08105E+01, 1.4446E+01, 2.6273E+03, &
      2.544258E+00, 3.09762E+01, 1.4426E+01, 2.6211E+03, &
      2.582232E+00, 3.23607E+01, 1.4114E+01, 2.5269E+03, &
      2.717235E+00, 3.45926E+01, 1.3113E+01, 2.2316E+03, &
      2.904724E+00, 3.63503E+01, 1.1927E+01, 1.8994E+03, &
      3.105150E+00, 3.75776E+01, 1.0765E+01, 1.6045E+03, &
      3.319406E+00, 3.84098E+01, 9.6735E+00, 1.3494E+03, &
      3.548445E+00, 3.89875E+01, 8.6866E+00, 1.1342E+03, &
      3.793288E+00, 3.93928E+01, 7.8053E+00, 9.5390E+02, &
      4.055024E+00, 3.96799E+01, 7.0192E+00, 8.0303E+02, &
      4.334821E+00, 3.98855E+01, 6.3175E+00, 6.7665E+02, &
      4.633924E+00, 4.01747E+01, 5.6673E+00, 5.6835E+02, &
      4.953664E+00, 4.02531E+01, 5.0616E+00, 4.7536E+02, &
      5.295467E+00, 4.03220E+01, 4.5149E+00, 3.9713E+02, &
      5.660855E+00, 4.03110E+01, 4.0262E+00, 3.3174E+02, &
      6.051453E+00, 4.02733E+01, 3.5936E+00, 2.7740E+02, &
      6.469004E+00, 4.02179E+01, 3.2086E+00, 2.3209E+02, &
      6.915365E+00, 4.01495E+01, 2.8651E+00, 1.9424E+02, &
      7.392525E+00, 4.00724E+01, 2.5609E+00, 1.6274E+02, &
      7.902609E+00, 3.99904E+01, 2.2911E+00, 1.3651E+02, &
      8.447890E+00, 3.99071E+01, 2.0510E+00, 1.1461E+02, &
      9.030794E+00, 3.98312E+01, 1.8332E+00, 9.6089E+01, &
      9.653919E+00, 3.97383E+01, 1.6261E+00, 7.9996E+01, &
      1.032004E+01, 3.96345E+01, 1.4376E+00, 6.6411E+01, &
      1.103212E+01, 3.95203E+01, 1.2709E+00, 5.5150E+01, &
      1.179334E+01, 3.93936E+01, 1.1197E+00, 4.5671E+01, &
      1.260708E+01, 3.92506E+01, 9.8744E-01, 3.7876E+01, &
      1.347697E+01, 3.90847E+01, 8.7156E-01, 3.1456E+01, &
      1.440688E+01, 3.88813E+01, 7.6995E-01, 2.6162E+01, &
      1.540095E+01, 3.86073E+01, 6.8082E-01, 2.1793E+01, &
      1.646362E+01, 3.81655E+01, 6.0256E-01, 1.8181E+01, &
      1.759961E+01, 3.69666E+01, 5.3379E-01, 1.5192E+01, &
      1.763765E+01, 3.68730E+01, 5.3171E-01, 1.5105E+01, &
      1.790761E+01, 3.55375E+01, 5.1731E-01, 1.4503E+01, &
      1.797960E+01, 3.39278E+01, 5.1357E-01, 1.4349E+01, &
      1.808759E+01, 3.55694E+01, 3.6815E+00, 9.5056E+01, &
      1.835755E+01, 3.70539E+01, 3.5783E+00, 9.1064E+01, &
      1.881398E+01, 3.79492E+01, 3.4138E+00, 8.4814E+01, &
      2.011215E+01, 3.89585E+01, 3.0491E+00, 7.0963E+01, &
      2.149988E+01, 3.94548E+01, 2.7406E+00, 5.9752E+01, &
      2.298338E+01, 3.97696E+01, 2.4665E+00, 5.0383E+01, &
      2.456923E+01, 3.99873E+01, 2.2109E+00, 4.2320E+01, &
      2.626450E+01, 4.01362E+01, 1.9773E+00, 3.5475E+01, &
      2.807676E+01, 4.02404E+01, 1.7610E+00, 2.9623E+01, &
      3.001405E+01, 4.03063E+01, 1.5667E+00, 2.4716E+01, &
      3.208502E+01, 4.03483E+01, 1.3939E+00, 2.0629E+01, &
      3.429889E+01, 4.03736E+01, 1.2404E+00, 1.7225E+01, &
      3.666551E+01, 4.04228E+01, 1.1015E+00, 1.4362E+01, &
      3.919543E+01, 4.04242E+01, 9.7302E-01, 1.1917E+01, &
      4.189992E+01, 4.04164E+01, 8.5957E-01, 9.8949E+00, &
      4.479101E+01, 4.04028E+01, 7.5942E-01, 8.2215E+00, &
      4.788159E+01, 4.03857E+01, 6.7097E-01, 6.8362E+00, &
      5.118542E+01, 4.03663E+01, 5.9279E-01, 5.6887E+00, &
      5.471721E+01, 4.03456E+01, 5.2368E-01, 4.7379E+00, &
      5.849270E+01, 4.03244E+01, 4.6240E-01, 3.9484E+00, &
      6.252870E+01, 4.03032E+01, 4.0832E-01, 3.2949E+00, &
      6.684318E+01, 4.02823E+01, 3.6059E-01, 2.7536E+00, &
      7.145536E+01, 4.02622E+01, 3.1845E-01, 2.3052E+00, &
      7.638578E+01, 4.02432E+01, 2.8126E-01, 1.9336E+00, &
      8.165640E+01, 4.02252E+01, 2.4752E-01, 1.6203E+00, &
      8.729069E+01, 4.02075E+01, 2.1714E-01, 1.3578E+00, &
      9.331374E+01, 4.01904E+01, 1.9051E-01, 1.1413E+00, &
      9.975239E+01, 4.01742E+01, 1.6714E-01, 9.6272E-01, &
      1.066353E+02, 4.01590E+01, 1.4666E-01, 8.1529E-01, &
      1.139931E+02, 4.01449E+01, 1.2868E-01, 6.9347E-01, &
      1.218587E+02, 4.01318E+01, 1.1292E-01, 5.9272E-01, &
      1.302669E+02, 4.01196E+01, 9.9091E-02, 5.0929E-01, &
      1.392553E+02, 4.01085E+01, 8.6960E-02, 4.4014E-01, &
      1.488640E+02, 4.00982E+01, 7.6317E-02, 3.8270E-01, &
      1.591356E+02, 4.00888E+01, 6.6979E-02, 3.3493E-01, &
      1.701159E+02, 4.00803E+01, 5.8786E-02, 2.9510E-01, &
      1.818539E+02, 4.00724E+01, 5.1598E-02, 2.6182E-01, &
      1.944018E+02, 4.00653E+01, 4.5290E-02, 2.3392E-01, &
      2.078156E+02, 4.00588E+01, 3.9755E-02, 2.1047E-01, &
      2.221548E+02, 4.00530E+01, 3.4897E-02, 1.9067E-01, &
      2.374835E+02, 4.00477E+01, 3.0634E-02, 1.7390E-01, &
      2.538699E+02, 4.00428E+01, 2.6893E-02, 1.5960E-01, &
      2.713869E+02, 4.00385E+01, 2.3609E-02, 1.4735E-01, &
      2.901126E+02, 4.00346E+01, 2.0727E-02, 1.3680E-01, &
      3.101304E+02, 4.00310E+01, 1.8198E-02, 1.2764E-01, &
      3.315294E+02, 4.00278E+01, 1.5978E-02, 1.1964E-01, &
      3.544049E+02, 4.00249E+01, 1.4029E-02, 1.1259E-01, &
      3.788588E+02, 4.00223E+01, 1.2318E-02, 1.0634E-01, &
      4.050001E+02, 4.00200E+01, 1.0816E-02, 1.0074E-01, &
      4.329451E+02, 4.00179E+01, 9.4972E-03, 9.5691E-02]
   case(41)
      frel = -2.0280E-01
      allocate(nist_tab(856))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.00659E+01, 4.4264E+00, 1.8754E+05, &
      1.142761E-02, 1.05322E+01, 4.2928E+00, 1.7014E+05, &
      1.221612E-02, 1.09518E+01, 4.0896E+00, 1.5163E+05, &
      1.305903E-02, 1.13120E+01, 3.8319E+00, 1.3290E+05, &
      1.396010E-02, 1.16045E+01, 3.5347E+00, 1.1468E+05, &
      1.492335E-02, 1.18247E+01, 3.2159E+00, 9.7604E+04, &
      1.595306E-02, 1.19742E+01, 2.8920E+00, 8.2109E+04, &
      1.705382E-02, 1.20579E+01, 2.5762E+00, 6.8422E+04, &
      1.823053E-02, 1.20820E+01, 2.2779E+00, 5.6593E+04, &
      1.948844E-02, 1.20526E+01, 2.0030E+00, 4.6553E+04, &
      2.083314E-02, 1.19742E+01, 1.7549E+00, 3.8153E+04, &
      2.227063E-02, 1.18479E+01, 1.5345E+00, 3.1207E+04, &
      2.380730E-02, 1.16699E+01, 1.3411E+00, 2.5514E+04, &
      2.545001E-02, 1.14272E+01, 1.1731E+00, 2.0877E+04, &
      2.720606E-02, 1.10887E+01, 1.0283E+00, 1.7119E+04, &
      2.908327E-02, 1.05769E+01, 9.0410E-01, 1.4080E+04, &
      3.109002E-02, 9.64960E+00, 7.9807E-01, 1.1627E+04, &
      3.323523E-02, 6.61650E+00, 7.0772E-01, 9.6449E+03, &
      3.373050E-02, 3.18596E+00, 6.8959E-01, 9.2598E+03, &
      3.386610E-02, -1.19368E+00, 6.8479E-01, 9.1585E+03, &
      3.406950E-02, 3.70199E+00, 9.4040E+00, 1.2502E+05, &
      3.457800E-02, 8.02069E+00, 8.8157E+00, 1.1548E+05, &
      3.552846E-02, 1.07915E+01, 7.8365E+00, 9.9903E+04, &
      3.797993E-02, 1.32373E+01, 5.8871E+00, 7.0207E+04, &
      4.060054E-02, 1.39977E+01, 4.5061E+00, 5.0270E+04, &
      4.340198E-02, 1.42144E+01, 3.5327E+00, 3.6867E+04, &
      4.639671E-02, 1.42065E+01, 2.8358E+00, 2.7684E+04, &
      4.959809E-02, 1.40978E+01, 2.3295E+00, 2.1273E+04, &
      5.302035E-02, 1.39394E+01, 1.9564E+00, 1.6713E+04, &
      5.667876E-02, 1.37336E+01, 1.6780E+00, 1.3409E+04, &
      5.693800E-02, 1.37147E+01, 1.6617E+00, 1.3219E+04, &
      5.780950E-02, 1.36248E+01, 1.6096E+00, 1.2611E+04, &
      5.804190E-02, 1.35549E+01, 1.5964E+00, 1.2458E+04, &
      5.839050E-02, 1.35974E+01, 1.6935E+00, 1.3137E+04, &
      5.926200E-02, 1.36097E+01, 1.6519E+00, 1.2625E+04, &
      6.058959E-02, 1.35818E+01, 1.5957E+00, 1.1929E+04, &
      6.477028E-02, 1.34584E+01, 1.4641E+00, 1.0239E+04, &
      6.923942E-02, 1.33287E+01, 1.3708E+00, 8.9673E+03, &
      7.401695E-02, 1.32008E+01, 1.3053E+00, 7.9876E+03, &
      7.912411E-02, 1.30742E+01, 1.2606E+00, 7.2164E+03, &
      8.458368E-02, 1.29466E+01, 1.2317E+00, 6.5956E+03, &
      9.041995E-02, 1.28148E+01, 1.2146E+00, 6.0843E+03, &
      9.665893E-02, 1.26746E+01, 1.2064E+00, 5.6533E+03, &
      1.033284E-01, 1.25133E+01, 1.2049E+00, 5.2822E+03, &
      1.104581E-01, 1.23409E+01, 1.2085E+00, 4.9558E+03, &
      1.180797E-01, 1.21424E+01, 1.2156E+00, 4.6635E+03, &
      1.262272E-01, 1.19086E+01, 1.2254E+00, 4.3975E+03, &
      1.349368E-01, 1.16273E+01, 1.2367E+00, 4.1519E+03, &
      1.442475E-01, 1.12817E+01, 1.2489E+00, 3.9223E+03, &
      1.542005E-01, 1.08466E+01, 1.2611E+00, 3.7051E+03, &
      1.648404E-01, 1.02823E+01, 1.2727E+00, 3.4978E+03, &
      1.762144E-01, 9.51839E+00, 1.2830E+00, 3.2988E+03, &
      1.883732E-01, 8.38955E+00, 1.2916E+00, 3.1069E+03, &
      2.005080E-01, 6.42876E+00, 1.2978E+00, 2.9329E+03, &
      2.013709E-01, 6.18902E+00, 1.2981E+00, 2.9211E+03, &
      2.032520E-01, 5.47097E+00, 1.2988E+00, 2.8957E+03, &
      2.035770E-01, 5.28557E+00, 1.2990E+00, 2.8913E+03, &
      2.043954E-01, 4.41950E+00, 1.2992E+00, 2.8804E+03, &
      2.056230E-01, 4.71064E+00, 2.6448E+00, 5.8270E+03, &
      2.063630E-01, 4.65902E+00, 2.6845E+00, 5.8935E+03, &
      2.071926E-01, 4.22092E+00, 2.7303E+00, 5.9699E+03, &
      2.084370E-01, 4.61993E+00, 3.7235E+00, 8.0925E+03, &
      2.086920E-01, 4.66799E+00, 3.7474E+00, 8.1346E+03, &
      2.115480E-01, 4.79309E+00, 4.0300E+00, 8.6297E+03, &
      2.152655E-01, 4.64634E+00, 4.4399E+00, 9.3432E+03, &
      2.301188E-01, 3.68255E+00, 6.6351E+00, 1.3062E+04, &
      2.459970E-01, 3.31170E+00, 9.5437E+00, 1.7574E+04, &
      2.629708E-01, 3.84660E+00, 1.2665E+01, 2.1816E+04, &
      2.811158E-01, 5.28323E+00, 1.5581E+01, 2.5106E+04, &
      3.005128E-01, 7.40735E+00, 1.7948E+01, 2.7054E+04, &
      3.212482E-01, 9.86435E+00, 1.9577E+01, 2.7603E+04, &
      3.434143E-01, 1.21514E+01, 2.0429E+01, 2.6946E+04, &
      3.557400E-01, 1.28368E+01, 2.0588E+01, 2.6216E+04, &
      3.611850E-01, 1.23859E+01, 2.0601E+01, 2.5837E+04, &
      3.626370E-01, 1.13570E+01, 2.0599E+01, 2.5732E+04, &
      3.648150E-01, 1.27166E+01, 2.2873E+01, 2.8401E+04, &
      3.671099E-01, 1.35048E+01, 2.2871E+01, 2.8220E+04, &
      3.702600E-01, 1.41594E+01, 2.2859E+01, 2.7966E+04, &
      3.708320E-01, 1.42535E+01, 2.2856E+01, 2.7919E+04, &
      3.765080E-01, 1.48151E+01, 2.2811E+01, 2.7444E+04, &
      3.780216E-01, 1.45420E+01, 2.2794E+01, 2.7314E+04, &
      3.802920E-01, 1.53787E+01, 2.3757E+01, 2.8298E+04, &
      3.859680E-01, 1.65851E+01, 2.3685E+01, 2.7798E+04, &
      3.924405E-01, 1.75798E+01, 2.3579E+01, 2.7216E+04, &
      4.195189E-01, 2.06793E+01, 2.2919E+01, 2.4748E+04, &
      4.484657E-01, 2.29922E+01, 2.1970E+01, 2.2193E+04, &
      4.590320E-01, 2.35572E+01, 2.1592E+01, 2.1309E+04, &
      4.660580E-01, 2.36325E+01, 2.1335E+01, 2.0738E+04, &
      4.679316E-01, 2.32794E+01, 2.1266E+01, 2.0589E+04, &
      4.707420E-01, 2.39415E+01, 2.2096E+01, 2.1264E+04, &
      4.777680E-01, 2.47959E+01, 2.1838E+01, 2.0706E+04, &
      4.794098E-01, 2.49438E+01, 2.1777E+01, 2.0578E+04, &
      5.124891E-01, 2.71205E+01, 2.0561E+01, 1.8176E+04, &
      5.478508E-01, 2.87590E+01, 1.9311E+01, 1.5970E+04, &
      5.856525E-01, 3.00837E+01, 1.8064E+01, 1.3975E+04, &
      6.260625E-01, 3.11435E+01, 1.6848E+01, 1.2194E+04, &
      6.692609E-01, 3.20317E+01, 1.5670E+01, 1.0610E+04, &
      7.154399E-01, 3.27581E+01, 1.4545E+01, 9.2134E+03, &
      7.648052E-01, 3.33512E+01, 1.3480E+01, 7.9880E+03, &
      8.175768E-01, 3.38339E+01, 1.2471E+01, 6.9141E+03, &
      8.739896E-01, 3.42181E+01, 1.1516E+01, 5.9735E+03, &
      9.342948E-01, 3.45185E+01, 1.0619E+01, 5.1535E+03, &
      9.987612E-01, 3.48154E+01, 9.7821E+00, 4.4419E+03, &
      1.067676E+00, 3.51027E+01, 8.8727E+00, 3.7699E+03, &
      1.141345E+00, 3.51326E+01, 8.0514E+00, 3.2011E+03, &
      1.220098E+00, 3.50842E+01, 7.3122E+00, 2.7206E+03, &
      1.304285E+00, 3.49759E+01, 6.6472E+00, 2.3144E+03, &
      1.394281E+00, 3.48140E+01, 6.0491E+00, 1.9713E+03, &
      1.490486E+00, 3.45998E+01, 5.4997E+00, 1.6775E+03, &
      1.593329E+00, 3.43066E+01, 4.9797E+00, 1.4218E+03, &
      1.703269E+00, 3.39135E+01, 4.5124E+00, 1.2061E+03, &
      1.820795E+00, 3.33913E+01, 4.0948E+00, 1.0248E+03, &
      1.946430E+00, 3.26700E+01, 3.7178E+00, 8.7123E+02, &
      2.080733E+00, 3.15783E+01, 3.3776E+00, 7.4127E+02, &
      2.224304E+00, 2.95666E+01, 3.0723E+00, 6.3157E+02, &
      2.323090E+00, 2.62700E+01, 2.8904E+00, 5.6943E+02, &
      2.358648E+00, 2.23765E+01, 2.8297E+00, 5.4926E+02, &
      2.368130E+00, 1.79815E+01, 2.8139E+00, 5.4406E+02, &
      2.377781E+00, 2.08709E+01, 1.1274E+01, 2.1535E+03, &
      2.382353E+00, 2.21655E+01, 1.1232E+01, 2.1413E+03, &
      2.415406E+00, 2.53224E+01, 1.0933E+01, 2.0559E+03, &
      2.417910E+00, 2.54136E+01, 1.0911E+01, 2.0496E+03, &
      2.452376E+00, 2.52771E+01, 1.0612E+01, 1.9658E+03, &
      2.462235E+00, 2.34796E+01, 1.0530E+01, 1.9427E+03, &
      2.477023E+00, 2.60450E+01, 1.4574E+01, 2.6708E+03, &
      2.513994E+00, 2.87649E+01, 1.4182E+01, 2.5609E+03, &
      2.541848E+00, 2.98378E+01, 1.3909E+01, 2.4842E+03, &
      2.643746E+00, 3.17857E+01, 1.3024E+01, 2.2371E+03, &
      2.684212E+00, 3.16513E+01, 1.2715E+01, 2.1513E+03, &
      2.695002E+00, 3.08499E+01, 1.2636E+01, 2.1293E+03, &
      2.711189E+00, 3.20413E+01, 1.4411E+01, 2.4130E+03, &
      2.717235E+00, 3.23528E+01, 1.4364E+01, 2.3999E+03, &
      2.751654E+00, 3.34274E+01, 1.4102E+01, 2.3269E+03, &
      2.904724E+00, 3.57728E+01, 1.3054E+01, 2.0410E+03, &
      3.105150E+00, 3.75651E+01, 1.1824E+01, 1.7301E+03, &
      3.319406E+00, 3.87342E+01, 1.0626E+01, 1.4552E+03, &
      3.548445E+00, 3.95227E+01, 9.5442E+00, 1.2232E+03, &
      3.793288E+00, 4.00708E+01, 8.5721E+00, 1.0284E+03, &
      4.055024E+00, 4.04564E+01, 7.7022E+00, 8.6501E+02, &
      4.334821E+00, 4.07284E+01, 6.9255E+00, 7.2814E+02, &
      4.633924E+00, 4.09221E+01, 6.2323E+00, 6.1350E+02, &
      4.953664E+00, 4.11966E+01, 5.5898E+00, 5.1524E+02, &
      5.295467E+00, 4.12671E+01, 4.9939E+00, 4.3110E+02, &
      5.660855E+00, 4.13296E+01, 4.4558E+00, 3.6030E+02, &
      6.051453E+00, 4.13149E+01, 3.9755E+00, 3.0115E+02, &
      6.469004E+00, 4.12750E+01, 3.5503E+00, 2.5198E+02, &
      6.915365E+00, 4.12187E+01, 3.1728E+00, 2.1103E+02, &
      7.392525E+00, 4.11509E+01, 2.8352E+00, 1.7676E+02, &
      7.902609E+00, 4.10753E+01, 2.5355E+00, 1.4819E+02, &
      8.447890E+00, 4.09963E+01, 2.2696E+00, 1.2438E+02, &
      9.030794E+00, 4.09235E+01, 2.0284E+00, 1.0427E+02, &
      9.653919E+00, 4.08335E+01, 1.7989E+00, 8.6782E+01, &
      1.032004E+01, 4.07336E+01, 1.5966E+00, 7.2300E+01, &
      1.103212E+01, 4.06260E+01, 1.4125E+00, 6.0075E+01, &
      1.179334E+01, 4.05090E+01, 1.2492E+00, 4.9921E+01, &
      1.260708E+01, 4.03794E+01, 1.1014E+00, 4.1379E+01, &
      1.347697E+01, 4.02325E+01, 9.7189E-01, 3.4347E+01, &
      1.440688E+01, 4.00610E+01, 8.5843E-01, 2.8552E+01, &
      1.540095E+01, 3.98477E+01, 7.5891E-01, 2.3770E+01, &
      1.646362E+01, 3.95528E+01, 6.7149E-01, 1.9818E+01, &
      1.759961E+01, 3.90503E+01, 5.9468E-01, 1.6549E+01, &
      1.860589E+01, 3.78959E+01, 5.3780E-01, 1.4259E+01, &
      1.881398E+01, 3.71442E+01, 5.2714E-01, 1.3842E+01, &
      1.889067E+01, 3.65692E+01, 5.2329E-01, 1.3694E+01, &
      1.896661E+01, 3.49713E+01, 5.1953E-01, 1.3548E+01, &
      1.908053E+01, 3.65985E+01, 3.6636E+00, 8.8100E+01, &
      1.936531E+01, 3.80702E+01, 3.5646E+00, 8.4486E+01, &
      2.011215E+01, 3.92483E+01, 3.3283E+00, 7.6020E+01, &
      2.149988E+01, 4.00917E+01, 2.9812E+00, 6.3787E+01, &
      2.298338E+01, 4.05400E+01, 2.6792E+00, 5.3707E+01, &
      2.456923E+01, 4.08303E+01, 2.4080E+00, 4.5230E+01, &
      2.626450E+01, 4.10302E+01, 2.1563E+00, 3.7959E+01, &
      2.807676E+01, 4.11681E+01, 1.9214E+00, 3.1711E+01, &
      3.001405E+01, 4.12581E+01, 1.7100E+00, 2.6464E+01, &
      3.208502E+01, 4.13169E+01, 1.5220E+00, 2.2093E+01, &
      3.429889E+01, 4.13540E+01, 1.3548E+00, 1.8453E+01, &
      3.666551E+01, 4.13762E+01, 1.2062E+00, 1.5419E+01, &
      3.919543E+01, 4.14228E+01, 1.0700E+00, 1.2845E+01, &
      4.189992E+01, 4.14217E+01, 9.4541E-01, 1.0665E+01, &
      4.479101E+01, 4.14125E+01, 8.3542E-01, 8.8606E+00, &
      4.788159E+01, 4.13981E+01, 7.3828E-01, 7.3671E+00, &
      5.118542E+01, 4.13804E+01, 6.5247E-01, 6.1302E+00, &
      5.471721E+01, 4.13607E+01, 5.7661E-01, 5.1053E+00, &
      5.849270E+01, 4.13401E+01, 5.0951E-01, 4.2556E+00, &
      6.252870E+01, 4.13189E+01, 4.5000E-01, 3.5498E+00, &
      6.684318E+01, 4.12979E+01, 3.9747E-01, 2.9654E+00, &
      7.145536E+01, 4.12773E+01, 3.5110E-01, 2.4811E+00, &
      7.638578E+01, 4.12575E+01, 3.1015E-01, 2.0798E+00, &
      8.165640E+01, 4.12389E+01, 2.7365E-01, 1.7450E+00, &
      8.729069E+01, 4.12208E+01, 2.4026E-01, 1.4615E+00, &
      9.331374E+01, 4.12031E+01, 2.1090E-01, 1.2274E+00, &
      9.975239E+01, 4.11862E+01, 1.8514E-01, 1.0341E+00, &
      1.066353E+02, 4.11703E+01, 1.6253E-01, 8.7455E-01, &
      1.139931E+02, 4.11553E+01, 1.4269E-01, 7.4268E-01, &
      1.218587E+02, 4.11415E+01, 1.2528E-01, 6.3360E-01, &
      1.302669E+02, 4.11286E+01, 1.0999E-01, 5.4328E-01, &
      1.392553E+02, 4.11167E+01, 9.6581E-02, 4.6838E-01, &
      1.488640E+02, 4.11058E+01, 8.4806E-02, 4.0620E-01, &
      1.591356E+02, 4.10958E+01, 7.4470E-02, 3.5449E-01, &
      1.701159E+02, 4.10866E+01, 6.5397E-02, 3.1140E-01, &
      1.818539E+02, 4.10783E+01, 5.7431E-02, 2.7541E-01, &
      1.944018E+02, 4.10706E+01, 5.0438E-02, 2.4526E-01, &
      2.078156E+02, 4.10637E+01, 4.4297E-02, 2.1995E-01, &
      2.221548E+02, 4.10574E+01, 3.8906E-02, 1.9859E-01, &
      2.374835E+02, 4.10517E+01, 3.4172E-02, 1.8052E-01, &
      2.538699E+02, 4.10465E+01, 3.0015E-02, 1.6516E-01, &
      2.713869E+02, 4.10418E+01, 2.6365E-02, 1.5203E-01, &
      2.901126E+02, 4.10375E+01, 2.3159E-02, 1.4074E-01, &
      3.101304E+02, 4.10337E+01, 2.0344E-02, 1.3097E-01, &
      3.315294E+02, 4.10303E+01, 1.7872E-02, 1.2246E-01, &
      3.544049E+02, 4.10271E+01, 1.5700E-02, 1.1499E-01, &
      3.788588E+02, 4.10243E+01, 1.3793E-02, 1.0839E-01, &
      4.050001E+02, 4.10218E+01, 1.2118E-02, 1.0251E-01, &
      4.329451E+02, 4.10195E+01, 1.0647E-02, 9.7222E-02]
   case(42)
      frel = -2.1540E-01
      allocate(nist_tab(840))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.43077E+00, 4.2268E+00, 1.7343E+05, &
      1.142761E-02, 5.65740E+00, 4.3599E+00, 1.6734E+05, &
      1.221612E-02, 5.89493E+00, 4.4827E+00, 1.6095E+05, &
      1.305903E-02, 6.14160E+00, 4.5937E+00, 1.5429E+05, &
      1.396010E-02, 6.39538E+00, 4.6917E+00, 1.4741E+05, &
      1.492335E-02, 6.65388E+00, 4.7757E+00, 1.4036E+05, &
      1.595306E-02, 6.91442E+00, 4.8450E+00, 1.3321E+05, &
      1.705382E-02, 7.17398E+00, 4.8990E+00, 1.2600E+05, &
      1.823053E-02, 7.42913E+00, 4.9378E+00, 1.1880E+05, &
      1.948844E-02, 7.67590E+00, 4.9614E+00, 1.1166E+05, &
      2.083314E-02, 7.90949E+00, 4.9703E+00, 1.0464E+05, &
      2.227063E-02, 8.12377E+00, 4.9652E+00, 9.7787E+04, &
      2.380730E-02, 8.31018E+00, 4.9470E+00, 9.1140E+04, &
      2.545001E-02, 8.45545E+00, 4.9167E+00, 8.4736E+04, &
      2.720606E-02, 8.53633E+00, 4.8757E+00, 7.8604E+04, &
      2.908327E-02, 8.50468E+00, 4.8251E+00, 7.2768E+04, &
      3.109002E-02, 8.23348E+00, 4.7664E+00, 6.7243E+04, &
      3.323523E-02, 7.17095E+00, 4.7010E+00, 6.2040E+04, &
      3.410400E-02, 5.77629E+00, 4.6591E+00, 5.9921E+04, &
      3.462600E-02, 2.90324E+00, 4.6174E+00, 5.8489E+04, &
      3.476520E-02, -7.78485E-01, 4.6063E+00, 5.8115E+04, &
      3.497400E-02, 3.36242E+00, 1.1959E+01, 1.4998E+05, &
      3.552846E-02, 7.19047E+00, 1.1443E+01, 1.4127E+05, &
      3.797993E-02, 1.12789E+01, 9.5682E+00, 1.1050E+05, &
      4.060054E-02, 1.26164E+01, 8.1374E+00, 8.7909E+04, &
      4.340198E-02, 1.32362E+01, 7.0647E+00, 7.1394E+04, &
      4.639671E-02, 1.35538E+01, 6.2404E+00, 5.8993E+04, &
      4.959809E-02, 1.37210E+01, 5.5908E+00, 4.9441E+04, &
      5.302035E-02, 1.38042E+01, 5.0665E+00, 4.1912E+04, &
      5.667876E-02, 1.38311E+01, 4.6336E+00, 3.5857E+04, &
      6.058959E-02, 1.37870E+01, 4.2690E+00, 3.0904E+04, &
      6.149100E-02, 1.37286E+01, 4.1958E+00, 2.9929E+04, &
      6.173820E-02, 1.36601E+01, 4.1764E+00, 2.9671E+04, &
      6.210900E-02, 1.37257E+01, 4.2808E+00, 3.0231E+04, &
      6.303600E-02, 1.37810E+01, 4.2156E+00, 2.9332E+04, &
      6.477028E-02, 1.38132E+01, 4.1021E+00, 2.7778E+04, &
      6.923942E-02, 1.38363E+01, 3.8525E+00, 2.4404E+04, &
      7.401695E-02, 1.38351E+01, 3.6366E+00, 2.1550E+04, &
      7.912411E-02, 1.38196E+01, 3.4473E+00, 1.9109E+04, &
      8.458368E-02, 1.37912E+01, 3.2799E+00, 1.7008E+04, &
      9.041995E-02, 1.37496E+01, 3.1308E+00, 1.5187E+04, &
      9.665893E-02, 1.36931E+01, 2.9973E+00, 1.3601E+04, &
      1.033284E-01, 1.36193E+01, 2.8770E+00, 1.2212E+04, &
      1.104581E-01, 1.35141E+01, 2.7682E+00, 1.0992E+04, &
      1.180797E-01, 1.33944E+01, 2.6694E+00, 9.9159E+03, &
      1.262272E-01, 1.32439E+01, 2.5793E+00, 8.9631E+03, &
      1.349368E-01, 1.30549E+01, 2.4969E+00, 8.1169E+03, &
      1.442475E-01, 1.28173E+01, 2.4212E+00, 7.3629E+03, &
      1.542005E-01, 1.25165E+01, 2.3510E+00, 6.6881E+03, &
      1.648404E-01, 1.21311E+01, 2.2854E+00, 6.0820E+03, &
      1.762144E-01, 1.16281E+01, 2.2236E+00, 5.5358E+03, &
      1.883732E-01, 1.09533E+01, 2.1649E+00, 5.0419E+03, &
      2.013709E-01, 9.99069E+00, 2.1085E+00, 4.5940E+03, &
      2.152655E-01, 8.43463E+00, 2.0539E+00, 4.1864E+03, &
      2.224600E-01, 7.04766E+00, 2.0275E+00, 3.9991E+03, &
      2.256940E-01, 5.83330E+00, 2.0160E+00, 3.9195E+03, &
      2.258650E-01, 5.72490E+00, 2.0154E+00, 3.9153E+03, &
      2.267730E-01, 4.69403E+00, 2.0122E+00, 3.8935E+03, &
      2.281350E-01, 5.09083E+00, 3.6403E+00, 7.0005E+03, &
      2.291485E-01, 5.02968E+00, 3.6929E+00, 7.0703E+03, &
      2.301188E-01, 4.41639E+00, 3.7449E+00, 7.1395E+03, &
      2.314515E-01, 5.00987E+00, 4.9447E+00, 9.3722E+03, &
      2.315400E-01, 5.03178E+00, 4.9530E+00, 9.3843E+03, &
      2.349060E-01, 5.28899E+00, 5.2834E+00, 9.8667E+03, &
      2.459970E-01, 4.84517E+00, 6.6278E+00, 1.1819E+04, &
      2.629708E-01, 4.24159E+00, 9.4064E+00, 1.5691E+04, &
      2.811158E-01, 4.54616E+00, 1.2567E+01, 1.9609E+04, &
      3.005128E-01, 5.81417E+00, 1.5621E+01, 2.2801E+04, &
      3.212482E-01, 7.86636E+00, 1.8172E+01, 2.4814E+04, &
      3.434143E-01, 1.03493E+01, 1.9977E+01, 2.5517E+04, &
      3.671099E-01, 1.27662E+01, 2.0964E+01, 2.5050E+04, &
      3.844540E-01, 1.37735E+01, 2.1198E+01, 2.4187E+04, &
      3.903385E-01, 1.32606E+01, 2.1204E+01, 2.3829E+04, &
      3.919077E-01, 1.21417E+01, 2.1200E+01, 2.3729E+04, &
      3.924405E-01, 1.13840E+01, 2.3659E+01, 2.6445E+04, &
      3.942615E-01, 1.36060E+01, 2.3656E+01, 2.6320E+04, &
      4.001460E-01, 1.51521E+01, 2.3628E+01, 2.5903E+04, &
      4.015060E-01, 1.53665E+01, 2.3618E+01, 2.5804E+04, &
      4.076515E-01, 1.59187E+01, 2.3555E+01, 2.5347E+04, &
      4.092903E-01, 1.56122E+01, 2.3534E+01, 2.5222E+04, &
      4.117485E-01, 1.65015E+01, 2.4569E+01, 2.6175E+04, &
      4.178940E-01, 1.77696E+01, 2.4476E+01, 2.5693E+04, &
      4.195189E-01, 1.80296E+01, 2.4448E+01, 2.5564E+04, &
      4.484657E-01, 2.14691E+01, 2.3758E+01, 2.3240E+04, &
      4.794098E-01, 2.39738E+01, 2.2755E+01, 2.0823E+04, &
      4.945080E-01, 2.47940E+01, 2.2216E+01, 1.9709E+04, &
      5.020770E-01, 2.48741E+01, 2.1940E+01, 1.9170E+04, &
      5.040954E-01, 2.45128E+01, 2.1866E+01, 1.9029E+04, &
      5.071230E-01, 2.51922E+01, 2.2711E+01, 1.9647E+04, &
      5.124891E-01, 2.58644E+01, 2.2514E+01, 1.9272E+04, &
      5.146920E-01, 2.60690E+01, 2.2433E+01, 1.9121E+04, &
      5.478508E-01, 2.82255E+01, 2.1221E+01, 1.6994E+04, &
      5.856525E-01, 2.99319E+01, 1.9898E+01, 1.4907E+04, &
      6.260625E-01, 3.12977E+01, 1.8585E+01, 1.3025E+04, &
      6.692609E-01, 3.24106E+01, 1.7309E+01, 1.1349E+04, &
      7.154399E-01, 3.33261E+01, 1.6076E+01, 9.8605E+03, &
      7.648052E-01, 3.40693E+01, 1.4895E+01, 8.5476E+03, &
      8.175768E-01, 3.46737E+01, 1.3782E+01, 7.3994E+03, &
      8.739896E-01, 3.51699E+01, 1.2732E+01, 6.3952E+03, &
      9.342948E-01, 3.55813E+01, 1.1746E+01, 5.5201E+03, &
      9.987612E-01, 3.59549E+01, 1.0827E+01, 4.7606E+03, &
      1.067676E+00, 3.61742E+01, 9.8128E+00, 4.0372E+03, &
      1.141345E+00, 3.62585E+01, 8.9000E+00, 3.4263E+03, &
      1.220098E+00, 3.62574E+01, 8.0802E+00, 2.9109E+03, &
      1.304285E+00, 3.61943E+01, 7.3440E+00, 2.4760E+03, &
      1.394281E+00, 3.60724E+01, 6.6828E+00, 2.1086E+03, &
      1.490486E+00, 3.59002E+01, 6.0889E+00, 1.7981E+03, &
      1.593329E+00, 3.56798E+01, 5.5491E+00, 1.5339E+03, &
      1.703269E+00, 3.53859E+01, 5.0351E+00, 1.3029E+03, &
      1.820795E+00, 3.49914E+01, 4.5682E+00, 1.1067E+03, &
      1.946430E+00, 3.44650E+01, 4.1501E+00, 9.4143E+02, &
      2.080733E+00, 3.37340E+01, 3.7728E+00, 8.0145E+02, &
      2.224304E+00, 3.26163E+01, 3.4311E+00, 6.8266E+02, &
      2.377781E+00, 3.05017E+01, 3.1240E+00, 5.8224E+02, &
      2.469796E+00, 2.75231E+01, 2.9632E+00, 5.3216E+02, &
      2.507599E+00, 2.37090E+01, 2.9015E+00, 5.1341E+02, &
      2.517680E+00, 1.94030E+01, 2.8855E+00, 5.0858E+02, &
      2.532801E+00, 2.35006E+01, 1.1137E+01, 1.9346E+03, &
      2.541848E+00, 2.48690E+01, 1.1065E+01, 1.9152E+03, &
      2.572598E+00, 2.67832E+01, 1.0825E+01, 1.8514E+03, &
      2.611974E+00, 2.66161E+01, 1.0530E+01, 1.7740E+03, &
      2.622475E+00, 2.48332E+01, 1.0453E+01, 1.7541E+03, &
      2.638225E+00, 2.73320E+01, 1.4436E+01, 2.4057E+03, &
      2.677602E+00, 2.99784E+01, 1.4071E+01, 2.3107E+03, &
      2.717235E+00, 3.12991E+01, 1.3727E+01, 2.2215E+03, &
      2.808190E+00, 3.28300E+01, 1.3020E+01, 2.0392E+03, &
      2.851172E+00, 3.27180E+01, 1.2718E+01, 1.9621E+03, &
      2.862634E+00, 3.19251E+01, 1.2640E+01, 1.9423E+03, &
      2.879827E+00, 3.31218E+01, 1.4410E+01, 2.2004E+03, &
      2.904724E+00, 3.40687E+01, 1.4233E+01, 2.1547E+03, &
      2.922810E+00, 3.45276E+01, 1.4106E+01, 2.1225E+03, &
      3.105150E+00, 3.71639E+01, 1.2897E+01, 1.8272E+03, &
      3.319406E+00, 3.88104E+01, 1.1637E+01, 1.5430E+03, &
      3.548445E+00, 3.99027E+01, 1.0453E+01, 1.2972E+03, &
      3.793288E+00, 4.06420E+01, 9.3804E+00, 1.0896E+03, &
      4.055024E+00, 4.11562E+01, 8.4242E+00, 9.1600E+02, &
      4.334821E+00, 4.15188E+01, 7.5706E+00, 7.7062E+02, &
      4.633924E+00, 4.17742E+01, 6.8064E+00, 6.4866E+02, &
      4.953664E+00, 4.19551E+01, 6.1245E+00, 5.4651E+02, &
      5.295467E+00, 4.22145E+01, 5.4907E+00, 4.5882E+02, &
      5.660855E+00, 4.22760E+01, 4.9080E+00, 3.8413E+02, &
      6.051453E+00, 4.23318E+01, 4.3807E+00, 3.2117E+02, &
      6.469004E+00, 4.23136E+01, 3.9113E+00, 2.6866E+02, &
      6.915365E+00, 4.22721E+01, 3.4954E+00, 2.2499E+02, &
      7.392525E+00, 4.22157E+01, 3.1265E+00, 1.8861E+02, &
      7.902609E+00, 4.21495E+01, 2.7961E+00, 1.5812E+02, &
      8.447890E+00, 4.20778E+01, 2.5023E+00, 1.3268E+02, &
      9.030794E+00, 4.20075E+01, 2.2363E+00, 1.1121E+02, &
      9.653919E+00, 4.19204E+01, 1.9837E+00, 9.2557E+01, &
      1.032004E+01, 4.18235E+01, 1.7606E+00, 7.7102E+01, &
      1.103212E+01, 4.17208E+01, 1.5633E+00, 6.4278E+01, &
      1.179334E+01, 4.16110E+01, 1.3840E+00, 5.3456E+01, &
      1.260708E+01, 4.14919E+01, 1.2244E+00, 4.4446E+01, &
      1.347697E+01, 4.13594E+01, 1.0802E+00, 3.6876E+01, &
      1.440688E+01, 4.12086E+01, 9.5392E-01, 3.0640E+01, &
      1.540095E+01, 4.10305E+01, 8.4315E-01, 2.5496E+01, &
      1.646362E+01, 4.08051E+01, 7.4588E-01, 2.1246E+01, &
      1.759961E+01, 4.04829E+01, 6.6041E-01, 1.7732E+01, &
      1.881398E+01, 3.98893E+01, 5.8527E-01, 1.4822E+01, &
      1.959951E+01, 3.89193E+01, 5.4374E-01, 1.3290E+01, &
      1.989950E+01, 3.76005E+01, 5.2913E-01, 1.2766E+01, &
      1.997950E+01, 3.60126E+01, 5.2534E-01, 1.2630E+01, &
      2.011215E+01, 3.77527E+01, 3.6454E+00, 8.0588E+01, &
      2.039949E+01, 3.90911E+01, 3.5517E+00, 7.7437E+01, &
      2.149988E+01, 4.05117E+01, 3.2372E+00, 6.7047E+01, &
      2.298338E+01, 4.12203E+01, 2.9040E+00, 5.6349E+01, &
      2.456923E+01, 4.16227E+01, 2.6105E+00, 4.7460E+01, &
      2.626450E+01, 4.18901E+01, 2.3438E+00, 3.9932E+01, &
      2.807676E+01, 4.20745E+01, 2.0896E+00, 3.3374E+01, &
      3.001405E+01, 4.21944E+01, 1.8604E+00, 2.7860E+01, &
      3.208502E+01, 4.22738E+01, 1.6566E+00, 2.3268E+01, &
      3.429889E+01, 4.23257E+01, 1.4753E+00, 1.9440E+01, &
      3.666551E+01, 4.23582E+01, 1.3140E+00, 1.6248E+01, &
      3.919543E+01, 4.23775E+01, 1.1704E+00, 1.3588E+01, &
      4.189992E+01, 4.24210E+01, 1.0370E+00, 1.1309E+01, &
      4.479101E+01, 4.24178E+01, 9.1661E-01, 9.3968E+00, &
      4.788159E+01, 4.24073E+01, 8.1030E-01, 7.8133E+00, &
      5.118542E+01, 4.23921E+01, 7.1637E-01, 6.5018E+00, &
      5.471721E+01, 4.23741E+01, 6.3337E-01, 5.4151E+00, &
      5.849270E+01, 4.23543E+01, 5.5996E-01, 4.5143E+00, &
      6.252870E+01, 4.23337E+01, 4.9496E-01, 3.7667E+00, &
      6.684318E+01, 4.23128E+01, 4.3732E-01, 3.1456E+00, &
      7.145536E+01, 4.22920E+01, 3.8642E-01, 2.6310E+00, &
      7.638578E+01, 4.22718E+01, 3.4147E-01, 2.2043E+00, &
      8.165640E+01, 4.22527E+01, 3.0134E-01, 1.8483E+00, &
      8.729069E+01, 4.22342E+01, 2.6504E-01, 1.5488E+00, &
      9.331374E+01, 4.22160E+01, 2.3277E-01, 1.2995E+00, &
      9.975239E+01, 4.21984E+01, 2.0445E-01, 1.0939E+00, &
      1.066353E+02, 4.21817E+01, 1.7958E-01, 9.2396E-01, &
      1.139931E+02, 4.21660E+01, 1.5774E-01, 7.8351E-01, &
      1.218587E+02, 4.21514E+01, 1.3856E-01, 6.6732E-01, &
      1.302669E+02, 4.21378E+01, 1.2172E-01, 5.7108E-01, &
      1.392553E+02, 4.21253E+01, 1.0694E-01, 4.9128E-01, &
      1.488640E+02, 4.21137E+01, 9.3950E-02, 4.2503E-01, &
      1.591356E+02, 4.21030E+01, 8.2544E-02, 3.6994E-01, &
      1.701159E+02, 4.20933E+01, 7.2525E-02, 3.2404E-01, &
      1.818539E+02, 4.20843E+01, 6.3725E-02, 2.8573E-01, &
      1.944018E+02, 4.20762E+01, 5.5995E-02, 2.5366E-01, &
      2.078156E+02, 4.20688E+01, 4.9205E-02, 2.2674E-01, &
      2.221548E+02, 4.20620E+01, 4.3239E-02, 2.0407E-01, &
      2.374835E+02, 4.20559E+01, 3.7999E-02, 1.8492E-01, &
      2.538699E+02, 4.20503E+01, 3.3394E-02, 1.6866E-01, &
      2.713869E+02, 4.20453E+01, 2.9348E-02, 1.5478E-01, &
      2.901126E+02, 4.20407E+01, 2.5794E-02, 1.4289E-01, &
      3.101304E+02, 4.20366E+01, 2.2671E-02, 1.3262E-01, &
      3.315294E+02, 4.20328E+01, 1.9926E-02, 1.2372E-01, &
      3.544049E+02, 4.20295E+01, 1.7514E-02, 1.1593E-01, &
      3.788588E+02, 4.20264E+01, 1.5395E-02, 1.0907E-01, &
      4.050001E+02, 4.20237E+01, 1.3533E-02, 1.0299E-01, &
      4.329451E+02, 4.20212E+01, 1.1896E-02, 9.7549E-02]
   case(43)
      frel = -2.2800E-01
      allocate(nist_tab(896))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.594833E-03, 4.82558E+00, 0.0000E+00, 1.3816E-05, &
      6.695774E-03, 4.59453E+00, 0.0000E+00, 1.4971E-05, &
      6.722692E-03, 4.41601E+00, 0.0000E+00, 1.5292E-05, &
      6.763068E-03, 4.50917E+00, 2.8287E-01, 1.7960E+04, &
      6.864010E-03, 4.44636E+00, 2.7410E-01, 1.7147E+04, &
      6.871351E-03, 4.43283E+00, 2.7348E-01, 1.7090E+04, &
      6.976524E-03, 3.99241E+00, 2.6479E-01, 1.6297E+04, &
      7.004571E-03, 3.43742E+00, 2.6254E-01, 1.6094E+04, &
      7.046640E-03, 4.02391E+00, 1.3562E+00, 8.2638E+04, &
      7.151814E-03, 4.52865E+00, 1.3160E+00, 7.9014E+04, &
      1.069000E-02, 3.69822E+00, 1.9224E+00, 7.7220E+04, &
      1.142761E-02, 3.35022E+00, 2.4517E+00, 9.2123E+04, &
      1.221612E-02, 3.04621E+00, 3.1433E+00, 1.1049E+05, &
      1.305903E-02, 2.83967E+00, 4.0047E+00, 1.3168E+05, &
      1.396010E-02, 2.79407E+00, 5.0239E+00, 1.5453E+05, &
      1.492335E-02, 2.97422E+00, 6.1624E+00, 1.7731E+05, &
      1.595306E-02, 3.43337E+00, 7.3533E+00, 1.9792E+05, &
      1.705382E-02, 4.19913E+00, 8.5056E+00, 2.1416E+05, &
      1.823053E-02, 5.26243E+00, 9.5168E+00, 2.2415E+05, &
      1.948844E-02, 6.57362E+00, 1.0290E+01, 2.2673E+05, &
      2.083314E-02, 8.05140E+00, 1.0753E+01, 2.2163E+05, &
      2.227063E-02, 9.59139E+00, 1.0869E+01, 2.0957E+05, &
      2.380730E-02, 1.10679E+01, 1.0643E+01, 1.9197E+05, &
      2.545001E-02, 1.23918E+01, 1.0118E+01, 1.7071E+05, &
      2.720606E-02, 1.34889E+01, 9.3601E+00, 1.4773E+05, &
      2.908327E-02, 1.43111E+01, 8.4491E+00, 1.2474E+05, &
      3.109002E-02, 1.48282E+01, 7.4627E+00, 1.0307E+05, &
      3.323523E-02, 1.50121E+01, 6.4738E+00, 8.3640E+04, &
      3.552846E-02, 1.47684E+01, 5.5364E+00, 6.6912E+04, &
      3.797993E-02, 1.33431E+01, 4.6841E+00, 5.2957E+04, &
      3.812200E-02, 1.31246E+01, 4.6393E+00, 5.2256E+04, &
      3.870550E-02, 1.11702E+01, 4.4606E+00, 4.9484E+04, &
      3.886110E-02, 8.71925E+00, 4.4142E+00, 4.8775E+04, &
      3.909450E-02, 1.14330E+01, 9.2224E+00, 1.0129E+05, &
      3.967800E-02, 1.38226E+01, 8.7692E+00, 9.4900E+04, &
      4.060054E-02, 1.51868E+01, 8.1102E+00, 8.5773E+04, &
      4.340198E-02, 1.65957E+01, 6.4701E+00, 6.4011E+04, &
      4.639671E-02, 1.69305E+01, 5.2237E+00, 4.8345E+04, &
      4.959809E-02, 1.69190E+01, 4.2849E+00, 3.7096E+04, &
      5.302035E-02, 1.67624E+01, 3.5682E+00, 2.8898E+04, &
      5.667876E-02, 1.65406E+01, 3.0159E+00, 2.2848E+04, &
      6.058959E-02, 1.62876E+01, 2.5871E+00, 1.8335E+04, &
      6.477028E-02, 1.60082E+01, 2.2526E+00, 1.4934E+04, &
      6.703200E-02, 1.58314E+01, 2.1100E+00, 1.3516E+04, &
      6.805800E-02, 1.56989E+01, 2.0523E+00, 1.2948E+04, &
      6.833160E-02, 1.55940E+01, 2.0376E+00, 1.2804E+04, &
      6.874200E-02, 1.56614E+01, 2.1929E+00, 1.3698E+04, &
      6.923942E-02, 1.56855E+01, 2.1696E+00, 1.3455E+04, &
      6.976800E-02, 1.56858E+01, 2.1457E+00, 1.3206E+04, &
      7.401695E-02, 1.55655E+01, 1.9838E+00, 1.1508E+04, &
      7.912411E-02, 1.53932E+01, 1.8436E+00, 1.0005E+04, &
      8.458368E-02, 1.52214E+01, 1.7381E+00, 8.8237E+03, &
      9.041995E-02, 1.50539E+01, 1.6596E+00, 7.8815E+03, &
      9.665893E-02, 1.48894E+01, 1.6023E+00, 7.1182E+03, &
      1.033284E-01, 1.47251E+01, 1.5616E+00, 6.4898E+03, &
      1.104581E-01, 1.45568E+01, 1.5338E+00, 5.9629E+03, &
      1.180797E-01, 1.43659E+01, 1.5160E+00, 5.5135E+03, &
      1.262272E-01, 1.41734E+01, 1.5058E+00, 5.1229E+03, &
      1.349368E-01, 1.39586E+01, 1.5076E+00, 4.7982E+03, &
      1.442475E-01, 1.37169E+01, 1.5202E+00, 4.5259E+03, &
      1.542005E-01, 1.34373E+01, 1.5352E+00, 4.2759E+03, &
      1.648404E-01, 1.31040E+01, 1.5516E+00, 4.0427E+03, &
      1.762144E-01, 1.26951E+01, 1.5680E+00, 3.8218E+03, &
      1.883732E-01, 1.21779E+01, 1.5835E+00, 3.6107E+03, &
      2.013709E-01, 1.14995E+01, 1.5974E+00, 3.4075E+03, &
      2.152655E-01, 1.05653E+01, 1.6090E+00, 3.2109E+03, &
      2.301188E-01, 9.14953E+00, 1.6179E+00, 3.0205E+03, &
      2.459970E-01, 6.38418E+00, 1.6235E+00, 2.8356E+03, &
      2.478420E-01, 5.80494E+00, 1.6239E+00, 2.8152E+03, &
      2.512720E-01, 4.08107E+00, 1.6245E+00, 2.7779E+03, &
      2.516355E-01, 3.76123E+00, 1.6246E+00, 2.7740E+03, &
      2.526471E-01, 2.03092E+00, 1.6247E+00, 2.7631E+03, &
      2.541645E-01, 2.86996E+00, 4.4826E+00, 7.5748E+03, &
      2.551180E-01, 2.87946E+00, 4.5451E+00, 7.6517E+03, &
      2.561436E-01, 2.07427E+00, 4.6135E+00, 7.7359E+03, &
      2.576820E-01, 3.14086E+00, 6.6747E+00, 1.1124E+04, &
      2.579580E-01, 3.26901E+00, 6.7057E+00, 1.1164E+04, &
      2.615280E-01, 4.00850E+00, 7.1221E+00, 1.1695E+04, &
      2.629708E-01, 4.13181E+00, 7.2981E+00, 1.1919E+04, &
      2.811158E-01, 4.40739E+00, 9.9312E+00, 1.5171E+04, &
      3.005128E-01, 5.07732E+00, 1.3083E+01, 1.8696E+04, &
      3.212482E-01, 6.59798E+00, 1.6043E+01, 2.1447E+04, &
      3.434143E-01, 8.79609E+00, 1.8465E+01, 2.3091E+04, &
      3.671099E-01, 1.13486E+01, 2.0151E+01, 2.3573E+04, &
      3.924405E-01, 1.38240E+01, 2.1056E+01, 2.3041E+04, &
      4.165000E-01, 1.51673E+01, 2.1258E+01, 2.1919E+04, &
      4.195189E-01, 1.50733E+01, 2.1248E+01, 2.1751E+04, &
      4.228750E-01, 1.45720E+01, 2.1230E+01, 2.1560E+04, &
      4.245750E-01, 1.33707E+01, 2.1217E+01, 2.1461E+04, &
      4.271250E-01, 1.49086E+01, 2.3814E+01, 2.3944E+04, &
      4.335000E-01, 1.65109E+01, 2.3757E+01, 2.3534E+04, &
      4.360020E-01, 1.68695E+01, 2.3727E+01, 2.3372E+04, &
      4.426755E-01, 1.73684E+01, 2.3633E+01, 2.2928E+04, &
      4.444551E-01, 1.70243E+01, 2.3604E+01, 2.2808E+04, &
      4.471245E-01, 1.79400E+01, 2.4693E+01, 2.3717E+04, &
      4.484657E-01, 1.82734E+01, 2.4670E+01, 2.3625E+04, &
      4.537980E-01, 1.92209E+01, 2.4574E+01, 2.3257E+04, &
      4.794098E-01, 2.21872E+01, 2.3978E+01, 2.1481E+04, &
      5.124891E-01, 2.48222E+01, 2.2983E+01, 1.9261E+04, &
      5.366480E-01, 2.60979E+01, 2.2175E+01, 1.7747E+04, &
      5.448620E-01, 2.61559E+01, 2.1894E+01, 1.7258E+04, &
      5.470524E-01, 2.57769E+01, 2.1819E+01, 1.7130E+04, &
      5.478508E-01, 2.55775E+01, 2.2778E+01, 1.7857E+04, &
      5.503380E-01, 2.64692E+01, 2.2693E+01, 1.7710E+04, &
      5.585520E-01, 2.73518E+01, 2.2410E+01, 1.7232E+04, &
      5.856525E-01, 2.90479E+01, 2.1484E+01, 1.5756E+04, &
      6.260625E-01, 3.08296E+01, 2.0157E+01, 1.3830E+04, &
      6.692609E-01, 3.22455E+01, 1.8842E+01, 1.2094E+04, &
      7.154399E-01, 3.34075E+01, 1.7572E+01, 1.0551E+04, &
      7.648052E-01, 3.43822E+01, 1.6319E+01, 9.1673E+03, &
      8.175768E-01, 3.51671E+01, 1.5113E+01, 7.9428E+03, &
      8.739896E-01, 3.57977E+01, 1.3979E+01, 6.8737E+03, &
      9.342948E-01, 3.63196E+01, 1.2921E+01, 5.9441E+03, &
      9.987612E-01, 3.67772E+01, 1.1935E+01, 5.1372E+03, &
      1.067676E+00, 3.71357E+01, 1.0834E+01, 4.3633E+03, &
      1.141345E+00, 3.73028E+01, 9.8127E+00, 3.6979E+03, &
      1.220098E+00, 3.73608E+01, 8.9011E+00, 3.1389E+03, &
      1.304285E+00, 3.73470E+01, 8.0867E+00, 2.6686E+03, &
      1.394281E+00, 3.72693E+01, 7.3578E+00, 2.2723E+03, &
      1.490486E+00, 3.71397E+01, 6.7054E+00, 1.9381E+03, &
      1.593329E+00, 3.69636E+01, 6.1210E+00, 1.6560E+03, &
      1.703269E+00, 3.67440E+01, 5.5959E+00, 1.4171E+03, &
      1.820795E+00, 3.64560E+01, 5.0865E+00, 1.2059E+03, &
      1.946430E+00, 3.60649E+01, 4.6190E+00, 1.0254E+03, &
      2.080733E+00, 3.55373E+01, 4.2000E+00, 8.7303E+02, &
      2.224304E+00, 3.47971E+01, 3.8225E+00, 7.4413E+02, &
      2.377781E+00, 3.36483E+01, 3.4791E+00, 6.3439E+02, &
      2.541848E+00, 3.13904E+01, 3.1701E+00, 5.4152E+02, &
      2.623362E+00, 2.87881E+01, 3.0348E+00, 5.0269E+02, &
      2.663515E+00, 2.50323E+01, 2.9720E+00, 4.8505E+02, &
      2.674223E+00, 2.07808E+01, 2.9556E+00, 4.8050E+02, &
      2.690284E+00, 2.48440E+01, 1.1112E+01, 1.7794E+03, &
      2.717235E+00, 2.75114E+01, 1.0908E+01, 1.7297E+03, &
      2.730438E+00, 2.80698E+01, 1.0811E+01, 1.7060E+03, &
      2.737336E+00, 2.82666E+01, 1.0760E+01, 1.6938E+03, &
      2.779234E+00, 2.80038E+01, 1.0462E+01, 1.6221E+03, &
      2.790407E+00, 2.62259E+01, 1.0384E+01, 1.6038E+03, &
      2.807166E+00, 2.86758E+01, 1.4318E+01, 2.1959E+03, &
      2.849064E+00, 3.12570E+01, 1.3966E+01, 2.1107E+03, &
      2.904724E+00, 3.28473E+01, 1.3532E+01, 2.0062E+03, &
      2.981650E+00, 3.39650E+01, 1.3001E+01, 1.8781E+03, &
      3.027288E+00, 3.38793E+01, 1.2694E+01, 1.8063E+03, &
      3.039458E+00, 3.30922E+01, 1.2611E+01, 1.7873E+03, &
      3.057713E+00, 3.42883E+01, 1.4368E+01, 2.0233E+03, &
      3.105150E+00, 3.57277E+01, 1.4032E+01, 1.9460E+03, &
      3.319406E+00, 3.84624E+01, 1.2651E+01, 1.6419E+03, &
      3.548445E+00, 3.99967E+01, 1.1386E+01, 1.3831E+03, &
      3.793288E+00, 4.10105E+01, 1.0235E+01, 1.1636E+03, &
      4.055024E+00, 4.17156E+01, 9.1995E+00, 9.7906E+02, &
      4.334821E+00, 4.22095E+01, 8.2654E+00, 8.2347E+02, &
      4.633924E+00, 4.25576E+01, 7.4299E+00, 6.9302E+02, &
      4.953664E+00, 4.28030E+01, 6.6808E+00, 5.8345E+02, &
      5.295467E+00, 4.29757E+01, 6.0099E+00, 4.9147E+02, &
      5.660855E+00, 4.32230E+01, 5.3838E+00, 4.1234E+02, &
      6.051453E+00, 4.32768E+01, 4.8130E+00, 3.4528E+02, &
      6.469004E+00, 4.33256E+01, 4.2974E+00, 2.8882E+02, &
      6.915365E+00, 4.33043E+01, 3.8397E+00, 2.4180E+02, &
      7.392525E+00, 4.32619E+01, 3.4340E+00, 2.0266E+02, &
      7.902609E+00, 4.32061E+01, 3.0740E+00, 1.7004E+02, &
      8.447890E+00, 4.31423E+01, 2.7519E+00, 1.4271E+02, &
      9.030794E+00, 4.30782E+01, 2.4588E+00, 1.1957E+02, &
      9.653919E+00, 4.30003E+01, 2.1809E+00, 9.9500E+01, &
      1.032004E+01, 4.29078E+01, 1.9363E+00, 8.2900E+01, &
      1.103212E+01, 4.28080E+01, 1.7198E+00, 6.9119E+01, &
      1.179334E+01, 4.27034E+01, 1.5276E+00, 5.7656E+01, &
      1.260708E+01, 4.25915E+01, 1.3533E+00, 4.7989E+01, &
      1.347697E+01, 4.24704E+01, 1.1976E+00, 3.9919E+01, &
      1.440688E+01, 4.23348E+01, 1.0573E+00, 3.3150E+01, &
      1.540095E+01, 4.21793E+01, 9.3432E-01, 2.7569E+01, &
      1.646362E+01, 4.19933E+01, 8.2640E-01, 2.2961E+01, &
      1.759961E+01, 4.17525E+01, 7.3162E-01, 1.9152E+01, &
      1.881398E+01, 4.13935E+01, 6.4823E-01, 1.5998E+01, &
      2.011215E+01, 4.06536E+01, 5.7486E-01, 1.3385E+01, &
      2.062312E+01, 3.99411E+01, 5.4960E-01, 1.2522E+01, &
      2.093878E+01, 3.86303E+01, 5.3488E-01, 1.2028E+01, &
      2.102296E+01, 3.70523E+01, 5.3106E-01, 1.1901E+01, &
      2.114922E+01, 3.86579E+01, 3.6354E+00, 7.4856E+01, &
      2.146488E+01, 4.01100E+01, 3.5390E+00, 7.1824E+01, &
      2.149988E+01, 4.01953E+01, 3.5286E+00, 7.1498E+01, &
      2.298338E+01, 4.17384E+01, 3.1461E+00, 5.9725E+01, &
      2.456923E+01, 4.23426E+01, 2.8263E+00, 5.0268E+01, &
      2.626450E+01, 4.27056E+01, 2.5402E+00, 4.2334E+01, &
      2.807676E+01, 4.29520E+01, 2.2694E+00, 3.5449E+01, &
      3.001405E+01, 4.31118E+01, 2.0210E+00, 2.9596E+01, &
      3.208502E+01, 4.32180E+01, 1.7999E+00, 2.4719E+01, &
      3.429889E+01, 4.32885E+01, 1.6032E+00, 2.0653E+01, &
      3.666551E+01, 4.33341E+01, 1.4282E+00, 1.7263E+01, &
      3.919543E+01, 4.33624E+01, 1.2725E+00, 1.4436E+01, &
      4.189992E+01, 4.33789E+01, 1.1338E+00, 1.2079E+01, &
      4.479101E+01, 4.34189E+01, 1.0031E+00, 1.0041E+01, &
      4.788159E+01, 4.34135E+01, 8.8705E-01, 8.3490E+00, &
      5.118542E+01, 4.34018E+01, 7.8449E-01, 6.9471E+00, &
      5.471721E+01, 4.33859E+01, 6.9384E-01, 5.7855E+00, &
      5.849270E+01, 4.33675E+01, 6.1369E-01, 4.8226E+00, &
      6.252870E+01, 4.33477E+01, 5.4278E-01, 4.0238E+00, &
      6.684318E+01, 4.33271E+01, 4.7993E-01, 3.3605E+00, &
      7.145536E+01, 4.33064E+01, 4.2419E-01, 2.8093E+00, &
      7.638578E+01, 4.32860E+01, 3.7495E-01, 2.3524E+00, &
      8.165640E+01, 4.32669E+01, 3.3100E-01, 1.9711E+00, &
      8.729069E+01, 4.32479E+01, 2.9131E-01, 1.6508E+00, &
      9.331374E+01, 4.32291E+01, 2.5624E-01, 1.3853E+00, &
      9.975239E+01, 4.32109E+01, 2.2515E-01, 1.1648E+00, &
      1.066353E+02, 4.31935E+01, 1.9785E-01, 9.8270E-01, &
      1.139931E+02, 4.31771E+01, 1.7387E-01, 8.3218E-01, &
      1.218587E+02, 4.31617E+01, 1.5280E-01, 7.0765E-01, &
      1.302669E+02, 4.31474E+01, 1.3429E-01, 6.0456E-01, &
      1.392553E+02, 4.31342E+01, 1.1803E-01, 5.1909E-01, &
      1.488640E+02, 4.31219E+01, 1.0374E-01, 4.4817E-01, &
      1.591356E+02, 4.31106E+01, 9.1183E-02, 3.8922E-01, &
      1.701159E+02, 4.31002E+01, 8.0151E-02, 3.4013E-01, &
      1.818539E+02, 4.30907E+01, 7.0456E-02, 2.9918E-01, &
      1.944018E+02, 4.30820E+01, 6.1936E-02, 2.6494E-01, &
      2.078156E+02, 4.30741E+01, 5.4449E-02, 2.3623E-01, &
      2.221548E+02, 4.30669E+01, 4.7869E-02, 2.1208E-01, &
      2.374835E+02, 4.30603E+01, 4.2085E-02, 1.9169E-01, &
      2.538699E+02, 4.30543E+01, 3.7001E-02, 1.7440E-01, &
      2.713869E+02, 4.30489E+01, 3.2533E-02, 1.5968E-01, &
      2.901126E+02, 4.30440E+01, 2.8605E-02, 1.4708E-01, &
      3.101304E+02, 4.30396E+01, 2.5152E-02, 1.3621E-01, &
      3.315294E+02, 4.30356E+01, 2.2117E-02, 1.2680E-01, &
      3.544049E+02, 4.30319E+01, 1.9448E-02, 1.1859E-01, &
      3.788588E+02, 4.30287E+01, 1.7102E-02, 1.1137E-01, &
      4.050001E+02, 4.30257E+01, 1.5040E-02, 1.0497E-01, &
      4.329451E+02, 4.30230E+01, 1.3226E-02, 9.9250E-02]
   case(44)
      frel = -2.4060E-01
      allocate(nist_tab(856))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.00155E+00, 5.1187E+00, 1.9936E+05, &
      1.142761E-02, 1.21062E+00, 5.4437E+00, 1.9833E+05, &
      1.221612E-02, 1.45119E+00, 5.7730E+00, 1.9675E+05, &
      1.305903E-02, 1.72392E+00, 6.1017E+00, 1.9453E+05, &
      1.396010E-02, 2.02883E+00, 6.4268E+00, 1.9168E+05, &
      1.492335E-02, 2.36557E+00, 6.7445E+00, 1.8816E+05, &
      1.595306E-02, 2.73311E+00, 7.0507E+00, 1.8401E+05, &
      1.705382E-02, 3.12973E+00, 7.3418E+00, 1.7924E+05, &
      1.823053E-02, 3.55307E+00, 7.6143E+00, 1.7390E+05, &
      1.948844E-02, 4.00016E+00, 7.8653E+00, 1.6803E+05, &
      2.083314E-02, 4.46753E+00, 8.0923E+00, 1.6172E+05, &
      2.227063E-02, 4.95122E+00, 8.2934E+00, 1.5505E+05, &
      2.380730E-02, 5.44688E+00, 8.4674E+00, 1.4808E+05, &
      2.545001E-02, 5.94974E+00, 8.6137E+00, 1.4092E+05, &
      2.720606E-02, 6.45448E+00, 8.7324E+00, 1.3364E+05, &
      2.908327E-02, 6.95483E+00, 8.8241E+00, 1.2632E+05, &
      3.109002E-02, 7.44255E+00, 8.8900E+00, 1.1905E+05, &
      3.323523E-02, 7.90470E+00, 8.9319E+00, 1.1189E+05, &
      3.552846E-02, 8.31580E+00, 8.9519E+00, 1.0490E+05, &
      3.797993E-02, 8.60832E+00, 8.9057E+00, 9.7628E+04, &
      4.060054E-02, 8.40350E+00, 8.6080E+00, 8.8273E+04, &
      4.223800E-02, 7.19875E+00, 8.4221E+00, 8.3018E+04, &
      4.288450E-02, 4.95206E+00, 8.3490E+00, 8.1057E+04, &
      4.305690E-02, 1.97649E+00, 8.3296E+00, 8.0545E+04, &
      4.331550E-02, 5.41269E+00, 1.4303E+01, 1.3748E+05, &
      4.340198E-02, 6.14069E+00, 1.4235E+01, 1.3656E+05, &
      4.396200E-02, 8.52148E+00, 1.3811E+01, 1.3080E+05, &
      4.639671E-02, 1.17689E+01, 1.2250E+01, 1.0993E+05, &
      4.959809E-02, 1.32029E+01, 1.0748E+01, 9.0227E+04, &
      5.302035E-02, 1.39004E+01, 9.6507E+00, 7.5783E+04, &
      5.667876E-02, 1.43271E+01, 8.8029E+00, 6.4664E+04, &
      6.058959E-02, 1.46223E+01, 8.1135E+00, 5.5753E+04, &
      6.477028E-02, 1.48381E+01, 7.5299E+00, 4.8403E+04, &
      6.923942E-02, 1.49907E+01, 7.0211E+00, 4.2219E+04, &
      7.340200E-02, 1.50472E+01, 6.6222E+00, 3.7562E+04, &
      7.401695E-02, 1.50352E+01, 6.5681E+00, 3.6946E+04, &
      7.452550E-02, 1.50028E+01, 6.5243E+00, 3.6449E+04, &
      7.482510E-02, 1.49268E+01, 6.4988E+00, 3.6161E+04, &
      7.527450E-02, 1.50174E+01, 6.6227E+00, 3.6630E+04, &
      7.639800E-02, 1.51110E+01, 6.5353E+00, 3.5616E+04, &
      7.912411E-02, 1.52164E+01, 6.3367E+00, 3.3344E+04, &
      8.458368E-02, 1.53454E+01, 5.9877E+00, 2.9474E+04, &
      9.041995E-02, 1.54374E+01, 5.6701E+00, 2.6109E+04, &
      9.665893E-02, 1.55029E+01, 5.3793E+00, 2.3171E+04, &
      1.033284E-01, 1.55443E+01, 5.1122E+00, 2.0599E+04, &
      1.104581E-01, 1.55624E+01, 4.8664E+00, 1.8343E+04, &
      1.180797E-01, 1.55564E+01, 4.6398E+00, 1.6360E+04, &
      1.262272E-01, 1.55248E+01, 4.4308E+00, 1.4615E+04, &
      1.349368E-01, 1.54530E+01, 4.2376E+00, 1.3076E+04, &
      1.442475E-01, 1.53607E+01, 4.0589E+00, 1.1716E+04, &
      1.542005E-01, 1.52304E+01, 3.8933E+00, 1.0513E+04, &
      1.648404E-01, 1.50544E+01, 3.7394E+00, 9.4457E+03, &
      1.762144E-01, 1.48216E+01, 3.5959E+00, 8.4971E+03, &
      1.883732E-01, 1.45159E+01, 3.4611E+00, 7.6509E+03, &
      2.013709E-01, 1.41129E+01, 3.3336E+00, 6.8938E+03, &
      2.152655E-01, 1.35743E+01, 3.2125E+00, 6.2147E+03, &
      2.301188E-01, 1.28364E+01, 3.0966E+00, 5.6042E+03, &
      2.459970E-01, 1.17634E+01, 2.9852E+00, 5.0543E+03, &
      2.629708E-01, 9.99936E+00, 2.8775E+00, 4.5579E+03, &
      2.738120E-01, 7.92428E+00, 2.8139E+00, 4.2808E+03, &
      2.779280E-01, 6.16444E+00, 2.7907E+00, 4.1826E+03, &
      2.780030E-01, 6.10788E+00, 2.7902E+00, 4.1809E+03, &
      2.791206E-01, 4.61041E+00, 2.7840E+00, 4.1548E+03, &
      2.807970E-01, 5.30456E+00, 5.2239E+00, 7.7478E+03, &
      2.811158E-01, 5.34954E+00, 5.2405E+00, 7.7636E+03, &
      2.821820E-01, 5.26510E+00, 5.2969E+00, 7.8176E+03, &
      2.833164E-01, 4.50713E+00, 5.3584E+00, 7.8767E+03, &
      2.850180E-01, 5.36581E+00, 7.1567E+00, 1.0456E+04, &
      2.892720E-01, 5.99852E+00, 7.5805E+00, 1.0913E+04, &
      3.005128E-01, 6.05492E+00, 8.8910E+00, 1.2320E+04, &
      3.212482E-01, 5.99312E+00, 1.1975E+01, 1.5523E+04, &
      3.434143E-01, 6.94055E+00, 1.5257E+01, 1.8501E+04, &
      3.671099E-01, 8.82922E+00, 1.8158E+01, 2.0597E+04, &
      3.924405E-01, 1.13403E+01, 2.0327E+01, 2.1568E+04, &
      4.195189E-01, 1.39957E+01, 2.1614E+01, 2.1453E+04, &
      4.484657E-01, 1.59267E+01, 2.2049E+01, 2.0474E+04, &
      4.513880E-01, 1.59534E+01, 2.2050E+01, 2.0343E+04, &
      4.582970E-01, 1.53295E+01, 2.2028E+01, 2.0016E+04, &
      4.601394E-01, 1.40529E+01, 2.2017E+01, 1.9926E+04, &
      4.629030E-01, 1.56982E+01, 2.4785E+01, 2.2297E+04, &
      4.698120E-01, 1.74209E+01, 2.4722E+01, 2.1913E+04, &
      4.731440E-01, 1.78891E+01, 2.4682E+01, 2.1723E+04, &
      4.794098E-01, 1.84095E+01, 2.4590E+01, 2.1360E+04, &
      4.803860E-01, 1.84097E+01, 2.4574E+01, 2.1302E+04, &
      4.823172E-01, 1.80427E+01, 2.4541E+01, 2.1189E+04, &
      4.852140E-01, 1.90159E+01, 2.5694E+01, 2.2051E+04, &
      4.924560E-01, 2.03756E+01, 2.5556E+01, 2.1610E+04, &
      5.124891E-01, 2.27608E+01, 2.5078E+01, 2.0378E+04, &
      5.478508E-01, 2.56936E+01, 2.4006E+01, 1.8248E+04, &
      5.733000E-01, 2.70826E+01, 2.3139E+01, 1.6809E+04, &
      5.820750E-01, 2.71746E+01, 2.2832E+01, 1.6336E+04, &
      5.844150E-01, 2.68006E+01, 2.2750E+01, 1.6213E+04, &
      5.856525E-01, 2.69064E+01, 2.3702E+01, 1.6855E+04, &
      5.879250E-01, 2.75104E+01, 2.3622E+01, 1.6734E+04, &
      5.967000E-01, 2.84278E+01, 2.3314E+01, 1.6273E+04, &
      6.260625E-01, 3.02296E+01, 2.2293E+01, 1.4830E+04, &
      6.692609E-01, 3.20850E+01, 2.0854E+01, 1.2978E+04, &
      7.154399E-01, 3.35416E+01, 1.9435E+01, 1.1315E+04, &
      7.648052E-01, 3.47170E+01, 1.8073E+01, 9.8440E+03, &
      8.175768E-01, 3.56847E+01, 1.6761E+01, 8.5410E+03, &
      8.739896E-01, 3.64799E+01, 1.5496E+01, 7.3880E+03, &
      9.342948E-01, 3.71117E+01, 1.4312E+01, 6.3839E+03, &
      9.987612E-01, 3.76704E+01, 1.3210E+01, 5.5128E+03, &
      1.067676E+00, 3.80802E+01, 1.2096E+01, 4.7230E+03, &
      1.141345E+00, 3.83830E+01, 1.1038E+01, 4.0329E+03, &
      1.220098E+00, 3.85496E+01, 9.9896E+00, 3.4153E+03, &
      1.304285E+00, 3.86027E+01, 9.0550E+00, 2.8970E+03, &
      1.394281E+00, 3.85796E+01, 8.2212E+00, 2.4615E+03, &
      1.490486E+00, 3.84965E+01, 7.4755E+00, 2.0948E+03, &
      1.593329E+00, 3.83601E+01, 6.8087E+00, 1.7858E+03, &
      1.703269E+00, 3.81767E+01, 6.2118E+00, 1.5250E+03, &
      1.820795E+00, 3.79482E+01, 5.6769E+00, 1.3047E+03, &
      1.946430E+00, 3.76565E+01, 5.1681E+00, 1.1120E+03, &
      2.080733E+00, 3.72619E+01, 4.6920E+00, 9.4534E+02, &
      2.224304E+00, 3.67266E+01, 4.2654E+00, 8.0481E+02, &
      2.377781E+00, 3.59697E+01, 3.8813E+00, 6.8591E+02, &
      2.541848E+00, 3.47747E+01, 3.5320E+00, 5.8473E+02, &
      2.717235E+00, 3.22942E+01, 3.2179E+00, 4.9912E+02, &
      2.781142E+00, 3.02369E+01, 3.1159E+00, 4.7247E+02, &
      2.823710E+00, 2.65904E+01, 3.0513E+00, 4.5588E+02, &
      2.835062E+00, 2.24727E+01, 3.0344E+00, 4.5160E+02, &
      2.852089E+00, 2.63780E+01, 1.0932E+01, 1.6019E+03, &
      2.894658E+00, 2.95073E+01, 1.0698E+01, 1.5446E+03, &
      2.904724E+00, 2.97685E+01, 1.0643E+01, 1.5315E+03, &
      2.907562E+00, 2.98248E+01, 1.0628E+01, 1.5278E+03, &
      2.952066E+00, 2.94490E+01, 1.0395E+01, 1.4719E+03, &
      2.963933E+00, 2.76815E+01, 1.0334E+01, 1.4575E+03, &
      2.981735E+00, 2.99606E+01, 1.4292E+01, 2.0015E+03, &
      3.026238E+00, 3.27712E+01, 1.4151E+01, 1.9527E+03, &
      3.105150E+00, 3.49892E+01, 1.3295E+01, 1.7884E+03, &
      3.159520E+00, 3.56507E+01, 1.2738E+01, 1.6842E+03, &
      3.207880E+00, 3.55236E+01, 1.2272E+01, 1.5985E+03, &
      3.220776E+00, 3.47222E+01, 1.2154E+01, 1.5768E+03, &
      3.240120E+00, 3.58840E+01, 1.3854E+01, 1.7860E+03, &
      3.288480E+00, 3.71842E+01, 1.3431E+01, 1.7061E+03, &
      3.319406E+00, 3.76671E+01, 1.3184E+01, 1.6592E+03, &
      3.548445E+00, 3.95868E+01, 1.1858E+01, 1.3968E+03, &
      3.793288E+00, 4.08496E+01, 1.1047E+01, 1.2178E+03, &
      4.055024E+00, 4.20086E+01, 1.0035E+01, 1.0354E+03, &
      4.334821E+00, 4.27363E+01, 9.0167E+00, 8.7090E+02, &
      4.633924E+00, 4.32299E+01, 8.0998E+00, 7.3242E+02, &
      4.953664E+00, 4.35730E+01, 7.2799E+00, 6.1633E+02, &
      5.295467E+00, 4.38127E+01, 6.5464E+00, 5.1897E+02, &
      5.660855E+00, 4.39808E+01, 5.8883E+00, 4.3715E+02, &
      6.051453E+00, 4.42178E+01, 5.2718E+00, 3.6658E+02, &
      6.469004E+00, 4.43028E+01, 4.7148E+00, 3.0712E+02, &
      6.915365E+00, 4.43118E+01, 4.2116E+00, 2.5704E+02, &
      7.392525E+00, 4.42900E+01, 3.7656E+00, 2.1536E+02, &
      7.902609E+00, 4.42488E+01, 3.3700E+00, 1.8064E+02, &
      8.447890E+00, 4.41964E+01, 3.0188E+00, 1.5169E+02, &
      9.030794E+00, 4.41479E+01, 2.6988E+00, 1.2715E+02, &
      9.653919E+00, 4.40745E+01, 2.3932E+00, 1.0577E+02, &
      1.032004E+01, 4.39861E+01, 2.1242E+00, 8.8090E+01, &
      1.103212E+01, 4.38895E+01, 1.8872E+00, 7.3457E+01, &
      1.179334E+01, 4.37875E+01, 1.6774E+00, 6.1300E+01, &
      1.260708E+01, 4.36812E+01, 1.4903E+00, 5.1157E+01, &
      1.347697E+01, 4.35675E+01, 1.3210E+00, 4.2613E+01, &
      1.440688E+01, 4.34440E+01, 1.1691E+00, 3.5458E+01, &
      1.540095E+01, 4.33048E+01, 1.0329E+00, 2.9473E+01, &
      1.646362E+01, 4.31438E+01, 9.1336E-01, 2.4533E+01, &
      1.759961E+01, 4.29479E+01, 8.0840E-01, 2.0451E+01, &
      1.881398E+01, 4.26871E+01, 7.1616E-01, 1.7074E+01, &
      2.011215E+01, 4.22765E+01, 6.3499E-01, 1.4277E+01, &
      2.149988E+01, 4.12619E+01, 5.6348E-01, 1.1956E+01, &
      2.167486E+01, 4.09604E+01, 5.5540E-01, 1.1702E+01, &
      2.200661E+01, 3.96567E+01, 5.4057E-01, 1.1242E+01, &
      2.209508E+01, 3.80877E+01, 5.3673E-01, 1.1124E+01, &
      2.222779E+01, 3.96841E+01, 3.6233E+00, 6.8870E+01, &
      2.255954E+01, 4.11276E+01, 3.5276E+00, 6.6087E+01, &
      2.298338E+01, 4.18441E+01, 3.4109E+00, 6.2751E+01, &
      2.456923E+01, 4.29361E+01, 3.0547E+00, 5.2652E+01, &
      2.626450E+01, 4.34577E+01, 2.7483E+00, 4.4384E+01, &
      2.807676E+01, 4.37905E+01, 2.4608E+00, 3.7244E+01, &
      3.001405E+01, 4.40056E+01, 2.1940E+00, 3.1127E+01, &
      3.208502E+01, 4.41468E+01, 1.9540E+00, 2.5994E+01, &
      3.429889E+01, 4.42409E+01, 1.7404E+00, 2.1716E+01, &
      3.666551E+01, 4.43031E+01, 1.5504E+00, 1.8148E+01, &
      3.919543E+01, 4.43430E+01, 1.3813E+00, 1.5173E+01, &
      4.189992E+01, 4.43672E+01, 1.2307E+00, 1.2692E+01, &
      4.479101E+01, 4.44150E+01, 1.0952E+00, 1.0608E+01, &
      4.788159E+01, 4.44167E+01, 9.6878E-01, 8.8208E+00, &
      5.118542E+01, 4.44093E+01, 8.5705E-01, 7.3395E+00, &
      5.471721E+01, 4.43964E+01, 7.5827E-01, 6.1119E+00, &
      5.849270E+01, 4.43798E+01, 6.7093E-01, 5.0943E+00, &
      6.252870E+01, 4.43612E+01, 5.9365E-01, 4.2502E+00, &
      6.684318E+01, 4.43413E+01, 5.2527E-01, 3.5498E+00, &
      7.145536E+01, 4.43209E+01, 4.6455E-01, 2.9673E+00, &
      7.638578E+01, 4.43008E+01, 4.1073E-01, 2.4834E+00, &
      8.165640E+01, 4.42821E+01, 3.6268E-01, 2.0796E+00, &
      8.729069E+01, 4.42620E+01, 3.1930E-01, 1.7404E+00, &
      9.331374E+01, 4.42424E+01, 2.8113E-01, 1.4599E+00, &
      9.975239E+01, 4.42236E+01, 2.4728E-01, 1.2270E+00, &
      1.066353E+02, 4.42056E+01, 2.1738E-01, 1.0339E+00, &
      1.139931E+02, 4.41885E+01, 1.9110E-01, 8.7429E-01, &
      1.218587E+02, 4.41724E+01, 1.6801E-01, 7.4224E-01, &
      1.302669E+02, 4.41573E+01, 1.4771E-01, 6.3291E-01, &
      1.392553E+02, 4.41433E+01, 1.2987E-01, 5.4229E-01, &
      1.488640E+02, 4.41304E+01, 1.1419E-01, 4.6711E-01, &
      1.591356E+02, 4.41184E+01, 1.0041E-01, 4.0463E-01, &
      1.701159E+02, 4.41074E+01, 8.8295E-02, 3.5264E-01, &
      1.818539E+02, 4.40973E+01, 7.7645E-02, 3.0929E-01, &
      1.944018E+02, 4.40881E+01, 6.8282E-02, 2.7306E-01, &
      2.078156E+02, 4.40796E+01, 6.0050E-02, 2.4271E-01, &
      2.221548E+02, 4.40719E+01, 5.2813E-02, 2.1721E-01, &
      2.374835E+02, 4.40649E+01, 4.6449E-02, 1.9570E-01, &
      2.538699E+02, 4.40585E+01, 4.0854E-02, 1.7751E-01, &
      2.713869E+02, 4.40527E+01, 3.5934E-02, 1.6205E-01, &
      2.901126E+02, 4.40475E+01, 3.1607E-02, 1.4883E-01, &
      3.101304E+02, 4.40427E+01, 2.7802E-02, 1.3748E-01, &
      3.315294E+02, 4.40384E+01, 2.4456E-02, 1.2767E-01, &
      3.544049E+02, 4.40345E+01, 2.1513E-02, 1.1914E-01, &
      3.788588E+02, 4.40310E+01, 1.8925E-02, 1.1166E-01, &
      4.050001E+02, 4.40278E+01, 1.6649E-02, 1.0507E-01, &
      4.329451E+02, 4.40250E+01, 1.4647E-02, 9.9200E-02]
   case(45)
      frel = -2.5440E-01
      allocate(nist_tab(852))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.89417E+00, 4.8055E+00, 1.8382E+05, &
      1.142761E-02, 2.09662E+00, 5.1028E+00, 1.8260E+05, &
      1.221612E-02, 2.32705E+00, 5.4019E+00, 1.8082E+05, &
      1.305903E-02, 2.58575E+00, 5.6998E+00, 1.7848E+05, &
      1.396010E-02, 2.87263E+00, 5.9931E+00, 1.7555E+05, &
      1.492335E-02, 3.18714E+00, 6.2786E+00, 1.7204E+05, &
      1.595306E-02, 3.52819E+00, 6.5531E+00, 1.6797E+05, &
      1.705382E-02, 3.89415E+00, 6.8134E+00, 1.6337E+05, &
      1.823053E-02, 4.28288E+00, 7.0566E+00, 1.5828E+05, &
      1.948844E-02, 4.69173E+00, 7.2804E+00, 1.5276E+05, &
      2.083314E-02, 5.11762E+00, 7.4824E+00, 1.4687E+05, &
      2.227063E-02, 5.55710E+00, 7.6611E+00, 1.4067E+05, &
      2.380730E-02, 6.00634E+00, 7.8152E+00, 1.3424E+05, &
      2.545001E-02, 6.46121E+00, 7.9443E+00, 1.2765E+05, &
      2.720606E-02, 6.91721E+00, 8.0481E+00, 1.2097E+05, &
      2.908327E-02, 7.36933E+00, 8.1270E+00, 1.1427E+05, &
      3.109002E-02, 7.81174E+00, 8.1819E+00, 1.0762E+05, &
      3.323523E-02, 8.23697E+00, 8.2140E+00, 1.0106E+05, &
      3.552846E-02, 8.63416E+00, 8.2249E+00, 9.4666E+04, &
      3.797993E-02, 8.98459E+00, 8.2164E+00, 8.8465E+04, &
      4.060054E-02, 9.24829E+00, 8.1907E+00, 8.2495E+04, &
      4.340198E-02, 9.31015E+00, 8.1499E+00, 7.6786E+04, &
      4.639671E-02, 8.53632E+00, 8.0962E+00, 7.1357E+04, &
      4.694200E-02, 8.00116E+00, 8.0857E+00, 7.0436E+04, &
      4.766050E-02, 5.95081E+00, 8.0228E+00, 6.8835E+04, &
      4.785210E-02, 3.21176E+00, 8.0060E+00, 6.8416E+04, &
      4.813950E-02, 6.39978E+00, 1.3516E+01, 1.1482E+05, &
      4.885800E-02, 9.28778E+00, 1.3044E+01, 1.0917E+05, &
      4.959809E-02, 1.05693E+01, 1.2595E+01, 1.0384E+05, &
      5.302035E-02, 1.29851E+01, 1.0918E+01, 8.4209E+04, &
      5.667876E-02, 1.38997E+01, 9.7195E+00, 7.0124E+04, &
      6.058959E-02, 1.44010E+01, 8.8533E+00, 5.9751E+04, &
      6.477028E-02, 1.47360E+01, 8.1819E+00, 5.1656E+04, &
      6.923942E-02, 1.49838E+01, 7.6317E+00, 4.5072E+04, &
      7.401695E-02, 1.51694E+01, 7.1613E+00, 3.9564E+04, &
      7.912411E-02, 1.52714E+01, 6.7471E+00, 3.4870E+04, &
      7.938000E-02, 1.52708E+01, 6.7282E+00, 3.4660E+04, &
      8.059500E-02, 1.52279E+01, 6.6405E+00, 3.3693E+04, &
      8.091900E-02, 1.51480E+01, 6.6177E+00, 3.3443E+04, &
      8.140500E-02, 1.52468E+01, 6.7567E+00, 3.3941E+04, &
      8.262000E-02, 1.53518E+01, 6.6791E+00, 3.3058E+04, &
      8.458368E-02, 1.54415E+01, 6.5600E+00, 3.1715E+04, &
      9.041995E-02, 1.56159E+01, 6.2449E+00, 2.8242E+04, &
      9.665893E-02, 1.57489E+01, 5.9577E+00, 2.5205E+04, &
      1.033284E-01, 1.58551E+01, 5.6935E+00, 2.2532E+04, &
      1.104581E-01, 1.59375E+01, 5.4491E+00, 2.0173E+04, &
      1.180797E-01, 1.59967E+01, 5.2222E+00, 1.8085E+04, &
      1.262272E-01, 1.60318E+01, 5.0111E+00, 1.6235E+04, &
      1.349368E-01, 1.60413E+01, 4.8142E+00, 1.4590E+04, &
      1.442475E-01, 1.60170E+01, 4.6301E+00, 1.3127E+04, &
      1.542005E-01, 1.59658E+01, 4.4576E+00, 1.1822E+04, &
      1.648404E-01, 1.58769E+01, 4.2954E+00, 1.0657E+04, &
      1.762144E-01, 1.57426E+01, 4.1424E+00, 9.6138E+03, &
      1.883732E-01, 1.55526E+01, 3.9976E+00, 8.6791E+03, &
      2.013709E-01, 1.52919E+01, 3.8596E+00, 7.8389E+03, &
      2.152655E-01, 1.49381E+01, 3.7268E+00, 7.0809E+03, &
      2.301188E-01, 1.44564E+01, 3.5984E+00, 6.3960E+03, &
      2.459970E-01, 1.37917E+01, 3.4736E+00, 5.7760E+03, &
      2.629708E-01, 1.28315E+01, 3.3517E+00, 5.2140E+03, &
      2.811158E-01, 1.13237E+01, 3.2323E+00, 4.7039E+03, &
      3.005128E-01, 8.15945E+00, 3.1149E+00, 4.2408E+03, &
      3.008600E-01, 8.05768E+00, 3.1128E+00, 4.2333E+03, &
      3.054660E-01, 5.95237E+00, 3.0864E+00, 4.1341E+03, &
      3.066930E-01, 4.17242E+00, 3.0794E+00, 4.1082E+03, &
      3.085350E-01, 5.06359E+00, 6.0229E+00, 7.9849E+03, &
      3.101415E-01, 5.04698E+00, 6.1078E+00, 8.0556E+03, &
      3.113883E-01, 4.14952E+00, 6.1755E+00, 8.1123E+03, &
      3.132585E-01, 5.23483E+00, 8.3433E+00, 1.0893E+04, &
      3.179340E-01, 6.10712E+00, 8.8068E+00, 1.1330E+04, &
      3.212482E-01, 6.30213E+00, 9.1580E+00, 1.1660E+04, &
      3.434143E-01, 6.53387E+00, 1.2030E+01, 1.4328E+04, &
      3.671099E-01, 7.46300E+00, 1.5371E+01, 1.7124E+04, &
      3.924405E-01, 9.35507E+00, 1.8361E+01, 1.9135E+04, &
      4.195189E-01, 1.19079E+01, 2.0619E+01, 2.0102E+04, &
      4.484657E-01, 1.46475E+01, 2.1977E+01, 2.0043E+04, &
      4.794098E-01, 1.67922E+01, 2.2452E+01, 1.9155E+04, &
      4.862760E-01, 1.69162E+01, 2.2452E+01, 1.8884E+04, &
      4.937190E-01, 1.62427E+01, 2.2416E+01, 1.8570E+04, &
      4.957038E-01, 1.48958E+01, 2.2401E+01, 1.8483E+04, &
      4.986810E-01, 1.66220E+01, 2.5308E+01, 2.0757E+04, &
      5.061240E-01, 1.84232E+01, 2.5227E+01, 2.0386E+04, &
      5.105800E-01, 1.90150E+01, 2.5166E+01, 2.0159E+04, &
      5.124891E-01, 1.92134E+01, 2.5137E+01, 2.0061E+04, &
      5.183950E-01, 1.95194E+01, 2.5037E+01, 1.9753E+04, &
      5.204790E-01, 1.91250E+01, 2.4998E+01, 1.9644E+04, &
      5.236050E-01, 2.01348E+01, 2.6203E+01, 2.0468E+04, &
      5.314200E-01, 2.15333E+01, 2.6042E+01, 2.0043E+04, &
      5.478508E-01, 2.34547E+01, 2.5646E+01, 1.9147E+04, &
      5.856525E-01, 2.65544E+01, 2.4527E+01, 1.7131E+04, &
      6.145580E-01, 2.80923E+01, 2.3568E+01, 1.5687E+04, &
      6.239645E-01, 2.81885E+01, 2.3249E+01, 1.5241E+04, &
      6.260625E-01, 2.79470E+01, 2.3177E+01, 1.5144E+04, &
      6.264729E-01, 2.78066E+01, 2.3163E+01, 1.5125E+04, &
      6.302355E-01, 2.85331E+01, 2.4053E+01, 1.5611E+04, &
      6.396420E-01, 2.94719E+01, 2.3731E+01, 1.5176E+04, &
      6.692609E-01, 3.12211E+01, 2.2730E+01, 1.3893E+04, &
      7.154399E-01, 3.31285E+01, 2.1237E+01, 1.2143E+04, &
      7.648052E-01, 3.46156E+01, 1.9768E+01, 1.0576E+04, &
      8.175768E-01, 3.58079E+01, 1.8361E+01, 9.1892E+03, &
      8.739896E-01, 3.67848E+01, 1.7016E+01, 7.9676E+03, &
      9.342948E-01, 3.75849E+01, 1.5729E+01, 6.8904E+03, &
      9.987612E-01, 3.82442E+01, 1.4525E+01, 5.9534E+03, &
      1.067676E+00, 3.87727E+01, 1.3315E+01, 5.1061E+03, &
      1.141345E+00, 3.91539E+01, 1.2186E+01, 4.3724E+03, &
      1.220098E+00, 3.94325E+01, 1.1140E+01, 3.7403E+03, &
      1.304285E+00, 3.96085E+01, 1.0098E+01, 3.1728E+03, &
      1.394281E+00, 3.96637E+01, 9.1498E+00, 2.6903E+03, &
      1.490486E+00, 3.96411E+01, 8.3038E+00, 2.2850E+03, &
      1.593329E+00, 3.95546E+01, 7.5479E+00, 1.9439E+03, &
      1.703269E+00, 3.94114E+01, 6.8721E+00, 1.6567E+03, &
      1.820795E+00, 3.92194E+01, 6.2674E+00, 1.4144E+03, &
      1.946430E+00, 3.89783E+01, 5.7258E+00, 1.2097E+03, &
      2.080733E+00, 3.86746E+01, 5.2173E+00, 1.0320E+03, &
      2.224304E+00, 3.82656E+01, 4.7367E+00, 8.7742E+02, &
      2.377781E+00, 3.77071E+01, 4.3058E+00, 7.4700E+02, &
      2.541848E+00, 3.69090E+01, 3.9180E+00, 6.3672E+02, &
      2.717235E+00, 3.56223E+01, 3.5657E+00, 5.4288E+02, &
      2.904724E+00, 3.27378E+01, 3.2488E+00, 4.6348E+02, &
      2.943724E+00, 3.14079E+01, 3.1893E+00, 4.4913E+02, &
      2.988781E+00, 2.78447E+01, 3.1231E+00, 4.3336E+02, &
      3.000796E+00, 2.38203E+01, 3.1056E+00, 4.2925E+02, &
      3.018819E+00, 2.76279E+01, 1.0824E+01, 1.4722E+03, &
      3.063876E+00, 3.07009E+01, 1.0604E+01, 1.4212E+03, &
      3.083178E+00, 3.11221E+01, 1.0512E+01, 1.4002E+03, &
      3.105150E+00, 3.12624E+01, 1.0409E+01, 1.3767E+03, &
      3.130370E+00, 3.07066E+01, 1.0293E+01, 1.3505E+03, &
      3.142954E+00, 2.89990E+01, 1.0236E+01, 1.3377E+03, &
      3.161831E+00, 3.12954E+01, 1.3993E+01, 1.8157E+03, &
      3.209022E+00, 3.37259E+01, 1.3691E+01, 1.7506E+03, &
      3.319406E+00, 3.59162E+01, 1.3008E+01, 1.6082E+03, &
      3.343662E+00, 3.61407E+01, 1.2857E+01, 1.5782E+03, &
      3.394841E+00, 3.60599E+01, 1.2545E+01, 1.5168E+03, &
      3.408488E+00, 3.52798E+01, 1.2463E+01, 1.5009E+03, &
      3.428960E+00, 3.64699E+01, 1.4209E+01, 1.7002E+03, &
      3.480138E+00, 3.78650E+01, 1.3890E+01, 1.6377E+03, &
      3.548445E+00, 3.89158E+01, 1.3477E+01, 1.5587E+03, &
      3.793288E+00, 4.11094E+01, 1.2138E+01, 1.3139E+03, &
      4.055024E+00, 4.24388E+01, 1.0918E+01, 1.1062E+03, &
      4.334821E+00, 4.33289E+01, 9.8125E+00, 9.3069E+02, &
      4.633924E+00, 4.39500E+01, 8.8157E+00, 7.8279E+02, &
      4.953664E+00, 4.43826E+01, 7.9204E+00, 6.5845E+02, &
      5.295467E+00, 4.46862E+01, 7.1193E+00, 5.5418E+02, &
      5.660855E+00, 4.48983E+01, 6.4020E+00, 4.6667E+02, &
      6.051453E+00, 4.51300E+01, 5.7551E+00, 3.9290E+02, &
      6.469004E+00, 4.52610E+01, 5.1523E+00, 3.2949E+02, &
      6.915365E+00, 4.53329E+01, 4.6094E+00, 2.7616E+02, &
      7.392525E+00, 4.53324E+01, 4.1209E+00, 2.3134E+02, &
      7.902609E+00, 4.53055E+01, 3.6875E+00, 1.9401E+02, &
      8.447890E+00, 4.52629E+01, 3.3029E+00, 1.6289E+02, &
      9.030794E+00, 4.52202E+01, 2.9540E+00, 1.3658E+02, &
      9.653919E+00, 4.51573E+01, 2.6207E+00, 1.1366E+02, &
      1.032004E+01, 4.50732E+01, 2.3256E+00, 9.4622E+01, &
      1.103212E+01, 4.49788E+01, 2.0656E+00, 7.8873E+01, &
      1.179334E+01, 4.48783E+01, 1.8364E+00, 6.5829E+01, &
      1.260708E+01, 4.47736E+01, 1.6333E+00, 5.4982E+01, &
      1.347697E+01, 4.46648E+01, 1.4515E+00, 4.5902E+01, &
      1.440688E+01, 4.45483E+01, 1.2873E+00, 3.8263E+01, &
      1.540095E+01, 4.44213E+01, 1.1391E+00, 3.1845E+01, &
      1.646362E+01, 4.42772E+01, 1.0071E+00, 2.6493E+01, &
      1.759961E+01, 4.41089E+01, 8.9111E-01, 2.2073E+01, &
      1.881398E+01, 4.39001E+01, 7.8923E-01, 1.8417E+01, &
      2.011215E+01, 4.36125E+01, 6.9962E-01, 1.5390E+01, &
      2.149988E+01, 4.31259E+01, 6.2075E-01, 1.2881E+01, &
      2.275550E+01, 4.19842E+01, 5.6105E-01, 1.1084E+01, &
      2.298338E+01, 4.13616E+01, 5.5121E-01, 1.0797E+01, &
      2.310380E+01, 4.06879E+01, 5.4612E-01, 1.0650E+01, &
      2.319668E+01, 3.91279E+01, 5.4225E-01, 1.0538E+01, &
      2.333600E+01, 4.07147E+01, 3.6109E+00, 6.4245E+01, &
      2.368430E+01, 4.21495E+01, 3.5161E+00, 6.1662E+01, &
      2.456923E+01, 4.32772E+01, 3.2962E+00, 5.5773E+01, &
      2.626450E+01, 4.41227E+01, 2.9629E+00, 4.6970E+01, &
      2.807676E+01, 4.45784E+01, 2.6567E+00, 3.9467E+01, &
      3.001405E+01, 4.48642E+01, 2.3762E+00, 3.3086E+01, &
      3.208502E+01, 4.50536E+01, 2.1169E+00, 2.7634E+01, &
      3.429889E+01, 4.51785E+01, 1.8854E+00, 2.3081E+01, &
      3.666551E+01, 4.52617E+01, 1.6795E+00, 1.9287E+01, &
      3.919543E+01, 4.53162E+01, 1.4962E+00, 1.6123E+01, &
      4.189992E+01, 4.53506E+01, 1.3331E+00, 1.3483E+01, &
      4.479101E+01, 4.53709E+01, 1.1878E+00, 1.1283E+01, &
      4.788159E+01, 4.54151E+01, 1.0551E+00, 9.4161E+00, &
      5.118542E+01, 4.54136E+01, 9.3370E-01, 7.8348E+00, &
      5.471721E+01, 4.54044E+01, 8.2632E-01, 6.5241E+00, &
      5.849270E+01, 4.53903E+01, 7.3135E-01, 5.4373E+00, &
      6.252870E+01, 4.53732E+01, 6.4735E-01, 4.5359E+00, &
      6.684318E+01, 4.53543E+01, 5.7299E-01, 3.7879E+00, &
      7.145536E+01, 4.53345E+01, 5.0717E-01, 3.1669E+00, &
      7.638578E+01, 4.53146E+01, 4.4872E-01, 2.6504E+00, &
      8.165640E+01, 4.52951E+01, 3.9644E-01, 2.2187E+00, &
      8.729069E+01, 4.52749E+01, 3.4922E-01, 1.8560E+00, &
      9.331374E+01, 4.52549E+01, 3.0765E-01, 1.5561E+00, &
      9.975239E+01, 4.52356E+01, 2.7104E-01, 1.3080E+00, &
      1.066353E+02, 4.52171E+01, 2.3849E-01, 1.1014E+00, &
      1.139931E+02, 4.51994E+01, 2.0982E-01, 9.3047E-01, &
      1.218587E+02, 4.51827E+01, 1.8461E-01, 7.8897E-01, &
      1.302669E+02, 4.51670E+01, 1.6243E-01, 6.7177E-01, &
      1.392553E+02, 4.51524E+01, 1.4293E-01, 5.7461E-01, &
      1.488640E+02, 4.51388E+01, 1.2577E-01, 4.9397E-01, &
      1.591356E+02, 4.51262E+01, 1.1068E-01, 4.2696E-01, &
      1.701159E+02, 4.51146E+01, 9.7405E-02, 3.7119E-01, &
      1.818539E+02, 4.51039E+01, 8.5725E-02, 3.2470E-01, &
      1.944018E+02, 4.50942E+01, 7.5450E-02, 2.8586E-01, &
      2.078156E+02, 4.50852E+01, 6.6408E-02, 2.5334E-01, &
      2.221548E+02, 4.50771E+01, 5.8453E-02, 2.2603E-01, &
      2.374835E+02, 4.50696E+01, 5.1453E-02, 2.0304E-01, &
      2.538699E+02, 4.50628E+01, 4.5293E-02, 1.8360E-01, &
      2.713869E+02, 4.50566E+01, 3.9872E-02, 1.6711E-01, &
      2.901126E+02, 4.50510E+01, 3.5101E-02, 1.5304E-01, &
      3.101304E+02, 4.50460E+01, 3.0902E-02, 1.4099E-01, &
      3.315294E+02, 4.50414E+01, 2.7206E-02, 1.3060E-01, &
      3.544049E+02, 4.50372E+01, 2.3953E-02, 1.2160E-01, &
      3.788588E+02, 4.50334E+01, 2.1090E-02, 1.1374E-01, &
      4.050001E+02, 4.50300E+01, 1.8570E-02, 1.0683E-01, &
      4.329451E+02, 4.50270E+01, 1.6351E-02, 1.0071E-01]
   case(46)
      frel = -2.6820E-01
      allocate(nist_tab(888))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.918045E-03, 6.71496E+00, 0.0000E+00, 1.6607E-06, &
      4.993321E-03, 6.11262E+00, 0.0000E+00, 1.8159E-06, &
      5.013395E-03, 5.44009E+00, 0.0000E+00, 1.8592E-06, &
      5.043505E-03, 6.10578E+00, 1.2926E+00, 1.0134E+05, &
      5.118781E-03, 6.63715E+00, 1.2527E+00, 9.6767E+04, &
      5.337699E-03, 6.79644E+00, 1.1464E+00, 8.4925E+04, &
      5.419399E-03, 6.49543E+00, 1.1101E+00, 8.0997E+04, &
      5.441185E-03, 6.06873E+00, 1.1007E+00, 7.9991E+04, &
      5.473865E-03, 6.55164E+00, 1.9521E+00, 1.4102E+05, &
      5.555565E-03, 6.98189E+00, 1.8837E+00, 1.3407E+05, &
      1.069000E-02, 6.01744E+00, 5.9028E+00, 2.1834E+05, &
      1.142761E-02, 6.42158E+00, 6.9580E+00, 2.4076E+05, &
      1.221612E-02, 7.09077E+00, 7.9974E+00, 2.5886E+05, &
      1.305903E-02, 8.02678E+00, 8.9357E+00, 2.7057E+05, &
      1.396010E-02, 9.19859E+00, 9.6879E+00, 2.7441E+05, &
      1.492335E-02, 1.05437E+01, 1.0183E+01, 2.6981E+05, &
      1.595306E-02, 1.21215E+01, 1.0377E+01, 2.5720E+05, &
      1.705382E-02, 1.36564E+01, 1.0260E+01, 2.3789E+05, &
      1.823053E-02, 1.49854E+01, 9.8569E+00, 2.1379E+05, &
      1.948844E-02, 1.61464E+01, 9.2189E+00, 1.8705E+05, &
      2.083314E-02, 1.70936E+01, 8.4134E+00, 1.5969E+05, &
      2.227063E-02, 1.78069E+01, 7.5120E+00, 1.3338E+05, &
      2.380730E-02, 1.82908E+01, 6.5829E+00, 1.0934E+05, &
      2.545001E-02, 1.85688E+01, 5.6822E+00, 8.8284E+04, &
      2.720606E-02, 1.86751E+01, 4.8484E+00, 7.0467E+04, &
      2.908327E-02, 1.86465E+01, 4.1035E+00, 5.5791E+04, &
      3.109002E-02, 1.85158E+01, 3.4558E+00, 4.3953E+04, &
      3.323523E-02, 1.83091E+01, 2.9046E+00, 3.4557E+04, &
      3.552846E-02, 1.80435E+01, 2.4427E+00, 2.7186E+04, &
      3.797993E-02, 1.77246E+01, 2.0602E+00, 2.1449E+04, &
      4.060054E-02, 1.73419E+01, 1.7461E+00, 1.7005E+04, &
      4.340198E-02, 1.68525E+01, 1.4895E+00, 1.3570E+04, &
      4.639671E-02, 1.61137E+01, 1.2807E+00, 1.0915E+04, &
      4.959809E-02, 1.42976E+01, 1.1109E+00, 8.8566E+03, &
      5.007800E-02, 1.36230E+01, 1.0892E+00, 8.6002E+03, &
      5.084450E-02, 1.09636E+01, 1.0562E+00, 8.2142E+03, &
      5.104890E-02, 7.59590E+00, 1.0478E+00, 8.1159E+03, &
      5.135550E-02, 1.14201E+01, 7.7492E+00, 5.9666E+04, &
      5.212200E-02, 1.47671E+01, 7.1742E+00, 5.4426E+04, &
      5.302035E-02, 1.62824E+01, 6.5676E+00, 4.8980E+04, &
      5.667876E-02, 1.83065E+01, 4.6854E+00, 3.2687E+04, &
      6.058959E-02, 1.86618E+01, 3.4892E+00, 2.2771E+04, &
      6.477028E-02, 1.86221E+01, 2.7454E+00, 1.6760E+04, &
      6.923942E-02, 1.84661E+01, 2.2608E+00, 1.2911E+04, &
      7.401695E-02, 1.82750E+01, 1.9334E+00, 1.0329E+04, &
      7.912411E-02, 1.80705E+01, 1.7064E+00, 8.5278E+03, &
      8.458368E-02, 1.78268E+01, 1.5464E+00, 7.2293E+03, &
      8.467200E-02, 1.78216E+01, 1.5443E+00, 7.2120E+03, &
      8.596800E-02, 1.77138E+01, 1.5151E+00, 6.9689E+03, &
      8.631360E-02, 1.76255E+01, 1.5077E+00, 6.9073E+03, &
      8.683200E-02, 1.76845E+01, 1.6484E+00, 7.5066E+03, &
      8.812800E-02, 1.77103E+01, 1.6287E+00, 7.3078E+03, &
      9.041995E-02, 1.76846E+01, 1.5993E+00, 6.9942E+03, &
      9.665893E-02, 1.75705E+01, 1.5516E+00, 6.3478E+03, &
      1.033284E-01, 1.74529E+01, 1.5380E+00, 5.8859E+03, &
      1.104581E-01, 1.73420E+01, 1.5483E+00, 5.5429E+03, &
      1.180797E-01, 1.72375E+01, 1.5724E+00, 5.2659E+03, &
      1.262272E-01, 1.71364E+01, 1.6069E+00, 5.0341E+03, &
      1.349368E-01, 1.70353E+01, 1.6492E+00, 4.8333E+03, &
      1.442475E-01, 1.69303E+01, 1.6973E+00, 4.6533E+03, &
      1.542005E-01, 1.68155E+01, 1.7494E+00, 4.4869E+03, &
      1.648404E-01, 1.66888E+01, 1.8042E+00, 4.3289E+03, &
      1.762144E-01, 1.65424E+01, 1.8604E+00, 4.1756E+03, &
      1.883732E-01, 1.63683E+01, 1.9166E+00, 4.0244E+03, &
      2.013709E-01, 1.61564E+01, 1.9719E+00, 3.8733E+03, &
      2.152655E-01, 1.58928E+01, 2.0248E+00, 3.7207E+03, &
      2.301188E-01, 1.55574E+01, 2.0736E+00, 3.5647E+03, &
      2.459970E-01, 1.51204E+01, 2.1173E+00, 3.4052E+03, &
      2.629708E-01, 1.45368E+01, 2.1553E+00, 3.2428E+03, &
      2.811158E-01, 1.37202E+01, 2.1866E+00, 3.0779E+03, &
      3.005128E-01, 1.24975E+01, 2.2108E+00, 2.9114E+03, &
      3.212482E-01, 1.03106E+01, 2.2275E+00, 2.7444E+03, &
      3.280060E-01, 9.03284E+00, 2.2311E+00, 2.6924E+03, &
      3.332000E-01, 7.08114E+00, 2.2334E+00, 2.6531E+03, &
      3.343653E-01, 5.69216E+00, 2.2338E+00, 2.6444E+03, &
      3.363735E-01, 6.38916E+00, 4.6967E+00, 5.5239E+03, &
      3.383000E-01, 6.32003E+00, 4.7964E+00, 5.6090E+03, &
      3.396600E-01, 5.52007E+00, 4.8687E+00, 5.6708E+03, &
      3.417000E-01, 6.37537E+00, 6.7251E+00, 7.7851E+03, &
      3.434143E-01, 6.72139E+00, 6.8812E+00, 7.9260E+03, &
      3.468000E-01, 6.99095E+00, 7.2030E+00, 8.2157E+03, &
      3.671099E-01, 6.90251E+00, 9.5564E+00, 1.0296E+04, &
      3.924405E-01, 7.18951E+00, 1.3170E+01, 1.3273E+04, &
      4.195189E-01, 8.67316E+00, 1.6686E+01, 1.5731E+04, &
      4.484657E-01, 1.10867E+01, 1.9540E+01, 1.7233E+04, &
      4.794098E-01, 1.39123E+01, 2.1425E+01, 1.7675E+04, &
      5.124891E-01, 1.62885E+01, 2.2290E+01, 1.7202E+04, &
      5.208700E-01, 1.64916E+01, 2.2359E+01, 1.6978E+04, &
      5.288425E-01, 1.58269E+01, 2.2378E+01, 1.6736E+04, &
      5.309685E-01, 1.44342E+01, 2.2376E+01, 1.6668E+04, &
      5.341575E-01, 1.62489E+01, 2.5425E+01, 1.8825E+04, &
      5.421300E-01, 1.81690E+01, 2.5386E+01, 1.8520E+04, &
      5.478508E-01, 1.89247E+01, 2.5337E+01, 1.8292E+04, &
      5.479180E-01, 1.89321E+01, 2.5336E+01, 1.8289E+04, &
      5.563045E-01, 1.94716E+01, 2.5236E+01, 1.7943E+04, &
      5.585409E-01, 1.90688E+01, 2.5204E+01, 1.7848E+04, &
      5.618955E-01, 2.01262E+01, 2.6465E+01, 1.8629E+04, &
      5.702820E-01, 2.16034E+01, 2.6325E+01, 1.8258E+04, &
      5.856525E-01, 2.34214E+01, 2.6010E+01, 1.7566E+04, &
      6.260625E-01, 2.68121E+01, 2.4928E+01, 1.5750E+04, &
      6.565020E-01, 2.84827E+01, 2.3978E+01, 1.4447E+04, &
      6.665505E-01, 2.86202E+01, 2.3654E+01, 1.4037E+04, &
      6.692609E-01, 2.82322E+01, 2.3566E+01, 1.3928E+04, &
      6.732495E-01, 2.89922E+01, 2.4461E+01, 1.4372E+04, &
      6.832980E-01, 2.99729E+01, 2.4134E+01, 1.3971E+04, &
      7.154399E-01, 3.18543E+01, 2.3094E+01, 1.2769E+04, &
      7.648052E-01, 3.38810E+01, 2.1563E+01, 1.1154E+04, &
      8.175768E-01, 3.54626E+01, 2.0056E+01, 9.7061E+03, &
      8.739896E-01, 3.67357E+01, 1.8617E+01, 8.4288E+03, &
      9.342948E-01, 3.77973E+01, 1.7250E+01, 7.3070E+03, &
      9.987612E-01, 3.87285E+01, 1.5952E+01, 6.3219E+03, &
      1.067676E+00, 3.94074E+01, 1.4612E+01, 5.4184E+03, &
      1.141345E+00, 3.98884E+01, 1.3372E+01, 4.6394E+03, &
      1.220098E+00, 4.02408E+01, 1.2224E+01, 3.9685E+03, &
      1.304285E+00, 4.04949E+01, 1.1173E+01, 3.3942E+03, &
      1.394281E+00, 4.06675E+01, 1.0160E+01, 2.8884E+03, &
      1.490486E+00, 4.07247E+01, 9.2068E+00, 2.4495E+03, &
      1.593329E+00, 4.06964E+01, 8.3565E+00, 2.0808E+03, &
      1.703269E+00, 4.06170E+01, 7.5968E+00, 1.7706E+03, &
      1.820795E+00, 4.04717E+01, 6.9175E+00, 1.5093E+03, &
      1.946430E+00, 4.02765E+01, 6.3097E+00, 1.2887E+03, &
      2.080733E+00, 4.00306E+01, 5.7651E+00, 1.1025E+03, &
      2.224304E+00, 3.97193E+01, 5.2551E+00, 9.4098E+02, &
      2.377781E+00, 3.92989E+01, 4.7721E+00, 8.0025E+02, &
      2.541848E+00, 3.87190E+01, 4.3369E+00, 6.8122E+02, &
      2.717235E+00, 3.78775E+01, 3.9451E+00, 5.8052E+02, &
      2.904724E+00, 3.64793E+01, 3.5896E+00, 4.9492E+02, &
      3.105150E+00, 3.29123E+01, 3.2270E+00, 4.1704E+02, &
      3.109834E+00, 3.27326E+01, 3.2184E+00, 4.1533E+02, &
      3.157434E+00, 2.93641E+01, 3.1330E+00, 3.9843E+02, &
      3.170127E+00, 2.56358E+01, 3.1109E+00, 3.9409E+02, &
      3.189167E+00, 2.90306E+01, 1.0218E+01, 1.2729E+03, &
      3.236766E+00, 3.17693E+01, 1.0188E+01, 1.2506E+03, &
      3.263694E+00, 3.22340E+01, 1.0172E+01, 1.2384E+03, &
      3.313649E+00, 3.17769E+01, 1.0145E+01, 1.2164E+03, &
      3.319406E+00, 3.14036E+01, 1.0142E+01, 1.2140E+03, &
      3.326970E+00, 3.01956E+01, 1.0138E+01, 1.2108E+03, &
      3.346952E+00, 3.23154E+01, 1.3662E+01, 1.6200E+03, &
      3.396906E+00, 3.45954E+01, 1.3509E+01, 1.5784E+03, &
      3.532214E+00, 3.69631E+01, 1.2986E+01, 1.4596E+03, &
      3.548445E+00, 3.70718E+01, 1.2905E+01, 1.4439E+03, &
      3.586279E+00, 3.69573E+01, 1.2710E+01, 1.4071E+03, &
      3.600696E+00, 3.62000E+01, 1.2634E+01, 1.3932E+03, &
      3.622322E+00, 3.74090E+01, 1.4377E+01, 1.5751E+03, &
      3.676386E+00, 3.88586E+01, 1.4066E+01, 1.5186E+03, &
      3.793288E+00, 4.05122E+01, 1.3384E+01, 1.4008E+03, &
      4.055024E+00, 4.25804E+01, 1.1932E+01, 1.1689E+03, &
      4.334821E+00, 4.37716E+01, 1.0633E+01, 9.7511E+02, &
      4.633924E+00, 4.45459E+01, 9.5222E+00, 8.1749E+02, &
      4.953664E+00, 4.50808E+01, 8.5451E+00, 6.8684E+02, &
      5.295467E+00, 4.54549E+01, 7.6856E+00, 5.7841E+02, &
      5.660855E+00, 4.57234E+01, 6.9230E+00, 4.8789E+02, &
      6.051453E+00, 4.59179E+01, 6.2398E+00, 4.1182E+02, &
      6.469004E+00, 4.61414E+01, 5.6154E+00, 3.4712E+02, &
      6.915365E+00, 4.62653E+01, 5.0297E+00, 2.9127E+02, &
      7.392525E+00, 4.63334E+01, 4.5015E+00, 2.4425E+02, &
      7.902609E+00, 4.63307E+01, 4.0278E+00, 2.0481E+02, &
      8.447890E+00, 4.63063E+01, 3.6074E+00, 1.7192E+02, &
      9.030794E+00, 4.62851E+01, 3.2259E+00, 1.4413E+02, &
      9.653919E+00, 4.62264E+01, 2.8623E+00, 1.1994E+02, &
      1.032004E+01, 4.61473E+01, 2.5411E+00, 9.9888E+01, &
      1.103212E+01, 4.60568E+01, 2.2566E+00, 8.3235E+01, &
      1.179334E+01, 4.59590E+01, 2.0056E+00, 6.9441E+01, &
      1.260708E+01, 4.58564E+01, 1.7844E+00, 5.8007E+01, &
      1.347697E+01, 4.57502E+01, 1.5881E+00, 4.8489E+01, &
      1.440688E+01, 4.56397E+01, 1.4114E+00, 4.0495E+01, &
      1.540095E+01, 4.55209E+01, 1.2523E+00, 3.3780E+01, &
      1.646362E+01, 4.53904E+01, 1.1080E+00, 2.8117E+01, &
      1.759961E+01, 4.52410E+01, 9.8014E-01, 2.3412E+01, &
      1.881398E+01, 4.50641E+01, 8.6786E-01, 1.9524E+01, &
      2.011215E+01, 4.48392E+01, 7.6913E-01, 1.6306E+01, &
      2.149988E+01, 4.45155E+01, 6.8225E-01, 1.3639E+01, &
      2.298338E+01, 4.39078E+01, 6.0572E-01, 1.1426E+01, &
      2.386329E+01, 4.30041E+01, 5.6667E-01, 1.0349E+01, &
      2.422855E+01, 4.17152E+01, 5.5164E-01, 9.9438E+00, &
      2.432595E+01, 4.01649E+01, 5.4775E-01, 9.8397E+00, &
      2.447205E+01, 4.17399E+01, 3.5968E+00, 5.9046E+01, &
      2.456923E+01, 4.23366E+01, 3.5720E+00, 5.8413E+01, &
      2.483731E+01, 4.31644E+01, 3.5051E+00, 5.6714E+01, &
      2.626450E+01, 4.46043E+01, 3.1939E+00, 4.8936E+01, &
      2.807676E+01, 4.52922E+01, 2.8664E+00, 4.1151E+01, &
      3.001405E+01, 4.56821E+01, 2.5645E+00, 3.4506E+01, &
      3.208502E+01, 4.59330E+01, 2.2894E+00, 2.8877E+01, &
      3.429889E+01, 4.60979E+01, 2.0390E+00, 2.4117E+01, &
      3.666551E+01, 4.62076E+01, 1.8163E+00, 2.0149E+01, &
      3.919543E+01, 4.62804E+01, 1.6181E+00, 1.6842E+01, &
      4.189992E+01, 4.63277E+01, 1.4416E+00, 1.4083E+01, &
      4.479101E+01, 4.63569E+01, 1.2846E+00, 1.1781E+01, &
      4.788159E+01, 4.63737E+01, 1.1447E+00, 9.8615E+00, &
      5.118542E+01, 4.64139E+01, 1.0151E+00, 8.2196E+00, &
      5.471721E+01, 4.64097E+01, 8.9866E-01, 6.8447E+00, &
      5.849270E+01, 4.63988E+01, 7.9565E-01, 5.7045E+00, &
      6.252870E+01, 4.63839E+01, 7.0451E-01, 4.7585E+00, &
      6.684318E+01, 4.63663E+01, 6.2385E-01, 3.9737E+00, &
      7.145536E+01, 4.63473E+01, 5.5241E-01, 3.3218E+00, &
      7.638578E+01, 4.63277E+01, 4.8916E-01, 2.7805E+00, &
      8.165640E+01, 4.63082E+01, 4.3235E-01, 2.3269E+00, &
      8.729069E+01, 4.62880E+01, 3.8101E-01, 1.9456E+00, &
      9.331374E+01, 4.62677E+01, 3.3579E-01, 1.6303E+00, &
      9.975239E+01, 4.62479E+01, 2.9595E-01, 1.3693E+00, &
      1.066353E+02, 4.62288E+01, 2.6080E-01, 1.1532E+00, &
      1.139931E+02, 4.62106E+01, 2.2958E-01, 9.7323E-01, &
      1.218587E+02, 4.61933E+01, 2.0211E-01, 8.2428E-01, &
      1.302669E+02, 4.61770E+01, 1.7794E-01, 7.0088E-01, &
      1.392553E+02, 4.61617E+01, 1.5666E-01, 5.9855E-01, &
      1.488640E+02, 4.61475E+01, 1.3794E-01, 5.1363E-01, &
      1.591356E+02, 4.61343E+01, 1.2146E-01, 4.4304E-01, &
      1.701159E+02, 4.61221E+01, 1.0695E-01, 3.8430E-01, &
      1.818539E+02, 4.61109E+01, 9.4185E-02, 3.3534E-01, &
      1.944018E+02, 4.61005E+01, 8.2944E-02, 2.9445E-01, &
      2.078156E+02, 4.60911E+01, 7.3048E-02, 2.6023E-01, &
      2.221548E+02, 4.60824E+01, 6.4336E-02, 2.3151E-01, &
      2.374835E+02, 4.60745E+01, 5.6665E-02, 2.0735E-01, &
      2.538699E+02, 4.60673E+01, 4.9911E-02, 1.8695E-01, &
      2.713869E+02, 4.60607E+01, 4.3964E-02, 1.6966E-01, &
      2.901126E+02, 4.60548E+01, 3.8726E-02, 1.5494E-01, &
      3.101304E+02, 4.60494E+01, 3.4114E-02, 1.4236E-01, &
      3.315294E+02, 4.60445E+01, 3.0053E-02, 1.3154E-01, &
      3.544049E+02, 4.60400E+01, 2.6475E-02, 1.2218E-01, &
      3.788588E+02, 4.60360E+01, 2.3325E-02, 1.1404E-01, &
      4.050001E+02, 4.60323E+01, 2.0550E-02, 1.0690E-01, &
      4.329451E+02, 4.60291E+01, 1.8106E-02, 1.0061E-01]
   case(47)
      frel = -2.8260E-01
      allocate(nist_tab(880))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.84090E+00, 4.5969E+00, 1.6775E+05, &
      1.142761E-02, 3.04646E+00, 4.9050E+00, 1.6744E+05, &
      1.221612E-02, 3.28198E+00, 5.2143E+00, 1.6651E+05, &
      1.305903E-02, 3.54889E+00, 5.5216E+00, 1.6495E+05, &
      1.396010E-02, 3.84865E+00, 5.8214E+00, 1.6268E+05, &
      1.492335E-02, 4.17867E+00, 6.1064E+00, 1.5963E+05, &
      1.595306E-02, 4.53536E+00, 6.3737E+00, 1.5586E+05, &
      1.705382E-02, 4.91531E+00, 6.6206E+00, 1.5145E+05, &
      1.823053E-02, 5.31506E+00, 6.8451E+00, 1.4647E+05, &
      1.948844E-02, 5.73104E+00, 7.0457E+00, 1.4104E+05, &
      2.083314E-02, 6.15960E+00, 7.2214E+00, 1.3522E+05, &
      2.227063E-02, 6.59705E+00, 7.3718E+00, 1.2913E+05, &
      2.380730E-02, 7.03979E+00, 7.4968E+00, 1.2284E+05, &
      2.545001E-02, 7.48426E+00, 7.5970E+00, 1.1645E+05, &
      2.720606E-02, 7.92702E+00, 7.6732E+00, 1.1003E+05, &
      2.908327E-02, 8.36475E+00, 7.7266E+00, 1.0364E+05, &
      3.109002E-02, 8.79411E+00, 7.7586E+00, 9.7352E+04, &
      3.323523E-02, 9.21172E+00, 7.7709E+00, 9.1213E+04, &
      3.552846E-02, 9.61381E+00, 7.7652E+00, 8.5263E+04, &
      3.797993E-02, 9.99579E+00, 7.7435E+00, 7.9537E+04, &
      4.060054E-02, 1.03512E+01, 7.7077E+00, 7.4059E+04, &
      4.340198E-02, 1.06696E+01, 7.6596E+00, 6.8847E+04, &
      4.639671E-02, 1.09302E+01, 7.6011E+00, 6.3911E+04, &
      4.959809E-02, 1.10819E+01, 7.5340E+00, 5.9258E+04, &
      5.302035E-02, 1.09250E+01, 7.4599E+00, 5.4888E+04, &
      5.478200E-02, 1.03446E+01, 7.4215E+00, 5.2849E+04, &
      5.562050E-02, 9.15723E+00, 7.4033E+00, 5.1925E+04, &
      5.584410E-02, 7.57811E+00, 7.3984E+00, 5.1683E+04, &
      5.617950E-02, 9.41576E+00, 1.0577E+01, 7.3446E+04, &
      5.667876E-02, 1.06211E+01, 1.0414E+01, 7.1677E+04, &
      5.701800E-02, 1.10650E+01, 1.0308E+01, 7.0527E+04, &
      6.058959E-02, 1.26538E+01, 9.3944E+00, 6.0486E+04, &
      6.134800E-02, 1.26611E+01, 9.2390E+00, 5.8750E+04, &
      6.228700E-02, 1.22300E+01, 9.0530E+00, 5.6700E+04, &
      6.253740E-02, 1.14631E+01, 9.0013E+00, 5.6150E+04, &
      6.291300E-02, 1.24638E+01, 1.0557E+01, 6.5461E+04, &
      6.385200E-02, 1.34374E+01, 1.0250E+01, 6.2625E+04, &
      6.477028E-02, 1.38959E+01, 9.9783E+00, 6.0099E+04, &
      6.923942E-02, 1.49135E+01, 8.9470E+00, 5.0409E+04, &
      7.401695E-02, 1.53648E+01, 8.2211E+00, 4.3330E+04, &
      7.912411E-02, 1.56512E+01, 7.6870E+00, 3.7899E+04, &
      8.458368E-02, 1.58628E+01, 7.2641E+00, 3.3503E+04, &
      9.041995E-02, 1.60128E+01, 6.9106E+00, 2.9815E+04, &
      9.329600E-02, 1.60390E+01, 6.7617E+00, 2.8273E+04, &
      9.472400E-02, 1.59917E+01, 6.6926E+00, 2.7562E+04, &
      9.510480E-02, 1.59036E+01, 6.6746E+00, 2.7378E+04, &
      9.567600E-02, 1.60128E+01, 6.8386E+00, 2.7884E+04, &
      9.665893E-02, 1.61030E+01, 6.7975E+00, 2.7434E+04, &
      9.710400E-02, 1.61295E+01, 6.7793E+00, 2.7235E+04, &
      1.033284E-01, 1.63562E+01, 6.5480E+00, 2.4722E+04, &
      1.104581E-01, 1.65356E+01, 6.3253E+00, 2.2339E+04, &
      1.180797E-01, 1.66898E+01, 6.1216E+00, 2.0224E+04, &
      1.262272E-01, 1.68238E+01, 5.9328E+00, 1.8337E+04, &
      1.349368E-01, 1.69382E+01, 5.7564E+00, 1.6643E+04, &
      1.442475E-01, 1.70318E+01, 5.5901E+00, 1.5119E+04, &
      1.542005E-01, 1.71026E+01, 5.4324E+00, 1.3744E+04, &
      1.648404E-01, 1.71474E+01, 5.2817E+00, 1.2501E+04, &
      1.762144E-01, 1.71640E+01, 5.1370E+00, 1.1373E+04, &
      1.883732E-01, 1.71454E+01, 4.9970E+00, 1.0349E+04, &
      2.013709E-01, 1.70808E+01, 4.8607E+00, 9.4178E+03, &
      2.152655E-01, 1.69624E+01, 4.7272E+00, 8.5682E+03, &
      2.301188E-01, 1.67760E+01, 4.5956E+00, 7.7923E+03, &
      2.459970E-01, 1.65012E+01, 4.4645E+00, 7.0817E+03, &
      2.629708E-01, 1.61067E+01, 4.3331E+00, 6.4300E+03, &
      2.811158E-01, 1.55436E+01, 4.2008E+00, 5.8317E+03, &
      3.005128E-01, 1.47270E+01, 4.0673E+00, 5.2824E+03, &
      3.212482E-01, 1.34807E+01, 3.9327E+00, 4.7783E+03, &
      3.434143E-01, 1.13239E+01, 3.7970E+00, 4.3161E+03, &
      3.593660E-01, 8.17693E+00, 3.7041E+00, 4.0240E+03, &
      3.648665E-01, 5.38571E+00, 3.6730E+00, 3.9301E+03, &
      3.653440E-01, 4.87928E+00, 3.6703E+00, 3.9221E+03, &
      3.663333E-01, 2.90880E+00, 3.6648E+00, 3.9057E+03, &
      3.671099E-01, 2.81916E+00, 7.8010E+00, 8.2928E+03, &
      3.685335E-01, 4.33783E+00, 7.8731E+00, 8.3371E+03, &
      3.709360E-01, 4.39445E+00, 7.9977E+00, 8.4141E+03, &
      3.724272E-01, 3.12064E+00, 8.0768E+00, 8.4634E+03, &
      3.740340E-01, 4.35482E+00, 1.1122E+01, 1.1603E+04, &
      3.746640E-01, 4.77825E+00, 1.1180E+01, 1.1644E+04, &
      3.802560E-01, 6.26493E+00, 1.1714E+01, 1.2020E+04, &
      3.924405E-01, 7.23412E+00, 1.2994E+01, 1.2920E+04, &
      4.195189E-01, 8.52533E+00, 1.6302E+01, 1.5163E+04, &
      4.484657E-01, 1.06783E+01, 1.9390E+01, 1.6871E+04, &
      4.794098E-01, 1.34741E+01, 2.1687E+01, 1.7651E+04, &
      5.124891E-01, 1.64378E+01, 2.3018E+01, 1.7525E+04, &
      5.478508E-01, 1.88361E+01, 2.3418E+01, 1.6680E+04, &
      5.599720E-01, 1.91053E+01, 2.3372E+01, 1.6287E+04, &
      5.685430E-01, 1.83518E+01, 2.3295E+01, 1.5989E+04, &
      5.708286E-01, 1.68858E+01, 2.3269E+01, 1.5907E+04, &
      5.742570E-01, 1.87523E+01, 2.6407E+01, 1.7944E+04, &
      5.828280E-01, 2.06937E+01, 2.6281E+01, 1.7596E+04, &
      5.856525E-01, 2.10721E+01, 2.6233E+01, 1.7479E+04, &
      5.903520E-01, 2.15803E+01, 2.6149E+01, 1.7284E+04, &
      5.993880E-01, 2.20441E+01, 2.5967E+01, 1.6906E+04, &
      6.017976E-01, 2.16033E+01, 2.5915E+01, 1.6804E+04, &
      6.054120E-01, 2.26665E+01, 2.7194E+01, 1.7528E+04, &
      6.144480E-01, 2.41171E+01, 2.6984E+01, 1.7137E+04, &
      6.260625E-01, 2.53913E+01, 2.6689E+01, 1.6635E+04, &
      6.692609E-01, 2.87197E+01, 2.5430E+01, 1.4828E+04, &
      7.031500E-01, 3.03701E+01, 2.4352E+01, 1.3515E+04, &
      7.139125E-01, 3.04724E+01, 2.4005E+01, 1.3123E+04, &
      7.154399E-01, 3.03653E+01, 2.3955E+01, 1.3068E+04, &
      7.167825E-01, 3.00829E+01, 2.3912E+01, 1.3020E+04, &
      7.210875E-01, 3.08260E+01, 2.4813E+01, 1.3430E+04, &
      7.318500E-01, 3.17868E+01, 2.4467E+01, 1.3048E+04, &
      7.648052E-01, 3.35328E+01, 2.3424E+01, 1.1954E+04, &
      8.175768E-01, 3.54838E+01, 2.1840E+01, 1.0427E+04, &
      8.739896E-01, 3.69977E+01, 2.0291E+01, 9.0633E+03, &
      9.342948E-01, 3.82082E+01, 1.8813E+01, 7.8615E+03, &
      9.987612E-01, 3.92179E+01, 1.7415E+01, 6.8085E+03, &
      1.067676E+00, 4.00468E+01, 1.5985E+01, 5.8474E+03, &
      1.141345E+00, 4.06725E+01, 1.4642E+01, 5.0113E+03, &
      1.220098E+00, 4.11291E+01, 1.3398E+01, 4.2908E+03, &
      1.304285E+00, 4.14587E+01, 1.2245E+01, 3.6696E+03, &
      1.394281E+00, 4.16876E+01, 1.1189E+01, 3.1377E+03, &
      1.490486E+00, 4.18417E+01, 1.0211E+01, 2.6796E+03, &
      1.593329E+00, 4.18978E+01, 9.2467E+00, 2.2712E+03, &
      1.703269E+00, 4.18626E+01, 8.3868E+00, 1.9281E+03, &
      1.820795E+00, 4.17704E+01, 7.6195E+00, 1.6396E+03, &
      1.946430E+00, 4.16141E+01, 6.9336E+00, 1.3968E+03, &
      2.080733E+00, 4.14062E+01, 6.3204E+00, 1.1920E+03, &
      2.224304E+00, 4.11462E+01, 5.7714E+00, 1.0192E+03, &
      2.377781E+00, 4.08180E+01, 5.2608E+00, 8.6996E+02, &
      2.541848E+00, 4.03771E+01, 4.7785E+00, 7.4012E+02, &
      2.717235E+00, 3.97652E+01, 4.3434E+00, 6.3018E+02, &
      2.904724E+00, 3.88627E+01, 3.9522E+00, 5.3724E+02, &
      3.105150E+00, 3.73492E+01, 3.5510E+00, 4.5242E+02, &
      3.284078E+00, 3.39635E+01, 3.2163E+00, 3.8821E+02, &
      3.319406E+00, 3.20969E+01, 3.1564E+00, 3.7707E+02, &
      3.334344E+00, 3.05512E+01, 3.1317E+00, 3.7251E+02, &
      3.347749E+00, 2.67378E+01, 3.1097E+00, 3.6847E+02, &
      3.367855E+00, 3.02844E+01, 1.0402E+01, 1.2110E+03, &
      3.418122E+00, 3.31732E+01, 1.0281E+01, 1.1794E+03, &
      3.453226E+00, 3.37562E+01, 1.0200E+01, 1.1582E+03, &
      3.506081E+00, 3.32333E+01, 1.0080E+01, 1.1276E+03, &
      3.520176E+00, 3.15999E+01, 1.0043E+01, 1.1190E+03, &
      3.541318E+00, 3.37502E+01, 1.3624E+01, 1.5067E+03, &
      3.548445E+00, 3.42479E+01, 1.3594E+01, 1.5003E+03, &
      3.594174E+00, 3.60240E+01, 1.3398E+01, 1.4601E+03, &
      3.729684E+00, 3.81788E+01, 1.2784E+01, 1.3430E+03, &
      3.786771E+00, 3.81381E+01, 1.2494E+01, 1.2928E+03, &
      3.793288E+00, 3.79709E+01, 1.2461E+01, 1.2872E+03, &
      3.801994E+00, 3.73722E+01, 1.2416E+01, 1.2797E+03, &
      3.824829E+00, 3.85660E+01, 1.4155E+01, 1.4494E+03, &
      3.881916E+00, 3.99808E+01, 1.3844E+01, 1.3969E+03, &
      4.055024E+00, 4.20364E+01, 1.2923E+01, 1.2487E+03, &
      4.334821E+00, 4.38319E+01, 1.1573E+01, 1.0468E+03, &
      4.633924E+00, 4.49334E+01, 1.0367E+01, 8.7786E+02, &
      4.953664E+00, 4.56723E+01, 9.3056E+00, 7.3772E+02, &
      5.295467E+00, 4.61852E+01, 8.3559E+00, 6.2024E+02, &
      5.660855E+00, 4.65439E+01, 7.5133E+00, 5.2222E+02, &
      6.051453E+00, 4.67976E+01, 6.7612E+00, 4.4009E+02, &
      6.469004E+00, 4.69766E+01, 6.0868E+00, 3.7107E+02, &
      6.915365E+00, 4.71807E+01, 5.4706E+00, 3.1241E+02, &
      7.392525E+00, 4.72894E+01, 4.9010E+00, 2.6222E+02, &
      7.902609E+00, 4.73505E+01, 4.3883E+00, 2.2001E+02, &
      8.447890E+00, 4.73467E+01, 3.9300E+00, 1.8466E+02, &
      9.030794E+00, 4.73333E+01, 3.5141E+00, 1.5478E+02, &
      9.653919E+00, 4.72863E+01, 3.1173E+00, 1.2876E+02, &
      1.032004E+01, 4.72146E+01, 2.7679E+00, 1.0724E+02, &
      1.103212E+01, 4.71294E+01, 2.4591E+00, 8.9393E+01, &
      1.179334E+01, 4.70354E+01, 2.1854E+00, 7.4558E+01, &
      1.260708E+01, 4.69353E+01, 1.9437E+00, 6.2254E+01, &
      1.347697E+01, 4.68312E+01, 1.7304E+00, 5.2050E+01, &
      1.440688E+01, 4.67237E+01, 1.5411E+00, 4.3548E+01, &
      1.540095E+01, 4.66114E+01, 1.3695E+00, 3.6371E+01, &
      1.646362E+01, 4.64900E+01, 1.2159E+00, 3.0368E+01, &
      1.759961E+01, 4.63552E+01, 1.0755E+00, 2.5277E+01, &
      1.881398E+01, 4.61995E+01, 9.5208E-01, 2.1067E+01, &
      2.011215E+01, 4.60118E+01, 8.4356E-01, 1.7584E+01, &
      2.149988E+01, 4.57660E+01, 7.4808E-01, 1.4700E+01, &
      2.298338E+01, 4.53918E+01, 6.6401E-01, 1.2307E+01, &
      2.456923E+01, 4.45660E+01, 5.8990E-01, 1.0320E+01, &
      2.500372E+01, 4.40222E+01, 5.7193E-01, 9.8558E+00, &
      2.538643E+01, 4.27404E+01, 5.5683E-01, 9.4716E+00, &
      2.548849E+01, 4.11991E+01, 5.5291E-01, 9.3728E+00, &
      2.564157E+01, 4.27653E+01, 3.5841E+00, 5.5433E+01, &
      2.602428E+01, 4.41811E+01, 3.4923E+00, 5.3237E+01, &
      2.626450E+01, 4.45838E+01, 3.4365E+00, 5.1919E+01, &
      2.807676E+01, 4.58610E+01, 3.0917E+00, 4.3763E+01, &
      3.001405E+01, 4.64369E+01, 2.7803E+00, 3.6879E+01, &
      3.208502E+01, 4.67849E+01, 2.4770E+00, 3.0799E+01, &
      3.429889E+01, 4.69986E+01, 2.2014E+00, 2.5665E+01, &
      3.666551E+01, 4.71402E+01, 1.9609E+00, 2.1440E+01, &
      3.919543E+01, 4.72353E+01, 1.7469E+00, 1.7917E+01, &
      4.189992E+01, 4.72982E+01, 1.5564E+00, 1.4981E+01, &
      4.479101E+01, 4.73384E+01, 1.3868E+00, 1.2531E+01, &
      4.788159E+01, 4.73627E+01, 1.2359E+00, 1.0487E+01, &
      5.118542E+01, 4.74096E+01, 1.1011E+00, 8.7792E+00, &
      5.471721E+01, 4.74120E+01, 9.7517E-01, 7.3111E+00, &
      5.849270E+01, 4.74053E+01, 8.6367E-01, 6.0933E+00, &
      6.252870E+01, 4.73930E+01, 7.6500E-01, 5.0827E+00, &
      6.684318E+01, 4.73772E+01, 6.7765E-01, 4.2440E+00, &
      7.145536E+01, 4.73593E+01, 6.0032E-01, 3.5476E+00, &
      7.638578E+01, 4.73402E+01, 5.3180E-01, 2.9690E+00, &
      8.165640E+01, 4.73210E+01, 4.7041E-01, 2.4851E+00, &
      8.729069E+01, 4.73010E+01, 4.1471E-01, 2.0772E+00, &
      9.331374E+01, 4.72805E+01, 3.6563E-01, 1.7397E+00, &
      9.975239E+01, 4.72603E+01, 3.2238E-01, 1.4605E+00, &
      1.066353E+02, 4.72407E+01, 2.8427E-01, 1.2292E+00, &
      1.139931E+02, 4.72220E+01, 2.5054E-01, 1.0373E+00, &
      1.218587E+02, 4.72041E+01, 2.2070E-01, 8.7777E-01, &
      1.302669E+02, 4.71872E+01, 1.9442E-01, 7.4562E-01, &
      1.392553E+02, 4.71713E+01, 1.7128E-01, 6.3601E-01, &
      1.488640E+02, 4.71564E+01, 1.5090E-01, 5.4502E-01, &
      1.591356E+02, 4.71426E+01, 1.3295E-01, 4.6940E-01, &
      1.701159E+02, 4.71298E+01, 1.1714E-01, 4.0647E-01, &
      1.818539E+02, 4.71180E+01, 1.0322E-01, 3.5402E-01, &
      1.944018E+02, 4.71071E+01, 9.0955E-02, 3.1023E-01, &
      2.078156E+02, 4.70971E+01, 8.0152E-02, 2.7359E-01, &
      2.221548E+02, 4.70880E+01, 7.0635E-02, 2.4286E-01, &
      2.374835E+02, 4.70796E+01, 6.2250E-02, 2.1701E-01, &
      2.538699E+02, 4.70720E+01, 5.4863E-02, 1.9519E-01, &
      2.713869E+02, 4.70650E+01, 4.8355E-02, 1.7672E-01, &
      2.901126E+02, 4.70587E+01, 4.2620E-02, 1.6100E-01, &
      3.101304E+02, 4.70529E+01, 3.7567E-02, 1.4758E-01, &
      3.315294E+02, 4.70477E+01, 3.3114E-02, 1.3605E-01, &
      3.544049E+02, 4.70429E+01, 2.9190E-02, 1.2609E-01, &
      3.788588E+02, 4.70386E+01, 2.5732E-02, 1.1743E-01, &
      4.050001E+02, 4.70348E+01, 2.2684E-02, 1.0986E-01, &
      4.329451E+02, 4.70313E+01, 1.9998E-02, 1.0319E-01]
   case(48)
      frel = -2.9760E-01
      allocate(nist_tab(880))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      9.114000E-03, -2.55482E-01, 0.0000E+00, 3.8335E-05, &
      9.253500E-03, -1.31522E+00, 0.0000E+00, 4.1467E-05, &
      9.290700E-03, -2.57675E+00, 0.0000E+00, 4.2334E-05, &
      9.346500E-03, -1.23253E+00, 2.4746E+00, 9.9114E+04, &
      9.486000E-03, -8.88866E-02, 2.3409E+00, 9.2377E+04, &
      1.069000E-02, 8.05089E-01, 1.5527E+00, 5.4375E+04, &
      1.142761E-02, 4.47887E-01, 1.4576E+00, 4.7747E+04, &
      1.221612E-02, -4.65177E-02, 1.5479E+00, 4.7432E+04, &
      1.305903E-02, -6.38675E-01, 1.8001E+00, 5.1600E+04, &
      1.396010E-02, -1.31418E+00, 2.2301E+00, 5.9801E+04, &
      1.492335E-02, -2.05340E+00, 2.8761E+00, 7.2145E+04, &
      1.595306E-02, -2.81785E+00, 3.7878E+00, 8.8882E+04, &
      1.705382E-02, -3.54108E+00, 5.0153E+00, 1.1009E+05, &
      1.823053E-02, -4.12454E+00, 6.5929E+00, 1.3538E+05, &
      1.948844E-02, -4.44143E+00, 8.5207E+00, 1.6367E+05, &
      2.083314E-02, -4.35157E+00, 1.0746E+01, 1.9310E+05, &
      2.227063E-02, -3.72673E+00, 1.3156E+01, 2.2114E+05, &
      2.380730E-02, -2.48166E+00, 1.5578E+01, 2.4494E+05, &
      2.545001E-02, -6.01492E-01, 1.7804E+01, 2.6188E+05, &
      2.720606E-02, 1.84418E+00, 1.9625E+01, 2.7003E+05, &
      2.908327E-02, 4.65148E+00, 2.0871E+01, 2.6864E+05, &
      3.109002E-02, 7.72864E+00, 2.1439E+01, 2.5814E+05, &
      3.323523E-02, 1.07988E+01, 2.1312E+01, 2.4005E+05, &
      3.552846E-02, 1.36598E+01, 2.0551E+01, 2.1654E+05, &
      3.797993E-02, 1.61566E+01, 1.9278E+01, 1.9001E+05, &
      4.060054E-02, 1.81877E+01, 1.7636E+01, 1.6261E+05, &
      4.340198E-02, 1.97108E+01, 1.5795E+01, 1.3623E+05, &
      4.639671E-02, 2.07453E+01, 1.3904E+01, 1.1218E+05, &
      4.959809E-02, 2.13457E+01, 1.2077E+01, 9.1154E+04, &
      5.302035E-02, 2.15780E+01, 1.0390E+01, 7.3356E+04, &
      5.667876E-02, 2.14957E+01, 8.8820E+00, 5.8663E+04, &
      6.058959E-02, 2.10974E+01, 7.5685E+00, 4.6761E+04, &
      6.477028E-02, 2.00512E+01, 6.4461E+00, 3.7256E+04, &
      6.556200E-02, 1.95940E+01, 6.2613E+00, 3.5751E+04, &
      6.656550E-02, 1.81776E+01, 6.0383E+00, 3.3958E+04, &
      6.683310E-02, 1.64471E+01, 5.9809E+00, 3.3500E+04, &
      6.723450E-02, 1.83530E+01, 9.3131E+00, 5.1853E+04, &
      6.823800E-02, 1.99870E+01, 8.8611E+00, 4.8611E+04, &
      6.923942E-02, 2.06144E+01, 8.4419E+00, 4.5642E+04, &
      7.401695E-02, 2.14606E+01, 6.8057E+00, 3.4421E+04, &
      7.912411E-02, 2.13777E+01, 5.6179E+00, 2.6579E+04, &
      8.458368E-02, 2.10768E+01, 4.7652E+00, 2.1090E+04, &
      9.041995E-02, 2.07147E+01, 4.1313E+00, 1.7104E+04, &
      9.665893E-02, 2.03349E+01, 3.6500E+00, 1.4136E+04, &
      1.033284E-01, 1.99289E+01, 3.2803E+00, 1.1884E+04, &
      1.054480E-01, 1.97777E+01, 3.1855E+00, 1.1309E+04, &
      1.070620E-01, 1.95992E+01, 3.1194E+00, 1.0907E+04, &
      1.074924E-01, 1.94588E+01, 3.1025E+00, 1.0805E+04, &
      1.081380E-01, 1.95484E+01, 3.3138E+00, 1.1472E+04, &
      1.097520E-01, 1.95803E+01, 3.2602E+00, 1.1120E+04, &
      1.104581E-01, 1.95713E+01, 3.2381E+00, 1.0974E+04, &
      1.180797E-01, 1.93730E+01, 3.0421E+00, 9.6449E+03, &
      1.262272E-01, 1.91492E+01, 2.8981E+00, 8.5954E+03, &
      1.349368E-01, 1.89330E+01, 2.7933E+00, 7.7499E+03, &
      1.442475E-01, 1.87262E+01, 2.7191E+00, 7.0573E+03, &
      1.542005E-01, 1.85253E+01, 2.6687E+00, 6.4794E+03, &
      1.648404E-01, 1.83249E+01, 2.6363E+00, 5.9878E+03, &
      1.762144E-01, 1.81169E+01, 2.6226E+00, 5.5724E+03, &
      1.883732E-01, 1.79042E+01, 2.6536E+00, 5.2744E+03, &
      2.013709E-01, 1.76951E+01, 2.6910E+00, 5.0039E+03, &
      2.152655E-01, 1.74569E+01, 2.7323E+00, 4.7529E+03, &
      2.301188E-01, 1.71855E+01, 2.7750E+00, 4.5159E+03, &
      2.459970E-01, 1.68631E+01, 2.8169E+00, 4.2884E+03, &
      2.629708E-01, 1.64655E+01, 2.8562E+00, 4.0678E+03, &
      2.811158E-01, 1.59582E+01, 2.8906E+00, 3.8514E+03, &
      3.005128E-01, 1.52867E+01, 2.9178E+00, 3.6370E+03, &
      3.212482E-01, 1.43590E+01, 2.9368E+00, 3.4247E+03, &
      3.434143E-01, 1.29964E+01, 2.9468E+00, 3.2151E+03, &
      3.671099E-01, 1.07613E+01, 2.9475E+00, 3.0086E+03, &
      3.924405E-01, 5.74265E+00, 2.9384E+00, 2.8061E+03, &
      3.956260E-01, 4.46424E+00, 2.9367E+00, 2.7820E+03, &
      4.016815E-01, -1.52779E-01, 2.9330E+00, 2.7367E+03, &
      4.022900E-01, -1.19614E+00, 2.9326E+00, 2.7322E+03, &
      4.032963E-01, -4.55670E+00, 2.9320E+00, 2.7248E+03, &
      4.057185E-01, -1.56461E+00, 1.0686E+01, 9.8634E+03, &
      4.084475E-01, -1.12406E+00, 1.0825E+01, 9.9242E+03, &
      4.100895E-01, -3.26016E+00, 1.0908E+01, 9.9611E+03, &
      4.117740E-01, -9.11513E-01, 1.6362E+01, 1.4878E+04, &
      4.125525E-01, 6.86458E-02, 1.6428E+01, 1.4910E+04, &
      4.187100E-01, 3.39588E+00, 1.6954E+01, 1.5160E+04, &
      4.195189E-01, 3.65174E+00, 1.7024E+01, 1.5194E+04, &
      4.484657E-01, 8.37082E+00, 1.9663E+01, 1.6417E+04, &
      4.794098E-01, 1.19797E+01, 2.2013E+01, 1.7193E+04, &
      5.124891E-01, 1.54735E+01, 2.3387E+01, 1.7087E+04, &
      5.478508E-01, 1.86093E+01, 2.3936E+01, 1.6361E+04, &
      5.856525E-01, 2.10086E+01, 2.3768E+01, 1.5198E+04, &
      6.041700E-01, 2.13556E+01, 2.3501E+01, 1.4566E+04, &
      6.134175E-01, 2.04904E+01, 2.3338E+01, 1.4247E+04, &
      6.158835E-01, 1.89637E+01, 2.3292E+01, 1.4162E+04, &
      6.195825E-01, 2.08403E+01, 2.6480E+01, 1.6004E+04, &
      6.260625E-01, 2.23481E+01, 2.6351E+01, 1.5761E+04, &
      6.288300E-01, 2.27411E+01, 2.6294E+01, 1.5658E+04, &
      6.376860E-01, 2.36380E+01, 2.6105E+01, 1.5330E+04, &
      6.474465E-01, 2.40110E+01, 2.5888E+01, 1.4973E+04, &
      6.500493E-01, 2.35362E+01, 2.5828E+01, 1.4879E+04, &
      6.539535E-01, 2.45866E+01, 2.7128E+01, 1.5534E+04, &
      6.637140E-01, 2.59872E+01, 2.6905E+01, 1.5180E+04, &
      6.692609E-01, 2.65609E+01, 2.6776E+01, 1.4982E+04, &
      7.154399E-01, 3.00001E+01, 2.5514E+01, 1.3355E+04, &
      7.547960E-01, 3.17476E+01, 2.4279E+01, 1.2047E+04, &
      7.648052E-01, 3.18609E+01, 2.3969E+01, 1.1738E+04, &
      7.663490E-01, 3.18163E+01, 2.3922E+01, 1.1691E+04, &
      7.694298E-01, 3.14077E+01, 2.3827E+01, 1.1598E+04, &
      7.740510E-01, 3.21559E+01, 2.4751E+01, 1.1976E+04, &
      7.856040E-01, 3.31070E+01, 2.4400E+01, 1.1633E+04, &
      8.175768E-01, 3.46667E+01, 2.3457E+01, 1.0746E+04, &
      8.739896E-01, 3.65926E+01, 2.1905E+01, 9.3884E+03, &
      9.342948E-01, 3.81048E+01, 2.0411E+01, 8.1843E+03, &
      9.987612E-01, 3.93637E+01, 1.8998E+01, 7.1273E+03, &
      1.067676E+00, 4.04395E+01, 1.7471E+01, 6.1323E+03, &
      1.141345E+00, 4.12418E+01, 1.6005E+01, 5.2562E+03, &
      1.220098E+00, 4.18352E+01, 1.4641E+01, 4.4991E+03, &
      1.304285E+00, 4.22838E+01, 1.3382E+01, 3.8479E+03, &
      1.394281E+00, 4.25930E+01, 1.2216E+01, 3.2870E+03, &
      1.490486E+00, 4.28011E+01, 1.1152E+01, 2.8079E+03, &
      1.593329E+00, 4.29337E+01, 1.0182E+01, 2.3994E+03, &
      1.703269E+00, 4.29952E+01, 9.2462E+00, 2.0392E+03, &
      1.820795E+00, 4.29620E+01, 8.3851E+00, 1.7310E+03, &
      1.946430E+00, 4.28569E+01, 7.6167E+00, 1.4720E+03, &
      2.080733E+00, 4.26965E+01, 6.9297E+00, 1.2537E+03, &
      2.224304E+00, 4.24829E+01, 6.3148E+00, 1.0698E+03, &
      2.377781E+00, 4.22148E+01, 5.7641E+00, 9.1433E+02, &
      2.541848E+00, 4.18768E+01, 5.2570E+00, 7.8096E+02, &
      2.717235E+00, 4.14267E+01, 4.7781E+00, 6.6488E+02, &
      2.904724E+00, 4.08113E+01, 4.3441E+00, 5.6630E+02, &
      3.105150E+00, 3.99196E+01, 3.9055E+00, 4.7714E+02, &
      3.319406E+00, 3.80892E+01, 3.4718E+00, 3.9766E+02, &
      3.466750E+00, 3.51406E+01, 3.2183E+00, 3.5353E+02, &
      3.519812E+00, 3.16870E+01, 3.1345E+00, 3.3934E+02, &
      3.533962E+00, 2.77885E+01, 3.1128E+00, 3.3570E+02, &
      3.548445E+00, 3.03560E+01, 1.0619E+01, 1.1262E+03, &
      3.555187E+00, 3.14895E+01, 1.0592E+01, 1.1211E+03, &
      3.608250E+00, 3.45292E+01, 1.0378E+01, 1.0825E+03, &
      3.652460E+00, 3.52329E+01, 1.0205E+01, 1.0518E+03, &
      3.708365E+00, 3.46373E+01, 9.9942E+00, 1.0147E+03, &
      3.723273E+00, 3.29486E+01, 9.9338E+00, 1.0046E+03, &
      3.745635E+00, 3.51307E+01, 1.3575E+01, 1.3625E+03, &
      3.793288E+00, 3.71821E+01, 1.3324E+01, 1.3206E+03, &
      3.801540E+00, 3.73998E+01, 1.3281E+01, 1.3135E+03, &
      3.937640E+00, 3.93697E+01, 1.2588E+01, 1.2023E+03, &
      3.997910E+00, 3.92963E+01, 1.2284E+01, 1.1558E+03, &
      4.013982E+00, 3.85238E+01, 1.2204E+01, 1.1438E+03, &
      4.038090E+00, 3.97007E+01, 1.3936E+01, 1.2975E+03, &
      4.055024E+00, 4.02290E+01, 1.3848E+01, 1.2839E+03, &
      4.098360E+00, 4.10801E+01, 1.3626E+01, 1.2501E+03, &
      4.334821E+00, 4.34595E+01, 1.2493E+01, 1.0841E+03, &
      4.633924E+00, 4.50537E+01, 1.1248E+01, 9.1375E+02, &
      4.953664E+00, 4.60867E+01, 1.0110E+01, 7.6889E+02, &
      5.295467E+00, 4.67933E+01, 9.0844E+00, 6.4689E+02, &
      5.660855E+00, 4.72835E+01, 8.1591E+00, 5.4402E+02, &
      6.051453E+00, 4.76250E+01, 7.3317E+00, 4.5780E+02, &
      6.469004E+00, 4.78624E+01, 6.5913E+00, 3.8545E+02, &
      6.915365E+00, 4.80258E+01, 5.9281E+00, 3.2472E+02, &
      7.392525E+00, 4.82126E+01, 5.3234E+00, 2.7317E+02, &
      7.902609E+00, 4.83099E+01, 4.7715E+00, 2.2942E+02, &
      8.447890E+00, 4.83643E+01, 4.2744E+00, 1.9260E+02, &
      9.030794E+00, 4.83786E+01, 3.8216E+00, 1.6140E+02, &
      9.653919E+00, 4.83459E+01, 3.3892E+00, 1.3422E+02, &
      1.032004E+01, 4.82818E+01, 3.0085E+00, 1.1175E+02, &
      1.103212E+01, 4.82016E+01, 2.6731E+00, 9.3152E+01, &
      1.179334E+01, 4.81115E+01, 2.3769E+00, 7.7726E+01, &
      1.260708E+01, 4.80147E+01, 2.1138E+00, 6.4885E+01, &
      1.347697E+01, 4.79129E+01, 1.8813E+00, 5.4225E+01, &
      1.440688E+01, 4.78076E+01, 1.6761E+00, 4.5376E+01, &
      1.540095E+01, 4.76992E+01, 1.4934E+00, 3.7988E+01, &
      1.646362E+01, 4.75850E+01, 1.3271E+00, 3.1737E+01, &
      1.759961E+01, 4.74612E+01, 1.1782E+00, 2.6503E+01, &
      1.881398E+01, 4.73218E+01, 1.0427E+00, 2.2078E+01, &
      2.011215E+01, 4.71592E+01, 9.2363E-01, 1.8418E+01, &
      2.149988E+01, 4.69590E+01, 8.1889E-01, 1.5387E+01, &
      2.298338E+01, 4.66870E+01, 7.2667E-01, 1.2875E+01, &
      2.456923E+01, 4.62404E+01, 6.4542E-01, 1.0789E+01, &
      2.617698E+01, 4.50549E+01, 5.7713E-01, 9.1356E+00, &
      2.626450E+01, 4.48946E+01, 5.7375E-01, 9.0563E+00, &
      2.657764E+01, 4.37823E+01, 5.6193E-01, 8.7804E+00, &
      2.668449E+01, 4.22514E+01, 5.5799E-01, 8.6890E+00, &
      2.684476E+01, 4.38062E+01, 3.5688E+00, 5.0621E+01, &
      2.724542E+01, 4.52128E+01, 3.4780E+00, 4.8626E+01, &
      2.807676E+01, 4.61921E+01, 3.3013E+00, 4.4823E+01, &
      3.001405E+01, 4.70987E+01, 2.9524E+00, 3.7566E+01, &
      3.208502E+01, 4.75564E+01, 2.6447E+00, 3.1539E+01, &
      3.429889E+01, 4.78434E+01, 2.3707E+00, 2.6501E+01, &
      3.666551E+01, 4.80376E+01, 2.1173E+00, 2.2195E+01, &
      3.919543E+01, 4.81669E+01, 1.8884E+00, 1.8566E+01, &
      4.189992E+01, 4.82538E+01, 1.6831E+00, 1.5526E+01, &
      4.479101E+01, 4.83108E+01, 1.4993E+00, 1.2981E+01, &
      4.788159E+01, 4.83465E+01, 1.3351E+00, 1.0855E+01, &
      5.118542E+01, 4.83670E+01, 1.1886E+00, 9.0787E+00, &
      5.471721E+01, 4.84096E+01, 1.0564E+00, 7.5844E+00, &
      5.849270E+01, 4.84082E+01, 9.3595E-01, 6.3215E+00, &
      6.252870E+01, 4.83995E+01, 8.2931E-01, 5.2732E+00, &
      6.684318E+01, 4.83861E+01, 7.3488E-01, 4.4029E+00, &
      7.145536E+01, 4.83697E+01, 6.5126E-01, 3.6803E+00, &
      7.638578E+01, 4.83515E+01, 5.7719E-01, 3.0800E+00, &
      8.165640E+01, 4.83330E+01, 5.1076E-01, 2.5773E+00, &
      8.729069E+01, 4.83135E+01, 4.5045E-01, 2.1536E+00, &
      9.331374E+01, 4.82930E+01, 3.9729E-01, 1.8030E+00, &
      9.975239E+01, 4.82726E+01, 3.5043E-01, 1.5129E+00, &
      1.066353E+02, 4.82525E+01, 3.0911E-01, 1.2725E+00, &
      1.139931E+02, 4.82333E+01, 2.7268E-01, 1.0734E+00, &
      1.218587E+02, 4.82149E+01, 2.4040E-01, 9.0788E-01, &
      1.302669E+02, 4.81974E+01, 2.1190E-01, 7.7050E-01, &
      1.392553E+02, 4.81809E+01, 1.8679E-01, 6.5654E-01, &
      1.488640E+02, 4.81654E+01, 1.6466E-01, 5.6192E-01, &
      1.591356E+02, 4.81510E+01, 1.4517E-01, 4.8328E-01, &
      1.701159E+02, 4.81376E+01, 1.2798E-01, 4.1783E-01, &
      1.818539E+02, 4.81252E+01, 1.1284E-01, 3.6328E-01, &
      1.944018E+02, 4.81138E+01, 9.9495E-02, 3.1773E-01, &
      2.078156E+02, 4.81033E+01, 8.7731E-02, 2.7963E-01, &
      2.221548E+02, 4.80936E+01, 7.7361E-02, 2.4769E-01, &
      2.374835E+02, 4.80848E+01, 6.8220E-02, 2.2084E-01, &
      2.538699E+02, 4.80767E+01, 6.0162E-02, 1.9818E-01, &
      2.713869E+02, 4.80694E+01, 5.3057E-02, 1.7902E-01, &
      2.901126E+02, 4.80627E+01, 4.6794E-02, 1.6272E-01, &
      3.101304E+02, 4.80566E+01, 4.1271E-02, 1.4882E-01, &
      3.315294E+02, 4.80510E+01, 3.6402E-02, 1.3688E-01, &
      3.544049E+02, 4.80460E+01, 3.2108E-02, 1.2659E-01, &
      3.788588E+02, 4.80414E+01, 2.8321E-02, 1.1765E-01, &
      4.050001E+02, 4.80373E+01, 2.4982E-02, 1.0984E-01, &
      4.329451E+02, 4.80335E+01, 2.2038E-02, 1.0296E-01]
   case(49)
      frel = -3.1260E-01
      allocate(nist_tab(880))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.10675E+00, 0.0000E+00, 7.9937E-05, &
      1.142761E-02, 1.92675E+00, 0.0000E+00, 1.1129E-04, &
      1.221612E-02, 1.70380E+00, 0.0000E+00, 1.5407E-04, &
      1.305903E-02, 1.41977E+00, 0.0000E+00, 2.1209E-04, &
      1.396010E-02, 1.03965E+00, 0.0000E+00, 2.9032E-04, &
      1.492335E-02, 4.73339E-01, 0.0000E+00, 3.9521E-04, &
      1.587600E-02, -6.44411E-01, 0.0000E+00, 5.2349E-04, &
      1.595306E-02, -8.37616E-01, 0.0000E+00, 5.3503E-04, &
      1.611900E-02, -1.58291E+00, 0.0000E+00, 5.6048E-04, &
      1.618380E-02, -2.59561E+00, 0.0000E+00, 5.7065E-04, &
      1.628100E-02, -1.65847E+00, 1.9310E+00, 4.3466E+04, &
      1.652400E-02, -9.06518E-01, 1.9665E+00, 4.3615E+04, &
      1.705382E-02, -5.03092E-01, 2.0444E+00, 4.3934E+04, &
      1.823053E-02, -4.08900E-01, 2.2269E+00, 4.4767E+04, &
      1.948844E-02, -6.32308E-01, 2.5294E+00, 4.7567E+04, &
      2.083314E-02, -9.68126E-01, 2.9881E+00, 5.2566E+04, &
      2.227063E-02, -1.34282E+00, 3.6169E+00, 5.9521E+04, &
      2.380730E-02, -1.70181E+00, 4.4309E+00, 6.8209E+04, &
      2.545001E-02, -1.98858E+00, 5.4388E+00, 7.8321E+04, &
      2.720606E-02, -2.14107E+00, 6.6361E+00, 8.9394E+04, &
      2.908327E-02, -2.09479E+00, 7.9989E+00, 1.0080E+05, &
      3.109002E-02, -1.79013E+00, 9.4806E+00, 1.1176E+05, &
      3.323523E-02, -1.18182E+00, 1.1013E+01, 1.2144E+05, &
      3.552846E-02, -2.47938E-01, 1.2510E+01, 1.2904E+05, &
      3.797993E-02, 1.00374E+00, 1.3880E+01, 1.3394E+05, &
      4.060054E-02, 2.53426E+00, 1.5036E+01, 1.3573E+05, &
      4.340198E-02, 4.27733E+00, 1.5908E+01, 1.3433E+05, &
      4.639671E-02, 6.14773E+00, 1.6450E+01, 1.2994E+05, &
      4.959809E-02, 8.01761E+00, 1.6647E+01, 1.2301E+05, &
      5.302035E-02, 9.86577E+00, 1.6513E+01, 1.1414E+05, &
      5.667876E-02, 1.15783E+01, 1.6084E+01, 1.0400E+05, &
      6.058959E-02, 1.30912E+01, 1.5414E+01, 9.3237E+04, &
      6.477028E-02, 1.43529E+01, 1.4565E+01, 8.2413E+04, &
      6.923942E-02, 1.53047E+01, 1.3597E+01, 7.1968E+04, &
      7.401695E-02, 1.57640E+01, 1.2568E+01, 6.2231E+04, &
      7.585200E-02, 1.55786E+01, 1.2186E+01, 5.8880E+04, &
      7.701300E-02, 1.48156E+01, 1.1950E+01, 5.6867E+04, &
      7.732260E-02, 1.37034E+01, 1.1888E+01, 5.6344E+04, &
      7.778700E-02, 1.50356E+01, 1.4087E+01, 6.6368E+04, &
      7.894800E-02, 1.62971E+01, 1.3725E+01, 6.3716E+04, &
      7.912411E-02, 1.64066E+01, 1.3672E+01, 6.3326E+04, &
      8.458368E-02, 1.80342E+01, 1.2163E+01, 5.2700E+04, &
      9.041995E-02, 1.86695E+01, 1.0863E+01, 4.4030E+04, &
      9.665893E-02, 1.89771E+01, 9.7759E+00, 3.7066E+04, &
      1.033284E-01, 1.91093E+01, 8.8509E+00, 3.1393E+04, &
      1.104581E-01, 1.91198E+01, 8.0593E+00, 2.6740E+04, &
      1.180797E-01, 1.90030E+01, 7.3815E+00, 2.2910E+04, &
      1.194620E-01, 1.89541E+01, 7.2738E+00, 2.2315E+04, &
      1.212905E-01, 1.88183E+01, 7.1376E+00, 2.1567E+04, &
      1.217781E-01, 1.86749E+01, 7.1024E+00, 2.1375E+04, &
      1.225095E-01, 1.88039E+01, 7.3213E+00, 2.1902E+04, &
      1.243380E-01, 1.89026E+01, 7.2014E+00, 2.1226E+04, &
      1.262272E-01, 1.89310E+01, 7.0840E+00, 2.0568E+04, &
      1.349368E-01, 1.89234E+01, 6.6167E+00, 1.7972E+04, &
      1.442475E-01, 1.88536E+01, 6.2239E+00, 1.5814E+04, &
      1.542005E-01, 1.87580E+01, 5.8934E+00, 1.4008E+04, &
      1.648404E-01, 1.86457E+01, 5.6158E+00, 1.2487E+04, &
      1.762144E-01, 1.85194E+01, 5.3826E+00, 1.1196E+04, &
      1.883732E-01, 1.83786E+01, 5.1860E+00, 1.0091E+04, &
      2.013709E-01, 1.82195E+01, 5.0191E+00, 9.1359E+03, &
      2.152655E-01, 1.80359E+01, 4.8758E+00, 8.3024E+03, &
      2.301188E-01, 1.78183E+01, 4.7503E+00, 7.5670E+03, &
      2.459970E-01, 1.75604E+01, 4.6379E+00, 6.9113E+03, &
      2.629708E-01, 1.72294E+01, 4.5342E+00, 6.3210E+03, &
      2.811158E-01, 1.68057E+01, 4.4356E+00, 5.7848E+03, &
      3.005128E-01, 1.62501E+01, 4.3391E+00, 5.2940E+03, &
      3.212482E-01, 1.55051E+01, 4.2591E+00, 4.8614E+03, &
      3.434143E-01, 1.44769E+01, 4.1793E+00, 4.4629E+03, &
      3.671099E-01, 1.29744E+01, 4.0943E+00, 4.0904E+03, &
      3.924405E-01, 1.05627E+01, 4.0036E+00, 3.7421E+03, &
      4.195189E-01, 5.77615E+00, 3.9067E+00, 3.4164E+03, &
      4.342380E-01, -2.31978E-01, 3.8542E+00, 3.2565E+03, &
      4.408845E-01, -7.80538E+00, 3.8306E+00, 3.1879E+03, &
      4.417840E-01, -1.03777E+01, 3.8274E+00, 3.1788E+03, &
      4.426569E-01, -1.54694E+01, 3.8243E+00, 3.1699E+03, &
      4.453155E-01, -9.57113E+00, 1.7656E+01, 1.4535E+04, &
      4.484657E-01, -8.22734E+00, 1.7680E+01, 1.4452E+04, &
      4.485460E-01, -8.26674E+00, 1.7681E+01, 1.4450E+04, &
      4.503492E-01, -1.18403E+01, 1.7695E+01, 1.4404E+04, &
      4.519620E-01, -8.09054E+00, 2.7173E+01, 2.2038E+04, &
      4.530540E-01, -5.57207E+00, 2.7187E+01, 2.1996E+04, &
      4.598160E-01, 9.83374E-01, 2.7272E+01, 2.1740E+04, &
      4.794098E-01, 8.30071E+00, 2.7512E+01, 2.1036E+04, &
      5.124891E-01, 1.46157E+01, 2.7734E+01, 1.9837E+04, &
      5.478508E-01, 1.91024E+01, 2.7329E+01, 1.8287E+04, &
      5.856525E-01, 2.24492E+01, 2.6443E+01, 1.6553E+04, &
      6.260625E-01, 2.46655E+01, 2.5263E+01, 1.4793E+04, &
      6.510140E-01, 2.49558E+01, 2.4496E+01, 1.3795E+04, &
      6.609785E-01, 2.39656E+01, 2.4189E+01, 1.3417E+04, &
      6.636357E-01, 2.23863E+01, 2.4108E+01, 1.3319E+04, &
      6.676215E-01, 2.42457E+01, 2.7294E+01, 1.4988E+04, &
      6.692609E-01, 2.47328E+01, 2.7245E+01, 1.4924E+04, &
      6.775860E-01, 2.60673E+01, 2.6995E+01, 1.4606E+04, &
      6.881560E-01, 2.69478E+01, 2.6681E+01, 1.4215E+04, &
      6.986890E-01, 2.71941E+01, 2.6373E+01, 1.3839E+04, &
      7.014978E-01, 2.66741E+01, 2.6292E+01, 1.3741E+04, &
      7.057110E-01, 2.77002E+01, 2.7594E+01, 1.4335E+04, &
      7.154399E-01, 2.89399E+01, 2.7322E+01, 1.4002E+04, &
      7.162440E-01, 2.90168E+01, 2.7300E+01, 1.3975E+04, &
      7.648052E-01, 3.22695E+01, 2.5935E+01, 1.2434E+04, &
      8.090880E-01, 3.39638E+01, 2.4513E+01, 1.1110E+04, &
      8.175768E-01, 3.40589E+01, 2.4251E+01, 1.0877E+04, &
      8.214720E-01, 3.39818E+01, 2.4132E+01, 1.0772E+04, &
      8.247744E-01, 3.35525E+01, 2.4032E+01, 1.0685E+04, &
      8.297280E-01, 3.42947E+01, 2.4967E+01, 1.1034E+04, &
      8.421120E-01, 3.52159E+01, 2.4598E+01, 1.0711E+04, &
      8.739896E-01, 3.66029E+01, 2.3685E+01, 9.9381E+03, &
      9.342948E-01, 3.84530E+01, 2.2110E+01, 8.6791E+03, &
      9.987612E-01, 4.00262E+01, 2.0631E+01, 7.5769E+03, &
      1.067676E+00, 4.13313E+01, 1.8950E+01, 6.5113E+03, &
      1.141345E+00, 4.22130E+01, 1.7387E+01, 5.5897E+03, &
      1.220098E+00, 4.28776E+01, 1.5906E+01, 4.7849E+03, &
      1.304285E+00, 4.33582E+01, 1.4538E+01, 4.0920E+03, &
      1.394281E+00, 4.37113E+01, 1.3290E+01, 3.5004E+03, &
      1.490486E+00, 4.39566E+01, 1.2140E+01, 2.9922E+03, &
      1.593329E+00, 4.41135E+01, 1.1086E+01, 2.5572E+03, &
      1.703269E+00, 4.42037E+01, 1.0130E+01, 2.1868E+03, &
      1.820795E+00, 4.42438E+01, 9.2342E+00, 1.8659E+03, &
      1.946430E+00, 4.41999E+01, 8.3710E+00, 1.5834E+03, &
      2.080733E+00, 4.40835E+01, 7.6005E+00, 1.3458E+03, &
      2.224304E+00, 4.39110E+01, 6.9120E+00, 1.1460E+03, &
      2.377781E+00, 4.36841E+01, 6.2955E+00, 9.7729E+02, &
      2.541848E+00, 4.34025E+01, 5.7429E+00, 8.3486E+02, &
      2.717235E+00, 4.30533E+01, 5.2385E+00, 7.1325E+02, &
      2.904724E+00, 4.26007E+01, 4.7633E+00, 6.0755E+02, &
      3.105150E+00, 4.19395E+01, 4.2790E+00, 5.1145E+02, &
      3.319406E+00, 4.08316E+01, 3.8064E+00, 4.2648E+02, &
      3.548445E+00, 3.86509E+01, 3.3869E+00, 3.5586E+02, &
      3.655498E+00, 3.63248E+01, 3.2169E+00, 3.2848E+02, &
      3.711449E+00, 3.28236E+01, 3.1337E+00, 3.1536E+02, &
      3.726370E+00, 2.88247E+01, 3.1121E+00, 3.1199E+02, &
      3.748750E+00, 3.27064E+01, 1.0810E+01, 1.0627E+03, &
      3.793288E+00, 3.55740E+01, 1.0551E+01, 1.0253E+03, &
      3.804702E+00, 3.59170E+01, 1.0487E+01, 1.0159E+03, &
      3.859240E+00, 3.67476E+01, 1.0186E+01, 9.7308E+02, &
      3.918310E+00, 3.60579E+01, 9.8767E+00, 9.2955E+02, &
      3.934062E+00, 3.43002E+01, 9.8038E+00, 9.1904E+02, &
      3.957690E+00, 3.65333E+01, 1.3528E+01, 1.2585E+03, &
      4.016760E+00, 3.88109E+01, 1.3157E+01, 1.2061E+03, &
      4.055024E+00, 3.95586E+01, 1.2926E+01, 1.1739E+03, &
      4.152750E+00, 4.06024E+01, 1.2377E+01, 1.0979E+03, &
      4.216313E+00, 4.04916E+01, 1.2056E+01, 1.0534E+03, &
      4.233263E+00, 3.97108E+01, 1.1973E+01, 1.0421E+03, &
      4.258688E+00, 4.08706E+01, 1.3698E+01, 1.1843E+03, &
      4.322250E+00, 4.22127E+01, 1.3387E+01, 1.1405E+03, &
      4.334821E+00, 4.23881E+01, 1.3327E+01, 1.1321E+03, &
      4.633924E+00, 4.48579E+01, 1.2054E+01, 9.5858E+02, &
      4.953664E+00, 4.62849E+01, 1.0916E+01, 8.1257E+02, &
      5.295467E+00, 4.72544E+01, 9.8417E+00, 6.8590E+02, &
      5.660855E+00, 4.79270E+01, 8.8515E+00, 5.7761E+02, &
      6.051453E+00, 4.83927E+01, 7.9492E+00, 4.8575E+02, &
      6.469004E+00, 4.87137E+01, 7.1376E+00, 4.0848E+02, &
      6.915365E+00, 4.89322E+01, 6.4106E+00, 3.4363E+02, &
      7.392525E+00, 4.90788E+01, 5.7615E+00, 2.8931E+02, &
      7.902609E+00, 4.92898E+01, 5.1709E+00, 2.4327E+02, &
      8.447890E+00, 4.93396E+01, 4.6378E+00, 2.0446E+02, &
      9.030794E+00, 4.94159E+01, 4.1462E+00, 1.7132E+02, &
      9.653919E+00, 4.93952E+01, 3.6763E+00, 1.4243E+02, &
      1.032004E+01, 4.93405E+01, 3.2627E+00, 1.1855E+02, &
      1.103212E+01, 4.92665E+01, 2.8984E+00, 9.8787E+01, &
      1.179334E+01, 4.91806E+01, 2.5771E+00, 8.2418E+01, &
      1.260708E+01, 4.90870E+01, 2.2933E+00, 6.8836E+01, &
      1.347697E+01, 4.89881E+01, 2.0411E+00, 5.7519E+01, &
      1.440688E+01, 4.88850E+01, 1.8179E+00, 4.8111E+01, &
      1.540095E+01, 4.87786E+01, 1.6207E+00, 4.0296E+01, &
      1.646362E+01, 4.86691E+01, 1.4441E+00, 3.3746E+01, &
      1.759961E+01, 4.85527E+01, 1.2838E+00, 2.8210E+01, &
      1.881398E+01, 4.84256E+01, 1.1395E+00, 2.3559E+01, &
      2.011215E+01, 4.82806E+01, 1.0092E+00, 1.9644E+01, &
      2.149988E+01, 4.81090E+01, 8.9450E-01, 1.6404E+01, &
      2.298338E+01, 4.78919E+01, 7.9358E-01, 1.3717E+01, &
      2.456923E+01, 4.75830E+01, 7.0468E-01, 1.1489E+01, &
      2.626450E+01, 4.70178E+01, 6.2629E-01, 9.6380E+00, &
      2.738110E+01, 4.60767E+01, 5.7897E-01, 8.6019E+00, &
      2.780020E+01, 4.48110E+01, 5.6192E-01, 8.2441E+00, &
      2.791196E+01, 4.32899E+01, 5.5751E-01, 8.1523E+00, &
      2.807676E+01, 4.48120E+01, 3.5483E+00, 4.7142E+01, &
      2.807960E+01, 4.48324E+01, 3.5477E+00, 4.7129E+01, &
      2.849870E+01, 4.62279E+01, 3.4601E+00, 4.5306E+01, &
      3.001405E+01, 4.75892E+01, 3.1720E+00, 3.9491E+01, &
      3.208502E+01, 4.82836E+01, 2.8394E+00, 3.3130E+01, &
      3.429889E+01, 4.86701E+01, 2.5429E+00, 2.7812E+01, &
      3.666551E+01, 4.89207E+01, 2.2763E+00, 2.3342E+01, &
      3.919543E+01, 4.90881E+01, 2.0305E+00, 1.9527E+01, &
      4.189992E+01, 4.92000E+01, 1.8103E+00, 1.6333E+01, &
      4.479101E+01, 4.92746E+01, 1.6133E+00, 1.3660E+01, &
      4.788159E+01, 4.93230E+01, 1.4372E+00, 1.1426E+01, &
      5.118542E+01, 4.93526E+01, 1.2802E+00, 9.5587E+00, &
      5.471721E+01, 4.93689E+01, 1.1402E+00, 8.0002E+00, &
      5.849270E+01, 4.94076E+01, 1.0121E+00, 6.6787E+00, &
      6.252870E+01, 4.94035E+01, 8.9714E-01, 5.5716E+00, &
      6.684318E+01, 4.93931E+01, 7.9529E-01, 4.6523E+00, &
      7.145536E+01, 4.93788E+01, 7.0507E-01, 3.8886E+00, &
      7.638578E+01, 4.93619E+01, 6.2514E-01, 3.2541E+00, &
      8.165640E+01, 4.93449E+01, 5.5344E-01, 2.7228E+00, &
      8.729069E+01, 4.93260E+01, 4.8826E-01, 2.2746E+00, &
      9.331374E+01, 4.93055E+01, 4.3080E-01, 1.9036E+00, &
      9.975239E+01, 4.92849E+01, 3.8012E-01, 1.5964E+00, &
      1.066353E+02, 4.92645E+01, 3.3543E-01, 1.3420E+00, &
      1.139931E+02, 4.92447E+01, 2.9601E-01, 1.1312E+00, &
      1.218587E+02, 4.92257E+01, 2.6122E-01, 9.5641E-01, &
      1.302669E+02, 4.92077E+01, 2.3040E-01, 8.1098E-01, &
      1.392553E+02, 4.91906E+01, 2.0323E-01, 6.9032E-01, &
      1.488640E+02, 4.91745E+01, 1.7927E-01, 5.9011E-01, &
      1.591356E+02, 4.91595E+01, 1.5814E-01, 5.0679E-01, &
      1.701159E+02, 4.91456E+01, 1.3952E-01, 4.3744E-01, &
      1.818539E+02, 4.91326E+01, 1.2309E-01, 3.7964E-01, &
      1.944018E+02, 4.91206E+01, 1.0860E-01, 3.3138E-01, &
      2.078156E+02, 4.91096E+01, 9.5820E-02, 2.9102E-01, &
      2.221548E+02, 4.90995E+01, 8.4549E-02, 2.5719E-01, &
      2.374835E+02, 4.90902E+01, 7.4607E-02, 2.2877E-01, &
      2.538699E+02, 4.90817E+01, 6.5837E-02, 2.0480E-01, &
      2.713869E+02, 4.90739E+01, 5.8100E-02, 1.8453E-01, &
      2.901126E+02, 4.90668E+01, 5.1275E-02, 1.6732E-01, &
      3.101304E+02, 4.90603E+01, 4.5253E-02, 1.5266E-01, &
      3.315294E+02, 4.90545E+01, 3.9939E-02, 1.4009E-01, &
      3.544049E+02, 4.90491E+01, 3.5251E-02, 1.2926E-01, &
      3.788588E+02, 4.90443E+01, 3.1115E-02, 1.1988E-01, &
      4.050001E+02, 4.90399E+01, 2.7464E-02, 1.1170E-01, &
      4.329451E+02, 4.90359E+01, 2.4243E-02, 1.0452E-01]
   case(50)
      frel = -3.2820E-01
      allocate(nist_tab(876))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 8.40533E+00, 0.0000E+00, 7.9989E-05, &
      1.142761E-02, 8.31660E+00, 0.0000E+00, 1.1133E-04, &
      1.221612E-02, 8.21206E+00, 0.0000E+00, 1.5407E-04, &
      1.305903E-02, 8.08822E+00, 0.0000E+00, 2.1202E-04, &
      1.396010E-02, 7.94054E+00, 0.0000E+00, 2.9014E-04, &
      1.492335E-02, 7.76297E+00, 0.0000E+00, 3.9485E-04, &
      1.595306E-02, 7.54725E+00, 0.0000E+00, 5.3439E-04, &
      1.705382E-02, 7.28160E+00, 0.0000E+00, 7.1929E-04, &
      1.823053E-02, 6.94848E+00, 0.0000E+00, 9.6289E-04, &
      1.948844E-02, 6.51984E+00, 0.0000E+00, 1.2820E-03, &
      2.083314E-02, 5.94558E+00, 0.0000E+00, 1.6978E-03, &
      2.227063E-02, 5.11472E+00, 0.0000E+00, 2.2363E-03, &
      2.342200E-02, 4.07447E+00, 0.0000E+00, 2.7441E-03, &
      2.378050E-02, 3.44880E+00, 0.0000E+00, 2.9169E-03, &
      2.380730E-02, 3.36580E+00, 0.0000E+00, 2.9301E-03, &
      2.387610E-02, 2.99671E+00, 0.0000E+00, 2.9642E-03, &
      2.401950E-02, 3.16193E+00, 7.1303E-01, 1.0525E+04, &
      2.437800E-02, 3.07327E+00, 8.6220E-01, 1.2539E+04, &
      2.545001E-02, 2.19506E+00, 1.4971E+00, 2.0856E+04, &
      2.720606E-02, 6.54972E-01, 3.4165E+00, 4.4523E+04, &
      2.908327E-02, -1.53839E-01, 6.3990E+00, 7.8007E+04, &
      3.109002E-02, 3.64264E-01, 9.9099E+00, 1.1301E+05, &
      3.323523E-02, 2.36543E+00, 1.3138E+01, 1.4015E+05, &
      3.552846E-02, 5.47285E+00, 1.5366E+01, 1.5334E+05, &
      3.797993E-02, 9.01769E+00, 1.6277E+01, 1.5194E+05, &
      4.060054E-02, 1.23644E+01, 1.5966E+01, 1.3942E+05, &
      4.340198E-02, 1.51082E+01, 1.4784E+01, 1.2077E+05, &
      4.639671E-02, 1.71028E+01, 1.3132E+01, 1.0035E+05, &
      4.959809E-02, 1.83880E+01, 1.1341E+01, 8.1072E+04, &
      5.302035E-02, 1.90958E+01, 9.6305E+00, 6.4398E+04, &
      5.667876E-02, 1.93800E+01, 8.1137E+00, 5.0753E+04, &
      6.058959E-02, 1.93766E+01, 6.8318E+00, 3.9976E+04, &
      6.477028E-02, 1.91892E+01, 5.7821E+00, 3.1650E+04, &
      6.923942E-02, 1.88876E+01, 4.9401E+00, 2.5296E+04, &
      7.401695E-02, 1.85413E+01, 4.2744E+00, 2.0474E+04, &
      7.912411E-02, 1.80845E+01, 3.7535E+00, 1.6819E+04, &
      8.458368E-02, 1.74338E+01, 3.3495E+00, 1.4040E+04, &
      8.682800E-02, 1.69441E+01, 3.2177E+00, 1.3139E+04, &
      8.815700E-02, 1.61855E+01, 3.1474E+00, 1.2658E+04, &
      8.851140E-02, 1.53182E+01, 3.1295E+00, 1.2536E+04, &
      8.904300E-02, 1.62085E+01, 4.7868E+00, 1.9059E+04, &
      9.041995E-02, 1.69648E+01, 4.6507E+00, 1.8236E+04, &
      9.665893E-02, 1.75538E+01, 4.1380E+00, 1.5178E+04, &
      1.033284E-01, 1.75484E+01, 3.7616E+00, 1.2907E+04, &
      1.104581E-01, 1.74115E+01, 3.5187E+00, 1.1294E+04, &
      1.180797E-01, 1.72315E+01, 3.3688E+00, 1.0115E+04, &
      1.262272E-01, 1.70235E+01, 3.2869E+00, 9.2326E+03, &
      1.337700E-01, 1.67668E+01, 3.2580E+00, 8.6354E+03, &
      1.349368E-01, 1.66942E+01, 3.2565E+00, 8.5568E+03, &
      1.358175E-01, 1.66008E+01, 3.2558E+00, 8.4996E+03, &
      1.363635E-01, 1.64394E+01, 3.2556E+00, 8.4650E+03, &
      1.371825E-01, 1.65757E+01, 3.5528E+00, 9.1826E+03, &
      1.392300E-01, 1.66725E+01, 3.5593E+00, 9.0641E+03, &
      1.442475E-01, 1.66976E+01, 3.5821E+00, 8.8049E+03, &
      1.542005E-01, 1.66565E+01, 3.6492E+00, 8.3911E+03, &
      1.648404E-01, 1.66020E+01, 3.7388E+00, 8.0422E+03, &
      1.762144E-01, 1.65491E+01, 3.8437E+00, 7.7343E+03, &
      1.883732E-01, 1.64986E+01, 3.9584E+00, 7.4512E+03, &
      2.013709E-01, 1.64471E+01, 4.0778E+00, 7.1807E+03, &
      2.152655E-01, 1.63884E+01, 4.1973E+00, 6.9143E+03, &
      2.301188E-01, 1.63138E+01, 4.3123E+00, 6.6454E+03, &
      2.459970E-01, 1.62115E+01, 4.4188E+00, 6.3703E+03, &
      2.629708E-01, 1.60658E+01, 4.5133E+00, 6.0867E+03, &
      2.811158E-01, 1.58589E+01, 4.5924E+00, 5.7940E+03, &
      3.005128E-01, 1.55550E+01, 4.6539E+00, 5.4929E+03, &
      3.212482E-01, 1.51156E+01, 4.6958E+00, 5.1849E+03, &
      3.434143E-01, 1.44774E+01, 4.7170E+00, 4.8726E+03, &
      3.671099E-01, 1.35372E+01, 4.7169E+00, 4.5583E+03, &
      3.924405E-01, 1.21052E+01, 4.6945E+00, 4.2443E+03, &
      4.195189E-01, 9.77113E+00, 4.6509E+00, 3.9340E+03, &
      4.484657E-01, 5.34162E+00, 4.5879E+00, 3.6307E+03, &
      4.751040E-01, -5.39380E+00, 4.4758E+00, 3.3440E+03, &
      4.794098E-01, -1.00246E+01, 4.4569E+00, 3.3001E+03, &
      4.823760E-01, -1.59897E+01, 4.4439E+00, 3.2702E+03, &
      4.834340E-01, -2.00565E+01, 4.4393E+00, 3.2597E+03, &
      4.843152E-01, -2.71125E+01, 4.4354E+00, 3.2509E+03, &
      4.872240E-01, -1.79385E+01, 2.4794E+01, 1.8046E+04, &
      4.908335E-01, -1.55411E+01, 2.4594E+01, 1.7769E+04, &
      4.928067E-01, -2.05115E+01, 2.4486E+01, 1.7620E+04, &
      4.944960E-01, -1.51896E+01, 3.8081E+01, 2.7307E+04, &
      4.957665E-01, -1.10983E+01, 3.7968E+01, 2.7156E+04, &
      5.031660E-01, -1.09926E+00, 3.7320E+01, 2.6300E+04, &
      5.124891E-01, 5.02345E+00, 3.6532E+01, 2.5277E+04, &
      5.478508E-01, 1.60755E+01, 3.3809E+01, 2.1884E+04, &
      5.856525E-01, 2.17958E+01, 3.1371E+01, 1.8996E+04, &
      6.260625E-01, 2.54521E+01, 2.9150E+01, 1.6512E+04, &
      6.692609E-01, 2.76980E+01, 2.7071E+01, 1.4346E+04, &
      7.001120E-01, 2.79859E+01, 2.5730E+01, 1.3035E+04, &
      7.108280E-01, 2.69092E+01, 2.5288E+01, 1.2618E+04, &
      7.136856E-01, 2.52636E+01, 2.5173E+01, 1.2510E+04, &
      7.154399E-01, 2.57353E+01, 2.8519E+01, 1.4139E+04, &
      7.179720E-01, 2.71651E+01, 2.8418E+01, 1.4039E+04, &
      7.286880E-01, 2.89963E+01, 2.7994E+01, 1.3626E+04, &
      7.412720E-01, 2.99222E+01, 2.7510E+01, 1.3163E+04, &
      7.526180E-01, 3.00970E+01, 2.7086E+01, 1.2765E+04, &
      7.556436E-01, 2.95426E+01, 2.6974E+01, 1.2662E+04, &
      7.601820E-01, 3.05673E+01, 2.8271E+01, 1.3191E+04, &
      7.648052E-01, 3.12055E+01, 2.8106E+01, 1.3035E+04, &
      7.715280E-01, 3.18439E+01, 2.7869E+01, 1.2813E+04, &
      8.175768E-01, 3.44946E+01, 2.6343E+01, 1.1429E+04, &
      8.661240E-01, 3.60710E+01, 2.4806E+01, 1.0160E+04, &
      8.739896E-01, 3.61442E+01, 2.4568E+01, 9.9724E+03, &
      8.793810E-01, 3.60506E+01, 2.4407E+01, 9.8463E+03, &
      8.829162E-01, 3.56078E+01, 2.4302E+01, 9.7649E+03, &
      8.882190E-01, 3.63432E+01, 2.5241E+01, 1.0081E+04, &
      9.014760E-01, 3.72414E+01, 2.4857E+01, 9.7821E+03, &
      9.342948E-01, 3.85344E+01, 2.3947E+01, 9.0938E+03, &
      9.987612E-01, 4.03800E+01, 2.2333E+01, 7.9344E+03, &
      1.067676E+00, 4.17705E+01, 2.0514E+01, 6.8186E+03, &
      1.141345E+00, 4.27950E+01, 1.8831E+01, 5.8565E+03, &
      1.220098E+00, 4.35956E+01, 1.7281E+01, 5.0286E+03, &
      1.304285E+00, 4.42078E+01, 1.5808E+01, 4.3041E+03, &
      1.394281E+00, 4.46469E+01, 1.4442E+01, 3.6793E+03, &
      1.490486E+00, 4.49678E+01, 1.3197E+01, 3.1462E+03, &
      1.593329E+00, 4.51804E+01, 1.2054E+01, 2.6893E+03, &
      1.703269E+00, 4.53112E+01, 1.1010E+01, 2.2990E+03, &
      1.820795E+00, 4.53763E+01, 1.0055E+01, 1.9651E+03, &
      1.946430E+00, 4.53928E+01, 9.1859E+00, 1.6804E+03, &
      2.080733E+00, 4.53446E+01, 8.3312E+00, 1.4268E+03, &
      2.224304E+00, 4.52231E+01, 7.5618E+00, 1.2124E+03, &
      2.377781E+00, 4.50418E+01, 6.8741E+00, 1.0320E+03, &
      2.541848E+00, 4.48028E+01, 6.2588E+00, 8.7986E+02, &
      2.717235E+00, 4.45039E+01, 5.7068E+00, 7.5135E+02, &
      2.904724E+00, 4.41350E+01, 5.2058E+00, 6.4200E+02, &
      3.105150E+00, 4.36444E+01, 4.6802E+00, 5.4081E+02, &
      3.319406E+00, 4.28802E+01, 4.1619E+00, 4.5080E+02, &
      3.548445E+00, 4.16355E+01, 3.7073E+00, 3.7649E+02, &
      3.793288E+00, 3.89148E+01, 3.3031E+00, 3.1462E+02, &
      3.850224E+00, 3.75113E+01, 3.2198E+00, 3.0233E+02, &
      3.909156E+00, 3.40273E+01, 3.1372E+00, 2.9033E+02, &
      3.924871E+00, 3.00304E+01, 3.1158E+00, 2.8724E+02, &
      3.948444E+00, 3.39408E+01, 1.0820E+01, 9.7734E+02, &
      4.007376E+00, 3.71907E+01, 1.0465E+01, 9.3162E+02, &
      4.055024E+00, 3.79931E+01, 1.0191E+01, 8.9671E+02, &
      4.072978E+00, 3.81101E+01, 1.0090E+01, 8.8401E+02, &
      4.135319E+00, 3.73576E+01, 9.7618E+00, 8.4255E+02, &
      4.151944E+00, 3.55879E+01, 9.6868E+00, 8.3278E+02, &
      4.176880E+00, 3.78115E+01, 1.3408E+01, 1.1437E+03, &
      4.239222E+00, 4.00581E+01, 1.3018E+01, 1.0942E+03, &
      4.334821E+00, 4.14391E+01, 1.2464E+01, 1.0249E+03, &
      4.375406E+00, 4.17235E+01, 1.2248E+01, 9.9792E+02, &
      4.442377E+00, 4.16034E+01, 1.1923E+01, 9.5698E+02, &
      4.460236E+00, 4.08219E+01, 1.1840E+01, 9.4656E+02, &
      4.487024E+00, 4.19740E+01, 1.3561E+01, 1.0769E+03, &
      4.553994E+00, 4.33023E+01, 1.3250E+01, 1.0368E+03, &
      4.633924E+00, 4.41919E+01, 1.2901E+01, 9.9231E+02, &
      4.953664E+00, 4.62573E+01, 1.1694E+01, 8.4202E+02, &
      5.295467E+00, 4.75557E+01, 1.0597E+01, 7.1432E+02, &
      5.660855E+00, 4.84498E+01, 9.5551E+00, 6.0302E+02, &
      6.051453E+00, 4.90715E+01, 8.5908E+00, 5.0768E+02, &
      6.469004E+00, 4.95000E+01, 7.7135E+00, 4.2688E+02, &
      6.915365E+00, 4.97938E+01, 6.9246E+00, 3.5893E+02, &
      7.392525E+00, 4.99926E+01, 6.2190E+00, 3.0197E+02, &
      7.902609E+00, 5.02028E+01, 5.5891E+00, 2.5424E+02, &
      8.447890E+00, 5.03272E+01, 5.0166E+00, 2.1382E+02, &
      9.030794E+00, 5.04269E+01, 4.4908E+00, 1.7940E+02, &
      9.653919E+00, 5.04277E+01, 3.9815E+00, 1.4911E+02, &
      1.032004E+01, 5.03869E+01, 3.5329E+00, 1.2408E+02, &
      1.103212E+01, 5.03224E+01, 3.1377E+00, 1.0337E+02, &
      1.179334E+01, 5.02428E+01, 2.7892E+00, 8.6205E+01, &
      1.260708E+01, 5.01534E+01, 2.4818E+00, 7.1983E+01, &
      1.347697E+01, 5.00578E+01, 2.2102E+00, 6.0177E+01, &
      1.440688E+01, 4.99577E+01, 1.9686E+00, 5.0332E+01, &
      1.540095E+01, 4.98538E+01, 1.7545E+00, 4.2136E+01, &
      1.646362E+01, 4.97468E+01, 1.5653E+00, 3.5323E+01, &
      1.759961E+01, 4.96362E+01, 1.3947E+00, 2.9587E+01, &
      1.881398E+01, 4.95174E+01, 1.2404E+00, 2.4751E+01, &
      2.011215E+01, 4.93865E+01, 1.1005E+00, 2.0669E+01, &
      2.149988E+01, 4.92350E+01, 9.7529E-01, 1.7250E+01, &
      2.298338E+01, 4.90522E+01, 8.6507E-01, 1.4418E+01, &
      2.456923E+01, 4.88135E+01, 7.6798E-01, 1.2070E+01, &
      2.626450E+01, 4.84518E+01, 6.8240E-01, 1.0119E+01, &
      2.807676E+01, 4.76648E+01, 6.0051E-01, 8.4169E+00, &
      2.861610E+01, 4.70981E+01, 5.7853E-01, 7.9817E+00, &
      2.905410E+01, 4.58386E+01, 5.6159E-01, 7.6514E+00, &
      2.917090E+01, 4.43250E+01, 5.5721E-01, 7.5665E+00, &
      2.934610E+01, 4.58608E+01, 3.5328E+00, 4.3469E+01, &
      2.978410E+01, 4.72498E+01, 3.4443E+00, 4.1774E+01, &
      3.001405E+01, 4.75904E+01, 3.3992E+00, 4.0919E+01, &
      3.208502E+01, 4.88917E+01, 3.0381E+00, 3.4275E+01, &
      3.429889E+01, 4.94402E+01, 2.7208E+00, 2.8771E+01, &
      3.666551E+01, 4.97671E+01, 2.4382E+00, 2.4171E+01, &
      3.919543E+01, 4.99853E+01, 2.1809E+00, 2.0274E+01, &
      4.189992E+01, 5.01305E+01, 1.9457E+00, 1.6966E+01, &
      4.479101E+01, 5.02279E+01, 1.7347E+00, 1.4195E+01, &
      4.788159E+01, 5.02924E+01, 1.5459E+00, 1.1874E+01, &
      5.118542E+01, 5.03335E+01, 1.3771E+00, 9.9338E+00, &
      5.471721E+01, 5.03579E+01, 1.2266E+00, 8.3131E+00, &
      5.849270E+01, 5.04028E+01, 1.0924E+00, 6.9599E+00, &
      6.252870E+01, 5.04047E+01, 9.6868E-01, 5.8069E+00, &
      6.684318E+01, 5.03982E+01, 8.5903E-01, 4.8490E+00, &
      7.145536E+01, 5.03866E+01, 7.6186E-01, 4.0530E+00, &
      7.638578E+01, 5.03718E+01, 6.7575E-01, 3.3915E+00, &
      8.165640E+01, 5.03569E+01, 5.9847E-01, 2.8375E+00, &
      8.729069E+01, 5.03380E+01, 5.2820E-01, 2.3698E+00, &
      9.331374E+01, 5.03177E+01, 4.6621E-01, 1.9826E+00, &
      9.975239E+01, 5.02969E+01, 4.1152E-01, 1.6620E+00, &
      1.066353E+02, 5.02763E+01, 3.6327E-01, 1.3964E+00, &
      1.139931E+02, 5.02561E+01, 3.2071E-01, 1.1763E+00, &
      1.218587E+02, 5.02366E+01, 2.8314E-01, 9.9375E-01, &
      1.302669E+02, 5.02181E+01, 2.4995E-01, 8.4214E-01, &
      1.392553E+02, 5.02004E+01, 2.2061E-01, 7.1614E-01, &
      1.488640E+02, 5.01838E+01, 1.9472E-01, 6.1145E-01, &
      1.591356E+02, 5.01682E+01, 1.7188E-01, 5.2441E-01, &
      1.701159E+02, 5.01536E+01, 1.5173E-01, 4.5194E-01, &
      1.818539E+02, 5.01401E+01, 1.3395E-01, 3.9154E-01, &
      1.944018E+02, 5.01276E+01, 1.1825E-01, 3.4110E-01, &
      2.078156E+02, 5.01161E+01, 1.0440E-01, 2.9893E-01, &
      2.221548E+02, 5.01054E+01, 9.2181E-02, 2.6358E-01, &
      2.374835E+02, 5.00957E+01, 8.1392E-02, 2.3389E-01, &
      2.538699E+02, 5.00867E+01, 7.1870E-02, 2.0889E-01, &
      2.713869E+02, 5.00785E+01, 6.3464E-02, 1.8775E-01, &
      2.901126E+02, 5.00711E+01, 5.6043E-02, 1.6982E-01, &
      3.101304E+02, 5.00642E+01, 4.9492E-02, 1.5456E-01, &
      3.315294E+02, 5.00580E+01, 4.3709E-02, 1.4150E-01, &
      3.544049E+02, 5.00524E+01, 3.8603E-02, 1.3027E-01, &
      3.788588E+02, 5.00473E+01, 3.4094E-02, 1.2056E-01, &
      4.050001E+02, 5.00426E+01, 3.0114E-02, 1.1211E-01, &
      4.329451E+02, 5.00384E+01, 2.6598E-02, 1.0471E-01]
   case(51)
      frel = -3.4500E-01
      allocate(nist_tab(876))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.60066E+00, 0.0000E+00, 8.4545E-05, &
      1.142761E-02, 5.50128E+00, 0.0000E+00, 1.1749E-04, &
      1.221612E-02, 5.38564E+00, 0.0000E+00, 1.6235E-04, &
      1.305903E-02, 5.25072E+00, 0.0000E+00, 2.2308E-04, &
      1.396010E-02, 5.09278E+00, 0.0000E+00, 3.0484E-04, &
      1.492335E-02, 4.90714E+00, 0.0000E+00, 4.1428E-04, &
      1.595306E-02, 4.68795E+00, 0.0000E+00, 5.5991E-04, &
      1.705382E-02, 4.42767E+00, 0.0000E+00, 7.5262E-04, &
      1.823053E-02, 4.11650E+00, 0.0000E+00, 1.0062E-03, &
      1.948844E-02, 3.74142E+00, 0.0000E+00, 1.3379E-03, &
      2.083314E-02, 3.28467E+00, 0.0000E+00, 1.7696E-03, &
      2.227063E-02, 2.72124E+00, 0.0000E+00, 2.3281E-03, &
      2.380730E-02, 2.01461E+00, 0.0000E+00, 3.0467E-03, &
      2.545001E-02, 1.10848E+00, 0.0000E+00, 3.9662E-03, &
      2.720606E-02, -9.05118E-02, 0.0000E+00, 5.1364E-03, &
      2.908327E-02, -1.75747E+00, 0.0000E+00, 6.6175E-03, &
      3.077200E-02, -3.84438E+00, 0.0000E+00, 8.1662E-03, &
      3.109002E-02, -4.36403E+00, 0.0000E+00, 8.4818E-03, &
      3.124300E-02, -4.65656E+00, 0.0000E+00, 8.6363E-03, &
      3.136860E-02, -4.98693E+00, 0.0000E+00, 8.7645E-03, &
      3.155700E-02, -5.20918E+00, 2.5761E-01, 2.8215E+03, &
      3.202800E-02, -5.96971E+00, 3.7031E-01, 3.9962E+03, &
      3.323523E-02, -8.47735E+00, 9.1655E-01, 9.5316E+03, &
      3.552846E-02, -1.49079E+01, 4.4790E+00, 4.3573E+04, &
      3.797993E-02, -2.07779E+01, 1.4207E+01, 1.2929E+05, &
      4.060054E-02, -1.99494E+01, 2.9219E+01, 2.4873E+05, &
      4.340198E-02, -9.21232E+00, 4.2628E+01, 3.3946E+05, &
      4.639671E-02, 7.33816E+00, 4.7650E+01, 3.5496E+05, &
      4.959809E-02, 2.25266E+01, 4.3565E+01, 3.0359E+05, &
      5.302035E-02, 3.20942E+01, 3.4420E+01, 2.2438E+05, &
      5.667876E-02, 3.59127E+01, 2.4603E+01, 1.5003E+05, &
      6.058959E-02, 3.58926E+01, 1.6521E+01, 9.4243E+04, &
      6.477028E-02, 3.40081E+01, 1.0746E+01, 5.7344E+04, &
      6.923942E-02, 3.15490E+01, 6.9378E+00, 3.4632E+04, &
      7.401695E-02, 2.91518E+01, 4.5311E+00, 2.1158E+04, &
      7.912411E-02, 2.70458E+01, 3.0369E+00, 1.3266E+04, &
      8.458368E-02, 2.52544E+01, 2.1106E+00, 8.6244E+03, &
      9.041995E-02, 2.36943E+01, 1.5317E+00, 5.8550E+03, &
      9.643200E-02, 2.20731E+01, 1.1759E+00, 4.2150E+03, &
      9.665893E-02, 2.19889E+01, 1.1656E+00, 4.1683E+03, &
      9.790800E-02, 2.12835E+01, 1.1122E+00, 3.9266E+03, &
      9.830160E-02, 2.05485E+01, 1.0964E+00, 3.8554E+03, &
      9.889200E-02, 2.11727E+01, 2.4069E+00, 8.4125E+03, &
      1.003680E-01, 2.15874E+01, 2.3200E+00, 7.9894E+03, &
      1.033284E-01, 2.16265E+01, 2.1662E+00, 7.2462E+03, &
      1.104581E-01, 2.12341E+01, 1.8840E+00, 5.8956E+03, &
      1.180797E-01, 2.07308E+01, 1.7036E+00, 4.9870E+03, &
      1.262272E-01, 2.02370E+01, 1.6018E+00, 4.3865E+03, &
      1.349368E-01, 1.97643E+01, 1.5574E+00, 3.9897E+03, &
      1.442475E-01, 1.92875E+01, 1.5578E+00, 3.7334E+03, &
      1.489600E-01, 1.90102E+01, 1.5713E+00, 3.6465E+03, &
      1.512400E-01, 1.87825E+01, 1.5803E+00, 3.6121E+03, &
      1.518480E-01, 1.85939E+01, 1.5829E+00, 3.6037E+03, &
      1.527600E-01, 1.87254E+01, 1.9125E+00, 4.3278E+03, &
      1.542005E-01, 1.87836E+01, 1.9230E+00, 4.3110E+03, &
      1.550400E-01, 1.87879E+01, 1.9293E+00, 4.3019E+03, &
      1.648404E-01, 1.86293E+01, 2.0151E+00, 4.2260E+03, &
      1.762144E-01, 1.83933E+01, 2.1333E+00, 4.1853E+03, &
      1.883732E-01, 1.81581E+01, 2.2710E+00, 4.1680E+03, &
      2.013709E-01, 1.79281E+01, 2.4256E+00, 4.1646E+03, &
      2.152655E-01, 1.77023E+01, 2.5975E+00, 4.1720E+03, &
      2.301188E-01, 1.74794E+01, 2.7840E+00, 4.1831E+03, &
      2.459970E-01, 1.72549E+01, 2.9781E+00, 4.1860E+03, &
      2.629708E-01, 1.70152E+01, 3.1730E+00, 4.1722E+03, &
      2.811158E-01, 1.67428E+01, 3.3666E+00, 4.1413E+03, &
      3.005128E-01, 1.64168E+01, 3.5553E+00, 4.0914E+03, &
      3.212482E-01, 1.60178E+01, 3.7348E+00, 4.0209E+03, &
      3.434143E-01, 1.55023E+01, 3.9011E+00, 3.9290E+03, &
      3.671099E-01, 1.48094E+01, 4.0507E+00, 3.8167E+03, &
      3.924405E-01, 1.38406E+01, 4.1813E+00, 3.6859E+03, &
      4.195189E-01, 1.24195E+01, 4.2907E+00, 3.5385E+03, &
      4.484657E-01, 1.01810E+01, 4.3779E+00, 3.3778E+03, &
      4.794098E-01, 6.15466E+00, 4.4440E+00, 3.2079E+03, &
      5.124891E-01, -4.39626E+00, 4.4816E+00, 3.0267E+03, &
      5.169500E-01, -7.65911E+00, 4.4842E+00, 3.0024E+03, &
      5.248625E-01, -1.99185E+01, 4.4879E+00, 2.9597E+03, &
      5.261620E-01, -2.55442E+01, 4.4884E+00, 2.9528E+03, &
      5.269725E-01, -3.30107E+01, 4.4887E+00, 2.9484E+03, &
      5.301375E-01, -2.18369E+01, 2.8634E+01, 1.8673E+04, &
      5.342155E-01, -1.86600E+01, 2.8262E+01, 1.8289E+04, &
      5.363631E-01, -2.43979E+01, 2.8069E+01, 1.8091E+04, &
      5.380500E-01, -1.88391E+01, 4.3941E+01, 2.8230E+04, &
      5.395845E-01, -1.32225E+01, 4.3714E+01, 2.8005E+04, &
      5.478508E-01, -1.09635E+00, 4.2526E+01, 2.6834E+04, &
      5.856525E-01, 1.63151E+01, 3.7707E+01, 2.2258E+04, &
      6.260625E-01, 2.32994E+01, 3.3734E+01, 1.8629E+04, &
      6.692609E-01, 2.72541E+01, 3.0618E+01, 1.5817E+04, &
      7.154399E-01, 2.95461E+01, 2.8003E+01, 1.3534E+04, &
      7.502880E-01, 2.98255E+01, 2.6331E+01, 1.2135E+04, &
      7.617720E-01, 2.87013E+01, 2.5821E+01, 1.1721E+04, &
      7.648052E-01, 2.70528E+01, 2.5689E+01, 1.1615E+04, &
      7.648344E-01, 2.70121E+01, 2.5688E+01, 1.1614E+04, &
      7.694280E-01, 2.89494E+01, 2.8986E+01, 1.3027E+04, &
      7.809120E-01, 3.08036E+01, 2.8502E+01, 1.2621E+04, &
      7.956620E-01, 3.17950E+01, 2.7902E+01, 1.2126E+04, &
      8.078405E-01, 3.19318E+01, 2.7424E+01, 1.1739E+04, &
      8.110881E-01, 3.13556E+01, 2.7299E+01, 1.1639E+04, &
      8.159595E-01, 3.23883E+01, 2.8607E+01, 1.2124E+04, &
      8.175768E-01, 3.26333E+01, 2.8546E+01, 1.2074E+04, &
      8.281380E-01, 3.36578E+01, 2.8156E+01, 1.1757E+04, &
      8.739896E-01, 3.60758E+01, 2.6568E+01, 1.0513E+04, &
      9.248260E-01, 3.75487E+01, 2.4989E+01, 9.3452E+03, &
      9.342948E-01, 3.76111E+01, 2.4714E+01, 9.1489E+03, &
      9.389815E-01, 3.75172E+01, 2.4580E+01, 9.0539E+03, &
      9.427563E-01, 3.70730E+01, 2.4472E+01, 8.9784E+03, &
      9.484185E-01, 3.78019E+01, 2.5405E+01, 9.2648E+03, &
      9.625740E-01, 3.86901E+01, 2.5015E+01, 8.9886E+03, &
      9.987612E-01, 4.00143E+01, 2.4069E+01, 8.3360E+03, &
      1.067676E+00, 4.18815E+01, 2.2119E+01, 7.1673E+03, &
      1.141345E+00, 4.31430E+01, 2.0307E+01, 6.1563E+03, &
      1.220098E+00, 4.40900E+01, 1.8640E+01, 5.2873E+03, &
      1.304285E+00, 4.48358E+01, 1.7107E+01, 4.5404E+03, &
      1.394281E+00, 4.54081E+01, 1.5648E+01, 3.8862E+03, &
      1.490486E+00, 4.58155E+01, 1.4293E+01, 3.3216E+03, &
      1.593329E+00, 4.61106E+01, 1.3058E+01, 2.8398E+03, &
      1.703269E+00, 4.63046E+01, 1.1926E+01, 2.4273E+03, &
      1.820795E+00, 4.64133E+01, 1.0897E+01, 2.0757E+03, &
      1.946430E+00, 4.64607E+01, 9.9536E+00, 1.7748E+03, &
      2.080733E+00, 4.64584E+01, 9.0959E+00, 1.5181E+03, &
      2.224304E+00, 4.64054E+01, 8.2687E+00, 1.2921E+03, &
      2.377781E+00, 4.62816E+01, 7.5066E+00, 1.0982E+03, &
      2.541848E+00, 4.60945E+01, 6.8253E+00, 9.3502E+02, &
      2.717235E+00, 4.58465E+01, 6.2155E+00, 7.9743E+02, &
      2.904724E+00, 4.55381E+01, 5.6689E+00, 6.8122E+02, &
      3.105150E+00, 4.51717E+01, 5.1180E+00, 5.7620E+02, &
      3.319406E+00, 4.46190E+01, 4.5597E+00, 4.8111E+02, &
      3.548445E+00, 4.37679E+01, 4.0612E+00, 4.0173E+02, &
      3.793288E+00, 4.23394E+01, 3.6230E+00, 3.3607E+02, &
      4.049556E+00, 3.86985E+01, 3.2402E+00, 2.8231E+02, &
      4.055024E+00, 3.85259E+01, 3.2328E+00, 2.8129E+02, &
      4.111539E+00, 3.52495E+01, 3.1579E+00, 2.7117E+02, &
      4.128068E+00, 3.12844E+01, 3.1365E+00, 2.6831E+02, &
      4.152861E+00, 3.51768E+01, 1.0786E+01, 9.0334E+02, &
      4.214844E+00, 3.84252E+01, 1.0425E+01, 8.6050E+02, &
      4.292792E+00, 3.94207E+01, 9.9960E+00, 8.1039E+02, &
      4.334821E+00, 3.92292E+01, 9.7756E+00, 7.8498E+02, &
      4.358498E+00, 3.86196E+01, 9.6715E+00, 7.7247E+02, &
      4.376020E+00, 3.68523E+01, 9.5967E+00, 7.6348E+02, &
      4.402302E+00, 3.90524E+01, 1.3294E+01, 1.0492E+03, &
      4.468008E+00, 4.12611E+01, 1.2901E+01, 1.0034E+03, &
      4.604334E+00, 4.28218E+01, 1.2162E+01, 9.1830E+02, &
      4.633924E+00, 4.29076E+01, 1.2022E+01, 9.0204E+02, &
      4.674808E+00, 4.27008E+01, 1.1837E+01, 8.8043E+02, &
      4.693602E+00, 4.19213E+01, 1.1754E+01, 8.7080E+02, &
      4.721791E+00, 4.30690E+01, 1.3468E+01, 9.9108E+02, &
      4.792266E+00, 4.43895E+01, 1.3158E+01, 9.5417E+02, &
      4.953664E+00, 4.58807E+01, 1.2511E+01, 8.7803E+02, &
      5.295467E+00, 4.76632E+01, 1.1356E+01, 7.4607E+02, &
      5.660855E+00, 4.88405E+01, 1.0281E+01, 6.3236E+02, &
      6.051453E+00, 4.96541E+01, 9.2620E+00, 5.3343E+02, &
      6.469004E+00, 5.02187E+01, 8.3229E+00, 4.4888E+02, &
      6.915365E+00, 5.06070E+01, 7.4727E+00, 3.7747E+02, &
      7.392525E+00, 5.08727E+01, 6.7098E+00, 3.1747E+02, &
      7.902609E+00, 5.10531E+01, 6.0285E+00, 2.6721E+02, &
      8.447890E+00, 5.12517E+01, 5.4184E+00, 2.2502E+02, &
      9.030794E+00, 5.13849E+01, 4.8549E+00, 1.8894E+02, &
      9.653919E+00, 5.14451E+01, 4.3109E+00, 1.5728E+02, &
      1.032004E+01, 5.14241E+01, 3.8246E+00, 1.3084E+02, &
      1.103212E+01, 5.13726E+01, 3.3957E+00, 1.0896E+02, &
      1.179334E+01, 5.13015E+01, 3.0177E+00, 9.0831E+01, &
      1.260708E+01, 5.12178E+01, 2.6842E+00, 7.5814E+01, &
      1.347697E+01, 5.11259E+01, 2.3898E+00, 6.3355E+01, &
      1.440688E+01, 5.10287E+01, 2.1296E+00, 5.3006E+01, &
      1.540095E+01, 5.09275E+01, 1.8978E+00, 4.4366E+01, &
      1.646362E+01, 5.08228E+01, 1.6923E+00, 3.7170E+01, &
      1.759961E+01, 5.07150E+01, 1.5106E+00, 3.1182E+01, &
      1.881398E+01, 5.06025E+01, 1.3455E+00, 2.6117E+01, &
      2.011215E+01, 5.04806E+01, 1.1969E+00, 2.1858E+01, &
      2.149988E+01, 5.03444E+01, 1.0614E+00, 1.8249E+01, &
      2.298338E+01, 5.01845E+01, 9.4120E-01, 1.5246E+01, &
      2.456923E+01, 4.99872E+01, 8.3540E-01, 1.2756E+01, &
      2.626450E+01, 4.97185E+01, 7.4215E-01, 1.0690E+01, &
      2.807676E+01, 4.92783E+01, 6.5316E-01, 8.8885E+00, &
      2.988138E+01, 4.81193E+01, 5.7836E-01, 7.4730E+00, &
      3.001405E+01, 4.79024E+01, 5.7340E-01, 7.3819E+00, &
      3.033874E+01, 4.68668E+01, 5.6152E-01, 7.1653E+00, &
      3.046071E+01, 4.53623E+01, 5.5716E-01, 7.0862E+00, &
      3.064366E+01, 4.68879E+01, 3.5145E+00, 4.0399E+01, &
      3.110102E+01, 4.82676E+01, 3.4275E+00, 3.8835E+01, &
      3.208502E+01, 4.92519E+01, 3.2520E+00, 3.5746E+01, &
      3.429889E+01, 5.01312E+01, 2.9110E+00, 2.9991E+01, &
      3.666551E+01, 5.05777E+01, 2.6076E+00, 2.5185E+01, &
      3.919543E+01, 5.08571E+01, 2.3365E+00, 2.1159E+01, &
      4.189992E+01, 5.10451E+01, 2.0865E+00, 1.7723E+01, &
      4.479101E+01, 5.11700E+01, 1.8611E+00, 1.4831E+01, &
      4.788159E+01, 5.12536E+01, 1.6592E+00, 1.2411E+01, &
      5.118542E+01, 5.13083E+01, 1.4786E+00, 1.0384E+01, &
      5.471721E+01, 5.13425E+01, 1.3173E+00, 8.6913E+00, &
      5.849270E+01, 5.13619E+01, 1.1735E+00, 7.2774E+00, &
      6.252870E+01, 5.14029E+01, 1.0438E+00, 6.0881E+00, &
      6.684318E+01, 5.14014E+01, 9.2605E-01, 5.0843E+00, &
      7.145536E+01, 5.13932E+01, 8.2164E-01, 4.2500E+00, &
      7.638578E+01, 5.13809E+01, 7.2906E-01, 3.5564E+00, &
      8.165640E+01, 5.13667E+01, 6.4593E-01, 2.9750E+00, &
      8.729069E+01, 5.13491E+01, 5.7023E-01, 2.4840E+00, &
      9.331374E+01, 5.13295E+01, 5.0344E-01, 2.0774E+00, &
      9.975239E+01, 5.13089E+01, 4.4450E-01, 1.7406E+00, &
      1.066353E+02, 5.12882E+01, 3.9249E-01, 1.4617E+00, &
      1.139931E+02, 5.12677E+01, 3.4659E-01, 1.2304E+00, &
      1.218587E+02, 5.12478E+01, 3.0608E-01, 1.0386E+00, &
      1.302669E+02, 5.12287E+01, 2.7032E-01, 8.7943E-01, &
      1.392553E+02, 5.12105E+01, 2.3871E-01, 7.4716E-01, &
      1.488640E+02, 5.11933E+01, 2.1081E-01, 6.3723E-01, &
      1.591356E+02, 5.11771E+01, 1.8617E-01, 5.4580E-01, &
      1.701159E+02, 5.11620E+01, 1.6443E-01, 4.6967E-01, &
      1.818539E+02, 5.11479E+01, 1.4523E-01, 4.0622E-01, &
      1.944018E+02, 5.11348E+01, 1.2828E-01, 3.5325E-01, &
      2.078156E+02, 5.11228E+01, 1.1331E-01, 3.0896E-01, &
      2.221548E+02, 5.11116E+01, 1.0010E-01, 2.7185E-01, &
      2.374835E+02, 5.11014E+01, 8.8426E-02, 2.4068E-01, &
      2.538699E+02, 5.10920E+01, 7.8119E-02, 2.1445E-01, &
      2.713869E+02, 5.10834E+01, 6.9017E-02, 1.9231E-01, &
      2.901126E+02, 5.10755E+01, 6.0977E-02, 1.7354E-01, &
      3.101304E+02, 5.10683E+01, 5.3876E-02, 1.5757E-01, &
      3.315294E+02, 5.10617E+01, 4.7604E-02, 1.4394E-01, &
      3.544049E+02, 5.10558E+01, 4.2063E-02, 1.3223E-01, &
      3.788588E+02, 5.10503E+01, 3.7169E-02, 1.2213E-01, &
      4.050001E+02, 5.10454E+01, 3.2845E-02, 1.1336E-01, &
      4.329451E+02, 5.10409E+01, 2.9025E-02, 1.0570E-01]
   case(52)
      frel = -3.6120E-01
      allocate(nist_tab(872))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 5.02171E+00, 0.0000E+00, 9.1175E-05, &
      1.142761E-02, 4.95273E+00, 0.0000E+00, 1.2637E-04, &
      1.221612E-02, 4.87306E+00, 0.0000E+00, 1.7418E-04, &
      1.305903E-02, 4.78087E+00, 0.0000E+00, 2.3875E-04, &
      1.396010E-02, 4.67402E+00, 0.0000E+00, 3.2546E-04, &
      1.492335E-02, 4.54987E+00, 0.0000E+00, 4.4124E-04, &
      1.595306E-02, 4.40528E+00, 0.0000E+00, 5.9495E-04, &
      1.705382E-02, 4.23635E+00, 0.0000E+00, 7.9789E-04, &
      1.823053E-02, 4.03830E+00, 0.0000E+00, 1.0643E-03, &
      1.948844E-02, 3.80510E+00, 0.0000E+00, 1.4121E-03, &
      2.083314E-02, 3.52916E+00, 0.0000E+00, 1.8637E-03, &
      2.227063E-02, 3.20063E+00, 0.0000E+00, 2.4467E-03, &
      2.380730E-02, 2.80664E+00, 0.0000E+00, 3.1953E-03, &
      2.545001E-02, 2.32987E+00, 0.0000E+00, 4.1513E-03, &
      2.720606E-02, 1.74643E+00, 0.0000E+00, 5.3655E-03, &
      2.908327E-02, 1.02221E+00, 0.0000E+00, 6.8992E-03, &
      3.109002E-02, 1.06352E-01, 0.0000E+00, 8.8262E-03, &
      3.323523E-02, -1.08165E+00, 0.0000E+00, 1.1234E-02, &
      3.552846E-02, -2.68102E+00, 0.0000E+00, 1.4227E-02, &
      3.797993E-02, -4.97638E+00, 0.0000E+00, 1.7928E-02, &
      3.900400E-02, -6.23915E+00, 0.0000E+00, 1.9632E-02, &
      3.960100E-02, -7.16454E+00, 0.0000E+00, 2.0670E-02, &
      3.976020E-02, -7.53159E+00, 0.0000E+00, 2.0952E-02, &
      3.999900E-02, -7.79861E+00, 2.6938E-01, 2.2210E+03, &
      4.060054E-02, -8.69306E+00, 3.9162E-01, 3.1810E+03, &
      4.340198E-02, -1.46624E+01, 2.0857E+00, 1.5848E+04, &
      4.639671E-02, -2.27141E+01, 9.0143E+00, 6.4072E+04, &
      4.959809E-02, -2.71819E+01, 2.3696E+01, 1.5756E+05, &
      5.302035E-02, -2.11494E+01, 4.1736E+01, 2.5959E+05, &
      5.667876E-02, -4.60076E+00, 5.3777E+01, 3.1290E+05, &
      6.058959E-02, 1.49668E+01, 5.4618E+01, 2.9728E+05, &
      6.477028E-02, 3.00555E+01, 4.6556E+01, 2.3704E+05, &
      6.923942E-02, 3.80387E+01, 3.5090E+01, 1.6713E+05, &
      7.401695E-02, 4.02145E+01, 2.4414E+01, 1.0878E+05, &
      7.912411E-02, 3.90346E+01, 1.6236E+01, 6.7671E+04, &
      8.458368E-02, 3.64569E+01, 1.0612E+01, 4.1375E+04, &
      9.041995E-02, 3.35946E+01, 6.9660E+00, 2.5407E+04, &
      9.665893E-02, 3.09241E+01, 4.6687E+00, 1.5929E+04, &
      1.033284E-01, 2.85341E+01, 3.2331E+00, 1.0319E+04, &
      1.079960E-01, 2.69218E+01, 2.5916E+00, 7.9143E+03, &
      1.096490E-01, 2.60467E+01, 2.4126E+00, 7.2566E+03, &
      1.100898E-01, 2.53687E+01, 2.3683E+00, 7.0950E+03, &
      1.104581E-01, 2.56020E+01, 3.4710E+00, 1.0363E+04, &
      1.107510E-01, 2.58166E+01, 3.4406E+00, 1.0245E+04, &
      1.124040E-01, 2.59837E+01, 3.2799E+00, 9.6235E+03, &
      1.180797E-01, 2.54097E+01, 2.8411E+00, 7.9351E+03, &
      1.262272E-01, 2.44131E+01, 2.4261E+00, 6.3389E+03, &
      1.349368E-01, 2.35329E+01, 2.1678E+00, 5.2986E+03, &
      1.442475E-01, 2.27549E+01, 2.0161E+00, 4.6100E+03, &
      1.542005E-01, 2.20521E+01, 1.9390E+00, 4.1477E+03, &
      1.648404E-01, 2.13284E+01, 1.9170E+00, 3.8360E+03, &
      1.649340E-01, 2.13207E+01, 1.9170E+00, 3.8339E+03, &
      1.674585E-01, 2.10437E+01, 1.9185E+00, 3.7790E+03, &
      1.681317E-01, 2.08308E+01, 1.9192E+00, 3.7654E+03, &
      1.691415E-01, 2.09635E+01, 2.2751E+00, 4.4368E+03, &
      1.716660E-01, 2.10059E+01, 2.2855E+00, 4.3915E+03, &
      1.762144E-01, 2.09123E+01, 2.3086E+00, 4.3214E+03, &
      1.883732E-01, 2.05780E+01, 2.3917E+00, 4.1883E+03, &
      2.013709E-01, 2.02470E+01, 2.5018E+00, 4.0984E+03, &
      2.152655E-01, 1.99365E+01, 2.6328E+00, 4.0348E+03, &
      2.301188E-01, 1.96437E+01, 2.7816E+00, 3.9879E+03, &
      2.459970E-01, 1.93624E+01, 2.9453E+00, 3.9502E+03, &
      2.629708E-01, 1.90847E+01, 3.1207E+00, 3.9155E+03, &
      2.811158E-01, 1.88008E+01, 3.3043E+00, 3.8785E+03, &
      3.005128E-01, 1.84978E+01, 3.4917E+00, 3.8341E+03, &
      3.212482E-01, 1.81581E+01, 3.6779E+00, 3.7782E+03, &
      3.434143E-01, 1.77479E+01, 3.8574E+00, 3.7071E+03, &
      3.671099E-01, 1.72479E+01, 4.0242E+00, 3.6180E+03, &
      3.924405E-01, 1.65932E+01, 4.1725E+00, 3.5096E+03, &
      4.195189E-01, 1.56938E+01, 4.2972E+00, 3.3816E+03, &
      4.484657E-01, 1.43924E+01, 4.3941E+00, 3.2350E+03, &
      4.794098E-01, 1.23763E+01, 4.4592E+00, 3.0714E+03, &
      5.124891E-01, 8.87098E+00, 4.4910E+00, 2.8943E+03, &
      5.478508E-01, 7.87758E-01, 4.4905E+00, 2.7076E+03, &
      5.606580E-01, -6.28018E+00, 4.4830E+00, 2.6415E+03, &
      5.692395E-01, -1.87866E+01, 4.4762E+00, 2.5979E+03, &
      5.708500E-01, -2.57911E+01, 4.4748E+00, 2.5898E+03, &
      5.715279E-01, -3.22115E+01, 4.4742E+00, 2.5864E+03, &
      5.749605E-01, -2.06063E+01, 2.9296E+01, 1.6809E+04, &
      5.795875E-01, -1.71582E+01, 2.8851E+01, 1.6421E+04, &
      5.819175E-01, -2.30368E+01, 2.8631E+01, 1.6231E+04, &
      5.835420E-01, -1.84736E+01, 4.4853E+01, 2.5353E+04, &
      5.856525E-01, -1.10295E+01, 4.4528E+01, 2.5079E+04, &
      5.941500E-01, 5.71545E-01, 4.3258E+01, 2.4016E+04, &
      6.260625E-01, 1.60555E+01, 3.8960E+01, 2.0527E+04, &
      6.692609E-01, 2.41327E+01, 3.4422E+01, 1.6966E+04, &
      7.154399E-01, 2.83771E+01, 3.1032E+01, 1.4310E+04, &
      7.648052E-01, 3.07857E+01, 2.8276E+01, 1.2199E+04, &
      8.023260E-01, 3.11006E+01, 2.6502E+01, 1.0899E+04, &
      8.146065E-01, 2.99606E+01, 2.5969E+01, 1.0519E+04, &
      8.175768E-01, 2.85887E+01, 2.5843E+01, 1.0430E+04, &
      8.178813E-01, 2.82393E+01, 2.5831E+01, 1.0421E+04, &
      8.227935E-01, 3.02193E+01, 2.9191E+01, 1.1706E+04, &
      8.350740E-01, 3.21210E+01, 2.8686E+01, 1.1334E+04, &
      8.523060E-01, 3.32176E+01, 2.8006E+01, 1.0842E+04, &
      8.653515E-01, 3.33477E+01, 2.7511E+01, 1.0490E+04, &
      8.688303E-01, 3.27617E+01, 2.7381E+01, 1.0399E+04, &
      8.739896E-01, 3.37989E+01, 2.8707E+01, 1.0838E+04, &
      8.740485E-01, 3.38085E+01, 2.8705E+01, 1.0837E+04, &
      8.870940E-01, 3.50964E+01, 2.8239E+01, 1.0504E+04, &
      9.342948E-01, 3.74949E+01, 2.6663E+01, 9.4177E+03, &
      9.858800E-01, 3.90244E+01, 2.5112E+01, 8.4068E+03, &
      9.987612E-01, 3.91176E+01, 2.4759E+01, 8.1818E+03, &
      1.000970E+00, 3.90626E+01, 2.4694E+01, 8.1424E+03, &
      1.004994E+00, 3.86076E+01, 2.4564E+01, 8.0670E+03, &
      1.011030E+00, 3.93620E+01, 2.5499E+01, 8.3240E+03, &
      1.026120E+00, 4.02800E+01, 2.5017E+01, 8.0467E+03, &
      1.067676E+00, 4.16539E+01, 2.3768E+01, 7.3482E+03, &
      1.141345E+00, 4.32237E+01, 2.1817E+01, 6.3106E+03, &
      1.220098E+00, 4.43693E+01, 2.0028E+01, 5.4205E+03, &
      1.304285E+00, 4.52519E+01, 1.8393E+01, 4.6576E+03, &
      1.394281E+00, 4.59578E+01, 1.6880E+01, 3.9995E+03, &
      1.490486E+00, 4.64985E+01, 1.5440E+01, 3.4233E+03, &
      1.593329E+00, 4.68807E+01, 1.4101E+01, 2.9258E+03, &
      1.703269E+00, 4.71403E+01, 1.2882E+01, 2.5013E+03, &
      1.820795E+00, 4.73202E+01, 1.1773E+01, 2.1395E+03, &
      1.946430E+00, 4.74163E+01, 1.0766E+01, 1.8312E+03, &
      2.080733E+00, 4.74762E+01, 9.8467E+00, 1.5678E+03, &
      2.224304E+00, 4.74681E+01, 8.9995E+00, 1.3414E+03, &
      2.377781E+00, 4.74141E+01, 8.1995E+00, 1.1442E+03, &
      2.541848E+00, 4.72925E+01, 7.4470E+00, 9.7306E+02, &
      2.717235E+00, 4.70977E+01, 6.7743E+00, 8.2892E+02, &
      2.904724E+00, 4.68476E+01, 6.1722E+00, 7.0736E+02, &
      3.105150E+00, 4.66153E+01, 5.5748E+00, 5.9852E+02, &
      3.319406E+00, 4.61977E+01, 4.9906E+00, 5.0210E+02, &
      3.548445E+00, 4.55660E+01, 4.4525E+00, 4.1990E+02, &
      3.793288E+00, 4.46074E+01, 3.9705E+00, 3.5109E+02, &
      4.055024E+00, 4.29161E+01, 3.5466E+00, 2.9414E+02, &
      4.254572E+00, 3.98866E+01, 3.2731E+00, 2.5925E+02, &
      4.319693E+00, 3.64828E+01, 3.1916E+00, 2.4916E+02, &
      4.334821E+00, 3.35784E+01, 3.1732E+00, 2.4690E+02, &
      4.337059E+00, 3.25661E+01, 3.1704E+00, 2.4657E+02, &
      4.363107E+00, 3.64143E+01, 1.0730E+01, 8.1647E+02, &
      4.428228E+00, 3.96387E+01, 1.0375E+01, 7.7805E+02, &
      4.519760E+00, 4.06986E+01, 9.9045E+00, 7.2804E+02, &
      4.588940E+00, 3.98588E+01, 9.5928E+00, 6.9469E+02, &
      4.607388E+00, 3.81010E+01, 9.5193E+00, 6.8666E+02, &
      4.633924E+00, 4.01981E+01, 1.3189E+01, 9.4389E+02, &
      4.635060E+00, 4.02695E+01, 1.3182E+01, 9.4319E+02, &
      4.704240E+00, 4.24355E+01, 1.2795E+01, 9.0219E+02, &
      4.840416E+00, 4.39010E+01, 1.2099E+01, 8.2944E+02, &
      4.914504E+00, 4.37836E+01, 1.1775E+01, 7.9525E+02, &
      4.934261E+00, 4.30070E+01, 1.1693E+01, 7.8656E+02, &
      4.953664E+00, 4.37620E+01, 1.3446E+01, 9.0020E+02, &
      4.963896E+00, 4.41527E+01, 1.3402E+01, 8.9541E+02, &
      5.037984E+00, 4.54697E+01, 1.3093E+01, 8.6204E+02, &
      5.295467E+00, 4.74921E+01, 1.2143E+01, 7.6105E+02, &
      5.660855E+00, 4.90633E+01, 1.1024E+01, 6.4685E+02, &
      6.051453E+00, 5.01223E+01, 9.9599E+00, 5.4717E+02, &
      6.469004E+00, 5.08541E+01, 8.9652E+00, 4.6121E+02, &
      6.915365E+00, 5.13603E+01, 8.0533E+00, 3.8800E+02, &
      7.392525E+00, 5.17086E+01, 7.2315E+00, 3.2633E+02, &
      7.902609E+00, 5.19470E+01, 6.4957E+00, 2.7459E+02, &
      8.447890E+00, 5.21110E+01, 5.8397E+00, 2.3128E+02, &
      9.030794E+00, 5.23131E+01, 5.2387E+00, 1.9442E+02, &
      9.653919E+00, 5.24118E+01, 4.6582E+00, 1.6205E+02, &
      1.032004E+01, 5.24525E+01, 4.1358E+00, 1.3489E+02, &
      1.103212E+01, 5.24176E+01, 3.6712E+00, 1.1229E+02, &
      1.179334E+01, 5.23573E+01, 3.2610E+00, 9.3564E+01, &
      1.260708E+01, 5.22804E+01, 2.8992E+00, 7.8049E+01, &
      1.347697E+01, 5.21929E+01, 2.5799E+00, 6.5184E+01, &
      1.440688E+01, 5.20983E+01, 2.2978E+00, 5.4505E+01, &
      1.540095E+01, 5.19993E+01, 2.0485E+00, 4.5629E+01, &
      1.646362E+01, 5.18967E+01, 1.8263E+00, 3.8216E+01, &
      1.759961E+01, 5.17907E+01, 1.6292E+00, 3.2037E+01, &
      1.881398E+01, 5.16814E+01, 1.4547E+00, 2.6891E+01, &
      2.011215E+01, 5.15662E+01, 1.2957E+00, 2.2529E+01, &
      2.149988E+01, 5.14406E+01, 1.1532E+00, 1.8871E+01, &
      2.298338E+01, 5.12980E+01, 1.0224E+00, 1.5758E+01, &
      2.456923E+01, 5.11280E+01, 9.0723E-01, 1.3177E+01, &
      2.626450E+01, 5.09118E+01, 8.0574E-01, 1.1036E+01, &
      2.807676E+01, 5.06060E+01, 7.0918E-01, 9.1741E+00, &
      3.001405E+01, 5.00202E+01, 6.2270E-01, 7.6174E+00, &
      3.117752E+01, 4.91381E+01, 5.7843E-01, 6.8570E+00, &
      3.165473E+01, 4.78928E+01, 5.6169E-01, 6.5761E+00, &
      3.178199E+01, 4.63982E+01, 5.5735E-01, 6.5040E+00, &
      3.197287E+01, 4.79119E+01, 3.4947E+00, 3.6761E+01, &
      3.208502E+01, 4.84330E+01, 3.4746E+00, 3.6426E+01, &
      3.245008E+01, 4.92809E+01, 3.4107E+00, 3.5364E+01, &
      3.429889E+01, 5.06702E+01, 3.1135E+00, 3.0591E+01, &
      3.666551E+01, 5.13278E+01, 2.7886E+00, 2.5683E+01, &
      3.919543E+01, 5.16980E+01, 2.4971E+00, 2.1562E+01, &
      4.189992E+01, 5.19380E+01, 2.2343E+00, 1.8094E+01, &
      4.479101E+01, 5.20975E+01, 1.9932E+00, 1.5143E+01, &
      4.788159E+01, 5.22041E+01, 1.7775E+00, 1.2673E+01, &
      5.118542E+01, 5.22749E+01, 1.5846E+00, 1.0607E+01, &
      5.471721E+01, 5.23206E+01, 1.4123E+00, 8.8798E+00, &
      5.849270E+01, 5.23484E+01, 1.2587E+00, 7.4364E+00, &
      6.252870E+01, 5.23636E+01, 1.1216E+00, 6.2311E+00, &
      6.684318E+01, 5.24005E+01, 9.9655E-01, 5.2097E+00, &
      7.145536E+01, 5.23964E+01, 8.8454E-01, 4.3551E+00, &
      7.638578E+01, 5.23867E+01, 7.8519E-01, 3.6443E+00, &
      8.165640E+01, 5.23744E+01, 6.9597E-01, 3.0487E+00, &
      8.729069E+01, 5.23584E+01, 6.1479E-01, 2.5457E+00, &
      9.331374E+01, 5.23395E+01, 5.4312E-01, 2.1290E+00, &
      9.975239E+01, 5.23193E+01, 4.7984E-01, 1.7837E+00, &
      1.066353E+02, 5.22986E+01, 4.2397E-01, 1.4974E+00, &
      1.139931E+02, 5.22780E+01, 3.7464E-01, 1.2600E+00, &
      1.218587E+02, 5.22578E+01, 3.3107E-01, 1.0630E+00, &
      1.302669E+02, 5.22384E+01, 2.9258E-01, 8.9949E-01, &
      1.392553E+02, 5.22197E+01, 2.5859E-01, 7.6363E-01, &
      1.488640E+02, 5.22020E+01, 2.2858E-01, 6.5071E-01, &
      1.591356E+02, 5.21854E+01, 2.0207E-01, 5.5673E-01, &
      1.701159E+02, 5.21697E+01, 1.7864E-01, 4.7844E-01, &
      1.818539E+02, 5.21552E+01, 1.5794E-01, 4.1316E-01, &
      1.944018E+02, 5.21416E+01, 1.3964E-01, 3.5865E-01, &
      2.078156E+02, 5.21291E+01, 1.2347E-01, 3.1305E-01, &
      2.221548E+02, 5.21175E+01, 1.0918E-01, 2.7485E-01, &
      2.374835E+02, 5.21068E+01, 9.6550E-02, 2.4276E-01, &
      2.538699E+02, 5.20970E+01, 8.5384E-02, 2.1577E-01, &
      2.713869E+02, 5.20880E+01, 7.5514E-02, 1.9298E-01, &
      2.901126E+02, 5.20797E+01, 6.6788E-02, 1.7370E-01, &
      3.101304E+02, 5.20722E+01, 5.9073E-02, 1.5731E-01, &
      3.315294E+02, 5.20653E+01, 5.2252E-02, 1.4334E-01, &
      3.544049E+02, 5.20590E+01, 4.6221E-02, 1.3136E-01, &
      3.788588E+02, 5.20533E+01, 4.0888E-02, 1.2105E-01, &
      4.050001E+02, 5.20481E+01, 3.6171E-02, 1.1212E-01, &
      4.329451E+02, 5.20434E+01, 3.2001E-02, 1.0435E-01]
   case(53)
      frel = -3.7860E-01
      allocate(nist_tab(860))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.18518E+01, 0.0000E+00, 1.0672E-04, &
      1.142761E-02, 1.18252E+01, 0.0000E+00, 1.4743E-04, &
      1.221612E-02, 1.17946E+01, 0.0000E+00, 2.0255E-04, &
      1.305903E-02, 1.17594E+01, 0.0000E+00, 2.7674E-04, &
      1.396010E-02, 1.17188E+01, 0.0000E+00, 3.7607E-04, &
      1.492335E-02, 1.16719E+01, 0.0000E+00, 5.0827E-04, &
      1.595306E-02, 1.16177E+01, 0.0000E+00, 6.8327E-04, &
      1.705382E-02, 1.15549E+01, 0.0000E+00, 9.1362E-04, &
      1.823053E-02, 1.14821E+01, 0.0000E+00, 1.2151E-03, &
      1.948844E-02, 1.13973E+01, 0.0000E+00, 1.6077E-03, &
      2.083314E-02, 1.12983E+01, 0.0000E+00, 2.1158E-03, &
      2.227063E-02, 1.11823E+01, 0.0000E+00, 2.7701E-03, &
      2.380730E-02, 1.10460E+01, 0.0000E+00, 3.6080E-03, &
      2.545001E-02, 1.08849E+01, 0.0000E+00, 4.6752E-03, &
      2.720606E-02, 1.06935E+01, 0.0000E+00, 6.0271E-03, &
      2.908327E-02, 1.04643E+01, 0.0000E+00, 7.7306E-03, &
      3.109002E-02, 1.01874E+01, 0.0000E+00, 9.8654E-03, &
      3.323523E-02, 9.84922E+00, 0.0000E+00, 1.2527E-02, &
      3.552846E-02, 9.43029E+00, 0.0000E+00, 1.5827E-02, &
      3.797993E-02, 8.90135E+00, 0.0000E+00, 1.9898E-02, &
      4.060054E-02, 8.21570E+00, 0.0000E+00, 2.4893E-02, &
      4.340198E-02, 7.29092E+00, 0.0000E+00, 3.0991E-02, &
      4.639671E-02, 5.95148E+00, 0.0000E+00, 3.8395E-02, &
      4.860800E-02, 4.48414E+00, 0.0000E+00, 4.4459E-02, &
      4.935200E-02, 3.67328E+00, 0.0000E+00, 4.6614E-02, &
      4.959809E-02, 2.55839E+00, 0.0000E+00, 4.7340E-02, &
      4.984800E-02, 3.20612E+00, 6.2754E-01, 4.1744E+03, &
      5.059200E-02, 2.78632E+00, 8.1817E-01, 5.3626E+03, &
      5.302035E-02, 6.05141E-01, 1.8942E+00, 1.1846E+04, &
      5.667876E-02, -2.97990E+00, 5.8610E+00, 3.4289E+04, &
      6.058959E-02, -4.41342E+00, 1.2865E+01, 7.0404E+04, &
      6.477028E-02, -1.57643E+00, 2.0756E+01, 1.0626E+05, &
      6.923942E-02, 5.22924E+00, 2.6340E+01, 1.2614E+05, &
      7.401695E-02, 1.35284E+01, 2.7841E+01, 1.2473E+05, &
      7.912411E-02, 2.07455E+01, 2.5717E+01, 1.0777E+05, &
      8.458368E-02, 2.56007E+01, 2.1603E+01, 8.4689E+04, &
      9.041995E-02, 2.80894E+01, 1.7050E+01, 6.2528E+04, &
      9.665893E-02, 2.88251E+01, 1.2982E+01, 4.4535E+04, &
      1.033284E-01, 2.84904E+01, 9.7369E+00, 3.1247E+04, &
      1.104581E-01, 2.75842E+01, 7.3115E+00, 2.1949E+04, &
      1.180797E-01, 2.63152E+01, 5.5642E+00, 1.5625E+04, &
      1.202460E-01, 2.58390E+01, 5.1835E+00, 1.4294E+04, &
      1.220865E-01, 2.51550E+01, 4.8922E+00, 1.3288E+04, &
      1.225773E-01, 2.45579E+01, 4.8191E+00, 1.3037E+04, &
      1.233135E-01, 2.50124E+01, 5.7654E+00, 1.5504E+04, &
      1.251540E-01, 2.52665E+01, 5.5146E+00, 1.4612E+04, &
      1.262272E-01, 2.52677E+01, 5.3788E+00, 1.4131E+04, &
      1.349368E-01, 2.47795E+01, 4.5057E+00, 1.1073E+04, &
      1.442475E-01, 2.41393E+01, 3.9032E+00, 8.9733E+03, &
      1.542005E-01, 2.35207E+01, 3.5000E+00, 7.5272E+03, &
      1.648404E-01, 2.29449E+01, 3.2395E+00, 6.5175E+03, &
      1.762144E-01, 2.23849E+01, 3.0832E+00, 5.8029E+03, &
      1.826720E-01, 2.20302E+01, 3.0324E+00, 5.5056E+03, &
      1.854680E-01, 2.17663E+01, 3.0170E+00, 5.3951E+03, &
      1.862136E-01, 2.15453E+01, 3.0134E+00, 5.3672E+03, &
      1.873320E-01, 2.17026E+01, 3.3918E+00, 6.0049E+03, &
      1.883732E-01, 2.17597E+01, 3.3898E+00, 5.9682E+03, &
      1.901280E-01, 2.17817E+01, 3.3874E+00, 5.9090E+03, &
      2.013709E-01, 2.16355E+01, 3.3965E+00, 5.5943E+03, &
      2.152655E-01, 2.13975E+01, 3.4508E+00, 5.3170E+03, &
      2.301188E-01, 2.11701E+01, 3.5381E+00, 5.0999E+03, &
      2.459970E-01, 2.09590E+01, 3.6501E+00, 4.9220E+03, &
      2.629708E-01, 2.07604E+01, 3.7800E+00, 4.7683E+03, &
      2.811158E-01, 2.05671E+01, 3.9218E+00, 4.6283E+03, &
      3.005128E-01, 2.03694E+01, 4.0698E+00, 4.4932E+03, &
      3.212482E-01, 2.01545E+01, 4.2182E+00, 4.3567E+03, &
      3.434143E-01, 1.99050E+01, 4.3612E+00, 4.2140E+03, &
      3.671099E-01, 1.95966E+01, 4.4930E+00, 4.0615E+03, &
      3.924405E-01, 1.91892E+01, 4.6080E+00, 3.8969E+03, &
      4.195189E-01, 1.86426E+01, 4.7012E+00, 3.7196E+03, &
      4.484657E-01, 1.78709E+01, 4.7685E+00, 3.5298E+03, &
      4.794098E-01, 1.67376E+01, 4.8069E+00, 3.3289E+03, &
      5.124891E-01, 1.49816E+01, 4.8143E+00, 3.1194E+03, &
      5.478508E-01, 1.19899E+01, 4.7894E+00, 2.9036E+03, &
      5.856525E-01, 5.63492E+00, 4.7339E+00, 2.6853E+03, &
      6.070120E-01, -3.38493E+00, 4.6924E+00, 2.5685E+03, &
      6.163030E-01, -1.53534E+01, 4.6724E+00, 2.5191E+03, &
      6.186740E-01, -2.69803E+01, 4.6672E+00, 2.5067E+03, &
      6.187806E-01, -2.82232E+01, 4.6670E+00, 2.5061E+03, &
      6.224970E-01, -1.69818E+01, 2.8524E+01, 1.5199E+04, &
      6.260625E-01, -1.32224E+01, 2.8226E+01, 1.4955E+04, &
      6.281435E-01, -1.34083E+01, 2.8054E+01, 1.4814E+04, &
      6.306687E-01, -1.92036E+01, 2.7848E+01, 1.4647E+04, &
      6.317880E-01, -1.95875E+01, 4.3524E+01, 2.2848E+04, &
      6.344565E-01, -8.36804E+00, 4.3172E+01, 2.2568E+04, &
      6.439260E-01, 3.18632E+00, 4.1956E+01, 2.1610E+04, &
      6.692609E-01, 1.53340E+01, 3.8956E+01, 1.9307E+04, &
      7.154399E-01, 2.45449E+01, 3.4443E+01, 1.5970E+04, &
      7.648052E-01, 2.91643E+01, 3.0997E+01, 1.3445E+04, &
      8.175768E-01, 3.17254E+01, 2.8202E+01, 1.1444E+04, &
      8.571080E-01, 3.20836E+01, 2.6462E+01, 1.0243E+04, &
      8.702270E-01, 3.09430E+01, 2.5931E+01, 9.8874E+03, &
      8.739896E-01, 2.87963E+01, 2.5783E+01, 9.7886E+03, &
      8.789730E-01, 3.12151E+01, 2.9204E+01, 1.1023E+04, &
      8.920920E-01, 3.31591E+01, 2.8702E+01, 1.0676E+04, &
      9.118900E-01, 3.43584E+01, 2.7977E+01, 1.0180E+04, &
      9.258475E-01, 3.44902E+01, 2.7486E+01, 9.8506E+03, &
      9.295695E-01, 3.39013E+01, 2.7358E+01, 9.7656E+03, &
      9.342948E-01, 3.48163E+01, 2.8719E+01, 1.0200E+04, &
      9.351525E-01, 3.49563E+01, 2.8690E+01, 1.0180E+04, &
      9.491100E-01, 3.62578E+01, 2.8218E+01, 9.8652E+03, &
      9.987612E-01, 3.86538E+01, 2.6646E+01, 8.8533E+03, &
      1.050658E+00, 4.01104E+01, 2.5110E+01, 7.9319E+03, &
      1.067676E+00, 4.00667E+01, 2.4603E+01, 7.6480E+03, &
      1.071028E+00, 3.96513E+01, 2.4500E+01, 7.5922E+03, &
      1.077461E+00, 4.04143E+01, 2.5442E+01, 7.8368E+03, &
      1.093542E+00, 4.13447E+01, 2.4955E+01, 7.5740E+03, &
      1.141345E+00, 4.28266E+01, 2.3600E+01, 6.8636E+03, &
      1.220098E+00, 4.43871E+01, 2.1641E+01, 5.8887E+03, &
      1.304285E+00, 4.55247E+01, 1.9848E+01, 5.0534E+03, &
      1.394281E+00, 4.63982E+01, 1.8211E+01, 4.3385E+03, &
      1.490486E+00, 4.70925E+01, 1.6699E+01, 3.7224E+03, &
      1.593329E+00, 4.76210E+01, 1.5264E+01, 3.1842E+03, &
      1.703269E+00, 4.79930E+01, 1.3933E+01, 2.7202E+03, &
      1.820795E+00, 4.82438E+01, 1.2723E+01, 2.3246E+03, &
      1.946430E+00, 4.84003E+01, 1.1623E+01, 1.9876E+03, &
      2.080733E+00, 4.84890E+01, 1.0626E+01, 1.7008E+03, &
      2.224304E+00, 4.85412E+01, 9.7173E+00, 1.4560E+03, &
      2.377781E+00, 4.85293E+01, 8.8857E+00, 1.2464E+03, &
      2.541848E+00, 4.84780E+01, 8.1102E+00, 1.0652E+03, &
      2.717235E+00, 4.83604E+01, 7.3706E+00, 9.0650E+02, &
      2.904724E+00, 4.82078E+01, 6.7093E+00, 7.7280E+02, &
      3.105150E+00, 4.80650E+01, 6.0553E+00, 6.5335E+02, &
      3.319406E+00, 4.76838E+01, 5.4240E+00, 5.4838E+02, &
      3.548445E+00, 4.71716E+01, 4.8629E+00, 4.6079E+02, &
      3.793288E+00, 4.64591E+01, 4.3446E+00, 3.8595E+02, &
      4.055024E+00, 4.53714E+01, 3.8793E+00, 3.2317E+02, &
      4.334821E+00, 4.32826E+01, 3.4694E+00, 2.7112E+02, &
      4.465958E+00, 4.10615E+01, 3.3023E+00, 2.5082E+02, &
      4.534314E+00, 3.77027E+01, 3.2206E+00, 2.4110E+02, &
      4.552543E+00, 3.38351E+01, 3.1994E+00, 2.3860E+02, &
      4.579885E+00, 3.76379E+01, 1.0667E+01, 7.7787E+02, &
      4.633924E+00, 4.04735E+01, 1.0391E+01, 7.4909E+02, &
      4.648242E+00, 4.08360E+01, 1.0320E+01, 7.4171E+02, &
      4.755058E+00, 4.19574E+01, 9.8092E+00, 6.8946E+02, &
      4.827839E+00, 4.10845E+01, 9.5109E+00, 6.5861E+02, &
      4.847248E+00, 3.93389E+01, 9.4389E+00, 6.5105E+02, &
      4.876360E+00, 4.14727E+01, 1.3062E+01, 8.9354E+02, &
      4.953664E+00, 4.36693E+01, 1.2660E+01, 8.5270E+02, &
      5.084338E+00, 4.49619E+01, 1.2036E+01, 7.9013E+02, &
      5.162159E+00, 4.48502E+01, 1.1713E+01, 7.5753E+02, &
      5.182912E+00, 4.40786E+01, 1.1631E+01, 7.4926E+02, &
      5.214040E+00, 4.52183E+01, 1.3332E+01, 8.5293E+02, &
      5.295467E+00, 4.65711E+01, 1.3013E+01, 8.1990E+02, &
      5.660855E+00, 4.90314E+01, 1.1779E+01, 6.9478E+02, &
      6.051453E+00, 5.04303E+01, 1.0679E+01, 5.8972E+02, &
      6.469004E+00, 5.13802E+01, 9.6346E+00, 4.9820E+02, &
      6.915365E+00, 5.20355E+01, 8.6656E+00, 4.1964E+02, &
      7.392525E+00, 5.24876E+01, 7.7844E+00, 3.5307E+02, &
      7.902609E+00, 5.27994E+01, 6.9929E+00, 2.9710E+02, &
      8.447890E+00, 5.30150E+01, 6.2859E+00, 2.5019E+02, &
      9.030794E+00, 5.31826E+01, 5.6458E+00, 2.1055E+02, &
      9.653919E+00, 5.33587E+01, 5.0279E+00, 1.7575E+02, &
      1.032004E+01, 5.34298E+01, 4.4712E+00, 1.4651E+02, &
      1.103212E+01, 5.34524E+01, 3.9684E+00, 1.2195E+02, &
      1.179334E+01, 5.34062E+01, 3.5241E+00, 1.0157E+02, &
      1.260708E+01, 5.33387E+01, 3.1314E+00, 8.4677E+01, &
      1.347697E+01, 5.32571E+01, 2.7850E+00, 7.0674E+01, &
      1.440688E+01, 5.31664E+01, 2.4792E+00, 5.9057E+01, &
      1.540095E+01, 5.30697E+01, 2.2090E+00, 4.9408E+01, &
      1.646362E+01, 5.29691E+01, 1.9699E+00, 4.1386E+01, &
      1.759961E+01, 5.28653E+01, 1.7569E+00, 3.4682E+01, &
      1.881398E+01, 5.27582E+01, 1.5678E+00, 2.9090E+01, &
      2.011215E+01, 5.26473E+01, 1.3991E+00, 2.4411E+01, &
      2.149988E+01, 5.25289E+01, 1.2463E+00, 2.0460E+01, &
      2.298338E+01, 5.23987E+01, 1.1086E+00, 1.7134E+01, &
      2.456923E+01, 5.22486E+01, 9.8355E-01, 1.4322E+01, &
      2.626450E+01, 5.20687E+01, 8.7336E-01, 1.1989E+01, &
      2.807676E+01, 5.18330E+01, 7.6881E-01, 9.9644E+00, &
      3.001405E+01, 5.14549E+01, 6.7523E-01, 8.2723E+00, &
      3.208502E+01, 5.05798E+01, 5.9354E-01, 6.8802E+00, &
      3.250601E+01, 5.01580E+01, 5.7883E-01, 6.6383E+00, &
      3.300355E+01, 4.89195E+01, 5.6217E-01, 6.3680E+00, &
      3.313623E+01, 4.74332E+01, 5.5785E-01, 6.2985E+00, &
      3.333525E+01, 4.89383E+01, 3.4787E+00, 3.5314E+01, &
      3.383279E+01, 5.02993E+01, 3.3951E+00, 3.3972E+01, &
      3.429889E+01, 5.08358E+01, 3.3198E+00, 3.2780E+01, &
      3.666551E+01, 5.19788E+01, 2.9745E+00, 2.7529E+01, &
      3.919543E+01, 5.24920E+01, 2.6643E+00, 2.3118E+01, &
      4.189992E+01, 5.28009E+01, 2.3861E+00, 1.9415E+01, &
      4.479101E+01, 5.30062E+01, 2.1326E+00, 1.6277E+01, &
      4.788159E+01, 5.31422E+01, 1.9025E+00, 1.3626E+01, &
      5.118542E+01, 5.32332E+01, 1.6966E+00, 1.1407E+01, &
      5.471721E+01, 5.32932E+01, 1.5126E+00, 9.5506E+00, &
      5.849270E+01, 5.33312E+01, 1.3483E+00, 7.9987E+00, &
      6.252870E+01, 5.33535E+01, 1.2017E+00, 6.7020E+00, &
      6.684318E+01, 5.33967E+01, 1.0706E+00, 5.6167E+00, &
      7.145536E+01, 5.33976E+01, 9.5063E-01, 4.6959E+00, &
      7.638578E+01, 5.33913E+01, 8.4419E-01, 3.9298E+00, &
      8.165640E+01, 5.33819E+01, 7.4857E-01, 3.2876E+00, &
      8.729069E+01, 5.33679E+01, 6.6149E-01, 2.7448E+00, &
      9.331374E+01, 5.33500E+01, 5.8460E-01, 2.2952E+00, &
      9.975239E+01, 5.33303E+01, 5.1669E-01, 1.9224E+00, &
      1.066353E+02, 5.33097E+01, 4.5670E-01, 1.6135E+00, &
      1.139931E+02, 5.32891E+01, 4.0372E-01, 1.3571E+00, &
      1.218587E+02, 5.32686E+01, 3.5690E-01, 1.1443E+00, &
      1.302669E+02, 5.32488E+01, 3.1554E-01, 9.6763E-01, &
      1.392553E+02, 5.32296E+01, 2.7900E-01, 8.2081E-01, &
      1.488640E+02, 5.32114E+01, 2.4674E-01, 6.9881E-01, &
      1.591356E+02, 5.31942E+01, 2.1827E-01, 5.9734E-01, &
      1.701159E+02, 5.31781E+01, 1.9309E-01, 5.1278E-01, &
      1.818539E+02, 5.31630E+01, 1.7083E-01, 4.4224E-01, &
      1.944018E+02, 5.31489E+01, 1.5114E-01, 3.8333E-01, &
      2.078156E+02, 5.31359E+01, 1.3373E-01, 3.3404E-01, &
      2.221548E+02, 5.31238E+01, 1.1833E-01, 2.9275E-01, &
      2.374835E+02, 5.31126E+01, 1.0472E-01, 2.5808E-01, &
      2.538699E+02, 5.31024E+01, 9.2670E-02, 2.2891E-01, &
      2.713869E+02, 5.30929E+01, 8.2014E-02, 2.0430E-01, &
      2.901126E+02, 5.30843E+01, 7.2587E-02, 1.8348E-01, &
      3.101304E+02, 5.30764E+01, 6.4247E-02, 1.6580E-01, &
      3.315294E+02, 5.30692E+01, 5.6869E-02, 1.5074E-01, &
      3.544049E+02, 5.30626E+01, 5.0340E-02, 1.3784E-01, &
      3.788588E+02, 5.30566E+01, 4.4562E-02, 1.2676E-01, &
      4.050001E+02, 5.30511E+01, 3.9450E-02, 1.1718E-01, &
      4.329451E+02, 5.30461E+01, 3.4926E-02, 1.0885E-01]
   case(54)
      frel = -3.9600E-01
      allocate(nist_tab(868))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.74817E+01, 0.0000E+00, 1.2269E-04, &
      1.142761E-02, 1.74729E+01, 0.0000E+00, 1.6884E-04, &
      1.221612E-02, 1.74627E+01, 0.0000E+00, 2.3108E-04, &
      1.305903E-02, 1.74510E+01, 0.0000E+00, 3.1456E-04, &
      1.396010E-02, 1.74376E+01, 0.0000E+00, 4.2589E-04, &
      1.492335E-02, 1.74222E+01, 0.0000E+00, 5.7356E-04, &
      1.595306E-02, 1.74044E+01, 0.0000E+00, 7.6833E-04, &
      1.705382E-02, 1.73839E+01, 0.0000E+00, 1.0238E-03, &
      1.823053E-02, 1.73603E+01, 0.0000E+00, 1.3571E-03, &
      1.948844E-02, 1.73330E+01, 0.0000E+00, 1.7896E-03, &
      2.083314E-02, 1.73014E+01, 0.0000E+00, 2.3476E-03, &
      2.227063E-02, 1.72647E+01, 0.0000E+00, 3.0638E-03, &
      2.380730E-02, 1.72221E+01, 0.0000E+00, 3.9781E-03, &
      2.545001E-02, 1.71724E+01, 0.0000E+00, 5.1389E-03, &
      2.720606E-02, 1.71143E+01, 0.0000E+00, 6.6051E-03, &
      2.908327E-02, 1.70460E+01, 0.0000E+00, 8.4470E-03, &
      3.109002E-02, 1.69653E+01, 0.0000E+00, 1.0749E-02, &
      3.323523E-02, 1.68696E+01, 0.0000E+00, 1.3610E-02, &
      3.552846E-02, 1.67551E+01, 0.0000E+00, 1.7149E-02, &
      3.797993E-02, 1.66169E+01, 0.0000E+00, 2.1502E-02, &
      4.060054E-02, 1.64483E+01, 0.0000E+00, 2.6829E-02, &
      4.340198E-02, 1.62392E+01, 0.0000E+00, 3.3314E-02, &
      4.639671E-02, 1.59749E+01, 0.0000E+00, 4.1170E-02, &
      4.959809E-02, 1.56310E+01, 0.0000E+00, 5.0637E-02, &
      5.302035E-02, 1.51635E+01, 0.0000E+00, 6.1986E-02, &
      5.667876E-02, 1.44787E+01, 0.0000E+00, 7.5523E-02, &
      6.058959E-02, 1.32889E+01, 0.0000E+00, 9.1589E-02, &
      6.272000E-02, 1.19994E+01, 0.0000E+00, 1.0103E-01, &
      6.368000E-02, 1.05181E+01, 0.0000E+00, 1.0543E-01, &
      6.393600E-02, 9.04168E+00, 0.0000E+00, 1.0663E-01, &
      6.432000E-02, 1.01989E+01, 2.7373E+00, 1.3641E+04, &
      6.477028E-02, 1.07681E+01, 2.8683E+00, 1.4194E+04, &
      6.528000E-02, 1.10295E+01, 3.0230E+00, 1.4842E+04, &
      6.923942E-02, 1.13451E+01, 4.4855E+00, 2.0764E+04, &
      7.401695E-02, 1.19353E+01, 6.6806E+00, 2.8929E+04, &
      7.912411E-02, 1.34352E+01, 8.5363E+00, 3.4579E+04, &
      8.458368E-02, 1.54567E+01, 9.6404E+00, 3.6531E+04, &
      9.041995E-02, 1.75482E+01, 9.9217E+00, 3.5170E+04, &
      9.665893E-02, 1.93672E+01, 9.5457E+00, 3.1653E+04, &
      1.033284E-01, 2.07487E+01, 8.7673E+00, 2.7195E+04, &
      1.104581E-01, 2.16728E+01, 7.8187E+00, 2.2687E+04, &
      1.180797E-01, 2.21972E+01, 6.8627E+00, 1.8628E+04, &
      1.262272E-01, 2.23982E+01, 5.9916E+00, 1.5215E+04, &
      1.349368E-01, 2.23204E+01, 5.2451E+00, 1.2460E+04, &
      1.437660E-01, 2.18272E+01, 4.6587E+00, 1.0387E+04, &
      1.442475E-01, 2.17629E+01, 4.6310E+00, 1.0291E+04, &
      1.459665E-01, 2.13416E+01, 4.5353E+00, 9.9595E+03, &
      1.465533E-01, 2.07968E+01, 4.5038E+00, 9.8506E+03, &
      1.474335E-01, 2.13224E+01, 5.5082E+00, 1.1976E+04, &
      1.496340E-01, 2.17567E+01, 5.4034E+00, 1.1575E+04, &
      1.542005E-01, 2.20064E+01, 5.2082E+00, 1.0827E+04, &
      1.648404E-01, 2.21290E+01, 4.8526E+00, 9.4363E+03, &
      1.762144E-01, 2.20852E+01, 4.5958E+00, 8.3603E+03, &
      1.883732E-01, 2.19645E+01, 4.4226E+00, 7.5262E+03, &
      2.013709E-01, 2.17702E+01, 4.3179E+00, 6.8740E+03, &
      2.039380E-01, 2.16898E+01, 4.3047E+00, 6.7667E+03, &
      2.070595E-01, 2.14862E+01, 4.2913E+00, 6.6441E+03, &
      2.078919E-01, 2.12694E+01, 4.2883E+00, 6.6128E+03, &
      2.091405E-01, 2.14691E+01, 4.6957E+00, 7.1979E+03, &
      2.122620E-01, 2.16278E+01, 4.6923E+00, 7.0868E+03, &
      2.152655E-01, 2.16791E+01, 4.6913E+00, 6.9864E+03, &
      2.301188E-01, 2.17426E+01, 4.7129E+00, 6.5659E+03, &
      2.459970E-01, 2.17454E+01, 4.7685E+00, 6.2149E+03, &
      2.629708E-01, 2.17359E+01, 4.8475E+00, 5.9103E+03, &
      2.811158E-01, 2.17211E+01, 4.9411E+00, 5.6359E+03, &
      3.005128E-01, 2.16987E+01, 5.0418E+00, 5.3799E+03, &
      3.212482E-01, 2.16617E+01, 5.1427E+00, 5.1336E+03, &
      3.434143E-01, 2.15992E+01, 5.2375E+00, 4.8912E+03, &
      3.671099E-01, 2.14951E+01, 5.3204E+00, 4.6483E+03, &
      3.924405E-01, 2.13273E+01, 5.3862E+00, 4.4026E+03, &
      4.195189E-01, 2.10644E+01, 5.4308E+00, 4.1529E+03, &
      4.484657E-01, 2.06582E+01, 5.4507E+00, 3.8995E+03, &
      4.794098E-01, 2.00448E+01, 5.4436E+00, 3.6436E+03, &
      5.124891E-01, 1.91067E+01, 5.4084E+00, 3.3870E+03, &
      5.478508E-01, 1.76322E+01, 5.3453E+00, 3.1320E+03, &
      5.856525E-01, 1.51523E+01, 5.2553E+00, 2.8812E+03, &
      6.260625E-01, 1.02292E+01, 5.1399E+00, 2.6367E+03, &
      6.588540E-01, -2.98003E-01, 5.0359E+00, 2.4553E+03, &
      6.692609E-01, -1.24339E+01, 5.0016E+00, 2.4009E+03, &
      6.716920E-01, -2.45681E+01, 4.9935E+00, 2.3884E+03, &
      6.756615E-01, -1.32073E+01, 2.7559E+01, 1.3079E+04, &
      6.819730E-01, -9.82041E+00, 2.7119E+01, 1.2752E+04, &
      6.847146E-01, -1.53958E+01, 2.6932E+01, 1.2613E+04, &
      6.857460E-01, -1.79455E+01, 4.1885E+01, 1.9583E+04, &
      6.888270E-01, -5.13804E+00, 4.1543E+01, 1.9336E+04, &
      6.991080E-01, 5.78404E+00, 4.0435E+01, 1.8544E+04, &
      7.154399E-01, 1.37642E+01, 3.8768E+01, 1.7374E+04, &
      7.648052E-01, 2.45976E+01, 3.4400E+01, 1.4422E+04, &
      8.175768E-01, 2.97070E+01, 3.0971E+01, 1.2147E+04, &
      8.739896E-01, 3.25139E+01, 2.8170E+01, 1.0338E+04, &
      9.182600E-01, 3.29938E+01, 2.6334E+01, 9.1986E+03, &
      9.323150E-01, 3.18535E+01, 2.5798E+01, 8.8758E+03, &
      9.342948E-01, 3.12765E+01, 2.5725E+01, 8.8316E+03, &
      9.360630E-01, 3.00986E+01, 2.5659E+01, 8.7925E+03, &
      9.416850E-01, 3.21316E+01, 2.9100E+01, 9.9114E+03, &
      9.557400E-01, 3.40977E+01, 2.8595E+01, 9.5964E+03, &
      9.790200E-01, 3.53823E+01, 2.7794E+01, 9.1061E+03, &
      9.940050E-01, 3.54177E+01, 2.7300E+01, 8.8096E+03, &
      9.987612E-01, 3.39244E+01, 2.7146E+01, 8.7184E+03, &
      1.003995E+00, 3.59190E+01, 2.8757E+01, 9.1872E+03, &
      1.018980E+00, 3.73675E+01, 2.8243E+01, 8.8904E+03, &
      1.067676E+00, 3.97178E+01, 2.6672E+01, 8.0140E+03, &
      1.121708E+00, 4.11467E+01, 2.5123E+01, 7.1858E+03, &
      1.138877E+00, 4.11545E+01, 2.4669E+01, 6.9499E+03, &
      1.141345E+00, 4.10201E+01, 2.4600E+01, 6.9155E+03, &
      1.143455E+00, 4.07001E+01, 2.4542E+01, 6.8864E+03, &
      1.150323E+00, 4.14707E+01, 2.5494E+01, 7.1105E+03, &
      1.167492E+00, 4.24232E+01, 2.5011E+01, 6.8735E+03, &
      1.220098E+00, 4.39835E+01, 2.3587E+01, 6.2034E+03, &
      1.304285E+00, 4.55566E+01, 2.1576E+01, 5.3095E+03, &
      1.394281E+00, 4.66886E+01, 1.9756E+01, 4.5489E+03, &
      1.490486E+00, 4.75513E+01, 1.8107E+01, 3.9012E+03, &
      1.593329E+00, 4.82356E+01, 1.6595E+01, 3.3458E+03, &
      1.703269E+00, 4.87650E+01, 1.5167E+01, 2.8616E+03, &
      1.820795E+00, 4.91379E+01, 1.3829E+01, 2.4420E+03, &
      1.946430E+00, 4.93845E+01, 1.2613E+01, 2.0845E+03, &
      2.080733E+00, 4.95404E+01, 1.1509E+01, 1.7804E+03, &
      2.224304E+00, 4.96205E+01, 1.0509E+01, 1.5217E+03, &
      2.377781E+00, 4.96630E+01, 9.5998E+00, 1.3013E+03, &
      2.541848E+00, 4.96398E+01, 8.7741E+00, 1.1136E+03, &
      2.717235E+00, 4.95830E+01, 8.0226E+00, 9.5338E+02, &
      2.904724E+00, 4.94927E+01, 7.2962E+00, 8.1199E+02, &
      3.105150E+00, 4.93460E+01, 6.5778E+00, 6.8572E+02, &
      3.319406E+00, 4.90369E+01, 5.8878E+00, 5.7509E+02, &
      3.548445E+00, 4.86191E+01, 5.2812E+00, 4.8341E+02, &
      3.793288E+00, 4.80668E+01, 4.7420E+00, 4.0686E+02, &
      4.055024E+00, 4.72888E+01, 4.2430E+00, 3.4133E+02, &
      4.334821E+00, 4.60511E+01, 3.7928E+00, 2.8619E+02, &
      4.633924E+00, 4.32904E+01, 3.3957E+00, 2.4039E+02, &
      4.686556E+00, 4.22221E+01, 3.3332E+00, 2.3344E+02, &
      4.758289E+00, 3.89074E+01, 3.2512E+00, 2.2443E+02, &
      4.777418E+00, 3.50859E+01, 3.2299E+00, 2.2211E+02, &
      4.806111E+00, 3.88484E+01, 1.0612E+01, 7.1311E+02, &
      4.877844E+00, 4.20237E+01, 1.0271E+01, 6.8023E+02, &
      4.953664E+00, 4.30390E+01, 9.9278E+00, 6.4765E+02, &
      5.001626E+00, 4.32058E+01, 9.7191E+00, 6.2808E+02, &
      5.078182E+00, 4.23055E+01, 9.4330E+00, 6.0058E+02, &
      5.098596E+00, 4.05721E+01, 9.3624E+00, 5.9374E+02, &
      5.129219E+00, 4.26732E+01, 1.2947E+01, 8.1420E+02, &
      5.205774E+00, 4.47523E+01, 1.2575E+01, 7.7935E+02, &
      5.295467E+00, 4.57697E+01, 1.2166E+01, 7.4145E+02, &
      5.343744E+00, 4.60586E+01, 1.1958E+01, 7.2224E+02, &
      5.425536E+00, 4.59458E+01, 1.1638E+01, 6.9247E+02, &
      5.447347E+00, 4.51763E+01, 1.1557E+01, 6.8493E+02, &
      5.480064E+00, 4.63112E+01, 1.3252E+01, 7.8001E+02, &
      5.561856E+00, 4.76166E+01, 1.2949E+01, 7.5113E+02, &
      5.660855E+00, 4.84989E+01, 1.2605E+01, 7.1852E+02, &
      6.051453E+00, 5.05102E+01, 1.1420E+01, 6.0947E+02, &
      6.469004E+00, 5.17598E+01, 1.0336E+01, 5.1646E+02, &
      6.915365E+00, 5.26129E+01, 9.3148E+00, 4.3587E+02, &
      7.392525E+00, 5.32022E+01, 8.3751E+00, 3.6703E+02, &
      7.902609E+00, 5.36109E+01, 7.5253E+00, 3.0890E+02, &
      8.447890E+00, 5.38997E+01, 6.7644E+00, 2.6012E+02, &
      9.030794E+00, 5.41224E+01, 6.0747E+00, 2.1887E+02, &
      9.653919E+00, 5.43248E+01, 5.4221E+00, 1.8308E+02, &
      1.032004E+01, 5.44259E+01, 4.8271E+00, 1.5279E+02, &
      1.103212E+01, 5.44732E+01, 4.2915E+00, 1.2736E+02, &
      1.179334E+01, 5.44466E+01, 3.8087E+00, 1.0601E+02, &
      1.260708E+01, 5.43916E+01, 3.3831E+00, 8.8337E+01, &
      1.347697E+01, 5.43183E+01, 3.0070E+00, 7.3675E+01, &
      1.440688E+01, 5.42329E+01, 2.6749E+00, 6.1516E+01, &
      1.540095E+01, 5.41397E+01, 2.3817E+00, 5.1425E+01, &
      1.646362E+01, 5.40413E+01, 2.1226E+00, 4.3042E+01, &
      1.759961E+01, 5.39394E+01, 1.8933E+00, 3.6070E+01, &
      1.881398E+01, 5.38345E+01, 1.6890E+00, 3.0242E+01, &
      2.011215E+01, 5.37261E+01, 1.5075E+00, 2.5377E+01, &
      2.149988E+01, 5.36131E+01, 1.3446E+00, 2.1291E+01, &
      2.298338E+01, 5.34914E+01, 1.1978E+00, 1.7852E+01, &
      2.456923E+01, 5.33563E+01, 1.0650E+00, 1.4948E+01, &
      2.626450E+01, 5.31993E+01, 9.4548E-01, 1.2508E+01, &
      2.807676E+01, 5.30051E+01, 8.3239E-01, 1.0393E+01, &
      3.001405E+01, 5.27236E+01, 7.3122E-01, 8.6265E+00, &
      3.208502E+01, 5.22415E+01, 6.4288E-01, 7.1733E+00, &
      3.387017E+01, 5.11766E+01, 5.7946E-01, 6.1844E+00, &
      3.429889E+01, 5.03220E+01, 5.6569E-01, 5.9761E+00, &
      3.438859E+01, 4.99447E+01, 5.6288E-01, 5.9337E+00, &
      3.452684E+01, 4.84666E+01, 5.5858E-01, 5.8693E+00, &
      3.473421E+01, 4.99634E+01, 3.4633E+00, 3.2637E+01, &
      3.525263E+01, 5.13169E+01, 3.3800E+00, 3.1396E+01, &
      3.666551E+01, 5.24379E+01, 3.1686E+00, 2.8332E+01, &
      3.919543E+01, 5.32174E+01, 2.8388E+00, 2.3796E+01, &
      4.189992E+01, 5.36296E+01, 2.5432E+00, 1.9989E+01, &
      4.479101E+01, 5.38905E+01, 2.2781E+00, 1.6794E+01, &
      4.788159E+01, 5.40650E+01, 2.0335E+00, 1.4065E+01, &
      5.118542E+01, 5.41809E+01, 1.8141E+00, 1.1778E+01, &
      5.471721E+01, 5.42583E+01, 1.6179E+00, 9.8627E+00, &
      5.849270E+01, 5.43087E+01, 1.4425E+00, 8.2608E+00, &
      6.252870E+01, 5.43398E+01, 1.2859E+00, 6.9217E+00, &
      6.684318E+01, 5.43573E+01, 1.1462E+00, 5.8029E+00, &
      7.145536E+01, 5.43963E+01, 1.0200E+00, 4.8603E+00, &
      7.638578E+01, 5.43945E+01, 9.0616E-01, 4.0677E+00, &
      8.165640E+01, 5.43894E+01, 8.0382E-01, 3.4030E+00, &
      8.729069E+01, 5.43768E+01, 7.1056E-01, 2.8411E+00, &
      9.331374E+01, 5.43600E+01, 6.2818E-01, 2.3753E+00, &
      9.975239E+01, 5.43409E+01, 5.5540E-01, 1.9892E+00, &
      1.066353E+02, 5.43206E+01, 4.9109E-01, 1.6689E+00, &
      1.139931E+02, 5.43000E+01, 4.3427E-01, 1.4031E+00, &
      1.218587E+02, 5.42794E+01, 3.8405E-01, 1.1825E+00, &
      1.302669E+02, 5.42592E+01, 3.3966E-01, 9.9930E-01, &
      1.392553E+02, 5.42397E+01, 3.0042E-01, 8.4702E-01, &
      1.488640E+02, 5.42210E+01, 2.6574E-01, 7.2039E-01, &
      1.591356E+02, 5.42032E+01, 2.3520E-01, 6.1523E-01, &
      1.701159E+02, 5.41866E+01, 2.0820E-01, 5.2760E-01, &
      1.818539E+02, 5.41709E+01, 1.8430E-01, 4.5448E-01, &
      1.944018E+02, 5.41564E+01, 1.6316E-01, 3.9340E-01, &
      2.078156E+02, 5.41428E+01, 1.4446E-01, 3.4231E-01, &
      2.221548E+02, 5.41302E+01, 1.2790E-01, 2.9948E-01, &
      2.374835E+02, 5.41186E+01, 1.1325E-01, 2.6354E-01, &
      2.538699E+02, 5.41079E+01, 1.0029E-01, 2.3330E-01, &
      2.713869E+02, 5.40981E+01, 8.8808E-02, 2.0779E-01, &
      2.901126E+02, 5.40890E+01, 7.8648E-02, 1.8623E-01, &
      3.101304E+02, 5.40807E+01, 6.9654E-02, 1.6793E-01, &
      3.315294E+02, 5.40732E+01, 6.1692E-02, 1.5235E-01, &
      3.544049E+02, 5.40662E+01, 5.4642E-02, 1.3903E-01, &
      3.788588E+02, 5.40599E+01, 4.8400E-02, 1.2758E-01, &
      4.050001E+02, 5.40542E+01, 4.2873E-02, 1.1771E-01, &
      4.329451E+02, 5.40489E+01, 3.7979E-02, 1.0914E-01]
   case(55)
      frel = -4.1400E-01
      allocate(nist_tab(976))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, -1.21026E+01, 0.0000E+00, 1.2288E-04, &
      1.117200E-02, -1.74936E+01, 0.0000E+00, 1.5180E-04, &
      1.134300E-02, -2.44803E+01, 0.0000E+00, 1.6317E-04, &
      1.138860E-02, -3.28699E+01, 0.0000E+00, 1.6631E-04, &
      1.142761E-02, -2.79838E+01, 1.6606E+01, 4.6009E+05, &
      1.145700E-02, -2.40414E+01, 1.6512E+01, 4.5631E+05, &
      1.162800E-02, -1.63832E+01, 1.5981E+01, 4.3516E+05, &
      1.221612E-02, -9.79847E+00, 1.4334E+01, 3.7151E+05, &
      1.283800E-02, -9.31038E+00, 1.2830E+01, 3.1643E+05, &
      1.303450E-02, -1.23379E+01, 1.2378E+01, 3.0068E+05, &
      1.305903E-02, -1.35900E+01, 1.2323E+01, 2.9876E+05, &
      1.308690E-02, -1.68117E+01, 1.2259E+01, 2.9660E+05, &
      1.316550E-02, -1.15245E+01, 2.1314E+01, 5.1259E+05, &
      1.336200E-02, -6.49255E+00, 2.0509E+01, 4.8596E+05, &
      1.396010E-02, -1.15375E+00, 1.8246E+01, 4.1382E+05, &
      1.492335E-02, 2.52001E+00, 1.5149E+01, 3.2141E+05, &
      1.595306E-02, 4.35711E+00, 1.2455E+01, 2.4720E+05, &
      1.705382E-02, 5.23957E+00, 1.0168E+01, 1.8877E+05, &
      1.823053E-02, 5.54833E+00, 8.2640E+00, 1.4352E+05, &
      1.948844E-02, 5.49875E+00, 6.7034E+00, 1.0891E+05, &
      2.083314E-02, 5.22332E+00, 5.4384E+00, 8.2651E+04, &
      2.224600E-02, 4.81062E+00, 4.4360E+00, 6.3135E+04, &
      2.227063E-02, 4.80269E+00, 4.4209E+00, 6.2851E+04, &
      2.258650E-02, 4.69752E+00, 4.2335E+00, 5.9345E+04, &
      2.267730E-02, 4.66282E+00, 4.1817E+00, 5.8384E+04, &
      2.281350E-02, 4.62350E+00, 4.1161E+00, 5.7125E+04, &
      2.315400E-02, 4.51550E+00, 3.9350E+00, 5.3809E+04, &
      2.380730E-02, 4.29825E+00, 3.6186E+00, 4.8124E+04, &
      2.545001E-02, 3.73335E+00, 2.9719E+00, 3.6972E+04, &
      2.720606E-02, 3.11987E+00, 2.4576E+00, 2.8601E+04, &
      2.908327E-02, 2.46113E+00, 2.0490E+00, 2.2306E+04, &
      3.109002E-02, 1.75118E+00, 1.7242E+00, 1.7559E+04, &
      3.323523E-02, 9.76396E-01, 1.4659E+00, 1.3965E+04, &
      3.552846E-02, 1.21130E-01, 1.2603E+00, 1.1231E+04, &
      3.797993E-02, -8.56516E-01, 1.0965E+00, 9.1406E+03, &
      4.060054E-02, -1.99122E+00, 9.6587E-01, 7.5322E+03, &
      4.340198E-02, -3.34832E+00, 8.6170E-01, 6.2861E+03, &
      4.639671E-02, -5.01452E+00, 7.7862E-01, 5.3134E+03, &
      4.959809E-02, -7.12612E+00, 7.1281E-01, 4.5504E+03, &
      5.302035E-02, -9.90451E+00, 6.6082E-01, 3.9463E+03, &
      5.667876E-02, -1.37385E+01, 6.1996E-01, 3.4633E+03, &
      6.058959E-02, -1.93856E+01, 5.8791E-01, 3.0723E+03, &
      6.477028E-02, -2.85658E+01, 5.6270E-01, 2.7508E+03, &
      6.923942E-02, -4.63389E+01, 5.4280E-01, 2.4822E+03, &
      7.401695E-02, -9.99590E+01, 5.2698E-01, 2.2544E+03, &
      7.497000E-02, -1.29058E+02, 5.2433E-01, 2.2146E+03, &
      7.611750E-02, -2.21925E+02, 5.2132E-01, 2.1687E+03, &
      7.642350E-02, -3.40742E+02, 5.2055E-01, 2.1568E+03, &
      7.688250E-02, -1.96862E+02, 2.3160E+02, 9.5377E+05, &
      7.722400E-02, -1.47117E+02, 2.1473E+02, 8.8041E+05, &
      7.803000E-02, -1.17448E+02, 1.7989E+02, 7.2993E+05, &
      7.840600E-02, -1.39264E+02, 1.6574E+02, 6.6928E+05, &
      7.872120E-02, -2.36373E+02, 1.5478E+02, 6.2254E+05, &
      7.912411E-02, -9.56342E+01, 3.6460E+02, 1.4590E+06, &
      7.919400E-02, -7.70908E+01, 3.5890E+02, 1.4349E+06, &
      8.037600E-02, 5.84873E+01, 2.7558E+02, 1.0856E+06, &
      8.458368E-02, 1.22570E+02, 1.1118E+02, 4.1617E+05, &
      9.041995E-02, 9.31188E+01, 3.7396E+01, 1.3095E+05, &
      9.665893E-02, 6.88276E+01, 1.6039E+01, 5.2538E+04, &
      1.033284E-01, 5.43537E+01, 8.5172E+00, 2.6098E+04, &
      1.104581E-01, 4.54066E+01, 5.3851E+00, 1.5436E+04, &
      1.180797E-01, 3.95075E+01, 3.9085E+00, 1.0480E+04, &
      1.262272E-01, 3.53773E+01, 3.1538E+00, 7.9114E+03, &
      1.349368E-01, 3.23276E+01, 2.7563E+00, 6.4681E+03, &
      1.442475E-01, 2.99529E+01, 2.5570E+00, 5.6132E+03, &
      1.542005E-01, 2.79430E+01, 2.4801E+00, 5.0932E+03, &
      1.583680E-01, 2.71051E+01, 2.4742E+00, 4.9474E+03, &
      1.607920E-01, 2.64045E+01, 2.4760E+00, 4.8765E+03, &
      1.614384E-01, 2.58952E+01, 2.4771E+00, 4.8591E+03, &
      1.624080E-01, 2.61848E+01, 3.3029E+00, 6.4400E+03, &
      1.648404E-01, 2.62309E+01, 3.3175E+00, 6.3732E+03, &
      1.688540E-01, 2.59271E+01, 3.3477E+00, 6.2782E+03, &
      1.714385E-01, 2.56084E+01, 3.3705E+00, 6.2258E+03, &
      1.721277E-01, 2.53979E+01, 3.3770E+00, 6.2129E+03, &
      1.731615E-01, 2.54860E+01, 3.7029E+00, 6.7717E+03, &
      1.757460E-01, 2.54439E+01, 3.7310E+00, 6.7228E+03, &
      1.762144E-01, 2.54243E+01, 3.7364E+00, 6.7145E+03, &
      1.883732E-01, 2.47988E+01, 3.8938E+00, 6.5459E+03, &
      2.013709E-01, 2.41919E+01, 4.0923E+00, 6.4358E+03, &
      2.152655E-01, 2.36419E+01, 4.3240E+00, 6.3614E+03, &
      2.261840E-01, 2.31850E+01, 4.5120E+00, 6.3178E+03, &
      2.296460E-01, 2.29041E+01, 4.5717E+00, 6.3048E+03, &
      2.301188E-01, 2.28143E+01, 4.5798E+00, 6.3030E+03, &
      2.305692E-01, 2.26410E+01, 4.5876E+00, 6.3014E+03, &
      2.319540E-01, 2.28565E+01, 5.0882E+00, 6.9471E+03, &
      2.354160E-01, 2.30017E+01, 5.1508E+00, 6.9292E+03, &
      2.459970E-01, 2.30229E+01, 5.3390E+00, 6.8736E+03, &
      2.629708E-01, 2.29649E+01, 5.6267E+00, 6.7767E+03, &
      2.811158E-01, 2.29380E+01, 5.9045E+00, 6.6525E+03, &
      3.005128E-01, 2.29549E+01, 6.1620E+00, 6.4947E+03, &
      3.212482E-01, 2.30087E+01, 6.3883E+00, 6.2991E+03, &
      3.434143E-01, 2.30822E+01, 6.5697E+00, 6.0601E+03, &
      3.671099E-01, 2.31512E+01, 6.7002E+00, 5.7819E+03, &
      3.924405E-01, 2.31889E+01, 6.7780E+00, 5.4719E+03, &
      4.195189E-01, 2.31648E+01, 6.8036E+00, 5.1386E+03, &
      4.484657E-01, 2.30427E+01, 6.7792E+00, 4.7902E+03, &
      4.794098E-01, 2.27759E+01, 6.7095E+00, 4.4355E+03, &
      5.124891E-01, 2.23011E+01, 6.5999E+00, 4.0820E+03, &
      5.478508E-01, 2.15152E+01, 6.4564E+00, 3.7361E+03, &
      5.856525E-01, 2.02337E+01, 6.2849E+00, 3.4029E+03, &
      6.260625E-01, 1.80683E+01, 6.0915E+00, 3.0860E+03, &
      6.692609E-01, 1.38951E+01, 5.8812E+00, 2.7879E+03, &
      7.109900E-01, 2.98332E+00, 5.6798E+00, 2.5352E+03, &
      7.154399E-01, 6.04433E-02, 5.6585E+00, 2.5101E+03, &
      7.218725E-01, -7.72576E+00, 5.6279E+00, 2.4744E+03, &
      7.247100E-01, -1.85884E+01, 5.6145E+00, 2.4588E+03, &
      7.247745E-01, -1.91802E+01, 5.6142E+00, 2.4585E+03, &
      7.291275E-01, -9.26291E+00, 2.6798E+01, 1.1643E+04, &
      7.358025E-01, -6.15356E+00, 2.6411E+01, 1.1371E+04, &
      7.387605E-01, -1.13635E+01, 2.6242E+01, 1.1253E+04, &
      7.400100E-01, -1.22738E+01, 4.0226E+01, 1.7217E+04, &
      7.431975E-01, -1.76746E+00, 3.9937E+01, 1.7020E+04, &
      7.542900E-01, 8.45083E+00, 3.8958E+01, 1.6359E+04, &
      7.648052E-01, 1.34408E+01, 3.8065E+01, 1.5764E+04, &
      8.175768E-01, 2.52035E+01, 3.4073E+01, 1.3201E+04, &
      8.739896E-01, 3.05574E+01, 3.0811E+01, 1.1169E+04, &
      9.342948E-01, 3.34544E+01, 2.8078E+01, 9.5219E+03, &
      9.776480E-01, 3.38710E+01, 2.6398E+01, 8.5561E+03, &
      9.926120E-01, 3.26294E+01, 2.5859E+01, 8.2553E+03, &
      9.966024E-01, 3.06735E+01, 2.5718E+01, 8.1775E+03, &
      9.987612E-01, 3.09608E+01, 2.9754E+01, 9.4392E+03, &
      1.002588E+00, 3.29983E+01, 2.9597E+01, 9.3539E+03, &
      1.017552E+00, 3.52241E+01, 2.9002E+01, 9.0312E+03, &
      1.043700E+00, 3.67290E+01, 2.8014E+01, 8.5055E+03, &
      1.059675E+00, 3.68058E+01, 2.7441E+01, 8.2061E+03, &
      1.063935E+00, 3.61034E+01, 2.7292E+01, 8.1289E+03, &
      1.067676E+00, 3.67852E+01, 2.8939E+01, 8.5888E+03, &
      1.070325E+00, 3.72994E+01, 2.8836E+01, 8.5374E+03, &
      1.086300E+00, 3.87407E+01, 2.8234E+01, 8.2364E+03, &
      1.141345E+00, 4.10242E+01, 2.6429E+01, 7.3389E+03, &
      1.192758E+00, 4.21692E+01, 2.5038E+01, 6.6536E+03, &
      1.211015E+00, 4.21411E+01, 2.4588E+01, 6.4358E+03, &
      1.215883E+00, 4.16738E+01, 2.4471E+01, 6.3797E+03, &
      1.220098E+00, 4.21059E+01, 2.5527E+01, 6.6317E+03, &
      1.223186E+00, 4.24448E+01, 2.5452E+01, 6.5956E+03, &
      1.241442E+00, 4.33922E+01, 2.4988E+01, 6.3803E+03, &
      1.304285E+00, 4.50837E+01, 2.3435E+01, 5.6965E+03, &
      1.394281E+00, 4.66262E+01, 2.1528E+01, 4.8963E+03, &
      1.490486E+00, 4.77757E+01, 1.9742E+01, 4.2013E+03, &
      1.593329E+00, 4.86563E+01, 1.8090E+01, 3.6025E+03, &
      1.703269E+00, 4.93538E+01, 1.6560E+01, 3.0860E+03, &
      1.820795E+00, 4.98622E+01, 1.5099E+01, 2.6333E+03, &
      1.946430E+00, 5.02114E+01, 1.3769E+01, 2.2474E+03, &
      2.080733E+00, 5.04447E+01, 1.2563E+01, 1.9193E+03, &
      2.224304E+00, 5.06470E+01, 1.1459E+01, 1.6387E+03, &
      2.377781E+00, 5.07185E+01, 1.0455E+01, 1.3996E+03, &
      2.541848E+00, 5.07506E+01, 9.5448E+00, 1.1962E+03, &
      2.717235E+00, 5.07159E+01, 8.7209E+00, 1.0234E+03, &
      2.904724E+00, 5.06493E+01, 7.9771E+00, 8.7654E+02, &
      3.105150E+00, 5.05732E+01, 7.1995E+00, 7.4096E+02, &
      3.319406E+00, 5.03418E+01, 6.4441E+00, 6.2135E+02, &
      3.548445E+00, 5.00012E+01, 5.7780E+00, 5.2205E+02, &
      3.793288E+00, 4.95485E+01, 5.1910E+00, 4.3957E+02, &
      4.055024E+00, 4.89460E+01, 4.6666E+00, 3.7044E+02, &
      4.334821E+00, 4.80760E+01, 4.1851E+00, 3.1153E+02, &
      4.633924E+00, 4.66164E+01, 3.7579E+00, 2.6240E+02, &
      4.911662E+00, 4.33790E+01, 3.4164E+00, 2.2566E+02, &
      4.953664E+00, 4.20880E+01, 3.3694E+00, 2.2076E+02, &
      4.986840E+00, 4.01016E+01, 3.3331E+00, 2.1699E+02, &
      5.006888E+00, 3.63183E+01, 3.3114E+00, 2.1476E+02, &
      5.036959E+00, 4.00489E+01, 1.0623E+01, 6.7309E+02, &
      5.112138E+00, 4.32059E+01, 1.0283E+01, 6.4217E+02, &
      5.252212E+00, 4.44330E+01, 9.6920E+00, 5.8945E+02, &
      5.295467E+00, 4.42277E+01, 9.5385E+00, 5.7547E+02, &
      5.332603E+00, 4.35105E+01, 9.4138E+00, 5.6407E+02, &
      5.354040E+00, 4.17843E+01, 9.3437E+00, 5.5768E+02, &
      5.386197E+00, 4.38628E+01, 1.2901E+01, 7.6348E+02, &
      5.466588E+00, 4.59086E+01, 1.2529E+01, 7.3070E+02, &
      5.600014E+00, 4.71111E+01, 1.1957E+01, 6.8099E+02, &
      5.660855E+00, 4.71838E+01, 1.1722E+01, 6.6058E+02, &
      5.685729E+00, 4.70101E+01, 1.1632E+01, 6.5266E+02, &
      5.708586E+00, 4.62463E+01, 1.1551E+01, 6.4553E+02, &
      5.742872E+00, 4.73775E+01, 1.3236E+01, 7.3464E+02, &
      5.828586E+00, 4.86805E+01, 1.2933E+01, 7.0739E+02, &
      6.051453E+00, 5.03007E+01, 1.2217E+01, 6.4392E+02, &
      6.469004E+00, 5.19890E+01, 1.1075E+01, 5.4650E+02, &
      6.915365E+00, 5.30921E+01, 1.0007E+01, 4.6241E+02, &
      7.392525E+00, 5.38505E+01, 9.0110E+00, 3.8994E+02, &
      7.902609E+00, 5.43753E+01, 8.1009E+00, 3.2834E+02, &
      8.447890E+00, 5.47429E+01, 7.2827E+00, 2.7651E+02, &
      9.030794E+00, 5.50192E+01, 6.5399E+00, 2.3263E+02, &
      9.653919E+00, 5.51931E+01, 5.8397E+00, 1.9466E+02, &
      1.032004E+01, 5.53657E+01, 5.2103E+00, 1.6279E+02, &
      1.103212E+01, 5.54449E+01, 4.6386E+00, 1.3588E+02, &
      1.179334E+01, 5.54785E+01, 4.1219E+00, 1.1322E+02, &
      1.260708E+01, 5.54399E+01, 3.6594E+00, 9.4289E+01, &
      1.347697E+01, 5.53771E+01, 3.2515E+00, 7.8607E+01, &
      1.440688E+01, 5.52986E+01, 2.8911E+00, 6.5595E+01, &
      1.540095E+01, 5.52096E+01, 2.5727E+00, 5.4799E+01, &
      1.646362E+01, 5.51139E+01, 2.2914E+00, 4.5834E+01, &
      1.759961E+01, 5.50136E+01, 2.0427E+00, 3.8382E+01, &
      1.881398E+01, 5.49102E+01, 1.8226E+00, 3.2181E+01, &
      2.011215E+01, 5.48037E+01, 1.6265E+00, 2.6997E+01, &
      2.149988E+01, 5.46938E+01, 1.4529E+00, 2.2677E+01, &
      2.298338E+01, 5.45784E+01, 1.2958E+00, 1.9030E+01, &
      2.456923E+01, 5.44532E+01, 1.1546E+00, 1.5964E+01, &
      2.626450E+01, 5.43136E+01, 1.0260E+00, 1.3366E+01, &
      2.807676E+01, 5.41460E+01, 9.0351E-01, 1.1105E+01, &
      3.001405E+01, 5.39188E+01, 7.9396E-01, 9.2164E+00, &
      3.208502E+01, 5.35825E+01, 6.9828E-01, 7.6628E+00, &
      3.429889E+01, 5.29105E+01, 6.1464E-01, 6.3829E+00, &
      3.526491E+01, 5.21950E+01, 5.8299E-01, 5.9185E+00, &
      3.580468E+01, 5.09706E+01, 5.6641E-01, 5.6799E+00, &
      3.594862E+01, 4.95025E+01, 5.6212E-01, 5.6186E+00, &
      3.616452E+01, 5.09879E+01, 3.4469E+00, 3.0840E+01, &
      3.666551E+01, 5.22763E+01, 3.3719E+00, 2.9768E+01, &
      3.670429E+01, 5.23309E+01, 3.3662E+00, 2.9688E+01, &
      3.919543E+01, 5.38290E+01, 3.0270E+00, 2.5050E+01, &
      4.189992E+01, 5.44128E+01, 2.7121E+00, 2.1044E+01, &
      4.479101E+01, 5.47513E+01, 2.4284E+00, 1.7672E+01, &
      4.788159E+01, 5.49722E+01, 2.1715E+00, 1.4824E+01, &
      5.118542E+01, 5.51185E+01, 1.9375E+00, 1.2414E+01, &
      5.471721E+01, 5.52163E+01, 1.7284E+00, 1.0397E+01, &
      5.849270E+01, 5.52810E+01, 1.5415E+00, 8.7099E+00, &
      6.252870E+01, 5.53226E+01, 1.3747E+00, 7.2995E+00, &
      6.684318E+01, 5.53477E+01, 1.2258E+00, 6.1204E+00, &
      7.145536E+01, 5.53613E+01, 1.0930E+00, 5.1348E+00, &
      7.638578E+01, 5.53958E+01, 9.7171E-01, 4.2994E+00, &
      8.165640E+01, 5.53951E+01, 8.6233E-01, 3.5973E+00, &
      8.729069E+01, 5.53844E+01, 7.6264E-01, 3.0034E+00, &
      9.331374E+01, 5.53689E+01, 6.7454E-01, 2.5112E+00, &
      9.975239E+01, 5.53507E+01, 5.9667E-01, 2.1029E+00, &
      1.066353E+02, 5.53310E+01, 5.2783E-01, 1.7641E+00, &
      1.139931E+02, 5.53105E+01, 4.6698E-01, 1.4829E+00, &
      1.218587E+02, 5.52899E+01, 4.1317E-01, 1.2494E+00, &
      1.302669E+02, 5.52695E+01, 3.6559E-01, 1.0555E+00, &
      1.392553E+02, 5.52497E+01, 3.2351E-01, 8.9423E-01, &
      1.488640E+02, 5.52306E+01, 2.8630E-01, 7.6009E-01, &
      1.591356E+02, 5.52123E+01, 2.5345E-01, 6.4854E-01, &
      1.701159E+02, 5.51951E+01, 2.2452E-01, 5.5583E-01, &
      1.818539E+02, 5.51790E+01, 1.9890E-01, 4.7843E-01, &
      1.944018E+02, 5.51639E+01, 1.7622E-01, 4.1376E-01, &
      2.078156E+02, 5.51498E+01, 1.5613E-01, 3.5963E-01, &
      2.221548E+02, 5.51368E+01, 1.3835E-01, 3.1426E-01, &
      2.374835E+02, 5.51247E+01, 1.2259E-01, 2.7617E-01, &
      2.538699E+02, 5.51136E+01, 1.0864E-01, 2.4412E-01, &
      2.713869E+02, 5.51033E+01, 9.6275E-02, 2.1708E-01, &
      2.901126E+02, 5.50939E+01, 8.5325E-02, 1.9421E-01, &
      3.101304E+02, 5.50852E+01, 7.5625E-02, 1.7481E-01, &
      3.315294E+02, 5.50773E+01, 6.7031E-02, 1.5828E-01, &
      3.544049E+02, 5.50700E+01, 5.9416E-02, 1.4416E-01, &
      3.788588E+02, 5.50634E+01, 5.2669E-02, 1.3203E-01, &
      4.050001E+02, 5.50574E+01, 4.6690E-02, 1.2155E-01, &
      4.329451E+02, 5.50519E+01, 4.1392E-02, 1.1247E-01]
   case(56)
      frel = -4.3260E-01
      allocate(nist_tab(964))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.43338E+01, 0.0000E+00, 1.2687E-04, &
      1.142761E-02, 1.39405E+01, 0.0000E+00, 1.7422E-04, &
      1.221612E-02, 1.33645E+01, 0.0000E+00, 2.3795E-04, &
      1.305903E-02, 1.24158E+01, 0.0000E+00, 3.2327E-04, &
      1.396010E-02, 1.03261E+01, 0.0000E+00, 4.3685E-04, &
      1.430800E-02, 8.32930E+00, 0.0000E+00, 4.8753E-04, &
      1.452700E-02, 4.62144E+00, 0.0000E+00, 5.2150E-04, &
      1.458540E-02, 1.64260E-01, 0.0000E+00, 5.3084E-04, &
      1.467300E-02, 4.86407E+00, 8.7740E+00, 1.8323E+05, &
      1.489200E-02, 8.93756E+00, 8.4767E+00, 1.7442E+05, &
      1.492335E-02, 9.23744E+00, 8.4354E+00, 1.7320E+05, &
      1.595306E-02, 1.27559E+01, 7.2225E+00, 1.3873E+05, &
      1.626800E-02, 1.25921E+01, 6.8990E+00, 1.2995E+05, &
      1.651700E-02, 1.11690E+01, 6.6500E+00, 1.2337E+05, &
      1.658340E-02, 9.00419E+00, 6.5846E+00, 1.2167E+05, &
      1.668300E-02, 1.15742E+01, 1.0979E+01, 2.0164E+05, &
      1.693200E-02, 1.40476E+01, 1.0596E+01, 1.9175E+05, &
      1.705382E-02, 1.46943E+01, 1.0413E+01, 1.8711E+05, &
      1.823053E-02, 1.77878E+01, 8.8180E+00, 1.4821E+05, &
      1.948844E-02, 1.92404E+01, 7.3910E+00, 1.1621E+05, &
      2.083314E-02, 2.00237E+01, 6.1288E+00, 9.0144E+04, &
      2.227063E-02, 2.04129E+01, 5.0483E+00, 6.9458E+04, &
      2.380730E-02, 2.05556E+01, 4.1450E+00, 5.3349E+04, &
      2.545001E-02, 2.05428E+01, 3.4028E+00, 4.0970E+04, &
      2.720606E-02, 2.04336E+01, 2.8003E+00, 3.1539E+04, &
      2.908327E-02, 2.02658E+01, 2.3150E+00, 2.4390E+04, &
      3.109002E-02, 2.00629E+01, 1.9260E+00, 1.8982E+04, &
      3.323523E-02, 1.98383E+01, 1.6149E+00, 1.4889E+04, &
      3.552846E-02, 1.95981E+01, 1.3662E+00, 1.1783E+04, &
      3.797993E-02, 1.93408E+01, 1.1671E+00, 9.4162E+03, &
      3.831800E-02, 1.93043E+01, 1.1438E+00, 9.1469E+03, &
      3.890450E-02, 1.92365E+01, 1.1055E+00, 8.7070E+03, &
      3.906090E-02, 1.92113E+01, 1.0957E+00, 8.5951E+03, &
      3.929550E-02, 1.91959E+01, 1.0993E+00, 8.5725E+03, &
      3.988200E-02, 1.91437E+01, 1.0663E+00, 8.1927E+03, &
      4.060054E-02, 1.90744E+01, 1.0288E+00, 7.7647E+03, &
      4.340198E-02, 1.87969E+01, 9.0911E-01, 6.4183E+03, &
      4.639671E-02, 1.84984E+01, 8.1645E-01, 5.3921E+03, &
      4.959809E-02, 1.81587E+01, 7.4504E-01, 4.6030E+03, &
      5.302035E-02, 1.77710E+01, 6.9046E-01, 3.9905E+03, &
      5.667876E-02, 1.73012E+01, 6.4907E-01, 3.5091E+03, &
      6.058959E-02, 1.67134E+01, 6.1794E-01, 3.1252E+03, &
      6.477028E-02, 1.59402E+01, 5.9492E-01, 2.8146E+03, &
      6.923942E-02, 1.48562E+01, 5.7800E-01, 2.5580E+03, &
      7.401695E-02, 1.31961E+01, 5.6569E-01, 2.3421E+03, &
      7.912411E-02, 1.02689E+01, 5.5668E-01, 2.1560E+03, &
      8.458368E-02, 3.36627E+00, 5.4997E-01, 1.9926E+03, &
      8.810200E-02, -1.08739E+01, 5.4668E-01, 1.9016E+03, &
      8.945050E-02, -3.35580E+01, 5.4558E-01, 1.8692E+03, &
      8.981010E-02, -6.36243E+01, 5.4530E-01, 1.8608E+03, &
      9.041995E-02, -2.25694E+01, 5.8904E+01, 1.9961E+05, &
      9.065000E-02, -1.48234E+01, 5.6300E+01, 1.9031E+05, &
      9.169800E-02, -2.34496E+00, 4.5897E+01, 1.5337E+05, &
      9.203750E-02, -4.39435E+00, 4.2984E+01, 1.4311E+05, &
      9.240750E-02, -1.97423E+01, 4.0033E+01, 1.3275E+05, &
      9.296250E-02, 8.16737E+00, 7.3068E+01, 2.4084E+05, &
      9.435000E-02, 3.26022E+01, 5.6274E+01, 1.8276E+05, &
      9.665893E-02, 4.29232E+01, 3.6802E+01, 1.1666E+05, &
      1.033284E-01, 4.01512E+01, 1.2040E+01, 3.5704E+04, &
      1.104581E-01, 3.40274E+01, 5.2283E+00, 1.4504E+04, &
      1.180797E-01, 3.03135E+01, 3.0158E+00, 7.8266E+03, &
      1.262272E-01, 2.80185E+01, 2.1488E+00, 5.2169E+03, &
      1.349368E-01, 2.64755E+01, 1.7723E+00, 4.0252E+03, &
      1.442475E-01, 2.53420E+01, 1.6134E+00, 3.4281E+03, &
      1.542005E-01, 2.44300E+01, 1.5720E+00, 3.1245E+03, &
      1.648404E-01, 2.36073E+01, 1.6076E+00, 2.9894E+03, &
      1.762144E-01, 2.26102E+01, 1.7039E+00, 2.9640E+03, &
      1.788015E-01, 2.21082E+01, 1.7322E+00, 2.9696E+03, &
      1.795203E-01, 2.16302E+01, 1.7405E+00, 2.9718E+03, &
      1.805985E-01, 2.20139E+01, 2.6163E+00, 4.4401E+03, &
      1.832940E-01, 2.22568E+01, 2.6598E+00, 4.4477E+03, &
      1.879640E-01, 2.22328E+01, 2.7399E+00, 4.4678E+03, &
      1.883732E-01, 2.22205E+01, 2.7472E+00, 4.4699E+03, &
      1.908410E-01, 2.20787E+01, 2.7919E+00, 4.4840E+03, &
      1.916082E-01, 2.19129E+01, 2.8061E+00, 4.4889E+03, &
      1.927590E-01, 2.20571E+01, 3.1367E+00, 4.9876E+03, &
      1.956360E-01, 2.21562E+01, 3.1955E+00, 5.0063E+03, &
      2.013709E-01, 2.21642E+01, 3.3173E+00, 5.0492E+03, &
      2.152655E-01, 2.20525E+01, 3.6322E+00, 5.1716E+03, &
      2.301188E-01, 2.19070E+01, 3.9889E+00, 5.3132E+03, &
      2.459970E-01, 2.16928E+01, 4.3789E+00, 5.4564E+03, &
      2.479400E-01, 2.16416E+01, 4.4266E+00, 5.4725E+03, &
      2.517350E-01, 2.14250E+01, 4.5193E+00, 5.5029E+03, &
      2.527470E-01, 2.11727E+01, 4.5439E+00, 5.5108E+03, &
      2.542650E-01, 2.14262E+01, 5.0752E+00, 6.1182E+03, &
      2.580600E-01, 2.16501E+01, 5.1705E+00, 6.1414E+03, &
      2.629708E-01, 2.17706E+01, 5.2924E+00, 6.1689E+03, &
      2.811158E-01, 2.20321E+01, 5.7251E+00, 6.2427E+03, &
      3.005128E-01, 2.22810E+01, 6.1432E+00, 6.2664E+03, &
      3.212482E-01, 2.25628E+01, 6.5280E+00, 6.2293E+03, &
      3.434143E-01, 2.28757E+01, 6.8637E+00, 6.1273E+03, &
      3.671099E-01, 2.32063E+01, 7.1372E+00, 5.9604E+03, &
      3.924405E-01, 2.35321E+01, 7.3318E+00, 5.7282E+03, &
      4.195189E-01, 2.38184E+01, 7.4408E+00, 5.4385E+03, &
      4.484657E-01, 2.40286E+01, 7.4684E+00, 5.1068E+03, &
      4.794098E-01, 2.41237E+01, 7.4226E+00, 4.7484E+03, &
      5.124891E-01, 2.40578E+01, 7.3142E+00, 4.3777E+03, &
      5.478508E-01, 2.37758E+01, 7.1546E+00, 4.0064E+03, &
      5.856525E-01, 2.31801E+01, 6.9554E+00, 3.6441E+03, &
      6.260625E-01, 2.21087E+01, 6.7270E+00, 3.2977E+03, &
      6.692609E-01, 2.02275E+01, 6.4787E+00, 2.9718E+03, &
      7.154399E-01, 1.65998E+01, 6.2181E+00, 2.6690E+03, &
      7.648052E-01, 5.88726E+00, 5.9507E+00, 2.3902E+03, &
      7.650860E-01, 5.75176E+00, 5.9492E+00, 2.3888E+03, &
      7.767965E-01, -4.33866E+00, 5.8879E+00, 2.3286E+03, &
      7.801780E-01, -1.77417E+01, 5.8703E+00, 2.3117E+03, &
      7.846035E-01, -5.78321E+00, 2.5816E+01, 1.0088E+04, &
      7.921195E-01, -2.80807E+00, 2.5455E+01, 9.8529E+03, &
      7.953039E-01, -7.78728E+00, 2.5304E+01, 9.7556E+03, &
      7.963140E-01, -1.28192E+01, 3.8538E+01, 1.4835E+04, &
      8.000805E-01, 1.21160E+00, 3.8257E+01, 1.4658E+04, &
      8.120220E-01, 1.07864E+01, 3.7389E+01, 1.4115E+04, &
      8.175768E-01, 1.33265E+01, 3.6997E+01, 1.3872E+04, &
      8.739896E-01, 2.58483E+01, 3.3381E+01, 1.1710E+04, &
      9.342948E-01, 3.13908E+01, 3.0314E+01, 9.9487E+03, &
      9.987612E-01, 3.43674E+01, 2.7662E+01, 8.4934E+03, &
      1.040956E+00, 3.47296E+01, 2.6088E+01, 7.6864E+03, &
      1.056889E+00, 3.34815E+01, 2.5535E+01, 7.4104E+03, &
      1.061138E+00, 3.15154E+01, 2.5392E+01, 7.3392E+03, &
      1.067676E+00, 3.38505E+01, 2.9274E+01, 8.4086E+03, &
      1.083444E+00, 3.60736E+01, 2.8639E+01, 8.1066E+03, &
      1.113966E+00, 3.76618E+01, 2.7474E+01, 7.5644E+03, &
      1.131017E+00, 3.76671E+01, 2.6916E+01, 7.2994E+03, &
      1.135563E+00, 3.69535E+01, 2.6773E+01, 7.2316E+03, &
      1.141345E+00, 3.79670E+01, 2.8357E+01, 7.6202E+03, &
      1.142384E+00, 3.81225E+01, 2.8322E+01, 7.6038E+03, &
      1.159434E+00, 3.95218E+01, 2.7758E+01, 7.3430E+03, &
      1.220098E+00, 4.17996E+01, 2.6003E+01, 6.5376E+03, &
      1.266944E+00, 4.27472E+01, 2.4880E+01, 6.0248E+03, &
      1.286336E+00, 4.27243E+01, 2.4455E+01, 5.8328E+03, &
      1.291507E+00, 4.22596E+01, 2.4345E+01, 5.7834E+03, &
      1.299264E+00, 4.30282E+01, 2.5333E+01, 5.9819E+03, &
      1.304285E+00, 4.33576E+01, 2.5227E+01, 5.9339E+03, &
      1.318656E+00, 4.39869E+01, 2.4929E+01, 5.8001E+03, &
      1.394281E+00, 4.59814E+01, 2.3285E+01, 5.1248E+03, &
      1.490486E+00, 4.75979E+01, 2.1370E+01, 4.4009E+03, &
      1.593329E+00, 4.87836E+01, 1.9565E+01, 3.7702E+03, &
      1.703269E+00, 4.96753E+01, 1.7896E+01, 3.2271E+03, &
      1.820795E+00, 5.03359E+01, 1.6327E+01, 2.7552E+03, &
      1.946430E+00, 5.08105E+01, 1.4895E+01, 2.3525E+03, &
      2.080733E+00, 5.11484E+01, 1.3600E+01, 2.0103E+03, &
      2.224304E+00, 5.14850E+01, 1.2407E+01, 1.7167E+03, &
      2.377781E+00, 5.16848E+01, 1.1310E+01, 1.4648E+03, &
      2.541848E+00, 5.17595E+01, 1.0315E+01, 1.2508E+03, &
      2.717235E+00, 5.17921E+01, 9.4155E+00, 1.0689E+03, &
      2.904724E+00, 5.17628E+01, 8.6035E+00, 9.1456E+02, &
      3.105150E+00, 5.17419E+01, 7.8048E+00, 7.7700E+02, &
      3.319406E+00, 5.15927E+01, 6.9934E+00, 6.5221E+02, &
      3.548445E+00, 5.13222E+01, 6.2655E+00, 5.4751E+02, &
      3.793288E+00, 5.09469E+01, 5.6213E+00, 4.6034E+02, &
      4.055024E+00, 5.04516E+01, 5.0525E+00, 3.8784E+02, &
      4.334821E+00, 4.97836E+01, 4.5498E+00, 3.2745E+02, &
      4.633924E+00, 4.88026E+01, 4.1024E+00, 2.7688E+02, &
      4.953664E+00, 4.69999E+01, 3.6853E+00, 2.3335E+02, &
      5.142060E+00, 4.45430E+01, 3.4671E+00, 2.1187E+02, &
      5.220765E+00, 4.13094E+01, 3.3825E+00, 2.0373E+02, &
      5.241753E+00, 3.75721E+01, 3.3605E+00, 2.0164E+02, &
      5.273235E+00, 4.12613E+01, 1.0586E+01, 6.2030E+02, &
      5.295467E+00, 4.26868E+01, 1.0489E+01, 6.1210E+02, &
      5.351940E+00, 4.43928E+01, 1.0250E+01, 5.9195E+02, &
      5.511128E+00, 4.56654E+01, 9.6197E+00, 5.3987E+02, &
      5.595482E+00, 4.47233E+01, 9.3495E+00, 5.1695E+02, &
      5.617976E+00, 4.30095E+01, 9.2804E+00, 5.1113E+02, &
      5.651718E+00, 4.50562E+01, 1.2800E+01, 6.9891E+02, &
      5.660855E+00, 4.54367E+01, 1.2760E+01, 6.9558E+02, &
      5.736072E+00, 4.70616E+01, 1.2435E+01, 6.6915E+02, &
      5.869024E+00, 4.81910E+01, 1.1901E+01, 6.2612E+02, &
      5.958856E+00, 4.80966E+01, 1.1578E+01, 6.0009E+02, &
      5.982811E+00, 4.73384E+01, 1.1497E+01, 5.9355E+02, &
      6.018744E+00, 4.84624E+01, 1.3172E+01, 6.7530E+02, &
      6.051453E+00, 4.90789E+01, 1.3061E+01, 6.6602E+02, &
      6.108576E+00, 4.97589E+01, 1.2872E+01, 6.5033E+02, &
      6.469004E+00, 5.19659E+01, 1.1807E+01, 5.6370E+02, &
      6.915365E+00, 5.34251E+01, 1.0692E+01, 4.7798E+02, &
      7.392525E+00, 5.43968E+01, 9.6457E+00, 4.0381E+02, &
      7.902609E+00, 5.50660E+01, 8.6788E+00, 3.4029E+02, &
      8.447890E+00, 5.55333E+01, 7.8032E+00, 2.8659E+02, &
      9.030794E+00, 5.58880E+01, 7.0067E+00, 2.4109E+02, &
      9.653919E+00, 5.61120E+01, 6.2554E+00, 2.0169E+02, &
      1.032004E+01, 5.62453E+01, 5.5897E+00, 1.6891E+02, &
      1.103212E+01, 5.63911E+01, 4.9844E+00, 1.4119E+02, &
      1.179334E+01, 5.64523E+01, 4.4369E+00, 1.1785E+02, &
      1.260708E+01, 5.64722E+01, 3.9400E+00, 9.8157E+01, &
      1.347697E+01, 5.64235E+01, 3.4989E+00, 8.1779E+01, &
      1.440688E+01, 5.63541E+01, 3.1098E+00, 6.8210E+01, &
      1.540095E+01, 5.62713E+01, 2.7659E+00, 5.6948E+01, &
      1.646362E+01, 5.61795E+01, 2.4620E+00, 4.7598E+01, &
      1.759961E+01, 5.60818E+01, 2.1935E+00, 3.9831E+01, &
      1.881398E+01, 5.59801E+01, 1.9560E+00, 3.3374E+01, &
      2.011215E+01, 5.58754E+01, 1.7459E+00, 2.7997E+01, &
      2.149988E+01, 5.57679E+01, 1.5597E+00, 2.3517E+01, &
      2.298338E+01, 5.56571E+01, 1.3938E+00, 1.9768E+01, &
      2.456923E+01, 5.55396E+01, 1.2429E+00, 1.6592E+01, &
      2.626450E+01, 5.54124E+01, 1.1072E+00, 1.3920E+01, &
      2.807676E+01, 5.52621E+01, 9.7560E-01, 1.1568E+01, &
      3.001405E+01, 5.50700E+01, 8.5752E-01, 9.6004E+00, &
      3.208502E+01, 5.48096E+01, 7.5436E-01, 7.9810E+00, &
      3.429889E+01, 5.43908E+01, 6.6418E-01, 6.6467E+00, &
      3.666551E+01, 5.32415E+01, 5.8526E-01, 5.5459E+00, &
      3.669179E+01, 5.32120E+01, 5.8447E-01, 5.5352E+00, &
      3.725340E+01, 5.19948E+01, 5.6795E-01, 5.3131E+00, &
      3.740316E+01, 5.05352E+01, 5.6367E-01, 5.2561E+00, &
      3.762780E+01, 5.20120E+01, 3.4315E+00, 2.8578E+01, &
      3.818941E+01, 5.33471E+01, 3.3512E+00, 2.7511E+01, &
      3.919543E+01, 5.41930E+01, 3.2150E+00, 2.5734E+01, &
      4.189992E+01, 5.51167E+01, 2.8831E+00, 2.1637E+01, &
      4.479101E+01, 5.55719E+01, 2.5829E+00, 1.8178E+01, &
      4.788159E+01, 5.58522E+01, 2.3128E+00, 1.5269E+01, &
      5.118542E+01, 5.60387E+01, 2.0659E+00, 1.2798E+01, &
      5.471721E+01, 5.61622E+01, 1.8434E+00, 1.0721E+01, &
      5.849270E+01, 5.62448E+01, 1.6446E+00, 8.9823E+00, &
      6.252870E+01, 5.62991E+01, 1.4669E+00, 7.5284E+00, &
      6.684318E+01, 5.63333E+01, 1.3083E+00, 6.3125E+00, &
      7.145536E+01, 5.63534E+01, 1.1668E+00, 5.2959E+00, &
      7.638578E+01, 5.63946E+01, 1.0396E+00, 4.4426E+00, &
      8.165640E+01, 5.63972E+01, 9.2298E-01, 3.7174E+00, &
      8.729069E+01, 5.63892E+01, 8.1666E-01, 3.1041E+00, &
      9.331374E+01, 5.63757E+01, 7.2265E-01, 2.5954E+00, &
      9.975239E+01, 5.63589E+01, 6.3952E-01, 2.1734E+00, &
      1.066353E+02, 5.63400E+01, 5.6600E-01, 1.8231E+00, &
      1.139931E+02, 5.63200E+01, 5.0098E-01, 1.5323E+00, &
      1.218587E+02, 5.62996E+01, 4.4346E-01, 1.2908E+00, &
      1.302669E+02, 5.62792E+01, 3.9257E-01, 1.0900E+00, &
      1.392553E+02, 5.62592E+01, 3.4755E-01, 9.2301E-01, &
      1.488640E+02, 5.62398E+01, 3.0771E-01, 7.8409E-01, &
      1.591356E+02, 5.62212E+01, 2.7246E-01, 6.6842E-01, &
      1.701159E+02, 5.62034E+01, 2.4145E-01, 5.7236E-01, &
      1.818539E+02, 5.61868E+01, 2.1406E-01, 4.9229E-01, &
      1.944018E+02, 5.61713E+01, 1.8978E-01, 4.2535E-01, &
      2.078156E+02, 5.61568E+01, 1.6827E-01, 3.6931E-01, &
      2.221548E+02, 5.61433E+01, 1.4921E-01, 3.2233E-01, &
      2.374835E+02, 5.61308E+01, 1.3231E-01, 2.8287E-01, &
      2.538699E+02, 5.61192E+01, 1.1733E-01, 2.4967E-01, &
      2.713869E+02, 5.61085E+01, 1.0406E-01, 2.2166E-01, &
      2.901126E+02, 5.60987E+01, 9.2287E-02, 1.9796E-01, &
      3.101304E+02, 5.60897E+01, 8.1854E-02, 1.7786E-01, &
      3.315294E+02, 5.60814E+01, 7.2604E-02, 1.6075E-01, &
      3.544049E+02, 5.60738E+01, 6.4402E-02, 1.4612E-01, &
      3.788588E+02, 5.60669E+01, 5.7129E-02, 1.3356E-01, &
      4.050001E+02, 5.60606E+01, 5.0681E-02, 1.2272E-01, &
      4.329451E+02, 5.60548E+01, 4.4962E-02, 1.1332E-01]
   case(57)
      frel = -4.5180E-01
      allocate(nist_tab(936))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.87089E+01, 0.0000E+00, 1.5622E-04, &
      1.142761E-02, 1.83468E+01, 0.0000E+00, 2.1337E-04, &
      1.221612E-02, 1.77981E+01, 0.0000E+00, 2.8990E-04, &
      1.305903E-02, 1.68371E+01, 0.0000E+00, 3.9182E-04, &
      1.396010E-02, 1.43197E+01, 0.0000E+00, 5.2682E-04, &
      1.411200E-02, 1.32810E+01, 0.0000E+00, 5.5246E-04, &
      1.432800E-02, 9.70715E+00, 0.0000E+00, 5.9044E-04, &
      1.438560E-02, 5.39845E+00, 0.0000E+00, 6.0088E-04, &
      1.447200E-02, 9.90333E+00, 8.4982E+00, 1.7789E+05, &
      1.468800E-02, 1.38687E+01, 8.3345E+00, 1.7190E+05, &
      1.492335E-02, 1.56489E+01, 8.1623E+00, 1.6569E+05, &
      1.595306E-02, 1.90550E+01, 7.4767E+00, 1.4198E+05, &
      1.705382E-02, 2.08265E+01, 6.7507E+00, 1.1992E+05, &
      1.823053E-02, 2.19681E+01, 5.9723E+00, 9.9244E+04, &
      1.948844E-02, 2.27190E+01, 5.2034E+00, 8.0886E+04, &
      2.083314E-02, 2.31990E+01, 4.4843E+00, 6.5208E+04, &
      2.227063E-02, 2.34860E+01, 3.8371E+00, 5.2195E+04, &
      2.380730E-02, 2.36361E+01, 3.2705E+00, 4.1616E+04, &
      2.545001E-02, 2.36908E+01, 2.7843E+00, 3.3142E+04, &
      2.720606E-02, 2.36802E+01, 2.3731E+00, 2.6424E+04, &
      2.908327E-02, 2.36248E+01, 2.0287E+00, 2.1132E+04, &
      3.109002E-02, 2.35359E+01, 1.7424E+00, 1.6978E+04, &
      3.165400E-02, 2.35047E+01, 1.6740E+00, 1.6021E+04, &
      3.213850E-02, 2.34686E+01, 1.6189E+00, 1.5260E+04, &
      3.226770E-02, 2.34467E+01, 1.6048E+00, 1.5066E+04, &
      3.246150E-02, 2.34528E+01, 1.6149E+00, 1.5071E+04, &
      3.294600E-02, 2.34432E+01, 1.5665E+00, 1.4404E+04, &
      3.323523E-02, 2.34330E+01, 1.5389E+00, 1.4027E+04, &
      3.552846E-02, 2.33376E+01, 1.3514E+00, 1.1523E+04, &
      3.797993E-02, 2.32320E+01, 1.1995E+00, 9.5679E+03, &
      4.060054E-02, 2.31224E+01, 1.0768E+00, 8.0344E+03, &
      4.340198E-02, 2.30200E+01, 9.7787E-01, 6.8254E+03, &
      4.639671E-02, 2.29042E+01, 8.9840E-01, 5.8660E+03, &
      4.959809E-02, 2.27831E+01, 8.3474E-01, 5.0986E+03, &
      5.302035E-02, 2.26534E+01, 7.8385E-01, 4.4788E+03, &
      5.667876E-02, 2.25101E+01, 7.4326E-01, 3.9727E+03, &
      6.058959E-02, 2.23459E+01, 7.1114E-01, 3.5557E+03, &
      6.477028E-02, 2.21501E+01, 6.8620E-01, 3.2096E+03, &
      6.923942E-02, 2.19053E+01, 6.6685E-01, 2.9178E+03, &
      7.401695E-02, 2.15806E+01, 6.5186E-01, 2.6682E+03, &
      7.912411E-02, 2.11153E+01, 6.4020E-01, 2.4513E+03, &
      8.458368E-02, 2.03664E+01, 6.3111E-01, 2.2606E+03, &
      9.041995E-02, 1.88770E+01, 6.2394E-01, 2.0907E+03, &
      9.665893E-02, 1.33358E+01, 6.1820E-01, 1.9378E+03, &
      9.692200E-02, 1.26877E+01, 6.1799E-01, 1.9319E+03, &
      9.840550E-02, 4.27129E+00, 6.1685E-01, 1.8993E+03, &
      9.880110E-02, -7.36267E+00, 6.1656E-01, 1.8909E+03, &
      9.939450E-02, 7.57660E+00, 2.4131E+01, 7.3550E+04, &
      1.008780E-01, 2.01159E+01, 1.9837E+01, 5.9572E+04, &
      1.033284E-01, 2.61204E+01, 1.4480E+01, 4.2453E+04, &
      1.104581E-01, 2.83400E+01, 6.2404E+00, 1.7115E+04, &
      1.180797E-01, 2.70378E+01, 3.3591E+00, 8.6186E+03, &
      1.262272E-01, 2.59494E+01, 2.2914E+00, 5.4999E+03, &
      1.349368E-01, 2.51633E+01, 1.8404E+00, 4.1325E+03, &
      1.442475E-01, 2.45597E+01, 1.6467E+00, 3.4590E+03, &
      1.542005E-01, 2.40505E+01, 1.5839E+00, 3.1127E+03, &
      1.648404E-01, 2.35728E+01, 1.6033E+00, 2.9475E+03, &
      1.762144E-01, 2.30570E+01, 1.6852E+00, 2.8982E+03, &
      1.875720E-01, 2.23145E+01, 1.8119E+00, 2.9276E+03, &
      1.883732E-01, 2.22201E+01, 1.8222E+00, 2.9318E+03, &
      1.904430E-01, 2.18103E+01, 1.8498E+00, 2.9437E+03, &
      1.912086E-01, 2.13010E+01, 1.8602E+00, 2.9485E+03, &
      1.923570E-01, 2.17474E+01, 2.8253E+00, 4.4508E+03, &
      1.952280E-01, 2.20741E+01, 2.8775E+00, 4.4664E+03, &
      2.013709E-01, 2.21454E+01, 2.9952E+00, 4.5074E+03, &
      2.016840E-01, 2.21410E+01, 3.0014E+00, 4.5098E+03, &
      2.047710E-01, 2.20171E+01, 3.0637E+00, 4.5340E+03, &
      2.055942E-01, 2.18525E+01, 3.0806E+00, 4.5408E+03, &
      2.068290E-01, 2.20210E+01, 3.4333E+00, 5.0303E+03, &
      2.099160E-01, 2.21656E+01, 3.5027E+00, 5.0564E+03, &
      2.152655E-01, 2.22440E+01, 3.6262E+00, 5.1048E+03, &
      2.301188E-01, 2.22964E+01, 3.9852E+00, 5.2481E+03, &
      2.459970E-01, 2.22928E+01, 4.3834E+00, 5.4001E+03, &
      2.629708E-01, 2.22043E+01, 4.8098E+00, 5.5431E+03, &
      2.649920E-01, 2.21670E+01, 4.8600E+00, 5.5582E+03, &
      2.690480E-01, 2.19707E+01, 4.9598E+00, 5.5868E+03, &
      2.701296E-01, 2.17154E+01, 4.9863E+00, 5.5943E+03, &
      2.717520E-01, 2.19911E+01, 5.5410E+00, 6.1792E+03, &
      2.758080E-01, 2.22524E+01, 5.6421E+00, 6.1995E+03, &
      2.811158E-01, 2.24140E+01, 5.7724E+00, 6.2230E+03, &
      3.005128E-01, 2.27965E+01, 6.2234E+00, 6.2763E+03, &
      3.212482E-01, 2.31627E+01, 6.6495E+00, 6.2733E+03, &
      3.434143E-01, 2.35533E+01, 7.0310E+00, 6.2055E+03, &
      3.671099E-01, 2.39653E+01, 7.3528E+00, 6.0709E+03, &
      3.924405E-01, 2.43836E+01, 7.6033E+00, 5.8729E+03, &
      4.195189E-01, 2.47866E+01, 7.7705E+00, 5.6151E+03, &
      4.484657E-01, 2.51400E+01, 7.8465E+00, 5.3044E+03, &
      4.794098E-01, 2.54068E+01, 7.8382E+00, 4.9574E+03, &
      5.124891E-01, 2.55490E+01, 7.7563E+00, 4.5896E+03, &
      5.478508E-01, 2.55227E+01, 7.6133E+00, 4.2148E+03, &
      5.856525E-01, 2.52763E+01, 7.4219E+00, 3.8443E+03, &
      6.260625E-01, 2.47077E+01, 7.1940E+00, 3.4866E+03, &
      6.692609E-01, 2.36639E+01, 6.9402E+00, 3.1472E+03, &
      7.154399E-01, 2.18056E+01, 6.6701E+00, 2.8303E+03, &
      7.648052E-01, 1.81710E+01, 6.3909E+00, 2.5377E+03, &
      8.150660E-01, 7.99874E+00, 6.1211E+00, 2.2816E+03, &
      8.175768E-01, 6.80890E+00, 6.1080E+00, 2.2697E+03, &
      8.275415E-01, -1.69126E+00, 6.0566E+00, 2.2237E+03, &
      8.315300E-01, -2.19139E+01, 6.0363E+00, 2.2056E+03, &
      8.358585E-01, -3.05614E+00, 2.5206E+01, 9.1418E+03, &
      8.442575E-01, -1.29034E-01, 2.4855E+01, 8.9252E+03, &
      8.476515E-01, -4.97044E+00, 2.4715E+01, 8.8397E+03, &
      8.483340E-01, -1.12965E+01, 2.4688E+01, 8.8226E+03, &
      8.527425E-01, 3.63331E+00, 3.7187E+01, 1.3218E+04, &
      8.654700E-01, 1.27805E+01, 3.6387E+01, 1.2744E+04, &
      8.739896E-01, 1.61223E+01, 3.5868E+01, 1.2440E+04, &
      9.342948E-01, 2.75600E+01, 3.2536E+01, 1.0557E+04, &
      9.987612E-01, 3.28280E+01, 2.9646E+01, 8.9994E+03, &
      1.067676E+00, 3.55387E+01, 2.6971E+01, 7.6599E+03, &
      1.100932E+00, 3.56384E+01, 2.5830E+01, 7.1149E+03, &
      1.117783E+00, 3.43800E+01, 2.5284E+01, 6.8598E+03, &
      1.122277E+00, 3.23952E+01, 2.5142E+01, 6.7940E+03, &
      1.129017E+00, 3.47078E+01, 2.9075E+01, 7.8089E+03, &
      1.141345E+00, 3.65972E+01, 2.8607E+01, 7.6004E+03, &
      1.145868E+00, 3.70034E+01, 2.8438E+01, 7.5259E+03, &
      1.180312E+00, 3.86875E+01, 2.7215E+01, 6.9925E+03, &
      1.198378E+00, 3.86787E+01, 2.6665E+01, 6.7482E+03, &
      1.203196E+00, 3.79519E+01, 2.6523E+01, 6.6855E+03, &
      1.210422E+00, 3.91322E+01, 2.8098E+01, 7.0398E+03, &
      1.220098E+00, 4.00238E+01, 2.7793E+01, 6.9083E+03, &
      1.228488E+00, 4.05402E+01, 2.7535E+01, 6.7975E+03, &
      1.304285E+00, 4.30692E+01, 2.5510E+01, 5.9328E+03, &
      1.334074E+00, 4.35846E+01, 2.4842E+01, 5.6487E+03, &
      1.354493E+00, 4.35695E+01, 2.4412E+01, 5.4676E+03, &
      1.359939E+00, 4.31080E+01, 2.4301E+01, 5.4210E+03, &
      1.368106E+00, 4.38732E+01, 2.5282E+01, 5.6060E+03, &
      1.388526E+00, 4.48271E+01, 2.4877E+01, 5.4353E+03, &
      1.394281E+00, 4.50197E+01, 2.4766E+01, 5.3888E+03, &
      1.490486E+00, 4.72860E+01, 2.2845E+01, 4.6509E+03, &
      1.593329E+00, 4.88265E+01, 2.0959E+01, 3.9927E+03, &
      1.703269E+00, 4.99607E+01, 1.9183E+01, 3.4196E+03, &
      1.820795E+00, 5.08126E+01, 1.7542E+01, 2.9264E+03, &
      1.946430E+00, 5.14440E+01, 1.6009E+01, 2.4994E+03, &
      2.080733E+00, 5.18973E+01, 1.4606E+01, 2.1343E+03, &
      2.224304E+00, 5.22206E+01, 1.3340E+01, 1.8245E+03, &
      2.377781E+00, 5.25442E+01, 1.2171E+01, 1.5581E+03, &
      2.541848E+00, 5.27346E+01, 1.1099E+01, 1.3302E+03, &
      2.717235E+00, 5.28027E+01, 1.0131E+01, 1.1368E+03, &
      2.904724E+00, 5.28464E+01, 9.2543E+00, 9.7229E+02, &
      3.105150E+00, 5.28835E+01, 8.3917E+00, 8.2567E+02, &
      3.319406E+00, 5.27902E+01, 7.5590E+00, 6.9665E+02, &
      3.548445E+00, 5.25879E+01, 6.7776E+00, 5.8522E+02, &
      3.793288E+00, 5.22804E+01, 6.0781E+00, 4.9181E+02, &
      4.055024E+00, 5.18694E+01, 5.4587E+00, 4.1399E+02, &
      4.334821E+00, 5.13283E+01, 4.9111E+00, 3.4918E+02, &
      4.633924E+00, 5.05850E+01, 4.4267E+00, 2.9512E+02, &
      4.953664E+00, 4.94384E+01, 3.9916E+00, 2.4961E+02, &
      5.295467E+00, 4.69684E+01, 3.5875E+00, 2.1049E+02, &
      5.373046E+00, 4.57290E+01, 3.5036E+00, 2.0275E+02, &
      5.455286E+00, 4.25859E+01, 3.4185E+00, 1.9499E+02, &
      5.477217E+00, 3.89711E+01, 3.3963E+00, 1.9299E+02, &
      5.510113E+00, 4.25027E+01, 1.0377E+01, 5.7563E+02, &
      5.592354E+00, 4.55088E+01, 1.0105E+01, 5.5246E+02, &
      5.660855E+00, 4.63900E+01, 9.8871E+00, 5.3414E+02, &
      5.772788E+00, 4.68156E+01, 9.5487E+00, 5.0605E+02, &
      5.861147E+00, 4.58940E+01, 9.3016E+00, 4.8567E+02, &
      5.884710E+00, 4.42328E+01, 9.2377E+00, 4.8044E+02, &
      5.920053E+00, 4.61993E+01, 1.2650E+01, 6.5219E+02, &
      6.008412E+00, 4.81320E+01, 1.2328E+01, 6.2639E+02, &
      6.051453E+00, 4.86016E+01, 1.2176E+01, 6.1434E+02, &
      6.140974E+00, 4.91977E+01, 1.1871E+01, 5.9033E+02, &
      6.234969E+00, 4.91273E+01, 1.1566E+01, 5.6665E+02, &
      6.260034E+00, 4.83790E+01, 1.1488E+01, 5.6060E+02, &
      6.297632E+00, 4.95027E+01, 1.3155E+01, 6.3747E+02, &
      6.391626E+00, 5.08069E+01, 1.2861E+01, 6.1419E+02, &
      6.469004E+00, 5.14451E+01, 1.2629E+01, 5.9595E+02, &
      6.915365E+00, 5.35912E+01, 1.1409E+01, 5.0414E+02, &
      7.392525E+00, 5.48511E+01, 1.0292E+01, 4.2585E+02, &
      7.902609E+00, 5.56931E+01, 9.2671E+00, 3.5911E+02, &
      8.447890E+00, 5.62810E+01, 8.3384E+00, 3.0266E+02, &
      9.030794E+00, 5.67219E+01, 7.4904E+00, 2.5470E+02, &
      9.653919E+00, 5.69970E+01, 6.6873E+00, 2.1307E+02, &
      1.032004E+01, 5.71703E+01, 5.9756E+00, 1.7842E+02, &
      1.103212E+01, 5.73491E+01, 5.3423E+00, 1.4952E+02, &
      1.179334E+01, 5.74399E+01, 4.7625E+00, 1.2497E+02, &
      1.260708E+01, 5.74834E+01, 4.2378E+00, 1.0428E+02, &
      1.347697E+01, 5.74549E+01, 3.7621E+00, 8.6848E+01, &
      1.440688E+01, 5.73980E+01, 3.3427E+00, 7.2407E+01, &
      1.540095E+01, 5.73236E+01, 2.9725E+00, 6.0434E+01, &
      1.646362E+01, 5.72376E+01, 2.6451E+00, 5.0491E+01, &
      1.759961E+01, 5.71439E+01, 2.3557E+00, 4.2232E+01, &
      1.881398E+01, 5.70451E+01, 2.0999E+00, 3.5367E+01, &
      2.011215E+01, 5.69428E+01, 1.8735E+00, 2.9655E+01, &
      2.149988E+01, 5.68376E+01, 1.6731E+00, 2.4897E+01, &
      2.298338E+01, 5.67295E+01, 1.4955E+00, 2.0929E+01, &
      2.456923E+01, 5.66175E+01, 1.3359E+00, 1.7592E+01, &
      2.626450E+01, 5.64980E+01, 1.1917E+00, 1.4774E+01, &
      2.807676E+01, 5.63624E+01, 1.0525E+00, 1.2302E+01, &
      3.001405E+01, 5.61951E+01, 9.2569E-01, 1.0212E+01, &
      3.208502E+01, 5.59805E+01, 8.1452E-01, 8.4883E+00, &
      3.429889E+01, 5.56735E+01, 7.1730E-01, 7.0678E+00, &
      3.666551E+01, 5.51093E+01, 6.3221E-01, 5.8959E+00, &
      3.814611E+01, 5.42292E+01, 5.8681E-01, 5.2995E+00, &
      3.872998E+01, 5.30188E+01, 5.7031E-01, 5.0879E+00, &
      3.888567E+01, 5.15674E+01, 5.6603E-01, 5.0335E+00, &
      3.911922E+01, 5.30359E+01, 3.4178E+00, 2.7086E+01, &
      3.919543E+01, 5.33483E+01, 3.4071E+00, 2.6952E+01, &
      3.970309E+01, 5.43636E+01, 3.3377E+00, 2.6074E+01, &
      4.189992E+01, 5.56912E+01, 3.0600E+00, 2.2691E+01, &
      4.479101E+01, 5.63415E+01, 2.7431E+00, 1.9075E+01, &
      4.788159E+01, 5.67055E+01, 2.4574E+00, 1.6028E+01, &
      5.118542E+01, 5.69403E+01, 2.1998E+00, 1.3461E+01, &
      5.471721E+01, 5.70962E+01, 1.9636E+00, 1.1279E+01, &
      5.849270E+01, 5.72003E+01, 1.7523E+00, 9.4516E+00, &
      6.252870E+01, 5.72696E+01, 1.5634E+00, 7.9226E+00, &
      6.684318E+01, 5.73146E+01, 1.3947E+00, 6.6433E+00, &
      7.145536E+01, 5.73424E+01, 1.2440E+00, 5.5736E+00, &
      7.638578E+01, 5.73584E+01, 1.1096E+00, 4.6790E+00, &
      8.165640E+01, 5.73969E+01, 9.8626E-01, 3.9186E+00, &
      8.729069E+01, 5.73925E+01, 8.7300E-01, 3.2724E+00, &
      9.331374E+01, 5.73815E+01, 7.7283E-01, 2.7362E+00, &
      9.975239E+01, 5.73663E+01, 6.8421E-01, 2.2912E+00, &
      1.066353E+02, 5.73485E+01, 6.0580E-01, 1.9217E+00, &
      1.139931E+02, 5.73291E+01, 5.3642E-01, 1.6149E+00, &
      1.218587E+02, 5.73090E+01, 4.7503E-01, 1.3599E+00, &
      1.302669E+02, 5.72887E+01, 4.2070E-01, 1.1480E+00, &
      1.392553E+02, 5.72686E+01, 3.7261E-01, 9.7170E-01, &
      1.488640E+02, 5.72490E+01, 3.3004E-01, 8.2498E-01, &
      1.591356E+02, 5.72300E+01, 2.9236E-01, 7.0279E-01, &
      1.701159E+02, 5.72119E+01, 2.5904E-01, 6.0102E-01, &
      1.818539E+02, 5.71948E+01, 2.2981E-01, 5.1656E-01, &
      1.944018E+02, 5.71787E+01, 2.0389E-01, 4.4592E-01, &
      2.078156E+02, 5.71638E+01, 1.8090E-01, 3.8678E-01, &
      2.221548E+02, 5.71498E+01, 1.6052E-01, 3.3717E-01, &
      2.374835E+02, 5.71369E+01, 1.4244E-01, 2.9551E-01, &
      2.538699E+02, 5.71249E+01, 1.2641E-01, 2.6045E-01, &
      2.713869E+02, 5.71138E+01, 1.1218E-01, 2.3087E-01, &
      2.901126E+02, 5.71036E+01, 9.9565E-02, 2.0585E-01, &
      3.101304E+02, 5.70942E+01, 8.8371E-02, 1.8462E-01, &
      3.315294E+02, 5.70856E+01, 7.8440E-02, 1.6656E-01, &
      3.544049E+02, 5.70777E+01, 6.9628E-02, 1.5112E-01, &
      3.788588E+02, 5.70705E+01, 6.1809E-02, 1.3788E-01, &
      4.050001E+02, 5.70639E+01, 5.4871E-02, 1.2645E-01, &
      4.329451E+02, 5.70578E+01, 4.8714E-02, 1.1655E-01]
   case(58)
      frel = -4.7160E-01
      allocate(nist_tab(956))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.85477E+01, 0.0000E+00, 1.5991E-04, &
      1.142761E-02, 1.84321E+01, 0.0000E+00, 2.1848E-04, &
      1.221612E-02, 1.82878E+01, 0.0000E+00, 2.9695E-04, &
      1.305903E-02, 1.81041E+01, 0.0000E+00, 4.0149E-04, &
      1.396010E-02, 1.78640E+01, 0.0000E+00, 5.4001E-04, &
      1.492335E-02, 1.75380E+01, 0.0000E+00, 7.2256E-04, &
      1.595306E-02, 1.70701E+01, 0.0000E+00, 9.6187E-04, &
      1.705382E-02, 1.63339E+01, 0.0000E+00, 1.2739E-03, &
      1.823053E-02, 1.49461E+01, 0.0000E+00, 1.6785E-03, &
      1.940400E-02, 1.08521E+01, 0.0000E+00, 2.1624E-03, &
      1.948844E-02, 1.00681E+01, 0.0000E+00, 2.2005E-03, &
      1.970100E-02, 6.14720E+00, 0.0000E+00, 2.2985E-03, &
      1.978020E-02, 3.79223E-01, 0.0000E+00, 2.3357E-03, &
      1.989900E-02, 6.55466E+00, 1.1424E+01, 1.7242E+05, &
      2.019600E-02, 1.20134E+01, 1.1037E+01, 1.6413E+05, &
      2.083314E-02, 1.59795E+01, 1.0269E+01, 1.4803E+05, &
      2.227063E-02, 1.96057E+01, 8.7851E+00, 1.1847E+05, &
      2.380730E-02, 2.14408E+01, 7.3863E+00, 9.3174E+04, &
      2.545001E-02, 2.24561E+01, 6.1056E+00, 7.2048E+04, &
      2.720606E-02, 2.29871E+01, 4.9917E+00, 5.5102E+04, &
      2.908327E-02, 2.32158E+01, 4.0571E+00, 4.1894E+04, &
      3.109002E-02, 2.32563E+01, 3.2921E+00, 3.1801E+04, &
      3.323523E-02, 2.31818E+01, 2.6767E+00, 2.4187E+04, &
      3.552846E-02, 2.30382E+01, 2.1871E+00, 1.8487E+04, &
      3.704400E-02, 2.29227E+01, 1.9341E+00, 1.5680E+04, &
      3.761100E-02, 2.28708E+01, 1.8511E+00, 1.4781E+04, &
      3.776220E-02, 2.28481E+01, 1.8299E+00, 1.4553E+04, &
      3.797993E-02, 2.28421E+01, 1.8223E+00, 1.4409E+04, &
      3.798900E-02, 2.28418E+01, 1.8211E+00, 1.4396E+04, &
      3.855600E-02, 2.28084E+01, 1.7485E+00, 1.3619E+04, &
      4.060054E-02, 2.26640E+01, 1.5247E+00, 1.1278E+04, &
      4.340198E-02, 2.24654E+01, 1.2941E+00, 8.9543E+03, &
      4.639671E-02, 2.22623E+01, 1.1157E+00, 7.2215E+03, &
      4.959809E-02, 2.20558E+01, 9.7795E-01, 5.9216E+03, &
      5.302035E-02, 2.18440E+01, 8.7190E-01, 4.9387E+03, &
      5.667876E-02, 2.16221E+01, 7.9042E-01, 4.1882E+03, &
      6.058959E-02, 2.13955E+01, 7.2792E-01, 3.6081E+03, &
      6.477028E-02, 2.11284E+01, 6.8006E-01, 3.1533E+03, &
      6.923942E-02, 2.08174E+01, 6.4347E-01, 2.7912E+03, &
      7.401695E-02, 2.04375E+01, 6.1552E-01, 2.4976E+03, &
      7.912411E-02, 1.99452E+01, 5.9418E-01, 2.2554E+03, &
      8.418200E-02, 1.93008E+01, 5.7914E-01, 2.0663E+03, &
      8.458368E-02, 1.92365E+01, 5.7815E-01, 2.0531E+03, &
      8.547050E-02, 1.90720E+01, 5.7606E-01, 2.0244E+03, &
      8.581410E-02, 1.89691E+01, 5.7528E-01, 2.0136E+03, &
      8.632950E-02, 1.89340E+01, 6.7415E-01, 2.3455E+03, &
      8.761800E-02, 1.87633E+01, 6.8664E-01, 2.3538E+03, &
      9.041995E-02, 1.82758E+01, 7.2185E-01, 2.3978E+03, &
      9.665893E-02, 1.66918E+01, 8.4947E-01, 2.6396E+03, &
      1.033284E-01, 1.31370E+01, 1.0495E+00, 3.0506E+03, &
      1.078000E-01, 5.12263E+00, 1.2092E+00, 3.3691E+03, &
      1.094500E-01, -8.64721E+00, 1.2718E+00, 3.4900E+03, &
      1.098900E-01, -2.79860E+01, 1.2887E+00, 3.5223E+03, &
      1.104581E-01, -5.31339E+00, 4.0959E+01, 1.1136E+05, &
      1.105500E-01, -2.53344E+00, 4.0450E+01, 1.0988E+05, &
      1.122000E-01, 1.84513E+01, 3.2422E+01, 8.6781E+04, &
      1.180797E-01, 3.05692E+01, 1.5580E+01, 3.9627E+04, &
      1.262272E-01, 2.91226E+01, 7.1616E+00, 1.7040E+04, &
      1.349368E-01, 2.69869E+01, 4.7904E+00, 1.0663E+04, &
      1.442475E-01, 2.57041E+01, 4.0662E+00, 8.4665E+03, &
      1.542005E-01, 2.49525E+01, 3.8505E+00, 7.5000E+03, &
      1.648404E-01, 2.44842E+01, 3.8044E+00, 6.9322E+03, &
      1.762144E-01, 2.41452E+01, 3.8146E+00, 6.5023E+03, &
      1.883732E-01, 2.38195E+01, 3.8450E+00, 6.1312E+03, &
      2.013709E-01, 2.32986E+01, 3.8897E+00, 5.8024E+03, &
      2.030560E-01, 2.31721E+01, 3.8965E+00, 5.7644E+03, &
      2.061640E-01, 2.27074E+01, 3.9099E+00, 5.6971E+03, &
      2.069928E-01, 2.22092E+01, 3.9136E+00, 5.6797E+03, &
      2.082360E-01, 2.26691E+01, 4.8664E+00, 7.0199E+03, &
      2.113440E-01, 2.30311E+01, 4.8930E+00, 6.9545E+03, &
      2.152655E-01, 2.31562E+01, 4.9281E+00, 6.8769E+03, &
      2.188340E-01, 2.31771E+01, 4.9616E+00, 6.8107E+03, &
      2.221835E-01, 2.30847E+01, 4.9944E+00, 6.7526E+03, &
      2.230767E-01, 2.29305E+01, 5.0034E+00, 6.7376E+03, &
      2.244165E-01, 2.31075E+01, 5.3388E+00, 7.1462E+03, &
      2.277660E-01, 2.32773E+01, 5.3784E+00, 7.0934E+03, &
      2.301188E-01, 2.33369E+01, 5.4070E+00, 7.0583E+03, &
      2.459970E-01, 2.35260E+01, 5.6147E+00, 6.8565E+03, &
      2.629708E-01, 2.36066E+01, 5.8612E+00, 6.6958E+03, &
      2.811158E-01, 2.35791E+01, 6.1427E+00, 6.5647E+03, &
      2.838080E-01, 2.35410E+01, 6.1853E+00, 6.5475E+03, &
      2.881520E-01, 2.33542E+01, 6.2541E+00, 6.5207E+03, &
      2.893104E-01, 2.31025E+01, 6.2724E+00, 6.5137E+03, &
      2.910480E-01, 2.33794E+01, 6.8120E+00, 7.0316E+03, &
      2.953920E-01, 2.36458E+01, 6.8843E+00, 7.0016E+03, &
      3.005128E-01, 2.38000E+01, 6.9690E+00, 6.9671E+03, &
      3.212482E-01, 2.42058E+01, 7.3018E+00, 6.8291E+03, &
      3.434143E-01, 2.45712E+01, 7.6240E+00, 6.6704E+03, &
      3.671099E-01, 2.49630E+01, 7.9138E+00, 6.4774E+03, &
      3.924405E-01, 2.53727E+01, 8.1521E+00, 6.2421E+03, &
      4.195189E-01, 2.57854E+01, 8.3262E+00, 5.9644E+03, &
      4.484657E-01, 2.61800E+01, 8.4268E+00, 5.6474E+03, &
      4.794098E-01, 2.65263E+01, 8.4410E+00, 5.2923E+03, &
      5.124891E-01, 2.67830E+01, 8.3709E+00, 4.9102E+03, &
      5.478508E-01, 2.69084E+01, 8.2281E+00, 4.5155E+03, &
      5.856525E-01, 2.68554E+01, 8.0259E+00, 4.1210E+03, &
      6.260625E-01, 2.65618E+01, 7.7785E+00, 3.7370E+03, &
      6.692609E-01, 2.59329E+01, 7.4990E+00, 3.3710E+03, &
      7.154399E-01, 2.47956E+01, 7.1990E+00, 3.0281E+03, &
      7.648052E-01, 2.27724E+01, 6.8875E+00, 2.7110E+03, &
      8.175768E-01, 1.87435E+01, 6.5726E+00, 2.4210E+03, &
      8.656340E-01, 8.85327E+00, 6.3043E+00, 2.1941E+03, &
      8.739896E-01, 4.11953E+00, 6.2595E+00, 2.1578E+03, &
      8.788835E-01, -1.15258E+00, 6.2335E+00, 2.1370E+03, &
      8.832740E-01, -3.45756E+01, 6.2104E+00, 2.1185E+03, &
      8.877165E-01, -2.48566E+00, 2.6121E+01, 8.8437E+03, &
      8.967935E-01, 6.34175E-01, 2.5699E+01, 8.6130E+03, &
      9.003987E-01, -4.41287E+00, 2.5534E+01, 8.5237E+03, &
      9.009660E-01, -8.36830E+00, 2.5509E+01, 8.5097E+03, &
      9.058065E-01, 4.57327E+00, 3.8494E+01, 1.2770E+04, &
      9.193260E-01, 1.41198E+01, 3.7541E+01, 1.2270E+04, &
      9.342948E-01, 1.94031E+01, 3.6529E+01, 1.1749E+04, &
      9.987612E-01, 2.99944E+01, 3.2667E+01, 9.8300E+03, &
      1.067676E+00, 3.48425E+01, 2.9443E+01, 8.2893E+03, &
      1.141345E+00, 3.69803E+01, 2.6661E+01, 7.0228E+03, &
      1.161692E+00, 3.68591E+01, 2.5984E+01, 6.7250E+03, &
      1.179473E+00, 3.55587E+01, 2.5419E+01, 6.4798E+03, &
      1.184215E+00, 3.35310E+01, 2.5272E+01, 6.4167E+03, &
      1.191327E+00, 3.58829E+01, 2.9279E+01, 7.3886E+03, &
      1.209108E+00, 3.82100E+01, 2.8619E+01, 7.1163E+03, &
      1.220098E+00, 3.89332E+01, 2.8225E+01, 6.9552E+03, &
      1.247344E+00, 3.99517E+01, 2.7301E+01, 6.5810E+03, &
      1.266436E+00, 3.99120E+01, 2.6742E+01, 6.3493E+03, &
      1.271527E+00, 3.91626E+01, 2.6598E+01, 6.2900E+03, &
      1.279164E+00, 4.03586E+01, 2.8207E+01, 6.6301E+03, &
      1.298256E+00, 4.17722E+01, 2.7629E+01, 6.3992E+03, &
      1.304285E+00, 4.20651E+01, 2.7453E+01, 6.3291E+03, &
      1.394281E+00, 4.45314E+01, 2.5227E+01, 5.4417E+03, &
      1.405908E+00, 4.46848E+01, 2.4983E+01, 5.3446E+03, &
      1.427427E+00, 4.46619E+01, 2.4549E+01, 5.1728E+03, &
      1.433165E+00, 4.41922E+01, 2.4437E+01, 5.1287E+03, &
      1.441773E+00, 4.49648E+01, 2.5432E+01, 5.3053E+03, &
      1.463292E+00, 4.59230E+01, 2.5020E+01, 5.1430E+03, &
      1.490486E+00, 4.67032E+01, 2.4525E+01, 4.9496E+03, &
      1.593329E+00, 4.87721E+01, 2.2579E+01, 4.2639E+03, &
      1.703269E+00, 5.02271E+01, 2.0699E+01, 3.6577E+03, &
      1.820795E+00, 5.12984E+01, 1.8933E+01, 3.1309E+03, &
      1.946430E+00, 5.21013E+01, 1.7300E+01, 2.6773E+03, &
      2.080733E+00, 5.26929E+01, 1.5796E+01, 2.2879E+03, &
      2.224304E+00, 5.31185E+01, 1.4404E+01, 1.9526E+03, &
      2.377781E+00, 5.35173E+01, 1.3148E+01, 1.6684E+03, &
      2.541848E+00, 5.37256E+01, 1.1988E+01, 1.4241E+03, &
      2.717235E+00, 5.38825E+01, 1.0931E+01, 1.2157E+03, &
      2.904724E+00, 5.39726E+01, 9.9786E+00, 1.0391E+03, &
      3.105150E+00, 5.41316E+01, 9.0399E+00, 8.8155E+02, &
      3.319406E+00, 5.40412E+01, 8.1349E+00, 7.4305E+02, &
      3.548445E+00, 5.38652E+01, 7.3286E+00, 6.2710E+02, &
      3.793288E+00, 5.36071E+01, 6.5717E+00, 5.2694E+02, &
      4.055024E+00, 5.32539E+01, 5.8978E+00, 4.4322E+02, &
      4.334821E+00, 5.27952E+01, 5.3002E+00, 3.7340E+02, &
      4.633924E+00, 5.21906E+01, 4.7711E+00, 3.1517E+02, &
      4.953664E+00, 5.13382E+01, 4.3029E+00, 2.6658E+02, &
      5.295467E+00, 4.99245E+01, 3.8778E+00, 2.2540E+02, &
      5.608932E+00, 4.68297E+01, 3.5375E+00, 1.9468E+02, &
      5.660855E+00, 4.54426E+01, 3.4850E+00, 1.9011E+02, &
      5.694783E+00, 4.36597E+01, 3.4514E+00, 1.8722E+02, &
      5.717677E+00, 3.99880E+01, 3.4291E+00, 1.8531E+02, &
      5.752017E+00, 4.36201E+01, 1.0533E+01, 5.5513E+02, &
      5.837868E+00, 4.67211E+01, 1.0205E+01, 5.3008E+02, &
      6.040916E+00, 4.81015E+01, 9.5007E+00, 4.7730E+02, &
      6.051453E+00, 4.80772E+01, 9.4699E+00, 4.7495E+02, &
      6.133379E+00, 4.71455E+01, 9.2384E+00, 4.5729E+02, &
      6.158036E+00, 4.54890E+01, 9.1714E+00, 4.5219E+02, &
      6.195021E+00, 4.74235E+01, 1.2550E+01, 6.1330E+02, &
      6.287484E+00, 4.93080E+01, 1.2227E+01, 5.8885E+02, &
      6.417824E+00, 5.02841E+01, 1.1799E+01, 5.5689E+02, &
      6.469004E+00, 5.03955E+01, 1.1639E+01, 5.4508E+02, &
      6.516056E+00, 5.02108E+01, 1.1497E+01, 5.3462E+02, &
      6.542251E+00, 4.94645E+01, 1.1420E+01, 5.2892E+02, &
      6.581544E+00, 5.05806E+01, 1.3081E+01, 6.0155E+02, &
      6.679776E+00, 5.18733E+01, 1.2792E+01, 5.7975E+02, &
      6.915365E+00, 5.33992E+01, 1.2149E+01, 5.3210E+02, &
      7.392525E+00, 5.51261E+01, 1.0991E+01, 4.5077E+02, &
      7.902609E+00, 5.62210E+01, 9.9078E+00, 3.8054E+02, &
      8.447890E+00, 5.69654E+01, 8.9174E+00, 3.2080E+02, &
      9.030794E+00, 5.75029E+01, 8.0097E+00, 2.6993E+02, &
      9.653919E+00, 5.78541E+01, 7.1500E+00, 2.2577E+02, &
      1.032004E+01, 5.80788E+01, 6.3882E+00, 1.8904E+02, &
      1.103212E+01, 5.82206E+01, 5.7128E+00, 1.5844E+02, &
      1.179334E+01, 5.83774E+01, 5.1034E+00, 1.3270E+02, &
      1.260708E+01, 5.84490E+01, 4.5487E+00, 1.1091E+02, &
      1.347697E+01, 5.84781E+01, 4.0450E+00, 9.2512E+01, &
      1.440688E+01, 5.84372E+01, 3.5928E+00, 7.7099E+01, &
      1.540095E+01, 5.83727E+01, 3.1939E+00, 6.4327E+01, &
      1.646362E+01, 5.82934E+01, 2.8418E+00, 5.3732E+01, &
      1.759961E+01, 5.82044E+01, 2.5302E+00, 4.4926E+01, &
      1.881398E+01, 5.81087E+01, 2.2547E+00, 3.7608E+01, &
      2.011215E+01, 5.80087E+01, 2.0111E+00, 3.1522E+01, &
      2.149988E+01, 5.79055E+01, 1.7954E+00, 2.6454E+01, &
      2.298338E+01, 5.77995E+01, 1.6044E+00, 2.2230E+01, &
      2.456923E+01, 5.76904E+01, 1.4350E+00, 1.8704E+01, &
      2.626450E+01, 5.75765E+01, 1.2815E+00, 1.5724E+01, &
      2.807676E+01, 5.74526E+01, 1.1348E+00, 1.3121E+01, &
      3.001405E+01, 5.73038E+01, 9.9927E-01, 1.0901E+01, &
      3.208502E+01, 5.71194E+01, 8.7956E-01, 9.0609E+00, &
      3.429889E+01, 5.68744E+01, 7.7465E-01, 7.5428E+00, &
      3.666551E+01, 5.64958E+01, 6.8282E-01, 6.2903E+00, &
      3.919543E+01, 5.56078E+01, 6.0237E-01, 5.2557E+00, &
      3.963414E+01, 5.52451E+01, 5.8995E-01, 5.1014E+00, &
      4.024079E+01, 5.40411E+01, 5.7343E-01, 4.8986E+00, &
      4.040256E+01, 5.25980E+01, 5.6915E-01, 4.8464E+00, &
      4.064522E+01, 5.40571E+01, 3.4044E+00, 2.5765E+01, &
      4.125186E+01, 5.53764E+01, 3.3261E+00, 2.4814E+01, &
      4.189992E+01, 5.59523E+01, 3.2457E+00, 2.3851E+01, &
      4.479101E+01, 5.70307E+01, 2.9144E+00, 2.0080E+01, &
      4.788159E+01, 5.75239E+01, 2.6115E+00, 1.6876E+01, &
      5.118542E+01, 5.78210E+01, 2.3382E+00, 1.4176E+01, &
      5.471721E+01, 5.80169E+01, 2.0897E+00, 1.1890E+01, &
      5.849270E+01, 5.81468E+01, 1.8652E+00, 9.9653E+00, &
      6.252870E+01, 5.82336E+01, 1.6646E+00, 8.3544E+00, &
      6.684318E+01, 5.82911E+01, 1.4854E+00, 7.0066E+00, &
      7.145536E+01, 5.83279E+01, 1.3254E+00, 5.8790E+00, &
      7.638578E+01, 5.83503E+01, 1.1825E+00, 4.9359E+00, &
      8.165640E+01, 5.83941E+01, 1.0531E+00, 4.1403E+00, &
      8.729069E+01, 5.83944E+01, 9.3251E-01, 3.4575E+00, &
      9.331374E+01, 5.83864E+01, 8.2579E-01, 2.8910E+00, &
      9.975239E+01, 5.83732E+01, 7.3135E-01, 2.4206E+00, &
      1.066353E+02, 5.83567E+01, 6.4777E-01, 2.0300E+00, &
      1.139931E+02, 5.83381E+01, 5.7379E-01, 1.7056E+00, &
      1.218587E+02, 5.83185E+01, 5.0831E-01, 1.4358E+00, &
      1.302669E+02, 5.82983E+01, 4.5033E-01, 1.2116E+00, &
      1.392553E+02, 5.82782E+01, 3.9900E-01, 1.0250E+00, &
      1.488640E+02, 5.82583E+01, 3.5354E-01, 8.6974E-01, &
      1.591356E+02, 5.82391E+01, 3.1329E-01, 7.4039E-01, &
      1.701159E+02, 5.82206E+01, 2.7764E-01, 6.3255E-01, &
      1.818539E+02, 5.82029E+01, 2.4629E-01, 5.4295E-01, &
      1.944018E+02, 5.81864E+01, 2.1867E-01, 4.6830E-01, &
      2.078156E+02, 5.81710E+01, 1.9415E-01, 4.0577E-01, &
      2.221548E+02, 5.81566E+01, 1.7240E-01, 3.5332E-01, &
      2.374835E+02, 5.81432E+01, 1.5309E-01, 3.0925E-01, &
      2.538699E+02, 5.81308E+01, 1.3595E-01, 2.7216E-01, &
      2.713869E+02, 5.81193E+01, 1.2074E-01, 2.4087E-01, &
      2.901126E+02, 5.81087E+01, 1.0723E-01, 2.1441E-01, &
      3.101304E+02, 5.80989E+01, 9.5245E-02, 1.9195E-01, &
      3.315294E+02, 5.80899E+01, 8.4601E-02, 1.7285E-01, &
      3.544049E+02, 5.80817E+01, 7.5151E-02, 1.5654E-01, &
      3.788588E+02, 5.80742E+01, 6.6759E-02, 1.4255E-01, &
      4.050001E+02, 5.80673E+01, 5.9308E-02, 1.3049E-01, &
      4.329451E+02, 5.80609E+01, 5.2690E-02, 1.2005E-01]
   case(59)
      frel = -4.9140E-01
      allocate(nist_tab(936))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.72746E+01, 2.3728E-01, 6.6288E+03, &
      1.142761E-02, 1.71786E+01, 2.6237E-01, 6.8564E+03, &
      1.221612E-02, 1.70636E+01, 2.9076E-01, 7.1080E+03, &
      1.305903E-02, 1.69241E+01, 3.2288E-01, 7.3837E+03, &
      1.396010E-02, 1.67523E+01, 3.5918E-01, 7.6836E+03, &
      1.492335E-02, 1.65360E+01, 4.0006E-01, 8.0058E+03, &
      1.595306E-02, 1.62553E+01, 4.4589E-01, 8.3470E+03, &
      1.705382E-02, 1.58752E+01, 4.9710E-01, 8.7050E+03, &
      1.823053E-02, 1.53259E+01, 5.5414E-01, 9.0774E+03, &
      1.948844E-02, 1.44414E+01, 6.1742E-01, 9.4612E+03, &
      2.083314E-02, 1.26571E+01, 6.8733E-01, 9.8528E+03, &
      2.185400E-02, 8.78438E+00, 7.4176E-01, 1.0136E+04, &
      2.218850E-02, 3.65754E+00, 7.5981E-01, 1.0226E+04, &
      2.227063E-02, -1.56890E+00, 7.6426E-01, 1.0248E+04, &
      2.227770E-02, -2.66574E+00, 7.6464E-01, 1.0250E+04, &
      2.241150E-02, 4.16438E+00, 1.3314E+01, 1.7741E+05, &
      2.274600E-02, 1.02066E+01, 1.2834E+01, 1.6851E+05, &
      2.380730E-02, 1.57662E+01, 1.1483E+01, 1.4405E+05, &
      2.545001E-02, 1.90008E+01, 9.7813E+00, 1.1478E+05, &
      2.720606E-02, 2.06242E+01, 8.2672E+00, 9.0748E+04, &
      2.908327E-02, 2.14648E+01, 6.9800E+00, 7.1673E+04, &
      3.109002E-02, 2.18593E+01, 5.9361E+00, 5.7019E+04, &
      3.323523E-02, 2.19930E+01, 5.1215E+00, 4.6020E+04, &
      3.552846E-02, 2.19770E+01, 4.5077E+00, 3.7889E+04, &
      3.665200E-02, 2.19361E+01, 4.2806E+00, 3.4878E+04, &
      3.721300E-02, 2.19051E+01, 4.1821E+00, 3.3561E+04, &
      3.736260E-02, 2.18881E+01, 4.1573E+00, 3.3229E+04, &
      3.758700E-02, 2.18891E+01, 4.1426E+00, 3.2914E+04, &
      3.797993E-02, 2.18795E+01, 4.0837E+00, 3.2111E+04, &
      3.814800E-02, 2.18737E+01, 4.0597E+00, 3.1781E+04, &
      4.060054E-02, 2.17708E+01, 3.7806E+00, 2.7808E+04, &
      4.340198E-02, 2.16522E+01, 3.5873E+00, 2.4683E+04, &
      4.639671E-02, 2.15466E+01, 3.4800E+00, 2.2399E+04, &
      4.959809E-02, 2.14645E+01, 3.4368E+00, 2.0694E+04, &
      5.302035E-02, 2.13979E+01, 3.4456E+00, 1.9408E+04, &
      5.667876E-02, 2.13576E+01, 3.4962E+00, 1.8421E+04, &
      6.058959E-02, 2.13462E+01, 3.5799E+00, 1.7645E+04, &
      6.477028E-02, 2.13678E+01, 3.6904E+00, 1.7015E+04, &
      6.923942E-02, 2.14186E+01, 3.6967E+00, 1.5944E+04, &
      7.401695E-02, 2.14093E+01, 3.6665E+00, 1.4793E+04, &
      7.912411E-02, 2.13305E+01, 3.6299E+00, 1.3700E+04, &
      8.458368E-02, 2.11444E+01, 3.5846E+00, 1.2656E+04, &
      9.041995E-02, 2.07735E+01, 3.5296E+00, 1.1658E+04, &
      9.665893E-02, 2.00352E+01, 3.4646E+00, 1.0704E+04, &
      1.033284E-01, 1.83476E+01, 3.3896E+00, 9.7968E+03, &
      1.104581E-01, 1.16068E+01, 3.3048E+00, 8.9354E+03, &
      1.109360E-01, 1.03019E+01, 3.2990E+00, 8.8813E+03, &
      1.126340E-01, -9.81412E-01, 3.2783E+00, 8.6925E+03, &
      1.130868E-01, -1.67885E+01, 3.2727E+00, 8.6430E+03, &
      1.137660E-01, 3.85738E+00, 3.5265E+01, 9.2571E+04, &
      1.154640E-01, 2.10277E+01, 2.8922E+01, 7.4805E+04, &
      1.180797E-01, 2.86936E+01, 2.1620E+01, 5.4681E+04, &
      1.262272E-01, 3.14534E+01, 9.9813E+00, 2.3615E+04, &
      1.349368E-01, 2.96178E+01, 6.1318E+00, 1.3572E+04, &
      1.442475E-01, 2.82581E+01, 4.7184E+00, 9.7694E+03, &
      1.542005E-01, 2.73603E+01, 4.0747E+00, 7.8924E+03, &
      1.648404E-01, 2.67195E+01, 3.7343E+00, 6.7665E+03, &
      1.762144E-01, 2.62060E+01, 3.5421E+00, 6.0042E+03, &
      1.883732E-01, 2.57360E+01, 3.4409E+00, 5.4565E+03, &
      2.013709E-01, 2.52247E+01, 3.4093E+00, 5.0576E+03, &
      2.132480E-01, 2.45328E+01, 3.4319E+00, 4.8078E+03, &
      2.152655E-01, 2.42898E+01, 3.4399E+00, 4.7739E+03, &
      2.165120E-01, 2.40212E+01, 3.4454E+00, 4.7541E+03, &
      2.173824E-01, 2.34988E+01, 3.4495E+00, 4.7407E+03, &
      2.186880E-01, 2.39617E+01, 4.4336E+00, 6.0563E+03, &
      2.219520E-01, 2.43073E+01, 4.4636E+00, 6.0076E+03, &
      2.301188E-01, 2.44087E+01, 4.5497E+00, 5.9062E+03, &
      2.315740E-01, 2.43919E+01, 4.5665E+00, 5.8910E+03, &
      2.351185E-01, 2.42640E+01, 4.6095E+00, 5.8568E+03, &
      2.360637E-01, 2.40994E+01, 4.6214E+00, 5.8484E+03, &
      2.374815E-01, 2.42665E+01, 4.9652E+00, 6.2458E+03, &
      2.410260E-01, 2.44103E+01, 5.0167E+00, 6.2180E+03, &
      2.459970E-01, 2.44805E+01, 5.0924E+00, 6.1842E+03, &
      2.629708E-01, 2.45523E+01, 5.3744E+00, 6.1058E+03, &
      2.811158E-01, 2.45591E+01, 5.7025E+00, 6.0605E+03, &
      2.984100E-01, 2.44528E+01, 6.0245E+00, 6.0319E+03, &
      3.005128E-01, 2.43988E+01, 6.0636E+00, 6.0286E+03, &
      3.029775E-01, 2.42591E+01, 6.1093E+00, 6.0247E+03, &
      3.041955E-01, 2.40036E+01, 6.1318E+00, 6.0227E+03, &
      3.060225E-01, 2.42818E+01, 6.6829E+00, 6.5245E+03, &
      3.105900E-01, 2.45476E+01, 6.7700E+00, 6.5123E+03, &
      3.212482E-01, 2.48143E+01, 6.9693E+00, 6.4818E+03, &
      3.434143E-01, 2.51989E+01, 7.3577E+00, 6.4017E+03, &
      3.671099E-01, 2.55909E+01, 7.7181E+00, 6.2821E+03, &
      3.924405E-01, 2.60096E+01, 8.0300E+00, 6.1146E+03, &
      4.195189E-01, 2.64465E+01, 8.2777E+00, 5.8967E+03, &
      4.484657E-01, 2.68823E+01, 8.4482E+00, 5.6303E+03, &
      4.794098E-01, 2.72919E+01, 8.5326E+00, 5.3199E+03, &
      5.124891E-01, 2.76374E+01, 8.5245E+00, 4.9724E+03, &
      5.478508E-01, 2.78793E+01, 8.4340E+00, 4.6028E+03, &
      5.856525E-01, 2.79771E+01, 8.2746E+00, 4.2250E+03, &
      6.260625E-01, 2.78830E+01, 8.0606E+00, 3.8509E+03, &
      6.692609E-01, 2.75307E+01, 7.8062E+00, 3.4895E+03, &
      7.154399E-01, 2.68172E+01, 7.5237E+00, 3.1470E+03, &
      7.648052E-01, 2.55391E+01, 7.2236E+00, 2.8273E+03, &
      8.175768E-01, 2.32365E+01, 6.9148E+00, 2.5328E+03, &
      8.739896E-01, 1.83938E+01, 6.6041E+00, 2.2638E+03, &
      9.123800E-01, 9.90448E+00, 6.4050E+00, 2.1038E+03, &
      9.263450E-01, -1.85307E-01, 6.3351E+00, 2.0497E+03, &
      9.300690E-01, -1.10922E+01, 6.3167E+00, 2.0356E+03, &
      9.320780E-01, -1.03377E+01, 2.6811E+01, 8.5978E+03, &
      9.342948E-01, -3.36199E+00, 2.6698E+01, 8.5410E+03, &
      9.356550E-01, -1.35468E+00, 2.6628E+01, 8.5065E+03, &
      9.463445E-01, 2.11908E+00, 2.6093E+01, 8.2415E+03, &
      9.501489E-01, -3.13253E+00, 2.5906E+01, 8.1500E+03, &
      9.558555E-01, 6.00182E+00, 3.9126E+01, 1.2231E+04, &
      9.701220E-01, 1.56757E+01, 3.8024E+01, 1.1713E+04, &
      9.987612E-01, 2.39928E+01, 3.5955E+01, 1.0759E+04, &
      1.067676E+00, 3.26129E+01, 3.1912E+01, 8.9338E+03, &
      1.141345E+00, 3.67032E+01, 2.8707E+01, 7.5191E+03, &
      1.217356E+00, 3.78194E+01, 2.6058E+01, 6.4006E+03, &
      1.220098E+00, 3.77470E+01, 2.5972E+01, 6.3652E+03, &
      1.235989E+00, 3.64995E+01, 2.5485E+01, 6.1657E+03, &
      1.240958E+00, 3.44409E+01, 2.5336E+01, 6.1053E+03, &
      1.248411E+00, 3.68305E+01, 2.9405E+01, 7.0422E+03, &
      1.267044E+00, 3.91981E+01, 2.8731E+01, 6.7800E+03, &
      1.304285E+00, 4.09346E+01, 2.7473E+01, 6.2985E+03, &
      1.310652E+00, 4.10718E+01, 2.7287E+01, 6.2257E+03, &
      1.330713E+00, 4.10160E+01, 2.6726E+01, 6.0061E+03, &
      1.336063E+00, 4.02641E+01, 2.6582E+01, 5.9499E+03, &
      1.344087E+00, 4.14509E+01, 2.8188E+01, 6.2713E+03, &
      1.364148E+00, 4.28521E+01, 2.7617E+01, 6.0540E+03, &
      1.394281E+00, 4.40113E+01, 2.6813E+01, 5.7512E+03, &
      1.480780E+00, 4.58154E+01, 2.4922E+01, 5.0345E+03, &
      1.490486E+00, 4.58831E+01, 2.4735E+01, 4.9644E+03, &
      1.503445E+00, 4.57859E+01, 2.4492E+01, 4.8734E+03, &
      1.509489E+00, 4.53126E+01, 2.4381E+01, 4.8319E+03, &
      1.518555E+00, 4.60868E+01, 2.5383E+01, 5.0002E+03, &
      1.541220E+00, 4.70452E+01, 2.4974E+01, 4.8475E+03, &
      1.593329E+00, 4.83610E+01, 2.4063E+01, 4.5186E+03, &
      1.703269E+00, 5.02510E+01, 2.2110E+01, 3.8850E+03, &
      1.820795E+00, 5.16042E+01, 2.0250E+01, 3.3297E+03, &
      1.946430E+00, 5.26024E+01, 1.8518E+01, 2.8495E+03, &
      2.080733E+00, 5.33495E+01, 1.6915E+01, 2.4360E+03, &
      2.224304E+00, 5.38968E+01, 1.5428E+01, 2.0796E+03, &
      2.377781E+00, 5.42865E+01, 1.4073E+01, 1.7756E+03, &
      2.541848E+00, 5.46680E+01, 1.2844E+01, 1.5170E+03, &
      2.717235E+00, 5.49099E+01, 1.1717E+01, 1.2955E+03, &
      2.904724E+00, 5.50655E+01, 1.0692E+01, 1.1070E+03, &
      3.105150E+00, 5.52498E+01, 9.6854E+00, 9.3900E+02, &
      3.319406E+00, 5.51905E+01, 8.7117E+00, 7.9108E+02, &
      3.548445E+00, 5.50442E+01, 7.8467E+00, 6.6748E+02, &
      3.793288E+00, 5.48333E+01, 7.0689E+00, 5.6340E+02, &
      4.055024E+00, 5.45398E+01, 6.3421E+00, 4.7372E+02, &
      4.334821E+00, 5.41509E+01, 5.6964E+00, 3.9886E+02, &
      4.633924E+00, 5.36469E+01, 5.1242E+00, 3.3640E+02, &
      4.953664E+00, 5.29708E+01, 4.6174E+00, 2.8430E+02, &
      5.295467E+00, 5.19742E+01, 4.1684E+00, 2.4075E+02, &
      5.660855E+00, 5.01049E+01, 3.7541E+00, 2.0347E+02, &
      5.845014E+00, 4.79480E+01, 3.5682E+00, 1.8761E+02, &
      5.934479E+00, 4.48046E+01, 3.4816E+00, 1.8044E+02, &
      5.958336E+00, 4.11580E+01, 3.4592E+00, 1.7860E+02, &
      5.994122E+00, 4.47709E+01, 1.0518E+01, 5.2923E+02, &
      6.051453E+00, 4.71935E+01, 1.0305E+01, 5.1371E+02, &
      6.083586E+00, 4.78639E+01, 1.0188E+01, 5.0528E+02, &
      6.311592E+00, 4.92910E+01, 9.4366E+00, 4.5151E+02, &
      6.408198E+00, 4.83098E+01, 9.1775E+00, 4.3264E+02, &
      6.433960E+00, 4.66523E+01, 9.1113E+00, 4.2784E+02, &
      6.469004E+00, 4.84282E+01, 1.2493E+01, 5.8166E+02, &
      6.472602E+00, 4.85756E+01, 1.2481E+01, 5.8077E+02, &
      6.569208E+00, 5.04377E+01, 1.2156E+01, 5.5748E+02, &
      6.698104E+00, 5.13533E+01, 1.1748E+01, 5.2859E+02, &
      6.800626E+00, 5.12799E+01, 1.1445E+01, 5.0733E+02, &
      6.827965E+00, 5.05336E+01, 1.1368E+01, 5.0193E+02, &
      6.868974E+00, 5.16492E+01, 1.3028E+01, 5.7111E+02, &
      6.915365E+00, 5.23747E+01, 1.2896E+01, 5.6155E+02, &
      6.971496E+00, 5.29391E+01, 1.2739E+01, 5.5033E+02, &
      7.392525E+00, 5.51795E+01, 1.1668E+01, 4.7577E+02, &
      7.902609E+00, 5.66130E+01, 1.0546E+01, 4.0271E+02, &
      8.447890E+00, 5.75552E+01, 9.5034E+00, 3.3988E+02, &
      9.030794E+00, 5.82224E+01, 8.5385E+00, 2.8606E+02, &
      9.653919E+00, 5.86655E+01, 7.6226E+00, 2.3927E+02, &
      1.032004E+01, 5.89521E+01, 6.8109E+00, 2.0034E+02, &
      1.103212E+01, 5.91366E+01, 6.0909E+00, 1.6792E+02, &
      1.179334E+01, 5.92527E+01, 5.4519E+00, 1.4090E+02, &
      1.260708E+01, 5.93873E+01, 4.8664E+00, 1.1792E+02, &
      1.347697E+01, 5.94424E+01, 4.3371E+00, 9.8568E+01, &
      1.440688E+01, 5.94585E+01, 3.8547E+00, 8.2192E+01, &
      1.540095E+01, 5.94080E+01, 3.4263E+00, 6.8562E+01, &
      1.646362E+01, 5.93379E+01, 3.0482E+00, 5.7258E+01, &
      1.759961E+01, 5.92553E+01, 2.7141E+00, 4.7872E+01, &
      1.881398E+01, 5.91643E+01, 2.4183E+00, 4.0065E+01, &
      2.011215E+01, 5.90678E+01, 2.1566E+00, 3.3572E+01, &
      2.149988E+01, 5.89673E+01, 1.9251E+00, 2.8168E+01, &
      2.298338E+01, 5.88640E+01, 1.7200E+00, 2.3663E+01, &
      2.456923E+01, 5.87578E+01, 1.5382E+00, 1.9906E+01, &
      2.626450E+01, 5.86484E+01, 1.3762E+00, 1.6759E+01, &
      2.807676E+01, 5.85341E+01, 1.2199E+00, 1.3995E+01, &
      3.001405E+01, 5.83990E+01, 1.0772E+00, 1.1655E+01, &
      3.208502E+01, 5.82361E+01, 9.4878E-01, 9.6907E+00, &
      3.429889E+01, 5.80303E+01, 8.3572E-01, 8.0654E+00, &
      3.666551E+01, 5.77427E+01, 7.3670E-01, 6.7242E+00, &
      3.919543E+01, 5.72414E+01, 6.4993E-01, 5.6163E+00, &
      4.115079E+01, 5.62616E+01, 5.9345E-01, 4.9308E+00, &
      4.178065E+01, 5.50638E+01, 5.7690E-01, 4.7356E+00, &
      4.189992E+01, 5.43184E+01, 5.7385E-01, 4.6998E+00, &
      4.194861E+01, 5.36280E+01, 5.7261E-01, 4.6854E+00, &
      4.220055E+01, 5.50800E+01, 3.3934E+00, 2.4618E+01, &
      4.283041E+01, 5.63927E+01, 3.3149E+00, 2.3706E+01, &
      4.479101E+01, 5.75729E+01, 3.0868E+00, 2.1140E+01, &
      4.788159E+01, 5.82861E+01, 2.7683E+00, 1.7781E+01, &
      5.118542E+01, 5.86719E+01, 2.4801E+00, 1.4944E+01, &
      5.471721E+01, 5.89167E+01, 2.2208E+00, 1.2557E+01, &
      5.849270E+01, 5.90794E+01, 1.9833E+00, 1.0528E+01, &
      6.252870E+01, 5.91879E+01, 1.7708E+00, 8.8288E+00, &
      6.684318E+01, 5.92606E+01, 1.5808E+00, 7.4065E+00, &
      7.145536E+01, 5.93083E+01, 1.4110E+00, 6.2160E+00, &
      7.638578E+01, 5.93386E+01, 1.2594E+00, 5.2198E+00, &
      8.165640E+01, 5.93582E+01, 1.1226E+00, 4.3812E+00, &
      8.729069E+01, 5.93943E+01, 9.9577E-01, 3.6637E+00, &
      9.331374E+01, 5.93901E+01, 8.8198E-01, 3.0629E+00, &
      9.975239E+01, 5.93794E+01, 7.8127E-01, 2.5641E+00, &
      1.066353E+02, 5.93645E+01, 6.9213E-01, 2.1498E+00, &
      1.139931E+02, 5.93470E+01, 6.1320E-01, 1.8056E+00, &
      1.218587E+02, 5.93279E+01, 5.4333E-01, 1.5195E+00, &
      1.302669E+02, 5.93081E+01, 4.8145E-01, 1.2815E+00, &
      1.392553E+02, 5.92879E+01, 4.2666E-01, 1.0836E+00, &
      1.488640E+02, 5.92680E+01, 3.7813E-01, 9.1883E-01, &
      1.591356E+02, 5.92485E+01, 3.3515E-01, 7.8155E-01, &
      1.701159E+02, 5.92296E+01, 2.9707E-01, 6.6711E-01, &
      1.818539E+02, 5.92114E+01, 2.6334E-01, 5.7163E-01, &
      1.944018E+02, 5.91943E+01, 2.3394E-01, 4.9262E-01, &
      2.078156E+02, 5.91783E+01, 2.0785E-01, 4.2644E-01, &
      2.221548E+02, 5.91635E+01, 1.8468E-01, 3.7091E-01, &
      2.374835E+02, 5.91496E+01, 1.6410E-01, 3.2424E-01, &
      2.538699E+02, 5.91368E+01, 1.4582E-01, 2.8496E-01, &
      2.713869E+02, 5.91249E+01, 1.2959E-01, 2.5181E-01, &
      2.901126E+02, 5.91139E+01, 1.1517E-01, 2.2378E-01, &
      3.101304E+02, 5.91037E+01, 1.0236E-01, 2.0001E-01, &
      3.315294E+02, 5.90944E+01, 9.0977E-02, 1.7979E-01, &
      3.544049E+02, 5.90858E+01, 8.0867E-02, 1.6253E-01, &
      3.788588E+02, 5.90779E+01, 7.1883E-02, 1.4774E-01, &
      4.050001E+02, 5.90707E+01, 6.3900E-02, 1.3500E-01, &
      4.329451E+02, 5.90641E+01, 5.6807E-02, 1.2397E-01]
   case(60)
      frel = -5.1240E-01
      allocate(nist_tab(3572))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.48440E+01, 2.7095E-01, 7.3944E+03, &
      1.142761E-02, 1.47192E+01, 3.0133E-01, 7.6926E+03, &
      1.221612E-02, 1.45682E+01, 3.3573E-01, 8.0178E+03, &
      1.305903E-02, 1.43830E+01, 3.7466E-01, 8.3698E+03, &
      1.396010E-02, 1.41508E+01, 4.1861E-01, 8.7480E+03, &
      1.492335E-02, 1.38515E+01, 4.6815E-01, 9.1520E+03, &
      1.595306E-02, 1.34503E+01, 5.2390E-01, 9.5806E+03, &
      1.705382E-02, 1.28793E+01, 5.8645E-01, 1.0032E+04, &
      1.823053E-02, 1.19837E+01, 6.5646E-01, 1.0505E+04, &
      1.948844E-02, 1.02794E+01, 7.3456E-01, 1.0996E+04, &
      2.067800E-02, 5.53421E+00, 8.1122E-01, 1.1445E+04, &
      2.083314E-02, 3.72966E+00, 8.2140E-01, 1.1502E+04, &
      2.099450E-02, -9.60247E-02, 8.3203E-01, 1.1562E+04, &
      2.107890E-02, -7.02935E+00, 8.3761E-01, 1.1593E+04, &
      2.120550E-02, 4.43496E-01, 1.4593E+01, 2.0077E+05, &
      2.152200E-02, 7.05288E+00, 1.4087E+01, 1.9096E+05, &
      2.227063E-02, 1.21512E+01, 1.2999E+01, 1.7028E+05, &
      2.380730E-02, 1.62683E+01, 1.1145E+01, 1.3657E+05, &
      2.545001E-02, 1.82917E+01, 9.4814E+00, 1.0869E+05, &
      2.720606E-02, 1.93604E+01, 8.0415E+00, 8.6231E+04, &
      2.908327E-02, 1.98867E+01, 6.8589E+00, 6.8802E+04, &
      3.109002E-02, 2.00933E+01, 5.9286E+00, 5.5632E+04, &
      3.323523E-02, 2.01157E+01, 5.2254E+00, 4.5868E+04, &
      3.552846E-02, 2.00381E+01, 4.7161E+00, 3.8726E+04, &
      3.675000E-02, 1.99757E+01, 4.5213E+00, 3.5892E+04, &
      3.731250E-02, 1.99387E+01, 4.4460E+00, 3.4762E+04, &
      3.746250E-02, 1.99204E+01, 4.4273E+00, 3.4478E+04, &
      3.768750E-02, 1.99195E+01, 4.4216E+00, 3.4227E+04, &
      3.797993E-02, 1.99111E+01, 4.3890E+00, 3.3714E+04, &
      3.825000E-02, 1.99006E+01, 4.3607E+00, 3.3260E+04, &
      4.060054E-02, 1.97994E+01, 4.1768E+00, 3.0012E+04, &
      4.340198E-02, 1.96963E+01, 4.0705E+00, 2.7361E+04, &
      4.639671E-02, 1.96222E+01, 4.0486E+00, 2.5457E+04, &
      4.959809E-02, 1.95881E+01, 4.0943E+00, 2.4083E+04, &
      5.302035E-02, 1.96043E+01, 4.1942E+00, 2.3078E+04, &
      5.667876E-02, 1.96841E+01, 4.3227E+00, 2.2250E+04, &
      6.058959E-02, 1.97918E+01, 4.3219E+00, 2.0810E+04, &
      6.477028E-02, 1.99005E+01, 4.3249E+00, 1.9480E+04, &
      6.923942E-02, 1.99804E+01, 4.3260E+00, 1.8227E+04, &
      7.401695E-02, 2.00357E+01, 4.3208E+00, 1.7031E+04, &
      7.912411E-02, 2.00504E+01, 4.3063E+00, 1.5878E+04, &
      8.458368E-02, 1.99983E+01, 4.2801E+00, 1.4763E+04, &
      9.041995E-02, 1.98321E+01, 4.2412E+00, 1.3684E+04, &
      9.665893E-02, 1.94554E+01, 4.1888E+00, 1.2643E+04, &
      1.0000000E-01, 2.09142E+01, 4.1569E+00, 1.2128E+04, &
      1.0050000E-01, 2.08515E+01, 4.1519E+00, 1.2053E+04, &
      1.0100250E-01, 2.07848E+01, 4.1468E+00, 1.1978E+04, &
      1.0150751E-01, 2.07137E+01, 4.1416E+00, 1.1904E+04, &
      1.0201505E-01, 2.06377E+01, 4.1364E+00, 1.1830E+04, &
      1.0252513E-01, 2.05565E+01, 4.1311E+00, 1.1756E+04, &
      1.0303775E-01, 2.04695E+01, 4.1258E+00, 1.1682E+04, &
      1.0355294E-01, 2.03763E+01, 4.1203E+00, 1.1609E+04, &
      1.0407070E-01, 2.02762E+01, 4.1148E+00, 1.1535E+04, &
      1.0459106E-01, 2.01685E+01, 4.1092E+00, 1.1462E+04, &
      1.0511401E-01, 2.00523E+01, 4.1035E+00, 1.1390E+04, &
      1.0563958E-01, 1.99269E+01, 4.0978E+00, 1.1317E+04, &
      1.0616778E-01, 1.97909E+01, 4.0919E+00, 1.1245E+04, &
      1.0669862E-01, 1.96433E+01, 4.0860E+00, 1.1173E+04, &
      1.0723211E-01, 1.94824E+01, 4.0801E+00, 1.1101E+04, &
      1.0776827E-01, 1.93065E+01, 4.0740E+00, 1.1029E+04, &
      1.0830712E-01, 1.91135E+01, 4.0679E+00, 1.0958E+04, &
      1.0884865E-01, 1.89007E+01, 4.0617E+00, 1.0887E+04, &
      1.0939289E-01, 1.86649E+01, 4.0554E+00, 1.0816E+04, &
      1.0993986E-01, 1.84023E+01, 4.0491E+00, 1.0745E+04, &
      1.1048956E-01, 1.81078E+01, 4.0427E+00, 1.0675E+04, &
      1.1104201E-01, 1.77752E+01, 4.0362E+00, 1.0605E+04, &
      1.1159722E-01, 1.73961E+01, 4.0296E+00, 1.0535E+04, &
      1.1215520E-01, 1.69596E+01, 4.0230E+00, 1.0465E+04, &
      1.1271598E-01, 1.64510E+01, 4.0163E+00, 1.0396E+04, &
      1.1327956E-01, 1.58479E+01, 4.0096E+00, 1.0327E+04, &
      1.1384596E-01, 1.51193E+01, 4.0027E+00, 1.0258E+04, &
      1.1441519E-01, 1.42163E+01, 3.9958E+00, 1.0189E+04, &
      1.1498726E-01, 1.30569E+01, 3.9888E+00, 1.0121E+04, &
      1.1556220E-01, 1.14906E+01, 3.9818E+00, 1.0053E+04, &
      1.1614001E-01, 9.19412E+00, 3.9747E+00, 9.9847E+03, &
      1.1672071E-01, 5.25443E+00, 3.9675E+00, 9.9171E+03, &
      1.1730431E-01, -5.79019E+00, 3.9602E+00, 9.8498E+03, &
      1.1748954E-01, -3.19507E+01, 3.9579E+00, 9.8285E+03, &
      1.1751046E-01, -3.18147E+01, 3.2823E+01, 8.1490E+04, &
      1.1751048E-01, -3.17939E+01, 3.2823E+01, 8.1489E+04, &
      1.1789083E-01, 2.87043E+00, 3.1525E+01, 7.8014E+04, &
      1.1848029E-01, 1.24334E+01, 2.9635E+01, 7.2971E+04, &
      1.1907269E-01, 1.74057E+01, 2.7872E+01, 6.8290E+04, &
      1.1966805E-01, 2.06974E+01, 2.6230E+01, 6.3947E+04, &
      1.2026639E-01, 2.30793E+01, 2.4699E+01, 5.9916E+04, &
      1.2086772E-01, 2.48815E+01, 2.3273E+01, 5.6174E+04, &
      1.2147206E-01, 2.62796E+01, 2.1943E+01, 5.2700E+04, &
      1.2207942E-01, 2.73799E+01, 2.0703E+01, 4.9476E+04, &
      1.2268982E-01, 2.82520E+01, 1.9548E+01, 4.6482E+04, &
      1.2330327E-01, 2.89449E+01, 1.8470E+01, 4.3702E+04, &
      1.2391979E-01, 2.94940E+01, 1.7466E+01, 4.1120E+04, &
      1.2453939E-01, 2.99260E+01, 1.6530E+01, 3.8723E+04, &
      1.2516208E-01, 3.02618E+01, 1.5657E+01, 3.6495E+04, &
      1.2578789E-01, 3.05176E+01, 1.4843E+01, 3.4426E+04, &
      1.2641683E-01, 3.07063E+01, 1.4084E+01, 3.2503E+04, &
      1.2704892E-01, 3.08384E+01, 1.3376E+01, 3.0717E+04, &
      1.2768416E-01, 3.09224E+01, 1.2716E+01, 2.9056E+04, &
      1.2832258E-01, 3.09650E+01, 1.2101E+01, 2.7512E+04, &
      1.2896419E-01, 3.09715E+01, 1.1527E+01, 2.6076E+04, &
      1.2960902E-01, 3.09458E+01, 1.0992E+01, 2.4743E+04, &
      1.3025706E-01, 3.08930E+01, 1.0503E+01, 2.3524E+04, &
      1.3090835E-01, 3.08211E+01, 1.0056E+01, 2.2412E+04, &
      1.3156289E-01, 3.07355E+01, 9.6484E+00, 2.1396E+04, &
      1.3222070E-01, 3.06403E+01, 9.2749E+00, 2.0465E+04, &
      1.3288181E-01, 3.05385E+01, 8.9324E+00, 1.9612E+04, &
      1.3354621E-01, 3.04323E+01, 8.6180E+00, 1.8827E+04, &
      1.3421395E-01, 3.03234E+01, 8.3287E+00, 1.8105E+04, &
      1.3488502E-01, 3.02132E+01, 8.0624E+00, 1.7439E+04, &
      1.3555944E-01, 3.01027E+01, 7.8166E+00, 1.6823E+04, &
      1.3623724E-01, 2.99926E+01, 7.5896E+00, 1.6253E+04, &
      1.3691842E-01, 2.98837E+01, 7.3796E+00, 1.5725E+04, &
      1.3760302E-01, 2.97763E+01, 7.1851E+00, 1.5234E+04, &
      1.3829103E-01, 2.96708E+01, 7.0046E+00, 1.4778E+04, &
      1.3898249E-01, 2.95675E+01, 6.8369E+00, 1.4352E+04, &
      1.3967740E-01, 2.94665E+01, 6.6809E+00, 1.3955E+04, &
      1.4037579E-01, 2.93681E+01, 6.5356E+00, 1.3584E+04, &
      1.4107766E-01, 2.92722E+01, 6.4000E+00, 1.3236E+04, &
      1.4178305E-01, 2.91790E+01, 6.2735E+00, 1.2909E+04, &
      1.4249197E-01, 2.90884E+01, 6.1551E+00, 1.2603E+04, &
      1.4320443E-01, 2.90004E+01, 6.0443E+00, 1.2314E+04, &
      1.4392045E-01, 2.89151E+01, 5.9404E+00, 1.2043E+04, &
      1.4464005E-01, 2.88323E+01, 5.8429E+00, 1.1786E+04, &
      1.4536325E-01, 2.87520E+01, 5.7512E+00, 1.1543E+04, &
      1.4609007E-01, 2.86742E+01, 5.6650E+00, 1.1314E+04, &
      1.4682052E-01, 2.85987E+01, 5.5839E+00, 1.1096E+04, &
      1.4755462E-01, 2.85256E+01, 5.5073E+00, 1.0890E+04, &
      1.4829239E-01, 2.84547E+01, 5.4351E+00, 1.0693E+04, &
      1.4903386E-01, 2.83860E+01, 5.3668E+00, 1.0507E+04, &
      1.4977903E-01, 2.83194E+01, 5.3022E+00, 1.0329E+04, &
      1.5052792E-01, 2.82548E+01, 5.2411E+00, 1.0159E+04, &
      1.5128056E-01, 2.81921E+01, 5.1831E+00, 9.9964E+03, &
      1.5203696E-01, 2.81313E+01, 5.1282E+00, 9.8412E+03, &
      1.5279715E-01, 2.80722E+01, 5.0759E+00, 9.6926E+03, &
      1.5356113E-01, 2.80149E+01, 5.0263E+00, 9.5501E+03, &
      1.5432894E-01, 2.79592E+01, 4.9791E+00, 9.4134E+03, &
      1.5510058E-01, 2.79050E+01, 4.9342E+00, 9.2820E+03, &
      1.5587609E-01, 2.78523E+01, 4.8914E+00, 9.1557E+03, &
      1.5665547E-01, 2.78011E+01, 4.8505E+00, 9.0341E+03, &
      1.5743875E-01, 2.77512E+01, 4.8115E+00, 8.9170E+03, &
      1.5822594E-01, 2.77026E+01, 4.7743E+00, 8.8040E+03, &
      1.5901707E-01, 2.76552E+01, 4.7387E+00, 8.6949E+03, &
      1.5981215E-01, 2.76090E+01, 4.7047E+00, 8.5895E+03, &
      1.6061121E-01, 2.75639E+01, 4.6721E+00, 8.4876E+03, &
      1.6141427E-01, 2.75199E+01, 4.6409E+00, 8.3891E+03, &
      1.6222134E-01, 2.74769E+01, 4.6110E+00, 8.2936E+03, &
      1.6303245E-01, 2.74349E+01, 4.5824E+00, 8.2011E+03, &
      1.6384761E-01, 2.73938E+01, 4.5549E+00, 8.1113E+03, &
      1.6466685E-01, 2.73535E+01, 4.5285E+00, 8.0243E+03, &
      1.6549018E-01, 2.73141E+01, 4.5032E+00, 7.9397E+03, &
      1.6631763E-01, 2.72754E+01, 4.4789E+00, 7.8575E+03, &
      1.6714922E-01, 2.72375E+01, 4.4555E+00, 7.7777E+03, &
      1.6798497E-01, 2.72003E+01, 4.4330E+00, 7.7000E+03, &
      1.6882489E-01, 2.71637E+01, 4.4114E+00, 7.6243E+03, &
      1.6966902E-01, 2.71277E+01, 4.3906E+00, 7.5507E+03, &
      1.7051736E-01, 2.70924E+01, 4.3706E+00, 7.4789E+03, &
      1.7136995E-01, 2.70575E+01, 4.3514E+00, 7.4090E+03, &
      1.7222680E-01, 2.70232E+01, 4.3329E+00, 7.3408E+03, &
      1.7308793E-01, 2.69893E+01, 4.3151E+00, 7.2743E+03, &
      1.7395337E-01, 2.69559E+01, 4.2980E+00, 7.2094E+03, &
      1.7482314E-01, 2.69229E+01, 4.2815E+00, 7.1460E+03, &
      1.7569726E-01, 2.68903E+01, 4.2656E+00, 7.0841E+03, &
      1.7657574E-01, 2.68580E+01, 4.2503E+00, 7.0237E+03, &
      1.7745862E-01, 2.68260E+01, 4.2356E+00, 6.9646E+03, &
      1.7834591E-01, 2.67944E+01, 4.2215E+00, 6.9068E+03, &
      1.7923764E-01, 2.67629E+01, 4.2079E+00, 6.8504E+03, &
      1.8013383E-01, 2.67318E+01, 4.1948E+00, 6.7951E+03, &
      1.8103450E-01, 2.67008E+01, 4.1823E+00, 6.7411E+03, &
      1.8193967E-01, 2.66700E+01, 4.1702E+00, 6.6883E+03, &
      1.8284937E-01, 2.66394E+01, 4.1587E+00, 6.6366E+03, &
      1.8376362E-01, 2.66088E+01, 4.1476E+00, 6.5859E+03, &
      1.8468244E-01, 2.65784E+01, 4.1369E+00, 6.5364E+03, &
      1.8560585E-01, 2.65481E+01, 4.1268E+00, 6.4879E+03, &
      1.8653388E-01, 2.65178E+01, 4.1170E+00, 6.4404E+03, &
      1.8746655E-01, 2.64875E+01, 4.1077E+00, 6.3939E+03, &
      1.8840388E-01, 2.64572E+01, 4.0988E+00, 6.3483E+03, &
      1.8934590E-01, 2.64268E+01, 4.0903E+00, 6.3037E+03, &
      1.9029263E-01, 2.63964E+01, 4.0823E+00, 6.2600E+03, &
      1.9124409E-01, 2.63659E+01, 4.0746E+00, 6.2172E+03, &
      1.9220031E-01, 2.63352E+01, 4.0673E+00, 6.1752E+03, &
      1.9316131E-01, 2.63044E+01, 4.0605E+00, 6.1342E+03, &
      1.9412712E-01, 2.62733E+01, 4.0540E+00, 6.0939E+03, &
      1.9509776E-01, 2.62421E+01, 4.0479E+00, 6.0545E+03, &
      1.9607325E-01, 2.62105E+01, 4.0421E+00, 6.0158E+03, &
      1.9705361E-01, 2.61786E+01, 4.0368E+00, 5.9780E+03, &
      1.9803888E-01, 2.61463E+01, 4.0318E+00, 5.9409E+03, &
      1.9902907E-01, 2.61137E+01, 4.0271E+00, 5.9046E+03, &
      2.0002422E-01, 2.60805E+01, 4.0229E+00, 5.8690E+03, &
      2.0102434E-01, 2.60468E+01, 4.0189E+00, 5.8341E+03, &
      2.0202946E-01, 2.60125E+01, 4.0154E+00, 5.8000E+03, &
      2.0303961E-01, 2.59776E+01, 4.0122E+00, 5.7665E+03, &
      2.0405481E-01, 2.59418E+01, 4.0093E+00, 5.7338E+03, &
      2.0507508E-01, 2.59052E+01, 4.0068E+00, 5.7017E+03, &
      2.0610046E-01, 2.58677E+01, 4.0047E+00, 5.6703E+03, &
      2.0713096E-01, 2.58290E+01, 4.0029E+00, 5.6396E+03, &
      2.0816661E-01, 2.57891E+01, 4.0014E+00, 5.6095E+03, &
      2.0920745E-01, 2.57478E+01, 4.0003E+00, 5.5801E+03, &
      2.1025348E-01, 2.57048E+01, 3.9995E+00, 5.5513E+03, &
      2.1130475E-01, 2.56598E+01, 3.9991E+00, 5.5231E+03, &
      2.1236128E-01, 2.56127E+01, 3.9990E+00, 5.4955E+03, &
      2.1342308E-01, 2.55629E+01, 3.9992E+00, 5.4685E+03, &
      2.1449020E-01, 2.55098E+01, 3.9998E+00, 5.4421E+03, &
      2.1556265E-01, 2.54528E+01, 4.0008E+00, 5.4163E+03, &
      2.1664046E-01, 2.53908E+01, 4.0020E+00, 5.3911E+03, &
      2.1772366E-01, 2.53224E+01, 4.0036E+00, 5.3665E+03, &
      2.1881228E-01, 2.52454E+01, 4.0056E+00, 5.3424E+03, &
      2.1990634E-01, 2.51562E+01, 4.0079E+00, 5.3189E+03, &
      2.2100588E-01, 2.50481E+01, 4.0105E+00, 5.2959E+03, &
      2.2211090E-01, 2.49074E+01, 4.0134E+00, 5.2734E+03, &
      2.2322146E-01, 2.46945E+01, 4.0167E+00, 5.2515E+03, &
      2.2433757E-01, 2.41391E+01, 4.0204E+00, 5.2302E+03, &
      2.2447692E-01, 2.38940E+01, 4.0208E+00, 5.2275E+03, &
      2.2472308E-01, 2.38879E+01, 5.0250E+00, 6.5254E+03, &
      2.2545925E-01, 2.44907E+01, 5.0302E+00, 6.5108E+03, &
      2.2658655E-01, 2.47326E+01, 5.0384E+00, 6.4891E+03, &
      2.2771948E-01, 2.48514E+01, 5.0470E+00, 6.4679E+03, &
      2.2885808E-01, 2.49255E+01, 5.0560E+00, 6.4471E+03, &
      2.3000237E-01, 2.49759E+01, 5.0653E+00, 6.4269E+03, &
      2.3115238E-01, 2.50114E+01, 5.0749E+00, 6.4071E+03, &
      2.3230814E-01, 2.50365E+01, 5.0850E+00, 6.3878E+03, &
      2.3346969E-01, 2.50536E+01, 5.0953E+00, 6.3690E+03, &
      2.3463703E-01, 2.50640E+01, 5.1060E+00, 6.3506E+03, &
      2.3581022E-01, 2.50685E+01, 5.1171E+00, 6.3328E+03, &
      2.3698927E-01, 2.50671E+01, 5.1285E+00, 6.3153E+03, &
      2.3817422E-01, 2.50592E+01, 5.1402E+00, 6.2983E+03, &
      2.3936509E-01, 2.50433E+01, 5.1523E+00, 6.2817E+03, &
      2.4056191E-01, 2.50153E+01, 5.1648E+00, 6.2656E+03, &
      2.4176472E-01, 2.49630E+01, 5.1776E+00, 6.2499E+03, &
      2.4297355E-01, 2.48079E+01, 5.1907E+00, 6.2346E+03, &
      2.4315743E-01, 2.47223E+01, 5.1927E+00, 6.2323E+03, &
      2.4344258E-01, 2.47234E+01, 5.5238E+00, 6.6218E+03, &
      2.4418841E-01, 2.49173E+01, 5.5330E+00, 6.6126E+03, &
      2.4540936E-01, 2.50122E+01, 5.5484E+00, 6.5980E+03, &
      2.4663640E-01, 2.50643E+01, 5.5641E+00, 6.5837E+03, &
      2.4786959E-01, 2.51008E+01, 5.5801E+00, 6.5699E+03, &
      2.4910893E-01, 2.51292E+01, 5.5964E+00, 6.5564E+03, &
      2.5035448E-01, 2.51523E+01, 5.6131E+00, 6.5432E+03, &
      2.5160625E-01, 2.51719E+01, 5.6301E+00, 6.5304E+03, &
      2.5286428E-01, 2.51888E+01, 5.6474E+00, 6.5179E+03, &
      2.5412860E-01, 2.52036E+01, 5.6650E+00, 6.5057E+03, &
      2.5539925E-01, 2.52168E+01, 5.6829E+00, 6.4938E+03, &
      2.5667624E-01, 2.52287E+01, 5.7011E+00, 6.4822E+03, &
      2.5795962E-01, 2.52394E+01, 5.7196E+00, 6.4709E+03, &
      2.5924942E-01, 2.52493E+01, 5.7384E+00, 6.4599E+03, &
      2.6054567E-01, 2.52583E+01, 5.7575E+00, 6.4492E+03, &
      2.6184840E-01, 2.52666E+01, 5.7769E+00, 6.4387E+03, &
      2.6315764E-01, 2.52744E+01, 5.7966E+00, 6.4285E+03, &
      2.6447343E-01, 2.52816E+01, 5.8165E+00, 6.4186E+03, &
      2.6579579E-01, 2.52883E+01, 5.8368E+00, 6.4089E+03, &
      2.6712477E-01, 2.52947E+01, 5.8573E+00, 6.3995E+03, &
      2.6846040E-01, 2.53006E+01, 5.8781E+00, 6.3903E+03, &
      2.6980270E-01, 2.53063E+01, 5.8992E+00, 6.3813E+03, &
      2.7115171E-01, 2.53116E+01, 5.9205E+00, 6.3725E+03, &
      2.7250747E-01, 2.53167E+01, 5.9421E+00, 6.3640E+03, &
      2.7387001E-01, 2.53215E+01, 5.9639E+00, 6.3556E+03, &
      2.7523936E-01, 2.53260E+01, 5.9860E+00, 6.3474E+03, &
      2.7661556E-01, 2.53304E+01, 6.0084E+00, 6.3394E+03, &
      2.7799863E-01, 2.53345E+01, 6.0310E+00, 6.3316E+03, &
      2.7938863E-01, 2.53384E+01, 6.0538E+00, 6.3240E+03, &
      2.8078557E-01, 2.53420E+01, 6.0768E+00, 6.3165E+03, &
      2.8218950E-01, 2.53454E+01, 6.1001E+00, 6.3092E+03, &
      2.8360044E-01, 2.53486E+01, 6.1236E+00, 6.3020E+03, &
      2.8501845E-01, 2.53516E+01, 6.1473E+00, 6.2949E+03, &
      2.8644354E-01, 2.53542E+01, 6.1712E+00, 6.2880E+03, &
      2.8787576E-01, 2.53565E+01, 6.1953E+00, 6.2812E+03, &
      2.8931514E-01, 2.53585E+01, 6.2196E+00, 6.2745E+03, &
      2.9076171E-01, 2.53601E+01, 6.2441E+00, 6.2679E+03, &
      2.9221552E-01, 2.53612E+01, 6.2688E+00, 6.2614E+03, &
      2.9367660E-01, 2.53618E+01, 6.2937E+00, 6.2549E+03, &
      2.9514498E-01, 2.53618E+01, 6.3187E+00, 6.2486E+03, &
      2.9662071E-01, 2.53610E+01, 6.3439E+00, 6.2423E+03, &
      2.9810381E-01, 2.53593E+01, 6.3692E+00, 6.2361E+03, &
      2.9959433E-01, 2.53565E+01, 6.3947E+00, 6.2299E+03, &
      3.0109230E-01, 2.53522E+01, 6.4204E+00, 6.2238E+03, &
      3.0259776E-01, 2.53463E+01, 6.4461E+00, 6.2177E+03, &
      3.0411075E-01, 2.53381E+01, 6.4720E+00, 6.2117E+03, &
      3.0563130E-01, 2.53268E+01, 6.4980E+00, 6.2056E+03, &
      3.0715946E-01, 2.53114E+01, 6.5242E+00, 6.1996E+03, &
      3.0869526E-01, 2.52900E+01, 6.5504E+00, 6.1936E+03, &
      3.1023873E-01, 2.52590E+01, 6.5767E+00, 6.1875E+03, &
      3.1178993E-01, 2.52109E+01, 6.6031E+00, 6.1815E+03, &
      3.1334888E-01, 2.51239E+01, 6.6296E+00, 6.1754E+03, &
      3.1491562E-01, 2.48279E+01, 6.6562E+00, 6.1693E+03, &
      3.1551520E-01, 2.48503E+01, 7.1864E+00, 6.6479E+03, &
      3.1649020E-01, 2.50924E+01, 7.2036E+00, 6.6433E+03, &
      3.1807265E-01, 2.52399E+01, 7.2314E+00, 6.6358E+03, &
      3.1966301E-01, 2.53283E+01, 7.2592E+00, 6.6282E+03, &
      3.2126133E-01, 2.53949E+01, 7.2871E+00, 6.6206E+03, &
      3.2286764E-01, 2.54502E+01, 7.3150E+00, 6.6129E+03, &
      3.2448197E-01, 2.54986E+01, 7.3429E+00, 6.6051E+03, &
      3.2610438E-01, 2.55425E+01, 7.3709E+00, 6.5973E+03, &
      3.2773491E-01, 2.55832E+01, 7.3988E+00, 6.5894E+03, &
      3.2937358E-01, 2.56215E+01, 7.4268E+00, 6.5814E+03, &
      3.3102045E-01, 2.56580E+01, 7.4547E+00, 6.5733E+03, &
      3.3267555E-01, 2.56932E+01, 7.4826E+00, 6.5651E+03, &
      3.3433893E-01, 2.57274E+01, 7.5105E+00, 6.5568E+03, &
      3.3601062E-01, 2.57607E+01, 7.5384E+00, 6.5484E+03, &
      3.3769068E-01, 2.57934E+01, 7.5662E+00, 6.5399E+03, &
      3.3937913E-01, 2.58256E+01, 7.5939E+00, 6.5312E+03, &
      3.4107602E-01, 2.58574E+01, 7.6216E+00, 6.5225E+03, &
      3.4278140E-01, 2.58890E+01, 7.6492E+00, 6.5135E+03, &
      3.4449531E-01, 2.59203E+01, 7.6767E+00, 6.5045E+03, &
      3.4621779E-01, 2.59515E+01, 7.7041E+00, 6.4953E+03, &
      3.4794888E-01, 2.59826E+01, 7.7314E+00, 6.4859E+03, &
      3.4968862E-01, 2.60136E+01, 7.7586E+00, 6.4763E+03, &
      3.5143706E-01, 2.60446E+01, 7.7857E+00, 6.4666E+03, &
      3.5319425E-01, 2.60752E+01, 7.8126E+00, 6.4567E+03, &
      3.5496022E-01, 2.61063E+01, 7.8393E+00, 6.4466E+03, &
      3.5673502E-01, 2.61375E+01, 7.8659E+00, 6.4363E+03, &
      3.5851870E-01, 2.61687E+01, 7.8923E+00, 6.4258E+03, &
      3.6031129E-01, 2.62000E+01, 7.9185E+00, 6.4151E+03, &
      3.6211285E-01, 2.62314E+01, 7.9445E+00, 6.4042E+03, &
      3.6392341E-01, 2.62629E+01, 7.9704E+00, 6.3931E+03, &
      3.6574303E-01, 2.62945E+01, 7.9960E+00, 6.3817E+03, &
      3.6757174E-01, 2.63263E+01, 8.0214E+00, 6.3702E+03, &
      3.6940960E-01, 2.63582E+01, 8.0465E+00, 6.3584E+03, &
      3.7125665E-01, 2.63902E+01, 8.0715E+00, 6.3464E+03, &
      3.7311293E-01, 2.64224E+01, 8.0962E+00, 6.3342E+03, &
      3.7497850E-01, 2.64547E+01, 8.1206E+00, 6.3217E+03, &
      3.7685339E-01, 2.64872E+01, 8.1448E+00, 6.3090E+03, &
      3.7873766E-01, 2.65198E+01, 8.1687E+00, 6.2961E+03, &
      3.8063135E-01, 2.65526E+01, 8.1924E+00, 6.2829E+03, &
      3.8253450E-01, 2.65855E+01, 8.2157E+00, 6.2695E+03, &
      3.8444718E-01, 2.66186E+01, 8.2388E+00, 6.2559E+03, &
      3.8636941E-01, 2.66518E+01, 8.2616E+00, 6.2420E+03, &
      3.8830126E-01, 2.66852E+01, 8.2841E+00, 6.2279E+03, &
      3.9024276E-01, 2.67187E+01, 8.3062E+00, 6.2135E+03, &
      3.9219398E-01, 2.67523E+01, 8.3281E+00, 6.1989E+03, &
      3.9415495E-01, 2.67861E+01, 8.3496E+00, 6.1841E+03, &
      3.9612572E-01, 2.68200E+01, 8.3708E+00, 6.1690E+03, &
      3.9810635E-01, 2.68540E+01, 8.3916E+00, 6.1535E+03, &
      4.0009688E-01, 2.68882E+01, 8.4120E+00, 6.1378E+03, &
      4.0209737E-01, 2.69224E+01, 8.4321E+00, 6.1219E+03, &
      4.0410785E-01, 2.69568E+01, 8.4518E+00, 6.1057E+03, &
      4.0612839E-01, 2.69912E+01, 8.4712E+00, 6.0893E+03, &
      4.0815904E-01, 2.70258E+01, 8.4902E+00, 6.0726E+03, &
      4.1019983E-01, 2.70604E+01, 8.5088E+00, 6.0557E+03, &
      4.1225083E-01, 2.70951E+01, 8.5271E+00, 6.0385E+03, &
      4.1431208E-01, 2.71299E+01, 8.5449E+00, 6.0211E+03, &
      4.1638364E-01, 2.71647E+01, 8.5624E+00, 6.0035E+03, &
      4.1846556E-01, 2.71997E+01, 8.5795E+00, 5.9855E+03, &
      4.2055789E-01, 2.72346E+01, 8.5962E+00, 5.9674E+03, &
      4.2266068E-01, 2.72696E+01, 8.6125E+00, 5.9490E+03, &
      4.2477398E-01, 2.73046E+01, 8.6284E+00, 5.9304E+03, &
      4.2689785E-01, 2.73397E+01, 8.6439E+00, 5.9115E+03, &
      4.2903234E-01, 2.73747E+01, 8.6590E+00, 5.8924E+03, &
      4.3117750E-01, 2.74098E+01, 8.6736E+00, 5.8730E+03, &
      4.3333339E-01, 2.74449E+01, 8.6879E+00, 5.8534E+03, &
      4.3550006E-01, 2.74800E+01, 8.7017E+00, 5.8336E+03, &
      4.3767756E-01, 2.75150E+01, 8.7151E+00, 5.8136E+03, &
      4.3986595E-01, 2.75500E+01, 8.7281E+00, 5.7933E+03, &
      4.4206528E-01, 2.75850E+01, 8.7406E+00, 5.7728E+03, &
      4.4427560E-01, 2.76199E+01, 8.7527E+00, 5.7521E+03, &
      4.4649698E-01, 2.76548E+01, 8.7644E+00, 5.7311E+03, &
      4.4872947E-01, 2.76896E+01, 8.7756E+00, 5.7099E+03, &
      4.5097311E-01, 2.77244E+01, 8.7864E+00, 5.6886E+03, &
      4.5322798E-01, 2.77590E+01, 8.7967E+00, 5.6670E+03, &
      4.5549412E-01, 2.77936E+01, 8.8066E+00, 5.6451E+03, &
      4.5777159E-01, 2.78281E+01, 8.8161E+00, 5.6231E+03, &
      4.6006045E-01, 2.78624E+01, 8.8251E+00, 5.6009E+03, &
      4.6236075E-01, 2.78967E+01, 8.8337E+00, 5.5785E+03, &
      4.6467255E-01, 2.79308E+01, 8.8418E+00, 5.5559E+03, &
      4.6699592E-01, 2.79647E+01, 8.8494E+00, 5.5331E+03, &
      4.6933090E-01, 2.79985E+01, 8.8566E+00, 5.5101E+03, &
      4.7167755E-01, 2.80322E+01, 8.8634E+00, 5.4869E+03, &
      4.7403594E-01, 2.80657E+01, 8.8696E+00, 5.4634E+03, &
      4.7640612E-01, 2.80990E+01, 8.8752E+00, 5.4397E+03, &
      4.7878815E-01, 2.81320E+01, 8.8803E+00, 5.4158E+03, &
      4.8118209E-01, 2.81649E+01, 8.8849E+00, 5.3917E+03, &
      4.8358800E-01, 2.81975E+01, 8.8889E+00, 5.3674E+03, &
      4.8600594E-01, 2.82298E+01, 8.8925E+00, 5.3428E+03, &
      4.8843597E-01, 2.82619E+01, 8.8954E+00, 5.3181E+03, &
      4.9087815E-01, 2.82936E+01, 8.8979E+00, 5.2931E+03, &
      4.9333254E-01, 2.83250E+01, 8.8998E+00, 5.2680E+03, &
      4.9579920E-01, 2.83561E+01, 8.9012E+00, 5.2427E+03, &
      4.9827820E-01, 2.83869E+01, 8.9021E+00, 5.2171E+03, &
      5.0076959E-01, 2.84172E+01, 8.9025E+00, 5.1915E+03, &
      5.0327344E-01, 2.84472E+01, 8.9024E+00, 5.1656E+03, &
      5.0578980E-01, 2.84768E+01, 8.9018E+00, 5.1396E+03, &
      5.0831875E-01, 2.85060E+01, 8.9007E+00, 5.1134E+03, &
      5.1086035E-01, 2.85348E+01, 8.8990E+00, 5.0871E+03, &
      5.1341465E-01, 2.85630E+01, 8.8969E+00, 5.0607E+03, &
      5.1598172E-01, 2.85909E+01, 8.8943E+00, 5.0341E+03, &
      5.1856163E-01, 2.86182E+01, 8.8913E+00, 5.0073E+03, &
      5.2115444E-01, 2.86451E+01, 8.8877E+00, 4.9805E+03, &
      5.2376021E-01, 2.86715E+01, 8.8837E+00, 4.9535E+03, &
      5.2637901E-01, 2.86973E+01, 8.8792E+00, 4.9264E+03, &
      5.2901091E-01, 2.87226E+01, 8.8742E+00, 4.8992E+03, &
      5.3165596E-01, 2.87473E+01, 8.8688E+00, 4.8719E+03, &
      5.3431424E-01, 2.87715E+01, 8.8630E+00, 4.8445E+03, &
      5.3698581E-01, 2.87950E+01, 8.8566E+00, 4.8171E+03, &
      5.3967074E-01, 2.88180E+01, 8.8499E+00, 4.7895E+03, &
      5.4236910E-01, 2.88404E+01, 8.8427E+00, 4.7618E+03, &
      5.4508094E-01, 2.88621E+01, 8.8351E+00, 4.7341E+03, &
      5.4780635E-01, 2.88832E+01, 8.8271E+00, 4.7063E+03, &
      5.5054538E-01, 2.89036E+01, 8.8186E+00, 4.6785E+03, &
      5.5329810E-01, 2.89233E+01, 8.8097E+00, 4.6506E+03, &
      5.5606460E-01, 2.89424E+01, 8.8005E+00, 4.6226E+03, &
      5.5884492E-01, 2.89607E+01, 8.7908E+00, 4.5947E+03, &
      5.6163914E-01, 2.89783E+01, 8.7807E+00, 4.5666E+03, &
      5.6444734E-01, 2.89952E+01, 8.7703E+00, 4.5385E+03, &
      5.6726958E-01, 2.90113E+01, 8.7595E+00, 4.5104E+03, &
      5.7010592E-01, 2.90267E+01, 8.7483E+00, 4.4823E+03, &
      5.7295645E-01, 2.90412E+01, 8.7367E+00, 4.4542E+03, &
      5.7582123E-01, 2.90549E+01, 8.7247E+00, 4.4260E+03, &
      5.7870034E-01, 2.90679E+01, 8.7124E+00, 4.3978E+03, &
      5.8159384E-01, 2.90800E+01, 8.6998E+00, 4.3697E+03, &
      5.8450181E-01, 2.90912E+01, 8.6868E+00, 4.3415E+03, &
      5.8742432E-01, 2.91015E+01, 8.6735E+00, 4.3133E+03, &
      5.9036144E-01, 2.91110E+01, 8.6598E+00, 4.2852E+03, &
      5.9331325E-01, 2.91195E+01, 8.6458E+00, 4.2570E+03, &
      5.9627982E-01, 2.91272E+01, 8.6315E+00, 4.2289E+03, &
      5.9926122E-01, 2.91338E+01, 8.6169E+00, 4.2008E+03, &
      6.0225752E-01, 2.91395E+01, 8.6020E+00, 4.1727E+03, &
      6.0526881E-01, 2.91443E+01, 8.5867E+00, 4.1446E+03, &
      6.0829515E-01, 2.91480E+01, 8.5712E+00, 4.1166E+03, &
      6.1133663E-01, 2.91507E+01, 8.5554E+00, 4.0887E+03, &
      6.1439331E-01, 2.91523E+01, 8.5393E+00, 4.0607E+03, &
      6.1746528E-01, 2.91529E+01, 8.5229E+00, 4.0328E+03, &
      6.2055260E-01, 2.91524E+01, 8.5063E+00, 4.0050E+03, &
      6.2365537E-01, 2.91508E+01, 8.4894E+00, 3.9772E+03, &
      6.2677364E-01, 2.91481E+01, 8.4722E+00, 3.9495E+03, &
      6.2990751E-01, 2.91442E+01, 8.4548E+00, 3.9218E+03, &
      6.3305705E-01, 2.91391E+01, 8.4371E+00, 3.8942E+03, &
      6.3622234E-01, 2.91328E+01, 8.4192E+00, 3.8667E+03, &
      6.3940345E-01, 2.91253E+01, 8.4010E+00, 3.8392E+03, &
      6.4260046E-01, 2.91166E+01, 8.3826E+00, 3.8118E+03, &
      6.4581347E-01, 2.91066E+01, 8.3640E+00, 3.7845E+03, &
      6.4904253E-01, 2.90952E+01, 8.3452E+00, 3.7572E+03, &
      6.5228775E-01, 2.90825E+01, 8.3262E+00, 3.7301E+03, &
      6.5554919E-01, 2.90685E+01, 8.3069E+00, 3.7030E+03, &
      6.5882693E-01, 2.90531E+01, 8.2875E+00, 3.6760E+03, &
      6.6212107E-01, 2.90362E+01, 8.2678E+00, 3.6491E+03, &
      6.6543167E-01, 2.90179E+01, 8.2480E+00, 3.6223E+03, &
      6.6875883E-01, 2.89980E+01, 8.2280E+00, 3.5956E+03, &
      6.7210262E-01, 2.89767E+01, 8.2078E+00, 3.5690E+03, &
      6.7546314E-01, 2.89537E+01, 8.1874E+00, 3.5425E+03, &
      6.7884045E-01, 2.89292E+01, 8.1668E+00, 3.5161E+03, &
      6.8223466E-01, 2.89030E+01, 8.1461E+00, 3.4898E+03, &
      6.8564583E-01, 2.88752E+01, 8.1253E+00, 3.4636E+03, &
      6.8907406E-01, 2.88456E+01, 8.1042E+00, 3.4376E+03, &
      6.9251943E-01, 2.88143E+01, 8.0830E+00, 3.4116E+03, &
      6.9598202E-01, 2.87811E+01, 8.0617E+00, 3.3857E+03, &
      6.9946194E-01, 2.87461E+01, 8.0403E+00, 3.3600E+03, &
      7.0295924E-01, 2.87091E+01, 8.0187E+00, 3.3344E+03, &
      7.0647404E-01, 2.86702E+01, 7.9969E+00, 3.3088E+03, &
      7.1000641E-01, 2.86293E+01, 7.9750E+00, 3.2834E+03, &
      7.1355644E-01, 2.85863E+01, 7.9530E+00, 3.2581E+03, &
      7.1712423E-01, 2.85411E+01, 7.9308E+00, 3.2330E+03, &
      7.2070985E-01, 2.84937E+01, 7.9086E+00, 3.2079E+03, &
      7.2431340E-01, 2.84440E+01, 7.8862E+00, 3.1830E+03, &
      7.2793496E-01, 2.83920E+01, 7.8638E+00, 3.1582E+03, &
      7.3157464E-01, 2.83396E+01, 7.8412E+00, 3.1336E+03, &
      7.3523251E-01, 2.82827E+01, 7.8185E+00, 3.1090E+03, &
      7.3890867E-01, 2.82232E+01, 7.7958E+00, 3.0846E+03, &
      7.4260322E-01, 2.81610E+01, 7.7729E+00, 3.0604E+03, &
      7.4631623E-01, 2.80961E+01, 7.7500E+00, 3.0362E+03, &
      7.5004781E-01, 2.80283E+01, 7.7270E+00, 3.0122E+03, &
      7.5379805E-01, 2.79575E+01, 7.7039E+00, 2.9884E+03, &
      7.5756704E-01, 2.78837E+01, 7.6807E+00, 2.9646E+03, &
      7.6135488E-01, 2.78066E+01, 7.6575E+00, 2.9410E+03, &
      7.6516165E-01, 2.77263E+01, 7.6342E+00, 2.9176E+03, &
      7.6898746E-01, 2.76425E+01, 7.6109E+00, 2.8943E+03, &
      7.7283240E-01, 2.75551E+01, 7.5874E+00, 2.8711E+03, &
      7.7669656E-01, 2.74640E+01, 7.5640E+00, 2.8480E+03, &
      7.8058004E-01, 2.73691E+01, 7.5405E+00, 2.8251E+03, &
      7.8448294E-01, 2.72700E+01, 7.5169E+00, 2.8023E+03, &
      7.8840536E-01, 2.71668E+01, 7.4933E+00, 2.7797E+03, &
      7.9234738E-01, 2.70591E+01, 7.4696E+00, 2.7572E+03, &
      7.9630912E-01, 2.69467E+01, 7.4459E+00, 2.7349E+03, &
      8.0029067E-01, 2.68295E+01, 7.4222E+00, 2.7127E+03, &
      8.0429212E-01, 2.67072E+01, 7.3984E+00, 2.6906E+03, &
      8.0831358E-01, 2.65796E+01, 7.3746E+00, 2.6687E+03, &
      8.1235515E-01, 2.64463E+01, 7.3508E+00, 2.6469E+03, &
      8.1641693E-01, 2.63071E+01, 7.3269E+00, 2.6253E+03, &
      8.2049901E-01, 2.61616E+01, 7.3031E+00, 2.6038E+03, &
      8.2460150E-01, 2.60095E+01, 7.2792E+00, 2.5824E+03, &
      8.2872451E-01, 2.58504E+01, 7.2553E+00, 2.5612E+03, &
      8.3286813E-01, 2.56839E+01, 7.2314E+00, 2.5401E+03, &
      8.3703248E-01, 2.55094E+01, 7.2074E+00, 2.5192E+03, &
      8.4121764E-01, 2.53266E+01, 7.1835E+00, 2.4984E+03, &
      8.4542373E-01, 2.51348E+01, 7.1595E+00, 2.4778E+03, &
      8.4965084E-01, 2.49334E+01, 7.1356E+00, 2.4573E+03, &
      8.5389910E-01, 2.47218E+01, 7.1116E+00, 2.4369E+03, &
      8.5816859E-01, 2.44991E+01, 7.0877E+00, 2.4167E+03, &
      8.6245944E-01, 2.42646E+01, 7.0637E+00, 2.3967E+03, &
      8.6677173E-01, 2.40172E+01, 7.0398E+00, 2.3767E+03, &
      8.7110559E-01, 2.37560E+01, 7.0158E+00, 2.3569E+03, &
      8.7546112E-01, 2.34797E+01, 6.9919E+00, 2.3373E+03, &
      8.7983843E-01, 2.31869E+01, 6.9680E+00, 2.3178E+03, &
      8.8423762E-01, 2.28761E+01, 6.9441E+00, 2.2984E+03, &
      8.8865881E-01, 2.25455E+01, 6.9202E+00, 2.2792E+03, &
      8.9310210E-01, 2.21930E+01, 6.8963E+00, 2.2601E+03, &
      8.9756761E-01, 2.18163E+01, 6.8724E+00, 2.2412E+03, &
      9.0205545E-01, 2.14123E+01, 6.8486E+00, 2.2223E+03, &
      9.0656573E-01, 2.09779E+01, 6.8248E+00, 2.2037E+03, &
      9.1109856E-01, 2.05089E+01, 6.8010E+00, 2.1851E+03, &
      9.1565405E-01, 2.00005E+01, 6.7772E+00, 2.1667E+03, &
      9.2023232E-01, 1.94466E+01, 6.7534E+00, 2.1485E+03, &
      9.2483348E-01, 1.88399E+01, 6.7297E+00, 2.1304E+03, &
      9.2945765E-01, 1.81709E+01, 6.7060E+00, 2.1124E+03, &
      9.3410494E-01, 1.74272E+01, 6.6823E+00, 2.0945E+03, &
      9.3877546E-01, 1.65924E+01, 6.6586E+00, 2.0768E+03, &
      9.4346934E-01, 1.56442E+01, 6.6350E+00, 2.0592E+03, &
      9.4818668E-01, 1.45504E+01, 6.6115E+00, 2.0418E+03, &
      9.5292762E-01, 1.32624E+01, 6.5879E+00, 2.0245E+03, &
      9.5769226E-01, 1.17018E+01, 6.5644E+00, 2.0073E+03, &
      9.6248072E-01, 9.72743E+00, 6.5410E+00, 1.9903E+03, &
      9.6729312E-01, 7.03962E+00, 6.5176E+00, 1.9733E+03, &
      9.7212959E-01, 2.76831E+00, 6.4942E+00, 1.9566E+03, &
      9.7699023E-01, -1.04456E+01, 6.4709E+00, 1.9399E+03, &
      9.7760223E-01, -2.27447E+01, 6.4680E+00, 1.9379E+03, &
      9.7779777E-01, -2.27620E+01, 2.5854E+01, 7.7216E+03, &
      9.8187519E-01, -1.69924E-01, 2.5714E+01, 7.6480E+03, &
      9.8678456E-01, 3.55695E+00, 2.5547E+01, 7.5606E+03, &
      9.9171848E-01, 4.46341E+00, 2.5381E+01, 7.4742E+03, &
      9.9667708E-01, 2.40080E+00, 2.5217E+01, 7.3889E+03, &
      9.9936105E-01, -8.95807E+00, 2.5128E+01, 7.3433E+03, &
      9.9963891E-01, -8.87175E+00, 3.7894E+01, 1.1067E+04, &
      1.0016605E+00, 3.16228E+00, 3.8008E+01, 1.1078E+04, &
      1.0066688E+00, 9.52744E+00, 3.7726E+01, 1.0941E+04, &
      1.0117021E+00, 1.29304E+01, 3.7445E+01, 1.0806E+04, &
      1.0167606E+00, 1.54145E+01, 3.7166E+01, 1.0672E+04, &
      1.0218444E+00, 1.74130E+01, 3.6890E+01, 1.0540E+04, &
      1.0269536E+00, 1.91007E+01, 3.6616E+01, 1.0410E+04, &
      1.0320884E+00, 2.05681E+01, 3.6343E+01, 1.0281E+04, &
      1.0372489E+00, 2.18694E+01, 3.6073E+01, 1.0154E+04, &
      1.0424351E+00, 2.30399E+01, 3.5805E+01, 1.0028E+04, &
      1.0476473E+00, 2.41040E+01, 3.5539E+01, 9.9044E+03, &
      1.0528855E+00, 2.50795E+01, 3.5275E+01, 9.7820E+03, &
      1.0581499E+00, 2.59799E+01, 3.5013E+01, 9.6612E+03, &
      1.0634407E+00, 2.68155E+01, 3.4753E+01, 9.5418E+03, &
      1.0687579E+00, 2.75945E+01, 3.4495E+01, 9.4239E+03, &
      1.0741017E+00, 2.83237E+01, 3.4239E+01, 9.3075E+03, &
      1.0794722E+00, 2.90083E+01, 3.3985E+01, 9.1926E+03, &
      1.0848695E+00, 2.96529E+01, 3.3732E+01, 9.0791E+03, &
      1.0902939E+00, 3.02614E+01, 3.3482E+01, 8.9670E+03, &
      1.0957454E+00, 3.08368E+01, 3.3234E+01, 8.8563E+03, &
      1.1012241E+00, 3.13819E+01, 3.2987E+01, 8.7470E+03, &
      1.1067302E+00, 3.18992E+01, 3.2742E+01, 8.6390E+03, &
      1.1122639E+00, 3.23905E+01, 3.2500E+01, 8.5324E+03, &
      1.1178252E+00, 3.28578E+01, 3.2259E+01, 8.4271E+03, &
      1.1234143E+00, 3.33025E+01, 3.2020E+01, 8.3231E+03, &
      1.1290314E+00, 3.37260E+01, 3.1782E+01, 8.2205E+03, &
      1.1346765E+00, 3.41294E+01, 3.1547E+01, 8.1191E+03, &
      1.1403499E+00, 3.45139E+01, 3.1313E+01, 8.0189E+03, &
      1.1460517E+00, 3.48802E+01, 3.1081E+01, 7.9201E+03, &
      1.1517819E+00, 3.52293E+01, 3.0851E+01, 7.8224E+03, &
      1.1575408E+00, 3.55617E+01, 3.0622E+01, 7.7260E+03, &
      1.1633285E+00, 3.58780E+01, 3.0396E+01, 7.6307E+03, &
      1.1691452E+00, 3.61787E+01, 3.0171E+01, 7.5367E+03, &
      1.1749909E+00, 3.64642E+01, 2.9948E+01, 7.4438E+03, &
      1.1808659E+00, 3.67347E+01, 2.9726E+01, 7.3521E+03, &
      1.1867702E+00, 3.69904E+01, 2.9506E+01, 7.2615E+03, &
      1.1927040E+00, 3.72315E+01, 2.9288E+01, 7.1720E+03, &
      1.1986676E+00, 3.74577E+01, 2.9071E+01, 7.0837E+03, &
      1.2046609E+00, 3.76690E+01, 2.8856E+01, 6.9964E+03, &
      1.2106842E+00, 3.78650E+01, 2.8643E+01, 6.9103E+03, &
      1.2167376E+00, 3.80451E+01, 2.8431E+01, 6.8252E+03, &
      1.2228213E+00, 3.82085E+01, 2.8221E+01, 6.7412E+03, &
      1.2289354E+00, 3.83540E+01, 2.8013E+01, 6.6582E+03, &
      1.2350801E+00, 3.84802E+01, 2.7806E+01, 6.5762E+03, &
      1.2412555E+00, 3.85848E+01, 2.7601E+01, 6.4953E+03, &
      1.2474618E+00, 3.86646E+01, 2.7397E+01, 6.4154E+03, &
      1.2536991E+00, 3.87151E+01, 2.7195E+01, 6.3365E+03, &
      1.2599676E+00, 3.87296E+01, 2.6994E+01, 6.2585E+03, &
      1.2662674E+00, 3.86970E+01, 2.6794E+01, 6.1815E+03, &
      1.2725988E+00, 3.85988E+01, 2.6596E+01, 6.1054E+03, &
      1.2789618E+00, 3.83993E+01, 2.6400E+01, 6.0303E+03, &
      1.2853566E+00, 3.80146E+01, 2.6205E+01, 5.9561E+03, &
      1.2917833E+00, 3.71555E+01, 2.6012E+01, 5.8828E+03, &
      1.2965165E+00, 3.47103E+01, 2.5871E+01, 5.8296E+03, &
      1.2982423E+00, 3.46325E+01, 3.0254E+01, 6.8068E+03, &
      1.2982835E+00, 3.47017E+01, 3.0252E+01, 6.8062E+03, &
      1.3047335E+00, 3.79088E+01, 3.0007E+01, 6.7179E+03, &
      1.3112571E+00, 3.89968E+01, 2.9763E+01, 6.6303E+03, &
      1.3178134E+00, 3.97146E+01, 2.9521E+01, 6.5438E+03, &
      1.3244025E+00, 4.02643E+01, 2.9282E+01, 6.4584E+03, &
      1.3310245E+00, 4.07135E+01, 2.9044E+01, 6.3743E+03, &
      1.3376796E+00, 4.10926E+01, 2.8808E+01, 6.2912E+03, &
      1.3443680E+00, 4.14174E+01, 2.8575E+01, 6.2092E+03, &
      1.3510899E+00, 4.16960E+01, 2.8343E+01, 6.1284E+03, &
      1.3578453E+00, 4.19314E+01, 2.8113E+01, 6.0486E+03, &
      1.3646345E+00, 4.21234E+01, 2.7897E+01, 5.9724E+03, &
      1.3714577E+00, 4.22814E+01, 2.7695E+01, 5.8997E+03, &
      1.3783150E+00, 4.23984E+01, 2.7496E+01, 5.8283E+03, &
      1.3852066E+00, 4.24558E+01, 2.7300E+01, 5.7581E+03, &
      1.3921326E+00, 4.24046E+01, 2.7107E+01, 5.6889E+03, &
      1.3990933E+00, 4.20209E+01, 2.6916E+01, 5.6209E+03, &
      1.4018615E+00, 4.13093E+01, 2.6841E+01, 5.5943E+03, &
      1.4037384E+00, 4.13414E+01, 2.8630E+01, 5.9585E+03, &
      1.4060887E+00, 4.21539E+01, 2.8560E+01, 5.9341E+03, &
      1.4131192E+00, 4.30410E+01, 2.8354E+01, 5.8622E+03, &
      1.4201848E+00, 4.35497E+01, 2.8151E+01, 5.7913E+03, &
      1.4272857E+00, 4.39423E+01, 2.7951E+01, 5.7216E+03, &
      1.4344221E+00, 4.42742E+01, 2.7753E+01, 5.6529E+03, &
      1.4415942E+00, 4.45668E+01, 2.7557E+01, 5.5853E+03, &
      1.4488022E+00, 4.48307E+01, 2.7364E+01, 5.5187E+03, &
      1.4560462E+00, 4.50718E+01, 2.7174E+01, 5.4531E+03, &
      1.4633265E+00, 4.52936E+01, 2.6985E+01, 5.3884E+03, &
      1.4706431E+00, 4.54979E+01, 2.6799E+01, 5.3247E+03, &
      1.4779963E+00, 4.56869E+01, 2.6623E+01, 5.2635E+03, &
      1.4853863E+00, 4.58662E+01, 2.6451E+01, 5.2036E+03, &
      1.4928132E+00, 4.60356E+01, 2.6282E+01, 5.1446E+03, &
      1.5002773E+00, 4.61952E+01, 2.6115E+01, 5.0866E+03, &
      1.5077787E+00, 4.63447E+01, 2.5950E+01, 5.0295E+03, &
      1.5153176E+00, 4.64839E+01, 2.5787E+01, 4.9732E+03, &
      1.5228942E+00, 4.66118E+01, 2.5627E+01, 4.9178E+03, &
      1.5305086E+00, 4.67269E+01, 2.5469E+01, 4.8632E+03, &
      1.5381612E+00, 4.68266E+01, 2.5312E+01, 4.8094E+03, &
      1.5458520E+00, 4.69058E+01, 2.5158E+01, 4.7564E+03, &
      1.5535812E+00, 4.69546E+01, 2.5006E+01, 4.7042E+03, &
      1.5613491E+00, 4.69488E+01, 2.4855E+01, 4.6527E+03, &
      1.5691559E+00, 4.67997E+01, 2.4707E+01, 4.6020E+03, &
      1.5732678E+00, 4.64683E+01, 2.4630E+01, 4.5757E+03, &
      1.5770017E+00, 4.64656E+01, 2.5733E+01, 4.7689E+03, &
      1.5773321E+00, 4.65384E+01, 2.5726E+01, 4.7668E+03, &
      1.5848867E+00, 4.72640E+01, 2.5583E+01, 4.7177E+03, &
      1.5928111E+00, 4.76379E+01, 2.5435E+01, 4.6672E+03, &
      1.6007752E+00, 4.79239E+01, 2.5289E+01, 4.6174E+03, &
      1.6087790E+00, 4.81694E+01, 2.5144E+01, 4.5682E+03, &
      1.6168229E+00, 4.83912E+01, 2.5001E+01, 4.5197E+03, &
      1.6249070E+00, 4.85972E+01, 2.4859E+01, 4.4718E+03, &
      1.6330316E+00, 4.87917E+01, 2.4719E+01, 4.4245E+03, &
      1.6411967E+00, 4.89777E+01, 2.4580E+01, 4.3778E+03, &
      1.6494027E+00, 4.91569E+01, 2.4442E+01, 4.3317E+03, &
      1.6576497E+00, 4.93300E+01, 2.4301E+01, 4.2853E+03, &
      1.6659380E+00, 4.94977E+01, 2.4158E+01, 4.2390E+03, &
      1.6742677E+00, 4.96612E+01, 2.4016E+01, 4.1933E+03, &
      1.6826390E+00, 4.98214E+01, 2.3876E+01, 4.1482E+03, &
      1.6910522E+00, 4.99783E+01, 2.3731E+01, 4.1026E+03, &
      1.6995075E+00, 5.01293E+01, 2.3579E+01, 4.0561E+03, &
      1.7080050E+00, 5.02750E+01, 2.3428E+01, 4.0102E+03, &
      1.7165450E+00, 5.04161E+01, 2.3278E+01, 3.9648E+03, &
      1.7251278E+00, 5.05529E+01, 2.3129E+01, 3.9200E+03, &
      1.7337534E+00, 5.06858E+01, 2.2982E+01, 3.8756E+03, &
      1.7424222E+00, 5.08151E+01, 2.2835E+01, 3.8318E+03, &
      1.7511343E+00, 5.09411E+01, 2.2689E+01, 3.7885E+03, &
      1.7598899E+00, 5.10640E+01, 2.2544E+01, 3.7457E+03, &
      1.7686894E+00, 5.11841E+01, 2.2400E+01, 3.7034E+03, &
      1.7775328E+00, 5.13014E+01, 2.2257E+01, 3.6615E+03, &
      1.7864205E+00, 5.14163E+01, 2.2115E+01, 3.6202E+03, &
      1.7953526E+00, 5.15288E+01, 2.1974E+01, 3.5793E+03, &
      1.8043294E+00, 5.16391E+01, 2.1834E+01, 3.5388E+03, &
      1.8133510E+00, 5.17475E+01, 2.1695E+01, 3.4989E+03, &
      1.8224178E+00, 5.18542E+01, 2.1557E+01, 3.4593E+03, &
      1.8315299E+00, 5.19587E+01, 2.1416E+01, 3.4197E+03, &
      1.8406875E+00, 5.20603E+01, 2.1275E+01, 3.3804E+03, &
      1.8498909E+00, 5.21594E+01, 2.1135E+01, 3.3416E+03, &
      1.8591404E+00, 5.22561E+01, 2.0996E+01, 3.3032E+03, &
      1.8684361E+00, 5.23506E+01, 2.0857E+01, 3.2652E+03, &
      1.8777783E+00, 5.24428E+01, 2.0720E+01, 3.2276E+03, &
      1.8871672E+00, 5.25329E+01, 2.0583E+01, 3.1905E+03, &
      1.8966030E+00, 5.26211E+01, 2.0447E+01, 3.1537E+03, &
      1.9060860E+00, 5.27073E+01, 2.0312E+01, 3.1174E+03, &
      1.9156165E+00, 5.27916E+01, 2.0178E+01, 3.0815E+03, &
      1.9251945E+00, 5.28740E+01, 2.0045E+01, 3.0460E+03, &
      1.9348205E+00, 5.29548E+01, 1.9912E+01, 3.0108E+03, &
      1.9444946E+00, 5.30338E+01, 1.9780E+01, 2.9761E+03, &
      1.9542171E+00, 5.31111E+01, 1.9649E+01, 2.9418E+03, &
      1.9639882E+00, 5.31868E+01, 1.9519E+01, 2.9078E+03, &
      1.9738081E+00, 5.32609E+01, 1.9389E+01, 2.8742E+03, &
      1.9836772E+00, 5.33335E+01, 1.9260E+01, 2.8410E+03, &
      1.9935955E+00, 5.34046E+01, 1.9132E+01, 2.8082E+03, &
      2.0035635E+00, 5.34743E+01, 1.9005E+01, 2.7758E+03, &
      2.0135813E+00, 5.35425E+01, 1.8879E+01, 2.7437E+03, &
      2.0236492E+00, 5.36094E+01, 1.8753E+01, 2.7119E+03, &
      2.0337675E+00, 5.36750E+01, 1.8628E+01, 2.6806E+03, &
      2.0439363E+00, 5.37394E+01, 1.8504E+01, 2.6496E+03, &
      2.0541560E+00, 5.38025E+01, 1.8380E+01, 2.6188E+03, &
      2.0644268E+00, 5.38641E+01, 1.8257E+01, 2.5884E+03, &
      2.0747489E+00, 5.39243E+01, 1.8134E+01, 2.5583E+03, &
      2.0851227E+00, 5.39832E+01, 1.8012E+01, 2.5285E+03, &
      2.0955483E+00, 5.40407E+01, 1.7891E+01, 2.4991E+03, &
      2.1060260E+00, 5.40969E+01, 1.7770E+01, 2.4700E+03, &
      2.1165562E+00, 5.41520E+01, 1.7651E+01, 2.4413E+03, &
      2.1271389E+00, 5.42057E+01, 1.7532E+01, 2.4129E+03, &
      2.1377746E+00, 5.42583E+01, 1.7414E+01, 2.3848E+03, &
      2.1484635E+00, 5.43098E+01, 1.7296E+01, 2.3570E+03, &
      2.1592058E+00, 5.43601E+01, 1.7180E+01, 2.3296E+03, &
      2.1700018E+00, 5.44093E+01, 1.7064E+01, 2.3025E+03, &
      2.1808519E+00, 5.44575E+01, 1.6949E+01, 2.2757E+03, &
      2.1917561E+00, 5.45045E+01, 1.6835E+01, 2.2492E+03, &
      2.2027149E+00, 5.45506E+01, 1.6721E+01, 2.2230E+03, &
      2.2137285E+00, 5.45957E+01, 1.6607E+01, 2.1969E+03, &
      2.2247971E+00, 5.46397E+01, 1.6492E+01, 2.1710E+03, &
      2.2359211E+00, 5.46827E+01, 1.6379E+01, 2.1454E+03, &
      2.2471007E+00, 5.47246E+01, 1.6266E+01, 2.1201E+03, &
      2.2583362E+00, 5.47654E+01, 1.6154E+01, 2.0951E+03, &
      2.2696279E+00, 5.48053E+01, 1.6042E+01, 2.0704E+03, &
      2.2809760E+00, 5.48442E+01, 1.5932E+01, 2.0460E+03, &
      2.2923809E+00, 5.48821E+01, 1.5822E+01, 2.0219E+03, &
      2.3038428E+00, 5.49191E+01, 1.5713E+01, 1.9981E+03, &
      2.3153620E+00, 5.49552E+01, 1.5605E+01, 1.9745E+03, &
      2.3269388E+00, 5.49904E+01, 1.5498E+01, 1.9513E+03, &
      2.3385735E+00, 5.50247E+01, 1.5391E+01, 1.9283E+03, &
      2.3502664E+00, 5.50582E+01, 1.5286E+01, 1.9057E+03, &
      2.3620177E+00, 5.50909E+01, 1.5181E+01, 1.8832E+03, &
      2.3738278E+00, 5.51227E+01, 1.5077E+01, 1.8611E+03, &
      2.3856970E+00, 5.51538E+01, 1.4973E+01, 1.8393E+03, &
      2.3976254E+00, 5.51841E+01, 1.4871E+01, 1.8177E+03, &
      2.4096136E+00, 5.52137E+01, 1.4769E+01, 1.7963E+03, &
      2.4216616E+00, 5.52426E+01, 1.4668E+01, 1.7753E+03, &
      2.4337699E+00, 5.52707E+01, 1.4568E+01, 1.7545E+03, &
      2.4459388E+00, 5.52982E+01, 1.4469E+01, 1.7339E+03, &
      2.4581685E+00, 5.53251E+01, 1.4370E+01, 1.7136E+03, &
      2.4704593E+00, 5.53513E+01, 1.4272E+01, 1.6936E+03, &
      2.4828116E+00, 5.53768E+01, 1.4175E+01, 1.6738E+03, &
      2.4952257E+00, 5.54018E+01, 1.4079E+01, 1.6542E+03, &
      2.5077018E+00, 5.54262E+01, 1.3983E+01, 1.6349E+03, &
      2.5202403E+00, 5.54501E+01, 1.3888E+01, 1.6158E+03, &
      2.5328415E+00, 5.54734E+01, 1.3794E+01, 1.5970E+03, &
      2.5455057E+00, 5.54962E+01, 1.3701E+01, 1.5783E+03, &
      2.5582333E+00, 5.55185E+01, 1.3608E+01, 1.5600E+03, &
      2.5710244E+00, 5.55404E+01, 1.3517E+01, 1.5418E+03, &
      2.5838796E+00, 5.55618E+01, 1.3425E+01, 1.5239E+03, &
      2.5967990E+00, 5.56852E+01, 1.3335E+01, 1.5061E+03, &
      2.6097829E+00, 5.57063E+01, 1.3243E+01, 1.4884E+03, &
      2.6228319E+00, 5.57268E+01, 1.3152E+01, 1.4709E+03, &
      2.6359460E+00, 5.57467E+01, 1.3061E+01, 1.4536E+03, &
      2.6491257E+00, 5.57661E+01, 1.2972E+01, 1.4365E+03, &
      2.6623714E+00, 5.57849E+01, 1.2883E+01, 1.4197E+03, &
      2.6756832E+00, 5.58032E+01, 1.2794E+01, 1.4030E+03, &
      2.6890617E+00, 5.58211E+01, 1.2707E+01, 1.3865E+03, &
      2.7025070E+00, 5.58385E+01, 1.2620E+01, 1.3703E+03, &
      2.7160195E+00, 5.58555E+01, 1.2534E+01, 1.3542E+03, &
      2.7295996E+00, 5.58721E+01, 1.2448E+01, 1.3384E+03, &
      2.7432476E+00, 5.58884E+01, 1.2363E+01, 1.3227E+03, &
      2.7569638E+00, 5.59043E+01, 1.2279E+01, 1.3073E+03, &
      2.7707486E+00, 5.59200E+01, 1.2196E+01, 1.2920E+03, &
      2.7846024E+00, 5.59354E+01, 1.2113E+01, 1.2769E+03, &
      2.7985254E+00, 5.59506E+01, 1.2030E+01, 1.2620E+03, &
      2.8125180E+00, 5.60125E+01, 1.1948E+01, 1.2472E+03, &
      2.8265806E+00, 5.60276E+01, 1.1866E+01, 1.2326E+03, &
      2.8407135E+00, 5.60425E+01, 1.1785E+01, 1.2181E+03, &
      2.8549171E+00, 5.60573E+01, 1.1704E+01, 1.2039E+03, &
      2.8691917E+00, 5.60720E+01, 1.1624E+01, 1.1898E+03, &
      2.8835376E+00, 5.60868E+01, 1.1545E+01, 1.1758E+03, &
      2.8979553E+00, 5.61016E+01, 1.1466E+01, 1.1621E+03, &
      2.9124451E+00, 5.61167E+01, 1.1388E+01, 1.1485E+03, &
      2.9270073E+00, 5.61323E+01, 1.1310E+01, 1.1351E+03, &
      2.9416424E+00, 5.61642E+01, 1.1233E+01, 1.1218E+03, &
      2.9563506E+00, 5.61826E+01, 1.1157E+01, 1.1087E+03, &
      2.9711323E+00, 5.62027E+01, 1.1081E+01, 1.0958E+03, &
      2.9859880E+00, 5.62254E+01, 1.1006E+01, 1.0830E+03, &
      3.0009179E+00, 5.62662E+01, 1.0930E+01, 1.0703E+03, &
      3.0159225E+00, 5.62762E+01, 1.0844E+01, 1.0567E+03, &
      3.0310021E+00, 5.62847E+01, 1.0758E+01, 1.0432E+03, &
      3.0461571E+00, 5.62920E+01, 1.0674E+01, 1.0299E+03, &
      3.0613879E+00, 5.62981E+01, 1.0589E+01, 1.0168E+03, &
      3.0766949E+00, 5.63033E+01, 1.0506E+01, 1.0038E+03, &
      3.0920783E+00, 5.63075E+01, 1.0423E+01, 9.9104E+02, &
      3.1075387E+00, 5.63109E+01, 1.0341E+01, 9.7844E+02, &
      3.1230764E+00, 5.63136E+01, 1.0260E+01, 9.6599E+02, &
      3.1386918E+00, 5.63154E+01, 1.0179E+01, 9.5372E+02, &
      3.1543853E+00, 5.63367E+01, 1.0099E+01, 9.4159E+02, &
      3.1701572E+00, 5.63375E+01, 1.0019E+01, 9.2957E+02, &
      3.1860080E+00, 5.63375E+01, 9.9400E+00, 9.1772E+02, &
      3.2019380E+00, 5.63368E+01, 9.8615E+00, 9.0602E+02, &
      3.2179477E+00, 5.63355E+01, 9.7836E+00, 8.9448E+02, &
      3.2340374E+00, 5.63336E+01, 9.7064E+00, 8.8308E+02, &
      3.2502076E+00, 5.63310E+01, 9.6299E+00, 8.7184E+02, &
      3.2664587E+00, 5.63279E+01, 9.5540E+00, 8.6075E+02, &
      3.2827910E+00, 5.63242E+01, 9.4788E+00, 8.4980E+02, &
      3.2992049E+00, 5.63200E+01, 9.4042E+00, 8.3900E+02, &
      3.3157009E+00, 5.63152E+01, 9.3302E+00, 8.2834E+02, &
      3.3322794E+00, 5.63100E+01, 9.2569E+00, 8.1782E+02, &
      3.3489408E+00, 5.63042E+01, 9.1842E+00, 8.0744E+02, &
      3.3656856E+00, 5.62979E+01, 9.1121E+00, 7.9720E+02, &
      3.3825140E+00, 5.62912E+01, 9.0407E+00, 7.8709E+02, &
      3.3994265E+00, 5.62840E+01, 8.9699E+00, 7.7712E+02, &
      3.4164237E+00, 5.62763E+01, 8.8996E+00, 7.6728E+02, &
      3.4335058E+00, 5.62682E+01, 8.8300E+00, 7.5756E+02, &
      3.4506733E+00, 5.62597E+01, 8.7610E+00, 7.4798E+02, &
      3.4679267E+00, 5.62507E+01, 8.6925E+00, 7.3852E+02, &
      3.4852663E+00, 5.62413E+01, 8.6247E+00, 7.2919E+02, &
      3.5026927E+00, 5.62315E+01, 8.5574E+00, 7.1998E+02, &
      3.5202061E+00, 5.62213E+01, 8.4907E+00, 7.1089E+02, &
      3.5378072E+00, 5.62107E+01, 8.4246E+00, 7.0192E+02, &
      3.5554962E+00, 5.61997E+01, 8.3591E+00, 6.9307E+02, &
      3.5732737E+00, 5.61883E+01, 8.2941E+00, 6.8434E+02, &
      3.5911400E+00, 5.61766E+01, 8.2297E+00, 6.7572E+02, &
      3.6090957E+00, 5.61645E+01, 8.1658E+00, 6.6721E+02, &
      3.6271412E+00, 5.61520E+01, 8.1025E+00, 6.5882E+02, &
      3.6452769E+00, 5.61391E+01, 8.0397E+00, 6.5054E+02, &
      3.6635033E+00, 5.61259E+01, 7.9775E+00, 6.4236E+02, &
      3.6818208E+00, 5.61124E+01, 7.9158E+00, 6.3430E+02, &
      3.7002299E+00, 5.60985E+01, 7.8546E+00, 6.2634E+02, &
      3.7187311E+00, 5.60842E+01, 7.7940E+00, 6.1848E+02, &
      3.7373247E+00, 5.60696E+01, 7.7339E+00, 6.1073E+02, &
      3.7560114E+00, 5.60547E+01, 7.6743E+00, 6.0308E+02, &
      3.7747914E+00, 5.60394E+01, 7.6152E+00, 5.9553E+02, &
      3.7936654E+00, 5.60239E+01, 7.5566E+00, 5.8808E+02, &
      3.8126337E+00, 5.60080E+01, 7.4985E+00, 5.8073E+02, &
      3.8316969E+00, 5.59917E+01, 7.4409E+00, 5.7347E+02, &
      3.8508554E+00, 5.59752E+01, 7.3839E+00, 5.6631E+02, &
      3.8701096E+00, 5.59584E+01, 7.3273E+00, 5.5925E+02, &
      3.8894602E+00, 5.59412E+01, 7.2712E+00, 5.5227E+02, &
      3.9089075E+00, 5.59237E+01, 7.2155E+00, 5.4539E+02, &
      3.9284520E+00, 5.59059E+01, 7.1584E+00, 5.3845E+02, &
      3.9480943E+00, 5.58876E+01, 7.1016E+00, 5.3159E+02, &
      3.9678347E+00, 5.58688E+01, 7.0453E+00, 5.2482E+02, &
      3.9876739E+00, 5.58496E+01, 6.9895E+00, 5.1815E+02, &
      4.055024E+00, 5.58798E+01, 6.8021E+00, 4.9614E+02, &
      4.334821E+00, 5.55364E+01, 6.1071E+00, 4.1754E+02, &
      4.633924E+00, 5.50956E+01, 5.4906E+00, 3.5195E+02, &
      4.953664E+00, 5.45257E+01, 4.9438E+00, 2.9719E+02, &
      5.295467E+00, 5.37453E+01, 4.4599E+00, 2.5147E+02, &
      5.660855E+00, 5.25236E+01, 4.0284E+00, 2.1312E+02, &
      6.051453E+00, 4.96236E+01, 3.6265E+00, 1.8008E+02, &
      6.083742E+00, 4.90992E+01, 3.5963E+00, 1.7769E+02, &
      6.176861E+00, 4.59862E+01, 3.5094E+00, 1.7092E+02, &
      6.201692E+00, 4.23756E+01, 3.4869E+00, 1.6919E+02, &
      6.238940E+00, 4.59501E+01, 1.0477E+01, 4.9503E+02, &
      6.332058E+00, 4.90157E+01, 1.0157E+01, 4.7304E+02, &
      6.469004E+00, 5.03230E+01, 9.7134E+00, 4.4305E+02, &
      6.587070E+00, 5.04754E+01, 9.3795E+00, 4.2034E+02, &
      6.687892E+00, 4.94518E+01, 9.1261E+00, 4.0297E+02, &
      6.714778E+00, 4.77614E+01, 9.0613E+00, 3.9854E+02, &
      6.755107E+00, 4.97211E+01, 1.2490E+01, 5.4424E+02, &
      6.855930E+00, 5.16041E+01, 1.2143E+01, 5.2149E+02, &
      6.915365E+00, 5.21180E+01, 1.1947E+01, 5.0873E+02, &
      6.983480E+00, 5.24797E+01, 1.1729E+01, 4.9468E+02, &
      7.090370E+00, 5.24047E+01, 1.1409E+01, 4.7407E+02, &
      7.118874E+00, 5.16595E+01, 1.1330E+01, 4.6894E+02, &
      7.161630E+00, 5.27687E+01, 1.2981E+01, 5.3341E+02, &
      7.268520E+00, 5.40478E+01, 1.2689E+01, 5.1384E+02, &
      7.392525E+00, 5.48829E+01, 1.2368E+01, 4.9259E+02, &
      7.902609E+00, 5.68673E+01, 1.1191E+01, 4.1738E+02, &
      8.447890E+00, 5.80697E+01, 1.0105E+01, 3.5297E+02, &
      9.030794E+00, 5.88974E+01, 9.0889E+00, 2.9737E+02, &
      9.653919E+00, 5.94525E+01, 8.1175E+00, 2.4884E+02, &
      1.032004E+01, 5.98121E+01, 7.2528E+00, 2.0834E+02, &
      1.103212E+01, 6.00465E+01, 6.4859E+00, 1.7461E+02, &
      1.179334E+01, 6.01960E+01, 5.8052E+00, 1.4650E+02, &
      1.260708E+01, 6.03595E+01, 5.1951E+00, 1.2292E+02, &
      1.347697E+01, 6.04387E+01, 4.6369E+00, 1.0289E+02, &
      1.440688E+01, 6.04740E+01, 4.1298E+00, 8.5959E+01, &
      1.540095E+01, 6.04403E+01, 3.6706E+00, 7.1695E+01, &
      1.646362E+01, 6.03808E+01, 3.2654E+00, 5.9867E+01, &
      1.759961E+01, 6.03052E+01, 2.9074E+00, 5.0047E+01, &
      1.881398E+01, 6.02192E+01, 2.5908E+00, 4.1886E+01, &
      2.011215E+01, 6.01261E+01, 2.3103E+00, 3.5092E+01, &
      2.149988E+01, 6.00284E+01, 2.0622E+00, 2.9438E+01, &
      2.298338E+01, 5.99273E+01, 1.8424E+00, 2.4726E+01, &
      2.456923E+01, 5.98234E+01, 1.6476E+00, 2.0796E+01, &
      2.626450E+01, 5.97171E+01, 1.4747E+00, 1.7514E+01, &
      2.807676E+01, 5.96126E+01, 1.3090E+00, 1.4641E+01, &
      3.001405E+01, 5.94862E+01, 1.1578E+00, 1.2209E+01, &
      3.208502E+01, 5.93390E+01, 1.0217E+00, 1.0166E+01, &
      3.429889E+01, 5.91595E+01, 9.0046E-01, 8.4632E+00, &
      3.666551E+01, 5.89249E+01, 7.9380E-01, 7.0538E+00, &
      3.919543E+01, 5.85720E+01, 7.0034E-01, 5.8896E+00, &
      4.189992E+01, 5.78130E+01, 6.1837E-01, 4.9268E+00, &
      4.269752E+01, 5.72777E+01, 5.9708E-01, 4.6861E+00, &
      4.335106E+01, 5.60865E+01, 5.8049E-01, 4.5012E+00, &
      4.352533E+01, 5.46590E+01, 5.7619E-01, 4.4537E+00, &
      4.378675E+01, 5.61019E+01, 3.3802E+00, 2.3108E+01, &
      4.444028E+01, 5.74061E+01, 3.3033E+00, 2.2261E+01, &
      4.479101E+01, 5.77322E+01, 3.2633E+00, 2.1825E+01, &
      4.788159E+01, 5.89643E+01, 2.9336E+00, 1.8398E+01, &
      5.118542E+01, 5.94877E+01, 2.6295E+00, 1.5469E+01, &
      5.471721E+01, 5.97971E+01, 2.3545E+00, 1.2998E+01, &
      5.849270E+01, 5.99994E+01, 2.1055E+00, 1.0910E+01, &
      6.252870E+01, 6.01336E+01, 1.8805E+00, 9.1516E+00, &
      6.684318E+01, 6.02238E+01, 1.6795E+00, 7.6793E+00, &
      7.145536E+01, 6.02840E+01, 1.4999E+00, 6.4470E+00, &
      7.638578E+01, 6.03235E+01, 1.3393E+00, 5.4154E+00, &
      8.165640E+01, 6.03490E+01, 1.1944E+00, 4.5462E+00, &
      8.729069E+01, 6.03916E+01, 1.0619E+00, 3.8092E+00, &
      9.331374E+01, 6.03920E+01, 9.4080E-01, 3.1843E+00, &
      9.975239E+01, 6.03843E+01, 8.3358E-01, 2.6654E+00, &
      1.066353E+02, 6.03715E+01, 7.3864E-01, 2.2343E+00, &
      1.139931E+02, 6.03552E+01, 6.5458E-01, 1.8761E+00, &
      1.218587E+02, 6.03369E+01, 5.8013E-01, 1.5783E+00, &
      1.302669E+02, 6.03175E+01, 5.1419E-01, 1.3306E+00, &
      1.392553E+02, 6.02975E+01, 4.5578E-01, 1.1245E+00, &
      1.488640E+02, 6.02775E+01, 4.0404E-01, 9.5293E-01, &
      1.591356E+02, 6.02577E+01, 3.5820E-01, 8.0996E-01, &
      1.701159E+02, 6.02386E+01, 3.1759E-01, 6.9077E-01, &
      1.818539E+02, 6.02201E+01, 2.8159E-01, 5.9132E-01, &
      1.944018E+02, 6.02023E+01, 2.4998E-01, 5.0867E-01, &
      2.078156E+02, 6.01858E+01, 2.2225E-01, 4.3994E-01, &
      2.221548E+02, 6.01704E+01, 1.9761E-01, 3.8225E-01, &
      2.374835E+02, 6.01561E+01, 1.7571E-01, 3.3376E-01, &
      2.538699E+02, 6.01428E+01, 1.5625E-01, 2.9292E-01, &
      2.713869E+02, 6.01305E+01, 1.3895E-01, 2.5848E-01, &
      2.901126E+02, 6.01191E+01, 1.2357E-01, 2.2934E-01, &
      3.101304E+02, 6.01086E+01, 1.0990E-01, 2.0465E-01, &
      3.315294E+02, 6.00989E+01, 9.7751E-02, 1.8364E-01, &
      3.544049E+02, 6.00900E+01, 8.6948E-02, 1.6571E-01, &
      3.788588E+02, 6.00818E+01, 7.7342E-02, 1.5035E-01, &
      4.050001E+02, 6.00743E+01, 6.8801E-02, 1.3714E-01, &
      4.329451E+02, 6.00674E+01, 6.1207E-02, 1.2571E-01]
   case(61)
      frel = -5.3340E-01
      allocate(nist_tab(3568))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.25974E+01, 2.4241E-01, 6.5808E+03, &
      1.142761E-02, 1.24613E+01, 2.7242E-01, 6.9181E+03, &
      1.221612E-02, 1.23126E+01, 3.0634E-01, 7.2775E+03, &
      1.305903E-02, 1.21106E+01, 3.4476E-01, 7.6616E+03, &
      1.396010E-02, 1.18573E+01, 3.8832E-01, 8.0726E+03, &
      1.492335E-02, 1.15312E+01, 4.3773E-01, 8.5124E+03, &
      1.595306E-02, 1.10943E+01, 4.9378E-01, 8.9826E+03, &
      1.705382E-02, 1.04739E+01, 5.5717E-01, 9.4814E+03, &
      1.823053E-02, 9.50254E+00, 6.2855E-01, 1.0006E+04, &
      1.948844E-02, 7.65798E+00, 7.0875E-01, 1.0554E+04, &
      2.067800E-02, 2.52909E+00, 7.8801E-01, 1.1060E+04, &
      2.083314E-02, 5.78709E-01, 7.9858E-01, 1.1124E+04, &
      2.099450E-02, -3.55699E+00, 8.0963E-01, 1.1192E+04, &
      2.107890E-02, -1.10546E+01, 8.1543E-01, 1.1227E+04, &
      2.120550E-02, -2.96724E+00, 1.5691E+01, 2.1475E+05, &
      2.152200E-02, 4.18486E+00, 1.5134E+01, 2.0407E+05, &
      2.227063E-02, 9.69595E+00, 1.3936E+01, 1.8160E+05, &
      2.380730E-02, 1.41247E+01, 1.1898E+01, 1.4504E+05, &
      2.545001E-02, 1.62788E+01, 1.0082E+01, 1.1496E+05, &
      2.720606E-02, 1.73987E+01, 8.5183E+00, 9.0865E+04, &
      2.908327E-02, 1.79337E+01, 7.2406E+00, 7.2251E+04, &
      3.109002E-02, 1.81250E+01, 6.2405E+00, 5.8252E+04, &
      3.323523E-02, 1.81188E+01, 5.4887E+00, 4.7927E+04, &
      3.552846E-02, 1.80053E+01, 4.9481E+00, 4.0418E+04, &
      3.675000E-02, 1.79230E+01, 4.7430E+00, 3.7455E+04, &
      3.731250E-02, 1.78768E+01, 4.6642E+00, 3.6277E+04, &
      3.746250E-02, 1.78559E+01, 4.6447E+00, 3.5981E+04, &
      3.768750E-02, 1.78515E+01, 4.6381E+00, 3.5715E+04, &
      3.797993E-02, 1.78383E+01, 4.6041E+00, 3.5180E+04, &
      3.825000E-02, 1.78235E+01, 4.5747E+00, 3.4709E+04, &
      4.060054E-02, 1.76842E+01, 4.3856E+00, 3.1348E+04, &
      4.340198E-02, 1.75358E+01, 4.2820E+00, 2.8632E+04, &
      4.639671E-02, 1.74120E+01, 4.2700E+00, 2.6709E+04, &
      4.959809E-02, 1.73214E+01, 4.3313E+00, 2.5343E+04, &
      5.302035E-02, 1.72687E+01, 4.4514E+00, 2.4365E+04, &
      5.667876E-02, 1.72569E+01, 4.6186E+00, 2.3648E+04, &
      6.058959E-02, 1.72887E+01, 4.8239E+00, 2.3105E+04, &
      6.477028E-02, 1.73807E+01, 5.0598E+00, 2.2671E+04, &
      6.923942E-02, 1.75100E+01, 5.3207E+00, 2.2301E+04, &
      7.401695E-02, 1.76990E+01, 5.6013E+00, 2.1962E+04, &
      7.912411E-02, 1.79251E+01, 5.6952E+00, 2.0889E+04, &
      8.458368E-02, 1.80861E+01, 5.7150E+00, 1.9608E+04, &
      9.041995E-02, 1.81521E+01, 5.7131E+00, 1.8337E+04, &
      9.665893E-02, 1.80626E+01, 5.6880E+00, 1.7078E+04, &
      1.0000000E-01, 2.10426E+01, 5.6661E+00, 1.6444E+04, &
      1.0050000E-01, 2.10130E+01, 5.6624E+00, 1.6351E+04, &
      1.0100250E-01, 2.09808E+01, 5.6585E+00, 1.6259E+04, &
      1.0150751E-01, 2.09459E+01, 5.6545E+00, 1.6167E+04, &
      1.0201505E-01, 2.09081E+01, 5.6503E+00, 1.6074E+04, &
      1.0252513E-01, 2.08671E+01, 5.6461E+00, 1.5982E+04, &
      1.0303775E-01, 2.08226E+01, 5.6417E+00, 1.5890E+04, &
      1.0355294E-01, 2.07745E+01, 5.6371E+00, 1.5799E+04, &
      1.0407070E-01, 2.07223E+01, 5.6324E+00, 1.5707E+04, &
      1.0459106E-01, 2.06659E+01, 5.6276E+00, 1.5615E+04, &
      1.0511401E-01, 2.06047E+01, 5.6227E+00, 1.5524E+04, &
      1.0563958E-01, 2.05384E+01, 5.6176E+00, 1.5433E+04, &
      1.0616778E-01, 2.04664E+01, 5.6124E+00, 1.5342E+04, &
      1.0669862E-01, 2.03882E+01, 5.6071E+00, 1.5251E+04, &
      1.0723211E-01, 2.03033E+01, 5.6016E+00, 1.5161E+04, &
      1.0776827E-01, 2.02107E+01, 5.5960E+00, 1.5070E+04, &
      1.0830712E-01, 2.01098E+01, 5.5903E+00, 1.4980E+04, &
      1.0884865E-01, 1.99996E+01, 5.5844E+00, 1.4890E+04, &
      1.0939289E-01, 1.98788E+01, 5.5784E+00, 1.4800E+04, &
      1.0993986E-01, 1.97461E+01, 5.5723E+00, 1.4710E+04, &
      1.1048956E-01, 1.96000E+01, 5.5661E+00, 1.4620E+04, &
      1.1104201E-01, 1.94386E+01, 5.5597E+00, 1.4531E+04, &
      1.1159722E-01, 1.92594E+01, 5.5532E+00, 1.4442E+04, &
      1.1215520E-01, 1.90597E+01, 5.5465E+00, 1.4353E+04, &
      1.1271598E-01, 1.88364E+01, 5.5398E+00, 1.4264E+04, &
      1.1327956E-01, 1.85840E+01, 5.5329E+00, 1.4175E+04, &
      1.1384596E-01, 1.82973E+01, 5.5259E+00, 1.4087E+04, &
      1.1441519E-01, 1.79689E+01, 5.5187E+00, 1.3999E+04, &
      1.1498726E-01, 1.75888E+01, 5.5114E+00, 1.3911E+04, &
      1.1556220E-01, 1.71433E+01, 5.5041E+00, 1.3823E+04, &
      1.1614001E-01, 1.66130E+01, 5.4965E+00, 1.3735E+04, &
      1.1672071E-01, 1.59690E+01, 5.4889E+00, 1.3648E+04, &
      1.1730431E-01, 1.51662E+01, 5.4811E+00, 1.3561E+04, &
      1.1789083E-01, 1.41287E+01, 5.4732E+00, 1.3474E+04, &
      1.1848029E-01, 1.27149E+01, 5.4652E+00, 1.3387E+04, &
      1.1907269E-01, 1.06144E+01, 5.4571E+00, 1.3301E+04, &
      1.1966805E-01, 6.90986E+00, 5.4488E+00, 1.3215E+04, &
      1.2026639E-01, -5.15134E+00, 5.4405E+00, 1.3129E+04, &
      1.2038579E-01, -2.25841E+01, 5.4388E+00, 1.3112E+04, &
      1.2041420E-01, -2.24362E+01, 3.0450E+01, 7.3387E+04, &
      1.2086772E-01, 6.71061E+00, 2.9192E+01, 7.0093E+04, &
      1.2147206E-01, 1.42183E+01, 2.7620E+01, 6.5988E+04, &
      1.2207942E-01, 1.83192E+01, 2.6151E+01, 6.2166E+04, &
      1.2268982E-01, 2.10848E+01, 2.4777E+01, 5.8608E+04, &
      1.2330327E-01, 2.31102E+01, 2.3493E+01, 5.5295E+04, &
      1.2391979E-01, 2.46580E+01, 2.2293E+01, 5.2210E+04, &
      1.2453939E-01, 2.58700E+01, 2.1172E+01, 4.9336E+04, &
      1.2516208E-01, 2.68329E+01, 2.0123E+01, 4.6659E+04, &
      1.2578789E-01, 2.76041E+01, 1.9143E+01, 4.4165E+04, &
      1.2641683E-01, 2.82237E+01, 1.8226E+01, 4.1841E+04, &
      1.2704892E-01, 2.87212E+01, 1.7369E+01, 3.9676E+04, &
      1.2768416E-01, 2.91188E+01, 1.6568E+01, 3.7657E+04, &
      1.2832258E-01, 2.94339E+01, 1.5818E+01, 3.5775E+04, &
      1.2896419E-01, 2.96800E+01, 1.5118E+01, 3.4020E+04, &
      1.2960902E-01, 2.98680E+01, 1.4462E+01, 3.2383E+04, &
      1.3025706E-01, 3.00066E+01, 1.3849E+01, 3.0856E+04, &
      1.3090835E-01, 3.01029E+01, 1.3276E+01, 2.9431E+04, &
      1.3156289E-01, 3.01627E+01, 1.2739E+01, 2.8101E+04, &
      1.3222070E-01, 3.01903E+01, 1.2237E+01, 2.6860E+04, &
      1.3288181E-01, 3.01888E+01, 1.1769E+01, 2.5703E+04, &
      1.3354621E-01, 3.01633E+01, 1.1339E+01, 2.4642E+04, &
      1.3421395E-01, 3.01206E+01, 1.0946E+01, 2.3669E+04, &
      1.3488502E-01, 3.00653E+01, 1.0586E+01, 2.2776E+04, &
      1.3555944E-01, 3.00009E+01, 1.0255E+01, 2.1954E+04, &
      1.3623724E-01, 2.99301E+01, 9.9501E+00, 2.1196E+04, &
      1.3691842E-01, 2.98548E+01, 9.6696E+00, 2.0496E+04, &
      1.3760302E-01, 2.97764E+01, 9.4107E+00, 1.9848E+04, &
      1.3829103E-01, 2.96963E+01, 9.1715E+00, 1.9248E+04, &
      1.3898249E-01, 2.96153E+01, 8.9500E+00, 1.8689E+04, &
      1.3967740E-01, 2.95341E+01, 8.7448E+00, 1.8170E+04, &
      1.4037579E-01, 2.94533E+01, 8.5542E+00, 1.7686E+04, &
      1.4107766E-01, 2.93734E+01, 8.3770E+00, 1.7233E+04, &
      1.4178305E-01, 2.92947E+01, 8.2121E+00, 1.6810E+04, &
      1.4249197E-01, 2.92175E+01, 8.0583E+00, 1.6413E+04, &
      1.4320443E-01, 2.91419E+01, 7.9147E+00, 1.6040E+04, &
      1.4392045E-01, 2.90681E+01, 7.7804E+00, 1.5690E+04, &
      1.4464005E-01, 2.89962E+01, 7.6547E+00, 1.5359E+04, &
      1.4536325E-01, 2.89262E+01, 7.5368E+00, 1.5048E+04, &
      1.4609007E-01, 2.88582E+01, 7.4261E+00, 1.4753E+04, &
      1.4682052E-01, 2.87923E+01, 7.3221E+00, 1.4474E+04, &
      1.4755462E-01, 2.87283E+01, 7.2242E+00, 1.4209E+04, &
      1.4829239E-01, 2.86663E+01, 7.1319E+00, 1.3958E+04, &
      1.4903386E-01, 2.86063E+01, 7.0448E+00, 1.3719E+04, &
      1.4977903E-01, 2.85482E+01, 6.9625E+00, 1.3491E+04, &
      1.5052792E-01, 2.84920E+01, 6.8846E+00, 1.3274E+04, &
      1.5128056E-01, 2.84376E+01, 6.8109E+00, 1.3067E+04, &
      1.5203696E-01, 2.83849E+01, 6.7409E+00, 1.2868E+04, &
      1.5279715E-01, 2.83340E+01, 6.6745E+00, 1.2678E+04, &
      1.5356113E-01, 2.82847E+01, 6.6114E+00, 1.2496E+04, &
      1.5432894E-01, 2.82370E+01, 6.5514E+00, 1.2321E+04, &
      1.5510058E-01, 2.81909E+01, 6.4942E+00, 1.2152E+04, &
      1.5587609E-01, 2.81462E+01, 6.4397E+00, 1.1990E+04, &
      1.5665547E-01, 2.81029E+01, 6.3876E+00, 1.1834E+04, &
      1.5743875E-01, 2.80610E+01, 6.3379E+00, 1.1684E+04, &
      1.5822594E-01, 2.80204E+01, 6.2903E+00, 1.1538E+04, &
      1.5901707E-01, 2.79810E+01, 6.2447E+00, 1.1398E+04, &
      1.5981215E-01, 2.79428E+01, 6.2011E+00, 1.1262E+04, &
      1.6061121E-01, 2.79057E+01, 6.1592E+00, 1.1130E+04, &
      1.6141427E-01, 2.78697E+01, 6.1190E+00, 1.1003E+04, &
      1.6222134E-01, 2.78348E+01, 6.0804E+00, 1.0879E+04, &
      1.6303245E-01, 2.78008E+01, 6.0432E+00, 1.0759E+04, &
      1.6384761E-01, 2.77677E+01, 6.0075E+00, 1.0642E+04, &
      1.6466685E-01, 2.77356E+01, 5.9731E+00, 1.0528E+04, &
      1.6549018E-01, 2.77042E+01, 5.9399E+00, 1.0418E+04, &
      1.6631763E-01, 2.76737E+01, 5.9079E+00, 1.0310E+04, &
      1.6714922E-01, 2.76439E+01, 5.8770E+00, 1.0205E+04, &
      1.6798497E-01, 2.76148E+01, 5.8472E+00, 1.0103E+04, &
      1.6882489E-01, 2.75864E+01, 5.8183E+00, 1.0003E+04, &
      1.6966902E-01, 2.75587E+01, 5.7905E+00, 9.9055E+03, &
      1.7051736E-01, 2.75315E+01, 5.7635E+00, 9.8104E+03, &
      1.7136995E-01, 2.75049E+01, 5.7374E+00, 9.7174E+03, &
      1.7222680E-01, 2.74788E+01, 5.7121E+00, 9.6265E+03, &
      1.7308793E-01, 2.74533E+01, 5.6877E+00, 9.5375E+03, &
      1.7395337E-01, 2.74281E+01, 5.6639E+00, 9.4505E+03, &
      1.7482314E-01, 2.74035E+01, 5.6409E+00, 9.3653E+03, &
      1.7569726E-01, 2.73792E+01, 5.6186E+00, 9.2819E+03, &
      1.7657574E-01, 2.73553E+01, 5.5969E+00, 9.2001E+03, &
      1.7745862E-01, 2.73317E+01, 5.5759E+00, 9.1200E+03, &
      1.7834591E-01, 2.73085E+01, 5.5555E+00, 9.0415E+03, &
      1.7923764E-01, 2.72856E+01, 5.5357E+00, 8.9644E+03, &
      1.8013383E-01, 2.72629E+01, 5.5165E+00, 8.8888E+03, &
      1.8103450E-01, 2.72405E+01, 5.4978E+00, 8.8147E+03, &
      1.8193967E-01, 2.72183E+01, 5.4796E+00, 8.7419E+03, &
      1.8284937E-01, 2.71963E+01, 5.4620E+00, 8.6704E+03, &
      1.8376362E-01, 2.71744E+01, 5.4449E+00, 8.6003E+03, &
      1.8468244E-01, 2.71527E+01, 5.4282E+00, 8.5313E+03, &
      1.8560585E-01, 2.71311E+01, 5.4121E+00, 8.4636E+03, &
      1.8653388E-01, 2.71097E+01, 5.3963E+00, 8.3971E+03, &
      1.8746655E-01, 2.70883E+01, 5.3811E+00, 8.3317E+03, &
      1.8840388E-01, 2.70669E+01, 5.3663E+00, 8.2675E+03, &
      1.8934590E-01, 2.70457E+01, 5.3519E+00, 8.2043E+03, &
      1.9029263E-01, 2.70244E+01, 5.3379E+00, 8.1422E+03, &
      1.9124409E-01, 2.70031E+01, 5.3244E+00, 8.0812E+03, &
      1.9220031E-01, 2.69818E+01, 5.3112E+00, 8.0211E+03, &
      1.9316131E-01, 2.69604E+01, 5.2985E+00, 7.9621E+03, &
      1.9412712E-01, 2.69390E+01, 5.2861E+00, 7.9040E+03, &
      1.9509776E-01, 2.69174E+01, 5.2741E+00, 7.8469E+03, &
      1.9607325E-01, 2.68958E+01, 5.2626E+00, 7.7907E+03, &
      1.9705361E-01, 2.68740E+01, 5.2513E+00, 7.7355E+03, &
      1.9803888E-01, 2.68520E+01, 5.2405E+00, 7.6811E+03, &
      1.9902907E-01, 2.68299E+01, 5.2300E+00, 7.6277E+03, &
      2.0002422E-01, 2.68075E+01, 5.2199E+00, 7.5751E+03, &
      2.0102434E-01, 2.67849E+01, 5.2101E+00, 7.5233E+03, &
      2.0202946E-01, 2.67620E+01, 5.2007E+00, 7.4724E+03, &
      2.0303961E-01, 2.67388E+01, 5.1917E+00, 7.4223E+03, &
      2.0405481E-01, 2.67153E+01, 5.1830E+00, 7.3730E+03, &
      2.0507508E-01, 2.66914E+01, 5.1746E+00, 7.3245E+03, &
      2.0610046E-01, 2.66671E+01, 5.1666E+00, 7.2769E+03, &
      2.0713096E-01, 2.66423E+01, 5.1590E+00, 7.2299E+03, &
      2.0816661E-01, 2.66171E+01, 5.1517E+00, 7.1838E+03, &
      2.0920745E-01, 2.65913E+01, 5.1447E+00, 7.1384E+03, &
      2.1025348E-01, 2.65649E+01, 5.1380E+00, 7.0937E+03, &
      2.1130475E-01, 2.65378E+01, 5.1318E+00, 7.0498E+03, &
      2.1236128E-01, 2.65101E+01, 5.1258E+00, 7.0066E+03, &
      2.1342308E-01, 2.64815E+01, 5.1202E+00, 6.9641E+03, &
      2.1449020E-01, 2.64521E+01, 5.1149E+00, 6.9224E+03, &
      2.1556265E-01, 2.64217E+01, 5.1099E+00, 6.8813E+03, &
      2.1664046E-01, 2.63901E+01, 5.1053E+00, 6.8409E+03, &
      2.1772366E-01, 2.63574E+01, 5.1010E+00, 6.8012E+03, &
      2.1881228E-01, 2.63233E+01, 5.0971E+00, 6.7621E+03, &
      2.1990634E-01, 2.62876E+01, 5.0935E+00, 6.7237E+03, &
      2.2100588E-01, 2.62501E+01, 5.0902E+00, 6.6860E+03, &
      2.2211090E-01, 2.62106E+01, 5.0873E+00, 6.6489E+03, &
      2.2322146E-01, 2.61686E+01, 5.0847E+00, 6.6125E+03, &
      2.2433757E-01, 2.61238E+01, 5.0824E+00, 6.5767E+03, &
      2.2545925E-01, 2.60756E+01, 5.0804E+00, 6.5415E+03, &
      2.2658655E-01, 2.60231E+01, 5.0788E+00, 6.5069E+03, &
      2.2771948E-01, 2.59655E+01, 5.0776E+00, 6.4730E+03, &
      2.2885808E-01, 2.59010E+01, 5.0766E+00, 6.4396E+03, &
      2.3000237E-01, 2.58274E+01, 5.0760E+00, 6.4068E+03, &
      2.3115238E-01, 2.57407E+01, 5.0758E+00, 6.3747E+03, &
      2.3230814E-01, 2.56341E+01, 5.0759E+00, 6.3431E+03, &
      2.3346969E-01, 2.54922E+01, 5.0763E+00, 6.3120E+03, &
      2.3463703E-01, 2.52703E+01, 5.0770E+00, 6.2816E+03, &
      2.3581022E-01, 2.46034E+01, 5.0781E+00, 6.2517E+03, &
      2.3585864E-01, 2.45060E+01, 5.0781E+00, 6.2505E+03, &
      2.3614137E-01, 2.45005E+01, 6.1083E+00, 7.5091E+03, &
      2.3698927E-01, 2.51225E+01, 6.1123E+00, 7.4870E+03, &
      2.3817422E-01, 2.53599E+01, 6.1181E+00, 7.4569E+03, &
      2.3936509E-01, 2.54826E+01, 6.1242E+00, 7.4272E+03, &
      2.4056191E-01, 2.55617E+01, 6.1307E+00, 7.3982E+03, &
      2.4176472E-01, 2.56173E+01, 6.1375E+00, 7.3696E+03, &
      2.4297355E-01, 2.56579E+01, 6.1447E+00, 7.3416E+03, &
      2.4418841E-01, 2.56877E+01, 6.1523E+00, 7.3140E+03, &
      2.4540936E-01, 2.57091E+01, 6.1602E+00, 7.2870E+03, &
      2.4663640E-01, 2.57233E+01, 6.1684E+00, 7.2605E+03, &
      2.4786959E-01, 2.57307E+01, 6.1770E+00, 7.2344E+03, &
      2.4910893E-01, 2.57309E+01, 6.1860E+00, 7.2089E+03, &
      2.5035448E-01, 2.57223E+01, 6.1952E+00, 7.1838E+03, &
      2.5160625E-01, 2.57007E+01, 6.2049E+00, 7.1592E+03, &
      2.5286428E-01, 2.56532E+01, 6.2149E+00, 7.1351E+03, &
      2.5412860E-01, 2.54844E+01, 6.2252E+00, 7.1114E+03, &
      2.5423287E-01, 2.54345E+01, 6.2260E+00, 7.1095E+03, &
      2.5456716E-01, 2.54375E+01, 6.5590E+00, 7.4797E+03, &
      2.5539925E-01, 2.56329E+01, 6.5670E+00, 7.4645E+03, &
      2.5667624E-01, 2.57306E+01, 6.5796E+00, 7.4416E+03, &
      2.5795962E-01, 2.57882E+01, 6.5925E+00, 7.4191E+03, &
      2.5924942E-01, 2.58307E+01, 6.6057E+00, 7.3970E+03, &
      2.6054567E-01, 2.58649E+01, 6.6193E+00, 7.3754E+03, &
      2.6184840E-01, 2.58940E+01, 6.6332E+00, 7.3541E+03, &
      2.6315764E-01, 2.59195E+01, 6.6474E+00, 7.3332E+03, &
      2.6447343E-01, 2.59422E+01, 6.6619E+00, 7.3127E+03, &
      2.6579579E-01, 2.59628E+01, 6.6767E+00, 7.2925E+03, &
      2.6712477E-01, 2.59817E+01, 6.6919E+00, 7.2727E+03, &
      2.6846040E-01, 2.59992E+01, 6.7073E+00, 7.2532E+03, &
      2.6980270E-01, 2.60156E+01, 6.7230E+00, 7.2340E+03, &
      2.7115171E-01, 2.60310E+01, 6.7390E+00, 7.2152E+03, &
      2.7250747E-01, 2.60455E+01, 6.7552E+00, 7.1967E+03, &
      2.7387001E-01, 2.60593E+01, 6.7718E+00, 7.1784E+03, &
      2.7523936E-01, 2.60725E+01, 6.7886E+00, 7.1605E+03, &
      2.7661556E-01, 2.60851E+01, 6.8057E+00, 7.1429E+03, &
      2.7799863E-01, 2.60972E+01, 6.8231E+00, 7.1255E+03, &
      2.7938863E-01, 2.61089E+01, 6.8408E+00, 7.1084E+03, &
      2.8078557E-01, 2.61202E+01, 6.8587E+00, 7.0916E+03, &
      2.8218950E-01, 2.61311E+01, 6.8769E+00, 7.0750E+03, &
      2.8360044E-01, 2.61417E+01, 6.8953E+00, 7.0587E+03, &
      2.8501845E-01, 2.61520E+01, 6.9139E+00, 7.0426E+03, &
      2.8644354E-01, 2.61620E+01, 6.9328E+00, 7.0268E+03, &
      2.8787576E-01, 2.61718E+01, 6.9520E+00, 7.0112E+03, &
      2.8931514E-01, 2.61813E+01, 6.9714E+00, 6.9957E+03, &
      2.9076171E-01, 2.61906E+01, 6.9909E+00, 6.9805E+03, &
      2.9221552E-01, 2.61996E+01, 7.0108E+00, 6.9655E+03, &
      2.9367660E-01, 2.62084E+01, 7.0308E+00, 6.9507E+03, &
      2.9514498E-01, 2.62169E+01, 7.0510E+00, 6.9360E+03, &
      2.9662071E-01, 2.62252E+01, 7.0715E+00, 6.9215E+03, &
      2.9810381E-01, 2.62333E+01, 7.0921E+00, 6.9072E+03, &
      2.9959433E-01, 2.62410E+01, 7.1129E+00, 6.8931E+03, &
      3.0109230E-01, 2.62484E+01, 7.1339E+00, 6.8790E+03, &
      3.0259776E-01, 2.62555E+01, 7.1551E+00, 6.8651E+03, &
      3.0411075E-01, 2.62622E+01, 7.1764E+00, 6.8514E+03, &
      3.0563130E-01, 2.62685E+01, 7.1979E+00, 6.8377E+03, &
      3.0715946E-01, 2.62742E+01, 7.2195E+00, 6.8242E+03, &
      3.0869526E-01, 2.62793E+01, 7.2413E+00, 6.8108E+03, &
      3.1023873E-01, 2.62837E+01, 7.2633E+00, 6.7975E+03, &
      3.1178993E-01, 2.62873E+01, 7.2853E+00, 6.7842E+03, &
      3.1334888E-01, 2.62898E+01, 7.3075E+00, 6.7711E+03, &
      3.1491562E-01, 2.62910E+01, 7.3298E+00, 6.7580E+03, &
      3.1649020E-01, 2.62907E+01, 7.3523E+00, 6.7449E+03, &
      3.1807265E-01, 2.62883E+01, 7.3748E+00, 6.7320E+03, &
      3.1966301E-01, 2.62832E+01, 7.3974E+00, 6.7190E+03, &
      3.2126133E-01, 2.62745E+01, 7.4201E+00, 6.7061E+03, &
      3.2286764E-01, 2.62606E+01, 7.4429E+00, 6.6933E+03, &
      3.2448197E-01, 2.62390E+01, 7.4657E+00, 6.6805E+03, &
      3.2610438E-01, 2.62044E+01, 7.4886E+00, 6.6676E+03, &
      3.2773491E-01, 2.61438E+01, 7.5116E+00, 6.6548E+03, &
      3.2937358E-01, 2.60040E+01, 7.5346E+00, 6.6420E+03, &
      3.3005637E-01, 2.58298E+01, 7.5441E+00, 6.6367E+03, &
      3.3074361E-01, 2.58376E+01, 8.0762E+00, 7.0898E+03, &
      3.3102045E-01, 2.59390E+01, 8.0802E+00, 7.0874E+03, &
      3.3267555E-01, 2.61743E+01, 8.1044E+00, 7.0733E+03, &
      3.3433893E-01, 2.62855E+01, 8.1286E+00, 7.0591E+03, &
      3.3601062E-01, 2.63646E+01, 8.1528E+00, 7.0450E+03, &
      3.3769068E-01, 2.64290E+01, 8.1770E+00, 7.0308E+03, &
      3.3937913E-01, 2.64848E+01, 8.2013E+00, 7.0165E+03, &
      3.4107602E-01, 2.65351E+01, 8.2254E+00, 7.0022E+03, &
      3.4278140E-01, 2.65817E+01, 8.2496E+00, 6.9879E+03, &
      3.4449531E-01, 2.66255E+01, 8.2738E+00, 6.9735E+03, &
      3.4621779E-01, 2.66673E+01, 8.2979E+00, 6.9591E+03, &
      3.4794888E-01, 2.67075E+01, 8.3219E+00, 6.9445E+03, &
      3.4968862E-01, 2.67466E+01, 8.3459E+00, 6.9299E+03, &
      3.5143706E-01, 2.67847E+01, 8.3698E+00, 6.9152E+03, &
      3.5319425E-01, 2.68220E+01, 8.3937E+00, 6.9005E+03, &
      3.5496022E-01, 2.68588E+01, 8.4174E+00, 6.8856E+03, &
      3.5673502E-01, 2.68951E+01, 8.4411E+00, 6.8707E+03, &
      3.5851870E-01, 2.69311E+01, 8.4647E+00, 6.8556E+03, &
      3.6031129E-01, 2.69667E+01, 8.4881E+00, 6.8404E+03, &
      3.6211285E-01, 2.70045E+01, 8.5115E+00, 6.8251E+03, &
      3.6392341E-01, 2.70398E+01, 8.5347E+00, 6.8097E+03, &
      3.6574303E-01, 2.70750E+01, 8.5578E+00, 6.7942E+03, &
      3.6757174E-01, 2.71102E+01, 8.5807E+00, 6.7785E+03, &
      3.6940960E-01, 2.71453E+01, 8.6034E+00, 6.7627E+03, &
      3.7125665E-01, 2.71804E+01, 8.6259E+00, 6.7467E+03, &
      3.7311293E-01, 2.72155E+01, 8.6483E+00, 6.7306E+03, &
      3.7497850E-01, 2.72506E+01, 8.6704E+00, 6.7142E+03, &
      3.7685339E-01, 2.72858E+01, 8.6924E+00, 6.6978E+03, &
      3.7873766E-01, 2.73210E+01, 8.7141E+00, 6.6811E+03, &
      3.8063135E-01, 2.73563E+01, 8.7356E+00, 6.6643E+03, &
      3.8253450E-01, 2.73916E+01, 8.7568E+00, 6.6473E+03, &
      3.8444718E-01, 2.74271E+01, 8.7779E+00, 6.6302E+03, &
      3.8636941E-01, 2.74626E+01, 8.7986E+00, 6.6129E+03, &
      3.8830126E-01, 2.74982E+01, 8.8192E+00, 6.5953E+03, &
      3.9024276E-01, 2.75340E+01, 8.8394E+00, 6.5776E+03, &
      3.9219398E-01, 2.75698E+01, 8.8594E+00, 6.5597E+03, &
      3.9415495E-01, 2.76057E+01, 8.8791E+00, 6.5417E+03, &
      3.9612572E-01, 2.76417E+01, 8.8986E+00, 6.5234E+03, &
      3.9810635E-01, 2.76778E+01, 8.9176E+00, 6.5049E+03, &
      4.0009688E-01, 2.77141E+01, 8.9363E+00, 6.4861E+03, &
      4.0209737E-01, 2.77504E+01, 8.9548E+00, 6.4672E+03, &
      4.0410785E-01, 2.77868E+01, 8.9729E+00, 6.4481E+03, &
      4.0612839E-01, 2.78233E+01, 8.9907E+00, 6.4288E+03, &
      4.0815904E-01, 2.78599E+01, 9.0082E+00, 6.4093E+03, &
      4.1019983E-01, 2.78966E+01, 9.0254E+00, 6.3896E+03, &
      4.1225083E-01, 2.79334E+01, 9.0422E+00, 6.3697E+03, &
      4.1431208E-01, 2.79702E+01, 9.0587E+00, 6.3496E+03, &
      4.1638364E-01, 2.80071E+01, 9.0748E+00, 6.3293E+03, &
      4.1846556E-01, 2.80441E+01, 9.0906E+00, 6.3088E+03, &
      4.2055789E-01, 2.80811E+01, 9.1061E+00, 6.2881E+03, &
      4.2266068E-01, 2.81182E+01, 9.1211E+00, 6.2672E+03, &
      4.2477398E-01, 2.81554E+01, 9.1358E+00, 6.2461E+03, &
      4.2689785E-01, 2.81926E+01, 9.1501E+00, 6.2248E+03, &
      4.2903234E-01, 2.82298E+01, 9.1641E+00, 6.2034E+03, &
      4.3117750E-01, 2.82671E+01, 9.1777E+00, 6.1817E+03, &
      4.3333339E-01, 2.83044E+01, 9.1909E+00, 6.1598E+03, &
      4.3550006E-01, 2.83417E+01, 9.2037E+00, 6.1377E+03, &
      4.3767756E-01, 2.83790E+01, 9.2161E+00, 6.1155E+03, &
      4.3986595E-01, 2.84163E+01, 9.2281E+00, 6.0930E+03, &
      4.4206528E-01, 2.84536E+01, 9.2397E+00, 6.0704E+03, &
      4.4427560E-01, 2.84909E+01, 9.2509E+00, 6.0475E+03, &
      4.4649698E-01, 2.85282E+01, 9.2617E+00, 6.0245E+03, &
      4.4872947E-01, 2.85654E+01, 9.2721E+00, 6.0013E+03, &
      4.5097311E-01, 2.86027E+01, 9.2821E+00, 5.9779E+03, &
      4.5322798E-01, 2.86398E+01, 9.2916E+00, 5.9543E+03, &
      4.5549412E-01, 2.86769E+01, 9.3008E+00, 5.9306E+03, &
      4.5777159E-01, 2.87140E+01, 9.3095E+00, 5.9066E+03, &
      4.6006045E-01, 2.87509E+01, 9.3178E+00, 5.8825E+03, &
      4.6236075E-01, 2.87878E+01, 9.3257E+00, 5.8583E+03, &
      4.6467255E-01, 2.88246E+01, 9.3331E+00, 5.8338E+03, &
      4.6699592E-01, 2.88613E+01, 9.3401E+00, 5.8092E+03, &
      4.6933090E-01, 2.88979E+01, 9.3467E+00, 5.7844E+03, &
      4.7167755E-01, 2.89344E+01, 9.3529E+00, 5.7595E+03, &
      4.7403594E-01, 2.89708E+01, 9.3586E+00, 5.7344E+03, &
      4.7640612E-01, 2.90070E+01, 9.3639E+00, 5.7091E+03, &
      4.7878815E-01, 2.90431E+01, 9.3688E+00, 5.6837E+03, &
      4.8118209E-01, 2.90790E+01, 9.3732E+00, 5.6582E+03, &
      4.8358800E-01, 2.91148E+01, 9.3772E+00, 5.6324E+03, &
      4.8600594E-01, 2.91504E+01, 9.3806E+00, 5.6065E+03, &
      4.8843597E-01, 2.91858E+01, 9.3835E+00, 5.5804E+03, &
      4.9087815E-01, 2.92211E+01, 9.3859E+00, 5.5541E+03, &
      4.9333254E-01, 2.92560E+01, 9.3878E+00, 5.5276E+03, &
      4.9579920E-01, 2.92908E+01, 9.3892E+00, 5.5010E+03, &
      4.9827820E-01, 2.93253E+01, 9.3900E+00, 5.4741E+03, &
      5.0076959E-01, 2.93595E+01, 9.3903E+00, 5.4472E+03, &
      5.0327344E-01, 2.93934E+01, 9.3901E+00, 5.4200E+03, &
      5.0578980E-01, 2.94270E+01, 9.3895E+00, 5.3927E+03, &
      5.0831875E-01, 2.94602E+01, 9.3883E+00, 5.3652E+03, &
      5.1086035E-01, 2.94931E+01, 9.3866E+00, 5.3376E+03, &
      5.1341465E-01, 2.95257E+01, 9.3844E+00, 5.3099E+03, &
      5.1598172E-01, 2.95579E+01, 9.3818E+00, 5.2820E+03, &
      5.1856163E-01, 2.95897E+01, 9.3786E+00, 5.2540E+03, &
      5.2115444E-01, 2.96211E+01, 9.3750E+00, 5.2259E+03, &
      5.2376021E-01, 2.96521E+01, 9.3709E+00, 5.1977E+03, &
      5.2637901E-01, 2.96827E+01, 9.3663E+00, 5.1694E+03, &
      5.2901091E-01, 2.97128E+01, 9.3612E+00, 5.1409E+03, &
      5.3165596E-01, 2.97424E+01, 9.3557E+00, 5.1124E+03, &
      5.3431424E-01, 2.97716E+01, 9.3498E+00, 5.0838E+03, &
      5.3698581E-01, 2.98003E+01, 9.3433E+00, 5.0550E+03, &
      5.3967074E-01, 2.98285E+01, 9.3365E+00, 5.0262E+03, &
      5.4236910E-01, 2.98561E+01, 9.3291E+00, 4.9974E+03, &
      5.4508094E-01, 2.98833E+01, 9.3214E+00, 4.9684E+03, &
      5.4780635E-01, 2.99099E+01, 9.3132E+00, 4.9394E+03, &
      5.5054538E-01, 2.99359E+01, 9.3046E+00, 4.9104E+03, &
      5.5329810E-01, 2.99613E+01, 9.2955E+00, 4.8813E+03, &
      5.5606460E-01, 2.99862E+01, 9.2861E+00, 4.8521E+03, &
      5.5884492E-01, 3.00105E+01, 9.2762E+00, 4.8229E+03, &
      5.6163914E-01, 3.00341E+01, 9.2660E+00, 4.7936E+03, &
      5.6444734E-01, 3.00571E+01, 9.2553E+00, 4.7643E+03, &
      5.6726958E-01, 3.00795E+01, 9.2442E+00, 4.7350E+03, &
      5.7010592E-01, 3.01012E+01, 9.2328E+00, 4.7057E+03, &
      5.7295645E-01, 3.01222E+01, 9.2210E+00, 4.6764E+03, &
      5.7582123E-01, 3.01426E+01, 9.2088E+00, 4.6470E+03, &
      5.7870034E-01, 3.01622E+01, 9.1962E+00, 4.6176E+03, &
      5.8159384E-01, 3.01812E+01, 9.1833E+00, 4.5882E+03, &
      5.8450181E-01, 3.01994E+01, 9.1700E+00, 4.5589E+03, &
      5.8742432E-01, 3.02168E+01, 9.1563E+00, 4.5295E+03, &
      5.9036144E-01, 3.02335E+01, 9.1423E+00, 4.5001E+03, &
      5.9331325E-01, 3.02495E+01, 9.1280E+00, 4.4708E+03, &
      5.9627982E-01, 3.02646E+01, 9.1134E+00, 4.4415E+03, &
      5.9926122E-01, 3.02789E+01, 9.0984E+00, 4.4122E+03, &
      6.0225752E-01, 3.02924E+01, 9.0831E+00, 4.3829E+03, &
      6.0526881E-01, 3.03051E+01, 9.0674E+00, 4.3536E+03, &
      6.0829515E-01, 3.03170E+01, 9.0515E+00, 4.3244E+03, &
      6.1133663E-01, 3.03279E+01, 9.0353E+00, 4.2953E+03, &
      6.1439331E-01, 3.03380E+01, 9.0187E+00, 4.2661E+03, &
      6.1746528E-01, 3.03472E+01, 9.0019E+00, 4.2371E+03, &
      6.2055260E-01, 3.03555E+01, 8.9848E+00, 4.2080E+03, &
      6.2365537E-01, 3.03628E+01, 8.9674E+00, 4.1791E+03, &
      6.2677364E-01, 3.03692E+01, 8.9498E+00, 4.1502E+03, &
      6.2990751E-01, 3.03746E+01, 8.9318E+00, 4.1213E+03, &
      6.3305705E-01, 3.03790E+01, 8.9137E+00, 4.0925E+03, &
      6.3622234E-01, 3.03825E+01, 8.8952E+00, 4.0638E+03, &
      6.3940345E-01, 3.03849E+01, 8.8765E+00, 4.0351E+03, &
      6.4260046E-01, 3.03862E+01, 8.8576E+00, 4.0066E+03, &
      6.4581347E-01, 3.03865E+01, 8.8384E+00, 3.9781E+03, &
      6.4904253E-01, 3.03857E+01, 8.8190E+00, 3.9497E+03, &
      6.5228775E-01, 3.03839E+01, 8.7994E+00, 3.9213E+03, &
      6.5554919E-01, 3.03808E+01, 8.7796E+00, 3.8931E+03, &
      6.5882693E-01, 3.03767E+01, 8.7595E+00, 3.8649E+03, &
      6.6212107E-01, 3.03714E+01, 8.7392E+00, 3.8369E+03, &
      6.6543167E-01, 3.03648E+01, 8.7188E+00, 3.8089E+03, &
      6.6875883E-01, 3.03571E+01, 8.6981E+00, 3.7811E+03, &
      6.7210262E-01, 3.03481E+01, 8.6772E+00, 3.7533E+03, &
      6.7546314E-01, 3.03379E+01, 8.6562E+00, 3.7256E+03, &
      6.7884045E-01, 3.03264E+01, 8.6349E+00, 3.6981E+03, &
      6.8223466E-01, 3.03135E+01, 8.6135E+00, 3.6706E+03, &
      6.8564583E-01, 3.02993E+01, 8.5919E+00, 3.6433E+03, &
      6.8907406E-01, 3.02838E+01, 8.5702E+00, 3.6160E+03, &
      6.9251943E-01, 3.02668E+01, 8.5483E+00, 3.5889E+03, &
      6.9598202E-01, 3.02484E+01, 8.5262E+00, 3.5619E+03, &
      6.9946194E-01, 3.02285E+01, 8.5040E+00, 3.5350E+03, &
      7.0295924E-01, 3.02071E+01, 8.4816E+00, 3.5083E+03, &
      7.0647404E-01, 3.01841E+01, 8.4591E+00, 3.4816E+03, &
      7.1000641E-01, 3.01621E+01, 8.4364E+00, 3.4551E+03, &
      7.1355644E-01, 3.01360E+01, 8.4136E+00, 3.4287E+03, &
      7.1712423E-01, 3.01082E+01, 8.3907E+00, 3.4024E+03, &
      7.2070985E-01, 3.00788E+01, 8.3676E+00, 3.3762E+03, &
      7.2431340E-01, 3.00476E+01, 8.3444E+00, 3.3502E+03, &
      7.2793496E-01, 3.00146E+01, 8.3211E+00, 3.3243E+03, &
      7.3157464E-01, 2.99798E+01, 8.2977E+00, 3.2985E+03, &
      7.3523251E-01, 2.99431E+01, 8.2742E+00, 3.2729E+03, &
      7.3890867E-01, 2.99045E+01, 8.2506E+00, 3.2474E+03, &
      7.4260322E-01, 2.98639E+01, 8.2268E+00, 3.2220E+03, &
      7.4631623E-01, 2.98213E+01, 8.2030E+00, 3.1968E+03, &
      7.5004781E-01, 2.97766E+01, 8.1791E+00, 3.1717E+03, &
      7.5379805E-01, 2.97297E+01, 8.1552E+00, 3.1467E+03, &
      7.5756704E-01, 2.96806E+01, 8.1311E+00, 3.1219E+03, &
      7.6135488E-01, 2.96292E+01, 8.1070E+00, 3.0972E+03, &
      7.6516165E-01, 2.95776E+01, 8.0827E+00, 3.0727E+03, &
      7.6898746E-01, 2.95214E+01, 8.0585E+00, 3.0483E+03, &
      7.7283240E-01, 2.94627E+01, 8.0341E+00, 3.0240E+03, &
      7.7669656E-01, 2.94014E+01, 8.0097E+00, 2.9999E+03, &
      7.8058004E-01, 2.93375E+01, 7.9853E+00, 2.9760E+03, &
      7.8448294E-01, 2.92707E+01, 7.9607E+00, 2.9521E+03, &
      7.8840536E-01, 2.92010E+01, 7.9362E+00, 2.9285E+03, &
      7.9234738E-01, 2.91284E+01, 7.9115E+00, 2.9049E+03, &
      7.9630912E-01, 2.90527E+01, 7.8869E+00, 2.8815E+03, &
      8.0029067E-01, 2.89737E+01, 7.8622E+00, 2.8583E+03, &
      8.0429212E-01, 2.88915E+01, 7.8374E+00, 2.8352E+03, &
      8.0831358E-01, 2.88057E+01, 7.8126E+00, 2.8123E+03, &
      8.1235515E-01, 2.87164E+01, 7.7878E+00, 2.7895E+03, &
      8.1641693E-01, 2.86233E+01, 7.7630E+00, 2.7668E+03, &
      8.2049901E-01, 2.85262E+01, 7.7381E+00, 2.7443E+03, &
      8.2460150E-01, 2.84251E+01, 7.7132E+00, 2.7219E+03, &
      8.2872451E-01, 2.83197E+01, 7.6883E+00, 2.6997E+03, &
      8.3286813E-01, 2.82099E+01, 7.6634E+00, 2.6777E+03, &
      8.3703248E-01, 2.80953E+01, 7.6384E+00, 2.6557E+03, &
      8.4121764E-01, 2.79759E+01, 7.6135E+00, 2.6340E+03, &
      8.4542373E-01, 2.78513E+01, 7.5885E+00, 2.6124E+03, &
      8.4965084E-01, 2.77212E+01, 7.5635E+00, 2.5909E+03, &
      8.5389910E-01, 2.75855E+01, 7.5386E+00, 2.5696E+03, &
      8.5816859E-01, 2.74438E+01, 7.5136E+00, 2.5484E+03, &
      8.6245944E-01, 2.72957E+01, 7.4886E+00, 2.5274E+03, &
      8.6677173E-01, 2.71410E+01, 7.4636E+00, 2.5065E+03, &
      8.7110559E-01, 2.69791E+01, 7.4386E+00, 2.4857E+03, &
      8.7546112E-01, 2.68097E+01, 7.4137E+00, 2.4651E+03, &
      8.7983843E-01, 2.66323E+01, 7.3887E+00, 2.4447E+03, &
      8.8423762E-01, 2.64464E+01, 7.3637E+00, 2.4244E+03, &
      8.8865881E-01, 2.62514E+01, 7.3388E+00, 2.4042E+03, &
      8.9310210E-01, 2.60467E+01, 7.3139E+00, 2.3842E+03, &
      8.9756761E-01, 2.58315E+01, 7.2890E+00, 2.3644E+03, &
      9.0205545E-01, 2.56051E+01, 7.2641E+00, 2.3447E+03, &
      9.0656573E-01, 2.53667E+01, 7.2392E+00, 2.3251E+03, &
      9.1109856E-01, 2.51153E+01, 7.2144E+00, 2.3057E+03, &
      9.1565405E-01, 2.48497E+01, 7.1895E+00, 2.2864E+03, &
      9.2023232E-01, 2.45687E+01, 7.1647E+00, 2.2672E+03, &
      9.2483348E-01, 2.42709E+01, 7.1400E+00, 2.2482E+03, &
      9.2945765E-01, 2.39548E+01, 7.1152E+00, 2.2294E+03, &
      9.3410494E-01, 2.36184E+01, 7.0905E+00, 2.2107E+03, &
      9.3877546E-01, 2.32597E+01, 7.0658E+00, 2.1921E+03, &
      9.4346934E-01, 2.28761E+01, 7.0412E+00, 2.1737E+03, &
      9.4818668E-01, 2.24647E+01, 7.0165E+00, 2.1554E+03, &
      9.5292762E-01, 2.20220E+01, 6.9920E+00, 2.1372E+03, &
      9.5769226E-01, 2.15438E+01, 6.9674E+00, 2.1192E+03, &
      9.6248072E-01, 2.10251E+01, 6.9429E+00, 2.1013E+03, &
      9.6729312E-01, 2.04596E+01, 6.9184E+00, 2.0836E+03, &
      9.7212959E-01, 1.98395E+01, 6.8940E+00, 2.0660E+03, &
      9.7699023E-01, 1.91548E+01, 6.8696E+00, 2.0485E+03, &
      9.8187519E-01, 1.83927E+01, 6.8453E+00, 2.0312E+03, &
      9.8678456E-01, 1.75357E+01, 6.8210E+00, 2.0140E+03, &
      9.9171848E-01, 1.65599E+01, 6.7968E+00, 1.9969E+03, &
      9.9667708E-01, 1.54309E+01, 6.7726E+00, 1.9800E+03, &
      1.0016605E+00, 1.41039E+01, 6.7441E+00, 1.9620E+03, &
      1.0066688E+00, 1.24974E+01, 6.7071E+00, 1.9416E+03, &
      1.0117021E+00, 1.04350E+01, 6.6703E+00, 1.9214E+03, &
      1.0167606E+00, 7.56515E+00, 6.6338E+00, 1.9015E+03, &
      1.0218444E+00, 2.77945E+00, 6.5967E+00, 1.8815E+03, &
      1.0267933E+00, -2.21483E+01, 6.5608E+00, 1.8624E+03, &
      1.0269536E+00, -2.68792E+01, 2.6530E+01, 7.5054E+03, &
      1.0270069E+00, -2.24988E+01, 2.6528E+01, 7.5044E+03, &
      1.0320884E+00, 1.58433E+00, 2.6326E+01, 7.4106E+03, &
      1.0372489E+00, 4.98823E+00, 2.6123E+01, 7.3171E+03, &
      1.0424351E+00, 5.96659E+00, 2.5922E+01, 7.2248E+03, &
      1.0476473E+00, 4.51624E+00, 2.5723E+01, 7.1337E+03, &
      1.0513391E+00, -7.48748E+00, 2.5583E+01, 7.0701E+03, &
      1.0516608E+00, -7.60111E+00, 3.8691E+01, 1.0685E+04, &
      1.0528855E+00, 1.77888E+00, 3.8620E+01, 1.0653E+04, &
      1.0581499E+00, 9.77361E+00, 3.8318E+01, 1.0517E+04, &
      1.0634407E+00, 1.34392E+01, 3.8017E+01, 1.0383E+04, &
      1.0687579E+00, 1.60406E+01, 3.7719E+01, 1.0250E+04, &
      1.0741017E+00, 1.81099E+01, 3.7424E+01, 1.0120E+04, &
      1.0794722E+00, 1.98470E+01, 3.7131E+01, 9.9905E+03, &
      1.0848695E+00, 2.13520E+01, 3.6840E+01, 9.8631E+03, &
      1.0902939E+00, 2.26832E+01, 3.6551E+01, 9.7373E+03, &
      1.0957454E+00, 2.38785E+01, 3.6265E+01, 9.6131E+03, &
      1.1012241E+00, 2.49635E+01, 3.5982E+01, 9.4906E+03, &
      1.1067302E+00, 2.59570E+01, 3.5700E+01, 9.3696E+03, &
      1.1122639E+00, 2.68730E+01, 3.5421E+01, 9.2502E+03, &
      1.1178252E+00, 2.77223E+01, 3.5144E+01, 9.1323E+03, &
      1.1234143E+00, 2.85133E+01, 3.4869E+01, 9.0159E+03, &
      1.1290314E+00, 2.92530E+01, 3.4596E+01, 8.9011E+03, &
      1.1346765E+00, 2.99468E+01, 3.4326E+01, 8.7877E+03, &
      1.1403499E+00, 3.05995E+01, 3.4058E+01, 8.6758E+03, &
      1.1460517E+00, 3.12150E+01, 3.3792E+01, 8.5653E+03, &
      1.1517819E+00, 3.17965E+01, 3.3528E+01, 8.4562E+03, &
      1.1575408E+00, 3.23468E+01, 3.3266E+01, 8.3486E+03, &
      1.1633285E+00, 3.28684E+01, 3.3007E+01, 8.2423E+03, &
      1.1691452E+00, 3.33633E+01, 3.2749E+01, 8.1374E+03, &
      1.1749909E+00, 3.38334E+01, 3.2493E+01, 8.0339E+03, &
      1.1808659E+00, 3.42802E+01, 3.2240E+01, 7.9317E+03, &
      1.1867702E+00, 3.47051E+01, 3.1989E+01, 7.8308E+03, &
      1.1927040E+00, 3.51092E+01, 3.1739E+01, 7.7312E+03, &
      1.1986676E+00, 3.54938E+01, 3.1492E+01, 7.6329E+03, &
      1.2046609E+00, 3.58596E+01, 3.1246E+01, 7.5358E+03, &
      1.2106842E+00, 3.62074E+01, 3.1003E+01, 7.4401E+03, &
      1.2167376E+00, 3.65379E+01, 3.0761E+01, 7.3455E+03, &
      1.2228213E+00, 3.68517E+01, 3.0522E+01, 7.2521E+03, &
      1.2289354E+00, 3.71491E+01, 3.0284E+01, 7.1600E+03, &
      1.2350801E+00, 3.74307E+01, 3.0048E+01, 7.0690E+03, &
      1.2412555E+00, 3.76965E+01, 2.9815E+01, 6.9793E+03, &
      1.2474618E+00, 3.79467E+01, 2.9583E+01, 6.8906E+03, &
      1.2536991E+00, 3.81812E+01, 2.9353E+01, 6.8031E+03, &
      1.2599676E+00, 3.84001E+01, 2.9124E+01, 6.7168E+03, &
      1.2662674E+00, 3.86028E+01, 2.8898E+01, 6.6315E+03, &
      1.2725988E+00, 3.87889E+01, 2.8673E+01, 6.5473E+03, &
      1.2789618E+00, 3.89576E+01, 2.8451E+01, 6.4643E+03, &
      1.2853566E+00, 3.91078E+01, 2.8230E+01, 6.3823E+03, &
      1.2917833E+00, 3.92379E+01, 2.8010E+01, 6.3013E+03, &
      1.2982423E+00, 3.93457E+01, 2.7793E+01, 6.2214E+03, &
      1.3047335E+00, 3.94280E+01, 2.7577E+01, 6.1425E+03, &
      1.3112571E+00, 3.94803E+01, 2.7363E+01, 6.0646E+03, &
      1.3178134E+00, 3.94957E+01, 2.7151E+01, 5.9878E+03, &
      1.3244025E+00, 3.94633E+01, 2.6940E+01, 5.9118E+03, &
      1.3310245E+00, 3.93642E+01, 2.6731E+01, 5.8369E+03, &
      1.3376796E+00, 3.91625E+01, 2.6524E+01, 5.7631E+03, &
      1.3443680E+00, 3.87743E+01, 2.6321E+01, 5.6906E+03, &
      1.3510899E+00, 3.79059E+01, 2.6119E+01, 5.6190E+03, &
      1.3559325E+00, 3.55390E+01, 2.5976E+01, 5.5682E+03, &
      1.3578453E+00, 3.54989E+01, 3.0344E+01, 6.4940E+03, &
      1.3578674E+00, 3.55324E+01, 3.0343E+01, 6.4938E+03, &
      1.3646345E+00, 3.86761E+01, 3.0096E+01, 6.4090E+03, &
      1.3714577E+00, 3.97544E+01, 2.9851E+01, 6.3253E+03, &
      1.3783150E+00, 4.04681E+01, 2.9608E+01, 6.2427E+03, &
      1.3852066E+00, 4.10163E+01, 2.9367E+01, 6.1613E+03, &
      1.3921326E+00, 4.14657E+01, 2.9129E+01, 6.0810E+03, &
      1.3990933E+00, 4.18464E+01, 2.8892E+01, 6.0017E+03, &
      1.4060887E+00, 4.21743E+01, 2.8658E+01, 5.9235E+03, &
      1.4131192E+00, 4.24576E+01, 2.8425E+01, 5.8464E+03, &
      1.4201848E+00, 4.27000E+01, 2.8195E+01, 5.7702E+03, &
      1.4272857E+00, 4.29021E+01, 2.7977E+01, 5.6972E+03, &
      1.4344221E+00, 4.30731E+01, 2.7771E+01, 5.6273E+03, &
      1.4415942E+00, 4.32089E+01, 2.7568E+01, 5.5586E+03, &
      1.4488022E+00, 4.32972E+01, 2.7369E+01, 5.4909E+03, &
      1.4560462E+00, 4.33111E+01, 2.7171E+01, 5.4243E+03, &
      1.4633265E+00, 4.31658E+01, 2.6977E+01, 5.3588E+03, &
      1.4703391E+00, 4.21883E+01, 2.6792E+01, 5.2969E+03, &
      1.4706431E+00, 4.19985E+01, 2.6784E+01, 5.2942E+03, &
      1.4724609E+00, 4.22231E+01, 2.8592E+01, 5.6439E+03, &
      1.4779963E+00, 4.34706E+01, 2.8433E+01, 5.5917E+03, &
      1.4853863E+00, 4.41218E+01, 2.8225E+01, 5.5232E+03, &
      1.4928132E+00, 4.45680E+01, 2.8019E+01, 5.4557E+03, &
      1.5002773E+00, 4.49287E+01, 2.7815E+01, 5.3893E+03, &
      1.5077787E+00, 4.52394E+01, 2.7614E+01, 5.3238E+03, &
      1.5153176E+00, 4.55159E+01, 2.7416E+01, 5.2593E+03, &
      1.5228942E+00, 4.57663E+01, 2.7219E+01, 5.1958E+03, &
      1.5305086E+00, 4.59953E+01, 2.7025E+01, 5.1332E+03, &
      1.5381612E+00, 4.62055E+01, 2.6834E+01, 5.0715E+03, &
      1.5458520E+00, 4.63980E+01, 2.6646E+01, 5.0112E+03, &
      1.5535812E+00, 4.65781E+01, 2.6469E+01, 4.9531E+03, &
      1.5613491E+00, 4.67480E+01, 2.6294E+01, 4.8960E+03, &
      1.5691559E+00, 4.69076E+01, 2.6121E+01, 4.8398E+03, &
      1.5770017E+00, 4.70567E+01, 2.5951E+01, 4.7844E+03, &
      1.5848867E+00, 4.71947E+01, 2.5783E+01, 4.7298E+03, &
      1.5928111E+00, 4.73209E+01, 2.5617E+01, 4.6761E+03, &
      1.6007752E+00, 4.74334E+01, 2.5453E+01, 4.6232E+03, &
      1.6087790E+00, 4.75295E+01, 2.5291E+01, 4.5711E+03, &
      1.6168229E+00, 4.76037E+01, 2.5131E+01, 4.5197E+03, &
      1.6249070E+00, 4.76449E+01, 2.4974E+01, 4.4691E+03, &
      1.6330316E+00, 4.76250E+01, 2.4818E+01, 4.4192E+03, &
      1.6411967E+00, 4.74301E+01, 2.4663E+01, 4.3700E+03, &
      1.6442772E+00, 4.71621E+01, 2.4606E+01, 4.3517E+03, &
      1.6487228E+00, 4.72348E+01, 2.5699E+01, 4.5323E+03, &
      1.6494027E+00, 4.73473E+01, 2.5686E+01, 4.5282E+03, &
      1.6576497E+00, 4.79992E+01, 2.5531E+01, 4.4786E+03, &
      1.6659380E+00, 4.83513E+01, 2.5378E+01, 4.4296E+03, &
      1.6742677E+00, 4.86254E+01, 2.5226E+01, 4.3813E+03, &
      1.6826390E+00, 4.88617E+01, 2.5076E+01, 4.3337E+03, &
      1.6910522E+00, 4.90749E+01, 2.4927E+01, 4.2867E+03, &
      1.6995075E+00, 4.92722E+01, 2.4780E+01, 4.2402E+03, &
      1.7080050E+00, 4.94578E+01, 2.4634E+01, 4.1944E+03, &
      1.7165450E+00, 4.96341E+01, 2.4490E+01, 4.1492E+03, &
      1.7251278E+00, 4.98028E+01, 2.4347E+01, 4.1045E+03, &
      1.7337534E+00, 4.99652E+01, 2.4205E+01, 4.0605E+03, &
      1.7424222E+00, 5.01230E+01, 2.4066E+01, 4.0171E+03, &
      1.7511343E+00, 5.02769E+01, 2.3928E+01, 3.9743E+03, &
      1.7598899E+00, 5.04276E+01, 2.3791E+01, 3.9320E+03, &
      1.7686894E+00, 5.05752E+01, 2.3651E+01, 3.8895E+03, &
      1.7775328E+00, 5.07180E+01, 2.3508E+01, 3.8469E+03, &
      1.7864205E+00, 5.08566E+01, 2.3367E+01, 3.8048E+03, &
      1.7953526E+00, 5.09915E+01, 2.3227E+01, 3.7632E+03, &
      1.8043294E+00, 5.11230E+01, 2.3087E+01, 3.7221E+03, &
      1.8133510E+00, 5.12513E+01, 2.2949E+01, 3.6815E+03, &
      1.8224178E+00, 5.13767E+01, 2.2811E+01, 3.6413E+03, &
      1.8315299E+00, 5.14995E+01, 2.2675E+01, 3.6016E+03, &
      1.8406875E+00, 5.16197E+01, 2.2539E+01, 3.5624E+03, &
      1.8498909E+00, 5.17377E+01, 2.2404E+01, 3.5235E+03, &
      1.8591404E+00, 5.18534E+01, 2.2270E+01, 3.4852E+03, &
      1.8684361E+00, 5.19672E+01, 2.2137E+01, 3.4472E+03, &
      1.8777783E+00, 5.20791E+01, 2.2005E+01, 3.4097E+03, &
      1.8871672E+00, 5.21892E+01, 2.1874E+01, 3.3726E+03, &
      1.8966030E+00, 5.22979E+01, 2.1744E+01, 3.3359E+03, &
      1.9060860E+00, 5.24052E+01, 2.1614E+01, 3.2996E+03, &
      1.9156165E+00, 5.25112E+01, 2.1484E+01, 3.2635E+03, &
      1.9251945E+00, 5.26150E+01, 2.1352E+01, 3.2274E+03, &
      1.9348205E+00, 5.27166E+01, 2.1220E+01, 3.1916E+03, &
      1.9444946E+00, 5.28161E+01, 2.1090E+01, 3.1563E+03, &
      1.9542171E+00, 5.29137E+01, 2.0960E+01, 3.1213E+03, &
      1.9639882E+00, 5.30095E+01, 2.0830E+01, 3.0867E+03, &
      1.9738081E+00, 5.31035E+01, 2.0702E+01, 3.0525E+03, &
      1.9836772E+00, 5.31960E+01, 2.0574E+01, 3.0187E+03, &
      1.9935955E+00, 5.32868E+01, 2.0447E+01, 2.9852E+03, &
      2.0035635E+00, 5.33762E+01, 2.0321E+01, 2.9521E+03, &
      2.0135813E+00, 5.34641E+01, 2.0195E+01, 2.9193E+03, &
      2.0236492E+00, 5.35507E+01, 2.0070E+01, 2.8869E+03, &
      2.0337675E+00, 5.36360E+01, 1.9946E+01, 2.8549E+03, &
      2.0439363E+00, 5.37201E+01, 1.9822E+01, 2.8232E+03, &
      2.0541560E+00, 5.40324E+01, 1.9699E+01, 2.7918E+03, &
      2.0644268E+00, 5.41145E+01, 1.9571E+01, 2.7599E+03, &
      2.0747489E+00, 5.41948E+01, 1.9443E+01, 2.7283E+03, &
      2.0851227E+00, 5.42733E+01, 1.9316E+01, 2.6970E+03, &
      2.0955483E+00, 5.43502E+01, 1.9189E+01, 2.6662E+03, &
      2.1060260E+00, 5.45801E+01, 1.9062E+01, 2.6355E+03, &
      2.1165562E+00, 5.46540E+01, 1.8933E+01, 2.6047E+03, &
      2.1271389E+00, 5.47261E+01, 1.8805E+01, 2.5743E+03, &
      2.1377746E+00, 5.47965E+01, 1.8678E+01, 2.5442E+03, &
      2.1484635E+00, 5.48651E+01, 1.8551E+01, 2.5144E+03, &
      2.1592058E+00, 5.49318E+01, 1.8424E+01, 2.4849E+03, &
      2.1700018E+00, 5.49968E+01, 1.8298E+01, 2.4558E+03, &
      2.1808519E+00, 5.50600E+01, 1.8173E+01, 2.4270E+03, &
      2.1917561E+00, 5.51217E+01, 1.8049E+01, 2.3985E+03, &
      2.2027149E+00, 5.51819E+01, 1.7926E+01, 2.3704E+03, &
      2.2137285E+00, 5.52405E+01, 1.7803E+01, 2.3425E+03, &
      2.2247971E+00, 5.52978E+01, 1.7682E+01, 2.3151E+03, &
      2.2359211E+00, 5.53536E+01, 1.7561E+01, 2.2879E+03, &
      2.2471007E+00, 5.54081E+01, 1.7441E+01, 2.2610E+03, &
      2.2583362E+00, 5.54613E+01, 1.7322E+01, 2.2345E+03, &
      2.2696279E+00, 5.55133E+01, 1.7201E+01, 2.2080E+03, &
      2.2809760E+00, 5.55640E+01, 1.7081E+01, 2.1818E+03, &
      2.2923809E+00, 5.56134E+01, 1.6961E+01, 2.1558E+03, &
      2.3038428E+00, 5.56614E+01, 1.6843E+01, 2.1302E+03, &
      2.3153620E+00, 5.57083E+01, 1.6725E+01, 2.1049E+03, &
      2.3269388E+00, 5.57540E+01, 1.6608E+01, 2.0798E+03, &
      2.3385735E+00, 5.57984E+01, 1.6492E+01, 2.0551E+03, &
      2.3502664E+00, 5.58418E+01, 1.6377E+01, 2.0307E+03, &
      2.3620177E+00, 5.58840E+01, 1.6262E+01, 2.0066E+03, &
      2.3738278E+00, 5.59252E+01, 1.6149E+01, 1.9828E+03, &
      2.3856970E+00, 5.59653E+01, 1.6036E+01, 1.9592E+03, &
      2.3976254E+00, 5.60044E+01, 1.5925E+01, 1.9360E+03, &
      2.4096136E+00, 5.60425E+01, 1.5814E+01, 1.9130E+03, &
      2.4216616E+00, 5.60797E+01, 1.5704E+01, 1.8904E+03, &
      2.4337699E+00, 5.61159E+01, 1.5594E+01, 1.8680E+03, &
      2.4459388E+00, 5.61512E+01, 1.5486E+01, 1.8459E+03, &
      2.4581685E+00, 5.61857E+01, 1.5378E+01, 1.8240E+03, &
      2.4704593E+00, 5.62192E+01, 1.5272E+01, 1.8024E+03, &
      2.4828116E+00, 5.62520E+01, 1.5166E+01, 1.7811E+03, &
      2.4952257E+00, 5.62839E+01, 1.5061E+01, 1.7601E+03, &
      2.5077018E+00, 5.63151E+01, 1.4957E+01, 1.7393E+03, &
      2.5202403E+00, 5.63455E+01, 1.4853E+01, 1.7188E+03, &
      2.5328415E+00, 5.63751E+01, 1.4751E+01, 1.6985E+03, &
      2.5455057E+00, 5.64041E+01, 1.4649E+01, 1.6785E+03, &
      2.5582333E+00, 5.64324E+01, 1.4548E+01, 1.6587E+03, &
      2.5710244E+00, 5.64600E+01, 1.4448E+01, 1.6392E+03, &
      2.5838796E+00, 5.64870E+01, 1.4348E+01, 1.6199E+03, &
      2.5967990E+00, 5.65133E+01, 1.4250E+01, 1.6008E+03, &
      2.6097829E+00, 5.65391E+01, 1.4152E+01, 1.5820E+03, &
      2.6228319E+00, 5.65643E+01, 1.4055E+01, 1.5635E+03, &
      2.6359460E+00, 5.65890E+01, 1.3959E+01, 1.5451E+03, &
      2.6491257E+00, 5.66132E+01, 1.3864E+01, 1.5270E+03, &
      2.6623714E+00, 5.66369E+01, 1.3769E+01, 1.5091E+03, &
      2.6756832E+00, 5.66601E+01, 1.3675E+01, 1.4915E+03, &
      2.6890617E+00, 5.66830E+01, 1.3582E+01, 1.4740E+03, &
      2.7025070E+00, 5.67054E+01, 1.3490E+01, 1.4568E+03, &
      2.7160195E+00, 5.68219E+01, 1.3398E+01, 1.4398E+03, &
      2.7295996E+00, 5.68437E+01, 1.3305E+01, 1.4228E+03, &
      2.7432476E+00, 5.68650E+01, 1.3213E+01, 1.4060E+03, &
      2.7569638E+00, 5.68858E+01, 1.3121E+01, 1.3894E+03, &
      2.7707486E+00, 5.69061E+01, 1.3031E+01, 1.3730E+03, &
      2.7846024E+00, 5.69259E+01, 1.2941E+01, 1.3568E+03, &
      2.7985254E+00, 5.69453E+01, 1.2852E+01, 1.3409E+03, &
      2.8125180E+00, 5.69644E+01, 1.2763E+01, 1.3251E+03, &
      2.8265806E+00, 5.69831E+01, 1.2676E+01, 1.3095E+03, &
      2.8407135E+00, 5.70015E+01, 1.2589E+01, 1.2942E+03, &
      2.8549171E+00, 5.70196E+01, 1.2502E+01, 1.2790E+03, &
      2.8691917E+00, 5.70375E+01, 1.2417E+01, 1.2640E+03, &
      2.8835376E+00, 5.70553E+01, 1.2332E+01, 1.2492E+03, &
      2.8979553E+00, 5.70730E+01, 1.2247E+01, 1.2345E+03, &
      2.9124451E+00, 5.70906E+01, 1.2164E+01, 1.2201E+03, &
      2.9270073E+00, 5.71082E+01, 1.2081E+01, 1.2058E+03, &
      2.9416424E+00, 5.71260E+01, 1.1999E+01, 1.1918E+03, &
      2.9563506E+00, 5.71955E+01, 1.1916E+01, 1.1778E+03, &
      2.9711323E+00, 5.72142E+01, 1.1835E+01, 1.1639E+03, &
      2.9859880E+00, 5.72332E+01, 1.1753E+01, 1.1503E+03, &
      3.0009179E+00, 5.72688E+01, 1.1672E+01, 1.1367E+03, &
      3.0159225E+00, 5.73099E+01, 1.1578E+01, 1.1221E+03, &
      3.0310021E+00, 5.73283E+01, 1.1485E+01, 1.1076E+03, &
      3.0461571E+00, 5.73431E+01, 1.1393E+01, 1.0933E+03, &
      3.0613879E+00, 5.73556E+01, 1.1302E+01, 1.0793E+03, &
      3.0766949E+00, 5.73663E+01, 1.1211E+01, 1.0654E+03, &
      3.0920783E+00, 5.73755E+01, 1.1121E+01, 1.0517E+03, &
      3.1075387E+00, 5.73833E+01, 1.1033E+01, 1.0382E+03, &
      3.1230764E+00, 5.73900E+01, 1.0945E+01, 1.0249E+03, &
      3.1386918E+00, 5.73956E+01, 1.0857E+01, 1.0117E+03, &
      3.1543853E+00, 5.74003E+01, 1.0771E+01, 9.9874E+02, &
      3.1701572E+00, 5.74040E+01, 1.0685E+01, 9.8595E+02, &
      3.1860080E+00, 5.74069E+01, 1.0600E+01, 9.7333E+02, &
      3.2019380E+00, 5.74090E+01, 1.0516E+01, 9.6089E+02, &
      3.2179477E+00, 5.74104E+01, 1.0433E+01, 9.4861E+02, &
      3.2340374E+00, 5.74111E+01, 1.0350E+01, 9.3649E+02, &
      3.2502076E+00, 5.74112E+01, 1.0268E+01, 9.2454E+02, &
      3.2664587E+00, 5.74106E+01, 1.0187E+01, 9.1276E+02, &
      3.2827910E+00, 5.74093E+01, 1.0106E+01, 9.0112E+02, &
      3.2992049E+00, 5.74285E+01, 1.0026E+01, 8.8963E+02, &
      3.3157009E+00, 5.74263E+01, 9.9467E+00, 8.7825E+02, &
      3.3322794E+00, 5.74234E+01, 9.8677E+00, 8.6703E+02, &
      3.3489408E+00, 5.74200E+01, 9.7894E+00, 8.5595E+02, &
      3.3656856E+00, 5.74160E+01, 9.7119E+00, 8.4503E+02, &
      3.3825140E+00, 5.74114E+01, 9.6350E+00, 8.3425E+02, &
      3.3994265E+00, 5.74063E+01, 9.5588E+00, 8.2362E+02, &
      3.4164237E+00, 5.74007E+01, 9.4833E+00, 8.1313E+02, &
      3.4335058E+00, 5.73946E+01, 9.4084E+00, 8.0278E+02, &
      3.4506733E+00, 5.73880E+01, 9.3343E+00, 7.9256E+02, &
      3.4679267E+00, 5.73809E+01, 9.2607E+00, 7.8249E+02, &
      3.4852663E+00, 5.73734E+01, 9.1879E+00, 7.7255E+02, &
      3.5026927E+00, 5.73654E+01, 9.1157E+00, 7.6274E+02, &
      3.5202061E+00, 5.73569E+01, 9.0441E+00, 7.5307E+02, &
      3.5378072E+00, 5.73481E+01, 8.9731E+00, 7.4352E+02, &
      3.5554962E+00, 5.73388E+01, 8.9028E+00, 7.3410E+02, &
      3.5732737E+00, 5.73291E+01, 8.8331E+00, 7.2481E+02, &
      3.5911400E+00, 5.73189E+01, 8.7640E+00, 7.1564E+02, &
      3.6090957E+00, 5.73084E+01, 8.6955E+00, 7.0660E+02, &
      3.6271412E+00, 5.72975E+01, 8.6277E+00, 6.9767E+02, &
      3.6452769E+00, 5.72862E+01, 8.5604E+00, 6.8886E+02, &
      3.6635033E+00, 5.72745E+01, 8.4937E+00, 6.8017E+02, &
      3.6818208E+00, 5.72625E+01, 8.4276E+00, 6.7160E+02, &
      3.7002299E+00, 5.72500E+01, 8.3621E+00, 6.6314E+02, &
      3.7187311E+00, 5.72373E+01, 8.2972E+00, 6.5479E+02, &
      3.7373247E+00, 5.72241E+01, 8.2328E+00, 6.4655E+02, &
      3.7560114E+00, 5.72106E+01, 8.1690E+00, 6.3842E+02, &
      3.7747914E+00, 5.71967E+01, 8.1057E+00, 6.3040E+02, &
      3.7936654E+00, 5.71825E+01, 8.0431E+00, 6.2249E+02, &
      3.8126337E+00, 5.71679E+01, 7.9809E+00, 6.1468E+02, &
      3.8316969E+00, 5.71530E+01, 7.9193E+00, 6.0698E+02, &
      3.8508554E+00, 5.71378E+01, 7.8582E+00, 5.9937E+02, &
      3.8701096E+00, 5.71222E+01, 7.7977E+00, 5.9187E+02, &
      3.8894602E+00, 5.71063E+01, 7.7377E+00, 5.8446E+02, &
      3.9089075E+00, 5.70901E+01, 7.6782E+00, 5.7716E+02, &
      3.9284520E+00, 5.70735E+01, 7.6193E+00, 5.6995E+02, &
      3.9480943E+00, 5.70566E+01, 7.5608E+00, 5.6283E+02, &
      3.9678347E+00, 5.70393E+01, 7.5029E+00, 5.5581E+02, &
      3.9876739E+00, 5.70217E+01, 7.4454E+00, 5.4888E+02, &
      4.055024E+00, 5.72709E+01, 7.2561E+00, 5.2631E+02, &
      4.334821E+00, 5.69459E+01, 6.5544E+00, 4.4556E+02, &
      4.633924E+00, 5.65589E+01, 5.9072E+00, 3.7645E+02, &
      4.953664E+00, 5.60639E+01, 5.3120E+00, 3.1744E+02, &
      5.295467E+00, 5.54141E+01, 4.7850E+00, 2.6820E+02, &
      5.660855E+00, 5.44933E+01, 4.3185E+00, 2.2710E+02, &
      6.051453E+00, 5.29075E+01, 3.8956E+00, 1.9226E+02, &
      6.330114E+00, 5.02829E+01, 3.6294E+00, 1.7165E+02, &
      6.427004E+00, 4.71930E+01, 3.5428E+00, 1.6518E+02, &
      6.452841E+00, 4.36091E+01, 3.5200E+00, 1.6350E+02, &
      6.469004E+00, 4.44402E+01, 1.0534E+01, 4.7775E+02, &
      6.491597E+00, 4.71568E+01, 1.0459E+01, 4.7272E+02, &
      6.588486E+00, 5.02049E+01, 1.0145E+01, 4.5198E+02, &
      6.872544E+00, 5.17067E+01, 9.3341E+00, 3.9910E+02, &
      6.915365E+00, 5.15211E+01, 9.2306E+00, 3.9228E+02, &
      6.977736E+00, 5.06636E+01, 9.0844E+00, 3.8271E+02, &
      7.005787E+00, 4.89754E+01, 9.0205E+00, 3.7852E+02, &
      7.047864E+00, 5.09205E+01, 1.2435E+01, 5.1687E+02, &
      7.153056E+00, 5.27796E+01, 1.2089E+01, 4.9524E+02, &
      7.279342E+00, 5.36031E+01, 1.1696E+01, 4.7099E+02, &
      7.392525E+00, 5.35120E+01, 1.1369E+01, 4.5098E+02, &
      7.420472E+00, 5.27858E+01, 1.1295E+01, 4.4640E+02, &
      7.465039E+00, 5.38921E+01, 1.2942E+01, 5.0775E+02, &
      7.576458E+00, 5.51658E+01, 1.2650E+01, 4.8912E+02, &
      7.902609E+00, 5.68936E+01, 1.1871E+01, 4.4034E+02, &
      8.447890E+00, 5.84739E+01, 1.0736E+01, 3.7297E+02, &
      9.030794E+00, 5.95133E+01, 9.6674E+00, 3.1457E+02, &
      9.653919E+00, 6.02080E+01, 8.6411E+00, 2.6342E+02, &
      1.032004E+01, 6.06562E+01, 7.7194E+00, 2.2052E+02, &
      1.103212E+01, 6.09484E+01, 6.9021E+00, 1.8477E+02, &
      1.179334E+01, 6.11371E+01, 6.1767E+00, 1.5500E+02, &
      1.260708E+01, 6.12555E+01, 5.5326E+00, 1.3016E+02, &
      1.347697E+01, 6.13937E+01, 4.9456E+00, 1.0910E+02, &
      1.440688E+01, 6.14520E+01, 4.4134E+00, 9.1323E+01, &
      1.540095E+01, 6.14702E+01, 3.9290E+00, 7.6282E+01, &
      1.646362E+01, 6.14233E+01, 3.4954E+00, 6.3694E+01, &
      1.759961E+01, 6.13558E+01, 3.1123E+00, 5.3244E+01, &
      1.881398E+01, 6.12751E+01, 2.7736E+00, 4.4562E+01, &
      2.011215E+01, 6.11858E+01, 2.4740E+00, 3.7339E+01, &
      2.149988E+01, 6.10911E+01, 2.2088E+00, 3.1326E+01, &
      2.298338E+01, 6.09926E+01, 1.9735E+00, 2.6311E+01, &
      2.456923E+01, 6.08916E+01, 1.7648E+00, 2.2125E+01, &
      2.626450E+01, 6.07901E+01, 1.5796E+00, 1.8631E+01, &
      2.807676E+01, 6.06889E+01, 1.4043E+00, 1.5597E+01, &
      3.001405E+01, 6.05693E+01, 1.2431E+00, 1.3011E+01, &
      3.208502E+01, 6.04331E+01, 1.1000E+00, 1.0859E+01, &
      3.429889E+01, 6.02729E+01, 9.6981E-01, 9.0407E+00, &
      3.666551E+01, 6.00727E+01, 8.5514E-01, 7.5343E+00, &
      3.919543E+01, 5.97978E+01, 7.5440E-01, 6.2882E+00, &
      4.189992E+01, 5.93350E+01, 6.6606E-01, 5.2579E+00, &
      4.428032E+01, 5.82962E+01, 6.0115E-01, 4.5404E+00, &
      4.479101E+01, 5.75906E+01, 5.8853E-01, 4.4049E+00, &
      4.495808E+01, 5.71110E+01, 5.8449E-01, 4.3619E+00, &
      4.513881E+01, 5.56908E+01, 5.8017E-01, 4.3160E+00, &
      4.540992E+01, 5.71266E+01, 3.3700E+00, 2.2118E+01, &
      4.608768E+01, 5.84243E+01, 3.2928E+00, 2.1305E+01, &
      4.788159E+01, 5.94838E+01, 3.1007E+00, 1.9337E+01, &
      5.118542E+01, 6.02477E+01, 2.7819E+00, 1.6273E+01, &
      5.471721E+01, 6.06492E+01, 2.4927E+00, 1.3681E+01, &
      5.849270E+01, 6.09005E+01, 2.2324E+00, 1.1500E+01, &
      6.252870E+01, 6.10674E+01, 1.9950E+00, 9.6502E+00, &
      6.684318E+01, 6.11788E+01, 1.7825E+00, 8.0999E+00, &
      7.145536E+01, 6.12539E+01, 1.5924E+00, 6.8017E+00, &
      7.638578E+01, 6.13038E+01, 1.4225E+00, 5.7146E+00, &
      8.165640E+01, 6.13369E+01, 1.2691E+00, 4.7982E+00, &
      8.729069E+01, 6.13547E+01, 1.1288E+00, 4.0211E+00, &
      9.331374E+01, 6.13914E+01, 1.0024E+00, 3.3678E+00, &
      9.975239E+01, 6.13874E+01, 8.8835E-01, 2.8187E+00, &
      1.066353E+02, 6.13770E+01, 7.8738E-01, 2.3625E+00, &
      1.139931E+02, 6.13625E+01, 6.9796E-01, 1.9833E+00, &
      1.218587E+02, 6.13452E+01, 6.1874E-01, 1.6681E+00, &
      1.302669E+02, 6.13264E+01, 5.4857E-01, 1.4058E+00, &
      1.392553E+02, 6.13067E+01, 4.8639E-01, 1.1875E+00, &
      1.488640E+02, 6.12867E+01, 4.3129E-01, 1.0058E+00, &
      1.591356E+02, 6.12669E+01, 3.8247E-01, 8.5431E-01, &
      1.701159E+02, 6.12475E+01, 3.3920E-01, 7.2803E-01, &
      1.818539E+02, 6.12287E+01, 3.0084E-01, 6.2265E-01, &
      1.944018E+02, 6.12105E+01, 2.6684E-01, 5.3463E-01, &
      2.078156E+02, 6.11934E+01, 2.3733E-01, 4.6190E-01, &
      2.221548E+02, 6.11775E+01, 2.1117E-01, 4.0095E-01, &
      2.374835E+02, 6.11627E+01, 1.8790E-01, 3.4970E-01, &
      2.538699E+02, 6.11489E+01, 1.6721E-01, 3.0654E-01, &
      2.713869E+02, 6.11362E+01, 1.4880E-01, 2.7011E-01, &
      2.901126E+02, 6.11244E+01, 1.3243E-01, 2.3931E-01, &
      3.101304E+02, 6.11135E+01, 1.1786E-01, 2.1320E-01, &
      3.315294E+02, 6.11035E+01, 1.0491E-01, 1.9099E-01, &
      3.544049E+02, 6.10942E+01, 9.3380E-02, 1.7205E-01, &
      3.788588E+02, 6.10857E+01, 8.3124E-02, 1.5584E-01, &
      4.050001E+02, 6.10779E+01, 7.3998E-02, 1.4189E-01, &
      4.329451E+02, 6.10707E+01, 6.5877E-02, 1.2985E-01]
   case(62)
      frel = -5.5500E-01
      allocate(nist_tab(3588))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.390000E-03, 1.26627E+01, 0.0000E+00, 6.9802E-06, &
      5.472500E-03, 1.26395E+01, 0.0000E+00, 7.5801E-06, &
      5.494500E-03, 1.26200E+01, 0.0000E+00, 7.7466E-06, &
      5.527500E-03, 1.26329E+01, 3.3428E-02, 1.6925E+03, &
      5.610000E-03, 1.26384E+01, 3.5067E-02, 1.7494E+03, &
      1.069000E-02, 1.19895E+01, 1.9065E-01, 4.9912E+03, &
      1.142761E-02, 1.18453E+01, 2.1975E-01, 5.3817E+03, &
      1.221612E-02, 1.16714E+01, 2.5270E-01, 5.7893E+03, &
      1.305903E-02, 1.14588E+01, 2.9009E-01, 6.2168E+03, &
      1.396010E-02, 1.11936E+01, 3.3254E-01, 6.6666E+03, &
      1.492335E-02, 1.08539E+01, 3.8080E-01, 7.1413E+03, &
      1.595306E-02, 1.04023E+01, 4.3567E-01, 7.6430E+03, &
      1.705382E-02, 9.80180E+00, 4.9806E-01, 8.1735E+03, &
      1.823053E-02, 8.82703E+00, 5.6895E-01, 8.7342E+03, &
      1.948844E-02, 7.04772E+00, 6.4942E-01, 9.3260E+03, &
      2.083314E-02, 1.54367E+00, 7.4060E-01, 9.9489E+03, &
      2.087400E-02, 1.13415E+00, 7.4345E-01, 9.9677E+03, &
      2.119350E-02, -5.42605E+00, 7.6591E-01, 1.0114E+04, &
      2.127870E-02, -1.35144E+01, 7.7195E-01, 1.0153E+04, &
      2.140650E-02, -4.77791E+00, 1.6823E+01, 2.1994E+05, &
      2.172600E-02, 2.94811E+00, 1.6205E+01, 2.0874E+05, &
      2.227063E-02, 7.74165E+00, 1.5230E+01, 1.9139E+05, &
      2.380730E-02, 1.31791E+01, 1.2929E+01, 1.5198E+05, &
      2.545001E-02, 1.56856E+01, 1.0907E+01, 1.1994E+05, &
      2.720606E-02, 1.69697E+01, 9.1675E+00, 9.4305E+04, &
      2.908327E-02, 1.75776E+01, 7.7466E+00, 7.4545E+04, &
      3.109002E-02, 1.77918E+01, 6.6351E+00, 5.9727E+04, &
      3.323523E-02, 1.77805E+01, 5.7999E+00, 4.8839E+04, &
      3.552846E-02, 1.76467E+01, 5.1995E+00, 4.0958E+04, &
      3.665200E-02, 1.75585E+01, 4.9880E+00, 3.8087E+04, &
      3.721300E-02, 1.75059E+01, 4.8988E+00, 3.6842E+04, &
      3.736260E-02, 1.74830E+01, 4.8768E+00, 3.6529E+04, &
      3.758700E-02, 1.74764E+01, 4.8670E+00, 3.6238E+04, &
      3.797993E-02, 1.74536E+01, 4.8155E+00, 3.5484E+04, &
      3.814800E-02, 1.74423E+01, 4.7949E+00, 3.5176E+04, &
      4.060054E-02, 1.72685E+01, 4.5712E+00, 3.1510E+04, &
      4.340198E-02, 1.70890E+01, 4.4534E+00, 2.8716E+04, &
      4.639671E-02, 1.69349E+01, 4.4363E+00, 2.6760E+04, &
      4.959809E-02, 1.68151E+01, 4.4994E+00, 2.5388E+04, &
      5.302035E-02, 1.67350E+01, 4.6244E+00, 2.4410E+04, &
      5.667876E-02, 1.66946E+01, 4.7895E+00, 2.3649E+04, &
      6.058959E-02, 1.66887E+01, 4.9941E+00, 2.3068E+04, &
      6.477028E-02, 1.67318E+01, 5.2307E+00, 2.2601E+04, &
      6.923942E-02, 1.67934E+01, 5.4931E+00, 2.2203E+04, &
      7.401695E-02, 1.68862E+01, 5.7767E+00, 2.1842E+04, &
      7.912411E-02, 1.70075E+01, 6.0781E+00, 2.1498E+04, &
      8.458368E-02, 1.71534E+01, 6.3945E+00, 2.1158E+04, &
      9.041995E-02, 1.73175E+01, 6.7245E+00, 2.0813E+04, &
      9.665893E-02, 1.74882E+01, 7.0670E+00, 2.0462E+04, &
      1.0000000E-01, 2.05652E+01, 7.2460E+00, 2.0279E+04, &
      1.0050000E-01, 2.05766E+01, 7.2725E+00, 2.0252E+04, &
      1.0100250E-01, 2.05878E+01, 7.2991E+00, 2.0225E+04, &
      1.0150751E-01, 2.05987E+01, 7.3257E+00, 2.0198E+04, &
      1.0201505E-01, 2.06092E+01, 7.3525E+00, 2.0171E+04, &
      1.0252513E-01, 2.06194E+01, 7.3792E+00, 2.0144E+04, &
      1.0303775E-01, 2.06291E+01, 7.4061E+00, 2.0116E+04, &
      1.0355294E-01, 2.06388E+01, 7.4181E+00, 2.0049E+04, &
      1.0407070E-01, 2.06473E+01, 7.4167E+00, 1.9945E+04, &
      1.0459106E-01, 2.06540E+01, 7.4150E+00, 1.9842E+04, &
      1.0511401E-01, 2.06588E+01, 7.4132E+00, 1.9738E+04, &
      1.0563958E-01, 2.06616E+01, 7.4111E+00, 1.9634E+04, &
      1.0616778E-01, 2.06622E+01, 7.4088E+00, 1.9531E+04, &
      1.0669862E-01, 2.06607E+01, 7.4063E+00, 1.9427E+04, &
      1.0723211E-01, 2.06569E+01, 7.4036E+00, 1.9323E+04, &
      1.0776827E-01, 2.06506E+01, 7.4007E+00, 1.9219E+04, &
      1.0830712E-01, 2.06417E+01, 7.3976E+00, 1.9116E+04, &
      1.0884865E-01, 2.06301E+01, 7.3943E+00, 1.9012E+04, &
      1.0939289E-01, 2.06155E+01, 7.3907E+00, 1.8909E+04, &
      1.0993986E-01, 2.05979E+01, 7.3870E+00, 1.8805E+04, &
      1.1048956E-01, 2.05769E+01, 7.3830E+00, 1.8701E+04, &
      1.1104201E-01, 2.05524E+01, 7.3789E+00, 1.8598E+04, &
      1.1159722E-01, 2.05240E+01, 7.3745E+00, 1.8494E+04, &
      1.1215520E-01, 2.04915E+01, 7.3699E+00, 1.8391E+04, &
      1.1271598E-01, 2.04550E+01, 7.3651E+00, 1.8288E+04, &
      1.1327956E-01, 2.04131E+01, 7.3602E+00, 1.8184E+04, &
      1.1384596E-01, 2.03659E+01, 7.3550E+00, 1.8081E+04, &
      1.1441519E-01, 2.03129E+01, 7.3496E+00, 1.7978E+04, &
      1.1498726E-01, 2.02535E+01, 7.3440E+00, 1.7875E+04, &
      1.1556220E-01, 2.01871E+01, 7.3381E+00, 1.7772E+04, &
      1.1614001E-01, 2.01128E+01, 7.3321E+00, 1.7669E+04, &
      1.1672071E-01, 2.00299E+01, 7.3259E+00, 1.7566E+04, &
      1.1730431E-01, 1.99372E+01, 7.3195E+00, 1.7463E+04, &
      1.1789083E-01, 1.98336E+01, 7.3129E+00, 1.7361E+04, &
      1.1848029E-01, 1.97175E+01, 7.3061E+00, 1.7258E+04, &
      1.1907269E-01, 1.95872E+01, 7.2990E+00, 1.7156E+04, &
      1.1966805E-01, 1.94406E+01, 7.2918E+00, 1.7054E+04, &
      1.2026639E-01, 1.92750E+01, 7.2844E+00, 1.6952E+04, &
      1.2086772E-01, 1.90872E+01, 7.2768E+00, 1.6850E+04, &
      1.2147206E-01, 1.88731E+01, 7.2689E+00, 1.6748E+04, &
      1.2207942E-01, 1.86273E+01, 7.2609E+00, 1.6646E+04, &
      1.2268982E-01, 1.83429E+01, 7.2527E+00, 1.6545E+04, &
      1.2330327E-01, 1.80107E+01, 7.2443E+00, 1.6443E+04, &
      1.2391979E-01, 1.76178E+01, 7.2357E+00, 1.6342E+04, &
      1.2453939E-01, 1.71458E+01, 7.2269E+00, 1.6241E+04, &
      1.2516208E-01, 1.65674E+01, 7.2179E+00, 1.6140E+04, &
      1.2578789E-01, 1.58392E+01, 7.2087E+00, 1.6039E+04, &
      1.2641683E-01, 1.48876E+01, 7.1994E+00, 1.5939E+04, &
      1.2704892E-01, 1.35719E+01, 7.1898E+00, 1.5838E+04, &
      1.2768416E-01, 1.15723E+01, 7.1801E+00, 1.5738E+04, &
      1.2832258E-01, 7.86869E+00, 7.1701E+00, 1.5638E+04, &
      1.2896419E-01, -1.13090E+01, 7.1600E+00, 1.5539E+04, &
      1.2897987E-01, -1.52959E+01, 7.1598E+00, 1.5536E+04, &
      1.2902012E-01, -1.51333E+01, 2.9133E+01, 6.3194E+04, &
      1.2960902E-01, 1.00196E+01, 2.7850E+01, 6.0137E+04, &
      1.3025706E-01, 1.58212E+01, 2.6530E+01, 5.7001E+04, &
      1.3090835E-01, 1.91850E+01, 2.5292E+01, 5.4071E+04, &
      1.3156289E-01, 2.15087E+01, 2.4132E+01, 5.1334E+04, &
      1.3222070E-01, 2.32368E+01, 2.3044E+01, 4.8776E+04, &
      1.3288181E-01, 2.45738E+01, 2.2024E+01, 4.6386E+04, &
      1.3354621E-01, 2.56325E+01, 2.1068E+01, 4.4152E+04, &
      1.3421395E-01, 2.64830E+01, 2.0171E+01, 4.2062E+04, &
      1.3488502E-01, 2.71720E+01, 1.9331E+01, 4.0109E+04, &
      1.3555944E-01, 2.77327E+01, 1.8542E+01, 3.8282E+04, &
      1.3623724E-01, 2.81894E+01, 1.7803E+01, 3.6572E+04, &
      1.3691842E-01, 2.85606E+01, 1.7109E+01, 3.4973E+04, &
      1.3760302E-01, 2.88605E+01, 1.6459E+01, 3.3475E+04, &
      1.3829103E-01, 2.91007E+01, 1.5848E+01, 3.2074E+04, &
      1.3898249E-01, 2.92902E+01, 1.5276E+01, 3.0761E+04, &
      1.3967740E-01, 2.94363E+01, 1.4738E+01, 2.9531E+04, &
      1.4037579E-01, 2.95450E+01, 1.4234E+01, 2.8378E+04, &
      1.4107766E-01, 2.96211E+01, 1.3760E+01, 2.7297E+04, &
      1.4178305E-01, 2.96681E+01, 1.3315E+01, 2.6284E+04, &
      1.4249197E-01, 2.96888E+01, 1.2900E+01, 2.5338E+04, &
      1.4320443E-01, 2.96881E+01, 1.2519E+01, 2.4467E+04, &
      1.4392045E-01, 2.96719E+01, 1.2169E+01, 2.3664E+04, &
      1.4464005E-01, 2.96442E+01, 1.1846E+01, 2.2923E+04, &
      1.4536325E-01, 2.96080E+01, 1.1549E+01, 2.2237E+04, &
      1.4609007E-01, 2.95655E+01, 1.1275E+01, 2.1601E+04, &
      1.4682052E-01, 2.95185E+01, 1.1022E+01, 2.1010E+04, &
      1.4755462E-01, 2.94682E+01, 1.0787E+01, 2.0460E+04, &
      1.4829239E-01, 2.94158E+01, 1.0569E+01, 1.9947E+04, &
      1.4903386E-01, 2.93622E+01, 1.0367E+01, 1.9468E+04, &
      1.4977903E-01, 2.93078E+01, 1.0178E+01, 1.9019E+04, &
      1.5052792E-01, 2.92534E+01, 1.0003E+01, 1.8598E+04, &
      1.5128056E-01, 2.91992E+01, 9.8390E+00, 1.8203E+04, &
      1.5203696E-01, 2.91457E+01, 9.6858E+00, 1.7830E+04, &
      1.5279715E-01, 2.90930E+01, 9.5424E+00, 1.7479E+04, &
      1.5356113E-01, 2.90414E+01, 9.4079E+00, 1.7147E+04, &
      1.5432894E-01, 2.89910E+01, 9.2816E+00, 1.6833E+04, &
      1.5510058E-01, 2.89419E+01, 9.1628E+00, 1.6535E+04, &
      1.5587609E-01, 2.88941E+01, 9.0510E+00, 1.6251E+04, &
      1.5665547E-01, 2.88478E+01, 8.9455E+00, 1.5982E+04, &
      1.5743875E-01, 2.88030E+01, 8.8458E+00, 1.5725E+04, &
      1.5822594E-01, 2.87596E+01, 8.7516E+00, 1.5481E+04, &
      1.5901707E-01, 2.87177E+01, 8.6623E+00, 1.5246E+04, &
      1.5981215E-01, 2.86773E+01, 8.5777E+00, 1.5022E+04, &
      1.6061121E-01, 2.86382E+01, 8.4973E+00, 1.4808E+04, &
      1.6141427E-01, 2.86006E+01, 8.4208E+00, 1.4601E+04, &
      1.6222134E-01, 2.85644E+01, 8.3480E+00, 1.4403E+04, &
      1.6303245E-01, 2.85295E+01, 8.2786E+00, 1.4212E+04, &
      1.6384761E-01, 2.84959E+01, 8.2124E+00, 1.4029E+04, &
      1.6466685E-01, 2.84635E+01, 8.1491E+00, 1.3851E+04, &
      1.6549018E-01, 2.84324E+01, 8.0885E+00, 1.3680E+04, &
      1.6631763E-01, 2.84024E+01, 8.0305E+00, 1.3514E+04, &
      1.6714922E-01, 2.83735E+01, 7.9749E+00, 1.3354E+04, &
      1.6798497E-01, 2.83457E+01, 7.9215E+00, 1.3198E+04, &
      1.6882489E-01, 2.83189E+01, 7.8702E+00, 1.3048E+04, &
      1.6966902E-01, 2.82930E+01, 7.8208E+00, 1.2901E+04, &
      1.7051736E-01, 2.82681E+01, 7.7732E+00, 1.2759E+04, &
      1.7136995E-01, 2.82441E+01, 7.7274E+00, 1.2621E+04, &
      1.7222680E-01, 2.82209E+01, 7.6831E+00, 1.2486E+04, &
      1.7308793E-01, 2.81985E+01, 7.6404E+00, 1.2355E+04, &
      1.7395337E-01, 2.81769E+01, 7.5991E+00, 1.2227E+04, &
      1.7482314E-01, 2.81559E+01, 7.5591E+00, 1.2102E+04, &
      1.7569726E-01, 2.81357E+01, 7.5204E+00, 1.1980E+04, &
      1.7657574E-01, 2.81160E+01, 7.4829E+00, 1.1861E+04, &
      1.7745862E-01, 2.80970E+01, 7.4465E+00, 1.1745E+04, &
      1.7834591E-01, 2.80785E+01, 7.4112E+00, 1.1631E+04, &
      1.7923764E-01, 2.80606E+01, 7.3769E+00, 1.1520E+04, &
      1.8013383E-01, 2.80431E+01, 7.3436E+00, 1.1411E+04, &
      1.8103450E-01, 2.80261E+01, 7.3112E+00, 1.1304E+04, &
      1.8193967E-01, 2.80095E+01, 7.2796E+00, 1.1199E+04, &
      1.8284937E-01, 2.79933E+01, 7.2489E+00, 1.1096E+04, &
      1.8376362E-01, 2.79774E+01, 7.2190E+00, 1.0996E+04, &
      1.8468244E-01, 2.79619E+01, 7.1898E+00, 1.0897E+04, &
      1.8560585E-01, 2.79467E+01, 7.1614E+00, 1.0800E+04, &
      1.8653388E-01, 2.79318E+01, 7.1336E+00, 1.0704E+04, &
      1.8746655E-01, 2.79171E+01, 7.1065E+00, 1.0611E+04, &
      1.8840388E-01, 2.79026E+01, 7.0801E+00, 1.0519E+04, &
      1.8934590E-01, 2.78883E+01, 7.0543E+00, 1.0428E+04, &
      1.9029263E-01, 2.78742E+01, 7.0290E+00, 1.0339E+04, &
      1.9124409E-01, 2.78603E+01, 7.0043E+00, 1.0252E+04, &
      1.9220031E-01, 2.78464E+01, 6.9802E+00, 1.0165E+04, &
      1.9316131E-01, 2.78327E+01, 6.9566E+00, 1.0081E+04, &
      1.9412712E-01, 2.78190E+01, 6.9335E+00, 9.9973E+03, &
      1.9509776E-01, 2.78054E+01, 6.9110E+00, 9.9152E+03, &
      1.9607325E-01, 2.77918E+01, 6.8889E+00, 9.8344E+03, &
      1.9705361E-01, 2.77782E+01, 6.8673E+00, 9.7548E+03, &
      1.9803888E-01, 2.77645E+01, 6.8461E+00, 9.6764E+03, &
      1.9902907E-01, 2.77509E+01, 6.8254E+00, 9.5992E+03, &
      2.0002422E-01, 2.77372E+01, 6.8051E+00, 9.5231E+03, &
      2.0102434E-01, 2.77234E+01, 6.7853E+00, 9.4481E+03, &
      2.0202946E-01, 2.77095E+01, 6.7659E+00, 9.3742E+03, &
      2.0303961E-01, 2.76954E+01, 6.7469E+00, 9.3014E+03, &
      2.0405481E-01, 2.76812E+01, 6.7283E+00, 9.2296E+03, &
      2.0507508E-01, 2.76669E+01, 6.7100E+00, 9.1589E+03, &
      2.0610046E-01, 2.76523E+01, 6.6922E+00, 9.0891E+03, &
      2.0713096E-01, 2.76375E+01, 6.6748E+00, 9.0204E+03, &
      2.0816661E-01, 2.76225E+01, 6.6577E+00, 8.9526E+03, &
      2.0920745E-01, 2.76071E+01, 6.6411E+00, 8.8858E+03, &
      2.1025348E-01, 2.75915E+01, 6.6248E+00, 8.8199E+03, &
      2.1130475E-01, 2.75756E+01, 6.6088E+00, 8.7549E+03, &
      2.1236128E-01, 2.75593E+01, 6.5932E+00, 8.6908E+03, &
      2.1342308E-01, 2.75425E+01, 6.5780E+00, 8.6277E+03, &
      2.1449020E-01, 2.75254E+01, 6.5632E+00, 8.5654E+03, &
      2.1556265E-01, 2.75078E+01, 6.5487E+00, 8.5040E+03, &
      2.1664046E-01, 2.74897E+01, 6.5345E+00, 8.4434E+03, &
      2.1772366E-01, 2.74710E+01, 6.5207E+00, 8.3837E+03, &
      2.1881228E-01, 2.74518E+01, 6.5072E+00, 8.3248E+03, &
      2.1990634E-01, 2.74319E+01, 6.4941E+00, 8.2667E+03, &
      2.2100588E-01, 2.74112E+01, 6.4814E+00, 8.2094E+03, &
      2.2211090E-01, 2.73899E+01, 6.4689E+00, 8.1530E+03, &
      2.2322146E-01, 2.73676E+01, 6.4569E+00, 8.0973E+03, &
      2.2433757E-01, 2.73445E+01, 6.4451E+00, 8.0424E+03, &
      2.2545925E-01, 2.73203E+01, 6.4337E+00, 7.9883E+03, &
      2.2658655E-01, 2.72950E+01, 6.4227E+00, 7.9349E+03, &
      2.2771948E-01, 2.72684E+01, 6.4120E+00, 7.8823E+03, &
      2.2885808E-01, 2.72404E+01, 6.4016E+00, 7.8304E+03, &
      2.3000237E-01, 2.72109E+01, 6.3916E+00, 7.7793E+03, &
      2.3115238E-01, 2.71796E+01, 6.3819E+00, 7.7289E+03, &
      2.3230814E-01, 2.71462E+01, 6.3726E+00, 7.6792E+03, &
      2.3346969E-01, 2.71104E+01, 6.3636E+00, 7.6302E+03, &
      2.3463703E-01, 2.70719E+01, 6.3549E+00, 7.5820E+03, &
      2.3581022E-01, 2.70301E+01, 6.3466E+00, 7.5344E+03, &
      2.3698927E-01, 2.69843E+01, 6.3386E+00, 7.4875E+03, &
      2.3817422E-01, 2.69336E+01, 6.3310E+00, 7.4413E+03, &
      2.3936509E-01, 2.68767E+01, 6.3237E+00, 7.3958E+03, &
      2.4056191E-01, 2.68116E+01, 6.3167E+00, 7.3509E+03, &
      2.4176472E-01, 2.67352E+01, 6.3101E+00, 7.3067E+03, &
      2.4297355E-01, 2.66420E+01, 6.3039E+00, 7.2632E+03, &
      2.4418841E-01, 2.65215E+01, 6.2979E+00, 7.2203E+03, &
      2.4540936E-01, 2.63473E+01, 6.2924E+00, 7.1781E+03, &
      2.4663640E-01, 2.60106E+01, 6.2871E+00, 7.1364E+03, &
      2.4723820E-01, 2.54816E+01, 6.2847E+00, 7.1163E+03, &
      2.4756180E-01, 2.54768E+01, 7.3380E+00, 8.2978E+03, &
      2.4786959E-01, 2.58299E+01, 7.3379E+00, 8.2873E+03, &
      2.4910893E-01, 2.62469E+01, 7.3374E+00, 8.2456E+03, &
      2.5035448E-01, 2.64145E+01, 7.3374E+00, 8.2046E+03, &
      2.5160625E-01, 2.65167E+01, 7.3377E+00, 8.1641E+03, &
      2.5286428E-01, 2.65877E+01, 7.3383E+00, 8.1242E+03, &
      2.5412860E-01, 2.66399E+01, 7.3393E+00, 8.0850E+03, &
      2.5539925E-01, 2.66792E+01, 7.3407E+00, 8.0463E+03, &
      2.5667624E-01, 2.67086E+01, 7.3424E+00, 8.0082E+03, &
      2.5795962E-01, 2.67296E+01, 7.3445E+00, 7.9706E+03, &
      2.5924942E-01, 2.67429E+01, 7.3470E+00, 7.9337E+03, &
      2.6054567E-01, 2.67477E+01, 7.3498E+00, 7.8972E+03, &
      2.6184840E-01, 2.67422E+01, 7.3530E+00, 7.8614E+03, &
      2.6315764E-01, 2.67202E+01, 7.3565E+00, 7.8261E+03, &
      2.6447343E-01, 2.66594E+01, 7.3604E+00, 7.7913E+03, &
      2.6540451E-01, 2.64878E+01, 7.3634E+00, 7.7671E+03, &
      2.6579579E-01, 2.64932E+01, 7.6966E+00, 8.1065E+03, &
      2.6712477E-01, 2.67272E+01, 7.7027E+00, 8.0727E+03, &
      2.6846040E-01, 2.68103E+01, 7.7093E+00, 8.0393E+03, &
      2.6980270E-01, 2.68667E+01, 7.7162E+00, 8.0065E+03, &
      2.7115171E-01, 2.69111E+01, 7.7235E+00, 7.9742E+03, &
      2.7250747E-01, 2.69484E+01, 7.7311E+00, 7.9424E+03, &
      2.7387001E-01, 2.69811E+01, 7.7390E+00, 7.9111E+03, &
      2.7523936E-01, 2.70104E+01, 7.7473E+00, 7.8802E+03, &
      2.7661556E-01, 2.70371E+01, 7.7559E+00, 7.8497E+03, &
      2.7799863E-01, 2.70617E+01, 7.7649E+00, 7.8197E+03, &
      2.7938863E-01, 2.70847E+01, 7.7741E+00, 7.7901E+03, &
      2.8078557E-01, 2.71063E+01, 7.7837E+00, 7.7609E+03, &
      2.8218950E-01, 2.71266E+01, 7.7935E+00, 7.7321E+03, &
      2.8360044E-01, 2.71460E+01, 7.8037E+00, 7.7036E+03, &
      2.8501845E-01, 2.71645E+01, 7.8141E+00, 7.6756E+03, &
      2.8644354E-01, 2.71822E+01, 7.8249E+00, 7.6480E+03, &
      2.8787576E-01, 2.71992E+01, 7.8360E+00, 7.6207E+03, &
      2.8931514E-01, 2.72157E+01, 7.8473E+00, 7.5939E+03, &
      2.9076171E-01, 2.72316E+01, 7.8590E+00, 7.5673E+03, &
      2.9221552E-01, 2.72470E+01, 7.8709E+00, 7.5411E+03, &
      2.9367660E-01, 2.72619E+01, 7.8831E+00, 7.5153E+03, &
      2.9514498E-01, 2.72764E+01, 7.8956E+00, 7.4898E+03, &
      2.9662071E-01, 2.72906E+01, 7.9084E+00, 7.4646E+03, &
      2.9810381E-01, 2.73044E+01, 7.9214E+00, 7.4397E+03, &
      2.9959433E-01, 2.73179E+01, 7.9347E+00, 7.4151E+03, &
      3.0109230E-01, 2.73311E+01, 7.9482E+00, 7.3909E+03, &
      3.0259776E-01, 2.73440E+01, 7.9620E+00, 7.3669E+03, &
      3.0411075E-01, 2.73566E+01, 7.9761E+00, 7.3432E+03, &
      3.0563130E-01, 2.73690E+01, 7.9903E+00, 7.3198E+03, &
      3.0715946E-01, 2.73810E+01, 8.0049E+00, 7.2966E+03, &
      3.0869526E-01, 2.73928E+01, 8.0196E+00, 7.2737E+03, &
      3.1023873E-01, 2.74042E+01, 8.0345E+00, 7.2510E+03, &
      3.1178993E-01, 2.74154E+01, 8.0497E+00, 7.2286E+03, &
      3.1334888E-01, 2.74262E+01, 8.0651E+00, 7.2064E+03, &
      3.1491562E-01, 2.74367E+01, 8.0807E+00, 7.1845E+03, &
      3.1649020E-01, 2.74468E+01, 8.0965E+00, 7.1627E+03, &
      3.1807265E-01, 2.74565E+01, 8.1124E+00, 7.1412E+03, &
      3.1966301E-01, 2.74657E+01, 8.1286E+00, 7.1198E+03, &
      3.2126133E-01, 2.74743E+01, 8.1449E+00, 7.0986E+03, &
      3.2286764E-01, 2.74823E+01, 8.1614E+00, 7.0777E+03, &
      3.2448197E-01, 2.74896E+01, 8.1781E+00, 7.0569E+03, &
      3.2610438E-01, 2.74959E+01, 8.1949E+00, 7.0362E+03, &
      3.2773491E-01, 2.75013E+01, 8.2118E+00, 7.0157E+03, &
      3.2937358E-01, 2.75053E+01, 8.2289E+00, 6.9954E+03, &
      3.3102045E-01, 2.75077E+01, 8.2462E+00, 6.9752E+03, &
      3.3267555E-01, 2.75080E+01, 8.2635E+00, 6.9551E+03, &
      3.3433893E-01, 2.75057E+01, 8.2810E+00, 6.9352E+03, &
      3.3601062E-01, 2.74998E+01, 8.2986E+00, 6.9154E+03, &
      3.3769068E-01, 2.74889E+01, 8.3163E+00, 6.8956E+03, &
      3.3937913E-01, 2.74705E+01, 8.3340E+00, 6.8760E+03, &
      3.4107602E-01, 2.74395E+01, 8.3519E+00, 6.8565E+03, &
      3.4278140E-01, 2.73841E+01, 8.3698E+00, 6.8371E+03, &
      3.4449531E-01, 2.72578E+01, 8.3878E+00, 6.8177E+03, &
      3.4532318E-01, 2.70743E+01, 8.3965E+00, 6.8084E+03, &
      3.4607681E-01, 2.70834E+01, 8.9281E+00, 7.2235E+03, &
      3.4621779E-01, 2.71381E+01, 8.9296E+00, 7.2218E+03, &
      3.4794888E-01, 2.74045E+01, 8.9488E+00, 7.2014E+03, &
      3.4968862E-01, 2.75221E+01, 8.9681E+00, 7.1810E+03, &
      3.5143706E-01, 2.76054E+01, 8.9874E+00, 7.1607E+03, &
      3.5319425E-01, 2.76729E+01, 9.0067E+00, 7.1404E+03, &
      3.5496022E-01, 2.77316E+01, 9.0261E+00, 7.1202E+03, &
      3.5673502E-01, 2.77845E+01, 9.0454E+00, 7.1000E+03, &
      3.5851870E-01, 2.78336E+01, 9.0648E+00, 7.0798E+03, &
      3.6031129E-01, 2.78798E+01, 9.0841E+00, 7.0596E+03, &
      3.6211285E-01, 2.79239E+01, 9.1035E+00, 7.0395E+03, &
      3.6392341E-01, 2.79664E+01, 9.1228E+00, 7.0194E+03, &
      3.6574303E-01, 2.80077E+01, 9.1420E+00, 6.9992E+03, &
      3.6757174E-01, 2.80479E+01, 9.1612E+00, 6.9791E+03, &
      3.6940960E-01, 2.80874E+01, 9.1804E+00, 6.9589E+03, &
      3.7125665E-01, 2.81263E+01, 9.1995E+00, 6.9387E+03, &
      3.7311293E-01, 2.81646E+01, 9.2185E+00, 6.9185E+03, &
      3.7497850E-01, 2.82026E+01, 9.2375E+00, 6.8983E+03, &
      3.7685339E-01, 2.82403E+01, 9.2563E+00, 6.8780E+03, &
      3.7873766E-01, 2.82777E+01, 9.2751E+00, 6.8577E+03, &
      3.8063135E-01, 2.83149E+01, 9.2938E+00, 6.8373E+03, &
      3.8253450E-01, 2.83520E+01, 9.3123E+00, 6.8169E+03, &
      3.8444718E-01, 2.83890E+01, 9.3307E+00, 6.7964E+03, &
      3.8636941E-01, 2.84260E+01, 9.3489E+00, 6.7759E+03, &
      3.8830126E-01, 2.84366E+01, 9.3670E+00, 6.7552E+03, &
      3.9024276E-01, 2.84735E+01, 9.3849E+00, 6.7345E+03, &
      3.9219398E-01, 2.85103E+01, 9.4026E+00, 6.7137E+03, &
      3.9415495E-01, 2.85472E+01, 9.4201E+00, 6.6928E+03, &
      3.9612572E-01, 2.85841E+01, 9.4375E+00, 6.6718E+03, &
      3.9810635E-01, 2.86211E+01, 9.4546E+00, 6.6507E+03, &
      4.0009688E-01, 2.86581E+01, 9.4715E+00, 6.6294E+03, &
      4.0209737E-01, 2.86952E+01, 9.4881E+00, 6.6081E+03, &
      4.0410785E-01, 2.87324E+01, 9.5045E+00, 6.5866E+03, &
      4.0612839E-01, 2.87696E+01, 9.5207E+00, 6.5650E+03, &
      4.0815904E-01, 2.88069E+01, 9.5366E+00, 6.5433E+03, &
      4.1019983E-01, 2.88443E+01, 9.5522E+00, 6.5215E+03, &
      4.1225083E-01, 2.88817E+01, 9.5676E+00, 6.4995E+03, &
      4.1431208E-01, 2.89193E+01, 9.5828E+00, 6.4774E+03, &
      4.1638364E-01, 2.89569E+01, 9.5976E+00, 6.4552E+03, &
      4.1846556E-01, 2.89946E+01, 9.6122E+00, 6.4329E+03, &
      4.2055789E-01, 2.90324E+01, 9.6265E+00, 6.4105E+03, &
      4.2266068E-01, 2.90702E+01, 9.6405E+00, 6.3879E+03, &
      4.2477398E-01, 2.91082E+01, 9.6542E+00, 6.3652E+03, &
      4.2689785E-01, 2.91462E+01, 9.6676E+00, 6.3424E+03, &
      4.2903234E-01, 2.91842E+01, 9.6807E+00, 6.3194E+03, &
      4.3117750E-01, 2.92224E+01, 9.6935E+00, 6.2963E+03, &
      4.3333339E-01, 2.92606E+01, 9.7060E+00, 6.2731E+03, &
      4.3550006E-01, 2.92988E+01, 9.7181E+00, 6.2497E+03, &
      4.3767756E-01, 2.93372E+01, 9.7299E+00, 6.2262E+03, &
      4.3986595E-01, 2.93755E+01, 9.7413E+00, 6.2025E+03, &
      4.4206528E-01, 2.94139E+01, 9.7524E+00, 6.1787E+03, &
      4.4427560E-01, 2.94524E+01, 9.7632E+00, 6.1548E+03, &
      4.4649698E-01, 2.94908E+01, 9.7736E+00, 6.1308E+03, &
      4.4872947E-01, 2.95293E+01, 9.7836E+00, 6.1066E+03, &
      4.5097311E-01, 2.95678E+01, 9.7933E+00, 6.0823E+03, &
      4.5322798E-01, 2.96063E+01, 9.8026E+00, 6.0578E+03, &
      4.5549412E-01, 2.96449E+01, 9.8116E+00, 6.0332E+03, &
      4.5777159E-01, 2.96834E+01, 9.8201E+00, 6.0085E+03, &
      4.6006045E-01, 2.97219E+01, 9.8283E+00, 5.9836E+03, &
      4.6236075E-01, 2.97604E+01, 9.8361E+00, 5.9586E+03, &
      4.6467255E-01, 2.97988E+01, 9.8435E+00, 5.9335E+03, &
      4.6699592E-01, 2.98372E+01, 9.8505E+00, 5.9082E+03, &
      4.6933090E-01, 2.98756E+01, 9.8572E+00, 5.8828E+03, &
      4.7167755E-01, 2.99139E+01, 9.8634E+00, 5.8573E+03, &
      4.7403594E-01, 2.99522E+01, 9.8692E+00, 5.8316E+03, &
      4.7640612E-01, 2.99904E+01, 9.8747E+00, 5.8058E+03, &
      4.7878815E-01, 3.00285E+01, 9.8797E+00, 5.7799E+03, &
      4.8118209E-01, 3.00665E+01, 9.8843E+00, 5.7539E+03, &
      4.8358800E-01, 3.01044E+01, 9.8885E+00, 5.7278E+03, &
      4.8600594E-01, 3.01423E+01, 9.8923E+00, 5.7015E+03, &
      4.8843597E-01, 3.01800E+01, 9.8957E+00, 5.6752E+03, &
      4.9087815E-01, 3.02176E+01, 9.8987E+00, 5.6487E+03, &
      4.9333254E-01, 3.02550E+01, 9.9012E+00, 5.6221E+03, &
      4.9579920E-01, 3.02923E+01, 9.9034E+00, 5.5953E+03, &
      4.9827820E-01, 3.03295E+01, 9.9051E+00, 5.5685E+03, &
      5.0076959E-01, 3.03665E+01, 9.9064E+00, 5.5416E+03, &
      5.0327344E-01, 3.04033E+01, 9.9073E+00, 5.5146E+03, &
      5.0578980E-01, 3.04400E+01, 9.9078E+00, 5.4875E+03, &
      5.0831875E-01, 3.04765E+01, 9.9078E+00, 5.4602E+03, &
      5.1086035E-01, 3.05128E+01, 9.9075E+00, 5.4329E+03, &
      5.1341465E-01, 3.05488E+01, 9.9067E+00, 5.4055E+03, &
      5.1598172E-01, 3.05847E+01, 9.9055E+00, 5.3780E+03, &
      5.1856163E-01, 3.06204E+01, 9.9038E+00, 5.3504E+03, &
      5.2115444E-01, 3.06558E+01, 9.9016E+00, 5.3227E+03, &
      5.2376021E-01, 3.06909E+01, 9.8989E+00, 5.2948E+03, &
      5.2637901E-01, 3.07258E+01, 9.8957E+00, 5.2668E+03, &
      5.2901091E-01, 3.07604E+01, 9.8920E+00, 5.2387E+03, &
      5.3165596E-01, 3.07946E+01, 9.8878E+00, 5.2105E+03, &
      5.3431424E-01, 3.08286E+01, 9.8831E+00, 5.1821E+03, &
      5.3698581E-01, 3.08622E+01, 9.8780E+00, 5.1537E+03, &
      5.3967074E-01, 3.08954E+01, 9.8723E+00, 5.1252E+03, &
      5.4236910E-01, 3.09283E+01, 9.8662E+00, 5.0966E+03, &
      5.4508094E-01, 3.09607E+01, 9.8595E+00, 5.0678E+03, &
      5.4780635E-01, 3.09928E+01, 9.8524E+00, 5.0391E+03, &
      5.5054538E-01, 3.10244E+01, 9.8449E+00, 5.0102E+03, &
      5.5329810E-01, 3.10556E+01, 9.8368E+00, 4.9813E+03, &
      5.5606460E-01, 3.10863E+01, 9.8284E+00, 4.9523E+03, &
      5.5884492E-01, 3.11166E+01, 9.8194E+00, 4.9232E+03, &
      5.6163914E-01, 3.11464E+01, 9.8100E+00, 4.8941E+03, &
      5.6444734E-01, 3.11757E+01, 9.8002E+00, 4.8649E+03, &
      5.6726958E-01, 3.12044E+01, 9.7900E+00, 4.8357E+03, &
      5.7010592E-01, 3.12327E+01, 9.7793E+00, 4.8064E+03, &
      5.7295645E-01, 3.12604E+01, 9.7681E+00, 4.7771E+03, &
      5.7582123E-01, 3.12875E+01, 9.7566E+00, 4.7478E+03, &
      5.7870034E-01, 3.13141E+01, 9.7447E+00, 4.7185E+03, &
      5.8159384E-01, 3.13401E+01, 9.7323E+00, 4.6891E+03, &
      5.8450181E-01, 3.13655E+01, 9.7195E+00, 4.6597E+03, &
      5.8742432E-01, 3.13903E+01, 9.7064E+00, 4.6303E+03, &
      5.9036144E-01, 3.14144E+01, 9.6928E+00, 4.6009E+03, &
      5.9331325E-01, 3.14379E+01, 9.6789E+00, 4.5715E+03, &
      5.9627982E-01, 3.14608E+01, 9.6646E+00, 4.5421E+03, &
      5.9926122E-01, 3.14830E+01, 9.6499E+00, 4.5127E+03, &
      6.0225752E-01, 3.15045E+01, 9.6349E+00, 4.4833E+03, &
      6.0526881E-01, 3.15253E+01, 9.6195E+00, 4.4539E+03, &
      6.0829515E-01, 3.15454E+01, 9.6037E+00, 4.4246E+03, &
      6.1133663E-01, 3.15647E+01, 9.5876E+00, 4.3952E+03, &
      6.1439331E-01, 3.15834E+01, 9.5712E+00, 4.3659E+03, &
      6.1746528E-01, 3.16013E+01, 9.5544E+00, 4.3367E+03, &
      6.2055260E-01, 3.16184E+01, 9.5373E+00, 4.3074E+03, &
      6.2365537E-01, 3.16347E+01, 9.5199E+00, 4.2782E+03, &
      6.2677364E-01, 3.16502E+01, 9.5021E+00, 4.2491E+03, &
      6.2990751E-01, 3.16650E+01, 9.4841E+00, 4.2200E+03, &
      6.3305705E-01, 3.16789E+01, 9.4657E+00, 4.1909E+03, &
      6.3622234E-01, 3.16919E+01, 9.4471E+00, 4.1619E+03, &
      6.3940345E-01, 3.17041E+01, 9.4281E+00, 4.1330E+03, &
      6.4260046E-01, 3.17155E+01, 9.4089E+00, 4.1041E+03, &
      6.4581347E-01, 3.17259E+01, 9.3894E+00, 4.0753E+03, &
      6.4904253E-01, 3.17354E+01, 9.3696E+00, 4.0465E+03, &
      6.5228775E-01, 3.17440E+01, 9.3496E+00, 4.0179E+03, &
      6.5554919E-01, 3.17517E+01, 9.3293E+00, 3.9893E+03, &
      6.5882693E-01, 3.17585E+01, 9.3088E+00, 3.9607E+03, &
      6.6212107E-01, 3.17642E+01, 9.2880E+00, 3.9323E+03, &
      6.6543167E-01, 3.17690E+01, 9.2669E+00, 3.9039E+03, &
      6.6875883E-01, 3.17727E+01, 9.2457E+00, 3.8757E+03, &
      6.7210262E-01, 3.17755E+01, 9.2242E+00, 3.8475E+03, &
      6.7546314E-01, 3.17772E+01, 9.2025E+00, 3.8194E+03, &
      6.7884045E-01, 3.17778E+01, 9.1805E+00, 3.7914E+03, &
      6.8223466E-01, 3.17774E+01, 9.1584E+00, 3.7635E+03, &
      6.8564583E-01, 3.17758E+01, 9.1361E+00, 3.7357E+03, &
      6.8907406E-01, 3.17731E+01, 9.1135E+00, 3.7081E+03, &
      6.9251943E-01, 3.17693E+01, 9.0908E+00, 3.6805E+03, &
      6.9598202E-01, 3.17643E+01, 9.0679E+00, 3.6530E+03, &
      6.9946194E-01, 3.17582E+01, 9.0448E+00, 3.6256E+03, &
      7.0295924E-01, 3.17508E+01, 9.0215E+00, 3.5984E+03, &
      7.0647404E-01, 3.17422E+01, 8.9980E+00, 3.5712E+03, &
      7.1000641E-01, 3.17323E+01, 8.9743E+00, 3.5442E+03, &
      7.1355644E-01, 3.17211E+01, 8.9505E+00, 3.5173E+03, &
      7.1712423E-01, 3.17086E+01, 8.9265E+00, 3.4905E+03, &
      7.2070985E-01, 3.16948E+01, 8.9024E+00, 3.4638E+03, &
      7.2431340E-01, 3.16796E+01, 8.8781E+00, 3.4372E+03, &
      7.2793496E-01, 3.16630E+01, 8.8537E+00, 3.4108E+03, &
      7.3157464E-01, 3.16450E+01, 8.8292E+00, 3.3845E+03, &
      7.3523251E-01, 3.16255E+01, 8.8045E+00, 3.3583E+03, &
      7.3890867E-01, 3.16046E+01, 8.7797E+00, 3.3323E+03, &
      7.4260322E-01, 3.15867E+01, 8.7548E+00, 3.3064E+03, &
      7.4631623E-01, 3.15627E+01, 8.7298E+00, 3.2806E+03, &
      7.5004781E-01, 3.15371E+01, 8.7046E+00, 3.2549E+03, &
      7.5379805E-01, 3.15098E+01, 8.6794E+00, 3.2294E+03, &
      7.5756704E-01, 3.14809E+01, 8.6540E+00, 3.2041E+03, &
      7.6135488E-01, 3.14502E+01, 8.6286E+00, 3.1788E+03, &
      7.6516165E-01, 3.14178E+01, 8.6031E+00, 3.1537E+03, &
      7.6898746E-01, 3.13836E+01, 8.5775E+00, 3.1288E+03, &
      7.7283240E-01, 3.13475E+01, 8.5518E+00, 3.1039E+03, &
      7.7669656E-01, 3.13095E+01, 8.5260E+00, 3.0793E+03, &
      7.8058004E-01, 3.12695E+01, 8.5001E+00, 3.0547E+03, &
      7.8448294E-01, 3.12276E+01, 8.4742E+00, 3.0303E+03, &
      7.8840536E-01, 3.11836E+01, 8.4482E+00, 3.0061E+03, &
      7.9234738E-01, 3.11374E+01, 8.4222E+00, 2.9820E+03, &
      7.9630912E-01, 3.10891E+01, 8.3961E+00, 2.9580E+03, &
      8.0029067E-01, 3.10402E+01, 8.3699E+00, 2.9342E+03, &
      8.0429212E-01, 3.09873E+01, 8.3437E+00, 2.9106E+03, &
      8.0831358E-01, 3.09320E+01, 8.3174E+00, 2.8870E+03, &
      8.1235515E-01, 3.08742E+01, 8.2911E+00, 2.8637E+03, &
      8.1641693E-01, 3.08138E+01, 8.2648E+00, 2.8404E+03, &
      8.2049901E-01, 3.07508E+01, 8.2384E+00, 2.8174E+03, &
      8.2460150E-01, 3.06851E+01, 8.2120E+00, 2.7944E+03, &
      8.2872451E-01, 3.06166E+01, 8.1855E+00, 2.7717E+03, &
      8.3286813E-01, 3.05451E+01, 8.1591E+00, 2.7490E+03, &
      8.3703248E-01, 3.04705E+01, 8.1326E+00, 2.7266E+03, &
      8.4121764E-01, 3.03928E+01, 8.1060E+00, 2.7042E+03, &
      8.4542373E-01, 3.03119E+01, 8.0795E+00, 2.6820E+03, &
      8.4965084E-01, 3.02275E+01, 8.0530E+00, 2.6600E+03, &
      8.5389910E-01, 3.01396E+01, 8.0264E+00, 2.6381E+03, &
      8.5816859E-01, 3.00480E+01, 7.9998E+00, 2.6164E+03, &
      8.6245944E-01, 2.99525E+01, 7.9733E+00, 2.5948E+03, &
      8.6677173E-01, 2.98531E+01, 7.9467E+00, 2.5734E+03, &
      8.7110559E-01, 2.97494E+01, 7.9201E+00, 2.5521E+03, &
      8.7546112E-01, 2.96414E+01, 7.8936E+00, 2.5310E+03, &
      8.7983843E-01, 2.95288E+01, 7.8670E+00, 2.5100E+03, &
      8.8423762E-01, 2.94113E+01, 7.8404E+00, 2.4891E+03, &
      8.8865881E-01, 2.92888E+01, 7.8139E+00, 2.4684E+03, &
      8.9310210E-01, 2.91611E+01, 7.7873E+00, 2.4479E+03, &
      8.9756761E-01, 2.90277E+01, 7.7608E+00, 2.4275E+03, &
      9.0205545E-01, 2.88884E+01, 7.7343E+00, 2.4073E+03, &
      9.0656573E-01, 2.87430E+01, 7.7078E+00, 2.3872E+03, &
      9.1109856E-01, 2.85909E+01, 7.6814E+00, 2.3672E+03, &
      9.1565405E-01, 2.84320E+01, 7.6549E+00, 2.3474E+03, &
      9.2023232E-01, 2.82656E+01, 7.6285E+00, 2.3277E+03, &
      9.2483348E-01, 2.80915E+01, 7.6021E+00, 2.3082E+03, &
      9.2945765E-01, 2.79090E+01, 7.5757E+00, 2.2889E+03, &
      9.3410494E-01, 2.77176E+01, 7.5494E+00, 2.2696E+03, &
      9.3877546E-01, 2.75167E+01, 7.5231E+00, 2.2506E+03, &
      9.4346934E-01, 2.73056E+01, 7.4969E+00, 2.2316E+03, &
      9.4818668E-01, 2.70836E+01, 7.4706E+00, 2.2129E+03, &
      9.5292762E-01, 2.68498E+01, 7.4445E+00, 2.1942E+03, &
      9.5769226E-01, 2.66032E+01, 7.4183E+00, 2.1757E+03, &
      9.6248072E-01, 2.63429E+01, 7.3922E+00, 2.1573E+03, &
      9.6729312E-01, 2.60676E+01, 7.3661E+00, 2.1391E+03, &
      9.7212959E-01, 2.57759E+01, 7.3401E+00, 2.1210E+03, &
      9.7699023E-01, 2.54663E+01, 7.3142E+00, 2.1031E+03, &
      9.8187519E-01, 2.51370E+01, 7.2882E+00, 2.0853E+03, &
      9.8678456E-01, 2.47860E+01, 7.2624E+00, 2.0677E+03, &
      9.9171848E-01, 2.44108E+01, 7.2365E+00, 2.0501E+03, &
      9.9667708E-01, 2.40086E+01, 7.2108E+00, 2.0328E+03, &
      1.0016605E+00, 2.38623E+01, 7.1801E+00, 2.0141E+03, &
      1.0066688E+00, 2.34100E+01, 7.1397E+00, 1.9929E+03, &
      1.0117021E+00, 2.28983E+01, 7.0995E+00, 1.9719E+03, &
      1.0167606E+00, 2.23385E+01, 7.0596E+00, 1.9512E+03, &
      1.0218444E+00, 2.17245E+01, 7.0199E+00, 1.9307E+03, &
      1.0269536E+00, 2.10471E+01, 6.9806E+00, 1.9104E+03, &
      1.0320884E+00, 2.02942E+01, 6.9414E+00, 1.8903E+03, &
      1.0372489E+00, 1.94493E+01, 6.9025E+00, 1.8705E+03, &
      1.0424351E+00, 1.84895E+01, 6.8639E+00, 1.8509E+03, &
      1.0476473E+00, 1.73824E+01, 6.8256E+00, 1.8315E+03, &
      1.0528855E+00, 1.60787E+01, 6.7874E+00, 1.8123E+03, &
      1.0581499E+00, 1.44986E+01, 6.7496E+00, 1.7933E+03, &
      1.0634407E+00, 1.24979E+01, 6.7118E+00, 1.7745E+03, &
      1.0687579E+00, 9.76901E+00, 6.6735E+00, 1.7557E+03, &
      1.0741017E+00, 5.41125E+00, 6.6354E+00, 1.7371E+03, &
      1.0794722E+00, -8.48450E+00, 6.5977E+00, 1.7187E+03, &
      1.0800822E+00, -2.00658E+01, 6.5934E+00, 1.7167E+03, &
      1.0803177E+00, -2.04132E+01, 2.6480E+01, 6.8679E+03, &
      1.0848695E+00, 2.37973E+00, 2.6307E+01, 6.7947E+03, &
      1.0902939E+00, 6.31812E+00, 2.6104E+01, 6.7088E+03, &
      1.0957454E+00, 7.54898E+00, 2.5903E+01, 6.6241E+03, &
      1.1012241E+00, 6.54865E+00, 2.5703E+01, 6.5404E+03, &
      1.1058130E+00, -5.44034E+00, 2.5538E+01, 6.4715E+03, &
      1.1061869E+00, -5.54165E+00, 3.8594E+01, 9.7725E+03, &
      1.1067302E+00, 2.93779E-01, 3.8564E+01, 9.7601E+03, &
      1.1122639E+00, 1.07026E+01, 3.8261E+01, 9.6354E+03, &
      1.1178252E+00, 1.45771E+01, 3.7961E+01, 9.5124E+03, &
      1.1234143E+00, 1.72549E+01, 3.7663E+01, 9.3909E+03, &
      1.1290314E+00, 1.93618E+01, 3.7368E+01, 9.2710E+03, &
      1.1346765E+00, 2.11195E+01, 3.7075E+01, 9.1526E+03, &
      1.1403499E+00, 2.26362E+01, 3.6784E+01, 9.0358E+03, &
      1.1460517E+00, 2.39740E+01, 3.6495E+01, 8.9205E+03, &
      1.1517819E+00, 2.51724E+01, 3.6209E+01, 8.8067E+03, &
      1.1575408E+00, 2.62585E+01, 3.5926E+01, 8.6943E+03, &
      1.1633285E+00, 2.72515E+01, 3.5644E+01, 8.5834E+03, &
      1.1691452E+00, 2.81659E+01, 3.5365E+01, 8.4739E+03, &
      1.1749909E+00, 2.90127E+01, 3.5088E+01, 8.3659E+03, &
      1.1808659E+00, 2.98008E+01, 3.4814E+01, 8.2592E+03, &
      1.1867702E+00, 3.05370E+01, 3.4541E+01, 8.1539E+03, &
      1.1927040E+00, 3.12271E+01, 3.4271E+01, 8.0500E+03, &
      1.1986676E+00, 3.18758E+01, 3.4003E+01, 7.9474E+03, &
      1.2046609E+00, 3.24870E+01, 3.3737E+01, 7.8461E+03, &
      1.2106842E+00, 3.30640E+01, 3.3473E+01, 7.7462E+03, &
      1.2167376E+00, 3.36098E+01, 3.3211E+01, 7.6475E+03, &
      1.2228213E+00, 3.41267E+01, 3.2952E+01, 7.5501E+03, &
      1.2289354E+00, 3.46168E+01, 3.2695E+01, 7.4540E+03, &
      1.2350801E+00, 3.50820E+01, 3.2439E+01, 7.3591E+03, &
      1.2412555E+00, 3.55238E+01, 3.2186E+01, 7.2654E+03, &
      1.2474618E+00, 3.59436E+01, 3.1935E+01, 7.1729E+03, &
      1.2536991E+00, 3.63426E+01, 3.1685E+01, 7.0817E+03, &
      1.2599676E+00, 3.67218E+01, 3.1438E+01, 6.9916E+03, &
      1.2662674E+00, 3.70821E+01, 3.1193E+01, 6.9026E+03, &
      1.2725988E+00, 3.74242E+01, 3.0950E+01, 6.8149E+03, &
      1.2789618E+00, 3.77489E+01, 3.0708E+01, 6.7282E+03, &
      1.2853566E+00, 3.80566E+01, 3.0469E+01, 6.6427E+03, &
      1.2917833E+00, 3.83478E+01, 3.0232E+01, 6.5582E+03, &
      1.2982423E+00, 3.86227E+01, 2.9996E+01, 6.4749E+03, &
      1.3047335E+00, 3.88814E+01, 2.9762E+01, 6.3926E+03, &
      1.3112571E+00, 3.91242E+01, 2.9531E+01, 6.3114E+03, &
      1.3178134E+00, 3.93508E+01, 2.9301E+01, 6.2312E+03, &
      1.3244025E+00, 3.95609E+01, 2.9073E+01, 6.1521E+03, &
      1.3310245E+00, 3.97542E+01, 2.8847E+01, 6.0740E+03, &
      1.3376796E+00, 3.99298E+01, 2.8622E+01, 5.9969E+03, &
      1.3443680E+00, 4.00866E+01, 2.8400E+01, 5.9207E+03, &
      1.3510899E+00, 4.02233E+01, 2.8179E+01, 5.8456E+03, &
      1.3578453E+00, 4.03376E+01, 2.7960E+01, 5.7714E+03, &
      1.3646345E+00, 4.04264E+01, 2.7743E+01, 5.6982E+03, &
      1.3714577E+00, 4.04855E+01, 2.7527E+01, 5.6259E+03, &
      1.3783150E+00, 4.05082E+01, 2.7313E+01, 5.5546E+03, &
      1.3852066E+00, 4.04839E+01, 2.7101E+01, 5.4842E+03, &
      1.3921326E+00, 4.03947E+01, 2.6891E+01, 5.4146E+03, &
      1.3990933E+00, 4.02067E+01, 2.6682E+01, 5.3459E+03, &
      1.4060887E+00, 3.98419E+01, 2.6475E+01, 5.2782E+03, &
      1.4131192E+00, 3.90431E+01, 2.6271E+01, 5.2116E+03, &
      1.4187409E+00, 3.66193E+01, 2.6110E+01, 5.1592E+03, &
      1.4201848E+00, 3.51582E+01, 3.0507E+01, 6.0206E+03, &
      1.4208592E+00, 3.66130E+01, 3.0484E+01, 6.0131E+03, &
      1.4272857E+00, 3.95710E+01, 3.0261E+01, 5.9423E+03, &
      1.4344221E+00, 4.07074E+01, 3.0016E+01, 5.8650E+03, &
      1.4415942E+00, 4.14447E+01, 2.9774E+01, 5.7889E+03, &
      1.4488022E+00, 4.20074E+01, 2.9534E+01, 5.7138E+03, &
      1.4560462E+00, 4.24677E+01, 2.9296E+01, 5.6397E+03, &
      1.4633265E+00, 4.28577E+01, 2.9061E+01, 5.5666E+03, &
      1.4706431E+00, 4.31941E+01, 2.8827E+01, 5.4945E+03, &
      1.4779963E+00, 4.34859E+01, 2.8595E+01, 5.4233E+03, &
      1.4853863E+00, 4.37373E+01, 2.8365E+01, 5.3531E+03, &
      1.4928132E+00, 4.39488E+01, 2.8145E+01, 5.2852E+03, &
      1.5002773E+00, 4.41290E+01, 2.7938E+01, 5.2204E+03, &
      1.5077787E+00, 4.42753E+01, 2.7734E+01, 5.1565E+03, &
      1.5153176E+00, 4.43773E+01, 2.7532E+01, 5.0937E+03, &
      1.5228942E+00, 4.44127E+01, 2.7333E+01, 5.0318E+03, &
      1.5305086E+00, 4.43185E+01, 2.7137E+01, 4.9709E+03, &
      1.5381612E+00, 4.37226E+01, 2.6943E+01, 4.9109E+03, &
      1.5395028E+00, 4.33145E+01, 2.6909E+01, 4.9005E+03, &
      1.5418971E+00, 4.33563E+01, 2.8725E+01, 5.2225E+03, &
      1.5458520E+00, 4.43465E+01, 2.8615E+01, 5.1893E+03, &
      1.5535812E+00, 4.51079E+01, 2.8403E+01, 5.1253E+03, &
      1.5613491E+00, 4.55900E+01, 2.8193E+01, 5.0622E+03, &
      1.5691559E+00, 4.59703E+01, 2.7986E+01, 5.0001E+03, &
      1.5770017E+00, 4.62944E+01, 2.7781E+01, 4.9390E+03, &
      1.5848867E+00, 4.65810E+01, 2.7579E+01, 4.8787E+03, &
      1.5928111E+00, 4.68395E+01, 2.7379E+01, 4.8193E+03, &
      1.6007752E+00, 4.70753E+01, 2.7181E+01, 4.7608E+03, &
      1.6087790E+00, 4.72916E+01, 2.6985E+01, 4.7031E+03, &
      1.6168229E+00, 4.74894E+01, 2.6792E+01, 4.6464E+03, &
      1.6249070E+00, 4.76728E+01, 2.6611E+01, 4.5922E+03, &
      1.6330316E+00, 4.78461E+01, 2.6433E+01, 4.5388E+03, &
      1.6411967E+00, 4.80090E+01, 2.6257E+01, 4.4863E+03, &
      1.6494027E+00, 4.81611E+01, 2.6084E+01, 4.4346E+03, &
      1.6576497E+00, 4.83022E+01, 2.5913E+01, 4.3837E+03, &
      1.6659380E+00, 4.84312E+01, 2.5744E+01, 4.3335E+03, &
      1.6742677E+00, 4.85467E+01, 2.5577E+01, 4.2841E+03, &
      1.6826390E+00, 4.86459E+01, 2.5412E+01, 4.2355E+03, &
      1.6910522E+00, 4.87236E+01, 2.5249E+01, 4.1875E+03, &
      1.6995075E+00, 4.87694E+01, 2.5089E+01, 4.1402E+03, &
      1.7080050E+00, 4.87575E+01, 2.4930E+01, 4.0937E+03, &
      1.7165450E+00, 4.85899E+01, 2.4773E+01, 4.0477E+03, &
      1.7203536E+00, 4.83059E+01, 2.4704E+01, 4.0275E+03, &
      1.7251278E+00, 4.83630E+01, 2.5796E+01, 4.1937E+03, &
      1.7252464E+00, 4.83838E+01, 2.5794E+01, 4.1931E+03, &
      1.7337534E+00, 4.90945E+01, 2.5638E+01, 4.1474E+03, &
      1.7424222E+00, 4.94588E+01, 2.5482E+01, 4.1017E+03, &
      1.7511343E+00, 4.97386E+01, 2.5327E+01, 4.0566E+03, &
      1.7598899E+00, 4.99783E+01, 2.5174E+01, 4.0121E+03, &
      1.7686894E+00, 5.01939E+01, 2.5022E+01, 3.9682E+03, &
      1.7775328E+00, 5.03931E+01, 2.4872E+01, 3.9248E+03, &
      1.7864205E+00, 5.05800E+01, 2.4724E+01, 3.8820E+03, &
      1.7953526E+00, 5.07574E+01, 2.4576E+01, 3.8398E+03, &
      1.8043294E+00, 5.09269E+01, 2.4430E+01, 3.7981E+03, &
      1.8133510E+00, 5.10899E+01, 2.4286E+01, 3.7570E+03, &
      1.8224178E+00, 5.12480E+01, 2.4144E+01, 3.7166E+03, &
      1.8315299E+00, 5.14021E+01, 2.4004E+01, 3.6766E+03, &
      1.8406875E+00, 5.15528E+01, 2.3864E+01, 3.6372E+03, &
      1.8498909E+00, 5.17004E+01, 2.3723E+01, 3.5977E+03, &
      1.8591404E+00, 5.18434E+01, 2.3578E+01, 3.5581E+03, &
      1.8684361E+00, 5.19820E+01, 2.3435E+01, 3.5189E+03, &
      1.8777783E+00, 5.21169E+01, 2.3292E+01, 3.4802E+03, &
      1.8871672E+00, 5.22482E+01, 2.3151E+01, 3.4420E+03, &
      1.8966030E+00, 5.23764E+01, 2.3010E+01, 3.4042E+03, &
      1.9060860E+00, 5.25017E+01, 2.2871E+01, 3.3669E+03, &
      1.9156165E+00, 5.26242E+01, 2.2733E+01, 3.3300E+03, &
      1.9251945E+00, 5.27442E+01, 2.2596E+01, 3.2935E+03, &
      1.9348205E+00, 5.28619E+01, 2.2459E+01, 3.2574E+03, &
      1.9444946E+00, 5.29773E+01, 2.2324E+01, 3.2218E+03, &
      1.9542171E+00, 5.30907E+01, 2.2189E+01, 3.1865E+03, &
      1.9639882E+00, 5.32022E+01, 2.2056E+01, 3.1516E+03, &
      1.9738081E+00, 5.33119E+01, 2.1923E+01, 3.1172E+03, &
      1.9836772E+00, 5.34200E+01, 2.1791E+01, 3.0831E+03, &
      1.9935955E+00, 5.35268E+01, 2.1660E+01, 3.0494E+03, &
      2.0035635E+00, 5.36323E+01, 2.1529E+01, 3.0160E+03, &
      2.0135813E+00, 5.37358E+01, 2.1396E+01, 2.9825E+03, &
      2.0236492E+00, 5.38370E+01, 2.1263E+01, 2.9493E+03, &
      2.0337675E+00, 5.39360E+01, 2.1131E+01, 2.9165E+03, &
      2.0439363E+00, 5.40331E+01, 2.0999E+01, 2.8841E+03, &
      2.0541560E+00, 5.41284E+01, 2.0869E+01, 2.8520E+03, &
      2.0644268E+00, 5.42218E+01, 2.0739E+01, 2.8202E+03, &
      2.0747489E+00, 5.43136E+01, 2.0610E+01, 2.7888E+03, &
      2.0851227E+00, 5.44038E+01, 2.0482E+01, 2.7577E+03, &
      2.0955483E+00, 5.44924E+01, 2.0354E+01, 2.7270E+03, &
      2.1060260E+00, 5.45796E+01, 2.0227E+01, 2.6966E+03, &
      2.1165562E+00, 5.46653E+01, 2.0101E+01, 2.6666E+03, &
      2.1271389E+00, 5.47497E+01, 1.9976E+01, 2.6369E+03, &
      2.1377746E+00, 5.48329E+01, 1.9851E+01, 2.6074E+03, &
      2.1484635E+00, 5.49148E+01, 1.9727E+01, 2.5784E+03, &
      2.1592058E+00, 5.49955E+01, 1.9604E+01, 2.5496E+03, &
      2.1700018E+00, 5.52956E+01, 1.9476E+01, 2.5205E+03, &
      2.1808519E+00, 5.53737E+01, 1.9348E+01, 2.4916E+03, &
      2.1917561E+00, 5.54502E+01, 1.9221E+01, 2.4630E+03, &
      2.2027149E+00, 5.55250E+01, 1.9095E+01, 2.4348E+03, &
      2.2137285E+00, 5.57481E+01, 1.8969E+01, 2.4068E+03, &
      2.2247971E+00, 5.58202E+01, 1.8841E+01, 2.3787E+03, &
      2.2359211E+00, 5.58904E+01, 1.8713E+01, 2.3509E+03, &
      2.2471007E+00, 5.59589E+01, 1.8586E+01, 2.3234E+03, &
      2.2583362E+00, 5.60256E+01, 1.8459E+01, 2.2961E+03, &
      2.2696279E+00, 5.60905E+01, 1.8333E+01, 2.2692E+03, &
      2.2809760E+00, 5.61536E+01, 1.8208E+01, 2.2426E+03, &
      2.2923809E+00, 5.62152E+01, 1.8084E+01, 2.2163E+03, &
      2.3038428E+00, 5.62753E+01, 1.7960E+01, 2.1903E+03, &
      2.3153620E+00, 5.63339E+01, 1.7838E+01, 2.1646E+03, &
      2.3269388E+00, 5.63910E+01, 1.7716E+01, 2.1392E+03, &
      2.3385735E+00, 5.64468E+01, 1.7595E+01, 2.1142E+03, &
      2.3502664E+00, 5.65012E+01, 1.7475E+01, 2.0894E+03, &
      2.3620177E+00, 5.65543E+01, 1.7355E+01, 2.0649E+03, &
      2.3738278E+00, 5.66062E+01, 1.7237E+01, 2.0407E+03, &
      2.3856970E+00, 5.66569E+01, 1.7119E+01, 2.0167E+03, &
      2.3976254E+00, 5.67064E+01, 1.7002E+01, 1.9931E+03, &
      2.4096136E+00, 5.67547E+01, 1.6886E+01, 1.9698E+03, &
      2.4216616E+00, 5.68020E+01, 1.6771E+01, 1.9467E+03, &
      2.4337699E+00, 5.68482E+01, 1.6654E+01, 1.9236E+03, &
      2.4459388E+00, 5.68933E+01, 1.6538E+01, 1.9007E+03, &
      2.4581685E+00, 5.69372E+01, 1.6423E+01, 1.8782E+03, &
      2.4704593E+00, 5.69801E+01, 1.6308E+01, 1.8559E+03, &
      2.4828116E+00, 5.70218E+01, 1.6194E+01, 1.8339E+03, &
      2.4952257E+00, 5.70626E+01, 1.6082E+01, 1.8121E+03, &
      2.5077018E+00, 5.71023E+01, 1.5970E+01, 1.7906E+03, &
      2.5202403E+00, 5.71411E+01, 1.5858E+01, 1.7694E+03, &
      2.5328415E+00, 5.71789E+01, 1.5748E+01, 1.7485E+03, &
      2.5455057E+00, 5.72158E+01, 1.5639E+01, 1.7278E+03, &
      2.5582333E+00, 5.72518E+01, 1.5530E+01, 1.7073E+03, &
      2.5710244E+00, 5.72869E+01, 1.5422E+01, 1.6871E+03, &
      2.5838796E+00, 5.73213E+01, 1.5315E+01, 1.6672E+03, &
      2.5967990E+00, 5.73548E+01, 1.5209E+01, 1.6475E+03, &
      2.6097829E+00, 5.73875E+01, 1.5104E+01, 1.6280E+03, &
      2.6228319E+00, 5.74195E+01, 1.5000E+01, 1.6088E+03, &
      2.6359460E+00, 5.74507E+01, 1.4896E+01, 1.5899E+03, &
      2.6491257E+00, 5.74813E+01, 1.4794E+01, 1.5711E+03, &
      2.6623714E+00, 5.75112E+01, 1.4692E+01, 1.5526E+03, &
      2.6756832E+00, 5.75404E+01, 1.4591E+01, 1.5344E+03, &
      2.6890617E+00, 5.75691E+01, 1.4490E+01, 1.5163E+03, &
      2.7025070E+00, 5.75971E+01, 1.4391E+01, 1.4985E+03, &
      2.7160195E+00, 5.76247E+01, 1.4292E+01, 1.4809E+03, &
      2.7295996E+00, 5.76516E+01, 1.4194E+01, 1.4635E+03, &
      2.7432476E+00, 5.76781E+01, 1.4097E+01, 1.4464E+03, &
      2.7569638E+00, 5.77042E+01, 1.4001E+01, 1.4295E+03, &
      2.7707486E+00, 5.77298E+01, 1.3905E+01, 1.4127E+03, &
      2.7846024E+00, 5.77551E+01, 1.3811E+01, 1.3962E+03, &
      2.7985254E+00, 5.77800E+01, 1.3717E+01, 1.3799E+03, &
      2.8125180E+00, 5.78046E+01, 1.3624E+01, 1.3638E+03, &
      2.8265806E+00, 5.78289E+01, 1.3531E+01, 1.3479E+03, &
      2.8407135E+00, 5.79556E+01, 1.3440E+01, 1.3322E+03, &
      2.8549171E+00, 5.79802E+01, 1.3347E+01, 1.3165E+03, &
      2.8691917E+00, 5.80046E+01, 1.3255E+01, 1.3010E+03, &
      2.8835376E+00, 5.80289E+01, 1.3164E+01, 1.2857E+03, &
      2.8979553E+00, 5.80531E+01, 1.3073E+01, 1.2706E+03, &
      2.9124451E+00, 5.80774E+01, 1.2983E+01, 1.2557E+03, &
      2.9270073E+00, 5.81021E+01, 1.2894E+01, 1.2409E+03, &
      2.9416424E+00, 5.81275E+01, 1.2806E+01, 1.2264E+03, &
      2.9563506E+00, 5.81540E+01, 1.2718E+01, 1.2120E+03, &
      2.9711323E+00, 5.81830E+01, 1.2632E+01, 1.1978E+03, &
      2.9859880E+00, 5.82179E+01, 1.2545E+01, 1.1838E+03, &
      3.0009179E+00, 5.82834E+01, 1.2459E+01, 1.1699E+03, &
      3.0159225E+00, 5.83010E+01, 1.2360E+01, 1.1549E+03, &
      3.0310021E+00, 5.83166E+01, 1.2262E+01, 1.1401E+03, &
      3.0461571E+00, 5.83307E+01, 1.2165E+01, 1.1256E+03, &
      3.0613879E+00, 5.83436E+01, 1.2069E+01, 1.1112E+03, &
      3.0766949E+00, 5.83556E+01, 1.1973E+01, 1.0970E+03, &
      3.0920783E+00, 5.84146E+01, 1.1877E+01, 1.0829E+03, &
      3.1075387E+00, 5.84249E+01, 1.1782E+01, 1.0690E+03, &
      3.1230764E+00, 5.84343E+01, 1.1687E+01, 1.0552E+03, &
      3.1386918E+00, 5.84427E+01, 1.1594E+01, 1.0416E+03, &
      3.1543853E+00, 5.84503E+01, 1.1501E+01, 1.0282E+03, &
      3.1701572E+00, 5.84570E+01, 1.1409E+01, 1.0150E+03, &
      3.1860080E+00, 5.84629E+01, 1.1318E+01, 1.0020E+03, &
      3.2019380E+00, 5.84681E+01, 1.1227E+01, 9.8910E+02, &
      3.2179477E+00, 5.84725E+01, 1.1138E+01, 9.7642E+02, &
      3.2340374E+00, 5.84763E+01, 1.1049E+01, 9.6391E+02, &
      3.2502076E+00, 5.84793E+01, 1.0961E+01, 9.5156E+02, &
      3.2664587E+00, 5.84817E+01, 1.0874E+01, 9.3939E+02, &
      3.2827910E+00, 5.84834E+01, 1.0788E+01, 9.2738E+02, &
      3.2992049E+00, 5.84846E+01, 1.0702E+01, 9.1553E+02, &
      3.3157009E+00, 5.84851E+01, 1.0617E+01, 9.0384E+02, &
      3.3322794E+00, 5.84850E+01, 1.0533E+01, 8.9230E+02, &
      3.3489408E+00, 5.84844E+01, 1.0450E+01, 8.8093E+02, &
      3.3656856E+00, 5.84832E+01, 1.0367E+01, 8.6970E+02, &
      3.3825140E+00, 5.84816E+01, 1.0286E+01, 8.5863E+02, &
      3.3994265E+00, 5.84793E+01, 1.0205E+01, 8.4771E+02, &
      3.4164237E+00, 5.84766E+01, 1.0124E+01, 8.3693E+02, &
      3.4335058E+00, 5.84735E+01, 1.0045E+01, 8.2630E+02, &
      3.4506733E+00, 5.84903E+01, 9.9655E+00, 8.1579E+02, &
      3.4679267E+00, 5.84863E+01, 9.8865E+00, 8.0538E+02, &
      3.4852663E+00, 5.84818E+01, 9.8083E+00, 7.9512E+02, &
      3.5026927E+00, 5.84768E+01, 9.7308E+00, 7.8499E+02, &
      3.5202061E+00, 5.84713E+01, 9.6540E+00, 7.7500E+02, &
      3.5378072E+00, 5.84653E+01, 9.5778E+00, 7.6514E+02, &
      3.5554962E+00, 5.84588E+01, 9.5024E+00, 7.5541E+02, &
      3.5732737E+00, 5.84519E+01, 9.4276E+00, 7.4581E+02, &
      3.5911400E+00, 5.84445E+01, 9.3534E+00, 7.3635E+02, &
      3.6090957E+00, 5.84367E+01, 9.2799E+00, 7.2701E+02, &
      3.6271412E+00, 5.84285E+01, 9.2071E+00, 7.1779E+02, &
      3.6452769E+00, 5.84198E+01, 9.1349E+00, 7.0870E+02, &
      3.6635033E+00, 5.84107E+01, 9.0634E+00, 6.9972E+02, &
      3.6818208E+00, 5.84012E+01, 8.9925E+00, 6.9087E+02, &
      3.7002299E+00, 5.83914E+01, 8.9222E+00, 6.8214E+02, &
      3.7187311E+00, 5.83811E+01, 8.8525E+00, 6.7352E+02, &
      3.7373247E+00, 5.83704E+01, 8.7834E+00, 6.6502E+02, &
      3.7560114E+00, 5.83594E+01, 8.7150E+00, 6.5663E+02, &
      3.7747914E+00, 5.83480E+01, 8.6472E+00, 6.4835E+02, &
      3.7936654E+00, 5.83362E+01, 8.5799E+00, 6.4018E+02, &
      3.8126337E+00, 5.83241E+01, 8.5132E+00, 6.3212E+02, &
      3.8316969E+00, 5.83116E+01, 8.4472E+00, 6.2417E+02, &
      3.8508554E+00, 5.82987E+01, 8.3817E+00, 6.1632E+02, &
      3.8701096E+00, 5.82855E+01, 8.3167E+00, 6.0858E+02, &
      3.8894602E+00, 5.82720E+01, 8.2524E+00, 6.0094E+02, &
      3.9089075E+00, 5.82581E+01, 8.1886E+00, 5.9340E+02, &
      3.9284520E+00, 5.82439E+01, 8.1254E+00, 5.8596E+02, &
      3.9480943E+00, 5.82293E+01, 8.0627E+00, 5.7862E+02, &
      3.9678347E+00, 5.82144E+01, 8.0005E+00, 5.7137E+02, &
      3.9876739E+00, 5.81992E+01, 7.9390E+00, 5.6422E+02, &
      4.055024E+00, 5.84761E+01, 7.7360E+00, 5.4093E+02, &
      4.334821E+00, 5.81800E+01, 6.9839E+00, 4.5766E+02, &
      4.633924E+00, 5.78318E+01, 6.3137E+00, 3.8783E+02, &
      4.953664E+00, 5.74103E+01, 5.6849E+00, 3.2743E+02, &
      5.295467E+00, 5.68644E+01, 5.1162E+00, 2.7638E+02, &
      5.660855E+00, 5.61302E+01, 4.6131E+00, 2.3380E+02, &
      6.051453E+00, 5.50264E+01, 4.1674E+00, 1.9819E+02, &
      6.469004E+00, 5.27472E+01, 3.7560E+00, 1.6768E+02, &
      6.581876E+00, 5.13760E+01, 3.6557E+00, 1.6057E+02, &
      6.682619E+00, 4.82974E+01, 3.5699E+00, 1.5457E+02, &
      6.709484E+00, 4.47163E+01, 3.5469E+00, 1.5300E+02, &
      6.749781E+00, 4.82774E+01, 1.0486E+01, 4.3981E+02, &
      6.850524E+00, 5.13435E+01, 1.0155E+01, 4.1985E+02, &
      6.915365E+00, 5.21218E+01, 9.9509E+00, 4.0764E+02, &
      7.165564E+00, 5.28838E+01, 9.2642E+00, 3.6662E+02, &
      7.275241E+00, 5.18181E+01, 9.0154E+00, 3.5153E+02, &
      7.304488E+00, 5.01326E+01, 8.9518E+00, 3.4770E+02, &
      7.348359E+00, 5.20594E+01, 1.2348E+01, 4.7498E+02, &
      7.392525E+00, 5.30809E+01, 1.2208E+01, 4.6683E+02, &
      7.458036E+00, 5.38886E+01, 1.2005E+01, 4.5513E+02, &
      7.582064E+00, 5.46545E+01, 1.1638E+01, 4.3414E+02, &
      7.698116E+00, 5.45824E+01, 1.1318E+01, 4.1598E+02, &
      7.729063E+00, 5.38405E+01, 1.1241E+01, 4.1151E+02, &
      7.775484E+00, 5.49428E+01, 1.2884E+01, 4.6820E+02, &
      7.891536E+00, 5.62108E+01, 1.2596E+01, 4.5112E+02, &
      7.902609E+00, 5.62920E+01, 1.2569E+01, 4.4954E+02, &
      8.447890E+00, 5.86461E+01, 1.1380E+01, 3.8116E+02, &
      9.030794E+00, 6.00049E+01, 1.0264E+01, 3.2201E+02, &
      9.653919E+00, 6.08642E+01, 9.1830E+00, 2.6989E+02, &
      1.032004E+01, 6.14251E+01, 8.2105E+00, 2.2611E+02, &
      1.103212E+01, 6.17932E+01, 7.3402E+00, 1.8944E+02, &
      1.179334E+01, 6.20339E+01, 6.5679E+00, 1.5888E+02, &
      1.260708E+01, 6.21879E+01, 5.8820E+00, 1.3338E+02, &
      1.347697E+01, 6.23534E+01, 5.2711E+00, 1.1208E+02, &
      1.440688E+01, 6.23992E+01, 4.7077E+00, 9.3885E+01, &
      1.540095E+01, 6.24411E+01, 4.2005E+00, 7.8589E+01, &
      1.646362E+01, 6.24459E+01, 3.7379E+00, 6.5634E+01, &
      1.759961E+01, 6.23903E+01, 3.3284E+00, 5.4863E+01, &
      1.881398E+01, 6.23177E+01, 2.9663E+00, 4.5914E+01, &
      2.011215E+01, 6.22344E+01, 2.6460E+00, 3.8471E+01, &
      2.149988E+01, 6.21440E+01, 2.3624E+00, 3.2273E+01, &
      2.298338E+01, 6.20492E+01, 2.1111E+00, 2.7107E+01, &
      2.456923E+01, 6.19516E+01, 1.8879E+00, 2.2794E+01, &
      2.626450E+01, 6.18538E+01, 1.6898E+00, 1.9190E+01, &
      2.807676E+01, 6.17569E+01, 1.5024E+00, 1.6064E+01, &
      3.001405E+01, 6.16432E+01, 1.3321E+00, 1.3420E+01, &
      3.208502E+01, 6.15154E+01, 1.1796E+00, 1.1205E+01, &
      3.429889E+01, 6.13696E+01, 1.0428E+00, 9.3495E+00, &
      3.666551E+01, 6.11938E+01, 9.1997E-01, 7.7927E+00, &
      3.919543E+01, 6.09665E+01, 8.1156E-01, 6.5016E+00, &
      4.189992E+01, 6.06307E+01, 7.1648E-01, 5.4340E+00, &
      4.479101E+01, 5.99454E+01, 6.3304E-01, 4.5504E+00, &
      4.589752E+01, 5.93118E+01, 6.0513E-01, 4.2665E+00, &
      4.660003E+01, 5.81331E+01, 5.8841E-01, 4.0994E+00, &
      4.678737E+01, 5.67210E+01, 5.8407E-01, 4.0564E+00, &
      4.706837E+01, 5.81477E+01, 3.3575E+00, 2.0521E+01, &
      4.777089E+01, 5.94373E+01, 3.2817E+00, 1.9774E+01, &
      4.788159E+01, 5.95434E+01, 3.2700E+00, 1.9659E+01, &
      5.118542E+01, 6.09184E+01, 2.9424E+00, 1.6590E+01, &
      5.471721E+01, 6.14636E+01, 2.6381E+00, 1.3955E+01, &
      5.849270E+01, 6.17813E+01, 2.3625E+00, 1.1728E+01, &
      6.252870E+01, 6.19873E+01, 2.1137E+00, 9.8525E+00, &
      6.684318E+01, 6.21242E+01, 1.8891E+00, 8.2714E+00, &
      7.145536E+01, 6.22165E+01, 1.6883E+00, 6.9473E+00, &
      7.638578E+01, 6.22786E+01, 1.5088E+00, 5.8385E+00, &
      8.165640E+01, 6.23211E+01, 1.3466E+00, 4.9035E+00, &
      8.729069E+01, 6.23457E+01, 1.1983E+00, 4.1099E+00, &
      9.331374E+01, 6.23573E+01, 1.0664E+00, 3.4483E+00, &
      9.975239E+01, 6.23883E+01, 9.4560E-01, 2.8867E+00, &
      1.066353E+02, 6.23810E+01, 8.3837E-01, 2.4192E+00, &
      1.139931E+02, 6.23684E+01, 7.4336E-01, 2.0306E+00, &
      1.218587E+02, 6.23525E+01, 6.5918E-01, 1.7074E+00, &
      1.302669E+02, 6.23345E+01, 5.8459E-01, 1.4385E+00, &
      1.392553E+02, 6.23153E+01, 5.1848E-01, 1.2147E+00, &
      1.488640E+02, 6.22955E+01, 4.5988E-01, 1.0282E+00, &
      1.591356E+02, 6.22757E+01, 4.0794E-01, 8.7285E-01, &
      1.701159E+02, 6.22562E+01, 3.6189E-01, 7.4328E-01, &
      1.818539E+02, 6.22371E+01, 3.2107E-01, 6.3515E-01, &
      1.944018E+02, 6.22187E+01, 2.8487E-01, 5.4482E-01, &
      2.078156E+02, 6.22010E+01, 2.5307E-01, 4.6969E-01, &
      2.221548E+02, 6.21845E+01, 2.2533E-01, 4.0734E-01, &
      2.374835E+02, 6.21693E+01, 2.0065E-01, 3.5490E-01, &
      2.538699E+02, 6.21551E+01, 1.7868E-01, 3.1072E-01, &
      2.713869E+02, 6.21419E+01, 1.5913E-01, 2.7343E-01, &
      2.901126E+02, 6.21297E+01, 1.4172E-01, 2.4190E-01, &
      3.101304E+02, 6.21185E+01, 1.2623E-01, 2.1517E-01, &
      3.315294E+02, 6.21081E+01, 1.1243E-01, 1.9244E-01, &
      3.544049E+02, 6.20985E+01, 1.0015E-01, 1.7306E-01, &
      3.788588E+02, 6.20896E+01, 8.9218E-02, 1.5648E-01, &
      4.050001E+02, 6.20815E+01, 7.9481E-02, 1.4223E-01, &
      4.329451E+02, 6.20741E+01, 7.0810E-02, 1.2994E-01]
   case(63)
      frel = -5.7720E-01
      allocate(nist_tab(3572))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.39833E+01, 2.5065E-01, 6.4928E+03, &
      1.142761E-02, 1.38468E+01, 2.8095E-01, 6.8081E+03, &
      1.221612E-02, 1.36829E+01, 3.1548E-01, 7.1513E+03, &
      1.305903E-02, 1.34837E+01, 3.5479E-01, 7.5232E+03, &
      1.396010E-02, 1.32373E+01, 3.9950E-01, 7.9247E+03, &
      1.492335E-02, 1.29256E+01, 4.5032E-01, 8.3561E+03, &
      1.595306E-02, 1.25184E+01, 5.0798E-01, 8.8177E+03, &
      1.705382E-02, 1.19616E+01, 5.7330E-01, 9.3091E+03, &
      1.823053E-02, 1.11444E+01, 6.4715E-01, 9.8300E+03, &
      1.948844E-02, 9.78838E+00, 7.3045E-01, 1.0379E+04, &
      2.083314E-02, 6.82187E+00, 8.2418E-01, 1.0955E+04, &
      2.156000E-02, 2.42628E+00, 8.7675E-01, 1.1261E+04, &
      2.189000E-02, -4.57615E+00, 9.0104E-01, 1.1398E+04, &
      2.197800E-02, -1.32262E+01, 9.0756E-01, 1.1435E+04, &
      2.211000E-02, -3.85948E+00, 1.8081E+01, 2.2646E+05, &
      2.227063E-02, 1.46570E+00, 1.7741E+01, 2.2060E+05, &
      2.244000E-02, 4.42720E+00, 1.7394E+01, 2.1464E+05, &
      2.380730E-02, 1.32311E+01, 1.4940E+01, 1.7378E+05, &
      2.545001E-02, 1.69560E+01, 1.2596E+01, 1.3706E+05, &
      2.720606E-02, 1.88177E+01, 1.0552E+01, 1.0740E+05, &
      2.908327E-02, 1.97321E+01, 8.8615E+00, 8.4375E+04, &
      3.109002E-02, 2.01046E+01, 7.5261E+00, 6.7035E+04, &
      3.116400E-02, 2.01107E+01, 7.4848E+00, 6.6508E+04, &
      3.164100E-02, 2.01363E+01, 7.2300E+00, 6.3276E+04, &
      3.176820E-02, 2.01315E+01, 7.1655E+00, 6.2460E+04, &
      3.195900E-02, 2.01522E+01, 7.0947E+00, 6.1474E+04, &
      3.243600E-02, 2.01771E+01, 6.8729E+00, 5.8676E+04, &
      3.323523E-02, 2.01866E+01, 6.5399E+00, 5.4490E+04, &
      3.552846E-02, 2.01102E+01, 5.8100E+00, 4.5285E+04, &
      3.797993E-02, 1.99568E+01, 5.3111E+00, 3.8724E+04, &
      4.060054E-02, 1.97831E+01, 5.0003E+00, 3.4105E+04, &
      4.340198E-02, 1.96243E+01, 4.8414E+00, 3.0890E+04, &
      4.639671E-02, 1.95039E+01, 4.8054E+00, 2.8681E+04, &
      4.959809E-02, 1.94397E+01, 4.8696E+00, 2.7188E+04, &
      5.302035E-02, 1.94498E+01, 5.0163E+00, 2.6199E+04, &
      5.667876E-02, 1.95454E+01, 5.1276E+00, 2.5052E+04, &
      6.058959E-02, 1.96637E+01, 5.1817E+00, 2.3682E+04, &
      6.477028E-02, 1.97952E+01, 5.2494E+00, 2.2443E+04, &
      6.923942E-02, 1.99539E+01, 5.3225E+00, 2.1287E+04, &
      7.401695E-02, 2.01062E+01, 5.3942E+00, 2.0181E+04, &
      7.912411E-02, 2.02598E+01, 5.4590E+00, 1.9105E+04, &
      8.458368E-02, 2.04046E+01, 5.5126E+00, 1.8048E+04, &
      9.041995E-02, 2.05260E+01, 5.5518E+00, 1.7003E+04, &
      9.665893E-02, 2.06028E+01, 5.5743E+00, 1.5970E+04, &
      1.0000000E-01, 2.35129E+01, 5.5787E+00, 1.5449E+04, &
      1.0050000E-01, 2.35122E+01, 5.5790E+00, 1.5373E+04, &
      1.0100250E-01, 2.35108E+01, 5.5791E+00, 1.5297E+04, &
      1.0150751E-01, 2.35088E+01, 5.5791E+00, 1.5220E+04, &
      1.0201505E-01, 2.35060E+01, 5.5790E+00, 1.5145E+04, &
      1.0252513E-01, 2.35024E+01, 5.5788E+00, 1.5069E+04, &
      1.0303775E-01, 2.34981E+01, 5.5785E+00, 1.4993E+04, &
      1.0355294E-01, 2.34929E+01, 5.5780E+00, 1.4917E+04, &
      1.0407070E-01, 2.34868E+01, 5.5775E+00, 1.4841E+04, &
      1.0459106E-01, 2.34797E+01, 5.5769E+00, 1.4766E+04, &
      1.0511401E-01, 2.34717E+01, 5.5761E+00, 1.4690E+04, &
      1.0563958E-01, 2.34626E+01, 5.5752E+00, 1.4615E+04, &
      1.0616778E-01, 2.34524E+01, 5.5742E+00, 1.4540E+04, &
      1.0669862E-01, 2.34410E+01, 5.5731E+00, 1.4465E+04, &
      1.0723211E-01, 2.34284E+01, 5.5719E+00, 1.4389E+04, &
      1.0776827E-01, 2.34145E+01, 5.5706E+00, 1.4314E+04, &
      1.0830712E-01, 2.33992E+01, 5.5691E+00, 1.4240E+04, &
      1.0884865E-01, 2.33824E+01, 5.5676E+00, 1.4165E+04, &
      1.0939289E-01, 2.33640E+01, 5.5659E+00, 1.4090E+04, &
      1.0993986E-01, 2.33439E+01, 5.5641E+00, 1.4015E+04, &
      1.1048956E-01, 2.33221E+01, 5.5622E+00, 1.3941E+04, &
      1.1104201E-01, 2.32983E+01, 5.5602E+00, 1.3867E+04, &
      1.1159722E-01, 2.32726E+01, 5.5581E+00, 1.3792E+04, &
      1.1215520E-01, 2.32446E+01, 5.5558E+00, 1.3718E+04, &
      1.1271598E-01, 2.32143E+01, 5.5535E+00, 1.3644E+04, &
      1.1327956E-01, 2.31814E+01, 5.5510E+00, 1.3570E+04, &
      1.1384596E-01, 2.31458E+01, 5.5484E+00, 1.3496E+04, &
      1.1441519E-01, 2.31073E+01, 5.5457E+00, 1.3423E+04, &
      1.1498726E-01, 2.30656E+01, 5.5429E+00, 1.3349E+04, &
      1.1556220E-01, 2.30205E+01, 5.5400E+00, 1.3276E+04, &
      1.1614001E-01, 2.29716E+01, 5.5369E+00, 1.3202E+04, &
      1.1672071E-01, 2.29187E+01, 5.5337E+00, 1.3129E+04, &
      1.1730431E-01, 2.28613E+01, 5.5305E+00, 1.3056E+04, &
      1.1789083E-01, 2.27989E+01, 5.5271E+00, 1.2983E+04, &
      1.1848029E-01, 2.27312E+01, 5.5236E+00, 1.2911E+04, &
      1.1907269E-01, 2.26575E+01, 5.5199E+00, 1.2838E+04, &
      1.1966805E-01, 2.25772E+01, 5.5162E+00, 1.2765E+04, &
      1.2026639E-01, 2.24895E+01, 5.5123E+00, 1.2693E+04, &
      1.2086772E-01, 2.23935E+01, 5.5084E+00, 1.2621E+04, &
      1.2147206E-01, 2.22882E+01, 5.5043E+00, 1.2549E+04, &
      1.2207942E-01, 2.21724E+01, 5.5001E+00, 1.2477E+04, &
      1.2268982E-01, 2.20445E+01, 5.4958E+00, 1.2405E+04, &
      1.2330327E-01, 2.19029E+01, 5.4914E+00, 1.2333E+04, &
      1.2391979E-01, 2.17452E+01, 5.4868E+00, 1.2262E+04, &
      1.2453939E-01, 2.15689E+01, 5.4822E+00, 1.2190E+04, &
      1.2516208E-01, 2.13706E+01, 5.4774E+00, 1.2119E+04, &
      1.2578789E-01, 2.11459E+01, 5.4725E+00, 1.2048E+04, &
      1.2641683E-01, 2.08893E+01, 5.4675E+00, 1.1977E+04, &
      1.2704892E-01, 2.05935E+01, 5.4624E+00, 1.1907E+04, &
      1.2768416E-01, 2.02483E+01, 5.4572E+00, 1.1836E+04, &
      1.2832258E-01, 1.98395E+01, 5.4519E+00, 1.1766E+04, &
      1.2896419E-01, 1.93466E+01, 5.4465E+00, 1.1696E+04, &
      1.2960902E-01, 1.87376E+01, 5.4409E+00, 1.1626E+04, &
      1.3025706E-01, 1.79602E+01, 5.4353E+00, 1.1556E+04, &
      1.3090835E-01, 1.69196E+01, 5.4295E+00, 1.1486E+04, &
      1.3156289E-01, 1.54187E+01, 5.4236E+00, 1.1417E+04, &
      1.3222070E-01, 1.29303E+01, 5.4176E+00, 1.1347E+04, &
      1.3288181E-01, 6.86825E+00, 5.4115E+00, 1.1278E+04, &
      1.3317243E-01, -7.84966E+00, 5.4088E+00, 1.1248E+04, &
      1.3322758E-01, -7.66782E+00, 2.5088E+01, 5.2146E+04, &
      1.3354621E-01, 8.89574E+00, 2.4497E+01, 5.0797E+04, &
      1.3421395E-01, 1.64402E+01, 2.3320E+01, 4.8115E+04, &
      1.3488502E-01, 2.01017E+01, 2.2214E+01, 4.5606E+04, &
      1.3555944E-01, 2.25011E+01, 2.1176E+01, 4.3259E+04, &
      1.3623724E-01, 2.42431E+01, 2.0201E+01, 4.1062E+04, &
      1.3691842E-01, 2.55737E+01, 1.9285E+01, 3.9005E+04, &
      1.3760302E-01, 2.66201E+01, 1.8425E+01, 3.7080E+04, &
      1.3829103E-01, 2.74577E+01, 1.7617E+01, 3.5278E+04, &
      1.3898249E-01, 2.81354E+01, 1.6858E+01, 3.3591E+04, &
      1.3967740E-01, 2.86869E+01, 1.6146E+01, 3.2010E+04, &
      1.4037579E-01, 2.91369E+01, 1.5476E+01, 3.0530E+04, &
      1.4107766E-01, 2.95036E+01, 1.4847E+01, 2.9143E+04, &
      1.4178305E-01, 2.98012E+01, 1.4256E+01, 2.7844E+04, &
      1.4249197E-01, 3.00407E+01, 1.3700E+01, 2.6626E+04, &
      1.4320443E-01, 3.02312E+01, 1.3178E+01, 2.5484E+04, &
      1.4392045E-01, 3.03796E+01, 1.2688E+01, 2.4413E+04, &
      1.4464005E-01, 3.04919E+01, 1.2227E+01, 2.3409E+04, &
      1.4536325E-01, 3.05726E+01, 1.1793E+01, 2.2467E+04, &
      1.4609007E-01, 3.06253E+01, 1.1386E+01, 2.1583E+04, &
      1.4682052E-01, 3.06528E+01, 1.1003E+01, 2.0754E+04, &
      1.4755462E-01, 3.06580E+01, 1.0650E+01, 1.9988E+04, &
      1.4829239E-01, 3.06466E+01, 1.0325E+01, 1.9282E+04, &
      1.4903386E-01, 3.06229E+01, 1.0027E+01, 1.8631E+04, &
      1.4977903E-01, 3.05901E+01, 9.7515E+00, 1.8030E+04, &
      1.5052792E-01, 3.05503E+01, 9.4978E+00, 1.7474E+04, &
      1.5128056E-01, 3.05054E+01, 9.2634E+00, 1.6958E+04, &
      1.5203696E-01, 3.04567E+01, 9.0465E+00, 1.6478E+04, &
      1.5279715E-01, 3.04054E+01, 8.8455E+00, 1.6032E+04, &
      1.5356113E-01, 3.03522E+01, 8.6590E+00, 1.5616E+04, &
      1.5432894E-01, 3.02980E+01, 8.4858E+00, 1.5227E+04, &
      1.5510058E-01, 3.02431E+01, 8.3245E+00, 1.4864E+04, &
      1.5587609E-01, 3.01882E+01, 8.1743E+00, 1.4523E+04, &
      1.5665547E-01, 3.01335E+01, 8.0342E+00, 1.4203E+04, &
      1.5743875E-01, 3.00792E+01, 7.9033E+00, 1.3902E+04, &
      1.5822594E-01, 3.00257E+01, 7.7808E+00, 1.3619E+04, &
      1.5901707E-01, 2.99731E+01, 7.6661E+00, 1.3351E+04, &
      1.5981215E-01, 2.99215E+01, 7.5585E+00, 1.3098E+04, &
      1.6061121E-01, 2.98709E+01, 7.4575E+00, 1.2859E+04, &
      1.6141427E-01, 2.98216E+01, 7.3626E+00, 1.2632E+04, &
      1.6222134E-01, 2.97734E+01, 7.2732E+00, 1.2417E+04, &
      1.6303245E-01, 2.97265E+01, 7.1890E+00, 1.2212E+04, &
      1.6384761E-01, 2.96809E+01, 7.1095E+00, 1.2017E+04, &
      1.6466685E-01, 2.96365E+01, 7.0345E+00, 1.1831E+04, &
      1.6549018E-01, 2.95934E+01, 6.9636E+00, 1.1653E+04, &
      1.6631763E-01, 2.95516E+01, 6.8964E+00, 1.1484E+04, &
      1.6714922E-01, 2.95109E+01, 6.8328E+00, 1.1321E+04, &
      1.6798497E-01, 2.94715E+01, 6.7724E+00, 1.1165E+04, &
      1.6882489E-01, 2.94332E+01, 6.7151E+00, 1.1016E+04, &
      1.6966902E-01, 2.93960E+01, 6.6606E+00, 1.0872E+04, &
      1.7051736E-01, 2.93600E+01, 6.6087E+00, 1.0734E+04, &
      1.7136995E-01, 2.93250E+01, 6.5594E+00, 1.0601E+04, &
      1.7222680E-01, 2.92911E+01, 6.5123E+00, 1.0472E+04, &
      1.7308793E-01, 2.92582E+01, 6.4674E+00, 1.0348E+04, &
      1.7395337E-01, 2.92262E+01, 6.4245E+00, 1.0229E+04, &
      1.7482314E-01, 2.91951E+01, 6.3835E+00, 1.0113E+04, &
      1.7569726E-01, 2.91649E+01, 6.3443E+00, 1.0001E+04, &
      1.7657574E-01, 2.91355E+01, 6.3068E+00, 9.8921E+03, &
      1.7745862E-01, 2.91069E+01, 6.2709E+00, 9.7868E+03, &
      1.7834591E-01, 2.90792E+01, 6.2364E+00, 9.6847E+03, &
      1.7923764E-01, 2.90521E+01, 6.2033E+00, 9.5854E+03, &
      1.8013383E-01, 2.90257E+01, 6.1716E+00, 9.4889E+03, &
      1.8103450E-01, 2.90000E+01, 6.1411E+00, 9.3951E+03, &
      1.8193967E-01, 2.89749E+01, 6.1118E+00, 9.3037E+03, &
      1.8284937E-01, 2.89504E+01, 6.0836E+00, 9.2148E+03, &
      1.8376362E-01, 2.89265E+01, 6.0564E+00, 9.1280E+03, &
      1.8468244E-01, 2.89031E+01, 6.0303E+00, 9.0435E+03, &
      1.8560585E-01, 2.88802E+01, 6.0051E+00, 8.9609E+03, &
      1.8653388E-01, 2.88578E+01, 5.9808E+00, 8.8803E+03, &
      1.8746655E-01, 2.88358E+01, 5.9574E+00, 8.8016E+03, &
      1.8840388E-01, 2.88142E+01, 5.9349E+00, 8.7247E+03, &
      1.8934590E-01, 2.87930E+01, 5.9131E+00, 8.6494E+03, &
      1.9029263E-01, 2.87722E+01, 5.8921E+00, 8.5758E+03, &
      1.9124409E-01, 2.87517E+01, 5.8718E+00, 8.5038E+03, &
      1.9220031E-01, 2.87315E+01, 5.8522E+00, 8.4332E+03, &
      1.9316131E-01, 2.87116E+01, 5.8332E+00, 8.3641E+03, &
      1.9412712E-01, 2.86919E+01, 5.8149E+00, 8.2964E+03, &
      1.9509776E-01, 2.86725E+01, 5.7972E+00, 8.2301E+03, &
      1.9607325E-01, 2.86533E+01, 5.7801E+00, 8.1650E+03, &
      1.9705361E-01, 2.86343E+01, 5.7636E+00, 8.1012E+03, &
      1.9803888E-01, 2.86155E+01, 5.7477E+00, 8.0386E+03, &
      1.9902907E-01, 2.85968E+01, 5.7322E+00, 7.9772E+03, &
      2.0002422E-01, 2.85782E+01, 5.7173E+00, 7.9169E+03, &
      2.0102434E-01, 2.85598E+01, 5.7029E+00, 7.8577E+03, &
      2.0202946E-01, 2.85414E+01, 5.6890E+00, 7.7996E+03, &
      2.0303961E-01, 2.85231E+01, 5.6756E+00, 7.7425E+03, &
      2.0405481E-01, 2.85048E+01, 5.6626E+00, 7.6864E+03, &
      2.0507508E-01, 2.84866E+01, 5.6501E+00, 7.6312E+03, &
      2.0610046E-01, 2.84684E+01, 5.6380E+00, 7.5771E+03, &
      2.0713096E-01, 2.84501E+01, 5.6264E+00, 7.5238E+03, &
      2.0816661E-01, 2.84318E+01, 5.6152E+00, 7.4715E+03, &
      2.0920745E-01, 2.84135E+01, 5.6044E+00, 7.4200E+03, &
      2.1025348E-01, 2.83951E+01, 5.5940E+00, 7.3695E+03, &
      2.1130475E-01, 2.83766E+01, 5.5840E+00, 7.3197E+03, &
      2.1236128E-01, 2.83580E+01, 5.5744E+00, 7.2708E+03, &
      2.1342308E-01, 2.83392E+01, 5.5651E+00, 7.2227E+03, &
      2.1449020E-01, 2.83203E+01, 5.5563E+00, 7.1754E+03, &
      2.1556265E-01, 2.83011E+01, 5.5478E+00, 7.1288E+03, &
      2.1664046E-01, 2.82818E+01, 5.5397E+00, 7.0830E+03, &
      2.1772366E-01, 2.82622E+01, 5.5320E+00, 7.0380E+03, &
      2.1881228E-01, 2.82424E+01, 5.5246E+00, 6.9937E+03, &
      2.1990634E-01, 2.82222E+01, 5.5176E+00, 6.9501E+03, &
      2.2100588E-01, 2.82018E+01, 5.5110E+00, 6.9072E+03, &
      2.2211090E-01, 2.81810E+01, 5.5047E+00, 6.8650E+03, &
      2.2322146E-01, 2.81597E+01, 5.4987E+00, 6.8235E+03, &
      2.2433757E-01, 2.81381E+01, 5.4931E+00, 6.7826E+03, &
      2.2545925E-01, 2.81160E+01, 5.4879E+00, 6.7425E+03, &
      2.2658655E-01, 2.80934E+01, 5.4830E+00, 6.7029E+03, &
      2.2771948E-01, 2.80702E+01, 5.4784E+00, 6.6641E+03, &
      2.2885808E-01, 2.80464E+01, 5.4741E+00, 6.6258E+03, &
      2.3000237E-01, 2.80219E+01, 5.4703E+00, 6.5882E+03, &
      2.3115238E-01, 2.79966E+01, 5.4667E+00, 6.5512E+03, &
      2.3230814E-01, 2.79706E+01, 5.4635E+00, 6.5148E+03, &
      2.3346969E-01, 2.79436E+01, 5.4606E+00, 6.4789E+03, &
      2.3463703E-01, 2.79155E+01, 5.4580E+00, 6.4437E+03, &
      2.3581022E-01, 2.78864E+01, 5.4558E+00, 6.4091E+03, &
      2.3698927E-01, 2.78560E+01, 5.4539E+00, 6.3750E+03, &
      2.3817422E-01, 2.78241E+01, 5.4523E+00, 6.3415E+03, &
      2.3936509E-01, 2.77905E+01, 5.4511E+00, 6.3086E+03, &
      2.4056191E-01, 2.77551E+01, 5.4502E+00, 6.2762E+03, &
      2.4176472E-01, 2.77175E+01, 5.4497E+00, 6.2443E+03, &
      2.4297355E-01, 2.76774E+01, 5.4494E+00, 6.2130E+03, &
      2.4418841E-01, 2.76342E+01, 5.4495E+00, 6.1822E+03, &
      2.4540936E-01, 2.75873E+01, 5.4499E+00, 6.1520E+03, &
      2.4663640E-01, 2.75359E+01, 5.4507E+00, 6.1223E+03, &
      2.4786959E-01, 2.74788E+01, 5.4518E+00, 6.0930E+03, &
      2.4910893E-01, 2.74142E+01, 5.4532E+00, 6.0643E+03, &
      2.5035448E-01, 2.73394E+01, 5.4549E+00, 6.0361E+03, &
      2.5160625E-01, 2.72499E+01, 5.4570E+00, 6.0084E+03, &
      2.5286428E-01, 2.71370E+01, 5.4594E+00, 5.9812E+03, &
      2.5412860E-01, 2.69814E+01, 5.4621E+00, 5.9544E+03, &
      2.5539925E-01, 2.67189E+01, 5.4652E+00, 5.9281E+03, &
      2.5641678E-01, 2.60606E+01, 5.4679E+00, 5.9075E+03, &
      2.5667624E-01, 2.57561E+01, 6.5471E+00, 7.0658E+03, &
      2.5678320E-01, 2.60554E+01, 6.5477E+00, 7.0636E+03, &
      2.5795962E-01, 2.67284E+01, 6.5549E+00, 7.0391E+03, &
      2.5924942E-01, 2.69422E+01, 6.5631E+00, 7.0129E+03, &
      2.6054567E-01, 2.70641E+01, 6.5716E+00, 6.9871E+03, &
      2.6184840E-01, 2.71475E+01, 6.5805E+00, 6.9617E+03, &
      2.6315764E-01, 2.72093E+01, 6.5897E+00, 6.9368E+03, &
      2.6447343E-01, 2.72573E+01, 6.5992E+00, 6.9123E+03, &
      2.6579579E-01, 2.72956E+01, 6.6091E+00, 6.8883E+03, &
      2.6712477E-01, 2.73266E+01, 6.6193E+00, 6.8646E+03, &
      2.6846040E-01, 2.73518E+01, 6.6299E+00, 6.8414E+03, &
      2.6980270E-01, 2.73722E+01, 6.6408E+00, 6.8186E+03, &
      2.7115171E-01, 2.73884E+01, 6.6520E+00, 6.7961E+03, &
      2.7250747E-01, 2.74006E+01, 6.6636E+00, 6.7741E+03, &
      2.7387001E-01, 2.74091E+01, 6.6755E+00, 6.7525E+03, &
      2.7523936E-01, 2.74137E+01, 6.6877E+00, 6.7312E+03, &
      2.7661556E-01, 2.74139E+01, 6.7003E+00, 6.7104E+03, &
      2.7799863E-01, 2.74088E+01, 6.7132E+00, 6.6898E+03, &
      2.7938863E-01, 2.73963E+01, 6.7264E+00, 6.6697E+03, &
      2.8078557E-01, 2.73718E+01, 6.7399E+00, 6.6499E+03, &
      2.8218950E-01, 2.73221E+01, 6.7538E+00, 6.6305E+03, &
      2.8360044E-01, 2.71502E+01, 6.7680E+00, 6.6114E+03, &
      2.8366578E-01, 2.71247E+01, 6.7686E+00, 6.6105E+03, &
      2.8413421E-01, 2.71284E+01, 7.1061E+00, 6.9285E+03, &
      2.8501845E-01, 2.73011E+01, 7.1162E+00, 6.9168E+03, &
      2.8644354E-01, 2.73998E+01, 7.1326E+00, 6.8983E+03, &
      2.8787576E-01, 2.74586E+01, 7.1492E+00, 6.8800E+03, &
      2.8931514E-01, 2.75026E+01, 7.1662E+00, 6.8620E+03, &
      2.9076171E-01, 2.75387E+01, 7.1834E+00, 6.8444E+03, &
      2.9221552E-01, 2.75699E+01, 7.2009E+00, 6.8269E+03, &
      2.9367660E-01, 2.75977E+01, 7.2187E+00, 6.8098E+03, &
      2.9514498E-01, 2.76229E+01, 7.2368E+00, 6.7929E+03, &
      2.9662071E-01, 2.76463E+01, 7.2551E+00, 6.7762E+03, &
      2.9810381E-01, 2.76681E+01, 7.2737E+00, 6.7598E+03, &
      2.9959433E-01, 2.76886E+01, 7.2925E+00, 6.7436E+03, &
      3.0109230E-01, 2.77082E+01, 7.3116E+00, 6.7276E+03, &
      3.0259776E-01, 2.77269E+01, 7.3309E+00, 6.7119E+03, &
      3.0411075E-01, 2.77448E+01, 7.3505E+00, 6.6963E+03, &
      3.0563130E-01, 2.77622E+01, 7.3703E+00, 6.6810E+03, &
      3.0715946E-01, 2.77789E+01, 7.3903E+00, 6.6659E+03, &
      3.0869526E-01, 2.77952E+01, 7.4106E+00, 6.6509E+03, &
      3.1023873E-01, 2.78111E+01, 7.4311E+00, 6.6362E+03, &
      3.1178993E-01, 2.78265E+01, 7.4518E+00, 6.6216E+03, &
      3.1334888E-01, 2.78416E+01, 7.4728E+00, 6.6072E+03, &
      3.1491562E-01, 2.78564E+01, 7.4939E+00, 6.5929E+03, &
      3.1649020E-01, 2.78709E+01, 7.5152E+00, 6.5788E+03, &
      3.1807265E-01, 2.78851E+01, 7.5367E+00, 6.5649E+03, &
      3.1966301E-01, 2.78990E+01, 7.5584E+00, 6.5511E+03, &
      3.2126133E-01, 2.79127E+01, 7.5803E+00, 6.5374E+03, &
      3.2286764E-01, 2.79260E+01, 7.6024E+00, 6.5238E+03, &
      3.2448197E-01, 2.79391E+01, 7.6246E+00, 6.5103E+03, &
      3.2610438E-01, 2.79519E+01, 7.6470E+00, 6.4970E+03, &
      3.2773491E-01, 2.79643E+01, 7.6695E+00, 6.4837E+03, &
      3.2937358E-01, 2.79764E+01, 7.6922E+00, 6.4706E+03, &
      3.3102045E-01, 2.79881E+01, 7.7150E+00, 6.4575E+03, &
      3.3267555E-01, 2.79994E+01, 7.7379E+00, 6.4445E+03, &
      3.3433893E-01, 2.80101E+01, 7.7609E+00, 6.4316E+03, &
      3.3601062E-01, 2.80203E+01, 7.7841E+00, 6.4187E+03, &
      3.3769068E-01, 2.80298E+01, 7.8074E+00, 6.4059E+03, &
      3.3937913E-01, 2.80384E+01, 7.8307E+00, 6.3931E+03, &
      3.4107602E-01, 2.80461E+01, 7.8542E+00, 6.3804E+03, &
      3.4278140E-01, 2.80526E+01, 7.8777E+00, 6.3677E+03, &
      3.4449531E-01, 2.80576E+01, 7.9014E+00, 6.3550E+03, &
      3.4621779E-01, 2.80607E+01, 7.9250E+00, 6.3424E+03, &
      3.4794888E-01, 2.80613E+01, 7.9488E+00, 6.3298E+03, &
      3.4968862E-01, 2.80586E+01, 7.9726E+00, 6.3172E+03, &
      3.5143706E-01, 2.80513E+01, 7.9964E+00, 6.3046E+03, &
      3.5319425E-01, 2.80373E+01, 8.0202E+00, 6.2919E+03, &
      3.5496022E-01, 2.80123E+01, 8.0441E+00, 6.2793E+03, &
      3.5673502E-01, 2.79669E+01, 8.0680E+00, 6.2667E+03, &
      3.5851870E-01, 2.78699E+01, 8.0919E+00, 6.2540E+03, &
      3.5978936E-01, 2.76514E+01, 8.1089E+00, 6.2450E+03, &
      3.6031129E-01, 2.74403E+01, 8.6402E+00, 6.6443E+03, &
      3.6061062E-01, 2.76621E+01, 8.6444E+00, 6.6420E+03, &
      3.6211285E-01, 2.79389E+01, 8.6652E+00, 6.6304E+03, &
      3.6392341E-01, 2.80747E+01, 8.6902E+00, 6.6165E+03, &
      3.6574303E-01, 2.81662E+01, 8.7151E+00, 6.6025E+03, &
      3.6757174E-01, 2.82393E+01, 8.7401E+00, 6.5884E+03, &
      3.6940960E-01, 2.83024E+01, 8.7650E+00, 6.5744E+03, &
      3.7125665E-01, 2.83592E+01, 8.7898E+00, 6.5602E+03, &
      3.7311293E-01, 2.84118E+01, 8.8146E+00, 6.5460E+03, &
      3.7497850E-01, 2.84614E+01, 8.8393E+00, 6.5317E+03, &
      3.7685339E-01, 2.85087E+01, 8.8639E+00, 6.5173E+03, &
      3.7873766E-01, 2.85544E+01, 8.8884E+00, 6.5029E+03, &
      3.8063135E-01, 2.85987E+01, 8.9128E+00, 6.4884E+03, &
      3.8253450E-01, 2.86420E+01, 8.9371E+00, 6.4737E+03, &
      3.8444718E-01, 2.86845E+01, 8.9614E+00, 6.4590E+03, &
      3.8636941E-01, 2.87263E+01, 8.9854E+00, 6.4442E+03, &
      3.8830126E-01, 2.87677E+01, 9.0094E+00, 6.4292E+03, &
      3.9024276E-01, 2.88086E+01, 9.0332E+00, 6.4142E+03, &
      3.9219398E-01, 2.88493E+01, 9.0569E+00, 6.3990E+03, &
      3.9415495E-01, 2.88897E+01, 9.0804E+00, 6.3838E+03, &
      3.9612572E-01, 2.89299E+01, 9.1037E+00, 6.3683E+03, &
      3.9810635E-01, 2.89700E+01, 9.1269E+00, 6.3528E+03, &
      4.0009688E-01, 2.89879E+01, 9.1498E+00, 6.3371E+03, &
      4.0209737E-01, 2.90279E+01, 9.1725E+00, 6.3213E+03, &
      4.0410785E-01, 2.90678E+01, 9.1950E+00, 6.3053E+03, &
      4.0612839E-01, 2.91076E+01, 9.2173E+00, 6.2892E+03, &
      4.0815904E-01, 2.91475E+01, 9.2394E+00, 6.2729E+03, &
      4.1019983E-01, 2.91874E+01, 9.2612E+00, 6.2564E+03, &
      4.1225083E-01, 2.92274E+01, 9.2827E+00, 6.2398E+03, &
      4.1431208E-01, 2.92674E+01, 9.3039E+00, 6.2230E+03, &
      4.1638364E-01, 2.93075E+01, 9.3248E+00, 6.2060E+03, &
      4.1846556E-01, 2.93476E+01, 9.3455E+00, 6.1889E+03, &
      4.2055789E-01, 2.93878E+01, 9.3659E+00, 6.1715E+03, &
      4.2266068E-01, 2.94281E+01, 9.3859E+00, 6.1540E+03, &
      4.2477398E-01, 2.94684E+01, 9.4057E+00, 6.1363E+03, &
      4.2689785E-01, 2.95088E+01, 9.4251E+00, 6.1185E+03, &
      4.2903234E-01, 2.95493E+01, 9.4443E+00, 6.1004E+03, &
      4.3117750E-01, 2.95899E+01, 9.4631E+00, 6.0822E+03, &
      4.3333339E-01, 2.96306E+01, 9.4816E+00, 6.0638E+03, &
      4.3550006E-01, 2.96714E+01, 9.4998E+00, 6.0453E+03, &
      4.3767756E-01, 2.97122E+01, 9.5176E+00, 6.0265E+03, &
      4.3986595E-01, 2.97531E+01, 9.5351E+00, 6.0076E+03, &
      4.4206528E-01, 2.97941E+01, 9.5522E+00, 5.9885E+03, &
      4.4427560E-01, 2.98352E+01, 9.5690E+00, 5.9692E+03, &
      4.4649698E-01, 2.98763E+01, 9.5854E+00, 5.9497E+03, &
      4.4872947E-01, 2.99175E+01, 9.6015E+00, 5.9301E+03, &
      4.5097311E-01, 2.99588E+01, 9.6172E+00, 5.9102E+03, &
      4.5322798E-01, 3.00001E+01, 9.6325E+00, 5.8902E+03, &
      4.5549412E-01, 3.00414E+01, 9.6474E+00, 5.8700E+03, &
      4.5777159E-01, 3.00828E+01, 9.6619E+00, 5.8497E+03, &
      4.6006045E-01, 3.01243E+01, 9.6760E+00, 5.8291E+03, &
      4.6236075E-01, 3.01658E+01, 9.6898E+00, 5.8084E+03, &
      4.6467255E-01, 3.02073E+01, 9.7031E+00, 5.7875E+03, &
      4.6699592E-01, 3.02488E+01, 9.7160E+00, 5.7664E+03, &
      4.6933090E-01, 3.02903E+01, 9.7285E+00, 5.7451E+03, &
      4.7167755E-01, 3.03318E+01, 9.7406E+00, 5.7237E+03, &
      4.7403594E-01, 3.03733E+01, 9.7523E+00, 5.7021E+03, &
      4.7640612E-01, 3.04149E+01, 9.7636E+00, 5.6804E+03, &
      4.7878815E-01, 3.04564E+01, 9.7744E+00, 5.6584E+03, &
      4.8118209E-01, 3.04978E+01, 9.7849E+00, 5.6363E+03, &
      4.8358800E-01, 3.05393E+01, 9.7949E+00, 5.6140E+03, &
      4.8600594E-01, 3.05806E+01, 9.8044E+00, 5.5916E+03, &
      4.8843597E-01, 3.06220E+01, 9.8135E+00, 5.5690E+03, &
      4.9087815E-01, 3.06633E+01, 9.8222E+00, 5.5463E+03, &
      4.9333254E-01, 3.07045E+01, 9.8305E+00, 5.5233E+03, &
      4.9579920E-01, 3.07456E+01, 9.8383E+00, 5.5003E+03, &
      4.9827820E-01, 3.07866E+01, 9.8457E+00, 5.4771E+03, &
      5.0076959E-01, 3.08276E+01, 9.8526E+00, 5.4537E+03, &
      5.0327344E-01, 3.08684E+01, 9.8591E+00, 5.4302E+03, &
      5.0578980E-01, 3.09092E+01, 9.8651E+00, 5.4065E+03, &
      5.0831875E-01, 3.09498E+01, 9.8707E+00, 5.3827E+03, &
      5.1086035E-01, 3.09902E+01, 9.8758E+00, 5.3588E+03, &
      5.1341465E-01, 3.10306E+01, 9.8805E+00, 5.3347E+03, &
      5.1598172E-01, 3.10708E+01, 9.8847E+00, 5.3105E+03, &
      5.1856163E-01, 3.11108E+01, 9.8885E+00, 5.2861E+03, &
      5.2115444E-01, 3.11507E+01, 9.8919E+00, 5.2616E+03, &
      5.2376021E-01, 3.11904E+01, 9.8948E+00, 5.2370E+03, &
      5.2637901E-01, 3.12299E+01, 9.8972E+00, 5.2123E+03, &
      5.2901091E-01, 3.12692E+01, 9.8992E+00, 5.1875E+03, &
      5.3165596E-01, 3.13083E+01, 9.9007E+00, 5.1625E+03, &
      5.3431424E-01, 3.13472E+01, 9.9018E+00, 5.1374E+03, &
      5.3698581E-01, 3.13859E+01, 9.9023E+00, 5.1122E+03, &
      5.3967074E-01, 3.14243E+01, 9.9023E+00, 5.0868E+03, &
      5.4236910E-01, 3.14625E+01, 9.9018E+00, 5.0613E+03, &
      5.4508094E-01, 3.15004E+01, 9.9007E+00, 5.0356E+03, &
      5.4780635E-01, 3.15380E+01, 9.8991E+00, 5.0098E+03, &
      5.5054538E-01, 3.15753E+01, 9.8970E+00, 4.9839E+03, &
      5.5329810E-01, 3.16122E+01, 9.8944E+00, 4.9578E+03, &
      5.5606460E-01, 3.16489E+01, 9.8912E+00, 4.9316E+03, &
      5.5884492E-01, 3.16851E+01, 9.8876E+00, 4.9053E+03, &
      5.6163914E-01, 3.17210E+01, 9.8834E+00, 4.8789E+03, &
      5.6444734E-01, 3.17565E+01, 9.8787E+00, 4.8524E+03, &
      5.6726958E-01, 3.17915E+01, 9.8736E+00, 4.8258E+03, &
      5.7010592E-01, 3.18262E+01, 9.8679E+00, 4.7991E+03, &
      5.7295645E-01, 3.18604E+01, 9.8618E+00, 4.7723E+03, &
      5.7582123E-01, 3.18941E+01, 9.8552E+00, 4.7455E+03, &
      5.7870034E-01, 3.19274E+01, 9.8481E+00, 4.7185E+03, &
      5.8159384E-01, 3.19602E+01, 9.8405E+00, 4.6915E+03, &
      5.8450181E-01, 3.19926E+01, 9.8325E+00, 4.6644E+03, &
      5.8742432E-01, 3.20244E+01, 9.8241E+00, 4.6373E+03, &
      5.9036144E-01, 3.20557E+01, 9.8151E+00, 4.6101E+03, &
      5.9331325E-01, 3.20864E+01, 9.8058E+00, 4.5828E+03, &
      5.9627982E-01, 3.21166E+01, 9.7960E+00, 4.5555E+03, &
      5.9926122E-01, 3.21462E+01, 9.7857E+00, 4.5282E+03, &
      6.0225752E-01, 3.21753E+01, 9.7750E+00, 4.5008E+03, &
      6.0526881E-01, 3.22038E+01, 9.7639E+00, 4.4733E+03, &
      6.0829515E-01, 3.22316E+01, 9.7523E+00, 4.4459E+03, &
      6.1133663E-01, 3.22588E+01, 9.7404E+00, 4.4184E+03, &
      6.1439331E-01, 3.22855E+01, 9.7280E+00, 4.3909E+03, &
      6.1746528E-01, 3.23114E+01, 9.7153E+00, 4.3634E+03, &
      6.2055260E-01, 3.23367E+01, 9.7021E+00, 4.3359E+03, &
      6.2365537E-01, 3.23613E+01, 9.6886E+00, 4.3084E+03, &
      6.2677364E-01, 3.23853E+01, 9.6747E+00, 4.2808E+03, &
      6.2990751E-01, 3.24085E+01, 9.6605E+00, 4.2533E+03, &
      6.3305705E-01, 3.24311E+01, 9.6459E+00, 4.2258E+03, &
      6.3622234E-01, 3.24529E+01, 9.6309E+00, 4.1983E+03, &
      6.3940345E-01, 3.24740E+01, 9.6156E+00, 4.1709E+03, &
      6.4260046E-01, 3.24943E+01, 9.5999E+00, 4.1434E+03, &
      6.4581347E-01, 3.25138E+01, 9.5839E+00, 4.1160E+03, &
      6.4904253E-01, 3.25326E+01, 9.5676E+00, 4.0886E+03, &
      6.5228775E-01, 3.25506E+01, 9.5510E+00, 4.0613E+03, &
      6.5554919E-01, 3.25678E+01, 9.5340E+00, 4.0340E+03, &
      6.5882693E-01, 3.25841E+01, 9.5168E+00, 4.0067E+03, &
      6.6212107E-01, 3.25997E+01, 9.4992E+00, 3.9795E+03, &
      6.6543167E-01, 3.26143E+01, 9.4813E+00, 3.9523E+03, &
      6.6875883E-01, 3.26281E+01, 9.4632E+00, 3.9252E+03, &
      6.7210262E-01, 3.26411E+01, 9.4448E+00, 3.8981E+03, &
      6.7546314E-01, 3.26531E+01, 9.4261E+00, 3.8711E+03, &
      6.7884045E-01, 3.26643E+01, 9.4071E+00, 3.8442E+03, &
      6.8223466E-01, 3.26745E+01, 9.3879E+00, 3.8173E+03, &
      6.8564583E-01, 3.26837E+01, 9.3684E+00, 3.7905E+03, &
      6.8907406E-01, 3.26920E+01, 9.3486E+00, 3.7637E+03, &
      6.9251943E-01, 3.26994E+01, 9.3286E+00, 3.7371E+03, &
      6.9598202E-01, 3.27057E+01, 9.3084E+00, 3.7105E+03, &
      6.9946194E-01, 3.27111E+01, 9.2879E+00, 3.6840E+03, &
      7.0295924E-01, 3.27154E+01, 9.2673E+00, 3.6576E+03, &
      7.0647404E-01, 3.27187E+01, 9.2464E+00, 3.6312E+03, &
      7.1000641E-01, 3.27209E+01, 9.2252E+00, 3.6050E+03, &
      7.1355644E-01, 3.27221E+01, 9.2039E+00, 3.5788E+03, &
      7.1712423E-01, 3.27221E+01, 9.1824E+00, 3.5528E+03, &
      7.2070985E-01, 3.27211E+01, 9.1607E+00, 3.5268E+03, &
      7.2431340E-01, 3.27189E+01, 9.1388E+00, 3.5009E+03, &
      7.2793496E-01, 3.27155E+01, 9.1167E+00, 3.4752E+03, &
      7.3157464E-01, 3.27110E+01, 9.0944E+00, 3.4495E+03, &
      7.3523251E-01, 3.27052E+01, 9.0719E+00, 3.4239E+03, &
      7.3890867E-01, 3.26983E+01, 9.0493E+00, 3.3985E+03, &
      7.4260322E-01, 3.26901E+01, 9.0265E+00, 3.3731E+03, &
      7.4631623E-01, 3.26806E+01, 9.0036E+00, 3.3479E+03, &
      7.5004781E-01, 3.26698E+01, 8.9805E+00, 3.3228E+03, &
      7.5379805E-01, 3.26577E+01, 8.9572E+00, 3.2978E+03, &
      7.5756704E-01, 3.26442E+01, 8.9338E+00, 3.2729E+03, &
      7.6135488E-01, 3.26294E+01, 8.9103E+00, 3.2481E+03, &
      7.6516165E-01, 3.26132E+01, 8.8867E+00, 3.2234E+03, &
      7.6898746E-01, 3.25955E+01, 8.8629E+00, 3.1989E+03, &
      7.7283240E-01, 3.25805E+01, 8.8390E+00, 3.1744E+03, &
      7.7669656E-01, 3.25599E+01, 8.8150E+00, 3.1501E+03, &
      7.8058004E-01, 3.25378E+01, 8.7908E+00, 3.1260E+03, &
      7.8448294E-01, 3.25141E+01, 8.7666E+00, 3.1019E+03, &
      7.8840536E-01, 3.24887E+01, 8.7422E+00, 3.0780E+03, &
      7.9234738E-01, 3.24618E+01, 8.7178E+00, 3.0542E+03, &
      7.9630912E-01, 3.24331E+01, 8.6932E+00, 3.0305E+03, &
      8.0029067E-01, 3.24028E+01, 8.6686E+00, 3.0070E+03, &
      8.0429212E-01, 3.23706E+01, 8.6439E+00, 2.9835E+03, &
      8.0831358E-01, 3.23367E+01, 8.6191E+00, 2.9603E+03, &
      8.1235515E-01, 3.23009E+01, 8.5942E+00, 2.9371E+03, &
      8.1641693E-01, 3.22632E+01, 8.5692E+00, 2.9141E+03, &
      8.2049901E-01, 3.22235E+01, 8.5442E+00, 2.8912E+03, &
      8.2460150E-01, 3.21818E+01, 8.5191E+00, 2.8684E+03, &
      8.2872451E-01, 3.21380E+01, 8.4940E+00, 2.8458E+03, &
      8.3286813E-01, 3.20922E+01, 8.4687E+00, 2.8233E+03, &
      8.3703248E-01, 3.20441E+01, 8.4435E+00, 2.8010E+03, &
      8.4121764E-01, 3.19938E+01, 8.4182E+00, 2.7788E+03, &
      8.4542373E-01, 3.19411E+01, 8.3928E+00, 2.7567E+03, &
      8.4965084E-01, 3.18861E+01, 8.3674E+00, 2.7348E+03, &
      8.5389910E-01, 3.18295E+01, 8.3419E+00, 2.7130E+03, &
      8.5816859E-01, 3.17695E+01, 8.3164E+00, 2.6913E+03, &
      8.6245944E-01, 3.17068E+01, 8.2909E+00, 2.6698E+03, &
      8.6677173E-01, 3.16414E+01, 8.2653E+00, 2.6484E+03, &
      8.7110559E-01, 3.15732E+01, 8.2398E+00, 2.6271E+03, &
      8.7546112E-01, 3.15021E+01, 8.2141E+00, 2.6060E+03, &
      8.7983843E-01, 3.14280E+01, 8.1885E+00, 2.5850E+03, &
      8.8423762E-01, 3.13507E+01, 8.1629E+00, 2.5642E+03, &
      8.8865881E-01, 3.12703E+01, 8.1372E+00, 2.5435E+03, &
      8.9310210E-01, 3.11864E+01, 8.1115E+00, 2.5229E+03, &
      8.9756761E-01, 3.10991E+01, 8.0858E+00, 2.5025E+03, &
      9.0205545E-01, 3.10082E+01, 8.0601E+00, 2.4822E+03, &
      9.0656573E-01, 3.09135E+01, 8.0344E+00, 2.4621E+03, &
      9.1109856E-01, 3.08150E+01, 8.0087E+00, 2.4421E+03, &
      9.1565405E-01, 3.07123E+01, 7.9830E+00, 2.4222E+03, &
      9.2023232E-01, 3.06054E+01, 7.9573E+00, 2.4025E+03, &
      9.2483348E-01, 3.04941E+01, 7.9316E+00, 2.3829E+03, &
      9.2945765E-01, 3.03783E+01, 7.9059E+00, 2.3634E+03, &
      9.3410494E-01, 3.02576E+01, 7.8802E+00, 2.3441E+03, &
      9.3877546E-01, 3.01320E+01, 7.8545E+00, 2.3249E+03, &
      9.4346934E-01, 3.00011E+01, 7.8289E+00, 2.3059E+03, &
      9.4818668E-01, 2.98649E+01, 7.8032E+00, 2.2870E+03, &
      9.5292762E-01, 2.97231E+01, 7.7776E+00, 2.2682E+03, &
      9.5769226E-01, 2.95754E+01, 7.7520E+00, 2.2496E+03, &
      9.6248072E-01, 2.94218E+01, 7.7264E+00, 2.2311E+03, &
      9.6729312E-01, 2.92620E+01, 7.7009E+00, 2.2127E+03, &
      9.7212959E-01, 2.90959E+01, 7.6753E+00, 2.1945E+03, &
      9.7699023E-01, 2.89235E+01, 7.6498E+00, 2.1764E+03, &
      9.8187519E-01, 2.87450E+01, 7.6243E+00, 2.1584E+03, &
      9.8678456E-01, 2.85607E+01, 7.5989E+00, 2.1406E+03, &
      9.9171848E-01, 2.83715E+01, 7.5735E+00, 2.1229E+03, &
      9.9667708E-01, 2.81790E+01, 7.5481E+00, 2.1054E+03, &
      1.0016605E+00, 2.79686E+01, 7.5180E+00, 2.0866E+03, &
      1.0066688E+00, 2.77137E+01, 7.4786E+00, 2.0655E+03, &
      1.0117021E+00, 2.74438E+01, 7.4394E+00, 2.0445E+03, &
      1.0167606E+00, 2.71576E+01, 7.4004E+00, 2.0238E+03, &
      1.0218444E+00, 2.68535E+01, 7.3617E+00, 2.0033E+03, &
      1.0269536E+00, 2.65299E+01, 7.3230E+00, 1.9829E+03, &
      1.0320884E+00, 2.61848E+01, 7.2827E+00, 1.9623E+03, &
      1.0372489E+00, 2.58156E+01, 7.2427E+00, 1.9419E+03, &
      1.0424351E+00, 2.54197E+01, 7.2029E+00, 1.9217E+03, &
      1.0476473E+00, 2.49940E+01, 7.1633E+00, 1.9018E+03, &
      1.0528855E+00, 2.45345E+01, 7.1241E+00, 1.8821E+03, &
      1.0581499E+00, 2.40367E+01, 7.0850E+00, 1.8625E+03, &
      1.0634407E+00, 2.34948E+01, 7.0462E+00, 1.8432E+03, &
      1.0687579E+00, 2.29019E+01, 7.0077E+00, 1.8241E+03, &
      1.0741017E+00, 2.22490E+01, 6.9694E+00, 1.8052E+03, &
      1.0794722E+00, 2.15245E+01, 6.9313E+00, 1.7865E+03, &
      1.0848695E+00, 2.07134E+01, 6.8935E+00, 1.7680E+03, &
      1.0902939E+00, 1.97944E+01, 6.8559E+00, 1.7497E+03, &
      1.0957454E+00, 1.87378E+01, 6.8185E+00, 1.7316E+03, &
      1.1012241E+00, 1.74994E+01, 6.7814E+00, 1.7137E+03, &
      1.1067302E+00, 1.60081E+01, 6.7445E+00, 1.6960E+03, &
      1.1122639E+00, 1.41386E+01, 6.7078E+00, 1.6785E+03, &
      1.1178252E+00, 1.16332E+01, 6.6714E+00, 1.6612E+03, &
      1.1234143E+00, 7.79666E+00, 6.6352E+00, 1.6441E+03, &
      1.1290314E+00, -1.32844E+00, 6.5992E+00, 1.6271E+03, &
      1.1307711E+00, -1.83139E+01, 6.5881E+00, 1.6219E+03, &
      1.1310289E+00, -1.86591E+01, 2.6407E+01, 6.4740E+03, &
      1.1346765E+00, 2.34454E+00, 2.6277E+01, 6.4213E+03, &
      1.1403499E+00, 7.40466E+00, 2.6075E+01, 6.3405E+03, &
      1.1460517E+00, 9.28125E+00, 2.5875E+01, 6.2607E+03, &
      1.1517819E+00, 9.48813E+00, 2.5677E+01, 6.1819E+03, &
      1.1575408E+00, 6.90042E+00, 2.5480E+01, 6.1041E+03, &
      1.1603841E+00, -3.30656E+00, 2.5383E+01, 6.0662E+03, &
      1.1608158E+00, -3.40633E+00, 3.8394E+01, 9.1676E+03, &
      1.1633285E+00, 7.75267E+00, 3.8263E+01, 9.1167E+03, &
      1.1691452E+00, 1.37680E+01, 3.7964E+01, 9.0005E+03, &
      1.1749909E+00, 1.70308E+01, 3.7667E+01, 8.8858E+03, &
      1.1808659E+00, 1.94196E+01, 3.7373E+01, 8.7726E+03, &
      1.1867702E+00, 2.13458E+01, 3.7080E+01, 8.6609E+03, &
      1.1927040E+00, 2.29755E+01, 3.6791E+01, 8.5506E+03, &
      1.1986676E+00, 2.43950E+01, 3.6503E+01, 8.4417E+03, &
      1.2046609E+00, 2.56554E+01, 3.6218E+01, 8.3342E+03, &
      1.2106842E+00, 2.67903E+01, 3.5935E+01, 8.2281E+03, &
      1.2167376E+00, 2.78230E+01, 3.5655E+01, 8.1234E+03, &
      1.2228213E+00, 2.87702E+01, 3.5376E+01, 8.0200E+03, &
      1.2289354E+00, 2.96448E+01, 3.5100E+01, 7.9179E+03, &
      1.2350801E+00, 3.04566E+01, 3.4827E+01, 7.8172E+03, &
      1.2412555E+00, 3.12134E+01, 3.4555E+01, 7.7178E+03, &
      1.2474618E+00, 3.19215E+01, 3.4286E+01, 7.6196E+03, &
      1.2536991E+00, 3.25862E+01, 3.4018E+01, 7.5227E+03, &
      1.2599676E+00, 3.32116E+01, 3.3753E+01, 7.4271E+03, &
      1.2662674E+00, 3.38014E+01, 3.3490E+01, 7.3327E+03, &
      1.2725988E+00, 3.43586E+01, 3.3229E+01, 7.2395E+03, &
      1.2789618E+00, 3.48859E+01, 3.2970E+01, 7.1475E+03, &
      1.2853566E+00, 3.53854E+01, 3.2714E+01, 7.0567E+03, &
      1.2917833E+00, 3.58591E+01, 3.2459E+01, 6.9670E+03, &
      1.2982423E+00, 3.63087E+01, 3.2207E+01, 6.8785E+03, &
      1.3047335E+00, 3.67356E+01, 3.1956E+01, 6.7912E+03, &
      1.3112571E+00, 3.71411E+01, 3.1707E+01, 6.7049E+03, &
      1.3178134E+00, 3.75262E+01, 3.1461E+01, 6.6198E+03, &
      1.3244025E+00, 3.78918E+01, 3.1216E+01, 6.5358E+03, &
      1.3310245E+00, 3.82389E+01, 3.0973E+01, 6.4527E+03, &
      1.3376796E+00, 3.85679E+01, 3.0732E+01, 6.3707E+03, &
      1.3443680E+00, 3.88795E+01, 3.0492E+01, 6.2898E+03, &
      1.3510899E+00, 3.91740E+01, 3.0255E+01, 6.2099E+03, &
      1.3578453E+00, 3.94517E+01, 3.0020E+01, 6.1310E+03, &
      1.3646345E+00, 3.97127E+01, 2.9786E+01, 6.0532E+03, &
      1.3714577E+00, 3.99571E+01, 2.9554E+01, 5.9763E+03, &
      1.3783150E+00, 4.01847E+01, 2.9324E+01, 5.9005E+03, &
      1.3852066E+00, 4.03952E+01, 2.9097E+01, 5.8256E+03, &
      1.3921326E+00, 4.05878E+01, 2.8870E+01, 5.7517E+03, &
      1.3990933E+00, 4.07619E+01, 2.8646E+01, 5.6787E+03, &
      1.4060887E+00, 4.09159E+01, 2.8424E+01, 5.6067E+03, &
      1.4131192E+00, 4.10482E+01, 2.8203E+01, 5.5356E+03, &
      1.4201848E+00, 4.11560E+01, 2.7984E+01, 5.4655E+03, &
      1.4272857E+00, 4.12357E+01, 2.7767E+01, 5.3962E+03, &
      1.4344221E+00, 4.12815E+01, 2.7551E+01, 5.3278E+03, &
      1.4415942E+00, 4.12848E+01, 2.7338E+01, 5.2603E+03, &
      1.4488022E+00, 4.12313E+01, 2.7125E+01, 5.1936E+03, &
      1.4560462E+00, 4.10948E+01, 2.6915E+01, 5.1278E+03, &
      1.4633265E+00, 4.08209E+01, 2.6706E+01, 5.0628E+03, &
      1.4706431E+00, 4.02600E+01, 2.6499E+01, 4.9986E+03, &
      1.4779963E+00, 3.85729E+01, 2.6295E+01, 4.9356E+03, &
      1.4794437E+00, 3.74597E+01, 2.6255E+01, 4.9233E+03, &
      1.4817563E+00, 3.74574E+01, 3.0678E+01, 5.7422E+03, &
      1.4853863E+00, 3.96033E+01, 3.0555E+01, 5.7053E+03, &
      1.4928132E+00, 4.11518E+01, 3.0306E+01, 5.6307E+03, &
      1.5002773E+00, 4.20233E+01, 3.0059E+01, 5.5571E+03, &
      1.5077787E+00, 4.26591E+01, 2.9814E+01, 5.4846E+03, &
      1.5153176E+00, 4.31692E+01, 2.9571E+01, 5.4130E+03, &
      1.5228942E+00, 4.35979E+01, 2.9331E+01, 5.3424E+03, &
      1.5305086E+00, 4.39672E+01, 2.9093E+01, 5.2728E+03, &
      1.5381612E+00, 4.42891E+01, 2.8856E+01, 5.2041E+03, &
      1.5458520E+00, 4.45699E+01, 2.8622E+01, 5.1362E+03, &
      1.5535812E+00, 4.48120E+01, 2.8390E+01, 5.0694E+03, &
      1.5613491E+00, 4.50210E+01, 2.8179E+01, 5.0068E+03, &
      1.5691559E+00, 4.52033E+01, 2.7971E+01, 4.9453E+03, &
      1.5770017E+00, 4.53541E+01, 2.7766E+01, 4.8847E+03, &
      1.5848867E+00, 4.54652E+01, 2.7564E+01, 4.8250E+03, &
      1.5928111E+00, 4.55195E+01, 2.7364E+01, 4.7663E+03, &
      1.6007752E+00, 4.54731E+01, 2.7167E+01, 4.7086E+03, &
      1.6087790E+00, 4.51426E+01, 2.6972E+01, 4.6516E+03, &
      1.6125491E+00, 4.44561E+01, 2.6881E+01, 4.6252E+03, &
      1.6152508E+00, 4.45036E+01, 2.8687E+01, 4.9272E+03, &
      1.6168229E+00, 4.50086E+01, 2.8646E+01, 4.9153E+03, &
      1.6249070E+00, 4.60143E+01, 2.8434E+01, 4.8548E+03, &
      1.6330316E+00, 4.65443E+01, 2.8225E+01, 4.7952E+03, &
      1.6411967E+00, 4.69454E+01, 2.8018E+01, 4.7365E+03, &
      1.6494027E+00, 4.72812E+01, 2.7814E+01, 4.6787E+03, &
      1.6576497E+00, 4.75756E+01, 2.7612E+01, 4.6217E+03, &
      1.6659380E+00, 4.78398E+01, 2.7412E+01, 4.5656E+03, &
      1.6742677E+00, 4.80804E+01, 2.7215E+01, 4.5102E+03, &
      1.6826390E+00, 4.83007E+01, 2.7019E+01, 4.4557E+03, &
      1.6910522E+00, 4.85027E+01, 2.6826E+01, 4.4020E+03, &
      1.6995075E+00, 4.86883E+01, 2.6642E+01, 4.3502E+03, &
      1.7080050E+00, 4.88632E+01, 2.6464E+01, 4.2996E+03, &
      1.7165450E+00, 4.90275E+01, 2.6288E+01, 4.2498E+03, &
      1.7251278E+00, 4.91808E+01, 2.6114E+01, 4.2008E+03, &
      1.7337534E+00, 4.93227E+01, 2.5942E+01, 4.1525E+03, &
      1.7424222E+00, 4.94521E+01, 2.5773E+01, 4.1050E+03, &
      1.7511343E+00, 4.95674E+01, 2.5605E+01, 4.0582E+03, &
      1.7598899E+00, 4.96653E+01, 2.5440E+01, 4.0120E+03, &
      1.7686894E+00, 4.97400E+01, 2.5277E+01, 3.9666E+03, &
      1.7775328E+00, 4.97791E+01, 2.5116E+01, 3.9217E+03, &
      1.7864205E+00, 4.97502E+01, 2.4956E+01, 3.8776E+03, &
      1.7953526E+00, 4.95046E+01, 2.4799E+01, 3.8340E+03, &
      1.7973000E+00, 4.93344E+01, 2.4765E+01, 3.8246E+03, &
      1.8027000E+00, 4.94181E+01, 2.5851E+01, 3.9801E+03, &
      1.8043294E+00, 4.96237E+01, 2.5822E+01, 3.9721E+03, &
      1.8133510E+00, 5.01984E+01, 2.5664E+01, 3.9281E+03, &
      1.8224178E+00, 5.05401E+01, 2.5507E+01, 3.8848E+03, &
      1.8315299E+00, 5.08117E+01, 2.5351E+01, 3.8420E+03, &
      1.8406875E+00, 5.10477E+01, 2.5197E+01, 3.7997E+03, &
      1.8498909E+00, 5.12615E+01, 2.5045E+01, 3.7580E+03, &
      1.8591404E+00, 5.14599E+01, 2.4894E+01, 3.7169E+03, &
      1.8684361E+00, 5.16467E+01, 2.4744E+01, 3.6763E+03, &
      1.8777783E+00, 5.18243E+01, 2.4596E+01, 3.6362E+03, &
      1.8871672E+00, 5.19943E+01, 2.4449E+01, 3.5966E+03, &
      1.8966030E+00, 5.21580E+01, 2.4305E+01, 3.5577E+03, &
      1.9060860E+00, 5.23172E+01, 2.4162E+01, 3.5192E+03, &
      1.9156165E+00, 5.24725E+01, 2.4020E+01, 3.4813E+03, &
      1.9251945E+00, 5.26247E+01, 2.3880E+01, 3.4438E+03, &
      1.9348205E+00, 5.27725E+01, 2.3733E+01, 3.4057E+03, &
      1.9444946E+00, 5.29156E+01, 2.3587E+01, 3.3680E+03, &
      1.9542171E+00, 5.30545E+01, 2.3442E+01, 3.3308E+03, &
      1.9639882E+00, 5.31896E+01, 2.3299E+01, 3.2941E+03, &
      1.9738081E+00, 5.33213E+01, 2.3156E+01, 3.2577E+03, &
      1.9836772E+00, 5.34498E+01, 2.3015E+01, 3.2218E+03, &
      1.9935955E+00, 5.35754E+01, 2.2875E+01, 3.1863E+03, &
      2.0035635E+00, 5.36983E+01, 2.2735E+01, 3.1512E+03, &
      2.0135813E+00, 5.38186E+01, 2.2597E+01, 3.1166E+03, &
      2.0236492E+00, 5.39365E+01, 2.2459E+01, 3.0823E+03, &
      2.0337675E+00, 5.40521E+01, 2.2323E+01, 3.0484E+03, &
      2.0439363E+00, 5.41657E+01, 2.2187E+01, 3.0149E+03, &
      2.0541560E+00, 5.42773E+01, 2.2052E+01, 2.9818E+03, &
      2.0644268E+00, 5.43870E+01, 2.1918E+01, 2.9490E+03, &
      2.0747489E+00, 5.44951E+01, 2.1785E+01, 2.9166E+03, &
      2.0851227E+00, 5.46017E+01, 2.1653E+01, 2.8846E+03, &
      2.0955483E+00, 5.47070E+01, 2.1521E+01, 2.8529E+03, &
      2.1060260E+00, 5.48106E+01, 2.1388E+01, 2.8212E+03, &
      2.1165562E+00, 5.49118E+01, 2.1254E+01, 2.7897E+03, &
      2.1271389E+00, 5.50110E+01, 2.1122E+01, 2.7586E+03, &
      2.1377746E+00, 5.51081E+01, 2.0990E+01, 2.7278E+03, &
      2.1484635E+00, 5.52033E+01, 2.0858E+01, 2.6973E+03, &
      2.1592058E+00, 5.52968E+01, 2.0728E+01, 2.6672E+03, &
      2.1700018E+00, 5.53885E+01, 2.0598E+01, 2.6374E+03, &
      2.1808519E+00, 5.54787E+01, 2.0469E+01, 2.6080E+03, &
      2.1917561E+00, 5.55672E+01, 2.0341E+01, 2.5788E+03, &
      2.2027149E+00, 5.56543E+01, 2.0213E+01, 2.5500E+03, &
      2.2137285E+00, 5.57400E+01, 2.0086E+01, 2.5215E+03, &
      2.2247971E+00, 5.58243E+01, 1.9960E+01, 2.4933E+03, &
      2.2359211E+00, 5.59072E+01, 1.9835E+01, 2.4654E+03, &
      2.2471007E+00, 5.59890E+01, 1.9711E+01, 2.4378E+03, &
      2.2583362E+00, 5.60695E+01, 1.9587E+01, 2.4106E+03, &
      2.2696279E+00, 5.63690E+01, 1.9460E+01, 2.3831E+03, &
      2.2809760E+00, 5.64470E+01, 1.9332E+01, 2.3558E+03, &
      2.2923809E+00, 5.65233E+01, 1.9205E+01, 2.3288E+03, &
      2.3038428E+00, 5.65981E+01, 1.9079E+01, 2.3020E+03, &
      2.3153620E+00, 5.66713E+01, 1.8953E+01, 2.2756E+03, &
      2.3269388E+00, 5.68928E+01, 1.8827E+01, 2.2493E+03, &
      2.3385735E+00, 5.69633E+01, 1.8700E+01, 2.2231E+03, &
      2.3502664E+00, 5.70319E+01, 1.8573E+01, 2.1970E+03, &
      2.3620177E+00, 5.70988E+01, 1.8446E+01, 2.1713E+03, &
      2.3738278E+00, 5.71639E+01, 1.8320E+01, 2.1459E+03, &
      2.3856970E+00, 5.72274E+01, 1.8196E+01, 2.1208E+03, &
      2.3976254E+00, 5.72893E+01, 1.8072E+01, 2.0959E+03, &
      2.4096136E+00, 5.73497E+01, 1.7948E+01, 2.0714E+03, &
      2.4216616E+00, 5.74087E+01, 1.7826E+01, 2.0471E+03, &
      2.4337699E+00, 5.74662E+01, 1.7705E+01, 2.0232E+03, &
      2.4459388E+00, 5.75225E+01, 1.7584E+01, 1.9995E+03, &
      2.4581685E+00, 5.75774E+01, 1.7464E+01, 1.9761E+03, &
      2.4704593E+00, 5.76311E+01, 1.7345E+01, 1.9529E+03, &
      2.4828116E+00, 5.76835E+01, 1.7227E+01, 1.9301E+03, &
      2.4952257E+00, 5.77348E+01, 1.7110E+01, 1.9075E+03, &
      2.5077018E+00, 5.77851E+01, 1.6992E+01, 1.8850E+03, &
      2.5202403E+00, 5.78341E+01, 1.6874E+01, 1.8627E+03, &
      2.5328415E+00, 5.78820E+01, 1.6756E+01, 1.8406E+03, &
      2.5455057E+00, 5.79287E+01, 1.6639E+01, 1.8188E+03, &
      2.5582333E+00, 5.79743E+01, 1.6524E+01, 1.7972E+03, &
      2.5710244E+00, 5.80188E+01, 1.6409E+01, 1.7759E+03, &
      2.5838796E+00, 5.80623E+01, 1.6295E+01, 1.7549E+03, &
      2.5967990E+00, 5.81048E+01, 1.6182E+01, 1.7341E+03, &
      2.6097829E+00, 5.81463E+01, 1.6069E+01, 1.7136E+03, &
      2.6228319E+00, 5.81869E+01, 1.5958E+01, 1.6934E+03, &
      2.6359460E+00, 5.82266E+01, 1.5847E+01, 1.6733E+03, &
      2.6491257E+00, 5.82654E+01, 1.5737E+01, 1.6536E+03, &
      2.6623714E+00, 5.83033E+01, 1.5629E+01, 1.6341E+03, &
      2.6756832E+00, 5.83405E+01, 1.5521E+01, 1.6148E+03, &
      2.6890617E+00, 5.83768E+01, 1.5413E+01, 1.5958E+03, &
      2.7025070E+00, 5.84125E+01, 1.5307E+01, 1.5770E+03, &
      2.7160195E+00, 5.84474E+01, 1.5202E+01, 1.5584E+03, &
      2.7295996E+00, 5.84816E+01, 1.5097E+01, 1.5400E+03, &
      2.7432476E+00, 5.85153E+01, 1.4993E+01, 1.5219E+03, &
      2.7569638E+00, 5.85483E+01, 1.4890E+01, 1.5041E+03, &
      2.7707486E+00, 5.85807E+01, 1.4788E+01, 1.4864E+03, &
      2.7846024E+00, 5.86127E+01, 1.4687E+01, 1.4690E+03, &
      2.7985254E+00, 5.86441E+01, 1.4587E+01, 1.4517E+03, &
      2.8125180E+00, 5.86752E+01, 1.4487E+01, 1.4347E+03, &
      2.8265806E+00, 5.87058E+01, 1.4388E+01, 1.4180E+03, &
      2.8407135E+00, 5.87361E+01, 1.4290E+01, 1.4014E+03, &
      2.8549171E+00, 5.87662E+01, 1.4193E+01, 1.3850E+03, &
      2.8691917E+00, 5.87960E+01, 1.4097E+01, 1.3688E+03, &
      2.8835376E+00, 5.88257E+01, 1.4001E+01, 1.3529E+03, &
      2.8979553E+00, 5.88553E+01, 1.3906E+01, 1.3371E+03, &
      2.9124451E+00, 5.88850E+01, 1.3812E+01, 1.3215E+03, &
      2.9270073E+00, 5.89148E+01, 1.3719E+01, 1.3062E+03, &
      2.9416424E+00, 5.89450E+01, 1.3626E+01, 1.2910E+03, &
      2.9563506E+00, 5.89756E+01, 1.3535E+01, 1.2760E+03, &
      2.9711323E+00, 5.91179E+01, 1.3443E+01, 1.2611E+03, &
      2.9859880E+00, 5.91515E+01, 1.3351E+01, 1.2463E+03, &
      3.0009179E+00, 5.91880E+01, 1.3258E+01, 1.2317E+03, &
      3.0159225E+00, 5.92104E+01, 1.3153E+01, 1.2158E+03, &
      3.0310021E+00, 5.92313E+01, 1.3048E+01, 1.2003E+03, &
      3.0461571E+00, 5.92511E+01, 1.2944E+01, 1.1849E+03, &
      3.0613879E+00, 5.92696E+01, 1.2841E+01, 1.1697E+03, &
      3.0766949E+00, 5.92870E+01, 1.2739E+01, 1.1547E+03, &
      3.0920783E+00, 5.93033E+01, 1.2638E+01, 1.1400E+03, &
      3.1075387E+00, 5.93186E+01, 1.2538E+01, 1.1254E+03, &
      3.1230764E+00, 5.93329E+01, 1.2439E+01, 1.1110E+03, &
      3.1386918E+00, 5.93464E+01, 1.2341E+01, 1.0969E+03, &
      3.1543853E+00, 5.93589E+01, 1.2243E+01, 1.0829E+03, &
      3.1701572E+00, 5.93707E+01, 1.2147E+01, 1.0691E+03, &
      3.1860080E+00, 5.93816E+01, 1.2051E+01, 1.0555E+03, &
      3.2019380E+00, 5.93918E+01, 1.1957E+01, 1.0421E+03, &
      3.2179477E+00, 5.94012E+01, 1.1863E+01, 1.0288E+03, &
      3.2340374E+00, 5.94577E+01, 1.1769E+01, 1.0157E+03, &
      3.2502076E+00, 5.94661E+01, 1.1675E+01, 1.0027E+03, &
      3.2664587E+00, 5.94737E+01, 1.1582E+01, 9.8979E+02, &
      3.2827910E+00, 5.94804E+01, 1.1489E+01, 9.7710E+02, &
      3.2992049E+00, 5.94863E+01, 1.1398E+01, 9.6458E+02, &
      3.3157009E+00, 5.94915E+01, 1.1307E+01, 9.5223E+02, &
      3.3322794E+00, 5.94959E+01, 1.1217E+01, 9.4005E+02, &
      3.3489408E+00, 5.94997E+01, 1.1128E+01, 9.2803E+02, &
      3.3656856E+00, 5.95027E+01, 1.1040E+01, 9.1617E+02, &
      3.3825140E+00, 5.95051E+01, 1.0952E+01, 9.0448E+02, &
      3.3994265E+00, 5.95069E+01, 1.0866E+01, 8.9294E+02, &
      3.4164237E+00, 5.95081E+01, 1.0780E+01, 8.8155E+02, &
      3.4335058E+00, 5.95087E+01, 1.0695E+01, 8.7032E+02, &
      3.4506733E+00, 5.95087E+01, 1.0610E+01, 8.5924E+02, &
      3.4679267E+00, 5.95082E+01, 1.0527E+01, 8.4831E+02, &
      3.4852663E+00, 5.95071E+01, 1.0444E+01, 8.3753E+02, &
      3.5026927E+00, 5.95055E+01, 1.0362E+01, 8.2689E+02, &
      3.5202061E+00, 5.95034E+01, 1.0280E+01, 8.1639E+02, &
      3.5378072E+00, 5.95008E+01, 1.0200E+01, 8.0604E+02, &
      3.5554962E+00, 5.94978E+01, 1.0120E+01, 7.9582E+02, &
      3.5732737E+00, 5.94942E+01, 1.0040E+01, 7.8574E+02, &
      3.5911400E+00, 5.94903E+01, 9.9618E+00, 7.7580E+02, &
      3.6090957E+00, 5.95065E+01, 9.8836E+00, 7.6596E+02, &
      3.6271412E+00, 5.95018E+01, 9.8057E+00, 7.5622E+02, &
      3.6452769E+00, 5.94966E+01, 9.7285E+00, 7.4661E+02, &
      3.6635033E+00, 5.94909E+01, 9.6520E+00, 7.3713E+02, &
      3.6818208E+00, 5.94847E+01, 9.5761E+00, 7.2778E+02, &
      3.7002299E+00, 5.94781E+01, 9.5010E+00, 7.1855E+02, &
      3.7187311E+00, 5.94710E+01, 9.4264E+00, 7.0945E+02, &
      3.7373247E+00, 5.94634E+01, 9.3526E+00, 7.0047E+02, &
      3.7560114E+00, 5.94555E+01, 9.2794E+00, 6.9160E+02, &
      3.7747914E+00, 5.94471E+01, 9.2068E+00, 6.8286E+02, &
      3.7936654E+00, 5.94383E+01, 9.1349E+00, 6.7423E+02, &
      3.8126337E+00, 5.94291E+01, 9.0636E+00, 6.6572E+02, &
      3.8316969E+00, 5.94195E+01, 8.9929E+00, 6.5732E+02, &
      3.8508554E+00, 5.94095E+01, 8.9229E+00, 6.4903E+02, &
      3.8701096E+00, 5.93991E+01, 8.8535E+00, 6.4085E+02, &
      3.8894602E+00, 5.93884E+01, 8.7847E+00, 6.3278E+02, &
      3.9089075E+00, 5.93773E+01, 8.7165E+00, 6.2482E+02, &
      3.9284520E+00, 5.93658E+01, 8.6488E+00, 6.1696E+02, &
      3.9480943E+00, 5.93539E+01, 8.5818E+00, 6.0921E+02, &
      3.9678347E+00, 5.93417E+01, 8.5154E+00, 6.0156E+02, &
      3.9876739E+00, 5.93292E+01, 8.4495E+00, 5.9401E+02, &
      4.055024E+00, 5.96358E+01, 8.2325E+00, 5.6942E+02, &
      4.334821E+00, 5.93705E+01, 7.4286E+00, 4.8151E+02, &
      4.633924E+00, 5.90558E+01, 6.7125E+00, 4.0783E+02, &
      4.953664E+00, 5.86852E+01, 6.0676E+00, 3.4563E+02, &
      5.295467E+00, 5.82226E+01, 5.4616E+00, 2.9177E+02, &
      5.660855E+00, 5.76151E+01, 4.9205E+00, 2.4659E+02, &
      6.051453E+00, 5.67667E+01, 4.4412E+00, 2.0885E+02, &
      6.469004E+00, 5.53693E+01, 4.0134E+00, 1.7715E+02, &
      6.837362E+00, 5.24608E+01, 3.6797E+00, 1.5415E+02, &
      6.915365E+00, 5.06522E+01, 3.6152E+00, 1.4985E+02, &
      6.942016E+00, 4.94006E+01, 3.5936E+00, 1.4841E+02, &
      6.969923E+00, 4.58346E+01, 3.5713E+00, 1.4694E+02, &
      7.011785E+00, 4.93893E+01, 1.0485E+01, 4.1912E+02, &
      7.116438E+00, 5.24556E+01, 1.0149E+01, 3.9989E+02, &
      7.392525E+00, 5.41115E+01, 9.3669E+00, 3.5569E+02, &
      7.464758E+00, 5.40325E+01, 9.2026E+00, 3.4615E+02, &
      7.579014E+00, 5.29494E+01, 8.9580E+00, 3.3201E+02, &
      7.609483E+00, 5.12685E+01, 8.8957E+00, 3.2842E+02, &
      7.655185E+00, 5.31756E+01, 1.2274E+01, 4.4866E+02, &
      7.769442E+00, 5.49778E+01, 1.1938E+01, 4.3010E+02, &
      7.890960E+00, 5.56937E+01, 1.1599E+01, 4.1160E+02, &
      7.902609E+00, 5.57303E+01, 1.1568E+01, 4.0990E+02, &
      8.011740E+00, 5.56299E+01, 1.1284E+01, 3.9450E+02, &
      8.043948E+00, 5.48926E+01, 1.1207E+01, 3.9030E+02, &
      8.092260E+00, 5.59907E+01, 1.2844E+01, 4.4397E+02, &
      8.213040E+00, 5.72565E+01, 1.2562E+01, 4.2794E+02, &
      8.447890E+00, 5.85224E+01, 1.2052E+01, 3.9934E+02, &
      9.030794E+00, 6.03515E+01, 1.0886E+01, 3.3783E+02, &
      9.653919E+00, 6.14328E+01, 9.7453E+00, 2.8333E+02, &
      1.032004E+01, 6.21312E+01, 8.7220E+00, 2.3760E+02, &
      1.103212E+01, 6.25970E+01, 7.7991E+00, 1.9909E+02, &
      1.179334E+01, 6.29021E+01, 6.9771E+00, 1.6694E+02, &
      1.260708E+01, 6.31004E+01, 6.2471E+00, 1.4012E+02, &
      1.347697E+01, 6.32258E+01, 5.5984E+00, 1.1773E+02, &
      1.440688E+01, 6.33696E+01, 5.0106E+00, 9.8822E+01, &
      1.540095E+01, 6.34331E+01, 4.4768E+00, 8.2826E+01, &
      1.646362E+01, 6.34567E+01, 3.9915E+00, 6.9296E+01, &
      1.759961E+01, 6.34161E+01, 3.5544E+00, 5.7925E+01, &
      1.881398E+01, 6.33537E+01, 3.1680E+00, 4.8475E+01, &
      2.011215E+01, 6.32776E+01, 2.8261E+00, 4.0616E+01, &
      2.149988E+01, 6.31925E+01, 2.5233E+00, 3.4071E+01, &
      2.298338E+01, 6.31018E+01, 2.2550E+00, 2.8616E+01, &
      2.456923E+01, 6.30078E+01, 2.0169E+00, 2.4064E+01, &
      2.626450E+01, 6.29135E+01, 1.8054E+00, 2.0258E+01, &
      2.807676E+01, 6.28223E+01, 1.6051E+00, 1.6955E+01, &
      3.001405E+01, 6.27126E+01, 1.4248E+00, 1.4176E+01, &
      3.208502E+01, 6.25915E+01, 1.2627E+00, 1.1844E+01, &
      3.429889E+01, 6.24558E+01, 1.1186E+00, 9.8994E+00, &
      3.666551E+01, 6.22983E+01, 9.8790E-01, 8.2570E+00, &
      3.919543E+01, 6.21028E+01, 8.7194E-01, 6.8902E+00, &
      4.189992E+01, 6.18370E+01, 7.6975E-01, 5.7567E+00, &
      4.479101E+01, 6.13990E+01, 6.8007E-01, 4.8187E+00, &
      4.754862E+01, 6.03280E+01, 6.0908E-01, 4.1161E+00, &
      4.788159E+01, 5.99783E+01, 6.0131E-01, 4.0414E+00, &
      4.827641E+01, 5.91554E+01, 5.9230E-01, 3.9554E+00, &
      4.847048E+01, 5.77507E+01, 5.8795E-01, 3.9141E+00, &
      4.876160E+01, 5.91703E+01, 3.3470E+00, 1.9559E+01, &
      4.948938E+01, 6.04533E+01, 3.2710E+00, 1.8844E+01, &
      5.118542E+01, 6.14212E+01, 3.1039E+00, 1.7310E+01, &
      5.471721E+01, 6.22220E+01, 2.7859E+00, 1.4575E+01, &
      5.849270E+01, 6.26336E+01, 2.4966E+00, 1.2257E+01, &
      6.252870E+01, 6.28888E+01, 2.2360E+00, 1.0306E+01, &
      6.684318E+01, 6.30578E+01, 1.9997E+00, 8.6571E+00, &
      7.145536E+01, 6.31711E+01, 1.7878E+00, 7.2731E+00, &
      7.638578E+01, 6.32479E+01, 1.5984E+00, 6.1137E+00, &
      8.165640E+01, 6.33021E+01, 1.4271E+00, 5.1356E+00, &
      8.729069E+01, 6.33337E+01, 1.2703E+00, 4.3052E+00, &
      9.331374E+01, 6.33506E+01, 1.1309E+00, 3.6126E+00, &
      9.975239E+01, 6.33867E+01, 1.0053E+00, 3.0302E+00, &
      1.066353E+02, 6.33832E+01, 8.9153E-01, 2.5393E+00, &
      1.139931E+02, 6.33731E+01, 7.9073E-01, 2.1312E+00, &
      1.218587E+02, 6.33589E+01, 7.0139E-01, 1.7916E+00, &
      1.302669E+02, 6.33420E+01, 6.2220E-01, 1.5091E+00, &
      1.392553E+02, 6.33234E+01, 5.5201E-01, 1.2738E+00, &
      1.488640E+02, 6.33040E+01, 4.8977E-01, 1.0778E+00, &
      1.591356E+02, 6.32843E+01, 4.3458E-01, 9.1439E-01, &
      1.701159E+02, 6.32647E+01, 3.8565E-01, 7.7814E-01, &
      1.818539E+02, 6.32454E+01, 3.4225E-01, 6.6439E-01, &
      1.944018E+02, 6.32268E+01, 3.0375E-01, 5.6937E-01, &
      2.078156E+02, 6.32088E+01, 2.6961E-01, 4.8991E-01, &
      2.221548E+02, 6.31916E+01, 2.4007E-01, 4.2431E-01, &
      2.374835E+02, 6.31759E+01, 2.1392E-01, 3.6932E-01, &
      2.538699E+02, 6.31612E+01, 1.9064E-01, 3.2298E-01, &
      2.713869E+02, 6.31477E+01, 1.6990E-01, 2.8386E-01, &
      2.901126E+02, 6.31351E+01, 1.5143E-01, 2.5077E-01, &
      3.101304E+02, 6.31234E+01, 1.3497E-01, 2.2272E-01, &
      3.315294E+02, 6.31127E+01, 1.2031E-01, 1.9888E-01, &
      3.544049E+02, 6.31028E+01, 1.0724E-01, 1.7856E-01, &
      3.788588E+02, 6.30936E+01, 9.5604E-02, 1.6118E-01, &
      4.050001E+02, 6.30852E+01, 8.5233E-02, 1.4626E-01, &
      4.329451E+02, 6.30775E+01, 7.5990E-02, 1.3339E-01]
   case(64)
      frel = -6.0000E-01
      allocate(nist_tab(3608))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      8.353710E-03, 1.32097E+01, 0.0000E+00, 8.0877E-05, &
      8.481573E-03, 1.31819E+01, 0.0000E+00, 8.7032E-05, &
      8.515670E-03, 1.31665E+01, 0.0000E+00, 8.8730E-05, &
      8.566815E-03, 1.31686E+01, 1.9667E-02, 6.1434E+02, &
      8.694678E-03, 1.31559E+01, 1.9324E-02, 5.9476E+02, &
      9.093814E-03, 1.30748E+01, 1.8323E-02, 5.3918E+02, &
      9.233005E-03, 1.30083E+01, 1.7996E-02, 5.2158E+02, &
      9.270123E-03, 1.29446E+01, 1.7910E-02, 5.1702E+02, &
      9.325799E-03, 1.29986E+01, 1.3389E-01, 3.8419E+03, &
      9.464990E-03, 1.30338E+01, 1.3155E-01, 3.7193E+03, &
      1.069000E-02, 1.29235E+01, 1.1642E-01, 2.9142E+03, &
      1.142761E-02, 1.27999E+01, 1.1490E-01, 2.6906E+03, &
      1.221612E-02, 1.26419E+01, 1.1823E-01, 2.5900E+03, &
      1.305903E-02, 1.24423E+01, 1.2603E-01, 2.5827E+03, &
      1.396010E-02, 1.21876E+01, 1.3838E-01, 2.6525E+03, &
      1.492335E-02, 1.18541E+01, 1.5568E-01, 2.7917E+03, &
      1.595306E-02, 1.13991E+01, 1.7868E-01, 2.9972E+03, &
      1.705382E-02, 1.07332E+01, 2.0836E-01, 3.2694E+03, &
      1.823053E-02, 9.62346E+00, 2.4598E-01, 3.6107E+03, &
      1.948844E-02, 7.04472E+00, 2.9310E-01, 4.0247E+03, &
      1.989400E-02, 4.95758E+00, 3.0984E-01, 4.1678E+03, &
      2.019850E-02, 5.08358E-01, 3.2291E-01, 4.2781E+03, &
      2.027970E-02, -4.87034E+00, 3.2647E-01, 4.3079E+03, &
      2.040150E-02, 7.85012E-01, 1.0947E+01, 1.4359E+05, &
      2.070600E-02, 5.76199E+00, 1.0710E+01, 1.3842E+05, &
      2.083314E-02, 6.77123E+00, 1.0614E+01, 1.3634E+05, &
      2.227063E-02, 1.17709E+01, 9.6389E+00, 1.1582E+05, &
      2.380730E-02, 1.40334E+01, 8.6774E+00, 9.7536E+04, &
      2.545001E-02, 1.53779E+01, 7.6765E+00, 8.0717E+04, &
      2.720606E-02, 1.61513E+01, 6.7379E+00, 6.6274E+04, &
      2.908327E-02, 1.64765E+01, 5.9222E+00, 5.4491E+04, &
      3.109002E-02, 1.66427E+01, 5.2582E+00, 4.5259E+04, &
      3.323523E-02, 1.66450E+01, 4.7528E+00, 3.8268E+04, &
      3.537800E-02, 1.65486E+01, 4.4181E+00, 3.3419E+04, &
      3.552846E-02, 1.65385E+01, 4.4000E+00, 3.3141E+04, &
      3.591950E-02, 1.65057E+01, 4.3559E+00, 3.2451E+04, &
      3.606390E-02, 1.64811E+01, 4.3406E+00, 3.2208E+04, &
      3.628050E-02, 1.64852E+01, 4.3514E+00, 3.2095E+04, &
      3.682200E-02, 1.64682E+01, 4.3032E+00, 3.1273E+04, &
      3.797993E-02, 1.64096E+01, 4.2232E+00, 2.9756E+04, &
      4.060054E-02, 1.62666E+01, 4.1376E+00, 2.7272E+04, &
      4.340198E-02, 1.61333E+01, 4.1554E+00, 2.5621E+04, &
      4.639671E-02, 1.60235E+01, 4.2536E+00, 2.4533E+04, &
      4.959809E-02, 1.59462E+01, 4.4190E+00, 2.3842E+04, &
      5.302035E-02, 1.59079E+01, 4.6376E+00, 2.3407E+04, &
      5.667876E-02, 1.59114E+01, 4.8971E+00, 2.3121E+04, &
      6.058959E-02, 1.59574E+01, 5.1869E+00, 2.2909E+04, &
      6.477028E-02, 1.60625E+01, 5.4977E+00, 2.2714E+04, &
      6.923942E-02, 1.61889E+01, 5.8218E+00, 2.2501E+04, &
      7.401695E-02, 1.63506E+01, 6.1529E+00, 2.2245E+04, &
      7.912411E-02, 1.65431E+01, 6.4858E+00, 2.1935E+04, &
      8.458368E-02, 1.67603E+01, 6.8167E+00, 2.1566E+04, &
      9.041995E-02, 1.69955E+01, 7.1432E+00, 2.1141E+04, &
      9.665893E-02, 1.72392E+01, 7.4631E+00, 2.0662E+04, &
      1.0000000E-01, 2.00918E+01, 7.6231E+00, 2.0400E+04, &
      1.0050000E-01, 2.01097E+01, 7.6464E+00, 2.0361E+04, &
      1.0100250E-01, 2.01276E+01, 7.6697E+00, 2.0321E+04, &
      1.0150751E-01, 2.01454E+01, 7.6929E+00, 2.0281E+04, &
      1.0201505E-01, 2.01631E+01, 7.7161E+00, 2.0241E+04, &
      1.0252513E-01, 2.01807E+01, 7.7392E+00, 2.0201E+04, &
      1.0303775E-01, 2.01981E+01, 7.7623E+00, 2.0160E+04, &
      1.0355294E-01, 2.02154E+01, 7.7853E+00, 2.0119E+04, &
      1.0407070E-01, 2.02325E+01, 7.8083E+00, 2.0078E+04, &
      1.0459106E-01, 2.02494E+01, 7.8312E+00, 2.0037E+04, &
      1.0511401E-01, 2.02662E+01, 7.8541E+00, 1.9996E+04, &
      1.0563958E-01, 2.02827E+01, 7.8770E+00, 1.9954E+04, &
      1.0616778E-01, 2.02991E+01, 7.8998E+00, 1.9912E+04, &
      1.0669862E-01, 2.03151E+01, 7.9225E+00, 1.9870E+04, &
      1.0723211E-01, 2.03310E+01, 7.9452E+00, 1.9828E+04, &
      1.0776827E-01, 2.03465E+01, 7.9679E+00, 1.9786E+04, &
      1.0830712E-01, 2.03624E+01, 7.9905E+00, 1.9743E+04, &
      1.0884865E-01, 2.03773E+01, 8.0131E+00, 1.9701E+04, &
      1.0939289E-01, 2.03919E+01, 8.0356E+00, 1.9658E+04, &
      1.0993986E-01, 2.04062E+01, 8.0581E+00, 1.9615E+04, &
      1.1048956E-01, 2.04200E+01, 8.0805E+00, 1.9571E+04, &
      1.1104201E-01, 2.04334E+01, 8.1029E+00, 1.9528E+04, &
      1.1159722E-01, 2.04464E+01, 8.1253E+00, 1.9484E+04, &
      1.1215520E-01, 2.04589E+01, 8.1476E+00, 1.9441E+04, &
      1.1271598E-01, 2.04708E+01, 8.1699E+00, 1.9397E+04, &
      1.1327956E-01, 2.04822E+01, 8.1921E+00, 1.9353E+04, &
      1.1384596E-01, 2.04930E+01, 8.2143E+00, 1.9309E+04, &
      1.1441519E-01, 2.05032E+01, 8.2364E+00, 1.9264E+04, &
      1.1498726E-01, 2.05126E+01, 8.2585E+00, 1.9220E+04, &
      1.1556220E-01, 2.05214E+01, 8.2806E+00, 1.9176E+04, &
      1.1614001E-01, 2.05293E+01, 8.3026E+00, 1.9131E+04, &
      1.1672071E-01, 2.05363E+01, 8.3246E+00, 1.9086E+04, &
      1.1730431E-01, 2.05425E+01, 8.3465E+00, 1.9041E+04, &
      1.1789083E-01, 2.05476E+01, 8.3684E+00, 1.8996E+04, &
      1.1848029E-01, 2.05516E+01, 8.3902E+00, 1.8951E+04, &
      1.1907269E-01, 2.05545E+01, 8.4120E+00, 1.8906E+04, &
      1.1966805E-01, 2.05561E+01, 8.4338E+00, 1.8860E+04, &
      1.2026639E-01, 2.05563E+01, 8.4556E+00, 1.8815E+04, &
      1.2086772E-01, 2.05551E+01, 8.4773E+00, 1.8769E+04, &
      1.2147206E-01, 2.05521E+01, 8.4989E+00, 1.8724E+04, &
      1.2207942E-01, 2.05474E+01, 8.5205E+00, 1.8678E+04, &
      1.2268982E-01, 2.05408E+01, 8.5421E+00, 1.8632E+04, &
      1.2330327E-01, 2.05320E+01, 8.5637E+00, 1.8586E+04, &
      1.2391979E-01, 2.05209E+01, 8.5852E+00, 1.8540E+04, &
      1.2453939E-01, 2.05071E+01, 8.6067E+00, 1.8494E+04, &
      1.2516208E-01, 2.04905E+01, 8.6282E+00, 1.8448E+04, &
      1.2578789E-01, 2.04707E+01, 8.6496E+00, 1.8402E+04, &
      1.2641683E-01, 2.04473E+01, 8.6710E+00, 1.8356E+04, &
      1.2704892E-01, 2.04200E+01, 8.6923E+00, 1.8309E+04, &
      1.2768416E-01, 2.03882E+01, 8.7137E+00, 1.8263E+04, &
      1.2832258E-01, 2.03513E+01, 8.7349E+00, 1.8216E+04, &
      1.2896419E-01, 2.03087E+01, 8.7562E+00, 1.8170E+04, &
      1.2960902E-01, 2.02596E+01, 8.7775E+00, 1.8123E+04, &
      1.3025706E-01, 2.02028E+01, 8.7987E+00, 1.8077E+04, &
      1.3090835E-01, 2.01373E+01, 8.8199E+00, 1.8030E+04, &
      1.3156289E-01, 2.00615E+01, 8.8410E+00, 1.7984E+04, &
      1.3222070E-01, 1.99734E+01, 8.8621E+00, 1.7937E+04, &
      1.3288181E-01, 1.98708E+01, 8.8832E+00, 1.7890E+04, &
      1.3354621E-01, 1.97505E+01, 8.9043E+00, 1.7843E+04, &
      1.3421395E-01, 1.96081E+01, 8.9254E+00, 1.7797E+04, &
      1.3488502E-01, 1.94381E+01, 8.9464E+00, 1.7750E+04, &
      1.3555944E-01, 1.92323E+01, 8.9674E+00, 1.7703E+04, &
      1.3623724E-01, 1.89787E+01, 8.9884E+00, 1.7656E+04, &
      1.3691842E-01, 1.86585E+01, 9.0094E+00, 1.7609E+04, &
      1.3760302E-01, 1.82403E+01, 9.0304E+00, 1.7563E+04, &
      1.3829103E-01, 1.76653E+01, 9.0513E+00, 1.7516E+04, &
      1.3898249E-01, 1.68045E+01, 9.0722E+00, 1.7469E+04, &
      1.3967740E-01, 1.52719E+01, 9.0931E+00, 1.7422E+04, &
      1.4037579E-01, 9.93257E+00, 9.1140E+00, 1.7375E+04, &
      1.4046403E-01, 6.19564E+00, 9.1166E+00, 1.7369E+04, &
      1.4053596E-01, 6.28437E+00, 1.8785E+01, 3.5771E+04, &
      1.4107766E-01, 1.52536E+01, 1.8453E+01, 3.5004E+04, &
      1.4178305E-01, 1.80396E+01, 1.8041E+01, 3.4052E+04, &
      1.4249197E-01, 1.96242E+01, 1.7649E+01, 3.3147E+04, &
      1.4320443E-01, 2.07340E+01, 1.7277E+01, 3.2285E+04, &
      1.4392045E-01, 2.15809E+01, 1.6922E+01, 3.1465E+04, &
      1.4464005E-01, 2.22580E+01, 1.6585E+01, 3.0685E+04, &
      1.4536325E-01, 2.28151E+01, 1.6265E+01, 2.9943E+04, &
      1.4609007E-01, 2.32827E+01, 1.5960E+01, 2.9236E+04, &
      1.4682052E-01, 2.36805E+01, 1.5670E+01, 2.8563E+04, &
      1.4755462E-01, 2.40225E+01, 1.5395E+01, 2.7922E+04, &
      1.4829239E-01, 2.43187E+01, 1.5134E+01, 2.7311E+04, &
      1.4903386E-01, 2.45768E+01, 1.4886E+01, 2.6730E+04, &
      1.4977903E-01, 2.48027E+01, 1.4650E+01, 2.6175E+04, &
      1.5052792E-01, 2.50010E+01, 1.4426E+01, 2.5647E+04, &
      1.5128056E-01, 2.51754E+01, 1.4214E+01, 2.5144E+04, &
      1.5203696E-01, 2.53290E+01, 1.4012E+01, 2.4664E+04, &
      1.5279715E-01, 2.54641E+01, 1.3821E+01, 2.4206E+04, &
      1.5356113E-01, 2.55827E+01, 1.3639E+01, 2.3770E+04, &
      1.5432894E-01, 2.56862E+01, 1.3467E+01, 2.3353E+04, &
      1.5510058E-01, 2.57757E+01, 1.3305E+01, 2.2957E+04, &
      1.5587609E-01, 2.58529E+01, 1.3155E+01, 2.2586E+04, &
      1.5665547E-01, 2.59207E+01, 1.3018E+01, 2.2239E+04, &
      1.5743875E-01, 2.59812E+01, 1.2892E+01, 2.1913E+04, &
      1.5822594E-01, 2.60358E+01, 1.2776E+01, 2.1608E+04, &
      1.5901707E-01, 2.60858E+01, 1.2669E+01, 2.1321E+04, &
      1.5981215E-01, 2.61321E+01, 1.2571E+01, 2.1052E+04, &
      1.6061121E-01, 2.61755E+01, 1.2477E+01, 2.0790E+04, &
      1.6141427E-01, 2.62165E+01, 1.2388E+01, 2.0539E+04, &
      1.6222134E-01, 2.62555E+01, 1.2306E+01, 2.0302E+04, &
      1.6303245E-01, 2.62929E+01, 1.2230E+01, 2.0076E+04, &
      1.6384761E-01, 2.63291E+01, 1.2160E+01, 1.9862E+04, &
      1.6466685E-01, 2.63644E+01, 1.2096E+01, 1.9658E+04, &
      1.6549018E-01, 2.63991E+01, 1.2036E+01, 1.9464E+04, &
      1.6631763E-01, 2.64335E+01, 1.1981E+01, 1.9279E+04, &
      1.6714922E-01, 2.64677E+01, 1.1931E+01, 1.9102E+04, &
      1.6798497E-01, 2.65020E+01, 1.1884E+01, 1.8932E+04, &
      1.6882489E-01, 2.65364E+01, 1.1841E+01, 1.8770E+04, &
      1.6966902E-01, 2.65712E+01, 1.1802E+01, 1.8615E+04, &
      1.7051736E-01, 2.66064E+01, 1.1765E+01, 1.8465E+04, &
      1.7136995E-01, 2.66421E+01, 1.1732E+01, 1.8322E+04, &
      1.7222680E-01, 2.66785E+01, 1.1702E+01, 1.8184E+04, &
      1.7308793E-01, 2.67156E+01, 1.1674E+01, 1.8050E+04, &
      1.7395337E-01, 2.67535E+01, 1.1649E+01, 1.7922E+04, &
      1.7482314E-01, 2.67928E+01, 1.1601E+01, 1.7759E+04, &
      1.7569726E-01, 2.68320E+01, 1.1542E+01, 1.7580E+04, &
      1.7657574E-01, 2.68704E+01, 1.1484E+01, 1.7405E+04, &
      1.7745862E-01, 2.69080E+01, 1.1428E+01, 1.7235E+04, &
      1.7834591E-01, 2.69451E+01, 1.1374E+01, 1.7067E+04, &
      1.7923764E-01, 2.69814E+01, 1.1321E+01, 1.6904E+04, &
      1.8013383E-01, 2.70172E+01, 1.1270E+01, 1.6744E+04, &
      1.8103450E-01, 2.70524E+01, 1.1220E+01, 1.6586E+04, &
      1.8193967E-01, 2.70870E+01, 1.1171E+01, 1.6432E+04, &
      1.8284937E-01, 2.71211E+01, 1.1124E+01, 1.6281E+04, &
      1.8376362E-01, 2.71546E+01, 1.1077E+01, 1.6133E+04, &
      1.8468244E-01, 2.71877E+01, 1.1032E+01, 1.5987E+04, &
      1.8560585E-01, 2.72202E+01, 1.0988E+01, 1.5843E+04, &
      1.8653388E-01, 2.72522E+01, 1.0944E+01, 1.5702E+04, &
      1.8746655E-01, 2.72838E+01, 1.0902E+01, 1.5564E+04, &
      1.8840388E-01, 2.73149E+01, 1.0860E+01, 1.5427E+04, &
      1.8934590E-01, 2.73455E+01, 1.0820E+01, 1.5293E+04, &
      1.9029263E-01, 2.73757E+01, 1.0780E+01, 1.5160E+04, &
      1.9124409E-01, 2.74054E+01, 1.0740E+01, 1.5030E+04, &
      1.9220031E-01, 2.74346E+01, 1.0702E+01, 1.4902E+04, &
      1.9316131E-01, 2.74634E+01, 1.0664E+01, 1.4775E+04, &
      1.9412712E-01, 2.74917E+01, 1.0626E+01, 1.4650E+04, &
      1.9509776E-01, 2.75195E+01, 1.0590E+01, 1.4527E+04, &
      1.9607325E-01, 2.75469E+01, 1.0554E+01, 1.4405E+04, &
      1.9705361E-01, 2.75739E+01, 1.0518E+01, 1.4285E+04, &
      1.9803888E-01, 2.76004E+01, 1.0483E+01, 1.4167E+04, &
      1.9902907E-01, 2.76264E+01, 1.0449E+01, 1.4050E+04, &
      2.0002422E-01, 2.76520E+01, 1.0414E+01, 1.3935E+04, &
      2.0102434E-01, 2.76771E+01, 1.0381E+01, 1.3821E+04, &
      2.0202946E-01, 2.77018E+01, 1.0348E+01, 1.3708E+04, &
      2.0303961E-01, 2.77259E+01, 1.0315E+01, 1.3597E+04, &
      2.0405481E-01, 2.77497E+01, 1.0283E+01, 1.3487E+04, &
      2.0507508E-01, 2.77729E+01, 1.0251E+01, 1.3379E+04, &
      2.0610046E-01, 2.77956E+01, 1.0220E+01, 1.3271E+04, &
      2.0713096E-01, 2.78179E+01, 1.0189E+01, 1.3165E+04, &
      2.0816661E-01, 2.78397E+01, 1.0158E+01, 1.3061E+04, &
      2.0920745E-01, 2.78610E+01, 1.0128E+01, 1.2957E+04, &
      2.1025348E-01, 2.78817E+01, 1.0098E+01, 1.2855E+04, &
      2.1130475E-01, 2.79020E+01, 1.0069E+01, 1.2753E+04, &
      2.1236128E-01, 2.79218E+01, 1.0040E+01, 1.2653E+04, &
      2.1342308E-01, 2.79410E+01, 1.0011E+01, 1.2554E+04, &
      2.1449020E-01, 2.79597E+01, 9.9825E+00, 1.2456E+04, &
      2.1556265E-01, 2.79779E+01, 9.9544E+00, 1.2359E+04, &
      2.1664046E-01, 2.79956E+01, 9.9266E+00, 1.2264E+04, &
      2.1772366E-01, 2.80127E+01, 9.8992E+00, 1.2169E+04, &
      2.1881228E-01, 2.80293E+01, 9.8721E+00, 1.2075E+04, &
      2.1990634E-01, 2.80452E+01, 9.8453E+00, 1.1983E+04, &
      2.2100588E-01, 2.80606E+01, 9.8188E+00, 1.1891E+04, &
      2.2211090E-01, 2.80755E+01, 9.7927E+00, 1.1800E+04, &
      2.2322146E-01, 2.80897E+01, 9.7668E+00, 1.1711E+04, &
      2.2433757E-01, 2.81033E+01, 9.7413E+00, 1.1622E+04, &
      2.2545925E-01, 2.81163E+01, 9.7161E+00, 1.1534E+04, &
      2.2658655E-01, 2.81286E+01, 9.6911E+00, 1.1447E+04, &
      2.2771948E-01, 2.81403E+01, 9.6665E+00, 1.1362E+04, &
      2.2885808E-01, 2.81513E+01, 9.6422E+00, 1.1277E+04, &
      2.3000237E-01, 2.81617E+01, 9.6183E+00, 1.1193E+04, &
      2.3115238E-01, 2.81713E+01, 9.5946E+00, 1.1110E+04, &
      2.3230814E-01, 2.81802E+01, 9.5712E+00, 1.1027E+04, &
      2.3346969E-01, 2.81883E+01, 9.5481E+00, 1.0946E+04, &
      2.3463703E-01, 2.81956E+01, 9.5253E+00, 1.0866E+04, &
      2.3581022E-01, 2.82021E+01, 9.5029E+00, 1.0786E+04, &
      2.3698927E-01, 2.82078E+01, 9.4807E+00, 1.0708E+04, &
      2.3817422E-01, 2.82126E+01, 9.4589E+00, 1.0630E+04, &
      2.3936509E-01, 2.82164E+01, 9.4373E+00, 1.0553E+04, &
      2.4056191E-01, 2.82193E+01, 9.4161E+00, 1.0477E+04, &
      2.4176472E-01, 2.82211E+01, 9.3951E+00, 1.0401E+04, &
      2.4297355E-01, 2.82218E+01, 9.3745E+00, 1.0327E+04, &
      2.4418841E-01, 2.82214E+01, 9.3542E+00, 1.0253E+04, &
      2.4540936E-01, 2.82197E+01, 9.3342E+00, 1.0181E+04, &
      2.4663640E-01, 2.82166E+01, 9.3145E+00, 1.0109E+04, &
      2.4786959E-01, 2.82122E+01, 9.2951E+00, 1.0037E+04, &
      2.4910893E-01, 2.82061E+01, 9.2760E+00, 9.9670E+03, &
      2.5035448E-01, 2.81983E+01, 9.2573E+00, 9.8974E+03, &
      2.5160625E-01, 2.81886E+01, 9.2388E+00, 9.8286E+03, &
      2.5286428E-01, 2.81768E+01, 9.2207E+00, 9.7606E+03, &
      2.5412860E-01, 2.81626E+01, 9.2029E+00, 9.6933E+03, &
      2.5539925E-01, 2.81457E+01, 9.1854E+00, 9.6267E+03, &
      2.5667624E-01, 2.81256E+01, 9.1682E+00, 9.5610E+03, &
      2.5795962E-01, 2.81019E+01, 9.1513E+00, 9.4959E+03, &
      2.5924942E-01, 2.80738E+01, 9.1348E+00, 9.4316E+03, &
      2.6054567E-01, 2.80404E+01, 9.1186E+00, 9.3681E+03, &
      2.6184840E-01, 2.80004E+01, 9.1027E+00, 9.3053E+03, &
      2.6315764E-01, 2.79517E+01, 9.0871E+00, 9.2432E+03, &
      2.6447343E-01, 2.78913E+01, 9.0718E+00, 9.1818E+03, &
      2.6579579E-01, 2.78140E+01, 9.0569E+00, 9.1211E+03, &
      2.6712477E-01, 2.77097E+01, 9.0423E+00, 9.0611E+03, &
      2.6846040E-01, 2.75550E+01, 9.0280E+00, 9.0018E+03, &
      2.6980270E-01, 2.72664E+01, 9.0140E+00, 8.9432E+03, &
      2.7068519E-01, 2.66714E+01, 9.0051E+00, 8.9052E+03, &
      2.7111483E-01, 2.66713E+01, 1.0149E+01, 1.0020E+04, &
      2.7115171E-01, 2.67292E+01, 1.0149E+01, 1.0019E+04, &
      2.7250747E-01, 2.74074E+01, 1.0139E+01, 9.9596E+03, &
      2.7387001E-01, 2.76326E+01, 1.0130E+01, 9.9011E+03, &
      2.7523936E-01, 2.77715E+01, 1.0121E+01, 9.8434E+03, &
      2.7661556E-01, 2.78715E+01, 1.0113E+01, 9.7863E+03, &
      2.7799863E-01, 2.79486E+01, 1.0105E+01, 9.7298E+03, &
      2.7938863E-01, 2.80099E+01, 1.0097E+01, 9.6740E+03, &
      2.8078557E-01, 2.80589E+01, 1.0090E+01, 9.6188E+03, &
      2.8218950E-01, 2.80969E+01, 1.0083E+01, 9.5642E+03, &
      2.8360044E-01, 2.81236E+01, 1.0076E+01, 9.5103E+03, &
      2.8501845E-01, 2.81360E+01, 1.0069E+01, 9.4569E+03, &
      2.8644354E-01, 2.81244E+01, 1.0063E+01, 9.4042E+03, &
      2.8787576E-01, 2.80362E+01, 1.0057E+01, 9.3521E+03, &
      2.8823661E-01, 2.79515E+01, 1.0056E+01, 9.3391E+03, &
      2.8876341E-01, 2.79661E+01, 1.0400E+01, 9.6410E+03, &
      2.8931514E-01, 2.81056E+01, 1.0399E+01, 9.6213E+03, &
      2.9076171E-01, 2.82566E+01, 1.0395E+01, 9.5702E+03, &
      2.9221552E-01, 2.83485E+01, 1.0392E+01, 9.5197E+03, &
      2.9367660E-01, 2.84209E+01, 1.0389E+01, 9.4697E+03, &
      2.9514498E-01, 2.84830E+01, 1.0387E+01, 9.4203E+03, &
      2.9662071E-01, 2.85386E+01, 1.0384E+01, 9.3715E+03, &
      2.9810381E-01, 2.85897E+01, 1.0382E+01, 9.3232E+03, &
      2.9959433E-01, 2.86373E+01, 1.0381E+01, 9.2754E+03, &
      3.0109230E-01, 2.86822E+01, 1.0380E+01, 9.2281E+03, &
      3.0259776E-01, 2.87248E+01, 1.0379E+01, 9.1814E+03, &
      3.0411075E-01, 2.87656E+01, 1.0378E+01, 9.1351E+03, &
      3.0563130E-01, 2.88048E+01, 1.0377E+01, 9.0893E+03, &
      3.0715946E-01, 2.88426E+01, 1.0377E+01, 9.0439E+03, &
      3.0869526E-01, 2.88792E+01, 1.0377E+01, 8.9990E+03, &
      3.1023873E-01, 2.89148E+01, 1.0378E+01, 8.9546E+03, &
      3.1178993E-01, 2.89493E+01, 1.0378E+01, 8.9107E+03, &
      3.1334888E-01, 2.89831E+01, 1.0379E+01, 8.8671E+03, &
      3.1491562E-01, 2.90160E+01, 1.0380E+01, 8.8240E+03, &
      3.1649020E-01, 2.90483E+01, 1.0382E+01, 8.7814E+03, &
      3.1807265E-01, 2.90799E+01, 1.0383E+01, 8.7391E+03, &
      3.1966301E-01, 2.91109E+01, 1.0385E+01, 8.6973E+03, &
      3.2126133E-01, 2.91413E+01, 1.0387E+01, 8.6558E+03, &
      3.2286764E-01, 2.91712E+01, 1.0390E+01, 8.6148E+03, &
      3.2448197E-01, 2.92006E+01, 1.0392E+01, 8.5741E+03, &
      3.2610438E-01, 2.92296E+01, 1.0395E+01, 8.5338E+03, &
      3.2773491E-01, 2.92580E+01, 1.0398E+01, 8.4938E+03, &
      3.2937358E-01, 2.92861E+01, 1.0401E+01, 8.4542E+03, &
      3.3102045E-01, 2.93137E+01, 1.0405E+01, 8.4150E+03, &
      3.3267555E-01, 2.93409E+01, 1.0409E+01, 8.3761E+03, &
      3.3433893E-01, 2.93676E+01, 1.0412E+01, 8.3375E+03, &
      3.3601062E-01, 2.93940E+01, 1.0416E+01, 8.2993E+03, &
      3.3769068E-01, 2.94199E+01, 1.0421E+01, 8.2613E+03, &
      3.3937913E-01, 2.94453E+01, 1.0425E+01, 8.2237E+03, &
      3.4107602E-01, 2.94703E+01, 1.0429E+01, 8.1864E+03, &
      3.4278140E-01, 2.94948E+01, 1.0434E+01, 8.1494E+03, &
      3.4449531E-01, 2.95187E+01, 1.0439E+01, 8.1126E+03, &
      3.4621779E-01, 2.95421E+01, 1.0444E+01, 8.0761E+03, &
      3.4794888E-01, 2.95648E+01, 1.0449E+01, 8.0399E+03, &
      3.4968862E-01, 2.95869E+01, 1.0454E+01, 8.0040E+03, &
      3.5143706E-01, 2.96081E+01, 1.0460E+01, 7.9683E+03, &
      3.5319425E-01, 2.96284E+01, 1.0465E+01, 7.9328E+03, &
      3.5496022E-01, 2.96477E+01, 1.0471E+01, 7.8976E+03, &
      3.5673502E-01, 2.96656E+01, 1.0476E+01, 7.8627E+03, &
      3.5851870E-01, 2.96821E+01, 1.0482E+01, 7.8279E+03, &
      3.6031129E-01, 2.96967E+01, 1.0488E+01, 7.7933E+03, &
      3.6211285E-01, 2.97089E+01, 1.0494E+01, 7.7590E+03, &
      3.6392341E-01, 2.97182E+01, 1.0500E+01, 7.7249E+03, &
      3.6574303E-01, 2.97233E+01, 1.0506E+01, 7.6909E+03, &
      3.6757174E-01, 2.97226E+01, 1.0512E+01, 7.6572E+03, &
      3.6940960E-01, 2.97132E+01, 1.0518E+01, 7.6236E+03, &
      3.7125665E-01, 2.96888E+01, 1.0525E+01, 7.5902E+03, &
      3.7311293E-01, 2.96331E+01, 1.0531E+01, 7.5570E+03, &
      3.7497850E-01, 2.94654E+01, 1.0537E+01, 7.5239E+03, &
      3.7534905E-01, 2.93702E+01, 1.0538E+01, 7.5174E+03, &
      3.7625097E-01, 2.93860E+01, 1.1074E+01, 7.8806E+03, &
      3.7685339E-01, 2.95405E+01, 1.1077E+01, 7.8697E+03, &
      3.7873766E-01, 2.97481E+01, 1.1084E+01, 7.8358E+03, &
      3.8063135E-01, 2.98664E+01, 1.1092E+01, 7.8020E+03, &
      3.8253450E-01, 2.99570E+01, 1.1099E+01, 7.7683E+03, &
      3.8444718E-01, 3.00339E+01, 1.1106E+01, 7.7348E+03, &
      3.8636941E-01, 3.01027E+01, 1.1113E+01, 7.7014E+03, &
      3.8830126E-01, 3.01662E+01, 1.1121E+01, 7.6681E+03, &
      3.9024276E-01, 3.02258E+01, 1.1128E+01, 7.6349E+03, &
      3.9219398E-01, 3.02827E+01, 1.1135E+01, 7.6019E+03, &
      3.9415495E-01, 3.03374E+01, 1.1142E+01, 7.5689E+03, &
      3.9612572E-01, 3.03904E+01, 1.1149E+01, 7.5361E+03, &
      3.9810635E-01, 3.04421E+01, 1.1156E+01, 7.5033E+03, &
      4.0009688E-01, 3.04928E+01, 1.1163E+01, 7.4706E+03, &
      4.0209737E-01, 3.05425E+01, 1.1170E+01, 7.4380E+03, &
      4.0410785E-01, 3.05915E+01, 1.1176E+01, 7.4055E+03, &
      4.0612839E-01, 3.06399E+01, 1.1183E+01, 7.3731E+03, &
      4.0815904E-01, 3.06877E+01, 1.1190E+01, 7.3407E+03, &
      4.1019983E-01, 3.07352E+01, 1.1196E+01, 7.3084E+03, &
      4.1225083E-01, 3.07822E+01, 1.1202E+01, 7.2761E+03, &
      4.1431208E-01, 3.08289E+01, 1.1208E+01, 7.2439E+03, &
      4.1638364E-01, 3.08754E+01, 1.1214E+01, 7.2118E+03, &
      4.1846556E-01, 3.09217E+01, 1.1220E+01, 7.1797E+03, &
      4.2055789E-01, 3.09677E+01, 1.1226E+01, 7.1476E+03, &
      4.2266068E-01, 3.10021E+01, 1.1231E+01, 7.1155E+03, &
      4.2477398E-01, 3.10478E+01, 1.1237E+01, 7.0835E+03, &
      4.2689785E-01, 3.10934E+01, 1.1242E+01, 7.0515E+03, &
      4.2903234E-01, 3.11389E+01, 1.1247E+01, 7.0195E+03, &
      4.3117750E-01, 3.11842E+01, 1.1251E+01, 6.9875E+03, &
      4.3333339E-01, 3.12295E+01, 1.1256E+01, 6.9556E+03, &
      4.3550006E-01, 3.12747E+01, 1.1260E+01, 6.9236E+03, &
      4.3767756E-01, 3.13198E+01, 1.1264E+01, 6.8917E+03, &
      4.3986595E-01, 3.13649E+01, 1.1268E+01, 6.8597E+03, &
      4.4206528E-01, 3.14099E+01, 1.1271E+01, 6.8278E+03, &
      4.4427560E-01, 3.14548E+01, 1.1275E+01, 6.7959E+03, &
      4.4649698E-01, 3.14997E+01, 1.1278E+01, 6.7640E+03, &
      4.4872947E-01, 3.15445E+01, 1.1280E+01, 6.7320E+03, &
      4.5097311E-01, 3.15892E+01, 1.1283E+01, 6.7001E+03, &
      4.5322798E-01, 3.16339E+01, 1.1285E+01, 6.6682E+03, &
      4.5549412E-01, 3.16786E+01, 1.1287E+01, 6.6362E+03, &
      4.5777159E-01, 3.17232E+01, 1.1289E+01, 6.6043E+03, &
      4.6006045E-01, 3.17678E+01, 1.1291E+01, 6.5723E+03, &
      4.6236075E-01, 3.18123E+01, 1.1292E+01, 6.5403E+03, &
      4.6467255E-01, 3.18567E+01, 1.1293E+01, 6.5084E+03, &
      4.6699592E-01, 3.19011E+01, 1.1293E+01, 6.4764E+03, &
      4.6933090E-01, 3.19454E+01, 1.1294E+01, 6.4444E+03, &
      4.7167755E-01, 3.19897E+01, 1.1294E+01, 6.4124E+03, &
      4.7403594E-01, 3.20339E+01, 1.1293E+01, 6.3804E+03, &
      4.7640612E-01, 3.20780E+01, 1.1293E+01, 6.3483E+03, &
      4.7878815E-01, 3.21220E+01, 1.1292E+01, 6.3163E+03, &
      4.8118209E-01, 3.21660E+01, 1.1290E+01, 6.2842E+03, &
      4.8358800E-01, 3.22098E+01, 1.1289E+01, 6.2521E+03, &
      4.8600594E-01, 3.22536E+01, 1.1287E+01, 6.2200E+03, &
      4.8843597E-01, 3.22973E+01, 1.1285E+01, 6.1879E+03, &
      4.9087815E-01, 3.23408E+01, 1.1282E+01, 6.1558E+03, &
      4.9333254E-01, 3.23843E+01, 1.1279E+01, 6.1237E+03, &
      4.9579920E-01, 3.24276E+01, 1.1276E+01, 6.0915E+03, &
      4.9827820E-01, 3.24708E+01, 1.1273E+01, 6.0594E+03, &
      5.0076959E-01, 3.25139E+01, 1.1269E+01, 6.0272E+03, &
      5.0327344E-01, 3.25568E+01, 1.1265E+01, 5.9951E+03, &
      5.0578980E-01, 3.25996E+01, 1.1260E+01, 5.9629E+03, &
      5.0831875E-01, 3.26423E+01, 1.1255E+01, 5.9307E+03, &
      5.1086035E-01, 3.26848E+01, 1.1250E+01, 5.8985E+03, &
      5.1341465E-01, 3.27271E+01, 1.1244E+01, 5.8663E+03, &
      5.1598172E-01, 3.27693E+01, 1.1238E+01, 5.8341E+03, &
      5.1856163E-01, 3.28112E+01, 1.1232E+01, 5.8019E+03, &
      5.2115444E-01, 3.28530E+01, 1.1226E+01, 5.7696E+03, &
      5.2376021E-01, 3.28946E+01, 1.1219E+01, 5.7374E+03, &
      5.2637901E-01, 3.29360E+01, 1.1211E+01, 5.7052E+03, &
      5.2901091E-01, 3.29772E+01, 1.1204E+01, 5.6730E+03, &
      5.3165596E-01, 3.30182E+01, 1.1196E+01, 5.6408E+03, &
      5.3431424E-01, 3.30590E+01, 1.1187E+01, 5.6086E+03, &
      5.3698581E-01, 3.30995E+01, 1.1178E+01, 5.5763E+03, &
      5.3967074E-01, 3.31398E+01, 1.1169E+01, 5.5441E+03, &
      5.4236910E-01, 3.31798E+01, 1.1160E+01, 5.5120E+03, &
      5.4508094E-01, 3.32196E+01, 1.1150E+01, 5.4798E+03, &
      5.4780635E-01, 3.32592E+01, 1.1140E+01, 5.4476E+03, &
      5.5054538E-01, 3.32984E+01, 1.1129E+01, 5.4155E+03, &
      5.5329810E-01, 3.33374E+01, 1.1119E+01, 5.3833E+03, &
      5.5606460E-01, 3.33761E+01, 1.1107E+01, 5.3512E+03, &
      5.5884492E-01, 3.34146E+01, 1.1096E+01, 5.3191E+03, &
      5.6163914E-01, 3.34527E+01, 1.1084E+01, 5.2870E+03, &
      5.6444734E-01, 3.34905E+01, 1.1072E+01, 5.2549E+03, &
      5.6726958E-01, 3.35280E+01, 1.1059E+01, 5.2228E+03, &
      5.7010592E-01, 3.35652E+01, 1.1046E+01, 5.1907E+03, &
      5.7295645E-01, 3.36020E+01, 1.1032E+01, 5.1586E+03, &
      5.7582123E-01, 3.36385E+01, 1.1018E+01, 5.1265E+03, &
      5.7870034E-01, 3.36746E+01, 1.1004E+01, 5.0944E+03, &
      5.8159384E-01, 3.37102E+01, 1.0989E+01, 5.0623E+03, &
      5.8450181E-01, 3.37455E+01, 1.0974E+01, 5.0302E+03, &
      5.8742432E-01, 3.37803E+01, 1.0958E+01, 4.9981E+03, &
      5.9036144E-01, 3.38147E+01, 1.0942E+01, 4.9660E+03, &
      5.9331325E-01, 3.38487E+01, 1.0926E+01, 4.9340E+03, &
      5.9627982E-01, 3.38822E+01, 1.0909E+01, 4.9019E+03, &
      5.9926122E-01, 3.39152E+01, 1.0892E+01, 4.8699E+03, &
      6.0225752E-01, 3.39477E+01, 1.0874E+01, 4.8380E+03, &
      6.0526881E-01, 3.39797E+01, 1.0856E+01, 4.8060E+03, &
      6.0829515E-01, 3.40112E+01, 1.0838E+01, 4.7741E+03, &
      6.1133663E-01, 3.40421E+01, 1.0819E+01, 4.7422E+03, &
      6.1439331E-01, 3.40726E+01, 1.0800E+01, 4.7104E+03, &
      6.1746528E-01, 3.41024E+01, 1.0781E+01, 4.6787E+03, &
      6.2055260E-01, 3.41317E+01, 1.0761E+01, 4.6469E+03, &
      6.2365537E-01, 3.41604E+01, 1.0741E+01, 4.6153E+03, &
      6.2677364E-01, 3.41886E+01, 1.0721E+01, 4.5837E+03, &
      6.2990751E-01, 3.42161E+01, 1.0700E+01, 4.5521E+03, &
      6.3305705E-01, 3.42430E+01, 1.0679E+01, 4.5207E+03, &
      6.3622234E-01, 3.42693E+01, 1.0658E+01, 4.4893E+03, &
      6.3940345E-01, 3.42950E+01, 1.0636E+01, 4.4579E+03, &
      6.4260046E-01, 3.43200E+01, 1.0614E+01, 4.4267E+03, &
      6.4581347E-01, 3.43443E+01, 1.0592E+01, 4.3955E+03, &
      6.4904253E-01, 3.43680E+01, 1.0570E+01, 4.3644E+03, &
      6.5228775E-01, 3.43910E+01, 1.0547E+01, 4.3334E+03, &
      6.5554919E-01, 3.44133E+01, 1.0524E+01, 4.3025E+03, &
      6.5882693E-01, 3.44349E+01, 1.0500E+01, 4.2717E+03, &
      6.6212107E-01, 3.44558E+01, 1.0477E+01, 4.2410E+03, &
      6.6543167E-01, 3.44760E+01, 1.0453E+01, 4.2103E+03, &
      6.6875883E-01, 3.44954E+01, 1.0429E+01, 4.1798E+03, &
      6.7210262E-01, 3.45141E+01, 1.0405E+01, 4.1494E+03, &
      6.7546314E-01, 3.45320E+01, 1.0380E+01, 4.1190E+03, &
      6.7884045E-01, 3.45491E+01, 1.0355E+01, 4.0888E+03, &
      6.8223466E-01, 3.45655E+01, 1.0330E+01, 4.0587E+03, &
      6.8564583E-01, 3.45810E+01, 1.0305E+01, 4.0286E+03, &
      6.8907406E-01, 3.45957E+01, 1.0279E+01, 3.9987E+03, &
      6.9251943E-01, 3.46097E+01, 1.0253E+01, 3.9690E+03, &
      6.9598202E-01, 3.46227E+01, 1.0227E+01, 3.9393E+03, &
      6.9946194E-01, 3.46349E+01, 1.0201E+01, 3.9097E+03, &
      7.0295924E-01, 3.46463E+01, 1.0175E+01, 3.8803E+03, &
      7.0647404E-01, 3.46568E+01, 1.0148E+01, 3.8510E+03, &
      7.1000641E-01, 3.46664E+01, 1.0122E+01, 3.8218E+03, &
      7.1355644E-01, 3.46751E+01, 1.0095E+01, 3.7928E+03, &
      7.1712423E-01, 3.46828E+01, 1.0068E+01, 3.7639E+03, &
      7.2070985E-01, 3.46897E+01, 1.0041E+01, 3.7351E+03, &
      7.2431340E-01, 3.46956E+01, 1.0013E+01, 3.7065E+03, &
      7.2793496E-01, 3.47005E+01, 9.9857E+00, 3.6780E+03, &
      7.3157464E-01, 3.47045E+01, 9.9580E+00, 3.6496E+03, &
      7.3523251E-01, 3.47074E+01, 9.9302E+00, 3.6214E+03, &
      7.3890867E-01, 3.47094E+01, 9.9022E+00, 3.5933E+03, &
      7.4260322E-01, 3.47103E+01, 9.8741E+00, 3.5653E+03, &
      7.4631623E-01, 3.47102E+01, 9.8459E+00, 3.5375E+03, &
      7.5004781E-01, 3.47091E+01, 9.8176E+00, 3.5099E+03, &
      7.5379805E-01, 3.47069E+01, 9.7891E+00, 3.4824E+03, &
      7.5756704E-01, 3.47035E+01, 9.7605E+00, 3.4550E+03, &
      7.6135488E-01, 3.46991E+01, 9.7319E+00, 3.4278E+03, &
      7.6516165E-01, 3.46935E+01, 9.7031E+00, 3.4007E+03, &
      7.6898746E-01, 3.46868E+01, 9.6742E+00, 3.3738E+03, &
      7.7283240E-01, 3.46789E+01, 9.6452E+00, 3.3471E+03, &
      7.7669656E-01, 3.46699E+01, 9.6162E+00, 3.3205E+03, &
      7.8058004E-01, 3.46596E+01, 9.5870E+00, 3.2940E+03, &
      7.8448294E-01, 3.46480E+01, 9.5578E+00, 3.2677E+03, &
      7.8840536E-01, 3.46353E+01, 9.5285E+00, 3.2416E+03, &
      7.9234738E-01, 3.46212E+01, 9.4991E+00, 3.2156E+03, &
      7.9630912E-01, 3.46058E+01, 9.4697E+00, 3.1897E+03, &
      8.0029067E-01, 3.45891E+01, 9.4402E+00, 3.1641E+03, &
      8.0429212E-01, 3.45710E+01, 9.4107E+00, 3.1385E+03, &
      8.0831358E-01, 3.45516E+01, 9.3811E+00, 3.1132E+03, &
      8.1235515E-01, 3.45307E+01, 9.3514E+00, 3.0880E+03, &
      8.1641693E-01, 3.45113E+01, 9.3217E+00, 3.0629E+03, &
      8.2049901E-01, 3.44875E+01, 9.2920E+00, 3.0381E+03, &
      8.2460150E-01, 3.44622E+01, 9.2622E+00, 3.0133E+03, &
      8.2872451E-01, 3.44354E+01, 9.2324E+00, 2.9888E+03, &
      8.3286813E-01, 3.44070E+01, 9.2025E+00, 2.9644E+03, &
      8.3703248E-01, 3.43770E+01, 9.1727E+00, 2.9401E+03, &
      8.4121764E-01, 3.43453E+01, 9.1428E+00, 2.9160E+03, &
      8.4542373E-01, 3.43120E+01, 9.1129E+00, 2.8921E+03, &
      8.4965084E-01, 3.42769E+01, 9.0829E+00, 2.8684E+03, &
      8.5389910E-01, 3.42400E+01, 9.0530E+00, 2.8448E+03, &
      8.5816859E-01, 3.42013E+01, 9.0230E+00, 2.8213E+03, &
      8.6245944E-01, 3.41607E+01, 8.9931E+00, 2.7981E+03, &
      8.6677173E-01, 3.41190E+01, 8.9631E+00, 2.7749E+03, &
      8.7110559E-01, 3.40745E+01, 8.9332E+00, 2.7520E+03, &
      8.7546112E-01, 3.40281E+01, 8.9032E+00, 2.7292E+03, &
      8.7983843E-01, 3.39795E+01, 8.8732E+00, 2.7066E+03, &
      8.8423762E-01, 3.39288E+01, 8.8433E+00, 2.6841E+03, &
      8.8865881E-01, 3.38760E+01, 8.8134E+00, 2.6618E+03, &
      8.9310210E-01, 3.38208E+01, 8.7835E+00, 2.6396E+03, &
      8.9756761E-01, 3.37633E+01, 8.7536E+00, 2.6177E+03, &
      9.0205545E-01, 3.37035E+01, 8.7237E+00, 2.5958E+03, &
      9.0656573E-01, 3.36411E+01, 8.6938E+00, 2.5741E+03, &
      9.1109856E-01, 3.35762E+01, 8.6640E+00, 2.5526E+03, &
      9.1565405E-01, 3.35087E+01, 8.6342E+00, 2.5313E+03, &
      9.2023232E-01, 3.34385E+01, 8.6044E+00, 2.5101E+03, &
      9.2483348E-01, 3.33655E+01, 8.5747E+00, 2.4891E+03, &
      9.2945765E-01, 3.32895E+01, 8.5450E+00, 2.4682E+03, &
      9.3410494E-01, 3.32106E+01, 8.5153E+00, 2.4475E+03, &
      9.3877546E-01, 3.31286E+01, 8.4857E+00, 2.4269E+03, &
      9.4346934E-01, 3.30435E+01, 8.4561E+00, 2.4065E+03, &
      9.4818668E-01, 3.29550E+01, 8.4266E+00, 2.3862E+03, &
      9.5292762E-01, 3.28631E+01, 8.3971E+00, 2.3661E+03, &
      9.5769226E-01, 3.27677E+01, 8.3676E+00, 2.3462E+03, &
      9.6248072E-01, 3.26686E+01, 8.3382E+00, 2.3264E+03, &
      9.6729312E-01, 3.25658E+01, 8.3089E+00, 2.3067E+03, &
      9.7212959E-01, 3.24592E+01, 8.2796E+00, 2.2873E+03, &
      9.7699023E-01, 3.23485E+01, 8.2503E+00, 2.2679E+03, &
      9.8187519E-01, 3.22338E+01, 8.2211E+00, 2.2487E+03, &
      9.8678456E-01, 3.21148E+01, 8.1920E+00, 2.2297E+03, &
      9.9171848E-01, 3.19916E+01, 8.1630E+00, 2.2108E+03, &
      9.9667708E-01, 3.18640E+01, 8.1339E+00, 2.1921E+03, &
      1.0016605E+00, 3.17259E+01, 8.0996E+00, 2.1721E+03, &
      1.0066688E+00, 3.15686E+01, 8.0546E+00, 2.1494E+03, &
      1.0117021E+00, 3.14033E+01, 8.0100E+00, 2.1269E+03, &
      1.0167606E+00, 3.12296E+01, 7.9656E+00, 2.1047E+03, &
      1.0218444E+00, 3.10473E+01, 7.9215E+00, 2.0827E+03, &
      1.0269536E+00, 3.08556E+01, 7.8777E+00, 2.0610E+03, &
      1.0320884E+00, 3.06542E+01, 7.8341E+00, 2.0395E+03, &
      1.0372489E+00, 3.04423E+01, 7.7909E+00, 2.0183E+03, &
      1.0424351E+00, 3.02191E+01, 7.7479E+00, 1.9973E+03, &
      1.0476473E+00, 2.99839E+01, 7.7053E+00, 1.9765E+03, &
      1.0528855E+00, 2.97357E+01, 7.6629E+00, 1.9559E+03, &
      1.0581499E+00, 2.94734E+01, 7.6208E+00, 1.9356E+03, &
      1.0634407E+00, 2.91958E+01, 7.5789E+00, 1.9155E+03, &
      1.0687579E+00, 2.89016E+01, 7.5374E+00, 1.8956E+03, &
      1.0741017E+00, 2.85893E+01, 7.4961E+00, 1.8760E+03, &
      1.0794722E+00, 2.82568E+01, 7.4551E+00, 1.8565E+03, &
      1.0848695E+00, 2.79023E+01, 7.4140E+00, 1.8372E+03, &
      1.0902939E+00, 2.75231E+01, 7.3716E+00, 1.8177E+03, &
      1.0957454E+00, 2.71163E+01, 7.3296E+00, 1.7985E+03, &
      1.1012241E+00, 2.66784E+01, 7.2878E+00, 1.7794E+03, &
      1.1067302E+00, 2.62054E+01, 7.2464E+00, 1.7606E+03, &
      1.1122639E+00, 2.56921E+01, 7.2052E+00, 1.7420E+03, &
      1.1178252E+00, 2.51323E+01, 7.1643E+00, 1.7236E+03, &
      1.1234143E+00, 2.45182E+01, 7.1237E+00, 1.7054E+03, &
      1.1290314E+00, 2.38396E+01, 7.0834E+00, 1.6874E+03, &
      1.1346765E+00, 2.30837E+01, 7.0433E+00, 1.6696E+03, &
      1.1403499E+00, 2.22325E+01, 7.0035E+00, 1.6520E+03, &
      1.1460517E+00, 2.12613E+01, 6.9640E+00, 1.6346E+03, &
      1.1517819E+00, 2.01339E+01, 6.9248E+00, 1.6175E+03, &
      1.1575408E+00, 1.87939E+01, 6.8853E+00, 1.6003E+03, &
      1.1633285E+00, 1.71463E+01, 6.8457E+00, 1.5833E+03, &
      1.1691452E+00, 1.50091E+01, 6.8064E+00, 1.5665E+03, &
      1.1749909E+00, 1.19519E+01, 6.7674E+00, 1.5499E+03, &
      1.1808659E+00, 6.37618E+00, 6.7286E+00, 1.5334E+03, &
      1.1850578E+00, -1.47683E+01, 6.7012E+00, 1.5218E+03, &
      1.1853422E+00, -1.51056E+01, 2.5900E+01, 5.8559E+03, &
      1.1867702E+00, -5.17733E-01, 2.5854E+01, 5.8383E+03, &
      1.1927040E+00, 8.46718E+00, 2.5662E+01, 5.7663E+03, &
      1.1986676E+00, 1.11758E+01, 2.5472E+01, 5.6952E+03, &
      1.2046609E+00, 1.20958E+01, 2.5283E+01, 5.6251E+03, &
      1.2106842E+00, 1.13510E+01, 2.5096E+01, 5.5557E+03, &
      1.2167376E+00, 2.28360E+00, 2.4910E+01, 5.4873E+03, &
      1.2169590E+00, -2.80404E-01, 2.4904E+01, 5.4848E+03, &
      1.2174410E+00, -3.74992E-01, 3.7507E+01, 8.2530E+03, &
      1.2228213E+00, 1.34452E+01, 3.7258E+01, 8.1621E+03, &
      1.2289354E+00, 1.75253E+01, 3.6978E+01, 8.0606E+03, &
      1.2350801E+00, 2.01965E+01, 3.6700E+01, 7.9604E+03, &
      1.2412555E+00, 2.22583E+01, 3.6424E+01, 7.8614E+03, &
      1.2474618E+00, 2.39629E+01, 3.6151E+01, 7.7636E+03, &
      1.2536991E+00, 2.54265E+01, 3.5879E+01, 7.6671E+03, &
      1.2599676E+00, 2.67135E+01, 3.5610E+01, 7.5718E+03, &
      1.2662674E+00, 2.78643E+01, 3.5343E+01, 7.4777E+03, &
      1.2725988E+00, 2.89059E+01, 3.5077E+01, 7.3848E+03, &
      1.2789618E+00, 2.98574E+01, 3.4814E+01, 7.2931E+03, &
      1.2853566E+00, 3.07332E+01, 3.4553E+01, 7.2025E+03, &
      1.2917833E+00, 3.15438E+01, 3.4294E+01, 7.1130E+03, &
      1.2982423E+00, 3.22980E+01, 3.4037E+01, 7.0247E+03, &
      1.3047335E+00, 3.30023E+01, 3.3782E+01, 6.9375E+03, &
      1.3112571E+00, 3.36624E+01, 3.3529E+01, 6.8514E+03, &
      1.3178134E+00, 3.42827E+01, 3.3278E+01, 6.7664E+03, &
      1.3244025E+00, 3.48671E+01, 3.3029E+01, 6.6824E+03, &
      1.3310245E+00, 3.54186E+01, 3.2781E+01, 6.5995E+03, &
      1.3376796E+00, 3.59401E+01, 3.2536E+01, 6.5176E+03, &
      1.3443680E+00, 3.64337E+01, 3.2293E+01, 6.4368E+03, &
      1.3510899E+00, 3.69016E+01, 3.2051E+01, 6.3570E+03, &
      1.3578453E+00, 3.73453E+01, 3.1811E+01, 6.2782E+03, &
      1.3646345E+00, 3.77665E+01, 3.1574E+01, 6.2004E+03, &
      1.3714577E+00, 3.81662E+01, 3.1338E+01, 6.1235E+03, &
      1.3783150E+00, 3.85457E+01, 3.1103E+01, 6.0476E+03, &
      1.3852066E+00, 3.89058E+01, 3.0871E+01, 5.9727E+03, &
      1.3921326E+00, 3.92473E+01, 3.0640E+01, 5.8987E+03, &
      1.3990933E+00, 3.95709E+01, 3.0412E+01, 5.8257E+03, &
      1.4060887E+00, 3.98771E+01, 3.0185E+01, 5.7535E+03, &
      1.4131192E+00, 4.01663E+01, 2.9959E+01, 5.6823E+03, &
      1.4201848E+00, 4.04386E+01, 2.9736E+01, 5.6120E+03, &
      1.4272857E+00, 4.06942E+01, 2.9514E+01, 5.5425E+03, &
      1.4344221E+00, 4.09331E+01, 2.9294E+01, 5.4739E+03, &
      1.4415942E+00, 4.11549E+01, 2.9076E+01, 5.4062E+03, &
      1.4488022E+00, 4.13593E+01, 2.8859E+01, 5.3393E+03, &
      1.4560462E+00, 4.15454E+01, 2.8644E+01, 5.2733E+03, &
      1.4633265E+00, 4.17121E+01, 2.8431E+01, 5.2081E+03, &
      1.4706431E+00, 4.18580E+01, 2.8219E+01, 5.1437E+03, &
      1.4779963E+00, 4.19806E+01, 2.8009E+01, 5.0801E+03, &
      1.4853863E+00, 4.20769E+01, 2.7800E+01, 5.0173E+03, &
      1.4928132E+00, 4.21419E+01, 2.7593E+01, 4.9553E+03, &
      1.5002773E+00, 4.21684E+01, 2.7388E+01, 4.8941E+03, &
      1.5077787E+00, 4.21448E+01, 2.7184E+01, 4.8336E+03, &
      1.5153176E+00, 4.20507E+01, 2.6982E+01, 4.7739E+03, &
      1.5228942E+00, 4.18463E+01, 2.6781E+01, 4.7149E+03, &
      1.5305086E+00, 4.14359E+01, 2.6582E+01, 4.6567E+03, &
      1.5381612E+00, 4.04625E+01, 2.6384E+01, 4.5991E+03, &
      1.5427108E+00, 3.84229E+01, 2.6267E+01, 4.5653E+03, &
      1.5452893E+00, 3.84315E+01, 3.0743E+01, 5.3328E+03, &
      1.5458520E+00, 3.89750E+01, 3.0725E+01, 5.3277E+03, &
      1.5535812E+00, 4.15876E+01, 3.0476E+01, 5.2584E+03, &
      1.5613491E+00, 4.26547E+01, 3.0229E+01, 5.1899E+03, &
      1.5691559E+00, 4.33830E+01, 2.9984E+01, 5.1224E+03, &
      1.5770017E+00, 4.39517E+01, 2.9741E+01, 5.0557E+03, &
      1.5848867E+00, 4.44237E+01, 2.9499E+01, 4.9898E+03, &
      1.5928111E+00, 4.48279E+01, 2.9258E+01, 4.9245E+03, &
      1.6007752E+00, 4.51798E+01, 2.9019E+01, 4.8601E+03, &
      1.6087790E+00, 4.54880E+01, 2.8782E+01, 4.7965E+03, &
      1.6168229E+00, 4.57569E+01, 2.8547E+01, 4.7338E+03, &
      1.6249070E+00, 4.59888E+01, 2.8326E+01, 4.6740E+03, &
      1.6330316E+00, 4.61953E+01, 2.8116E+01, 4.6164E+03, &
      1.6411967E+00, 4.63751E+01, 2.7910E+01, 4.5597E+03, &
      1.6494027E+00, 4.65238E+01, 2.7705E+01, 4.5040E+03, &
      1.6576497E+00, 4.66335E+01, 2.7504E+01, 4.4491E+03, &
      1.6659380E+00, 4.66876E+01, 2.7305E+01, 4.3950E+03, &
      1.6742677E+00, 4.66433E+01, 2.7108E+01, 4.3418E+03, &
      1.6826390E+00, 4.63288E+01, 2.6914E+01, 4.2894E+03, &
      1.6868076E+00, 4.56433E+01, 2.6819E+01, 4.2637E+03, &
      1.6897925E+00, 4.56961E+01, 2.8633E+01, 4.5434E+03, &
      1.6910522E+00, 4.60974E+01, 2.8601E+01, 4.5349E+03, &
      1.6995075E+00, 4.71547E+01, 2.8389E+01, 4.4791E+03, &
      1.7080050E+00, 4.76933E+01, 2.8179E+01, 4.4240E+03, &
      1.7165450E+00, 4.80981E+01, 2.7972E+01, 4.3698E+03, &
      1.7251278E+00, 4.84359E+01, 2.7768E+01, 4.3164E+03, &
      1.7337534E+00, 4.87315E+01, 2.7566E+01, 4.2637E+03, &
      1.7424222E+00, 4.89965E+01, 2.7366E+01, 4.2118E+03, &
      1.7511343E+00, 4.92374E+01, 2.7168E+01, 4.1607E+03, &
      1.7598899E+00, 4.94580E+01, 2.6972E+01, 4.1103E+03, &
      1.7686894E+00, 4.96599E+01, 2.6779E+01, 4.0606E+03, &
      1.7775328E+00, 4.98450E+01, 2.6594E+01, 4.0127E+03, &
      1.7864205E+00, 5.00193E+01, 2.6416E+01, 3.9660E+03, &
      1.7953526E+00, 5.01830E+01, 2.6240E+01, 3.9201E+03, &
      1.8043294E+00, 5.03355E+01, 2.6066E+01, 3.8748E+03, &
      1.8133510E+00, 5.04763E+01, 2.5894E+01, 3.8303E+03, &
      1.8224178E+00, 5.06043E+01, 2.5725E+01, 3.7864E+03, &
      1.8315299E+00, 5.07176E+01, 2.5557E+01, 3.7431E+03, &
      1.8406875E+00, 5.08127E+01, 2.5392E+01, 3.7005E+03, &
      1.8498909E+00, 5.08829E+01, 2.5229E+01, 3.6585E+03, &
      1.8591404E+00, 5.09135E+01, 2.5067E+01, 3.6171E+03, &
      1.8684361E+00, 5.08640E+01, 2.4908E+01, 3.5763E+03, &
      1.8777783E+00, 5.04941E+01, 2.4750E+01, 3.5361E+03, &
      1.8778472E+00, 5.04867E+01, 2.4749E+01, 3.5358E+03, &
      1.8837529E+00, 5.05743E+01, 2.5825E+01, 3.6777E+03, &
      1.8871672E+00, 5.09179E+01, 2.5767E+01, 3.6628E+03, &
      1.8966030E+00, 5.14087E+01, 2.5609E+01, 3.6223E+03, &
      1.9060860E+00, 5.17311E+01, 2.5452E+01, 3.5823E+03, &
      1.9156165E+00, 5.19939E+01, 2.5297E+01, 3.5428E+03, &
      1.9251945E+00, 5.22248E+01, 2.5143E+01, 3.5038E+03, &
      1.9348205E+00, 5.24354E+01, 2.4991E+01, 3.4654E+03, &
      1.9444946E+00, 5.26314E+01, 2.4840E+01, 3.4274E+03, &
      1.9542171E+00, 5.28165E+01, 2.4690E+01, 3.3899E+03, &
      1.9639882E+00, 5.29929E+01, 2.4542E+01, 3.3529E+03, &
      1.9738081E+00, 5.31619E+01, 2.4395E+01, 3.3164E+03, &
      1.9836772E+00, 5.33252E+01, 2.4251E+01, 3.2804E+03, &
      1.9935955E+00, 5.34841E+01, 2.4108E+01, 3.2450E+03, &
      2.0035635E+00, 5.36394E+01, 2.3966E+01, 3.2099E+03, &
      2.0135813E+00, 5.37909E+01, 2.3821E+01, 3.1746E+03, &
      2.0236492E+00, 5.39373E+01, 2.3673E+01, 3.1394E+03, &
      2.0337675E+00, 5.40792E+01, 2.3527E+01, 3.1046E+03, &
      2.0439363E+00, 5.42170E+01, 2.3382E+01, 3.0702E+03, &
      2.0541560E+00, 5.43512E+01, 2.3238E+01, 3.0362E+03, &
      2.0644268E+00, 5.44819E+01, 2.3095E+01, 3.0026E+03, &
      2.0747489E+00, 5.46095E+01, 2.2953E+01, 2.9694E+03, &
      2.0851227E+00, 5.47342E+01, 2.2812E+01, 2.9366E+03, &
      2.0955483E+00, 5.48562E+01, 2.2672E+01, 2.9042E+03, &
      2.1060260E+00, 5.49756E+01, 2.2533E+01, 2.8721E+03, &
      2.1165562E+00, 5.50926E+01, 2.2395E+01, 2.8404E+03, &
      2.1271389E+00, 5.52073E+01, 2.2258E+01, 2.8090E+03, &
      2.1377746E+00, 5.53199E+01, 2.2122E+01, 2.7780E+03, &
      2.1484635E+00, 5.54305E+01, 2.1987E+01, 2.7474E+03, &
      2.1592058E+00, 5.55393E+01, 2.1852E+01, 2.7171E+03, &
      2.1700018E+00, 5.56464E+01, 2.1718E+01, 2.6871E+03, &
      2.1808519E+00, 5.57519E+01, 2.1586E+01, 2.6575E+03, &
      2.1917561E+00, 5.58560E+01, 2.1454E+01, 2.6282E+03, &
      2.2027149E+00, 5.59585E+01, 2.1320E+01, 2.5989E+03, &
      2.2137285E+00, 5.60586E+01, 2.1186E+01, 2.5698E+03, &
      2.2247971E+00, 5.61566E+01, 2.1052E+01, 2.5410E+03, &
      2.2359211E+00, 5.62525E+01, 2.0920E+01, 2.5126E+03, &
      2.2471007E+00, 5.63466E+01, 2.0788E+01, 2.4844E+03, &
      2.2583362E+00, 5.64389E+01, 2.0657E+01, 2.4566E+03, &
      2.2696279E+00, 5.65294E+01, 2.0527E+01, 2.4291E+03, &
      2.2809760E+00, 5.66183E+01, 2.0398E+01, 2.4018E+03, &
      2.2923809E+00, 5.67056E+01, 2.0269E+01, 2.3749E+03, &
      2.3038428E+00, 5.67914E+01, 2.0141E+01, 2.3483E+03, &
      2.3153620E+00, 5.68757E+01, 2.0014E+01, 2.3219E+03, &
      2.3269388E+00, 5.69586E+01, 1.9888E+01, 2.2959E+03, &
      2.3385735E+00, 5.70402E+01, 1.9762E+01, 2.2701E+03, &
      2.3502664E+00, 5.71205E+01, 1.9637E+01, 2.2446E+03, &
      2.3620177E+00, 5.71996E+01, 1.9513E+01, 2.2195E+03, &
      2.3738278E+00, 5.74955E+01, 1.9389E+01, 2.1944E+03, &
      2.3856970E+00, 5.75721E+01, 1.9261E+01, 2.1692E+03, &
      2.3976254E+00, 5.76471E+01, 1.9134E+01, 2.1443E+03, &
      2.4096136E+00, 5.77206E+01, 1.9009E+01, 2.1197E+03, &
      2.4216616E+00, 5.77926E+01, 1.8884E+01, 2.0954E+03, &
      2.4337699E+00, 5.78632E+01, 1.8759E+01, 2.0713E+03, &
      2.4459388E+00, 5.80807E+01, 1.8633E+01, 2.0472E+03, &
      2.4581685E+00, 5.81483E+01, 1.8507E+01, 2.0233E+03, &
      2.4704593E+00, 5.82142E+01, 1.8381E+01, 1.9997E+03, &
      2.4828116E+00, 5.82783E+01, 1.8256E+01, 1.9763E+03, &
      2.4952257E+00, 5.83409E+01, 1.8132E+01, 1.9532E+03, &
      2.5077018E+00, 5.84020E+01, 1.8009E+01, 1.9304E+03, &
      2.5202403E+00, 5.84617E+01, 1.7887E+01, 1.9079E+03, &
      2.5328415E+00, 5.85199E+01, 1.7766E+01, 1.8856E+03, &
      2.5455057E+00, 5.85768E+01, 1.7645E+01, 1.8636E+03, &
      2.5582333E+00, 5.86324E+01, 1.7525E+01, 1.8418E+03, &
      2.5710244E+00, 5.86868E+01, 1.7407E+01, 1.8203E+03, &
      2.5838796E+00, 5.87400E+01, 1.7289E+01, 1.7991E+03, &
      2.5967990E+00, 5.87920E+01, 1.7172E+01, 1.7781E+03, &
      2.6097829E+00, 5.88429E+01, 1.7055E+01, 1.7573E+03, &
      2.6228319E+00, 5.88927E+01, 1.6940E+01, 1.7369E+03, &
      2.6359460E+00, 5.89415E+01, 1.6825E+01, 1.7166E+03, &
      2.6491257E+00, 5.89893E+01, 1.6709E+01, 1.6964E+03, &
      2.6623714E+00, 5.90360E+01, 1.6594E+01, 1.6763E+03, &
      2.6756832E+00, 5.90817E+01, 1.6479E+01, 1.6565E+03, &
      2.6890617E+00, 5.91263E+01, 1.6365E+01, 1.6370E+03, &
      2.7025070E+00, 5.91700E+01, 1.6252E+01, 1.6177E+03, &
      2.7160195E+00, 5.92127E+01, 1.6140E+01, 1.5986E+03, &
      2.7295996E+00, 5.92546E+01, 1.6028E+01, 1.5798E+03, &
      2.7432476E+00, 5.92955E+01, 1.5918E+01, 1.5612E+03, &
      2.7569638E+00, 5.93356E+01, 1.5808E+01, 1.5428E+03, &
      2.7707486E+00, 5.93750E+01, 1.5700E+01, 1.5247E+03, &
      2.7846024E+00, 5.94135E+01, 1.5592E+01, 1.5067E+03, &
      2.7985254E+00, 5.94514E+01, 1.5485E+01, 1.4890E+03, &
      2.8125180E+00, 5.94885E+01, 1.5379E+01, 1.4716E+03, &
      2.8265806E+00, 5.95250E+01, 1.5273E+01, 1.4543E+03, &
      2.8407135E+00, 5.95610E+01, 1.5169E+01, 1.4373E+03, &
      2.8549171E+00, 5.95963E+01, 1.5065E+01, 1.4204E+03, &
      2.8691917E+00, 5.96312E+01, 1.4963E+01, 1.4038E+03, &
      2.8835376E+00, 5.96655E+01, 1.4861E+01, 1.3874E+03, &
      2.8979553E+00, 5.96995E+01, 1.4760E+01, 1.3712E+03, &
      2.9124451E+00, 5.97331E+01, 1.4660E+01, 1.3552E+03, &
      2.9270073E+00, 5.97664E+01, 1.4560E+01, 1.3394E+03, &
      2.9416424E+00, 5.97994E+01, 1.4462E+01, 1.3238E+03, &
      2.9563506E+00, 5.98323E+01, 1.4364E+01, 1.3084E+03, &
      2.9711323E+00, 5.98651E+01, 1.4267E+01, 1.2932E+03, &
      2.9859880E+00, 5.98980E+01, 1.4171E+01, 1.2782E+03, &
      3.0009179E+00, 5.99312E+01, 1.4075E+01, 1.2633E+03, &
      3.0159225E+00, 5.99661E+01, 1.3966E+01, 1.2473E+03, &
      3.0310021E+00, 5.99971E+01, 1.3857E+01, 1.2316E+03, &
      3.0461571E+00, 6.00255E+01, 1.3750E+01, 1.2161E+03, &
      3.0613879E+00, 6.00522E+01, 1.3644E+01, 1.2008E+03, &
      3.0766949E+00, 6.00773E+01, 1.3539E+01, 1.1856E+03, &
      3.0920783E+00, 6.02037E+01, 1.3433E+01, 1.1707E+03, &
      3.1075387E+00, 6.02269E+01, 1.3327E+01, 1.1557E+03, &
      3.1230764E+00, 6.02486E+01, 1.3221E+01, 1.1409E+03, &
      3.1386918E+00, 6.02690E+01, 1.3116E+01, 1.1263E+03, &
      3.1543853E+00, 6.02883E+01, 1.3012E+01, 1.1119E+03, &
      3.1701572E+00, 6.03063E+01, 1.2909E+01, 1.0977E+03, &
      3.1860080E+00, 6.03233E+01, 1.2807E+01, 1.0837E+03, &
      3.2019380E+00, 6.03392E+01, 1.2706E+01, 1.0699E+03, &
      3.2179477E+00, 6.03542E+01, 1.2606E+01, 1.0563E+03, &
      3.2340374E+00, 6.03682E+01, 1.2507E+01, 1.0428E+03, &
      3.2502076E+00, 6.03813E+01, 1.2409E+01, 1.0296E+03, &
      3.2664587E+00, 6.03936E+01, 1.2311E+01, 1.0165E+03, &
      3.2827910E+00, 6.04051E+01, 1.2215E+01, 1.0036E+03, &
      3.2992049E+00, 6.04158E+01, 1.2119E+01, 9.9085E+02, &
      3.3157009E+00, 6.04257E+01, 1.2024E+01, 9.7829E+02, &
      3.3322794E+00, 6.04350E+01, 1.1930E+01, 9.6590E+02, &
      3.3489408E+00, 6.04435E+01, 1.1837E+01, 9.5368E+02, &
      3.3656856E+00, 6.04514E+01, 1.1745E+01, 9.4162E+02, &
      3.3825140E+00, 6.05064E+01, 1.1652E+01, 9.2967E+02, &
      3.3994265E+00, 6.05134E+01, 1.1560E+01, 9.1778E+02, &
      3.4164237E+00, 6.05195E+01, 1.1468E+01, 9.0605E+02, &
      3.4335058E+00, 6.05249E+01, 1.1377E+01, 8.9448E+02, &
      3.4506733E+00, 6.05295E+01, 1.1287E+01, 8.8306E+02, &
      3.4679267E+00, 6.05334E+01, 1.1198E+01, 8.7180E+02, &
      3.4852663E+00, 6.05367E+01, 1.1109E+01, 8.6069E+02, &
      3.5026927E+00, 6.05393E+01, 1.1022E+01, 8.4972E+02, &
      3.5202061E+00, 6.05412E+01, 1.0935E+01, 8.3891E+02, &
      3.5378072E+00, 6.05426E+01, 1.0849E+01, 8.2824E+02, &
      3.5554962E+00, 6.05434E+01, 1.0763E+01, 8.1772E+02, &
      3.5732737E+00, 6.05436E+01, 1.0678E+01, 8.0733E+02, &
      3.5911400E+00, 6.05432E+01, 1.0595E+01, 7.9709E+02, &
      3.6090957E+00, 6.05423E+01, 1.0512E+01, 7.8698E+02, &
      3.6271412E+00, 6.05409E+01, 1.0429E+01, 7.7701E+02, &
      3.6452769E+00, 6.05389E+01, 1.0347E+01, 7.6717E+02, &
      3.6635033E+00, 6.05365E+01, 1.0267E+01, 7.5746E+02, &
      3.6818208E+00, 6.05336E+01, 1.0186E+01, 7.4788E+02, &
      3.7002299E+00, 6.05303E+01, 1.0107E+01, 7.3843E+02, &
      3.7187311E+00, 6.05265E+01, 1.0028E+01, 7.2911E+02, &
      3.7373247E+00, 6.05222E+01, 9.9500E+00, 7.1991E+02, &
      3.7560114E+00, 6.05175E+01, 9.8726E+00, 7.1083E+02, &
      3.7747914E+00, 6.05343E+01, 9.7954E+00, 7.0184E+02, &
      3.7936654E+00, 6.05290E+01, 9.7185E+00, 6.9295E+02, &
      3.8126337E+00, 6.05231E+01, 9.6424E+00, 6.8417E+02, &
      3.8316969E+00, 6.05168E+01, 9.5669E+00, 6.7552E+02, &
      3.8508554E+00, 6.05100E+01, 9.4921E+00, 6.6698E+02, &
      3.8701096E+00, 6.05027E+01, 9.4179E+00, 6.5855E+02, &
      3.8894602E+00, 6.04950E+01, 9.3444E+00, 6.5023E+02, &
      3.9089075E+00, 6.04869E+01, 9.2715E+00, 6.4203E+02, &
      3.9284520E+00, 6.04784E+01, 9.1993E+00, 6.3393E+02, &
      3.9480943E+00, 6.04695E+01, 9.1277E+00, 6.2594E+02, &
      3.9678347E+00, 6.04602E+01, 9.0567E+00, 6.1806E+02, &
      3.9876739E+00, 6.04504E+01, 8.9864E+00, 6.1028E+02, &
      4.055024E+00, 6.07797E+01, 8.7546E+00, 5.8494E+02, &
      4.334821E+00, 6.05462E+01, 7.8959E+00, 4.9438E+02, &
      4.633924E+00, 6.02633E+01, 7.1311E+00, 4.1850E+02, &
      4.953664E+00, 5.99288E+01, 6.4492E+00, 3.5481E+02, &
      5.295467E+00, 5.95306E+01, 5.8260E+00, 3.0056E+02, &
      5.660855E+00, 5.90193E+01, 5.2446E+00, 2.5380E+02, &
      6.051453E+00, 5.83326E+01, 4.7294E+00, 2.1475E+02, &
      6.469004E+00, 5.73192E+01, 4.2728E+00, 1.8209E+02, &
      6.915365E+00, 5.53684E+01, 3.8570E+00, 1.5432E+02, &
      7.097944E+00, 5.35634E+01, 3.7036E+00, 1.4460E+02, &
      7.206586E+00, 5.05305E+01, 3.6172E+00, 1.3923E+02, &
      7.235557E+00, 4.69937E+01, 3.5948E+00, 1.3784E+02, &
      7.279014E+00, 5.05208E+01, 1.0455E+01, 3.8922E+02, &
      7.392525E+00, 5.36385E+01, 1.0112E+01, 3.7087E+02, &
      7.771694E+00, 5.52011E+01, 9.1507E+00, 3.1970E+02, &
      7.890649E+00, 5.41336E+01, 8.9120E+00, 3.0679E+02, &
      7.902609E+00, 5.37852E+01, 8.8889E+00, 3.0554E+02, &
      7.922370E+00, 5.25006E+01, 8.8510E+00, 3.0350E+02, &
      7.969952E+00, 5.43302E+01, 1.2125E+01, 4.1161E+02, &
      8.088906E+00, 5.60583E+01, 1.1824E+01, 3.9562E+02, &
      8.208088E+00, 5.67170E+01, 1.1537E+01, 3.8054E+02, &
      8.333722E+00, 5.66681E+01, 1.1249E+01, 3.6554E+02, &
      8.367224E+00, 5.59394E+01, 1.1175E+01, 3.6172E+02, &
      8.417478E+00, 5.70383E+01, 1.2806E+01, 4.1142E+02, &
      8.447890E+00, 5.74719E+01, 1.2738E+01, 4.0780E+02, &
      8.543112E+00, 5.83113E+01, 1.2530E+01, 3.9674E+02, &
      9.030794E+00, 6.04779E+01, 1.1524E+01, 3.4552E+02, &
      9.653919E+00, 6.18945E+01, 1.0333E+01, 2.9022E+02, &
      1.032004E+01, 6.27750E+01, 9.2524E+00, 2.4348E+02, &
      1.103212E+01, 6.33568E+01, 8.2759E+00, 2.0407E+02, &
      1.179334E+01, 6.37404E+01, 7.4018E+00, 1.7107E+02, &
      1.260708E+01, 6.39922E+01, 6.6258E+00, 1.4355E+02, &
      1.347697E+01, 6.41543E+01, 5.9363E+00, 1.2058E+02, &
      1.440688E+01, 6.42554E+01, 5.3231E+00, 1.0139E+02, &
      1.540095E+01, 6.43753E+01, 4.7588E+00, 8.5023E+01, &
      1.646362E+01, 6.44222E+01, 4.2515E+00, 7.1269E+01, &
      1.759961E+01, 6.44323E+01, 3.7894E+00, 5.9622E+01, &
      1.881398E+01, 6.43823E+01, 3.3778E+00, 4.9898E+01, &
      2.011215E+01, 6.43148E+01, 3.0136E+00, 4.1807E+01, &
      2.149988E+01, 6.42361E+01, 2.6910E+00, 3.5071E+01, &
      2.298338E+01, 6.41503E+01, 2.4051E+00, 2.9456E+01, &
      2.456923E+01, 6.40603E+01, 2.1515E+00, 2.4770E+01, &
      2.626450E+01, 6.39699E+01, 1.9262E+00, 2.0854E+01, &
      2.807676E+01, 6.38844E+01, 1.7125E+00, 1.7450E+01, &
      3.001405E+01, 6.37780E+01, 1.5200E+00, 1.4588E+01, &
      3.208502E+01, 6.36621E+01, 1.3491E+00, 1.2202E+01, &
      3.429889E+01, 6.35343E+01, 1.1961E+00, 1.0204E+01, &
      3.666551E+01, 6.33900E+01, 1.0592E+00, 8.5295E+00, &
      3.919543E+01, 6.32174E+01, 9.3498E-01, 7.1164E+00, &
      4.189992E+01, 6.29955E+01, 8.2563E-01, 5.9452E+00, &
      4.479101E+01, 6.26711E+01, 7.2944E-01, 4.9746E+00, &
      4.788159E+01, 6.20283E+01, 6.4496E-01, 4.1704E+00, &
      4.923432E+01, 6.13442E+01, 6.1279E-01, 3.8768E+00, &
      4.998791E+01, 6.01781E+01, 5.9596E-01, 3.7260E+00, &
      5.018886E+01, 5.87811E+01, 5.9160E-01, 3.6872E+00, &
      5.049030E+01, 6.01934E+01, 3.3355E+00, 1.8208E+01, &
      5.118542E+01, 6.14133E+01, 3.2652E+00, 1.7591E+01, &
      5.124388E+01, 6.14695E+01, 3.2594E+00, 1.7540E+01, &
      5.471721E+01, 6.28932E+01, 2.9371E+00, 1.4842E+01, &
      5.849270E+01, 6.34489E+01, 2.6343E+00, 1.2492E+01, &
      6.252870E+01, 6.37699E+01, 2.3606E+00, 1.0508E+01, &
      6.684318E+01, 6.39778E+01, 2.1138E+00, 8.8359E+00, &
      7.145536E+01, 6.41165E+01, 1.8907E+00, 7.4258E+00, &
      7.638578E+01, 6.42109E+01, 1.6911E+00, 6.2438E+00, &
      8.165640E+01, 6.42766E+01, 1.5105E+00, 5.2462E+00, &
      8.729069E+01, 6.43175E+01, 1.3450E+00, 4.3985E+00, &
      9.331374E+01, 6.43408E+01, 1.1979E+00, 3.6914E+00, &
      9.975239E+01, 6.43522E+01, 1.0669E+00, 3.1012E+00, &
      1.066353E+02, 6.43832E+01, 9.4693E-01, 2.5999E+00, &
      1.139931E+02, 6.43762E+01, 8.4015E-01, 2.1819E+00, &
      1.218587E+02, 6.43640E+01, 7.4547E-01, 1.8340E+00, &
      1.302669E+02, 6.43484E+01, 6.6153E-01, 1.5444E+00, &
      1.392553E+02, 6.43308E+01, 5.8709E-01, 1.3033E+00, &
      1.488640E+02, 6.43119E+01, 5.2107E-01, 1.1023E+00, &
      1.591356E+02, 6.42924E+01, 4.6251E-01, 9.3480E-01, &
      1.701159E+02, 6.42729E+01, 4.1057E-01, 7.9505E-01, &
      1.818539E+02, 6.42535E+01, 3.6449E-01, 6.7838E-01, &
      1.944018E+02, 6.42347E+01, 3.2360E-01, 5.8089E-01, &
      2.078156E+02, 6.42164E+01, 2.8733E-01, 4.9936E-01, &
      2.221548E+02, 6.41988E+01, 2.5546E-01, 4.3149E-01, &
      2.374835E+02, 6.41825E+01, 2.2781E-01, 3.7526E-01, &
      2.538699E+02, 6.41674E+01, 2.0316E-01, 3.2786E-01, &
      2.713869E+02, 6.41534E+01, 1.8119E-01, 2.8783E-01, &
      2.901126E+02, 6.41404E+01, 1.6161E-01, 2.5398E-01, &
      3.101304E+02, 6.41284E+01, 1.4415E-01, 2.2527E-01, &
      3.315294E+02, 6.41173E+01, 1.2859E-01, 2.0088E-01, &
      3.544049E+02, 6.41071E+01, 1.1471E-01, 1.8008E-01, &
      3.788588E+02, 6.40977E+01, 1.0234E-01, 1.6230E-01, &
      4.050001E+02, 6.40890E+01, 9.1305E-02, 1.4704E-01, &
      4.329451E+02, 6.40810E+01, 8.1465E-02, 1.3388E-01]
   case(65)
      frel = -6.2340E-01
      allocate(nist_tab(3612))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      8.428000E-03, 1.29810E+01, 0.0000E+00, 8.1435E-05, &
      8.557000E-03, 1.29472E+01, 0.0000E+00, 8.7650E-05, &
      8.591400E-03, 1.29206E+01, 0.0000E+00, 8.9365E-05, &
      8.643000E-03, 1.29376E+01, 4.4183E-02, 1.3535E+03, &
      8.772000E-03, 1.29409E+01, 4.3243E-02, 1.3053E+03, &
      9.212000E-03, 1.28773E+01, 4.0279E-02, 1.1577E+03, &
      9.353000E-03, 1.28122E+01, 3.9401E-02, 1.1154E+03, &
      9.390600E-03, 1.27429E+01, 3.9172E-02, 1.1045E+03, &
      9.447000E-03, 1.28081E+01, 1.6984E-01, 4.7604E+03, &
      9.588000E-03, 1.28578E+01, 1.6622E-01, 4.5903E+03, &
      1.069000E-02, 1.28370E+01, 1.4498E-01, 3.5911E+03, &
      1.142761E-02, 1.27683E+01, 1.4004E-01, 3.2446E+03, &
      1.221612E-02, 1.26780E+01, 1.4113E-01, 3.0589E+03, &
      1.305903E-02, 1.25663E+01, 1.4746E-01, 2.9899E+03, &
      1.396010E-02, 1.24303E+01, 1.5886E-01, 3.0132E+03, &
      1.492335E-02, 1.22650E+01, 1.7558E-01, 3.1152E+03, &
      1.595306E-02, 1.20622E+01, 1.9819E-01, 3.2894E+03, &
      1.705382E-02, 1.18097E+01, 2.2759E-01, 3.5336E+03, &
      1.823053E-02, 1.14875E+01, 2.6496E-01, 3.8482E+03, &
      1.948844E-02, 1.10610E+01, 3.1175E-01, 4.2356E+03, &
      2.083314E-02, 1.04620E+01, 3.6973E-01, 4.6991E+03, &
      2.227063E-02, 9.52814E+00, 4.4091E-01, 5.2421E+03, &
      2.380730E-02, 7.69941E+00, 5.2762E-01, 5.8681E+03, &
      2.489200E-02, 4.21220E+00, 5.9540E-01, 6.3334E+03, &
      2.527300E-02, -4.41891E-01, 6.2050E-01, 6.5008E+03, &
      2.537460E-02, -6.10700E+00, 6.2730E-01, 6.5458E+03, &
      2.545001E-02, -3.53795E+00, 1.1903E+01, 1.2384E+05, &
      2.552700E-02, -8.75725E-02, 1.1837E+01, 1.2278E+05, &
      2.590800E-02, 5.18255E+00, 1.1522E+01, 1.1775E+05, &
      2.720606E-02, 1.02633E+01, 1.0564E+01, 1.0281E+05, &
      2.908327E-02, 1.30050E+01, 9.3993E+00, 8.5574E+04, &
      3.109002E-02, 1.45382E+01, 8.2623E+00, 7.0367E+04, &
      3.323523E-02, 1.53677E+01, 7.2428E+00, 5.7703E+04, &
      3.552846E-02, 1.57615E+01, 6.4036E+00, 4.7723E+04, &
      3.797993E-02, 1.58784E+01, 5.7661E+00, 4.0199E+04, &
      3.822000E-02, 1.58782E+01, 5.7162E+00, 3.9601E+04, &
      3.880500E-02, 1.58646E+01, 5.6032E+00, 3.8233E+04, &
      3.896100E-02, 1.58469E+01, 5.5751E+00, 3.7888E+04, &
      3.919500E-02, 1.58610E+01, 5.5671E+00, 3.7608E+04, &
      3.978000E-02, 1.58659E+01, 5.4739E+00, 3.6435E+04, &
      4.060054E-02, 1.58529E+01, 5.3603E+00, 3.4958E+04, &
      4.340198E-02, 1.57544E+01, 5.1021E+00, 3.1126E+04, &
      4.639671E-02, 1.56225E+01, 4.9983E+00, 2.8525E+04, &
      4.959809E-02, 1.55046E+01, 5.0180E+00, 2.6789E+04, &
      5.302035E-02, 1.54090E+01, 5.1339E+00, 2.5638E+04, &
      5.667876E-02, 1.53482E+01, 5.3289E+00, 2.4895E+04, &
      6.058959E-02, 1.53299E+01, 5.5850E+00, 2.4407E+04, &
      6.477028E-02, 1.53572E+01, 5.8872E+00, 2.4067E+04, &
      6.923942E-02, 1.54298E+01, 6.2227E+00, 2.3796E+04, &
      7.401695E-02, 1.55456E+01, 6.5815E+00, 2.3544E+04, &
      7.912411E-02, 1.57156E+01, 6.9551E+00, 2.3275E+04, &
      8.458368E-02, 1.59045E+01, 7.3372E+00, 2.2968E+04, &
      9.041995E-02, 1.61200E+01, 7.7227E+00, 2.2615E+04, &
      9.665893E-02, 1.63527E+01, 8.1079E+00, 2.2210E+04, &
      1.0000000E-01, 1.90901E+01, 8.3031E+00, 2.1986E+04, &
      1.0050000E-01, 1.91079E+01, 8.3317E+00, 2.1951E+04, &
      1.0100250E-01, 1.91257E+01, 8.3603E+00, 2.1917E+04, &
      1.0150751E-01, 1.91434E+01, 8.3888E+00, 2.1883E+04, &
      1.0201505E-01, 1.91611E+01, 8.4173E+00, 2.1848E+04, &
      1.0252513E-01, 1.91787E+01, 8.4458E+00, 2.1813E+04, &
      1.0303775E-01, 1.91962E+01, 8.4743E+00, 2.1777E+04, &
      1.0355294E-01, 1.92136E+01, 8.5028E+00, 2.1742E+04, &
      1.0407070E-01, 1.92309E+01, 8.5312E+00, 2.1706E+04, &
      1.0459106E-01, 1.92481E+01, 8.5596E+00, 2.1670E+04, &
      1.0511401E-01, 1.92651E+01, 8.5880E+00, 2.1634E+04, &
      1.0563958E-01, 1.92821E+01, 8.6164E+00, 2.1597E+04, &
      1.0616778E-01, 1.92988E+01, 8.6447E+00, 2.1560E+04, &
      1.0669862E-01, 1.93154E+01, 8.6731E+00, 2.1523E+04, &
      1.0723211E-01, 1.93318E+01, 8.7014E+00, 2.1486E+04, &
      1.0776827E-01, 1.93480E+01, 8.7296E+00, 2.1449E+04, &
      1.0830712E-01, 1.93639E+01, 8.7579E+00, 2.1411E+04, &
      1.0884865E-01, 1.93797E+01, 8.7861E+00, 2.1373E+04, &
      1.0939289E-01, 1.93951E+01, 8.8143E+00, 2.1335E+04, &
      1.0993986E-01, 1.94103E+01, 8.8425E+00, 2.1297E+04, &
      1.1048956E-01, 1.94252E+01, 8.8707E+00, 2.1258E+04, &
      1.1104201E-01, 1.94397E+01, 8.8988E+00, 2.1220E+04, &
      1.1159722E-01, 1.94539E+01, 8.9269E+00, 2.1181E+04, &
      1.1215520E-01, 1.94677E+01, 8.9550E+00, 2.1142E+04, &
      1.1271598E-01, 1.94811E+01, 8.9830E+00, 2.1103E+04, &
      1.1327956E-01, 1.94940E+01, 9.0110E+00, 2.1063E+04, &
      1.1384596E-01, 1.95065E+01, 9.0390E+00, 2.1023E+04, &
      1.1441519E-01, 1.95185E+01, 9.0670E+00, 2.0984E+04, &
      1.1498726E-01, 1.95299E+01, 9.0950E+00, 2.0944E+04, &
      1.1556220E-01, 1.95408E+01, 9.1229E+00, 2.0903E+04, &
      1.1614001E-01, 1.95510E+01, 9.1508E+00, 2.0863E+04, &
      1.1672071E-01, 1.95605E+01, 9.1787E+00, 2.0822E+04, &
      1.1730431E-01, 1.95699E+01, 9.2065E+00, 2.0782E+04, &
      1.1789083E-01, 1.95780E+01, 9.2343E+00, 2.0741E+04, &
      1.1848029E-01, 1.95852E+01, 9.2621E+00, 2.0700E+04, &
      1.1907269E-01, 1.95916E+01, 9.2899E+00, 2.0659E+04, &
      1.1966805E-01, 1.95970E+01, 9.3177E+00, 2.0617E+04, &
      1.2026639E-01, 1.96013E+01, 9.3454E+00, 2.0576E+04, &
      1.2086772E-01, 1.96046E+01, 9.3731E+00, 2.0534E+04, &
      1.2147206E-01, 1.96066E+01, 9.4008E+00, 2.0492E+04, &
      1.2207942E-01, 1.96074E+01, 9.4284E+00, 2.0450E+04, &
      1.2268982E-01, 1.96068E+01, 9.4560E+00, 2.0408E+04, &
      1.2330327E-01, 1.96047E+01, 9.4836E+00, 2.0366E+04, &
      1.2391979E-01, 1.96010E+01, 9.5112E+00, 2.0323E+04, &
      1.2453939E-01, 1.95955E+01, 9.5388E+00, 2.0281E+04, &
      1.2516208E-01, 1.95881E+01, 9.5663E+00, 2.0238E+04, &
      1.2578789E-01, 1.95786E+01, 9.5939E+00, 2.0196E+04, &
      1.2641683E-01, 1.95668E+01, 9.6214E+00, 2.0153E+04, &
      1.2704892E-01, 1.95526E+01, 9.6488E+00, 2.0110E+04, &
      1.2768416E-01, 1.95357E+01, 9.6763E+00, 2.0067E+04, &
      1.2832258E-01, 1.95159E+01, 9.7037E+00, 2.0023E+04, &
      1.2896419E-01, 1.94927E+01, 9.7312E+00, 1.9980E+04, &
      1.2960902E-01, 1.94660E+01, 9.7586E+00, 1.9937E+04, &
      1.3025706E-01, 1.94354E+01, 9.7859E+00, 1.9893E+04, &
      1.3090835E-01, 1.94003E+01, 9.8133E+00, 1.9850E+04, &
      1.3156289E-01, 1.93604E+01, 9.8407E+00, 1.9806E+04, &
      1.3222070E-01, 1.93150E+01, 9.8680E+00, 1.9762E+04, &
      1.3288181E-01, 1.92636E+01, 9.8953E+00, 1.9718E+04, &
      1.3354621E-01, 1.92052E+01, 9.9226E+00, 1.9674E+04, &
      1.3421395E-01, 1.91390E+01, 9.9499E+00, 1.9630E+04, &
      1.3488502E-01, 1.90640E+01, 9.9772E+00, 1.9586E+04, &
      1.3555944E-01, 1.89787E+01, 1.0004E+01, 1.9542E+04, &
      1.3623724E-01, 1.88818E+01, 1.0032E+01, 1.9498E+04, &
      1.3691842E-01, 1.87711E+01, 1.0059E+01, 1.9453E+04, &
      1.3760302E-01, 1.86444E+01, 1.0086E+01, 1.9409E+04, &
      1.3829103E-01, 1.84986E+01, 1.0113E+01, 1.9365E+04, &
      1.3898249E-01, 1.83300E+01, 1.0141E+01, 1.9320E+04, &
      1.3967740E-01, 1.81334E+01, 1.0168E+01, 1.9276E+04, &
      1.4037579E-01, 1.79023E+01, 1.0195E+01, 1.9231E+04, &
      1.4107766E-01, 1.76275E+01, 1.0222E+01, 1.9186E+04, &
      1.4178305E-01, 1.72961E+01, 1.0249E+01, 1.9142E+04, &
      1.4249197E-01, 1.68889E+01, 1.0277E+01, 1.9097E+04, &
      1.4320443E-01, 1.63762E+01, 1.0304E+01, 1.9052E+04, &
      1.4392045E-01, 1.57080E+01, 1.0331E+01, 1.9007E+04, &
      1.4464005E-01, 1.47917E+01, 1.0358E+01, 1.8963E+04, &
      1.4536325E-01, 1.34255E+01, 1.0385E+01, 1.8918E+04, &
      1.4609007E-01, 1.10209E+01, 1.0412E+01, 1.8873E+04, &
      1.4682052E-01, 3.47099E+00, 1.0440E+01, 1.8828E+04, &
      1.4695355E-01, -3.34446E+00, 1.0444E+01, 1.8820E+04, &
      1.4704645E-01, -3.13135E+00, 2.6773E+01, 4.8210E+04, &
      1.4755462E-01, 1.05112E+01, 2.6136E+01, 4.6900E+04, &
      1.4829239E-01, 1.55232E+01, 2.5261E+01, 4.5105E+04, &
      1.4903386E-01, 1.82741E+01, 2.4438E+01, 4.3419E+04, &
      1.4977903E-01, 2.01565E+01, 2.3664E+01, 4.1834E+04, &
      1.5052792E-01, 2.15618E+01, 2.2936E+01, 4.0346E+04, &
      1.5128056E-01, 2.26604E+01, 2.2252E+01, 3.8948E+04, &
      1.5203696E-01, 2.35434E+01, 2.1608E+01, 3.7633E+04, &
      1.5279715E-01, 2.42662E+01, 2.1004E+01, 3.6398E+04, &
      1.5356113E-01, 2.48653E+01, 2.0435E+01, 3.5237E+04, &
      1.5432894E-01, 2.53661E+01, 1.9901E+01, 3.4145E+04, &
      1.5510058E-01, 2.57872E+01, 1.9399E+01, 3.3119E+04, &
      1.5587609E-01, 2.61426E+01, 1.8928E+01, 3.2153E+04, &
      1.5665547E-01, 2.64432E+01, 1.8485E+01, 3.1245E+04, &
      1.5743875E-01, 2.66977E+01, 1.8070E+01, 3.0391E+04, &
      1.5822594E-01, 2.69127E+01, 1.7680E+01, 2.9587E+04, &
      1.5901707E-01, 2.70940E+01, 1.7314E+01, 2.8831E+04, &
      1.5981215E-01, 2.72460E+01, 1.6970E+01, 2.8118E+04, &
      1.6061121E-01, 2.73723E+01, 1.6648E+01, 2.7448E+04, &
      1.6141427E-01, 2.74756E+01, 1.6347E+01, 2.6816E+04, &
      1.6222134E-01, 2.75581E+01, 1.6054E+01, 2.6204E+04, &
      1.6303245E-01, 2.76224E+01, 1.5784E+01, 2.5635E+04, &
      1.6384761E-01, 2.76730E+01, 1.5536E+01, 2.5108E+04, &
      1.6466685E-01, 2.77135E+01, 1.5310E+01, 2.4620E+04, &
      1.6549018E-01, 2.77462E+01, 1.5103E+01, 2.4166E+04, &
      1.6631763E-01, 2.77731E+01, 1.4913E+01, 2.3743E+04, &
      1.6714922E-01, 2.77956E+01, 1.4738E+01, 2.3348E+04, &
      1.6798497E-01, 2.78150E+01, 1.4578E+01, 2.2979E+04, &
      1.6882489E-01, 2.78322E+01, 1.4430E+01, 2.2634E+04, &
      1.6966902E-01, 2.78480E+01, 1.4295E+01, 2.2309E+04, &
      1.7051736E-01, 2.78629E+01, 1.4170E+01, 2.2005E+04, &
      1.7136995E-01, 2.78775E+01, 1.4055E+01, 2.1718E+04, &
      1.7222680E-01, 2.78921E+01, 1.3950E+01, 2.1447E+04, &
      1.7308793E-01, 2.79072E+01, 1.3852E+01, 2.1192E+04, &
      1.7395337E-01, 2.79231E+01, 1.3763E+01, 2.0950E+04, &
      1.7482314E-01, 2.79398E+01, 1.3680E+01, 2.0720E+04, &
      1.7569726E-01, 2.79578E+01, 1.3604E+01, 2.0503E+04, &
      1.7657574E-01, 2.79771E+01, 1.3531E+01, 2.0292E+04, &
      1.7745862E-01, 2.79983E+01, 1.3428E+01, 2.0037E+04, &
      1.7834591E-01, 2.80193E+01, 1.3330E+01, 1.9792E+04, &
      1.7923764E-01, 2.80402E+01, 1.3237E+01, 1.9556E+04, &
      1.8013383E-01, 2.80611E+01, 1.3148E+01, 1.9328E+04, &
      1.8103450E-01, 2.80821E+01, 1.3062E+01, 1.9106E+04, &
      1.8193967E-01, 2.81031E+01, 1.2981E+01, 1.8892E+04, &
      1.8284937E-01, 2.81242E+01, 1.2902E+01, 1.8685E+04, &
      1.8376362E-01, 2.81455E+01, 1.2826E+01, 1.8483E+04, &
      1.8468244E-01, 2.81669E+01, 1.2754E+01, 1.8287E+04, &
      1.8560585E-01, 2.81885E+01, 1.2684E+01, 1.8096E+04, &
      1.8653388E-01, 2.82102E+01, 1.2616E+01, 1.7910E+04, &
      1.8746655E-01, 2.82321E+01, 1.2551E+01, 1.7728E+04, &
      1.8840388E-01, 2.82542E+01, 1.2487E+01, 1.7551E+04, &
      1.8934590E-01, 2.82763E+01, 1.2426E+01, 1.7378E+04, &
      1.9029263E-01, 2.82986E+01, 1.2367E+01, 1.7209E+04, &
      1.9124409E-01, 2.83210E+01, 1.2309E+01, 1.7044E+04, &
      1.9220031E-01, 2.83435E+01, 1.2253E+01, 1.6882E+04, &
      1.9316131E-01, 2.83661E+01, 1.2198E+01, 1.6723E+04, &
      1.9412712E-01, 2.83888E+01, 1.2145E+01, 1.6567E+04, &
      1.9509776E-01, 2.84115E+01, 1.2093E+01, 1.6414E+04, &
      1.9607325E-01, 2.84343E+01, 1.2043E+01, 1.6264E+04, &
      1.9705361E-01, 2.84571E+01, 1.1993E+01, 1.6117E+04, &
      1.9803888E-01, 2.84799E+01, 1.1945E+01, 1.5972E+04, &
      1.9902907E-01, 2.85027E+01, 1.1898E+01, 1.5830E+04, &
      2.0002422E-01, 2.85254E+01, 1.1851E+01, 1.5690E+04, &
      2.0102434E-01, 2.85481E+01, 1.1806E+01, 1.5552E+04, &
      2.0202946E-01, 2.85708E+01, 1.1761E+01, 1.5416E+04, &
      2.0303961E-01, 2.85934E+01, 1.1718E+01, 1.5282E+04, &
      2.0405481E-01, 2.86158E+01, 1.1675E+01, 1.5151E+04, &
      2.0507508E-01, 2.86382E+01, 1.1632E+01, 1.5021E+04, &
      2.0610046E-01, 2.86604E+01, 1.1591E+01, 1.4893E+04, &
      2.0713096E-01, 2.86825E+01, 1.1550E+01, 1.4766E+04, &
      2.0816661E-01, 2.87045E+01, 1.1510E+01, 1.4642E+04, &
      2.0920745E-01, 2.87262E+01, 1.1470E+01, 1.4519E+04, &
      2.1025348E-01, 2.87478E+01, 1.1431E+01, 1.4397E+04, &
      2.1130475E-01, 2.87691E+01, 1.1393E+01, 1.4278E+04, &
      2.1236128E-01, 2.87903E+01, 1.1355E+01, 1.4159E+04, &
      2.1342308E-01, 2.88112E+01, 1.1317E+01, 1.4042E+04, &
      2.1449020E-01, 2.88318E+01, 1.1280E+01, 1.3927E+04, &
      2.1556265E-01, 2.88522E+01, 1.1244E+01, 1.3813E+04, &
      2.1664046E-01, 2.88723E+01, 1.1208E+01, 1.3700E+04, &
      2.1772366E-01, 2.88921E+01, 1.1172E+01, 1.3589E+04, &
      2.1881228E-01, 2.89116E+01, 1.1137E+01, 1.3479E+04, &
      2.1990634E-01, 2.89308E+01, 1.1102E+01, 1.3370E+04, &
      2.2100588E-01, 2.89497E+01, 1.1068E+01, 1.3262E+04, &
      2.2211090E-01, 2.89682E+01, 1.1034E+01, 1.3156E+04, &
      2.2322146E-01, 2.89864E+01, 1.1000E+01, 1.3051E+04, &
      2.2433757E-01, 2.90041E+01, 1.0967E+01, 1.2947E+04, &
      2.2545925E-01, 2.90215E+01, 1.0934E+01, 1.2844E+04, &
      2.2658655E-01, 2.90385E+01, 1.0902E+01, 1.2742E+04, &
      2.2771948E-01, 2.90551E+01, 1.0870E+01, 1.2641E+04, &
      2.2885808E-01, 2.90713E+01, 1.0838E+01, 1.2541E+04, &
      2.3000237E-01, 2.90870E+01, 1.0807E+01, 1.2443E+04, &
      2.3115238E-01, 2.91023E+01, 1.0776E+01, 1.2345E+04, &
      2.3230814E-01, 2.91171E+01, 1.0745E+01, 1.2249E+04, &
      2.3346969E-01, 2.91314E+01, 1.0714E+01, 1.2154E+04, &
      2.3463703E-01, 2.91452E+01, 1.0684E+01, 1.2059E+04, &
      2.3581022E-01, 2.91585E+01, 1.0654E+01, 1.1966E+04, &
      2.3698927E-01, 2.91712E+01, 1.0625E+01, 1.1873E+04, &
      2.3817422E-01, 2.91834E+01, 1.0596E+01, 1.1782E+04, &
      2.3936509E-01, 2.91951E+01, 1.0567E+01, 1.1691E+04, &
      2.4056191E-01, 2.92061E+01, 1.0538E+01, 1.1602E+04, &
      2.4176472E-01, 2.92165E+01, 1.0510E+01, 1.1513E+04, &
      2.4297355E-01, 2.92263E+01, 1.0482E+01, 1.1426E+04, &
      2.4418841E-01, 2.92354E+01, 1.0455E+01, 1.1339E+04, &
      2.4540936E-01, 2.92439E+01, 1.0427E+01, 1.1253E+04, &
      2.4663640E-01, 2.92516E+01, 1.0400E+01, 1.1168E+04, &
      2.4786959E-01, 2.92585E+01, 1.0374E+01, 1.1084E+04, &
      2.4910893E-01, 2.92647E+01, 1.0347E+01, 1.1001E+04, &
      2.5035448E-01, 2.92701E+01, 1.0321E+01, 1.0919E+04, &
      2.5160625E-01, 2.92746E+01, 1.0296E+01, 1.0837E+04, &
      2.5286428E-01, 2.92781E+01, 1.0270E+01, 1.0757E+04, &
      2.5412860E-01, 2.92807E+01, 1.0245E+01, 1.0677E+04, &
      2.5539925E-01, 2.92823E+01, 1.0220E+01, 1.0598E+04, &
      2.5667624E-01, 2.92828E+01, 1.0196E+01, 1.0520E+04, &
      2.5795962E-01, 2.92821E+01, 1.0172E+01, 1.0443E+04, &
      2.5924942E-01, 2.92802E+01, 1.0148E+01, 1.0367E+04, &
      2.6054567E-01, 2.92768E+01, 1.0124E+01, 1.0291E+04, &
      2.6184840E-01, 2.92720E+01, 1.0101E+01, 1.0217E+04, &
      2.6315764E-01, 2.92656E+01, 1.0078E+01, 1.0143E+04, &
      2.6447343E-01, 2.92574E+01, 1.0056E+01, 1.0070E+04, &
      2.6579579E-01, 2.92471E+01, 1.0033E+01, 9.9977E+03, &
      2.6712477E-01, 2.92346E+01, 1.0011E+01, 9.9263E+03, &
      2.6846040E-01, 2.92196E+01, 9.9898E+00, 9.8556E+03, &
      2.6980270E-01, 2.92016E+01, 9.9686E+00, 9.7858E+03, &
      2.7115171E-01, 2.91802E+01, 9.9476E+00, 9.7167E+03, &
      2.7250747E-01, 2.91547E+01, 9.9270E+00, 9.6483E+03, &
      2.7387001E-01, 2.91243E+01, 9.9068E+00, 9.5808E+03, &
      2.7523936E-01, 2.90876E+01, 9.8868E+00, 9.5140E+03, &
      2.7661556E-01, 2.90430E+01, 9.8672E+00, 9.4479E+03, &
      2.7799863E-01, 2.89877E+01, 9.8480E+00, 9.3826E+03, &
      2.7938863E-01, 2.89170E+01, 9.8290E+00, 9.3180E+03, &
      2.8078557E-01, 2.88223E+01, 9.8104E+00, 9.2541E+03, &
      2.8218950E-01, 2.86841E+01, 9.7922E+00, 9.1910E+03, &
      2.8360044E-01, 2.84403E+01, 9.7743E+00, 9.1286E+03, &
      2.8474920E-01, 2.78284E+01, 9.7600E+00, 9.0785E+03, &
      2.8501845E-01, 2.68959E+01, 1.0880E+01, 1.0110E+04, &
      2.8525080E-01, 2.78290E+01, 1.0878E+01, 1.0100E+04, &
      2.8644354E-01, 2.84574E+01, 1.0867E+01, 1.0048E+04, &
      2.8787576E-01, 2.87078E+01, 1.0854E+01, 9.9866E+03, &
      2.8931514E-01, 2.88569E+01, 1.0842E+01, 9.9257E+03, &
      2.9076171E-01, 2.89640E+01, 1.0830E+01, 9.8655E+03, &
      2.9221552E-01, 2.90477E+01, 1.0819E+01, 9.8060E+03, &
      2.9367660E-01, 2.91163E+01, 1.0807E+01, 9.7471E+03, &
      2.9514498E-01, 2.91740E+01, 1.0797E+01, 9.6890E+03, &
      2.9662071E-01, 2.92234E+01, 1.0786E+01, 9.6314E+03, &
      2.9810381E-01, 2.92659E+01, 1.0776E+01, 9.5746E+03, &
      2.9959433E-01, 2.93022E+01, 1.0766E+01, 9.5184E+03, &
      3.0109230E-01, 2.93327E+01, 1.0757E+01, 9.4628E+03, &
      3.0259776E-01, 2.93572E+01, 1.0748E+01, 9.4078E+03, &
      3.0411075E-01, 2.93748E+01, 1.0739E+01, 9.3535E+03, &
      3.0563130E-01, 2.93832E+01, 1.0731E+01, 9.2998E+03, &
      3.0715946E-01, 2.93770E+01, 1.0723E+01, 9.2467E+03, &
      3.0869526E-01, 2.93374E+01, 1.0715E+01, 9.1942E+03, &
      3.0988670E-01, 2.91967E+01, 1.0710E+01, 9.1540E+03, &
      3.1023873E-01, 2.89823E+01, 1.1041E+01, 9.4267E+03, &
      3.1051331E-01, 2.92097E+01, 1.1040E+01, 9.4176E+03, &
      3.1178993E-01, 2.94082E+01, 1.1036E+01, 9.3754E+03, &
      3.1334888E-01, 2.95122E+01, 1.1031E+01, 9.3246E+03, &
      3.1491562E-01, 2.95859E+01, 1.1027E+01, 9.2744E+03, &
      3.1649020E-01, 2.96467E+01, 1.1022E+01, 9.2248E+03, &
      3.1807265E-01, 2.97001E+01, 1.1018E+01, 9.1757E+03, &
      3.1966301E-01, 2.97486E+01, 1.1015E+01, 9.1271E+03, &
      3.2126133E-01, 2.97936E+01, 1.1011E+01, 9.0790E+03, &
      3.2286764E-01, 2.98359E+01, 1.1008E+01, 9.0314E+03, &
      3.2448197E-01, 2.98761E+01, 1.1006E+01, 8.9843E+03, &
      3.2610438E-01, 2.99146E+01, 1.1003E+01, 8.9377E+03, &
      3.2773491E-01, 2.99516E+01, 1.1001E+01, 8.8916E+03, &
      3.2937358E-01, 2.99874E+01, 1.0999E+01, 8.8459E+03, &
      3.3102045E-01, 3.00221E+01, 1.0998E+01, 8.8007E+03, &
      3.3267555E-01, 3.00558E+01, 1.0997E+01, 8.7560E+03, &
      3.3433893E-01, 3.00887E+01, 1.0996E+01, 8.7117E+03, &
      3.3601062E-01, 3.01208E+01, 1.0995E+01, 8.6679E+03, &
      3.3769068E-01, 3.01523E+01, 1.0995E+01, 8.6245E+03, &
      3.3937913E-01, 3.01831E+01, 1.0994E+01, 8.5815E+03, &
      3.4107602E-01, 3.02133E+01, 1.0995E+01, 8.5389E+03, &
      3.4278140E-01, 3.02431E+01, 1.0995E+01, 8.4968E+03, &
      3.4449531E-01, 3.02723E+01, 1.0996E+01, 8.4550E+03, &
      3.4621779E-01, 3.03010E+01, 1.0996E+01, 8.4136E+03, &
      3.4794888E-01, 3.03293E+01, 1.0998E+01, 8.3726E+03, &
      3.4968862E-01, 3.03572E+01, 1.0999E+01, 8.3320E+03, &
      3.5143706E-01, 3.03846E+01, 1.1000E+01, 8.2918E+03, &
      3.5319425E-01, 3.04117E+01, 1.1002E+01, 8.2519E+03, &
      3.5496022E-01, 3.04383E+01, 1.1004E+01, 8.2123E+03, &
      3.5673502E-01, 3.04645E+01, 1.1006E+01, 8.1731E+03, &
      3.5851870E-01, 3.04903E+01, 1.1009E+01, 8.1342E+03, &
      3.6031129E-01, 3.05156E+01, 1.1011E+01, 8.0957E+03, &
      3.6211285E-01, 3.05405E+01, 1.1014E+01, 8.0574E+03, &
      3.6392341E-01, 3.05649E+01, 1.1017E+01, 8.0195E+03, &
      3.6574303E-01, 3.05887E+01, 1.1020E+01, 7.9818E+03, &
      3.6757174E-01, 3.06120E+01, 1.1023E+01, 7.9445E+03, &
      3.6940960E-01, 3.06346E+01, 1.1026E+01, 7.9074E+03, &
      3.7125665E-01, 3.06565E+01, 1.1030E+01, 7.8706E+03, &
      3.7311293E-01, 3.06775E+01, 1.1034E+01, 7.8341E+03, &
      3.7497850E-01, 3.06976E+01, 1.1037E+01, 7.7978E+03, &
      3.7685339E-01, 3.07166E+01, 1.1041E+01, 7.7618E+03, &
      3.7873766E-01, 3.07342E+01, 1.1045E+01, 7.7261E+03, &
      3.8063135E-01, 3.07502E+01, 1.1049E+01, 7.6905E+03, &
      3.8253450E-01, 3.07641E+01, 1.1054E+01, 7.6552E+03, &
      3.8444718E-01, 3.07754E+01, 1.1058E+01, 7.6201E+03, &
      3.8636941E-01, 3.07832E+01, 1.1062E+01, 7.5853E+03, &
      3.8830126E-01, 3.07863E+01, 1.1067E+01, 7.5506E+03, &
      3.9024276E-01, 3.07822E+01, 1.1071E+01, 7.5162E+03, &
      3.9219398E-01, 3.07669E+01, 1.1076E+01, 7.4819E+03, &
      3.9415495E-01, 3.07304E+01, 1.1080E+01, 7.4478E+03, &
      3.9612572E-01, 3.06395E+01, 1.1085E+01, 7.4139E+03, &
      3.9739863E-01, 3.04502E+01, 1.1088E+01, 7.3922E+03, &
      3.9810635E-01, 3.03141E+01, 1.1614E+01, 7.7286E+03, &
      3.9840134E-01, 3.04670E+01, 1.1614E+01, 7.7235E+03, &
      4.0009688E-01, 3.07421E+01, 1.1619E+01, 7.6941E+03, &
      4.0209737E-01, 3.08843E+01, 1.1625E+01, 7.6597E+03, &
      4.0410785E-01, 3.09841E+01, 1.1631E+01, 7.6254E+03, &
      4.0612839E-01, 3.10659E+01, 1.1637E+01, 7.5914E+03, &
      4.0815904E-01, 3.11378E+01, 1.1643E+01, 7.5574E+03, &
      4.1019983E-01, 3.12034E+01, 1.1649E+01, 7.5236E+03, &
      4.1225083E-01, 3.12647E+01, 1.1654E+01, 7.4899E+03, &
      4.1431208E-01, 3.13229E+01, 1.1660E+01, 7.4564E+03, &
      4.1638364E-01, 3.13788E+01, 1.1666E+01, 7.4229E+03, &
      4.1846556E-01, 3.14328E+01, 1.1671E+01, 7.3896E+03, &
      4.2055789E-01, 3.14855E+01, 1.1677E+01, 7.3564E+03, &
      4.2266068E-01, 3.15370E+01, 1.1682E+01, 7.3233E+03, &
      4.2477398E-01, 3.15875E+01, 1.1688E+01, 7.2903E+03, &
      4.2689785E-01, 3.16373E+01, 1.1693E+01, 7.2573E+03, &
      4.2903234E-01, 3.16865E+01, 1.1698E+01, 7.2245E+03, &
      4.3117750E-01, 3.17351E+01, 1.1703E+01, 7.1917E+03, &
      4.3333339E-01, 3.17833E+01, 1.1708E+01, 7.1591E+03, &
      4.3550006E-01, 3.18312E+01, 1.1713E+01, 7.1265E+03, &
      4.3767756E-01, 3.18787E+01, 1.1718E+01, 7.0939E+03, &
      4.3986595E-01, 3.19260E+01, 1.1723E+01, 7.0614E+03, &
      4.4206528E-01, 3.19700E+01, 1.1727E+01, 7.0290E+03, &
      4.4427560E-01, 3.20169E+01, 1.1731E+01, 6.9966E+03, &
      4.4649698E-01, 3.20636E+01, 1.1735E+01, 6.9643E+03, &
      4.4872947E-01, 3.21102E+01, 1.1739E+01, 6.9320E+03, &
      4.5097311E-01, 3.21567E+01, 1.1743E+01, 6.8997E+03, &
      4.5322798E-01, 3.22031E+01, 1.1746E+01, 6.8674E+03, &
      4.5549412E-01, 3.22494E+01, 1.1750E+01, 6.8352E+03, &
      4.5777159E-01, 3.22956E+01, 1.1753E+01, 6.8030E+03, &
      4.6006045E-01, 3.23417E+01, 1.1756E+01, 6.7708E+03, &
      4.6236075E-01, 3.23878E+01, 1.1758E+01, 6.7387E+03, &
      4.6467255E-01, 3.24338E+01, 1.1761E+01, 6.7065E+03, &
      4.6699592E-01, 3.24798E+01, 1.1763E+01, 6.6744E+03, &
      4.6933090E-01, 3.25257E+01, 1.1764E+01, 6.6423E+03, &
      4.7167755E-01, 3.25716E+01, 1.1766E+01, 6.6102E+03, &
      4.7403594E-01, 3.26175E+01, 1.1767E+01, 6.5781E+03, &
      4.7640612E-01, 3.26633E+01, 1.1768E+01, 6.5460E+03, &
      4.7878815E-01, 3.27091E+01, 1.1769E+01, 6.5139E+03, &
      4.8118209E-01, 3.27549E+01, 1.1769E+01, 6.4817E+03, &
      4.8358800E-01, 3.28006E+01, 1.1770E+01, 6.4496E+03, &
      4.8600594E-01, 3.28462E+01, 1.1769E+01, 6.4174E+03, &
      4.8843597E-01, 3.28918E+01, 1.1769E+01, 6.3852E+03, &
      4.9087815E-01, 3.29374E+01, 1.1768E+01, 6.3531E+03, &
      4.9333254E-01, 3.29829E+01, 1.1767E+01, 6.3209E+03, &
      4.9579920E-01, 3.30283E+01, 1.1765E+01, 6.2888E+03, &
      4.9827820E-01, 3.30737E+01, 1.1764E+01, 6.2566E+03, &
      5.0076959E-01, 3.31190E+01, 1.1762E+01, 6.2244E+03, &
      5.0327344E-01, 3.31642E+01, 1.1759E+01, 6.1922E+03, &
      5.0578980E-01, 3.32094E+01, 1.1756E+01, 6.1601E+03, &
      5.0831875E-01, 3.32545E+01, 1.1753E+01, 6.1279E+03, &
      5.1086035E-01, 3.32995E+01, 1.1750E+01, 6.0957E+03, &
      5.1341465E-01, 3.33444E+01, 1.1746E+01, 6.0635E+03, &
      5.1598172E-01, 3.33892E+01, 1.1742E+01, 6.0313E+03, &
      5.1856163E-01, 3.34339E+01, 1.1738E+01, 5.9990E+03, &
      5.2115444E-01, 3.34784E+01, 1.1733E+01, 5.9668E+03, &
      5.2376021E-01, 3.35229E+01, 1.1728E+01, 5.9346E+03, &
      5.2637901E-01, 3.35672E+01, 1.1722E+01, 5.9024E+03, &
      5.2901091E-01, 3.36114E+01, 1.1717E+01, 5.8701E+03, &
      5.3165596E-01, 3.36555E+01, 1.1710E+01, 5.8379E+03, &
      5.3431424E-01, 3.36994E+01, 1.1704E+01, 5.8056E+03, &
      5.3698581E-01, 3.37431E+01, 1.1697E+01, 5.7734E+03, &
      5.3967074E-01, 3.37867E+01, 1.1690E+01, 5.7411E+03, &
      5.4236910E-01, 3.38302E+01, 1.1682E+01, 5.7089E+03, &
      5.4508094E-01, 3.38734E+01, 1.1674E+01, 5.6767E+03, &
      5.4780635E-01, 3.39164E+01, 1.1665E+01, 5.6444E+03, &
      5.5054538E-01, 3.39593E+01, 1.1657E+01, 5.6122E+03, &
      5.5329810E-01, 3.40019E+01, 1.1648E+01, 5.5799E+03, &
      5.5606460E-01, 3.40444E+01, 1.1638E+01, 5.5477E+03, &
      5.5884492E-01, 3.40866E+01, 1.1628E+01, 5.5155E+03, &
      5.6163914E-01, 3.41286E+01, 1.1618E+01, 5.4833E+03, &
      5.6444734E-01, 3.41703E+01, 1.1607E+01, 5.4511E+03, &
      5.6726958E-01, 3.42119E+01, 1.1596E+01, 5.4189E+03, &
      5.7010592E-01, 3.42531E+01, 1.1585E+01, 5.3867E+03, &
      5.7295645E-01, 3.42941E+01, 1.1573E+01, 5.3546E+03, &
      5.7582123E-01, 3.43349E+01, 1.1561E+01, 5.3224E+03, &
      5.7870034E-01, 3.43753E+01, 1.1549E+01, 5.2903E+03, &
      5.8159384E-01, 3.44155E+01, 1.1536E+01, 5.2582E+03, &
      5.8450181E-01, 3.44554E+01, 1.1523E+01, 5.2261E+03, &
      5.8742432E-01, 3.44950E+01, 1.1509E+01, 5.1941E+03, &
      5.9036144E-01, 3.45343E+01, 1.1495E+01, 5.1620E+03, &
      5.9331325E-01, 3.45732E+01, 1.1481E+01, 5.1300E+03, &
      5.9627982E-01, 3.46119E+01, 1.1466E+01, 5.0979E+03, &
      5.9926122E-01, 3.46501E+01, 1.1451E+01, 5.0659E+03, &
      6.0225752E-01, 3.46880E+01, 1.1435E+01, 5.0338E+03, &
      6.0526881E-01, 3.47255E+01, 1.1419E+01, 5.0017E+03, &
      6.0829515E-01, 3.47626E+01, 1.1402E+01, 4.9696E+03, &
      6.1133663E-01, 3.47993E+01, 1.1385E+01, 4.9376E+03, &
      6.1439331E-01, 3.48356E+01, 1.1368E+01, 4.9056E+03, &
      6.1746528E-01, 3.48713E+01, 1.1350E+01, 4.8735E+03, &
      6.2055260E-01, 3.49067E+01, 1.1332E+01, 4.8415E+03, &
      6.2365537E-01, 3.49415E+01, 1.1313E+01, 4.8096E+03, &
      6.2677364E-01, 3.49759E+01, 1.1294E+01, 4.7776E+03, &
      6.2990751E-01, 3.50097E+01, 1.1274E+01, 4.7457E+03, &
      6.3305705E-01, 3.50430E+01, 1.1254E+01, 4.7139E+03, &
      6.3622234E-01, 3.50758E+01, 1.1234E+01, 4.6820E+03, &
      6.3940345E-01, 3.51081E+01, 1.1214E+01, 4.6503E+03, &
      6.4260046E-01, 3.51398E+01, 1.1193E+01, 4.6185E+03, &
      6.4581347E-01, 3.51709E+01, 1.1171E+01, 4.5869E+03, &
      6.4904253E-01, 3.52014E+01, 1.1150E+01, 4.5552E+03, &
      6.5228775E-01, 3.52313E+01, 1.1127E+01, 4.5237E+03, &
      6.5554919E-01, 3.52607E+01, 1.1105E+01, 4.4922E+03, &
      6.5882693E-01, 3.52894E+01, 1.1082E+01, 4.4608E+03, &
      6.6212107E-01, 3.53175E+01, 1.1059E+01, 4.4294E+03, &
      6.6543167E-01, 3.53449E+01, 1.1036E+01, 4.3981E+03, &
      6.6875883E-01, 3.53717E+01, 1.1012E+01, 4.3669E+03, &
      6.7210262E-01, 3.53978E+01, 1.0988E+01, 4.3358E+03, &
      6.7546314E-01, 3.54232E+01, 1.0964E+01, 4.3048E+03, &
      6.7884045E-01, 3.54480E+01, 1.0939E+01, 4.2738E+03, &
      6.8223466E-01, 3.54720E+01, 1.0915E+01, 4.2430E+03, &
      6.8564583E-01, 3.54953E+01, 1.0889E+01, 4.2122E+03, &
      6.8907406E-01, 3.55180E+01, 1.0864E+01, 4.1815E+03, &
      6.9251943E-01, 3.55398E+01, 1.0838E+01, 4.1510E+03, &
      6.9598202E-01, 3.55610E+01, 1.0812E+01, 4.1205E+03, &
      6.9946194E-01, 3.55813E+01, 1.0786E+01, 4.0902E+03, &
      7.0295924E-01, 3.56009E+01, 1.0760E+01, 4.0599E+03, &
      7.0647404E-01, 3.56197E+01, 1.0733E+01, 4.0298E+03, &
      7.1000641E-01, 3.56378E+01, 1.0706E+01, 3.9997E+03, &
      7.1355644E-01, 3.56550E+01, 1.0679E+01, 3.9698E+03, &
      7.1712423E-01, 3.56714E+01, 1.0652E+01, 3.9400E+03, &
      7.2070985E-01, 3.56870E+01, 1.0624E+01, 3.9104E+03, &
      7.2431340E-01, 3.57017E+01, 1.0596E+01, 3.8808E+03, &
      7.2793496E-01, 3.57156E+01, 1.0568E+01, 3.8514E+03, &
      7.3157464E-01, 3.57286E+01, 1.0540E+01, 3.8221E+03, &
      7.3523251E-01, 3.57408E+01, 1.0512E+01, 3.7929E+03, &
      7.3890867E-01, 3.57520E+01, 1.0483E+01, 3.7639E+03, &
      7.4260322E-01, 3.57624E+01, 1.0454E+01, 3.7349E+03, &
      7.4631623E-01, 3.57718E+01, 1.0426E+01, 3.7061E+03, &
      7.5004781E-01, 3.57803E+01, 1.0396E+01, 3.6775E+03, &
      7.5379805E-01, 3.57879E+01, 1.0367E+01, 3.6490E+03, &
      7.5756704E-01, 3.57945E+01, 1.0338E+01, 3.6206E+03, &
      7.6135488E-01, 3.58001E+01, 1.0308E+01, 3.5924E+03, &
      7.6516165E-01, 3.58048E+01, 1.0279E+01, 3.5643E+03, &
      7.6898746E-01, 3.58084E+01, 1.0249E+01, 3.5364E+03, &
      7.7283240E-01, 3.58110E+01, 1.0219E+01, 3.5086E+03, &
      7.7669656E-01, 3.58126E+01, 1.0189E+01, 3.4809E+03, &
      7.8058004E-01, 3.58132E+01, 1.0159E+01, 3.4534E+03, &
      7.8448294E-01, 3.58126E+01, 1.0128E+01, 3.4261E+03, &
      7.8840536E-01, 3.58110E+01, 1.0098E+01, 3.3989E+03, &
      7.9234738E-01, 3.58083E+01, 1.0067E+01, 3.3719E+03, &
      7.9630912E-01, 3.58045E+01, 1.0037E+01, 3.3450E+03, &
      8.0029067E-01, 3.57996E+01, 1.0006E+01, 3.3183E+03, &
      8.0429212E-01, 3.57935E+01, 9.9755E+00, 3.2917E+03, &
      8.0831358E-01, 3.57862E+01, 9.9447E+00, 3.2653E+03, &
      8.1235515E-01, 3.57777E+01, 9.9138E+00, 3.2390E+03, &
      8.1641693E-01, 3.57680E+01, 9.8828E+00, 3.2129E+03, &
      8.2049901E-01, 3.57571E+01, 9.8518E+00, 3.1870E+03, &
      8.2460150E-01, 3.57449E+01, 9.8207E+00, 3.1612E+03, &
      8.2872451E-01, 3.57315E+01, 9.7895E+00, 3.1356E+03, &
      8.3286813E-01, 3.57167E+01, 9.7583E+00, 3.1101E+03, &
      8.3703248E-01, 3.57007E+01, 9.7271E+00, 3.0848E+03, &
      8.4121764E-01, 3.56832E+01, 9.6958E+00, 3.0597E+03, &
      8.4542373E-01, 3.56644E+01, 9.6645E+00, 3.0347E+03, &
      8.4965084E-01, 3.56442E+01, 9.6332E+00, 3.0099E+03, &
      8.5389910E-01, 3.56226E+01, 9.6018E+00, 2.9853E+03, &
      8.5816859E-01, 3.56013E+01, 9.5704E+00, 2.9608E+03, &
      8.6245944E-01, 3.55767E+01, 9.5390E+00, 2.9365E+03, &
      8.6677173E-01, 3.55506E+01, 9.5075E+00, 2.9123E+03, &
      8.7110559E-01, 3.55230E+01, 9.4761E+00, 2.8883E+03, &
      8.7546112E-01, 3.54938E+01, 9.4446E+00, 2.8645E+03, &
      8.7983843E-01, 3.54629E+01, 9.4132E+00, 2.8408E+03, &
      8.8423762E-01, 3.54304E+01, 9.3817E+00, 2.8173E+03, &
      8.8865881E-01, 3.53962E+01, 9.3502E+00, 2.7940E+03, &
      8.9310210E-01, 3.53602E+01, 9.3188E+00, 2.7708E+03, &
      8.9756761E-01, 3.53225E+01, 9.2874E+00, 2.7478E+03, &
      9.0205545E-01, 3.52829E+01, 9.2559E+00, 2.7250E+03, &
      9.0656573E-01, 3.52415E+01, 9.2245E+00, 2.7023E+03, &
      9.1109856E-01, 3.51981E+01, 9.1931E+00, 2.6798E+03, &
      9.1565405E-01, 3.51528E+01, 9.1617E+00, 2.6575E+03, &
      9.2023232E-01, 3.51055E+01, 9.1304E+00, 2.6353E+03, &
      9.2483348E-01, 3.50561E+01, 9.0991E+00, 2.6133E+03, &
      9.2945765E-01, 3.50046E+01, 9.0678E+00, 2.5914E+03, &
      9.3410494E-01, 3.49509E+01, 9.0365E+00, 2.5697E+03, &
      9.3877546E-01, 3.48950E+01, 9.0053E+00, 2.5482E+03, &
      9.4346934E-01, 3.48368E+01, 8.9741E+00, 2.5268E+03, &
      9.4818668E-01, 3.47763E+01, 8.9430E+00, 2.5056E+03, &
      9.5292762E-01, 3.47133E+01, 8.9119E+00, 2.4845E+03, &
      9.5769226E-01, 3.46478E+01, 8.8809E+00, 2.4637E+03, &
      9.6248072E-01, 3.45798E+01, 8.8499E+00, 2.4429E+03, &
      9.6729312E-01, 3.45092E+01, 8.8190E+00, 2.4224E+03, &
      9.7212959E-01, 3.44358E+01, 8.7881E+00, 2.4020E+03, &
      9.7699023E-01, 3.43597E+01, 8.7572E+00, 2.3817E+03, &
      9.8187519E-01, 3.42807E+01, 8.7265E+00, 2.3616E+03, &
      9.8678456E-01, 3.41987E+01, 8.6958E+00, 2.3417E+03, &
      9.9171848E-01, 3.41138E+01, 8.6651E+00, 2.3219E+03, &
      9.9667708E-01, 3.40258E+01, 8.6345E+00, 2.3023E+03, &
      1.0016605E+00, 3.39309E+01, 8.5979E+00, 2.2812E+03, &
      1.0066688E+00, 3.38244E+01, 8.5491E+00, 2.2571E+03, &
      1.0117021E+00, 3.37127E+01, 8.5006E+00, 2.2332E+03, &
      1.0167606E+00, 3.35959E+01, 8.4525E+00, 2.2096E+03, &
      1.0218444E+00, 3.34736E+01, 8.4047E+00, 2.1863E+03, &
      1.0269536E+00, 3.33455E+01, 8.3572E+00, 2.1633E+03, &
      1.0320884E+00, 3.32115E+01, 8.3100E+00, 2.1404E+03, &
      1.0372489E+00, 3.30713E+01, 8.2632E+00, 2.1179E+03, &
      1.0424351E+00, 3.29245E+01, 8.2166E+00, 2.0956E+03, &
      1.0476473E+00, 3.27707E+01, 8.1704E+00, 2.0735E+03, &
      1.0528855E+00, 3.26097E+01, 8.1245E+00, 2.0517E+03, &
      1.0581499E+00, 3.24410E+01, 8.0789E+00, 2.0302E+03, &
      1.0634407E+00, 3.22642E+01, 8.0337E+00, 2.0089E+03, &
      1.0687579E+00, 3.20788E+01, 7.9887E+00, 1.9878E+03, &
      1.0741017E+00, 3.18842E+01, 7.9440E+00, 1.9670E+03, &
      1.0794722E+00, 3.16798E+01, 7.8997E+00, 1.9463E+03, &
      1.0848695E+00, 3.14650E+01, 7.8556E+00, 1.9260E+03, &
      1.0902939E+00, 3.12390E+01, 7.8119E+00, 1.9058E+03, &
      1.0957454E+00, 3.10010E+01, 7.7684E+00, 1.8859E+03, &
      1.1012241E+00, 3.07501E+01, 7.7253E+00, 1.8662E+03, &
      1.1067302E+00, 3.04851E+01, 7.6824E+00, 1.8467E+03, &
      1.1122639E+00, 3.02050E+01, 7.6398E+00, 1.8274E+03, &
      1.1178252E+00, 2.99083E+01, 7.5976E+00, 1.8084E+03, &
      1.1234143E+00, 2.95935E+01, 7.5556E+00, 1.7896E+03, &
      1.1290314E+00, 2.92589E+01, 7.5139E+00, 1.7709E+03, &
      1.1346765E+00, 2.89023E+01, 7.4725E+00, 1.7525E+03, &
      1.1403499E+00, 2.85214E+01, 7.4312E+00, 1.7343E+03, &
      1.1460517E+00, 2.81132E+01, 7.3885E+00, 1.7158E+03, &
      1.1517819E+00, 2.76743E+01, 7.3461E+00, 1.6976E+03, &
      1.1575408E+00, 2.72007E+01, 7.3040E+00, 1.6796E+03, &
      1.1633285E+00, 2.66875E+01, 7.2621E+00, 1.6617E+03, &
      1.1691452E+00, 2.61287E+01, 7.2206E+00, 1.6441E+03, &
      1.1749909E+00, 2.55167E+01, 7.1793E+00, 1.6267E+03, &
      1.1808659E+00, 2.48419E+01, 7.1384E+00, 1.6095E+03, &
      1.1867702E+00, 2.40918E+01, 7.0977E+00, 1.5925E+03, &
      1.1927040E+00, 2.32496E+01, 7.0573E+00, 1.5756E+03, &
      1.1986676E+00, 2.22917E+01, 7.0172E+00, 1.5590E+03, &
      1.2046609E+00, 2.11857E+01, 6.9774E+00, 1.5425E+03, &
      1.2106842E+00, 1.98799E+01, 6.9379E+00, 1.5263E+03, &
      1.2167376E+00, 1.82900E+01, 6.8986E+00, 1.5102E+03, &
      1.2228213E+00, 1.62608E+01, 6.8596E+00, 1.4943E+03, &
      1.2289354E+00, 1.34483E+01, 6.8209E+00, 1.4786E+03, &
      1.2350801E+00, 8.76614E+00, 6.7824E+00, 1.4630E+03, &
      1.2410448E+00, -1.46252E+01, 6.7455E+00, 1.4482E+03, &
      1.2412555E+00, -2.14128E+01, 2.6448E+01, 5.6509E+03, &
      1.2413551E+00, -1.49723E+01, 2.6445E+01, 5.6497E+03, &
      1.2474618E+00, 7.73230E+00, 2.6244E+01, 5.5795E+03, &
      1.2536991E+00, 1.12864E+01, 2.6042E+01, 5.5090E+03, &
      1.2599676E+00, 1.26566E+01, 2.5841E+01, 5.4394E+03, &
      1.2662674E+00, 1.24859E+01, 2.5641E+01, 5.3707E+03, &
      1.2725988E+00, 8.84739E+00, 2.5444E+01, 5.3029E+03, &
      1.2747297E+00, 3.75774E-01, 2.5378E+01, 5.2803E+03, &
      1.2752703E+00, 2.86762E-01, 3.8277E+01, 7.9564E+03, &
      1.2789618E+00, 1.21356E+01, 3.8103E+01, 7.8974E+03, &
      1.2853566E+00, 1.72921E+01, 3.7804E+01, 7.7966E+03, &
      1.2917833E+00, 2.03204E+01, 3.7508E+01, 7.6972E+03, &
      1.2982423E+00, 2.25811E+01, 3.7214E+01, 7.5990E+03, &
      1.3047335E+00, 2.44201E+01, 3.6923E+01, 7.5021E+03, &
      1.3112571E+00, 2.59839E+01, 3.6634E+01, 7.4065E+03, &
      1.3178134E+00, 2.73503E+01, 3.6347E+01, 7.3121E+03, &
      1.3244025E+00, 2.85662E+01, 3.6063E+01, 7.2189E+03, &
      1.3310245E+00, 2.96627E+01, 3.5780E+01, 7.1269E+03, &
      1.3376796E+00, 3.06615E+01, 3.5501E+01, 7.0361E+03, &
      1.3443680E+00, 3.15784E+01, 3.5223E+01, 6.9465E+03, &
      1.3510899E+00, 3.24253E+01, 3.4948E+01, 6.8580E+03, &
      1.3578453E+00, 3.32117E+01, 3.4675E+01, 6.7707E+03, &
      1.3646345E+00, 3.39449E+01, 3.4404E+01, 6.6845E+03, &
      1.3714577E+00, 3.46310E+01, 3.4135E+01, 6.5994E+03, &
      1.3783150E+00, 3.52747E+01, 3.3869E+01, 6.5155E+03, &
      1.3852066E+00, 3.58803E+01, 3.3604E+01, 6.4326E+03, &
      1.3921326E+00, 3.64511E+01, 3.3342E+01, 6.3507E+03, &
      1.3990933E+00, 3.69901E+01, 3.3082E+01, 6.2700E+03, &
      1.4060887E+00, 3.74997E+01, 3.2824E+01, 6.1903E+03, &
      1.4131192E+00, 3.79820E+01, 3.2568E+01, 6.1116E+03, &
      1.4201848E+00, 3.84389E+01, 3.2314E+01, 6.0339E+03, &
      1.4272857E+00, 3.88719E+01, 3.2062E+01, 5.9572E+03, &
      1.4344221E+00, 3.92824E+01, 3.1813E+01, 5.8815E+03, &
      1.4415942E+00, 3.96715E+01, 3.1565E+01, 5.8068E+03, &
      1.4488022E+00, 4.00403E+01, 3.1319E+01, 5.7331E+03, &
      1.4560462E+00, 4.03894E+01, 3.1075E+01, 5.6603E+03, &
      1.4633265E+00, 4.07198E+01, 3.0834E+01, 5.5884E+03, &
      1.4706431E+00, 4.10317E+01, 3.0594E+01, 5.5175E+03, &
      1.4779963E+00, 4.13257E+01, 3.0356E+01, 5.4475E+03, &
      1.4853863E+00, 4.16020E+01, 3.0120E+01, 5.3783E+03, &
      1.4928132E+00, 4.18607E+01, 2.9886E+01, 5.3101E+03, &
      1.5002773E+00, 4.21016E+01, 2.9654E+01, 5.2428E+03, &
      1.5077787E+00, 4.23245E+01, 2.9423E+01, 5.1763E+03, &
      1.5153176E+00, 4.25289E+01, 2.9195E+01, 5.1107E+03, &
      1.5228942E+00, 4.27137E+01, 2.8968E+01, 5.0459E+03, &
      1.5305086E+00, 4.28779E+01, 2.8744E+01, 4.9820E+03, &
      1.5381612E+00, 4.30195E+01, 2.8521E+01, 4.9188E+03, &
      1.5458520E+00, 4.31358E+01, 2.8300E+01, 4.8565E+03, &
      1.5535812E+00, 4.32231E+01, 2.8080E+01, 4.7950E+03, &
      1.5613491E+00, 4.32756E+01, 2.7862E+01, 4.7343E+03, &
      1.5691559E+00, 4.32843E+01, 2.7647E+01, 4.6744E+03, &
      1.5770017E+00, 4.32344E+01, 2.7432E+01, 4.6152E+03, &
      1.5848867E+00, 4.30989E+01, 2.7220E+01, 4.5568E+03, &
      1.5928111E+00, 4.28205E+01, 2.7009E+01, 4.4992E+03, &
      1.6007752E+00, 4.22397E+01, 2.6800E+01, 4.4422E+03, &
      1.6087790E+00, 4.04058E+01, 2.6592E+01, 4.3860E+03, &
      1.6098611E+00, 3.96252E+01, 2.6565E+01, 4.3785E+03, &
      1.6127389E+00, 3.96414E+01, 3.1048E+01, 5.1067E+03, &
      1.6168229E+00, 4.17185E+01, 3.0919E+01, 5.0728E+03, &
      1.6249070E+00, 4.32470E+01, 3.0667E+01, 5.0064E+03, &
      1.6330316E+00, 4.41265E+01, 3.0416E+01, 4.9410E+03, &
      1.6411967E+00, 4.47741E+01, 3.0168E+01, 4.8764E+03, &
      1.6494027E+00, 4.52968E+01, 2.9922E+01, 4.8127E+03, &
      1.6576497E+00, 4.57386E+01, 2.9678E+01, 4.7499E+03, &
      1.6659380E+00, 4.61214E+01, 2.9436E+01, 4.6878E+03, &
      1.6742677E+00, 4.64572E+01, 2.9196E+01, 4.6265E+03, &
      1.6826390E+00, 4.67528E+01, 2.8957E+01, 4.5660E+03, &
      1.6910522E+00, 4.70108E+01, 2.8722E+01, 4.5065E+03, &
      1.6995075E+00, 4.72386E+01, 2.8507E+01, 4.4507E+03, &
      1.7080050E+00, 4.74430E+01, 2.8295E+01, 4.3957E+03, &
      1.7165450E+00, 4.76215E+01, 2.8086E+01, 4.3417E+03, &
      1.7251278E+00, 4.77705E+01, 2.7880E+01, 4.2884E+03, &
      1.7337534E+00, 4.78831E+01, 2.7676E+01, 4.2360E+03, &
      1.7424222E+00, 4.79450E+01, 2.7475E+01, 4.1844E+03, &
      1.7511343E+00, 4.79218E+01, 2.7276E+01, 4.1336E+03, &
      1.7598899E+00, 4.76933E+01, 2.7080E+01, 4.0836E+03, &
      1.7660507E+00, 4.69089E+01, 2.6944E+01, 4.0489E+03, &
      1.7686894E+00, 4.66372E+01, 2.8792E+01, 4.3196E+03, &
      1.7693492E+00, 4.69650E+01, 2.8776E+01, 4.3155E+03, &
      1.7775328E+00, 4.82501E+01, 2.8575E+01, 4.2659E+03, &
      1.7864205E+00, 4.88419E+01, 2.8361E+01, 4.2130E+03, &
      1.7953526E+00, 4.92675E+01, 2.8150E+01, 4.1609E+03, &
      1.8043294E+00, 4.96165E+01, 2.7941E+01, 4.1095E+03, &
      1.8133510E+00, 4.99189E+01, 2.7734E+01, 4.0590E+03, &
      1.8224178E+00, 5.01884E+01, 2.7530E+01, 4.0091E+03, &
      1.8315299E+00, 5.04324E+01, 2.7328E+01, 3.9600E+03, &
      1.8406875E+00, 5.06550E+01, 2.7128E+01, 3.9116E+03, &
      1.8498909E+00, 5.08584E+01, 2.6930E+01, 3.8639E+03, &
      1.8591404E+00, 5.10436E+01, 2.6740E+01, 3.8176E+03, &
      1.8684361E+00, 5.12174E+01, 2.6558E+01, 3.7729E+03, &
      1.8777783E+00, 5.13805E+01, 2.6379E+01, 3.7289E+03, &
      1.8871672E+00, 5.15326E+01, 2.6202E+01, 3.6856E+03, &
      1.8966030E+00, 5.16730E+01, 2.6028E+01, 3.6429E+03, &
      1.9060860E+00, 5.18006E+01, 2.5855E+01, 3.6009E+03, &
      1.9156165E+00, 5.19135E+01, 2.5685E+01, 3.5595E+03, &
      1.9251945E+00, 5.20083E+01, 2.5517E+01, 3.5188E+03, &
      1.9348205E+00, 5.20783E+01, 2.5351E+01, 3.4786E+03, &
      1.9444946E+00, 5.21094E+01, 2.5187E+01, 3.4390E+03, &
      1.9542171E+00, 5.20624E+01, 2.5025E+01, 3.4000E+03, &
      1.9639882E+00, 5.17185E+01, 2.4865E+01, 3.3615E+03, &
      1.9642536E+00, 5.16931E+01, 2.4861E+01, 3.3605E+03, &
      1.9707463E+00, 5.17851E+01, 2.5938E+01, 3.4942E+03, &
      1.9738081E+00, 5.20833E+01, 2.5888E+01, 3.4820E+03, &
      1.9836772E+00, 5.25884E+01, 2.5726E+01, 3.4432E+03, &
      1.9935955E+00, 5.29146E+01, 2.5566E+01, 3.4048E+03, &
      2.0035635E+00, 5.31790E+01, 2.5408E+01, 3.3670E+03, &
      2.0135813E+00, 5.34106E+01, 2.5251E+01, 3.3297E+03, &
      2.0236492E+00, 5.36213E+01, 2.5096E+01, 3.2928E+03, &
      2.0337675E+00, 5.38172E+01, 2.4942E+01, 3.2565E+03, &
      2.0439363E+00, 5.40019E+01, 2.4790E+01, 3.2206E+03, &
      2.0541560E+00, 5.41777E+01, 2.4639E+01, 3.1851E+03, &
      2.0644268E+00, 5.43459E+01, 2.4489E+01, 3.1501E+03, &
      2.0747489E+00, 5.45082E+01, 2.4342E+01, 3.1158E+03, &
      2.0851227E+00, 5.46662E+01, 2.4197E+01, 3.0818E+03, &
      2.0955483E+00, 5.48205E+01, 2.4052E+01, 3.0482E+03, &
      2.1060260E+00, 5.49700E+01, 2.3902E+01, 3.0143E+03, &
      2.1165562E+00, 5.51146E+01, 2.3753E+01, 2.9807E+03, &
      2.1271389E+00, 5.52548E+01, 2.3606E+01, 2.9475E+03, &
      2.1377746E+00, 5.53910E+01, 2.3459E+01, 2.9148E+03, &
      2.1484635E+00, 5.55237E+01, 2.3314E+01, 2.8824E+03, &
      2.1592058E+00, 5.56530E+01, 2.3169E+01, 2.8504E+03, &
      2.1700018E+00, 5.57793E+01, 2.3026E+01, 2.8188E+03, &
      2.1808519E+00, 5.59026E+01, 2.2884E+01, 2.7875E+03, &
      2.1917561E+00, 5.60233E+01, 2.2743E+01, 2.7566E+03, &
      2.2027149E+00, 5.61415E+01, 2.2602E+01, 2.7261E+03, &
      2.2137285E+00, 5.62573E+01, 2.2463E+01, 2.6959E+03, &
      2.2247971E+00, 5.63709E+01, 2.2325E+01, 2.6661E+03, &
      2.2359211E+00, 5.64824E+01, 2.2187E+01, 2.6366E+03, &
      2.2471007E+00, 5.65919E+01, 2.2051E+01, 2.6074E+03, &
      2.2583362E+00, 5.66995E+01, 2.1915E+01, 2.5786E+03, &
      2.2696279E+00, 5.68055E+01, 2.1781E+01, 2.5500E+03, &
      2.2809760E+00, 5.69099E+01, 2.1647E+01, 2.5219E+03, &
      2.2923809E+00, 5.70129E+01, 2.1514E+01, 2.4940E+03, &
      2.3038428E+00, 5.71145E+01, 2.1379E+01, 2.4662E+03, &
      2.3153620E+00, 5.72137E+01, 2.1244E+01, 2.4385E+03, &
      2.3269388E+00, 5.73108E+01, 2.1110E+01, 2.4111E+03, &
      2.3385735E+00, 5.74059E+01, 2.0977E+01, 2.3841E+03, &
      2.3502664E+00, 5.74991E+01, 2.0844E+01, 2.3573E+03, &
      2.3620177E+00, 5.75905E+01, 2.0712E+01, 2.3308E+03, &
      2.3738278E+00, 5.76802E+01, 2.0581E+01, 2.3046E+03, &
      2.3856970E+00, 5.77683E+01, 2.0451E+01, 2.2787E+03, &
      2.3976254E+00, 5.78547E+01, 2.0321E+01, 2.2531E+03, &
      2.4096136E+00, 5.79397E+01, 2.0192E+01, 2.2278E+03, &
      2.4216616E+00, 5.80231E+01, 2.0064E+01, 2.2028E+03, &
      2.4337699E+00, 5.81052E+01, 1.9937E+01, 2.1780E+03, &
      2.4459388E+00, 5.81860E+01, 1.9811E+01, 2.1535E+03, &
      2.4581685E+00, 5.82654E+01, 1.9685E+01, 2.1293E+03, &
      2.4704593E+00, 5.83436E+01, 1.9560E+01, 2.1053E+03, &
      2.4828116E+00, 5.86401E+01, 1.9436E+01, 2.0816E+03, &
      2.4952257E+00, 5.87160E+01, 1.9308E+01, 2.0577E+03, &
      2.5077018E+00, 5.87903E+01, 1.9181E+01, 2.0341E+03, &
      2.5202403E+00, 5.88630E+01, 1.9054E+01, 2.0108E+03, &
      2.5328415E+00, 5.89343E+01, 1.8929E+01, 1.9877E+03, &
      2.5455057E+00, 5.90042E+01, 1.8804E+01, 1.9648E+03, &
      2.5582333E+00, 5.92221E+01, 1.8678E+01, 1.9421E+03, &
      2.5710244E+00, 5.92892E+01, 1.8552E+01, 1.9194E+03, &
      2.5838796E+00, 5.93545E+01, 1.8426E+01, 1.8970E+03, &
      2.5967990E+00, 5.94182E+01, 1.8301E+01, 1.8749E+03, &
      2.6097829E+00, 5.94804E+01, 1.8177E+01, 1.8530E+03, &
      2.6228319E+00, 5.95411E+01, 1.8054E+01, 1.8314E+03, &
      2.6359460E+00, 5.96004E+01, 1.7931E+01, 1.8100E+03, &
      2.6491257E+00, 5.96584E+01, 1.7810E+01, 1.7889E+03, &
      2.6623714E+00, 5.97150E+01, 1.7689E+01, 1.7680E+03, &
      2.6756832E+00, 5.97704E+01, 1.7570E+01, 1.7474E+03, &
      2.6890617E+00, 5.98246E+01, 1.7451E+01, 1.7270E+03, &
      2.7025070E+00, 5.98777E+01, 1.7333E+01, 1.7069E+03, &
      2.7160195E+00, 5.99296E+01, 1.7216E+01, 1.6870E+03, &
      2.7295996E+00, 5.99805E+01, 1.7099E+01, 1.6674E+03, &
      2.7432476E+00, 6.00304E+01, 1.6984E+01, 1.6480E+03, &
      2.7569638E+00, 6.00793E+01, 1.6869E+01, 1.6288E+03, &
      2.7707486E+00, 6.01273E+01, 1.6753E+01, 1.6097E+03, &
      2.7846024E+00, 6.01743E+01, 1.6638E+01, 1.5907E+03, &
      2.7985254E+00, 6.02204E+01, 1.6523E+01, 1.5719E+03, &
      2.8125180E+00, 6.02655E+01, 1.6409E+01, 1.5534E+03, &
      2.8265806E+00, 6.03097E+01, 1.6296E+01, 1.5351E+03, &
      2.8407135E+00, 6.03531E+01, 1.6184E+01, 1.5171E+03, &
      2.8549171E+00, 6.03958E+01, 1.6073E+01, 1.4993E+03, &
      2.8691917E+00, 6.04377E+01, 1.5962E+01, 1.4816E+03, &
      2.8835376E+00, 6.04789E+01, 1.5853E+01, 1.4643E+03, &
      2.8979553E+00, 6.05195E+01, 1.5745E+01, 1.4471E+03, &
      2.9124451E+00, 6.05595E+01, 1.5637E+01, 1.4301E+03, &
      2.9270073E+00, 6.05992E+01, 1.5530E+01, 1.4134E+03, &
      2.9416424E+00, 6.06384E+01, 1.5424E+01, 1.3969E+03, &
      2.9563506E+00, 6.06775E+01, 1.5319E+01, 1.3805E+03, &
      2.9711323E+00, 6.07165E+01, 1.5215E+01, 1.3644E+03, &
      2.9859880E+00, 6.07557E+01, 1.5112E+01, 1.3485E+03, &
      3.0009179E+00, 6.07956E+01, 1.5009E+01, 1.3327E+03, &
      3.0159225E+00, 6.08359E+01, 1.4891E+01, 1.3158E+03, &
      3.0310021E+00, 6.08734E+01, 1.4775E+01, 1.2991E+03, &
      3.0461571E+00, 6.09086E+01, 1.4660E+01, 1.2827E+03, &
      3.0613879E+00, 6.09417E+01, 1.4546E+01, 1.2664E+03, &
      3.0766949E+00, 6.09732E+01, 1.4433E+01, 1.2504E+03, &
      3.0920783E+00, 6.10030E+01, 1.4321E+01, 1.2346E+03, &
      3.1075387E+00, 6.10315E+01, 1.4210E+01, 1.2191E+03, &
      3.1230764E+00, 6.10586E+01, 1.4100E+01, 1.2037E+03, &
      3.1386918E+00, 6.10844E+01, 1.3990E+01, 1.1885E+03, &
      3.1543853E+00, 6.11091E+01, 1.3882E+01, 1.1736E+03, &
      3.1701572E+00, 6.11328E+01, 1.3775E+01, 1.1588E+03, &
      3.1860080E+00, 6.11554E+01, 1.3669E+01, 1.1443E+03, &
      3.2019380E+00, 6.11770E+01, 1.3564E+01, 1.1299E+03, &
      3.2179477E+00, 6.11978E+01, 1.3460E+01, 1.1158E+03, &
      3.2340374E+00, 6.13211E+01, 1.3354E+01, 1.1016E+03, &
      3.2502076E+00, 6.13406E+01, 1.3249E+01, 1.0875E+03, &
      3.2664587E+00, 6.13590E+01, 1.3144E+01, 1.0736E+03, &
      3.2827910E+00, 6.13763E+01, 1.3040E+01, 1.0599E+03, &
      3.2992049E+00, 6.13925E+01, 1.2937E+01, 1.0464E+03, &
      3.3157009E+00, 6.14078E+01, 1.2835E+01, 1.0331E+03, &
      3.3322794E+00, 6.14220E+01, 1.2734E+01, 1.0199E+03, &
      3.3489408E+00, 6.14354E+01, 1.2634E+01, 1.0070E+03, &
      3.3656856E+00, 6.14479E+01, 1.2534E+01, 9.9417E+02, &
      3.3825140E+00, 6.14596E+01, 1.2436E+01, 9.8155E+02, &
      3.3994265E+00, 6.14705E+01, 1.2338E+01, 9.6909E+02, &
      3.4164237E+00, 6.14807E+01, 1.2242E+01, 9.5681E+02, &
      3.4335058E+00, 6.14901E+01, 1.2146E+01, 9.4469E+02, &
      3.4506733E+00, 6.14988E+01, 1.2051E+01, 9.3273E+02, &
      3.4679267E+00, 6.15069E+01, 1.1957E+01, 9.2094E+02, &
      3.4852663E+00, 6.15143E+01, 1.1864E+01, 9.0930E+02, &
      3.5026927E+00, 6.15211E+01, 1.1772E+01, 8.9782E+02, &
      3.5202061E+00, 6.15273E+01, 1.1680E+01, 8.8649E+02, &
      3.5378072E+00, 6.15803E+01, 1.1589E+01, 8.7530E+02, &
      3.5554962E+00, 6.15858E+01, 1.1497E+01, 8.6412E+02, &
      3.5732737E+00, 6.15905E+01, 1.1406E+01, 8.5309E+02, &
      3.5911400E+00, 6.15945E+01, 1.1316E+01, 8.4221E+02, &
      3.6090957E+00, 6.15978E+01, 1.1227E+01, 8.3148E+02, &
      3.6271412E+00, 6.16004E+01, 1.1138E+01, 8.2089E+02, &
      3.6452769E+00, 6.16023E+01, 1.1050E+01, 8.1044E+02, &
      3.6635033E+00, 6.16036E+01, 1.0963E+01, 8.0013E+02, &
      3.6818208E+00, 6.16044E+01, 1.0877E+01, 7.8997E+02, &
      3.7002299E+00, 6.16045E+01, 1.0791E+01, 7.7993E+02, &
      3.7187311E+00, 6.16041E+01, 1.0706E+01, 7.7004E+02, &
      3.7373247E+00, 6.16031E+01, 1.0622E+01, 7.6027E+02, &
      3.7560114E+00, 6.16016E+01, 1.0539E+01, 7.5064E+02, &
      3.7747914E+00, 6.15996E+01, 1.0456E+01, 7.4113E+02, &
      3.7936654E+00, 6.15971E+01, 1.0374E+01, 7.3176E+02, &
      3.8126337E+00, 6.15941E+01, 1.0293E+01, 7.2250E+02, &
      3.8316969E+00, 6.15907E+01, 1.0213E+01, 7.1337E+02, &
      3.8508554E+00, 6.15868E+01, 1.0133E+01, 7.0437E+02, &
      3.8701096E+00, 6.15824E+01, 1.0054E+01, 6.9548E+02, &
      3.8894602E+00, 6.15776E+01, 9.9761E+00, 6.8671E+02, &
      3.9089075E+00, 6.15724E+01, 9.8986E+00, 6.7806E+02, &
      3.9284520E+00, 6.15668E+01, 9.8217E+00, 6.6952E+02, &
      3.9480943E+00, 6.15835E+01, 9.7449E+00, 6.6106E+02, &
      3.9678347E+00, 6.15773E+01, 9.6685E+00, 6.5270E+02, &
      3.9876739E+00, 6.15705E+01, 9.5928E+00, 6.4444E+02, &
      4.055024E+00, 6.19269E+01, 9.3434E+00, 6.1753E+02, &
      4.334821E+00, 6.17228E+01, 8.4197E+00, 5.2147E+02, &
      4.633924E+00, 6.14665E+01, 7.5975E+00, 4.4103E+02, &
      4.953664E+00, 6.11604E+01, 6.8649E+00, 3.7359E+02, &
      5.295467E+00, 6.07991E+01, 6.2113E+00, 3.1694E+02, &
      5.660855E+00, 6.03598E+01, 5.6017E+00, 2.6811E+02, &
      6.051453E+00, 5.97832E+01, 5.0463E+00, 2.2661E+02, &
      6.469004E+00, 5.89836E+01, 4.5534E+00, 1.9190E+02, &
      6.915365E+00, 5.77015E+01, 4.1161E+00, 1.6285E+02, &
      7.363720E+00, 5.46368E+01, 3.7337E+00, 1.3924E+02, &
      7.392525E+00, 5.41724E+01, 3.7112E+00, 1.3790E+02, &
      7.476430E+00, 5.16171E+01, 3.6468E+00, 1.3408E+02, &
      7.506486E+00, 4.80919E+01, 3.6242E+00, 1.3276E+02, &
      7.551570E+00, 5.16124E+01, 1.0464E+01, 3.7178E+02, &
      7.664280E+00, 5.46597E+01, 1.0133E+01, 3.5490E+02, &
      7.902609E+00, 5.62924E+01, 9.4871E+00, 3.2258E+02, &
      8.086568E+00, 5.63069E+01, 9.1064E+00, 3.0279E+02, &
      8.210342E+00, 5.51953E+01, 8.8738E+00, 2.9073E+02, &
      8.243349E+00, 5.35209E+01, 8.8144E+00, 2.8766E+02, &
      8.292858E+00, 5.54015E+01, 1.2167E+01, 3.9299E+02, &
      8.416632E+00, 5.71642E+01, 1.1841E+01, 3.7697E+02, &
      8.447890E+00, 5.73859E+01, 1.1761E+01, 3.7307E+02, &
      8.533840E+00, 5.77981E+01, 1.1547E+01, 3.6267E+02, &
      8.664460E+00, 5.77495E+01, 1.1235E+01, 3.4767E+02, &
      8.699292E+00, 5.70199E+01, 1.1163E+01, 3.4409E+02, &
      8.751540E+00, 5.81155E+01, 1.2789E+01, 3.9124E+02, &
      8.882160E+00, 5.93813E+01, 1.2511E+01, 3.7721E+02, &
      9.030794E+00, 6.02005E+01, 1.2202E+01, 3.6193E+02, &
      9.653919E+00, 6.21877E+01, 1.1006E+01, 3.0580E+02, &
      1.032004E+01, 6.33585E+01, 9.8620E+00, 2.5672E+02, &
      1.103212E+01, 6.40963E+01, 8.8004E+00, 2.1467E+02, &
      1.179334E+01, 6.45643E+01, 7.8620E+00, 1.7975E+02, &
      1.260708E+01, 6.48759E+01, 7.0353E+00, 1.5077E+02, &
      1.347697E+01, 6.50801E+01, 6.3010E+00, 1.2660E+02, &
      1.440688E+01, 6.52095E+01, 5.6482E+00, 1.0642E+02, &
      1.540095E+01, 6.53548E+01, 5.0597E+00, 8.9410E+01, &
      1.646362E+01, 6.53846E+01, 4.5252E+00, 7.5022E+01, &
      1.759961E+01, 6.54482E+01, 4.0404E+00, 6.2864E+01, &
      1.881398E+01, 6.54127E+01, 3.6017E+00, 5.2609E+01, &
      2.011215E+01, 6.53547E+01, 3.2134E+00, 4.4077E+01, &
      2.149988E+01, 6.52827E+01, 2.8695E+00, 3.6974E+01, &
      2.298338E+01, 6.52020E+01, 2.5647E+00, 3.1052E+01, &
      2.456923E+01, 6.51162E+01, 2.2943E+00, 2.6110E+01, &
      2.626450E+01, 6.50301E+01, 2.0543E+00, 2.1982E+01, &
      2.807676E+01, 6.49464E+01, 1.8264E+00, 1.8392E+01, &
      3.001405E+01, 6.48428E+01, 1.6209E+00, 1.5372E+01, &
      3.208502E+01, 6.47303E+01, 1.4399E+00, 1.2867E+01, &
      3.429889E+01, 6.46085E+01, 1.2777E+00, 1.0765E+01, &
      3.666551E+01, 6.44733E+01, 1.1329E+00, 9.0083E+00, &
      3.919543E+01, 6.43176E+01, 1.0021E+00, 7.5274E+00, &
      4.189992E+01, 6.41253E+01, 8.8527E-01, 6.2892E+00, &
      4.479101E+01, 6.38653E+01, 7.8210E-01, 5.2605E+00, &
      4.788159E+01, 6.34418E+01, 6.9148E-01, 4.4083E+00, &
      5.095579E+01, 6.23594E+01, 6.1688E-01, 3.7451E+00, &
      5.118542E+01, 6.21547E+01, 6.1183E-01, 3.7015E+00, &
      5.173572E+01, 6.11993E+01, 5.9999E-01, 3.5999E+00, &
      5.194371E+01, 5.98099E+01, 5.9561E-01, 3.5626E+00, &
      5.225568E+01, 6.12140E+01, 3.3244E+00, 1.7368E+01, &
      5.303561E+01, 6.24827E+01, 3.2494E+00, 1.6736E+01, &
      5.471721E+01, 6.33932E+01, 3.0964E+00, 1.5477E+01, &
      5.849270E+01, 6.42121E+01, 2.7800E+00, 1.3037E+01, &
      6.252870E+01, 6.46272E+01, 2.4918E+00, 1.0968E+01, &
      6.684318E+01, 6.48831E+01, 2.2322E+00, 9.2269E+00, &
      7.145536E+01, 6.50525E+01, 1.9977E+00, 7.7575E+00, &
      7.638578E+01, 6.51669E+01, 1.7875E+00, 6.5243E+00, &
      8.165640E+01, 6.52468E+01, 1.5972E+00, 5.4832E+00, &
      8.729069E+01, 6.52984E+01, 1.4228E+00, 4.5981E+00, &
      9.331374E+01, 6.53289E+01, 1.2675E+00, 3.8594E+00, &
      9.975239E+01, 6.53454E+01, 1.1294E+00, 3.2428E+00, &
      1.066353E+02, 6.53814E+01, 1.0049E+00, 2.7239E+00, &
      1.139931E+02, 6.53781E+01, 8.9182E-01, 2.2858E+00, &
      1.218587E+02, 6.53683E+01, 7.9157E-01, 1.9212E+00, &
      1.302669E+02, 6.53544E+01, 7.0266E-01, 1.6175E+00, &
      1.392553E+02, 6.53378E+01, 6.2379E-01, 1.3646E+00, &
      1.488640E+02, 6.53196E+01, 5.5382E-01, 1.1538E+00, &
      1.591356E+02, 6.53004E+01, 4.9174E-01, 9.7804E-01, &
      1.701159E+02, 6.52810E+01, 4.3666E-01, 8.3139E-01, &
      1.818539E+02, 6.52617E+01, 3.8778E-01, 7.0893E-01, &
      1.944018E+02, 6.52427E+01, 3.4439E-01, 6.0659E-01, &
      2.078156E+02, 6.52242E+01, 3.0589E-01, 5.2100E-01, &
      2.221548E+02, 6.52063E+01, 2.7171E-01, 4.4932E-01, &
      2.374835E+02, 6.51893E+01, 2.4224E-01, 3.9020E-01, &
      2.538699E+02, 6.51737E+01, 2.1619E-01, 3.4061E-01, &
      2.713869E+02, 6.51593E+01, 1.9295E-01, 2.9872E-01, &
      2.901126E+02, 6.51459E+01, 1.7223E-01, 2.6329E-01, &
      3.101304E+02, 6.51335E+01, 1.5374E-01, 2.3324E-01, &
      3.315294E+02, 6.51221E+01, 1.3724E-01, 2.0769E-01, &
      3.544049E+02, 6.51115E+01, 1.2252E-01, 1.8592E-01, &
      3.788588E+02, 6.51017E+01, 1.0939E-01, 1.6731E-01, &
      4.050001E+02, 6.50928E+01, 9.7665E-02, 1.5134E-01, &
      4.329451E+02, 6.50845E+01, 8.7205E-02, 1.3758E-01]
   case(66)
      frel = -6.4740E-01
      allocate(nist_tab(3592))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.116000E-03, 1.20399E+01, 0.0000E+00, 1.9080E-06, &
      4.179000E-03, 1.19981E+01, 0.0000E+00, 2.0805E-06, &
      4.195800E-03, 1.19535E+01, 0.0000E+00, 2.1285E-06, &
      4.221000E-03, 1.19946E+01, 8.4705E-02, 5.1966E+03, &
      4.284000E-03, 1.20271E+01, 8.5785E-02, 5.1855E+03, &
      1.069000E-02, 1.15061E+01, 2.6494E-01, 6.4178E+03, &
      1.142761E-02, 1.13999E+01, 2.9523E-01, 6.6901E+03, &
      1.221612E-02, 1.12755E+01, 3.2993E-01, 6.9939E+03, &
      1.305903E-02, 1.13472E+01, 3.6970E-01, 7.3311E+03, &
      1.396010E-02, 1.11736E+01, 4.1529E-01, 7.7035E+03, &
      1.492335E-02, 1.09656E+01, 4.6755E-01, 8.1130E+03, &
      1.595306E-02, 1.07127E+01, 5.2742E-01, 8.5613E+03, &
      1.705382E-02, 1.03998E+01, 5.9600E-01, 9.0500E+03, &
      1.823053E-02, 1.00025E+01, 6.7432E-01, 9.5784E+03, &
      1.948844E-02, 9.47980E+00, 7.6357E-01, 1.0146E+04, &
      2.083314E-02, 8.75445E+00, 8.6507E-01, 1.0753E+04, &
      2.227063E-02, 7.65884E+00, 9.8022E-01, 1.1398E+04, &
      2.380730E-02, 5.72391E+00, 1.1105E+00, 1.2079E+04, &
      2.545001E-02, 5.35432E-01, 1.2576E+00, 1.2796E+04, &
      2.577400E-02, -2.09144E+00, 1.2875E+00, 1.2936E+04, &
      2.616850E-02, -1.03771E+01, 1.3243E+00, 1.3105E+04, &
      2.627370E-02, -2.06809E+01, 1.3342E+00, 1.3150E+04, &
      2.643150E-02, -9.41095E+00, 2.1825E+01, 2.1383E+05, &
      2.682600E-02, 5.61530E-01, 2.0871E+01, 2.0147E+05, &
      2.720606E-02, 4.62242E+00, 2.0011E+01, 1.9047E+05, &
      2.908327E-02, 1.27652E+01, 1.6478E+01, 1.4672E+05, &
      3.109002E-02, 1.59925E+01, 1.3594E+01, 1.1323E+05, &
      3.323523E-02, 1.74640E+01, 1.1256E+01, 8.7701E+04, &
      3.552846E-02, 1.80436E+01, 9.4549E+00, 6.8914E+04, &
      3.797993E-02, 1.81477E+01, 8.1320E+00, 5.5446E+04, &
      4.060054E-02, 1.80083E+01, 7.2103E+00, 4.5988E+04, &
      4.340198E-02, 1.77603E+01, 6.6139E+00, 3.9462E+04, &
      4.639671E-02, 1.74826E+01, 6.2771E+00, 3.5034E+04, &
      4.959809E-02, 1.72211E+01, 6.1460E+00, 3.2089E+04, &
      5.302035E-02, 1.70027E+01, 6.1788E+00, 3.0178E+04, &
      5.667876E-02, 1.68431E+01, 6.3436E+00, 2.8983E+04, &
      6.058959E-02, 1.67508E+01, 6.6164E+00, 2.8278E+04, &
      6.164200E-02, 1.67367E+01, 6.7021E+00, 2.8155E+04, &
      6.258550E-02, 1.67217E+01, 6.7824E+00, 2.8063E+04, &
      6.283710E-02, 1.67082E+01, 6.8044E+00, 2.8041E+04, &
      6.321450E-02, 1.67212E+01, 6.8637E+00, 2.8117E+04, &
      6.415800E-02, 1.67362E+01, 6.9503E+00, 2.8053E+04, &
      6.477028E-02, 1.67444E+01, 7.0080E+00, 2.8018E+04, &
      6.923942E-02, 1.68490E+01, 7.4562E+00, 2.7886E+04, &
      7.401695E-02, 1.70661E+01, 7.9731E+00, 2.7895E+04, &
      7.912411E-02, 1.74445E+01, 8.5332E+00, 2.7927E+04, &
      8.458368E-02, 1.78946E+01, 8.7702E+00, 2.6850E+04, &
      9.041995E-02, 1.83409E+01, 8.9903E+00, 2.5747E+04, &
      9.665893E-02, 1.87849E+01, 9.1862E+00, 2.4610E+04, &
      1.0000000E-01, 2.14950E+01, 9.2747E+00, 2.4018E+04, &
      1.0050000E-01, 2.15276E+01, 9.2870E+00, 2.3930E+04, &
      1.0100250E-01, 2.15600E+01, 9.2991E+00, 2.3842E+04, &
      1.0150751E-01, 2.15924E+01, 9.3110E+00, 2.3754E+04, &
      1.0201505E-01, 2.16246E+01, 9.3228E+00, 2.3665E+04, &
      1.0252513E-01, 2.16568E+01, 9.3343E+00, 2.3577E+04, &
      1.0303775E-01, 2.16888E+01, 9.3456E+00, 2.3488E+04, &
      1.0355294E-01, 2.17206E+01, 9.3568E+00, 2.3399E+04, &
      1.0407070E-01, 2.17524E+01, 9.3677E+00, 2.3310E+04, &
      1.0459106E-01, 2.17839E+01, 9.3785E+00, 2.3221E+04, &
      1.0511401E-01, 2.18153E+01, 9.3890E+00, 2.3131E+04, &
      1.0563958E-01, 2.18466E+01, 9.3994E+00, 2.3041E+04, &
      1.0616778E-01, 2.18776E+01, 9.4095E+00, 2.2952E+04, &
      1.0669862E-01, 2.19084E+01, 9.4195E+00, 2.2861E+04, &
      1.0723211E-01, 2.19391E+01, 9.4292E+00, 2.2771E+04, &
      1.0776827E-01, 2.19695E+01, 9.4388E+00, 2.2681E+04, &
      1.0830712E-01, 2.19997E+01, 9.4481E+00, 2.2590E+04, &
      1.0884865E-01, 2.20296E+01, 9.4572E+00, 2.2500E+04, &
      1.0939289E-01, 2.20593E+01, 9.4661E+00, 2.2409E+04, &
      1.0993986E-01, 2.20887E+01, 9.4748E+00, 2.2318E+04, &
      1.1048956E-01, 2.21178E+01, 9.4833E+00, 2.2227E+04, &
      1.1104201E-01, 2.21466E+01, 9.4916E+00, 2.2135E+04, &
      1.1159722E-01, 2.21751E+01, 9.4997E+00, 2.2044E+04, &
      1.1215520E-01, 2.22033E+01, 9.5075E+00, 2.1953E+04, &
      1.1271598E-01, 2.22311E+01, 9.5151E+00, 2.1861E+04, &
      1.1327956E-01, 2.22585E+01, 9.5225E+00, 2.1769E+04, &
      1.1384596E-01, 2.22856E+01, 9.5297E+00, 2.1677E+04, &
      1.1441519E-01, 2.23122E+01, 9.5367E+00, 2.1585E+04, &
      1.1498726E-01, 2.23384E+01, 9.5434E+00, 2.1493E+04, &
      1.1556220E-01, 2.23642E+01, 9.5499E+00, 2.1400E+04, &
      1.1614001E-01, 2.23894E+01, 9.5562E+00, 2.1308E+04, &
      1.1672071E-01, 2.24141E+01, 9.5623E+00, 2.1215E+04, &
      1.1730431E-01, 2.24384E+01, 9.5681E+00, 2.1123E+04, &
      1.1789083E-01, 2.24620E+01, 9.5737E+00, 2.1030E+04, &
      1.1848029E-01, 2.24850E+01, 9.5791E+00, 2.0937E+04, &
      1.1907269E-01, 2.25075E+01, 9.5842E+00, 2.0844E+04, &
      1.1966805E-01, 2.25292E+01, 9.5891E+00, 2.0751E+04, &
      1.2026639E-01, 2.25502E+01, 9.5938E+00, 2.0658E+04, &
      1.2086772E-01, 2.25706E+01, 9.5983E+00, 2.0565E+04, &
      1.2147206E-01, 2.25901E+01, 9.6025E+00, 2.0471E+04, &
      1.2207942E-01, 2.26088E+01, 9.6065E+00, 2.0378E+04, &
      1.2268982E-01, 2.26266E+01, 9.6102E+00, 2.0285E+04, &
      1.2330327E-01, 2.26435E+01, 9.6137E+00, 2.0191E+04, &
      1.2391979E-01, 2.26594E+01, 9.6170E+00, 2.0097E+04, &
      1.2453939E-01, 2.26743E+01, 9.6200E+00, 2.0004E+04, &
      1.2516208E-01, 2.26880E+01, 9.6228E+00, 1.9910E+04, &
      1.2578789E-01, 2.27006E+01, 9.6254E+00, 1.9816E+04, &
      1.2641683E-01, 2.27119E+01, 9.6277E+00, 1.9722E+04, &
      1.2704892E-01, 2.27219E+01, 9.6298E+00, 1.9629E+04, &
      1.2768416E-01, 2.27305E+01, 9.6317E+00, 1.9535E+04, &
      1.2832258E-01, 2.27376E+01, 9.6333E+00, 1.9441E+04, &
      1.2896419E-01, 2.27430E+01, 9.6346E+00, 1.9347E+04, &
      1.2960902E-01, 2.27467E+01, 9.6358E+00, 1.9253E+04, &
      1.3025706E-01, 2.27486E+01, 9.6366E+00, 1.9159E+04, &
      1.3090835E-01, 2.27484E+01, 9.6373E+00, 1.9065E+04, &
      1.3156289E-01, 2.27461E+01, 9.6377E+00, 1.8971E+04, &
      1.3222070E-01, 2.27414E+01, 9.6379E+00, 1.8877E+04, &
      1.3288181E-01, 2.27343E+01, 9.6378E+00, 1.8783E+04, &
      1.3354621E-01, 2.27244E+01, 9.6375E+00, 1.8689E+04, &
      1.3421395E-01, 2.27116E+01, 9.6369E+00, 1.8595E+04, &
      1.3488502E-01, 2.26955E+01, 9.6361E+00, 1.8501E+04, &
      1.3555944E-01, 2.26760E+01, 9.6351E+00, 1.8407E+04, &
      1.3623724E-01, 2.26526E+01, 9.6338E+00, 1.8313E+04, &
      1.3691842E-01, 2.26249E+01, 9.6323E+00, 1.8219E+04, &
      1.3760302E-01, 2.25927E+01, 9.6305E+00, 1.8125E+04, &
      1.3829103E-01, 2.25552E+01, 9.6285E+00, 1.8031E+04, &
      1.3898249E-01, 2.25121E+01, 9.6262E+00, 1.7937E+04, &
      1.3967740E-01, 2.24625E+01, 9.6237E+00, 1.7843E+04, &
      1.4037579E-01, 2.24057E+01, 9.6210E+00, 1.7749E+04, &
      1.4107766E-01, 2.23409E+01, 9.6180E+00, 1.7655E+04, &
      1.4178305E-01, 2.22668E+01, 9.6148E+00, 1.7562E+04, &
      1.4249197E-01, 2.21822E+01, 9.6114E+00, 1.7468E+04, &
      1.4320443E-01, 2.20854E+01, 9.6077E+00, 1.7375E+04, &
      1.4392045E-01, 2.19746E+01, 9.6038E+00, 1.7281E+04, &
      1.4464005E-01, 2.18471E+01, 9.5996E+00, 1.7188E+04, &
      1.4536325E-01, 2.17000E+01, 9.5952E+00, 1.7094E+04, &
      1.4609007E-01, 2.15292E+01, 9.5905E+00, 1.7001E+04, &
      1.4682052E-01, 2.13295E+01, 9.5856E+00, 1.6908E+04, &
      1.4755462E-01, 2.10937E+01, 9.5805E+00, 1.6815E+04, &
      1.4829239E-01, 2.08121E+01, 9.5751E+00, 1.6722E+04, &
      1.4903386E-01, 2.04705E+01, 9.5695E+00, 1.6629E+04, &
      1.4977903E-01, 2.00477E+01, 9.5637E+00, 1.6536E+04, &
      1.5052792E-01, 1.95095E+01, 9.5576E+00, 1.6443E+04, &
      1.5128056E-01, 1.87967E+01, 9.5513E+00, 1.6351E+04, &
      1.5203696E-01, 1.77930E+01, 9.5448E+00, 1.6258E+04, &
      1.5279715E-01, 1.62198E+01, 9.5380E+00, 1.6166E+04, &
      1.5356113E-01, 1.30688E+01, 9.5310E+00, 1.6074E+04, &
      1.5414295E-01, 2.08158E+00, 9.5255E+00, 1.6004E+04, &
      1.5425706E-01, 2.30352E+00, 2.4699E+01, 4.1463E+04, &
      1.5432894E-01, 6.37497E+00, 2.4616E+01, 4.1306E+04, &
      1.5510058E-01, 1.65698E+01, 2.3763E+01, 3.9676E+04, &
      1.5587609E-01, 2.00324E+01, 2.2957E+01, 3.8139E+04, &
      1.5665547E-01, 2.21796E+01, 2.2195E+01, 3.6691E+04, &
      1.5743875E-01, 2.37146E+01, 2.1476E+01, 3.5325E+04, &
      1.5822594E-01, 2.48860E+01, 2.0796E+01, 3.4036E+04, &
      1.5901707E-01, 2.58134E+01, 2.0153E+01, 3.2820E+04, &
      1.5981215E-01, 2.65647E+01, 1.9546E+01, 3.1673E+04, &
      1.6061121E-01, 2.71824E+01, 1.8972E+01, 3.0590E+04, &
      1.6141427E-01, 2.76954E+01, 1.8430E+01, 2.9568E+04, &
      1.6222134E-01, 2.81242E+01, 1.7917E+01, 2.8602E+04, &
      1.6303245E-01, 2.84839E+01, 1.7432E+01, 2.7689E+04, &
      1.6384761E-01, 2.87863E+01, 1.6973E+01, 2.6827E+04, &
      1.6466685E-01, 2.90402E+01, 1.6540E+01, 2.6012E+04, &
      1.6549018E-01, 2.92530E+01, 1.6130E+01, 2.5241E+04, &
      1.6631763E-01, 2.94304E+01, 1.5742E+01, 2.4511E+04, &
      1.6714922E-01, 2.95770E+01, 1.5375E+01, 2.3821E+04, &
      1.6798497E-01, 2.96966E+01, 1.5028E+01, 2.3167E+04, &
      1.6882489E-01, 2.97922E+01, 1.4699E+01, 2.2548E+04, &
      1.6966902E-01, 2.98659E+01, 1.4388E+01, 2.1961E+04, &
      1.7051736E-01, 2.99196E+01, 1.4097E+01, 2.1409E+04, &
      1.7136995E-01, 2.99573E+01, 1.3827E+01, 2.0896E+04, &
      1.7222680E-01, 2.99828E+01, 1.3578E+01, 2.0418E+04, &
      1.7308793E-01, 2.99991E+01, 1.3348E+01, 1.9971E+04, &
      1.7395337E-01, 3.00081E+01, 1.3134E+01, 1.9553E+04, &
      1.7482314E-01, 3.00115E+01, 1.2935E+01, 1.9161E+04, &
      1.7569726E-01, 3.00105E+01, 1.2750E+01, 1.8793E+04, &
      1.7657574E-01, 3.00061E+01, 1.2577E+01, 1.8447E+04, &
      1.7745862E-01, 2.99991E+01, 1.2416E+01, 1.8120E+04, &
      1.7834591E-01, 2.99902E+01, 1.2266E+01, 1.7811E+04, &
      1.7923764E-01, 2.99798E+01, 1.2125E+01, 1.7519E+04, &
      1.8013383E-01, 2.99684E+01, 1.1992E+01, 1.7241E+04, &
      1.8103450E-01, 2.99563E+01, 1.1868E+01, 1.6978E+04, &
      1.8193967E-01, 2.99439E+01, 1.1751E+01, 1.6727E+04, &
      1.8284937E-01, 2.99313E+01, 1.1641E+01, 1.6488E+04, &
      1.8376362E-01, 2.99187E+01, 1.1537E+01, 1.6260E+04, &
      1.8468244E-01, 2.99063E+01, 1.1439E+01, 1.6042E+04, &
      1.8560585E-01, 2.98943E+01, 1.1347E+01, 1.5832E+04, &
      1.8653388E-01, 2.98825E+01, 1.1259E+01, 1.5632E+04, &
      1.8746655E-01, 2.98713E+01, 1.1176E+01, 1.5439E+04, &
      1.8840388E-01, 2.98606E+01, 1.1097E+01, 1.5254E+04, &
      1.8934590E-01, 2.98520E+01, 1.1021E+01, 1.5075E+04, &
      1.9029263E-01, 2.98424E+01, 1.0950E+01, 1.4902E+04, &
      1.9124409E-01, 2.98334E+01, 1.0882E+01, 1.4736E+04, &
      1.9220031E-01, 2.98250E+01, 1.0816E+01, 1.4575E+04, &
      1.9316131E-01, 2.98172E+01, 1.0754E+01, 1.4419E+04, &
      1.9412712E-01, 2.98101E+01, 1.0694E+01, 1.4268E+04, &
      1.9509776E-01, 2.98035E+01, 1.0637E+01, 1.4121E+04, &
      1.9607325E-01, 2.97975E+01, 1.0583E+01, 1.3978E+04, &
      1.9705361E-01, 2.97921E+01, 1.0530E+01, 1.3840E+04, &
      1.9803888E-01, 2.97873E+01, 1.0479E+01, 1.3705E+04, &
      1.9902907E-01, 2.97830E+01, 1.0431E+01, 1.3573E+04, &
      2.0002422E-01, 2.97793E+01, 1.0384E+01, 1.3445E+04, &
      2.0102434E-01, 2.97760E+01, 1.0339E+01, 1.3320E+04, &
      2.0202946E-01, 2.97732E+01, 1.0295E+01, 1.3198E+04, &
      2.0303961E-01, 2.97709E+01, 1.0253E+01, 1.3078E+04, &
      2.0405481E-01, 2.97690E+01, 1.0212E+01, 1.2961E+04, &
      2.0507508E-01, 2.97675E+01, 1.0172E+01, 1.2847E+04, &
      2.0610046E-01, 2.97664E+01, 1.0134E+01, 1.2735E+04, &
      2.0713096E-01, 2.97657E+01, 1.0097E+01, 1.2625E+04, &
      2.0816661E-01, 2.97653E+01, 1.0061E+01, 1.2518E+04, &
      2.0920745E-01, 2.97652E+01, 1.0026E+01, 1.2412E+04, &
      2.1025348E-01, 2.97655E+01, 9.9921E+00, 1.2309E+04, &
      2.1130475E-01, 2.97660E+01, 9.9590E+00, 1.2207E+04, &
      2.1236128E-01, 2.97667E+01, 9.9268E+00, 1.2107E+04, &
      2.1342308E-01, 2.97677E+01, 9.8954E+00, 1.2009E+04, &
      2.1449020E-01, 2.97689E+01, 9.8648E+00, 1.1912E+04, &
      2.1556265E-01, 2.97702E+01, 9.8350E+00, 1.1817E+04, &
      2.1664046E-01, 2.97717E+01, 9.8059E+00, 1.1723E+04, &
      2.1772366E-01, 2.97734E+01, 9.7775E+00, 1.1631E+04, &
      2.1881228E-01, 2.97752E+01, 9.7497E+00, 1.1540E+04, &
      2.1990634E-01, 2.97771E+01, 9.7226E+00, 1.1451E+04, &
      2.2100588E-01, 2.97791E+01, 9.6960E+00, 1.1363E+04, &
      2.2211090E-01, 2.97811E+01, 9.6701E+00, 1.1276E+04, &
      2.2322146E-01, 2.97832E+01, 9.6447E+00, 1.1191E+04, &
      2.2433757E-01, 2.97853E+01, 9.6198E+00, 1.1106E+04, &
      2.2545925E-01, 2.97874E+01, 9.5955E+00, 1.1023E+04, &
      2.2658655E-01, 2.97895E+01, 9.5716E+00, 1.0941E+04, &
      2.2771948E-01, 2.97915E+01, 9.5482E+00, 1.0860E+04, &
      2.2885808E-01, 2.97935E+01, 9.5253E+00, 1.0780E+04, &
      2.3000237E-01, 2.97955E+01, 9.5029E+00, 1.0701E+04, &
      2.3115238E-01, 2.97974E+01, 9.4809E+00, 1.0623E+04, &
      2.3230814E-01, 2.97991E+01, 9.4593E+00, 1.0547E+04, &
      2.3346969E-01, 2.98008E+01, 9.4381E+00, 1.0471E+04, &
      2.3463703E-01, 2.98023E+01, 9.4173E+00, 1.0396E+04, &
      2.3581022E-01, 2.98036E+01, 9.3969E+00, 1.0322E+04, &
      2.3698927E-01, 2.98048E+01, 9.3769E+00, 1.0248E+04, &
      2.3817422E-01, 2.98058E+01, 9.3573E+00, 1.0176E+04, &
      2.3936509E-01, 2.98065E+01, 9.3381E+00, 1.0105E+04, &
      2.4056191E-01, 2.98071E+01, 9.3192E+00, 1.0034E+04, &
      2.4176472E-01, 2.98074E+01, 9.3007E+00, 9.9644E+03, &
      2.4297355E-01, 2.98074E+01, 9.2825E+00, 9.8955E+03, &
      2.4418841E-01, 2.98071E+01, 9.2647E+00, 9.8274E+03, &
      2.4540936E-01, 2.98065E+01, 9.2472E+00, 9.7601E+03, &
      2.4663640E-01, 2.98056E+01, 9.2300E+00, 9.6935E+03, &
      2.4786959E-01, 2.98043E+01, 9.2132E+00, 9.6278E+03, &
      2.4910893E-01, 2.98027E+01, 9.1967E+00, 9.5628E+03, &
      2.5035448E-01, 2.98006E+01, 9.1806E+00, 9.4985E+03, &
      2.5160625E-01, 2.97981E+01, 9.1648E+00, 9.4350E+03, &
      2.5286428E-01, 2.97951E+01, 9.1492E+00, 9.3722E+03, &
      2.5412860E-01, 2.97916E+01, 9.1341E+00, 9.3101E+03, &
      2.5539925E-01, 2.97875E+01, 9.1192E+00, 9.2488E+03, &
      2.5667624E-01, 2.97829E+01, 9.1046E+00, 9.1881E+03, &
      2.5795962E-01, 2.97777E+01, 9.0904E+00, 9.1281E+03, &
      2.5924942E-01, 2.97718E+01, 9.0765E+00, 9.0688E+03, &
      2.6054567E-01, 2.97651E+01, 9.0628E+00, 9.0102E+03, &
      2.6184840E-01, 2.97577E+01, 9.0495E+00, 8.9522E+03, &
      2.6315764E-01, 2.97494E+01, 9.0365E+00, 8.8949E+03, &
      2.6447343E-01, 2.97402E+01, 9.0238E+00, 8.8383E+03, &
      2.6579579E-01, 2.97300E+01, 9.0115E+00, 8.7823E+03, &
      2.6712477E-01, 2.97187E+01, 8.9994E+00, 8.7269E+03, &
      2.6846040E-01, 2.97062E+01, 8.9877E+00, 8.6722E+03, &
      2.6980270E-01, 2.96923E+01, 8.9762E+00, 8.6181E+03, &
      2.7115171E-01, 2.96769E+01, 8.9651E+00, 8.5646E+03, &
      2.7250747E-01, 2.96598E+01, 8.9542E+00, 8.5118E+03, &
      2.7387001E-01, 2.96408E+01, 8.9437E+00, 8.4595E+03, &
      2.7523936E-01, 2.96196E+01, 8.9335E+00, 8.4079E+03, &
      2.7661556E-01, 2.95958E+01, 8.9236E+00, 8.3568E+03, &
      2.7799863E-01, 2.95690E+01, 8.9140E+00, 8.3063E+03, &
      2.7938863E-01, 2.95386E+01, 8.9047E+00, 8.2564E+03, &
      2.8078557E-01, 2.95039E+01, 8.8958E+00, 8.2071E+03, &
      2.8218950E-01, 2.94638E+01, 8.8871E+00, 8.1584E+03, &
      2.8360044E-01, 2.94169E+01, 8.8788E+00, 8.1102E+03, &
      2.8501845E-01, 2.93608E+01, 8.8707E+00, 8.0626E+03, &
      2.8644354E-01, 2.92919E+01, 8.8630E+00, 8.0155E+03, &
      2.8787576E-01, 2.92039E+01, 8.8556E+00, 7.9690E+03, &
      2.8931514E-01, 2.90837E+01, 8.8485E+00, 7.9230E+03, &
      2.9076171E-01, 2.88976E+01, 8.8417E+00, 7.8776E+03, &
      2.9221552E-01, 2.84835E+01, 8.8352E+00, 7.8327E+03, &
      2.9262731E-01, 2.81482E+01, 8.8334E+00, 7.8201E+03, &
      2.9317269E-01, 2.81476E+01, 9.9758E+00, 8.8146E+03, &
      2.9367660E-01, 2.85284E+01, 9.9751E+00, 8.7989E+03, &
      2.9514498E-01, 2.89155E+01, 9.9735E+00, 8.7537E+03, &
      2.9662071E-01, 2.91008E+01, 9.9722E+00, 8.7091E+03, &
      2.9810381E-01, 2.92247E+01, 9.9711E+00, 8.6649E+03, &
      2.9959433E-01, 2.93182E+01, 9.9705E+00, 8.6212E+03, &
      3.0109230E-01, 2.93936E+01, 9.9701E+00, 8.5781E+03, &
      3.0259776E-01, 2.94569E+01, 9.9701E+00, 8.5354E+03, &
      3.0411075E-01, 2.95114E+01, 9.9704E+00, 8.4932E+03, &
      3.0563130E-01, 2.95594E+01, 9.9710E+00, 8.4515E+03, &
      3.0715946E-01, 2.96020E+01, 9.9719E+00, 8.4103E+03, &
      3.0869526E-01, 2.96404E+01, 9.9732E+00, 8.3696E+03, &
      3.1023873E-01, 2.96750E+01, 9.9748E+00, 8.3293E+03, &
      3.1178993E-01, 2.97065E+01, 9.9767E+00, 8.2895E+03, &
      3.1334888E-01, 2.97352E+01, 9.9789E+00, 8.2501E+03, &
      3.1491562E-01, 2.97611E+01, 9.9814E+00, 8.2112E+03, &
      3.1649020E-01, 2.97845E+01, 9.9843E+00, 8.1727E+03, &
      3.1807265E-01, 2.98052E+01, 9.9874E+00, 8.1346E+03, &
      3.1966301E-01, 2.98233E+01, 9.9909E+00, 8.0970E+03, &
      3.2126133E-01, 2.98384E+01, 9.9947E+00, 8.0598E+03, &
      3.2286764E-01, 2.98499E+01, 9.9988E+00, 8.0230E+03, &
      3.2448197E-01, 2.98570E+01, 1.0003E+01, 7.9866E+03, &
      3.2610438E-01, 2.98579E+01, 1.0008E+01, 7.9506E+03, &
      3.2773491E-01, 2.98488E+01, 1.0013E+01, 7.9150E+03, &
      3.2937358E-01, 2.98200E+01, 1.0018E+01, 7.8797E+03, &
      3.3102045E-01, 2.97246E+01, 1.0023E+01, 7.8447E+03, &
      3.3144831E-01, 2.96465E+01, 1.0025E+01, 7.8358E+03, &
      3.3215172E-01, 2.96565E+01, 1.0360E+01, 8.0807E+03, &
      3.3267555E-01, 2.97606E+01, 1.0362E+01, 8.0698E+03, &
      3.3433893E-01, 2.98975E+01, 1.0370E+01, 8.0356E+03, &
      3.3601062E-01, 2.99753E+01, 1.0378E+01, 8.0018E+03, &
      3.3769068E-01, 3.00349E+01, 1.0386E+01, 7.9682E+03, &
      3.3937913E-01, 3.00855E+01, 1.0394E+01, 7.9350E+03, &
      3.4107602E-01, 3.01306E+01, 1.0403E+01, 7.9021E+03, &
      3.4278140E-01, 3.01722E+01, 1.0412E+01, 7.8695E+03, &
      3.4449531E-01, 3.02110E+01, 1.0421E+01, 7.8372E+03, &
      3.4621779E-01, 3.02479E+01, 1.0430E+01, 7.8052E+03, &
      3.4794888E-01, 3.02831E+01, 1.0440E+01, 7.7735E+03, &
      3.4968862E-01, 3.03171E+01, 1.0450E+01, 7.7421E+03, &
      3.5143706E-01, 3.03501E+01, 1.0460E+01, 7.7110E+03, &
      3.5319425E-01, 3.03821E+01, 1.0470E+01, 7.6801E+03, &
      3.5496022E-01, 3.04134E+01, 1.0480E+01, 7.6495E+03, &
      3.5673502E-01, 3.04440E+01, 1.0491E+01, 7.6191E+03, &
      3.5851870E-01, 3.04740E+01, 1.0501E+01, 7.5890E+03, &
      3.6031129E-01, 3.05035E+01, 1.0512E+01, 7.5592E+03, &
      3.6211285E-01, 3.05326E+01, 1.0523E+01, 7.5295E+03, &
      3.6392341E-01, 3.05612E+01, 1.0535E+01, 7.5001E+03, &
      3.6574303E-01, 3.05894E+01, 1.0546E+01, 7.4710E+03, &
      3.6757174E-01, 3.06172E+01, 1.0558E+01, 7.4420E+03, &
      3.6940960E-01, 3.06447E+01, 1.0569E+01, 7.4132E+03, &
      3.7125665E-01, 3.06718E+01, 1.0581E+01, 7.3847E+03, &
      3.7311293E-01, 3.06986E+01, 1.0593E+01, 7.3563E+03, &
      3.7497850E-01, 3.07250E+01, 1.0605E+01, 7.3281E+03, &
      3.7685339E-01, 3.07510E+01, 1.0618E+01, 7.3001E+03, &
      3.7873766E-01, 3.07767E+01, 1.0630E+01, 7.2723E+03, &
      3.8063135E-01, 3.08019E+01, 1.0642E+01, 7.2446E+03, &
      3.8253450E-01, 3.08267E+01, 1.0655E+01, 7.2171E+03, &
      3.8444718E-01, 3.08509E+01, 1.0668E+01, 7.1897E+03, &
      3.8636941E-01, 3.08747E+01, 1.0680E+01, 7.1625E+03, &
      3.8830126E-01, 3.08977E+01, 1.0693E+01, 7.1354E+03, &
      3.9024276E-01, 3.09201E+01, 1.0706E+01, 7.1085E+03, &
      3.9219398E-01, 3.09415E+01, 1.0719E+01, 7.0817E+03, &
      3.9415495E-01, 3.09619E+01, 1.0732E+01, 7.0550E+03, &
      3.9612572E-01, 3.09811E+01, 1.0745E+01, 7.0284E+03, &
      3.9810635E-01, 3.09987E+01, 1.0757E+01, 7.0019E+03, &
      4.0009688E-01, 3.10144E+01, 1.0770E+01, 6.9755E+03, &
      4.0209737E-01, 3.10275E+01, 1.0783E+01, 6.9492E+03, &
      4.0410785E-01, 3.10374E+01, 1.0796E+01, 6.9230E+03, &
      4.0612839E-01, 3.10426E+01, 1.0809E+01, 6.8969E+03, &
      4.0815904E-01, 3.10411E+01, 1.0822E+01, 6.8709E+03, &
      4.1019983E-01, 3.10287E+01, 1.0835E+01, 6.8449E+03, &
      4.1225083E-01, 3.09962E+01, 1.0848E+01, 6.8190E+03, &
      4.1431208E-01, 3.09136E+01, 1.0861E+01, 6.7932E+03, &
      4.1572967E-01, 3.07302E+01, 1.0870E+01, 6.7755E+03, &
      4.1638364E-01, 3.04221E+01, 1.1397E+01, 7.0924E+03, &
      4.1687033E-01, 3.07496E+01, 1.1400E+01, 7.0861E+03, &
      4.1846556E-01, 3.09990E+01, 1.1410E+01, 7.0657E+03, &
      4.2055789E-01, 3.11479E+01, 1.1424E+01, 7.0391E+03, &
      4.2266068E-01, 3.12515E+01, 1.1438E+01, 7.0126E+03, &
      4.2477398E-01, 3.13364E+01, 1.1452E+01, 6.9861E+03, &
      4.2689785E-01, 3.14111E+01, 1.1465E+01, 6.9596E+03, &
      4.2903234E-01, 3.14795E+01, 1.1479E+01, 6.9331E+03, &
      4.3117750E-01, 3.15435E+01, 1.1492E+01, 6.9067E+03, &
      4.3333339E-01, 3.16044E+01, 1.1505E+01, 6.8803E+03, &
      4.3550006E-01, 3.16630E+01, 1.1518E+01, 6.8539E+03, &
      4.3767756E-01, 3.17198E+01, 1.1531E+01, 6.8275E+03, &
      4.3986595E-01, 3.17752E+01, 1.1544E+01, 6.8011E+03, &
      4.4206528E-01, 3.18296E+01, 1.1557E+01, 6.7748E+03, &
      4.4427560E-01, 3.18830E+01, 1.1569E+01, 6.7484E+03, &
      4.4649698E-01, 3.19357E+01, 1.1582E+01, 6.7220E+03, &
      4.4872947E-01, 3.19878E+01, 1.1594E+01, 6.6956E+03, &
      4.5097311E-01, 3.20395E+01, 1.1606E+01, 6.6692E+03, &
      4.5322798E-01, 3.20907E+01, 1.1617E+01, 6.6428E+03, &
      4.5549412E-01, 3.21417E+01, 1.1629E+01, 6.6164E+03, &
      4.5777159E-01, 3.21923E+01, 1.1640E+01, 6.5899E+03, &
      4.6006045E-01, 3.22428E+01, 1.1651E+01, 6.5635E+03, &
      4.6236075E-01, 3.22930E+01, 1.1662E+01, 6.5369E+03, &
      4.6467255E-01, 3.23492E+01, 1.1673E+01, 6.5104E+03, &
      4.6699592E-01, 3.23992E+01, 1.1683E+01, 6.4838E+03, &
      4.6933090E-01, 3.24491E+01, 1.1693E+01, 6.4571E+03, &
      4.7167755E-01, 3.24990E+01, 1.1703E+01, 6.4304E+03, &
      4.7403594E-01, 3.25488E+01, 1.1713E+01, 6.4036E+03, &
      4.7640612E-01, 3.25985E+01, 1.1722E+01, 6.3768E+03, &
      4.7878815E-01, 3.26481E+01, 1.1731E+01, 6.3499E+03, &
      4.8118209E-01, 3.26978E+01, 1.1739E+01, 6.3230E+03, &
      4.8358800E-01, 3.27474E+01, 1.1747E+01, 6.2960E+03, &
      4.8600594E-01, 3.27969E+01, 1.1755E+01, 6.2689E+03, &
      4.8843597E-01, 3.28465E+01, 1.1763E+01, 6.2418E+03, &
      4.9087815E-01, 3.28960E+01, 1.1770E+01, 6.2147E+03, &
      4.9333254E-01, 3.29455E+01, 1.1777E+01, 6.1874E+03, &
      4.9579920E-01, 3.29949E+01, 1.1784E+01, 6.1601E+03, &
      4.9827820E-01, 3.30444E+01, 1.1790E+01, 6.1327E+03, &
      5.0076959E-01, 3.30938E+01, 1.1795E+01, 6.1052E+03, &
      5.0327344E-01, 3.31432E+01, 1.1801E+01, 6.0777E+03, &
      5.0578980E-01, 3.31926E+01, 1.1806E+01, 6.0501E+03, &
      5.0831875E-01, 3.32419E+01, 1.1811E+01, 6.0224E+03, &
      5.1086035E-01, 3.32912E+01, 1.1815E+01, 5.9947E+03, &
      5.1341465E-01, 3.33404E+01, 1.1819E+01, 5.9669E+03, &
      5.1598172E-01, 3.33896E+01, 1.1822E+01, 5.9390E+03, &
      5.1856163E-01, 3.34388E+01, 1.1826E+01, 5.9111E+03, &
      5.2115444E-01, 3.34878E+01, 1.1828E+01, 5.8831E+03, &
      5.2376021E-01, 3.35369E+01, 1.1831E+01, 5.8551E+03, &
      5.2637901E-01, 3.35858E+01, 1.1833E+01, 5.8270E+03, &
      5.2901091E-01, 3.36347E+01, 1.1834E+01, 5.7988E+03, &
      5.3165596E-01, 3.36835E+01, 1.1835E+01, 5.7706E+03, &
      5.3431424E-01, 3.37323E+01, 1.1836E+01, 5.7423E+03, &
      5.3698581E-01, 3.37809E+01, 1.1837E+01, 5.7140E+03, &
      5.3967074E-01, 3.38294E+01, 1.1837E+01, 5.6856E+03, &
      5.4236910E-01, 3.38779E+01, 1.1836E+01, 5.6571E+03, &
      5.4508094E-01, 3.39262E+01, 1.1835E+01, 5.6286E+03, &
      5.4780635E-01, 3.39744E+01, 1.1834E+01, 5.6001E+03, &
      5.5054538E-01, 3.40224E+01, 1.1832E+01, 5.5715E+03, &
      5.5329810E-01, 3.40703E+01, 1.1830E+01, 5.5428E+03, &
      5.5606460E-01, 3.41181E+01, 1.1828E+01, 5.5141E+03, &
      5.5884492E-01, 3.41657E+01, 1.1825E+01, 5.4854E+03, &
      5.6163914E-01, 3.42132E+01, 1.1821E+01, 5.4566E+03, &
      5.6444734E-01, 3.42605E+01, 1.1817E+01, 5.4277E+03, &
      5.6726958E-01, 3.43076E+01, 1.1813E+01, 5.3989E+03, &
      5.7010592E-01, 3.43545E+01, 1.1809E+01, 5.3700E+03, &
      5.7295645E-01, 3.44013E+01, 1.1804E+01, 5.3410E+03, &
      5.7582123E-01, 3.44478E+01, 1.1798E+01, 5.3120E+03, &
      5.7870034E-01, 3.44941E+01, 1.1792E+01, 5.2830E+03, &
      5.8159384E-01, 3.45402E+01, 1.1786E+01, 5.2540E+03, &
      5.8450181E-01, 3.45861E+01, 1.1779E+01, 5.2249E+03, &
      5.8742432E-01, 3.46317E+01, 1.1772E+01, 5.1958E+03, &
      5.9036144E-01, 3.46771E+01, 1.1764E+01, 5.1667E+03, &
      5.9331325E-01, 3.47222E+01, 1.1756E+01, 5.1375E+03, &
      5.9627982E-01, 3.47671E+01, 1.1748E+01, 5.1083E+03, &
      5.9926122E-01, 3.48117E+01, 1.1739E+01, 5.0792E+03, &
      6.0225752E-01, 3.48560E+01, 1.1730E+01, 5.0500E+03, &
      6.0526881E-01, 3.49000E+01, 1.1720E+01, 5.0208E+03, &
      6.0829515E-01, 3.49438E+01, 1.1710E+01, 4.9915E+03, &
      6.1133663E-01, 3.49872E+01, 1.1699E+01, 4.9623E+03, &
      6.1439331E-01, 3.50303E+01, 1.1689E+01, 4.9331E+03, &
      6.1746528E-01, 3.50732E+01, 1.1677E+01, 4.9039E+03, &
      6.2055260E-01, 3.51156E+01, 1.1665E+01, 4.8746E+03, &
      6.2365537E-01, 3.51578E+01, 1.1653E+01, 4.8453E+03, &
      6.2677364E-01, 3.51995E+01, 1.1640E+01, 4.8160E+03, &
      6.2990751E-01, 3.52409E+01, 1.1627E+01, 4.7866E+03, &
      6.3305705E-01, 3.52819E+01, 1.1613E+01, 4.7572E+03, &
      6.3622234E-01, 3.53225E+01, 1.1599E+01, 4.7278E+03, &
      6.3940345E-01, 3.53626E+01, 1.1585E+01, 4.6984E+03, &
      6.4260046E-01, 3.54023E+01, 1.1569E+01, 4.6690E+03, &
      6.4581347E-01, 3.54416E+01, 1.1554E+01, 4.6396E+03, &
      6.4904253E-01, 3.54803E+01, 1.1538E+01, 4.6102E+03, &
      6.5228775E-01, 3.55186E+01, 1.1521E+01, 4.5807E+03, &
      6.5554919E-01, 3.55564E+01, 1.1504E+01, 4.5513E+03, &
      6.5882693E-01, 3.55936E+01, 1.1487E+01, 4.5219E+03, &
      6.6212107E-01, 3.56304E+01, 1.1469E+01, 4.4926E+03, &
      6.6543167E-01, 3.56666E+01, 1.1451E+01, 4.4632E+03, &
      6.6875883E-01, 3.57022E+01, 1.1433E+01, 4.4339E+03, &
      6.7210262E-01, 3.57373E+01, 1.1414E+01, 4.4046E+03, &
      6.7546314E-01, 3.57718E+01, 1.1394E+01, 4.3753E+03, &
      6.7884045E-01, 3.58058E+01, 1.1375E+01, 4.3461E+03, &
      6.8223466E-01, 3.58391E+01, 1.1355E+01, 4.3169E+03, &
      6.8564583E-01, 3.58718E+01, 1.1334E+01, 4.2878E+03, &
      6.8907406E-01, 3.59040E+01, 1.1313E+01, 4.2587E+03, &
      6.9251943E-01, 3.59355E+01, 1.1292E+01, 4.2296E+03, &
      6.9598202E-01, 3.59664E+01, 1.1271E+01, 4.2006E+03, &
      6.9946194E-01, 3.59966E+01, 1.1249E+01, 4.1717E+03, &
      7.0295924E-01, 3.60262E+01, 1.1227E+01, 4.1429E+03, &
      7.0647404E-01, 3.60551E+01, 1.1204E+01, 4.1140E+03, &
      7.1000641E-01, 3.60833E+01, 1.1181E+01, 4.0851E+03, &
      7.1355644E-01, 3.61107E+01, 1.1157E+01, 4.0563E+03, &
      7.1712423E-01, 3.61374E+01, 1.1133E+01, 4.0276E+03, &
      7.2070985E-01, 3.61633E+01, 1.1109E+01, 3.9989E+03, &
      7.2431340E-01, 3.61884E+01, 1.1085E+01, 3.9703E+03, &
      7.2793496E-01, 3.62128E+01, 1.1060E+01, 3.9419E+03, &
      7.3157464E-01, 3.62364E+01, 1.1035E+01, 3.9135E+03, &
      7.3523251E-01, 3.62592E+01, 1.1010E+01, 3.8852E+03, &
      7.3890867E-01, 3.62811E+01, 1.0984E+01, 3.8570E+03, &
      7.4260322E-01, 3.63023E+01, 1.0959E+01, 3.8289E+03, &
      7.4631623E-01, 3.63226E+01, 1.0933E+01, 3.8009E+03, &
      7.5004781E-01, 3.63421E+01, 1.0907E+01, 3.7730E+03, &
      7.5379805E-01, 3.63608E+01, 1.0880E+01, 3.7452E+03, &
      7.5756704E-01, 3.63785E+01, 1.0853E+01, 3.7175E+03, &
      7.6135488E-01, 3.63954E+01, 1.0826E+01, 3.6899E+03, &
      7.6516165E-01, 3.64115E+01, 1.0799E+01, 3.6624E+03, &
      7.6898746E-01, 3.64266E+01, 1.0772E+01, 3.6351E+03, &
      7.7283240E-01, 3.64408E+01, 1.0745E+01, 3.6078E+03, &
      7.7669656E-01, 3.64541E+01, 1.0717E+01, 3.5807E+03, &
      7.8058004E-01, 3.64665E+01, 1.0689E+01, 3.5537E+03, &
      7.8448294E-01, 3.64779E+01, 1.0661E+01, 3.5268E+03, &
      7.8840536E-01, 3.64883E+01, 1.0633E+01, 3.5000E+03, &
      7.9234738E-01, 3.64978E+01, 1.0604E+01, 3.4734E+03, &
      7.9630912E-01, 3.65063E+01, 1.0576E+01, 3.4469E+03, &
      8.0029067E-01, 3.65138E+01, 1.0547E+01, 3.4205E+03, &
      8.0429212E-01, 3.65203E+01, 1.0518E+01, 3.3943E+03, &
      8.0831358E-01, 3.65258E+01, 1.0489E+01, 3.3682E+03, &
      8.1235515E-01, 3.65302E+01, 1.0460E+01, 3.3422E+03, &
      8.1641693E-01, 3.65336E+01, 1.0431E+01, 3.3164E+03, &
      8.2049901E-01, 3.65359E+01, 1.0402E+01, 3.2907E+03, &
      8.2460150E-01, 3.65371E+01, 1.0372E+01, 3.2651E+03, &
      8.2872451E-01, 3.65372E+01, 1.0343E+01, 3.2397E+03, &
      8.3286813E-01, 3.65362E+01, 1.0313E+01, 3.2144E+03, &
      8.3703248E-01, 3.65341E+01, 1.0283E+01, 3.1893E+03, &
      8.4121764E-01, 3.65307E+01, 1.0253E+01, 3.1643E+03, &
      8.4542373E-01, 3.65263E+01, 1.0223E+01, 3.1394E+03, &
      8.4965084E-01, 3.65206E+01, 1.0193E+01, 3.1147E+03, &
      8.5389910E-01, 3.65137E+01, 1.0163E+01, 3.0901E+03, &
      8.5816859E-01, 3.65056E+01, 1.0133E+01, 3.0657E+03, &
      8.6245944E-01, 3.64962E+01, 1.0103E+01, 3.0414E+03, &
      8.6677173E-01, 3.64855E+01, 1.0073E+01, 3.0173E+03, &
      8.7110559E-01, 3.64736E+01, 1.0042E+01, 2.9933E+03, &
      8.7546112E-01, 3.64603E+01, 1.0012E+01, 2.9695E+03, &
      8.7983843E-01, 3.64469E+01, 9.9813E+00, 2.9458E+03, &
      8.8423762E-01, 3.64310E+01, 9.9508E+00, 2.9223E+03, &
      8.8865881E-01, 3.64136E+01, 9.9203E+00, 2.8989E+03, &
      8.9310210E-01, 3.63948E+01, 9.8897E+00, 2.8757E+03, &
      8.9756761E-01, 3.63746E+01, 9.8591E+00, 2.8526E+03, &
      9.0205545E-01, 3.63529E+01, 9.8285E+00, 2.8297E+03, &
      9.0656573E-01, 3.63297E+01, 9.7979E+00, 2.8069E+03, &
      9.1109856E-01, 3.63050E+01, 9.7672E+00, 2.7843E+03, &
      9.1565405E-01, 3.62786E+01, 9.7366E+00, 2.7618E+03, &
      9.2023232E-01, 3.62507E+01, 9.7059E+00, 2.7395E+03, &
      9.2483348E-01, 3.62212E+01, 9.6752E+00, 2.7174E+03, &
      9.2945765E-01, 3.61900E+01, 9.6445E+00, 2.6953E+03, &
      9.3410494E-01, 3.61570E+01, 9.6138E+00, 2.6735E+03, &
      9.3877546E-01, 3.61223E+01, 9.5832E+00, 2.6518E+03, &
      9.4346934E-01, 3.60859E+01, 9.5525E+00, 2.6302E+03, &
      9.4818668E-01, 3.60476E+01, 9.5219E+00, 2.6088E+03, &
      9.5292762E-01, 3.60074E+01, 9.4912E+00, 2.5876E+03, &
      9.5769226E-01, 3.59653E+01, 9.4606E+00, 2.5665E+03, &
      9.6248072E-01, 3.59213E+01, 9.4300E+00, 2.5456E+03, &
      9.6729312E-01, 3.58752E+01, 9.3995E+00, 2.5248E+03, &
      9.7212959E-01, 3.58271E+01, 9.3689E+00, 2.5041E+03, &
      9.7699023E-01, 3.57769E+01, 9.3384E+00, 2.4837E+03, &
      9.8187519E-01, 3.57245E+01, 9.3080E+00, 2.4633E+03, &
      9.8678456E-01, 3.56700E+01, 9.2776E+00, 2.4431E+03, &
      9.9171848E-01, 3.56131E+01, 9.2472E+00, 2.4231E+03, &
      9.9667708E-01, 3.55541E+01, 9.2168E+00, 2.4032E+03, &
      1.0016605E+00, 3.55075E+01, 9.1789E+00, 2.3815E+03, &
      1.0066688E+00, 3.54827E+01, 9.1261E+00, 2.3561E+03, &
      1.0117021E+00, 3.54478E+01, 9.0736E+00, 2.3310E+03, &
      1.0167606E+00, 3.54039E+01, 9.0214E+00, 2.3062E+03, &
      1.0218444E+00, 3.53516E+01, 8.9696E+00, 2.2817E+03, &
      1.0269536E+00, 3.52915E+01, 8.9182E+00, 2.2574E+03, &
      1.0320884E+00, 3.52241E+01, 8.8671E+00, 2.2334E+03, &
      1.0372489E+00, 3.51497E+01, 8.8164E+00, 2.2097E+03, &
      1.0424351E+00, 3.50684E+01, 8.7660E+00, 2.1863E+03, &
      1.0476473E+00, 3.49805E+01, 8.7160E+00, 2.1631E+03, &
      1.0528855E+00, 3.48859E+01, 8.6663E+00, 2.1401E+03, &
      1.0581499E+00, 3.47847E+01, 8.6169E+00, 2.1175E+03, &
      1.0634407E+00, 3.46769E+01, 8.5679E+00, 2.0951E+03, &
      1.0687579E+00, 3.45624E+01, 8.5193E+00, 2.0729E+03, &
      1.0741017E+00, 3.44411E+01, 8.4709E+00, 2.0510E+03, &
      1.0794722E+00, 3.43129E+01, 8.4229E+00, 2.0293E+03, &
      1.0848695E+00, 3.41775E+01, 8.3753E+00, 2.0079E+03, &
      1.0902939E+00, 3.40347E+01, 8.3279E+00, 1.9867E+03, &
      1.0957454E+00, 3.38842E+01, 8.2809E+00, 1.9658E+03, &
      1.1012241E+00, 3.37257E+01, 8.2342E+00, 1.9451E+03, &
      1.1067302E+00, 3.35589E+01, 8.1878E+00, 1.9246E+03, &
      1.1122639E+00, 3.33833E+01, 8.1418E+00, 1.9044E+03, &
      1.1178252E+00, 3.31985E+01, 8.0961E+00, 1.8844E+03, &
      1.1234143E+00, 3.30039E+01, 8.0507E+00, 1.8646E+03, &
      1.1290314E+00, 3.27990E+01, 8.0056E+00, 1.8450E+03, &
      1.1346765E+00, 3.25831E+01, 7.9608E+00, 1.8257E+03, &
      1.1403499E+00, 3.23554E+01, 7.9163E+00, 1.8066E+03, &
      1.1460517E+00, 3.21151E+01, 7.8721E+00, 1.7877E+03, &
      1.1517819E+00, 3.18612E+01, 7.8283E+00, 1.7690E+03, &
      1.1575408E+00, 3.15927E+01, 7.7847E+00, 1.7505E+03, &
      1.1633285E+00, 3.13083E+01, 7.7415E+00, 1.7322E+03, &
      1.1691452E+00, 3.10067E+01, 7.6985E+00, 1.7141E+03, &
      1.1749909E+00, 3.06860E+01, 7.6548E+00, 1.6960E+03, &
      1.1808659E+00, 3.03445E+01, 7.6105E+00, 1.6779E+03, &
      1.1867702E+00, 2.99798E+01, 7.5666E+00, 1.6600E+03, &
      1.1927040E+00, 2.95894E+01, 7.5229E+00, 1.6424E+03, &
      1.1986676E+00, 2.91703E+01, 7.4795E+00, 1.6249E+03, &
      1.2046609E+00, 2.87188E+01, 7.4365E+00, 1.6076E+03, &
      1.2106842E+00, 2.82306E+01, 7.3938E+00, 1.5905E+03, &
      1.2167376E+00, 2.77003E+01, 7.3513E+00, 1.5736E+03, &
      1.2228213E+00, 2.71213E+01, 7.3092E+00, 1.5569E+03, &
      1.2289354E+00, 2.64852E+01, 7.2673E+00, 1.5404E+03, &
      1.2350801E+00, 2.57812E+01, 7.2258E+00, 1.5241E+03, &
      1.2412555E+00, 2.49950E+01, 7.1845E+00, 1.5080E+03, &
      1.2474618E+00, 2.41072E+01, 7.1435E+00, 1.4920E+03, &
      1.2536991E+00, 2.30911E+01, 7.1029E+00, 1.4762E+03, &
      1.2599676E+00, 2.19047E+01, 7.0625E+00, 1.4606E+03, &
      1.2662674E+00, 2.04841E+01, 7.0223E+00, 1.4452E+03, &
      1.2725988E+00, 1.87174E+01, 6.9825E+00, 1.4300E+03, &
      1.2789618E+00, 1.63805E+01, 6.9430E+00, 1.4149E+03, &
      1.2853566E+00, 1.28991E+01, 6.9037E+00, 1.4000E+03, &
      1.2917833E+00, 5.58834E+00, 6.8647E+00, 1.3853E+03, &
      1.2947290E+00, -1.27100E+01, 6.8469E+00, 1.3786E+03, &
      1.2950709E+00, -1.30561E+01, 2.6489E+01, 5.3057E+03, &
      1.2982423E+00, 5.35530E+00, 2.6389E+01, 5.2728E+03, &
      1.3047335E+00, 1.15342E+01, 2.6186E+01, 5.2064E+03, &
      1.3112571E+00, 1.38769E+01, 2.5985E+01, 5.1409E+03, &
      1.3178134E+00, 1.47162E+01, 2.5786E+01, 5.0762E+03, &
      1.3244025E+00, 1.40991E+01, 2.5588E+01, 5.0124E+03, &
      1.3310245E+00, 8.65102E+00, 2.5392E+01, 4.9493E+03, &
      1.3321988E+00, 2.39694E+00, 2.5357E+01, 4.9382E+03, &
      1.3328011E+00, 2.31057E+00, 3.8199E+01, 7.4311E+03, &
      1.3376796E+00, 1.49106E+01, 3.7980E+01, 7.3616E+03, &
      1.3443680E+00, 1.94254E+01, 3.7683E+01, 7.2679E+03, &
      1.3510899E+00, 2.22413E+01, 3.7389E+01, 7.1754E+03, &
      1.3578453E+00, 2.43810E+01, 3.7097E+01, 7.0840E+03, &
      1.3646345E+00, 2.61366E+01, 3.6807E+01, 6.9939E+03, &
      1.3714577E+00, 2.76375E+01, 3.6520E+01, 6.9049E+03, &
      1.3783150E+00, 2.89537E+01, 3.6235E+01, 6.8170E+03, &
      1.3852066E+00, 3.01281E+01, 3.5952E+01, 6.7303E+03, &
      1.3921326E+00, 3.11894E+01, 3.5672E+01, 6.6447E+03, &
      1.3990933E+00, 3.21577E+01, 3.5394E+01, 6.5603E+03, &
      1.4060887E+00, 3.30477E+01, 3.5118E+01, 6.4769E+03, &
      1.4131192E+00, 3.38708E+01, 3.4844E+01, 6.3945E+03, &
      1.4201848E+00, 3.46356E+01, 3.4573E+01, 6.3133E+03, &
      1.4272857E+00, 3.53491E+01, 3.4303E+01, 6.2331E+03, &
      1.4344221E+00, 3.60170E+01, 3.4036E+01, 6.1539E+03, &
      1.4415942E+00, 3.66440E+01, 3.3771E+01, 6.0757E+03, &
      1.4488022E+00, 3.72340E+01, 3.3509E+01, 5.9986E+03, &
      1.4560462E+00, 3.77901E+01, 3.3248E+01, 5.9224E+03, &
      1.4633265E+00, 3.83152E+01, 3.2989E+01, 5.8473E+03, &
      1.4706431E+00, 3.88115E+01, 3.2733E+01, 5.7731E+03, &
      1.4779963E+00, 3.92812E+01, 3.2479E+01, 5.6998E+03, &
      1.4853863E+00, 3.97259E+01, 3.2226E+01, 5.6275E+03, &
      1.4928132E+00, 4.01470E+01, 3.1976E+01, 5.5562E+03, &
      1.5002773E+00, 4.05459E+01, 3.1728E+01, 5.4857E+03, &
      1.5077787E+00, 4.09237E+01, 3.1481E+01, 5.4162E+03, &
      1.5153176E+00, 4.12811E+01, 3.1237E+01, 5.3475E+03, &
      1.5228942E+00, 4.16190E+01, 3.0995E+01, 5.2798E+03, &
      1.5305086E+00, 4.19380E+01, 3.0754E+01, 5.2129E+03, &
      1.5381612E+00, 4.22384E+01, 3.0516E+01, 5.1469E+03, &
      1.5458520E+00, 4.25206E+01, 3.0279E+01, 5.0817E+03, &
      1.5535812E+00, 4.27846E+01, 3.0045E+01, 5.0173E+03, &
      1.5613491E+00, 4.30304E+01, 2.9812E+01, 4.9538E+03, &
      1.5691559E+00, 4.32576E+01, 2.9581E+01, 4.8911E+03, &
      1.5770017E+00, 4.34658E+01, 2.9352E+01, 4.8292E+03, &
      1.5848867E+00, 4.36539E+01, 2.9125E+01, 4.7681E+03, &
      1.5928111E+00, 4.38206E+01, 2.8900E+01, 4.7078E+03, &
      1.6007752E+00, 4.39641E+01, 2.8676E+01, 4.6483E+03, &
      1.6087790E+00, 4.40814E+01, 2.8454E+01, 4.5895E+03, &
      1.6168229E+00, 4.41685E+01, 2.8234E+01, 4.5315E+03, &
      1.6249070E+00, 4.42193E+01, 2.8016E+01, 4.4743E+03, &
      1.6330316E+00, 4.42240E+01, 2.7800E+01, 4.4177E+03, &
      1.6411967E+00, 4.41663E+01, 2.7585E+01, 4.3619E+03, &
      1.6494027E+00, 4.40159E+01, 2.7372E+01, 4.3068E+03, &
      1.6576497E+00, 4.37067E+01, 2.7161E+01, 4.2525E+03, &
      1.6659380E+00, 4.30420E+01, 2.6951E+01, 4.1988E+03, &
      1.6740015E+00, 4.06438E+01, 2.6750E+01, 4.1474E+03, &
      1.6742677E+00, 4.03848E+01, 2.6743E+01, 4.1457E+03, &
      1.6771986E+00, 4.06710E+01, 3.1276E+01, 4.8383E+03, &
      1.6826390E+00, 4.30080E+01, 3.1109E+01, 4.7970E+03, &
      1.6910522E+00, 4.43844E+01, 3.0853E+01, 4.7340E+03, &
      1.6995075E+00, 4.52282E+01, 3.0599E+01, 4.6719E+03, &
      1.7080050E+00, 4.58623E+01, 3.0348E+01, 4.6105E+03, &
      1.7165450E+00, 4.63795E+01, 3.0098E+01, 4.5500E+03, &
      1.7251278E+00, 4.68192E+01, 2.9851E+01, 4.4903E+03, &
      1.7337534E+00, 4.72017E+01, 2.9606E+01, 4.4314E+03, &
      1.7424222E+00, 4.75382E+01, 2.9362E+01, 4.3732E+03, &
      1.7511343E+00, 4.78347E+01, 2.9120E+01, 4.3157E+03, &
      1.7598899E+00, 4.80934E+01, 2.8885E+01, 4.2596E+03, &
      1.7686894E+00, 4.83245E+01, 2.8668E+01, 4.2067E+03, &
      1.7775328E+00, 4.85326E+01, 2.8454E+01, 4.1547E+03, &
      1.7864205E+00, 4.87158E+01, 2.8243E+01, 4.1034E+03, &
      1.7953526E+00, 4.88707E+01, 2.8034E+01, 4.0530E+03, &
      1.8043294E+00, 4.89917E+01, 2.7829E+01, 4.0034E+03, &
      1.8133510E+00, 4.90667E+01, 2.7626E+01, 3.9545E+03, &
      1.8224178E+00, 4.90685E+01, 2.7425E+01, 3.9064E+03, &
      1.8315299E+00, 4.89126E+01, 2.7227E+01, 3.8590E+03, &
      1.8399858E+00, 4.80751E+01, 2.7047E+01, 3.8159E+03, &
      1.8406875E+00, 4.77976E+01, 2.7032E+01, 3.8123E+03, &
      1.8436141E+00, 4.81344E+01, 2.8861E+01, 4.0633E+03, &
      1.8498909E+00, 4.91843E+01, 2.8714E+01, 4.0289E+03, &
      1.8591404E+00, 4.98465E+01, 2.8500E+01, 3.9791E+03, &
      1.8684361E+00, 5.02957E+01, 2.8289E+01, 3.9301E+03, &
      1.8777783E+00, 5.06570E+01, 2.8080E+01, 3.8818E+03, &
      1.8871672E+00, 5.09672E+01, 2.7873E+01, 3.8341E+03, &
      1.8966030E+00, 5.12425E+01, 2.7669E+01, 3.7872E+03, &
      1.9060860E+00, 5.14913E+01, 2.7467E+01, 3.7410E+03, &
      1.9156165E+00, 5.17182E+01, 2.7267E+01, 3.6953E+03, &
      1.9251945E+00, 5.19258E+01, 2.7068E+01, 3.6503E+03, &
      1.9348205E+00, 5.21149E+01, 2.6875E+01, 3.6063E+03, &
      1.9444946E+00, 5.22909E+01, 2.6692E+01, 3.5640E+03, &
      1.9542171E+00, 5.24563E+01, 2.6511E+01, 3.5224E+03, &
      1.9639882E+00, 5.26103E+01, 2.6332E+01, 3.4813E+03, &
      1.9738081E+00, 5.27525E+01, 2.6156E+01, 3.4410E+03, &
      1.9836772E+00, 5.28816E+01, 2.5982E+01, 3.4012E+03, &
      1.9935955E+00, 5.29957E+01, 2.5811E+01, 3.3620E+03, &
      2.0035635E+00, 5.30912E+01, 2.5641E+01, 3.3234E+03, &
      2.0135813E+00, 5.31611E+01, 2.5474E+01, 3.2854E+03, &
      2.0236492E+00, 5.31904E+01, 2.5308E+01, 3.2479E+03, &
      2.0337675E+00, 5.31357E+01, 2.5144E+01, 3.2109E+03, &
      2.0432385E+00, 5.27952E+01, 2.4994E+01, 3.1770E+03, &
      2.0439363E+00, 5.27239E+01, 2.4983E+01, 3.1745E+03, &
      2.0503613E+00, 5.28942E+01, 2.6064E+01, 3.3012E+03, &
      2.0541560E+00, 5.32252E+01, 2.6003E+01, 3.2874E+03, &
      2.0644268E+00, 5.37063E+01, 2.5840E+01, 3.2506E+03, &
      2.0747489E+00, 5.40275E+01, 2.5678E+01, 3.2142E+03, &
      2.0851227E+00, 5.42903E+01, 2.5518E+01, 3.1784E+03, &
      2.0955483E+00, 5.45215E+01, 2.5359E+01, 3.1430E+03, &
      2.1060260E+00, 5.47324E+01, 2.5202E+01, 3.1081E+03, &
      2.1165562E+00, 5.49288E+01, 2.5046E+01, 3.0736E+03, &
      2.1271389E+00, 5.51141E+01, 2.4892E+01, 3.0396E+03, &
      2.1377746E+00, 5.52905E+01, 2.4739E+01, 3.0060E+03, &
      2.1484635E+00, 5.54593E+01, 2.4587E+01, 2.9728E+03, &
      2.1592058E+00, 5.56224E+01, 2.4438E+01, 2.9402E+03, &
      2.1700018E+00, 5.57810E+01, 2.4291E+01, 2.9080E+03, &
      2.1808519E+00, 5.59360E+01, 2.4143E+01, 2.8760E+03, &
      2.1917561E+00, 5.60858E+01, 2.3991E+01, 2.8438E+03, &
      2.2027149E+00, 5.62307E+01, 2.3841E+01, 2.8120E+03, &
      2.2137285E+00, 5.63713E+01, 2.3691E+01, 2.7806E+03, &
      2.2247971E+00, 5.65079E+01, 2.3543E+01, 2.7495E+03, &
      2.2359211E+00, 5.66409E+01, 2.3396E+01, 2.7189E+03, &
      2.2471007E+00, 5.67706E+01, 2.3250E+01, 2.6885E+03, &
      2.2583362E+00, 5.68971E+01, 2.3105E+01, 2.6586E+03, &
      2.2696279E+00, 5.70207E+01, 2.2961E+01, 2.6289E+03, &
      2.2809760E+00, 5.71417E+01, 2.2818E+01, 2.5997E+03, &
      2.2923809E+00, 5.72600E+01, 2.2676E+01, 2.5707E+03, &
      2.3038428E+00, 5.73759E+01, 2.2535E+01, 2.5421E+03, &
      2.3153620E+00, 5.74896E+01, 2.2395E+01, 2.5138E+03, &
      2.3269388E+00, 5.76011E+01, 2.2255E+01, 2.4859E+03, &
      2.3385735E+00, 5.77105E+01, 2.2117E+01, 2.4582E+03, &
      2.3502664E+00, 5.78181E+01, 2.1980E+01, 2.4309E+03, &
      2.3620177E+00, 5.79238E+01, 2.1843E+01, 2.4039E+03, &
      2.3738278E+00, 5.80279E+01, 2.1707E+01, 2.3771E+03, &
      2.3856970E+00, 5.81305E+01, 2.1573E+01, 2.3507E+03, &
      2.3976254E+00, 5.82317E+01, 2.1438E+01, 2.3245E+03, &
      2.4096136E+00, 5.83307E+01, 2.1301E+01, 2.2983E+03, &
      2.4216616E+00, 5.84276E+01, 2.1166E+01, 2.2724E+03, &
      2.4337699E+00, 5.85224E+01, 2.1031E+01, 2.2468E+03, &
      2.4459388E+00, 5.86153E+01, 2.0897E+01, 2.2215E+03, &
      2.4581685E+00, 5.87064E+01, 2.0764E+01, 2.1965E+03, &
      2.4704593E+00, 5.87958E+01, 2.0632E+01, 2.1717E+03, &
      2.4828116E+00, 5.88835E+01, 2.0500E+01, 2.1472E+03, &
      2.4952257E+00, 5.89696E+01, 2.0370E+01, 2.1230E+03, &
      2.5077018E+00, 5.90542E+01, 2.0240E+01, 2.0991E+03, &
      2.5202403E+00, 5.91374E+01, 2.0111E+01, 2.0754E+03, &
      2.5328415E+00, 5.92191E+01, 1.9983E+01, 2.0520E+03, &
      2.5455057E+00, 5.92994E+01, 1.9855E+01, 2.0289E+03, &
      2.5582333E+00, 5.93784E+01, 1.9728E+01, 2.0060E+03, &
      2.5710244E+00, 5.94562E+01, 1.9602E+01, 1.9833E+03, &
      2.5838796E+00, 5.95328E+01, 1.9477E+01, 1.9610E+03, &
      2.5967990E+00, 5.98280E+01, 1.9351E+01, 1.9386E+03, &
      2.6097829E+00, 5.99021E+01, 1.9223E+01, 1.9163E+03, &
      2.6228319E+00, 5.99746E+01, 1.9096E+01, 1.8943E+03, &
      2.6359460E+00, 6.00456E+01, 1.8970E+01, 1.8725E+03, &
      2.6491257E+00, 6.01154E+01, 1.8845E+01, 1.8510E+03, &
      2.6623714E+00, 6.01838E+01, 1.8721E+01, 1.8298E+03, &
      2.6756832E+00, 6.03943E+01, 1.8594E+01, 1.8085E+03, &
      2.6890617E+00, 6.04596E+01, 1.8469E+01, 1.7874E+03, &
      2.7025070E+00, 6.05233E+01, 1.8344E+01, 1.7665E+03, &
      2.7160195E+00, 6.05855E+01, 1.8220E+01, 1.7460E+03, &
      2.7295996E+00, 6.06463E+01, 1.8096E+01, 1.7256E+03, &
      2.7432476E+00, 6.07057E+01, 1.7974E+01, 1.7055E+03, &
      2.7569638E+00, 6.07638E+01, 1.7853E+01, 1.6856E+03, &
      2.7707486E+00, 6.08207E+01, 1.7732E+01, 1.6660E+03, &
      2.7846024E+00, 6.08763E+01, 1.7612E+01, 1.6466E+03, &
      2.7985254E+00, 6.09309E+01, 1.7493E+01, 1.6275E+03, &
      2.8125180E+00, 6.09843E+01, 1.7376E+01, 1.6086E+03, &
      2.8265806E+00, 6.10368E+01, 1.7259E+01, 1.5899E+03, &
      2.8407135E+00, 6.10882E+01, 1.7142E+01, 1.5714E+03, &
      2.8549171E+00, 6.11387E+01, 1.7027E+01, 1.5532E+03, &
      2.8691917E+00, 6.11884E+01, 1.6913E+01, 1.5351E+03, &
      2.8835376E+00, 6.12372E+01, 1.6799E+01, 1.5174E+03, &
      2.8979553E+00, 6.12854E+01, 1.6686E+01, 1.4997E+03, &
      2.9124451E+00, 6.13328E+01, 1.6572E+01, 1.4821E+03, &
      2.9270073E+00, 6.13795E+01, 1.6458E+01, 1.4647E+03, &
      2.9416424E+00, 6.14255E+01, 1.6345E+01, 1.4475E+03, &
      2.9563506E+00, 6.14711E+01, 1.6234E+01, 1.4306E+03, &
      2.9711323E+00, 6.15162E+01, 1.6123E+01, 1.4138E+03, &
      2.9859880E+00, 6.15613E+01, 1.6013E+01, 1.3973E+03, &
      3.0009179E+00, 6.16075E+01, 1.5903E+01, 1.3808E+03, &
      3.0159225E+00, 6.16638E+01, 1.5778E+01, 1.3633E+03, &
      3.0310021E+00, 6.17137E+01, 1.5654E+01, 1.3459E+03, &
      3.0461571E+00, 6.17593E+01, 1.5531E+01, 1.3288E+03, &
      3.0613879E+00, 6.18015E+01, 1.5410E+01, 1.3120E+03, &
      3.0766949E+00, 6.18410E+01, 1.5289E+01, 1.2953E+03, &
      3.0920783E+00, 6.18782E+01, 1.5170E+01, 1.2789E+03, &
      3.1075387E+00, 6.19134E+01, 1.5051E+01, 1.2627E+03, &
      3.1230764E+00, 6.19468E+01, 1.4934E+01, 1.2467E+03, &
      3.1386918E+00, 6.19786E+01, 1.4818E+01, 1.2310E+03, &
      3.1543853E+00, 6.20089E+01, 1.4703E+01, 1.2154E+03, &
      3.1701572E+00, 6.20378E+01, 1.4589E+01, 1.2001E+03, &
      3.1860080E+00, 6.20654E+01, 1.4476E+01, 1.1850E+03, &
      3.2019380E+00, 6.20918E+01, 1.4364E+01, 1.1701E+03, &
      3.2179477E+00, 6.21171E+01, 1.4253E+01, 1.1553E+03, &
      3.2340374E+00, 6.21413E+01, 1.4143E+01, 1.1408E+03, &
      3.2502076E+00, 6.21646E+01, 1.4035E+01, 1.1265E+03, &
      3.2664587E+00, 6.21868E+01, 1.3927E+01, 1.1124E+03, &
      3.2827910E+00, 6.22082E+01, 1.3820E+01, 1.0984E+03, &
      3.2992049E+00, 6.22287E+01, 1.3714E+01, 1.0847E+03, &
      3.3157009E+00, 6.22484E+01, 1.3609E+01, 1.0711E+03, &
      3.3322794E+00, 6.22674E+01, 1.3505E+01, 1.0577E+03, &
      3.3489408E+00, 6.22856E+01, 1.3402E+01, 1.0445E+03, &
      3.3656856E+00, 6.24074E+01, 1.3296E+01, 1.0312E+03, &
      3.3825140E+00, 6.24245E+01, 1.3191E+01, 1.0181E+03, &
      3.3994265E+00, 6.24405E+01, 1.3087E+01, 1.0051E+03, &
      3.4164237E+00, 6.24556E+01, 1.2984E+01, 9.9233E+02, &
      3.4335058E+00, 6.24697E+01, 1.2882E+01, 9.7972E+02, &
      3.4506733E+00, 6.24829E+01, 1.2781E+01, 9.6727E+02, &
      3.4679267E+00, 6.24953E+01, 1.2681E+01, 9.5500E+02, &
      3.4852663E+00, 6.25068E+01, 1.2581E+01, 9.4288E+02, &
      3.5026927E+00, 6.25176E+01, 1.2483E+01, 9.3094E+02, &
      3.5202061E+00, 6.25276E+01, 1.2385E+01, 9.1915E+02, &
      3.5378072E+00, 6.25369E+01, 1.2289E+01, 9.0752E+02, &
      3.5554962E+00, 6.25456E+01, 1.2193E+01, 8.9605E+02, &
      3.5732737E+00, 6.25535E+01, 1.2098E+01, 8.8473E+02, &
      3.5911400E+00, 6.25608E+01, 1.2004E+01, 8.7357E+02, &
      3.6090957E+00, 6.25675E+01, 1.1911E+01, 8.6255E+02, &
      3.6271412E+00, 6.25737E+01, 1.1818E+01, 8.5169E+02, &
      3.6452769E+00, 6.25792E+01, 1.1727E+01, 8.4097E+02, &
      3.6635033E+00, 6.25843E+01, 1.1636E+01, 8.3039E+02, &
      3.6818208E+00, 6.25888E+01, 1.1546E+01, 8.1995E+02, &
      3.7002299E+00, 6.26415E+01, 1.1455E+01, 8.0952E+02, &
      3.7187311E+00, 6.26452E+01, 1.1364E+01, 7.9921E+02, &
      3.7373247E+00, 6.26482E+01, 1.1274E+01, 7.8904E+02, &
      3.7560114E+00, 6.26506E+01, 1.1186E+01, 7.7900E+02, &
      3.7747914E+00, 6.26523E+01, 1.1097E+01, 7.6910E+02, &
      3.7936654E+00, 6.26534E+01, 1.1010E+01, 7.5933E+02, &
      3.8126337E+00, 6.26538E+01, 1.0924E+01, 7.4970E+02, &
      3.8316969E+00, 6.26537E+01, 1.0838E+01, 7.4019E+02, &
      3.8508554E+00, 6.26531E+01, 1.0753E+01, 7.3081E+02, &
      3.8701096E+00, 6.26519E+01, 1.0669E+01, 7.2156E+02, &
      3.8894602E+00, 6.26502E+01, 1.0585E+01, 7.1243E+02, &
      3.9089075E+00, 6.26480E+01, 1.0502E+01, 7.0342E+02, &
      3.9284520E+00, 6.26452E+01, 1.0420E+01, 6.9453E+02, &
      3.9480943E+00, 6.26421E+01, 1.0339E+01, 6.8576E+02, &
      3.9678347E+00, 6.26384E+01, 1.0258E+01, 6.7711E+02, &
      3.9876739E+00, 6.26343E+01, 1.0178E+01, 6.6858E+02, &
      4.055024E+00, 6.30127E+01, 9.9154E+00, 6.4075E+02, &
      4.334821E+00, 6.28643E+01, 8.9317E+00, 5.4085E+02, &
      4.633924E+00, 6.26369E+01, 8.0550E+00, 4.5716E+02, &
      4.953664E+00, 6.23597E+01, 7.2743E+00, 3.8702E+02, &
      5.295467E+00, 6.20314E+01, 6.5780E+00, 3.2814E+02, &
      5.660855E+00, 6.16424E+01, 5.9509E+00, 2.7841E+02, &
      6.051453E+00, 6.11523E+01, 5.3613E+00, 2.3532E+02, &
      6.469004E+00, 6.04954E+01, 4.8341E+00, 1.9913E+02, &
      6.915365E+00, 5.95372E+01, 4.3657E+00, 1.6882E+02, &
      7.392525E+00, 5.77675E+01, 3.9446E+00, 1.4324E+02, &
      7.634298E+00, 5.57189E+01, 3.7521E+00, 1.3220E+02, &
      7.751150E+00, 5.27186E+01, 3.6650E+00, 1.2731E+02, &
      7.782310E+00, 4.92132E+01, 3.6423E+00, 1.2605E+02, &
      7.829051E+00, 5.27159E+01, 1.0445E+01, 3.5032E+02, &
      7.902609E+00, 5.50591E+01, 1.0238E+01, 3.4028E+02, &
      7.945902E+00, 5.57528E+01, 1.0119E+01, 3.3455E+02, &
      8.408988E+00, 5.74449E+01, 9.0602E+00, 2.8355E+02, &
      8.447890E+00, 5.72929E+01, 8.9898E+00, 2.8009E+02, &
      8.537697E+00, 5.63280E+01, 8.8326E+00, 2.7238E+02, &
      8.572019E+00, 5.46603E+01, 8.7742E+00, 2.6952E+02, &
      8.623503E+00, 5.65372E+01, 1.2108E+01, 3.6804E+02, &
      8.752212E+00, 5.82818E+01, 1.1765E+01, 3.5249E+02, &
      8.864884E+00, 5.88592E+01, 1.1478E+01, 3.3963E+02, &
      9.000571E+00, 5.87877E+01, 1.1188E+01, 3.2617E+02, &
      9.030794E+00, 5.83174E+01, 1.1127E+01, 3.2332E+02, &
      9.036754E+00, 5.80648E+01, 1.1115E+01, 3.2275E+02, &
      9.091029E+00, 5.91577E+01, 1.2735E+01, 3.6697E+02, &
      9.226716E+00, 6.04204E+01, 1.2459E+01, 3.5386E+02, &
      9.653919E+00, 6.22520E+01, 1.1649E+01, 3.1648E+02, &
      1.032004E+01, 6.38114E+01, 1.0464E+01, 2.6633E+02, &
      1.103212E+01, 6.47480E+01, 9.3197E+00, 2.2228E+02, &
      1.179334E+01, 6.53232E+01, 8.3321E+00, 1.8624E+02, &
      1.260708E+01, 6.57140E+01, 7.4556E+00, 1.5621E+02, &
      1.347697E+01, 6.59725E+01, 6.6764E+00, 1.3114E+02, &
      1.440688E+01, 6.61399E+01, 5.9834E+00, 1.1021E+02, &
      1.540095E+01, 6.62445E+01, 5.3666E+00, 9.2703E+01, &
      1.646362E+01, 6.63671E+01, 4.8016E+00, 7.7811E+01, &
      1.759961E+01, 6.64171E+01, 4.2936E+00, 6.5291E+01, &
      1.881398E+01, 6.64299E+01, 3.8318E+00, 5.4698E+01, &
      2.011215E+01, 6.63840E+01, 3.4190E+00, 4.5828E+01, &
      2.149988E+01, 6.63205E+01, 3.0534E+00, 3.8442E+01, &
      2.298338E+01, 6.62462E+01, 2.7293E+00, 3.2285E+01, &
      2.456923E+01, 6.61656E+01, 2.4418E+00, 2.7147E+01, &
      2.626450E+01, 6.60849E+01, 2.1865E+00, 2.2855E+01, &
      2.807676E+01, 6.60029E+01, 1.9442E+00, 1.9122E+01, &
      3.001405E+01, 6.59027E+01, 1.7255E+00, 1.5980E+01, &
      3.208502E+01, 6.57934E+01, 1.5327E+00, 1.3373E+01, &
      3.429889E+01, 6.56765E+01, 1.3618E+00, 1.1201E+01, &
      3.666551E+01, 6.55487E+01, 1.2085E+00, 9.3779E+00, &
      3.919543E+01, 6.54053E+01, 1.0720E+00, 7.8549E+00, &
      4.189992E+01, 6.52349E+01, 9.4701E-01, 6.5608E+00, &
      4.479101E+01, 6.50163E+01, 8.3701E-01, 5.4880E+00, &
      4.788159E+01, 6.46984E+01, 7.4000E-01, 4.5972E+00, &
      5.118542E+01, 6.40762E+01, 6.5474E-01, 3.8584E+00, &
      5.271273E+01, 6.33752E+01, 6.2051E-01, 3.5741E+00, &
      5.351956E+01, 6.22214E+01, 6.0356E-01, 3.4361E+00, &
      5.373471E+01, 6.08393E+01, 5.9917E-01, 3.4006E+00, &
      5.405744E+01, 6.22361E+01, 3.3135E+00, 1.6382E+01, &
      5.471721E+01, 6.33622E+01, 3.2518E+00, 1.5891E+01, &
      5.486427E+01, 6.34983E+01, 3.2383E+00, 1.5785E+01, &
      5.849270E+01, 6.48889E+01, 2.9264E+00, 1.3417E+01, &
      6.252870E+01, 6.54469E+01, 2.6252E+00, 1.1296E+01, &
      6.684318E+01, 6.57676E+01, 2.3528E+00, 9.5056E+00, &
      7.145536E+01, 6.59746E+01, 2.1076E+00, 7.9986E+00, &
      7.638578E+01, 6.61132E+01, 1.8865E+00, 6.7286E+00, &
      8.165640E+01, 6.62115E+01, 1.6863E+00, 5.6561E+00, &
      8.729069E+01, 6.62745E+01, 1.5027E+00, 4.7441E+00, &
      9.331374E+01, 6.63134E+01, 1.3393E+00, 3.9827E+00, &
      9.975239E+01, 6.63358E+01, 1.1938E+00, 3.3469E+00, &
      1.066353E+02, 6.63469E+01, 1.0643E+00, 2.8158E+00, &
      1.139931E+02, 6.63776E+01, 9.4540E-01, 2.3641E+00, &
      1.218587E+02, 6.63708E+01, 8.3942E-01, 1.9868E+00, &
      1.302669E+02, 6.63589E+01, 7.4538E-01, 1.6726E+00, &
      1.392553E+02, 6.63437E+01, 6.6194E-01, 1.4107E+00, &
      1.488640E+02, 6.63263E+01, 5.8790E-01, 1.1926E+00, &
      1.591356E+02, 6.63078E+01, 5.2218E-01, 1.0105E+00, &
      1.701159E+02, 6.62886E+01, 4.6385E-01, 8.5857E-01, &
      1.818539E+02, 6.62693E+01, 4.1207E-01, 7.3169E-01, &
      1.944018E+02, 6.62503E+01, 3.6610E-01, 6.2565E-01, &
      2.078156E+02, 6.62317E+01, 3.2528E-01, 5.3693E-01, &
      2.221548E+02, 6.62136E+01, 2.8903E-01, 4.6263E-01, &
      2.374835E+02, 6.61961E+01, 2.5720E-01, 4.0073E-01, &
      2.538699E+02, 6.61800E+01, 2.2971E-01, 3.4952E-01, &
      2.713869E+02, 6.61651E+01, 2.0517E-01, 3.0625E-01, &
      2.901126E+02, 6.61513E+01, 1.8327E-01, 2.6963E-01, &
      3.101304E+02, 6.61386E+01, 1.6371E-01, 2.3858E-01, &
      3.315294E+02, 6.61268E+01, 1.4625E-01, 2.1218E-01, &
      3.544049E+02, 6.61159E+01, 1.3066E-01, 1.8968E-01, &
      3.788588E+02, 6.61058E+01, 1.1674E-01, 1.7044E-01, &
      4.050001E+02, 6.60966E+01, 1.0431E-01, 1.5395E-01, &
      4.329451E+02, 6.60880E+01, 9.3206E-02, 1.3975E-01]
   case(67)
      frel = -6.7140E-01
      allocate(nist_tab(3572))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 9.73216E+00, 2.9493E-01, 7.0392E+03, &
      1.142761E-02, 9.93946E+00, 3.2621E-01, 7.2831E+03, &
      1.221612E-02, 9.65648E+00, 3.6196E-01, 7.5596E+03, &
      1.305903E-02, 9.30247E+00, 4.0282E-01, 7.8701E+03, &
      1.396010E-02, 8.84880E+00, 4.4955E-01, 8.2162E+03, &
      1.492335E-02, 8.24754E+00, 5.0299E-01, 8.5995E+03, &
      1.595306E-02, 7.41052E+00, 5.6408E-01, 9.0215E+03, &
      1.705382E-02, 6.15120E+00, 6.3386E-01, 9.4831E+03, &
      1.823053E-02, 3.97033E+00, 7.1346E-01, 9.9851E+03, &
      1.948844E-02, -1.39811E+00, 8.0413E-01, 1.0528E+04, &
      1.989400E-02, -5.95584E+00, 8.3457E-01, 1.0703E+04, &
      2.019850E-02, -1.60324E+01, 8.5779E-01, 1.0835E+04, &
      2.027970E-02, -2.85314E+01, 8.6404E-01, 1.0871E+04, &
      2.040150E-02, -1.49118E+01, 2.5698E+01, 3.2137E+05, &
      2.070600E-02, -2.86167E+00, 2.4583E+01, 3.0291E+05, &
      2.083314E-02, -4.10622E-01, 2.4138E+01, 2.9561E+05, &
      2.227063E-02, 1.12389E+01, 1.9827E+01, 2.2714E+05, &
      2.380730E-02, 1.55912E+01, 1.6295E+01, 1.7464E+05, &
      2.545001E-02, 1.76656E+01, 1.3389E+01, 1.3423E+05, &
      2.720606E-02, 1.86010E+01, 1.1076E+01, 1.0387E+05, &
      2.908327E-02, 1.89102E+01, 9.2885E+00, 8.1485E+04, &
      3.109002E-02, 1.88728E+01, 7.9449E+00, 6.5200E+04, &
      3.323523E-02, 1.86521E+01, 6.9671E+00, 5.3485E+04, &
      3.552846E-02, 1.83458E+01, 6.2858E+00, 4.5140E+04, &
      3.797993E-02, 1.80125E+01, 5.8440E+00, 3.9258E+04, &
      4.060054E-02, 1.76872E+01, 5.5958E+00, 3.5165E+04, &
      4.340198E-02, 1.73911E+01, 5.5061E+00, 3.2368E+04, &
      4.639671E-02, 1.71367E+01, 5.5480E+00, 3.0509E+04, &
      4.959809E-02, 1.69294E+01, 5.7012E+00, 2.9328E+04, &
      5.017600E-02, 1.68969E+01, 5.7380E+00, 2.9177E+04, &
      5.094400E-02, 1.68496E+01, 5.7905E+00, 2.9000E+04, &
      5.114880E-02, 1.68262E+01, 5.8051E+00, 2.8957E+04, &
      5.145600E-02, 1.68282E+01, 5.8567E+00, 2.9040E+04, &
      5.222400E-02, 1.68126E+01, 5.9165E+00, 2.8905E+04, &
      5.302035E-02, 1.67927E+01, 5.9822E+00, 2.8787E+04, &
      5.667876E-02, 1.67390E+01, 6.3228E+00, 2.8462E+04, &
      6.058959E-02, 1.67654E+01, 6.7412E+00, 2.8387E+04, &
      6.477028E-02, 1.69116E+01, 7.2311E+00, 2.8485E+04, &
      6.923942E-02, 1.71692E+01, 7.7880E+00, 2.8698E+04, &
      7.401695E-02, 1.75279E+01, 8.0914E+00, 2.7892E+04, &
      7.912411E-02, 1.79004E+01, 8.3762E+00, 2.7010E+04, &
      8.458368E-02, 1.82919E+01, 8.6510E+00, 2.6095E+04, &
      9.041995E-02, 1.87022E+01, 8.9095E+00, 2.5140E+04, &
      9.665893E-02, 1.91268E+01, 9.1457E+00, 2.4141E+04, &
      1.0000000E-01, 2.16923E+01, 9.2556E+00, 2.3615E+04, &
      1.0050000E-01, 2.17247E+01, 9.2711E+00, 2.3537E+04, &
      1.0100250E-01, 2.17570E+01, 9.2864E+00, 2.3459E+04, &
      1.0150751E-01, 2.17893E+01, 9.3015E+00, 2.3380E+04, &
      1.0201505E-01, 2.18216E+01, 9.3165E+00, 2.3301E+04, &
      1.0252513E-01, 2.18539E+01, 9.3313E+00, 2.3222E+04, &
      1.0303775E-01, 2.18862E+01, 9.3459E+00, 2.3143E+04, &
      1.0355294E-01, 2.19184E+01, 9.3603E+00, 2.3063E+04, &
      1.0407070E-01, 2.19506E+01, 9.3745E+00, 2.2983E+04, &
      1.0459106E-01, 2.19827E+01, 9.3886E+00, 2.2903E+04, &
      1.0511401E-01, 2.20148E+01, 9.4024E+00, 2.2823E+04, &
      1.0563958E-01, 2.20468E+01, 9.4161E+00, 2.2742E+04, &
      1.0616778E-01, 2.20787E+01, 9.4296E+00, 2.2662E+04, &
      1.0669862E-01, 2.21106E+01, 9.4429E+00, 2.2581E+04, &
      1.0723211E-01, 2.21423E+01, 9.4560E+00, 2.2500E+04, &
      1.0776827E-01, 2.21740E+01, 9.4689E+00, 2.2418E+04, &
      1.0830712E-01, 2.22056E+01, 9.4817E+00, 2.2337E+04, &
      1.0884865E-01, 2.22370E+01, 9.4942E+00, 2.2255E+04, &
      1.0939289E-01, 2.22684E+01, 9.5065E+00, 2.2173E+04, &
      1.0993986E-01, 2.22995E+01, 9.5186E+00, 2.2091E+04, &
      1.1048956E-01, 2.23306E+01, 9.5306E+00, 2.2008E+04, &
      1.1104201E-01, 2.23615E+01, 9.5423E+00, 2.1926E+04, &
      1.1159722E-01, 2.23922E+01, 9.5538E+00, 2.1843E+04, &
      1.1215520E-01, 2.24228E+01, 9.5651E+00, 2.1760E+04, &
      1.1271598E-01, 2.24531E+01, 9.5762E+00, 2.1677E+04, &
      1.1327956E-01, 2.24833E+01, 9.5871E+00, 2.1594E+04, &
      1.1384596E-01, 2.25132E+01, 9.5978E+00, 2.1510E+04, &
      1.1441519E-01, 2.25430E+01, 9.6082E+00, 2.1427E+04, &
      1.1498726E-01, 2.25724E+01, 9.6185E+00, 2.1343E+04, &
      1.1556220E-01, 2.26017E+01, 9.6286E+00, 2.1259E+04, &
      1.1614001E-01, 2.26306E+01, 9.6384E+00, 2.1175E+04, &
      1.1672071E-01, 2.26592E+01, 9.6480E+00, 2.1090E+04, &
      1.1730431E-01, 2.26876E+01, 9.6574E+00, 2.1006E+04, &
      1.1789083E-01, 2.27156E+01, 9.6666E+00, 2.0921E+04, &
      1.1848029E-01, 2.27433E+01, 9.6755E+00, 2.0836E+04, &
      1.1907269E-01, 2.27706E+01, 9.6843E+00, 2.0751E+04, &
      1.1966805E-01, 2.27975E+01, 9.6928E+00, 2.0666E+04, &
      1.2026639E-01, 2.28240E+01, 9.7011E+00, 2.0581E+04, &
      1.2086772E-01, 2.28501E+01, 9.7092E+00, 2.0496E+04, &
      1.2147206E-01, 2.28758E+01, 9.7170E+00, 2.0410E+04, &
      1.2207942E-01, 2.29009E+01, 9.7246E+00, 2.0325E+04, &
      1.2268982E-01, 2.29256E+01, 9.7320E+00, 2.0239E+04, &
      1.2330327E-01, 2.29497E+01, 9.7392E+00, 2.0153E+04, &
      1.2391979E-01, 2.29733E+01, 9.7462E+00, 2.0067E+04, &
      1.2453939E-01, 2.29962E+01, 9.7529E+00, 1.9981E+04, &
      1.2516208E-01, 2.30186E+01, 9.7594E+00, 1.9895E+04, &
      1.2578789E-01, 2.30402E+01, 9.7656E+00, 1.9809E+04, &
      1.2641683E-01, 2.30612E+01, 9.7717E+00, 1.9722E+04, &
      1.2704892E-01, 2.30814E+01, 9.7774E+00, 1.9636E+04, &
      1.2768416E-01, 2.31008E+01, 9.7830E+00, 1.9549E+04, &
      1.2832258E-01, 2.31194E+01, 9.7883E+00, 1.9463E+04, &
      1.2896419E-01, 2.31371E+01, 9.7934E+00, 1.9376E+04, &
      1.2960902E-01, 2.31538E+01, 9.7983E+00, 1.9289E+04, &
      1.3025706E-01, 2.31696E+01, 9.8029E+00, 1.9202E+04, &
      1.3090835E-01, 2.31843E+01, 9.8073E+00, 1.9115E+04, &
      1.3156289E-01, 2.31978E+01, 9.8115E+00, 1.9028E+04, &
      1.3222070E-01, 2.32102E+01, 9.8154E+00, 1.8941E+04, &
      1.3288181E-01, 2.32213E+01, 9.8191E+00, 1.8854E+04, &
      1.3354621E-01, 2.32309E+01, 9.8226E+00, 1.8767E+04, &
      1.3421395E-01, 2.32392E+01, 9.8258E+00, 1.8680E+04, &
      1.3488502E-01, 2.32458E+01, 9.8287E+00, 1.8592E+04, &
      1.3555944E-01, 2.32508E+01, 9.8315E+00, 1.8505E+04, &
      1.3623724E-01, 2.32539E+01, 9.8340E+00, 1.8418E+04, &
      1.3691842E-01, 2.32552E+01, 9.8363E+00, 1.8330E+04, &
      1.3760302E-01, 2.32543E+01, 9.8383E+00, 1.8243E+04, &
      1.3829103E-01, 2.32512E+01, 9.8401E+00, 1.8155E+04, &
      1.3898249E-01, 2.32456E+01, 9.8416E+00, 1.8068E+04, &
      1.3967740E-01, 2.32373E+01, 9.8429E+00, 1.7980E+04, &
      1.4037579E-01, 2.32262E+01, 9.8440E+00, 1.7893E+04, &
      1.4107766E-01, 2.32120E+01, 9.8448E+00, 1.7805E+04, &
      1.4178305E-01, 2.31943E+01, 9.8454E+00, 1.7718E+04, &
      1.4249197E-01, 2.31728E+01, 9.8457E+00, 1.7630E+04, &
      1.4320443E-01, 2.31472E+01, 9.8459E+00, 1.7543E+04, &
      1.4392045E-01, 2.31170E+01, 9.8457E+00, 1.7455E+04, &
      1.4464005E-01, 2.30817E+01, 9.8454E+00, 1.7368E+04, &
      1.4536325E-01, 2.30408E+01, 9.8447E+00, 1.7280E+04, &
      1.4609007E-01, 2.29936E+01, 9.8439E+00, 1.7193E+04, &
      1.4682052E-01, 2.29393E+01, 9.8428E+00, 1.7106E+04, &
      1.4755462E-01, 2.28769E+01, 9.8415E+00, 1.7018E+04, &
      1.4829239E-01, 2.28054E+01, 9.8399E+00, 1.6931E+04, &
      1.4903386E-01, 2.27234E+01, 9.8381E+00, 1.6844E+04, &
      1.4977903E-01, 2.26293E+01, 9.8361E+00, 1.6756E+04, &
      1.5052792E-01, 2.25211E+01, 9.8338E+00, 1.6669E+04, &
      1.5128056E-01, 2.23962E+01, 9.8313E+00, 1.6582E+04, &
      1.5203696E-01, 2.22514E+01, 9.8285E+00, 1.6495E+04, &
      1.5279715E-01, 2.20827E+01, 9.8255E+00, 1.6408E+04, &
      1.5356113E-01, 2.18844E+01, 9.8223E+00, 1.6321E+04, &
      1.5432894E-01, 2.16505E+01, 9.8188E+00, 1.6234E+04, &
      1.5510058E-01, 2.13677E+01, 9.8151E+00, 1.6147E+04, &
      1.5587609E-01, 2.10222E+01, 9.8111E+00, 1.6060E+04, &
      1.5665547E-01, 2.05904E+01, 9.8069E+00, 1.5973E+04, &
      1.5743875E-01, 2.00337E+01, 9.8025E+00, 1.5887E+04, &
      1.5822594E-01, 1.92825E+01, 9.7979E+00, 1.5800E+04, &
      1.5901707E-01, 1.81924E+01, 9.7930E+00, 1.5714E+04, &
      1.5981215E-01, 1.63791E+01, 9.7878E+00, 1.5628E+04, &
      1.6061121E-01, 1.19740E+01, 9.7825E+00, 1.5541E+04, &
      1.6093012E-01, 4.52486E+00, 9.7803E+00, 1.5507E+04, &
      1.6106987E-01, 4.76173E+00, 2.4061E+01, 3.8114E+04, &
      1.6141427E-01, 1.32915E+01, 2.3717E+01, 3.7490E+04, &
      1.6222134E-01, 1.88069E+01, 2.2947E+01, 3.6092E+04, &
      1.6303245E-01, 2.14996E+01, 2.2218E+01, 3.4772E+04, &
      1.6384761E-01, 2.32847E+01, 2.1529E+01, 3.3525E+04, &
      1.6466685E-01, 2.45995E+01, 2.0877E+01, 3.2348E+04, &
      1.6549018E-01, 2.56206E+01, 2.0260E+01, 3.1236E+04, &
      1.6631763E-01, 2.64387E+01, 1.9676E+01, 3.0185E+04, &
      1.6714922E-01, 2.71074E+01, 1.9124E+01, 2.9192E+04, &
      1.6798497E-01, 2.76612E+01, 1.8601E+01, 2.8253E+04, &
      1.6882489E-01, 2.81241E+01, 1.8107E+01, 2.7365E+04, &
      1.6966902E-01, 2.85131E+01, 1.7639E+01, 2.6525E+04, &
      1.7051736E-01, 2.88413E+01, 1.7195E+01, 2.5730E+04, &
      1.7136995E-01, 2.91186E+01, 1.6776E+01, 2.4978E+04, &
      1.7222680E-01, 2.93527E+01, 1.6379E+01, 2.4265E+04, &
      1.7308793E-01, 2.95498E+01, 1.6003E+01, 2.3590E+04, &
      1.7395337E-01, 2.97151E+01, 1.5647E+01, 2.2951E+04, &
      1.7482314E-01, 2.98524E+01, 1.5309E+01, 2.2344E+04, &
      1.7569726E-01, 2.99651E+01, 1.4990E+01, 2.1769E+04, &
      1.7657574E-01, 3.00557E+01, 1.4687E+01, 2.1223E+04, &
      1.7745862E-01, 3.01258E+01, 1.4400E+01, 2.0706E+04, &
      1.7834591E-01, 3.01778E+01, 1.4134E+01, 2.0221E+04, &
      1.7923764E-01, 3.02159E+01, 1.3887E+01, 1.9770E+04, &
      1.8013383E-01, 3.02432E+01, 1.3659E+01, 1.9348E+04, &
      1.8103450E-01, 3.02623E+01, 1.3447E+01, 1.8953E+04, &
      1.8193967E-01, 3.02749E+01, 1.3251E+01, 1.8583E+04, &
      1.8284937E-01, 3.02824E+01, 1.3068E+01, 1.8236E+04, &
      1.8376362E-01, 3.02858E+01, 1.2897E+01, 1.7908E+04, &
      1.8468244E-01, 3.02862E+01, 1.2738E+01, 1.7600E+04, &
      1.8560585E-01, 3.02842E+01, 1.2590E+01, 1.7308E+04, &
      1.8653388E-01, 3.02803E+01, 1.2451E+01, 1.7032E+04, &
      1.8746655E-01, 3.02751E+01, 1.2321E+01, 1.6770E+04, &
      1.8840388E-01, 3.02689E+01, 1.2199E+01, 1.6521E+04, &
      1.8934590E-01, 3.02620E+01, 1.2084E+01, 1.6284E+04, &
      1.9029263E-01, 3.02548E+01, 1.1976E+01, 1.6059E+04, &
      1.9124409E-01, 3.02473E+01, 1.1874E+01, 1.5843E+04, &
      1.9220031E-01, 3.02399E+01, 1.1778E+01, 1.5637E+04, &
      1.9316131E-01, 3.02325E+01, 1.1687E+01, 1.5439E+04, &
      1.9412712E-01, 3.02254E+01, 1.1601E+01, 1.5249E+04, &
      1.9509776E-01, 3.02185E+01, 1.1520E+01, 1.5067E+04, &
      1.9607325E-01, 3.02121E+01, 1.1443E+01, 1.4892E+04, &
      1.9705361E-01, 3.02060E+01, 1.1369E+01, 1.4723E+04, &
      1.9803888E-01, 3.02005E+01, 1.1300E+01, 1.4559E+04, &
      1.9902907E-01, 3.01954E+01, 1.1233E+01, 1.4402E+04, &
      2.0002422E-01, 3.01908E+01, 1.1170E+01, 1.4249E+04, &
      2.0102434E-01, 3.01868E+01, 1.1109E+01, 1.4102E+04, &
      2.0202946E-01, 3.01832E+01, 1.1052E+01, 1.3959E+04, &
      2.0303961E-01, 3.01802E+01, 1.0996E+01, 1.3820E+04, &
      2.0405481E-01, 3.01778E+01, 1.0943E+01, 1.3685E+04, &
      2.0507508E-01, 3.01758E+01, 1.0892E+01, 1.3553E+04, &
      2.0610046E-01, 3.01743E+01, 1.0844E+01, 1.3426E+04, &
      2.0713096E-01, 3.01734E+01, 1.0797E+01, 1.3301E+04, &
      2.0816661E-01, 3.01728E+01, 1.0752E+01, 1.3180E+04, &
      2.0920745E-01, 3.01728E+01, 1.0708E+01, 1.3061E+04, &
      2.1025348E-01, 3.01732E+01, 1.0666E+01, 1.2945E+04, &
      2.1130475E-01, 3.01740E+01, 1.0626E+01, 1.2832E+04, &
      2.1236128E-01, 3.01752E+01, 1.0587E+01, 1.2721E+04, &
      2.1342308E-01, 3.01767E+01, 1.0549E+01, 1.2613E+04, &
      2.1449020E-01, 3.01786E+01, 1.0512E+01, 1.2507E+04, &
      2.1556265E-01, 3.01809E+01, 1.0477E+01, 1.2402E+04, &
      2.1664046E-01, 3.01834E+01, 1.0443E+01, 1.2300E+04, &
      2.1772366E-01, 3.01863E+01, 1.0409E+01, 1.2200E+04, &
      2.1881228E-01, 3.01894E+01, 1.0377E+01, 1.2102E+04, &
      2.1990634E-01, 3.01928E+01, 1.0345E+01, 1.2005E+04, &
      2.2100588E-01, 3.01964E+01, 1.0315E+01, 1.1910E+04, &
      2.2211090E-01, 3.02002E+01, 1.0285E+01, 1.1817E+04, &
      2.2322146E-01, 3.02042E+01, 1.0256E+01, 1.1725E+04, &
      2.2433757E-01, 3.02084E+01, 1.0228E+01, 1.1635E+04, &
      2.2545925E-01, 3.02127E+01, 1.0201E+01, 1.1546E+04, &
      2.2658655E-01, 3.02171E+01, 1.0174E+01, 1.1458E+04, &
      2.2771948E-01, 3.02217E+01, 1.0148E+01, 1.1372E+04, &
      2.2885808E-01, 3.02263E+01, 1.0122E+01, 1.1287E+04, &
      2.3000237E-01, 3.02310E+01, 1.0097E+01, 1.1203E+04, &
      2.3115238E-01, 3.02358E+01, 1.0073E+01, 1.1120E+04, &
      2.3230814E-01, 3.02406E+01, 1.0049E+01, 1.1039E+04, &
      2.3346969E-01, 3.02454E+01, 1.0026E+01, 1.0958E+04, &
      2.3463703E-01, 3.02502E+01, 1.0003E+01, 1.0879E+04, &
      2.3581022E-01, 3.02550E+01, 9.9804E+00, 1.0801E+04, &
      2.3698927E-01, 3.02598E+01, 9.9586E+00, 1.0724E+04, &
      2.3817422E-01, 3.02645E+01, 9.9372E+00, 1.0647E+04, &
      2.3936509E-01, 3.02692E+01, 9.9163E+00, 1.0572E+04, &
      2.4056191E-01, 3.02737E+01, 9.8958E+00, 1.0498E+04, &
      2.4176472E-01, 3.02782E+01, 9.8757E+00, 1.0425E+04, &
      2.4297355E-01, 3.02825E+01, 9.8561E+00, 1.0352E+04, &
      2.4418841E-01, 3.02868E+01, 9.8368E+00, 1.0280E+04, &
      2.4540936E-01, 3.02908E+01, 9.8179E+00, 1.0210E+04, &
      2.4663640E-01, 3.02947E+01, 9.7994E+00, 1.0140E+04, &
      2.4786959E-01, 3.02984E+01, 9.7812E+00, 1.0071E+04, &
      2.4910893E-01, 3.03019E+01, 9.7634E+00, 1.0002E+04, &
      2.5035448E-01, 3.03052E+01, 9.7460E+00, 9.9349E+03, &
      2.5160625E-01, 3.03082E+01, 9.7289E+00, 9.8681E+03, &
      2.5286428E-01, 3.03110E+01, 9.7122E+00, 9.8022E+03, &
      2.5412860E-01, 3.03135E+01, 9.6958E+00, 9.7370E+03, &
      2.5539925E-01, 3.03157E+01, 9.6798E+00, 9.6725E+03, &
      2.5667624E-01, 3.03176E+01, 9.6640E+00, 9.6088E+03, &
      2.5795962E-01, 3.03191E+01, 9.6486E+00, 9.5458E+03, &
      2.5924942E-01, 3.03203E+01, 9.6335E+00, 9.4835E+03, &
      2.6054567E-01, 3.03210E+01, 9.6188E+00, 9.4219E+03, &
      2.6184840E-01, 3.03214E+01, 9.6043E+00, 9.3610E+03, &
      2.6315764E-01, 3.03213E+01, 9.5902E+00, 9.3007E+03, &
      2.6447343E-01, 3.03207E+01, 9.5764E+00, 9.2412E+03, &
      2.6579579E-01, 3.03196E+01, 9.5629E+00, 9.1822E+03, &
      2.6712477E-01, 3.03180E+01, 9.5496E+00, 9.1240E+03, &
      2.6846040E-01, 3.03158E+01, 9.5367E+00, 9.0664E+03, &
      2.6980270E-01, 3.03130E+01, 9.5242E+00, 9.0094E+03, &
      2.7115171E-01, 3.03095E+01, 9.5119E+00, 8.9530E+03, &
      2.7250747E-01, 3.03052E+01, 9.4999E+00, 8.8973E+03, &
      2.7387001E-01, 3.03002E+01, 9.4882E+00, 8.8422E+03, &
      2.7523936E-01, 3.02943E+01, 9.4768E+00, 8.7877E+03, &
      2.7661556E-01, 3.02875E+01, 9.4657E+00, 8.7338E+03, &
      2.7799863E-01, 3.02797E+01, 9.4550E+00, 8.6805E+03, &
      2.7938863E-01, 3.02708E+01, 9.4445E+00, 8.6277E+03, &
      2.8078557E-01, 3.02607E+01, 9.4343E+00, 8.5756E+03, &
      2.8218950E-01, 3.02492E+01, 9.4244E+00, 8.5240E+03, &
      2.8360044E-01, 3.02362E+01, 9.4148E+00, 8.4730E+03, &
      2.8501845E-01, 3.02215E+01, 9.4056E+00, 8.4226E+03, &
      2.8644354E-01, 3.02049E+01, 9.3966E+00, 8.3728E+03, &
      2.8787576E-01, 3.01861E+01, 9.3879E+00, 8.3234E+03, &
      2.8931514E-01, 3.01648E+01, 9.3795E+00, 8.2747E+03, &
      2.9076171E-01, 3.01405E+01, 9.3714E+00, 8.2265E+03, &
      2.9221552E-01, 3.01126E+01, 9.3637E+00, 8.1788E+03, &
      2.9367660E-01, 3.00806E+01, 9.3562E+00, 8.1316E+03, &
      2.9514498E-01, 3.00432E+01, 9.3490E+00, 8.0850E+03, &
      2.9662071E-01, 2.99991E+01, 9.3421E+00, 8.0389E+03, &
      2.9810381E-01, 2.99462E+01, 9.3356E+00, 7.9933E+03, &
      2.9959433E-01, 2.98810E+01, 9.3293E+00, 7.9482E+03, &
      3.0109230E-01, 2.97978E+01, 9.3233E+00, 7.9037E+03, &
      3.0259776E-01, 2.96848E+01, 9.3176E+00, 7.8596E+03, &
      3.0411075E-01, 2.95135E+01, 9.3123E+00, 7.8160E+03, &
      3.0563130E-01, 2.91672E+01, 9.3072E+00, 7.7730E+03, &
      3.0629770E-01, 2.87365E+01, 9.3051E+00, 7.7543E+03, &
      3.0690231E-01, 2.87368E+01, 1.0468E+01, 8.7058E+03, &
      3.0715946E-01, 2.89651E+01, 1.0468E+01, 8.6985E+03, &
      3.0869526E-01, 2.94571E+01, 1.0468E+01, 8.6550E+03, &
      3.1023873E-01, 2.96653E+01, 1.0468E+01, 8.6120E+03, &
      3.1178993E-01, 2.98008E+01, 1.0468E+01, 8.5695E+03, &
      3.1334888E-01, 2.99022E+01, 1.0469E+01, 8.5275E+03, &
      3.1491562E-01, 2.99837E+01, 1.0470E+01, 8.4859E+03, &
      3.1649020E-01, 3.00520E+01, 1.0471E+01, 8.4447E+03, &
      3.1807265E-01, 3.01109E+01, 1.0473E+01, 8.4041E+03, &
      3.1966301E-01, 3.01626E+01, 1.0475E+01, 8.3638E+03, &
      3.2126133E-01, 3.02087E+01, 1.0477E+01, 8.3240E+03, &
      3.2286764E-01, 3.02501E+01, 1.0479E+01, 8.2846E+03, &
      3.2448197E-01, 3.02875E+01, 1.0482E+01, 8.2457E+03, &
      3.2610438E-01, 3.03214E+01, 1.0485E+01, 8.2071E+03, &
      3.2773491E-01, 3.03520E+01, 1.0489E+01, 8.1690E+03, &
      3.2937358E-01, 3.03795E+01, 1.0492E+01, 8.1313E+03, &
      3.3102045E-01, 3.04038E+01, 1.0496E+01, 8.0940E+03, &
      3.3267555E-01, 3.04247E+01, 1.0501E+01, 8.0571E+03, &
      3.3433893E-01, 3.04417E+01, 1.0505E+01, 8.0206E+03, &
      3.3601062E-01, 3.04539E+01, 1.0510E+01, 7.9844E+03, &
      3.3769068E-01, 3.04596E+01, 1.0515E+01, 7.9487E+03, &
      3.3937913E-01, 3.04548E+01, 1.0521E+01, 7.9133E+03, &
      3.4107602E-01, 3.04294E+01, 1.0527E+01, 7.8782E+03, &
      3.4278140E-01, 3.03304E+01, 1.0533E+01, 7.8435E+03, &
      3.4311527E-01, 3.02700E+01, 1.0534E+01, 7.8367E+03, &
      3.4388471E-01, 3.02826E+01, 1.0869E+01, 8.0678E+03, &
      3.4449531E-01, 3.03931E+01, 1.0872E+01, 8.0556E+03, &
      3.4621779E-01, 3.05278E+01, 1.0880E+01, 8.0216E+03, &
      3.4794888E-01, 3.06084E+01, 1.0888E+01, 7.9878E+03, &
      3.4968862E-01, 3.06716E+01, 1.0897E+01, 7.9544E+03, &
      3.5143706E-01, 3.07261E+01, 1.0906E+01, 7.9213E+03, &
      3.5319425E-01, 3.07751E+01, 1.0915E+01, 7.8885E+03, &
      3.5496022E-01, 3.08205E+01, 1.0924E+01, 7.8559E+03, &
      3.5673502E-01, 3.08633E+01, 1.0933E+01, 7.8237E+03, &
      3.5851870E-01, 3.09041E+01, 1.0943E+01, 7.7917E+03, &
      3.6031129E-01, 3.09432E+01, 1.0953E+01, 7.7600E+03, &
      3.6211285E-01, 3.09811E+01, 1.0963E+01, 7.7286E+03, &
      3.6392341E-01, 3.10179E+01, 1.0974E+01, 7.6974E+03, &
      3.6574303E-01, 3.10538E+01, 1.0984E+01, 7.6665E+03, &
      3.6757174E-01, 3.10889E+01, 1.0995E+01, 7.6359E+03, &
      3.6940960E-01, 3.11234E+01, 1.1006E+01, 7.6054E+03, &
      3.7125665E-01, 3.11573E+01, 1.1017E+01, 7.5752E+03, &
      3.7311293E-01, 3.11907E+01, 1.1028E+01, 7.5453E+03, &
      3.7497850E-01, 3.12236E+01, 1.1039E+01, 7.5155E+03, &
      3.7685339E-01, 3.12561E+01, 1.1051E+01, 7.4860E+03, &
      3.7873766E-01, 3.12882E+01, 1.1063E+01, 7.4567E+03, &
      3.8063135E-01, 3.13199E+01, 1.1074E+01, 7.4275E+03, &
      3.8253450E-01, 3.13513E+01, 1.1086E+01, 7.3986E+03, &
      3.8444718E-01, 3.13824E+01, 1.1098E+01, 7.3699E+03, &
      3.8636941E-01, 3.14132E+01, 1.1111E+01, 7.3413E+03, &
      3.8830126E-01, 3.14437E+01, 1.1123E+01, 7.3130E+03, &
      3.9024276E-01, 3.14738E+01, 1.1135E+01, 7.2847E+03, &
      3.9219398E-01, 3.15037E+01, 1.1148E+01, 7.2567E+03, &
      3.9415495E-01, 3.15332E+01, 1.1161E+01, 7.2288E+03, &
      3.9612572E-01, 3.15623E+01, 1.1173E+01, 7.2010E+03, &
      3.9810635E-01, 3.15911E+01, 1.1186E+01, 7.1734E+03, &
      4.0009688E-01, 3.16194E+01, 1.1199E+01, 7.1460E+03, &
      4.0209737E-01, 3.16473E+01, 1.1212E+01, 7.1186E+03, &
      4.0410785E-01, 3.16747E+01, 1.1225E+01, 7.0914E+03, &
      4.0612839E-01, 3.17014E+01, 1.1237E+01, 7.0643E+03, &
      4.0815904E-01, 3.17274E+01, 1.1250E+01, 7.0373E+03, &
      4.1019983E-01, 3.17526E+01, 1.1263E+01, 7.0104E+03, &
      4.1225083E-01, 3.17768E+01, 1.1276E+01, 6.9836E+03, &
      4.1431208E-01, 3.17998E+01, 1.1289E+01, 6.9569E+03, &
      4.1638364E-01, 3.18212E+01, 1.1302E+01, 6.9303E+03, &
      4.1846556E-01, 3.18409E+01, 1.1315E+01, 6.9038E+03, &
      4.2055789E-01, 3.18581E+01, 1.1328E+01, 6.8773E+03, &
      4.2266068E-01, 3.18721E+01, 1.1341E+01, 6.8510E+03, &
      4.2477398E-01, 3.18818E+01, 1.1354E+01, 6.8247E+03, &
      4.2689785E-01, 3.18851E+01, 1.1367E+01, 6.7984E+03, &
      4.2903234E-01, 3.18783E+01, 1.1380E+01, 6.7722E+03, &
      4.3117750E-01, 3.18532E+01, 1.1392E+01, 6.7461E+03, &
      4.3333339E-01, 3.17852E+01, 1.1405E+01, 6.7200E+03, &
      4.3505516E-01, 3.16007E+01, 1.1415E+01, 6.6993E+03, &
      4.3550006E-01, 3.14146E+01, 1.1417E+01, 6.6939E+03, &
      4.3634484E-01, 3.16239E+01, 1.1943E+01, 6.9886E+03, &
      4.3767756E-01, 3.18338E+01, 1.1952E+01, 6.9721E+03, &
      4.3986595E-01, 3.19973E+01, 1.1965E+01, 6.9453E+03, &
      4.4206528E-01, 3.21080E+01, 1.1979E+01, 6.9185E+03, &
      4.4427560E-01, 3.21982E+01, 1.1992E+01, 6.8917E+03, &
      4.4649698E-01, 3.22775E+01, 1.2005E+01, 6.8650E+03, &
      4.4872947E-01, 3.23500E+01, 1.2018E+01, 6.8383E+03, &
      4.5097311E-01, 3.24180E+01, 1.2031E+01, 6.8115E+03, &
      4.5322798E-01, 3.24828E+01, 1.2043E+01, 6.7848E+03, &
      4.5549412E-01, 3.25452E+01, 1.2056E+01, 6.7581E+03, &
      4.5777159E-01, 3.26058E+01, 1.2068E+01, 6.7314E+03, &
      4.6006045E-01, 3.26649E+01, 1.2080E+01, 6.7047E+03, &
      4.6236075E-01, 3.27229E+01, 1.2092E+01, 6.6780E+03, &
      4.6467255E-01, 3.27799E+01, 1.2104E+01, 6.6513E+03, &
      4.6699592E-01, 3.28362E+01, 1.2116E+01, 6.6246E+03, &
      4.6933090E-01, 3.28919E+01, 1.2127E+01, 6.5978E+03, &
      4.7167755E-01, 3.29471E+01, 1.2138E+01, 6.5711E+03, &
      4.7403594E-01, 3.30018E+01, 1.2149E+01, 6.5443E+03, &
      4.7640612E-01, 3.30563E+01, 1.2160E+01, 6.5175E+03, &
      4.7878815E-01, 3.31104E+01, 1.2170E+01, 6.4907E+03, &
      4.8118209E-01, 3.31643E+01, 1.2180E+01, 6.4638E+03, &
      4.8358800E-01, 3.32323E+01, 1.2190E+01, 6.4369E+03, &
      4.8600594E-01, 3.32859E+01, 1.2200E+01, 6.4099E+03, &
      4.8843597E-01, 3.33393E+01, 1.2209E+01, 6.3830E+03, &
      4.9087815E-01, 3.33926E+01, 1.2218E+01, 6.3559E+03, &
      4.9333254E-01, 3.34459E+01, 1.2226E+01, 6.3288E+03, &
      4.9579920E-01, 3.34990E+01, 1.2235E+01, 6.3017E+03, &
      4.9827820E-01, 3.35520E+01, 1.2243E+01, 6.2744E+03, &
      5.0076959E-01, 3.36050E+01, 1.2250E+01, 6.2472E+03, &
      5.0327344E-01, 3.36579E+01, 1.2258E+01, 6.2199E+03, &
      5.0578980E-01, 3.37108E+01, 1.2265E+01, 6.1925E+03, &
      5.0831875E-01, 3.37636E+01, 1.2271E+01, 6.1651E+03, &
      5.1086035E-01, 3.38163E+01, 1.2278E+01, 6.1376E+03, &
      5.1341465E-01, 3.38690E+01, 1.2283E+01, 6.1100E+03, &
      5.1598172E-01, 3.39217E+01, 1.2289E+01, 6.0824E+03, &
      5.1856163E-01, 3.39743E+01, 1.2294E+01, 6.0548E+03, &
      5.2115444E-01, 3.40269E+01, 1.2299E+01, 6.0270E+03, &
      5.2376021E-01, 3.40794E+01, 1.2303E+01, 5.9992E+03, &
      5.2637901E-01, 3.41319E+01, 1.2307E+01, 5.9714E+03, &
      5.2901091E-01, 3.41843E+01, 1.2311E+01, 5.9435E+03, &
      5.3165596E-01, 3.42366E+01, 1.2314E+01, 5.9155E+03, &
      5.3431424E-01, 3.42889E+01, 1.2317E+01, 5.8875E+03, &
      5.3698581E-01, 3.43411E+01, 1.2320E+01, 5.8595E+03, &
      5.3967074E-01, 3.43933E+01, 1.2322E+01, 5.8313E+03, &
      5.4236910E-01, 3.44454E+01, 1.2323E+01, 5.8031E+03, &
      5.4508094E-01, 3.44974E+01, 1.2324E+01, 5.7749E+03, &
      5.4780635E-01, 3.45493E+01, 1.2325E+01, 5.7466E+03, &
      5.5054538E-01, 3.46011E+01, 1.2326E+01, 5.7182E+03, &
      5.5329810E-01, 3.46529E+01, 1.2326E+01, 5.6898E+03, &
      5.5606460E-01, 3.47045E+01, 1.2325E+01, 5.6614E+03, &
      5.5884492E-01, 3.47560E+01, 1.2324E+01, 5.6329E+03, &
      5.6163914E-01, 3.48074E+01, 1.2323E+01, 5.6043E+03, &
      5.6444734E-01, 3.48587E+01, 1.2321E+01, 5.5757E+03, &
      5.6726958E-01, 3.49098E+01, 1.2319E+01, 5.5471E+03, &
      5.7010592E-01, 3.49608E+01, 1.2317E+01, 5.5184E+03, &
      5.7295645E-01, 3.50116E+01, 1.2314E+01, 5.4897E+03, &
      5.7582123E-01, 3.50623E+01, 1.2310E+01, 5.4609E+03, &
      5.7870034E-01, 3.51129E+01, 1.2306E+01, 5.4321E+03, &
      5.8159384E-01, 3.51632E+01, 1.2302E+01, 5.4032E+03, &
      5.8450181E-01, 3.52134E+01, 1.2297E+01, 5.3743E+03, &
      5.8742432E-01, 3.52634E+01, 1.2292E+01, 5.3454E+03, &
      5.9036144E-01, 3.53132E+01, 1.2287E+01, 5.3165E+03, &
      5.9331325E-01, 3.53627E+01, 1.2281E+01, 5.2875E+03, &
      5.9627982E-01, 3.54121E+01, 1.2274E+01, 5.2585E+03, &
      5.9926122E-01, 3.54613E+01, 1.2267E+01, 5.2294E+03, &
      6.0225752E-01, 3.55102E+01, 1.2260E+01, 5.2004E+03, &
      6.0526881E-01, 3.55588E+01, 1.2252E+01, 5.1713E+03, &
      6.0829515E-01, 3.56073E+01, 1.2244E+01, 5.1422E+03, &
      6.1133663E-01, 3.56554E+01, 1.2235E+01, 5.1131E+03, &
      6.1439331E-01, 3.57034E+01, 1.2226E+01, 5.0839E+03, &
      6.1746528E-01, 3.57510E+01, 1.2217E+01, 5.0548E+03, &
      6.2055260E-01, 3.57983E+01, 1.2207E+01, 5.0256E+03, &
      6.2365537E-01, 3.58454E+01, 1.2197E+01, 4.9965E+03, &
      6.2677364E-01, 3.58922E+01, 1.2186E+01, 4.9673E+03, &
      6.2990751E-01, 3.59386E+01, 1.2175E+01, 4.9381E+03, &
      6.3305705E-01, 3.59848E+01, 1.2163E+01, 4.9089E+03, &
      6.3622234E-01, 3.60306E+01, 1.2151E+01, 4.8798E+03, &
      6.3940345E-01, 3.60761E+01, 1.2139E+01, 4.8506E+03, &
      6.4260046E-01, 3.61213E+01, 1.2126E+01, 4.8214E+03, &
      6.4581347E-01, 3.61661E+01, 1.2113E+01, 4.7923E+03, &
      6.4904253E-01, 3.62106E+01, 1.2099E+01, 4.7631E+03, &
      6.5228775E-01, 3.62546E+01, 1.2085E+01, 4.7339E+03, &
      6.5554919E-01, 3.62983E+01, 1.2070E+01, 4.7047E+03, &
      6.5882693E-01, 3.63416E+01, 1.2055E+01, 4.6754E+03, &
      6.6212107E-01, 3.63845E+01, 1.2039E+01, 4.6462E+03, &
      6.6543167E-01, 3.64269E+01, 1.2023E+01, 4.6169E+03, &
      6.6875883E-01, 3.64688E+01, 1.2006E+01, 4.5877E+03, &
      6.7210262E-01, 3.65103E+01, 1.1989E+01, 4.5584E+03, &
      6.7546314E-01, 3.65513E+01, 1.1972E+01, 4.5292E+03, &
      6.7884045E-01, 3.65917E+01, 1.1954E+01, 4.5000E+03, &
      6.8223466E-01, 3.66317E+01, 1.1935E+01, 4.4708E+03, &
      6.8564583E-01, 3.66711E+01, 1.1917E+01, 4.4416E+03, &
      6.8907406E-01, 3.67100E+01, 1.1897E+01, 4.4124E+03, &
      6.9251943E-01, 3.67484E+01, 1.1878E+01, 4.3833E+03, &
      6.9598202E-01, 3.67862E+01, 1.1858E+01, 4.3542E+03, &
      6.9946194E-01, 3.68234E+01, 1.1837E+01, 4.3251E+03, &
      7.0295924E-01, 3.68600E+01, 1.1817E+01, 4.2961E+03, &
      7.0647404E-01, 3.68960E+01, 1.1795E+01, 4.2672E+03, &
      7.1000641E-01, 3.69315E+01, 1.1774E+01, 4.2383E+03, &
      7.1355644E-01, 3.69663E+01, 1.1752E+01, 4.2094E+03, &
      7.1712423E-01, 3.70005E+01, 1.1730E+01, 4.1806E+03, &
      7.2070985E-01, 3.70340E+01, 1.1707E+01, 4.1518E+03, &
      7.2431340E-01, 3.70669E+01, 1.1684E+01, 4.1232E+03, &
      7.2793496E-01, 3.70992E+01, 1.1661E+01, 4.0946E+03, &
      7.3157464E-01, 3.71308E+01, 1.1637E+01, 4.0660E+03, &
      7.3523251E-01, 3.71617E+01, 1.1613E+01, 4.0376E+03, &
      7.3890867E-01, 3.71920E+01, 1.1589E+01, 4.0091E+03, &
      7.4260322E-01, 3.72215E+01, 1.1564E+01, 3.9806E+03, &
      7.4631623E-01, 3.72502E+01, 1.1539E+01, 3.9523E+03, &
      7.5004781E-01, 3.72782E+01, 1.1513E+01, 3.9240E+03, &
      7.5379805E-01, 3.73053E+01, 1.1488E+01, 3.8958E+03, &
      7.5756704E-01, 3.73318E+01, 1.1461E+01, 3.8677E+03, &
      7.6135488E-01, 3.73574E+01, 1.1435E+01, 3.8397E+03, &
      7.6516165E-01, 3.73822E+01, 1.1409E+01, 3.8118E+03, &
      7.6898746E-01, 3.74062E+01, 1.1382E+01, 3.7840E+03, &
      7.7283240E-01, 3.74294E+01, 1.1355E+01, 3.7563E+03, &
      7.7669656E-01, 3.74518E+01, 1.1327E+01, 3.7287E+03, &
      7.8058004E-01, 3.74733E+01, 1.1300E+01, 3.7012E+03, &
      7.8448294E-01, 3.74940E+01, 1.1272E+01, 3.6739E+03, &
      7.8840536E-01, 3.75138E+01, 1.1244E+01, 3.6466E+03, &
      7.9234738E-01, 3.75327E+01, 1.1216E+01, 3.6194E+03, &
      7.9630912E-01, 3.75508E+01, 1.1188E+01, 3.5924E+03, &
      8.0029067E-01, 3.75680E+01, 1.1159E+01, 3.5655E+03, &
      8.0429212E-01, 3.75843E+01, 1.1130E+01, 3.5387E+03, &
      8.0831358E-01, 3.75997E+01, 1.1101E+01, 3.5120E+03, &
      8.1235515E-01, 3.76141E+01, 1.1072E+01, 3.4855E+03, &
      8.1641693E-01, 3.76276E+01, 1.1043E+01, 3.4591E+03, &
      8.2049901E-01, 3.76402E+01, 1.1014E+01, 3.4328E+03, &
      8.2460150E-01, 3.76518E+01, 1.0984E+01, 3.4066E+03, &
      8.2872451E-01, 3.76625E+01, 1.0955E+01, 3.3806E+03, &
      8.3286813E-01, 3.76721E+01, 1.0925E+01, 3.3547E+03, &
      8.3703248E-01, 3.76808E+01, 1.0895E+01, 3.3289E+03, &
      8.4121764E-01, 3.76885E+01, 1.0865E+01, 3.3033E+03, &
      8.4542373E-01, 3.76951E+01, 1.0834E+01, 3.2778E+03, &
      8.4965084E-01, 3.77007E+01, 1.0804E+01, 3.2525E+03, &
      8.5389910E-01, 3.77053E+01, 1.0774E+01, 3.2273E+03, &
      8.5816859E-01, 3.77088E+01, 1.0743E+01, 3.2022E+03, &
      8.6245944E-01, 3.77112E+01, 1.0713E+01, 3.1773E+03, &
      8.6677173E-01, 3.77125E+01, 1.0682E+01, 3.1525E+03, &
      8.7110559E-01, 3.77127E+01, 1.0651E+01, 3.1278E+03, &
      8.7546112E-01, 3.77118E+01, 1.0620E+01, 3.1033E+03, &
      8.7983843E-01, 3.77098E+01, 1.0589E+01, 3.0790E+03, &
      8.8423762E-01, 3.77066E+01, 1.0558E+01, 3.0548E+03, &
      8.8865881E-01, 3.77022E+01, 1.0527E+01, 3.0307E+03, &
      8.9310210E-01, 3.76966E+01, 1.0496E+01, 3.0068E+03, &
      8.9756761E-01, 3.76898E+01, 1.0465E+01, 2.9831E+03, &
      9.0205545E-01, 3.76818E+01, 1.0434E+01, 2.9595E+03, &
      9.0656573E-01, 3.76725E+01, 1.0403E+01, 2.9360E+03, &
      9.1109856E-01, 3.76620E+01, 1.0371E+01, 2.9127E+03, &
      9.1565405E-01, 3.76501E+01, 1.0340E+01, 2.8895E+03, &
      9.2023232E-01, 3.76373E+01, 1.0309E+01, 2.8665E+03, &
      9.2483348E-01, 3.76228E+01, 1.0277E+01, 2.8437E+03, &
      9.2945765E-01, 3.76070E+01, 1.0246E+01, 2.8209E+03, &
      9.3410494E-01, 3.75897E+01, 1.0214E+01, 2.7984E+03, &
      9.3877546E-01, 3.75711E+01, 1.0183E+01, 2.7760E+03, &
      9.4346934E-01, 3.75510E+01, 1.0152E+01, 2.7537E+03, &
      9.4818668E-01, 3.75295E+01, 1.0120E+01, 2.7316E+03, &
      9.5292762E-01, 3.75065E+01, 1.0089E+01, 2.7097E+03, &
      9.5769226E-01, 3.74819E+01, 1.0057E+01, 2.6879E+03, &
      9.6248072E-01, 3.74559E+01, 1.0026E+01, 2.6663E+03, &
      9.6729312E-01, 3.74282E+01, 9.9944E+00, 2.6448E+03, &
      9.7212959E-01, 3.73989E+01, 9.9630E+00, 2.6234E+03, &
      9.7699023E-01, 3.73680E+01, 9.9316E+00, 2.6022E+03, &
      9.8187519E-01, 3.73354E+01, 9.9002E+00, 2.5812E+03, &
      9.8678456E-01, 3.73010E+01, 9.8689E+00, 2.5603E+03, &
      9.9171848E-01, 3.72650E+01, 9.8375E+00, 2.5396E+03, &
      9.9667708E-01, 3.72271E+01, 9.8063E+00, 2.5190E+03, &
      1.0016605E+00, 3.71944E+01, 9.7664E+00, 2.4963E+03, &
      1.0066688E+00, 3.71717E+01, 9.7094E+00, 2.4695E+03, &
      1.0117021E+00, 3.71444E+01, 9.6527E+00, 2.4430E+03, &
      1.0167606E+00, 3.71126E+01, 9.5965E+00, 2.4168E+03, &
      1.0218444E+00, 3.70764E+01, 9.5407E+00, 2.3909E+03, &
      1.0269536E+00, 3.70358E+01, 9.4853E+00, 2.3653E+03, &
      1.0320884E+00, 3.69915E+01, 9.4303E+00, 2.3400E+03, &
      1.0372489E+00, 3.69422E+01, 9.3756E+00, 2.3150E+03, &
      1.0424351E+00, 3.68886E+01, 9.3213E+00, 2.2903E+03, &
      1.0476473E+00, 3.68306E+01, 9.2674E+00, 2.2658E+03, &
      1.0528855E+00, 3.67682E+01, 9.2139E+00, 2.2416E+03, &
      1.0581499E+00, 3.67014E+01, 9.1608E+00, 2.2177E+03, &
      1.0634407E+00, 3.66301E+01, 9.1080E+00, 2.1941E+03, &
      1.0687579E+00, 3.65542E+01, 9.0556E+00, 2.1707E+03, &
      1.0741017E+00, 3.64737E+01, 9.0036E+00, 2.1476E+03, &
      1.0794722E+00, 3.63884E+01, 8.9519E+00, 2.1247E+03, &
      1.0848695E+00, 3.62982E+01, 8.9005E+00, 2.1022E+03, &
      1.0902939E+00, 3.62031E+01, 8.8496E+00, 2.0798E+03, &
      1.0957454E+00, 3.61028E+01, 8.7990E+00, 2.0578E+03, &
      1.1012241E+00, 3.59972E+01, 8.7487E+00, 2.0359E+03, &
      1.1067302E+00, 3.58862E+01, 8.6988E+00, 2.0144E+03, &
      1.1122639E+00, 3.57695E+01, 8.6492E+00, 1.9930E+03, &
      1.1178252E+00, 3.56469E+01, 8.6000E+00, 1.9719E+03, &
      1.1234143E+00, 3.55182E+01, 8.5511E+00, 1.9511E+03, &
      1.1290314E+00, 3.53831E+01, 8.5026E+00, 1.9305E+03, &
      1.1346765E+00, 3.52413E+01, 8.4544E+00, 1.9101E+03, &
      1.1403499E+00, 3.50925E+01, 8.4065E+00, 1.8899E+03, &
      1.1460517E+00, 3.49364E+01, 8.3590E+00, 1.8700E+03, &
      1.1517819E+00, 3.47725E+01, 8.3118E+00, 1.8503E+03, &
      1.1575408E+00, 3.46005E+01, 8.2649E+00, 1.8308E+03, &
      1.1633285E+00, 3.44198E+01, 8.2183E+00, 1.8116E+03, &
      1.1691452E+00, 3.42300E+01, 8.1721E+00, 1.7925E+03, &
      1.1749909E+00, 3.40305E+01, 8.1262E+00, 1.7737E+03, &
      1.1808659E+00, 3.38205E+01, 8.0806E+00, 1.7551E+03, &
      1.1867702E+00, 3.35995E+01, 8.0353E+00, 1.7367E+03, &
      1.1927040E+00, 3.33666E+01, 7.9904E+00, 1.7185E+03, &
      1.1986676E+00, 3.31208E+01, 7.9457E+00, 1.7005E+03, &
      1.2046609E+00, 3.28613E+01, 7.9014E+00, 1.6827E+03, &
      1.2106842E+00, 3.25867E+01, 7.8574E+00, 1.6651E+03, &
      1.2167376E+00, 3.22959E+01, 7.8137E+00, 1.6477E+03, &
      1.2228213E+00, 3.19872E+01, 7.7702E+00, 1.6305E+03, &
      1.2289354E+00, 3.16591E+01, 7.7264E+00, 1.6133E+03, &
      1.2350801E+00, 3.13092E+01, 7.6817E+00, 1.5961E+03, &
      1.2412555E+00, 3.09354E+01, 7.6374E+00, 1.5791E+03, &
      1.2474618E+00, 3.05348E+01, 7.5934E+00, 1.5623E+03, &
      1.2536991E+00, 3.01041E+01, 7.5497E+00, 1.5457E+03, &
      1.2599676E+00, 2.96396E+01, 7.5063E+00, 1.5293E+03, &
      1.2662674E+00, 2.91364E+01, 7.4632E+00, 1.5131E+03, &
      1.2725988E+00, 2.85886E+01, 7.4204E+00, 1.4970E+03, &
      1.2789618E+00, 2.79891E+01, 7.3779E+00, 1.4811E+03, &
      1.2853566E+00, 2.73285E+01, 7.3357E+00, 1.4655E+03, &
      1.2917833E+00, 2.65947E+01, 7.2939E+00, 1.4499E+03, &
      1.2982423E+00, 2.57716E+01, 7.2523E+00, 1.4346E+03, &
      1.3047335E+00, 2.48368E+01, 7.2110E+00, 1.4195E+03, &
      1.3112571E+00, 2.37589E+01, 7.1700E+00, 1.4045E+03, &
      1.3178134E+00, 2.24875E+01, 7.1292E+00, 1.3897E+03, &
      1.3244025E+00, 2.09428E+01, 7.0888E+00, 1.3750E+03, &
      1.3310245E+00, 1.89771E+01, 7.0486E+00, 1.3605E+03, &
      1.3376796E+00, 1.62672E+01, 7.0088E+00, 1.3462E+03, &
      1.3443680E+00, 1.18184E+01, 6.9692E+00, 1.3320E+03, &
      1.3510899E+00, -7.94075E+00, 6.9299E+00, 1.3180E+03, &
      1.3512135E+00, -1.11161E+01, 6.9291E+00, 1.3178E+03, &
      1.3515865E+00, -1.14577E+01, 2.6506E+01, 5.0129E+03, &
      1.3578453E+00, 1.02390E+01, 2.6318E+01, 4.9545E+03, &
      1.3646345E+00, 1.40730E+01, 2.6116E+01, 4.8923E+03, &
      1.3714577E+00, 1.57145E+01, 2.5916E+01, 4.8308E+03, &
      1.3783150E+00, 1.60735E+01, 2.5718E+01, 4.7700E+03, &
      1.3852066E+00, 1.47112E+01, 2.5520E+01, 4.7100E+03, &
      1.3911646E+00, 3.96765E+00, 2.5352E+01, 4.6590E+03, &
      1.3918353E+00, 3.88801E+00, 3.8170E+01, 7.0065E+03, &
      1.3921326E+00, 6.54188E+00, 3.8158E+01, 7.0027E+03, &
      1.3990933E+00, 1.79314E+01, 3.7860E+01, 6.9136E+03, &
      1.4060887E+00, 2.16526E+01, 3.7564E+01, 6.8256E+03, &
      1.4131192E+00, 2.41870E+01, 3.7271E+01, 6.7388E+03, &
      1.4201848E+00, 2.61718E+01, 3.6980E+01, 6.6531E+03, &
      1.4272857E+00, 2.78253E+01, 3.6692E+01, 6.5685E+03, &
      1.4344221E+00, 2.92520E+01, 3.6406E+01, 6.4850E+03, &
      1.4415942E+00, 3.05108E+01, 3.6122E+01, 6.4025E+03, &
      1.4488022E+00, 3.16391E+01, 3.5840E+01, 6.3212E+03, &
      1.4560462E+00, 3.26620E+01, 3.5561E+01, 6.2408E+03, &
      1.4633265E+00, 3.35976E+01, 3.5284E+01, 6.1616E+03, &
      1.4706431E+00, 3.44594E+01, 3.5010E+01, 6.0833E+03, &
      1.4779963E+00, 3.52575E+01, 3.4737E+01, 6.0061E+03, &
      1.4853863E+00, 3.60001E+01, 3.4467E+01, 5.9298E+03, &
      1.4928132E+00, 3.66936E+01, 3.4199E+01, 5.8545E+03, &
      1.5002773E+00, 3.73433E+01, 3.3933E+01, 5.7802E+03, &
      1.5077787E+00, 3.79536E+01, 3.3669E+01, 5.7069E+03, &
      1.5153176E+00, 3.85279E+01, 3.3407E+01, 5.6345E+03, &
      1.5228942E+00, 3.90695E+01, 3.3148E+01, 5.5630E+03, &
      1.5305086E+00, 3.95808E+01, 3.2890E+01, 5.4925E+03, &
      1.5381612E+00, 4.00641E+01, 3.2635E+01, 5.4228E+03, &
      1.5458520E+00, 4.05212E+01, 3.2382E+01, 5.3541E+03, &
      1.5535812E+00, 4.09537E+01, 3.2130E+01, 5.2863E+03, &
      1.5613491E+00, 4.13631E+01, 3.1881E+01, 5.2193E+03, &
      1.5691559E+00, 4.17504E+01, 3.1634E+01, 5.1532E+03, &
      1.5770017E+00, 4.21167E+01, 3.1389E+01, 5.0879E+03, &
      1.5848867E+00, 4.24626E+01, 3.1145E+01, 5.0235E+03, &
      1.5928111E+00, 4.27889E+01, 3.0904E+01, 4.9599E+03, &
      1.6007752E+00, 4.30961E+01, 3.0665E+01, 4.8971E+03, &
      1.6087790E+00, 4.33842E+01, 3.0427E+01, 4.8351E+03, &
      1.6168229E+00, 4.36536E+01, 3.0192E+01, 4.7740E+03, &
      1.6249070E+00, 4.39041E+01, 2.9958E+01, 4.7136E+03, &
      1.6330316E+00, 4.41353E+01, 2.9726E+01, 4.6540E+03, &
      1.6411967E+00, 4.43466E+01, 2.9497E+01, 4.5951E+03, &
      1.6494027E+00, 4.45370E+01, 2.9268E+01, 4.5370E+03, &
      1.6576497E+00, 4.47050E+01, 2.9042E+01, 4.4797E+03, &
      1.6659380E+00, 4.48485E+01, 2.8818E+01, 4.4231E+03, &
      1.6742677E+00, 4.49643E+01, 2.8595E+01, 4.3672E+03, &
      1.6826390E+00, 4.50477E+01, 2.8374E+01, 4.3120E+03, &
      1.6910522E+00, 4.50918E+01, 2.8154E+01, 4.2574E+03, &
      1.6995075E+00, 4.50849E+01, 2.7936E+01, 4.2035E+03, &
      1.7080050E+00, 4.50075E+01, 2.7719E+01, 4.1503E+03, &
      1.7165450E+00, 4.48213E+01, 2.7505E+01, 4.0978E+03, &
      1.7251278E+00, 4.44371E+01, 2.7292E+01, 4.0459E+03, &
      1.7337534E+00, 4.35431E+01, 2.7080E+01, 3.9947E+03, &
      1.7394239E+00, 4.15726E+01, 2.6943E+01, 3.9616E+03, &
      1.7424222E+00, 4.10342E+01, 3.1568E+01, 4.6320E+03, &
      1.7429760E+00, 4.16119E+01, 3.1550E+01, 4.6280E+03, &
      1.7511343E+00, 4.44274E+01, 3.1301E+01, 4.5702E+03, &
      1.7598899E+00, 4.55999E+01, 3.1037E+01, 4.5092E+03, &
      1.7686894E+00, 4.63850E+01, 3.0775E+01, 4.4490E+03, &
      1.7775328E+00, 4.69944E+01, 3.0515E+01, 4.3896E+03, &
      1.7864205E+00, 4.74994E+01, 3.0257E+01, 4.3310E+03, &
      1.7953526E+00, 4.79326E+01, 3.0002E+01, 4.2732E+03, &
      1.8043294E+00, 4.83114E+01, 2.9749E+01, 4.2162E+03, &
      1.8133510E+00, 4.86457E+01, 2.9498E+01, 4.1599E+03, &
      1.8224178E+00, 4.89403E+01, 2.9248E+01, 4.1044E+03, &
      1.8315299E+00, 4.91977E+01, 2.9013E+01, 4.0512E+03, &
      1.8406875E+00, 4.94321E+01, 2.8792E+01, 4.0004E+03, &
      1.8498909E+00, 4.96449E+01, 2.8574E+01, 3.9506E+03, &
      1.8591404E+00, 4.98346E+01, 2.8359E+01, 3.9015E+03, &
      1.8684361E+00, 4.99991E+01, 2.8148E+01, 3.8532E+03, &
      1.8777783E+00, 5.01343E+01, 2.7939E+01, 3.8057E+03, &
      1.8871672E+00, 5.02326E+01, 2.7733E+01, 3.7590E+03, &
      1.8966030E+00, 5.02778E+01, 2.7530E+01, 3.7130E+03, &
      1.9060860E+00, 5.02299E+01, 2.7329E+01, 3.6677E+03, &
      1.9156165E+00, 4.99370E+01, 2.7131E+01, 3.6231E+03, &
      1.9208002E+00, 4.92783E+01, 2.7025E+01, 3.5993E+03, &
      1.9247997E+00, 4.93413E+01, 2.8854E+01, 3.8343E+03, &
      1.9251945E+00, 4.94604E+01, 2.8845E+01, 3.8323E+03, &
      1.9348205E+00, 5.06537E+01, 2.8626E+01, 3.7845E+03, &
      1.9444946E+00, 5.12077E+01, 2.8411E+01, 3.7374E+03, &
      1.9542171E+00, 5.16160E+01, 2.8198E+01, 3.6910E+03, &
      1.9639882E+00, 5.19535E+01, 2.7987E+01, 3.6453E+03, &
      1.9738081E+00, 5.22468E+01, 2.7779E+01, 3.6003E+03, &
      1.9836772E+00, 5.25085E+01, 2.7573E+01, 3.5560E+03, &
      1.9935955E+00, 5.27452E+01, 2.7370E+01, 3.5123E+03, &
      2.0035635E+00, 5.29608E+01, 2.7169E+01, 3.4693E+03, &
      2.0135813E+00, 5.31570E+01, 2.6970E+01, 3.4268E+03, &
      2.0236492E+00, 5.33351E+01, 2.6780E+01, 3.3859E+03, &
      2.0337675E+00, 5.35019E+01, 2.6597E+01, 3.3462E+03, &
      2.0439363E+00, 5.36573E+01, 2.6417E+01, 3.3071E+03, &
      2.0541560E+00, 5.38004E+01, 2.6240E+01, 3.2687E+03, &
      2.0644268E+00, 5.39300E+01, 2.6065E+01, 3.2308E+03, &
      2.0747489E+00, 5.40439E+01, 2.5892E+01, 3.1935E+03, &
      2.0851227E+00, 5.41380E+01, 2.5721E+01, 3.1568E+03, &
      2.0955483E+00, 5.42047E+01, 2.5552E+01, 3.1206E+03, &
      2.1060260E+00, 5.42263E+01, 2.5386E+01, 3.0849E+03, &
      2.1165562E+00, 5.41485E+01, 2.5221E+01, 3.0497E+03, &
      2.1244052E+00, 5.38523E+01, 2.5100E+01, 3.0240E+03, &
      2.1271389E+00, 5.34378E+01, 2.5058E+01, 3.0151E+03, &
      2.1321947E+00, 5.39589E+01, 2.6163E+01, 3.1402E+03, &
      2.1377746E+00, 5.43759E+01, 2.6076E+01, 3.1217E+03, &
      2.1484635E+00, 5.48132E+01, 2.5912E+01, 3.0866E+03, &
      2.1592058E+00, 5.51226E+01, 2.5749E+01, 3.0521E+03, &
      2.1700018E+00, 5.53804E+01, 2.5588E+01, 3.0180E+03, &
      2.1808519E+00, 5.56091E+01, 2.5429E+01, 2.9844E+03, &
      2.1917561E+00, 5.58188E+01, 2.5271E+01, 2.9512E+03, &
      2.2027149E+00, 5.60145E+01, 2.5114E+01, 2.9184E+03, &
      2.2137285E+00, 5.61996E+01, 2.4959E+01, 2.8860E+03, &
      2.2247971E+00, 5.63761E+01, 2.4805E+01, 2.8541E+03, &
      2.2359211E+00, 5.65451E+01, 2.4653E+01, 2.8226E+03, &
      2.2471007E+00, 5.67088E+01, 2.4504E+01, 2.7916E+03, &
      2.2583362E+00, 5.68683E+01, 2.4356E+01, 2.7610E+03, &
      2.2696279E+00, 5.70237E+01, 2.4205E+01, 2.7304E+03, &
      2.2809760E+00, 5.71737E+01, 2.4053E+01, 2.6998E+03, &
      2.2923809E+00, 5.73191E+01, 2.3901E+01, 2.6696E+03, &
      2.3038428E+00, 5.74601E+01, 2.3751E+01, 2.6397E+03, &
      2.3153620E+00, 5.75972E+01, 2.3602E+01, 2.6102E+03, &
      2.3269388E+00, 5.77308E+01, 2.3454E+01, 2.5810E+03, &
      2.3385735E+00, 5.78610E+01, 2.3308E+01, 2.5522E+03, &
      2.3502664E+00, 5.79882E+01, 2.3162E+01, 2.5237E+03, &
      2.3620177E+00, 5.81124E+01, 2.3017E+01, 2.4956E+03, &
      2.3738278E+00, 5.82340E+01, 2.2873E+01, 2.4677E+03, &
      2.3856970E+00, 5.83530E+01, 2.2730E+01, 2.4402E+03, &
      2.3976254E+00, 5.84696E+01, 2.2589E+01, 2.4130E+03, &
      2.4096136E+00, 5.85839E+01, 2.2448E+01, 2.3861E+03, &
      2.4216616E+00, 5.86960E+01, 2.2308E+01, 2.3596E+03, &
      2.4337699E+00, 5.88061E+01, 2.2169E+01, 2.3333E+03, &
      2.4459388E+00, 5.89142E+01, 2.2030E+01, 2.3073E+03, &
      2.4581685E+00, 5.90205E+01, 2.1893E+01, 2.2816E+03, &
      2.4704593E+00, 5.91251E+01, 2.1757E+01, 2.2562E+03, &
      2.4828116E+00, 5.92282E+01, 2.1621E+01, 2.2310E+03, &
      2.4952257E+00, 5.93298E+01, 2.1486E+01, 2.2062E+03, &
      2.5077018E+00, 5.94298E+01, 2.1350E+01, 2.1814E+03, &
      2.5202403E+00, 5.95276E+01, 2.1214E+01, 2.1568E+03, &
      2.5328415E+00, 5.96233E+01, 2.1078E+01, 2.1325E+03, &
      2.5455057E+00, 5.97171E+01, 2.0944E+01, 2.1084E+03, &
      2.5582333E+00, 5.98091E+01, 2.0810E+01, 2.0846E+03, &
      2.5710244E+00, 5.98993E+01, 2.0677E+01, 2.0611E+03, &
      2.5838796E+00, 5.99879E+01, 2.0545E+01, 2.0378E+03, &
      2.5967990E+00, 6.00748E+01, 2.0414E+01, 2.0148E+03, &
      2.6097829E+00, 6.01603E+01, 2.0283E+01, 1.9920E+03, &
      2.6228319E+00, 6.02443E+01, 2.0153E+01, 1.9695E+03, &
      2.6359460E+00, 6.03268E+01, 2.0024E+01, 1.9473E+03, &
      2.6491257E+00, 6.04081E+01, 1.9896E+01, 1.9253E+03, &
      2.6623714E+00, 6.04880E+01, 1.9769E+01, 1.9035E+03, &
      2.6756832E+00, 6.05667E+01, 1.9642E+01, 1.8820E+03, &
      2.6890617E+00, 6.06442E+01, 1.9516E+01, 1.8608E+03, &
      2.7025070E+00, 6.07206E+01, 1.9391E+01, 1.8397E+03, &
      2.7160195E+00, 6.09983E+01, 1.9263E+01, 1.8186E+03, &
      2.7295996E+00, 6.10715E+01, 1.9136E+01, 1.7977E+03, &
      2.7432476E+00, 6.11433E+01, 1.9010E+01, 1.7771E+03, &
      2.7569638E+00, 6.12137E+01, 1.8885E+01, 1.7567E+03, &
      2.7707486E+00, 6.12830E+01, 1.8760E+01, 1.7365E+03, &
      2.7846024E+00, 6.14922E+01, 1.8636E+01, 1.7164E+03, &
      2.7985254E+00, 6.15586E+01, 1.8510E+01, 1.6965E+03, &
      2.8125180E+00, 6.16236E+01, 1.8385E+01, 1.6767E+03, &
      2.8265806E+00, 6.16872E+01, 1.8261E+01, 1.6572E+03, &
      2.8407135E+00, 6.17495E+01, 1.8138E+01, 1.6380E+03, &
      2.8549171E+00, 6.18106E+01, 1.8016E+01, 1.6190E+03, &
      2.8691917E+00, 6.18707E+01, 1.7895E+01, 1.6002E+03, &
      2.8835376E+00, 6.19297E+01, 1.7774E+01, 1.5816E+03, &
      2.8979553E+00, 6.19878E+01, 1.7655E+01, 1.5632E+03, &
      2.9124451E+00, 6.20452E+01, 1.7537E+01, 1.5451E+03, &
      2.9270073E+00, 6.21020E+01, 1.7419E+01, 1.5272E+03, &
      2.9416424E+00, 6.21583E+01, 1.7302E+01, 1.5095E+03, &
      2.9563506E+00, 6.22144E+01, 1.7187E+01, 1.4921E+03, &
      2.9711323E+00, 6.22708E+01, 1.7072E+01, 1.4748E+03, &
      2.9859880E+00, 6.23279E+01, 1.6958E+01, 1.4578E+03, &
      3.0009179E+00, 6.23911E+01, 1.6844E+01, 1.4408E+03, &
      3.0159225E+00, 6.24781E+01, 1.6713E+01, 1.4226E+03, &
      3.0310021E+00, 6.25382E+01, 1.6583E+01, 1.4046E+03, &
      3.0461571E+00, 6.25903E+01, 1.6452E+01, 1.3867E+03, &
      3.0613879E+00, 6.26380E+01, 1.6322E+01, 1.3690E+03, &
      3.0766949E+00, 6.26828E+01, 1.6194E+01, 1.3516E+03, &
      3.0920783E+00, 6.27251E+01, 1.6067E+01, 1.3344E+03, &
      3.1075387E+00, 6.27655E+01, 1.5941E+01, 1.3175E+03, &
      3.1230764E+00, 6.28040E+01, 1.5816E+01, 1.3007E+03, &
      3.1386918E+00, 6.28409E+01, 1.5693E+01, 1.2843E+03, &
      3.1543853E+00, 6.28762E+01, 1.5570E+01, 1.2680E+03, &
      3.1701572E+00, 6.29101E+01, 1.5449E+01, 1.2519E+03, &
      3.1860080E+00, 6.29427E+01, 1.5329E+01, 1.2361E+03, &
      3.2019380E+00, 6.29740E+01, 1.5210E+01, 1.2205E+03, &
      3.2179477E+00, 6.30041E+01, 1.5092E+01, 1.2051E+03, &
      3.2340374E+00, 6.30330E+01, 1.4975E+01, 1.1899E+03, &
      3.2502076E+00, 6.30608E+01, 1.4859E+01, 1.1749E+03, &
      3.2664587E+00, 6.30876E+01, 1.4744E+01, 1.1601E+03, &
      3.2827910E+00, 6.31133E+01, 1.4630E+01, 1.1455E+03, &
      3.2992049E+00, 6.31381E+01, 1.4518E+01, 1.1311E+03, &
      3.3157009E+00, 6.31619E+01, 1.4406E+01, 1.1169E+03, &
      3.3322794E+00, 6.31848E+01, 1.4295E+01, 1.1029E+03, &
      3.3489408E+00, 6.32068E+01, 1.4185E+01, 1.0891E+03, &
      3.3656856E+00, 6.32280E+01, 1.4077E+01, 1.0755E+03, &
      3.3825140E+00, 6.32485E+01, 1.3969E+01, 1.0620E+03, &
      3.3994265E+00, 6.32681E+01, 1.3862E+01, 1.0488E+03, &
      3.4164237E+00, 6.32870E+01, 1.3757E+01, 1.0357E+03, &
      3.4335058E+00, 6.33053E+01, 1.3652E+01, 1.0228E+03, &
      3.4506733E+00, 6.33228E+01, 1.3548E+01, 1.0100E+03, &
      3.4679267E+00, 6.33398E+01, 1.3445E+01, 9.9745E+02, &
      3.4852663E+00, 6.34590E+01, 1.3342E+01, 9.8500E+02, &
      3.5026927E+00, 6.34754E+01, 1.3237E+01, 9.7248E+02, &
      3.5202061E+00, 6.34908E+01, 1.3133E+01, 9.6012E+02, &
      3.5378072E+00, 6.35051E+01, 1.3030E+01, 9.4794E+02, &
      3.5554962E+00, 6.35186E+01, 1.2928E+01, 9.3592E+02, &
      3.5732737E+00, 6.35312E+01, 1.2827E+01, 9.2406E+02, &
      3.5911400E+00, 6.35430E+01, 1.2727E+01, 9.1235E+02, &
      3.6090957E+00, 6.35540E+01, 1.2627E+01, 9.0081E+02, &
      3.6271412E+00, 6.35643E+01, 1.2529E+01, 8.8942E+02, &
      3.6452769E+00, 6.35738E+01, 1.2431E+01, 8.7819E+02, &
      3.6635033E+00, 6.35826E+01, 1.2335E+01, 8.6711E+02, &
      3.6818208E+00, 6.35908E+01, 1.2239E+01, 8.5617E+02, &
      3.7002299E+00, 6.35983E+01, 1.2144E+01, 8.4538E+02, &
      3.7187311E+00, 6.36052E+01, 1.2050E+01, 8.3474E+02, &
      3.7373247E+00, 6.36115E+01, 1.1956E+01, 8.2424E+02, &
      3.7560114E+00, 6.36172E+01, 1.1864E+01, 8.1388E+02, &
      3.7747914E+00, 6.36224E+01, 1.1772E+01, 8.0366E+02, &
      3.7936654E+00, 6.36271E+01, 1.1681E+01, 7.9358E+02, &
      3.8126337E+00, 6.36313E+01, 1.1591E+01, 7.8363E+02, &
      3.8316969E+00, 6.36349E+01, 1.1502E+01, 7.7381E+02, &
      3.8508554E+00, 6.36885E+01, 1.1413E+01, 7.6408E+02, &
      3.8701096E+00, 6.36917E+01, 1.1323E+01, 7.5437E+02, &
      3.8894602E+00, 6.36941E+01, 1.1234E+01, 7.4480E+02, &
      3.9089075E+00, 6.36959E+01, 1.1146E+01, 7.3535E+02, &
      3.9284520E+00, 6.36970E+01, 1.1058E+01, 7.2602E+02, &
      3.9480943E+00, 6.36975E+01, 1.0972E+01, 7.1683E+02, &
      3.9678347E+00, 6.36975E+01, 1.0886E+01, 7.0775E+02, &
      3.9876739E+00, 6.36969E+01, 1.0800E+01, 6.9880E+02, &
      4.055024E+00, 6.40956E+01, 1.0520E+01, 6.6962E+02, &
      4.334821E+00, 6.39832E+01, 9.4779E+00, 5.6531E+02, &
      4.633924E+00, 6.37889E+01, 8.5432E+00, 4.7756E+02, &
      4.953664E+00, 6.35420E+01, 7.7110E+00, 4.0406E+02, &
      5.295467E+00, 6.32455E+01, 6.9692E+00, 3.4240E+02, &
      5.660855E+00, 6.28946E+01, 6.3070E+00, 2.9059E+02, &
      6.051453E+00, 6.24720E+01, 5.7000E+00, 2.4636E+02, &
      6.469004E+00, 6.19213E+01, 5.1362E+00, 2.0832E+02, &
      6.915365E+00, 6.11594E+01, 4.6345E+00, 1.7646E+02, &
      7.392525E+00, 5.99548E+01, 4.1893E+00, 1.4977E+02, &
      7.902609E+00, 5.69139E+01, 3.7812E+00, 1.2698E+02, &
      7.909678E+00, 5.68216E+01, 3.7760E+00, 1.2670E+02, &
      8.030745E+00, 5.38366E+01, 3.6885E+00, 1.2203E+02, &
      8.063029E+00, 5.03421E+01, 3.6658E+00, 1.2082E+02, &
      8.111456E+00, 5.38428E+01, 1.0452E+01, 3.3356E+02, &
      8.232522E+00, 5.68834E+01, 1.0119E+01, 3.1833E+02, &
      8.447890E+00, 5.84369E+01, 9.5635E+00, 2.9346E+02, &
      8.739444E+00, 5.86150E+01, 9.0214E+00, 2.6786E+02, &
      8.873211E+00, 5.75023E+01, 8.8050E+00, 2.5761E+02, &
      8.908882E+00, 5.58405E+01, 8.7477E+00, 2.5494E+02, &
      8.962389E+00, 5.77058E+01, 1.2075E+01, 3.4813E+02, &
      9.030794E+00, 5.88604E+01, 1.1899E+01, 3.4053E+02, &
      9.096156E+00, 5.94438E+01, 1.1735E+01, 3.3348E+02, &
      9.206316E+00, 5.99990E+01, 1.1465E+01, 3.2203E+02, &
      9.347229E+00, 5.99731E+01, 1.1132E+01, 3.0809E+02, &
      9.384806E+00, 5.92505E+01, 1.1049E+01, 3.0461E+02, &
      9.441171E+00, 6.03395E+01, 1.2653E+01, 3.4613E+02, &
      9.582084E+00, 6.15929E+01, 1.2348E+01, 3.3293E+02, &
      9.653919E+00, 6.19877E+01, 1.2198E+01, 3.2648E+02, &
      1.032004E+01, 6.40767E+01, 1.0941E+01, 2.7434E+02, &
      1.103212E+01, 6.52185E+01, 9.8237E+00, 2.3080E+02, &
      1.179334E+01, 6.59704E+01, 8.8246E+00, 1.9428E+02, &
      1.260708E+01, 6.64794E+01, 7.9097E+00, 1.6322E+02, &
      1.347697E+01, 6.68189E+01, 7.0859E+00, 1.3708E+02, &
      1.440688E+01, 6.70426E+01, 6.3487E+00, 1.1516E+02, &
      1.540095E+01, 6.71849E+01, 5.6900E+00, 9.6794E+01, &
      1.646362E+01, 6.73367E+01, 5.0967E+00, 8.1332E+01, &
      1.759961E+01, 6.73708E+01, 4.5570E+00, 6.8235E+01, &
      1.881398E+01, 6.74348E+01, 4.0737E+00, 5.7252E+01, &
      2.011215E+01, 6.74038E+01, 3.6352E+00, 4.7970E+01, &
      2.149988E+01, 6.73504E+01, 3.2468E+00, 4.0240E+01, &
      2.298338E+01, 6.72834E+01, 2.9024E+00, 3.3794E+01, &
      2.456923E+01, 6.72083E+01, 2.5968E+00, 2.8416E+01, &
      2.626450E+01, 6.71306E+01, 2.3254E+00, 2.3921E+01, &
      2.807676E+01, 6.70572E+01, 2.0677E+00, 2.0013E+01, &
      3.001405E+01, 6.69602E+01, 1.8352E+00, 1.6722E+01, &
      3.208502E+01, 6.68537E+01, 1.6301E+00, 1.3991E+01, &
      3.429889E+01, 6.67401E+01, 1.4491E+00, 1.1724E+01, &
      3.666551E+01, 6.66184E+01, 1.2871E+00, 9.8213E+00, &
      3.919543E+01, 6.64839E+01, 1.1426E+00, 8.2298E+00, &
      4.189992E+01, 6.63296E+01, 1.0121E+00, 6.8887E+00, &
      4.479101E+01, 6.61395E+01, 8.9477E-01, 5.7622E+00, &
      4.788159E+01, 6.58829E+01, 7.9118E-01, 4.8258E+00, &
      5.118542E+01, 6.54662E+01, 7.0001E-01, 4.0488E+00, &
      5.450534E+01, 6.43897E+01, 6.2421E-01, 3.4380E+00, &
      5.471721E+01, 6.42184E+01, 6.1983E-01, 3.4038E+00, &
      5.533961E+01, 6.32423E+01, 6.0722E-01, 3.3058E+00, &
      5.556208E+01, 6.18682E+01, 6.0281E-01, 3.2717E+00, &
      5.589579E+01, 6.32566E+01, 3.3013E+00, 1.5569E+01, &
      5.673005E+01, 6.45109E+01, 3.2271E+00, 1.5004E+01, &
      5.849270E+01, 6.53995E+01, 3.0790E+00, 1.3902E+01, &
      6.252870E+01, 6.62164E+01, 2.7650E+00, 1.1716E+01, &
      6.684318E+01, 6.66288E+01, 2.4787E+00, 9.8608E+00, &
      7.145536E+01, 6.68827E+01, 2.2211E+00, 8.2992E+00, &
      7.638578E+01, 6.70512E+01, 1.9890E+00, 6.9840E+00, &
      8.165640E+01, 6.71685E+01, 1.7786E+00, 5.8720E+00, &
      8.729069E+01, 6.72458E+01, 1.5856E+00, 4.9262E+00, &
      9.331374E+01, 6.72943E+01, 1.4137E+00, 4.1364E+00, &
      9.975239E+01, 6.73236E+01, 1.2606E+00, 3.4766E+00, &
      1.066353E+02, 6.73395E+01, 1.1242E+00, 2.9253E+00, &
      1.139931E+02, 6.73751E+01, 1.0012E+00, 2.4609E+00, &
      1.218587E+02, 6.73719E+01, 8.8923E-01, 2.0680E+00, &
      1.302669E+02, 6.73624E+01, 7.8989E-01, 1.7408E+00, &
      1.392553E+02, 6.73488E+01, 7.0172E-01, 1.4681E+00, &
      1.488640E+02, 6.73325E+01, 6.2345E-01, 1.2406E+00, &
      1.591356E+02, 6.73146E+01, 5.5395E-01, 1.0509E+00, &
      1.701159E+02, 6.72959E+01, 4.9225E-01, 8.9255E-01, &
      1.818539E+02, 6.72768E+01, 4.3746E-01, 7.6024E-01, &
      1.944018E+02, 6.72578E+01, 3.8879E-01, 6.4965E-01, &
      2.078156E+02, 6.72390E+01, 3.4557E-01, 5.5711E-01, &
      2.221548E+02, 6.72208E+01, 3.0718E-01, 4.7960E-01, &
      2.374835E+02, 6.72031E+01, 2.7307E-01, 4.1461E-01, &
      2.538699E+02, 6.71863E+01, 2.4378E-01, 3.6106E-01, &
      2.713869E+02, 6.71709E+01, 2.1790E-01, 3.1608E-01, &
      2.901126E+02, 6.71568E+01, 1.9478E-01, 2.7801E-01, &
      3.101304E+02, 6.71436E+01, 1.7413E-01, 2.4571E-01, &
      3.315294E+02, 6.71315E+01, 1.5567E-01, 2.1826E-01, &
      3.544049E+02, 6.71203E+01, 1.3919E-01, 1.9486E-01, &
      3.788588E+02, 6.71099E+01, 1.2445E-01, 1.7486E-01, &
      4.050001E+02, 6.71004E+01, 1.1128E-01, 1.5771E-01, &
      4.329451E+02, 6.70916E+01, 9.9514E-02, 1.4295E-01]
   case(68)
      frel = -6.9660E-01
      allocate(nist_tab(3596))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.214000E-03, 1.13349E+01, 0.0000E+00, 2.2241E-06, &
      4.278500E-03, 1.12763E+01, 0.0000E+00, 2.4241E-06, &
      4.295700E-03, 1.12116E+01, 0.0000E+00, 2.4798E-06, &
      4.321500E-03, 1.12735E+01, 1.2433E-01, 7.2383E+03, &
      4.386000E-03, 1.13244E+01, 1.2488E-01, 7.1632E+03, &
      1.069000E-02, 1.09260E+01, 2.8242E-01, 6.6467E+03, &
      1.142761E-02, 1.08355E+01, 3.1136E-01, 6.8548E+03, &
      1.221612E-02, 1.07304E+01, 3.4450E-01, 7.0949E+03, &
      1.305903E-02, 1.10140E+01, 3.8246E-01, 7.3683E+03, &
      1.396010E-02, 1.08707E+01, 4.2594E-01, 7.6763E+03, &
      1.492335E-02, 1.07017E+01, 4.7574E-01, 8.0203E+03, &
      1.595306E-02, 1.05009E+01, 5.3277E-01, 8.4020E+03, &
      1.705382E-02, 1.02594E+01, 5.9806E-01, 8.8228E+03, &
      1.823053E-02, 9.96477E+00, 6.7274E-01, 9.2840E+03, &
      1.948844E-02, 9.59762E+00, 7.5803E-01, 9.7858E+03, &
      2.083314E-02, 9.12687E+00, 8.5526E-01, 1.0328E+04, &
      2.227063E-02, 8.49800E+00, 9.6593E-01, 1.0912E+04, &
      2.380730E-02, 7.60417E+00, 1.0916E+00, 1.1536E+04, &
      2.545001E-02, 6.19628E+00, 1.2341E+00, 1.2200E+04, &
      2.720606E-02, 3.47413E+00, 1.3952E+00, 1.2902E+04, &
      2.881200E-02, -3.98499E+00, 1.5500E+00, 1.3535E+04, &
      2.908327E-02, -7.97707E+00, 1.5769E+00, 1.3641E+04, &
      2.925300E-02, -1.32047E+01, 1.5937E+00, 1.3707E+04, &
      2.937060E-02, -2.47216E+01, 1.6055E+00, 1.3752E+04, &
      2.954700E-02, -1.20411E+01, 2.4533E+01, 2.0890E+05, &
      2.998800E-02, -8.20267E-01, 2.3362E+01, 1.9600E+05, &
      3.109002E-02, 8.00407E+00, 2.0782E+01, 1.6817E+05, &
      3.323523E-02, 1.41917E+01, 1.6869E+01, 1.2770E+05, &
      3.552846E-02, 1.67708E+01, 1.3748E+01, 9.7355E+04, &
      3.797993E-02, 1.78228E+01, 1.1346E+01, 7.5159E+04, &
      4.060054E-02, 1.81090E+01, 9.5843E+00, 5.9391E+04, &
      4.340198E-02, 1.80104E+01, 8.3567E+00, 4.8441E+04, &
      4.639671E-02, 1.77364E+01, 7.5582E+00, 4.0984E+04, &
      4.959809E-02, 1.74051E+01, 7.0975E+00, 3.6002E+04, &
      5.302035E-02, 1.70826E+01, 6.9015E+00, 3.2748E+04, &
      5.667876E-02, 1.68045E+01, 6.9136E+00, 3.0688E+04, &
      5.860400E-02, 1.66851E+01, 6.9842E+00, 2.9983E+04, &
      5.950100E-02, 1.66278E+01, 7.0288E+00, 2.9720E+04, &
      5.974020E-02, 1.66014E+01, 7.0418E+00, 2.9656E+04, &
      6.009900E-02, 1.66008E+01, 7.0928E+00, 2.9692E+04, &
      6.058959E-02, 1.65898E+01, 7.1229E+00, 2.9576E+04, &
      6.099600E-02, 1.65783E+01, 7.1492E+00, 2.9488E+04, &
      6.477028E-02, 1.64960E+01, 7.4422E+00, 2.8907E+04, &
      6.923942E-02, 1.64964E+01, 7.8749E+00, 2.8614E+04, &
      7.401695E-02, 1.66185E+01, 8.4045E+00, 2.8567E+04, &
      7.912411E-02, 1.68920E+01, 9.0191E+00, 2.8678E+04, &
      8.458368E-02, 1.73191E+01, 9.4355E+00, 2.8065E+04, &
      9.041995E-02, 1.77819E+01, 9.7393E+00, 2.7099E+04, &
      9.665893E-02, 1.82536E+01, 1.0030E+01, 2.6105E+04, &
      1.0000000E-01, 2.07708E+01, 1.0169E+01, 2.5585E+04, &
      1.0050000E-01, 2.08075E+01, 1.0189E+01, 2.5508E+04, &
      1.0100250E-01, 2.08444E+01, 1.0209E+01, 2.5430E+04, &
      1.0150751E-01, 2.08813E+01, 1.0229E+01, 2.5353E+04, &
      1.0201505E-01, 2.09183E+01, 1.0248E+01, 2.5274E+04, &
      1.0252513E-01, 2.09554E+01, 1.0268E+01, 2.5196E+04, &
      1.0303775E-01, 2.09925E+01, 1.0287E+01, 2.5118E+04, &
      1.0355294E-01, 2.10297E+01, 1.0306E+01, 2.5039E+04, &
      1.0407070E-01, 2.10670E+01, 1.0325E+01, 2.4960E+04, &
      1.0459106E-01, 2.11043E+01, 1.0343E+01, 2.4881E+04, &
      1.0511401E-01, 2.11417E+01, 1.0362E+01, 2.4801E+04, &
      1.0563958E-01, 2.11791E+01, 1.0380E+01, 2.4721E+04, &
      1.0616778E-01, 2.12165E+01, 1.0398E+01, 2.4641E+04, &
      1.0669862E-01, 2.12540E+01, 1.0416E+01, 2.4561E+04, &
      1.0723211E-01, 2.12915E+01, 1.0434E+01, 2.4481E+04, &
      1.0776827E-01, 2.13291E+01, 1.0452E+01, 2.4400E+04, &
      1.0830712E-01, 2.13666E+01, 1.0469E+01, 2.4319E+04, &
      1.0884865E-01, 2.14042E+01, 1.0486E+01, 2.4238E+04, &
      1.0939289E-01, 2.14417E+01, 1.0503E+01, 2.4156E+04, &
      1.0993986E-01, 2.14793E+01, 1.0520E+01, 2.4075E+04, &
      1.1048956E-01, 2.15168E+01, 1.0537E+01, 2.3993E+04, &
      1.1104201E-01, 2.15543E+01, 1.0553E+01, 2.3911E+04, &
      1.1159722E-01, 2.15918E+01, 1.0569E+01, 2.3828E+04, &
      1.1215520E-01, 2.16293E+01, 1.0585E+01, 2.3746E+04, &
      1.1271598E-01, 2.16668E+01, 1.0601E+01, 2.3663E+04, &
      1.1327956E-01, 2.17041E+01, 1.0617E+01, 2.3580E+04, &
      1.1384596E-01, 2.17415E+01, 1.0632E+01, 2.3496E+04, &
      1.1441519E-01, 2.17787E+01, 1.0647E+01, 2.3413E+04, &
      1.1498726E-01, 2.18159E+01, 1.0662E+01, 2.3329E+04, &
      1.1556220E-01, 2.18531E+01, 1.0677E+01, 2.3245E+04, &
      1.1614001E-01, 2.18901E+01, 1.0691E+01, 2.3161E+04, &
      1.1672071E-01, 2.19270E+01, 1.0706E+01, 2.3076E+04, &
      1.1730431E-01, 2.19638E+01, 1.0720E+01, 2.2992E+04, &
      1.1789083E-01, 2.20005E+01, 1.0734E+01, 2.2907E+04, &
      1.1848029E-01, 2.20370E+01, 1.0747E+01, 2.2822E+04, &
      1.1907269E-01, 2.20734E+01, 1.0761E+01, 2.2737E+04, &
      1.1966805E-01, 2.21096E+01, 1.0774E+01, 2.2651E+04, &
      1.2026639E-01, 2.21457E+01, 1.0787E+01, 2.2566E+04, &
      1.2086772E-01, 2.21816E+01, 1.0799E+01, 2.2480E+04, &
      1.2147206E-01, 2.22173E+01, 1.0812E+01, 2.2394E+04, &
      1.2207942E-01, 2.22528E+01, 1.0824E+01, 2.2307E+04, &
      1.2268982E-01, 2.22880E+01, 1.0836E+01, 2.2221E+04, &
      1.2330327E-01, 2.23230E+01, 1.0848E+01, 2.2134E+04, &
      1.2391979E-01, 2.23578E+01, 1.0859E+01, 2.2048E+04, &
      1.2453939E-01, 2.23922E+01, 1.0871E+01, 2.1961E+04, &
      1.2516208E-01, 2.24264E+01, 1.0882E+01, 2.1874E+04, &
      1.2578789E-01, 2.24603E+01, 1.0892E+01, 2.1786E+04, &
      1.2641683E-01, 2.24938E+01, 1.0903E+01, 2.1699E+04, &
      1.2704892E-01, 2.25270E+01, 1.0913E+01, 2.1611E+04, &
      1.2768416E-01, 2.25598E+01, 1.0923E+01, 2.1524E+04, &
      1.2832258E-01, 2.25922E+01, 1.0933E+01, 2.1436E+04, &
      1.2896419E-01, 2.26242E+01, 1.0942E+01, 2.1348E+04, &
      1.2960902E-01, 2.26557E+01, 1.0952E+01, 2.1259E+04, &
      1.3025706E-01, 2.26868E+01, 1.0961E+01, 2.1171E+04, &
      1.3090835E-01, 2.27173E+01, 1.0969E+01, 2.1082E+04, &
      1.3156289E-01, 2.27473E+01, 1.0978E+01, 2.0994E+04, &
      1.3222070E-01, 2.27768E+01, 1.0986E+01, 2.0905E+04, &
      1.3288181E-01, 2.28056E+01, 1.0994E+01, 2.0816E+04, &
      1.3354621E-01, 2.28338E+01, 1.1002E+01, 2.0727E+04, &
      1.3421395E-01, 2.28614E+01, 1.1009E+01, 2.0638E+04, &
      1.3488502E-01, 2.28882E+01, 1.1016E+01, 2.0549E+04, &
      1.3555944E-01, 2.29142E+01, 1.1023E+01, 2.0459E+04, &
      1.3623724E-01, 2.29394E+01, 1.1030E+01, 2.0370E+04, &
      1.3691842E-01, 2.29638E+01, 1.1036E+01, 2.0280E+04, &
      1.3760302E-01, 2.29872E+01, 1.1043E+01, 2.0191E+04, &
      1.3829103E-01, 2.30097E+01, 1.1048E+01, 2.0101E+04, &
      1.3898249E-01, 2.30311E+01, 1.1054E+01, 2.0011E+04, &
      1.3967740E-01, 2.30514E+01, 1.1059E+01, 1.9921E+04, &
      1.4037579E-01, 2.30705E+01, 1.1064E+01, 1.9831E+04, &
      1.4107766E-01, 2.30883E+01, 1.1069E+01, 1.9741E+04, &
      1.4178305E-01, 2.31047E+01, 1.1074E+01, 1.9651E+04, &
      1.4249197E-01, 2.31197E+01, 1.1078E+01, 1.9560E+04, &
      1.4320443E-01, 2.31331E+01, 1.1082E+01, 1.9470E+04, &
      1.4392045E-01, 2.31448E+01, 1.1086E+01, 1.9380E+04, &
      1.4464005E-01, 2.31547E+01, 1.1089E+01, 1.9289E+04, &
      1.4536325E-01, 2.31626E+01, 1.1092E+01, 1.9199E+04, &
      1.4609007E-01, 2.31683E+01, 1.1095E+01, 1.9108E+04, &
      1.4682052E-01, 2.31717E+01, 1.1098E+01, 1.9017E+04, &
      1.4755462E-01, 2.31726E+01, 1.1100E+01, 1.8927E+04, &
      1.4829239E-01, 2.31707E+01, 1.1102E+01, 1.8836E+04, &
      1.4903386E-01, 2.31658E+01, 1.1104E+01, 1.8745E+04, &
      1.4977903E-01, 2.31576E+01, 1.1105E+01, 1.8655E+04, &
      1.5052792E-01, 2.31458E+01, 1.1106E+01, 1.8564E+04, &
      1.5128056E-01, 2.31299E+01, 1.1107E+01, 1.8473E+04, &
      1.5203696E-01, 2.31095E+01, 1.1108E+01, 1.8382E+04, &
      1.5279715E-01, 2.30842E+01, 1.1108E+01, 1.8291E+04, &
      1.5356113E-01, 2.30532E+01, 1.1108E+01, 1.8201E+04, &
      1.5432894E-01, 2.30158E+01, 1.1108E+01, 1.8110E+04, &
      1.5510058E-01, 2.29713E+01, 1.1108E+01, 1.8019E+04, &
      1.5587609E-01, 2.29186E+01, 1.1107E+01, 1.7928E+04, &
      1.5665547E-01, 2.28563E+01, 1.1106E+01, 1.7837E+04, &
      1.5743875E-01, 2.27829E+01, 1.1105E+01, 1.7747E+04, &
      1.5822594E-01, 2.26965E+01, 1.1103E+01, 1.7656E+04, &
      1.5901707E-01, 2.25945E+01, 1.1101E+01, 1.7565E+04, &
      1.5981215E-01, 2.24735E+01, 1.1099E+01, 1.7475E+04, &
      1.6061121E-01, 2.23291E+01, 1.1097E+01, 1.7384E+04, &
      1.6141427E-01, 2.21551E+01, 1.1094E+01, 1.7293E+04, &
      1.6222134E-01, 2.19426E+01, 1.1091E+01, 1.7203E+04, &
      1.6303245E-01, 2.16779E+01, 1.1088E+01, 1.7112E+04, &
      1.6384761E-01, 2.13391E+01, 1.1085E+01, 1.7022E+04, &
      1.6466685E-01, 2.08872E+01, 1.1081E+01, 1.6931E+04, &
      1.6549018E-01, 2.02424E+01, 1.1077E+01, 1.6841E+04, &
      1.6631763E-01, 1.91976E+01, 1.1073E+01, 1.6751E+04, &
      1.6714922E-01, 1.68073E+01, 1.1068E+01, 1.6660E+04, &
      1.6751369E-01, 1.27085E+01, 1.1066E+01, 1.6621E+04, &
      1.6768632E-01, 1.28511E+01, 1.9211E+01, 2.8824E+04, &
      1.6798497E-01, 1.69178E+01, 1.9050E+01, 2.8532E+04, &
      1.6882489E-01, 2.02020E+01, 1.8615E+01, 2.7742E+04, &
      1.6966902E-01, 2.16970E+01, 1.8203E+01, 2.6993E+04, &
      1.7051736E-01, 2.26320E+01, 1.7812E+01, 2.6282E+04, &
      1.7136995E-01, 2.32653E+01, 1.7442E+01, 2.5608E+04, &
      1.7222680E-01, 2.36915E+01, 1.7091E+01, 2.4968E+04, &
      1.7308793E-01, 2.39468E+01, 1.6759E+01, 2.4361E+04, &
      1.7395337E-01, 2.40300E+01, 1.6444E+01, 2.3784E+04, &
      1.7482314E-01, 2.38895E+01, 1.6145E+01, 2.3235E+04, &
      1.7569726E-01, 2.33199E+01, 1.5861E+01, 2.2714E+04, &
      1.7657574E-01, 2.03429E+01, 1.5593E+01, 2.2218E+04, &
      1.7660087E-01, 1.99814E+01, 1.5585E+01, 2.2204E+04, &
      1.7679912E-01, 2.01340E+01, 2.0764E+01, 2.9549E+04, &
      1.7745862E-01, 2.39084E+01, 2.0369E+01, 2.8879E+04, &
      1.7834591E-01, 2.55540E+01, 1.9863E+01, 2.8022E+04, &
      1.7923764E-01, 2.65431E+01, 1.9384E+01, 2.7210E+04, &
      1.8013383E-01, 2.72564E+01, 1.8930E+01, 2.6440E+04, &
      1.8103450E-01, 2.78061E+01, 1.8499E+01, 2.5709E+04, &
      1.8193967E-01, 2.82463E+01, 1.8089E+01, 2.5016E+04, &
      1.8284937E-01, 2.86060E+01, 1.7701E+01, 2.4357E+04, &
      1.8376362E-01, 2.89034E+01, 1.7333E+01, 2.3732E+04, &
      1.8468244E-01, 2.91503E+01, 1.6984E+01, 2.3138E+04, &
      1.8560585E-01, 2.93557E+01, 1.6655E+01, 2.2577E+04, &
      1.8653388E-01, 2.95275E+01, 1.6346E+01, 2.2049E+04, &
      1.8746655E-01, 2.96720E+01, 1.6057E+01, 2.1550E+04, &
      1.8840388E-01, 2.97938E+01, 1.5784E+01, 2.1079E+04, &
      1.8934590E-01, 2.98966E+01, 1.5528E+01, 2.0634E+04, &
      1.9029263E-01, 2.99834E+01, 1.5287E+01, 2.0212E+04, &
      1.9124409E-01, 3.00563E+01, 1.5059E+01, 1.9813E+04, &
      1.9220031E-01, 3.01174E+01, 1.4845E+01, 1.9433E+04, &
      1.9316131E-01, 3.01680E+01, 1.4642E+01, 1.9073E+04, &
      1.9412712E-01, 3.02094E+01, 1.4451E+01, 1.8730E+04, &
      1.9509776E-01, 3.02423E+01, 1.4271E+01, 1.8405E+04, &
      1.9607325E-01, 3.02681E+01, 1.4103E+01, 1.8097E+04, &
      1.9705361E-01, 3.02886E+01, 1.3945E+01, 1.7807E+04, &
      1.9803888E-01, 3.03049E+01, 1.3799E+01, 1.7531E+04, &
      1.9902907E-01, 3.03180E+01, 1.3661E+01, 1.7270E+04, &
      2.0002422E-01, 3.03287E+01, 1.3532E+01, 1.7022E+04, &
      2.0102434E-01, 3.03374E+01, 1.3411E+01, 1.6786E+04, &
      2.0202946E-01, 3.03447E+01, 1.3297E+01, 1.6561E+04, &
      2.0303961E-01, 3.03508E+01, 1.3190E+01, 1.6346E+04, &
      2.0405481E-01, 3.03562E+01, 1.3089E+01, 1.6140E+04, &
      2.0507508E-01, 3.03609E+01, 1.2993E+01, 1.5942E+04, &
      2.0610046E-01, 3.03653E+01, 1.2903E+01, 1.5753E+04, &
      2.0713096E-01, 3.03695E+01, 1.2817E+01, 1.5570E+04, &
      2.0816661E-01, 3.03736E+01, 1.2736E+01, 1.5395E+04, &
      2.0920745E-01, 3.03778E+01, 1.2659E+01, 1.5225E+04, &
      2.1025348E-01, 3.03820E+01, 1.2586E+01, 1.5062E+04, &
      2.1130475E-01, 3.03864E+01, 1.2516E+01, 1.4903E+04, &
      2.1236128E-01, 3.03910E+01, 1.2449E+01, 1.4750E+04, &
      2.1342308E-01, 3.03958E+01, 1.2385E+01, 1.4602E+04, &
      2.1449020E-01, 3.04009E+01, 1.2324E+01, 1.4458E+04, &
      2.1556265E-01, 3.04064E+01, 1.2266E+01, 1.4318E+04, &
      2.1664046E-01, 3.04121E+01, 1.2210E+01, 1.4181E+04, &
      2.1772366E-01, 3.04182E+01, 1.2156E+01, 1.4049E+04, &
      2.1881228E-01, 3.04246E+01, 1.2104E+01, 1.3920E+04, &
      2.1990634E-01, 3.04313E+01, 1.2055E+01, 1.3793E+04, &
      2.2100588E-01, 3.04384E+01, 1.2007E+01, 1.3670E+04, &
      2.2211090E-01, 3.04457E+01, 1.1961E+01, 1.3550E+04, &
      2.2322146E-01, 3.04534E+01, 1.1916E+01, 1.3433E+04, &
      2.2433757E-01, 3.04613E+01, 1.1873E+01, 1.3317E+04, &
      2.2545925E-01, 3.04695E+01, 1.1831E+01, 1.3205E+04, &
      2.2658655E-01, 3.04780E+01, 1.1791E+01, 1.3094E+04, &
      2.2771948E-01, 3.04868E+01, 1.1752E+01, 1.2986E+04, &
      2.2885808E-01, 3.04957E+01, 1.1714E+01, 1.2880E+04, &
      2.3000237E-01, 3.05049E+01, 1.1677E+01, 1.2776E+04, &
      2.3115238E-01, 3.05143E+01, 1.1642E+01, 1.2673E+04, &
      2.3230814E-01, 3.05239E+01, 1.1607E+01, 1.2572E+04, &
      2.3346969E-01, 3.05336E+01, 1.1573E+01, 1.2473E+04, &
      2.3463703E-01, 3.05434E+01, 1.1540E+01, 1.2376E+04, &
      2.3581022E-01, 3.05534E+01, 1.1508E+01, 1.2280E+04, &
      2.3698927E-01, 3.05635E+01, 1.1477E+01, 1.2186E+04, &
      2.3817422E-01, 3.05737E+01, 1.1446E+01, 1.2093E+04, &
      2.3936509E-01, 3.05840E+01, 1.1416E+01, 1.2002E+04, &
      2.4056191E-01, 3.05943E+01, 1.1387E+01, 1.1911E+04, &
      2.4176472E-01, 3.06047E+01, 1.1359E+01, 1.1823E+04, &
      2.4297355E-01, 3.06150E+01, 1.1331E+01, 1.1735E+04, &
      2.4418841E-01, 3.06254E+01, 1.1303E+01, 1.1648E+04, &
      2.4540936E-01, 3.06358E+01, 1.1277E+01, 1.1563E+04, &
      2.4663640E-01, 3.06461E+01, 1.1251E+01, 1.1479E+04, &
      2.4786959E-01, 3.06564E+01, 1.1225E+01, 1.1396E+04, &
      2.4910893E-01, 3.06666E+01, 1.1200E+01, 1.1314E+04, &
      2.5035448E-01, 3.06767E+01, 1.1175E+01, 1.1233E+04, &
      2.5160625E-01, 3.06868E+01, 1.1151E+01, 1.1153E+04, &
      2.5286428E-01, 3.06967E+01, 1.1127E+01, 1.1074E+04, &
      2.5412860E-01, 3.07064E+01, 1.1104E+01, 1.0996E+04, &
      2.5539925E-01, 3.07161E+01, 1.1081E+01, 1.0918E+04, &
      2.5667624E-01, 3.07255E+01, 1.1059E+01, 1.0842E+04, &
      2.5795962E-01, 3.07348E+01, 1.1037E+01, 1.0767E+04, &
      2.5924942E-01, 3.07438E+01, 1.1015E+01, 1.0692E+04, &
      2.6054567E-01, 3.07526E+01, 1.0994E+01, 1.0619E+04, &
      2.6184840E-01, 3.07612E+01, 1.0973E+01, 1.0546E+04, &
      2.6315764E-01, 3.07695E+01, 1.0953E+01, 1.0474E+04, &
      2.6447343E-01, 3.07776E+01, 1.0933E+01, 1.0403E+04, &
      2.6579579E-01, 3.07853E+01, 1.0913E+01, 1.0332E+04, &
      2.6712477E-01, 3.07927E+01, 1.0893E+01, 1.0262E+04, &
      2.6846040E-01, 3.07997E+01, 1.0874E+01, 1.0194E+04, &
      2.6980270E-01, 3.08064E+01, 1.0855E+01, 1.0125E+04, &
      2.7115171E-01, 3.08127E+01, 1.0837E+01, 1.0058E+04, &
      2.7250747E-01, 3.08186E+01, 1.0819E+01, 9.9913E+03, &
      2.7387001E-01, 3.08240E+01, 1.0801E+01, 9.9253E+03, &
      2.7523936E-01, 3.08290E+01, 1.0784E+01, 9.8600E+03, &
      2.7661556E-01, 3.08334E+01, 1.0767E+01, 9.7954E+03, &
      2.7799863E-01, 3.08373E+01, 1.0750E+01, 9.7315E+03, &
      2.7938863E-01, 3.08406E+01, 1.0733E+01, 9.6682E+03, &
      2.8078557E-01, 3.08433E+01, 1.0717E+01, 9.6056E+03, &
      2.8218950E-01, 3.08453E+01, 1.0701E+01, 9.5437E+03, &
      2.8360044E-01, 3.08466E+01, 1.0686E+01, 9.4824E+03, &
      2.8501845E-01, 3.08471E+01, 1.0670E+01, 9.4218E+03, &
      2.8644354E-01, 3.08468E+01, 1.0655E+01, 9.3618E+03, &
      2.8787576E-01, 3.08456E+01, 1.0641E+01, 9.3025E+03, &
      2.8931514E-01, 3.08434E+01, 1.0626E+01, 9.2437E+03, &
      2.9076171E-01, 3.08401E+01, 1.0612E+01, 9.1856E+03, &
      2.9221552E-01, 3.08356E+01, 1.0598E+01, 9.1281E+03, &
      2.9367660E-01, 3.08299E+01, 1.0585E+01, 9.0712E+03, &
      2.9514498E-01, 3.08227E+01, 1.0572E+01, 9.0148E+03, &
      2.9662071E-01, 3.08138E+01, 1.0559E+01, 8.9591E+03, &
      2.9810381E-01, 3.08032E+01, 1.0546E+01, 8.9039E+03, &
      2.9959433E-01, 3.07905E+01, 1.0534E+01, 8.8494E+03, &
      3.0109230E-01, 3.07754E+01, 1.0522E+01, 8.7954E+03, &
      3.0259776E-01, 3.07576E+01, 1.0510E+01, 8.7419E+03, &
      3.0411075E-01, 3.07365E+01, 1.0499E+01, 8.6891E+03, &
      3.0563130E-01, 3.07116E+01, 1.0488E+01, 8.6367E+03, &
      3.0715946E-01, 3.06819E+01, 1.0477E+01, 8.5850E+03, &
      3.0869526E-01, 3.06463E+01, 1.0467E+01, 8.5337E+03, &
      3.1023873E-01, 3.06031E+01, 1.0456E+01, 8.4831E+03, &
      3.1178993E-01, 3.05497E+01, 1.0447E+01, 8.4329E+03, &
      3.1334888E-01, 3.04818E+01, 1.0437E+01, 8.3833E+03, &
      3.1491562E-01, 3.03915E+01, 1.0428E+01, 8.3342E+03, &
      3.1649020E-01, 3.02622E+01, 1.0419E+01, 8.2856E+03, &
      3.1807265E-01, 3.00460E+01, 1.0410E+01, 8.2375E+03, &
      3.1966301E-01, 2.93972E+01, 1.0402E+01, 8.1899E+03, &
      3.2033919E-01, 2.94022E+01, 1.1582E+01, 9.0997E+03, &
      3.2126133E-01, 2.99010E+01, 1.1580E+01, 9.0719E+03, &
      3.2286764E-01, 3.02189E+01, 1.1576E+01, 9.0240E+03, &
      3.2448197E-01, 3.03963E+01, 1.1573E+01, 8.9766E+03, &
      3.2610438E-01, 3.05222E+01, 1.1570E+01, 8.9297E+03, &
      3.2773491E-01, 3.06210E+01, 1.1567E+01, 8.8832E+03, &
      3.2937358E-01, 3.07031E+01, 1.1565E+01, 8.8373E+03, &
      3.3102045E-01, 3.07738E+01, 1.1563E+01, 8.7918E+03, &
      3.3267555E-01, 3.08362E+01, 1.1561E+01, 8.7467E+03, &
      3.3433893E-01, 3.08922E+01, 1.1560E+01, 8.7022E+03, &
      3.3601062E-01, 3.09432E+01, 1.1558E+01, 8.6581E+03, &
      3.3769068E-01, 3.09899E+01, 1.1558E+01, 8.6144E+03, &
      3.3937913E-01, 3.10332E+01, 1.1557E+01, 8.5712E+03, &
      3.4107602E-01, 3.10734E+01, 1.1557E+01, 8.5284E+03, &
      3.4278140E-01, 3.11110E+01, 1.1557E+01, 8.4861E+03, &
      3.4449531E-01, 3.11460E+01, 1.1557E+01, 8.4441E+03, &
      3.4621779E-01, 3.11788E+01, 1.1558E+01, 8.4026E+03, &
      3.4794888E-01, 3.12093E+01, 1.1559E+01, 8.3616E+03, &
      3.4968862E-01, 3.12376E+01, 1.1560E+01, 8.3209E+03, &
      3.5143706E-01, 3.12635E+01, 1.1561E+01, 8.2806E+03, &
      3.5319425E-01, 3.12869E+01, 1.1563E+01, 8.2407E+03, &
      3.5496022E-01, 3.13075E+01, 1.1565E+01, 8.2012E+03, &
      3.5673502E-01, 3.13245E+01, 1.1568E+01, 8.1621E+03, &
      3.5851870E-01, 3.13368E+01, 1.1570E+01, 8.1233E+03, &
      3.6031129E-01, 3.13425E+01, 1.1573E+01, 8.0849E+03, &
      3.6211285E-01, 3.13371E+01, 1.1576E+01, 8.0468E+03, &
      3.6392341E-01, 3.13077E+01, 1.1579E+01, 8.0090E+03, &
      3.6574303E-01, 3.11700E+01, 1.1583E+01, 7.9716E+03, &
      3.6576056E-01, 3.11662E+01, 1.1583E+01, 7.9713E+03, &
      3.6663944E-01, 3.11807E+01, 1.1915E+01, 8.1804E+03, &
      3.6757174E-01, 3.13160E+01, 1.1918E+01, 8.1616E+03, &
      3.6940960E-01, 3.14363E+01, 1.1924E+01, 8.1249E+03, &
      3.7125665E-01, 3.15149E+01, 1.1930E+01, 8.0885E+03, &
      3.7311293E-01, 3.15786E+01, 1.1936E+01, 8.0524E+03, &
      3.7497850E-01, 3.16344E+01, 1.1942E+01, 8.0166E+03, &
      3.7685339E-01, 3.16853E+01, 1.1948E+01, 7.9811E+03, &
      3.7873766E-01, 3.17328E+01, 1.1955E+01, 7.9458E+03, &
      3.8063135E-01, 3.17778E+01, 1.1962E+01, 7.9109E+03, &
      3.8253450E-01, 3.18210E+01, 1.1969E+01, 7.8763E+03, &
      3.8444718E-01, 3.18627E+01, 1.1976E+01, 7.8419E+03, &
      3.8636941E-01, 3.19031E+01, 1.1984E+01, 7.8078E+03, &
      3.8830126E-01, 3.19424E+01, 1.1991E+01, 7.7739E+03, &
      3.9024276E-01, 3.19809E+01, 1.1999E+01, 7.7403E+03, &
      3.9219398E-01, 3.20186E+01, 1.2007E+01, 7.7069E+03, &
      3.9415495E-01, 3.20557E+01, 1.2015E+01, 7.6738E+03, &
      3.9612572E-01, 3.20921E+01, 1.2023E+01, 7.6409E+03, &
      3.9810635E-01, 3.21279E+01, 1.2032E+01, 7.6082E+03, &
      4.0009688E-01, 3.21632E+01, 1.2040E+01, 7.5758E+03, &
      4.0209737E-01, 3.21980E+01, 1.2049E+01, 7.5435E+03, &
      4.0410785E-01, 3.22323E+01, 1.2058E+01, 7.5115E+03, &
      4.0612839E-01, 3.22661E+01, 1.2067E+01, 7.4797E+03, &
      4.0815904E-01, 3.22994E+01, 1.2076E+01, 7.4480E+03, &
      4.1019983E-01, 3.23322E+01, 1.2085E+01, 7.4166E+03, &
      4.1225083E-01, 3.23645E+01, 1.2094E+01, 7.3853E+03, &
      4.1431208E-01, 3.23962E+01, 1.2103E+01, 7.3542E+03, &
      4.1638364E-01, 3.24272E+01, 1.2112E+01, 7.3232E+03, &
      4.1846556E-01, 3.24576E+01, 1.2122E+01, 7.2925E+03, &
      4.2055789E-01, 3.24872E+01, 1.2131E+01, 7.2619E+03, &
      4.2266068E-01, 3.25159E+01, 1.2140E+01, 7.2314E+03, &
      4.2477398E-01, 3.25436E+01, 1.2150E+01, 7.2011E+03, &
      4.2689785E-01, 3.25700E+01, 1.2159E+01, 7.1709E+03, &
      4.2903234E-01, 3.25948E+01, 1.2169E+01, 7.1408E+03, &
      4.3117750E-01, 3.26178E+01, 1.2178E+01, 7.1109E+03, &
      4.3333339E-01, 3.26384E+01, 1.2188E+01, 7.0811E+03, &
      4.3550006E-01, 3.26558E+01, 1.2197E+01, 7.0514E+03, &
      4.3767756E-01, 3.26689E+01, 1.2207E+01, 7.0218E+03, &
      4.3986595E-01, 3.26758E+01, 1.2216E+01, 6.9923E+03, &
      4.4206528E-01, 3.26730E+01, 1.2226E+01, 6.9629E+03, &
      4.4427560E-01, 3.26529E+01, 1.2235E+01, 6.9336E+03, &
      4.4649698E-01, 3.25932E+01, 1.2244E+01, 6.9044E+03, &
      4.4843083E-01, 3.24053E+01, 1.2252E+01, 6.8792E+03, &
      4.4872947E-01, 3.23132E+01, 1.2254E+01, 6.8753E+03, &
      4.4976915E-01, 3.24302E+01, 1.2777E+01, 7.1521E+03, &
      4.5097311E-01, 3.26227E+01, 1.2782E+01, 7.1361E+03, &
      4.5322798E-01, 3.27958E+01, 1.2793E+01, 7.1064E+03, &
      4.5549412E-01, 3.29111E+01, 1.2803E+01, 7.0767E+03, &
      4.5777159E-01, 3.30047E+01, 1.2813E+01, 7.0470E+03, &
      4.6006045E-01, 3.30871E+01, 1.2823E+01, 7.0175E+03, &
      4.6236075E-01, 3.31625E+01, 1.2833E+01, 6.9880E+03, &
      4.6467255E-01, 3.32332E+01, 1.2842E+01, 6.9585E+03, &
      4.6699592E-01, 3.33006E+01, 1.2852E+01, 6.9291E+03, &
      4.6933090E-01, 3.33656E+01, 1.2861E+01, 6.8997E+03, &
      4.7167755E-01, 3.34287E+01, 1.2871E+01, 6.8704E+03, &
      4.7403594E-01, 3.34903E+01, 1.2880E+01, 6.8411E+03, &
      4.7640612E-01, 3.35507E+01, 1.2889E+01, 6.8119E+03, &
      4.7878815E-01, 3.36101E+01, 1.2897E+01, 6.7827E+03, &
      4.8118209E-01, 3.36688E+01, 1.2906E+01, 6.7535E+03, &
      4.8358800E-01, 3.37269E+01, 1.2914E+01, 6.7243E+03, &
      4.8600594E-01, 3.37844E+01, 1.2923E+01, 6.6952E+03, &
      4.8843597E-01, 3.38415E+01, 1.2931E+01, 6.6661E+03, &
      4.9087815E-01, 3.38983E+01, 1.2939E+01, 6.6369E+03, &
      4.9333254E-01, 3.39547E+01, 1.2946E+01, 6.6078E+03, &
      4.9579920E-01, 3.40109E+01, 1.2953E+01, 6.5787E+03, &
      4.9827820E-01, 3.40669E+01, 1.2961E+01, 6.5496E+03, &
      5.0076959E-01, 3.41227E+01, 1.2967E+01, 6.5205E+03, &
      5.0327344E-01, 3.41920E+01, 1.2974E+01, 6.4914E+03, &
      5.0578980E-01, 3.42476E+01, 1.2980E+01, 6.4623E+03, &
      5.0831875E-01, 3.43031E+01, 1.2986E+01, 6.4331E+03, &
      5.1086035E-01, 3.43584E+01, 1.2992E+01, 6.4039E+03, &
      5.1341465E-01, 3.44137E+01, 1.2997E+01, 6.3747E+03, &
      5.1598172E-01, 3.44689E+01, 1.3002E+01, 6.3455E+03, &
      5.1856163E-01, 3.45240E+01, 1.3007E+01, 6.3163E+03, &
      5.2115444E-01, 3.45790E+01, 1.3011E+01, 6.2870E+03, &
      5.2376021E-01, 3.46340E+01, 1.3015E+01, 6.2577E+03, &
      5.2637901E-01, 3.46890E+01, 1.3019E+01, 6.2284E+03, &
      5.2901091E-01, 3.47439E+01, 1.3022E+01, 6.1990E+03, &
      5.3165596E-01, 3.48116E+01, 1.3025E+01, 6.1697E+03, &
      5.3431424E-01, 3.48665E+01, 1.3028E+01, 6.1403E+03, &
      5.3698581E-01, 3.49213E+01, 1.3030E+01, 6.1109E+03, &
      5.3967074E-01, 3.49760E+01, 1.3032E+01, 6.0814E+03, &
      5.4236910E-01, 3.50307E+01, 1.3033E+01, 6.0519E+03, &
      5.4508094E-01, 3.50853E+01, 1.3035E+01, 6.0224E+03, &
      5.4780635E-01, 3.51399E+01, 1.3035E+01, 5.9929E+03, &
      5.5054538E-01, 3.51944E+01, 1.3036E+01, 5.9633E+03, &
      5.5329810E-01, 3.52489E+01, 1.3036E+01, 5.9337E+03, &
      5.5606460E-01, 3.53032E+01, 1.3035E+01, 5.9040E+03, &
      5.5884492E-01, 3.53575E+01, 1.3034E+01, 5.8743E+03, &
      5.6163914E-01, 3.54118E+01, 1.3033E+01, 5.8445E+03, &
      5.6444734E-01, 3.54659E+01, 1.3031E+01, 5.8148E+03, &
      5.6726958E-01, 3.55200E+01, 1.3029E+01, 5.7850E+03, &
      5.7010592E-01, 3.55739E+01, 1.3027E+01, 5.7552E+03, &
      5.7295645E-01, 3.56278E+01, 1.3024E+01, 5.7253E+03, &
      5.7582123E-01, 3.56815E+01, 1.3021E+01, 5.6954E+03, &
      5.7870034E-01, 3.57351E+01, 1.3017E+01, 5.6656E+03, &
      5.8159384E-01, 3.57886E+01, 1.3013E+01, 5.6356E+03, &
      5.8450181E-01, 3.58419E+01, 1.3008E+01, 5.6057E+03, &
      5.8742432E-01, 3.58951E+01, 1.3003E+01, 5.5757E+03, &
      5.9036144E-01, 3.59482E+01, 1.2998E+01, 5.5458E+03, &
      5.9331325E-01, 3.60011E+01, 1.2992E+01, 5.5158E+03, &
      5.9627982E-01, 3.60539E+01, 1.2986E+01, 5.4857E+03, &
      5.9926122E-01, 3.61064E+01, 1.2979E+01, 5.4557E+03, &
      6.0225752E-01, 3.61588E+01, 1.2972E+01, 5.4257E+03, &
      6.0526881E-01, 3.62110E+01, 1.2965E+01, 5.3956E+03, &
      6.0829515E-01, 3.62630E+01, 1.2957E+01, 5.3655E+03, &
      6.1133663E-01, 3.63148E+01, 1.2948E+01, 5.3354E+03, &
      6.1439331E-01, 3.63664E+01, 1.2940E+01, 5.3053E+03, &
      6.1746528E-01, 3.64178E+01, 1.2930E+01, 5.2752E+03, &
      6.2055260E-01, 3.64689E+01, 1.2921E+01, 5.2451E+03, &
      6.2365537E-01, 3.65198E+01, 1.2911E+01, 5.2150E+03, &
      6.2677364E-01, 3.65705E+01, 1.2900E+01, 5.1849E+03, &
      6.2990751E-01, 3.66209E+01, 1.2889E+01, 5.1548E+03, &
      6.3305705E-01, 3.66710E+01, 1.2878E+01, 5.1247E+03, &
      6.3622234E-01, 3.67208E+01, 1.2866E+01, 5.0946E+03, &
      6.3940345E-01, 3.67704E+01, 1.2854E+01, 5.0645E+03, &
      6.4260046E-01, 3.68197E+01, 1.2841E+01, 5.0344E+03, &
      6.4581347E-01, 3.68687E+01, 1.2828E+01, 5.0044E+03, &
      6.4904253E-01, 3.69173E+01, 1.2815E+01, 4.9743E+03, &
      6.5228775E-01, 3.69657E+01, 1.2801E+01, 4.9443E+03, &
      6.5554919E-01, 3.70137E+01, 1.2786E+01, 4.9142E+03, &
      6.5882693E-01, 3.70614E+01, 1.2772E+01, 4.8842E+03, &
      6.6212107E-01, 3.71088E+01, 1.2757E+01, 4.8542E+03, &
      6.6543167E-01, 3.71558E+01, 1.2741E+01, 4.8243E+03, &
      6.6875883E-01, 3.72024E+01, 1.2725E+01, 4.7943E+03, &
      6.7210262E-01, 3.72487E+01, 1.2709E+01, 4.7644E+03, &
      6.7546314E-01, 3.72946E+01, 1.2692E+01, 4.7345E+03, &
      6.7884045E-01, 3.73401E+01, 1.2675E+01, 4.7046E+03, &
      6.8223466E-01, 3.73852E+01, 1.2657E+01, 4.6748E+03, &
      6.8564583E-01, 3.74298E+01, 1.2639E+01, 4.6449E+03, &
      6.8907406E-01, 3.74741E+01, 1.2620E+01, 4.6151E+03, &
      6.9251943E-01, 3.75178E+01, 1.2601E+01, 4.5853E+03, &
      6.9598202E-01, 3.75612E+01, 1.2582E+01, 4.5555E+03, &
      6.9946194E-01, 3.76040E+01, 1.2562E+01, 4.5258E+03, &
      7.0295924E-01, 3.76464E+01, 1.2542E+01, 4.4960E+03, &
      7.0647404E-01, 3.76883E+01, 1.2521E+01, 4.4664E+03, &
      7.1000641E-01, 3.77297E+01, 1.2500E+01, 4.4367E+03, &
      7.1355644E-01, 3.77705E+01, 1.2478E+01, 4.4071E+03, &
      7.1712423E-01, 3.78108E+01, 1.2457E+01, 4.3776E+03, &
      7.2070985E-01, 3.78506E+01, 1.2434E+01, 4.3481E+03, &
      7.2431340E-01, 3.78898E+01, 1.2411E+01, 4.3186E+03, &
      7.2793496E-01, 3.79284E+01, 1.2388E+01, 4.2892E+03, &
      7.3157464E-01, 3.79665E+01, 1.2365E+01, 4.2598E+03, &
      7.3523251E-01, 3.80039E+01, 1.2341E+01, 4.2305E+03, &
      7.3890867E-01, 3.80408E+01, 1.2317E+01, 4.2012E+03, &
      7.4260322E-01, 3.80770E+01, 1.2292E+01, 4.1720E+03, &
      7.4631623E-01, 3.81126E+01, 1.2267E+01, 4.1429E+03, &
      7.5004781E-01, 3.81476E+01, 1.2242E+01, 4.1139E+03, &
      7.5379805E-01, 3.81819E+01, 1.2216E+01, 4.0849E+03, &
      7.5756704E-01, 3.82156E+01, 1.2190E+01, 4.0560E+03, &
      7.6135488E-01, 3.82485E+01, 1.2163E+01, 4.0271E+03, &
      7.6516165E-01, 3.82808E+01, 1.2136E+01, 3.9982E+03, &
      7.6898746E-01, 3.83123E+01, 1.2109E+01, 3.9695E+03, &
      7.7283240E-01, 3.83430E+01, 1.2081E+01, 3.9408E+03, &
      7.7669656E-01, 3.83730E+01, 1.2053E+01, 3.9122E+03, &
      7.8058004E-01, 3.84022E+01, 1.2025E+01, 3.8837E+03, &
      7.8448294E-01, 3.84306E+01, 1.1997E+01, 3.8553E+03, &
      7.8840536E-01, 3.84582E+01, 1.1968E+01, 3.8270E+03, &
      7.9234738E-01, 3.84850E+01, 1.1939E+01, 3.7988E+03, &
      7.9630912E-01, 3.85111E+01, 1.1910E+01, 3.7707E+03, &
      8.0029067E-01, 3.85363E+01, 1.1880E+01, 3.7428E+03, &
      8.0429212E-01, 3.85606E+01, 1.1850E+01, 3.7149E+03, &
      8.0831358E-01, 3.85842E+01, 1.1821E+01, 3.6872E+03, &
      8.1235515E-01, 3.86069E+01, 1.1790E+01, 3.6595E+03, &
      8.1641693E-01, 3.86287E+01, 1.1760E+01, 3.6320E+03, &
      8.2049901E-01, 3.86497E+01, 1.1729E+01, 3.6047E+03, &
      8.2460150E-01, 3.86698E+01, 1.1699E+01, 3.5774E+03, &
      8.2872451E-01, 3.86890E+01, 1.1668E+01, 3.5503E+03, &
      8.3286813E-01, 3.87073E+01, 1.1637E+01, 3.5233E+03, &
      8.3703248E-01, 3.87247E+01, 1.1605E+01, 3.4964E+03, &
      8.4121764E-01, 3.87412E+01, 1.1574E+01, 3.4696E+03, &
      8.4542373E-01, 3.87567E+01, 1.1542E+01, 3.4430E+03, &
      8.4965084E-01, 3.87713E+01, 1.1511E+01, 3.4166E+03, &
      8.5389910E-01, 3.87850E+01, 1.1479E+01, 3.3902E+03, &
      8.5816859E-01, 3.87977E+01, 1.1447E+01, 3.3640E+03, &
      8.6245944E-01, 3.88094E+01, 1.1414E+01, 3.3380E+03, &
      8.6677173E-01, 3.88201E+01, 1.1382E+01, 3.3121E+03, &
      8.7110559E-01, 3.88298E+01, 1.1350E+01, 3.2863E+03, &
      8.7546112E-01, 3.88386E+01, 1.1317E+01, 3.2607E+03, &
      8.7983843E-01, 3.88463E+01, 1.1285E+01, 3.2352E+03, &
      8.8423762E-01, 3.88529E+01, 1.1252E+01, 3.2099E+03, &
      8.8865881E-01, 3.88585E+01, 1.1219E+01, 3.1847E+03, &
      8.9310210E-01, 3.88631E+01, 1.1186E+01, 3.1597E+03, &
      8.9756761E-01, 3.88665E+01, 1.1154E+01, 3.1348E+03, &
      9.0205545E-01, 3.88689E+01, 1.1121E+01, 3.1100E+03, &
      9.0656573E-01, 3.88702E+01, 1.1088E+01, 3.0855E+03, &
      9.1109856E-01, 3.88703E+01, 1.1054E+01, 3.0610E+03, &
      9.1565405E-01, 3.88693E+01, 1.1021E+01, 3.0368E+03, &
      9.2023232E-01, 3.88672E+01, 1.0988E+01, 3.0126E+03, &
      9.2483348E-01, 3.88639E+01, 1.0955E+01, 2.9887E+03, &
      9.2945765E-01, 3.88594E+01, 1.0922E+01, 2.9648E+03, &
      9.3410494E-01, 3.88537E+01, 1.0888E+01, 2.9412E+03, &
      9.3877546E-01, 3.88467E+01, 1.0855E+01, 2.9177E+03, &
      9.4346934E-01, 3.88386E+01, 1.0822E+01, 2.8943E+03, &
      9.4818668E-01, 3.88291E+01, 1.0788E+01, 2.8712E+03, &
      9.5292762E-01, 3.88184E+01, 1.0755E+01, 2.8481E+03, &
      9.5769226E-01, 3.88064E+01, 1.0722E+01, 2.8253E+03, &
      9.6248072E-01, 3.87929E+01, 1.0688E+01, 2.8025E+03, &
      9.6729312E-01, 3.87782E+01, 1.0655E+01, 2.7800E+03, &
      9.7212959E-01, 3.87622E+01, 1.0621E+01, 2.7576E+03, &
      9.7699023E-01, 3.87448E+01, 1.0588E+01, 2.7353E+03, &
      9.8187519E-01, 3.87260E+01, 1.0555E+01, 2.7133E+03, &
      9.8678456E-01, 3.87057E+01, 1.0522E+01, 2.6913E+03, &
      9.9171848E-01, 3.86840E+01, 1.0488E+01, 2.6696E+03, &
      9.9667708E-01, 3.86607E+01, 1.0455E+01, 2.6480E+03, &
      1.0016605E+00, 3.86781E+01, 1.0412E+01, 2.6241E+03, &
      1.0066688E+00, 3.87605E+01, 1.0350E+01, 2.5955E+03, &
      1.0117021E+00, 3.88201E+01, 1.0288E+01, 2.5673E+03, &
      1.0167606E+00, 3.88620E+01, 1.0227E+01, 2.5395E+03, &
      1.0218444E+00, 3.88895E+01, 1.0166E+01, 2.5119E+03, &
      1.0269536E+00, 3.89049E+01, 1.0106E+01, 2.4847E+03, &
      1.0320884E+00, 3.89099E+01, 1.0046E+01, 2.4578E+03, &
      1.0372489E+00, 3.89060E+01, 9.9865E+00, 2.4312E+03, &
      1.0424351E+00, 3.88941E+01, 9.9274E+00, 2.4049E+03, &
      1.0476473E+00, 3.88750E+01, 9.8688E+00, 2.3789E+03, &
      1.0528855E+00, 3.88493E+01, 9.8106E+00, 2.3532E+03, &
      1.0581499E+00, 3.88175E+01, 9.7528E+00, 2.3279E+03, &
      1.0634407E+00, 3.87800E+01, 9.6955E+00, 2.3028E+03, &
      1.0687579E+00, 3.87372E+01, 9.6386E+00, 2.2780E+03, &
      1.0741017E+00, 3.86892E+01, 9.5820E+00, 2.2535E+03, &
      1.0794722E+00, 3.86362E+01, 9.5259E+00, 2.2292E+03, &
      1.0848695E+00, 3.85784E+01, 9.4702E+00, 2.2053E+03, &
      1.0902939E+00, 3.85159E+01, 9.4149E+00, 2.1816E+03, &
      1.0957454E+00, 3.84487E+01, 9.3600E+00, 2.1582E+03, &
      1.1012241E+00, 3.83780E+01, 9.3055E+00, 2.1351E+03, &
      1.1067302E+00, 3.83015E+01, 9.2514E+00, 2.1122E+03, &
      1.1122639E+00, 3.82205E+01, 9.1977E+00, 2.0896E+03, &
      1.1178252E+00, 3.81347E+01, 9.1444E+00, 2.0673E+03, &
      1.1234143E+00, 3.80443E+01, 9.0914E+00, 2.0452E+03, &
      1.1290314E+00, 3.79490E+01, 9.0388E+00, 2.0234E+03, &
      1.1346765E+00, 3.78488E+01, 8.9866E+00, 2.0018E+03, &
      1.1403499E+00, 3.77437E+01, 8.9347E+00, 1.9804E+03, &
      1.1460517E+00, 3.76333E+01, 8.8832E+00, 1.9593E+03, &
      1.1517819E+00, 3.75177E+01, 8.8321E+00, 1.9385E+03, &
      1.1575408E+00, 3.73966E+01, 8.7813E+00, 1.9179E+03, &
      1.1633285E+00, 3.72698E+01, 8.7309E+00, 1.8975E+03, &
      1.1691452E+00, 3.71371E+01, 8.6809E+00, 1.8773E+03, &
      1.1749909E+00, 3.69982E+01, 8.6312E+00, 1.8574E+03, &
      1.1808659E+00, 3.68529E+01, 8.5819E+00, 1.8377E+03, &
      1.1867702E+00, 3.67010E+01, 8.5329E+00, 1.8182E+03, &
      1.1927040E+00, 3.65419E+01, 8.4843E+00, 1.7990E+03, &
      1.1986676E+00, 3.63755E+01, 8.4360E+00, 1.7800E+03, &
      1.2046609E+00, 3.62012E+01, 8.3881E+00, 1.7612E+03, &
      1.2106842E+00, 3.60187E+01, 8.3405E+00, 1.7426E+03, &
      1.2167376E+00, 3.58274E+01, 8.2933E+00, 1.7242E+03, &
      1.2228213E+00, 3.56267E+01, 8.2464E+00, 1.7060E+03, &
      1.2289354E+00, 3.54162E+01, 8.1998E+00, 1.6881E+03, &
      1.2350801E+00, 3.51950E+01, 8.1536E+00, 1.6703E+03, &
      1.2412555E+00, 3.49624E+01, 8.1076E+00, 1.6527E+03, &
      1.2474618E+00, 3.47177E+01, 8.0621E+00, 1.6354E+03, &
      1.2536991E+00, 3.44597E+01, 8.0168E+00, 1.6182E+03, &
      1.2599676E+00, 3.41875E+01, 7.9719E+00, 1.6013E+03, &
      1.2662674E+00, 3.38997E+01, 7.9272E+00, 1.5845E+03, &
      1.2725988E+00, 3.35950E+01, 7.8829E+00, 1.5679E+03, &
      1.2789618E+00, 3.32718E+01, 7.8390E+00, 1.5515E+03, &
      1.2853566E+00, 3.29281E+01, 7.7938E+00, 1.5350E+03, &
      1.2917833E+00, 3.25616E+01, 7.7486E+00, 1.5186E+03, &
      1.2982423E+00, 3.21700E+01, 7.7038E+00, 1.5024E+03, &
      1.3047335E+00, 3.17500E+01, 7.6593E+00, 1.4864E+03, &
      1.3112571E+00, 3.12982E+01, 7.6151E+00, 1.4706E+03, &
      1.3178134E+00, 3.08101E+01, 7.5713E+00, 1.4550E+03, &
      1.3244025E+00, 3.02806E+01, 7.5277E+00, 1.4395E+03, &
      1.3310245E+00, 2.97029E+01, 7.4845E+00, 1.4243E+03, &
      1.3376796E+00, 2.90689E+01, 7.4416E+00, 1.4092E+03, &
      1.3443680E+00, 2.83678E+01, 7.3990E+00, 1.3942E+03, &
      1.3510899E+00, 2.75859E+01, 7.3567E+00, 1.3795E+03, &
      1.3578453E+00, 2.67029E+01, 7.3147E+00, 1.3649E+03, &
      1.3646345E+00, 2.56921E+01, 7.2730E+00, 1.3505E+03, &
      1.3714577E+00, 2.45128E+01, 7.2316E+00, 1.3362E+03, &
      1.3783150E+00, 2.31005E+01, 7.1905E+00, 1.3221E+03, &
      1.3852066E+00, 2.13426E+01, 7.1496E+00, 1.3082E+03, &
      1.3921326E+00, 1.90125E+01, 7.1091E+00, 1.2944E+03, &
      1.3990933E+00, 1.55226E+01, 7.0689E+00, 1.2808E+03, &
      1.4060887E+00, 8.03916E+00, 7.0289E+00, 1.2673E+03, &
      1.4090970E+00, -9.27823E+00, 7.0119E+00, 1.2616E+03, &
      1.4095029E+00, -9.62017E+00, 2.6576E+01, 4.7533E+03, &
      1.4131192E+00, 8.49489E+00, 2.6471E+01, 4.7224E+03, &
      1.4201848E+00, 1.45127E+01, 2.6268E+01, 4.6630E+03, &
      1.4272857E+00, 1.69061E+01, 2.6066E+01, 4.6043E+03, &
      1.4344221E+00, 1.79144E+01, 2.5866E+01, 4.5464E+03, &
      1.4415942E+00, 1.77593E+01, 2.5668E+01, 4.4892E+03, &
      1.4488022E+00, 1.53518E+01, 2.5471E+01, 4.4328E+03, &
      1.4529192E+00, 6.05814E+00, 2.5360E+01, 4.4010E+03, &
      1.4536808E+00, 5.98292E+00, 3.8139E+01, 6.6104E+03, &
      1.4560462E+00, 1.44499E+01, 3.8042E+01, 6.5829E+03, &
      1.4633265E+00, 2.08507E+01, 3.7746E+01, 6.4992E+03, &
      1.4706431E+00, 2.40594E+01, 3.7451E+01, 6.4165E+03, &
      1.4779963E+00, 2.63660E+01, 3.7158E+01, 6.3348E+03, &
      1.4853863E+00, 2.82119E+01, 3.6868E+01, 6.2542E+03, &
      1.4928132E+00, 2.97680E+01, 3.6580E+01, 6.1746E+03, &
      1.5002773E+00, 3.11205E+01, 3.6295E+01, 6.0961E+03, &
      1.5077787E+00, 3.23200E+01, 3.6011E+01, 6.0185E+03, &
      1.5153176E+00, 3.33990E+01, 3.5730E+01, 5.9420E+03, &
      1.5228942E+00, 3.43800E+01, 3.5452E+01, 5.8665E+03, &
      1.5305086E+00, 3.52792E+01, 3.5175E+01, 5.7919E+03, &
      1.5381612E+00, 3.61088E+01, 3.4901E+01, 5.7183E+03, &
      1.5458520E+00, 3.68781E+01, 3.4629E+01, 5.6457E+03, &
      1.5535812E+00, 3.75947E+01, 3.4360E+01, 5.5739E+03, &
      1.5613491E+00, 3.82644E+01, 3.4092E+01, 5.5032E+03, &
      1.5691559E+00, 3.88921E+01, 3.3827E+01, 5.4333E+03, &
      1.5770017E+00, 3.94818E+01, 3.3564E+01, 5.3643E+03, &
      1.5848867E+00, 4.00370E+01, 3.3302E+01, 5.2962E+03, &
      1.5928111E+00, 4.05605E+01, 3.3043E+01, 5.2290E+03, &
      1.6007752E+00, 4.10546E+01, 3.2787E+01, 5.1627E+03, &
      1.6087790E+00, 4.15215E+01, 3.2532E+01, 5.0972E+03, &
      1.6168229E+00, 4.19628E+01, 3.2279E+01, 5.0326E+03, &
      1.6249070E+00, 4.23800E+01, 3.2028E+01, 4.9688E+03, &
      1.6330316E+00, 4.27745E+01, 3.1780E+01, 4.9058E+03, &
      1.6411967E+00, 4.31472E+01, 3.1533E+01, 4.8436E+03, &
      1.6494027E+00, 4.34990E+01, 3.1288E+01, 4.7823E+03, &
      1.6576497E+00, 4.38306E+01, 3.1046E+01, 4.7217E+03, &
      1.6659380E+00, 4.41425E+01, 3.0805E+01, 4.6619E+03, &
      1.6742677E+00, 4.44350E+01, 3.0566E+01, 4.6029E+03, &
      1.6826390E+00, 4.47083E+01, 3.0329E+01, 4.5446E+03, &
      1.6910522E+00, 4.49622E+01, 3.0095E+01, 4.4871E+03, &
      1.6995075E+00, 4.51965E+01, 2.9862E+01, 4.4303E+03, &
      1.7080050E+00, 4.54105E+01, 2.9630E+01, 4.3743E+03, &
      1.7165450E+00, 4.56033E+01, 2.9401E+01, 4.3190E+03, &
      1.7251278E+00, 4.57732E+01, 2.9174E+01, 4.2644E+03, &
      1.7337534E+00, 4.59182E+01, 2.8948E+01, 4.2105E+03, &
      1.7424222E+00, 4.60350E+01, 2.8724E+01, 4.1573E+03, &
      1.7511343E+00, 4.61189E+01, 2.8502E+01, 4.1047E+03, &
      1.7598899E+00, 4.61625E+01, 2.8282E+01, 4.0529E+03, &
      1.7686894E+00, 4.61543E+01, 2.8064E+01, 4.0017E+03, &
      1.7775328E+00, 4.60738E+01, 2.7847E+01, 3.9512E+03, &
      1.7864205E+00, 4.58818E+01, 2.7632E+01, 3.9013E+03, &
      1.7953526E+00, 4.54850E+01, 2.7419E+01, 3.8521E+03, &
      1.8043294E+00, 4.45513E+01, 2.7207E+01, 3.8034E+03, &
      1.8098795E+00, 4.26606E+01, 2.7078E+01, 3.7738E+03, &
      1.8133510E+00, 4.23695E+01, 3.1752E+01, 4.4150E+03, &
      1.8137205E+00, 4.27054E+01, 3.1741E+01, 4.4126E+03, &
      1.8224178E+00, 4.55466E+01, 3.1481E+01, 4.3558E+03, &
      1.8315299E+00, 4.67160E+01, 3.1213E+01, 4.2973E+03, &
      1.8406875E+00, 4.75047E+01, 3.0947E+01, 4.2396E+03, &
      1.8498909E+00, 4.81184E+01, 3.0683E+01, 4.1827E+03, &
      1.8591404E+00, 4.86274E+01, 3.0421E+01, 4.1265E+03, &
      1.8684361E+00, 4.90644E+01, 3.0162E+01, 4.0712E+03, &
      1.8777783E+00, 4.94466E+01, 2.9906E+01, 4.0165E+03, &
      1.8871672E+00, 4.97839E+01, 2.9651E+01, 3.9627E+03, &
      1.8966030E+00, 5.00811E+01, 2.9398E+01, 3.9094E+03, &
      1.9060860E+00, 5.03408E+01, 2.9160E+01, 3.8587E+03, &
      1.9156165E+00, 5.05784E+01, 2.8938E+01, 3.8103E+03, &
      1.9251945E+00, 5.07951E+01, 2.8718E+01, 3.7627E+03, &
      1.9348205E+00, 5.09898E+01, 2.8502E+01, 3.7159E+03, &
      1.9444946E+00, 5.11607E+01, 2.8289E+01, 3.6699E+03, &
      1.9542171E+00, 5.13050E+01, 2.8079E+01, 3.6246E+03, &
      1.9639882E+00, 5.14170E+01, 2.7871E+01, 3.5800E+03, &
      1.9738081E+00, 5.14853E+01, 2.7665E+01, 3.5360E+03, &
      1.9836772E+00, 5.14844E+01, 2.7463E+01, 3.4928E+03, &
      1.9935955E+00, 5.13397E+01, 2.7263E+01, 3.4502E+03, &
      2.0035635E+00, 5.05184E+01, 2.7065E+01, 3.4083E+03, &
      2.0036337E+00, 5.05004E+01, 2.7064E+01, 3.4080E+03, &
      2.0079663E+00, 5.05651E+01, 2.8901E+01, 3.6309E+03, &
      2.0135813E+00, 5.14684E+01, 2.8777E+01, 3.6053E+03, &
      2.0236492E+00, 5.21751E+01, 2.8558E+01, 3.5602E+03, &
      2.0337675E+00, 5.26349E+01, 2.8342E+01, 3.5157E+03, &
      2.0439363E+00, 5.29986E+01, 2.8128E+01, 3.4720E+03, &
      2.0541560E+00, 5.33076E+01, 2.7917E+01, 3.4289E+03, &
      2.0644268E+00, 5.35797E+01, 2.7709E+01, 3.3865E+03, &
      2.0747489E+00, 5.38235E+01, 2.7502E+01, 3.3447E+03, &
      2.0851227E+00, 5.40442E+01, 2.7298E+01, 3.3035E+03, &
      2.0955483E+00, 5.42441E+01, 2.7097E+01, 3.2629E+03, &
      2.1060260E+00, 5.44238E+01, 2.6899E+01, 3.2231E+03, &
      2.1165562E+00, 5.45884E+01, 2.6715E+01, 3.1851E+03, &
      2.1271389E+00, 5.47410E+01, 2.6533E+01, 3.1478E+03, &
      2.1377746E+00, 5.48801E+01, 2.6353E+01, 3.1111E+03, &
      2.1484635E+00, 5.50036E+01, 2.6177E+01, 3.0750E+03, &
      2.1592058E+00, 5.51081E+01, 2.6002E+01, 3.0394E+03, &
      2.1700018E+00, 5.51871E+01, 2.5830E+01, 3.0043E+03, &
      2.1808519E+00, 5.52263E+01, 2.5660E+01, 2.9698E+03, &
      2.1917561E+00, 5.51860E+01, 2.5492E+01, 2.9358E+03, &
      2.2022856E+00, 5.48733E+01, 2.5332E+01, 2.9035E+03, &
      2.2027149E+00, 5.48392E+01, 2.5326E+01, 2.9022E+03, &
      2.2107145E+00, 5.49879E+01, 2.6387E+01, 3.0126E+03, &
      2.2137285E+00, 5.52349E+01, 2.6341E+01, 3.0032E+03, &
      2.2247971E+00, 5.57415E+01, 2.6174E+01, 2.9694E+03, &
      2.2359211E+00, 5.60726E+01, 2.6008E+01, 2.9360E+03, &
      2.2471007E+00, 5.63423E+01, 2.5844E+01, 2.9031E+03, &
      2.2583362E+00, 5.65792E+01, 2.5682E+01, 2.8706E+03, &
      2.2696279E+00, 5.67951E+01, 2.5521E+01, 2.8386E+03, &
      2.2809760E+00, 5.69959E+01, 2.5362E+01, 2.8069E+03, &
      2.2923809E+00, 5.71853E+01, 2.5204E+01, 2.7757E+03, &
      2.3038428E+00, 5.73655E+01, 2.5048E+01, 2.7448E+03, &
      2.3153620E+00, 5.75378E+01, 2.4893E+01, 2.7144E+03, &
      2.3269388E+00, 5.77040E+01, 2.4741E+01, 2.6845E+03, &
      2.3385735E+00, 5.78655E+01, 2.4591E+01, 2.6550E+03, &
      2.3502664E+00, 5.80232E+01, 2.4442E+01, 2.6259E+03, &
      2.3620177E+00, 5.81770E+01, 2.4290E+01, 2.5967E+03, &
      2.3738278E+00, 5.83257E+01, 2.4137E+01, 2.5676E+03, &
      2.3856970E+00, 5.84699E+01, 2.3984E+01, 2.5388E+03, &
      2.3976254E+00, 5.86099E+01, 2.3833E+01, 2.5103E+03, &
      2.4096136E+00, 5.87462E+01, 2.3683E+01, 2.4822E+03, &
      2.4216616E+00, 5.88791E+01, 2.3535E+01, 2.4545E+03, &
      2.4337699E+00, 5.90087E+01, 2.3387E+01, 2.4270E+03, &
      2.4459388E+00, 5.91353E+01, 2.3240E+01, 2.3999E+03, &
      2.4581685E+00, 5.92591E+01, 2.3094E+01, 2.3731E+03, &
      2.4704593E+00, 5.93802E+01, 2.2950E+01, 2.3466E+03, &
      2.4828116E+00, 5.94988E+01, 2.2806E+01, 2.3204E+03, &
      2.4952257E+00, 5.96151E+01, 2.2663E+01, 2.2944E+03, &
      2.5077018E+00, 5.97291E+01, 2.2521E+01, 2.2688E+03, &
      2.5202403E+00, 5.98409E+01, 2.2380E+01, 2.2435E+03, &
      2.5328415E+00, 5.99507E+01, 2.2240E+01, 2.2185E+03, &
      2.5455057E+00, 6.00586E+01, 2.2101E+01, 2.1937E+03, &
      2.5582333E+00, 6.01646E+01, 2.1962E+01, 2.1692E+03, &
      2.5710244E+00, 6.02689E+01, 2.1825E+01, 2.1450E+03, &
      2.5838796E+00, 6.03715E+01, 2.1688E+01, 2.1210E+03, &
      2.5967990E+00, 6.04727E+01, 2.1552E+01, 2.0973E+03, &
      2.6097829E+00, 6.05725E+01, 2.1416E+01, 2.0739E+03, &
      2.6228319E+00, 6.06703E+01, 2.1279E+01, 2.0504E+03, &
      2.6359460E+00, 6.07660E+01, 2.1143E+01, 2.0273E+03, &
      2.6491257E+00, 6.08597E+01, 2.1007E+01, 2.0043E+03, &
      2.6623714E+00, 6.09517E+01, 2.0873E+01, 1.9817E+03, &
      2.6756832E+00, 6.10419E+01, 2.0739E+01, 1.9593E+03, &
      2.6890617E+00, 6.11305E+01, 2.0606E+01, 1.9371E+03, &
      2.7025070E+00, 6.12175E+01, 2.0474E+01, 1.9152E+03, &
      2.7160195E+00, 6.13030E+01, 2.0342E+01, 1.8935E+03, &
      2.7295996E+00, 6.13870E+01, 2.0212E+01, 1.8721E+03, &
      2.7432476E+00, 6.14697E+01, 2.0082E+01, 1.8509E+03, &
      2.7569638E+00, 6.15510E+01, 1.9953E+01, 1.8300E+03, &
      2.7707486E+00, 6.16310E+01, 1.9825E+01, 1.8093E+03, &
      2.7846024E+00, 6.17099E+01, 1.9697E+01, 1.7888E+03, &
      2.7985254E+00, 6.17876E+01, 1.9571E+01, 1.7685E+03, &
      2.8125180E+00, 6.18643E+01, 1.9445E+01, 1.7485E+03, &
      2.8265806E+00, 6.21510E+01, 1.9318E+01, 1.7286E+03, &
      2.8407135E+00, 6.22255E+01, 1.9191E+01, 1.7087E+03, &
      2.8549171E+00, 6.22989E+01, 1.9064E+01, 1.6891E+03, &
      2.8691917E+00, 6.23712E+01, 1.8939E+01, 1.6697E+03, &
      2.8835376E+00, 6.24424E+01, 1.8814E+01, 1.6505E+03, &
      2.8979553E+00, 6.25128E+01, 1.8689E+01, 1.6315E+03, &
      2.9124451E+00, 6.27315E+01, 1.8565E+01, 1.6127E+03, &
      2.9270073E+00, 6.28009E+01, 1.8440E+01, 1.5940E+03, &
      2.9416424E+00, 6.28699E+01, 1.8317E+01, 1.5755E+03, &
      2.9563506E+00, 6.29388E+01, 1.8194E+01, 1.5573E+03, &
      2.9711323E+00, 6.30083E+01, 1.8072E+01, 1.5393E+03, &
      2.9859880E+00, 6.30795E+01, 1.7951E+01, 1.5214E+03, &
      3.0009179E+00, 6.31550E+01, 1.7830E+01, 1.5038E+03, &
      3.0159225E+00, 6.32207E+01, 1.7691E+01, 1.4847E+03, &
      3.0310021E+00, 6.32811E+01, 1.7553E+01, 1.4659E+03, &
      3.0461571E+00, 6.33383E+01, 1.7416E+01, 1.4473E+03, &
      3.0613879E+00, 6.33929E+01, 1.7281E+01, 1.4290E+03, &
      3.0766949E+00, 6.34452E+01, 1.7146E+01, 1.4109E+03, &
      3.0920783E+00, 6.34955E+01, 1.7013E+01, 1.3931E+03, &
      3.1075387E+00, 6.35439E+01, 1.6881E+01, 1.3755E+03, &
      3.1230764E+00, 6.35904E+01, 1.6751E+01, 1.3582E+03, &
      3.1386918E+00, 6.36352E+01, 1.6621E+01, 1.3411E+03, &
      3.1543853E+00, 6.36785E+01, 1.6492E+01, 1.3242E+03, &
      3.1701572E+00, 6.37201E+01, 1.6364E+01, 1.3074E+03, &
      3.1860080E+00, 6.37602E+01, 1.6237E+01, 1.2909E+03, &
      3.2019380E+00, 6.37989E+01, 1.6111E+01, 1.2746E+03, &
      3.2179477E+00, 6.38361E+01, 1.5986E+01, 1.2585E+03, &
      3.2340374E+00, 6.38720E+01, 1.5862E+01, 1.2426E+03, &
      3.2502076E+00, 6.39066E+01, 1.5739E+01, 1.2270E+03, &
      3.2664587E+00, 6.39399E+01, 1.5618E+01, 1.2115E+03, &
      3.2827910E+00, 6.39721E+01, 1.5497E+01, 1.1963E+03, &
      3.2992049E+00, 6.40031E+01, 1.5378E+01, 1.1813E+03, &
      3.3157009E+00, 6.40330E+01, 1.5259E+01, 1.1665E+03, &
      3.3322794E+00, 6.40618E+01, 1.5142E+01, 1.1518E+03, &
      3.3489408E+00, 6.40896E+01, 1.5025E+01, 1.1373E+03, &
      3.3656856E+00, 6.41163E+01, 1.4909E+01, 1.1230E+03, &
      3.3825140E+00, 6.41420E+01, 1.4794E+01, 1.1089E+03, &
      3.3994265E+00, 6.41668E+01, 1.4680E+01, 1.0950E+03, &
      3.4164237E+00, 6.41907E+01, 1.4567E+01, 1.0813E+03, &
      3.4335058E+00, 6.42137E+01, 1.4456E+01, 1.0677E+03, &
      3.4506733E+00, 6.42358E+01, 1.4345E+01, 1.0544E+03, &
      3.4679267E+00, 6.42571E+01, 1.4235E+01, 1.0412E+03, &
      3.4852663E+00, 6.42776E+01, 1.4127E+01, 1.0282E+03, &
      3.5026927E+00, 6.42973E+01, 1.4019E+01, 1.0154E+03, &
      3.5202061E+00, 6.43163E+01, 1.3912E+01, 1.0027E+03, &
      3.5378072E+00, 6.43346E+01, 1.3806E+01, 9.9022E+02, &
      3.5554962E+00, 6.43522E+01, 1.3702E+01, 9.7790E+02, &
      3.5732737E+00, 6.43692E+01, 1.3598E+01, 9.6575E+02, &
      3.5911400E+00, 6.43855E+01, 1.3495E+01, 9.5376E+02, &
      3.6090957E+00, 6.44013E+01, 1.3393E+01, 9.4193E+02, &
      3.6271412E+00, 6.45202E+01, 1.3291E+01, 9.3020E+02, &
      3.6452769E+00, 6.45356E+01, 1.3187E+01, 9.1840E+02, &
      3.6635033E+00, 6.45500E+01, 1.3084E+01, 9.0676E+02, &
      3.6818208E+00, 6.45634E+01, 1.2981E+01, 8.9528E+02, &
      3.7002299E+00, 6.45759E+01, 1.2880E+01, 8.8395E+02, &
      3.7187311E+00, 6.45876E+01, 1.2779E+01, 8.7277E+02, &
      3.7373247E+00, 6.45985E+01, 1.2680E+01, 8.6175E+02, &
      3.7560114E+00, 6.46087E+01, 1.2581E+01, 8.5087E+02, &
      3.7747914E+00, 6.46181E+01, 1.2483E+01, 8.4014E+02, &
      3.7936654E+00, 6.46267E+01, 1.2386E+01, 8.2955E+02, &
      3.8126337E+00, 6.46348E+01, 1.2290E+01, 8.1911E+02, &
      3.8316969E+00, 6.46421E+01, 1.2195E+01, 8.0880E+02, &
      3.8508554E+00, 6.46489E+01, 1.2101E+01, 7.9864E+02, &
      3.8701096E+00, 6.46550E+01, 1.2007E+01, 7.8861E+02, &
      3.8894602E+00, 6.46606E+01, 1.1914E+01, 7.7871E+02, &
      3.9089075E+00, 6.46656E+01, 1.1823E+01, 7.6894E+02, &
      3.9284520E+00, 6.46701E+01, 1.1732E+01, 7.5931E+02, &
      3.9480943E+00, 6.46741E+01, 1.1641E+01, 7.4981E+02, &
      3.9678347E+00, 6.46776E+01, 1.1552E+01, 7.4043E+02, &
      3.9876739E+00, 6.46807E+01, 1.1463E+01, 7.3118E+02, &
      4.055024E+00, 6.51585E+01, 1.1166E+01, 7.0070E+02, &
      4.334821E+00, 6.50633E+01, 1.0057E+01, 5.9133E+02, &
      4.633924E+00, 6.49252E+01, 9.0625E+00, 4.9937E+02, &
      4.953664E+00, 6.47070E+01, 8.1745E+00, 4.2223E+02, &
      5.295467E+00, 6.44383E+01, 7.3833E+00, 3.5755E+02, &
      5.660855E+00, 6.41184E+01, 6.6775E+00, 3.0325E+02, &
      6.051453E+00, 6.37384E+01, 6.0465E+00, 2.5757E+02, &
      6.469004E+00, 6.32673E+01, 5.4548E+00, 2.1803E+02, &
      6.915365E+00, 6.26355E+01, 4.9195E+00, 1.8458E+02, &
      7.392525E+00, 6.17181E+01, 4.4427E+00, 1.5651E+02, &
      7.902609E+00, 6.00572E+01, 4.0175E+00, 1.3292E+02, &
      8.190742E+00, 5.78968E+01, 3.8007E+00, 1.2161E+02, &
      8.316110E+00, 5.49282E+01, 3.7129E+00, 1.1714E+02, &
      8.349542E+00, 5.14509E+01, 3.6901E+00, 1.1598E+02, &
      8.399689E+00, 5.49359E+01, 1.0444E+01, 3.1759E+02, &
      8.447890E+00, 5.66313E+01, 1.0315E+01, 3.1194E+02, &
      8.525058E+00, 5.79659E+01, 1.0115E+01, 3.0321E+02, &
      9.030794E+00, 5.98477E+01, 9.0727E+00, 2.5722E+02, &
      9.079014E+00, 5.97452E+01, 8.9917E+00, 2.5362E+02, &
      9.217979E+00, 5.86124E+01, 8.7652E+00, 2.4362E+02, &
      9.255036E+00, 5.69490E+01, 8.7063E+00, 2.4104E+02, &
      9.310622E+00, 5.88004E+01, 1.2022E+01, 3.2920E+02, &
      9.449586E+00, 6.05211E+01, 1.1691E+01, 3.1556E+02, &
      9.556274E+00, 6.10405E+01, 1.1445E+01, 3.0556E+02, &
      9.653919E+00, 6.11845E+01, 1.1224E+01, 2.9671E+02, &
      9.702543E+00, 6.10288E+01, 1.1116E+01, 2.9242E+02, &
      9.741549E+00, 6.03128E+01, 1.1033E+01, 2.8910E+02, &
      9.800056E+00, 6.13998E+01, 1.2628E+01, 3.2833E+02, &
      9.946326E+00, 6.26541E+01, 1.2323E+01, 3.1578E+02, &
      1.032004E+01, 6.41771E+01, 1.1592E+01, 2.8655E+02, &
      1.103212E+01, 6.56930E+01, 1.0376E+01, 2.4032E+02, &
      1.179334E+01, 6.66121E+01, 9.3223E+00, 2.0233E+02, &
      1.260708E+01, 6.72371E+01, 8.3663E+00, 1.7019E+02, &
      1.347697E+01, 6.76565E+01, 7.4967E+00, 1.4296E+02, &
      1.440688E+01, 6.79349E+01, 6.7173E+00, 1.2010E+02, &
      1.540095E+01, 6.81159E+01, 6.0205E+00, 1.0095E+02, &
      1.646362E+01, 6.82286E+01, 5.3984E+00, 8.4904E+01, &
      1.759961E+01, 6.83550E+01, 4.8304E+00, 7.1281E+01, &
      1.881398E+01, 6.84065E+01, 4.3214E+00, 5.9848E+01, &
      2.011215E+01, 6.84211E+01, 3.8614E+00, 5.0205E+01, &
      2.149988E+01, 6.83798E+01, 3.4490E+00, 4.2115E+01, &
      2.298338E+01, 6.83214E+01, 3.0834E+00, 3.5368E+01, &
      2.456923E+01, 6.82530E+01, 2.7590E+00, 2.9738E+01, &
      2.626450E+01, 6.81838E+01, 2.4708E+00, 2.5035E+01, &
      2.807676E+01, 6.81129E+01, 2.1971E+00, 2.0942E+01, &
      3.001405E+01, 6.80170E+01, 1.9502E+00, 1.7499E+01, &
      3.208502E+01, 6.79127E+01, 1.7322E+00, 1.4640E+01, &
      3.429889E+01, 6.78019E+01, 1.5400E+00, 1.2264E+01, &
      3.666551E+01, 6.76845E+01, 1.3696E+00, 1.0285E+01, &
      3.919543E+01, 6.75570E+01, 1.2165E+00, 8.6215E+00, &
      4.189992E+01, 6.74143E+01, 1.0801E+00, 7.2304E+00, &
      4.479101E+01, 6.72454E+01, 9.5556E-01, 6.0497E+00, &
      4.788159E+01, 6.70286E+01, 8.4529E-01, 5.0668E+00, &
      5.118542E+01, 6.67132E+01, 7.4787E-01, 4.2494E+00, &
      5.471721E+01, 6.60951E+01, 6.6218E-01, 3.5708E+00, &
      5.633579E+01, 6.54055E+01, 6.2805E-01, 3.3116E+00, &
      5.719807E+01, 6.42643E+01, 6.1100E-01, 3.1846E+00, &
      5.742801E+01, 6.28976E+01, 6.0658E-01, 3.1519E+00, &
      5.777293E+01, 6.42790E+01, 3.2906E+00, 1.4821E+01, &
      5.849270E+01, 6.54056E+01, 3.2283E+00, 1.4368E+01, &
      5.863521E+01, 6.55268E+01, 3.2162E+00, 1.4282E+01, &
      6.252870E+01, 6.69049E+01, 2.9062E+00, 1.2137E+01, &
      6.684318E+01, 6.74553E+01, 2.6074E+00, 1.0223E+01, &
      7.145536E+01, 6.77716E+01, 2.3374E+00, 8.6065E+00, &
      7.638578E+01, 6.79759E+01, 2.0950E+00, 7.2478E+00, &
      8.165640E+01, 6.81195E+01, 1.8742E+00, 6.0960E+00, &
      8.729069E+01, 6.82120E+01, 1.6714E+00, 5.1150E+00, &
      9.331374E+01, 6.82714E+01, 1.4907E+00, 4.2956E+00, &
      9.975239E+01, 6.83086E+01, 1.3298E+00, 3.6109E+00, &
      1.066353E+02, 6.83301E+01, 1.1863E+00, 3.0385E+00, &
      1.139931E+02, 6.83409E+01, 1.0585E+00, 2.5601E+00, &
      1.218587E+02, 6.83712E+01, 9.4103E-01, 2.1524E+00, &
      1.302669E+02, 6.83646E+01, 8.3621E-01, 1.8118E+00, &
      1.392553E+02, 6.83529E+01, 7.4314E-01, 1.5277E+00, &
      1.488640E+02, 6.83380E+01, 6.6049E-01, 1.2908E+00, &
      1.591356E+02, 6.83210E+01, 5.8709E-01, 1.0931E+00, &
      1.701159E+02, 6.83027E+01, 5.2189E-01, 9.2800E-01, &
      1.818539E+02, 6.82839E+01, 4.6397E-01, 7.9008E-01, &
      1.944018E+02, 6.82650E+01, 4.1252E-01, 6.7475E-01, &
      2.078156E+02, 6.82463E+01, 3.6680E-01, 5.7823E-01, &
      2.221548E+02, 6.82279E+01, 3.2617E-01, 4.9739E-01, &
      2.374835E+02, 6.82101E+01, 2.9007E-01, 4.2959E-01, &
      2.538699E+02, 6.81928E+01, 2.5839E-01, 3.7308E-01, &
      2.713869E+02, 6.81769E+01, 2.3113E-01, 3.2635E-01, &
      2.901126E+02, 6.81622E+01, 2.0676E-01, 2.8677E-01, &
      3.101304E+02, 6.81487E+01, 1.8498E-01, 2.5320E-01, &
      3.315294E+02, 6.81363E+01, 1.6550E-01, 2.2464E-01, &
      3.544049E+02, 6.81247E+01, 1.4808E-01, 2.0031E-01, &
      3.788588E+02, 6.81141E+01, 1.3250E-01, 1.7951E-01, &
      4.050001E+02, 6.81042E+01, 1.1857E-01, 1.6168E-01, &
      4.329451E+02, 6.80951E+01, 1.0611E-01, 1.4634E-01]
   case(69)
      frel = -7.2240E-01
      allocate(nist_tab(3596))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.194000E-03, 1.05318E+01, 0.0000E+00, 7.0183E-06, &
      5.273500E-03, 1.04639E+01, 0.0000E+00, 7.6206E-06, &
      5.294700E-03, 1.03894E+01, 0.0000E+00, 7.7877E-06, &
      5.326500E-03, 1.04604E+01, 1.4292E-01, 6.6835E+03, &
      5.406000E-03, 1.05184E+01, 1.4344E-01, 6.6095E+03, &
      1.069000E-02, 1.02397E+01, 2.5905E-01, 6.0363E+03, &
      1.142761E-02, 1.01620E+01, 2.8489E-01, 6.2100E+03, &
      1.221612E-02, 1.00721E+01, 3.1469E-01, 6.4167E+03, &
      1.305903E-02, 9.96790E+00, 3.4900E-01, 6.6570E+03, &
      1.396010E-02, 9.84691E+00, 3.8849E-01, 6.9319E+03, &
      1.492335E-02, 9.70585E+00, 4.3390E-01, 7.2425E+03, &
      1.595306E-02, 1.02087E+01, 4.8611E-01, 7.5902E+03, &
      1.705382E-02, 1.00135E+01, 5.4609E-01, 7.9763E+03, &
      1.823053E-02, 9.78076E+00, 6.1496E-01, 8.4026E+03, &
      1.948844E-02, 9.49946E+00, 6.9400E-01, 8.8704E+03, &
      2.083314E-02, 9.15334E+00, 7.8460E-01, 9.3811E+03, &
      2.227063E-02, 8.71683E+00, 8.8828E-01, 9.9352E+03, &
      2.380730E-02, 8.14646E+00, 1.0065E+00, 1.0531E+04, &
      2.545001E-02, 7.36100E+00, 1.1409E+00, 1.1167E+04, &
      2.720606E-02, 6.18552E+00, 1.2935E+00, 1.1843E+04, &
      2.908327E-02, 4.13989E+00, 1.4661E+00, 1.2557E+04, &
      3.109002E-02, -1.07371E+00, 1.6608E+00, 1.3306E+04, &
      3.165400E-02, -4.94686E+00, 1.7174E+00, 1.3514E+04, &
      3.213850E-02, -1.44758E+01, 1.7665E+00, 1.3692E+04, &
      3.226770E-02, -2.64192E+01, 1.7797E+00, 1.3739E+04, &
      3.246150E-02, -1.32016E+01, 2.5575E+01, 1.9625E+05, &
      3.294600E-02, -1.50865E+00, 2.4276E+01, 1.8354E+05, &
      3.323523E-02, 1.71508E+00, 2.3547E+01, 1.7648E+05, &
      3.552846E-02, 1.20535E+01, 1.8798E+01, 1.3180E+05, &
      3.797993E-02, 1.56441E+01, 1.5152E+01, 9.9372E+04, &
      4.060054E-02, 1.70816E+01, 1.2365E+01, 7.5863E+04, &
      4.340198E-02, 1.75084E+01, 1.0336E+01, 5.9323E+04, &
      4.639671E-02, 1.74405E+01, 8.9323E+00, 4.7956E+04, &
      4.959809E-02, 1.71456E+01, 8.0236E+00, 4.0296E+04, &
      5.213600E-02, 1.68602E+01, 7.6007E+00, 3.6314E+04, &
      5.293400E-02, 1.67592E+01, 7.5087E+00, 3.5334E+04, &
      5.302035E-02, 1.67457E+01, 7.4997E+00, 3.5234E+04, &
      5.314680E-02, 1.67187E+01, 7.4870E+00, 3.5091E+04, &
      5.346600E-02, 1.67026E+01, 7.4917E+00, 3.4903E+04, &
      5.426400E-02, 1.66351E+01, 7.4271E+00, 3.4093E+04, &
      5.667876E-02, 1.64141E+01, 7.3118E+00, 3.2134E+04, &
      6.058959E-02, 1.61064E+01, 7.3198E+00, 3.0093E+04, &
      6.477028E-02, 1.58667E+01, 7.5089E+00, 2.8878E+04, &
      6.923942E-02, 1.57111E+01, 7.8428E+00, 2.8215E+04, &
      7.401695E-02, 1.56505E+01, 8.2953E+00, 2.7917E+04, &
      7.912411E-02, 1.56952E+01, 8.8472E+00, 2.7852E+04, &
      8.458368E-02, 1.58582E+01, 9.4845E+00, 2.7931E+04, &
      9.041995E-02, 1.61588E+01, 1.0197E+01, 2.8090E+04, &
      9.665893E-02, 1.66281E+01, 1.0976E+01, 2.8286E+04, &
      1.0000000E-01, 1.90910E+01, 1.1363E+01, 2.8305E+04, &
      1.0050000E-01, 1.91424E+01, 1.1390E+01, 2.8230E+04, &
      1.0100250E-01, 1.91938E+01, 1.1416E+01, 2.8156E+04, &
      1.0150751E-01, 1.92449E+01, 1.1443E+01, 2.8080E+04, &
      1.0201505E-01, 1.92960E+01, 1.1469E+01, 2.8005E+04, &
      1.0252513E-01, 1.93470E+01, 1.1495E+01, 2.7929E+04, &
      1.0303775E-01, 1.93979E+01, 1.1521E+01, 2.7853E+04, &
      1.0355294E-01, 1.94486E+01, 1.1547E+01, 2.7777E+04, &
      1.0407070E-01, 1.94993E+01, 1.1573E+01, 2.7700E+04, &
      1.0459106E-01, 1.95500E+01, 1.1598E+01, 2.7623E+04, &
      1.0511401E-01, 1.96005E+01, 1.1624E+01, 2.7546E+04, &
      1.0563958E-01, 1.96510E+01, 1.1649E+01, 2.7468E+04, &
      1.0616778E-01, 1.97015E+01, 1.1674E+01, 2.7390E+04, &
      1.0669862E-01, 1.97519E+01, 1.1699E+01, 2.7311E+04, &
      1.0723211E-01, 1.98022E+01, 1.1723E+01, 2.7233E+04, &
      1.0776827E-01, 1.98525E+01, 1.1747E+01, 2.7153E+04, &
      1.0830712E-01, 1.99028E+01, 1.1772E+01, 2.7074E+04, &
      1.0884865E-01, 1.99530E+01, 1.1796E+01, 2.6994E+04, &
      1.0939289E-01, 2.00032E+01, 1.1819E+01, 2.6914E+04, &
      1.0993986E-01, 2.00533E+01, 1.1843E+01, 2.6834E+04, &
      1.1048956E-01, 2.01034E+01, 1.1866E+01, 2.6753E+04, &
      1.1104201E-01, 2.01535E+01, 1.1890E+01, 2.6672E+04, &
      1.1159722E-01, 2.02035E+01, 1.1912E+01, 2.6590E+04, &
      1.1215520E-01, 2.02535E+01, 1.1935E+01, 2.6508E+04, &
      1.1271598E-01, 2.03035E+01, 1.1958E+01, 2.6426E+04, &
      1.1327956E-01, 2.03534E+01, 1.1980E+01, 2.6344E+04, &
      1.1384596E-01, 2.04033E+01, 1.2002E+01, 2.6261E+04, &
      1.1441519E-01, 2.04531E+01, 1.2024E+01, 2.6177E+04, &
      1.1498726E-01, 2.05029E+01, 1.2045E+01, 2.6094E+04, &
      1.1556220E-01, 2.05526E+01, 1.2067E+01, 2.6010E+04, &
      1.1614001E-01, 2.06023E+01, 1.2088E+01, 2.5926E+04, &
      1.1672071E-01, 2.06519E+01, 1.2109E+01, 2.5841E+04, &
      1.1730431E-01, 2.07014E+01, 1.2129E+01, 2.5757E+04, &
      1.1789083E-01, 2.07509E+01, 1.2149E+01, 2.5671E+04, &
      1.1848029E-01, 2.08003E+01, 1.2170E+01, 2.5586E+04, &
      1.1907269E-01, 2.08496E+01, 1.2189E+01, 2.5500E+04, &
      1.1966805E-01, 2.08989E+01, 1.2209E+01, 2.5414E+04, &
      1.2026639E-01, 2.09480E+01, 1.2228E+01, 2.5328E+04, &
      1.2086772E-01, 2.09971E+01, 1.2247E+01, 2.5241E+04, &
      1.2147206E-01, 2.10460E+01, 1.2266E+01, 2.5154E+04, &
      1.2207942E-01, 2.10948E+01, 1.2285E+01, 2.5066E+04, &
      1.2268982E-01, 2.11436E+01, 1.2303E+01, 2.4979E+04, &
      1.2330327E-01, 2.11921E+01, 1.2321E+01, 2.4891E+04, &
      1.2391979E-01, 2.12406E+01, 1.2338E+01, 2.4802E+04, &
      1.2453939E-01, 2.12889E+01, 1.2356E+01, 2.4714E+04, &
      1.2516208E-01, 2.13370E+01, 1.2373E+01, 2.4625E+04, &
      1.2578789E-01, 2.13850E+01, 1.2390E+01, 2.4536E+04, &
      1.2641683E-01, 2.14328E+01, 1.2406E+01, 2.4446E+04, &
      1.2704892E-01, 2.14804E+01, 1.2423E+01, 2.4357E+04, &
      1.2768416E-01, 2.15278E+01, 1.2439E+01, 2.4267E+04, &
      1.2832258E-01, 2.15750E+01, 1.2454E+01, 2.4176E+04, &
      1.2896419E-01, 2.16219E+01, 1.2470E+01, 2.4086E+04, &
      1.2960902E-01, 2.16686E+01, 1.2485E+01, 2.3995E+04, &
      1.3025706E-01, 2.17151E+01, 1.2500E+01, 2.3904E+04, &
      1.3090835E-01, 2.17612E+01, 1.2514E+01, 2.3813E+04, &
      1.3156289E-01, 2.18071E+01, 1.2528E+01, 2.3721E+04, &
      1.3222070E-01, 2.18527E+01, 1.2542E+01, 2.3629E+04, &
      1.3288181E-01, 2.18980E+01, 1.2556E+01, 2.3537E+04, &
      1.3354621E-01, 2.19429E+01, 1.2569E+01, 2.3445E+04, &
      1.3421395E-01, 2.19874E+01, 1.2582E+01, 2.3353E+04, &
      1.3488502E-01, 2.20316E+01, 1.2595E+01, 2.3260E+04, &
      1.3555944E-01, 2.20753E+01, 1.2607E+01, 2.3167E+04, &
      1.3623724E-01, 2.21187E+01, 1.2619E+01, 2.3074E+04, &
      1.3691842E-01, 2.21615E+01, 1.2631E+01, 2.2980E+04, &
      1.3760302E-01, 2.22039E+01, 1.2643E+01, 2.2887E+04, &
      1.3829103E-01, 2.22458E+01, 1.2654E+01, 2.2793E+04, &
      1.3898249E-01, 2.22871E+01, 1.2665E+01, 2.2699E+04, &
      1.3967740E-01, 2.23279E+01, 1.2675E+01, 2.2605E+04, &
      1.4037579E-01, 2.23680E+01, 1.2685E+01, 2.2511E+04, &
      1.4107766E-01, 2.24075E+01, 1.2695E+01, 2.2416E+04, &
      1.4178305E-01, 2.24464E+01, 1.2705E+01, 2.2321E+04, &
      1.4249197E-01, 2.24845E+01, 1.2714E+01, 2.2226E+04, &
      1.4320443E-01, 2.25218E+01, 1.2723E+01, 2.2131E+04, &
      1.4392045E-01, 2.25583E+01, 1.2731E+01, 2.2036E+04, &
      1.4464005E-01, 2.25940E+01, 1.2740E+01, 2.1941E+04, &
      1.4536325E-01, 2.26288E+01, 1.2748E+01, 2.1845E+04, &
      1.4609007E-01, 2.26625E+01, 1.2755E+01, 2.1749E+04, &
      1.4682052E-01, 2.26953E+01, 1.2762E+01, 2.1654E+04, &
      1.4755462E-01, 2.27269E+01, 1.2769E+01, 2.1558E+04, &
      1.4829239E-01, 2.27573E+01, 1.2776E+01, 2.1461E+04, &
      1.4903386E-01, 2.27865E+01, 1.2782E+01, 2.1365E+04, &
      1.4977903E-01, 2.28143E+01, 1.2788E+01, 2.1269E+04, &
      1.5052792E-01, 2.28407E+01, 1.2794E+01, 2.1172E+04, &
      1.5128056E-01, 2.28655E+01, 1.2799E+01, 2.1076E+04, &
      1.5203696E-01, 2.28887E+01, 1.2804E+01, 2.0979E+04, &
      1.5279715E-01, 2.29101E+01, 1.2809E+01, 2.0882E+04, &
      1.5356113E-01, 2.29295E+01, 1.2813E+01, 2.0785E+04, &
      1.5432894E-01, 2.29469E+01, 1.2817E+01, 2.0688E+04, &
      1.5510058E-01, 2.29620E+01, 1.2821E+01, 2.0591E+04, &
      1.5587609E-01, 2.29747E+01, 1.2824E+01, 2.0494E+04, &
      1.5665547E-01, 2.29847E+01, 1.2827E+01, 2.0397E+04, &
      1.5743875E-01, 2.29918E+01, 1.2829E+01, 2.0299E+04, &
      1.5822594E-01, 2.29957E+01, 1.2832E+01, 2.0202E+04, &
      1.5901707E-01, 2.29962E+01, 1.2834E+01, 2.0104E+04, &
      1.5981215E-01, 2.29944E+01, 1.2835E+01, 2.0007E+04, &
      1.6061121E-01, 2.29868E+01, 1.2836E+01, 1.9909E+04, &
      1.6141427E-01, 2.29746E+01, 1.2837E+01, 1.9812E+04, &
      1.6222134E-01, 2.29572E+01, 1.2838E+01, 1.9714E+04, &
      1.6303245E-01, 2.29340E+01, 1.2838E+01, 1.9616E+04, &
      1.6384761E-01, 2.29042E+01, 1.2838E+01, 1.9519E+04, &
      1.6466685E-01, 2.28671E+01, 1.2838E+01, 1.9421E+04, &
      1.6549018E-01, 2.28215E+01, 1.2837E+01, 1.9323E+04, &
      1.6631763E-01, 2.27663E+01, 1.2836E+01, 1.9225E+04, &
      1.6714922E-01, 2.27000E+01, 1.2834E+01, 1.9128E+04, &
      1.6798497E-01, 2.26207E+01, 1.2832E+01, 1.9030E+04, &
      1.6882489E-01, 2.25262E+01, 1.2830E+01, 1.8932E+04, &
      1.6966902E-01, 2.24134E+01, 1.2828E+01, 1.8834E+04, &
      1.7051736E-01, 2.22785E+01, 1.2825E+01, 1.8736E+04, &
      1.7136995E-01, 2.21165E+01, 1.2822E+01, 1.8639E+04, &
      1.7222680E-01, 2.19205E+01, 1.2819E+01, 1.8541E+04, &
      1.7308793E-01, 2.16808E+01, 1.2815E+01, 1.8443E+04, &
      1.7395337E-01, 2.13833E+01, 1.2811E+01, 1.8346E+04, &
      1.7482314E-01, 2.10060E+01, 1.2806E+01, 1.8248E+04, &
      1.7569726E-01, 2.05130E+01, 1.2801E+01, 1.8150E+04, &
      1.7657574E-01, 1.98386E+01, 1.2796E+01, 1.8053E+04, &
      1.7745862E-01, 1.88450E+01, 1.2791E+01, 1.7955E+04, &
      1.7834591E-01, 1.71548E+01, 1.2785E+01, 1.7858E+04, &
      1.7923764E-01, 1.27504E+01, 1.2779E+01, 1.7761E+04, &
      1.7946997E-01, 8.81247E+00, 1.2777E+01, 1.7735E+04, &
      1.7949044E-01, 8.13869E+00, 1.2777E+01, 1.7733E+04, &
      1.7970956E-01, 8.41529E+00, 2.5450E+01, 3.5278E+04, &
      1.7973003E-01, 9.13343E+00, 2.5435E+01, 3.5252E+04, &
      1.8013383E-01, 1.52124E+01, 2.5131E+01, 3.4754E+04, &
      1.8103450E-01, 1.97264E+01, 2.4482E+01, 3.3687E+04, &
      1.8193967E-01, 2.20462E+01, 2.3866E+01, 3.2676E+04, &
      1.8284937E-01, 2.36141E+01, 2.3281E+01, 3.1717E+04, &
      1.8376362E-01, 2.47851E+01, 2.2726E+01, 3.0807E+04, &
      1.8468244E-01, 2.57057E+01, 2.2199E+01, 2.9943E+04, &
      1.8560585E-01, 2.64522E+01, 2.1699E+01, 2.9122E+04, &
      1.8653388E-01, 2.70700E+01, 2.1224E+01, 2.8343E+04, &
      1.8746655E-01, 2.75885E+01, 2.0772E+01, 2.7603E+04, &
      1.8840388E-01, 2.80278E+01, 2.0344E+01, 2.6899E+04, &
      1.8934590E-01, 2.84029E+01, 1.9937E+01, 2.6229E+04, &
      1.9029263E-01, 2.87246E+01, 1.9550E+01, 2.5593E+04, &
      1.9124409E-01, 2.90014E+01, 1.9183E+01, 2.4987E+04, &
      1.9220031E-01, 2.92401E+01, 1.8833E+01, 2.4410E+04, &
      1.9316131E-01, 2.94459E+01, 1.8501E+01, 2.3860E+04, &
      1.9412712E-01, 2.96232E+01, 1.8186E+01, 2.3337E+04, &
      1.9509776E-01, 2.97754E+01, 1.7886E+01, 2.2838E+04, &
      1.9607325E-01, 2.99052E+01, 1.7600E+01, 2.2361E+04, &
      1.9705361E-01, 3.00149E+01, 1.7329E+01, 2.1907E+04, &
      1.9803888E-01, 3.01058E+01, 1.7071E+01, 2.1474E+04, &
      1.9902907E-01, 3.01799E+01, 1.6830E+01, 2.1066E+04, &
      2.0002422E-01, 3.02412E+01, 1.6607E+01, 2.0683E+04, &
      2.0102434E-01, 3.02925E+01, 1.6400E+01, 2.0323E+04, &
      2.0202946E-01, 3.03359E+01, 1.6207E+01, 1.9984E+04, &
      2.0303961E-01, 3.03731E+01, 1.6027E+01, 1.9664E+04, &
      2.0405481E-01, 3.04052E+01, 1.5858E+01, 1.9360E+04, &
      2.0507508E-01, 3.04334E+01, 1.5701E+01, 1.9073E+04, &
      2.0610046E-01, 3.04584E+01, 1.5554E+01, 1.8800E+04, &
      2.0713096E-01, 3.04808E+01, 1.5415E+01, 1.8540E+04, &
      2.0816661E-01, 3.05012E+01, 1.5285E+01, 1.8293E+04, &
      2.0920745E-01, 3.05200E+01, 1.5163E+01, 1.8056E+04, &
      2.1025348E-01, 3.05376E+01, 1.5048E+01, 1.7829E+04, &
      2.1130475E-01, 3.05542E+01, 1.4939E+01, 1.7612E+04, &
      2.1236128E-01, 3.05702E+01, 1.4836E+01, 1.7404E+04, &
      2.1342308E-01, 3.05857E+01, 1.4738E+01, 1.7204E+04, &
      2.1449020E-01, 3.06009E+01, 1.4646E+01, 1.7011E+04, &
      2.1556265E-01, 3.06158E+01, 1.4558E+01, 1.6825E+04, &
      2.1664046E-01, 3.06307E+01, 1.4475E+01, 1.6645E+04, &
      2.1772366E-01, 3.06456E+01, 1.4395E+01, 1.6471E+04, &
      2.1881228E-01, 3.06606E+01, 1.4319E+01, 1.6303E+04, &
      2.1990634E-01, 3.06758E+01, 1.4247E+01, 1.6140E+04, &
      2.2100588E-01, 3.06911E+01, 1.4177E+01, 1.5981E+04, &
      2.2211090E-01, 3.07066E+01, 1.4111E+01, 1.5827E+04, &
      2.2322146E-01, 3.07224E+01, 1.4047E+01, 1.5677E+04, &
      2.2433757E-01, 3.07384E+01, 1.3986E+01, 1.5531E+04, &
      2.2545925E-01, 3.07547E+01, 1.3927E+01, 1.5389E+04, &
      2.2658655E-01, 3.07713E+01, 1.3870E+01, 1.5250E+04, &
      2.2771948E-01, 3.07881E+01, 1.3816E+01, 1.5115E+04, &
      2.2885808E-01, 3.08053E+01, 1.3763E+01, 1.4982E+04, &
      2.3000237E-01, 3.08227E+01, 1.3712E+01, 1.4853E+04, &
      2.3115238E-01, 3.08403E+01, 1.3663E+01, 1.4726E+04, &
      2.3230814E-01, 3.08582E+01, 1.3615E+01, 1.4601E+04, &
      2.3346969E-01, 3.08763E+01, 1.3569E+01, 1.4479E+04, &
      2.3463703E-01, 3.08947E+01, 1.3524E+01, 1.4360E+04, &
      2.3581022E-01, 3.09133E+01, 1.3481E+01, 1.4242E+04, &
      2.3698927E-01, 3.09320E+01, 1.3438E+01, 1.4127E+04, &
      2.3817422E-01, 3.09510E+01, 1.3397E+01, 1.4014E+04, &
      2.3936509E-01, 3.09701E+01, 1.3357E+01, 1.3902E+04, &
      2.4056191E-01, 3.09894E+01, 1.3318E+01, 1.3792E+04, &
      2.4176472E-01, 3.10088E+01, 1.3280E+01, 1.3685E+04, &
      2.4297355E-01, 3.10283E+01, 1.3242E+01, 1.3578E+04, &
      2.4418841E-01, 3.10479E+01, 1.3206E+01, 1.3474E+04, &
      2.4540936E-01, 3.10676E+01, 1.3170E+01, 1.3370E+04, &
      2.4663640E-01, 3.10873E+01, 1.3135E+01, 1.3269E+04, &
      2.4786959E-01, 3.11071E+01, 1.3101E+01, 1.3168E+04, &
      2.4910893E-01, 3.11269E+01, 1.3068E+01, 1.3069E+04, &
      2.5035448E-01, 3.11468E+01, 1.3035E+01, 1.2972E+04, &
      2.5160625E-01, 3.11666E+01, 1.3003E+01, 1.2875E+04, &
      2.5286428E-01, 3.11864E+01, 1.2971E+01, 1.2780E+04, &
      2.5412860E-01, 3.12062E+01, 1.2940E+01, 1.2686E+04, &
      2.5539925E-01, 3.12259E+01, 1.2909E+01, 1.2593E+04, &
      2.5667624E-01, 3.12455E+01, 1.2879E+01, 1.2502E+04, &
      2.5795962E-01, 3.12650E+01, 1.2850E+01, 1.2411E+04, &
      2.5924942E-01, 3.12845E+01, 1.2821E+01, 1.2321E+04, &
      2.6054567E-01, 3.13038E+01, 1.2792E+01, 1.2233E+04, &
      2.6184840E-01, 3.13229E+01, 1.2764E+01, 1.2145E+04, &
      2.6315764E-01, 3.13420E+01, 1.2737E+01, 1.2059E+04, &
      2.6447343E-01, 3.13608E+01, 1.2709E+01, 1.1973E+04, &
      2.6579579E-01, 3.13795E+01, 1.2683E+01, 1.1888E+04, &
      2.6712477E-01, 3.13979E+01, 1.2656E+01, 1.1805E+04, &
      2.6846040E-01, 3.14162E+01, 1.2630E+01, 1.1722E+04, &
      2.6980270E-01, 3.14342E+01, 1.2604E+01, 1.1640E+04, &
      2.7115171E-01, 3.14519E+01, 1.2579E+01, 1.1559E+04, &
      2.7250747E-01, 3.14694E+01, 1.2554E+01, 1.1478E+04, &
      2.7387001E-01, 3.14866E+01, 1.2529E+01, 1.1399E+04, &
      2.7523936E-01, 3.15035E+01, 1.2505E+01, 1.1320E+04, &
      2.7661556E-01, 3.15200E+01, 1.2481E+01, 1.1242E+04, &
      2.7799863E-01, 3.15363E+01, 1.2457E+01, 1.1165E+04, &
      2.7938863E-01, 3.15521E+01, 1.2434E+01, 1.1089E+04, &
      2.8078557E-01, 3.15676E+01, 1.2411E+01, 1.1013E+04, &
      2.8218950E-01, 3.15827E+01, 1.2388E+01, 1.0938E+04, &
      2.8360044E-01, 3.15973E+01, 1.2366E+01, 1.0864E+04, &
      2.8501845E-01, 3.16115E+01, 1.2344E+01, 1.0791E+04, &
      2.8644354E-01, 3.16252E+01, 1.2322E+01, 1.0718E+04, &
      2.8787576E-01, 3.16384E+01, 1.2301E+01, 1.0647E+04, &
      2.8931514E-01, 3.16511E+01, 1.2279E+01, 1.0575E+04, &
      2.9076171E-01, 3.16632E+01, 1.2258E+01, 1.0505E+04, &
      2.9221552E-01, 3.16747E+01, 1.2238E+01, 1.0435E+04, &
      2.9367660E-01, 3.16856E+01, 1.2217E+01, 1.0366E+04, &
      2.9514498E-01, 3.16958E+01, 1.2197E+01, 1.0297E+04, &
      2.9662071E-01, 3.17053E+01, 1.2178E+01, 1.0230E+04, &
      2.9810381E-01, 3.17141E+01, 1.2158E+01, 1.0162E+04, &
      2.9959433E-01, 3.17220E+01, 1.2139E+01, 1.0096E+04, &
      3.0109230E-01, 3.17290E+01, 1.2120E+01, 1.0030E+04, &
      3.0259776E-01, 3.17351E+01, 1.2101E+01, 9.9649E+03, &
      3.0411075E-01, 3.17401E+01, 1.2083E+01, 9.9003E+03, &
      3.0563130E-01, 3.17440E+01, 1.2065E+01, 9.8364E+03, &
      3.0715946E-01, 3.17468E+01, 1.2047E+01, 9.7731E+03, &
      3.0869526E-01, 3.17481E+01, 1.2029E+01, 9.7103E+03, &
      3.1023873E-01, 3.17480E+01, 1.2012E+01, 9.6482E+03, &
      3.1178993E-01, 3.17462E+01, 1.1995E+01, 9.5867E+03, &
      3.1334888E-01, 3.17426E+01, 1.1979E+01, 9.5258E+03, &
      3.1491562E-01, 3.17369E+01, 1.1962E+01, 9.4655E+03, &
      3.1649020E-01, 3.17288E+01, 1.1946E+01, 9.4058E+03, &
      3.1807265E-01, 3.17179E+01, 1.1930E+01, 9.3467E+03, &
      3.1966301E-01, 3.17038E+01, 1.1915E+01, 9.2881E+03, &
      3.2126133E-01, 3.16858E+01, 1.1900E+01, 9.2301E+03, &
      3.2286764E-01, 3.16631E+01, 1.1885E+01, 9.1727E+03, &
      3.2448197E-01, 3.16346E+01, 1.1870E+01, 9.1159E+03, &
      3.2610438E-01, 3.15986E+01, 1.1856E+01, 9.0596E+03, &
      3.2773491E-01, 3.15527E+01, 1.1842E+01, 9.0038E+03, &
      3.2937358E-01, 3.14927E+01, 1.1828E+01, 8.9486E+03, &
      3.3102045E-01, 3.14116E+01, 1.1814E+01, 8.8940E+03, &
      3.3267555E-01, 3.12942E+01, 1.1801E+01, 8.8399E+03, &
      3.3433893E-01, 3.10997E+01, 1.1788E+01, 8.7863E+03, &
      3.3601062E-01, 3.06005E+01, 1.1775E+01, 8.7332E+03, &
      3.3621965E-01, 3.04348E+01, 1.1774E+01, 8.7266E+03, &
      3.3698036E-01, 3.04405E+01, 1.2970E+01, 9.5911E+03, &
      3.3769068E-01, 3.08490E+01, 1.2967E+01, 9.5685E+03, &
      3.3937913E-01, 3.12216E+01, 1.2959E+01, 9.5152E+03, &
      3.4107602E-01, 3.14195E+01, 1.2951E+01, 9.4624E+03, &
      3.4278140E-01, 3.15590E+01, 1.2944E+01, 9.4101E+03, &
      3.4449531E-01, 3.16687E+01, 1.2937E+01, 9.3583E+03, &
      3.4621779E-01, 3.17604E+01, 1.2930E+01, 9.3070E+03, &
      3.4794888E-01, 3.18399E+01, 1.2924E+01, 9.2562E+03, &
      3.4968862E-01, 3.19106E+01, 1.2918E+01, 9.2059E+03, &
      3.5143706E-01, 3.19746E+01, 1.2912E+01, 9.1561E+03, &
      3.5319425E-01, 3.20333E+01, 1.2907E+01, 9.1068E+03, &
      3.5496022E-01, 3.20876E+01, 1.2902E+01, 9.0579E+03, &
      3.5673502E-01, 3.21383E+01, 1.2897E+01, 9.0095E+03, &
      3.5851870E-01, 3.21859E+01, 1.2892E+01, 8.9615E+03, &
      3.6031129E-01, 3.22307E+01, 1.2888E+01, 8.9140E+03, &
      3.6211285E-01, 3.22730E+01, 1.2884E+01, 8.8670E+03, &
      3.6392341E-01, 3.23129E+01, 1.2880E+01, 8.8204E+03, &
      3.6574303E-01, 3.23506E+01, 1.2877E+01, 8.7742E+03, &
      3.6757174E-01, 3.23861E+01, 1.2874E+01, 8.7285E+03, &
      3.6940960E-01, 3.24194E+01, 1.2871E+01, 8.6831E+03, &
      3.7125665E-01, 3.24503E+01, 1.2868E+01, 8.6383E+03, &
      3.7311293E-01, 3.24785E+01, 1.2866E+01, 8.5937E+03, &
      3.7497850E-01, 3.25035E+01, 1.2864E+01, 8.5496E+03, &
      3.7685339E-01, 3.25246E+01, 1.2862E+01, 8.5059E+03, &
      3.7873766E-01, 3.25402E+01, 1.2860E+01, 8.4625E+03, &
      3.8063135E-01, 3.25474E+01, 1.2859E+01, 8.4195E+03, &
      3.8253450E-01, 3.25391E+01, 1.2858E+01, 8.3769E+03, &
      3.8444718E-01, 3.24890E+01, 1.2857E+01, 8.3346E+03, &
      3.8540604E-01, 3.23946E+01, 1.2856E+01, 8.3136E+03, &
      3.8636941E-01, 3.24074E+01, 1.3185E+01, 8.5049E+03, &
      3.8639394E-01, 3.24131E+01, 1.3185E+01, 8.5043E+03, &
      3.8830126E-01, 3.26148E+01, 1.3186E+01, 8.4633E+03, &
      3.9024276E-01, 3.27137E+01, 1.3187E+01, 8.4221E+03, &
      3.9219398E-01, 3.27893E+01, 1.3189E+01, 8.3813E+03, &
      3.9415495E-01, 3.28543E+01, 1.3191E+01, 8.3407E+03, &
      3.9612572E-01, 3.29132E+01, 1.3193E+01, 8.3005E+03, &
      3.9810635E-01, 3.29681E+01, 1.3195E+01, 8.2606E+03, &
      4.0009688E-01, 3.30201E+01, 1.3197E+01, 8.2210E+03, &
      4.0209737E-01, 3.30700E+01, 1.3200E+01, 8.1817E+03, &
      4.0410785E-01, 3.31182E+01, 1.3202E+01, 8.1427E+03, &
      4.0612839E-01, 3.31649E+01, 1.3205E+01, 8.1040E+03, &
      4.0815904E-01, 3.32106E+01, 1.3208E+01, 8.0655E+03, &
      4.1019983E-01, 3.32552E+01, 1.3211E+01, 8.0274E+03, &
      4.1225083E-01, 3.32990E+01, 1.3215E+01, 7.9895E+03, &
      4.1431208E-01, 3.33421E+01, 1.3218E+01, 7.9518E+03, &
      4.1638364E-01, 3.33845E+01, 1.3222E+01, 7.9144E+03, &
      4.1846556E-01, 3.34262E+01, 1.3225E+01, 7.8773E+03, &
      4.2055789E-01, 3.34674E+01, 1.3229E+01, 7.8404E+03, &
      4.2266068E-01, 3.35080E+01, 1.3233E+01, 7.8037E+03, &
      4.2477398E-01, 3.35481E+01, 1.3237E+01, 7.7673E+03, &
      4.2689785E-01, 3.35877E+01, 1.3241E+01, 7.7311E+03, &
      4.2903234E-01, 3.36267E+01, 1.3245E+01, 7.6951E+03, &
      4.3117750E-01, 3.36652E+01, 1.3249E+01, 7.6593E+03, &
      4.3333339E-01, 3.37031E+01, 1.3254E+01, 7.6237E+03, &
      4.3550006E-01, 3.37404E+01, 1.3258E+01, 7.5883E+03, &
      4.3767756E-01, 3.37770E+01, 1.3262E+01, 7.5531E+03, &
      4.3986595E-01, 3.38129E+01, 1.3267E+01, 7.5181E+03, &
      4.4206528E-01, 3.38480E+01, 1.3271E+01, 7.4833E+03, &
      4.4427560E-01, 3.38822E+01, 1.3276E+01, 7.4486E+03, &
      4.4649698E-01, 3.39152E+01, 1.3280E+01, 7.4142E+03, &
      4.4872947E-01, 3.39469E+01, 1.3285E+01, 7.3798E+03, &
      4.5097311E-01, 3.39770E+01, 1.3289E+01, 7.3457E+03, &
      4.5322798E-01, 3.40052E+01, 1.3294E+01, 7.3116E+03, &
      4.5549412E-01, 3.40308E+01, 1.3298E+01, 7.2778E+03, &
      4.5777159E-01, 3.40532E+01, 1.3303E+01, 7.2440E+03, &
      4.6006045E-01, 3.40709E+01, 1.3307E+01, 7.2105E+03, &
      4.6236075E-01, 3.40821E+01, 1.3312E+01, 7.1770E+03, &
      4.6467255E-01, 3.40828E+01, 1.3316E+01, 7.1436E+03, &
      4.6699592E-01, 3.40642E+01, 1.3320E+01, 7.1104E+03, &
      4.6933090E-01, 3.39988E+01, 1.3324E+01, 7.0773E+03, &
      4.7099246E-01, 3.38336E+01, 1.3327E+01, 7.0539E+03, &
      4.7167755E-01, 3.32805E+01, 1.3329E+01, 7.0443E+03, &
      4.7240756E-01, 3.38619E+01, 1.3846E+01, 7.3065E+03, &
      4.7403594E-01, 3.40908E+01, 1.3850E+01, 7.2833E+03, &
      4.7640612E-01, 3.42537E+01, 1.3855E+01, 7.2497E+03, &
      4.7878815E-01, 3.43692E+01, 1.3860E+01, 7.2163E+03, &
      4.8118209E-01, 3.44655E+01, 1.3865E+01, 7.1830E+03, &
      4.8358800E-01, 3.45515E+01, 1.3869E+01, 7.1497E+03, &
      4.8600594E-01, 3.46309E+01, 1.3874E+01, 7.1166E+03, &
      4.8843597E-01, 3.47059E+01, 1.3878E+01, 7.0835E+03, &
      4.9087815E-01, 3.47778E+01, 1.3883E+01, 7.0505E+03, &
      4.9333254E-01, 3.48473E+01, 1.3887E+01, 7.0175E+03, &
      4.9579920E-01, 3.49149E+01, 1.3891E+01, 6.9847E+03, &
      4.9827820E-01, 3.49811E+01, 1.3895E+01, 6.9519E+03, &
      5.0076959E-01, 3.50462E+01, 1.3898E+01, 6.9191E+03, &
      5.0327344E-01, 3.51103E+01, 1.3902E+01, 6.8865E+03, &
      5.0578980E-01, 3.51736E+01, 1.3905E+01, 6.8539E+03, &
      5.0831875E-01, 3.52363E+01, 1.3908E+01, 6.8213E+03, &
      5.1086035E-01, 3.52985E+01, 1.3911E+01, 6.7888E+03, &
      5.1341465E-01, 3.53602E+01, 1.3913E+01, 6.7563E+03, &
      5.1598172E-01, 3.54215E+01, 1.3916E+01, 6.7239E+03, &
      5.1856163E-01, 3.54825E+01, 1.3918E+01, 6.6915E+03, &
      5.2115444E-01, 3.55433E+01, 1.3920E+01, 6.6592E+03, &
      5.2376021E-01, 3.56038E+01, 1.3921E+01, 6.6268E+03, &
      5.2637901E-01, 3.56641E+01, 1.3923E+01, 6.5945E+03, &
      5.2901091E-01, 3.57242E+01, 1.3924E+01, 6.5623E+03, &
      5.3165596E-01, 3.57842E+01, 1.3924E+01, 6.5300E+03, &
      5.3431424E-01, 3.58441E+01, 1.3925E+01, 6.4977E+03, &
      5.3698581E-01, 3.59038E+01, 1.3925E+01, 6.4655E+03, &
      5.3967074E-01, 3.60000E+01, 1.3924E+01, 6.4333E+03, &
      5.4236910E-01, 3.60596E+01, 1.3924E+01, 6.4011E+03, &
      5.4508094E-01, 3.61191E+01, 1.3923E+01, 6.3689E+03, &
      5.4780635E-01, 3.61785E+01, 1.3922E+01, 6.3367E+03, &
      5.5054538E-01, 3.62378E+01, 1.3920E+01, 6.3045E+03, &
      5.5329810E-01, 3.62970E+01, 1.3918E+01, 6.2723E+03, &
      5.5606460E-01, 3.63562E+01, 1.3916E+01, 6.2402E+03, &
      5.5884492E-01, 3.64152E+01, 1.3913E+01, 6.2080E+03, &
      5.6163914E-01, 3.64742E+01, 1.3910E+01, 6.1759E+03, &
      5.6444734E-01, 3.65332E+01, 1.3907E+01, 6.1437E+03, &
      5.6726958E-01, 3.65920E+01, 1.3903E+01, 6.1116E+03, &
      5.7010592E-01, 3.66508E+01, 1.3899E+01, 6.0794E+03, &
      5.7295645E-01, 3.67095E+01, 1.3895E+01, 6.0473E+03, &
      5.7582123E-01, 3.67681E+01, 1.3890E+01, 6.0152E+03, &
      5.7870034E-01, 3.68266E+01, 1.3885E+01, 5.9830E+03, &
      5.8159384E-01, 3.68850E+01, 1.3879E+01, 5.9509E+03, &
      5.8450181E-01, 3.69433E+01, 1.3873E+01, 5.9188E+03, &
      5.8742432E-01, 3.70015E+01, 1.3867E+01, 5.8867E+03, &
      5.9036144E-01, 3.70597E+01, 1.3860E+01, 5.8546E+03, &
      5.9331325E-01, 3.71177E+01, 1.3852E+01, 5.8225E+03, &
      5.9627982E-01, 3.71756E+01, 1.3845E+01, 5.7904E+03, &
      5.9926122E-01, 3.72334E+01, 1.3837E+01, 5.7583E+03, &
      6.0225752E-01, 3.72910E+01, 1.3828E+01, 5.7262E+03, &
      6.0526881E-01, 3.73485E+01, 1.3820E+01, 5.6941E+03, &
      6.0829515E-01, 3.74059E+01, 1.3810E+01, 5.6621E+03, &
      6.1133663E-01, 3.74632E+01, 1.3800E+01, 5.6300E+03, &
      6.1439331E-01, 3.75202E+01, 1.3790E+01, 5.5979E+03, &
      6.1746528E-01, 3.75772E+01, 1.3780E+01, 5.5658E+03, &
      6.2055260E-01, 3.76339E+01, 1.3769E+01, 5.5338E+03, &
      6.2365537E-01, 3.76905E+01, 1.3757E+01, 5.5017E+03, &
      6.2677364E-01, 3.77469E+01, 1.3745E+01, 5.4697E+03, &
      6.2990751E-01, 3.78031E+01, 1.3733E+01, 5.4377E+03, &
      6.3305705E-01, 3.78591E+01, 1.3720E+01, 5.4057E+03, &
      6.3622234E-01, 3.79149E+01, 1.3707E+01, 5.3737E+03, &
      6.3940345E-01, 3.79704E+01, 1.3694E+01, 5.3418E+03, &
      6.4260046E-01, 3.80258E+01, 1.3680E+01, 5.3099E+03, &
      6.4581347E-01, 3.80809E+01, 1.3666E+01, 5.2780E+03, &
      6.4904253E-01, 3.81358E+01, 1.3651E+01, 5.2461E+03, &
      6.5228775E-01, 3.81905E+01, 1.3636E+01, 5.2143E+03, &
      6.5554919E-01, 3.82449E+01, 1.3620E+01, 5.1825E+03, &
      6.5882693E-01, 3.82990E+01, 1.3604E+01, 5.1507E+03, &
      6.6212107E-01, 3.83529E+01, 1.3588E+01, 5.1190E+03, &
      6.6543167E-01, 3.84065E+01, 1.3571E+01, 5.0873E+03, &
      6.6875883E-01, 3.84599E+01, 1.3554E+01, 5.0556E+03, &
      6.7210262E-01, 3.85129E+01, 1.3536E+01, 5.0240E+03, &
      6.7546314E-01, 3.85656E+01, 1.3518E+01, 4.9924E+03, &
      6.7884045E-01, 3.86181E+01, 1.3499E+01, 4.9609E+03, &
      6.8223466E-01, 3.86702E+01, 1.3481E+01, 4.9294E+03, &
      6.8564583E-01, 3.87220E+01, 1.3461E+01, 4.8979E+03, &
      6.8907406E-01, 3.87735E+01, 1.3442E+01, 4.8665E+03, &
      6.9251943E-01, 3.88247E+01, 1.3422E+01, 4.8351E+03, &
      6.9598202E-01, 3.88755E+01, 1.3401E+01, 4.8038E+03, &
      6.9946194E-01, 3.89260E+01, 1.3380E+01, 4.7726E+03, &
      7.0295924E-01, 3.89761E+01, 1.3359E+01, 4.7414E+03, &
      7.0647404E-01, 3.90259E+01, 1.3338E+01, 4.7103E+03, &
      7.1000641E-01, 3.90753E+01, 1.3316E+01, 4.6792E+03, &
      7.1355644E-01, 3.91244E+01, 1.3293E+01, 4.6482E+03, &
      7.1712423E-01, 3.91731E+01, 1.3271E+01, 4.6173E+03, &
      7.2070985E-01, 3.92214E+01, 1.3248E+01, 4.5864E+03, &
      7.2431340E-01, 3.92693E+01, 1.3224E+01, 4.5555E+03, &
      7.2793496E-01, 3.93168E+01, 1.3200E+01, 4.5247E+03, &
      7.3157464E-01, 3.93638E+01, 1.3176E+01, 4.4940E+03, &
      7.3523251E-01, 3.94105E+01, 1.3151E+01, 4.4633E+03, &
      7.3890867E-01, 3.94566E+01, 1.3126E+01, 4.4326E+03, &
      7.4260322E-01, 3.95024E+01, 1.3100E+01, 4.4020E+03, &
      7.4631623E-01, 3.95476E+01, 1.3074E+01, 4.3714E+03, &
      7.5004781E-01, 3.95924E+01, 1.3047E+01, 4.3409E+03, &
      7.5379805E-01, 3.96366E+01, 1.3021E+01, 4.3105E+03, &
      7.5756704E-01, 3.96804E+01, 1.2993E+01, 4.2802E+03, &
      7.6135488E-01, 3.97236E+01, 1.2966E+01, 4.2499E+03, &
      7.6516165E-01, 3.97663E+01, 1.2938E+01, 4.2197E+03, &
      7.6898746E-01, 3.98085E+01, 1.2909E+01, 4.1896E+03, &
      7.7283240E-01, 3.98502E+01, 1.2881E+01, 4.1596E+03, &
      7.7669656E-01, 3.98913E+01, 1.2852E+01, 4.1297E+03, &
      7.8058004E-01, 3.99319E+01, 1.2822E+01, 4.0998E+03, &
      7.8448294E-01, 3.99719E+01, 1.2793E+01, 4.0701E+03, &
      7.8840536E-01, 4.00114E+01, 1.2763E+01, 4.0404E+03, &
      7.9234738E-01, 4.00503E+01, 1.2733E+01, 4.0109E+03, &
      7.9630912E-01, 4.00886E+01, 1.2702E+01, 3.9815E+03, &
      8.0029067E-01, 4.01264E+01, 1.2671E+01, 3.9521E+03, &
      8.0429212E-01, 4.01635E+01, 1.2640E+01, 3.9227E+03, &
      8.0831358E-01, 4.02000E+01, 1.2608E+01, 3.8935E+03, &
      8.1235515E-01, 4.02359E+01, 1.2576E+01, 3.8644E+03, &
      8.1641693E-01, 4.02711E+01, 1.2544E+01, 3.8354E+03, &
      8.2049901E-01, 4.03058E+01, 1.2511E+01, 3.8066E+03, &
      8.2460150E-01, 4.03397E+01, 1.2479E+01, 3.7778E+03, &
      8.2872451E-01, 4.03731E+01, 1.2446E+01, 3.7492E+03, &
      8.3286813E-01, 4.04058E+01, 1.2413E+01, 3.7207E+03, &
      8.3703248E-01, 4.04379E+01, 1.2380E+01, 3.6924E+03, &
      8.4121764E-01, 4.04694E+01, 1.2346E+01, 3.6642E+03, &
      8.4542373E-01, 4.05002E+01, 1.2312E+01, 3.6361E+03, &
      8.4965084E-01, 4.05304E+01, 1.2279E+01, 3.6082E+03, &
      8.5389910E-01, 4.05600E+01, 1.2245E+01, 3.5803E+03, &
      8.5816859E-01, 4.05890E+01, 1.2211E+01, 3.5527E+03, &
      8.6245944E-01, 4.06174E+01, 1.2176E+01, 3.5252E+03, &
      8.6677173E-01, 4.06451E+01, 1.2142E+01, 3.4978E+03, &
      8.7110559E-01, 4.06722E+01, 1.2107E+01, 3.4706E+03, &
      8.7546112E-01, 4.06988E+01, 1.2073E+01, 3.4435E+03, &
      8.7983843E-01, 4.07247E+01, 1.2038E+01, 3.4166E+03, &
      8.8423762E-01, 4.07501E+01, 1.2003E+01, 3.3898E+03, &
      8.8865881E-01, 4.07749E+01, 1.1968E+01, 3.3632E+03, &
      8.9310210E-01, 4.07991E+01, 1.1933E+01, 3.3367E+03, &
      8.9756761E-01, 4.08227E+01, 1.1897E+01, 3.3104E+03, &
      9.0205545E-01, 4.08459E+01, 1.1862E+01, 3.2842E+03, &
      9.0656573E-01, 4.08685E+01, 1.1827E+01, 3.2582E+03, &
      9.1109856E-01, 4.08906E+01, 1.1791E+01, 3.2324E+03, &
      9.1565405E-01, 4.09122E+01, 1.1756E+01, 3.2067E+03, &
      9.2023232E-01, 4.09334E+01, 1.1720E+01, 3.1812E+03, &
      9.2483348E-01, 4.09541E+01, 1.1685E+01, 3.1559E+03, &
      9.2945765E-01, 4.09744E+01, 1.1649E+01, 3.1307E+03, &
      9.3410494E-01, 4.09943E+01, 1.1613E+01, 3.1056E+03, &
      9.3877546E-01, 4.10139E+01, 1.1578E+01, 3.0808E+03, &
      9.4346934E-01, 4.10331E+01, 1.1542E+01, 3.0561E+03, &
      9.4818668E-01, 4.10521E+01, 1.1506E+01, 3.0315E+03, &
      9.5292762E-01, 4.10708E+01, 1.1470E+01, 3.0072E+03, &
      9.5769226E-01, 4.10894E+01, 1.1434E+01, 2.9830E+03, &
      9.6248072E-01, 4.11079E+01, 1.1399E+01, 2.9589E+03, &
      9.6729312E-01, 4.11263E+01, 1.1363E+01, 2.9350E+03, &
      9.7212959E-01, 4.11448E+01, 1.1327E+01, 2.9113E+03, &
      9.7699023E-01, 4.11634E+01, 1.1291E+01, 2.8878E+03, &
      9.8187519E-01, 4.11822E+01, 1.1256E+01, 2.8644E+03, &
      9.8678456E-01, 4.12014E+01, 1.1220E+01, 2.8412E+03, &
      9.9171848E-01, 4.12210E+01, 1.1184E+01, 2.8182E+03, &
      9.9667708E-01, 4.12412E+01, 1.1149E+01, 2.7953E+03, &
      1.0016605E+00, 4.12515E+01, 1.1102E+01, 2.7699E+03, &
      1.0066688E+00, 4.12383E+01, 1.1034E+01, 2.7392E+03, &
      1.0117021E+00, 4.12230E+01, 1.0966E+01, 2.7089E+03, &
      1.0167606E+00, 4.12037E+01, 1.0898E+01, 2.6790E+03, &
      1.0218444E+00, 4.11816E+01, 1.0831E+01, 2.6495E+03, &
      1.0269536E+00, 4.11566E+01, 1.0765E+01, 2.6202E+03, &
      1.0320884E+00, 4.11288E+01, 1.0699E+01, 2.5914E+03, &
      1.0372489E+00, 4.10983E+01, 1.0634E+01, 2.5629E+03, &
      1.0424351E+00, 4.10651E+01, 1.0569E+01, 2.5347E+03, &
      1.0476473E+00, 4.10293E+01, 1.0505E+01, 2.5069E+03, &
      1.0528855E+00, 4.09908E+01, 1.0441E+01, 2.4794E+03, &
      1.0581499E+00, 4.09497E+01, 1.0378E+01, 2.4522E+03, &
      1.0634407E+00, 4.09059E+01, 1.0315E+01, 2.4253E+03, &
      1.0687579E+00, 4.08595E+01, 1.0253E+01, 2.3988E+03, &
      1.0741017E+00, 4.08104E+01, 1.0191E+01, 2.3726E+03, &
      1.0794722E+00, 4.07587E+01, 1.0129E+01, 2.3467E+03, &
      1.0848695E+00, 4.07042E+01, 1.0068E+01, 2.3211E+03, &
      1.0902939E+00, 4.06470E+01, 1.0008E+01, 2.2958E+03, &
      1.0957454E+00, 4.05871E+01, 9.9479E+00, 2.2708E+03, &
      1.1012241E+00, 4.05243E+01, 9.8884E+00, 2.2460E+03, &
      1.1067302E+00, 4.04586E+01, 9.8293E+00, 2.2216E+03, &
      1.1122639E+00, 4.03901E+01, 9.7706E+00, 2.1975E+03, &
      1.1178252E+00, 4.03185E+01, 9.7124E+00, 2.1737E+03, &
      1.1234143E+00, 4.02439E+01, 9.6547E+00, 2.1501E+03, &
      1.1290314E+00, 4.01661E+01, 9.5973E+00, 2.1268E+03, &
      1.1346765E+00, 4.00851E+01, 9.5404E+00, 2.1038E+03, &
      1.1403499E+00, 4.00008E+01, 9.4840E+00, 2.0811E+03, &
      1.1460517E+00, 3.99131E+01, 9.4279E+00, 2.0586E+03, &
      1.1517819E+00, 3.98219E+01, 9.3723E+00, 2.0364E+03, &
      1.1575408E+00, 3.97271E+01, 9.3171E+00, 2.0144E+03, &
      1.1633285E+00, 3.96269E+01, 9.2623E+00, 1.9927E+03, &
      1.1691452E+00, 3.95244E+01, 9.2080E+00, 1.9713E+03, &
      1.1749909E+00, 3.94179E+01, 9.1540E+00, 1.9501E+03, &
      1.1808659E+00, 3.93073E+01, 9.1004E+00, 1.9292E+03, &
      1.1867702E+00, 3.91922E+01, 9.0473E+00, 1.9085E+03, &
      1.1927040E+00, 3.90726E+01, 8.9945E+00, 1.8880E+03, &
      1.1986676E+00, 3.89483E+01, 8.9421E+00, 1.8678E+03, &
      1.2046609E+00, 3.88190E+01, 8.8901E+00, 1.8478E+03, &
      1.2106842E+00, 3.86846E+01, 8.8385E+00, 1.8281E+03, &
      1.2167376E+00, 3.85447E+01, 8.7873E+00, 1.8086E+03, &
      1.2228213E+00, 3.83992E+01, 8.7365E+00, 1.7893E+03, &
      1.2289354E+00, 3.82476E+01, 8.6860E+00, 1.7702E+03, &
      1.2350801E+00, 3.80898E+01, 8.6359E+00, 1.7513E+03, &
      1.2412555E+00, 3.79253E+01, 8.5862E+00, 1.7327E+03, &
      1.2474618E+00, 3.77537E+01, 8.5369E+00, 1.7143E+03, &
      1.2536991E+00, 3.75747E+01, 8.4879E+00, 1.6961E+03, &
      1.2599676E+00, 3.73878E+01, 8.4393E+00, 1.6781E+03, &
      1.2662674E+00, 3.71925E+01, 8.3910E+00, 1.6603E+03, &
      1.2725988E+00, 3.69883E+01, 8.3431E+00, 1.6427E+03, &
      1.2789618E+00, 3.67744E+01, 8.2955E+00, 1.6254E+03, &
      1.2853566E+00, 3.65503E+01, 8.2483E+00, 1.6082E+03, &
      1.2917833E+00, 3.63152E+01, 8.2015E+00, 1.5912E+03, &
      1.2982423E+00, 3.60682E+01, 8.1550E+00, 1.5744E+03, &
      1.3047335E+00, 3.58083E+01, 8.1088E+00, 1.5578E+03, &
      1.3112571E+00, 3.55346E+01, 8.0630E+00, 1.5414E+03, &
      1.3178134E+00, 3.52458E+01, 8.0175E+00, 1.5252E+03, &
      1.3244025E+00, 3.49405E+01, 7.9723E+00, 1.5092E+03, &
      1.3310245E+00, 3.46172E+01, 7.9275E+00, 1.4934E+03, &
      1.3376796E+00, 3.42739E+01, 7.8830E+00, 1.4777E+03, &
      1.3443680E+00, 3.39086E+01, 7.8388E+00, 1.4622E+03, &
      1.3510899E+00, 3.35188E+01, 7.7937E+00, 1.4467E+03, &
      1.3578453E+00, 3.31015E+01, 7.7484E+00, 1.4312E+03, &
      1.3646345E+00, 3.26531E+01, 7.7035E+00, 1.4160E+03, &
      1.3714577E+00, 3.21695E+01, 7.6589E+00, 1.4009E+03, &
      1.3783150E+00, 3.16455E+01, 7.6146E+00, 1.3860E+03, &
      1.3852066E+00, 3.10748E+01, 7.5707E+00, 1.3712E+03, &
      1.3921326E+00, 3.04494E+01, 7.5271E+00, 1.3567E+03, &
      1.3990933E+00, 2.97590E+01, 7.4837E+00, 1.3423E+03, &
      1.4060887E+00, 2.89902E+01, 7.4408E+00, 1.3280E+03, &
      1.4131192E+00, 2.81245E+01, 7.3981E+00, 1.3139E+03, &
      1.4201848E+00, 2.71371E+01, 7.3557E+00, 1.3000E+03, &
      1.4272857E+00, 2.59882E+01, 7.3137E+00, 1.2863E+03, &
      1.4344221E+00, 2.46188E+01, 7.2719E+00, 1.2727E+03, &
      1.4415942E+00, 2.29259E+01, 7.2305E+00, 1.2593E+03, &
      1.4488022E+00, 2.07073E+01, 7.1894E+00, 1.2460E+03, &
      1.4560462E+00, 1.74624E+01, 7.1485E+00, 1.2328E+03, &
      1.4633265E+00, 1.11020E+01, 7.1080E+00, 1.2199E+03, &
      1.4674798E+00, -7.62740E+00, 7.0851E+00, 1.2126E+03, &
      1.4679202E+00, -7.97086E+00, 2.6608E+01, 4.5251E+03, &
      1.4706431E+00, 8.04023E+00, 2.6532E+01, 4.5039E+03, &
      1.4779963E+00, 1.53680E+01, 2.6329E+01, 4.4472E+03, &
      1.4853863E+00, 1.80630E+01, 2.6127E+01, 4.3912E+03, &
      1.4928132E+00, 1.92819E+01, 2.5926E+01, 4.3360E+03, &
      1.5002773E+00, 1.94196E+01, 2.5727E+01, 4.2815E+03, &
      1.5077787E+00, 1.79223E+01, 2.5530E+01, 4.2277E+03, &
      1.5141684E+00, 7.84877E+00, 2.5364E+01, 4.1826E+03, &
      1.5150317E+00, 7.78268E+00, 3.8095E+01, 6.2733E+03, &
      1.5153176E+00, 9.89537E+00, 3.8084E+01, 6.2703E+03, &
      1.5228942E+00, 2.09882E+01, 3.7787E+01, 6.1906E+03, &
      1.5305086E+00, 2.46248E+01, 3.7493E+01, 6.1120E+03, &
      1.5381612E+00, 2.70972E+01, 3.7201E+01, 6.0344E+03, &
      1.5458520E+00, 2.90321E+01, 3.6912E+01, 5.9578E+03, &
      1.5535812E+00, 3.06439E+01, 3.6624E+01, 5.8820E+03, &
      1.5613491E+00, 3.20345E+01, 3.6338E+01, 5.8073E+03, &
      1.5691559E+00, 3.32616E+01, 3.6055E+01, 5.7335E+03, &
      1.5770017E+00, 3.43615E+01, 3.5774E+01, 5.6607E+03, &
      1.5848867E+00, 3.53588E+01, 3.5496E+01, 5.5888E+03, &
      1.5928111E+00, 3.62710E+01, 3.5219E+01, 5.5178E+03, &
      1.6007752E+00, 3.71112E+01, 3.4945E+01, 5.4478E+03, &
      1.6087790E+00, 3.78892E+01, 3.4673E+01, 5.3786E+03, &
      1.6168229E+00, 3.86130E+01, 3.4404E+01, 5.3104E+03, &
      1.6249070E+00, 3.92887E+01, 3.4136E+01, 5.2430E+03, &
      1.6330316E+00, 3.99216E+01, 3.3871E+01, 5.1765E+03, &
      1.6411967E+00, 4.05157E+01, 3.3608E+01, 5.1109E+03, &
      1.6494027E+00, 4.10746E+01, 3.3347E+01, 5.0461E+03, &
      1.6576497E+00, 4.16013E+01, 3.3088E+01, 4.9821E+03, &
      1.6659380E+00, 4.20982E+01, 3.2831E+01, 4.9190E+03, &
      1.6742677E+00, 4.25674E+01, 3.2577E+01, 4.8567E+03, &
      1.6826390E+00, 4.30107E+01, 3.2324E+01, 4.7952E+03, &
      1.6910522E+00, 4.34297E+01, 3.2073E+01, 4.7344E+03, &
      1.6995075E+00, 4.38256E+01, 3.1825E+01, 4.6745E+03, &
      1.7080050E+00, 4.41996E+01, 3.1578E+01, 4.6154E+03, &
      1.7165450E+00, 4.45524E+01, 3.1334E+01, 4.5570E+03, &
      1.7251278E+00, 4.48848E+01, 3.1091E+01, 4.4993E+03, &
      1.7337534E+00, 4.51973E+01, 3.0851E+01, 4.4424E+03, &
      1.7424222E+00, 4.54903E+01, 3.0612E+01, 4.3862E+03, &
      1.7511343E+00, 4.57638E+01, 3.0375E+01, 4.3308E+03, &
      1.7598899E+00, 4.60179E+01, 3.0140E+01, 4.2760E+03, &
      1.7686894E+00, 4.62520E+01, 2.9907E+01, 4.2220E+03, &
      1.7775328E+00, 4.64657E+01, 2.9676E+01, 4.1687E+03, &
      1.7864205E+00, 4.66579E+01, 2.9447E+01, 4.1160E+03, &
      1.7953526E+00, 4.68271E+01, 2.9220E+01, 4.0641E+03, &
      1.8043294E+00, 4.69709E+01, 2.8994E+01, 4.0128E+03, &
      1.8133510E+00, 4.70860E+01, 2.8771E+01, 3.9621E+03, &
      1.8224178E+00, 4.71676E+01, 2.8549E+01, 3.9121E+03, &
      1.8315299E+00, 4.72079E+01, 2.8329E+01, 3.8628E+03, &
      1.8406875E+00, 4.71947E+01, 2.8110E+01, 3.8141E+03, &
      1.8498909E+00, 4.71065E+01, 2.7894E+01, 3.7660E+03, &
      1.8591404E+00, 4.69006E+01, 2.7679E+01, 3.7185E+03, &
      1.8684361E+00, 4.64731E+01, 2.7466E+01, 3.6717E+03, &
      1.8777783E+00, 4.54244E+01, 2.7255E+01, 3.6254E+03, &
      1.8824082E+00, 4.38040E+01, 2.7151E+01, 3.6028E+03, &
      1.8865918E+00, 4.38500E+01, 3.1766E+01, 4.2042E+03, &
      1.8871672E+00, 4.42318E+01, 3.1750E+01, 4.2008E+03, &
      1.8966030E+00, 4.67542E+01, 3.1488E+01, 4.1455E+03, &
      1.9060860E+00, 4.78491E+01, 3.1229E+01, 4.0911E+03, &
      1.9156165E+00, 4.86072E+01, 3.0971E+01, 4.0372E+03, &
      1.9251945E+00, 4.92042E+01, 3.0714E+01, 3.9840E+03, &
      1.9348205E+00, 4.97033E+01, 3.0460E+01, 3.9315E+03, &
      1.9444946E+00, 5.01343E+01, 3.0209E+01, 3.8798E+03, &
      1.9542171E+00, 5.05134E+01, 2.9959E+01, 3.8287E+03, &
      1.9639882E+00, 5.08498E+01, 2.9712E+01, 3.7783E+03, &
      1.9738081E+00, 5.11481E+01, 2.9466E+01, 3.7285E+03, &
      1.9836772E+00, 5.14118E+01, 2.9235E+01, 3.6810E+03, &
      1.9935955E+00, 5.16542E+01, 2.9014E+01, 3.6352E+03, &
      2.0035635E+00, 5.18756E+01, 2.8797E+01, 3.5902E+03, &
      2.0135813E+00, 5.20754E+01, 2.8583E+01, 3.5458E+03, &
      2.0236492E+00, 5.22523E+01, 2.8371E+01, 3.5022E+03, &
      2.0337675E+00, 5.24038E+01, 2.8162E+01, 3.4592E+03, &
      2.0439363E+00, 5.25249E+01, 2.7956E+01, 3.4169E+03, &
      2.0541560E+00, 5.26060E+01, 2.7753E+01, 3.3753E+03, &
      2.0644268E+00, 5.26266E+01, 2.7552E+01, 3.3343E+03, &
      2.0747489E+00, 5.25309E+01, 2.7353E+01, 3.2939E+03, &
      2.0851227E+00, 5.20407E+01, 2.7157E+01, 3.2541E+03, &
      2.0874384E+00, 5.16756E+01, 2.7113E+01, 3.2454E+03, &
      2.0921614E+00, 5.17443E+01, 2.8924E+01, 3.4537E+03, &
      2.0955483E+00, 5.23530E+01, 2.8854E+01, 3.4397E+03, &
      2.1060260E+00, 5.31829E+01, 2.8638E+01, 3.3972E+03, &
      2.1165562E+00, 5.36739E+01, 2.8425E+01, 3.3552E+03, &
      2.1271389E+00, 5.40531E+01, 2.8214E+01, 3.3139E+03, &
      2.1377746E+00, 5.43726E+01, 2.8006E+01, 3.2731E+03, &
      2.1484635E+00, 5.46529E+01, 2.7800E+01, 3.2330E+03, &
      2.1592058E+00, 5.49043E+01, 2.7596E+01, 3.1934E+03, &
      2.1700018E+00, 5.51323E+01, 2.7394E+01, 3.1544E+03, &
      2.1808519E+00, 5.53400E+01, 2.7194E+01, 3.1159E+03, &
      2.1917561E+00, 5.55284E+01, 2.6996E+01, 3.0780E+03, &
      2.2027149E+00, 5.57004E+01, 2.6811E+01, 3.0418E+03, &
      2.2137285E+00, 5.58613E+01, 2.6629E+01, 3.0062E+03, &
      2.2247971E+00, 5.60099E+01, 2.6450E+01, 2.9712E+03, &
      2.2359211E+00, 5.61451E+01, 2.6273E+01, 2.9367E+03, &
      2.2471007E+00, 5.62648E+01, 2.6098E+01, 2.9028E+03, &
      2.2583362E+00, 5.63654E+01, 2.5925E+01, 2.8693E+03, &
      2.2696279E+00, 5.64399E+01, 2.5754E+01, 2.8363E+03, &
      2.2809760E+00, 5.64730E+01, 2.5585E+01, 2.8038E+03, &
      2.2923809E+00, 5.64203E+01, 2.5418E+01, 2.7718E+03, &
      2.3022094E+00, 5.61306E+01, 2.5276E+01, 2.7446E+03, &
      2.3038428E+00, 5.59885E+01, 2.5253E+01, 2.7402E+03, &
      2.3113904E+00, 5.62497E+01, 2.6325E+01, 2.8467E+03, &
      2.3153620E+00, 5.65389E+01, 2.6267E+01, 2.8356E+03, &
      2.3269388E+00, 5.70159E+01, 2.6100E+01, 2.8037E+03, &
      2.3385735E+00, 5.73383E+01, 2.5934E+01, 2.7722E+03, &
      2.3502664E+00, 5.76033E+01, 2.5770E+01, 2.7410E+03, &
      2.3620177E+00, 5.78370E+01, 2.5608E+01, 2.7103E+03, &
      2.3738278E+00, 5.80503E+01, 2.5447E+01, 2.6800E+03, &
      2.3856970E+00, 5.82491E+01, 2.5288E+01, 2.6500E+03, &
      2.3976254E+00, 5.84366E+01, 2.5129E+01, 2.6204E+03, &
      2.4096136E+00, 5.86151E+01, 2.4972E+01, 2.5912E+03, &
      2.4216616E+00, 5.87858E+01, 2.4816E+01, 2.5623E+03, &
      2.4337699E+00, 5.89506E+01, 2.4663E+01, 2.5339E+03, &
      2.4459388E+00, 5.91110E+01, 2.4511E+01, 2.5059E+03, &
      2.4581685E+00, 5.92668E+01, 2.4357E+01, 2.4778E+03, &
      2.4704593E+00, 5.94173E+01, 2.4202E+01, 2.4499E+03, &
      2.4828116E+00, 5.95631E+01, 2.4048E+01, 2.4224E+03, &
      2.4952257E+00, 5.97047E+01, 2.3896E+01, 2.3951E+03, &
      2.5077018E+00, 5.98424E+01, 2.3744E+01, 2.3682E+03, &
      2.5202403E+00, 5.99765E+01, 2.3594E+01, 2.3416E+03, &
      2.5328415E+00, 6.01072E+01, 2.3445E+01, 2.3153E+03, &
      2.5455057E+00, 6.02349E+01, 2.3297E+01, 2.2893E+03, &
      2.5582333E+00, 6.03597E+01, 2.3150E+01, 2.2636E+03, &
      2.5710244E+00, 6.04817E+01, 2.3003E+01, 2.2383E+03, &
      2.5838796E+00, 6.06011E+01, 2.2858E+01, 2.2132E+03, &
      2.5967990E+00, 6.07181E+01, 2.2714E+01, 2.1884E+03, &
      2.6097829E+00, 6.08327E+01, 2.2571E+01, 2.1638E+03, &
      2.6228319E+00, 6.09452E+01, 2.2428E+01, 2.1396E+03, &
      2.6359460E+00, 6.10555E+01, 2.2287E+01, 2.1156E+03, &
      2.6491257E+00, 6.11638E+01, 2.2146E+01, 2.0919E+03, &
      2.6623714E+00, 6.12702E+01, 2.2007E+01, 2.0684E+03, &
      2.6756832E+00, 6.13748E+01, 2.1868E+01, 2.0453E+03, &
      2.6890617E+00, 6.14777E+01, 2.1730E+01, 2.0223E+03, &
      2.7025070E+00, 6.15790E+01, 2.1592E+01, 1.9996E+03, &
      2.7160195E+00, 6.16788E+01, 2.1456E+01, 1.9772E+03, &
      2.7295996E+00, 6.17768E+01, 2.1318E+01, 1.9549E+03, &
      2.7432476E+00, 6.18728E+01, 2.1181E+01, 1.9327E+03, &
      2.7569638E+00, 6.19669E+01, 2.1045E+01, 1.9109E+03, &
      2.7707486E+00, 6.20592E+01, 2.0910E+01, 1.8892E+03, &
      2.7846024E+00, 6.21499E+01, 2.0776E+01, 1.8678E+03, &
      2.7985254E+00, 6.22390E+01, 2.0642E+01, 1.8467E+03, &
      2.8125180E+00, 6.23265E+01, 2.0509E+01, 1.8258E+03, &
      2.8265806E+00, 6.24126E+01, 2.0377E+01, 1.8051E+03, &
      2.8407135E+00, 6.24974E+01, 2.0246E+01, 1.7847E+03, &
      2.8549171E+00, 6.25808E+01, 2.0116E+01, 1.7645E+03, &
      2.8691917E+00, 6.26630E+01, 1.9987E+01, 1.7445E+03, &
      2.8835376E+00, 6.27441E+01, 1.9858E+01, 1.7247E+03, &
      2.8979553E+00, 6.28241E+01, 1.9731E+01, 1.7052E+03, &
      2.9124451E+00, 6.29032E+01, 1.9604E+01, 1.6859E+03, &
      2.9270073E+00, 6.29813E+01, 1.9478E+01, 1.6669E+03, &
      2.9416424E+00, 6.32813E+01, 1.9352E+01, 1.6479E+03, &
      2.9563506E+00, 6.33587E+01, 1.9225E+01, 1.6290E+03, &
      2.9711323E+00, 6.34355E+01, 1.9098E+01, 1.6103E+03, &
      2.9859880E+00, 6.35120E+01, 1.8973E+01, 1.5919E+03, &
      3.0009179E+00, 6.35908E+01, 1.8847E+01, 1.5736E+03, &
      3.0159225E+00, 6.36825E+01, 1.8704E+01, 1.5539E+03, &
      3.0310021E+00, 6.39106E+01, 1.8561E+01, 1.5345E+03, &
      3.0461571E+00, 6.39821E+01, 1.8416E+01, 1.5150E+03, &
      3.0613879E+00, 6.40500E+01, 1.8271E+01, 1.4958E+03, &
      3.0766949E+00, 6.41148E+01, 1.8129E+01, 1.4768E+03, &
      3.0920783E+00, 6.41769E+01, 1.7987E+01, 1.4581E+03, &
      3.1075387E+00, 6.42364E+01, 1.7847E+01, 1.4396E+03, &
      3.1230764E+00, 6.42936E+01, 1.7708E+01, 1.4214E+03, &
      3.1386918E+00, 6.43486E+01, 1.7570E+01, 1.4034E+03, &
      3.1543853E+00, 6.44016E+01, 1.7434E+01, 1.3857E+03, &
      3.1701572E+00, 6.44526E+01, 1.7299E+01, 1.3682E+03, &
      3.1860080E+00, 6.45018E+01, 1.7165E+01, 1.3510E+03, &
      3.2019380E+00, 6.45492E+01, 1.7032E+01, 1.3340E+03, &
      3.2179477E+00, 6.45949E+01, 1.6901E+01, 1.3172E+03, &
      3.2340374E+00, 6.46390E+01, 1.6770E+01, 1.3006E+03, &
      3.2502076E+00, 6.46816E+01, 1.6641E+01, 1.2843E+03, &
      3.2664587E+00, 6.47227E+01, 1.6513E+01, 1.2682E+03, &
      3.2827910E+00, 6.47624E+01, 1.6387E+01, 1.2523E+03, &
      3.2992049E+00, 6.48007E+01, 1.6261E+01, 1.2366E+03, &
      3.3157009E+00, 6.48378E+01, 1.6137E+01, 1.2211E+03, &
      3.3322794E+00, 6.48736E+01, 1.6013E+01, 1.2058E+03, &
      3.3489408E+00, 6.49081E+01, 1.5891E+01, 1.1908E+03, &
      3.3656856E+00, 6.49416E+01, 1.5770E+01, 1.1759E+03, &
      3.3825140E+00, 6.49739E+01, 1.5648E+01, 1.1611E+03, &
      3.3994265E+00, 6.50051E+01, 1.5527E+01, 1.1465E+03, &
      3.4164237E+00, 6.50351E+01, 1.5407E+01, 1.1321E+03, &
      3.4335058E+00, 6.50641E+01, 1.5288E+01, 1.1179E+03, &
      3.4506733E+00, 6.50920E+01, 1.5171E+01, 1.1038E+03, &
      3.4679267E+00, 6.51188E+01, 1.5054E+01, 1.0900E+03, &
      3.4852663E+00, 6.51447E+01, 1.4938E+01, 1.0763E+03, &
      3.5026927E+00, 6.51696E+01, 1.4824E+01, 1.0628E+03, &
      3.5202061E+00, 6.51936E+01, 1.4710E+01, 1.0495E+03, &
      3.5378072E+00, 6.52168E+01, 1.4597E+01, 1.0364E+03, &
      3.5554962E+00, 6.52390E+01, 1.4486E+01, 1.0235E+03, &
      3.5732737E+00, 6.52605E+01, 1.4375E+01, 1.0107E+03, &
      3.5911400E+00, 6.52811E+01, 1.4266E+01, 9.9809E+02, &
      3.6090957E+00, 6.53010E+01, 1.4157E+01, 9.8566E+02, &
      3.6271412E+00, 6.53201E+01, 1.4050E+01, 9.7339E+02, &
      3.6452769E+00, 6.53386E+01, 1.3943E+01, 9.6130E+02, &
      3.6635033E+00, 6.53563E+01, 1.3838E+01, 9.4936E+02, &
      3.6818208E+00, 6.53734E+01, 1.3733E+01, 9.3758E+02, &
      3.7002299E+00, 6.53899E+01, 1.3630E+01, 9.2596E+02, &
      3.7187311E+00, 6.54057E+01, 1.3527E+01, 9.1449E+02, &
      3.7373247E+00, 6.54210E+01, 1.3425E+01, 9.0318E+02, &
      3.7560114E+00, 6.54358E+01, 1.3324E+01, 8.9202E+02, &
      3.7747914E+00, 6.55572E+01, 1.3223E+01, 8.8092E+02, &
      3.7936654E+00, 6.55716E+01, 1.3119E+01, 8.6977E+02, &
      3.8126337E+00, 6.55850E+01, 1.3017E+01, 8.5878E+02, &
      3.8316969E+00, 6.55975E+01, 1.2916E+01, 8.4793E+02, &
      3.8508554E+00, 6.56091E+01, 1.2815E+01, 8.3723E+02, &
      3.8701096E+00, 6.56199E+01, 1.2715E+01, 8.2667E+02, &
      3.8894602E+00, 6.56299E+01, 1.2617E+01, 8.1625E+02, &
      3.9089075E+00, 6.56392E+01, 1.2519E+01, 8.0598E+02, &
      3.9284520E+00, 6.56478E+01, 1.2422E+01, 7.9584E+02, &
      3.9480943E+00, 6.56557E+01, 1.2326E+01, 7.8583E+02, &
      3.9678347E+00, 6.56629E+01, 1.2230E+01, 7.7596E+02, &
      3.9876739E+00, 6.56695E+01, 1.2136E+01, 7.6623E+02, &
      4.055024E+00, 6.61088E+01, 1.1825E+01, 7.3450E+02, &
      4.334821E+00, 6.61107E+01, 1.0658E+01, 6.2025E+02, &
      4.633924E+00, 6.60094E+01, 9.6063E+00, 5.2393E+02, &
      4.953664E+00, 6.58258E+01, 8.6601E+00, 4.4273E+02, &
      5.295467E+00, 6.55870E+01, 7.8175E+00, 3.7469E+02, &
      5.660855E+00, 6.52974E+01, 7.0662E+00, 3.1759E+02, &
      6.051453E+00, 6.49529E+01, 6.3955E+00, 2.6961E+02, &
      6.469004E+00, 6.45390E+01, 5.7885E+00, 2.2895E+02, &
      6.915365E+00, 6.40048E+01, 5.2191E+00, 1.9375E+02, &
      7.392525E+00, 6.32653E+01, 4.7110E+00, 1.6420E+02, &
      7.902609E+00, 6.21040E+01, 4.2584E+00, 1.3940E+02, &
      8.447890E+00, 5.92839E+01, 3.8477E+00, 1.1833E+02, &
      8.475040E+00, 5.89756E+01, 3.8287E+00, 1.1740E+02, &
      8.604760E+00, 5.60429E+01, 3.7405E+00, 1.1308E+02, &
      8.639352E+00, 5.25930E+01, 3.7176E+00, 1.1198E+02, &
      8.691240E+00, 5.60824E+01, 1.0419E+01, 3.0336E+02, &
      8.820960E+00, 5.90778E+01, 9.9983E+00, 2.8704E+02, &
      9.030794E+00, 6.03859E+01, 9.5496E+00, 2.6800E+02, &
      9.424561E+00, 6.07158E+01, 9.0035E+00, 2.4239E+02, &
      9.568814E+00, 5.95927E+01, 8.8174E+00, 2.3389E+02, &
      9.607282E+00, 5.79397E+01, 8.7676E+00, 2.3167E+02, &
      9.653919E+00, 5.94809E+01, 1.2105E+01, 3.1668E+02, &
      9.664983E+00, 5.97910E+01, 1.2082E+01, 3.1570E+02, &
      9.809237E+00, 6.15257E+01, 1.1774E+01, 3.0325E+02, &
      9.913386E+00, 6.20369E+01, 1.1552E+01, 2.9449E+02, &
      1.006512E+01, 6.20671E+01, 1.1228E+01, 2.8202E+02, &
      1.010558E+01, 6.13640E+01, 1.1143E+01, 2.7880E+02, &
      1.016628E+01, 6.24607E+01, 1.2729E+01, 3.1601E+02, &
      1.032004E+01, 6.37534E+01, 1.2408E+01, 3.0355E+02, &
      1.103212E+01, 6.61062E+01, 1.0996E+01, 2.5209E+02, &
      1.179334E+01, 6.72118E+01, 9.8298E+00, 2.1118E+02, &
      1.260708E+01, 6.79569E+01, 8.8364E+00, 1.7791E+02, &
      1.347697E+01, 6.84675E+01, 7.9243E+00, 1.4956E+02, &
      1.440688E+01, 6.88093E+01, 7.1023E+00, 1.2568E+02, &
      1.540095E+01, 6.90350E+01, 6.3661E+00, 1.0564E+02, &
      1.646362E+01, 6.91791E+01, 5.7080E+00, 8.8843E+01, &
      1.759961E+01, 6.93315E+01, 5.1161E+00, 7.4708E+01, &
      1.881398E+01, 6.93678E+01, 4.5777E+00, 6.2731E+01, &
      2.011215E+01, 6.94030E+01, 4.0962E+00, 5.2693E+01, &
      2.149988E+01, 6.94058E+01, 3.6598E+00, 4.4211E+01, &
      2.298338E+01, 6.93572E+01, 3.2721E+00, 3.7129E+01, &
      2.456923E+01, 6.92964E+01, 2.9280E+00, 3.1219E+01, &
      2.626450E+01, 6.92330E+01, 2.6224E+00, 2.6280E+01, &
      2.807676E+01, 6.91640E+01, 2.3318E+00, 2.1981E+01, &
      3.001405E+01, 6.90712E+01, 2.0697E+00, 1.8364E+01, &
      3.208502E+01, 6.89695E+01, 1.8385E+00, 1.5362E+01, &
      3.429889E+01, 6.88613E+01, 1.6344E+00, 1.2867E+01, &
      3.666551E+01, 6.87471E+01, 1.4542E+00, 1.0793E+01, &
      3.919543E+01, 6.86254E+01, 1.2928E+00, 9.0528E+00, &
      4.189992E+01, 6.84912E+01, 1.1489E+00, 7.5967E+00, &
      4.479101E+01, 6.83377E+01, 1.0190E+00, 6.3695E+00, &
      4.788159E+01, 6.81488E+01, 9.0164E-01, 5.3342E+00, &
      5.118542E+01, 6.78932E+01, 7.9799E-01, 4.4733E+00, &
      5.471721E+01, 6.74763E+01, 7.0656E-01, 3.7575E+00, &
      5.820181E+01, 6.64210E+01, 6.3175E-01, 3.2036E+00, &
      5.849270E+01, 6.61954E+01, 6.2608E-01, 3.1628E+00, &
      5.909265E+01, 6.52863E+01, 6.1465E-01, 3.0811E+00, &
      5.933021E+01, 6.39276E+01, 6.1021E-01, 3.0496E+00, &
      5.968655E+01, 6.53004E+01, 3.2783E+00, 1.4166E+01, &
      6.057739E+01, 6.65405E+01, 3.2050E+00, 1.3654E+01, &
      6.252870E+01, 6.74400E+01, 3.0529E+00, 1.2618E+01, &
      6.684318E+01, 6.82358E+01, 2.7418E+00, 1.0637E+01, &
      7.145536E+01, 6.86397E+01, 2.4583E+00, 8.9561E+00, &
      7.638578E+01, 6.88894E+01, 2.2038E+00, 7.5431E+00, &
      8.165640E+01, 6.90612E+01, 1.9726E+00, 6.3469E+00, &
      8.729069E+01, 6.91715E+01, 1.7598E+00, 5.3267E+00, &
      9.331374E+01, 6.92436E+01, 1.5702E+00, 4.4743E+00, &
      9.975239E+01, 6.92898E+01, 1.4011E+00, 3.7617E+00, &
      1.066353E+02, 6.93180E+01, 1.2505E+00, 3.1660E+00, &
      1.139931E+02, 6.93333E+01, 1.1162E+00, 2.6676E+00, &
      1.218587E+02, 6.93682E+01, 9.9475E-01, 2.2473E+00, &
      1.302669E+02, 6.93651E+01, 8.8429E-01, 1.8915E+00, &
      1.392553E+02, 6.93557E+01, 7.8617E-01, 1.5948E+00, &
      1.488640E+02, 6.93424E+01, 6.9900E-01, 1.3472E+00, &
      1.591356E+02, 6.93265E+01, 6.2156E-01, 1.1406E+00, &
      1.701159E+02, 6.93090E+01, 5.5275E-01, 9.6805E-01, &
      1.818539E+02, 6.92906E+01, 4.9159E-01, 8.2383E-01, &
      1.944018E+02, 6.92719E+01, 4.3724E-01, 7.0320E-01, &
      2.078156E+02, 6.92532E+01, 3.8893E-01, 6.0224E-01, &
      2.221548E+02, 6.92348E+01, 3.4599E-01, 5.1765E-01, &
      2.374835E+02, 6.92169E+01, 3.0781E-01, 4.4670E-01, &
      2.538699E+02, 6.91994E+01, 2.7387E-01, 3.8712E-01, &
      2.713869E+02, 6.91828E+01, 2.4486E-01, 3.3809E-01, &
      2.901126E+02, 6.91677E+01, 2.1921E-01, 2.9684E-01, &
      3.101304E+02, 6.91538E+01, 1.9626E-01, 2.6183E-01, &
      3.315294E+02, 6.91410E+01, 1.7572E-01, 2.3206E-01, &
      3.544049E+02, 6.91291E+01, 1.5735E-01, 2.0668E-01, &
      3.788588E+02, 6.91182E+01, 1.4090E-01, 1.8499E-01, &
      4.050001E+02, 6.91081E+01, 1.2618E-01, 1.6639E-01, &
      4.329451E+02, 6.90987E+01, 1.1301E-01, 1.5040E-01]
   case(70)
      frel = -7.4880E-01
      allocate(nist_tab(3596))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.174000E-03, 8.70177E+00, 0.0000E+00, 1.6844E-05, &
      6.268500E-03, 8.61958E+00, 0.0000E+00, 1.8235E-05, &
      6.293700E-03, 8.53448E+00, 0.0000E+00, 1.8621E-05, &
      6.331500E-03, 8.61150E+00, 1.5997E-01, 6.1442E+03, &
      6.426000E-03, 8.66995E+00, 1.6048E-01, 6.0730E+03, &
      1.069000E-02, 8.08034E+00, 2.4028E-01, 5.4660E+03, &
      1.142761E-02, 7.90296E+00, 2.6322E-01, 5.6013E+03, &
      1.221612E-02, 7.68974E+00, 2.8998E-01, 5.7726E+03, &
      1.305903E-02, 7.43114E+00, 3.2108E-01, 5.9790E+03, &
      1.396010E-02, 7.11351E+00, 3.5710E-01, 6.2206E+03, &
      1.492335E-02, 6.71672E+00, 3.9875E-01, 6.4978E+03, &
      1.595306E-02, 6.20942E+00, 4.4685E-01, 6.8116E+03, &
      1.705382E-02, 5.53952E+00, 5.0232E-01, 7.1630E+03, &
      1.823053E-02, 4.61228E+00, 5.6624E-01, 7.5533E+03, &
      1.948844E-02, 3.33375E+00, 6.3982E-01, 7.9839E+03, &
      2.083314E-02, 9.89324E-01, 7.2443E-01, 8.4562E+03, &
      2.227063E-02, -4.40734E+00, 8.2162E-01, 8.9716E+03, &
      2.293200E-02, -1.12079E+01, 8.6864E-01, 9.2115E+03, &
      2.328300E-02, -2.32927E+01, 8.9418E-01, 9.3394E+03, &
      2.337660E-02, -3.83698E+01, 9.0106E-01, 9.3736E+03, &
      2.351700E-02, -2.18016E+01, 3.0895E+01, 3.1948E+05, &
      2.380730E-02, -8.64846E+00, 2.9627E+01, 3.0263E+05, &
      2.386800E-02, -7.13978E+00, 2.9371E+01, 2.9926E+05, &
      2.545001E-02, 8.78620E+00, 2.3655E+01, 2.2603E+05, &
      2.720606E-02, 1.42828E+01, 1.8939E+01, 1.6929E+05, &
      2.908327E-02, 1.66838E+01, 1.5192E+01, 1.2703E+05, &
      3.109002E-02, 1.76344E+01, 1.2300E+01, 9.6207E+04, &
      3.323523E-02, 1.78325E+01, 1.0127E+01, 7.4097E+04, &
      3.552846E-02, 1.76392E+01, 8.5366E+00, 5.8430E+04, &
      3.797993E-02, 1.72551E+01, 7.4086E+00, 4.7437E+04, &
      4.060054E-02, 1.67932E+01, 6.6435E+00, 3.9792E+04, &
      4.340198E-02, 1.63169E+01, 6.1631E+00, 3.4532E+04, &
      4.639671E-02, 1.58609E+01, 5.9074E+00, 3.0963E+04, &
      4.959809E-02, 1.54428E+01, 5.8318E+00, 2.8594E+04, &
      5.302035E-02, 1.50639E+01, 5.9031E+00, 2.7075E+04, &
      5.382950E-02, 1.49722E+01, 5.9370E+00, 2.6821E+04, &
      5.404590E-02, 1.49329E+01, 5.9470E+00, 2.6759E+04, &
      5.437050E-02, 1.49244E+01, 6.0019E+00, 2.6845E+04, &
      5.518200E-02, 1.48735E+01, 6.0455E+00, 2.6642E+04, &
      5.667876E-02, 1.47690E+01, 6.1379E+00, 2.6335E+04, &
      6.058959E-02, 1.45370E+01, 6.4395E+00, 2.5846E+04, &
      6.477028E-02, 1.43639E+01, 6.8326E+00, 2.5653E+04, &
      6.923942E-02, 1.42565E+01, 7.3074E+00, 2.5665E+04, &
      7.401695E-02, 1.42383E+01, 7.8560E+00, 2.5811E+04, &
      7.912411E-02, 1.42782E+01, 8.4722E+00, 2.6039E+04, &
      8.458368E-02, 1.44017E+01, 9.1509E+00, 2.6309E+04, &
      9.041995E-02, 1.46183E+01, 9.8876E+00, 2.6593E+04, &
      9.665893E-02, 1.49408E+01, 1.0679E+01, 2.6866E+04, &
      1.0000000E-01, 1.71539E+01, 1.1101E+01, 2.6997E+04, &
      1.0050000E-01, 1.71878E+01, 1.1165E+01, 2.7016E+04, &
      1.0100250E-01, 1.72224E+01, 1.1228E+01, 2.7034E+04, &
      1.0150751E-01, 1.72578E+01, 1.1292E+01, 2.7052E+04, &
      1.0201505E-01, 1.72940E+01, 1.1356E+01, 2.7070E+04, &
      1.0252513E-01, 1.73310E+01, 1.1420E+01, 2.7088E+04, &
      1.0303775E-01, 1.73688E+01, 1.1484E+01, 2.7105E+04, &
      1.0355294E-01, 1.74075E+01, 1.1549E+01, 2.7122E+04, &
      1.0407070E-01, 1.74470E+01, 1.1614E+01, 2.7139E+04, &
      1.0459106E-01, 1.74874E+01, 1.1679E+01, 2.7156E+04, &
      1.0511401E-01, 1.75286E+01, 1.1745E+01, 2.7173E+04, &
      1.0563958E-01, 1.75708E+01, 1.1811E+01, 2.7189E+04, &
      1.0616778E-01, 1.76138E+01, 1.1877E+01, 2.7205E+04, &
      1.0669862E-01, 1.76578E+01, 1.1943E+01, 2.7221E+04, &
      1.0723211E-01, 1.77027E+01, 1.2010E+01, 2.7237E+04, &
      1.0776827E-01, 1.77485E+01, 1.2077E+01, 2.7252E+04, &
      1.0830712E-01, 1.77954E+01, 1.2144E+01, 2.7267E+04, &
      1.0884865E-01, 1.78432E+01, 1.2211E+01, 2.7282E+04, &
      1.0939289E-01, 1.78921E+01, 1.2279E+01, 2.7297E+04, &
      1.0993986E-01, 1.79420E+01, 1.2347E+01, 2.7311E+04, &
      1.1048956E-01, 1.79929E+01, 1.2415E+01, 2.7325E+04, &
      1.1104201E-01, 1.80450E+01, 1.2483E+01, 2.7339E+04, &
      1.1159722E-01, 1.80981E+01, 1.2552E+01, 2.7352E+04, &
      1.1215520E-01, 1.81524E+01, 1.2621E+01, 2.7366E+04, &
      1.1271598E-01, 1.82078E+01, 1.2690E+01, 2.7379E+04, &
      1.1327956E-01, 1.82644E+01, 1.2759E+01, 2.7391E+04, &
      1.1384596E-01, 1.83222E+01, 1.2829E+01, 2.7404E+04, &
      1.1441519E-01, 1.83813E+01, 1.2899E+01, 2.7416E+04, &
      1.1498726E-01, 1.84416E+01, 1.2969E+01, 2.7428E+04, &
      1.1556220E-01, 1.85032E+01, 1.3039E+01, 2.7439E+04, &
      1.1614001E-01, 1.85662E+01, 1.3110E+01, 2.7450E+04, &
      1.1672071E-01, 1.86305E+01, 1.3180E+01, 2.7461E+04, &
      1.1730431E-01, 1.86962E+01, 1.3251E+01, 2.7472E+04, &
      1.1789083E-01, 1.87634E+01, 1.3323E+01, 2.7482E+04, &
      1.1848029E-01, 1.88325E+01, 1.3381E+01, 2.7466E+04, &
      1.1907269E-01, 1.89028E+01, 1.3410E+01, 2.7388E+04, &
      1.1966805E-01, 1.89727E+01, 1.3438E+01, 2.7309E+04, &
      1.2026639E-01, 1.90423E+01, 1.3466E+01, 2.7230E+04, &
      1.2086772E-01, 1.91115E+01, 1.3494E+01, 2.7150E+04, &
      1.2147206E-01, 1.91804E+01, 1.3521E+01, 2.7069E+04, &
      1.2207942E-01, 1.92489E+01, 1.3548E+01, 2.6989E+04, &
      1.2268982E-01, 1.93172E+01, 1.3575E+01, 2.6907E+04, &
      1.2330327E-01, 1.93852E+01, 1.3601E+01, 2.6826E+04, &
      1.2391979E-01, 1.94528E+01, 1.3627E+01, 2.6743E+04, &
      1.2453939E-01, 1.95202E+01, 1.3653E+01, 2.6661E+04, &
      1.2516208E-01, 1.95873E+01, 1.3679E+01, 2.6578E+04, &
      1.2578789E-01, 1.96541E+01, 1.3704E+01, 2.6494E+04, &
      1.2641683E-01, 1.97206E+01, 1.3729E+01, 2.6410E+04, &
      1.2704892E-01, 1.97869E+01, 1.3753E+01, 2.6325E+04, &
      1.2768416E-01, 1.98529E+01, 1.3777E+01, 2.6240E+04, &
      1.2832258E-01, 1.99186E+01, 1.3801E+01, 2.6155E+04, &
      1.2896419E-01, 1.99840E+01, 1.3825E+01, 2.6069E+04, &
      1.2960902E-01, 2.00492E+01, 1.3848E+01, 2.5983E+04, &
      1.3025706E-01, 2.01141E+01, 1.3870E+01, 2.5896E+04, &
      1.3090835E-01, 2.01787E+01, 1.3893E+01, 2.5809E+04, &
      1.3156289E-01, 2.02430E+01, 1.3915E+01, 2.5721E+04, &
      1.3222070E-01, 2.03071E+01, 1.3937E+01, 2.5634E+04, &
      1.3288181E-01, 2.03708E+01, 1.3958E+01, 2.5545E+04, &
      1.3354621E-01, 2.04343E+01, 1.3979E+01, 2.5456E+04, &
      1.3421395E-01, 2.04974E+01, 1.4000E+01, 2.5367E+04, &
      1.3488502E-01, 2.05602E+01, 1.4020E+01, 2.5278E+04, &
      1.3555944E-01, 2.06227E+01, 1.4040E+01, 2.5188E+04, &
      1.3623724E-01, 2.06849E+01, 1.4060E+01, 2.5097E+04, &
      1.3691842E-01, 2.07467E+01, 1.4079E+01, 2.5007E+04, &
      1.3760302E-01, 2.08082E+01, 1.4098E+01, 2.4916E+04, &
      1.3829103E-01, 2.08693E+01, 1.4116E+01, 2.4824E+04, &
      1.3898249E-01, 2.09300E+01, 1.4134E+01, 2.4733E+04, &
      1.3967740E-01, 2.09904E+01, 1.4152E+01, 2.4640E+04, &
      1.4037579E-01, 2.10503E+01, 1.4170E+01, 2.4548E+04, &
      1.4107766E-01, 2.11098E+01, 1.4187E+01, 2.4455E+04, &
      1.4178305E-01, 2.11688E+01, 1.4203E+01, 2.4362E+04, &
      1.4249197E-01, 2.12274E+01, 1.4219E+01, 2.4269E+04, &
      1.4320443E-01, 2.12855E+01, 1.4235E+01, 2.4175E+04, &
      1.4392045E-01, 2.13430E+01, 1.4251E+01, 2.4081E+04, &
      1.4464005E-01, 2.14001E+01, 1.4266E+01, 2.3986E+04, &
      1.4536325E-01, 2.14566E+01, 1.4281E+01, 2.3892E+04, &
      1.4609007E-01, 2.15125E+01, 1.4295E+01, 2.3797E+04, &
      1.4682052E-01, 2.15677E+01, 1.4309E+01, 2.3701E+04, &
      1.4755462E-01, 2.16223E+01, 1.4323E+01, 2.3606E+04, &
      1.4829239E-01, 2.16763E+01, 1.4336E+01, 2.3510E+04, &
      1.4903386E-01, 2.17295E+01, 1.4348E+01, 2.3414E+04, &
      1.4977903E-01, 2.17820E+01, 1.4361E+01, 2.3317E+04, &
      1.5052792E-01, 2.18336E+01, 1.4373E+01, 2.3221E+04, &
      1.5128056E-01, 2.18844E+01, 1.4384E+01, 2.3124E+04, &
      1.5203696E-01, 2.19343E+01, 1.4396E+01, 2.3027E+04, &
      1.5279715E-01, 2.19833E+01, 1.4406E+01, 2.2929E+04, &
      1.5356113E-01, 2.20312E+01, 1.4417E+01, 2.2832E+04, &
      1.5432894E-01, 2.20781E+01, 1.4427E+01, 2.2734E+04, &
      1.5510058E-01, 2.21238E+01, 1.4436E+01, 2.2636E+04, &
      1.5587609E-01, 2.21683E+01, 1.4446E+01, 2.2538E+04, &
      1.5665547E-01, 2.22115E+01, 1.4454E+01, 2.2439E+04, &
      1.5743875E-01, 2.22533E+01, 1.4463E+01, 2.2341E+04, &
      1.5822594E-01, 2.22936E+01, 1.4471E+01, 2.2242E+04, &
      1.5901707E-01, 2.23323E+01, 1.4478E+01, 2.2143E+04, &
      1.5981215E-01, 2.23693E+01, 1.4486E+01, 2.2044E+04, &
      1.6061121E-01, 2.24044E+01, 1.4492E+01, 2.1944E+04, &
      1.6141427E-01, 2.24376E+01, 1.4499E+01, 2.1845E+04, &
      1.6222134E-01, 2.24685E+01, 1.4505E+01, 2.1745E+04, &
      1.6303245E-01, 2.24987E+01, 1.4510E+01, 2.1645E+04, &
      1.6384761E-01, 2.25247E+01, 1.4515E+01, 2.1545E+04, &
      1.6466685E-01, 2.25480E+01, 1.4520E+01, 2.1445E+04, &
      1.6549018E-01, 2.25681E+01, 1.4525E+01, 2.1345E+04, &
      1.6631763E-01, 2.25849E+01, 1.4528E+01, 2.1244E+04, &
      1.6714922E-01, 2.25979E+01, 1.4532E+01, 2.1144E+04, &
      1.6798497E-01, 2.26067E+01, 1.4535E+01, 2.1043E+04, &
      1.6882489E-01, 2.26109E+01, 1.4538E+01, 2.0942E+04, &
      1.6966902E-01, 2.26099E+01, 1.4540E+01, 2.0841E+04, &
      1.7051736E-01, 2.26030E+01, 1.4542E+01, 2.0740E+04, &
      1.7136995E-01, 2.25894E+01, 1.4544E+01, 2.0639E+04, &
      1.7222680E-01, 2.25682E+01, 1.4545E+01, 2.0538E+04, &
      1.7308793E-01, 2.25381E+01, 1.4545E+01, 2.0437E+04, &
      1.7395337E-01, 2.24976E+01, 1.4546E+01, 2.0336E+04, &
      1.7482314E-01, 2.24448E+01, 1.4545E+01, 2.0234E+04, &
      1.7569726E-01, 2.23773E+01, 1.4545E+01, 2.0133E+04, &
      1.7657574E-01, 2.22918E+01, 1.4544E+01, 2.0032E+04, &
      1.7745862E-01, 2.21839E+01, 1.4543E+01, 1.9930E+04, &
      1.7834591E-01, 2.20474E+01, 1.4541E+01, 1.9829E+04, &
      1.7923764E-01, 2.18733E+01, 1.4539E+01, 1.9727E+04, &
      1.8013383E-01, 2.16475E+01, 1.4536E+01, 1.9625E+04, &
      1.8103450E-01, 2.13465E+01, 1.4533E+01, 1.9524E+04, &
      1.8193967E-01, 2.09273E+01, 1.4530E+01, 1.9422E+04, &
      1.8284937E-01, 2.02960E+01, 1.4526E+01, 1.9321E+04, &
      1.8376362E-01, 1.91799E+01, 1.4522E+01, 1.9219E+04, &
      1.8468244E-01, 1.56490E+01, 1.4517E+01, 1.9117E+04, &
      1.8476613E-01, 1.45455E+01, 1.4517E+01, 1.9108E+04, &
      1.8503387E-01, 1.47266E+01, 2.1900E+01, 2.8783E+04, &
      1.8560585E-01, 1.89116E+01, 2.1659E+01, 2.8379E+04, &
      1.8653388E-01, 2.11444E+01, 2.1285E+01, 2.7751E+04, &
      1.8746655E-01, 2.23655E+01, 2.0930E+01, 2.7152E+04, &
      1.8840388E-01, 2.31919E+01, 2.0593E+01, 2.6582E+04, &
      1.8934590E-01, 2.37954E+01, 2.0272E+01, 2.6038E+04, &
      1.9029263E-01, 2.42488E+01, 1.9967E+01, 2.5519E+04, &
      1.9124409E-01, 2.45883E+01, 1.9677E+01, 2.5023E+04, &
      1.9220031E-01, 2.48323E+01, 1.9401E+01, 2.4549E+04, &
      1.9316131E-01, 2.49873E+01, 1.9139E+01, 2.4097E+04, &
      1.9412712E-01, 2.50483E+01, 1.8889E+01, 2.3664E+04, &
      1.9509776E-01, 2.49927E+01, 1.8652E+01, 2.3250E+04, &
      1.9607325E-01, 2.47576E+01, 1.8425E+01, 2.2854E+04, &
      1.9705361E-01, 2.41288E+01, 1.8210E+01, 2.2475E+04, &
      1.9791478E-01, 2.19002E+01, 1.8030E+01, 2.2156E+04, &
      1.9803888E-01, 2.03066E+01, 1.8005E+01, 2.2111E+04, &
      1.9828522E-01, 2.20937E+01, 2.2599E+01, 2.7718E+04, &
      1.9902907E-01, 2.47765E+01, 2.2280E+01, 2.7225E+04, &
      2.0002422E-01, 2.61333E+01, 2.1873E+01, 2.6595E+04, &
      2.0102434E-01, 2.69668E+01, 2.1486E+01, 2.5994E+04, &
      2.0202946E-01, 2.75734E+01, 2.1117E+01, 2.5420E+04, &
      2.0303961E-01, 2.80466E+01, 2.0766E+01, 2.4873E+04, &
      2.0405481E-01, 2.84290E+01, 2.0431E+01, 2.4351E+04, &
      2.0507508E-01, 2.87451E+01, 2.0116E+01, 2.3856E+04, &
      2.0610046E-01, 2.90112E+01, 1.9819E+01, 2.3387E+04, &
      2.0713096E-01, 2.92382E+01, 1.9539E+01, 2.2942E+04, &
      2.0816661E-01, 2.94338E+01, 1.9275E+01, 2.2519E+04, &
      2.0920745E-01, 2.96038E+01, 1.9025E+01, 2.2117E+04, &
      2.1025348E-01, 2.97524E+01, 1.8789E+01, 2.1734E+04, &
      2.1130475E-01, 2.98831E+01, 1.8565E+01, 2.1368E+04, &
      2.1236128E-01, 2.99985E+01, 1.8354E+01, 2.1019E+04, &
      2.1342308E-01, 3.01008E+01, 1.8153E+01, 2.0686E+04, &
      2.1449020E-01, 3.01918E+01, 1.7962E+01, 2.0367E+04, &
      2.1556265E-01, 3.02727E+01, 1.7781E+01, 2.0061E+04, &
      2.1664046E-01, 3.03448E+01, 1.7609E+01, 1.9768E+04, &
      2.1772366E-01, 3.04088E+01, 1.7445E+01, 1.9487E+04, &
      2.1881228E-01, 3.04655E+01, 1.7289E+01, 1.9217E+04, &
      2.1990634E-01, 3.05158E+01, 1.7144E+01, 1.8960E+04, &
      2.2100588E-01, 3.05614E+01, 1.7007E+01, 1.8715E+04, &
      2.2211090E-01, 3.06032E+01, 1.6878E+01, 1.8481E+04, &
      2.2322146E-01, 3.06421E+01, 1.6756E+01, 1.8257E+04, &
      2.2433757E-01, 3.06786E+01, 1.6642E+01, 1.8042E+04, &
      2.2545925E-01, 3.07131E+01, 1.6533E+01, 1.7835E+04, &
      2.2658655E-01, 3.07462E+01, 1.6430E+01, 1.7636E+04, &
      2.2771948E-01, 3.07781E+01, 1.6333E+01, 1.7444E+04, &
      2.2885808E-01, 3.08090E+01, 1.6241E+01, 1.7259E+04, &
      2.3000237E-01, 3.08391E+01, 1.6152E+01, 1.7080E+04, &
      2.3115238E-01, 3.08687E+01, 1.6068E+01, 1.6907E+04, &
      2.3230814E-01, 3.08978E+01, 1.5988E+01, 1.6739E+04, &
      2.3346969E-01, 3.09266E+01, 1.5912E+01, 1.6576E+04, &
      2.3463703E-01, 3.09551E+01, 1.5838E+01, 1.6417E+04, &
      2.3581022E-01, 3.09835E+01, 1.5768E+01, 1.6263E+04, &
      2.3698927E-01, 3.10118E+01, 1.5700E+01, 1.6113E+04, &
      2.3817422E-01, 3.10400E+01, 1.5635E+01, 1.5967E+04, &
      2.3936509E-01, 3.10681E+01, 1.5573E+01, 1.5824E+04, &
      2.4056191E-01, 3.10963E+01, 1.5512E+01, 1.5684E+04, &
      2.4176472E-01, 3.11245E+01, 1.5454E+01, 1.5547E+04, &
      2.4297355E-01, 3.11527E+01, 1.5398E+01, 1.5414E+04, &
      2.4418841E-01, 3.11810E+01, 1.5343E+01, 1.5283E+04, &
      2.4540936E-01, 3.12093E+01, 1.5290E+01, 1.5154E+04, &
      2.4663640E-01, 3.12377E+01, 1.5239E+01, 1.5028E+04, &
      2.4786959E-01, 3.12661E+01, 1.5189E+01, 1.4905E+04, &
      2.4910893E-01, 3.12946E+01, 1.5141E+01, 1.4783E+04, &
      2.5035448E-01, 3.13231E+01, 1.5094E+01, 1.4664E+04, &
      2.5160625E-01, 3.13516E+01, 1.5048E+01, 1.4547E+04, &
      2.5286428E-01, 3.13802E+01, 1.5003E+01, 1.4431E+04, &
      2.5412860E-01, 3.14087E+01, 1.4959E+01, 1.4318E+04, &
      2.5539925E-01, 3.14373E+01, 1.4916E+01, 1.4206E+04, &
      2.5667624E-01, 3.14658E+01, 1.4875E+01, 1.4095E+04, &
      2.5795962E-01, 3.14943E+01, 1.4834E+01, 1.3987E+04, &
      2.5924942E-01, 3.15228E+01, 1.4794E+01, 1.3880E+04, &
      2.6054567E-01, 3.15512E+01, 1.4754E+01, 1.3774E+04, &
      2.6184840E-01, 3.15795E+01, 1.4716E+01, 1.3670E+04, &
      2.6315764E-01, 3.16077E+01, 1.4678E+01, 1.3567E+04, &
      2.6447343E-01, 3.16358E+01, 1.4641E+01, 1.3465E+04, &
      2.6579579E-01, 3.16638E+01, 1.4604E+01, 1.3365E+04, &
      2.6712477E-01, 3.16917E+01, 1.4568E+01, 1.3266E+04, &
      2.6846040E-01, 3.17194E+01, 1.4533E+01, 1.3168E+04, &
      2.6980270E-01, 3.17469E+01, 1.4498E+01, 1.3071E+04, &
      2.7115171E-01, 3.17742E+01, 1.4464E+01, 1.2975E+04, &
      2.7250747E-01, 3.18013E+01, 1.4430E+01, 1.2880E+04, &
      2.7387001E-01, 3.18282E+01, 1.4397E+01, 1.2787E+04, &
      2.7523936E-01, 3.18549E+01, 1.4364E+01, 1.2694E+04, &
      2.7661556E-01, 3.18812E+01, 1.4332E+01, 1.2603E+04, &
      2.7799863E-01, 3.19073E+01, 1.4300E+01, 1.2512E+04, &
      2.7938863E-01, 3.19331E+01, 1.4269E+01, 1.2423E+04, &
      2.8078557E-01, 3.19586E+01, 1.4238E+01, 1.2334E+04, &
      2.8218950E-01, 3.19838E+01, 1.4207E+01, 1.2246E+04, &
      2.8360044E-01, 3.20085E+01, 1.4177E+01, 1.2159E+04, &
      2.8501845E-01, 3.20329E+01, 1.4147E+01, 1.2073E+04, &
      2.8644354E-01, 3.20569E+01, 1.4117E+01, 1.1988E+04, &
      2.8787576E-01, 3.20805E+01, 1.4088E+01, 1.1904E+04, &
      2.8931514E-01, 3.21036E+01, 1.4059E+01, 1.1820E+04, &
      2.9076171E-01, 3.21262E+01, 1.4030E+01, 1.1738E+04, &
      2.9221552E-01, 3.21484E+01, 1.4002E+01, 1.1656E+04, &
      2.9367660E-01, 3.21700E+01, 1.3974E+01, 1.1575E+04, &
      2.9514498E-01, 3.21910E+01, 1.3946E+01, 1.1494E+04, &
      2.9662071E-01, 3.22115E+01, 1.3919E+01, 1.1415E+04, &
      2.9810381E-01, 3.22313E+01, 1.3892E+01, 1.1336E+04, &
      2.9959433E-01, 3.22504E+01, 1.3865E+01, 1.1258E+04, &
      3.0109230E-01, 3.22689E+01, 1.3839E+01, 1.1181E+04, &
      3.0259776E-01, 3.22865E+01, 1.3813E+01, 1.1104E+04, &
      3.0411075E-01, 3.23034E+01, 1.3787E+01, 1.1028E+04, &
      3.0563130E-01, 3.23194E+01, 1.3761E+01, 1.0953E+04, &
      3.0715946E-01, 3.23345E+01, 1.3736E+01, 1.0879E+04, &
      3.0869526E-01, 3.23486E+01, 1.3711E+01, 1.0805E+04, &
      3.1023873E-01, 3.23616E+01, 1.3686E+01, 1.0732E+04, &
      3.1178993E-01, 3.23735E+01, 1.3662E+01, 1.0659E+04, &
      3.1334888E-01, 3.23840E+01, 1.3638E+01, 1.0588E+04, &
      3.1491562E-01, 3.23932E+01, 1.3614E+01, 1.0516E+04, &
      3.1649020E-01, 3.24009E+01, 1.3590E+01, 1.0446E+04, &
      3.1807265E-01, 3.24068E+01, 1.3567E+01, 1.0376E+04, &
      3.1966301E-01, 3.24109E+01, 1.3544E+01, 1.0307E+04, &
      3.2126133E-01, 3.24128E+01, 1.3521E+01, 1.0239E+04, &
      3.2286764E-01, 3.24122E+01, 1.3499E+01, 1.0171E+04, &
      3.2448197E-01, 3.24089E+01, 1.3477E+01, 1.0104E+04, &
      3.2610438E-01, 3.24022E+01, 1.3455E+01, 1.0037E+04, &
      3.2773491E-01, 3.23917E+01, 1.3433E+01, 9.9712E+03, &
      3.2937358E-01, 3.23764E+01, 1.3412E+01, 9.9059E+03, &
      3.3102045E-01, 3.23552E+01, 1.3391E+01, 9.8411E+03, &
      3.3267555E-01, 3.23265E+01, 1.3370E+01, 9.7770E+03, &
      3.3433893E-01, 3.22880E+01, 1.3349E+01, 9.7135E+03, &
      3.3601062E-01, 3.22355E+01, 1.3329E+01, 9.6505E+03, &
      3.3769068E-01, 3.21621E+01, 1.3309E+01, 9.5882E+03, &
      3.3937913E-01, 3.20535E+01, 1.3289E+01, 9.5264E+03, &
      3.4107602E-01, 3.18715E+01, 1.3270E+01, 9.4652E+03, &
      3.4278140E-01, 3.14224E+01, 1.3251E+01, 9.4046E+03, &
      3.4308435E-01, 3.12139E+01, 1.3247E+01, 9.3939E+03, &
      3.4391562E-01, 3.12236E+01, 1.4457E+01, 1.0226E+04, &
      3.4449531E-01, 3.15691E+01, 1.4452E+01, 1.0206E+04, &
      3.4621779E-01, 3.19806E+01, 1.4438E+01, 1.0145E+04, &
      3.4794888E-01, 3.21944E+01, 1.4423E+01, 1.0085E+04, &
      3.4968862E-01, 3.23455E+01, 1.4410E+01, 1.0025E+04, &
      3.5143706E-01, 3.24651E+01, 1.4396E+01, 9.9657E+03, &
      3.5319425E-01, 3.25659E+01, 1.4383E+01, 9.9071E+03, &
      3.5496022E-01, 3.26539E+01, 1.4370E+01, 9.8490E+03, &
      3.5673502E-01, 3.27328E+01, 1.4357E+01, 9.7915E+03, &
      3.5851870E-01, 3.28048E+01, 1.4345E+01, 9.7345E+03, &
      3.6031129E-01, 3.28713E+01, 1.4333E+01, 9.6780E+03, &
      3.6211285E-01, 3.29333E+01, 1.4321E+01, 9.6220E+03, &
      3.6392341E-01, 3.29916E+01, 1.4310E+01, 9.5665E+03, &
      3.6574303E-01, 3.30466E+01, 1.4299E+01, 9.5115E+03, &
      3.6757174E-01, 3.30989E+01, 1.4288E+01, 9.4571E+03, &
      3.6940960E-01, 3.31487E+01, 1.4277E+01, 9.4031E+03, &
      3.7125665E-01, 3.31961E+01, 1.4267E+01, 9.3496E+03, &
      3.7311293E-01, 3.32414E+01, 1.4257E+01, 9.2967E+03, &
      3.7497850E-01, 3.32846E+01, 1.4247E+01, 9.2442E+03, &
      3.7685339E-01, 3.33258E+01, 1.4238E+01, 9.1921E+03, &
      3.7873766E-01, 3.33649E+01, 1.4229E+01, 9.1406E+03, &
      3.8063135E-01, 3.34018E+01, 1.4220E+01, 9.0895E+03, &
      3.8253450E-01, 3.34363E+01, 1.4211E+01, 9.0388E+03, &
      3.8444718E-01, 3.34680E+01, 1.4203E+01, 8.9885E+03, &
      3.8636941E-01, 3.34962E+01, 1.4195E+01, 8.9387E+03, &
      3.8830126E-01, 3.35199E+01, 1.4187E+01, 8.8893E+03, &
      3.9024276E-01, 3.35371E+01, 1.4179E+01, 8.8404E+03, &
      3.9219398E-01, 3.35435E+01, 1.4172E+01, 8.7918E+03, &
      3.9415495E-01, 3.35272E+01, 1.4164E+01, 8.7436E+03, &
      3.9612572E-01, 3.34144E+01, 1.4157E+01, 8.6959E+03, &
      3.9615652E-01, 3.34093E+01, 1.4157E+01, 8.6951E+03, &
      3.9724347E-01, 3.34318E+01, 1.4481E+01, 8.8694E+03, &
      3.9810635E-01, 3.35488E+01, 1.4479E+01, 8.8489E+03, &
      4.0009688E-01, 3.36822E+01, 1.4474E+01, 8.8020E+03, &
      4.0209737E-01, 3.37720E+01, 1.4469E+01, 8.7554E+03, &
      4.0410785E-01, 3.38464E+01, 1.4465E+01, 8.7092E+03, &
      4.0612839E-01, 3.39128E+01, 1.4460E+01, 8.6634E+03, &
      4.0815904E-01, 3.39743E+01, 1.4456E+01, 8.6180E+03, &
      4.1019983E-01, 3.40323E+01, 1.4453E+01, 8.5729E+03, &
      4.1225083E-01, 3.40878E+01, 1.4449E+01, 8.5281E+03, &
      4.1431208E-01, 3.41414E+01, 1.4445E+01, 8.4837E+03, &
      4.1638364E-01, 3.41934E+01, 1.4442E+01, 8.4397E+03, &
      4.1846556E-01, 3.42441E+01, 1.4439E+01, 8.3959E+03, &
      4.2055789E-01, 3.42937E+01, 1.4436E+01, 8.3525E+03, &
      4.2266068E-01, 3.43424E+01, 1.4433E+01, 8.3094E+03, &
      4.2477398E-01, 3.43903E+01, 1.4431E+01, 8.2667E+03, &
      4.2689785E-01, 3.44375E+01, 1.4428E+01, 8.2242E+03, &
      4.2903234E-01, 3.44839E+01, 1.4426E+01, 8.1820E+03, &
      4.3117750E-01, 3.45298E+01, 1.4424E+01, 8.1401E+03, &
      4.3333339E-01, 3.45751E+01, 1.4422E+01, 8.0986E+03, &
      4.3550006E-01, 3.46198E+01, 1.4420E+01, 8.0573E+03, &
      4.3767756E-01, 3.46639E+01, 1.4418E+01, 8.0162E+03, &
      4.3986595E-01, 3.47076E+01, 1.4417E+01, 7.9755E+03, &
      4.4206528E-01, 3.47507E+01, 1.4415E+01, 7.9350E+03, &
      4.4427560E-01, 3.47933E+01, 1.4414E+01, 7.8948E+03, &
      4.4649698E-01, 3.48352E+01, 1.4412E+01, 7.8548E+03, &
      4.4872947E-01, 3.48766E+01, 1.4411E+01, 7.8151E+03, &
      4.5097311E-01, 3.49174E+01, 1.4410E+01, 7.7756E+03, &
      4.5322798E-01, 3.49574E+01, 1.4409E+01, 7.7363E+03, &
      4.5549412E-01, 3.49966E+01, 1.4407E+01, 7.6973E+03, &
      4.5777159E-01, 3.50349E+01, 1.4406E+01, 7.6585E+03, &
      4.6006045E-01, 3.50722E+01, 1.4405E+01, 7.6199E+03, &
      4.6236075E-01, 3.51082E+01, 1.4404E+01, 7.5815E+03, &
      4.6467255E-01, 3.51428E+01, 1.4403E+01, 7.5434E+03, &
      4.6699592E-01, 3.51756E+01, 1.4402E+01, 7.5054E+03, &
      4.6933090E-01, 3.52062E+01, 1.4401E+01, 7.4676E+03, &
      4.7167755E-01, 3.52339E+01, 1.4401E+01, 7.4300E+03, &
      4.7403594E-01, 3.52576E+01, 1.4400E+01, 7.3926E+03, &
      4.7640612E-01, 3.52759E+01, 1.4399E+01, 7.3554E+03, &
      4.7878815E-01, 3.52860E+01, 1.4398E+01, 7.3183E+03, &
      4.8118209E-01, 3.52820E+01, 1.4396E+01, 7.2814E+03, &
      4.8358800E-01, 3.52493E+01, 1.4395E+01, 7.2447E+03, &
      4.8600594E-01, 3.51189E+01, 1.4394E+01, 7.2081E+03, &
      4.8646432E-01, 3.50492E+01, 1.4394E+01, 7.2012E+03, &
      4.8793566E-01, 3.50796E+01, 1.4907E+01, 7.4353E+03, &
      4.8843597E-01, 3.51748E+01, 1.4907E+01, 7.4277E+03, &
      4.9087815E-01, 3.54040E+01, 1.4907E+01, 7.3907E+03, &
      4.9333254E-01, 3.55389E+01, 1.4907E+01, 7.3538E+03, &
      4.9579920E-01, 3.56457E+01, 1.4906E+01, 7.3171E+03, &
      4.9827820E-01, 3.57390E+01, 1.4906E+01, 7.2806E+03, &
      5.0076959E-01, 3.58242E+01, 1.4905E+01, 7.2441E+03, &
      5.0327344E-01, 3.59042E+01, 1.4905E+01, 7.2078E+03, &
      5.0578980E-01, 3.59805E+01, 1.4904E+01, 7.1717E+03, &
      5.0831875E-01, 3.60540E+01, 1.4903E+01, 7.1356E+03, &
      5.1086035E-01, 3.61255E+01, 1.4902E+01, 7.0996E+03, &
      5.1341465E-01, 3.61953E+01, 1.4901E+01, 7.0638E+03, &
      5.1598172E-01, 3.62638E+01, 1.4899E+01, 7.0281E+03, &
      5.1856163E-01, 3.63313E+01, 1.4898E+01, 6.9924E+03, &
      5.2115444E-01, 3.63979E+01, 1.4896E+01, 6.9569E+03, &
      5.2376021E-01, 3.64638E+01, 1.4894E+01, 6.9215E+03, &
      5.2637901E-01, 3.65290E+01, 1.4892E+01, 6.8861E+03, &
      5.2901091E-01, 3.65938E+01, 1.4890E+01, 6.8509E+03, &
      5.3165596E-01, 3.66581E+01, 1.4887E+01, 6.8157E+03, &
      5.3431424E-01, 3.67221E+01, 1.4885E+01, 6.7807E+03, &
      5.3698581E-01, 3.67857E+01, 1.4882E+01, 6.7457E+03, &
      5.3967074E-01, 3.68491E+01, 1.4879E+01, 6.7107E+03, &
      5.4236910E-01, 3.69123E+01, 1.4875E+01, 6.6759E+03, &
      5.4508094E-01, 3.69752E+01, 1.4871E+01, 6.6411E+03, &
      5.4780635E-01, 3.70380E+01, 1.4867E+01, 6.6063E+03, &
      5.5054538E-01, 3.71006E+01, 1.4863E+01, 6.5716E+03, &
      5.5329810E-01, 3.71630E+01, 1.4859E+01, 6.5370E+03, &
      5.5606460E-01, 3.72514E+01, 1.4854E+01, 6.5024E+03, &
      5.5884492E-01, 3.73137E+01, 1.4849E+01, 6.4679E+03, &
      5.6163914E-01, 3.73758E+01, 1.4843E+01, 6.4334E+03, &
      5.6444734E-01, 3.74378E+01, 1.4838E+01, 6.3990E+03, &
      5.6726958E-01, 3.74997E+01, 1.4831E+01, 6.3646E+03, &
      5.7010592E-01, 3.75615E+01, 1.4825E+01, 6.3303E+03, &
      5.7295645E-01, 3.76232E+01, 1.4818E+01, 6.2960E+03, &
      5.7582123E-01, 3.76848E+01, 1.4811E+01, 6.2617E+03, &
      5.7870034E-01, 3.77463E+01, 1.4804E+01, 6.2275E+03, &
      5.8159384E-01, 3.78077E+01, 1.4796E+01, 6.1934E+03, &
      5.8450181E-01, 3.78691E+01, 1.4788E+01, 6.1592E+03, &
      5.8742432E-01, 3.79303E+01, 1.4780E+01, 6.1252E+03, &
      5.9036144E-01, 3.79915E+01, 1.4771E+01, 6.0911E+03, &
      5.9331325E-01, 3.80525E+01, 1.4762E+01, 6.0571E+03, &
      5.9627982E-01, 3.81187E+01, 1.4752E+01, 6.0232E+03, &
      5.9926122E-01, 3.81795E+01, 1.4742E+01, 5.9892E+03, &
      6.0225752E-01, 3.82402E+01, 1.4732E+01, 5.9554E+03, &
      6.0526881E-01, 3.83008E+01, 1.4721E+01, 5.9215E+03, &
      6.0829515E-01, 3.83613E+01, 1.4710E+01, 5.8877E+03, &
      6.1133663E-01, 3.84216E+01, 1.4699E+01, 5.8539E+03, &
      6.1439331E-01, 3.84818E+01, 1.4687E+01, 5.8202E+03, &
      6.1746528E-01, 3.85419E+01, 1.4675E+01, 5.7865E+03, &
      6.2055260E-01, 3.86018E+01, 1.4662E+01, 5.7529E+03, &
      6.2365537E-01, 3.86616E+01, 1.4649E+01, 5.7193E+03, &
      6.2677364E-01, 3.87213E+01, 1.4636E+01, 5.6857E+03, &
      6.2990751E-01, 3.87807E+01, 1.4622E+01, 5.6522E+03, &
      6.3305705E-01, 3.88401E+01, 1.4608E+01, 5.6187E+03, &
      6.3622234E-01, 3.88992E+01, 1.4594E+01, 5.5852E+03, &
      6.3940345E-01, 3.89582E+01, 1.4579E+01, 5.5518E+03, &
      6.4260046E-01, 3.90169E+01, 1.4563E+01, 5.5185E+03, &
      6.4581347E-01, 3.90755E+01, 1.4548E+01, 5.4851E+03, &
      6.4904253E-01, 3.91339E+01, 1.4532E+01, 5.4519E+03, &
      6.5228775E-01, 3.91921E+01, 1.4515E+01, 5.4187E+03, &
      6.5554919E-01, 3.92501E+01, 1.4498E+01, 5.3855E+03, &
      6.5882693E-01, 3.93078E+01, 1.4481E+01, 5.3524E+03, &
      6.6212107E-01, 3.93654E+01, 1.4463E+01, 5.3193E+03, &
      6.6543167E-01, 3.94226E+01, 1.4445E+01, 5.2863E+03, &
      6.6875883E-01, 3.94797E+01, 1.4426E+01, 5.2533E+03, &
      6.7210262E-01, 3.95365E+01, 1.4408E+01, 5.2204E+03, &
      6.7546314E-01, 3.95930E+01, 1.4388E+01, 5.1875E+03, &
      6.7884045E-01, 3.96493E+01, 1.4369E+01, 5.1547E+03, &
      6.8223466E-01, 3.97052E+01, 1.4348E+01, 5.1219E+03, &
      6.8564583E-01, 3.97609E+01, 1.4328E+01, 5.0892E+03, &
      6.8907406E-01, 3.98163E+01, 1.4307E+01, 5.0566E+03, &
      6.9251943E-01, 3.98715E+01, 1.4286E+01, 5.0240E+03, &
      6.9598202E-01, 3.99263E+01, 1.4264E+01, 4.9915E+03, &
      6.9946194E-01, 3.99808E+01, 1.4242E+01, 4.9591E+03, &
      7.0295924E-01, 4.00349E+01, 1.4220E+01, 4.9267E+03, &
      7.0647404E-01, 4.00888E+01, 1.4197E+01, 4.8944E+03, &
      7.1000641E-01, 4.01423E+01, 1.4173E+01, 4.8622E+03, &
      7.1355644E-01, 4.01954E+01, 1.4150E+01, 4.8300E+03, &
      7.1712423E-01, 4.02482E+01, 1.4126E+01, 4.7979E+03, &
      7.2070985E-01, 4.03007E+01, 1.4102E+01, 4.7659E+03, &
      7.2431340E-01, 4.03528E+01, 1.4077E+01, 4.7339E+03, &
      7.2793496E-01, 4.04045E+01, 1.4052E+01, 4.7020E+03, &
      7.3157464E-01, 4.04558E+01, 1.4026E+01, 4.6703E+03, &
      7.3523251E-01, 4.05068E+01, 1.4000E+01, 4.6385E+03, &
      7.3890867E-01, 4.05573E+01, 1.3974E+01, 4.6069E+03, &
      7.4260322E-01, 4.06075E+01, 1.3948E+01, 4.5754E+03, &
      7.4631623E-01, 4.06572E+01, 1.3921E+01, 4.5439E+03, &
      7.5004781E-01, 4.07065E+01, 1.3893E+01, 4.5125E+03, &
      7.5379805E-01, 4.07554E+01, 1.3866E+01, 4.4811E+03, &
      7.5756704E-01, 4.08038E+01, 1.3838E+01, 4.4499E+03, &
      7.6135488E-01, 4.08518E+01, 1.3809E+01, 4.4187E+03, &
      7.6516165E-01, 4.08993E+01, 1.3780E+01, 4.3876E+03, &
      7.6898746E-01, 4.09463E+01, 1.3751E+01, 4.3566E+03, &
      7.7283240E-01, 4.09928E+01, 1.3721E+01, 4.3256E+03, &
      7.7669656E-01, 4.10388E+01, 1.3691E+01, 4.2948E+03, &
      7.8058004E-01, 4.10843E+01, 1.3661E+01, 4.2640E+03, &
      7.8448294E-01, 4.11293E+01, 1.3630E+01, 4.2334E+03, &
      7.8840536E-01, 4.11737E+01, 1.3599E+01, 4.2028E+03, &
      7.9234738E-01, 4.12176E+01, 1.3568E+01, 4.1724E+03, &
      7.9630912E-01, 4.12610E+01, 1.3536E+01, 4.1420E+03, &
      8.0029067E-01, 4.13039E+01, 1.3504E+01, 4.1118E+03, &
      8.0429212E-01, 4.13461E+01, 1.3472E+01, 4.0816E+03, &
      8.0831358E-01, 4.13878E+01, 1.3440E+01, 4.0515E+03, &
      8.1235515E-01, 4.14290E+01, 1.3407E+01, 4.0216E+03, &
      8.1641693E-01, 4.14695E+01, 1.3373E+01, 3.9917E+03, &
      8.2049901E-01, 4.15095E+01, 1.3340E+01, 3.9620E+03, &
      8.2460150E-01, 4.15488E+01, 1.3306E+01, 3.9324E+03, &
      8.2872451E-01, 4.15875E+01, 1.3272E+01, 3.9028E+03, &
      8.3286813E-01, 4.16255E+01, 1.3237E+01, 3.8733E+03, &
      8.3703248E-01, 4.16628E+01, 1.3202E+01, 3.8439E+03, &
      8.4121764E-01, 4.16995E+01, 1.3167E+01, 3.8147E+03, &
      8.4542373E-01, 4.17355E+01, 1.3131E+01, 3.7856E+03, &
      8.4965084E-01, 4.17708E+01, 1.3096E+01, 3.7566E+03, &
      8.5389910E-01, 4.18053E+01, 1.3060E+01, 3.7278E+03, &
      8.5816859E-01, 4.18392E+01, 1.3024E+01, 3.6991E+03, &
      8.6245944E-01, 4.18724E+01, 1.2987E+01, 3.6705E+03, &
      8.6677173E-01, 4.19049E+01, 1.2951E+01, 3.6421E+03, &
      8.7110559E-01, 4.19367E+01, 1.2914E+01, 3.6138E+03, &
      8.7546112E-01, 4.19677E+01, 1.2878E+01, 3.5857E+03, &
      8.7983843E-01, 4.19981E+01, 1.2841E+01, 3.5577E+03, &
      8.8423762E-01, 4.20277E+01, 1.2804E+01, 3.5298E+03, &
      8.8865881E-01, 4.20565E+01, 1.2766E+01, 3.5021E+03, &
      8.9310210E-01, 4.20847E+01, 1.2729E+01, 3.4746E+03, &
      8.9756761E-01, 4.21121E+01, 1.2691E+01, 3.4472E+03, &
      9.0205545E-01, 4.21388E+01, 1.2654E+01, 3.4200E+03, &
      9.0656573E-01, 4.21648E+01, 1.2616E+01, 3.3929E+03, &
      9.1109856E-01, 4.21900E+01, 1.2578E+01, 3.3660E+03, &
      9.1565405E-01, 4.22144E+01, 1.2540E+01, 3.3393E+03, &
      9.2023232E-01, 4.22382E+01, 1.2503E+01, 3.3127E+03, &
      9.2483348E-01, 4.22611E+01, 1.2465E+01, 3.2863E+03, &
      9.2945765E-01, 4.22834E+01, 1.2426E+01, 3.2601E+03, &
      9.3410494E-01, 4.23049E+01, 1.2388E+01, 3.2340E+03, &
      9.3877546E-01, 4.23256E+01, 1.2350E+01, 3.2080E+03, &
      9.4346934E-01, 4.23456E+01, 1.2312E+01, 3.1823E+03, &
      9.4818668E-01, 4.23649E+01, 1.2274E+01, 3.1567E+03, &
      9.5292762E-01, 4.23834E+01, 1.2235E+01, 3.1313E+03, &
      9.5769226E-01, 4.24012E+01, 1.2197E+01, 3.1061E+03, &
      9.6248072E-01, 4.24182E+01, 1.2159E+01, 3.0810E+03, &
      9.6729312E-01, 4.24345E+01, 1.2120E+01, 3.0561E+03, &
      9.7212959E-01, 4.24501E+01, 1.2082E+01, 3.0314E+03, &
      9.7699023E-01, 4.24649E+01, 1.2044E+01, 3.0068E+03, &
      9.8187519E-01, 4.24790E+01, 1.2006E+01, 2.9825E+03, &
      9.8678456E-01, 4.24924E+01, 1.1967E+01, 2.9583E+03, &
      9.9171848E-01, 4.25051E+01, 1.1929E+01, 2.9342E+03, &
      9.9667708E-01, 4.25171E+01, 1.1891E+01, 2.9104E+03, &
      1.0016605E+00, 4.25334E+01, 1.1840E+01, 2.8837E+03, &
      1.0066688E+00, 4.25528E+01, 1.1765E+01, 2.8512E+03, &
      1.0117021E+00, 4.25643E+01, 1.1690E+01, 2.8191E+03, &
      1.0167606E+00, 4.25695E+01, 1.1616E+01, 2.7874E+03, &
      1.0218444E+00, 4.25694E+01, 1.1543E+01, 2.7561E+03, &
      1.0269536E+00, 4.25649E+01, 1.1470E+01, 2.7252E+03, &
      1.0320884E+00, 4.25578E+01, 1.1397E+01, 2.6947E+03, &
      1.0372489E+00, 4.25458E+01, 1.1326E+01, 2.6645E+03, &
      1.0424351E+00, 4.25305E+01, 1.1255E+01, 2.6347E+03, &
      1.0476473E+00, 4.25121E+01, 1.1184E+01, 2.6053E+03, &
      1.0528855E+00, 4.24908E+01, 1.1114E+01, 2.5762E+03, &
      1.0581499E+00, 4.24667E+01, 1.1045E+01, 2.5475E+03, &
      1.0634407E+00, 4.24399E+01, 1.0976E+01, 2.5192E+03, &
      1.0687579E+00, 4.24105E+01, 1.0907E+01, 2.4911E+03, &
      1.0741017E+00, 4.23786E+01, 1.0840E+01, 2.4635E+03, &
      1.0794722E+00, 4.23441E+01, 1.0772E+01, 2.4361E+03, &
      1.0848695E+00, 4.23071E+01, 1.0706E+01, 2.4091E+03, &
      1.0902939E+00, 4.22676E+01, 1.0640E+01, 2.3825E+03, &
      1.0957454E+00, 4.22256E+01, 1.0574E+01, 2.3561E+03, &
      1.1012241E+00, 4.21812E+01, 1.0509E+01, 2.3301E+03, &
      1.1067302E+00, 4.21343E+01, 1.0444E+01, 2.3044E+03, &
      1.1122639E+00, 4.20849E+01, 1.0380E+01, 2.2790E+03, &
      1.1178252E+00, 4.20330E+01, 1.0317E+01, 2.2539E+03, &
      1.1234143E+00, 4.19785E+01, 1.0254E+01, 2.2291E+03, &
      1.1290314E+00, 4.19214E+01, 1.0191E+01, 2.2046E+03, &
      1.1346765E+00, 4.18617E+01, 1.0129E+01, 2.1804E+03, &
      1.1403499E+00, 4.17994E+01, 1.0068E+01, 2.1564E+03, &
      1.1460517E+00, 4.17343E+01, 1.0007E+01, 2.1328E+03, &
      1.1517819E+00, 4.16664E+01, 9.9460E+00, 2.1095E+03, &
      1.1575408E+00, 4.15958E+01, 9.8859E+00, 2.0864E+03, &
      1.1633285E+00, 4.15222E+01, 9.8263E+00, 2.0636E+03, &
      1.1691452E+00, 4.14456E+01, 9.7672E+00, 2.0411E+03, &
      1.1749909E+00, 4.13660E+01, 9.7085E+00, 2.0189E+03, &
      1.1808659E+00, 4.12832E+01, 9.6502E+00, 1.9969E+03, &
      1.1867702E+00, 4.11972E+01, 9.5925E+00, 1.9752E+03, &
      1.1927040E+00, 4.11064E+01, 9.5351E+00, 1.9537E+03, &
      1.1986676E+00, 4.10136E+01, 9.4783E+00, 1.9325E+03, &
      1.2046609E+00, 4.09172E+01, 9.4218E+00, 1.9116E+03, &
      1.2106842E+00, 4.08172E+01, 9.3658E+00, 1.8909E+03, &
      1.2167376E+00, 4.07133E+01, 9.3103E+00, 1.8704E+03, &
      1.2228213E+00, 4.06055E+01, 9.2551E+00, 1.8502E+03, &
      1.2289354E+00, 4.04935E+01, 9.2004E+00, 1.8303E+03, &
      1.2350801E+00, 4.03772E+01, 9.1462E+00, 1.8105E+03, &
      1.2412555E+00, 4.02564E+01, 9.0923E+00, 1.7910E+03, &
      1.2474618E+00, 4.01310E+01, 9.0389E+00, 1.7718E+03, &
      1.2536991E+00, 4.00006E+01, 8.9858E+00, 1.7527E+03, &
      1.2599676E+00, 3.98652E+01, 8.9332E+00, 1.7339E+03, &
      1.2662674E+00, 3.97243E+01, 8.8810E+00, 1.7153E+03, &
      1.2725988E+00, 3.95778E+01, 8.8292E+00, 1.6969E+03, &
      1.2789618E+00, 3.94254E+01, 8.7778E+00, 1.6788E+03, &
      1.2853566E+00, 3.92667E+01, 8.7268E+00, 1.6608E+03, &
      1.2917833E+00, 3.91014E+01, 8.6761E+00, 1.6431E+03, &
      1.2982423E+00, 3.89292E+01, 8.6259E+00, 1.6256E+03, &
      1.3047335E+00, 3.87495E+01, 8.5760E+00, 1.6083E+03, &
      1.3112571E+00, 3.85620E+01, 8.5266E+00, 1.5911E+03, &
      1.3178134E+00, 3.83662E+01, 8.4775E+00, 1.5742E+03, &
      1.3244025E+00, 3.81614E+01, 8.4287E+00, 1.5575E+03, &
      1.3310245E+00, 3.79471E+01, 8.3804E+00, 1.5410E+03, &
      1.3376796E+00, 3.77227E+01, 8.3324E+00, 1.5246E+03, &
      1.3443680E+00, 3.74873E+01, 8.2848E+00, 1.5085E+03, &
      1.3510899E+00, 3.72401E+01, 8.2375E+00, 1.4925E+03, &
      1.3578453E+00, 3.69801E+01, 8.1906E+00, 1.4768E+03, &
      1.3646345E+00, 3.67064E+01, 8.1441E+00, 1.4612E+03, &
      1.3714577E+00, 3.64177E+01, 8.0979E+00, 1.4458E+03, &
      1.3783150E+00, 3.61126E+01, 8.0509E+00, 1.4304E+03, &
      1.3852066E+00, 3.57894E+01, 8.0037E+00, 1.4150E+03, &
      1.3921326E+00, 3.54463E+01, 7.9568E+00, 1.3998E+03, &
      1.3990933E+00, 3.50812E+01, 7.9103E+00, 1.3849E+03, &
      1.4060887E+00, 3.46916E+01, 7.8641E+00, 1.3700E+03, &
      1.4131192E+00, 3.42747E+01, 7.8183E+00, 1.3554E+03, &
      1.4201848E+00, 3.38270E+01, 7.7728E+00, 1.3409E+03, &
      1.4272857E+00, 3.33443E+01, 7.7277E+00, 1.3266E+03, &
      1.4344221E+00, 3.28216E+01, 7.6829E+00, 1.3125E+03, &
      1.4415942E+00, 3.22526E+01, 7.6385E+00, 1.2985E+03, &
      1.4488022E+00, 3.16294E+01, 7.5944E+00, 1.2847E+03, &
      1.4560462E+00, 3.09419E+01, 7.5506E+00, 1.2711E+03, &
      1.4633265E+00, 3.01780E+01, 7.5072E+00, 1.2576E+03, &
      1.4706431E+00, 2.93172E+01, 7.4641E+00, 1.2442E+03, &
      1.4779963E+00, 2.83353E+01, 7.4213E+00, 1.2311E+03, &
      1.4853863E+00, 2.71953E+01, 7.3788E+00, 1.2180E+03, &
      1.4928132E+00, 2.58385E+01, 7.3366E+00, 1.2052E+03, &
      1.5002773E+00, 2.41652E+01, 7.2948E+00, 1.1924E+03, &
      1.5077787E+00, 2.19805E+01, 7.2533E+00, 1.1799E+03, &
      1.5153176E+00, 1.88094E+01, 7.2120E+00, 1.1674E+03, &
      1.5228942E+00, 1.27485E+01, 7.1711E+00, 1.1552E+03, &
      1.5275616E+00, -6.16539E+00, 7.1462E+00, 1.1477E+03, &
      1.5280383E+00, -6.50870E+00, 2.6615E+01, 4.2458E+03, &
      1.5305086E+00, 8.46563E+00, 2.6549E+01, 4.2284E+03, &
      1.5381612E+00, 1.63279E+01, 2.6345E+01, 4.1752E+03, &
      1.5458520E+00, 1.91031E+01, 2.6144E+01, 4.1228E+03, &
      1.5535812E+00, 2.03540E+01, 2.5943E+01, 4.0710E+03, &
      1.5613491E+00, 2.05113E+01, 2.5745E+01, 4.0198E+03, &
      1.5691559E+00, 1.90375E+01, 2.5548E+01, 3.9694E+03, &
      1.5758114E+00, 9.29434E+00, 2.5382E+01, 3.9271E+03, &
      1.5767887E+00, 9.24297E+00, 3.8098E+01, 5.8858E+03, &
      1.5770017E+00, 1.07459E+01, 3.8090E+01, 5.8837E+03, &
      1.5848867E+00, 2.20634E+01, 3.7793E+01, 5.8090E+03, &
      1.5928111E+00, 2.57088E+01, 3.7499E+01, 5.7352E+03, &
      1.6007752E+00, 2.81835E+01, 3.7206E+01, 5.6623E+03, &
      1.6087790E+00, 3.01187E+01, 3.6916E+01, 5.5903E+03, &
      1.6168229E+00, 3.17298E+01, 3.6629E+01, 5.5193E+03, &
      1.6249070E+00, 3.31192E+01, 3.6343E+01, 5.4492E+03, &
      1.6330316E+00, 3.43448E+01, 3.6060E+01, 5.3800E+03, &
      1.6411967E+00, 3.54429E+01, 3.5779E+01, 5.3117E+03, &
      1.6494027E+00, 3.64381E+01, 3.5501E+01, 5.2442E+03, &
      1.6576497E+00, 3.73481E+01, 3.5225E+01, 5.1777E+03, &
      1.6659380E+00, 3.81859E+01, 3.4951E+01, 5.1120E+03, &
      1.6742677E+00, 3.89614E+01, 3.4679E+01, 5.0472E+03, &
      1.6826390E+00, 3.96825E+01, 3.4410E+01, 4.9832E+03, &
      1.6910522E+00, 4.03554E+01, 3.4142E+01, 4.9200E+03, &
      1.6995075E+00, 4.09853E+01, 3.3877E+01, 4.8576E+03, &
      1.7080050E+00, 4.15763E+01, 3.3614E+01, 4.7961E+03, &
      1.7165450E+00, 4.21319E+01, 3.3354E+01, 4.7353E+03, &
      1.7251278E+00, 4.26550E+01, 3.3095E+01, 4.6753E+03, &
      1.7337534E+00, 4.31481E+01, 3.2838E+01, 4.6161E+03, &
      1.7424222E+00, 4.36134E+01, 3.2584E+01, 4.5577E+03, &
      1.7511343E+00, 4.40524E+01, 3.2331E+01, 4.5000E+03, &
      1.7598899E+00, 4.44668E+01, 3.2081E+01, 4.4431E+03, &
      1.7686894E+00, 4.48578E+01, 3.1832E+01, 4.3868E+03, &
      1.7775328E+00, 4.52264E+01, 3.1586E+01, 4.3314E+03, &
      1.7864205E+00, 4.55733E+01, 3.1342E+01, 4.2766E+03, &
      1.7953526E+00, 4.58993E+01, 3.1099E+01, 4.2225E+03, &
      1.8043294E+00, 4.62047E+01, 3.0859E+01, 4.1692E+03, &
      1.8133510E+00, 4.64896E+01, 3.0620E+01, 4.1165E+03, &
      1.8224178E+00, 4.67541E+01, 3.0384E+01, 4.0645E+03, &
      1.8315299E+00, 4.69979E+01, 3.0149E+01, 4.0132E+03, &
      1.8406875E+00, 4.72202E+01, 2.9916E+01, 3.9625E+03, &
      1.8498909E+00, 4.74201E+01, 2.9686E+01, 3.9125E+03, &
      1.8591404E+00, 4.75958E+01, 2.9457E+01, 3.8631E+03, &
      1.8684361E+00, 4.77450E+01, 2.9230E+01, 3.8144E+03, &
      1.8777783E+00, 4.78640E+01, 2.9004E+01, 3.7663E+03, &
      1.8871672E+00, 4.79476E+01, 2.8781E+01, 3.7188E+03, &
      1.8966030E+00, 4.79874E+01, 2.8559E+01, 3.6719E+03, &
      1.9060860E+00, 4.79698E+01, 2.8339E+01, 3.6257E+03, &
      1.9156165E+00, 4.78704E+01, 2.8121E+01, 3.5800E+03, &
      1.9251945E+00, 4.76397E+01, 2.7905E+01, 3.5349E+03, &
      1.9348205E+00, 4.71503E+01, 2.7690E+01, 3.4904E+03, &
      1.9444946E+00, 4.58346E+01, 2.7477E+01, 3.4465E+03, &
      1.9475382E+00, 4.46174E+01, 2.7411E+01, 3.4328E+03, &
      1.9520618E+00, 4.46759E+01, 3.2118E+01, 4.0113E+03, &
      1.9542171E+00, 4.57658E+01, 3.2058E+01, 3.9994E+03, &
      1.9639882E+00, 4.78151E+01, 3.1786E+01, 3.9459E+03, &
      1.9738081E+00, 4.88564E+01, 3.1517E+01, 3.8931E+03, &
      1.9836772E+00, 4.96011E+01, 3.1250E+01, 3.8411E+03, &
      1.9935955E+00, 5.01960E+01, 3.0985E+01, 3.7897E+03, &
      2.0035635E+00, 5.06968E+01, 3.0722E+01, 3.7389E+03, &
      2.0135813E+00, 5.11310E+01, 3.0461E+01, 3.6889E+03, &
      2.0236492E+00, 5.15135E+01, 3.0202E+01, 3.6395E+03, &
      2.0337675E+00, 5.18530E+01, 2.9946E+01, 3.5908E+03, &
      2.0439363E+00, 5.21534E+01, 2.9692E+01, 3.5428E+03, &
      2.0541560E+00, 5.24207E+01, 2.9461E+01, 3.4978E+03, &
      2.0644268E+00, 5.26687E+01, 2.9237E+01, 3.4540E+03, &
      2.0747489E+00, 5.28970E+01, 2.9016E+01, 3.4110E+03, &
      2.0851227E+00, 5.31051E+01, 2.8799E+01, 3.3687E+03, &
      2.0955483E+00, 5.32923E+01, 2.8584E+01, 3.3271E+03, &
      2.1060260E+00, 5.34572E+01, 2.8373E+01, 3.2862E+03, &
      2.1165562E+00, 5.35967E+01, 2.8164E+01, 3.2459E+03, &
      2.1271389E+00, 5.37051E+01, 2.7958E+01, 3.2063E+03, &
      2.1377746E+00, 5.37715E+01, 2.7755E+01, 3.1673E+03, &
      2.1484635E+00, 5.37716E+01, 2.7554E+01, 3.1289E+03, &
      2.1592058E+00, 5.36356E+01, 2.7356E+01, 3.0910E+03, &
      2.1700018E+00, 5.29164E+01, 2.7161E+01, 3.0538E+03, &
      2.1704360E+00, 5.28293E+01, 2.7153E+01, 3.0523E+03, &
      2.1755642E+00, 5.29028E+01, 2.8982E+01, 3.2495E+03, &
      2.1808519E+00, 5.36916E+01, 2.8875E+01, 3.2298E+03, &
      2.1917561E+00, 5.44223E+01, 2.8658E+01, 3.1896E+03, &
      2.2027149E+00, 5.48883E+01, 2.8443E+01, 3.1501E+03, &
      2.2137285E+00, 5.52548E+01, 2.8230E+01, 3.1111E+03, &
      2.2247971E+00, 5.55658E+01, 2.8020E+01, 3.0727E+03, &
      2.2359211E+00, 5.58396E+01, 2.7812E+01, 3.0349E+03, &
      2.2471007E+00, 5.60852E+01, 2.7607E+01, 2.9976E+03, &
      2.2583362E+00, 5.63078E+01, 2.7404E+01, 2.9608E+03, &
      2.2696279E+00, 5.65101E+01, 2.7203E+01, 2.9246E+03, &
      2.2809760E+00, 5.66927E+01, 2.7005E+01, 2.8890E+03, &
      2.2923809E+00, 5.68603E+01, 2.6821E+01, 2.8552E+03, &
      2.3038428E+00, 5.70170E+01, 2.6640E+01, 2.8219E+03, &
      2.3153620E+00, 5.71615E+01, 2.6461E+01, 2.7891E+03, &
      2.3269388E+00, 5.72923E+01, 2.6284E+01, 2.7568E+03, &
      2.3385735E+00, 5.74073E+01, 2.6110E+01, 2.7250E+03, &
      2.3502664E+00, 5.75023E+01, 2.5938E+01, 2.6937E+03, &
      2.3620177E+00, 5.75690E+01, 2.5768E+01, 2.6628E+03, &
      2.3738278E+00, 5.75888E+01, 2.5600E+01, 2.6324E+03, &
      2.3856970E+00, 5.75021E+01, 2.5434E+01, 2.6024E+03, &
      2.3931119E+00, 5.72620E+01, 2.5331E+01, 2.5839E+03, &
      2.3976254E+00, 5.64412E+01, 2.5270E+01, 2.5728E+03, &
      2.4030879E+00, 5.73875E+01, 2.6372E+01, 2.6785E+03, &
      2.4096136E+00, 5.77872E+01, 2.6280E+01, 2.6621E+03, &
      2.4216616E+00, 5.82101E+01, 2.6114E+01, 2.6322E+03, &
      2.4337699E+00, 5.85158E+01, 2.5950E+01, 2.6027E+03, &
      2.4459388E+00, 5.87724E+01, 2.5786E+01, 2.5735E+03, &
      2.4581685E+00, 5.90011E+01, 2.5625E+01, 2.5448E+03, &
      2.4704593E+00, 5.92111E+01, 2.5465E+01, 2.5164E+03, &
      2.4828116E+00, 5.94075E+01, 2.5306E+01, 2.4884E+03, &
      2.4952257E+00, 5.95935E+01, 2.5148E+01, 2.4606E+03, &
      2.5077018E+00, 5.97708E+01, 2.4992E+01, 2.4333E+03, &
      2.5202403E+00, 5.99408E+01, 2.4837E+01, 2.4063E+03, &
      2.5328415E+00, 6.01056E+01, 2.4685E+01, 2.3798E+03, &
      2.5455057E+00, 6.02653E+01, 2.4530E+01, 2.3531E+03, &
      2.5582333E+00, 6.04194E+01, 2.4375E+01, 2.3267E+03, &
      2.5710244E+00, 6.05687E+01, 2.4221E+01, 2.3006E+03, &
      2.5838796E+00, 6.07135E+01, 2.4068E+01, 2.2748E+03, &
      2.5967990E+00, 6.08544E+01, 2.3916E+01, 2.2494E+03, &
      2.6097829E+00, 6.09915E+01, 2.3766E+01, 2.2242E+03, &
      2.6228319E+00, 6.11252E+01, 2.3616E+01, 2.1993E+03, &
      2.6359460E+00, 6.12558E+01, 2.3468E+01, 2.1747E+03, &
      2.6491257E+00, 6.13833E+01, 2.3321E+01, 2.1504E+03, &
      2.6623714E+00, 6.15081E+01, 2.3174E+01, 2.1263E+03, &
      2.6756832E+00, 6.16302E+01, 2.3029E+01, 2.1026E+03, &
      2.6890617E+00, 6.17499E+01, 2.2884E+01, 2.0791E+03, &
      2.7025070E+00, 6.18672E+01, 2.2740E+01, 2.0558E+03, &
      2.7160195E+00, 6.19822E+01, 2.2597E+01, 2.0328E+03, &
      2.7295996E+00, 6.20951E+01, 2.2455E+01, 2.0101E+03, &
      2.7432476E+00, 6.22059E+01, 2.2314E+01, 1.9876E+03, &
      2.7569638E+00, 6.23149E+01, 2.2174E+01, 1.9654E+03, &
      2.7707486E+00, 6.24219E+01, 2.2035E+01, 1.9434E+03, &
      2.7846024E+00, 6.25273E+01, 2.1896E+01, 1.9217E+03, &
      2.7985254E+00, 6.26309E+01, 2.1758E+01, 1.9002E+03, &
      2.8125180E+00, 6.27331E+01, 2.1621E+01, 1.8789E+03, &
      2.8265806E+00, 6.28338E+01, 2.1484E+01, 1.8578E+03, &
      2.8407135E+00, 6.29327E+01, 2.1347E+01, 1.8368E+03, &
      2.8549171E+00, 6.30297E+01, 2.1210E+01, 1.8161E+03, &
      2.8691917E+00, 6.31251E+01, 2.1074E+01, 1.7956E+03, &
      2.8835376E+00, 6.32191E+01, 2.0939E+01, 1.7753E+03, &
      2.8979553E+00, 6.33116E+01, 2.0805E+01, 1.7552E+03, &
      2.9124451E+00, 6.34028E+01, 2.0671E+01, 1.7353E+03, &
      2.9270073E+00, 6.34930E+01, 2.0538E+01, 1.7157E+03, &
      2.9416424E+00, 6.35823E+01, 2.0406E+01, 1.6963E+03, &
      2.9563506E+00, 6.36712E+01, 2.0275E+01, 1.6771E+03, &
      2.9711323E+00, 6.37602E+01, 2.0145E+01, 1.6582E+03, &
      2.9859880E+00, 6.38504E+01, 2.0016E+01, 1.6394E+03, &
      3.0009179E+00, 6.39454E+01, 1.9887E+01, 1.6208E+03, &
      3.0159225E+00, 6.40414E+01, 1.9743E+01, 1.6012E+03, &
      3.0310021E+00, 6.41313E+01, 1.9600E+01, 1.5818E+03, &
      3.0461571E+00, 6.42174E+01, 1.9458E+01, 1.5626E+03, &
      3.0613879E+00, 6.45182E+01, 1.9315E+01, 1.5435E+03, &
      3.0766949E+00, 6.45976E+01, 1.9168E+01, 1.5242E+03, &
      3.0920783E+00, 6.46741E+01, 1.9023E+01, 1.5052E+03, &
      3.1075387E+00, 6.47478E+01, 1.8879E+01, 1.4865E+03, &
      3.1230764E+00, 6.48190E+01, 1.8736E+01, 1.4680E+03, &
      3.1386918E+00, 6.48878E+01, 1.8593E+01, 1.4497E+03, &
      3.1543853E+00, 6.50964E+01, 1.8452E+01, 1.4316E+03, &
      3.1701572E+00, 6.51611E+01, 1.8308E+01, 1.4135E+03, &
      3.1860080E+00, 6.52232E+01, 1.8165E+01, 1.3955E+03, &
      3.2019380E+00, 6.52829E+01, 1.8023E+01, 1.3779E+03, &
      3.2179477E+00, 6.53402E+01, 1.7882E+01, 1.3604E+03, &
      3.2340374E+00, 6.53954E+01, 1.7743E+01, 1.3432E+03, &
      3.2502076E+00, 6.54485E+01, 1.7606E+01, 1.3263E+03, &
      3.2664587E+00, 6.54997E+01, 1.7469E+01, 1.3095E+03, &
      3.2827910E+00, 6.55490E+01, 1.7334E+01, 1.2930E+03, &
      3.2992049E+00, 6.55966E+01, 1.7200E+01, 1.2767E+03, &
      3.3157009E+00, 6.56425E+01, 1.7067E+01, 1.2607E+03, &
      3.3322794E+00, 6.56868E+01, 1.6935E+01, 1.2448E+03, &
      3.3489408E+00, 6.57295E+01, 1.6805E+01, 1.2292E+03, &
      3.3656856E+00, 6.57708E+01, 1.6675E+01, 1.2137E+03, &
      3.3825140E+00, 6.58106E+01, 1.6547E+01, 1.1985E+03, &
      3.3994265E+00, 6.58491E+01, 1.6420E+01, 1.1835E+03, &
      3.4164237E+00, 6.58863E+01, 1.6294E+01, 1.1687E+03, &
      3.4335058E+00, 6.59222E+01, 1.6170E+01, 1.1541E+03, &
      3.4506733E+00, 6.59569E+01, 1.6046E+01, 1.1396E+03, &
      3.4679267E+00, 6.59904E+01, 1.5924E+01, 1.1254E+03, &
      3.4852663E+00, 6.60228E+01, 1.5802E+01, 1.1113E+03, &
      3.5026927E+00, 6.60541E+01, 1.5680E+01, 1.0974E+03, &
      3.5202061E+00, 6.60843E+01, 1.5560E+01, 1.0836E+03, &
      3.5378072E+00, 6.61134E+01, 1.5440E+01, 1.0701E+03, &
      3.5554962E+00, 6.61415E+01, 1.5322E+01, 1.0567E+03, &
      3.5732737E+00, 6.61686E+01, 1.5205E+01, 1.0435E+03, &
      3.5911400E+00, 6.61948E+01, 1.5089E+01, 1.0304E+03, &
      3.6090957E+00, 6.62200E+01, 1.4974E+01, 1.0176E+03, &
      3.6271412E+00, 6.62443E+01, 1.4860E+01, 1.0049E+03, &
      3.6452769E+00, 6.62678E+01, 1.4747E+01, 9.9241E+02, &
      3.6635033E+00, 6.62904E+01, 1.4635E+01, 9.8007E+02, &
      3.6818208E+00, 6.63122E+01, 1.4524E+01, 9.6789E+02, &
      3.7002299E+00, 6.63332E+01, 1.4415E+01, 9.5588E+02, &
      3.7187311E+00, 6.63535E+01, 1.4306E+01, 9.4403E+02, &
      3.7373247E+00, 6.63731E+01, 1.4197E+01, 9.3229E+02, &
      3.7560114E+00, 6.63920E+01, 1.4089E+01, 9.2070E+02, &
      3.7747914E+00, 6.64102E+01, 1.3983E+01, 9.0927E+02, &
      3.7936654E+00, 6.64276E+01, 1.3877E+01, 8.9799E+02, &
      3.8126337E+00, 6.64445E+01, 1.3772E+01, 8.8686E+02, &
      3.8316969E+00, 6.64608E+01, 1.3668E+01, 8.7588E+02, &
      3.8508554E+00, 6.64764E+01, 1.3565E+01, 8.6504E+02, &
      3.8701096E+00, 6.64915E+01, 1.3463E+01, 8.5435E+02, &
      3.8894602E+00, 6.65061E+01, 1.3362E+01, 8.4381E+02, &
      3.9089075E+00, 6.66298E+01, 1.3260E+01, 8.3327E+02, &
      3.9284520E+00, 6.66439E+01, 1.3156E+01, 8.2273E+02, &
      3.9480943E+00, 6.66571E+01, 1.3054E+01, 8.1233E+02, &
      3.9678347E+00, 6.66694E+01, 1.2952E+01, 8.0207E+02, &
      3.9876739E+00, 6.66808E+01, 1.2851E+01, 7.9195E+02, &
      4.055024E+00, 6.71293E+01, 1.2519E+01, 7.5898E+02, &
      4.334821E+00, 6.71246E+01, 1.1292E+01, 6.4139E+02, &
      4.633924E+00, 6.70963E+01, 1.0172E+01, 5.4145E+02, &
      4.953664E+00, 6.69694E+01, 9.1701E+00, 4.5752E+02, &
      5.295467E+00, 6.67626E+01, 8.2727E+00, 3.8696E+02, &
      5.660855E+00, 6.65036E+01, 7.4733E+00, 3.2779E+02, &
      6.051453E+00, 6.61931E+01, 6.7601E+00, 2.7810E+02, &
      6.469004E+00, 6.58226E+01, 6.1228E+00, 2.3631E+02, &
      6.915365E+00, 6.53663E+01, 5.5324E+00, 2.0039E+02, &
      7.392525E+00, 6.47554E+01, 4.9914E+00, 1.6974E+02, &
      7.902609E+00, 6.38684E+01, 4.5094E+00, 1.4400E+02, &
      8.447890E+00, 6.22694E+01, 4.0808E+00, 1.2242E+02, &
      8.764728E+00, 6.01186E+01, 3.8563E+00, 1.1179E+02, &
      8.898882E+00, 5.71931E+01, 3.7678E+00, 1.0770E+02, &
      8.934656E+00, 5.37535E+01, 3.7409E+00, 1.0653E+02, &
      8.988318E+00, 5.72170E+01, 1.0431E+01, 2.8689E+02, &
      9.030794E+00, 5.86884E+01, 1.0321E+01, 2.8258E+02, &
      9.122472E+00, 6.02430E+01, 1.0089E+01, 2.7357E+02, &
      9.653919E+00, 6.22929E+01, 8.9588E+00, 2.3006E+02, &
      9.778636E+00, 6.20963E+01, 8.7436E+00, 2.2178E+02, &
      9.928309E+00, 6.09185E+01, 8.5005E+00, 2.1249E+02, &
      9.968222E+00, 5.92549E+01, 8.4382E+00, 2.1012E+02, &
      1.002809E+01, 6.10691E+01, 1.1721E+01, 2.8848E+02, &
      1.017776E+01, 6.27204E+01, 1.1387E+01, 2.7626E+02, &
      1.027667E+01, 6.31439E+01, 1.1176E+01, 2.6860E+02, &
      1.032004E+01, 6.32436E+01, 1.1086E+01, 2.6534E+02, &
      1.043397E+01, 6.31156E+01, 1.0854E+01, 2.5706E+02, &
      1.047591E+01, 6.24005E+01, 1.0772E+01, 2.5411E+02, &
      1.053883E+01, 6.34667E+01, 1.2356E+01, 2.8915E+02, &
      1.069613E+01, 6.46852E+01, 1.2066E+01, 2.7832E+02, &
      1.103212E+01, 6.59637E+01, 1.1498E+01, 2.5731E+02, &
      1.179334E+01, 6.75511E+01, 1.0372E+01, 2.1749E+02, &
      1.260708E+01, 6.85201E+01, 9.3393E+00, 1.8352E+02, &
      1.347697E+01, 6.91784E+01, 8.3938E+00, 1.5460E+02, &
      1.440688E+01, 6.96219E+01, 7.5242E+00, 1.2992E+02, &
      1.540095E+01, 6.99154E+01, 6.7398E+00, 1.0914E+02, &
      1.646362E+01, 7.01045E+01, 6.0368E+00, 9.1689E+01, &
      1.759961E+01, 7.02200E+01, 5.4096E+00, 7.7079E+01, &
      1.881398E+01, 7.03448E+01, 4.8427E+00, 6.4751E+01, &
      2.011215E+01, 7.03981E+01, 4.3363E+00, 5.4423E+01, &
      2.149988E+01, 7.04166E+01, 3.8790E+00, 4.5711E+01, &
      2.298338E+01, 7.03806E+01, 3.4684E+00, 3.8390E+01, &
      2.456923E+01, 7.03291E+01, 3.1040E+00, 3.2279E+01, &
      2.626450E+01, 7.02721E+01, 2.7802E+00, 2.7172E+01, &
      2.807676E+01, 7.02074E+01, 2.4723E+00, 2.2726E+01, &
      3.001405E+01, 7.01192E+01, 2.1945E+00, 1.8984E+01, &
      3.208502E+01, 7.00208E+01, 1.9495E+00, 1.5880E+01, &
      3.429889E+01, 6.99157E+01, 1.7332E+00, 1.3301E+01, &
      3.666551E+01, 6.98048E+01, 1.5421E+00, 1.1155E+01, &
      3.919543E+01, 6.96878E+01, 1.3727E+00, 9.3657E+00, &
      4.189992E+01, 6.95608E+01, 1.2204E+00, 7.8602E+00, &
      4.479101E+01, 6.94188E+01, 1.0847E+00, 6.6007E+00, &
      4.788159E+01, 6.92508E+01, 9.6081E-01, 5.5316E+00, &
      5.118542E+01, 6.90346E+01, 8.5078E-01, 4.6394E+00, &
      5.471721E+01, 6.87183E+01, 7.5329E-01, 3.8954E+00, &
      5.849270E+01, 6.80890E+01, 6.6748E-01, 3.2773E+00, &
      6.010565E+01, 6.74370E+01, 6.3550E-01, 3.0563E+00, &
      6.102564E+01, 6.63087E+01, 6.1835E-01, 2.9399E+00, &
      6.127097E+01, 6.49576E+01, 6.1390E-01, 2.9100E+00, &
      6.163896E+01, 6.63231E+01, 3.2671E+00, 1.3360E+01, &
      6.252870E+01, 6.75338E+01, 3.1960E+00, 1.2891E+01, &
      6.255894E+01, 6.75565E+01, 3.1936E+00, 1.2875E+01, &
      6.684318E+01, 6.89414E+01, 2.8776E+00, 1.0893E+01, &
      7.145536E+01, 6.94756E+01, 2.5819E+00, 9.1772E+00, &
      7.638578E+01, 6.97851E+01, 2.3152E+00, 7.7305E+00, &
      8.165640E+01, 6.99894E+01, 2.0735E+00, 6.5073E+00, &
      8.729069E+01, 7.01221E+01, 1.8505E+00, 5.4628E+00, &
      9.331374E+01, 7.02094E+01, 1.6518E+00, 4.5896E+00, &
      9.975239E+01, 7.02663E+01, 1.4746E+00, 3.8595E+00, &
      1.066353E+02, 7.03022E+01, 1.3166E+00, 3.2489E+00, &
      1.139931E+02, 7.03230E+01, 1.1757E+00, 2.7379E+00, &
      1.218587E+02, 7.03335E+01, 1.0500E+00, 2.3102E+00, &
      1.302669E+02, 7.03634E+01, 9.3416E-01, 1.9452E+00, &
      1.392553E+02, 7.03570E+01, 8.3084E-01, 1.6400E+00, &
      1.488640E+02, 7.03456E+01, 7.3901E-01, 1.3852E+00, &
      1.591356E+02, 7.03310E+01, 6.5740E-01, 1.1726E+00, &
      1.701159E+02, 7.03144E+01, 5.8485E-01, 9.9489E-01, &
      1.818539E+02, 7.02967E+01, 5.2036E-01, 8.4635E-01, &
      1.944018E+02, 7.02783E+01, 4.6302E-01, 7.2211E-01, &
      2.078156E+02, 7.02598E+01, 4.1203E-01, 6.1808E-01, &
      2.221548E+02, 7.02414E+01, 3.6668E-01, 5.3091E-01, &
      2.374835E+02, 7.02234E+01, 3.2635E-01, 4.5779E-01, &
      2.538699E+02, 7.02059E+01, 2.9048E-01, 3.9638E-01, &
      2.713869E+02, 7.01888E+01, 2.5909E-01, 3.4518E-01, &
      2.901126E+02, 7.01732E+01, 2.3212E-01, 3.0284E-01, &
      3.101304E+02, 7.01589E+01, 2.0797E-01, 2.6689E-01, &
      3.315294E+02, 7.01457E+01, 1.8635E-01, 2.3630E-01, &
      3.544049E+02, 7.01335E+01, 1.6699E-01, 2.1023E-01, &
      3.788588E+02, 7.01223E+01, 1.4965E-01, 1.8795E-01, &
      4.050001E+02, 7.01119E+01, 1.3412E-01, 1.6884E-01, &
      4.329451E+02, 7.01023E+01, 1.2020E-01, 1.5242E-01]
   case(71)
      frel = -7.7580E-01
      allocate(nist_tab(3600))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.762000E-03, 1.09733E+01, 0.0000E+00, 2.7418E-05, &
      6.865500E-03, 1.07902E+01, 0.0000E+00, 2.9631E-05, &
      6.893100E-03, 1.05760E+01, 0.0000E+00, 3.0244E-05, &
      6.934500E-03, 1.07961E+01, 4.1866E-01, 1.4520E+04, &
      7.038000E-03, 1.09845E+01, 4.0834E-01, 1.3954E+04, &
      1.069000E-02, 1.10473E+01, 2.7823E-01, 6.2595E+03, &
      1.142761E-02, 1.09705E+01, 2.8190E-01, 5.9328E+03, &
      1.221612E-02, 1.08782E+01, 2.9040E-01, 5.7172E+03, &
      1.305903E-02, 1.07684E+01, 3.0371E-01, 5.5933E+03, &
      1.396010E-02, 1.06376E+01, 3.2202E-01, 5.5478E+03, &
      1.492335E-02, 1.04815E+01, 3.4570E-01, 5.5713E+03, &
      1.595306E-02, 1.02939E+01, 3.7529E-01, 5.6577E+03, &
      1.705382E-02, 1.00658E+01, 4.1148E-01, 5.8030E+03, &
      1.823053E-02, 9.78414E+00, 4.5519E-01, 6.0050E+03, &
      1.948844E-02, 9.42840E+00, 5.0748E-01, 6.2628E+03, &
      2.083314E-02, 8.84927E+00, 5.6964E-01, 6.5761E+03, &
      2.227063E-02, 8.21276E+00, 6.4316E-01, 6.9456E+03, &
      2.380730E-02, 7.26572E+00, 7.2979E-01, 7.3724E+03, &
      2.545001E-02, 5.62002E+00, 8.3149E-01, 7.8576E+03, &
      2.720606E-02, 1.17691E+00, 9.5050E-01, 8.4025E+03, &
      2.744000E-02, -2.78294E-01, 9.6715E-01, 8.4768E+03, &
      2.786000E-02, -6.41303E+00, 9.9751E-01, 8.6111E+03, &
      2.797200E-02, -1.39079E+01, 1.0057E+00, 8.6470E+03, &
      2.814000E-02, -5.90857E+00, 1.5849E+01, 1.3546E+05, &
      2.856000E-02, 1.14942E+00, 1.5393E+01, 1.2962E+05, &
      2.908327E-02, 4.64754E+00, 1.4857E+01, 1.2286E+05, &
      3.109002E-02, 1.02590E+01, 1.3092E+01, 1.0127E+05, &
      3.323523E-02, 1.28947E+01, 1.1446E+01, 8.2827E+04, &
      3.552846E-02, 1.43130E+01, 9.9275E+00, 6.7202E+04, &
      3.797993E-02, 1.50174E+01, 8.6395E+00, 5.4709E+04, &
      4.060054E-02, 1.52796E+01, 7.6209E+00, 4.5144E+04, &
      4.340198E-02, 1.52719E+01, 6.8706E+00, 3.8072E+04, &
      4.639671E-02, 1.51082E+01, 6.3673E+00, 3.3006E+04, &
      4.959809E-02, 1.48634E+01, 6.0817E+00, 2.9490E+04, &
      5.302035E-02, 1.45849E+01, 5.9838E+00, 2.7143E+04, &
      5.566400E-02, 1.43716E+01, 6.0153E+00, 2.5990E+04, &
      5.651600E-02, 1.42925E+01, 6.0412E+00, 2.5708E+04, &
      5.667876E-02, 1.42695E+01, 6.0469E+00, 2.5659E+04, &
      5.674320E-02, 1.42545E+01, 6.0493E+00, 2.5640E+04, &
      5.708400E-02, 1.42534E+01, 6.1060E+00, 2.5725E+04, &
      5.793600E-02, 1.42164E+01, 6.1436E+00, 2.5503E+04, &
      6.058959E-02, 1.40740E+01, 6.2959E+00, 2.4991E+04, &
      6.477028E-02, 1.38832E+01, 6.6220E+00, 2.4589E+04, &
      6.923942E-02, 1.37405E+01, 7.0540E+00, 2.4502E+04, &
      7.401695E-02, 1.36575E+01, 7.5802E+00, 2.4631E+04, &
      7.912411E-02, 1.36432E+01, 8.1914E+00, 2.4898E+04, &
      8.458368E-02, 1.37230E+01, 8.8798E+00, 2.5249E+04, &
      9.041995E-02, 1.38739E+01, 9.6388E+00, 2.5638E+04, &
      9.665893E-02, 1.41238E+01, 1.0463E+01, 2.6034E+04, &
      1.0000000E-01, 1.62066E+01, 1.0906E+01, 2.6230E+04, &
      1.0050000E-01, 1.62342E+01, 1.0973E+01, 2.6259E+04, &
      1.0100250E-01, 1.62624E+01, 1.1039E+01, 2.6287E+04, &
      1.0150751E-01, 1.62914E+01, 1.1106E+01, 2.6314E+04, &
      1.0201505E-01, 1.63211E+01, 1.1173E+01, 2.6342E+04, &
      1.0252513E-01, 1.63515E+01, 1.1241E+01, 2.6370E+04, &
      1.0303775E-01, 1.63826E+01, 1.1309E+01, 2.6397E+04, &
      1.0355294E-01, 1.64144E+01, 1.1377E+01, 2.6424E+04, &
      1.0407070E-01, 1.64470E+01, 1.1446E+01, 2.6451E+04, &
      1.0459106E-01, 1.64804E+01, 1.1514E+01, 2.6478E+04, &
      1.0511401E-01, 1.65145E+01, 1.1584E+01, 2.6504E+04, &
      1.0563958E-01, 1.65493E+01, 1.1653E+01, 2.6530E+04, &
      1.0616778E-01, 1.65850E+01, 1.1723E+01, 2.6556E+04, &
      1.0669862E-01, 1.66214E+01, 1.1793E+01, 2.6582E+04, &
      1.0723211E-01, 1.66587E+01, 1.1863E+01, 2.6608E+04, &
      1.0776827E-01, 1.66967E+01, 1.1934E+01, 2.6633E+04, &
      1.0830712E-01, 1.67356E+01, 1.2005E+01, 2.6658E+04, &
      1.0884865E-01, 1.67753E+01, 1.2076E+01, 2.6683E+04, &
      1.0939289E-01, 1.68159E+01, 1.2148E+01, 2.6707E+04, &
      1.0993986E-01, 1.68573E+01, 1.2219E+01, 2.6732E+04, &
      1.1048956E-01, 1.68997E+01, 1.2292E+01, 2.6756E+04, &
      1.1104201E-01, 1.69429E+01, 1.2364E+01, 2.6779E+04, &
      1.1159722E-01, 1.69870E+01, 1.2437E+01, 2.6803E+04, &
      1.1215520E-01, 1.70320E+01, 1.2510E+01, 2.6826E+04, &
      1.1271598E-01, 1.70780E+01, 1.2583E+01, 2.6849E+04, &
      1.1327956E-01, 1.71249E+01, 1.2656E+01, 2.6871E+04, &
      1.1384596E-01, 1.71727E+01, 1.2730E+01, 2.6894E+04, &
      1.1441519E-01, 1.72216E+01, 1.2804E+01, 2.6916E+04, &
      1.1498726E-01, 1.72714E+01, 1.2879E+01, 2.6937E+04, &
      1.1556220E-01, 1.73223E+01, 1.2953E+01, 2.6959E+04, &
      1.1614001E-01, 1.73742E+01, 1.3028E+01, 2.6980E+04, &
      1.1672071E-01, 1.74272E+01, 1.3104E+01, 2.7001E+04, &
      1.1730431E-01, 1.74812E+01, 1.3179E+01, 2.7021E+04, &
      1.1789083E-01, 1.75364E+01, 1.3255E+01, 2.7041E+04, &
      1.1848029E-01, 1.75927E+01, 1.3331E+01, 2.7061E+04, &
      1.1907269E-01, 1.76501E+01, 1.3407E+01, 2.7081E+04, &
      1.1966805E-01, 1.77086E+01, 1.3484E+01, 2.7100E+04, &
      1.2026639E-01, 1.77684E+01, 1.3561E+01, 2.7119E+04, &
      1.2086772E-01, 1.78294E+01, 1.3638E+01, 2.7137E+04, &
      1.2147206E-01, 1.78916E+01, 1.3715E+01, 2.7156E+04, &
      1.2207942E-01, 1.79551E+01, 1.3793E+01, 2.7173E+04, &
      1.2268982E-01, 1.80199E+01, 1.3871E+01, 2.7191E+04, &
      1.2330327E-01, 1.80860E+01, 1.3949E+01, 2.7208E+04, &
      1.2391979E-01, 1.81534E+01, 1.4027E+01, 2.7225E+04, &
      1.2453939E-01, 1.82223E+01, 1.4106E+01, 2.7242E+04, &
      1.2516208E-01, 1.82926E+01, 1.4185E+01, 2.7258E+04, &
      1.2578789E-01, 1.83644E+01, 1.4264E+01, 2.7274E+04, &
      1.2641683E-01, 1.84376E+01, 1.4344E+01, 2.7289E+04, &
      1.2704892E-01, 1.85124E+01, 1.4423E+01, 2.7304E+04, &
      1.2768416E-01, 1.85888E+01, 1.4503E+01, 2.7319E+04, &
      1.2832258E-01, 1.86668E+01, 1.4583E+01, 2.7333E+04, &
      1.2896419E-01, 1.87465E+01, 1.4664E+01, 2.7347E+04, &
      1.2960902E-01, 1.88279E+01, 1.4742E+01, 2.7356E+04, &
      1.3025706E-01, 1.89117E+01, 1.4772E+01, 2.7276E+04, &
      1.3090835E-01, 1.89951E+01, 1.4803E+01, 2.7197E+04, &
      1.3156289E-01, 1.90780E+01, 1.4833E+01, 2.7117E+04, &
      1.3222070E-01, 1.91605E+01, 1.4863E+01, 2.7036E+04, &
      1.3288181E-01, 1.92426E+01, 1.4892E+01, 2.6954E+04, &
      1.3354621E-01, 1.93244E+01, 1.4921E+01, 2.6872E+04, &
      1.3421395E-01, 1.94058E+01, 1.4950E+01, 2.6790E+04, &
      1.3488502E-01, 1.94868E+01, 1.4978E+01, 2.6707E+04, &
      1.3555944E-01, 1.95675E+01, 1.5006E+01, 2.6623E+04, &
      1.3623724E-01, 1.96479E+01, 1.5033E+01, 2.6539E+04, &
      1.3691842E-01, 1.97279E+01, 1.5060E+01, 2.6454E+04, &
      1.3760302E-01, 1.98076E+01, 1.5086E+01, 2.6369E+04, &
      1.3829103E-01, 1.98869E+01, 1.5112E+01, 2.6283E+04, &
      1.3898249E-01, 1.99660E+01, 1.5138E+01, 2.6197E+04, &
      1.3967740E-01, 2.00447E+01, 1.5163E+01, 2.6110E+04, &
      1.4037579E-01, 2.01231E+01, 1.5188E+01, 2.6023E+04, &
      1.4107766E-01, 2.02012E+01, 1.5213E+01, 2.5935E+04, &
      1.4178305E-01, 2.02790E+01, 1.5237E+01, 2.5847E+04, &
      1.4249197E-01, 2.03564E+01, 1.5260E+01, 2.5758E+04, &
      1.4320443E-01, 2.04336E+01, 1.5283E+01, 2.5669E+04, &
      1.4392045E-01, 2.05104E+01, 1.5306E+01, 2.5579E+04, &
      1.4464005E-01, 2.05868E+01, 1.5329E+01, 2.5489E+04, &
      1.4536325E-01, 2.06630E+01, 1.5350E+01, 2.5398E+04, &
      1.4609007E-01, 2.07388E+01, 1.5372E+01, 2.5307E+04, &
      1.4682052E-01, 2.08142E+01, 1.5393E+01, 2.5216E+04, &
      1.4755462E-01, 2.08893E+01, 1.5413E+01, 2.5124E+04, &
      1.4829239E-01, 2.09640E+01, 1.5433E+01, 2.5031E+04, &
      1.4903386E-01, 2.10384E+01, 1.5453E+01, 2.4939E+04, &
      1.4977903E-01, 2.11123E+01, 1.5472E+01, 2.4845E+04, &
      1.5052792E-01, 2.11859E+01, 1.5491E+01, 2.4752E+04, &
      1.5128056E-01, 2.12590E+01, 1.5509E+01, 2.4658E+04, &
      1.5203696E-01, 2.13317E+01, 1.5527E+01, 2.4563E+04, &
      1.5279715E-01, 2.14040E+01, 1.5545E+01, 2.4468E+04, &
      1.5356113E-01, 2.14757E+01, 1.5562E+01, 2.4373E+04, &
      1.5432894E-01, 2.15470E+01, 1.5578E+01, 2.4278E+04, &
      1.5510058E-01, 2.16178E+01, 1.5594E+01, 2.4182E+04, &
      1.5587609E-01, 2.16881E+01, 1.5610E+01, 2.4086E+04, &
      1.5665547E-01, 2.17578E+01, 1.5625E+01, 2.3989E+04, &
      1.5743875E-01, 2.18269E+01, 1.5639E+01, 2.3892E+04, &
      1.5822594E-01, 2.18955E+01, 1.5654E+01, 2.3795E+04, &
      1.5901707E-01, 2.19633E+01, 1.5667E+01, 2.3697E+04, &
      1.5981215E-01, 2.20306E+01, 1.5680E+01, 2.3599E+04, &
      1.6061121E-01, 2.20971E+01, 1.5693E+01, 2.3501E+04, &
      1.6141427E-01, 2.21628E+01, 1.5706E+01, 2.3402E+04, &
      1.6222134E-01, 2.22278E+01, 1.5717E+01, 2.3304E+04, &
      1.6303245E-01, 2.22919E+01, 1.5729E+01, 2.3204E+04, &
      1.6384761E-01, 2.23552E+01, 1.5740E+01, 2.3105E+04, &
      1.6466685E-01, 2.24175E+01, 1.5750E+01, 2.3005E+04, &
      1.6549018E-01, 2.24789E+01, 1.5760E+01, 2.2905E+04, &
      1.6631763E-01, 2.25391E+01, 1.5770E+01, 2.2805E+04, &
      1.6714922E-01, 2.25983E+01, 1.5779E+01, 2.2705E+04, &
      1.6798497E-01, 2.26562E+01, 1.5787E+01, 2.2604E+04, &
      1.6882489E-01, 2.27128E+01, 1.5795E+01, 2.2503E+04, &
      1.6966902E-01, 2.27681E+01, 1.5803E+01, 2.2402E+04, &
      1.7051736E-01, 2.28236E+01, 1.5810E+01, 2.2301E+04, &
      1.7136995E-01, 2.28758E+01, 1.5817E+01, 2.2199E+04, &
      1.7222680E-01, 2.29262E+01, 1.5823E+01, 2.2097E+04, &
      1.7308793E-01, 2.29747E+01, 1.5829E+01, 2.1995E+04, &
      1.7395337E-01, 2.30211E+01, 1.5834E+01, 2.1893E+04, &
      1.7482314E-01, 2.30653E+01, 1.5839E+01, 2.1791E+04, &
      1.7569726E-01, 2.31071E+01, 1.5843E+01, 2.1688E+04, &
      1.7657574E-01, 2.31461E+01, 1.5847E+01, 2.1586E+04, &
      1.7745862E-01, 2.31821E+01, 1.5850E+01, 2.1483E+04, &
      1.7834591E-01, 2.32148E+01, 1.5853E+01, 2.1380E+04, &
      1.7923764E-01, 2.32437E+01, 1.5855E+01, 2.1277E+04, &
      1.8013383E-01, 2.32685E+01, 1.5857E+01, 2.1173E+04, &
      1.8103450E-01, 2.32885E+01, 1.5859E+01, 2.1070E+04, &
      1.8193967E-01, 2.33030E+01, 1.5860E+01, 2.0966E+04, &
      1.8284937E-01, 2.33113E+01, 1.5860E+01, 2.0863E+04, &
      1.8376362E-01, 2.33123E+01, 1.5860E+01, 2.0759E+04, &
      1.8468244E-01, 2.33048E+01, 1.5860E+01, 2.0655E+04, &
      1.8560585E-01, 2.32869E+01, 1.5859E+01, 2.0551E+04, &
      1.8653388E-01, 2.32566E+01, 1.5858E+01, 2.0447E+04, &
      1.8746655E-01, 2.32107E+01, 1.5856E+01, 2.0343E+04, &
      1.8840388E-01, 2.31449E+01, 1.5853E+01, 2.0239E+04, &
      1.8934590E-01, 2.30531E+01, 1.5851E+01, 2.0135E+04, &
      1.9029263E-01, 2.29251E+01, 1.5847E+01, 2.0031E+04, &
      1.9124409E-01, 2.27442E+01, 1.5844E+01, 1.9926E+04, &
      1.9220031E-01, 2.24780E+01, 1.5840E+01, 1.9822E+04, &
      1.9316131E-01, 2.20516E+01, 1.5835E+01, 1.9718E+04, &
      1.9412712E-01, 2.12081E+01, 1.5830E+01, 1.9613E+04, &
      1.9485647E-01, 1.89299E+01, 1.5826E+01, 1.9535E+04, &
      1.9509776E-01, 1.84860E+01, 2.0045E+01, 2.4712E+04, &
      1.9514351E-01, 1.90163E+01, 2.0037E+01, 2.4696E+04, &
      1.9607325E-01, 2.18997E+01, 1.9876E+01, 2.4382E+04, &
      1.9705361E-01, 2.28823E+01, 1.9713E+01, 2.4062E+04, &
      1.9803888E-01, 2.34754E+01, 1.9555E+01, 2.3751E+04, &
      1.9902907E-01, 2.38829E+01, 1.9403E+01, 2.3449E+04, &
      2.0002422E-01, 2.41682E+01, 1.9256E+01, 2.3155E+04, &
      2.0102434E-01, 2.43518E+01, 1.9114E+01, 2.2870E+04, &
      2.0202946E-01, 2.44274E+01, 1.8977E+01, 2.2593E+04, &
      2.0303961E-01, 2.43464E+01, 1.8844E+01, 2.2323E+04, &
      2.0405481E-01, 2.38847E+01, 1.8715E+01, 2.2060E+04, &
      2.0462018E-01, 2.27717E+01, 1.8646E+01, 2.1918E+04, &
      2.0497981E-01, 2.29072E+01, 2.1463E+01, 2.5184E+04, &
      2.0507508E-01, 2.33293E+01, 2.1441E+01, 2.5147E+04, &
      2.0610046E-01, 2.50450E+01, 2.1211E+01, 2.4754E+04, &
      2.0713096E-01, 2.58196E+01, 2.0989E+01, 2.4373E+04, &
      2.0816661E-01, 2.63637E+01, 2.0774E+01, 2.4004E+04, &
      2.0920745E-01, 2.67927E+01, 2.0567E+01, 2.3646E+04, &
      2.1025348E-01, 2.71494E+01, 2.0367E+01, 2.3300E+04, &
      2.1130475E-01, 2.74552E+01, 2.0174E+01, 2.2964E+04, &
      2.1236128E-01, 2.77223E+01, 1.9988E+01, 2.2639E+04, &
      2.1342308E-01, 2.79584E+01, 1.9807E+01, 2.2323E+04, &
      2.1449020E-01, 2.81689E+01, 1.9633E+01, 2.2016E+04, &
      2.1556265E-01, 2.83576E+01, 1.9465E+01, 2.1719E+04, &
      2.1664046E-01, 2.85278E+01, 1.9305E+01, 2.1434E+04, &
      2.1772366E-01, 2.86828E+01, 1.9152E+01, 2.1158E+04, &
      2.1881228E-01, 2.88248E+01, 1.9006E+01, 2.0892E+04, &
      2.1990634E-01, 2.89556E+01, 1.8866E+01, 2.0636E+04, &
      2.2100588E-01, 2.90766E+01, 1.8732E+01, 2.0387E+04, &
      2.2211090E-01, 2.91891E+01, 1.8604E+01, 2.0147E+04, &
      2.2322146E-01, 2.92938E+01, 1.8481E+01, 1.9914E+04, &
      2.2433757E-01, 2.93916E+01, 1.8362E+01, 1.9688E+04, &
      2.2545925E-01, 2.94829E+01, 1.8249E+01, 1.9469E+04, &
      2.2658655E-01, 2.95683E+01, 1.8140E+01, 1.9256E+04, &
      2.2771948E-01, 2.96486E+01, 1.8037E+01, 1.9052E+04, &
      2.2885808E-01, 2.97249E+01, 1.7939E+01, 1.8854E+04, &
      2.3000237E-01, 2.97976E+01, 1.7845E+01, 1.8662E+04, &
      2.3115238E-01, 2.98674E+01, 1.7756E+01, 1.8477E+04, &
      2.3230814E-01, 2.99346E+01, 1.7671E+01, 1.8297E+04, &
      2.3346969E-01, 2.99996E+01, 1.7590E+01, 1.8122E+04, &
      2.3463703E-01, 3.00627E+01, 1.7512E+01, 1.7952E+04, &
      2.3581022E-01, 3.01241E+01, 1.7437E+01, 1.7787E+04, &
      2.3698927E-01, 3.01841E+01, 1.7365E+01, 1.7625E+04, &
      2.3817422E-01, 3.02427E+01, 1.7296E+01, 1.7468E+04, &
      2.3936509E-01, 3.03002E+01, 1.7230E+01, 1.7314E+04, &
      2.4056191E-01, 3.03566E+01, 1.7166E+01, 1.7164E+04, &
      2.4176472E-01, 3.04122E+01, 1.7104E+01, 1.7017E+04, &
      2.4297355E-01, 3.04668E+01, 1.7044E+01, 1.6873E+04, &
      2.4418841E-01, 3.05207E+01, 1.6986E+01, 1.6732E+04, &
      2.4540936E-01, 3.05739E+01, 1.6930E+01, 1.6594E+04, &
      2.4663640E-01, 3.06265E+01, 1.6875E+01, 1.6458E+04, &
      2.4786959E-01, 3.06784E+01, 1.6822E+01, 1.6325E+04, &
      2.4910893E-01, 3.07298E+01, 1.6771E+01, 1.6194E+04, &
      2.5035448E-01, 3.07807E+01, 1.6720E+01, 1.6065E+04, &
      2.5160625E-01, 3.08312E+01, 1.6672E+01, 1.5939E+04, &
      2.5286428E-01, 3.08811E+01, 1.6624E+01, 1.5814E+04, &
      2.5412860E-01, 3.09306E+01, 1.6577E+01, 1.5691E+04, &
      2.5539925E-01, 3.09797E+01, 1.6532E+01, 1.5570E+04, &
      2.5667624E-01, 3.10284E+01, 1.6487E+01, 1.5451E+04, &
      2.5795962E-01, 3.10766E+01, 1.6443E+01, 1.5333E+04, &
      2.5924942E-01, 3.11245E+01, 1.6401E+01, 1.5218E+04, &
      2.6054567E-01, 3.11720E+01, 1.6359E+01, 1.5103E+04, &
      2.6184840E-01, 3.12192E+01, 1.6317E+01, 1.4990E+04, &
      2.6315764E-01, 3.12659E+01, 1.6277E+01, 1.4879E+04, &
      2.6447343E-01, 3.13123E+01, 1.6237E+01, 1.4768E+04, &
      2.6579579E-01, 3.13583E+01, 1.6198E+01, 1.4660E+04, &
      2.6712477E-01, 3.14040E+01, 1.6160E+01, 1.4552E+04, &
      2.6846040E-01, 3.14493E+01, 1.6122E+01, 1.4446E+04, &
      2.6980270E-01, 3.14942E+01, 1.6084E+01, 1.4341E+04, &
      2.7115171E-01, 3.15387E+01, 1.6048E+01, 1.4237E+04, &
      2.7250747E-01, 3.15829E+01, 1.6011E+01, 1.4134E+04, &
      2.7387001E-01, 3.16266E+01, 1.5975E+01, 1.4032E+04, &
      2.7523936E-01, 3.16700E+01, 1.5940E+01, 1.3931E+04, &
      2.7661556E-01, 3.17130E+01, 1.5905E+01, 1.3832E+04, &
      2.7799863E-01, 3.17555E+01, 1.5871E+01, 1.3733E+04, &
      2.7938863E-01, 3.17977E+01, 1.5836E+01, 1.3635E+04, &
      2.8078557E-01, 3.18394E+01, 1.5803E+01, 1.3539E+04, &
      2.8218950E-01, 3.18807E+01, 1.5769E+01, 1.3443E+04, &
      2.8360044E-01, 3.19216E+01, 1.5736E+01, 1.3348E+04, &
      2.8501845E-01, 3.19620E+01, 1.5704E+01, 1.3254E+04, &
      2.8644354E-01, 3.20020E+01, 1.5672E+01, 1.3161E+04, &
      2.8787576E-01, 3.20415E+01, 1.5640E+01, 1.3069E+04, &
      2.8931514E-01, 3.20805E+01, 1.5608E+01, 1.2978E+04, &
      2.9076171E-01, 3.21191E+01, 1.5577E+01, 1.2887E+04, &
      2.9221552E-01, 3.21571E+01, 1.5546E+01, 1.2798E+04, &
      2.9367660E-01, 3.21947E+01, 1.5515E+01, 1.2709E+04, &
      2.9514498E-01, 3.22317E+01, 1.5484E+01, 1.2621E+04, &
      2.9662071E-01, 3.22681E+01, 1.5454E+01, 1.2534E+04, &
      2.9810381E-01, 3.23040E+01, 1.5424E+01, 1.2447E+04, &
      2.9959433E-01, 3.23394E+01, 1.5395E+01, 1.2362E+04, &
      3.0109230E-01, 3.23741E+01, 1.5365E+01, 1.2277E+04, &
      3.0259776E-01, 3.24082E+01, 1.5336E+01, 1.2193E+04, &
      3.0411075E-01, 3.24417E+01, 1.5307E+01, 1.2109E+04, &
      3.0563130E-01, 3.24746E+01, 1.5279E+01, 1.2027E+04, &
      3.0715946E-01, 3.25067E+01, 1.5251E+01, 1.1945E+04, &
      3.0869526E-01, 3.25382E+01, 1.5222E+01, 1.1863E+04, &
      3.1023873E-01, 3.25689E+01, 1.5195E+01, 1.1783E+04, &
      3.1178993E-01, 3.25988E+01, 1.5167E+01, 1.1703E+04, &
      3.1334888E-01, 3.26280E+01, 1.5140E+01, 1.1624E+04, &
      3.1491562E-01, 3.26563E+01, 1.5113E+01, 1.1545E+04, &
      3.1649020E-01, 3.26837E+01, 1.5086E+01, 1.1467E+04, &
      3.1807265E-01, 3.27102E+01, 1.5059E+01, 1.1390E+04, &
      3.1966301E-01, 3.27357E+01, 1.5033E+01, 1.1314E+04, &
      3.2126133E-01, 3.27601E+01, 1.5007E+01, 1.1238E+04, &
      3.2286764E-01, 3.27835E+01, 1.4981E+01, 1.1163E+04, &
      3.2448197E-01, 3.28056E+01, 1.4955E+01, 1.1088E+04, &
      3.2610438E-01, 3.28264E+01, 1.4930E+01, 1.1015E+04, &
      3.2773491E-01, 3.28459E+01, 1.4905E+01, 1.0941E+04, &
      3.2937358E-01, 3.28638E+01, 1.4880E+01, 1.0869E+04, &
      3.3102045E-01, 3.28800E+01, 1.4855E+01, 1.0797E+04, &
      3.3267555E-01, 3.28944E+01, 1.4831E+01, 1.0726E+04, &
      3.3433893E-01, 3.29068E+01, 1.4806E+01, 1.0655E+04, &
      3.3601062E-01, 3.29169E+01, 1.4783E+01, 1.0585E+04, &
      3.3769068E-01, 3.29243E+01, 1.4759E+01, 1.0515E+04, &
      3.3937913E-01, 3.29288E+01, 1.4735E+01, 1.0446E+04, &
      3.4107602E-01, 3.29298E+01, 1.4712E+01, 1.0378E+04, &
      3.4278140E-01, 3.29266E+01, 1.4689E+01, 1.0310E+04, &
      3.4449531E-01, 3.29185E+01, 1.4666E+01, 1.0243E+04, &
      3.4621779E-01, 3.29042E+01, 1.4644E+01, 1.0177E+04, &
      3.4794888E-01, 3.28821E+01, 1.4622E+01, 1.0111E+04, &
      3.4968862E-01, 3.28496E+01, 1.4600E+01, 1.0045E+04, &
      3.5143706E-01, 3.28026E+01, 1.4578E+01, 9.9805E+03, &
      3.5319425E-01, 3.27339E+01, 1.4556E+01, 9.9162E+03, &
      3.5496022E-01, 3.26287E+01, 1.4535E+01, 9.8525E+03, &
      3.5673502E-01, 3.24476E+01, 1.4514E+01, 9.7894E+03, &
      3.5851870E-01, 3.19961E+01, 1.4493E+01, 9.7268E+03, &
      3.5885446E-01, 3.17736E+01, 1.4489E+01, 9.7151E+03, &
      3.5974552E-01, 3.17880E+01, 1.5756E+01, 1.0537E+04, &
      3.6031129E-01, 3.21302E+01, 1.5750E+01, 1.0517E+04, &
      3.6211285E-01, 3.25763E+01, 1.5734E+01, 1.0454E+04, &
      3.6392341E-01, 3.28094E+01, 1.5718E+01, 1.0392E+04, &
      3.6574303E-01, 3.29756E+01, 1.5702E+01, 1.0330E+04, &
      3.6757174E-01, 3.31086E+01, 1.5686E+01, 1.0268E+04, &
      3.6940960E-01, 3.32215E+01, 1.5671E+01, 1.0207E+04, &
      3.7125665E-01, 3.33210E+01, 1.5656E+01, 1.0146E+04, &
      3.7311293E-01, 3.34107E+01, 1.5641E+01, 1.0086E+04, &
      3.7497850E-01, 3.34930E+01, 1.5627E+01, 1.0027E+04, &
      3.7685339E-01, 3.35695E+01, 1.5612E+01, 9.9680E+03, &
      3.7873766E-01, 3.36411E+01, 1.5598E+01, 9.9095E+03, &
      3.8063135E-01, 3.37086E+01, 1.5584E+01, 9.8515E+03, &
      3.8253450E-01, 3.37727E+01, 1.5571E+01, 9.7940E+03, &
      3.8444718E-01, 3.38336E+01, 1.5557E+01, 9.7369E+03, &
      3.8636941E-01, 3.38917E+01, 1.5544E+01, 9.6804E+03, &
      3.8830126E-01, 3.39470E+01, 1.5531E+01, 9.6243E+03, &
      3.9024276E-01, 3.39998E+01, 1.5519E+01, 9.5687E+03, &
      3.9219398E-01, 3.40501E+01, 1.5506E+01, 9.5135E+03, &
      3.9415495E-01, 3.40976E+01, 1.5494E+01, 9.4588E+03, &
      3.9612572E-01, 3.41422E+01, 1.5482E+01, 9.4045E+03, &
      3.9810635E-01, 3.41836E+01, 1.5470E+01, 9.3507E+03, &
      4.0009688E-01, 3.42208E+01, 1.5459E+01, 9.2973E+03, &
      4.0209737E-01, 3.42527E+01, 1.5447E+01, 9.2443E+03, &
      4.0410785E-01, 3.42766E+01, 1.5436E+01, 9.1917E+03, &
      4.0612839E-01, 3.42867E+01, 1.5425E+01, 9.1395E+03, &
      4.0815904E-01, 3.42633E+01, 1.5415E+01, 9.0878E+03, &
      4.0952587E-01, 3.41682E+01, 1.5407E+01, 9.0533E+03, &
      4.1019983E-01, 3.39986E+01, 1.5739E+01, 9.2327E+03, &
      4.1067415E-01, 3.41968E+01, 1.5737E+01, 9.2209E+03, &
      4.1225083E-01, 3.43770E+01, 1.5730E+01, 9.1817E+03, &
      4.1431208E-01, 3.44999E+01, 1.5722E+01, 9.1311E+03, &
      4.1638364E-01, 3.45937E+01, 1.5713E+01, 9.0809E+03, &
      4.1846556E-01, 3.46750E+01, 1.5705E+01, 9.0311E+03, &
      4.2055789E-01, 3.47494E+01, 1.5697E+01, 8.9816E+03, &
      4.2266068E-01, 3.48192E+01, 1.5689E+01, 8.9324E+03, &
      4.2477398E-01, 3.48858E+01, 1.5681E+01, 8.8837E+03, &
      4.2689785E-01, 3.49499E+01, 1.5674E+01, 8.8352E+03, &
      4.2903234E-01, 3.50121E+01, 1.5666E+01, 8.7872E+03, &
      4.3117750E-01, 3.50728E+01, 1.5659E+01, 8.7394E+03, &
      4.3333339E-01, 3.51321E+01, 1.5652E+01, 8.6920E+03, &
      4.3550006E-01, 3.51903E+01, 1.5645E+01, 8.6449E+03, &
      4.3767756E-01, 3.52475E+01, 1.5638E+01, 8.5981E+03, &
      4.3986595E-01, 3.53038E+01, 1.5631E+01, 8.5517E+03, &
      4.4206528E-01, 3.53593E+01, 1.5624E+01, 8.5055E+03, &
      4.4427560E-01, 3.54141E+01, 1.5617E+01, 8.4597E+03, &
      4.4649698E-01, 3.54683E+01, 1.5611E+01, 8.4141E+03, &
      4.4872947E-01, 3.55218E+01, 1.5604E+01, 8.3689E+03, &
      4.5097311E-01, 3.55746E+01, 1.5598E+01, 8.3239E+03, &
      4.5322798E-01, 3.56269E+01, 1.5592E+01, 8.2792E+03, &
      4.5549412E-01, 3.56786E+01, 1.5586E+01, 8.2348E+03, &
      4.5777159E-01, 3.57298E+01, 1.5579E+01, 8.1906E+03, &
      4.6006045E-01, 3.57803E+01, 1.5573E+01, 8.1467E+03, &
      4.6236075E-01, 3.58302E+01, 1.5567E+01, 8.1031E+03, &
      4.6467255E-01, 3.58795E+01, 1.5561E+01, 8.0597E+03, &
      4.6699592E-01, 3.59281E+01, 1.5555E+01, 8.0165E+03, &
      4.6933090E-01, 3.59760E+01, 1.5549E+01, 7.9736E+03, &
      4.7167755E-01, 3.60230E+01, 1.5543E+01, 7.9310E+03, &
      4.7403594E-01, 3.60692E+01, 1.5537E+01, 7.8885E+03, &
      4.7640612E-01, 3.61143E+01, 1.5531E+01, 7.8463E+03, &
      4.7878815E-01, 3.61583E+01, 1.5525E+01, 7.8043E+03, &
      4.8118209E-01, 3.62009E+01, 1.5519E+01, 7.7625E+03, &
      4.8358800E-01, 3.62418E+01, 1.5513E+01, 7.7209E+03, &
      4.8600594E-01, 3.62807E+01, 1.5507E+01, 7.6796E+03, &
      4.8843597E-01, 3.63171E+01, 1.5501E+01, 7.6384E+03, &
      4.9087815E-01, 3.63502E+01, 1.5495E+01, 7.5974E+03, &
      4.9333254E-01, 3.63789E+01, 1.5488E+01, 7.5566E+03, &
      4.9579920E-01, 3.64012E+01, 1.5482E+01, 7.5160E+03, &
      4.9827820E-01, 3.64136E+01, 1.5476E+01, 7.4756E+03, &
      5.0076959E-01, 3.64085E+01, 1.5469E+01, 7.4354E+03, &
      5.0327344E-01, 3.63633E+01, 1.5463E+01, 7.3953E+03, &
      5.0541541E-01, 3.61934E+01, 1.5457E+01, 7.3613E+03, &
      5.0578980E-01, 3.60943E+01, 1.5456E+01, 7.3554E+03, &
      5.0698463E-01, 3.62288E+01, 1.5974E+01, 7.5840E+03, &
      5.0831875E-01, 3.64238E+01, 1.5971E+01, 7.5627E+03, &
      5.1086035E-01, 3.66121E+01, 1.5966E+01, 7.5223E+03, &
      5.1341465E-01, 3.67424E+01, 1.5960E+01, 7.4822E+03, &
      5.1598172E-01, 3.68509E+01, 1.5953E+01, 7.4421E+03, &
      5.1856163E-01, 3.69478E+01, 1.5947E+01, 7.4023E+03, &
      5.2115444E-01, 3.70377E+01, 1.5941E+01, 7.3625E+03, &
      5.2376021E-01, 3.71227E+01, 1.5934E+01, 7.3229E+03, &
      5.2637901E-01, 3.72042E+01, 1.5927E+01, 7.2835E+03, &
      5.2901091E-01, 3.72831E+01, 1.5921E+01, 7.2442E+03, &
      5.3165596E-01, 3.73600E+01, 1.5913E+01, 7.2050E+03, &
      5.3431424E-01, 3.74353E+01, 1.5906E+01, 7.1659E+03, &
      5.3698581E-01, 3.75093E+01, 1.5899E+01, 7.1270E+03, &
      5.3967074E-01, 3.75821E+01, 1.5891E+01, 7.0881E+03, &
      5.4236910E-01, 3.76541E+01, 1.5883E+01, 7.0495E+03, &
      5.4508094E-01, 3.77252E+01, 1.5875E+01, 7.0109E+03, &
      5.4780635E-01, 3.77957E+01, 1.5867E+01, 6.9724E+03, &
      5.5054538E-01, 3.78656E+01, 1.5858E+01, 6.9341E+03, &
      5.5329810E-01, 3.79350E+01, 1.5850E+01, 6.8959E+03, &
      5.5606460E-01, 3.80040E+01, 1.5841E+01, 6.8578E+03, &
      5.5884492E-01, 3.80725E+01, 1.5831E+01, 6.8197E+03, &
      5.6163914E-01, 3.81407E+01, 1.5822E+01, 6.7818E+03, &
      5.6444734E-01, 3.82086E+01, 1.5812E+01, 6.7440E+03, &
      5.6726958E-01, 3.82763E+01, 1.5802E+01, 6.7063E+03, &
      5.7010592E-01, 3.83436E+01, 1.5792E+01, 6.6686E+03, &
      5.7295645E-01, 3.84107E+01, 1.5782E+01, 6.6311E+03, &
      5.7582123E-01, 3.84775E+01, 1.5771E+01, 6.5936E+03, &
      5.7870034E-01, 3.85442E+01, 1.5760E+01, 6.5563E+03, &
      5.8159384E-01, 3.86106E+01, 1.5748E+01, 6.5190E+03, &
      5.8450181E-01, 3.86768E+01, 1.5736E+01, 6.4818E+03, &
      5.8742432E-01, 3.87608E+01, 1.5724E+01, 6.4447E+03, &
      5.9036144E-01, 3.88266E+01, 1.5712E+01, 6.4076E+03, &
      5.9331325E-01, 3.88922E+01, 1.5699E+01, 6.3707E+03, &
      5.9627982E-01, 3.89577E+01, 1.5686E+01, 6.3338E+03, &
      5.9926122E-01, 3.90230E+01, 1.5673E+01, 6.2970E+03, &
      6.0225752E-01, 3.90881E+01, 1.5659E+01, 6.2603E+03, &
      6.0526881E-01, 3.91531E+01, 1.5645E+01, 6.2236E+03, &
      6.0829515E-01, 3.92178E+01, 1.5631E+01, 6.1871E+03, &
      6.1133663E-01, 3.92824E+01, 1.5616E+01, 6.1506E+03, &
      6.1439331E-01, 3.93469E+01, 1.5601E+01, 6.1142E+03, &
      6.1746528E-01, 3.94010E+01, 1.5586E+01, 6.0779E+03, &
      6.2055260E-01, 3.94650E+01, 1.5571E+01, 6.0416E+03, &
      6.2365537E-01, 3.95288E+01, 1.5555E+01, 6.0055E+03, &
      6.2677364E-01, 3.95924E+01, 1.5538E+01, 5.9694E+03, &
      6.2990751E-01, 3.96558E+01, 1.5521E+01, 5.9334E+03, &
      6.3305705E-01, 3.97190E+01, 1.5504E+01, 5.8974E+03, &
      6.3622234E-01, 3.97821E+01, 1.5487E+01, 5.8616E+03, &
      6.3940345E-01, 3.98449E+01, 1.5469E+01, 5.8258E+03, &
      6.4260046E-01, 3.99075E+01, 1.5451E+01, 5.7901E+03, &
      6.4581347E-01, 3.99700E+01, 1.5433E+01, 5.7545E+03, &
      6.4904253E-01, 4.00322E+01, 1.5414E+01, 5.7190E+03, &
      6.5228775E-01, 4.00942E+01, 1.5395E+01, 5.6835E+03, &
      6.5554919E-01, 4.01560E+01, 1.5375E+01, 5.6481E+03, &
      6.5882693E-01, 4.02176E+01, 1.5355E+01, 5.6128E+03, &
      6.6212107E-01, 4.02789E+01, 1.5335E+01, 5.5776E+03, &
      6.6543167E-01, 4.03401E+01, 1.5315E+01, 5.5425E+03, &
      6.6875883E-01, 4.04009E+01, 1.5294E+01, 5.5074E+03, &
      6.7210262E-01, 4.04616E+01, 1.5272E+01, 5.4725E+03, &
      6.7546314E-01, 4.05220E+01, 1.5251E+01, 5.4376E+03, &
      6.7884045E-01, 4.05822E+01, 1.5229E+01, 5.4028E+03, &
      6.8223466E-01, 4.06421E+01, 1.5206E+01, 5.3681E+03, &
      6.8564583E-01, 4.07018E+01, 1.5184E+01, 5.3335E+03, &
      6.8907406E-01, 4.07612E+01, 1.5160E+01, 5.2990E+03, &
      6.9251943E-01, 4.08204E+01, 1.5137E+01, 5.2645E+03, &
      6.9598202E-01, 4.08793E+01, 1.5113E+01, 5.2300E+03, &
      6.9946194E-01, 4.09378E+01, 1.5088E+01, 5.1956E+03, &
      7.0295924E-01, 4.09960E+01, 1.5063E+01, 5.1613E+03, &
      7.0647404E-01, 4.10538E+01, 1.5038E+01, 5.1270E+03, &
      7.1000641E-01, 4.11113E+01, 1.5012E+01, 5.0929E+03, &
      7.1355644E-01, 4.11684E+01, 1.4986E+01, 5.0588E+03, &
      7.1712423E-01, 4.12252E+01, 1.4960E+01, 5.0249E+03, &
      7.2070985E-01, 4.12815E+01, 1.4933E+01, 4.9910E+03, &
      7.2431340E-01, 4.13375E+01, 1.4906E+01, 4.9572E+03, &
      7.2793496E-01, 4.13932E+01, 1.4878E+01, 4.9236E+03, &
      7.3157464E-01, 4.14484E+01, 1.4851E+01, 4.8900E+03, &
      7.3523251E-01, 4.15033E+01, 1.4822E+01, 4.8565E+03, &
      7.3890867E-01, 4.15577E+01, 1.4794E+01, 4.8232E+03, &
      7.4260322E-01, 4.16118E+01, 1.4765E+01, 4.7899E+03, &
      7.4631623E-01, 4.16654E+01, 1.4736E+01, 4.7568E+03, &
      7.5004781E-01, 4.17186E+01, 1.4707E+01, 4.7237E+03, &
      7.5379805E-01, 4.17714E+01, 1.4677E+01, 4.6908E+03, &
      7.5756704E-01, 4.18238E+01, 1.4647E+01, 4.6580E+03, &
      7.6135488E-01, 4.18757E+01, 1.4616E+01, 4.6253E+03, &
      7.6516165E-01, 4.19272E+01, 1.4586E+01, 4.5927E+03, &
      7.6898746E-01, 4.19783E+01, 1.4555E+01, 4.5602E+03, &
      7.7283240E-01, 4.20289E+01, 1.4523E+01, 4.5278E+03, &
      7.7669656E-01, 4.20790E+01, 1.4492E+01, 4.4955E+03, &
      7.8058004E-01, 4.21287E+01, 1.4460E+01, 4.4634E+03, &
      7.8448294E-01, 4.21779E+01, 1.4428E+01, 4.4314E+03, &
      7.8840536E-01, 4.22267E+01, 1.4395E+01, 4.3994E+03, &
      7.9234738E-01, 4.22749E+01, 1.4362E+01, 4.3676E+03, &
      7.9630912E-01, 4.23226E+01, 1.4329E+01, 4.3359E+03, &
      8.0029067E-01, 4.23699E+01, 1.4295E+01, 4.3043E+03, &
      8.0429212E-01, 4.24166E+01, 1.4261E+01, 4.2728E+03, &
      8.0831358E-01, 4.24627E+01, 1.4227E+01, 4.2415E+03, &
      8.1235515E-01, 4.25084E+01, 1.4193E+01, 4.2102E+03, &
      8.1641693E-01, 4.25534E+01, 1.4158E+01, 4.1791E+03, &
      8.2049901E-01, 4.25979E+01, 1.4123E+01, 4.1481E+03, &
      8.2460150E-01, 4.26419E+01, 1.4088E+01, 4.1172E+03, &
      8.2872451E-01, 4.26853E+01, 1.4052E+01, 4.0865E+03, &
      8.3286813E-01, 4.27281E+01, 1.4016E+01, 4.0558E+03, &
      8.3703248E-01, 4.27703E+01, 1.3980E+01, 4.0253E+03, &
      8.4121764E-01, 4.28119E+01, 1.3943E+01, 3.9949E+03, &
      8.4542373E-01, 4.28529E+01, 1.3907E+01, 3.9647E+03, &
      8.4965084E-01, 4.28933E+01, 1.3870E+01, 3.9345E+03, &
      8.5389910E-01, 4.29330E+01, 1.3832E+01, 3.9046E+03, &
      8.5816859E-01, 4.29721E+01, 1.3795E+01, 3.8747E+03, &
      8.6245944E-01, 4.30106E+01, 1.3757E+01, 3.8449E+03, &
      8.6677173E-01, 4.30483E+01, 1.3719E+01, 3.8152E+03, &
      8.7110559E-01, 4.30854E+01, 1.3681E+01, 3.7857E+03, &
      8.7546112E-01, 4.31217E+01, 1.3642E+01, 3.7564E+03, &
      8.7983843E-01, 4.31574E+01, 1.3603E+01, 3.7272E+03, &
      8.8423762E-01, 4.31923E+01, 1.3565E+01, 3.6982E+03, &
      8.8865881E-01, 4.32266E+01, 1.3526E+01, 3.6693E+03, &
      8.9310210E-01, 4.32601E+01, 1.3487E+01, 3.6406E+03, &
      8.9756761E-01, 4.32929E+01, 1.3447E+01, 3.6120E+03, &
      9.0205545E-01, 4.33250E+01, 1.3408E+01, 3.5836E+03, &
      9.0656573E-01, 4.33564E+01, 1.3368E+01, 3.5554E+03, &
      9.1109856E-01, 4.33870E+01, 1.3329E+01, 3.5273E+03, &
      9.1565405E-01, 4.34170E+01, 1.3289E+01, 3.4994E+03, &
      9.2023232E-01, 4.34462E+01, 1.3249E+01, 3.4717E+03, &
      9.2483348E-01, 4.34747E+01, 1.3210E+01, 3.4441E+03, &
      9.2945765E-01, 4.35024E+01, 1.3170E+01, 3.4167E+03, &
      9.3410494E-01, 4.35294E+01, 1.3130E+01, 3.3895E+03, &
      9.3877546E-01, 4.35557E+01, 1.3090E+01, 3.3625E+03, &
      9.4346934E-01, 4.35813E+01, 1.3050E+01, 3.3356E+03, &
      9.4818668E-01, 4.36062E+01, 1.3010E+01, 3.3089E+03, &
      9.5292762E-01, 4.36303E+01, 1.2970E+01, 3.2824E+03, &
      9.5769226E-01, 4.36537E+01, 1.2930E+01, 3.2561E+03, &
      9.6248072E-01, 4.36764E+01, 1.2890E+01, 3.2299E+03, &
      9.6729312E-01, 4.36984E+01, 1.2850E+01, 3.2040E+03, &
      9.7212959E-01, 4.37197E+01, 1.2810E+01, 3.1782E+03, &
      9.7699023E-01, 4.37404E+01, 1.2770E+01, 3.1526E+03, &
      9.8187519E-01, 4.37606E+01, 1.2730E+01, 3.1272E+03, &
      9.8678456E-01, 4.37804E+01, 1.2690E+01, 3.1019E+03, &
      9.9171848E-01, 4.38005E+01, 1.2650E+01, 3.0769E+03, &
      9.9667708E-01, 4.38254E+01, 1.2610E+01, 3.0520E+03, &
      1.0016605E+00, 4.40197E+01, 1.2556E+01, 3.0240E+03, &
      1.0066688E+00, 4.40422E+01, 1.2475E+01, 2.9896E+03, &
      1.0117021E+00, 4.40588E+01, 1.2394E+01, 2.9556E+03, &
      1.0167606E+00, 4.40703E+01, 1.2314E+01, 2.9220E+03, &
      1.0218444E+00, 4.40775E+01, 1.2235E+01, 2.8889E+03, &
      1.0269536E+00, 4.40810E+01, 1.2156E+01, 2.8562E+03, &
      1.0320884E+00, 4.40811E+01, 1.2078E+01, 2.8239E+03, &
      1.0372489E+00, 4.40781E+01, 1.2001E+01, 2.7920E+03, &
      1.0424351E+00, 4.40722E+01, 1.1924E+01, 2.7605E+03, &
      1.0476473E+00, 4.40636E+01, 1.1848E+01, 2.7293E+03, &
      1.0528855E+00, 4.40524E+01, 1.1773E+01, 2.6986E+03, &
      1.0581499E+00, 4.40388E+01, 1.1698E+01, 2.6683E+03, &
      1.0634407E+00, 4.40228E+01, 1.1624E+01, 2.6383E+03, &
      1.0687579E+00, 4.40046E+01, 1.1551E+01, 2.6087E+03, &
      1.0741017E+00, 4.39841E+01, 1.1478E+01, 2.5795E+03, &
      1.0794722E+00, 4.39634E+01, 1.1405E+01, 2.5506E+03, &
      1.0848695E+00, 4.39386E+01, 1.1334E+01, 2.5220E+03, &
      1.0902939E+00, 4.39117E+01, 1.1263E+01, 2.4939E+03, &
      1.0957454E+00, 4.38826E+01, 1.1192E+01, 2.4660E+03, &
      1.1012241E+00, 4.38515E+01, 1.1122E+01, 2.4386E+03, &
      1.1067302E+00, 4.38182E+01, 1.1053E+01, 2.4114E+03, &
      1.1122639E+00, 4.37828E+01, 1.0984E+01, 2.3846E+03, &
      1.1178252E+00, 4.37453E+01, 1.0916E+01, 2.3581E+03, &
      1.1234143E+00, 4.37057E+01, 1.0848E+01, 2.3320E+03, &
      1.1290314E+00, 4.36639E+01, 1.0781E+01, 2.3061E+03, &
      1.1346765E+00, 4.36200E+01, 1.0714E+01, 2.2806E+03, &
      1.1403499E+00, 4.35739E+01, 1.0648E+01, 2.2554E+03, &
      1.1460517E+00, 4.35255E+01, 1.0583E+01, 2.2305E+03, &
      1.1517819E+00, 4.34749E+01, 1.0518E+01, 2.2059E+03, &
      1.1575408E+00, 4.34221E+01, 1.0454E+01, 2.1816E+03, &
      1.1633285E+00, 4.33669E+01, 1.0390E+01, 2.1576E+03, &
      1.1691452E+00, 4.33094E+01, 1.0326E+01, 2.1339E+03, &
      1.1749909E+00, 4.32494E+01, 1.0263E+01, 2.1105E+03, &
      1.1808659E+00, 4.31870E+01, 1.0201E+01, 2.0874E+03, &
      1.1867702E+00, 4.31220E+01, 1.0139E+01, 2.0645E+03, &
      1.1927040E+00, 4.30545E+01, 1.0078E+01, 2.0419E+03, &
      1.1986676E+00, 4.29844E+01, 1.0017E+01, 2.0196E+03, &
      1.2046609E+00, 4.29115E+01, 9.9568E+00, 1.9976E+03, &
      1.2106842E+00, 4.28359E+01, 9.8969E+00, 1.9758E+03, &
      1.2167376E+00, 4.27573E+01, 9.8375E+00, 1.9543E+03, &
      1.2228213E+00, 4.26759E+01, 9.7786E+00, 1.9331E+03, &
      1.2289354E+00, 4.25914E+01, 9.7202E+00, 1.9121E+03, &
      1.2350801E+00, 4.25047E+01, 9.6622E+00, 1.8913E+03, &
      1.2412555E+00, 4.24138E+01, 9.6047E+00, 1.8708E+03, &
      1.2474618E+00, 4.23196E+01, 9.5477E+00, 1.8506E+03, &
      1.2536991E+00, 4.22218E+01, 9.4911E+00, 1.8306E+03, &
      1.2599676E+00, 4.21205E+01, 9.4349E+00, 1.8108E+03, &
      1.2662674E+00, 4.20154E+01, 9.3792E+00, 1.7913E+03, &
      1.2725988E+00, 4.19064E+01, 9.3240E+00, 1.7720E+03, &
      1.2789618E+00, 4.17933E+01, 9.2692E+00, 1.7529E+03, &
      1.2853566E+00, 4.16761E+01, 9.2148E+00, 1.7341E+03, &
      1.2917833E+00, 4.15544E+01, 9.1609E+00, 1.7155E+03, &
      1.2982423E+00, 4.14281E+01, 9.1073E+00, 1.6971E+03, &
      1.3047335E+00, 4.12970E+01, 9.0542E+00, 1.6789E+03, &
      1.3112571E+00, 4.11609E+01, 9.0015E+00, 1.6610E+03, &
      1.3178134E+00, 4.10194E+01, 8.9493E+00, 1.6432E+03, &
      1.3244025E+00, 4.08724E+01, 8.8974E+00, 1.6257E+03, &
      1.3310245E+00, 4.07195E+01, 8.8460E+00, 1.6084E+03, &
      1.3376796E+00, 4.05604E+01, 8.7949E+00, 1.5913E+03, &
      1.3443680E+00, 4.03948E+01, 8.7443E+00, 1.5743E+03, &
      1.3510899E+00, 4.02223E+01, 8.6940E+00, 1.5576E+03, &
      1.3578453E+00, 4.00424E+01, 8.6442E+00, 1.5411E+03, &
      1.3646345E+00, 3.98548E+01, 8.5947E+00, 1.5248E+03, &
      1.3714577E+00, 3.96588E+01, 8.5456E+00, 1.5086E+03, &
      1.3783150E+00, 3.94541E+01, 8.4969E+00, 1.4927E+03, &
      1.3852066E+00, 3.92398E+01, 8.4486E+00, 1.4769E+03, &
      1.3921326E+00, 3.90155E+01, 8.4006E+00, 1.4614E+03, &
      1.3990933E+00, 3.87802E+01, 8.3530E+00, 1.4460E+03, &
      1.4060887E+00, 3.85332E+01, 8.3058E+00, 1.4308E+03, &
      1.4131192E+00, 3.82736E+01, 8.2590E+00, 1.4157E+03, &
      1.4201848E+00, 3.80001E+01, 8.2125E+00, 1.4009E+03, &
      1.4272857E+00, 3.77117E+01, 8.1664E+00, 1.3862E+03, &
      1.4344221E+00, 3.74070E+01, 8.1206E+00, 1.3717E+03, &
      1.4415942E+00, 3.70843E+01, 8.0741E+00, 1.3572E+03, &
      1.4488022E+00, 3.67416E+01, 8.0274E+00, 1.3427E+03, &
      1.4560462E+00, 3.63770E+01, 7.9810E+00, 1.3284E+03, &
      1.4633265E+00, 3.59879E+01, 7.9350E+00, 1.3143E+03, &
      1.4706431E+00, 3.55713E+01, 7.8893E+00, 1.3003E+03, &
      1.4779963E+00, 3.51237E+01, 7.8440E+00, 1.2866E+03, &
      1.4853863E+00, 3.46410E+01, 7.7991E+00, 1.2729E+03, &
      1.4928132E+00, 3.41180E+01, 7.7545E+00, 1.2595E+03, &
      1.5002773E+00, 3.35481E+01, 7.7102E+00, 1.2462E+03, &
      1.5077787E+00, 3.29234E+01, 7.6663E+00, 1.2330E+03, &
      1.5153176E+00, 3.22334E+01, 7.6227E+00, 1.2200E+03, &
      1.5228942E+00, 3.14659E+01, 7.5794E+00, 1.2072E+03, &
      1.5305086E+00, 3.05989E+01, 7.5365E+00, 1.1945E+03, &
      1.5381612E+00, 2.96074E+01, 7.4939E+00, 1.1819E+03, &
      1.5458520E+00, 2.84518E+01, 7.4516E+00, 1.1695E+03, &
      1.5535812E+00, 2.70693E+01, 7.4096E+00, 1.1573E+03, &
      1.5613491E+00, 2.53499E+01, 7.3680E+00, 1.1452E+03, &
      1.5691559E+00, 2.30725E+01, 7.3267E+00, 1.1332E+03, &
      1.5770017E+00, 1.96632E+01, 7.2857E+00, 1.1213E+03, &
      1.5848867E+00, 1.23501E+01, 7.2450E+00, 1.1096E+03, &
      1.5882395E+00, -3.75676E+00, 7.2278E+00, 1.1047E+03, &
      1.5887605E+00, -4.09227E+00, 2.6332E+01, 3.9963E+03, &
      1.5928111E+00, 1.28184E+01, 2.6231E+01, 3.9709E+03, &
      1.6007752E+00, 1.87021E+01, 2.6034E+01, 3.9217E+03, &
      1.6087790E+00, 2.10642E+01, 2.5840E+01, 3.8731E+03, &
      1.6168229E+00, 2.21012E+01, 2.5646E+01, 3.8252E+03, &
      1.6249070E+00, 2.20761E+01, 2.5455E+01, 3.7778E+03, &
      1.6330316E+00, 2.02447E+01, 2.5265E+01, 3.7311E+03, &
      1.6388836E+00, 1.11683E+01, 2.5129E+01, 3.6979E+03, &
      1.6399164E+00, 1.11210E+01, 3.7610E+01, 5.5259E+03, &
      1.6411967E+00, 1.62781E+01, 3.7565E+01, 5.5151E+03, &
      1.6494027E+00, 2.42105E+01, 3.7278E+01, 5.4459E+03, &
      1.6576497E+00, 2.75530E+01, 3.6994E+01, 5.3777E+03, &
      1.6659380E+00, 2.98847E+01, 3.6712E+01, 5.3103E+03, &
      1.6742677E+00, 3.17276E+01, 3.6433E+01, 5.2437E+03, &
      1.6826390E+00, 3.32704E+01, 3.6155E+01, 5.1781E+03, &
      1.6910522E+00, 3.46057E+01, 3.5880E+01, 5.1132E+03, &
      1.6995075E+00, 3.57863E+01, 3.5607E+01, 5.0492E+03, &
      1.7080050E+00, 3.68459E+01, 3.5337E+01, 4.9860E+03, &
      1.7165450E+00, 3.78077E+01, 3.5068E+01, 4.9236E+03, &
      1.7251278E+00, 3.86880E+01, 3.4801E+01, 4.8620E+03, &
      1.7337534E+00, 3.94991E+01, 3.4537E+01, 4.8012E+03, &
      1.7424222E+00, 4.02505E+01, 3.4275E+01, 4.7412E+03, &
      1.7511343E+00, 4.09496E+01, 3.4015E+01, 4.6819E+03, &
      1.7598899E+00, 4.16024E+01, 3.3757E+01, 4.6234E+03, &
      1.7686894E+00, 4.22135E+01, 3.3500E+01, 4.5656E+03, &
      1.7775328E+00, 4.27872E+01, 3.3246E+01, 4.5086E+03, &
      1.7864205E+00, 4.33266E+01, 3.2994E+01, 4.4523E+03, &
      1.7953526E+00, 4.38346E+01, 3.2744E+01, 4.3967E+03, &
      1.8043294E+00, 4.43134E+01, 3.2496E+01, 4.3418E+03, &
      1.8133510E+00, 4.47651E+01, 3.2250E+01, 4.2876E+03, &
      1.8224178E+00, 4.51914E+01, 3.2006E+01, 4.2341E+03, &
      1.8315299E+00, 4.55935E+01, 3.1764E+01, 4.1813E+03, &
      1.8406875E+00, 4.59728E+01, 3.1524E+01, 4.1292E+03, &
      1.8498909E+00, 4.63300E+01, 3.1286E+01, 4.0777E+03, &
      1.8591404E+00, 4.66660E+01, 3.1049E+01, 4.0269E+03, &
      1.8684361E+00, 4.69813E+01, 3.0815E+01, 3.9767E+03, &
      1.8777783E+00, 4.72760E+01, 3.0582E+01, 3.9272E+03, &
      1.8871672E+00, 4.75504E+01, 3.0351E+01, 3.8783E+03, &
      1.8966030E+00, 4.78043E+01, 3.0122E+01, 3.8300E+03, &
      1.9060860E+00, 4.80371E+01, 2.9895E+01, 3.7823E+03, &
      1.9156165E+00, 4.82481E+01, 2.9670E+01, 3.7352E+03, &
      1.9251945E+00, 4.84357E+01, 2.9446E+01, 3.6888E+03, &
      1.9348205E+00, 4.85981E+01, 2.9224E+01, 3.6429E+03, &
      1.9444946E+00, 4.87323E+01, 2.9004E+01, 3.5976E+03, &
      1.9542171E+00, 4.88337E+01, 2.8786E+01, 3.5529E+03, &
      1.9639882E+00, 4.88955E+01, 2.8569E+01, 3.5087E+03, &
      1.9738081E+00, 4.89070E+01, 2.8354E+01, 3.4651E+03, &
      1.9836772E+00, 4.88495E+01, 2.8141E+01, 3.4221E+03, &
      1.9935955E+00, 4.86876E+01, 2.7929E+01, 3.3796E+03, &
      2.0035635E+00, 4.83410E+01, 2.7720E+01, 3.3376E+03, &
      2.0135813E+00, 4.75503E+01, 2.7511E+01, 3.2962E+03, &
      2.0213134E+00, 4.54969E+01, 2.7352E+01, 3.2647E+03, &
      2.0236492E+00, 3.95706E+01, 3.2141E+01, 3.8300E+03, &
      2.0258868E+00, 4.55521E+01, 3.2081E+01, 3.8187E+03, &
      2.0337675E+00, 4.80629E+01, 3.1873E+01, 3.7794E+03, &
      2.0439363E+00, 4.93775E+01, 3.1608E+01, 3.7294E+03, &
      2.0541560E+00, 5.02322E+01, 3.1345E+01, 3.6801E+03, &
      2.0644268E+00, 5.08904E+01, 3.1084E+01, 3.6315E+03, &
      2.0747489E+00, 5.14348E+01, 3.0823E+01, 3.5832E+03, &
      2.0851227E+00, 5.19021E+01, 3.0564E+01, 3.5356E+03, &
      2.0955483E+00, 5.23120E+01, 3.0308E+01, 3.4886E+03, &
      2.1060260E+00, 5.26757E+01, 3.0054E+01, 3.4422E+03, &
      2.1165562E+00, 5.29993E+01, 2.9801E+01, 3.3965E+03, &
      2.1271389E+00, 5.32853E+01, 2.9561E+01, 3.3525E+03, &
      2.1377746E+00, 5.35488E+01, 2.9338E+01, 3.3108E+03, &
      2.1484635E+00, 5.37932E+01, 2.9118E+01, 3.2697E+03, &
      2.1592058E+00, 5.40184E+01, 2.8901E+01, 3.2293E+03, &
      2.1700018E+00, 5.42241E+01, 2.8687E+01, 3.1895E+03, &
      2.1808519E+00, 5.44098E+01, 2.8475E+01, 3.1504E+03, &
      2.1917561E+00, 5.45741E+01, 2.8267E+01, 3.1119E+03, &
      2.2027149E+00, 5.47142E+01, 2.8062E+01, 3.0740E+03, &
      2.2137285E+00, 5.48252E+01, 2.7859E+01, 3.0368E+03, &
      2.2247971E+00, 5.48972E+01, 2.7659E+01, 3.0000E+03, &
      2.2359211E+00, 5.49095E+01, 2.7461E+01, 2.9639E+03, &
      2.2471007E+00, 5.48064E+01, 2.7265E+01, 2.9283E+03, &
      2.2583362E+00, 5.43130E+01, 2.7072E+01, 2.8932E+03, &
      2.2608064E+00, 5.39624E+01, 2.7030E+01, 2.8856E+03, &
      2.2661935E+00, 5.40354E+01, 2.8855E+01, 3.0724E+03, &
      2.2696279E+00, 5.46022E+01, 2.8789E+01, 3.0608E+03, &
      2.2809760E+00, 5.54404E+01, 2.8574E+01, 3.0229E+03, &
      2.2923809E+00, 5.59313E+01, 2.8361E+01, 2.9856E+03, &
      2.3038428E+00, 5.63087E+01, 2.8151E+01, 2.9488E+03, &
      2.3153620E+00, 5.66256E+01, 2.7943E+01, 2.9126E+03, &
      2.3269388E+00, 5.69029E+01, 2.7737E+01, 2.8768E+03, &
      2.3385735E+00, 5.71509E+01, 2.7533E+01, 2.8416E+03, &
      2.3502664E+00, 5.73751E+01, 2.7332E+01, 2.8069E+03, &
      2.3620177E+00, 5.75785E+01, 2.7133E+01, 2.7727E+03, &
      2.3738278E+00, 5.77621E+01, 2.6935E+01, 2.7389E+03, &
      2.3856970E+00, 5.79287E+01, 2.6752E+01, 2.7069E+03, &
      2.3976254E+00, 5.80841E+01, 2.6572E+01, 2.6754E+03, &
      2.4096136E+00, 5.82269E+01, 2.6395E+01, 2.6444E+03, &
      2.4216616E+00, 5.83553E+01, 2.6220E+01, 2.6139E+03, &
      2.4337699E+00, 5.84665E+01, 2.6047E+01, 2.5839E+03, &
      2.4459388E+00, 5.85554E+01, 2.5876E+01, 2.5543E+03, &
      2.4581685E+00, 5.86112E+01, 2.5707E+01, 2.5251E+03, &
      2.4704593E+00, 5.86071E+01, 2.5541E+01, 2.4964E+03, &
      2.4828116E+00, 5.84361E+01, 2.5376E+01, 2.4680E+03, &
      2.4861428E+00, 5.82920E+01, 2.5332E+01, 2.4604E+03, &
      2.4952257E+00, 5.83177E+01, 2.6380E+01, 2.5526E+03, &
      2.4962571E+00, 5.84160E+01, 2.6366E+01, 2.5502E+03, &
      2.5077018E+00, 5.90018E+01, 2.6214E+01, 2.5239E+03, &
      2.5202403E+00, 5.93665E+01, 2.6049E+01, 2.4957E+03, &
      2.5328415E+00, 5.96510E+01, 2.5885E+01, 2.4678E+03, &
      2.5455057E+00, 5.98968E+01, 2.5724E+01, 2.4403E+03, &
      2.5582333E+00, 6.01188E+01, 2.5563E+01, 2.4131E+03, &
      2.5710244E+00, 6.03246E+01, 2.5404E+01, 2.3862E+03, &
      2.5838796E+00, 6.05180E+01, 2.5246E+01, 2.3597E+03, &
      2.5967990E+00, 6.07018E+01, 2.5089E+01, 2.3335E+03, &
      2.6097829E+00, 6.08776E+01, 2.4934E+01, 2.3076E+03, &
      2.6228319E+00, 6.10466E+01, 2.4781E+01, 2.2821E+03, &
      2.6359460E+00, 6.12107E+01, 2.4627E+01, 2.2568E+03, &
      2.6491257E+00, 6.13691E+01, 2.4472E+01, 2.2315E+03, &
      2.6623714E+00, 6.15223E+01, 2.4318E+01, 2.2065E+03, &
      2.6756832E+00, 6.16710E+01, 2.4166E+01, 2.1819E+03, &
      2.6890617E+00, 6.18155E+01, 2.4014E+01, 2.1575E+03, &
      2.7025070E+00, 6.19562E+01, 2.3863E+01, 2.1334E+03, &
      2.7160195E+00, 6.20933E+01, 2.3714E+01, 2.1096E+03, &
      2.7295996E+00, 6.22273E+01, 2.3565E+01, 2.0860E+03, &
      2.7432476E+00, 6.23581E+01, 2.3418E+01, 2.0627E+03, &
      2.7569638E+00, 6.24862E+01, 2.3271E+01, 2.0397E+03, &
      2.7707486E+00, 6.26115E+01, 2.3125E+01, 2.0170E+03, &
      2.7846024E+00, 6.27344E+01, 2.2981E+01, 1.9945E+03, &
      2.7985254E+00, 6.28549E+01, 2.2837E+01, 1.9722E+03, &
      2.8125180E+00, 6.29732E+01, 2.2694E+01, 1.9502E+03, &
      2.8265806E+00, 6.30894E+01, 2.2552E+01, 1.9285E+03, &
      2.8407135E+00, 6.32036E+01, 2.2410E+01, 1.9069E+03, &
      2.8549171E+00, 6.33160E+01, 2.2270E+01, 1.8856E+03, &
      2.8691917E+00, 6.34266E+01, 2.2130E+01, 1.8646E+03, &
      2.8835376E+00, 6.35356E+01, 2.1991E+01, 1.8437E+03, &
      2.8979553E+00, 6.36432E+01, 2.1853E+01, 1.8231E+03, &
      2.9124451E+00, 6.37495E+01, 2.1716E+01, 1.8028E+03, &
      2.9270073E+00, 6.38548E+01, 2.1579E+01, 1.7826E+03, &
      2.9416424E+00, 6.39593E+01, 2.1443E+01, 1.7626E+03, &
      2.9563506E+00, 6.40632E+01, 2.1307E+01, 1.7428E+03, &
      2.9711323E+00, 6.41671E+01, 2.1171E+01, 1.7232E+03, &
      2.9859880E+00, 6.42727E+01, 2.1037E+01, 1.7038E+03, &
      3.0009179E+00, 6.43872E+01, 2.0902E+01, 1.6846E+03, &
      3.0159225E+00, 6.44921E+01, 2.0751E+01, 1.6642E+03, &
      3.0310021E+00, 6.45929E+01, 2.0602E+01, 1.6441E+03, &
      3.0461571E+00, 6.46902E+01, 2.0453E+01, 1.6242E+03, &
      3.0613879E+00, 6.47842E+01, 2.0306E+01, 1.6046E+03, &
      3.0766949E+00, 6.48752E+01, 2.0159E+01, 1.5852E+03, &
      3.0920783E+00, 6.49635E+01, 2.0014E+01, 1.5661E+03, &
      3.1075387E+00, 6.50491E+01, 1.9870E+01, 1.5471E+03, &
      3.1230764E+00, 6.51324E+01, 1.9727E+01, 1.5285E+03, &
      3.1386918E+00, 6.52134E+01, 1.9585E+01, 1.5100E+03, &
      3.1543853E+00, 6.52924E+01, 1.9444E+01, 1.4918E+03, &
      3.1701572E+00, 6.53695E+01, 1.9304E+01, 1.4738E+03, &
      3.1860080E+00, 6.56549E+01, 1.9161E+01, 1.4557E+03, &
      3.2019380E+00, 6.57286E+01, 1.9016E+01, 1.4376E+03, &
      3.2179477E+00, 6.57998E+01, 1.8873E+01, 1.4197E+03, &
      3.2340374E+00, 6.58688E+01, 1.8730E+01, 1.4021E+03, &
      3.2502076E+00, 6.59355E+01, 1.8589E+01, 1.3847E+03, &
      3.2664587E+00, 6.60001E+01, 1.8448E+01, 1.3675E+03, &
      3.2827910E+00, 6.62105E+01, 1.8308E+01, 1.3504E+03, &
      3.2992049E+00, 6.62710E+01, 1.8165E+01, 1.3333E+03, &
      3.3157009E+00, 6.63292E+01, 1.8023E+01, 1.3164E+03, &
      3.3322794E+00, 6.63852E+01, 1.7883E+01, 1.2998E+03, &
      3.3489408E+00, 6.64391E+01, 1.7744E+01, 1.2833E+03, &
      3.3656856E+00, 6.64910E+01, 1.7606E+01, 1.2671E+03, &
      3.3825140E+00, 6.65410E+01, 1.7470E+01, 1.2512E+03, &
      3.3994265E+00, 6.65892E+01, 1.7335E+01, 1.2354E+03, &
      3.4164237E+00, 6.66357E+01, 1.7201E+01, 1.2199E+03, &
      3.4335058E+00, 6.66806E+01, 1.7068E+01, 1.2045E+03, &
      3.4506733E+00, 6.67239E+01, 1.6936E+01, 1.1894E+03, &
      3.4679267E+00, 6.67658E+01, 1.6806E+01, 1.1745E+03, &
      3.4852663E+00, 6.68062E+01, 1.6677E+01, 1.1597E+03, &
      3.5026927E+00, 6.68452E+01, 1.6549E+01, 1.1452E+03, &
      3.5202061E+00, 6.68829E+01, 1.6422E+01, 1.1309E+03, &
      3.5378072E+00, 6.69193E+01, 1.6296E+01, 1.1167E+03, &
      3.5554962E+00, 6.69545E+01, 1.6172E+01, 1.1028E+03, &
      3.5732737E+00, 6.69885E+01, 1.6049E+01, 1.0890E+03, &
      3.5911400E+00, 6.70214E+01, 1.5926E+01, 1.0754E+03, &
      3.6090957E+00, 6.70532E+01, 1.5805E+01, 1.0620E+03, &
      3.6271412E+00, 6.70839E+01, 1.5685E+01, 1.0488E+03, &
      3.6452769E+00, 6.71136E+01, 1.5566E+01, 1.0358E+03, &
      3.6635033E+00, 6.71423E+01, 1.5448E+01, 1.0229E+03, &
      3.6818208E+00, 6.71700E+01, 1.5330E+01, 1.0101E+03, &
      3.7002299E+00, 6.71968E+01, 1.5213E+01, 9.9750E+02, &
      3.7187311E+00, 6.72226E+01, 1.5097E+01, 9.8506E+02, &
      3.7373247E+00, 6.72475E+01, 1.4982E+01, 9.7279E+02, &
      3.7560114E+00, 6.72716E+01, 1.4868E+01, 9.6068E+02, &
      3.7747914E+00, 6.72948E+01, 1.4755E+01, 9.4874E+02, &
      3.7936654E+00, 6.73171E+01, 1.4644E+01, 9.3695E+02, &
      3.8126337E+00, 6.73387E+01, 1.4533E+01, 9.2533E+02, &
      3.8316969E+00, 6.73595E+01, 1.4423E+01, 9.1386E+02, &
      3.8508554E+00, 6.73796E+01, 1.4314E+01, 9.0253E+02, &
      3.8701096E+00, 6.73989E+01, 1.4206E+01, 8.9131E+02, &
      3.8894602E+00, 6.74176E+01, 1.4098E+01, 8.8024E+02, &
      3.9089075E+00, 6.74355E+01, 1.3991E+01, 8.6933E+02, &
      3.9284520E+00, 6.74528E+01, 1.3886E+01, 8.5855E+02, &
      3.9480943E+00, 6.74695E+01, 1.3781E+01, 8.4793E+02, &
      3.9678347E+00, 6.74855E+01, 1.3677E+01, 8.3744E+02, &
      3.9876739E+00, 6.75010E+01, 1.3574E+01, 8.2710E+02, &
      4.055024E+00, 6.80705E+01, 1.3234E+01, 7.9328E+02, &
      4.334821E+00, 6.81253E+01, 1.1926E+01, 6.6975E+02, &
      4.633924E+00, 6.81421E+01, 1.0755E+01, 5.6597E+02, &
      4.953664E+00, 6.80331E+01, 9.6944E+00, 4.7816E+02, &
      5.295467E+00, 6.78818E+01, 8.7414E+00, 4.0420E+02, &
      5.660855E+00, 6.76534E+01, 7.8920E+00, 3.4218E+02, &
      6.051453E+00, 6.73736E+01, 7.1349E+00, 2.9013E+02, &
      6.469004E+00, 6.70389E+01, 6.4591E+00, 2.4640E+02, &
      6.915365E+00, 6.66357E+01, 5.8517E+00, 2.0947E+02, &
      7.392525E+00, 6.61193E+01, 5.2816E+00, 1.7748E+02, &
      7.902609E+00, 6.54029E+01, 4.7703E+00, 1.5053E+02, &
      8.447890E+00, 6.42755E+01, 4.3147E+00, 1.2790E+02, &
      9.030794E+00, 6.15220E+01, 3.8874E+00, 1.0830E+02, &
      9.059218E+00, 6.12237E+01, 3.8656E+00, 1.0737E+02, &
      9.197879E+00, 5.83103E+01, 3.7620E+00, 1.0306E+02, &
      9.234856E+00, 5.48774E+01, 3.7351E+00, 1.0195E+02, &
      9.290320E+00, 5.83353E+01, 1.0408E+01, 2.7409E+02, &
      9.428982E+00, 6.13582E+01, 1.0061E+01, 2.6120E+02, &
      9.653919E+00, 6.28874E+01, 9.5328E+00, 2.4198E+02, &
      1.014163E+01, 6.32255E+01, 8.6510E+00, 2.0943E+02, &
      1.029686E+01, 6.20344E+01, 8.4114E+00, 2.0068E+02, &
      1.032004E+01, 6.14352E+01, 8.3769E+00, 1.9943E+02, &
      1.033825E+01, 6.03722E+01, 8.3500E+00, 1.9845E+02, &
      1.040034E+01, 6.21758E+01, 1.1623E+01, 2.7296E+02, &
      1.055557E+01, 6.38087E+01, 1.1291E+01, 2.6137E+02, &
      1.065299E+01, 6.42064E+01, 1.1091E+01, 2.5448E+02, &
      1.081605E+01, 6.41778E+01, 1.0771E+01, 2.4353E+02, &
      1.085953E+01, 6.34651E+01, 1.0689E+01, 2.4073E+02, &
      1.092475E+01, 6.45250E+01, 1.2266E+01, 2.7402E+02, &
      1.103212E+01, 6.54276E+01, 1.2075E+01, 2.6720E+02, &
      1.108781E+01, 6.57342E+01, 1.1979E+01, 2.6377E+02, &
      1.179334E+01, 6.78323E+01, 1.0890E+01, 2.2578E+02, &
      1.260708E+01, 6.90599E+01, 9.8194E+00, 1.9077E+02, &
      1.347697E+01, 6.98582E+01, 8.8462E+00, 1.6108E+02, &
      1.440688E+01, 7.04053E+01, 7.9399E+00, 1.3554E+02, &
      1.540095E+01, 7.07705E+01, 7.1159E+00, 1.1390E+02, &
      1.646362E+01, 7.10101E+01, 6.3749E+00, 9.5703E+01, &
      1.759961E+01, 7.11611E+01, 5.7119E+00, 8.0442E+01, &
      1.881398E+01, 7.13132E+01, 5.1183E+00, 6.7637E+01, &
      2.011215E+01, 7.13517E+01, 4.5831E+00, 5.6844E+01, &
      2.149988E+01, 7.13908E+01, 4.1049E+00, 4.7799E+01, &
      2.298338E+01, 7.13990E+01, 3.6721E+00, 4.0158E+01, &
      2.456923E+01, 7.13572E+01, 3.2866E+00, 3.3766E+01, &
      2.626450E+01, 7.13068E+01, 2.9442E+00, 2.8425E+01, &
      2.807676E+01, 7.12484E+01, 2.6182E+00, 2.3772E+01, &
      3.001405E+01, 7.11650E+01, 2.3240E+00, 1.9856E+01, &
      3.208502E+01, 7.10701E+01, 2.0646E+00, 1.6608E+01, &
      3.429889E+01, 7.09678E+01, 1.8357E+00, 1.3909E+01, &
      3.666551E+01, 7.08598E+01, 1.6333E+00, 1.1664E+01, &
      3.919543E+01, 7.07462E+01, 1.4545E+00, 9.7946E+00, &
      4.189992E+01, 7.06252E+01, 1.2942E+00, 8.2247E+00, &
      4.479101E+01, 7.04918E+01, 1.1513E+00, 6.9108E+00, &
      4.788159E+01, 7.03391E+01, 1.0224E+00, 5.8030E+00, &
      5.118542E+01, 7.01506E+01, 9.0531E-01, 4.8653E+00, &
      5.471721E+01, 6.98941E+01, 8.0196E-01, 4.0855E+00, &
      5.849270E+01, 6.94707E+01, 7.1065E-01, 3.4360E+00, &
      6.204752E+01, 6.84517E+01, 6.3901E-01, 2.9535E+00, &
      6.252870E+01, 6.80708E+01, 6.3021E-01, 2.8959E+00, &
      6.299723E+01, 6.73303E+01, 6.2183E-01, 2.8413E+00, &
      6.325049E+01, 6.59876E+01, 6.1737E-01, 2.8126E+00, &
      6.363037E+01, 6.73441E+01, 3.2538E+00, 1.2760E+01, &
      6.458008E+01, 6.85694E+01, 3.1814E+00, 1.2301E+01, &
      6.684318E+01, 6.95080E+01, 3.0182E+00, 1.1294E+01, &
      7.145536E+01, 7.02671E+01, 2.7105E+00, 9.5220E+00, &
      7.638578E+01, 7.06580E+01, 2.4307E+00, 8.0208E+00, &
      8.165640E+01, 7.09032E+01, 2.1793E+00, 6.7582E+00, &
      8.729069E+01, 7.10647E+01, 1.9462E+00, 5.6760E+00, &
      9.331374E+01, 7.11701E+01, 1.7376E+00, 4.7690E+00, &
      9.975239E+01, 7.12395E+01, 1.5515E+00, 4.0104E+00, &
      1.066353E+02, 7.12841E+01, 1.3855E+00, 3.3758E+00, &
      1.139931E+02, 7.13112E+01, 1.2375E+00, 2.8447E+00, &
      1.218587E+02, 7.13261E+01, 1.1054E+00, 2.4001E+00, &
      1.302669E+02, 7.13602E+01, 9.8577E-01, 2.0246E+00, &
      1.392553E+02, 7.13570E+01, 8.7710E-01, 1.7068E+00, &
      1.488640E+02, 7.13479E+01, 7.8049E-01, 1.4417E+00, &
      1.591356E+02, 7.13349E+01, 6.9458E-01, 1.2200E+00, &
      1.701159E+02, 7.13193E+01, 6.1819E-01, 1.0349E+00, &
      1.818539E+02, 7.13022E+01, 5.5025E-01, 8.8012E-01, &
      1.944018E+02, 7.12843E+01, 4.8983E-01, 7.5058E-01, &
      2.078156E+02, 7.12660E+01, 4.3607E-01, 6.4210E-01, &
      2.221548E+02, 7.12477E+01, 3.8825E-01, 5.5119E-01, &
      2.374835E+02, 7.12297E+01, 3.4570E-01, 4.7490E-01, &
      2.538699E+02, 7.12122E+01, 3.0783E-01, 4.1083E-01, &
      2.713869E+02, 7.11950E+01, 2.7414E-01, 3.5693E-01, &
      2.901126E+02, 7.11787E+01, 2.4552E-01, 3.1268E-01, &
      3.101304E+02, 7.11639E+01, 2.2016E-01, 2.7532E-01, &
      3.315294E+02, 7.11504E+01, 1.9742E-01, 2.4354E-01, &
      3.544049E+02, 7.11379E+01, 1.7705E-01, 2.1643E-01, &
      3.788588E+02, 7.11263E+01, 1.5879E-01, 1.9326E-01, &
      4.050001E+02, 7.11157E+01, 1.4242E-01, 1.7341E-01, &
      4.329451E+02, 7.11058E+01, 1.2775E-01, 1.5634E-01]
   case(72)
      frel = -8.0280E-01
      allocate(nist_tab(3640))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.900000E-03, 8.07421E+00, 0.0000E+00, 5.1658E-06, &
      4.975000E-03, 7.62222E+00, 0.0000E+00, 5.6145E-06, &
      4.995000E-03, 7.09825E+00, 0.0000E+00, 5.7391E-06, &
      5.025000E-03, 7.62497E+00, 1.0218E+00, 4.7938E+04, &
      5.100000E-03, 8.07584E+00, 1.0141E+00, 4.6879E+04, &
      1.069000E-02, 8.92457E+00, 1.9327E+00, 4.2624E+04, &
      1.142761E-02, 9.03059E+00, 2.0717E+00, 4.2740E+04, &
      1.221612E-02, 9.15381E+00, 2.1926E+00, 4.2314E+04, &
      1.305903E-02, 9.28691E+00, 2.2883E+00, 4.1310E+04, &
      1.396010E-02, 9.41862E+00, 2.3530E+00, 3.9738E+04, &
      1.492335E-02, 9.53125E+00, 2.3829E+00, 3.7645E+04, &
      1.595306E-02, 9.60064E+00, 2.3763E+00, 3.5117E+04, &
      1.675800E-02, 9.50328E+00, 2.3483E+00, 3.3036E+04, &
      1.701450E-02, 9.28984E+00, 2.3358E+00, 3.2365E+04, &
      1.705382E-02, 9.18348E+00, 2.3338E+00, 3.2263E+04, &
      1.708290E-02, 9.00587E+00, 2.3322E+00, 3.2186E+04, &
      1.718550E-02, 9.32500E+00, 2.9002E+00, 3.9787E+04, &
      1.744200E-02, 9.61790E+00, 2.8639E+00, 3.8711E+04, &
      1.823053E-02, 9.90396E+00, 2.7511E+00, 3.5577E+04, &
      1.948844E-02, 1.00524E+01, 2.5735E+00, 3.1132E+04, &
      2.083314E-02, 1.00556E+01, 2.4055E+00, 2.7222E+04, &
      2.227063E-02, 9.94564E+00, 2.2437E+00, 2.3752E+04, &
      2.380730E-02, 9.70585E+00, 2.0894E+00, 2.0691E+04, &
      2.545001E-02, 9.28520E+00, 1.9469E+00, 1.8035E+04, &
      2.720606E-02, 8.54989E+00, 1.8213E+00, 1.5782E+04, &
      2.908327E-02, 6.97692E+00, 1.7179E+00, 1.3926E+04, &
      2.998800E-02, 5.07695E+00, 1.6794E+00, 1.3203E+04, &
      3.044700E-02, 2.01609E+00, 1.6625E+00, 1.2873E+04, &
      3.056940E-02, -1.66293E+00, 1.6584E+00, 1.2790E+04, &
      3.075300E-02, 2.20774E+00, 8.8980E+00, 6.8213E+04, &
      3.109002E-02, 5.03434E+00, 8.7271E+00, 6.6178E+04, &
      3.121200E-02, 5.58090E+00, 8.6669E+00, 6.5464E+04, &
      3.323523E-02, 9.04873E+00, 7.7757E+00, 5.5158E+04, &
      3.552846E-02, 1.00316E+01, 6.9188E+00, 4.5911E+04, &
      3.733800E-02, 9.46532E+00, 6.2957E+00, 3.9752E+04, &
      3.790950E-02, 8.11001E+00, 6.1146E+00, 3.8026E+04, &
      3.797993E-02, 7.61463E+00, 6.0929E+00, 3.7821E+04, &
      3.806190E-02, 6.34795E+00, 6.0677E+00, 3.7584E+04, &
      3.829050E-02, 8.26411E+00, 9.5352E+00, 5.8709E+04, &
      3.886200E-02, 1.00071E+01, 9.3044E+00, 5.6445E+04, &
      4.060054E-02, 1.16878E+01, 8.6704E+00, 5.0347E+04, &
      4.340198E-02, 1.27823E+01, 7.8368E+00, 4.2569E+04, &
      4.639671E-02, 1.32949E+01, 7.0954E+00, 3.6054E+04, &
      4.959809E-02, 1.34529E+01, 6.4926E+00, 3.0862E+04, &
      5.302035E-02, 1.33571E+01, 6.0680E+00, 2.6982E+04, &
      5.667876E-02, 1.31397E+01, 5.8379E+00, 2.4283E+04, &
      6.058959E-02, 1.28444E+01, 5.8027E+00, 2.2579E+04, &
      6.360200E-02, 1.26033E+01, 5.8945E+00, 2.1849E+04, &
      6.457550E-02, 1.25128E+01, 5.9426E+00, 2.1696E+04, &
      6.477028E-02, 1.24850E+01, 5.9533E+00, 2.1669E+04, &
      6.483510E-02, 1.24699E+01, 5.9569E+00, 2.1661E+04, &
      6.522450E-02, 1.24674E+01, 6.0278E+00, 2.1788E+04, &
      6.619800E-02, 1.24235E+01, 6.0902E+00, 2.1690E+04, &
      6.923942E-02, 1.22607E+01, 6.3289E+00, 2.1550E+04, &
      7.401695E-02, 1.20617E+01, 6.7968E+00, 2.1649E+04, &
      7.912411E-02, 1.19422E+01, 7.3742E+00, 2.1972E+04, &
      8.458368E-02, 1.19154E+01, 8.0377E+00, 2.2403E+04, &
      9.041995E-02, 1.19887E+01, 8.7649E+00, 2.2853E+04, &
      9.665893E-02, 1.21710E+01, 9.5362E+00, 2.3259E+04, &
      1.0000000E-01, 1.40652E+01, 9.9402E+00, 2.3435E+04, &
      1.0050000E-01, 1.40864E+01, 9.9998E+00, 2.3459E+04, &
      1.0100250E-01, 1.41083E+01, 1.0060E+01, 2.3481E+04, &
      1.0150751E-01, 1.41307E+01, 1.0119E+01, 2.3503E+04, &
      1.0201505E-01, 1.41537E+01, 1.0179E+01, 2.3525E+04, &
      1.0252513E-01, 1.41773E+01, 1.0239E+01, 2.3545E+04, &
      1.0303775E-01, 1.42014E+01, 1.0299E+01, 2.3565E+04, &
      1.0355294E-01, 1.42260E+01, 1.0359E+01, 2.3585E+04, &
      1.0407070E-01, 1.42512E+01, 1.0419E+01, 2.3603E+04, &
      1.0459106E-01, 1.42770E+01, 1.0479E+01, 2.3621E+04, &
      1.0511401E-01, 1.43033E+01, 1.0539E+01, 2.3639E+04, &
      1.0563958E-01, 1.43301E+01, 1.0599E+01, 2.3655E+04, &
      1.0616778E-01, 1.43575E+01, 1.0659E+01, 2.3671E+04, &
      1.0669862E-01, 1.43854E+01, 1.0719E+01, 2.3686E+04, &
      1.0723211E-01, 1.44139E+01, 1.0779E+01, 2.3700E+04, &
      1.0776827E-01, 1.44429E+01, 1.0840E+01, 2.3713E+04, &
      1.0830712E-01, 1.44724E+01, 1.0900E+01, 2.3726E+04, &
      1.0884865E-01, 1.45025E+01, 1.0960E+01, 2.3738E+04, &
      1.0939289E-01, 1.45330E+01, 1.1020E+01, 2.3749E+04, &
      1.0993986E-01, 1.45641E+01, 1.1080E+01, 2.3760E+04, &
      1.1048956E-01, 1.45958E+01, 1.1139E+01, 2.3770E+04, &
      1.1104201E-01, 1.46279E+01, 1.1199E+01, 2.3778E+04, &
      1.1159722E-01, 1.46605E+01, 1.1259E+01, 2.3786E+04, &
      1.1215520E-01, 1.46936E+01, 1.1319E+01, 2.3794E+04, &
      1.1271598E-01, 1.47273E+01, 1.1379E+01, 2.3800E+04, &
      1.1327956E-01, 1.47614E+01, 1.1438E+01, 2.3806E+04, &
      1.1384596E-01, 1.47960E+01, 1.1498E+01, 2.3811E+04, &
      1.1441519E-01, 1.48312E+01, 1.1557E+01, 2.3815E+04, &
      1.1498726E-01, 1.48668E+01, 1.1617E+01, 2.3818E+04, &
      1.1556220E-01, 1.49029E+01, 1.1676E+01, 2.3821E+04, &
      1.1614001E-01, 1.49394E+01, 1.1735E+01, 2.3823E+04, &
      1.1672071E-01, 1.49765E+01, 1.1794E+01, 2.3823E+04, &
      1.1730431E-01, 1.50140E+01, 1.1853E+01, 2.3824E+04, &
      1.1789083E-01, 1.50520E+01, 1.1912E+01, 2.3823E+04, &
      1.1848029E-01, 1.50904E+01, 1.1971E+01, 2.3821E+04, &
      1.1907269E-01, 1.51293E+01, 1.2030E+01, 2.3819E+04, &
      1.1966805E-01, 1.51687E+01, 1.2088E+01, 2.3816E+04, &
      1.2026639E-01, 1.52085E+01, 1.2147E+01, 2.3812E+04, &
      1.2086772E-01, 1.52488E+01, 1.2205E+01, 2.3807E+04, &
      1.2147206E-01, 1.52895E+01, 1.2263E+01, 2.3802E+04, &
      1.2207942E-01, 1.53306E+01, 1.2321E+01, 2.3795E+04, &
      1.2268982E-01, 1.53722E+01, 1.2379E+01, 2.3788E+04, &
      1.2330327E-01, 1.54142E+01, 1.2437E+01, 2.3780E+04, &
      1.2391979E-01, 1.54566E+01, 1.2495E+01, 2.3772E+04, &
      1.2453939E-01, 1.54994E+01, 1.2552E+01, 2.3762E+04, &
      1.2516208E-01, 1.55427E+01, 1.2609E+01, 2.3752E+04, &
      1.2578789E-01, 1.55863E+01, 1.2667E+01, 2.3741E+04, &
      1.2641683E-01, 1.56304E+01, 1.2724E+01, 2.3729E+04, &
      1.2704892E-01, 1.56748E+01, 1.2780E+01, 2.3717E+04, &
      1.2768416E-01, 1.57197E+01, 1.2837E+01, 2.3703E+04, &
      1.2832258E-01, 1.57649E+01, 1.2894E+01, 2.3689E+04, &
      1.2896419E-01, 1.58106E+01, 1.2950E+01, 2.3674E+04, &
      1.2960902E-01, 1.58566E+01, 1.3006E+01, 2.3659E+04, &
      1.3025706E-01, 1.59030E+01, 1.3062E+01, 2.3642E+04, &
      1.3090835E-01, 1.59497E+01, 1.3118E+01, 2.3625E+04, &
      1.3156289E-01, 1.59968E+01, 1.3173E+01, 2.3607E+04, &
      1.3222070E-01, 1.60443E+01, 1.3229E+01, 2.3588E+04, &
      1.3288181E-01, 1.60921E+01, 1.3284E+01, 2.3569E+04, &
      1.3354621E-01, 1.61403E+01, 1.3339E+01, 2.3549E+04, &
      1.3421395E-01, 1.61888E+01, 1.3394E+01, 2.3528E+04, &
      1.3488502E-01, 1.62377E+01, 1.3448E+01, 2.3506E+04, &
      1.3555944E-01, 1.62869E+01, 1.3503E+01, 2.3484E+04, &
      1.3623724E-01, 1.63364E+01, 1.3557E+01, 2.3461E+04, &
      1.3691842E-01, 1.63862E+01, 1.3611E+01, 2.3437E+04, &
      1.3760302E-01, 1.64364E+01, 1.3665E+01, 2.3413E+04, &
      1.3829103E-01, 1.64868E+01, 1.3718E+01, 2.3388E+04, &
      1.3898249E-01, 1.65376E+01, 1.3772E+01, 2.3362E+04, &
      1.3967740E-01, 1.65886E+01, 1.3825E+01, 2.3335E+04, &
      1.4037579E-01, 1.66400E+01, 1.3878E+01, 2.3308E+04, &
      1.4107766E-01, 1.66916E+01, 1.3930E+01, 2.3280E+04, &
      1.4178305E-01, 1.67435E+01, 1.3983E+01, 2.3252E+04, &
      1.4249197E-01, 1.67957E+01, 1.4035E+01, 2.3223E+04, &
      1.4320443E-01, 1.68482E+01, 1.4087E+01, 2.3193E+04, &
      1.4392045E-01, 1.69009E+01, 1.4139E+01, 2.3162E+04, &
      1.4464005E-01, 1.69538E+01, 1.4191E+01, 2.3131E+04, &
      1.4536325E-01, 1.70070E+01, 1.4242E+01, 2.3099E+04, &
      1.4609007E-01, 1.70605E+01, 1.4293E+01, 2.3067E+04, &
      1.4682052E-01, 1.71142E+01, 1.4344E+01, 2.3034E+04, &
      1.4755462E-01, 1.71681E+01, 1.4395E+01, 2.3000E+04, &
      1.4829239E-01, 1.72222E+01, 1.4445E+01, 2.2966E+04, &
      1.4903386E-01, 1.72765E+01, 1.4495E+01, 2.2931E+04, &
      1.4977903E-01, 1.73311E+01, 1.4545E+01, 2.2896E+04, &
      1.5052792E-01, 1.73858E+01, 1.4595E+01, 2.2860E+04, &
      1.5128056E-01, 1.74407E+01, 1.4644E+01, 2.2823E+04, &
      1.5203696E-01, 1.74958E+01, 1.4694E+01, 2.2786E+04, &
      1.5279715E-01, 1.75511E+01, 1.4743E+01, 2.2748E+04, &
      1.5356113E-01, 1.76066E+01, 1.4791E+01, 2.2710E+04, &
      1.5432894E-01, 1.76622E+01, 1.4840E+01, 2.2671E+04, &
      1.5510058E-01, 1.77179E+01, 1.4888E+01, 2.2632E+04, &
      1.5587609E-01, 1.77738E+01, 1.4936E+01, 2.2592E+04, &
      1.5665547E-01, 1.78298E+01, 1.4984E+01, 2.2551E+04, &
      1.5743875E-01, 1.78859E+01, 1.5032E+01, 2.2511E+04, &
      1.5822594E-01, 1.79422E+01, 1.5079E+01, 2.2469E+04, &
      1.5901707E-01, 1.79985E+01, 1.5126E+01, 2.2427E+04, &
      1.5981215E-01, 1.80550E+01, 1.5173E+01, 2.2385E+04, &
      1.6061121E-01, 1.81115E+01, 1.5220E+01, 2.2342E+04, &
      1.6141427E-01, 1.81681E+01, 1.5266E+01, 2.2298E+04, &
      1.6222134E-01, 1.82247E+01, 1.5312E+01, 2.2254E+04, &
      1.6303245E-01, 1.82814E+01, 1.5358E+01, 2.2210E+04, &
      1.6384761E-01, 1.83382E+01, 1.5404E+01, 2.2165E+04, &
      1.6466685E-01, 1.83949E+01, 1.5449E+01, 2.2120E+04, &
      1.6549018E-01, 1.84517E+01, 1.5494E+01, 2.2074E+04, &
      1.6631763E-01, 1.85085E+01, 1.5539E+01, 2.2028E+04, &
      1.6714922E-01, 1.85652E+01, 1.5584E+01, 2.1981E+04, &
      1.6798497E-01, 1.86219E+01, 1.5628E+01, 2.1934E+04, &
      1.6882489E-01, 1.86786E+01, 1.5672E+01, 2.1887E+04, &
      1.6966902E-01, 1.87352E+01, 1.5716E+01, 2.1839E+04, &
      1.7051736E-01, 1.87917E+01, 1.5760E+01, 2.1791E+04, &
      1.7136995E-01, 1.88481E+01, 1.5803E+01, 2.1742E+04, &
      1.7222680E-01, 1.89044E+01, 1.5847E+01, 2.1693E+04, &
      1.7308793E-01, 1.89605E+01, 1.5890E+01, 2.1644E+04, &
      1.7395337E-01, 1.90165E+01, 1.5932E+01, 2.1594E+04, &
      1.7482314E-01, 1.90723E+01, 1.5975E+01, 2.1544E+04, &
      1.7569726E-01, 1.91279E+01, 1.6017E+01, 2.1494E+04, &
      1.7657574E-01, 1.91832E+01, 1.6059E+01, 2.1443E+04, &
      1.7745862E-01, 1.92383E+01, 1.6101E+01, 2.1392E+04, &
      1.7834591E-01, 1.92930E+01, 1.6143E+01, 2.1341E+04, &
      1.7923764E-01, 1.93475E+01, 1.6184E+01, 2.1289E+04, &
      1.8013383E-01, 1.94015E+01, 1.6225E+01, 2.1237E+04, &
      1.8103450E-01, 1.94552E+01, 1.6266E+01, 2.1184E+04, &
      1.8193967E-01, 1.95084E+01, 1.6307E+01, 2.1132E+04, &
      1.8284937E-01, 1.95611E+01, 1.6347E+01, 2.1079E+04, &
      1.8376362E-01, 1.96133E+01, 1.6387E+01, 2.1025E+04, &
      1.8468244E-01, 1.96649E+01, 1.6427E+01, 2.0972E+04, &
      1.8560585E-01, 1.97158E+01, 1.6467E+01, 2.0918E+04, &
      1.8653388E-01, 1.97659E+01, 1.6506E+01, 2.0864E+04, &
      1.8746655E-01, 1.98153E+01, 1.6546E+01, 2.0810E+04, &
      1.8840388E-01, 1.98638E+01, 1.6585E+01, 2.0755E+04, &
      1.8934590E-01, 1.99113E+01, 1.6624E+01, 2.0700E+04, &
      1.9029263E-01, 1.99577E+01, 1.6662E+01, 2.0645E+04, &
      1.9124409E-01, 2.00030E+01, 1.6701E+01, 2.0590E+04, &
      1.9220031E-01, 2.00469E+01, 1.6739E+01, 2.0534E+04, &
      1.9316131E-01, 2.00893E+01, 1.6777E+01, 2.0478E+04, &
      1.9412712E-01, 2.01300E+01, 1.6815E+01, 2.0422E+04, &
      1.9509776E-01, 2.01710E+01, 1.6852E+01, 2.0366E+04, &
      1.9607325E-01, 2.02078E+01, 1.6890E+01, 2.0310E+04, &
      1.9705361E-01, 2.02422E+01, 1.6927E+01, 2.0253E+04, &
      1.9803888E-01, 2.02739E+01, 1.6964E+01, 2.0197E+04, &
      1.9902907E-01, 2.03025E+01, 1.7001E+01, 2.0140E+04, &
      2.0002422E-01, 2.03275E+01, 1.7037E+01, 2.0083E+04, &
      2.0102434E-01, 2.03483E+01, 1.7074E+01, 2.0025E+04, &
      2.0202946E-01, 2.03642E+01, 1.7110E+01, 1.9968E+04, &
      2.0303961E-01, 2.03743E+01, 1.7146E+01, 1.9911E+04, &
      2.0405481E-01, 2.03772E+01, 1.7182E+01, 1.9853E+04, &
      2.0507508E-01, 2.03713E+01, 1.7217E+01, 1.9795E+04, &
      2.0610046E-01, 2.03544E+01, 1.7253E+01, 1.9737E+04, &
      2.0713096E-01, 2.03232E+01, 1.7288E+01, 1.9679E+04, &
      2.0816661E-01, 2.02725E+01, 1.7323E+01, 1.9621E+04, &
      2.0920745E-01, 2.01942E+01, 1.7358E+01, 1.9562E+04, &
      2.1025348E-01, 2.00733E+01, 1.7392E+01, 1.9504E+04, &
      2.1130475E-01, 1.98782E+01, 1.7427E+01, 1.9445E+04, &
      2.1236128E-01, 1.95191E+01, 1.7461E+01, 1.9387E+04, &
      2.1342308E-01, 1.84033E+01, 1.7495E+01, 1.9328E+04, &
      2.1354015E-01, 1.79915E+01, 1.7499E+01, 1.9322E+04, &
      2.1385984E-01, 1.80336E+01, 1.9932E+01, 2.1975E+04, &
      2.1449020E-01, 1.93322E+01, 1.9917E+01, 2.1893E+04, &
      2.1556265E-01, 2.00728E+01, 1.9891E+01, 2.1757E+04, &
      2.1664046E-01, 2.04815E+01, 1.9868E+01, 2.1623E+04, &
      2.1772366E-01, 2.07598E+01, 1.9845E+01, 2.1491E+04, &
      2.1881228E-01, 2.09582E+01, 1.9824E+01, 2.1361E+04, &
      2.1990634E-01, 2.10913E+01, 1.9803E+01, 2.1233E+04, &
      2.2100588E-01, 2.11533E+01, 1.9784E+01, 2.1107E+04, &
      2.2211090E-01, 2.11048E+01, 1.9766E+01, 2.0983E+04, &
      2.2322146E-01, 2.07351E+01, 1.9750E+01, 2.0861E+04, &
      2.2361559E-01, 2.01824E+01, 1.9744E+01, 2.0818E+04, &
      2.2398441E-01, 2.02635E+01, 2.1474E+01, 2.2605E+04, &
      2.2433757E-01, 2.09271E+01, 2.1455E+01, 2.2549E+04, &
      2.2545925E-01, 2.17506E+01, 2.1396E+01, 2.2375E+04, &
      2.2658655E-01, 2.22159E+01, 2.1339E+01, 2.2205E+04, &
      2.2771948E-01, 2.25684E+01, 2.1284E+01, 2.2037E+04, &
      2.2885808E-01, 2.28618E+01, 2.1231E+01, 2.1873E+04, &
      2.3000237E-01, 2.31173E+01, 2.1180E+01, 2.1712E+04, &
      2.3115238E-01, 2.33457E+01, 2.1131E+01, 2.1554E+04, &
      2.3230814E-01, 2.35535E+01, 2.1084E+01, 2.1399E+04, &
      2.3346969E-01, 2.37447E+01, 2.1039E+01, 2.1247E+04, &
      2.3463703E-01, 2.39221E+01, 2.0995E+01, 2.1098E+04, &
      2.3581022E-01, 2.40876E+01, 2.0954E+01, 2.0951E+04, &
      2.3698927E-01, 2.42428E+01, 2.0915E+01, 2.0809E+04, &
      2.3817422E-01, 2.43895E+01, 2.0879E+01, 2.0670E+04, &
      2.3936509E-01, 2.45288E+01, 2.0846E+01, 2.0535E+04, &
      2.4056191E-01, 2.46618E+01, 2.0816E+01, 2.0403E+04, &
      2.4176472E-01, 2.47893E+01, 2.0788E+01, 2.0274E+04, &
      2.4297355E-01, 2.49118E+01, 2.0762E+01, 2.0148E+04, &
      2.4418841E-01, 2.50299E+01, 2.0739E+01, 2.0026E+04, &
      2.4540936E-01, 2.51440E+01, 2.0718E+01, 1.9906E+04, &
      2.4663640E-01, 2.52543E+01, 2.0699E+01, 1.9788E+04, &
      2.4786959E-01, 2.53612E+01, 2.0682E+01, 1.9674E+04, &
      2.4910893E-01, 2.54652E+01, 2.0668E+01, 1.9563E+04, &
      2.5035448E-01, 2.55668E+01, 2.0657E+01, 1.9455E+04, &
      2.5160625E-01, 2.56664E+01, 2.0648E+01, 1.9350E+04, &
      2.5286428E-01, 2.57644E+01, 2.0642E+01, 1.9248E+04, &
      2.5412860E-01, 2.58609E+01, 2.0637E+01, 1.9148E+04, &
      2.5539925E-01, 2.59563E+01, 2.0635E+01, 1.9051E+04, &
      2.5667624E-01, 2.60507E+01, 2.0635E+01, 1.8956E+04, &
      2.5795962E-01, 2.61443E+01, 2.0636E+01, 1.8863E+04, &
      2.5924942E-01, 2.62373E+01, 2.0639E+01, 1.8772E+04, &
      2.6054567E-01, 2.63297E+01, 2.0644E+01, 1.8683E+04, &
      2.6184840E-01, 2.64216E+01, 2.0650E+01, 1.8595E+04, &
      2.6315764E-01, 2.65133E+01, 2.0658E+01, 1.8510E+04, &
      2.6447343E-01, 2.66048E+01, 2.0667E+01, 1.8426E+04, &
      2.6579579E-01, 2.66961E+01, 2.0678E+01, 1.8344E+04, &
      2.6712477E-01, 2.67874E+01, 2.0690E+01, 1.8263E+04, &
      2.6846040E-01, 2.68787E+01, 2.0703E+01, 1.8184E+04, &
      2.6980270E-01, 2.69702E+01, 2.0717E+01, 1.8105E+04, &
      2.7115171E-01, 2.70617E+01, 2.0732E+01, 1.8029E+04, &
      2.7250747E-01, 2.71535E+01, 2.0748E+01, 1.7953E+04, &
      2.7387001E-01, 2.72455E+01, 2.0766E+01, 1.7879E+04, &
      2.7523936E-01, 2.73379E+01, 2.0784E+01, 1.7806E+04, &
      2.7661556E-01, 2.74306E+01, 2.0803E+01, 1.7734E+04, &
      2.7799863E-01, 2.75237E+01, 2.0823E+01, 1.7662E+04, &
      2.7938863E-01, 2.76173E+01, 2.0845E+01, 1.7592E+04, &
      2.8078557E-01, 2.77114E+01, 2.0866E+01, 1.7523E+04, &
      2.8218950E-01, 2.78060E+01, 2.0889E+01, 1.7455E+04, &
      2.8360044E-01, 2.79012E+01, 2.0913E+01, 1.7388E+04, &
      2.8501845E-01, 2.79970E+01, 2.0937E+01, 1.7322E+04, &
      2.8644354E-01, 2.80934E+01, 2.0962E+01, 1.7256E+04, &
      2.8787576E-01, 2.81962E+01, 2.0979E+01, 1.7184E+04, &
      2.8931514E-01, 2.83021E+01, 2.0990E+01, 1.7108E+04, &
      2.9076171E-01, 2.84080E+01, 2.1003E+01, 1.7033E+04, &
      2.9221552E-01, 2.85140E+01, 2.1016E+01, 1.6959E+04, &
      2.9367660E-01, 2.86200E+01, 2.1029E+01, 1.6885E+04, &
      2.9514498E-01, 2.87262E+01, 2.1043E+01, 1.6812E+04, &
      2.9662071E-01, 2.88325E+01, 2.1057E+01, 1.6740E+04, &
      2.9810381E-01, 2.89391E+01, 2.1072E+01, 1.6668E+04, &
      2.9959433E-01, 2.90460E+01, 2.1087E+01, 1.6597E+04, &
      3.0109230E-01, 2.91532E+01, 2.1102E+01, 1.6527E+04, &
      3.0259776E-01, 2.92608E+01, 2.1118E+01, 1.6457E+04, &
      3.0411075E-01, 2.93689E+01, 2.1135E+01, 1.6388E+04, &
      3.0563130E-01, 2.94773E+01, 2.1152E+01, 1.6319E+04, &
      3.0715946E-01, 2.95883E+01, 2.1161E+01, 1.6245E+04, &
      3.0869526E-01, 2.97002E+01, 2.1167E+01, 1.6169E+04, &
      3.1023873E-01, 2.98122E+01, 2.1173E+01, 1.6093E+04, &
      3.1178993E-01, 2.99243E+01, 2.1179E+01, 1.6018E+04, &
      3.1334888E-01, 3.00365E+01, 2.1186E+01, 1.5943E+04, &
      3.1491562E-01, 3.01490E+01, 2.1193E+01, 1.5869E+04, &
      3.1649020E-01, 3.02617E+01, 2.1200E+01, 1.5796E+04, &
      3.1807265E-01, 3.03746E+01, 2.1207E+01, 1.5722E+04, &
      3.1966301E-01, 3.04878E+01, 2.1215E+01, 1.5650E+04, &
      3.2126133E-01, 3.06016E+01, 2.1216E+01, 1.5573E+04, &
      3.2286764E-01, 3.07163E+01, 2.1163E+01, 1.5457E+04, &
      3.2448197E-01, 3.08286E+01, 2.1109E+01, 1.5341E+04, &
      3.2610438E-01, 3.09386E+01, 2.1056E+01, 1.5226E+04, &
      3.2773491E-01, 3.10463E+01, 2.1003E+01, 1.5112E+04, &
      3.2937358E-01, 3.11517E+01, 2.0949E+01, 1.4999E+04, &
      3.3102045E-01, 3.12549E+01, 2.0896E+01, 1.4886E+04, &
      3.3267555E-01, 3.13558E+01, 2.0843E+01, 1.4775E+04, &
      3.3433893E-01, 3.14544E+01, 2.0790E+01, 1.4664E+04, &
      3.3601062E-01, 3.15508E+01, 2.0737E+01, 1.4554E+04, &
      3.3769068E-01, 3.16450E+01, 2.0684E+01, 1.4444E+04, &
      3.3937913E-01, 3.17368E+01, 2.0631E+01, 1.4336E+04, &
      3.4107602E-01, 3.18263E+01, 2.0578E+01, 1.4228E+04, &
      3.4278140E-01, 3.19135E+01, 2.0525E+01, 1.4121E+04, &
      3.4449531E-01, 3.19982E+01, 2.0472E+01, 1.4014E+04, &
      3.4621779E-01, 3.20804E+01, 2.0420E+01, 1.3909E+04, &
      3.4794888E-01, 3.21600E+01, 2.0367E+01, 1.3804E+04, &
      3.4968862E-01, 3.22369E+01, 2.0315E+01, 1.3700E+04, &
      3.5143706E-01, 3.23109E+01, 2.0262E+01, 1.3597E+04, &
      3.5319425E-01, 3.23819E+01, 2.0210E+01, 1.3494E+04, &
      3.5496022E-01, 3.24497E+01, 2.0158E+01, 1.3393E+04, &
      3.5673502E-01, 3.25140E+01, 2.0106E+01, 1.3292E+04, &
      3.5851870E-01, 3.25744E+01, 2.0054E+01, 1.3192E+04, &
      3.6031129E-01, 3.26305E+01, 2.0002E+01, 1.3092E+04, &
      3.6211285E-01, 3.26818E+01, 1.9951E+01, 1.2993E+04, &
      3.6392341E-01, 3.27275E+01, 1.9899E+01, 1.2895E+04, &
      3.6574303E-01, 3.27665E+01, 1.9848E+01, 1.2798E+04, &
      3.6757174E-01, 3.27974E+01, 1.9796E+01, 1.2702E+04, &
      3.6940960E-01, 3.28179E+01, 1.9745E+01, 1.2606E+04, &
      3.7125665E-01, 3.28248E+01, 1.9694E+01, 1.2511E+04, &
      3.7311293E-01, 3.28122E+01, 1.9644E+01, 1.2417E+04, &
      3.7497850E-01, 3.27692E+01, 1.9593E+01, 1.2323E+04, &
      3.7685339E-01, 3.26703E+01, 1.9543E+01, 1.2230E+04, &
      3.7873766E-01, 3.24286E+01, 1.9492E+01, 1.2138E+04, &
      3.7992070E-01, 3.19507E+01, 1.9461E+01, 1.2081E+04, &
      3.8063135E-01, 3.16709E+01, 2.0771E+01, 1.2870E+04, &
      3.8087931E-01, 3.19886E+01, 2.0765E+01, 1.2857E+04, &
      3.8253450E-01, 3.26859E+01, 2.0724E+01, 1.2777E+04, &
      3.8444718E-01, 3.30324E+01, 2.0678E+01, 1.2685E+04, &
      3.8636941E-01, 3.32722E+01, 2.0632E+01, 1.2594E+04, &
      3.8830126E-01, 3.34656E+01, 2.0587E+01, 1.2504E+04, &
      3.9024276E-01, 3.36327E+01, 2.0541E+01, 1.2414E+04, &
      3.9219398E-01, 3.37826E+01, 2.0496E+01, 1.2325E+04, &
      3.9415495E-01, 3.39203E+01, 2.0451E+01, 1.2237E+04, &
      3.9612572E-01, 3.40487E+01, 2.0406E+01, 1.2150E+04, &
      3.9810635E-01, 3.41698E+01, 2.0361E+01, 1.2063E+04, &
      4.0009688E-01, 3.42848E+01, 2.0317E+01, 1.1977E+04, &
      4.0209737E-01, 3.43947E+01, 2.0273E+01, 1.1891E+04, &
      4.0410785E-01, 3.45002E+01, 2.0228E+01, 1.1806E+04, &
      4.0612839E-01, 3.46018E+01, 2.0185E+01, 1.1722E+04, &
      4.0815904E-01, 3.46998E+01, 2.0141E+01, 1.1638E+04, &
      4.1019983E-01, 3.47946E+01, 2.0097E+01, 1.1556E+04, &
      4.1225083E-01, 3.48862E+01, 2.0054E+01, 1.1473E+04, &
      4.1431208E-01, 3.49748E+01, 2.0011E+01, 1.1392E+04, &
      4.1638364E-01, 3.50605E+01, 1.9968E+01, 1.1311E+04, &
      4.1846556E-01, 3.51431E+01, 1.9925E+01, 1.1231E+04, &
      4.2055789E-01, 3.52226E+01, 1.9883E+01, 1.1151E+04, &
      4.2266068E-01, 3.52986E+01, 1.9841E+01, 1.1072E+04, &
      4.2477398E-01, 3.53706E+01, 1.9799E+01, 1.0994E+04, &
      4.2689785E-01, 3.54378E+01, 1.9757E+01, 1.0916E+04, &
      4.2903234E-01, 3.54985E+01, 1.9715E+01, 1.0839E+04, &
      4.3117750E-01, 3.55496E+01, 1.9673E+01, 1.0762E+04, &
      4.3333339E-01, 3.55832E+01, 1.9632E+01, 1.0686E+04, &
      4.3550006E-01, 3.55686E+01, 1.9591E+01, 1.0611E+04, &
      4.3637510E-01, 3.55062E+01, 1.9574E+01, 1.0580E+04, &
      4.3762492E-01, 3.55521E+01, 1.9892E+01, 1.0721E+04, &
      4.3767756E-01, 3.55628E+01, 1.9891E+01, 1.0719E+04, &
      4.3986595E-01, 3.57992E+01, 1.9851E+01, 1.0645E+04, &
      4.4206528E-01, 3.59405E+01, 1.9812E+01, 1.0571E+04, &
      4.4427560E-01, 3.60587E+01, 1.9774E+01, 1.0498E+04, &
      4.4649698E-01, 3.61657E+01, 1.9735E+01, 1.0426E+04, &
      4.4872947E-01, 3.62660E+01, 1.9696E+01, 1.0354E+04, &
      4.5097311E-01, 3.63617E+01, 1.9658E+01, 1.0282E+04, &
      4.5322798E-01, 3.64539E+01, 1.9620E+01, 1.0211E+04, &
      4.5549412E-01, 3.65433E+01, 1.9582E+01, 1.0141E+04, &
      4.5777159E-01, 3.66305E+01, 1.9544E+01, 1.0071E+04, &
      4.6006045E-01, 3.67157E+01, 1.9506E+01, 1.0001E+04, &
      4.6236075E-01, 3.67992E+01, 1.9469E+01, 9.9326E+03, &
      4.6467255E-01, 3.68811E+01, 1.9431E+01, 9.8642E+03, &
      4.6699592E-01, 3.69617E+01, 1.9394E+01, 9.7963E+03, &
      4.6933090E-01, 3.70409E+01, 1.9357E+01, 9.7290E+03, &
      4.7167755E-01, 3.71190E+01, 1.9320E+01, 9.6621E+03, &
      4.7403594E-01, 3.71959E+01, 1.9283E+01, 9.5957E+03, &
      4.7640612E-01, 3.72717E+01, 1.9246E+01, 9.5298E+03, &
      4.7878815E-01, 3.73465E+01, 1.9209E+01, 9.4644E+03, &
      4.8118209E-01, 3.74203E+01, 1.9173E+01, 9.3994E+03, &
      4.8358800E-01, 3.74930E+01, 1.9136E+01, 9.3349E+03, &
      4.8600594E-01, 3.75648E+01, 1.9100E+01, 9.2709E+03, &
      4.8843597E-01, 3.76356E+01, 1.9063E+01, 9.2073E+03, &
      4.9087815E-01, 3.77053E+01, 1.9027E+01, 9.1442E+03, &
      4.9333254E-01, 3.77740E+01, 1.8991E+01, 9.0814E+03, &
      4.9579920E-01, 3.78417E+01, 1.8955E+01, 9.0192E+03, &
      4.9827820E-01, 3.79082E+01, 1.8919E+01, 8.9573E+03, &
      5.0076959E-01, 3.79735E+01, 1.8883E+01, 8.8959E+03, &
      5.0327344E-01, 3.80376E+01, 1.8847E+01, 8.8349E+03, &
      5.0578980E-01, 3.81004E+01, 1.8811E+01, 8.7743E+03, &
      5.0831875E-01, 3.81616E+01, 1.8776E+01, 8.7141E+03, &
      5.1086035E-01, 3.82211E+01, 1.8740E+01, 8.6543E+03, &
      5.1341465E-01, 3.82786E+01, 1.8704E+01, 8.5949E+03, &
      5.1598172E-01, 3.83339E+01, 1.8668E+01, 8.5359E+03, &
      5.1856163E-01, 3.83864E+01, 1.8633E+01, 8.4773E+03, &
      5.2115444E-01, 3.84355E+01, 1.8597E+01, 8.4190E+03, &
      5.2376021E-01, 3.84801E+01, 1.8561E+01, 8.3612E+03, &
      5.2637901E-01, 3.85185E+01, 1.8526E+01, 8.3037E+03, &
      5.2901091E-01, 3.85477E+01, 1.8490E+01, 8.2466E+03, &
      5.3165596E-01, 3.85615E+01, 1.8455E+01, 8.1898E+03, &
      5.3431424E-01, 3.85434E+01, 1.8419E+01, 8.1334E+03, &
      5.3698581E-01, 3.84090E+01, 1.8383E+01, 8.0773E+03, &
      5.3724443E-01, 3.83715E+01, 1.8380E+01, 8.0719E+03, &
      5.3895558E-01, 3.84161E+01, 1.8884E+01, 8.2666E+03, &
      5.3967074E-01, 3.85364E+01, 1.8874E+01, 8.2517E+03, &
      5.4236910E-01, 3.87738E+01, 1.8839E+01, 8.1956E+03, &
      5.4508094E-01, 3.89261E+01, 1.8805E+01, 8.1398E+03, &
      5.4780635E-01, 3.90511E+01, 1.8770E+01, 8.0843E+03, &
      5.5054538E-01, 3.91623E+01, 1.8735E+01, 8.0292E+03, &
      5.5329810E-01, 3.92652E+01, 1.8700E+01, 7.9745E+03, &
      5.5606460E-01, 3.93625E+01, 1.8665E+01, 7.9200E+03, &
      5.5884492E-01, 3.94556E+01, 1.8630E+01, 7.8659E+03, &
      5.6163914E-01, 3.95456E+01, 1.8595E+01, 7.8121E+03, &
      5.6444734E-01, 3.96331E+01, 1.8560E+01, 7.7586E+03, &
      5.6726958E-01, 3.97186E+01, 1.8524E+01, 7.7054E+03, &
      5.7010592E-01, 3.98024E+01, 1.8489E+01, 7.6525E+03, &
      5.7295645E-01, 3.98847E+01, 1.8454E+01, 7.5999E+03, &
      5.7582123E-01, 3.99658E+01, 1.8418E+01, 7.5476E+03, &
      5.7870034E-01, 4.00457E+01, 1.8382E+01, 7.4956E+03, &
      5.8159384E-01, 4.01247E+01, 1.8347E+01, 7.4438E+03, &
      5.8450181E-01, 4.02027E+01, 1.8311E+01, 7.3924E+03, &
      5.8742432E-01, 4.02800E+01, 1.8275E+01, 7.3412E+03, &
      5.9036144E-01, 4.03565E+01, 1.8239E+01, 7.2903E+03, &
      5.9331325E-01, 4.04323E+01, 1.8202E+01, 7.2397E+03, &
      5.9627982E-01, 4.05074E+01, 1.8166E+01, 7.1894E+03, &
      5.9926122E-01, 4.05820E+01, 1.8130E+01, 7.1393E+03, &
      6.0225752E-01, 4.06560E+01, 1.8093E+01, 7.0895E+03, &
      6.0526881E-01, 4.07295E+01, 1.8056E+01, 7.0400E+03, &
      6.0829515E-01, 4.08024E+01, 1.8019E+01, 6.9907E+03, &
      6.1133663E-01, 4.08749E+01, 1.7982E+01, 6.9417E+03, &
      6.1439331E-01, 4.09468E+01, 1.7945E+01, 6.8929E+03, &
      6.1746528E-01, 4.10183E+01, 1.7907E+01, 6.8444E+03, &
      6.2055260E-01, 4.10893E+01, 1.7870E+01, 6.7961E+03, &
      6.2365537E-01, 4.11599E+01, 1.7832E+01, 6.7481E+03, &
      6.2677364E-01, 4.12301E+01, 1.7794E+01, 6.7003E+03, &
      6.2990751E-01, 4.12999E+01, 1.7756E+01, 6.6528E+03, &
      6.3305705E-01, 4.13692E+01, 1.7718E+01, 6.6055E+03, &
      6.3622234E-01, 4.14381E+01, 1.7679E+01, 6.5584E+03, &
      6.3940345E-01, 4.15066E+01, 1.7641E+01, 6.5116E+03, &
      6.4260046E-01, 4.15614E+01, 1.7602E+01, 6.4651E+03, &
      6.4581347E-01, 4.16290E+01, 1.7563E+01, 6.4187E+03, &
      6.4904253E-01, 4.16963E+01, 1.7524E+01, 6.3726E+03, &
      6.5228775E-01, 4.17632E+01, 1.7484E+01, 6.3268E+03, &
      6.5554919E-01, 4.18297E+01, 1.7445E+01, 6.2811E+03, &
      6.5882693E-01, 4.18958E+01, 1.7405E+01, 6.2357E+03, &
      6.6212107E-01, 4.19616E+01, 1.7365E+01, 6.1905E+03, &
      6.6543167E-01, 4.20269E+01, 1.7325E+01, 6.1456E+03, &
      6.6875883E-01, 4.20919E+01, 1.7285E+01, 6.1009E+03, &
      6.7210262E-01, 4.21505E+01, 1.7244E+01, 6.0564E+03, &
      6.7546314E-01, 4.22147E+01, 1.7204E+01, 6.0121E+03, &
      6.7884045E-01, 4.22786E+01, 1.7163E+01, 5.9681E+03, &
      6.8223466E-01, 4.23421E+01, 1.7122E+01, 5.9243E+03, &
      6.8564583E-01, 4.24052E+01, 1.7080E+01, 5.8807E+03, &
      6.8907406E-01, 4.24680E+01, 1.7039E+01, 5.8373E+03, &
      6.9251943E-01, 4.25303E+01, 1.6997E+01, 5.7941E+03, &
      6.9598202E-01, 4.25923E+01, 1.6956E+01, 5.7512E+03, &
      6.9946194E-01, 4.26540E+01, 1.6913E+01, 5.7085E+03, &
      7.0295924E-01, 4.27152E+01, 1.6871E+01, 5.6660E+03, &
      7.0647404E-01, 4.27761E+01, 1.6829E+01, 5.6237E+03, &
      7.1000641E-01, 4.28366E+01, 1.6786E+01, 5.5817E+03, &
      7.1355644E-01, 4.28967E+01, 1.6744E+01, 5.5398E+03, &
      7.1712423E-01, 4.29565E+01, 1.6701E+01, 5.4982E+03, &
      7.2070985E-01, 4.30159E+01, 1.6657E+01, 5.4568E+03, &
      7.2431340E-01, 4.30749E+01, 1.6614E+01, 5.4156E+03, &
      7.2793496E-01, 4.31336E+01, 1.6571E+01, 5.3746E+03, &
      7.3157464E-01, 4.31919E+01, 1.6527E+01, 5.3339E+03, &
      7.3523251E-01, 4.32499E+01, 1.6483E+01, 5.2933E+03, &
      7.3890867E-01, 4.33075E+01, 1.6439E+01, 5.2530E+03, &
      7.4260322E-01, 4.33646E+01, 1.6394E+01, 5.2127E+03, &
      7.4631623E-01, 4.34214E+01, 1.6349E+01, 5.1727E+03, &
      7.5004781E-01, 4.34776E+01, 1.6304E+01, 5.1328E+03, &
      7.5379805E-01, 4.35335E+01, 1.6259E+01, 5.0932E+03, &
      7.5756704E-01, 4.35889E+01, 1.6213E+01, 5.0538E+03, &
      7.6135488E-01, 4.36440E+01, 1.6168E+01, 5.0145E+03, &
      7.6516165E-01, 4.36985E+01, 1.6122E+01, 4.9755E+03, &
      7.6898746E-01, 4.37527E+01, 1.6076E+01, 4.9368E+03, &
      7.7283240E-01, 4.38064E+01, 1.6030E+01, 4.8982E+03, &
      7.7669656E-01, 4.38597E+01, 1.5983E+01, 4.8598E+03, &
      7.8058004E-01, 4.39126E+01, 1.5937E+01, 4.8216E+03, &
      7.8448294E-01, 4.39651E+01, 1.5890E+01, 4.7837E+03, &
      7.8840536E-01, 4.40172E+01, 1.5843E+01, 4.7460E+03, &
      7.9234738E-01, 4.40688E+01, 1.5796E+01, 4.7084E+03, &
      7.9630912E-01, 4.41200E+01, 1.5749E+01, 4.6711E+03, &
      8.0029067E-01, 4.41708E+01, 1.5702E+01, 4.6340E+03, &
      8.0429212E-01, 4.42213E+01, 1.5655E+01, 4.5971E+03, &
      8.0831358E-01, 4.42713E+01, 1.5607E+01, 4.5604E+03, &
      8.1235515E-01, 4.43209E+01, 1.5559E+01, 4.5239E+03, &
      8.1641693E-01, 4.43701E+01, 1.5511E+01, 4.4877E+03, &
      8.2049901E-01, 4.44189E+01, 1.5463E+01, 4.4516E+03, &
      8.2460150E-01, 4.44674E+01, 1.5415E+01, 4.4158E+03, &
      8.2872451E-01, 4.45154E+01, 1.5367E+01, 4.3801E+03, &
      8.3286813E-01, 4.45631E+01, 1.5319E+01, 4.3447E+03, &
      8.3703248E-01, 4.46105E+01, 1.5270E+01, 4.3095E+03, &
      8.4121764E-01, 4.46575E+01, 1.5221E+01, 4.2745E+03, &
      8.4542373E-01, 4.47041E+01, 1.5173E+01, 4.2397E+03, &
      8.4965084E-01, 4.47504E+01, 1.5124E+01, 4.2051E+03, &
      8.5389910E-01, 4.47964E+01, 1.5075E+01, 4.1707E+03, &
      8.5816859E-01, 4.48421E+01, 1.5026E+01, 4.1365E+03, &
      8.6245944E-01, 4.48875E+01, 1.4976E+01, 4.1025E+03, &
      8.6677173E-01, 4.49327E+01, 1.4927E+01, 4.0688E+03, &
      8.7110559E-01, 4.49776E+01, 1.4877E+01, 4.0352E+03, &
      8.7546112E-01, 4.50222E+01, 1.4828E+01, 4.0018E+03, &
      8.7983843E-01, 4.50666E+01, 1.4778E+01, 3.9686E+03, &
      8.8423762E-01, 4.51108E+01, 1.4728E+01, 3.9356E+03, &
      8.8865881E-01, 4.51549E+01, 1.4678E+01, 3.9029E+03, &
      8.9310210E-01, 4.51988E+01, 1.4628E+01, 3.8703E+03, &
      8.9756761E-01, 4.52426E+01, 1.4578E+01, 3.8379E+03, &
      9.0205545E-01, 4.52863E+01, 1.4528E+01, 3.8058E+03, &
      9.0656573E-01, 4.53301E+01, 1.4477E+01, 3.7738E+03, &
      9.1109856E-01, 4.53739E+01, 1.4427E+01, 3.7421E+03, &
      9.1565405E-01, 4.54177E+01, 1.4376E+01, 3.7104E+03, &
      9.2023232E-01, 4.54617E+01, 1.4325E+01, 3.6790E+03, &
      9.2483348E-01, 4.55058E+01, 1.4275E+01, 3.6478E+03, &
      9.2945765E-01, 4.55503E+01, 1.4224E+01, 3.6168E+03, &
      9.3410494E-01, 4.55951E+01, 1.4173E+01, 3.5861E+03, &
      9.3877546E-01, 4.56404E+01, 1.4122E+01, 3.5555E+03, &
      9.4346934E-01, 4.56864E+01, 1.4071E+01, 3.5252E+03, &
      9.4818668E-01, 4.57333E+01, 1.4020E+01, 3.4951E+03, &
      9.5292762E-01, 4.57811E+01, 1.3969E+01, 3.4652E+03, &
      9.5769226E-01, 4.58301E+01, 1.3919E+01, 3.4355E+03, &
      9.6248072E-01, 4.58807E+01, 1.3868E+01, 3.4061E+03, &
      9.6729312E-01, 4.59330E+01, 1.3817E+01, 3.3769E+03, &
      9.7212959E-01, 4.59876E+01, 1.3767E+01, 3.3479E+03, &
      9.7699023E-01, 4.60449E+01, 1.3716E+01, 3.3191E+03, &
      9.8187519E-01, 4.61055E+01, 1.3666E+01, 3.2906E+03, &
      9.8678456E-01, 4.61702E+01, 1.3616E+01, 3.2623E+03, &
      9.9171848E-01, 4.62401E+01, 1.3566E+01, 3.2342E+03, &
      9.9667708E-01, 4.63166E+01, 1.3516E+01, 3.2064E+03, &
      1.0016605E+00, 4.63740E+01, 1.3453E+01, 3.1757E+03, &
      1.0066688E+00, 4.63772E+01, 1.3365E+01, 3.1393E+03, &
      1.0117021E+00, 4.63782E+01, 1.3277E+01, 3.1034E+03, &
      1.0167606E+00, 4.63771E+01, 1.3191E+01, 3.0679E+03, &
      1.0218444E+00, 4.63742E+01, 1.3105E+01, 3.0329E+03, &
      1.0269536E+00, 4.63696E+01, 1.3020E+01, 2.9983E+03, &
      1.0320884E+00, 4.63632E+01, 1.2935E+01, 2.9641E+03, &
      1.0372489E+00, 4.63551E+01, 1.2851E+01, 2.9304E+03, &
      1.0424351E+00, 4.63454E+01, 1.2768E+01, 2.8971E+03, &
      1.0476473E+00, 4.63341E+01, 1.2686E+01, 2.8642E+03, &
      1.0528855E+00, 4.63212E+01, 1.2604E+01, 2.8317E+03, &
      1.0581499E+00, 4.63067E+01, 1.2523E+01, 2.7997E+03, &
      1.0634407E+00, 4.62907E+01, 1.2443E+01, 2.7680E+03, &
      1.0687579E+00, 4.62731E+01, 1.2364E+01, 2.7368E+03, &
      1.0741017E+00, 4.62539E+01, 1.2285E+01, 2.7059E+03, &
      1.0794722E+00, 4.62332E+01, 1.2206E+01, 2.6754E+03, &
      1.0848695E+00, 4.62109E+01, 1.2129E+01, 2.6453E+03, &
      1.0902939E+00, 4.61871E+01, 1.2052E+01, 2.6156E+03, &
      1.0957454E+00, 4.61617E+01, 1.1976E+01, 2.5862E+03, &
      1.1012241E+00, 4.61347E+01, 1.1900E+01, 2.5572E+03, &
      1.1067302E+00, 4.61062E+01, 1.1825E+01, 2.5286E+03, &
      1.1122639E+00, 4.60760E+01, 1.1751E+01, 2.5003E+03, &
      1.1178252E+00, 4.60443E+01, 1.1677E+01, 2.4724E+03, &
      1.1234143E+00, 4.60110E+01, 1.1604E+01, 2.4448E+03, &
      1.1290314E+00, 4.59760E+01, 1.1531E+01, 2.4175E+03, &
      1.1346765E+00, 4.59394E+01, 1.1459E+01, 2.3906E+03, &
      1.1403499E+00, 4.59011E+01, 1.1388E+01, 2.3640E+03, &
      1.1460517E+00, 4.58595E+01, 1.1317E+01, 2.3378E+03, &
      1.1517819E+00, 4.58179E+01, 1.1247E+01, 2.3119E+03, &
      1.1575408E+00, 4.57745E+01, 1.1177E+01, 2.2862E+03, &
      1.1633285E+00, 4.57294E+01, 1.1108E+01, 2.2609E+03, &
      1.1691452E+00, 4.56825E+01, 1.1040E+01, 2.2360E+03, &
      1.1749909E+00, 4.56339E+01, 1.0972E+01, 2.2113E+03, &
      1.1808659E+00, 4.55833E+01, 1.0905E+01, 2.1869E+03, &
      1.1867702E+00, 4.55309E+01, 1.0838E+01, 2.1628E+03, &
      1.1927040E+00, 4.54766E+01, 1.0772E+01, 2.1390E+03, &
      1.1986676E+00, 4.54203E+01, 1.0706E+01, 2.1155E+03, &
      1.2046609E+00, 4.53620E+01, 1.0641E+01, 2.0923E+03, &
      1.2106842E+00, 4.53017E+01, 1.0576E+01, 2.0694E+03, &
      1.2167376E+00, 4.52393E+01, 1.0512E+01, 2.0467E+03, &
      1.2228213E+00, 4.51747E+01, 1.0449E+01, 2.0244E+03, &
      1.2289354E+00, 4.51080E+01, 1.0386E+01, 2.0022E+03, &
      1.2350801E+00, 4.50390E+01, 1.0323E+01, 1.9804E+03, &
      1.2412555E+00, 4.49677E+01, 1.0261E+01, 1.9588E+03, &
      1.2474618E+00, 4.48940E+01, 1.0199E+01, 1.9375E+03, &
      1.2536991E+00, 4.48179E+01, 1.0138E+01, 1.9164E+03, &
      1.2599676E+00, 4.47392E+01, 1.0078E+01, 1.8956E+03, &
      1.2662674E+00, 4.46579E+01, 1.0017E+01, 1.8750E+03, &
      1.2725988E+00, 4.45740E+01, 9.9578E+00, 1.8547E+03, &
      1.2789618E+00, 4.44873E+01, 9.8986E+00, 1.8346E+03, &
      1.2853566E+00, 4.43977E+01, 9.8399E+00, 1.8148E+03, &
      1.2917833E+00, 4.43052E+01, 9.7817E+00, 1.7952E+03, &
      1.2982423E+00, 4.42095E+01, 9.7239E+00, 1.7759E+03, &
      1.3047335E+00, 4.41107E+01, 9.6666E+00, 1.7567E+03, &
      1.3112571E+00, 4.40095E+01, 9.6098E+00, 1.7378E+03, &
      1.3178134E+00, 4.39040E+01, 9.5534E+00, 1.7191E+03, &
      1.3244025E+00, 4.37949E+01, 9.4974E+00, 1.7007E+03, &
      1.3310245E+00, 4.36821E+01, 9.4419E+00, 1.6825E+03, &
      1.3376796E+00, 4.35654E+01, 9.3869E+00, 1.6644E+03, &
      1.3443680E+00, 4.34447E+01, 9.3322E+00, 1.6466E+03, &
      1.3510899E+00, 4.33197E+01, 9.2780E+00, 1.6291E+03, &
      1.3578453E+00, 4.31903E+01, 9.2242E+00, 1.6117E+03, &
      1.3646345E+00, 4.30563E+01, 9.1709E+00, 1.5945E+03, &
      1.3714577E+00, 4.29174E+01, 9.1180E+00, 1.5775E+03, &
      1.3783150E+00, 4.27733E+01, 9.0654E+00, 1.5607E+03, &
      1.3852066E+00, 4.26239E+01, 9.0133E+00, 1.5442E+03, &
      1.3921326E+00, 4.24689E+01, 8.9616E+00, 1.5278E+03, &
      1.3990933E+00, 4.23078E+01, 8.9103E+00, 1.5116E+03, &
      1.4060887E+00, 4.21404E+01, 8.8594E+00, 1.4956E+03, &
      1.4131192E+00, 4.19663E+01, 8.8089E+00, 1.4798E+03, &
      1.4201848E+00, 4.17852E+01, 8.7588E+00, 1.4642E+03, &
      1.4272857E+00, 4.15964E+01, 8.7091E+00, 1.4487E+03, &
      1.4344221E+00, 4.13996E+01, 8.6597E+00, 1.4335E+03, &
      1.4415942E+00, 4.11943E+01, 8.6108E+00, 1.4184E+03, &
      1.4488022E+00, 4.09797E+01, 8.5622E+00, 1.4035E+03, &
      1.4560462E+00, 4.07553E+01, 8.5140E+00, 1.3888E+03, &
      1.4633265E+00, 4.05203E+01, 8.4662E+00, 1.3742E+03, &
      1.4706431E+00, 4.02739E+01, 8.4187E+00, 1.3598E+03, &
      1.4779963E+00, 4.00152E+01, 8.3716E+00, 1.3456E+03, &
      1.4853863E+00, 3.97431E+01, 8.3249E+00, 1.3315E+03, &
      1.4928132E+00, 3.94564E+01, 8.2785E+00, 1.3176E+03, &
      1.5002773E+00, 3.91538E+01, 8.2325E+00, 1.3039E+03, &
      1.5077787E+00, 3.88338E+01, 8.1868E+00, 1.2903E+03, &
      1.5153176E+00, 3.84945E+01, 8.1415E+00, 1.2769E+03, &
      1.5228942E+00, 3.81339E+01, 8.0962E+00, 1.2636E+03, &
      1.5305086E+00, 3.77496E+01, 8.0500E+00, 1.2503E+03, &
      1.5381612E+00, 3.73385E+01, 8.0041E+00, 1.2371E+03, &
      1.5458520E+00, 3.68973E+01, 7.9586E+00, 1.2240E+03, &
      1.5535812E+00, 3.64219E+01, 7.9134E+00, 1.2112E+03, &
      1.5613491E+00, 3.59073E+01, 7.8686E+00, 1.1984E+03, &
      1.5691559E+00, 3.53473E+01, 7.8241E+00, 1.1858E+03, &
      1.5770017E+00, 3.47340E+01, 7.7800E+00, 1.1734E+03, &
      1.5848867E+00, 3.40573E+01, 7.7362E+00, 1.1611E+03, &
      1.5928111E+00, 3.33039E+01, 7.6927E+00, 1.1489E+03, &
      1.6007752E+00, 3.24556E+01, 7.6496E+00, 1.1369E+03, &
      1.6087790E+00, 3.14868E+01, 7.6067E+00, 1.1250E+03, &
      1.6168229E+00, 3.03613E+01, 7.5643E+00, 1.1133E+03, &
      1.6249070E+00, 2.90148E+01, 7.5221E+00, 1.1017E+03, &
      1.6330316E+00, 2.73439E+01, 7.4802E+00, 1.0902E+03, &
      1.6411967E+00, 2.51380E+01, 7.4387E+00, 1.0789E+03, &
      1.6494027E+00, 2.18562E+01, 7.3975E+00, 1.0677E+03, &
      1.6576497E+00, 1.49802E+01, 7.3566E+00, 1.0566E+03, &
      1.6614142E+00, -9.22329E-01, 7.3381E+00, 1.0516E+03, &
      1.6619858E+00, -1.24775E+00, 2.6040E+01, 3.7041E+03, &
      1.6659380E+00, 1.46692E+01, 2.5949E+01, 3.6825E+03, &
      1.6742677E+00, 2.06909E+01, 2.5759E+01, 3.6375E+03, &
      1.6826390E+00, 2.30870E+01, 2.5570E+01, 3.5930E+03, &
      1.6910522E+00, 2.41856E+01, 2.5383E+01, 3.5491E+03, &
      1.6995075E+00, 2.43088E+01, 2.5198E+01, 3.5058E+03, &
      1.7080050E+00, 2.29893E+01, 2.5014E+01, 3.4630E+03, &
      1.7158508E+00, 1.34871E+01, 2.4846E+01, 3.4242E+03, &
      1.7165450E+00, 8.18242E+00, 3.7092E+01, 5.1047E+03, &
      1.7169493E+00, 1.34382E+01, 3.7079E+01, 5.1017E+03, &
      1.7251278E+00, 2.52606E+01, 3.6816E+01, 5.0417E+03, &
      1.7337534E+00, 2.88729E+01, 3.6543E+01, 4.9795E+03, &
      1.7424222E+00, 3.12774E+01, 3.6272E+01, 4.9181E+03, &
      1.7511343E+00, 3.31437E+01, 3.6002E+01, 4.8574E+03, &
      1.7598899E+00, 3.46916E+01, 3.5734E+01, 4.7974E+03, &
      1.7686894E+00, 3.60234E+01, 3.5468E+01, 4.7381E+03, &
      1.7775328E+00, 3.71964E+01, 3.5204E+01, 4.6796E+03, &
      1.7864205E+00, 3.82463E+01, 3.4943E+01, 4.6218E+03, &
      1.7953526E+00, 3.91971E+01, 3.4683E+01, 4.5647E+03, &
      1.8043294E+00, 4.00660E+01, 3.4425E+01, 4.5084E+03, &
      1.8133510E+00, 4.08655E+01, 3.4169E+01, 4.4528E+03, &
      1.8224178E+00, 4.16053E+01, 3.3916E+01, 4.3979E+03, &
      1.8315299E+00, 4.22928E+01, 3.3664E+01, 4.3436E+03, &
      1.8406875E+00, 4.29342E+01, 3.3414E+01, 4.2901E+03, &
      1.8498909E+00, 4.35342E+01, 3.3166E+01, 4.2372E+03, &
      1.8591404E+00, 4.40969E+01, 3.2920E+01, 4.1850E+03, &
      1.8684361E+00, 4.46256E+01, 3.2676E+01, 4.1334E+03, &
      1.8777783E+00, 4.51230E+01, 3.2434E+01, 4.0825E+03, &
      1.8871672E+00, 4.55915E+01, 3.2194E+01, 4.0322E+03, &
      1.8966030E+00, 4.60330E+01, 3.1956E+01, 3.9826E+03, &
      1.9060860E+00, 4.64492E+01, 3.1719E+01, 3.9336E+03, &
      1.9156165E+00, 4.68414E+01, 3.1485E+01, 3.8852E+03, &
      1.9251945E+00, 4.72106E+01, 3.1252E+01, 3.8374E+03, &
      1.9348205E+00, 4.75578E+01, 3.1021E+01, 3.7902E+03, &
      1.9444946E+00, 4.78837E+01, 3.0792E+01, 3.7436E+03, &
      1.9542171E+00, 4.81885E+01, 3.0564E+01, 3.6976E+03, &
      1.9639882E+00, 4.84726E+01, 3.0339E+01, 3.6522E+03, &
      1.9738081E+00, 4.87359E+01, 3.0115E+01, 3.6073E+03, &
      1.9836772E+00, 4.89779E+01, 2.9892E+01, 3.5630E+03, &
      1.9935955E+00, 4.91980E+01, 2.9672E+01, 3.5192E+03, &
      2.0035635E+00, 4.93949E+01, 2.9453E+01, 3.4760E+03, &
      2.0135813E+00, 4.95668E+01, 2.9236E+01, 3.4334E+03, &
      2.0236492E+00, 4.97109E+01, 2.9021E+01, 3.3912E+03, &
      2.0337675E+00, 4.98232E+01, 2.8807E+01, 3.3497E+03, &
      2.0439363E+00, 4.98973E+01, 2.8595E+01, 3.3086E+03, &
      2.0541560E+00, 4.99234E+01, 2.8385E+01, 3.2680E+03, &
      2.0644268E+00, 4.98846E+01, 2.8176E+01, 3.2280E+03, &
      2.0747489E+00, 4.97497E+01, 2.7969E+01, 3.1884E+03, &
      2.0851227E+00, 4.94500E+01, 2.7763E+01, 3.1494E+03, &
      2.0955483E+00, 4.87803E+01, 2.7559E+01, 3.1108E+03, &
      2.1052605E+00, 4.65603E+01, 2.7372E+01, 3.0755E+03, &
      2.1060260E+00, 4.59823E+01, 2.7357E+01, 3.0727E+03, &
      2.1099394E+00, 4.66100E+01, 3.2023E+01, 3.5884E+03, &
      2.1165562E+00, 4.88117E+01, 3.1865E+01, 3.5596E+03, &
      2.1271389E+00, 5.02415E+01, 3.1616E+01, 3.5144E+03, &
      2.1377746E+00, 5.11310E+01, 3.1369E+01, 3.4697E+03, &
      2.1484635E+00, 5.18076E+01, 3.1124E+01, 3.4255E+03, &
      2.1592058E+00, 5.23662E+01, 3.0880E+01, 3.3820E+03, &
      2.1700018E+00, 5.28464E+01, 3.0634E+01, 3.3385E+03, &
      2.1808519E+00, 5.32687E+01, 3.0390E+01, 3.2955E+03, &
      2.1917561E+00, 5.36453E+01, 3.0147E+01, 3.2530E+03, &
      2.2027149E+00, 5.39832E+01, 2.9907E+01, 3.2111E+03, &
      2.2137285E+00, 5.42854E+01, 2.9672E+01, 3.1702E+03, &
      2.2247971E+00, 5.45633E+01, 2.9456E+01, 3.1316E+03, &
      2.2359211E+00, 5.48228E+01, 2.9241E+01, 3.0934E+03, &
      2.2471007E+00, 5.50631E+01, 2.9027E+01, 3.0556E+03, &
      2.2583362E+00, 5.52843E+01, 2.8816E+01, 3.0184E+03, &
      2.2696279E+00, 5.54865E+01, 2.8607E+01, 2.9817E+03, &
      2.2809760E+00, 5.56691E+01, 2.8401E+01, 2.9456E+03, &
      2.2923809E+00, 5.58303E+01, 2.8197E+01, 2.9100E+03, &
      2.3038428E+00, 5.59673E+01, 2.7995E+01, 2.8749E+03, &
      2.3153620E+00, 5.60743E+01, 2.7796E+01, 2.8404E+03, &
      2.3269388E+00, 5.61405E+01, 2.7599E+01, 2.8063E+03, &
      2.3385735E+00, 5.61421E+01, 2.7404E+01, 2.7728E+03, &
      2.3502664E+00, 5.60115E+01, 2.7211E+01, 2.7397E+03, &
      2.3620177E+00, 5.53232E+01, 2.7020E+01, 2.7070E+03, &
      2.3625852E+00, 5.52231E+01, 2.7011E+01, 2.7055E+03, &
      2.3682149E+00, 5.52961E+01, 2.8804E+01, 2.8776E+03, &
      2.3738278E+00, 5.60536E+01, 2.8704E+01, 2.8608E+03, &
      2.3856970E+00, 5.67760E+01, 2.8494E+01, 2.8259E+03, &
      2.3976254E+00, 5.72360E+01, 2.8286E+01, 2.7914E+03, &
      2.4096136E+00, 5.75983E+01, 2.8080E+01, 2.7574E+03, &
      2.4216616E+00, 5.79062E+01, 2.7876E+01, 2.7239E+03, &
      2.4337699E+00, 5.81776E+01, 2.7674E+01, 2.6908E+03, &
      2.4459388E+00, 5.84214E+01, 2.7474E+01, 2.6582E+03, &
      2.4581685E+00, 5.86426E+01, 2.7276E+01, 2.6260E+03, &
      2.4704593E+00, 5.88438E+01, 2.7079E+01, 2.5942E+03, &
      2.4828116E+00, 5.90255E+01, 2.6886E+01, 2.5629E+03, &
      2.4952257E+00, 5.91919E+01, 2.6704E+01, 2.5331E+03, &
      2.5077018E+00, 5.93466E+01, 2.6526E+01, 2.5038E+03, &
      2.5202403E+00, 5.94879E+01, 2.6349E+01, 2.4748E+03, &
      2.5328415E+00, 5.96140E+01, 2.6175E+01, 2.4463E+03, &
      2.5455057E+00, 5.97214E+01, 2.6002E+01, 2.4182E+03, &
      2.5582333E+00, 5.98036E+01, 2.5832E+01, 2.3905E+03, &
      2.5710244E+00, 5.98462E+01, 2.5663E+01, 2.3632E+03, &
      2.5838796E+00, 5.98088E+01, 2.5496E+01, 2.3362E+03, &
      2.5957502E+00, 5.95217E+01, 2.5344E+01, 2.3118E+03, &
      2.5967990E+00, 5.94512E+01, 2.5331E+01, 2.3096E+03, &
      2.6060497E+00, 5.96441E+01, 2.6371E+01, 2.3955E+03, &
      2.6097829E+00, 5.98917E+01, 2.6323E+01, 2.3878E+03, &
      2.6228319E+00, 6.03839E+01, 2.6156E+01, 2.3610E+03, &
      2.6359460E+00, 6.07114E+01, 2.5991E+01, 2.3345E+03, &
      2.6491257E+00, 6.09801E+01, 2.5828E+01, 2.3084E+03, &
      2.6623714E+00, 6.12172E+01, 2.5666E+01, 2.2826E+03, &
      2.6756832E+00, 6.14339E+01, 2.5505E+01, 2.2571E+03, &
      2.6890617E+00, 6.16361E+01, 2.5345E+01, 2.2319E+03, &
      2.7025070E+00, 6.18272E+01, 2.5186E+01, 2.2070E+03, &
      2.7160195E+00, 6.20092E+01, 2.5029E+01, 2.1824E+03, &
      2.7295996E+00, 6.21837E+01, 2.4873E+01, 2.1581E+03, &
      2.7432476E+00, 6.23521E+01, 2.4718E+01, 2.1341E+03, &
      2.7569638E+00, 6.25148E+01, 2.4563E+01, 2.1102E+03, &
      2.7707486E+00, 6.26724E+01, 2.4408E+01, 2.0866E+03, &
      2.7846024E+00, 6.28253E+01, 2.4254E+01, 2.0632E+03, &
      2.7985254E+00, 6.29741E+01, 2.4102E+01, 2.0401E+03, &
      2.8125180E+00, 6.31191E+01, 2.3951E+01, 2.0173E+03, &
      2.8265806E+00, 6.32608E+01, 2.3800E+01, 1.9948E+03, &
      2.8407135E+00, 6.33993E+01, 2.3651E+01, 1.9725E+03, &
      2.8549171E+00, 6.35351E+01, 2.3503E+01, 1.9505E+03, &
      2.8691917E+00, 6.36683E+01, 2.3355E+01, 1.9287E+03, &
      2.8835376E+00, 6.37992E+01, 2.3209E+01, 1.9072E+03, &
      2.8979553E+00, 6.39282E+01, 2.3063E+01, 1.8859E+03, &
      2.9124451E+00, 6.40556E+01, 2.2919E+01, 1.8648E+03, &
      2.9270073E+00, 6.41816E+01, 2.2775E+01, 1.8440E+03, &
      2.9416424E+00, 6.43070E+01, 2.2632E+01, 1.8234E+03, &
      2.9563506E+00, 6.44323E+01, 2.2490E+01, 1.8031E+03, &
      2.9711323E+00, 6.45588E+01, 2.2349E+01, 1.7829E+03, &
      2.9859880E+00, 6.46888E+01, 2.2209E+01, 1.7630E+03, &
      3.0009179E+00, 6.48273E+01, 2.2068E+01, 1.7432E+03, &
      3.0159225E+00, 6.49674E+01, 2.1910E+01, 1.7222E+03, &
      3.0310021E+00, 6.50948E+01, 2.1753E+01, 1.7014E+03, &
      3.0461571E+00, 6.52135E+01, 2.1597E+01, 1.6809E+03, &
      3.0613879E+00, 6.53258E+01, 2.1442E+01, 1.6607E+03, &
      3.0766949E+00, 6.54330E+01, 2.1288E+01, 1.6406E+03, &
      3.0920783E+00, 6.55355E+01, 2.1133E+01, 1.6207E+03, &
      3.1075387E+00, 6.56339E+01, 2.0980E+01, 1.6011E+03, &
      3.1230764E+00, 6.57287E+01, 2.0828E+01, 1.5817E+03, &
      3.1386918E+00, 6.58203E+01, 2.0678E+01, 1.5625E+03, &
      3.1543853E+00, 6.59090E+01, 2.0528E+01, 1.5436E+03, &
      3.1701572E+00, 6.59951E+01, 2.0380E+01, 1.5249E+03, &
      3.1860080E+00, 6.60786E+01, 2.0232E+01, 1.5065E+03, &
      3.2019380E+00, 6.61599E+01, 2.0086E+01, 1.4882E+03, &
      3.2179477E+00, 6.62390E+01, 1.9941E+01, 1.4703E+03, &
      3.2340374E+00, 6.63161E+01, 1.9798E+01, 1.4525E+03, &
      3.2502076E+00, 6.63913E+01, 1.9655E+01, 1.4349E+03, &
      3.2664587E+00, 6.64647E+01, 1.9513E+01, 1.4176E+03, &
      3.2827910E+00, 6.65365E+01, 1.9373E+01, 1.4005E+03, &
      3.2992049E+00, 6.66066E+01, 1.9234E+01, 1.3836E+03, &
      3.3157009E+00, 6.66753E+01, 1.9096E+01, 1.3669E+03, &
      3.3322794E+00, 6.69563E+01, 1.8955E+01, 1.3502E+03, &
      3.3489408E+00, 6.70220E+01, 1.8811E+01, 1.3334E+03, &
      3.3656856E+00, 6.70856E+01, 1.8669E+01, 1.3169E+03, &
      3.3825140E+00, 6.71473E+01, 1.8528E+01, 1.3005E+03, &
      3.3994265E+00, 6.72070E+01, 1.8388E+01, 1.2843E+03, &
      3.4164237E+00, 6.72649E+01, 1.8249E+01, 1.2684E+03, &
      3.4335058E+00, 6.74643E+01, 1.8111E+01, 1.2526E+03, &
      3.4506733E+00, 6.75190E+01, 1.7969E+01, 1.2367E+03, &
      3.4679267E+00, 6.75714E+01, 1.7829E+01, 1.2211E+03, &
      3.4852663E+00, 6.76219E+01, 1.7690E+01, 1.2056E+03, &
      3.5026927E+00, 6.76704E+01, 1.7553E+01, 1.1904E+03, &
      3.5202061E+00, 6.77171E+01, 1.7416E+01, 1.1754E+03, &
      3.5378072E+00, 6.77622E+01, 1.7281E+01, 1.1606E+03, &
      3.5554962E+00, 6.78056E+01, 1.7147E+01, 1.1459E+03, &
      3.5732737E+00, 6.78474E+01, 1.7015E+01, 1.1315E+03, &
      3.5911400E+00, 6.78877E+01, 1.6883E+01, 1.1173E+03, &
      3.6090957E+00, 6.79266E+01, 1.6753E+01, 1.1032E+03, &
      3.6271412E+00, 6.79642E+01, 1.6624E+01, 1.0894E+03, &
      3.6452769E+00, 6.80004E+01, 1.6496E+01, 1.0757E+03, &
      3.6635033E+00, 6.80354E+01, 1.6369E+01, 1.0622E+03, &
      3.6818208E+00, 6.80691E+01, 1.6244E+01, 1.0489E+03, &
      3.7002299E+00, 6.81017E+01, 1.6119E+01, 1.0358E+03, &
      3.7187311E+00, 6.81331E+01, 1.5996E+01, 1.0229E+03, &
      3.7373247E+00, 6.81635E+01, 1.5874E+01, 1.0101E+03, &
      3.7560114E+00, 6.81928E+01, 1.5753E+01, 9.9751E+02, &
      3.7747914E+00, 6.82211E+01, 1.5633E+01, 9.8508E+02, &
      3.7936654E+00, 6.82484E+01, 1.5514E+01, 9.7282E+02, &
      3.8126337E+00, 6.82748E+01, 1.5397E+01, 9.6072E+02, &
      3.8316969E+00, 6.83002E+01, 1.5280E+01, 9.4879E+02, &
      3.8508554E+00, 6.83248E+01, 1.5164E+01, 9.3701E+02, &
      3.8701096E+00, 6.83485E+01, 1.5050E+01, 9.2539E+02, &
      3.8894602E+00, 6.83714E+01, 1.4936E+01, 9.1393E+02, &
      3.9089075E+00, 6.83935E+01, 1.4824E+01, 9.0262E+02, &
      3.9284520E+00, 6.84148E+01, 1.4712E+01, 8.9146E+02, &
      3.9480943E+00, 6.84354E+01, 1.4602E+01, 8.8045E+02, &
      3.9678347E+00, 6.84552E+01, 1.4492E+01, 8.6958E+02, &
      3.9876739E+00, 6.84744E+01, 1.4384E+01, 8.5886E+02, &
      4.055024E+00, 6.89327E+01, 1.4024E+01, 8.2378E+02, &
      4.334821E+00, 6.91536E+01, 1.2659E+01, 6.9666E+02, &
      4.633924E+00, 6.91625E+01, 1.1413E+01, 5.8853E+02, &
      4.953664E+00, 6.91439E+01, 1.0286E+01, 4.9712E+02, &
      5.295467E+00, 6.90244E+01, 9.2735E+00, 4.2012E+02, &
      5.660855E+00, 6.88248E+01, 8.3649E+00, 3.5532E+02, &
      6.051453E+00, 6.85712E+01, 7.5559E+00, 3.0101E+02, &
      6.469004E+00, 6.82652E+01, 6.8346E+00, 2.5541E+02, &
      6.915365E+00, 6.78987E+01, 6.1907E+00, 2.1707E+02, &
      7.392525E+00, 6.74502E+01, 5.6052E+00, 1.8447E+02, &
      7.902609E+00, 6.68549E+01, 5.0609E+00, 1.5639E+02, &
      8.447890E+00, 6.59896E+01, 4.5757E+00, 1.3280E+02, &
      9.030794E+00, 6.44545E+01, 4.1277E+00, 1.1258E+02, &
      9.369486E+00, 6.23361E+01, 3.8676E+00, 1.0198E+02, &
      9.512897E+00, 5.94352E+01, 3.7646E+00, 9.7899E+01, &
      9.551140E+00, 5.60210E+01, 3.7379E+00, 9.6849E+01, &
      9.608504E+00, 5.94578E+01, 1.0373E+01, 2.5908E+02, &
      9.653919E+00, 6.09185E+01, 1.0263E+01, 2.5517E+02, &
      9.751914E+00, 6.24638E+01, 1.0031E+01, 2.4701E+02, &
      1.032004E+01, 6.45998E+01, 8.9060E+00, 2.0771E+02, &
      1.052461E+01, 6.43529E+01, 8.5827E+00, 1.9644E+02, &
      1.068570E+01, 6.31524E+01, 8.3464E+00, 1.8827E+02, &
      1.072866E+01, 6.14934E+01, 8.2858E+00, 1.8619E+02, &
      1.079310E+01, 6.32855E+01, 1.1547E+01, 2.5630E+02, &
      1.095419E+01, 6.48990E+01, 1.1216E+01, 2.4542E+02, &
      1.103212E+01, 6.52180E+01, 1.1063E+01, 2.4041E+02, &
      1.104529E+01, 6.52568E+01, 1.1037E+01, 2.3957E+02, &
      1.121435E+01, 6.52382E+01, 1.0718E+01, 2.2925E+02, &
      1.125943E+01, 6.45312E+01, 1.0636E+01, 2.2661E+02, &
      1.132705E+01, 6.55849E+01, 1.2203E+01, 2.5787E+02, &
      1.149611E+01, 6.67877E+01, 1.1917E+01, 2.4824E+02, &
      1.179334E+01, 6.78684E+01, 1.1453E+01, 2.3270E+02, &
      1.260708E+01, 6.95052E+01, 1.0335E+01, 1.9677E+02, &
      1.347697E+01, 7.04863E+01, 9.3120E+00, 1.6616E+02, &
      1.440688E+01, 7.11522E+01, 8.3767E+00, 1.4011E+02, &
      1.540095E+01, 7.16018E+01, 7.5130E+00, 1.1783E+02, &
      1.646362E+01, 7.18997E+01, 6.7329E+00, 9.9026E+01, &
      1.759961E+01, 7.20921E+01, 6.0332E+00, 8.3239E+01, &
      1.881398E+01, 7.22104E+01, 5.4086E+00, 7.0013E+01, &
      2.011215E+01, 7.23359E+01, 4.8448E+00, 5.8858E+01, &
      2.149988E+01, 7.23919E+01, 4.3415E+00, 4.9514E+01, &
      2.298338E+01, 7.24141E+01, 3.8876E+00, 4.1634E+01, &
      2.456923E+01, 7.23832E+01, 3.4797E+00, 3.5006E+01, &
      2.626450E+01, 7.23387E+01, 3.1172E+00, 2.9466E+01, &
      2.807676E+01, 7.22919E+01, 2.7722E+00, 2.4642E+01, &
      3.001405E+01, 7.22127E+01, 2.4608E+00, 2.0581E+01, &
      3.208502E+01, 7.21207E+01, 2.1862E+00, 1.7212E+01, &
      3.429889E+01, 7.20209E+01, 1.9439E+00, 1.4414E+01, &
      3.666551E+01, 7.19153E+01, 1.7298E+00, 1.2085E+01, &
      3.919543E+01, 7.18046E+01, 1.5404E+00, 1.0147E+01, &
      4.189992E+01, 7.16879E+01, 1.3723E+00, 8.5288E+00, &
      4.479101E+01, 7.15613E+01, 1.2211E+00, 7.1665E+00, &
      4.788159E+01, 7.14197E+01, 1.0864E+00, 6.0257E+00, &
      5.118542E+01, 7.12520E+01, 9.6347E-01, 5.0574E+00, &
      5.471721E+01, 7.10351E+01, 8.5374E-01, 4.2463E+00, &
      5.849270E+01, 7.07149E+01, 7.5663E-01, 3.5703E+00, &
      6.252870E+01, 7.00604E+01, 6.7096E-01, 3.0076E+00, &
      6.404378E+01, 6.94683E+01, 6.4276E-01, 2.8295E+00, &
      6.502405E+01, 6.83536E+01, 6.2552E-01, 2.7224E+00, &
      6.528545E+01, 6.70188E+01, 6.2105E-01, 2.6949E+00, &
      6.567755E+01, 6.83677E+01, 3.2419E+00, 1.2086E+01, &
      6.665782E+01, 6.95858E+01, 3.1695E+00, 1.1650E+01, &
      6.684318E+01, 6.97049E+01, 3.1561E+00, 1.1571E+01, &
      7.145536E+01, 7.09895E+01, 2.8423E+00, 9.7814E+00, &
      7.638578E+01, 7.15015E+01, 2.5508E+00, 8.2445E+00, &
      8.165640E+01, 7.18021E+01, 2.2872E+00, 6.9465E+00, &
      8.729069E+01, 7.19969E+01, 2.0458E+00, 5.8424E+00, &
      9.331374E+01, 7.21242E+01, 1.8267E+00, 4.9087E+00, &
      9.975239E+01, 7.22082E+01, 1.6314E+00, 4.1278E+00, &
      1.066353E+02, 7.22630E+01, 1.4571E+00, 3.4744E+00, &
      1.139931E+02, 7.22974E+01, 1.3016E+00, 2.9276E+00, &
      1.218587E+02, 7.23173E+01, 1.1628E+00, 2.4698E+00, &
      1.302669E+02, 7.23272E+01, 1.0390E+00, 2.0864E+00, &
      1.392553E+02, 7.23560E+01, 9.2499E-01, 1.7592E+00, &
      1.488640E+02, 7.23494E+01, 8.2342E-01, 1.4857E+00, &
      1.591356E+02, 7.23382E+01, 7.3307E-01, 1.2571E+00, &
      1.701159E+02, 7.23239E+01, 6.5271E-01, 1.0661E+00, &
      1.818539E+02, 7.23076E+01, 5.8120E-01, 9.0637E-01, &
      1.944018E+02, 7.22902E+01, 5.1758E-01, 7.7266E-01, &
      2.078156E+02, 7.22722E+01, 4.6096E-01, 6.6065E-01, &
      2.221548E+02, 7.22541E+01, 4.1057E-01, 5.6676E-01, &
      2.374835E+02, 7.22361E+01, 3.6571E-01, 4.8798E-01, &
      2.538699E+02, 7.22185E+01, 3.2579E-01, 4.2179E-01, &
      2.713869E+02, 7.22012E+01, 2.9024E-01, 3.6613E-01, &
      2.901126E+02, 7.21845E+01, 2.5925E-01, 3.1976E-01, &
      3.101304E+02, 7.21691E+01, 2.3263E-01, 2.8133E-01, &
      3.315294E+02, 7.21552E+01, 2.0876E-01, 2.4863E-01, &
      3.544049E+02, 7.21423E+01, 1.8735E-01, 2.2074E-01, &
      3.788588E+02, 7.21305E+01, 1.6815E-01, 1.9691E-01, &
      4.050001E+02, 7.21195E+01, 1.5093E-01, 1.7647E-01, &
      4.329451E+02, 7.21094E+01, 1.3548E-01, 1.5891E-01]
   case(73)
      frel = -8.3100E-01
      allocate(nist_tab(3636))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.586000E-03, 7.28675E+00, 0.0000E+00, 1.1072E-05, &
      5.671500E-03, 6.76355E+00, 0.0000E+00, 1.2001E-05, &
      5.694300E-03, 6.16326E+00, 0.0000E+00, 1.2259E-05, &
      5.728500E-03, 6.76103E+00, 1.1669E+00, 4.7371E+04, &
      5.814000E-03, 7.26809E+00, 1.1599E+00, 4.6396E+04, &
      1.069000E-02, 7.69658E+00, 2.5477E+00, 5.5425E+04, &
      1.142761E-02, 7.80670E+00, 2.8946E+00, 5.8905E+04, &
      1.221612E-02, 7.98392E+00, 3.2454E+00, 6.1782E+04, &
      1.305903E-02, 8.23109E+00, 3.5808E+00, 6.3767E+04, &
      1.396010E-02, 8.54464E+00, 3.8799E+00, 6.4634E+04, &
      1.492335E-02, 8.91402E+00, 4.1228E+00, 6.4246E+04, &
      1.595306E-02, 9.32207E+00, 4.2927E+00, 6.2577E+04, &
      1.705382E-02, 9.74633E+00, 4.3785E+00, 5.9707E+04, &
      1.823053E-02, 1.01860E+01, 4.3754E+00, 5.5814E+04, &
      1.948844E-02, 1.05634E+01, 4.2858E+00, 5.1143E+04, &
      2.083314E-02, 1.08749E+01, 4.1184E+00, 4.5972E+04, &
      2.227063E-02, 1.10871E+01, 3.8864E+00, 4.0582E+04, &
      2.380730E-02, 1.11318E+01, 3.6051E+00, 3.5215E+04, &
      2.450000E-02, 1.10153E+01, 3.4725E+00, 3.2961E+04, &
      2.487500E-02, 1.07494E+01, 3.4002E+00, 3.1788E+04, &
      2.497500E-02, 1.04085E+01, 3.3809E+00, 3.1481E+04, &
      2.512500E-02, 1.07805E+01, 4.0323E+00, 3.7323E+04, &
      2.545001E-02, 1.10837E+01, 3.9475E+00, 3.6071E+04, &
      2.550000E-02, 1.11091E+01, 3.9346E+00, 3.5883E+04, &
      2.720606E-02, 1.13983E+01, 3.5119E+00, 3.0020E+04, &
      2.908327E-02, 1.12851E+01, 3.1071E+00, 2.4845E+04, &
      3.109002E-02, 1.08955E+01, 2.7577E+00, 2.0628E+04, &
      3.323523E-02, 1.01007E+01, 2.4597E+00, 1.7211E+04, &
      3.552846E-02, 7.89513E+00, 2.2126E+00, 1.4483E+04, &
      3.567200E-02, 7.57475E+00, 2.1993E+00, 1.4338E+04, &
      3.621800E-02, 5.03659E+00, 2.1510E+00, 1.3811E+04, &
      3.636360E-02, 1.99183E+00, 2.1387E+00, 1.3677E+04, &
      3.658200E-02, 5.19860E+00, 8.1125E+00, 5.1572E+04, &
      3.712800E-02, 7.98196E+00, 7.8708E+00, 4.9299E+04, &
      3.797993E-02, 9.52533E+00, 7.5184E+00, 4.6036E+04, &
      4.060054E-02, 1.11476E+01, 6.5983E+00, 3.7794E+04, &
      4.340198E-02, 1.11433E+01, 5.7736E+00, 3.0936E+04, &
      4.400200E-02, 1.08095E+01, 5.6160E+00, 2.9681E+04, &
      4.467550E-02, 9.67052E+00, 5.4478E+00, 2.8358E+04, &
      4.485510E-02, 8.21964E+00, 5.4045E+00, 2.8020E+04, &
      4.512450E-02, 9.77292E+00, 8.2372E+00, 4.2452E+04, &
      4.579800E-02, 1.11684E+01, 8.0408E+00, 4.0830E+04, &
      4.639671E-02, 1.17160E+01, 7.8751E+00, 3.9472E+04, &
      4.959809E-02, 1.29598E+01, 7.1262E+00, 3.3413E+04, &
      5.302035E-02, 1.34527E+01, 6.4845E+00, 2.8442E+04, &
      5.667876E-02, 1.35799E+01, 5.9403E+00, 2.4373E+04, &
      6.058959E-02, 1.34579E+01, 5.5463E+00, 2.1288E+04, &
      6.477028E-02, 1.31680E+01, 5.3333E+00, 1.9149E+04, &
      6.923942E-02, 1.27668E+01, 5.3146E+00, 1.7850E+04, &
      6.967800E-02, 1.27230E+01, 5.3230E+00, 1.7766E+04, &
      7.074450E-02, 1.26034E+01, 5.3504E+00, 1.7588E+04, &
      7.102890E-02, 1.25498E+01, 5.3593E+00, 1.7547E+04, &
      7.145550E-02, 1.25391E+01, 5.4288E+00, 1.7668E+04, &
      7.252200E-02, 1.24678E+01, 5.4730E+00, 1.7550E+04, &
      7.401695E-02, 1.23489E+01, 5.5495E+00, 1.7436E+04, &
      7.912411E-02, 1.21364E+01, 5.9225E+00, 1.7407E+04, &
      8.458368E-02, 1.17681E+01, 6.4758E+00, 1.7804E+04, &
      9.041995E-02, 1.14742E+01, 7.1961E+00, 1.8508E+04, &
      9.665893E-02, 1.12899E+01, 8.0678E+00, 1.9411E+04, &
      1.0000000E-01, 1.28748E+01, 8.5641E+00, 1.9917E+04, &
      1.0050000E-01, 1.28726E+01, 8.6395E+00, 1.9992E+04, &
      1.0100250E-01, 1.28714E+01, 8.7155E+00, 2.0068E+04, &
      1.0150751E-01, 1.28712E+01, 8.7918E+00, 2.0143E+04, &
      1.0201505E-01, 1.28719E+01, 8.8686E+00, 2.0217E+04, &
      1.0252513E-01, 1.28735E+01, 8.9457E+00, 2.0292E+04, &
      1.0303775E-01, 1.28761E+01, 9.0233E+00, 2.0366E+04, &
      1.0355294E-01, 1.28797E+01, 9.1013E+00, 2.0440E+04, &
      1.0407070E-01, 1.28842E+01, 9.1796E+00, 2.0513E+04, &
      1.0459106E-01, 1.28896E+01, 9.2583E+00, 2.0586E+04, &
      1.0511401E-01, 1.28960E+01, 9.3373E+00, 2.0658E+04, &
      1.0563958E-01, 1.29034E+01, 9.4166E+00, 2.0730E+04, &
      1.0616778E-01, 1.29116E+01, 9.4962E+00, 2.0802E+04, &
      1.0669862E-01, 1.29209E+01, 9.5762E+00, 2.0872E+04, &
      1.0723211E-01, 1.29310E+01, 9.6566E+00, 2.0943E+04, &
      1.0776827E-01, 1.29422E+01, 9.7375E+00, 2.1013E+04, &
      1.0830712E-01, 1.29542E+01, 9.8187E+00, 2.1083E+04, &
      1.0884865E-01, 1.29673E+01, 9.9001E+00, 2.1152E+04, &
      1.0939289E-01, 1.29802E+01, 9.9818E+00, 2.1221E+04, &
      1.0993986E-01, 1.29951E+01, 1.0064E+01, 2.1288E+04, &
      1.1048956E-01, 1.30110E+01, 1.0146E+01, 2.1355E+04, &
      1.1104201E-01, 1.30279E+01, 1.0228E+01, 2.1421E+04, &
      1.1159722E-01, 1.30457E+01, 1.0310E+01, 2.1486E+04, &
      1.1215520E-01, 1.30645E+01, 1.0393E+01, 2.1551E+04, &
      1.1271598E-01, 1.30842E+01, 1.0476E+01, 2.1614E+04, &
      1.1327956E-01, 1.31049E+01, 1.0559E+01, 2.1677E+04, &
      1.1384596E-01, 1.31265E+01, 1.0642E+01, 2.1738E+04, &
      1.1441519E-01, 1.31490E+01, 1.0725E+01, 2.1799E+04, &
      1.1498726E-01, 1.31725E+01, 1.0808E+01, 2.1859E+04, &
      1.1556220E-01, 1.31969E+01, 1.0891E+01, 2.1918E+04, &
      1.1614001E-01, 1.32223E+01, 1.0974E+01, 2.1975E+04, &
      1.1672071E-01, 1.32486E+01, 1.1058E+01, 2.2032E+04, &
      1.1730431E-01, 1.32758E+01, 1.1141E+01, 2.2088E+04, &
      1.1789083E-01, 1.33039E+01, 1.1225E+01, 2.2143E+04, &
      1.1848029E-01, 1.33330E+01, 1.1308E+01, 2.2196E+04, &
      1.1907269E-01, 1.33630E+01, 1.1391E+01, 2.2249E+04, &
      1.1966805E-01, 1.33939E+01, 1.1475E+01, 2.2300E+04, &
      1.2026639E-01, 1.34257E+01, 1.1558E+01, 2.2351E+04, &
      1.2086772E-01, 1.34584E+01, 1.1642E+01, 2.2400E+04, &
      1.2147206E-01, 1.34920E+01, 1.1725E+01, 2.2448E+04, &
      1.2207942E-01, 1.35265E+01, 1.1808E+01, 2.2495E+04, &
      1.2268982E-01, 1.35620E+01, 1.1891E+01, 2.2541E+04, &
      1.2330327E-01, 1.35983E+01, 1.1975E+01, 2.2585E+04, &
      1.2391979E-01, 1.36354E+01, 1.2058E+01, 2.2629E+04, &
      1.2453939E-01, 1.36735E+01, 1.2141E+01, 2.2671E+04, &
      1.2516208E-01, 1.37124E+01, 1.2223E+01, 2.2712E+04, &
      1.2578789E-01, 1.37522E+01, 1.2306E+01, 2.2752E+04, &
      1.2641683E-01, 1.37929E+01, 1.2388E+01, 2.2791E+04, &
      1.2704892E-01, 1.38344E+01, 1.2471E+01, 2.2828E+04, &
      1.2768416E-01, 1.38768E+01, 1.2553E+01, 2.2864E+04, &
      1.2832258E-01, 1.39200E+01, 1.2635E+01, 2.2899E+04, &
      1.2896419E-01, 1.39640E+01, 1.2717E+01, 2.2933E+04, &
      1.2960902E-01, 1.40089E+01, 1.2799E+01, 2.2965E+04, &
      1.3025706E-01, 1.40546E+01, 1.2880E+01, 2.2996E+04, &
      1.3090835E-01, 1.41012E+01, 1.2961E+01, 2.3026E+04, &
      1.3156289E-01, 1.41485E+01, 1.3042E+01, 2.3055E+04, &
      1.3222070E-01, 1.41967E+01, 1.3123E+01, 2.3082E+04, &
      1.3288181E-01, 1.42456E+01, 1.3203E+01, 2.3108E+04, &
      1.3354621E-01, 1.42954E+01, 1.3284E+01, 2.3133E+04, &
      1.3421395E-01, 1.43459E+01, 1.3364E+01, 2.3156E+04, &
      1.3488502E-01, 1.43974E+01, 1.3443E+01, 2.3179E+04, &
      1.3555944E-01, 1.44494E+01, 1.3523E+01, 2.3200E+04, &
      1.3623724E-01, 1.45023E+01, 1.3602E+01, 2.3219E+04, &
      1.3691842E-01, 1.45559E+01, 1.3681E+01, 2.3238E+04, &
      1.3760302E-01, 1.46102E+01, 1.3759E+01, 2.3255E+04, &
      1.3829103E-01, 1.46653E+01, 1.3837E+01, 2.3270E+04, &
      1.3898249E-01, 1.47211E+01, 1.3915E+01, 2.3285E+04, &
      1.3967740E-01, 1.47776E+01, 1.3993E+01, 2.3298E+04, &
      1.4037579E-01, 1.48348E+01, 1.4070E+01, 2.3310E+04, &
      1.4107766E-01, 1.48928E+01, 1.4147E+01, 2.3321E+04, &
      1.4178305E-01, 1.49514E+01, 1.4223E+01, 2.3330E+04, &
      1.4249197E-01, 1.50108E+01, 1.4299E+01, 2.3338E+04, &
      1.4320443E-01, 1.50708E+01, 1.4375E+01, 2.3345E+04, &
      1.4392045E-01, 1.51314E+01, 1.4450E+01, 2.3350E+04, &
      1.4464005E-01, 1.51928E+01, 1.4525E+01, 2.3355E+04, &
      1.4536325E-01, 1.52548E+01, 1.4600E+01, 2.3358E+04, &
      1.4609007E-01, 1.53174E+01, 1.4674E+01, 2.3359E+04, &
      1.4682052E-01, 1.53807E+01, 1.4747E+01, 2.3360E+04, &
      1.4755462E-01, 1.54445E+01, 1.4821E+01, 2.3359E+04, &
      1.4829239E-01, 1.55090E+01, 1.4893E+01, 2.3357E+04, &
      1.4903386E-01, 1.55741E+01, 1.4966E+01, 2.3354E+04, &
      1.4977903E-01, 1.56398E+01, 1.5038E+01, 2.3349E+04, &
      1.5052792E-01, 1.57061E+01, 1.5109E+01, 2.3344E+04, &
      1.5128056E-01, 1.57729E+01, 1.5180E+01, 2.3337E+04, &
      1.5203696E-01, 1.58404E+01, 1.5251E+01, 2.3329E+04, &
      1.5279715E-01, 1.59083E+01, 1.5321E+01, 2.3319E+04, &
      1.5356113E-01, 1.59768E+01, 1.5391E+01, 2.3309E+04, &
      1.5432894E-01, 1.60459E+01, 1.5460E+01, 2.3297E+04, &
      1.5510058E-01, 1.61154E+01, 1.5529E+01, 2.3284E+04, &
      1.5587609E-01, 1.61855E+01, 1.5597E+01, 2.3270E+04, &
      1.5665547E-01, 1.62561E+01, 1.5664E+01, 2.3255E+04, &
      1.5743875E-01, 1.63272E+01, 1.5732E+01, 2.3239E+04, &
      1.5822594E-01, 1.63987E+01, 1.5798E+01, 2.3221E+04, &
      1.5901707E-01, 1.64707E+01, 1.5865E+01, 2.3203E+04, &
      1.5981215E-01, 1.65432E+01, 1.5930E+01, 2.3183E+04, &
      1.6061121E-01, 1.66161E+01, 1.5996E+01, 2.3162E+04, &
      1.6141427E-01, 1.66895E+01, 1.6060E+01, 2.3140E+04, &
      1.6222134E-01, 1.67632E+01, 1.6125E+01, 2.3117E+04, &
      1.6303245E-01, 1.68374E+01, 1.6188E+01, 2.3093E+04, &
      1.6384761E-01, 1.69120E+01, 1.6251E+01, 2.3068E+04, &
      1.6466685E-01, 1.69870E+01, 1.6314E+01, 2.3041E+04, &
      1.6549018E-01, 1.70623E+01, 1.6376E+01, 2.3014E+04, &
      1.6631763E-01, 1.71380E+01, 1.6438E+01, 2.2986E+04, &
      1.6714922E-01, 1.72141E+01, 1.6499E+01, 2.2956E+04, &
      1.6798497E-01, 1.72905E+01, 1.6559E+01, 2.2926E+04, &
      1.6882489E-01, 1.73672E+01, 1.6619E+01, 2.2894E+04, &
      1.6966902E-01, 1.74443E+01, 1.6679E+01, 2.2862E+04, &
      1.7051736E-01, 1.75216E+01, 1.6737E+01, 2.2828E+04, &
      1.7136995E-01, 1.75992E+01, 1.6796E+01, 2.2794E+04, &
      1.7222680E-01, 1.76771E+01, 1.6854E+01, 2.2759E+04, &
      1.7308793E-01, 1.77552E+01, 1.6911E+01, 2.2722E+04, &
      1.7395337E-01, 1.78336E+01, 1.6967E+01, 2.2685E+04, &
      1.7482314E-01, 1.79123E+01, 1.7024E+01, 2.2647E+04, &
      1.7569726E-01, 1.79911E+01, 1.7079E+01, 2.2608E+04, &
      1.7657574E-01, 1.80701E+01, 1.7134E+01, 2.2568E+04, &
      1.7745862E-01, 1.81494E+01, 1.7189E+01, 2.2527E+04, &
      1.7834591E-01, 1.82288E+01, 1.7243E+01, 2.2485E+04, &
      1.7923764E-01, 1.83083E+01, 1.7296E+01, 2.2442E+04, &
      1.8013383E-01, 1.83880E+01, 1.7349E+01, 2.2399E+04, &
      1.8103450E-01, 1.84678E+01, 1.7401E+01, 2.2355E+04, &
      1.8193967E-01, 1.85477E+01, 1.7453E+01, 2.2309E+04, &
      1.8284937E-01, 1.86277E+01, 1.7504E+01, 2.2264E+04, &
      1.8376362E-01, 1.87078E+01, 1.7554E+01, 2.2217E+04, &
      1.8468244E-01, 1.87879E+01, 1.7604E+01, 2.2169E+04, &
      1.8560585E-01, 1.88680E+01, 1.7654E+01, 2.2121E+04, &
      1.8653388E-01, 1.89482E+01, 1.7703E+01, 2.2072E+04, &
      1.8746655E-01, 1.90283E+01, 1.7751E+01, 2.2022E+04, &
      1.8840388E-01, 1.91084E+01, 1.7799E+01, 2.1972E+04, &
      1.8934590E-01, 1.91884E+01, 1.7846E+01, 2.1920E+04, &
      1.9029263E-01, 1.92683E+01, 1.7893E+01, 2.1868E+04, &
      1.9124409E-01, 1.93481E+01, 1.7939E+01, 2.1816E+04, &
      1.9220031E-01, 1.94278E+01, 1.7984E+01, 2.1762E+04, &
      1.9316131E-01, 1.95073E+01, 1.8030E+01, 2.1708E+04, &
      1.9412712E-01, 1.95866E+01, 1.8074E+01, 2.1654E+04, &
      1.9509776E-01, 1.96657E+01, 1.8118E+01, 2.1598E+04, &
      1.9607325E-01, 1.97444E+01, 1.8161E+01, 2.1542E+04, &
      1.9705361E-01, 1.98229E+01, 1.8204E+01, 2.1486E+04, &
      1.9803888E-01, 1.99010E+01, 1.8247E+01, 2.1429E+04, &
      1.9902907E-01, 1.99787E+01, 1.8288E+01, 2.1371E+04, &
      2.0002422E-01, 2.00559E+01, 1.8330E+01, 2.1312E+04, &
      2.0102434E-01, 2.01327E+01, 1.8370E+01, 2.1254E+04, &
      2.0202946E-01, 2.02088E+01, 1.8410E+01, 2.1194E+04, &
      2.0303961E-01, 2.02844E+01, 1.8450E+01, 2.1134E+04, &
      2.0405481E-01, 2.03592E+01, 1.8489E+01, 2.1074E+04, &
      2.0507508E-01, 2.04332E+01, 1.8528E+01, 2.1013E+04, &
      2.0610046E-01, 2.05063E+01, 1.8566E+01, 2.0951E+04, &
      2.0713096E-01, 2.05785E+01, 1.8603E+01, 2.0889E+04, &
      2.0816661E-01, 2.06495E+01, 1.8640E+01, 2.0826E+04, &
      2.0920745E-01, 2.07192E+01, 1.8677E+01, 2.0763E+04, &
      2.1025348E-01, 2.07876E+01, 1.8713E+01, 2.0700E+04, &
      2.1130475E-01, 2.08543E+01, 1.8748E+01, 2.0636E+04, &
      2.1236128E-01, 2.09192E+01, 1.8783E+01, 2.0572E+04, &
      2.1342308E-01, 2.09841E+01, 1.8818E+01, 2.0507E+04, &
      2.1449020E-01, 2.10445E+01, 1.8852E+01, 2.0442E+04, &
      2.1556265E-01, 2.11022E+01, 1.8885E+01, 2.0376E+04, &
      2.1664046E-01, 2.11565E+01, 1.8918E+01, 2.0310E+04, &
      2.1772366E-01, 2.12070E+01, 1.8951E+01, 2.0244E+04, &
      2.1881228E-01, 2.12528E+01, 1.8983E+01, 2.0177E+04, &
      2.1990634E-01, 2.12929E+01, 1.9014E+01, 2.0110E+04, &
      2.2100588E-01, 2.13258E+01, 1.9045E+01, 2.0043E+04, &
      2.2211090E-01, 2.13495E+01, 1.9076E+01, 1.9975E+04, &
      2.2322146E-01, 2.13608E+01, 1.9106E+01, 1.9907E+04, &
      2.2433757E-01, 2.13548E+01, 1.9136E+01, 1.9839E+04, &
      2.2545925E-01, 2.13227E+01, 1.9165E+01, 1.9770E+04, &
      2.2658655E-01, 2.12460E+01, 1.9194E+01, 1.9702E+04, &
      2.2771948E-01, 2.10755E+01, 1.9222E+01, 1.9632E+04, &
      2.2885808E-01, 2.05488E+01, 1.9250E+01, 1.9563E+04, &
      2.2912551E-01, 2.01182E+01, 1.9256E+01, 1.9547E+04, &
      2.2947450E-01, 2.01515E+01, 2.0798E+01, 2.1080E+04, &
      2.3000237E-01, 2.08776E+01, 2.0801E+01, 2.1035E+04, &
      2.3115238E-01, 2.14396E+01, 2.0809E+01, 2.0937E+04, &
      2.3230814E-01, 2.17556E+01, 2.0816E+01, 2.0840E+04, &
      2.3346969E-01, 2.19858E+01, 2.0822E+01, 2.0743E+04, &
      2.3463703E-01, 2.21673E+01, 2.0829E+01, 2.0647E+04, &
      2.3581022E-01, 2.23129E+01, 2.0836E+01, 2.0550E+04, &
      2.3698927E-01, 2.24251E+01, 2.0842E+01, 2.0454E+04, &
      2.3817422E-01, 2.24973E+01, 2.0848E+01, 2.0359E+04, &
      2.3936509E-01, 2.25048E+01, 2.0854E+01, 2.0263E+04, &
      2.4056191E-01, 2.23329E+01, 2.0860E+01, 2.0168E+04, &
      2.4111348E-01, 2.19165E+01, 2.0862E+01, 2.0124E+04, &
      2.4148653E-01, 2.19736E+01, 2.1999E+01, 2.1188E+04, &
      2.4176472E-01, 2.23450E+01, 2.1997E+01, 2.1161E+04, &
      2.4297355E-01, 2.29759E+01, 2.1985E+01, 2.1045E+04, &
      2.4418841E-01, 2.33313E+01, 2.1974E+01, 2.0930E+04, &
      2.4540936E-01, 2.36106E+01, 2.1963E+01, 2.0815E+04, &
      2.4663640E-01, 2.38518E+01, 2.1951E+01, 2.0701E+04, &
      2.4786959E-01, 2.40694E+01, 2.1940E+01, 2.0587E+04, &
      2.4910893E-01, 2.42706E+01, 2.1929E+01, 2.0475E+04, &
      2.5035448E-01, 2.44594E+01, 2.1918E+01, 2.0363E+04, &
      2.5160625E-01, 2.46386E+01, 2.1907E+01, 2.0251E+04, &
      2.5286428E-01, 2.48097E+01, 2.1895E+01, 2.0139E+04, &
      2.5412860E-01, 2.49737E+01, 2.1875E+01, 2.0021E+04, &
      2.5539925E-01, 2.51317E+01, 2.1856E+01, 1.9904E+04, &
      2.5667624E-01, 2.52844E+01, 2.1837E+01, 1.9788E+04, &
      2.5795962E-01, 2.54326E+01, 2.1819E+01, 1.9673E+04, &
      2.5924942E-01, 2.55769E+01, 2.1801E+01, 1.9559E+04, &
      2.6054567E-01, 2.57176E+01, 2.1784E+01, 1.9447E+04, &
      2.6184840E-01, 2.58552E+01, 2.1767E+01, 1.9335E+04, &
      2.6315764E-01, 2.59882E+01, 2.1743E+01, 1.9218E+04, &
      2.6447343E-01, 2.61181E+01, 2.1719E+01, 1.9101E+04, &
      2.6579579E-01, 2.62449E+01, 2.1696E+01, 1.8985E+04, &
      2.6712477E-01, 2.63690E+01, 2.1673E+01, 1.8871E+04, &
      2.6846040E-01, 2.64904E+01, 2.1650E+01, 1.8758E+04, &
      2.6980270E-01, 2.66096E+01, 2.1629E+01, 1.8646E+04, &
      2.7115171E-01, 2.67268E+01, 2.1608E+01, 1.8535E+04, &
      2.7250747E-01, 2.68421E+01, 2.1587E+01, 1.8425E+04, &
      2.7387001E-01, 2.69557E+01, 2.1567E+01, 1.8317E+04, &
      2.7523936E-01, 2.70677E+01, 2.1548E+01, 1.8209E+04, &
      2.7661556E-01, 2.71783E+01, 2.1529E+01, 1.8103E+04, &
      2.7799863E-01, 2.72875E+01, 2.1510E+01, 1.7997E+04, &
      2.7938863E-01, 2.73954E+01, 2.1492E+01, 1.7893E+04, &
      2.8078557E-01, 2.75021E+01, 2.1474E+01, 1.7789E+04, &
      2.8218950E-01, 2.76077E+01, 2.1457E+01, 1.7686E+04, &
      2.8360044E-01, 2.77122E+01, 2.1439E+01, 1.7584E+04, &
      2.8501845E-01, 2.78157E+01, 2.1422E+01, 1.7482E+04, &
      2.8644354E-01, 2.79182E+01, 2.1406E+01, 1.7382E+04, &
      2.8787576E-01, 2.80198E+01, 2.1389E+01, 1.7282E+04, &
      2.8931514E-01, 2.81205E+01, 2.1373E+01, 1.7183E+04, &
      2.9076171E-01, 2.82203E+01, 2.1357E+01, 1.7085E+04, &
      2.9221552E-01, 2.83193E+01, 2.1341E+01, 1.6987E+04, &
      2.9367660E-01, 2.84174E+01, 2.1325E+01, 1.6890E+04, &
      2.9514498E-01, 2.85149E+01, 2.1309E+01, 1.6794E+04, &
      2.9662071E-01, 2.86115E+01, 2.1294E+01, 1.6698E+04, &
      2.9810381E-01, 2.87074E+01, 2.1278E+01, 1.6603E+04, &
      2.9959433E-01, 2.88026E+01, 2.1263E+01, 1.6509E+04, &
      3.0109230E-01, 2.88971E+01, 2.1248E+01, 1.6415E+04, &
      3.0259776E-01, 2.89909E+01, 2.1233E+01, 1.6322E+04, &
      3.0411075E-01, 2.90841E+01, 2.1218E+01, 1.6229E+04, &
      3.0563130E-01, 2.91766E+01, 2.1203E+01, 1.6137E+04, &
      3.0715946E-01, 2.92684E+01, 2.1188E+01, 1.6045E+04, &
      3.0869526E-01, 2.93596E+01, 2.1173E+01, 1.5954E+04, &
      3.1023873E-01, 2.94502E+01, 2.1159E+01, 1.5864E+04, &
      3.1178993E-01, 2.95401E+01, 2.1144E+01, 1.5774E+04, &
      3.1334888E-01, 2.96295E+01, 2.1129E+01, 1.5685E+04, &
      3.1491562E-01, 2.97182E+01, 2.1115E+01, 1.5596E+04, &
      3.1649020E-01, 2.98063E+01, 2.1100E+01, 1.5508E+04, &
      3.1807265E-01, 2.98938E+01, 2.1086E+01, 1.5420E+04, &
      3.1966301E-01, 2.99807E+01, 2.1071E+01, 1.5333E+04, &
      3.2126133E-01, 3.00670E+01, 2.1057E+01, 1.5246E+04, &
      3.2286764E-01, 3.01527E+01, 2.1043E+01, 1.5160E+04, &
      3.2448197E-01, 3.02378E+01, 2.1028E+01, 1.5075E+04, &
      3.2610438E-01, 3.03224E+01, 2.1014E+01, 1.4990E+04, &
      3.2773491E-01, 3.04063E+01, 2.1000E+01, 1.4905E+04, &
      3.2937358E-01, 3.04896E+01, 2.0986E+01, 1.4821E+04, &
      3.3102045E-01, 3.05722E+01, 2.0971E+01, 1.4737E+04, &
      3.3267555E-01, 3.06543E+01, 2.0957E+01, 1.4654E+04, &
      3.3433893E-01, 3.07358E+01, 2.0943E+01, 1.4571E+04, &
      3.3601062E-01, 3.08166E+01, 2.0929E+01, 1.4489E+04, &
      3.3769068E-01, 3.08968E+01, 2.0915E+01, 1.4407E+04, &
      3.3937913E-01, 3.09763E+01, 2.0901E+01, 1.4326E+04, &
      3.4107602E-01, 3.10552E+01, 2.0887E+01, 1.4245E+04, &
      3.4278140E-01, 3.11333E+01, 2.0873E+01, 1.4165E+04, &
      3.4449531E-01, 3.12108E+01, 2.0859E+01, 1.4085E+04, &
      3.4621779E-01, 3.12876E+01, 2.0845E+01, 1.4006E+04, &
      3.4794888E-01, 3.13637E+01, 2.0832E+01, 1.3927E+04, &
      3.4968862E-01, 3.14390E+01, 2.0818E+01, 1.3849E+04, &
      3.5143706E-01, 3.15135E+01, 2.0804E+01, 1.3771E+04, &
      3.5319425E-01, 3.15871E+01, 2.0791E+01, 1.3693E+04, &
      3.5496022E-01, 3.16600E+01, 2.0777E+01, 1.3616E+04, &
      3.5673502E-01, 3.17319E+01, 2.0764E+01, 1.3540E+04, &
      3.5851870E-01, 3.18029E+01, 2.0750E+01, 1.3464E+04, &
      3.6031129E-01, 3.18729E+01, 2.0737E+01, 1.3388E+04, &
      3.6211285E-01, 3.19419E+01, 2.0723E+01, 1.3313E+04, &
      3.6392341E-01, 3.20097E+01, 2.0710E+01, 1.3238E+04, &
      3.6574303E-01, 3.20764E+01, 2.0697E+01, 1.3164E+04, &
      3.6757174E-01, 3.21417E+01, 2.0684E+01, 1.3090E+04, &
      3.6940960E-01, 3.22056E+01, 2.0670E+01, 1.3017E+04, &
      3.7125665E-01, 3.22680E+01, 2.0657E+01, 1.2944E+04, &
      3.7311293E-01, 3.23287E+01, 2.0644E+01, 1.2872E+04, &
      3.7497850E-01, 3.23875E+01, 2.0631E+01, 1.2800E+04, &
      3.7685339E-01, 3.24442E+01, 2.0619E+01, 1.2728E+04, &
      3.7873766E-01, 3.24985E+01, 2.0606E+01, 1.2657E+04, &
      3.8063135E-01, 3.25500E+01, 2.0593E+01, 1.2586E+04, &
      3.8253450E-01, 3.25984E+01, 2.0580E+01, 1.2516E+04, &
      3.8444718E-01, 3.26431E+01, 2.0568E+01, 1.2446E+04, &
      3.8636941E-01, 3.26833E+01, 2.0555E+01, 1.2377E+04, &
      3.8830126E-01, 3.27181E+01, 2.0543E+01, 1.2308E+04, &
      3.9024276E-01, 3.27460E+01, 2.0531E+01, 1.2239E+04, &
      3.9219398E-01, 3.27651E+01, 2.0518E+01, 1.2171E+04, &
      3.9415495E-01, 3.27724E+01, 2.0506E+01, 1.2104E+04, &
      3.9612572E-01, 3.27627E+01, 2.0494E+01, 1.2036E+04, &
      3.9810635E-01, 3.27270E+01, 2.0482E+01, 1.1969E+04, &
      4.0009688E-01, 3.26455E+01, 2.0470E+01, 1.1903E+04, &
      4.0209737E-01, 3.24609E+01, 2.0458E+01, 1.1837E+04, &
      4.0397820E-01, 3.18647E+01, 2.0447E+01, 1.1775E+04, &
      4.0410785E-01, 3.17442E+01, 2.0446E+01, 1.1771E+04, &
      4.0502181E-01, 3.19057E+01, 2.1823E+01, 1.2535E+04, &
      4.0612839E-01, 3.24501E+01, 2.1819E+01, 1.2499E+04, &
      4.0815904E-01, 3.28876E+01, 2.1811E+01, 1.2432E+04, &
      4.1019983E-01, 3.31646E+01, 2.1803E+01, 1.2366E+04, &
      4.1225083E-01, 3.33823E+01, 2.1795E+01, 1.2300E+04, &
      4.1431208E-01, 3.35687E+01, 2.1788E+01, 1.2235E+04, &
      4.1638364E-01, 3.37359E+01, 2.1780E+01, 1.2169E+04, &
      4.1846556E-01, 3.38900E+01, 2.1772E+01, 1.2105E+04, &
      4.2055789E-01, 3.40346E+01, 2.1765E+01, 1.2040E+04, &
      4.2266068E-01, 3.41720E+01, 2.1757E+01, 1.1976E+04, &
      4.2477398E-01, 3.43037E+01, 2.1750E+01, 1.1913E+04, &
      4.2689785E-01, 3.44309E+01, 2.1743E+01, 1.1850E+04, &
      4.2903234E-01, 3.45542E+01, 2.1736E+01, 1.1787E+04, &
      4.3117750E-01, 3.46744E+01, 2.1729E+01, 1.1724E+04, &
      4.3333339E-01, 3.47917E+01, 2.1721E+01, 1.1662E+04, &
      4.3550006E-01, 3.49065E+01, 2.1714E+01, 1.1601E+04, &
      4.3767756E-01, 3.50190E+01, 2.1707E+01, 1.1539E+04, &
      4.3986595E-01, 3.51294E+01, 2.1701E+01, 1.1478E+04, &
      4.4206528E-01, 3.52378E+01, 2.1694E+01, 1.1418E+04, &
      4.4427560E-01, 3.53441E+01, 2.1687E+01, 1.1357E+04, &
      4.4649698E-01, 3.54482E+01, 2.1680E+01, 1.1297E+04, &
      4.4872947E-01, 3.55500E+01, 2.1673E+01, 1.1238E+04, &
      4.5097311E-01, 3.56489E+01, 2.1667E+01, 1.1178E+04, &
      4.5322798E-01, 3.57443E+01, 2.1660E+01, 1.1119E+04, &
      4.5549412E-01, 3.58349E+01, 2.1653E+01, 1.1061E+04, &
      4.5777159E-01, 3.59182E+01, 2.1647E+01, 1.1002E+04, &
      4.6006045E-01, 3.59887E+01, 2.1640E+01, 1.0944E+04, &
      4.6236075E-01, 3.60289E+01, 2.1633E+01, 1.0887E+04, &
      4.6412139E-01, 3.59738E+01, 2.1628E+01, 1.0843E+04, &
      4.6467255E-01, 3.58159E+01, 2.1627E+01, 1.0829E+04, &
      4.6547861E-01, 3.60396E+01, 2.1972E+01, 1.0983E+04, &
      4.6699592E-01, 3.62432E+01, 2.1969E+01, 1.0946E+04, &
      4.6933090E-01, 3.64370E+01, 2.1964E+01, 1.0889E+04, &
      4.7167755E-01, 3.65981E+01, 2.1915E+01, 1.0811E+04, &
      4.7403594E-01, 3.67442E+01, 2.1864E+01, 1.0732E+04, &
      4.7640612E-01, 3.68813E+01, 2.1813E+01, 1.0654E+04, &
      4.7878815E-01, 3.70123E+01, 2.1762E+01, 1.0576E+04, &
      4.8118209E-01, 3.71385E+01, 2.1712E+01, 1.0499E+04, &
      4.8358800E-01, 3.72609E+01, 2.1661E+01, 1.0422E+04, &
      4.8600594E-01, 3.73800E+01, 2.1610E+01, 1.0346E+04, &
      4.8843597E-01, 3.74963E+01, 2.1559E+01, 1.0271E+04, &
      4.9087815E-01, 3.76100E+01, 2.1509E+01, 1.0196E+04, &
      4.9333254E-01, 3.77214E+01, 2.1458E+01, 1.0121E+04, &
      4.9579920E-01, 3.78307E+01, 2.1407E+01, 1.0047E+04, &
      4.9827820E-01, 3.79379E+01, 2.1357E+01, 9.9735E+03, &
      5.0076959E-01, 3.80432E+01, 2.1306E+01, 9.9005E+03, &
      5.0327344E-01, 3.81467E+01, 2.1256E+01, 9.8279E+03, &
      5.0578980E-01, 3.82484E+01, 2.1205E+01, 9.7559E+03, &
      5.0831875E-01, 3.83484E+01, 2.1154E+01, 9.6843E+03, &
      5.1086035E-01, 3.84467E+01, 2.1104E+01, 9.6132E+03, &
      5.1341465E-01, 3.85433E+01, 2.1054E+01, 9.5425E+03, &
      5.1598172E-01, 3.86383E+01, 2.1003E+01, 9.4724E+03, &
      5.1856163E-01, 3.87317E+01, 2.0953E+01, 9.4027E+03, &
      5.2115444E-01, 3.88234E+01, 2.0902E+01, 9.3335E+03, &
      5.2376021E-01, 3.89134E+01, 2.0852E+01, 9.2647E+03, &
      5.2637901E-01, 3.90017E+01, 2.0801E+01, 9.1964E+03, &
      5.2901091E-01, 3.90882E+01, 2.0751E+01, 9.1286E+03, &
      5.3165596E-01, 3.91728E+01, 2.0701E+01, 9.0611E+03, &
      5.3431424E-01, 3.92553E+01, 2.0650E+01, 8.9942E+03, &
      5.3698581E-01, 3.93356E+01, 2.0600E+01, 8.9277E+03, &
      5.3967074E-01, 3.94135E+01, 2.0549E+01, 8.8616E+03, &
      5.4236910E-01, 3.94885E+01, 2.0499E+01, 8.7959E+03, &
      5.4508094E-01, 3.95603E+01, 2.0448E+01, 8.7307E+03, &
      5.4780635E-01, 3.96281E+01, 2.0398E+01, 8.6659E+03, &
      5.5054538E-01, 3.96910E+01, 2.0347E+01, 8.6015E+03, &
      5.5329810E-01, 3.97470E+01, 2.0297E+01, 8.5375E+03, &
      5.5606460E-01, 3.97932E+01, 2.0246E+01, 8.4740E+03, &
      5.5884492E-01, 3.98231E+01, 2.0196E+01, 8.4108E+03, &
      5.6163914E-01, 3.98191E+01, 2.0145E+01, 8.3481E+03, &
      5.6444734E-01, 3.96866E+01, 2.0095E+01, 8.2858E+03, &
      5.6457826E-01, 3.96682E+01, 2.0092E+01, 8.2829E+03, &
      5.6642179E-01, 3.97246E+01, 2.0590E+01, 8.4603E+03, &
      5.6726958E-01, 3.98605E+01, 2.0575E+01, 8.4416E+03, &
      5.7010592E-01, 4.01080E+01, 2.0525E+01, 8.3793E+03, &
      5.7295645E-01, 4.02747E+01, 2.0475E+01, 8.3174E+03, &
      5.7582123E-01, 4.04145E+01, 2.0425E+01, 8.2559E+03, &
      5.7870034E-01, 4.05403E+01, 2.0375E+01, 8.1948E+03, &
      5.8159384E-01, 4.06575E+01, 2.0325E+01, 8.1341E+03, &
      5.8450181E-01, 4.07687E+01, 2.0275E+01, 8.0737E+03, &
      5.8742432E-01, 4.08755E+01, 2.0225E+01, 8.0138E+03, &
      5.9036144E-01, 4.09788E+01, 2.0175E+01, 7.9542E+03, &
      5.9331325E-01, 4.10792E+01, 2.0125E+01, 7.8950E+03, &
      5.9627982E-01, 4.11773E+01, 2.0074E+01, 7.8362E+03, &
      5.9926122E-01, 4.12734E+01, 2.0024E+01, 7.7777E+03, &
      6.0225752E-01, 4.13676E+01, 1.9973E+01, 7.7196E+03, &
      6.0526881E-01, 4.14602E+01, 1.9923E+01, 7.6618E+03, &
      6.0829515E-01, 4.15515E+01, 1.9872E+01, 7.6044E+03, &
      6.1133663E-01, 4.16414E+01, 1.9822E+01, 7.5474E+03, &
      6.1439331E-01, 4.17300E+01, 1.9771E+01, 7.4907E+03, &
      6.1746528E-01, 4.18176E+01, 1.9720E+01, 7.4344E+03, &
      6.2055260E-01, 4.19041E+01, 1.9669E+01, 7.3784E+03, &
      6.2365537E-01, 4.19897E+01, 1.9619E+01, 7.3228E+03, &
      6.2677364E-01, 4.20743E+01, 1.9568E+01, 7.2675E+03, &
      6.2990751E-01, 4.21580E+01, 1.9516E+01, 7.2125E+03, &
      6.3305705E-01, 4.22409E+01, 1.9465E+01, 7.1579E+03, &
      6.3622234E-01, 4.23229E+01, 1.9414E+01, 7.1036E+03, &
      6.3940345E-01, 4.24041E+01, 1.9363E+01, 7.0496E+03, &
      6.4260046E-01, 4.24846E+01, 1.9311E+01, 6.9960E+03, &
      6.4581347E-01, 4.25643E+01, 1.9259E+01, 6.9427E+03, &
      6.4904253E-01, 4.26433E+01, 1.9208E+01, 6.8897E+03, &
      6.5228775E-01, 4.27216E+01, 1.9156E+01, 6.8370E+03, &
      6.5554919E-01, 4.27991E+01, 1.9104E+01, 6.7846E+03, &
      6.5882693E-01, 4.28760E+01, 1.9052E+01, 6.7326E+03, &
      6.6212107E-01, 4.29523E+01, 1.9000E+01, 6.6808E+03, &
      6.6543167E-01, 4.30278E+01, 1.8948E+01, 6.6294E+03, &
      6.6875883E-01, 4.31027E+01, 1.8895E+01, 6.5783E+03, &
      6.7210262E-01, 4.31770E+01, 1.8843E+01, 6.5275E+03, &
      6.7546314E-01, 4.32507E+01, 1.8790E+01, 6.4770E+03, &
      6.7884045E-01, 4.33237E+01, 1.8738E+01, 6.4268E+03, &
      6.8223466E-01, 4.33961E+01, 1.8685E+01, 6.3769E+03, &
      6.8564583E-01, 4.34679E+01, 1.8632E+01, 6.3273E+03, &
      6.8907406E-01, 4.35370E+01, 1.8579E+01, 6.2779E+03, &
      6.9251943E-01, 4.36076E+01, 1.8526E+01, 6.2289E+03, &
      6.9598202E-01, 4.36776E+01, 1.8472E+01, 6.1802E+03, &
      6.9946194E-01, 4.37471E+01, 1.8419E+01, 6.1317E+03, &
      7.0295924E-01, 4.38159E+01, 1.8365E+01, 6.0836E+03, &
      7.0647404E-01, 4.38842E+01, 1.8312E+01, 6.0357E+03, &
      7.1000641E-01, 4.39519E+01, 1.8258E+01, 5.9881E+03, &
      7.1355644E-01, 4.40190E+01, 1.8204E+01, 5.9409E+03, &
      7.1712423E-01, 4.40855E+01, 1.8150E+01, 5.8939E+03, &
      7.2070985E-01, 4.41515E+01, 1.8096E+01, 5.8472E+03, &
      7.2431340E-01, 4.42237E+01, 1.8042E+01, 5.8007E+03, &
      7.2793496E-01, 4.42886E+01, 1.7988E+01, 5.7546E+03, &
      7.3157464E-01, 4.43529E+01, 1.7933E+01, 5.7088E+03, &
      7.3523251E-01, 4.44167E+01, 1.7879E+01, 5.6632E+03, &
      7.3890867E-01, 4.44799E+01, 1.7824E+01, 5.6179E+03, &
      7.4260322E-01, 4.45426E+01, 1.7770E+01, 5.5729E+03, &
      7.4631623E-01, 4.46047E+01, 1.7715E+01, 5.5282E+03, &
      7.5004781E-01, 4.46662E+01, 1.7660E+01, 5.4838E+03, &
      7.5379805E-01, 4.47272E+01, 1.7605E+01, 5.4396E+03, &
      7.5756704E-01, 4.47876E+01, 1.7550E+01, 5.3957E+03, &
      7.6135488E-01, 4.48475E+01, 1.7495E+01, 5.3521E+03, &
      7.6516165E-01, 4.49069E+01, 1.7440E+01, 5.3088E+03, &
      7.6898746E-01, 4.49657E+01, 1.7385E+01, 5.2657E+03, &
      7.7283240E-01, 4.50239E+01, 1.7329E+01, 5.2230E+03, &
      7.7669656E-01, 4.50817E+01, 1.7274E+01, 5.1804E+03, &
      7.8058004E-01, 4.51389E+01, 1.7219E+01, 5.1382E+03, &
      7.8448294E-01, 4.51956E+01, 1.7163E+01, 5.0962E+03, &
      7.8840536E-01, 4.52517E+01, 1.7107E+01, 5.0544E+03, &
      7.9234738E-01, 4.53072E+01, 1.7051E+01, 5.0129E+03, &
      7.9630912E-01, 4.53622E+01, 1.6995E+01, 4.9716E+03, &
      8.0029067E-01, 4.54165E+01, 1.6938E+01, 4.9305E+03, &
      8.0429212E-01, 4.54703E+01, 1.6882E+01, 4.8897E+03, &
      8.0831358E-01, 4.55234E+01, 1.6825E+01, 4.8492E+03, &
      8.1235515E-01, 4.55760E+01, 1.6769E+01, 4.8090E+03, &
      8.1641693E-01, 4.56280E+01, 1.6712E+01, 4.7690E+03, &
      8.2049901E-01, 4.56794E+01, 1.6656E+01, 4.7293E+03, &
      8.2460150E-01, 4.57303E+01, 1.6599E+01, 4.6898E+03, &
      8.2872451E-01, 4.57806E+01, 1.6542E+01, 4.6506E+03, &
      8.3286813E-01, 4.58303E+01, 1.6485E+01, 4.6116E+03, &
      8.3703248E-01, 4.58794E+01, 1.6428E+01, 4.5730E+03, &
      8.4121764E-01, 4.59280E+01, 1.6371E+01, 4.5345E+03, &
      8.4542373E-01, 4.59760E+01, 1.6314E+01, 4.4963E+03, &
      8.4965084E-01, 4.60234E+01, 1.6257E+01, 4.4584E+03, &
      8.5389910E-01, 4.60702E+01, 1.6200E+01, 4.4208E+03, &
      8.5816859E-01, 4.61166E+01, 1.6143E+01, 4.3833E+03, &
      8.6245944E-01, 4.61623E+01, 1.6086E+01, 4.3462E+03, &
      8.6677173E-01, 4.62075E+01, 1.6029E+01, 4.3093E+03, &
      8.7110559E-01, 4.62522E+01, 1.5971E+01, 4.2726E+03, &
      8.7546112E-01, 4.62963E+01, 1.5914E+01, 4.2362E+03, &
      8.7983843E-01, 4.63398E+01, 1.5857E+01, 4.2001E+03, &
      8.8423762E-01, 4.63829E+01, 1.5799E+01, 4.1642E+03, &
      8.8865881E-01, 4.64254E+01, 1.5742E+01, 4.1285E+03, &
      8.9310210E-01, 4.64674E+01, 1.5685E+01, 4.0931E+03, &
      8.9756761E-01, 4.65089E+01, 1.5627E+01, 4.0580E+03, &
      9.0205545E-01, 4.65499E+01, 1.5570E+01, 4.0230E+03, &
      9.0656573E-01, 4.65903E+01, 1.5513E+01, 3.9884E+03, &
      9.1109856E-01, 4.66304E+01, 1.5455E+01, 3.9540E+03, &
      9.1565405E-01, 4.66699E+01, 1.5398E+01, 3.9198E+03, &
      9.2023232E-01, 4.67090E+01, 1.5341E+01, 3.8859E+03, &
      9.2483348E-01, 4.67477E+01, 1.5283E+01, 3.8522E+03, &
      9.2945765E-01, 4.67859E+01, 1.5226E+01, 3.8187E+03, &
      9.3410494E-01, 4.68237E+01, 1.5169E+01, 3.7855E+03, &
      9.3877546E-01, 4.68612E+01, 1.5111E+01, 3.7525E+03, &
      9.4346934E-01, 4.68983E+01, 1.5054E+01, 3.7198E+03, &
      9.4818668E-01, 4.69350E+01, 1.4996E+01, 3.6872E+03, &
      9.5292762E-01, 4.69715E+01, 1.4939E+01, 3.6550E+03, &
      9.5769226E-01, 4.70078E+01, 1.4882E+01, 3.6229E+03, &
      9.6248072E-01, 4.70438E+01, 1.4824E+01, 3.5911E+03, &
      9.6729312E-01, 4.70796E+01, 1.4767E+01, 3.5595E+03, &
      9.7212959E-01, 4.71154E+01, 1.4709E+01, 3.5281E+03, &
      9.7699023E-01, 4.71512E+01, 1.4652E+01, 3.4970E+03, &
      9.8187519E-01, 4.71871E+01, 1.4595E+01, 3.4661E+03, &
      9.8678456E-01, 4.72233E+01, 1.4538E+01, 3.4355E+03, &
      9.9171848E-01, 4.72599E+01, 1.4481E+01, 3.4052E+03, &
      9.9667708E-01, 4.72974E+01, 1.4424E+01, 3.3750E+03, &
      1.0016605E+00, 4.73377E+01, 1.4355E+01, 3.3422E+03, &
      1.0066688E+00, 4.73679E+01, 1.4260E+01, 3.3037E+03, &
      1.0117021E+00, 4.73922E+01, 1.4166E+01, 3.2657E+03, &
      1.0167606E+00, 4.74135E+01, 1.4073E+01, 3.2282E+03, &
      1.0218444E+00, 4.74322E+01, 1.3980E+01, 3.1911E+03, &
      1.0269536E+00, 4.74487E+01, 1.3888E+01, 3.1545E+03, &
      1.0320884E+00, 4.74631E+01, 1.3798E+01, 3.1184E+03, &
      1.0372489E+00, 4.74754E+01, 1.3707E+01, 3.0828E+03, &
      1.0424351E+00, 4.74857E+01, 1.3618E+01, 3.0476E+03, &
      1.0476473E+00, 4.74940E+01, 1.3530E+01, 3.0128E+03, &
      1.0528855E+00, 4.75005E+01, 1.3442E+01, 2.9785E+03, &
      1.0581499E+00, 4.75051E+01, 1.3355E+01, 2.9446E+03, &
      1.0634407E+00, 4.75079E+01, 1.3268E+01, 2.9112E+03, &
      1.0687579E+00, 4.75088E+01, 1.3183E+01, 2.8781E+03, &
      1.0741017E+00, 4.75080E+01, 1.3098E+01, 2.8455E+03, &
      1.0794722E+00, 4.75055E+01, 1.3014E+01, 2.8133E+03, &
      1.0848695E+00, 4.75012E+01, 1.2930E+01, 2.7815E+03, &
      1.0902939E+00, 4.74952E+01, 1.2848E+01, 2.7501E+03, &
      1.0957454E+00, 4.74875E+01, 1.2766E+01, 2.7191E+03, &
      1.1012241E+00, 4.74781E+01, 1.2685E+01, 2.6884E+03, &
      1.1067302E+00, 4.74670E+01, 1.2604E+01, 2.6582E+03, &
      1.1122639E+00, 4.74543E+01, 1.2524E+01, 2.6283E+03, &
      1.1178252E+00, 4.74400E+01, 1.2445E+01, 2.5988E+03, &
      1.1234143E+00, 4.74239E+01, 1.2366E+01, 2.5697E+03, &
      1.1290314E+00, 4.74063E+01, 1.2288E+01, 2.5409E+03, &
      1.1346765E+00, 4.73870E+01, 1.2211E+01, 2.5125E+03, &
      1.1403499E+00, 4.73660E+01, 1.2134E+01, 2.4844E+03, &
      1.1460517E+00, 4.73434E+01, 1.2058E+01, 2.4567E+03, &
      1.1517819E+00, 4.73191E+01, 1.1983E+01, 2.4293E+03, &
      1.1575408E+00, 4.72932E+01, 1.1908E+01, 2.4023E+03, &
      1.1633285E+00, 4.72657E+01, 1.1834E+01, 2.3756E+03, &
      1.1691452E+00, 4.72364E+01, 1.1761E+01, 2.3492E+03, &
      1.1749909E+00, 4.72055E+01, 1.1688E+01, 2.3232E+03, &
      1.1808659E+00, 4.71729E+01, 1.1616E+01, 2.2974E+03, &
      1.1867702E+00, 4.71386E+01, 1.1544E+01, 2.2720E+03, &
      1.1927040E+00, 4.71026E+01, 1.1473E+01, 2.2469E+03, &
      1.1986676E+00, 4.70648E+01, 1.1402E+01, 2.2221E+03, &
      1.2046609E+00, 4.70253E+01, 1.1332E+01, 2.1976E+03, &
      1.2106842E+00, 4.69840E+01, 1.1263E+01, 2.1734E+03, &
      1.2167376E+00, 4.69387E+01, 1.1194E+01, 2.1495E+03, &
      1.2228213E+00, 4.68938E+01, 1.1126E+01, 2.1259E+03, &
      1.2289354E+00, 4.68471E+01, 1.1058E+01, 2.1026E+03, &
      1.2350801E+00, 4.67984E+01, 1.0991E+01, 2.0796E+03, &
      1.2412555E+00, 4.67478E+01, 1.0925E+01, 2.0568E+03, &
      1.2474618E+00, 4.66953E+01, 1.0859E+01, 2.0343E+03, &
      1.2536991E+00, 4.66408E+01, 1.0793E+01, 2.0121E+03, &
      1.2599676E+00, 4.65842E+01, 1.0728E+01, 1.9902E+03, &
      1.2662674E+00, 4.65256E+01, 1.0664E+01, 1.9685E+03, &
      1.2725988E+00, 4.64648E+01, 1.0600E+01, 1.9471E+03, &
      1.2789618E+00, 4.64019E+01, 1.0536E+01, 1.9259E+03, &
      1.2853566E+00, 4.63368E+01, 1.0473E+01, 1.9050E+03, &
      1.2917833E+00, 4.62693E+01, 1.0411E+01, 1.8844E+03, &
      1.2982423E+00, 4.61996E+01, 1.0349E+01, 1.8640E+03, &
      1.3047335E+00, 4.61274E+01, 1.0288E+01, 1.8438E+03, &
      1.3112571E+00, 4.60527E+01, 1.0227E+01, 1.8239E+03, &
      1.3178134E+00, 4.59756E+01, 1.0166E+01, 1.8043E+03, &
      1.3244025E+00, 4.58958E+01, 1.0106E+01, 1.7848E+03, &
      1.3310245E+00, 4.58132E+01, 1.0047E+01, 1.7656E+03, &
      1.3376796E+00, 4.57280E+01, 9.9880E+00, 1.7466E+03, &
      1.3443680E+00, 4.56398E+01, 9.9294E+00, 1.7279E+03, &
      1.3510899E+00, 4.55486E+01, 9.8713E+00, 1.7093E+03, &
      1.3578453E+00, 4.54544E+01, 9.8137E+00, 1.6910E+03, &
      1.3646345E+00, 4.53569E+01, 9.7565E+00, 1.6729E+03, &
      1.3714577E+00, 4.52561E+01, 9.6997E+00, 1.6550E+03, &
      1.3783150E+00, 4.51519E+01, 9.6434E+00, 1.6373E+03, &
      1.3852066E+00, 4.50441E+01, 9.5875E+00, 1.6199E+03, &
      1.3921326E+00, 4.49325E+01, 9.5321E+00, 1.6026E+03, &
      1.3990933E+00, 4.48183E+01, 9.4771E+00, 1.5856E+03, &
      1.4060887E+00, 4.46988E+01, 9.4226E+00, 1.5687E+03, &
      1.4131192E+00, 4.45751E+01, 9.3684E+00, 1.5521E+03, &
      1.4201848E+00, 4.44469E+01, 9.3147E+00, 1.5356E+03, &
      1.4272857E+00, 4.43141E+01, 9.2614E+00, 1.5193E+03, &
      1.4344221E+00, 4.41763E+01, 9.2086E+00, 1.5033E+03, &
      1.4415942E+00, 4.40335E+01, 9.1561E+00, 1.4874E+03, &
      1.4488022E+00, 4.38852E+01, 9.1040E+00, 1.4717E+03, &
      1.4560462E+00, 4.37312E+01, 9.0524E+00, 1.4562E+03, &
      1.4633265E+00, 4.35713E+01, 9.0011E+00, 1.4408E+03, &
      1.4706431E+00, 4.34049E+01, 8.9502E+00, 1.4257E+03, &
      1.4779963E+00, 4.32318E+01, 8.8998E+00, 1.4107E+03, &
      1.4853863E+00, 4.30516E+01, 8.8497E+00, 1.3959E+03, &
      1.4928132E+00, 4.28638E+01, 8.8000E+00, 1.3813E+03, &
      1.5002773E+00, 4.26679E+01, 8.7507E+00, 1.3668E+03, &
      1.5077787E+00, 4.24634E+01, 8.7017E+00, 1.3525E+03, &
      1.5153176E+00, 4.22496E+01, 8.6532E+00, 1.3384E+03, &
      1.5228942E+00, 4.20259E+01, 8.6050E+00, 1.3244E+03, &
      1.5305086E+00, 4.17915E+01, 8.5571E+00, 1.3106E+03, &
      1.5381612E+00, 4.15455E+01, 8.5097E+00, 1.2970E+03, &
      1.5458520E+00, 4.12872E+01, 8.4626E+00, 1.2835E+03, &
      1.5535812E+00, 4.10153E+01, 8.4158E+00, 1.2702E+03, &
      1.5613491E+00, 4.07286E+01, 8.3694E+00, 1.2570E+03, &
      1.5691559E+00, 4.04259E+01, 8.3234E+00, 1.2440E+03, &
      1.5770017E+00, 4.01054E+01, 8.2777E+00, 1.2311E+03, &
      1.5848867E+00, 3.97655E+01, 8.2324E+00, 1.2184E+03, &
      1.5928111E+00, 3.94038E+01, 8.1874E+00, 1.2058E+03, &
      1.6007752E+00, 3.90180E+01, 8.1427E+00, 1.1934E+03, &
      1.6087790E+00, 3.86051E+01, 8.0984E+00, 1.1811E+03, &
      1.6168229E+00, 3.81615E+01, 8.0544E+00, 1.1690E+03, &
      1.6249070E+00, 3.76829E+01, 8.0094E+00, 1.1568E+03, &
      1.6330316E+00, 3.71639E+01, 7.9646E+00, 1.1447E+03, &
      1.6411967E+00, 3.65981E+01, 7.9201E+00, 1.1327E+03, &
      1.6494027E+00, 3.59770E+01, 7.8760E+00, 1.1209E+03, &
      1.6576497E+00, 3.52899E+01, 7.8322E+00, 1.1093E+03, &
      1.6659380E+00, 3.45223E+01, 7.7887E+00, 1.0977E+03, &
      1.6742677E+00, 3.36544E+01, 7.7455E+00, 1.0863E+03, &
      1.6826390E+00, 3.26576E+01, 7.7027E+00, 1.0751E+03, &
      1.6910522E+00, 3.14889E+01, 7.6602E+00, 1.0639E+03, &
      1.6995075E+00, 3.00801E+01, 7.6180E+00, 1.0529E+03, &
      1.7080050E+00, 2.82994E+01, 7.5761E+00, 1.0420E+03, &
      1.7165450E+00, 2.58762E+01, 7.5346E+00, 1.0313E+03, &
      1.7251278E+00, 2.20117E+01, 7.4933E+00, 1.0206E+03, &
      1.7337534E+00, 1.00711E+01, 7.4524E+00, 1.0101E+03, &
      1.7347877E+00, 1.51614E+00, 7.4475E+00, 1.0089E+03, &
      1.7354123E+00, 1.19537E+00, 2.5774E+01, 3.4644E+03, &
      1.7424222E+00, 1.93420E+01, 2.5625E+01, 3.4305E+03, &
      1.7511343E+00, 2.34233E+01, 2.5441E+01, 3.3891E+03, &
      1.7598899E+00, 2.52767E+01, 2.5259E+01, 3.3482E+03, &
      1.7686894E+00, 2.60669E+01, 2.5078E+01, 3.3079E+03, &
      1.7775328E+00, 2.59037E+01, 2.4899E+01, 3.2680E+03, &
      1.7864205E+00, 2.39938E+01, 2.4721E+01, 3.2286E+03, &
      1.7926172E+00, 1.53905E+01, 2.4598E+01, 3.2016E+03, &
      1.7937828E+00, 1.53399E+01, 3.6635E+01, 4.7601E+03, &
      1.7953526E+00, 2.05553E+01, 3.6588E+01, 4.7498E+03, &
      1.8043294E+00, 2.78700E+01, 3.6322E+01, 4.6919E+03, &
      1.8133510E+00, 3.10235E+01, 3.6057E+01, 4.6346E+03, &
      1.8224178E+00, 3.32274E+01, 3.5794E+01, 4.5781E+03, &
      1.8315299E+00, 3.49701E+01, 3.5534E+01, 4.5223E+03, &
      1.8406875E+00, 3.64297E+01, 3.5275E+01, 4.4672E+03, &
      1.8498909E+00, 3.76932E+01, 3.5019E+01, 4.4128E+03, &
      1.8591404E+00, 3.88107E+01, 3.4764E+01, 4.3590E+03, &
      1.8684361E+00, 3.98139E+01, 3.4510E+01, 4.3058E+03, &
      1.8777783E+00, 4.07245E+01, 3.4259E+01, 4.2533E+03, &
      1.8871672E+00, 4.15580E+01, 3.4009E+01, 4.2014E+03, &
      1.8966030E+00, 4.23260E+01, 3.3761E+01, 4.1502E+03, &
      1.9060860E+00, 4.30373E+01, 3.3515E+01, 4.0995E+03, &
      1.9156165E+00, 4.36990E+01, 3.3271E+01, 4.0496E+03, &
      1.9251945E+00, 4.43166E+01, 3.3029E+01, 4.0002E+03, &
      1.9348205E+00, 4.48947E+01, 3.2788E+01, 3.9515E+03, &
      1.9444946E+00, 4.54369E+01, 3.2550E+01, 3.9033E+03, &
      1.9542171E+00, 4.59464E+01, 3.2313E+01, 3.8558E+03, &
      1.9639882E+00, 4.64258E+01, 3.2078E+01, 3.8088E+03, &
      1.9738081E+00, 4.68771E+01, 3.1845E+01, 3.7625E+03, &
      1.9836772E+00, 4.73022E+01, 3.1614E+01, 3.7167E+03, &
      1.9935955E+00, 4.77026E+01, 3.1384E+01, 3.6715E+03, &
      2.0035635E+00, 4.80794E+01, 3.1157E+01, 3.6268E+03, &
      2.0135813E+00, 4.84337E+01, 3.0931E+01, 3.5827E+03, &
      2.0236492E+00, 4.87662E+01, 3.0706E+01, 3.5392E+03, &
      2.0337675E+00, 4.90774E+01, 3.0484E+01, 3.4961E+03, &
      2.0439363E+00, 4.93674E+01, 3.0263E+01, 3.4537E+03, &
      2.0541560E+00, 4.96364E+01, 3.0043E+01, 3.4117E+03, &
      2.0644268E+00, 4.98840E+01, 2.9826E+01, 3.3703E+03, &
      2.0747489E+00, 5.01095E+01, 2.9610E+01, 3.3294E+03, &
      2.0851227E+00, 5.03117E+01, 2.9396E+01, 3.2889E+03, &
      2.0955483E+00, 5.04889E+01, 2.9183E+01, 3.2490E+03, &
      2.1060260E+00, 5.06384E+01, 2.8972E+01, 3.2096E+03, &
      2.1165562E+00, 5.07562E+01, 2.8763E+01, 3.1707E+03, &
      2.1271389E+00, 5.08362E+01, 2.8555E+01, 3.1322E+03, &
      2.1377746E+00, 5.08686E+01, 2.8349E+01, 3.0943E+03, &
      2.1484635E+00, 5.08373E+01, 2.8144E+01, 3.0568E+03, &
      2.1592058E+00, 5.07121E+01, 2.7941E+01, 3.0197E+03, &
      2.1700018E+00, 5.04274E+01, 2.7740E+01, 2.9832E+03, &
      2.1808519E+00, 4.97924E+01, 2.7540E+01, 2.9470E+03, &
      2.1917561E+00, 4.73967E+01, 2.7341E+01, 2.9114E+03, &
      2.1963695E+00, 4.75241E+01, 3.2008E+01, 3.3994E+03, &
      2.2027149E+00, 4.96584E+01, 3.1866E+01, 3.3746E+03, &
      2.2137285E+00, 5.11518E+01, 3.1621E+01, 3.3321E+03, &
      2.2247971E+00, 5.20640E+01, 3.1377E+01, 3.2902E+03, &
      2.2359211E+00, 5.27545E+01, 3.1136E+01, 3.2488E+03, &
      2.2471007E+00, 5.33238E+01, 3.0897E+01, 3.2079E+03, &
      2.2583362E+00, 5.38141E+01, 3.0658E+01, 3.1674E+03, &
      2.2696279E+00, 5.42458E+01, 3.0416E+01, 3.1269E+03, &
      2.2809760E+00, 5.46310E+01, 3.0176E+01, 3.0868E+03, &
      2.2923809E+00, 5.49771E+01, 2.9937E+01, 3.0473E+03, &
      2.3038428E+00, 5.52874E+01, 2.9704E+01, 3.0086E+03, &
      2.3153620E+00, 5.55725E+01, 2.9489E+01, 2.9722E+03, &
      2.3269388E+00, 5.58396E+01, 2.9278E+01, 2.9363E+03, &
      2.3385735E+00, 5.60885E+01, 2.9066E+01, 2.9007E+03, &
      2.3502664E+00, 5.63185E+01, 2.8856E+01, 2.8655E+03, &
      2.3620177E+00, 5.65300E+01, 2.8648E+01, 2.8308E+03, &
      2.3738278E+00, 5.67226E+01, 2.8442E+01, 2.7966E+03, &
      2.3856970E+00, 5.68952E+01, 2.8239E+01, 2.7629E+03, &
      2.3976254E+00, 5.70457E+01, 2.8038E+01, 2.7297E+03, &
      2.4096136E+00, 5.71701E+01, 2.7840E+01, 2.6970E+03, &
      2.4216616E+00, 5.72608E+01, 2.7643E+01, 2.6648E+03, &
      2.4337699E+00, 5.73030E+01, 2.7449E+01, 2.6330E+03, &
      2.4459388E+00, 5.72607E+01, 2.7256E+01, 2.6016E+03, &
      2.4581685E+00, 5.70077E+01, 2.7066E+01, 2.5707E+03, &
      2.4657375E+00, 5.63718E+01, 2.6949E+01, 2.5519E+03, &
      2.4704593E+00, 5.61112E+01, 2.8778E+01, 2.7192E+03, &
      2.4716624E+00, 5.64484E+01, 2.8758E+01, 2.7159E+03, &
      2.4828116E+00, 5.75794E+01, 2.8566E+01, 2.6858E+03, &
      2.4952257E+00, 5.81520E+01, 2.8356E+01, 2.6529E+03, &
      2.5077018E+00, 5.85644E+01, 2.8148E+01, 2.6205E+03, &
      2.5202403E+00, 5.89024E+01, 2.7942E+01, 2.5884E+03, &
      2.5328415E+00, 5.91946E+01, 2.7738E+01, 2.5569E+03, &
      2.5455057E+00, 5.94544E+01, 2.7536E+01, 2.5257E+03, &
      2.5582333E+00, 5.96887E+01, 2.7335E+01, 2.4950E+03, &
      2.5710244E+00, 5.99011E+01, 2.7137E+01, 2.4646E+03, &
      2.5838796E+00, 6.00933E+01, 2.6940E+01, 2.4347E+03, &
      2.5967990E+00, 6.02657E+01, 2.6752E+01, 2.4058E+03, &
      2.6097829E+00, 6.04255E+01, 2.6572E+01, 2.3778E+03, &
      2.6228319E+00, 6.05724E+01, 2.6395E+01, 2.3503E+03, &
      2.6359460E+00, 6.07044E+01, 2.6220E+01, 2.3232E+03, &
      2.6491257E+00, 6.08183E+01, 2.6047E+01, 2.2966E+03, &
      2.6623714E+00, 6.09079E+01, 2.5876E+01, 2.2703E+03, &
      2.6756832E+00, 6.09602E+01, 2.5707E+01, 2.2443E+03, &
      2.6890617E+00, 6.09393E+01, 2.5540E+01, 2.2187E+03, &
      2.7025070E+00, 6.06585E+01, 2.5375E+01, 2.1935E+03, &
      2.7027735E+00, 6.06437E+01, 2.5372E+01, 2.1930E+03, &
      2.7132264E+00, 6.07657E+01, 2.6390E+01, 2.2719E+03, &
      2.7160195E+00, 6.09564E+01, 2.6356E+01, 2.2666E+03, &
      2.7295996E+00, 6.14815E+01, 2.6190E+01, 2.2412E+03, &
      2.7432476E+00, 6.18198E+01, 2.6026E+01, 2.2162E+03, &
      2.7569638E+00, 6.20959E+01, 2.5863E+01, 2.1915E+03, &
      2.7707486E+00, 6.23394E+01, 2.5701E+01, 2.1670E+03, &
      2.7846024E+00, 6.25624E+01, 2.5541E+01, 2.1429E+03, &
      2.7985254E+00, 6.27708E+01, 2.5382E+01, 2.1191E+03, &
      2.8125180E+00, 6.29684E+01, 2.5224E+01, 2.0955E+03, &
      2.8265806E+00, 6.31572E+01, 2.5067E+01, 2.0722E+03, &
      2.8407135E+00, 6.33389E+01, 2.4911E+01, 2.0491E+03, &
      2.8549171E+00, 6.35148E+01, 2.4757E+01, 2.0264E+03, &
      2.8691917E+00, 6.36858E+01, 2.4602E+01, 2.0038E+03, &
      2.8835376E+00, 6.38525E+01, 2.4448E+01, 1.9815E+03, &
      2.8979553E+00, 6.40155E+01, 2.4295E+01, 1.9594E+03, &
      2.9124451E+00, 6.41756E+01, 2.4144E+01, 1.9376E+03, &
      2.9270073E+00, 6.43333E+01, 2.3993E+01, 1.9160E+03, &
      2.9416424E+00, 6.44893E+01, 2.3843E+01, 1.8947E+03, &
      2.9563506E+00, 6.46444E+01, 2.3695E+01, 1.8736E+03, &
      2.9711323E+00, 6.47995E+01, 2.3547E+01, 1.8527E+03, &
      2.9859880E+00, 6.49561E+01, 2.3400E+01, 1.8321E+03, &
      3.0009179E+00, 6.51254E+01, 2.3253E+01, 1.8116E+03, &
      3.0159225E+00, 6.52708E+01, 2.3086E+01, 1.7898E+03, &
      3.0310021E+00, 6.54014E+01, 2.2921E+01, 1.7682E+03, &
      3.0461571E+00, 6.55258E+01, 2.2756E+01, 1.7469E+03, &
      3.0613879E+00, 6.56455E+01, 2.2593E+01, 1.7258E+03, &
      3.0766949E+00, 6.57613E+01, 2.2430E+01, 1.7050E+03, &
      3.0920783E+00, 6.58737E+01, 2.2269E+01, 1.6844E+03, &
      3.1075387E+00, 6.59828E+01, 2.2109E+01, 1.6641E+03, &
      3.1230764E+00, 6.60889E+01, 2.1950E+01, 1.6440E+03, &
      3.1386918E+00, 6.61921E+01, 2.1793E+01, 1.6242E+03, &
      3.1543853E+00, 6.62928E+01, 2.1636E+01, 1.6046E+03, &
      3.1701572E+00, 6.63909E+01, 2.1481E+01, 1.5852E+03, &
      3.1860080E+00, 6.64866E+01, 2.1326E+01, 1.5661E+03, &
      3.2019380E+00, 6.65802E+01, 2.1173E+01, 1.5472E+03, &
      3.2179477E+00, 6.66716E+01, 2.1020E+01, 1.5285E+03, &
      3.2340374E+00, 6.67605E+01, 2.0868E+01, 1.5100E+03, &
      3.2502076E+00, 6.68471E+01, 2.0717E+01, 1.4917E+03, &
      3.2664587E+00, 6.69315E+01, 2.0567E+01, 1.4736E+03, &
      3.2827910E+00, 6.70138E+01, 2.0418E+01, 1.4558E+03, &
      3.2992049E+00, 6.70941E+01, 2.0271E+01, 1.4382E+03, &
      3.3157009E+00, 6.71725E+01, 2.0124E+01, 1.4208E+03, &
      3.3322794E+00, 6.72491E+01, 1.9979E+01, 1.4036E+03, &
      3.3489408E+00, 6.73240E+01, 1.9835E+01, 1.3866E+03, &
      3.3656856E+00, 6.73972E+01, 1.9692E+01, 1.3699E+03, &
      3.3825140E+00, 6.74688E+01, 1.9550E+01, 1.3533E+03, &
      3.3994265E+00, 6.75389E+01, 1.9409E+01, 1.3370E+03, &
      3.4164237E+00, 6.76075E+01, 1.9270E+01, 1.3209E+03, &
      3.4335058E+00, 6.76748E+01, 1.9131E+01, 1.3050E+03, &
      3.4506733E+00, 6.77408E+01, 1.8994E+01, 1.2892E+03, &
      3.4679267E+00, 6.78056E+01, 1.8858E+01, 1.2737E+03, &
      3.4852663E+00, 6.80766E+01, 1.8716E+01, 1.2579E+03, &
      3.5026927E+00, 6.81386E+01, 1.8574E+01, 1.2423E+03, &
      3.5202061E+00, 6.81987E+01, 1.8434E+01, 1.2269E+03, &
      3.5378072E+00, 6.82569E+01, 1.8294E+01, 1.2116E+03, &
      3.5554962E+00, 6.83133E+01, 1.8156E+01, 1.1966E+03, &
      3.5732737E+00, 6.83681E+01, 1.8018E+01, 1.1817E+03, &
      3.5911400E+00, 6.85614E+01, 1.7881E+01, 1.1670E+03, &
      3.6090957E+00, 6.86131E+01, 1.7741E+01, 1.1522E+03, &
      3.6271412E+00, 6.86628E+01, 1.7603E+01, 1.1376E+03, &
      3.6452769E+00, 6.87107E+01, 1.7466E+01, 1.1232E+03, &
      3.6635033E+00, 6.87567E+01, 1.7330E+01, 1.1090E+03, &
      3.6818208E+00, 6.88009E+01, 1.7195E+01, 1.0950E+03, &
      3.7002299E+00, 6.88436E+01, 1.7061E+01, 1.0812E+03, &
      3.7187311E+00, 6.88847E+01, 1.6929E+01, 1.0676E+03, &
      3.7373247E+00, 6.89244E+01, 1.6798E+01, 1.0541E+03, &
      3.7560114E+00, 6.89626E+01, 1.6668E+01, 1.0409E+03, &
      3.7747914E+00, 6.89994E+01, 1.6540E+01, 1.0278E+03, &
      3.7936654E+00, 6.90350E+01, 1.6412E+01, 1.0149E+03, &
      3.8126337E+00, 6.90693E+01, 1.6286E+01, 1.0022E+03, &
      3.8316969E+00, 6.91024E+01, 1.6161E+01, 9.8962E+02, &
      3.8508554E+00, 6.91343E+01, 1.6037E+01, 9.7724E+02, &
      3.8701096E+00, 6.91651E+01, 1.5914E+01, 9.6502E+02, &
      3.8894602E+00, 6.91948E+01, 1.5792E+01, 9.5296E+02, &
      3.9089075E+00, 6.92235E+01, 1.5672E+01, 9.4107E+02, &
      3.9284520E+00, 6.92511E+01, 1.5552E+01, 9.2934E+02, &
      3.9480943E+00, 6.92778E+01, 1.5434E+01, 9.1776E+02, &
      3.9678347E+00, 6.93036E+01, 1.5317E+01, 9.0634E+02, &
      3.9876739E+00, 6.93284E+01, 1.5200E+01, 8.9508E+02, &
      4.055024E+00, 6.97903E+01, 1.4818E+01, 8.5836E+02, &
      4.334821E+00, 6.99592E+01, 1.3402E+01, 7.2729E+02, &
      4.633924E+00, 7.01464E+01, 1.2084E+01, 6.1445E+02, &
      4.953664E+00, 7.01733E+01, 1.0900E+01, 5.1939E+02, &
      5.295467E+00, 7.00792E+01, 9.8206E+00, 4.3866E+02, &
      5.660855E+00, 6.99372E+01, 8.8550E+00, 3.7085E+02, &
      6.051453E+00, 6.97167E+01, 7.9928E+00, 3.1391E+02, &
      6.469004E+00, 6.94432E+01, 7.2248E+00, 2.6617E+02, &
      6.915365E+00, 6.91140E+01, 6.5400E+00, 2.2605E+02, &
      7.392525E+00, 6.87163E+01, 5.9283E+00, 1.9230E+02, &
      7.902609E+00, 6.82155E+01, 5.3635E+00, 1.6333E+02, &
      8.447890E+00, 6.75254E+01, 4.8481E+00, 1.3866E+02, &
      9.030794E+00, 6.64617E+01, 4.3729E+00, 1.1752E+02, &
      9.653919E+00, 6.37140E+01, 3.8954E+00, 9.8456E+01, &
      9.683478E+00, 6.34252E+01, 3.8744E+00, 9.7651E+01, &
      9.831694E+00, 6.05401E+01, 3.7718E+00, 9.3759E+01, &
      9.871219E+00, 5.71434E+01, 3.7453E+00, 9.2759E+01, &
      9.930505E+00, 6.05654E+01, 1.0348E+01, 2.4683E+02, &
      1.007872E+01, 6.35612E+01, 1.0006E+01, 2.3531E+02, &
      1.032004E+01, 6.50977E+01, 9.4836E+00, 2.1805E+02, &
      1.091338E+01, 6.54822E+01, 8.5087E+00, 1.8542E+02, &
      1.103212E+01, 6.48825E+01, 8.3416E+00, 1.7991E+02, &
      1.108042E+01, 6.43050E+01, 8.2758E+00, 1.7774E+02, &
      1.112496E+01, 6.27014E+01, 8.2160E+00, 1.7579E+02, &
      1.119178E+01, 6.44029E+01, 1.1357E+01, 2.4000E+02, &
      1.135882E+01, 6.59310E+01, 1.1070E+01, 2.3059E+02, &
      1.144787E+01, 6.62523E+01, 1.0923E+01, 2.2581E+02, &
      1.162309E+01, 6.62303E+01, 1.0644E+01, 2.1683E+02, &
      1.166982E+01, 6.55306E+01, 1.0572E+01, 2.1454E+02, &
      1.173991E+01, 6.65825E+01, 1.2137E+01, 2.4424E+02, &
      1.179334E+01, 6.70814E+01, 1.2053E+01, 2.4149E+02, &
      1.191513E+01, 6.77871E+01, 1.1867E+01, 2.3539E+02, &
      1.260708E+01, 6.97701E+01, 1.0898E+01, 2.0459E+02, &
      1.347697E+01, 7.10512E+01, 9.8172E+00, 1.7272E+02, &
      1.440688E+01, 7.18606E+01, 8.8261E+00, 1.4556E+02, &
      1.540095E+01, 7.24049E+01, 7.9175E+00, 1.2242E+02, &
      1.646362E+01, 7.27659E+01, 7.0951E+00, 1.0288E+02, &
      1.759961E+01, 7.30023E+01, 6.3589E+00, 8.6490E+01, &
      1.881398E+01, 7.31523E+01, 5.7019E+00, 7.2761E+01, &
      2.011215E+01, 7.33039E+01, 5.1142E+00, 6.1242E+01, &
      2.149988E+01, 7.33452E+01, 4.5840E+00, 5.1526E+01, &
      2.298338E+01, 7.33888E+01, 4.1096E+00, 4.3374E+01, &
      2.456923E+01, 7.34025E+01, 3.6801E+00, 3.6481E+01, &
      2.626450E+01, 7.33713E+01, 3.2969E+00, 3.0707E+01, &
      2.807676E+01, 7.33311E+01, 2.9320E+00, 2.5676E+01, &
      3.001405E+01, 7.32561E+01, 2.6027E+00, 2.1442E+01, &
      3.208502E+01, 7.31674E+01, 2.3123E+00, 1.7929E+01, &
      3.429889E+01, 7.30703E+01, 2.0559E+00, 1.5012E+01, &
      3.666551E+01, 7.29671E+01, 1.8295E+00, 1.2586E+01, &
      3.919543E+01, 7.28589E+01, 1.6292E+00, 1.0565E+01, &
      4.189992E+01, 7.27455E+01, 1.4520E+00, 8.8817E+00, &
      4.479101E+01, 7.26247E+01, 1.2929E+00, 7.4661E+00, &
      4.788159E+01, 7.24914E+01, 1.1512E+00, 6.2808E+00, &
      5.118542E+01, 7.23388E+01, 1.0236E+00, 5.2820E+00, &
      5.471721E+01, 7.21497E+01, 9.0721E-01, 4.4341E+00, &
      5.849270E+01, 7.18904E+01, 8.0445E-01, 3.7285E+00, &
      6.252870E+01, 7.14546E+01, 7.1341E-01, 3.1397E+00, &
      6.606807E+01, 7.04835E+01, 6.4647E-01, 2.7288E+00, &
      6.684318E+01, 6.98055E+01, 6.3316E-01, 2.6494E+00, &
      6.707932E+01, 6.93757E+01, 6.2919E-01, 2.6260E+00, &
      6.734898E+01, 6.80495E+01, 6.2470E-01, 2.5995E+00, &
      6.775348E+01, 6.93893E+01, 3.2286E+00, 1.1521E+01, &
      6.876473E+01, 7.05993E+01, 3.1573E+00, 1.1109E+01, &
      7.145536E+01, 7.15926E+01, 2.9775E+00, 1.0101E+01, &
      7.638578E+01, 7.23062E+01, 2.6741E+00, 8.5192E+00, &
      8.165640E+01, 7.26821E+01, 2.3979E+00, 7.1777E+00, &
      8.729069E+01, 7.29153E+01, 2.1482E+00, 6.0451E+00, &
      9.331374E+01, 7.30695E+01, 1.9189E+00, 5.0802E+00, &
      9.975239E+01, 7.31707E+01, 1.7140E+00, 4.2720E+00, &
      1.066353E+02, 7.32373E+01, 1.5312E+00, 3.5958E+00, &
      1.139931E+02, 7.32801E+01, 1.3680E+00, 3.0298E+00, &
      1.218587E+02, 7.33060E+01, 1.2224E+00, 2.5557E+00, &
      1.302669E+02, 7.33200E+01, 1.0924E+00, 2.1586E+00, &
      1.392553E+02, 7.33528E+01, 9.7465E-01, 1.8231E+00, &
      1.488640E+02, 7.33493E+01, 8.6801E-01, 1.5396E+00, &
      1.591356E+02, 7.33401E+01, 7.7311E-01, 1.3026E+00, &
      1.701159E+02, 7.33273E+01, 6.8866E-01, 1.1045E+00, &
      1.818539E+02, 7.33121E+01, 6.1349E-01, 9.3878E-01, &
      1.944018E+02, 7.32954E+01, 5.4657E-01, 8.0001E-01, &
      2.078156E+02, 7.32778E+01, 4.8699E-01, 6.8375E-01, &
      2.221548E+02, 7.32599E+01, 4.3395E-01, 5.8627E-01, &
      2.374835E+02, 7.32420E+01, 3.8672E-01, 5.0444E-01, &
      2.538699E+02, 7.32244E+01, 3.4465E-01, 4.3570E-01, &
      2.713869E+02, 7.32072E+01, 3.0719E-01, 3.7786E-01, &
      2.901126E+02, 7.31903E+01, 2.7382E-01, 3.2913E-01, &
      3.101304E+02, 7.31743E+01, 2.4570E-01, 2.8923E-01, &
      3.315294E+02, 7.31599E+01, 2.2066E-01, 2.5541E-01, &
      3.544049E+02, 7.31467E+01, 1.9819E-01, 2.2655E-01, &
      3.788588E+02, 7.31345E+01, 1.7802E-01, 2.0189E-01, &
      4.050001E+02, 7.31233E+01, 1.5992E-01, 1.8074E-01, &
      4.329451E+02, 7.31130E+01, 1.4366E-01, 1.6256E-01]
   case(74)
      frel = -8.5980E-01
      allocate(nist_tab(3656))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.978000E-03, 6.65660E+00, 0.0000E+00, 1.6514E-05, &
      6.069500E-03, 6.08378E+00, 0.0000E+00, 1.7873E-05, &
      6.093900E-03, 5.43188E+00, 0.0000E+00, 1.8250E-05, &
      6.130500E-03, 6.07665E+00, 1.2640E+00, 4.7191E+04, &
      6.222000E-03, 6.61931E+00, 1.2569E+00, 4.6237E+04, &
      1.069000E-02, 6.56496E+00, 2.8092E+00, 6.0149E+04, &
      1.142761E-02, 6.58098E+00, 3.3151E+00, 6.6397E+04, &
      1.221612E-02, 6.68973E+00, 3.8688E+00, 7.2486E+04, &
      1.305903E-02, 6.90985E+00, 4.4462E+00, 7.7928E+04, &
      1.396010E-02, 7.25243E+00, 5.0157E+00, 8.2236E+04, &
      1.492335E-02, 7.71790E+00, 5.5412E+00, 8.4987E+04, &
      1.595306E-02, 8.29414E+00, 5.9859E+00, 8.5881E+04, &
      1.705382E-02, 8.95658E+00, 6.3173E+00, 8.4786E+04, &
      1.823053E-02, 9.67031E+00, 6.5117E+00, 8.1755E+04, &
      1.948844E-02, 1.04305E+01, 6.5573E+00, 7.7013E+04, &
      2.083314E-02, 1.11203E+01, 6.4550E+00, 7.0918E+04, &
      2.227063E-02, 1.17349E+01, 6.2179E+00, 6.3904E+04, &
      2.380730E-02, 1.22377E+01, 5.8684E+00, 5.6419E+04, &
      2.545001E-02, 1.25975E+01, 5.4333E+00, 4.8864E+04, &
      2.720606E-02, 1.27802E+01, 4.9392E+00, 4.1553E+04, &
      2.908327E-02, 1.27425E+01, 4.4187E+00, 3.4775E+04, &
      3.109002E-02, 1.23960E+01, 3.8996E+00, 2.8709E+04, &
      3.292800E-02, 1.16002E+01, 3.4700E+00, 2.4120E+04, &
      3.323523E-02, 1.13527E+01, 3.4028E+00, 2.3434E+04, &
      3.343200E-02, 1.11252E+01, 3.3604E+00, 2.3006E+04, &
      3.356640E-02, 1.08065E+01, 3.3318E+00, 2.2719E+04, &
      3.376800E-02, 1.09013E+01, 3.7405E+00, 2.5354E+04, &
      3.427200E-02, 1.06426E+01, 3.6194E+00, 2.4172E+04, &
      3.488800E-02, 9.79398E+00, 3.4773E+00, 2.2813E+04, &
      3.542200E-02, 7.64986E+00, 3.3593E+00, 2.1707E+04, &
      3.552846E-02, 6.18367E+00, 3.3363E+00, 2.1494E+04, &
      3.556440E-02, 5.04927E+00, 3.3286E+00, 2.1422E+04, &
      3.577000E-02, 7.68767E+00, 8.4061E+00, 5.3789E+04, &
      3.577800E-02, 7.76490E+00, 8.4024E+00, 5.3753E+04, &
      3.631750E-02, 1.00580E+01, 8.1578E+00, 5.1413E+04, &
      3.646350E-02, 1.02313E+01, 8.0933E+00, 5.0802E+04, &
      3.668250E-02, 1.07991E+01, 8.3052E+00, 5.1821E+04, &
      3.723000E-02, 1.16701E+01, 8.0634E+00, 4.9573E+04, &
      3.797993E-02, 1.23990E+01, 7.7505E+00, 4.6708E+04, &
      4.060054E-02, 1.36087E+01, 6.7833E+00, 3.8241E+04, &
      4.340198E-02, 1.38839E+01, 5.8781E+00, 3.0999E+04, &
      4.586400E-02, 1.31951E+01, 5.2040E+00, 2.5971E+04, &
      4.639671E-02, 1.25737E+01, 5.0734E+00, 2.5028E+04, &
      4.656600E-02, 1.21538E+01, 5.0330E+00, 2.4738E+04, &
      4.675320E-02, 1.08814E+01, 4.9889E+00, 2.4424E+04, &
      4.703400E-02, 1.22052E+01, 7.4381E+00, 3.6197E+04, &
      4.773600E-02, 1.33694E+01, 7.2554E+00, 3.4788E+04, &
      4.959809E-02, 1.43109E+01, 6.8159E+00, 3.1454E+04, &
      5.302035E-02, 1.49342E+01, 6.1523E+00, 2.6559E+04, &
      5.667876E-02, 1.51540E+01, 5.5407E+00, 2.2375E+04, &
      6.058959E-02, 1.51050E+01, 5.0115E+00, 1.8932E+04, &
      6.477028E-02, 1.48547E+01, 4.6086E+00, 1.6286E+04, &
      6.923942E-02, 1.44550E+01, 4.3611E+00, 1.4416E+04, &
      7.401695E-02, 1.39447E+01, 4.2864E+00, 1.3255E+04, &
      7.555800E-02, 1.37648E+01, 4.2998E+00, 1.3025E+04, &
      7.671450E-02, 1.36117E+01, 4.3209E+00, 1.2892E+04, &
      7.702290E-02, 1.35465E+01, 4.3281E+00, 1.2862E+04, &
      7.748550E-02, 1.35269E+01, 4.4018E+00, 1.3002E+04, &
      7.864200E-02, 1.34278E+01, 4.4392E+00, 1.2920E+04, &
      7.912411E-02, 1.33807E+01, 4.4574E+00, 1.2894E+04, &
      8.458368E-02, 1.28282E+01, 4.7586E+00, 1.2877E+04, &
      9.041995E-02, 1.22717E+01, 5.2496E+00, 1.3288E+04, &
      9.665893E-02, 1.17538E+01, 5.9297E+00, 1.4041E+04, &
      1.0000000E-01, 1.30562E+01, 6.3475E+00, 1.4529E+04, &
      1.0050000E-01, 1.30233E+01, 6.4128E+00, 1.4605E+04, &
      1.0100250E-01, 1.29096E+01, 6.4792E+00, 1.4683E+04, &
      1.0150751E-01, 1.28777E+01, 6.5465E+00, 1.4762E+04, &
      1.0201505E-01, 1.28465E+01, 6.6148E+00, 1.4842E+04, &
      1.0252513E-01, 1.28158E+01, 6.6841E+00, 1.4923E+04, &
      1.0303775E-01, 1.27857E+01, 6.7544E+00, 1.5004E+04, &
      1.0355294E-01, 1.27562E+01, 6.8257E+00, 1.5087E+04, &
      1.0407070E-01, 1.27273E+01, 6.8979E+00, 1.5171E+04, &
      1.0459106E-01, 1.26990E+01, 6.9712E+00, 1.5256E+04, &
      1.0511401E-01, 1.26714E+01, 7.0454E+00, 1.5342E+04, &
      1.0563958E-01, 1.26445E+01, 7.1206E+00, 1.5428E+04, &
      1.0616778E-01, 1.26183E+01, 7.1967E+00, 1.5516E+04, &
      1.0669862E-01, 1.25927E+01, 7.2738E+00, 1.5604E+04, &
      1.0723211E-01, 1.25679E+01, 7.3518E+00, 1.5693E+04, &
      1.0776827E-01, 1.25437E+01, 7.4307E+00, 1.5782E+04, &
      1.0830712E-01, 1.25204E+01, 7.5106E+00, 1.5873E+04, &
      1.0884865E-01, 1.24977E+01, 7.5914E+00, 1.5964E+04, &
      1.0939289E-01, 1.24759E+01, 7.6732E+00, 1.6055E+04, &
      1.0993986E-01, 1.23657E+01, 7.7558E+00, 1.6147E+04, &
      1.1048956E-01, 1.23454E+01, 7.8393E+00, 1.6240E+04, &
      1.1104201E-01, 1.23259E+01, 7.9238E+00, 1.6333E+04, &
      1.1159722E-01, 1.23073E+01, 8.0091E+00, 1.6427E+04, &
      1.1215520E-01, 1.22895E+01, 8.0953E+00, 1.6521E+04, &
      1.1271598E-01, 1.22726E+01, 8.1823E+00, 1.6616E+04, &
      1.1327956E-01, 1.22565E+01, 8.2703E+00, 1.6711E+04, &
      1.1384596E-01, 1.22414E+01, 8.3590E+00, 1.6806E+04, &
      1.1441519E-01, 1.22271E+01, 8.4486E+00, 1.6902E+04, &
      1.1498726E-01, 1.22137E+01, 8.5396E+00, 1.6999E+04, &
      1.1556220E-01, 1.22013E+01, 8.6315E+00, 1.7096E+04, &
      1.1614001E-01, 1.21899E+01, 8.7242E+00, 1.7194E+04, &
      1.1672071E-01, 1.21794E+01, 8.8177E+00, 1.7292E+04, &
      1.1730431E-01, 1.21700E+01, 8.9120E+00, 1.7390E+04, &
      1.1789083E-01, 1.21616E+01, 9.0070E+00, 1.7488E+04, &
      1.1848029E-01, 1.21542E+01, 9.1028E+00, 1.7586E+04, &
      1.1907269E-01, 1.21478E+01, 9.1994E+00, 1.7684E+04, &
      1.1966805E-01, 1.21426E+01, 9.2967E+00, 1.7782E+04, &
      1.2026639E-01, 1.21384E+01, 9.3948E+00, 1.7880E+04, &
      1.2086772E-01, 1.21352E+01, 9.4935E+00, 1.7978E+04, &
      1.2147206E-01, 1.21332E+01, 9.5930E+00, 1.8076E+04, &
      1.2207942E-01, 1.21323E+01, 9.6932E+00, 1.8174E+04, &
      1.2268982E-01, 1.21325E+01, 9.7940E+00, 1.8272E+04, &
      1.2330327E-01, 1.21339E+01, 9.8955E+00, 1.8369E+04, &
      1.2391979E-01, 1.21364E+01, 9.9977E+00, 1.8467E+04, &
      1.2453939E-01, 1.21401E+01, 1.0100E+01, 1.8564E+04, &
      1.2516208E-01, 1.21450E+01, 1.0204E+01, 1.8661E+04, &
      1.2578789E-01, 1.21511E+01, 1.0308E+01, 1.8757E+04, &
      1.2641683E-01, 1.21584E+01, 1.0413E+01, 1.8853E+04, &
      1.2704892E-01, 1.21670E+01, 1.0518E+01, 1.8949E+04, &
      1.2768416E-01, 1.21768E+01, 1.0624E+01, 1.9044E+04, &
      1.2832258E-01, 1.21878E+01, 1.0730E+01, 1.9139E+04, &
      1.2896419E-01, 1.22002E+01, 1.0837E+01, 1.9234E+04, &
      1.2960902E-01, 1.22138E+01, 1.0944E+01, 1.9328E+04, &
      1.3025706E-01, 1.22287E+01, 1.1052E+01, 1.9421E+04, &
      1.3090835E-01, 1.22450E+01, 1.1161E+01, 1.9514E+04, &
      1.3156289E-01, 1.22626E+01, 1.1269E+01, 1.9607E+04, &
      1.3222070E-01, 1.22816E+01, 1.1379E+01, 1.9698E+04, &
      1.3288181E-01, 1.23019E+01, 1.1489E+01, 1.9790E+04, &
      1.3354621E-01, 1.23237E+01, 1.1599E+01, 1.9880E+04, &
      1.3421395E-01, 1.23468E+01, 1.1709E+01, 1.9970E+04, &
      1.3488502E-01, 1.23714E+01, 1.1820E+01, 2.0059E+04, &
      1.3555944E-01, 1.23974E+01, 1.1932E+01, 2.0147E+04, &
      1.3623724E-01, 1.24248E+01, 1.2043E+01, 2.0234E+04, &
      1.3691842E-01, 1.24537E+01, 1.2155E+01, 2.0320E+04, &
      1.3760302E-01, 1.24840E+01, 1.2266E+01, 2.0404E+04, &
      1.3829103E-01, 1.25158E+01, 1.2378E+01, 2.0488E+04, &
      1.3898249E-01, 1.25489E+01, 1.2490E+01, 2.0570E+04, &
      1.3967740E-01, 1.25835E+01, 1.2602E+01, 2.0651E+04, &
      1.4037579E-01, 1.26195E+01, 1.2714E+01, 2.0731E+04, &
      1.4107766E-01, 1.26583E+01, 1.2826E+01, 2.0810E+04, &
      1.4178305E-01, 1.26972E+01, 1.2938E+01, 2.0887E+04, &
      1.4249197E-01, 1.27375E+01, 1.3050E+01, 2.0964E+04, &
      1.4320443E-01, 1.27792E+01, 1.3162E+01, 2.1038E+04, &
      1.4392045E-01, 1.28224E+01, 1.3274E+01, 2.1112E+04, &
      1.4464005E-01, 1.28670E+01, 1.3386E+01, 2.1184E+04, &
      1.4536325E-01, 1.29130E+01, 1.3498E+01, 2.1255E+04, &
      1.4609007E-01, 1.29605E+01, 1.3610E+01, 2.1325E+04, &
      1.4682052E-01, 1.30094E+01, 1.3722E+01, 2.1392E+04, &
      1.4755462E-01, 1.30597E+01, 1.3833E+01, 2.1459E+04, &
      1.4829239E-01, 1.31114E+01, 1.3944E+01, 2.1523E+04, &
      1.4903386E-01, 1.31645E+01, 1.4055E+01, 2.1586E+04, &
      1.4977903E-01, 1.32190E+01, 1.4165E+01, 2.1648E+04, &
      1.5052792E-01, 1.32749E+01, 1.4275E+01, 2.1707E+04, &
      1.5128056E-01, 1.33321E+01, 1.4385E+01, 2.1765E+04, &
      1.5203696E-01, 1.33907E+01, 1.4494E+01, 2.1821E+04, &
      1.5279715E-01, 1.34505E+01, 1.4603E+01, 2.1876E+04, &
      1.5356113E-01, 1.35117E+01, 1.4711E+01, 2.1929E+04, &
      1.5432894E-01, 1.35742E+01, 1.4819E+01, 2.1980E+04, &
      1.5510058E-01, 1.36379E+01, 1.4927E+01, 2.2029E+04, &
      1.5587609E-01, 1.37030E+01, 1.5034E+01, 2.2076E+04, &
      1.5665547E-01, 1.37692E+01, 1.5140E+01, 2.2122E+04, &
      1.5743875E-01, 1.38367E+01, 1.5246E+01, 2.2166E+04, &
      1.5822594E-01, 1.39054E+01, 1.5352E+01, 2.2208E+04, &
      1.5901707E-01, 1.39754E+01, 1.5456E+01, 2.2249E+04, &
      1.5981215E-01, 1.40465E+01, 1.5561E+01, 2.2287E+04, &
      1.6061121E-01, 1.41188E+01, 1.5664E+01, 2.2324E+04, &
      1.6141427E-01, 1.41922E+01, 1.5767E+01, 2.2359E+04, &
      1.6222134E-01, 1.42668E+01, 1.5870E+01, 2.2393E+04, &
      1.6303245E-01, 1.43426E+01, 1.5972E+01, 2.2424E+04, &
      1.6384761E-01, 1.44194E+01, 1.6073E+01, 2.2454E+04, &
      1.6466685E-01, 1.44973E+01, 1.6173E+01, 2.2482E+04, &
      1.6549018E-01, 1.45763E+01, 1.6273E+01, 2.2508E+04, &
      1.6631763E-01, 1.46564E+01, 1.6372E+01, 2.2533E+04, &
      1.6714922E-01, 1.47375E+01, 1.6471E+01, 2.2555E+04, &
      1.6798497E-01, 1.48197E+01, 1.6568E+01, 2.2576E+04, &
      1.6882489E-01, 1.49029E+01, 1.6665E+01, 2.2595E+04, &
      1.6966902E-01, 1.49870E+01, 1.6761E+01, 2.2613E+04, &
      1.7051736E-01, 1.50721E+01, 1.6857E+01, 2.2628E+04, &
      1.7136995E-01, 1.51582E+01, 1.6952E+01, 2.2642E+04, &
      1.7222680E-01, 1.52453E+01, 1.7045E+01, 2.2654E+04, &
      1.7308793E-01, 1.53332E+01, 1.7138E+01, 2.2665E+04, &
      1.7395337E-01, 1.54221E+01, 1.7231E+01, 2.2673E+04, &
      1.7482314E-01, 1.55118E+01, 1.7322E+01, 2.2680E+04, &
      1.7569726E-01, 1.56025E+01, 1.7413E+01, 2.2685E+04, &
      1.7657574E-01, 1.56939E+01, 1.7503E+01, 2.2689E+04, &
      1.7745862E-01, 1.57862E+01, 1.7591E+01, 2.2691E+04, &
      1.7834591E-01, 1.58794E+01, 1.7679E+01, 2.2691E+04, &
      1.7923764E-01, 1.59733E+01, 1.7767E+01, 2.2689E+04, &
      1.8013383E-01, 1.60679E+01, 1.7853E+01, 2.2686E+04, &
      1.8103450E-01, 1.61634E+01, 1.7938E+01, 2.2681E+04, &
      1.8193967E-01, 1.62595E+01, 1.8023E+01, 2.2675E+04, &
      1.8284937E-01, 1.63564E+01, 1.8106E+01, 2.2667E+04, &
      1.8376362E-01, 1.64540E+01, 1.8189E+01, 2.2657E+04, &
      1.8468244E-01, 1.65522E+01, 1.8271E+01, 2.2646E+04, &
      1.8560585E-01, 1.66511E+01, 1.8352E+01, 2.2633E+04, &
      1.8653388E-01, 1.67507E+01, 1.8432E+01, 2.2618E+04, &
      1.8746655E-01, 1.68508E+01, 1.8511E+01, 2.2602E+04, &
      1.8840388E-01, 1.69515E+01, 1.8589E+01, 2.2584E+04, &
      1.8934590E-01, 1.70528E+01, 1.8666E+01, 2.2565E+04, &
      1.9029263E-01, 1.71547E+01, 1.8742E+01, 2.2545E+04, &
      1.9124409E-01, 1.72570E+01, 1.8817E+01, 2.2523E+04, &
      1.9220031E-01, 1.73599E+01, 1.8891E+01, 2.2499E+04, &
      1.9316131E-01, 1.74633E+01, 1.8965E+01, 2.2474E+04, &
      1.9412712E-01, 1.75671E+01, 1.9037E+01, 2.2447E+04, &
      1.9509776E-01, 1.76713E+01, 1.9108E+01, 2.2419E+04, &
      1.9607325E-01, 1.77760E+01, 1.9179E+01, 2.2390E+04, &
      1.9705361E-01, 1.78810E+01, 1.9248E+01, 2.2359E+04, &
      1.9803888E-01, 1.79864E+01, 1.9316E+01, 2.2327E+04, &
      1.9902907E-01, 1.80922E+01, 1.9384E+01, 2.2293E+04, &
      2.0002422E-01, 1.81983E+01, 1.9450E+01, 2.2258E+04, &
      2.0102434E-01, 1.83046E+01, 1.9516E+01, 2.2222E+04, &
      2.0202946E-01, 1.84112E+01, 1.9580E+01, 2.2185E+04, &
      2.0303961E-01, 1.85181E+01, 1.9643E+01, 2.2146E+04, &
      2.0405481E-01, 1.86251E+01, 1.9706E+01, 2.2106E+04, &
      2.0507508E-01, 1.87324E+01, 1.9767E+01, 2.2064E+04, &
      2.0610046E-01, 1.88398E+01, 1.9828E+01, 2.2022E+04, &
      2.0713096E-01, 1.89473E+01, 1.9887E+01, 2.1978E+04, &
      2.0816661E-01, 1.90549E+01, 1.9946E+01, 2.1933E+04, &
      2.0920745E-01, 1.91626E+01, 2.0003E+01, 2.1887E+04, &
      2.1025348E-01, 1.92702E+01, 2.0060E+01, 2.1839E+04, &
      2.1130475E-01, 1.93779E+01, 2.0115E+01, 2.1791E+04, &
      2.1236128E-01, 1.94855E+01, 2.0170E+01, 2.1741E+04, &
      2.1342308E-01, 1.95931E+01, 2.0223E+01, 2.1690E+04, &
      2.1449020E-01, 1.97004E+01, 2.0276E+01, 2.1638E+04, &
      2.1556265E-01, 1.98077E+01, 2.0327E+01, 2.1586E+04, &
      2.1664046E-01, 1.99147E+01, 2.0378E+01, 2.1532E+04, &
      2.1772366E-01, 2.00214E+01, 2.0427E+01, 2.1477E+04, &
      2.1881228E-01, 2.01277E+01, 2.0476E+01, 2.1421E+04, &
      2.1990634E-01, 2.02337E+01, 2.0524E+01, 2.1364E+04, &
      2.2100588E-01, 2.03392E+01, 2.0570E+01, 2.1306E+04, &
      2.2211090E-01, 2.04442E+01, 2.0616E+01, 2.1247E+04, &
      2.2322146E-01, 2.05485E+01, 2.0661E+01, 2.1187E+04, &
      2.2433757E-01, 2.06521E+01, 2.0704E+01, 2.1126E+04, &
      2.2545925E-01, 2.07549E+01, 2.0747E+01, 2.1065E+04, &
      2.2658655E-01, 2.08567E+01, 2.0789E+01, 2.1002E+04, &
      2.2771948E-01, 2.09573E+01, 2.0830E+01, 2.0939E+04, &
      2.2885808E-01, 2.10567E+01, 2.0870E+01, 2.0875E+04, &
      2.3000237E-01, 2.11545E+01, 2.0909E+01, 2.0810E+04, &
      2.3115238E-01, 2.12506E+01, 2.0947E+01, 2.0744E+04, &
      2.3230814E-01, 2.13466E+01, 2.0984E+01, 2.0677E+04, &
      2.3346969E-01, 2.14379E+01, 2.1020E+01, 2.0610E+04, &
      2.3463703E-01, 2.15261E+01, 2.1056E+01, 2.0542E+04, &
      2.3581022E-01, 2.16104E+01, 2.1090E+01, 2.0473E+04, &
      2.3698927E-01, 2.16897E+01, 2.1124E+01, 2.0404E+04, &
      2.3817422E-01, 2.17625E+01, 2.1156E+01, 2.0334E+04, &
      2.3936509E-01, 2.18262E+01, 2.1188E+01, 2.0263E+04, &
      2.4056191E-01, 2.18767E+01, 2.1219E+01, 2.0191E+04, &
      2.4176472E-01, 2.19062E+01, 2.1249E+01, 2.0119E+04, &
      2.4297355E-01, 2.18968E+01, 2.1278E+01, 2.0046E+04, &
      2.4418841E-01, 2.17913E+01, 2.1306E+01, 1.9973E+04, &
      2.4521055E-01, 2.12760E+01, 2.1328E+01, 1.9911E+04, &
      2.4540936E-01, 2.02858E+01, 2.2386E+01, 2.0881E+04, &
      2.4558945E-01, 2.13122E+01, 2.2389E+01, 2.0869E+04, &
      2.4663640E-01, 2.20383E+01, 2.2404E+01, 2.0794E+04, &
      2.4786959E-01, 2.23776E+01, 2.2417E+01, 2.0702E+04, &
      2.4910893E-01, 2.26171E+01, 2.2428E+01, 2.0610E+04, &
      2.5035448E-01, 2.28125E+01, 2.2439E+01, 2.0518E+04, &
      2.5160625E-01, 2.29801E+01, 2.2449E+01, 2.0424E+04, &
      2.5286428E-01, 2.31256E+01, 2.2457E+01, 2.0330E+04, &
      2.5412860E-01, 2.32494E+01, 2.2465E+01, 2.0236E+04, &
      2.5539925E-01, 2.33463E+01, 2.2471E+01, 2.0141E+04, &
      2.5667624E-01, 2.33987E+01, 2.2476E+01, 2.0045E+04, &
      2.5795962E-01, 2.33312E+01, 2.2480E+01, 1.9949E+04, &
      2.5859581E-01, 2.30534E+01, 2.2481E+01, 1.9901E+04, &
      2.5900419E-01, 2.31065E+01, 2.3281E+01, 2.0577E+04, &
      2.5924942E-01, 2.33386E+01, 2.3281E+01, 2.0557E+04, &
      2.6054567E-01, 2.38455E+01, 2.3279E+01, 2.0453E+04, &
      2.6184840E-01, 2.41448E+01, 2.3275E+01, 2.0348E+04, &
      2.6315764E-01, 2.43900E+01, 2.3271E+01, 2.0243E+04, &
      2.6447343E-01, 2.46085E+01, 2.3265E+01, 2.0137E+04, &
      2.6579579E-01, 2.48106E+01, 2.3258E+01, 2.0031E+04, &
      2.6712477E-01, 2.50014E+01, 2.3251E+01, 1.9925E+04, &
      2.6846040E-01, 2.51835E+01, 2.3242E+01, 1.9819E+04, &
      2.6980270E-01, 2.53587E+01, 2.3232E+01, 1.9712E+04, &
      2.7115171E-01, 2.55282E+01, 2.3222E+01, 1.9605E+04, &
      2.7250747E-01, 2.56928E+01, 2.3210E+01, 1.9498E+04, &
      2.7387001E-01, 2.58531E+01, 2.3199E+01, 1.9391E+04, &
      2.7523936E-01, 2.60099E+01, 2.3186E+01, 1.9284E+04, &
      2.7661556E-01, 2.61634E+01, 2.3173E+01, 1.9178E+04, &
      2.7799863E-01, 2.63140E+01, 2.3160E+01, 1.9071E+04, &
      2.7938863E-01, 2.64619E+01, 2.3145E+01, 1.8965E+04, &
      2.8078557E-01, 2.66073E+01, 2.3130E+01, 1.8858E+04, &
      2.8218950E-01, 2.67504E+01, 2.3115E+01, 1.8752E+04, &
      2.8360044E-01, 2.68914E+01, 2.3099E+01, 1.8646E+04, &
      2.8501845E-01, 2.70301E+01, 2.3082E+01, 1.8539E+04, &
      2.8644354E-01, 2.71668E+01, 2.3065E+01, 1.8434E+04, &
      2.8787576E-01, 2.73016E+01, 2.3048E+01, 1.8328E+04, &
      2.8931514E-01, 2.74346E+01, 2.3030E+01, 1.8223E+04, &
      2.9076171E-01, 2.75660E+01, 2.3012E+01, 1.8118E+04, &
      2.9221552E-01, 2.76959E+01, 2.2994E+01, 1.8014E+04, &
      2.9367660E-01, 2.78244E+01, 2.2975E+01, 1.7910E+04, &
      2.9514498E-01, 2.79514E+01, 2.2957E+01, 1.7806E+04, &
      2.9662071E-01, 2.80772E+01, 2.2938E+01, 1.7703E+04, &
      2.9810381E-01, 2.82017E+01, 2.2918E+01, 1.7600E+04, &
      2.9959433E-01, 2.83250E+01, 2.2898E+01, 1.7497E+04, &
      3.0109230E-01, 2.84471E+01, 2.2878E+01, 1.7395E+04, &
      3.0259776E-01, 2.85681E+01, 2.2858E+01, 1.7293E+04, &
      3.0411075E-01, 2.86879E+01, 2.2837E+01, 1.7192E+04, &
      3.0563130E-01, 2.88067E+01, 2.2816E+01, 1.7091E+04, &
      3.0715946E-01, 2.89245E+01, 2.2795E+01, 1.6990E+04, &
      3.0869526E-01, 2.90412E+01, 2.2774E+01, 1.6889E+04, &
      3.1023873E-01, 2.91569E+01, 2.2752E+01, 1.6789E+04, &
      3.1178993E-01, 2.92715E+01, 2.2729E+01, 1.6689E+04, &
      3.1334888E-01, 2.93853E+01, 2.2707E+01, 1.6590E+04, &
      3.1491562E-01, 2.94980E+01, 2.2684E+01, 1.6491E+04, &
      3.1649020E-01, 2.96098E+01, 2.2661E+01, 1.6392E+04, &
      3.1807265E-01, 2.97206E+01, 2.2638E+01, 1.6294E+04, &
      3.1966301E-01, 2.98305E+01, 2.2614E+01, 1.6196E+04, &
      3.2126133E-01, 2.99395E+01, 2.2590E+01, 1.6098E+04, &
      3.2286764E-01, 3.00476E+01, 2.2566E+01, 1.6001E+04, &
      3.2448197E-01, 3.01547E+01, 2.2541E+01, 1.5904E+04, &
      3.2610438E-01, 3.02609E+01, 2.2516E+01, 1.5807E+04, &
      3.2773491E-01, 3.03663E+01, 2.2491E+01, 1.5711E+04, &
      3.2937358E-01, 3.04707E+01, 2.2466E+01, 1.5615E+04, &
      3.3102045E-01, 3.05742E+01, 2.2440E+01, 1.5520E+04, &
      3.3267555E-01, 3.06768E+01, 2.2414E+01, 1.5425E+04, &
      3.3433893E-01, 3.07785E+01, 2.2388E+01, 1.5330E+04, &
      3.3601062E-01, 3.08793E+01, 2.2362E+01, 1.5236E+04, &
      3.3769068E-01, 3.09792E+01, 2.2335E+01, 1.5142E+04, &
      3.3937913E-01, 3.10782E+01, 2.2308E+01, 1.5049E+04, &
      3.4107602E-01, 3.11763E+01, 2.2281E+01, 1.4956E+04, &
      3.4278140E-01, 3.12735E+01, 2.2254E+01, 1.4864E+04, &
      3.4449531E-01, 3.13697E+01, 2.2226E+01, 1.4771E+04, &
      3.4621779E-01, 3.14650E+01, 2.2199E+01, 1.4680E+04, &
      3.4794888E-01, 3.15594E+01, 2.2171E+01, 1.4588E+04, &
      3.4968862E-01, 3.16529E+01, 2.2143E+01, 1.4497E+04, &
      3.5143706E-01, 3.17454E+01, 2.2114E+01, 1.4407E+04, &
      3.5319425E-01, 3.18370E+01, 2.2086E+01, 1.4317E+04, &
      3.5496022E-01, 3.19276E+01, 2.2057E+01, 1.4227E+04, &
      3.5673502E-01, 3.20171E+01, 2.2029E+01, 1.4138E+04, &
      3.5851870E-01, 3.21057E+01, 2.2000E+01, 1.4049E+04, &
      3.6031129E-01, 3.21933E+01, 2.1971E+01, 1.3961E+04, &
      3.6211285E-01, 3.22799E+01, 2.1941E+01, 1.3873E+04, &
      3.6392341E-01, 3.23654E+01, 2.1912E+01, 1.3786E+04, &
      3.6574303E-01, 3.24498E+01, 2.1882E+01, 1.3699E+04, &
      3.6757174E-01, 3.25330E+01, 2.1853E+01, 1.3612E+04, &
      3.6940960E-01, 3.26152E+01, 2.1823E+01, 1.3526E+04, &
      3.7125665E-01, 3.26962E+01, 2.1793E+01, 1.3440E+04, &
      3.7311293E-01, 3.27759E+01, 2.1763E+01, 1.3355E+04, &
      3.7497850E-01, 3.28544E+01, 2.1733E+01, 1.3270E+04, &
      3.7685339E-01, 3.29316E+01, 2.1703E+01, 1.3186E+04, &
      3.7873766E-01, 3.30075E+01, 2.1673E+01, 1.3102E+04, &
      3.8063135E-01, 3.30818E+01, 2.1642E+01, 1.3019E+04, &
      3.8253450E-01, 3.31547E+01, 2.1612E+01, 1.2936E+04, &
      3.8444718E-01, 3.32260E+01, 2.1581E+01, 1.2853E+04, &
      3.8636941E-01, 3.32956E+01, 2.1551E+01, 1.2771E+04, &
      3.8830126E-01, 3.33634E+01, 2.1520E+01, 1.2690E+04, &
      3.9024276E-01, 3.34292E+01, 2.1489E+01, 1.2609E+04, &
      3.9219398E-01, 3.34929E+01, 2.1459E+01, 1.2528E+04, &
      3.9415495E-01, 3.35543E+01, 2.1428E+01, 1.2448E+04, &
      3.9612572E-01, 3.36132E+01, 2.1397E+01, 1.2368E+04, &
      3.9810635E-01, 3.36692E+01, 2.1366E+01, 1.2289E+04, &
      4.0009688E-01, 3.37221E+01, 2.1335E+01, 1.2210E+04, &
      4.0209737E-01, 3.37712E+01, 2.1304E+01, 1.2132E+04, &
      4.0410785E-01, 3.38162E+01, 2.1273E+01, 1.2054E+04, &
      4.0612839E-01, 3.38562E+01, 2.1242E+01, 1.1977E+04, &
      4.0815904E-01, 3.38902E+01, 2.1211E+01, 1.1900E+04, &
      4.1019983E-01, 3.39168E+01, 2.1180E+01, 1.1823E+04, &
      4.1225083E-01, 3.39340E+01, 2.1149E+01, 1.1747E+04, &
      4.1431208E-01, 3.39387E+01, 2.1118E+01, 1.1671E+04, &
      4.1638364E-01, 3.39257E+01, 2.1087E+01, 1.1596E+04, &
      4.1846556E-01, 3.38860E+01, 2.1055E+01, 1.1522E+04, &
      4.2055789E-01, 3.37998E+01, 2.1024E+01, 1.1447E+04, &
      4.2266068E-01, 3.36117E+01, 2.0993E+01, 1.1374E+04, &
      4.2477398E-01, 3.29521E+01, 2.0962E+01, 1.1300E+04, &
      4.2477688E-01, 3.29497E+01, 2.0962E+01, 1.1300E+04, &
      4.2582312E-01, 3.29876E+01, 2.2384E+01, 1.2037E+04, &
      4.2689785E-01, 3.35375E+01, 2.2370E+01, 1.1999E+04, &
      4.2903234E-01, 3.40037E+01, 2.2343E+01, 1.1925E+04, &
      4.3117750E-01, 3.42898E+01, 2.2315E+01, 1.1851E+04, &
      4.3333339E-01, 3.45110E+01, 2.2288E+01, 1.1777E+04, &
      4.3550006E-01, 3.46984E+01, 2.2260E+01, 1.1704E+04, &
      4.3767756E-01, 3.48647E+01, 2.2232E+01, 1.1632E+04, &
      4.3986595E-01, 3.50166E+01, 2.2205E+01, 1.1560E+04, &
      4.4206528E-01, 3.51580E+01, 2.2177E+01, 1.1488E+04, &
      4.4427560E-01, 3.52914E+01, 2.2150E+01, 1.1417E+04, &
      4.4649698E-01, 3.54183E+01, 2.2122E+01, 1.1346E+04, &
      4.4872947E-01, 3.55398E+01, 2.2095E+01, 1.1275E+04, &
      4.5097311E-01, 3.56569E+01, 2.2067E+01, 1.1205E+04, &
      4.5322798E-01, 3.57701E+01, 2.2040E+01, 1.1136E+04, &
      4.5549412E-01, 3.58798E+01, 2.2012E+01, 1.1067E+04, &
      4.5777159E-01, 3.59865E+01, 2.1985E+01, 1.0998E+04, &
      4.6006045E-01, 3.60903E+01, 2.1957E+01, 1.0930E+04, &
      4.6236075E-01, 3.61914E+01, 2.1930E+01, 1.0862E+04, &
      4.6467255E-01, 3.62899E+01, 2.1902E+01, 1.0794E+04, &
      4.6699592E-01, 3.63859E+01, 2.1875E+01, 1.0727E+04, &
      4.6933090E-01, 3.64793E+01, 2.1847E+01, 1.0660E+04, &
      4.7167755E-01, 3.65701E+01, 2.1820E+01, 1.0594E+04, &
      4.7403594E-01, 3.66579E+01, 2.1793E+01, 1.0528E+04, &
      4.7640612E-01, 3.67424E+01, 2.1765E+01, 1.0463E+04, &
      4.7878815E-01, 3.68229E+01, 2.1737E+01, 1.0397E+04, &
      4.8118209E-01, 3.68983E+01, 2.1710E+01, 1.0333E+04, &
      4.8358800E-01, 3.69663E+01, 2.1682E+01, 1.0268E+04, &
      4.8600594E-01, 3.70224E+01, 2.1655E+01, 1.0204E+04, &
      4.8843597E-01, 3.70537E+01, 2.1627E+01, 1.0141E+04, &
      4.9087815E-01, 3.69815E+01, 2.1599E+01, 1.0077E+04, &
      4.9092160E-01, 3.69762E+01, 2.1599E+01, 1.0076E+04, &
      4.9227841E-01, 3.70276E+01, 2.1937E+01, 1.0206E+04, &
      4.9333254E-01, 3.71730E+01, 2.1926E+01, 1.0179E+04, &
      4.9579920E-01, 3.73657E+01, 2.1900E+01, 1.0116E+04, &
      4.9827820E-01, 3.75106E+01, 2.1874E+01, 1.0054E+04, &
      5.0076959E-01, 3.76386E+01, 2.1848E+01, 9.9921E+03, &
      5.0327344E-01, 3.77577E+01, 2.1822E+01, 9.9305E+03, &
      5.0578980E-01, 3.78710E+01, 2.1796E+01, 9.8693E+03, &
      5.0831875E-01, 3.79804E+01, 2.1769E+01, 9.8084E+03, &
      5.1086035E-01, 3.80868E+01, 2.1743E+01, 9.7479E+03, &
      5.1341465E-01, 3.81908E+01, 2.1717E+01, 9.6876E+03, &
      5.1598172E-01, 3.82928E+01, 2.1690E+01, 9.6278E+03, &
      5.1856163E-01, 3.83931E+01, 2.1664E+01, 9.5682E+03, &
      5.2115444E-01, 3.84920E+01, 2.1637E+01, 9.5090E+03, &
      5.2376021E-01, 3.85895E+01, 2.1610E+01, 9.4501E+03, &
      5.2637901E-01, 3.86859E+01, 2.1583E+01, 9.3915E+03, &
      5.2901091E-01, 3.87811E+01, 2.1557E+01, 9.3332E+03, &
      5.3165596E-01, 3.88754E+01, 2.1530E+01, 9.2752E+03, &
      5.3431424E-01, 3.89686E+01, 2.1503E+01, 9.2175E+03, &
      5.3698581E-01, 3.90609E+01, 2.1476E+01, 9.1602E+03, &
      5.3967074E-01, 3.91523E+01, 2.1448E+01, 9.1031E+03, &
      5.4236910E-01, 3.92428E+01, 2.1421E+01, 9.0464E+03, &
      5.4508094E-01, 3.93323E+01, 2.1394E+01, 8.9899E+03, &
      5.4780635E-01, 3.94208E+01, 2.1366E+01, 8.9338E+03, &
      5.5054538E-01, 3.95084E+01, 2.1338E+01, 8.8779E+03, &
      5.5329810E-01, 3.95949E+01, 2.1311E+01, 8.8223E+03, &
      5.5606460E-01, 3.96803E+01, 2.1283E+01, 8.7670E+03, &
      5.5884492E-01, 3.97645E+01, 2.1255E+01, 8.7120E+03, &
      5.6163914E-01, 3.98473E+01, 2.1227E+01, 8.6572E+03, &
      5.6444734E-01, 3.99286E+01, 2.1198E+01, 8.6028E+03, &
      5.6726958E-01, 4.00082E+01, 2.1170E+01, 8.5486E+03, &
      5.7010592E-01, 4.00856E+01, 2.1141E+01, 8.4946E+03, &
      5.7295645E-01, 4.01605E+01, 2.1113E+01, 8.4410E+03, &
      5.7582123E-01, 4.02323E+01, 2.1084E+01, 8.3876E+03, &
      5.7870034E-01, 4.02998E+01, 2.1055E+01, 8.3344E+03, &
      5.8159384E-01, 4.03616E+01, 2.1026E+01, 8.2816E+03, &
      5.8450181E-01, 4.04149E+01, 2.0997E+01, 8.2290E+03, &
      5.8742432E-01, 4.04542E+01, 2.0967E+01, 8.1766E+03, &
      5.9036144E-01, 4.04654E+01, 2.0937E+01, 8.1245E+03, &
      5.9331325E-01, 4.03880E+01, 2.0908E+01, 8.0727E+03, &
      5.9410753E-01, 4.03052E+01, 2.0900E+01, 8.0588E+03, &
      5.9589253E-01, 4.03616E+01, 2.1414E+01, 8.2324E+03, &
      5.9627982E-01, 4.04348E+01, 2.1411E+01, 8.2256E+03, &
      5.9926122E-01, 4.07326E+01, 2.1381E+01, 8.1735E+03, &
      6.0225752E-01, 4.09169E+01, 2.1352E+01, 8.1217E+03, &
      6.0526881E-01, 4.10698E+01, 2.1322E+01, 8.0702E+03, &
      6.0829515E-01, 4.12078E+01, 2.1292E+01, 8.0189E+03, &
      6.1133663E-01, 4.13369E+01, 2.1262E+01, 7.9678E+03, &
      6.1439331E-01, 4.14602E+01, 2.1232E+01, 7.9170E+03, &
      6.1746528E-01, 4.15795E+01, 2.1202E+01, 7.8664E+03, &
      6.2055260E-01, 4.16956E+01, 2.1171E+01, 7.8161E+03, &
      6.2365537E-01, 4.18095E+01, 2.1141E+01, 7.7660E+03, &
      6.2677364E-01, 4.19215E+01, 2.1110E+01, 7.7161E+03, &
      6.2990751E-01, 4.20320E+01, 2.1079E+01, 7.6665E+03, &
      6.3305705E-01, 4.21416E+01, 2.1036E+01, 7.6129E+03, &
      6.3622234E-01, 4.22496E+01, 2.0987E+01, 7.5574E+03, &
      6.3940345E-01, 4.23560E+01, 2.0937E+01, 7.5023E+03, &
      6.4260046E-01, 4.24610E+01, 2.0888E+01, 7.4475E+03, &
      6.4581347E-01, 4.25646E+01, 2.0839E+01, 7.3930E+03, &
      6.4904253E-01, 4.26670E+01, 2.0789E+01, 7.3389E+03, &
      6.5228775E-01, 4.27682E+01, 2.0740E+01, 7.2850E+03, &
      6.5554919E-01, 4.28684E+01, 2.0690E+01, 7.2315E+03, &
      6.5882693E-01, 4.29676E+01, 2.0640E+01, 7.1782E+03, &
      6.6212107E-01, 4.30658E+01, 2.0590E+01, 7.1253E+03, &
      6.6543167E-01, 4.31631E+01, 2.0540E+01, 7.0727E+03, &
      6.6875883E-01, 4.32595E+01, 2.0490E+01, 7.0203E+03, &
      6.7210262E-01, 4.33552E+01, 2.0439E+01, 6.9683E+03, &
      6.7546314E-01, 4.34500E+01, 2.0389E+01, 6.9165E+03, &
      6.7884045E-01, 4.35441E+01, 2.0338E+01, 6.8650E+03, &
      6.8223466E-01, 4.36375E+01, 2.0287E+01, 6.8138E+03, &
      6.8564583E-01, 4.37303E+01, 2.0235E+01, 6.7627E+03, &
      6.8907406E-01, 4.38225E+01, 2.0172E+01, 6.7082E+03, &
      6.9251943E-01, 4.39136E+01, 2.0109E+01, 6.6541E+03, &
      6.9598202E-01, 4.40035E+01, 2.0046E+01, 6.6004E+03, &
      6.9946194E-01, 4.40924E+01, 1.9983E+01, 6.5470E+03, &
      7.0295924E-01, 4.41802E+01, 1.9920E+01, 6.4939E+03, &
      7.0647404E-01, 4.42670E+01, 1.9857E+01, 6.4412E+03, &
      7.1000641E-01, 4.43529E+01, 1.9794E+01, 6.3889E+03, &
      7.1355644E-01, 4.44378E+01, 1.9730E+01, 6.3368E+03, &
      7.1712423E-01, 4.45218E+01, 1.9667E+01, 6.2852E+03, &
      7.2070985E-01, 4.46048E+01, 1.9603E+01, 6.2336E+03, &
      7.2431340E-01, 4.46867E+01, 1.9538E+01, 6.1822E+03, &
      7.2793496E-01, 4.47676E+01, 1.9474E+01, 6.1312E+03, &
      7.3157464E-01, 4.48474E+01, 1.9409E+01, 6.0805E+03, &
      7.3523251E-01, 4.49262E+01, 1.9344E+01, 6.0301E+03, &
      7.3890867E-01, 4.50187E+01, 1.9279E+01, 5.9801E+03, &
      7.4260322E-01, 4.50957E+01, 1.9214E+01, 5.9303E+03, &
      7.4631623E-01, 4.51718E+01, 1.9149E+01, 5.8810E+03, &
      7.5004781E-01, 4.52469E+01, 1.9084E+01, 5.8319E+03, &
      7.5379805E-01, 4.53211E+01, 1.9019E+01, 5.7832E+03, &
      7.5756704E-01, 4.53944E+01, 1.8954E+01, 5.7348E+03, &
      7.6135488E-01, 4.54667E+01, 1.8889E+01, 5.6868E+03, &
      7.6516165E-01, 4.55382E+01, 1.8823E+01, 5.6390E+03, &
      7.6898746E-01, 4.56088E+01, 1.8758E+01, 5.5916E+03, &
      7.7283240E-01, 4.56786E+01, 1.8693E+01, 5.5445E+03, &
      7.7669656E-01, 4.57586E+01, 1.8628E+01, 5.4978E+03, &
      7.8058004E-01, 4.58268E+01, 1.8562E+01, 5.4513E+03, &
      7.8448294E-01, 4.58941E+01, 1.8497E+01, 5.4052E+03, &
      7.8840536E-01, 4.59606E+01, 1.8432E+01, 5.3594E+03, &
      7.9234738E-01, 4.60263E+01, 1.8366E+01, 5.3140E+03, &
      7.9630912E-01, 4.60912E+01, 1.8301E+01, 5.2688E+03, &
      8.0029067E-01, 4.61553E+01, 1.8236E+01, 5.2240E+03, &
      8.0429212E-01, 4.62186E+01, 1.8170E+01, 5.1795E+03, &
      8.0831358E-01, 4.62811E+01, 1.8105E+01, 5.1353E+03, &
      8.1235515E-01, 4.63428E+01, 1.8040E+01, 5.0914E+03, &
      8.1641693E-01, 4.64038E+01, 1.7975E+01, 5.0479E+03, &
      8.2049901E-01, 4.64640E+01, 1.7909E+01, 5.0046E+03, &
      8.2460150E-01, 4.65235E+01, 1.7844E+01, 4.9617E+03, &
      8.2872451E-01, 4.65823E+01, 1.7779E+01, 4.9190E+03, &
      8.3286813E-01, 4.66403E+01, 1.7713E+01, 4.8767E+03, &
      8.3703248E-01, 4.66975E+01, 1.7648E+01, 4.8345E+03, &
      8.4121764E-01, 4.67539E+01, 1.7582E+01, 4.7927E+03, &
      8.4542373E-01, 4.68096E+01, 1.7517E+01, 4.7512E+03, &
      8.4965084E-01, 4.68645E+01, 1.7451E+01, 4.7100E+03, &
      8.5389910E-01, 4.69187E+01, 1.7386E+01, 4.6691E+03, &
      8.5816859E-01, 4.69721E+01, 1.7320E+01, 4.6285E+03, &
      8.6245944E-01, 4.70247E+01, 1.7255E+01, 4.5881E+03, &
      8.6677173E-01, 4.70767E+01, 1.7190E+01, 4.5481E+03, &
      8.7110559E-01, 4.71279E+01, 1.7124E+01, 4.5084E+03, &
      8.7546112E-01, 4.71784E+01, 1.7059E+01, 4.4690E+03, &
      8.7983843E-01, 4.72281E+01, 1.6994E+01, 4.4299E+03, &
      8.8423762E-01, 4.72772E+01, 1.6929E+01, 4.3911E+03, &
      8.8865881E-01, 4.73256E+01, 1.6864E+01, 4.3525E+03, &
      8.9310210E-01, 4.73733E+01, 1.6799E+01, 4.3143E+03, &
      8.9756761E-01, 4.74203E+01, 1.6734E+01, 4.2763E+03, &
      9.0205545E-01, 4.74666E+01, 1.6669E+01, 4.2387E+03, &
      9.0656573E-01, 4.75122E+01, 1.6605E+01, 4.2013E+03, &
      9.1109856E-01, 4.75572E+01, 1.6540E+01, 4.1642E+03, &
      9.1565405E-01, 4.76016E+01, 1.6475E+01, 4.1275E+03, &
      9.2023232E-01, 4.76453E+01, 1.6411E+01, 4.0910E+03, &
      9.2483348E-01, 4.76884E+01, 1.6346E+01, 4.0547E+03, &
      9.2945765E-01, 4.77309E+01, 1.6282E+01, 4.0188E+03, &
      9.3410494E-01, 4.77727E+01, 1.6218E+01, 3.9831E+03, &
      9.3877546E-01, 4.78140E+01, 1.6154E+01, 3.9478E+03, &
      9.4346934E-01, 4.78547E+01, 1.6090E+01, 3.9127E+03, &
      9.4818668E-01, 4.78948E+01, 1.6026E+01, 3.8779E+03, &
      9.5292762E-01, 4.79344E+01, 1.5962E+01, 3.8433E+03, &
      9.5769226E-01, 4.79735E+01, 1.5899E+01, 3.8091E+03, &
      9.6248072E-01, 4.80121E+01, 1.5835E+01, 3.7751E+03, &
      9.6729312E-01, 4.80502E+01, 1.5772E+01, 3.7414E+03, &
      9.7212959E-01, 4.80878E+01, 1.5709E+01, 3.7079E+03, &
      9.7699023E-01, 4.81250E+01, 1.5646E+01, 3.6748E+03, &
      9.8187519E-01, 4.81618E+01, 1.5583E+01, 3.6419E+03, &
      9.8678456E-01, 4.81983E+01, 1.5520E+01, 3.6093E+03, &
      9.9171848E-01, 4.82345E+01, 1.5457E+01, 3.5769E+03, &
      9.9667708E-01, 4.82704E+01, 1.5395E+01, 3.5448E+03, &
      1.0016605E+00, 4.86747E+01, 1.5321E+01, 3.5105E+03, &
      1.0066688E+00, 4.87529E+01, 1.5226E+01, 3.4715E+03, &
      1.0117021E+00, 4.88078E+01, 1.5131E+01, 3.4328E+03, &
      1.0167606E+00, 4.88558E+01, 1.5038E+01, 3.3947E+03, &
      1.0218444E+00, 4.88986E+01, 1.4944E+01, 3.3570E+03, &
      1.0269536E+00, 4.89373E+01, 1.4852E+01, 3.3198E+03, &
      1.0320884E+00, 4.89722E+01, 1.4760E+01, 3.2830E+03, &
      1.0372489E+00, 4.90039E+01, 1.4667E+01, 3.2460E+03, &
      1.0424351E+00, 4.90320E+01, 1.4569E+01, 3.2086E+03, &
      1.0476473E+00, 4.90568E+01, 1.4473E+01, 3.1716E+03, &
      1.0528855E+00, 4.90786E+01, 1.4377E+01, 3.1351E+03, &
      1.0581499E+00, 4.90975E+01, 1.4282E+01, 3.0990E+03, &
      1.0634407E+00, 4.91137E+01, 1.4188E+01, 3.0635E+03, &
      1.0687579E+00, 4.91273E+01, 1.4095E+01, 3.0283E+03, &
      1.0741017E+00, 4.91385E+01, 1.4003E+01, 2.9937E+03, &
      1.0794722E+00, 4.91474E+01, 1.3911E+01, 2.9594E+03, &
      1.0848695E+00, 4.91540E+01, 1.3821E+01, 2.9256E+03, &
      1.0902939E+00, 4.91585E+01, 1.3731E+01, 2.8923E+03, &
      1.0957454E+00, 4.91609E+01, 1.3642E+01, 2.8593E+03, &
      1.1012241E+00, 4.91614E+01, 1.3553E+01, 2.8268E+03, &
      1.1067302E+00, 4.91599E+01, 1.3466E+01, 2.7947E+03, &
      1.1122639E+00, 4.91566E+01, 1.3379E+01, 2.7630E+03, &
      1.1178252E+00, 4.91514E+01, 1.3293E+01, 2.7317E+03, &
      1.1234143E+00, 4.91444E+01, 1.3207E+01, 2.7008E+03, &
      1.1290314E+00, 4.91357E+01, 1.3123E+01, 2.6702E+03, &
      1.1346765E+00, 4.91252E+01, 1.3039E+01, 2.6401E+03, &
      1.1403499E+00, 4.91131E+01, 1.2956E+01, 2.6104E+03, &
      1.1460517E+00, 4.90993E+01, 1.2873E+01, 2.5810E+03, &
      1.1517819E+00, 4.90838E+01, 1.2792E+01, 2.5520E+03, &
      1.1575408E+00, 4.90668E+01, 1.2711E+01, 2.5233E+03, &
      1.1633285E+00, 4.90481E+01, 1.2630E+01, 2.4950E+03, &
      1.1691452E+00, 4.90278E+01, 1.2551E+01, 2.4671E+03, &
      1.1749909E+00, 4.90060E+01, 1.2472E+01, 2.4395E+03, &
      1.1808659E+00, 4.89825E+01, 1.2394E+01, 2.4123E+03, &
      1.1867702E+00, 4.89575E+01, 1.2316E+01, 2.3854E+03, &
      1.1927040E+00, 4.89309E+01, 1.2239E+01, 2.3588E+03, &
      1.1986676E+00, 4.89027E+01, 1.2163E+01, 2.3326E+03, &
      1.2046609E+00, 4.88730E+01, 1.2087E+01, 2.3067E+03, &
      1.2106842E+00, 4.88417E+01, 1.2012E+01, 2.2811E+03, &
      1.2167376E+00, 4.88088E+01, 1.1938E+01, 2.2558E+03, &
      1.2228213E+00, 4.87743E+01, 1.1864E+01, 2.2309E+03, &
      1.2289354E+00, 4.87382E+01, 1.1791E+01, 2.2062E+03, &
      1.2350801E+00, 4.87005E+01, 1.1719E+01, 2.1819E+03, &
      1.2412555E+00, 4.86611E+01, 1.1647E+01, 2.1578E+03, &
      1.2474618E+00, 4.86201E+01, 1.1576E+01, 2.1341E+03, &
      1.2536991E+00, 4.85775E+01, 1.1505E+01, 2.1106E+03, &
      1.2599676E+00, 4.85332E+01, 1.1435E+01, 2.0875E+03, &
      1.2662674E+00, 4.84871E+01, 1.1365E+01, 2.0646E+03, &
      1.2725988E+00, 4.84394E+01, 1.1297E+01, 2.0420E+03, &
      1.2789618E+00, 4.83920E+01, 1.1228E+01, 2.0197E+03, &
      1.2853566E+00, 4.83407E+01, 1.1161E+01, 1.9976E+03, &
      1.2917833E+00, 4.82876E+01, 1.1093E+01, 1.9758E+03, &
      1.2982423E+00, 4.82326E+01, 1.1027E+01, 1.9543E+03, &
      1.3047335E+00, 4.81757E+01, 1.0961E+01, 1.9331E+03, &
      1.3112571E+00, 4.81170E+01, 1.0895E+01, 1.9121E+03, &
      1.3178134E+00, 4.80562E+01, 1.0830E+01, 1.8913E+03, &
      1.3244025E+00, 4.79934E+01, 1.0766E+01, 1.8708E+03, &
      1.3310245E+00, 4.79286E+01, 1.0702E+01, 1.8506E+03, &
      1.3376796E+00, 4.78617E+01, 1.0638E+01, 1.8306E+03, &
      1.3443680E+00, 4.77926E+01, 1.0575E+01, 1.8109E+03, &
      1.3510899E+00, 4.77213E+01, 1.0513E+01, 1.7913E+03, &
      1.3578453E+00, 4.76477E+01, 1.0451E+01, 1.7721E+03, &
      1.3646345E+00, 4.75717E+01, 1.0390E+01, 1.7530E+03, &
      1.3714577E+00, 4.74934E+01, 1.0329E+01, 1.7342E+03, &
      1.3783150E+00, 4.74125E+01, 1.0269E+01, 1.7156E+03, &
      1.3852066E+00, 4.73291E+01, 1.0209E+01, 1.6973E+03, &
      1.3921326E+00, 4.72430E+01, 1.0149E+01, 1.6791E+03, &
      1.3990933E+00, 4.71542E+01, 1.0091E+01, 1.6612E+03, &
      1.4060887E+00, 4.70625E+01, 1.0032E+01, 1.6435E+03, &
      1.4131192E+00, 4.69679E+01, 9.9742E+00, 1.6260E+03, &
      1.4201848E+00, 4.68702E+01, 9.9168E+00, 1.6087E+03, &
      1.4272857E+00, 4.67693E+01, 9.8598E+00, 1.5916E+03, &
      1.4344221E+00, 4.66652E+01, 9.8032E+00, 1.5747E+03, &
      1.4415942E+00, 4.65576E+01, 9.7471E+00, 1.5580E+03, &
      1.4488022E+00, 4.64465E+01, 9.6915E+00, 1.5415E+03, &
      1.4560462E+00, 4.63316E+01, 9.6362E+00, 1.5252E+03, &
      1.4633265E+00, 4.62129E+01, 9.5814E+00, 1.5091E+03, &
      1.4706431E+00, 4.60901E+01, 9.5270E+00, 1.4932E+03, &
      1.4779963E+00, 4.59654E+01, 9.4730E+00, 1.4775E+03, &
      1.4853863E+00, 4.58339E+01, 9.4195E+00, 1.4620E+03, &
      1.4928132E+00, 4.56978E+01, 9.3664E+00, 1.4466E+03, &
      1.5002773E+00, 4.55567E+01, 9.3137E+00, 1.4314E+03, &
      1.5077787E+00, 4.54105E+01, 9.2615E+00, 1.4164E+03, &
      1.5153176E+00, 4.52588E+01, 9.2096E+00, 1.4016E+03, &
      1.5228942E+00, 4.51013E+01, 9.1582E+00, 1.3870E+03, &
      1.5305086E+00, 4.49378E+01, 9.1072E+00, 1.3725E+03, &
      1.5381612E+00, 4.47679E+01, 9.0565E+00, 1.3582E+03, &
      1.5458520E+00, 4.45911E+01, 9.0063E+00, 1.3440E+03, &
      1.5535812E+00, 4.44071E+01, 8.9565E+00, 1.3301E+03, &
      1.5613491E+00, 4.42154E+01, 8.9070E+00, 1.3163E+03, &
      1.5691559E+00, 4.40154E+01, 8.8580E+00, 1.3026E+03, &
      1.5770017E+00, 4.38066E+01, 8.8093E+00, 1.2891E+03, &
      1.5848867E+00, 4.35884E+01, 8.7610E+00, 1.2758E+03, &
      1.5928111E+00, 4.33600E+01, 8.7131E+00, 1.2626E+03, &
      1.6007752E+00, 4.31207E+01, 8.6655E+00, 1.2496E+03, &
      1.6087790E+00, 4.28695E+01, 8.6183E+00, 1.2367E+03, &
      1.6168229E+00, 4.26055E+01, 8.5715E+00, 1.2240E+03, &
      1.6249070E+00, 4.23276E+01, 8.5250E+00, 1.2114E+03, &
      1.6330316E+00, 4.20344E+01, 8.4789E+00, 1.1990E+03, &
      1.6411967E+00, 4.17244E+01, 8.4332E+00, 1.1867E+03, &
      1.6494027E+00, 4.13961E+01, 8.3878E+00, 1.1746E+03, &
      1.6576497E+00, 4.10474E+01, 8.3428E+00, 1.1625E+03, &
      1.6659380E+00, 4.06759E+01, 8.2981E+00, 1.1507E+03, &
      1.6742677E+00, 4.02791E+01, 8.2537E+00, 1.1389E+03, &
      1.6826390E+00, 3.98536E+01, 8.2097E+00, 1.1273E+03, &
      1.6910522E+00, 3.93956E+01, 8.1660E+00, 1.1159E+03, &
      1.6995075E+00, 3.89003E+01, 8.1227E+00, 1.1045E+03, &
      1.7080050E+00, 3.83617E+01, 8.0781E+00, 1.0931E+03, &
      1.7165450E+00, 3.77723E+01, 8.0334E+00, 1.0818E+03, &
      1.7251278E+00, 3.71225E+01, 7.9890E+00, 1.0706E+03, &
      1.7337534E+00, 3.63998E+01, 7.9450E+00, 1.0595E+03, &
      1.7424222E+00, 3.55869E+01, 7.9014E+00, 1.0485E+03, &
      1.7511343E+00, 3.46597E+01, 7.8580E+00, 1.0377E+03, &
      1.7598899E+00, 3.35823E+01, 7.8150E+00, 1.0270E+03, &
      1.7686894E+00, 3.22981E+01, 7.7723E+00, 1.0164E+03, &
      1.7775328E+00, 3.07091E+01, 7.7299E+00, 1.0060E+03, &
      1.7864205E+00, 2.86218E+01, 7.6878E+00, 9.9561E+02, &
      1.7953526E+00, 2.55410E+01, 7.6460E+00, 9.8539E+02, &
      1.8043294E+00, 1.92952E+01, 7.6046E+00, 9.7528E+02, &
      1.8088491E+00, 4.06674E+00, 7.5839E+00, 9.7025E+02, &
      1.8095510E+00, 3.74869E+00, 2.5626E+01, 3.2520E+03, &
      1.8133510E+00, 1.78305E+01, 2.5549E+01, 3.2355E+03, &
      1.8224178E+00, 2.40826E+01, 2.5369E+01, 3.1968E+03, &
      1.8315299E+00, 2.65264E+01, 2.5189E+01, 3.1585E+03, &
      1.8406875E+00, 2.77107E+01, 2.5011E+01, 3.1207E+03, &
      1.8498909E+00, 2.80311E+01, 2.4835E+01, 3.0834E+03, &
      1.8591404E+00, 2.72913E+01, 2.4659E+01, 3.0465E+03, &
      1.8684361E+00, 2.32827E+01, 2.4485E+01, 3.0101E+03, &
      1.8709394E+00, 1.76545E+01, 2.4439E+01, 3.0004E+03, &
      1.8722607E+00, 1.76137E+01, 3.6298E+01, 4.4481E+03, &
      1.8777783E+00, 2.66817E+01, 3.6144E+01, 4.4162E+03, &
      1.8871672E+00, 3.10855E+01, 3.5884E+01, 4.3627E+03, &
      1.8966030E+00, 3.36780E+01, 3.5625E+01, 4.3099E+03, &
      1.9060860E+00, 3.56113E+01, 3.5369E+01, 4.2577E+03, &
      1.9156165E+00, 3.71838E+01, 3.5115E+01, 4.2062E+03, &
      1.9251945E+00, 3.85216E+01, 3.4862E+01, 4.1553E+03, &
      1.9348205E+00, 3.96913E+01, 3.4612E+01, 4.1051E+03, &
      1.9444946E+00, 4.07328E+01, 3.4363E+01, 4.0554E+03, &
      1.9542171E+00, 4.16726E+01, 3.4117E+01, 4.0064E+03, &
      1.9639882E+00, 4.25289E+01, 3.3872E+01, 3.9580E+03, &
      1.9738081E+00, 4.33150E+01, 3.3628E+01, 3.9101E+03, &
      1.9836772E+00, 4.40410E+01, 3.3386E+01, 3.8628E+03, &
      1.9935955E+00, 4.47146E+01, 3.3146E+01, 3.8160E+03, &
      2.0035635E+00, 4.53419E+01, 3.2907E+01, 3.7698E+03, &
      2.0135813E+00, 4.59281E+01, 3.2670E+01, 3.7242E+03, &
      2.0236492E+00, 4.64771E+01, 3.2436E+01, 3.6792E+03, &
      2.0337675E+00, 4.69922E+01, 3.2202E+01, 3.6347E+03, &
      2.0439363E+00, 4.74762E+01, 3.1971E+01, 3.5908E+03, &
      2.0541560E+00, 4.79314E+01, 3.1742E+01, 3.5474E+03, &
      2.0644268E+00, 4.83597E+01, 3.1514E+01, 3.5045E+03, &
      2.0747489E+00, 4.87627E+01, 3.1288E+01, 3.4622E+03, &
      2.0851227E+00, 4.91417E+01, 3.1063E+01, 3.4203E+03, &
      2.0955483E+00, 4.94977E+01, 3.0840E+01, 3.3791E+03, &
      2.1060260E+00, 4.98314E+01, 3.0619E+01, 3.3383E+03, &
      2.1165562E+00, 5.01433E+01, 3.0400E+01, 3.2980E+03, &
      2.1271389E+00, 5.04338E+01, 3.0182E+01, 3.2582E+03, &
      2.1377746E+00, 5.07027E+01, 2.9966E+01, 3.2189E+03, &
      2.1484635E+00, 5.09498E+01, 2.9752E+01, 3.1801E+03, &
      2.1592058E+00, 5.11743E+01, 2.9539E+01, 3.1418E+03, &
      2.1700018E+00, 5.13749E+01, 2.9328E+01, 3.1039E+03, &
      2.1808519E+00, 5.15496E+01, 2.9118E+01, 3.0665E+03, &
      2.1917561E+00, 5.16957E+01, 2.8910E+01, 3.0296E+03, &
      2.2027149E+00, 5.18086E+01, 2.8704E+01, 2.9931E+03, &
      2.2137285E+00, 5.18817E+01, 2.8499E+01, 2.9571E+03, &
      2.2247971E+00, 5.19041E+01, 2.8296E+01, 2.9215E+03, &
      2.2359211E+00, 5.18573E+01, 2.8094E+01, 2.8863E+03, &
      2.2471007E+00, 5.17061E+01, 2.7893E+01, 2.8516E+03, &
      2.2583362E+00, 5.13704E+01, 2.7695E+01, 2.8173E+03, &
      2.2696279E+00, 5.05932E+01, 2.7497E+01, 2.7834E+03, &
      2.2786505E+00, 4.83982E+01, 2.7341E+01, 2.7568E+03, &
      2.2809760E+00, 4.13917E+01, 2.7301E+01, 2.7500E+03, &
      2.2833493E+00, 4.84449E+01, 3.2090E+01, 3.2271E+03, &
      2.2923809E+00, 5.11010E+01, 3.1891E+01, 3.1946E+03, &
      2.3038428E+00, 5.24327E+01, 3.1641E+01, 3.1539E+03, &
      2.3153620E+00, 5.33017E+01, 3.1393E+01, 3.1138E+03, &
      2.3269388E+00, 5.39744E+01, 3.1148E+01, 3.0742E+03, &
      2.3385735E+00, 5.45346E+01, 3.0904E+01, 3.0351E+03, &
      2.3502664E+00, 5.50199E+01, 3.0663E+01, 2.9965E+03, &
      2.3620177E+00, 5.54489E+01, 3.0417E+01, 2.9579E+03, &
      2.3738278E+00, 5.58317E+01, 3.0172E+01, 2.9196E+03, &
      2.3856970E+00, 5.61749E+01, 2.9929E+01, 2.8817E+03, &
      2.3976254E+00, 5.64813E+01, 2.9696E+01, 2.8452E+03, &
      2.4096136E+00, 5.67657E+01, 2.9481E+01, 2.8107E+03, &
      2.4216616E+00, 5.70327E+01, 2.9270E+01, 2.7768E+03, &
      2.4337699E+00, 5.72825E+01, 2.9061E+01, 2.7433E+03, &
      2.4459388E+00, 5.75145E+01, 2.8850E+01, 2.7100E+03, &
      2.4581685E+00, 5.77286E+01, 2.8642E+01, 2.6772E+03, &
      2.4704593E+00, 5.79246E+01, 2.8436E+01, 2.6449E+03, &
      2.4828116E+00, 5.81017E+01, 2.8233E+01, 2.6130E+03, &
      2.4952257E+00, 5.82584E+01, 2.8033E+01, 2.5817E+03, &
      2.5077018E+00, 5.83915E+01, 2.7835E+01, 2.5508E+03, &
      2.5202403E+00, 5.84954E+01, 2.7639E+01, 2.5203E+03, &
      2.5328415E+00, 5.85591E+01, 2.7445E+01, 2.4903E+03, &
      2.5455057E+00, 5.85592E+01, 2.7253E+01, 2.4607E+03, &
      2.5582333E+00, 5.84289E+01, 2.7064E+01, 2.4316E+03, &
      2.5710244E+00, 5.77597E+01, 2.6876E+01, 2.4028E+03, &
      2.5716555E+00, 5.76629E+01, 2.6867E+01, 2.4014E+03, &
      2.5781443E+00, 5.77438E+01, 2.8665E+01, 2.5551E+03, &
      2.5838796E+00, 5.84515E+01, 2.8571E+01, 2.5411E+03, &
      2.5967990E+00, 5.91845E+01, 2.8362E+01, 2.5100E+03, &
      2.6097829E+00, 5.96478E+01, 2.8154E+01, 2.4793E+03, &
      2.6228319E+00, 6.00117E+01, 2.7949E+01, 2.4491E+03, &
      2.6359460E+00, 6.03202E+01, 2.7745E+01, 2.4193E+03, &
      2.6491257E+00, 6.05917E+01, 2.7544E+01, 2.3899E+03, &
      2.6623714E+00, 6.08352E+01, 2.7344E+01, 2.3609E+03, &
      2.6756832E+00, 6.10555E+01, 2.7146E+01, 2.3322E+03, &
      2.6890617E+00, 6.12549E+01, 2.6950E+01, 2.3039E+03, &
      2.7025070E+00, 6.14337E+01, 2.6756E+01, 2.2761E+03, &
      2.7160195E+00, 6.15963E+01, 2.6578E+01, 2.2498E+03, &
      2.7295996E+00, 6.17470E+01, 2.6402E+01, 2.2239E+03, &
      2.7432476E+00, 6.18834E+01, 2.6228E+01, 2.1984E+03, &
      2.7569638E+00, 6.20023E+01, 2.6057E+01, 2.1733E+03, &
      2.7707486E+00, 6.20982E+01, 2.5888E+01, 2.1485E+03, &
      2.7846024E+00, 6.21590E+01, 2.5720E+01, 2.1241E+03, &
      2.7985254E+00, 6.21531E+01, 2.5554E+01, 2.1000E+03, &
      2.8125180E+00, 6.19324E+01, 2.5390E+01, 2.0762E+03, &
      2.8140455E+00, 6.18632E+01, 2.5373E+01, 2.0737E+03, &
      2.8251547E+00, 6.19917E+01, 2.6383E+01, 2.1474E+03, &
      2.8265806E+00, 6.20919E+01, 2.6366E+01, 2.1450E+03, &
      2.8407135E+00, 6.26619E+01, 2.6201E+01, 2.1211E+03, &
      2.8549171E+00, 6.30133E+01, 2.6038E+01, 2.0975E+03, &
      2.8691917E+00, 6.32980E+01, 2.5876E+01, 2.0741E+03, &
      2.8835376E+00, 6.35490E+01, 2.5715E+01, 2.0511E+03, &
      2.8979553E+00, 6.37792E+01, 2.5556E+01, 2.0283E+03, &
      2.9124451E+00, 6.39949E+01, 2.5397E+01, 2.0057E+03, &
      2.9270073E+00, 6.42001E+01, 2.5239E+01, 1.9835E+03, &
      2.9416424E+00, 6.43972E+01, 2.5082E+01, 1.9614E+03, &
      2.9563506E+00, 6.45879E+01, 2.4926E+01, 1.9396E+03, &
      2.9711323E+00, 6.47745E+01, 2.4772E+01, 1.9181E+03, &
      2.9859880E+00, 6.49610E+01, 2.4619E+01, 1.8969E+03, &
      3.0009179E+00, 6.51549E+01, 2.4466E+01, 1.8758E+03, &
      3.0159225E+00, 6.53159E+01, 2.4291E+01, 1.8532E+03, &
      3.0310021E+00, 6.54708E+01, 2.4117E+01, 1.8309E+03, &
      3.0461571E+00, 6.56203E+01, 2.3944E+01, 1.8089E+03, &
      3.0613879E+00, 6.57647E+01, 2.3773E+01, 1.7871E+03, &
      3.0766949E+00, 6.59045E+01, 2.3603E+01, 1.7656E+03, &
      3.0920783E+00, 6.60400E+01, 2.3434E+01, 1.7443E+03, &
      3.1075387E+00, 6.61714E+01, 2.3266E+01, 1.7233E+03, &
      3.1230764E+00, 6.62990E+01, 2.3099E+01, 1.7025E+03, &
      3.1386918E+00, 6.64231E+01, 2.2934E+01, 1.6820E+03, &
      3.1543853E+00, 6.65437E+01, 2.2769E+01, 1.6618E+03, &
      3.1701572E+00, 6.66610E+01, 2.2606E+01, 1.6418E+03, &
      3.1860080E+00, 6.67752E+01, 2.2444E+01, 1.6220E+03, &
      3.2019380E+00, 6.68864E+01, 2.2283E+01, 1.6024E+03, &
      3.2179477E+00, 6.69948E+01, 2.2123E+01, 1.5831E+03, &
      3.2340374E+00, 6.71004E+01, 2.1964E+01, 1.5640E+03, &
      3.2502076E+00, 6.72033E+01, 2.1807E+01, 1.5452E+03, &
      3.2664587E+00, 6.73037E+01, 2.1650E+01, 1.5265E+03, &
      3.2827910E+00, 6.74017E+01, 2.1495E+01, 1.5081E+03, &
      3.2992049E+00, 6.74973E+01, 2.1340E+01, 1.4900E+03, &
      3.3157009E+00, 6.75906E+01, 2.1187E+01, 1.4720E+03, &
      3.3322794E+00, 6.76819E+01, 2.1035E+01, 1.4542E+03, &
      3.3489408E+00, 6.77711E+01, 2.0884E+01, 1.4367E+03, &
      3.3656856E+00, 6.78581E+01, 2.0733E+01, 1.4193E+03, &
      3.3825140E+00, 6.79429E+01, 2.0583E+01, 1.4021E+03, &
      3.3994265E+00, 6.80255E+01, 2.0434E+01, 1.3852E+03, &
      3.4164237E+00, 6.81062E+01, 2.0286E+01, 1.3684E+03, &
      3.4335058E+00, 6.81849E+01, 2.0140E+01, 1.3519E+03, &
      3.4506733E+00, 6.82617E+01, 1.9994E+01, 1.3355E+03, &
      3.4679267E+00, 6.83368E+01, 1.9850E+01, 1.3194E+03, &
      3.4852663E+00, 6.84102E+01, 1.9707E+01, 1.3034E+03, &
      3.5026927E+00, 6.84819E+01, 1.9565E+01, 1.2877E+03, &
      3.5202061E+00, 6.85521E+01, 1.9424E+01, 1.2721E+03, &
      3.5378072E+00, 6.86208E+01, 1.9284E+01, 1.2568E+03, &
      3.5554962E+00, 6.86880E+01, 1.9145E+01, 1.2416E+03, &
      3.5732737E+00, 6.87538E+01, 1.9007E+01, 1.2267E+03, &
      3.5911400E+00, 6.88184E+01, 1.8871E+01, 1.2119E+03, &
      3.6090957E+00, 6.88817E+01, 1.8735E+01, 1.1973E+03, &
      3.6271412E+00, 6.91474E+01, 1.8597E+01, 1.1826E+03, &
      3.6452769E+00, 6.92084E+01, 1.8456E+01, 1.1679E+03, &
      3.6635033E+00, 6.92674E+01, 1.8316E+01, 1.1534E+03, &
      3.6818208E+00, 6.93244E+01, 1.8177E+01, 1.1391E+03, &
      3.7002299E+00, 6.93798E+01, 1.8040E+01, 1.1249E+03, &
      3.7187311E+00, 6.94334E+01, 1.7904E+01, 1.1110E+03, &
      3.7373247E+00, 6.94855E+01, 1.7769E+01, 1.0972E+03, &
      3.7560114E+00, 6.96769E+01, 1.7631E+01, 1.0834E+03, &
      3.7747914E+00, 6.97259E+01, 1.7493E+01, 1.0697E+03, &
      3.7936654E+00, 6.97730E+01, 1.7357E+01, 1.0561E+03, &
      3.8126337E+00, 6.98183E+01, 1.7222E+01, 1.0428E+03, &
      3.8316969E+00, 6.98618E+01, 1.7088E+01, 1.0296E+03, &
      3.8508554E+00, 6.99037E+01, 1.6955E+01, 1.0166E+03, &
      3.8701096E+00, 6.99441E+01, 1.6823E+01, 1.0038E+03, &
      3.8894602E+00, 6.99829E+01, 1.6693E+01, 9.9118E+02, &
      3.9089075E+00, 7.00204E+01, 1.6564E+01, 9.7871E+02, &
      3.9284520E+00, 7.00565E+01, 1.6436E+01, 9.6641E+02, &
      3.9480943E+00, 7.00913E+01, 1.6309E+01, 9.5428E+02, &
      3.9678347E+00, 7.01248E+01, 1.6184E+01, 9.4232E+02, &
      3.9876739E+00, 7.01571E+01, 1.6060E+01, 9.3051E+02, &
      4.055024E+00, 7.06313E+01, 1.5650E+01, 8.9204E+02, &
      4.334821E+00, 7.08697E+01, 1.4139E+01, 7.5492E+02, &
      4.633924E+00, 7.11122E+01, 1.2784E+01, 6.3956E+02, &
      4.953664E+00, 7.11475E+01, 1.1526E+01, 5.4034E+02, &
      5.295467E+00, 7.11450E+01, 1.0392E+01, 4.5666E+02, &
      5.660855E+00, 7.10404E+01, 9.3697E+00, 3.8600E+02, &
      6.051453E+00, 7.08546E+01, 8.4516E+00, 3.2652E+02, &
      6.469004E+00, 7.06120E+01, 7.6347E+00, 2.7666E+02, &
      6.915365E+00, 7.03157E+01, 6.9068E+00, 2.3481E+02, &
      7.392525E+00, 6.99589E+01, 6.2574E+00, 1.9964E+02, &
      7.902609E+00, 6.95221E+01, 5.6752E+00, 1.6996E+02, &
      8.447890E+00, 6.89513E+01, 5.1320E+00, 1.4432E+02, &
      9.030794E+00, 6.81293E+01, 4.6283E+00, 1.2228E+02, &
      9.653919E+00, 6.65592E+01, 4.1268E+00, 1.0253E+02, &
      1.000266E+01, 6.45033E+01, 3.8823E+00, 9.3374E+01, &
      1.015577E+01, 6.16346E+01, 3.7802E+00, 8.9669E+01, &
      1.019659E+01, 5.82555E+01, 3.7538E+00, 8.8717E+01, &
      1.025783E+01, 6.16606E+01, 1.0324E+01, 2.3478E+02, &
      1.032004E+01, 6.33883E+01, 1.0184E+01, 2.3028E+02, &
      1.041094E+01, 6.46451E+01, 9.9850E+00, 2.2389E+02, &
      1.103212E+01, 6.68651E+01, 8.8485E+00, 1.8771E+02, &
      1.131312E+01, 6.65940E+01, 8.4442E+00, 1.7487E+02, &
      1.148628E+01, 6.54086E+01, 8.2142E+00, 1.6765E+02, &
      1.153246E+01, 6.38067E+01, 8.1551E+00, 1.6580E+02, &
      1.160172E+01, 6.55005E+01, 1.1288E+01, 2.2662E+02, &
      1.177488E+01, 6.70136E+01, 1.1001E+01, 2.1771E+02, &
      1.179334E+01, 6.70913E+01, 1.0971E+01, 2.1679E+02, &
      1.185780E+01, 6.73020E+01, 1.0868E+01, 2.1363E+02, &
      1.203930E+01, 6.72878E+01, 1.0589E+01, 2.0510E+02, &
      1.208770E+01, 6.65917E+01, 1.0516E+01, 2.0290E+02, &
      1.216030E+01, 6.76405E+01, 1.2074E+01, 2.3100E+02, &
      1.234180E+01, 6.88404E+01, 1.1804E+01, 2.2260E+02, &
      1.260708E+01, 6.97737E+01, 1.1430E+01, 2.1112E+02, &
      1.347697E+01, 7.14875E+01, 1.0319E+01, 1.7863E+02, &
      1.440688E+01, 7.24910E+01, 9.2845E+00, 1.5064E+02, &
      1.540095E+01, 7.31546E+01, 8.3430E+00, 1.2691E+02, &
      1.646362E+01, 7.36002E+01, 7.4814E+00, 1.0671E+02, &
      1.759961E+01, 7.38944E+01, 6.7060E+00, 8.9718E+01, &
      1.881398E+01, 7.40850E+01, 6.0128E+00, 7.5468E+01, &
      2.011215E+01, 7.42037E+01, 5.3947E+00, 6.3534E+01, &
      2.149988E+01, 7.43309E+01, 4.8368E+00, 5.3466E+01, &
      2.298338E+01, 7.43562E+01, 4.3382E+00, 4.5023E+01, &
      2.456923E+01, 7.44178E+01, 3.8884E+00, 3.7899E+01, &
      2.626450E+01, 7.43985E+01, 3.4838E+00, 3.1900E+01, &
      2.807676E+01, 7.43669E+01, 3.0983E+00, 2.6671E+01, &
      3.001405E+01, 7.42970E+01, 2.7503E+00, 2.2270E+01, &
      3.208502E+01, 7.42121E+01, 2.4435E+00, 1.8621E+01, &
      3.429889E+01, 7.41179E+01, 2.1727E+00, 1.5589E+01, &
      3.666551E+01, 7.40173E+01, 1.9334E+00, 1.3067E+01, &
      3.919543E+01, 7.39116E+01, 1.7219E+00, 1.0969E+01, &
      4.189992E+01, 7.38011E+01, 1.5346E+00, 9.2195E+00, &
      4.479101E+01, 7.36848E+01, 1.3682E+00, 7.7570E+00, &
      4.788159E+01, 7.35584E+01, 1.2184E+00, 6.5249E+00, &
      5.118542E+01, 7.34168E+01, 1.0851E+00, 5.4934E+00, &
      5.471721E+01, 7.32486E+01, 9.6347E-01, 4.6175E+00, &
      5.849270E+01, 7.30295E+01, 8.5449E-01, 3.8819E+00, &
      6.252870E+01, 7.27015E+01, 7.5806E-01, 3.2685E+00, &
      6.684318E+01, 7.20009E+01, 6.7279E-01, 2.7569E+00, &
      6.813450E+01, 7.15004E+01, 6.5025E-01, 2.6266E+00, &
      6.917738E+01, 7.03996E+01, 6.3292E-01, 2.5279E+00, &
      6.945548E+01, 6.90815E+01, 6.2842E-01, 2.5025E+00, &
      6.987263E+01, 7.04134E+01, 3.2163E+00, 1.0964E+01, &
      7.091550E+01, 7.16162E+01, 3.1450E+00, 1.0571E+01, &
      7.145536E+01, 7.19078E+01, 3.1091E+00, 1.0376E+01, &
      7.638578E+01, 7.30529E+01, 2.7986E+00, 8.7689E+00, &
      8.165640E+01, 7.35353E+01, 2.5097E+00, 7.3878E+00, &
      8.729069E+01, 7.38178E+01, 2.2497E+00, 6.2249E+00, &
      9.331374E+01, 7.40042E+01, 2.0139E+00, 5.2412E+00, &
      9.975239E+01, 7.41261E+01, 1.7992E+00, 4.4076E+00, &
      1.066353E+02, 7.42067E+01, 1.6076E+00, 3.7099E+00, &
      1.139931E+02, 7.42592E+01, 1.4366E+00, 3.1258E+00, &
      1.218587E+02, 7.42920E+01, 1.2839E+00, 2.6366E+00, &
      1.302669E+02, 7.43109E+01, 1.1476E+00, 2.2268E+00, &
      1.392553E+02, 7.43475E+01, 1.0259E+00, 1.8832E+00, &
      1.488640E+02, 7.43477E+01, 9.1402E-01, 1.5903E+00, &
      1.591356E+02, 7.43410E+01, 8.1445E-01, 1.3455E+00, &
      1.701159E+02, 7.43299E+01, 7.2581E-01, 1.1408E+00, &
      1.818539E+02, 7.43159E+01, 6.4687E-01, 9.6937E-01, &
      1.944018E+02, 7.43000E+01, 5.7657E-01, 8.2584E-01, &
      2.078156E+02, 7.42830E+01, 5.1396E-01, 7.0555E-01, &
      2.221548E+02, 7.42654E+01, 4.5818E-01, 6.0466E-01, &
      2.374835E+02, 7.42477E+01, 4.0850E-01, 5.1997E-01, &
      2.538699E+02, 7.42302E+01, 3.6423E-01, 4.4879E-01, &
      2.713869E+02, 7.42129E+01, 3.2479E-01, 3.8891E-01, &
      2.901126E+02, 7.41960E+01, 2.8963E-01, 3.3845E-01, &
      3.101304E+02, 7.41795E+01, 2.5917E-01, 2.9650E-01, &
      3.315294E+02, 7.41646E+01, 2.3293E-01, 2.6163E-01, &
      3.544049E+02, 7.41511E+01, 2.0937E-01, 2.3188E-01, &
      3.788588E+02, 7.41386E+01, 1.8820E-01, 2.0643E-01, &
      4.050001E+02, 7.41271E+01, 1.6919E-01, 1.8463E-01, &
      4.329451E+02, 7.41165E+01, 1.5211E-01, 1.6588E-01]
   case(75)
      frel = -8.8920E-01
      allocate(nist_tab(3064))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.104943E-03, 6.39581E+00, 0.0000E+00, 7.3520E-06, &
      5.183079E-03, 6.22399E+00, 0.0000E+00, 7.9793E-06, &
      5.203916E-03, 6.06267E+00, 0.0000E+00, 8.1534E-06, &
      5.235171E-03, 6.19350E+00, 2.9128E-01, 1.2574E+04, &
      5.313308E-03, 6.27314E+00, 2.9108E-01, 1.2380E+04, &
      5.941418E-03, 5.68875E+00, 2.9183E-01, 1.1100E+04, &
      6.032358E-03, 5.20729E+00, 2.9392E-01, 1.1011E+04, &
      6.056608E-03, 4.66114E+00, 2.9460E-01, 1.0992E+04, &
      6.092984E-03, 5.19845E+00, 1.3530E+00, 5.0183E+04, &
      6.183924E-03, 5.64768E+00, 1.3515E+00, 4.9391E+04, &
      1.069000E-02, 5.27266E+00, 3.4278E+00, 7.2464E+04, &
      1.142761E-02, 5.29226E+00, 4.0421E+00, 7.9935E+04, &
      1.221612E-02, 5.41943E+00, 4.7143E+00, 8.7210E+04, &
      1.305903E-02, 5.67608E+00, 5.4183E+00, 9.3764E+04, &
      1.396010E-02, 6.07660E+00, 6.1198E+00, 9.9067E+04, &
      1.492335E-02, 6.62422E+00, 6.7783E+00, 1.0264E+05, &
      1.595306E-02, 7.32014E+00, 7.3518E+00, 1.0414E+05, &
      1.705382E-02, 8.11734E+00, 7.8016E+00, 1.0338E+05, &
      1.823053E-02, 9.03318E+00, 8.0975E+00, 1.0038E+05, &
      1.948844E-02, 9.93792E+00, 8.2213E+00, 9.5333E+04, &
      2.083314E-02, 1.08250E+01, 8.1690E+00, 8.8613E+04, &
      2.227063E-02, 1.16460E+01, 7.9498E+00, 8.0669E+04, &
      2.380730E-02, 1.23560E+01, 7.5848E+00, 7.1997E+04, &
      2.545001E-02, 1.29157E+01, 7.1023E+00, 6.3066E+04, &
      2.720606E-02, 1.32835E+01, 6.5331E+00, 5.4267E+04, &
      2.908327E-02, 1.34089E+01, 5.9161E+00, 4.5970E+04, &
      3.109002E-02, 1.31912E+01, 5.2858E+00, 3.8422E+04, &
      3.323523E-02, 1.21962E+01, 4.6699E+00, 3.1754E+04, &
      3.390800E-02, 1.13408E+01, 4.4909E+00, 2.9930E+04, &
      3.442700E-02, 9.48514E+00, 4.3575E+00, 2.8604E+04, &
      3.456540E-02, 7.26152E+00, 4.3227E+00, 2.8261E+04, &
      3.477300E-02, 9.56931E+00, 8.6457E+00, 5.6187E+04, &
      3.529200E-02, 1.15804E+01, 8.4385E+00, 5.4035E+04, &
      3.552846E-02, 1.20120E+01, 8.3464E+00, 5.3089E+04, &
      3.797993E-02, 1.37977E+01, 7.4692E+00, 4.4443E+04, &
      3.978800E-02, 1.40851E+01, 6.8764E+00, 3.9056E+04, &
      4.039700E-02, 1.38521E+01, 6.6792E+00, 3.7365E+04, &
      4.055940E-02, 1.34579E+01, 6.6271E+00, 3.6924E+04, &
      4.060054E-02, 1.23158E+01, 7.4507E+00, 4.1471E+04, &
      4.080300E-02, 1.39357E+01, 7.3752E+00, 4.0847E+04, &
      4.141200E-02, 1.43817E+01, 7.1514E+00, 3.9026E+04, &
      4.340198E-02, 1.46487E+01, 6.4590E+00, 3.3631E+04, &
      4.468800E-02, 1.42887E+01, 6.0458E+00, 3.0574E+04, &
      4.537200E-02, 1.34221E+01, 5.8392E+00, 2.9084E+04, &
      4.555440E-02, 1.23252E+01, 5.7861E+00, 2.8704E+04, &
      4.582800E-02, 1.34803E+01, 7.8921E+00, 3.8917E+04, &
      4.639671E-02, 1.44092E+01, 7.7273E+00, 3.7638E+04, &
      4.651200E-02, 1.45135E+01, 7.6948E+00, 3.7387E+04, &
      4.959809E-02, 1.56927E+01, 6.9252E+00, 3.1553E+04, &
      5.302035E-02, 1.61494E+01, 6.2240E+00, 2.6528E+04, &
      5.667876E-02, 1.63094E+01, 5.5605E+00, 2.2170E+04, &
      6.058959E-02, 1.62441E+01, 4.9727E+00, 1.8547E+04, &
      6.477028E-02, 1.59999E+01, 4.4941E+00, 1.5680E+04, &
      6.923942E-02, 1.56151E+01, 4.1483E+00, 1.3540E+04, &
      7.401695E-02, 1.51211E+01, 3.9512E+00, 1.2064E+04, &
      7.912411E-02, 1.45397E+01, 3.9137E+00, 1.1178E+04, &
      8.114400E-02, 1.42945E+01, 3.9429E+00, 1.0981E+04, &
      8.238600E-02, 1.41235E+01, 3.9724E+00, 1.0896E+04, &
      8.271720E-02, 1.40506E+01, 3.9817E+00, 1.0878E+04, &
      8.321400E-02, 1.40293E+01, 4.0659E+00, 1.1042E+04, &
      8.445600E-02, 1.39194E+01, 4.1106E+00, 1.0999E+04, &
      8.458368E-02, 1.39067E+01, 4.1157E+00, 1.0996E+04, &
      9.041995E-02, 1.32976E+01, 4.4316E+00, 1.1076E+04, &
      9.665893E-02, 1.26720E+01, 4.9343E+00, 1.1536E+04, &
      1.033284E-01, 1.20734E+01, 5.6326E+00, 1.2320E+04, &
      1.104581E-01, 1.15404E+01, 6.5277E+00, 1.3356E+04, &
      1.180797E-01, 1.11085E+01, 7.6212E+00, 1.4587E+04, &
      1.262272E-01, 1.07184E+01, 8.9002E+00, 1.5935E+04, &
      1.349368E-01, 1.06211E+01, 1.0339E+01, 1.7316E+04, &
      1.442475E-01, 1.07502E+01, 1.1905E+01, 1.8652E+04, &
      1.542005E-01, 1.11382E+01, 1.3559E+01, 1.9872E+04, &
      1.648404E-01, 1.18180E+01, 1.5249E+01, 2.0907E+04, &
      1.762144E-01, 1.27984E+01, 1.6875E+01, 2.1643E+04, &
      1.883732E-01, 1.40334E+01, 1.8361E+01, 2.2030E+04, &
      2.013709E-01, 1.54684E+01, 1.9667E+01, 2.2073E+04, &
      2.152655E-01, 1.70427E+01, 2.0765E+01, 2.1801E+04, &
      2.301188E-01, 1.86854E+01, 2.1642E+01, 2.1255E+04, &
      2.459970E-01, 2.02842E+01, 2.2298E+01, 2.0487E+04, &
      2.549960E-01, 2.09838E+01, 2.2546E+01, 1.9984E+04, &
      2.588990E-01, 2.10168E+01, 2.2617E+01, 1.9745E+04, &
      2.599398E-01, 2.07081E+01, 2.2633E+01, 1.9680E+04, &
      2.615010E-01, 2.12542E+01, 2.3451E+01, 2.0269E+04, &
      2.629708E-01, 2.15754E+01, 2.3476E+01, 2.0177E+04, &
      2.654040E-01, 2.19410E+01, 2.3512E+01, 2.0023E+04, &
      2.682260E-01, 2.22642E+01, 2.3548E+01, 1.9843E+04, &
      2.723315E-01, 2.25202E+01, 2.3590E+01, 1.9578E+04, &
      2.734263E-01, 2.23395E+01, 2.3599E+01, 1.9507E+04, &
      2.750685E-01, 2.28457E+01, 2.4223E+01, 1.9903E+04, &
      2.791740E-01, 2.35829E+01, 2.4248E+01, 1.9632E+04, &
      2.811158E-01, 2.38558E+01, 2.4256E+01, 1.9503E+04, &
      3.005128E-01, 2.60380E+01, 2.4224E+01, 1.8220E+04, &
      3.212482E-01, 2.78948E+01, 2.4038E+01, 1.6914E+04, &
      3.434143E-01, 2.95561E+01, 2.3740E+01, 1.5626E+04, &
      3.671099E-01, 3.10382E+01, 2.3355E+01, 1.4382E+04, &
      3.924405E-01, 3.23241E+01, 2.2909E+01, 1.3197E+04, &
      4.195189E-01, 3.33221E+01, 2.2423E+01, 1.2084E+04, &
      4.355120E-01, 3.35331E+01, 2.2138E+01, 1.1492E+04, &
      4.421780E-01, 3.31448E+01, 2.2021E+01, 1.1260E+04, &
      4.439556E-01, 3.24487E+01, 2.1990E+01, 1.1199E+04, &
      4.466220E-01, 3.33164E+01, 2.3440E+01, 1.1866E+04, &
      4.484657E-01, 3.36747E+01, 2.3411E+01, 1.1803E+04, &
      4.532880E-01, 3.42307E+01, 2.3334E+01, 1.1639E+04, &
      4.794098E-01, 3.58103E+01, 2.2929E+01, 1.0814E+04, &
      5.0000000E-01, 3.81737E+01, 2.2620E+01, 1.0230E+04, &
      5.0250000E-01, 3.82614E+01, 2.2582E+01, 1.0162E+04, &
      5.0501250E-01, 3.83446E+01, 2.2545E+01, 1.0095E+04, &
      5.0753756E-01, 3.84222E+01, 2.2508E+01, 1.0028E+04, &
      5.1007525E-01, 3.84914E+01, 2.2471E+01, 9.9619E+03, &
      5.1262563E-01, 3.85464E+01, 2.2433E+01, 9.8960E+03, &
      5.1518875E-01, 3.85692E+01, 2.2396E+01, 9.8303E+03, &
      5.1717493E-01, 3.84935E+01, 2.2367E+01, 9.7800E+03, &
      5.1776470E-01, 3.83234E+01, 2.2358E+01, 9.7651E+03, &
      5.1862505E-01, 3.85476E+01, 2.2706E+01, 9.9004E+03, &
      5.2035352E-01, 3.87517E+01, 2.2682E+01, 9.8572E+03, &
      5.2295529E-01, 3.89309E+01, 2.2646E+01, 9.7927E+03, &
      5.2557007E-01, 3.90747E+01, 2.2610E+01, 9.7286E+03, &
      5.2819792E-01, 3.92038E+01, 2.2574E+01, 9.6648E+03, &
      5.3083891E-01, 3.93248E+01, 2.2538E+01, 9.6013E+03, &
      5.3349310E-01, 3.94403E+01, 2.2502E+01, 9.5383E+03, &
      5.3616057E-01, 3.95520E+01, 2.2465E+01, 9.4755E+03, &
      5.3884137E-01, 3.96606E+01, 2.2429E+01, 9.4132E+03, &
      5.4153558E-01, 3.97667E+01, 2.2392E+01, 9.3511E+03, &
      5.4424325E-01, 3.98708E+01, 2.2356E+01, 9.2895E+03, &
      5.4696447E-01, 3.99731E+01, 2.2319E+01, 9.2281E+03, &
      5.4969929E-01, 4.00738E+01, 2.2282E+01, 9.1671E+03, &
      5.5244779E-01, 4.01730E+01, 2.2245E+01, 9.1065E+03, &
      5.5521003E-01, 4.02708E+01, 2.2208E+01, 9.0461E+03, &
      5.5798608E-01, 4.03674E+01, 2.2171E+01, 8.9861E+03, &
      5.6077601E-01, 4.04628E+01, 2.2134E+01, 8.9265E+03, &
      5.6357989E-01, 4.05570E+01, 2.2096E+01, 8.8671E+03, &
      5.6639779E-01, 4.06501E+01, 2.2059E+01, 8.8081E+03, &
      5.6922978E-01, 4.07420E+01, 2.2021E+01, 8.7494E+03, &
      5.7207593E-01, 4.08328E+01, 2.1984E+01, 8.6911E+03, &
      5.7493630E-01, 4.09224E+01, 2.1946E+01, 8.6330E+03, &
      5.7781099E-01, 4.10109E+01, 2.1908E+01, 8.5753E+03, &
      5.8070004E-01, 4.10981E+01, 2.1870E+01, 8.5179E+03, &
      5.8360354E-01, 4.11840E+01, 2.1831E+01, 8.4607E+03, &
      5.8652156E-01, 4.12685E+01, 2.1793E+01, 8.4039E+03, &
      5.8945417E-01, 4.13514E+01, 2.1755E+01, 8.3474E+03, &
      5.9240144E-01, 4.14327E+01, 2.1716E+01, 8.2912E+03, &
      5.9536345E-01, 4.15120E+01, 2.1677E+01, 8.2353E+03, &
      5.9834026E-01, 4.15891E+01, 2.1638E+01, 8.1797E+03, &
      6.0133196E-01, 4.16635E+01, 2.1599E+01, 8.1244E+03, &
      6.0433862E-01, 4.17346E+01, 2.1560E+01, 8.0694E+03, &
      6.0736032E-01, 4.18015E+01, 2.1521E+01, 8.0147E+03, &
      6.1039712E-01, 4.18628E+01, 2.1481E+01, 7.9602E+03, &
      6.1344910E-01, 4.19160E+01, 2.1442E+01, 7.9061E+03, &
      6.1651635E-01, 4.19562E+01, 2.1402E+01, 7.8522E+03, &
      6.1959893E-01, 4.19716E+01, 2.1362E+01, 7.7987E+03, &
      6.2269693E-01, 4.19186E+01, 2.1322E+01, 7.7454E+03, &
      6.2404375E-01, 4.18089E+01, 2.1304E+01, 7.7224E+03, &
      6.2581041E-01, 4.18323E+01, 2.1814E+01, 7.8849E+03, &
      6.2595625E-01, 4.18646E+01, 2.1813E+01, 7.8824E+03, &
      6.2893946E-01, 4.21912E+01, 2.1775E+01, 7.8315E+03, &
      6.3208416E-01, 4.23815E+01, 2.1735E+01, 7.7784E+03, &
      6.3524458E-01, 4.25346E+01, 2.1696E+01, 7.7257E+03, &
      6.3842080E-01, 4.26708E+01, 2.1656E+01, 7.6732E+03, &
      6.4161291E-01, 4.27972E+01, 2.1616E+01, 7.6209E+03, &
      6.4482097E-01, 4.29171E+01, 2.1575E+01, 7.5690E+03, &
      6.4804508E-01, 4.30325E+01, 2.1535E+01, 7.5173E+03, &
      6.5128530E-01, 4.31444E+01, 2.1494E+01, 7.4658E+03, &
      6.5454173E-01, 4.32535E+01, 2.1453E+01, 7.4147E+03, &
      6.5781444E-01, 4.33605E+01, 2.1412E+01, 7.3637E+03, &
      6.6110351E-01, 4.34656E+01, 2.1371E+01, 7.3131E+03, &
      6.6440903E-01, 4.35692E+01, 2.1330E+01, 7.2627E+03, &
      6.6773107E-01, 4.36714E+01, 2.1288E+01, 7.2126E+03, &
      6.7106973E-01, 4.37725E+01, 2.1247E+01, 7.1627E+03, &
      6.7442508E-01, 4.38725E+01, 2.1205E+01, 7.1131E+03, &
      6.7779720E-01, 4.39716E+01, 2.1163E+01, 7.0638E+03, &
      6.8118619E-01, 4.40699E+01, 2.1120E+01, 7.0147E+03, &
      6.8459212E-01, 4.41675E+01, 2.1078E+01, 6.9659E+03, &
      6.8801508E-01, 4.42644E+01, 2.1036E+01, 6.9173E+03, &
      6.9145515E-01, 4.43607E+01, 2.0993E+01, 6.8690E+03, &
      6.9491243E-01, 4.44564E+01, 2.0950E+01, 6.8210E+03, &
      6.9838699E-01, 4.45516E+01, 2.0907E+01, 6.7732E+03, &
      7.0187893E-01, 4.46463E+01, 2.0864E+01, 6.7256E+03, &
      7.0538832E-01, 4.47405E+01, 2.0820E+01, 6.6783E+03, &
      7.0891526E-01, 4.48344E+01, 2.0777E+01, 6.6312E+03, &
      7.1245984E-01, 4.49278E+01, 2.0733E+01, 6.5844E+03, &
      7.1602214E-01, 4.50208E+01, 2.0689E+01, 6.5378E+03, &
      7.1960225E-01, 4.51135E+01, 2.0645E+01, 6.4915E+03, &
      7.2320026E-01, 4.52059E+01, 2.0600E+01, 6.4454E+03, &
      7.2681626E-01, 4.52980E+01, 2.0556E+01, 6.3995E+03, &
      7.3045034E-01, 4.53899E+01, 2.0511E+01, 6.3539E+03, &
      7.3410260E-01, 4.54814E+01, 2.0466E+01, 6.3085E+03, &
      7.3777311E-01, 4.55728E+01, 2.0421E+01, 6.2634E+03, &
      7.4146197E-01, 4.56640E+01, 2.0376E+01, 6.2185E+03, &
      7.4516928E-01, 4.57550E+01, 2.0330E+01, 6.1738E+03, &
      7.4889513E-01, 4.58458E+01, 2.0285E+01, 6.1294E+03, &
      7.5263961E-01, 4.59366E+01, 2.0238E+01, 6.0851E+03, &
      7.5640280E-01, 4.60270E+01, 2.0191E+01, 6.0408E+03, &
      7.6018482E-01, 4.61171E+01, 2.0144E+01, 5.9968E+03, &
      7.6398574E-01, 4.62073E+01, 2.0087E+01, 5.9501E+03, &
      7.6780567E-01, 4.62969E+01, 2.0016E+01, 5.8997E+03, &
      7.7164470E-01, 4.63852E+01, 1.9945E+01, 5.8496E+03, &
      7.7550292E-01, 4.64724E+01, 1.9874E+01, 5.7999E+03, &
      7.7938044E-01, 4.65584E+01, 1.9803E+01, 5.7505E+03, &
      7.8327734E-01, 4.66603E+01, 1.9732E+01, 5.7015E+03, &
      7.8719373E-01, 4.67442E+01, 1.9661E+01, 5.6527E+03, &
      7.9112969E-01, 4.68271E+01, 1.9589E+01, 5.6044E+03, &
      7.9508534E-01, 4.69089E+01, 1.9518E+01, 5.5563E+03, &
      7.9906077E-01, 4.69896E+01, 1.9447E+01, 5.5086E+03, &
      8.0305607E-01, 4.70694E+01, 1.9376E+01, 5.4612E+03, &
      8.0707135E-01, 4.71481E+01, 1.9305E+01, 5.4142E+03, &
      8.1110671E-01, 4.72259E+01, 1.9233E+01, 5.3675E+03, &
      8.1516224E-01, 4.73028E+01, 1.9162E+01, 5.3211E+03, &
      8.1923806E-01, 4.73787E+01, 1.9091E+01, 5.2751E+03, &
      8.2333425E-01, 4.74642E+01, 1.9020E+01, 5.2294E+03, &
      8.2745092E-01, 4.75385E+01, 1.8949E+01, 5.1840E+03, &
      8.3158817E-01, 4.76119E+01, 1.8878E+01, 5.1389E+03, &
      8.3574611E-01, 4.76846E+01, 1.8807E+01, 5.0942E+03, &
      8.3992484E-01, 4.77564E+01, 1.8736E+01, 5.0498E+03, &
      8.4412447E-01, 4.78274E+01, 1.8665E+01, 5.0058E+03, &
      8.4834509E-01, 4.78977E+01, 1.8594E+01, 4.9620E+03, &
      8.5258682E-01, 4.79672E+01, 1.8523E+01, 4.9186E+03, &
      8.5684975E-01, 4.80360E+01, 1.8452E+01, 4.8756E+03, &
      8.6113400E-01, 4.81042E+01, 1.8381E+01, 4.8328E+03, &
      8.6543967E-01, 4.81717E+01, 1.8311E+01, 4.7904E+03, &
      8.6976687E-01, 4.82386E+01, 1.8240E+01, 4.7483E+03, &
      8.7411570E-01, 4.83049E+01, 1.8169E+01, 4.7065E+03, &
      8.7848628E-01, 4.83706E+01, 1.8099E+01, 4.6649E+03, &
      8.8287871E-01, 4.84356E+01, 1.8028E+01, 4.6237E+03, &
      8.8729310E-01, 4.85001E+01, 1.7957E+01, 4.5827E+03, &
      8.9172957E-01, 4.85641E+01, 1.7887E+01, 4.5421E+03, &
      8.9618822E-01, 4.86275E+01, 1.7816E+01, 4.5018E+03, &
      9.0066916E-01, 4.86905E+01, 1.7746E+01, 4.4618E+03, &
      9.0517250E-01, 4.87531E+01, 1.7676E+01, 4.4221E+03, &
      9.0969837E-01, 4.88152E+01, 1.7605E+01, 4.3828E+03, &
      9.1424686E-01, 4.88771E+01, 1.7535E+01, 4.3437E+03, &
      9.1881809E-01, 4.89386E+01, 1.7465E+01, 4.3050E+03, &
      9.2341218E-01, 4.89999E+01, 1.7395E+01, 4.2665E+03, &
      9.2802924E-01, 4.90610E+01, 1.7326E+01, 4.2284E+03, &
      9.3266939E-01, 4.91220E+01, 1.7256E+01, 4.1905E+03, &
      9.3733274E-01, 4.91830E+01, 1.7187E+01, 4.1530E+03, &
      9.4201940E-01, 4.92439E+01, 1.7117E+01, 4.1158E+03, &
      9.4672950E-01, 4.93050E+01, 1.7048E+01, 4.0789E+03, &
      9.5146315E-01, 4.93663E+01, 1.6979E+01, 4.0423E+03, &
      9.5622046E-01, 4.94279E+01, 1.6910E+01, 4.0059E+03, &
      9.6100156E-01, 4.94899E+01, 1.6842E+01, 3.9699E+03, &
      9.6580657E-01, 4.95525E+01, 1.6773E+01, 3.9342E+03, &
      9.7063560E-01, 4.96158E+01, 1.6705E+01, 3.8988E+03, &
      9.7548878E-01, 4.96800E+01, 1.6637E+01, 3.8636E+03, &
      9.8036623E-01, 4.97453E+01, 1.6568E+01, 3.8288E+03, &
      9.8526806E-01, 4.98119E+01, 1.6501E+01, 3.7943E+03, &
      9.9019440E-01, 4.98802E+01, 1.6433E+01, 3.7600E+03, &
      9.9514537E-01, 4.99503E+01, 1.6365E+01, 3.7260E+03, &
      1.0001211E+00, 5.00219E+01, 1.6298E+01, 3.6922E+03, &
      1.0051217E+00, 5.00610E+01, 1.6208E+01, 3.6537E+03, &
      1.0101473E+00, 5.00969E+01, 1.6118E+01, 3.6156E+03, &
      1.0151980E+00, 5.01305E+01, 1.6030E+01, 3.5780E+03, &
      1.0202740E+00, 5.01623E+01, 1.5942E+01, 3.5407E+03, &
      1.0253754E+00, 5.01926E+01, 1.5854E+01, 3.5039E+03, &
      1.0305023E+00, 5.02215E+01, 1.5767E+01, 3.4674E+03, &
      1.0356548E+00, 5.02493E+01, 1.5681E+01, 3.4314E+03, &
      1.0408331E+00, 5.02761E+01, 1.5595E+01, 3.3958E+03, &
      1.0460372E+00, 5.03014E+01, 1.5499E+01, 3.3582E+03, &
      1.0512674E+00, 5.03249E+01, 1.5404E+01, 3.3211E+03, &
      1.0565238E+00, 5.03467E+01, 1.5309E+01, 3.2844E+03, &
      1.0618064E+00, 5.03668E+01, 1.5215E+01, 3.2481E+03, &
      1.0671154E+00, 5.03853E+01, 1.5121E+01, 3.2122E+03, &
      1.0724510E+00, 5.04022E+01, 1.5029E+01, 3.1768E+03, &
      1.0778132E+00, 5.04177E+01, 1.4937E+01, 3.1417E+03, &
      1.0832023E+00, 5.04316E+01, 1.4846E+01, 3.1072E+03, &
      1.0886183E+00, 5.04442E+01, 1.4755E+01, 3.0730E+03, &
      1.0940614E+00, 5.04554E+01, 1.4666E+01, 3.0393E+03, &
      1.0995317E+00, 5.04650E+01, 1.4571E+01, 3.0047E+03, &
      1.1050294E+00, 5.04729E+01, 1.4476E+01, 2.9704E+03, &
      1.1105545E+00, 5.04790E+01, 1.4382E+01, 2.9365E+03, &
      1.1161073E+00, 5.04833E+01, 1.4288E+01, 2.9030E+03, &
      1.1216878E+00, 5.04859E+01, 1.4195E+01, 2.8700E+03, &
      1.1272963E+00, 5.04869E+01, 1.4104E+01, 2.8374E+03, &
      1.1329328E+00, 5.04862E+01, 1.4013E+01, 2.8052E+03, &
      1.1385974E+00, 5.04839E+01, 1.3922E+01, 2.7734E+03, &
      1.1442904E+00, 5.04801E+01, 1.3833E+01, 2.7420E+03, &
      1.1500119E+00, 5.04747E+01, 1.3744E+01, 2.7110E+03, &
      1.1557619E+00, 5.04678E+01, 1.3656E+01, 2.6804E+03, &
      1.1615407E+00, 5.04594E+01, 1.3569E+01, 2.6501E+03, &
      1.1673484E+00, 5.04495E+01, 1.3483E+01, 2.6203E+03, &
      1.1731852E+00, 5.04381E+01, 1.3397E+01, 2.5908E+03, &
      1.1790511E+00, 5.04252E+01, 1.3312E+01, 2.5617E+03, &
      1.1849464E+00, 5.04109E+01, 1.3228E+01, 2.5330E+03, &
      1.1908711E+00, 5.03951E+01, 1.3144E+01, 2.5046E+03, &
      1.1968254E+00, 5.03779E+01, 1.3062E+01, 2.4765E+03, &
      1.2028096E+00, 5.03593E+01, 1.2980E+01, 2.4489E+03, &
      1.2088236E+00, 5.03392E+01, 1.2898E+01, 2.4215E+03, &
      1.2148677E+00, 5.03177E+01, 1.2817E+01, 2.3945E+03, &
      1.2209421E+00, 5.02948E+01, 1.2737E+01, 2.3679E+03, &
      1.2270468E+00, 5.02704E+01, 1.2658E+01, 2.3415E+03, &
      1.2331820E+00, 5.02446E+01, 1.2579E+01, 2.3155E+03, &
      1.2393479E+00, 5.02173E+01, 1.2501E+01, 2.2899E+03, &
      1.2455447E+00, 5.01886E+01, 1.2424E+01, 2.2645E+03, &
      1.2517724E+00, 5.01584E+01, 1.2347E+01, 2.2394E+03, &
      1.2580312E+00, 5.01268E+01, 1.2271E+01, 2.2147E+03, &
      1.2643214E+00, 5.00936E+01, 1.2196E+01, 2.1903E+03, &
      1.2706430E+00, 5.00590E+01, 1.2121E+01, 2.1661E+03, &
      1.2769962E+00, 5.00229E+01, 1.2047E+01, 2.1423E+03, &
      1.2833812E+00, 4.99853E+01, 1.1973E+01, 2.1187E+03, &
      1.2897981E+00, 4.99461E+01, 1.1900E+01, 2.0955E+03, &
      1.2962471E+00, 4.99054E+01, 1.1828E+01, 2.0725E+03, &
      1.3027283E+00, 4.98630E+01, 1.1756E+01, 2.0498E+03, &
      1.3092420E+00, 4.98191E+01, 1.1685E+01, 2.0274E+03, &
      1.3157882E+00, 4.97736E+01, 1.1615E+01, 2.0052E+03, &
      1.3223671E+00, 4.97263E+01, 1.1545E+01, 1.9834E+03, &
      1.3289790E+00, 4.96775E+01, 1.1475E+01, 1.9618E+03, &
      1.3356239E+00, 4.96284E+01, 1.1406E+01, 1.9404E+03, &
      1.3423020E+00, 4.95761E+01, 1.1338E+01, 1.9193E+03, &
      1.3490135E+00, 4.95220E+01, 1.1270E+01, 1.8985E+03, &
      1.3557586E+00, 4.94661E+01, 1.1203E+01, 1.8779E+03, &
      1.3625374E+00, 4.94083E+01, 1.1137E+01, 1.8576E+03, &
      1.3693500E+00, 4.93487E+01, 1.1071E+01, 1.8375E+03, &
      1.3761968E+00, 4.92871E+01, 1.1005E+01, 1.8177E+03, &
      1.3830778E+00, 4.92235E+01, 1.0940E+01, 1.7981E+03, &
      1.3899932E+00, 4.91578E+01, 1.0876E+01, 1.7787E+03, &
      1.3969431E+00, 4.90900E+01, 1.0812E+01, 1.7596E+03, &
      1.4039278E+00, 4.90201E+01, 1.0748E+01, 1.7407E+03, &
      1.4109475E+00, 4.89480E+01, 1.0685E+01, 1.7220E+03, &
      1.4180022E+00, 4.88736E+01, 1.0623E+01, 1.7036E+03, &
      1.4250922E+00, 4.87969E+01, 1.0561E+01, 1.6853E+03, &
      1.4322177E+00, 4.87177E+01, 1.0500E+01, 1.6673E+03, &
      1.4393788E+00, 4.86360E+01, 1.0439E+01, 1.6495E+03, &
      1.4465757E+00, 4.85518E+01, 1.0378E+01, 1.6320E+03, &
      1.4538086E+00, 4.84648E+01, 1.0319E+01, 1.6146E+03, &
      1.4610776E+00, 4.83751E+01, 1.0259E+01, 1.5974E+03, &
      1.4683830E+00, 4.82826E+01, 1.0200E+01, 1.5805E+03, &
      1.4757249E+00, 4.81871E+01, 1.0142E+01, 1.5637E+03, &
      1.4831035E+00, 4.80885E+01, 1.0084E+01, 1.5471E+03, &
      1.4905190E+00, 4.79867E+01, 1.0026E+01, 1.5308E+03, &
      1.4979716E+00, 4.78815E+01, 9.9689E+00, 1.5146E+03, &
      1.5054615E+00, 4.77729E+01, 9.9123E+00, 1.4986E+03, &
      1.5129888E+00, 4.76607E+01, 9.8561E+00, 1.4828E+03, &
      1.5205537E+00, 4.75447E+01, 9.8003E+00, 1.4672E+03, &
      1.5281565E+00, 4.74247E+01, 9.7450E+00, 1.4518E+03, &
      1.5357973E+00, 4.73007E+01, 9.6901E+00, 1.4366E+03, &
      1.5434763E+00, 4.71723E+01, 9.6356E+00, 1.4215E+03, &
      1.5511937E+00, 4.70393E+01, 9.5816E+00, 1.4066E+03, &
      1.5589496E+00, 4.69051E+01, 9.5280E+00, 1.3919E+03, &
      1.5667444E+00, 4.67624E+01, 9.4747E+00, 1.3773E+03, &
      1.5745781E+00, 4.66145E+01, 9.4219E+00, 1.3630E+03, &
      1.5824510E+00, 4.64609E+01, 9.3695E+00, 1.3488E+03, &
      1.5903633E+00, 4.63015E+01, 9.3174E+00, 1.3347E+03, &
      1.5983151E+00, 4.61358E+01, 9.2658E+00, 1.3208E+03, &
      1.6063066E+00, 4.59636E+01, 9.2146E+00, 1.3071E+03, &
      1.6143382E+00, 4.57843E+01, 9.1638E+00, 1.2936E+03, &
      1.6224099E+00, 4.55976E+01, 9.1134E+00, 1.2802E+03, &
      1.6305219E+00, 4.54029E+01, 9.0634E+00, 1.2669E+03, &
      1.6386745E+00, 4.51997E+01, 9.0138E+00, 1.2538E+03, &
      1.6468679E+00, 4.49874E+01, 8.9645E+00, 1.2409E+03, &
      1.6551022E+00, 4.47653E+01, 8.9156E+00, 1.2281E+03, &
      1.6633777E+00, 4.45327E+01, 8.8671E+00, 1.2154E+03, &
      1.6716946E+00, 4.42886E+01, 8.8190E+00, 1.2029E+03, &
      1.6800531E+00, 4.40323E+01, 8.7712E+00, 1.1906E+03, &
      1.6884534E+00, 4.37625E+01, 8.7238E+00, 1.1784E+03, &
      1.6968956E+00, 4.34782E+01, 8.6768E+00, 1.1663E+03, &
      1.7053801E+00, 4.31778E+01, 8.6301E+00, 1.1544E+03, &
      1.7139070E+00, 4.28598E+01, 8.5838E+00, 1.1426E+03, &
      1.7224766E+00, 4.25224E+01, 8.5379E+00, 1.1309E+03, &
      1.7310889E+00, 4.21634E+01, 8.4922E+00, 1.1194E+03, &
      1.7397444E+00, 4.17802E+01, 8.4470E+00, 1.1080E+03, &
      1.7484431E+00, 4.13698E+01, 8.4020E+00, 1.0967E+03, &
      1.7571853E+00, 4.09287E+01, 8.3574E+00, 1.0856E+03, &
      1.7659712E+00, 4.04524E+01, 8.3132E+00, 1.0746E+03, &
      1.7748011E+00, 3.99355E+01, 8.2693E+00, 1.0637E+03, &
      1.7836751E+00, 3.93712E+01, 8.2243E+00, 1.0528E+03, &
      1.7925935E+00, 3.87506E+01, 8.1791E+00, 1.0419E+03, &
      1.8015565E+00, 3.80624E+01, 8.1342E+00, 1.0311E+03, &
      1.8105642E+00, 3.72913E+01, 8.0896E+00, 1.0205E+03, &
      1.8196171E+00, 3.64161E+01, 8.0454E+00, 1.0100E+03, &
      1.8287151E+00, 3.54057E+01, 8.0015E+00, 9.9958E+02, &
      1.8378587E+00, 3.42121E+01, 7.9579E+00, 9.8930E+02, &
      1.8470480E+00, 3.27551E+01, 7.9147E+00, 9.7914E+02, &
      1.8562833E+00, 3.08827E+01, 7.8717E+00, 9.6909E+02, &
      1.8655647E+00, 2.82448E+01, 7.8291E+00, 9.5916E+02, &
      1.8748925E+00, 2.36363E+01, 7.7868E+00, 9.4934E+02, &
      1.8825065E+00, 6.36850E+00, 7.7527E+00, 9.4144E+02, &
      1.8832935E+00, 6.05504E+00, 2.5555E+01, 3.0773E+03, &
      1.8842670E+00, 1.30938E+01, 2.5537E+01, 3.0735E+03, &
      1.8936883E+00, 2.44712E+01, 2.5357E+01, 3.0368E+03, &
      1.9031567E+00, 2.75494E+01, 2.5179E+01, 3.0007E+03, &
      1.9126725E+00, 2.90524E+01, 2.5003E+01, 2.9649E+03, &
      1.9222359E+00, 2.96683E+01, 2.4828E+01, 2.9296E+03, &
      1.9318471E+00, 2.94164E+01, 2.4654E+01, 2.8947E+03, &
      1.9415063E+00, 2.74998E+01, 2.4481E+01, 2.8603E+03, &
      1.9481516E+00, 1.96941E+01, 2.4364E+01, 2.8369E+03, &
      1.9496484E+00, 1.96648E+01, 3.6070E+01, 4.1917E+03, &
      1.9512138E+00, 2.40511E+01, 3.6028E+01, 4.1835E+03, &
      1.9609699E+00, 3.11633E+01, 3.5771E+01, 4.1331E+03, &
      1.9707747E+00, 3.42046E+01, 3.5515E+01, 4.0832E+03, &
      1.9806286E+00, 3.63252E+01, 3.5262E+01, 4.0340E+03, &
      1.9905318E+00, 3.80003E+01, 3.5010E+01, 3.9854E+03, &
      2.0004844E+00, 3.94024E+01, 3.4760E+01, 3.9374E+03, &
      2.0104868E+00, 4.06159E+01, 3.4512E+01, 3.8900E+03, &
      2.0205393E+00, 4.16890E+01, 3.4266E+01, 3.8432E+03, &
      2.0306420E+00, 4.26523E+01, 3.4022E+01, 3.7970E+03, &
      2.0407952E+00, 4.35266E+01, 3.3779E+01, 3.7513E+03, &
      2.0509992E+00, 4.43268E+01, 3.3539E+01, 3.7062E+03, &
      2.0612542E+00, 4.50641E+01, 3.3300E+01, 3.6616E+03, &
      2.0715604E+00, 4.57469E+01, 3.3063E+01, 3.6176E+03, &
      2.0819182E+00, 4.63818E+01, 3.2827E+01, 3.5740E+03, &
      2.0923278E+00, 4.69742E+01, 3.2593E+01, 3.5309E+03, &
      2.1027895E+00, 4.75283E+01, 3.2360E+01, 3.4884E+03, &
      2.1133034E+00, 4.80476E+01, 3.2129E+01, 3.4464E+03, &
      2.1238699E+00, 4.85352E+01, 3.1900E+01, 3.4050E+03, &
      2.1344893E+00, 4.89933E+01, 3.1673E+01, 3.3640E+03, &
      2.1451617E+00, 4.94240E+01, 3.1447E+01, 3.3235E+03, &
      2.1558875E+00, 4.98290E+01, 3.1223E+01, 3.2835E+03, &
      2.1666670E+00, 5.02095E+01, 3.1001E+01, 3.2441E+03, &
      2.1775003E+00, 5.05667E+01, 3.0780E+01, 3.2051E+03, &
      2.1883878E+00, 5.09012E+01, 3.0561E+01, 3.1666E+03, &
      2.1993297E+00, 5.12137E+01, 3.0344E+01, 3.1285E+03, &
      2.2103264E+00, 5.15044E+01, 3.0128E+01, 3.0909E+03, &
      2.2213780E+00, 5.17732E+01, 2.9914E+01, 3.0538E+03, &
      2.2324849E+00, 5.20198E+01, 2.9701E+01, 3.0172E+03, &
      2.2436473E+00, 5.22434E+01, 2.9490E+01, 2.9810E+03, &
      2.2548656E+00, 5.24425E+01, 2.9281E+01, 2.9452E+03, &
      2.2661399E+00, 5.26152E+01, 2.9073E+01, 2.9099E+03, &
      2.2774706E+00, 5.27583E+01, 2.8867E+01, 2.8750E+03, &
      2.2888579E+00, 5.28670E+01, 2.8662E+01, 2.8405E+03, &
      2.3003022E+00, 5.29340E+01, 2.8459E+01, 2.8065E+03, &
      2.3118037E+00, 5.29471E+01, 2.8258E+01, 2.7729E+03, &
      2.3233628E+00, 5.28855E+01, 2.8058E+01, 2.7396E+03, &
      2.3349796E+00, 5.27075E+01, 2.7859E+01, 2.7068E+03, &
      2.3466545E+00, 5.23133E+01, 2.7662E+01, 2.6744E+03, &
      2.3583878E+00, 5.13290E+01, 2.7466E+01, 2.6424E+03, &
      2.3649801E+00, 4.94646E+01, 2.7357E+01, 2.6247E+03, &
      2.3696200E+00, 4.95016E+01, 3.2000E+01, 3.0623E+03, &
      2.3701797E+00, 4.98406E+01, 3.1988E+01, 3.0605E+03, &
      2.3820306E+00, 5.25668E+01, 3.1753E+01, 3.0230E+03, &
      2.3939407E+00, 5.37078E+01, 3.1519E+01, 2.9859E+03, &
      2.4059104E+00, 5.45014E+01, 3.1287E+01, 2.9493E+03, &
      2.4179400E+00, 5.51326E+01, 3.1057E+01, 2.9132E+03, &
      2.4300297E+00, 5.56670E+01, 3.0829E+01, 2.8775E+03, &
      2.4421798E+00, 5.61356E+01, 3.0603E+01, 2.8423E+03, &
      2.4543907E+00, 5.65546E+01, 3.0374E+01, 2.8071E+03, &
      2.4666627E+00, 5.69319E+01, 3.0142E+01, 2.7720E+03, &
      2.4789960E+00, 5.72732E+01, 2.9913E+01, 2.7373E+03, &
      2.4913910E+00, 5.75828E+01, 2.9695E+01, 2.7040E+03, &
      2.5038479E+00, 5.78726E+01, 2.9486E+01, 2.6717E+03, &
      2.5163672E+00, 5.81446E+01, 2.9280E+01, 2.6400E+03, &
      2.5289490E+00, 5.83997E+01, 2.9076E+01, 2.6087E+03, &
      2.5415938E+00, 5.86382E+01, 2.8872E+01, 2.5775E+03, &
      2.5543017E+00, 5.88592E+01, 2.8667E+01, 2.5466E+03, &
      2.5670732E+00, 5.90629E+01, 2.8465E+01, 2.5162E+03, &
      2.5799086E+00, 5.92488E+01, 2.8265E+01, 2.4862E+03, &
      2.5928082E+00, 5.94157E+01, 2.8066E+01, 2.4566E+03, &
      2.6057722E+00, 5.95611E+01, 2.7870E+01, 2.4274E+03, &
      2.6188011E+00, 5.96808E+01, 2.7676E+01, 2.3986E+03, &
      2.6318951E+00, 5.97670E+01, 2.7484E+01, 2.3702E+03, &
      2.6450545E+00, 5.98035E+01, 2.7293E+01, 2.3421E+03, &
      2.6582798E+00, 5.97515E+01, 2.7104E+01, 2.3145E+03, &
      2.6715712E+00, 5.94635E+01, 2.6917E+01, 2.2871E+03, &
      2.6780604E+00, 5.89517E+01, 2.6826E+01, 2.2740E+03, &
      2.6849291E+00, 5.89966E+01, 2.8579E+01, 2.4157E+03, &
      2.6851398E+00, 5.90361E+01, 2.8576E+01, 2.4152E+03, &
      2.6983537E+00, 6.01491E+01, 2.8374E+01, 2.3866E+03, &
      2.7118455E+00, 6.06837E+01, 2.8171E+01, 2.3578E+03, &
      2.7254047E+00, 6.10784E+01, 2.7970E+01, 2.3294E+03, &
      2.7390317E+00, 6.14054E+01, 2.7770E+01, 2.3014E+03, &
      2.7527269E+00, 6.16901E+01, 2.7572E+01, 2.2737E+03, &
      2.7664905E+00, 6.19444E+01, 2.7375E+01, 2.2464E+03, &
      2.7803230E+00, 6.21744E+01, 2.7180E+01, 2.2194E+03, &
      2.7942246E+00, 6.23834E+01, 2.6986E+01, 2.1927E+03, &
      2.8081957E+00, 6.25725E+01, 2.6794E+01, 2.1664E+03, &
      2.8222367E+00, 6.27423E+01, 2.6612E+01, 2.1410E+03, &
      2.8363479E+00, 6.28995E+01, 2.6436E+01, 2.1164E+03, &
      2.8505296E+00, 6.30430E+01, 2.6263E+01, 2.0922E+03, &
      2.8647823E+00, 6.31698E+01, 2.6093E+01, 2.0684E+03, &
      2.8791062E+00, 6.32748E+01, 2.5924E+01, 2.0449E+03, &
      2.8935017E+00, 6.33474E+01, 2.5757E+01, 2.0217E+03, &
      2.9079692E+00, 6.33605E+01, 2.5591E+01, 1.9988E+03, &
      2.9225091E+00, 6.32012E+01, 2.5427E+01, 1.9762E+03, &
      2.9258366E+00, 6.30810E+01, 2.5390E+01, 1.9711E+03, &
      2.9371216E+00, 6.31838E+01, 2.6391E+01, 2.0406E+03, &
      2.9375634E+00, 6.32173E+01, 2.6386E+01, 2.0399E+03, &
      2.9518072E+00, 6.38305E+01, 2.6225E+01, 2.0177E+03, &
      2.9665662E+00, 6.42004E+01, 2.6060E+01, 1.9952E+03, &
      2.9813991E+00, 6.44973E+01, 2.5896E+01, 1.9728E+03, &
      2.9963061E+00, 6.47594E+01, 2.5733E+01, 1.9508E+03, &
      3.0112876E+00, 6.49992E+01, 2.5556E+01, 1.9278E+03, &
      3.0263440E+00, 6.52189E+01, 2.5376E+01, 1.9048E+03, &
      3.0414758E+00, 6.54241E+01, 2.5197E+01, 1.8821E+03, &
      3.0566831E+00, 6.56181E+01, 2.5020E+01, 1.8596E+03, &
      3.0719666E+00, 6.58030E+01, 2.4844E+01, 1.8375E+03, &
      3.0873264E+00, 6.59800E+01, 2.4667E+01, 1.8154E+03, &
      3.1027630E+00, 6.61492E+01, 2.4491E+01, 1.7936E+03, &
      3.1182768E+00, 6.63114E+01, 2.4316E+01, 1.7720E+03, &
      3.1338682E+00, 6.64675E+01, 2.4142E+01, 1.7507E+03, &
      3.1495376E+00, 6.66181E+01, 2.3969E+01, 1.7296E+03, &
      3.1652853E+00, 6.67636E+01, 2.3798E+01, 1.7088E+03, &
      3.1811117E+00, 6.69043E+01, 2.3628E+01, 1.6883E+03, &
      3.1970172E+00, 6.70407E+01, 2.3459E+01, 1.6680E+03, &
      3.2130023E+00, 6.71730E+01, 2.3291E+01, 1.6479E+03, &
      3.2290673E+00, 6.73014E+01, 2.3125E+01, 1.6281E+03, &
      3.2452127E+00, 6.74261E+01, 2.2959E+01, 1.6085E+03, &
      3.2614387E+00, 6.75473E+01, 2.2795E+01, 1.5891E+03, &
      3.2777459E+00, 6.76653E+01, 2.2631E+01, 1.5700E+03, &
      3.2941347E+00, 6.77800E+01, 2.2469E+01, 1.5511E+03, &
      3.3106053E+00, 6.78917E+01, 2.2308E+01, 1.5324E+03, &
      3.3271584E+00, 6.80005E+01, 2.2148E+01, 1.5139E+03, &
      3.3437941E+00, 6.81065E+01, 2.1989E+01, 1.4957E+03, &
      3.3605131E+00, 6.82098E+01, 2.1832E+01, 1.4777E+03, &
      3.3773157E+00, 6.83105E+01, 2.1675E+01, 1.4599E+03, &
      3.3942023E+00, 6.84087E+01, 2.1519E+01, 1.4423E+03, &
      3.4111733E+00, 6.85045E+01, 2.1365E+01, 1.4249E+03, &
      3.4282291E+00, 6.85980E+01, 2.1211E+01, 1.4077E+03, &
      3.4453703E+00, 6.86892E+01, 2.1059E+01, 1.3907E+03, &
      3.4625971E+00, 6.87783E+01, 2.0908E+01, 1.3740E+03, &
      3.4799101E+00, 6.88653E+01, 2.0758E+01, 1.3574E+03, &
      3.4973097E+00, 6.89504E+01, 2.0608E+01, 1.3410E+03, &
      3.5147962E+00, 6.90332E+01, 2.0459E+01, 1.3248E+03, &
      3.5323702E+00, 6.91141E+01, 2.0312E+01, 1.3088E+03, &
      3.5500321E+00, 6.91929E+01, 2.0165E+01, 1.2930E+03, &
      3.5677822E+00, 6.92699E+01, 2.0020E+01, 1.2774E+03, &
      3.5856211E+00, 6.93451E+01, 1.9875E+01, 1.2620E+03, &
      3.6035492E+00, 6.94186E+01, 1.9732E+01, 1.2467E+03, &
      3.6215670E+00, 6.94904E+01, 1.9590E+01, 1.2317E+03, &
      3.6396748E+00, 6.95606E+01, 1.9449E+01, 1.2168E+03, &
      3.6578732E+00, 6.96293E+01, 1.9309E+01, 1.2022E+03, &
      3.6761626E+00, 6.96965E+01, 1.9170E+01, 1.1877E+03, &
      3.6945434E+00, 6.97623E+01, 1.9033E+01, 1.1734E+03, &
      3.7130161E+00, 6.98267E+01, 1.8896E+01, 1.1592E+03, &
      3.7315812E+00, 6.98899E+01, 1.8760E+01, 1.1453E+03, &
      3.7502391E+00, 6.99518E+01, 1.8626E+01, 1.1315E+03, &
      3.7689903E+00, 7.02170E+01, 1.8491E+01, 1.1178E+03, &
      3.7878352E+00, 7.02769E+01, 1.8351E+01, 1.1039E+03, &
      3.8067744E+00, 7.03348E+01, 1.8212E+01, 1.0902E+03, &
      3.8258083E+00, 7.03908E+01, 1.8074E+01, 1.0766E+03, &
      3.8449373E+00, 7.04451E+01, 1.7937E+01, 1.0633E+03, &
      3.8641620E+00, 7.04976E+01, 1.7802E+01, 1.0501E+03, &
      3.8834828E+00, 7.05487E+01, 1.7668E+01, 1.0371E+03, &
      3.9029002E+00, 7.07387E+01, 1.7533E+01, 1.0242E+03, &
      3.9224147E+00, 7.07870E+01, 1.7396E+01, 1.0112E+03, &
      3.9420268E+00, 7.08333E+01, 1.7260E+01, 9.9840E+02, &
      3.9617369E+00, 7.08778E+01, 1.7126E+01, 9.8578E+02, &
      3.9815456E+00, 7.09206E+01, 1.6992E+01, 9.7333E+02, &
      4.0014533E+00, 7.09617E+01, 1.6860E+01, 9.6105E+02, &
      4.0214606E+00, 7.10013E+01, 1.6729E+01, 9.4894E+02, &
      4.0415679E+00, 7.10393E+01, 1.6600E+01, 9.3700E+02, &
      4.0617757E+00, 7.10760E+01, 1.6471E+01, 9.2521E+02, &
      4.0820846E+00, 7.11113E+01, 1.6344E+01, 9.1358E+02, &
      4.1024950E+00, 7.11453E+01, 1.6218E+01, 9.0212E+02, &
      4.1230075E+00, 7.11780E+01, 1.6093E+01, 8.9080E+02, &
      4.1436226E+00, 7.12096E+01, 1.5969E+01, 8.7964E+02, &
      4.1643407E+00, 7.12399E+01, 1.5847E+01, 8.6863E+02, &
      4.1851624E+00, 7.12692E+01, 1.5725E+01, 8.5777E+02, &
      4.2060882E+00, 7.12974E+01, 1.5605E+01, 8.4706E+02, &
      4.2271186E+00, 7.13246E+01, 1.5485E+01, 8.3649E+02, &
      4.2482542E+00, 7.13507E+01, 1.5367E+01, 8.2606E+02, &
      4.2694955E+00, 7.13759E+01, 1.5250E+01, 8.1577E+02, &
      4.2908430E+00, 7.14002E+01, 1.5134E+01, 8.0562E+02, &
      4.3122972E+00, 7.14236E+01, 1.5019E+01, 7.9561E+02, &
      4.3338587E+00, 7.14461E+01, 1.4905E+01, 7.8573E+02, &
      4.3555280E+00, 7.14677E+01, 1.4792E+01, 7.7598E+02, &
      4.3773056E+00, 7.14886E+01, 1.4680E+01, 7.6637E+02, &
      4.3991921E+00, 7.15087E+01, 1.4570E+01, 7.5688E+02, &
      4.4211881E+00, 7.15280E+01, 1.4460E+01, 7.4752E+02, &
      4.4432940E+00, 7.15466E+01, 1.4351E+01, 7.3829E+02, &
      4.4655105E+00, 7.15645E+01, 1.4243E+01, 7.2917E+02, &
      4.4878381E+00, 7.15817E+01, 1.4136E+01, 7.2018E+02, &
      4.5102772E+00, 7.15983E+01, 1.4030E+01, 7.1131E+02, &
      4.5328286E+00, 7.16142E+01, 1.3925E+01, 7.0256E+02, &
      4.5554928E+00, 7.16295E+01, 1.3821E+01, 6.9393E+02, &
      4.5782702E+00, 7.16443E+01, 1.3718E+01, 6.8541E+02, &
      4.6011616E+00, 7.16585E+01, 1.3616E+01, 6.7700E+02, &
      4.6241674E+00, 7.16722E+01, 1.3515E+01, 6.6870E+02, &
      4.6472882E+00, 7.16854E+01, 1.3415E+01, 6.6052E+02, &
      4.6705247E+00, 7.16981E+01, 1.3315E+01, 6.5244E+02, &
      4.6938773E+00, 7.17103E+01, 1.3217E+01, 6.4447E+02, &
      4.7173467E+00, 7.17221E+01, 1.3119E+01, 6.3661E+02, &
      4.7409334E+00, 7.18385E+01, 1.3021E+01, 6.2879E+02, &
      4.7646381E+00, 7.18502E+01, 1.2921E+01, 6.2091E+02, &
      4.7884613E+00, 7.18611E+01, 1.2821E+01, 6.1315E+02, &
      4.8124036E+00, 7.18710E+01, 1.2723E+01, 6.0549E+02, &
      4.8364656E+00, 7.18802E+01, 1.2625E+01, 5.9793E+02, &
      4.8606479E+00, 7.18887E+01, 1.2528E+01, 5.9047E+02, &
      4.8849512E+00, 7.18964E+01, 1.2432E+01, 5.8311E+02, &
      4.9093759E+00, 7.19035E+01, 1.2336E+01, 5.7580E+02, &
      4.9339228E+00, 7.19098E+01, 1.2241E+01, 5.6859E+02, &
      4.9585924E+00, 7.19156E+01, 1.2147E+01, 5.6148E+02, &
      4.9833854E+00, 7.19207E+01, 1.2053E+01, 5.5446E+02, &
      5.0083023E+00, 7.19251E+01, 1.1961E+01, 5.4753E+02, &
      5.0333438E+00, 7.19290E+01, 1.1869E+01, 5.4070E+02, &
      5.0585105E+00, 7.19324E+01, 1.1778E+01, 5.3396E+02, &
      5.0838031E+00, 7.19352E+01, 1.1688E+01, 5.2731E+02, &
      5.1092221E+00, 7.19375E+01, 1.1598E+01, 5.2075E+02, &
      5.1347682E+00, 7.19392E+01, 1.1510E+01, 5.1428E+02, &
      5.1604421E+00, 7.19406E+01, 1.1421E+01, 5.0786E+02, &
      5.1862443E+00, 7.19414E+01, 1.1334E+01, 5.0153E+02, &
      5.2121755E+00, 7.19417E+01, 1.1247E+01, 4.9529E+02, &
      5.2382364E+00, 7.19416E+01, 1.1161E+01, 4.8913E+02, &
      5.2644276E+00, 7.19411E+01, 1.1076E+01, 4.8305E+02, &
      5.2907497E+00, 7.19401E+01, 1.0991E+01, 4.7705E+02, &
      5.3172034E+00, 7.19388E+01, 1.0907E+01, 4.7113E+02, &
      5.3437895E+00, 7.19370E+01, 1.0824E+01, 4.6529E+02, &
      5.3705084E+00, 7.19807E+01, 1.0742E+01, 4.5951E+02, &
      5.3973609E+00, 7.19786E+01, 1.0658E+01, 4.5373E+02, &
      5.4243477E+00, 7.19759E+01, 1.0575E+01, 4.4803E+02, &
      5.4514695E+00, 7.19727E+01, 1.0493E+01, 4.4240E+02, &
      5.4787268E+00, 7.19689E+01, 1.0411E+01, 4.3685E+02, &
      5.5061205E+00, 7.19646E+01, 1.0330E+01, 4.3138E+02, &
      5.5336511E+00, 7.19598E+01, 1.0250E+01, 4.2598E+02, &
      5.5613193E+00, 7.19544E+01, 1.0171E+01, 4.2065E+02, &
      5.5891259E+00, 7.19487E+01, 1.0093E+01, 4.1539E+02, &
      5.6170716E+00, 7.19425E+01, 1.0015E+01, 4.1020E+02, &
      5.6451569E+00, 7.19358E+01, 9.9374E+00, 4.0508E+02, &
      5.6733827E+00, 7.19287E+01, 9.8608E+00, 4.0003E+02, &
      5.7017496E+00, 7.19213E+01, 9.7850E+00, 3.9504E+02, &
      5.7302584E+00, 7.19134E+01, 9.7098E+00, 3.9012E+02, &
      5.7589096E+00, 7.19051E+01, 9.6353E+00, 3.8527E+02, &
      5.7877042E+00, 7.18965E+01, 9.5614E+00, 3.8048E+02, &
      5.8166427E+00, 7.18875E+01, 9.4882E+00, 3.7576E+02, &
      5.8457259E+00, 7.18782E+01, 9.4157E+00, 3.7109E+02, &
      5.8749546E+00, 7.18871E+01, 9.3434E+00, 3.6648E+02, &
      5.9043293E+00, 7.18773E+01, 9.2714E+00, 3.6191E+02, &
      5.9338510E+00, 7.18670E+01, 9.2000E+00, 3.5740E+02, &
      5.9635202E+00, 7.18563E+01, 9.1292E+00, 3.5295E+02, &
      5.9933378E+00, 7.18452E+01, 9.0590E+00, 3.4856E+02, &
      6.0233045E+00, 7.18337E+01, 8.9894E+00, 3.4423E+02, &
      6.0534210E+00, 7.18219E+01, 8.9205E+00, 3.3995E+02, &
      6.0836882E+00, 7.18096E+01, 8.8522E+00, 3.3573E+02, &
      6.1141066E+00, 7.17971E+01, 8.7844E+00, 3.3156E+02, &
      6.1446771E+00, 7.17841E+01, 8.7173E+00, 3.2746E+02, &
      6.1754005E+00, 7.17708E+01, 8.6507E+00, 3.2340E+02, &
      6.2062775E+00, 7.17572E+01, 8.5847E+00, 3.1940E+02, &
      6.2373089E+00, 7.17432E+01, 8.5193E+00, 3.1545E+02, &
      6.2684954E+00, 7.17289E+01, 8.4544E+00, 3.1155E+02, &
      6.2998379E+00, 7.17143E+01, 8.3902E+00, 3.0770E+02, &
      6.3313371E+00, 7.16994E+01, 8.3264E+00, 3.0391E+02, &
      6.3629938E+00, 7.16841E+01, 8.2633E+00, 3.0016E+02, &
      6.3948088E+00, 7.16685E+01, 8.2007E+00, 2.9646E+02, &
      6.4267828E+00, 7.16526E+01, 8.1386E+00, 2.9282E+02, &
      6.4589167E+00, 7.16363E+01, 8.0770E+00, 2.8921E+02, &
      6.4912113E+00, 7.16198E+01, 8.0160E+00, 2.8566E+02, &
      6.5236674E+00, 7.16029E+01, 7.9556E+00, 2.8215E+02, &
      6.5562857E+00, 7.15857E+01, 7.8956E+00, 2.7869E+02, &
      6.5890671E+00, 7.15683E+01, 7.8362E+00, 2.7527E+02, &
      6.6220125E+00, 7.15505E+01, 7.7772E+00, 2.7190E+02, &
      6.6551225E+00, 7.15324E+01, 7.7188E+00, 2.6857E+02, &
      6.6883981E+00, 7.15139E+01, 7.6609E+00, 2.6529E+02, &
      6.7218401E+00, 7.14952E+01, 7.6035E+00, 2.6205E+02, &
      6.7554493E+00, 7.14762E+01, 7.5466E+00, 2.5885E+02, &
      6.7892266E+00, 7.14568E+01, 7.4901E+00, 2.5569E+02, &
      6.8231727E+00, 7.14371E+01, 7.4342E+00, 2.5257E+02, &
      6.8572886E+00, 7.14172E+01, 7.3787E+00, 2.4949E+02, &
      6.8915750E+00, 7.13969E+01, 7.3237E+00, 2.4645E+02, &
      6.9260329E+00, 7.13762E+01, 7.2692E+00, 2.4346E+02, &
      6.9606631E+00, 7.13553E+01, 7.2151E+00, 2.4050E+02, &
      6.9954664E+00, 7.13340E+01, 7.1615E+00, 2.3758E+02, &
      7.0304437E+00, 7.13124E+01, 7.1084E+00, 2.3469E+02, &
      7.0655959E+00, 7.12905E+01, 7.0557E+00, 2.3185E+02, &
      7.1009239E+00, 7.12683E+01, 7.0034E+00, 2.2904E+02, &
      7.1364285E+00, 7.12457E+01, 6.9516E+00, 2.2626E+02, &
      7.1721107E+00, 7.12227E+01, 6.9003E+00, 2.2353E+02, &
      7.2079712E+00, 7.11994E+01, 6.8494E+00, 2.2082E+02, &
      7.2440111E+00, 7.11758E+01, 6.7989E+00, 2.1816E+02, &
      7.2802311E+00, 7.11518E+01, 6.7488E+00, 2.1552E+02, &
      7.3166323E+00, 7.11275E+01, 6.6992E+00, 2.1292E+02, &
      7.3532155E+00, 7.11027E+01, 6.6500E+00, 2.1036E+02, &
      7.3899815E+00, 7.10776E+01, 6.6011E+00, 2.0783E+02, &
      7.4269314E+00, 7.10521E+01, 6.5528E+00, 2.0532E+02, &
      7.4640661E+00, 7.10262E+01, 6.5048E+00, 2.0286E+02, &
      7.5013864E+00, 7.10000E+01, 6.4572E+00, 2.0042E+02, &
      7.5388934E+00, 7.09733E+01, 6.4100E+00, 1.9801E+02, &
      7.5765878E+00, 7.09462E+01, 6.3632E+00, 1.9564E+02, &
      7.6144708E+00, 7.09186E+01, 6.3169E+00, 1.9329E+02, &
      7.6525431E+00, 7.08907E+01, 6.2709E+00, 1.9098E+02, &
      7.6908058E+00, 7.08622E+01, 6.2252E+00, 1.8869E+02, &
      7.7292599E+00, 7.08334E+01, 6.1800E+00, 1.8644E+02, &
      7.7679062E+00, 7.08040E+01, 6.1352E+00, 1.8421E+02, &
      7.8067457E+00, 7.07742E+01, 6.0907E+00, 1.8201E+02, &
      7.8457794E+00, 7.07439E+01, 6.0466E+00, 1.7984E+02, &
      7.8850083E+00, 7.07130E+01, 6.0028E+00, 1.7770E+02, &
      7.9244334E+00, 7.06817E+01, 5.9595E+00, 1.7558E+02, &
      7.9640555E+00, 7.06498E+01, 5.9165E+00, 1.7349E+02, &
      8.0038758E+00, 7.06173E+01, 5.8738E+00, 1.7143E+02, &
      8.0438952E+00, 7.05843E+01, 5.8315E+00, 1.6939E+02, &
      8.0841147E+00, 7.05507E+01, 5.7896E+00, 1.6738E+02, &
      8.1245352E+00, 7.05164E+01, 5.7480E+00, 1.6539E+02, &
      8.1651579E+00, 7.04816E+01, 5.7065E+00, 1.6343E+02, &
      8.2059837E+00, 7.04460E+01, 5.6644E+00, 1.6146E+02, &
      8.2470136E+00, 7.04097E+01, 5.6227E+00, 1.5952E+02, &
      8.2882487E+00, 7.03726E+01, 5.5813E+00, 1.5760E+02, &
      8.3296899E+00, 7.03347E+01, 5.5403E+00, 1.5571E+02, &
      8.3713384E+00, 7.02959E+01, 5.4994E+00, 1.5383E+02, &
      8.4131951E+00, 7.02562E+01, 5.4588E+00, 1.5198E+02, &
      8.4552610E+00, 7.02157E+01, 5.4183E+00, 1.5014E+02, &
      8.4975373E+00, 7.01741E+01, 5.3776E+00, 1.4832E+02, &
      8.5400250E+00, 7.01315E+01, 5.3372E+00, 1.4651E+02, &
      9.030794E+00, 6.96381E+01, 4.8913E+00, 1.2747E+02, &
      9.653919E+00, 6.85353E+01, 4.3617E+00, 1.0687E+02, &
      1.032004E+01, 6.56674E+01, 3.8942E+00, 8.9759E+01, &
      1.032459E+01, 6.56244E+01, 3.8912E+00, 8.9653E+01, &
      1.048262E+01, 6.28399E+01, 3.7895E+00, 8.6112E+01, &
      1.052476E+01, 5.95870E+01, 3.7631E+00, 8.5202E+01, &
      1.058798E+01, 6.28140E+01, 1.0075E+01, 2.1941E+02, &
      1.074601E+01, 6.56471E+01, 9.8178E+00, 2.1078E+02, &
      1.103212E+01, 6.72448E+01, 9.3788E+00, 1.9633E+02, &
      1.171953E+01, 6.76542E+01, 8.4363E+00, 1.6664E+02, &
      1.179334E+01, 6.73841E+01, 8.3432E+00, 1.6381E+02, &
      1.189891E+01, 6.64793E+01, 8.2128E+00, 1.5988E+02, &
      1.194674E+01, 6.48867E+01, 8.1547E+00, 1.5815E+02, &
      1.201849E+01, 6.65730E+01, 1.1272E+01, 2.1582E+02, &
      1.219787E+01, 6.80807E+01, 1.0988E+01, 2.0737E+02, &
      1.227617E+01, 6.83474E+01, 1.0867E+01, 2.0383E+02, &
      1.246407E+01, 6.83508E+01, 1.0586E+01, 1.9566E+02, &
      1.251417E+01, 6.76623E+01, 1.0513E+01, 1.9357E+02, &
      1.258933E+01, 6.87093E+01, 1.2059E+01, 2.2015E+02, &
      1.260708E+01, 6.88899E+01, 1.2033E+01, 2.1938E+02, &
      1.277723E+01, 6.99110E+01, 1.1785E+01, 2.1207E+02, &
      1.347697E+01, 7.18024E+01, 1.0844E+01, 1.8527E+02, &
      1.440688E+01, 7.30764E+01, 9.7497E+00, 1.5612E+02, &
      1.540095E+01, 7.38716E+01, 8.7608E+00, 1.3152E+02, &
      1.646362E+01, 7.44047E+01, 7.8621E+00, 1.1067E+02, &
      1.759961E+01, 7.47592E+01, 7.0514E+00, 9.3095E+01, &
      1.881398E+01, 7.49938E+01, 6.3267E+00, 7.8355E+01, &
      2.011215E+01, 7.51456E+01, 5.6795E+00, 6.5997E+01, &
      2.149988E+01, 7.53012E+01, 5.0980E+00, 5.5597E+01, &
      2.298338E+01, 7.53469E+01, 4.5723E+00, 4.6812E+01, &
      2.456923E+01, 7.53954E+01, 4.1022E+00, 3.9438E+01, &
      2.626450E+01, 7.54193E+01, 3.6772E+00, 3.3207E+01, &
      2.807676E+01, 7.53939E+01, 3.2705E+00, 2.7764E+01, &
      3.001405E+01, 7.53316E+01, 2.9034E+00, 2.3181E+01, &
      3.208502E+01, 7.52520E+01, 2.5796E+00, 1.9380E+01, &
      3.429889E+01, 7.51615E+01, 2.2939E+00, 1.6224E+01, &
      3.666551E+01, 7.50638E+01, 2.0415E+00, 1.3599E+01, &
      3.919543E+01, 7.49608E+01, 1.8183E+00, 1.1414E+01, &
      4.189992E+01, 7.48531E+01, 1.6208E+00, 9.5927E+00, &
      4.479101E+01, 7.47403E+01, 1.4457E+00, 8.0731E+00, &
      4.788159E+01, 7.46201E+01, 1.2884E+00, 6.7935E+00, &
      5.118542E+01, 7.44871E+01, 1.1483E+00, 5.7221E+00, &
      5.471721E+01, 7.43343E+01, 1.0222E+00, 4.8190E+00, &
      5.849270E+01, 7.41439E+01, 9.0681E-01, 4.0506E+00, &
      6.252870E+01, 7.38796E+01, 8.0487E-01, 3.4104E+00, &
      6.684318E+01, 7.34235E+01, 7.1433E-01, 2.8754E+00, &
      7.024287E+01, 7.25172E+01, 6.5400E-01, 2.5362E+00, &
      7.131802E+01, 7.14235E+01, 6.3662E-01, 2.4411E+00, &
      7.145536E+01, 7.10332E+01, 6.3445E-01, 2.4293E+00, &
      7.160472E+01, 7.01142E+01, 6.3211E-01, 2.4166E+00, &
      7.203478E+01, 7.14371E+01, 3.2029E+00, 1.0467E+01, &
      7.310993E+01, 7.26320E+01, 3.1325E+00, 1.0094E+01, &
      7.638578E+01, 7.36992E+01, 2.9282E+00, 9.0518E+00, &
      8.165640E+01, 7.43552E+01, 2.6232E+00, 7.6181E+00, &
      8.729069E+01, 7.47028E+01, 2.3520E+00, 6.4196E+00, &
      9.331374E+01, 7.49250E+01, 2.1121E+00, 5.4207E+00, &
      9.975239E+01, 7.50733E+01, 1.8873E+00, 4.5586E+00, &
      1.066353E+02, 7.51704E+01, 1.6866E+00, 3.8370E+00, &
      1.139931E+02, 7.52342E+01, 1.5075E+00, 3.2327E+00, &
      1.218587E+02, 7.52751E+01, 1.3476E+00, 2.7267E+00, &
      1.302669E+02, 7.52997E+01, 1.2048E+00, 2.3027E+00, &
      1.392553E+02, 7.53128E+01, 1.0772E+00, 1.9473E+00, &
      1.488640E+02, 7.53443E+01, 9.6158E-01, 1.6465E+00, &
      1.591356E+02, 7.53404E+01, 8.5722E-01, 1.3930E+00, &
      1.701159E+02, 7.53313E+01, 7.6426E-01, 1.1809E+00, &
      1.818539E+02, 7.53187E+01, 6.8146E-01, 1.0033E+00, &
      1.944018E+02, 7.53038E+01, 6.0768E-01, 8.5453E-01, &
      2.078156E+02, 7.52875E+01, 5.4194E-01, 7.2982E-01, &
      2.221548E+02, 7.52704E+01, 4.8335E-01, 6.2519E-01, &
      2.374835E+02, 7.52530E+01, 4.3114E-01, 5.3733E-01, &
      2.538699E+02, 7.52356E+01, 3.8460E-01, 4.6349E-01, &
      2.713869E+02, 7.52184E+01, 3.4311E-01, 4.0135E-01, &
      2.901126E+02, 7.52015E+01, 3.0612E-01, 3.4899E-01, &
      3.101304E+02, 7.51849E+01, 2.7314E-01, 3.0479E-01, &
      3.315294E+02, 7.51693E+01, 2.4564E-01, 2.6873E-01, &
      3.544049E+02, 7.51553E+01, 2.2097E-01, 2.3799E-01, &
      3.788588E+02, 7.51425E+01, 1.9879E-01, 2.1170E-01, &
      4.050001E+02, 7.51308E+01, 1.7886E-01, 1.8916E-01, &
      4.329451E+02, 7.51199E+01, 1.6093E-01, 1.6979E-01]
   case(76)
      frel = -9.1920E-01
      allocate(nist_tab(3060))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.907383E-03, 5.43687E+00, 0.0000E+00, 1.5995E-05, &
      5.997802E-03, 5.15071E+00, 0.0000E+00, 1.7314E-05, &
      6.021914E-03, 4.85715E+00, 0.0000E+00, 1.7679E-05, &
      6.058082E-03, 5.12209E+00, 5.4897E-01, 2.0049E+04, &
      6.148501E-03, 5.31598E+00, 5.4399E-01, 1.9575E+04, &
      6.911593E-03, 4.82996E+00, 5.1397E-01, 1.6452E+04, &
      7.017383E-03, 4.35271E+00, 5.1580E-01, 1.6262E+04, &
      7.045593E-03, 3.81101E+00, 5.1659E-01, 1.6222E+04, &
      7.087909E-03, 4.34862E+00, 1.5660E+00, 4.8882E+04, &
      7.193699E-03, 4.79431E+00, 1.5545E+00, 4.7807E+04, &
      1.069000E-02, 4.06001E+00, 2.8563E+00, 5.9114E+04, &
      1.142761E-02, 3.83030E+00, 3.4898E+00, 6.7564E+04, &
      1.221612E-02, 3.67817E+00, 4.2589E+00, 7.7132E+04, &
      1.305903E-02, 3.64823E+00, 5.1556E+00, 8.7344E+04, &
      1.396010E-02, 3.78702E+00, 6.1562E+00, 9.7565E+04, &
      1.492335E-02, 4.13658E+00, 7.2192E+00, 1.0703E+05, &
      1.595306E-02, 4.72647E+00, 8.2856E+00, 1.1491E+05, &
      1.705382E-02, 5.56591E+00, 9.2837E+00, 1.2044E+05, &
      1.823053E-02, 6.66196E+00, 1.0137E+01, 1.2303E+05, &
      1.948844E-02, 7.92370E+00, 1.0778E+01, 1.2235E+05, &
      2.083314E-02, 9.30795E+00, 1.1152E+01, 1.1843E+05, &
      2.227063E-02, 1.07730E+01, 1.1234E+01, 1.1160E+05, &
      2.380730E-02, 1.21547E+01, 1.1025E+01, 1.0246E+05, &
      2.545001E-02, 1.34151E+01, 1.0553E+01, 9.1742E+04, &
      2.720606E-02, 1.44901E+01, 9.8659E+00, 8.0230E+04, &
      2.908327E-02, 1.53372E+01, 9.0253E+00, 6.8657E+04, &
      3.109002E-02, 1.59315E+01, 8.0922E+00, 5.7586E+04, &
      3.323523E-02, 1.62636E+01, 7.1306E+00, 4.7467E+04, &
      3.552846E-02, 1.63346E+01, 6.1929E+00, 3.8564E+04, &
      3.797993E-02, 1.61358E+01, 5.3160E+00, 3.0967E+04, &
      4.060054E-02, 1.56027E+01, 4.5225E+00, 2.4644E+04, &
      4.340198E-02, 1.43750E+01, 3.8227E+00, 1.9486E+04, &
      4.449200E-02, 1.32731E+01, 3.5870E+00, 1.7837E+04, &
      4.517300E-02, 1.13899E+01, 3.4490E+00, 1.6892E+04, &
      4.537400E-02, 8.51787E+00, 3.4096E+00, 1.6625E+04, &
      4.562700E-02, 1.13463E+01, 7.4612E+00, 3.6179E+04, &
      4.606850E-02, 1.25223E+01, 7.2925E+00, 3.5022E+04, &
      4.625370E-02, 1.24059E+01, 7.2233E+00, 3.4551E+04, &
      4.630800E-02, 1.19946E+01, 8.0959E+00, 3.8679E+04, &
      4.639671E-02, 1.28348E+01, 8.0585E+00, 3.8427E+04, &
      4.653150E-02, 1.32592E+01, 8.0023E+00, 3.8048E+04, &
      4.722600E-02, 1.42971E+01, 7.7210E+00, 3.6171E+04, &
      4.959809E-02, 1.55895E+01, 6.8584E+00, 3.0593E+04, &
      5.302035E-02, 1.59956E+01, 5.8309E+00, 2.4331E+04, &
      5.667876E-02, 1.53856E+01, 4.9842E+00, 1.9455E+04, &
      5.684000E-02, 1.53069E+01, 4.9520E+00, 1.9275E+04, &
      5.771000E-02, 1.44436E+01, 4.7851E+00, 1.8345E+04, &
      5.794200E-02, 1.34254E+01, 4.7425E+00, 1.8109E+04, &
      5.829000E-02, 1.44565E+01, 6.6712E+00, 2.5321E+04, &
      5.916000E-02, 1.53382E+01, 6.5044E+00, 2.4325E+04, &
      6.058959E-02, 1.58224E+01, 6.2525E+00, 2.2831E+04, &
      6.477028E-02, 1.62596E+01, 5.6534E+00, 1.9311E+04, &
      6.923942E-02, 1.63067E+01, 5.1344E+00, 1.6406E+04, &
      7.401695E-02, 1.61153E+01, 4.6965E+00, 1.4038E+04, &
      7.912411E-02, 1.57335E+01, 4.3797E+00, 1.2246E+04, &
      8.202600E-02, 1.54516E+01, 4.2706E+00, 1.1519E+04, &
      8.328150E-02, 1.52980E+01, 4.2389E+00, 1.1261E+04, &
      8.361630E-02, 1.52257E+01, 4.2320E+00, 1.1197E+04, &
      8.411850E-02, 1.52156E+01, 4.3005E+00, 1.1311E+04, &
      8.458368E-02, 1.51878E+01, 4.2938E+00, 1.1231E+04, &
      8.537400E-02, 1.51242E+01, 4.2852E+00, 1.1105E+04, &
      9.041995E-02, 1.46327E+01, 4.3116E+00, 1.0550E+04, &
      9.665893E-02, 1.39803E+01, 4.5235E+00, 1.0354E+04, &
      1.033284E-01, 1.32929E+01, 4.9436E+00, 1.0586E+04, &
      1.104581E-01, 1.26108E+01, 5.5827E+00, 1.1183E+04, &
      1.180797E-01, 1.19764E+01, 6.4502E+00, 1.2086E+04, &
      1.262272E-01, 1.14366E+01, 7.5436E+00, 1.3223E+04, &
      1.349368E-01, 1.10393E+01, 8.8536E+00, 1.4517E+04, &
      1.442475E-01, 1.08266E+01, 1.0362E+01, 1.5894E+04, &
      1.542005E-01, 1.08645E+01, 1.2028E+01, 1.7258E+04, &
      1.648404E-01, 1.11817E+01, 1.3805E+01, 1.8529E+04, &
      1.762144E-01, 1.18116E+01, 1.5635E+01, 1.9633E+04, &
      1.883732E-01, 1.27716E+01, 1.7455E+01, 2.0502E+04, &
      2.013709E-01, 1.40558E+01, 1.9145E+01, 2.1036E+04, &
      2.152655E-01, 1.55949E+01, 2.0628E+01, 2.1203E+04, &
      2.301188E-01, 1.73100E+01, 2.1867E+01, 2.1025E+04, &
      2.459970E-01, 1.91095E+01, 2.2842E+01, 2.0546E+04, &
      2.629708E-01, 2.08401E+01, 2.3542E+01, 1.9809E+04, &
      2.673440E-01, 2.11921E+01, 2.3659E+01, 1.9583E+04, &
      2.714360E-01, 2.13260E+01, 2.3751E+01, 1.9362E+04, &
      2.725272E-01, 2.10971E+01, 2.3772E+01, 1.9302E+04, &
      2.741640E-01, 2.15895E+01, 2.4465E+01, 1.9746E+04, &
      2.782560E-01, 2.22684E+01, 2.4553E+01, 1.9525E+04, &
      2.811158E-01, 2.26139E+01, 2.4606E+01, 1.9368E+04, &
      2.836120E-01, 2.28744E+01, 2.4646E+01, 1.9229E+04, &
      2.879530E-01, 2.31714E+01, 2.4704E+01, 1.8984E+04, &
      2.891106E-01, 2.30454E+01, 2.4717E+01, 1.8918E+04, &
      2.908470E-01, 2.34970E+01, 2.5243E+01, 1.9205E+04, &
      2.951880E-01, 2.41977E+01, 2.5291E+01, 1.8959E+04, &
      3.005128E-01, 2.48701E+01, 2.5334E+01, 1.8655E+04, &
      3.212482E-01, 2.70645E+01, 2.5351E+01, 1.7463E+04, &
      3.434143E-01, 2.90260E+01, 2.5183E+01, 1.6228E+04, &
      3.671099E-01, 3.08046E+01, 2.4870E+01, 1.4993E+04, &
      3.924405E-01, 3.23961E+01, 2.4446E+01, 1.3787E+04, &
      4.195189E-01, 3.37687E+01, 2.3940E+01, 1.2630E+04, &
      4.484657E-01, 3.47750E+01, 2.3377E+01, 1.1538E+04, &
      4.588360E-01, 3.48718E+01, 2.3174E+01, 1.1180E+04, &
      4.658590E-01, 3.44856E+01, 2.3037E+01, 1.0947E+04, &
      4.677318E-01, 3.37681E+01, 2.3000E+01, 1.0885E+04, &
      4.705410E-01, 3.46743E+01, 2.4498E+01, 1.1525E+04, &
      4.775640E-01, 3.56373E+01, 2.4371E+01, 1.1296E+04, &
      4.794098E-01, 3.57982E+01, 2.4338E+01, 1.1238E+04, &
      5.0000000E-01, 3.84805E+01, 2.3970E+01, 1.0612E+04, &
      5.0250000E-01, 3.86059E+01, 2.3926E+01, 1.0540E+04, &
      5.0501250E-01, 3.87279E+01, 2.3881E+01, 1.0468E+04, &
      5.0753756E-01, 3.88468E+01, 2.3837E+01, 1.0397E+04, &
      5.1007525E-01, 3.89629E+01, 2.3792E+01, 1.0326E+04, &
      5.1262563E-01, 3.90762E+01, 2.3747E+01, 1.0255E+04, &
      5.1518875E-01, 3.91868E+01, 2.3702E+01, 1.0185E+04, &
      5.1776470E-01, 3.92949E+01, 2.3658E+01, 1.0115E+04, &
      5.2035352E-01, 3.94002E+01, 2.3612E+01, 1.0046E+04, &
      5.2295529E-01, 3.95028E+01, 2.3567E+01, 9.9769E+03, &
      5.2557007E-01, 3.96023E+01, 2.3522E+01, 9.9082E+03, &
      5.2819792E-01, 3.96985E+01, 2.3476E+01, 9.8399E+03, &
      5.3083891E-01, 3.97909E+01, 2.3431E+01, 9.7719E+03, &
      5.3349310E-01, 3.98784E+01, 2.3385E+01, 9.7044E+03, &
      5.3616057E-01, 3.99595E+01, 2.3339E+01, 9.6372E+03, &
      5.3884137E-01, 4.00312E+01, 2.3293E+01, 9.5704E+03, &
      5.4153558E-01, 4.00860E+01, 2.3247E+01, 9.5040E+03, &
      5.4424325E-01, 4.00981E+01, 2.3200E+01, 9.4379E+03, &
      5.4571853E-01, 4.00302E+01, 2.3175E+01, 9.4022E+03, &
      5.4696447E-01, 4.00160E+01, 2.3520E+01, 9.5202E+03, &
      5.4728152E-01, 4.00883E+01, 2.3514E+01, 9.5126E+03, &
      5.4969929E-01, 4.03421E+01, 2.3475E+01, 9.4549E+03, &
      5.5244779E-01, 4.05153E+01, 2.3430E+01, 9.3899E+03, &
      5.5521003E-01, 4.06601E+01, 2.3385E+01, 9.3253E+03, &
      5.5798608E-01, 4.07920E+01, 2.3340E+01, 9.2611E+03, &
      5.6077601E-01, 4.09163E+01, 2.3295E+01, 9.1972E+03, &
      5.6357989E-01, 4.10353E+01, 2.3249E+01, 9.1337E+03, &
      5.6639779E-01, 4.11504E+01, 2.3204E+01, 9.0705E+03, &
      5.6922978E-01, 4.12625E+01, 2.3158E+01, 9.0077E+03, &
      5.7207593E-01, 4.13720E+01, 2.3112E+01, 8.9453E+03, &
      5.7493630E-01, 4.14793E+01, 2.3066E+01, 8.8831E+03, &
      5.7781099E-01, 4.15847E+01, 2.3020E+01, 8.8214E+03, &
      5.8070004E-01, 4.16883E+01, 2.2974E+01, 8.7600E+03, &
      5.8360354E-01, 4.17903E+01, 2.2928E+01, 8.6989E+03, &
      5.8652156E-01, 4.18908E+01, 2.2881E+01, 8.6382E+03, &
      5.8945417E-01, 4.19898E+01, 2.2835E+01, 8.5778E+03, &
      5.9240144E-01, 4.20874E+01, 2.2788E+01, 8.5177E+03, &
      5.9536345E-01, 4.21836E+01, 2.2741E+01, 8.4580E+03, &
      5.9834026E-01, 4.22784E+01, 2.2694E+01, 8.3987E+03, &
      6.0133196E-01, 4.23719E+01, 2.2647E+01, 8.3396E+03, &
      6.0433862E-01, 4.24640E+01, 2.2600E+01, 8.2810E+03, &
      6.0736032E-01, 4.25547E+01, 2.2553E+01, 8.2226E+03, &
      6.1039712E-01, 4.26439E+01, 2.2505E+01, 8.1645E+03, &
      6.1344910E-01, 4.27315E+01, 2.2458E+01, 8.1068E+03, &
      6.1651635E-01, 4.28174E+01, 2.2410E+01, 8.0494E+03, &
      6.1959893E-01, 4.29014E+01, 2.2362E+01, 7.9924E+03, &
      6.2269693E-01, 4.29834E+01, 2.2314E+01, 7.9356E+03, &
      6.2581041E-01, 4.30630E+01, 2.2266E+01, 7.8792E+03, &
      6.2893946E-01, 4.31398E+01, 2.2218E+01, 7.8230E+03, &
      6.3208416E-01, 4.32133E+01, 2.2169E+01, 7.7672E+03, &
      6.3524458E-01, 4.32827E+01, 2.2121E+01, 7.7117E+03, &
      6.3842080E-01, 4.33465E+01, 2.2072E+01, 7.6566E+03, &
      6.4161291E-01, 4.34026E+01, 2.2023E+01, 7.6017E+03, &
      6.4482097E-01, 4.34467E+01, 2.1974E+01, 7.5471E+03, &
      6.4804508E-01, 4.34690E+01, 2.1925E+01, 7.4929E+03, &
      6.5128530E-01, 4.34371E+01, 2.1876E+01, 7.4389E+03, &
      6.5327272E-01, 4.33097E+01, 2.1846E+01, 7.4061E+03, &
      6.5454173E-01, 4.30989E+01, 2.2362E+01, 7.5662E+03, &
      6.5532723E-01, 4.33672E+01, 2.2350E+01, 7.5532E+03, &
      6.5781444E-01, 4.36460E+01, 2.2313E+01, 7.5123E+03, &
      6.6110351E-01, 4.38496E+01, 2.2264E+01, 7.4586E+03, &
      6.6440903E-01, 4.40079E+01, 2.2215E+01, 7.4053E+03, &
      6.6773107E-01, 4.41467E+01, 2.2166E+01, 7.3523E+03, &
      6.7106973E-01, 4.42746E+01, 2.2117E+01, 7.2996E+03, &
      6.7442508E-01, 4.43953E+01, 2.2068E+01, 7.2472E+03, &
      6.7779720E-01, 4.45110E+01, 2.2018E+01, 7.1950E+03, &
      6.8118619E-01, 4.46228E+01, 2.1969E+01, 7.1432E+03, &
      6.8459212E-01, 4.47317E+01, 2.1919E+01, 7.0917E+03, &
      6.8801508E-01, 4.48380E+01, 2.1869E+01, 7.0404E+03, &
      6.9145515E-01, 4.49424E+01, 2.1819E+01, 6.9895E+03, &
      6.9491243E-01, 4.50449E+01, 2.1769E+01, 6.9388E+03, &
      6.9838699E-01, 4.51459E+01, 2.1719E+01, 6.8884E+03, &
      7.0187893E-01, 4.52455E+01, 2.1669E+01, 6.8384E+03, &
      7.0538832E-01, 4.53440E+01, 2.1618E+01, 6.7886E+03, &
      7.0891526E-01, 4.54413E+01, 2.1568E+01, 6.7391E+03, &
      7.1245984E-01, 4.55376E+01, 2.1517E+01, 6.6899E+03, &
      7.1602214E-01, 4.56329E+01, 2.1466E+01, 6.6409E+03, &
      7.1960225E-01, 4.57275E+01, 2.1415E+01, 6.5923E+03, &
      7.2320026E-01, 4.58212E+01, 2.1364E+01, 6.5439E+03, &
      7.2681626E-01, 4.59141E+01, 2.1313E+01, 6.4958E+03, &
      7.3045034E-01, 4.60064E+01, 2.1261E+01, 6.4480E+03, &
      7.3410260E-01, 4.60980E+01, 2.1210E+01, 6.4005E+03, &
      7.3777311E-01, 4.61889E+01, 2.1158E+01, 6.3532E+03, &
      7.4146197E-01, 4.62792E+01, 2.1106E+01, 6.3062E+03, &
      7.4516928E-01, 4.63690E+01, 2.1054E+01, 6.2594E+03, &
      7.4889513E-01, 4.64581E+01, 2.1002E+01, 6.2130E+03, &
      7.5263961E-01, 4.65467E+01, 2.0950E+01, 6.1668E+03, &
      7.5640280E-01, 4.66348E+01, 2.0898E+01, 6.1208E+03, &
      7.6018482E-01, 4.67224E+01, 2.0845E+01, 6.0752E+03, &
      7.6398574E-01, 4.68095E+01, 2.0792E+01, 6.0298E+03, &
      7.6780567E-01, 4.68961E+01, 2.0740E+01, 5.9846E+03, &
      7.7164470E-01, 4.69823E+01, 2.0687E+01, 5.9397E+03, &
      7.7550292E-01, 4.70681E+01, 2.0634E+01, 5.8951E+03, &
      7.7938044E-01, 4.71535E+01, 2.0581E+01, 5.8508E+03, &
      7.8327734E-01, 4.72384E+01, 2.0527E+01, 5.8067E+03, &
      7.8719373E-01, 4.73231E+01, 2.0474E+01, 5.7629E+03, &
      7.9112969E-01, 4.74074E+01, 2.0420E+01, 5.7193E+03, &
      7.9508534E-01, 4.74912E+01, 2.0366E+01, 5.6757E+03, &
      7.9906077E-01, 4.75745E+01, 2.0311E+01, 5.6324E+03, &
      8.0305607E-01, 4.76574E+01, 2.0256E+01, 5.5893E+03, &
      8.0707135E-01, 4.77398E+01, 2.0201E+01, 5.5465E+03, &
      8.1110671E-01, 4.78217E+01, 2.0146E+01, 5.5039E+03, &
      8.1516224E-01, 4.79032E+01, 2.0091E+01, 5.4616E+03, &
      8.1923806E-01, 4.80002E+01, 2.0035E+01, 5.4195E+03, &
      8.2333425E-01, 4.80811E+01, 1.9980E+01, 5.3777E+03, &
      8.2745092E-01, 4.81617E+01, 1.9924E+01, 5.3361E+03, &
      8.3158817E-01, 4.82418E+01, 1.9868E+01, 5.2948E+03, &
      8.3574611E-01, 4.83216E+01, 1.9812E+01, 5.2538E+03, &
      8.3992484E-01, 4.84011E+01, 1.9756E+01, 5.2130E+03, &
      8.4412447E-01, 4.84802E+01, 1.9700E+01, 5.1724E+03, &
      8.4834509E-01, 4.85591E+01, 1.9644E+01, 5.1321E+03, &
      8.5258682E-01, 4.86376E+01, 1.9588E+01, 5.0920E+03, &
      8.5684975E-01, 4.87158E+01, 1.9532E+01, 5.0522E+03, &
      8.6113400E-01, 4.87938E+01, 1.9475E+01, 5.0126E+03, &
      8.6543967E-01, 4.88715E+01, 1.9418E+01, 4.9732E+03, &
      8.6976687E-01, 4.89580E+01, 1.9360E+01, 4.9338E+03, &
      8.7411570E-01, 4.90356E+01, 1.9284E+01, 4.8901E+03, &
      8.7848628E-01, 4.91121E+01, 1.9209E+01, 4.8468E+03, &
      8.8287871E-01, 4.91876E+01, 1.9133E+01, 4.8038E+03, &
      8.8729310E-01, 4.92620E+01, 1.9058E+01, 4.7611E+03, &
      8.9172957E-01, 4.93355E+01, 1.8982E+01, 4.7188E+03, &
      8.9618822E-01, 4.94081E+01, 1.8907E+01, 4.6768E+03, &
      9.0066916E-01, 4.94797E+01, 1.8831E+01, 4.6351E+03, &
      9.0517250E-01, 4.95505E+01, 1.8756E+01, 4.5937E+03, &
      9.0969837E-01, 4.96204E+01, 1.8681E+01, 4.5526E+03, &
      9.1424686E-01, 4.96895E+01, 1.8606E+01, 4.5119E+03, &
      9.1881809E-01, 4.97578E+01, 1.8531E+01, 4.4715E+03, &
      9.2341218E-01, 4.98253E+01, 1.8456E+01, 4.4314E+03, &
      9.2802924E-01, 4.98921E+01, 1.8381E+01, 4.3915E+03, &
      9.3266939E-01, 4.99581E+01, 1.8307E+01, 4.3520E+03, &
      9.3733274E-01, 5.00233E+01, 1.8232E+01, 4.3127E+03, &
      9.4201940E-01, 5.00879E+01, 1.8157E+01, 4.2738E+03, &
      9.4672950E-01, 5.01518E+01, 1.8083E+01, 4.2352E+03, &
      9.5146315E-01, 5.02151E+01, 1.8008E+01, 4.1970E+03, &
      9.5622046E-01, 5.02779E+01, 1.7934E+01, 4.1590E+03, &
      9.6100156E-01, 5.03401E+01, 1.7860E+01, 4.1214E+03, &
      9.6580657E-01, 5.04019E+01, 1.7787E+01, 4.0840E+03, &
      9.7063560E-01, 5.04632E+01, 1.7713E+01, 4.0470E+03, &
      9.7548878E-01, 5.05242E+01, 1.7640E+01, 4.0103E+03, &
      9.8036623E-01, 5.05850E+01, 1.7566E+01, 3.9739E+03, &
      9.8526806E-01, 5.06457E+01, 1.7493E+01, 3.9378E+03, &
      9.9019440E-01, 5.07064E+01, 1.7421E+01, 3.9020E+03, &
      9.9514537E-01, 5.07674E+01, 1.7348E+01, 3.8665E+03, &
      1.0001211E+00, 5.08291E+01, 1.7275E+01, 3.8312E+03, &
      1.0051217E+00, 5.08794E+01, 1.7180E+01, 3.7912E+03, &
      1.0101473E+00, 5.09231E+01, 1.7084E+01, 3.7515E+03, &
      1.0151980E+00, 5.09648E+01, 1.6990E+01, 3.7123E+03, &
      1.0202740E+00, 5.10051E+01, 1.6896E+01, 3.6736E+03, &
      1.0253754E+00, 5.10441E+01, 1.6803E+01, 3.6352E+03, &
      1.0305023E+00, 5.10818E+01, 1.6710E+01, 3.5973E+03, &
      1.0356548E+00, 5.11182E+01, 1.6618E+01, 3.5598E+03, &
      1.0408331E+00, 5.11535E+01, 1.6526E+01, 3.5227E+03, &
      1.0460372E+00, 5.11875E+01, 1.6435E+01, 3.4860E+03, &
      1.0512674E+00, 5.12205E+01, 1.6345E+01, 3.4497E+03, &
      1.0565238E+00, 5.12523E+01, 1.6255E+01, 3.4138E+03, &
      1.0618064E+00, 5.12830E+01, 1.6166E+01, 3.3783E+03, &
      1.0671154E+00, 5.13127E+01, 1.6078E+01, 3.3433E+03, &
      1.0724510E+00, 5.13414E+01, 1.5990E+01, 3.3086E+03, &
      1.0778132E+00, 5.13692E+01, 1.5903E+01, 3.2743E+03, &
      1.0832023E+00, 5.13959E+01, 1.5816E+01, 3.2404E+03, &
      1.0886183E+00, 5.14218E+01, 1.5731E+01, 3.2069E+03, &
      1.0940614E+00, 5.14468E+01, 1.5640E+01, 3.1727E+03, &
      1.0995317E+00, 5.14702E+01, 1.5544E+01, 3.1378E+03, &
      1.1050294E+00, 5.14919E+01, 1.5450E+01, 3.1033E+03, &
      1.1105545E+00, 5.15120E+01, 1.5356E+01, 3.0692E+03, &
      1.1161073E+00, 5.15306E+01, 1.5262E+01, 3.0355E+03, &
      1.1216878E+00, 5.15475E+01, 1.5170E+01, 3.0022E+03, &
      1.1272963E+00, 5.15630E+01, 1.5078E+01, 2.9693E+03, &
      1.1329328E+00, 5.15770E+01, 1.4987E+01, 2.9368E+03, &
      1.1385974E+00, 5.15897E+01, 1.4896E+01, 2.9047E+03, &
      1.1442904E+00, 5.16009E+01, 1.4807E+01, 2.8729E+03, &
      1.1500119E+00, 5.16108E+01, 1.4718E+01, 2.8416E+03, &
      1.1557619E+00, 5.16194E+01, 1.4630E+01, 2.8107E+03, &
      1.1615407E+00, 5.16267E+01, 1.4538E+01, 2.7792E+03, &
      1.1673484E+00, 5.16322E+01, 1.4444E+01, 2.7478E+03, &
      1.1731852E+00, 5.16359E+01, 1.4352E+01, 2.7167E+03, &
      1.1790511E+00, 5.16380E+01, 1.4260E+01, 2.6860E+03, &
      1.1849464E+00, 5.16384E+01, 1.4169E+01, 2.6557E+03, &
      1.1908711E+00, 5.16372E+01, 1.4079E+01, 2.6258E+03, &
      1.1968254E+00, 5.16344E+01, 1.3989E+01, 2.5963E+03, &
      1.2028096E+00, 5.16300E+01, 1.3900E+01, 2.5671E+03, &
      1.2088236E+00, 5.16240E+01, 1.3812E+01, 2.5383E+03, &
      1.2148677E+00, 5.16166E+01, 1.3725E+01, 2.5098E+03, &
      1.2209421E+00, 5.16076E+01, 1.3639E+01, 2.4818E+03, &
      1.2270468E+00, 5.15971E+01, 1.3553E+01, 2.4540E+03, &
      1.2331820E+00, 5.15851E+01, 1.3468E+01, 2.4266E+03, &
      1.2393479E+00, 5.15716E+01, 1.3384E+01, 2.3995E+03, &
      1.2455447E+00, 5.15566E+01, 1.3300E+01, 2.3728E+03, &
      1.2517724E+00, 5.15402E+01, 1.3217E+01, 2.3464E+03, &
      1.2580312E+00, 5.15223E+01, 1.3135E+01, 2.3203E+03, &
      1.2643214E+00, 5.15030E+01, 1.3053E+01, 2.2946E+03, &
      1.2706430E+00, 5.14822E+01, 1.2972E+01, 2.2692E+03, &
      1.2769962E+00, 5.14599E+01, 1.2892E+01, 2.2441E+03, &
      1.2833812E+00, 5.14362E+01, 1.2813E+01, 2.2192E+03, &
      1.2897981E+00, 5.14110E+01, 1.2734E+01, 2.1947E+03, &
      1.2962471E+00, 5.13844E+01, 1.2656E+01, 2.1705E+03, &
      1.3027283E+00, 5.13562E+01, 1.2578E+01, 2.1466E+03, &
      1.3092420E+00, 5.13266E+01, 1.2501E+01, 2.1230E+03, &
      1.3157882E+00, 5.12955E+01, 1.2425E+01, 2.0997E+03, &
      1.3223671E+00, 5.12629E+01, 1.2349E+01, 2.0766E+03, &
      1.3289790E+00, 5.12288E+01, 1.2274E+01, 2.0539E+03, &
      1.3356239E+00, 5.11931E+01, 1.2200E+01, 2.0314E+03, &
      1.3423020E+00, 5.11559E+01, 1.2126E+01, 2.0092E+03, &
      1.3490135E+00, 5.11172E+01, 1.2053E+01, 1.9873E+03, &
      1.3557586E+00, 5.10768E+01, 1.1980E+01, 1.9656E+03, &
      1.3625374E+00, 5.10348E+01, 1.1908E+01, 1.9442E+03, &
      1.3693500E+00, 5.09912E+01, 1.1837E+01, 1.9230E+03, &
      1.3761968E+00, 5.09459E+01, 1.1766E+01, 1.9021E+03, &
      1.3830778E+00, 5.08990E+01, 1.1696E+01, 1.8815E+03, &
      1.3899932E+00, 5.08503E+01, 1.1626E+01, 1.8611E+03, &
      1.3969431E+00, 5.07999E+01, 1.1557E+01, 1.8409E+03, &
      1.4039278E+00, 5.07477E+01, 1.1488E+01, 1.8210E+03, &
      1.4109475E+00, 5.06965E+01, 1.1420E+01, 1.8014E+03, &
      1.4180022E+00, 5.06406E+01, 1.1353E+01, 1.7819E+03, &
      1.4250922E+00, 5.05829E+01, 1.1286E+01, 1.7628E+03, &
      1.4322177E+00, 5.05233E+01, 1.1219E+01, 1.7438E+03, &
      1.4393788E+00, 5.04617E+01, 1.1154E+01, 1.7251E+03, &
      1.4465757E+00, 5.03980E+01, 1.1088E+01, 1.7065E+03, &
      1.4538086E+00, 5.03322E+01, 1.1023E+01, 1.6882E+03, &
      1.4610776E+00, 5.02643E+01, 1.0959E+01, 1.6702E+03, &
      1.4683830E+00, 5.01942E+01, 1.0895E+01, 1.6523E+03, &
      1.4757249E+00, 5.01218E+01, 1.0832E+01, 1.6347E+03, &
      1.4831035E+00, 5.00471E+01, 1.0769E+01, 1.6172E+03, &
      1.4905190E+00, 4.99700E+01, 1.0707E+01, 1.6000E+03, &
      1.4979716E+00, 4.98904E+01, 1.0645E+01, 1.5830E+03, &
      1.5054615E+00, 4.98083E+01, 1.0584E+01, 1.5662E+03, &
      1.5129888E+00, 4.97235E+01, 1.0523E+01, 1.5495E+03, &
      1.5205537E+00, 4.96359E+01, 1.0463E+01, 1.5331E+03, &
      1.5281565E+00, 4.95455E+01, 1.0403E+01, 1.5169E+03, &
      1.5357973E+00, 4.94522E+01, 1.0344E+01, 1.5008E+03, &
      1.5434763E+00, 4.93559E+01, 1.0285E+01, 1.4850E+03, &
      1.5511937E+00, 4.92563E+01, 1.0226E+01, 1.4693E+03, &
      1.5589496E+00, 4.91535E+01, 1.0168E+01, 1.4539E+03, &
      1.5667444E+00, 4.90472E+01, 1.0111E+01, 1.4386E+03, &
      1.5745781E+00, 4.89373E+01, 1.0054E+01, 1.4234E+03, &
      1.5824510E+00, 4.88238E+01, 9.9971E+00, 1.4085E+03, &
      1.5903633E+00, 4.87063E+01, 9.9408E+00, 1.3937E+03, &
      1.5983151E+00, 4.85847E+01, 9.8850E+00, 1.3791E+03, &
      1.6063066E+00, 4.84589E+01, 9.8297E+00, 1.3647E+03, &
      1.6143382E+00, 4.83286E+01, 9.7747E+00, 1.3504E+03, &
      1.6224099E+00, 4.81935E+01, 9.7202E+00, 1.3363E+03, &
      1.6305219E+00, 4.80536E+01, 9.6661E+00, 1.3224E+03, &
      1.6386745E+00, 4.79084E+01, 9.6124E+00, 1.3086E+03, &
      1.6468679E+00, 4.77619E+01, 9.5591E+00, 1.2950E+03, &
      1.6551022E+00, 4.76054E+01, 9.5062E+00, 1.2815E+03, &
      1.6633777E+00, 4.74428E+01, 9.4538E+00, 1.2682E+03, &
      1.6716946E+00, 4.72738E+01, 9.4017E+00, 1.2551E+03, &
      1.6800531E+00, 4.70978E+01, 9.3501E+00, 1.2421E+03, &
      1.6884534E+00, 4.69144E+01, 9.2988E+00, 1.2293E+03, &
      1.6968956E+00, 4.67232E+01, 9.2479E+00, 1.2166E+03, &
      1.7053801E+00, 4.65237E+01, 9.1974E+00, 1.2040E+03, &
      1.7139070E+00, 4.63152E+01, 9.1473E+00, 1.1916E+03, &
      1.7224766E+00, 4.60970E+01, 9.0976E+00, 1.1794E+03, &
      1.7310889E+00, 4.58685E+01, 9.0483E+00, 1.1673E+03, &
      1.7397444E+00, 4.56289E+01, 8.9993E+00, 1.1553E+03, &
      1.7484431E+00, 4.53771E+01, 8.9507E+00, 1.1434E+03, &
      1.7571853E+00, 4.51121E+01, 8.9025E+00, 1.1317E+03, &
      1.7659712E+00, 4.48328E+01, 8.8546E+00, 1.1202E+03, &
      1.7748011E+00, 4.45378E+01, 8.8071E+00, 1.1087E+03, &
      1.7836751E+00, 4.42255E+01, 8.7600E+00, 1.0974E+03, &
      1.7925935E+00, 4.38942E+01, 8.7132E+00, 1.0862E+03, &
      1.8015565E+00, 4.35416E+01, 8.6667E+00, 1.0752E+03, &
      1.8105642E+00, 4.31655E+01, 8.6206E+00, 1.0642E+03, &
      1.8196171E+00, 4.27627E+01, 8.5749E+00, 1.0534E+03, &
      1.8287151E+00, 4.23298E+01, 8.5295E+00, 1.0428E+03, &
      1.8378587E+00, 4.18625E+01, 8.4844E+00, 1.0322E+03, &
      1.8470480E+00, 4.13555E+01, 8.4397E+00, 1.0218E+03, &
      1.8562833E+00, 4.08023E+01, 8.3953E+00, 1.0114E+03, &
      1.8655647E+00, 4.01943E+01, 8.3512E+00, 1.0012E+03, &
      1.8748925E+00, 3.95206E+01, 8.3070E+00, 9.9109E+02, &
      1.8842670E+00, 3.87663E+01, 8.2616E+00, 9.8088E+02, &
      1.8936883E+00, 3.79108E+01, 8.2166E+00, 9.7079E+02, &
      1.9031567E+00, 3.69241E+01, 8.1719E+00, 9.6082E+02, &
      1.9126725E+00, 3.57604E+01, 8.1275E+00, 9.5096E+02, &
      1.9222359E+00, 3.43427E+01, 8.0835E+00, 9.4121E+02, &
      1.9318471E+00, 3.25270E+01, 8.0397E+00, 9.3157E+02, &
      1.9415063E+00, 2.99862E+01, 7.9963E+00, 9.2203E+02, &
      1.9512138E+00, 2.56313E+01, 7.9532E+00, 9.1261E+02, &
      1.9596590E+00, 8.61191E+00, 7.9161E+00, 9.0454E+02, &
      1.9605411E+00, 8.30119E+00, 2.5515E+01, 2.8902E+03, &
      1.9609699E+00, 1.21007E+01, 2.5507E+01, 2.8886E+03, &
      1.9707747E+00, 2.58504E+01, 2.5329E+01, 2.8543E+03, &
      1.9806286E+00, 2.90594E+01, 2.5152E+01, 2.8203E+03, &
      1.9905318E+00, 3.06406E+01, 2.4976E+01, 2.7868E+03, &
      2.0004844E+00, 3.13542E+01, 2.4801E+01, 2.7537E+03, &
      2.0104868E+00, 3.12867E+01, 2.4628E+01, 2.7210E+03, &
      2.0205393E+00, 2.99555E+01, 2.4456E+01, 2.6887E+03, &
      2.0299512E+00, 2.17230E+01, 2.4297E+01, 2.6589E+03, &
      2.0306420E+00, 1.55705E+01, 2.4285E+01, 2.6567E+03, &
      2.0316490E+00, 2.17027E+01, 3.5899E+01, 3.9201E+03, &
      2.0407952E+00, 3.16850E+01, 3.5668E+01, 3.8775E+03, &
      2.0509992E+00, 3.51111E+01, 3.5413E+01, 3.8308E+03, &
      2.0612542E+00, 3.73616E+01, 3.5160E+01, 3.7846E+03, &
      2.0715604E+00, 3.90993E+01, 3.4909E+01, 3.7391E+03, &
      2.0819182E+00, 4.05368E+01, 3.4660E+01, 3.6941E+03, &
      2.0923278E+00, 4.17719E+01, 3.4413E+01, 3.6496E+03, &
      2.1027895E+00, 4.28590E+01, 3.4168E+01, 3.6057E+03, &
      2.1133034E+00, 4.38315E+01, 3.3924E+01, 3.5623E+03, &
      2.1238699E+00, 4.47118E+01, 3.3683E+01, 3.5195E+03, &
      2.1344893E+00, 4.55160E+01, 3.3443E+01, 3.4772E+03, &
      2.1451617E+00, 4.62557E+01, 3.3205E+01, 3.4354E+03, &
      2.1558875E+00, 4.69397E+01, 3.2969E+01, 3.3941E+03, &
      2.1666670E+00, 4.75751E+01, 3.2735E+01, 3.3533E+03, &
      2.1775003E+00, 4.81673E+01, 3.2502E+01, 3.3131E+03, &
      2.1883878E+00, 4.87207E+01, 3.2271E+01, 3.2733E+03, &
      2.1993297E+00, 4.92390E+01, 3.2041E+01, 3.2339E+03, &
      2.2103264E+00, 4.97251E+01, 3.1812E+01, 3.1950E+03, &
      2.2213780E+00, 5.01815E+01, 3.1586E+01, 3.1565E+03, &
      2.2324849E+00, 5.06101E+01, 3.1361E+01, 3.1186E+03, &
      2.2436473E+00, 5.10126E+01, 3.1138E+01, 3.0811E+03, &
      2.2548656E+00, 5.13903E+01, 3.0916E+01, 3.0441E+03, &
      2.2661399E+00, 5.17444E+01, 3.0696E+01, 3.0075E+03, &
      2.2774706E+00, 5.20754E+01, 3.0478E+01, 2.9714E+03, &
      2.2888579E+00, 5.23840E+01, 3.0261E+01, 2.9357E+03, &
      2.3003022E+00, 5.26702E+01, 3.0046E+01, 2.9005E+03, &
      2.3118037E+00, 5.29340E+01, 2.9833E+01, 2.8657E+03, &
      2.3233628E+00, 5.31748E+01, 2.9621E+01, 2.8313E+03, &
      2.3349796E+00, 5.33915E+01, 2.9411E+01, 2.7973E+03, &
      2.3466545E+00, 5.35826E+01, 2.9202E+01, 2.7638E+03, &
      2.3583878E+00, 5.37454E+01, 2.8995E+01, 2.7306E+03, &
      2.3701797E+00, 5.38762E+01, 2.8789E+01, 2.6979E+03, &
      2.3820306E+00, 5.39689E+01, 2.8585E+01, 2.6656E+03, &
      2.3939407E+00, 5.40142E+01, 2.8383E+01, 2.6337E+03, &
      2.4059104E+00, 5.39961E+01, 2.8182E+01, 2.6021E+03, &
      2.4179400E+00, 5.38854E+01, 2.7983E+01, 2.5710E+03, &
      2.4300297E+00, 5.36185E+01, 2.7785E+01, 2.5402E+03, &
      2.4421798E+00, 5.30125E+01, 2.7588E+01, 2.5098E+03, &
      2.4543907E+00, 5.07883E+01, 2.7393E+01, 2.4798E+03, &
      2.4549050E+00, 5.04981E+01, 2.7385E+01, 2.4785E+03, &
      2.4594951E+00, 5.05312E+01, 3.2005E+01, 2.8895E+03, &
      2.4666627E+00, 5.28129E+01, 3.1869E+01, 2.8690E+03, &
      2.4789960E+00, 5.43202E+01, 3.1639E+01, 2.8342E+03, &
      2.4913910E+00, 5.52373E+01, 3.1410E+01, 2.7998E+03, &
      2.5038479E+00, 5.59333E+01, 3.1183E+01, 2.7658E+03, &
      2.5163672E+00, 5.65092E+01, 3.0957E+01, 2.7323E+03, &
      2.5289490E+00, 5.70078E+01, 3.0733E+01, 2.6991E+03, &
      2.5415938E+00, 5.74511E+01, 3.0511E+01, 2.6664E+03, &
      2.5543017E+00, 5.78515E+01, 3.0287E+01, 2.6337E+03, &
      2.5670732E+00, 5.82143E+01, 3.0060E+01, 2.6011E+03, &
      2.5799086E+00, 5.85432E+01, 2.9836E+01, 2.5690E+03, &
      2.5928082E+00, 5.88474E+01, 2.9628E+01, 2.5385E+03, &
      2.6057722E+00, 5.91334E+01, 2.9422E+01, 2.5085E+03, &
      2.6188011E+00, 5.94021E+01, 2.9219E+01, 2.4788E+03, &
      2.6318951E+00, 5.96547E+01, 2.9017E+01, 2.4496E+03, &
      2.6450545E+00, 5.98917E+01, 2.8816E+01, 2.4207E+03, &
      2.6582798E+00, 6.01122E+01, 2.8614E+01, 2.3918E+03, &
      2.6715712E+00, 6.03159E+01, 2.8414E+01, 2.3634E+03, &
      2.6849291E+00, 6.05026E+01, 2.8216E+01, 2.3354E+03, &
      2.6983537E+00, 6.06709E+01, 2.8020E+01, 2.3077E+03, &
      2.7118455E+00, 6.08188E+01, 2.7825E+01, 2.2804E+03, &
      2.7254047E+00, 6.09420E+01, 2.7633E+01, 2.2534E+03, &
      2.7390317E+00, 6.10334E+01, 2.7442E+01, 2.2268E+03, &
      2.7527269E+00, 6.10781E+01, 2.7253E+01, 2.2006E+03, &
      2.7664905E+00, 6.10415E+01, 2.7065E+01, 2.1747E+03, &
      2.7803230E+00, 6.08010E+01, 2.6879E+01, 2.1491E+03, &
      2.7883189E+00, 6.02693E+01, 2.6772E+01, 2.1345E+03, &
      2.7942246E+00, 5.99526E+01, 2.8526E+01, 2.2689E+03, &
      2.7960812E+00, 6.03609E+01, 2.8499E+01, 2.2652E+03, &
      2.8081957E+00, 6.13624E+01, 2.8323E+01, 2.2416E+03, &
      2.8222367E+00, 6.19197E+01, 2.8122E+01, 2.2147E+03, &
      2.8363479E+00, 6.23245E+01, 2.7922E+01, 2.1882E+03, &
      2.8505296E+00, 6.26585E+01, 2.7724E+01, 2.1619E+03, &
      2.8647823E+00, 6.29495E+01, 2.7528E+01, 2.1360E+03, &
      2.8791062E+00, 6.32100E+01, 2.7332E+01, 2.1105E+03, &
      2.8935017E+00, 6.34467E+01, 2.7139E+01, 2.0852E+03, &
      2.9079692E+00, 6.36630E+01, 2.6946E+01, 2.0602E+03, &
      2.9225091E+00, 6.38605E+01, 2.6756E+01, 2.0356E+03, &
      2.9371216E+00, 6.40395E+01, 2.6574E+01, 2.0118E+03, &
      2.9518072E+00, 6.42081E+01, 2.6401E+01, 1.9889E+03, &
      2.9665662E+00, 6.43669E+01, 2.6231E+01, 1.9663E+03, &
      2.9813991E+00, 6.45145E+01, 2.6063E+01, 1.9441E+03, &
      2.9963061E+00, 6.46501E+01, 2.5896E+01, 1.9221E+03, &
      3.0112876E+00, 6.47310E+01, 2.5713E+01, 1.8991E+03, &
      3.0263440E+00, 6.47300E+01, 2.5525E+01, 1.8760E+03, &
      3.0414758E+00, 6.44990E+01, 2.5339E+01, 1.8532E+03, &
      3.0422811E+00, 6.44650E+01, 2.5329E+01, 1.8520E+03, &
      3.0547190E+00, 6.45986E+01, 2.6294E+01, 1.9143E+03, &
      3.0566831E+00, 6.47181E+01, 2.6270E+01, 1.9114E+03, &
      3.0719666E+00, 6.52592E+01, 2.6085E+01, 1.8885E+03, &
      3.0873264E+00, 6.55995E+01, 2.5900E+01, 1.8660E+03, &
      3.1027630E+00, 6.58751E+01, 2.5717E+01, 1.8437E+03, &
      3.1182768E+00, 6.61167E+01, 2.5536E+01, 1.8217E+03, &
      3.1338682E+00, 6.63365E+01, 2.5356E+01, 1.7999E+03, &
      3.1495376E+00, 6.65407E+01, 2.5177E+01, 1.7784E+03, &
      3.1652853E+00, 6.67329E+01, 2.4999E+01, 1.7572E+03, &
      3.1811117E+00, 6.69155E+01, 2.4822E+01, 1.7362E+03, &
      3.1970172E+00, 6.70899E+01, 2.4646E+01, 1.7154E+03, &
      3.2130023E+00, 6.72567E+01, 2.4469E+01, 1.6947E+03, &
      3.2290673E+00, 6.74166E+01, 2.4294E+01, 1.6743E+03, &
      3.2452127E+00, 6.75705E+01, 2.4119E+01, 1.6541E+03, &
      3.2614387E+00, 6.77188E+01, 2.3946E+01, 1.6341E+03, &
      3.2777459E+00, 6.78620E+01, 2.3774E+01, 1.6144E+03, &
      3.2941347E+00, 6.80005E+01, 2.3603E+01, 1.5949E+03, &
      3.3106053E+00, 6.81346E+01, 2.3433E+01, 1.5756E+03, &
      3.3271584E+00, 6.82647E+01, 2.3264E+01, 1.5566E+03, &
      3.3437941E+00, 6.83908E+01, 2.3096E+01, 1.5378E+03, &
      3.3605131E+00, 6.85133E+01, 2.2930E+01, 1.5192E+03, &
      3.3773157E+00, 6.86322E+01, 2.2765E+01, 1.5009E+03, &
      3.3942023E+00, 6.87479E+01, 2.2601E+01, 1.4827E+03, &
      3.4111733E+00, 6.88603E+01, 2.2438E+01, 1.4648E+03, &
      3.4282291E+00, 6.89697E+01, 2.2276E+01, 1.4471E+03, &
      3.4453703E+00, 6.90762E+01, 2.2115E+01, 1.4296E+03, &
      3.4625971E+00, 6.91798E+01, 2.1955E+01, 1.4123E+03, &
      3.4799101E+00, 6.92807E+01, 2.1797E+01, 1.3953E+03, &
      3.4973097E+00, 6.93790E+01, 2.1640E+01, 1.3784E+03, &
      3.5147962E+00, 6.94748E+01, 2.1483E+01, 1.3617E+03, &
      3.5323702E+00, 6.95681E+01, 2.1328E+01, 1.3453E+03, &
      3.5500321E+00, 6.96590E+01, 2.1175E+01, 1.3290E+03, &
      3.5677822E+00, 6.97477E+01, 2.1022E+01, 1.3130E+03, &
      3.5856211E+00, 6.98343E+01, 2.0870E+01, 1.2971E+03, &
      3.6035492E+00, 6.99186E+01, 2.0720E+01, 1.2815E+03, &
      3.6215670E+00, 7.00010E+01, 2.0571E+01, 1.2660E+03, &
      3.6396748E+00, 7.00814E+01, 2.0423E+01, 1.2507E+03, &
      3.6578732E+00, 7.01598E+01, 2.0276E+01, 1.2357E+03, &
      3.6761626E+00, 7.02365E+01, 2.0130E+01, 1.2208E+03, &
      3.6945434E+00, 7.03114E+01, 1.9986E+01, 1.2061E+03, &
      3.7130161E+00, 7.03846E+01, 1.9843E+01, 1.1916E+03, &
      3.7315812E+00, 7.04562E+01, 1.9700E+01, 1.1772E+03, &
      3.7502391E+00, 7.05263E+01, 1.9559E+01, 1.1631E+03, &
      3.7689903E+00, 7.05948E+01, 1.9420E+01, 1.1491E+03, &
      3.7878352E+00, 7.06620E+01, 1.9281E+01, 1.1353E+03, &
      3.8067744E+00, 7.07278E+01, 1.9143E+01, 1.1217E+03, &
      3.8258083E+00, 7.07923E+01, 1.9007E+01, 1.1083E+03, &
      3.8449373E+00, 7.08555E+01, 1.8871E+01, 1.0950E+03, &
      3.8641620E+00, 7.09176E+01, 1.8737E+01, 1.0819E+03, &
      3.8834828E+00, 7.09786E+01, 1.8604E+01, 1.0689E+03, &
      3.9029002E+00, 7.10386E+01, 1.8472E+01, 1.0561E+03, &
      3.9224147E+00, 7.13005E+01, 1.8340E+01, 1.0435E+03, &
      3.9420268E+00, 7.13589E+01, 1.8202E+01, 1.0305E+03, &
      3.9617369E+00, 7.14155E+01, 1.8065E+01, 1.0178E+03, &
      3.9815456E+00, 7.14702E+01, 1.7929E+01, 1.0052E+03, &
      4.0014533E+00, 7.15232E+01, 1.7793E+01, 9.9273E+02, &
      4.0214606E+00, 7.15746E+01, 1.7659E+01, 9.8044E+02, &
      4.0415679E+00, 7.16243E+01, 1.7526E+01, 9.6831E+02, &
      4.0617757E+00, 7.18105E+01, 1.7395E+01, 9.5634E+02, &
      4.0820846E+00, 7.18578E+01, 1.7259E+01, 9.4424E+02, &
      4.1024950E+00, 7.19031E+01, 1.7124E+01, 9.3230E+02, &
      4.1230075E+00, 7.19466E+01, 1.6991E+01, 9.2053E+02, &
      4.1436226E+00, 7.19883E+01, 1.6859E+01, 9.0892E+02, &
      4.1643407E+00, 7.20284E+01, 1.6728E+01, 8.9746E+02, &
      4.1851624E+00, 7.20670E+01, 1.6598E+01, 8.8616E+02, &
      4.2060882E+00, 7.21041E+01, 1.6470E+01, 8.7501E+02, &
      4.2271186E+00, 7.21398E+01, 1.6342E+01, 8.6402E+02, &
      4.2482542E+00, 7.21741E+01, 1.6216E+01, 8.5317E+02, &
      4.2694955E+00, 7.22071E+01, 1.6091E+01, 8.4247E+02, &
      4.2908430E+00, 7.22390E+01, 1.5967E+01, 8.3191E+02, &
      4.3122972E+00, 7.22696E+01, 1.5844E+01, 8.2149E+02, &
      4.3338587E+00, 7.22991E+01, 1.5723E+01, 8.1122E+02, &
      4.3555280E+00, 7.23275E+01, 1.5602E+01, 8.0108E+02, &
      4.3773056E+00, 7.23548E+01, 1.5483E+01, 7.9108E+02, &
      4.3991921E+00, 7.23811E+01, 1.5365E+01, 7.8122E+02, &
      4.4211881E+00, 7.24064E+01, 1.5247E+01, 7.7148E+02, &
      4.4432940E+00, 7.24308E+01, 1.5131E+01, 7.6188E+02, &
      4.4655105E+00, 7.24543E+01, 1.5016E+01, 7.5241E+02, &
      4.4878381E+00, 7.24769E+01, 1.4902E+01, 7.4306E+02, &
      4.5102772E+00, 7.24986E+01, 1.4789E+01, 7.3384E+02, &
      4.5328286E+00, 7.25195E+01, 1.4677E+01, 7.2474E+02, &
      4.5554928E+00, 7.25396E+01, 1.4566E+01, 7.1576E+02, &
      4.5782702E+00, 7.25590E+01, 1.4456E+01, 7.0690E+02, &
      4.6011616E+00, 7.25776E+01, 1.4347E+01, 6.9817E+02, &
      4.6241674E+00, 7.25954E+01, 1.4239E+01, 6.8954E+02, &
      4.6472882E+00, 7.26126E+01, 1.4132E+01, 6.8104E+02, &
      4.6705247E+00, 7.26291E+01, 1.4026E+01, 6.7264E+02, &
      4.6938773E+00, 7.26450E+01, 1.3921E+01, 6.6436E+02, &
      4.7173467E+00, 7.26603E+01, 1.3817E+01, 6.5619E+02, &
      4.7409334E+00, 7.26750E+01, 1.3714E+01, 6.4812E+02, &
      4.7646381E+00, 7.26891E+01, 1.3611E+01, 6.4017E+02, &
      4.7884613E+00, 7.27026E+01, 1.3510E+01, 6.3231E+02, &
      4.8124036E+00, 7.27157E+01, 1.3409E+01, 6.2457E+02, &
      4.8364656E+00, 7.27282E+01, 1.3310E+01, 6.1692E+02, &
      4.8606479E+00, 7.27403E+01, 1.3211E+01, 6.0938E+02, &
      4.8849512E+00, 7.27519E+01, 1.3113E+01, 6.0193E+02, &
      4.9093759E+00, 7.27631E+01, 1.3016E+01, 5.9459E+02, &
      4.9339228E+00, 7.28768E+01, 1.2917E+01, 5.8718E+02, &
      4.9585924E+00, 7.28876E+01, 1.2817E+01, 5.7984E+02, &
      4.9833854E+00, 7.28976E+01, 1.2719E+01, 5.7259E+02, &
      5.0083023E+00, 7.29067E+01, 1.2621E+01, 5.6544E+02, &
      5.0333438E+00, 7.29151E+01, 1.2524E+01, 5.5839E+02, &
      5.0585105E+00, 7.29227E+01, 1.2428E+01, 5.5143E+02, &
      5.0838031E+00, 7.29297E+01, 1.2333E+01, 5.4456E+02, &
      5.1092221E+00, 7.29361E+01, 1.2239E+01, 5.3779E+02, &
      5.1347682E+00, 7.29418E+01, 1.2145E+01, 5.3108E+02, &
      5.1604421E+00, 7.29469E+01, 1.2051E+01, 5.2444E+02, &
      5.1862443E+00, 7.29513E+01, 1.1959E+01, 5.1789E+02, &
      5.2121755E+00, 7.29552E+01, 1.1867E+01, 5.1143E+02, &
      5.2382364E+00, 7.29585E+01, 1.1776E+01, 5.0505E+02, &
      5.2644276E+00, 7.29613E+01, 1.1685E+01, 4.9876E+02, &
      5.2907497E+00, 7.29635E+01, 1.1596E+01, 4.9255E+02, &
      5.3172034E+00, 7.29653E+01, 1.1507E+01, 4.8642E+02, &
      5.3437895E+00, 7.29665E+01, 1.1419E+01, 4.8038E+02, &
      5.3705084E+00, 7.29673E+01, 1.1332E+01, 4.7442E+02, &
      5.3973609E+00, 7.29676E+01, 1.1246E+01, 4.6853E+02, &
      5.4243477E+00, 7.29674E+01, 1.1161E+01, 4.6273E+02, &
      5.4514695E+00, 7.29669E+01, 1.1075E+01, 4.5698E+02, &
      5.4787268E+00, 7.29659E+01, 1.0991E+01, 4.5131E+02, &
      5.5061205E+00, 7.29645E+01, 1.0907E+01, 4.4570E+02, &
      5.5336511E+00, 7.29627E+01, 1.0824E+01, 4.4018E+02, &
      5.5613193E+00, 7.29605E+01, 1.0742E+01, 4.3472E+02, &
      5.5891259E+00, 7.30031E+01, 1.0660E+01, 4.2933E+02, &
      5.6170716E+00, 7.30006E+01, 1.0577E+01, 4.2394E+02, &
      5.6451569E+00, 7.29975E+01, 1.0494E+01, 4.1862E+02, &
      5.6733827E+00, 7.29938E+01, 1.0413E+01, 4.1337E+02, &
      5.7017496E+00, 7.29896E+01, 1.0332E+01, 4.0819E+02, &
      5.7302584E+00, 7.29849E+01, 1.0252E+01, 4.0308E+02, &
      5.7589096E+00, 7.29797E+01, 1.0173E+01, 3.9804E+02, &
      5.7877042E+00, 7.29740E+01, 1.0094E+01, 3.9306E+02, &
      5.8166427E+00, 7.29678E+01, 1.0016E+01, 3.8815E+02, &
      5.8457259E+00, 7.29612E+01, 9.9388E+00, 3.8331E+02, &
      5.8749546E+00, 7.29542E+01, 9.8622E+00, 3.7853E+02, &
      5.9043293E+00, 7.29468E+01, 9.7863E+00, 3.7381E+02, &
      5.9338510E+00, 7.29389E+01, 9.7111E+00, 3.6916E+02, &
      5.9635202E+00, 7.29307E+01, 9.6365E+00, 3.6457E+02, &
      5.9933378E+00, 7.29220E+01, 9.5626E+00, 3.6004E+02, &
      6.0233045E+00, 7.29130E+01, 9.4894E+00, 3.5557E+02, &
      6.0534210E+00, 7.29037E+01, 9.4168E+00, 3.5115E+02, &
      6.0836882E+00, 7.28940E+01, 9.3449E+00, 3.4680E+02, &
      6.1141066E+00, 7.29019E+01, 9.2732E+00, 3.4249E+02, &
      6.1446771E+00, 7.28917E+01, 9.2019E+00, 3.3823E+02, &
      6.1754005E+00, 7.28810E+01, 9.1312E+00, 3.3403E+02, &
      6.2062775E+00, 7.28699E+01, 9.0611E+00, 3.2988E+02, &
      6.2373089E+00, 7.28585E+01, 8.9917E+00, 3.2578E+02, &
      6.2684954E+00, 7.28466E+01, 8.9228E+00, 3.2174E+02, &
      6.2998379E+00, 7.28345E+01, 8.8546E+00, 3.1775E+02, &
      6.3313371E+00, 7.28219E+01, 8.7870E+00, 3.1382E+02, &
      6.3629938E+00, 7.28090E+01, 8.7199E+00, 3.0993E+02, &
      6.3948088E+00, 7.27958E+01, 8.6534E+00, 3.0610E+02, &
      6.4267828E+00, 7.27822E+01, 8.5876E+00, 3.0232E+02, &
      6.4589167E+00, 7.27683E+01, 8.5223E+00, 2.9859E+02, &
      6.4912113E+00, 7.27540E+01, 8.4575E+00, 2.9490E+02, &
      6.5236674E+00, 7.27394E+01, 8.3934E+00, 2.9127E+02, &
      6.5562857E+00, 7.27245E+01, 8.3297E+00, 2.8768E+02, &
      6.5890671E+00, 7.27093E+01, 8.2667E+00, 2.8414E+02, &
      6.6220125E+00, 7.26938E+01, 8.2042E+00, 2.8065E+02, &
      6.6551225E+00, 7.26779E+01, 8.1422E+00, 2.7720E+02, &
      6.6883981E+00, 7.26617E+01, 8.0808E+00, 2.7380E+02, &
      6.7218401E+00, 7.26452E+01, 8.0199E+00, 2.7044E+02, &
      6.7554493E+00, 7.26284E+01, 7.9595E+00, 2.6712E+02, &
      6.7892266E+00, 7.26113E+01, 7.8996E+00, 2.6385E+02, &
      6.8231727E+00, 7.25939E+01, 7.8403E+00, 2.6062E+02, &
      6.8572886E+00, 7.25762E+01, 7.7815E+00, 2.5744E+02, &
      6.8915750E+00, 7.25582E+01, 7.7232E+00, 2.5429E+02, &
      6.9260329E+00, 7.25398E+01, 7.6653E+00, 2.5119E+02, &
      6.9606631E+00, 7.25212E+01, 7.6080E+00, 2.4812E+02, &
      6.9954664E+00, 7.25023E+01, 7.5512E+00, 2.4510E+02, &
      7.0304437E+00, 7.24830E+01, 7.4949E+00, 2.4211E+02, &
      7.0655959E+00, 7.24634E+01, 7.4390E+00, 2.3917E+02, &
      7.1009239E+00, 7.24435E+01, 7.3836E+00, 2.3626E+02, &
      7.1364285E+00, 7.24233E+01, 7.3288E+00, 2.3339E+02, &
      7.1721107E+00, 7.24028E+01, 7.2743E+00, 2.3055E+02, &
      7.2079712E+00, 7.23820E+01, 7.2204E+00, 2.2776E+02, &
      7.2440111E+00, 7.23608E+01, 7.1669E+00, 2.2499E+02, &
      7.2802311E+00, 7.23394E+01, 7.1138E+00, 2.2227E+02, &
      7.3166323E+00, 7.23176E+01, 7.0612E+00, 2.1958E+02, &
      7.3532155E+00, 7.22954E+01, 7.0091E+00, 2.1692E+02, &
      7.3899815E+00, 7.22729E+01, 6.9574E+00, 2.1430E+02, &
      7.4269314E+00, 7.22501E+01, 6.9061E+00, 2.1172E+02, &
      7.4640661E+00, 7.22270E+01, 6.8553E+00, 2.0916E+02, &
      7.5013864E+00, 7.22035E+01, 6.8049E+00, 2.0664E+02, &
      7.5388934E+00, 7.21796E+01, 6.7549E+00, 2.0415E+02, &
      7.5765878E+00, 7.21554E+01, 6.7054E+00, 2.0170E+02, &
      7.6144708E+00, 7.21309E+01, 6.6563E+00, 1.9927E+02, &
      7.6525431E+00, 7.21059E+01, 6.6076E+00, 1.9688E+02, &
      7.6908058E+00, 7.20806E+01, 6.5593E+00, 1.9451E+02, &
      7.7292599E+00, 7.20549E+01, 6.5114E+00, 1.9218E+02, &
      7.7679062E+00, 7.20288E+01, 6.4639E+00, 1.8987E+02, &
      7.8067457E+00, 7.20023E+01, 6.4168E+00, 1.8760E+02, &
      7.8457794E+00, 7.19754E+01, 6.3701E+00, 1.8536E+02, &
      7.8850083E+00, 7.19480E+01, 6.3238E+00, 1.8314E+02, &
      7.9244334E+00, 7.19203E+01, 6.2779E+00, 1.8095E+02, &
      7.9640555E+00, 7.18921E+01, 6.2324E+00, 1.7879E+02, &
      8.0038758E+00, 7.18635E+01, 6.1872E+00, 1.7666E+02, &
      8.0438952E+00, 7.18344E+01, 6.1425E+00, 1.7455E+02, &
      8.0841147E+00, 7.18048E+01, 6.0981E+00, 1.7247E+02, &
      8.1245352E+00, 7.17748E+01, 6.0541E+00, 1.7042E+02, &
      8.1651579E+00, 7.17443E+01, 6.0104E+00, 1.6839E+02, &
      8.2059837E+00, 7.17133E+01, 5.9671E+00, 1.6639E+02, &
      8.2470136E+00, 7.16817E+01, 5.9242E+00, 1.6442E+02, &
      8.2882487E+00, 7.16497E+01, 5.8816E+00, 1.6247E+02, &
      8.3296899E+00, 7.16171E+01, 5.8394E+00, 1.6054E+02, &
      8.3713384E+00, 7.15839E+01, 5.7976E+00, 1.5864E+02, &
      8.4131951E+00, 7.15502E+01, 5.7560E+00, 1.5676E+02, &
      8.4552610E+00, 7.15160E+01, 5.7149E+00, 1.5491E+02, &
      8.4975373E+00, 7.14811E+01, 5.6739E+00, 1.5308E+02, &
      8.5400250E+00, 7.14457E+01, 5.6323E+00, 1.5124E+02, &
      9.030794E+00, 7.10612E+01, 5.1662E+00, 1.3167E+02, &
      9.653919E+00, 7.02032E+01, 4.6054E+00, 1.1035E+02, &
      1.032004E+01, 6.85702E+01, 4.1140E+00, 9.2720E+01, &
      1.065348E+01, 6.67008E+01, 3.9003E+00, 8.5388E+01, &
      1.081655E+01, 6.39315E+01, 3.7989E+00, 8.2028E+01, &
      1.086003E+01, 6.06949E+01, 3.7726E+00, 8.1166E+01, &
      1.092525E+01, 6.39065E+01, 1.0054E+01, 2.0788E+02, &
      1.103212E+01, 6.61286E+01, 9.8862E+00, 2.0250E+02, &
      1.108832E+01, 6.67292E+01, 9.7998E+00, 1.9975E+02, &
      1.179334E+01, 6.90428E+01, 8.8051E+00, 1.6915E+02, &
      1.213730E+01, 6.87669E+01, 8.3732E+00, 1.5649E+02, &
      1.232308E+01, 6.75864E+01, 8.1516E+00, 1.5015E+02, &
      1.237262E+01, 6.59983E+01, 8.0940E+00, 1.4851E+02, &
      1.244693E+01, 6.76734E+01, 1.1198E+01, 2.0281E+02, &
      1.260708E+01, 6.90535E+01, 1.0954E+01, 1.9594E+02, &
      1.263270E+01, 6.91650E+01, 1.0915E+01, 1.9487E+02, &
      1.270864E+01, 6.94128E+01, 1.0803E+01, 1.9175E+02, &
      1.290316E+01, 6.94181E+01, 1.0524E+01, 1.8406E+02, &
      1.295503E+01, 6.87330E+01, 1.0451E+01, 1.8209E+02, &
      1.303284E+01, 6.97742E+01, 1.1989E+01, 2.0711E+02, &
      1.322736E+01, 7.09684E+01, 1.1716E+01, 1.9949E+02, &
      1.347697E+01, 7.18031E+01, 1.1380E+01, 1.9028E+02, &
      1.440688E+01, 7.35317E+01, 1.0243E+01, 1.6052E+02, &
      1.540095E+01, 7.45193E+01, 9.2061E+00, 1.3525E+02, &
      1.646362E+01, 7.51662E+01, 8.2709E+00, 1.1393E+02, &
      1.759961E+01, 7.56001E+01, 7.4210E+00, 9.5865E+01, &
      1.881398E+01, 7.58884E+01, 6.6588E+00, 8.0688E+01, &
      2.011215E+01, 7.60779E+01, 5.9775E+00, 6.7957E+01, &
      2.149988E+01, 7.61991E+01, 5.3686E+00, 5.7276E+01, &
      2.298338E+01, 7.63299E+01, 4.8164E+00, 4.8236E+01, &
      2.456923E+01, 7.63600E+01, 4.3229E+00, 4.0650E+01, &
      2.626450E+01, 7.64321E+01, 3.8784E+00, 3.4253E+01, &
      2.807676E+01, 7.64168E+01, 3.4496E+00, 2.8637E+01, &
      3.001405E+01, 7.63631E+01, 3.0625E+00, 2.3909E+01, &
      3.208502E+01, 7.62893E+01, 2.7212E+00, 1.9987E+01, &
      3.429889E+01, 7.62029E+01, 2.4198E+00, 1.6729E+01, &
      3.666551E+01, 7.61083E+01, 2.1537E+00, 1.4021E+01, &
      3.919543E+01, 7.60079E+01, 1.9184E+00, 1.1767E+01, &
      4.189992E+01, 7.59027E+01, 1.7101E+00, 9.8888E+00, &
      4.479101E+01, 7.57929E+01, 1.5255E+00, 8.3211E+00, &
      4.788159E+01, 7.56773E+01, 1.3611E+00, 7.0082E+00, &
      5.118542E+01, 7.55514E+01, 1.2132E+00, 5.9020E+00, &
      5.471721E+01, 7.54097E+01, 1.0816E+00, 4.9757E+00, &
      5.849270E+01, 7.52406E+01, 9.6140E-01, 4.1880E+00, &
      6.252870E+01, 7.50181E+01, 8.5345E-01, 3.5252E+00, &
      6.684318E+01, 7.46783E+01, 7.5785E-01, 2.9721E+00, &
      7.145536E+01, 7.39039E+01, 6.7312E-01, 2.5099E+00, &
      7.239338E+01, 7.35326E+01, 6.5775E-01, 2.4291E+00, &
      7.350144E+01, 7.24458E+01, 6.4032E-01, 2.3383E+00, &
      7.379693E+01, 7.11446E+01, 6.3580E-01, 2.3149E+00, &
      7.424015E+01, 7.24596E+01, 3.1906E+00, 9.9140E+00, &
      7.534821E+01, 7.36472E+01, 3.1202E+00, 9.5600E+00, &
      7.638578E+01, 7.41160E+01, 3.0566E+00, 9.2447E+00, &
      8.165640E+01, 7.51235E+01, 2.7456E+00, 7.7995E+00, &
      8.729069E+01, 7.55687E+01, 2.4600E+00, 6.5674E+00, &
      9.331374E+01, 7.58340E+01, 2.2080E+00, 5.5422E+00, &
      9.975239E+01, 7.60122E+01, 1.9780E+00, 4.6714E+00, &
      1.066353E+02, 7.61284E+01, 1.7681E+00, 3.9320E+00, &
      1.139931E+02, 7.62052E+01, 1.5807E+00, 3.3129E+00, &
      1.218587E+02, 7.62552E+01, 1.4134E+00, 2.7943E+00, &
      1.302669E+02, 7.62863E+01, 1.2639E+00, 2.3596E+00, &
      1.392553E+02, 7.63040E+01, 1.1304E+00, 1.9952E+00, &
      1.488640E+02, 7.63392E+01, 1.0107E+00, 1.6890E+00, &
      1.591356E+02, 7.63387E+01, 9.0147E-01, 1.4289E+00, &
      1.701159E+02, 7.63318E+01, 8.0410E-01, 1.2112E+00, &
      1.818539E+02, 7.63209E+01, 7.1732E-01, 1.0288E+00, &
      1.944018E+02, 7.63072E+01, 6.3996E-01, 8.7601E-01, &
      2.078156E+02, 7.62917E+01, 5.7101E-01, 7.4788E-01, &
      2.221548E+02, 7.62751E+01, 5.0952E-01, 6.4035E-01, &
      2.374835E+02, 7.62580E+01, 4.5470E-01, 5.5004E-01, &
      2.538699E+02, 7.62408E+01, 4.0581E-01, 4.7412E-01, &
      2.713869E+02, 7.62237E+01, 3.6221E-01, 4.1022E-01, &
      2.901126E+02, 7.62068E+01, 3.2332E-01, 3.5636E-01, &
      3.101304E+02, 7.61903E+01, 2.8863E-01, 3.1091E-01, &
      3.315294E+02, 7.61742E+01, 2.5880E-01, 2.7324E-01, &
      3.544049E+02, 7.61597E+01, 2.3298E-01, 2.4177E-01, &
      3.788588E+02, 7.61466E+01, 2.0976E-01, 2.1487E-01, &
      4.050001E+02, 7.61345E+01, 1.8887E-01, 1.9180E-01, &
      4.329451E+02, 7.61234E+01, 1.7007E-01, 1.7197E-01]
   case(77)
      frel = -9.4980E-01
      allocate(nist_tab(3076))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.717466E-03, 4.56877E+00, 0.0000E+00, 3.0701E-05, &
      6.820284E-03, 4.16587E+00, 0.0000E+00, 3.3158E-05, &
      6.847702E-03, 3.73576E+00, 0.0000E+00, 3.3838E-05, &
      6.888830E-03, 4.14282E+00, 8.1569E-01, 2.5921E+04, &
      6.991648E-03, 4.45737E+00, 8.0003E-01, 2.5050E+04, &
      7.901491E-03, 4.03188E+00, 6.9950E-01, 1.9380E+04, &
      8.022432E-03, 3.53288E+00, 6.9891E-01, 1.9072E+04, &
      8.054683E-03, 2.96880E+00, 6.9931E-01, 1.9006E+04, &
      8.103059E-03, 3.53226E+00, 1.7893E+00, 4.8342E+04, &
      8.224000E-03, 3.99314E+00, 1.7635E+00, 4.6942E+04, &
      1.069000E-02, 3.38848E+00, 2.2596E+00, 4.6273E+04, &
      1.142761E-02, 2.96561E+00, 2.7630E+00, 5.2931E+04, &
      1.221612E-02, 2.55014E+00, 3.4451E+00, 6.1738E+04, &
      1.305903E-02, 2.18886E+00, 4.3234E+00, 7.2476E+04, &
      1.396010E-02, 1.94176E+00, 5.4058E+00, 8.4772E+04, &
      1.492335E-02, 1.88002E+00, 6.6815E+00, 9.8015E+04, &
      1.595306E-02, 2.07886E+00, 8.1130E+00, 1.1133E+05, &
      1.705382E-02, 2.60570E+00, 9.6315E+00, 1.2364E+05, &
      1.823053E-02, 3.50548E+00, 1.1138E+01, 1.3375E+05, &
      1.948844E-02, 4.78653E+00, 1.2515E+01, 1.4058E+05, &
      2.083314E-02, 6.44486E+00, 1.3638E+01, 1.4331E+05, &
      2.227063E-02, 8.33051E+00, 1.4402E+01, 1.4157E+05, &
      2.380730E-02, 1.03573E+01, 1.4736E+01, 1.3550E+05, &
      2.545001E-02, 1.24238E+01, 1.4616E+01, 1.2572E+05, &
      2.720606E-02, 1.43236E+01, 1.4069E+01, 1.1321E+05, &
      2.908327E-02, 1.59807E+01, 1.3163E+01, 9.9084E+04, &
      3.109002E-02, 1.73146E+01, 1.1994E+01, 8.4454E+04, &
      3.323523E-02, 1.82867E+01, 1.0670E+01, 7.0284E+04, &
      3.552846E-02, 1.88911E+01, 9.2918E+00, 5.7254E+04, &
      3.797993E-02, 1.91469E+01, 7.9469E+00, 4.5806E+04, &
      4.060054E-02, 1.90841E+01, 6.6989E+00, 3.6121E+04, &
      4.340198E-02, 1.87149E+01, 5.5846E+00, 2.8168E+04, &
      4.639671E-02, 1.79644E+01, 4.6190E+00, 2.1794E+04, &
      4.949000E-02, 1.61569E+01, 3.8261E+00, 1.6925E+04, &
      4.959809E-02, 1.60180E+01, 3.8017E+00, 1.6780E+04, &
      5.024750E-02, 1.44719E+01, 3.6588E+00, 1.5941E+04, &
      5.044950E-02, 1.25153E+01, 3.6158E+00, 1.5690E+04, &
      5.075250E-02, 1.45315E+01, 7.3654E+00, 3.1770E+04, &
      5.151000E-02, 1.62312E+01, 7.0780E+00, 3.0081E+04, &
      5.302035E-02, 1.72269E+01, 6.5492E+00, 2.7041E+04, &
      5.667876E-02, 1.76944E+01, 5.4755E+00, 2.1149E+04, &
      5.929000E-02, 1.73548E+01, 4.8462E+00, 1.7894E+04, &
      6.019750E-02, 1.69764E+01, 4.6499E+00, 1.6910E+04, &
      6.043950E-02, 1.66584E+01, 4.5994E+00, 1.6660E+04, &
      6.058959E-02, 1.66908E+01, 5.0898E+00, 1.8390E+04, &
      6.080250E-02, 1.68392E+01, 5.0424E+00, 1.8155E+04, &
      6.171000E-02, 1.67342E+01, 4.8471E+00, 1.7195E+04, &
      6.174000E-02, 1.67231E+01, 4.8408E+00, 1.7164E+04, &
      6.213200E-02, 1.65270E+01, 4.7598E+00, 1.6771E+04, &
      6.268500E-02, 1.59261E+01, 4.6488E+00, 1.6235E+04, &
      6.293700E-02, 1.49598E+01, 4.5995E+00, 1.5999E+04, &
      6.308300E-02, 1.50907E+01, 6.4019E+00, 2.2217E+04, &
      6.333660E-02, 1.57433E+01, 6.3493E+00, 2.1946E+04, &
      6.371700E-02, 1.63888E+01, 6.6245E+00, 2.2760E+04, &
      6.426000E-02, 1.68506E+01, 6.5117E+00, 2.2184E+04, &
      6.466800E-02, 1.70671E+01, 6.4291E+00, 2.1764E+04, &
      6.477028E-02, 1.71121E+01, 6.4087E+00, 2.1661E+04, &
      6.923942E-02, 1.79097E+01, 5.6306E+00, 1.7803E+04, &
      7.401695E-02, 1.80316E+01, 4.9850E+00, 1.4744E+04, &
      7.912411E-02, 1.78796E+01, 4.4354E+00, 1.2272E+04, &
      8.458368E-02, 1.75331E+01, 3.9991E+00, 1.0350E+04, &
      9.041995E-02, 1.70275E+01, 3.6960E+00, 8.9488E+03, &
      9.329600E-02, 1.67328E+01, 3.6047E+00, 8.4587E+03, &
      9.472400E-02, 1.65559E+01, 3.5729E+00, 8.2578E+03, &
      9.510480E-02, 1.64751E+01, 3.5659E+00, 8.2086E+03, &
      9.567600E-02, 1.64603E+01, 3.6404E+00, 8.3303E+03, &
      9.665893E-02, 1.63907E+01, 3.6286E+00, 8.2186E+03, &
      9.710400E-02, 1.63524E+01, 3.6245E+00, 8.1718E+03, &
      1.033284E-01, 1.57408E+01, 3.6506E+00, 7.7349E+03, &
      1.104581E-01, 1.49948E+01, 3.8583E+00, 7.6474E+03, &
      1.180797E-01, 1.41999E+01, 4.2715E+00, 7.9200E+03, &
      1.262272E-01, 1.33897E+01, 4.9107E+00, 8.5177E+03, &
      1.349368E-01, 1.26056E+01, 5.7993E+00, 9.4096E+03, &
      1.442475E-01, 1.18996E+01, 6.9456E+00, 1.0542E+04, &
      1.542005E-01, 1.13326E+01, 8.3499E+00, 1.1855E+04, &
      1.648404E-01, 1.09631E+01, 9.9997E+00, 1.3281E+04, &
      1.762144E-01, 1.08566E+01, 1.1853E+01, 1.4728E+04, &
      1.883732E-01, 1.11168E+01, 1.3856E+01, 1.6105E+04, &
      2.013709E-01, 1.17275E+01, 1.5936E+01, 1.7327E+04, &
      2.152655E-01, 1.26646E+01, 1.8011E+01, 1.8319E+04, &
      2.301188E-01, 1.39679E+01, 1.9993E+01, 1.9022E+04, &
      2.459970E-01, 1.56081E+01, 2.1797E+01, 1.9400E+04, &
      2.629708E-01, 1.75168E+01, 2.3315E+01, 1.9412E+04, &
      2.811158E-01, 1.95218E+01, 2.4462E+01, 1.9053E+04, &
      2.890020E-01, 2.02676E+01, 2.4820E+01, 1.8804E+04, &
      2.934255E-01, 2.04939E+01, 2.4987E+01, 1.8646E+04, &
      2.946051E-01, 2.03212E+01, 2.5027E+01, 1.8602E+04, &
      2.963745E-01, 2.07932E+01, 2.5669E+01, 1.8965E+04, &
      3.005128E-01, 2.14565E+01, 2.5820E+01, 1.8813E+04, &
      3.007980E-01, 2.14938E+01, 2.5829E+01, 1.8802E+04, &
      3.051720E-01, 2.20091E+01, 2.5968E+01, 1.8632E+04, &
      3.098430E-01, 2.23954E+01, 2.6095E+01, 1.8441E+04, &
      3.110886E-01, 2.23178E+01, 2.6126E+01, 1.8389E+04, &
      3.129570E-01, 2.27603E+01, 2.6615E+01, 1.8621E+04, &
      3.176280E-01, 2.34958E+01, 2.6729E+01, 1.8426E+04, &
      3.212482E-01, 2.39718E+01, 2.6805E+01, 1.8271E+04, &
      3.434143E-01, 2.65060E+01, 2.7073E+01, 1.7262E+04, &
      3.671099E-01, 2.88296E+01, 2.7057E+01, 1.6140E+04, &
      3.924405E-01, 3.09648E+01, 2.6807E+01, 1.4959E+04, &
      4.195189E-01, 3.28844E+01, 2.6369E+01, 1.3765E+04, &
      4.484657E-01, 3.45337E+01, 2.5788E+01, 1.2595E+04, &
      4.794098E-01, 3.56763E+01, 2.5103E+01, 1.1469E+04, &
      4.844140E-01, 3.57171E+01, 2.4989E+01, 1.1299E+04, &
      4.918285E-01, 3.53653E+01, 2.4819E+01, 1.1053E+04, &
      4.938057E-01, 3.46352E+01, 2.4774E+01, 1.0989E+04, &
      4.967715E-01, 3.55910E+01, 2.6311E+01, 1.1601E+04, &
      5.0000000E-01, 3.74872E+01, 2.6241E+01, 1.1495E+04, &
      5.0250000E-01, 3.77853E+01, 2.6186E+01, 1.1414E+04, &
      5.0501250E-01, 3.80335E+01, 2.6131E+01, 1.1334E+04, &
      5.0753756E-01, 3.82526E+01, 2.6076E+01, 1.1254E+04, &
      5.1007525E-01, 3.84523E+01, 2.6020E+01, 1.1174E+04, &
      5.1262563E-01, 3.86381E+01, 2.5964E+01, 1.1095E+04, &
      5.1518875E-01, 3.88133E+01, 2.5908E+01, 1.1016E+04, &
      5.1776470E-01, 3.89802E+01, 2.5852E+01, 1.0937E+04, &
      5.2035352E-01, 3.91402E+01, 2.5795E+01, 1.0859E+04, &
      5.2295529E-01, 3.92943E+01, 2.5738E+01, 1.0781E+04, &
      5.2557007E-01, 3.94434E+01, 2.5681E+01, 1.0704E+04, &
      5.2819792E-01, 3.95880E+01, 2.5624E+01, 1.0627E+04, &
      5.3083891E-01, 3.97286E+01, 2.5566E+01, 1.0550E+04, &
      5.3349310E-01, 3.98656E+01, 2.5508E+01, 1.0474E+04, &
      5.3616057E-01, 3.99991E+01, 2.5450E+01, 1.0398E+04, &
      5.3884137E-01, 4.01295E+01, 2.5392E+01, 1.0323E+04, &
      5.4153558E-01, 4.02568E+01, 2.5333E+01, 1.0248E+04, &
      5.4424325E-01, 4.03811E+01, 2.5274E+01, 1.0173E+04, &
      5.4696447E-01, 4.05024E+01, 2.5215E+01, 1.0099E+04, &
      5.4969929E-01, 4.06207E+01, 2.5156E+01, 1.0025E+04, &
      5.5244779E-01, 4.07359E+01, 2.5097E+01, 9.9519E+03, &
      5.5521003E-01, 4.08478E+01, 2.5037E+01, 9.8790E+03, &
      5.5798608E-01, 4.09559E+01, 2.4977E+01, 9.8064E+03, &
      5.6077601E-01, 4.10597E+01, 2.4917E+01, 9.7342E+03, &
      5.6357989E-01, 4.11583E+01, 2.4857E+01, 9.6625E+03, &
      5.6639779E-01, 4.12499E+01, 2.4797E+01, 9.5911E+03, &
      5.6922978E-01, 4.13313E+01, 2.4736E+01, 9.5201E+03, &
      5.7207593E-01, 4.13942E+01, 2.4675E+01, 9.4495E+03, &
      5.7493630E-01, 4.14089E+01, 2.4614E+01, 9.3793E+03, &
      5.7626318E-01, 4.13485E+01, 2.4586E+01, 9.3470E+03, &
      5.7781099E-01, 4.13893E+01, 2.4925E+01, 9.4507E+03, &
      5.7793677E-01, 4.14134E+01, 2.4923E+01, 9.4477E+03, &
      5.8070004E-01, 4.16930E+01, 2.4866E+01, 9.3813E+03, &
      5.8360354E-01, 4.18739E+01, 2.4806E+01, 9.3123E+03, &
      5.8652156E-01, 4.20278E+01, 2.4746E+01, 9.2437E+03, &
      5.8945417E-01, 4.21688E+01, 2.4686E+01, 9.1754E+03, &
      5.9240144E-01, 4.23020E+01, 2.4626E+01, 9.1076E+03, &
      5.9536345E-01, 4.24297E+01, 2.4566E+01, 9.0401E+03, &
      5.9834026E-01, 4.25534E+01, 2.4505E+01, 8.9731E+03, &
      6.0133196E-01, 4.26736E+01, 2.4444E+01, 8.9064E+03, &
      6.0433862E-01, 4.27911E+01, 2.4384E+01, 8.8402E+03, &
      6.0736032E-01, 4.29060E+01, 2.4323E+01, 8.7743E+03, &
      6.1039712E-01, 4.30188E+01, 2.4262E+01, 8.7088E+03, &
      6.1344910E-01, 4.31295E+01, 2.4200E+01, 8.6437E+03, &
      6.1651635E-01, 4.32383E+01, 2.4139E+01, 8.5790E+03, &
      6.1959893E-01, 4.33453E+01, 2.4078E+01, 8.5146E+03, &
      6.2269693E-01, 4.34505E+01, 2.4016E+01, 8.4507E+03, &
      6.2581041E-01, 4.35541E+01, 2.3954E+01, 8.3871E+03, &
      6.2893946E-01, 4.36560E+01, 2.3892E+01, 8.3239E+03, &
      6.3208416E-01, 4.37563E+01, 2.3831E+01, 8.2611E+03, &
      6.3524458E-01, 4.38549E+01, 2.3768E+01, 8.1987E+03, &
      6.3842080E-01, 4.39518E+01, 2.3706E+01, 8.1367E+03, &
      6.4161291E-01, 4.40471E+01, 2.3644E+01, 8.0750E+03, &
      6.4482097E-01, 4.41405E+01, 2.3582E+01, 8.0137E+03, &
      6.4804508E-01, 4.42321E+01, 2.3519E+01, 7.9528E+03, &
      6.5128530E-01, 4.43216E+01, 2.3457E+01, 7.8923E+03, &
      6.5454173E-01, 4.44090E+01, 2.3394E+01, 7.8321E+03, &
      6.5781444E-01, 4.44939E+01, 2.3331E+01, 7.7723E+03, &
      6.6110351E-01, 4.45762E+01, 2.3268E+01, 7.7129E+03, &
      6.6440903E-01, 4.46552E+01, 2.3205E+01, 7.6538E+03, &
      6.6773107E-01, 4.47304E+01, 2.3142E+01, 7.5951E+03, &
      6.7106973E-01, 4.48008E+01, 2.3079E+01, 7.5367E+03, &
      6.7442508E-01, 4.48648E+01, 2.3015E+01, 7.4788E+03, &
      6.7779720E-01, 4.49197E+01, 2.2952E+01, 7.4211E+03, &
      6.8118619E-01, 4.49601E+01, 2.2889E+01, 7.3639E+03, &
      6.8459212E-01, 4.49722E+01, 2.2825E+01, 7.3070E+03, &
      6.8801508E-01, 4.48994E+01, 2.2761E+01, 7.2505E+03, &
      6.8899585E-01, 4.48169E+01, 2.2743E+01, 7.2344E+03, &
      6.9120417E-01, 4.48764E+01, 2.3242E+01, 7.3692E+03, &
      6.9145515E-01, 4.49183E+01, 2.3237E+01, 7.3651E+03, &
      6.9491243E-01, 4.52283E+01, 2.3174E+01, 7.3087E+03, &
      6.9838699E-01, 4.54135E+01, 2.3111E+01, 7.2526E+03, &
      7.0187893E-01, 4.55652E+01, 2.3048E+01, 7.1969E+03, &
      7.0538832E-01, 4.57008E+01, 2.2985E+01, 7.1415E+03, &
      7.0891526E-01, 4.58266E+01, 2.2921E+01, 7.0865E+03, &
      7.1245984E-01, 4.59457E+01, 2.2858E+01, 7.0318E+03, &
      7.1602214E-01, 4.60600E+01, 2.2794E+01, 6.9775E+03, &
      7.1960225E-01, 4.61705E+01, 2.2731E+01, 6.9235E+03, &
      7.2320026E-01, 4.62779E+01, 2.2667E+01, 6.8699E+03, &
      7.2681626E-01, 4.63828E+01, 2.2604E+01, 6.8166E+03, &
      7.3045034E-01, 4.64855E+01, 2.2540E+01, 6.7637E+03, &
      7.3410260E-01, 4.65862E+01, 2.2476E+01, 6.7111E+03, &
      7.3777311E-01, 4.66852E+01, 2.2412E+01, 6.6588E+03, &
      7.4146197E-01, 4.67827E+01, 2.2349E+01, 6.6069E+03, &
      7.4516928E-01, 4.68787E+01, 2.2285E+01, 6.5553E+03, &
      7.4889513E-01, 4.69735E+01, 2.2221E+01, 6.5041E+03, &
      7.5263961E-01, 4.70670E+01, 2.2157E+01, 6.4532E+03, &
      7.5640280E-01, 4.71594E+01, 2.2093E+01, 6.4026E+03, &
      7.6018482E-01, 4.72507E+01, 2.2029E+01, 6.3524E+03, &
      7.6398574E-01, 4.73410E+01, 2.1964E+01, 6.3024E+03, &
      7.6780567E-01, 4.74303E+01, 2.1900E+01, 6.2529E+03, &
      7.7164470E-01, 4.75187E+01, 2.1836E+01, 6.2036E+03, &
      7.7550292E-01, 4.76063E+01, 2.1772E+01, 6.1546E+03, &
      7.7938044E-01, 4.76929E+01, 2.1707E+01, 6.1060E+03, &
      7.8327734E-01, 4.77787E+01, 2.1643E+01, 6.0577E+03, &
      7.8719373E-01, 4.78637E+01, 2.1578E+01, 6.0097E+03, &
      7.9112969E-01, 4.79479E+01, 2.1514E+01, 5.9620E+03, &
      7.9508534E-01, 4.80314E+01, 2.1449E+01, 5.9147E+03, &
      7.9906077E-01, 4.81141E+01, 2.1385E+01, 5.8676E+03, &
      8.0305607E-01, 4.81961E+01, 2.1320E+01, 5.8209E+03, &
      8.0707135E-01, 4.82773E+01, 2.1256E+01, 5.7745E+03, &
      8.1110671E-01, 4.83579E+01, 2.1191E+01, 5.7284E+03, &
      8.1516224E-01, 4.84378E+01, 2.1126E+01, 5.6826E+03, &
      8.1923806E-01, 4.85171E+01, 2.1062E+01, 5.6371E+03, &
      8.2333425E-01, 4.85957E+01, 2.0997E+01, 5.5919E+03, &
      8.2745092E-01, 4.86737E+01, 2.0932E+01, 5.5470E+03, &
      8.3158817E-01, 4.87511E+01, 2.0867E+01, 5.5024E+03, &
      8.3574611E-01, 4.88280E+01, 2.0803E+01, 5.4582E+03, &
      8.3992484E-01, 4.89041E+01, 2.0737E+01, 5.4139E+03, &
      8.4412447E-01, 4.89795E+01, 2.0671E+01, 5.3700E+03, &
      8.4834509E-01, 4.90541E+01, 2.0605E+01, 5.3264E+03, &
      8.5258682E-01, 4.91281E+01, 2.0539E+01, 5.2830E+03, &
      8.5684975E-01, 4.92014E+01, 2.0474E+01, 5.2400E+03, &
      8.6113400E-01, 4.92740E+01, 2.0408E+01, 5.1972E+03, &
      8.6543967E-01, 4.93460E+01, 2.0342E+01, 5.1548E+03, &
      8.6976687E-01, 4.94173E+01, 2.0276E+01, 5.1126E+03, &
      8.7411570E-01, 4.94880E+01, 2.0210E+01, 5.0707E+03, &
      8.7848628E-01, 4.95580E+01, 2.0144E+01, 5.0291E+03, &
      8.8287871E-01, 4.96275E+01, 2.0078E+01, 4.9879E+03, &
      8.8729310E-01, 4.97090E+01, 2.0012E+01, 4.9468E+03, &
      8.9172957E-01, 4.97773E+01, 1.9946E+01, 4.9061E+03, &
      8.9618822E-01, 4.98450E+01, 1.9880E+01, 4.8657E+03, &
      9.0066916E-01, 4.99121E+01, 1.9814E+01, 4.8255E+03, &
      9.0517250E-01, 4.99787E+01, 1.9749E+01, 4.7857E+03, &
      9.0969837E-01, 5.00446E+01, 1.9683E+01, 4.7461E+03, &
      9.1424686E-01, 5.01100E+01, 1.9617E+01, 4.7068E+03, &
      9.1881809E-01, 5.01748E+01, 1.9551E+01, 4.6677E+03, &
      9.2341218E-01, 5.02391E+01, 1.9485E+01, 4.6290E+03, &
      9.2802924E-01, 5.03029E+01, 1.9420E+01, 4.5905E+03, &
      9.3266939E-01, 5.03661E+01, 1.9354E+01, 4.5523E+03, &
      9.3733274E-01, 5.04365E+01, 1.9288E+01, 4.5144E+03, &
      9.4201940E-01, 5.04988E+01, 1.9223E+01, 4.4767E+03, &
      9.4672950E-01, 5.05606E+01, 1.9157E+01, 4.4394E+03, &
      9.5146315E-01, 5.06219E+01, 1.9091E+01, 4.4022E+03, &
      9.5622046E-01, 5.06828E+01, 1.9026E+01, 4.3654E+03, &
      9.6100156E-01, 5.07432E+01, 1.8960E+01, 4.3288E+03, &
      9.6580657E-01, 5.08031E+01, 1.8894E+01, 4.2925E+03, &
      9.7063560E-01, 5.08627E+01, 1.8829E+01, 4.2564E+03, &
      9.7548878E-01, 5.09220E+01, 1.8764E+01, 4.2206E+03, &
      9.8036623E-01, 5.09808E+01, 1.8698E+01, 4.1850E+03, &
      9.8526806E-01, 5.10392E+01, 1.8632E+01, 4.1497E+03, &
      9.9019440E-01, 5.10974E+01, 1.8567E+01, 4.1146E+03, &
      9.9514537E-01, 5.11553E+01, 1.8501E+01, 4.0798E+03, &
      1.0001211E+00, 5.12133E+01, 1.8435E+01, 4.0451E+03, &
      1.0051217E+00, 5.12831E+01, 1.8349E+01, 4.0063E+03, &
      1.0101473E+00, 5.13507E+01, 1.8263E+01, 3.9678E+03, &
      1.0151980E+00, 5.14160E+01, 1.8177E+01, 3.9296E+03, &
      1.0202740E+00, 5.14793E+01, 1.8092E+01, 3.8918E+03, &
      1.0253754E+00, 5.15405E+01, 1.8007E+01, 3.8543E+03, &
      1.0305023E+00, 5.15998E+01, 1.7922E+01, 3.8172E+03, &
      1.0356548E+00, 5.16573E+01, 1.7837E+01, 3.7804E+03, &
      1.0408331E+00, 5.17131E+01, 1.7753E+01, 3.7440E+03, &
      1.0460372E+00, 5.17671E+01, 1.7670E+01, 3.7079E+03, &
      1.0512674E+00, 5.18196E+01, 1.7586E+01, 3.6722E+03, &
      1.0565238E+00, 5.18705E+01, 1.7503E+01, 3.6367E+03, &
      1.0618064E+00, 5.19199E+01, 1.7420E+01, 3.6017E+03, &
      1.0671154E+00, 5.19679E+01, 1.7338E+01, 3.5669E+03, &
      1.0724510E+00, 5.20145E+01, 1.7256E+01, 3.5325E+03, &
      1.0778132E+00, 5.20598E+01, 1.7174E+01, 3.4984E+03, &
      1.0832023E+00, 5.21039E+01, 1.7093E+01, 3.4646E+03, &
      1.0886183E+00, 5.21466E+01, 1.7012E+01, 3.4312E+03, &
      1.0940614E+00, 5.21882E+01, 1.6931E+01, 3.3980E+03, &
      1.0995317E+00, 5.22286E+01, 1.6851E+01, 3.3652E+03, &
      1.1050294E+00, 5.22679E+01, 1.6771E+01, 3.3327E+03, &
      1.1105545E+00, 5.23062E+01, 1.6692E+01, 3.3006E+03, &
      1.1161073E+00, 5.23434E+01, 1.6613E+01, 3.2687E+03, &
      1.1216878E+00, 5.23795E+01, 1.6534E+01, 3.2372E+03, &
      1.1272963E+00, 5.24147E+01, 1.6456E+01, 3.2059E+03, &
      1.1329328E+00, 5.24490E+01, 1.6378E+01, 3.1750E+03, &
      1.1385974E+00, 5.24825E+01, 1.6297E+01, 3.1436E+03, &
      1.1442904E+00, 5.25149E+01, 1.6211E+01, 3.1116E+03, &
      1.1500119E+00, 5.25462E+01, 1.6126E+01, 3.0800E+03, &
      1.1557619E+00, 5.25762E+01, 1.6041E+01, 3.0487E+03, &
      1.1615407E+00, 5.26052E+01, 1.5957E+01, 3.0178E+03, &
      1.1673484E+00, 5.26331E+01, 1.5873E+01, 2.9872E+03, &
      1.1731852E+00, 5.26598E+01, 1.5790E+01, 2.9568E+03, &
      1.1790511E+00, 5.26855E+01, 1.5707E+01, 2.9268E+03, &
      1.1849464E+00, 5.27099E+01, 1.5616E+01, 2.8955E+03, &
      1.1908711E+00, 5.27325E+01, 1.5524E+01, 2.8642E+03, &
      1.1968254E+00, 5.27535E+01, 1.5427E+01, 2.8323E+03, &
      1.2028096E+00, 5.27725E+01, 1.5331E+01, 2.8007E+03, &
      1.2088236E+00, 5.27896E+01, 1.5235E+01, 2.7696E+03, &
      1.2148677E+00, 5.28050E+01, 1.5141E+01, 2.7388E+03, &
      1.2209421E+00, 5.28186E+01, 1.5047E+01, 2.7084E+03, &
      1.2270468E+00, 5.28305E+01, 1.4954E+01, 2.6784E+03, &
      1.2331820E+00, 5.28407E+01, 1.4861E+01, 2.6488E+03, &
      1.2393479E+00, 5.28493E+01, 1.4770E+01, 2.6195E+03, &
      1.2455447E+00, 5.28564E+01, 1.4679E+01, 2.5906E+03, &
      1.2517724E+00, 5.28616E+01, 1.4583E+01, 2.5609E+03, &
      1.2580312E+00, 5.28649E+01, 1.4488E+01, 2.5317E+03, &
      1.2643214E+00, 5.28663E+01, 1.4393E+01, 2.5028E+03, &
      1.2706430E+00, 5.28657E+01, 1.4300E+01, 2.4742E+03, &
      1.2769962E+00, 5.28632E+01, 1.4207E+01, 2.4461E+03, &
      1.2833812E+00, 5.28590E+01, 1.4115E+01, 2.4183E+03, &
      1.2897981E+00, 5.28529E+01, 1.4023E+01, 2.3908E+03, &
      1.2962471E+00, 5.28450E+01, 1.3933E+01, 2.3637E+03, &
      1.3027283E+00, 5.28354E+01, 1.3843E+01, 2.3369E+03, &
      1.3092420E+00, 5.28240E+01, 1.3754E+01, 2.3105E+03, &
      1.3157882E+00, 5.28109E+01, 1.3666E+01, 2.2844E+03, &
      1.3223671E+00, 5.27961E+01, 1.3578E+01, 2.2586E+03, &
      1.3289790E+00, 5.27796E+01, 1.3492E+01, 2.2331E+03, &
      1.3356239E+00, 5.27614E+01, 1.3406E+01, 2.2080E+03, &
      1.3423020E+00, 5.27416E+01, 1.3321E+01, 2.1832E+03, &
      1.3490135E+00, 5.27201E+01, 1.3236E+01, 2.1587E+03, &
      1.3557586E+00, 5.26969E+01, 1.3152E+01, 2.1345E+03, &
      1.3625374E+00, 5.26720E+01, 1.3069E+01, 2.1106E+03, &
      1.3693500E+00, 5.26455E+01, 1.2987E+01, 2.0870E+03, &
      1.3761968E+00, 5.26173E+01, 1.2905E+01, 2.0637E+03, &
      1.3830778E+00, 5.25874E+01, 1.2824E+01, 2.0407E+03, &
      1.3899932E+00, 5.25558E+01, 1.2744E+01, 2.0179E+03, &
      1.3969431E+00, 5.25226E+01, 1.2665E+01, 1.9955E+03, &
      1.4039278E+00, 5.24876E+01, 1.2586E+01, 1.9733E+03, &
      1.4109475E+00, 5.24509E+01, 1.2507E+01, 1.9514E+03, &
      1.4180022E+00, 5.24125E+01, 1.2430E+01, 1.9298E+03, &
      1.4250922E+00, 5.23724E+01, 1.2353E+01, 1.9084E+03, &
      1.4322177E+00, 5.23304E+01, 1.2277E+01, 1.8873E+03, &
      1.4393788E+00, 5.22867E+01, 1.2201E+01, 1.8665E+03, &
      1.4465757E+00, 5.22412E+01, 1.2126E+01, 1.8459E+03, &
      1.4538086E+00, 5.21938E+01, 1.2052E+01, 1.8256E+03, &
      1.4610776E+00, 5.21445E+01, 1.1978E+01, 1.8055E+03, &
      1.4683830E+00, 5.20934E+01, 1.1905E+01, 1.7857E+03, &
      1.4757249E+00, 5.20403E+01, 1.1832E+01, 1.7661E+03, &
      1.4831035E+00, 5.19901E+01, 1.1760E+01, 1.7468E+03, &
      1.4905190E+00, 5.19331E+01, 1.1689E+01, 1.7277E+03, &
      1.4979716E+00, 5.18740E+01, 1.1618E+01, 1.7088E+03, &
      1.5054615E+00, 5.18128E+01, 1.1548E+01, 1.6901E+03, &
      1.5129888E+00, 5.17494E+01, 1.1478E+01, 1.6717E+03, &
      1.5205537E+00, 5.16839E+01, 1.1409E+01, 1.6535E+03, &
      1.5281565E+00, 5.16161E+01, 1.1341E+01, 1.6356E+03, &
      1.5357973E+00, 5.15460E+01, 1.1273E+01, 1.6178E+03, &
      1.5434763E+00, 5.14735E+01, 1.1206E+01, 1.6003E+03, &
      1.5511937E+00, 5.13985E+01, 1.1139E+01, 1.5829E+03, &
      1.5589496E+00, 5.13211E+01, 1.1073E+01, 1.5658E+03, &
      1.5667444E+00, 5.12410E+01, 1.1007E+01, 1.5489E+03, &
      1.5745781E+00, 5.11583E+01, 1.0942E+01, 1.5322E+03, &
      1.5824510E+00, 5.10728E+01, 1.0877E+01, 1.5157E+03, &
      1.5903633E+00, 5.09844E+01, 1.0813E+01, 1.4994E+03, &
      1.5983151E+00, 5.08931E+01, 1.0749E+01, 1.4833E+03, &
      1.6063066E+00, 5.07987E+01, 1.0686E+01, 1.4674E+03, &
      1.6143382E+00, 5.07011E+01, 1.0624E+01, 1.4516E+03, &
      1.6224099E+00, 5.06002E+01, 1.0562E+01, 1.4361E+03, &
      1.6305219E+00, 5.04959E+01, 1.0500E+01, 1.4208E+03, &
      1.6386745E+00, 5.03880E+01, 1.0439E+01, 1.4056E+03, &
      1.6468679E+00, 5.02763E+01, 1.0378E+01, 1.3906E+03, &
      1.6551022E+00, 5.01608E+01, 1.0318E+01, 1.3758E+03, &
      1.6633777E+00, 5.00412E+01, 1.0259E+01, 1.3612E+03, &
      1.6716946E+00, 4.99173E+01, 1.0200E+01, 1.3467E+03, &
      1.6800531E+00, 4.97890E+01, 1.0141E+01, 1.3324E+03, &
      1.6884534E+00, 4.96560E+01, 1.0083E+01, 1.3183E+03, &
      1.6968956E+00, 4.95181E+01, 1.0025E+01, 1.3044E+03, &
      1.7053801E+00, 4.93751E+01, 9.9678E+00, 1.2906E+03, &
      1.7139070E+00, 4.92266E+01, 9.9110E+00, 1.2769E+03, &
      1.7224766E+00, 4.90723E+01, 9.8546E+00, 1.2635E+03, &
      1.7310889E+00, 4.89119E+01, 9.7987E+00, 1.2502E+03, &
      1.7397444E+00, 4.87495E+01, 9.7432E+00, 1.2370E+03, &
      1.7484431E+00, 4.85759E+01, 9.6882E+00, 1.2241E+03, &
      1.7571853E+00, 4.83950E+01, 9.6336E+00, 1.2112E+03, &
      1.7659712E+00, 4.82064E+01, 9.5794E+00, 1.1985E+03, &
      1.7748011E+00, 4.80095E+01, 9.5257E+00, 1.1860E+03, &
      1.7836751E+00, 4.78037E+01, 9.4724E+00, 1.1736E+03, &
      1.7925935E+00, 4.75885E+01, 9.4194E+00, 1.1614E+03, &
      1.8015565E+00, 4.73630E+01, 9.3669E+00, 1.1492E+03, &
      1.8105642E+00, 4.71265E+01, 9.3147E+00, 1.1373E+03, &
      1.8196171E+00, 4.68780E+01, 9.2630E+00, 1.1254E+03, &
      1.8287151E+00, 4.66166E+01, 9.2116E+00, 1.1138E+03, &
      1.8378587E+00, 4.63410E+01, 9.1607E+00, 1.1022E+03, &
      1.8470480E+00, 4.60499E+01, 9.1101E+00, 1.0908E+03, &
      1.8562833E+00, 4.57419E+01, 9.0600E+00, 1.0795E+03, &
      1.8655647E+00, 4.54151E+01, 9.0102E+00, 1.0683E+03, &
      1.8748925E+00, 4.50674E+01, 8.9608E+00, 1.0573E+03, &
      1.8842670E+00, 4.46965E+01, 8.9118E+00, 1.0464E+03, &
      1.8936883E+00, 4.42995E+01, 8.8632E+00, 1.0356E+03, &
      1.9031567E+00, 4.38729E+01, 8.8149E+00, 1.0250E+03, &
      1.9126725E+00, 4.34125E+01, 8.7671E+00, 1.0145E+03, &
      1.9222359E+00, 4.29132E+01, 8.7195E+00, 1.0041E+03, &
      1.9318471E+00, 4.23686E+01, 8.6724E+00, 9.9377E+02, &
      1.9415063E+00, 4.17703E+01, 8.6256E+00, 9.8360E+02, &
      1.9512138E+00, 4.11078E+01, 8.5791E+00, 9.7355E+02, &
      1.9609699E+00, 4.03666E+01, 8.5331E+00, 9.6361E+02, &
      1.9707747E+00, 3.95267E+01, 8.4873E+00, 9.5379E+02, &
      1.9806286E+00, 3.85591E+01, 8.4413E+00, 9.4400E+02, &
      1.9905318E+00, 3.74194E+01, 8.3943E+00, 9.3419E+02, &
      2.0004844E+00, 3.60335E+01, 8.3477E+00, 9.2449E+02, &
      2.0104868E+00, 3.42637E+01, 8.3014E+00, 9.1491E+02, &
      2.0205393E+00, 3.18011E+01, 8.2555E+00, 9.0543E+02, &
      2.0306420E+00, 2.76452E+01, 8.2099E+00, 8.9607E+02, &
      2.0399144E+00, 1.07769E+01, 8.1686E+00, 8.8760E+02, &
      2.0407952E+00, 9.32243E+00, 2.5629E+01, 2.7602E+03, &
      2.0408856E+00, 1.04675E+01, 2.5627E+01, 2.7599E+03, &
      2.0509992E+00, 2.73377E+01, 2.5448E+01, 2.7272E+03, &
      2.0612542E+00, 3.06781E+01, 2.5268E+01, 2.6946E+03, &
      2.0715604E+00, 3.23347E+01, 2.5090E+01, 2.6624E+03, &
      2.0819182E+00, 3.31343E+01, 2.4913E+01, 2.6306E+03, &
      2.0923278E+00, 3.32153E+01, 2.4737E+01, 2.5991E+03, &
      2.1027895E+00, 3.22773E+01, 2.4563E+01, 2.5681E+03, &
      2.1133034E+00, 2.75966E+01, 2.4390E+01, 2.5375E+03, &
      2.1153256E+00, 2.30704E+01, 2.4357E+01, 2.5316E+03, &
      2.1168746E+00, 2.30261E+01, 3.5877E+01, 3.7211E+03, &
      2.1238699E+00, 3.21225E+01, 3.5705E+01, 3.6913E+03, &
      2.1344893E+00, 3.61133E+01, 3.5448E+01, 3.6466E+03, &
      2.1451617E+00, 3.85261E+01, 3.5193E+01, 3.6024E+03, &
      2.1558875E+00, 4.03384E+01, 3.4939E+01, 3.5588E+03, &
      2.1666670E+00, 4.18173E+01, 3.4688E+01, 3.5157E+03, &
      2.1775003E+00, 4.30778E+01, 3.4438E+01, 3.4732E+03, &
      2.1883878E+00, 4.41813E+01, 3.4191E+01, 3.4312E+03, &
      2.1993297E+00, 4.51648E+01, 3.3945E+01, 3.3897E+03, &
      2.2103264E+00, 4.60525E+01, 3.3701E+01, 3.3488E+03, &
      2.2213780E+00, 4.68616E+01, 3.3459E+01, 3.3083E+03, &
      2.2324849E+00, 4.76044E+01, 3.3219E+01, 3.2683E+03, &
      2.2436473E+00, 4.82902E+01, 3.2981E+01, 3.2289E+03, &
      2.2548656E+00, 4.89263E+01, 3.2744E+01, 3.1899E+03, &
      2.2661399E+00, 4.95183E+01, 3.2510E+01, 3.1514E+03, &
      2.2774706E+00, 5.00710E+01, 3.2277E+01, 3.1134E+03, &
      2.2888579E+00, 5.05880E+01, 3.2046E+01, 3.0758E+03, &
      2.3003022E+00, 5.10723E+01, 3.1816E+01, 3.0387E+03, &
      2.3118037E+00, 5.15265E+01, 3.1588E+01, 3.0021E+03, &
      2.3233628E+00, 5.19525E+01, 3.1361E+01, 2.9658E+03, &
      2.3349796E+00, 5.23520E+01, 3.1136E+01, 2.9300E+03, &
      2.3466545E+00, 5.27262E+01, 3.0913E+01, 2.8946E+03, &
      2.3583878E+00, 5.30763E+01, 3.0691E+01, 2.8597E+03, &
      2.3701797E+00, 5.34029E+01, 3.0471E+01, 2.8252E+03, &
      2.3820306E+00, 5.37065E+01, 3.0252E+01, 2.7911E+03, &
      2.3939407E+00, 5.39870E+01, 3.0036E+01, 2.7574E+03, &
      2.4059104E+00, 5.42442E+01, 2.9821E+01, 2.7242E+03, &
      2.4179400E+00, 5.44774E+01, 2.9607E+01, 2.6913E+03, &
      2.4300297E+00, 5.46854E+01, 2.9395E+01, 2.6589E+03, &
      2.4421798E+00, 5.48659E+01, 2.9185E+01, 2.6268E+03, &
      2.4543907E+00, 5.50160E+01, 2.8976E+01, 2.5952E+03, &
      2.4666627E+00, 5.51307E+01, 2.8769E+01, 2.5640E+03, &
      2.4789960E+00, 5.52025E+01, 2.8564E+01, 2.5331E+03, &
      2.4913910E+00, 5.52192E+01, 2.8360E+01, 2.5026E+03, &
      2.5038479E+00, 5.51590E+01, 2.8157E+01, 2.4725E+03, &
      2.5163672E+00, 5.49790E+01, 2.7956E+01, 2.4428E+03, &
      2.5289490E+00, 5.45741E+01, 2.7757E+01, 2.4134E+03, &
      2.5415938E+00, 5.35405E+01, 2.7559E+01, 2.3844E+03, &
      2.5483176E+00, 5.16624E+01, 2.7455E+01, 2.3692E+03, &
      2.5530823E+00, 5.16985E+01, 3.2140E+01, 2.7665E+03, &
      2.5543017E+00, 5.23543E+01, 3.2117E+01, 2.7632E+03, &
      2.5670732E+00, 5.49291E+01, 3.1877E+01, 2.7290E+03, &
      2.5799086E+00, 5.60628E+01, 3.1639E+01, 2.6953E+03, &
      2.5928082E+00, 5.68583E+01, 3.1402E+01, 2.6619E+03, &
      2.6057722E+00, 5.74933E+01, 3.1168E+01, 2.6290E+03, &
      2.6188011E+00, 5.80317E+01, 3.0935E+01, 2.5965E+03, &
      2.6318951E+00, 5.85039E+01, 3.0704E+01, 2.5645E+03, &
      2.6450545E+00, 5.89267E+01, 3.0475E+01, 2.5328E+03, &
      2.6582798E+00, 5.93094E+01, 3.0244E+01, 2.5012E+03, &
      2.6715712E+00, 5.96553E+01, 3.0010E+01, 2.4697E+03, &
      2.6849291E+00, 5.99687E+01, 2.9791E+01, 2.4396E+03, &
      2.6983537E+00, 6.02631E+01, 2.9582E+01, 2.4105E+03, &
      2.7118455E+00, 6.05404E+01, 2.9376E+01, 2.3819E+03, &
      2.7254047E+00, 6.08014E+01, 2.9172E+01, 2.3537E+03, &
      2.7390317E+00, 6.10473E+01, 2.8972E+01, 2.3260E+03, &
      2.7527269E+00, 6.12786E+01, 2.8773E+01, 2.2986E+03, &
      2.7664905E+00, 6.14945E+01, 2.8571E+01, 2.2713E+03, &
      2.7803230E+00, 6.16946E+01, 2.8372E+01, 2.2444E+03, &
      2.7942246E+00, 6.18786E+01, 2.8175E+01, 2.2178E+03, &
      2.8081957E+00, 6.20453E+01, 2.7980E+01, 2.1916E+03, &
      2.8222367E+00, 6.21927E+01, 2.7787E+01, 2.1657E+03, &
      2.8363479E+00, 6.23171E+01, 2.7595E+01, 2.1402E+03, &
      2.8505296E+00, 6.24121E+01, 2.7406E+01, 2.1151E+03, &
      2.8647823E+00, 6.24650E+01, 2.7218E+01, 2.0902E+03, &
      2.8791062E+00, 6.24469E+01, 2.7032E+01, 2.0657E+03, &
      2.8935017E+00, 6.22670E+01, 2.6847E+01, 2.0415E+03, &
      2.9045696E+00, 6.16647E+01, 2.6706E+01, 2.0231E+03, &
      2.9079692E+00, 6.06972E+01, 2.6663E+01, 2.0175E+03, &
      2.9128303E+00, 6.17623E+01, 2.8444E+01, 2.1480E+03, &
      2.9225091E+00, 6.26103E+01, 2.8302E+01, 2.1303E+03, &
      2.9371216E+00, 6.32269E+01, 2.8091E+01, 2.1040E+03, &
      2.9518072E+00, 6.36512E+01, 2.7881E+01, 2.0780E+03, &
      2.9665662E+00, 6.39929E+01, 2.7674E+01, 2.0524E+03, &
      2.9813991E+00, 6.42848E+01, 2.7468E+01, 2.0271E+03, &
      2.9963061E+00, 6.45374E+01, 2.7264E+01, 2.0021E+03, &
      3.0112876E+00, 6.47643E+01, 2.7057E+01, 1.9772E+03, &
      3.0263440E+00, 6.49730E+01, 2.6853E+01, 1.9526E+03, &
      3.0414758E+00, 6.51638E+01, 2.6653E+01, 1.9285E+03, &
      3.0566831E+00, 6.53382E+01, 2.6455E+01, 1.9048E+03, &
      3.0719666E+00, 6.54971E+01, 2.6261E+01, 1.8816E+03, &
      3.0873264E+00, 6.56398E+01, 2.6070E+01, 1.8586E+03, &
      3.1027630E+00, 6.57637E+01, 2.5880E+01, 1.8361E+03, &
      3.1182768E+00, 6.58635E+01, 2.5692E+01, 1.8138E+03, &
      3.1338682E+00, 6.59280E+01, 2.5506E+01, 1.7918E+03, &
      3.1495376E+00, 6.59274E+01, 2.5321E+01, 1.7701E+03, &
      3.1652853E+00, 6.57257E+01, 2.5138E+01, 1.7486E+03, &
      3.1671623E+00, 6.56566E+01, 2.5117E+01, 1.7461E+03, &
      3.1802379E+00, 6.57879E+01, 2.6078E+01, 1.8051E+03, &
      3.1811117E+00, 6.58414E+01, 2.6068E+01, 1.8039E+03, &
      3.1970172E+00, 6.64097E+01, 2.5885E+01, 1.7824E+03, &
      3.2130023E+00, 6.67523E+01, 2.5703E+01, 1.7612E+03, &
      3.2290673E+00, 6.70270E+01, 2.5523E+01, 1.7403E+03, &
      3.2452127E+00, 6.72670E+01, 2.5344E+01, 1.7196E+03, &
      3.2614387E+00, 6.74850E+01, 2.5166E+01, 1.6991E+03, &
      3.2777459E+00, 6.76875E+01, 2.4989E+01, 1.6789E+03, &
      3.2941347E+00, 6.78780E+01, 2.4814E+01, 1.6589E+03, &
      3.3106053E+00, 6.80589E+01, 2.4639E+01, 1.6391E+03, &
      3.3271584E+00, 6.82316E+01, 2.4464E+01, 1.6195E+03, &
      3.3437941E+00, 6.83965E+01, 2.4289E+01, 1.6000E+03, &
      3.3605131E+00, 6.85549E+01, 2.4115E+01, 1.5807E+03, &
      3.3773157E+00, 6.87073E+01, 2.3942E+01, 1.5617E+03, &
      3.3942023E+00, 6.88543E+01, 2.3771E+01, 1.5429E+03, &
      3.4111733E+00, 6.89964E+01, 2.3600E+01, 1.5243E+03, &
      3.4282291E+00, 6.91339E+01, 2.3431E+01, 1.5059E+03, &
      3.4453703E+00, 6.92670E+01, 2.3262E+01, 1.4877E+03, &
      3.4625971E+00, 6.93961E+01, 2.3094E+01, 1.4698E+03, &
      3.4799101E+00, 6.95213E+01, 2.2928E+01, 1.4520E+03, &
      3.4973097E+00, 6.96428E+01, 2.2762E+01, 1.4344E+03, &
      3.5147962E+00, 6.97608E+01, 2.2597E+01, 1.4171E+03, &
      3.5323702E+00, 6.98754E+01, 2.2434E+01, 1.3999E+03, &
      3.5500321E+00, 6.99869E+01, 2.2271E+01, 1.3829E+03, &
      3.5677822E+00, 7.00952E+01, 2.2109E+01, 1.3662E+03, &
      3.5856211E+00, 7.02005E+01, 2.1949E+01, 1.3496E+03, &
      3.6035492E+00, 7.03028E+01, 2.1789E+01, 1.3332E+03, &
      3.6215670E+00, 7.04024E+01, 2.1631E+01, 1.3170E+03, &
      3.6396748E+00, 7.04993E+01, 2.1473E+01, 1.3010E+03, &
      3.6578732E+00, 7.05935E+01, 2.1317E+01, 1.2852E+03, &
      3.6761626E+00, 7.06851E+01, 2.1161E+01, 1.2696E+03, &
      3.6945434E+00, 7.07742E+01, 2.1007E+01, 1.2542E+03, &
      3.7130161E+00, 7.08609E+01, 2.0854E+01, 1.2389E+03, &
      3.7315812E+00, 7.09451E+01, 2.0702E+01, 1.2239E+03, &
      3.7502391E+00, 7.10270E+01, 2.0551E+01, 1.2090E+03, &
      3.7689903E+00, 7.11066E+01, 2.0401E+01, 1.1943E+03, &
      3.7878352E+00, 7.11840E+01, 2.0254E+01, 1.1799E+03, &
      3.8067744E+00, 7.12596E+01, 2.0108E+01, 1.1657E+03, &
      3.8258083E+00, 7.13337E+01, 1.9964E+01, 1.1516E+03, &
      3.8449373E+00, 7.14062E+01, 1.9821E+01, 1.1378E+03, &
      3.8641620E+00, 7.14771E+01, 1.9679E+01, 1.1241E+03, &
      3.8834828E+00, 7.15466E+01, 1.9538E+01, 1.1106E+03, &
      3.9029002E+00, 7.16147E+01, 1.9399E+01, 1.0973E+03, &
      3.9224147E+00, 7.16813E+01, 1.9260E+01, 1.0841E+03, &
      3.9420268E+00, 7.17466E+01, 1.9123E+01, 1.0711E+03, &
      3.9617369E+00, 7.18106E+01, 1.8986E+01, 1.0583E+03, &
      3.9815456E+00, 7.18734E+01, 1.8851E+01, 1.0456E+03, &
      4.0014533E+00, 7.19351E+01, 1.8717E+01, 1.0331E+03, &
      4.0214606E+00, 7.19956E+01, 1.8584E+01, 1.0207E+03, &
      4.0415679E+00, 7.20550E+01, 1.8453E+01, 1.0086E+03, &
      4.0617757E+00, 7.21135E+01, 1.8322E+01, 9.9651E+02, &
      4.0820846E+00, 7.23705E+01, 1.8192E+01, 9.8459E+02, &
      4.1024950E+00, 7.24277E+01, 1.8055E+01, 9.7240E+02, &
      4.1230075E+00, 7.24830E+01, 1.7919E+01, 9.6038E+02, &
      4.1436226E+00, 7.25366E+01, 1.7784E+01, 9.4848E+02, &
      4.1643407E+00, 7.25884E+01, 1.7649E+01, 9.3673E+02, &
      4.1851624E+00, 7.26385E+01, 1.7516E+01, 9.2514E+02, &
      4.2060882E+00, 7.26871E+01, 1.7385E+01, 9.1370E+02, &
      4.2271186E+00, 7.27343E+01, 1.7254E+01, 9.0241E+02, &
      4.2482542E+00, 7.29168E+01, 1.7120E+01, 8.9106E+02, &
      4.2694955E+00, 7.29612E+01, 1.6987E+01, 8.7980E+02, &
      4.2908430E+00, 7.30036E+01, 1.6855E+01, 8.6869E+02, &
      4.3122972E+00, 7.30444E+01, 1.6723E+01, 8.5773E+02, &
      4.3338587E+00, 7.30836E+01, 1.6594E+01, 8.4693E+02, &
      4.3555280E+00, 7.31212E+01, 1.6465E+01, 8.3626E+02, &
      4.3773056E+00, 7.31574E+01, 1.6337E+01, 8.2575E+02, &
      4.3991921E+00, 7.31921E+01, 1.6211E+01, 8.1537E+02, &
      4.4211881E+00, 7.32255E+01, 1.6086E+01, 8.0514E+02, &
      4.4432940E+00, 7.32577E+01, 1.5962E+01, 7.9504E+02, &
      4.4655105E+00, 7.32886E+01, 1.5839E+01, 7.8508E+02, &
      4.4878381E+00, 7.33184E+01, 1.5717E+01, 7.7525E+02, &
      4.5102772E+00, 7.33470E+01, 1.5596E+01, 7.6556E+02, &
      4.5328286E+00, 7.33745E+01, 1.5477E+01, 7.5600E+02, &
      4.5554928E+00, 7.34010E+01, 1.5358E+01, 7.4656E+02, &
      4.5782702E+00, 7.34265E+01, 1.5241E+01, 7.3725E+02, &
      4.6011616E+00, 7.34510E+01, 1.5125E+01, 7.2807E+02, &
      4.6241674E+00, 7.34745E+01, 1.5009E+01, 7.1901E+02, &
      4.6472882E+00, 7.34972E+01, 1.4895E+01, 7.1007E+02, &
      4.6705247E+00, 7.35190E+01, 1.4782E+01, 7.0125E+02, &
      4.6938773E+00, 7.35399E+01, 1.4670E+01, 6.9255E+02, &
      4.7173467E+00, 7.35600E+01, 1.4559E+01, 6.8397E+02, &
      4.7409334E+00, 7.35794E+01, 1.4449E+01, 6.7550E+02, &
      4.7646381E+00, 7.35980E+01, 1.4340E+01, 6.6714E+02, &
      4.7884613E+00, 7.36158E+01, 1.4231E+01, 6.5889E+02, &
      4.8124036E+00, 7.36329E+01, 1.4124E+01, 6.5076E+02, &
      4.8364656E+00, 7.36494E+01, 1.4018E+01, 6.4273E+02, &
      4.8606479E+00, 7.36652E+01, 1.3913E+01, 6.3481E+02, &
      4.8849512E+00, 7.36803E+01, 1.3809E+01, 6.2700E+02, &
      4.9093759E+00, 7.36949E+01, 1.3705E+01, 6.1929E+02, &
      4.9339228E+00, 7.37088E+01, 1.3603E+01, 6.1168E+02, &
      4.9585924E+00, 7.37222E+01, 1.3501E+01, 6.0417E+02, &
      4.9833854E+00, 7.37351E+01, 1.3401E+01, 5.9676E+02, &
      5.0083023E+00, 7.37474E+01, 1.3301E+01, 5.8945E+02, &
      5.0333438E+00, 7.37592E+01, 1.3202E+01, 5.8224E+02, &
      5.0585105E+00, 7.37706E+01, 1.3104E+01, 5.7512E+02, &
      5.0838031E+00, 7.37816E+01, 1.3007E+01, 5.6810E+02, &
      5.1092221E+00, 7.38934E+01, 1.2910E+01, 5.6111E+02, &
      5.1347682E+00, 7.39041E+01, 1.2810E+01, 5.5410E+02, &
      5.1604421E+00, 7.39140E+01, 1.2712E+01, 5.4718E+02, &
      5.1862443E+00, 7.39230E+01, 1.2614E+01, 5.4035E+02, &
      5.2121755E+00, 7.39313E+01, 1.2517E+01, 5.3361E+02, &
      5.2382364E+00, 7.39388E+01, 1.2422E+01, 5.2696E+02, &
      5.2644276E+00, 7.39457E+01, 1.2326E+01, 5.2040E+02, &
      5.2907497E+00, 7.39518E+01, 1.2232E+01, 5.1393E+02, &
      5.3172034E+00, 7.39574E+01, 1.2139E+01, 5.0754E+02, &
      5.3437895E+00, 7.39623E+01, 1.2046E+01, 5.0124E+02, &
      5.3705084E+00, 7.39667E+01, 1.1955E+01, 4.9502E+02, &
      5.3973609E+00, 7.39705E+01, 1.1864E+01, 4.8888E+02, &
      5.4243477E+00, 7.39737E+01, 1.1774E+01, 4.8283E+02, &
      5.4514695E+00, 7.39764E+01, 1.1684E+01, 4.7685E+02, &
      5.4787268E+00, 7.39787E+01, 1.1596E+01, 4.7096E+02, &
      5.5061205E+00, 7.39804E+01, 1.1508E+01, 4.6514E+02, &
      5.5336511E+00, 7.39817E+01, 1.1421E+01, 4.5940E+02, &
      5.5613193E+00, 7.39826E+01, 1.1334E+01, 4.5371E+02, &
      5.5891259E+00, 7.39830E+01, 1.1248E+01, 4.4808E+02, &
      5.6170716E+00, 7.39829E+01, 1.1162E+01, 4.4253E+02, &
      5.6451569E+00, 7.39824E+01, 1.1078E+01, 4.3706E+02, &
      5.6733827E+00, 7.39814E+01, 1.0994E+01, 4.3165E+02, &
      5.7017496E+00, 7.39801E+01, 1.0910E+01, 4.2632E+02, &
      5.7302584E+00, 7.39783E+01, 1.0828E+01, 4.2106E+02, &
      5.7589096E+00, 7.39762E+01, 1.0746E+01, 4.1587E+02, &
      5.7877042E+00, 7.39737E+01, 1.0665E+01, 4.1075E+02, &
      5.8166427E+00, 7.39709E+01, 1.0585E+01, 4.0569E+02, &
      5.8457259E+00, 7.40126E+01, 1.0503E+01, 4.0063E+02, &
      5.8749546E+00, 7.40093E+01, 1.0422E+01, 3.9561E+02, &
      5.9043293E+00, 7.40054E+01, 1.0341E+01, 3.9066E+02, &
      5.9338510E+00, 7.40010E+01, 1.0261E+01, 3.8577E+02, &
      5.9635202E+00, 7.39962E+01, 1.0182E+01, 3.8095E+02, &
      5.9933378E+00, 7.39908E+01, 1.0103E+01, 3.7620E+02, &
      6.0233045E+00, 7.39849E+01, 1.0025E+01, 3.7150E+02, &
      6.0534210E+00, 7.39786E+01, 9.9478E+00, 3.6687E+02, &
      6.0836882E+00, 7.39718E+01, 9.8712E+00, 3.6230E+02, &
      6.1141066E+00, 7.39646E+01, 9.7953E+00, 3.5779E+02, &
      6.1446771E+00, 7.39570E+01, 9.7201E+00, 3.5335E+02, &
      6.1754005E+00, 7.39490E+01, 9.6456E+00, 3.4896E+02, &
      6.2062775E+00, 7.39406E+01, 9.5718E+00, 3.4462E+02, &
      6.2373089E+00, 7.39318E+01, 9.4985E+00, 3.4035E+02, &
      6.2684954E+00, 7.39226E+01, 9.4260E+00, 3.3613E+02, &
      6.2998379E+00, 7.39131E+01, 9.3541E+00, 3.3197E+02, &
      6.3313371E+00, 7.39032E+01, 9.2828E+00, 3.2786E+02, &
      6.3629938E+00, 7.39102E+01, 9.2118E+00, 3.2380E+02, &
      6.3948088E+00, 7.38998E+01, 9.1412E+00, 3.1978E+02, &
      6.4267828E+00, 7.38889E+01, 9.0712E+00, 3.1582E+02, &
      6.4589167E+00, 7.38777E+01, 9.0018E+00, 3.1190E+02, &
      6.4912113E+00, 7.38661E+01, 8.9330E+00, 3.0804E+02, &
      6.5236674E+00, 7.38541E+01, 8.8649E+00, 3.0423E+02, &
      6.5562857E+00, 7.38417E+01, 8.7973E+00, 3.0047E+02, &
      6.5890671E+00, 7.38290E+01, 8.7303E+00, 2.9676E+02, &
      6.6220125E+00, 7.38160E+01, 8.6639E+00, 2.9309E+02, &
      6.6551225E+00, 7.38026E+01, 8.5981E+00, 2.8948E+02, &
      6.6883981E+00, 7.37889E+01, 8.5329E+00, 2.8591E+02, &
      6.7218401E+00, 7.37748E+01, 8.4682E+00, 2.8239E+02, &
      6.7554493E+00, 7.37604E+01, 8.4041E+00, 2.7892E+02, &
      6.7892266E+00, 7.37457E+01, 8.3405E+00, 2.7549E+02, &
      6.8231727E+00, 7.37307E+01, 8.2775E+00, 2.7210E+02, &
      6.8572886E+00, 7.37153E+01, 8.2151E+00, 2.6876E+02, &
      6.8915750E+00, 7.36996E+01, 8.1532E+00, 2.6547E+02, &
      6.9260329E+00, 7.36836E+01, 8.0918E+00, 2.6221E+02, &
      6.9606631E+00, 7.36673E+01, 8.0310E+00, 2.5900E+02, &
      6.9954664E+00, 7.36507E+01, 7.9707E+00, 2.5583E+02, &
      7.0304437E+00, 7.36338E+01, 7.9109E+00, 2.5271E+02, &
      7.0655959E+00, 7.36166E+01, 7.8516E+00, 2.4962E+02, &
      7.1009239E+00, 7.35990E+01, 7.7928E+00, 2.4657E+02, &
      7.1364285E+00, 7.35812E+01, 7.7346E+00, 2.4357E+02, &
      7.1721107E+00, 7.35630E+01, 7.6768E+00, 2.4060E+02, &
      7.2079712E+00, 7.35445E+01, 7.6196E+00, 2.3767E+02, &
      7.2440111E+00, 7.35258E+01, 7.5628E+00, 2.3478E+02, &
      7.2802311E+00, 7.35067E+01, 7.5066E+00, 2.3193E+02, &
      7.3166323E+00, 7.34873E+01, 7.4508E+00, 2.2911E+02, &
      7.3532155E+00, 7.34676E+01, 7.3955E+00, 2.2633E+02, &
      7.3899815E+00, 7.34476E+01, 7.3406E+00, 2.2358E+02, &
      7.4269314E+00, 7.34273E+01, 7.2863E+00, 2.2088E+02, &
      7.4640661E+00, 7.34066E+01, 7.2324E+00, 2.1820E+02, &
      7.5013864E+00, 7.33857E+01, 7.1789E+00, 2.1556E+02, &
      7.5388934E+00, 7.33644E+01, 7.1259E+00, 2.1296E+02, &
      7.5765878E+00, 7.33428E+01, 7.0734E+00, 2.1039E+02, &
      7.6144708E+00, 7.33208E+01, 7.0213E+00, 2.0785E+02, &
      7.6525431E+00, 7.32986E+01, 6.9697E+00, 2.0534E+02, &
      7.6908058E+00, 7.32760E+01, 6.9185E+00, 2.0287E+02, &
      7.7292599E+00, 7.32530E+01, 6.8677E+00, 2.0043E+02, &
      7.7679062E+00, 7.32297E+01, 6.8174E+00, 1.9801E+02, &
      7.8067457E+00, 7.32061E+01, 6.7675E+00, 1.9564E+02, &
      7.8457794E+00, 7.31821E+01, 6.7180E+00, 1.9329E+02, &
      7.8850083E+00, 7.31578E+01, 6.6689E+00, 1.9097E+02, &
      7.9244334E+00, 7.31331E+01, 6.6203E+00, 1.8868E+02, &
      7.9640555E+00, 7.31080E+01, 6.5721E+00, 1.8642E+02, &
      8.0038758E+00, 7.30826E+01, 6.5242E+00, 1.8419E+02, &
      8.0438952E+00, 7.30567E+01, 6.4768E+00, 1.8198E+02, &
      8.0841147E+00, 7.30305E+01, 6.4298E+00, 1.7981E+02, &
      8.1245352E+00, 7.30039E+01, 6.3831E+00, 1.7766E+02, &
      8.1651579E+00, 7.29768E+01, 6.3369E+00, 1.7554E+02, &
      8.2059837E+00, 7.29494E+01, 6.2910E+00, 1.7345E+02, &
      8.2470136E+00, 7.29216E+01, 6.2456E+00, 1.7139E+02, &
      8.2882487E+00, 7.28933E+01, 6.2005E+00, 1.6935E+02, &
      8.3296899E+00, 7.28646E+01, 6.1558E+00, 1.6733E+02, &
      8.3713384E+00, 7.28354E+01, 6.1115E+00, 1.6534E+02, &
      8.4131951E+00, 7.28058E+01, 6.0675E+00, 1.6338E+02, &
      8.4552610E+00, 7.27758E+01, 6.0239E+00, 1.6144E+02, &
      8.4975373E+00, 7.27453E+01, 5.9807E+00, 1.5953E+02, &
      8.5400250E+00, 7.27143E+01, 5.9378E+00, 1.5764E+02, &
      9.030794E+00, 7.23985E+01, 5.4576E+00, 1.3751E+02, &
      9.653919E+00, 7.17147E+01, 4.8670E+00, 1.1528E+02, &
      1.032004E+01, 7.05598E+01, 4.3460E+00, 9.6803E+01, &
      1.099090E+01, 6.77771E+01, 3.9125E+00, 8.2271E+01, &
      1.103212E+01, 6.73739E+01, 3.8882E+00, 8.1482E+01, &
      1.115912E+01, 6.50235E+01, 3.8127E+00, 7.9074E+01, &
      1.120399E+01, 6.18030E+01, 3.7864E+00, 7.8245E+01, &
      1.127128E+01, 6.50010E+01, 1.0038E+01, 1.9920E+02, &
      1.143950E+01, 6.78144E+01, 9.7835E+00, 1.9140E+02, &
      1.179334E+01, 6.95633E+01, 9.2813E+00, 1.7634E+02, &
      1.256762E+01, 6.98713E+01, 8.3062E+00, 1.4849E+02, &
      1.260708E+01, 6.97514E+01, 8.2608E+00, 1.4723E+02, &
      1.275998E+01, 6.86840E+01, 8.0870E+00, 1.4249E+02, &
      1.281128E+01, 6.71004E+01, 8.0300E+00, 1.4094E+02, &
      1.288822E+01, 6.87619E+01, 1.1120E+01, 1.9258E+02, &
      1.308058E+01, 7.02336E+01, 1.0842E+01, 1.8509E+02, &
      1.315013E+01, 7.04517E+01, 1.0744E+01, 1.8248E+02, &
      1.335141E+01, 7.04642E+01, 1.0468E+01, 1.7520E+02, &
      1.340508E+01, 6.97841E+01, 1.0396E+01, 1.7333E+02, &
      1.347697E+01, 7.07259E+01, 1.1938E+01, 1.9744E+02, &
      1.348559E+01, 7.08202E+01, 1.1926E+01, 1.9713E+02, &
      1.368687E+01, 7.20088E+01, 1.1656E+01, 1.8990E+02, &
      1.440688E+01, 7.38230E+01, 1.0762E+01, 1.6682E+02, &
      1.540095E+01, 7.50940E+01, 9.6762E+00, 1.4060E+02, &
      1.646362E+01, 7.58828E+01, 8.6962E+00, 1.1847E+02, &
      1.759961E+01, 7.64106E+01, 7.8074E+00, 9.9741E+01, &
      1.881398E+01, 7.67621E+01, 7.0052E+00, 8.3941E+01, &
      2.011215E+01, 7.69951E+01, 6.2881E+00, 7.0690E+01, &
      2.149988E+01, 7.71466E+01, 5.6474E+00, 5.9574E+01, &
      2.298338E+01, 7.73024E+01, 5.0712E+00, 5.0212E+01, &
      2.456923E+01, 7.73502E+01, 4.5513E+00, 4.2310E+01, &
      2.626450E+01, 7.74048E+01, 4.0864E+00, 3.5675E+01, &
      2.807676E+01, 7.74354E+01, 3.6367E+00, 2.9838E+01, &
      3.001405E+01, 7.73918E+01, 3.2288E+00, 2.4910E+01, &
      3.208502E+01, 7.73244E+01, 2.8690E+00, 2.0821E+01, &
      3.429889E+01, 7.72424E+01, 2.5514E+00, 1.7427E+01, &
      3.666551E+01, 7.71510E+01, 2.2709E+00, 1.4604E+01, &
      3.919543E+01, 7.70532E+01, 2.0228E+00, 1.2254E+01, &
      4.189992E+01, 7.69504E+01, 1.8034E+00, 1.0297E+01, &
      4.479101E+01, 7.68433E+01, 1.6090E+00, 8.6643E+00, &
      4.788159E+01, 7.67310E+01, 1.4365E+00, 7.2999E+00, &
      5.118542E+01, 7.66111E+01, 1.2811E+00, 6.1489E+00, &
      5.471721E+01, 7.64779E+01, 1.1430E+00, 5.1856E+00, &
      5.849270E+01, 7.63242E+01, 1.0186E+00, 4.3731E+00, &
      6.252870E+01, 7.61313E+01, 9.0442E-01, 3.6804E+00, &
      6.684318E+01, 7.58591E+01, 8.0341E-01, 3.1025E+00, &
      7.145536E+01, 7.53720E+01, 7.1368E-01, 2.6191E+00, &
      7.458878E+01, 7.45466E+01, 6.6151E-01, 2.3512E+00, &
      7.573045E+01, 7.34668E+01, 6.4402E-01, 2.2636E+00, &
      7.603489E+01, 7.21743E+01, 6.3949E-01, 2.2411E+00, &
      7.638578E+01, 7.32084E+01, 3.1835E+00, 9.5213E+00, &
      7.649156E+01, 7.34802E+01, 3.1769E+00, 9.4892E+00, &
      7.763322E+01, 7.46592E+01, 3.1075E+00, 9.1524E+00, &
      8.165640E+01, 7.58030E+01, 2.8740E+00, 8.0709E+00, &
      8.729069E+01, 7.64034E+01, 2.5766E+00, 6.7992E+00, &
      9.331374E+01, 7.67318E+01, 2.3102E+00, 5.7312E+00, &
      9.975239E+01, 7.69419E+01, 2.0719E+00, 4.8350E+00, &
      1.066353E+02, 7.70807E+01, 1.8524E+00, 4.0699E+00, &
      1.139931E+02, 7.71722E+01, 1.6564E+00, 3.4290E+00, &
      1.218587E+02, 7.72324E+01, 1.4813E+00, 2.8920E+00, &
      1.302669E+02, 7.72708E+01, 1.3250E+00, 2.4420E+00, &
      1.392553E+02, 7.72938E+01, 1.1853E+00, 2.0646E+00, &
      1.488640E+02, 7.73058E+01, 1.0604E+00, 1.7481E+00, &
      1.591356E+02, 7.73357E+01, 9.4708E-01, 1.4801E+00, &
      1.701159E+02, 7.73314E+01, 8.4519E-01, 1.2544E+00, &
      1.818539E+02, 7.73223E+01, 7.5434E-01, 1.0655E+00, &
      1.944018E+02, 7.73099E+01, 6.7332E-01, 9.0702E-01, &
      2.078156E+02, 7.72953E+01, 6.0105E-01, 7.7411E-01, &
      2.221548E+02, 7.72794E+01, 5.3660E-01, 6.6255E-01, &
      2.374835E+02, 7.72627E+01, 4.7910E-01, 5.6882E-01, &
      2.538699E+02, 7.72457E+01, 4.2779E-01, 4.9002E-01, &
      2.713869E+02, 7.72287E+01, 3.8202E-01, 4.2367E-01, &
      2.901126E+02, 7.72119E+01, 3.4117E-01, 3.6775E-01, &
      3.101304E+02, 7.71954E+01, 3.0471E-01, 3.2056E-01, &
      3.315294E+02, 7.71791E+01, 2.7235E-01, 2.8078E-01, &
      3.544049E+02, 7.71640E+01, 2.4539E-01, 2.4828E-01, &
      3.788588E+02, 7.71505E+01, 2.2112E-01, 2.2047E-01, &
      4.050001E+02, 7.71381E+01, 1.9926E-01, 1.9664E-01, &
      4.329451E+02, 7.71268E+01, 1.7958E-01, 1.7615E-01]
   case(78)
      frel = -9.8160E-01
      allocate(nist_tab(3068))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.002872E-03, 3.45968E+00, 0.0000E+00, 1.6617E-05, &
      6.094753E-03, 2.90784E+00, 0.0000E+00, 1.7989E-05, &
      6.119254E-03, 2.30428E+00, 0.0000E+00, 1.8369E-05, &
      6.156007E-03, 2.88178E+00, 1.1551E+00, 4.0474E+04, &
      6.247887E-03, 3.34662E+00, 1.1481E+00, 3.9638E+04, &
      7.291108E-03, 3.09785E+00, 1.1299E+00, 3.3428E+04, &
      7.402706E-03, 2.65794E+00, 1.1443E+00, 3.3345E+04, &
      7.432466E-03, 2.16631E+00, 1.1487E+00, 3.3339E+04, &
      7.477105E-03, 2.64967E+00, 2.1018E+00, 6.0636E+04, &
      7.588704E-03, 3.04264E+00, 2.1026E+00, 5.9765E+04, &
      1.069000E-02, 2.24208E+00, 3.6800E+00, 7.4256E+04, &
      1.142761E-02, 1.99002E+00, 4.4201E+00, 8.3433E+04, &
      1.221612E-02, 1.81942E+00, 5.3113E+00, 9.3785E+04, &
      1.305903E-02, 1.77552E+00, 6.3469E+00, 1.0484E+05, &
      1.396010E-02, 1.90630E+00, 7.5068E+00, 1.1599E+05, &
      1.492335E-02, 2.25771E+00, 8.7543E+00, 1.2654E+05, &
      1.595306E-02, 2.86702E+00, 1.0035E+01, 1.3569E+05, &
      1.705382E-02, 3.75549E+00, 1.1280E+01, 1.4268E+05, &
      1.823053E-02, 4.92172E+00, 1.2411E+01, 1.4685E+05, &
      1.948844E-02, 6.40179E+00, 1.3349E+01, 1.4775E+05, &
      2.083314E-02, 8.01344E+00, 1.4024E+01, 1.4520E+05, &
      2.227063E-02, 9.74487E+00, 1.4387E+01, 1.3935E+05, &
      2.380730E-02, 1.15501E+01, 1.4416E+01, 1.3062E+05, &
      2.545001E-02, 1.32561E+01, 1.4117E+01, 1.1965E+05, &
      2.720606E-02, 1.48194E+01, 1.3517E+01, 1.0717E+05, &
      2.908327E-02, 1.61720E+01, 1.2675E+01, 9.4012E+04, &
      3.109002E-02, 1.72718E+01, 1.1661E+01, 8.0908E+04, &
      3.323523E-02, 1.80957E+01, 1.0543E+01, 6.8426E+04, &
      3.552846E-02, 1.86414E+01, 9.3903E+00, 5.7012E+04, &
      3.797993E-02, 1.89220E+01, 8.2613E+00, 4.6920E+04, &
      4.060054E-02, 1.89530E+01, 7.1971E+00, 3.8238E+04, &
      4.340198E-02, 1.87355E+01, 6.2238E+00, 3.0932E+04, &
      4.639671E-02, 1.82131E+01, 5.3545E+00, 2.4894E+04, &
      4.959809E-02, 1.70219E+01, 4.5925E+00, 1.9973E+04, &
      5.066600E-02, 1.61027E+01, 4.3714E+00, 1.8611E+04, &
      5.144150E-02, 1.42792E+01, 4.2200E+00, 1.7696E+04, &
      5.164830E-02, 1.21130E+01, 4.1809E+00, 1.7462E+04, &
      5.195850E-02, 1.43873E+01, 8.3732E+00, 3.4762E+04, &
      5.273400E-02, 1.63401E+01, 8.0731E+00, 3.3023E+04, &
      5.302035E-02, 1.66791E+01, 7.9662E+00, 3.2410E+04, &
      5.667876E-02, 1.82365E+01, 6.7610E+00, 2.5731E+04, &
      6.058959E-02, 1.83622E+01, 5.7339E+00, 2.0414E+04, &
      6.399400E-02, 1.76527E+01, 5.0073E+00, 1.6878E+04, &
      6.477028E-02, 1.70874E+01, 4.8606E+00, 1.6187E+04, &
      6.497350E-02, 1.67849E+01, 4.8232E+00, 1.6013E+04, &
      6.523470E-02, 1.57840E+01, 4.7758E+00, 1.5792E+04, &
      6.562650E-02, 1.67772E+01, 6.6487E+00, 2.1854E+04, &
      6.660600E-02, 1.76018E+01, 6.4648E+00, 2.0937E+04, &
      6.923942E-02, 1.80672E+01, 6.0159E+00, 1.8742E+04, &
      6.967800E-02, 1.80661E+01, 5.9470E+00, 1.8411E+04, &
      7.074450E-02, 1.78893E+01, 5.7860E+00, 1.7642E+04, &
      7.102890E-02, 1.75907E+01, 5.7446E+00, 1.7446E+04, &
      7.145550E-02, 1.79428E+01, 6.3025E+00, 1.9026E+04, &
      7.252200E-02, 1.82507E+01, 6.1304E+00, 1.8234E+04, &
      7.281400E-02, 1.82857E+01, 6.0840E+00, 1.8023E+04, &
      7.392850E-02, 1.82725E+01, 5.9091E+00, 1.7241E+04, &
      7.401695E-02, 1.82476E+01, 5.8954E+00, 1.7181E+04, &
      7.422570E-02, 1.80963E+01, 5.8631E+00, 1.7039E+04, &
      7.467150E-02, 1.83595E+01, 6.2059E+00, 1.7927E+04, &
      7.578600E-02, 1.86350E+01, 6.0247E+00, 1.7148E+04, &
      7.912411E-02, 1.89032E+01, 5.5118E+00, 1.5026E+04, &
      8.458368E-02, 1.88750E+01, 4.8128E+00, 1.2274E+04, &
      9.041995E-02, 1.85797E+01, 4.2658E+00, 1.0177E+04, &
      9.665893E-02, 1.81081E+01, 3.8676E+00, 8.6316E+03, &
      9.966600E-02, 1.78325E+01, 3.7348E+00, 8.0838E+03, &
      1.011915E-01, 1.76603E+01, 3.6809E+00, 7.8470E+03, &
      1.015983E-01, 1.75791E+01, 3.6680E+00, 7.7881E+03, &
      1.022085E-01, 1.75688E+01, 3.7376E+00, 7.8886E+03, &
      1.033284E-01, 1.74993E+01, 3.7087E+00, 7.7428E+03, &
      1.037340E-01, 1.74683E+01, 3.6993E+00, 7.6930E+03, &
      1.104581E-01, 1.68687E+01, 3.6250E+00, 7.0797E+03, &
      1.180797E-01, 1.61244E+01, 3.7183E+00, 6.7933E+03, &
      1.262272E-01, 1.53098E+01, 4.0106E+00, 6.8546E+03, &
      1.349368E-01, 1.44497E+01, 4.5188E+00, 7.2246E+03, &
      1.442475E-01, 1.35764E+01, 5.2758E+00, 7.8905E+03, &
      1.542005E-01, 1.27388E+01, 6.3070E+00, 8.8240E+03, &
      1.648404E-01, 1.20002E+01, 7.6233E+00, 9.9771E+03, &
      1.762144E-01, 1.14223E+01, 9.2248E+00, 1.1294E+04, &
      1.883732E-01, 1.10854E+01, 1.1089E+01, 1.2701E+04, &
      2.013709E-01, 1.10666E+01, 1.3170E+01, 1.4109E+04, &
      2.152655E-01, 1.14312E+01, 1.5392E+01, 1.5426E+04, &
      2.301188E-01, 1.21856E+01, 1.7665E+01, 1.6562E+04, &
      2.459970E-01, 1.33649E+01, 1.9883E+01, 1.7438E+04, &
      2.629708E-01, 1.49325E+01, 2.1941E+01, 1.8000E+04, &
      2.811158E-01, 1.68186E+01, 2.3742E+01, 1.8221E+04, &
      3.005128E-01, 1.88722E+01, 2.5194E+01, 1.8088E+04, &
      3.070340E-01, 1.94815E+01, 2.5567E+01, 1.7966E+04, &
      3.117335E-01, 1.97467E+01, 2.5802E+01, 1.7858E+04, &
      3.129867E-01, 1.95919E+01, 2.5860E+01, 1.7826E+04, &
      3.148665E-01, 2.00670E+01, 2.6509E+01, 1.8165E+04, &
      3.195660E-01, 2.07951E+01, 2.6733E+01, 1.8049E+04, &
      3.212482E-01, 2.10057E+01, 2.6807E+01, 1.8004E+04, &
      3.241840E-01, 2.13431E+01, 2.6930E+01, 1.7923E+04, &
      3.291460E-01, 2.17758E+01, 2.7116E+01, 1.7775E+04, &
      3.304692E-01, 2.17179E+01, 2.7161E+01, 1.7733E+04, &
      3.324540E-01, 2.21665E+01, 2.7655E+01, 1.7948E+04, &
      3.374160E-01, 2.29359E+01, 2.7824E+01, 1.7792E+04, &
      3.434143E-01, 2.37157E+01, 2.8001E+01, 1.7592E+04, &
      3.671099E-01, 2.64497E+01, 2.8433E+01, 1.6712E+04, &
      3.924405E-01, 2.90098E+01, 2.8517E+01, 1.5679E+04, &
      4.195189E-01, 3.13791E+01, 2.8305E+01, 1.4559E+04, &
      4.484657E-01, 3.35086E+01, 2.7854E+01, 1.3404E+04, &
      4.794098E-01, 3.53128E+01, 2.7220E+01, 1.2254E+04, &
      5.0000000E-01, 3.73774E+01, 2.6748E+01, 1.1546E+04, &
      5.0250000E-01, 3.74426E+01, 2.6689E+01, 1.1463E+04, &
      5.0501250E-01, 3.74938E+01, 2.6629E+01, 1.1381E+04, &
      5.0753756E-01, 3.75258E+01, 2.6569E+01, 1.1298E+04, &
      5.1007525E-01, 3.75295E+01, 2.6508E+01, 1.1217E+04, &
      5.1262563E-01, 3.74861E+01, 2.6447E+01, 1.1135E+04, &
      5.1518875E-01, 3.73474E+01, 2.6385E+01, 1.1054E+04, &
      5.1776470E-01, 3.68834E+01, 2.6322E+01, 1.0973E+04, &
      5.1833567E-01, 3.65845E+01, 2.6308E+01, 1.0955E+04, &
      5.1966431E-01, 3.66506E+01, 2.7935E+01, 1.1602E+04, &
      5.2035352E-01, 3.70606E+01, 2.7919E+01, 1.1580E+04, &
      5.2295529E-01, 3.77556E+01, 2.7859E+01, 1.1498E+04, &
      5.2557007E-01, 3.81518E+01, 2.7797E+01, 1.1415E+04, &
      5.2819792E-01, 3.84568E+01, 2.7736E+01, 1.1334E+04, &
      5.3083891E-01, 3.87166E+01, 2.7674E+01, 1.1252E+04, &
      5.3349310E-01, 3.89489E+01, 2.7611E+01, 1.1171E+04, &
      5.3616057E-01, 3.91625E+01, 2.7548E+01, 1.1090E+04, &
      5.3884137E-01, 3.93625E+01, 2.7485E+01, 1.1009E+04, &
      5.4153558E-01, 3.95520E+01, 2.7421E+01, 1.0929E+04, &
      5.4424325E-01, 3.97330E+01, 2.7357E+01, 1.0850E+04, &
      5.4696447E-01, 3.99071E+01, 2.7292E+01, 1.0770E+04, &
      5.4969929E-01, 4.00751E+01, 2.7227E+01, 1.0691E+04, &
      5.5244779E-01, 4.02380E+01, 2.7162E+01, 1.0613E+04, &
      5.5521003E-01, 4.03962E+01, 2.7096E+01, 1.0534E+04, &
      5.5798608E-01, 4.05502E+01, 2.7030E+01, 1.0456E+04, &
      5.6077601E-01, 4.07004E+01, 2.6964E+01, 1.0379E+04, &
      5.6357989E-01, 4.08471E+01, 2.6897E+01, 1.0302E+04, &
      5.6639779E-01, 4.09903E+01, 2.6830E+01, 1.0225E+04, &
      5.6922978E-01, 4.11304E+01, 2.6763E+01, 1.0149E+04, &
      5.7207593E-01, 4.12673E+01, 2.6696E+01, 1.0073E+04, &
      5.7493630E-01, 4.14011E+01, 2.6628E+01, 9.9977E+03, &
      5.7781099E-01, 4.15319E+01, 2.6560E+01, 9.9226E+03, &
      5.8070004E-01, 4.16595E+01, 2.6492E+01, 9.8479E+03, &
      5.8360354E-01, 4.17837E+01, 2.6423E+01, 9.7736E+03, &
      5.8652156E-01, 4.19044E+01, 2.6354E+01, 9.6997E+03, &
      5.8945417E-01, 4.20212E+01, 2.6285E+01, 9.6263E+03, &
      5.9240144E-01, 4.21333E+01, 2.6216E+01, 9.5532E+03, &
      5.9536345E-01, 4.22398E+01, 2.6146E+01, 9.4805E+03, &
      5.9834026E-01, 4.23387E+01, 2.6076E+01, 9.4082E+03, &
      6.0133196E-01, 4.24263E+01, 2.6006E+01, 9.3363E+03, &
      6.0433862E-01, 4.24930E+01, 2.5936E+01, 9.2649E+03, &
      6.0736032E-01, 4.24988E+01, 2.5865E+01, 9.1938E+03, &
      6.0832884E-01, 4.24477E+01, 2.5843E+01, 9.1712E+03, &
      6.1007116E-01, 4.25165E+01, 2.6181E+01, 9.2644E+03, &
      6.1039712E-01, 4.25675E+01, 2.6173E+01, 9.2568E+03, &
      6.1344910E-01, 4.28395E+01, 2.6104E+01, 9.1865E+03, &
      6.1651635E-01, 4.30235E+01, 2.6035E+01, 9.1167E+03, &
      6.1959893E-01, 4.31834E+01, 2.5965E+01, 9.0472E+03, &
      6.2269693E-01, 4.33310E+01, 2.5896E+01, 8.9781E+03, &
      6.2581041E-01, 4.34711E+01, 2.5826E+01, 8.9094E+03, &
      6.2893946E-01, 4.36057E+01, 2.5756E+01, 8.8411E+03, &
      6.3208416E-01, 4.37361E+01, 2.5685E+01, 8.7733E+03, &
      6.3524458E-01, 4.38630E+01, 2.5615E+01, 8.7058E+03, &
      6.3842080E-01, 4.39869E+01, 2.5545E+01, 8.6388E+03, &
      6.4161291E-01, 4.41081E+01, 2.5474E+01, 8.5721E+03, &
      6.4482097E-01, 4.42270E+01, 2.5403E+01, 8.5059E+03, &
      6.4804508E-01, 4.43436E+01, 2.5333E+01, 8.4400E+03, &
      6.5128530E-01, 4.44581E+01, 2.5262E+01, 8.3746E+03, &
      6.5454173E-01, 4.45706E+01, 2.5190E+01, 8.3096E+03, &
      6.5781444E-01, 4.46811E+01, 2.5119E+01, 8.2450E+03, &
      6.6110351E-01, 4.47897E+01, 2.5048E+01, 8.1807E+03, &
      6.6440903E-01, 4.48964E+01, 2.4977E+01, 8.1169E+03, &
      6.6773107E-01, 4.50012E+01, 2.4905E+01, 8.0535E+03, &
      6.7106973E-01, 4.51040E+01, 2.4833E+01, 7.9905E+03, &
      6.7442508E-01, 4.52048E+01, 2.4762E+01, 7.9279E+03, &
      6.7779720E-01, 4.53036E+01, 2.4690E+01, 7.8656E+03, &
      6.8118619E-01, 4.54002E+01, 2.4618E+01, 7.8038E+03, &
      6.8459212E-01, 4.54944E+01, 2.4546E+01, 7.7424E+03, &
      6.8801508E-01, 4.55860E+01, 2.4474E+01, 7.6813E+03, &
      6.9145515E-01, 4.56747E+01, 2.4402E+01, 7.6207E+03, &
      6.9491243E-01, 4.57602E+01, 2.4330E+01, 7.5604E+03, &
      6.9838699E-01, 4.58416E+01, 2.4257E+01, 7.5006E+03, &
      7.0187893E-01, 4.59183E+01, 2.4185E+01, 7.4411E+03, &
      7.0538832E-01, 4.59885E+01, 2.4113E+01, 7.3820E+03, &
      7.0891526E-01, 4.60497E+01, 2.4040E+01, 7.3233E+03, &
      7.1245984E-01, 4.60968E+01, 2.3968E+01, 7.2650E+03, &
      7.1602214E-01, 4.61171E+01, 2.3895E+01, 7.2071E+03, &
      7.1960225E-01, 4.60602E+01, 2.3823E+01, 7.1495E+03, &
      7.2085202E-01, 4.59688E+01, 2.3797E+01, 7.1296E+03, &
      7.2320026E-01, 4.60399E+01, 2.4286E+01, 7.2521E+03, &
      7.2681626E-01, 4.63737E+01, 2.4214E+01, 7.1948E+03, &
      7.3045034E-01, 4.65660E+01, 2.4142E+01, 7.1379E+03, &
      7.3410260E-01, 4.67230E+01, 2.4070E+01, 7.0813E+03, &
      7.3777311E-01, 4.68632E+01, 2.3998E+01, 7.0251E+03, &
      7.4146197E-01, 4.69933E+01, 2.3926E+01, 6.9693E+03, &
      7.4516928E-01, 4.71166E+01, 2.3855E+01, 6.9139E+03, &
      7.4889513E-01, 4.72348E+01, 2.3783E+01, 6.8589E+03, &
      7.5263961E-01, 4.73492E+01, 2.3711E+01, 6.8042E+03, &
      7.5640280E-01, 4.74603E+01, 2.3639E+01, 6.7499E+03, &
      7.6018482E-01, 4.75687E+01, 2.3567E+01, 6.6959E+03, &
      7.6398574E-01, 4.76749E+01, 2.3495E+01, 6.6424E+03, &
      7.6780567E-01, 4.77789E+01, 2.3423E+01, 6.5892E+03, &
      7.7164470E-01, 4.78812E+01, 2.3351E+01, 6.5363E+03, &
      7.7550292E-01, 4.79818E+01, 2.3279E+01, 6.4839E+03, &
      7.7938044E-01, 4.80809E+01, 2.3207E+01, 6.4317E+03, &
      7.8327734E-01, 4.81786E+01, 2.3135E+01, 6.3800E+03, &
      7.8719373E-01, 4.82750E+01, 2.3063E+01, 6.3286E+03, &
      7.9112969E-01, 4.83702E+01, 2.2991E+01, 6.2776E+03, &
      7.9508534E-01, 4.84642E+01, 2.2919E+01, 6.2269E+03, &
      7.9906077E-01, 4.85571E+01, 2.2847E+01, 6.1765E+03, &
      8.0305607E-01, 4.86490E+01, 2.2775E+01, 6.1266E+03, &
      8.0707135E-01, 4.87399E+01, 2.2703E+01, 6.0769E+03, &
      8.1110671E-01, 4.88298E+01, 2.2631E+01, 6.0276E+03, &
      8.1516224E-01, 4.89187E+01, 2.2559E+01, 5.9787E+03, &
      8.1923806E-01, 4.90068E+01, 2.2487E+01, 5.9300E+03, &
      8.2333425E-01, 4.90939E+01, 2.2415E+01, 5.8817E+03, &
      8.2745092E-01, 4.91802E+01, 2.2343E+01, 5.8338E+03, &
      8.3158817E-01, 4.92657E+01, 2.2271E+01, 5.7862E+03, &
      8.3574611E-01, 4.93504E+01, 2.2199E+01, 5.7389E+03, &
      8.3992484E-01, 4.94342E+01, 2.2128E+01, 5.6920E+03, &
      8.4412447E-01, 4.95173E+01, 2.2056E+01, 5.6454E+03, &
      8.4834509E-01, 4.95997E+01, 2.1984E+01, 5.5991E+03, &
      8.5258682E-01, 4.96813E+01, 2.1912E+01, 5.5531E+03, &
      8.5684975E-01, 4.97622E+01, 2.1840E+01, 5.5075E+03, &
      8.6113400E-01, 4.98425E+01, 2.1769E+01, 5.4622E+03, &
      8.6543967E-01, 4.99221E+01, 2.1697E+01, 5.4172E+03, &
      8.6976687E-01, 5.00011E+01, 2.1625E+01, 5.3726E+03, &
      8.7411570E-01, 5.00794E+01, 2.1554E+01, 5.3282E+03, &
      8.7848628E-01, 5.01572E+01, 2.1482E+01, 5.2842E+03, &
      8.8287871E-01, 5.02342E+01, 2.1409E+01, 5.2402E+03, &
      8.8729310E-01, 5.03105E+01, 2.1337E+01, 5.1966E+03, &
      8.9172957E-01, 5.03862E+01, 2.1265E+01, 5.1534E+03, &
      8.9618822E-01, 5.04612E+01, 2.1192E+01, 5.1104E+03, &
      9.0066916E-01, 5.05356E+01, 2.1120E+01, 5.0677E+03, &
      9.0517250E-01, 5.06094E+01, 2.1048E+01, 5.0254E+03, &
      9.0969837E-01, 5.06826E+01, 2.0976E+01, 4.9833E+03, &
      9.1424686E-01, 5.07553E+01, 2.0904E+01, 4.9416E+03, &
      9.1881809E-01, 5.08275E+01, 2.0832E+01, 4.9002E+03, &
      9.2341218E-01, 5.08992E+01, 2.0760E+01, 4.8590E+03, &
      9.2802924E-01, 5.09705E+01, 2.0688E+01, 4.8182E+03, &
      9.3266939E-01, 5.10415E+01, 2.0616E+01, 4.7777E+03, &
      9.3733274E-01, 5.11122E+01, 2.0544E+01, 4.7375E+03, &
      9.4201940E-01, 5.11945E+01, 2.0472E+01, 4.6976E+03, &
      9.4672950E-01, 5.12650E+01, 2.0401E+01, 4.6579E+03, &
      9.5146315E-01, 5.13357E+01, 2.0329E+01, 4.6186E+03, &
      9.5622046E-01, 5.14065E+01, 2.0258E+01, 4.5796E+03, &
      9.6100156E-01, 5.14780E+01, 2.0186E+01, 4.5409E+03, &
      9.6580657E-01, 5.15503E+01, 2.0115E+01, 4.5024E+03, &
      9.7063560E-01, 5.16240E+01, 2.0044E+01, 4.4643E+03, &
      9.7548878E-01, 5.17000E+01, 1.9973E+01, 4.4264E+03, &
      9.8036623E-01, 5.17797E+01, 1.9901E+01, 4.3888E+03, &
      9.8526806E-01, 5.18653E+01, 1.9830E+01, 4.3515E+03, &
      9.9019440E-01, 5.19615E+01, 1.9760E+01, 4.3144E+03, &
      9.9514537E-01, 5.20885E+01, 1.9689E+01, 4.2777E+03, &
      1.0001211E+00, 5.22511E+01, 1.9617E+01, 4.2411E+03, &
      1.0051217E+00, 5.23662E+01, 1.9523E+01, 4.1998E+03, &
      1.0101473E+00, 5.24653E+01, 1.9429E+01, 4.1589E+03, &
      1.0151980E+00, 5.25532E+01, 1.9335E+01, 4.1184E+03, &
      1.0202740E+00, 5.26328E+01, 1.9242E+01, 4.0782E+03, &
      1.0253754E+00, 5.27062E+01, 1.9149E+01, 4.0385E+03, &
      1.0305023E+00, 5.27744E+01, 1.9056E+01, 3.9990E+03, &
      1.0356548E+00, 5.28385E+01, 1.8963E+01, 3.9599E+03, &
      1.0408331E+00, 5.28989E+01, 1.8871E+01, 3.9210E+03, &
      1.0460372E+00, 5.29562E+01, 1.8779E+01, 3.8826E+03, &
      1.0512674E+00, 5.30109E+01, 1.8687E+01, 3.8445E+03, &
      1.0565238E+00, 5.30631E+01, 1.8596E+01, 3.8068E+03, &
      1.0618064E+00, 5.31132E+01, 1.8505E+01, 3.7695E+03, &
      1.0671154E+00, 5.31614E+01, 1.8414E+01, 3.7325E+03, &
      1.0724510E+00, 5.32077E+01, 1.8324E+01, 3.6959E+03, &
      1.0778132E+00, 5.32524E+01, 1.8234E+01, 3.6596E+03, &
      1.0832023E+00, 5.32955E+01, 1.8145E+01, 3.6237E+03, &
      1.0886183E+00, 5.33371E+01, 1.8056E+01, 3.5881E+03, &
      1.0940614E+00, 5.33773E+01, 1.7968E+01, 3.5529E+03, &
      1.0995317E+00, 5.34162E+01, 1.7880E+01, 3.5181E+03, &
      1.1050294E+00, 5.34538E+01, 1.7792E+01, 3.4835E+03, &
      1.1105545E+00, 5.34902E+01, 1.7705E+01, 3.4494E+03, &
      1.1161073E+00, 5.35254E+01, 1.7619E+01, 3.4155E+03, &
      1.1216878E+00, 5.35595E+01, 1.7532E+01, 3.3820E+03, &
      1.1272963E+00, 5.35925E+01, 1.7447E+01, 3.3488E+03, &
      1.1329328E+00, 5.36245E+01, 1.7361E+01, 3.3160E+03, &
      1.1385974E+00, 5.36554E+01, 1.7276E+01, 3.2835E+03, &
      1.1442904E+00, 5.36852E+01, 1.7192E+01, 3.2513E+03, &
      1.1500119E+00, 5.37141E+01, 1.7108E+01, 3.2194E+03, &
      1.1557619E+00, 5.37421E+01, 1.7024E+01, 3.1878E+03, &
      1.1615407E+00, 5.37691E+01, 1.6941E+01, 3.1566E+03, &
      1.1673484E+00, 5.37952E+01, 1.6858E+01, 3.1257E+03, &
      1.1731852E+00, 5.38205E+01, 1.6776E+01, 3.0950E+03, &
      1.1790511E+00, 5.38448E+01, 1.6694E+01, 3.0647E+03, &
      1.1849464E+00, 5.38683E+01, 1.6613E+01, 3.0347E+03, &
      1.1908711E+00, 5.38910E+01, 1.6532E+01, 3.0051E+03, &
      1.1968254E+00, 5.39129E+01, 1.6451E+01, 2.9757E+03, &
      1.2028096E+00, 5.39341E+01, 1.6371E+01, 2.9466E+03, &
      1.2088236E+00, 5.39544E+01, 1.6292E+01, 2.9178E+03, &
      1.2148677E+00, 5.39741E+01, 1.6213E+01, 2.8893E+03, &
      1.2209421E+00, 5.39930E+01, 1.6134E+01, 2.8611E+03, &
      1.2270468E+00, 5.40111E+01, 1.6055E+01, 2.8330E+03, &
      1.2331820E+00, 5.40285E+01, 1.5976E+01, 2.8053E+03, &
      1.2393479E+00, 5.40452E+01, 1.5898E+01, 2.7778E+03, &
      1.2455447E+00, 5.40611E+01, 1.5821E+01, 2.7506E+03, &
      1.2517724E+00, 5.40763E+01, 1.5744E+01, 2.7237E+03, &
      1.2580312E+00, 5.40907E+01, 1.5660E+01, 2.6958E+03, &
      1.2643214E+00, 5.41037E+01, 1.5574E+01, 2.6679E+03, &
      1.2706430E+00, 5.41153E+01, 1.5489E+01, 2.6403E+03, &
      1.2769962E+00, 5.41256E+01, 1.5405E+01, 2.6130E+03, &
      1.2833812E+00, 5.41346E+01, 1.5321E+01, 2.5859E+03, &
      1.2897981E+00, 5.41424E+01, 1.5238E+01, 2.5592E+03, &
      1.2962471E+00, 5.41490E+01, 1.5155E+01, 2.5328E+03, &
      1.3027283E+00, 5.41543E+01, 1.5073E+01, 2.5066E+03, &
      1.3092420E+00, 5.41585E+01, 1.4992E+01, 2.4808E+03, &
      1.3157882E+00, 5.41616E+01, 1.4910E+01, 2.4552E+03, &
      1.3223671E+00, 5.41635E+01, 1.4830E+01, 2.4300E+03, &
      1.3289790E+00, 5.41642E+01, 1.4745E+01, 2.4041E+03, &
      1.3356239E+00, 5.41633E+01, 1.4659E+01, 2.3783E+03, &
      1.3423020E+00, 5.41610E+01, 1.4567E+01, 2.3518E+03, &
      1.3490135E+00, 5.41568E+01, 1.4476E+01, 2.3256E+03, &
      1.3557586E+00, 5.41508E+01, 1.4386E+01, 2.2998E+03, &
      1.3625374E+00, 5.41431E+01, 1.4296E+01, 2.2742E+03, &
      1.3693500E+00, 5.41337E+01, 1.4208E+01, 2.2490E+03, &
      1.3761968E+00, 5.41225E+01, 1.4120E+01, 2.2241E+03, &
      1.3830778E+00, 5.41096E+01, 1.4032E+01, 2.1994E+03, &
      1.3899932E+00, 5.40951E+01, 1.3946E+01, 2.1751E+03, &
      1.3969431E+00, 5.40789E+01, 1.3858E+01, 2.1509E+03, &
      1.4039278E+00, 5.40610E+01, 1.3768E+01, 2.1264E+03, &
      1.4109475E+00, 5.40413E+01, 1.3679E+01, 2.1022E+03, &
      1.4180022E+00, 5.40197E+01, 1.3590E+01, 2.0783E+03, &
      1.4250922E+00, 5.39962E+01, 1.3502E+01, 2.0547E+03, &
      1.4322177E+00, 5.39709E+01, 1.3415E+01, 2.0314E+03, &
      1.4393788E+00, 5.39438E+01, 1.3328E+01, 2.0084E+03, &
      1.4465757E+00, 5.39148E+01, 1.3243E+01, 1.9857E+03, &
      1.4538086E+00, 5.38840E+01, 1.3158E+01, 1.9633E+03, &
      1.4610776E+00, 5.38514E+01, 1.3074E+01, 1.9412E+03, &
      1.4683830E+00, 5.38169E+01, 1.2990E+01, 1.9193E+03, &
      1.4757249E+00, 5.37805E+01, 1.2908E+01, 1.8978E+03, &
      1.4831035E+00, 5.37423E+01, 1.2826E+01, 1.8764E+03, &
      1.4905190E+00, 5.37022E+01, 1.2744E+01, 1.8554E+03, &
      1.4979716E+00, 5.36602E+01, 1.2664E+01, 1.8346E+03, &
      1.5054615E+00, 5.36163E+01, 1.2584E+01, 1.8141E+03, &
      1.5129888E+00, 5.35704E+01, 1.2505E+01, 1.7939E+03, &
      1.5205537E+00, 5.35226E+01, 1.2426E+01, 1.7739E+03, &
      1.5281565E+00, 5.34728E+01, 1.2348E+01, 1.7541E+03, &
      1.5357973E+00, 5.34210E+01, 1.2271E+01, 1.7346E+03, &
      1.5434763E+00, 5.33671E+01, 1.2194E+01, 1.7153E+03, &
      1.5511937E+00, 5.33112E+01, 1.2119E+01, 1.6963E+03, &
      1.5589496E+00, 5.32599E+01, 1.2043E+01, 1.6775E+03, &
      1.5667444E+00, 5.31997E+01, 1.1969E+01, 1.6590E+03, &
      1.5745781E+00, 5.31373E+01, 1.1895E+01, 1.6406E+03, &
      1.5824510E+00, 5.30726E+01, 1.1821E+01, 1.6225E+03, &
      1.5903633E+00, 5.30056E+01, 1.1749E+01, 1.6047E+03, &
      1.5983151E+00, 5.29363E+01, 1.1677E+01, 1.5870E+03, &
      1.6063066E+00, 5.28645E+01, 1.1605E+01, 1.5696E+03, &
      1.6143382E+00, 5.27902E+01, 1.1534E+01, 1.5523E+03, &
      1.6224099E+00, 5.27134E+01, 1.1464E+01, 1.5353E+03, &
      1.6305219E+00, 5.26339E+01, 1.1394E+01, 1.5185E+03, &
      1.6386745E+00, 5.25517E+01, 1.1325E+01, 1.5019E+03, &
      1.6468679E+00, 5.24667E+01, 1.1256E+01, 1.4855E+03, &
      1.6551022E+00, 5.23788E+01, 1.1188E+01, 1.4694E+03, &
      1.6633777E+00, 5.22879E+01, 1.1121E+01, 1.4534E+03, &
      1.6716946E+00, 5.21939E+01, 1.1054E+01, 1.4376E+03, &
      1.6800531E+00, 5.20967E+01, 1.0988E+01, 1.4220E+03, &
      1.6884534E+00, 5.19961E+01, 1.0922E+01, 1.4065E+03, &
      1.6968956E+00, 5.18921E+01, 1.0857E+01, 1.3913E+03, &
      1.7053801E+00, 5.17845E+01, 1.0792E+01, 1.3763E+03, &
      1.7139070E+00, 5.16731E+01, 1.0728E+01, 1.3614E+03, &
      1.7224766E+00, 5.15578E+01, 1.0665E+01, 1.3467E+03, &
      1.7310889E+00, 5.14384E+01, 1.0602E+01, 1.3322E+03, &
      1.7397444E+00, 5.13148E+01, 1.0539E+01, 1.3179E+03, &
      1.7484431E+00, 5.11867E+01, 1.0477E+01, 1.3037E+03, &
      1.7571853E+00, 5.10538E+01, 1.0415E+01, 1.2898E+03, &
      1.7659712E+00, 5.09161E+01, 1.0354E+01, 1.2760E+03, &
      1.7748011E+00, 5.07731E+01, 1.0294E+01, 1.2623E+03, &
      1.7836751E+00, 5.06247E+01, 1.0234E+01, 1.2488E+03, &
      1.7925935E+00, 5.04705E+01, 1.0174E+01, 1.2355E+03, &
      1.8015565E+00, 5.03102E+01, 1.0115E+01, 1.2223E+03, &
      1.8105642E+00, 5.01434E+01, 1.0057E+01, 1.2093E+03, &
      1.8196171E+00, 4.99698E+01, 9.9984E+00, 1.1965E+03, &
      1.8287151E+00, 4.97898E+01, 9.9408E+00, 1.1838E+03, &
      1.8378587E+00, 4.96011E+01, 9.8836E+00, 1.1712E+03, &
      1.8470480E+00, 4.94042E+01, 9.8269E+00, 1.1588E+03, &
      1.8562833E+00, 4.91983E+01, 9.7706E+00, 1.1466E+03, &
      1.8655647E+00, 4.89829E+01, 9.7148E+00, 1.1345E+03, &
      1.8748925E+00, 4.87572E+01, 9.6594E+00, 1.1225E+03, &
      1.8842670E+00, 4.85205E+01, 9.6045E+00, 1.1107E+03, &
      1.8936883E+00, 4.82717E+01, 9.5500E+00, 1.0990E+03, &
      1.9031567E+00, 4.80099E+01, 9.4960E+00, 1.0875E+03, &
      1.9126725E+00, 4.77339E+01, 9.4424E+00, 1.0761E+03, &
      1.9222359E+00, 4.74424E+01, 9.3891E+00, 1.0648E+03, &
      1.9318471E+00, 4.71337E+01, 9.3363E+00, 1.0537E+03, &
      1.9415063E+00, 4.68061E+01, 9.2839E+00, 1.0427E+03, &
      1.9512138E+00, 4.64575E+01, 9.2319E+00, 1.0318E+03, &
      1.9609699E+00, 4.60855E+01, 9.1803E+00, 1.0210E+03, &
      1.9707747E+00, 4.56870E+01, 9.1291E+00, 1.0104E+03, &
      1.9806286E+00, 4.52586E+01, 9.0783E+00, 9.9988E+02, &
      1.9905318E+00, 4.47960E+01, 9.0279E+00, 9.8950E+02, &
      2.0004844E+00, 4.42939E+01, 8.9779E+00, 9.7924E+02, &
      2.0104868E+00, 4.37457E+01, 8.9283E+00, 9.6910E+02, &
      2.0205393E+00, 4.31428E+01, 8.8791E+00, 9.5907E+02, &
      2.0306420E+00, 4.24742E+01, 8.8303E+00, 9.4917E+02, &
      2.0407952E+00, 4.17247E+01, 8.7819E+00, 9.3938E+02, &
      2.0509992E+00, 4.08734E+01, 8.7338E+00, 9.2970E+02, &
      2.0612542E+00, 3.98895E+01, 8.6861E+00, 9.2013E+02, &
      2.0715604E+00, 3.87253E+01, 8.6388E+00, 9.1068E+02, &
      2.0819182E+00, 3.72998E+01, 8.5908E+00, 9.0122E+02, &
      2.0923278E+00, 3.54583E+01, 8.5423E+00, 8.9179E+02, &
      2.1027895E+00, 3.28376E+01, 8.4942E+00, 8.8247E+02, &
      2.1133034E+00, 2.81254E+01, 8.4464E+00, 8.7325E+02, &
      2.1210653E+00, 1.28681E+01, 8.4116E+00, 8.6655E+02, &
      2.1221346E+00, 1.25624E+01, 2.5721E+01, 2.6256E+03, &
      2.1238699E+00, 2.05054E+01, 2.5691E+01, 2.6204E+03, &
      2.1344893E+00, 2.97864E+01, 2.5509E+01, 2.5890E+03, &
      2.1451617E+00, 3.26834E+01, 2.5328E+01, 2.5580E+03, &
      2.1558875E+00, 3.41696E+01, 2.5148E+01, 2.5273E+03, &
      2.1666670E+00, 3.48770E+01, 2.4970E+01, 2.4971E+03, &
      2.1775003E+00, 3.48926E+01, 2.4793E+01, 2.4672E+03, &
      2.1883878E+00, 3.38880E+01, 2.4618E+01, 2.4377E+03, &
      2.1993297E+00, 2.88564E+01, 2.4444E+01, 2.4085E+03, &
      2.2011954E+00, 2.43030E+01, 2.4415E+01, 2.4036E+03, &
      2.2026046E+00, 2.42383E+01, 3.5850E+01, 3.5220E+03, &
      2.2103264E+00, 3.39315E+01, 3.5668E+01, 3.4919E+03, &
      2.2213780E+00, 3.77893E+01, 3.5410E+01, 3.4495E+03, &
      2.2324849E+00, 4.01466E+01, 3.5154E+01, 3.4077E+03, &
      2.2436473E+00, 4.19226E+01, 3.4900E+01, 3.3664E+03, &
      2.2548656E+00, 4.33738E+01, 3.4648E+01, 3.3256E+03, &
      2.2661399E+00, 4.46117E+01, 3.4398E+01, 3.2853E+03, &
      2.2774706E+00, 4.56958E+01, 3.4149E+01, 3.2454E+03, &
      2.2888579E+00, 4.66622E+01, 3.3903E+01, 3.2061E+03, &
      2.3003022E+00, 4.75347E+01, 3.3659E+01, 3.1673E+03, &
      2.3118037E+00, 4.83299E+01, 3.3416E+01, 3.1290E+03, &
      2.3233628E+00, 4.90598E+01, 3.3176E+01, 3.0911E+03, &
      2.3349796E+00, 4.97337E+01, 3.2937E+01, 3.0537E+03, &
      2.3466545E+00, 5.03585E+01, 3.2700E+01, 3.0168E+03, &
      2.3583878E+00, 5.09399E+01, 3.2465E+01, 2.9803E+03, &
      2.3701797E+00, 5.14823E+01, 3.2231E+01, 2.9443E+03, &
      2.3820306E+00, 5.19894E+01, 3.2000E+01, 2.9087E+03, &
      2.3939407E+00, 5.24641E+01, 3.1770E+01, 2.8736E+03, &
      2.4059104E+00, 5.29089E+01, 3.1542E+01, 2.8389E+03, &
      2.4179400E+00, 5.33257E+01, 3.1316E+01, 2.8046E+03, &
      2.4300297E+00, 5.37160E+01, 3.1091E+01, 2.7708E+03, &
      2.4421798E+00, 5.40812E+01, 3.0868E+01, 2.7373E+03, &
      2.4543907E+00, 5.44221E+01, 3.0646E+01, 2.7042E+03, &
      2.4666627E+00, 5.47392E+01, 3.0425E+01, 2.6715E+03, &
      2.4789960E+00, 5.50328E+01, 3.0206E+01, 2.6392E+03, &
      2.4913910E+00, 5.53029E+01, 2.9989E+01, 2.6074E+03, &
      2.5038479E+00, 5.55489E+01, 2.9774E+01, 2.5759E+03, &
      2.5163672E+00, 5.57698E+01, 2.9560E+01, 2.5448E+03, &
      2.5289490E+00, 5.59639E+01, 2.9348E+01, 2.5141E+03, &
      2.5415938E+00, 5.61284E+01, 2.9137E+01, 2.4837E+03, &
      2.5543017E+00, 5.62593E+01, 2.8928E+01, 2.4538E+03, &
      2.5670732E+00, 5.63501E+01, 2.8721E+01, 2.4242E+03, &
      2.5799086E+00, 5.63905E+01, 2.8515E+01, 2.3950E+03, &
      2.5928082E+00, 5.63628E+01, 2.8311E+01, 2.3661E+03, &
      2.6057722E+00, 5.62333E+01, 2.8108E+01, 2.3376E+03, &
      2.6188011E+00, 5.59261E+01, 2.7907E+01, 2.3094E+03, &
      2.6318951E+00, 5.52002E+01, 2.7708E+01, 2.2816E+03, &
      2.6429266E+00, 5.29149E+01, 2.7541E+01, 2.2586E+03, &
      2.6450545E+00, 5.00359E+01, 2.7510E+01, 2.2542E+03, &
      2.6478735E+00, 5.29484E+01, 3.2133E+01, 2.6284E+03, &
      2.6582798E+00, 5.56183E+01, 3.1952E+01, 2.6035E+03, &
      2.6715712E+00, 5.69294E+01, 3.1724E+01, 2.5721E+03, &
      2.6849291E+00, 5.77856E+01, 3.1497E+01, 2.5412E+03, &
      2.6983537E+00, 5.84520E+01, 3.1272E+01, 2.5106E+03, &
      2.7118455E+00, 5.90107E+01, 3.1049E+01, 2.4804E+03, &
      2.7254047E+00, 5.94983E+01, 3.0827E+01, 2.4505E+03, &
      2.7390317E+00, 5.99343E+01, 3.0607E+01, 2.4210E+03, &
      2.7527269E+00, 6.03302E+01, 3.0389E+01, 2.3919E+03, &
      2.7664905E+00, 6.06921E+01, 3.0167E+01, 2.3628E+03, &
      2.7803230E+00, 6.10216E+01, 2.9949E+01, 2.3341E+03, &
      2.7942246E+00, 6.13290E+01, 2.9741E+01, 2.3065E+03, &
      2.8081957E+00, 6.16185E+01, 2.9537E+01, 2.2794E+03, &
      2.8222367E+00, 6.18915E+01, 2.9334E+01, 2.2526E+03, &
      2.8363479E+00, 6.21491E+01, 2.9134E+01, 2.2262E+03, &
      2.8505296E+00, 6.23923E+01, 2.8936E+01, 2.2002E+03, &
      2.8647823E+00, 6.26218E+01, 2.8740E+01, 2.1745E+03, &
      2.8791062E+00, 6.28365E+01, 2.8541E+01, 2.1488E+03, &
      2.8935017E+00, 6.30361E+01, 2.8344E+01, 2.1235E+03, &
      2.9079692E+00, 6.32201E+01, 2.8148E+01, 2.0984E+03, &
      2.9225091E+00, 6.33874E+01, 2.7954E+01, 2.0737E+03, &
      2.9371216E+00, 6.35359E+01, 2.7761E+01, 2.0493E+03, &
      2.9518072E+00, 6.36620E+01, 2.7571E+01, 2.0252E+03, &
      2.9665662E+00, 6.37592E+01, 2.7381E+01, 2.0014E+03, &
      2.9813991E+00, 6.38150E+01, 2.7193E+01, 1.9778E+03, &
      2.9963061E+00, 6.38003E+01, 2.7007E+01, 1.9546E+03, &
      3.0112876E+00, 6.36373E+01, 2.6803E+01, 1.9303E+03, &
      3.0221418E+00, 6.30881E+01, 2.6653E+01, 1.9127E+03, &
      3.0263440E+00, 6.12660E+01, 2.6595E+01, 1.9059E+03, &
      3.0308581E+00, 6.31851E+01, 2.8305E+01, 2.0248E+03, &
      3.0414758E+00, 6.40249E+01, 2.8149E+01, 2.0067E+03, &
      3.0566831E+00, 6.46111E+01, 2.7929E+01, 1.9812E+03, &
      3.0719666E+00, 6.50211E+01, 2.7711E+01, 1.9561E+03, &
      3.0873264E+00, 6.53540E+01, 2.7494E+01, 1.9312E+03, &
      3.1027630E+00, 6.56406E+01, 2.7279E+01, 1.9067E+03, &
      3.1182768E+00, 6.58946E+01, 2.7065E+01, 1.8824E+03, &
      3.1338682E+00, 6.61231E+01, 2.6853E+01, 1.8585E+03, &
      3.1495376E+00, 6.63300E+01, 2.6642E+01, 1.8349E+03, &
      3.1652853E+00, 6.65170E+01, 2.6433E+01, 1.8115E+03, &
      3.1811117E+00, 6.66847E+01, 2.6228E+01, 1.7887E+03, &
      3.1970172E+00, 6.68356E+01, 2.6029E+01, 1.7663E+03, &
      3.2130023E+00, 6.69692E+01, 2.5831E+01, 1.7443E+03, &
      3.2290673E+00, 6.70817E+01, 2.5636E+01, 1.7226E+03, &
      3.2452127E+00, 6.71661E+01, 2.5442E+01, 1.7012E+03, &
      3.2614387E+00, 6.72060E+01, 2.5250E+01, 1.6801E+03, &
      3.2777459E+00, 6.71514E+01, 2.5060E+01, 1.6592E+03, &
      3.2891773E+00, 6.69216E+01, 2.4928E+01, 1.6448E+03, &
      3.2941347E+00, 6.65163E+01, 2.4871E+01, 1.6387E+03, &
      3.3028227E+00, 6.70492E+01, 2.5873E+01, 1.6998E+03, &
      3.3106053E+00, 6.73904E+01, 2.5784E+01, 1.6900E+03, &
      3.3271584E+00, 6.78095E+01, 2.5597E+01, 1.6695E+03, &
      3.3437941E+00, 6.81080E+01, 2.5411E+01, 1.6492E+03, &
      3.3605131E+00, 6.83571E+01, 2.5227E+01, 1.6293E+03, &
      3.3773157E+00, 6.85780E+01, 2.5045E+01, 1.6096E+03, &
      3.3942023E+00, 6.87799E+01, 2.4865E+01, 1.5901E+03, &
      3.4111733E+00, 6.89679E+01, 2.4686E+01, 1.5709E+03, &
      3.4282291E+00, 6.91449E+01, 2.4509E+01, 1.5520E+03, &
      3.4453703E+00, 6.93130E+01, 2.4332E+01, 1.5332E+03, &
      3.4625971E+00, 6.94729E+01, 2.4155E+01, 1.5146E+03, &
      3.4799101E+00, 6.96255E+01, 2.3981E+01, 1.4963E+03, &
      3.4973097E+00, 6.97719E+01, 2.3807E+01, 1.4782E+03, &
      3.5147962E+00, 6.99126E+01, 2.3636E+01, 1.4603E+03, &
      3.5323702E+00, 7.00482E+01, 2.3466E+01, 1.4427E+03, &
      3.5500321E+00, 7.01792E+01, 2.3297E+01, 1.4253E+03, &
      3.5677822E+00, 7.03058E+01, 2.3130E+01, 1.4082E+03, &
      3.5856211E+00, 7.04284E+01, 2.2965E+01, 1.3912E+03, &
      3.6035492E+00, 7.05473E+01, 2.2801E+01, 1.3745E+03, &
      3.6215670E+00, 7.06627E+01, 2.2638E+01, 1.3580E+03, &
      3.6396748E+00, 7.07749E+01, 2.2477E+01, 1.3418E+03, &
      3.6578732E+00, 7.08839E+01, 2.2318E+01, 1.3257E+03, &
      3.6761626E+00, 7.09900E+01, 2.2159E+01, 1.3098E+03, &
      3.6945434E+00, 7.10934E+01, 2.2002E+01, 1.2942E+03, &
      3.7130161E+00, 7.11941E+01, 2.1847E+01, 1.2788E+03, &
      3.7315812E+00, 7.12923E+01, 2.1693E+01, 1.2635E+03, &
      3.7502391E+00, 7.13882E+01, 2.1540E+01, 1.2484E+03, &
      3.7689903E+00, 7.14818E+01, 2.1388E+01, 1.2336E+03, &
      3.7878352E+00, 7.15732E+01, 2.1238E+01, 1.2189E+03, &
      3.8067744E+00, 7.16626E+01, 2.1089E+01, 1.2044E+03, &
      3.8258083E+00, 7.17501E+01, 2.0941E+01, 1.1901E+03, &
      3.8449373E+00, 7.18357E+01, 2.0795E+01, 1.1760E+03, &
      3.8641620E+00, 7.19197E+01, 2.0649E+01, 1.1621E+03, &
      3.8834828E+00, 7.20020E+01, 2.0505E+01, 1.1483E+03, &
      3.9029002E+00, 7.20829E+01, 2.0362E+01, 1.1347E+03, &
      3.9224147E+00, 7.21624E+01, 2.0221E+01, 1.1213E+03, &
      3.9420268E+00, 7.22406E+01, 2.0079E+01, 1.1080E+03, &
      3.9617369E+00, 7.23169E+01, 1.9936E+01, 1.0948E+03, &
      3.9815456E+00, 7.23912E+01, 1.9795E+01, 1.0817E+03, &
      4.0014533E+00, 7.24637E+01, 1.9655E+01, 1.0688E+03, &
      4.0214606E+00, 7.25346E+01, 1.9516E+01, 1.0560E+03, &
      4.0415679E+00, 7.26038E+01, 1.9378E+01, 1.0434E+03, &
      4.0617757E+00, 7.26716E+01, 1.9241E+01, 1.0310E+03, &
      4.0820846E+00, 7.27379E+01, 1.9105E+01, 1.0187E+03, &
      4.1024950E+00, 7.28028E+01, 1.8970E+01, 1.0066E+03, &
      4.1230075E+00, 7.28664E+01, 1.8836E+01, 9.9459E+02, &
      4.1436226E+00, 7.29288E+01, 1.8704E+01, 9.8275E+02, &
      4.1643407E+00, 7.29901E+01, 1.8572E+01, 9.7107E+02, &
      4.1851624E+00, 7.30502E+01, 1.8441E+01, 9.5952E+02, &
      4.2060882E+00, 7.31093E+01, 1.8312E+01, 9.4812E+02, &
      4.2271186E+00, 7.31674E+01, 1.8183E+01, 9.3686E+02, &
      4.2482542E+00, 7.34221E+01, 1.8053E+01, 9.2564E+02, &
      4.2694955E+00, 7.34787E+01, 1.7919E+01, 9.1424E+02, &
      4.2908430E+00, 7.35336E+01, 1.7785E+01, 9.0299E+02, &
      4.3122972E+00, 7.35866E+01, 1.7651E+01, 8.9185E+02, &
      4.3338587E+00, 7.36378E+01, 1.7519E+01, 8.8085E+02, &
      4.3555280E+00, 7.36875E+01, 1.7388E+01, 8.7000E+02, &
      4.3773056E+00, 7.37357E+01, 1.7258E+01, 8.5929E+02, &
      4.3991921E+00, 7.37825E+01, 1.7129E+01, 8.4872E+02, &
      4.4211881E+00, 7.39631E+01, 1.6997E+01, 8.3808E+02, &
      4.4432940E+00, 7.40071E+01, 1.6866E+01, 8.2754E+02, &
      4.4655105E+00, 7.40492E+01, 1.6735E+01, 8.1713E+02, &
      4.4878381E+00, 7.40897E+01, 1.6606E+01, 8.0686E+02, &
      4.5102772E+00, 7.41286E+01, 1.6477E+01, 7.9673E+02, &
      4.5328286E+00, 7.41660E+01, 1.6350E+01, 7.8674E+02, &
      4.5554928E+00, 7.42019E+01, 1.6224E+01, 7.7688E+02, &
      4.5782702E+00, 7.42364E+01, 1.6099E+01, 7.6714E+02, &
      4.6011616E+00, 7.42697E+01, 1.5975E+01, 7.5754E+02, &
      4.6241674E+00, 7.43016E+01, 1.5853E+01, 7.4807E+02, &
      4.6472882E+00, 7.43324E+01, 1.5731E+01, 7.3872E+02, &
      4.6705247E+00, 7.43620E+01, 1.5610E+01, 7.2949E+02, &
      4.6938773E+00, 7.43904E+01, 1.5491E+01, 7.2039E+02, &
      4.7173467E+00, 7.44178E+01, 1.5373E+01, 7.1141E+02, &
      4.7409334E+00, 7.44441E+01, 1.5255E+01, 7.0255E+02, &
      4.7646381E+00, 7.44694E+01, 1.5139E+01, 6.9380E+02, &
      4.7884613E+00, 7.44938E+01, 1.5023E+01, 6.8517E+02, &
      4.8124036E+00, 7.45172E+01, 1.4909E+01, 6.7666E+02, &
      4.8364656E+00, 7.45397E+01, 1.4796E+01, 6.6826E+02, &
      4.8606479E+00, 7.45613E+01, 1.4684E+01, 6.5997E+02, &
      4.8849512E+00, 7.45821E+01, 1.4572E+01, 6.5178E+02, &
      4.9093759E+00, 7.46020E+01, 1.4462E+01, 6.4371E+02, &
      4.9339228E+00, 7.46212E+01, 1.4353E+01, 6.3574E+02, &
      4.9585924E+00, 7.46396E+01, 1.4244E+01, 6.2788E+02, &
      4.9833854E+00, 7.46572E+01, 1.4137E+01, 6.2013E+02, &
      5.0083023E+00, 7.46741E+01, 1.4030E+01, 6.1247E+02, &
      5.0333438E+00, 7.46904E+01, 1.3925E+01, 6.0492E+02, &
      5.0585105E+00, 7.47059E+01, 1.3820E+01, 5.9746E+02, &
      5.0838031E+00, 7.47208E+01, 1.3716E+01, 5.9010E+02, &
      5.1092221E+00, 7.47351E+01, 1.3614E+01, 5.8284E+02, &
      5.1347682E+00, 7.47488E+01, 1.3512E+01, 5.7568E+02, &
      5.1604421E+00, 7.47619E+01, 1.3411E+01, 5.6861E+02, &
      5.1862443E+00, 7.47745E+01, 1.3311E+01, 5.6163E+02, &
      5.2121755E+00, 7.47865E+01, 1.3211E+01, 5.5475E+02, &
      5.2382364E+00, 7.47981E+01, 1.3113E+01, 5.4796E+02, &
      5.2644276E+00, 7.48091E+01, 1.3016E+01, 5.4125E+02, &
      5.2907497E+00, 7.48198E+01, 1.2919E+01, 5.3463E+02, &
      5.3172034E+00, 7.49308E+01, 1.2819E+01, 5.2794E+02, &
      5.3437895E+00, 7.49408E+01, 1.2720E+01, 5.2134E+02, &
      5.3705084E+00, 7.49499E+01, 1.2622E+01, 5.1482E+02, &
      5.3973609E+00, 7.49583E+01, 1.2525E+01, 5.0840E+02, &
      5.4243477E+00, 7.49659E+01, 1.2429E+01, 5.0205E+02, &
      5.4514695E+00, 7.49728E+01, 1.2334E+01, 4.9579E+02, &
      5.4787268E+00, 7.49790E+01, 1.2239E+01, 4.8962E+02, &
      5.5061205E+00, 7.49845E+01, 1.2145E+01, 4.8353E+02, &
      5.5336511E+00, 7.49894E+01, 1.2053E+01, 4.7751E+02, &
      5.5613193E+00, 7.49937E+01, 1.1960E+01, 4.7158E+02, &
      5.5891259E+00, 7.49974E+01, 1.1869E+01, 4.6573E+02, &
      5.6170716E+00, 7.50005E+01, 1.1779E+01, 4.5995E+02, &
      5.6451569E+00, 7.50031E+01, 1.1689E+01, 4.5425E+02, &
      5.6733827E+00, 7.50052E+01, 1.1600E+01, 4.4862E+02, &
      5.7017496E+00, 7.50067E+01, 1.1512E+01, 4.4307E+02, &
      5.7302584E+00, 7.50078E+01, 1.1425E+01, 4.3760E+02, &
      5.7589096E+00, 7.50084E+01, 1.1339E+01, 4.3219E+02, &
      5.7877042E+00, 7.50085E+01, 1.1253E+01, 4.2686E+02, &
      5.8166427E+00, 7.50082E+01, 1.1168E+01, 4.2160E+02, &
      5.8457259E+00, 7.50075E+01, 1.1084E+01, 4.1640E+02, &
      5.8749546E+00, 7.50063E+01, 1.1000E+01, 4.1128E+02, &
      5.9043293E+00, 7.50048E+01, 1.0917E+01, 4.0620E+02, &
      5.9338510E+00, 7.50028E+01, 1.0834E+01, 4.0117E+02, &
      5.9635202E+00, 7.50005E+01, 1.0752E+01, 3.9622E+02, &
      5.9933378E+00, 7.49977E+01, 1.0670E+01, 3.9133E+02, &
      6.0233045E+00, 7.49946E+01, 1.0590E+01, 3.8650E+02, &
      6.0534210E+00, 7.50335E+01, 1.0510E+01, 3.8174E+02, &
      6.0836882E+00, 7.50299E+01, 1.0429E+01, 3.7699E+02, &
      6.1141066E+00, 7.50259E+01, 1.0349E+01, 3.7231E+02, &
      6.1446771E+00, 7.50213E+01, 1.0270E+01, 3.6768E+02, &
      6.1754005E+00, 7.50163E+01, 1.0191E+01, 3.6312E+02, &
      6.2062775E+00, 7.50108E+01, 1.0113E+01, 3.5862E+02, &
      6.2373089E+00, 7.50049E+01, 1.0036E+01, 3.5416E+02, &
      6.2684954E+00, 7.49985E+01, 9.9587E+00, 3.4975E+02, &
      6.2998379E+00, 7.49918E+01, 9.8823E+00, 3.4541E+02, &
      6.3313371E+00, 7.49845E+01, 9.8066E+00, 3.4112E+02, &
      6.3629938E+00, 7.49769E+01, 9.7316E+00, 3.3689E+02, &
      6.3948088E+00, 7.49689E+01, 9.6573E+00, 3.3272E+02, &
      6.4267828E+00, 7.49605E+01, 9.5836E+00, 3.2860E+02, &
      6.4589167E+00, 7.49517E+01, 9.5105E+00, 3.2453E+02, &
      6.4912113E+00, 7.49426E+01, 9.4381E+00, 3.2052E+02, &
      6.5236674E+00, 7.49330E+01, 9.3664E+00, 3.1656E+02, &
      6.5562857E+00, 7.49232E+01, 9.2952E+00, 3.1266E+02, &
      6.5890671E+00, 7.49130E+01, 9.2247E+00, 3.0880E+02, &
      6.6220125E+00, 7.49187E+01, 9.1543E+00, 3.0498E+02, &
      6.6551225E+00, 7.49079E+01, 9.0845E+00, 3.0121E+02, &
      6.6883981E+00, 7.48967E+01, 9.0153E+00, 2.9748E+02, &
      6.7218401E+00, 7.48852E+01, 8.9467E+00, 2.9381E+02, &
      6.7554493E+00, 7.48733E+01, 8.8786E+00, 2.9019E+02, &
      6.7892266E+00, 7.48610E+01, 8.8112E+00, 2.8661E+02, &
      6.8231727E+00, 7.48484E+01, 8.7444E+00, 2.8308E+02, &
      6.8572886E+00, 7.48355E+01, 8.6782E+00, 2.7959E+02, &
      6.8915750E+00, 7.48222E+01, 8.6125E+00, 2.7615E+02, &
      6.9260329E+00, 7.48085E+01, 8.5474E+00, 2.7276E+02, &
      6.9606631E+00, 7.47946E+01, 8.4829E+00, 2.6941E+02, &
      6.9954664E+00, 7.47803E+01, 8.4189E+00, 2.6611E+02, &
      7.0304437E+00, 7.47657E+01, 8.3555E+00, 2.6284E+02, &
      7.0655959E+00, 7.47507E+01, 8.2926E+00, 2.5962E+02, &
      7.1009239E+00, 7.47355E+01, 8.2303E+00, 2.5645E+02, &
      7.1364285E+00, 7.47199E+01, 8.1686E+00, 2.5331E+02, &
      7.1721107E+00, 7.47041E+01, 8.1073E+00, 2.5022E+02, &
      7.2079712E+00, 7.46879E+01, 8.0466E+00, 2.4716E+02, &
      7.2440111E+00, 7.46714E+01, 7.9864E+00, 2.4415E+02, &
      7.2802311E+00, 7.46546E+01, 7.9268E+00, 2.4117E+02, &
      7.3166323E+00, 7.46376E+01, 7.8676E+00, 2.3823E+02, &
      7.3532155E+00, 7.46202E+01, 7.8090E+00, 2.3533E+02, &
      7.3899815E+00, 7.46025E+01, 7.7509E+00, 2.3247E+02, &
      7.4269314E+00, 7.45845E+01, 7.6933E+00, 2.2965E+02, &
      7.4640661E+00, 7.45662E+01, 7.6361E+00, 2.2686E+02, &
      7.5013864E+00, 7.45476E+01, 7.5795E+00, 2.2411E+02, &
      7.5388934E+00, 7.45287E+01, 7.5233E+00, 2.2139E+02, &
      7.5765878E+00, 7.45095E+01, 7.4677E+00, 2.1871E+02, &
      7.6144708E+00, 7.44900E+01, 7.4125E+00, 2.1607E+02, &
      7.6525431E+00, 7.44701E+01, 7.3577E+00, 2.1346E+02, &
      7.6908058E+00, 7.44500E+01, 7.3035E+00, 2.1088E+02, &
      7.7292599E+00, 7.44296E+01, 7.2497E+00, 2.0833E+02, &
      7.7679062E+00, 7.44089E+01, 7.1964E+00, 2.0582E+02, &
      7.8067457E+00, 7.43878E+01, 7.1435E+00, 2.0334E+02, &
      7.8457794E+00, 7.43664E+01, 7.0911E+00, 2.0089E+02, &
      7.8850083E+00, 7.43448E+01, 7.0391E+00, 1.9847E+02, &
      7.9244334E+00, 7.43228E+01, 6.9875E+00, 1.9609E+02, &
      7.9640555E+00, 7.43004E+01, 6.9364E+00, 1.9373E+02, &
      8.0038758E+00, 7.42778E+01, 6.8858E+00, 1.9141E+02, &
      8.0438952E+00, 7.42548E+01, 6.8355E+00, 1.8911E+02, &
      8.0841147E+00, 7.42315E+01, 6.7857E+00, 1.8685E+02, &
      8.1245352E+00, 7.42079E+01, 6.7363E+00, 1.8461E+02, &
      8.1651579E+00, 7.41839E+01, 6.6873E+00, 1.8240E+02, &
      8.2059837E+00, 7.41596E+01, 6.6388E+00, 1.8022E+02, &
      8.2470136E+00, 7.41350E+01, 6.5906E+00, 1.7807E+02, &
      8.2882487E+00, 7.41100E+01, 6.5429E+00, 1.7595E+02, &
      8.3296899E+00, 7.40847E+01, 6.4955E+00, 1.7385E+02, &
      8.3713384E+00, 7.40590E+01, 6.4486E+00, 1.7178E+02, &
      8.4131951E+00, 7.40330E+01, 6.4021E+00, 1.6974E+02, &
      8.4552610E+00, 7.40066E+01, 6.3559E+00, 1.6772E+02, &
      8.4975373E+00, 7.39799E+01, 6.3101E+00, 1.6572E+02, &
      8.5400250E+00, 7.39529E+01, 6.2647E+00, 1.6376E+02, &
      9.030794E+00, 7.36951E+01, 5.7619E+00, 1.4293E+02, &
      9.653919E+00, 7.31300E+01, 5.1487E+00, 1.2003E+02, &
      1.032004E+01, 7.22348E+01, 4.5951E+00, 1.0073E+02, &
      1.103212E+01, 7.05039E+01, 4.1092E+00, 8.4743E+01, &
      1.133243E+01, 6.88504E+01, 3.9307E+00, 7.9101E+01, &
      1.150588E+01, 6.61079E+01, 3.8327E+00, 7.6070E+01, &
      1.155214E+01, 6.28977E+01, 3.8063E+00, 7.5275E+01, &
      1.162152E+01, 6.60884E+01, 1.0039E+01, 1.9052E+02, &
      1.179334E+01, 6.88841E+01, 9.7860E+00, 1.8310E+02, &
      1.179497E+01, 6.88982E+01, 9.7837E+00, 1.8303E+02, &
      1.260708E+01, 7.13032E+01, 8.7112E+00, 1.5289E+02, &
      1.300715E+01, 7.09797E+01, 8.2482E+00, 1.4051E+02, &
      1.320624E+01, 6.97834E+01, 8.0311E+00, 1.3484E+02, &
      1.325933E+01, 6.81990E+01, 7.9745E+00, 1.3338E+02, &
      1.333896E+01, 6.98565E+01, 1.1061E+01, 1.8250E+02, &
      1.347697E+01, 7.10542E+01, 1.0866E+01, 1.7750E+02, &
      1.353805E+01, 7.13171E+01, 1.0781E+01, 1.7535E+02, &
      1.360230E+01, 7.15115E+01, 1.0693E+01, 1.7312E+02, &
      1.381050E+01, 7.15297E+01, 1.0415E+01, 1.6616E+02, &
      1.386602E+01, 7.08526E+01, 1.0343E+01, 1.6438E+02, &
      1.394930E+01, 7.18848E+01, 1.1865E+01, 1.8694E+02, &
      1.415750E+01, 7.30669E+01, 1.1594E+01, 1.8005E+02, &
      1.440688E+01, 7.38451E+01, 1.1283E+01, 1.7227E+02, &
      1.540095E+01, 7.55654E+01, 1.0156E+01, 1.4534E+02, &
      1.646362E+01, 7.65401E+01, 9.1308E+00, 1.2251E+02, &
      1.759961E+01, 7.71783E+01, 8.2076E+00, 1.0327E+02, &
      1.881398E+01, 7.76070E+01, 7.3678E+00, 8.6941E+01, &
      2.011215E+01, 7.78921E+01, 6.6140E+00, 7.3215E+01, &
      2.149988E+01, 7.80801E+01, 5.9398E+00, 6.1696E+01, &
      2.298338E+01, 7.82008E+01, 5.3372E+00, 5.2028E+01, &
      2.456923E+01, 7.83316E+01, 4.7908E+00, 4.3843E+01, &
      2.626450E+01, 7.83662E+01, 4.3029E+00, 3.6978E+01, &
      2.807676E+01, 7.84496E+01, 3.8346E+00, 3.0964E+01, &
      3.001405E+01, 7.84172E+01, 3.4044E+00, 2.5846E+01, &
      3.208502E+01, 7.83568E+01, 3.0249E+00, 2.1601E+01, &
      3.429889E+01, 7.82796E+01, 2.6899E+00, 1.8074E+01, &
      3.666551E+01, 7.81915E+01, 2.3940E+00, 1.5143E+01, &
      3.919543E+01, 7.80962E+01, 2.1323E+00, 1.2704E+01, &
      4.189992E+01, 7.79955E+01, 1.9008E+00, 1.0672E+01, &
      4.479101E+01, 7.78905E+01, 1.6959E+00, 8.9773E+00, &
      4.788159E+01, 7.77808E+01, 1.5140E+00, 7.5616E+00, &
      5.118542E+01, 7.76651E+01, 1.3516E+00, 6.3736E+00, &
      5.471721E+01, 7.75387E+01, 1.2057E+00, 5.3730E+00, &
      5.849270E+01, 7.73958E+01, 1.0758E+00, 4.5347E+00, &
      6.252870E+01, 7.72241E+01, 9.5706E-01, 3.8211E+00, &
      6.684318E+01, 7.69955E+01, 8.5033E-01, 3.2203E+00, &
      7.145536E+01, 7.66375E+01, 7.5579E-01, 2.7185E+00, &
      7.638578E+01, 7.57414E+01, 6.7191E-01, 2.2988E+00, &
      7.682690E+01, 7.55554E+01, 6.6514E-01, 2.2660E+00, &
      7.800282E+01, 7.44831E+01, 6.4764E-01, 2.1819E+00, &
      7.831640E+01, 7.31997E+01, 6.4310E-01, 2.1603E+00, &
      7.878677E+01, 7.44981E+01, 3.1616E+00, 9.0422E+00, &
      7.996269E+01, 7.56651E+01, 3.0867E+00, 8.7057E+00, &
      8.165640E+01, 7.62943E+01, 2.9959E+00, 8.2833E+00, &
      8.729069E+01, 7.71764E+01, 2.7112E+00, 7.0399E+00, &
      9.331374E+01, 7.76103E+01, 2.4374E+00, 5.9483E+00, &
      9.975239E+01, 7.78719E+01, 2.1733E+00, 4.9898E+00, &
      1.066353E+02, 7.80302E+01, 1.9395E+00, 4.1923E+00, &
      1.139931E+02, 7.81365E+01, 1.7348E+00, 3.5323E+00, &
      1.218587E+02, 7.82074E+01, 1.5519E+00, 2.9792E+00, &
      1.302669E+02, 7.82536E+01, 1.3884E+00, 2.5156E+00, &
      1.392553E+02, 7.82822E+01, 1.2424E+00, 2.1267E+00, &
      1.488640E+02, 7.82982E+01, 1.1118E+00, 1.8005E+00, &
      1.591356E+02, 7.83315E+01, 9.9455E-01, 1.5260E+00, &
      1.701159E+02, 7.83300E+01, 8.8804E-01, 1.2934E+00, &
      1.818539E+02, 7.83228E+01, 7.9302E-01, 1.0985E+00, &
      1.944018E+02, 7.83119E+01, 7.0825E-01, 9.3506E-01, &
      2.078156E+02, 7.82983E+01, 6.3260E-01, 7.9788E-01, &
      2.221548E+02, 7.82831E+01, 5.6508E-01, 6.8270E-01, &
      2.374835E+02, 7.82669E+01, 5.0482E-01, 5.8590E-01, &
      2.538699E+02, 7.82503E+01, 4.5103E-01, 5.0449E-01, &
      2.713869E+02, 7.82334E+01, 4.0300E-01, 4.3593E-01, &
      2.901126E+02, 7.82167E+01, 3.6012E-01, 3.7814E-01, &
      3.101304E+02, 7.82003E+01, 3.2183E-01, 3.2935E-01, &
      3.315294E+02, 7.81841E+01, 2.8763E-01, 2.8811E-01, &
      3.544049E+02, 7.81683E+01, 2.5855E-01, 2.5407E-01, &
      3.788588E+02, 7.81543E+01, 2.3320E-01, 2.2546E-01, &
      4.050001E+02, 7.81417E+01, 2.1035E-01, 2.0093E-01, &
      4.329451E+02, 7.81300E+01, 1.8975E-01, 1.7985E-01]
   case(79)
      frel = -1.0134E+00
      allocate(nist_tab(3072))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.654510E-03, 2.88762E+00, 0.0000E+00, 2.6831E-05, &
      6.756365E-03, 2.24634E+00, 0.0000E+00, 2.9003E-05, &
      6.783526E-03, 1.53627E+00, 0.0000E+00, 2.9604E-05, &
      6.824268E-03, 2.22782E+00, 1.3641E+00, 4.2705E+04, &
      6.926123E-03, 2.79003E+00, 1.3448E+00, 4.1481E+04, &
      8.142217E-03, 2.53588E+00, 1.2374E+00, 3.2469E+04, &
      8.266843E-03, 2.06540E+00, 1.2519E+00, 3.2352E+04, &
      8.300076E-03, 1.54542E+00, 1.2564E+00, 3.2341E+04, &
      8.349926E-03, 2.05749E+00, 2.2605E+00, 5.7837E+04, &
      8.474552E-03, 2.46605E+00, 2.2498E+00, 5.6718E+04, &
      1.069000E-02, 1.80913E+00, 2.9953E+00, 5.9861E+04, &
      1.142761E-02, 1.37655E+00, 3.6048E+00, 6.7392E+04, &
      1.221612E-02, 9.65328E-01, 4.4060E+00, 7.7054E+04, &
      1.305903E-02, 6.24303E-01, 5.4098E+00, 8.8504E+04, &
      1.396010E-02, 4.11793E-01, 6.6185E+00, 1.0129E+05, &
      1.492335E-02, 3.94079E-01, 8.0179E+00, 1.1478E+05, &
      1.595306E-02, 6.39962E-01, 9.5717E+00, 1.2818E+05, &
      1.705382E-02, 1.21194E+00, 1.1217E+01, 1.4053E+05, &
      1.823053E-02, 2.15504E+00, 1.2867E+01, 1.5079E+05, &
      1.948844E-02, 3.48533E+00, 1.4413E+01, 1.5801E+05, &
      2.083314E-02, 5.25490E+00, 1.5740E+01, 1.6141E+05, &
      2.227063E-02, 7.25321E+00, 1.6739E+01, 1.6058E+05, &
      2.380730E-02, 9.45427E+00, 1.7327E+01, 1.5549E+05, &
      2.545001E-02, 1.17727E+01, 1.7457E+01, 1.4655E+05, &
      2.720606E-02, 1.39985E+01, 1.7130E+01, 1.3452E+05, &
      2.908327E-02, 1.60474E+01, 1.6382E+01, 1.2034E+05, &
      3.109002E-02, 1.78149E+01, 1.5290E+01, 1.0507E+05, &
      3.323523E-02, 1.92362E+01, 1.3958E+01, 8.9725E+04, &
      3.552846E-02, 2.02840E+01, 1.2491E+01, 7.5109E+04, &
      3.797993E-02, 2.09586E+01, 1.0985E+01, 6.1791E+04, &
      4.060054E-02, 2.12841E+01, 9.5236E+00, 5.0114E+04, &
      4.340198E-02, 2.12906E+01, 8.1647E+00, 4.0190E+04, &
      4.639671E-02, 2.09863E+01, 6.9419E+00, 3.1965E+04, &
      4.959809E-02, 2.02798E+01, 5.8697E+00, 2.5284E+04, &
      5.262600E-02, 1.86073E+01, 5.0440E+00, 2.0477E+04, &
      5.302035E-02, 1.80425E+01, 4.9482E+00, 1.9938E+04, &
      5.343150E-02, 1.68898E+01, 4.8509E+00, 1.9396E+04, &
      5.364630E-02, 1.48645E+01, 4.8011E+00, 1.9120E+04, &
      5.396850E-02, 1.69775E+01, 8.6930E+00, 3.4412E+04, &
      5.477400E-02, 1.87833E+01, 8.3716E+00, 3.2653E+04, &
      5.667876E-02, 2.00123E+01, 7.6750E+00, 2.8930E+04, &
      6.058959E-02, 2.06579E+01, 6.4803E+00, 2.2850E+04, &
      6.477028E-02, 2.05746E+01, 5.4680E+00, 1.8036E+04, &
      6.923942E-02, 1.98937E+01, 4.6215E+00, 1.4260E+04, &
      7.026600E-02, 1.95410E+01, 4.4554E+00, 1.3546E+04, &
      7.134150E-02, 1.86908E+01, 4.2912E+00, 1.2851E+04, &
      7.162830E-02, 1.77435E+01, 4.2490E+00, 1.2673E+04, &
      7.205850E-02, 1.86642E+01, 6.0093E+00, 1.7817E+04, &
      7.313400E-02, 1.94156E+01, 5.8445E+00, 1.7073E+04, &
      7.401695E-02, 1.96412E+01, 5.7157E+00, 1.6498E+04, &
      7.912411E-02, 1.98939E+01, 5.0728E+00, 1.3697E+04, &
      8.114400E-02, 1.97664E+01, 4.8498E+00, 1.2769E+04, &
      8.238600E-02, 1.94939E+01, 4.7156E+00, 1.2228E+04, &
      8.271720E-02, 1.91793E+01, 4.6802E+00, 1.2088E+04, &
      8.321400E-02, 1.94857E+01, 5.2303E+00, 1.3428E+04, &
      8.445600E-02, 1.97029E+01, 5.0814E+00, 1.2854E+04, &
      8.458368E-02, 1.97089E+01, 5.0663E+00, 1.2797E+04, &
      8.467200E-02, 1.97120E+01, 5.0559E+00, 1.2757E+04, &
      8.596800E-02, 1.96317E+01, 4.9054E+00, 1.2191E+04, &
      8.631360E-02, 1.94433E+01, 4.8660E+00, 1.2044E+04, &
      8.683200E-02, 1.96712E+01, 5.2064E+00, 1.2810E+04, &
      8.812800E-02, 1.98761E+01, 5.0542E+00, 1.2253E+04, &
      9.041995E-02, 1.99742E+01, 4.7971E+00, 1.1335E+04, &
      9.665893E-02, 1.98306E+01, 4.2015E+00, 9.2869E+03, &
      1.033284E-01, 1.94054E+01, 3.7598E+00, 7.7744E+03, &
      1.056440E-01, 1.92152E+01, 3.6471E+00, 7.3761E+03, &
      1.072610E-01, 1.90492E+01, 3.5794E+00, 7.1300E+03, &
      1.076922E-01, 1.89666E+01, 3.5628E+00, 7.0686E+03, &
      1.083390E-01, 1.89635E+01, 3.6344E+00, 7.1675E+03, &
      1.099560E-01, 1.88743E+01, 3.5821E+00, 6.9604E+03, &
      1.104581E-01, 1.88392E+01, 3.5674E+00, 6.9006E+03, &
      1.180797E-01, 1.82190E+01, 3.4351E+00, 6.2158E+03, &
      1.262272E-01, 1.74805E+01, 3.4634E+00, 5.8627E+03, &
      1.349368E-01, 1.66550E+01, 3.6848E+00, 5.8351E+03, &
      1.442475E-01, 1.57662E+01, 4.1139E+00, 6.0941E+03, &
      1.542005E-01, 1.48392E+01, 4.7809E+00, 6.6251E+03, &
      1.648404E-01, 1.39200E+01, 5.7275E+00, 7.4246E+03, &
      1.762144E-01, 1.30583E+01, 6.9748E+00, 8.4579E+03, &
      1.883732E-01, 1.23320E+01, 8.5341E+00, 9.6807E+03, &
      2.013709E-01, 1.18252E+01, 1.0395E+01, 1.1031E+04, &
      2.152655E-01, 1.16285E+01, 1.2519E+01, 1.2427E+04, &
      2.301188E-01, 1.18101E+01, 1.4836E+01, 1.3777E+04, &
      2.459970E-01, 1.24275E+01, 1.7247E+01, 1.4982E+04, &
      2.629708E-01, 1.35101E+01, 1.9637E+01, 1.5957E+04, &
      2.811158E-01, 1.50151E+01, 2.1884E+01, 1.6634E+04, &
      3.005128E-01, 1.68766E+01, 2.3873E+01, 1.6976E+04, &
      3.212482E-01, 1.89208E+01, 2.5508E+01, 1.6968E+04, &
      3.272220E-01, 1.94424E+01, 2.5888E+01, 1.6906E+04, &
      3.322305E-01, 1.97143E+01, 2.6177E+01, 1.6837E+04, &
      3.335661E-01, 1.95627E+01, 2.6249E+01, 1.6816E+04, &
      3.355695E-01, 2.00379E+01, 2.6917E+01, 1.7141E+04, &
      3.405780E-01, 2.07741E+01, 2.7197E+01, 1.7065E+04, &
      3.434143E-01, 2.11074E+01, 2.7345E+01, 1.7015E+04, &
      3.449600E-01, 2.12760E+01, 2.7421E+01, 1.6987E+04, &
      3.502400E-01, 2.17325E+01, 2.7665E+01, 1.6880E+04, &
      3.516480E-01, 2.16840E+01, 2.7726E+01, 1.6849E+04, &
      3.537600E-01, 2.21370E+01, 2.8234E+01, 1.7056E+04, &
      3.590400E-01, 2.29277E+01, 2.8458E+01, 1.6939E+04, &
      3.671099E-01, 2.39458E+01, 2.8755E+01, 1.6739E+04, &
      3.924405E-01, 2.68316E+01, 2.9359E+01, 1.5988E+04, &
      4.195189E-01, 2.95822E+01, 2.9557E+01, 1.5057E+04, &
      4.484657E-01, 3.21468E+01, 2.9402E+01, 1.4012E+04, &
      4.794098E-01, 3.44526E+01, 2.8958E+01, 1.2911E+04, &
      5.0000000E-01, 3.68596E+01, 2.8559E+01, 1.2209E+04, &
      5.0250000E-01, 3.69982E+01, 2.8506E+01, 1.2126E+04, &
      5.0501250E-01, 3.71334E+01, 2.8452E+01, 1.2043E+04, &
      5.0753756E-01, 3.72650E+01, 2.8398E+01, 1.1960E+04, &
      5.1007525E-01, 3.73927E+01, 2.8342E+01, 1.1878E+04, &
      5.1262563E-01, 3.75160E+01, 2.8286E+01, 1.1795E+04, &
      5.1518875E-01, 3.76344E+01, 2.8228E+01, 1.1713E+04, &
      5.1776470E-01, 3.77473E+01, 2.8170E+01, 1.1630E+04, &
      5.2035352E-01, 3.78538E+01, 2.8111E+01, 1.1548E+04, &
      5.2295529E-01, 3.79528E+01, 2.8051E+01, 1.1466E+04, &
      5.2557007E-01, 3.80427E+01, 2.7990E+01, 1.1385E+04, &
      5.2819792E-01, 3.81212E+01, 2.7928E+01, 1.1303E+04, &
      5.3083891E-01, 3.81850E+01, 2.7866E+01, 1.1222E+04, &
      5.3349310E-01, 3.82285E+01, 2.7802E+01, 1.1141E+04, &
      5.3616057E-01, 3.82419E+01, 2.7738E+01, 1.1060E+04, &
      5.3884137E-01, 3.82052E+01, 2.7673E+01, 1.0979E+04, &
      5.4153558E-01, 3.80654E+01, 2.7608E+01, 1.0899E+04, &
      5.4424325E-01, 3.75543E+01, 2.7542E+01, 1.0818E+04, &
      5.4469646E-01, 3.73074E+01, 2.7531E+01, 1.0805E+04, &
      5.4610359E-01, 3.73821E+01, 2.9209E+01, 1.1434E+04, &
      5.4696447E-01, 3.78633E+01, 2.9188E+01, 1.1408E+04, &
      5.4969929E-01, 3.85583E+01, 2.9123E+01, 1.1326E+04, &
      5.5244779E-01, 3.89711E+01, 2.9058E+01, 1.1244E+04, &
      5.5521003E-01, 3.92936E+01, 2.8991E+01, 1.1163E+04, &
      5.5798608E-01, 3.95706E+01, 2.8924E+01, 1.1082E+04, &
      5.6077601E-01, 3.98197E+01, 2.8856E+01, 1.1001E+04, &
      5.6357989E-01, 4.00498E+01, 2.8788E+01, 1.0920E+04, &
      5.6639779E-01, 4.02659E+01, 2.8720E+01, 1.0840E+04, &
      5.6922978E-01, 4.04711E+01, 2.8650E+01, 1.0760E+04, &
      5.7207593E-01, 4.06676E+01, 2.8581E+01, 1.0681E+04, &
      5.7493630E-01, 4.08568E+01, 2.8510E+01, 1.0601E+04, &
      5.7781099E-01, 4.10397E+01, 2.8439E+01, 1.0523E+04, &
      5.8070004E-01, 4.12172E+01, 2.8368E+01, 1.0444E+04, &
      5.8360354E-01, 4.13899E+01, 2.8296E+01, 1.0366E+04, &
      5.8652156E-01, 4.15581E+01, 2.8224E+01, 1.0288E+04, &
      5.8945417E-01, 4.17223E+01, 2.8152E+01, 1.0211E+04, &
      5.9240144E-01, 4.18828E+01, 2.8079E+01, 1.0134E+04, &
      5.9536345E-01, 4.20397E+01, 2.8005E+01, 1.0057E+04, &
      5.9834026E-01, 4.21933E+01, 2.7931E+01, 9.9806E+03, &
      6.0133196E-01, 4.23436E+01, 2.7857E+01, 9.9047E+03, &
      6.0433862E-01, 4.24907E+01, 2.7783E+01, 9.8291E+03, &
      6.0736032E-01, 4.26347E+01, 2.7708E+01, 9.7539E+03, &
      6.1039712E-01, 4.27755E+01, 2.7632E+01, 9.6791E+03, &
      6.1344910E-01, 4.29130E+01, 2.7557E+01, 9.6047E+03, &
      6.1651635E-01, 4.30471E+01, 2.7481E+01, 9.5306E+03, &
      6.1959893E-01, 4.31775E+01, 2.7404E+01, 9.4570E+03, &
      6.2269693E-01, 4.33038E+01, 2.7328E+01, 9.3837E+03, &
      6.2581041E-01, 4.34254E+01, 2.7251E+01, 9.3108E+03, &
      6.2893946E-01, 4.35413E+01, 2.7174E+01, 9.2383E+03, &
      6.3208416E-01, 4.36495E+01, 2.7096E+01, 9.1662E+03, &
      6.3524458E-01, 4.37464E+01, 2.7018E+01, 9.0945E+03, &
      6.3842080E-01, 4.38227E+01, 2.6940E+01, 9.0232E+03, &
      6.4161291E-01, 4.38414E+01, 2.6862E+01, 8.9523E+03, &
      6.4278595E-01, 4.37886E+01, 2.6833E+01, 8.9265E+03, &
      6.4461406E-01, 4.38618E+01, 2.7171E+01, 9.0131E+03, &
      6.4482097E-01, 4.38949E+01, 2.7166E+01, 9.0086E+03, &
      6.4804508E-01, 4.41878E+01, 2.7089E+01, 8.9384E+03, &
      6.5128530E-01, 4.43826E+01, 2.7012E+01, 8.8687E+03, &
      6.5454173E-01, 4.45518E+01, 2.6934E+01, 8.7994E+03, &
      6.5781444E-01, 4.47084E+01, 2.6857E+01, 8.7305E+03, &
      6.6110351E-01, 4.48569E+01, 2.6779E+01, 8.6620E+03, &
      6.6440903E-01, 4.49998E+01, 2.6701E+01, 8.5939E+03, &
      6.6773107E-01, 4.51382E+01, 2.6623E+01, 8.5262E+03, &
      6.7106973E-01, 4.52729E+01, 2.6545E+01, 8.4590E+03, &
      6.7442508E-01, 4.54044E+01, 2.6466E+01, 8.3921E+03, &
      6.7779720E-01, 4.55331E+01, 2.6388E+01, 8.3257E+03, &
      6.8118619E-01, 4.56593E+01, 2.6309E+01, 8.2596E+03, &
      6.8459212E-01, 4.57830E+01, 2.6230E+01, 8.1940E+03, &
      6.8801508E-01, 4.59045E+01, 2.6151E+01, 8.1288E+03, &
      6.9145515E-01, 4.60238E+01, 2.6072E+01, 8.0640E+03, &
      6.9491243E-01, 4.61409E+01, 2.5993E+01, 7.9996E+03, &
      6.9838699E-01, 4.62559E+01, 2.5914E+01, 7.9356E+03, &
      7.0187893E-01, 4.63689E+01, 2.5834E+01, 7.8720E+03, &
      7.0538832E-01, 4.64797E+01, 2.5755E+01, 7.8088E+03, &
      7.0891526E-01, 4.65884E+01, 2.5675E+01, 7.7461E+03, &
      7.1245984E-01, 4.66948E+01, 2.5596E+01, 7.6837E+03, &
      7.1602214E-01, 4.67990E+01, 2.5516E+01, 7.6218E+03, &
      7.1960225E-01, 4.69006E+01, 2.5436E+01, 7.5603E+03, &
      7.2320026E-01, 4.69995E+01, 2.5356E+01, 7.4991E+03, &
      7.2681626E-01, 4.70954E+01, 2.5276E+01, 7.4384E+03, &
      7.3045034E-01, 4.71878E+01, 2.5197E+01, 7.3781E+03, &
      7.3410260E-01, 4.72762E+01, 2.5117E+01, 7.3182E+03, &
      7.3777311E-01, 4.73596E+01, 2.5036E+01, 7.2587E+03, &
      7.4146197E-01, 4.74364E+01, 2.4956E+01, 7.1995E+03, &
      7.4516928E-01, 4.75040E+01, 2.4876E+01, 7.1408E+03, &
      7.4889513E-01, 4.75571E+01, 2.4796E+01, 7.0825E+03, &
      7.5263961E-01, 4.75825E+01, 2.4716E+01, 7.0246E+03, &
      7.5640280E-01, 4.75268E+01, 2.4636E+01, 6.9671E+03, &
      7.5759354E-01, 4.74426E+01, 2.4611E+01, 6.9490E+03, &
      7.6000652E-01, 4.75088E+01, 2.5095E+01, 7.0633E+03, &
      7.6018482E-01, 4.75372E+01, 2.5092E+01, 7.0606E+03, &
      7.6398574E-01, 4.78657E+01, 2.5012E+01, 7.0034E+03, &
      7.6780567E-01, 4.80626E+01, 2.4933E+01, 6.9466E+03, &
      7.7164470E-01, 4.82251E+01, 2.4854E+01, 6.8901E+03, &
      7.7550292E-01, 4.83711E+01, 2.4775E+01, 6.8341E+03, &
      7.7938044E-01, 4.85071E+01, 2.4695E+01, 6.7784E+03, &
      7.8327734E-01, 4.86363E+01, 2.4616E+01, 6.7232E+03, &
      7.8719373E-01, 4.87605E+01, 2.4537E+01, 6.6683E+03, &
      7.9112969E-01, 4.88808E+01, 2.4458E+01, 6.6138E+03, &
      7.9508534E-01, 4.89978E+01, 2.4379E+01, 6.5597E+03, &
      7.9906077E-01, 4.91121E+01, 2.4300E+01, 6.5060E+03, &
      8.0305607E-01, 4.92241E+01, 2.4221E+01, 6.4527E+03, &
      8.0707135E-01, 4.93341E+01, 2.4141E+01, 6.3997E+03, &
      8.1110671E-01, 4.94422E+01, 2.4063E+01, 6.3471E+03, &
      8.1516224E-01, 4.95487E+01, 2.3984E+01, 6.2949E+03, &
      8.1923806E-01, 4.96537E+01, 2.3905E+01, 6.2431E+03, &
      8.2333425E-01, 4.97573E+01, 2.3826E+01, 6.1917E+03, &
      8.2745092E-01, 4.98596E+01, 2.3747E+01, 6.1406E+03, &
      8.3158817E-01, 4.99607E+01, 2.3668E+01, 6.0899E+03, &
      8.3574611E-01, 5.00607E+01, 2.3590E+01, 6.0396E+03, &
      8.3992484E-01, 5.01597E+01, 2.3511E+01, 5.9896E+03, &
      8.4412447E-01, 5.02577E+01, 2.3433E+01, 5.9400E+03, &
      8.4834509E-01, 5.03547E+01, 2.3354E+01, 5.8908E+03, &
      8.5258682E-01, 5.04508E+01, 2.3276E+01, 5.8419E+03, &
      8.5684975E-01, 5.05461E+01, 2.3197E+01, 5.7933E+03, &
      8.6113400E-01, 5.06405E+01, 2.3119E+01, 5.7452E+03, &
      8.6543967E-01, 5.07342E+01, 2.3041E+01, 5.6973E+03, &
      8.6976687E-01, 5.08271E+01, 2.2963E+01, 5.6498E+03, &
      8.7411570E-01, 5.09193E+01, 2.2884E+01, 5.6027E+03, &
      8.7848628E-01, 5.10108E+01, 2.2806E+01, 5.5559E+03, &
      8.8287871E-01, 5.11017E+01, 2.2728E+01, 5.5095E+03, &
      8.8729310E-01, 5.11919E+01, 2.2651E+01, 5.4634E+03, &
      8.9172957E-01, 5.12816E+01, 2.2573E+01, 5.4176E+03, &
      8.9618822E-01, 5.13708E+01, 2.2495E+01, 5.3722E+03, &
      9.0066916E-01, 5.14595E+01, 2.2417E+01, 5.3272E+03, &
      9.0517250E-01, 5.15477E+01, 2.2340E+01, 5.2824E+03, &
      9.0969837E-01, 5.16355E+01, 2.2262E+01, 5.2380E+03, &
      9.1424686E-01, 5.17230E+01, 2.2185E+01, 5.1940E+03, &
      9.1881809E-01, 5.18101E+01, 2.2108E+01, 5.1503E+03, &
      9.2341218E-01, 5.18970E+01, 2.2031E+01, 5.1068E+03, &
      9.2802924E-01, 5.19834E+01, 2.1953E+01, 5.0636E+03, &
      9.3266939E-01, 5.20696E+01, 2.1875E+01, 5.0206E+03, &
      9.3733274E-01, 5.21555E+01, 2.1797E+01, 4.9780E+03, &
      9.4201940E-01, 5.22412E+01, 2.1720E+01, 4.9358E+03, &
      9.4672950E-01, 5.23267E+01, 2.1643E+01, 4.8938E+03, &
      9.5146315E-01, 5.24122E+01, 2.1565E+01, 4.8522E+03, &
      9.5622046E-01, 5.24977E+01, 2.1488E+01, 4.8109E+03, &
      9.6100156E-01, 5.25833E+01, 2.1411E+01, 4.7699E+03, &
      9.6580657E-01, 5.26691E+01, 2.1334E+01, 4.7292E+03, &
      9.7063560E-01, 5.27551E+01, 2.1257E+01, 4.6889E+03, &
      9.7548878E-01, 5.28415E+01, 2.1181E+01, 4.6488E+03, &
      9.8036623E-01, 5.29284E+01, 2.1104E+01, 4.6091E+03, &
      9.8526806E-01, 5.30160E+01, 2.1028E+01, 4.5697E+03, &
      9.9019440E-01, 5.31045E+01, 2.0952E+01, 4.5306E+03, &
      9.9514537E-01, 5.31943E+01, 2.0876E+01, 4.4918E+03, &
      1.0001211E+00, 5.32862E+01, 2.0799E+01, 4.4532E+03, &
      1.0051217E+00, 5.33772E+01, 2.0696E+01, 4.4091E+03, &
      1.0101473E+00, 5.34425E+01, 2.0593E+01, 4.3654E+03, &
      1.0151980E+00, 5.35046E+01, 2.0490E+01, 4.3222E+03, &
      1.0202740E+00, 5.35648E+01, 2.0388E+01, 4.2793E+03, &
      1.0253754E+00, 5.36233E+01, 2.0286E+01, 4.2369E+03, &
      1.0305023E+00, 5.36804E+01, 2.0185E+01, 4.1949E+03, &
      1.0356548E+00, 5.37361E+01, 2.0084E+01, 4.1533E+03, &
      1.0408331E+00, 5.37905E+01, 1.9984E+01, 4.1121E+03, &
      1.0460372E+00, 5.38437E+01, 1.9884E+01, 4.0714E+03, &
      1.0512674E+00, 5.38957E+01, 1.9785E+01, 4.0310E+03, &
      1.0565238E+00, 5.39595E+01, 1.9686E+01, 3.9910E+03, &
      1.0618064E+00, 5.40092E+01, 1.9587E+01, 3.9515E+03, &
      1.0671154E+00, 5.40578E+01, 1.9490E+01, 3.9123E+03, &
      1.0724510E+00, 5.41054E+01, 1.9392E+01, 3.8735E+03, &
      1.0778132E+00, 5.41518E+01, 1.9296E+01, 3.8351E+03, &
      1.0832023E+00, 5.41973E+01, 1.9199E+01, 3.7971E+03, &
      1.0886183E+00, 5.42417E+01, 1.9103E+01, 3.7595E+03, &
      1.0940614E+00, 5.42850E+01, 1.9007E+01, 3.7221E+03, &
      1.0995317E+00, 5.43271E+01, 1.8911E+01, 3.6850E+03, &
      1.1050294E+00, 5.43681E+01, 1.8816E+01, 3.6483E+03, &
      1.1105545E+00, 5.44080E+01, 1.8721E+01, 3.6120E+03, &
      1.1161073E+00, 5.44468E+01, 1.8627E+01, 3.5761E+03, &
      1.1216878E+00, 5.44846E+01, 1.8533E+01, 3.5405E+03, &
      1.1272963E+00, 5.45214E+01, 1.8440E+01, 3.5052E+03, &
      1.1329328E+00, 5.45571E+01, 1.8347E+01, 3.4704E+03, &
      1.1385974E+00, 5.45918E+01, 1.8255E+01, 3.4358E+03, &
      1.1442904E+00, 5.46255E+01, 1.8163E+01, 3.4017E+03, &
      1.1500119E+00, 5.46582E+01, 1.8071E+01, 3.3678E+03, &
      1.1557619E+00, 5.46899E+01, 1.7981E+01, 3.3343E+03, &
      1.1615407E+00, 5.47207E+01, 1.7890E+01, 3.3012E+03, &
      1.1673484E+00, 5.47505E+01, 1.7800E+01, 3.2684E+03, &
      1.1731852E+00, 5.47793E+01, 1.7711E+01, 3.2359E+03, &
      1.1790511E+00, 5.48073E+01, 1.7622E+01, 3.2038E+03, &
      1.1849464E+00, 5.48343E+01, 1.7533E+01, 3.1720E+03, &
      1.1908711E+00, 5.48604E+01, 1.7446E+01, 3.1405E+03, &
      1.1968254E+00, 5.48856E+01, 1.7358E+01, 3.1093E+03, &
      1.2028096E+00, 5.49100E+01, 1.7271E+01, 3.0784E+03, &
      1.2088236E+00, 5.49334E+01, 1.7185E+01, 3.0479E+03, &
      1.2148677E+00, 5.49560E+01, 1.7099E+01, 3.0177E+03, &
      1.2209421E+00, 5.49777E+01, 1.7013E+01, 2.9878E+03, &
      1.2270468E+00, 5.49986E+01, 1.6928E+01, 2.9582E+03, &
      1.2331820E+00, 5.50186E+01, 1.6844E+01, 2.9289E+03, &
      1.2393479E+00, 5.50378E+01, 1.6759E+01, 2.8999E+03, &
      1.2455447E+00, 5.50562E+01, 1.6676E+01, 2.8712E+03, &
      1.2517724E+00, 5.50738E+01, 1.6593E+01, 2.8428E+03, &
      1.2580312E+00, 5.50906E+01, 1.6510E+01, 2.8147E+03, &
      1.2643214E+00, 5.51066E+01, 1.6428E+01, 2.7869E+03, &
      1.2706430E+00, 5.51219E+01, 1.6347E+01, 2.7594E+03, &
      1.2769962E+00, 5.51364E+01, 1.6265E+01, 2.7321E+03, &
      1.2833812E+00, 5.51502E+01, 1.6185E+01, 2.7052E+03, &
      1.2897981E+00, 5.51631E+01, 1.6104E+01, 2.6784E+03, &
      1.2962471E+00, 5.51753E+01, 1.6024E+01, 2.6519E+03, &
      1.3027283E+00, 5.51866E+01, 1.5944E+01, 2.6257E+03, &
      1.3092420E+00, 5.51972E+01, 1.5865E+01, 2.5998E+03, &
      1.3157882E+00, 5.52070E+01, 1.5786E+01, 2.5741E+03, &
      1.3223671E+00, 5.52161E+01, 1.5707E+01, 2.5487E+03, &
      1.3289790E+00, 5.52245E+01, 1.5629E+01, 2.5236E+03, &
      1.3356239E+00, 5.52322E+01, 1.5552E+01, 2.4987E+03, &
      1.3423020E+00, 5.52388E+01, 1.5466E+01, 2.4727E+03, &
      1.3490135E+00, 5.52440E+01, 1.5381E+01, 2.4469E+03, &
      1.3557586E+00, 5.52479E+01, 1.5296E+01, 2.4215E+03, &
      1.3625374E+00, 5.52505E+01, 1.5212E+01, 2.3963E+03, &
      1.3693500E+00, 5.52518E+01, 1.5128E+01, 2.3714E+03, &
      1.3761968E+00, 5.52518E+01, 1.5045E+01, 2.3467E+03, &
      1.3830778E+00, 5.52505E+01, 1.4963E+01, 2.3224E+03, &
      1.3899932E+00, 5.52480E+01, 1.4881E+01, 2.2983E+03, &
      1.3969431E+00, 5.52443E+01, 1.4799E+01, 2.2745E+03, &
      1.4039278E+00, 5.52394E+01, 1.4719E+01, 2.2509E+03, &
      1.4109475E+00, 5.52333E+01, 1.4636E+01, 2.2273E+03, &
      1.4180022E+00, 5.52256E+01, 1.4551E+01, 2.2034E+03, &
      1.4250922E+00, 5.52163E+01, 1.4466E+01, 2.1798E+03, &
      1.4322177E+00, 5.52055E+01, 1.4382E+01, 2.1565E+03, &
      1.4393788E+00, 5.51930E+01, 1.4298E+01, 2.1334E+03, &
      1.4465757E+00, 5.51790E+01, 1.4215E+01, 2.1106E+03, &
      1.4538086E+00, 5.51634E+01, 1.4133E+01, 2.0881E+03, &
      1.4610776E+00, 5.51463E+01, 1.4051E+01, 2.0658E+03, &
      1.4683830E+00, 5.51277E+01, 1.3970E+01, 2.0438E+03, &
      1.4757249E+00, 5.51075E+01, 1.3889E+01, 2.0220E+03, &
      1.4831035E+00, 5.50857E+01, 1.3809E+01, 2.0005E+03, &
      1.4905190E+00, 5.50624E+01, 1.3730E+01, 1.9792E+03, &
      1.4979716E+00, 5.50376E+01, 1.3651E+01, 1.9582E+03, &
      1.5054615E+00, 5.50112E+01, 1.3573E+01, 1.9374E+03, &
      1.5129888E+00, 5.49832E+01, 1.3495E+01, 1.9168E+03, &
      1.5205537E+00, 5.49537E+01, 1.3418E+01, 1.8965E+03, &
      1.5281565E+00, 5.49226E+01, 1.3341E+01, 1.8764E+03, &
      1.5357973E+00, 5.48898E+01, 1.3265E+01, 1.8566E+03, &
      1.5434763E+00, 5.48555E+01, 1.3189E+01, 1.8369E+03, &
      1.5511937E+00, 5.48194E+01, 1.3114E+01, 1.8175E+03, &
      1.5589496E+00, 5.47818E+01, 1.3037E+01, 1.7979E+03, &
      1.5667444E+00, 5.47424E+01, 1.2957E+01, 1.7782E+03, &
      1.5745781E+00, 5.47010E+01, 1.2878E+01, 1.7587E+03, &
      1.5824510E+00, 5.46577E+01, 1.2800E+01, 1.7394E+03, &
      1.5903633E+00, 5.46123E+01, 1.2722E+01, 1.7204E+03, &
      1.5983151E+00, 5.45649E+01, 1.2645E+01, 1.7016E+03, &
      1.6063066E+00, 5.45154E+01, 1.2569E+01, 1.6831E+03, &
      1.6143382E+00, 5.44639E+01, 1.2493E+01, 1.6647E+03, &
      1.6224099E+00, 5.44102E+01, 1.2418E+01, 1.6466E+03, &
      1.6305219E+00, 5.43544E+01, 1.2340E+01, 1.6282E+03, &
      1.6386745E+00, 5.43040E+01, 1.2262E+01, 1.6100E+03, &
      1.6468679E+00, 5.42434E+01, 1.2184E+01, 1.5920E+03, &
      1.6551022E+00, 5.41804E+01, 1.2108E+01, 1.5742E+03, &
      1.6633777E+00, 5.41148E+01, 1.2032E+01, 1.5567E+03, &
      1.6716946E+00, 5.40467E+01, 1.1957E+01, 1.5394E+03, &
      1.6800531E+00, 5.39759E+01, 1.1882E+01, 1.5223E+03, &
      1.6884534E+00, 5.39024E+01, 1.1808E+01, 1.5054E+03, &
      1.6968956E+00, 5.38261E+01, 1.1735E+01, 1.4888E+03, &
      1.7053801E+00, 5.37470E+01, 1.1662E+01, 1.4723E+03, &
      1.7139070E+00, 5.36650E+01, 1.1590E+01, 1.4561E+03, &
      1.7224766E+00, 5.35799E+01, 1.1518E+01, 1.4400E+03, &
      1.7310889E+00, 5.34918E+01, 1.1447E+01, 1.4242E+03, &
      1.7397444E+00, 5.34004E+01, 1.1377E+01, 1.4085E+03, &
      1.7484431E+00, 5.33057E+01, 1.1307E+01, 1.3930E+03, &
      1.7571853E+00, 5.32076E+01, 1.1238E+01, 1.3778E+03, &
      1.7659712E+00, 5.31060E+01, 1.1170E+01, 1.3627E+03, &
      1.7748011E+00, 5.30006E+01, 1.1102E+01, 1.3478E+03, &
      1.7836751E+00, 5.28914E+01, 1.1035E+01, 1.3331E+03, &
      1.7925935E+00, 5.27782E+01, 1.0968E+01, 1.3185E+03, &
      1.8015565E+00, 5.26608E+01, 1.0902E+01, 1.3042E+03, &
      1.8105642E+00, 5.25391E+01, 1.0836E+01, 1.2900E+03, &
      1.8196171E+00, 5.24128E+01, 1.0771E+01, 1.2760E+03, &
      1.8287151E+00, 5.22817E+01, 1.0706E+01, 1.2622E+03, &
      1.8378587E+00, 5.21456E+01, 1.0642E+01, 1.2485E+03, &
      1.8470480E+00, 5.20043E+01, 1.0579E+01, 1.2350E+03, &
      1.8562833E+00, 5.18574E+01, 1.0516E+01, 1.2217E+03, &
      1.8655647E+00, 5.17047E+01, 1.0453E+01, 1.2085E+03, &
      1.8748925E+00, 5.15457E+01, 1.0391E+01, 1.1955E+03, &
      1.8842670E+00, 5.13803E+01, 1.0330E+01, 1.1826E+03, &
      1.8936883E+00, 5.12079E+01, 1.0269E+01, 1.1699E+03, &
      1.9031567E+00, 5.10281E+01, 1.0209E+01, 1.1574E+03, &
      1.9126725E+00, 5.08404E+01, 1.0149E+01, 1.1450E+03, &
      1.9222359E+00, 5.06444E+01, 1.0089E+01, 1.1327E+03, &
      1.9318471E+00, 5.04395E+01, 1.0030E+01, 1.1206E+03, &
      1.9415063E+00, 5.02249E+01, 9.9719E+00, 1.1087E+03, &
      1.9512138E+00, 4.99998E+01, 9.9140E+00, 1.0969E+03, &
      1.9609699E+00, 4.97636E+01, 9.8565E+00, 1.0852E+03, &
      1.9707747E+00, 4.95152E+01, 9.7994E+00, 1.0737E+03, &
      1.9806286E+00, 4.92537E+01, 9.7429E+00, 1.0623E+03, &
      1.9905318E+00, 4.89778E+01, 9.6868E+00, 1.0510E+03, &
      2.0004844E+00, 4.86861E+01, 9.6311E+00, 1.0399E+03, &
      2.0104868E+00, 4.83770E+01, 9.5759E+00, 1.0289E+03, &
      2.0205393E+00, 4.80488E+01, 9.5212E+00, 1.0181E+03, &
      2.0306420E+00, 4.76993E+01, 9.4668E+00, 1.0073E+03, &
      2.0407952E+00, 4.73259E+01, 9.4129E+00, 9.9674E+02, &
      2.0509992E+00, 4.69257E+01, 9.3594E+00, 9.8626E+02, &
      2.0612542E+00, 4.64950E+01, 9.3063E+00, 9.7591E+02, &
      2.0715604E+00, 4.60294E+01, 9.2537E+00, 9.6567E+02, &
      2.0819182E+00, 4.55233E+01, 9.2015E+00, 9.5556E+02, &
      2.0923278E+00, 4.49700E+01, 9.1496E+00, 9.4557E+02, &
      2.1027895E+00, 4.43604E+01, 9.0983E+00, 9.3570E+02, &
      2.1133034E+00, 4.36829E+01, 9.0473E+00, 9.2594E+02, &
      2.1238699E+00, 4.29216E+01, 8.9967E+00, 9.1629E+02, &
      2.1344893E+00, 4.20539E+01, 8.9465E+00, 9.0676E+02, &
      2.1451617E+00, 4.10468E+01, 8.8967E+00, 8.9734E+02, &
      2.1558875E+00, 3.98476E+01, 8.8473E+00, 8.8804E+02, &
      2.1666670E+00, 3.83658E+01, 8.7983E+00, 8.7884E+02, &
      2.1775003E+00, 3.64219E+01, 8.7497E+00, 8.6974E+02, &
      2.1883878E+00, 3.35677E+01, 8.7003E+00, 8.6064E+02, &
      2.1993297E+00, 2.78928E+01, 8.6506E+00, 8.5158E+02, &
      2.2051132E+00, 1.47180E+01, 8.6246E+00, 8.4685E+02, &
      2.2062866E+00, 1.44147E+01, 2.5848E+01, 2.5142E+03, &
      2.2103264E+00, 2.56735E+01, 2.5780E+01, 2.5031E+03, &
      2.2213780E+00, 3.20631E+01, 2.5595E+01, 2.4728E+03, &
      2.2324849E+00, 3.45666E+01, 2.5411E+01, 2.4430E+03, &
      2.2436473E+00, 3.58899E+01, 2.5228E+01, 2.4135E+03, &
      2.2548656E+00, 3.65015E+01, 2.5047E+01, 2.3843E+03, &
      2.2661399E+00, 3.64465E+01, 2.4867E+01, 2.3556E+03, &
      2.2774706E+00, 3.53612E+01, 2.4689E+01, 2.3272E+03, &
      2.2888579E+00, 2.97952E+01, 2.4512E+01, 2.2992E+03, &
      2.2904585E+00, 2.53813E+01, 2.4487E+01, 2.2953E+03, &
      2.2917415E+00, 2.52987E+01, 3.5886E+01, 3.3566E+03, &
      2.3003022E+00, 3.56491E+01, 3.5689E+01, 3.3259E+03, &
      2.3118037E+00, 3.93788E+01, 3.5427E+01, 3.2852E+03, &
      2.3233628E+00, 4.16868E+01, 3.5168E+01, 3.2450E+03, &
      2.3349796E+00, 4.34321E+01, 3.4910E+01, 3.2053E+03, &
      2.3466545E+00, 4.48607E+01, 3.4654E+01, 3.1661E+03, &
      2.3583878E+00, 4.60806E+01, 3.4400E+01, 3.1274E+03, &
      2.3701797E+00, 4.71495E+01, 3.4148E+01, 3.0892E+03, &
      2.3820306E+00, 4.81028E+01, 3.3898E+01, 3.0515E+03, &
      2.3939407E+00, 4.89635E+01, 3.3650E+01, 3.0142E+03, &
      2.4059104E+00, 4.97480E+01, 3.3405E+01, 2.9774E+03, &
      2.4179400E+00, 5.04680E+01, 3.3160E+01, 2.9411E+03, &
      2.4300297E+00, 5.11326E+01, 3.2918E+01, 2.9052E+03, &
      2.4421798E+00, 5.17487E+01, 3.2678E+01, 2.8698E+03, &
      2.4543907E+00, 5.23217E+01, 3.2440E+01, 2.8348E+03, &
      2.4666627E+00, 5.28561E+01, 3.2203E+01, 2.8002E+03, &
      2.4789960E+00, 5.33553E+01, 3.1968E+01, 2.7661E+03, &
      2.4913910E+00, 5.38222E+01, 3.1736E+01, 2.7324E+03, &
      2.5038479E+00, 5.42592E+01, 3.1504E+01, 2.6992E+03, &
      2.5163672E+00, 5.46683E+01, 3.1275E+01, 2.6663E+03, &
      2.5289490E+00, 5.50508E+01, 3.1048E+01, 2.6339E+03, &
      2.5415938E+00, 5.54079E+01, 3.0822E+01, 2.6018E+03, &
      2.5543017E+00, 5.57405E+01, 3.0598E+01, 2.5702E+03, &
      2.5670732E+00, 5.60490E+01, 3.0376E+01, 2.5390E+03, &
      2.5799086E+00, 5.63336E+01, 3.0155E+01, 2.5081E+03, &
      2.5928082E+00, 5.65938E+01, 2.9935E+01, 2.4776E+03, &
      2.6057722E+00, 5.68290E+01, 2.9717E+01, 2.4474E+03, &
      2.6188011E+00, 5.70376E+01, 2.9501E+01, 2.4176E+03, &
      2.6318951E+00, 5.72174E+01, 2.9286E+01, 2.3882E+03, &
      2.6450545E+00, 5.73650E+01, 2.9073E+01, 2.3592E+03, &
      2.6582798E+00, 5.74747E+01, 2.8862E+01, 2.3305E+03, &
      2.6715712E+00, 5.75380E+01, 2.8652E+01, 2.3022E+03, &
      2.6849291E+00, 5.75399E+01, 2.8444E+01, 2.2742E+03, &
      2.6983537E+00, 5.74535E+01, 2.8238E+01, 2.2466E+03, &
      2.7118455E+00, 5.72210E+01, 2.8033E+01, 2.2193E+03, &
      2.7254047E+00, 5.66801E+01, 2.7830E+01, 2.1924E+03, &
      2.7390317E+00, 5.48263E+01, 2.7628E+01, 2.1658E+03, &
      2.7404353E+00, 5.42320E+01, 2.7607E+01, 2.1631E+03, &
      2.7455647E+00, 5.42566E+01, 3.2004E+01, 2.5012E+03, &
      2.7527269E+00, 5.62907E+01, 3.1894E+01, 2.4861E+03, &
      2.7664905E+00, 5.77964E+01, 3.1684E+01, 2.4576E+03, &
      2.7803230E+00, 5.86963E+01, 3.1476E+01, 2.4294E+03, &
      2.7942246E+00, 5.93792E+01, 3.1270E+01, 2.4016E+03, &
      2.8081957E+00, 5.99469E+01, 3.1064E+01, 2.3741E+03, &
      2.8222367E+00, 6.04418E+01, 3.0861E+01, 2.3469E+03, &
      2.8363479E+00, 6.08855E+01, 3.0659E+01, 2.3200E+03, &
      2.8505296E+00, 6.12908E+01, 3.0458E+01, 2.2935E+03, &
      2.8647823E+00, 6.16661E+01, 3.0259E+01, 2.2673E+03, &
      2.8791062E+00, 6.20157E+01, 3.0055E+01, 2.2409E+03, &
      2.8935017E+00, 6.23422E+01, 2.9851E+01, 2.2147E+03, &
      2.9079692E+00, 6.26487E+01, 2.9649E+01, 2.1889E+03, &
      2.9225091E+00, 6.29375E+01, 2.9448E+01, 2.1634E+03, &
      2.9371216E+00, 6.32102E+01, 2.9249E+01, 2.1381E+03, &
      2.9518072E+00, 6.34683E+01, 2.9051E+01, 2.1132E+03, &
      2.9665662E+00, 6.37128E+01, 2.8854E+01, 2.0885E+03, &
      2.9813991E+00, 6.39446E+01, 2.8658E+01, 2.0641E+03, &
      2.9963061E+00, 6.41630E+01, 2.8460E+01, 2.0398E+03, &
      3.0112876E+00, 6.43822E+01, 2.8244E+01, 2.0143E+03, &
      3.0263440E+00, 6.45811E+01, 2.8023E+01, 1.9888E+03, &
      3.0414758E+00, 6.47577E+01, 2.7805E+01, 1.9636E+03, &
      3.0566831E+00, 6.49116E+01, 2.7587E+01, 1.9386E+03, &
      3.0719666E+00, 6.50396E+01, 2.7372E+01, 1.9140E+03, &
      3.0873264E+00, 6.51351E+01, 2.7158E+01, 1.8897E+03, &
      3.1027630E+00, 6.51847E+01, 2.6945E+01, 1.8657E+03, &
      3.1182768E+00, 6.51563E+01, 2.6734E+01, 1.8420E+03, &
      3.1338682E+00, 6.49396E+01, 2.6525E+01, 1.8186E+03, &
      3.1431727E+00, 6.44455E+01, 2.6401E+01, 1.8049E+03, &
      3.1495376E+00, 6.39561E+01, 2.8058E+01, 1.9136E+03, &
      3.1524272E+00, 6.45350E+01, 2.8018E+01, 1.9091E+03, &
      3.1652853E+00, 6.54239E+01, 2.7839E+01, 1.8894E+03, &
      3.1811117E+00, 6.59569E+01, 2.7623E+01, 1.8655E+03, &
      3.1970172E+00, 6.63394E+01, 2.7408E+01, 1.8419E+03, &
      3.2130023E+00, 6.66525E+01, 2.7195E+01, 1.8186E+03, &
      3.2290673E+00, 6.69232E+01, 2.6983E+01, 1.7955E+03, &
      3.2452127E+00, 6.71635E+01, 2.6773E+01, 1.7728E+03, &
      3.2614387E+00, 6.73798E+01, 2.6565E+01, 1.7504E+03, &
      3.2777459E+00, 6.75756E+01, 2.6358E+01, 1.7283E+03, &
      3.2941347E+00, 6.77524E+01, 2.6153E+01, 1.7064E+03, &
      3.3106053E+00, 6.79105E+01, 2.5953E+01, 1.6850E+03, &
      3.3271584E+00, 6.80521E+01, 2.5756E+01, 1.6640E+03, &
      3.3437941E+00, 6.81757E+01, 2.5562E+01, 1.6434E+03, &
      3.3605131E+00, 6.82767E+01, 2.5369E+01, 1.6230E+03, &
      3.3773157E+00, 6.83466E+01, 2.5179E+01, 1.6029E+03, &
      3.3942023E+00, 6.83634E+01, 2.4990E+01, 1.5830E+03, &
      3.4111733E+00, 6.82480E+01, 2.4802E+01, 1.5635E+03, &
      3.4177420E+00, 6.80844E+01, 2.4731E+01, 1.5560E+03, &
      3.4282291E+00, 6.79097E+01, 2.5703E+01, 1.6118E+03, &
      3.4320581E+00, 6.82092E+01, 2.5662E+01, 1.6075E+03, &
      3.4453703E+00, 6.86908E+01, 2.5518E+01, 1.5924E+03, &
      3.4625971E+00, 6.90500E+01, 2.5335E+01, 1.5732E+03, &
      3.4799101E+00, 6.93243E+01, 2.5153E+01, 1.5542E+03, &
      3.4973097E+00, 6.95588E+01, 2.4973E+01, 1.5355E+03, &
      3.5147962E+00, 6.97692E+01, 2.4795E+01, 1.5171E+03, &
      3.5323702E+00, 6.99630E+01, 2.4618E+01, 1.4989E+03, &
      3.5500321E+00, 7.01445E+01, 2.4443E+01, 1.4809E+03, &
      3.5677822E+00, 7.03163E+01, 2.4269E+01, 1.4632E+03, &
      3.5856211E+00, 7.04792E+01, 2.4094E+01, 1.4455E+03, &
      3.6035492E+00, 7.06344E+01, 2.3920E+01, 1.4280E+03, &
      3.6215670E+00, 7.07828E+01, 2.3748E+01, 1.4107E+03, &
      3.6396748E+00, 7.09251E+01, 2.3577E+01, 1.3937E+03, &
      3.6578732E+00, 7.10619E+01, 2.3407E+01, 1.3769E+03, &
      3.6761626E+00, 7.11939E+01, 2.3239E+01, 1.3603E+03, &
      3.6945434E+00, 7.13213E+01, 2.3073E+01, 1.3440E+03, &
      3.7130161E+00, 7.14445E+01, 2.2908E+01, 1.3278E+03, &
      3.7315812E+00, 7.15639E+01, 2.2744E+01, 1.3119E+03, &
      3.7502391E+00, 7.16797E+01, 2.2582E+01, 1.2961E+03, &
      3.7689903E+00, 7.17921E+01, 2.2421E+01, 1.2806E+03, &
      3.7878352E+00, 7.19013E+01, 2.2262E+01, 1.2653E+03, &
      3.8067744E+00, 7.20075E+01, 2.2104E+01, 1.2501E+03, &
      3.8258083E+00, 7.21108E+01, 2.1947E+01, 1.2352E+03, &
      3.8449373E+00, 7.22114E+01, 2.1792E+01, 1.2204E+03, &
      3.8641620E+00, 7.23095E+01, 2.1638E+01, 1.2059E+03, &
      3.8834828E+00, 7.24051E+01, 2.1485E+01, 1.1915E+03, &
      3.9029002E+00, 7.24984E+01, 2.1333E+01, 1.1773E+03, &
      3.9224147E+00, 7.25895E+01, 2.1183E+01, 1.1633E+03, &
      3.9420268E+00, 7.26785E+01, 2.1034E+01, 1.1494E+03, &
      3.9617369E+00, 7.27655E+01, 2.0886E+01, 1.1358E+03, &
      3.9815456E+00, 7.28506E+01, 2.0740E+01, 1.1223E+03, &
      4.0014533E+00, 7.29338E+01, 2.0595E+01, 1.1090E+03, &
      4.0214606E+00, 7.30154E+01, 2.0451E+01, 1.0958E+03, &
      4.0415679E+00, 7.30954E+01, 2.0308E+01, 1.0829E+03, &
      4.0617757E+00, 7.31740E+01, 2.0166E+01, 1.0700E+03, &
      4.0820846E+00, 7.32512E+01, 2.0025E+01, 1.0574E+03, &
      4.1024950E+00, 7.33271E+01, 1.9884E+01, 1.0448E+03, &
      4.1230075E+00, 7.34010E+01, 1.9743E+01, 1.0323E+03, &
      4.1436226E+00, 7.34730E+01, 1.9603E+01, 1.0200E+03, &
      4.1643407E+00, 7.35433E+01, 1.9464E+01, 1.0078E+03, &
      4.1851624E+00, 7.36120E+01, 1.9326E+01, 9.9578E+02, &
      4.2060882E+00, 7.36791E+01, 1.9190E+01, 9.8390E+02, &
      4.2271186E+00, 7.37447E+01, 1.9054E+01, 9.7217E+02, &
      4.2482542E+00, 7.38090E+01, 1.8919E+01, 9.6059E+02, &
      4.2694955E+00, 7.38719E+01, 1.8786E+01, 9.4915E+02, &
      4.2908430E+00, 7.39335E+01, 1.8653E+01, 9.3785E+02, &
      4.3122972E+00, 7.39939E+01, 1.8522E+01, 9.2669E+02, &
      4.3338587E+00, 7.40532E+01, 1.8391E+01, 9.1567E+02, &
      4.3555280E+00, 7.41114E+01, 1.8262E+01, 9.0478E+02, &
      4.3773056E+00, 7.41685E+01, 1.8133E+01, 8.9403E+02, &
      4.3991921E+00, 7.42247E+01, 1.8006E+01, 8.8341E+02, &
      4.4211881E+00, 7.44747E+01, 1.7876E+01, 8.7275E+02, &
      4.4432940E+00, 7.45292E+01, 1.7742E+01, 8.6202E+02, &
      4.4655105E+00, 7.45820E+01, 1.7610E+01, 8.5140E+02, &
      4.4878381E+00, 7.46330E+01, 1.7478E+01, 8.4091E+02, &
      4.5102772E+00, 7.46823E+01, 1.7347E+01, 8.3056E+02, &
      4.5328286E+00, 7.47301E+01, 1.7218E+01, 8.2034E+02, &
      4.5554928E+00, 7.47765E+01, 1.7089E+01, 8.1025E+02, &
      4.5782702E+00, 7.48214E+01, 1.6962E+01, 8.0029E+02, &
      4.6011616E+00, 7.49982E+01, 1.6831E+01, 7.9027E+02, &
      4.6241674E+00, 7.50404E+01, 1.6701E+01, 7.8034E+02, &
      4.6472882E+00, 7.50809E+01, 1.6572E+01, 7.7054E+02, &
      4.6705247E+00, 7.51197E+01, 1.6444E+01, 7.6087E+02, &
      4.6938773E+00, 7.51570E+01, 1.6317E+01, 7.5134E+02, &
      4.7173467E+00, 7.51929E+01, 1.6191E+01, 7.4193E+02, &
      4.7409334E+00, 7.52273E+01, 1.6067E+01, 7.3264E+02, &
      4.7646381E+00, 7.52604E+01, 1.5943E+01, 7.2348E+02, &
      4.7884613E+00, 7.52922E+01, 1.5821E+01, 7.1444E+02, &
      4.8124036E+00, 7.53228E+01, 1.5700E+01, 7.0552E+02, &
      4.8364656E+00, 7.53522E+01, 1.5580E+01, 6.9672E+02, &
      4.8606479E+00, 7.53804E+01, 1.5460E+01, 6.8804E+02, &
      4.8849512E+00, 7.54076E+01, 1.5342E+01, 6.7947E+02, &
      4.9093759E+00, 7.54337E+01, 1.5225E+01, 6.7101E+02, &
      4.9339228E+00, 7.54588E+01, 1.5109E+01, 6.6267E+02, &
      4.9585924E+00, 7.54830E+01, 1.4994E+01, 6.5444E+02, &
      4.9833854E+00, 7.55061E+01, 1.4881E+01, 6.4631E+02, &
      5.0083023E+00, 7.55284E+01, 1.4768E+01, 6.3830E+02, &
      5.0333438E+00, 7.55498E+01, 1.4656E+01, 6.3039E+02, &
      5.0585105E+00, 7.55703E+01, 1.4545E+01, 6.2258E+02, &
      5.0838031E+00, 7.55900E+01, 1.4435E+01, 6.1488E+02, &
      5.1092221E+00, 7.56090E+01, 1.4326E+01, 6.0728E+02, &
      5.1347682E+00, 7.56271E+01, 1.4218E+01, 5.9978E+02, &
      5.1604421E+00, 7.56445E+01, 1.4111E+01, 5.9238E+02, &
      5.1862443E+00, 7.56612E+01, 1.4005E+01, 5.8508E+02, &
      5.2121755E+00, 7.56772E+01, 1.3899E+01, 5.7787E+02, &
      5.2382364E+00, 7.56925E+01, 1.3795E+01, 5.7076E+02, &
      5.2644276E+00, 7.57071E+01, 1.3692E+01, 5.6374E+02, &
      5.2907497E+00, 7.57212E+01, 1.3589E+01, 5.5681E+02, &
      5.3172034E+00, 7.57346E+01, 1.3488E+01, 5.4998E+02, &
      5.3437895E+00, 7.57475E+01, 1.3387E+01, 5.4324E+02, &
      5.3705084E+00, 7.57598E+01, 1.3288E+01, 5.3658E+02, &
      5.3973609E+00, 7.57716E+01, 1.3189E+01, 5.3001E+02, &
      5.4243477E+00, 7.57828E+01, 1.3091E+01, 5.2353E+02, &
      5.4514695E+00, 7.57936E+01, 1.2994E+01, 5.1713E+02, &
      5.4787268E+00, 7.58040E+01, 1.2897E+01, 5.1082E+02, &
      5.5061205E+00, 7.59134E+01, 1.2799E+01, 5.0448E+02, &
      5.5336511E+00, 7.59232E+01, 1.2701E+01, 4.9819E+02, &
      5.5613193E+00, 7.59322E+01, 1.2603E+01, 4.9198E+02, &
      5.5891259E+00, 7.59404E+01, 1.2507E+01, 4.8585E+02, &
      5.6170716E+00, 7.59478E+01, 1.2411E+01, 4.7980E+02, &
      5.6451569E+00, 7.59546E+01, 1.2316E+01, 4.7384E+02, &
      5.6733827E+00, 7.59606E+01, 1.2222E+01, 4.6795E+02, &
      5.7017496E+00, 7.59660E+01, 1.2129E+01, 4.6214E+02, &
      5.7302584E+00, 7.59708E+01, 1.2036E+01, 4.5641E+02, &
      5.7589096E+00, 7.59750E+01, 1.1945E+01, 4.5075E+02, &
      5.7877042E+00, 7.59785E+01, 1.1854E+01, 4.4517E+02, &
      5.8166427E+00, 7.59816E+01, 1.1764E+01, 4.3966E+02, &
      5.8457259E+00, 7.59841E+01, 1.1675E+01, 4.3423E+02, &
      5.8749546E+00, 7.59860E+01, 1.1586E+01, 4.2887E+02, &
      5.9043293E+00, 7.59875E+01, 1.1499E+01, 4.2357E+02, &
      5.9338510E+00, 7.59884E+01, 1.1412E+01, 4.1835E+02, &
      5.9635202E+00, 7.59890E+01, 1.1326E+01, 4.1320E+02, &
      5.9933378E+00, 7.59890E+01, 1.1240E+01, 4.0811E+02, &
      6.0233045E+00, 7.59886E+01, 1.1156E+01, 4.0310E+02, &
      6.0534210E+00, 7.59878E+01, 1.1072E+01, 3.9814E+02, &
      6.0836882E+00, 7.59865E+01, 1.0989E+01, 3.9326E+02, &
      6.1141066E+00, 7.59849E+01, 1.0907E+01, 3.8843E+02, &
      6.1446771E+00, 7.59829E+01, 1.0825E+01, 3.8367E+02, &
      6.1754005E+00, 7.59805E+01, 1.0744E+01, 3.7898E+02, &
      6.2062775E+00, 7.59777E+01, 1.0664E+01, 3.7434E+02, &
      6.2373089E+00, 7.59746E+01, 1.0584E+01, 3.6977E+02, &
      6.2684954E+00, 7.59712E+01, 1.0506E+01, 3.6525E+02, &
      6.2998379E+00, 7.60088E+01, 1.0427E+01, 3.6077E+02, &
      6.3313371E+00, 7.60049E+01, 1.0347E+01, 3.5629E+02, &
      6.3629938E+00, 7.60006E+01, 1.0268E+01, 3.5188E+02, &
      6.3948088E+00, 7.59959E+01, 1.0190E+01, 3.4753E+02, &
      6.4267828E+00, 7.59906E+01, 1.0112E+01, 3.4323E+02, &
      6.4589167E+00, 7.59849E+01, 1.0035E+01, 3.3899E+02, &
      6.4912113E+00, 7.59788E+01, 9.9592E+00, 3.3480E+02, &
      6.5236674E+00, 7.59722E+01, 9.8836E+00, 3.3067E+02, &
      6.5562857E+00, 7.59653E+01, 9.8088E+00, 3.2660E+02, &
      6.5890671E+00, 7.59579E+01, 9.7346E+00, 3.2257E+02, &
      6.6220125E+00, 7.59502E+01, 9.6608E+00, 3.1860E+02, &
      6.6551225E+00, 7.59421E+01, 9.5874E+00, 3.1467E+02, &
      6.6883981E+00, 7.59337E+01, 9.5146E+00, 3.1078E+02, &
      6.7218401E+00, 7.59248E+01, 9.4424E+00, 3.0695E+02, &
      6.7554493E+00, 7.59156E+01, 9.3709E+00, 3.0317E+02, &
      6.7892266E+00, 7.59061E+01, 9.3000E+00, 2.9944E+02, &
      6.8231727E+00, 7.58961E+01, 9.2297E+00, 2.9576E+02, &
      6.8572886E+00, 7.59023E+01, 9.1600E+00, 2.9212E+02, &
      6.8915750E+00, 7.58919E+01, 9.0904E+00, 2.8852E+02, &
      6.9260329E+00, 7.58810E+01, 9.0214E+00, 2.8496E+02, &
      6.9606631E+00, 7.58698E+01, 8.9530E+00, 2.8145E+02, &
      6.9954664E+00, 7.58583E+01, 8.8852E+00, 2.7799E+02, &
      7.0304437E+00, 7.58463E+01, 8.8181E+00, 2.7457E+02, &
      7.0655959E+00, 7.58340E+01, 8.7515E+00, 2.7120E+02, &
      7.1009239E+00, 7.58214E+01, 8.6855E+00, 2.6787E+02, &
      7.1364285E+00, 7.58084E+01, 8.6200E+00, 2.6459E+02, &
      7.1721107E+00, 7.57951E+01, 8.5551E+00, 2.6135E+02, &
      7.2079712E+00, 7.57815E+01, 8.4908E+00, 2.5815E+02, &
      7.2440111E+00, 7.57676E+01, 8.4271E+00, 2.5499E+02, &
      7.2802311E+00, 7.57533E+01, 8.3639E+00, 2.5187E+02, &
      7.3166323E+00, 7.57387E+01, 8.3012E+00, 2.4880E+02, &
      7.3532155E+00, 7.57238E+01, 8.2391E+00, 2.4576E+02, &
      7.3899815E+00, 7.57086E+01, 8.1776E+00, 2.4277E+02, &
      7.4269314E+00, 7.56931E+01, 8.1165E+00, 2.3981E+02, &
      7.4640661E+00, 7.56773E+01, 8.0560E+00, 2.3689E+02, &
      7.5013864E+00, 7.56611E+01, 7.9961E+00, 2.3401E+02, &
      7.5388934E+00, 7.56447E+01, 7.9366E+00, 2.3117E+02, &
      7.5765878E+00, 7.56280E+01, 7.8776E+00, 2.2836E+02, &
      7.6144708E+00, 7.56110E+01, 7.8192E+00, 2.2559E+02, &
      7.6525431E+00, 7.55937E+01, 7.7613E+00, 2.2286E+02, &
      7.6908058E+00, 7.55761E+01, 7.7038E+00, 2.2016E+02, &
      7.7292599E+00, 7.55582E+01, 7.6469E+00, 2.1749E+02, &
      7.7679062E+00, 7.55400E+01, 7.5904E+00, 2.1486E+02, &
      7.8067457E+00, 7.55215E+01, 7.5344E+00, 2.1227E+02, &
      7.8457794E+00, 7.55028E+01, 7.4789E+00, 2.0970E+02, &
      7.8850083E+00, 7.54837E+01, 7.4239E+00, 2.0718E+02, &
      7.9244334E+00, 7.54644E+01, 7.3693E+00, 2.0468E+02, &
      7.9640555E+00, 7.54447E+01, 7.3152E+00, 2.0221E+02, &
      8.0038758E+00, 7.54248E+01, 7.2616E+00, 1.9978E+02, &
      8.0438952E+00, 7.54046E+01, 7.2084E+00, 1.9738E+02, &
      8.0841147E+00, 7.53841E+01, 7.1557E+00, 1.9501E+02, &
      8.1245352E+00, 7.53633E+01, 7.1035E+00, 1.9267E+02, &
      8.1651579E+00, 7.53422E+01, 7.0516E+00, 1.9036E+02, &
      8.2059837E+00, 7.53208E+01, 7.0002E+00, 1.8808E+02, &
      8.2470136E+00, 7.52992E+01, 6.9493E+00, 1.8583E+02, &
      8.2882487E+00, 7.52772E+01, 6.8988E+00, 1.8360E+02, &
      8.3296899E+00, 7.52550E+01, 6.8487E+00, 1.8141E+02, &
      8.3713384E+00, 7.52325E+01, 6.7990E+00, 1.7924E+02, &
      8.4131951E+00, 7.52097E+01, 6.7498E+00, 1.7711E+02, &
      8.4552610E+00, 7.51867E+01, 6.7009E+00, 1.7499E+02, &
      8.4975373E+00, 7.51634E+01, 6.6525E+00, 1.7291E+02, &
      8.5400250E+00, 7.51399E+01, 6.6045E+00, 1.7085E+02, &
      9.030794E+00, 7.49382E+01, 6.0726E+00, 1.4906E+02, &
      9.653919E+00, 7.44523E+01, 5.4359E+00, 1.2539E+02, &
      1.032004E+01, 7.37229E+01, 4.8556E+00, 1.0530E+02, &
      1.103212E+01, 7.25078E+01, 4.3394E+00, 8.8517E+01, &
      1.168033E+01, 6.98967E+01, 3.9480E+00, 7.6448E+01, &
      1.179334E+01, 6.86327E+01, 3.8861E+00, 7.4595E+01, &
      1.185911E+01, 6.71655E+01, 3.8509E+00, 7.3547E+01, &
      1.190678E+01, 6.39647E+01, 3.8253E+00, 7.2792E+01, &
      1.197829E+01, 6.71528E+01, 1.0042E+01, 1.8324E+02, &
      1.215707E+01, 6.99622E+01, 9.7808E+00, 1.7595E+02, &
      1.260708E+01, 7.18985E+01, 9.1691E+00, 1.5930E+02, &
      1.345893E+01, 7.20537E+01, 8.1755E+00, 1.3343E+02, &
      1.347697E+01, 7.20042E+01, 8.1564E+00, 1.3296E+02, &
      1.366493E+01, 7.08490E+01, 7.9615E+00, 1.2807E+02, &
      1.371987E+01, 6.92676E+01, 7.9054E+00, 1.2669E+02, &
      1.380227E+01, 7.09132E+01, 1.0981E+01, 1.7354E+02, &
      1.400827E+01, 7.23549E+01, 1.0704E+01, 1.6676E+02, &
      1.406574E+01, 7.25220E+01, 1.0629E+01, 1.6494E+02, &
      1.428104E+01, 7.25469E+01, 1.0355E+01, 1.5835E+02, &
      1.433845E+01, 7.18746E+01, 1.0284E+01, 1.5665E+02, &
      1.440688E+01, 7.27129E+01, 1.1821E+01, 1.7869E+02, &
      1.442456E+01, 7.29020E+01, 1.1798E+01, 1.7814E+02, &
      1.463986E+01, 7.40777E+01, 1.1530E+01, 1.7162E+02, &
      1.540095E+01, 7.58488E+01, 1.0657E+01, 1.5100E+02, &
      1.646362E+01, 7.70993E+01, 9.5849E+00, 1.2732E+02, &
      1.759961E+01, 7.78742E+01, 8.6190E+00, 1.0735E+02, &
      1.881398E+01, 7.83937E+01, 7.7427E+00, 9.0437E+01, &
      2.011215E+01, 7.87403E+01, 6.9510E+00, 7.6160E+01, &
      2.149988E+01, 7.89707E+01, 6.2427E+00, 6.4175E+01, &
      2.298338E+01, 7.91216E+01, 5.6095E+00, 5.4116E+01, &
      2.456923E+01, 7.92758E+01, 5.0398E+00, 4.5639E+01, &
      2.626450E+01, 7.93291E+01, 4.5271E+00, 3.8493E+01, &
      2.807676E+01, 7.94002E+01, 4.0394E+00, 3.2267E+01, &
      3.001405E+01, 7.94096E+01, 3.5892E+00, 2.6952E+01, &
      3.208502E+01, 7.93581E+01, 3.1891E+00, 2.2522E+01, &
      3.429889E+01, 7.92868E+01, 2.8358E+00, 1.8842E+01, &
      3.666551E+01, 7.92030E+01, 2.5237E+00, 1.5783E+01, &
      3.919543E+01, 7.91108E+01, 2.2477E+00, 1.3238E+01, &
      4.189992E+01, 7.90127E+01, 2.0036E+00, 1.1117E+01, &
      4.479101E+01, 7.89100E+01, 1.7874E+00, 9.3493E+00, &
      4.788159E+01, 7.88030E+01, 1.5957E+00, 7.8732E+00, &
      5.118542E+01, 7.86909E+01, 1.4256E+00, 6.6388E+00, &
      5.471721E+01, 7.85707E+01, 1.2719E+00, 5.5957E+00, &
      5.849270E+01, 7.84367E+01, 1.1355E+00, 4.7234E+00, &
      6.252870E+01, 7.82813E+01, 1.0126E+00, 3.9870E+00, &
      6.684318E+01, 7.80842E+01, 8.9999E-01, 3.3597E+00, &
      7.145536E+01, 7.78014E+01, 8.0018E-01, 2.8356E+00, &
      7.638578E+01, 7.72704E+01, 7.1163E-01, 2.3973E+00, &
      7.911040E+01, 7.65488E+01, 6.6924E-01, 2.1966E+00, &
      8.032127E+01, 7.54855E+01, 6.5138E-01, 2.1146E+00, &
      8.064417E+01, 7.42123E+01, 6.4651E-01, 2.0927E+00, &
      8.112852E+01, 7.55407E+01, 3.1459E+00, 8.6614E+00, &
      8.165640E+01, 7.62370E+01, 3.1142E+00, 8.5221E+00, &
      8.233940E+01, 7.67032E+01, 3.0740E+00, 8.3462E+00, &
      8.729069E+01, 7.79164E+01, 2.8038E+00, 7.2069E+00, &
      9.331374E+01, 7.84584E+01, 2.5207E+00, 6.0889E+00, &
      9.975239E+01, 7.87653E+01, 2.2620E+00, 5.1383E+00, &
      1.066353E+02, 7.89609E+01, 2.0288E+00, 4.3367E+00, &
      1.139931E+02, 7.90922E+01, 1.8193E+00, 3.6619E+00, &
      1.218587E+02, 7.91810E+01, 1.6277E+00, 3.0883E+00, &
      1.302669E+02, 7.92389E+01, 1.4560E+00, 2.6066E+00, &
      1.392553E+02, 7.92755E+01, 1.3024E+00, 2.2022E+00, &
      1.488640E+02, 7.92969E+01, 1.1649E+00, 1.8630E+00, &
      1.591356E+02, 7.93074E+01, 1.0421E+00, 1.5784E+00, &
      1.701159E+02, 7.93102E+01, 9.3205E-01, 1.3393E+00, &
      1.818539E+02, 7.93069E+01, 8.3281E-01, 1.1375E+00, &
      1.944018E+02, 7.92990E+01, 7.4421E-01, 9.6814E-01, &
      2.078156E+02, 7.92880E+01, 6.6511E-01, 8.2599E-01, &
      2.221548E+02, 7.92750E+01, 5.9448E-01, 7.0659E-01, &
      2.374835E+02, 7.92607E+01, 5.3140E-01, 6.0621E-01, &
      2.538699E+02, 7.92457E+01, 4.7505E-01, 5.2175E-01, &
      2.713869E+02, 7.92304E+01, 4.2472E-01, 4.5060E-01, &
      2.901126E+02, 7.92152E+01, 3.7976E-01, 3.9062E-01, &
      3.101304E+02, 7.92002E+01, 3.3958E-01, 3.3997E-01, &
      3.315294E+02, 7.91857E+01, 3.0368E-01, 2.9715E-01, &
      3.544049E+02, 7.91718E+01, 2.7160E-01, 2.6087E-01, &
      3.788588E+02, 7.91585E+01, 2.4292E-01, 2.3010E-01, &
      4.050001E+02, 7.91459E+01, 2.1729E-01, 2.0391E-01, &
      4.329451E+02, 7.91340E+01, 1.9438E-01, 1.8160E-01]
   case(80)
      frel = -1.0458E+00
      allocate(nist_tab(3068))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.272000E-03, 2.49501E+00, 0.0000E+00, 1.8668E-05, &
      6.368000E-03, 1.42832E+00, 0.0000E+00, 2.0206E-05, &
      6.393600E-03, 2.00589E-01, 0.0000E+00, 2.0632E-05, &
      6.432000E-03, 1.44193E+00, 2.3873E+00, 7.7863E+04, &
      6.528000E-03, 2.48802E+00, 2.3365E+00, 7.5086E+04, &
      7.559334E-03, 3.10590E+00, 1.9852E+00, 5.5092E+04, &
      7.675038E-03, 2.74244E+00, 1.9863E+00, 5.4292E+04, &
      7.705893E-03, 2.33399E+00, 1.9879E+00, 5.4117E+04, &
      7.752174E-03, 2.73258E+00, 2.7770E+00, 7.5150E+04, &
      7.867878E-03, 3.05471E+00, 2.7791E+00, 7.4100E+04, &
      1.069000E-02, 2.45759E+00, 4.1468E+00, 8.1377E+04, &
      1.142761E-02, 2.16853E+00, 4.8442E+00, 8.8928E+04, &
      1.221612E-02, 1.93957E+00, 5.7226E+00, 9.8272E+04, &
      1.305903E-02, 1.82849E+00, 6.7781E+00, 1.0889E+05, &
      1.396010E-02, 1.89424E+00, 7.9896E+00, 1.2006E+05, &
      1.492335E-02, 2.19117E+00, 9.3163E+00, 1.3096E+05, &
      1.595306E-02, 2.76201E+00, 1.0699E+01, 1.4069E+05, &
      1.705382E-02, 3.63080E+00, 1.2063E+01, 1.4839E+05, &
      1.823053E-02, 4.79730E+00, 1.3327E+01, 1.5335E+05, &
      1.948844E-02, 6.38775E+00, 1.4409E+01, 1.5510E+05, &
      2.083314E-02, 8.04416E+00, 1.5239E+01, 1.5345E+05, &
      2.227063E-02, 9.84804E+00, 1.5767E+01, 1.4852E+05, &
      2.380730E-02, 1.17169E+01, 1.5967E+01, 1.4070E+05, &
      2.545001E-02, 1.35693E+01, 1.5841E+01, 1.3057E+05, &
      2.720606E-02, 1.53289E+01, 1.5400E+01, 1.1875E+05, &
      2.908327E-02, 1.69150E+01, 1.4680E+01, 1.0589E+05, &
      3.109002E-02, 1.82727E+01, 1.3750E+01, 9.2780E+04, &
      3.323523E-02, 1.93757E+01, 1.2683E+01, 8.0058E+04, &
      3.552846E-02, 2.02196E+01, 1.1547E+01, 6.8179E+04, &
      3.797993E-02, 2.08155E+01, 1.0396E+01, 5.7425E+04, &
      4.060054E-02, 2.11830E+01, 9.2771E+00, 4.7935E+04, &
      4.340198E-02, 2.13436E+01, 8.2200E+00, 3.9731E+04, &
      4.639671E-02, 2.13123E+01, 7.2453E+00, 3.2760E+04, &
      4.959809E-02, 2.10827E+01, 6.3634E+00, 2.6915E+04, &
      5.302035E-02, 2.05739E+01, 5.5774E+00, 2.2068E+04, &
      5.644800E-02, 1.92513E+01, 4.9250E+00, 1.8303E+04, &
      5.667876E-02, 1.90360E+01, 4.8852E+00, 1.8082E+04, &
      5.731200E-02, 1.79046E+01, 4.7787E+00, 1.7492E+04, &
      5.754240E-02, 1.63099E+01, 4.7409E+00, 1.7284E+04, &
      5.788800E-02, 1.79732E+01, 7.8113E+00, 2.8308E+04, &
      5.875200E-02, 1.94028E+01, 7.5739E+00, 2.7044E+04, &
      6.058959E-02, 2.03418E+01, 7.1042E+00, 2.4597E+04, &
      6.477028E-02, 2.09820E+01, 6.1885E+00, 2.0044E+04, &
      6.923942E-02, 2.10514E+01, 5.3921E+00, 1.6337E+04, &
      7.401695E-02, 2.07897E+01, 4.7069E+00, 1.3341E+04, &
      7.889000E-02, 2.00015E+01, 4.1481E+00, 1.1030E+04, &
      7.912411E-02, 1.99183E+01, 4.1242E+00, 1.0935E+04, &
      8.009750E-02, 1.92918E+01, 4.0276E+00, 1.0549E+04, &
      8.041950E-02, 1.84995E+01, 3.9966E+00, 1.0426E+04, &
      8.090250E-02, 1.92719E+01, 5.4765E+00, 1.4201E+04, &
      8.211000E-02, 1.99056E+01, 5.3527E+00, 1.3676E+04, &
      8.458368E-02, 2.02531E+01, 5.1173E+00, 1.2692E+04, &
      9.041995E-02, 2.03692E+01, 4.6406E+00, 1.0767E+04, &
      9.653000E-02, 2.00752E+01, 4.1848E+00, 9.0950E+03, &
      9.665893E-02, 2.00607E+01, 4.1755E+00, 9.0628E+03, &
      9.800750E-02, 1.98095E+01, 4.0799E+00, 8.7334E+03, &
      9.840150E-02, 1.95494E+01, 4.0524E+00, 8.6398E+03, &
      9.899250E-02, 1.97596E+01, 4.4825E+00, 9.4998E+03, &
      1.001560E-01, 1.98640E+01, 4.4029E+00, 9.2227E+03, &
      1.004700E-01, 1.98637E+01, 4.3818E+00, 9.1497E+03, &
      1.016890E-01, 1.97713E+01, 4.3008E+00, 8.8729E+03, &
      1.020978E-01, 1.96098E+01, 4.2740E+00, 8.7825E+03, &
      1.027110E-01, 1.97752E+01, 4.5570E+00, 9.3079E+03, &
      1.033284E-01, 1.98528E+01, 4.5185E+00, 9.1741E+03, &
      1.042440E-01, 1.99021E+01, 4.4623E+00, 8.9807E+03, &
      1.104581E-01, 1.98318E+01, 4.1148E+00, 7.8154E+03, &
      1.178940E-01, 1.94244E+01, 3.8128E+00, 6.7853E+03, &
      1.180797E-01, 1.94106E+01, 3.8071E+00, 6.7645E+03, &
      1.196985E-01, 1.92669E+01, 3.7608E+00, 6.5920E+03, &
      1.201797E-01, 1.91823E+01, 3.7483E+00, 6.5438E+03, &
      1.209015E-01, 1.91886E+01, 3.8387E+00, 6.6616E+03, &
      1.227060E-01, 1.91141E+01, 3.8085E+00, 6.5119E+03, &
      1.262272E-01, 1.88983E+01, 3.7697E+00, 6.2659E+03, &
      1.349368E-01, 1.82729E+01, 3.7829E+00, 5.8822E+03, &
      1.442475E-01, 1.75385E+01, 3.9599E+00, 5.7601E+03, &
      1.542005E-01, 1.67188E+01, 4.3288E+00, 5.8904E+03, &
      1.648404E-01, 1.58389E+01, 4.9248E+00, 6.2689E+03, &
      1.762144E-01, 1.49425E+01, 5.7859E+00, 6.8897E+03, &
      1.883732E-01, 1.40719E+01, 6.9460E+00, 7.7373E+03, &
      2.013709E-01, 1.33063E+01, 8.4265E+00, 8.7805E+03, &
      2.152655E-01, 1.27330E+01, 1.0226E+01, 9.9681E+03, &
      2.301188E-01, 1.24451E+01, 1.2315E+01, 1.1228E+04, &
      2.459970E-01, 1.25328E+01, 1.4627E+01, 1.2476E+04, &
      2.629708E-01, 1.30524E+01, 1.7066E+01, 1.3617E+04, &
      2.811158E-01, 1.40382E+01, 1.9513E+01, 1.4565E+04, &
      3.005128E-01, 1.54764E+01, 2.1838E+01, 1.5249E+04, &
      3.212482E-01, 1.72942E+01, 2.3915E+01, 1.5621E+04, &
      3.434143E-01, 1.93207E+01, 2.5639E+01, 1.5666E+04, &
      3.526040E-01, 2.00658E+01, 2.6205E+01, 1.5595E+04, &
      3.580010E-01, 2.03115E+01, 2.6499E+01, 1.5533E+04, &
      3.594402E-01, 2.01357E+01, 2.6573E+01, 1.5514E+04, &
      3.615990E-01, 2.06264E+01, 2.7286E+01, 1.5835E+04, &
      3.671099E-01, 2.13794E+01, 2.7585E+01, 1.5768E+04, &
      3.707340E-01, 2.17622E+01, 2.7768E+01, 1.5718E+04, &
      3.764085E-01, 2.22111E+01, 2.8033E+01, 1.5628E+04, &
      3.779217E-01, 2.21524E+01, 2.8099E+01, 1.5603E+04, &
      3.801915E-01, 2.26131E+01, 2.8637E+01, 1.5806E+04, &
      3.858660E-01, 2.34096E+01, 2.8891E+01, 1.5712E+04, &
      3.924405E-01, 2.41932E+01, 2.9159E+01, 1.5592E+04, &
      4.195189E-01, 2.71474E+01, 2.9963E+01, 1.4988E+04, &
      4.484657E-01, 3.00221E+01, 3.0316E+01, 1.4187E+04, &
      4.794098E-01, 3.27346E+01, 3.0258E+01, 1.3247E+04, &
      5.0000000E-01, 3.55563E+01, 3.0041E+01, 1.2611E+04, &
      5.0250000E-01, 3.57355E+01, 3.0007E+01, 1.2534E+04, &
      5.0501250E-01, 3.59127E+01, 2.9972E+01, 1.2457E+04, &
      5.0753756E-01, 3.60877E+01, 2.9935E+01, 1.2380E+04, &
      5.1007525E-01, 3.62605E+01, 2.9896E+01, 1.2302E+04, &
      5.1262563E-01, 3.64309E+01, 2.9856E+01, 1.2224E+04, &
      5.1518875E-01, 3.65988E+01, 2.9814E+01, 1.2147E+04, &
      5.1776470E-01, 3.67641E+01, 2.9771E+01, 1.2069E+04, &
      5.2035352E-01, 3.69267E+01, 2.9726E+01, 1.1991E+04, &
      5.2295529E-01, 3.70863E+01, 2.9680E+01, 1.1913E+04, &
      5.2557007E-01, 3.72427E+01, 2.9632E+01, 1.1834E+04, &
      5.2819792E-01, 3.73958E+01, 2.9583E+01, 1.1756E+04, &
      5.3083891E-01, 3.75452E+01, 2.9533E+01, 1.1678E+04, &
      5.3349310E-01, 3.76906E+01, 2.9481E+01, 1.1600E+04, &
      5.3616057E-01, 3.78316E+01, 2.9428E+01, 1.1521E+04, &
      5.3884137E-01, 3.79677E+01, 2.9374E+01, 1.1443E+04, &
      5.4153558E-01, 3.80981E+01, 2.9319E+01, 1.1365E+04, &
      5.4424325E-01, 3.82222E+01, 2.9262E+01, 1.1286E+04, &
      5.4696447E-01, 3.83387E+01, 2.9204E+01, 1.1208E+04, &
      5.4969929E-01, 3.84461E+01, 2.9146E+01, 1.1130E+04, &
      5.5244779E-01, 3.85423E+01, 2.9085E+01, 1.1052E+04, &
      5.5521003E-01, 3.86240E+01, 2.9024E+01, 1.0974E+04, &
      5.5798608E-01, 3.86861E+01, 2.8962E+01, 1.0896E+04, &
      5.6077601E-01, 3.87198E+01, 2.8899E+01, 1.0818E+04, &
      5.6357989E-01, 3.87072E+01, 2.8835E+01, 1.0740E+04, &
      5.6639779E-01, 3.86046E+01, 2.8769E+01, 1.0663E+04, &
      5.6922978E-01, 3.82315E+01, 2.8703E+01, 1.0585E+04, &
      5.7028052E-01, 3.77853E+01, 2.8678E+01, 1.0557E+04, &
      5.7171944E-01, 3.78670E+01, 3.0411E+01, 1.1166E+04, &
      5.7207593E-01, 3.81133E+01, 3.0402E+01, 1.1156E+04, &
      5.7493630E-01, 3.90033E+01, 3.0336E+01, 1.1076E+04, &
      5.7781099E-01, 3.94715E+01, 3.0270E+01, 1.0997E+04, &
      5.8070004E-01, 3.98291E+01, 3.0202E+01, 1.0918E+04, &
      5.8360354E-01, 4.01339E+01, 3.0133E+01, 1.0839E+04, &
      5.8652156E-01, 4.04073E+01, 3.0064E+01, 1.0760E+04, &
      5.8945417E-01, 4.06595E+01, 2.9994E+01, 1.0682E+04, &
      5.9240144E-01, 4.08963E+01, 2.9923E+01, 1.0604E+04, &
      5.9536345E-01, 4.11213E+01, 2.9851E+01, 1.0526E+04, &
      5.9834026E-01, 4.13368E+01, 2.9779E+01, 1.0448E+04, &
      6.0133196E-01, 4.15444E+01, 2.9705E+01, 1.0371E+04, &
      6.0433862E-01, 4.17452E+01, 2.9632E+01, 1.0293E+04, &
      6.0736032E-01, 4.19402E+01, 2.9557E+01, 1.0217E+04, &
      6.1039712E-01, 4.21300E+01, 2.9482E+01, 1.0140E+04, &
      6.1344910E-01, 4.23150E+01, 2.9406E+01, 1.0064E+04, &
      6.1651635E-01, 4.24957E+01, 2.9329E+01, 9.9876E+03, &
      6.1959893E-01, 4.26724E+01, 2.9252E+01, 9.9119E+03, &
      6.2269693E-01, 4.28453E+01, 2.9175E+01, 9.8365E+03, &
      6.2581041E-01, 4.30146E+01, 2.9097E+01, 9.7615E+03, &
      6.2893946E-01, 4.31805E+01, 2.9018E+01, 9.6867E+03, &
      6.3208416E-01, 4.33430E+01, 2.8939E+01, 9.6123E+03, &
      6.3524458E-01, 4.35022E+01, 2.8859E+01, 9.5382E+03, &
      6.3842080E-01, 4.36582E+01, 2.8779E+01, 9.4644E+03, &
      6.4161291E-01, 4.38108E+01, 2.8698E+01, 9.3910E+03, &
      6.4482097E-01, 4.39600E+01, 2.8617E+01, 9.3179E+03, &
      6.4804508E-01, 4.41056E+01, 2.8535E+01, 9.2452E+03, &
      6.5128530E-01, 4.42475E+01, 2.8453E+01, 9.1728E+03, &
      6.5454173E-01, 4.43851E+01, 2.8370E+01, 9.1008E+03, &
      6.5781444E-01, 4.45179E+01, 2.8287E+01, 9.0291E+03, &
      6.6110351E-01, 4.46448E+01, 2.8204E+01, 8.9578E+03, &
      6.6440903E-01, 4.47640E+01, 2.8120E+01, 8.8868E+03, &
      6.6773107E-01, 4.48721E+01, 2.8036E+01, 8.8163E+03, &
      6.7106973E-01, 4.49604E+01, 2.7952E+01, 8.7461E+03, &
      6.7442508E-01, 4.49967E+01, 2.7867E+01, 8.6763E+03, &
      6.7594560E-01, 4.49425E+01, 2.7828E+01, 8.6449E+03, &
      6.7779720E-01, 4.50110E+01, 2.8169E+01, 8.7268E+03, &
      6.7785446E-01, 4.50209E+01, 2.8168E+01, 8.7256E+03, &
      6.8118619E-01, 4.53418E+01, 2.8086E+01, 8.6577E+03, &
      6.8459212E-01, 4.55509E+01, 2.8002E+01, 8.5890E+03, &
      6.8801508E-01, 4.57317E+01, 2.7918E+01, 8.5207E+03, &
      6.9145515E-01, 4.58990E+01, 2.7833E+01, 8.4528E+03, &
      6.9491243E-01, 4.60577E+01, 2.7749E+01, 8.3852E+03, &
      6.9838699E-01, 4.62104E+01, 2.7664E+01, 8.3181E+03, &
      7.0187893E-01, 4.63584E+01, 2.7579E+01, 8.2513E+03, &
      7.0538832E-01, 4.65024E+01, 2.7493E+01, 8.1849E+03, &
      7.0891526E-01, 4.66431E+01, 2.7408E+01, 8.1190E+03, &
      7.1245984E-01, 4.67807E+01, 2.7322E+01, 8.0534E+03, &
      7.1602214E-01, 4.69156E+01, 2.7236E+01, 7.9882E+03, &
      7.1960225E-01, 4.70479E+01, 2.7150E+01, 7.9235E+03, &
      7.2320026E-01, 4.71777E+01, 2.7064E+01, 7.8591E+03, &
      7.2681626E-01, 4.73052E+01, 2.6977E+01, 7.7951E+03, &
      7.3045034E-01, 4.74305E+01, 2.6891E+01, 7.7316E+03, &
      7.3410260E-01, 4.75535E+01, 2.6804E+01, 7.6684E+03, &
      7.3777311E-01, 4.76742E+01, 2.6717E+01, 7.6057E+03, &
      7.4146197E-01, 4.77928E+01, 2.6631E+01, 7.5433E+03, &
      7.4516928E-01, 4.79091E+01, 2.6544E+01, 7.4814E+03, &
      7.4889513E-01, 4.80231E+01, 2.6457E+01, 7.4198E+03, &
      7.5263961E-01, 4.81347E+01, 2.6369E+01, 7.3587E+03, &
      7.5640280E-01, 4.82437E+01, 2.6282E+01, 7.2980E+03, &
      7.6018482E-01, 4.83501E+01, 2.6195E+01, 7.2376E+03, &
      7.6398574E-01, 4.84535E+01, 2.6107E+01, 7.1777E+03, &
      7.6780567E-01, 4.85536E+01, 2.6020E+01, 7.1182E+03, &
      7.7164470E-01, 4.86500E+01, 2.5933E+01, 7.0590E+03, &
      7.7550292E-01, 4.87418E+01, 2.5845E+01, 7.0003E+03, &
      7.7938044E-01, 4.88280E+01, 2.5757E+01, 6.9420E+03, &
      7.8327734E-01, 4.89068E+01, 2.5670E+01, 6.8841E+03, &
      7.8719373E-01, 4.89748E+01, 2.5582E+01, 6.8265E+03, &
      7.9112969E-01, 4.90250E+01, 2.5495E+01, 6.7694E+03, &
      7.9508534E-01, 4.90382E+01, 2.5407E+01, 6.7127E+03, &
      7.9906077E-01, 4.88992E+01, 2.5319E+01, 6.6564E+03, &
      7.9906754E-01, 4.88984E+01, 2.5319E+01, 6.6563E+03, &
      8.0153246E-01, 4.89654E+01, 2.5809E+01, 6.7639E+03, &
      8.0305607E-01, 4.91457E+01, 2.5776E+01, 6.7425E+03, &
      8.0707135E-01, 4.94087E+01, 2.5689E+01, 6.6864E+03, &
      8.1110671E-01, 4.95963E+01, 2.5602E+01, 6.6308E+03, &
      8.1516224E-01, 4.97572E+01, 2.5515E+01, 6.5755E+03, &
      8.1923806E-01, 4.99039E+01, 2.5428E+01, 6.5207E+03, &
      8.2333425E-01, 5.00417E+01, 2.5342E+01, 6.4662E+03, &
      8.2745092E-01, 5.01731E+01, 2.5255E+01, 6.4121E+03, &
      8.3158817E-01, 5.02998E+01, 2.5168E+01, 6.3584E+03, &
      8.3574611E-01, 5.04226E+01, 2.5082E+01, 6.3051E+03, &
      8.3992484E-01, 5.05422E+01, 2.4995E+01, 6.2522E+03, &
      8.4412447E-01, 5.06591E+01, 2.4909E+01, 6.1997E+03, &
      8.4834509E-01, 5.07736E+01, 2.4822E+01, 6.1476E+03, &
      8.5258682E-01, 5.08860E+01, 2.4736E+01, 6.0958E+03, &
      8.5684975E-01, 5.09965E+01, 2.4650E+01, 6.0444E+03, &
      8.6113400E-01, 5.11052E+01, 2.4564E+01, 5.9935E+03, &
      8.6543967E-01, 5.12124E+01, 2.4478E+01, 5.9429E+03, &
      8.6976687E-01, 5.13180E+01, 2.4392E+01, 5.8927E+03, &
      8.7411570E-01, 5.14223E+01, 2.4306E+01, 5.8428E+03, &
      8.7848628E-01, 5.15253E+01, 2.4220E+01, 5.7933E+03, &
      8.8287871E-01, 5.16270E+01, 2.4135E+01, 5.7443E+03, &
      8.8729310E-01, 5.17276E+01, 2.4049E+01, 5.6955E+03, &
      8.9172957E-01, 5.18271E+01, 2.3964E+01, 5.6472E+03, &
      8.9618822E-01, 5.19256E+01, 2.3878E+01, 5.5992E+03, &
      9.0066916E-01, 5.20230E+01, 2.3793E+01, 5.5516E+03, &
      9.0517250E-01, 5.21194E+01, 2.3708E+01, 5.5043E+03, &
      9.0969837E-01, 5.22149E+01, 2.3623E+01, 5.4574E+03, &
      9.1424686E-01, 5.23095E+01, 2.3538E+01, 5.4109E+03, &
      9.1881809E-01, 5.24032E+01, 2.3454E+01, 5.3647E+03, &
      9.2341218E-01, 5.24961E+01, 2.3369E+01, 5.3188E+03, &
      9.2802924E-01, 5.25882E+01, 2.3285E+01, 5.2734E+03, &
      9.3266939E-01, 5.26796E+01, 2.3200E+01, 5.2283E+03, &
      9.3733274E-01, 5.27702E+01, 2.3116E+01, 5.1835E+03, &
      9.4201940E-01, 5.28601E+01, 2.3032E+01, 5.1391E+03, &
      9.4672950E-01, 5.29495E+01, 2.2948E+01, 5.0950E+03, &
      9.5146315E-01, 5.30382E+01, 2.2865E+01, 5.0513E+03, &
      9.5622046E-01, 5.31263E+01, 2.2781E+01, 5.0079E+03, &
      9.6100156E-01, 5.32139E+01, 2.2698E+01, 4.9649E+03, &
      9.6580657E-01, 5.33011E+01, 2.2615E+01, 4.9221E+03, &
      9.7063560E-01, 5.33877E+01, 2.2531E+01, 4.8796E+03, &
      9.7548878E-01, 5.34738E+01, 2.2447E+01, 4.8374E+03, &
      9.8036623E-01, 5.35595E+01, 2.2364E+01, 4.7956E+03, &
      9.8526806E-01, 5.36449E+01, 2.2281E+01, 4.7541E+03, &
      9.9019440E-01, 5.37300E+01, 2.2198E+01, 4.7129E+03, &
      9.9514537E-01, 5.38151E+01, 2.2115E+01, 4.6721E+03, &
      1.0001211E+00, 5.39003E+01, 2.2032E+01, 4.6314E+03, &
      1.0051217E+00, 5.39836E+01, 2.1918E+01, 4.5847E+03, &
      1.0101473E+00, 5.40644E+01, 2.1804E+01, 4.5384E+03, &
      1.0151980E+00, 5.41431E+01, 2.1692E+01, 4.4927E+03, &
      1.0202740E+00, 5.42195E+01, 2.1580E+01, 4.4473E+03, &
      1.0253754E+00, 5.42940E+01, 2.1468E+01, 4.4025E+03, &
      1.0305023E+00, 5.43665E+01, 2.1358E+01, 4.3581E+03, &
      1.0356548E+00, 5.44372E+01, 2.1248E+01, 4.3142E+03, &
      1.0408331E+00, 5.45060E+01, 2.1138E+01, 4.2707E+03, &
      1.0460372E+00, 5.45731E+01, 2.1029E+01, 4.2277E+03, &
      1.0512674E+00, 5.46385E+01, 2.0921E+01, 4.1851E+03, &
      1.0565238E+00, 5.47022E+01, 2.0813E+01, 4.1430E+03, &
      1.0618064E+00, 5.47643E+01, 2.0706E+01, 4.1012E+03, &
      1.0671154E+00, 5.48248E+01, 2.0599E+01, 4.0600E+03, &
      1.0724510E+00, 5.48839E+01, 2.0493E+01, 4.0191E+03, &
      1.0778132E+00, 5.49414E+01, 2.0388E+01, 3.9787E+03, &
      1.0832023E+00, 5.50167E+01, 2.0283E+01, 3.9386E+03, &
      1.0886183E+00, 5.50715E+01, 2.0178E+01, 3.8989E+03, &
      1.0940614E+00, 5.51249E+01, 2.0074E+01, 3.8597E+03, &
      1.0995317E+00, 5.51770E+01, 1.9971E+01, 3.8208E+03, &
      1.1050294E+00, 5.52278E+01, 1.9868E+01, 3.7823E+03, &
      1.1105545E+00, 5.52772E+01, 1.9766E+01, 3.7443E+03, &
      1.1161073E+00, 5.53254E+01, 1.9664E+01, 3.7066E+03, &
      1.1216878E+00, 5.53724E+01, 1.9563E+01, 3.6693E+03, &
      1.1272963E+00, 5.54182E+01, 1.9462E+01, 3.6324E+03, &
      1.1329328E+00, 5.54629E+01, 1.9362E+01, 3.5959E+03, &
      1.1385974E+00, 5.55107E+01, 1.9263E+01, 3.5598E+03, &
      1.1442904E+00, 5.55533E+01, 1.9164E+01, 3.5240E+03, &
      1.1500119E+00, 5.55945E+01, 1.9065E+01, 3.4884E+03, &
      1.1557619E+00, 5.56345E+01, 1.8966E+01, 3.4532E+03, &
      1.1615407E+00, 5.56733E+01, 1.8868E+01, 3.4184E+03, &
      1.1673484E+00, 5.57110E+01, 1.8770E+01, 3.3839E+03, &
      1.1731852E+00, 5.57474E+01, 1.8673E+01, 3.3498E+03, &
      1.1790511E+00, 5.57828E+01, 1.8577E+01, 3.3160E+03, &
      1.1849464E+00, 5.58170E+01, 1.8481E+01, 3.2826E+03, &
      1.1908711E+00, 5.58501E+01, 1.8385E+01, 3.2495E+03, &
      1.1968254E+00, 5.58821E+01, 1.8290E+01, 3.2168E+03, &
      1.2028096E+00, 5.59131E+01, 1.8196E+01, 3.1844E+03, &
      1.2088236E+00, 5.59430E+01, 1.8102E+01, 3.1523E+03, &
      1.2148677E+00, 5.59719E+01, 1.8009E+01, 3.1206E+03, &
      1.2209421E+00, 5.59997E+01, 1.7916E+01, 3.0892E+03, &
      1.2270468E+00, 5.60266E+01, 1.7824E+01, 3.0582E+03, &
      1.2331820E+00, 5.60524E+01, 1.7732E+01, 3.0274E+03, &
      1.2393479E+00, 5.60772E+01, 1.7641E+01, 2.9970E+03, &
      1.2455447E+00, 5.61011E+01, 1.7551E+01, 2.9669E+03, &
      1.2517724E+00, 5.61240E+01, 1.7461E+01, 2.9371E+03, &
      1.2580312E+00, 5.61460E+01, 1.7371E+01, 2.9077E+03, &
      1.2643214E+00, 5.61670E+01, 1.7282E+01, 2.8785E+03, &
      1.2706430E+00, 5.61870E+01, 1.7194E+01, 2.8496E+03, &
      1.2769962E+00, 5.62061E+01, 1.7106E+01, 2.8211E+03, &
      1.2833812E+00, 5.62244E+01, 1.7018E+01, 2.7928E+03, &
      1.2897981E+00, 5.62417E+01, 1.6931E+01, 2.7648E+03, &
      1.2962471E+00, 5.62581E+01, 1.6845E+01, 2.7372E+03, &
      1.3027283E+00, 5.62736E+01, 1.6759E+01, 2.7098E+03, &
      1.3092420E+00, 5.62882E+01, 1.6674E+01, 2.6827E+03, &
      1.3157882E+00, 5.63020E+01, 1.6589E+01, 2.6559E+03, &
      1.3223671E+00, 5.63149E+01, 1.6505E+01, 2.6294E+03, &
      1.3289790E+00, 5.63269E+01, 1.6421E+01, 2.6031E+03, &
      1.3356239E+00, 5.63381E+01, 1.6337E+01, 2.5772E+03, &
      1.3423020E+00, 5.63485E+01, 1.6255E+01, 2.5515E+03, &
      1.3490135E+00, 5.63581E+01, 1.6172E+01, 2.5261E+03, &
      1.3557586E+00, 5.63668E+01, 1.6090E+01, 2.5009E+03, &
      1.3625374E+00, 5.63746E+01, 1.6008E+01, 2.4759E+03, &
      1.3693500E+00, 5.63815E+01, 1.5927E+01, 2.4511E+03, &
      1.3761968E+00, 5.63876E+01, 1.5846E+01, 2.4267E+03, &
      1.3830778E+00, 5.63927E+01, 1.5766E+01, 2.4025E+03, &
      1.3899932E+00, 5.63970E+01, 1.5686E+01, 2.3785E+03, &
      1.3969431E+00, 5.64005E+01, 1.5607E+01, 2.3549E+03, &
      1.4039278E+00, 5.64031E+01, 1.5528E+01, 2.3314E+03, &
      1.4109475E+00, 5.64049E+01, 1.5449E+01, 2.3082E+03, &
      1.4180022E+00, 5.64059E+01, 1.5371E+01, 2.2853E+03, &
      1.4250922E+00, 5.64060E+01, 1.5294E+01, 2.2626E+03, &
      1.4322177E+00, 5.64054E+01, 1.5217E+01, 2.2401E+03, &
      1.4393788E+00, 5.64040E+01, 1.5140E+01, 2.2179E+03, &
      1.4465757E+00, 5.64015E+01, 1.5056E+01, 2.1947E+03, &
      1.4538086E+00, 5.63975E+01, 1.4972E+01, 2.1718E+03, &
      1.4610776E+00, 5.63921E+01, 1.4889E+01, 2.1491E+03, &
      1.4683830E+00, 5.63854E+01, 1.4807E+01, 2.1267E+03, &
      1.4757249E+00, 5.63773E+01, 1.4725E+01, 2.1045E+03, &
      1.4831035E+00, 5.63679E+01, 1.4644E+01, 2.0826E+03, &
      1.4905190E+00, 5.63571E+01, 1.4563E+01, 2.0610E+03, &
      1.4979716E+00, 5.63451E+01, 1.4483E+01, 2.0395E+03, &
      1.5054615E+00, 5.63317E+01, 1.4403E+01, 2.0184E+03, &
      1.5129888E+00, 5.63171E+01, 1.4324E+01, 1.9974E+03, &
      1.5205537E+00, 5.63009E+01, 1.4240E+01, 1.9760E+03, &
      1.5281565E+00, 5.62830E+01, 1.4157E+01, 1.9548E+03, &
      1.5357973E+00, 5.62635E+01, 1.4074E+01, 1.9338E+03, &
      1.5434763E+00, 5.62423E+01, 1.3992E+01, 1.9131E+03, &
      1.5511937E+00, 5.62193E+01, 1.3910E+01, 1.8926E+03, &
      1.5589496E+00, 5.61947E+01, 1.3830E+01, 1.8723E+03, &
      1.5667444E+00, 5.61684E+01, 1.3749E+01, 1.8523E+03, &
      1.5745781E+00, 5.61405E+01, 1.3669E+01, 1.8326E+03, &
      1.5824510E+00, 5.61108E+01, 1.3590E+01, 1.8130E+03, &
      1.5903633E+00, 5.60794E+01, 1.3512E+01, 1.7937E+03, &
      1.5983151E+00, 5.60463E+01, 1.3434E+01, 1.7746E+03, &
      1.6063066E+00, 5.60115E+01, 1.3356E+01, 1.7557E+03, &
      1.6143382E+00, 5.59750E+01, 1.3279E+01, 1.7370E+03, &
      1.6224099E+00, 5.59367E+01, 1.3203E+01, 1.7186E+03, &
      1.6305219E+00, 5.58966E+01, 1.3127E+01, 1.7003E+03, &
      1.6386745E+00, 5.58546E+01, 1.3052E+01, 1.6823E+03, &
      1.6468679E+00, 5.58109E+01, 1.2977E+01, 1.6644E+03, &
      1.6551022E+00, 5.57653E+01, 1.2903E+01, 1.6468E+03, &
      1.6633777E+00, 5.57178E+01, 1.2829E+01, 1.6294E+03, &
      1.6716946E+00, 5.56683E+01, 1.2756E+01, 1.6122E+03, &
      1.6800531E+00, 5.56169E+01, 1.2683E+01, 1.5951E+03, &
      1.6884534E+00, 5.55635E+01, 1.2611E+01, 1.5783E+03, &
      1.6968956E+00, 5.55080E+01, 1.2540E+01, 1.5617E+03, &
      1.7053801E+00, 5.54504E+01, 1.2469E+01, 1.5452E+03, &
      1.7139070E+00, 5.53995E+01, 1.2398E+01, 1.5289E+03, &
      1.7224766E+00, 5.53376E+01, 1.2328E+01, 1.5129E+03, &
      1.7310889E+00, 5.52734E+01, 1.2258E+01, 1.4970E+03, &
      1.7397444E+00, 5.52068E+01, 1.2189E+01, 1.4812E+03, &
      1.7484431E+00, 5.51379E+01, 1.2121E+01, 1.4657E+03, &
      1.7571853E+00, 5.50665E+01, 1.2053E+01, 1.4503E+03, &
      1.7659712E+00, 5.49926E+01, 1.1985E+01, 1.4352E+03, &
      1.7748011E+00, 5.49160E+01, 1.1918E+01, 1.4201E+03, &
      1.7836751E+00, 5.48367E+01, 1.1851E+01, 1.4053E+03, &
      1.7925935E+00, 5.47546E+01, 1.1785E+01, 1.3906E+03, &
      1.8015565E+00, 5.46696E+01, 1.1719E+01, 1.3761E+03, &
      1.8105642E+00, 5.45816E+01, 1.1654E+01, 1.3617E+03, &
      1.8196171E+00, 5.44904E+01, 1.1589E+01, 1.3476E+03, &
      1.8287151E+00, 5.43960E+01, 1.1525E+01, 1.3335E+03, &
      1.8378587E+00, 5.42982E+01, 1.1461E+01, 1.3197E+03, &
      1.8470480E+00, 5.41969E+01, 1.1397E+01, 1.3059E+03, &
      1.8562833E+00, 5.40920E+01, 1.1331E+01, 1.2920E+03, &
      1.8655647E+00, 5.39831E+01, 1.1265E+01, 1.2782E+03, &
      1.8748925E+00, 5.38700E+01, 1.1199E+01, 1.2645E+03, &
      1.8842670E+00, 5.37525E+01, 1.1133E+01, 1.2509E+03, &
      1.8936883E+00, 5.36305E+01, 1.1068E+01, 1.2375E+03, &
      1.9031567E+00, 5.35037E+01, 1.1003E+01, 1.2243E+03, &
      1.9126725E+00, 5.33719E+01, 1.0939E+01, 1.2112E+03, &
      1.9222359E+00, 5.32348E+01, 1.0875E+01, 1.1982E+03, &
      1.9318471E+00, 5.30921E+01, 1.0808E+01, 1.1851E+03, &
      1.9415063E+00, 5.29435E+01, 1.0743E+01, 1.1722E+03, &
      1.9512138E+00, 5.27886E+01, 1.0678E+01, 1.1594E+03, &
      1.9609699E+00, 5.26270E+01, 1.0613E+01, 1.1468E+03, &
      1.9707747E+00, 5.24584E+01, 1.0549E+01, 1.1344E+03, &
      1.9806286E+00, 5.22823E+01, 1.0486E+01, 1.1221E+03, &
      1.9905318E+00, 5.20982E+01, 1.0423E+01, 1.1099E+03, &
      2.0004844E+00, 5.19057E+01, 1.0361E+01, 1.0979E+03, &
      2.0104868E+00, 5.17041E+01, 1.0299E+01, 1.0861E+03, &
      2.0205393E+00, 5.14927E+01, 1.0238E+01, 1.0744E+03, &
      2.0306420E+00, 5.12709E+01, 1.0177E+01, 1.0628E+03, &
      2.0407952E+00, 5.10388E+01, 1.0117E+01, 1.0514E+03, &
      2.0509992E+00, 5.07935E+01, 1.0057E+01, 1.0401E+03, &
      2.0612542E+00, 5.05350E+01, 9.9978E+00, 1.0289E+03, &
      2.0715604E+00, 5.02621E+01, 9.9390E+00, 1.0179E+03, &
      2.0819182E+00, 4.99733E+01, 9.8808E+00, 1.0070E+03, &
      2.0923278E+00, 4.96672E+01, 9.8230E+00, 9.9626E+02, &
      2.1027895E+00, 4.93418E+01, 9.7657E+00, 9.8564E+02, &
      2.1133034E+00, 4.89950E+01, 9.7088E+00, 9.7515E+02, &
      2.1238699E+00, 4.86243E+01, 9.6524E+00, 9.6478E+02, &
      2.1344893E+00, 4.82267E+01, 9.5965E+00, 9.5453E+02, &
      2.1451617E+00, 4.77985E+01, 9.5411E+00, 9.4441E+02, &
      2.1558875E+00, 4.73353E+01, 9.4861E+00, 9.3441E+02, &
      2.1666670E+00, 4.68315E+01, 9.4315E+00, 9.2454E+02, &
      2.1775003E+00, 4.62802E+01, 9.3774E+00, 9.1477E+02, &
      2.1883878E+00, 4.56724E+01, 9.3238E+00, 9.0513E+02, &
      2.1993297E+00, 4.49963E+01, 9.2706E+00, 8.9560E+02, &
      2.2103264E+00, 4.42358E+01, 9.2178E+00, 8.8618E+02, &
      2.2213780E+00, 4.33681E+01, 9.1654E+00, 8.7688E+02, &
      2.2324849E+00, 4.23593E+01, 9.1135E+00, 8.6768E+02, &
      2.2436473E+00, 4.11558E+01, 9.0619E+00, 8.5860E+02, &
      2.2548656E+00, 3.96644E+01, 9.0108E+00, 8.4962E+02, &
      2.2661399E+00, 3.76989E+01, 8.9601E+00, 8.4075E+02, &
      2.2774706E+00, 3.47858E+01, 8.9098E+00, 8.3198E+02, &
      2.2888579E+00, 2.88037E+01, 8.8590E+00, 8.2323E+02, &
      2.2942849E+00, 1.62146E+01, 8.8346E+00, 8.1907E+02, &
      2.2955150E+00, 1.59129E+01, 2.6029E+01, 2.3900E+03, &
      2.3003022E+00, 2.77369E+01, 2.5949E+01, 2.3777E+03, &
      2.3118037E+00, 3.36852E+01, 2.5758E+01, 2.3486E+03, &
      2.3233628E+00, 3.61133E+01, 2.5568E+01, 2.3199E+03, &
      2.3349796E+00, 3.74118E+01, 2.5380E+01, 2.2915E+03, &
      2.3466545E+00, 3.80194E+01, 2.5194E+01, 2.2635E+03, &
      2.3583878E+00, 3.79796E+01, 2.5009E+01, 2.2358E+03, &
      2.3701797E+00, 3.69517E+01, 2.4826E+01, 2.2085E+03, &
      2.3820306E+00, 3.20152E+01, 2.4644E+01, 2.1815E+03, &
      2.3842276E+00, 2.69354E+01, 2.4610E+01, 2.1766E+03, &
      2.3855726E+00, 2.68547E+01, 3.5970E+01, 3.1743E+03, &
      2.3939407E+00, 3.68932E+01, 3.5781E+01, 3.1467E+03, &
      2.4059104E+00, 4.07295E+01, 3.5513E+01, 3.1077E+03, &
      2.4179400E+00, 4.30624E+01, 3.5247E+01, 3.0693E+03, &
      2.4300297E+00, 4.48169E+01, 3.4984E+01, 3.0313E+03, &
      2.4421798E+00, 4.62493E+01, 3.4722E+01, 2.9938E+03, &
      2.4543907E+00, 4.74704E+01, 3.4463E+01, 2.9568E+03, &
      2.4666627E+00, 4.85392E+01, 3.4206E+01, 2.9203E+03, &
      2.4789960E+00, 4.94914E+01, 3.3951E+01, 2.8842E+03, &
      2.4913910E+00, 5.03506E+01, 3.3698E+01, 2.8486E+03, &
      2.5038479E+00, 5.11332E+01, 3.3447E+01, 2.8134E+03, &
      2.5163672E+00, 5.18510E+01, 3.3198E+01, 2.7787E+03, &
      2.5289490E+00, 5.25131E+01, 3.2950E+01, 2.7444E+03, &
      2.5415938E+00, 5.31265E+01, 3.2705E+01, 2.7106E+03, &
      2.5543017E+00, 5.36965E+01, 3.2462E+01, 2.6771E+03, &
      2.5670732E+00, 5.42277E+01, 3.2221E+01, 2.6442E+03, &
      2.5799086E+00, 5.47234E+01, 3.1982E+01, 2.6116E+03, &
      2.5928082E+00, 5.51867E+01, 3.1744E+01, 2.5794E+03, &
      2.6057722E+00, 5.56197E+01, 3.1509E+01, 2.5477E+03, &
      2.6188011E+00, 5.60245E+01, 3.1275E+01, 2.5163E+03, &
      2.6318951E+00, 5.64023E+01, 3.1043E+01, 2.4854E+03, &
      2.6450545E+00, 5.67544E+01, 3.0813E+01, 2.4548E+03, &
      2.6582798E+00, 5.70814E+01, 3.0585E+01, 2.4246E+03, &
      2.6715712E+00, 5.73837E+01, 3.0359E+01, 2.3948E+03, &
      2.6849291E+00, 5.76613E+01, 3.0134E+01, 2.3654E+03, &
      2.6983537E+00, 5.79137E+01, 2.9911E+01, 2.3364E+03, &
      2.7118455E+00, 5.81399E+01, 2.9690E+01, 2.3077E+03, &
      2.7254047E+00, 5.83380E+01, 2.9470E+01, 2.2793E+03, &
      2.7390317E+00, 5.85052E+01, 2.9252E+01, 2.2513E+03, &
      2.7527269E+00, 5.86369E+01, 2.9035E+01, 2.2236E+03, &
      2.7664905E+00, 5.87262E+01, 2.8820E+01, 2.1963E+03, &
      2.7803230E+00, 5.87616E+01, 2.8607E+01, 2.1693E+03, &
      2.7942246E+00, 5.87228E+01, 2.8395E+01, 2.1427E+03, &
      2.8081957E+00, 5.85700E+01, 2.8185E+01, 2.1164E+03, &
      2.8222367E+00, 5.82069E+01, 2.7977E+01, 2.0904E+03, &
      2.8363479E+00, 5.72756E+01, 2.7771E+01, 2.0648E+03, &
      2.8443469E+00, 5.54687E+01, 2.7655E+01, 2.0504E+03, &
      2.8498532E+00, 5.55003E+01, 3.2096E+01, 2.3734E+03, &
      2.8505296E+00, 5.58297E+01, 3.2086E+01, 2.3721E+03, &
      2.8647823E+00, 5.84538E+01, 3.1866E+01, 2.3442E+03, &
      2.8791062E+00, 5.95579E+01, 3.1648E+01, 2.3167E+03, &
      2.8935017E+00, 6.03325E+01, 3.1432E+01, 2.2896E+03, &
      2.9079692E+00, 6.09542E+01, 3.1217E+01, 2.2627E+03, &
      2.9225091E+00, 6.14855E+01, 3.1003E+01, 2.2361E+03, &
      2.9371216E+00, 6.19557E+01, 3.0791E+01, 2.2099E+03, &
      2.9518072E+00, 6.23814E+01, 3.0580E+01, 2.1840E+03, &
      2.9665662E+00, 6.27725E+01, 3.0371E+01, 2.1584E+03, &
      2.9813991E+00, 6.31359E+01, 3.0164E+01, 2.1331E+03, &
      2.9963061E+00, 6.34754E+01, 2.9955E+01, 2.1079E+03, &
      3.0112876E+00, 6.38311E+01, 2.9729E+01, 2.0817E+03, &
      3.0263440E+00, 6.41655E+01, 2.9499E+01, 2.0554E+03, &
      3.0414758E+00, 6.44728E+01, 2.9271E+01, 2.0295E+03, &
      3.0566831E+00, 6.47575E+01, 2.9045E+01, 2.0039E+03, &
      3.0719666E+00, 6.50223E+01, 2.8820E+01, 1.9786E+03, &
      3.0873264E+00, 6.52691E+01, 2.8597E+01, 1.9537E+03, &
      3.1027630E+00, 6.54990E+01, 2.8376E+01, 1.9291E+03, &
      3.1182768E+00, 6.57117E+01, 2.8154E+01, 1.9046E+03, &
      3.1338682E+00, 6.59074E+01, 2.7934E+01, 1.8804E+03, &
      3.1495376E+00, 6.60857E+01, 2.7715E+01, 1.8565E+03, &
      3.1652853E+00, 6.62456E+01, 2.7499E+01, 1.8329E+03, &
      3.1811117E+00, 6.63851E+01, 2.7284E+01, 1.8097E+03, &
      3.1970172E+00, 6.65007E+01, 2.7070E+01, 1.7867E+03, &
      3.2130023E+00, 6.65860E+01, 2.6859E+01, 1.7640E+03, &
      3.2290673E+00, 6.66285E+01, 2.6648E+01, 1.7416E+03, &
      3.2452127E+00, 6.66002E+01, 2.6440E+01, 1.7195E+03, &
      3.2614387E+00, 6.64119E+01, 2.6233E+01, 1.6977E+03, &
      3.2736807E+00, 6.58359E+01, 2.6079E+01, 1.6815E+03, &
      3.2777459E+00, 6.48291E+01, 2.6028E+01, 1.6761E+03, &
      3.2833195E+00, 6.59237E+01, 2.7740E+01, 1.7827E+03, &
      3.2941347E+00, 6.67125E+01, 2.7594E+01, 1.7675E+03, &
      3.3106053E+00, 6.72914E+01, 2.7373E+01, 1.7448E+03, &
      3.3271584E+00, 6.76869E+01, 2.7155E+01, 1.7224E+03, &
      3.3437941E+00, 6.80037E+01, 2.6938E+01, 1.7003E+03, &
      3.3605131E+00, 6.82737E+01, 2.6723E+01, 1.6784E+03, &
      3.3773157E+00, 6.85110E+01, 2.6509E+01, 1.6568E+03, &
      3.3942023E+00, 6.87226E+01, 2.6298E+01, 1.6355E+03, &
      3.4111733E+00, 6.89123E+01, 2.6088E+01, 1.6145E+03, &
      3.4282291E+00, 6.90819E+01, 2.5880E+01, 1.5938E+03, &
      3.4453703E+00, 6.92312E+01, 2.5676E+01, 1.5735E+03, &
      3.4625971E+00, 6.93634E+01, 2.5480E+01, 1.5538E+03, &
      3.4799101E+00, 6.94775E+01, 2.5286E+01, 1.5344E+03, &
      3.4973097E+00, 6.95685E+01, 2.5094E+01, 1.5153E+03, &
      3.5147962E+00, 6.96263E+01, 2.4903E+01, 1.4964E+03, &
      3.5323702E+00, 6.96255E+01, 2.4715E+01, 1.4778E+03, &
      3.5500321E+00, 6.94622E+01, 2.4529E+01, 1.4595E+03, &
      3.5540850E+00, 6.93485E+01, 2.4487E+01, 1.4553E+03, &
      3.5677822E+00, 6.93913E+01, 2.5428E+01, 1.5051E+03, &
      3.5691149E+00, 6.94698E+01, 2.5414E+01, 1.5037E+03, &
      3.5856211E+00, 7.00065E+01, 2.5243E+01, 1.4869E+03, &
      3.6035492E+00, 7.03407E+01, 2.5060E+01, 1.4688E+03, &
      3.6215670E+00, 7.06015E+01, 2.4879E+01, 1.4511E+03, &
      3.6396748E+00, 7.08260E+01, 2.4700E+01, 1.4336E+03, &
      3.6578732E+00, 7.10280E+01, 2.4523E+01, 1.4163E+03, &
      3.6761626E+00, 7.12143E+01, 2.4347E+01, 1.3992E+03, &
      3.6945434E+00, 7.13889E+01, 2.4173E+01, 1.3824E+03, &
      3.7130161E+00, 7.15539E+01, 2.3998E+01, 1.3657E+03, &
      3.7315812E+00, 7.17099E+01, 2.3824E+01, 1.3491E+03, &
      3.7502391E+00, 7.18583E+01, 2.3651E+01, 1.3328E+03, &
      3.7689903E+00, 7.20003E+01, 2.3480E+01, 1.3167E+03, &
      3.7878352E+00, 7.21364E+01, 2.3311E+01, 1.3008E+03, &
      3.8067744E+00, 7.22673E+01, 2.3143E+01, 1.2851E+03, &
      3.8258083E+00, 7.23934E+01, 2.2977E+01, 1.2696E+03, &
      3.8449373E+00, 7.25153E+01, 2.2812E+01, 1.2543E+03, &
      3.8641620E+00, 7.26331E+01, 2.2649E+01, 1.2392E+03, &
      3.8834828E+00, 7.27473E+01, 2.2487E+01, 1.2244E+03, &
      3.9029002E+00, 7.28580E+01, 2.2327E+01, 1.2097E+03, &
      3.9224147E+00, 7.29654E+01, 2.2169E+01, 1.1952E+03, &
      3.9420268E+00, 7.30699E+01, 2.2012E+01, 1.1809E+03, &
      3.9617369E+00, 7.31714E+01, 2.1856E+01, 1.1668E+03, &
      3.9815456E+00, 7.32703E+01, 2.1701E+01, 1.1529E+03, &
      4.0014533E+00, 7.33666E+01, 2.1548E+01, 1.1392E+03, &
      4.0214606E+00, 7.34604E+01, 2.1397E+01, 1.1256E+03, &
      4.0415679E+00, 7.35520E+01, 2.1247E+01, 1.1123E+03, &
      4.0617757E+00, 7.36414E+01, 2.1098E+01, 1.0991E+03, &
      4.0820846E+00, 7.37287E+01, 2.0950E+01, 1.0860E+03, &
      4.1024950E+00, 7.38141E+01, 2.0803E+01, 1.0732E+03, &
      4.1230075E+00, 7.38976E+01, 2.0658E+01, 1.0605E+03, &
      4.1436226E+00, 7.39793E+01, 2.0514E+01, 1.0479E+03, &
      4.1643407E+00, 7.40594E+01, 2.0372E+01, 1.0355E+03, &
      4.1851624E+00, 7.41379E+01, 2.0230E+01, 1.0233E+03, &
      4.2060882E+00, 7.42151E+01, 2.0090E+01, 1.0113E+03, &
      4.2271186E+00, 7.42908E+01, 1.9951E+01, 9.9934E+02, &
      4.2482542E+00, 7.43655E+01, 1.9813E+01, 9.8758E+02, &
      4.2694955E+00, 7.44390E+01, 1.9675E+01, 9.7590E+02, &
      4.2908430E+00, 7.45107E+01, 1.9535E+01, 9.6427E+02, &
      4.3122972E+00, 7.45805E+01, 1.9397E+01, 9.5278E+02, &
      4.3338587E+00, 7.46485E+01, 1.9260E+01, 9.4143E+02, &
      4.3555280E+00, 7.47150E+01, 1.9125E+01, 9.3023E+02, &
      4.3773056E+00, 7.47799E+01, 1.8990E+01, 9.1916E+02, &
      4.3991921E+00, 7.48435E+01, 1.8856E+01, 9.0823E+02, &
      4.4211881E+00, 7.49056E+01, 1.8723E+01, 8.9743E+02, &
      4.4432940E+00, 7.49665E+01, 1.8592E+01, 8.8677E+02, &
      4.4655105E+00, 7.50261E+01, 1.8461E+01, 8.7624E+02, &
      4.4878381E+00, 7.50846E+01, 1.8331E+01, 8.6584E+02, &
      4.5102772E+00, 7.51420E+01, 1.8202E+01, 8.5557E+02, &
      4.5328286E+00, 7.51983E+01, 1.8075E+01, 8.4542E+02, &
      4.5554928E+00, 7.52537E+01, 1.7948E+01, 8.3540E+02, &
      4.5782702E+00, 7.53081E+01, 1.7822E+01, 8.2550E+02, &
      4.6011616E+00, 7.55526E+01, 1.7694E+01, 8.1555E+02, &
      4.6241674E+00, 7.56054E+01, 1.7562E+01, 8.0554E+02, &
      4.6472882E+00, 7.56565E+01, 1.7431E+01, 7.9564E+02, &
      4.6705247E+00, 7.57058E+01, 1.7301E+01, 7.8586E+02, &
      4.6938773E+00, 7.57535E+01, 1.7172E+01, 7.7621E+02, &
      4.7173467E+00, 7.57997E+01, 1.7044E+01, 7.6668E+02, &
      4.7409334E+00, 7.58445E+01, 1.6917E+01, 7.5727E+02, &
      4.7646381E+00, 7.58880E+01, 1.6792E+01, 7.4798E+02, &
      4.7884613E+00, 7.60591E+01, 1.6663E+01, 7.3865E+02, &
      4.8124036E+00, 7.60999E+01, 1.6535E+01, 7.2940E+02, &
      4.8364656E+00, 7.61390E+01, 1.6408E+01, 7.2027E+02, &
      4.8606479E+00, 7.61766E+01, 1.6281E+01, 7.1126E+02, &
      4.8849512E+00, 7.62126E+01, 1.6156E+01, 7.0237E+02, &
      4.9093759E+00, 7.62472E+01, 1.6032E+01, 6.9360E+02, &
      4.9339228E+00, 7.62804E+01, 1.5910E+01, 6.8494E+02, &
      4.9585924E+00, 7.63124E+01, 1.5788E+01, 6.7640E+02, &
      4.9833854E+00, 7.63431E+01, 1.5667E+01, 6.6797E+02, &
      5.0083023E+00, 7.63726E+01, 1.5547E+01, 6.5965E+02, &
      5.0333438E+00, 7.64009E+01, 1.5429E+01, 6.5144E+02, &
      5.0585105E+00, 7.64282E+01, 1.5311E+01, 6.4334E+02, &
      5.0838031E+00, 7.64544E+01, 1.5195E+01, 6.3535E+02, &
      5.1092221E+00, 7.64795E+01, 1.5079E+01, 6.2746E+02, &
      5.1347682E+00, 7.65037E+01, 1.4965E+01, 6.1967E+02, &
      5.1604421E+00, 7.65269E+01, 1.4851E+01, 6.1199E+02, &
      5.1862443E+00, 7.65492E+01, 1.4739E+01, 6.0441E+02, &
      5.2121755E+00, 7.65706E+01, 1.4627E+01, 5.9693E+02, &
      5.2382364E+00, 7.65911E+01, 1.4516E+01, 5.8955E+02, &
      5.2644276E+00, 7.66108E+01, 1.4407E+01, 5.8226E+02, &
      5.2907497E+00, 7.66297E+01, 1.4298E+01, 5.7507E+02, &
      5.3172034E+00, 7.66478E+01, 1.4190E+01, 5.6797E+02, &
      5.3437895E+00, 7.66652E+01, 1.4084E+01, 5.6097E+02, &
      5.3705084E+00, 7.66818E+01, 1.3978E+01, 5.5406E+02, &
      5.3973609E+00, 7.66977E+01, 1.3873E+01, 5.4724E+02, &
      5.4243477E+00, 7.67129E+01, 1.3769E+01, 5.4051E+02, &
      5.4514695E+00, 7.67275E+01, 1.3666E+01, 5.3387E+02, &
      5.4787268E+00, 7.67414E+01, 1.3564E+01, 5.2732E+02, &
      5.5061205E+00, 7.67548E+01, 1.3462E+01, 5.2085E+02, &
      5.5336511E+00, 7.67675E+01, 1.3362E+01, 5.1446E+02, &
      5.5613193E+00, 7.67797E+01, 1.3262E+01, 5.0816E+02, &
      5.5891259E+00, 7.67913E+01, 1.3164E+01, 5.0195E+02, &
      5.6170716E+00, 7.68024E+01, 1.3066E+01, 4.9581E+02, &
      5.6451569E+00, 7.68130E+01, 1.2969E+01, 4.8976E+02, &
      5.6733827E+00, 7.68231E+01, 1.2873E+01, 4.8378E+02, &
      5.7017496E+00, 7.69308E+01, 1.2777E+01, 4.7785E+02, &
      5.7302584E+00, 7.69406E+01, 1.2679E+01, 4.7189E+02, &
      5.7589096E+00, 7.69495E+01, 1.2582E+01, 4.6602E+02, &
      5.7877042E+00, 7.69576E+01, 1.2485E+01, 4.6022E+02, &
      5.8166427E+00, 7.69650E+01, 1.2390E+01, 4.5450E+02, &
      5.8457259E+00, 7.69716E+01, 1.2295E+01, 4.4886E+02, &
      5.8749546E+00, 7.69776E+01, 1.2201E+01, 4.4329E+02, &
      5.9043293E+00, 7.69829E+01, 1.2109E+01, 4.3779E+02, &
      5.9338510E+00, 7.69876E+01, 1.2016E+01, 4.3237E+02, &
      5.9635202E+00, 7.69916E+01, 1.1925E+01, 4.2702E+02, &
      5.9933378E+00, 7.69951E+01, 1.1835E+01, 4.2174E+02, &
      6.0233045E+00, 7.69980E+01, 1.1745E+01, 4.1653E+02, &
      6.0534210E+00, 7.70004E+01, 1.1656E+01, 4.1138E+02, &
      6.0836882E+00, 7.70022E+01, 1.1568E+01, 4.0631E+02, &
      6.1141066E+00, 7.70035E+01, 1.1480E+01, 4.0130E+02, &
      6.1446771E+00, 7.70044E+01, 1.1394E+01, 3.9636E+02, &
      6.1754005E+00, 7.70047E+01, 1.1308E+01, 3.9149E+02, &
      6.2062775E+00, 7.70046E+01, 1.1223E+01, 3.8667E+02, &
      6.2373089E+00, 7.70041E+01, 1.1139E+01, 3.8193E+02, &
      6.2684954E+00, 7.70031E+01, 1.1055E+01, 3.7724E+02, &
      6.2998379E+00, 7.70017E+01, 1.0972E+01, 3.7262E+02, &
      6.3313371E+00, 7.69999E+01, 1.0890E+01, 3.6805E+02, &
      6.3629938E+00, 7.69976E+01, 1.0809E+01, 3.6355E+02, &
      6.3948088E+00, 7.69951E+01, 1.0728E+01, 3.5910E+02, &
      6.4267828E+00, 7.69921E+01, 1.0648E+01, 3.5472E+02, &
      6.4589167E+00, 7.69888E+01, 1.0569E+01, 3.5039E+02, &
      6.4912113E+00, 7.69852E+01, 1.0491E+01, 3.4612E+02, &
      6.5236674E+00, 7.69812E+01, 1.0413E+01, 3.4190E+02, &
      6.5562857E+00, 7.69769E+01, 1.0336E+01, 3.3774E+02, &
      6.5890671E+00, 7.70119E+01, 1.0258E+01, 3.3360E+02, &
      6.6220125E+00, 7.70071E+01, 1.0181E+01, 3.2951E+02, &
      6.6551225E+00, 7.70019E+01, 1.0105E+01, 3.2547E+02, &
      6.6883981E+00, 7.69963E+01, 1.0029E+01, 3.2148E+02, &
      6.7218401E+00, 7.69903E+01, 9.9538E+00, 3.1755E+02, &
      6.7554493E+00, 7.69839E+01, 9.8794E+00, 3.1367E+02, &
      6.7892266E+00, 7.69772E+01, 9.8049E+00, 3.0981E+02, &
      6.8231727E+00, 7.69700E+01, 9.7310E+00, 3.0601E+02, &
      6.8572886E+00, 7.69625E+01, 9.6577E+00, 3.0225E+02, &
      6.8915750E+00, 7.69546E+01, 9.5851E+00, 2.9855E+02, &
      6.9260329E+00, 7.69463E+01, 9.5132E+00, 2.9489E+02, &
      6.9606631E+00, 7.69377E+01, 9.4418E+00, 2.9128E+02, &
      6.9954664E+00, 7.69288E+01, 9.3711E+00, 2.8772E+02, &
      7.0304437E+00, 7.69195E+01, 9.3010E+00, 2.8420E+02, &
      7.0655959E+00, 7.69099E+01, 9.2315E+00, 2.8073E+02, &
      7.1009239E+00, 7.68999E+01, 9.1626E+00, 2.7731E+02, &
      7.1364285E+00, 7.69064E+01, 9.0937E+00, 2.7391E+02, &
      7.1721107E+00, 7.68959E+01, 9.0250E+00, 2.7054E+02, &
      7.2079712E+00, 7.68851E+01, 8.9569E+00, 2.6722E+02, &
      7.2440111E+00, 7.68739E+01, 8.8894E+00, 2.6395E+02, &
      7.2802311E+00, 7.68624E+01, 8.8224E+00, 2.6071E+02, &
      7.3166323E+00, 7.68505E+01, 8.7561E+00, 2.5752E+02, &
      7.3532155E+00, 7.68383E+01, 8.6903E+00, 2.5437E+02, &
      7.3899815E+00, 7.68257E+01, 8.6252E+00, 2.5126E+02, &
      7.4269314E+00, 7.68128E+01, 8.5605E+00, 2.4819E+02, &
      7.4640661E+00, 7.67996E+01, 8.4965E+00, 2.4516E+02, &
      7.5013864E+00, 7.67860E+01, 8.4330E+00, 2.4217E+02, &
      7.5388934E+00, 7.67722E+01, 8.3700E+00, 2.3922E+02, &
      7.5765878E+00, 7.67580E+01, 8.3076E+00, 2.3631E+02, &
      7.6144708E+00, 7.67435E+01, 8.2458E+00, 2.3344E+02, &
      7.6525431E+00, 7.67287E+01, 8.1844E+00, 2.3060E+02, &
      7.6908058E+00, 7.67137E+01, 8.1236E+00, 2.2780E+02, &
      7.7292599E+00, 7.66983E+01, 8.0633E+00, 2.2503E+02, &
      7.7679062E+00, 7.66827E+01, 8.0036E+00, 2.2231E+02, &
      7.8067457E+00, 7.66667E+01, 7.9443E+00, 2.1961E+02, &
      7.8457794E+00, 7.66505E+01, 7.8856E+00, 2.1696E+02, &
      7.8850083E+00, 7.66340E+01, 7.8274E+00, 2.1433E+02, &
      7.9244334E+00, 7.66172E+01, 7.7697E+00, 2.1174E+02, &
      7.9640555E+00, 7.66001E+01, 7.7124E+00, 2.0919E+02, &
      8.0038758E+00, 7.65828E+01, 7.6557E+00, 2.0666E+02, &
      8.0438952E+00, 7.65652E+01, 7.5994E+00, 2.0417E+02, &
      8.0841147E+00, 7.65473E+01, 7.5436E+00, 2.0171E+02, &
      8.1245352E+00, 7.65292E+01, 7.4883E+00, 1.9929E+02, &
      8.1651579E+00, 7.65108E+01, 7.4335E+00, 1.9689E+02, &
      8.2059837E+00, 7.64921E+01, 7.3791E+00, 1.9453E+02, &
      8.2470136E+00, 7.64732E+01, 7.3252E+00, 1.9219E+02, &
      8.2882487E+00, 7.64541E+01, 7.2718E+00, 1.8989E+02, &
      8.3296899E+00, 7.64347E+01, 7.2188E+00, 1.8761E+02, &
      8.3713384E+00, 7.64151E+01, 7.1663E+00, 1.8537E+02, &
      8.4131951E+00, 7.63952E+01, 7.1142E+00, 1.8315E+02, &
      8.4552610E+00, 7.63752E+01, 7.0625E+00, 1.8096E+02, &
      8.4975373E+00, 7.63549E+01, 7.0113E+00, 1.7880E+02, &
      8.5400250E+00, 7.63345E+01, 6.9606E+00, 1.7667E+02, &
      9.030794E+00, 7.61838E+01, 6.3981E+00, 1.5408E+02, &
      9.653919E+00, 7.57597E+01, 5.7278E+00, 1.2960E+02, &
      1.032004E+01, 7.51472E+01, 5.1273E+00, 1.0905E+02, &
      1.103212E+01, 7.42114E+01, 4.5818E+00, 9.1652E+01, &
      1.179334E+01, 7.23323E+01, 4.1008E+00, 7.7181E+01, &
      1.203822E+01, 7.09760E+01, 3.9647E+00, 7.3240E+01, &
      1.222248E+01, 6.82600E+01, 3.8675E+00, 7.0466E+01, &
      1.227162E+01, 6.50762E+01, 3.8422E+00, 6.9752E+01, &
      1.234532E+01, 6.82479E+01, 1.0028E+01, 1.7445E+02, &
      1.252958E+01, 7.10452E+01, 9.7696E+00, 1.6755E+02, &
      1.260708E+01, 7.15833E+01, 9.6640E+00, 1.6477E+02, &
      1.347697E+01, 7.35382E+01, 8.5946E+00, 1.3747E+02, &
      1.392453E+01, 7.31608E+01, 8.1174E+00, 1.2587E+02, &
      1.413766E+01, 7.19530E+01, 7.9052E+00, 1.2081E+02, &
      1.419449E+01, 7.03784E+01, 7.8501E+00, 1.1951E+02, &
      1.427974E+01, 7.20100E+01, 1.0908E+01, 1.6371E+02, &
      1.440688E+01, 7.30797E+01, 1.0743E+01, 1.5988E+02, &
      1.449287E+01, 7.34317E+01, 1.0634E+01, 1.5734E+02, &
      1.454251E+01, 7.35707E+01, 1.0572E+01, 1.5591E+02, &
      1.476510E+01, 7.36042E+01, 1.0301E+01, 1.4971E+02, &
      1.482446E+01, 7.29374E+01, 1.0231E+01, 1.4810E+02, &
      1.491350E+01, 7.39597E+01, 1.1735E+01, 1.6838E+02, &
      1.513609E+01, 7.51292E+01, 1.1469E+01, 1.6222E+02, &
      1.540095E+01, 7.58937E+01, 1.1166E+01, 1.5529E+02, &
      1.646362E+01, 7.75917E+01, 1.0054E+01, 1.3107E+02, &
      1.759961E+01, 7.85498E+01, 9.0411E+00, 1.1052E+02, &
      1.881398E+01, 7.91746E+01, 8.1288E+00, 9.3178E+01, &
      2.011215E+01, 7.95942E+01, 7.3007E+00, 7.8496E+01, &
      2.149988E+01, 7.98742E+01, 6.5577E+00, 6.6149E+01, &
      2.298338E+01, 8.00602E+01, 5.8932E+00, 5.5783E+01, &
      2.456923E+01, 8.02412E+01, 5.2991E+00, 4.7078E+01, &
      2.626450E+01, 8.03201E+01, 4.7609E+00, 3.9710E+01, &
      2.807676E+01, 8.03721E+01, 4.2505E+00, 3.3304E+01, &
      3.001405E+01, 8.04278E+01, 3.7827E+00, 2.7856E+01, &
      3.208502E+01, 8.03867E+01, 3.3608E+00, 2.3272E+01, &
      3.429889E+01, 8.03222E+01, 2.9883E+00, 1.9465E+01, &
      3.666551E+01, 8.02429E+01, 2.6592E+00, 1.6302E+01, &
      3.919543E+01, 8.01540E+01, 2.3682E+00, 1.3669E+01, &
      4.189992E+01, 8.00584E+01, 2.1106E+00, 1.1476E+01, &
      4.479101E+01, 7.99578E+01, 1.8826E+00, 9.6481E+00, &
      4.788159E+01, 7.98530E+01, 1.6806E+00, 8.1222E+00, &
      5.118542E+01, 7.97436E+01, 1.5013E+00, 6.8468E+00, &
      5.471721E+01, 7.96279E+01, 1.3407E+00, 5.7744E+00, &
      5.849270E+01, 7.95009E+01, 1.1966E+00, 4.8717E+00, &
      6.252870E+01, 7.93567E+01, 1.0684E+00, 4.1153E+00, &
      6.684318E+01, 7.91821E+01, 9.5129E-01, 3.4718E+00, &
      7.145536E+01, 7.89465E+01, 8.4623E-01, 2.9302E+00, &
      7.638578E+01, 7.85660E+01, 7.5312E-01, 2.4776E+00, &
      8.144026E+01, 7.75656E+01, 6.7195E-01, 2.1083E+00, &
      8.165640E+01, 7.74618E+01, 6.6864E-01, 2.0940E+00, &
      8.268679E+01, 7.65087E+01, 6.5321E-01, 2.0275E+00, &
      8.301920E+01, 7.52427E+01, 6.4835E-01, 2.0068E+00, &
      8.351781E+01, 7.65634E+01, 3.1333E+00, 8.2361E+00, &
      8.476435E+01, 7.77188E+01, 3.0616E+00, 7.9362E+00, &
      8.729069E+01, 7.85120E+01, 2.9234E+00, 7.3720E+00, &
      9.331374E+01, 7.92696E+01, 2.6296E+00, 6.2311E+00, &
      9.975239E+01, 7.96510E+01, 2.3616E+00, 5.2615E+00, &
      1.066353E+02, 7.98848E+01, 2.1189E+00, 4.4416E+00, &
      1.139931E+02, 8.00388E+01, 1.9006E+00, 3.7511E+00, &
      1.218587E+02, 8.01439E+01, 1.7028E+00, 3.1668E+00, &
      1.302669E+02, 8.02131E+01, 1.5238E+00, 2.6734E+00, &
      1.392553E+02, 8.02578E+01, 1.3635E+00, 2.2589E+00, &
      1.488640E+02, 8.02851E+01, 1.2201E+00, 1.9111E+00, &
      1.591356E+02, 8.02998E+01, 1.0918E+00, 1.6192E+00, &
      1.701159E+02, 8.03056E+01, 9.7710E-01, 1.3740E+00, &
      1.818539E+02, 8.03049E+01, 8.7381E-01, 1.1674E+00, &
      1.944018E+02, 8.02990E+01, 7.8130E-01, 9.9363E-01, &
      2.078156E+02, 8.02894E+01, 6.9866E-01, 8.4771E-01, &
      2.221548E+02, 8.02774E+01, 6.2482E-01, 7.2508E-01, &
      2.374835E+02, 8.02639E+01, 5.5884E-01, 6.2196E-01, &
      2.538699E+02, 8.02494E+01, 4.9988E-01, 5.3515E-01, &
      2.713869E+02, 8.02345E+01, 4.4718E-01, 4.6201E-01, &
      2.901126E+02, 8.02195E+01, 4.0007E-01, 4.0031E-01, &
      3.101304E+02, 8.02046E+01, 3.5796E-01, 3.4820E-01, &
      3.315294E+02, 8.01901E+01, 3.2030E-01, 3.0414E-01, &
      3.544049E+02, 8.01761E+01, 2.8663E-01, 2.6679E-01, &
      3.788588E+02, 8.01627E+01, 2.5652E-01, 2.3510E-01, &
      4.050001E+02, 8.01500E+01, 2.2959E-01, 2.0814E-01, &
      4.329451E+02, 8.01380E+01, 2.0550E-01, 1.8515E-01]
   case(81)
      frel = -1.0794E+00
      allocate(nist_tab(3084))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      9.471535E-03, 2.89482E+00, 0.0000E+00, 1.4636E-04, &
      9.616508E-03, 2.43653E+00, 0.0000E+00, 1.5724E-04, &
      9.655167E-03, 1.93601E+00, 0.0000E+00, 1.6024E-04, &
      9.713156E-03, 2.41962E+00, 9.5738E-01, 2.0294E+04, &
      9.858128E-03, 2.80732E+00, 9.4093E-01, 1.9652E+04, &
      1.069000E-02, 3.01632E+00, 8.5572E-01, 1.6481E+04, &
      1.142761E-02, 2.81513E+00, 7.6359E-01, 1.3758E+04, &
      1.221612E-02, 2.31826E+00, 6.4656E-01, 1.0897E+04, &
      1.283800E-02, 1.41314E+00, 5.5653E-01, 8.9254E+03, &
      1.303450E-02, 4.79375E-01, 5.2967E-01, 8.3666E+03, &
      1.305903E-02, 1.70913E-01, 5.2639E-01, 8.2991E+03, &
      1.308690E-02, -5.76446E-01, 5.2267E-01, 8.2229E+03, &
      1.316550E-02, 4.89650E-01, 2.5504E+00, 3.9884E+04, &
      1.336200E-02, 1.35585E+00, 2.4444E+00, 3.7665E+04, &
      1.396010E-02, 1.81934E+00, 2.1538E+00, 3.1765E+04, &
      1.492335E-02, 1.26447E+00, 1.8083E+00, 2.4948E+04, &
      1.499400E-02, 1.16047E+00, 1.7938E+00, 2.4632E+04, &
      1.522350E-02, 5.44732E-01, 1.7565E+00, 2.3756E+04, &
      1.528470E-02, -9.42078E-02, 1.7489E+00, 2.3558E+04, &
      1.537650E-02, 4.91862E-01, 2.9353E+00, 3.9304E+04, &
      1.560600E-02, 9.17787E-01, 2.8917E+00, 3.8150E+04, &
      1.595306E-02, 1.01599E+00, 2.8481E+00, 3.6757E+04, &
      1.705382E-02, 6.15267E-01, 2.8623E+00, 3.4556E+04, &
      1.823053E-02, -2.11345E-01, 3.1946E+00, 3.6079E+04, &
      1.948844E-02, -1.22195E+00, 3.8982E+00, 4.1184E+04, &
      2.083314E-02, -2.31842E+00, 5.0236E+00, 4.9647E+04, &
      2.227063E-02, -3.38921E+00, 6.6455E+00, 6.1437E+04, &
      2.380730E-02, -4.27166E+00, 8.8266E+00, 7.6334E+04, &
      2.545001E-02, -4.74810E+00, 1.1575E+01, 9.3643E+04, &
      2.720606E-02, -4.56925E+00, 1.4806E+01, 1.1205E+05, &
      2.908327E-02, -3.50478E+00, 1.8312E+01, 1.2963E+05, &
      3.109002E-02, -1.40836E+00, 2.1773E+01, 1.4419E+05, &
      3.323523E-02, 1.72000E+00, 2.4800E+01, 1.5363E+05, &
      3.552846E-02, 5.70679E+00, 2.7017E+01, 1.5656E+05, &
      3.797993E-02, 1.02244E+01, 2.8143E+01, 1.5256E+05, &
      4.060054E-02, 1.48506E+01, 2.8061E+01, 1.4230E+05, &
      4.340198E-02, 1.91780E+01, 2.6836E+01, 1.2730E+05, &
      4.639671E-02, 2.28632E+01, 2.4683E+01, 1.0953E+05, &
      4.959809E-02, 2.56983E+01, 2.1909E+01, 9.0947E+04, &
      5.302035E-02, 2.76143E+01, 1.8838E+01, 7.3154E+04, &
      5.667876E-02, 2.86585E+01, 1.5762E+01, 5.7258E+04, &
      6.058959E-02, 2.89544E+01, 1.2896E+01, 4.3821E+04, &
      6.477028E-02, 2.86433E+01, 1.0368E+01, 3.2956E+04, &
      6.923942E-02, 2.78132E+01, 8.2343E+00, 2.4485E+04, &
      7.389200E-02, 2.61391E+01, 6.5313E+00, 1.8198E+04, &
      7.401695E-02, 2.60553E+01, 6.4917E+00, 1.8058E+04, &
      7.502300E-02, 2.48949E+01, 6.1837E+00, 1.6970E+04, &
      7.532460E-02, 2.35501E+01, 6.0949E+00, 1.6660E+04, &
      7.577700E-02, 2.48694E+01, 8.5263E+00, 2.3166E+04, &
      7.690800E-02, 2.59023E+01, 8.1044E+00, 2.1696E+04, &
      7.912411E-02, 2.63345E+01, 7.3530E+00, 1.9133E+04, &
      8.458368E-02, 2.61109E+01, 5.8588E+00, 1.4261E+04, &
      9.041995E-02, 2.53542E+01, 4.7070E+00, 1.0718E+04, &
      9.665893E-02, 2.41818E+01, 3.8281E+00, 8.1546E+03, &
      9.760800E-02, 2.39080E+01, 3.7182E+00, 7.8436E+03, &
      9.910200E-02, 2.31352E+01, 3.5558E+00, 7.3879E+03, &
      9.950040E-02, 2.23738E+01, 3.5146E+00, 7.2730E+03, &
      1.000980E-01, 2.30489E+01, 4.8603E+00, 9.9976E+03, &
      1.015920E-01, 2.35242E+01, 4.6850E+00, 9.4952E+03, &
      1.033284E-01, 2.36206E+01, 4.4954E+00, 8.9579E+03, &
      1.104581E-01, 2.32784E+01, 3.8498E+00, 7.1764E+03, &
      1.161300E-01, 2.26952E+01, 3.4384E+00, 6.0967E+03, &
      1.179075E-01, 2.23619E+01, 3.3227E+00, 5.8028E+03, &
      1.180797E-01, 2.23037E+01, 3.3118E+00, 5.7754E+03, &
      1.183815E-01, 2.21224E+01, 3.2929E+00, 5.7277E+03, &
      1.190925E-01, 2.22329E+01, 3.6236E+00, 6.2653E+03, &
      1.203440E-01, 2.22225E+01, 3.5593E+00, 6.0903E+03, &
      1.208700E-01, 2.21855E+01, 3.5332E+00, 6.0192E+03, &
      1.221860E-01, 2.20234E+01, 3.4700E+00, 5.8480E+03, &
      1.226772E-01, 2.18574E+01, 3.4473E+00, 5.7864E+03, &
      1.234140E-01, 2.19417E+01, 3.6815E+00, 6.1425E+03, &
      1.252560E-01, 2.19267E+01, 3.6150E+00, 5.9429E+03, &
      1.262272E-01, 2.18818E+01, 3.5824E+00, 5.8440E+03, &
      1.335740E-01, 2.13523E+01, 3.3886E+00, 5.2240E+03, &
      1.349368E-01, 2.12226E+01, 3.3636E+00, 5.1332E+03, &
      1.356185E-01, 2.11444E+01, 3.3524E+00, 5.0904E+03, &
      1.361637E-01, 2.10446E+01, 3.3441E+00, 5.0576E+03, &
      1.369815E-01, 2.10336E+01, 3.4424E+00, 5.1752E+03, &
      1.390260E-01, 2.09125E+01, 3.4222E+00, 5.0691E+03, &
      1.442475E-01, 2.05035E+01, 3.4079E+00, 4.8653E+03, &
      1.542005E-01, 1.96476E+01, 3.5226E+00, 4.7047E+03, &
      1.648404E-01, 1.87015E+01, 3.8405E+00, 4.7983E+03, &
      1.762144E-01, 1.76979E+01, 4.3944E+00, 5.1360E+03, &
      1.883732E-01, 1.66756E+01, 5.2206E+00, 5.7079E+03, &
      2.013709E-01, 1.56886E+01, 6.3526E+00, 6.4972E+03, &
      2.152655E-01, 1.48088E+01, 7.8117E+00, 7.4737E+03, &
      2.301188E-01, 1.41292E+01, 9.5974E+00, 8.5893E+03, &
      2.459970E-01, 1.37290E+01, 1.1679E+01, 9.7773E+03, &
      2.629708E-01, 1.37031E+01, 1.4004E+01, 1.0967E+04, &
      2.811158E-01, 1.41194E+01, 1.6464E+01, 1.2061E+04, &
      3.005128E-01, 1.50075E+01, 1.8944E+01, 1.2983E+04, &
      3.212482E-01, 1.63534E+01, 2.1308E+01, 1.3660E+04, &
      3.434143E-01, 1.80811E+01, 2.3426E+01, 1.4049E+04, &
      3.671099E-01, 2.00316E+01, 2.5189E+01, 1.4132E+04, &
      3.784760E-01, 2.08453E+01, 2.5854E+01, 1.4070E+04, &
      3.842690E-01, 2.10185E+01, 2.6150E+01, 1.4016E+04, &
      3.858138E-01, 2.07765E+01, 2.6224E+01, 1.3999E+04, &
      3.881310E-01, 2.13155E+01, 2.7056E+01, 1.4357E+04, &
      3.924405E-01, 2.19191E+01, 2.7282E+01, 1.4318E+04, &
      3.939240E-01, 2.20824E+01, 2.7357E+01, 1.4304E+04, &
      3.984680E-01, 2.25239E+01, 2.7576E+01, 1.4254E+04, &
      4.045670E-01, 2.29375E+01, 2.7848E+01, 1.4177E+04, &
      4.061934E-01, 2.28421E+01, 2.7916E+01, 1.4155E+04, &
      4.086330E-01, 2.33320E+01, 2.8524E+01, 1.4377E+04, &
      4.147320E-01, 2.41478E+01, 2.8789E+01, 1.4297E+04, &
      4.195189E-01, 2.46859E+01, 2.8983E+01, 1.4229E+04, &
      4.484657E-01, 2.76204E+01, 2.9895E+01, 1.3731E+04, &
      4.794098E-01, 3.04932E+01, 3.0414E+01, 1.3068E+04, &
      5.0000000E-01, 3.37172E+01, 3.0527E+01, 1.2577E+04, &
      5.0250000E-01, 3.39239E+01, 3.0529E+01, 1.2515E+04, &
      5.0501250E-01, 3.41295E+01, 3.0528E+01, 1.2453E+04, &
      5.0753756E-01, 3.43340E+01, 3.0526E+01, 1.2390E+04, &
      5.1007525E-01, 3.45374E+01, 3.0520E+01, 1.2326E+04, &
      5.1262563E-01, 3.47395E+01, 3.0513E+01, 1.2262E+04, &
      5.1518875E-01, 3.49403E+01, 3.0503E+01, 1.2197E+04, &
      5.1776470E-01, 3.51398E+01, 3.0492E+01, 1.2132E+04, &
      5.2035352E-01, 3.53378E+01, 3.0478E+01, 1.2066E+04, &
      5.2295529E-01, 3.55342E+01, 3.0461E+01, 1.1999E+04, &
      5.2557007E-01, 3.57291E+01, 3.0443E+01, 1.1933E+04, &
      5.2819792E-01, 3.59224E+01, 3.0423E+01, 1.1865E+04, &
      5.3083891E-01, 3.61139E+01, 3.0400E+01, 1.1798E+04, &
      5.3349310E-01, 3.63036E+01, 3.0376E+01, 1.1730E+04, &
      5.3616057E-01, 3.64913E+01, 3.0350E+01, 1.1661E+04, &
      5.3884137E-01, 3.66771E+01, 3.0321E+01, 1.1593E+04, &
      5.4153558E-01, 3.68607E+01, 3.0291E+01, 1.1523E+04, &
      5.4424325E-01, 3.70422E+01, 3.0259E+01, 1.1454E+04, &
      5.4696447E-01, 3.72213E+01, 3.0225E+01, 1.1384E+04, &
      5.4969929E-01, 3.73979E+01, 3.0189E+01, 1.1314E+04, &
      5.5244779E-01, 3.75720E+01, 3.0152E+01, 1.1244E+04, &
      5.5521003E-01, 3.77432E+01, 3.0112E+01, 1.1174E+04, &
      5.5798608E-01, 3.79115E+01, 3.0071E+01, 1.1103E+04, &
      5.6077601E-01, 3.80766E+01, 3.0029E+01, 1.1032E+04, &
      5.6357989E-01, 3.82382E+01, 2.9984E+01, 1.0961E+04, &
      5.6639779E-01, 3.83961E+01, 2.9938E+01, 1.0890E+04, &
      5.6922978E-01, 3.85499E+01, 2.9891E+01, 1.0819E+04, &
      5.7207593E-01, 3.86992E+01, 2.9842E+01, 1.0747E+04, &
      5.7493630E-01, 3.88434E+01, 2.9791E+01, 1.0676E+04, &
      5.7781099E-01, 3.89818E+01, 2.9739E+01, 1.0604E+04, &
      5.8070004E-01, 3.91136E+01, 2.9685E+01, 1.0532E+04, &
      5.8360354E-01, 3.92376E+01, 2.9630E+01, 1.0461E+04, &
      5.8652156E-01, 3.93521E+01, 2.9574E+01, 1.0389E+04, &
      5.8945417E-01, 3.94550E+01, 2.9516E+01, 1.0317E+04, &
      5.9240144E-01, 3.95428E+01, 2.9457E+01, 1.0245E+04, &
      5.9536345E-01, 3.96100E+01, 2.9397E+01, 1.0174E+04, &
      5.9834026E-01, 3.96471E+01, 2.9335E+01, 1.0102E+04, &
      6.0133196E-01, 3.96350E+01, 2.9272E+01, 1.0030E+04, &
      6.0433862E-01, 3.95249E+01, 2.9208E+01, 9.9584E+03, &
      6.0736032E-01, 3.90999E+01, 2.9143E+01, 9.8869E+03, &
      6.0824487E-01, 3.87065E+01, 2.9124E+01, 9.8660E+03, &
      6.0975519E-01, 3.87907E+01, 3.0886E+01, 1.0437E+04, &
      6.1039712E-01, 3.91779E+01, 3.0873E+01, 1.0421E+04, &
      6.1344910E-01, 4.00086E+01, 3.0808E+01, 1.0348E+04, &
      6.1651635E-01, 4.04763E+01, 3.0742E+01, 1.0274E+04, &
      6.1959893E-01, 4.08395E+01, 3.0675E+01, 1.0201E+04, &
      6.2269693E-01, 4.11516E+01, 3.0608E+01, 1.0128E+04, &
      6.2581041E-01, 4.14328E+01, 3.0539E+01, 1.0055E+04, &
      6.2893946E-01, 4.16931E+01, 3.0469E+01, 9.9820E+03, &
      6.3208416E-01, 4.19381E+01, 3.0398E+01, 9.9093E+03, &
      6.3524458E-01, 4.21712E+01, 3.0326E+01, 9.8369E+03, &
      6.3842080E-01, 4.23947E+01, 3.0254E+01, 9.7646E+03, &
      6.4161291E-01, 4.26102E+01, 3.0180E+01, 9.6925E+03, &
      6.4482097E-01, 4.28189E+01, 3.0106E+01, 9.6206E+03, &
      6.4804508E-01, 4.30216E+01, 3.0030E+01, 9.5489E+03, &
      6.5128530E-01, 4.32190E+01, 2.9955E+01, 9.4774E+03, &
      6.5454173E-01, 4.34115E+01, 2.9878E+01, 9.4062E+03, &
      6.5781444E-01, 4.35996E+01, 2.9800E+01, 9.3353E+03, &
      6.6110351E-01, 4.37836E+01, 2.9722E+01, 9.2645E+03, &
      6.6440903E-01, 4.39636E+01, 2.9643E+01, 9.1941E+03, &
      6.6773107E-01, 4.41400E+01, 2.9564E+01, 9.1239E+03, &
      6.7106973E-01, 4.43127E+01, 2.9484E+01, 9.0540E+03, &
      6.7442508E-01, 4.44820E+01, 2.9403E+01, 8.9843E+03, &
      6.7779720E-01, 4.46479E+01, 2.9321E+01, 8.9149E+03, &
      6.8118619E-01, 4.48103E+01, 2.9239E+01, 8.8458E+03, &
      6.8459212E-01, 4.49692E+01, 2.9156E+01, 8.7770E+03, &
      6.8801508E-01, 4.51245E+01, 2.9073E+01, 8.7084E+03, &
      6.9145515E-01, 4.52760E+01, 2.8989E+01, 8.6402E+03, &
      6.9491243E-01, 4.54234E+01, 2.8905E+01, 8.5722E+03, &
      6.9838699E-01, 4.55663E+01, 2.8820E+01, 8.5046E+03, &
      7.0187893E-01, 4.57040E+01, 2.8734E+01, 8.4373E+03, &
      7.0538832E-01, 4.58352E+01, 2.8648E+01, 8.3703E+03, &
      7.0891526E-01, 4.59578E+01, 2.8562E+01, 8.3036E+03, &
      7.1245984E-01, 4.60675E+01, 2.8475E+01, 8.2372E+03, &
      7.1602214E-01, 4.61526E+01, 2.8387E+01, 8.1712E+03, &
      7.1960225E-01, 4.61582E+01, 2.8299E+01, 8.1055E+03, &
      7.2029019E-01, 4.61213E+01, 2.8283E+01, 8.0929E+03, &
      7.2230983E-01, 4.62022E+01, 2.8625E+01, 8.1680E+03, &
      7.2320026E-01, 4.63166E+01, 2.8604E+01, 8.1518E+03, &
      7.2681626E-01, 4.65926E+01, 2.8517E+01, 8.0868E+03, &
      7.3045034E-01, 4.67975E+01, 2.8430E+01, 8.0221E+03, &
      7.3410260E-01, 4.69796E+01, 2.8343E+01, 7.9577E+03, &
      7.3777311E-01, 4.71495E+01, 2.8255E+01, 7.8937E+03, &
      7.4146197E-01, 4.73115E+01, 2.8167E+01, 7.8301E+03, &
      7.4516928E-01, 4.74676E+01, 2.8079E+01, 7.7668E+03, &
      7.4889513E-01, 4.76190E+01, 2.7990E+01, 7.7039E+03, &
      7.5263961E-01, 4.77665E+01, 2.7901E+01, 7.6413E+03, &
      7.5640280E-01, 4.79105E+01, 2.7812E+01, 7.5791E+03, &
      7.6018482E-01, 4.80514E+01, 2.7723E+01, 7.5173E+03, &
      7.6398574E-01, 4.81893E+01, 2.7633E+01, 7.4558E+03, &
      7.6780567E-01, 4.83245E+01, 2.7543E+01, 7.3947E+03, &
      7.7164470E-01, 4.84571E+01, 2.7454E+01, 7.3340E+03, &
      7.7550292E-01, 4.85871E+01, 2.7363E+01, 7.2737E+03, &
      7.7938044E-01, 4.87147E+01, 2.7273E+01, 7.2137E+03, &
      7.8327734E-01, 4.88397E+01, 2.7183E+01, 7.1542E+03, &
      7.8719373E-01, 4.89622E+01, 2.7092E+01, 7.0950E+03, &
      7.9112969E-01, 4.90822E+01, 2.7002E+01, 7.0361E+03, &
      7.9508534E-01, 4.91996E+01, 2.6911E+01, 6.9777E+03, &
      7.9906077E-01, 4.93142E+01, 2.6820E+01, 6.9197E+03, &
      8.0305607E-01, 4.94260E+01, 2.6729E+01, 6.8620E+03, &
      8.0707135E-01, 4.95346E+01, 2.6638E+01, 6.8047E+03, &
      8.1110671E-01, 4.96398E+01, 2.6547E+01, 6.7478E+03, &
      8.1516224E-01, 4.97410E+01, 2.6456E+01, 6.6912E+03, &
      8.1923806E-01, 4.98375E+01, 2.6364E+01, 6.6351E+03, &
      8.2333425E-01, 4.99282E+01, 2.6273E+01, 6.5793E+03, &
      8.2745092E-01, 5.00114E+01, 2.6182E+01, 6.5240E+03, &
      8.3158817E-01, 5.00836E+01, 2.6091E+01, 6.4690E+03, &
      8.3574611E-01, 5.01381E+01, 2.5999E+01, 6.4143E+03, &
      8.3992484E-01, 5.01557E+01, 2.5908E+01, 6.3601E+03, &
      8.4412447E-01, 5.00257E+01, 2.5817E+01, 6.3062E+03, &
      8.4423174E-01, 5.00145E+01, 2.5814E+01, 6.3049E+03, &
      8.4676824E-01, 5.00820E+01, 2.6306E+01, 6.4055E+03, &
      8.4834509E-01, 5.02642E+01, 2.6272E+01, 6.3855E+03, &
      8.5258682E-01, 5.05341E+01, 2.6181E+01, 6.3319E+03, &
      8.5684975E-01, 5.07262E+01, 2.6091E+01, 6.2786E+03, &
      8.6113400E-01, 5.08910E+01, 2.6000E+01, 6.2258E+03, &
      8.6543967E-01, 5.10413E+01, 2.5909E+01, 6.1734E+03, &
      8.6976687E-01, 5.11825E+01, 2.5819E+01, 6.1213E+03, &
      8.7411570E-01, 5.13172E+01, 2.5728E+01, 6.0696E+03, &
      8.7848628E-01, 5.14469E+01, 2.5638E+01, 6.0183E+03, &
      8.8287871E-01, 5.15726E+01, 2.5548E+01, 5.9674E+03, &
      8.8729310E-01, 5.16950E+01, 2.5458E+01, 5.9169E+03, &
      8.9172957E-01, 5.18146E+01, 2.5368E+01, 5.8667E+03, &
      8.9618822E-01, 5.19317E+01, 2.5278E+01, 5.8169E+03, &
      9.0066916E-01, 5.20466E+01, 2.5188E+01, 5.7675E+03, &
      9.0517250E-01, 5.21595E+01, 2.5098E+01, 5.7184E+03, &
      9.0969837E-01, 5.22707E+01, 2.5008E+01, 5.6698E+03, &
      9.1424686E-01, 5.23801E+01, 2.4919E+01, 5.6215E+03, &
      9.1881809E-01, 5.24881E+01, 2.4829E+01, 5.5736E+03, &
      9.2341218E-01, 5.25947E+01, 2.4740E+01, 5.5260E+03, &
      9.2802924E-01, 5.26999E+01, 2.4651E+01, 5.4788E+03, &
      9.3266939E-01, 5.28040E+01, 2.4562E+01, 5.4320E+03, &
      9.3733274E-01, 5.29069E+01, 2.4473E+01, 5.3856E+03, &
      9.4201940E-01, 5.30088E+01, 2.4385E+01, 5.3395E+03, &
      9.4672950E-01, 5.31098E+01, 2.4296E+01, 5.2937E+03, &
      9.5146315E-01, 5.32100E+01, 2.4208E+01, 5.2483E+03, &
      9.5622046E-01, 5.33094E+01, 2.4120E+01, 5.2033E+03, &
      9.6100156E-01, 5.34082E+01, 2.4032E+01, 5.1586E+03, &
      9.6580657E-01, 5.35067E+01, 2.3944E+01, 5.1143E+03, &
      9.7063560E-01, 5.36049E+01, 2.3856E+01, 5.0703E+03, &
      9.7548878E-01, 5.37032E+01, 2.3769E+01, 5.0267E+03, &
      9.8036623E-01, 5.38019E+01, 2.3681E+01, 4.9835E+03, &
      9.8526806E-01, 5.39017E+01, 2.3594E+01, 4.9405E+03, &
      9.9019440E-01, 5.40034E+01, 2.3507E+01, 4.8980E+03, &
      9.9514537E-01, 5.41085E+01, 2.3421E+01, 4.8557E+03, &
      1.0001211E+00, 5.42213E+01, 2.3333E+01, 4.8137E+03, &
      1.0051217E+00, 5.43845E+01, 2.3209E+01, 4.7644E+03, &
      1.0101473E+00, 5.45337E+01, 2.3086E+01, 4.7156E+03, &
      1.0151980E+00, 5.46718E+01, 2.2963E+01, 4.6674E+03, &
      1.0202740E+00, 5.48005E+01, 2.2842E+01, 4.6196E+03, &
      1.0253754E+00, 5.49214E+01, 2.2721E+01, 4.5724E+03, &
      1.0305023E+00, 5.50355E+01, 2.2600E+01, 4.5256E+03, &
      1.0356548E+00, 5.51433E+01, 2.2478E+01, 4.4790E+03, &
      1.0408331E+00, 5.52457E+01, 2.2357E+01, 4.4329E+03, &
      1.0460372E+00, 5.53432E+01, 2.2237E+01, 4.3872E+03, &
      1.0512674E+00, 5.54362E+01, 2.2118E+01, 4.3421E+03, &
      1.0565238E+00, 5.55253E+01, 2.1999E+01, 4.2975E+03, &
      1.0618064E+00, 5.56107E+01, 2.1881E+01, 4.2533E+03, &
      1.0671154E+00, 5.56928E+01, 2.1764E+01, 4.2096E+03, &
      1.0724510E+00, 5.57717E+01, 2.1648E+01, 4.1663E+03, &
      1.0778132E+00, 5.58478E+01, 2.1532E+01, 4.1236E+03, &
      1.0832023E+00, 5.59211E+01, 2.1417E+01, 4.0812E+03, &
      1.0886183E+00, 5.59919E+01, 2.1302E+01, 4.0394E+03, &
      1.0940614E+00, 5.60603E+01, 2.1189E+01, 3.9979E+03, &
      1.0995317E+00, 5.61265E+01, 2.1076E+01, 3.9570E+03, &
      1.1050294E+00, 5.61905E+01, 2.0963E+01, 3.9164E+03, &
      1.1105545E+00, 5.62524E+01, 2.0852E+01, 3.8763E+03, &
      1.1161073E+00, 5.63125E+01, 2.0741E+01, 3.8366E+03, &
      1.1216878E+00, 5.63706E+01, 2.0630E+01, 3.7973E+03, &
      1.1272963E+00, 5.64270E+01, 2.0521E+01, 3.7585E+03, &
      1.1329328E+00, 5.64816E+01, 2.0412E+01, 3.7201E+03, &
      1.1385974E+00, 5.65346E+01, 2.0303E+01, 3.6820E+03, &
      1.1442904E+00, 5.65860E+01, 2.0196E+01, 3.6444E+03, &
      1.1500119E+00, 5.66359E+01, 2.0089E+01, 3.6072E+03, &
      1.1557619E+00, 5.66843E+01, 1.9982E+01, 3.5704E+03, &
      1.1615407E+00, 5.67299E+01, 1.9877E+01, 3.5340E+03, &
      1.1673484E+00, 5.67756E+01, 1.9772E+01, 3.4979E+03, &
      1.1731852E+00, 5.68200E+01, 1.9667E+01, 3.4623E+03, &
      1.1790511E+00, 5.68630E+01, 1.9563E+01, 3.4270E+03, &
      1.1849464E+00, 5.69047E+01, 1.9460E+01, 3.3921E+03, &
      1.1908711E+00, 5.69452E+01, 1.9358E+01, 3.3576E+03, &
      1.1968254E+00, 5.69846E+01, 1.9256E+01, 3.3234E+03, &
      1.2028096E+00, 5.70227E+01, 1.9155E+01, 3.2896E+03, &
      1.2088236E+00, 5.70598E+01, 1.9054E+01, 3.2562E+03, &
      1.2148677E+00, 5.70957E+01, 1.8954E+01, 3.2231E+03, &
      1.2209421E+00, 5.71319E+01, 1.8854E+01, 3.1903E+03, &
      1.2270468E+00, 5.71656E+01, 1.8755E+01, 3.1578E+03, &
      1.2331820E+00, 5.71980E+01, 1.8655E+01, 3.1255E+03, &
      1.2393479E+00, 5.72294E+01, 1.8557E+01, 3.0937E+03, &
      1.2455447E+00, 5.72595E+01, 1.8459E+01, 3.0621E+03, &
      1.2517724E+00, 5.72885E+01, 1.8361E+01, 3.0309E+03, &
      1.2580312E+00, 5.73165E+01, 1.8264E+01, 3.0001E+03, &
      1.2643214E+00, 5.73433E+01, 1.8168E+01, 2.9695E+03, &
      1.2706430E+00, 5.73691E+01, 1.8072E+01, 2.9393E+03, &
      1.2769962E+00, 5.73938E+01, 1.7977E+01, 2.9094E+03, &
      1.2833812E+00, 5.74175E+01, 1.7883E+01, 2.8798E+03, &
      1.2897981E+00, 5.74402E+01, 1.7789E+01, 2.8506E+03, &
      1.2962471E+00, 5.74618E+01, 1.7695E+01, 2.8216E+03, &
      1.3027283E+00, 5.74825E+01, 1.7602E+01, 2.7930E+03, &
      1.3092420E+00, 5.75021E+01, 1.7510E+01, 2.7647E+03, &
      1.3157882E+00, 5.75209E+01, 1.7418E+01, 2.7366E+03, &
      1.3223671E+00, 5.75386E+01, 1.7327E+01, 2.7089E+03, &
      1.3289790E+00, 5.75554E+01, 1.7237E+01, 2.6815E+03, &
      1.3356239E+00, 5.75712E+01, 1.7147E+01, 2.6543E+03, &
      1.3423020E+00, 5.75862E+01, 1.7057E+01, 2.6275E+03, &
      1.3490135E+00, 5.76002E+01, 1.6969E+01, 2.6009E+03, &
      1.3557586E+00, 5.76132E+01, 1.6880E+01, 2.5746E+03, &
      1.3625374E+00, 5.76254E+01, 1.6792E+01, 2.5486E+03, &
      1.3693500E+00, 5.76367E+01, 1.6705E+01, 2.5229E+03, &
      1.3761968E+00, 5.76470E+01, 1.6619E+01, 2.4974E+03, &
      1.3830778E+00, 5.76565E+01, 1.6532E+01, 2.4722E+03, &
      1.3899932E+00, 5.76651E+01, 1.6447E+01, 2.4473E+03, &
      1.3969431E+00, 5.76729E+01, 1.6362E+01, 2.4227E+03, &
      1.4039278E+00, 5.76798E+01, 1.6277E+01, 2.3983E+03, &
      1.4109475E+00, 5.76858E+01, 1.6193E+01, 2.3742E+03, &
      1.4180022E+00, 5.76910E+01, 1.6110E+01, 2.3503E+03, &
      1.4250922E+00, 5.76953E+01, 1.6026E+01, 2.3266E+03, &
      1.4322177E+00, 5.76988E+01, 1.5943E+01, 2.3032E+03, &
      1.4393788E+00, 5.77013E+01, 1.5861E+01, 2.2800E+03, &
      1.4465757E+00, 5.77029E+01, 1.5778E+01, 2.2570E+03, &
      1.4538086E+00, 5.77035E+01, 1.5696E+01, 2.2342E+03, &
      1.4610776E+00, 5.77033E+01, 1.5615E+01, 2.2117E+03, &
      1.4683830E+00, 5.77021E+01, 1.5534E+01, 2.1894E+03, &
      1.4757249E+00, 5.77000E+01, 1.5454E+01, 2.1674E+03, &
      1.4831035E+00, 5.76970E+01, 1.5374E+01, 2.1456E+03, &
      1.4905190E+00, 5.76931E+01, 1.5295E+01, 2.1241E+03, &
      1.4979716E+00, 5.76883E+01, 1.5216E+01, 2.1027E+03, &
      1.5054615E+00, 5.76826E+01, 1.5138E+01, 2.0817E+03, &
      1.5129888E+00, 5.76760E+01, 1.5061E+01, 2.0608E+03, &
      1.5205537E+00, 5.76685E+01, 1.4983E+01, 2.0401E+03, &
      1.5281565E+00, 5.76602E+01, 1.4907E+01, 2.0197E+03, &
      1.5357973E+00, 5.76510E+01, 1.4830E+01, 1.9995E+03, &
      1.5434763E+00, 5.76409E+01, 1.4755E+01, 1.9795E+03, &
      1.5511937E+00, 5.76298E+01, 1.4673E+01, 1.9589E+03, &
      1.5589496E+00, 5.76172E+01, 1.4591E+01, 1.9384E+03, &
      1.5667444E+00, 5.76032E+01, 1.4509E+01, 1.9180E+03, &
      1.5745781E+00, 5.75877E+01, 1.4428E+01, 1.8979E+03, &
      1.5824510E+00, 5.75708E+01, 1.4347E+01, 1.8781E+03, &
      1.5903633E+00, 5.75524E+01, 1.4267E+01, 1.8585E+03, &
      1.5983151E+00, 5.75327E+01, 1.4188E+01, 1.8390E+03, &
      1.6063066E+00, 5.75116E+01, 1.4109E+01, 1.8198E+03, &
      1.6143382E+00, 5.74890E+01, 1.4031E+01, 1.8009E+03, &
      1.6224099E+00, 5.74651E+01, 1.3953E+01, 1.7821E+03, &
      1.6305219E+00, 5.74397E+01, 1.3873E+01, 1.7632E+03, &
      1.6386745E+00, 5.74126E+01, 1.3792E+01, 1.7443E+03, &
      1.6468679E+00, 5.73837E+01, 1.3711E+01, 1.7256E+03, &
      1.6551022E+00, 5.73530E+01, 1.3631E+01, 1.7070E+03, &
      1.6633777E+00, 5.73205E+01, 1.3551E+01, 1.6887E+03, &
      1.6716946E+00, 5.72862E+01, 1.3472E+01, 1.6707E+03, &
      1.6800531E+00, 5.72500E+01, 1.3393E+01, 1.6528E+03, &
      1.6884534E+00, 5.72121E+01, 1.3315E+01, 1.6351E+03, &
      1.6968956E+00, 5.71723E+01, 1.3238E+01, 1.6177E+03, &
      1.7053801E+00, 5.71307E+01, 1.3161E+01, 1.6004E+03, &
      1.7139070E+00, 5.70871E+01, 1.3085E+01, 1.5833E+03, &
      1.7224766E+00, 5.70417E+01, 1.3009E+01, 1.5665E+03, &
      1.7310889E+00, 5.69943E+01, 1.2934E+01, 1.5498E+03, &
      1.7397444E+00, 5.69450E+01, 1.2860E+01, 1.5333E+03, &
      1.7484431E+00, 5.68936E+01, 1.2786E+01, 1.5170E+03, &
      1.7571853E+00, 5.68403E+01, 1.2712E+01, 1.5009E+03, &
      1.7659712E+00, 5.67848E+01, 1.2639E+01, 1.4850E+03, &
      1.7748011E+00, 5.67272E+01, 1.2567E+01, 1.4693E+03, &
      1.7836751E+00, 5.66674E+01, 1.2495E+01, 1.4537E+03, &
      1.7925935E+00, 5.66054E+01, 1.2423E+01, 1.4384E+03, &
      1.8015565E+00, 5.65411E+01, 1.2352E+01, 1.4232E+03, &
      1.8105642E+00, 5.64745E+01, 1.2282E+01, 1.4081E+03, &
      1.8196171E+00, 5.64054E+01, 1.2212E+01, 1.3933E+03, &
      1.8287151E+00, 5.63350E+01, 1.2143E+01, 1.3786E+03, &
      1.8378587E+00, 5.62609E+01, 1.2074E+01, 1.3641E+03, &
      1.8470480E+00, 5.61842E+01, 1.2006E+01, 1.3497E+03, &
      1.8562833E+00, 5.61047E+01, 1.1938E+01, 1.3356E+03, &
      1.8655647E+00, 5.60225E+01, 1.1870E+01, 1.3215E+03, &
      1.8748925E+00, 5.59373E+01, 1.1804E+01, 1.3077E+03, &
      1.8842670E+00, 5.58491E+01, 1.1737E+01, 1.2939E+03, &
      1.8936883E+00, 5.57578E+01, 1.1671E+01, 1.2804E+03, &
      1.9031567E+00, 5.56633E+01, 1.1605E+01, 1.2670E+03, &
      1.9126725E+00, 5.55653E+01, 1.1540E+01, 1.2537E+03, &
      1.9222359E+00, 5.54638E+01, 1.1475E+01, 1.2406E+03, &
      1.9318471E+00, 5.53587E+01, 1.1411E+01, 1.2276E+03, &
      1.9415063E+00, 5.52497E+01, 1.1347E+01, 1.2148E+03, &
      1.9512138E+00, 5.51367E+01, 1.1284E+01, 1.2021E+03, &
      1.9609699E+00, 5.50195E+01, 1.1221E+01, 1.1896E+03, &
      1.9707747E+00, 5.48980E+01, 1.1159E+01, 1.1772E+03, &
      1.9806286E+00, 5.47718E+01, 1.1097E+01, 1.1650E+03, &
      1.9905318E+00, 5.46408E+01, 1.1035E+01, 1.1529E+03, &
      2.0004844E+00, 5.45047E+01, 1.0974E+01, 1.1409E+03, &
      2.0104868E+00, 5.43632E+01, 1.0914E+01, 1.1291E+03, &
      2.0205393E+00, 5.42161E+01, 1.0853E+01, 1.1174E+03, &
      2.0306420E+00, 5.40629E+01, 1.0793E+01, 1.1058E+03, &
      2.0407952E+00, 5.39034E+01, 1.0734E+01, 1.0944E+03, &
      2.0509992E+00, 5.37372E+01, 1.0675E+01, 1.0830E+03, &
      2.0612542E+00, 5.35638E+01, 1.0617E+01, 1.0719E+03, &
      2.0715604E+00, 5.33827E+01, 1.0558E+01, 1.0608E+03, &
      2.0819182E+00, 5.31934E+01, 1.0501E+01, 1.0499E+03, &
      2.0923278E+00, 5.29954E+01, 1.0443E+01, 1.0391E+03, &
      2.1027895E+00, 5.27880E+01, 1.0386E+01, 1.0284E+03, &
      2.1133034E+00, 5.25704E+01, 1.0330E+01, 1.0178E+03, &
      2.1238699E+00, 5.23419E+01, 1.0274E+01, 1.0073E+03, &
      2.1344893E+00, 5.21016E+01, 1.0218E+01, 9.9700E+02, &
      2.1451617E+00, 5.18483E+01, 1.0163E+01, 9.8677E+02, &
      2.1558875E+00, 5.15810E+01, 1.0108E+01, 9.7666E+02, &
      2.1666670E+00, 5.13005E+01, 1.0053E+01, 9.6667E+02, &
      2.1775003E+00, 5.10009E+01, 9.9988E+00, 9.5678E+02, &
      2.1883878E+00, 5.06824E+01, 9.9449E+00, 9.4701E+02, &
      2.1993297E+00, 5.03432E+01, 9.8915E+00, 9.3734E+02, &
      2.2103264E+00, 4.99805E+01, 9.8384E+00, 9.2778E+02, &
      2.2213780E+00, 4.95916E+01, 9.7856E+00, 9.1833E+02, &
      2.2324849E+00, 4.91729E+01, 9.7313E+00, 9.0880E+02, &
      2.2436473E+00, 4.87199E+01, 9.6763E+00, 8.9928E+02, &
      2.2548656E+00, 4.82272E+01, 9.6218E+00, 8.8988E+02, &
      2.2661399E+00, 4.76881E+01, 9.5677E+00, 8.8058E+02, &
      2.2774706E+00, 4.70938E+01, 9.5139E+00, 8.7139E+02, &
      2.2888579E+00, 4.64329E+01, 9.4607E+00, 8.6231E+02, &
      2.3003022E+00, 4.56899E+01, 9.4078E+00, 8.5334E+02, &
      2.3118037E+00, 4.48428E+01, 9.3542E+00, 8.4437E+02, &
      2.3233628E+00, 4.38592E+01, 9.2999E+00, 8.3541E+02, &
      2.3349796E+00, 4.26879E+01, 9.2461E+00, 8.2656E+02, &
      2.3466545E+00, 4.12408E+01, 9.1928E+00, 8.1781E+02, &
      2.3583878E+00, 3.93445E+01, 9.1399E+00, 8.0917E+02, &
      2.3701797E+00, 3.65691E+01, 9.0874E+00, 8.0063E+02, &
      2.3820306E+00, 3.11333E+01, 9.0354E+00, 7.9220E+02, &
      2.3886550E+00, 1.78561E+01, 9.0066E+00, 7.8754E+02, &
      2.3899452E+00, 1.75600E+01, 2.6110E+01, 2.2606E+03, &
      2.3939407E+00, 2.82573E+01, 2.6045E+01, 2.2512E+03, &
      2.4059104E+00, 3.49344E+01, 2.5852E+01, 2.2236E+03, &
      2.4179400E+00, 3.75104E+01, 2.5661E+01, 2.1963E+03, &
      2.4300297E+00, 3.88982E+01, 2.5471E+01, 2.1693E+03, &
      2.4421798E+00, 3.95964E+01, 2.5282E+01, 2.1426E+03, &
      2.4543907E+00, 3.96905E+01, 2.5094E+01, 2.1162E+03, &
      2.4666627E+00, 3.89611E+01, 2.4907E+01, 2.0901E+03, &
      2.4789960E+00, 3.59590E+01, 2.4722E+01, 2.0644E+03, &
      2.4843943E+00, 2.85815E+01, 2.4642E+01, 2.0533E+03, &
      2.4858058E+00, 2.85013E+01, 3.5959E+01, 2.9895E+03, &
      2.4913910E+00, 3.68012E+01, 3.5836E+01, 2.9726E+03, &
      2.5038479E+00, 4.15359E+01, 3.5563E+01, 2.9354E+03, &
      2.5163672E+00, 4.40836E+01, 3.5293E+01, 2.8988E+03, &
      2.5289490E+00, 4.59357E+01, 3.5024E+01, 2.8625E+03, &
      2.5415938E+00, 4.74244E+01, 3.4758E+01, 2.8268E+03, &
      2.5543017E+00, 4.86821E+01, 3.4494E+01, 2.7915E+03, &
      2.5670732E+00, 4.97767E+01, 3.4233E+01, 2.7567E+03, &
      2.5799086E+00, 5.07481E+01, 3.3973E+01, 2.7223E+03, &
      2.5928082E+00, 5.16222E+01, 3.3715E+01, 2.6883E+03, &
      2.6057722E+00, 5.24165E+01, 3.3460E+01, 2.6548E+03, &
      2.6188011E+00, 5.31438E+01, 3.3206E+01, 2.6217E+03, &
      2.6318951E+00, 5.38137E+01, 3.2955E+01, 2.5891E+03, &
      2.6450545E+00, 5.44335E+01, 3.2706E+01, 2.5568E+03, &
      2.6582798E+00, 5.50090E+01, 3.2459E+01, 2.5250E+03, &
      2.6715712E+00, 5.55447E+01, 3.2213E+01, 2.4936E+03, &
      2.6849291E+00, 5.60442E+01, 3.1970E+01, 2.4625E+03, &
      2.6983537E+00, 5.65106E+01, 3.1729E+01, 2.4319E+03, &
      2.7118455E+00, 5.69461E+01, 3.1489E+01, 2.4017E+03, &
      2.7254047E+00, 5.73529E+01, 3.1252E+01, 2.3718E+03, &
      2.7390317E+00, 5.77321E+01, 3.1016E+01, 2.3424E+03, &
      2.7527269E+00, 5.80851E+01, 3.0783E+01, 2.3133E+03, &
      2.7664905E+00, 5.84124E+01, 3.0551E+01, 2.2846E+03, &
      2.7803230E+00, 5.87144E+01, 3.0321E+01, 2.2562E+03, &
      2.7942246E+00, 5.89909E+01, 3.0093E+01, 2.2282E+03, &
      2.8081957E+00, 5.92412E+01, 2.9867E+01, 2.2006E+03, &
      2.8222367E+00, 5.94642E+01, 2.9643E+01, 2.1733E+03, &
      2.8363479E+00, 5.96574E+01, 2.9420E+01, 2.1464E+03, &
      2.8505296E+00, 5.98175E+01, 2.9200E+01, 2.1198E+03, &
      2.8647823E+00, 5.99389E+01, 2.8981E+01, 2.0936E+03, &
      2.8791062E+00, 6.00127E+01, 2.8763E+01, 2.0677E+03, &
      2.8935017E+00, 6.00239E+01, 2.8548E+01, 2.0421E+03, &
      2.9079692E+00, 5.99446E+01, 2.8333E+01, 2.0168E+03, &
      2.9225091E+00, 5.97160E+01, 2.8120E+01, 1.9918E+03, &
      2.9371216E+00, 5.91734E+01, 2.7909E+01, 1.9671E+03, &
      2.9518072E+00, 5.73502E+01, 2.7700E+01, 1.9428E+03, &
      2.9536434E+00, 5.66537E+01, 2.7674E+01, 1.9398E+03, &
      2.9595566E+00, 5.67161E+01, 3.2380E+01, 2.2633E+03, &
      2.9665662E+00, 5.87250E+01, 3.2253E+01, 2.2492E+03, &
      2.9813991E+00, 6.04119E+01, 3.1988E+01, 2.2197E+03, &
      2.9963061E+00, 6.14038E+01, 3.1725E+01, 2.1906E+03, &
      3.0112876E+00, 6.22697E+01, 3.1455E+01, 2.1613E+03, &
      3.0263440E+00, 6.29126E+01, 3.1187E+01, 2.1324E+03, &
      3.0414758E+00, 6.34445E+01, 3.0924E+01, 2.1040E+03, &
      3.0566831E+00, 6.39078E+01, 3.0665E+01, 2.0761E+03, &
      3.0719666E+00, 6.43214E+01, 3.0409E+01, 2.0486E+03, &
      3.0873264E+00, 6.46956E+01, 3.0156E+01, 2.0216E+03, &
      3.1027630E+00, 6.50359E+01, 2.9907E+01, 1.9950E+03, &
      3.1182768E+00, 6.53500E+01, 2.9670E+01, 1.9696E+03, &
      3.1338682E+00, 6.56450E+01, 2.9436E+01, 1.9444E+03, &
      3.1495376E+00, 6.59220E+01, 2.9204E+01, 1.9196E+03, &
      3.1652853E+00, 6.61822E+01, 2.8975E+01, 1.8952E+03, &
      3.1811117E+00, 6.64270E+01, 2.8748E+01, 1.8711E+03, &
      3.1970172E+00, 6.66571E+01, 2.8524E+01, 1.8474E+03, &
      3.2130023E+00, 6.68732E+01, 2.8302E+01, 1.8240E+03, &
      3.2290673E+00, 6.70758E+01, 2.8082E+01, 1.8009E+03, &
      3.2452127E+00, 6.72644E+01, 2.7861E+01, 1.7780E+03, &
      3.2614387E+00, 6.74384E+01, 2.7643E+01, 1.7554E+03, &
      3.2777459E+00, 6.75971E+01, 2.7426E+01, 1.7331E+03, &
      3.2941347E+00, 6.77395E+01, 2.7211E+01, 1.7111E+03, &
      3.3106053E+00, 6.78635E+01, 2.6998E+01, 1.6893E+03, &
      3.3271584E+00, 6.79660E+01, 2.6787E+01, 1.6679E+03, &
      3.3437941E+00, 6.80410E+01, 2.6578E+01, 1.6468E+03, &
      3.3605131E+00, 6.80778E+01, 2.6371E+01, 1.6259E+03, &
      3.3773157E+00, 6.80529E+01, 2.6165E+01, 1.6053E+03, &
      3.3942023E+00, 6.78992E+01, 2.5961E+01, 1.5850E+03, &
      3.4107130E+00, 6.72314E+01, 2.5764E+01, 1.5654E+03, &
      3.4111733E+00, 6.71804E+01, 2.5758E+01, 1.5649E+03, &
      3.4206869E+00, 6.73136E+01, 2.7454E+01, 1.6626E+03, &
      3.4282291E+00, 6.79227E+01, 2.7355E+01, 1.6530E+03, &
      3.4453703E+00, 6.85836E+01, 2.7133E+01, 1.6316E+03, &
      3.4625971E+00, 6.89994E+01, 2.6913E+01, 1.6104E+03, &
      3.4799101E+00, 6.93225E+01, 2.6694E+01, 1.5895E+03, &
      3.4973097E+00, 6.95934E+01, 2.6478E+01, 1.5689E+03, &
      3.5147962E+00, 6.98287E+01, 2.6263E+01, 1.5485E+03, &
      3.5323702E+00, 7.00367E+01, 2.6050E+01, 1.5284E+03, &
      3.5500321E+00, 7.02218E+01, 2.5839E+01, 1.5086E+03, &
      3.5677822E+00, 7.03858E+01, 2.5629E+01, 1.4890E+03, &
      3.5856211E+00, 7.05286E+01, 2.5423E+01, 1.4698E+03, &
      3.6035492E+00, 7.06528E+01, 2.5227E+01, 1.4513E+03, &
      3.6215670E+00, 7.07590E+01, 2.5033E+01, 1.4331E+03, &
      3.6396748E+00, 7.08416E+01, 2.4842E+01, 1.4152E+03, &
      3.6578732E+00, 7.08892E+01, 2.4653E+01, 1.3975E+03, &
      3.6761626E+00, 7.08727E+01, 2.4465E+01, 1.3801E+03, &
      3.6945434E+00, 7.06565E+01, 2.4280E+01, 1.3630E+03, &
      3.6962102E+00, 7.06044E+01, 2.4263E+01, 1.3614E+03, &
      3.7119896E+00, 7.07221E+01, 2.5181E+01, 1.4065E+03, &
      3.7130161E+00, 7.07719E+01, 2.5170E+01, 1.4056E+03, &
      3.7315812E+00, 7.12981E+01, 2.4987E+01, 1.3885E+03, &
      3.7502391E+00, 7.16117E+01, 2.4806E+01, 1.3716E+03, &
      3.7689903E+00, 7.18606E+01, 2.4626E+01, 1.3550E+03, &
      3.7878352E+00, 7.20763E+01, 2.4448E+01, 1.3387E+03, &
      3.8067744E+00, 7.22709E+01, 2.4272E+01, 1.3225E+03, &
      3.8258083E+00, 7.24508E+01, 2.4098E+01, 1.3066E+03, &
      3.8449373E+00, 7.26195E+01, 2.3925E+01, 1.2909E+03, &
      3.8641620E+00, 7.27784E+01, 2.3751E+01, 1.2752E+03, &
      3.8834828E+00, 7.29288E+01, 2.3578E+01, 1.2597E+03, &
      3.9029002E+00, 7.30720E+01, 2.3406E+01, 1.2444E+03, &
      3.9224147E+00, 7.32088E+01, 2.3237E+01, 1.2293E+03, &
      3.9420268E+00, 7.33400E+01, 2.3069E+01, 1.2145E+03, &
      3.9617369E+00, 7.34662E+01, 2.2903E+01, 1.1998E+03, &
      3.9815456E+00, 7.35878E+01, 2.2738E+01, 1.1854E+03, &
      4.0014533E+00, 7.37053E+01, 2.2575E+01, 1.1711E+03, &
      4.0214606E+00, 7.38189E+01, 2.2414E+01, 1.1570E+03, &
      4.0415679E+00, 7.39289E+01, 2.2254E+01, 1.1432E+03, &
      4.0617757E+00, 7.40356E+01, 2.2095E+01, 1.1295E+03, &
      4.0820846E+00, 7.41391E+01, 2.1938E+01, 1.1160E+03, &
      4.1024950E+00, 7.42398E+01, 2.1783E+01, 1.1026E+03, &
      4.1230075E+00, 7.43377E+01, 2.1629E+01, 1.0895E+03, &
      4.1436226E+00, 7.44330E+01, 2.1476E+01, 1.0765E+03, &
      4.1643407E+00, 7.45259E+01, 2.1325E+01, 1.0637E+03, &
      4.1851624E+00, 7.46164E+01, 2.1175E+01, 1.0510E+03, &
      4.2060882E+00, 7.47047E+01, 2.1027E+01, 1.0386E+03, &
      4.2271186E+00, 7.47910E+01, 2.0879E+01, 1.0263E+03, &
      4.2482542E+00, 7.48753E+01, 2.0734E+01, 1.0141E+03, &
      4.2694955E+00, 7.49577E+01, 2.0589E+01, 1.0021E+03, &
      4.2908430E+00, 7.50383E+01, 2.0445E+01, 9.9026E+02, &
      4.3122972E+00, 7.51173E+01, 2.0303E+01, 9.7857E+02, &
      4.3338587E+00, 7.51947E+01, 2.0162E+01, 9.6702E+02, &
      4.3555280E+00, 7.52707E+01, 2.0023E+01, 9.5563E+02, &
      4.3773056E+00, 7.53453E+01, 1.9884E+01, 9.4438E+02, &
      4.3991921E+00, 7.54186E+01, 1.9747E+01, 9.3327E+02, &
      4.4211881E+00, 7.54910E+01, 1.9610E+01, 9.2230E+02, &
      4.4432940E+00, 7.55624E+01, 1.9475E+01, 9.1145E+02, &
      4.4655105E+00, 7.56320E+01, 1.9337E+01, 9.0059E+02, &
      4.4878381E+00, 7.56998E+01, 1.9200E+01, 8.8986E+02, &
      4.5102772E+00, 7.57659E+01, 1.9065E+01, 8.7926E+02, &
      4.5328286E+00, 7.58303E+01, 1.8930E+01, 8.6879E+02, &
      4.5554928E+00, 7.58933E+01, 1.8797E+01, 8.5846E+02, &
      4.5782702E+00, 7.59549E+01, 1.8664E+01, 8.4825E+02, &
      4.6011616E+00, 7.60150E+01, 1.8533E+01, 8.3816E+02, &
      4.6241674E+00, 7.60740E+01, 1.8402E+01, 8.2820E+02, &
      4.6472882E+00, 7.61317E+01, 1.8273E+01, 8.1837E+02, &
      4.6705247E+00, 7.61882E+01, 1.8144E+01, 8.0865E+02, &
      4.6938773E+00, 7.62437E+01, 1.8017E+01, 7.9905E+02, &
      4.7173467E+00, 7.62981E+01, 1.7890E+01, 7.8957E+02, &
      4.7409334E+00, 7.63515E+01, 1.7764E+01, 7.8021E+02, &
      4.7646381E+00, 7.64041E+01, 1.7640E+01, 7.7096E+02, &
      4.7884613E+00, 7.66394E+01, 1.7513E+01, 7.6170E+02, &
      4.8124036E+00, 7.66904E+01, 1.7383E+01, 7.5237E+02, &
      4.8364656E+00, 7.67396E+01, 1.7254E+01, 7.4313E+02, &
      4.8606479E+00, 7.67871E+01, 1.7125E+01, 7.3401E+02, &
      4.8849512E+00, 7.68331E+01, 1.6998E+01, 7.2500E+02, &
      4.9093759E+00, 7.68776E+01, 1.6871E+01, 7.1611E+02, &
      4.9339228E+00, 7.69207E+01, 1.6746E+01, 7.0734E+02, &
      4.9585924E+00, 7.69625E+01, 1.6622E+01, 6.9868E+02, &
      4.9833854E+00, 7.71287E+01, 1.6496E+01, 6.9003E+02, &
      5.0083023E+00, 7.71680E+01, 1.6369E+01, 6.8140E+02, &
      5.0333438E+00, 7.72057E+01, 1.6244E+01, 6.7290E+02, &
      5.0585105E+00, 7.72419E+01, 1.6119E+01, 6.6450E+02, &
      5.0838031E+00, 7.72766E+01, 1.5996E+01, 6.5621E+02, &
      5.1092221E+00, 7.73099E+01, 1.5874E+01, 6.4803E+02, &
      5.1347682E+00, 7.73419E+01, 1.5752E+01, 6.3996E+02, &
      5.1604421E+00, 7.73727E+01, 1.5632E+01, 6.3200E+02, &
      5.1862443E+00, 7.74022E+01, 1.5513E+01, 6.2414E+02, &
      5.2121755E+00, 7.74306E+01, 1.5395E+01, 6.1639E+02, &
      5.2382364E+00, 7.74578E+01, 1.5278E+01, 6.0873E+02, &
      5.2644276E+00, 7.74840E+01, 1.5162E+01, 6.0118E+02, &
      5.2907497E+00, 7.75092E+01, 1.5047E+01, 5.9372E+02, &
      5.3172034E+00, 7.75333E+01, 1.4933E+01, 5.8637E+02, &
      5.3437895E+00, 7.75565E+01, 1.4819E+01, 5.7911E+02, &
      5.3705084E+00, 7.75788E+01, 1.4707E+01, 5.7194E+02, &
      5.3973609E+00, 7.76001E+01, 1.4596E+01, 5.6487E+02, &
      5.4243477E+00, 7.76206E+01, 1.4486E+01, 5.5789E+02, &
      5.4514695E+00, 7.76403E+01, 1.4377E+01, 5.5101E+02, &
      5.4787268E+00, 7.76591E+01, 1.4268E+01, 5.4421E+02, &
      5.5061205E+00, 7.76771E+01, 1.4161E+01, 5.3750E+02, &
      5.5336511E+00, 7.76944E+01, 1.4055E+01, 5.3089E+02, &
      5.5613193E+00, 7.77109E+01, 1.3949E+01, 5.2435E+02, &
      5.5891259E+00, 7.77268E+01, 1.3845E+01, 5.1791E+02, &
      5.6170716E+00, 7.77419E+01, 1.3741E+01, 5.1154E+02, &
      5.6451569E+00, 7.77564E+01, 1.3638E+01, 5.0526E+02, &
      5.6733827E+00, 7.77702E+01, 1.3536E+01, 4.9907E+02, &
      5.7017496E+00, 7.77834E+01, 1.3435E+01, 4.9295E+02, &
      5.7302584E+00, 7.77960E+01, 1.3335E+01, 4.8691E+02, &
      5.7589096E+00, 7.78080E+01, 1.3236E+01, 4.8096E+02, &
      5.7877042E+00, 7.78194E+01, 1.3138E+01, 4.7508E+02, &
      5.8166427E+00, 7.78304E+01, 1.3040E+01, 4.6927E+02, &
      5.8457259E+00, 7.78408E+01, 1.2943E+01, 4.6355E+02, &
      5.8749546E+00, 7.78507E+01, 1.2848E+01, 4.5790E+02, &
      5.9043293E+00, 7.78602E+01, 1.2753E+01, 4.5232E+02, &
      5.9338510E+00, 7.79663E+01, 1.2656E+01, 4.4673E+02, &
      5.9635202E+00, 7.79752E+01, 1.2559E+01, 4.4118E+02, &
      5.9933378E+00, 7.79834E+01, 1.2463E+01, 4.3570E+02, &
      6.0233045E+00, 7.79907E+01, 1.2368E+01, 4.3029E+02, &
      6.0534210E+00, 7.79974E+01, 1.2274E+01, 4.2495E+02, &
      6.0836882E+00, 7.80033E+01, 1.2180E+01, 4.1968E+02, &
      6.1141066E+00, 7.80086E+01, 1.2088E+01, 4.1449E+02, &
      6.1446771E+00, 7.80132E+01, 1.1996E+01, 4.0936E+02, &
      6.1754005E+00, 7.80172E+01, 1.1905E+01, 4.0430E+02, &
      6.2062775E+00, 7.80207E+01, 1.1815E+01, 3.9930E+02, &
      6.2373089E+00, 7.80235E+01, 1.1725E+01, 3.9438E+02, &
      6.2684954E+00, 7.80258E+01, 1.1637E+01, 3.8951E+02, &
      6.2998379E+00, 7.80276E+01, 1.1549E+01, 3.8471E+02, &
      6.3313371E+00, 7.80289E+01, 1.1462E+01, 3.7998E+02, &
      6.3629938E+00, 7.80297E+01, 1.1375E+01, 3.7530E+02, &
      6.3948088E+00, 7.80300E+01, 1.1290E+01, 3.7069E+02, &
      6.4267828E+00, 7.80298E+01, 1.1205E+01, 3.6614E+02, &
      6.4589167E+00, 7.80292E+01, 1.1121E+01, 3.6165E+02, &
      6.4912113E+00, 7.80281E+01, 1.1038E+01, 3.5722E+02, &
      6.5236674E+00, 7.80267E+01, 1.0955E+01, 3.5285E+02, &
      6.5562857E+00, 7.80248E+01, 1.0873E+01, 3.4853E+02, &
      6.5890671E+00, 7.80225E+01, 1.0792E+01, 3.4427E+02, &
      6.6220125E+00, 7.80198E+01, 1.0712E+01, 3.4007E+02, &
      6.6551225E+00, 7.80168E+01, 1.0632E+01, 3.3592E+02, &
      6.6883981E+00, 7.80134E+01, 1.0553E+01, 3.3183E+02, &
      6.7218401E+00, 7.80096E+01, 1.0475E+01, 3.2779E+02, &
      6.7554493E+00, 7.80055E+01, 1.0397E+01, 3.2380E+02, &
      6.7892266E+00, 7.80011E+01, 1.0320E+01, 3.1987E+02, &
      6.8231727E+00, 7.79964E+01, 1.0244E+01, 3.1598E+02, &
      6.8572886E+00, 7.80312E+01, 1.0168E+01, 3.1213E+02, &
      6.8915750E+00, 7.80261E+01, 1.0092E+01, 3.0831E+02, &
      6.9260329E+00, 7.80205E+01, 1.0016E+01, 3.0454E+02, &
      6.9606631E+00, 7.80145E+01, 9.9415E+00, 3.0083E+02, &
      6.9954664E+00, 7.80082E+01, 9.8674E+00, 2.9716E+02, &
      7.0304437E+00, 7.80015E+01, 9.7939E+00, 2.9353E+02, &
      7.0655959E+00, 7.79944E+01, 9.7211E+00, 2.8996E+02, &
      7.1009239E+00, 7.79870E+01, 9.6489E+00, 2.8643E+02, &
      7.1364285E+00, 7.79793E+01, 9.5773E+00, 2.8295E+02, &
      7.1721107E+00, 7.79712E+01, 9.5063E+00, 2.7951E+02, &
      7.2079712E+00, 7.79628E+01, 9.4359E+00, 2.7612E+02, &
      7.2440111E+00, 7.79541E+01, 9.3662E+00, 2.7277E+02, &
      7.2802311E+00, 7.79451E+01, 9.2965E+00, 2.6944E+02, &
      7.3166323E+00, 7.79357E+01, 9.2272E+00, 2.6616E+02, &
      7.3532155E+00, 7.79261E+01, 9.1585E+00, 2.6292E+02, &
      7.3899815E+00, 7.79161E+01, 9.0904E+00, 2.5972E+02, &
      7.4269314E+00, 7.79228E+01, 9.0227E+00, 2.5656E+02, &
      7.4640661E+00, 7.79123E+01, 8.9553E+00, 2.5343E+02, &
      7.5013864E+00, 7.79015E+01, 8.8885E+00, 2.5034E+02, &
      7.5388934E+00, 7.78904E+01, 8.8223E+00, 2.4729E+02, &
      7.5765878E+00, 7.78790E+01, 8.7566E+00, 2.4429E+02, &
      7.6144708E+00, 7.78672E+01, 8.6915E+00, 2.4132E+02, &
      7.6525431E+00, 7.78551E+01, 8.6268E+00, 2.3838E+02, &
      7.6908058E+00, 7.78427E+01, 8.5623E+00, 2.3547E+02, &
      7.7292599E+00, 7.78300E+01, 8.4984E+00, 2.3261E+02, &
      7.7679062E+00, 7.78169E+01, 8.4351E+00, 2.2978E+02, &
      7.8067457E+00, 7.78036E+01, 8.3723E+00, 2.2698E+02, &
      7.8457794E+00, 7.77899E+01, 8.3101E+00, 2.2423E+02, &
      7.8850083E+00, 7.77760E+01, 8.2484E+00, 2.2151E+02, &
      7.9244334E+00, 7.77618E+01, 8.1872E+00, 2.1882E+02, &
      7.9640555E+00, 7.77473E+01, 8.1266E+00, 2.1617E+02, &
      8.0038758E+00, 7.77325E+01, 8.0665E+00, 2.1355E+02, &
      8.0438952E+00, 7.77174E+01, 8.0069E+00, 2.1097E+02, &
      8.0841147E+00, 7.77021E+01, 7.9478E+00, 2.0842E+02, &
      8.1245352E+00, 7.76865E+01, 7.8892E+00, 2.0590E+02, &
      8.1651579E+00, 7.76706E+01, 7.8312E+00, 2.0342E+02, &
      8.2059837E+00, 7.76545E+01, 7.7736E+00, 2.0097E+02, &
      8.2470136E+00, 7.76381E+01, 7.7165E+00, 1.9855E+02, &
      8.2882487E+00, 7.76215E+01, 7.6599E+00, 1.9616E+02, &
      8.3296899E+00, 7.76047E+01, 7.6038E+00, 1.9380E+02, &
      8.3713384E+00, 7.75877E+01, 7.5482E+00, 1.9147E+02, &
      8.4131951E+00, 7.75704E+01, 7.4930E+00, 1.8917E+02, &
      8.4552610E+00, 7.75530E+01, 7.4384E+00, 1.8690E+02, &
      8.4975373E+00, 7.75354E+01, 7.3841E+00, 1.8466E+02, &
      8.5400250E+00, 7.75176E+01, 7.3304E+00, 1.8245E+02, &
      9.030794E+00, 7.74289E+01, 6.7354E+00, 1.5905E+02, &
      9.653919E+00, 7.70484E+01, 6.0270E+00, 1.3371E+02, &
      1.032004E+01, 7.65139E+01, 5.4021E+00, 1.1263E+02, &
      1.103212E+01, 7.57526E+01, 4.8327E+00, 9.4754E+01, &
      1.179334E+01, 7.44692E+01, 4.3245E+00, 7.9771E+01, &
      1.240435E+01, 7.20647E+01, 3.9797E+00, 7.0115E+01, &
      1.259421E+01, 6.93694E+01, 3.8824E+00, 6.7466E+01, &
      1.260708E+01, 6.89254E+01, 3.8759E+00, 6.7292E+01, &
      1.264484E+01, 6.62102E+01, 3.8571E+00, 6.6784E+01, &
      1.272079E+01, 6.93552E+01, 9.9955E+00, 1.6574E+02, &
      1.291065E+01, 7.21315E+01, 9.7442E+00, 1.5929E+02, &
      1.347697E+01, 7.42423E+01, 9.0459E+00, 1.4193E+02, &
      1.440688E+01, 7.42598E+01, 8.0525E+00, 1.1855E+02, &
      1.462441E+01, 7.30572E+01, 7.8450E+00, 1.1386E+02, &
      1.468320E+01, 7.14892E+01, 7.7903E+00, 1.1264E+02, &
      1.477139E+01, 7.31078E+01, 1.0832E+01, 1.5436E+02, &
      1.499186E+01, 7.45137E+01, 1.0563E+01, 1.4839E+02, &
      1.503977E+01, 7.46421E+01, 1.0506E+01, 1.4713E+02, &
      1.526997E+01, 7.46766E+01, 1.0239E+01, 1.4131E+02, &
      1.533135E+01, 7.40146E+01, 1.0169E+01, 1.3981E+02, &
      1.540095E+01, 7.48024E+01, 1.1689E+01, 1.5949E+02, &
      1.542343E+01, 7.50298E+01, 1.1663E+01, 1.5891E+02, &
      1.565363E+01, 7.61906E+01, 1.1400E+01, 1.5311E+02, &
      1.646362E+01, 7.79302E+01, 1.0543E+01, 1.3484E+02, &
      1.759961E+01, 7.91574E+01, 9.4813E+00, 1.1369E+02, &
      1.881398E+01, 7.99149E+01, 8.5258E+00, 9.5867E+01, &
      2.011215E+01, 8.04206E+01, 7.6618E+00, 8.0804E+01, &
      2.149988E+01, 8.07590E+01, 6.8829E+00, 6.8098E+01, &
      2.298338E+01, 8.09858E+01, 6.1862E+00, 5.7430E+01, &
      2.456923E+01, 8.11377E+01, 5.5631E+00, 4.8471E+01, &
      2.626450E+01, 8.13026E+01, 5.0021E+00, 4.0914E+01, &
      2.807676E+01, 8.13663E+01, 4.4664E+00, 3.4315E+01, &
      3.001405E+01, 8.14085E+01, 3.9805E+00, 2.8738E+01, &
      3.208502E+01, 8.14083E+01, 3.5398E+00, 2.4028E+01, &
      3.429889E+01, 8.13527E+01, 3.1474E+00, 2.0095E+01, &
      3.666551E+01, 8.12794E+01, 2.8006E+00, 1.6825E+01, &
      3.919543E+01, 8.11946E+01, 2.4940E+00, 1.4106E+01, &
      4.189992E+01, 8.11020E+01, 2.2227E+00, 1.1841E+01, &
      4.479101E+01, 8.10040E+01, 1.9825E+00, 9.9525E+00, &
      4.788159E+01, 8.09015E+01, 1.7695E+00, 8.3763E+00, &
      5.118542E+01, 8.07947E+01, 1.5807E+00, 7.0594E+00, &
      5.471721E+01, 8.06827E+01, 1.4129E+00, 5.9576E+00, &
      5.849270E+01, 8.05618E+01, 1.2610E+00, 5.0247E+00, &
      6.252870E+01, 8.04267E+01, 1.1264E+00, 4.2451E+00, &
      6.684318E+01, 8.02686E+01, 1.0051E+00, 3.5870E+00, &
      7.145536E+01, 8.00661E+01, 8.9427E-01, 3.0268E+00, &
      7.638578E+01, 7.97690E+01, 7.9602E-01, 2.5586E+00, &
      8.165640E+01, 7.91784E+01, 7.0702E-01, 2.1621E+00, &
      8.381980E+01, 7.85868E+01, 6.7349E-01, 2.0215E+00, &
      8.510275E+01, 7.75370E+01, 6.5478E-01, 1.9444E+00, &
      8.544487E+01, 7.62807E+01, 6.4993E-01, 1.9246E+00, &
      8.595806E+01, 7.75904E+01, 3.1153E+00, 7.8178E+00, &
      8.729069E+01, 7.87601E+01, 3.0417E+00, 7.5233E+00, &
      9.331374E+01, 8.00246E+01, 2.7389E+00, 6.3649E+00, &
      9.975239E+01, 8.05126E+01, 2.4627E+00, 5.3801E+00, &
      1.066353E+02, 8.07956E+01, 2.2109E+00, 4.5436E+00, &
      1.139931E+02, 8.09776E+01, 1.9839E+00, 3.8381E+00, &
      1.218587E+02, 8.11005E+01, 1.7796E+00, 3.2434E+00, &
      1.302669E+02, 8.11831E+01, 1.5932E+00, 2.7385E+00, &
      1.392553E+02, 8.12370E+01, 1.4262E+00, 2.3143E+00, &
      1.488640E+02, 8.12709E+01, 1.2766E+00, 1.9580E+00, &
      1.591356E+02, 8.12905E+01, 1.1428E+00, 1.6589E+00, &
      1.701159E+02, 8.12998E+01, 1.0231E+00, 1.4077E+00, &
      1.818539E+02, 8.13018E+01, 9.1573E-01, 1.1964E+00, &
      1.944018E+02, 8.12980E+01, 8.1924E-01, 1.0183E+00, &
      2.078156E+02, 8.12900E+01, 7.3300E-01, 8.6872E-01, &
      2.221548E+02, 8.12792E+01, 6.5591E-01, 7.4293E-01, &
      2.374835E+02, 8.12665E+01, 5.8698E-01, 6.3711E-01, &
      2.538699E+02, 8.12527E+01, 5.2535E-01, 5.4800E-01, &
      2.713869E+02, 8.12382E+01, 4.7023E-01, 4.7289E-01, &
      2.901126E+02, 8.12234E+01, 4.2093E-01, 4.0952E-01, &
      3.101304E+02, 8.12087E+01, 3.7684E-01, 3.5599E-01, &
      3.315294E+02, 8.11943E+01, 3.3739E-01, 3.1070E-01, &
      3.544049E+02, 8.11803E+01, 3.0210E-01, 2.7233E-01, &
      3.788588E+02, 8.11668E+01, 2.7052E-01, 2.3975E-01, &
      4.050001E+02, 8.11540E+01, 2.4226E-01, 2.1205E-01, &
      4.329451E+02, 8.11418E+01, 2.1697E-01, 1.8842E-01]
   case(82)
      frel = -1.1136E+00
      allocate(nist_tab(3112))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      4.813430E-03, 3.38841E+00, 0.0000E+00, 3.9317E-06, &
      4.887105E-03, 2.16509E+00, 0.0000E+00, 4.2792E-06, &
      4.906751E-03, 7.08129E-01, 0.0000E+00, 4.3757E-06, &
      4.936221E-03, 2.21405E+00, 2.8636E+00, 1.1782E+05, &
      5.009896E-03, 3.52849E+00, 2.8173E+00, 1.1421E+05, &
      1.069000E-02, 6.12159E+00, 1.3573E+00, 2.5787E+04, &
      1.142761E-02, 5.67907E+00, 1.2692E+00, 2.2557E+04, &
      1.145657E-02, 5.64148E+00, 1.2659E+00, 2.2441E+04, &
      1.163193E-02, 5.19902E+00, 1.2463E+00, 2.1759E+04, &
      1.167869E-02, 4.68408E+00, 1.2411E+00, 2.1582E+04, &
      1.174883E-02, 5.21287E+00, 2.2376E+00, 3.8679E+04, &
      1.192419E-02, 5.66050E+00, 2.1885E+00, 3.7273E+04, &
      1.221612E-02, 5.90044E+00, 2.1105E+00, 3.5086E+04, &
      1.305903E-02, 6.07869E+00, 1.9076E+00, 2.9667E+04, &
      1.396010E-02, 6.02655E+00, 1.7018E+00, 2.4757E+04, &
      1.492335E-02, 5.81490E+00, 1.4986E+00, 2.0394E+04, &
      1.595306E-02, 5.45014E+00, 1.3107E+00, 1.6685E+04, &
      1.705382E-02, 4.89700E+00, 1.1436E+00, 1.3619E+04, &
      1.823053E-02, 4.02014E+00, 9.9837E-01, 1.1122E+04, &
      1.881600E-02, 3.29032E+00, 9.3683E-01, 1.0112E+04, &
      1.910400E-02, 2.51533E+00, 9.0880E-01, 9.6613E+03, &
      1.918080E-02, 1.79146E+00, 9.0157E-01, 9.5460E+03, &
      1.929600E-02, 2.34505E+00, 2.1978E+00, 2.3132E+04, &
      1.948844E-02, 2.64261E+00, 2.2270E+00, 2.3207E+04, &
      1.958400E-02, 2.68664E+00, 2.2421E+00, 2.3251E+04, &
      2.083314E-02, 2.28932E+00, 2.4791E+00, 2.4168E+04, &
      2.136400E-02, 1.86513E+00, 2.6027E+00, 2.4742E+04, &
      2.169100E-02, 1.37937E+00, 2.6896E+00, 2.5182E+04, &
      2.177820E-02, 9.80532E-01, 2.7143E+00, 2.5312E+04, &
      2.190900E-02, 1.22209E+00, 3.4302E+00, 3.1797E+04, &
      2.223600E-02, 1.30011E+00, 3.5893E+00, 3.2783E+04, &
      2.227063E-02, 1.29420E+00, 3.6069E+00, 3.2892E+04, &
      2.380730E-02, 6.54426E-01, 4.5357E+00, 3.8692E+04, &
      2.545001E-02, -1.46130E-01, 5.8566E+00, 4.6736E+04, &
      2.720606E-02, -8.49767E-01, 7.6272E+00, 5.6937E+04, &
      2.908327E-02, -1.26341E+00, 9.8580E+00, 6.8839E+04, &
      3.109002E-02, -1.18520E+00, 1.2488E+01, 8.1576E+04, &
      3.323523E-02, -4.30245E-01, 1.5367E+01, 9.3901E+04, &
      3.552846E-02, 1.12440E+00, 1.8258E+01, 1.0437E+05, &
      3.797993E-02, 3.49835E+00, 2.0870E+01, 1.1160E+05, &
      4.060054E-02, 6.58400E+00, 2.2914E+01, 1.1462E+05, &
      4.340198E-02, 1.01580E+01, 2.4161E+01, 1.1306E+05, &
      4.639671E-02, 1.39223E+01, 2.4492E+01, 1.0721E+05, &
      4.959809E-02, 1.75636E+01, 2.3914E+01, 9.7922E+04, &
      5.302035E-02, 2.08108E+01, 2.2553E+01, 8.6389E+04, &
      5.667876E-02, 2.34749E+01, 2.0610E+01, 7.3850E+04, &
      6.058959E-02, 2.54864E+01, 1.8316E+01, 6.1393E+04, &
      6.477028E-02, 2.67900E+01, 1.5890E+01, 4.9824E+04, &
      6.923942E-02, 2.74834E+01, 1.3511E+01, 3.9629E+04, &
      7.401695E-02, 2.75980E+01, 1.1304E+01, 3.1017E+04, &
      7.912411E-02, 2.71926E+01, 9.3444E+00, 2.3985E+04, &
      8.428000E-02, 2.60022E+01, 7.7509E+00, 1.8677E+04, &
      8.458368E-02, 2.58519E+01, 7.6677E+00, 1.8411E+04, &
      8.557000E-02, 2.49747E+01, 7.4054E+00, 1.7576E+04, &
      8.591400E-02, 2.38305E+01, 7.3165E+00, 1.7295E+04, &
      8.643000E-02, 2.49774E+01, 9.3836E+00, 2.2049E+04, &
      8.772000E-02, 2.58943E+01, 8.9716E+00, 2.0771E+04, &
      9.041995E-02, 2.63161E+01, 8.1838E+00, 1.8381E+04, &
      9.665893E-02, 2.60953E+01, 6.6942E+00, 1.4065E+04, &
      1.027040E-01, 2.51822E+01, 5.6160E+00, 1.1106E+04, &
      1.033284E-01, 2.49953E+01, 5.5211E+00, 1.0853E+04, &
      1.042760E-01, 2.45098E+01, 5.3820E+00, 1.0483E+04, &
      1.046952E-01, 2.38244E+01, 5.3224E+00, 1.0326E+04, &
      1.053240E-01, 2.44499E+01, 6.5140E+00, 1.2562E+04, &
      1.068960E-01, 2.49052E+01, 6.2743E+00, 1.1922E+04, &
      1.104581E-01, 2.50147E+01, 5.7855E+00, 1.0638E+04, &
      1.180797E-01, 2.46519E+01, 4.9423E+00, 8.5012E+03, &
      1.262272E-01, 2.40125E+01, 4.2469E+00, 6.8338E+03, &
      1.349368E-01, 2.30915E+01, 3.6741E+00, 5.5307E+03, &
      1.353380E-01, 2.30368E+01, 3.6512E+00, 5.4800E+03, &
      1.374095E-01, 2.26752E+01, 3.5374E+00, 5.2293E+03, &
      1.379619E-01, 2.24527E+01, 3.5082E+00, 5.1654E+03, &
      1.387905E-01, 2.25043E+01, 3.7780E+00, 5.5292E+03, &
      1.400420E-01, 2.24463E+01, 3.7306E+00, 5.4111E+03, &
      1.408620E-01, 2.23694E+01, 3.7011E+00, 5.3371E+03, &
      1.421855E-01, 2.21897E+01, 3.6561E+00, 5.2232E+03, &
      1.427571E-01, 2.20244E+01, 3.6377E+00, 5.1762E+03, &
      1.436145E-01, 2.20538E+01, 3.8378E+00, 5.4283E+03, &
      1.442475E-01, 2.20363E+01, 3.8247E+00, 5.3861E+03, &
      1.443540E-01, 2.20310E+01, 3.8226E+00, 5.3791E+03, &
      1.457580E-01, 2.19311E+01, 3.7968E+00, 5.2913E+03, &
      1.465635E-01, 2.18494E+01, 3.7837E+00, 5.2441E+03, &
      1.471527E-01, 2.17475E+01, 3.7750E+00, 5.2111E+03, &
      1.480365E-01, 2.17399E+01, 3.8797E+00, 5.3236E+03, &
      1.502460E-01, 2.16148E+01, 3.8586E+00, 5.2170E+03, &
      1.542005E-01, 2.13110E+01, 3.8440E+00, 5.0640E+03, &
      1.648404E-01, 2.03907E+01, 3.9496E+00, 4.8674E+03, &
      1.762144E-01, 1.93663E+01, 4.2868E+00, 4.9423E+03, &
      1.883732E-01, 1.82872E+01, 4.8926E+00, 5.2766E+03, &
      2.013709E-01, 1.72039E+01, 5.8050E+00, 5.8565E+03, &
      2.152655E-01, 1.61834E+01, 7.0547E+00, 6.6579E+03, &
      2.301188E-01, 1.53101E+01, 8.6547E+00, 7.6406E+03, &
      2.459970E-01, 1.46794E+01, 1.0591E+01, 8.7463E+03, &
      2.629708E-01, 1.43928E+01, 1.2815E+01, 9.8998E+03, &
      2.811158E-01, 1.45214E+01, 1.5243E+01, 1.1015E+04, &
      3.005128E-01, 1.51233E+01, 1.7763E+01, 1.2009E+04, &
      3.212482E-01, 1.62088E+01, 2.0243E+01, 1.2801E+04, &
      3.434143E-01, 1.77399E+01, 2.2547E+01, 1.3338E+04, &
      3.671099E-01, 1.96145E+01, 2.4557E+01, 1.3590E+04, &
      3.924405E-01, 2.16247E+01, 2.6183E+01, 1.3555E+04, &
      4.046420E-01, 2.24077E+01, 2.6778E+01, 1.3445E+04, &
      4.108355E-01, 2.25013E+01, 2.7036E+01, 1.3370E+04, &
      4.124871E-01, 2.21556E+01, 2.7101E+01, 1.3348E+04, &
      4.149645E-01, 2.27816E+01, 2.8126E+01, 1.3770E+04, &
      4.195189E-01, 2.34580E+01, 2.8328E+01, 1.3719E+04, &
      4.211580E-01, 2.36418E+01, 2.8398E+01, 1.3699E+04, &
      4.264960E-01, 2.41533E+01, 2.8613E+01, 1.3630E+04, &
      4.330240E-01, 2.45440E+01, 2.8852E+01, 1.3538E+04, &
      4.347648E-01, 2.43947E+01, 2.8912E+01, 1.3512E+04, &
      4.373760E-01, 2.49579E+01, 2.9630E+01, 1.3764E+04, &
      4.439040E-01, 2.58547E+01, 2.9868E+01, 1.3671E+04, &
      4.484657E-01, 2.63693E+01, 3.0021E+01, 1.3601E+04, &
      4.794098E-01, 2.94526E+01, 3.0808E+01, 1.3057E+04, &
      5.0000000E-01, 3.29286E+01, 3.1090E+01, 1.2634E+04, &
      5.0250000E-01, 3.31489E+01, 3.1112E+01, 1.2581E+04, &
      5.0501250E-01, 3.33686E+01, 3.1131E+01, 1.2526E+04, &
      5.0753756E-01, 3.35874E+01, 3.1149E+01, 1.2471E+04, &
      5.1007525E-01, 3.38055E+01, 3.1163E+01, 1.2414E+04, &
      5.1262563E-01, 3.40228E+01, 3.1176E+01, 1.2358E+04, &
      5.1518875E-01, 3.42391E+01, 3.1186E+01, 1.2300E+04, &
      5.1776470E-01, 3.44546E+01, 3.1194E+01, 1.2242E+04, &
      5.2035352E-01, 3.46691E+01, 3.1200E+01, 1.2184E+04, &
      5.2295529E-01, 3.48826E+01, 3.1204E+01, 1.2125E+04, &
      5.2557007E-01, 3.50951E+01, 3.1205E+01, 1.2065E+04, &
      5.2819792E-01, 3.53065E+01, 3.1204E+01, 1.2005E+04, &
      5.3083891E-01, 3.55167E+01, 3.1201E+01, 1.1944E+04, &
      5.3349310E-01, 3.57259E+01, 3.1196E+01, 1.1883E+04, &
      5.3616057E-01, 3.59338E+01, 3.1189E+01, 1.1821E+04, &
      5.3884137E-01, 3.61404E+01, 3.1180E+01, 1.1759E+04, &
      5.4153558E-01, 3.63457E+01, 3.1169E+01, 1.1696E+04, &
      5.4424325E-01, 3.65497E+01, 3.1155E+01, 1.1633E+04, &
      5.4696447E-01, 3.67523E+01, 3.1140E+01, 1.1569E+04, &
      5.4969929E-01, 3.69535E+01, 3.1123E+01, 1.1506E+04, &
      5.5244779E-01, 3.71531E+01, 3.1104E+01, 1.1441E+04, &
      5.5521003E-01, 3.73512E+01, 3.1083E+01, 1.1377E+04, &
      5.5798608E-01, 3.75477E+01, 3.1059E+01, 1.1312E+04, &
      5.6077601E-01, 3.77424E+01, 3.1034E+01, 1.1246E+04, &
      5.6357989E-01, 3.79353E+01, 3.1007E+01, 1.1181E+04, &
      5.6639779E-01, 3.81263E+01, 3.0978E+01, 1.1115E+04, &
      5.6922978E-01, 3.83153E+01, 3.0946E+01, 1.1048E+04, &
      5.7207593E-01, 3.85022E+01, 3.0913E+01, 1.0982E+04, &
      5.7493630E-01, 3.86869E+01, 3.0878E+01, 1.0915E+04, &
      5.7781099E-01, 3.88693E+01, 3.0842E+01, 1.0847E+04, &
      5.8070004E-01, 3.90492E+01, 3.0803E+01, 1.0780E+04, &
      5.8360354E-01, 3.92264E+01, 3.0762E+01, 1.0712E+04, &
      5.8652156E-01, 3.94008E+01, 3.0720E+01, 1.0644E+04, &
      5.8945417E-01, 3.95721E+01, 3.0675E+01, 1.0576E+04, &
      5.9240144E-01, 3.97403E+01, 3.0630E+01, 1.0508E+04, &
      5.9536345E-01, 3.99049E+01, 3.0582E+01, 1.0440E+04, &
      5.9834026E-01, 4.00658E+01, 3.0533E+01, 1.0371E+04, &
      6.0133196E-01, 4.02226E+01, 3.0482E+01, 1.0302E+04, &
      6.0433862E-01, 4.03748E+01, 3.0430E+01, 1.0234E+04, &
      6.0736032E-01, 4.05220E+01, 3.0377E+01, 1.0165E+04, &
      6.1039712E-01, 4.06634E+01, 3.0322E+01, 1.0096E+04, &
      6.1344910E-01, 4.07984E+01, 3.0266E+01, 1.0027E+04, &
      6.1651635E-01, 4.09259E+01, 3.0208E+01, 9.9587E+03, &
      6.1959893E-01, 4.10443E+01, 3.0149E+01, 9.8900E+03, &
      6.2269693E-01, 4.11517E+01, 3.0090E+01, 9.8213E+03, &
      6.2581041E-01, 4.12451E+01, 3.0029E+01, 9.7527E+03, &
      6.2893946E-01, 4.13198E+01, 2.9966E+01, 9.6842E+03, &
      6.3208416E-01, 4.13679E+01, 2.9903E+01, 9.6158E+03, &
      6.3524458E-01, 4.13747E+01, 2.9839E+01, 9.5476E+03, &
      6.3842080E-01, 4.13056E+01, 2.9774E+01, 9.4794E+03, &
      6.4161291E-01, 4.10450E+01, 2.9708E+01, 9.4115E+03, &
      6.4372017E-01, 4.03933E+01, 2.9665E+01, 9.3669E+03, &
      6.4482097E-01, 3.99332E+01, 3.1478E+01, 9.9221E+03, &
      6.4527986E-01, 4.04764E+01, 3.1469E+01, 9.9122E+03, &
      6.4804508E-01, 4.15080E+01, 3.1413E+01, 9.8525E+03, &
      6.5128530E-01, 4.20580E+01, 3.1348E+01, 9.7832E+03, &
      6.5454173E-01, 4.24568E+01, 3.1282E+01, 9.7141E+03, &
      6.5781444E-01, 4.27904E+01, 3.1215E+01, 9.6452E+03, &
      6.6110351E-01, 4.30870E+01, 3.1148E+01, 9.5766E+03, &
      6.6440903E-01, 4.33597E+01, 3.1080E+01, 9.5083E+03, &
      6.6773107E-01, 4.36154E+01, 3.1012E+01, 9.4403E+03, &
      6.7106973E-01, 4.38584E+01, 3.0943E+01, 9.3726E+03, &
      6.7442508E-01, 4.40915E+01, 3.0874E+01, 9.3053E+03, &
      6.7779720E-01, 4.43166E+01, 3.0804E+01, 9.2382E+03, &
      6.8118619E-01, 4.45352E+01, 3.0735E+01, 9.1715E+03, &
      6.8459212E-01, 4.47484E+01, 3.0665E+01, 9.1052E+03, &
      6.8801508E-01, 4.49569E+01, 3.0594E+01, 9.0392E+03, &
      6.9145515E-01, 4.51617E+01, 3.0524E+01, 8.9737E+03, &
      6.9491243E-01, 4.53633E+01, 3.0454E+01, 8.9085E+03, &
      6.9838699E-01, 4.55622E+01, 3.0381E+01, 8.8430E+03, &
      7.0187893E-01, 4.57561E+01, 3.0296E+01, 8.7746E+03, &
      7.0538832E-01, 4.59457E+01, 3.0211E+01, 8.7066E+03, &
      7.0891526E-01, 4.61312E+01, 3.0126E+01, 8.6389E+03, &
      7.1245984E-01, 4.63130E+01, 3.0040E+01, 8.5715E+03, &
      7.1602214E-01, 4.64911E+01, 2.9954E+01, 8.5045E+03, &
      7.1960225E-01, 4.66657E+01, 2.9867E+01, 8.4378E+03, &
      7.2320026E-01, 4.68370E+01, 2.9780E+01, 8.3715E+03, &
      7.2681626E-01, 4.70048E+01, 2.9693E+01, 8.3056E+03, &
      7.3045034E-01, 4.71693E+01, 2.9606E+01, 8.2400E+03, &
      7.3410260E-01, 4.73304E+01, 2.9518E+01, 8.1748E+03, &
      7.3777311E-01, 4.74870E+01, 2.9425E+01, 8.1086E+03, &
      7.4146197E-01, 4.76378E+01, 2.9329E+01, 8.0419E+03, &
      7.4516928E-01, 4.77820E+01, 2.9232E+01, 7.9756E+03, &
      7.4889513E-01, 4.79182E+01, 2.9135E+01, 7.9096E+03, &
      7.5263961E-01, 4.80434E+01, 2.9037E+01, 7.8440E+03, &
      7.5640280E-01, 4.81506E+01, 2.8939E+01, 7.7788E+03, &
      7.6018482E-01, 4.82177E+01, 2.8841E+01, 7.7139E+03, &
      7.6283052E-01, 4.81669E+01, 2.8772E+01, 7.6690E+03, &
      7.6398574E-01, 4.78935E+01, 2.9140E+01, 7.7552E+03, &
      7.6496944E-01, 4.82518E+01, 2.9115E+01, 7.7386E+03, &
      7.6780567E-01, 4.85275E+01, 2.9043E+01, 7.6910E+03, &
      7.7164470E-01, 4.87632E+01, 2.8946E+01, 7.6273E+03, &
      7.7550292E-01, 4.89616E+01, 2.8849E+01, 7.5639E+03, &
      7.7938044E-01, 4.91433E+01, 2.8751E+01, 7.5010E+03, &
      7.8327734E-01, 4.93150E+01, 2.8654E+01, 7.4384E+03, &
      7.8719373E-01, 4.94796E+01, 2.8556E+01, 7.3763E+03, &
      7.9112969E-01, 4.96387E+01, 2.8458E+01, 7.3146E+03, &
      7.9508534E-01, 4.97933E+01, 2.8360E+01, 7.2532E+03, &
      7.9906077E-01, 4.99439E+01, 2.8263E+01, 7.1923E+03, &
      8.0305607E-01, 5.00911E+01, 2.8165E+01, 7.1318E+03, &
      8.0707135E-01, 5.02351E+01, 2.8067E+01, 7.0718E+03, &
      8.1110671E-01, 5.03762E+01, 2.7969E+01, 7.0122E+03, &
      8.1516224E-01, 5.05144E+01, 2.7871E+01, 6.9529E+03, &
      8.1923806E-01, 5.06500E+01, 2.7773E+01, 6.8942E+03, &
      8.2333425E-01, 5.07829E+01, 2.7675E+01, 6.8358E+03, &
      8.2745092E-01, 5.09134E+01, 2.7577E+01, 6.7779E+03, &
      8.3158817E-01, 5.10413E+01, 2.7480E+01, 6.7204E+03, &
      8.3574611E-01, 5.11667E+01, 2.7382E+01, 6.6634E+03, &
      8.3992484E-01, 5.12896E+01, 2.7285E+01, 6.6068E+03, &
      8.4412447E-01, 5.14098E+01, 2.7188E+01, 6.5506E+03, &
      8.4834509E-01, 5.15272E+01, 2.7091E+01, 6.4949E+03, &
      8.5258682E-01, 5.16417E+01, 2.6994E+01, 6.4396E+03, &
      8.5684975E-01, 5.17529E+01, 2.6898E+01, 6.3848E+03, &
      8.6113400E-01, 5.18605E+01, 2.6801E+01, 6.3304E+03, &
      8.6543967E-01, 5.19637E+01, 2.6705E+01, 6.2764E+03, &
      8.6976687E-01, 5.20616E+01, 2.6610E+01, 6.2229E+03, &
      8.7411570E-01, 5.21525E+01, 2.6514E+01, 6.1698E+03, &
      8.7848628E-01, 5.22333E+01, 2.6419E+01, 6.1171E+03, &
      8.8287871E-01, 5.22978E+01, 2.6324E+01, 6.0649E+03, &
      8.8729310E-01, 5.23291E+01, 2.6229E+01, 6.0131E+03, &
      8.9172957E-01, 5.22414E+01, 2.6135E+01, 5.9618E+03, &
      8.9230427E-01, 5.21938E+01, 2.6123E+01, 5.9552E+03, &
      8.9489571E-01, 5.22641E+01, 2.6610E+01, 6.0485E+03, &
      8.9618822E-01, 5.24183E+01, 2.6583E+01, 6.0337E+03, &
      9.0066916E-01, 5.27122E+01, 2.6490E+01, 5.9829E+03, &
      9.0517250E-01, 5.29177E+01, 2.6398E+01, 5.9325E+03, &
      9.0969837E-01, 5.30949E+01, 2.6306E+01, 5.8825E+03, &
      9.1424686E-01, 5.32579E+01, 2.6214E+01, 5.8329E+03, &
      9.1881809E-01, 5.34124E+01, 2.6123E+01, 5.7838E+03, &
      9.2341218E-01, 5.35613E+01, 2.6032E+01, 5.7352E+03, &
      9.2802924E-01, 5.37064E+01, 2.5942E+01, 5.6869E+03, &
      9.3266939E-01, 5.38488E+01, 2.5852E+01, 5.6391E+03, &
      9.3733274E-01, 5.39892E+01, 2.5762E+01, 5.5918E+03, &
      9.4201940E-01, 5.41285E+01, 2.5673E+01, 5.5448E+03, &
      9.4672950E-01, 5.42670E+01, 2.5585E+01, 5.4984E+03, &
      9.5146315E-01, 5.44053E+01, 2.5497E+01, 5.4523E+03, &
      9.5622046E-01, 5.45439E+01, 2.5409E+01, 5.4066E+03, &
      9.6100156E-01, 5.46831E+01, 2.5322E+01, 5.3614E+03, &
      9.6580657E-01, 5.48235E+01, 2.5236E+01, 5.3166E+03, &
      9.7063560E-01, 5.49656E+01, 2.5150E+01, 5.2723E+03, &
      9.7548878E-01, 5.51099E+01, 2.5065E+01, 5.2283E+03, &
      9.8036623E-01, 5.52571E+01, 2.4980E+01, 5.1848E+03, &
      9.8526806E-01, 5.54081E+01, 2.4895E+01, 5.1417E+03, &
      9.9019440E-01, 5.55638E+01, 2.4812E+01, 5.0990E+03, &
      9.9514537E-01, 5.57256E+01, 2.4728E+01, 5.0567E+03, &
      1.0001211E+00, 5.58951E+01, 2.4644E+01, 5.0145E+03, &
      1.0051217E+00, 5.60068E+01, 2.4499E+01, 4.9604E+03, &
      1.0101473E+00, 5.61129E+01, 2.4356E+01, 4.9069E+03, &
      1.0151980E+00, 5.62144E+01, 2.4213E+01, 4.8541E+03, &
      1.0202740E+00, 5.63117E+01, 2.4072E+01, 4.8019E+03, &
      1.0253754E+00, 5.64053E+01, 2.3932E+01, 4.7503E+03, &
      1.0305023E+00, 5.64957E+01, 2.3793E+01, 4.6994E+03, &
      1.0356548E+00, 5.65830E+01, 2.3655E+01, 4.6490E+03, &
      1.0408331E+00, 5.66674E+01, 2.3518E+01, 4.5993E+03, &
      1.0460372E+00, 5.67493E+01, 2.3383E+01, 4.5501E+03, &
      1.0512674E+00, 5.68287E+01, 2.3248E+01, 4.5016E+03, &
      1.0565238E+00, 5.69058E+01, 2.3114E+01, 4.4536E+03, &
      1.0618064E+00, 5.69807E+01, 2.2982E+01, 4.4061E+03, &
      1.0671154E+00, 5.70536E+01, 2.2851E+01, 4.3593E+03, &
      1.0724510E+00, 5.71244E+01, 2.2720E+01, 4.3130E+03, &
      1.0778132E+00, 5.71934E+01, 2.2591E+01, 4.2672E+03, &
      1.0832023E+00, 5.72607E+01, 2.2463E+01, 4.2220E+03, &
      1.0886183E+00, 5.73263E+01, 2.2336E+01, 4.1774E+03, &
      1.0940614E+00, 5.73899E+01, 2.2208E+01, 4.1329E+03, &
      1.0995317E+00, 5.74517E+01, 2.2080E+01, 4.0889E+03, &
      1.1050294E+00, 5.75116E+01, 2.1954E+01, 4.0454E+03, &
      1.1105545E+00, 5.75698E+01, 2.1829E+01, 4.0025E+03, &
      1.1161073E+00, 5.76263E+01, 2.1704E+01, 3.9600E+03, &
      1.1216878E+00, 5.76812E+01, 2.1581E+01, 3.9181E+03, &
      1.1272963E+00, 5.77344E+01, 2.1459E+01, 3.8766E+03, &
      1.1329328E+00, 5.77862E+01, 2.1338E+01, 3.8356E+03, &
      1.1385974E+00, 5.78364E+01, 2.1217E+01, 3.7951E+03, &
      1.1442904E+00, 5.78852E+01, 2.1098E+01, 3.7551E+03, &
      1.1500119E+00, 5.79326E+01, 2.0979E+01, 3.7156E+03, &
      1.1557619E+00, 5.79787E+01, 2.0862E+01, 3.6765E+03, &
      1.1615407E+00, 5.80234E+01, 2.0745E+01, 3.6379E+03, &
      1.1673484E+00, 5.80667E+01, 2.0629E+01, 3.5997E+03, &
      1.1731852E+00, 5.81089E+01, 2.0514E+01, 3.5620E+03, &
      1.1790511E+00, 5.81497E+01, 2.0400E+01, 3.5247E+03, &
      1.1849464E+00, 5.81894E+01, 2.0287E+01, 3.4879E+03, &
      1.1908711E+00, 5.82279E+01, 2.0175E+01, 3.4515E+03, &
      1.1968254E+00, 5.82652E+01, 2.0064E+01, 3.4155E+03, &
      1.2028096E+00, 5.83014E+01, 1.9953E+01, 3.3799E+03, &
      1.2088236E+00, 5.83364E+01, 1.9844E+01, 3.3448E+03, &
      1.2148677E+00, 5.83704E+01, 1.9735E+01, 3.3100E+03, &
      1.2209421E+00, 5.84033E+01, 1.9627E+01, 3.2757E+03, &
      1.2270468E+00, 5.84352E+01, 1.9520E+01, 3.2417E+03, &
      1.2331820E+00, 5.84660E+01, 1.9414E+01, 3.2082E+03, &
      1.2393479E+00, 5.85221E+01, 1.9309E+01, 3.1751E+03, &
      1.2455447E+00, 5.85507E+01, 1.9204E+01, 3.1423E+03, &
      1.2517724E+00, 5.85784E+01, 1.9101E+01, 3.1099E+03, &
      1.2580312E+00, 5.86051E+01, 1.8998E+01, 3.0779E+03, &
      1.2643214E+00, 5.86310E+01, 1.8896E+01, 3.0462E+03, &
      1.2706430E+00, 5.86559E+01, 1.8794E+01, 3.0150E+03, &
      1.2769962E+00, 5.86801E+01, 1.8694E+01, 2.9841E+03, &
      1.2833812E+00, 5.87034E+01, 1.8594E+01, 2.9535E+03, &
      1.2897981E+00, 5.87259E+01, 1.8495E+01, 2.9233E+03, &
      1.2962471E+00, 5.87475E+01, 1.8396E+01, 2.8933E+03, &
      1.3027283E+00, 5.87681E+01, 1.8298E+01, 2.8636E+03, &
      1.3092420E+00, 5.87986E+01, 1.8200E+01, 2.8342E+03, &
      1.3157882E+00, 5.88173E+01, 1.8103E+01, 2.8052E+03, &
      1.3223671E+00, 5.88351E+01, 1.8006E+01, 2.7765E+03, &
      1.3289790E+00, 5.88520E+01, 1.7911E+01, 2.7482E+03, &
      1.3356239E+00, 5.88681E+01, 1.7816E+01, 2.7202E+03, &
      1.3423020E+00, 5.88833E+01, 1.7722E+01, 2.6924E+03, &
      1.3490135E+00, 5.88976E+01, 1.7628E+01, 2.6650E+03, &
      1.3557586E+00, 5.89111E+01, 1.7535E+01, 2.6380E+03, &
      1.3625374E+00, 5.89237E+01, 1.7443E+01, 2.6112E+03, &
      1.3693500E+00, 5.89356E+01, 1.7352E+01, 2.5847E+03, &
      1.3761968E+00, 5.89466E+01, 1.7261E+01, 2.5585E+03, &
      1.3830778E+00, 5.89568E+01, 1.7171E+01, 2.5327E+03, &
      1.3899932E+00, 5.89662E+01, 1.7082E+01, 2.5071E+03, &
      1.3969431E+00, 5.89748E+01, 1.6994E+01, 2.4818E+03, &
      1.4039278E+00, 5.89826E+01, 1.6906E+01, 2.4568E+03, &
      1.4109475E+00, 5.89896E+01, 1.6818E+01, 2.4321E+03, &
      1.4180022E+00, 5.89959E+01, 1.6732E+01, 2.4077E+03, &
      1.4250922E+00, 5.90014E+01, 1.6646E+01, 2.3835E+03, &
      1.4322177E+00, 5.90062E+01, 1.6561E+01, 2.3596E+03, &
      1.4393788E+00, 5.90102E+01, 1.6476E+01, 2.3360E+03, &
      1.4465757E+00, 5.90134E+01, 1.6392E+01, 2.3126E+03, &
      1.4538086E+00, 5.90159E+01, 1.6308E+01, 2.2895E+03, &
      1.4610776E+00, 5.90177E+01, 1.6226E+01, 2.2667E+03, &
      1.4683830E+00, 5.90187E+01, 1.6144E+01, 2.2441E+03, &
      1.4757249E+00, 5.90190E+01, 1.6062E+01, 2.2218E+03, &
      1.4831035E+00, 5.90186E+01, 1.5981E+01, 2.1998E+03, &
      1.4905190E+00, 5.90174E+01, 1.5901E+01, 2.1779E+03, &
      1.4979716E+00, 5.90156E+01, 1.5821E+01, 2.1564E+03, &
      1.5054615E+00, 5.90130E+01, 1.5742E+01, 2.1350E+03, &
      1.5129888E+00, 5.90098E+01, 1.5663E+01, 2.1139E+03, &
      1.5205537E+00, 5.90057E+01, 1.5585E+01, 2.0930E+03, &
      1.5281565E+00, 5.90009E+01, 1.5507E+01, 2.0723E+03, &
      1.5357973E+00, 5.89953E+01, 1.5430E+01, 2.0518E+03, &
      1.5434763E+00, 5.89889E+01, 1.5353E+01, 2.0316E+03, &
      1.5511937E+00, 5.89817E+01, 1.5277E+01, 2.0115E+03, &
      1.5589496E+00, 5.89737E+01, 1.5201E+01, 1.9917E+03, &
      1.5667444E+00, 5.89650E+01, 1.5126E+01, 1.9722E+03, &
      1.5745781E+00, 5.89556E+01, 1.5052E+01, 1.9528E+03, &
      1.5824510E+00, 5.89454E+01, 1.4978E+01, 1.9337E+03, &
      1.5903633E+00, 5.89344E+01, 1.4904E+01, 1.9147E+03, &
      1.5983151E+00, 5.89227E+01, 1.4831E+01, 1.8960E+03, &
      1.6063066E+00, 5.89103E+01, 1.4759E+01, 1.8775E+03, &
      1.6143382E+00, 5.88971E+01, 1.4687E+01, 1.8592E+03, &
      1.6224099E+00, 5.88832E+01, 1.4616E+01, 1.8410E+03, &
      1.6305219E+00, 5.88685E+01, 1.4545E+01, 1.8231E+03, &
      1.6386745E+00, 5.88531E+01, 1.4474E+01, 1.8053E+03, &
      1.6468679E+00, 5.88370E+01, 1.4404E+01, 1.7878E+03, &
      1.6551022E+00, 5.88200E+01, 1.4330E+01, 1.7699E+03, &
      1.6633777E+00, 5.88017E+01, 1.4252E+01, 1.7516E+03, &
      1.6716946E+00, 5.87819E+01, 1.4173E+01, 1.7334E+03, &
      1.6800531E+00, 5.87606E+01, 1.4096E+01, 1.7155E+03, &
      1.6884534E+00, 5.87378E+01, 1.4019E+01, 1.6977E+03, &
      1.6968956E+00, 5.87136E+01, 1.3943E+01, 1.6802E+03, &
      1.7053801E+00, 5.86879E+01, 1.3867E+01, 1.6629E+03, &
      1.7139070E+00, 5.86608E+01, 1.3791E+01, 1.6457E+03, &
      1.7224766E+00, 5.86322E+01, 1.3717E+01, 1.6288E+03, &
      1.7310889E+00, 5.86022E+01, 1.3643E+01, 1.6121E+03, &
      1.7397444E+00, 5.85707E+01, 1.3569E+01, 1.5955E+03, &
      1.7484431E+00, 5.85376E+01, 1.3490E+01, 1.5785E+03, &
      1.7571853E+00, 5.85025E+01, 1.3411E+01, 1.5616E+03, &
      1.7659712E+00, 5.84654E+01, 1.3333E+01, 1.5448E+03, &
      1.7748011E+00, 5.84263E+01, 1.3255E+01, 1.5283E+03, &
      1.7836751E+00, 5.83851E+01, 1.3177E+01, 1.5119E+03, &
      1.7925935E+00, 5.83420E+01, 1.3101E+01, 1.4958E+03, &
      1.8015565E+00, 5.82968E+01, 1.3024E+01, 1.4798E+03, &
      1.8105642E+00, 5.82495E+01, 1.2949E+01, 1.4640E+03, &
      1.8196171E+00, 5.82002E+01, 1.2874E+01, 1.4484E+03, &
      1.8287151E+00, 5.81487E+01, 1.2799E+01, 1.4330E+03, &
      1.8378587E+00, 5.80950E+01, 1.2725E+01, 1.4178E+03, &
      1.8470480E+00, 5.80391E+01, 1.2652E+01, 1.4027E+03, &
      1.8562833E+00, 5.79810E+01, 1.2579E+01, 1.3878E+03, &
      1.8655647E+00, 5.79206E+01, 1.2507E+01, 1.3731E+03, &
      1.8748925E+00, 5.78579E+01, 1.2435E+01, 1.3585E+03, &
      1.8842670E+00, 5.77928E+01, 1.2364E+01, 1.3441E+03, &
      1.8936883E+00, 5.77252E+01, 1.2293E+01, 1.3299E+03, &
      1.9031567E+00, 5.76550E+01, 1.2223E+01, 1.3159E+03, &
      1.9126725E+00, 5.75823E+01, 1.2153E+01, 1.3020E+03, &
      1.9222359E+00, 5.75069E+01, 1.2084E+01, 1.2882E+03, &
      1.9318471E+00, 5.74287E+01, 1.2015E+01, 1.2746E+03, &
      1.9415063E+00, 5.73475E+01, 1.1947E+01, 1.2612E+03, &
      1.9512138E+00, 5.72635E+01, 1.1879E+01, 1.2480E+03, &
      1.9609699E+00, 5.71765E+01, 1.1812E+01, 1.2348E+03, &
      1.9707747E+00, 5.70864E+01, 1.1745E+01, 1.2219E+03, &
      1.9806286E+00, 5.69930E+01, 1.1678E+01, 1.2090E+03, &
      1.9905318E+00, 5.68961E+01, 1.1612E+01, 1.1963E+03, &
      2.0004844E+00, 5.67957E+01, 1.1547E+01, 1.1838E+03, &
      2.0104868E+00, 5.66916E+01, 1.1482E+01, 1.1714E+03, &
      2.0205393E+00, 5.65837E+01, 1.1417E+01, 1.1591E+03, &
      2.0306420E+00, 5.64717E+01, 1.1353E+01, 1.1470E+03, &
      2.0407952E+00, 5.63556E+01, 1.1290E+01, 1.1350E+03, &
      2.0509992E+00, 5.62350E+01, 1.1227E+01, 1.1232E+03, &
      2.0612542E+00, 5.61098E+01, 1.1164E+01, 1.1115E+03, &
      2.0715604E+00, 5.59797E+01, 1.1102E+01, 1.0999E+03, &
      2.0819182E+00, 5.58445E+01, 1.1040E+01, 1.0884E+03, &
      2.0923278E+00, 5.57040E+01, 1.0978E+01, 1.0771E+03, &
      2.1027895E+00, 5.55577E+01, 1.0917E+01, 1.0659E+03, &
      2.1133034E+00, 5.54054E+01, 1.0857E+01, 1.0548E+03, &
      2.1238699E+00, 5.52467E+01, 1.0797E+01, 1.0439E+03, &
      2.1344893E+00, 5.50813E+01, 1.0737E+01, 1.0331E+03, &
      2.1451617E+00, 5.49086E+01, 1.0678E+01, 1.0224E+03, &
      2.1558875E+00, 5.47283E+01, 1.0619E+01, 1.0118E+03, &
      2.1666670E+00, 5.45397E+01, 1.0561E+01, 1.0014E+03, &
      2.1775003E+00, 5.43424E+01, 1.0503E+01, 9.9102E+02, &
      2.1883878E+00, 5.41355E+01, 1.0445E+01, 9.8079E+02, &
      2.1993297E+00, 5.39185E+01, 1.0388E+01, 9.7068E+02, &
      2.2103264E+00, 5.36905E+01, 1.0331E+01, 9.6069E+02, &
      2.2213780E+00, 5.34505E+01, 1.0275E+01, 9.5081E+02, &
      2.2324849E+00, 5.31975E+01, 1.0219E+01, 9.4103E+02, &
      2.2436473E+00, 5.29304E+01, 1.0163E+01, 9.3137E+02, &
      2.2548656E+00, 5.26477E+01, 1.0108E+01, 9.2181E+02, &
      2.2661399E+00, 5.23478E+01, 1.0053E+01, 9.1236E+02, &
      2.2774706E+00, 5.20290E+01, 9.9987E+00, 9.0302E+02, &
      2.2888579E+00, 5.16891E+01, 9.9446E+00, 8.9378E+02, &
      2.3003022E+00, 5.13286E+01, 9.8909E+00, 8.8464E+02, &
      2.3118037E+00, 5.09384E+01, 9.8376E+00, 8.7560E+02, &
      2.3233628E+00, 5.05178E+01, 9.7847E+00, 8.6667E+02, &
      2.3349796E+00, 5.00624E+01, 9.7321E+00, 8.5783E+02, &
      2.3466545E+00, 4.95667E+01, 9.6799E+00, 8.4910E+02, &
      2.3583878E+00, 4.90235E+01, 9.6280E+00, 8.4046E+02, &
      2.3701797E+00, 4.84238E+01, 9.5765E+00, 8.3191E+02, &
      2.3820306E+00, 4.77555E+01, 9.5254E+00, 8.2346E+02, &
      2.3939407E+00, 4.70021E+01, 9.4746E+00, 8.1510E+02, &
      2.4059104E+00, 4.61403E+01, 9.4242E+00, 8.0684E+02, &
      2.4179400E+00, 4.51349E+01, 9.3741E+00, 7.9866E+02, &
      2.4300297E+00, 4.39297E+01, 9.3244E+00, 7.9058E+02, &
      2.4421798E+00, 4.24256E+01, 9.2750E+00, 7.8259E+02, &
      2.4543907E+00, 4.04199E+01, 9.2260E+00, 7.7468E+02, &
      2.4666627E+00, 3.73738E+01, 9.1772E+00, 7.6686E+02, &
      2.4789960E+00, 3.04980E+01, 9.1289E+00, 7.5913E+02, &
      2.4833243E+00, 1.96236E+01, 9.1120E+00, 7.5644E+02, &
      2.4846756E+00, 1.93319E+01, 2.6093E+01, 2.1440E+03, &
      2.4913910E+00, 3.21831E+01, 2.5990E+01, 2.1298E+03, &
      2.5038479E+00, 3.72476E+01, 2.5800E+01, 2.1039E+03, &
      2.5163672E+00, 3.94971E+01, 2.5611E+01, 2.0782E+03, &
      2.5289490E+00, 4.07389E+01, 2.5424E+01, 2.0529E+03, &
      2.5415938E+00, 4.13497E+01, 2.5238E+01, 2.0279E+03, &
      2.5543017E+00, 4.13776E+01, 2.5054E+01, 2.0032E+03, &
      2.5670732E+00, 4.05726E+01, 2.4872E+01, 1.9789E+03, &
      2.5799086E+00, 3.73004E+01, 2.4690E+01, 1.9548E+03, &
      2.5848604E+00, 3.03317E+01, 2.4620E+01, 1.9456E+03, &
      2.5863394E+00, 3.02515E+01, 3.5872E+01, 2.8280E+03, &
      2.5928082E+00, 3.88634E+01, 3.5735E+01, 2.8103E+03, &
      2.6057722E+00, 4.33327E+01, 3.5462E+01, 2.7750E+03, &
      2.6188011E+00, 4.58091E+01, 3.5190E+01, 2.7402E+03, &
      2.6318951E+00, 4.76229E+01, 3.4921E+01, 2.7058E+03, &
      2.6450545E+00, 4.90858E+01, 3.4654E+01, 2.6718E+03, &
      2.6582798E+00, 5.03245E+01, 3.4389E+01, 2.6383E+03, &
      2.6715712E+00, 5.14043E+01, 3.4126E+01, 2.6053E+03, &
      2.6849291E+00, 5.23620E+01, 3.3865E+01, 2.5726E+03, &
      2.6983537E+00, 5.32260E+01, 3.3605E+01, 2.5403E+03, &
      2.7118455E+00, 5.40118E+01, 3.3346E+01, 2.5084E+03, &
      2.7254047E+00, 5.47318E+01, 3.3090E+01, 2.4769E+03, &
      2.7390317E+00, 5.53953E+01, 3.2837E+01, 2.4457E+03, &
      2.7527269E+00, 5.60094E+01, 3.2585E+01, 2.4150E+03, &
      2.7664905E+00, 5.65799E+01, 3.2335E+01, 2.3847E+03, &
      2.7803230E+00, 5.71111E+01, 3.2088E+01, 2.3548E+03, &
      2.7942246E+00, 5.76066E+01, 3.1842E+01, 2.3253E+03, &
      2.8081957E+00, 5.80693E+01, 3.1598E+01, 2.2962E+03, &
      2.8222367E+00, 5.85016E+01, 3.1357E+01, 2.2674E+03, &
      2.8363479E+00, 5.89052E+01, 3.1117E+01, 2.2390E+03, &
      2.8505296E+00, 5.92816E+01, 3.0879E+01, 2.2109E+03, &
      2.8647823E+00, 5.96318E+01, 3.0644E+01, 2.1833E+03, &
      2.8791062E+00, 5.99563E+01, 3.0410E+01, 2.1560E+03, &
      2.8935017E+00, 6.02554E+01, 3.0178E+01, 2.1290E+03, &
      2.9079692E+00, 6.05288E+01, 2.9948E+01, 2.1024E+03, &
      2.9225091E+00, 6.07755E+01, 2.9720E+01, 2.0761E+03, &
      2.9371216E+00, 6.09939E+01, 2.9494E+01, 2.0502E+03, &
      2.9518072E+00, 6.11812E+01, 2.9269E+01, 2.0246E+03, &
      2.9665662E+00, 6.13329E+01, 2.9047E+01, 1.9993E+03, &
      2.9813991E+00, 6.14420E+01, 2.8826E+01, 1.9744E+03, &
      2.9963061E+00, 6.14964E+01, 2.8607E+01, 1.9498E+03, &
      3.0112876E+00, 6.14556E+01, 2.8374E+01, 1.9244E+03, &
      3.0263440E+00, 6.12783E+01, 2.8137E+01, 1.8989E+03, &
      3.0414758E+00, 6.08558E+01, 2.7902E+01, 1.8739E+03, &
      3.0566831E+00, 5.97132E+01, 2.7669E+01, 1.8491E+03, &
      3.0632417E+00, 5.81196E+01, 2.7569E+01, 1.8385E+03, &
      3.0695584E+00, 5.81766E+01, 3.2297E+01, 2.1475E+03, &
      3.0719666E+00, 5.90977E+01, 3.2253E+01, 2.1429E+03, &
      3.0873264E+00, 6.14244E+01, 3.1972E+01, 2.1138E+03, &
      3.1027630E+00, 6.25402E+01, 3.1694E+01, 2.0852E+03, &
      3.1182768E+00, 6.33354E+01, 3.1418E+01, 2.0569E+03, &
      3.1338682E+00, 6.39735E+01, 3.1145E+01, 2.0290E+03, &
      3.1495376E+00, 6.45153E+01, 3.0874E+01, 2.0014E+03, &
      3.1652853E+00, 6.49900E+01, 3.0606E+01, 1.9743E+03, &
      3.1811117E+00, 6.54139E+01, 3.0340E+01, 1.9475E+03, &
      3.1970172E+00, 6.57967E+01, 3.0076E+01, 1.9211E+03, &
      3.2130023E+00, 6.61440E+01, 2.9814E+01, 1.8950E+03, &
      3.2290673E+00, 6.64611E+01, 2.9568E+01, 1.8702E+03, &
      3.2452127E+00, 6.67587E+01, 2.9324E+01, 1.8457E+03, &
      3.2614387E+00, 6.70379E+01, 2.9084E+01, 1.8215E+03, &
      3.2777459E+00, 6.72999E+01, 2.8847E+01, 1.7978E+03, &
      3.2941347E+00, 6.75459E+01, 2.8612E+01, 1.7744E+03, &
      3.3106053E+00, 6.77769E+01, 2.8381E+01, 1.7514E+03, &
      3.3271584E+00, 6.79937E+01, 2.8152E+01, 1.7288E+03, &
      3.3437941E+00, 6.81970E+01, 2.7926E+01, 1.7065E+03, &
      3.3605131E+00, 6.83870E+01, 2.7703E+01, 1.6846E+03, &
      3.3773157E+00, 6.85632E+01, 2.7479E+01, 1.6628E+03, &
      3.3942023E+00, 6.87250E+01, 2.7258E+01, 1.6413E+03, &
      3.4111733E+00, 6.88717E+01, 2.7039E+01, 1.6201E+03, &
      3.4282291E+00, 6.90023E+01, 2.6823E+01, 1.5993E+03, &
      3.4453703E+00, 6.91148E+01, 2.6609E+01, 1.5787E+03, &
      3.4625971E+00, 6.92058E+01, 2.6397E+01, 1.5585E+03, &
      3.4799101E+00, 6.92696E+01, 2.6188E+01, 1.5386E+03, &
      3.4973097E+00, 6.92952E+01, 2.5981E+01, 1.5190E+03, &
      3.5147962E+00, 6.92587E+01, 2.5777E+01, 1.4996E+03, &
      3.5323702E+00, 6.90914E+01, 2.5574E+01, 1.4805E+03, &
      3.5490108E+00, 6.84316E+01, 2.5386E+01, 1.4628E+03, &
      3.5500321E+00, 6.83166E+01, 2.5374E+01, 1.4617E+03, &
      3.5593891E+00, 6.85065E+01, 2.7055E+01, 1.5538E+03, &
      3.5677822E+00, 6.91322E+01, 2.6951E+01, 1.5443E+03, &
      3.5856211E+00, 6.97623E+01, 2.6733E+01, 1.5243E+03, &
      3.6035492E+00, 7.01606E+01, 2.6517E+01, 1.5045E+03, &
      3.6215670E+00, 7.04697E+01, 2.6304E+01, 1.4851E+03, &
      3.6396748E+00, 7.07282E+01, 2.6092E+01, 1.4660E+03, &
      3.6578732E+00, 7.09523E+01, 2.5883E+01, 1.4471E+03, &
      3.6761626E+00, 7.11499E+01, 2.5676E+01, 1.4285E+03, &
      3.6945434E+00, 7.13251E+01, 2.5471E+01, 1.4101E+03, &
      3.7130161E+00, 7.14799E+01, 2.5267E+01, 1.3920E+03, &
      3.7315812E+00, 7.16138E+01, 2.5068E+01, 1.3742E+03, &
      3.7502391E+00, 7.17294E+01, 2.4878E+01, 1.3572E+03, &
      3.7689903E+00, 7.18265E+01, 2.4691E+01, 1.3404E+03, &
      3.7878352E+00, 7.18988E+01, 2.4506E+01, 1.3238E+03, &
      3.8067744E+00, 7.19330E+01, 2.4324E+01, 1.3075E+03, &
      3.8258083E+00, 7.18915E+01, 2.4143E+01, 1.2915E+03, &
      3.8424209E+00, 7.16468E+01, 2.3988E+01, 1.2777E+03, &
      3.8449373E+00, 7.15435E+01, 2.3965E+01, 1.2756E+03, &
      3.8589789E+00, 7.17617E+01, 2.4895E+01, 1.3199E+03, &
      3.8641620E+00, 7.19632E+01, 2.4847E+01, 1.3157E+03, &
      3.8834828E+00, 7.23999E+01, 2.4672E+01, 1.3000E+03, &
      3.9029002E+00, 7.26895E+01, 2.4498E+01, 1.2845E+03, &
      3.9224147E+00, 7.29259E+01, 2.4327E+01, 1.2693E+03, &
      3.9420268E+00, 7.31335E+01, 2.4157E+01, 1.2542E+03, &
      3.9617369E+00, 7.33228E+01, 2.3989E+01, 1.2394E+03, &
      3.9815456E+00, 7.34991E+01, 2.3822E+01, 1.2248E+03, &
      4.0014533E+00, 7.36649E+01, 2.3653E+01, 1.2101E+03, &
      4.0214606E+00, 7.38213E+01, 2.3484E+01, 1.1956E+03, &
      4.0415679E+00, 7.39697E+01, 2.3317E+01, 1.1813E+03, &
      4.0617757E+00, 7.41114E+01, 2.3151E+01, 1.1671E+03, &
      4.0820846E+00, 7.42469E+01, 2.2986E+01, 1.1531E+03, &
      4.1024950E+00, 7.43771E+01, 2.2823E+01, 1.1394E+03, &
      4.1230075E+00, 7.45025E+01, 2.2661E+01, 1.1258E+03, &
      4.1436226E+00, 7.46236E+01, 2.2501E+01, 1.1123E+03, &
      4.1643407E+00, 7.47406E+01, 2.2342E+01, 1.0991E+03, &
      4.1851624E+00, 7.48539E+01, 2.2185E+01, 1.0860E+03, &
      4.2060882E+00, 7.49638E+01, 2.2029E+01, 1.0731E+03, &
      4.2271186E+00, 7.50705E+01, 2.1874E+01, 1.0603E+03, &
      4.2482542E+00, 7.51742E+01, 2.1721E+01, 1.0477E+03, &
      4.2694955E+00, 7.52751E+01, 2.1569E+01, 1.0353E+03, &
      4.2908430E+00, 7.53733E+01, 2.1418E+01, 1.0231E+03, &
      4.3122972E+00, 7.54690E+01, 2.1268E+01, 1.0109E+03, &
      4.3338587E+00, 7.55623E+01, 2.1120E+01, 9.9898E+02, &
      4.3555280E+00, 7.56533E+01, 2.0973E+01, 9.8718E+02, &
      4.3773056E+00, 7.57422E+01, 2.0827E+01, 9.7552E+02, &
      4.3991921E+00, 7.58290E+01, 2.0682E+01, 9.6401E+02, &
      4.4211881E+00, 7.59139E+01, 2.0539E+01, 9.5264E+02, &
      4.4432940E+00, 7.59970E+01, 2.0396E+01, 9.4141E+02, &
      4.4655105E+00, 7.60783E+01, 2.0255E+01, 9.3032E+02, &
      4.4878381E+00, 7.61579E+01, 2.0115E+01, 9.1937E+02, &
      4.5102772E+00, 7.62360E+01, 1.9976E+01, 9.0855E+02, &
      4.5328286E+00, 7.63126E+01, 1.9838E+01, 8.9786E+02, &
      4.5554928E+00, 7.63879E+01, 1.9701E+01, 8.8731E+02, &
      4.5782702E+00, 7.64619E+01, 1.9565E+01, 8.7689E+02, &
      4.6011616E+00, 7.65348E+01, 1.9430E+01, 8.6659E+02, &
      4.6241674E+00, 7.66068E+01, 1.9295E+01, 8.5639E+02, &
      4.6472882E+00, 7.66770E+01, 1.9159E+01, 8.4618E+02, &
      4.6705247E+00, 7.67453E+01, 1.9023E+01, 8.3610E+02, &
      4.6938773E+00, 7.68119E+01, 1.8889E+01, 8.2613E+02, &
      4.7173467E+00, 7.68768E+01, 1.8755E+01, 8.1629E+02, &
      4.7409334E+00, 7.69402E+01, 1.8622E+01, 8.0657E+02, &
      4.7646381E+00, 7.70022E+01, 1.8491E+01, 7.9696E+02, &
      4.7884613E+00, 7.70627E+01, 1.8360E+01, 7.8746E+02, &
      4.8124036E+00, 7.71220E+01, 1.8230E+01, 7.7809E+02, &
      4.8364656E+00, 7.71799E+01, 1.8101E+01, 7.6882E+02, &
      4.8606479E+00, 7.72367E+01, 1.7973E+01, 7.5966E+02, &
      4.8849512E+00, 7.72924E+01, 1.7846E+01, 7.5062E+02, &
      4.9093759E+00, 7.73469E+01, 1.7720E+01, 7.4168E+02, &
      4.9339228E+00, 7.74004E+01, 1.7594E+01, 7.3286E+02, &
      4.9585924E+00, 7.74530E+01, 1.7470E+01, 7.2414E+02, &
      4.9833854E+00, 7.76836E+01, 1.7342E+01, 7.1535E+02, &
      5.0083023E+00, 7.77342E+01, 1.7213E+01, 7.0656E+02, &
      5.0333438E+00, 7.77830E+01, 1.7084E+01, 6.9786E+02, &
      5.0585105E+00, 7.78302E+01, 1.6956E+01, 6.8927E+02, &
      5.0838031E+00, 7.78758E+01, 1.6829E+01, 6.8079E+02, &
      5.1092221E+00, 7.79200E+01, 1.6704E+01, 6.7242E+02, &
      5.1347682E+00, 7.79627E+01, 1.6579E+01, 6.6416E+02, &
      5.1604421E+00, 7.80042E+01, 1.6455E+01, 6.5600E+02, &
      5.1862443E+00, 7.81673E+01, 1.6330E+01, 6.4784E+02, &
      5.2121755E+00, 7.82062E+01, 1.6204E+01, 6.3973E+02, &
      5.2382364E+00, 7.82434E+01, 1.6079E+01, 6.3172E+02, &
      5.2644276E+00, 7.82792E+01, 1.5955E+01, 6.2382E+02, &
      5.2907497E+00, 7.83134E+01, 1.5832E+01, 6.1602E+02, &
      5.3172034E+00, 7.83463E+01, 1.5711E+01, 6.0832E+02, &
      5.3437895E+00, 7.83778E+01, 1.5590E+01, 6.0072E+02, &
      5.3705084E+00, 7.84081E+01, 1.5470E+01, 5.9322E+02, &
      5.3973609E+00, 7.84371E+01, 1.5352E+01, 5.8582E+02, &
      5.4243477E+00, 7.84650E+01, 1.5234E+01, 5.7852E+02, &
      5.4514695E+00, 7.84918E+01, 1.5117E+01, 5.7131E+02, &
      5.4787268E+00, 7.85174E+01, 1.5002E+01, 5.6419E+02, &
      5.5061205E+00, 7.85420E+01, 1.4887E+01, 5.5717E+02, &
      5.5336511E+00, 7.85656E+01, 1.4773E+01, 5.5024E+02, &
      5.5613193E+00, 7.85882E+01, 1.4661E+01, 5.4340E+02, &
      5.5891259E+00, 7.86098E+01, 1.4549E+01, 5.3666E+02, &
      5.6170716E+00, 7.86305E+01, 1.4438E+01, 5.3000E+02, &
      5.6451569E+00, 7.86504E+01, 1.4328E+01, 5.2342E+02, &
      5.6733827E+00, 7.86693E+01, 1.4220E+01, 5.1694E+02, &
      5.7017496E+00, 7.86874E+01, 1.4112E+01, 5.1054E+02, &
      5.7302584E+00, 7.87048E+01, 1.4005E+01, 5.0422E+02, &
      5.7589096E+00, 7.87213E+01, 1.3899E+01, 4.9799E+02, &
      5.7877042E+00, 7.87371E+01, 1.3794E+01, 4.9183E+02, &
      5.8166427E+00, 7.87521E+01, 1.3689E+01, 4.8576E+02, &
      5.8457259E+00, 7.87664E+01, 1.3586E+01, 4.7977E+02, &
      5.8749546E+00, 7.87800E+01, 1.3484E+01, 4.7386E+02, &
      5.9043293E+00, 7.87930E+01, 1.3382E+01, 4.6802E+02, &
      5.9338510E+00, 7.88053E+01, 1.3282E+01, 4.6227E+02, &
      5.9635202E+00, 7.88170E+01, 1.3182E+01, 4.5658E+02, &
      5.9933378E+00, 7.88281E+01, 1.3083E+01, 4.5098E+02, &
      6.0233045E+00, 7.88386E+01, 1.2985E+01, 4.4544E+02, &
      6.0534210E+00, 7.88485E+01, 1.2888E+01, 4.3998E+02, &
      6.0836882E+00, 7.88579E+01, 1.2792E+01, 4.3460E+02, &
      6.1141066E+00, 7.88668E+01, 1.2697E+01, 4.2928E+02, &
      6.1446771E+00, 7.89685E+01, 1.2601E+01, 4.2400E+02, &
      6.1754005E+00, 7.89767E+01, 1.2505E+01, 4.1874E+02, &
      6.2062775E+00, 7.89842E+01, 1.2410E+01, 4.1356E+02, &
      6.2373089E+00, 7.89910E+01, 1.2316E+01, 4.0844E+02, &
      6.2684954E+00, 7.89971E+01, 1.2222E+01, 4.0339E+02, &
      6.2998379E+00, 7.90026E+01, 1.2130E+01, 3.9840E+02, &
      6.3313371E+00, 7.90075E+01, 1.2038E+01, 3.9349E+02, &
      6.3629938E+00, 7.90118E+01, 1.1947E+01, 3.8863E+02, &
      6.3948088E+00, 7.90155E+01, 1.1857E+01, 3.8384E+02, &
      6.4267828E+00, 7.90186E+01, 1.1767E+01, 3.7912E+02, &
      6.4589167E+00, 7.90213E+01, 1.1678E+01, 3.7445E+02, &
      6.4912113E+00, 7.90234E+01, 1.1591E+01, 3.6985E+02, &
      6.5236674E+00, 7.90250E+01, 1.1503E+01, 3.6531E+02, &
      6.5562857E+00, 7.90261E+01, 1.1417E+01, 3.6082E+02, &
      6.5890671E+00, 7.90267E+01, 1.1332E+01, 3.5640E+02, &
      6.6220125E+00, 7.90269E+01, 1.1247E+01, 3.5204E+02, &
      6.6551225E+00, 7.90266E+01, 1.1163E+01, 3.4773E+02, &
      6.6883981E+00, 7.90259E+01, 1.1079E+01, 3.4348E+02, &
      6.7218401E+00, 7.90248E+01, 1.0997E+01, 3.3928E+02, &
      6.7554493E+00, 7.90233E+01, 1.0915E+01, 3.3514E+02, &
      6.7892266E+00, 7.90214E+01, 1.0834E+01, 3.3105E+02, &
      6.8231727E+00, 7.90191E+01, 1.0753E+01, 3.2702E+02, &
      6.8572886E+00, 7.90164E+01, 1.0674E+01, 3.2304E+02, &
      6.8915750E+00, 7.90134E+01, 1.0594E+01, 3.1911E+02, &
      6.9260329E+00, 7.90100E+01, 1.0516E+01, 3.1524E+02, &
      6.9606631E+00, 7.90063E+01, 1.0439E+01, 3.1141E+02, &
      6.9954664E+00, 7.90022E+01, 1.0362E+01, 3.0764E+02, &
      7.0304437E+00, 7.89979E+01, 1.0285E+01, 3.0391E+02, &
      7.0655959E+00, 7.89932E+01, 1.0210E+01, 3.0023E+02, &
      7.1009239E+00, 7.89883E+01, 1.0135E+01, 2.9660E+02, &
      7.1364285E+00, 7.90233E+01, 1.0060E+01, 2.9300E+02, &
      7.1721107E+00, 7.90179E+01, 9.9849E+00, 2.8943E+02, &
      7.2079712E+00, 7.90122E+01, 9.9108E+00, 2.8591E+02, &
      7.2440111E+00, 7.90061E+01, 9.8373E+00, 2.8244E+02, &
      7.2802311E+00, 7.89996E+01, 9.7645E+00, 2.7901E+02, &
      7.3166323E+00, 7.89928E+01, 9.6923E+00, 2.7562E+02, &
      7.3532155E+00, 7.89856E+01, 9.6208E+00, 2.7228E+02, &
      7.3899815E+00, 7.89781E+01, 9.5498E+00, 2.6898E+02, &
      7.4269314E+00, 7.89703E+01, 9.4795E+00, 2.6573E+02, &
      7.4640661E+00, 7.89622E+01, 9.4097E+00, 2.6252E+02, &
      7.5013864E+00, 7.89538E+01, 9.3406E+00, 2.5934E+02, &
      7.5388934E+00, 7.89451E+01, 9.2721E+00, 2.5621E+02, &
      7.5765878E+00, 7.89361E+01, 9.2041E+00, 2.5312E+02, &
      7.6144708E+00, 7.89268E+01, 9.1367E+00, 2.5007E+02, &
      7.6525431E+00, 7.89173E+01, 9.0699E+00, 2.4706E+02, &
      7.6908058E+00, 7.89075E+01, 9.0037E+00, 2.4409E+02, &
      7.7292599E+00, 7.89147E+01, 8.9377E+00, 2.4115E+02, &
      7.7679062E+00, 7.89045E+01, 8.8719E+00, 2.3823E+02, &
      7.8067457E+00, 7.88940E+01, 8.8062E+00, 2.3534E+02, &
      7.8457794E+00, 7.88832E+01, 8.7411E+00, 2.3249E+02, &
      7.8850083E+00, 7.88722E+01, 8.6765E+00, 2.2968E+02, &
      7.9244334E+00, 7.88608E+01, 8.6125E+00, 2.2690E+02, &
      7.9640555E+00, 7.88491E+01, 8.5490E+00, 2.2416E+02, &
      8.0038758E+00, 7.88371E+01, 8.4861E+00, 2.2145E+02, &
      8.0438952E+00, 7.88249E+01, 8.4238E+00, 2.1878E+02, &
      8.0841147E+00, 7.88125E+01, 8.3619E+00, 2.1614E+02, &
      8.1245352E+00, 7.87998E+01, 8.3006E+00, 2.1354E+02, &
      8.1651579E+00, 7.87868E+01, 8.2398E+00, 2.1097E+02, &
      8.2059837E+00, 7.87736E+01, 8.1792E+00, 2.0843E+02, &
      8.2470136E+00, 7.87602E+01, 8.1190E+00, 2.0591E+02, &
      8.2882487E+00, 7.87466E+01, 8.0593E+00, 2.0343E+02, &
      8.3296899E+00, 7.87328E+01, 8.0001E+00, 2.0098E+02, &
      8.3713384E+00, 7.87187E+01, 7.9415E+00, 1.9856E+02, &
      8.4131951E+00, 7.87045E+01, 7.8833E+00, 1.9617E+02, &
      8.4552610E+00, 7.86901E+01, 7.8257E+00, 1.9382E+02, &
      8.4975373E+00, 7.86756E+01, 7.7685E+00, 1.9149E+02, &
      8.5400250E+00, 7.86610E+01, 7.7118E+00, 1.8919E+02, &
      9.030794E+00, 7.86366E+01, 7.0842E+00, 1.6487E+02, &
      9.653919E+00, 7.82954E+01, 6.3370E+00, 1.3855E+02, &
      1.032004E+01, 7.78206E+01, 5.6779E+00, 1.1666E+02, &
      1.103212E+01, 7.71762E+01, 5.0898E+00, 9.8318E+01, &
      1.179334E+01, 7.61947E+01, 4.5557E+00, 8.2779E+01, &
      1.260708E+01, 7.40992E+01, 4.0822E+00, 6.9804E+01, &
      1.277450E+01, 7.31416E+01, 3.9953E+00, 6.7505E+01, &
      1.297002E+01, 7.04638E+01, 3.8979E+00, 6.4960E+01, &
      1.302216E+01, 6.73245E+01, 3.8726E+00, 6.4306E+01, &
      1.310038E+01, 7.04488E+01, 9.9729E+00, 1.5850E+02, &
      1.329590E+01, 7.32095E+01, 9.7275E+00, 1.5241E+02, &
      1.347697E+01, 7.42235E+01, 9.5058E+00, 1.4702E+02, &
      1.440688E+01, 7.57933E+01, 8.4714E+00, 1.2293E+02, &
      1.489600E+01, 7.53700E+01, 7.9928E+00, 1.1236E+02, &
      1.512400E+01, 7.41555E+01, 7.7840E+00, 1.0786E+02, &
      1.518480E+01, 7.25905E+01, 7.7297E+00, 1.0670E+02, &
      1.527600E+01, 7.42010E+01, 1.0761E+01, 1.4636E+02, &
      1.540095E+01, 7.51978E+01, 1.0613E+01, 1.4322E+02, &
      1.550400E+01, 7.55924E+01, 1.0493E+01, 1.4069E+02, &
      1.554358E+01, 7.56951E+01, 1.0447E+01, 1.3974E+02, &
      1.578150E+01, 7.57392E+01, 1.0180E+01, 1.3418E+02, &
      1.584494E+01, 7.50825E+01, 1.0110E+01, 1.3274E+02, &
      1.594010E+01, 7.60919E+01, 1.1594E+01, 1.5085E+02, &
      1.617802E+01, 7.72456E+01, 1.1331E+01, 1.4533E+02, &
      1.646362E+01, 7.80037E+01, 1.1029E+01, 1.3908E+02, &
      1.759961E+01, 7.96648E+01, 9.9294E+00, 1.1739E+02, &
      1.881398E+01, 8.05990E+01, 8.9312E+00, 9.9005E+01, &
      2.011215E+01, 8.12072E+01, 8.0330E+00, 8.3513E+01, &
      2.149988E+01, 8.16163E+01, 7.2192E+00, 7.0404E+01, &
      2.298338E+01, 8.18914E+01, 6.4893E+00, 5.9378E+01, &
      2.456923E+01, 8.20773E+01, 5.8363E+00, 5.0116E+01, &
      2.626450E+01, 8.22675E+01, 5.2514E+00, 4.2328E+01, &
      2.807676E+01, 8.23534E+01, 4.6894E+00, 3.5499E+01, &
      3.001405E+01, 8.23744E+01, 4.1819E+00, 2.9745E+01, &
      3.208502E+01, 8.24220E+01, 3.7254E+00, 2.4907E+01, &
      3.429889E+01, 8.23775E+01, 3.3123E+00, 2.0827E+01, &
      3.666551E+01, 8.23113E+01, 2.9473E+00, 1.7435E+01, &
      3.919543E+01, 8.22314E+01, 2.6246E+00, 1.4615E+01, &
      4.189992E+01, 8.21424E+01, 2.3390E+00, 1.2265E+01, &
      4.479101E+01, 8.20471E+01, 2.0861E+00, 1.0306E+01, &
      4.788159E+01, 8.19471E+01, 1.8619E+00, 8.6715E+00, &
      5.118542E+01, 8.18428E+01, 1.6631E+00, 7.3059E+00, &
      5.471721E+01, 8.17338E+01, 1.4866E+00, 6.1638E+00, &
      5.849270E+01, 8.16181E+01, 1.3280E+00, 5.2015E+00, &
      6.252870E+01, 8.14905E+01, 1.1860E+00, 4.3923E+00, &
      6.684318E+01, 8.13447E+01, 1.0596E+00, 3.7140E+00, &
      7.145536E+01, 8.11664E+01, 9.4428E-01, 3.1368E+00, &
      7.638578E+01, 8.09221E+01, 8.4086E-01, 2.6511E+00, &
      8.165640E+01, 8.05139E+01, 7.4709E-01, 2.2397E+00, &
      8.624441E+01, 7.96017E+01, 6.7514E-01, 1.9461E+00, &
      8.729069E+01, 7.89273E+01, 6.6026E-01, 1.8872E+00, &
      8.756448E+01, 7.85581E+01, 6.5646E-01, 1.8721E+00, &
      8.791650E+01, 7.73094E+01, 6.5162E-01, 1.8532E+00, &
      8.844452E+01, 7.86113E+01, 3.1021E+00, 7.4685E+00, &
      8.976459E+01, 7.97499E+01, 3.0313E+00, 7.1972E+00, &
      9.331374E+01, 8.06837E+01, 2.8524E+00, 6.5313E+00, &
      9.975239E+01, 8.13428E+01, 2.5665E+00, 5.5237E+00, &
      1.066353E+02, 8.16907E+01, 2.3054E+00, 4.6666E+00, &
      1.139931E+02, 8.19070E+01, 2.0695E+00, 3.9428E+00, &
      1.218587E+02, 8.20504E+01, 1.8573E+00, 3.3329E+00, &
      1.302669E+02, 8.21481E+01, 1.6646E+00, 2.8161E+00, &
      1.392553E+02, 8.22125E+01, 1.4907E+00, 2.3803E+00, &
      1.488640E+02, 8.22539E+01, 1.3349E+00, 2.0140E+00, &
      1.591356E+02, 8.22790E+01, 1.1954E+00, 1.7063E+00, &
      1.701159E+02, 8.22923E+01, 1.0706E+00, 1.4478E+00, &
      1.818539E+02, 8.22972E+01, 9.5890E-01, 1.2307E+00, &
      1.944018E+02, 8.22958E+01, 8.5844E-01, 1.0476E+00, &
      2.078156E+02, 8.22896E+01, 7.6852E-01, 8.9365E-01, &
      2.221548E+02, 8.22802E+01, 6.8810E-01, 7.6416E-01, &
      2.374835E+02, 8.22685E+01, 6.1616E-01, 6.5519E-01, &
      2.538699E+02, 8.22554E+01, 5.5180E-01, 5.6339E-01, &
      2.713869E+02, 8.22414E+01, 4.9421E-01, 4.8600E-01, &
      2.901126E+02, 8.22270E+01, 4.4267E-01, 4.2068E-01, &
      3.101304E+02, 8.22125E+01, 3.9654E-01, 3.6548E-01, &
      3.315294E+02, 8.21982E+01, 3.5525E-01, 3.1877E-01, &
      3.544049E+02, 8.21842E+01, 3.1829E-01, 2.7920E-01, &
      3.788588E+02, 8.21707E+01, 2.8519E-01, 2.4560E-01, &
      4.050001E+02, 8.21578E+01, 2.5556E-01, 2.1702E-01, &
      4.329451E+02, 8.21455E+01, 2.2902E-01, 1.9265E-01]
   case(83)
      frel = -1.1484E+00
      allocate(nist_tab(3100))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.046510E-03, 1.28464E+00, 0.0000E+00, 1.1814E-05, &
      6.139059E-03, -1.80675E-01, 0.0000E+00, 1.2814E-05, &
      6.163738E-03, -1.91459E+00, 0.0000E+00, 1.3092E-05, &
      6.200758E-03, -1.37486E-01, 3.4020E+00, 1.1047E+05, &
      6.293306E-03, 1.40890E+00, 3.3618E+00, 1.0756E+05, &
      1.069000E-02, 4.97863E+00, 2.2826E+00, 4.2995E+04, &
      1.142761E-02, 4.97869E+00, 2.1596E+00, 3.8053E+04, &
      1.221612E-02, 4.91679E+00, 2.0341E+00, 3.3528E+04, &
      1.305903E-02, 4.76197E+00, 1.9063E+00, 2.9394E+04, &
      1.396010E-02, 4.32198E+00, 1.7772E+00, 2.5635E+04, &
      1.416222E-02, 3.93071E+00, 1.7493E+00, 2.4872E+04, &
      1.421916E-02, 3.46074E+00, 1.7415E+00, 2.4662E+04, &
      1.430456E-02, 3.94974E+00, 2.6458E+00, 3.7244E+04, &
      1.451806E-02, 4.35860E+00, 2.5748E+00, 3.5711E+04, &
      1.492335E-02, 4.57910E+00, 2.4480E+00, 3.3031E+04, &
      1.595306E-02, 4.63786E+00, 2.1658E+00, 2.7337E+04, &
      1.705382E-02, 4.45056E+00, 1.9095E+00, 2.2546E+04, &
      1.823053E-02, 4.09469E+00, 1.6789E+00, 1.8544E+04, &
      1.948844E-02, 3.57158E+00, 1.4744E+00, 1.5234E+04, &
      2.083314E-02, 2.85179E+00, 1.2942E+00, 1.2509E+04, &
      2.227063E-02, 1.84894E+00, 1.1360E+00, 1.0271E+04, &
      2.380730E-02, 2.79116E-01, 9.9700E-01, 8.4326E+03, &
      2.391200E-02, 1.27349E-01, 9.8848E-01, 8.3239E+03, &
      2.427800E-02, -6.05839E-01, 9.5954E-01, 7.9584E+03, &
      2.437560E-02, -1.10786E+00, 9.5204E-01, 7.8645E+03, &
      2.452200E-02, -9.40591E-01, 1.7065E+00, 1.4012E+04, &
      2.488800E-02, -1.08707E+00, 1.8034E+00, 1.4591E+04, &
      2.545001E-02, -1.64041E+00, 1.9899E+00, 1.5744E+04, &
      2.597000E-02, -2.28637E+00, 2.2093E+00, 1.7130E+04, &
      2.636750E-02, -2.93846E+00, 2.4120E+00, 1.8420E+04, &
      2.647350E-02, -3.27904E+00, 2.4718E+00, 1.8801E+04, &
      2.663250E-02, -3.29644E+00, 2.9799E+00, 2.2530E+04, &
      2.703000E-02, -3.63457E+00, 3.3173E+00, 2.4712E+04, &
      2.720606E-02, -3.82014E+00, 3.4805E+00, 2.5760E+04, &
      2.908327E-02, -5.89222E+00, 5.7491E+00, 3.9804E+04, &
      3.109002E-02, -7.62257E+00, 9.2212E+00, 5.9723E+04, &
      3.323523E-02, -8.35979E+00, 1.3756E+01, 8.3345E+04, &
      3.552846E-02, -7.57550E+00, 1.8920E+01, 1.0723E+05, &
      3.797993E-02, -5.00556E+00, 2.4015E+01, 1.2732E+05, &
      4.060054E-02, -7.72435E-01, 2.8271E+01, 1.4021E+05, &
      4.340198E-02, 4.64125E+00, 3.1078E+01, 1.4419E+05, &
      4.639671E-02, 1.05467E+01, 3.2139E+01, 1.3948E+05, &
      4.959809E-02, 1.62480E+01, 3.1504E+01, 1.2790E+05, &
      5.302035E-02, 2.12033E+01, 2.9491E+01, 1.1200E+05, &
      5.667876E-02, 2.51009E+01, 2.6556E+01, 9.4344E+04, &
      6.058959E-02, 2.78531E+01, 2.3160E+01, 7.6968E+04, &
      6.477028E-02, 2.95388E+01, 1.9688E+01, 6.1207E+04, &
      6.923942E-02, 3.03295E+01, 1.6411E+01, 4.7726E+04, &
      7.401695E-02, 3.04572E+01, 1.3486E+01, 3.6688E+04, &
      7.912411E-02, 3.00351E+01, 1.0980E+01, 2.7942E+04, &
      8.458368E-02, 2.92314E+01, 8.8954E+00, 2.1176E+04, &
      9.041995E-02, 2.78160E+01, 7.1994E+00, 1.6033E+04, &
      9.094400E-02, 2.76060E+01, 7.0692E+00, 1.5652E+04, &
      9.233600E-02, 2.66090E+01, 6.7387E+00, 1.4695E+04, &
      9.270720E-02, 2.55780E+01, 6.6542E+00, 1.4453E+04, &
      9.326400E-02, 2.65515E+01, 8.4652E+00, 1.8277E+04, &
      9.465600E-02, 2.72752E+01, 8.0859E+00, 1.7201E+04, &
      9.665893E-02, 2.74823E+01, 7.5808E+00, 1.5792E+04, &
      1.033284E-01, 2.71254E+01, 6.1948E+00, 1.2073E+04, &
      1.104581E-01, 2.62374E+01, 5.1249E+00, 9.3430E+03, &
      1.144640E-01, 2.54904E+01, 4.6628E+00, 8.2031E+03, &
      1.162160E-01, 2.47960E+01, 4.4852E+00, 7.7719E+03, &
      1.166832E-01, 2.41212E+01, 4.4401E+00, 7.6630E+03, &
      1.173840E-01, 2.47170E+01, 5.6135E+00, 9.6301E+03, &
      1.180797E-01, 2.49747E+01, 5.5334E+00, 9.4368E+03, &
      1.191360E-01, 2.51250E+01, 5.4159E+00, 9.1546E+03, &
      1.262272E-01, 2.50157E+01, 4.7387E+00, 7.5601E+03, &
      1.349368E-01, 2.44093E+01, 4.1073E+00, 6.1301E+03, &
      1.442475E-01, 2.35910E+01, 3.5964E+00, 5.0213E+03, &
      1.542005E-01, 2.24846E+01, 3.1864E+00, 4.1621E+03, &
      1.542520E-01, 2.24774E+01, 3.1846E+00, 4.1584E+03, &
      1.561140E-01, 2.21808E+01, 3.1203E+00, 4.0259E+03, &
      1.566130E-01, 2.20759E+01, 3.1036E+00, 3.9916E+03, &
      1.572426E-01, 2.18580E+01, 3.0830E+00, 3.9492E+03, &
      1.581870E-01, 2.18413E+01, 3.3158E+00, 4.2221E+03, &
      1.586620E-01, 2.17964E+01, 3.3071E+00, 4.1984E+03, &
      1.591407E-01, 2.17049E+01, 3.2986E+00, 4.1750E+03, &
      1.600965E-01, 2.16729E+01, 3.4080E+00, 4.2876E+03, &
      1.605480E-01, 2.16360E+01, 3.4013E+00, 4.2673E+03, &
      1.610905E-01, 2.15720E+01, 3.3937E+00, 4.2434E+03, &
      1.617381E-01, 2.14279E+01, 3.3853E+00, 4.2159E+03, &
      1.624860E-01, 2.14394E+01, 3.5669E+00, 4.4216E+03, &
      1.627095E-01, 2.14392E+01, 3.5665E+00, 4.4150E+03, &
      1.648404E-01, 2.13246E+01, 3.5670E+00, 4.3587E+03, &
      1.651380E-01, 2.13028E+01, 3.5678E+00, 4.3518E+03, &
      1.762144E-01, 2.03011E+01, 3.7195E+00, 4.2518E+03, &
      1.883732E-01, 1.91101E+01, 4.1589E+00, 4.4473E+03, &
      2.013709E-01, 1.78619E+01, 4.9263E+00, 4.9280E+03, &
      2.152655E-01, 1.66344E+01, 6.0678E+00, 5.6780E+03, &
      2.301188E-01, 1.55253E+01, 7.6096E+00, 6.6610E+03, &
      2.459970E-01, 1.46470E+01, 9.5459E+00, 7.8165E+03, &
      2.629708E-01, 1.41138E+01, 1.1830E+01, 9.0615E+03, &
      2.811158E-01, 1.40267E+01, 1.4375E+01, 1.0300E+04, &
      3.005128E-01, 1.44493E+01, 1.7058E+01, 1.1434E+04, &
      3.212482E-01, 1.54066E+01, 1.9738E+01, 1.2376E+04, &
      3.434143E-01, 1.68702E+01, 2.2267E+01, 1.3060E+04, &
      3.671099E-01, 1.87560E+01, 2.4511E+01, 1.3449E+04, &
      3.924405E-01, 2.09220E+01, 2.6362E+01, 1.3531E+04, &
      4.195189E-01, 2.30901E+01, 2.7747E+01, 1.3323E+04, &
      4.312000E-01, 2.37719E+01, 2.8172E+01, 1.3162E+04, &
      4.378000E-01, 2.37625E+01, 2.8370E+01, 1.3054E+04, &
      4.395600E-01, 2.32706E+01, 2.8418E+01, 1.3024E+04, &
      4.422000E-01, 2.40733E+01, 2.9716E+01, 1.3537E+04, &
      4.484657E-01, 2.50378E+01, 2.9921E+01, 1.3440E+04, &
      4.488000E-01, 2.50758E+01, 2.9931E+01, 1.3435E+04, &
      4.543280E-01, 2.56187E+01, 3.0093E+01, 1.3343E+04, &
      4.612820E-01, 2.60072E+01, 3.0274E+01, 1.3221E+04, &
      4.631364E-01, 2.57858E+01, 3.0319E+01, 1.3188E+04, &
      4.659180E-01, 2.64552E+01, 3.1184E+01, 1.3483E+04, &
      4.728720E-01, 2.74852E+01, 3.1369E+01, 1.3364E+04, &
      4.794098E-01, 2.82360E+01, 3.1525E+01, 1.3247E+04, &
      5.0000000E-01, 3.21000E+01, 3.1903E+01, 1.2854E+04, &
      5.0250000E-01, 3.23386E+01, 3.1937E+01, 1.2804E+04, &
      5.0501250E-01, 3.25759E+01, 3.1970E+01, 1.2754E+04, &
      5.0753756E-01, 3.28121E+01, 3.2000E+01, 1.2702E+04, &
      5.1007525E-01, 3.30473E+01, 3.2028E+01, 1.2650E+04, &
      5.1262563E-01, 3.32815E+01, 3.2054E+01, 1.2597E+04, &
      5.1518875E-01, 3.35148E+01, 3.2078E+01, 1.2544E+04, &
      5.1776470E-01, 3.37472E+01, 3.2098E+01, 1.2490E+04, &
      5.2035352E-01, 3.39786E+01, 3.2117E+01, 1.2435E+04, &
      5.2295529E-01, 3.42089E+01, 3.2133E+01, 1.2379E+04, &
      5.2557007E-01, 3.44383E+01, 3.2147E+01, 1.2323E+04, &
      5.2819792E-01, 3.46666E+01, 3.2158E+01, 1.2266E+04, &
      5.3083891E-01, 3.48938E+01, 3.2168E+01, 1.2209E+04, &
      5.3349310E-01, 3.51199E+01, 3.2174E+01, 1.2151E+04, &
      5.3616057E-01, 3.53449E+01, 3.2179E+01, 1.2092E+04, &
      5.3884137E-01, 3.55688E+01, 3.2181E+01, 1.2033E+04, &
      5.4153558E-01, 3.57915E+01, 3.2181E+01, 1.1973E+04, &
      5.4424325E-01, 3.60129E+01, 3.2179E+01, 1.1912E+04, &
      5.4696447E-01, 3.62332E+01, 3.2174E+01, 1.1852E+04, &
      5.4969929E-01, 3.64521E+01, 3.2168E+01, 1.1790E+04, &
      5.5244779E-01, 3.66698E+01, 3.2159E+01, 1.1729E+04, &
      5.5521003E-01, 3.68861E+01, 3.2148E+01, 1.1666E+04, &
      5.5798608E-01, 3.71010E+01, 3.2135E+01, 1.1604E+04, &
      5.6077601E-01, 3.73145E+01, 3.2120E+01, 1.1541E+04, &
      5.6357989E-01, 3.75265E+01, 3.2103E+01, 1.1477E+04, &
      5.6639779E-01, 3.77371E+01, 3.2084E+01, 1.1413E+04, &
      5.6922978E-01, 3.79461E+01, 3.2063E+01, 1.1349E+04, &
      5.7207593E-01, 3.81535E+01, 3.2040E+01, 1.1285E+04, &
      5.7493630E-01, 3.83592E+01, 3.2015E+01, 1.1220E+04, &
      5.7781099E-01, 3.85633E+01, 3.1989E+01, 1.1155E+04, &
      5.8070004E-01, 3.87657E+01, 3.1960E+01, 1.1090E+04, &
      5.8360354E-01, 3.89662E+01, 3.1929E+01, 1.1024E+04, &
      5.8652156E-01, 3.91649E+01, 3.1897E+01, 1.0958E+04, &
      5.8945417E-01, 3.93617E+01, 3.1863E+01, 1.0892E+04, &
      5.9240144E-01, 3.95565E+01, 3.1827E+01, 1.0826E+04, &
      5.9536345E-01, 3.97493E+01, 3.1790E+01, 1.0759E+04, &
      5.9834026E-01, 3.99399E+01, 3.1751E+01, 1.0693E+04, &
      6.0133196E-01, 4.01284E+01, 3.1710E+01, 1.0626E+04, &
      6.0433862E-01, 4.03145E+01, 3.1668E+01, 1.0559E+04, &
      6.0736032E-01, 4.04983E+01, 3.1624E+01, 1.0492E+04, &
      6.1039712E-01, 4.06795E+01, 3.1579E+01, 1.0425E+04, &
      6.1344910E-01, 4.08581E+01, 3.1532E+01, 1.0358E+04, &
      6.1651635E-01, 4.10340E+01, 3.1483E+01, 1.0290E+04, &
      6.1959893E-01, 4.12068E+01, 3.1433E+01, 1.0223E+04, &
      6.2269693E-01, 4.13766E+01, 3.1382E+01, 1.0156E+04, &
      6.2581041E-01, 4.15430E+01, 3.1330E+01, 1.0088E+04, &
      6.2893946E-01, 4.17058E+01, 3.1276E+01, 1.0021E+04, &
      6.3208416E-01, 4.18647E+01, 3.1221E+01, 9.9536E+03, &
      6.3524458E-01, 4.20192E+01, 3.1164E+01, 9.8861E+03, &
      6.3842080E-01, 4.21690E+01, 3.1106E+01, 9.8187E+03, &
      6.4161291E-01, 4.23134E+01, 3.1046E+01, 9.7513E+03, &
      6.4482097E-01, 4.24516E+01, 3.0986E+01, 9.6839E+03, &
      6.4804508E-01, 4.25827E+01, 3.0924E+01, 9.6165E+03, &
      6.5128530E-01, 4.27053E+01, 3.0860E+01, 9.5491E+03, &
      6.5454173E-01, 4.28177E+01, 3.0796E+01, 9.4818E+03, &
      6.5781444E-01, 4.29172E+01, 3.0730E+01, 9.4145E+03, &
      6.6110351E-01, 4.29999E+01, 3.0663E+01, 9.3473E+03, &
      6.6440903E-01, 4.30593E+01, 3.0594E+01, 9.2802E+03, &
      6.6773107E-01, 4.30839E+01, 3.0525E+01, 9.2132E+03, &
      6.7106973E-01, 4.30497E+01, 3.0455E+01, 9.1463E+03, &
      6.7442508E-01, 4.28903E+01, 3.0383E+01, 9.0795E+03, &
      6.7779720E-01, 4.22211E+01, 3.0311E+01, 9.0129E+03, &
      6.7809211E-01, 4.20486E+01, 3.0304E+01, 9.0071E+03, &
      6.7970789E-01, 4.21317E+01, 3.2165E+01, 9.5371E+03, &
      6.8118619E-01, 4.28350E+01, 3.2134E+01, 9.5072E+03, &
      6.8459212E-01, 4.35588E+01, 3.2063E+01, 9.4389E+03, &
      6.8801508E-01, 4.40154E+01, 3.1990E+01, 9.3707E+03, &
      6.9145515E-01, 4.43798E+01, 3.1917E+01, 9.3028E+03, &
      6.9491243E-01, 4.46967E+01, 3.1842E+01, 9.2351E+03, &
      6.9838699E-01, 4.49840E+01, 3.1768E+01, 9.1677E+03, &
      7.0187893E-01, 4.52510E+01, 3.1692E+01, 9.1005E+03, &
      7.0538832E-01, 4.55031E+01, 3.1616E+01, 9.0336E+03, &
      7.0891526E-01, 4.57436E+01, 3.1540E+01, 8.9671E+03, &
      7.1245984E-01, 4.59749E+01, 3.1463E+01, 8.9008E+03, &
      7.1602214E-01, 4.61985E+01, 3.1386E+01, 8.8348E+03, &
      7.1960225E-01, 4.64157E+01, 3.1308E+01, 8.7692E+03, &
      7.2320026E-01, 4.66274E+01, 3.1230E+01, 8.7039E+03, &
      7.2681626E-01, 4.68344E+01, 3.1151E+01, 8.6389E+03, &
      7.3045034E-01, 4.70373E+01, 3.1073E+01, 8.5743E+03, &
      7.3410260E-01, 4.72366E+01, 3.0994E+01, 8.5101E+03, &
      7.3777311E-01, 4.74328E+01, 3.0915E+01, 8.4463E+03, &
      7.4146197E-01, 4.76263E+01, 3.0836E+01, 8.3829E+03, &
      7.4516928E-01, 4.78169E+01, 3.0752E+01, 8.3185E+03, &
      7.4889513E-01, 4.80030E+01, 3.0660E+01, 8.2526E+03, &
      7.5263961E-01, 4.81852E+01, 3.0568E+01, 8.1870E+03, &
      7.5640280E-01, 4.83634E+01, 3.0476E+01, 8.1218E+03, &
      7.6018482E-01, 4.85379E+01, 3.0383E+01, 8.0569E+03, &
      7.6398574E-01, 4.87087E+01, 3.0291E+01, 7.9924E+03, &
      7.6780567E-01, 4.88759E+01, 3.0197E+01, 7.9283E+03, &
      7.7164470E-01, 4.90392E+01, 3.0104E+01, 7.8645E+03, &
      7.7550292E-01, 4.91987E+01, 3.0010E+01, 7.8011E+03, &
      7.7938044E-01, 4.93538E+01, 2.9916E+01, 7.7381E+03, &
      7.8327734E-01, 4.95042E+01, 2.9822E+01, 7.6754E+03, &
      7.8719373E-01, 4.96475E+01, 2.9721E+01, 7.6115E+03, &
      7.9112969E-01, 4.97815E+01, 2.9619E+01, 7.5478E+03, &
      7.9508534E-01, 4.99021E+01, 2.9517E+01, 7.4844E+03, &
      7.9906077E-01, 4.99987E+01, 2.9414E+01, 7.4215E+03, &
      8.0305607E-01, 5.00244E+01, 2.9312E+01, 7.3589E+03, &
      8.0418063E-01, 4.99786E+01, 2.9283E+01, 7.3414E+03, &
      8.0641936E-01, 5.00640E+01, 2.9629E+01, 7.4074E+03, &
      8.0707135E-01, 5.01476E+01, 2.9612E+01, 7.3973E+03, &
      8.1110671E-01, 5.04519E+01, 2.9511E+01, 7.3355E+03, &
      8.1516224E-01, 5.06701E+01, 2.9410E+01, 7.2740E+03, &
      8.1923806E-01, 5.08627E+01, 2.9308E+01, 7.2130E+03, &
      8.2333425E-01, 5.10420E+01, 2.9207E+01, 7.1523E+03, &
      8.2745092E-01, 5.12126E+01, 2.9105E+01, 7.0920E+03, &
      8.3158817E-01, 5.13768E+01, 2.9003E+01, 7.0322E+03, &
      8.3574611E-01, 5.15359E+01, 2.8902E+01, 6.9727E+03, &
      8.3992484E-01, 5.16907E+01, 2.8800E+01, 6.9137E+03, &
      8.4412447E-01, 5.18418E+01, 2.8698E+01, 6.8551E+03, &
      8.4834509E-01, 5.19894E+01, 2.8596E+01, 6.7969E+03, &
      8.5258682E-01, 5.21340E+01, 2.8494E+01, 6.7392E+03, &
      8.5684975E-01, 5.22756E+01, 2.8393E+01, 6.6818E+03, &
      8.6113400E-01, 5.24144E+01, 2.8291E+01, 6.6249E+03, &
      8.6543967E-01, 5.25506E+01, 2.8190E+01, 6.5684E+03, &
      8.6976687E-01, 5.26841E+01, 2.8089E+01, 6.5124E+03, &
      8.7411570E-01, 5.28150E+01, 2.7988E+01, 6.4568E+03, &
      8.7848628E-01, 5.29434E+01, 2.7887E+01, 6.4016E+03, &
      8.8287871E-01, 5.30692E+01, 2.7786E+01, 6.3469E+03, &
      8.8729310E-01, 5.31922E+01, 2.7686E+01, 6.2926E+03, &
      8.9172957E-01, 5.33124E+01, 2.7585E+01, 6.2387E+03, &
      8.9618822E-01, 5.34296E+01, 2.7485E+01, 6.1853E+03, &
      9.0066916E-01, 5.35434E+01, 2.7386E+01, 6.1323E+03, &
      9.0517250E-01, 5.36534E+01, 2.7286E+01, 6.0797E+03, &
      9.0969837E-01, 5.37589E+01, 2.7187E+01, 6.0276E+03, &
      9.1424686E-01, 5.38587E+01, 2.7088E+01, 5.9759E+03, &
      9.1881809E-01, 5.39511E+01, 2.6990E+01, 5.9247E+03, &
      9.2341218E-01, 5.40323E+01, 2.6892E+01, 5.8739E+03, &
      9.2802924E-01, 5.40948E+01, 2.6794E+01, 5.8235E+03, &
      9.3266939E-01, 5.41160E+01, 2.6697E+01, 5.7736E+03, &
      9.3687713E-01, 5.39831E+01, 2.6609E+01, 5.7289E+03, &
      9.3733274E-01, 5.39226E+01, 2.6600E+01, 5.7241E+03, &
      9.3952286E-01, 5.40533E+01, 2.7094E+01, 5.8168E+03, &
      9.4201940E-01, 5.43016E+01, 2.7044E+01, 5.7906E+03, &
      9.4672950E-01, 5.45639E+01, 2.6948E+01, 5.7417E+03, &
      9.5146315E-01, 5.47639E+01, 2.6854E+01, 5.6931E+03, &
      9.5622046E-01, 5.49407E+01, 2.6760E+01, 5.6451E+03, &
      9.6100156E-01, 5.51053E+01, 2.6666E+01, 5.5974E+03, &
      9.6580657E-01, 5.52626E+01, 2.6573E+01, 5.5502E+03, &
      9.7063560E-01, 5.54152E+01, 2.6480E+01, 5.5034E+03, &
      9.7548878E-01, 5.55648E+01, 2.6388E+01, 5.4571E+03, &
      9.8036623E-01, 5.57127E+01, 2.6296E+01, 5.4112E+03, &
      9.8526806E-01, 5.58600E+01, 2.6205E+01, 5.3657E+03, &
      9.9019440E-01, 5.60079E+01, 2.6114E+01, 5.3207E+03, &
      9.9514537E-01, 5.61581E+01, 2.6024E+01, 5.2761E+03, &
      1.0001211E+00, 5.63129E+01, 2.5933E+01, 5.2316E+03, &
      1.0051217E+00, 5.64625E+01, 2.5777E+01, 5.1743E+03, &
      1.0101473E+00, 5.66013E+01, 2.5622E+01, 5.1177E+03, &
      1.0151980E+00, 5.67321E+01, 2.5468E+01, 5.0619E+03, &
      1.0202740E+00, 5.68564E+01, 2.5316E+01, 5.0067E+03, &
      1.0253754E+00, 5.69753E+01, 2.5165E+01, 4.9522E+03, &
      1.0305023E+00, 5.70893E+01, 2.5015E+01, 4.8983E+03, &
      1.0356548E+00, 5.71990E+01, 2.4866E+01, 4.8451E+03, &
      1.0408331E+00, 5.73047E+01, 2.4718E+01, 4.7924E+03, &
      1.0460372E+00, 5.74067E+01, 2.4572E+01, 4.7404E+03, &
      1.0512674E+00, 5.75052E+01, 2.4426E+01, 4.6891E+03, &
      1.0565238E+00, 5.76005E+01, 2.4282E+01, 4.6383E+03, &
      1.0618064E+00, 5.76928E+01, 2.4139E+01, 4.5882E+03, &
      1.0671154E+00, 5.77823E+01, 2.3997E+01, 4.5387E+03, &
      1.0724510E+00, 5.78690E+01, 2.3856E+01, 4.4897E+03, &
      1.0778132E+00, 5.79532E+01, 2.3717E+01, 4.4414E+03, &
      1.0832023E+00, 5.80349E+01, 2.3578E+01, 4.3936E+03, &
      1.0886183E+00, 5.81143E+01, 2.3441E+01, 4.3464E+03, &
      1.0940614E+00, 5.81914E+01, 2.3305E+01, 4.2998E+03, &
      1.0995317E+00, 5.82664E+01, 2.3170E+01, 4.2538E+03, &
      1.1050294E+00, 5.83393E+01, 2.3036E+01, 4.2083E+03, &
      1.1105545E+00, 5.84103E+01, 2.2903E+01, 4.1633E+03, &
      1.1161073E+00, 5.84793E+01, 2.2771E+01, 4.1189E+03, &
      1.1216878E+00, 5.85466E+01, 2.2640E+01, 4.0750E+03, &
      1.1272963E+00, 5.86121E+01, 2.2511E+01, 4.0316E+03, &
      1.1329328E+00, 5.86759E+01, 2.2382E+01, 3.9888E+03, &
      1.1385974E+00, 5.87382E+01, 2.2255E+01, 3.9465E+03, &
      1.1442904E+00, 5.87989E+01, 2.2128E+01, 3.9047E+03, &
      1.1500119E+00, 5.88581E+01, 2.2002E+01, 3.8632E+03, &
      1.1557619E+00, 5.89154E+01, 2.1875E+01, 3.8220E+03, &
      1.1615407E+00, 5.89710E+01, 2.1749E+01, 3.7812E+03, &
      1.1673484E+00, 5.90249E+01, 2.1625E+01, 3.7410E+03, &
      1.1731852E+00, 5.90772E+01, 2.1501E+01, 3.7012E+03, &
      1.1790511E+00, 5.91279E+01, 2.1379E+01, 3.6619E+03, &
      1.1849464E+00, 5.91772E+01, 2.1257E+01, 3.6231E+03, &
      1.1908711E+00, 5.92249E+01, 2.1136E+01, 3.5847E+03, &
      1.1968254E+00, 5.92713E+01, 2.1016E+01, 3.5468E+03, &
      1.2028096E+00, 5.93162E+01, 2.0898E+01, 3.5094E+03, &
      1.2088236E+00, 5.93598E+01, 2.0780E+01, 3.4724E+03, &
      1.2148677E+00, 5.94021E+01, 2.0663E+01, 3.4358E+03, &
      1.2209421E+00, 5.94430E+01, 2.0547E+01, 3.3996E+03, &
      1.2270468E+00, 5.94828E+01, 2.0432E+01, 3.3639E+03, &
      1.2331820E+00, 5.95212E+01, 2.0318E+01, 3.3286E+03, &
      1.2393479E+00, 5.95585E+01, 2.0204E+01, 3.2937E+03, &
      1.2455447E+00, 5.95946E+01, 2.0092E+01, 3.2592E+03, &
      1.2517724E+00, 5.96296E+01, 1.9981E+01, 3.2251E+03, &
      1.2580312E+00, 5.96635E+01, 1.9870E+01, 3.1915E+03, &
      1.2643214E+00, 5.96962E+01, 1.9760E+01, 3.1582E+03, &
      1.2706430E+00, 5.97278E+01, 1.9651E+01, 3.1253E+03, &
      1.2769962E+00, 5.97585E+01, 1.9544E+01, 3.0928E+03, &
      1.2833812E+00, 5.97880E+01, 1.9436E+01, 3.0607E+03, &
      1.2897981E+00, 5.98166E+01, 1.9330E+01, 3.0289E+03, &
      1.2962471E+00, 5.98441E+01, 1.9225E+01, 2.9975E+03, &
      1.3027283E+00, 5.98707E+01, 1.9120E+01, 2.9665E+03, &
      1.3092420E+00, 5.98963E+01, 1.9016E+01, 2.9359E+03, &
      1.3157882E+00, 5.99210E+01, 1.8913E+01, 2.9056E+03, &
      1.3223671E+00, 5.99584E+01, 1.8811E+01, 2.8756E+03, &
      1.3289790E+00, 5.99815E+01, 1.8710E+01, 2.8461E+03, &
      1.3356239E+00, 6.00036E+01, 1.8609E+01, 2.8168E+03, &
      1.3423020E+00, 6.00249E+01, 1.8510E+01, 2.7879E+03, &
      1.3490135E+00, 6.00454E+01, 1.8411E+01, 2.7593E+03, &
      1.3557586E+00, 6.00651E+01, 1.8312E+01, 2.7311E+03, &
      1.3625374E+00, 6.00839E+01, 1.8215E+01, 2.7031E+03, &
      1.3693500E+00, 6.01018E+01, 1.8117E+01, 2.6754E+03, &
      1.3761968E+00, 6.01188E+01, 1.8020E+01, 2.6479E+03, &
      1.3830778E+00, 6.01348E+01, 1.7923E+01, 2.6208E+03, &
      1.3899932E+00, 6.01498E+01, 1.7828E+01, 2.5940E+03, &
      1.3969431E+00, 6.01713E+01, 1.7733E+01, 2.5674E+03, &
      1.4039278E+00, 6.01847E+01, 1.7639E+01, 2.5412E+03, &
      1.4109475E+00, 6.01974E+01, 1.7545E+01, 2.5153E+03, &
      1.4180022E+00, 6.02091E+01, 1.7453E+01, 2.4897E+03, &
      1.4250922E+00, 6.02200E+01, 1.7361E+01, 2.4644E+03, &
      1.4322177E+00, 6.02301E+01, 1.7269E+01, 2.4393E+03, &
      1.4393788E+00, 6.02393E+01, 1.7179E+01, 2.4146E+03, &
      1.4465757E+00, 6.02477E+01, 1.7089E+01, 2.3901E+03, &
      1.4538086E+00, 6.02553E+01, 1.6999E+01, 2.3660E+03, &
      1.4610776E+00, 6.02621E+01, 1.6911E+01, 2.3420E+03, &
      1.4683830E+00, 6.02681E+01, 1.6823E+01, 2.3184E+03, &
      1.4757249E+00, 6.02732E+01, 1.6736E+01, 2.2950E+03, &
      1.4831035E+00, 6.02776E+01, 1.6649E+01, 2.2719E+03, &
      1.4905190E+00, 6.02813E+01, 1.6563E+01, 2.2491E+03, &
      1.4979716E+00, 6.02841E+01, 1.6478E+01, 2.2265E+03, &
      1.5054615E+00, 6.02861E+01, 1.6393E+01, 2.2042E+03, &
      1.5129888E+00, 6.02874E+01, 1.6310E+01, 2.1821E+03, &
      1.5205537E+00, 6.02879E+01, 1.6226E+01, 2.1603E+03, &
      1.5281565E+00, 6.02877E+01, 1.6143E+01, 2.1387E+03, &
      1.5357973E+00, 6.02867E+01, 1.6061E+01, 2.1174E+03, &
      1.5434763E+00, 6.02850E+01, 1.5980E+01, 2.0963E+03, &
      1.5511937E+00, 6.02825E+01, 1.5899E+01, 2.0754E+03, &
      1.5589496E+00, 6.02792E+01, 1.5819E+01, 2.0548E+03, &
      1.5667444E+00, 6.02752E+01, 1.5739E+01, 2.0344E+03, &
      1.5745781E+00, 6.02705E+01, 1.5660E+01, 2.0142E+03, &
      1.5824510E+00, 6.02651E+01, 1.5582E+01, 1.9943E+03, &
      1.5903633E+00, 6.02588E+01, 1.5504E+01, 1.9745E+03, &
      1.5983151E+00, 6.02518E+01, 1.5426E+01, 1.9550E+03, &
      1.6063066E+00, 6.02439E+01, 1.5349E+01, 1.9356E+03, &
      1.6143382E+00, 6.02352E+01, 1.5272E+01, 1.9165E+03, &
      1.6224099E+00, 6.02257E+01, 1.5196E+01, 1.8976E+03, &
      1.6305219E+00, 6.02154E+01, 1.5120E+01, 1.8789E+03, &
      1.6386745E+00, 6.02043E+01, 1.5045E+01, 1.8604E+03, &
      1.6468679E+00, 6.01923E+01, 1.4971E+01, 1.8421E+03, &
      1.6551022E+00, 6.01796E+01, 1.4897E+01, 1.8240E+03, &
      1.6633777E+00, 6.01660E+01, 1.4824E+01, 1.8061E+03, &
      1.6716946E+00, 6.01517E+01, 1.4751E+01, 1.7885E+03, &
      1.6800531E+00, 6.01365E+01, 1.4679E+01, 1.7710E+03, &
      1.6884534E+00, 6.01205E+01, 1.4607E+01, 1.7537E+03, &
      1.6968956E+00, 6.01037E+01, 1.4536E+01, 1.7366E+03, &
      1.7053801E+00, 6.00860E+01, 1.4465E+01, 1.7196E+03, &
      1.7139070E+00, 6.00676E+01, 1.4395E+01, 1.7029E+03, &
      1.7224766E+00, 6.00483E+01, 1.4326E+01, 1.6864E+03, &
      1.7310889E+00, 6.00282E+01, 1.4257E+01, 1.6700E+03, &
      1.7397444E+00, 6.00073E+01, 1.4188E+01, 1.6538E+03, &
      1.7484431E+00, 5.99856E+01, 1.4120E+01, 1.6378E+03, &
      1.7571853E+00, 5.99631E+01, 1.4052E+01, 1.6219E+03, &
      1.7659712E+00, 5.99395E+01, 1.3978E+01, 1.6055E+03, &
      1.7748011E+00, 5.99144E+01, 1.3902E+01, 1.5889E+03, &
      1.7836751E+00, 5.98877E+01, 1.3826E+01, 1.5725E+03, &
      1.7925935E+00, 5.98594E+01, 1.3751E+01, 1.5563E+03, &
      1.8015565E+00, 5.98295E+01, 1.3676E+01, 1.5403E+03, &
      1.8105642E+00, 5.97980E+01, 1.3602E+01, 1.5244E+03, &
      1.8196171E+00, 5.97650E+01, 1.3529E+01, 1.5088E+03, &
      1.8287151E+00, 5.97304E+01, 1.3456E+01, 1.4933E+03, &
      1.8378587E+00, 5.96942E+01, 1.3383E+01, 1.4780E+03, &
      1.8470480E+00, 5.96564E+01, 1.3311E+01, 1.4628E+03, &
      1.8562833E+00, 5.96169E+01, 1.3239E+01, 1.4477E+03, &
      1.8655647E+00, 5.95755E+01, 1.3161E+01, 1.4323E+03, &
      1.8748925E+00, 5.95319E+01, 1.3085E+01, 1.4170E+03, &
      1.8842670E+00, 5.94862E+01, 1.3009E+01, 1.4018E+03, &
      1.8936883E+00, 5.94382E+01, 1.2933E+01, 1.3869E+03, &
      1.9031567E+00, 5.93881E+01, 1.2858E+01, 1.3721E+03, &
      1.9126725E+00, 5.93357E+01, 1.2784E+01, 1.3575E+03, &
      1.9222359E+00, 5.92810E+01, 1.2710E+01, 1.3431E+03, &
      1.9318471E+00, 5.92239E+01, 1.2636E+01, 1.3288E+03, &
      1.9415063E+00, 5.91646E+01, 1.2564E+01, 1.3147E+03, &
      1.9512138E+00, 5.91028E+01, 1.2491E+01, 1.3008E+03, &
      1.9609699E+00, 5.90385E+01, 1.2420E+01, 1.2870E+03, &
      1.9707747E+00, 5.89717E+01, 1.2348E+01, 1.2733E+03, &
      1.9806286E+00, 5.89024E+01, 1.2278E+01, 1.2599E+03, &
      1.9905318E+00, 5.88304E+01, 1.2207E+01, 1.2466E+03, &
      2.0004844E+00, 5.87556E+01, 1.2138E+01, 1.2334E+03, &
      2.0104868E+00, 5.86780E+01, 1.2069E+01, 1.2204E+03, &
      2.0205393E+00, 5.85976E+01, 1.2000E+01, 1.2075E+03, &
      2.0306420E+00, 5.85141E+01, 1.1932E+01, 1.1948E+03, &
      2.0407952E+00, 5.84292E+01, 1.1864E+01, 1.1823E+03, &
      2.0509992E+00, 5.83394E+01, 1.1797E+01, 1.1698E+03, &
      2.0612542E+00, 5.82464E+01, 1.1730E+01, 1.1576E+03, &
      2.0715604E+00, 5.81498E+01, 1.1664E+01, 1.1454E+03, &
      2.0819182E+00, 5.80497E+01, 1.1598E+01, 1.1334E+03, &
      2.0923278E+00, 5.79459E+01, 1.1533E+01, 1.1216E+03, &
      2.1027895E+00, 5.78381E+01, 1.1468E+01, 1.1098E+03, &
      2.1133034E+00, 5.77263E+01, 1.1404E+01, 1.0982E+03, &
      2.1238699E+00, 5.76102E+01, 1.1340E+01, 1.0868E+03, &
      2.1344893E+00, 5.74896E+01, 1.1277E+01, 1.0754E+03, &
      2.1451617E+00, 5.73644E+01, 1.1214E+01, 1.0642E+03, &
      2.1558875E+00, 5.72342E+01, 1.1151E+01, 1.0532E+03, &
      2.1666670E+00, 5.70989E+01, 1.1089E+01, 1.0422E+03, &
      2.1775003E+00, 5.69580E+01, 1.1028E+01, 1.0314E+03, &
      2.1883878E+00, 5.68114E+01, 1.0967E+01, 1.0207E+03, &
      2.1993297E+00, 5.66587E+01, 1.0906E+01, 1.0101E+03, &
      2.2103264E+00, 5.64995E+01, 1.0845E+01, 9.9960E+02, &
      2.2213780E+00, 5.63335E+01, 1.0785E+01, 9.8924E+02, &
      2.2324849E+00, 5.61600E+01, 1.0726E+01, 9.7900E+02, &
      2.2436473E+00, 5.59788E+01, 1.0667E+01, 9.6887E+02, &
      2.2548656E+00, 5.57891E+01, 1.0608E+01, 9.5885E+02, &
      2.2661399E+00, 5.55905E+01, 1.0550E+01, 9.4895E+02, &
      2.2774706E+00, 5.53822E+01, 1.0492E+01, 9.3916E+02, &
      2.2888579E+00, 5.51635E+01, 1.0434E+01, 9.2948E+02, &
      2.3003022E+00, 5.49335E+01, 1.0377E+01, 9.1990E+02, &
      2.3118037E+00, 5.46912E+01, 1.0320E+01, 9.1044E+02, &
      2.3233628E+00, 5.44357E+01, 1.0264E+01, 9.0108E+02, &
      2.3349796E+00, 5.41655E+01, 1.0208E+01, 8.9182E+02, &
      2.3466545E+00, 5.38793E+01, 1.0153E+01, 8.8267E+02, &
      2.3583878E+00, 5.35754E+01, 1.0098E+01, 8.7362E+02, &
      2.3701797E+00, 5.32519E+01, 1.0043E+01, 8.6468E+02, &
      2.3820306E+00, 5.29065E+01, 9.9884E+00, 8.5583E+02, &
      2.3939407E+00, 5.25365E+01, 9.9344E+00, 8.4708E+02, &
      2.4059104E+00, 5.21386E+01, 9.8809E+00, 8.3843E+02, &
      2.4179400E+00, 5.17125E+01, 9.8276E+00, 8.2987E+02, &
      2.4300297E+00, 5.12462E+01, 9.7748E+00, 8.2141E+02, &
      2.4421798E+00, 5.07372E+01, 9.7223E+00, 8.1304E+02, &
      2.4543907E+00, 5.01776E+01, 9.6702E+00, 8.0477E+02, &
      2.4666627E+00, 4.95575E+01, 9.6184E+00, 7.9659E+02, &
      2.4789960E+00, 4.88632E+01, 9.5670E+00, 7.8850E+02, &
      2.4913910E+00, 4.80758E+01, 9.5160E+00, 7.8049E+02, &
      2.5038479E+00, 4.71678E+01, 9.4653E+00, 7.7258E+02, &
      2.5163672E+00, 4.60971E+01, 9.4149E+00, 7.6476E+02, &
      2.5289490E+00, 4.47936E+01, 9.3649E+00, 7.5702E+02, &
      2.5415938E+00, 4.31265E+01, 9.3153E+00, 7.4936E+02, &
      2.5543017E+00, 4.08023E+01, 9.2660E+00, 7.4179E+02, &
      2.5670732E+00, 3.68771E+01, 9.2170E+00, 7.3431E+02, &
      2.5788933E+00, 2.12884E+01, 9.1722E+00, 7.2748E+02, &
      2.5799086E+00, 1.65394E+01, 2.6074E+01, 2.0463E+03, &
      2.5803069E+00, 2.10000E+01, 2.6068E+01, 2.0456E+03, &
      2.5928082E+00, 3.65808E+01, 2.5879E+01, 2.0211E+03, &
      2.6057722E+00, 3.99542E+01, 2.5686E+01, 1.9962E+03, &
      2.6188011E+00, 4.17054E+01, 2.5495E+01, 1.9716E+03, &
      2.6318951E+00, 4.26865E+01, 2.5305E+01, 1.9473E+03, &
      2.6450545E+00, 4.31094E+01, 2.5117E+01, 1.9233E+03, &
      2.6582798E+00, 4.29465E+01, 2.4930E+01, 1.8997E+03, &
      2.6715712E+00, 4.18129E+01, 2.4745E+01, 1.8763E+03, &
      2.6849291E+00, 3.63620E+01, 2.4561E+01, 1.8532E+03, &
      2.6868259E+00, 3.20847E+01, 2.4535E+01, 1.8500E+03, &
      2.6883739E+00, 3.20049E+01, 3.5686E+01, 2.6841E+03, &
      2.6983537E+00, 4.19890E+01, 3.5482E+01, 2.6590E+03, &
      2.7118455E+00, 4.56172E+01, 3.5209E+01, 2.6255E+03, &
      2.7254047E+00, 4.78545E+01, 3.4937E+01, 2.5924E+03, &
      2.7390317E+00, 4.95441E+01, 3.4667E+01, 2.5597E+03, &
      2.7527269E+00, 5.09266E+01, 3.4399E+01, 2.5274E+03, &
      2.7664905E+00, 5.21068E+01, 3.4133E+01, 2.4955E+03, &
      2.7803230E+00, 5.31409E+01, 3.3870E+01, 2.4641E+03, &
      2.7942246E+00, 5.40629E+01, 3.3608E+01, 2.4330E+03, &
      2.8081957E+00, 5.48954E+01, 3.3349E+01, 2.4024E+03, &
      2.8222367E+00, 5.56538E+01, 3.3092E+01, 2.3721E+03, &
      2.8363479E+00, 5.63498E+01, 3.2837E+01, 2.3422E+03, &
      2.8505296E+00, 5.69919E+01, 3.2584E+01, 2.3128E+03, &
      2.8647823E+00, 5.75865E+01, 3.2333E+01, 2.2837E+03, &
      2.8791062E+00, 5.81390E+01, 3.2084E+01, 2.2550E+03, &
      2.8935017E+00, 5.86535E+01, 3.1838E+01, 2.2266E+03, &
      2.9079692E+00, 5.91332E+01, 3.1593E+01, 2.1986E+03, &
      2.9225091E+00, 5.95809E+01, 3.1350E+01, 2.1710E+03, &
      2.9371216E+00, 5.99987E+01, 3.1109E+01, 2.1437E+03, &
      2.9518072E+00, 6.03882E+01, 3.0871E+01, 2.1168E+03, &
      2.9665662E+00, 6.07507E+01, 3.0633E+01, 2.0902E+03, &
      2.9813991E+00, 6.10868E+01, 3.0396E+01, 2.0638E+03, &
      2.9963061E+00, 6.13972E+01, 3.0161E+01, 2.0378E+03, &
      3.0112876E+00, 6.16726E+01, 2.9914E+01, 2.0112E+03, &
      3.0263440E+00, 6.19164E+01, 2.9664E+01, 1.9846E+03, &
      3.0414758E+00, 6.21293E+01, 2.9417E+01, 1.9584E+03, &
      3.0566831E+00, 6.23087E+01, 2.9170E+01, 1.9324E+03, &
      3.0719666E+00, 6.24498E+01, 2.8926E+01, 1.9068E+03, &
      3.0873264E+00, 6.25453E+01, 2.8683E+01, 1.8816E+03, &
      3.1027630E+00, 6.25832E+01, 2.8443E+01, 1.8567E+03, &
      3.1182768E+00, 6.25419E+01, 2.8205E+01, 1.8321E+03, &
      3.1338682E+00, 6.23789E+01, 2.7969E+01, 1.8079E+03, &
      3.1495376E+00, 6.19902E+01, 2.7735E+01, 1.7839E+03, &
      3.1652853E+00, 6.09806E+01, 2.7503E+01, 1.7603E+03, &
      3.1735006E+00, 5.92587E+01, 2.7383E+01, 1.7482E+03, &
      3.1802992E+00, 5.93216E+01, 3.2091E+01, 2.0426E+03, &
      3.1811117E+00, 5.96663E+01, 3.2077E+01, 2.0411E+03, &
      3.1970172E+00, 6.23551E+01, 3.1800E+01, 2.0136E+03, &
      3.2130023E+00, 6.35185E+01, 3.1525E+01, 1.9864E+03, &
      3.2290673E+00, 6.43326E+01, 3.1253E+01, 1.9595E+03, &
      3.2452127E+00, 6.49813E+01, 3.0982E+01, 1.9330E+03, &
      3.2614387E+00, 6.55301E+01, 3.0714E+01, 1.9069E+03, &
      3.2777459E+00, 6.60102E+01, 3.0448E+01, 1.8811E+03, &
      3.2941347E+00, 6.64386E+01, 3.0185E+01, 1.8557E+03, &
      3.3106053E+00, 6.68256E+01, 2.9923E+01, 1.8306E+03, &
      3.3271584E+00, 6.71770E+01, 2.9665E+01, 1.8059E+03, &
      3.3437941E+00, 6.74976E+01, 2.9419E+01, 1.7821E+03, &
      3.3605131E+00, 6.77991E+01, 2.9180E+01, 1.7589E+03, &
      3.3773157E+00, 6.80821E+01, 2.8941E+01, 1.7360E+03, &
      3.3942023E+00, 6.83479E+01, 2.8705E+01, 1.7134E+03, &
      3.4111733E+00, 6.85976E+01, 2.8472E+01, 1.6912E+03, &
      3.4282291E+00, 6.88325E+01, 2.8242E+01, 1.6693E+03, &
      3.4453703E+00, 6.90534E+01, 2.8015E+01, 1.6477E+03, &
      3.4625971E+00, 6.92609E+01, 2.7790E+01, 1.6265E+03, &
      3.4799101E+00, 6.94556E+01, 2.7569E+01, 1.6056E+03, &
      3.4973097E+00, 6.96376E+01, 2.7349E+01, 1.5850E+03, &
      3.5147962E+00, 6.98060E+01, 2.7128E+01, 1.5645E+03, &
      3.5323702E+00, 6.99603E+01, 2.6910E+01, 1.5443E+03, &
      3.5500321E+00, 7.00998E+01, 2.6695E+01, 1.5245E+03, &
      3.5677822E+00, 7.02232E+01, 2.6482E+01, 1.5049E+03, &
      3.5856211E+00, 7.03284E+01, 2.6271E+01, 1.4856E+03, &
      3.6035492E+00, 7.04118E+01, 2.6063E+01, 1.4666E+03, &
      3.6215670E+00, 7.04670E+01, 2.5857E+01, 1.4479E+03, &
      3.6396748E+00, 7.04818E+01, 2.5653E+01, 1.4294E+03, &
      3.6578732E+00, 7.04290E+01, 2.5451E+01, 1.4112E+03, &
      3.6761626E+00, 7.02250E+01, 2.5252E+01, 1.3933E+03, &
      3.6909034E+00, 6.96035E+01, 2.5093E+01, 1.3791E+03, &
      3.6945434E+00, 6.89929E+01, 2.5054E+01, 1.3756E+03, &
      3.7016966E+00, 6.96780E+01, 2.6777E+01, 1.4667E+03, &
      3.7130161E+00, 7.04262E+01, 2.6642E+01, 1.4549E+03, &
      3.7315812E+00, 7.10039E+01, 2.6424E+01, 1.4359E+03, &
      3.7502391E+00, 7.13851E+01, 2.6208E+01, 1.4172E+03, &
      3.7689903E+00, 7.16842E+01, 2.5994E+01, 1.3988E+03, &
      3.7878352E+00, 7.19352E+01, 2.5782E+01, 1.3806E+03, &
      3.8067744E+00, 7.21526E+01, 2.5573E+01, 1.3627E+03, &
      3.8258083E+00, 7.23438E+01, 2.5365E+01, 1.3450E+03, &
      3.8449373E+00, 7.25125E+01, 2.5160E+01, 1.3276E+03, &
      3.8641620E+00, 7.26600E+01, 2.4957E+01, 1.3104E+03, &
      3.8834828E+00, 7.27857E+01, 2.4758E+01, 1.2936E+03, &
      3.9029002E+00, 7.28929E+01, 2.4570E+01, 1.2775E+03, &
      3.9224147E+00, 7.29798E+01, 2.4384E+01, 1.2617E+03, &
      3.9420268E+00, 7.30380E+01, 2.4201E+01, 1.2460E+03, &
      3.9617369E+00, 7.30478E+01, 2.4019E+01, 1.2306E+03, &
      3.9815456E+00, 7.29420E+01, 2.3840E+01, 1.2155E+03, &
      3.9904219E+00, 7.27708E+01, 2.3761E+01, 1.2088E+03, &
      4.0014533E+00, 7.24061E+01, 2.4711E+01, 1.2533E+03, &
      4.0077780E+00, 7.28852E+01, 2.4655E+01, 1.2485E+03, &
      4.0214606E+00, 7.32941E+01, 2.4535E+01, 1.2383E+03, &
      4.0415679E+00, 7.36411E+01, 2.4362E+01, 1.2235E+03, &
      4.0617757E+00, 7.39004E+01, 2.4190E+01, 1.2089E+03, &
      4.0820846E+00, 7.41203E+01, 2.4021E+01, 1.1946E+03, &
      4.1024950E+00, 7.43170E+01, 2.3853E+01, 1.1804E+03, &
      4.1230075E+00, 7.44982E+01, 2.3687E+01, 1.1665E+03, &
      4.1436226E+00, 7.46677E+01, 2.3519E+01, 1.1525E+03, &
      4.1643407E+00, 7.48266E+01, 2.3350E+01, 1.1387E+03, &
      4.1851624E+00, 7.49771E+01, 2.3184E+01, 1.1250E+03, &
      4.2060882E+00, 7.51204E+01, 2.3018E+01, 1.1115E+03, &
      4.2271186E+00, 7.52573E+01, 2.2854E+01, 1.0982E+03, &
      4.2482542E+00, 7.53885E+01, 2.2691E+01, 1.0850E+03, &
      4.2694955E+00, 7.55147E+01, 2.2530E+01, 1.0720E+03, &
      4.2908430E+00, 7.56362E+01, 2.2370E+01, 1.0592E+03, &
      4.3122972E+00, 7.57536E+01, 2.2212E+01, 1.0466E+03, &
      4.3338587E+00, 7.58672E+01, 2.2055E+01, 1.0341E+03, &
      4.3555280E+00, 7.59772E+01, 2.1899E+01, 1.0218E+03, &
      4.3773056E+00, 7.60839E+01, 2.1745E+01, 1.0097E+03, &
      4.3991921E+00, 7.61876E+01, 2.1592E+01, 9.9767E+02, &
      4.4211881E+00, 7.62884E+01, 2.1441E+01, 9.8583E+02, &
      4.4432940E+00, 7.63864E+01, 2.1291E+01, 9.7414E+02, &
      4.4655105E+00, 7.64819E+01, 2.1142E+01, 9.6260E+02, &
      4.4878381E+00, 7.65750E+01, 2.0994E+01, 9.5121E+02, &
      4.5102772E+00, 7.66658E+01, 2.0848E+01, 9.3996E+02, &
      4.5328286E+00, 7.67543E+01, 2.0702E+01, 9.2885E+02, &
      4.5554928E+00, 7.68409E+01, 2.0558E+01, 9.1789E+02, &
      4.5782702E+00, 7.69254E+01, 2.0416E+01, 9.0706E+02, &
      4.6011616E+00, 7.70081E+01, 2.0274E+01, 8.9636E+02, &
      4.6241674E+00, 7.70889E+01, 2.0133E+01, 8.8580E+02, &
      4.6472882E+00, 7.71682E+01, 1.9994E+01, 8.7538E+02, &
      4.6705247E+00, 7.72458E+01, 1.9855E+01, 8.6508E+02, &
      4.6938773E+00, 7.73219E+01, 1.9718E+01, 8.5490E+02, &
      4.7173467E+00, 7.73966E+01, 1.9582E+01, 8.4485E+02, &
      4.7409334E+00, 7.74701E+01, 1.9447E+01, 8.3493E+02, &
      4.7646381E+00, 7.75424E+01, 1.9313E+01, 8.2513E+02, &
      4.7884613E+00, 7.76137E+01, 1.9180E+01, 8.1544E+02, &
      4.8124036E+00, 7.76840E+01, 1.9046E+01, 8.0583E+02, &
      4.8364656E+00, 7.77525E+01, 1.8911E+01, 7.9622E+02, &
      4.8606479E+00, 7.78192E+01, 1.8777E+01, 7.8672E+02, &
      4.8849512E+00, 7.78841E+01, 1.8644E+01, 7.7734E+02, &
      4.9093759E+00, 7.79474E+01, 1.8512E+01, 7.6807E+02, &
      4.9339228E+00, 7.80091E+01, 1.8381E+01, 7.5891E+02, &
      4.9585924E+00, 7.80695E+01, 1.8250E+01, 7.4987E+02, &
      4.9833854E+00, 7.81284E+01, 1.8121E+01, 7.4093E+02, &
      5.0083023E+00, 7.81861E+01, 1.7993E+01, 7.3210E+02, &
      5.0333438E+00, 7.82425E+01, 1.7865E+01, 7.2337E+02, &
      5.0585105E+00, 7.82977E+01, 1.7739E+01, 7.1475E+02, &
      5.0838031E+00, 7.83517E+01, 1.7613E+01, 7.0623E+02, &
      5.1092221E+00, 7.84047E+01, 1.7488E+01, 6.9782E+02, &
      5.1347682E+00, 7.84567E+01, 1.7364E+01, 6.8951E+02, &
      5.1604421E+00, 7.86824E+01, 1.7241E+01, 6.8128E+02, &
      5.1862443E+00, 7.87327E+01, 1.7112E+01, 6.7292E+02, &
      5.2121755E+00, 7.87814E+01, 1.6985E+01, 6.6466E+02, &
      5.2382364E+00, 7.88284E+01, 1.6858E+01, 6.5649E+02, &
      5.2644276E+00, 7.88738E+01, 1.6732E+01, 6.4842E+02, &
      5.2907497E+00, 7.89177E+01, 1.6607E+01, 6.4046E+02, &
      5.3172034E+00, 7.89602E+01, 1.6483E+01, 6.3259E+02, &
      5.3437895E+00, 7.90014E+01, 1.6360E+01, 6.2483E+02, &
      5.3705084E+00, 7.90413E+01, 1.6238E+01, 6.1716E+02, &
      5.3973609E+00, 7.92005E+01, 1.6114E+01, 6.0948E+02, &
      5.4243477E+00, 7.92379E+01, 1.5990E+01, 6.0186E+02, &
      5.4514695E+00, 7.92736E+01, 1.5867E+01, 5.9435E+02, &
      5.4787268E+00, 7.93080E+01, 1.5746E+01, 5.8693E+02, &
      5.5061205E+00, 7.93409E+01, 1.5625E+01, 5.7961E+02, &
      5.5336511E+00, 7.93725E+01, 1.5505E+01, 5.7238E+02, &
      5.5613193E+00, 7.94028E+01, 1.5386E+01, 5.6525E+02, &
      5.5891259E+00, 7.94320E+01, 1.5269E+01, 5.5822E+02, &
      5.6170716E+00, 7.94599E+01, 1.5152E+01, 5.5127E+02, &
      5.6451569E+00, 7.94867E+01, 1.5036E+01, 5.4442E+02, &
      5.6733827E+00, 7.95124E+01, 1.4922E+01, 5.3765E+02, &
      5.7017496E+00, 7.95371E+01, 1.4808E+01, 5.3097E+02, &
      5.7302584E+00, 7.95608E+01, 1.4695E+01, 5.2438E+02, &
      5.7589096E+00, 7.95835E+01, 1.4583E+01, 5.1788E+02, &
      5.7877042E+00, 7.96052E+01, 1.4473E+01, 5.1146E+02, &
      5.8166427E+00, 7.96260E+01, 1.4363E+01, 5.0512E+02, &
      5.8457259E+00, 7.96459E+01, 1.4254E+01, 4.9887E+02, &
      5.8749546E+00, 7.96650E+01, 1.4146E+01, 4.9270E+02, &
      5.9043293E+00, 7.96832E+01, 1.4039E+01, 4.8661E+02, &
      5.9338510E+00, 7.97006E+01, 1.3933E+01, 4.8060E+02, &
      5.9635202E+00, 7.97173E+01, 1.3827E+01, 4.7467E+02, &
      5.9933378E+00, 7.97332E+01, 1.3723E+01, 4.6882E+02, &
      6.0233045E+00, 7.97483E+01, 1.3620E+01, 4.6304E+02, &
      6.0534210E+00, 7.97627E+01, 1.3517E+01, 4.5734E+02, &
      6.0836882E+00, 7.97765E+01, 1.3416E+01, 4.5172E+02, &
      6.1141066E+00, 7.97896E+01, 1.3315E+01, 4.4617E+02, &
      6.1446771E+00, 7.98020E+01, 1.3215E+01, 4.4069E+02, &
      6.1754005E+00, 7.98138E+01, 1.3116E+01, 4.3528E+02, &
      6.2062775E+00, 7.98250E+01, 1.3018E+01, 4.2995E+02, &
      6.2373089E+00, 7.98356E+01, 1.2921E+01, 4.2468E+02, &
      6.2684954E+00, 7.98457E+01, 1.2825E+01, 4.1948E+02, &
      6.2998379E+00, 7.98552E+01, 1.2729E+01, 4.1436E+02, &
      6.3313371E+00, 7.98642E+01, 1.2635E+01, 4.0930E+02, &
      6.3629938E+00, 7.99627E+01, 1.2540E+01, 4.0428E+02, &
      6.3948088E+00, 7.99711E+01, 1.2445E+01, 3.9928E+02, &
      6.4267828E+00, 7.99787E+01, 1.2351E+01, 3.9435E+02, &
      6.4589167E+00, 7.99856E+01, 1.2257E+01, 3.8948E+02, &
      6.4912113E+00, 7.99919E+01, 1.2164E+01, 3.8468E+02, &
      6.5236674E+00, 7.99975E+01, 1.2072E+01, 3.7993E+02, &
      6.5562857E+00, 8.00026E+01, 1.1981E+01, 3.7526E+02, &
      6.5890671E+00, 8.00070E+01, 1.1891E+01, 3.7064E+02, &
      6.6220125E+00, 8.00109E+01, 1.1801E+01, 3.6608E+02, &
      6.6551225E+00, 8.00142E+01, 1.1713E+01, 3.6159E+02, &
      6.6883981E+00, 8.00170E+01, 1.1625E+01, 3.5715E+02, &
      6.7218401E+00, 8.00193E+01, 1.1538E+01, 3.5277E+02, &
      6.7554493E+00, 8.00211E+01, 1.1451E+01, 3.4845E+02, &
      6.7892266E+00, 8.00224E+01, 1.1365E+01, 3.4418E+02, &
      6.8231727E+00, 8.00232E+01, 1.1281E+01, 3.3997E+02, &
      6.8572886E+00, 8.00236E+01, 1.1196E+01, 3.3582E+02, &
      6.8915750E+00, 8.00235E+01, 1.1113E+01, 3.3172E+02, &
      6.9260329E+00, 8.00230E+01, 1.1030E+01, 3.2767E+02, &
      6.9606631E+00, 8.00221E+01, 1.0948E+01, 3.2368E+02, &
      6.9954664E+00, 8.00208E+01, 1.0867E+01, 3.1974E+02, &
      7.0304437E+00, 8.00192E+01, 1.0787E+01, 3.1585E+02, &
      7.0655959E+00, 8.00171E+01, 1.0707E+01, 3.1202E+02, &
      7.1009239E+00, 8.00147E+01, 1.0628E+01, 3.0823E+02, &
      7.1364285E+00, 8.00119E+01, 1.0549E+01, 3.0449E+02, &
      7.1721107E+00, 8.00087E+01, 1.0472E+01, 3.0080E+02, &
      7.2079712E+00, 8.00053E+01, 1.0395E+01, 2.9716E+02, &
      7.2440111E+00, 8.00015E+01, 1.0318E+01, 2.9357E+02, &
      7.2802311E+00, 7.99974E+01, 1.0243E+01, 2.9002E+02, &
      7.3166323E+00, 7.99930E+01, 1.0168E+01, 2.8652E+02, &
      7.3532155E+00, 7.99883E+01, 1.0093E+01, 2.8307E+02, &
      7.3899815E+00, 7.99834E+01, 1.0020E+01, 2.7966E+02, &
      7.4269314E+00, 8.00187E+01, 9.9455E+00, 2.7627E+02, &
      7.4640661E+00, 8.00134E+01, 9.8721E+00, 2.7292E+02, &
      7.5013864E+00, 8.00077E+01, 9.7993E+00, 2.6961E+02, &
      7.5388934E+00, 8.00016E+01, 9.7271E+00, 2.6635E+02, &
      7.5765878E+00, 7.99953E+01, 9.6556E+00, 2.6313E+02, &
      7.6144708E+00, 7.99885E+01, 9.5846E+00, 2.5995E+02, &
      7.6525431E+00, 7.99815E+01, 9.5143E+00, 2.5681E+02, &
      7.6908058E+00, 7.99742E+01, 9.4446E+00, 2.5372E+02, &
      7.7292599E+00, 7.99666E+01, 9.3754E+00, 2.5066E+02, &
      7.7679062E+00, 7.99587E+01, 9.3069E+00, 2.4764E+02, &
      7.8067457E+00, 7.99505E+01, 9.2389E+00, 2.4466E+02, &
      7.8457794E+00, 7.99421E+01, 9.1716E+00, 2.4172E+02, &
      7.8850083E+00, 7.99334E+01, 9.1048E+00, 2.3882E+02, &
      7.9244334E+00, 7.99245E+01, 9.0385E+00, 2.3595E+02, &
      7.9640555E+00, 7.99153E+01, 8.9729E+00, 2.3312E+02, &
      8.0038758E+00, 7.99218E+01, 8.9077E+00, 2.3033E+02, &
      8.0438952E+00, 7.99123E+01, 8.8427E+00, 2.2756E+02, &
      8.0841147E+00, 7.99025E+01, 8.7783E+00, 2.2483E+02, &
      8.1245352E+00, 7.98925E+01, 8.7144E+00, 2.2213E+02, &
      8.1651579E+00, 7.98822E+01, 8.6510E+00, 2.1947E+02, &
      8.2059837E+00, 7.98717E+01, 8.5882E+00, 2.1684E+02, &
      8.2470136E+00, 7.98610E+01, 8.5259E+00, 2.1425E+02, &
      8.2882487E+00, 7.98501E+01, 8.4638E+00, 2.1167E+02, &
      8.3296899E+00, 7.98390E+01, 8.4019E+00, 2.0913E+02, &
      8.3713384E+00, 7.98277E+01, 8.3406E+00, 2.0662E+02, &
      8.4131951E+00, 7.98163E+01, 8.2797E+00, 2.0414E+02, &
      8.4552610E+00, 7.98047E+01, 8.2194E+00, 2.0169E+02, &
      8.4975373E+00, 7.97929E+01, 8.1596E+00, 1.9928E+02, &
      8.5400250E+00, 7.97811E+01, 8.1004E+00, 1.9689E+02, &
      9.030794E+00, 7.98266E+01, 7.4409E+00, 1.7157E+02, &
      9.653919E+00, 7.95165E+01, 6.6542E+00, 1.4412E+02, &
      1.032004E+01, 7.90905E+01, 5.9606E+00, 1.2131E+02, &
      1.103212E+01, 7.85285E+01, 5.3481E+00, 1.0231E+02, &
      1.179334E+01, 7.77329E+01, 4.7928E+00, 8.6233E+01, &
      1.260708E+01, 7.63493E+01, 4.2946E+00, 7.2705E+01, &
      1.315023E+01, 7.42164E+01, 4.0092E+00, 6.5327E+01, &
      1.335151E+01, 7.15534E+01, 3.9118E+00, 6.2869E+01, &
      1.340518E+01, 6.84285E+01, 3.8865E+00, 6.2236E+01, &
      1.347697E+01, 7.12647E+01, 9.9711E+00, 1.5282E+02, &
      1.348569E+01, 7.15420E+01, 9.9601E+00, 1.5255E+02, &
      1.368697E+01, 7.42949E+01, 9.7124E+00, 1.4666E+02, &
      1.440688E+01, 7.65928E+01, 8.8901E+00, 1.2782E+02, &
      1.540095E+01, 7.64559E+01, 7.9170E+00, 1.0683E+02, &
      1.563254E+01, 7.52480E+01, 7.7141E+00, 1.0263E+02, &
      1.569539E+01, 7.36894E+01, 7.6605E+00, 1.0153E+02, &
      1.578966E+01, 7.52858E+01, 1.0675E+01, 1.3937E+02, &
      1.602532E+01, 7.66589E+01, 1.0413E+01, 1.3403E+02, &
      1.605975E+01, 7.67446E+01, 1.0376E+01, 1.3326E+02, &
      1.630556E+01, 7.67930E+01, 1.0113E+01, 1.2799E+02, &
      1.637111E+01, 7.61419E+01, 1.0044E+01, 1.2664E+02, &
      1.646362E+01, 7.70952E+01, 1.1524E+01, 1.4403E+02, &
      1.646944E+01, 7.71442E+01, 1.1518E+01, 1.4390E+02, &
      1.671525E+01, 7.82901E+01, 1.1259E+01, 1.3866E+02, &
      1.759961E+01, 8.00284E+01, 1.0396E+01, 1.2180E+02, &
      1.881398E+01, 8.12193E+01, 9.3528E+00, 1.0274E+02, &
      2.011215E+01, 8.19554E+01, 8.4136E+00, 8.6673E+01, &
      2.149988E+01, 8.24469E+01, 7.5653E+00, 7.3101E+01, &
      2.298338E+01, 8.27776E+01, 6.8012E+00, 6.1656E+01, &
      2.456923E+01, 8.30020E+01, 6.1175E+00, 5.2040E+01, &
      2.626450E+01, 8.31578E+01, 5.5060E+00, 4.3960E+01, &
      2.807676E+01, 8.33329E+01, 4.9197E+00, 3.6887E+01, &
      3.001405E+01, 8.33697E+01, 4.3877E+00, 3.0906E+01, &
      3.208502E+01, 8.34019E+01, 3.9147E+00, 2.5915E+01, &
      3.429889E+01, 8.33971E+01, 3.4839E+00, 2.1685E+01, &
      3.666551E+01, 8.33394E+01, 3.1000E+00, 1.8152E+01, &
      3.919543E+01, 8.32653E+01, 2.7606E+00, 1.5212E+01, &
      4.189992E+01, 8.31805E+01, 2.4602E+00, 1.2764E+01, &
      4.479101E+01, 8.30883E+01, 2.1941E+00, 1.0723E+01, &
      4.788159E+01, 8.29909E+01, 1.9583E+00, 9.0202E+00, &
      5.118542E+01, 8.28892E+01, 1.7492E+00, 7.5978E+00, &
      5.471721E+01, 8.27830E+01, 1.5635E+00, 6.4084E+00, &
      5.849270E+01, 8.26715E+01, 1.3982E+00, 5.4115E+00, &
      6.252870E+01, 8.25504E+01, 1.2486E+00, 4.5676E+00, &
      6.684318E+01, 8.24143E+01, 1.1160E+00, 3.8622E+00, &
      7.145536E+01, 8.22538E+01, 9.9662E-01, 3.2668E+00, &
      7.638578E+01, 8.20458E+01, 8.8760E-01, 2.7599E+00, &
      8.165640E+01, 8.17328E+01, 7.8887E-01, 2.3310E+00, &
      8.729069E+01, 8.10362E+01, 6.9733E-01, 1.9632E+00, &
      8.871538E+01, 8.06187E+01, 6.7685E-01, 1.8838E+00, &
      9.007327E+01, 7.95832E+01, 6.5820E-01, 1.8124E+00, &
      9.043538E+01, 7.83443E+01, 6.5336E-01, 1.7941E+00, &
      9.097853E+01, 7.96356E+01, 3.0842E+00, 7.1629E+00, &
      9.233642E+01, 8.07647E+01, 3.0142E+00, 6.9037E+00, &
      9.331374E+01, 8.11246E+01, 2.9652E+00, 6.7251E+00, &
      9.975239E+01, 8.21272E+01, 2.6704E+00, 5.6917E+00, &
      1.066353E+02, 8.25650E+01, 2.4016E+00, 4.8133E+00, &
      1.139931E+02, 8.28245E+01, 2.1570E+00, 4.0680E+00, &
      1.218587E+02, 8.29931E+01, 1.9366E+00, 3.4394E+00, &
      1.302669E+02, 8.31076E+01, 1.7377E+00, 2.9087E+00, &
      1.392553E+02, 8.31841E+01, 1.5569E+00, 2.4588E+00, &
      1.488640E+02, 8.32340E+01, 1.3948E+00, 2.0806E+00, &
      1.591356E+02, 8.32652E+01, 1.2495E+00, 1.7628E+00, &
      1.701159E+02, 8.32831E+01, 1.1195E+00, 1.4958E+00, &
      1.818539E+02, 8.32912E+01, 1.0030E+00, 1.2712E+00, &
      1.944018E+02, 8.32924E+01, 8.9864E-01, 1.0823E+00, &
      2.078156E+02, 8.32882E+01, 8.0500E-01, 9.2319E-01, &
      2.221548E+02, 8.32803E+01, 7.2120E-01, 7.8932E-01, &
      2.374835E+02, 8.32698E+01, 6.4618E-01, 6.7659E-01, &
      2.538699E+02, 8.32575E+01, 5.7903E-01, 5.8162E-01, &
      2.713869E+02, 8.32441E+01, 5.1891E-01, 5.0152E-01, &
      2.901126E+02, 8.32301E+01, 4.6508E-01, 4.3389E-01, &
      3.101304E+02, 8.32159E+01, 4.1687E-01, 3.7673E-01, &
      3.315294E+02, 8.32018E+01, 3.7369E-01, 3.2837E-01, &
      3.544049E+02, 8.31879E+01, 3.3502E-01, 2.8737E-01, &
      3.788588E+02, 8.31744E+01, 3.0037E-01, 2.5257E-01, &
      4.050001E+02, 8.31615E+01, 2.6933E-01, 2.2297E-01, &
      4.329451E+02, 8.31491E+01, 2.4151E-01, 1.9773E-01]
   case(84)
      frel = -1.1838E+00
      allocate(nist_tab(3096))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      5.286876E-03, -9.37416E-01, 0.0000E+00, 5.1517E-06, &
      5.367797E-03, -2.41276E+00, 0.0000E+00, 5.6056E-06, &
      5.389376E-03, -4.15553E+00, 0.0000E+00, 5.7317E-06, &
      5.421745E-03, -2.35734E+00, 3.4161E+00, 1.2686E+05, &
      5.502667E-03, -8.04519E-01, 3.3425E+00, 1.2230E+05, &
      7.408549E-03, -4.02942E-01, 2.3271E+00, 6.3243E+04, &
      7.521945E-03, -1.99692E+00, 2.2951E+00, 6.1432E+04, &
      7.552184E-03, -3.91935E+00, 2.2868E+00, 6.0965E+04, &
      7.597543E-03, -1.93282E+00, 6.0662E+00, 1.6076E+05, &
      7.710939E-03, -1.78513E-01, 6.0099E+00, 1.5692E+05, &
      1.069000E-02, 4.34264E+00, 4.8462E+00, 9.1275E+04, &
      1.142761E-02, 4.66024E+00, 4.5909E+00, 8.0886E+04, &
      1.221612E-02, 4.90165E+00, 4.3239E+00, 7.1265E+04, &
      1.305903E-02, 5.06722E+00, 4.0467E+00, 6.2391E+04, &
      1.396010E-02, 5.15153E+00, 3.7622E+00, 5.4260E+04, &
      1.492335E-02, 5.13926E+00, 3.4740E+00, 4.6870E+04, &
      1.595306E-02, 4.97290E+00, 3.1862E+00, 4.0212E+04, &
      1.644213E-02, 4.75222E+00, 3.0571E+00, 3.7436E+04, &
      1.669379E-02, 4.40369E+00, 2.9927E+00, 3.6094E+04, &
      1.676090E-02, 3.99143E+00, 2.9757E+00, 3.5746E+04, &
      1.686157E-02, 4.42715E+00, 3.7563E+00, 4.4854E+04, &
      1.705382E-02, 4.74215E+00, 3.6743E+00, 4.3380E+04, &
      1.711323E-02, 4.79144E+00, 3.6495E+00, 4.2937E+04, &
      1.823053E-02, 5.03153E+00, 3.2248E+00, 3.5615E+04, &
      1.948844E-02, 4.88825E+00, 2.8269E+00, 2.9205E+04, &
      2.083314E-02, 4.55514E+00, 2.4755E+00, 2.3924E+04, &
      2.227063E-02, 4.05987E+00, 2.1655E+00, 1.9578E+04, &
      2.380730E-02, 3.39001E+00, 1.8925E+00, 1.6005E+04, &
      2.545001E-02, 2.50807E+00, 1.6524E+00, 1.3072E+04, &
      2.720606E-02, 1.33278E+00, 1.4416E+00, 1.0668E+04, &
      2.908327E-02, -3.08155E-01, 1.2569E+00, 8.7014E+03, &
      3.077200E-02, -2.46486E+00, 1.1190E+00, 7.3217E+03, &
      3.109002E-02, -3.08405E+00, 1.0956E+00, 7.0951E+03, &
      3.124300E-02, -3.49185E+00, 1.0846E+00, 6.9894E+03, &
      3.136860E-02, -4.11851E+00, 1.0757E+00, 6.9041E+03, &
      3.155700E-02, -4.04359E+00, 1.9350E+00, 1.2346E+04, &
      3.202800E-02, -4.46733E+00, 2.1199E+00, 1.3327E+04, &
      3.323523E-02, -6.27486E+00, 2.8516E+00, 1.7275E+04, &
      3.552846E-02, -1.03112E+01, 5.8749E+00, 3.3293E+04, &
      3.797993E-02, -1.37002E+01, 1.1670E+01, 6.1867E+04, &
      4.060054E-02, -1.46203E+01, 1.9832E+01, 9.8350E+04, &
      4.340198E-02, -1.17266E+01, 2.8708E+01, 1.3317E+05, &
      4.639671E-02, -5.05348E+00, 3.6047E+01, 1.5643E+05, &
      4.959809E-02, 4.00012E+00, 4.0152E+01, 1.6299E+05, &
      5.302035E-02, 1.34722E+01, 4.0538E+01, 1.5394E+05, &
      5.667876E-02, 2.17164E+01, 3.7827E+01, 1.3437E+05, &
      6.058959E-02, 2.78497E+01, 3.3190E+01, 1.1029E+05, &
      6.477028E-02, 3.17236E+01, 2.7795E+01, 8.6400E+04, &
      6.923942E-02, 3.36554E+01, 2.2503E+01, 6.5438E+04, &
      7.401695E-02, 3.41429E+01, 1.7810E+01, 4.8446E+04, &
      7.912411E-02, 3.36700E+01, 1.3907E+01, 3.5388E+04, &
      8.458368E-02, 3.26109E+01, 1.0798E+01, 2.5704E+04, &
      9.041995E-02, 3.11832E+01, 8.3912E+00, 1.8685E+04, &
      9.634786E-02, 2.93011E+01, 6.6374E+00, 1.3870E+04, &
      9.665893E-02, 2.91520E+01, 6.5603E+00, 1.3665E+04, &
      9.782257E-02, 2.82875E+01, 6.2831E+00, 1.2932E+04, &
      9.821583E-02, 2.73356E+01, 6.1932E+00, 1.2696E+04, &
      9.880571E-02, 2.81544E+01, 7.7889E+00, 1.5872E+04, &
      1.002804E-01, 2.86859E+01, 7.4102E+00, 1.4878E+04, &
      1.033284E-01, 2.86682E+01, 6.7105E+00, 1.3077E+04, &
      1.104581E-01, 2.78604E+01, 5.4257E+00, 9.8905E+03, &
      1.180797E-01, 2.67265E+01, 4.4669E+00, 7.6173E+03, &
      1.231813E-01, 2.57564E+01, 3.9909E+00, 6.5240E+03, &
      1.250667E-01, 2.50497E+01, 3.8405E+00, 6.1835E+03, &
      1.255695E-01, 2.44003E+01, 3.8024E+00, 6.0976E+03, &
      1.262272E-01, 2.48901E+01, 4.9240E+00, 7.8549E+03, &
      1.263237E-01, 2.49441E+01, 4.9147E+00, 7.8341E+03, &
      1.282091E-01, 2.52820E+01, 4.7404E+00, 7.4452E+03, &
      1.349368E-01, 2.50633E+01, 4.2126E+00, 6.2865E+03, &
      1.442475E-01, 2.43241E+01, 3.6646E+00, 5.1160E+03, &
      1.542005E-01, 2.33895E+01, 3.2370E+00, 4.2278E+03, &
      1.648404E-01, 2.22058E+01, 2.9046E+00, 3.5492E+03, &
      1.659746E-01, 2.20528E+01, 2.8750E+00, 3.4891E+03, &
      1.674882E-01, 2.18240E+01, 2.8371E+00, 3.4119E+03, &
      1.685150E-01, 2.16308E+01, 2.8123E+00, 3.3615E+03, &
      1.691924E-01, 2.14179E+01, 2.7963E+00, 3.3291E+03, &
      1.702086E-01, 2.13609E+01, 3.0056E+00, 3.5568E+03, &
      1.707354E-01, 2.12674E+01, 3.0006E+00, 3.5399E+03, &
      1.718369E-01, 2.12395E+01, 3.1272E+00, 3.6656E+03, &
      1.727490E-01, 2.11695E+01, 3.1215E+00, 3.6396E+03, &
      1.744671E-01, 2.09684E+01, 3.1137E+00, 3.5949E+03, &
      1.751685E-01, 2.08169E+01, 3.1117E+00, 3.5781E+03, &
      1.762144E-01, 2.07907E+01, 3.2793E+00, 3.7484E+03, &
      1.762205E-01, 2.07905E+01, 3.2793E+00, 3.7483E+03, &
      1.788507E-01, 2.05881E+01, 3.3075E+00, 3.7250E+03, &
      1.883732E-01, 1.96124E+01, 3.5269E+00, 3.7714E+03, &
      2.013709E-01, 1.82002E+01, 4.1359E+00, 4.1372E+03, &
      2.152655E-01, 1.67447E+01, 5.1630E+00, 4.8313E+03, &
      2.301188E-01, 1.53646E+01, 6.6594E+00, 5.8290E+03, &
      2.459970E-01, 1.42028E+01, 8.6349E+00, 7.0701E+03, &
      2.629708E-01, 1.34078E+01, 1.1044E+01, 8.4588E+03, &
      2.811158E-01, 1.31090E+01, 1.3786E+01, 9.8768E+03, &
      3.005128E-01, 1.33937E+01, 1.6714E+01, 1.1202E+04, &
      3.212482E-01, 1.42949E+01, 1.9659E+01, 1.2325E+04, &
      3.434143E-01, 1.57831E+01, 2.2449E+01, 1.3166E+04, &
      3.671099E-01, 1.77728E+01, 2.4932E+01, 1.3679E+04, &
      3.924405E-01, 2.01271E+01, 2.6992E+01, 1.3853E+04, &
      4.195189E-01, 2.26628E+01, 2.8551E+01, 1.3708E+04, &
      4.484657E-01, 2.50738E+01, 2.9576E+01, 1.3284E+04, &
      4.639320E-01, 2.59443E+01, 2.9893E+01, 1.2979E+04, &
      4.710330E-01, 2.58165E+01, 2.9991E+01, 1.2826E+04, &
      4.729266E-01, 2.51573E+01, 3.0013E+01, 1.2784E+04, &
      4.757670E-01, 2.61423E+01, 3.1612E+01, 1.3384E+04, &
      4.794098E-01, 2.68550E+01, 3.1677E+01, 1.3310E+04, &
      4.828680E-01, 2.73126E+01, 3.1733E+01, 1.3238E+04, &
      4.901960E-01, 2.80349E+01, 3.1835E+01, 1.3082E+04, &
      4.976990E-01, 2.83940E+01, 3.1915E+01, 1.2917E+04, &
      4.996998E-01, 2.80886E+01, 3.1933E+01, 1.2873E+04, &
      5.0000000E-01, 2.98543E+01, 3.1936E+01, 1.2866E+04, &
      5.0056912E-01, 3.00971E+01, 3.2917E+01, 1.3246E+04, &
      5.0250000E-01, 3.08473E+01, 3.2943E+01, 1.3206E+04, &
      5.0501250E-01, 3.13145E+01, 3.2975E+01, 1.3153E+04, &
      5.0753756E-01, 3.16828E+01, 3.3005E+01, 1.3100E+04, &
      5.1007525E-01, 3.20415E+01, 3.3033E+01, 1.3046E+04, &
      5.1262563E-01, 3.23479E+01, 3.3059E+01, 1.2991E+04, &
      5.1518875E-01, 3.26413E+01, 3.3083E+01, 1.2936E+04, &
      5.1776470E-01, 3.29232E+01, 3.3106E+01, 1.2880E+04, &
      5.2035352E-01, 3.31976E+01, 3.3126E+01, 1.2824E+04, &
      5.2295529E-01, 3.34661E+01, 3.3145E+01, 1.2768E+04, &
      5.2557007E-01, 3.37299E+01, 3.3162E+01, 1.2711E+04, &
      5.2819792E-01, 3.40177E+01, 3.3176E+01, 1.2653E+04, &
      5.3083891E-01, 3.42740E+01, 3.3189E+01, 1.2595E+04, &
      5.3349310E-01, 3.45272E+01, 3.3199E+01, 1.2536E+04, &
      5.3616057E-01, 3.47775E+01, 3.3207E+01, 1.2477E+04, &
      5.3884137E-01, 3.50253E+01, 3.3213E+01, 1.2417E+04, &
      5.4153558E-01, 3.52706E+01, 3.3216E+01, 1.2357E+04, &
      5.4424325E-01, 3.55138E+01, 3.3218E+01, 1.2296E+04, &
      5.4696447E-01, 3.57550E+01, 3.3218E+01, 1.2235E+04, &
      5.4969929E-01, 3.59943E+01, 3.3216E+01, 1.2173E+04, &
      5.5244779E-01, 3.62318E+01, 3.3212E+01, 1.2111E+04, &
      5.5521003E-01, 3.64677E+01, 3.3206E+01, 1.2049E+04, &
      5.5798608E-01, 3.67019E+01, 3.3197E+01, 1.1986E+04, &
      5.6077601E-01, 3.69344E+01, 3.3187E+01, 1.1922E+04, &
      5.6357989E-01, 3.71652E+01, 3.3174E+01, 1.1859E+04, &
      5.6639779E-01, 3.73942E+01, 3.3159E+01, 1.1794E+04, &
      5.6922978E-01, 3.76216E+01, 3.3142E+01, 1.1730E+04, &
      5.7207593E-01, 3.78472E+01, 3.3123E+01, 1.1665E+04, &
      5.7493630E-01, 3.80710E+01, 3.3101E+01, 1.1599E+04, &
      5.7781099E-01, 3.82931E+01, 3.3078E+01, 1.1534E+04, &
      5.8070004E-01, 3.85135E+01, 3.3053E+01, 1.1468E+04, &
      5.8360354E-01, 3.87320E+01, 3.3026E+01, 1.1401E+04, &
      5.8652156E-01, 3.89487E+01, 3.2997E+01, 1.1335E+04, &
      5.8945417E-01, 3.91636E+01, 3.2966E+01, 1.1268E+04, &
      5.9240144E-01, 3.93765E+01, 3.2934E+01, 1.1201E+04, &
      5.9536345E-01, 3.95876E+01, 3.2900E+01, 1.1134E+04, &
      5.9834026E-01, 3.97968E+01, 3.2865E+01, 1.1067E+04, &
      6.0133196E-01, 4.00040E+01, 3.2827E+01, 1.0999E+04, &
      6.0433862E-01, 4.02092E+01, 3.2788E+01, 1.0931E+04, &
      6.0736032E-01, 4.04124E+01, 3.2747E+01, 1.0863E+04, &
      6.1039712E-01, 4.06135E+01, 3.2705E+01, 1.0795E+04, &
      6.1344910E-01, 4.08124E+01, 3.2661E+01, 1.0727E+04, &
      6.1651635E-01, 4.10091E+01, 3.2615E+01, 1.0659E+04, &
      6.1959893E-01, 4.12036E+01, 3.2567E+01, 1.0591E+04, &
      6.2269693E-01, 4.13956E+01, 3.2518E+01, 1.0522E+04, &
      6.2581041E-01, 4.15853E+01, 3.2468E+01, 1.0454E+04, &
      6.2893946E-01, 4.17724E+01, 3.2416E+01, 1.0385E+04, &
      6.3208416E-01, 4.19568E+01, 3.2362E+01, 1.0316E+04, &
      6.3524458E-01, 4.21385E+01, 3.2307E+01, 1.0248E+04, &
      6.3842080E-01, 4.23174E+01, 3.2250E+01, 1.0179E+04, &
      6.4161291E-01, 4.24931E+01, 3.2193E+01, 1.0110E+04, &
      6.4482097E-01, 4.26657E+01, 3.2133E+01, 1.0042E+04, &
      6.4804508E-01, 4.28349E+01, 3.2073E+01, 9.9728E+03, &
      6.5128530E-01, 4.30003E+01, 3.2011E+01, 9.9042E+03, &
      6.5454173E-01, 4.31619E+01, 3.1948E+01, 9.8356E+03, &
      6.5781444E-01, 4.33191E+01, 3.1884E+01, 9.7670E+03, &
      6.6110351E-01, 4.34716E+01, 3.1818E+01, 9.6986E+03, &
      6.6440903E-01, 4.36189E+01, 3.1752E+01, 9.6302E+03, &
      6.6773107E-01, 4.37603E+01, 3.1684E+01, 9.5619E+03, &
      6.7106973E-01, 4.38949E+01, 3.1615E+01, 9.4938E+03, &
      6.7442508E-01, 4.40216E+01, 3.1545E+01, 9.4257E+03, &
      6.7779720E-01, 4.41389E+01, 3.1474E+01, 9.3578E+03, &
      6.8118619E-01, 4.42447E+01, 3.1402E+01, 9.2901E+03, &
      6.8459212E-01, 4.43358E+01, 3.1329E+01, 9.2224E+03, &
      6.8801508E-01, 4.44072E+01, 3.1255E+01, 9.1548E+03, &
      6.9145515E-01, 4.44505E+01, 3.1179E+01, 9.0874E+03, &
      6.9491243E-01, 4.44498E+01, 3.1103E+01, 9.0201E+03, &
      6.9838699E-01, 4.43684E+01, 3.1026E+01, 8.9531E+03, &
      7.0187893E-01, 4.40818E+01, 3.0947E+01, 8.8861E+03, &
      7.0406938E-01, 4.34430E+01, 3.0898E+01, 8.8444E+03, &
      7.0538832E-01, 4.29682E+01, 3.2812E+01, 9.3742E+03, &
      7.0593058E-01, 4.35357E+01, 3.2800E+01, 9.3636E+03, &
      7.0891526E-01, 4.45722E+01, 3.2734E+01, 9.3055E+03, &
      7.1245984E-01, 4.51446E+01, 3.2655E+01, 9.2369E+03, &
      7.1602214E-01, 4.55585E+01, 3.2575E+01, 9.1686E+03, &
      7.1960225E-01, 4.59035E+01, 3.2494E+01, 9.1004E+03, &
      7.2320026E-01, 4.62095E+01, 3.2413E+01, 9.0326E+03, &
      7.2681626E-01, 4.64898E+01, 3.2331E+01, 8.9649E+03, &
      7.3045034E-01, 4.67520E+01, 3.2248E+01, 8.8975E+03, &
      7.3410260E-01, 4.70003E+01, 3.2164E+01, 8.8304E+03, &
      7.3777311E-01, 4.72377E+01, 3.2080E+01, 8.7636E+03, &
      7.4146197E-01, 4.74661E+01, 3.1995E+01, 8.6971E+03, &
      7.4516928E-01, 4.76871E+01, 3.1910E+01, 8.6309E+03, &
      7.4889513E-01, 4.79015E+01, 3.1825E+01, 8.5650E+03, &
      7.5263961E-01, 4.81103E+01, 3.1738E+01, 8.4994E+03, &
      7.5640280E-01, 4.83141E+01, 3.1652E+01, 8.4342E+03, &
      7.6018482E-01, 4.85134E+01, 3.1565E+01, 8.3693E+03, &
      7.6398574E-01, 4.87087E+01, 3.1478E+01, 8.3048E+03, &
      7.6780567E-01, 4.89002E+01, 3.1391E+01, 8.2407E+03, &
      7.7164470E-01, 4.90883E+01, 3.1303E+01, 8.1769E+03, &
      7.7550292E-01, 4.92733E+01, 3.1216E+01, 8.1135E+03, &
      7.7938044E-01, 4.94554E+01, 3.1128E+01, 8.0505E+03, &
      7.8327734E-01, 4.96348E+01, 3.1040E+01, 7.9879E+03, &
      7.8719373E-01, 4.98118E+01, 3.0952E+01, 7.9257E+03, &
      7.9112969E-01, 4.99864E+01, 3.0863E+01, 7.8639E+03, &
      7.9508534E-01, 5.01589E+01, 3.0775E+01, 7.8025E+03, &
      7.9906077E-01, 5.03294E+01, 3.0687E+01, 7.7415E+03, &
      8.0305607E-01, 5.04968E+01, 3.0591E+01, 7.6790E+03, &
      8.0707135E-01, 5.06603E+01, 3.0492E+01, 7.6162E+03, &
      8.1110671E-01, 5.08200E+01, 3.0393E+01, 7.5539E+03, &
      8.1516224E-01, 5.09759E+01, 3.0294E+01, 7.4919E+03, &
      8.1923806E-01, 5.11277E+01, 3.0195E+01, 7.4303E+03, &
      8.2333425E-01, 5.12751E+01, 3.0095E+01, 7.3691E+03, &
      8.2745092E-01, 5.14174E+01, 2.9996E+01, 7.3083E+03, &
      8.3158817E-01, 5.15535E+01, 2.9896E+01, 7.2479E+03, &
      8.3574611E-01, 5.16817E+01, 2.9797E+01, 7.1879E+03, &
      8.3992484E-01, 5.17978E+01, 2.9697E+01, 7.1284E+03, &
      8.4412447E-01, 5.18920E+01, 2.9598E+01, 7.0692E+03, &
      8.4834509E-01, 5.19223E+01, 2.9494E+01, 7.0095E+03, &
      8.4973202E-01, 5.18761E+01, 2.9459E+01, 6.9898E+03, &
      8.5226800E-01, 5.19655E+01, 2.9803E+01, 7.0504E+03, &
      8.5258682E-01, 5.20058E+01, 2.9795E+01, 7.0459E+03, &
      8.5684975E-01, 5.23234E+01, 2.9691E+01, 6.9865E+03, &
      8.6113400E-01, 5.25415E+01, 2.9587E+01, 6.9275E+03, &
      8.6543967E-01, 5.27324E+01, 2.9483E+01, 6.8688E+03, &
      8.6976687E-01, 5.29096E+01, 2.9379E+01, 6.8106E+03, &
      8.7411570E-01, 5.30779E+01, 2.9275E+01, 6.7528E+03, &
      8.7848628E-01, 5.32398E+01, 2.9171E+01, 6.6954E+03, &
      8.8287871E-01, 5.33966E+01, 2.9067E+01, 6.6385E+03, &
      8.8729310E-01, 5.35493E+01, 2.8963E+01, 6.5819E+03, &
      8.9172957E-01, 5.36983E+01, 2.8859E+01, 6.5258E+03, &
      8.9618822E-01, 5.38441E+01, 2.8755E+01, 6.4701E+03, &
      9.0066916E-01, 5.39870E+01, 2.8651E+01, 6.4149E+03, &
      9.0517250E-01, 5.41272E+01, 2.8548E+01, 6.3600E+03, &
      9.0969837E-01, 5.42648E+01, 2.8445E+01, 6.3056E+03, &
      9.1424686E-01, 5.44000E+01, 2.8342E+01, 6.2517E+03, &
      9.1881809E-01, 5.45328E+01, 2.8240E+01, 6.1982E+03, &
      9.2341218E-01, 5.46635E+01, 2.8137E+01, 6.1451E+03, &
      9.2802924E-01, 5.47919E+01, 2.8035E+01, 6.0924E+03, &
      9.3266939E-01, 5.49181E+01, 2.7933E+01, 6.0402E+03, &
      9.3733274E-01, 5.50421E+01, 2.7832E+01, 5.9885E+03, &
      9.4201940E-01, 5.51638E+01, 2.7731E+01, 5.9372E+03, &
      9.4672950E-01, 5.52832E+01, 2.7630E+01, 5.8863E+03, &
      9.5146315E-01, 5.54000E+01, 2.7530E+01, 5.8359E+03, &
      9.5622046E-01, 5.55139E+01, 2.7430E+01, 5.7859E+03, &
      9.6100156E-01, 5.56246E+01, 2.7331E+01, 5.7363E+03, &
      9.6580657E-01, 5.57312E+01, 2.7232E+01, 5.6872E+03, &
      9.7063560E-01, 5.58328E+01, 2.7133E+01, 5.6385E+03, &
      9.7548878E-01, 5.59271E+01, 2.7035E+01, 5.5903E+03, &
      9.8036623E-01, 5.60104E+01, 2.6937E+01, 5.5425E+03, &
      9.8526806E-01, 5.60733E+01, 2.6840E+01, 5.4952E+03, &
      9.9019440E-01, 5.60866E+01, 2.6744E+01, 5.4483E+03, &
      9.9395634E-01, 5.59468E+01, 2.6671E+01, 5.4129E+03, &
      9.9514537E-01, 5.55822E+01, 2.6648E+01, 5.4018E+03, &
      9.9664365E-01, 5.60239E+01, 2.7198E+01, 5.5049E+03, &
      1.0001211E+00, 5.63594E+01, 2.7128E+01, 5.4718E+03, &
      1.0051217E+00, 5.66399E+01, 2.6963E+01, 5.4115E+03, &
      1.0101473E+00, 5.68589E+01, 2.6799E+01, 5.3520E+03, &
      1.0151980E+00, 5.70500E+01, 2.6636E+01, 5.2932E+03, &
      1.0202740E+00, 5.72236E+01, 2.6475E+01, 5.2351E+03, &
      1.0253754E+00, 5.73845E+01, 2.6315E+01, 5.1777E+03, &
      1.0305023E+00, 5.75356E+01, 2.6156E+01, 5.1210E+03, &
      1.0356548E+00, 5.76785E+01, 2.5999E+01, 5.0650E+03, &
      1.0408331E+00, 5.78144E+01, 2.5843E+01, 5.0097E+03, &
      1.0460372E+00, 5.79441E+01, 2.5688E+01, 4.9550E+03, &
      1.0512674E+00, 5.80682E+01, 2.5534E+01, 4.9010E+03, &
      1.0565238E+00, 5.81873E+01, 2.5382E+01, 4.8477E+03, &
      1.0618064E+00, 5.83018E+01, 2.5231E+01, 4.7950E+03, &
      1.0671154E+00, 5.84121E+01, 2.5081E+01, 4.7429E+03, &
      1.0724510E+00, 5.85183E+01, 2.4932E+01, 4.6915E+03, &
      1.0778132E+00, 5.86209E+01, 2.4785E+01, 4.6407E+03, &
      1.0832023E+00, 5.87199E+01, 2.4639E+01, 4.5905E+03, &
      1.0886183E+00, 5.88156E+01, 2.4494E+01, 4.5409E+03, &
      1.0940614E+00, 5.89081E+01, 2.4350E+01, 4.4919E+03, &
      1.0995317E+00, 5.89976E+01, 2.4207E+01, 4.4436E+03, &
      1.1050294E+00, 5.90844E+01, 2.4066E+01, 4.3958E+03, &
      1.1105545E+00, 5.91686E+01, 2.3926E+01, 4.3487E+03, &
      1.1161073E+00, 5.92503E+01, 2.3788E+01, 4.3021E+03, &
      1.1216878E+00, 5.93297E+01, 2.3650E+01, 4.2560E+03, &
      1.1272963E+00, 5.94067E+01, 2.3514E+01, 4.2106E+03, &
      1.1329328E+00, 5.94816E+01, 2.3378E+01, 4.1656E+03, &
      1.1385974E+00, 5.95544E+01, 2.3244E+01, 4.1213E+03, &
      1.1442904E+00, 5.96251E+01, 2.3111E+01, 4.0774E+03, &
      1.1500119E+00, 5.96940E+01, 2.2979E+01, 4.0341E+03, &
      1.1557619E+00, 5.97609E+01, 2.2848E+01, 3.9913E+03, &
      1.1615407E+00, 5.98261E+01, 2.2718E+01, 3.9490E+03, &
      1.1673484E+00, 5.98896E+01, 2.2589E+01, 3.9072E+03, &
      1.1731852E+00, 5.99514E+01, 2.2461E+01, 3.8659E+03, &
      1.1790511E+00, 6.00117E+01, 2.2335E+01, 3.8251E+03, &
      1.1849464E+00, 6.00705E+01, 2.2209E+01, 3.7848E+03, &
      1.1908711E+00, 6.01279E+01, 2.2084E+01, 3.7450E+03, &
      1.1968254E+00, 6.01836E+01, 2.1959E+01, 3.7053E+03, &
      1.2028096E+00, 6.02376E+01, 2.1834E+01, 3.6660E+03, &
      1.2088236E+00, 6.02899E+01, 2.1710E+01, 3.6272E+03, &
      1.2148677E+00, 6.03407E+01, 2.1587E+01, 3.5888E+03, &
      1.2209421E+00, 6.03899E+01, 2.1465E+01, 3.5510E+03, &
      1.2270468E+00, 6.04376E+01, 2.1344E+01, 3.5135E+03, &
      1.2331820E+00, 6.04838E+01, 2.1224E+01, 3.4765E+03, &
      1.2393479E+00, 6.05287E+01, 2.1105E+01, 3.4400E+03, &
      1.2455447E+00, 6.05722E+01, 2.0987E+01, 3.4038E+03, &
      1.2517724E+00, 6.06144E+01, 2.0870E+01, 3.3681E+03, &
      1.2580312E+00, 6.06554E+01, 2.0754E+01, 3.3328E+03, &
      1.2643214E+00, 6.06950E+01, 2.0638E+01, 3.2980E+03, &
      1.2706430E+00, 6.07334E+01, 2.0524E+01, 3.2635E+03, &
      1.2769962E+00, 6.07707E+01, 2.0411E+01, 3.2294E+03, &
      1.2833812E+00, 6.08067E+01, 2.0298E+01, 3.1958E+03, &
      1.2897981E+00, 6.08417E+01, 2.0186E+01, 3.1625E+03, &
      1.2962471E+00, 6.08755E+01, 2.0075E+01, 3.1296E+03, &
      1.3027283E+00, 6.09082E+01, 1.9966E+01, 3.0972E+03, &
      1.3092420E+00, 6.09398E+01, 1.9856E+01, 3.0650E+03, &
      1.3157882E+00, 6.09704E+01, 1.9748E+01, 3.0333E+03, &
      1.3223671E+00, 6.10000E+01, 1.9641E+01, 3.0019E+03, &
      1.3289790E+00, 6.10285E+01, 1.9534E+01, 2.9709E+03, &
      1.3356239E+00, 6.10561E+01, 1.9429E+01, 2.9403E+03, &
      1.3423020E+00, 6.10827E+01, 1.9324E+01, 2.9100E+03, &
      1.3490135E+00, 6.11084E+01, 1.9220E+01, 2.8801E+03, &
      1.3557586E+00, 6.11331E+01, 1.9117E+01, 2.8505E+03, &
      1.3625374E+00, 6.11569E+01, 1.9014E+01, 2.8213E+03, &
      1.3693500E+00, 6.11798E+01, 1.8913E+01, 2.7923E+03, &
      1.3761968E+00, 6.12018E+01, 1.8812E+01, 2.7638E+03, &
      1.3830778E+00, 6.12230E+01, 1.8712E+01, 2.7355E+03, &
      1.3899932E+00, 6.12433E+01, 1.8613E+01, 2.7076E+03, &
      1.3969431E+00, 6.12628E+01, 1.8514E+01, 2.6800E+03, &
      1.4039278E+00, 6.12814E+01, 1.8416E+01, 2.6528E+03, &
      1.4109475E+00, 6.12993E+01, 1.8320E+01, 2.6258E+03, &
      1.4180022E+00, 6.13164E+01, 1.8223E+01, 2.5991E+03, &
      1.4250922E+00, 6.13461E+01, 1.8128E+01, 2.5728E+03, &
      1.4322177E+00, 6.13620E+01, 1.8033E+01, 2.5468E+03, &
      1.4393788E+00, 6.13772E+01, 1.7939E+01, 2.5210E+03, &
      1.4465757E+00, 6.13914E+01, 1.7845E+01, 2.4954E+03, &
      1.4538086E+00, 6.14047E+01, 1.7751E+01, 2.4700E+03, &
      1.4610776E+00, 6.14171E+01, 1.7658E+01, 2.4450E+03, &
      1.4683830E+00, 6.14287E+01, 1.7566E+01, 2.4202E+03, &
      1.4757249E+00, 6.14394E+01, 1.7474E+01, 2.3958E+03, &
      1.4831035E+00, 6.14493E+01, 1.7383E+01, 2.3716E+03, &
      1.4905190E+00, 6.14583E+01, 1.7293E+01, 2.3476E+03, &
      1.4979716E+00, 6.14666E+01, 1.7203E+01, 2.3240E+03, &
      1.5054615E+00, 6.14876E+01, 1.7114E+01, 2.3006E+03, &
      1.5129888E+00, 6.14944E+01, 1.7026E+01, 2.2775E+03, &
      1.5205537E+00, 6.15003E+01, 1.6938E+01, 2.2546E+03, &
      1.5281565E+00, 6.15055E+01, 1.6851E+01, 2.2320E+03, &
      1.5357973E+00, 6.15099E+01, 1.6765E+01, 2.2097E+03, &
      1.5434763E+00, 6.15135E+01, 1.6680E+01, 2.1876E+03, &
      1.5511937E+00, 6.15163E+01, 1.6595E+01, 2.1657E+03, &
      1.5589496E+00, 6.15184E+01, 1.6510E+01, 2.1441E+03, &
      1.5667444E+00, 6.15197E+01, 1.6426E+01, 2.1228E+03, &
      1.5745781E+00, 6.15202E+01, 1.6343E+01, 2.1016E+03, &
      1.5824510E+00, 6.15200E+01, 1.6261E+01, 2.0807E+03, &
      1.5903633E+00, 6.15190E+01, 1.6179E+01, 2.0601E+03, &
      1.5983151E+00, 6.15173E+01, 1.6098E+01, 2.0397E+03, &
      1.6063066E+00, 6.15148E+01, 1.6017E+01, 2.0195E+03, &
      1.6143382E+00, 6.15116E+01, 1.5937E+01, 1.9995E+03, &
      1.6224099E+00, 6.15076E+01, 1.5857E+01, 1.9798E+03, &
      1.6305219E+00, 6.15028E+01, 1.5778E+01, 1.9602E+03, &
      1.6386745E+00, 6.14973E+01, 1.5700E+01, 1.9409E+03, &
      1.6468679E+00, 6.14911E+01, 1.5622E+01, 1.9218E+03, &
      1.6551022E+00, 6.14841E+01, 1.5545E+01, 1.9029E+03, &
      1.6633777E+00, 6.14764E+01, 1.5469E+01, 1.8842E+03, &
      1.6716946E+00, 6.14680E+01, 1.5392E+01, 1.8658E+03, &
      1.6800531E+00, 6.14588E+01, 1.5317E+01, 1.8475E+03, &
      1.6884534E+00, 6.14488E+01, 1.5241E+01, 1.8294E+03, &
      1.6968956E+00, 6.14379E+01, 1.5166E+01, 1.8114E+03, &
      1.7053801E+00, 6.14261E+01, 1.5092E+01, 1.7937E+03, &
      1.7139070E+00, 6.14135E+01, 1.5018E+01, 1.7761E+03, &
      1.7224766E+00, 6.14001E+01, 1.4944E+01, 1.7587E+03, &
      1.7310889E+00, 6.13857E+01, 1.4871E+01, 1.7416E+03, &
      1.7397444E+00, 6.13705E+01, 1.4799E+01, 1.7246E+03, &
      1.7484431E+00, 6.13544E+01, 1.4727E+01, 1.7078E+03, &
      1.7571853E+00, 6.13374E+01, 1.4655E+01, 1.6911E+03, &
      1.7659712E+00, 6.13196E+01, 1.4584E+01, 1.6747E+03, &
      1.7748011E+00, 6.13009E+01, 1.4514E+01, 1.6584E+03, &
      1.7836751E+00, 6.12812E+01, 1.4444E+01, 1.6424E+03, &
      1.7925935E+00, 6.12607E+01, 1.4374E+01, 1.6264E+03, &
      1.8015565E+00, 6.12393E+01, 1.4305E+01, 1.6107E+03, &
      1.8105642E+00, 6.12169E+01, 1.4237E+01, 1.5951E+03, &
      1.8196171E+00, 6.11936E+01, 1.4169E+01, 1.5798E+03, &
      1.8287151E+00, 6.11694E+01, 1.4102E+01, 1.5645E+03, &
      1.8378587E+00, 6.11442E+01, 1.4035E+01, 1.5495E+03, &
      1.8470480E+00, 6.11181E+01, 1.3968E+01, 1.5346E+03, &
      1.8562833E+00, 6.10910E+01, 1.3902E+01, 1.5198E+03, &
      1.8655647E+00, 6.10629E+01, 1.3836E+01, 1.5052E+03, &
      1.8748925E+00, 6.10339E+01, 1.3770E+01, 1.4907E+03, &
      1.8842670E+00, 6.10038E+01, 1.3705E+01, 1.4764E+03, &
      1.8936883E+00, 6.09727E+01, 1.3641E+01, 1.4623E+03, &
      1.9031567E+00, 6.09402E+01, 1.3568E+01, 1.4473E+03, &
      1.9126725E+00, 6.09060E+01, 1.3495E+01, 1.4325E+03, &
      1.9222359E+00, 6.08699E+01, 1.3423E+01, 1.4179E+03, &
      1.9318471E+00, 6.08321E+01, 1.3351E+01, 1.4034E+03, &
      1.9415063E+00, 6.07925E+01, 1.3280E+01, 1.3891E+03, &
      1.9512138E+00, 6.07511E+01, 1.3209E+01, 1.3750E+03, &
      1.9609699E+00, 6.07078E+01, 1.3139E+01, 1.3610E+03, &
      1.9707747E+00, 6.06627E+01, 1.3070E+01, 1.3472E+03, &
      1.9806286E+00, 6.06158E+01, 1.3001E+01, 1.3335E+03, &
      1.9905318E+00, 6.05669E+01, 1.2932E+01, 1.3200E+03, &
      2.0004844E+00, 6.05161E+01, 1.2864E+01, 1.3066E+03, &
      2.0104868E+00, 6.04632E+01, 1.2790E+01, 1.2928E+03, &
      2.0205393E+00, 6.04077E+01, 1.2717E+01, 1.2791E+03, &
      2.0306420E+00, 6.03497E+01, 1.2644E+01, 1.2656E+03, &
      2.0407952E+00, 6.02891E+01, 1.2572E+01, 1.2523E+03, &
      2.0509992E+00, 6.02259E+01, 1.2500E+01, 1.2390E+03, &
      2.0612542E+00, 6.01601E+01, 1.2429E+01, 1.2260E+03, &
      2.0715604E+00, 6.00915E+01, 1.2358E+01, 1.2131E+03, &
      2.0819182E+00, 6.00200E+01, 1.2288E+01, 1.2003E+03, &
      2.0923278E+00, 5.99458E+01, 1.2219E+01, 1.1877E+03, &
      2.1027895E+00, 5.98685E+01, 1.2149E+01, 1.1752E+03, &
      2.1133034E+00, 5.97882E+01, 1.2081E+01, 1.1629E+03, &
      2.1238699E+00, 5.97080E+01, 1.2013E+01, 1.1507E+03, &
      2.1344893E+00, 5.96213E+01, 1.1945E+01, 1.1386E+03, &
      2.1451617E+00, 5.95313E+01, 1.1878E+01, 1.1267E+03, &
      2.1558875E+00, 5.94378E+01, 1.1811E+01, 1.1149E+03, &
      2.1666670E+00, 5.93407E+01, 1.1745E+01, 1.1033E+03, &
      2.1775003E+00, 5.92399E+01, 1.1679E+01, 1.0917E+03, &
      2.1883878E+00, 5.91351E+01, 1.1613E+01, 1.0804E+03, &
      2.1993297E+00, 5.90262E+01, 1.1549E+01, 1.0691E+03, &
      2.2103264E+00, 5.89130E+01, 1.1484E+01, 1.0580E+03, &
      2.2213780E+00, 5.87954E+01, 1.1420E+01, 1.0470E+03, &
      2.2324849E+00, 5.86731E+01, 1.1357E+01, 1.0361E+03, &
      2.2436473E+00, 5.85458E+01, 1.1294E+01, 1.0253E+03, &
      2.2548656E+00, 5.84134E+01, 1.1231E+01, 1.0147E+03, &
      2.2661399E+00, 5.82755E+01, 1.1169E+01, 1.0042E+03, &
      2.2774706E+00, 5.81319E+01, 1.1107E+01, 9.9376E+02, &
      2.2888579E+00, 5.79821E+01, 1.1046E+01, 9.8347E+02, &
      2.3003022E+00, 5.78259E+01, 1.0985E+01, 9.7331E+02, &
      2.3118037E+00, 5.76628E+01, 1.0925E+01, 9.6325E+02, &
      2.3233628E+00, 5.74924E+01, 1.0865E+01, 9.5331E+02, &
      2.3349796E+00, 5.73142E+01, 1.0805E+01, 9.4348E+02, &
      2.3466545E+00, 5.71277E+01, 1.0746E+01, 9.3376E+02, &
      2.3583878E+00, 5.69322E+01, 1.0687E+01, 9.2415E+02, &
      2.3701797E+00, 5.67271E+01, 1.0629E+01, 9.1464E+02, &
      2.3820306E+00, 5.65116E+01, 1.0571E+01, 9.0523E+02, &
      2.3939407E+00, 5.62848E+01, 1.0513E+01, 8.9592E+02, &
      2.4059104E+00, 5.60459E+01, 1.0456E+01, 8.8672E+02, &
      2.4179400E+00, 5.57936E+01, 1.0399E+01, 8.7762E+02, &
      2.4300297E+00, 5.55268E+01, 1.0342E+01, 8.6862E+02, &
      2.4421798E+00, 5.52440E+01, 1.0286E+01, 8.5972E+02, &
      2.4543907E+00, 5.49436E+01, 1.0230E+01, 8.5092E+02, &
      2.4666627E+00, 5.46235E+01, 1.0175E+01, 8.4222E+02, &
      2.4789960E+00, 5.42815E+01, 1.0120E+01, 8.3362E+02, &
      2.4913910E+00, 5.39150E+01, 1.0066E+01, 8.2511E+02, &
      2.5038479E+00, 5.35205E+01, 1.0011E+01, 8.1669E+02, &
      2.5163672E+00, 5.30941E+01, 9.9576E+00, 8.0837E+02, &
      2.5289490E+00, 5.26310E+01, 9.9041E+00, 8.0014E+02, &
      2.5415938E+00, 5.21249E+01, 9.8511E+00, 7.9200E+02, &
      2.5543017E+00, 5.15679E+01, 9.7983E+00, 7.8395E+02, &
      2.5670732E+00, 5.09498E+01, 9.7460E+00, 7.7599E+02, &
      2.5799086E+00, 5.02566E+01, 9.6940E+00, 7.6812E+02, &
      2.5928082E+00, 4.94687E+01, 9.6424E+00, 7.6033E+02, &
      2.6057722E+00, 4.85579E+01, 9.5911E+00, 7.5264E+02, &
      2.6188011E+00, 4.74800E+01, 9.5401E+00, 7.4502E+02, &
      2.6318951E+00, 4.61608E+01, 9.4895E+00, 7.3749E+02, &
      2.6450545E+00, 4.44594E+01, 9.4393E+00, 7.3004E+02, &
      2.6582798E+00, 4.20500E+01, 9.3894E+00, 7.2268E+02, &
      2.6715712E+00, 3.78063E+01, 9.3399E+00, 7.1539E+02, &
      2.6822596E+00, 2.31295E+01, 9.3004E+00, 7.0961E+02, &
      2.6837406E+00, 2.28458E+01, 2.6036E+01, 1.9648E+03, &
      2.6849291E+00, 2.79442E+01, 2.6019E+01, 1.9626E+03, &
      2.6983537E+00, 3.88204E+01, 2.5821E+01, 1.9382E+03, &
      2.7118455E+00, 4.18826E+01, 2.5626E+01, 1.9140E+03, &
      2.7254047E+00, 4.35247E+01, 2.5431E+01, 1.8902E+03, &
      2.7390317E+00, 4.44561E+01, 2.5239E+01, 1.8667E+03, &
      2.7527269E+00, 4.48609E+01, 2.5048E+01, 1.8435E+03, &
      2.7664905E+00, 4.47102E+01, 2.4859E+01, 1.8206E+03, &
      2.7803230E+00, 4.36560E+01, 2.4671E+01, 1.7980E+03, &
      2.7942246E+00, 3.90917E+01, 2.4485E+01, 1.7757E+03, &
      2.7971803E+00, 3.38657E+01, 2.4445E+01, 1.7710E+03, &
      2.7988199E+00, 3.37869E+01, 3.5560E+01, 2.5695E+03, &
      2.8081957E+00, 4.32845E+01, 3.5368E+01, 2.5471E+03, &
      2.8222367E+00, 4.70858E+01, 3.5082E+01, 2.5142E+03, &
      2.8363479E+00, 4.93630E+01, 3.4798E+01, 2.4815E+03, &
      2.8505296E+00, 5.10660E+01, 3.4516E+01, 2.4493E+03, &
      2.8647823E+00, 5.24520E+01, 3.4237E+01, 2.4176E+03, &
      2.8791062E+00, 5.36311E+01, 3.3960E+01, 2.3862E+03, &
      2.8935017E+00, 5.46615E+01, 3.3686E+01, 2.3553E+03, &
      2.9079692E+00, 5.55780E+01, 3.3414E+01, 2.3248E+03, &
      2.9225091E+00, 5.64036E+01, 3.3144E+01, 2.2947E+03, &
      2.9371216E+00, 5.71541E+01, 3.2877E+01, 2.2650E+03, &
      2.9518072E+00, 5.78411E+01, 3.2612E+01, 2.2356E+03, &
      2.9665662E+00, 5.84730E+01, 3.2349E+01, 2.2067E+03, &
      2.9813991E+00, 5.90566E+01, 3.2088E+01, 2.1782E+03, &
      2.9963061E+00, 5.95993E+01, 3.1830E+01, 2.1500E+03, &
      3.0112876E+00, 6.00956E+01, 3.1564E+01, 2.1216E+03, &
      3.0263440E+00, 6.05594E+01, 3.1296E+01, 2.0933E+03, &
      3.0414758E+00, 6.09913E+01, 3.1032E+01, 2.0654E+03, &
      3.0566831E+00, 6.13922E+01, 3.0770E+01, 2.0379E+03, &
      3.0719666E+00, 6.17636E+01, 3.0511E+01, 2.0108E+03, &
      3.0873264E+00, 6.21064E+01, 3.0255E+01, 1.9842E+03, &
      3.1027630E+00, 6.24213E+01, 3.0001E+01, 1.9579E+03, &
      3.1182768E+00, 6.27082E+01, 2.9750E+01, 1.9320E+03, &
      3.1338682E+00, 6.29670E+01, 2.9502E+01, 1.9064E+03, &
      3.1495376E+00, 6.31964E+01, 2.9256E+01, 1.8812E+03, &
      3.1652853E+00, 6.33946E+01, 2.9012E+01, 1.8564E+03, &
      3.1811117E+00, 6.35582E+01, 2.8771E+01, 1.8319E+03, &
      3.1970172E+00, 6.36822E+01, 2.8530E+01, 1.8077E+03, &
      3.2130023E+00, 6.37582E+01, 2.8292E+01, 1.7839E+03, &
      3.2290673E+00, 6.37722E+01, 2.8057E+01, 1.7603E+03, &
      3.2452127E+00, 6.36988E+01, 2.7823E+01, 1.7371E+03, &
      3.2614387E+00, 6.34853E+01, 2.7592E+01, 1.7142E+03, &
      3.2777459E+00, 6.29924E+01, 2.7363E+01, 1.6917E+03, &
      3.2941347E+00, 6.15485E+01, 2.7136E+01, 1.6694E+03, &
      3.2982348E+00, 6.04706E+01, 2.7080E+01, 1.6639E+03, &
      3.3055650E+00, 6.05387E+01, 3.1805E+01, 1.9480E+03, &
      3.3106053E+00, 6.19636E+01, 3.1720E+01, 1.9399E+03, &
      3.3271584E+00, 6.38848E+01, 3.1445E+01, 1.9137E+03, &
      3.3437941E+00, 6.49232E+01, 3.1172E+01, 1.8878E+03, &
      3.3605131E+00, 6.56833E+01, 3.0902E+01, 1.8622E+03, &
      3.3773157E+00, 6.63003E+01, 3.0635E+01, 1.8371E+03, &
      3.3942023E+00, 6.68274E+01, 3.0370E+01, 1.8122E+03, &
      3.4111733E+00, 6.72910E+01, 3.0107E+01, 1.7877E+03, &
      3.4282291E+00, 6.77061E+01, 2.9846E+01, 1.7636E+03, &
      3.4453703E+00, 6.80815E+01, 2.9588E+01, 1.7397E+03, &
      3.4625971E+00, 6.84221E+01, 2.9332E+01, 1.7162E+03, &
      3.4799101E+00, 6.87347E+01, 2.9094E+01, 1.6939E+03, &
      3.4973097E+00, 6.90300E+01, 2.8858E+01, 1.6720E+03, &
      3.5147962E+00, 6.93072E+01, 2.8623E+01, 1.6502E+03, &
      3.5323702E+00, 6.95678E+01, 2.8391E+01, 1.6288E+03, &
      3.5500321E+00, 6.98129E+01, 2.8162E+01, 1.6077E+03, &
      3.5677822E+00, 7.00437E+01, 2.7935E+01, 1.5870E+03, &
      3.5856211E+00, 7.02611E+01, 2.7712E+01, 1.5666E+03, &
      3.6035492E+00, 7.04656E+01, 2.7491E+01, 1.5465E+03, &
      3.6215670E+00, 7.06579E+01, 2.7273E+01, 1.5267E+03, &
      3.6396748E+00, 7.08382E+01, 2.7056E+01, 1.5072E+03, &
      3.6578732E+00, 7.10054E+01, 2.6839E+01, 1.4877E+03, &
      3.6761626E+00, 7.11589E+01, 2.6623E+01, 1.4685E+03, &
      3.6945434E+00, 7.12983E+01, 2.6410E+01, 1.4497E+03, &
      3.7130161E+00, 7.14225E+01, 2.6200E+01, 1.4311E+03, &
      3.7315812E+00, 7.15297E+01, 2.5992E+01, 1.4128E+03, &
      3.7502391E+00, 7.16169E+01, 2.5786E+01, 1.3947E+03, &
      3.7689903E+00, 7.16791E+01, 2.5582E+01, 1.3769E+03, &
      3.7878352E+00, 7.17070E+01, 2.5381E+01, 1.3594E+03, &
      3.8067744E+00, 7.16811E+01, 2.5182E+01, 1.3421E+03, &
      3.8258083E+00, 7.15504E+01, 2.4984E+01, 1.3251E+03, &
      3.8449373E+00, 7.10694E+01, 2.4789E+01, 1.3083E+03, &
      3.8485501E+00, 7.08145E+01, 2.4753E+01, 1.3052E+03, &
      3.8596499E+00, 7.08842E+01, 2.6427E+01, 1.3888E+03, &
      3.8641620E+00, 7.12611E+01, 2.6376E+01, 1.3845E+03, &
      3.8834828E+00, 7.20211E+01, 2.6160E+01, 1.3664E+03, &
      3.9029002E+00, 7.24465E+01, 2.5946E+01, 1.3486E+03, &
      3.9224147E+00, 7.27642E+01, 2.5735E+01, 1.3311E+03, &
      3.9420268E+00, 7.30245E+01, 2.5526E+01, 1.3138E+03, &
      3.9617369E+00, 7.32467E+01, 2.5318E+01, 1.2968E+03, &
      3.9815456E+00, 7.34401E+01, 2.5113E+01, 1.2800E+03, &
      4.0014533E+00, 7.36091E+01, 2.4910E+01, 1.2634E+03, &
      4.0214606E+00, 7.37554E+01, 2.4708E+01, 1.2471E+03, &
      4.0415679E+00, 7.38785E+01, 2.4509E+01, 1.2310E+03, &
      4.0617757E+00, 7.39776E+01, 2.4321E+01, 1.2155E+03, &
      4.0820846E+00, 7.40522E+01, 2.4137E+01, 1.2004E+03, &
      4.1024950E+00, 7.40886E+01, 2.3955E+01, 1.1856E+03, &
      4.1230075E+00, 7.40483E+01, 2.3775E+01, 1.1709E+03, &
      4.1406035E+00, 7.38077E+01, 2.3624E+01, 1.1586E+03, &
      4.1436226E+00, 7.36888E+01, 2.3598E+01, 1.1565E+03, &
      4.1581970E+00, 7.39217E+01, 2.4520E+01, 1.1971E+03, &
      4.1643407E+00, 7.41397E+01, 2.4468E+01, 1.1928E+03, &
      4.1851624E+00, 7.45674E+01, 2.4293E+01, 1.1785E+03, &
      4.2060882E+00, 7.48537E+01, 2.4119E+01, 1.1644E+03, &
      4.2271186E+00, 7.50878E+01, 2.3948E+01, 1.1504E+03, &
      4.2482542E+00, 7.52936E+01, 2.3779E+01, 1.1367E+03, &
      4.2694955E+00, 7.54812E+01, 2.3612E+01, 1.1232E+03, &
      4.2908430E+00, 7.56559E+01, 2.3446E+01, 1.1099E+03, &
      4.3122972E+00, 7.58197E+01, 2.3277E+01, 1.0965E+03, &
      4.3338587E+00, 7.59739E+01, 2.3108E+01, 1.0832E+03, &
      4.3555280E+00, 7.61202E+01, 2.2942E+01, 1.0701E+03, &
      4.3773056E+00, 7.62596E+01, 2.2776E+01, 1.0572E+03, &
      4.3991921E+00, 7.63930E+01, 2.2613E+01, 1.0445E+03, &
      4.4211881E+00, 7.65210E+01, 2.2451E+01, 1.0320E+03, &
      4.4432940E+00, 7.66441E+01, 2.2290E+01, 1.0196E+03, &
      4.4655105E+00, 7.67629E+01, 2.2131E+01, 1.0074E+03, &
      4.4878381E+00, 7.68777E+01, 2.1974E+01, 9.9530E+02, &
      4.5102772E+00, 7.69887E+01, 2.1818E+01, 9.8340E+02, &
      4.5328286E+00, 7.70963E+01, 2.1663E+01, 9.7165E+02, &
      4.5554928E+00, 7.72007E+01, 2.1510E+01, 9.6006E+02, &
      4.5782702E+00, 7.73021E+01, 2.1358E+01, 9.4863E+02, &
      4.6011616E+00, 7.74007E+01, 2.1207E+01, 9.3734E+02, &
      4.6241674E+00, 7.74965E+01, 2.1058E+01, 9.2619E+02, &
      4.6472882E+00, 7.75899E+01, 2.0910E+01, 9.1519E+02, &
      4.6705247E+00, 7.76809E+01, 2.0763E+01, 9.0433E+02, &
      4.6938773E+00, 7.77696E+01, 2.0617E+01, 8.9361E+02, &
      4.7173467E+00, 7.78561E+01, 2.0473E+01, 8.8302E+02, &
      4.7409334E+00, 7.79406E+01, 2.0330E+01, 8.7257E+02, &
      4.7646381E+00, 7.80232E+01, 2.0188E+01, 8.6225E+02, &
      4.7884613E+00, 7.81039E+01, 2.0047E+01, 8.5206E+02, &
      4.8124036E+00, 7.81828E+01, 1.9907E+01, 8.4200E+02, &
      4.8364656E+00, 7.82601E+01, 1.9769E+01, 8.3207E+02, &
      4.8606479E+00, 7.83358E+01, 1.9631E+01, 8.2226E+02, &
      4.8849512E+00, 7.84100E+01, 1.9495E+01, 8.1257E+02, &
      4.9093759E+00, 7.84828E+01, 1.9360E+01, 8.0300E+02, &
      4.9339228E+00, 7.85544E+01, 1.9226E+01, 7.9355E+02, &
      4.9585924E+00, 7.86248E+01, 1.9093E+01, 7.8421E+02, &
      4.9833854E+00, 7.86941E+01, 1.8961E+01, 7.7499E+02, &
      5.0083023E+00, 7.87627E+01, 1.8830E+01, 7.6589E+02, &
      5.0333438E+00, 7.88297E+01, 1.8696E+01, 7.5675E+02, &
      5.0585105E+00, 7.88948E+01, 1.8563E+01, 7.4771E+02, &
      5.0838031E+00, 7.89582E+01, 1.8431E+01, 7.3878E+02, &
      5.1092221E+00, 7.90200E+01, 1.8300E+01, 7.2995E+02, &
      5.1347682E+00, 7.90803E+01, 1.8170E+01, 7.2124E+02, &
      5.1604421E+00, 7.91391E+01, 1.8040E+01, 7.1262E+02, &
      5.1862443E+00, 7.91965E+01, 1.7912E+01, 7.0412E+02, &
      5.2121755E+00, 7.92526E+01, 1.7785E+01, 6.9571E+02, &
      5.2382364E+00, 7.93075E+01, 1.7658E+01, 6.8741E+02, &
      5.2644276E+00, 7.93613E+01, 1.7533E+01, 6.7921E+02, &
      5.2907497E+00, 7.94139E+01, 1.7408E+01, 6.7110E+02, &
      5.3172034E+00, 7.94654E+01, 1.7285E+01, 6.6310E+02, &
      5.3437895E+00, 7.95159E+01, 1.7162E+01, 6.5519E+02, &
      5.3705084E+00, 7.97360E+01, 1.7039E+01, 6.4734E+02, &
      5.3973609E+00, 7.97848E+01, 1.6912E+01, 6.3940E+02, &
      5.4243477E+00, 7.98319E+01, 1.6786E+01, 6.3154E+02, &
      5.4514695E+00, 7.98773E+01, 1.6660E+01, 6.2378E+02, &
      5.4787268E+00, 7.99212E+01, 1.6536E+01, 6.1611E+02, &
      5.5061205E+00, 7.99637E+01, 1.6412E+01, 6.0855E+02, &
      5.5336511E+00, 8.00047E+01, 1.6290E+01, 6.0108E+02, &
      5.5613193E+00, 8.00445E+01, 1.6168E+01, 5.9370E+02, &
      5.5891259E+00, 8.00831E+01, 1.6047E+01, 5.8642E+02, &
      5.6170716E+00, 8.02379E+01, 1.5925E+01, 5.7914E+02, &
      5.6451569E+00, 8.02739E+01, 1.5803E+01, 5.7192E+02, &
      5.6733827E+00, 8.03084E+01, 1.5682E+01, 5.6479E+02, &
      5.7017496E+00, 8.03415E+01, 1.5562E+01, 5.5776E+02, &
      5.7302584E+00, 8.03733E+01, 1.5443E+01, 5.5081E+02, &
      5.7589096E+00, 8.04038E+01, 1.5325E+01, 5.4396E+02, &
      5.7877042E+00, 8.04331E+01, 1.5208E+01, 5.3720E+02, &
      5.8166427E+00, 8.04612E+01, 1.5092E+01, 5.3052E+02, &
      5.8457259E+00, 8.04882E+01, 1.4977E+01, 5.2394E+02, &
      5.8749546E+00, 8.05141E+01, 1.4863E+01, 5.1743E+02, &
      5.9043293E+00, 8.05389E+01, 1.4750E+01, 5.1102E+02, &
      5.9338510E+00, 8.05627E+01, 1.4638E+01, 5.0468E+02, &
      5.9635202E+00, 8.05855E+01, 1.4527E+01, 4.9843E+02, &
      5.9933378E+00, 8.06074E+01, 1.4416E+01, 4.9226E+02, &
      6.0233045E+00, 8.06284E+01, 1.4307E+01, 4.8617E+02, &
      6.0534210E+00, 8.06484E+01, 1.4199E+01, 4.8016E+02, &
      6.0836882E+00, 8.06676E+01, 1.4091E+01, 4.7423E+02, &
      6.1141066E+00, 8.06860E+01, 1.3985E+01, 4.6838E+02, &
      6.1446771E+00, 8.07035E+01, 1.3879E+01, 4.6260E+02, &
      6.1754005E+00, 8.07203E+01, 1.3775E+01, 4.5690E+02, &
      6.2062775E+00, 8.07363E+01, 1.3671E+01, 4.5127E+02, &
      6.2373089E+00, 8.07516E+01, 1.3568E+01, 4.4572E+02, &
      6.2684954E+00, 8.07661E+01, 1.3466E+01, 4.4024E+02, &
      6.2998379E+00, 8.07800E+01, 1.3365E+01, 4.3483E+02, &
      6.3313371E+00, 8.07932E+01, 1.3265E+01, 4.2950E+02, &
      6.3629938E+00, 8.08057E+01, 1.3166E+01, 4.2423E+02, &
      6.3948088E+00, 8.08176E+01, 1.3067E+01, 4.1903E+02, &
      6.4267828E+00, 8.08289E+01, 1.2970E+01, 4.1390E+02, &
      6.4589167E+00, 8.08396E+01, 1.2873E+01, 4.0884E+02, &
      6.4912113E+00, 8.08498E+01, 1.2777E+01, 4.0384E+02, &
      6.5236674E+00, 8.08594E+01, 1.2682E+01, 3.9891E+02, &
      6.5562857E+00, 8.08685E+01, 1.2588E+01, 3.9404E+02, &
      6.5890671E+00, 8.08771E+01, 1.2495E+01, 3.8924E+02, &
      6.6220125E+00, 8.09737E+01, 1.2401E+01, 3.8447E+02, &
      6.6551225E+00, 8.09816E+01, 1.2307E+01, 3.7973E+02, &
      6.6883981E+00, 8.09888E+01, 1.2214E+01, 3.7505E+02, &
      6.7218401E+00, 8.09953E+01, 1.2122E+01, 3.7043E+02, &
      6.7554493E+00, 8.10011E+01, 1.2031E+01, 3.6587E+02, &
      6.7892266E+00, 8.10064E+01, 1.1940E+01, 3.6138E+02, &
      6.8231727E+00, 8.10111E+01, 1.1851E+01, 3.5694E+02, &
      6.8572886E+00, 8.10152E+01, 1.1762E+01, 3.5256E+02, &
      6.8915750E+00, 8.10188E+01, 1.1674E+01, 3.4824E+02, &
      6.9260329E+00, 8.10218E+01, 1.1586E+01, 3.4398E+02, &
      6.9606631E+00, 8.10243E+01, 1.1499E+01, 3.3977E+02, &
      6.9954664E+00, 8.10264E+01, 1.1414E+01, 3.3561E+02, &
      7.0304437E+00, 8.10280E+01, 1.1328E+01, 3.3151E+02, &
      7.0655959E+00, 8.10291E+01, 1.1244E+01, 3.2747E+02, &
      7.1009239E+00, 8.10297E+01, 1.1160E+01, 3.2348E+02, &
      7.1364285E+00, 8.10299E+01, 1.1078E+01, 3.1954E+02, &
      7.1721107E+00, 8.10297E+01, 1.0995E+01, 3.1565E+02, &
      7.2079712E+00, 8.10291E+01, 1.0914E+01, 3.1181E+02, &
      7.2440111E+00, 8.10281E+01, 1.0833E+01, 3.0802E+02, &
      7.2802311E+00, 8.10267E+01, 1.0753E+01, 3.0428E+02, &
      7.3166323E+00, 8.10250E+01, 1.0674E+01, 3.0060E+02, &
      7.3532155E+00, 8.10229E+01, 1.0595E+01, 2.9695E+02, &
      7.3899815E+00, 8.10204E+01, 1.0517E+01, 2.9336E+02, &
      7.4269314E+00, 8.10176E+01, 1.0440E+01, 2.8981E+02, &
      7.4640661E+00, 8.10145E+01, 1.0364E+01, 2.8631E+02, &
      7.5013864E+00, 8.10110E+01, 1.0288E+01, 2.8286E+02, &
      7.5388934E+00, 8.10073E+01, 1.0212E+01, 2.7945E+02, &
      7.5765878E+00, 8.10032E+01, 1.0138E+01, 2.7609E+02, &
      7.6144708E+00, 8.09989E+01, 1.0064E+01, 2.7276E+02, &
      7.6525431E+00, 8.09943E+01, 9.9907E+00, 2.6949E+02, &
      7.6908058E+00, 8.09895E+01, 9.9181E+00, 2.6625E+02, &
      7.7292599E+00, 8.10253E+01, 9.8456E+00, 2.6304E+02, &
      7.7679062E+00, 8.10202E+01, 9.7733E+00, 2.5987E+02, &
      7.8067457E+00, 8.10147E+01, 9.7016E+00, 2.5673E+02, &
      7.8457794E+00, 8.10089E+01, 9.6305E+00, 2.5363E+02, &
      7.8850083E+00, 8.10028E+01, 9.5600E+00, 2.5058E+02, &
      7.9244334E+00, 8.09964E+01, 9.4902E+00, 2.4756E+02, &
      7.9640555E+00, 8.09898E+01, 9.4209E+00, 2.4458E+02, &
      8.0038758E+00, 8.09828E+01, 9.3522E+00, 2.4164E+02, &
      8.0438952E+00, 8.09757E+01, 9.2841E+00, 2.3874E+02, &
      8.0841147E+00, 8.09683E+01, 9.2166E+00, 2.3588E+02, &
      8.1245352E+00, 8.09606E+01, 9.1496E+00, 2.3305E+02, &
      8.1651579E+00, 8.09528E+01, 9.0833E+00, 2.3026E+02, &
      8.2059837E+00, 8.09447E+01, 9.0175E+00, 2.2751E+02, &
      8.2470136E+00, 8.09365E+01, 8.9522E+00, 2.2478E+02, &
      8.2882487E+00, 8.09281E+01, 8.8875E+00, 2.2210E+02, &
      8.3296899E+00, 8.09360E+01, 8.8231E+00, 2.1944E+02, &
      8.3713384E+00, 8.09275E+01, 8.7591E+00, 2.1682E+02, &
      8.4131951E+00, 8.09187E+01, 8.6956E+00, 2.1422E+02, &
      8.4552610E+00, 8.09099E+01, 8.6327E+00, 2.1166E+02, &
      8.4975373E+00, 8.09010E+01, 8.5704E+00, 2.0914E+02, &
      8.5400250E+00, 8.08920E+01, 8.5085E+00, 2.0664E+02, &
      9.030794E+00, 8.10095E+01, 7.8208E+00, 1.8016E+02, &
      9.653919E+00, 8.07345E+01, 6.9929E+00, 1.5130E+02, &
      1.032004E+01, 8.03512E+01, 6.2610E+00, 1.2728E+02, &
      1.103212E+01, 7.98518E+01, 5.6151E+00, 1.0730E+02, &
      1.179334E+01, 7.91788E+01, 5.0408E+00, 9.0568E+01, &
      1.260708E+01, 7.81391E+01, 4.5179E+00, 7.6367E+01, &
      1.347697E+01, 7.56777E+01, 4.0535E+00, 6.4489E+01, &
      1.353752E+01, 7.53005E+01, 4.0242E+00, 6.3763E+01, &
      1.374473E+01, 7.26547E+01, 3.9267E+00, 6.1370E+01, &
      1.379999E+01, 6.95496E+01, 3.9014E+00, 6.0755E+01, &
      1.388287E+01, 7.26427E+01, 9.9368E+00, 1.4792E+02, &
      1.409008E+01, 7.53795E+01, 9.6932E+00, 1.4226E+02, &
      1.440688E+01, 7.68213E+01, 9.3377E+00, 1.3416E+02, &
      1.540095E+01, 7.80433E+01, 8.3210E+00, 1.1218E+02, &
      1.591941E+01, 7.75740E+01, 7.8563E+00, 1.0264E+02, &
      1.616308E+01, 7.63557E+01, 7.6515E+00, 9.8535E+01, &
      1.622806E+01, 7.48051E+01, 7.5982E+00, 9.7479E+01, &
      1.632552E+01, 7.63871E+01, 1.0594E+01, 1.3384E+02, &
      1.646362E+01, 7.73820E+01, 1.0446E+01, 1.3091E+02, &
      1.656919E+01, 7.77430E+01, 1.0336E+01, 1.2872E+02, &
      1.660051E+01, 7.78177E+01, 1.0303E+01, 1.2809E+02, &
      1.685460E+01, 7.78673E+01, 1.0045E+01, 1.2306E+02, &
      1.692236E+01, 7.72212E+01, 9.9772E+00, 1.2177E+02, &
      1.702400E+01, 7.82161E+01, 1.1440E+01, 1.3834E+02, &
      1.727809E+01, 7.93527E+01, 1.1183E+01, 1.3331E+02, &
      1.759961E+01, 8.01319E+01, 1.0872E+01, 1.2730E+02, &
      1.881398E+01, 8.17458E+01, 9.7930E+00, 1.0750E+02, &
      2.011215E+01, 8.26546E+01, 8.8107E+00, 9.0699E+01, &
      2.149988E+01, 8.32446E+01, 7.9269E+00, 7.6535E+01, &
      2.298338E+01, 8.36422E+01, 7.1288E+00, 6.4570E+01, &
      2.456923E+01, 8.39129E+01, 6.4136E+00, 5.4508E+01, &
      2.626450E+01, 8.41068E+01, 5.7737E+00, 4.6050E+01, &
      2.807676E+01, 8.43076E+01, 5.1650E+00, 3.8680E+01, &
      3.001405E+01, 8.43615E+01, 4.6062E+00, 3.2404E+01, &
      3.208502E+01, 8.43709E+01, 4.1120E+00, 2.7182E+01, &
      3.429889E+01, 8.44133E+01, 3.6662E+00, 2.2782E+01, &
      3.666551E+01, 8.43661E+01, 3.2619E+00, 1.9065E+01, &
      3.919543E+01, 8.42986E+01, 2.9044E+00, 1.5973E+01, &
      4.189992E+01, 8.42183E+01, 2.5881E+00, 1.3399E+01, &
      4.479101E+01, 8.41295E+01, 2.3080E+00, 1.1254E+01, &
      4.788159E+01, 8.40346E+01, 2.0597E+00, 9.4638E+00, &
      5.118542E+01, 8.39352E+01, 1.8395E+00, 7.9688E+00, &
      5.471721E+01, 8.38314E+01, 1.6441E+00, 6.7192E+00, &
      5.849270E+01, 8.37228E+01, 1.4705E+00, 5.6734E+00, &
      6.252870E+01, 8.36071E+01, 1.3140E+00, 4.7905E+00, &
      6.684318E+01, 8.34787E+01, 1.1743E+00, 4.0489E+00, &
      7.145536E+01, 8.33311E+01, 1.0500E+00, 3.4272E+00, &
      7.638578E+01, 8.31488E+01, 9.3655E-01, 2.8982E+00, &
      8.165640E+01, 8.28940E+01, 8.3262E-01, 2.4474E+00, &
      8.729069E+01, 8.24362E+01, 7.3620E-01, 2.0607E+00, &
      9.124290E+01, 8.16339E+01, 6.7872E-01, 1.8409E+00, &
      9.263948E+01, 8.06048E+01, 6.6011E-01, 1.7715E+00, &
      9.301190E+01, 7.93737E+01, 6.5528E-01, 1.7537E+00, &
      9.331374E+01, 8.00216E+01, 3.0839E+00, 6.9880E+00, &
      9.357053E+01, 8.06569E+01, 3.0708E+00, 6.9403E+00, &
      9.496710E+01, 8.17789E+01, 3.0009E+00, 6.6890E+00, &
      9.975239E+01, 8.28390E+01, 2.7786E+00, 5.9165E+00, &
      1.066353E+02, 8.34136E+01, 2.5008E+00, 5.0065E+00, &
      1.139931E+02, 8.37293E+01, 2.2471E+00, 4.2326E+00, &
      1.218587E+02, 8.39284E+01, 2.0183E+00, 3.5794E+00, &
      1.302669E+02, 8.40613E+01, 1.8124E+00, 3.0290E+00, &
      1.392553E+02, 8.41518E+01, 1.6250E+00, 2.5618E+00, &
      1.488640E+02, 8.42113E+01, 1.4564E+00, 2.1683E+00, &
      1.591356E+02, 8.42495E+01, 1.3053E+00, 1.8374E+00, &
      1.701159E+02, 8.42724E+01, 1.1699E+00, 1.5593E+00, &
      1.818539E+02, 8.42843E+01, 1.0486E+00, 1.3253E+00, &
      1.944018E+02, 8.42882E+01, 9.4004E-01, 1.1286E+00, &
      2.078156E+02, 8.42862E+01, 8.4258E-01, 9.6286E-01, &
      2.221548E+02, 8.42799E+01, 7.5532E-01, 8.2336E-01, &
      2.374835E+02, 8.42706E+01, 6.7716E-01, 7.0585E-01, &
      2.538699E+02, 8.42592E+01, 6.0716E-01, 6.0679E-01, &
      2.713869E+02, 8.42465E+01, 5.4445E-01, 5.2321E-01, &
      2.901126E+02, 8.42330E+01, 4.8826E-01, 4.5263E-01, &
      3.101304E+02, 8.42192E+01, 4.3791E-01, 3.9295E-01, &
      3.315294E+02, 8.42053E+01, 3.9279E-01, 3.4244E-01, &
      3.544049E+02, 8.41915E+01, 3.5235E-01, 2.9961E-01, &
      3.788588E+02, 8.41781E+01, 3.1610E-01, 2.6324E-01, &
      4.050001E+02, 8.41651E+01, 2.8361E-01, 2.3231E-01, &
      4.329451E+02, 8.41527E+01, 2.5447E-01, 2.0593E-01]
   case(85)
      frel = -1.2198E+00
      allocate(nist_tab(3112))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.119614E-03, -2.41657E-01, 0.0000E+00, 1.0124E-05, &
      6.213281E-03, -2.69168E+00, 0.0000E+00, 1.0994E-05, &
      6.238259E-03, -5.59099E+00, 0.0000E+00, 1.1236E-05, &
      6.275726E-03, -2.60900E+00, 5.6874E+00, 1.8160E+05, &
      6.369394E-03, -2.10624E-02, 5.5961E+00, 1.7605E+05, &
      8.850419E-03, 2.63911E+00, 4.0899E+00, 9.2600E+04, &
      8.985884E-03, 9.21561E-01, 4.0368E+00, 9.0020E+04, &
      9.022009E-03, -1.18057E+00, 4.0229E+00, 8.9349E+04, &
      9.076195E-03, 1.00680E+00, 8.1650E+00, 1.8027E+05, &
      9.211660E-03, 2.96014E+00, 8.0974E+00, 1.7614E+05, &
      1.069000E-02, 6.82483E+00, 7.4081E+00, 1.3886E+05, &
      1.142761E-02, 7.64511E+00, 7.0693E+00, 1.2396E+05, &
      1.221612E-02, 8.30377E+00, 6.7047E+00, 1.0998E+05, &
      1.305903E-02, 8.83986E+00, 6.3150E+00, 9.6900E+04, &
      1.396010E-02, 9.26885E+00, 5.9041E+00, 8.4748E+04, &
      1.492335E-02, 9.59618E+00, 5.4778E+00, 7.3553E+04, &
      1.595306E-02, 9.82142E+00, 5.0430E+00, 6.3344E+04, &
      1.705382E-02, 9.93647E+00, 4.6073E+00, 5.4135E+04, &
      1.823053E-02, 9.90480E+00, 4.1779E+00, 4.5921E+04, &
      1.895220E-02, 9.72269E+00, 3.9337E+00, 4.1591E+04, &
      1.924229E-02, 9.43058E+00, 3.8396E+00, 3.9984E+04, &
      1.931964E-02, 9.06984E+00, 3.8149E+00, 3.9568E+04, &
      1.943568E-02, 9.46307E+00, 4.4914E+00, 4.6307E+04, &
      1.948844E-02, 9.56904E+00, 4.4669E+00, 4.5929E+04, &
      1.972576E-02, 9.80054E+00, 4.3587E+00, 4.4278E+04, &
      2.083314E-02, 1.00552E+01, 3.8991E+00, 3.7503E+04, &
      2.227063E-02, 9.99527E+00, 3.3974E+00, 3.0568E+04, &
      2.380730E-02, 9.75723E+00, 2.9578E+00, 2.4895E+04, &
      2.545001E-02, 9.38258E+00, 2.5720E+00, 2.0251E+04, &
      2.720606E-02, 8.87291E+00, 2.2342E+00, 1.6456E+04, &
      2.908327E-02, 8.21025E+00, 1.9390E+00, 1.3359E+04, &
      3.109002E-02, 7.35213E+00, 1.6817E+00, 1.0839E+04, &
      3.323523E-02, 6.21609E+00, 1.4583E+00, 8.7922E+03, &
      3.552846E-02, 4.61874E+00, 1.2649E+00, 7.1340E+03, &
      3.690857E-02, 3.28524E+00, 1.1666E+00, 6.3335E+03, &
      3.747350E-02, 2.46771E+00, 1.1297E+00, 6.0409E+03, &
      3.762415E-02, 2.01636E+00, 1.1202E+00, 5.9660E+03, &
      3.785012E-02, 1.99677E+00, 1.6773E+00, 8.8799E+03, &
      3.797993E-02, 1.92704E+00, 1.7019E+00, 8.9791E+03, &
      3.841505E-02, 1.54296E+00, 1.7979E+00, 9.3782E+03, &
      4.060054E-02, -1.22929E+00, 2.7299E+00, 1.3473E+04, &
      4.076232E-02, -1.47646E+00, 2.8398E+00, 1.3960E+04, &
      4.138623E-02, -2.54627E+00, 3.3371E+00, 1.6157E+04, &
      4.155261E-02, -2.98891E+00, 3.4911E+00, 1.6835E+04, &
      4.180217E-02, -3.18942E+00, 4.1313E+00, 1.9804E+04, &
      4.242608E-02, -3.93531E+00, 4.9135E+00, 2.3207E+04, &
      4.340198E-02, -5.16803E+00, 6.3762E+00, 2.9438E+04, &
      4.639671E-02, -7.92998E+00, 1.2625E+01, 5.4527E+04, &
      4.959809E-02, -7.50469E+00, 2.1002E+01, 8.4849E+04, &
      5.302035E-02, -2.81212E+00, 2.9113E+01, 1.1003E+05, &
      5.667876E-02, 5.29944E+00, 3.4450E+01, 1.2180E+05, &
      6.058959E-02, 1.46343E+01, 3.5751E+01, 1.1824E+05, &
      6.477028E-02, 2.29647E+01, 3.3389E+01, 1.0330E+05, &
      6.923942E-02, 2.89886E+01, 2.8736E+01, 8.3163E+04, &
      7.401695E-02, 3.24658E+01, 2.3282E+01, 6.3030E+04, &
      7.912411E-02, 3.38329E+01, 1.8092E+01, 4.5819E+04, &
      8.458368E-02, 3.37493E+01, 1.3704E+01, 3.2464E+04, &
      9.041995E-02, 3.28061E+01, 1.0255E+01, 2.2725E+04, &
      9.665893E-02, 3.14073E+01, 7.6660E+00, 1.5892E+04, &
      1.033284E-01, 2.96988E+01, 5.7761E+00, 1.1201E+04, &
      1.062575E-01, 2.87473E+01, 5.1509E+00, 9.7143E+03, &
      1.078839E-01, 2.77849E+01, 4.8460E+00, 9.0015E+03, &
      1.083176E-01, 2.69110E+01, 4.7693E+00, 8.8236E+03, &
      1.089681E-01, 2.76345E+01, 6.2229E+00, 1.1444E+04, &
      1.104581E-01, 2.80662E+01, 5.9294E+00, 1.0758E+04, &
      1.105945E-01, 2.80768E+01, 5.9037E+00, 1.0698E+04, &
      1.180797E-01, 2.75554E+01, 4.7243E+00, 8.0178E+03, &
      1.262272E-01, 2.65188E+01, 3.8450E+00, 6.1046E+03, &
      1.349368E-01, 2.52183E+01, 3.2157E+00, 4.7763E+03, &
      1.357294E-01, 2.50570E+01, 3.1698E+00, 4.6806E+03, &
      1.378069E-01, 2.43871E+01, 3.0565E+00, 4.4453E+03, &
      1.383609E-01, 2.37735E+01, 3.0280E+00, 4.3862E+03, &
      1.391919E-01, 2.42878E+01, 4.0882E+00, 5.8865E+03, &
      1.412694E-01, 2.46047E+01, 3.9450E+00, 5.5968E+03, &
      1.442475E-01, 2.45940E+01, 3.7579E+00, 5.2213E+03, &
      1.542005E-01, 2.40093E+01, 3.2571E+00, 4.2338E+03, &
      1.648404E-01, 2.31583E+01, 2.8777E+00, 3.4995E+03, &
      1.762144E-01, 2.21143E+01, 2.5933E+00, 2.9505E+03, &
      1.819046E-01, 2.15086E+01, 2.4850E+00, 2.7391E+03, &
      1.846888E-01, 2.11439E+01, 2.4386E+00, 2.6474E+03, &
      1.854313E-01, 2.09843E+01, 2.4268E+00, 2.6242E+03, &
      1.865450E-01, 2.09282E+01, 2.5574E+00, 2.7488E+03, &
      1.867657E-01, 2.09114E+01, 2.5542E+00, 2.7422E+03, &
      1.883732E-01, 2.07458E+01, 2.5320E+00, 2.6951E+03, &
      1.893292E-01, 2.06186E+01, 2.5193E+00, 2.6681E+03, &
      1.896243E-01, 2.05723E+01, 2.5155E+00, 2.6599E+03, &
      1.903866E-01, 2.03960E+01, 2.5057E+00, 2.6390E+03, &
      1.915301E-01, 2.03563E+01, 2.6836E+00, 2.8095E+03, &
      1.931341E-01, 2.02266E+01, 2.6841E+00, 2.7866E+03, &
      1.943887E-01, 2.00964E+01, 2.6863E+00, 2.7709E+03, &
      1.960902E-01, 1.98840E+01, 2.6921E+00, 2.7528E+03, &
      1.968785E-01, 1.97317E+01, 2.6959E+00, 2.7457E+03, &
      1.980610E-01, 1.96674E+01, 2.8430E+00, 2.8782E+03, &
      2.010171E-01, 1.93973E+01, 2.8950E+00, 2.8878E+03, &
      2.013709E-01, 1.93611E+01, 2.9023E+00, 2.8900E+03, &
      2.152655E-01, 1.77779E+01, 3.4011E+00, 3.1680E+03, &
      2.301188E-01, 1.60180E+01, 4.4301E+00, 3.8600E+03, &
      2.459970E-01, 1.42798E+01, 6.0819E+00, 4.9568E+03, &
      2.629708E-01, 1.27910E+01, 8.3924E+00, 6.3978E+03, &
      2.811158E-01, 1.17914E+01, 1.1290E+01, 8.0508E+03, &
      3.005128E-01, 1.14749E+01, 1.4601E+01, 9.7395E+03, &
      3.212482E-01, 1.19469E+01, 1.8085E+01, 1.1285E+04, &
      3.434143E-01, 1.32036E+01, 2.1484E+01, 1.2540E+04, &
      3.671099E-01, 1.51565E+01, 2.4568E+01, 1.3415E+04, &
      3.924405E-01, 1.76291E+01, 2.7162E+01, 1.3874E+04, &
      4.195189E-01, 2.03878E+01, 2.9159E+01, 1.3933E+04, &
      4.484657E-01, 2.30912E+01, 3.0520E+01, 1.3643E+04, &
      4.658769E-01, 2.42237E+01, 3.1015E+01, 1.3346E+04, &
      4.730077E-01, 2.40570E+01, 3.1157E+01, 1.3205E+04, &
      4.749092E-01, 2.32562E+01, 3.1189E+01, 1.3166E+04, &
      4.777615E-01, 2.44648E+01, 3.3142E+01, 1.3906E+04, &
      4.794098E-01, 2.49260E+01, 3.3180E+01, 1.3874E+04, &
      4.848923E-01, 2.59263E+01, 3.3297E+01, 1.3766E+04, &
      5.0000000E-01, 3.00410E+01, 3.3537E+01, 1.3447E+04, &
      5.0250000E-01, 3.03027E+01, 3.3567E+01, 1.3392E+04, &
      5.0501250E-01, 3.05564E+01, 3.3593E+01, 1.3336E+04, &
      5.0753756E-01, 3.08023E+01, 3.3617E+01, 1.3279E+04, &
      5.1007525E-01, 3.10402E+01, 3.3639E+01, 1.3222E+04, &
      5.1262563E-01, 3.12695E+01, 3.3658E+01, 1.3163E+04, &
      5.1518875E-01, 3.14893E+01, 3.3674E+01, 1.3104E+04, &
      5.1776470E-01, 3.16980E+01, 3.3688E+01, 1.3044E+04, &
      5.2035352E-01, 3.18926E+01, 3.3700E+01, 1.2984E+04, &
      5.2295529E-01, 3.20683E+01, 3.3709E+01, 1.2923E+04, &
      5.2557007E-01, 3.22157E+01, 3.3715E+01, 1.2861E+04, &
      5.2819792E-01, 3.23124E+01, 3.3719E+01, 1.2799E+04, &
      5.3083891E-01, 3.22834E+01, 3.3721E+01, 1.2736E+04, &
      5.3281612E-01, 3.17932E+01, 3.3720E+01, 1.2688E+04, &
      5.3349310E-01, 3.17529E+01, 3.4879E+01, 1.3108E+04, &
      5.3358393E-01, 3.18604E+01, 3.4879E+01, 1.3105E+04, &
      5.3616057E-01, 3.28451E+01, 3.4889E+01, 1.3046E+04, &
      5.3884137E-01, 3.33256E+01, 3.4898E+01, 1.2985E+04, &
      5.4153558E-01, 3.37128E+01, 3.4903E+01, 1.2922E+04, &
      5.4424325E-01, 3.40594E+01, 3.4907E+01, 1.2859E+04, &
      5.4696447E-01, 3.43829E+01, 3.4908E+01, 1.2796E+04, &
      5.4969929E-01, 3.46912E+01, 3.4907E+01, 1.2732E+04, &
      5.5244779E-01, 3.49885E+01, 3.4903E+01, 1.2667E+04, &
      5.5521003E-01, 3.52774E+01, 3.4898E+01, 1.2602E+04, &
      5.5798608E-01, 3.55611E+01, 3.4890E+01, 1.2537E+04, &
      5.6077601E-01, 3.58375E+01, 3.4880E+01, 1.2471E+04, &
      5.6357989E-01, 3.61091E+01, 3.4868E+01, 1.2405E+04, &
      5.6639779E-01, 3.63765E+01, 3.4855E+01, 1.2338E+04, &
      5.6922978E-01, 3.66401E+01, 3.4839E+01, 1.2271E+04, &
      5.7207593E-01, 3.69494E+01, 3.4821E+01, 1.2204E+04, &
      5.7493630E-01, 3.72065E+01, 3.4801E+01, 1.2136E+04, &
      5.7781099E-01, 3.74606E+01, 3.4779E+01, 1.2069E+04, &
      5.8070004E-01, 3.77120E+01, 3.4755E+01, 1.2000E+04, &
      5.8360354E-01, 3.79608E+01, 3.4730E+01, 1.1932E+04, &
      5.8652156E-01, 3.82072E+01, 3.4703E+01, 1.1864E+04, &
      5.8945417E-01, 3.84513E+01, 3.4674E+01, 1.1795E+04, &
      5.9240144E-01, 3.86893E+01, 3.4643E+01, 1.1726E+04, &
      5.9536345E-01, 3.89289E+01, 3.4610E+01, 1.1656E+04, &
      5.9834026E-01, 3.91663E+01, 3.4575E+01, 1.1587E+04, &
      6.0133196E-01, 3.94015E+01, 3.4538E+01, 1.1517E+04, &
      6.0433862E-01, 3.96345E+01, 3.4499E+01, 1.1447E+04, &
      6.0736032E-01, 3.98652E+01, 3.4458E+01, 1.1376E+04, &
      6.1039712E-01, 4.00938E+01, 3.4416E+01, 1.1306E+04, &
      6.1344910E-01, 4.03201E+01, 3.4371E+01, 1.1235E+04, &
      6.1651635E-01, 4.05442E+01, 3.4325E+01, 1.1164E+04, &
      6.1959893E-01, 4.07661E+01, 3.4278E+01, 1.1093E+04, &
      6.2269693E-01, 4.09857E+01, 3.4228E+01, 1.1022E+04, &
      6.2581041E-01, 4.12031E+01, 3.4177E+01, 1.0951E+04, &
      6.2893946E-01, 4.14181E+01, 3.4124E+01, 1.0880E+04, &
      6.3208416E-01, 4.16308E+01, 3.4070E+01, 1.0809E+04, &
      6.3524458E-01, 4.18412E+01, 3.4014E+01, 1.0737E+04, &
      6.3842080E-01, 4.20491E+01, 3.3957E+01, 1.0666E+04, &
      6.4161291E-01, 4.22546E+01, 3.3898E+01, 1.0595E+04, &
      6.4482097E-01, 4.24577E+01, 3.3837E+01, 1.0523E+04, &
      6.4804508E-01, 4.26581E+01, 3.3775E+01, 1.0452E+04, &
      6.5128530E-01, 4.28560E+01, 3.3712E+01, 1.0380E+04, &
      6.5454173E-01, 4.30512E+01, 3.3647E+01, 1.0309E+04, &
      6.5781444E-01, 4.32444E+01, 3.3581E+01, 1.0238E+04, &
      6.6110351E-01, 4.34355E+01, 3.3513E+01, 1.0166E+04, &
      6.6440903E-01, 4.36228E+01, 3.3444E+01, 1.0095E+04, &
      6.6773107E-01, 4.38067E+01, 3.3374E+01, 1.0024E+04, &
      6.7106973E-01, 4.39871E+01, 3.3302E+01, 9.9523E+03, &
      6.7442508E-01, 4.41641E+01, 3.3229E+01, 9.8813E+03, &
      6.7779720E-01, 4.43375E+01, 3.3155E+01, 9.8103E+03, &
      6.8118619E-01, 4.45071E+01, 3.3080E+01, 9.7395E+03, &
      6.8459212E-01, 4.46727E+01, 3.3004E+01, 9.6689E+03, &
      6.8801508E-01, 4.48339E+01, 3.2927E+01, 9.5984E+03, &
      6.9145515E-01, 4.49904E+01, 3.2849E+01, 9.5281E+03, &
      6.9491243E-01, 4.51417E+01, 3.2770E+01, 9.4580E+03, &
      6.9838699E-01, 4.52874E+01, 3.2690E+01, 9.3880E+03, &
      7.0187893E-01, 4.54266E+01, 3.2609E+01, 9.3183E+03, &
      7.0538832E-01, 4.55585E+01, 3.2527E+01, 9.2488E+03, &
      7.0891526E-01, 4.56818E+01, 3.2445E+01, 9.1795E+03, &
      7.1245984E-01, 4.57948E+01, 3.2361E+01, 9.1105E+03, &
      7.1602214E-01, 4.58950E+01, 3.2277E+01, 9.0416E+03, &
      7.1960225E-01, 4.59789E+01, 3.2192E+01, 8.9731E+03, &
      7.2320026E-01, 4.60406E+01, 3.2107E+01, 8.9048E+03, &
      7.2681626E-01, 4.60700E+01, 3.2020E+01, 8.8368E+03, &
      7.3045034E-01, 4.60470E+01, 3.1933E+01, 8.7690E+03, &
      7.3410260E-01, 4.59211E+01, 3.1846E+01, 8.7015E+03, &
      7.3777311E-01, 4.54815E+01, 3.1757E+01, 8.6344E+03, &
      7.3891221E-01, 4.50819E+01, 3.1730E+01, 8.6136E+03, &
      7.4108781E-01, 4.51853E+01, 3.3669E+01, 9.1125E+03, &
      7.4146197E-01, 4.53903E+01, 3.3660E+01, 9.1056E+03, &
      7.4516928E-01, 4.63649E+01, 3.3572E+01, 9.0368E+03, &
      7.4889513E-01, 4.68823E+01, 3.3484E+01, 8.9684E+03, &
      7.5263961E-01, 4.72773E+01, 3.3396E+01, 8.9003E+03, &
      7.5640280E-01, 4.76136E+01, 3.3307E+01, 8.8326E+03, &
      7.6018482E-01, 4.79150E+01, 3.3218E+01, 8.7652E+03, &
      7.6398574E-01, 4.81931E+01, 3.3128E+01, 8.6982E+03, &
      7.6780567E-01, 4.84543E+01, 3.3038E+01, 8.6315E+03, &
      7.7164470E-01, 4.87026E+01, 3.2948E+01, 8.5651E+03, &
      7.7550292E-01, 4.89408E+01, 3.2856E+01, 8.4989E+03, &
      7.7938044E-01, 4.91706E+01, 3.2765E+01, 8.4331E+03, &
      7.8327734E-01, 4.93933E+01, 3.2672E+01, 8.3677E+03, &
      7.8719373E-01, 4.96101E+01, 3.2580E+01, 8.3026E+03, &
      7.9112969E-01, 4.98217E+01, 3.2487E+01, 8.2379E+03, &
      7.9508534E-01, 5.00289E+01, 3.2394E+01, 8.1735E+03, &
      7.9906077E-01, 5.02322E+01, 3.2301E+01, 8.1096E+03, &
      8.0305607E-01, 5.04321E+01, 3.2207E+01, 8.0459E+03, &
      8.0707135E-01, 5.06275E+01, 3.2105E+01, 7.9806E+03, &
      8.1110671E-01, 5.08184E+01, 3.2001E+01, 7.9153E+03, &
      8.1516224E-01, 5.10051E+01, 3.1897E+01, 7.8504E+03, &
      8.1923806E-01, 5.11879E+01, 3.1793E+01, 7.7859E+03, &
      8.2333425E-01, 5.13668E+01, 3.1688E+01, 7.7217E+03, &
      8.2745092E-01, 5.15420E+01, 3.1583E+01, 7.6579E+03, &
      8.3158817E-01, 5.17135E+01, 3.1477E+01, 7.5944E+03, &
      8.3574611E-01, 5.18814E+01, 3.1371E+01, 7.5313E+03, &
      8.3992484E-01, 5.20456E+01, 3.1265E+01, 7.4686E+03, &
      8.4412447E-01, 5.22060E+01, 3.1159E+01, 7.4063E+03, &
      8.4834509E-01, 5.23626E+01, 3.1052E+01, 7.3443E+03, &
      8.5258682E-01, 5.25151E+01, 3.0946E+01, 7.2828E+03, &
      8.5684975E-01, 5.26630E+01, 3.0839E+01, 7.2217E+03, &
      8.6113400E-01, 5.28058E+01, 3.0732E+01, 7.1610E+03, &
      8.6543967E-01, 5.29424E+01, 3.0625E+01, 7.1007E+03, &
      8.6976687E-01, 5.30708E+01, 3.0518E+01, 7.0408E+03, &
      8.7411570E-01, 5.31874E+01, 3.0412E+01, 6.9814E+03, &
      8.7848628E-01, 5.32827E+01, 3.0305E+01, 6.9224E+03, &
      8.8287871E-01, 5.33210E+01, 3.0198E+01, 6.8638E+03, &
      8.8458238E-01, 5.32762E+01, 3.0157E+01, 6.8413E+03, &
      8.8729310E-01, 5.33566E+01, 3.0504E+01, 6.8988E+03, &
      8.8741758E-01, 5.33728E+01, 3.0501E+01, 6.8972E+03, &
      8.9172957E-01, 5.37019E+01, 3.0399E+01, 6.8411E+03, &
      8.9618822E-01, 5.39268E+01, 3.0295E+01, 6.7838E+03, &
      9.0066916E-01, 5.41232E+01, 3.0191E+01, 6.7269E+03, &
      9.0517250E-01, 5.43051E+01, 3.0082E+01, 6.6694E+03, &
      9.0969837E-01, 5.44778E+01, 2.9972E+01, 6.6121E+03, &
      9.1424686E-01, 5.46439E+01, 2.9863E+01, 6.5553E+03, &
      9.1881809E-01, 5.48049E+01, 2.9753E+01, 6.4989E+03, &
      9.2341218E-01, 5.49617E+01, 2.9644E+01, 6.4430E+03, &
      9.2802924E-01, 5.51150E+01, 2.9535E+01, 6.3875E+03, &
      9.3266939E-01, 5.52652E+01, 2.9426E+01, 6.3324E+03, &
      9.3733274E-01, 5.54128E+01, 2.9318E+01, 6.2778E+03, &
      9.4201940E-01, 5.55579E+01, 2.9210E+01, 6.2236E+03, &
      9.4672950E-01, 5.57008E+01, 2.9102E+01, 6.1698E+03, &
      9.5146315E-01, 5.58418E+01, 2.8994E+01, 6.1166E+03, &
      9.5622046E-01, 5.59811E+01, 2.8887E+01, 6.0637E+03, &
      9.6100156E-01, 5.61189E+01, 2.8780E+01, 6.0113E+03, &
      9.6580657E-01, 5.62554E+01, 2.8673E+01, 5.9594E+03, &
      9.7063560E-01, 5.63908E+01, 2.8567E+01, 5.9080E+03, &
      9.7548878E-01, 5.65256E+01, 2.8462E+01, 5.8569E+03, &
      9.8036623E-01, 5.66599E+01, 2.8357E+01, 5.8064E+03, &
      9.8526806E-01, 5.67944E+01, 2.8252E+01, 5.7563E+03, &
      9.9019440E-01, 5.69296E+01, 2.8148E+01, 5.7067E+03, &
      9.9514537E-01, 5.70667E+01, 2.8044E+01, 5.6575E+03, &
      1.0001211E+00, 5.72100E+01, 2.7940E+01, 5.6085E+03, &
      1.0051217E+00, 5.74350E+01, 2.7766E+01, 5.5460E+03, &
      1.0101473E+00, 5.76290E+01, 2.7593E+01, 5.4843E+03, &
      1.0151980E+00, 5.77974E+01, 2.7422E+01, 5.4233E+03, &
      1.0202740E+00, 5.79427E+01, 2.7253E+01, 5.3631E+03, &
      1.0253754E+00, 5.80643E+01, 2.7085E+01, 5.3036E+03, &
      1.0305023E+00, 5.81570E+01, 2.6918E+01, 5.2449E+03, &
      1.0356548E+00, 5.82008E+01, 2.6752E+01, 5.1868E+03, &
      1.0406559E+00, 5.80646E+01, 2.6594E+01, 5.1315E+03, &
      1.0408331E+00, 5.80446E+01, 2.6588E+01, 5.1295E+03, &
      1.0433442E+00, 5.81475E+01, 2.7057E+01, 5.2072E+03, &
      1.0460372E+00, 5.84062E+01, 2.6973E+01, 5.1778E+03, &
      1.0512674E+00, 5.86761E+01, 2.6812E+01, 5.1214E+03, &
      1.0565238E+00, 5.88740E+01, 2.6652E+01, 5.0657E+03, &
      1.0618064E+00, 5.90424E+01, 2.6494E+01, 5.0107E+03, &
      1.0671154E+00, 5.91935E+01, 2.6337E+01, 4.9564E+03, &
      1.0724510E+00, 5.93325E+01, 2.6181E+01, 4.9027E+03, &
      1.0778132E+00, 5.94622E+01, 2.6027E+01, 4.8496E+03, &
      1.0832023E+00, 5.95844E+01, 2.5873E+01, 4.7972E+03, &
      1.0886183E+00, 5.97003E+01, 2.5721E+01, 4.7455E+03, &
      1.0940614E+00, 5.98107E+01, 2.5571E+01, 4.6943E+03, &
      1.0995317E+00, 5.99162E+01, 2.5421E+01, 4.6438E+03, &
      1.1050294E+00, 6.00173E+01, 2.5273E+01, 4.5938E+03, &
      1.1105545E+00, 6.01145E+01, 2.5125E+01, 4.5445E+03, &
      1.1161073E+00, 6.02081E+01, 2.4979E+01, 4.4957E+03, &
      1.1216878E+00, 6.02983E+01, 2.4835E+01, 4.4476E+03, &
      1.1272963E+00, 6.03854E+01, 2.4691E+01, 4.4000E+03, &
      1.1329328E+00, 6.04696E+01, 2.4548E+01, 4.3530E+03, &
      1.1385974E+00, 6.05510E+01, 2.4407E+01, 4.3065E+03, &
      1.1442904E+00, 6.06299E+01, 2.4267E+01, 4.2606E+03, &
      1.1500119E+00, 6.07063E+01, 2.4128E+01, 4.2152E+03, &
      1.1557619E+00, 6.07804E+01, 2.3990E+01, 4.1704E+03, &
      1.1615407E+00, 6.08524E+01, 2.3853E+01, 4.1261E+03, &
      1.1673484E+00, 6.09222E+01, 2.3717E+01, 4.0824E+03, &
      1.1731852E+00, 6.09901E+01, 2.3583E+01, 4.0392E+03, &
      1.1790511E+00, 6.10560E+01, 2.3449E+01, 3.9964E+03, &
      1.1849464E+00, 6.11201E+01, 2.3317E+01, 3.9542E+03, &
      1.1908711E+00, 6.11825E+01, 2.3185E+01, 3.9125E+03, &
      1.1968254E+00, 6.12432E+01, 2.3055E+01, 3.8713E+03, &
      1.2028096E+00, 6.13023E+01, 2.2926E+01, 3.8306E+03, &
      1.2088236E+00, 6.13599E+01, 2.2797E+01, 3.7903E+03, &
      1.2148677E+00, 6.14160E+01, 2.2670E+01, 3.7506E+03, &
      1.2209421E+00, 6.14706E+01, 2.2544E+01, 3.7113E+03, &
      1.2270468E+00, 6.15239E+01, 2.2419E+01, 3.6724E+03, &
      1.2331820E+00, 6.15760E+01, 2.2295E+01, 3.6341E+03, &
      1.2393479E+00, 6.16268E+01, 2.2171E+01, 3.5961E+03, &
      1.2455447E+00, 6.16764E+01, 2.2049E+01, 3.5587E+03, &
      1.2517724E+00, 6.17249E+01, 2.1928E+01, 3.5216E+03, &
      1.2580312E+00, 6.17719E+01, 2.1805E+01, 3.4846E+03, &
      1.2643214E+00, 6.18175E+01, 2.1683E+01, 3.4480E+03, &
      1.2706430E+00, 6.18616E+01, 2.1563E+01, 3.4119E+03, &
      1.2769962E+00, 6.19045E+01, 2.1443E+01, 3.3762E+03, &
      1.2833812E+00, 6.19460E+01, 2.1324E+01, 3.3409E+03, &
      1.2897981E+00, 6.19863E+01, 2.1206E+01, 3.3061E+03, &
      1.2962471E+00, 6.20254E+01, 2.1089E+01, 3.2716E+03, &
      1.3027283E+00, 6.20634E+01, 2.0973E+01, 3.2375E+03, &
      1.3092420E+00, 6.21002E+01, 2.0858E+01, 3.2039E+03, &
      1.3157882E+00, 6.21359E+01, 2.0743E+01, 3.1706E+03, &
      1.3223671E+00, 6.21705E+01, 2.0630E+01, 3.1377E+03, &
      1.3289790E+00, 6.22041E+01, 2.0517E+01, 3.1052E+03, &
      1.3356239E+00, 6.22367E+01, 2.0406E+01, 3.0731E+03, &
      1.3423020E+00, 6.22683E+01, 2.0295E+01, 3.0413E+03, &
      1.3490135E+00, 6.22990E+01, 2.0185E+01, 3.0099E+03, &
      1.3557586E+00, 6.23287E+01, 2.0076E+01, 2.9789E+03, &
      1.3625374E+00, 6.23575E+01, 1.9967E+01, 2.9482E+03, &
      1.3693500E+00, 6.23852E+01, 1.9859E+01, 2.9178E+03, &
      1.3761968E+00, 6.24120E+01, 1.9752E+01, 2.8877E+03, &
      1.3830778E+00, 6.24378E+01, 1.9646E+01, 2.8580E+03, &
      1.3899932E+00, 6.24627E+01, 1.9540E+01, 2.8287E+03, &
      1.3969431E+00, 6.24866E+01, 1.9436E+01, 2.7996E+03, &
      1.4039278E+00, 6.25097E+01, 1.9332E+01, 2.7710E+03, &
      1.4109475E+00, 6.25318E+01, 1.9229E+01, 2.7426E+03, &
      1.4180022E+00, 6.25532E+01, 1.9126E+01, 2.7146E+03, &
      1.4250922E+00, 6.25736E+01, 1.9025E+01, 2.6869E+03, &
      1.4322177E+00, 6.26075E+01, 1.8924E+01, 2.6595E+03, &
      1.4393788E+00, 6.26266E+01, 1.8824E+01, 2.6324E+03, &
      1.4465757E+00, 6.26450E+01, 1.8725E+01, 2.6056E+03, &
      1.4538086E+00, 6.26625E+01, 1.8627E+01, 2.5792E+03, &
      1.4610776E+00, 6.26793E+01, 1.8529E+01, 2.5530E+03, &
      1.4683830E+00, 6.26953E+01, 1.8432E+01, 2.5271E+03, &
      1.4757249E+00, 6.27106E+01, 1.8336E+01, 2.5016E+03, &
      1.4831035E+00, 6.27253E+01, 1.8240E+01, 2.4763E+03, &
      1.4905190E+00, 6.27392E+01, 1.8145E+01, 2.4513E+03, &
      1.4979716E+00, 6.27525E+01, 1.8051E+01, 2.4266E+03, &
      1.5054615E+00, 6.27649E+01, 1.7957E+01, 2.4020E+03, &
      1.5129888E+00, 6.27764E+01, 1.7863E+01, 2.3777E+03, &
      1.5205537E+00, 6.27871E+01, 1.7770E+01, 2.3536E+03, &
      1.5281565E+00, 6.27970E+01, 1.7677E+01, 2.3298E+03, &
      1.5357973E+00, 6.28061E+01, 1.7585E+01, 2.3063E+03, &
      1.5434763E+00, 6.28143E+01, 1.7494E+01, 2.2831E+03, &
      1.5511937E+00, 6.28218E+01, 1.7404E+01, 2.2601E+03, &
      1.5589496E+00, 6.28285E+01, 1.7314E+01, 2.2374E+03, &
      1.5667444E+00, 6.28344E+01, 1.7225E+01, 2.2149E+03, &
      1.5745781E+00, 6.28396E+01, 1.7136E+01, 2.1927E+03, &
      1.5824510E+00, 6.28439E+01, 1.7048E+01, 2.1707E+03, &
      1.5903633E+00, 6.28476E+01, 1.6961E+01, 2.1490E+03, &
      1.5983151E+00, 6.28504E+01, 1.6874E+01, 2.1275E+03, &
      1.6063066E+00, 6.28710E+01, 1.6788E+01, 2.1063E+03, &
      1.6143382E+00, 6.28725E+01, 1.6703E+01, 2.0853E+03, &
      1.6224099E+00, 6.28732E+01, 1.6618E+01, 2.0645E+03, &
      1.6305219E+00, 6.28732E+01, 1.6534E+01, 2.0440E+03, &
      1.6386745E+00, 6.28725E+01, 1.6451E+01, 2.0237E+03, &
      1.6468679E+00, 6.28710E+01, 1.6368E+01, 2.0036E+03, &
      1.6551022E+00, 6.28688E+01, 1.6286E+01, 1.9837E+03, &
      1.6633777E+00, 6.28658E+01, 1.6204E+01, 1.9641E+03, &
      1.6716946E+00, 6.28621E+01, 1.6123E+01, 1.9447E+03, &
      1.6800531E+00, 6.28577E+01, 1.6042E+01, 1.9255E+03, &
      1.6884534E+00, 6.28525E+01, 1.5963E+01, 1.9065E+03, &
      1.6968956E+00, 6.28466E+01, 1.5883E+01, 1.8877E+03, &
      1.7053801E+00, 6.28400E+01, 1.5804E+01, 1.8691E+03, &
      1.7139070E+00, 6.28326E+01, 1.5726E+01, 1.8507E+03, &
      1.7224766E+00, 6.28244E+01, 1.5649E+01, 1.8325E+03, &
      1.7310889E+00, 6.28156E+01, 1.5572E+01, 1.8146E+03, &
      1.7397444E+00, 6.28060E+01, 1.5495E+01, 1.7968E+03, &
      1.7484431E+00, 6.27956E+01, 1.5419E+01, 1.7792E+03, &
      1.7571853E+00, 6.27845E+01, 1.5344E+01, 1.7618E+03, &
      1.7659712E+00, 6.27726E+01, 1.5269E+01, 1.7446E+03, &
      1.7748011E+00, 6.27599E+01, 1.5195E+01, 1.7276E+03, &
      1.7836751E+00, 6.27465E+01, 1.5121E+01, 1.7108E+03, &
      1.7925935E+00, 6.27324E+01, 1.5047E+01, 1.6942E+03, &
      1.8015565E+00, 6.27174E+01, 1.4975E+01, 1.6777E+03, &
      1.8105642E+00, 6.27017E+01, 1.4903E+01, 1.6614E+03, &
      1.8196171E+00, 6.26852E+01, 1.4831E+01, 1.6453E+03, &
      1.8287151E+00, 6.26679E+01, 1.4760E+01, 1.6294E+03, &
      1.8378587E+00, 6.26499E+01, 1.4689E+01, 1.6136E+03, &
      1.8470480E+00, 6.26310E+01, 1.4619E+01, 1.5981E+03, &
      1.8562833E+00, 6.26113E+01, 1.4549E+01, 1.5827E+03, &
      1.8655647E+00, 6.25909E+01, 1.4480E+01, 1.5674E+03, &
      1.8748925E+00, 6.25696E+01, 1.4411E+01, 1.5523E+03, &
      1.8842670E+00, 6.25475E+01, 1.4343E+01, 1.5374E+03, &
      1.8936883E+00, 6.25246E+01, 1.4275E+01, 1.5227E+03, &
      1.9031567E+00, 6.25009E+01, 1.4207E+01, 1.5079E+03, &
      1.9126725E+00, 6.24759E+01, 1.4131E+01, 1.4925E+03, &
      1.9222359E+00, 6.24493E+01, 1.4055E+01, 1.4773E+03, &
      1.9318471E+00, 6.24212E+01, 1.3981E+01, 1.4623E+03, &
      1.9415063E+00, 6.23915E+01, 1.3907E+01, 1.4474E+03, &
      1.9512138E+00, 6.23603E+01, 1.3833E+01, 1.4327E+03, &
      1.9609699E+00, 6.23275E+01, 1.3760E+01, 1.4181E+03, &
      1.9707747E+00, 6.22931E+01, 1.3687E+01, 1.4038E+03, &
      1.9806286E+00, 6.22571E+01, 1.3615E+01, 1.3895E+03, &
      1.9905318E+00, 6.22196E+01, 1.3543E+01, 1.3755E+03, &
      2.0004844E+00, 6.21804E+01, 1.3472E+01, 1.3616E+03, &
      2.0104868E+00, 6.21396E+01, 1.3402E+01, 1.3478E+03, &
      2.0205393E+00, 6.20971E+01, 1.3332E+01, 1.3342E+03, &
      2.0306420E+00, 6.20529E+01, 1.3262E+01, 1.3208E+03, &
      2.0407952E+00, 6.20071E+01, 1.3193E+01, 1.3075E+03, &
      2.0509992E+00, 6.19595E+01, 1.3124E+01, 1.2943E+03, &
      2.0612542E+00, 6.19100E+01, 1.3056E+01, 1.2813E+03, &
      2.0715604E+00, 6.18588E+01, 1.2988E+01, 1.2684E+03, &
      2.0819182E+00, 6.18057E+01, 1.2920E+01, 1.2557E+03, &
      2.0923278E+00, 6.17467E+01, 1.2853E+01, 1.2430E+03, &
      2.1027895E+00, 6.16898E+01, 1.2786E+01, 1.2305E+03, &
      2.1133034E+00, 6.16309E+01, 1.2720E+01, 1.2181E+03, &
      2.1238699E+00, 6.15699E+01, 1.2654E+01, 1.2059E+03, &
      2.1344893E+00, 6.15067E+01, 1.2587E+01, 1.1937E+03, &
      2.1451617E+00, 6.14410E+01, 1.2516E+01, 1.1812E+03, &
      2.1558875E+00, 6.13725E+01, 1.2445E+01, 1.1688E+03, &
      2.1666670E+00, 6.13012E+01, 1.2375E+01, 1.1566E+03, &
      2.1775003E+00, 6.12270E+01, 1.2306E+01, 1.1445E+03, &
      2.1883878E+00, 6.11498E+01, 1.2236E+01, 1.1325E+03, &
      2.1993297E+00, 6.10695E+01, 1.2168E+01, 1.1207E+03, &
      2.2103264E+00, 6.09861E+01, 1.2100E+01, 1.1090E+03, &
      2.2213780E+00, 6.09045E+01, 1.2032E+01, 1.0974E+03, &
      2.2324849E+00, 6.08145E+01, 1.1965E+01, 1.0859E+03, &
      2.2436473E+00, 6.07210E+01, 1.1898E+01, 1.0746E+03, &
      2.2548656E+00, 6.06238E+01, 1.1832E+01, 1.0635E+03, &
      2.2661399E+00, 6.05229E+01, 1.1766E+01, 1.0524E+03, &
      2.2774706E+00, 6.04180E+01, 1.1701E+01, 1.0415E+03, &
      2.2888579E+00, 6.03090E+01, 1.1636E+01, 1.0307E+03, &
      2.3003022E+00, 6.01957E+01, 1.1572E+01, 1.0200E+03, &
      2.3118037E+00, 6.00779E+01, 1.1508E+01, 1.0094E+03, &
      2.3233628E+00, 5.99554E+01, 1.1444E+01, 9.9898E+02, &
      2.3349796E+00, 5.98280E+01, 1.1381E+01, 9.8865E+02, &
      2.3466545E+00, 5.96953E+01, 1.1318E+01, 9.7844E+02, &
      2.3583878E+00, 5.95571E+01, 1.1256E+01, 9.6834E+02, &
      2.3701797E+00, 5.94131E+01, 1.1195E+01, 9.5835E+02, &
      2.3820306E+00, 5.92630E+01, 1.1133E+01, 9.4848E+02, &
      2.3939407E+00, 5.91063E+01, 1.1072E+01, 9.3872E+02, &
      2.4059104E+00, 5.89427E+01, 1.1012E+01, 9.2906E+02, &
      2.4179400E+00, 5.87717E+01, 1.0952E+01, 9.1952E+02, &
      2.4300297E+00, 5.85928E+01, 1.0892E+01, 9.1007E+02, &
      2.4421798E+00, 5.84054E+01, 1.0833E+01, 9.0074E+02, &
      2.4543907E+00, 5.82090E+01, 1.0774E+01, 8.9151E+02, &
      2.4666627E+00, 5.80028E+01, 1.0716E+01, 8.8238E+02, &
      2.4789960E+00, 5.77860E+01, 1.0658E+01, 8.7335E+02, &
      2.4913910E+00, 5.75578E+01, 1.0600E+01, 8.6443E+02, &
      2.5038479E+00, 5.73171E+01, 1.0543E+01, 8.5560E+02, &
      2.5163672E+00, 5.70629E+01, 1.0486E+01, 8.4687E+02, &
      2.5289490E+00, 5.67938E+01, 1.0430E+01, 8.3824E+02, &
      2.5415938E+00, 5.65083E+01, 1.0374E+01, 8.2970E+02, &
      2.5543017E+00, 5.62047E+01, 1.0318E+01, 8.2126E+02, &
      2.5670732E+00, 5.58809E+01, 1.0263E+01, 8.1291E+02, &
      2.5799086E+00, 5.55345E+01, 1.0208E+01, 8.0465E+02, &
      2.5928082E+00, 5.51626E+01, 1.0154E+01, 7.9648E+02, &
      2.6057722E+00, 5.47618E+01, 1.0100E+01, 7.8840E+02, &
      2.6188011E+00, 5.43277E+01, 1.0046E+01, 7.8040E+02, &
      2.6318951E+00, 5.38551E+01, 9.9924E+00, 7.7249E+02, &
      2.6450545E+00, 5.33372E+01, 9.9392E+00, 7.6467E+02, &
      2.6582798E+00, 5.27621E+01, 9.8864E+00, 7.5693E+02, &
      2.6715712E+00, 5.21250E+01, 9.8340E+00, 7.4928E+02, &
      2.6849291E+00, 5.14068E+01, 9.7819E+00, 7.4171E+02, &
      2.6983537E+00, 5.05855E+01, 9.7302E+00, 7.3422E+02, &
      2.7118455E+00, 4.96278E+01, 9.6788E+00, 7.2682E+02, &
      2.7254047E+00, 4.84808E+01, 9.6278E+00, 7.1949E+02, &
      2.7390317E+00, 4.70512E+01, 9.5771E+00, 7.1225E+02, &
      2.7527269E+00, 4.51506E+01, 9.5267E+00, 7.0509E+02, &
      2.7664905E+00, 4.22866E+01, 9.4767E+00, 6.9800E+02, &
      2.7803230E+00, 3.60743E+01, 9.4271E+00, 6.9099E+02, &
      2.7859225E+00, 2.49183E+01, 9.4071E+00, 6.8819E+02, &
      2.7874775E+00, 2.46386E+01, 2.6001E+01, 1.8807E+03, &
      2.7942246E+00, 3.65860E+01, 2.5901E+01, 1.8690E+03, &
      2.8081957E+00, 4.19162E+01, 2.5695E+01, 1.8451E+03, &
      2.8222367E+00, 4.42477E+01, 2.5491E+01, 1.8214E+03, &
      2.8363479E+00, 4.55720E+01, 2.5289E+01, 1.7982E+03, &
      2.8505296E+00, 4.63024E+01, 2.5089E+01, 1.7752E+03, &
      2.8647823E+00, 4.65380E+01, 2.4890E+01, 1.7525E+03, &
      2.8791062E+00, 4.61883E+01, 2.4694E+01, 1.7301E+03, &
      2.8935017E+00, 4.47409E+01, 2.4499E+01, 1.7080E+03, &
      2.9079692E+00, 3.46640E+01, 2.4305E+01, 1.6863E+03, &
      2.9095697E+00, 3.52111E+01, 3.5654E+01, 2.4670E+03, &
      2.9225091E+00, 4.59635E+01, 3.5360E+01, 2.4359E+03, &
      2.9371216E+00, 4.92886E+01, 3.5032E+01, 2.4014E+03, &
      2.9518072E+00, 5.14263E+01, 3.4708E+01, 2.3675E+03, &
      2.9665662E+00, 5.30571E+01, 3.4386E+01, 2.3341E+03, &
      2.9813991E+00, 5.43935E+01, 3.4067E+01, 2.3010E+03, &
      2.9963061E+00, 5.55315E+01, 3.3751E+01, 2.2685E+03, &
      3.0112876E+00, 5.65390E+01, 3.3427E+01, 2.2357E+03, &
      3.0263440E+00, 5.74338E+01, 3.3103E+01, 2.2032E+03, &
      3.0414758E+00, 5.82322E+01, 3.2783E+01, 2.1711E+03, &
      3.0566831E+00, 5.89481E+01, 3.2472E+01, 2.1400E+03, &
      3.0719666E+00, 5.96009E+01, 3.2177E+01, 2.1102E+03, &
      3.0873264E+00, 6.02011E+01, 3.1888E+01, 2.0809E+03, &
      3.1027630E+00, 6.07542E+01, 3.1603E+01, 2.0522E+03, &
      3.1182768E+00, 6.12649E+01, 3.1323E+01, 2.0240E+03, &
      3.1338682E+00, 6.17370E+01, 3.1046E+01, 1.9963E+03, &
      3.1495376E+00, 6.21737E+01, 3.0774E+01, 1.9691E+03, &
      3.1652853E+00, 6.25775E+01, 3.0507E+01, 1.9424E+03, &
      3.1811117E+00, 6.29504E+01, 3.0243E+01, 1.9162E+03, &
      3.1970172E+00, 6.32938E+01, 2.9982E+01, 1.8903E+03, &
      3.2130023E+00, 6.36087E+01, 2.9726E+01, 1.8650E+03, &
      3.2290673E+00, 6.38955E+01, 2.9472E+01, 1.8400E+03, &
      3.2452127E+00, 6.41543E+01, 2.9223E+01, 1.8155E+03, &
      3.2614387E+00, 6.43844E+01, 2.8976E+01, 1.7914E+03, &
      3.2777459E+00, 6.45842E+01, 2.8733E+01, 1.7676E+03, &
      3.2941347E+00, 6.47511E+01, 2.8493E+01, 1.7442E+03, &
      3.3106053E+00, 6.48811E+01, 2.8256E+01, 1.7212E+03, &
      3.3271584E+00, 6.49672E+01, 2.8022E+01, 1.6986E+03, &
      3.3437941E+00, 6.49983E+01, 2.7790E+01, 1.6763E+03, &
      3.3605131E+00, 6.49547E+01, 2.7561E+01, 1.6544E+03, &
      3.3773157E+00, 6.47979E+01, 2.7335E+01, 1.6328E+03, &
      3.3942023E+00, 6.44375E+01, 2.7112E+01, 1.6115E+03, &
      3.4111733E+00, 6.35591E+01, 2.6891E+01, 1.5905E+03, &
      3.4220602E+00, 6.17180E+01, 2.6751E+01, 1.5773E+03, &
      3.4282291E+00, 6.08811E+01, 3.1482E+01, 1.8510E+03, &
      3.4299400E+00, 6.17851E+01, 3.1455E+01, 1.8485E+03, &
      3.4453703E+00, 6.44809E+01, 3.1216E+01, 1.8264E+03, &
      3.4625971E+00, 6.57151E+01, 3.0953E+01, 1.8021E+03, &
      3.4799101E+00, 6.65510E+01, 3.0693E+01, 1.7782E+03, &
      3.4973097E+00, 6.72080E+01, 3.0435E+01, 1.7546E+03, &
      3.5147962E+00, 6.77600E+01, 3.0180E+01, 1.7314E+03, &
      3.5323702E+00, 6.82409E+01, 2.9928E+01, 1.7085E+03, &
      3.5500321E+00, 6.86694E+01, 2.9677E+01, 1.6859E+03, &
      3.5677822E+00, 6.90562E+01, 2.9429E+01, 1.6635E+03, &
      3.5856211E+00, 6.94079E+01, 2.9182E+01, 1.6415E+03, &
      3.6035492E+00, 6.97284E+01, 2.8945E+01, 1.6202E+03, &
      3.6215670E+00, 7.00300E+01, 2.8720E+01, 1.5997E+03, &
      3.6396748E+00, 7.03141E+01, 2.8491E+01, 1.5792E+03, &
      3.6578732E+00, 7.05812E+01, 2.8264E+01, 1.5589E+03, &
      3.6761626E+00, 7.08327E+01, 2.8040E+01, 1.5390E+03, &
      3.6945434E+00, 7.10697E+01, 2.7819E+01, 1.5194E+03, &
      3.7130161E+00, 7.12934E+01, 2.7600E+01, 1.5001E+03, &
      3.7315812E+00, 7.15044E+01, 2.7384E+01, 1.4810E+03, &
      3.7502391E+00, 7.17034E+01, 2.7170E+01, 1.4622E+03, &
      3.7689903E+00, 7.18908E+01, 2.6958E+01, 1.4437E+03, &
      3.7878352E+00, 7.20668E+01, 2.6747E+01, 1.4254E+03, &
      3.8067744E+00, 7.22297E+01, 2.6534E+01, 1.4071E+03, &
      3.8258083E+00, 7.23791E+01, 2.6324E+01, 1.3892E+03, &
      3.8449373E+00, 7.25145E+01, 2.6116E+01, 1.3714E+03, &
      3.8641620E+00, 7.26348E+01, 2.5910E+01, 1.3539E+03, &
      3.8834828E+00, 7.27379E+01, 2.5706E+01, 1.3367E+03, &
      3.9029002E+00, 7.28209E+01, 2.5504E+01, 1.3197E+03, &
      3.9224147E+00, 7.28781E+01, 2.5304E+01, 1.3030E+03, &
      3.9420268E+00, 7.28994E+01, 2.5107E+01, 1.2865E+03, &
      3.9617369E+00, 7.28631E+01, 2.4911E+01, 1.2702E+03, &
      3.9815456E+00, 7.27088E+01, 2.4717E+01, 1.2542E+03, &
      4.0014533E+00, 7.20824E+01, 2.4525E+01, 1.2383E+03, &
      4.0023486E+00, 7.20069E+01, 2.4516E+01, 1.2376E+03, &
      4.0136512E+00, 7.20742E+01, 2.6173E+01, 1.3169E+03, &
      4.0214606E+00, 7.26244E+01, 2.6090E+01, 1.3102E+03, &
      4.0415679E+00, 7.32843E+01, 2.5878E+01, 1.2932E+03, &
      4.0617757E+00, 7.36831E+01, 2.5668E+01, 1.2764E+03, &
      4.0820846E+00, 7.39871E+01, 2.5460E+01, 1.2599E+03, &
      4.1024950E+00, 7.42385E+01, 2.5254E+01, 1.2436E+03, &
      4.1230075E+00, 7.44541E+01, 2.5050E+01, 1.2275E+03, &
      4.1436226E+00, 7.46422E+01, 2.4848E+01, 1.2117E+03, &
      4.1643407E+00, 7.48068E+01, 2.4648E+01, 1.1960E+03, &
      4.1851624E+00, 7.49494E+01, 2.4450E+01, 1.1806E+03, &
      4.2060882E+00, 7.50688E+01, 2.4254E+01, 1.1654E+03, &
      4.2271186E+00, 7.51655E+01, 2.4070E+01, 1.1509E+03, &
      4.2482542E+00, 7.52375E+01, 2.3888E+01, 1.1366E+03, &
      4.2694955E+00, 7.52703E+01, 2.3708E+01, 1.1226E+03, &
      4.2908430E+00, 7.52228E+01, 2.3530E+01, 1.1087E+03, &
      4.3080637E+00, 7.49852E+01, 2.3389E+01, 1.0978E+03, &
      4.3122972E+00, 7.48018E+01, 2.3355E+01, 1.0951E+03, &
      4.3259361E+00, 7.50946E+01, 2.4280E+01, 1.1345E+03, &
      4.3338587E+00, 7.53546E+01, 2.4216E+01, 1.1295E+03, &
      4.3555280E+00, 7.57624E+01, 2.4043E+01, 1.1159E+03, &
      4.3773056E+00, 7.60413E+01, 2.3872E+01, 1.1026E+03, &
      4.3991921E+00, 7.62710E+01, 2.3703E+01, 1.0894E+03, &
      4.4211881E+00, 7.64735E+01, 2.3535E+01, 1.0764E+03, &
      4.4432940E+00, 7.66586E+01, 2.3369E+01, 1.0636E+03, &
      4.4655105E+00, 7.68307E+01, 2.3202E+01, 1.0508E+03, &
      4.4878381E+00, 7.69916E+01, 2.3034E+01, 1.0381E+03, &
      4.5102772E+00, 7.71433E+01, 2.2868E+01, 1.0256E+03, &
      4.5328286E+00, 7.72873E+01, 2.2702E+01, 1.0132E+03, &
      4.5554928E+00, 7.74247E+01, 2.2538E+01, 1.0010E+03, &
      4.5782702E+00, 7.75560E+01, 2.2376E+01, 9.8888E+02, &
      4.6011616E+00, 7.76821E+01, 2.2215E+01, 9.7697E+02, &
      4.6241674E+00, 7.78034E+01, 2.2055E+01, 9.6521E+02, &
      4.6472882E+00, 7.79203E+01, 2.1897E+01, 9.5362E+02, &
      4.6705247E+00, 7.80333E+01, 2.1740E+01, 9.4217E+02, &
      4.6938773E+00, 7.81426E+01, 2.1585E+01, 9.3087E+02, &
      4.7173467E+00, 7.82484E+01, 2.1431E+01, 9.1972E+02, &
      4.7409334E+00, 7.83511E+01, 2.1278E+01, 9.0871E+02, &
      4.7646381E+00, 7.84507E+01, 2.1127E+01, 8.9785E+02, &
      4.7884613E+00, 7.85475E+01, 2.0977E+01, 8.8713E+02, &
      4.8124036E+00, 7.86417E+01, 2.0828E+01, 8.7654E+02, &
      4.8364656E+00, 7.87333E+01, 2.0681E+01, 8.6609E+02, &
      4.8606479E+00, 7.88225E+01, 2.0535E+01, 8.5578E+02, &
      4.8849512E+00, 7.89094E+01, 2.0390E+01, 8.4559E+02, &
      4.9093759E+00, 7.89942E+01, 2.0246E+01, 8.3554E+02, &
      4.9339228E+00, 7.90770E+01, 2.0103E+01, 8.2561E+02, &
      4.9585924E+00, 7.91577E+01, 1.9962E+01, 8.1581E+02, &
      4.9833854E+00, 7.92366E+01, 1.9822E+01, 8.0613E+02, &
      5.0083023E+00, 7.93138E+01, 1.9683E+01, 7.9657E+02, &
      5.0333438E+00, 7.93892E+01, 1.9545E+01, 7.8713E+02, &
      5.0585105E+00, 7.94631E+01, 1.9408E+01, 7.7781E+02, &
      5.0838031E+00, 7.95355E+01, 1.9272E+01, 7.6861E+02, &
      5.1092221E+00, 7.96065E+01, 1.9137E+01, 7.5952E+02, &
      5.1347682E+00, 7.96762E+01, 1.9004E+01, 7.5055E+02, &
      5.1604421E+00, 7.97447E+01, 1.8871E+01, 7.4168E+02, &
      5.1862443E+00, 7.98122E+01, 1.8740E+01, 7.3293E+02, &
      5.2121755E+00, 7.98789E+01, 1.8609E+01, 7.2427E+02, &
      5.2382364E+00, 7.99439E+01, 1.8475E+01, 7.1558E+02, &
      5.2644276E+00, 8.00070E+01, 1.8343E+01, 7.0700E+02, &
      5.2907497E+00, 8.00684E+01, 1.8212E+01, 6.9852E+02, &
      5.3172034E+00, 8.01281E+01, 1.8081E+01, 6.9015E+02, &
      5.3437895E+00, 8.01863E+01, 1.7952E+01, 6.8187E+02, &
      5.3705084E+00, 8.02431E+01, 1.7823E+01, 6.7370E+02, &
      5.3973609E+00, 8.02985E+01, 1.7696E+01, 6.6563E+02, &
      5.4243477E+00, 8.03526E+01, 1.7569E+01, 6.5765E+02, &
      5.4514695E+00, 8.04055E+01, 1.7443E+01, 6.4978E+02, &
      5.4787268E+00, 8.04572E+01, 1.7318E+01, 6.4200E+02, &
      5.5061205E+00, 8.05077E+01, 1.7195E+01, 6.3431E+02, &
      5.5336511E+00, 8.05572E+01, 1.7072E+01, 6.2672E+02, &
      5.5613193E+00, 8.06057E+01, 1.6950E+01, 6.1922E+02, &
      5.5891259E+00, 8.08195E+01, 1.6825E+01, 6.1170E+02, &
      5.6170716E+00, 8.08659E+01, 1.6700E+01, 6.0419E+02, &
      5.6451569E+00, 8.09106E+01, 1.6575E+01, 5.9676E+02, &
      5.6733827E+00, 8.09538E+01, 1.6451E+01, 5.8943E+02, &
      5.7017496E+00, 8.09955E+01, 1.6328E+01, 5.8218E+02, &
      5.7302584E+00, 8.10358E+01, 1.6206E+01, 5.7503E+02, &
      5.7589096E+00, 8.10748E+01, 1.6085E+01, 5.6798E+02, &
      5.7877042E+00, 8.11126E+01, 1.5964E+01, 5.6101E+02, &
      5.8166427E+00, 8.11491E+01, 1.5845E+01, 5.5413E+02, &
      5.8457259E+00, 8.12989E+01, 1.5724E+01, 5.4723E+02, &
      5.8749546E+00, 8.13329E+01, 1.5604E+01, 5.4043E+02, &
      5.9043293E+00, 8.13655E+01, 1.5485E+01, 5.3371E+02, &
      5.9338510E+00, 8.13968E+01, 1.5367E+01, 5.2708E+02, &
      5.9635202E+00, 8.14269E+01, 1.5250E+01, 5.2054E+02, &
      5.9933378E+00, 8.14558E+01, 1.5134E+01, 5.1408E+02, &
      6.0233045E+00, 8.14835E+01, 1.5019E+01, 5.0771E+02, &
      6.0534210E+00, 8.15100E+01, 1.4905E+01, 5.0142E+02, &
      6.0836882E+00, 8.15356E+01, 1.4792E+01, 4.9521E+02, &
      6.1141066E+00, 8.15600E+01, 1.4679E+01, 4.8908E+02, &
      6.1446771E+00, 8.15835E+01, 1.4568E+01, 4.8303E+02, &
      6.1754005E+00, 8.16061E+01, 1.4458E+01, 4.7706E+02, &
      6.2062775E+00, 8.16277E+01, 1.4349E+01, 4.7117E+02, &
      6.2373089E+00, 8.16483E+01, 1.4240E+01, 4.6536E+02, &
      6.2684954E+00, 8.16682E+01, 1.4133E+01, 4.5962E+02, &
      6.2998379E+00, 8.16871E+01, 1.4026E+01, 4.5395E+02, &
      6.3313371E+00, 8.17053E+01, 1.3921E+01, 4.4836E+02, &
      6.3629938E+00, 8.17226E+01, 1.3816E+01, 4.4285E+02, &
      6.3948088E+00, 8.17392E+01, 1.3712E+01, 4.3740E+02, &
      6.4267828E+00, 8.17551E+01, 1.3609E+01, 4.3203E+02, &
      6.4589167E+00, 8.17702E+01, 1.3507E+01, 4.2672E+02, &
      6.4912113E+00, 8.17846E+01, 1.3406E+01, 4.2149E+02, &
      6.5236674E+00, 8.17983E+01, 1.3306E+01, 4.1632E+02, &
      6.5562857E+00, 8.18114E+01, 1.3206E+01, 4.1122E+02, &
      6.5890671E+00, 8.18238E+01, 1.3108E+01, 4.0619E+02, &
      6.6220125E+00, 8.18356E+01, 1.3010E+01, 4.0123E+02, &
      6.6551225E+00, 8.18468E+01, 1.2913E+01, 3.9633E+02, &
      6.6883981E+00, 8.18575E+01, 1.2817E+01, 3.9149E+02, &
      6.7218401E+00, 8.18676E+01, 1.2722E+01, 3.8672E+02, &
      6.7554493E+00, 8.18771E+01, 1.2628E+01, 3.8201E+02, &
      6.7892266E+00, 8.18862E+01, 1.2535E+01, 3.7736E+02, &
      6.8231727E+00, 8.18947E+01, 1.2442E+01, 3.7277E+02, &
      6.8572886E+00, 8.19920E+01, 1.2350E+01, 3.6823E+02, &
      6.8915750E+00, 8.19999E+01, 1.2257E+01, 3.6371E+02, &
      6.9260329E+00, 8.20071E+01, 1.2165E+01, 3.5924E+02, &
      6.9606631E+00, 8.20137E+01, 1.2073E+01, 3.5483E+02, &
      6.9954664E+00, 8.20197E+01, 1.1983E+01, 3.5048E+02, &
      7.0304437E+00, 8.20251E+01, 1.1893E+01, 3.4618E+02, &
      7.0655959E+00, 8.20299E+01, 1.1804E+01, 3.4195E+02, &
      7.1009239E+00, 8.20342E+01, 1.1716E+01, 3.3776E+02, &
      7.1364285E+00, 8.20380E+01, 1.1628E+01, 3.3364E+02, &
      7.1721107E+00, 8.20412E+01, 1.1542E+01, 3.2956E+02, &
      7.2079712E+00, 8.20439E+01, 1.1456E+01, 3.2554E+02, &
      7.2440111E+00, 8.20462E+01, 1.1371E+01, 3.2157E+02, &
      7.2802311E+00, 8.20480E+01, 1.1286E+01, 3.1766E+02, &
      7.3166323E+00, 8.20493E+01, 1.1203E+01, 3.1379E+02, &
      7.3532155E+00, 8.20503E+01, 1.1120E+01, 3.0998E+02, &
      7.3899815E+00, 8.20508E+01, 1.1037E+01, 3.0622E+02, &
      7.4269314E+00, 8.20509E+01, 1.0956E+01, 3.0250E+02, &
      7.4640661E+00, 8.20506E+01, 1.0875E+01, 2.9883E+02, &
      7.5013864E+00, 8.20499E+01, 1.0795E+01, 2.9521E+02, &
      7.5388934E+00, 8.20489E+01, 1.0716E+01, 2.9164E+02, &
      7.5765878E+00, 8.20475E+01, 1.0637E+01, 2.8812E+02, &
      7.6144708E+00, 8.20458E+01, 1.0559E+01, 2.8464E+02, &
      7.6525431E+00, 8.20437E+01, 1.0482E+01, 2.8121E+02, &
      7.6908058E+00, 8.20413E+01, 1.0405E+01, 2.7782E+02, &
      7.7292599E+00, 8.20387E+01, 1.0329E+01, 2.7447E+02, &
      7.7679062E+00, 8.20357E+01, 1.0254E+01, 2.7117E+02, &
      7.8067457E+00, 8.20325E+01, 1.0179E+01, 2.6791E+02, &
      7.8457794E+00, 8.20289E+01, 1.0105E+01, 2.6469E+02, &
      7.8850083E+00, 8.20252E+01, 1.0032E+01, 2.6152E+02, &
      7.9244334E+00, 8.20212E+01, 9.9593E+00, 2.5839E+02, &
      7.9640555E+00, 8.20169E+01, 9.8872E+00, 2.5529E+02, &
      8.0038758E+00, 8.20125E+01, 9.8158E+00, 2.5224E+02, &
      8.0438952E+00, 8.20452E+01, 9.7444E+00, 2.4921E+02, &
      8.0841147E+00, 8.20404E+01, 9.6733E+00, 2.4622E+02, &
      8.1245352E+00, 8.20354E+01, 9.6028E+00, 2.4326E+02, &
      8.1651579E+00, 8.20300E+01, 9.5329E+00, 2.4034E+02, &
      8.2059837E+00, 8.20244E+01, 9.4636E+00, 2.3745E+02, &
      8.2470136E+00, 8.20186E+01, 9.3948E+00, 2.3461E+02, &
      8.2882487E+00, 8.20126E+01, 9.3267E+00, 2.3180E+02, &
      8.3296899E+00, 8.20064E+01, 9.2592E+00, 2.2902E+02, &
      8.3713384E+00, 8.20000E+01, 9.1922E+00, 2.2629E+02, &
      8.4131951E+00, 8.19934E+01, 9.1258E+00, 2.2358E+02, &
      8.4552610E+00, 8.19867E+01, 9.0600E+00, 2.2091E+02, &
      8.4975373E+00, 8.19799E+01, 8.9947E+00, 2.1828E+02, &
      8.5400250E+00, 8.19730E+01, 8.9299E+00, 2.1568E+02, &
      9.030794E+00, 8.22117E+01, 8.2073E+00, 1.8801E+02, &
      9.653919E+00, 8.19647E+01, 7.3369E+00, 1.5784E+02, &
      1.032004E+01, 8.16115E+01, 6.5680E+00, 1.3275E+02, &
      1.103212E+01, 8.11566E+01, 5.8878E+00, 1.1184E+02, &
      1.179334E+01, 8.05644E+01, 5.2868E+00, 9.4413E+01, &
      1.260708E+01, 7.97232E+01, 4.7459E+00, 7.9717E+01, &
      1.347697E+01, 7.81972E+01, 4.2563E+00, 6.7280E+01, &
      1.392923E+01, 7.63888E+01, 4.0362E+00, 6.1920E+01, &
      1.414243E+01, 7.37593E+01, 3.9386E+00, 5.9601E+01, &
      1.419929E+01, 7.06732E+01, 3.9133E+00, 5.9003E+01, &
      1.428457E+01, 7.37471E+01, 9.9118E+00, 1.4279E+02, &
      1.440688E+01, 7.57156E+01, 9.7728E+00, 1.3965E+02, &
      1.449777E+01, 7.64679E+01, 9.6715E+00, 1.3737E+02, &
      1.540095E+01, 7.89363E+01, 8.7318E+00, 1.1707E+02, &
      1.646362E+01, 7.86416E+01, 7.7790E+00, 9.7886E+01, &
      1.670078E+01, 7.74554E+01, 7.5881E+00, 9.4199E+01, &
      1.676792E+01, 7.59108E+01, 7.5353E+00, 9.3190E+01, &
      1.686862E+01, 7.74800E+01, 1.0515E+01, 1.2803E+02, &
      1.712039E+01, 7.88175E+01, 1.0261E+01, 1.2316E+02, &
      1.714314E+01, 7.88698E+01, 1.0238E+01, 1.2273E+02, &
      1.740554E+01, 7.89277E+01, 9.9842E+00, 1.1795E+02, &
      1.747551E+01, 7.82876E+01, 9.9178E+00, 1.1671E+02, &
      1.758047E+01, 7.92760E+01, 1.1370E+01, 1.3256E+02, &
      1.759961E+01, 7.94102E+01, 1.1351E+01, 1.3221E+02, &
      1.784286E+01, 8.04060E+01, 1.1116E+01, 1.2776E+02, &
      1.881398E+01, 8.21471E+01, 1.0243E+01, 1.1185E+02, &
      2.011215E+01, 8.32985E+01, 9.2187E+00, 9.4382E+01, &
      2.149988E+01, 8.40104E+01, 8.2961E+00, 7.9657E+01, &
      2.298338E+01, 8.44861E+01, 7.4647E+00, 6.7233E+01, &
      2.456923E+01, 8.48104E+01, 6.7170E+00, 5.6760E+01, &
      2.626450E+01, 8.50419E+01, 6.0480E+00, 4.7958E+01, &
      2.807676E+01, 8.52036E+01, 5.4145E+00, 4.0308E+01, &
      3.001405E+01, 8.53412E+01, 4.8312E+00, 3.3780E+01, &
      3.208502E+01, 8.53658E+01, 4.3129E+00, 2.8333E+01, &
      3.429889E+01, 8.53929E+01, 3.8515E+00, 2.3780E+01, &
      3.666551E+01, 8.53852E+01, 3.4298E+00, 1.9913E+01, &
      3.919543E+01, 8.53260E+01, 3.0538E+00, 1.6679E+01, &
      4.189992E+01, 8.52512E+01, 2.7210E+00, 1.3987E+01, &
      4.479101E+01, 8.51663E+01, 2.4264E+00, 1.1744E+01, &
      4.788159E+01, 8.50743E+01, 2.1653E+00, 9.8734E+00, &
      5.118542E+01, 8.49773E+01, 1.9337E+00, 8.3112E+00, &
      5.471721E+01, 8.48760E+01, 1.7282E+00, 7.0054E+00, &
      5.849270E+01, 8.47701E+01, 1.5457E+00, 5.9131E+00, &
      6.252870E+01, 8.46586E+01, 1.3827E+00, 4.9957E+00, &
      6.684318E+01, 8.45368E+01, 1.2356E+00, 4.2206E+00, &
      7.145536E+01, 8.43993E+01, 1.1052E+00, 3.5721E+00, &
      7.638578E+01, 8.42357E+01, 9.8789E-01, 3.0247E+00, &
      8.165640E+01, 8.40203E+01, 8.7854E-01, 2.5534E+00, &
      8.729069E+01, 8.36783E+01, 7.7724E-01, 2.1495E+00, &
      9.331374E+01, 8.28155E+01, 6.8813E-01, 1.8139E+00, &
      9.381530E+01, 8.26491E+01, 6.8145E-01, 1.7895E+00, &
      9.525125E+01, 8.16283E+01, 6.6290E-01, 1.7224E+00, &
      9.563417E+01, 8.04074E+01, 6.5809E-01, 1.7052E+00, &
      9.620855E+01, 8.16796E+01, 3.0533E+00, 6.6833E+00, &
      9.764449E+01, 8.27918E+01, 2.9843E+00, 6.4423E+00, &
      9.975239E+01, 8.34034E+01, 2.8871E+00, 6.1096E+00, &
      1.066353E+02, 8.42221E+01, 2.6010E+00, 5.1740E+00, &
      1.139931E+02, 8.46138E+01, 2.3399E+00, 4.3783E+00, &
      1.218587E+02, 8.48511E+01, 2.1028E+00, 3.7040E+00, &
      1.302669E+02, 8.50066E+01, 1.8893E+00, 3.1351E+00, &
      1.392553E+02, 8.51125E+01, 1.6961E+00, 2.6538E+00, &
      1.488640E+02, 8.51831E+01, 1.5209E+00, 2.2466E+00, &
      1.591356E+02, 8.52292E+01, 1.3638E+00, 1.9040E+00, &
      1.701159E+02, 8.52579E+01, 1.2230E+00, 1.6159E+00, &
      1.818539E+02, 8.52741E+01, 1.0968E+00, 1.3736E+00, &
      1.944018E+02, 8.52812E+01, 9.8376E-01, 1.1696E+00, &
      2.078156E+02, 8.52817E+01, 8.8244E-01, 9.9794E-01, &
      2.221548E+02, 8.52773E+01, 7.9163E-01, 8.5335E-01, &
      2.374835E+02, 8.52695E+01, 7.1025E-01, 7.3149E-01, &
      2.538699E+02, 8.52593E+01, 6.3731E-01, 6.2872E-01, &
      2.713869E+02, 8.52474E+01, 5.7192E-01, 5.4198E-01, &
      2.901126E+02, 8.52346E+01, 5.1329E-01, 4.6870E-01, &
      3.101304E+02, 8.52212E+01, 4.6072E-01, 4.0672E-01, &
      3.315294E+02, 8.52076E+01, 4.1357E-01, 3.5424E-01, &
      3.544049E+02, 8.51941E+01, 3.7129E-01, 3.0974E-01, &
      3.788588E+02, 8.51808E+01, 3.3336E-01, 2.7195E-01, &
      4.050001E+02, 8.51679E+01, 2.9933E-01, 2.3980E-01, &
      4.329451E+02, 8.51555E+01, 2.6880E-01, 2.1239E-01]
   case(86)
      frel = -1.2570E+00
      allocate(nist_tab(3092))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      6.983358E-03, -1.40095E-01, 0.0000E+00, 1.7617E-05, &
      7.090247E-03, -3.65878E+00, 0.0000E+00, 1.9096E-05, &
      7.118750E-03, -7.81185E+00, 0.0000E+00, 1.9506E-05, &
      7.161505E-03, -3.56804E+00, 8.1423E+00, 2.1551E+05, &
      7.268393E-03, 1.18324E-01, 8.0559E+00, 2.1009E+05, &
      1.036116E-02, 5.66085E+00, 6.1586E+00, 1.1267E+05, &
      1.051975E-02, 3.81936E+00, 6.0731E+00, 1.0943E+05, &
      1.056204E-02, 1.52197E+00, 6.0503E+00, 1.0858E+05, &
      1.062547E-02, 3.94006E+00, 1.0589E+01, 1.8890E+05, &
      1.069000E-02, 5.17172E+00, 1.0549E+01, 1.8705E+05, &
      1.078406E-02, 6.12627E+00, 1.0490E+01, 1.8438E+05, &
      1.142761E-02, 8.74822E+00, 1.0095E+01, 1.6745E+05, &
      1.221612E-02, 1.02947E+01, 9.6193E+00, 1.4926E+05, &
      1.305903E-02, 1.14147E+01, 9.1051E+00, 1.3216E+05, &
      1.396010E-02, 1.22984E+01, 8.5549E+00, 1.1616E+05, &
      1.492335E-02, 1.30077E+01, 7.9757E+00, 1.0130E+05, &
      1.595306E-02, 1.35690E+01, 7.3766E+00, 8.7647E+04, &
      1.705382E-02, 1.39950E+01, 6.7681E+00, 7.5226E+04, &
      1.823053E-02, 1.42914E+01, 6.1609E+00, 6.4058E+04, &
      1.948844E-02, 1.44558E+01, 5.5654E+00, 5.4131E+04, &
      2.083314E-02, 1.44536E+01, 4.9909E+00, 4.5409E+04, &
      2.150088E-02, 1.43221E+01, 4.7288E+00, 4.1689E+04, &
      2.182997E-02, 1.40652E+01, 4.6051E+00, 3.9986E+04, &
      2.191773E-02, 1.37421E+01, 4.5727E+00, 3.9546E+04, &
      2.204937E-02, 1.40981E+01, 5.1661E+00, 4.4411E+04, &
      2.227063E-02, 1.43434E+01, 5.0600E+00, 4.3067E+04, &
      2.237846E-02, 1.44059E+01, 5.0094E+00, 4.2431E+04, &
      2.380730E-02, 1.46485E+01, 4.3980E+00, 3.5016E+04, &
      2.545001E-02, 1.45702E+01, 3.8144E+00, 2.8410E+04, &
      2.720606E-02, 1.43338E+01, 3.3050E+00, 2.3027E+04, &
      2.908327E-02, 1.39752E+01, 2.8602E+00, 1.8642E+04, &
      3.109002E-02, 1.34979E+01, 2.4732E+00, 1.5078E+04, &
      3.323523E-02, 1.28882E+01, 2.1375E+00, 1.2191E+04, &
      3.552846E-02, 1.21116E+01, 1.8474E+00, 9.8560E+03, &
      3.797993E-02, 1.11011E+01, 1.5975E+00, 7.9729E+03, &
      4.060054E-02, 9.71476E+00, 1.3831E+00, 6.4573E+03, &
      4.340198E-02, 7.50250E+00, 1.1998E+00, 5.2397E+03, &
      4.403377E-02, 6.61733E+00, 1.1638E+00, 5.0097E+03, &
      4.421079E-02, 6.06317E+00, 1.1540E+00, 4.9477E+03, &
      4.447632E-02, 6.11258E+00, 1.9003E+00, 8.0988E+03, &
      4.514014E-02, 5.70144E+00, 2.0626E+00, 8.6612E+03, &
      4.639671E-02, 4.46382E+00, 2.5115E+00, 1.0260E+04, &
      4.771733E-02, 2.86062E+00, 3.2560E+00, 1.2934E+04, &
      4.844769E-02, 1.73346E+00, 3.8310E+00, 1.4989E+04, &
      4.864246E-02, 1.20154E+00, 4.0084E+00, 1.5620E+04, &
      4.893461E-02, 1.09802E+00, 4.8805E+00, 1.8905E+04, &
      4.959809E-02, 5.48964E-01, 5.7157E+00, 2.1844E+04, &
      4.966497E-02, 4.87096E-01, 5.8056E+00, 2.2158E+04, &
      5.302035E-02, -1.93666E+00, 1.1563E+01, 4.1339E+04, &
      5.667876E-02, -1.22725E+00, 1.9562E+01, 6.5421E+04, &
      6.058959E-02, 3.88824E+00, 2.6915E+01, 8.4201E+04, &
      6.477028E-02, 1.20963E+01, 3.0979E+01, 9.0662E+04, &
      6.923942E-02, 2.07994E+01, 3.0837E+01, 8.4419E+04, &
      7.401695E-02, 2.77900E+01, 2.7401E+01, 7.0172E+04, &
      7.912411E-02, 3.21498E+01, 2.2374E+01, 5.3600E+04, &
      8.458368E-02, 3.40728E+01, 1.7225E+01, 3.8601E+04, &
      9.041995E-02, 3.42577E+01, 1.2783E+01, 2.6797E+04, &
      9.665893E-02, 3.34211E+01, 9.3169E+00, 1.8271E+04, &
      1.033284E-01, 3.20793E+01, 6.7727E+00, 1.2424E+04, &
      1.104581E-01, 3.04937E+01, 4.9700E+00, 8.5293E+03, &
      1.164403E-01, 2.89252E+01, 3.9417E+00, 6.4172E+03, &
      1.180797E-01, 2.81442E+01, 3.7156E+00, 5.9652E+03, &
      1.182225E-01, 2.80250E+01, 3.6969E+00, 5.9279E+03, &
      1.186978E-01, 2.72188E+01, 3.6356E+00, 5.8063E+03, &
      1.194107E-01, 2.78741E+01, 4.9830E+00, 7.9106E+03, &
      1.211929E-01, 2.82651E+01, 4.7285E+00, 7.3962E+03, &
      1.262272E-01, 2.80674E+01, 4.1171E+00, 6.1833E+03, &
      1.349368E-01, 2.71812E+01, 3.3446E+00, 4.6991E+03, &
      1.442475E-01, 2.61099E+01, 2.8039E+00, 3.6855E+03, &
      1.487358E-01, 2.54432E+01, 2.6129E+00, 3.3310E+03, &
      1.510123E-01, 2.48184E+01, 2.5291E+00, 3.1756E+03, &
      1.516194E-01, 2.42440E+01, 2.5080E+00, 3.1366E+03, &
      1.525301E-01, 2.47286E+01, 3.5104E+00, 4.3636E+03, &
      1.542005E-01, 2.49993E+01, 3.4236E+00, 4.2096E+03, &
      1.548066E-01, 2.50287E+01, 3.3934E+00, 4.1560E+03, &
      1.648404E-01, 2.47151E+01, 2.9726E+00, 3.4195E+03, &
      1.762144E-01, 2.39650E+01, 2.6352E+00, 2.8361E+03, &
      1.883732E-01, 2.30160E+01, 2.3872E+00, 2.4037E+03, &
      1.968143E-01, 2.22510E+01, 2.2627E+00, 2.1809E+03, &
      1.998268E-01, 2.19090E+01, 2.2257E+00, 2.1131E+03, &
      2.006301E-01, 2.17529E+01, 2.2165E+00, 2.0959E+03, &
      2.013709E-01, 2.17327E+01, 2.3672E+00, 2.2300E+03, &
      2.018351E-01, 2.17199E+01, 2.3624E+00, 2.2204E+03, &
      2.048475E-01, 2.14941E+01, 2.3331E+00, 2.1608E+03, &
      2.083358E-01, 2.11551E+01, 2.3028E+00, 2.0971E+03, &
      2.115247E-01, 2.07692E+01, 2.2781E+00, 2.0434E+03, &
      2.123750E-01, 2.05986E+01, 2.2719E+00, 2.0298E+03, &
      2.136505E-01, 2.05293E+01, 2.4240E+00, 2.1526E+03, &
      2.152655E-01, 2.03888E+01, 2.4306E+00, 2.1423E+03, &
      2.168393E-01, 2.02233E+01, 2.4393E+00, 2.1344E+03, &
      2.185326E-01, 2.00163E+01, 2.4512E+00, 2.1282E+03, &
      2.194112E-01, 1.98639E+01, 2.4584E+00, 2.1260E+03, &
      2.207289E-01, 1.97713E+01, 2.5887E+00, 2.2252E+03, &
      2.240234E-01, 1.94461E+01, 2.6548E+00, 2.2485E+03, &
      2.301188E-01, 1.87511E+01, 2.8285E+00, 2.3322E+03, &
      2.459970E-01, 1.67587E+01, 3.6929E+00, 2.8481E+03, &
      2.629708E-01, 1.46405E+01, 5.3284E+00, 3.8435E+03, &
      2.811158E-01, 1.27195E+01, 7.8484E+00, 5.2951E+03, &
      3.005128E-01, 1.13765E+01, 1.1196E+01, 7.0656E+03, &
      3.212482E-01, 1.09309E+01, 1.5124E+01, 8.9275E+03, &
      3.434143E-01, 1.15467E+01, 1.9260E+01, 1.0635E+04, &
      3.671099E-01, 1.31988E+01, 2.3215E+01, 1.1990E+04, &
      3.924405E-01, 1.57191E+01, 2.6664E+01, 1.2884E+04, &
      4.195189E-01, 1.88335E+01, 2.9399E+01, 1.3288E+04, &
      4.484657E-01, 2.22323E+01, 3.1330E+01, 1.3248E+04, &
      4.794098E-01, 2.55879E+01, 3.2465E+01, 1.2842E+04, &
      5.0000000E-01, 2.95309E+01, 3.2803E+01, 1.2442E+04, &
      5.0250000E-01, 2.97362E+01, 3.2825E+01, 1.2388E+04, &
      5.0501250E-01, 2.99351E+01, 3.2843E+01, 1.2334E+04, &
      5.0753756E-01, 3.01269E+01, 3.2858E+01, 1.2278E+04, &
      5.1007525E-01, 3.03105E+01, 3.2870E+01, 1.2221E+04, &
      5.1262563E-01, 3.04844E+01, 3.2878E+01, 1.2163E+04, &
      5.1518875E-01, 3.06471E+01, 3.2882E+01, 1.2105E+04, &
      5.1776470E-01, 3.07960E+01, 3.2884E+01, 1.2045E+04, &
      5.2035352E-01, 3.09280E+01, 3.2881E+01, 1.1984E+04, &
      5.2295529E-01, 3.10380E+01, 3.2876E+01, 1.1923E+04, &
      5.2557007E-01, 3.11182E+01, 3.2867E+01, 1.1860E+04, &
      5.2819792E-01, 3.11544E+01, 3.2855E+01, 1.1797E+04, &
      5.3083891E-01, 3.11176E+01, 3.2840E+01, 1.1733E+04, &
      5.3349310E-01, 3.09285E+01, 3.2822E+01, 1.1668E+04, &
      5.3616057E-01, 3.01218E+01, 3.2801E+01, 1.1603E+04, &
      5.3662464E-01, 2.95906E+01, 3.2797E+01, 1.1592E+04, &
      5.3737536E-01, 2.96396E+01, 3.4991E+01, 1.2349E+04, &
      5.3884137E-01, 3.08512E+01, 3.4990E+01, 1.2315E+04, &
      5.4153558E-01, 3.16665E+01, 3.4986E+01, 1.2253E+04, &
      5.4424325E-01, 3.21760E+01, 3.4979E+01, 1.2189E+04, &
      5.4696447E-01, 3.25751E+01, 3.4969E+01, 1.2125E+04, &
      5.4969929E-01, 3.29118E+01, 3.4958E+01, 1.2061E+04, &
      5.5244779E-01, 3.32033E+01, 3.4943E+01, 1.1996E+04, &
      5.5521003E-01, 3.34546E+01, 3.4926E+01, 1.1931E+04, &
      5.5798608E-01, 3.36619E+01, 3.4907E+01, 1.1865E+04, &
      5.6077601E-01, 3.38074E+01, 3.4886E+01, 1.1799E+04, &
      5.6357989E-01, 3.38277E+01, 3.4862E+01, 1.1732E+04, &
      5.6620227E-01, 3.32339E+01, 3.4838E+01, 1.1670E+04, &
      5.6639779E-01, 3.29664E+01, 3.4836E+01, 1.1665E+04, &
      5.6699778E-01, 3.33106E+01, 3.6163E+01, 1.2097E+04, &
      5.6922978E-01, 3.43287E+01, 3.6153E+01, 1.2046E+04, &
      5.7207593E-01, 3.49154E+01, 3.6137E+01, 1.1981E+04, &
      5.7493630E-01, 3.53665E+01, 3.6120E+01, 1.1916E+04, &
      5.7781099E-01, 3.57617E+01, 3.6101E+01, 1.1850E+04, &
      5.8070004E-01, 3.61253E+01, 3.6081E+01, 1.1784E+04, &
      5.8360354E-01, 3.64681E+01, 3.6058E+01, 1.1719E+04, &
      5.8652156E-01, 3.67957E+01, 3.6034E+01, 1.1653E+04, &
      5.8945417E-01, 3.71118E+01, 3.6008E+01, 1.1586E+04, &
      5.9240144E-01, 3.74186E+01, 3.5981E+01, 1.1520E+04, &
      5.9536345E-01, 3.77178E+01, 3.5951E+01, 1.1453E+04, &
      5.9834026E-01, 3.80104E+01, 3.5920E+01, 1.1387E+04, &
      6.0133196E-01, 3.82972E+01, 3.5887E+01, 1.1319E+04, &
      6.0433862E-01, 3.85799E+01, 3.5852E+01, 1.1252E+04, &
      6.0736032E-01, 3.88567E+01, 3.5815E+01, 1.1185E+04, &
      6.1039712E-01, 3.91291E+01, 3.5776E+01, 1.1117E+04, &
      6.1344910E-01, 3.93975E+01, 3.5735E+01, 1.1049E+04, &
      6.1651635E-01, 3.96621E+01, 3.5693E+01, 1.0982E+04, &
      6.1959893E-01, 3.99233E+01, 3.5649E+01, 1.0914E+04, &
      6.2269693E-01, 4.01811E+01, 3.5604E+01, 1.0845E+04, &
      6.2581041E-01, 4.04359E+01, 3.5557E+01, 1.0777E+04, &
      6.2893946E-01, 4.06877E+01, 3.5508E+01, 1.0709E+04, &
      6.3208416E-01, 4.09366E+01, 3.5457E+01, 1.0641E+04, &
      6.3524458E-01, 4.11827E+01, 3.5404E+01, 1.0572E+04, &
      6.3842080E-01, 4.13791E+01, 3.5350E+01, 1.0503E+04, &
      6.4161291E-01, 4.16194E+01, 3.5293E+01, 1.0434E+04, &
      6.4482097E-01, 4.18566E+01, 3.5235E+01, 1.0365E+04, &
      6.4804508E-01, 4.20910E+01, 3.5175E+01, 1.0296E+04, &
      6.5128530E-01, 4.23224E+01, 3.5113E+01, 1.0227E+04, &
      6.5454173E-01, 4.25510E+01, 3.5049E+01, 1.0158E+04, &
      6.5781444E-01, 4.27766E+01, 3.4984E+01, 1.0089E+04, &
      6.6110351E-01, 4.29767E+01, 3.4918E+01, 1.0019E+04, &
      6.6440903E-01, 4.31965E+01, 3.4849E+01, 9.9502E+03, &
      6.6773107E-01, 4.34133E+01, 3.4780E+01, 9.8810E+03, &
      6.7106973E-01, 4.36272E+01, 3.4709E+01, 9.8118E+03, &
      6.7442508E-01, 4.38380E+01, 3.4636E+01, 9.7427E+03, &
      6.7779720E-01, 4.40458E+01, 3.4562E+01, 9.6737E+03, &
      6.8118619E-01, 4.42505E+01, 3.4487E+01, 9.6047E+03, &
      6.8459212E-01, 4.44519E+01, 3.4411E+01, 9.5358E+03, &
      6.8801508E-01, 4.46501E+01, 3.4333E+01, 9.4670E+03, &
      6.9145515E-01, 4.48450E+01, 3.4254E+01, 9.3983E+03, &
      6.9491243E-01, 4.50363E+01, 3.4174E+01, 9.3298E+03, &
      6.9838699E-01, 4.52240E+01, 3.4092E+01, 9.2613E+03, &
      7.0187893E-01, 4.54079E+01, 3.4010E+01, 9.1931E+03, &
      7.0538832E-01, 4.55879E+01, 3.3927E+01, 9.1250E+03, &
      7.0891526E-01, 4.57637E+01, 3.3842E+01, 9.0570E+03, &
      7.1245984E-01, 4.59350E+01, 3.3756E+01, 8.9893E+03, &
      7.1602214E-01, 4.61016E+01, 3.3670E+01, 8.9217E+03, &
      7.1960225E-01, 4.62634E+01, 3.3582E+01, 8.8542E+03, &
      7.2320026E-01, 4.64192E+01, 3.3493E+01, 8.7869E+03, &
      7.2681626E-01, 4.65686E+01, 3.3403E+01, 8.7198E+03, &
      7.3045034E-01, 4.67106E+01, 3.3312E+01, 8.6530E+03, &
      7.3410260E-01, 4.68442E+01, 3.3221E+01, 8.5864E+03, &
      7.3777311E-01, 4.69678E+01, 3.3128E+01, 8.5200E+03, &
      7.4146197E-01, 4.70795E+01, 3.3035E+01, 8.4539E+03, &
      7.4516928E-01, 4.71760E+01, 3.2941E+01, 8.3880E+03, &
      7.4889513E-01, 4.72528E+01, 3.2847E+01, 8.3224E+03, &
      7.5263961E-01, 4.73017E+01, 3.2752E+01, 8.2571E+03, &
      7.5640280E-01, 4.73079E+01, 3.2656E+01, 8.1921E+03, &
      7.6018482E-01, 4.72386E+01, 3.2560E+01, 8.1274E+03, &
      7.6398574E-01, 4.69914E+01, 3.2463E+01, 8.0630E+03, &
      7.6674817E-01, 4.63685E+01, 3.2392E+01, 8.0166E+03, &
      7.6780567E-01, 4.52125E+01, 3.2365E+01, 7.9989E+03, &
      7.6925185E-01, 4.64868E+01, 3.4363E+01, 8.4762E+03, &
      7.7164470E-01, 4.72908E+01, 3.4303E+01, 8.4352E+03, &
      7.7550292E-01, 4.79373E+01, 3.4206E+01, 8.3697E+03, &
      7.7938044E-01, 4.83848E+01, 3.4109E+01, 8.3045E+03, &
      7.8327734E-01, 4.87516E+01, 3.4012E+01, 8.2396E+03, &
      7.8719373E-01, 4.90741E+01, 3.3914E+01, 8.1751E+03, &
      7.9112969E-01, 4.93681E+01, 3.3815E+01, 8.1110E+03, &
      7.9508534E-01, 4.96421E+01, 3.3717E+01, 8.0472E+03, &
      7.9906077E-01, 4.99010E+01, 3.3618E+01, 7.9838E+03, &
      8.0305607E-01, 5.01481E+01, 3.3519E+01, 7.9207E+03, &
      8.0707135E-01, 5.03857E+01, 3.3419E+01, 7.8580E+03, &
      8.1110671E-01, 5.06152E+01, 3.3320E+01, 7.7957E+03, &
      8.1516224E-01, 5.08379E+01, 3.3220E+01, 7.7338E+03, &
      8.1923806E-01, 5.10546E+01, 3.3120E+01, 7.6723E+03, &
      8.2333425E-01, 5.12662E+01, 3.3020E+01, 7.6112E+03, &
      8.2745092E-01, 5.14730E+01, 3.2920E+01, 7.5504E+03, &
      8.3158817E-01, 5.16756E+01, 3.2820E+01, 7.4901E+03, &
      8.3574611E-01, 5.18744E+01, 3.2719E+01, 7.4300E+03, &
      8.3992484E-01, 5.20697E+01, 3.2618E+01, 7.3704E+03, &
      8.4412447E-01, 5.22617E+01, 3.2517E+01, 7.3111E+03, &
      8.4834509E-01, 5.24507E+01, 3.2416E+01, 7.2522E+03, &
      8.5258682E-01, 5.26369E+01, 3.2315E+01, 7.1938E+03, &
      8.5684975E-01, 5.28203E+01, 3.2214E+01, 7.1357E+03, &
      8.6113400E-01, 5.30013E+01, 3.2113E+01, 7.0780E+03, &
      8.6543967E-01, 5.31797E+01, 3.2012E+01, 7.0207E+03, &
      8.6976687E-01, 5.33559E+01, 3.1910E+01, 6.9638E+03, &
      8.7411570E-01, 5.35297E+01, 3.1809E+01, 6.9073E+03, &
      8.7848628E-01, 5.37013E+01, 3.1708E+01, 6.8512E+03, &
      8.8287871E-01, 5.38706E+01, 3.1607E+01, 6.7954E+03, &
      8.8729310E-01, 5.40377E+01, 3.1506E+01, 6.7401E+03, &
      8.9172957E-01, 5.42024E+01, 3.1405E+01, 6.6852E+03, &
      8.9618822E-01, 5.43645E+01, 3.1304E+01, 6.6307E+03, &
      9.0066916E-01, 5.45238E+01, 3.1204E+01, 6.5766E+03, &
      9.0517250E-01, 5.46795E+01, 3.1103E+01, 6.5230E+03, &
      9.0969837E-01, 5.48299E+01, 3.0998E+01, 6.4687E+03, &
      9.1424686E-01, 5.49710E+01, 3.0888E+01, 6.4138E+03, &
      9.1881809E-01, 5.50973E+01, 3.0778E+01, 6.3592E+03, &
      9.2341218E-01, 5.51925E+01, 3.0668E+01, 6.3050E+03, &
      9.2741143E-01, 5.51769E+01, 3.0572E+01, 6.2584E+03, &
      9.2802924E-01, 5.51352E+01, 3.0558E+01, 6.2512E+03, &
      9.3058861E-01, 5.52949E+01, 3.0911E+01, 6.3059E+03, &
      9.3266939E-01, 5.54822E+01, 3.0862E+01, 6.2821E+03, &
      9.3733274E-01, 5.57633E+01, 3.0754E+01, 6.2291E+03, &
      9.4201940E-01, 5.59960E+01, 3.0646E+01, 6.1765E+03, &
      9.4672950E-01, 5.62118E+01, 3.0539E+01, 6.1243E+03, &
      9.5146315E-01, 5.64197E+01, 3.0432E+01, 6.0725E+03, &
      9.5622046E-01, 5.66239E+01, 3.0325E+01, 6.0212E+03, &
      9.6100156E-01, 5.68263E+01, 3.0214E+01, 5.9695E+03, &
      9.6580657E-01, 5.70283E+01, 3.0102E+01, 5.9179E+03, &
      9.7063560E-01, 5.72318E+01, 2.9991E+01, 5.8667E+03, &
      9.7548878E-01, 5.74383E+01, 2.9879E+01, 5.8159E+03, &
      9.8036623E-01, 5.76500E+01, 2.9768E+01, 5.7656E+03, &
      9.8526806E-01, 5.78693E+01, 2.9657E+01, 5.7157E+03, &
      9.9019440E-01, 5.80996E+01, 2.9547E+01, 5.6662E+03, &
      9.9514537E-01, 5.83452E+01, 2.9438E+01, 5.6172E+03, &
      1.0001211E+00, 5.86103E+01, 2.9327E+01, 5.5683E+03, &
      1.0051217E+00, 5.87658E+01, 2.9144E+01, 5.5063E+03, &
      1.0101473E+00, 5.89081E+01, 2.8963E+01, 5.4450E+03, &
      1.0151980E+00, 5.90411E+01, 2.8784E+01, 5.3845E+03, &
      1.0202740E+00, 5.91666E+01, 2.8605E+01, 5.3247E+03, &
      1.0253754E+00, 5.92858E+01, 2.8429E+01, 5.2656E+03, &
      1.0305023E+00, 5.93992E+01, 2.8253E+01, 5.2072E+03, &
      1.0356548E+00, 5.95072E+01, 2.8079E+01, 5.1495E+03, &
      1.0408331E+00, 5.96099E+01, 2.7906E+01, 5.0925E+03, &
      1.0460372E+00, 5.97075E+01, 2.7735E+01, 5.0362E+03, &
      1.0512674E+00, 5.97997E+01, 2.7565E+01, 4.9806E+03, &
      1.0565238E+00, 5.98863E+01, 2.7397E+01, 4.9256E+03, &
      1.0618064E+00, 5.99668E+01, 2.7230E+01, 4.8714E+03, &
      1.0671154E+00, 6.00404E+01, 2.7064E+01, 4.8178E+03, &
      1.0724510E+00, 6.01053E+01, 2.6899E+01, 4.7648E+03, &
      1.0778132E+00, 6.01590E+01, 2.6736E+01, 4.7125E+03, &
      1.0832023E+00, 6.01956E+01, 2.6574E+01, 4.6608E+03, &
      1.0886183E+00, 6.02002E+01, 2.6414E+01, 4.6097E+03, &
      1.0940614E+00, 6.01063E+01, 2.6255E+01, 4.5592E+03, &
      1.0956507E+00, 5.99958E+01, 2.6208E+01, 4.5447E+03, &
      1.0983493E+00, 6.00526E+01, 2.6680E+01, 4.6149E+03, &
      1.0995317E+00, 6.01813E+01, 2.6646E+01, 4.6041E+03, &
      1.1050294E+00, 6.04681E+01, 2.6489E+01, 4.5544E+03, &
      1.1105545E+00, 6.06426E+01, 2.6334E+01, 4.5053E+03, &
      1.1161073E+00, 6.07835E+01, 2.6180E+01, 4.4568E+03, &
      1.1216878E+00, 6.09072E+01, 2.6027E+01, 4.4088E+03, &
      1.1272963E+00, 6.10200E+01, 2.5875E+01, 4.3614E+03, &
      1.1329328E+00, 6.11249E+01, 2.5724E+01, 4.3146E+03, &
      1.1385974E+00, 6.12238E+01, 2.5575E+01, 4.2684E+03, &
      1.1442904E+00, 6.13177E+01, 2.5427E+01, 4.2227E+03, &
      1.1500119E+00, 6.14074E+01, 2.5280E+01, 4.1775E+03, &
      1.1557619E+00, 6.14935E+01, 2.5134E+01, 4.1329E+03, &
      1.1615407E+00, 6.15764E+01, 2.4990E+01, 4.0888E+03, &
      1.1673484E+00, 6.16564E+01, 2.4847E+01, 4.0453E+03, &
      1.1731852E+00, 6.17338E+01, 2.4704E+01, 4.0023E+03, &
      1.1790511E+00, 6.18087E+01, 2.4563E+01, 3.9597E+03, &
      1.1849464E+00, 6.18813E+01, 2.4423E+01, 3.9177E+03, &
      1.1908711E+00, 6.19518E+01, 2.4285E+01, 3.8762E+03, &
      1.1968254E+00, 6.20202E+01, 2.4147E+01, 3.8352E+03, &
      1.2028096E+00, 6.20867E+01, 2.4010E+01, 3.7947E+03, &
      1.2088236E+00, 6.21515E+01, 2.3875E+01, 3.7546E+03, &
      1.2148677E+00, 6.22144E+01, 2.3740E+01, 3.7150E+03, &
      1.2209421E+00, 6.22758E+01, 2.3607E+01, 3.6759E+03, &
      1.2270468E+00, 6.23355E+01, 2.3475E+01, 3.6373E+03, &
      1.2331820E+00, 6.23937E+01, 2.3344E+01, 3.5991E+03, &
      1.2393479E+00, 6.24505E+01, 2.3213E+01, 3.5613E+03, &
      1.2455447E+00, 6.25058E+01, 2.3084E+01, 3.5240E+03, &
      1.2517724E+00, 6.25598E+01, 2.2956E+01, 3.4872E+03, &
      1.2580312E+00, 6.26125E+01, 2.2829E+01, 3.4507E+03, &
      1.2643214E+00, 6.26640E+01, 2.2703E+01, 3.4147E+03, &
      1.2706430E+00, 6.27142E+01, 2.2578E+01, 3.3792E+03, &
      1.2769962E+00, 6.27634E+01, 2.2454E+01, 3.3440E+03, &
      1.2833812E+00, 6.28114E+01, 2.2331E+01, 3.3092E+03, &
      1.2897981E+00, 6.28584E+01, 2.2208E+01, 3.2749E+03, &
      1.2962471E+00, 6.29044E+01, 2.2087E+01, 3.2410E+03, &
      1.3027283E+00, 6.29493E+01, 2.1965E+01, 3.2071E+03, &
      1.3092420E+00, 6.29927E+01, 2.1844E+01, 3.1736E+03, &
      1.3157882E+00, 6.30349E+01, 2.1723E+01, 3.1405E+03, &
      1.3223671E+00, 6.30758E+01, 2.1603E+01, 3.1078E+03, &
      1.3289790E+00, 6.31156E+01, 2.1484E+01, 3.0754E+03, &
      1.3356239E+00, 6.31542E+01, 2.1366E+01, 3.0435E+03, &
      1.3423020E+00, 6.31916E+01, 2.1249E+01, 3.0119E+03, &
      1.3490135E+00, 6.32280E+01, 2.1133E+01, 2.9806E+03, &
      1.3557586E+00, 6.32633E+01, 2.1018E+01, 2.9497E+03, &
      1.3625374E+00, 6.32976E+01, 2.0903E+01, 2.9192E+03, &
      1.3693500E+00, 6.33309E+01, 2.0790E+01, 2.8890E+03, &
      1.3761968E+00, 6.33632E+01, 2.0677E+01, 2.8592E+03, &
      1.3830778E+00, 6.33945E+01, 2.0565E+01, 2.8297E+03, &
      1.3899932E+00, 6.34249E+01, 2.0454E+01, 2.8006E+03, &
      1.3969431E+00, 6.34544E+01, 2.0344E+01, 2.7718E+03, &
      1.4039278E+00, 6.34830E+01, 2.0235E+01, 2.7433E+03, &
      1.4109475E+00, 6.35108E+01, 2.0126E+01, 2.7152E+03, &
      1.4180022E+00, 6.35377E+01, 2.0019E+01, 2.6873E+03, &
      1.4250922E+00, 6.35638E+01, 1.9912E+01, 2.6598E+03, &
      1.4322177E+00, 6.35891E+01, 1.9805E+01, 2.6326E+03, &
      1.4393788E+00, 6.36134E+01, 1.9699E+01, 2.6056E+03, &
      1.4465757E+00, 6.36368E+01, 1.9594E+01, 2.5789E+03, &
      1.4538086E+00, 6.36593E+01, 1.9490E+01, 2.5526E+03, &
      1.4610776E+00, 6.36810E+01, 1.9387E+01, 2.5265E+03, &
      1.4683830E+00, 6.37018E+01, 1.9284E+01, 2.5008E+03, &
      1.4757249E+00, 6.37218E+01, 1.9182E+01, 2.4753E+03, &
      1.4831035E+00, 6.37410E+01, 1.9081E+01, 2.4501E+03, &
      1.4905190E+00, 6.37594E+01, 1.8981E+01, 2.4252E+03, &
      1.4979716E+00, 6.37770E+01, 1.8881E+01, 2.4006E+03, &
      1.5054615E+00, 6.37939E+01, 1.8782E+01, 2.3763E+03, &
      1.5129888E+00, 6.38099E+01, 1.8684E+01, 2.3522E+03, &
      1.5205537E+00, 6.38253E+01, 1.8586E+01, 2.3285E+03, &
      1.5281565E+00, 6.38399E+01, 1.8490E+01, 2.3049E+03, &
      1.5357973E+00, 6.38538E+01, 1.8394E+01, 2.2817E+03, &
      1.5434763E+00, 6.38670E+01, 1.8298E+01, 2.2587E+03, &
      1.5511937E+00, 6.38795E+01, 1.8204E+01, 2.2360E+03, &
      1.5589496E+00, 6.38914E+01, 1.8110E+01, 2.2135E+03, &
      1.5667444E+00, 6.39026E+01, 1.8017E+01, 2.1913E+03, &
      1.5745781E+00, 6.39131E+01, 1.7923E+01, 2.1692E+03, &
      1.5824510E+00, 6.39227E+01, 1.7830E+01, 2.1473E+03, &
      1.5903633E+00, 6.39314E+01, 1.7738E+01, 2.1257E+03, &
      1.5983151E+00, 6.39394E+01, 1.7646E+01, 2.1043E+03, &
      1.6063066E+00, 6.39465E+01, 1.7555E+01, 2.0832E+03, &
      1.6143382E+00, 6.39794E+01, 1.7465E+01, 2.0623E+03, &
      1.6224099E+00, 6.39851E+01, 1.7375E+01, 2.0416E+03, &
      1.6305219E+00, 6.39900E+01, 1.7286E+01, 2.0211E+03, &
      1.6386745E+00, 6.39942E+01, 1.7197E+01, 2.0009E+03, &
      1.6468679E+00, 6.39976E+01, 1.7110E+01, 1.9809E+03, &
      1.6551022E+00, 6.40002E+01, 1.7022E+01, 1.9611E+03, &
      1.6633777E+00, 6.40021E+01, 1.6936E+01, 1.9415E+03, &
      1.6716946E+00, 6.40032E+01, 1.6850E+01, 1.9222E+03, &
      1.6800531E+00, 6.40036E+01, 1.6764E+01, 1.9031E+03, &
      1.6884534E+00, 6.40032E+01, 1.6680E+01, 1.8841E+03, &
      1.6968956E+00, 6.40020E+01, 1.6595E+01, 1.8654E+03, &
      1.7053801E+00, 6.40179E+01, 1.6512E+01, 1.8469E+03, &
      1.7139070E+00, 6.40154E+01, 1.6429E+01, 1.8286E+03, &
      1.7224766E+00, 6.40121E+01, 1.6346E+01, 1.8105E+03, &
      1.7310889E+00, 6.40081E+01, 1.6265E+01, 1.7926E+03, &
      1.7397444E+00, 6.40033E+01, 1.6183E+01, 1.7749E+03, &
      1.7484431E+00, 6.39977E+01, 1.6103E+01, 1.7574E+03, &
      1.7571853E+00, 6.39914E+01, 1.6023E+01, 1.7401E+03, &
      1.7659712E+00, 6.39843E+01, 1.5943E+01, 1.7230E+03, &
      1.7748011E+00, 6.39764E+01, 1.5864E+01, 1.7060E+03, &
      1.7836751E+00, 6.39677E+01, 1.5786E+01, 1.6893E+03, &
      1.7925935E+00, 6.39583E+01, 1.5708E+01, 1.6727E+03, &
      1.8015565E+00, 6.39481E+01, 1.5631E+01, 1.6563E+03, &
      1.8105642E+00, 6.39370E+01, 1.5554E+01, 1.6401E+03, &
      1.8196171E+00, 6.39252E+01, 1.5478E+01, 1.6240E+03, &
      1.8287151E+00, 6.39126E+01, 1.5402E+01, 1.6082E+03, &
      1.8378587E+00, 6.38991E+01, 1.5327E+01, 1.5925E+03, &
      1.8470480E+00, 6.38849E+01, 1.5252E+01, 1.5769E+03, &
      1.8562833E+00, 6.38698E+01, 1.5178E+01, 1.5616E+03, &
      1.8655647E+00, 6.38538E+01, 1.5105E+01, 1.5464E+03, &
      1.8748925E+00, 6.38370E+01, 1.5031E+01, 1.5314E+03, &
      1.8842670E+00, 6.38194E+01, 1.4959E+01, 1.5165E+03, &
      1.8936883E+00, 6.38009E+01, 1.4887E+01, 1.5018E+03, &
      1.9031567E+00, 6.37815E+01, 1.4815E+01, 1.4873E+03, &
      1.9126725E+00, 6.37612E+01, 1.4744E+01, 1.4729E+03, &
      1.9222359E+00, 6.37399E+01, 1.4674E+01, 1.4587E+03, &
      1.9318471E+00, 6.37178E+01, 1.4604E+01, 1.4446E+03, &
      1.9415063E+00, 6.36948E+01, 1.4534E+01, 1.4307E+03, &
      1.9512138E+00, 6.36708E+01, 1.4465E+01, 1.4169E+03, &
      1.9609699E+00, 6.36458E+01, 1.4396E+01, 1.4033E+03, &
      1.9707747E+00, 6.36198E+01, 1.4328E+01, 1.3898E+03, &
      1.9806286E+00, 6.35929E+01, 1.4261E+01, 1.3765E+03, &
      1.9905318E+00, 6.35649E+01, 1.4193E+01, 1.3633E+03, &
      2.0004844E+00, 6.35360E+01, 1.4127E+01, 1.3502E+03, &
      2.0104868E+00, 6.35059E+01, 1.4060E+01, 1.3373E+03, &
      2.0205393E+00, 6.34748E+01, 1.3995E+01, 1.3246E+03, &
      2.0306420E+00, 6.34426E+01, 1.3929E+01, 1.3119E+03, &
      2.0407952E+00, 6.34093E+01, 1.3864E+01, 1.2994E+03, &
      2.0509992E+00, 6.33749E+01, 1.3800E+01, 1.2871E+03, &
      2.0612542E+00, 6.33393E+01, 1.3736E+01, 1.2748E+03, &
      2.0715604E+00, 6.33025E+01, 1.3672E+01, 1.2627E+03, &
      2.0819182E+00, 6.32646E+01, 1.3609E+01, 1.2508E+03, &
      2.0923278E+00, 6.32254E+01, 1.3547E+01, 1.2389E+03, &
      2.1027895E+00, 6.31849E+01, 1.3484E+01, 1.2272E+03, &
      2.1133034E+00, 6.31432E+01, 1.3422E+01, 1.2156E+03, &
      2.1238699E+00, 6.31002E+01, 1.3361E+01, 1.2041E+03, &
      2.1344893E+00, 6.30558E+01, 1.3300E+01, 1.1928E+03, &
      2.1451617E+00, 6.30100E+01, 1.3240E+01, 1.1815E+03, &
      2.1558875E+00, 6.29627E+01, 1.3173E+01, 1.1699E+03, &
      2.1666670E+00, 6.29132E+01, 1.3105E+01, 1.1581E+03, &
      2.1775003E+00, 6.28616E+01, 1.3037E+01, 1.1465E+03, &
      2.1883878E+00, 6.28078E+01, 1.2970E+01, 1.1350E+03, &
      2.1993297E+00, 6.27472E+01, 1.2902E+01, 1.1237E+03, &
      2.2103264E+00, 6.26890E+01, 1.2836E+01, 1.1124E+03, &
      2.2213780E+00, 6.26284E+01, 1.2769E+01, 1.1012E+03, &
      2.2324849E+00, 6.25654E+01, 1.2703E+01, 1.0902E+03, &
      2.2436473E+00, 6.25000E+01, 1.2637E+01, 1.0793E+03, &
      2.2548656E+00, 6.24319E+01, 1.2572E+01, 1.0685E+03, &
      2.2661399E+00, 6.23613E+01, 1.2507E+01, 1.0578E+03, &
      2.2774706E+00, 6.22877E+01, 1.2437E+01, 1.0467E+03, &
      2.2888579E+00, 6.22108E+01, 1.2367E+01, 1.0358E+03, &
      2.3003022E+00, 6.21306E+01, 1.2298E+01, 1.0250E+03, &
      2.3118037E+00, 6.20527E+01, 1.2229E+01, 1.0143E+03, &
      2.3233628E+00, 6.19656E+01, 1.2160E+01, 1.0037E+03, &
      2.3349796E+00, 6.18747E+01, 1.2093E+01, 9.9323E+02, &
      2.3466545E+00, 6.17801E+01, 1.2025E+01, 9.8289E+02, &
      2.3583878E+00, 6.16816E+01, 1.1958E+01, 9.7267E+02, &
      2.3701797E+00, 6.15789E+01, 1.1892E+01, 9.6256E+02, &
      2.3820306E+00, 6.14720E+01, 1.1826E+01, 9.5257E+02, &
      2.3939407E+00, 6.13607E+01, 1.1760E+01, 9.4269E+02, &
      2.4059104E+00, 6.12447E+01, 1.1695E+01, 9.3292E+02, &
      2.4179400E+00, 6.11238E+01, 1.1630E+01, 9.2326E+02, &
      2.4300297E+00, 6.09978E+01, 1.1566E+01, 9.1370E+02, &
      2.4421798E+00, 6.08664E+01, 1.1502E+01, 9.0426E+02, &
      2.4543907E+00, 6.07293E+01, 1.1439E+01, 8.9492E+02, &
      2.4666627E+00, 6.05863E+01, 1.1376E+01, 8.8568E+02, &
      2.4789960E+00, 6.04369E+01, 1.1314E+01, 8.7655E+02, &
      2.4913910E+00, 6.02808E+01, 1.1252E+01, 8.6752E+02, &
      2.5038479E+00, 6.01175E+01, 1.1190E+01, 8.5859E+02, &
      2.5163672E+00, 5.99466E+01, 1.1129E+01, 8.4976E+02, &
      2.5289490E+00, 5.97675E+01, 1.1068E+01, 8.4102E+02, &
      2.5415938E+00, 5.95797E+01, 1.1008E+01, 8.3239E+02, &
      2.5543017E+00, 5.93825E+01, 1.0948E+01, 8.2385E+02, &
      2.5670732E+00, 5.91752E+01, 1.0889E+01, 8.1540E+02, &
      2.5799086E+00, 5.89569E+01, 1.0829E+01, 8.0705E+02, &
      2.5928082E+00, 5.87268E+01, 1.0771E+01, 7.9880E+02, &
      2.6057722E+00, 5.84837E+01, 1.0713E+01, 7.9063E+02, &
      2.6188011E+00, 5.82264E+01, 1.0655E+01, 7.8255E+02, &
      2.6318951E+00, 5.79537E+01, 1.0597E+01, 7.7457E+02, &
      2.6450545E+00, 5.76637E+01, 1.0540E+01, 7.6667E+02, &
      2.6582798E+00, 5.73547E+01, 1.0483E+01, 7.5886E+02, &
      2.6715712E+00, 5.70244E+01, 1.0427E+01, 7.5113E+02, &
      2.6849291E+00, 5.66701E+01, 1.0371E+01, 7.4349E+02, &
      2.6983537E+00, 5.62888E+01, 1.0316E+01, 7.3594E+02, &
      2.7118455E+00, 5.58765E+01, 1.0261E+01, 7.2847E+02, &
      2.7254047E+00, 5.54284E+01, 1.0206E+01, 7.2108E+02, &
      2.7390317E+00, 5.49385E+01, 1.0151E+01, 7.1377E+02, &
      2.7527269E+00, 5.43992E+01, 1.0097E+01, 7.0654E+02, &
      2.7664905E+00, 5.38004E+01, 1.0044E+01, 6.9939E+02, &
      2.7803230E+00, 5.31286E+01, 9.9905E+00, 6.9232E+02, &
      2.7942246E+00, 5.23638E+01, 9.9375E+00, 6.8533E+02, &
      2.8081957E+00, 5.14805E+01, 9.8850E+00, 6.7841E+02, &
      2.8222367E+00, 5.04349E+01, 9.8328E+00, 6.7157E+02, &
      2.8363479E+00, 4.91554E+01, 9.7809E+00, 6.6481E+02, &
      2.8505296E+00, 4.75069E+01, 9.7294E+00, 6.5812E+02, &
      2.8647823E+00, 4.51815E+01, 9.6781E+00, 6.5149E+02, &
      2.8791062E+00, 4.11545E+01, 9.6271E+00, 6.4493E+02, &
      2.8915873E+00, 2.59768E+01, 9.5831E+00, 6.3930E+02, &
      2.8935017E+00, 2.73618E+01, 2.6619E+01, 1.7549E+03, &
      2.9079692E+00, 4.16766E+01, 2.6336E+01, 1.7277E+03, &
      2.9225091E+00, 4.50424E+01, 2.6056E+01, 1.7010E+03, &
      2.9371216E+00, 4.68582E+01, 2.5780E+01, 1.6748E+03, &
      2.9518072E+00, 4.79339E+01, 2.5507E+01, 1.6490E+03, &
      2.9665662E+00, 4.84939E+01, 2.5238E+01, 1.6236E+03, &
      2.9813991E+00, 4.85621E+01, 2.4972E+01, 1.5987E+03, &
      2.9963061E+00, 4.79571E+01, 2.4709E+01, 1.5741E+03, &
      3.0112876E+00, 4.58173E+01, 2.4433E+01, 1.5490E+03, &
      3.0205816E+00, 3.80654E+01, 2.4261E+01, 1.5334E+03, &
      3.0224186E+00, 3.79935E+01, 3.5050E+01, 2.2091E+03, &
      3.0263440E+00, 4.39426E+01, 3.4954E+01, 2.2003E+03, &
      3.0414758E+00, 4.95564E+01, 3.4602E+01, 2.1674E+03, &
      3.0566831E+00, 5.21432E+01, 3.4275E+01, 2.1364E+03, &
      3.0719666E+00, 5.39550E+01, 3.3954E+01, 2.1060E+03, &
      3.0873264E+00, 5.53834E+01, 3.3637E+01, 2.0761E+03, &
      3.1027630E+00, 5.65745E+01, 3.3327E+01, 2.0468E+03, &
      3.1182768E+00, 5.76004E+01, 3.3022E+01, 2.0182E+03, &
      3.1338682E+00, 5.85025E+01, 3.2722E+01, 1.9900E+03, &
      3.1495376E+00, 5.93073E+01, 3.2428E+01, 1.9625E+03, &
      3.1652853E+00, 6.00326E+01, 3.2138E+01, 1.9354E+03, &
      3.1811117E+00, 6.06914E+01, 3.1854E+01, 1.9089E+03, &
      3.1970172E+00, 6.12932E+01, 3.1574E+01, 1.8828E+03, &
      3.2130023E+00, 6.18451E+01, 3.1299E+01, 1.8573E+03, &
      3.2290673E+00, 6.23529E+01, 3.1029E+01, 1.8322E+03, &
      3.2452127E+00, 6.28209E+01, 3.0762E+01, 1.8075E+03, &
      3.2614387E+00, 6.32527E+01, 3.0499E+01, 1.7833E+03, &
      3.2777459E+00, 6.36508E+01, 3.0241E+01, 1.7595E+03, &
      3.2941347E+00, 6.40174E+01, 2.9986E+01, 1.7361E+03, &
      3.3106053E+00, 6.43539E+01, 2.9734E+01, 1.7131E+03, &
      3.3271584E+00, 6.46613E+01, 2.9486E+01, 1.6905E+03, &
      3.3437941E+00, 6.49400E+01, 2.9242E+01, 1.6683E+03, &
      3.3605131E+00, 6.51898E+01, 2.9001E+01, 1.6464E+03, &
      3.3773157E+00, 6.54097E+01, 2.8762E+01, 1.6249E+03, &
      3.3942023E+00, 6.55976E+01, 2.8527E+01, 1.6037E+03, &
      3.4111733E+00, 6.57504E+01, 2.8295E+01, 1.5828E+03, &
      3.4282291E+00, 6.58626E+01, 2.8065E+01, 1.5623E+03, &
      3.4453703E+00, 6.59253E+01, 2.7839E+01, 1.5421E+03, &
      3.4625971E+00, 6.59236E+01, 2.7614E+01, 1.5222E+03, &
      3.4799101E+00, 6.58294E+01, 2.7393E+01, 1.5026E+03, &
      3.4973097E+00, 6.55830E+01, 2.7174E+01, 1.4832E+03, &
      3.5147962E+00, 6.50178E+01, 2.6957E+01, 1.4642E+03, &
      3.5323702E+00, 6.31454E+01, 2.6742E+01, 1.4454E+03, &
      3.5337899E+00, 6.27247E+01, 2.6725E+01, 1.4439E+03, &
      3.5422103E+00, 6.27987E+01, 3.1431E+01, 1.6923E+03, &
      3.5500321E+00, 6.45407E+01, 3.1315E+01, 1.6824E+03, &
      3.5677822E+00, 6.62050E+01, 3.1054E+01, 1.6602E+03, &
      3.5856211E+00, 6.71795E+01, 3.0796E+01, 1.6384E+03, &
      3.6035492E+00, 6.79089E+01, 3.0541E+01, 1.6168E+03, &
      3.6215670E+00, 6.85076E+01, 3.0288E+01, 1.5955E+03, &
      3.6396748E+00, 6.90227E+01, 3.0037E+01, 1.5746E+03, &
      3.6578732E+00, 6.94782E+01, 2.9788E+01, 1.5539E+03, &
      3.6761626E+00, 6.98877E+01, 2.9542E+01, 1.5335E+03, &
      3.6945434E+00, 7.02597E+01, 2.9298E+01, 1.5134E+03, &
      3.7130161E+00, 7.05984E+01, 2.9057E+01, 1.4936E+03, &
      3.7315812E+00, 7.09125E+01, 2.8834E+01, 1.4748E+03, &
      3.7502391E+00, 7.12108E+01, 2.8613E+01, 1.4564E+03, &
      3.7689903E+00, 7.14936E+01, 2.8390E+01, 1.4380E+03, &
      3.7878352E+00, 7.17594E+01, 2.8164E+01, 1.4195E+03, &
      3.8067744E+00, 7.20095E+01, 2.7941E+01, 1.4014E+03, &
      3.8258083E+00, 7.22452E+01, 2.7720E+01, 1.3835E+03, &
      3.8449373E+00, 7.24675E+01, 2.7502E+01, 1.3659E+03, &
      3.8641620E+00, 7.26771E+01, 2.7286E+01, 1.3485E+03, &
      3.8834828E+00, 7.28747E+01, 2.7073E+01, 1.3314E+03, &
      3.9029002E+00, 7.30606E+01, 2.6862E+01, 1.3146E+03, &
      3.9224147E+00, 7.32351E+01, 2.6653E+01, 1.2980E+03, &
      3.9420268E+00, 7.33978E+01, 2.6445E+01, 1.2815E+03, &
      3.9617369E+00, 7.35480E+01, 2.6237E+01, 1.2653E+03, &
      3.9815456E+00, 7.36851E+01, 2.6032E+01, 1.2492E+03, &
      4.0014533E+00, 7.38082E+01, 2.5829E+01, 1.2334E+03, &
      4.0214606E+00, 7.39157E+01, 2.5627E+01, 1.2178E+03, &
      4.0415679E+00, 7.40049E+01, 2.5427E+01, 1.2024E+03, &
      4.0617757E+00, 7.40715E+01, 2.5229E+01, 1.1872E+03, &
      4.0820846E+00, 7.41077E+01, 2.5033E+01, 1.1722E+03, &
      4.1024950E+00, 7.40981E+01, 2.4838E+01, 1.1574E+03, &
      4.1230075E+00, 7.40053E+01, 2.4646E+01, 1.1428E+03, &
      4.1436226E+00, 7.36876E+01, 2.4454E+01, 1.1284E+03, &
      4.1532189E+00, 7.32146E+01, 2.4366E+01, 1.1218E+03, &
      4.1643407E+00, 7.32317E+01, 2.6017E+01, 1.1939E+03, &
      4.1647809E+00, 7.32794E+01, 2.6012E+01, 1.1936E+03, &
      4.1851624E+00, 7.42703E+01, 2.5805E+01, 1.1784E+03, &
      4.2060882E+00, 7.47351E+01, 2.5595E+01, 1.1631E+03, &
      4.2271186E+00, 7.50676E+01, 2.5388E+01, 1.1481E+03, &
      4.2482542E+00, 7.53355E+01, 2.5182E+01, 1.1332E+03, &
      4.2694955E+00, 7.55626E+01, 2.4978E+01, 1.1185E+03, &
      4.2908430E+00, 7.57598E+01, 2.4776E+01, 1.1041E+03, &
      4.3122972E+00, 7.59326E+01, 2.4576E+01, 1.0898E+03, &
      4.3338587E+00, 7.60833E+01, 2.4378E+01, 1.0757E+03, &
      4.3555280E+00, 7.62121E+01, 2.4181E+01, 1.0619E+03, &
      4.3773056E+00, 7.63179E+01, 2.3993E+01, 1.0485E+03, &
      4.3991921E+00, 7.64025E+01, 2.3811E+01, 1.0354E+03, &
      4.4211881E+00, 7.64572E+01, 2.3631E+01, 1.0226E+03, &
      4.4432940E+00, 7.64600E+01, 2.3454E+01, 1.0099E+03, &
      4.4655105E+00, 7.63275E+01, 2.3278E+01, 9.9747E+02, &
      4.4729462E+00, 7.61801E+01, 2.3220E+01, 9.9335E+02, &
      4.4878381E+00, 7.61223E+01, 2.4123E+01, 1.0282E+03, &
      4.4910535E+00, 7.62843E+01, 2.4098E+01, 1.0264E+03, &
      4.5102772E+00, 7.67693E+01, 2.3950E+01, 1.0159E+03, &
      4.5328286E+00, 7.70907E+01, 2.3780E+01, 1.0037E+03, &
      4.5554928E+00, 7.73380E+01, 2.3611E+01, 9.9171E+02, &
      4.5782702E+00, 7.75502E+01, 2.3444E+01, 9.7988E+02, &
      4.6011616E+00, 7.77414E+01, 2.3278E+01, 9.6820E+02, &
      4.6241674E+00, 7.79167E+01, 2.3109E+01, 9.5646E+02, &
      4.6472882E+00, 7.80797E+01, 2.2941E+01, 9.4487E+02, &
      4.6705247E+00, 7.82329E+01, 2.2774E+01, 9.3344E+02, &
      4.6938773E+00, 7.83780E+01, 2.2610E+01, 9.2216E+02, &
      4.7173467E+00, 7.85163E+01, 2.2446E+01, 9.1102E+02, &
      4.7409334E+00, 7.86483E+01, 2.2284E+01, 9.0002E+02, &
      4.7646381E+00, 7.87748E+01, 2.2123E+01, 8.8915E+02, &
      4.7884613E+00, 7.88963E+01, 2.1963E+01, 8.7843E+02, &
      4.8124036E+00, 7.90133E+01, 2.1804E+01, 8.6784E+02, &
      4.8364656E+00, 7.91262E+01, 2.1647E+01, 8.5739E+02, &
      4.8606479E+00, 7.92353E+01, 2.1492E+01, 8.4708E+02, &
      4.8849512E+00, 7.93409E+01, 2.1338E+01, 8.3690E+02, &
      4.9093759E+00, 7.94432E+01, 2.1185E+01, 8.2685E+02, &
      4.9339228E+00, 7.95424E+01, 2.1033E+01, 8.1693E+02, &
      4.9585924E+00, 7.96387E+01, 2.0883E+01, 8.0714E+02, &
      4.9833854E+00, 7.97323E+01, 2.0734E+01, 7.9748E+02, &
      5.0083023E+00, 7.98233E+01, 2.0586E+01, 7.8794E+02, &
      5.0333438E+00, 7.99118E+01, 2.0440E+01, 7.7852E+02, &
      5.0585105E+00, 7.99980E+01, 2.0295E+01, 7.6922E+02, &
      5.0838031E+00, 8.00821E+01, 2.0151E+01, 7.6004E+02, &
      5.1092221E+00, 8.01640E+01, 2.0008E+01, 7.5098E+02, &
      5.1347682E+00, 8.02439E+01, 1.9866E+01, 7.4203E+02, &
      5.1604421E+00, 8.03219E+01, 1.9726E+01, 7.3320E+02, &
      5.1862443E+00, 8.03981E+01, 1.9586E+01, 7.2448E+02, &
      5.2121755E+00, 8.04726E+01, 1.9448E+01, 7.1586E+02, &
      5.2382364E+00, 8.05454E+01, 1.9311E+01, 7.0736E+02, &
      5.2644276E+00, 8.06167E+01, 1.9175E+01, 6.9896E+02, &
      5.2907497E+00, 8.06866E+01, 1.9040E+01, 6.9067E+02, &
      5.3172034E+00, 8.07551E+01, 1.8906E+01, 6.8248E+02, &
      5.3437895E+00, 8.08225E+01, 1.8774E+01, 6.7439E+02, &
      5.3705084E+00, 8.08887E+01, 1.8642E+01, 6.6640E+02, &
      5.3973609E+00, 8.09540E+01, 1.8511E+01, 6.5851E+02, &
      5.4243477E+00, 8.10181E+01, 1.8379E+01, 6.5064E+02, &
      5.4514695E+00, 8.10803E+01, 1.8247E+01, 6.4281E+02, &
      5.4787268E+00, 8.11406E+01, 1.8115E+01, 6.3507E+02, &
      5.5061205E+00, 8.11993E+01, 1.7984E+01, 6.2743E+02, &
      5.5336511E+00, 8.12564E+01, 1.7855E+01, 6.1989E+02, &
      5.5613193E+00, 8.13120E+01, 1.7726E+01, 6.1243E+02, &
      5.5891259E+00, 8.13662E+01, 1.7599E+01, 6.0507E+02, &
      5.6170716E+00, 8.14191E+01, 1.7472E+01, 5.9780E+02, &
      5.6451569E+00, 8.14706E+01, 1.7346E+01, 5.9062E+02, &
      5.6733827E+00, 8.15210E+01, 1.7221E+01, 5.8353E+02, &
      5.7017496E+00, 8.15701E+01, 1.7097E+01, 5.7652E+02, &
      5.7302584E+00, 8.16182E+01, 1.6975E+01, 5.6960E+02, &
      5.7589096E+00, 8.16652E+01, 1.6853E+01, 5.6276E+02, &
      5.7877042E+00, 8.18721E+01, 1.6731E+01, 5.5600E+02, &
      5.8166427E+00, 8.19171E+01, 1.6607E+01, 5.4919E+02, &
      5.8457259E+00, 8.19606E+01, 1.6483E+01, 5.4245E+02, &
      5.8749546E+00, 8.20025E+01, 1.6360E+01, 5.3580E+02, &
      5.9043293E+00, 8.20431E+01, 1.6238E+01, 5.2923E+02, &
      5.9338510E+00, 8.20823E+01, 1.6117E+01, 5.2274E+02, &
      5.9635202E+00, 8.21203E+01, 1.5997E+01, 5.1634E+02, &
      5.9933378E+00, 8.21570E+01, 1.5877E+01, 5.1002E+02, &
      6.0233045E+00, 8.21926E+01, 1.5759E+01, 5.0377E+02, &
      6.0534210E+00, 8.23382E+01, 1.5641E+01, 4.9758E+02, &
      6.0836882E+00, 8.23714E+01, 1.5523E+01, 4.9142E+02, &
      6.1141066E+00, 8.24033E+01, 1.5405E+01, 4.8534E+02, &
      6.1446771E+00, 8.24340E+01, 1.5288E+01, 4.7934E+02, &
      6.1754005E+00, 8.24635E+01, 1.5173E+01, 4.7342E+02, &
      6.2062775E+00, 8.24918E+01, 1.5058E+01, 4.6757E+02, &
      6.2373089E+00, 8.25191E+01, 1.4944E+01, 4.6179E+02, &
      6.2684954E+00, 8.25452E+01, 1.4831E+01, 4.5609E+02, &
      6.2998379E+00, 8.25704E+01, 1.4719E+01, 4.5047E+02, &
      6.3313371E+00, 8.25945E+01, 1.4608E+01, 4.4491E+02, &
      6.3629938E+00, 8.26177E+01, 1.4498E+01, 4.3943E+02, &
      6.3948088E+00, 8.26399E+01, 1.4389E+01, 4.3402E+02, &
      6.4267828E+00, 8.26613E+01, 1.4281E+01, 4.2868E+02, &
      6.4589167E+00, 8.26818E+01, 1.4173E+01, 4.2341E+02, &
      6.4912113E+00, 8.27014E+01, 1.4067E+01, 4.1820E+02, &
      6.5236674E+00, 8.27202E+01, 1.3961E+01, 4.1307E+02, &
      6.5562857E+00, 8.27382E+01, 1.3857E+01, 4.0800E+02, &
      6.5890671E+00, 8.27555E+01, 1.3753E+01, 4.0299E+02, &
      6.6220125E+00, 8.27719E+01, 1.3650E+01, 3.9805E+02, &
      6.6551225E+00, 8.27877E+01, 1.3548E+01, 3.9318E+02, &
      6.6883981E+00, 8.28027E+01, 1.3447E+01, 3.8837E+02, &
      6.7218401E+00, 8.28171E+01, 1.3347E+01, 3.8362E+02, &
      6.7554493E+00, 8.28308E+01, 1.3247E+01, 3.7893E+02, &
      6.7892266E+00, 8.28439E+01, 1.3149E+01, 3.7430E+02, &
      6.8231727E+00, 8.28563E+01, 1.3051E+01, 3.6973E+02, &
      6.8572886E+00, 8.28681E+01, 1.2954E+01, 3.6523E+02, &
      6.8915750E+00, 8.28794E+01, 1.2858E+01, 3.6078E+02, &
      6.9260329E+00, 8.28901E+01, 1.2763E+01, 3.5638E+02, &
      6.9606631E+00, 8.29002E+01, 1.2669E+01, 3.5205E+02, &
      6.9954664E+00, 8.29098E+01, 1.2575E+01, 3.4777E+02, &
      7.0304437E+00, 8.29189E+01, 1.2483E+01, 3.4355E+02, &
      7.0655959E+00, 8.29276E+01, 1.2391E+01, 3.3938E+02, &
      7.1009239E+00, 8.30258E+01, 1.2298E+01, 3.3524E+02, &
      7.1364285E+00, 8.30337E+01, 1.2206E+01, 3.3113E+02, &
      7.1721107E+00, 8.30410E+01, 1.2115E+01, 3.2707E+02, &
      7.2079712E+00, 8.30476E+01, 1.2024E+01, 3.2307E+02, &
      7.2440111E+00, 8.30536E+01, 1.1934E+01, 3.1912E+02, &
      7.2802311E+00, 8.30591E+01, 1.1845E+01, 3.1522E+02, &
      7.3166323E+00, 8.30641E+01, 1.1757E+01, 3.1137E+02, &
      7.3532155E+00, 8.30685E+01, 1.1669E+01, 3.0757E+02, &
      7.3899815E+00, 8.30724E+01, 1.1583E+01, 3.0383E+02, &
      7.4269314E+00, 8.30758E+01, 1.1497E+01, 3.0013E+02, &
      7.4640661E+00, 8.30787E+01, 1.1411E+01, 2.9647E+02, &
      7.5013864E+00, 8.30812E+01, 1.1327E+01, 2.9287E+02, &
      7.5388934E+00, 8.30832E+01, 1.1243E+01, 2.8931E+02, &
      7.5765878E+00, 8.30848E+01, 1.1160E+01, 2.8580E+02, &
      7.6144708E+00, 8.30860E+01, 1.1078E+01, 2.8234E+02, &
      7.6525431E+00, 8.30868E+01, 1.0996E+01, 2.7892E+02, &
      7.6908058E+00, 8.30872E+01, 1.0915E+01, 2.7554E+02, &
      7.7292599E+00, 8.30872E+01, 1.0835E+01, 2.7221E+02, &
      7.7679062E+00, 8.30869E+01, 1.0755E+01, 2.6892E+02, &
      7.8067457E+00, 8.30862E+01, 1.0677E+01, 2.6568E+02, &
      7.8457794E+00, 8.30852E+01, 1.0598E+01, 2.6247E+02, &
      7.8850083E+00, 8.30839E+01, 1.0521E+01, 2.5931E+02, &
      7.9244334E+00, 8.30823E+01, 1.0444E+01, 2.5619E+02, &
      7.9640555E+00, 8.30803E+01, 1.0368E+01, 2.5311E+02, &
      8.0038758E+00, 8.30781E+01, 1.0293E+01, 2.5007E+02, &
      8.0438952E+00, 8.30757E+01, 1.0218E+01, 2.4707E+02, &
      8.0841147E+00, 8.30729E+01, 1.0144E+01, 2.4410E+02, &
      8.1245352E+00, 8.30700E+01, 1.0070E+01, 2.4118E+02, &
      8.1651579E+00, 8.30668E+01, 9.9973E+00, 2.3829E+02, &
      8.2059837E+00, 8.30634E+01, 9.9251E+00, 2.3545E+02, &
      8.2470136E+00, 8.30598E+01, 9.8535E+00, 2.3263E+02, &
      8.2882487E+00, 8.30561E+01, 9.7826E+00, 2.2986E+02, &
      8.3296899E+00, 8.30911E+01, 9.7120E+00, 2.2711E+02, &
      8.3713384E+00, 8.30873E+01, 9.6414E+00, 2.2439E+02, &
      8.4131951E+00, 8.30833E+01, 9.5713E+00, 2.2170E+02, &
      8.4552610E+00, 8.30791E+01, 9.5019E+00, 2.1904E+02, &
      8.4975373E+00, 8.30748E+01, 9.4331E+00, 2.1642E+02, &
      8.5400250E+00, 8.30703E+01, 9.3648E+00, 2.1383E+02, &
      9.030794E+00, 8.33675E+01, 8.6075E+00, 1.8640E+02, &
      9.653919E+00, 8.31474E+01, 7.6974E+00, 1.5654E+02, &
      1.032004E+01, 8.28248E+01, 6.8927E+00, 1.3167E+02, &
      1.103212E+01, 8.24107E+01, 6.1770E+00, 1.1089E+02, &
      1.179334E+01, 8.18825E+01, 5.5434E+00, 9.3555E+01, &
      1.260708E+01, 8.11713E+01, 4.9819E+00, 7.9068E+01, &
      1.347697E+01, 8.00480E+01, 4.4697E+00, 6.6749E+01, &
      1.432701E+01, 7.74631E+01, 4.0501E+00, 5.7221E+01, &
      1.440688E+01, 7.68640E+01, 4.0143E+00, 5.6432E+01, &
      1.454630E+01, 7.48515E+01, 3.9528E+00, 5.5085E+01, &
      1.460478E+01, 7.17863E+01, 3.9274E+00, 5.4535E+01, &
      1.469250E+01, 7.48389E+01, 9.8855E+00, 1.3107E+02, &
      1.491179E+01, 7.75422E+01, 9.6491E+00, 1.2613E+02, &
      1.540095E+01, 7.93574E+01, 9.1516E+00, 1.1600E+02, &
      1.646362E+01, 8.02686E+01, 8.1592E+00, 9.7055E+01, &
      1.699036E+01, 7.97622E+01, 7.7257E+00, 8.9197E+01, &
      1.725041E+01, 7.85366E+01, 7.5246E+00, 8.5635E+01, &
      1.731976E+01, 7.69944E+01, 7.4723E+00, 8.4719E+01, &
      1.742378E+01, 7.85556E+01, 1.0443E+01, 1.1653E+02, &
      1.759961E+01, 7.96359E+01, 1.0270E+01, 1.1350E+02, &
      1.768802E+01, 7.98846E+01, 1.0185E+01, 1.1201E+02, &
      1.795875E+01, 7.99644E+01, 9.9308E+00, 1.0764E+02, &
      1.803095E+01, 7.93318E+01, 9.8647E+00, 1.0650E+02, &
      1.813924E+01, 8.03157E+01, 1.1305E+01, 1.2092E+02, &
      1.840998E+01, 8.14412E+01, 1.1052E+01, 1.1653E+02, &
      1.881398E+01, 8.23219E+01, 1.0691E+01, 1.1038E+02, &
      2.011215E+01, 8.38524E+01, 9.6316E+00, 9.3238E+01, &
      2.149988E+01, 8.47234E+01, 8.6710E+00, 7.8717E+01, &
      2.298338E+01, 8.52917E+01, 7.8074E+00, 6.6479E+01, &
      2.456923E+01, 8.56786E+01, 7.0280E+00, 5.6142E+01, &
      2.626450E+01, 8.59512E+01, 6.3293E+00, 4.7441E+01, &
      2.807676E+01, 8.61471E+01, 5.6672E+00, 3.9877E+01, &
      3.001405E+01, 8.63113E+01, 5.0628E+00, 3.3455E+01, &
      3.208502E+01, 8.63539E+01, 4.5199E+00, 2.8058E+01, &
      3.429889E+01, 8.63577E+01, 4.0393E+00, 2.3562E+01, &
      3.666551E+01, 8.63973E+01, 3.6038E+00, 1.9764E+01, &
      3.919543E+01, 8.63486E+01, 3.2085E+00, 1.6551E+01, &
      4.189992E+01, 8.62805E+01, 2.8588E+00, 1.3877E+01, &
      4.479101E+01, 8.62003E+01, 2.5491E+00, 1.1650E+01, &
      4.788159E+01, 8.61118E+01, 2.2747E+00, 9.7911E+00, &
      5.118542E+01, 8.60176E+01, 2.0314E+00, 8.2398E+00, &
      5.471721E+01, 8.59188E+01, 1.8154E+00, 6.9434E+00, &
      5.849270E+01, 8.58157E+01, 1.6236E+00, 5.8588E+00, &
      6.252870E+01, 8.57075E+01, 1.4531E+00, 4.9505E+00, &
      6.684318E+01, 8.55914E+01, 1.2991E+00, 4.1825E+00, &
      7.145536E+01, 8.54621E+01, 1.1619E+00, 3.5384E+00, &
      7.638578E+01, 8.53123E+01, 1.0398E+00, 2.9978E+00, &
      8.165640E+01, 8.51253E+01, 9.2583E-01, 2.5320E+00, &
      8.729069E+01, 8.48517E+01, 8.1928E-01, 2.1306E+00, &
      9.331374E+01, 8.43273E+01, 7.2553E-01, 1.7971E+00, &
      9.643592E+01, 8.36637E+01, 6.8351E-01, 1.6539E+00, &
      9.791198E+01, 8.26496E+01, 6.6498E-01, 1.5921E+00, &
      9.830560E+01, 8.14368E+01, 6.6017E-01, 1.5761E+00, &
      9.889602E+01, 8.27006E+01, 3.0396E+00, 6.1268E+00, &
      9.975239E+01, 8.35003E+01, 2.9993E+00, 5.9972E+00, &
      1.003721E+02, 8.38054E+01, 2.9707E+00, 5.9058E+00, &
      1.066353E+02, 8.49779E+01, 2.7033E+00, 5.0802E+00, &
      1.139931E+02, 8.54790E+01, 2.4338E+00, 4.3013E+00, &
      1.218587E+02, 8.57643E+01, 2.1883E+00, 3.6395E+00, &
      1.302669E+02, 8.59468E+01, 1.9667E+00, 3.0808E+00, &
      1.392553E+02, 8.60696E+01, 1.7675E+00, 2.6097E+00, &
      1.488640E+02, 8.61528E+01, 1.5857E+00, 2.2094E+00, &
      1.591356E+02, 8.62076E+01, 1.4226E+00, 1.8726E+00, &
      1.701159E+02, 8.62427E+01, 1.2763E+00, 1.5891E+00, &
      1.818539E+02, 8.62636E+01, 1.1451E+00, 1.3506E+00, &
      1.944018E+02, 8.62742E+01, 1.0275E+00, 1.1498E+00, &
      2.078156E+02, 8.62772E+01, 9.2212E-01, 9.8079E-01, &
      2.221548E+02, 8.62748E+01, 8.2775E-01, 8.3848E-01, &
      2.374835E+02, 8.62685E+01, 7.4313E-01, 7.1851E-01, &
      2.538699E+02, 8.62595E+01, 6.6724E-01, 6.1729E-01, &
      2.713869E+02, 8.62486E+01, 5.9916E-01, 5.3183E-01, &
      2.901126E+02, 8.62365E+01, 5.3809E-01, 4.5961E-01, &
      3.101304E+02, 8.62236E+01, 4.8329E-01, 3.9851E-01, &
      3.315294E+02, 8.62103E+01, 4.3412E-01, 3.4677E-01, &
      3.544049E+02, 8.61970E+01, 3.8999E-01, 3.0289E-01, &
      3.788588E+02, 8.61839E+01, 3.5037E-01, 2.6562E-01, &
      4.050001E+02, 8.61711E+01, 3.1481E-01, 2.3392E-01, &
      4.329451E+02, 8.61587E+01, 2.8289E-01, 2.0690E-01]
   case(87)
      frel = -1.2942E+00
      allocate(nist_tab(3100))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.040001E-02, -4.29397E+00, 0.0000E+00, 1.2036E-04, &
      1.055920E-02, -1.04119E+01, 0.0000E+00, 1.2958E-04, &
      1.060165E-02, -1.78248E+01, 0.0000E+00, 1.3213E-04, &
      1.066532E-02, -9.97566E+00, 1.4635E+01, 2.5893E+05, &
      1.069000E-02, -8.10559E+00, 1.4564E+01, 2.5709E+05, &
      1.082450E-02, -3.08761E+00, 1.4189E+01, 2.4736E+05, &
      1.142761E-02, 3.75079E+00, 1.2671E+01, 2.0924E+05, &
      1.221612E-02, 6.79698E+00, 1.1021E+01, 1.7023E+05, &
      1.305903E-02, 8.07632E+00, 9.5691E+00, 1.3827E+05, &
      1.396010E-02, 8.09233E+00, 8.2905E+00, 1.1206E+05, &
      1.486167E-02, 5.35923E+00, 7.2306E+00, 9.1808E+04, &
      1.492335E-02, 4.76667E+00, 7.1649E+00, 9.0598E+04, &
      1.508915E-02, 1.47371E+00, 6.9921E+00, 8.7442E+04, &
      1.514981E-02, -3.48118E+00, 6.9303E+00, 8.6322E+04, &
      1.524080E-02, 2.02215E+00, 1.6744E+01, 2.0731E+05, &
      1.546827E-02, 6.94430E+00, 1.6024E+01, 1.9548E+05, &
      1.595306E-02, 1.05356E+01, 1.4623E+01, 1.7297E+05, &
      1.705382E-02, 1.35499E+01, 1.2017E+01, 1.3297E+05, &
      1.823053E-02, 1.47041E+01, 1.0024E+01, 1.0376E+05, &
      1.948844E-02, 1.51952E+01, 8.4856E+00, 8.2163E+04, &
      2.083314E-02, 1.53610E+01, 7.2605E+00, 6.5763E+04, &
      2.227063E-02, 1.53364E+01, 6.2608E+00, 5.3048E+04, &
      2.380730E-02, 1.51812E+01, 5.4288E+00, 4.3030E+04, &
      2.545001E-02, 1.49160E+01, 4.7256E+00, 3.5038E+04, &
      2.720606E-02, 1.44964E+01, 4.1235E+00, 2.8601E+04, &
      2.731051E-02, 1.44599E+01, 4.0915E+00, 2.8270E+04, &
      2.772853E-02, 1.42366E+01, 3.9675E+00, 2.7000E+04, &
      2.784000E-02, 1.40384E+01, 3.9354E+00, 2.6674E+04, &
      2.800721E-02, 1.41980E+01, 4.2374E+00, 2.8550E+04, &
      2.842523E-02, 1.42818E+01, 4.1033E+00, 2.7240E+04, &
      2.908327E-02, 1.42378E+01, 3.9051E+00, 2.5338E+04, &
      3.109002E-02, 1.38726E+01, 3.3834E+00, 2.0536E+04, &
      3.323523E-02, 1.33372E+01, 2.9381E+00, 1.6682E+04, &
      3.552846E-02, 1.26493E+01, 2.5561E+00, 1.3576E+04, &
      3.797993E-02, 1.17859E+01, 2.2261E+00, 1.1060E+04, &
      4.060054E-02, 1.06868E+01, 1.9401E+00, 9.0171E+03, &
      4.340198E-02, 9.24172E+00, 1.6920E+00, 7.3563E+03, &
      4.639671E-02, 7.23788E+00, 1.4774E+00, 6.0089E+03, &
      4.959809E-02, 4.16636E+00, 1.2924E+00, 4.9170E+03, &
      5.302035E-02, -1.94328E+00, 1.1330E+00, 4.0324E+03, &
      5.346183E-02, -3.39592E+00, 1.1148E+00, 3.9350E+03, &
      5.428013E-02, -8.28821E+00, 1.0824E+00, 3.7629E+03, &
      5.449834E-02, -1.29003E+01, 1.0740E+00, 3.7189E+03, &
      5.482565E-02, -9.56276E+00, 9.4014E+00, 3.2358E+04, &
      5.564395E-02, -7.57099E+00, 1.0254E+01, 3.4772E+04, &
      5.667876E-02, -7.78448E+00, 1.1446E+01, 3.8107E+04, &
      5.834708E-02, -1.02763E+01, 1.3666E+01, 4.4199E+04, &
      5.924014E-02, -1.46377E+01, 1.5021E+01, 4.7848E+04, &
      5.947829E-02, -1.96640E+01, 1.5404E+01, 4.8870E+04, &
      5.983552E-02, -1.49067E+01, 2.5839E+01, 8.1488E+04, &
      6.058959E-02, -1.09069E+01, 2.7721E+01, 8.6336E+04, &
      6.072859E-02, -1.04605E+01, 2.8065E+01, 8.7206E+04, &
      6.477028E-02, -7.63173E-01, 3.7010E+01, 1.0782E+05, &
      6.923942E-02, 1.19830E+01, 4.2431E+01, 1.1564E+05, &
      7.401695E-02, 2.49875E+01, 4.0957E+01, 1.0442E+05, &
      7.912411E-02, 3.44872E+01, 3.4661E+01, 8.2662E+04, &
      8.458368E-02, 3.95132E+01, 2.6691E+01, 5.9546E+04, &
      9.041995E-02, 4.09454E+01, 1.9311E+01, 4.0300E+04, &
      9.665893E-02, 4.01932E+01, 1.3486E+01, 2.6327E+04, &
      1.033284E-01, 3.84133E+01, 9.2950E+00, 1.6975E+04, &
      1.104581E-01, 3.63113E+01, 6.4375E+00, 1.0999E+04, &
      1.180797E-01, 3.42174E+01, 4.5422E+00, 7.2593E+03, &
      1.262272E-01, 3.21710E+01, 3.2975E+00, 4.9303E+03, &
      1.302983E-01, 3.10527E+01, 2.8656E+00, 4.1509E+03, &
      1.322926E-01, 3.01419E+01, 2.6873E+00, 3.8340E+03, &
      1.328244E-01, 2.93747E+01, 2.6430E+00, 3.7556E+03, &
      1.336222E-01, 2.99535E+01, 3.9126E+00, 5.5262E+03, &
      1.349368E-01, 3.02267E+01, 3.7871E+00, 5.2968E+03, &
      1.356165E-01, 3.02520E+01, 3.7252E+00, 5.1841E+03, &
      1.442475E-01, 2.96373E+01, 3.0822E+00, 4.0329E+03, &
      1.542005E-01, 2.86015E+01, 2.5847E+00, 3.1641E+03, &
      1.648404E-01, 2.73575E+01, 2.2433E+00, 2.5693E+03, &
      1.656291E-01, 2.72376E+01, 2.2234E+00, 2.5344E+03, &
      1.681643E-01, 2.66582E+01, 2.1635E+00, 2.4290E+03, &
      1.688403E-01, 2.61545E+01, 2.1485E+00, 2.4025E+03, &
      1.698543E-01, 2.65525E+01, 3.0117E+00, 3.3471E+03, &
      1.723895E-01, 2.67670E+01, 2.9216E+00, 3.1993E+03, &
      1.762144E-01, 2.66924E+01, 2.7986E+00, 2.9983E+03, &
      1.883732E-01, 2.60173E+01, 2.4902E+00, 2.4961E+03, &
      2.013709E-01, 2.50915E+01, 2.2659E+00, 2.1251E+03, &
      2.152655E-01, 2.39420E+01, 2.1083E+00, 1.8502E+03, &
      2.156348E-01, 2.39070E+01, 2.1050E+00, 1.8441E+03, &
      2.189353E-01, 2.35511E+01, 2.0767E+00, 1.7921E+03, &
      2.198155E-01, 2.33824E+01, 2.0697E+00, 1.7788E+03, &
      2.211357E-01, 2.33614E+01, 2.2421E+00, 1.9154E+03, &
      2.244362E-01, 2.31489E+01, 2.2200E+00, 1.8686E+03, &
      2.301188E-01, 2.26610E+01, 2.1874E+00, 1.7959E+03, &
      2.340857E-01, 2.22701E+01, 2.1682E+00, 1.7501E+03, &
      2.376687E-01, 2.18519E+01, 2.1531E+00, 1.7118E+03, &
      2.386241E-01, 2.16828E+01, 2.1494E+00, 1.7021E+03, &
      2.400573E-01, 2.15829E+01, 2.2791E+00, 1.7939E+03, &
      2.415586E-01, 2.14436E+01, 2.2876E+00, 1.7895E+03, &
      2.436403E-01, 2.12188E+01, 2.3022E+00, 1.7855E+03, &
      2.452560E-01, 2.10204E+01, 2.3159E+00, 1.7843E+03, &
      2.459970E-01, 2.09100E+01, 2.3229E+00, 1.7843E+03, &
      2.462419E-01, 2.08613E+01, 2.3253E+00, 1.7844E+03, &
      2.477208E-01, 2.07371E+01, 2.4424E+00, 1.8630E+03, &
      2.514182E-01, 2.03427E+01, 2.5174E+00, 1.8921E+03, &
      2.629708E-01, 1.88987E+01, 2.9120E+00, 2.0924E+03, &
      2.811158E-01, 1.63963E+01, 4.2112E+00, 2.8298E+03, &
      3.005128E-01, 1.38493E+01, 6.6175E+00, 4.1587E+03, &
      3.212482E-01, 1.18075E+01, 1.0220E+01, 6.0069E+03, &
      3.434143E-01, 1.08331E+01, 1.4767E+01, 8.1180E+03, &
      3.671099E-01, 1.12710E+01, 1.9728E+01, 1.0145E+04, &
      3.924405E-01, 1.31412E+01, 2.4493E+01, 1.1782E+04, &
      4.195189E-01, 1.61759E+01, 2.8553E+01, 1.2848E+04, &
      4.484657E-01, 1.99611E+01, 3.1609E+01, 1.3306E+04, &
      4.794098E-01, 2.40317E+01, 3.3572E+01, 1.3220E+04, &
      5.0000000E-01, 2.79322E+01, 3.4274E+01, 1.2941E+04, &
      5.0250000E-01, 2.82216E+01, 3.4332E+01, 1.2899E+04, &
      5.0501250E-01, 2.85084E+01, 3.4385E+01, 1.2854E+04, &
      5.0753756E-01, 2.87939E+01, 3.4433E+01, 1.2808E+04, &
      5.1007525E-01, 2.90749E+01, 3.4475E+01, 1.2760E+04, &
      5.1262563E-01, 2.93526E+01, 3.4513E+01, 1.2711E+04, &
      5.1518875E-01, 2.96269E+01, 3.4546E+01, 1.2660E+04, &
      5.1776470E-01, 2.98974E+01, 3.4574E+01, 1.2607E+04, &
      5.2035352E-01, 3.01639E+01, 3.4598E+01, 1.2553E+04, &
      5.2295529E-01, 3.04261E+01, 3.4617E+01, 1.2497E+04, &
      5.2557007E-01, 3.06836E+01, 3.4631E+01, 1.2440E+04, &
      5.2819792E-01, 3.09361E+01, 3.4641E+01, 1.2382E+04, &
      5.3083891E-01, 3.11833E+01, 3.4647E+01, 1.2323E+04, &
      5.3349310E-01, 3.14245E+01, 3.4648E+01, 1.2262E+04, &
      5.3616057E-01, 3.16594E+01, 3.4645E+01, 1.2200E+04, &
      5.3884137E-01, 3.18872E+01, 3.4638E+01, 1.2137E+04, &
      5.4153558E-01, 3.21073E+01, 3.4627E+01, 1.2072E+04, &
      5.4424325E-01, 3.23188E+01, 3.4612E+01, 1.2007E+04, &
      5.4696447E-01, 3.25205E+01, 3.4593E+01, 1.1941E+04, &
      5.4969929E-01, 3.27112E+01, 3.4570E+01, 1.1874E+04, &
      5.5244779E-01, 3.28890E+01, 3.4543E+01, 1.1806E+04, &
      5.5521003E-01, 3.30516E+01, 3.4513E+01, 1.1737E+04, &
      5.5798608E-01, 3.31958E+01, 3.4479E+01, 1.1667E+04, &
      5.6077601E-01, 3.33167E+01, 3.4442E+01, 1.1597E+04, &
      5.6357989E-01, 3.34071E+01, 3.4402E+01, 1.1525E+04, &
      5.6639779E-01, 3.34545E+01, 3.4358E+01, 1.1454E+04, &
      5.6922978E-01, 3.34350E+01, 3.4311E+01, 1.1381E+04, &
      5.7207593E-01, 3.32919E+01, 3.4260E+01, 1.1308E+04, &
      5.7493630E-01, 3.28112E+01, 3.4207E+01, 1.1234E+04, &
      5.7662439E-01, 3.15810E+01, 3.4174E+01, 1.1190E+04, &
      5.7737565E-01, 3.16271E+01, 3.6636E+01, 1.1981E+04, &
      5.7781099E-01, 3.22603E+01, 3.6630E+01, 1.1970E+04, &
      5.8070004E-01, 3.36387E+01, 3.6594E+01, 1.1898E+04, &
      5.8360354E-01, 3.42731E+01, 3.6555E+01, 1.1827E+04, &
      5.8652156E-01, 3.47286E+01, 3.6513E+01, 1.1754E+04, &
      5.8945417E-01, 3.50911E+01, 3.6468E+01, 1.1682E+04, &
      5.9240144E-01, 3.53850E+01, 3.6422E+01, 1.1609E+04, &
      5.9536345E-01, 3.56106E+01, 3.6372E+01, 1.1536E+04, &
      5.9834026E-01, 3.57396E+01, 3.6321E+01, 1.1462E+04, &
      6.0133196E-01, 3.56320E+01, 3.6267E+01, 1.1388E+04, &
      6.0290602E-01, 3.50309E+01, 3.6238E+01, 1.1349E+04, &
      6.0369393E-01, 3.51112E+01, 3.7725E+01, 1.1799E+04, &
      6.0433862E-01, 3.56398E+01, 3.7716E+01, 1.1784E+04, &
      6.0736032E-01, 3.65982E+01, 3.7673E+01, 1.1712E+04, &
      6.1039712E-01, 3.71673E+01, 3.7628E+01, 1.1640E+04, &
      6.1344910E-01, 3.76346E+01, 3.7582E+01, 1.1568E+04, &
      6.1651635E-01, 3.80516E+01, 3.7533E+01, 1.1496E+04, &
      6.1959893E-01, 3.84375E+01, 3.7483E+01, 1.1423E+04, &
      6.2269693E-01, 3.88015E+01, 3.7432E+01, 1.1351E+04, &
      6.2581041E-01, 3.91491E+01, 3.7379E+01, 1.1278E+04, &
      6.2893946E-01, 3.94836E+01, 3.7324E+01, 1.1206E+04, &
      6.3208416E-01, 3.98075E+01, 3.7268E+01, 1.1134E+04, &
      6.3524458E-01, 4.01225E+01, 3.7211E+01, 1.1061E+04, &
      6.3842080E-01, 4.04297E+01, 3.7152E+01, 1.0989E+04, &
      6.4161291E-01, 4.07302E+01, 3.7091E+01, 1.0916E+04, &
      6.4482097E-01, 4.10244E+01, 3.7029E+01, 1.0844E+04, &
      6.4804508E-01, 4.13129E+01, 3.6965E+01, 1.0771E+04, &
      6.5128530E-01, 4.15960E+01, 3.6899E+01, 1.0699E+04, &
      6.5454173E-01, 4.18742E+01, 3.6831E+01, 1.0626E+04, &
      6.5781444E-01, 4.21478E+01, 3.6762E+01, 1.0553E+04, &
      6.6110351E-01, 4.24181E+01, 3.6692E+01, 1.0481E+04, &
      6.6440903E-01, 4.26831E+01, 3.6620E+01, 1.0408E+04, &
      6.6773107E-01, 4.29442E+01, 3.6546E+01, 1.0336E+04, &
      6.7106973E-01, 4.32015E+01, 3.6471E+01, 1.0263E+04, &
      6.7442508E-01, 4.34550E+01, 3.6394E+01, 1.0191E+04, &
      6.7779720E-01, 4.37049E+01, 3.6316E+01, 1.0118E+04, &
      6.8118619E-01, 4.39511E+01, 3.6236E+01, 1.0046E+04, &
      6.8459212E-01, 4.41937E+01, 3.6155E+01, 9.9737E+03, &
      6.8801508E-01, 4.44327E+01, 3.6072E+01, 9.9015E+03, &
      6.9145515E-01, 4.46683E+01, 3.5988E+01, 9.8293E+03, &
      6.9491243E-01, 4.49003E+01, 3.5902E+01, 9.7573E+03, &
      6.9838699E-01, 4.51288E+01, 3.5815E+01, 9.6853E+03, &
      7.0187893E-01, 4.53539E+01, 3.5727E+01, 9.6135E+03, &
      7.0538832E-01, 4.55756E+01, 3.5638E+01, 9.5419E+03, &
      7.0891526E-01, 4.57937E+01, 3.5547E+01, 9.4704E+03, &
      7.1245984E-01, 4.60084E+01, 3.5456E+01, 9.3991E+03, &
      7.1602214E-01, 4.62196E+01, 3.5363E+01, 9.3279E+03, &
      7.1960225E-01, 4.64513E+01, 3.5269E+01, 9.2569E+03, &
      7.2320026E-01, 4.66555E+01, 3.5173E+01, 9.1860E+03, &
      7.2681626E-01, 4.68559E+01, 3.5077E+01, 9.1154E+03, &
      7.3045034E-01, 4.70524E+01, 3.4980E+01, 9.0450E+03, &
      7.3410260E-01, 4.72450E+01, 3.4882E+01, 8.9748E+03, &
      7.3777311E-01, 4.74335E+01, 3.4782E+01, 8.9049E+03, &
      7.4146197E-01, 4.76486E+01, 3.4682E+01, 8.8352E+03, &
      7.4516928E-01, 4.78287E+01, 3.4582E+01, 8.7658E+03, &
      7.4889513E-01, 4.80042E+01, 3.4480E+01, 8.6966E+03, &
      7.5263961E-01, 4.81747E+01, 3.4378E+01, 8.6278E+03, &
      7.5640280E-01, 4.83399E+01, 3.4275E+01, 8.5592E+03, &
      7.6018482E-01, 4.84994E+01, 3.4171E+01, 8.4909E+03, &
      7.6398574E-01, 4.86526E+01, 3.4066E+01, 8.4230E+03, &
      7.6780567E-01, 4.87988E+01, 3.3961E+01, 8.3553E+03, &
      7.7164470E-01, 4.89371E+01, 3.3856E+01, 8.2880E+03, &
      7.7550292E-01, 4.90662E+01, 3.3750E+01, 8.2210E+03, &
      7.7938044E-01, 4.91845E+01, 3.3643E+01, 8.1544E+03, &
      7.8327734E-01, 4.92895E+01, 3.3536E+01, 8.0881E+03, &
      7.8719373E-01, 4.93776E+01, 3.3428E+01, 8.0221E+03, &
      7.9112969E-01, 4.94432E+01, 3.3320E+01, 7.9565E+03, &
      7.9508534E-01, 4.94765E+01, 3.3212E+01, 7.8913E+03, &
      7.9906077E-01, 4.94582E+01, 3.3103E+01, 7.8265E+03, &
      8.0305607E-01, 4.93420E+01, 3.2994E+01, 7.7620E+03, &
      8.0707135E-01, 4.89529E+01, 3.2885E+01, 7.6979E+03, &
      8.0891460E-01, 4.83781E+01, 3.2835E+01, 7.6687E+03, &
      8.1110671E-01, 4.84888E+01, 3.4857E+01, 8.1184E+03, &
      8.1516224E-01, 4.96878E+01, 3.4748E+01, 8.0530E+03, &
      8.1923806E-01, 5.02509E+01, 3.4640E+01, 7.9880E+03, &
      8.2333425E-01, 5.06701E+01, 3.4531E+01, 7.9234E+03, &
      8.2745092E-01, 5.10229E+01, 3.4422E+01, 7.8592E+03, &
      8.3158817E-01, 5.13368E+01, 3.4313E+01, 7.7954E+03, &
      8.3574611E-01, 5.16249E+01, 3.4204E+01, 7.7321E+03, &
      8.3992484E-01, 5.18945E+01, 3.4095E+01, 7.6691E+03, &
      8.4412447E-01, 5.21499E+01, 3.3985E+01, 7.6066E+03, &
      8.4834509E-01, 5.23940E+01, 3.3876E+01, 7.5446E+03, &
      8.5258682E-01, 5.26289E+01, 3.3767E+01, 7.4829E+03, &
      8.5684975E-01, 5.28561E+01, 3.3658E+01, 7.4217E+03, &
      8.6113400E-01, 5.30767E+01, 3.3548E+01, 7.3609E+03, &
      8.6543967E-01, 5.32915E+01, 3.3439E+01, 7.3006E+03, &
      8.6976687E-01, 5.35012E+01, 3.3330E+01, 7.2407E+03, &
      8.7411570E-01, 5.37063E+01, 3.3221E+01, 7.1812E+03, &
      8.7848628E-01, 5.39072E+01, 3.3113E+01, 7.1222E+03, &
      8.8287871E-01, 5.41045E+01, 3.3004E+01, 7.0636E+03, &
      8.8729310E-01, 5.42983E+01, 3.2896E+01, 7.0055E+03, &
      8.9172957E-01, 5.44890E+01, 3.2787E+01, 6.9478E+03, &
      8.9618822E-01, 5.46767E+01, 3.2679E+01, 6.8906E+03, &
      9.0066916E-01, 5.48618E+01, 3.2572E+01, 6.8338E+03, &
      9.0517250E-01, 5.50442E+01, 3.2464E+01, 6.7774E+03, &
      9.0969837E-01, 5.52242E+01, 3.2357E+01, 6.7215E+03, &
      9.1424686E-01, 5.54019E+01, 3.2249E+01, 6.6660E+03, &
      9.1881809E-01, 5.55774E+01, 3.2142E+01, 6.6109E+03, &
      9.2341218E-01, 5.57507E+01, 3.2036E+01, 6.5563E+03, &
      9.2802924E-01, 5.59219E+01, 3.1929E+01, 6.5022E+03, &
      9.3266939E-01, 5.60910E+01, 3.1823E+01, 6.4484E+03, &
      9.3733274E-01, 5.62579E+01, 3.1718E+01, 6.3951E+03, &
      9.4201940E-01, 5.64225E+01, 3.1612E+01, 6.3423E+03, &
      9.4672950E-01, 5.65846E+01, 3.1507E+01, 6.2899E+03, &
      9.5146315E-01, 5.67437E+01, 3.1403E+01, 6.2379E+03, &
      9.5622046E-01, 5.68991E+01, 3.1298E+01, 6.1864E+03, &
      9.6100156E-01, 5.70492E+01, 3.1195E+01, 6.1353E+03, &
      9.6580657E-01, 5.71911E+01, 3.1091E+01, 6.0847E+03, &
      9.7063560E-01, 5.73175E+01, 3.0988E+01, 6.0345E+03, &
      9.7548878E-01, 5.74036E+01, 3.0885E+01, 5.9846E+03, &
      9.7859862E-01, 5.73653E+01, 3.0814E+01, 5.9518E+03, &
      9.8036623E-01, 5.72538E+01, 3.1188E+01, 6.0131E+03, &
      9.8140142E-01, 5.74694E+01, 3.1165E+01, 6.0023E+03, &
      9.8526806E-01, 5.77884E+01, 3.1078E+01, 5.9622E+03, &
      9.9019440E-01, 5.80602E+01, 3.0968E+01, 5.9118E+03, &
      9.9514537E-01, 5.82997E+01, 3.0859E+01, 5.8617E+03, &
      1.0001211E+00, 5.85305E+01, 3.0749E+01, 5.8118E+03, &
      1.0051217E+00, 5.87261E+01, 3.0568E+01, 5.7490E+03, &
      1.0101473E+00, 5.89113E+01, 3.0388E+01, 5.6869E+03, &
      1.0151980E+00, 5.90881E+01, 3.0210E+01, 5.6256E+03, &
      1.0202740E+00, 5.92577E+01, 3.0033E+01, 5.5649E+03, &
      1.0253754E+00, 5.94197E+01, 2.9850E+01, 5.5036E+03, &
      1.0305023E+00, 5.95748E+01, 2.9667E+01, 5.4428E+03, &
      1.0356548E+00, 5.97237E+01, 2.9485E+01, 5.3827E+03, &
      1.0408331E+00, 5.98667E+01, 2.9305E+01, 5.3234E+03, &
      1.0460372E+00, 6.00043E+01, 2.9127E+01, 5.2648E+03, &
      1.0512674E+00, 6.01367E+01, 2.8950E+01, 5.2068E+03, &
      1.0565238E+00, 6.02641E+01, 2.8774E+01, 5.1496E+03, &
      1.0618064E+00, 6.03869E+01, 2.8600E+01, 5.0931E+03, &
      1.0671154E+00, 6.05051E+01, 2.8427E+01, 5.0373E+03, &
      1.0724510E+00, 6.06190E+01, 2.8255E+01, 4.9821E+03, &
      1.0778132E+00, 6.07285E+01, 2.8085E+01, 4.9276E+03, &
      1.0832023E+00, 6.08337E+01, 2.7916E+01, 4.8737E+03, &
      1.0886183E+00, 6.09345E+01, 2.7748E+01, 4.8205E+03, &
      1.0940614E+00, 6.10309E+01, 2.7582E+01, 4.7678E+03, &
      1.0995317E+00, 6.11226E+01, 2.7417E+01, 4.7159E+03, &
      1.1050294E+00, 6.12096E+01, 2.7253E+01, 4.6645E+03, &
      1.1105545E+00, 6.12913E+01, 2.7090E+01, 4.6137E+03, &
      1.1161073E+00, 6.13673E+01, 2.6929E+01, 4.5636E+03, &
      1.1216878E+00, 6.14365E+01, 2.6768E+01, 4.5139E+03, &
      1.1272963E+00, 6.14974E+01, 2.6608E+01, 4.4647E+03, &
      1.1329328E+00, 6.15470E+01, 2.6449E+01, 4.4161E+03, &
      1.1385974E+00, 6.15793E+01, 2.6292E+01, 4.3681E+03, &
      1.1442904E+00, 6.15791E+01, 2.6135E+01, 4.3207E+03, &
      1.1500119E+00, 6.14763E+01, 2.5980E+01, 4.2738E+03, &
      1.1516164E+00, 6.13637E+01, 2.5937E+01, 4.2608E+03, &
      1.1543836E+00, 6.14176E+01, 2.6420E+01, 4.3296E+03, &
      1.1557619E+00, 6.15604E+01, 2.6383E+01, 4.3184E+03, &
      1.1615407E+00, 6.18434E+01, 2.6230E+01, 4.2720E+03, &
      1.1673484E+00, 6.20159E+01, 2.6077E+01, 4.2262E+03, &
      1.1731852E+00, 6.21549E+01, 2.5926E+01, 4.1810E+03, &
      1.1790511E+00, 6.22765E+01, 2.5776E+01, 4.1362E+03, &
      1.1849464E+00, 6.23872E+01, 2.5627E+01, 4.0920E+03, &
      1.1908711E+00, 6.24900E+01, 2.5480E+01, 4.0483E+03, &
      1.1968254E+00, 6.25866E+01, 2.5333E+01, 4.0051E+03, &
      1.2028096E+00, 6.26782E+01, 2.5188E+01, 3.9625E+03, &
      1.2088236E+00, 6.27655E+01, 2.5044E+01, 3.9203E+03, &
      1.2148677E+00, 6.28490E+01, 2.4901E+01, 3.8787E+03, &
      1.2209421E+00, 6.29295E+01, 2.4759E+01, 3.8376E+03, &
      1.2270468E+00, 6.30072E+01, 2.4619E+01, 3.7970E+03, &
      1.2331820E+00, 6.30823E+01, 2.4480E+01, 3.7569E+03, &
      1.2393479E+00, 6.31550E+01, 2.4342E+01, 3.7173E+03, &
      1.2455447E+00, 6.32254E+01, 2.4205E+01, 3.6781E+03, &
      1.2517724E+00, 6.32938E+01, 2.4069E+01, 3.6394E+03, &
      1.2580312E+00, 6.33602E+01, 2.3934E+01, 3.6012E+03, &
      1.2643214E+00, 6.34247E+01, 2.3801E+01, 3.5634E+03, &
      1.2706430E+00, 6.34875E+01, 2.3668E+01, 3.5260E+03, &
      1.2769962E+00, 6.35485E+01, 2.3536E+01, 3.4891E+03, &
      1.2833812E+00, 6.36080E+01, 2.3406E+01, 3.4526E+03, &
      1.2897981E+00, 6.36659E+01, 2.3276E+01, 3.4165E+03, &
      1.2962471E+00, 6.37223E+01, 2.3148E+01, 3.3809E+03, &
      1.3027283E+00, 6.37773E+01, 2.3020E+01, 3.3457E+03, &
      1.3092420E+00, 6.38309E+01, 2.2894E+01, 3.3109E+03, &
      1.3157882E+00, 6.38832E+01, 2.2768E+01, 3.2765E+03, &
      1.3223671E+00, 6.39343E+01, 2.2644E+01, 3.2425E+03, &
      1.3289790E+00, 6.39841E+01, 2.2520E+01, 3.2089E+03, &
      1.3356239E+00, 6.40328E+01, 2.2398E+01, 3.1757E+03, &
      1.3423020E+00, 6.40803E+01, 2.2276E+01, 3.1429E+03, &
      1.3490135E+00, 6.41268E+01, 2.2156E+01, 3.1104E+03, &
      1.3557586E+00, 6.41723E+01, 2.2036E+01, 3.0784E+03, &
      1.3625374E+00, 6.42169E+01, 2.1917E+01, 3.0467E+03, &
      1.3693500E+00, 6.42605E+01, 2.1799E+01, 3.0153E+03, &
      1.3761968E+00, 6.43028E+01, 2.1680E+01, 2.9840E+03, &
      1.3830778E+00, 6.43439E+01, 2.1561E+01, 2.9531E+03, &
      1.3899932E+00, 6.43837E+01, 2.1444E+01, 2.9225E+03, &
      1.3969431E+00, 6.44223E+01, 2.1327E+01, 2.8923E+03, &
      1.4039278E+00, 6.44599E+01, 2.1211E+01, 2.8624E+03, &
      1.4109475E+00, 6.44963E+01, 2.1097E+01, 2.8329E+03, &
      1.4180022E+00, 6.45317E+01, 2.0982E+01, 2.8037E+03, &
      1.4250922E+00, 6.45660E+01, 2.0869E+01, 2.7748E+03, &
      1.4322177E+00, 6.45993E+01, 2.0757E+01, 2.7463E+03, &
      1.4393788E+00, 6.46317E+01, 2.0646E+01, 2.7181E+03, &
      1.4465757E+00, 6.46630E+01, 2.0535E+01, 2.6902E+03, &
      1.4538086E+00, 6.46935E+01, 2.0425E+01, 2.6626E+03, &
      1.4610776E+00, 6.47230E+01, 2.0316E+01, 2.6354E+03, &
      1.4683830E+00, 6.47517E+01, 2.0208E+01, 2.6084E+03, &
      1.4757249E+00, 6.47795E+01, 2.0100E+01, 2.5818E+03, &
      1.4831035E+00, 6.48064E+01, 1.9994E+01, 2.5554E+03, &
      1.4905190E+00, 6.48326E+01, 1.9888E+01, 2.5294E+03, &
      1.4979716E+00, 6.48579E+01, 1.9783E+01, 2.5037E+03, &
      1.5054615E+00, 6.48825E+01, 1.9678E+01, 2.4781E+03, &
      1.5129888E+00, 6.49060E+01, 1.9574E+01, 2.4529E+03, &
      1.5205537E+00, 6.49287E+01, 1.9471E+01, 2.4279E+03, &
      1.5281565E+00, 6.49506E+01, 1.9368E+01, 2.4032E+03, &
      1.5357973E+00, 6.49715E+01, 1.9266E+01, 2.3788E+03, &
      1.5434763E+00, 6.49916E+01, 1.9165E+01, 2.3547E+03, &
      1.5511937E+00, 6.50109E+01, 1.9065E+01, 2.3308E+03, &
      1.5589496E+00, 6.50295E+01, 1.8965E+01, 2.3072E+03, &
      1.5667444E+00, 6.50472E+01, 1.8866E+01, 2.2839E+03, &
      1.5745781E+00, 6.50641E+01, 1.8768E+01, 2.2609E+03, &
      1.5824510E+00, 6.50803E+01, 1.8671E+01, 2.2380E+03, &
      1.5903633E+00, 6.50957E+01, 1.8574E+01, 2.2155E+03, &
      1.5983151E+00, 6.51103E+01, 1.8478E+01, 2.1932E+03, &
      1.6063066E+00, 6.51243E+01, 1.8382E+01, 2.1711E+03, &
      1.6143382E+00, 6.51375E+01, 1.8288E+01, 2.1493E+03, &
      1.6224099E+00, 6.51500E+01, 1.8194E+01, 2.1278E+03, &
      1.6305219E+00, 6.51618E+01, 1.8100E+01, 2.1064E+03, &
      1.6386745E+00, 6.51729E+01, 1.8008E+01, 2.0854E+03, &
      1.6468679E+00, 6.51834E+01, 1.7916E+01, 2.0645E+03, &
      1.6551022E+00, 6.51932E+01, 1.7824E+01, 2.0439E+03, &
      1.6633777E+00, 6.52022E+01, 1.7733E+01, 2.0234E+03, &
      1.6716946E+00, 6.52104E+01, 1.7641E+01, 2.0031E+03, &
      1.6800531E+00, 6.52178E+01, 1.7551E+01, 1.9830E+03, &
      1.6884534E+00, 6.52243E+01, 1.7461E+01, 1.9632E+03, &
      1.6968956E+00, 6.52300E+01, 1.7372E+01, 1.9435E+03, &
      1.7053801E+00, 6.52350E+01, 1.7283E+01, 1.9241E+03, &
      1.7139070E+00, 6.52392E+01, 1.7195E+01, 1.9049E+03, &
      1.7224766E+00, 6.52425E+01, 1.7107E+01, 1.8859E+03, &
      1.7310889E+00, 6.52683E+01, 1.7021E+01, 1.8671E+03, &
      1.7397444E+00, 6.52703E+01, 1.6934E+01, 1.8486E+03, &
      1.7484431E+00, 6.52715E+01, 1.6849E+01, 1.8302E+03, &
      1.7571853E+00, 6.52720E+01, 1.6764E+01, 1.8120E+03, &
      1.7659712E+00, 6.52717E+01, 1.6679E+01, 1.7940E+03, &
      1.7748011E+00, 6.52706E+01, 1.6596E+01, 1.7763E+03, &
      1.7836751E+00, 6.52687E+01, 1.6512E+01, 1.7587E+03, &
      1.7925935E+00, 6.52661E+01, 1.6430E+01, 1.7413E+03, &
      1.8015565E+00, 6.52627E+01, 1.6347E+01, 1.7241E+03, &
      1.8105642E+00, 6.52614E+01, 1.6266E+01, 1.7071E+03, &
      1.8196171E+00, 6.52565E+01, 1.6185E+01, 1.6902E+03, &
      1.8287151E+00, 6.52508E+01, 1.6104E+01, 1.6736E+03, &
      1.8378587E+00, 6.52443E+01, 1.6025E+01, 1.6571E+03, &
      1.8470480E+00, 6.52370E+01, 1.5945E+01, 1.6408E+03, &
      1.8562833E+00, 6.52290E+01, 1.5867E+01, 1.6247E+03, &
      1.8655647E+00, 6.52201E+01, 1.5788E+01, 1.6088E+03, &
      1.8748925E+00, 6.52105E+01, 1.5711E+01, 1.5930E+03, &
      1.8842670E+00, 6.52000E+01, 1.5633E+01, 1.5774E+03, &
      1.8936883E+00, 6.51887E+01, 1.5557E+01, 1.5620E+03, &
      1.9031567E+00, 6.51766E+01, 1.5481E+01, 1.5468E+03, &
      1.9126725E+00, 6.51636E+01, 1.5405E+01, 1.5317E+03, &
      1.9222359E+00, 6.51498E+01, 1.5330E+01, 1.5167E+03, &
      1.9318471E+00, 6.51352E+01, 1.5256E+01, 1.5020E+03, &
      1.9415063E+00, 6.51197E+01, 1.5182E+01, 1.4874E+03, &
      1.9512138E+00, 6.51033E+01, 1.5108E+01, 1.4729E+03, &
      1.9609699E+00, 6.50860E+01, 1.5035E+01, 1.4586E+03, &
      1.9707747E+00, 6.50679E+01, 1.4963E+01, 1.4445E+03, &
      1.9806286E+00, 6.50488E+01, 1.4891E+01, 1.4305E+03, &
      1.9905318E+00, 6.50288E+01, 1.4819E+01, 1.4166E+03, &
      2.0004844E+00, 6.50079E+01, 1.4748E+01, 1.4030E+03, &
      2.0104868E+00, 6.49860E+01, 1.4677E+01, 1.3894E+03, &
      2.0205393E+00, 6.49632E+01, 1.4607E+01, 1.3760E+03, &
      2.0306420E+00, 6.49394E+01, 1.4538E+01, 1.3628E+03, &
      2.0407952E+00, 6.49146E+01, 1.4469E+01, 1.3496E+03, &
      2.0509992E+00, 6.48887E+01, 1.4400E+01, 1.3367E+03, &
      2.0612542E+00, 6.48619E+01, 1.4332E+01, 1.3238E+03, &
      2.0715604E+00, 6.48340E+01, 1.4264E+01, 1.3111E+03, &
      2.0819182E+00, 6.48050E+01, 1.4197E+01, 1.2986E+03, &
      2.0923278E+00, 6.47749E+01, 1.4130E+01, 1.2862E+03, &
      2.1027895E+00, 6.47437E+01, 1.4064E+01, 1.2739E+03, &
      2.1133034E+00, 6.47114E+01, 1.3998E+01, 1.2617E+03, &
      2.1238699E+00, 6.46779E+01, 1.3933E+01, 1.2497E+03, &
      2.1344893E+00, 6.46433E+01, 1.3868E+01, 1.2377E+03, &
      2.1451617E+00, 6.46074E+01, 1.3803E+01, 1.2260E+03, &
      2.1558875E+00, 6.45702E+01, 1.3739E+01, 1.2143E+03, &
      2.1666670E+00, 6.45318E+01, 1.3675E+01, 1.2028E+03, &
      2.1775003E+00, 6.44921E+01, 1.3612E+01, 1.1914E+03, &
      2.1883878E+00, 6.44511E+01, 1.3549E+01, 1.1801E+03, &
      2.1993297E+00, 6.44086E+01, 1.3487E+01, 1.1689E+03, &
      2.2103264E+00, 6.43648E+01, 1.3425E+01, 1.1579E+03, &
      2.2213780E+00, 6.43196E+01, 1.3363E+01, 1.1469E+03, &
      2.2324849E+00, 6.42728E+01, 1.3302E+01, 1.1361E+03, &
      2.2436473E+00, 6.42245E+01, 1.3241E+01, 1.1254E+03, &
      2.2548656E+00, 6.41747E+01, 1.3181E+01, 1.1148E+03, &
      2.2661399E+00, 6.41233E+01, 1.3121E+01, 1.1043E+03, &
      2.2774706E+00, 6.40702E+01, 1.3062E+01, 1.0939E+03, &
      2.2888579E+00, 6.40154E+01, 1.3003E+01, 1.0837E+03, &
      2.3003022E+00, 6.39588E+01, 1.2944E+01, 1.0735E+03, &
      2.3118037E+00, 6.38966E+01, 1.2883E+01, 1.0632E+03, &
      2.3233628E+00, 6.38359E+01, 1.2816E+01, 1.0526E+03, &
      2.3349796E+00, 6.37726E+01, 1.2750E+01, 1.0421E+03, &
      2.3466545E+00, 6.37066E+01, 1.2684E+01, 1.0316E+03, &
      2.3583878E+00, 6.36378E+01, 1.2619E+01, 1.0213E+03, &
      2.3701797E+00, 6.35662E+01, 1.2554E+01, 1.0111E+03, &
      2.3820306E+00, 6.34916E+01, 1.2490E+01, 1.0011E+03, &
      2.3939407E+00, 6.34141E+01, 1.2426E+01, 9.9110E+02, &
      2.4059104E+00, 6.33334E+01, 1.2362E+01, 9.8124E+02, &
      2.4179400E+00, 6.32495E+01, 1.2297E+01, 9.7132E+02, &
      2.4300297E+00, 6.31683E+01, 1.2229E+01, 9.6125E+02, &
      2.4421798E+00, 6.30769E+01, 1.2161E+01, 9.5128E+02, &
      2.4543907E+00, 6.29814E+01, 1.2094E+01, 9.4143E+02, &
      2.4666627E+00, 6.28818E+01, 1.2027E+01, 9.3169E+02, &
      2.4789960E+00, 6.27778E+01, 1.1961E+01, 9.2206E+02, &
      2.4913910E+00, 6.26692E+01, 1.1895E+01, 9.1253E+02, &
      2.5038479E+00, 6.25559E+01, 1.1830E+01, 9.0310E+02, &
      2.5163672E+00, 6.24376E+01, 1.1764E+01, 8.9376E+02, &
      2.5289490E+00, 6.23142E+01, 1.1700E+01, 8.8453E+02, &
      2.5415938E+00, 6.21852E+01, 1.1635E+01, 8.7540E+02, &
      2.5543017E+00, 6.20504E+01, 1.1571E+01, 8.6637E+02, &
      2.5670732E+00, 6.19096E+01, 1.1508E+01, 8.5744E+02, &
      2.5799086E+00, 6.17623E+01, 1.1445E+01, 8.4861E+02, &
      2.5928082E+00, 6.16081E+01, 1.1382E+01, 8.3988E+02, &
      2.6057722E+00, 6.14466E+01, 1.1320E+01, 8.3124E+02, &
      2.6188011E+00, 6.12773E+01, 1.1258E+01, 8.2271E+02, &
      2.6318951E+00, 6.10997E+01, 1.1197E+01, 8.1426E+02, &
      2.6450545E+00, 6.09132E+01, 1.1136E+01, 8.0591E+02, &
      2.6582798E+00, 6.07171E+01, 1.1076E+01, 7.9766E+02, &
      2.6715712E+00, 6.05106E+01, 1.1016E+01, 7.8949E+02, &
      2.6849291E+00, 6.02928E+01, 1.0956E+01, 7.8141E+02, &
      2.6983537E+00, 6.00629E+01, 1.0897E+01, 7.7343E+02, &
      2.7118455E+00, 5.98196E+01, 1.0838E+01, 7.6553E+02, &
      2.7254047E+00, 5.95617E+01, 1.0779E+01, 7.5772E+02, &
      2.7390317E+00, 5.92877E+01, 1.0721E+01, 7.5000E+02, &
      2.7527269E+00, 5.89959E+01, 1.0664E+01, 7.4236E+02, &
      2.7664905E+00, 5.86842E+01, 1.0606E+01, 7.3480E+02, &
      2.7803230E+00, 5.83502E+01, 1.0550E+01, 7.2733E+02, &
      2.7942246E+00, 5.79911E+01, 1.0493E+01, 7.1994E+02, &
      2.8081957E+00, 5.76033E+01, 1.0437E+01, 7.1263E+02, &
      2.8222367E+00, 5.71826E+01, 1.0381E+01, 7.0541E+02, &
      2.8363479E+00, 5.67236E+01, 1.0326E+01, 6.9826E+02, &
      2.8505296E+00, 5.62195E+01, 1.0271E+01, 6.9119E+02, &
      2.8647823E+00, 5.56615E+01, 1.0216E+01, 6.8420E+02, &
      2.8791062E+00, 5.50377E+01, 1.0162E+01, 6.7728E+02, &
      2.8935017E+00, 5.43319E+01, 1.0108E+01, 6.7044E+02, &
      2.9079692E+00, 5.35207E+01, 1.0055E+01, 6.6368E+02, &
      2.9225091E+00, 5.25686E+01, 1.0002E+01, 6.5699E+02, &
      2.9371216E+00, 5.14179E+01, 9.9489E+00, 6.5037E+02, &
      2.9518072E+00, 4.99695E+01, 9.8965E+00, 6.4382E+02, &
      2.9665662E+00, 4.79916E+01, 9.8445E+00, 6.3735E+02, &
      2.9813991E+00, 4.48559E+01, 9.7928E+00, 6.3094E+02, &
      2.9963061E+00, 3.61443E+01, 9.7414E+00, 6.2461E+02, &
      2.9988482E+00, 2.90598E+01, 9.7327E+00, 6.2354E+02, &
      3.0005520E+00, 2.87932E+01, 2.5805E+01, 1.6339E+03, &
      3.0112876E+00, 4.21044E+01, 2.5649E+01, 1.6184E+03, &
      3.0263440E+00, 4.61755E+01, 2.5434E+01, 1.5970E+03, &
      3.0414758E+00, 4.81844E+01, 2.5220E+01, 1.5758E+03, &
      3.0566831E+00, 4.93687E+01, 2.5009E+01, 1.5549E+03, &
      3.0719666E+00, 5.00364E+01, 2.4799E+01, 1.5344E+03, &
      3.0873264E+00, 5.02632E+01, 2.4591E+01, 1.5141E+03, &
      3.1027630E+00, 4.99701E+01, 2.4385E+01, 1.4941E+03, &
      3.1182768E+00, 4.87402E+01, 2.4181E+01, 1.4743E+03, &
      3.1338682E+00, 4.26489E+01, 2.3979E+01, 1.4548E+03, &
      3.1352340E+00, 3.97354E+01, 2.3962E+01, 1.4532E+03, &
      3.1371659E+00, 3.96606E+01, 3.4572E+01, 2.0905E+03, &
      3.1495376E+00, 4.91388E+01, 3.4343E+01, 2.0686E+03, &
      3.1652853E+00, 5.24315E+01, 3.4055E+01, 2.0411E+03, &
      3.1811117E+00, 5.44860E+01, 3.3769E+01, 2.0141E+03, &
      3.1970172E+00, 5.60404E+01, 3.3486E+01, 1.9874E+03, &
      3.2130023E+00, 5.73118E+01, 3.3206E+01, 1.9611E+03, &
      3.2290673E+00, 5.83958E+01, 3.2928E+01, 1.9351E+03, &
      3.2452127E+00, 5.93438E+01, 3.2652E+01, 1.9095E+03, &
      3.2614387E+00, 6.01871E+01, 3.2377E+01, 1.8841E+03, &
      3.2777459E+00, 6.09461E+01, 3.2104E+01, 1.8591E+03, &
      3.2941347E+00, 6.16351E+01, 3.1834E+01, 1.8344E+03, &
      3.3106053E+00, 6.22646E+01, 3.1566E+01, 1.8100E+03, &
      3.3271584E+00, 6.28423E+01, 3.1301E+01, 1.7860E+03, &
      3.3437941E+00, 6.33742E+01, 3.1038E+01, 1.7623E+03, &
      3.3605131E+00, 6.38649E+01, 3.0777E+01, 1.7389E+03, &
      3.3773157E+00, 6.43180E+01, 3.0518E+01, 1.7159E+03, &
      3.3942023E+00, 6.47364E+01, 3.0262E+01, 1.6931E+03, &
      3.4111733E+00, 6.51221E+01, 3.0008E+01, 1.6707E+03, &
      3.4282291E+00, 6.54767E+01, 2.9756E+01, 1.6485E+03, &
      3.4453703E+00, 6.58011E+01, 2.9507E+01, 1.6267E+03, &
      3.4625971E+00, 6.60957E+01, 2.9259E+01, 1.6052E+03, &
      3.4799101E+00, 6.63603E+01, 2.9014E+01, 1.5839E+03, &
      3.4973097E+00, 6.65938E+01, 2.8771E+01, 1.5630E+03, &
      3.5147962E+00, 6.67943E+01, 2.8530E+01, 1.5423E+03, &
      3.5323702E+00, 6.69584E+01, 2.8291E+01, 1.5219E+03, &
      3.5500321E+00, 6.70806E+01, 2.8055E+01, 1.5018E+03, &
      3.5677822E+00, 6.71519E+01, 2.7820E+01, 1.4819E+03, &
      3.5856211E+00, 6.71568E+01, 2.7587E+01, 1.4623E+03, &
      3.6035492E+00, 6.70666E+01, 2.7357E+01, 1.4430E+03, &
      3.6215670E+00, 6.68193E+01, 2.7128E+01, 1.4240E+03, &
      3.6396748E+00, 6.62394E+01, 2.6902E+01, 1.4052E+03, &
      3.6578732E+00, 6.42154E+01, 2.6677E+01, 1.3866E+03, &
      3.6585679E+00, 6.40033E+01, 2.6669E+01, 1.3859E+03, &
      3.6674323E+00, 6.40829E+01, 3.1386E+01, 1.6253E+03, &
      3.6761626E+00, 6.59033E+01, 3.1255E+01, 1.6147E+03, &
      3.6945434E+00, 6.75280E+01, 3.0981E+01, 1.5927E+03, &
      3.7130161E+00, 6.84959E+01, 3.0711E+01, 1.5711E+03, &
      3.7315812E+00, 6.92238E+01, 3.0442E+01, 1.5497E+03, &
      3.7502391E+00, 6.98219E+01, 3.0176E+01, 1.5287E+03, &
      3.7689903E+00, 7.03363E+01, 2.9913E+01, 1.5079E+03, &
      3.7878352E+00, 7.07903E+01, 2.9652E+01, 1.4874E+03, &
      3.8067744E+00, 7.11976E+01, 2.9393E+01, 1.4673E+03, &
      3.8258083E+00, 7.15662E+01, 2.9137E+01, 1.4473E+03, &
      3.8449373E+00, 7.19002E+01, 2.8885E+01, 1.4278E+03, &
      3.8641620E+00, 7.22091E+01, 2.8652E+01, 1.4093E+03, &
      3.8834828E+00, 7.25011E+01, 2.8422E+01, 1.3912E+03, &
      3.9029002E+00, 7.27771E+01, 2.8194E+01, 1.3733E+03, &
      3.9224147E+00, 7.30372E+01, 2.7965E+01, 1.3555E+03, &
      3.9420268E+00, 7.32824E+01, 2.7739E+01, 1.3379E+03, &
      3.9617369E+00, 7.35138E+01, 2.7515E+01, 1.3206E+03, &
      3.9815456E+00, 7.37322E+01, 2.7294E+01, 1.3036E+03, &
      4.0014533E+00, 7.39385E+01, 2.7076E+01, 1.2868E+03, &
      4.0214606E+00, 7.41333E+01, 2.6860E+01, 1.2703E+03, &
      4.0415679E+00, 7.43170E+01, 2.6647E+01, 1.2541E+03, &
      4.0617757E+00, 7.44900E+01, 2.6436E+01, 1.2381E+03, &
      4.0820846E+00, 7.46524E+01, 2.6227E+01, 1.2223E+03, &
      4.1024950E+00, 7.48034E+01, 2.6018E+01, 1.2066E+03, &
      4.1230075E+00, 7.49424E+01, 2.5811E+01, 1.1912E+03, &
      4.1436226E+00, 7.50692E+01, 2.5607E+01, 1.1760E+03, &
      4.1643407E+00, 7.51826E+01, 2.5405E+01, 1.1610E+03, &
      4.1851624E+00, 7.52813E+01, 2.5205E+01, 1.1462E+03, &
      4.2060882E+00, 7.53628E+01, 2.5007E+01, 1.1317E+03, &
      4.2271186E+00, 7.54230E+01, 2.4811E+01, 1.1173E+03, &
      4.2482542E+00, 7.54545E+01, 2.4617E+01, 1.1032E+03, &
      4.2694955E+00, 7.54430E+01, 2.4425E+01, 1.0892E+03, &
      4.2908430E+00, 7.53525E+01, 2.4235E+01, 1.0755E+03, &
      4.3122972E+00, 7.50426E+01, 2.4047E+01, 1.0619E+03, &
      4.3208125E+00, 7.46680E+01, 2.3973E+01, 1.0566E+03, &
      4.3331878E+00, 7.47152E+01, 2.5411E+01, 1.1162E+03, &
      4.3338587E+00, 7.47698E+01, 2.5405E+01, 1.1158E+03, &
      4.3555280E+00, 7.55931E+01, 2.5219E+01, 1.1022E+03, &
      4.3773056E+00, 7.59883E+01, 2.5035E+01, 1.0888E+03, &
      4.3991921E+00, 7.62762E+01, 2.4852E+01, 1.0756E+03, &
      4.4211881E+00, 7.65112E+01, 2.4671E+01, 1.0625E+03, &
      4.4432940E+00, 7.67127E+01, 2.4491E+01, 1.0496E+03, &
      4.4655105E+00, 7.68894E+01, 2.4314E+01, 1.0369E+03, &
      4.4878381E+00, 7.70459E+01, 2.4138E+01, 1.0244E+03, &
      4.5102772E+00, 7.71840E+01, 2.3962E+01, 1.0120E+03, &
      4.5328286E+00, 7.73038E+01, 2.3789E+01, 9.9974E+02, &
      4.5554928E+00, 7.74031E+01, 2.3617E+01, 9.8765E+02, &
      4.5782702E+00, 7.74765E+01, 2.3446E+01, 9.7573E+02, &
      4.6011616E+00, 7.75107E+01, 2.3277E+01, 9.6396E+02, &
      4.6241674E+00, 7.74652E+01, 2.3109E+01, 9.5234E+02, &
      4.6423238E+00, 7.72379E+01, 2.2978E+01, 9.4331E+02, &
      4.6472882E+00, 7.70364E+01, 2.2942E+01, 9.4086E+02, &
      4.6616761E+00, 7.73474E+01, 2.3853E+01, 9.7481E+02, &
      4.6705247E+00, 7.76095E+01, 2.3789E+01, 9.7042E+02, &
      4.6938773E+00, 7.80081E+01, 2.3624E+01, 9.5897E+02, &
      4.7173467E+00, 7.82833E+01, 2.3460E+01, 9.4767E+02, &
      4.7409334E+00, 7.85112E+01, 2.3298E+01, 9.3650E+02, &
      4.7646381E+00, 7.87131E+01, 2.3136E+01, 9.2545E+02, &
      4.7884613E+00, 7.88967E+01, 2.2970E+01, 9.1433E+02, &
      4.8124036E+00, 7.90663E+01, 2.2805E+01, 9.0334E+02, &
      4.8364656E+00, 7.92252E+01, 2.2642E+01, 8.9248E+02, &
      4.8606479E+00, 7.93752E+01, 2.2479E+01, 8.8176E+02, &
      4.8849512E+00, 7.95178E+01, 2.2318E+01, 8.7117E+02, &
      4.9093759E+00, 7.96539E+01, 2.2158E+01, 8.6070E+02, &
      4.9339228E+00, 7.97842E+01, 2.1999E+01, 8.5035E+02, &
      4.9585924E+00, 7.99094E+01, 2.1841E+01, 8.4014E+02, &
      4.9833854E+00, 8.00298E+01, 2.1684E+01, 8.3004E+02, &
      5.0083023E+00, 8.01459E+01, 2.1529E+01, 8.2007E+02, &
      5.0333438E+00, 8.02581E+01, 2.1374E+01, 8.1020E+02, &
      5.0585105E+00, 8.03665E+01, 2.1220E+01, 8.0046E+02, &
      5.0838031E+00, 8.04715E+01, 2.1068E+01, 7.9084E+02, &
      5.1092221E+00, 8.05732E+01, 2.0917E+01, 7.8134E+02, &
      5.1347682E+00, 8.06718E+01, 2.0767E+01, 7.7195E+02, &
      5.1604421E+00, 8.07675E+01, 2.0618E+01, 7.6268E+02, &
      5.1862443E+00, 8.08605E+01, 2.0470E+01, 7.5352E+02, &
      5.2121755E+00, 8.09508E+01, 2.0323E+01, 7.4447E+02, &
      5.2382364E+00, 8.10386E+01, 2.0177E+01, 7.3553E+02, &
      5.2644276E+00, 8.11240E+01, 2.0032E+01, 7.2670E+02, &
      5.2907497E+00, 8.12071E+01, 1.9889E+01, 7.1798E+02, &
      5.3172034E+00, 8.12880E+01, 1.9746E+01, 7.0936E+02, &
      5.3437895E+00, 8.13668E+01, 1.9605E+01, 7.0086E+02, &
      5.3705084E+00, 8.14436E+01, 1.9464E+01, 6.9245E+02, &
      5.3973609E+00, 8.15184E+01, 1.9325E+01, 6.8415E+02, &
      5.4243477E+00, 8.15914E+01, 1.9186E+01, 6.7595E+02, &
      5.4514695E+00, 8.16625E+01, 1.9049E+01, 6.6785E+02, &
      5.4787268E+00, 8.17320E+01, 1.8913E+01, 6.5985E+02, &
      5.5061205E+00, 8.17998E+01, 1.8778E+01, 6.5195E+02, &
      5.5336511E+00, 8.18660E+01, 1.8643E+01, 6.4414E+02, &
      5.5613193E+00, 8.19308E+01, 1.8510E+01, 6.3644E+02, &
      5.5891259E+00, 8.19942E+01, 1.8378E+01, 6.2882E+02, &
      5.6170716E+00, 8.20563E+01, 1.8247E+01, 6.2130E+02, &
      5.6451569E+00, 8.21170E+01, 1.8116E+01, 6.1383E+02, &
      5.6733827E+00, 8.21760E+01, 1.7985E+01, 6.0644E+02, &
      5.7017496E+00, 8.22334E+01, 1.7855E+01, 5.9913E+02, &
      5.7302584E+00, 8.22892E+01, 1.7726E+01, 5.9192E+02, &
      5.7589096E+00, 8.23437E+01, 1.7598E+01, 5.8479E+02, &
      5.7877042E+00, 8.23967E+01, 1.7471E+01, 5.7776E+02, &
      5.8166427E+00, 8.24484E+01, 1.7345E+01, 5.7081E+02, &
      5.8457259E+00, 8.24988E+01, 1.7219E+01, 5.6394E+02, &
      5.8749546E+00, 8.25480E+01, 1.7095E+01, 5.5716E+02, &
      5.9043293E+00, 8.25960E+01, 1.6972E+01, 5.5047E+02, &
      5.9338510E+00, 8.26429E+01, 1.6850E+01, 5.4385E+02, &
      5.9635202E+00, 8.26887E+01, 1.6729E+01, 5.3732E+02, &
      5.9933378E+00, 8.27336E+01, 1.6608E+01, 5.3087E+02, &
      6.0233045E+00, 8.29336E+01, 1.6486E+01, 5.2441E+02, &
      6.0534210E+00, 8.29761E+01, 1.6364E+01, 5.1801E+02, &
      6.0836882E+00, 8.30172E+01, 1.6242E+01, 5.1167E+02, &
      6.1141066E+00, 8.30569E+01, 1.6121E+01, 5.0541E+02, &
      6.1446771E+00, 8.30953E+01, 1.6002E+01, 4.9923E+02, &
      6.1754005E+00, 8.31324E+01, 1.5883E+01, 4.9313E+02, &
      6.2062775E+00, 8.31684E+01, 1.5765E+01, 4.8711E+02, &
      6.2373089E+00, 8.32032E+01, 1.5648E+01, 4.8117E+02, &
      6.2684954E+00, 8.32370E+01, 1.5533E+01, 4.7530E+02, &
      6.2998379E+00, 8.33781E+01, 1.5416E+01, 4.6944E+02, &
      6.3313371E+00, 8.34095E+01, 1.5299E+01, 4.6365E+02, &
      6.3629938E+00, 8.34397E+01, 1.5184E+01, 4.5794E+02, &
      6.3948088E+00, 8.34687E+01, 1.5070E+01, 4.5230E+02, &
      6.4267828E+00, 8.34966E+01, 1.4957E+01, 4.4674E+02, &
      6.4589167E+00, 8.35234E+01, 1.4844E+01, 4.4124E+02, &
      6.4912113E+00, 8.35492E+01, 1.4733E+01, 4.3581E+02, &
      6.5236674E+00, 8.35740E+01, 1.4622E+01, 4.3046E+02, &
      6.5562857E+00, 8.35978E+01, 1.4513E+01, 4.2517E+02, &
      6.5890671E+00, 8.36208E+01, 1.4404E+01, 4.1996E+02, &
      6.6220125E+00, 8.36428E+01, 1.4296E+01, 4.1480E+02, &
      6.6551225E+00, 8.36639E+01, 1.4189E+01, 4.0972E+02, &
      6.6883981E+00, 8.36842E+01, 1.4083E+01, 4.0470E+02, &
      6.7218401E+00, 8.37036E+01, 1.3978E+01, 3.9975E+02, &
      6.7554493E+00, 8.37223E+01, 1.3874E+01, 3.9486E+02, &
      6.7892266E+00, 8.37402E+01, 1.3771E+01, 3.9003E+02, &
      6.8231727E+00, 8.37573E+01, 1.3668E+01, 3.8527E+02, &
      6.8572886E+00, 8.37737E+01, 1.3567E+01, 3.8056E+02, &
      6.8915750E+00, 8.37895E+01, 1.3466E+01, 3.7592E+02, &
      6.9260329E+00, 8.38045E+01, 1.3366E+01, 3.7134E+02, &
      6.9606631E+00, 8.38189E+01, 1.3267E+01, 3.6682E+02, &
      6.9954664E+00, 8.38326E+01, 1.3169E+01, 3.6235E+02, &
      7.0304437E+00, 8.38457E+01, 1.3072E+01, 3.5795E+02, &
      7.0655959E+00, 8.38582E+01, 1.2975E+01, 3.5360E+02, &
      7.1009239E+00, 8.38701E+01, 1.2879E+01, 3.4930E+02, &
      7.1364285E+00, 8.38814E+01, 1.2785E+01, 3.4506E+02, &
      7.1721107E+00, 8.38922E+01, 1.2691E+01, 3.4088E+02, &
      7.2079712E+00, 8.39025E+01, 1.2597E+01, 3.3675E+02, &
      7.2440111E+00, 8.39123E+01, 1.2505E+01, 3.3268E+02, &
      7.2802311E+00, 8.39216E+01, 1.2413E+01, 3.2865E+02, &
      7.3166323E+00, 8.39304E+01, 1.2322E+01, 3.2468E+02, &
      7.3532155E+00, 8.40293E+01, 1.2231E+01, 3.2073E+02, &
      7.3899815E+00, 8.40375E+01, 1.2140E+01, 3.1682E+02, &
      7.4269314E+00, 8.40449E+01, 1.2050E+01, 3.1295E+02, &
      7.4640661E+00, 8.40518E+01, 1.1960E+01, 3.0914E+02, &
      7.5013864E+00, 8.40582E+01, 1.1871E+01, 3.0538E+02, &
      7.5388934E+00, 8.40639E+01, 1.1783E+01, 3.0166E+02, &
      7.5765878E+00, 8.40692E+01, 1.1696E+01, 2.9799E+02, &
      7.6144708E+00, 8.40739E+01, 1.1610E+01, 2.9438E+02, &
      7.6525431E+00, 8.40782E+01, 1.1524E+01, 2.9080E+02, &
      7.6908058E+00, 8.40820E+01, 1.1439E+01, 2.8728E+02, &
      7.7292599E+00, 8.40853E+01, 1.1355E+01, 2.8379E+02, &
      7.7679062E+00, 8.40882E+01, 1.1271E+01, 2.8036E+02, &
      7.8067457E+00, 8.40906E+01, 1.1188E+01, 2.7696E+02, &
      7.8457794E+00, 8.40927E+01, 1.1106E+01, 2.7362E+02, &
      7.8850083E+00, 8.40943E+01, 1.1024E+01, 2.7031E+02, &
      7.9244334E+00, 8.40955E+01, 1.0944E+01, 2.6705E+02, &
      7.9640555E+00, 8.40964E+01, 1.0864E+01, 2.6383E+02, &
      8.0038758E+00, 8.40969E+01, 1.0784E+01, 2.6065E+02, &
      8.0438952E+00, 8.40971E+01, 1.0705E+01, 2.5751E+02, &
      8.0841147E+00, 8.40969E+01, 1.0627E+01, 2.5441E+02, &
      8.1245352E+00, 8.40964E+01, 1.0550E+01, 2.5135E+02, &
      8.1651579E+00, 8.40956E+01, 1.0473E+01, 2.4833E+02, &
      8.2059837E+00, 8.40945E+01, 1.0397E+01, 2.4535E+02, &
      8.2470136E+00, 8.40931E+01, 1.0322E+01, 2.4241E+02, &
      8.2882487E+00, 8.40914E+01, 1.0247E+01, 2.3951E+02, &
      8.3296899E+00, 8.40895E+01, 1.0173E+01, 2.3664E+02, &
      8.3713384E+00, 8.40873E+01, 1.0099E+01, 2.3381E+02, &
      8.4131951E+00, 8.40849E+01, 1.0026E+01, 2.3102E+02, &
      8.4552610E+00, 8.40823E+01, 9.9540E+00, 2.2826E+02, &
      8.4975373E+00, 8.40795E+01, 9.8824E+00, 2.2554E+02, &
      8.5400250E+00, 8.40766E+01, 9.8113E+00, 2.2285E+02, &
      9.030794E+00, 8.44112E+01, 9.0167E+00, 1.9422E+02, &
      9.653919E+00, 8.42523E+01, 8.0629E+00, 1.6307E+02, &
      1.032004E+01, 8.39640E+01, 7.2189E+00, 1.3714E+02, &
      1.103212E+01, 8.35906E+01, 6.4723E+00, 1.1554E+02, &
      1.179334E+01, 8.31200E+01, 5.8069E+00, 9.7431E+01, &
      1.260708E+01, 8.25053E+01, 5.2175E+00, 8.2316E+01, &
      1.347697E+01, 8.16146E+01, 4.6884E+00, 6.9582E+01, &
      1.440688E+01, 7.98741E+01, 4.2089E+00, 5.8792E+01, &
      1.473058E+01, 7.85098E+01, 4.0619E+00, 5.5610E+01, &
      1.495604E+01, 7.59168E+01, 3.9650E+00, 5.3542E+01, &
      1.501617E+01, 7.28721E+01, 3.9398E+00, 5.3011E+01, &
      1.510636E+01, 7.59044E+01, 9.8591E+00, 1.2664E+02, &
      1.533182E+01, 7.85922E+01, 9.6263E+00, 1.2190E+02, &
      1.540095E+01, 7.89881E+01, 9.5566E+00, 1.2049E+02, &
      1.646362E+01, 8.12238E+01, 8.5482E+00, 1.0113E+02, &
      1.754837E+01, 8.08361E+01, 7.6554E+00, 8.5253E+01, &
      1.759961E+01, 8.07123E+01, 7.6167E+00, 8.4590E+01, &
      1.781697E+01, 7.96108E+01, 7.4561E+00, 8.1850E+01, &
      1.788859E+01, 7.80761E+01, 7.4043E+00, 8.0973E+01, &
      1.799603E+01, 7.96241E+01, 1.0358E+01, 1.1146E+02, &
      1.826622E+01, 8.09327E+01, 1.0107E+01, 1.0721E+02, &
      1.854580E+01, 8.10130E+01, 9.8572E+00, 1.0305E+02, &
      1.862036E+01, 8.03858E+01, 9.7923E+00, 1.0197E+02, &
      1.873219E+01, 8.13617E+01, 1.1222E+01, 1.1577E+02, &
      1.881398E+01, 8.18093E+01, 1.1149E+01, 1.1452E+02, &
      1.901178E+01, 8.24782E+01, 1.0972E+01, 1.1158E+02, &
      2.011215E+01, 8.42648E+01, 1.0061E+01, 9.6892E+01, &
      2.149988E+01, 8.53656E+01, 9.0600E+00, 8.1817E+01, &
      2.298338E+01, 8.60511E+01, 8.1590E+00, 6.9104E+01, &
      2.456923E+01, 8.65130E+01, 7.3481E+00, 5.8381E+01, &
      2.626450E+01, 8.68367E+01, 6.6190E+00, 4.9340E+01, &
      2.807676E+01, 8.70718E+01, 5.9278E+00, 4.1477E+01, &
      3.001405E+01, 8.72035E+01, 5.3022E+00, 3.4834E+01, &
      3.208502E+01, 8.73281E+01, 4.7340E+00, 2.9214E+01, &
      3.429889E+01, 8.73466E+01, 4.2308E+00, 2.4530E+01, &
      3.666551E+01, 8.73712E+01, 3.7818E+00, 2.0609E+01, &
      3.919543E+01, 8.73621E+01, 3.3698E+00, 1.7269E+01, &
      4.189992E+01, 8.73029E+01, 3.0024E+00, 1.4477E+01, &
      4.479101E+01, 8.72287E+01, 2.6771E+00, 1.2149E+01, &
      4.788159E+01, 8.71447E+01, 2.3889E+00, 1.0209E+01, &
      5.118542E+01, 8.70540E+01, 2.1333E+00, 8.5897E+00, &
      5.471721E+01, 8.69582E+01, 1.9065E+00, 7.2364E+00, &
      5.849270E+01, 8.68580E+01, 1.7051E+00, 6.1044E+00, &
      6.252870E+01, 8.67531E+01, 1.5260E+00, 5.1566E+00, &
      6.684318E+01, 8.66420E+01, 1.3656E+00, 4.3591E+00, &
      7.145536E+01, 8.65199E+01, 1.2215E+00, 3.6866E+00, &
      7.638578E+01, 8.63810E+01, 1.0936E+00, 3.1234E+00, &
      8.165640E+01, 8.62161E+01, 9.7562E-01, 2.6414E+00, &
      8.729069E+01, 8.59879E+01, 8.6330E-01, 2.2213E+00, &
      9.331374E+01, 8.56091E+01, 7.6448E-01, 1.8725E+00, &
      9.911426E+01, 8.46796E+01, 6.8536E-01, 1.6080E+00, &
      9.975239E+01, 8.44086E+01, 6.7746E-01, 1.5824E+00, &
      1.006313E+02, 8.36723E+01, 6.6682E-01, 1.5481E+00, &
      1.010359E+02, 8.24676E+01, 6.6201E-01, 1.5326E+00, &
      1.016427E+02, 8.37230E+01, 3.0253E+00, 5.9106E+00, &
      1.031597E+02, 8.48204E+01, 2.9566E+00, 5.6971E+00, &
      1.066353E+02, 8.56341E+01, 2.8079E+00, 5.2465E+00, &
      1.139931E+02, 8.63148E+01, 2.5290E+00, 4.4433E+00, &
      1.218587E+02, 8.66638E+01, 2.2750E+00, 3.7610E+00, &
      1.302669E+02, 8.68793E+01, 2.0454E+00, 3.1842E+00, &
      1.392553E+02, 8.70218E+01, 1.8386E+00, 2.6976E+00, &
      1.488640E+02, 8.71190E+01, 1.6513E+00, 2.2856E+00, &
      1.591356E+02, 8.71836E+01, 1.4820E+00, 1.9374E+00, &
      1.701159E+02, 8.72258E+01, 1.3301E+00, 1.6443E+00, &
      1.818539E+02, 8.72519E+01, 1.1938E+00, 1.3976E+00, &
      1.944018E+02, 8.72664E+01, 1.0716E+00, 1.1899E+00, &
      2.078156E+02, 8.72723E+01, 9.6198E-01, 1.0149E+00, &
      2.221548E+02, 8.72719E+01, 8.6395E-01, 8.6772E-01, &
      2.374835E+02, 8.72673E+01, 7.7605E-01, 7.4362E-01, &
      2.538699E+02, 8.72596E+01, 6.9718E-01, 6.3889E-01, &
      2.713869E+02, 8.72496E+01, 6.2639E-01, 5.5043E-01, &
      2.901126E+02, 8.72382E+01, 5.6285E-01, 4.7565E-01, &
      3.101304E+02, 8.72258E+01, 5.0581E-01, 4.1238E-01, &
      3.315294E+02, 8.72130E+01, 4.5460E-01, 3.5877E-01, &
      3.544049E+02, 8.72000E+01, 4.0861E-01, 3.1330E-01, &
      3.788588E+02, 8.71870E+01, 3.6731E-01, 2.7468E-01, &
      4.050001E+02, 8.71742E+01, 3.3021E-01, 2.4182E-01, &
      4.329451E+02, 8.71619E+01, 2.9688E-01, 2.1381E-01]
   case(88)
      frel = -1.3326E+00
      allocate(nist_tab(3044))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 8.67455E+00, 0.0000E+00, 1.2723E-04, &
      1.142761E-02, 8.41850E+00, 0.0000E+00, 1.7555E-04, &
      1.221612E-02, 8.09757E+00, 0.0000E+00, 2.4093E-04, &
      1.305903E-02, 7.68553E+00, 0.0000E+00, 3.2890E-04, &
      1.396010E-02, 7.13831E+00, 0.0000E+00, 4.4664E-04, &
      1.492335E-02, 6.37387E+00, 0.0000E+00, 6.0336E-04, &
      1.595306E-02, 5.21453E+00, 0.0000E+00, 8.1083E-04, &
      1.705382E-02, 3.15972E+00, 0.0000E+00, 1.0840E-03, &
      1.823053E-02, -2.46801E+00, 0.0000E+00, 1.4418E-03, &
      1.842400E-02, -4.82521E+00, 0.0000E+00, 1.5076E-03, &
      1.870600E-02, -1.34118E+01, 0.0000E+00, 1.6073E-03, &
      1.878120E-02, -2.41391E+01, 0.0000E+00, 1.6346E-03, &
      1.889400E-02, -1.22995E+01, 2.1336E+01, 2.1024E+05, &
      1.917600E-02, -1.84644E+00, 2.0138E+01, 1.9552E+05, &
      1.948844E-02, 2.84485E+00, 1.8909E+01, 1.8064E+05, &
      2.083314E-02, 1.05589E+01, 1.4580E+01, 1.3030E+05, &
      2.227063E-02, 1.31235E+01, 1.1391E+01, 9.5222E+04, &
      2.380730E-02, 1.40976E+01, 9.0734E+00, 7.0955E+04, &
      2.545001E-02, 1.43662E+01, 7.3501E+00, 5.3769E+04, &
      2.720606E-02, 1.42663E+01, 6.0430E+00, 4.1353E+04, &
      2.908327E-02, 1.39478E+01, 5.0333E+00, 3.2220E+04, &
      3.109002E-02, 1.34779E+01, 4.2398E+00, 2.5389E+04, &
      3.323523E-02, 1.28811E+01, 3.6055E+00, 2.0197E+04, &
      3.552846E-02, 1.21549E+01, 3.0903E+00, 1.6194E+04, &
      3.797993E-02, 1.12753E+01, 2.6654E+00, 1.3066E+04, &
      4.060054E-02, 1.01903E+01, 2.3101E+00, 1.0593E+04, &
      4.263000E-02, 9.18721E+00, 2.0856E+00, 9.1084E+03, &
      4.328250E-02, 8.75905E+00, 2.0210E+00, 8.6930E+03, &
      4.340198E-02, 8.63305E+00, 2.0095E+00, 8.6198E+03, &
      4.345650E-02, 8.53660E+00, 2.0043E+00, 8.5867E+03, &
      4.371750E-02, 8.55332E+00, 2.2479E+00, 9.5727E+03, &
      4.437000E-02, 8.34232E+00, 2.1712E+00, 9.1102E+03, &
      4.639671E-02, 7.29145E+00, 1.9583E+00, 7.8579E+03, &
      4.959809E-02, 5.12149E+00, 1.6860E+00, 6.3286E+03, &
      5.302035E-02, 2.01512E+00, 1.4631E+00, 5.1378E+03, &
      5.667876E-02, -2.73805E+00, 1.2779E+00, 4.1976E+03, &
      6.058959E-02, -1.11748E+01, 1.1219E+00, 3.4474E+03, &
      6.477028E-02, -3.29070E+01, 9.8972E-01, 2.8449E+03, &
      6.585600E-02, -4.74805E+01, 9.5999E-01, 2.7140E+03, &
      6.686400E-02, -8.44775E+01, 9.3385E-01, 2.6003E+03, &
      6.713280E-02, -1.29793E+02, 9.2711E-01, 2.5712E+03, &
      6.753600E-02, -8.15129E+01, 9.0676E+01, 2.4997E+05, &
      6.854400E-02, -3.86979E+01, 8.8107E+01, 2.3931E+05, &
      6.923942E-02, -2.51830E+01, 8.6403E+01, 2.3233E+05, &
      7.401695E-02, 1.67949E+01, 7.6027E+01, 1.9123E+05, &
      7.912411E-02, 3.92032E+01, 6.2841E+01, 1.4786E+05, &
      8.458368E-02, 5.05308E+01, 4.7007E+01, 1.0347E+05, &
      9.041995E-02, 5.39992E+01, 3.2908E+01, 6.7758E+04, &
      9.665893E-02, 5.30533E+01, 2.2176E+01, 4.2713E+04, &
      1.033284E-01, 5.01907E+01, 1.4723E+01, 2.6528E+04, &
      1.104581E-01, 4.68363E+01, 9.8130E+00, 1.6541E+04, &
      1.180797E-01, 4.36472E+01, 6.6634E+00, 1.0507E+04, &
      1.262272E-01, 4.08441E+01, 4.6603E+00, 6.8743E+03, &
      1.349368E-01, 3.84318E+01, 3.3818E+00, 4.6667E+03, &
      1.442475E-01, 3.62828E+01, 2.5563E+00, 3.3002E+03, &
      1.497440E-01, 3.50009E+01, 2.2257E+00, 2.7682E+03, &
      1.520360E-01, 3.41577E+01, 2.1119E+00, 2.5871E+03, &
      1.526472E-01, 3.34855E+01, 2.0835E+00, 2.5422E+03, &
      1.535640E-01, 3.39519E+01, 3.1829E+00, 3.8598E+03, &
      1.542005E-01, 3.40869E+01, 3.1487E+00, 3.8027E+03, &
      1.558560E-01, 3.41482E+01, 3.0639E+00, 3.6610E+03, &
      1.648404E-01, 3.35149E+01, 2.6858E+00, 3.0346E+03, &
      1.762144E-01, 3.24912E+01, 2.3545E+00, 2.4889E+03, &
      1.883732E-01, 3.14074E+01, 2.1264E+00, 2.1031E+03, &
      1.963920E-01, 3.05976E+01, 2.0219E+00, 1.9185E+03, &
      1.993980E-01, 3.00729E+01, 1.9897E+00, 1.8595E+03, &
      2.001996E-01, 2.96323E+01, 1.9817E+00, 1.8446E+03, &
      2.013709E-01, 2.99598E+01, 2.7326E+00, 2.5281E+03, &
      2.014020E-01, 2.99658E+01, 2.7320E+00, 2.5271E+03, &
      2.044080E-01, 3.01560E+01, 2.6690E+00, 2.4327E+03, &
      2.152655E-01, 2.97735E+01, 2.4791E+00, 2.1461E+03, &
      2.301188E-01, 2.89357E+01, 2.2949E+00, 1.8589E+03, &
      2.459970E-01, 2.78931E+01, 2.1714E+00, 1.6458E+03, &
      2.493120E-01, 2.76465E+01, 2.1525E+00, 1.6099E+03, &
      2.531280E-01, 2.73080E+01, 2.1331E+00, 1.5715E+03, &
      2.541456E-01, 2.71421E+01, 2.1284E+00, 1.5617E+03, &
      2.556720E-01, 2.71336E+01, 2.3106E+00, 1.6852E+03, &
      2.594880E-01, 2.69504E+01, 2.2969E+00, 1.6507E+03, &
      2.629708E-01, 2.67324E+01, 2.2861E+00, 1.6212E+03, &
      2.811158E-01, 2.53662E+01, 2.2513E+00, 1.4939E+03, &
      2.929220E-01, 2.42392E+01, 2.2434E+00, 1.4291E+03, &
      2.974055E-01, 2.36634E+01, 2.2427E+00, 1.4072E+03, &
      2.986011E-01, 2.33898E+01, 2.2426E+00, 1.4016E+03, &
      3.005128E-01, 2.33078E+01, 2.5373E+00, 1.5752E+03, &
      3.048780E-01, 2.29194E+01, 2.6178E+00, 1.6020E+03, &
      3.212482E-01, 2.09903E+01, 3.1454E+00, 1.8265E+03, &
      3.434143E-01, 1.79636E+01, 4.8261E+00, 2.6204E+03, &
      3.671099E-01, 1.49829E+01, 8.0373E+00, 4.0803E+03, &
      3.924405E-01, 1.29109E+01, 1.2796E+01, 6.0750E+03, &
      4.195189E-01, 1.25181E+01, 1.8539E+01, 8.2323E+03, &
      4.484657E-01, 1.40880E+01, 2.4348E+01, 1.0113E+04, &
      4.794098E-01, 1.73436E+01, 2.9352E+01, 1.1405E+04, &
      5.0000000E-01, 1.99917E+01, 3.1830E+01, 1.1858E+04, &
      5.0250000E-01, 2.03224E+01, 3.2085E+01, 1.1893E+04, &
      5.0501250E-01, 2.06558E+01, 3.2331E+01, 1.1925E+04, &
      5.0753756E-01, 2.09916E+01, 3.2568E+01, 1.1953E+04, &
      5.1007525E-01, 2.13295E+01, 3.2798E+01, 1.1977E+04, &
      5.1262563E-01, 2.16690E+01, 3.3018E+01, 1.1998E+04, &
      5.1518875E-01, 2.20100E+01, 3.3230E+01, 1.2015E+04, &
      5.1776470E-01, 2.23520E+01, 3.3433E+01, 1.2028E+04, &
      5.2035352E-01, 2.26946E+01, 3.3628E+01, 1.2038E+04, &
      5.2295529E-01, 2.30376E+01, 3.3815E+01, 1.2045E+04, &
      5.2557007E-01, 2.33805E+01, 3.3993E+01, 1.2048E+04, &
      5.2819792E-01, 2.37231E+01, 3.4163E+01, 1.2048E+04, &
      5.3083891E-01, 2.40650E+01, 3.4324E+01, 1.2045E+04, &
      5.3349310E-01, 2.44057E+01, 3.4477E+01, 1.2038E+04, &
      5.3616057E-01, 2.47449E+01, 3.4622E+01, 1.2029E+04, &
      5.3884137E-01, 2.50822E+01, 3.4758E+01, 1.2016E+04, &
      5.4153558E-01, 2.54172E+01, 3.4887E+01, 1.2000E+04, &
      5.4424325E-01, 2.57495E+01, 3.5008E+01, 1.1982E+04, &
      5.4696447E-01, 2.60786E+01, 3.5121E+01, 1.1961E+04, &
      5.4969929E-01, 2.64041E+01, 3.5226E+01, 1.1937E+04, &
      5.5244779E-01, 2.67253E+01, 3.5323E+01, 1.1911E+04, &
      5.5521003E-01, 2.70418E+01, 3.5413E+01, 1.1882E+04, &
      5.5798608E-01, 2.73529E+01, 3.5495E+01, 1.1850E+04, &
      5.6077601E-01, 2.76579E+01, 3.5570E+01, 1.1816E+04, &
      5.6357989E-01, 2.79561E+01, 3.5638E+01, 1.1780E+04, &
      5.6639779E-01, 2.82464E+01, 3.5699E+01, 1.1741E+04, &
      5.6922978E-01, 2.85278E+01, 3.5752E+01, 1.1700E+04, &
      5.7207593E-01, 2.87989E+01, 3.5799E+01, 1.1657E+04, &
      5.7493630E-01, 2.90581E+01, 3.5839E+01, 1.1612E+04, &
      5.7781099E-01, 2.93031E+01, 3.5873E+01, 1.1565E+04, &
      5.8070004E-01, 2.95311E+01, 3.5900E+01, 1.1517E+04, &
      5.8360354E-01, 2.97381E+01, 3.5920E+01, 1.1466E+04, &
      5.8652156E-01, 2.99182E+01, 3.5935E+01, 1.1414E+04, &
      5.8945417E-01, 3.00619E+01, 3.5943E+01, 1.1360E+04, &
      5.9240144E-01, 3.01531E+01, 3.5945E+01, 1.1304E+04, &
      5.9536345E-01, 3.01592E+01, 3.5942E+01, 1.1247E+04, &
      5.9834026E-01, 2.99953E+01, 3.5933E+01, 1.1188E+04, &
      6.0133196E-01, 2.92551E+01, 3.5918E+01, 1.1128E+04, &
      6.0233476E-01, 2.81756E+01, 3.5912E+01, 1.1107E+04, &
      6.0306523E-01, 2.82393E+01, 3.8694E+01, 1.1953E+04, &
      6.0433862E-01, 2.96827E+01, 3.8694E+01, 1.1928E+04, &
      6.0736032E-01, 3.08787E+01, 3.8691E+01, 1.1867E+04, &
      6.1039712E-01, 3.15882E+01, 3.8683E+01, 1.1806E+04, &
      6.1344910E-01, 3.21403E+01, 3.8670E+01, 1.1743E+04, &
      6.1651635E-01, 3.26067E+01, 3.8653E+01, 1.1680E+04, &
      6.1959893E-01, 3.30128E+01, 3.8631E+01, 1.1615E+04, &
      6.2269693E-01, 3.33668E+01, 3.8605E+01, 1.1550E+04, &
      6.2581041E-01, 3.36656E+01, 3.8574E+01, 1.1483E+04, &
      6.2893946E-01, 3.38905E+01, 3.8539E+01, 1.1416E+04, &
      6.3208416E-01, 3.39760E+01, 3.8500E+01, 1.1347E+04, &
      6.3524458E-01, 3.34274E+01, 3.8457E+01, 1.1278E+04, &
      6.3551339E-01, 3.31788E+01, 3.8453E+01, 1.1273E+04, &
      6.3628665E-01, 3.32693E+01, 4.0113E+01, 1.1745E+04, &
      6.3842080E-01, 3.45166E+01, 4.0092E+01, 1.1699E+04, &
      6.4161291E-01, 3.53225E+01, 4.0056E+01, 1.1631E+04, &
      6.4482097E-01, 3.59249E+01, 4.0018E+01, 1.1562E+04, &
      6.4804508E-01, 3.64485E+01, 3.9976E+01, 1.1492E+04, &
      6.5128530E-01, 3.69281E+01, 3.9931E+01, 1.1422E+04, &
      6.5454173E-01, 3.73785E+01, 3.9883E+01, 1.1352E+04, &
      6.5781444E-01, 3.78076E+01, 3.9832E+01, 1.1281E+04, &
      6.6110351E-01, 3.82202E+01, 3.9778E+01, 1.1210E+04, &
      6.6440903E-01, 3.86194E+01, 3.9721E+01, 1.1138E+04, &
      6.6773107E-01, 3.90072E+01, 3.9661E+01, 1.1066E+04, &
      6.7106973E-01, 3.93852E+01, 3.9599E+01, 1.0994E+04, &
      6.7442508E-01, 3.97542E+01, 3.9533E+01, 1.0921E+04, &
      6.7779720E-01, 4.01151E+01, 3.9465E+01, 1.0848E+04, &
      6.8118619E-01, 4.04686E+01, 3.9394E+01, 1.0775E+04, &
      6.8459212E-01, 4.08152E+01, 3.9320E+01, 1.0701E+04, &
      6.8801508E-01, 4.11552E+01, 3.9243E+01, 1.0627E+04, &
      6.9145515E-01, 4.14891E+01, 3.9165E+01, 1.0553E+04, &
      6.9491243E-01, 4.18173E+01, 3.9084E+01, 1.0479E+04, &
      6.9838699E-01, 4.21399E+01, 3.9000E+01, 1.0405E+04, &
      7.0187893E-01, 4.24572E+01, 3.8915E+01, 1.0331E+04, &
      7.0538832E-01, 4.27696E+01, 3.8827E+01, 1.0256E+04, &
      7.0891526E-01, 4.30769E+01, 3.8737E+01, 1.0181E+04, &
      7.1245984E-01, 4.33794E+01, 3.8645E+01, 1.0107E+04, &
      7.1602214E-01, 4.36772E+01, 3.8551E+01, 1.0032E+04, &
      7.1960225E-01, 4.39702E+01, 3.8454E+01, 9.9573E+03, &
      7.2320026E-01, 4.42586E+01, 3.8356E+01, 9.8825E+03, &
      7.2681626E-01, 4.45425E+01, 3.8256E+01, 9.8078E+03, &
      7.3045034E-01, 4.48219E+01, 3.8154E+01, 9.7331E+03, &
      7.3410260E-01, 4.50973E+01, 3.8050E+01, 9.6584E+03, &
      7.3777311E-01, 4.53681E+01, 3.7945E+01, 9.5838E+03, &
      7.4146197E-01, 4.56343E+01, 3.7838E+01, 9.5093E+03, &
      7.4516928E-01, 4.58962E+01, 3.7729E+01, 9.4349E+03, &
      7.4889513E-01, 4.61537E+01, 3.7619E+01, 9.3606E+03, &
      7.5263961E-01, 4.64069E+01, 3.7507E+01, 9.2865E+03, &
      7.5640280E-01, 4.66557E+01, 3.7394E+01, 9.2126E+03, &
      7.6018482E-01, 4.69002E+01, 3.7280E+01, 9.1389E+03, &
      7.6398574E-01, 4.71418E+01, 3.7165E+01, 9.0655E+03, &
      7.6780567E-01, 4.73777E+01, 3.7049E+01, 8.9922E+03, &
      7.7164470E-01, 4.76093E+01, 3.6931E+01, 8.9192E+03, &
      7.7550292E-01, 4.78366E+01, 3.6813E+01, 8.8465E+03, &
      7.7938044E-01, 4.80595E+01, 3.6694E+01, 8.7741E+03, &
      7.8327734E-01, 4.82780E+01, 3.6573E+01, 8.7019E+03, &
      7.8719373E-01, 4.84920E+01, 3.6453E+01, 8.6301E+03, &
      7.9112969E-01, 4.87015E+01, 3.6331E+01, 8.5586E+03, &
      7.9508534E-01, 4.89063E+01, 3.6208E+01, 8.4874E+03, &
      7.9906077E-01, 4.91065E+01, 3.6085E+01, 8.4166E+03, &
      8.0305607E-01, 4.93018E+01, 3.5962E+01, 8.3461E+03, &
      8.0707135E-01, 4.94921E+01, 3.5838E+01, 8.2760E+03, &
      8.1110671E-01, 4.96771E+01, 3.5713E+01, 8.2063E+03, &
      8.1516224E-01, 4.98568E+01, 3.5588E+01, 8.1370E+03, &
      8.1923806E-01, 5.00306E+01, 3.5462E+01, 8.0681E+03, &
      8.2333425E-01, 5.01983E+01, 3.5336E+01, 7.9996E+03, &
      8.2745092E-01, 5.03593E+01, 3.5210E+01, 7.9314E+03, &
      8.3158817E-01, 5.05131E+01, 3.5084E+01, 7.8638E+03, &
      8.3574611E-01, 5.06587E+01, 3.4957E+01, 7.7965E+03, &
      8.3992484E-01, 5.07952E+01, 3.4831E+01, 7.7297E+03, &
      8.4412447E-01, 5.09210E+01, 3.4704E+01, 7.6633E+03, &
      8.4834509E-01, 5.10341E+01, 3.4577E+01, 7.5974E+03, &
      8.5258682E-01, 5.11313E+01, 3.4450E+01, 7.5319E+03, &
      8.5684975E-01, 5.12079E+01, 3.4322E+01, 7.4669E+03, &
      8.6113400E-01, 5.12561E+01, 3.4195E+01, 7.4024E+03, &
      8.6543967E-01, 5.12614E+01, 3.4068E+01, 7.3383E+03, &
      8.6976687E-01, 5.11919E+01, 3.3941E+01, 7.2747E+03, &
      8.7411570E-01, 5.09533E+01, 3.3815E+01, 7.2116E+03, &
      8.7813301E-01, 5.00116E+01, 3.3698E+01, 7.1540E+03, &
      8.7848628E-01, 4.97184E+01, 3.3688E+01, 7.1489E+03, &
      8.8006703E-01, 5.00925E+01, 3.5771E+01, 7.5767E+03, &
      8.8287871E-01, 5.11320E+01, 3.5691E+01, 7.5357E+03, &
      8.8729310E-01, 5.18370E+01, 3.5565E+01, 7.4718E+03, &
      8.9172957E-01, 5.23086E+01, 3.5437E+01, 7.4081E+03, &
      8.9618822E-01, 5.26899E+01, 3.5310E+01, 7.3449E+03, &
      9.0066916E-01, 5.30796E+01, 3.5183E+01, 7.2822E+03, &
      9.0517250E-01, 5.33810E+01, 3.5056E+01, 7.2199E+03, &
      9.0969837E-01, 5.36606E+01, 3.4929E+01, 7.1581E+03, &
      9.1424686E-01, 5.39237E+01, 3.4802E+01, 7.0968E+03, &
      9.1881809E-01, 5.41740E+01, 3.4676E+01, 7.0360E+03, &
      9.2341218E-01, 5.44138E+01, 3.4550E+01, 6.9757E+03, &
      9.2802924E-01, 5.46448E+01, 3.4425E+01, 6.9158E+03, &
      9.3266939E-01, 5.48684E+01, 3.4299E+01, 6.8565E+03, &
      9.3733274E-01, 5.50855E+01, 3.4174E+01, 6.7976E+03, &
      9.4201940E-01, 5.52969E+01, 3.4049E+01, 6.7392E+03, &
      9.4672950E-01, 5.55032E+01, 3.3925E+01, 6.6813E+03, &
      9.5146315E-01, 5.57050E+01, 3.3801E+01, 6.6239E+03, &
      9.5622046E-01, 5.59026E+01, 3.3678E+01, 6.5669E+03, &
      9.6100156E-01, 5.60964E+01, 3.3554E+01, 6.5105E+03, &
      9.6580657E-01, 5.62868E+01, 3.3432E+01, 6.4545E+03, &
      9.7063560E-01, 5.64740E+01, 3.3310E+01, 6.3991E+03, &
      9.7548878E-01, 5.66583E+01, 3.3188E+01, 6.3441E+03, &
      9.8036623E-01, 5.68399E+01, 3.3067E+01, 6.2895E+03, &
      9.8526806E-01, 5.70189E+01, 3.2946E+01, 6.2355E+03, &
      9.9019440E-01, 5.71956E+01, 3.2825E+01, 6.1819E+03, &
      9.9514537E-01, 5.73702E+01, 3.2705E+01, 6.1288E+03, &
      1.0001211E+00, 5.75463E+01, 3.2586E+01, 6.0761E+03, &
      1.0051217E+00, 5.77446E+01, 3.2450E+01, 6.0209E+03, &
      1.0101473E+00, 5.79277E+01, 3.2315E+01, 5.9661E+03, &
      1.0151980E+00, 5.81027E+01, 3.2180E+01, 5.9117E+03, &
      1.0202740E+00, 5.82709E+01, 3.2042E+01, 5.8571E+03, &
      1.0253754E+00, 5.84309E+01, 3.1896E+01, 5.8016E+03, &
      1.0305023E+00, 5.85829E+01, 3.1750E+01, 5.7465E+03, &
      1.0356548E+00, 5.87255E+01, 3.1605E+01, 5.6919E+03, &
      1.0408331E+00, 5.88563E+01, 3.1460E+01, 5.6377E+03, &
      1.0460372E+00, 5.89694E+01, 3.1316E+01, 5.5840E+03, &
      1.0512674E+00, 5.90469E+01, 3.1172E+01, 5.5307E+03, &
      1.0563415E+00, 5.89730E+01, 3.1033E+01, 5.4798E+03, &
      1.0565238E+00, 5.89564E+01, 3.1028E+01, 5.4779E+03, &
      1.0588586E+00, 5.90552E+01, 3.1408E+01, 5.5327E+03, &
      1.0618064E+00, 5.93130E+01, 3.1327E+01, 5.5033E+03, &
      1.0671154E+00, 5.95829E+01, 3.1184E+01, 5.4509E+03, &
      1.0724510E+00, 5.97980E+01, 3.1040E+01, 5.3990E+03, &
      1.0778132E+00, 5.99910E+01, 3.0893E+01, 5.3468E+03, &
      1.0832023E+00, 6.01701E+01, 3.0743E+01, 5.2944E+03, &
      1.0886183E+00, 6.03396E+01, 3.0593E+01, 5.2425E+03, &
      1.0940614E+00, 6.05016E+01, 3.0443E+01, 5.1911E+03, &
      1.0995317E+00, 6.06574E+01, 3.0294E+01, 5.1401E+03, &
      1.1050294E+00, 6.08076E+01, 3.0146E+01, 5.0896E+03, &
      1.1105545E+00, 6.09527E+01, 2.9998E+01, 5.0395E+03, &
      1.1161073E+00, 6.10936E+01, 2.9852E+01, 4.9901E+03, &
      1.1216878E+00, 6.12307E+01, 2.9706E+01, 4.9412E+03, &
      1.1272963E+00, 6.13642E+01, 2.9561E+01, 4.8927E+03, &
      1.1329328E+00, 6.14942E+01, 2.9416E+01, 4.8447E+03, &
      1.1385974E+00, 6.16206E+01, 2.9273E+01, 4.7972E+03, &
      1.1442904E+00, 6.17436E+01, 2.9129E+01, 4.7501E+03, &
      1.1500119E+00, 6.18630E+01, 2.8987E+01, 4.7034E+03, &
      1.1557619E+00, 6.19786E+01, 2.8845E+01, 4.6572E+03, &
      1.1615407E+00, 6.20901E+01, 2.8703E+01, 4.6115E+03, &
      1.1673484E+00, 6.21972E+01, 2.8563E+01, 4.5662E+03, &
      1.1731852E+00, 6.22989E+01, 2.8423E+01, 4.5213E+03, &
      1.1790511E+00, 6.23941E+01, 2.8284E+01, 4.4769E+03, &
      1.1849464E+00, 6.24806E+01, 2.8145E+01, 4.4329E+03, &
      1.1908711E+00, 6.25541E+01, 2.8007E+01, 4.3894E+03, &
      1.1968254E+00, 6.26045E+01, 2.7862E+01, 4.3450E+03, &
      1.2028096E+00, 6.25965E+01, 2.7686E+01, 4.2963E+03, &
      1.2069862E+00, 6.24346E+01, 2.7566E+01, 4.2629E+03, &
      1.2088236E+00, 6.22684E+01, 2.8070E+01, 4.3341E+03, &
      1.2098138E+00, 6.25012E+01, 2.8041E+01, 4.3262E+03, &
      1.2148677E+00, 6.28656E+01, 2.7897E+01, 4.2861E+03, &
      1.2209421E+00, 6.30927E+01, 2.7725E+01, 4.2386E+03, &
      1.2270468E+00, 6.32691E+01, 2.7555E+01, 4.1918E+03, &
      1.2331820E+00, 6.34222E+01, 2.7386E+01, 4.1455E+03, &
      1.2393479E+00, 6.35611E+01, 2.7219E+01, 4.0998E+03, &
      1.2455447E+00, 6.36899E+01, 2.7053E+01, 4.0547E+03, &
      1.2517724E+00, 6.38109E+01, 2.6888E+01, 4.0102E+03, &
      1.2580312E+00, 6.39254E+01, 2.6725E+01, 3.9662E+03, &
      1.2643214E+00, 6.40345E+01, 2.6562E+01, 3.9225E+03, &
      1.2706430E+00, 6.41386E+01, 2.6399E+01, 3.8792E+03, &
      1.2769962E+00, 6.42386E+01, 2.6238E+01, 3.8364E+03, &
      1.2833812E+00, 6.43348E+01, 2.6078E+01, 3.7942E+03, &
      1.2897981E+00, 6.44275E+01, 2.5920E+01, 3.7526E+03, &
      1.2962471E+00, 6.45170E+01, 2.5763E+01, 3.7114E+03, &
      1.3027283E+00, 6.46035E+01, 2.5607E+01, 3.6708E+03, &
      1.3092420E+00, 6.46871E+01, 2.5453E+01, 3.6306E+03, &
      1.3157882E+00, 6.47681E+01, 2.5300E+01, 3.5910E+03, &
      1.3223671E+00, 6.48465E+01, 2.5148E+01, 3.5518E+03, &
      1.3289790E+00, 6.49226E+01, 2.4998E+01, 3.5132E+03, &
      1.3356239E+00, 6.49964E+01, 2.4849E+01, 3.4750E+03, &
      1.3423020E+00, 6.50681E+01, 2.4701E+01, 3.4373E+03, &
      1.3490135E+00, 6.51377E+01, 2.4554E+01, 3.4000E+03, &
      1.3557586E+00, 6.52054E+01, 2.4409E+01, 3.3632E+03, &
      1.3625374E+00, 6.52713E+01, 2.4265E+01, 3.3269E+03, &
      1.3693500E+00, 6.53355E+01, 2.4122E+01, 3.2910E+03, &
      1.3761968E+00, 6.53980E+01, 2.3981E+01, 3.2555E+03, &
      1.3830778E+00, 6.54587E+01, 2.3839E+01, 3.2204E+03, &
      1.3899932E+00, 6.55176E+01, 2.3699E+01, 3.1857E+03, &
      1.3969431E+00, 6.55748E+01, 2.3560E+01, 3.1513E+03, &
      1.4039278E+00, 6.56303E+01, 2.3422E+01, 3.1174E+03, &
      1.4109475E+00, 6.56843E+01, 2.3285E+01, 3.0840E+03, &
      1.4180022E+00, 6.57367E+01, 2.3150E+01, 3.0509E+03, &
      1.4250922E+00, 6.57877E+01, 2.3015E+01, 3.0182E+03, &
      1.4322177E+00, 6.58373E+01, 2.2882E+01, 2.9859E+03, &
      1.4393788E+00, 6.58856E+01, 2.2750E+01, 2.9540E+03, &
      1.4465757E+00, 6.59325E+01, 2.2618E+01, 2.9225E+03, &
      1.4538086E+00, 6.59782E+01, 2.2488E+01, 2.8914E+03, &
      1.4610776E+00, 6.60228E+01, 2.2359E+01, 2.8606E+03, &
      1.4683830E+00, 6.60662E+01, 2.2231E+01, 2.8302E+03, &
      1.4757249E+00, 6.61085E+01, 2.2104E+01, 2.8002E+03, &
      1.4831035E+00, 6.61498E+01, 2.1978E+01, 2.7705E+03, &
      1.4905190E+00, 6.61899E+01, 2.1852E+01, 2.7410E+03, &
      1.4979716E+00, 6.62285E+01, 2.1725E+01, 2.7117E+03, &
      1.5054615E+00, 6.62658E+01, 2.1600E+01, 2.6828E+03, &
      1.5129888E+00, 6.63019E+01, 2.1476E+01, 2.6542E+03, &
      1.5205537E+00, 6.63367E+01, 2.1352E+01, 2.6260E+03, &
      1.5281565E+00, 6.63703E+01, 2.1230E+01, 2.5980E+03, &
      1.5357973E+00, 6.64027E+01, 2.1108E+01, 2.5705E+03, &
      1.5434763E+00, 6.64340E+01, 2.0988E+01, 2.5432E+03, &
      1.5511937E+00, 6.64643E+01, 2.0868E+01, 2.5163E+03, &
      1.5589496E+00, 6.64934E+01, 2.0749E+01, 2.4896E+03, &
      1.5667444E+00, 6.65216E+01, 2.0632E+01, 2.4633E+03, &
      1.5745781E+00, 6.65488E+01, 2.0515E+01, 2.4373E+03, &
      1.5824510E+00, 6.65749E+01, 2.0398E+01, 2.4115E+03, &
      1.5903633E+00, 6.65998E+01, 2.0282E+01, 2.3860E+03, &
      1.5983151E+00, 6.66236E+01, 2.0167E+01, 2.3609E+03, &
      1.6063066E+00, 6.66464E+01, 2.0054E+01, 2.3360E+03, &
      1.6143382E+00, 6.66681E+01, 1.9940E+01, 2.3114E+03, &
      1.6224099E+00, 6.66889E+01, 1.9828E+01, 2.2871E+03, &
      1.6305219E+00, 6.67086E+01, 1.9717E+01, 2.2630E+03, &
      1.6386745E+00, 6.67274E+01, 1.9606E+01, 2.2393E+03, &
      1.6468679E+00, 6.67452E+01, 1.9497E+01, 2.2158E+03, &
      1.6551022E+00, 6.67620E+01, 1.9388E+01, 2.1926E+03, &
      1.6633777E+00, 6.67779E+01, 1.9280E+01, 2.1697E+03, &
      1.6716946E+00, 6.67929E+01, 1.9173E+01, 2.1470E+03, &
      1.6800531E+00, 6.68070E+01, 1.9066E+01, 2.1246E+03, &
      1.6884534E+00, 6.68202E+01, 1.8961E+01, 2.1025E+03, &
      1.6968956E+00, 6.68325E+01, 1.8856E+01, 2.0806E+03, &
      1.7053801E+00, 6.68439E+01, 1.8752E+01, 2.0589E+03, &
      1.7139070E+00, 6.68544E+01, 1.8649E+01, 2.0376E+03, &
      1.7224766E+00, 6.68641E+01, 1.8547E+01, 2.0164E+03, &
      1.7310889E+00, 6.68728E+01, 1.8445E+01, 1.9955E+03, &
      1.7397444E+00, 6.68808E+01, 1.8344E+01, 1.9749E+03, &
      1.7484431E+00, 6.68879E+01, 1.8244E+01, 1.9544E+03, &
      1.7571853E+00, 6.68941E+01, 1.8145E+01, 1.9343E+03, &
      1.7659712E+00, 6.68995E+01, 1.8046E+01, 1.9143E+03, &
      1.7748011E+00, 6.69040E+01, 1.7948E+01, 1.8946E+03, &
      1.7836751E+00, 6.69078E+01, 1.7851E+01, 1.8751E+03, &
      1.7925935E+00, 6.69107E+01, 1.7755E+01, 1.8558E+03, &
      1.8015565E+00, 6.69127E+01, 1.7659E+01, 1.8367E+03, &
      1.8105642E+00, 6.69182E+01, 1.7564E+01, 1.8179E+03, &
      1.8196171E+00, 6.69187E+01, 1.7470E+01, 1.7993E+03, &
      1.8287151E+00, 6.69183E+01, 1.7376E+01, 1.7809E+03, &
      1.8378587E+00, 6.69172E+01, 1.7283E+01, 1.7627E+03, &
      1.8470480E+00, 6.69152E+01, 1.7191E+01, 1.7447E+03, &
      1.8562833E+00, 6.69124E+01, 1.7100E+01, 1.7269E+03, &
      1.8655647E+00, 6.69088E+01, 1.7009E+01, 1.7093E+03, &
      1.8748925E+00, 6.69044E+01, 1.6919E+01, 1.6919E+03, &
      1.8842670E+00, 6.68992E+01, 1.6830E+01, 1.6747E+03, &
      1.8936883E+00, 6.68931E+01, 1.6741E+01, 1.6577E+03, &
      1.9031567E+00, 6.68862E+01, 1.6653E+01, 1.6409E+03, &
      1.9126725E+00, 6.68771E+01, 1.6566E+01, 1.6243E+03, &
      1.9222359E+00, 6.68686E+01, 1.6479E+01, 1.6079E+03, &
      1.9318471E+00, 6.68593E+01, 1.6393E+01, 1.5917E+03, &
      1.9415063E+00, 6.68491E+01, 1.6307E+01, 1.5756E+03, &
      1.9512138E+00, 6.68381E+01, 1.6222E+01, 1.5597E+03, &
      1.9609699E+00, 6.68263E+01, 1.6138E+01, 1.5440E+03, &
      1.9707747E+00, 6.68136E+01, 1.6055E+01, 1.5285E+03, &
      1.9806286E+00, 6.68000E+01, 1.5972E+01, 1.5132E+03, &
      1.9905318E+00, 6.67856E+01, 1.5889E+01, 1.4980E+03, &
      2.0004844E+00, 6.67703E+01, 1.5808E+01, 1.4830E+03, &
      2.0104868E+00, 6.67542E+01, 1.5727E+01, 1.4682E+03, &
      2.0205393E+00, 6.67371E+01, 1.5646E+01, 1.4535E+03, &
      2.0306420E+00, 6.67191E+01, 1.5566E+01, 1.4390E+03, &
      2.0407952E+00, 6.67003E+01, 1.5487E+01, 1.4247E+03, &
      2.0509992E+00, 6.66805E+01, 1.5408E+01, 1.4105E+03, &
      2.0612542E+00, 6.66598E+01, 1.5330E+01, 1.3965E+03, &
      2.0715604E+00, 6.66382E+01, 1.5253E+01, 1.3826E+03, &
      2.0819182E+00, 6.66156E+01, 1.5176E+01, 1.3689E+03, &
      2.0923278E+00, 6.65921E+01, 1.5099E+01, 1.3554E+03, &
      2.1027895E+00, 6.65676E+01, 1.5023E+01, 1.3420E+03, &
      2.1133034E+00, 6.65422E+01, 1.4948E+01, 1.3287E+03, &
      2.1238699E+00, 6.65246E+01, 1.4873E+01, 1.3155E+03, &
      2.1344893E+00, 6.64974E+01, 1.4797E+01, 1.3025E+03, &
      2.1451617E+00, 6.64690E+01, 1.4723E+01, 1.2896E+03, &
      2.1558875E+00, 6.64394E+01, 1.4649E+01, 1.2768E+03, &
      2.1666670E+00, 6.64087E+01, 1.4575E+01, 1.2642E+03, &
      2.1775003E+00, 6.63767E+01, 1.4502E+01, 1.2517E+03, &
      2.1883878E+00, 6.63436E+01, 1.4429E+01, 1.2394E+03, &
      2.1993297E+00, 6.63092E+01, 1.4357E+01, 1.2272E+03, &
      2.2103264E+00, 6.62736E+01, 1.4286E+01, 1.2151E+03, &
      2.2213780E+00, 6.62367E+01, 1.4215E+01, 1.2032E+03, &
      2.2324849E+00, 6.61985E+01, 1.4145E+01, 1.1914E+03, &
      2.2436473E+00, 6.61590E+01, 1.4075E+01, 1.1797E+03, &
      2.2548656E+00, 6.61181E+01, 1.4005E+01, 1.1681E+03, &
      2.2661399E+00, 6.60759E+01, 1.3936E+01, 1.1567E+03, &
      2.2774706E+00, 6.60322E+01, 1.3868E+01, 1.1454E+03, &
      2.2888579E+00, 6.59870E+01, 1.3800E+01, 1.1343E+03, &
      2.3003022E+00, 6.59404E+01, 1.3732E+01, 1.1232E+03, &
      2.3118037E+00, 6.58922E+01, 1.3665E+01, 1.1123E+03, &
      2.3233628E+00, 6.58425E+01, 1.3599E+01, 1.1015E+03, &
      2.3349796E+00, 6.57911E+01, 1.3533E+01, 1.0908E+03, &
      2.3466545E+00, 6.57381E+01, 1.3467E+01, 1.0802E+03, &
      2.3583878E+00, 6.56834E+01, 1.3402E+01, 1.0697E+03, &
      2.3701797E+00, 6.56269E+01, 1.3338E+01, 1.0594E+03, &
      2.3820306E+00, 6.55686E+01, 1.3274E+01, 1.0492E+03, &
      2.3939407E+00, 6.55084E+01, 1.3210E+01, 1.0390E+03, &
      2.4059104E+00, 6.54463E+01, 1.3147E+01, 1.0290E+03, &
      2.4179400E+00, 6.53789E+01, 1.3079E+01, 1.0188E+03, &
      2.4300297E+00, 6.53122E+01, 1.3009E+01, 1.0083E+03, &
      2.4421798E+00, 6.52426E+01, 1.2939E+01, 9.9804E+02, &
      2.4543907E+00, 6.51702E+01, 1.2869E+01, 9.8785E+02, &
      2.4666627E+00, 6.50948E+01, 1.2800E+01, 9.7777E+02, &
      2.4789960E+00, 6.50164E+01, 1.2732E+01, 9.6781E+02, &
      2.4913910E+00, 6.49349E+01, 1.2664E+01, 9.5796E+02, &
      2.5038479E+00, 6.48501E+01, 1.2596E+01, 9.4822E+02, &
      2.5163672E+00, 6.47619E+01, 1.2529E+01, 9.3859E+02, &
      2.5289490E+00, 6.46703E+01, 1.2462E+01, 9.2907E+02, &
      2.5415938E+00, 6.45750E+01, 1.2396E+01, 9.1966E+02, &
      2.5543017E+00, 6.44758E+01, 1.2326E+01, 9.1004E+02, &
      2.5670732E+00, 6.43723E+01, 1.2256E+01, 9.0047E+02, &
      2.5799086E+00, 6.42642E+01, 1.2187E+01, 8.9101E+02, &
      2.5928082E+00, 6.41514E+01, 1.2117E+01, 8.8166E+02, &
      2.6057722E+00, 6.40336E+01, 1.2049E+01, 8.7241E+02, &
      2.6188011E+00, 6.39107E+01, 1.1981E+01, 8.6328E+02, &
      2.6318951E+00, 6.37824E+01, 1.1913E+01, 8.5424E+02, &
      2.6450545E+00, 6.36437E+01, 1.1846E+01, 8.4531E+02, &
      2.6582798E+00, 6.35034E+01, 1.1779E+01, 8.3648E+02, &
      2.6715712E+00, 6.33567E+01, 1.1713E+01, 8.2775E+02, &
      2.6849291E+00, 6.32033E+01, 1.1647E+01, 8.1912E+02, &
      2.6983537E+00, 6.30426E+01, 1.1582E+01, 8.1059E+02, &
      2.7118455E+00, 6.28742E+01, 1.1517E+01, 8.0215E+02, &
      2.7254047E+00, 6.26976E+01, 1.1453E+01, 7.9381E+02, &
      2.7390317E+00, 6.25122E+01, 1.1389E+01, 7.8557E+02, &
      2.7527269E+00, 6.23173E+01, 1.1326E+01, 7.7742E+02, &
      2.7664905E+00, 6.21121E+01, 1.1263E+01, 7.6936E+02, &
      2.7803230E+00, 6.18959E+01, 1.1200E+01, 7.6139E+02, &
      2.7942246E+00, 6.16677E+01, 1.1138E+01, 7.5351E+02, &
      2.8081957E+00, 6.14262E+01, 1.1077E+01, 7.4572E+02, &
      2.8222367E+00, 6.11704E+01, 1.1015E+01, 7.3802E+02, &
      2.8363479E+00, 6.08988E+01, 1.0955E+01, 7.3040E+02, &
      2.8505296E+00, 6.06095E+01, 1.0894E+01, 7.2288E+02, &
      2.8647823E+00, 6.03008E+01, 1.0834E+01, 7.1543E+02, &
      2.8791062E+00, 5.99701E+01, 1.0775E+01, 7.0806E+02, &
      2.8935017E+00, 5.96148E+01, 1.0715E+01, 7.0076E+02, &
      2.9079692E+00, 5.92313E+01, 1.0657E+01, 6.9354E+02, &
      2.9225091E+00, 5.88155E+01, 1.0598E+01, 6.8640E+02, &
      2.9371216E+00, 5.83622E+01, 1.0540E+01, 6.7935E+02, &
      2.9518072E+00, 5.78647E+01, 1.0482E+01, 6.7237E+02, &
      2.9665662E+00, 5.73146E+01, 1.0425E+01, 6.6547E+02, &
      2.9813991E+00, 5.67005E+01, 1.0368E+01, 6.5865E+02, &
      2.9963061E+00, 5.60066E+01, 1.0312E+01, 6.5190E+02, &
      3.0112876E+00, 5.52321E+01, 1.0238E+01, 6.4413E+02, &
      3.0263440E+00, 5.43068E+01, 1.0158E+01, 6.3609E+02, &
      3.0414758E+00, 5.31680E+01, 1.0080E+01, 6.2816E+02, &
      3.0566831E+00, 5.17110E+01, 1.0002E+01, 6.2034E+02, &
      3.0719666E+00, 4.97023E+01, 9.9252E+00, 6.1263E+02, &
      3.0873264E+00, 4.64433E+01, 9.8489E+00, 6.0502E+02, &
      3.1027630E+00, 3.57559E+01, 9.7733E+00, 5.9751E+02, &
      3.1040119E+00, 3.13348E+01, 9.7672E+00, 5.9691E+02, &
      3.1057879E+00, 3.10702E+01, 2.5557E+01, 1.5431E+03, &
      3.1182768E+00, 4.46231E+01, 2.5384E+01, 1.5266E+03, &
      3.1338682E+00, 4.82915E+01, 2.5171E+01, 1.5064E+03, &
      3.1495376E+00, 5.01566E+01, 2.4960E+01, 1.4865E+03, &
      3.1652853E+00, 5.12628E+01, 2.4751E+01, 1.4668E+03, &
      3.1811117E+00, 5.18834E+01, 2.4544E+01, 1.4474E+03, &
      3.1970172E+00, 5.20846E+01, 2.4339E+01, 1.4283E+03, &
      3.2130023E+00, 5.17901E+01, 2.4135E+01, 1.4095E+03, &
      3.2290673E+00, 5.06134E+01, 2.3933E+01, 1.3909E+03, &
      3.2452127E+00, 4.53623E+01, 2.3734E+01, 1.3725E+03, &
      3.2473832E+00, 4.16409E+01, 2.3707E+01, 1.3701E+03, &
      3.2494167E+00, 4.15668E+01, 3.4198E+01, 1.9703E+03, &
      3.2614387E+00, 5.06437E+01, 3.3985E+01, 1.9509E+03, &
      3.2777459E+00, 5.39920E+01, 3.3700E+01, 1.9250E+03, &
      3.2941347E+00, 5.60420E+01, 3.3417E+01, 1.8995E+03, &
      3.3106053E+00, 5.75824E+01, 3.3137E+01, 1.8744E+03, &
      3.3271584E+00, 5.88377E+01, 3.2860E+01, 1.8495E+03, &
      3.3437941E+00, 5.99055E+01, 3.2584E+01, 1.8251E+03, &
      3.3605131E+00, 6.08379E+01, 3.2312E+01, 1.8009E+03, &
      3.3773157E+00, 6.16663E+01, 3.2041E+01, 1.7771E+03, &
      3.3942023E+00, 6.24113E+01, 3.1773E+01, 1.7536E+03, &
      3.4111733E+00, 6.30873E+01, 3.1508E+01, 1.7304E+03, &
      3.4282291E+00, 6.37045E+01, 3.1244E+01, 1.7075E+03, &
      3.4453703E+00, 6.42708E+01, 3.0983E+01, 1.6849E+03, &
      3.4625971E+00, 6.47920E+01, 3.0725E+01, 1.6627E+03, &
      3.4799101E+00, 6.52728E+01, 3.0468E+01, 1.6407E+03, &
      3.4973097E+00, 6.57166E+01, 3.0214E+01, 1.6191E+03, &
      3.5147962E+00, 6.61264E+01, 2.9962E+01, 1.5977E+03, &
      3.5323702E+00, 6.65041E+01, 2.9711E+01, 1.5765E+03, &
      3.5500321E+00, 6.68511E+01, 2.9462E+01, 1.5557E+03, &
      3.5677822E+00, 6.71685E+01, 2.9215E+01, 1.5351E+03, &
      3.5856211E+00, 6.74565E+01, 2.8970E+01, 1.5148E+03, &
      3.6035492E+00, 6.77151E+01, 2.8727E+01, 1.4947E+03, &
      3.6215670E+00, 6.79432E+01, 2.8487E+01, 1.4749E+03, &
      3.6396748E+00, 6.81389E+01, 2.8248E+01, 1.4554E+03, &
      3.6578732E+00, 6.82989E+01, 2.8012E+01, 1.4362E+03, &
      3.6761626E+00, 6.84179E+01, 2.7778E+01, 1.4172E+03, &
      3.6945434E+00, 6.84869E+01, 2.7546E+01, 1.3985E+03, &
      3.7130161E+00, 6.84910E+01, 2.7315E+01, 1.3800E+03, &
      3.7315812E+00, 6.84015E+01, 2.7087E+01, 1.3618E+03, &
      3.7502391E+00, 6.81565E+01, 2.6861E+01, 1.3438E+03, &
      3.7689903E+00, 6.75759E+01, 2.6637E+01, 1.3261E+03, &
      3.7870982E+00, 6.55798E+01, 2.6423E+01, 1.3093E+03, &
      3.7878352E+00, 6.53467E+01, 2.6414E+01, 1.3086E+03, &
      3.7965019E+00, 6.56382E+01, 3.0799E+01, 1.5206E+03, &
      3.8067744E+00, 6.74348E+01, 3.0672E+01, 1.5103E+03, &
      3.8258083E+00, 6.88565E+01, 3.0438E+01, 1.4915E+03, &
      3.8449373E+00, 6.97323E+01, 3.0207E+01, 1.4729E+03, &
      3.8641620E+00, 7.04017E+01, 2.9977E+01, 1.4546E+03, &
      3.8834828E+00, 7.09589E+01, 2.9750E+01, 1.4364E+03, &
      3.9029002E+00, 7.14439E+01, 2.9524E+01, 1.4186E+03, &
      3.9224147E+00, 7.18775E+01, 2.9301E+01, 1.4009E+03, &
      3.9420268E+00, 7.22722E+01, 2.9079E+01, 1.3835E+03, &
      3.9617369E+00, 7.26359E+01, 2.8859E+01, 1.3663E+03, &
      3.9815456E+00, 7.29742E+01, 2.8641E+01, 1.3493E+03, &
      4.0014533E+00, 7.32908E+01, 2.8424E+01, 1.3326E+03, &
      4.0214606E+00, 7.35886E+01, 2.8209E+01, 1.3160E+03, &
      4.0415679E+00, 7.38699E+01, 2.7995E+01, 1.2996E+03, &
      4.0617757E+00, 7.41352E+01, 2.7778E+01, 1.2833E+03, &
      4.0820846E+00, 7.43857E+01, 2.7564E+01, 1.2671E+03, &
      4.1024950E+00, 7.46227E+01, 2.7351E+01, 1.2512E+03, &
      4.1230075E+00, 7.48470E+01, 2.7139E+01, 1.2354E+03, &
      4.1436226E+00, 7.50595E+01, 2.6930E+01, 1.2199E+03, &
      4.1643407E+00, 7.52607E+01, 2.6722E+01, 1.2046E+03, &
      4.1851624E+00, 7.54511E+01, 2.6516E+01, 1.1894E+03, &
      4.2060882E+00, 7.56310E+01, 2.6311E+01, 1.1745E+03, &
      4.2271186E+00, 7.58005E+01, 2.6108E+01, 1.1597E+03, &
      4.2482542E+00, 7.59588E+01, 2.5903E+01, 1.1450E+03, &
      4.2694955E+00, 7.61056E+01, 2.5700E+01, 1.1305E+03, &
      4.2908430E+00, 7.62404E+01, 2.5499E+01, 1.1161E+03, &
      4.3122972E+00, 7.63624E+01, 2.5300E+01, 1.1020E+03, &
      4.3338587E+00, 7.64705E+01, 2.5102E+01, 1.0880E+03, &
      4.3555280E+00, 7.65624E+01, 2.4905E+01, 1.0743E+03, &
      4.3773056E+00, 7.66349E+01, 2.4710E+01, 1.0606E+03, &
      4.3991921E+00, 7.66822E+01, 2.4517E+01, 1.0472E+03, &
      4.4211881E+00, 7.66935E+01, 2.4325E+01, 1.0339E+03, &
      4.4432940E+00, 7.66446E+01, 2.4135E+01, 1.0208E+03, &
      4.4655105E+00, 7.64608E+01, 2.3946E+01, 1.0079E+03, &
      4.4829005E+00, 7.59322E+01, 2.3800E+01, 9.9796E+02, &
      4.4878381E+00, 7.52855E+01, 2.3758E+01, 9.9515E+02, &
      4.4960996E+00, 7.59831E+01, 2.5229E+01, 1.0542E+03, &
      4.5102772E+00, 7.66248E+01, 2.5110E+01, 1.0460E+03, &
      4.5328286E+00, 7.71055E+01, 2.4922E+01, 1.0331E+03, &
      4.5554928E+00, 7.74258E+01, 2.4736E+01, 1.0204E+03, &
      4.5782702E+00, 7.76788E+01, 2.4552E+01, 1.0078E+03, &
      4.6011616E+00, 7.78922E+01, 2.4368E+01, 9.9543E+02, &
      4.6241674E+00, 7.80777E+01, 2.4186E+01, 9.8316E+02, &
      4.6472882E+00, 7.82410E+01, 2.4005E+01, 9.7104E+02, &
      4.6705247E+00, 7.83847E+01, 2.3826E+01, 9.5908E+02, &
      4.6938773E+00, 7.85094E+01, 2.3648E+01, 9.4727E+02, &
      4.7173467E+00, 7.86136E+01, 2.3471E+01, 9.3561E+02, &
      4.7409334E+00, 7.86930E+01, 2.3297E+01, 9.2413E+02, &
      4.7646381E+00, 7.87369E+01, 2.3124E+01, 9.1278E+02, &
      4.7884613E+00, 7.87140E+01, 2.2952E+01, 9.0159E+02, &
      4.8116809E+00, 7.84773E+01, 2.2787E+01, 8.9086E+02, &
      4.8124036E+00, 7.84584E+01, 2.2781E+01, 8.9053E+02, &
      4.8323191E+00, 7.85884E+01, 2.3641E+01, 9.1998E+02, &
      4.8364656E+00, 7.87192E+01, 2.3612E+01, 9.1807E+02, &
      4.8606479E+00, 7.91649E+01, 2.3443E+01, 9.0708E+02, &
      4.8849512E+00, 7.94489E+01, 2.3276E+01, 8.9623E+02, &
      4.9093759E+00, 7.96782E+01, 2.3111E+01, 8.8551E+02, &
      4.9339228E+00, 7.98784E+01, 2.2946E+01, 8.7491E+02, &
      4.9585924E+00, 8.00593E+01, 2.2780E+01, 8.6436E+02, &
      4.9833854E+00, 8.02261E+01, 2.2616E+01, 8.5393E+02, &
      5.0083023E+00, 8.03819E+01, 2.2453E+01, 8.4364E+02, &
      5.0333438E+00, 8.05290E+01, 2.2291E+01, 8.3348E+02, &
      5.0585105E+00, 8.06687E+01, 2.2131E+01, 8.2345E+02, &
      5.0838031E+00, 8.08020E+01, 2.1971E+01, 8.1352E+02, &
      5.1092221E+00, 8.09294E+01, 2.1812E+01, 8.0371E+02, &
      5.1347682E+00, 8.10516E+01, 2.1655E+01, 7.9403E+02, &
      5.1604421E+00, 8.11691E+01, 2.1499E+01, 7.8446E+02, &
      5.1862443E+00, 8.12823E+01, 2.1344E+01, 7.7502E+02, &
      5.2121755E+00, 8.13916E+01, 2.1190E+01, 7.6569E+02, &
      5.2382364E+00, 8.14972E+01, 2.1038E+01, 7.5648E+02, &
      5.2644276E+00, 8.15993E+01, 2.0886E+01, 7.4737E+02, &
      5.2907497E+00, 8.16983E+01, 2.0736E+01, 7.3838E+02, &
      5.3172034E+00, 8.17942E+01, 2.0587E+01, 7.2950E+02, &
      5.3437895E+00, 8.18872E+01, 2.0439E+01, 7.2072E+02, &
      5.3705084E+00, 8.19775E+01, 2.0292E+01, 7.1206E+02, &
      5.3973609E+00, 8.20651E+01, 2.0146E+01, 7.0351E+02, &
      5.4243477E+00, 8.21503E+01, 2.0001E+01, 6.9506E+02, &
      5.4514695E+00, 8.22332E+01, 1.9858E+01, 6.8672E+02, &
      5.4787268E+00, 8.23138E+01, 1.9715E+01, 6.7848E+02, &
      5.5061205E+00, 8.23922E+01, 1.9574E+01, 6.7034E+02, &
      5.5336511E+00, 8.24686E+01, 1.9434E+01, 6.6231E+02, &
      5.5613193E+00, 8.25430E+01, 1.9295E+01, 6.5438E+02, &
      5.5891259E+00, 8.26155E+01, 1.9157E+01, 6.4654E+02, &
      5.6170716E+00, 8.26862E+01, 1.9020E+01, 6.3880E+02, &
      5.6451569E+00, 8.27552E+01, 1.8884E+01, 6.3113E+02, &
      5.6733827E+00, 8.28224E+01, 1.8748E+01, 6.2356E+02, &
      5.7017496E+00, 8.28881E+01, 1.8614E+01, 6.1609E+02, &
      5.7302584E+00, 8.29522E+01, 1.8481E+01, 6.0871E+02, &
      5.7589096E+00, 8.30148E+01, 1.8348E+01, 6.0142E+02, &
      5.7877042E+00, 8.30761E+01, 1.8217E+01, 5.9422E+02, &
      5.8166427E+00, 8.31362E+01, 1.8087E+01, 5.8712E+02, &
      5.8457259E+00, 8.31950E+01, 1.7957E+01, 5.8008E+02, &
      5.8749546E+00, 8.32522E+01, 1.7828E+01, 5.7309E+02, &
      5.9043293E+00, 8.33077E+01, 1.7699E+01, 5.6619E+02, &
      5.9338510E+00, 8.33618E+01, 1.7571E+01, 5.5937E+02, &
      5.9635202E+00, 8.34144E+01, 1.7444E+01, 5.5264E+02, &
      5.9933378E+00, 8.34657E+01, 1.7318E+01, 5.4600E+02, &
      6.0233045E+00, 8.35156E+01, 1.7193E+01, 5.3943E+02, &
      6.0534210E+00, 8.35643E+01, 1.7069E+01, 5.3295E+02, &
      6.0836882E+00, 8.36118E+01, 1.6946E+01, 5.2655E+02, &
      6.1141066E+00, 8.36581E+01, 1.6824E+01, 5.2023E+02, &
      6.1446771E+00, 8.37034E+01, 1.6703E+01, 5.1399E+02, &
      6.1754005E+00, 8.37476E+01, 1.6583E+01, 5.0783E+02, &
      6.2062775E+00, 8.37908E+01, 1.6464E+01, 5.0174E+02, &
      6.2373089E+00, 8.39861E+01, 1.6343E+01, 4.9565E+02, &
      6.2684954E+00, 8.40270E+01, 1.6223E+01, 4.8961E+02, &
      6.2998379E+00, 8.40666E+01, 1.6103E+01, 4.8364E+02, &
      6.3313371E+00, 8.41048E+01, 1.5984E+01, 4.7775E+02, &
      6.3629938E+00, 8.41418E+01, 1.5866E+01, 4.7193E+02, &
      6.3948088E+00, 8.41776E+01, 1.5749E+01, 4.6619E+02, &
      6.4267828E+00, 8.42124E+01, 1.5633E+01, 4.6052E+02, &
      6.4589167E+00, 8.42460E+01, 1.5518E+01, 4.5492E+02, &
      6.4912113E+00, 8.42786E+01, 1.5403E+01, 4.4939E+02, &
      6.5236674E+00, 8.44154E+01, 1.5288E+01, 4.4388E+02, &
      6.5562857E+00, 8.44457E+01, 1.5174E+01, 4.3844E+02, &
      6.5890671E+00, 8.44749E+01, 1.5061E+01, 4.3306E+02, &
      6.6220125E+00, 8.45030E+01, 1.4948E+01, 4.2775E+02, &
      6.6551225E+00, 8.45300E+01, 1.4837E+01, 4.2251E+02, &
      6.6883981E+00, 8.45561E+01, 1.4726E+01, 4.1734E+02, &
      6.7218401E+00, 8.45811E+01, 1.4616E+01, 4.1224E+02, &
      6.7554493E+00, 8.46052E+01, 1.4507E+01, 4.0720E+02, &
      6.7892266E+00, 8.46284E+01, 1.4399E+01, 4.0222E+02, &
      6.8231727E+00, 8.46507E+01, 1.4292E+01, 3.9731E+02, &
      6.8572886E+00, 8.46721E+01, 1.4186E+01, 3.9246E+02, &
      6.8915750E+00, 8.46927E+01, 1.4081E+01, 3.8767E+02, &
      6.9260329E+00, 8.47125E+01, 1.3977E+01, 3.8295E+02, &
      6.9606631E+00, 8.47316E+01, 1.3873E+01, 3.7828E+02, &
      6.9954664E+00, 8.47498E+01, 1.3771E+01, 3.7368E+02, &
      7.0304437E+00, 8.47674E+01, 1.3669E+01, 3.6913E+02, &
      7.0655959E+00, 8.47842E+01, 1.3568E+01, 3.6465E+02, &
      7.1009239E+00, 8.48003E+01, 1.3468E+01, 3.6022E+02, &
      7.1364285E+00, 8.48158E+01, 1.3369E+01, 3.5584E+02, &
      7.1721107E+00, 8.48306E+01, 1.3270E+01, 3.5153E+02, &
      7.2079712E+00, 8.48447E+01, 1.3173E+01, 3.4726E+02, &
      7.2440111E+00, 8.48583E+01, 1.3076E+01, 3.4306E+02, &
      7.2802311E+00, 8.48713E+01, 1.2980E+01, 3.3890E+02, &
      7.3166323E+00, 8.48837E+01, 1.2885E+01, 3.3480E+02, &
      7.3532155E+00, 8.48955E+01, 1.2790E+01, 3.3076E+02, &
      7.3899815E+00, 8.49068E+01, 1.2697E+01, 3.2676E+02, &
      7.4269314E+00, 8.49176E+01, 1.2604E+01, 3.2282E+02, &
      7.4640661E+00, 8.49279E+01, 1.2512E+01, 3.1893E+02, &
      7.5013864E+00, 8.49377E+01, 1.2421E+01, 3.1508E+02, &
      7.5388934E+00, 8.49470E+01, 1.2330E+01, 3.1129E+02, &
      7.5765878E+00, 8.49559E+01, 1.2241E+01, 3.0754E+02, &
      7.6144708E+00, 8.50561E+01, 1.2150E+01, 3.0381E+02, &
      7.6525431E+00, 8.50643E+01, 1.2060E+01, 3.0011E+02, &
      7.6908058E+00, 8.50718E+01, 1.1971E+01, 2.9646E+02, &
      7.7292599E+00, 8.50788E+01, 1.1882E+01, 2.9286E+02, &
      7.7679062E+00, 8.50853E+01, 1.1794E+01, 2.8931E+02, &
      7.8067457E+00, 8.50912E+01, 1.1707E+01, 2.8580E+02, &
      7.8457794E+00, 8.50965E+01, 1.1621E+01, 2.8233E+02, &
      7.8850083E+00, 8.51014E+01, 1.1536E+01, 2.7892E+02, &
      7.9244334E+00, 8.51058E+01, 1.1451E+01, 2.7554E+02, &
      7.9640555E+00, 8.51098E+01, 1.1367E+01, 2.7221E+02, &
      8.0038758E+00, 8.51133E+01, 1.1283E+01, 2.6892E+02, &
      8.0438952E+00, 8.51164E+01, 1.1201E+01, 2.6567E+02, &
      8.0841147E+00, 8.51191E+01, 1.1119E+01, 2.6247E+02, &
      8.1245352E+00, 8.51214E+01, 1.1037E+01, 2.5931E+02, &
      8.1651579E+00, 8.51233E+01, 1.0957E+01, 2.5619E+02, &
      8.2059837E+00, 8.51249E+01, 1.0877E+01, 2.5310E+02, &
      8.2470136E+00, 8.51262E+01, 1.0798E+01, 2.5006E+02, &
      8.2882487E+00, 8.51271E+01, 1.0719E+01, 2.4706E+02, &
      8.3296899E+00, 8.51277E+01, 1.0641E+01, 2.4409E+02, &
      8.3713384E+00, 8.51280E+01, 1.0564E+01, 2.4117E+02, &
      8.4131951E+00, 8.51281E+01, 1.0488E+01, 2.3828E+02, &
      8.4552610E+00, 8.51280E+01, 1.0412E+01, 2.3543E+02, &
      8.4975373E+00, 8.51276E+01, 1.0336E+01, 2.3261E+02, &
      8.5400250E+00, 8.51271E+01, 1.0262E+01, 2.2983E+02, &
      9.030794E+00, 8.51707E+01, 9.4348E+00, 2.0038E+02, &
      9.653919E+00, 8.51076E+01, 8.4372E+00, 1.6824E+02, &
      1.032004E+01, 8.48921E+01, 7.5506E+00, 1.4141E+02, &
      1.103212E+01, 8.45849E+01, 6.7682E+00, 1.1910E+02, &
      1.179334E+01, 8.41863E+01, 6.0735E+00, 1.0045E+02, &
      1.260708E+01, 8.36659E+01, 5.4548E+00, 8.4821E+01, &
      1.347697E+01, 8.29402E+01, 4.9064E+00, 7.1758E+01, &
      1.440688E+01, 8.17231E+01, 4.4059E+00, 6.0640E+01, &
      1.513551E+01, 7.94783E+01, 4.0707E+00, 5.3581E+01, &
      1.536718E+01, 7.69041E+01, 3.9736E+00, 5.1592E+01, &
      1.540095E+01, 7.58263E+01, 3.9598E+00, 5.1311E+01, &
      1.542896E+01, 7.38766E+01, 3.9485E+00, 5.1079E+01, &
      1.552162E+01, 7.68957E+01, 9.8368E+00, 1.2140E+02, &
      1.575329E+01, 7.95752E+01, 9.6041E+00, 1.1686E+02, &
      1.646362E+01, 8.17385E+01, 8.9362E+00, 1.0424E+02, &
      1.759961E+01, 8.23754E+01, 7.9674E+00, 8.7239E+01, &
      1.811461E+01, 8.18554E+01, 7.5791E+00, 8.0754E+01, &
      1.839188E+01, 8.06326E+01, 7.3820E+00, 7.7534E+01, &
      1.846582E+01, 7.91067E+01, 7.3308E+00, 7.6705E+01, &
      1.857672E+01, 8.06413E+01, 1.0265E+01, 1.0564E+02, &
      1.881398E+01, 8.18406E+01, 1.0055E+01, 1.0223E+02, &
      1.885197E+01, 8.19278E+01, 1.0022E+01, 1.0170E+02, &
      1.885399E+01, 8.19320E+01, 1.0020E+01, 1.0167E+02, &
      1.914052E+01, 8.20105E+01, 9.7772E+00, 9.7779E+01, &
      1.921746E+01, 8.13891E+01, 9.7136E+00, 9.6769E+01, &
      1.933288E+01, 8.23574E+01, 1.1132E+01, 1.0985E+02, &
      1.962143E+01, 8.34660E+01, 1.0888E+01, 1.0590E+02, &
      2.011215E+01, 8.44292E+01, 1.0488E+01, 9.9608E+01, &
      2.149988E+01, 8.58910E+01, 9.4546E+00, 8.4196E+01, &
      2.298338E+01, 8.67343E+01, 8.5164E+00, 7.1127E+01, &
      2.456923E+01, 8.72884E+01, 7.6729E+00, 6.0109E+01, &
      2.626450E+01, 8.76735E+01, 6.9145E+00, 5.0820E+01, &
      2.807676E+01, 8.79564E+01, 6.1939E+00, 4.2726E+01, &
      3.001405E+01, 8.81213E+01, 5.5414E+00, 3.5889E+01, &
      3.208502E+01, 8.82735E+01, 4.9538E+00, 3.0132E+01, &
      3.429889E+01, 8.83117E+01, 4.4274E+00, 2.5300E+01, &
      3.666551E+01, 8.83147E+01, 3.9611E+00, 2.1272E+01, &
      3.919543E+01, 8.83551E+01, 3.5365E+00, 1.7856E+01, &
      4.189992E+01, 8.83080E+01, 3.1512E+00, 1.4967E+01, &
      4.479101E+01, 8.82421E+01, 2.8100E+00, 1.2561E+01, &
      4.788159E+01, 8.81644E+01, 2.5077E+00, 1.0553E+01, &
      5.118542E+01, 8.80786E+01, 2.2396E+00, 8.8784E+00, &
      5.471721E+01, 8.79870E+01, 2.0017E+00, 7.4788E+00, &
      5.849270E+01, 8.78906E+01, 1.7904E+00, 6.3081E+00, &
      6.252870E+01, 8.77896E+01, 1.6026E+00, 5.3281E+00, &
      6.684318E+01, 8.76832E+01, 1.4355E+00, 4.5066E+00, &
      7.145536E+01, 8.75680E+01, 1.2843E+00, 3.8110E+00, &
      7.638578E+01, 8.74390E+01, 1.1500E+00, 3.2282E+00, &
      8.165640E+01, 8.72913E+01, 1.0274E+00, 2.7324E+00, &
      8.729069E+01, 8.70947E+01, 9.1053E-01, 2.2998E+00, &
      9.331374E+01, 8.67978E+01, 8.0675E-01, 1.9384E+00, &
      9.975239E+01, 8.61750E+01, 7.1532E-01, 1.6379E+00, &
      1.018435E+02, 8.56929E+01, 6.8916E-01, 1.5550E+00, &
      1.034023E+02, 8.46943E+01, 6.7065E-01, 1.4974E+00, &
      1.038180E+02, 8.35001E+01, 6.6584E-01, 1.4825E+00, &
      1.044415E+02, 8.47442E+01, 3.0084E+00, 5.6491E+00, &
      1.060003E+02, 8.58317E+01, 2.9405E+00, 5.4460E+00, &
      1.066353E+02, 8.60452E+01, 2.9136E+00, 5.3662E+00, &
      1.139931E+02, 8.71022E+01, 2.6266E+00, 4.5481E+00, &
      1.218587E+02, 8.75403E+01, 2.3654E+00, 3.8531E+00, &
      1.302669E+02, 8.77977E+01, 2.1279E+00, 3.2634E+00, &
      1.392553E+02, 8.79644E+01, 1.9137E+00, 2.7653E+00, &
      1.488640E+02, 8.80774E+01, 1.7206E+00, 2.3448E+00, &
      1.591356E+02, 8.81537E+01, 1.5450E+00, 1.9880E+00, &
      1.701159E+02, 8.82040E+01, 1.3874E+00, 1.6875E+00, &
      1.818539E+02, 8.82362E+01, 1.2459E+00, 1.4346E+00, &
      1.944018E+02, 8.82551E+01, 1.1189E+00, 1.2213E+00, &
      2.078156E+02, 8.82645E+01, 1.0049E+00, 1.0418E+00, &
      2.221548E+02, 8.82666E+01, 9.0294E-01, 8.9056E-01, &
      2.374835E+02, 8.82640E+01, 8.1164E-01, 7.6320E-01, &
      2.538699E+02, 8.82578E+01, 7.2965E-01, 6.5566E-01, &
      2.713869E+02, 8.82490E+01, 6.5603E-01, 5.6479E-01, &
      2.901126E+02, 8.82385E+01, 5.8989E-01, 4.8793E-01, &
      3.101304E+02, 8.82269E+01, 5.3048E-01, 4.2287E-01, &
      3.315294E+02, 8.82146E+01, 4.7710E-01, 3.6773E-01, &
      3.544049E+02, 8.82020E+01, 4.2914E-01, 3.2096E-01, &
      3.788588E+02, 8.81893E+01, 3.8603E-01, 2.8122E-01, &
      4.050001E+02, 8.81767E+01, 3.4728E-01, 2.4739E-01, &
      4.329451E+02, 8.81645E+01, 3.1246E-01, 2.1856E-01]
   case(89)
      frel = -1.3722E+00
      allocate(nist_tab(3108))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 1.98738E+01, 0.0000E+00, 1.2142E-04, &
      1.142761E-02, 1.97310E+01, 0.0000E+00, 1.6777E-04, &
      1.221612E-02, 1.95519E+01, 0.0000E+00, 2.3056E-04, &
      1.305903E-02, 1.93217E+01, 0.0000E+00, 3.1518E-04, &
      1.396010E-02, 1.90154E+01, 0.0000E+00, 4.2858E-04, &
      1.492335E-02, 1.85856E+01, 0.0000E+00, 5.7971E-04, &
      1.595306E-02, 1.79255E+01, 0.0000E+00, 7.8006E-04, &
      1.705382E-02, 1.67021E+01, 0.0000E+00, 1.0442E-03, &
      1.803405E-02, 1.36882E+01, 0.0000E+00, 1.3278E-03, &
      1.823053E-02, 1.17526E+01, 0.0000E+00, 1.3906E-03, &
      1.831008E-02, 1.01151E+01, 0.0000E+00, 1.4165E-03, &
      1.838369E-02, 5.74963E+00, 0.0000E+00, 1.4409E-03, &
      1.849410E-02, 1.04311E+01, 8.6357E+00, 8.6550E+04, &
      1.877013E-02, 1.45434E+01, 8.2993E+00, 8.1955E+04, &
      1.948844E-02, 1.78163E+01, 7.5041E+00, 7.1371E+04, &
      2.083314E-02, 1.99561E+01, 6.2727E+00, 5.5808E+04, &
      2.227063E-02, 2.06820E+01, 5.2323E+00, 4.3547E+04, &
      2.380730E-02, 2.04120E+01, 4.3595E+00, 3.3941E+04, &
      2.468140E-02, 1.92537E+01, 3.9502E+00, 2.9665E+04, &
      2.505918E-02, 1.72280E+01, 3.7897E+00, 2.8031E+04, &
      2.515992E-02, 1.46607E+01, 3.7485E+00, 2.7615E+04, &
      2.531103E-02, 1.74902E+01, 8.8114E+00, 6.4526E+04, &
      2.545001E-02, 1.88207E+01, 8.6675E+00, 6.3126E+04, &
      2.568880E-02, 2.00133E+01, 8.4277E+00, 6.0808E+04, &
      2.720606E-02, 2.26397E+01, 7.0963E+00, 4.8347E+04, &
      2.908327E-02, 2.36170E+01, 5.8464E+00, 3.7260E+04, &
      3.109002E-02, 2.39791E+01, 4.9015E+00, 2.9222E+04, &
      3.323523E-02, 2.40796E+01, 4.1683E+00, 2.3247E+04, &
      3.552846E-02, 2.40336E+01, 3.5844E+00, 1.8700E+04, &
      3.797993E-02, 2.38786E+01, 3.1087E+00, 1.5172E+04, &
      3.965432E-02, 2.36941E+01, 2.8454E+00, 1.3300E+04, &
      4.026127E-02, 2.35483E+01, 2.7596E+00, 1.2705E+04, &
      4.042313E-02, 2.34121E+01, 2.7375E+00, 1.2552E+04, &
      4.060054E-02, 2.35007E+01, 2.9602E+00, 1.3514E+04, &
      4.066591E-02, 2.35279E+01, 2.9500E+00, 1.3446E+04, &
      4.127286E-02, 2.35965E+01, 2.8585E+00, 1.2837E+04, &
      4.340198E-02, 2.35120E+01, 2.5728E+00, 1.0987E+04, &
      4.639671E-02, 2.32370E+01, 2.2463E+00, 8.9737E+03, &
      4.959809E-02, 2.28483E+01, 1.9720E+00, 7.3695E+03, &
      5.302035E-02, 2.23395E+01, 1.7381E+00, 6.0765E+03, &
      5.667876E-02, 2.16788E+01, 1.5367E+00, 5.0256E+03, &
      6.058959E-02, 2.07723E+01, 1.3621E+00, 4.1671E+03, &
      6.477028E-02, 1.94632E+01, 1.2104E+00, 3.4639E+03, &
      6.923942E-02, 1.73383E+01, 1.0785E+00, 2.8873E+03, &
      7.401695E-02, 1.26956E+01, 9.6397E-01, 2.4142E+03, &
      7.540014E-02, 9.64471E+00, 9.3502E-01, 2.2987E+03, &
      7.655423E-02, 3.21458E+00, 9.1214E-01, 2.2087E+03, &
      7.686198E-02, -4.10506E+00, 9.0623E-01, 2.1856E+03, &
      7.732361E-02, 3.08047E+00, 1.5066E+01, 3.6115E+04, &
      7.847770E-02, 9.02096E+00, 1.4994E+01, 3.5415E+04, &
      7.912411E-02, 1.03552E+01, 1.4955E+01, 3.5033E+04, &
      8.147337E-02, 1.18469E+01, 1.4817E+01, 3.3710E+04, &
      8.272042E-02, 9.52342E+00, 1.4747E+01, 3.3044E+04, &
      8.305296E-02, 5.52554E+00, 1.4729E+01, 3.2871E+04, &
      8.355178E-02, 1.03287E+01, 2.3173E+01, 5.1408E+04, &
      8.458368E-02, 1.46493E+01, 2.3176E+01, 5.0787E+04, &
      8.479882E-02, 1.52205E+01, 2.3176E+01, 5.0659E+04, &
      9.041995E-02, 2.39206E+01, 2.2422E+01, 4.5962E+04, &
      9.665893E-02, 2.97628E+01, 2.0186E+01, 3.8708E+04, &
      1.033284E-01, 3.34927E+01, 1.6702E+01, 2.9961E+04, &
      1.104581E-01, 3.52903E+01, 1.3035E+01, 2.1874E+04, &
      1.180797E-01, 3.57040E+01, 9.8284E+00, 1.5429E+04, &
      1.262272E-01, 3.52888E+01, 7.3068E+00, 1.0730E+04, &
      1.349368E-01, 3.44492E+01, 5.4453E+00, 7.4807E+03, &
      1.442475E-01, 3.34164E+01, 4.1195E+00, 5.2943E+03, &
      1.542005E-01, 3.22488E+01, 3.1919E+00, 3.8377E+03, &
      1.599698E-01, 3.14171E+01, 2.8080E+00, 3.2546E+03, &
      1.624183E-01, 3.07548E+01, 2.6703E+00, 3.0486E+03, &
      1.630713E-01, 3.01621E+01, 2.6358E+00, 2.9972E+03, &
      1.640507E-01, 3.06338E+01, 3.6417E+00, 4.1157E+03, &
      1.648404E-01, 3.08022E+01, 3.5977E+00, 4.0466E+03, &
      1.664992E-01, 3.09145E+01, 3.5094E+00, 3.9079E+03, &
      1.762144E-01, 3.06429E+01, 3.0824E+00, 3.2437E+03, &
      1.883732E-01, 2.99615E+01, 2.7103E+00, 2.6683E+03, &
      2.013709E-01, 2.90378E+01, 2.4479E+00, 2.2549E+03, &
      2.020480E-01, 2.89721E+01, 2.4370E+00, 2.2374E+03, &
      2.051405E-01, 2.85233E+01, 2.3900E+00, 2.1613E+03, &
      2.059652E-01, 2.81320E+01, 2.3782E+00, 2.1420E+03, &
      2.072023E-01, 2.84398E+01, 3.0481E+00, 2.7285E+03, &
      2.102948E-01, 2.86015E+01, 2.9782E+00, 2.6269E+03, &
      2.152655E-01, 2.85224E+01, 2.8768E+00, 2.4790E+03, &
      2.301188E-01, 2.79004E+01, 2.6390E+00, 2.1278E+03, &
      2.459970E-01, 2.70176E+01, 2.4705E+00, 1.8639E+03, &
      2.560302E-01, 2.63471E+01, 2.3966E+00, 1.7376E+03, &
      2.599490E-01, 2.60025E+01, 2.3730E+00, 1.6947E+03, &
      2.609940E-01, 2.58305E+01, 2.3671E+00, 1.6838E+03, &
      2.625616E-01, 2.58246E+01, 2.5572E+00, 1.8079E+03, &
      2.629708E-01, 2.58135E+01, 2.5552E+00, 1.8037E+03, &
      2.664804E-01, 2.56364E+01, 2.5393E+00, 1.7691E+03, &
      2.811158E-01, 2.45672E+01, 2.4903E+00, 1.6449E+03, &
      2.891658E-01, 2.38229E+01, 2.4729E+00, 1.5882E+03, &
      2.935918E-01, 2.33060E+01, 2.4656E+00, 1.5598E+03, &
      2.947720E-01, 2.30852E+01, 2.4639E+00, 1.5525E+03, &
      2.965424E-01, 2.29742E+01, 2.6560E+00, 1.6635E+03, &
      2.978648E-01, 2.28596E+01, 2.6690E+00, 1.6642E+03, &
      3.005128E-01, 2.25818E+01, 2.6985E+00, 1.6677E+03, &
      3.009684E-01, 2.25289E+01, 2.7041E+00, 1.6686E+03, &
      3.024240E-01, 2.23454E+01, 2.7229E+00, 1.6722E+03, &
      3.036397E-01, 2.21388E+01, 2.7400E+00, 1.6760E+03, &
      3.054634E-01, 2.19987E+01, 2.9188E+00, 1.7745E+03, &
      3.100226E-01, 2.15241E+01, 3.0429E+00, 1.8228E+03, &
      3.212482E-01, 2.01342E+01, 3.5042E+00, 2.0255E+03, &
      3.434143E-01, 1.70858E+01, 5.3640E+00, 2.8992E+03, &
      3.671099E-01, 1.41668E+01, 8.8327E+00, 4.4639E+03, &
      3.924405E-01, 1.22832E+01, 1.3853E+01, 6.5475E+03, &
      4.195189E-01, 1.21882E+01, 1.9787E+01, 8.7474E+03, &
      4.484657E-01, 1.41098E+01, 2.5675E+01, 1.0618E+04, &
      4.794098E-01, 1.77163E+01, 3.0651E+01, 1.1857E+04, &
      5.0000000E-01, 2.05578E+01, 3.3072E+01, 1.2266E+04, &
      5.0250000E-01, 2.09129E+01, 3.3319E+01, 1.2296E+04, &
      5.0501250E-01, 2.12708E+01, 3.3556E+01, 1.2322E+04, &
      5.0753756E-01, 2.16309E+01, 3.3785E+01, 1.2345E+04, &
      5.1007525E-01, 2.19932E+01, 3.4006E+01, 1.2363E+04, &
      5.1262563E-01, 2.23573E+01, 3.4217E+01, 1.2378E+04, &
      5.1518875E-01, 2.27228E+01, 3.4420E+01, 1.2390E+04, &
      5.1776470E-01, 2.30894E+01, 3.4614E+01, 1.2398E+04, &
      5.2035352E-01, 2.34570E+01, 3.4799E+01, 1.2402E+04, &
      5.2295529E-01, 2.38251E+01, 3.4976E+01, 1.2403E+04, &
      5.2557007E-01, 2.41934E+01, 3.5145E+01, 1.2401E+04, &
      5.2819792E-01, 2.45618E+01, 3.5304E+01, 1.2395E+04, &
      5.3083891E-01, 2.49299E+01, 3.5456E+01, 1.2387E+04, &
      5.3349310E-01, 2.52973E+01, 3.5599E+01, 1.2375E+04, &
      5.3616057E-01, 2.56639E+01, 3.5734E+01, 1.2360E+04, &
      5.3884137E-01, 2.60293E+01, 3.5860E+01, 1.2342E+04, &
      5.4153558E-01, 2.63933E+01, 3.5979E+01, 1.2321E+04, &
      5.4424325E-01, 2.67556E+01, 3.6090E+01, 1.2298E+04, &
      5.4696447E-01, 2.71159E+01, 3.6192E+01, 1.2271E+04, &
      5.4969929E-01, 2.74739E+01, 3.6287E+01, 1.2242E+04, &
      5.5244779E-01, 2.78293E+01, 3.6375E+01, 1.2211E+04, &
      5.5521003E-01, 2.81819E+01, 3.6454E+01, 1.2177E+04, &
      5.5798608E-01, 2.85313E+01, 3.6527E+01, 1.2140E+04, &
      5.6077601E-01, 2.88773E+01, 3.6592E+01, 1.2102E+04, &
      5.6357989E-01, 2.92195E+01, 3.6650E+01, 1.2060E+04, &
      5.6639779E-01, 2.95577E+01, 3.6701E+01, 1.2017E+04, &
      5.6922978E-01, 2.98916E+01, 3.6745E+01, 1.1972E+04, &
      5.7207593E-01, 3.02207E+01, 3.6782E+01, 1.1924E+04, &
      5.7493630E-01, 3.05447E+01, 3.6812E+01, 1.1875E+04, &
      5.7781099E-01, 3.08632E+01, 3.6836E+01, 1.1824E+04, &
      5.8070004E-01, 3.11758E+01, 3.6854E+01, 1.1770E+04, &
      5.8360354E-01, 3.14821E+01, 3.6865E+01, 1.1716E+04, &
      5.8652156E-01, 3.17815E+01, 3.6871E+01, 1.1659E+04, &
      5.8945417E-01, 3.20734E+01, 3.6870E+01, 1.1601E+04, &
      5.9240144E-01, 3.23573E+01, 3.6863E+01, 1.1541E+04, &
      5.9536345E-01, 3.26322E+01, 3.6851E+01, 1.1480E+04, &
      5.9834026E-01, 3.28975E+01, 3.6834E+01, 1.1418E+04, &
      6.0133196E-01, 3.31518E+01, 3.6810E+01, 1.1354E+04, &
      6.0433862E-01, 3.33940E+01, 3.6782E+01, 1.1289E+04, &
      6.0736032E-01, 3.36223E+01, 3.6749E+01, 1.1222E+04, &
      6.1039712E-01, 3.38345E+01, 3.6710E+01, 1.1155E+04, &
      6.1344910E-01, 3.40278E+01, 3.6667E+01, 1.1086E+04, &
      6.1651635E-01, 3.41978E+01, 3.6619E+01, 1.1017E+04, &
      6.1959893E-01, 3.43407E+01, 3.6566E+01, 1.0946E+04, &
      6.2269693E-01, 3.44428E+01, 3.6509E+01, 1.0875E+04, &
      6.2581041E-01, 3.44897E+01, 3.6447E+01, 1.0803E+04, &
      6.2893946E-01, 3.44485E+01, 3.6381E+01, 1.0729E+04, &
      6.3208416E-01, 3.42369E+01, 3.6312E+01, 1.0656E+04, &
      6.3524458E-01, 3.34960E+01, 3.6238E+01, 1.0581E+04, &
      6.3664076E-01, 3.20703E+01, 3.6204E+01, 1.0548E+04, &
      6.3735929E-01, 3.21221E+01, 3.9221E+01, 1.1414E+04, &
      6.3842080E-01, 3.35266E+01, 3.9202E+01, 1.1389E+04, &
      6.4161291E-01, 3.48988E+01, 3.9142E+01, 1.1315E+04, &
      6.4482097E-01, 3.56393E+01, 3.9080E+01, 1.1241E+04, &
      6.4804508E-01, 3.61952E+01, 3.9013E+01, 1.1166E+04, &
      6.5128530E-01, 3.66550E+01, 3.8944E+01, 1.1091E+04, &
      6.5454173E-01, 3.70503E+01, 3.8872E+01, 1.1016E+04, &
      6.5781444E-01, 3.73935E+01, 3.8797E+01, 1.0940E+04, &
      6.6110351E-01, 3.76871E+01, 3.8718E+01, 1.0863E+04, &
      6.6440903E-01, 3.79238E+01, 3.8638E+01, 1.0787E+04, &
      6.6773107E-01, 3.80795E+01, 3.8554E+01, 1.0710E+04, &
      6.7106973E-01, 3.80741E+01, 3.8468E+01, 1.0633E+04, &
      6.7442508E-01, 3.71905E+01, 3.8380E+01, 1.0556E+04, &
      6.7451800E-01, 3.70715E+01, 3.8377E+01, 1.0554E+04, &
      6.7528199E-01, 3.71469E+01, 4.0210E+01, 1.1045E+04, &
      6.7779720E-01, 3.85905E+01, 4.0153E+01, 1.0989E+04, &
      6.8118619E-01, 3.93788E+01, 4.0075E+01, 1.0913E+04, &
      6.8459212E-01, 3.99600E+01, 3.9995E+01, 1.0837E+04, &
      6.8801508E-01, 4.04583E+01, 3.9913E+01, 1.0761E+04, &
      6.9145515E-01, 4.09098E+01, 3.9829E+01, 1.0685E+04, &
      6.9491243E-01, 4.13303E+01, 3.9743E+01, 1.0609E+04, &
      6.9838699E-01, 4.17282E+01, 3.9656E+01, 1.0533E+04, &
      7.0187893E-01, 4.21086E+01, 3.9567E+01, 1.0457E+04, &
      7.0538832E-01, 4.24747E+01, 3.9476E+01, 1.0381E+04, &
      7.0891526E-01, 4.28287E+01, 3.9383E+01, 1.0306E+04, &
      7.1245984E-01, 4.31722E+01, 3.9289E+01, 1.0230E+04, &
      7.1602214E-01, 4.35062E+01, 3.9193E+01, 1.0154E+04, &
      7.1960225E-01, 4.38317E+01, 3.9095E+01, 1.0078E+04, &
      7.2320026E-01, 4.41494E+01, 3.8995E+01, 1.0003E+04, &
      7.2681626E-01, 4.44598E+01, 3.8894E+01, 9.9273E+03, &
      7.3045034E-01, 4.47635E+01, 3.8792E+01, 9.8520E+03, &
      7.3410260E-01, 4.50609E+01, 3.8688E+01, 9.7769E+03, &
      7.3777311E-01, 4.53523E+01, 3.8583E+01, 9.7019E+03, &
      7.4146197E-01, 4.56380E+01, 3.8477E+01, 9.6272E+03, &
      7.4516928E-01, 4.59184E+01, 3.8369E+01, 9.5526E+03, &
      7.4889513E-01, 4.61936E+01, 3.8260E+01, 9.4782E+03, &
      7.5263961E-01, 4.64639E+01, 3.8150E+01, 9.4040E+03, &
      7.5640280E-01, 4.67291E+01, 3.8039E+01, 9.3300E+03, &
      7.6018482E-01, 4.69895E+01, 3.7926E+01, 9.2562E+03, &
      7.6398574E-01, 4.72451E+01, 3.7812E+01, 9.1825E+03, &
      7.6780567E-01, 4.74959E+01, 3.7697E+01, 9.1092E+03, &
      7.7164470E-01, 4.77419E+01, 3.7581E+01, 9.0360E+03, &
      7.7550292E-01, 4.79833E+01, 3.7464E+01, 8.9631E+03, &
      7.7938044E-01, 4.82201E+01, 3.7346E+01, 8.8905E+03, &
      7.8327734E-01, 4.84522E+01, 3.7227E+01, 8.8182E+03, &
      7.8719373E-01, 4.86809E+01, 3.7107E+01, 8.7462E+03, &
      7.9112969E-01, 4.89038E+01, 3.6987E+01, 8.6746E+03, &
      7.9508534E-01, 4.91221E+01, 3.6865E+01, 8.6032E+03, &
      7.9906077E-01, 4.93356E+01, 3.6743E+01, 8.5322E+03, &
      8.0305607E-01, 4.95445E+01, 3.6621E+01, 8.4616E+03, &
      8.0707135E-01, 4.97485E+01, 3.6498E+01, 8.3913E+03, &
      8.1110671E-01, 4.99475E+01, 3.6374E+01, 8.3214E+03, &
      8.1516224E-01, 5.01415E+01, 3.6250E+01, 8.2518E+03, &
      8.1923806E-01, 5.03303E+01, 3.6126E+01, 8.1827E+03, &
      8.2333425E-01, 5.05137E+01, 3.6001E+01, 8.1140E+03, &
      8.2745092E-01, 5.06914E+01, 3.5876E+01, 8.0457E+03, &
      8.3158817E-01, 5.08630E+01, 3.5751E+01, 7.9778E+03, &
      8.3574611E-01, 5.10282E+01, 3.5625E+01, 7.9103E+03, &
      8.3992484E-01, 5.11864E+01, 3.5499E+01, 7.8433E+03, &
      8.4412447E-01, 5.13369E+01, 3.5373E+01, 7.7767E+03, &
      8.4834509E-01, 5.14788E+01, 3.5247E+01, 7.7105E+03, &
      8.5258682E-01, 5.16109E+01, 3.5121E+01, 7.6449E+03, &
      8.5684975E-01, 5.17315E+01, 3.4995E+01, 7.5796E+03, &
      8.6113400E-01, 5.18383E+01, 3.4869E+01, 7.5148E+03, &
      8.6543967E-01, 5.19286E+01, 3.4743E+01, 7.4505E+03, &
      8.6976687E-01, 5.19947E+01, 3.4616E+01, 7.3865E+03, &
      8.7411570E-01, 5.20225E+01, 3.4489E+01, 7.3228E+03, &
      8.7848628E-01, 5.19931E+01, 3.4361E+01, 7.2595E+03, &
      8.8287871E-01, 5.18525E+01, 3.4233E+01, 7.1966E+03, &
      8.8729310E-01, 5.14100E+01, 3.4106E+01, 7.1343E+03, &
      8.8919009E-01, 5.06616E+01, 3.4051E+01, 7.1077E+03, &
      8.9080989E-01, 5.07289E+01, 3.6159E+01, 7.5334E+03, &
      8.9172957E-01, 5.12871E+01, 3.6133E+01, 7.5202E+03, &
      8.9618822E-01, 5.23436E+01, 3.6007E+01, 7.4568E+03, &
      9.0066916E-01, 5.28979E+01, 3.5881E+01, 7.3939E+03, &
      9.0517250E-01, 5.33184E+01, 3.5756E+01, 7.3315E+03, &
      9.0969837E-01, 5.36747E+01, 3.5630E+01, 7.2695E+03, &
      9.1424686E-01, 5.40175E+01, 3.5505E+01, 7.2080E+03, &
      9.1881809E-01, 5.43100E+01, 3.5380E+01, 7.1470E+03, &
      9.2341218E-01, 5.45838E+01, 3.5255E+01, 7.0865E+03, &
      9.2802924E-01, 5.48434E+01, 3.5131E+01, 7.0265E+03, &
      9.3266939E-01, 5.50916E+01, 3.5007E+01, 6.9669E+03, &
      9.3733274E-01, 5.53305E+01, 3.4883E+01, 6.9078E+03, &
      9.4201940E-01, 5.55617E+01, 3.4759E+01, 6.8492E+03, &
      9.4672950E-01, 5.57862E+01, 3.4636E+01, 6.7911E+03, &
      9.5146315E-01, 5.60050E+01, 3.4513E+01, 6.7335E+03, &
      9.5622046E-01, 5.62188E+01, 3.4391E+01, 6.6764E+03, &
      9.6100156E-01, 5.64283E+01, 3.4269E+01, 6.6197E+03, &
      9.6580657E-01, 5.66341E+01, 3.4148E+01, 6.5635E+03, &
      9.7063560E-01, 5.68366E+01, 3.4027E+01, 6.5079E+03, &
      9.7548878E-01, 5.70363E+01, 3.3906E+01, 6.4526E+03, &
      9.8036623E-01, 5.72337E+01, 3.3786E+01, 6.3979E+03, &
      9.8526806E-01, 5.74294E+01, 3.3666E+01, 6.3437E+03, &
      9.9019440E-01, 5.76240E+01, 3.3547E+01, 6.2899E+03, &
      9.9514537E-01, 5.78185E+01, 3.3428E+01, 6.2365E+03, &
      1.0001211E+00, 5.80156E+01, 3.3310E+01, 6.1836E+03, &
      1.0051217E+00, 5.82161E+01, 3.3175E+01, 6.1281E+03, &
      1.0101473E+00, 5.83998E+01, 3.3040E+01, 6.0729E+03, &
      1.0151980E+00, 5.85760E+01, 3.2906E+01, 6.0182E+03, &
      1.0202740E+00, 5.87470E+01, 3.2771E+01, 5.9639E+03, &
      1.0253754E+00, 5.89135E+01, 3.2636E+01, 5.9100E+03, &
      1.0305023E+00, 5.90759E+01, 3.2502E+01, 5.8565E+03, &
      1.0356548E+00, 5.92341E+01, 3.2368E+01, 5.8034E+03, &
      1.0408331E+00, 5.93879E+01, 3.2234E+01, 5.7507E+03, &
      1.0460372E+00, 5.95369E+01, 3.2100E+01, 5.6985E+03, &
      1.0512674E+00, 5.96805E+01, 3.1966E+01, 5.6466E+03, &
      1.0565238E+00, 5.98173E+01, 3.1833E+01, 5.5952E+03, &
      1.0618064E+00, 5.99448E+01, 3.1700E+01, 5.5442E+03, &
      1.0671154E+00, 6.00580E+01, 3.1567E+01, 5.4936E+03, &
      1.0724510E+00, 6.01425E+01, 3.1435E+01, 5.4435E+03, &
      1.0778132E+00, 6.01234E+01, 3.1300E+01, 5.3934E+03, &
      1.0789287E+00, 6.00524E+01, 3.1270E+01, 5.3827E+03, &
      1.0810714E+00, 6.01242E+01, 3.1672E+01, 5.4409E+03, &
      1.0832023E+00, 6.03467E+01, 3.1615E+01, 5.4205E+03, &
      1.0886183E+00, 6.06471E+01, 3.1470E+01, 5.3689E+03, &
      1.0940614E+00, 6.08712E+01, 3.1326E+01, 5.3178E+03, &
      1.0995317E+00, 6.10691E+01, 3.1182E+01, 5.2672E+03, &
      1.1050294E+00, 6.12526E+01, 3.1039E+01, 5.2170E+03, &
      1.1105545E+00, 6.14267E+01, 3.0896E+01, 5.1673E+03, &
      1.1161073E+00, 6.15938E+01, 3.0753E+01, 5.1180E+03, &
      1.1216878E+00, 6.17553E+01, 3.0611E+01, 5.0692E+03, &
      1.1272963E+00, 6.19122E+01, 3.0470E+01, 5.0208E+03, &
      1.1329328E+00, 6.20650E+01, 3.0329E+01, 4.9729E+03, &
      1.1385974E+00, 6.22145E+01, 3.0191E+01, 4.9257E+03, &
      1.1442904E+00, 6.23612E+01, 3.0048E+01, 4.8781E+03, &
      1.1500119E+00, 6.25043E+01, 2.9903E+01, 4.8305E+03, &
      1.1557619E+00, 6.26440E+01, 2.9758E+01, 4.7833E+03, &
      1.1615407E+00, 6.27808E+01, 2.9614E+01, 4.7366E+03, &
      1.1673484E+00, 6.29146E+01, 2.9471E+01, 4.6904E+03, &
      1.1731852E+00, 6.30457E+01, 2.9328E+01, 4.6446E+03, &
      1.1790511E+00, 6.31742E+01, 2.9186E+01, 4.5993E+03, &
      1.1849464E+00, 6.32995E+01, 2.9027E+01, 4.5515E+03, &
      1.1908711E+00, 6.34205E+01, 2.8864E+01, 4.5035E+03, &
      1.1968254E+00, 6.35372E+01, 2.8702E+01, 4.4561E+03, &
      1.2028096E+00, 6.36497E+01, 2.8541E+01, 4.4092E+03, &
      1.2088236E+00, 6.37578E+01, 2.8381E+01, 4.3629E+03, &
      1.2148677E+00, 6.38615E+01, 2.8223E+01, 4.3171E+03, &
      1.2209421E+00, 6.39598E+01, 2.8051E+01, 4.2696E+03, &
      1.2270468E+00, 6.40516E+01, 2.7878E+01, 4.2223E+03, &
      1.2331820E+00, 6.41359E+01, 2.7707E+01, 4.1756E+03, &
      1.2393479E+00, 6.42116E+01, 2.7537E+01, 4.1295E+03, &
      1.2455447E+00, 6.42763E+01, 2.7369E+01, 4.0840E+03, &
      1.2517724E+00, 6.43250E+01, 2.7201E+01, 4.0390E+03, &
      1.2580312E+00, 6.43461E+01, 2.7036E+01, 3.9946E+03, &
      1.2643214E+00, 6.42950E+01, 2.6872E+01, 3.9507E+03, &
      1.2675407E+00, 6.41421E+01, 2.6789E+01, 3.9286E+03, &
      1.2704594E+00, 6.41962E+01, 2.7255E+01, 3.9877E+03, &
      1.2706430E+00, 6.42193E+01, 2.7251E+01, 3.9864E+03, &
      1.2769962E+00, 6.45829E+01, 2.7090E+01, 3.9433E+03, &
      1.2833812E+00, 6.47727E+01, 2.6930E+01, 3.9007E+03, &
      1.2897981E+00, 6.49226E+01, 2.6772E+01, 3.8587E+03, &
      1.2962471E+00, 6.50532E+01, 2.6615E+01, 3.8171E+03, &
      1.3027283E+00, 6.51720E+01, 2.6460E+01, 3.7760E+03, &
      1.3092420E+00, 6.52823E+01, 2.6305E+01, 3.7355E+03, &
      1.3157882E+00, 6.53862E+01, 2.6152E+01, 3.6954E+03, &
      1.3223671E+00, 6.54849E+01, 2.6000E+01, 3.6558E+03, &
      1.3289790E+00, 6.55792E+01, 2.5850E+01, 3.6167E+03, &
      1.3356239E+00, 6.56698E+01, 2.5700E+01, 3.5781E+03, &
      1.3423020E+00, 6.57569E+01, 2.5552E+01, 3.5399E+03, &
      1.3490135E+00, 6.58410E+01, 2.5405E+01, 3.5021E+03, &
      1.3557586E+00, 6.59222E+01, 2.5259E+01, 3.4648E+03, &
      1.3625374E+00, 6.60009E+01, 2.5115E+01, 3.4280E+03, &
      1.3693500E+00, 6.60772E+01, 2.4971E+01, 3.3916E+03, &
      1.3761968E+00, 6.61514E+01, 2.4829E+01, 3.3556E+03, &
      1.3830778E+00, 6.62234E+01, 2.4688E+01, 3.3201E+03, &
      1.3899932E+00, 6.62936E+01, 2.4548E+01, 3.2849E+03, &
      1.3969431E+00, 6.63620E+01, 2.4409E+01, 3.2502E+03, &
      1.4039278E+00, 6.64287E+01, 2.4271E+01, 3.2159E+03, &
      1.4109475E+00, 6.64937E+01, 2.4133E+01, 3.1819E+03, &
      1.4180022E+00, 6.65568E+01, 2.3996E+01, 3.1483E+03, &
      1.4250922E+00, 6.66182E+01, 2.3861E+01, 3.1150E+03, &
      1.4322177E+00, 6.66780E+01, 2.3726E+01, 3.0822E+03, &
      1.4393788E+00, 6.67363E+01, 2.3592E+01, 3.0497E+03, &
      1.4465757E+00, 6.67930E+01, 2.3459E+01, 3.0176E+03, &
      1.4538086E+00, 6.68484E+01, 2.3326E+01, 2.9856E+03, &
      1.4610776E+00, 6.69023E+01, 2.3194E+01, 2.9540E+03, &
      1.4683830E+00, 6.69549E+01, 2.3062E+01, 2.9228E+03, &
      1.4757249E+00, 6.70061E+01, 2.2932E+01, 2.8920E+03, &
      1.4831035E+00, 6.70561E+01, 2.2803E+01, 2.8615E+03, &
      1.4905190E+00, 6.71049E+01, 2.2674E+01, 2.8314E+03, &
      1.4979716E+00, 6.71526E+01, 2.2547E+01, 2.8016E+03, &
      1.5054615E+00, 6.71992E+01, 2.2420E+01, 2.7721E+03, &
      1.5129888E+00, 6.72442E+01, 2.2293E+01, 2.7428E+03, &
      1.5205537E+00, 6.72878E+01, 2.2166E+01, 2.7138E+03, &
      1.5281565E+00, 6.73301E+01, 2.2040E+01, 2.6851E+03, &
      1.5357973E+00, 6.73711E+01, 2.1915E+01, 2.6567E+03, &
      1.5434763E+00, 6.74108E+01, 2.1791E+01, 2.6287E+03, &
      1.5511937E+00, 6.74493E+01, 2.1668E+01, 2.6010E+03, &
      1.5589496E+00, 6.74865E+01, 2.1546E+01, 2.5736E+03, &
      1.5667444E+00, 6.75227E+01, 2.1424E+01, 2.5464E+03, &
      1.5745781E+00, 6.75576E+01, 2.1303E+01, 2.5195E+03, &
      1.5824510E+00, 6.75914E+01, 2.1182E+01, 2.4929E+03, &
      1.5903633E+00, 6.76241E+01, 2.1062E+01, 2.4666E+03, &
      1.5983151E+00, 6.76556E+01, 2.0944E+01, 2.4407E+03, &
      1.6063066E+00, 6.76860E+01, 2.0826E+01, 2.4150E+03, &
      1.6143382E+00, 6.77154E+01, 2.0709E+01, 2.3896E+03, &
      1.6224099E+00, 6.77438E+01, 2.0593E+01, 2.3645E+03, &
      1.6305219E+00, 6.77712E+01, 2.0477E+01, 2.3397E+03, &
      1.6386745E+00, 6.77975E+01, 2.0363E+01, 2.3152E+03, &
      1.6468679E+00, 6.78230E+01, 2.0249E+01, 2.2909E+03, &
      1.6551022E+00, 6.78474E+01, 2.0136E+01, 2.2670E+03, &
      1.6633777E+00, 6.78709E+01, 2.0024E+01, 2.2432E+03, &
      1.6716946E+00, 6.78932E+01, 1.9912E+01, 2.2198E+03, &
      1.6800531E+00, 6.79146E+01, 1.9802E+01, 2.1966E+03, &
      1.6884534E+00, 6.79351E+01, 1.9692E+01, 2.1736E+03, &
      1.6968956E+00, 6.79545E+01, 1.9583E+01, 2.1510E+03, &
      1.7053801E+00, 6.79730E+01, 1.9474E+01, 2.1286E+03, &
      1.7139070E+00, 6.79906E+01, 1.9367E+01, 2.1064E+03, &
      1.7224766E+00, 6.80073E+01, 1.9260E+01, 2.0845E+03, &
      1.7310889E+00, 6.80230E+01, 1.9154E+01, 2.0629E+03, &
      1.7397444E+00, 6.80379E+01, 1.9049E+01, 2.0415E+03, &
      1.7484431E+00, 6.80518E+01, 1.8945E+01, 2.0203E+03, &
      1.7571853E+00, 6.80649E+01, 1.8841E+01, 1.9994E+03, &
      1.7659712E+00, 6.80772E+01, 1.8738E+01, 1.9787E+03, &
      1.7748011E+00, 6.80885E+01, 1.8636E+01, 1.9583E+03, &
      1.7836751E+00, 6.80990E+01, 1.8535E+01, 1.9380E+03, &
      1.7925935E+00, 6.81086E+01, 1.8434E+01, 1.9181E+03, &
      1.8015565E+00, 6.81174E+01, 1.8334E+01, 1.8983E+03, &
      1.8105642E+00, 6.81254E+01, 1.8235E+01, 1.8788E+03, &
      1.8196171E+00, 6.81325E+01, 1.8136E+01, 1.8595E+03, &
      1.8287151E+00, 6.81388E+01, 1.8039E+01, 1.8404E+03, &
      1.8378587E+00, 6.81443E+01, 1.7942E+01, 1.8215E+03, &
      1.8470480E+00, 6.81489E+01, 1.7845E+01, 1.8028E+03, &
      1.8562833E+00, 6.81528E+01, 1.7750E+01, 1.7844E+03, &
      1.8655647E+00, 6.81558E+01, 1.7655E+01, 1.7661E+03, &
      1.8748925E+00, 6.81580E+01, 1.7561E+01, 1.7481E+03, &
      1.8842670E+00, 6.81594E+01, 1.7467E+01, 1.7302E+03, &
      1.8936883E+00, 6.81599E+01, 1.7374E+01, 1.7126E+03, &
      1.9031567E+00, 6.81597E+01, 1.7282E+01, 1.6952E+03, &
      1.9126725E+00, 6.81562E+01, 1.7190E+01, 1.6779E+03, &
      1.9222359E+00, 6.81544E+01, 1.7100E+01, 1.6609E+03, &
      1.9318471E+00, 6.81518E+01, 1.7009E+01, 1.6440E+03, &
      1.9415063E+00, 6.81484E+01, 1.6920E+01, 1.6274E+03, &
      1.9512138E+00, 6.81441E+01, 1.6831E+01, 1.6109E+03, &
      1.9609699E+00, 6.81391E+01, 1.6743E+01, 1.5946E+03, &
      1.9707747E+00, 6.81333E+01, 1.6655E+01, 1.5785E+03, &
      1.9806286E+00, 6.81266E+01, 1.6568E+01, 1.5625E+03, &
      1.9905318E+00, 6.81191E+01, 1.6482E+01, 1.5468E+03, &
      2.0004844E+00, 6.81108E+01, 1.6396E+01, 1.5312E+03, &
      2.0104868E+00, 6.81016E+01, 1.6311E+01, 1.5158E+03, &
      2.0205393E+00, 6.80916E+01, 1.6226E+01, 1.5006E+03, &
      2.0306420E+00, 6.80843E+01, 1.6143E+01, 1.4855E+03, &
      2.0407952E+00, 6.80728E+01, 1.6059E+01, 1.4706E+03, &
      2.0509992E+00, 6.80603E+01, 1.5977E+01, 1.4559E+03, &
      2.0612542E+00, 6.80470E+01, 1.5895E+01, 1.4413E+03, &
      2.0715604E+00, 6.80329E+01, 1.5813E+01, 1.4269E+03, &
      2.0819182E+00, 6.80179E+01, 1.5732E+01, 1.4127E+03, &
      2.0923278E+00, 6.80020E+01, 1.5652E+01, 1.3986E+03, &
      2.1027895E+00, 6.79852E+01, 1.5572E+01, 1.3847E+03, &
      2.1133034E+00, 6.79676E+01, 1.5493E+01, 1.3709E+03, &
      2.1238699E+00, 6.79490E+01, 1.5414E+01, 1.3573E+03, &
      2.1344893E+00, 6.79296E+01, 1.5336E+01, 1.3438E+03, &
      2.1451617E+00, 6.79092E+01, 1.5259E+01, 1.3305E+03, &
      2.1558875E+00, 6.78879E+01, 1.5182E+01, 1.3173E+03, &
      2.1666670E+00, 6.78721E+01, 1.5105E+01, 1.3042E+03, &
      2.1775003E+00, 6.78491E+01, 1.5028E+01, 1.2913E+03, &
      2.1883878E+00, 6.78251E+01, 1.4952E+01, 1.2784E+03, &
      2.1993297E+00, 6.77999E+01, 1.4877E+01, 1.2658E+03, &
      2.2103264E+00, 6.77736E+01, 1.4801E+01, 1.2532E+03, &
      2.2213780E+00, 6.77463E+01, 1.4727E+01, 1.2408E+03, &
      2.2324849E+00, 6.77178E+01, 1.4653E+01, 1.2285E+03, &
      2.2436473E+00, 6.76881E+01, 1.4579E+01, 1.2164E+03, &
      2.2548656E+00, 6.76573E+01, 1.4506E+01, 1.2044E+03, &
      2.2661399E+00, 6.76253E+01, 1.4434E+01, 1.1925E+03, &
      2.2774706E+00, 6.75921E+01, 1.4362E+01, 1.1808E+03, &
      2.2888579E+00, 6.75576E+01, 1.4290E+01, 1.1692E+03, &
      2.3003022E+00, 6.75219E+01, 1.4219E+01, 1.1577E+03, &
      2.3118037E+00, 6.74849E+01, 1.4149E+01, 1.1463E+03, &
      2.3233628E+00, 6.74466E+01, 1.4079E+01, 1.1351E+03, &
      2.3349796E+00, 6.74070E+01, 1.4009E+01, 1.1240E+03, &
      2.3466545E+00, 6.73660E+01, 1.3940E+01, 1.1130E+03, &
      2.3583878E+00, 6.73235E+01, 1.3871E+01, 1.1021E+03, &
      2.3701797E+00, 6.72797E+01, 1.3803E+01, 1.0914E+03, &
      2.3820306E+00, 6.72343E+01, 1.3736E+01, 1.0807E+03, &
      2.3939407E+00, 6.71874E+01, 1.3669E+01, 1.0702E+03, &
      2.4059104E+00, 6.71390E+01, 1.3602E+01, 1.0598E+03, &
      2.4179400E+00, 6.70889E+01, 1.3536E+01, 1.0495E+03, &
      2.4300297E+00, 6.70372E+01, 1.3470E+01, 1.0393E+03, &
      2.4421798E+00, 6.69838E+01, 1.3405E+01, 1.0292E+03, &
      2.4543907E+00, 6.69287E+01, 1.3340E+01, 1.0193E+03, &
      2.4666627E+00, 6.68717E+01, 1.3275E+01, 1.0094E+03, &
      2.4789960E+00, 6.68128E+01, 1.3211E+01, 9.9965E+02, &
      2.4913910E+00, 6.67521E+01, 1.3148E+01, 9.9000E+02, &
      2.5038479E+00, 6.66893E+01, 1.3085E+01, 9.8045E+02, &
      2.5163672E+00, 6.66245E+01, 1.3022E+01, 9.7101E+02, &
      2.5289490E+00, 6.65575E+01, 1.2960E+01, 9.6167E+02, &
      2.5415938E+00, 6.64858E+01, 1.2898E+01, 9.5242E+02, &
      2.5543017E+00, 6.64144E+01, 1.2833E+01, 9.4300E+02, &
      2.5670732E+00, 6.63400E+01, 1.2764E+01, 9.3341E+02, &
      2.5799086E+00, 6.62626E+01, 1.2696E+01, 9.2394E+02, &
      2.5928082E+00, 6.61819E+01, 1.2629E+01, 9.1457E+02, &
      2.6057722E+00, 6.60980E+01, 1.2562E+01, 9.0531E+02, &
      2.6188011E+00, 6.60107E+01, 1.2496E+01, 8.9614E+02, &
      2.6318951E+00, 6.59198E+01, 1.2430E+01, 8.8708E+02, &
      2.6450545E+00, 6.58253E+01, 1.2364E+01, 8.7813E+02, &
      2.6582798E+00, 6.57269E+01, 1.2299E+01, 8.6927E+02, &
      2.6715712E+00, 6.56183E+01, 1.2234E+01, 8.6050E+02, &
      2.6849291E+00, 6.55116E+01, 1.2170E+01, 8.5184E+02, &
      2.6983537E+00, 6.54006E+01, 1.2106E+01, 8.4327E+02, &
      2.7118455E+00, 6.52848E+01, 1.2039E+01, 8.3452E+02, &
      2.7254047E+00, 6.51638E+01, 1.1972E+01, 8.2583E+02, &
      2.7390317E+00, 6.50374E+01, 1.1905E+01, 8.1725E+02, &
      2.7527269E+00, 6.49053E+01, 1.1839E+01, 8.0876E+02, &
      2.7664905E+00, 6.47671E+01, 1.1773E+01, 8.0036E+02, &
      2.7803230E+00, 6.46226E+01, 1.1707E+01, 7.9206E+02, &
      2.7942246E+00, 6.44713E+01, 1.1642E+01, 7.8386E+02, &
      2.8081957E+00, 6.43129E+01, 1.1578E+01, 7.7574E+02, &
      2.8222367E+00, 6.41468E+01, 1.1514E+01, 7.6772E+02, &
      2.8363479E+00, 6.39726E+01, 1.1450E+01, 7.5979E+02, &
      2.8505296E+00, 6.37897E+01, 1.1387E+01, 7.5195E+02, &
      2.8647823E+00, 6.35974E+01, 1.1324E+01, 7.4419E+02, &
      2.8791062E+00, 6.33951E+01, 1.1262E+01, 7.3653E+02, &
      2.8935017E+00, 6.31820E+01, 1.1200E+01, 7.2894E+02, &
      2.9079692E+00, 6.29571E+01, 1.1139E+01, 7.2145E+02, &
      2.9225091E+00, 6.27195E+01, 1.1078E+01, 7.1404E+02, &
      2.9371216E+00, 6.24682E+01, 1.1017E+01, 7.0670E+02, &
      2.9518072E+00, 6.22019E+01, 1.0957E+01, 6.9946E+02, &
      2.9665662E+00, 6.19195E+01, 1.0897E+01, 6.9229E+02, &
      2.9813991E+00, 6.16199E+01, 1.0838E+01, 6.8520E+02, &
      2.9963061E+00, 6.13048E+01, 1.0779E+01, 6.7819E+02, &
      3.0112876E+00, 6.09550E+01, 1.0703E+01, 6.7014E+02, &
      3.0263440E+00, 6.05592E+01, 1.0621E+01, 6.6182E+02, &
      3.0414758E+00, 6.01236E+01, 1.0539E+01, 6.5361E+02, &
      3.0566831E+00, 5.96434E+01, 1.0459E+01, 6.4552E+02, &
      3.0719666E+00, 5.91109E+01, 1.0379E+01, 6.3753E+02, &
      3.0873264E+00, 5.85155E+01, 1.0299E+01, 6.2963E+02, &
      3.1027630E+00, 5.78426E+01, 1.0221E+01, 6.2184E+02, &
      3.1182768E+00, 5.70708E+01, 1.0143E+01, 6.1415E+02, &
      3.1338682E+00, 5.61683E+01, 1.0066E+01, 6.0657E+02, &
      3.1495376E+00, 5.50837E+01, 9.9892E+00, 5.9909E+02, &
      3.1652853E+00, 5.37253E+01, 9.9134E+00, 5.9171E+02, &
      3.1811117E+00, 5.19045E+01, 9.8384E+00, 5.8443E+02, &
      3.1970172E+00, 4.91126E+01, 9.7640E+00, 5.7725E+02, &
      3.2130023E+00, 4.25973E+01, 9.6903E+00, 5.7017E+02, &
      3.2180698E+00, 3.33650E+01, 9.6672E+00, 5.6795E+02, &
      3.2199304E+00, 3.31070E+01, 2.5185E+01, 1.4609E+03, &
      3.2290673E+00, 4.48301E+01, 2.5065E+01, 1.4499E+03, &
      3.2452127E+00, 4.93765E+01, 2.4857E+01, 1.4308E+03, &
      3.2614387E+00, 5.14897E+01, 2.4650E+01, 1.4120E+03, &
      3.2777459E+00, 5.27388E+01, 2.4445E+01, 1.3934E+03, &
      3.2941347E+00, 5.34846E+01, 2.4242E+01, 1.3751E+03, &
      3.3106053E+00, 5.38354E+01, 2.4041E+01, 1.3570E+03, &
      3.3271584E+00, 5.37733E+01, 2.3841E+01, 1.3392E+03, &
      3.3437941E+00, 5.30984E+01, 2.3644E+01, 1.3216E+03, &
      3.3605131E+00, 5.07847E+01, 2.3448E+01, 1.3043E+03, &
      3.3691316E+00, 4.44946E+01, 2.3348E+01, 1.2955E+03, &
      3.3712683E+00, 4.44122E+01, 3.2752E+01, 1.8117E+03, &
      3.3773157E+00, 5.03172E+01, 3.2686E+01, 1.8049E+03, &
      3.3942023E+00, 5.45300E+01, 3.2506E+01, 1.7860E+03, &
      3.4111733E+00, 5.66534E+01, 3.2327E+01, 1.7675E+03, &
      3.4282291E+00, 5.81776E+01, 3.2149E+01, 1.7491E+03, &
      3.4453703E+00, 5.94012E+01, 3.1974E+01, 1.7310E+03, &
      3.4625971E+00, 6.04392E+01, 3.1800E+01, 1.7131E+03, &
      3.4799101E+00, 6.13498E+01, 3.1627E+01, 1.6954E+03, &
      3.4973097E+00, 6.21674E+01, 3.1457E+01, 1.6780E+03, &
      3.5147962E+00, 6.29154E+01, 3.1288E+01, 1.6608E+03, &
      3.5323702E+00, 6.36126E+01, 3.1120E+01, 1.6438E+03, &
      3.5500321E+00, 6.42670E+01, 3.0914E+01, 1.6248E+03, &
      3.5677822E+00, 6.48624E+01, 3.0695E+01, 1.6054E+03, &
      3.5856211E+00, 6.54084E+01, 3.0473E+01, 1.5860E+03, &
      3.6035492E+00, 6.59117E+01, 3.0250E+01, 1.5667E+03, &
      3.6215670E+00, 6.63767E+01, 3.0026E+01, 1.5474E+03, &
      3.6396748E+00, 6.68065E+01, 2.9801E+01, 1.5283E+03, &
      3.6578732E+00, 6.72032E+01, 2.9575E+01, 1.5093E+03, &
      3.6761626E+00, 6.75681E+01, 2.9349E+01, 1.4904E+03, &
      3.6945434E+00, 6.79020E+01, 2.9122E+01, 1.4716E+03, &
      3.7130161E+00, 6.82049E+01, 2.8894E+01, 1.4530E+03, &
      3.7315812E+00, 6.84759E+01, 2.8666E+01, 1.4345E+03, &
      3.7502391E+00, 6.87134E+01, 2.8438E+01, 1.4161E+03, &
      3.7689903E+00, 6.89142E+01, 2.8210E+01, 1.3979E+03, &
      3.7878352E+00, 6.90734E+01, 2.7982E+01, 1.3798E+03, &
      3.8067744E+00, 6.91825E+01, 2.7754E+01, 1.3619E+03, &
      3.8258083E+00, 6.92273E+01, 2.7527E+01, 1.3441E+03, &
      3.8449373E+00, 6.91815E+01, 2.7299E+01, 1.3265E+03, &
      3.8641620E+00, 6.89886E+01, 2.7073E+01, 1.3090E+03, &
      3.8834828E+00, 6.84912E+01, 2.6846E+01, 1.2917E+03, &
      3.9029002E+00, 6.67518E+01, 2.6621E+01, 1.2746E+03, &
      3.9040746E+00, 6.64653E+01, 2.6607E+01, 1.2736E+03, &
      3.9139253E+00, 6.65373E+01, 3.0960E+01, 1.4766E+03, &
      3.9224147E+00, 6.80869E+01, 3.0857E+01, 1.4685E+03, &
      3.9420268E+00, 6.96429E+01, 3.0620E+01, 1.4501E+03, &
      3.9617369E+00, 7.05704E+01, 3.0384E+01, 1.4319E+03, &
      3.9815456E+00, 7.12747E+01, 3.0149E+01, 1.4138E+03, &
      4.0014533E+00, 7.18600E+01, 2.9915E+01, 1.3960E+03, &
      4.0214606E+00, 7.23694E+01, 2.9683E+01, 1.3784E+03, &
      4.0415679E+00, 7.28250E+01, 2.9452E+01, 1.3609E+03, &
      4.0617757E+00, 7.32399E+01, 2.9222E+01, 1.3437E+03, &
      4.0820846E+00, 7.36222E+01, 2.8993E+01, 1.3266E+03, &
      4.1024950E+00, 7.39779E+01, 2.8766E+01, 1.3098E+03, &
      4.1230075E+00, 7.43107E+01, 2.8538E+01, 1.2931E+03, &
      4.1436226E+00, 7.46230E+01, 2.8313E+01, 1.2766E+03, &
      4.1643407E+00, 7.49174E+01, 2.8088E+01, 1.2603E+03, &
      4.1851624E+00, 7.51960E+01, 2.7865E+01, 1.2442E+03, &
      4.2060882E+00, 7.54589E+01, 2.7639E+01, 1.2280E+03, &
      4.2271186E+00, 7.57070E+01, 2.7414E+01, 1.2121E+03, &
      4.2482542E+00, 7.59412E+01, 2.7190E+01, 1.1963E+03, &
      4.2694955E+00, 7.61625E+01, 2.6969E+01, 1.1808E+03, &
      4.2908430E+00, 7.63714E+01, 2.6749E+01, 1.1654E+03, &
      4.3122972E+00, 7.65684E+01, 2.6530E+01, 1.1503E+03, &
      4.3338587E+00, 7.67539E+01, 2.6314E+01, 1.1353E+03, &
      4.3555280E+00, 7.69280E+01, 2.6099E+01, 1.1206E+03, &
      4.3773056E+00, 7.70909E+01, 2.5887E+01, 1.1060E+03, &
      4.3991921E+00, 7.72429E+01, 2.5678E+01, 1.0917E+03, &
      4.4211881E+00, 7.73842E+01, 2.5472E+01, 1.0777E+03, &
      4.4432940E+00, 7.75143E+01, 2.5267E+01, 1.0638E+03, &
      4.4655105E+00, 7.76325E+01, 2.5065E+01, 1.0501E+03, &
      4.4878381E+00, 7.77375E+01, 2.4864E+01, 1.0366E+03, &
      4.5102772E+00, 7.78276E+01, 2.4664E+01, 1.0233E+03, &
      4.5328286E+00, 7.78997E+01, 2.4467E+01, 1.0101E+03, &
      4.5554928E+00, 7.79492E+01, 2.4271E+01, 9.9718E+02, &
      4.5782702E+00, 7.79671E+01, 2.4077E+01, 9.8438E+02, &
      4.6011616E+00, 7.79352E+01, 2.3885E+01, 9.7176E+02, &
      4.6241674E+00, 7.78034E+01, 2.3694E+01, 9.5931E+02, &
      4.6472882E+00, 7.73019E+01, 2.3505E+01, 9.4702E+02, &
      4.6489696E+00, 7.72070E+01, 2.3491E+01, 9.4614E+02, &
      4.6630307E+00, 7.72566E+01, 2.4902E+01, 9.9935E+02, &
      4.6705247E+00, 7.76476E+01, 2.4841E+01, 9.9534E+02, &
      4.6938773E+00, 7.82289E+01, 2.4652E+01, 9.8294E+02, &
      4.7173467E+00, 7.85728E+01, 2.4464E+01, 9.7070E+02, &
      4.7409334E+00, 7.88341E+01, 2.4278E+01, 9.5863E+02, &
      4.7646381E+00, 7.90506E+01, 2.4094E+01, 9.4671E+02, &
      4.7884613E+00, 7.92369E+01, 2.3911E+01, 9.3495E+02, &
      4.8124036E+00, 7.94000E+01, 2.3730E+01, 9.2334E+02, &
      4.8364656E+00, 7.95434E+01, 2.3550E+01, 9.1189E+02, &
      4.8606479E+00, 7.96681E+01, 2.3373E+01, 9.0059E+02, &
      4.8849512E+00, 7.97736E+01, 2.3196E+01, 8.8944E+02, &
      4.9093759E+00, 7.98567E+01, 2.3022E+01, 8.7846E+02, &
      4.9339228E+00, 7.99097E+01, 2.2849E+01, 8.6762E+02, &
      4.9585924E+00, 7.99121E+01, 2.2678E+01, 8.5693E+02, &
      4.9833854E+00, 7.97849E+01, 2.2509E+01, 8.4638E+02, &
      4.9909955E+00, 7.96615E+01, 2.2457E+01, 8.4318E+02, &
      5.0083023E+00, 7.95717E+01, 2.3324E+01, 8.7236E+02, &
      5.0130043E+00, 7.97706E+01, 2.3292E+01, 8.7038E+02, &
      5.0333438E+00, 8.02020E+01, 2.3157E+01, 8.6190E+02, &
      5.0585105E+00, 8.05108E+01, 2.2992E+01, 8.5159E+02, &
      5.0838031E+00, 8.07472E+01, 2.2828E+01, 8.4140E+02, &
      5.1092221E+00, 8.09486E+01, 2.2664E+01, 8.3125E+02, &
      5.1347682E+00, 8.11283E+01, 2.2501E+01, 8.2125E+02, &
      5.1604421E+00, 8.12929E+01, 2.2339E+01, 8.1137E+02, &
      5.1862443E+00, 8.14461E+01, 2.2178E+01, 8.0162E+02, &
      5.2121755E+00, 8.15903E+01, 2.2019E+01, 7.9199E+02, &
      5.2382364E+00, 8.17270E+01, 2.1862E+01, 7.8249E+02, &
      5.2644276E+00, 8.18575E+01, 2.1705E+01, 7.7310E+02, &
      5.2907497E+00, 8.19820E+01, 2.1549E+01, 7.6381E+02, &
      5.3172034E+00, 8.21013E+01, 2.1394E+01, 7.5463E+02, &
      5.3437895E+00, 8.22159E+01, 2.1241E+01, 7.4558E+02, &
      5.3705084E+00, 8.23263E+01, 2.1089E+01, 7.3663E+02, &
      5.3973609E+00, 8.24328E+01, 2.0938E+01, 7.2780E+02, &
      5.4243477E+00, 8.25357E+01, 2.0788E+01, 7.1909E+02, &
      5.4514695E+00, 8.26352E+01, 2.0640E+01, 7.1048E+02, &
      5.4787268E+00, 8.27316E+01, 2.0493E+01, 7.0198E+02, &
      5.5061205E+00, 8.28251E+01, 2.0347E+01, 6.9358E+02, &
      5.5336511E+00, 8.29158E+01, 2.0201E+01, 6.8528E+02, &
      5.5613193E+00, 8.30039E+01, 2.0057E+01, 6.7708E+02, &
      5.5891259E+00, 8.30894E+01, 1.9914E+01, 6.6897E+02, &
      5.6170716E+00, 8.31726E+01, 1.9771E+01, 6.6097E+02, &
      5.6451569E+00, 8.32534E+01, 1.9630E+01, 6.5306E+02, &
      5.6733827E+00, 8.33321E+01, 1.9490E+01, 6.4526E+02, &
      5.7017496E+00, 8.34087E+01, 1.9352E+01, 6.3755E+02, &
      5.7302584E+00, 8.34832E+01, 1.9214E+01, 6.2993E+02, &
      5.7589096E+00, 8.35559E+01, 1.9077E+01, 6.2240E+02, &
      5.7877042E+00, 8.36267E+01, 1.8941E+01, 6.1497E+02, &
      5.8166427E+00, 8.36957E+01, 1.8806E+01, 6.0763E+02, &
      5.8457259E+00, 8.37631E+01, 1.8672E+01, 6.0038E+02, &
      5.8749546E+00, 8.38288E+01, 1.8540E+01, 5.9323E+02, &
      5.9043293E+00, 8.38930E+01, 1.8408E+01, 5.8616E+02, &
      5.9338510E+00, 8.39557E+01, 1.8278E+01, 5.7918E+02, &
      5.9635202E+00, 8.40170E+01, 1.8148E+01, 5.7229E+02, &
      5.9933378E+00, 8.40770E+01, 1.8020E+01, 5.6549E+02, &
      6.0233045E+00, 8.41358E+01, 1.7893E+01, 5.5877E+02, &
      6.0534210E+00, 8.41935E+01, 1.7766E+01, 5.5213E+02, &
      6.0836882E+00, 8.42497E+01, 1.7639E+01, 5.4552E+02, &
      6.1141066E+00, 8.43044E+01, 1.7513E+01, 5.3899E+02, &
      6.1446771E+00, 8.43576E+01, 1.7388E+01, 5.3255E+02, &
      6.1754005E+00, 8.44094E+01, 1.7263E+01, 5.2618E+02, &
      6.2062775E+00, 8.44600E+01, 1.7140E+01, 5.1990E+02, &
      6.2373089E+00, 8.45093E+01, 1.7018E+01, 5.1369E+02, &
      6.2684954E+00, 8.45574E+01, 1.6897E+01, 5.0756E+02, &
      6.2998379E+00, 8.46043E+01, 1.6776E+01, 5.0150E+02, &
      6.3313371E+00, 8.46502E+01, 1.6657E+01, 4.9552E+02, &
      6.3629938E+00, 8.46950E+01, 1.6538E+01, 4.8962E+02, &
      6.3948088E+00, 8.47388E+01, 1.6421E+01, 4.8378E+02, &
      6.4267828E+00, 8.47816E+01, 1.6304E+01, 4.7802E+02, &
      6.4589167E+00, 8.49702E+01, 1.6186E+01, 4.7227E+02, &
      6.4912113E+00, 8.50108E+01, 1.6068E+01, 4.6656E+02, &
      6.5236674E+00, 8.50501E+01, 1.5951E+01, 4.6092E+02, &
      6.5562857E+00, 8.50883E+01, 1.5834E+01, 4.5535E+02, &
      6.5890671E+00, 8.51253E+01, 1.5719E+01, 4.4984E+02, &
      6.6220125E+00, 8.51611E+01, 1.5604E+01, 4.4441E+02, &
      6.6551225E+00, 8.51959E+01, 1.5490E+01, 4.3904E+02, &
      6.6883981E+00, 8.52297E+01, 1.5378E+01, 4.3374E+02, &
      6.7218401E+00, 8.52626E+01, 1.5266E+01, 4.2850E+02, &
      6.7554493E+00, 8.54012E+01, 1.5153E+01, 4.2330E+02, &
      6.7892266E+00, 8.54320E+01, 1.5040E+01, 4.1811E+02, &
      6.8231727E+00, 8.54616E+01, 1.4928E+01, 4.1299E+02, &
      6.8572886E+00, 8.54901E+01, 1.4816E+01, 4.0794E+02, &
      6.8915750E+00, 8.55174E+01, 1.4706E+01, 4.0295E+02, &
      6.9260329E+00, 8.55437E+01, 1.4597E+01, 3.9803E+02, &
      6.9606631E+00, 8.55689E+01, 1.4488E+01, 3.9317E+02, &
      6.9954664E+00, 8.55932E+01, 1.4381E+01, 3.8837E+02, &
      7.0304437E+00, 8.56165E+01, 1.4274E+01, 3.8363E+02, &
      7.0655959E+00, 8.56390E+01, 1.4168E+01, 3.7896E+02, &
      7.1009239E+00, 8.56606E+01, 1.4063E+01, 3.7435E+02, &
      7.1364285E+00, 8.56813E+01, 1.3960E+01, 3.6979E+02, &
      7.1721107E+00, 8.57012E+01, 1.3856E+01, 3.6530E+02, &
      7.2079712E+00, 8.57204E+01, 1.3754E+01, 3.6086E+02, &
      7.2440111E+00, 8.57388E+01, 1.3653E+01, 3.5648E+02, &
      7.2802311E+00, 8.57565E+01, 1.3552E+01, 3.5216E+02, &
      7.3166323E+00, 8.57734E+01, 1.3453E+01, 3.4789E+02, &
      7.3532155E+00, 8.57897E+01, 1.3354E+01, 3.4368E+02, &
      7.3899815E+00, 8.58054E+01, 1.3256E+01, 3.3952E+02, &
      7.4269314E+00, 8.58204E+01, 1.3159E+01, 3.3541E+02, &
      7.4640661E+00, 8.58347E+01, 1.3063E+01, 3.3136E+02, &
      7.5013864E+00, 8.58485E+01, 1.2967E+01, 3.2736E+02, &
      7.5388934E+00, 8.58618E+01, 1.2873E+01, 3.2341E+02, &
      7.5765878E+00, 8.58744E+01, 1.2779E+01, 3.1952E+02, &
      7.6144708E+00, 8.58866E+01, 1.2686E+01, 3.1567E+02, &
      7.6525431E+00, 8.58982E+01, 1.2594E+01, 3.1187E+02, &
      7.6908058E+00, 8.59094E+01, 1.2502E+01, 3.0812E+02, &
      7.7292599E+00, 8.59200E+01, 1.2412E+01, 3.0442E+02, &
      7.7679062E+00, 8.59303E+01, 1.2322E+01, 3.0077E+02, &
      7.8067457E+00, 8.59401E+01, 1.2232E+01, 2.9716E+02, &
      7.8457794E+00, 8.60405E+01, 1.2143E+01, 2.9357E+02, &
      7.8850083E+00, 8.60497E+01, 1.2053E+01, 2.9002E+02, &
      7.9244334E+00, 8.60582E+01, 1.1965E+01, 2.8651E+02, &
      7.9640555E+00, 8.60662E+01, 1.1877E+01, 2.8304E+02, &
      8.0038758E+00, 8.60737E+01, 1.1790E+01, 2.7962E+02, &
      8.0438952E+00, 8.60807E+01, 1.1703E+01, 2.7625E+02, &
      8.0841147E+00, 8.60873E+01, 1.1618E+01, 2.7291E+02, &
      8.1245352E+00, 8.60934E+01, 1.1533E+01, 2.6963E+02, &
      8.1651579E+00, 8.60991E+01, 1.1449E+01, 2.6638E+02, &
      8.2059837E+00, 8.61044E+01, 1.1365E+01, 2.6317E+02, &
      8.2470136E+00, 8.61094E+01, 1.1283E+01, 2.6001E+02, &
      8.2882487E+00, 8.61140E+01, 1.1201E+01, 2.5689E+02, &
      8.3296899E+00, 8.61183E+01, 1.1119E+01, 2.5381E+02, &
      8.3713384E+00, 8.61224E+01, 1.1039E+01, 2.5076E+02, &
      8.4131951E+00, 8.61262E+01, 1.0959E+01, 2.4776E+02, &
      8.4552610E+00, 8.61299E+01, 1.0879E+01, 2.4479E+02, &
      8.4975373E+00, 8.61334E+01, 1.0801E+01, 2.4187E+02, &
      8.5400250E+00, 8.61368E+01, 1.0723E+01, 2.3898E+02, &
      9.030794E+00, 8.62298E+01, 9.8603E+00, 2.0837E+02, &
      9.653919E+00, 8.61994E+01, 8.8228E+00, 1.7503E+02, &
      1.032004E+01, 8.60250E+01, 7.8971E+00, 1.4715E+02, &
      1.103212E+01, 8.57468E+01, 7.0763E+00, 1.2387E+02, &
      1.179334E+01, 8.53853E+01, 6.3510E+00, 1.0448E+02, &
      1.260708E+01, 8.49223E+01, 5.7054E+00, 8.8241E+01, &
      1.347697E+01, 8.43009E+01, 5.1303E+00, 7.4624E+01, &
      1.440688E+01, 8.33587E+01, 4.6144E+00, 6.3151E+01, &
      1.540095E+01, 8.12584E+01, 4.1462E+00, 5.3418E+01, &
      1.555358E+01, 8.05389E+01, 4.0817E+00, 5.2121E+01, &
      1.579165E+01, 7.79818E+01, 3.9846E+00, 5.0189E+01, &
      1.585513E+01, 7.49735E+01, 3.9594E+00, 4.9692E+01, &
      1.595036E+01, 7.79733E+01, 9.8111E+00, 1.1739E+02, &
      1.618842E+01, 8.06378E+01, 9.5811E+00, 1.1303E+02, &
      1.646362E+01, 8.18116E+01, 9.3265E+00, 1.0826E+02, &
      1.759961E+01, 8.34273E+01, 8.3371E+00, 9.0825E+01, &
      1.870154E+01, 8.29309E+01, 7.5063E+00, 7.7210E+01, &
      1.881398E+01, 8.26417E+01, 7.4287E+00, 7.5981E+01, &
      1.898778E+01, 8.17056E+01, 7.3111E+00, 7.4132E+01, &
      1.906412E+01, 8.01855E+01, 7.2604E+00, 7.3340E+01, &
      1.917862E+01, 8.17070E+01, 1.0179E+01, 1.0110E+02, &
      1.944320E+01, 8.29316E+01, 9.9560E+00, 9.7597E+01, &
      1.946486E+01, 8.29754E+01, 9.9381E+00, 9.7318E+01, &
      1.974080E+01, 8.30354E+01, 9.7148E+00, 9.3853E+01, &
      1.982016E+01, 8.24220E+01, 9.6521E+00, 9.2889E+01, &
      1.993920E+01, 8.33853E+01, 1.1060E+01, 1.0542E+02, &
      2.011215E+01, 8.41382E+01, 1.0919E+01, 1.0321E+02, &
      2.023680E+01, 8.44903E+01, 1.0818E+01, 1.0165E+02, &
      2.149988E+01, 8.63479E+01, 9.8611E+00, 8.7391E+01, &
      2.298338E+01, 8.74007E+01, 8.8862E+00, 7.3852E+01, &
      2.456923E+01, 8.80633E+01, 8.0074E+00, 6.2420E+01, &
      2.626450E+01, 8.85179E+01, 7.2189E+00, 5.2791E+01, &
      2.807676E+01, 8.88544E+01, 6.4683E+00, 4.4394E+01, &
      3.001405E+01, 8.90513E+01, 5.7880E+00, 3.7294E+01, &
      3.208502E+01, 8.92282E+01, 5.1815E+00, 3.1352E+01, &
      3.429889E+01, 8.92867E+01, 4.6315E+00, 2.6326E+01, &
      3.666551E+01, 8.93029E+01, 4.1441E+00, 2.2135E+01, &
      3.919543E+01, 8.93280E+01, 3.7081E+00, 1.8617E+01, &
      4.189992E+01, 8.93195E+01, 3.3067E+00, 1.5615E+01, &
      4.479101E+01, 8.92620E+01, 2.9491E+00, 1.3104E+01, &
      4.788159E+01, 8.91902E+01, 2.6322E+00, 1.1010E+01, &
      5.118542E+01, 8.91089E+01, 2.3512E+00, 9.2626E+00, &
      5.471721E+01, 8.90209E+01, 2.1017E+00, 7.8023E+00, &
      5.849270E+01, 8.89277E+01, 1.8802E+00, 6.5808E+00, &
      6.252870E+01, 8.88300E+01, 1.6832E+00, 5.5580E+00, &
      6.684318E+01, 8.87272E+01, 1.5080E+00, 4.7008E+00, &
      7.145536E+01, 8.86177E+01, 1.3506E+00, 3.9778E+00, &
      7.638578E+01, 8.84967E+01, 1.2095E+00, 3.3690E+00, &
      8.165640E+01, 8.83607E+01, 1.0812E+00, 2.8519E+00, &
      8.729069E+01, 8.81880E+01, 9.5963E-01, 2.4025E+00, &
      9.331374E+01, 8.79430E+01, 8.5031E-01, 2.0242E+00, &
      9.975239E+01, 8.75154E+01, 7.5400E-01, 1.7095E+00, &
      1.046202E+02, 8.67083E+01, 6.9228E-01, 1.5176E+00, &
      1.062215E+02, 8.57166E+01, 6.7374E-01, 1.4614E+00, &
      1.066353E+02, 8.46176E+01, 6.6908E-01, 1.4475E+00, &
      1.066485E+02, 8.45308E+01, 6.6893E-01, 1.4470E+00, &
      1.072891E+02, 8.57662E+01, 2.9949E+00, 5.4564E+00, &
      1.088904E+02, 8.68460E+01, 2.9272E+00, 5.2601E+00, &
      1.139931E+02, 8.78237E+01, 2.7264E+00, 4.6959E+00, &
      1.218587E+02, 8.83964E+01, 2.4567E+00, 3.9802E+00, &
      1.302669E+02, 8.87071E+01, 2.2109E+00, 3.3718E+00, &
      1.392553E+02, 8.89025E+01, 1.9891E+00, 2.8577E+00, &
      1.488640E+02, 8.90330E+01, 1.7894E+00, 2.4239E+00, &
      1.591356E+02, 8.91220E+01, 1.6081E+00, 2.0562E+00, &
      1.701159E+02, 8.91812E+01, 1.4447E+00, 1.7457E+00, &
      1.818539E+02, 8.92198E+01, 1.2979E+00, 1.4841E+00, &
      1.944018E+02, 8.92436E+01, 1.1661E+00, 1.2636E+00, &
      2.078156E+02, 8.92565E+01, 1.0478E+00, 1.0778E+00, &
      2.221548E+02, 8.92614E+01, 9.4169E-01, 9.2113E-01, &
      2.374835E+02, 8.92607E+01, 8.4700E-01, 7.8943E-01, &
      2.538699E+02, 8.92560E+01, 7.6192E-01, 6.7817E-01, &
      2.713869E+02, 8.92485E+01, 6.8547E-01, 5.8411E-01, &
      2.901126E+02, 8.92389E+01, 6.1676E-01, 5.0453E-01, &
      3.101304E+02, 8.92280E+01, 5.5500E-01, 4.3713E-01, &
      3.315294E+02, 8.92162E+01, 4.9947E-01, 3.8000E-01, &
      3.544049E+02, 8.92040E+01, 4.4955E-01, 3.3151E-01, &
      3.788588E+02, 8.91916E+01, 4.0465E-01, 2.9031E-01, &
      4.050001E+02, 8.91792E+01, 3.6428E-01, 2.5525E-01, &
      4.329451E+02, 8.91671E+01, 3.2796E-01, 2.2536E-01]
   case(90)
      frel = -1.4118E+00
      allocate(nist_tab(1120))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.61909E+01, 0.0000E+00, 1.1681E-04, &
      1.142761E-02, 2.61747E+01, 0.0000E+00, 1.6149E-04, &
      1.221612E-02, 2.61560E+01, 0.0000E+00, 2.2208E-04, &
      1.305903E-02, 2.61344E+01, 0.0000E+00, 3.0377E-04, &
      1.396010E-02, 2.61094E+01, 0.0000E+00, 4.1332E-04, &
      1.492335E-02, 2.60805E+01, 0.0000E+00, 5.5943E-04, &
      1.595306E-02, 2.60468E+01, 0.0000E+00, 7.5323E-04, &
      1.705382E-02, 2.60075E+01, 0.0000E+00, 1.0089E-03, &
      1.823053E-02, 2.59615E+01, 0.0000E+00, 1.3444E-03, &
      1.948844E-02, 2.59075E+01, 0.0000E+00, 1.7823E-03, &
      2.083314E-02, 2.58437E+01, 0.0000E+00, 2.3507E-03, &
      2.227063E-02, 2.57680E+01, 0.0000E+00, 3.0847E-03, &
      2.380730E-02, 2.56773E+01, 0.0000E+00, 4.0275E-03, &
      2.545001E-02, 2.55675E+01, 0.0000E+00, 5.2322E-03, &
      2.720606E-02, 2.54331E+01, 0.0000E+00, 6.7634E-03, &
      2.908327E-02, 2.52654E+01, 0.0000E+00, 8.6995E-03, &
      3.109002E-02, 2.50513E+01, 0.0000E+00, 1.1135E-02, &
      3.323523E-02, 2.47680E+01, 0.0000E+00, 1.4182E-02, &
      3.552846E-02, 2.43716E+01, 0.0000E+00, 1.7977E-02, &
      3.797993E-02, 2.37586E+01, 0.0000E+00, 2.2676E-02, &
      4.060054E-02, 2.25604E+01, 0.0000E+00, 2.8467E-02, &
      4.214000E-02, 2.07833E+01, 0.0000E+00, 3.2250E-02, &
      4.278500E-02, 1.82078E+01, 0.0000E+00, 3.3920E-02, &
      4.295700E-02, 1.50579E+01, 0.0000E+00, 3.4375E-02, &
      4.321500E-02, 1.84936E+01, 6.2225E+00, 2.6113E+04, &
      4.340198E-02, 1.98275E+01, 6.1089E+00, 2.5526E+04, &
      4.386000E-02, 2.14550E+01, 5.8413E+00, 2.4152E+04, &
      4.639671E-02, 2.38408E+01, 4.5952E+00, 1.7961E+04, &
      4.802000E-02, 2.35458E+01, 3.9690E+00, 1.4989E+04, &
      4.875500E-02, 2.22214E+01, 3.7229E+00, 1.3848E+04, &
      4.895100E-02, 2.03983E+01, 3.6609E+00, 1.3563E+04, &
      4.924500E-02, 2.24783E+01, 7.2684E+00, 2.6767E+04, &
      4.959809E-02, 2.36759E+01, 7.0823E+00, 2.5896E+04, &
      4.998000E-02, 2.43778E+01, 6.8885E+00, 2.4995E+04, &
      5.302035E-02, 2.64301E+01, 5.5810E+00, 1.9089E+04, &
      5.667876E-02, 2.70453E+01, 4.4349E+00, 1.4190E+04, &
      5.831000E-02, 2.70765E+01, 4.0354E+00, 1.2551E+04, &
      5.920250E-02, 2.69974E+01, 3.8396E+00, 1.1762E+04, &
      5.944050E-02, 2.68837E+01, 3.7899E+00, 1.1563E+04, &
      5.979750E-02, 2.70060E+01, 3.9429E+00, 1.1958E+04, &
      6.058959E-02, 2.70943E+01, 3.7785E+00, 1.1310E+04, &
      6.069000E-02, 2.70987E+01, 3.7584E+00, 1.1231E+04, &
      6.477028E-02, 2.69725E+01, 3.0621E+00, 8.5736E+03, &
      6.923942E-02, 2.64968E+01, 2.5065E+00, 6.5650E+03, &
      7.401695E-02, 2.56863E+01, 2.0745E+00, 5.0831E+03, &
      7.912411E-02, 2.43204E+01, 1.7349E+00, 3.9766E+03, &
      8.458368E-02, 2.13271E+01, 1.4655E+00, 3.1424E+03, &
      8.614200E-02, 1.93923E+01, 1.4018E+00, 2.9513E+03, &
      8.746050E-02, 1.52600E+01, 1.3516E+00, 2.8029E+03, &
      8.781210E-02, 1.05314E+01, 1.3388E+00, 2.7651E+03, &
      8.833950E-02, 1.52455E+01, 1.0479E+01, 2.1513E+04, &
      8.965800E-02, 1.91082E+01, 1.0267E+01, 2.0767E+04, &
      9.041995E-02, 1.99602E+01, 1.0148E+01, 2.0353E+04, &
      9.241400E-02, 2.06364E+01, 9.8487E+00, 1.9327E+04, &
      9.382850E-02, 1.90670E+01, 9.6467E+00, 1.8645E+04, &
      9.420570E-02, 1.63088E+01, 9.5941E+00, 1.8469E+04, &
      9.477150E-02, 1.96559E+01, 1.5359E+01, 2.9390E+04, &
      9.618600E-02, 2.30007E+01, 1.5131E+01, 2.8528E+04, &
      9.665893E-02, 2.36508E+01, 1.5057E+01, 2.8249E+04, &
      1.033284E-01, 2.87800E+01, 1.3806E+01, 2.4230E+04, &
      1.104581E-01, 3.17444E+01, 1.2059E+01, 1.9800E+04, &
      1.180797E-01, 3.34529E+01, 9.9814E+00, 1.5331E+04, &
      1.262272E-01, 3.41995E+01, 7.9913E+00, 1.1482E+04, &
      1.349368E-01, 3.42958E+01, 6.2997E+00, 8.4673E+03, &
      1.442475E-01, 3.39918E+01, 4.9610E+00, 6.2380E+03, &
      1.542005E-01, 3.34567E+01, 3.9461E+00, 4.6419E+03, &
      1.648404E-01, 3.27642E+01, 3.1953E+00, 3.5166E+03, &
      1.762144E-01, 3.18283E+01, 2.6470E+00, 2.7254E+03, &
      1.781640E-01, 3.15922E+01, 2.5722E+00, 2.6196E+03, &
      1.808910E-01, 3.10082E+01, 2.4753E+00, 2.4830E+03, &
      1.816182E-01, 3.04467E+01, 2.4508E+00, 2.4486E+03, &
      1.827090E-01, 3.09264E+01, 3.4426E+00, 3.4184E+03, &
      1.854360E-01, 3.12521E+01, 3.3473E+00, 3.2750E+03, &
      1.883732E-01, 3.13039E+01, 3.2527E+00, 3.1329E+03, &
      2.013709E-01, 3.10131E+01, 2.9151E+00, 2.6270E+03, &
      2.152655E-01, 3.04439E+01, 2.6679E+00, 2.2495E+03, &
      2.248120E-01, 2.98917E+01, 2.5468E+00, 2.0566E+03, &
      2.282530E-01, 2.94946E+01, 2.5105E+00, 1.9968E+03, &
      2.291706E-01, 2.91368E+01, 2.5014E+00, 1.9816E+03, &
      2.301188E-01, 2.93458E+01, 3.1319E+00, 2.4704E+03, &
      2.305470E-01, 2.94284E+01, 3.1242E+00, 2.4598E+03, &
      2.339880E-01, 2.95932E+01, 3.0655E+00, 2.3781E+03, &
      2.459970E-01, 2.93664E+01, 2.8929E+00, 2.1351E+03, &
      2.629708E-01, 2.86973E+01, 2.7185E+00, 1.8774E+03, &
      2.811158E-01, 2.77574E+01, 2.6016E+00, 1.6813E+03, &
      2.843960E-01, 2.75493E+01, 2.5861E+00, 1.6521E+03, &
      2.887490E-01, 2.72131E+01, 2.5676E+00, 1.6157E+03, &
      2.899098E-01, 2.70395E+01, 2.5631E+00, 1.6064E+03, &
      2.916510E-01, 2.70428E+01, 2.7642E+00, 1.7219E+03, &
      2.960040E-01, 2.68697E+01, 2.7511E+00, 1.6887E+03, &
      3.005128E-01, 2.66201E+01, 2.7396E+00, 1.6566E+03, &
      3.212482E-01, 2.51289E+01, 2.7077E+00, 1.5321E+03, &
      3.284960E-01, 2.44414E+01, 2.7030E+00, 1.4959E+03, &
      3.335240E-01, 2.38213E+01, 2.7013E+00, 1.4726E+03, &
      3.348648E-01, 2.35307E+01, 2.7010E+00, 1.4666E+03, &
      3.368760E-01, 2.34319E+01, 2.9966E+00, 1.6170E+03, &
      3.375120E-01, 2.33851E+01, 3.0053E+00, 1.6186E+03, &
      3.419040E-01, 2.29230E+01, 3.0731E+00, 1.6339E+03, &
      3.426780E-01, 2.28178E+01, 3.0865E+00, 1.6373E+03, &
      3.434143E-01, 2.27010E+01, 3.0997E+00, 1.6408E+03, &
      3.440556E-01, 2.25576E+01, 3.1115E+00, 1.6440E+03, &
      3.461220E-01, 2.24336E+01, 3.3865E+00, 1.7783E+03, &
      3.512880E-01, 2.19364E+01, 3.5645E+00, 1.8443E+03, &
      3.671099E-01, 2.00231E+01, 4.4154E+00, 2.1855E+03, &
      3.924405E-01, 1.67831E+01, 7.1039E+00, 3.2875E+03, &
      4.195189E-01, 1.41508E+01, 1.1622E+01, 5.0290E+03, &
      4.484657E-01, 1.31384E+01, 1.7550E+01, 7.1023E+03, &
      4.794098E-01, 1.43156E+01, 2.3895E+01, 9.0448E+03, &
      5.124891E-01, 1.75452E+01, 2.9557E+01, 1.0465E+04, &
      5.478508E-01, 2.21551E+01, 3.3777E+01, 1.1188E+04, &
      5.856525E-01, 2.72567E+01, 3.6283E+01, 1.1242E+04, &
      6.260625E-01, 3.19663E+01, 3.7198E+01, 1.0783E+04, &
      6.628720E-01, 3.45665E+01, 3.6966E+01, 1.0121E+04, &
      6.692609E-01, 3.44716E+01, 3.6854E+01, 9.9943E+03, &
      6.730180E-01, 3.40022E+01, 3.6780E+01, 9.9187E+03, &
      6.757236E-01, 3.25186E+01, 3.6723E+01, 9.8638E+03, &
      6.797820E-01, 3.45273E+01, 3.9972E+01, 1.0672E+04, &
      6.899280E-01, 3.67196E+01, 3.9785E+01, 1.0466E+04, &
      6.998180E-01, 3.78532E+01, 3.9573E+01, 1.0263E+04, &
      7.105295E-01, 3.82815E+01, 3.9315E+01, 1.0043E+04, &
      7.133859E-01, 3.75786E+01, 3.9242E+01, 9.9841E+03, &
      7.154399E-01, 3.81919E+01, 4.1177E+01, 1.0446E+04, &
      7.176705E-01, 3.90442E+01, 4.1128E+01, 1.0401E+04, &
      7.283820E-01, 4.09945E+01, 4.0878E+01, 1.0186E+04, &
      7.648052E-01, 4.49421E+01, 3.9918E+01, 9.4741E+03, &
      8.175768E-01, 4.89586E+01, 3.8340E+01, 8.5135E+03, &
      8.739896E-01, 5.20627E+01, 3.6575E+01, 7.5988E+03, &
      9.342948E-01, 5.41871E+01, 3.4753E+01, 6.7557E+03, &
      9.479540E-01, 5.43718E+01, 3.4360E+01, 6.5833E+03, &
      9.624635E-01, 5.39628E+01, 3.3951E+01, 6.4073E+03, &
      9.663327E-01, 5.29747E+01, 3.3844E+01, 6.3616E+03, &
      9.721365E-01, 5.43283E+01, 3.5901E+01, 6.7074E+03, &
      9.866460E-01, 5.58372E+01, 3.5510E+01, 6.5372E+03, &
      9.987612E-01, 5.66113E+01, 3.5192E+01, 6.4003E+03, &
      1.067676E+00, 5.96223E+01, 3.3269E+01, 5.6614E+03, &
      1.141345E+00, 6.17056E+01, 3.1392E+01, 4.9988E+03, &
      1.144836E+00, 6.17815E+01, 3.1305E+01, 4.9697E+03, &
      1.162359E+00, 6.20428E+01, 3.0854E+01, 4.8247E+03, &
      1.167032E+00, 6.19319E+01, 3.0736E+01, 4.7871E+03, &
      1.174041E+00, 6.23361E+01, 3.1003E+01, 4.7999E+03, &
      1.191564E+00, 6.29405E+01, 3.0570E+01, 4.6637E+03, &
      1.220098E+00, 6.36643E+01, 2.9870E+01, 4.4509E+03, &
      1.302910E+00, 6.51130E+01, 2.7954E+01, 3.9021E+03, &
      1.304285E+00, 6.51281E+01, 2.7924E+01, 3.8939E+03, &
      1.322852E+00, 6.52187E+01, 2.7527E+01, 3.7851E+03, &
      1.328170E+00, 6.50333E+01, 2.7416E+01, 3.7548E+03, &
      1.336147E+00, 6.54457E+01, 2.7784E+01, 3.7824E+03, &
      1.356090E+00, 6.59912E+01, 2.7340E+01, 3.6676E+03, &
      1.394281E+00, 6.66735E+01, 2.6477E+01, 3.4553E+03, &
      1.490486E+00, 6.78083E+01, 2.4451E+01, 2.9867E+03, &
      1.593329E+00, 6.85722E+01, 2.2623E+01, 2.5868E+03, &
      1.703269E+00, 6.90935E+01, 2.0944E+01, 2.2419E+03, &
      1.820795E+00, 6.94038E+01, 1.9396E+01, 1.9438E+03, &
      1.946430E+00, 6.95328E+01, 1.7987E+01, 1.6879E+03, &
      2.080733E+00, 6.95040E+01, 1.6713E+01, 1.4686E+03, &
      2.224304E+00, 6.93177E+01, 1.5559E+01, 1.2805E+03, &
      2.377781E+00, 6.89539E+01, 1.4510E+01, 1.1186E+03, &
      2.541848E+00, 6.83469E+01, 1.3550E+01, 9.7854E+02, &
      2.717235E+00, 6.73970E+01, 1.2670E+01, 8.5728E+02, &
      2.904724E+00, 6.58273E+01, 1.1779E+01, 7.4684E+02, &
      3.105150E+00, 6.27555E+01, 1.0766E+01, 6.4002E+02, &
      3.265360E+00, 5.64413E+01, 9.9671E+00, 5.6465E+02, &
      3.315340E+00, 4.98314E+01, 9.7405E+00, 5.4385E+02, &
      3.319406E+00, 4.85588E+01, 9.7225E+00, 5.4221E+02, &
      3.328668E+00, 4.26554E+01, 9.6817E+00, 5.3850E+02, &
      3.348660E+00, 4.89995E+01, 2.3222E+01, 1.2686E+03, &
      3.398640E+00, 5.38257E+01, 2.3169E+01, 1.2473E+03, &
      3.420984E+00, 5.43109E+01, 2.3148E+01, 1.2380E+03, &
      3.473346E+00, 5.21394E+01, 2.3103E+01, 1.2172E+03, &
      3.487309E+00, 4.72186E+01, 2.3093E+01, 1.2118E+03, &
      3.508254E+00, 5.36217E+01, 3.3748E+01, 1.7553E+03, &
      3.548445E+00, 5.91957E+01, 3.3101E+01, 1.7025E+03, &
      3.560616E+00, 6.02205E+01, 3.2903E+01, 1.6866E+03, &
      3.793288E+00, 6.89910E+01, 2.9417E+01, 1.4169E+03, &
      3.965178E+00, 7.07696E+01, 2.7185E+01, 1.2536E+03, &
      4.025870E+00, 6.96889E+01, 2.6448E+01, 1.2017E+03, &
      4.042054E+00, 6.76045E+01, 2.6257E+01, 1.1882E+03, &
      4.055024E+00, 6.88475E+01, 3.0612E+01, 1.3793E+03, &
      4.066331E+00, 7.01740E+01, 3.0470E+01, 1.3691E+03, &
      4.127022E+00, 7.28891E+01, 2.9725E+01, 1.3163E+03, &
      4.334821E+00, 7.64784E+01, 2.7409E+01, 1.1566E+03, &
      4.633924E+00, 7.88363E+01, 2.4678E+01, 9.7537E+02, &
      4.733792E+00, 7.91340E+01, 2.3854E+01, 9.2330E+02, &
      4.806248E+00, 7.88658E+01, 2.3286E+01, 8.8804E+02, &
      4.825570E+00, 7.81943E+01, 2.3139E+01, 8.7897E+02, &
      4.854552E+00, 7.90758E+01, 2.4418E+01, 9.2153E+02, &
      4.927008E+00, 8.00402E+01, 2.3885E+01, 8.8841E+02, &
      4.953664E+00, 8.02534E+01, 2.3694E+01, 8.7667E+02, &
      5.078654E+00, 8.09038E+01, 2.2836E+01, 8.2456E+02, &
      5.156389E+00, 8.09152E+01, 2.2329E+01, 7.9436E+02, &
      5.177118E+00, 8.05251E+01, 2.2198E+01, 7.8658E+02, &
      5.208212E+00, 8.11716E+01, 2.2975E+01, 8.0897E+02, &
      5.285946E+00, 8.19615E+01, 2.2498E+01, 7.8077E+02, &
      5.295467E+00, 8.20302E+01, 2.2440E+01, 7.7739E+02, &
      5.660855E+00, 8.37431E+01, 2.0400E+01, 6.6206E+02, &
      6.051453E+00, 8.48168E+01, 1.8549E+01, 5.6405E+02, &
      6.469004E+00, 8.55864E+01, 1.6866E+01, 4.8062E+02, &
      6.915365E+00, 8.62858E+01, 1.5296E+01, 4.0856E+02, &
      7.392525E+00, 8.67388E+01, 1.3846E+01, 3.4673E+02, &
      7.902609E+00, 8.69518E+01, 1.2553E+01, 2.9477E+02, &
      8.447890E+00, 8.71749E+01, 1.1388E+01, 2.5082E+02, &
      9.030794E+00, 8.73194E+01, 1.0307E+01, 2.1298E+02, &
      9.653919E+00, 8.72509E+01, 9.2288E+00, 1.7902E+02, &
      1.032004E+01, 8.71265E+01, 8.2609E+00, 1.5048E+02, &
      1.103212E+01, 8.68947E+01, 7.3993E+00, 1.2662E+02, &
      1.179334E+01, 8.65667E+01, 6.6379E+00, 1.0674E+02, &
      1.260708E+01, 8.61496E+01, 5.9644E+00, 9.0160E+01, &
      1.347697E+01, 8.56059E+01, 5.3635E+00, 7.6244E+01, &
      1.440688E+01, 8.48326E+01, 4.8281E+00, 6.4566E+01, &
      1.540095E+01, 8.34505E+01, 4.3400E+00, 5.4626E+01, &
      1.597429E+01, 8.16017E+01, 4.0941E+00, 4.9860E+01, &
      1.621880E+01, 7.90611E+01, 3.9969E+00, 4.8015E+01, &
      1.628400E+01, 7.60712E+01, 3.9716E+00, 4.7540E+01, &
      1.638180E+01, 7.90529E+01, 9.7880E+00, 1.1165E+02, &
      1.646362E+01, 8.03818E+01, 9.7109E+00, 1.1024E+02, &
      1.662631E+01, 8.17029E+01, 9.5605E+00, 1.0751E+02, &
      1.759961E+01, 8.41490E+01, 8.7161E+00, 9.2839E+01, &
      1.881398E+01, 8.45346E+01, 7.7725E+00, 7.7718E+01, &
      1.929934E+01, 8.40137E+01, 7.4369E+00, 7.2597E+01, &
      1.959473E+01, 8.27868E+01, 7.2435E+00, 6.9704E+01, &
      1.967351E+01, 8.12699E+01, 7.1932E+00, 6.8960E+01, &
      1.979167E+01, 8.27857E+01, 1.0103E+01, 9.5211E+01, &
      2.006266E+01, 8.39989E+01, 9.8825E+00, 9.1920E+01, &
      2.008706E+01, 8.40465E+01, 9.8630E+00, 9.1632E+01, &
      2.011215E+01, 8.40894E+01, 9.8430E+00, 9.1337E+01, &
      2.036974E+01, 8.41042E+01, 9.6416E+00, 8.8381E+01, &
      2.045163E+01, 8.34960E+01, 9.5789E+00, 8.7470E+01, &
      2.057446E+01, 8.44507E+01, 1.0973E+01, 9.9236E+01, &
      2.088154E+01, 8.55442E+01, 1.0734E+01, 9.5688E+01, &
      2.149988E+01, 8.66258E+01, 1.0271E+01, 8.9012E+01, &
      2.298338E+01, 8.79963E+01, 9.2629E+00, 7.5280E+01, &
      2.456923E+01, 8.88005E+01, 8.3504E+00, 6.3650E+01, &
      2.626450E+01, 8.93430E+01, 7.5314E+00, 5.3851E+01, &
      2.807676E+01, 8.97360E+01, 6.7506E+00, 4.5297E+01, &
      3.001405E+01, 8.99689E+01, 6.0420E+00, 3.8059E+01, &
      3.208502E+01, 9.01138E+01, 5.4126E+00, 3.2013E+01, &
      3.429889E+01, 9.02537E+01, 4.8420E+00, 2.6899E+01, &
      3.666551E+01, 9.02855E+01, 4.3329E+00, 2.2616E+01, &
      3.919543E+01, 9.02857E+01, 3.8815E+00, 1.9040E+01, &
      4.189992E+01, 9.03247E+01, 3.4680E+00, 1.5996E+01, &
      4.479101E+01, 9.02774E+01, 3.0935E+00, 1.3424E+01, &
      4.788159E+01, 9.02124E+01, 2.7615E+00, 1.1279E+01, &
      5.118542E+01, 9.01362E+01, 2.4670E+00, 9.4879E+00, &
      5.471721E+01, 9.00521E+01, 2.2056E+00, 7.9913E+00, &
      5.849270E+01, 8.99624E+01, 1.9734E+00, 6.7395E+00, &
      6.252870E+01, 8.98678E+01, 1.7670E+00, 5.6914E+00, &
      6.684318E+01, 8.97685E+01, 1.5833E+00, 4.8128E+00, &
      7.145536E+01, 8.96635E+01, 1.4194E+00, 4.0746E+00, &
      7.638578E+01, 8.95493E+01, 1.2714E+00, 3.4503E+00, &
      8.165640E+01, 8.94238E+01, 1.1368E+00, 2.9200E+00, &
      8.729069E+01, 8.92693E+01, 1.0106E+00, 2.4620E+00, &
      9.331374E+01, 8.90604E+01, 8.9633E-01, 2.0746E+00, &
      9.975239E+01, 8.87342E+01, 7.9500E-01, 1.7511E+00, &
      1.066353E+02, 8.79446E+01, 7.0562E-01, 1.4817E+00, &
      1.074579E+02, 8.77255E+01, 6.9603E-01, 1.4538E+00, &
      1.091026E+02, 8.67428E+01, 6.7747E-01, 1.4002E+00, &
      1.095413E+02, 8.55680E+01, 6.7265E-01, 1.3863E+00, &
      1.101992E+02, 8.67915E+01, 2.9769E+00, 5.1698E+00, &
      1.118439E+02, 8.78609E+01, 2.9101E+00, 4.9846E+00, &
      1.139931E+02, 8.84013E+01, 2.8262E+00, 4.7562E+00, &
      1.218587E+02, 8.92165E+01, 2.5488E+00, 4.0337E+00, &
      1.302669E+02, 8.95995E+01, 2.2961E+00, 3.4196E+00, &
      1.392553E+02, 8.98307E+01, 2.0668E+00, 2.8989E+00, &
      1.488640E+02, 8.99822E+01, 1.8601E+00, 2.4591E+00, &
      1.591356E+02, 9.00854E+01, 1.6732E+00, 2.0871E+00, &
      1.701159E+02, 9.01548E+01, 1.5039E+00, 1.7720E+00, &
      1.818539E+02, 9.02007E+01, 1.3518E+00, 1.5065E+00, &
      1.944018E+02, 9.02299E+01, 1.2151E+00, 1.2826E+00, &
      2.078156E+02, 9.02469E+01, 1.0924E+00, 1.0938E+00, &
      2.221548E+02, 9.02549E+01, 9.8224E-01, 9.3459E-01, &
      2.374835E+02, 9.02562E+01, 8.8375E-01, 8.0058E-01, &
      2.538699E+02, 9.02533E+01, 7.9550E-01, 6.8756E-01, &
      2.713869E+02, 9.02471E+01, 7.1615E-01, 5.9198E-01, &
      2.901126E+02, 9.02387E+01, 6.4479E-01, 5.1108E-01, &
      3.101304E+02, 9.02286E+01, 5.8061E-01, 4.4255E-01, &
      3.315294E+02, 9.02174E+01, 5.2287E-01, 3.8445E-01, &
      3.544049E+02, 9.02056E+01, 4.7092E-01, 3.3512E-01, &
      3.788588E+02, 9.01936E+01, 4.2417E-01, 2.9320E-01, &
      4.050001E+02, 9.01815E+01, 3.8211E-01, 2.5753E-01, &
      4.329451E+02, 9.01695E+01, 3.4424E-01, 2.2712E-01]
   case(91)
      frel = -1.4526E+00
      allocate(nist_tab(1108))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.50708E+01, 0.0000E+00, 1.1911E-04, &
      1.142761E-02, 2.49705E+01, 0.0000E+00, 1.6472E-04, &
      1.221612E-02, 2.48477E+01, 0.0000E+00, 2.2657E-04, &
      1.305903E-02, 2.46946E+01, 0.0000E+00, 3.0998E-04, &
      1.396010E-02, 2.44998E+01, 0.0000E+00, 4.2187E-04, &
      1.492335E-02, 2.42445E+01, 0.0000E+00, 5.7112E-04, &
      1.595306E-02, 2.38950E+01, 0.0000E+00, 7.6912E-04, &
      1.705382E-02, 2.33822E+01, 0.0000E+00, 1.0304E-03, &
      1.823053E-02, 2.25268E+01, 0.0000E+00, 1.3733E-03, &
      1.948844E-02, 2.05469E+01, 0.0000E+00, 1.8209E-03, &
      1.991416E-02, 1.88448E+01, 0.0000E+00, 1.9930E-03, &
      2.021897E-02, 1.52893E+01, 0.0000E+00, 2.1229E-03, &
      2.030025E-02, 1.09389E+01, 0.0000E+00, 2.1585E-03, &
      2.042217E-02, 1.56094E+01, 8.6095E+00, 7.6785E+04, &
      2.072698E-02, 1.97167E+01, 8.2733E+00, 7.2702E+04, &
      2.083314E-02, 2.04219E+01, 8.1605E+00, 7.1344E+04, &
      2.227063E-02, 2.43846E+01, 6.8204E+00, 5.5780E+04, &
      2.380730E-02, 2.57184E+01, 5.6906E+00, 4.3536E+04, &
      2.545001E-02, 2.61112E+01, 4.7351E+00, 3.3887E+04, &
      2.720606E-02, 2.55660E+01, 3.9366E+00, 2.6354E+04, &
      2.797422E-02, 2.45268E+01, 3.6451E+00, 2.3733E+04, &
      2.840240E-02, 2.24967E+01, 3.4956E+00, 2.2416E+04, &
      2.851658E-02, 1.99372E+01, 3.4572E+00, 2.2081E+04, &
      2.868785E-02, 2.27492E+01, 8.5021E+00, 5.3980E+04, &
      2.908327E-02, 2.51408E+01, 8.1590E+00, 5.1096E+04, &
      2.911602E-02, 2.52510E+01, 8.1314E+00, 5.0867E+04, &
      3.109002E-02, 2.80231E+01, 6.6795E+00, 3.9131E+04, &
      3.323523E-02, 2.88788E+01, 5.5146E+00, 3.0222E+04, &
      3.552846E-02, 2.92112E+01, 4.6362E+00, 2.3768E+04, &
      3.797993E-02, 2.93172E+01, 3.9541E+00, 1.8962E+04, &
      4.060054E-02, 2.92964E+01, 3.4099E+00, 1.5297E+04, &
      4.340198E-02, 2.91722E+01, 2.9656E+00, 1.2445E+04, &
      4.454936E-02, 2.90710E+01, 2.8131E+00, 1.1501E+04, &
      4.523124E-02, 2.89320E+01, 2.7291E+00, 1.0989E+04, &
      4.541307E-02, 2.87868E+01, 2.7074E+00, 1.0859E+04, &
      4.568582E-02, 2.89250E+01, 2.9484E+00, 1.1754E+04, &
      4.639671E-02, 2.90279E+01, 2.8539E+00, 1.1204E+04, &
      4.959809E-02, 2.89993E+01, 2.4863E+00, 9.1304E+03, &
      5.302035E-02, 2.88226E+01, 2.1766E+00, 7.4774E+03, &
      5.667876E-02, 2.85661E+01, 1.9157E+00, 6.1562E+03, &
      6.058959E-02, 2.82300E+01, 1.6925E+00, 5.0879E+03, &
      6.477028E-02, 2.78004E+01, 1.4998E+00, 4.2176E+03, &
      6.923942E-02, 2.72176E+01, 1.3325E+00, 3.5055E+03, &
      7.401695E-02, 2.63971E+01, 1.1871E+00, 2.9214E+03, &
      7.912411E-02, 2.51189E+01, 1.0607E+00, 2.4419E+03, &
      8.458368E-02, 2.26037E+01, 9.5112E-01, 2.0484E+03, &
      8.745598E-02, 1.94682E+01, 9.0207E-01, 1.8790E+03, &
      8.879459E-02, 1.49770E+01, 8.8094E-01, 1.8073E+03, &
      8.915156E-02, 9.78526E+00, 8.7548E-01, 1.7889E+03, &
      8.968700E-02, 1.50191E+01, 1.0962E+01, 2.2261E+04, &
      9.041995E-02, 1.81167E+01, 1.0842E+01, 2.1839E+04, &
      9.102561E-02, 1.93811E+01, 1.0745E+01, 2.1499E+04, &
      9.474531E-02, 2.15288E+01, 1.0180E+01, 1.9570E+04, &
      9.619550E-02, 1.97449E+01, 9.9740E+00, 1.8885E+04, &
      9.665893E-02, 1.36294E+01, 9.9098E+00, 1.8673E+04, &
      9.716228E-02, 2.03515E+01, 1.6144E+01, 3.0263E+04, &
      9.861247E-02, 2.39384E+01, 1.5896E+01, 2.9360E+04, &
      1.033284E-01, 2.85122E+01, 1.4953E+01, 2.6358E+04, &
      1.104581E-01, 3.21935E+01, 1.3231E+01, 2.1818E+04, &
      1.180797E-01, 3.44097E+01, 1.1064E+01, 1.7067E+04, &
      1.262272E-01, 3.54872E+01, 8.8921E+00, 1.2832E+04, &
      1.349368E-01, 3.57756E+01, 7.0015E+00, 9.4515E+03, &
      1.442475E-01, 3.55733E+01, 5.4869E+00, 6.9290E+03, &
      1.542005E-01, 3.50857E+01, 4.3327E+00, 5.1187E+03, &
      1.648404E-01, 3.44208E+01, 3.4783E+00, 3.8445E+03, &
      1.762144E-01, 3.35341E+01, 2.8555E+00, 2.9529E+03, &
      1.793891E-01, 3.31731E+01, 2.7202E+00, 2.7633E+03, &
      1.821348E-01, 3.25794E+01, 2.6140E+00, 2.6155E+03, &
      1.828670E-01, 3.20043E+01, 2.5872E+00, 2.5784E+03, &
      1.839653E-01, 3.25005E+01, 3.6043E+00, 3.5700E+03, &
      1.867111E-01, 3.28435E+01, 3.5009E+00, 3.4167E+03, &
      1.883732E-01, 3.28928E+01, 3.4423E+00, 3.3299E+03, &
      2.013709E-01, 3.26972E+01, 3.0665E+00, 2.7754E+03, &
      2.152655E-01, 3.22041E+01, 2.7907E+00, 2.3632E+03, &
      2.289513E-01, 3.15075E+01, 2.6069E+00, 2.0760E+03, &
      2.301188E-01, 3.14129E+01, 2.5942E+00, 2.0555E+03, &
      2.324557E-01, 3.11167E+01, 2.5700E+00, 2.0160E+03, &
      2.333902E-01, 3.07558E+01, 2.5607E+00, 2.0007E+03, &
      2.347919E-01, 3.10590E+01, 3.1961E+00, 2.4816E+03, &
      2.382963E-01, 3.12429E+01, 3.1371E+00, 2.4002E+03, &
      2.459970E-01, 3.11804E+01, 3.0226E+00, 2.2405E+03, &
      2.629708E-01, 3.06892E+01, 2.8300E+00, 1.9629E+03, &
      2.811158E-01, 2.99713E+01, 2.6972E+00, 1.7506E+03, &
      3.005128E-01, 2.90080E+01, 2.6116E+00, 1.5863E+03, &
      3.034080E-01, 2.88368E+01, 2.6025E+00, 1.5657E+03, &
      3.080520E-01, 2.85018E+01, 2.5895E+00, 1.5346E+03, &
      3.092904E-01, 2.83273E+01, 2.5864E+00, 1.5266E+03, &
      3.111480E-01, 2.83342E+01, 2.7934E+00, 1.6387E+03, &
      3.157920E-01, 2.81673E+01, 2.7850E+00, 1.6099E+03, &
      3.212482E-01, 2.78892E+01, 2.7772E+00, 1.5783E+03, &
      3.434143E-01, 2.64306E+01, 2.7633E+00, 1.4696E+03, &
      3.523100E-01, 2.56656E+01, 2.7641E+00, 1.4332E+03, &
      3.577025E-01, 2.50689E+01, 2.7659E+00, 1.4127E+03, &
      3.591405E-01, 2.48016E+01, 2.7665E+00, 1.4073E+03, &
      3.612975E-01, 2.46890E+01, 3.0222E+00, 1.5279E+03, &
      3.637760E-01, 2.44868E+01, 3.0540E+00, 1.5335E+03, &
      3.666900E-01, 2.41950E+01, 3.0960E+00, 1.5422E+03, &
      3.671099E-01, 2.41491E+01, 3.1025E+00, 1.5437E+03, &
      3.693440E-01, 2.38800E+01, 3.1391E+00, 1.5524E+03, &
      3.708288E-01, 2.36267E+01, 3.1653E+00, 1.5592E+03, &
      3.730560E-01, 2.34759E+01, 3.4110E+00, 1.6699E+03, &
      3.786240E-01, 2.29352E+01, 3.5914E+00, 1.7323E+03, &
      3.924405E-01, 2.13116E+01, 4.2586E+00, 1.9813E+03, &
      4.195189E-01, 1.78707E+01, 6.8577E+00, 2.9825E+03, &
      4.484657E-01, 1.49563E+01, 1.1473E+01, 4.6653E+03, &
      4.794098E-01, 1.37542E+01, 1.7704E+01, 6.7321E+03, &
      5.124891E-01, 1.49624E+01, 2.4432E+01, 8.6896E+03, &
      5.478508E-01, 1.84317E+01, 3.0386E+01, 1.0109E+04, &
      5.856525E-01, 2.33653E+01, 3.4702E+01, 1.0799E+04, &
      6.260625E-01, 2.87149E+01, 3.7107E+01, 1.0803E+04, &
      6.692609E-01, 3.33920E+01, 3.7790E+01, 1.0292E+04, &
      6.940360E-01, 3.48809E+01, 3.7581E+01, 9.8708E+03, &
      7.046590E-01, 3.42970E+01, 3.7396E+01, 9.6744E+03, &
      7.074918E-01, 3.27413E+01, 3.7338E+01, 9.6210E+03, &
      7.117410E-01, 3.48461E+01, 4.0747E+01, 1.0436E+04, &
      7.154399E-01, 3.59109E+01, 4.0683E+01, 1.0366E+04, &
      7.223640E-01, 3.71181E+01, 4.0551E+01, 1.0234E+04, &
      7.285320E-01, 3.78521E+01, 4.0421E+01, 1.0114E+04, &
      7.396830E-01, 3.83786E+01, 4.0159E+01, 9.8975E+03, &
      7.426566E-01, 3.76376E+01, 4.0084E+01, 9.8396E+03, &
      7.471170E-01, 3.92231E+01, 4.2107E+01, 1.0274E+04, &
      7.582680E-01, 4.13366E+01, 4.1842E+01, 1.0060E+04, &
      7.648052E-01, 4.22280E+01, 4.1677E+01, 9.9344E+03, &
      8.175768E-01, 4.74197E+01, 4.0170E+01, 8.9583E+03, &
      8.739896E-01, 5.12441E+01, 3.8376E+01, 8.0073E+03, &
      9.342948E-01, 5.41140E+01, 3.6464E+01, 7.1188E+03, &
      9.865660E-01, 5.54733E+01, 3.4914E+01, 6.4561E+03, &
      9.987612E-01, 5.52614E+01, 3.4569E+01, 6.3146E+03, &
      1.001667E+00, 5.50287E+01, 3.4483E+01, 6.2807E+03, &
      1.005693E+00, 5.39388E+01, 3.4360E+01, 6.2332E+03, &
      1.011734E+00, 5.54380E+01, 3.6621E+01, 6.6033E+03, &
      1.026834E+00, 5.71018E+01, 3.6093E+01, 6.4127E+03, &
      1.067676E+00, 5.92296E+01, 3.4753E+01, 5.9394E+03, &
      1.141345E+00, 6.16897E+01, 3.2727E+01, 5.2337E+03, &
      1.199814E+00, 6.30658E+01, 3.1325E+01, 4.7666E+03, &
      1.218179E+00, 6.33013E+01, 3.0879E+01, 4.6283E+03, &
      1.220098E+00, 6.32904E+01, 3.0833E+01, 4.6142E+03, &
      1.223076E+00, 6.31774E+01, 3.0763E+01, 4.5925E+03, &
      1.230422E+00, 6.35866E+01, 3.1051E+01, 4.6078E+03, &
      1.248786E+00, 6.41887E+01, 3.0619E+01, 4.4773E+03, &
      1.304285E+00, 6.53954E+01, 2.9327E+01, 4.1070E+03, &
      1.359358E+00, 6.62232E+01, 2.8118E+01, 3.7793E+03, &
      1.380164E+00, 6.63250E+01, 2.7685E+01, 3.6654E+03, &
      1.385713E+00, 6.61400E+01, 2.7572E+01, 3.6359E+03, &
      1.394281E+00, 6.65556E+01, 2.7928E+01, 3.6601E+03, &
      1.414842E+00, 6.70838E+01, 2.7526E+01, 3.5554E+03, &
      1.490486E+00, 6.82841E+01, 2.6009E+01, 3.1903E+03, &
      1.593329E+00, 6.93211E+01, 2.4006E+01, 2.7564E+03, &
      1.703269E+00, 7.00083E+01, 2.2179E+01, 2.3840E+03, &
      1.820795E+00, 7.04447E+01, 2.0512E+01, 2.0641E+03, &
      1.946430E+00, 7.06811E+01, 1.8990E+01, 1.7894E+03, &
      2.080733E+00, 7.08077E+01, 1.7607E+01, 1.5537E+03, &
      2.224304E+00, 7.07175E+01, 1.6348E+01, 1.3509E+03, &
      2.377781E+00, 7.04533E+01, 1.5209E+01, 1.1772E+03, &
      2.541848E+00, 6.99855E+01, 1.4174E+01, 1.0277E+03, &
      2.717235E+00, 6.92284E+01, 1.3230E+01, 8.9878E+02, &
      2.904724E+00, 6.80338E+01, 1.2341E+01, 7.8562E+02, &
      3.105150E+00, 6.60065E+01, 1.1284E+01, 6.7351E+02, &
      3.319406E+00, 6.09274E+01, 1.0182E+01, 5.7008E+02, &
      3.372964E+00, 5.81306E+01, 9.9380E+00, 5.4795E+02, &
      3.424591E+00, 5.20006E+01, 9.7128E+00, 5.2783E+02, &
      3.438358E+00, 4.55480E+01, 9.6543E+00, 5.2264E+02, &
      3.459009E+00, 5.08881E+01, 2.1709E+01, 1.1543E+03, &
      3.510636E+00, 5.48531E+01, 2.2103E+01, 1.1580E+03, &
      3.538976E+00, 5.52403E+01, 2.2332E+01, 1.1605E+03, &
      3.548445E+00, 5.52074E+01, 2.2411E+01, 1.1614E+03, &
      3.593144E+00, 5.29824E+01, 2.2795E+01, 1.1666E+03, &
      3.607589E+00, 4.83109E+01, 2.2924E+01, 1.1684E+03, &
      3.629256E+00, 5.44110E+01, 3.3167E+01, 1.6755E+03, &
      3.683424E+00, 6.07885E+01, 3.2669E+01, 1.6264E+03, &
      3.793288E+00, 6.63647E+01, 3.1445E+01, 1.5207E+03, &
      4.055024E+00, 7.18484E+01, 2.8132E+01, 1.2741E+03, &
      4.090324E+00, 7.19797E+01, 2.7673E+01, 1.2428E+03, &
      4.152931E+00, 7.10026E+01, 2.6869E+01, 1.1888E+03, &
      4.169626E+00, 6.89424E+01, 2.6657E+01, 1.1748E+03, &
      4.194669E+00, 7.15401E+01, 3.0837E+01, 1.3494E+03, &
      4.257276E+00, 7.43148E+01, 3.0010E+01, 1.2942E+03, &
      4.334821E+00, 7.60516E+01, 2.9026E+01, 1.2298E+03, &
      4.633924E+00, 7.93694E+01, 2.5821E+01, 1.0248E+03, &
      4.900882E+00, 8.05241E+01, 2.3541E+01, 8.8447E+02, &
      4.953664E+00, 8.04272E+01, 2.3135E+01, 8.6016E+02, &
      4.975895E+00, 8.02284E+01, 2.2968E+01, 8.5024E+02, &
      4.995899E+00, 7.95584E+01, 2.2820E+01, 8.4145E+02, &
      5.025904E+00, 8.04156E+01, 2.4078E+01, 8.8202E+02, &
      5.100918E+00, 8.13388E+01, 2.3547E+01, 8.5015E+02, &
      5.259562E+00, 8.21480E+01, 2.2501E+01, 7.8840E+02, &
      5.295467E+00, 8.22146E+01, 2.2277E+01, 7.7537E+02, &
      5.340065E+00, 8.21358E+01, 2.2004E+01, 7.5961E+02, &
      5.361533E+00, 8.17427E+01, 2.1874E+01, 7.5220E+02, &
      5.393734E+00, 8.23772E+01, 2.2649E+01, 7.7387E+02, &
      5.474238E+00, 8.31425E+01, 2.2178E+01, 7.4688E+02, &
      5.660855E+00, 8.41112E+01, 2.1150E+01, 6.8929E+02, &
      6.051453E+00, 8.53779E+01, 1.9257E+01, 5.8803E+02, &
      6.469004E+00, 8.62622E+01, 1.7544E+01, 5.0201E+02, &
      6.915365E+00, 8.70698E+01, 1.5966E+01, 4.2818E+02, &
      7.392525E+00, 8.76251E+01, 1.4461E+01, 3.6358E+02, &
      7.902609E+00, 8.79055E+01, 1.3104E+01, 3.0894E+02, &
      8.447890E+00, 8.81798E+01, 1.1893E+01, 2.6296E+02, &
      9.030794E+00, 8.83335E+01, 1.0760E+01, 2.2320E+02, &
      9.653919E+00, 8.83089E+01, 9.6297E+00, 1.8751E+02, &
      1.032004E+01, 8.82175E+01, 8.6257E+00, 1.5771E+02, &
      1.103212E+01, 8.80177E+01, 7.7269E+00, 1.3272E+02, &
      1.179334E+01, 8.77227E+01, 6.9284E+00, 1.1182E+02, &
      1.260708E+01, 8.73451E+01, 6.2224E+00, 9.4403E+01, &
      1.347697E+01, 8.68612E+01, 5.5962E+00, 7.9835E+01, &
      1.440688E+01, 8.62028E+01, 5.0364E+00, 6.7587E+01, &
      1.540095E+01, 8.51599E+01, 4.5327E+00, 5.7244E+01, &
      1.639844E+01, 8.26512E+01, 4.1014E+00, 4.8946E+01, &
      1.646362E+01, 8.22583E+01, 4.0757E+00, 4.8466E+01, &
      1.664944E+01, 8.01210E+01, 4.0040E+00, 4.7136E+01, &
      1.671637E+01, 7.71410E+01, 3.9787E+00, 4.6670E+01, &
      1.681677E+01, 8.01159E+01, 9.7771E+00, 1.0919E+02, &
      1.706776E+01, 8.27610E+01, 9.5479E+00, 1.0513E+02, &
      1.759961E+01, 8.45178E+01, 9.0900E+00, 9.7200E+01, &
      1.881398E+01, 8.56756E+01, 8.1188E+00, 8.1490E+01, &
      1.990743E+01, 8.50829E+01, 7.3656E+00, 7.0094E+01, &
      2.011215E+01, 8.44801E+01, 7.2367E+00, 6.8207E+01, &
      2.021213E+01, 8.38530E+01, 7.1750E+00, 6.7311E+01, &
      2.029339E+01, 8.23394E+01, 7.1255E+00, 6.6594E+01, &
      2.041527E+01, 8.38467E+01, 1.0027E+01, 9.2076E+01, &
      2.068251E+01, 8.50222E+01, 9.8181E+00, 8.9044E+01, &
      2.071997E+01, 8.50933E+01, 9.7894E+00, 8.8630E+01, &
      2.099908E+01, 8.51412E+01, 9.5803E+00, 8.5630E+01, &
      2.108350E+01, 8.45398E+01, 9.5185E+00, 8.4751E+01, &
      2.121012E+01, 8.54892E+01, 1.0902E+01, 9.6123E+01, &
      2.149988E+01, 8.65135E+01, 1.0685E+01, 9.2982E+01, &
      2.152669E+01, 8.65772E+01, 1.0665E+01, 9.2698E+01, &
      2.298338E+01, 8.84990E+01, 9.6515E+00, 7.8751E+01, &
      2.456923E+01, 8.94946E+01, 8.7034E+00, 6.6603E+01, &
      2.626450E+01, 9.01380E+01, 7.8518E+00, 5.6363E+01, &
      2.807676E+01, 9.05887E+01, 7.0431E+00, 4.7444E+01, &
      3.001405E+01, 9.08676E+01, 6.3053E+00, 3.9869E+01, &
      3.208502E+01, 9.10439E+01, 5.6494E+00, 3.3540E+01, &
      3.429889E+01, 9.12097E+01, 5.0605E+00, 2.8217E+01, &
      3.666551E+01, 9.12605E+01, 4.5286E+00, 2.3724E+01, &
      3.919543E+01, 9.12730E+01, 4.0568E+00, 1.9972E+01, &
      4.189992E+01, 9.12968E+01, 3.6335E+00, 1.6817E+01, &
      4.479101E+01, 9.12866E+01, 3.2428E+00, 1.4118E+01, &
      4.788159E+01, 9.12296E+01, 2.8953E+00, 1.1863E+01, &
      5.118542E+01, 9.11591E+01, 2.5869E+00, 9.9802E+00, &
      5.471721E+01, 9.10794E+01, 2.3132E+00, 8.4066E+00, &
      5.849270E+01, 9.09933E+01, 2.0701E+00, 7.0900E+00, &
      6.252870E+01, 9.09020E+01, 1.8538E+00, 5.9877E+00, &
      6.684318E+01, 9.08060E+01, 1.6614E+00, 5.0637E+00, &
      7.145536E+01, 9.07049E+01, 1.4901E+00, 4.2883E+00, &
      7.638578E+01, 9.05963E+01, 1.3356E+00, 3.6329E+00, &
      8.165640E+01, 9.04821E+01, 1.1945E+00, 3.0746E+00, &
      8.729069E+01, 9.03401E+01, 1.0630E+00, 2.5939E+00, &
      9.331374E+01, 9.01568E+01, 9.4456E-01, 2.1888E+00, &
      9.975239E+01, 8.98920E+01, 8.3817E-01, 1.8477E+00, &
      1.066353E+02, 8.93943E+01, 7.4429E-01, 1.5636E+00, &
      1.103494E+02, 8.87393E+01, 7.0047E-01, 1.4367E+00, &
      1.120384E+02, 8.77637E+01, 6.8190E-01, 1.3839E+00, &
      1.124888E+02, 8.65975E+01, 6.7708E-01, 1.3704E+00, &
      1.131644E+02, 8.78121E+01, 2.9643E+00, 5.0433E+00, &
      1.139931E+02, 8.85011E+01, 2.9314E+00, 4.9535E+00, &
      1.148534E+02, 8.88736E+01, 2.8977E+00, 4.8627E+00, &
      1.218587E+02, 8.99853E+01, 2.6448E+00, 4.2027E+00, &
      1.302669E+02, 9.04719E+01, 2.3843E+00, 3.5650E+00, &
      1.392553E+02, 9.07481E+01, 2.1472E+00, 3.0234E+00, &
      1.488640E+02, 9.09248E+01, 1.9332E+00, 2.5656E+00, &
      1.591356E+02, 9.10438E+01, 1.7405E+00, 2.1790E+00, &
      1.701159E+02, 9.11248E+01, 1.5653E+00, 1.8508E+00, &
      1.818539E+02, 9.11789E+01, 1.4077E+00, 1.5740E+00, &
      1.944018E+02, 9.12140E+01, 1.2661E+00, 1.3406E+00, &
      2.078156E+02, 9.12355E+01, 1.1388E+00, 1.1436E+00, &
      2.221548E+02, 9.12470E+01, 1.0245E+00, 9.7741E-01, &
      2.374835E+02, 9.12508E+01, 9.2193E-01, 8.3723E-01, &
      2.538699E+02, 9.12497E+01, 8.3043E-01, 7.1927E-01, &
      2.713869E+02, 9.12450E+01, 7.4810E-01, 6.1945E-01, &
      2.901126E+02, 9.12377E+01, 6.7401E-01, 5.3493E-01, &
      3.101304E+02, 9.12286E+01, 6.0732E-01, 4.6330E-01, &
      3.315294E+02, 9.12181E+01, 5.4730E-01, 4.0253E-01, &
      3.544049E+02, 9.12069E+01, 4.9325E-01, 3.5092E-01, &
      3.788588E+02, 9.11953E+01, 4.4459E-01, 3.0704E-01, &
      4.050001E+02, 9.11835E+01, 4.0077E-01, 2.6968E-01, &
      4.329451E+02, 9.11717E+01, 3.6130E-01, 2.3783E-01]
   case(92)
      frel = -1.4940E+00
      allocate(nist_tab(1108))
      nist_tab(:) = [  &
!       E(keV)      f1(e atom-1) f2(e atom-1) mu/p(cm2g-1)
      1.069000E-02, 2.79528E+01, 0.0000E+00, 1.2189E-04, &
      1.142761E-02, 2.79240E+01, 0.0000E+00, 1.6843E-04, &
      1.221612E-02, 2.78903E+01, 0.0000E+00, 2.3151E-04, &
      1.305903E-02, 2.78510E+01, 0.0000E+00, 3.1653E-04, &
      1.396010E-02, 2.78047E+01, 0.0000E+00, 4.3048E-04, &
      1.492335E-02, 2.77500E+01, 0.0000E+00, 5.8239E-04, &
      1.595306E-02, 2.76850E+01, 0.0000E+00, 7.8379E-04, &
      1.705382E-02, 2.76071E+01, 0.0000E+00, 1.0494E-03, &
      1.823053E-02, 2.75129E+01, 0.0000E+00, 1.3977E-03, &
      1.948844E-02, 2.73977E+01, 0.0000E+00, 1.8522E-03, &
      2.083314E-02, 2.72543E+01, 0.0000E+00, 2.4419E-03, &
      2.227063E-02, 2.70722E+01, 0.0000E+00, 3.2031E-03, &
      2.380730E-02, 2.68339E+01, 0.0000E+00, 4.1805E-03, &
      2.545001E-02, 2.65079E+01, 0.0000E+00, 5.4288E-03, &
      2.720606E-02, 2.60290E+01, 0.0000E+00, 7.0149E-03, &
      2.908327E-02, 2.52244E+01, 0.0000E+00, 9.0196E-03, &
      3.109002E-02, 2.32943E+01, 0.0000E+00, 1.1540E-02, &
      3.165400E-02, 2.19218E+01, 0.0000E+00, 1.2322E-02, &
      3.213850E-02, 1.86305E+01, 0.0000E+00, 1.3021E-02, &
      3.226770E-02, 1.45636E+01, 0.0000E+00, 1.3211E-02, &
      3.246150E-02, 1.90028E+01, 8.0655E+00, 4.3925E+04, &
      3.294600E-02, 2.29015E+01, 7.6492E+00, 4.1045E+04, &
      3.323523E-02, 2.39626E+01, 7.4137E+00, 3.9435E+04, &
      3.552846E-02, 2.72256E+01, 5.8396E+00, 2.9057E+04, &
      3.797993E-02, 2.80338E+01, 4.5971E+00, 2.1398E+04, &
      4.060054E-02, 2.75227E+01, 3.6227E+00, 1.5774E+04, &
      4.145400E-02, 2.67247E+01, 3.3657E+00, 1.4353E+04, &
      4.208850E-02, 2.48339E+01, 3.1905E+00, 1.3401E+04, &
      4.225770E-02, 2.24394E+01, 3.1459E+00, 1.3161E+04, &
      4.251150E-02, 2.50834E+01, 7.8564E+00, 3.2671E+04, &
      4.314600E-02, 2.74314E+01, 7.4619E+00, 3.0575E+04, &
      4.340198E-02, 2.78951E+01, 7.3103E+00, 2.9776E+04, &
      4.639671E-02, 3.00844E+01, 5.8073E+00, 2.2128E+04, &
      4.959809E-02, 3.06978E+01, 4.6607E+00, 1.6612E+04, &
      5.302035E-02, 3.08446E+01, 3.8042E+00, 1.2684E+04, &
      5.667876E-02, 3.07720E+01, 3.1486E+00, 9.8210E+03, &
      6.058959E-02, 3.05638E+01, 2.6360E+00, 7.6914E+03, &
      6.477028E-02, 3.02417E+01, 2.2277E+00, 6.0805E+03, &
      6.923942E-02, 2.97524E+01, 1.8973E+00, 4.8444E+03, &
      6.928600E-02, 2.97454E+01, 1.8943E+00, 4.8335E+03, &
      7.034650E-02, 2.95268E+01, 1.8280E+00, 4.5942E+03, &
      7.062930E-02, 2.93570E+01, 1.8110E+00, 4.5333E+03, &
      7.105350E-02, 2.94678E+01, 2.0657E+00, 5.1398E+03, &
      7.211400E-02, 2.94897E+01, 1.9896E+00, 4.8777E+03, &
      7.401695E-02, 2.93537E+01, 1.8640E+00, 4.4523E+03, &
      7.912411E-02, 2.87159E+01, 1.5845E+00, 3.5405E+03, &
      8.458368E-02, 2.76572E+01, 1.3608E+00, 2.8445E+03, &
      9.041995E-02, 2.56551E+01, 1.1805E+00, 2.3083E+03, &
      9.437400E-02, 2.23349E+01, 1.0826E+00, 2.0283E+03, &
      9.581850E-02, 1.81716E+01, 1.0507E+00, 1.9389E+03, &
      9.620370E-02, 1.32881E+01, 1.0425E+00, 1.9161E+03, &
      9.665893E-02, 1.74096E+01, 1.0594E+01, 1.9377E+04, &
      9.678150E-02, 1.83211E+01, 1.0566E+01, 1.9300E+04, &
      9.822600E-02, 2.25593E+01, 1.0235E+01, 1.8422E+04, &
      1.029000E-01, 2.48399E+01, 9.2675E+00, 1.5922E+04, &
      1.033284E-01, 2.46539E+01, 9.1858E+00, 1.5716E+04, &
      1.044750E-01, 2.30502E+01, 8.9723E+00, 1.5182E+04, &
      1.048950E-01, 2.01569E+01, 8.8960E+00, 1.4993E+04, &
      1.055250E-01, 2.36270E+01, 1.4841E+01, 2.4864E+04, &
      1.071000E-01, 2.70271E+01, 1.4433E+01, 2.3825E+04, &
      1.104581E-01, 3.00483E+01, 1.3559E+01, 2.1702E+04, &
      1.180797E-01, 3.34113E+01, 1.1642E+01, 1.7432E+04, &
      1.262272E-01, 3.52065E+01, 9.6348E+00, 1.3495E+04, &
      1.349368E-01, 3.60127E+01, 7.7566E+00, 1.0163E+04, &
      1.442475E-01, 3.61795E+01, 6.1725E+00, 7.5658E+03, &
      1.542005E-01, 3.59585E+01, 4.9189E+00, 5.6404E+03, &
      1.648404E-01, 3.55078E+01, 3.9643E+00, 4.2528E+03, &
      1.762144E-01, 3.48996E+01, 3.2536E+00, 3.2655E+03, &
      1.883732E-01, 3.40579E+01, 2.7311E+00, 2.5647E+03, &
      1.911980E-01, 3.37615E+01, 2.6353E+00, 2.4382E+03, &
      1.941245E-01, 3.31784E+01, 2.5442E+00, 2.3186E+03, &
      1.949049E-01, 3.26034E+01, 2.5212E+00, 2.2885E+03, &
      1.960755E-01, 3.31077E+01, 3.5505E+00, 3.2029E+03, &
      1.990020E-01, 3.34659E+01, 3.4612E+00, 3.0765E+03, &
      2.013709E-01, 3.35341E+01, 3.3938E+00, 2.9813E+03, &
      2.152655E-01, 3.33723E+01, 3.0715E+00, 2.5245E+03, &
      2.301188E-01, 3.29363E+01, 2.8345E+00, 2.1798E+03, &
      2.459970E-01, 3.23325E+01, 2.6663E+00, 1.9186E+03, &
      2.541140E-01, 3.19033E+01, 2.6041E+00, 1.8143E+03, &
      2.580035E-01, 3.15076E+01, 2.5788E+00, 1.7697E+03, &
      2.590407E-01, 3.11474E+01, 2.5725E+00, 1.7583E+03, &
      2.605965E-01, 3.14447E+01, 3.2065E+00, 2.1780E+03, &
      2.629708E-01, 3.15923E+01, 3.1757E+00, 2.1377E+03, &
      2.644860E-01, 3.16172E+01, 3.1568E+00, 2.1128E+03, &
      2.811158E-01, 3.13188E+01, 2.9841E+00, 1.8797E+03, &
      3.005128E-01, 3.06073E+01, 2.8491E+00, 1.6795E+03, &
      3.172260E-01, 2.97959E+01, 2.7776E+00, 1.5515E+03, &
      3.212482E-01, 2.95385E+01, 2.7650E+00, 1.5253E+03, &
      3.220815E-01, 2.94695E+01, 2.7626E+00, 1.5200E+03, &
      3.233763E-01, 2.92950E+01, 2.7589E+00, 1.5120E+03, &
      3.253185E-01, 2.93089E+01, 2.9706E+00, 1.6181E+03, &
      3.301740E-01, 2.91549E+01, 2.9605E+00, 1.5890E+03, &
      3.434143E-01, 2.84880E+01, 2.9416E+00, 1.5184E+03, &
      3.671099E-01, 2.68836E+01, 2.9316E+00, 1.4162E+03, &
      3.732820E-01, 2.63425E+01, 2.9328E+00, 1.3935E+03, &
      3.789955E-01, 2.57157E+01, 2.9349E+00, 1.3736E+03, &
      3.805191E-01, 2.54349E+01, 2.9356E+00, 1.3685E+03, &
      3.828045E-01, 2.53129E+01, 3.2026E+00, 1.4837E+03, &
      3.834740E-01, 2.52644E+01, 3.2108E+00, 1.4849E+03, &
      3.885180E-01, 2.47687E+01, 3.2810E+00, 1.4976E+03, &
      3.893435E-01, 2.46666E+01, 3.2939E+00, 1.5004E+03, &
      3.909087E-01, 2.44069E+01, 3.3195E+00, 1.5060E+03, &
      3.924405E-01, 2.43091E+01, 3.5522E+00, 1.6050E+03, &
      3.932565E-01, 2.42563E+01, 3.5743E+00, 1.6116E+03, &
      3.991260E-01, 2.37060E+01, 3.7559E+00, 1.6685E+03, &
      4.195189E-01, 2.13041E+01, 4.8226E+00, 2.0375E+03, &
      4.484657E-01, 1.77332E+01, 7.9788E+00, 3.1509E+03, &
      4.794098E-01, 1.50367E+01, 1.3269E+01, 4.8990E+03, &
      5.124891E-01, 1.44442E+01, 2.0027E+01, 6.9149E+03, &
      5.478508E-01, 1.64762E+01, 2.6903E+01, 8.6883E+03, &
      5.856525E-01, 2.07109E+01, 3.2585E+01, 9.8436E+03, &
      6.260625E-01, 2.61225E+01, 3.6340E+01, 1.0270E+04, &
      6.692609E-01, 3.15630E+01, 3.8080E+01, 1.0067E+04, &
      7.154399E-01, 3.56536E+01, 3.8143E+01, 9.4337E+03, &
      7.229460E-01, 3.59117E+01, 3.8032E+01, 9.3089E+03, &
      7.340115E-01, 3.52859E+01, 3.7824E+01, 9.1187E+03, &
      7.369623E-01, 3.36416E+01, 3.7760E+01, 9.0670E+03, &
      7.413885E-01, 3.58717E+01, 4.1359E+01, 9.8709E+03, &
      7.524540E-01, 3.83022E+01, 4.1135E+01, 9.6734E+03, &
      7.648052E-01, 3.97072E+01, 4.0844E+01, 9.4501E+03, &
      7.764980E-01, 4.01205E+01, 4.0535E+01, 9.2377E+03, &
      7.796196E-01, 3.93107E+01, 4.0447E+01, 9.1810E+03, &
      7.843020E-01, 4.09476E+01, 4.2572E+01, 9.6050E+03, &
      7.960080E-01, 4.30958E+01, 4.2262E+01, 9.3953E+03, &
      8.175768E-01, 4.56281E+01, 4.1650E+01, 9.0155E+03, &
      8.739896E-01, 5.03000E+01, 3.9885E+01, 8.0775E+03, &
      9.342948E-01, 5.37743E+01, 3.7929E+01, 7.1871E+03, &
      9.987612E-01, 5.62746E+01, 3.5952E+01, 6.3743E+03, &
      1.024002E+00, 5.67617E+01, 3.5157E+01, 6.0802E+03, &
      1.039675E+00, 5.63907E+01, 3.4673E+01, 5.9065E+03, &
      1.043855E+00, 5.54427E+01, 3.4545E+01, 5.8613E+03, &
      1.050124E+00, 5.67055E+01, 3.6492E+01, 6.1541E+03, &
      1.065798E+00, 5.81523E+01, 3.6075E+01, 5.9946E+03, &
      1.067676E+00, 5.82710E+01, 3.6026E+01, 5.9759E+03, &
      1.141345E+00, 6.14510E+01, 3.4109E+01, 5.2944E+03, &
      1.220098E+00, 6.37054E+01, 3.2167E+01, 4.6723E+03, &
      1.247148E+00, 6.42924E+01, 3.1543E+01, 4.4828E+03, &
      1.266237E+00, 6.45437E+01, 3.1076E+01, 4.3502E+03, &
      1.271327E+00, 6.44228E+01, 3.0953E+01, 4.3158E+03, &
      1.278963E+00, 6.48365E+01, 3.1232E+01, 4.3286E+03, &
      1.298052E+00, 6.54451E+01, 3.0775E+01, 4.2029E+03, &
      1.304285E+00, 6.56027E+01, 3.0628E+01, 4.1630E+03, &
      1.394281E+00, 6.72306E+01, 2.8569E+01, 3.6342E+03, &
      1.411984E+00, 6.74332E+01, 2.8194E+01, 3.5419E+03, &
      1.433596E+00, 6.75238E+01, 2.7750E+01, 3.4340E+03, &
      1.439359E+00, 6.73356E+01, 2.7634E+01, 3.4060E+03, &
      1.448004E+00, 6.77376E+01, 2.7992E+01, 3.4294E+03, &
      1.469616E+00, 6.82637E+01, 2.7573E+01, 3.3289E+03, &
      1.490486E+00, 6.86245E+01, 2.7181E+01, 3.2360E+03, &
      1.593329E+00, 6.99370E+01, 2.5229E+01, 2.8114E+03, &
      1.703269E+00, 7.08029E+01, 2.3293E+01, 2.4299E+03, &
      1.820795E+00, 7.13695E+01, 2.1544E+01, 2.1041E+03, &
      1.946430E+00, 7.17207E+01, 1.9946E+01, 1.8239E+03, &
      2.080733E+00, 7.18915E+01, 1.8490E+01, 1.5832E+03, &
      2.224304E+00, 7.19718E+01, 1.7142E+01, 1.3747E+03, &
      2.377781E+00, 7.18121E+01, 1.5922E+01, 1.1960E+03, &
      2.541848E+00, 7.14495E+01, 1.4819E+01, 1.0428E+03, &
      2.717235E+00, 7.08654E+01, 1.3811E+01, 9.1051E+02, &
      2.904724E+00, 6.99412E+01, 1.2896E+01, 7.9665E+02, &
      3.105150E+00, 6.86251E+01, 1.1834E+01, 6.8534E+02, &
      3.319406E+00, 6.54390E+01, 1.0677E+01, 5.7995E+02, &
      3.480666E+00, 5.97039E+01, 9.9352E+00, 5.1577E+02, &
      3.533942E+00, 5.38613E+01, 9.7113E+00, 4.9690E+02, &
      3.548445E+00, 4.74966E+01, 9.6521E+00, 4.9194E+02, &
      3.569459E+00, 5.26347E+01, 2.0892E+01, 1.0457E+03, &
      3.622734E+00, 5.61730E+01, 2.1494E+01, 1.0599E+03, &
      3.653048E+00, 5.65213E+01, 2.1860E+01, 1.0688E+03, &
      3.708962E+00, 5.46240E+01, 2.2581E+01, 1.0872E+03, &
      3.723872E+00, 5.07198E+01, 2.2783E+01, 1.0925E+03, &
      3.746238E+00, 5.57172E+01, 3.1484E+01, 1.4966E+03, &
      3.793288E+00, 6.05481E+01, 3.1609E+01, 1.4839E+03, &
      3.802152E+00, 6.11356E+01, 3.1623E+01, 1.4811E+03, &
      4.055024E+00, 7.07049E+01, 3.0305E+01, 1.3317E+03, &
      4.217332E+00, 7.30339E+01, 2.8336E+01, 1.1981E+03, &
      4.281883E+00, 7.22360E+01, 2.7486E+01, 1.1450E+03, &
      4.299097E+00, 7.02440E+01, 2.7258E+01, 1.1311E+03, &
      4.324917E+00, 7.28482E+01, 3.1346E+01, 1.2915E+03, &
      4.334821E+00, 7.35305E+01, 3.1209E+01, 1.2830E+03, &
      4.389468E+00, 7.56949E+01, 3.0458E+01, 1.2368E+03, &
      4.633924E+00, 7.97420E+01, 2.7299E+01, 1.0513E+03, &
      4.953664E+00, 8.17842E+01, 2.4172E+01, 8.7213E+02, &
      5.078556E+00, 8.20216E+01, 2.3162E+01, 8.1564E+02, &
      5.156289E+00, 8.16845E+01, 2.2579E+01, 7.8340E+02, &
      5.177018E+00, 8.10069E+01, 2.2429E+01, 7.7515E+02, &
      5.208111E+00, 8.18451E+01, 2.3676E+01, 8.1288E+02, &
      5.285844E+00, 8.27243E+01, 2.3136E+01, 7.8292E+02, &
      5.295467E+00, 8.27906E+01, 2.3071E+01, 7.7934E+02, &
      5.437040E+00, 8.34056E+01, 2.2163E+01, 7.2963E+02, &
      5.520260E+00, 8.33692E+01, 2.1668E+01, 7.0282E+02, &
      5.542452E+00, 8.29745E+01, 2.1539E+01, 6.9592E+02, &
      5.575740E+00, 8.35910E+01, 2.2303E+01, 7.1598E+02, &
      5.660855E+00, 8.43339E+01, 2.1828E+01, 6.9044E+02, &
      6.051453E+00, 8.58919E+01, 1.9892E+01, 5.8951E+02, &
      6.469004E+00, 8.68845E+01, 1.8160E+01, 5.0430E+02, &
      6.915365E+00, 8.76315E+01, 1.6578E+01, 4.3142E+02, &
      7.392525E+00, 8.83400E+01, 1.5073E+01, 3.6771E+02, &
      7.902609E+00, 8.88088E+01, 1.3659E+01, 3.1244E+02, &
      8.447890E+00, 8.90434E+01, 1.2396E+01, 2.6592E+02, &
      9.030794E+00, 8.93244E+01, 1.1217E+01, 2.2573E+02, &
      9.653919E+00, 8.93484E+01, 1.0035E+01, 1.8957E+02, &
      1.032004E+01, 8.92472E+01, 8.9963E+00, 1.5956E+02, &
      1.103212E+01, 8.91021E+01, 8.0616E+00, 1.3430E+02, &
      1.179334E+01, 8.88563E+01, 7.2272E+00, 1.1313E+02, &
      1.260708E+01, 8.85144E+01, 6.4889E+00, 9.5471E+01, &
      1.347697E+01, 8.80793E+01, 5.8352E+00, 8.0723E+01, &
      1.440688E+01, 8.75056E+01, 5.2532E+00, 6.8351E+01, &
      1.540095E+01, 8.66632E+01, 4.7330E+00, 5.7944E+01, &
      1.646362E+01, 8.49904E+01, 4.2573E+00, 4.9068E+01, &
      1.682297E+01, 8.37020E+01, 4.1139E+00, 4.6504E+01, &
      1.708047E+01, 8.11840E+01, 4.0165E+00, 4.4787E+01, &
      1.714913E+01, 7.82159E+01, 3.9912E+00, 4.4345E+01, &
      1.725213E+01, 8.11816E+01, 9.7676E+00, 1.0327E+02, &
      1.750963E+01, 8.38201E+01, 9.5374E+00, 9.9419E+01, &
      1.759961E+01, 8.42601E+01, 9.4591E+00, 9.8122E+01, &
      1.881398E+01, 8.65214E+01, 8.4719E+00, 8.2478E+01, &
      2.011215E+01, 8.66430E+01, 7.5551E+00, 6.9059E+01, &
      2.052865E+01, 8.61569E+01, 7.2931E+00, 6.5391E+01, &
      2.084286E+01, 8.49264E+01, 7.1048E+00, 6.2800E+01, &
      2.092665E+01, 8.34187E+01, 7.0559E+00, 6.2133E+01, &
      2.105234E+01, 8.49155E+01, 9.9432E+00, 8.6005E+01, &
      2.132225E+01, 8.60676E+01, 9.7423E+00, 8.3243E+01, &
      2.136655E+01, 8.61487E+01, 9.7100E+00, 8.2801E+01, &
      2.149988E+01, 8.62893E+01, 9.6138E+00, 8.1494E+01, &
      2.164861E+01, 8.61911E+01, 9.5083E+00, 8.0069E+01, &
      2.173564E+01, 8.55954E+01, 9.4475E+00, 7.9251E+01, &
      2.186619E+01, 8.65378E+01, 1.0820E+01, 8.9871E+01, &
      2.219255E+01, 8.76176E+01, 1.0587E+01, 8.6681E+01, &
      2.298338E+01, 8.88440E+01, 1.0043E+01, 7.9497E+01, &
      2.456923E+01, 9.01206E+01, 9.0627E+00, 6.7277E+01, &
      2.626450E+01, 9.08923E+01, 8.1770E+00, 5.6936E+01, &
      2.807676E+01, 9.14167E+01, 7.3433E+00, 4.7975E+01, &
      3.001405E+01, 9.17506E+01, 6.5760E+00, 4.0325E+01, &
      3.208502E+01, 9.19635E+01, 5.8930E+00, 3.3926E+01, &
      3.429889E+01, 9.20978E+01, 5.2854E+00, 2.8572E+01, &
      3.666551E+01, 9.22294E+01, 4.7300E+00, 2.4020E+01, &
      3.919543E+01, 9.22566E+01, 4.2372E+00, 2.0219E+01, &
      4.189992E+01, 9.22905E+01, 3.7996E+00, 1.7042E+01, &
      4.479101E+01, 9.22915E+01, 3.3967E+00, 1.4328E+01, &
      4.788159E+01, 9.22438E+01, 3.0332E+00, 1.2038E+01, &
      5.118542E+01, 9.21798E+01, 2.7107E+00, 1.0127E+01, &
      5.471721E+01, 9.21051E+01, 2.4243E+00, 8.5295E+00, &
      5.849270E+01, 9.20229E+01, 2.1699E+00, 7.1930E+00, &
      6.252870E+01, 9.19351E+01, 1.9436E+00, 6.0738E+00, &
      6.684318E+01, 9.18426E+01, 1.7422E+00, 5.1356E+00, &
      7.145536E+01, 9.17453E+01, 1.5628E+00, 4.3484E+00, &
      7.638578E+01, 9.16425E+01, 1.4021E+00, 3.6850E+00, &
      8.165640E+01, 9.15374E+01, 1.2542E+00, 3.1180E+00, &
      8.729069E+01, 9.14048E+01, 1.1164E+00, 2.6295E+00, &
      9.331374E+01, 9.12401E+01, 9.9385E-01, 2.2208E+00, &
      9.975239E+01, 9.10167E+01, 8.8245E-01, 1.8743E+00, &
      1.066353E+02, 9.06512E+01, 7.8376E-01, 1.5850E+00, &
      1.132940E+02, 8.97540E+01, 7.0418E-01, 1.3641E+00, &
      1.139931E+02, 8.95066E+01, 6.9658E-01, 1.3436E+00, &
      1.150281E+02, 8.87858E+01, 6.8558E-01, 1.3142E+00, &
      1.154905E+02, 8.76284E+01, 6.8076E-01, 1.3013E+00, &
      1.161841E+02, 8.88339E+01, 2.9507E+00, 4.7472E+00, &
      1.179182E+02, 8.98873E+01, 2.8843E+00, 4.5771E+00, &
      1.218587E+02, 9.06637E+01, 2.7419E+00, 4.2211E+00, &
      1.302669E+02, 9.13182E+01, 2.4727E+00, 3.5810E+00, &
      1.392553E+02, 9.16532E+01, 2.2281E+00, 3.0376E+00, &
      1.488640E+02, 9.18604E+01, 2.0069E+00, 2.5778E+00, &
      1.591356E+02, 9.19976E+01, 1.8075E+00, 2.1893E+00, &
      1.701159E+02, 9.20914E+01, 1.6270E+00, 1.8603E+00, &
      1.818539E+02, 9.21545E+01, 1.4640E+00, 1.5820E+00, &
      1.944018E+02, 9.21962E+01, 1.3174E+00, 1.3472E+00, &
      2.078156E+02, 9.22226E+01, 1.1856E+00, 1.1490E+00, &
      2.221548E+02, 9.22378E+01, 1.0670E+00, 9.8175E-01, &
      2.374835E+02, 9.22446E+01, 9.6050E-01, 8.4046E-01, &
      2.538699E+02, 9.22454E+01, 8.6554E-01, 7.2164E-01, &
      2.713869E+02, 9.22423E+01, 7.8024E-01, 6.2121E-01, &
      2.901126E+02, 9.22362E+01, 7.0343E-01, 5.3614E-01, &
      3.101304E+02, 9.22281E+01, 6.3425E-01, 4.6402E-01, &
      3.315294E+02, 9.22184E+01, 5.7194E-01, 4.0283E-01, &
      3.544049E+02, 9.22078E+01, 5.1581E-01, 3.5085E-01, &
      3.788588E+02, 9.21966E+01, 4.6523E-01, 3.0666E-01, &
      4.050001E+02, 9.21851E+01, 4.1966E-01, 2.6904E-01, &
      4.329451E+02, 9.21736E+01, 3.7858E-01, 2.3698E-01]
   end select
!
   end subroutine load_nist_table
end module nist_mod
